
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary mkfile Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Mkfile

=head1 DESCRIPTION

B<DEPRECATED FOR VOLUME FILES>.

Note that this module is now deprecated for creating B<volume> files. It is
suggested that
L<< NACL::C::VolumeFile->create|lib-NACL-C-VolumeFile-pm/create >> is used
instead. That component provides support for find/state and instance methods.

Note that this component is still valid for B<aggregate> files.

C<NACL::C::Mkfile> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It provides 7Mode and Nodescope
support for the mkfile command.

Since this component does not model any element:
* There are no primary keys and there is no object constructor
* There is no support for 'find' 
* There is no support for 'state'
* There is no ComponentState file

=head1 ATTRIBUTES

None

=cut

package NACL::C::Mkfile;

use strict;
use warnings;

use Params::Validate qw(validate_with SCALAR UNDEF);
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use NACL::C::Volume;
use NATE::Exceptions::Argument;
use base 'NACL::C::Component::ONTAP';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 mkfile

    NACL::C::Mkfile->mkfile(command_interface => $ci, ...);

(Class method)
This method is used to create a file using the mkfile command.

Uses a 7Mode/Nodescope CLI APISet.

Invokes "mkfile" command for 7Mode/Nodescope CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item Command options

All of the options listed under "PROGRAMMATIC NAME" on the
L<mkfile|lib-NACL-APISet-Node-CLI-7Mode-default-mkfile-pm--mkfile>
page are accepted by this method.

=item C<< 'nodescope-node-name' => $node_name >>

(Optional)
When run in nodescope, the component method defaults to running on the "local"
node. This argument can be used to run the method on node other than "local".

Applicable for 7Mode CLI

=item C<< determine_appropriate_nodescope_node => 0|1 >>

(Optional, defaults to 0)
When set to 1, the method first determines which node the volume is present
on and makes the call in the context of that node. Note that when
C<nodescope-node-name> overrides this argument, meaning that if both
C<nodescope-node-name> is provided and
C<determine_appropriate_nodescope_node> is set to 1 then the node name
provided in C<nodescope-node-name> is used.

=back

=back

=cut

my $Already_Warned;

sub mkfile {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    $pkg->_nodescope_frontend_helper(
        @_,
        command => 'mkfile',
        choices => [
            {   method    => '_mkfile_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );
    $Log->exit() if $may_exit;
}

sub _mkfile_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $ci     = $opts{command_interface};
    my $apiset = $opts{apiset};

    my %mkfile_args;
    my @allowed_options
        = $apiset->get_allowed_options_for_command( command => 'mkfile' );
    push @allowed_options, 'nodescope-node-name';

    $pkg->_hash_copy(
        source => \%opts,
        target => \%mkfile_args,
        copy   => \@allowed_options
    );
    
    # Warn only for volume files (and if the warning hasn't been logged yet)
    if (!$Already_Warned && $mkfile_args{filename} =~ m!/vol/!) {
        my $pkg_to_use = 'NACL::' . $pkg->is_c_or_t() . '::VolumeFile';
        my $warn_message = "This module ($pkg) is deprecated. Please use $pkg_to_use "
            . 'instead to deal with Volume files';
        NACL::C::Component->_print_deprecation_warning($warn_message);
        $Already_Warned = 1;
    }

    $apiset->mkfile(%mkfile_args);
    $Log->exit() if $may_exit;
}

# Provided a file path, returns the volume/aggregate name
# This returns an array of two values: the first being the name,
# the second denoting what it's the name of (can be either "volume"
# or "aggregate")
sub _get_vol_or_aggr_name_from_path {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => { filename => { type => SCALAR } }
    );

    # The "filename" argument is really a path
    my $filepath = $opts{filename};
    if ( $filepath =~ m!/vol/(.*?)/.*! ) {
        $Log->exit() if $may_exit;
        return ( $1, 'volume' );
    } elsif ( $filepath =~ m!/aggr/(.*?)/.*! ) {
        $Log->exit() if $may_exit;
        return ( $1, 'aggregate' );
    } else {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "The 'filename' ($filepath) is not in the expected format" );
    }

    $Log->exit() if $may_exit;
}

sub get_local_node_name {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            vserver => { type => SCALAR | UNDEF, optional => 1 },
            command_interface =>
                { isa => 'NACL::C::CommandInterface::ONTAP' },
            filename => { type => SCALAR },
        },
        allow_extra => 1
    );
    
    my %common_params;
    $pkg->_copy_common_component_params(
        source => \%opts,
        target => \%common_params
    );

    my ( $name, $volume_or_aggr )
        = $pkg->_get_vol_or_aggr_name_from_path( filename => $opts{filename} );

    my %node_call_opts;
    my $component_to_call = 'NACL::C::Volume';

    if ( $volume_or_aggr eq 'aggregate' ) {
        $component_to_call = 'NACL::C::StorageAggregate';
        %node_call_opts    = (
            command_interface => $opts{command_interface},
            aggregate         => $name
        );

    } else {
        %node_call_opts = (
            command_interface => $opts{command_interface},
            volume            => $name,
            vserver           => $opts{vserver}
        );

    }

    my $node = $component_to_call->get_local_node_name(%node_call_opts, %common_params);

    $Log->exit() if $may_exit;
    return $node;
}

sub _copy_primary_keys {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_hash_copy(%opts, copy => [qw(vserver filename)]);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

1;
