#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Network Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Network

=head1 DESCRIPTION

C<NACL::C::Network> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. Unlike other components this
package does not model any element/service but is present to provide
component-level support for functionality. Since it does not model any
element/service there is no ComponentState module for this component.
For the same reason, 'find' and 'state' will not work for this component.

=cut

package NACL::C::Network;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars Dumper);
use NACL::CS::NetworkPing;
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR SCALARREF ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::Exceptions::HostUnreachable qw(:try);
use Net::IP qw(ip_is_ipv6);

=head2 new

    my $state = NACL::C::Network->new( command_interface => $ci, destination => $destination );

Return a new ping component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'destination',];

=head2 traceroute

   NACL::C::Network->traceroute(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to traceroute.

Uses a CMode CLI, 7Mode CLI APISet.

Invokes "traceroute" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "verbose" => $string >>

Applicable for CMode CLI, 7Mode CLI

Maps to:

7M CLI:  -v (Programmatic name: 'verbose')

=item C<< "destination" => $string >>

Applicable for CMode CLI, 7Mode CLI

Maps to:

7M CLI: host name (Programmatic name: 'host')

=item C<< "waittime" => $string >>

Applicable for CMode CLI, 7Mode CLI

Maps to:

7M CLI:  -w <value> (Programmatic name: 'waittime')

=item C<< "node" => $string >>

Applicable for CMode CLI

=item C<< "port" => $string >>

Applicable for CMode CLI, 7Mode CLI

Maps to:

7M CLI:  -p (Programmatic name: 'base-port')

=item C<< "packet-size" => $string >>

Applicable for CMode CLI, 7Mode CLI

Maps to:

7M CLI: packet size (Programmatic name: 'packet-size')

=item C<< "abort-signal" => $string >>

Applicable for CMode CLI.

=item C<< "maxttl" => $string >>

Applicable for 7Mode CLI

Maps to:

7M CLI:  -m (Programmatic name: 'max-hops')

=item C<< "lif-ipaddr" => $string >>

Applicable for CMode CLI.

=item C<< "cmd-pid" => $string >>

Applicable for CMode CLI.

=item C<< "lif-owner" => $string >>

Applicable for CMode CLI.

=item C<< "numeric" => $string >>

Applicable for CMode CLI, 7Mode CLI

Maps to:

7M CLI:  -n (Programmatic name: 'numeric-address')

=item C<< "is-lif-error" => $string >>

Applicable for CMode CLI.

=item C<< "nqueries" => $string >>

Applicable for CMode CLI, 7Mode CLI

Maps to:

7M CLI:  -q (Programmatic name: 'nqueries')

=item C<< "lif" => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< 'vserver' => $string >>

LIF Owner
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"network traceroute" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub traceroute {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( verbose destination waittime node port packet-size
                    abort-signal maxttl lif-ipaddr cmd-pid lif-owner numeric
                    is-lif-error nqueries lif

                    vserver
                    )
            ),
        },
        api => 'network_traceroute',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_traceroute_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_traceroute_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
        frontend => 'NACL::C::Network::traceroute',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub traceroute

sub _traceroute_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'network_traceroute',);
    $Log->exit() if $may_exit;
} ## end sub _traceroute_cmode_cli

sub _traceroute_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [qw( verbose waittime nqueries )];

    # for options that have different names in 7Mode CLI
    my $key_map_7mode_options = {
        'maxttl'      => "max-hops",
        'numeric'     => 'numaric-address',
        'port'        => 'base-port',
        'destination' => 'host',
        'packet-size' => 'packetsize',
    };

    my %target_hash;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => $key_map_7mode_options,
        target => \%target_hash,
    );

    $apiset->traceroute(%target_hash);
    $Log->exit() if $may_exit;
} ## end sub _traceroute_7mode_cli

=head2 ping

   NACL::C::Network->ping(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to ping. Throws a HostUnreachable exception
when the host is not alive.

Uses a CMode CLI/ZAPI, 7mode CLI APISet.

Invokes "net-cluster-ping" API for CMode ZAPI.

Invokes "ping6,ping" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "verbose" => $boolean >>

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : verbose

7M CLI  : -v (Programmatic name: 'verbose')

=item C<< "respTime" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : respTime

=item C<< "ttl" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : ttl

=item C<< "destination" => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : destination-address

7M CLI  : host name (Programmatic name: 'host')

=item C<< "flood" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : flood

=item C<< "node" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : node

=item C<< "packet-size" => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : packet-size

7M CLI  : -s <value> (Programmatic name: 'packetsize')

=item C<< "exitStatus" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : exitStatus

=item C<< "lif-ipaddr" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : lif-ipaddr

=item C<< "is-lif-error" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-lif-error

=item C<< "count" => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : retry-count

7M CLI  : -c <value> (Programmatic name: 'count')

=item C<< "record-route" => $boolean >>

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : record-route

7M CLI  : -R (Programmatic name: 'record-route')

=item C<< "abort-signal" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : abort-signal

=item C<< "disallow-fragmentation" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : disallow-fragmentation

=item C<< "lif-owner" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : interface-owner

=item C<< "wait" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : wait

=item C<< "cmd-pid" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : cmd-pid

=item C<< "lif" => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : interface

7M CLI  : -i <value> (Programmatic name: 'interface')

=item C<< "show-detail" => $boolean >>

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : interface

7M CLI  : -s (Programmatic name: 'datagram-every-second')

=item C<< "bypass-routing-tables" => $boolean >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI:  -r (Programmatic name: 'bypass-routing-tables')

=item C<< 'exit-status' => $string >>

Applicable for CMode CLI

=item C<< 'resp-time' => $string >>

Applicable for CMode CLI

=item C<< 'quiet' => $boolean >>

Applicable for 7Mode CLI

Maps to:

7M CLI:  -q (Programmatic name: 'quiet-output')

=item C<< 'reverse-lookup' => $boolean >>

(Optional)

Applicable for 7Mode CLI

Maps to:

7M CLI: -H (Programmatic name: 'reverse-lookup')

=item C<< 'numeric' => $boolean >>

(Optional)

Applicable for 7Mode CLI

Maps to:

7M CLI:  -n <value> (Programmatic name: 'numeric-output-only')

=item C<< 'numeric-output-only' => $boolean >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI:  -n <value> (Programmatic name: 'numeric-output-only')

=item C<< 'probe-node-information-multicast-group' => $boolean >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI:  -N (Programmatic name: 'probe-node-information-multicast-group')

=item C<< 'generate-icmpv6-node-info-fqdn-query' => $boolean >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI:  -w (Programmatic name: 'generate-icmpv6-node-info-fqdn-query')

=item C<< 'quiet-output' => $boolean >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI:  -q (Programmatic name: 'quiet-output')

=item C<< 'addrtype' => $boolean >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI: -a <value> (Programmatic name: 'addrtype')

=item C<< 'bufsize' => $string >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI:  -b <value> (Programmatic name: 'bufsize')

=item C<< 'set-so-debug' => $string >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI:  -d (Programmatic name: 'set-so-debug')

=item C<< 'so-debug' => $string >>

Applicable for 7Mode CLI

Maps to:

7M CLI:  -d (Programmatic name: 'set-so-debug')

=item C<< 'interface' => $string >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI: -i <value> (Programmatic name: 'interface')

=item C<< 'interval' => $string >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for CMode CLI.

=item C<< 'wait' => $string >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI: -T <value> (Programmatic name: 'wait')

=item C<< 'preload' => $string >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI: -l <value> (Programmatic name: 'preload')

=item C<< 'pattern' => $string >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI: -p <value> (Programmatic name: 'pattern')

=item C<< 'packetsize' => $string >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI:  -s <value> (Programmatic name: 'packetsize')

=item C<< 'sourceaddr' => $string >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI:  -S <value> (Programmatic name: 'sourceaddr')

=item C<< 'datasize' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'hoplimit' => $string >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI:  -h <value> (Programmatic name: "hoplimit")

=item C<< 'hops' => $string >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI:  hops list (Programmatic name: "hops")

=item C<< 'host' => $string >>

(Optional)
Applicable for ping6 in 7Mode only

Applicable for 7Mode CLI

Maps to:

7M CLI:  host name (Programmatic name: "host")

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'use-source-port' => $string >>

Use Source Port of Logical Interface possible value(s) are, true,false
Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< 'vserver' => $string >>

Vserver
Applicable for CMode CLI

=item C<< 'timeout' => $string >>

Timeout ranges from 1 to 600
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"network ping" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub ping {    
    $Log->enter() if $may_enter;
	my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( verbose respTime ttl destination flood node packet-size
                    exitStatus lif-ipaddr is-lif-error count record-route
                    abort-signal disallow-fragmentation lif-owner wait cmd-pid
                    lif show-detail datagram-every-second bypass-routing-tables
                    reverse-lookup numeric probe-node name-query sockbufsiz
                    interface interval preload pattern sourceaddr datasize
                    hoplimit hops exit-status resp-time quiet

                    use-source-port

                    vserver

                    timeout
                    )
            ),
        },
        api => 'network_ping',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_ping_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_ping_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_ping_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Network::ping',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub ping

sub _ping_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $command_interface = $opts{command_interface};
    my $destination       = $opts{destination};
    my $response;
    try {
        $pkg->_base_backend_cmode_cli(
            %opts,
            api                 => 'network_ping',
            apiset_response_obj => \$response
        );
    }
    catch NACL::APISet::Exceptions::CommandFailedException with {
        my $e = shift;
        if ($e->text() =~ /Host .* is unreachable|no answer/i) {
            NACL::Exceptions::HostUnreachable->throw(
                "Host '$opts{destination}' is unreachable");
        }
    }
    catch NACL::APISet::Exceptions::TimeoutException with {
        NACL::Exceptions::HostUnreachable->throw(
            "Host '$opts{destination}' is unreachable");
    };
    my $output = $response->get_parsed_output();
	my $raw_output = $response->get_raw_output();
	if ($raw_output =~ /unknown host/i)
        {
            NACL::Exceptions::HostUnreachable->throw(
                "Host '$opts{destination}' is unreachable");
        }
	my $state  = $output->[0]->{state};
	my $host   = $output->[0]->{host};
	my $obj    = NACL::CS::NetworkPing->new(
        command_interface => $command_interface,
        destination       => $destination,
        state             => $state,
        host              => $host
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub _ping_cmode_cli

sub _ping_7mode_cli {
    $Log->enter() if $may_enter;
	$Log->warn('inside 7mode ');
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $destination       = $opts{destination};
    my $apiset            = delete $opts{apiset};

    my $allowed_options;
    my $cmd;
    my %hash;

    if (Net::IP::ip_is_ipv6($opts{destination})) {
        $allowed_options = [
            qw( set-so-debug reverse-lookup numeric-output-only
                probe-node-information-multicast-group quiet-output verbose
                generate-icmpv6-node-info-fqdn-query addrtype bufsize
                count hoplimit interface wait preload pattern sourceaddr
                packetsize hops host)
        ];
        $pkg->_handle_boolean_options(
            api_options     => \%opts,
            boolean_options => [
                qw(reverse-lookup numeric-output-only
                    probe-node-information-multicast-group quiet-output verbose
                    generate-icmpv6-node-info-fqdn-query )
            ],
            is_7m_cli => 1
        );
        %hash = (
            'lif'         => 'interface',
            'destination' => 'host',
            'quiet'       => 'quiet-output',
            'packet-size' => 'packetsize',
            'numeric'     => 'numeric-output-only',
            'so-debug'    => 'set-so-debug',
            'source-addr' => 'sourceaddr'
        );
        $cmd = "ping6";
    } else {
        $allowed_options = [
            qw( destination count verbose record-route
                bypass-routing-tables packet-size
                )
        ];
        $pkg->_handle_boolean_options(
            api_options => \%opts,
            boolean_options =>
                [qw(show-detail record-route verbose bypass-routing-tables)],
            is_7m_cli => 1
        );
        %hash = (
            'lif'         => 'interface',
            'show-detail' => 'datagram-every-second',
        );
        $cmd = "ping";
    }
    my %cmd_args;

    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        map    => {%hash},
        target => \%cmd_args,
    );

    %opts = %cmd_args;

    my $response;
    try {
        $response = $apiset->$cmd(%opts);
    }
    catch NACL::APISet::Exceptions::CommandFailedException with {
        my $e = shift;
        if ($e->text()
            =~ /no answer|unknown host|No route to host|Network is unreachable/
            )
        {
            NACL::Exceptions::HostUnreachable->throw(
                "Host '$opts{destination}' is unreachable");
        }
    }
    catch NACL::APISet::Exceptions::TimeoutException with {
        NACL::Exceptions::HostUnreachable->throw(
            "Host '$opts{destination}' is unreachable");
    };
	$Log->warn('*****************inside _ping_7mode_cli **************');
    my $output = $response->get_parsed_output();
    my $state  = $output->[0]->{state};
    my $host   = $output->[0]->{host};
    my $obj    = NACL::CS::NetworkPing->new(
        command_interface => $command_interface,
        destination       => $destination,
        state             => $state,
        host              => $host
    );
    return $obj;

    $Log->exit() if $may_exit;
} ## end sub _ping_7mode_cli



sub _ping_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    my $command_interface = $opts{command_interface};
    my $destination       = $opts{destination};

    my $vserver_name = $opts{vserver};
    $opts{vserver_operated_on} = \$vserver_name if (defined $vserver_name);

    my $allowed_cmode_options = [
        qw( verbose respTime ttl flood node packet-size exitStatus lif-ipaddr
            is-lif-error record-route abort-signal disallow-fragmentation
            wait cmd-pid show-detail
            )
    ];

    try {
        $pkg->_base_backend_cmode_zapi(
            %opts,
            api  => 'net_cluster_ping',
            copy => $allowed_cmode_options,
            map => {
                "destination" => "destination-address",
                "lif"         => "interface",
                "lif-owner"   => "interface-owner",
                "count"       => "retry-count",
            },
            frontend_method => 'ping'
        );
    } catch NACL::APISet::Exceptions::CommandFailedException with {
        my $e = shift;
        if ($e->text() =~ /Host is down/i) {
            NACL::Exceptions::HostUnreachable->throw(
                "Host '$opts{destination}' is unreachable");
        }
    }
    catch NACL::APISet::Exceptions::TimeoutException with {
        NACL::Exceptions::HostUnreachable->throw(
            "Host '$opts{'destination-address'}' is unreachable");
    };

    my $obj = NACL::CS::NetworkPing->new(
        command_interface => $command_interface,
        destination       => $destination,
        state             => "alive"
    );
    return $obj;

    $Log->exit() if $may_exit;

} ## end sub _ping_cmode_zapi

=head2 ifconfig

    NACL::C::Network->ifconfig(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to [This command has been deprecated] -- Configure a logical interface or a port.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'a' => $string >>

Show All possible value(s) are, no_type
Applicable for CMode CLI

=item C<< 'flowcontrol' => $string >>

Flow Control possible value(s) are, none,receive,send,full
Applicable for CMode CLI

=item C<< 'vserver' => $string >>

Virtual Server possible value(s) are, vserver
Applicable for CMode CLI

=item C<< 'mtusize' => $string >>

MTU size possible value(s) are, integer
Applicable for CMode CLI

=item C<< 'node' => $string >>

Node name possible value(s) are, nodename
Applicable for CMode CLI

=item C<< 'port' => $string >>

Port possible value(s) are, <netport>,<ifgrp>
Applicable for CMode CLI

=item C<< 'lstatus' => $string >>

LIF Status possible value(s) are, up,down
Applicable for CMode CLI

=item C<< 'mediatype' => $string >>

Physical Media Type possible value(s) are, no_type
Applicable for CMode CLI

=item C<< 'pstatus' => $string >>

Port Status possible value(s) are, up,down
Applicable for CMode CLI

=item C<< 'netmask' => $string >>

Netmask possible value(s) are, IP Address
Applicable for CMode CLI

=item C<< 'role' => $string >>

Role possible value(s) are, no_type
Applicable for CMode CLI

=item C<< 'lif' => $string >>

Logical Interface possible value(s) are, lif-name
Applicable for CMode CLI

=item C<< 'alias' => $string >>

Alias possible value(s) are, IP Address
Applicable for CMode CLI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"network ifconfig" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub ifconfig {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( a flowcontrol vserver mtusize node port lstatus mediatype pstatus netmask role
                    lif alias
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'network_ifconfig',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_ifconfig_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Network::ifconfig',
    );

    $Log->exit() if $may_exit;
} ## end sub ifconfig

sub _ifconfig_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'network_ifconfig',);
    $Log->exit() if $may_exit;
} ## end sub _ifconfig_cmode_cli

=head2 l2ping

    NACL::C::Network->l2ping(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to layer 2 MAC ping.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'count' => $string >>

Count
Applicable for CMode CLI

=item C<< 'mac' => $string >>

MAC Address
Applicable for CMode CLI

=item C<< 'node' => $string >>

Node
Applicable for CMode CLI

=item C<< 'vlan' => $string >>

VLAN ID
Applicable for CMode CLI

=item C<< 'mdlevel' => $string >>

Maintenance Domain Level
Applicable for CMode CLI

=item C<< 'sourceport' => $string >>

Source Physical Interface
Applicable for CMode CLI

=item C<< 'pingself' => $string >>

Ping interface's own mac possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "network l2ping" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub l2ping {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( count mac node vlan mdlevel sourceport

                    pingself
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'network_l2ping',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_l2ping_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Network::l2ping',
    );

}

sub _l2ping_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'network_l2ping',);
    $Log->exit() if $may_exit;
}

=head2 ping6

my $ping6_object = NACL::C::Network->ping6(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to ping an IPv6 address.
Throws a HostUnreachable exception when the host is not alive.
It returns an object of type NACL::CS::NetworkPing

Uses a CMode CLI/ZAPI APISet.

Invokes "net-cluster-ping6" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'verbose' => $string >>

Show All ICMP Packets
Applicable for CMode CLI

=item C<< 'hop-limit' => $string >>

Intermediate Nodes
Applicable for CMode CLI

=item C<< 'destination' => $string >>

Destination
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : destination-address

=item C<< 'node' => $string >>

Node Name
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : node

=item C<< 'fqdn-query' => $string >>

FQDN query
Applicable for CMode CLI

=item C<< 'packet-size' => $string >>

Packet Size
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : packet-size

=item C<< 'numeric-output' => $string >>

Numeric output only
Applicable for CMode CLI

=item C<< 'reverse-lookup' => $string >>

Reverse-lookup of IPv6 addresses
Applicable for CMode CLI

=item C<< 'hops' => $string >>

IPv6 addresses of intermediate nodes
Applicable for CMode CLI

=item C<< 'fqdn-query-03-draft' => $string >>

FQDN query but with 03 draft
Applicable for CMode CLI

=item C<< 'count' => $string >>

Max Requests to Send/Recieve
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : retry-count

=item C<< 'vserver' => $string >>

Vserver Name
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : interface-owner

=item C<< 'interval' => $string >>

Wait between Packets (secs)
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : interval

=item C<< 'pattern' => $string >>

Up to 16 ``pad specified for out packet
Applicable for CMode CLI

=item C<< 'buffer-size' => $string >>

Socket Buffer Size
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : buffer-size

=item C<< 'preload' => $string >>

Send packets as fast as possible
Applicable for CMode CLI

=item C<< 'lif' => $string >>

Logical Interface
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : interface

=item C<< 'flood' => $string >>

Flood Ping
Applicable for CMode CLI

=item C<< 'disallow-fragmentation' => $string >>

Disallow Fragmentation for unicast and multicast addresses
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : disallow-fragmentation

=item C<< 'show-detail' => $string >>

Show Detail Output
Applicable for CMode CLI

=item C<< 'use-source-port' => $string >>

Use Source Port of Logical Interface
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "network ping6" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub ping6 {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( verbose hop-limit destination node fqdn-query packet-size numeric-output
                    reverse-lookup hops fqdn-query-03-draft count vserver interval pattern
                    buffer-size preload lif

                    flood disallow-fragmentation show-detail

                    use-source-port
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'network_ping6',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_ping6_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_ping6_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Network::ping6',
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _ping6_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $command_interface = $opts{command_interface};
    my $destination       = $opts{destination};
    my $response;
    try {
        $pkg->_base_backend_cmode_cli(
            %opts,
            api                 => 'network_ping6',
            apiset_response_obj => \$response
        );
    }
    catch NACL::APISet::Exceptions::CommandFailedException with {
        my $error = shift;
        if ($error->text() =~ /no\s+answer|ping\s+encountered\s+an\s+error/) {
            $Log->exit() if $may_exit;
            NACL::Exceptions::HostUnreachable->throw($error->text());
        }
    }
    catch NACL::APISet::Exceptions::TimeoutException with {
        $Log->exit() if $may_exit;
        NACL::Exceptions::HostUnreachable->throw(
            "Host $destination is unreachable");
    };

    my $output = $response->get_parsed_output();
    my $state  = $output->[0]->{state};
    my $host   = $output->[0]->{host};
    my $obj    = NACL::CS::NetworkPing->new(
        command_interface => $command_interface,
        destination       => $destination,
        state             => $state,
        host              => $host
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _ping6_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $command_interface = $opts{command_interface};
    my $destination       = $opts{destination};
    try {
        $pkg->_base_backend_cmode_zapi(
            %opts,
            api  => 'net-cluster-ping6',
            copy => [
                qw( interval node buffer-size disallow-fragmentation packet-size )
            ],
            map => {
                "destination" => "destination-address",
                "lif"         => "interface",
                "count"       => "retry-count",
                "vserver"     => "interface-owner",
            },
            frontend_method => 'ping6'
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $error = shift;
        if ($error->text()
            =~ /Host\s+is\s+down||ping\s+encountered\s+an\s+error/)
        {
            $Log->exit() if $may_exit;
            NACL::Exceptions::HostUnreachable->throw(
                "Host $destination is unreachable");
        }
    }
    catch NACL::APISet::Exceptions::TimeoutException with {
        $Log->exit() if $may_exit;
        NACL::Exceptions::HostUnreachable->throw(
            "Host $destination is unreachable");
    };
    my $obj = NACL::CS::NetworkPing->new(
        command_interface => $command_interface,
        destination       => $destination,
        state             => "alive"
    );
    $Log->exit() if $may_exit;
    return $obj;
}

=head2 traceroute6

    NACL::C::Network->traceroute6(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to traceroute6.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'verbose' => $string >>

Verbose Output
Applicable for CMode CLI

=item C<< 'wait-time' => $string >>

Wait Between Packets (secs)
Applicable for CMode CLI

=item C<< 'hop-limit' => $string >>

Maximum Number of Hops
Applicable for CMode CLI

=item C<< 'vserver' => $string >>

LIF Owner
Applicable for CMode CLI

=item C<< 'destination' => $string >>

Destination
Applicable for CMode CLI

=item C<< 'gateway' => $string >>

Intermediate Gateway
Applicable for CMode CLI

=item C<< 'icmp6' => $string >>

ICMP6 ECHO instead of UDP
Applicable for CMode CLI

=item C<< 'port' => $string >>

Base UDP Port Number
Applicable for CMode CLI

=item C<< 'node' => $string >>

Node
Applicable for CMode CLI

=item C<< 'debug-mode' => $string >>

Debug Mode
Applicable for CMode CLI

=item C<< 'packet-size' => $string >>

Packet Size ranges from 12 to 65534
Applicable for CMode CLI

=item C<< 'numeric' => $string >>

Print Hops Numerically
Applicable for CMode CLI

=item C<< 'udp' => $string >>

UDP
Applicable for CMode CLI

=item C<< 'lif' => $string >>

Logical Interface
Applicable for CMode CLI

=item C<< 'nqueries' => $string >>

Number of Queries
Applicable for CMode CLI

=item C<< 'first-hop' => $string >>

Number of Hops to Skip in Trace
Applicable for CMode CLI

=item C<< 'vserver' => $string >>

LIF Owner
Applicable for CMode CLI

=item C<< 'lif-ipaddr' => $string >>

LIF IP Address
Applicable for CMode CLI

=item C<< 'is-lif-error' => $string >>

Error when Querying LIF? possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'is-node-error' => $string >>

Node Set in Vserver Context? possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'abort-signal' => $string >>

Signal sent for Ctr-C
Applicable for CMode CLI

=item C<< 'cmd-pid' => $string >>

Command PID
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "network traceroute6" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub traceroute6 {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( verbose hop-limit wait-time gateway destination icmp6 node port
                    debug-mode packet-size udp numeric lif nqueries first-hop

                    vserver

                    lif-ipaddr is-lif-error is-node-error abort-signal cmd-pid
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'network_traceroute6',
    );
    if (defined $opts{vserver}
        || $opts{command_interface}->isa('NACL::C::Vserver'))
    {
        my $vserver;
        $opts{vserver_operated_on} = \$vserver;
    }

    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_traceroute6_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_traceroute6_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Network::traceroute6',
    );

    $Log->exit() if $may_exit;
    return $pkg->_format($obj);
}

sub _traceroute6_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $command_interface = $opts{command_interface};
    my $response;
    $pkg->_base_backend_cmode_cli(
        %opts,
        api                 => 'network_traceroute6',
        apiset_response_obj => \$response,
    );

    $Log->exit() if $may_exit;
    return $response;
}

sub _traceroute6_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $response;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'net-traceroute6',
        copy => [
            qw( node debug-mode numeric verbose first-hop gateway hop-limit
                port nqueries wait-time packet-size )
        ],
        map => {
            'icmp6'       => 'use-icmp6',
            'udp'         => 'use-udp',
            'lif'         => 'interface',
            'destination' => 'destination-address',
        },
        apiset_response_obj => \$response,
    );

    $Log->exit() if $may_exit;
    return $response;
} ## end sub _traceroute6_cmode_zapi

sub _format {
    $Log->enter() if $may_enter;
    my ($pkg, $pattern) = @_;
    my $array = $pattern->{output};
    my @array = split(/\n/, $array);
    my $line;
    if ($array[-1] =~ /<\/results>/) {
        $line = $array[-3];
    } else {
        $line = $array[-1];
    }
    my $key_pattern;
    if ($line =~ /^\s*\d+\s+([\w+\.\:]+)/s) {
        $key_pattern = $1;
    }
    if (ip_is_ipv6($key_pattern)
        || $key_pattern
        =~ /([a-zA-Z0-9]([a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])?\.)+[a-zA-Z]{2,6}/)
    {
        return 1;
    } else {
        return 0;
    }
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'destination' => {type => SCALAR},
        'node'        => {type => SCALAR, optional => 1},
    );
} ## end sub _primary_keys_validate_spec

1;

