#
# Copyright (c) 2001-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkInterface Component Module
## @author Rupamjyoti.Baruah@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::NetworkInterface

=head1 DESCRIPTION

C<NACL::C::NetworkInterface> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP Network NetworkInterface.  A related class is
L<NACL::CS::NetworkInterface|lib-NACL-CS-NetworkInterface-pm>, which represents
the past state of an ONTAP Network Interface.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2  lif

The name of the logical interface ,used to identify it uniquely.The name of the
logical interface is unique within a particular vserver

=head2  vserver

The name of the vserver that contains the lif.

=head2  server

The name of the vserver that contains the lif.

=cut

package NACL::C::NetworkInterface;

use strict;
use warnings;
use NACL::CS::NetworkInterface;
use NACL::CS::InterfaceShowRoutingGroup;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(:all);
use NACL::C::Exceptions::NetworkInterface::MaxInterfacesPerNodeReached;
use NACL::C::Exceptions::NetworkInterface::AlreadyExists;
use NACL::C::Exceptions::NetworkInterface::DoesNotExist;
use NACL::C::Exceptions::NetworkInterface::IpAlreadyBound;
use NACL::C::Exceptions::NetworkInterface::PortHealthStatusDegraded;
use NACL::C::Exceptions::NetworkInterface::LifEnabled;
use NACL::C::Exceptions::NetworkInterface::NoAddressInSubnet;
use NATE::Exceptions::Argument qw(:try);
use NACL::Exceptions::InvalidChoice;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 new

my $lif = NACL::C::NetworkInterface->new(command_interface=>$ci,
                                  vserver => $vserver_name,
                                  lif => $lif_name );

Return a new NetworkInterface component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'lif',
    scalar => 'vserver',
    scalar => 'server',
];

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::C::Interface'
    );
    $Log->exit() if $may_exit;
    return $isa;
} ## end sub isa

=head2 find

    my $Interface  = NACL::C::NetworkInterface->find(command_interface => $ci, ...);
    my @Interfaces = NACL::C::NetworkInterface->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Note that there are additional facilities (such as being able to limit
to interfaces local/remote to an element and be able to find interfaces
for a particular volume) provided for the find() of this
component. See the documentation of
L<NACL::CS::NetworkInterface::fetch|lib-NACL-CS-NetworkInterface-pm/fetch>
for a description of the additional options accepted.

More explanation, along with examples, are available at the User Guide:
http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#find

=head2 CANNED FILTERS

See
http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Canned_filters
for a description of canned filters.

=head2 up_interfaces

Returns those interfaces which are up, i.e. "status-admin" and "status-oper"
are both "up".

=head2 data_interfaces

Returns the data interfaces (i.e. role = "data" )

=head2 up_data_interfaces

Returns the data interfaces which are up. (see above for definitions of
"up" and "data")

=head2 up_data_nfs_interfaces

Returns the up data interfaces (see above) which have nfs as one of their
supported data protocols.

=head2 up_data_cifs_interfaces

Returns the up data interfaces (see above) which have cifs as one of their
supported data protocols.

=head2 up_data_iscsi_interfaces

Returns the up data interfaces (see above) which have iscsi as one of their
supported data protocols.

=head2 up_data_fcache_interfaces

Returns the up data interfaces (see above) which have fcache as one of their
supported data protocols.

=head2 FIND WRAPPERS

Each of the canned filters have a wrapper method (implemented through
AUTOLOAD) of the form:

    find_<canned-filter-name>

For example, there is a C<find_up_data_nfs_interfaces> method for the
C<up_data_nfs_interfaces> canned filter.

    NACL::C::NetworkInterface->find_up_data_nfs_interfaces(...);

These methods accept all of the arguments normally accepted by find()
but automatically apply the canned filter contained in its name.

=head2 state

    my $interface_state = $interface_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::NetworkInterface would return
a NACL::CS::NetworkInterface object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=head2 create

  my $lif  = NACL::C::NetworkInterface->create(command_interface=>$ci,
                                        vserver => $vserver_name,
                                        lif => $lif_name);

(Class method) Request creation of a new interface element, and  return a new
interface component object that refers to it.

Uses a CMode CLI/ZAPI APISet.

Invokes "net-interface-create" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< lif=> $lif_name >>

(Required) Name of logical interface to be created.
Maps to interface-name for CMode ZAPI

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : interface-name

=item C<< vserver=> $vserver_name >>

(Required) Name of the vserver for which the logical interface is created.
Maps to vserver for CMode ZAPI.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< role=> $role >>

(Required) Role of the logical interface that is created.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : role

=item C<< "home-node" => $homenode >>

(Required) Name of the node in which the lif needs to be created.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : home-node

=item C<< "home-port"=> $homeport >>

(Required) Name of the port in which the lif needs to be created.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : home-port

=item C<< address=> $address >>

(Optional) Ipaddress that will be assigned to this lif.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : address

=item C<< netmask=> $netmask >>

(Optional) Netmask for the ipaddress.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : netmask

=item C<< "data-protocol"=> $data_protocol >>

(Optional) data protocols that will be supported by this interface
Maps to data-protocols for CMode ZAPI.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : data-protocols

=item C<< "routing-group"=> $routing_group >>
Maps to routing-group-name for CMode ZAPI.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : routing-group-name

=item C<< server=> $vserver_name >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< "status-admin"=> $status_admin >>
Maps to administrative-status for CMode ZAPI.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : status-admin

=item C<< "failover-policy"=> $failover_policy >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : failover-policy

=item C<< "firewall-policy"=> $firewall_policy >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : firewall-policy

=item C<< "auto-revert"=> $boolean >>
Maps to is-auto-revert for CMode ZAPI

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : auto-revert

=item C<< "use-failover-group"=> $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : use-failover-group

=item C<< "dns-zone"=> $string >>
Maps to dns-domain-name for CMode ZAPI.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : dns-zone

=item C<< "allow-lb-migrate"=> $boolean >>

Applicable for CMode CLI

=item C<< "lb-weight"=> $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : lb-weight

=item C<< "failover-group"=> $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : failover-group

=item C<< 'status-vserver' => $string >>

Applicable for CMode CLI.

=item C<< 'comment' => $string >>

Applicable for CMode CLI.

=item C<< 'auto' => $string >>
Maps to is-ipv4-link-local for CMode ZAPI

Applicable for CMode CLI.

=item C<< 'listen-for-dns-query' => $string >>

Applicable for CMode CLI.

=item C<< 'netmask-length' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : netmask-length

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'auto-configuration' => $string >>

Use Address Autoconfiguration
Applicable for CMode CLI

=item C<< 'force-subnet-association' => $string >>

Force the LIF's Subnet Association
Applicable for CMode CLI/ZAPI

=item C<< 'subnet-name' => $string >>

Subnet Name
Applicable for CMode CLI/ZAPI

=item C<< 'force-subnet-association' => $string >>

Force the LIF's Subnet Association
Applicable for CMode CLI

=item C<< 'ipv6_address_expiry' => $string >>

IPv6 address expiry possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'default-rtpid' => $string >>

Default Relative Target Portal ID
Applicable for CMode CLI

=item C<< 'remote-home-node' => $string >>

Home Node On Primary Cluster
Applicable for CMode CLI

=item C<< 'remote-home-port' => $string >>

Home Port On Primary Cluster possible value(s) are, <netport>,<ifgrp>
Applicable for CMode CLI

=item C<< 'is-verify' => $string >>

Verify LIF Placement possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'preferred-lifetime' => $string >>

Preferred Lifetime (autoconf)
Applicable for CMode CLI

=item C<< 'valid-lifetime' => $string >>

Valid Lifetime (autoconf)
Applicable for CMode CLI

=item C<< 'alua-tpgid' => $string >>

ALUA tpgid for iSCSI and FCP LIF
Applicable for CMode CLI

=item C<< 'default-tpgt' => $string >>

Default TPGroup Tag
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"network interface create" command in ONTAP.

Applicable for CMode CLI.

=item C<< nacl_is_pre_cluster => 0 | 1 >>

(Optional, defaults to 0)

This option handles the network interface object behaviour to
prepend the node-name to the lif if it is not passed in case of
cluster lif. By default considers it to be post-cluster creation.

If this option is set to 1, prepending node-name in object is skipped.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::NetworkInterface::MaxInterfacesPerNodeReached>

This type of exception is thrown when an attempt is made to create a
logical interface to a node which has reached its maximum supported number of interfaces.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::NetworkInterface::AlreadyExists>

This type of exception is thrown when an attempt is made to create an
existing logical interface.

=item C<NACL::Exceptions::InvalidChoice>

This exception is thrown if allow-lb-migrate option is specified and interface
is specifed as ZAPI through the apiset_must rule.

=item C<NACL::C::Exceptions::NetworkInterface::IpAlreadyBound>

This type of exception is thrown when an attempt is made to create a
lif using an IP address which is already bound to another lif.

=item C<NACL::C::Exceptions::NetworkInterface::PortHealthStatusDegraded>

This type of exception is thrown when an attempt is made to create a
lif on degraded port.

=item C<NACL::C::Exceptions::NetworkInterface::NoAddressInSubnet>

This type of exception is thrown when an attempt is made to create a
lif on a subnet where no address is available.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "data-protocol" => {type => SCALAR | ARRAYREF, optional => 1},
            nacl_is_pre_cluster => {type => BOOLEAN, default => 0},
            _optional_scalars(
                qw (role home-node
                    home-port address netmask routing-group
                    status-admin failover-policy firewall-policy
                    auto-revert use-failover-group dns-zone
                    allow-lb-migrate lb-weight failover-group
                    status-vserver comment auto listen-for-dns-query
                    netmask-length auto-configuration subnet-name
                    force-subnet-association
                    ipv6_address_expiry default-rtpid remote-home-node remote-home-port
                    is-verify preferred-lifetime valid-lifetime alua-tpgid default-tpgt
                    )
            )
        },
        cli_cmd => 'network interface create',
    );

    my $failover_policy = $pkg->_set_failover_policy(args => \%opts);
    my $nacl_is_pre_cluster = delete $opts{nacl_is_pre_cluster};

    $pkg->_scalar_to_array_deprication_warning(
        params  => ['data-protocol'],
        options => \%opts
    );
    try {
         $pkg->call_on_apiset(
             %opts,
             choices => [
                 {   method    => '_create_cmode_cli',
                     interface => 'CLI',
                     set       => 'CMode',
                 },
                 {   method    => '_create_cmode_zapi',
                     interface => 'ZAPI',
                     set       => 'CMode',
                     check     => '_zapi_check_allow_lb_migrate'
                 },
             ],
             frontend => 'NACL::C::NetworkInterface::create',
         );
     }
     catch NACL::APISet::Exceptions::ResponseException with {
         my $exception = shift;
         NACL::C::Exceptions::NetworkInterface::MaxInterfacesPerNodeReached
             ->detect_convert_and_throw('exception' => $exception,);
         NACL::C::Exceptions::NetworkInterface::AlreadyExists
             ->detect_convert_and_throw('exception' => $exception,);
         NACL::C::Exceptions::NetworkInterface::IpAlreadyBound
             ->detect_convert_and_throw('exception' => $exception,);
	 NACL::C::Exceptions::NetworkInterface::PortHealthStatusDegraded
             ->detect_convert_and_throw('exception' => $exception,);
	 NACL::C::Exceptions::NetworkInterface::NoAddressInSubnet
	     ->detect_convert_and_throw('exception' => $exception,);
         $Log->exit() if $may_exit;
         $exception->throw();
     };
    my %new_opts;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%new_opts,
        copy   => [qw(lif vserver)],
        map    => {server => 'vserver'},
    );
    $pkg->_process_options(options => \%new_opts);
    unless ($nacl_is_pre_cluster) {
        if ( $new_opts{vserver} =~ /Cluster/ &&
            $new_opts{lif} !~ /^$opts{'home-node'}/  ) {
            $new_opts{lif} = $opts{'home-node'} . "_" . $new_opts{lif};
        }
    }
    my $obj = $pkg->new(command_interface => $opts{command_interface},
        %new_opts);
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;


    if (defined($opts{'data-protocol'})
        && $opts{'data-protocol'} =~ /fcp/i)
    {
        delete $opts{'address'};
    }
    $pkg->_base_backend_cmode_cli(%opts, api => 'network_interface_create');
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    # CLI defaults "role" to "data" but it is mandatory for ZAPI
    $opts{role} ||= 'data';
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'net-interface-create',
        copy => [
            qw(home-node home-port failover-policy firewall-policy
                use-failover-group lb-weight netmask-length
                failover-group role address netmask vserver
                subnet-name force-subnet-association)
        ],
        map  => {
            "lif"           => "interface-name",
            "server"        => "vserver",
            "routing-group" => "routing-group-name",
            "data-protocol" => "data-protocols",
            "status-admin"  => "administrative-status",
            "dns-zone"      => "dns-domain-name",
            "auto-revert"   => "is-auto-revert",
            "auto"          => "is-ipv4-link-local",
        },
    );

    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

=head2 delete

  NACL::C::NetworkInterface->delete(command_interface=>$ci,
                             lif=>$lif_name,
                             vserver=>$vserver_name...);

                            or

  $interface_object->delete(...);

(Class or instance method) This method deletes an interface element.

Uses a CMode CLI/ZAPI APISet.

Invokes "net-interface-delete" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< lif=>$lif_name >>

(Required for class method, Not Applicable for instance method)
Name of lif

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : interface-name

=item C<< server=> $vserver_name >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< vserver => $vserver_name >>
(Required for class method, Not Applicable for instance method)
Name of the vserver

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"network interface delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::NetworkInterface::DoesNotExist>

This type of exception is thrown when an attempt is made to delete a
non-existing logical interface.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd    => 'network interface delete',
    );

    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::NetworkInterface::delete',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        foreach my $type (
            qw(DoesNotExist LifEnabled)
            )
        {
            my $full_type = "NACL::C::Exceptions::NetworkInterface::$type";
            $full_type->detect_convert_and_throw(
                exception => $exception);
        }
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api  => 'network_interface_delete');
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'net_interface_delete',
            copy   => [qw(vserver)],
            map    => {
                "lif"    => "interface-name",
                "server" => "vserver"
            },
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 migrate

  NACL::C::NetworkInterface->migrate(command_interface=>$ci,
                              lif=>$lif_name,
                              vserver => $vserver_name,
                             "dest-node" => $destination_node );

                            or

  $interface_object->migrate(...);

(Class or instance method) This method migrates an interface element to a
different port .

Uses a CMode CLI APISet.

Invokes "net-interface-migrate" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< lif=>$lif_name >>

(Required for class method, Not Applicable for instance method)
Name of lif.

Applicable for CMode CLI/ZAPI

=item C<< vserver => $vserver_name >>

(Required for class method, Not Applicable for instance method)
Name of the vserver

Applicable for CMode CLI/ZAPI.

=item C<< "dest-node" => $destination_node >>

(Required) Name of the node to which this interface is migrated

Applicable for CMode CLI.

Maps to:

CM ZAPI : 'destination-node'

<net-interface-migrate>
<destination-node>$value</destination-node>
</net-interface-migrate>

=item C<< "dest-port" => $destination_port >>

(Required) Name of the destination-port to which this interface is migrated

Applicable for CMode CLI

=item C<< "force" => $boolean >>

Applicable for CMode CLI/ZAPI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'destination-port' => $string >>

Destination Port possible value(s) are, <netport>,<ifgrp>
Applicable for CMode CLI/ZAPI

=item C<< 'destination-node' => $string >>

Destination Node
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by
"network interface migrate" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub migrate {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw (dest-node
                    dest-port force
                    destination-port destination-node
                    )
            )
        },
        cli_cmd => 'network interface migrate',
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_migrate_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_migrate_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkInterface::migrate',
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub migrate

sub _migrate_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args, api  => 'network_interface_migrate');
    $Log->exit() if $may_exit;
} ## end sub _migrate_cmode_cli

sub _migrate_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $copy = [ qw(lif force vserver destination-node
                 destination-port)];
    my $map  = {'dest-node' => 'destination-node',};
    my $lifs = $opts{lif};
    my @lif_array;
    my $delimiter_exist = 0;
    my @lif_delimiters = ('\|', '\,');
    foreach my $delimiter (@lif_delimiters) {
        if ($lifs =~ /$delimiter/) {
            $delimiter_exist = 1;
            @lif_array = split("$delimiter", $lifs);
            last;
        }
    }
    if ($delimiter_exist == 1) {
         foreach my $lif (@lif_array) {
             $opts{lif} =  $lif;
             $pkg->_base_backend_cmode_zapi(
                 %opts,
                 api  => 'net-interface-migrate',
                 copy => $copy,
	         map  => $map,
             );
         }
     }else{
         $pkg->_base_backend_cmode_zapi(
                 %opts,
                 api  => 'net-interface-migrate',
                 copy => $copy,
                 map  => $map,
             );
     }
    $Log->exit() if $may_exit;
}

=head2 revert

  NACL::C::NetworkInterface->revert(command_interface=>$ci,
                             lif=>$lif_name,
                             vserver => $vserver_name
                             );

                            or

  $interface_object->revert(...);

(Class or instance method) This method reverts an interface element to its
home port .

Uses a CMode CLI APISet.

Invokes "net-interface-revert" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< lif=>$lif_name >>

(Required for class method, Not Applicable for instance method)
Name of lif.

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

<net-interface-revert>
 <interface-name>$value</interface-name>
</net-interface-revert>

=item C<< vserver => $vserver_name >>

(Required for class method, Not Applicable for instance method)
Name of the vserver

Applicable for CMode CLI/ZAPI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"network interface revert" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub revert {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'network_interface_revert',
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_revert_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_revert_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                check     => '_revert_zapi_check',
            },
        ],
        frontend => 'NACL::C::NetworkInterface::revert',
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub revert

sub _revert_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args, api => 'network_interface_revert');
    $Log->exit() if $may_exit;
} ## end sub _revert_cmode_cli

sub _revert_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(vserver)],
        map  => {'lif' => 'interface-name'},
        api  => 'net-interface-revert',
    );
    $Log->exit() if $may_exit;
}

sub _revert_zapi_check {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my @fields_to_check = ('lif','vserver');
    $pkg_or_obj->_fallback_for_relational_regex(%opts, fields_to_check => \@fields_to_check);

    $Log->exit() if $may_exit;
} ## end sub _revert_zapi_check    

=head2 rename

  NACL::C::NetworkInterface->rename(command_interface=>$ci,
                             lif=>$lif_name,
                             vserver => $vserver_name,
                             newname => $new_name
                             );

                            or

  $interface_object->rename(...);

(Class or instance method) This method renames an interface element.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< lif=>$lif_name >>

(Required for class method, Not Applicable for instance method)
Name of lif.

Applicable for CMode CLI.

=item C<< vserver => $vserver_name >>

(Required for class method, Not Applicable for instance method)
Name of the vserver

Applicable for CMode CLI.

=item C<< "new-name" => $new_name >>

 (Required) New name for the logical interface


Applicable for CMode CLI.

 =item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"network interface rename" command in ONTAP.

Applicable for CMode CLI.

=item C<< nacl_is_pre_cluster => 0 | 1 >>

(Optional, defaults to 0)

This option handles the network interface object behaviour to
prepend the node-name to the lif if it is not passed in case of
cluster lif. By default considers it to be post-cluster creation.

If this option is set to 1, prepending node-name in object is skipped.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub rename {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            nacl_is_pre_cluster => {type => BOOLEAN, default => 0},
            _updated_newname    => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw (newname
                    )
            )
        },
        cli_cmd => 'network interface rename',
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_rename_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkInterface::rename',
    );
 
    # See burt 802513 for cluster lif naming behaviour
    # Updating the lif name by prepending node-name
    $pkg_or_obj->_get_updated_newname(%opts);
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub rename

sub _rename_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args, api => 'network_interface_rename',);
    $Log->exit() if $may_exit;
} ## end sub _rename_cmode_cli

=head2 enable_readonly

  NACL::C::NetworkInterface->enable_readonly(command_interface=>$ci,
                             lif=>$lif_name,
                             vserver => $vserver_name);

                            or

  $interface_object->enable_readonly(...);

(Class or instance method) This method enables the logical
interface in read-only mode.

Uses a CMode CLI/ZAPI APISet.

Invokes "net-enable-readonly" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< lif=>$lif_name >>

(Required for class method, Not Applicable for instance method)
Name of lif.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: interface-name

=item C<< vserver => $vserver_name >>

(Required for class method, Not Applicable for instance method)
Name of the vserver

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: vserver

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"network interface enable-readonly" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub enable_readonly {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd    => 'network_interface_enable_readonly',
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_enable_readonly_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_enable_readonly_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkInterface::enable_readonly',
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub enable_readonly

sub _enable_readonly_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args, api  => 'network_interface_enable_readonly');
    $Log->exit() if $may_exit;
} ## end sub _enable_readonly_cmode_cli

sub _enable_readonly_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'net_enable_readonly',
            copy   => [qw( vserver )],
            map    => {'lif' => 'interface-name'},
	);
    $Log->exit() if $may_exit;
} ## end sub _enable_readonly_cmode_zapi

=head2 disable_readonly

  NACL::C::NetworkInterface->disable_readonly(command_interface=>$ci,
                             lif=>$lif_name,
                             vserver => $vserver_name);

                            or

  $interface_object->disable_readonly(...);

(Class or instance method)
This method disables the logical interface in read-only mode.

Uses a CMode CLI/ZAPI APISet.

Invokes "net-disable-readonly" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< lif=>$lif_name >>

(Required for class method, Not Applicable for instance method)
Name of lif.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: interface-name

=item C<< vserver => $vserver_name >>

(Required for class method, Not Applicable for instance method)
Name of the vserver

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: vserver

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"network interface disable-readonly" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub disable_readonly {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'network_interface_disable_readonly',
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_disable_readonly_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_disable_readonly_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkInterface::disable_readonly',
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub disable_readonly

sub _disable_readonly_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args, api => 'network_interface_disable_readonly');
    $Log->exit() if $may_exit;
} ## end sub _disable_readonly_cmode_cli

sub _disable_readonly_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1,
    );
    my $apiset = $opts{apiset};

    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw( vserver )],
        map    => {'lif' => 'interface-name'},
        target => \%cmode_args,
    );

    $apiset->net_disable_readonly(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _disable_readonly_cmode_zapi

=head2 modify

  NACL::C::NetworkInterface->modify(command_interface=>$ci,
                             server => $vserver_name,
                             lif => $lif_name);

                            or

  $interface_object->modify(...);

(Class or instance method) Modify a NetworkInterface element.

Uses a CMode ZAPI/CLI APISet.

Invokes "net-interface-modify-iter" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< lif=> $lif_name >>

(Required) Name of logical interface to be created.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : interface-name

=item C<< vserver=> $vserver_name >>

(Required) Name of the vserver for which the logical interface is created.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< server=> $vserver_name >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< role=> $role >>

(Required) Role of the logical interface that is created.

Applicable for CMode CLI.

=item C<< "home-node" => $homenode >>

(Required) Name of the node in which the lif needs to be created.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : home-node

=item C<< "home-port"=> $homeport >>

(Required) Name of the port in which the lif needs to be created.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : home-port

=item C<< address=> $address >>

(Optional) Ipaddress that will be assigned to this lif.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : address

=item C<< netmask=> $netmask >>

(Optional) Netmask for the ipaddress.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : netmask

=item C<< "data-protocol"=> $data_protocol >>

(Optional) data protocols that will be supported by this interface

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : data-protocol

=item C<< "routing-group"=> $routing_group >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : routing-group-name

=item C<< "status-admin"=> $status_admin >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : administrative-status

=item C<< "failover-policy"=> $failover_policy >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : failover-policy

=item C<< "firewall-policy"=> $firewall_policy >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : firewall-policy

=item C<< "auto-revert"=> $boolean >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-auto-revert

=item C<< "use-failover-group"=> $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : use-failover-group

=item C<< "dns-zone"=> $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : dns-domain-name

=item C<< "allow-lb-migrate"=> $boolean >>

Applicable for CMode CLI.

=item C<< "lb-weight"=> $string >>

Applicable for CMode CLI.

=item C<< "failover-group"=> $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : failover-group

=item C<< 'status-vserver' => $string >>

Applicable for CMode CLI.

=item C<< 'comment' => $string >>

Applicable for CMode CLI.

=item C<< 'auto' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-ipv4-link-local

=item C<< 'listen-for-dns-query' => $string >>

Applicable for CMode CLI.

=item C<< 'netmask-length' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : netmask-length

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'force-subnet-association' => $string >>

Force the LIF's Subnet Association
Applicable for CMode CLI/ZAPI

=item C<< 'subnet-name' => $string >>

Subnet Name
Applicable for CMode CLI

=item C<< 'force-subnet-association' => $string >>

Force the LIF's Subnet Association
Applicable for CMode CLI

=item C<< 'preferred-lifetime' => $string >>

Preferred Lifetime (autoconf)
Applicable for CMode CLI

=item C<< 'ipv6_address_expiry' => $string >>

IPv6 address expiry possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'valid-lifetime' => $string >>

Valid Lifetime (autoconf)
Applicable for CMode CLI

=item C<< 'alua-tpgid' => $string >>

ALUA tpgid for iSCSI and FCP LIF
Applicable for CMode CLI

=item C<< 'default-rtpid' => $string >>

Default Relative Target Portal ID
Applicable for CMode CLI

=item C<< 'remote-home-node' => $string >>

Home Node On Primary Cluster
Applicable for CMode CLI

=item C<< 'default-tpgt' => $string >>

Default TPGroup Tag
Applicable for CMode CLI

=item C<< 'remote-home-port' => $string >>

Home Port On Primary Cluster possible value(s) are, <netport>,<ifgrp>
Applicable for CMode CLI

=item C<< 'is-verify' => $string >>

Verify LIF Placement possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"network interface modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::Exceptions::InvalidChoice>

This exception is thrown if allow-lb-migrate option is specified and interface
is specifed as ZAPI through the apiset_must rule.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(role home-node
                    home-port data-protocol address netmask routing-group
                    status-admin failover-policy firewall-policy
                    auto-revert use-failover-group dns-zone
                    allow-lb-migrate lb-weight failover-group
                    status-vserver comment auto listen-for-dns-query
                    netmask-length subnet-name force-subnet-association

                    preferred-lifetime ipv6_address_expiry valid-lifetime alua-tpgid
                    default-rtpid remote-home-node default-tpgt remote-home-port is-verify
                    )
            )
        },
        cli_cmd => 'network interface modify',
    );

    $pkg_or_obj->_set_failover_policy(args => \%opts);

    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_modify_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_modify_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                    check     => '_zapi_check_allow_lb_migrate',
                },
            ],
            frontend => 'NACL::C::NetworkInterface::modify',
        );
    }
    catch NACL::C::Exceptions::Component with {
        $Log->exit() if $may_exit;
        $_[0]->throw();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        my $doesnot_exist
            = 'NACL::C::Exceptions::NetworkInterface::DoesNotExist';
        $doesnot_exist->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args , api =>'network_interface_modify');
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %args) = @_;

    $args{'failover-policy'} =~ s/-/_/g if (defined $args{'failover-policy'});

    $pkg_or_obj->_iter_helper(
        %args,
        api  => 'net-interface-modify-iter',
        'show_cmd' => 'network interface show',
        copy => [
            qw(home-node home-port address data-protocol comment role
                failover-group failover-policy firewall-policy vserver
                netmask use-failover-group netmask-length subnet-name
                force-subnet-association is-dns-update-enabled)
        ],
        map => {
            "lif"           => "interface-name",
            "server"        => "vserver",
            "routing-group" => "routing-group-name",
            "auto-revert"   => "is-auto-revert",
            "dns-zone"      => "dns-domain-name",
            "status-admin"  => "administrative-status",
            "auto"          => "is-ipv4-link-local",
        },
        does_not_exist_error => "entry doesn't exist",
        does_not_exist_exception =>
            'NACL::C::Exceptions::NetworkInterface::DoesNotExist'
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

sub _set_failover_policy {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;

    my $args = $opts{args};
    my $ci   = $args->{command_interface};
    # Burt: 774456
    if (defined $args->{'failover-policy'}
        and $args->{'failover-policy'} eq "ipspace-wide")
    {
        $args->{'failover-policy'} = "broadcast-domain-wide";
    }


    $Log->exit() if $may_exit;
}

=head2 init

Derived class of init - checks for any optional keys and calls
$self->SUPER::init(@_) first to get the rest of the argument checking

=cut

sub init {
    my ($self, %opts) = @_;
    $self->SUPER::init(%opts);
    if (!defined($opts{server}) && !defined($opts{vserver})) {
        NATE::Exceptions::Argument->throw("Specify either server or vserver");
    }
    unless ($opts{server} || $opts{vserver}) {
        NATE::Exceptions::Argument->throw("Missing one of the required keys");
    }
} ## end sub init

=head2 show_zones

    my $show_zones_obj = NACL::C::NetworkInterface->show_zones(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @show_zones_objs = NACL::C::NetworkInterface->show_zones(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $show_zones_obj = $NetworkInterface_obj->show_zones();

(Class or Instance method)
This method returns objects of the type L<NACL::CS::NetworkInterfaceShowZones|lib-NACL-CS-NetworkInterfaceShowZones-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch>
(requested_fields, filter, allow_empty) and has all of the other properties of
fetch methods (callable in scalar or list context, throws a NoElementsFound
exception if no matching elements are found)

=cut

sub show_zones {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::NetworkInterfaceShowZones;
    my @state_objs = NACL::CS::NetworkInterfaceShowZones->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub show_zones

=head2 start_cluster_check

    NACL::C::NetworkInterface->start_cluster_check(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to start the cluster check function.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"network interface start-cluster-check" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start_cluster_check {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        api                 => 'network_interface_start_cluster_check',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_cluster_check_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkInterface::start_cluster_check',
    );

    $Log->exit() if $may_exit;
} ## end sub start_cluster_check

sub _start_cluster_check_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args, api => 'network_interface_start_cluster_check');
    $Log->exit() if $may_exit;
} ## end sub _start_cluster_check_cmode_cli

=head2 migrate_all

    NACL::C::NetworkInterface->migrate_all(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to migrate all data and cluster management logical interfaces away from the specified node.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'synchronous' => $string >>

Issue a synchronous migrate, possible value(s) are true,false

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'port' => $string >>

Port possible value(s) are, <netport>,<ifgrp>
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"network interface migrate-all" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub migrate_all {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( node

                    synchronous

                    port
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'network_interface_migrate_all',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_migrate_all_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkInterface::migrate_all',
    );

    $Log->exit() if $may_exit;
} ## end sub migrate_all

sub _migrate_all_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args, api => 'network_interface_migrate_all');
    $Log->exit() if $may_exit;
} ## end sub _migrate_all_cmode_cli

=head2 show_routing_group

    my $show_routing_group_obj = NACL::C::NetworkInterface->show_routing_group(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @show_routing_group_objs = NACL::C::NetworkInterface->show_routing_group(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $show_routing_group_obj = $NetworkInterface_obj->show_routing_group();

(Class or Instance method)
This method returns objects of the type L<NACL::CS::NetworkInterfaceShowRoutingGroup|lib-NACL-CS-NetworkInterfaceShowRoutingGroup-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch>
(requested_fields, filter, allow_empty) and has all of the other properties of
fetch methods (callable in scalar or list context, throws a NoElementsFound
exception if no matching elements are found)

=cut

sub show_routing_group {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);

    # The original code supported passing lif, server and routing-group as
    # direct arguments to the method. Remove these if still passed and log
    # a warning informing the user of the correct way to make the call
    my @deprecated_args = qw(lif server routing-group);
    my $warning_printed;
    foreach my $deprecated (@deprecated_args) {
        if (exists $opts{$deprecated}) {
            $opts{filter}->{$deprecated} ||= $opts{$deprecated};
            delete $opts{$deprecated};
            if (!$warning_printed) {
                my $pkg = $pkg_or_obj->get_package_name();
                $Log->warn('The arguments '
                        . join(', ', @deprecated_args)
                        . ' should be passed in the filter, not as direct arguments, '
                        . "i.e. instead of making the call like this:\n"
                        . "$pkg->show_routing_group(\n"
                        . "\tlif => \$lif,\n"
                        . "\tserver => \$server,\n"
                        . "\trouting-group => \$group\n"
                        . ");\n\n"
                        . "The call should be made like this:\n"
                        . "$pkg->show_routing_group(\n"
                        . "\tfilter => {\n"
                        . "\t\tlif => \$lif,\n"
                        . "\t\tserver => \$server,\n"
                        . "\t\trouting-group => \$group\n" . "\t}\n"
                        . ");\n");
                $warning_printed = 1;
            }
        }
    }

    require NACL::CS::NetworkInterfaceShowRoutingGroup;
    my @state_objs = NACL::CS::NetworkInterfaceShowRoutingGroup->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub show_routing_group


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    my $pkg_or_obj = shift;
    my %opts       = validate(
        @_,
        {   command_interface => {
                type     => OBJECT,
                isa      => 'NACL::C::CommandInterface',
                optional => 1
            }
        }
    );
    my $command_interface = $opts{command_interface};

    my %default_spec = (
        vserver => {type => SCALAR | UNDEF, optional => 1},
        server  => {type => SCALAR | UNDEF, optional => 1},
        lif     => {type => SCALAR},
    );

    if (defined $command_interface) {
        return (
            vserver => {type => SCALAR},
            lif     => {type => SCALAR}
        );
    } else {
        return %default_spec;
    }
} ## end sub _primary_keys_validate_spec

=head2 name_attribute

Returns 'lif'.

=cut

sub name_attribute {
    return 'lif';
}

sub can {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my @opts = $pkg_or_obj->_can_validation(@args);
    my $code;
    if ($code = $pkg_or_obj->SUPER::can(@opts)) {
        $Log->exit() if $may_exit;
        return $code;
    } else {
        $code = $pkg_or_obj->_find_wrapper_generator(@opts);
        if ($code) {
            $Log->exit() if $may_exit;
            return $code;
        }
    }

    $Log->exit() if $may_exit;
    return undef;
} ## end sub can

# If "server" is provided in the call, then we need to pass that through
# to the APISet (the APISet will use compatibility to map it to "vserver").
# The field "server" is not listed in the CDEF since it was applicable
# only in BR, so won't get copied by the base class implementation.
# This implementation copies it if provided in the input arguments.
sub get_api_options {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {options => {type => HASHREF}},

        # Allow through other options taken by SUPER::get_api_options
        allow_extra => 1
    );
    my $input_options = $opts{options};
    my %cmode_args    = $pkg_or_obj->SUPER::get_api_options(%opts);
    if (exists $input_options->{server}) {
        $cmode_args{server} = $input_options->{server};
    }
    $Log->exit() if $may_exit;
    return %cmode_args;
} ## end sub get_api_options

sub _zapi_check_allow_lb_migrate {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    if (defined $opts{'allow-lb-migrate'}) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw('Could not use ZAPI '
                . 'since the option \'allow-lb-migrate\' is not supported '
                . 'by the ZAPI');
    }
    $Log->exit() if $may_exit;
}

sub _get_updated_newname {
    $Log->enter() if $may_enter;
    # See burt 802513 for cluster lif naming behaviour
    # Updating the lif name by prepending node-name
    my ($pkg_or_obj, %opts) = @_;
    my $updated_newname = $opts{newname};
    my $pre_cluster = delete $opts{nacl_is_precluster};
    my $vserver = $opts{vserver};
    my %common_opts;
    if (exists $opts{_updated_newname} || ref $pkg_or_obj) {
        # Updated value for network interface needs to be determined.
        if (!$pre_cluster && $vserver =~ /Cluster/) {
            # Lif would have been renamed
            $pkg_or_obj->_copy_common_component_params(
                source => \%opts,
                target => \%common_opts
            );
            my $home_node = NACL::CS::NetworkInterface->fetch(
                %common_opts,
                command_interface => $opts{command_interface},
                filter            => {vserver => $vserver,
                                      lif     => "*" . $opts{newname},
                },
                requested_fields  => [ 'home-node' ],
            )->home_node();
            if ($updated_newname !~ /^$home_node/) {
                $updated_newname = $home_node . '_' . $opts{newname};
            }
        }
    }
    # Update the scalar-reference, and update object instance attribute
    # either if scalar-reference is passed in or made with object instance call 
    if ( (exists $opts{_updated_newname} || ref $pkg_or_obj) ) {
        ${$opts{_updated_newname}} = $updated_newname;
        $pkg_or_obj->lif($updated_newname);
    }
    $Log->exit() if $may_exit;
}
1;
