# $Id$

#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkPortIfgrp Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::NetworkPortIfgrp

=head1 DESCRIPTION

C<NACL::C::NetworkPortIfgrp> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP NetworkPortIfgrp.  A related class is
L<NACL::CS::NetworkPortIfgrp|lib-NACL-CS-NetworkPortIfgrp-pm>, which represents
the past state of an ONTAP NetworkPortIfgrp.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 ifgrp

=head2 node

=cut

package NACL::C::NetworkPortIfgrp;

use strict;
use warnings;

use NACL::CS::NetworkPortIfgrp;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::BaseException qw(:try);
use NACL::C::Exceptions::NetworkPortIfgrp::IfgrpIsUp;
use NACL::C::Exceptions::NetworkPortIfgrp::ConfiguredUp;
use NACL::C::Exceptions::NetworkPortIfgrp::DoesNotExist;
use NACL::C::Exceptions::NetworkPortIfgrp::PortHasLifBound;
use NACL::C::Exceptions::NetworkPortIfgrp::PortAlreadyAssigned;
use NACL::C::Exceptions::NetworkPortIfgrp::HomePortOrFailoverTarget;
use NACL::C::Exceptions::NetworkPortIfgrp::AlreadyExists ();

=head1 METHODS

=head2 new

  my $NetworkPortIfgrp = NACL::C::NetworkPortIfgrp->new( command_interface=>$ci,
                                                         ifgrp =>$ifgrp,
                                                         node =>$node,
                                                       );

Return a new NetworkPortIfgrp component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'ifgrp',
    scalar => 'node',
];

=head2 find

    my $NetworkPortIfgrp  = NACL::C::NetworkPortIfgrp->find(command_interface => $ci, ...);
    my @NetworkPortIfgrps = NACL::C::NetworkPortIfgrp->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $NetworkPortIfgrp_state = $NACL::C::NetworkPortIfgrp->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::NetworkPortIfgrp would return
a NACL::CS::NetworkPortIfgrp object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 add_port

   NACL::C::NetworkPortIfgrp->add_port(
                                command_interface => $ci,
                                ifgrp => $ifgrp,
                                node => $node,
                                ...);

                                or

    $NetworkPortIfgrp_object->add_port(...);

(Class or instance method)
This method is used to add a port to an interface group.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "net-port-ifgrp-add-port" API for CMode ZAPI.

Invokes "ifgrp add" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "ifgrp" => $ifgrp >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : ifgrp-name

7M CLI  : ifgrp name (Programmatic name: "ifgrp-name")

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : node

=item C<< "port" => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : port

7M CLI  : interface list (Programmatic name: "interface-list")

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"network port ifgrp add-port" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::NetworkPortIfgrp::ConfiguredUp>

This exception is thrown when we attempt to add port to an ifgrp which is up.

=item C<NACL::C::Exceptions::NetworkPortIfgrp::PortHasLifBound>

This exception is thrown when we attempt to add a port on which logical
intefaces got created.

=back

=back

=cut

sub add_port {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {_optional_scalars(qw( port )),},
        cli_cmd         => 'network port ifgrp add-port',
    );
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_add_port_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_add_port_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
                {   method    => '_add_port_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode|Nodescope',
                },
            ],
            frontend => 'NACL::C::NetworkPortIfgrp::add_port',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::NetworkPortIfgrp::ConfiguredUp
            ->detect_convert_and_throw(exception => $exception);
        NACL::C::Exceptions::NetworkPortIfgrp::PortHasLifBound
            ->detect_convert_and_throw(exception => $exception);
        NACL::C::Exceptions::NetworkPortIfgrp::HomePortOrFailoverTarget
            ->detect_convert_and_throw(exception => $exception);
        NACL::C::Exceptions::NetworkPortIfgrp::PortAlreadyAssigned
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;

        # If none of above exception matches throw response exception.
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub add_port

sub _add_port_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args, api => 'network_port_ifgrp_add_port');
    $Log->exit() if $may_exit;
} ## end sub _add_port_cmode_cli

sub _add_port_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
	my $allowed_cmode_options = [qw( port node )];
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
            api    => 'net_port_ifgrp_add_port',
            copy   => $allowed_cmode_options,
            map    => {"ifgrp" => "ifgrp-name"},
	);
    $Log->exit() if $may_exit;
} ## end sub _add_port_cmode_zapi

sub _add_port_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %target_hash;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "port"  => "interface-list",
            "ifgrp" => "ifgrp-name",
        },
        target => \%target_hash,
    );

    $apiset->ifgrp_add(%target_hash);
    $Log->exit() if $may_exit;
} ## end sub _add_port_7mode_cli

=head2 create

    my $NetworkPortIfgrp_object = NACL::C::NetworkPortIfgrp->create(
                                command_interface => $ci,
                                ifgrp => $ifgrp,
                                node => $node,
                                  ...);

(Class method)
This method is used to create a port interface group.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "net-port-ifgrp-create" API for CMode ZAPI.

Invokes "ifgrp create" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "ifgrp" => $ifgrp >>

(Required)

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : ifgrp-name

7M CLI  : ifgrp name (Programmatic name: "ifgrp-name")

=item C<< "node" => $node >>

(Required)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : node

=item C<< "lacp" => $string >>

Applicable for CMode CLI.

=item C<< "distr-func" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : distribution-function

=item C<< "mode" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : mode

=item C<< "port" => $string >>

Applicable for 7Mode CLI

Maps to:

7M CLI : interface list (Programmatic name: "interface-list")

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"network port ifgrp create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        additional_spec =>
            {_optional_scalars(qw( lacp distr-func mode port)),},
        cli_cmd => 'network port ifgrp create',
    );
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_create_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
        frontend => 'NACL::C::NetworkPortIfgrp::create',
        nacl_exceptions => [qw(AlreadyExists)]
    );
    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        ifgrp             => $opts{'ifgrp'},
        node              => $opts{'node'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,  api => 'network_port_ifgrp_create');
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
	my $allowed_cmode_options = [
        qw( mode node
            )
    ];
    $pkg->_base_backend_cmode_zapi(%opts,
            api    => 'net_port_ifgrp_create',
            copy   => $allowed_cmode_options,
            map    => {
                "distr-func" => "distribution-function",
                "ifgrp"      => "ifgrp-name"
            },
	);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

sub _create_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my %target_hash;
    $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "ifgrp" => "ifgrp-name",
            "port"  => "interface-list",
        },
        target => \%target_hash,
    );

    if ($opts{'mode'} eq "multimode") {
        $target_hash{'multi'} = 1;
    } elsif ($opts{'mode'} eq "multimode_lacp") {
        $target_hash{'lacp'} = 1;
    } elsif ($opts{'mode'} eq "singlemode") {
        $target_hash{'single'} = 1;
    } else {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            'Invalid value' . $opts{mode} . "passed for 'mode'\n");
    }

    if (!$target_hash{'single'}) {
        if ($opts{'distr-func'} =~ /rr|mac|ip|port/i) {
            $target_hash{'load-balance'} = $opts{'distr-func'};
        } elsif ($opts{'distr-func'} eq "sequential") {
            $target_hash{'load-balance'} = "rr";
        } else {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw(
                'Invalid value' . $opts{mode} . "passed for 'distr-func'\n");
        }
    } ## end if ( !$target_hash{'single'...

    $apiset->ifgrp_create(%target_hash);
    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface => $command_interface,
        ifgrp             => $opts{'ifgrp'},
        node              => $opts{'node'},
    );
} ## end sub _create_7mode_cli

=head2 remove_port

   NACL::C::NetworkPortIfgrp->remove_port(
                                command_interface => $ci,
                                ifgrp => $ifgrp,
                                node => $node,
                                  ...);

                                or

    $NetworkPortIfgrp_object->remove_port(...);

(Class or instance method)
This method is used to remove a port from an interface group.

Uses a CMode CLI, CMode ZAPI or 7Mode CLI APISet.

Invokes "net-port-ifgrp-remove-port" API for CMode ZAPI.

Invokes "ifgrp delete" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "ifgrp" => $ifgrp >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : ifgrp-name

7M CLI  : ifgrp name (Programmatic name: "ifgrp-name")

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : node

=item C<< "port" => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : port

7M CLI  : interface name (Programmatic name: "interface-name")

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"network port ifgrp remove-port" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::NetworkPortIfgrp::IfgrpIsUp>

This exception is thrown when we attempt to delete a Network Port Ifgrp which is up.

=item C<NACL::C::Exceptions::NetworkPortIfgrp::DoesNotExist>

This exception is thrown when we attempt to delete a non-existing ifgrp or to remove
port which doesn't belongs to the ifgrp.

=back

=back

=cut

sub remove_port {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {_optional_scalars(qw( port )),},
        cli_cmd         => 'network port ifgrp remove-port',
    );

    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_remove_port_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_remove_port_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
                {   method    => '_remove_port_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode|Nodescope',
                },
            ],
            frontend => 'NACL::C::NetworkPortIfgrp::remove_port',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::NetworkPortIfgrp::IfgrpIsUp
            ->detect_convert_and_throw(exception => $exception);
        NACL::C::Exceptions::NetworkPortIfgrp::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);

        $Log->exit() if $may_exit;

        # If none of above exception matches throw response exception.
        $exception->throw();

    };
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub remove_port

sub _remove_port_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args, api => 'network_port_ifgrp_remove_port');
    $Log->exit() if $may_exit;
} ## end sub _remove_port_cmode_cli

sub _remove_port_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
	my $allowed_cmode_options = [
        qw( port node
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
            api    => 'net_port_ifgrp_remove_port',
            copy   => $allowed_cmode_options,
            map    => {"ifgrp" => "ifgrp-name"},
	);
    $Log->exit() if $may_exit;
} ## end sub _remove_port_cmode_zapi

sub _remove_port_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %target_hash;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "port"  => "interface-name",
            "ifgrp" => "ifgrp-name",
        },
        target => \%target_hash,
    );

    $apiset->ifgrp_delete(%target_hash);
    $Log->exit() if $may_exit;
} ## end sub _remove_port_7mode_cli

=head2 delete

   NACL::C::NetworkPortIfgrp->delete(
                                command_interface => $ci,
                                ifgrp => $ifgrp,
                                node => $node,
                                  ...);

                                or

    $NetworkPortIfgrp_object->delete(...);

(Class or instance method)
This method is used to destroy a port interface group.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "net-port-ifgrp-destroy" API for CMode ZAPI.

Invokes "ifgrp destroy" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "ifgrp" => $ifgrp >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : ifgrp-name

7M CLI  : ifgrp name (Programmatic name: "ifgrp-name")

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : node

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"network port ifgrp delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::NetworkPortIfgrp::HomePortOrFailoverTarget>

This exception is thrown when we attempt to delete an ifgrp which is either
home-port or failover target of lif.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'network port ifgrp delete',
    );
    my $ret;
    $ret = $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_delete_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
        frontend => 'NACL::C::NetworkPortIfgrp::delete',
        nacl_exceptions => [qw(DoesNotExist), qw(HomePortOrFailoverTarget)]
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api => 'network_port_ifgrp_delete');
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
	my $allowed_cmode_options = [
        qw( node
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
            api    => 'net_port_ifgrp_destroy',
            copy   => $allowed_cmode_options,
            map    => {"ifgrp" => "ifgrp-name"},
	);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

sub _delete_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %target_hash;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {"ifgrp" => "ifgrp-name"},
        target => \%target_hash,
    );

    $apiset->ifgrp_destroy(%target_hash);
    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'ifgrp' => {type => SCALAR},
        'node'  => {type => SCALAR, optional => 1},
    );
} ## end sub _primary_keys_validate_spec
1;
