# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkPortVlan Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::NetworkPortVlan

=head1 DESCRIPTION

C<NACL::C::NetworkPortVlan> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP NetworkPortVlan.  A related class is
L<NACL::CS::NetworkPortVlan|lib-NACL-CS-NetworkPortVlan-pm>, which represents
the past state of an ONTAP NetworkPortVlan.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vlan_id

=head2 port

=head2 node

=cut

package NACL::C::NetworkPortVlan;

use strict;
use warnings;

use NACL::CS::NetworkPortVlan;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with HASHREF SCALAR UNDEF BOOLEAN OBJECT);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::Exceptions::NetworkPortVlan::AlreadyExists;
use NACL::C::Exceptions::NetworkPortVlan::DoesNotExist;
use NACL::C::Exceptions::NetworkPortVlan::PortHasLifBound;
use NATE::BaseException qw(:try);

=head1 METHODS

=head2 new

  my $NetworkPortVlan = NACL::C::NetworkPortVlan->new(command_interface=>$ci,
                                   vlan_id =>$vlan_id,
                                   port =>$port,
                                   node =>$node,
                                  );

Return a new NetworkPortVlan component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vlan_id',
    scalar => 'port',
    scalar => 'node',
    scalar => [
        {   '-default_ctor' => sub {
                if (defined $_[0]->port() && defined $_[0]->vlan_id()) {
                    return $_[0]->_construct_vlan_name_from_vlan_id_and_port(
                        port    => $_[0]->port(),
                        vlan_id => $_[0]->vlan_id(),
                    );
                }
            },
        },
        'vlan_name'
    ],
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;
    $self->_construct_vlan_name_from_vlan_id_and_port(params => \%args);
    $self->vlan_name($args{vlan_name});
    $Log->exit() if $may_exit;
}

=head2 find

    my $NetworkPortVlan  = NACL::C::NetworkPortVlan->find(command_interface => $ci, ...);
    my @NetworkPortVlans = NACL::C::NetworkPortVlan->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $NetworkPortVlan_state = $NACL::C::NetworkPortVlan->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::NetworkPortVlan would return
a NACL::CS::NetworkPortVlan object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 delete

   NACL::C::NetworkPortVlan->delete(
                                command_interface => $ci,
                                 vlan_id => $vlan_id,
                                 port => $port,
                                 node => $node,
                                  ...);

                                 or

    $NetworkPortVlan_object->delete(...);

(Class or instance method)
This method is used to delete a Virtual LAN (VLAN).

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "net-vlan-delete" API for CMode ZAPI.

Invokes "vlan delete" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vlan-id" => $vlan_id >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI: vlanid

7M CLI : vlanid list (Programmatic name: "vlanid-list")

=item C<< "vlan-name" => $vlan_name >>

(Optional) Not used when vlan-id and port are provided.

=item C<< "port" => $port >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI: parent-interface

7M CLI : ifname (Programmatic name: "ifname")

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI : node

=item C<< "force" => $boolean >>

(Optional)

Applicable for 7Mode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"network port vlan delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::NetworkPortVlan::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete a
non-existing network port vlan.

=item C<NACL::C::Exceptions::NetworkPortVlan::PortHasLifBound>

This type of exception gets thrown if an attempt is made to delete a
network port vlan on which logical interfaces are created.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {force => {type => BOOLEAN, optional => 1},},
        cli_cmd         => 'network port vlan delete',
    );
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::NetworkPortVlan::delete'
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        foreach my $type (qw(DoesNotExist PortHasLifBound)) {
            my $ex_type = "NACL::C::Exceptions::NetworkPortVlan::$type";
            $ex_type->detect_convert_and_throw(exception => $exception);
        }
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api => 'network_port_vlan_delete');
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( vlan-id port node
            )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(force)],
        map    => {
            'vlan-id' => 'vlanid-list',
            'port'    => 'ifname',
        },
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $pkg_or_obj->_handle_boolean_options(
        api_options     => \%opts,
        boolean_options => [qw(force)],
        is_7m_cli       => 1
    );

    $apiset->vlan_delete(%opts);
    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj ,%opts)  = @_ ;

    my %cmd ;
    $cmd{'vlanid'}           = delete $opts{'vlan-id'} ;
    $cmd{'parent-interface'} = delete $opts{port} ;
    $cmd{'node'}             = delete $opts{node} ;
    $opts{'vlan-info'}       = \%cmd ;
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
                      api => 'net-vlan-delete' ,
                      copy => [qw(node vlan-info )],
                      map  => {
                         'vlan-id' => 'vlanid',
                      },
                 );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 create

    my $NetworkPortVlan_object    = NACL::C::NetworkPortVlan->create(
                                command_interface => $ci,
                                 vlan_id => $vlan_id,
                                 port => $port,
                                 node => $node,
                                  ...);

(Class method)
This method is used to create a Virtual LAN (VLAN).

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "net-vlan-create" API for CMode ZAPI.

Invokes "vlan create" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vlan-id" => $vlan_id >>

(Required)

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI: vlanid

7M CLI:  vlanid list (Programmatic name: "vlanid-list")

=item C<< "vlan-name" => $vlan_name >>

(Optional) Not used when vlan-id and port are provided.

=item C<< "port" => $port >>

(Required)

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI: parent-interface

7M CLI:  ifname (Programmatic name: "ifname")

=item C<< "node" => $node >>

(Required)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : node

=item C<< "gvrp" => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : gvrp-enabled

7M CLI  : -g <value> (Programmatic name: "gvrp")

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"network port vlan create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::NetworkPortVlan::AlreadyExists>

This type of exception gets thrown if an attempt is made to create a
existing network port vlan.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( gvrp
                    )
            ),
        },
        cli_cmd => 'network port vlan create',
    );

    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_create_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_create_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
                {   method    => '_create_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::NetworkPortVlan::create',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::NetworkPortVlan::AlreadyExists
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        'vlan_id'         => $opts{'vlan-id'},
        'port'            => $opts{'port'},
        'node'            => $opts{'node'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,  api => 'network_port_vlan_create');
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => [qw( gvrp )],
        map    => {
            'vlan-id' => 'vlanid-list',
            'port'    => 'ifname',
        },
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $apiset->vlan_create(%opts);
    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface => $command_interface,
        vlan_id           => $opts{'vlanid-list'},
        port              => $opts{'ifname'},
    );
} ## end sub _create_7mode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg ,%opts)  = @_ ;
    my %cmd ;
    $cmd{'vlanid'}           = delete $opts{'vlan-id'} ;
    $cmd{'parent-interface'} = delete $opts{port} ;
    $cmd{'node'}             = delete $opts{node} ;
    $opts{'vlan-info'}       = \%cmd ;
    $pkg->_base_backend_cmode_zapi(%opts,
                  api => 'net-vlan-create' ,
                  copy => [qw( node vlan-info  )],
                  map    => {
                      "gvrp"    => "gvrp-enabled"
                  },
         );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

=head2 modify

     NACL::C::NetworkPortVlan->modify(
                                command_interface => $ci,
                                 port => $port,
                                 gvrp => $gvrp,
                                  ...);

(Class or Instance method)
This method is used to modify a Virtual LAN (VLAN).

Uses a 7Mode CLI APISet.

Invokes 'vlan modify' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "port" => $port >>

(Required for class method, Not Applicable for instance method)

Applicable for 7Mode CLI

Maps to:

7M CLI  : ifname (Programmatic name: "ifname")

=item C<< "vlan-id" => $vlan-id >>

(Required for class method, Not Applicable for instance method)

Applicable for 7Mode CLI

Maps to:

7M CLI  : vlanid list (Programmatic name: "vlanid-list")

=item C<< "gvrp" => $gvrp >>

(Required)

Applicable for 7Mode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {_optional_scalars(qw( gvrp ))}
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw( gvrp )],
        map    => {
            'vlan-id' => 'vlanid-list',
            'port'    => 'ifname',
        },
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $Log->debug("Opts to API call 'vlan_modify' are: \n" . Dumper(\%opts));

    $apiset->vlan_modify(%opts);
    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_cli

=head2 add

    my $NetworkPortVlan_object    = NACL::C::NetworkPortVlan->add(
                                command_interface => $ci,
                                 vlan-id => $vlan_id
                                 port => $port,
                                 gvrp => $gvrp,
                                  ...);

(Class or Instance method)
This method is used to add a Virtual LAN (VLAN).

Uses a 7Mode CLI APISet.

Invokes 'vlan add' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vlan-id" => $vlan_id >>

(Required for class method, Not Applicable for instance method)

Applicable for 7Mode CLI

Maps to:

7M CLI  : vlanid list (Programmatic name: "vlanid-list")

=item C<< "port" => $port >>

(Required for class method, Not Applicable for instance method)

Applicable for 7Mode CLI

Maps to:

7M CLI  : ifname (Programmatic name: "ifname")

=item C<< "gvrp" => $gvrp >>

(Required)

Applicable for 7Mode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( gvrp
                    )
            )
        }
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_add_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub add

sub _add_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw( gvrp )],
        map    => {
            'vlan-id' => 'vlanid-list',
            'port'    => 'ifname',
        },
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $Log->debug("Opts to API call 'vlan_add' are: \n" . Dumper(\%opts));

    $apiset->vlan_add(%opts);
    $Log->exit() if $may_exit;
} ## end sub _add_7mode_cli

sub _common_validate_with {
    my $pkg_or_obj = shift;
    my %opts       = @_;
    if (!ref $pkg_or_obj) {
        my %command_opts = @{$opts{params}};
        if (exists($command_opts{'vlan-name'})) {
            if ($command_opts{'port'} || $command_opts{'vlan-id'}) {
                NATE::BaseException->throw(
                    "Either 'vlan-name' or ('vlan-id' and 'port') to be provided."
                );
            }
            ($command_opts{'port'}, $command_opts{'vlan-id'})
                = split(/\-/, delete $command_opts{'vlan-name'});
            $opts{params} = [%command_opts];
        } ## end if ( exists( $command_opts...
    } ## end if ( !ref $pkg_or_obj )
    return $pkg_or_obj->SUPER::_common_validate_with(%opts);
} ## end sub _common_validate_with


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vlan-id' => {type => SCALAR | OBJECT},
        'port'    => {type => SCALAR | OBJECT},
        'node'    => {type => SCALAR | OBJECT, optional => 1},
    );
} ## end sub _primary_keys_validate_spec

sub _construct_vlan_name_from_vlan_id_and_port {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    %opts = validate_with(
        params => \%opts,
        spec   => {params => {type => HASHREF},},
    );
    my $params = $opts{params};
    my %opts_for_processing;
    $pkg_or_obj->_hash_move(
        source => $params,
        target => \%opts_for_processing,
        move   => [qw(vlan_name)]
    );
    $pkg_or_obj->_process_options(options => \%opts_for_processing);
    if ($opts_for_processing{vlan_name}
        && ($params->{vlan_id} || $params->{port}))
    {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Cannot provide any of 'vlan_id' or 'port' along with 'vlan_name'"
        );
    }
    if (   !$opts_for_processing{vlan_name}
        && !($params->{vlan_id} || $params->{port}))
    {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "When 'vlan_name' is not provided, both 'vlan_id' and 'port' should be provided"
        );
    }
    if (!($opts_for_processing{vlan_name})) {
        $params->{vlan_name} = $params->{port} . '-' . $params->{vlan_id};
        $Log->exit() if $may_exit;
        return $params->{vlan_name};
    }
}

1;
