# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkRoutingGroups Component Module
## @author Sumit.Mittal@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::NetworkRoutingGroups

=head1 DESCRIPTION

C<NACL::C::NetworkRoutingGroups> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP Network Interface Routing Groups.  A related class is
L<NACL::CS::NetworkRoutingGroups|lib-NACL-CS-NetworkRoutingGroups-pm>, which represents
the past state of an ONTAP Routing Group.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

The name of the vserver, used to identify it uniquely.  (vserver
names are assumed to be unique within a node or within a cluster)

=head2 server

The name of the vserver, used to identify it uniquely.  (vserver
names are assumed to be unique within a node or within a cluster)

=head2 routing_group

The name of the routing group, used to identify it uniquely.
(routing groups names are assumed to be unique within a node or within a
vserver)

=cut

package NACL::C::NetworkRoutingGroups;

use strict;
use warnings;
use NACL::CS::NetworkRoutingGroups;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR UNDEF);
use NATE::Exceptions::Argument;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $route = NACL::C::NetworkRoutingGroups->new(command_interface=>$ci,
                                  'routing-group' =>$routing_group);

Return a new routing group component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'routing_group',
    scalar => 'vserver',
    scalar => 'server',
];

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::C::RoutingGroups'
    );
    $Log->exit() if $may_exit;
    return $isa;
}

=head2 create

  my $routing_group  = NACL::C::NetworkRoutingGroups->create(command_interface=>$ci,
                                             'routing-group'=>$aggr_name,...);

(Class method) Request creation of a new routing group element, and  return a
new routing group component object that refers to it.  Does not wait for
routing group creation to complete.  Uses a CMode-CLI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'routing-group'=>$string >>

(Required) Name of routing group to which the new route will belong to.

Applicable for CMode CLI.

=item C<< vserver=>$string >>

(Required for CMode)

Applicable for CMode CLI.

=item C<< server=>$string >>

(Required for CMode)

Applicable for CMode CLI.

=item C<< metric => $integer >>

Applicable for CMode CLI.

=item C<< subnet => $string >>

Applicable for CMode CLI.

=item C<< role => $string >>

=item C<< '*' >>

This method takes all other options supported by
"network routing-groups create" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    my ($pkg, @args) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {_optional_scalars(qw (subnet role metric))},
        api             => 'network_routing_groups_create',
    );
    return $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_create_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::NetworkRoutingGroups::create',
    );
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    if (exists $opts{server}) {
        $opts{'additional_args'}->{server} = $opts{server};
    }

    $pkg->_base_backend_cmode_cli(%opts, api => 'network_routing_groups_create');

    $Log->exit() if $may_exit;
    return $pkg->new(
        'routing_group'   => $opts{'routing-group'},
        'vserver'         => $opts{vserver},
        'server'          => $opts{server},
        command_interface => $opts{command_interface}
    );
} ## end sub _create_cmode_cli

=head2 delete

  NACL::C::NetworkRoutingGroups->delete(command_interface=>$ci,
                                 'routing-group'=>$routing_group
                                 ,...);

                            or

  $routing_groups_object->delete(...);

(Class or instance method) This method deletes a routing group element.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required for class method, Not Applicable for instance method)
(Name of the vserver or Node)

Applicable for CMode CLI.

=item C<< 'routing-group'=>$routing_group >>

(Required) Name of routing group to which the route belongs to.
(Required for class method, Not Applicable for instance method)

=item C<< '*' >>

This method takes all other options supported by
"network routing-groups delete" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'network_routing_groups_delete',
    );

    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_delete_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::NetworkRoutingGroups::delete',
    );
} ## end sub delete

sub _delete_cmode_cli {
    my ($pkg_or_obj, %opts) = @_;

    if (defined $opts{'server'}) {
        $opts{'additional_args'}->{'server'} = $opts{'server'};
    }
    $pkg_or_obj->_base_backend_cmode_cli(%opts, api => 'network_routing_groups_delete');
} ## end sub _delete_cmode_cli

=head2 init

Derived class of init - checks for any optional keys and calls 
$self->SUPER::init(@_) first to get the rest of the argument checking

=cut

sub init {
    my ($self, %opts) = @_;
    $self->SUPER::init(%opts);
    if (!defined($opts{server}) && !defined($opts{vserver})) {
        NATE::Exceptions::Argument->throw("Specify either server or vserver");
    }
    unless ($opts{server} || $opts{vserver}) {
        NATE::Exceptions::Argument->throw("Missing one of the required keys");
    }
} ## end sub init


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'routing-group' => {type => SCALAR},
        vserver         => {type => SCALAR | UNDEF, optional => 1},
        server          => {type => SCALAR | UNDEF, optional => 1},
    );
} ## end sub _primary_keys_validate_spec

1;
