# $Id$

#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkRoutingGroupsRoute Component Module
## @author Sumit.Mittal@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::NetworkRoutingGroupsRoute

=head1 DESCRIPTION

C<NACL::C::NetworkRoutingGroupsRoute> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP Network Interface Routing Groups NetworkRoutingGroupsRoute.  A related class is
L<NACL::CS::NetworkRoutingGroupsRoute|lib-NACL-CS-NetworkRoutingGroupsRoute-pm>, which represents
the past state of an ONTAP NetworkRoutingGroupsRoute.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

The name of the vserver, used to identify it uniquely.  (vserver
names are assumed to be unique within a node or within a cluster)

=head2  server

The name of the vserver, used to identify it uniquely.  (vserver
names are assumed to be unique within a node or within a cluster)

=head2 routing_group

The name of the routing group, used to identify it uniquely.
(routing groups names are assumed to be unique within a node or within a
vserver)

=head2 destination

The destination represented as IpAddress/Mask.

=cut

package NACL::C::NetworkRoutingGroupsRoute;

use strict;
use warnings;
use NACL::CS::NetworkRoutingGroupsRoute;
use NACL::STask::NetworkRoute;
use NACL::C::NetworkRoute;
use NACL::CS::NetworkRoute;
use NACL::ComponentUtils
    qw (_optional_scalars Dumper _ontap_ci_validate_spec);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(:all);
use NATE::Exceptions::Argument;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::Exceptions::NetworkRoutingGroupsRoute::AddressNotFound;
use NACL::C::Exceptions::NetworkRoutingGroupsRoute::DoesNotExist;
use NATE::BaseException qw(:try);

=head1 METHODS

=head2 new

  my $route = NACL::C::NetworkRoutingGroupsRoute->new(command_interface=>$ci,
                                  'routing-group' =>$routing_group);

Return a new route component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'routing_group',
    scalar => 'vserver',
    scalar => 'server',
    scalar => 'destination',
];

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::C::Route'
    );
    $Log->exit() if $may_exit;
    return $isa;
}

=head2 create

  my $route  = NACL::C::NetworkRoutingGroupsRoute->create(command_interface=>$ci,
                                      'routing-group'=>$routing_group_name,
                                      ...);

(Class method) Request creation of a new route element, and  return a new
route component object that refers to it.  Does not wait for route
creation to complete.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "net-routing-group-route-create" API for CMode ZAPI.

Invokes "cmd" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver'=>$vserver >>

(Required) Name of vserver or Node.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: vserver

=item C<< 'routing-group'=>$routing_group >>

(Required) Name of routing group to which the new route will belong to.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: routing-group

=item C<< 'destination'=>$destination >>

(Optional) Destination value in the format of Destination/Mask
(default: 0.0.0.0/0).

Applicable for CMode CLI/ZAPI

Maps to:
 
CM ZAPI: destination-address

=item C<< 'destination-type'=>$destination_type >>

(Optional) Type of Destination(host or net) only for 7Mode.
(default: host).

Applicable for 7Mode CLI.

=item C<< 'prefix'=>$prefix >>

(Optional) Prefix length to be used only for 7Mode.
(applicable only if destination-type is net).

Applicable for CMode CLI, 7Mode CLI

=item C<< 'netmask'=>$netmask >>

(Optional) netmask to be used only for 7Mode.
(appplicable only if destination-type is net).

Applicable for CMode CLI, 7Mode CLI.

=item C<< gateway => $gateway >>

(Required) Gateway ip address.

Applicable for CMode CLI/ZAPI

Maps to:
CM ZAPI: gateway-address

=item C<< metric => $metric_value >>

(Optional) Metric value (default: 20).

Applicable for CMode CLI/ZAPI, 7Mode CLI.

CM ZAPI: metric

=item C<< '*' >>

This method takes all other options supported by
"network routing-groups route create" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::NetworkRoutingGroupsRoute::AddressNotFound>

This type of exception gets thrown if an attempt is made to create a
network routing-groups route when ipv6 is disabled.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw (gateway metric netmask prefix destination-type))
        },
        cli_cmd => 'network routing-groups route create',
    );
    my $ret;
    $ret = $pkg->_redirect_call(_method => 'create', %opts);
    if ($ret) {
        $Log->exit() if $may_exit;
        return $ret;
    }
    try {
        $pkg->call_on_apiset_with_exceptions(
            %opts,
            choices => [
                {   method    => '_create_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_create_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
                {   method    => '_create_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
            ],
            frontend => 'NACL::C::NetworkRoutingGroupsRoute::create',
            nacl_exceptions => [qw(AlreadyExists)],
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::NetworkRoutingGroupsRoute::AddressNotFound
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    }
    my $obj = $pkg->new(
        routing_group     => $opts{'routing-group'},
        vserver           => $opts{vserver},
        server            => $opts{server},
        destination       => $opts{destination},
        command_interface => $opts{command_interface}
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    if (exists $opts{server}) {
        $opts{server} = $opts{server};
    }
    $pkg->_base_backend_cmode_cli(%opts,    api => 'network_routing_groups_route_create');
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_zapi(@args,
            api    => 'net_routing_group_route_create',
            copy   => [qw(metric vserver routing-group)],
            map    => {
                "destination" => "destination-address",
                "gateway"     => "gateway-address",
                "server"      => "vserver",
            },
	);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

sub _create_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $allowed_7mode_options
        = [qw( destination gateway metric destination-type )];
    my %args;

    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%args,
    );

    if ($opts{'prefix'}) {
        $args{'destination'} = $opts{'destination'} . '/' . $opts{'prefix'};
    } elsif ($opts{'netmask'}) {
        $args{'destination'} = $opts{'destination'} . '&' . $opts{'netmask'};
    }

    my $cmd = 'route_add';
    if ($pkg->is_ipv6(%opts)) {
        $opts{prefixlen} = delete $opts{'prefix'} if ($opts{'prefix'});
        delete $opts{'netmask'};
        $cmd = 'route_add_inet6';
    }

    $apiset->$cmd(%args);

    my $obj = $pkg->new(
        destination       => $opts{destination},
        command_interface => $command_interface
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub _create_7mode_cli

=head2 is_ipv6

    my $boolean = $route->is_ipv6();

    my $boolean = NACL::C::NetworkRoutingGroupsRoute->is_ipv6(destination => $destination);

(Class or instance method)

Returns a boolean representing whether the destination is IPv4 or IPv6.

=cut

sub is_ipv6 {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $destination;
    if (ref $pkg_or_obj) {
        $destination = $pkg_or_obj->destination();
    } else {
        $destination = $opts{destination};
    }
    $Log->exit() if $may_exit;
    if ($destination =~ /.*:/) {
        return 1;
    } else {
        return 0;
    }
}

=head2 delete

  NACL::C::NetworkRoutingGroupsRoute->delete(command_interface=>$ci,
                         'routing-group'=>$routing_group,...);

                            or

  $route_object->delete(...);

(Class or instance method) This method deletes a route element.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "net-routing-group-route-destroy" API for CMode ZAPI.

Invokes "cmd" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required for class method, Not Applicable for instance method)
Name of route.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: vserver

=item C<< 'routing-group'=>$routing_group >>

(Required) Name of routing group to which the route belongs to.
(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: routing-group

=item C<< 'destination'=>$destination >>

(Required) Destination value in the format of Destination/Mask
(default: 0.0.0.0/0).
(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI: destination-address

=item C<< 'destination-type'=>$destinationtype >>

(Optional for 7mode ipv4) Type may be host/net.
(Required for 7mode ipv6 host delete) default is net

Applicable for 7Mode CLI.

=item C<< 'prefix'=>$prefix >>

(Optional) Prefix length to be used only for 7Mode.
(applicable only if destination-type is net).

=item C<< '*' >>

This method takes all other options supported by
"network routing-groups route delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::NetworkRoutingGroupsRoute::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete a
non-existing network routing-groups route.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            {_optional_scalars(qw (gateway destination-type prefix))},
        cli_cmd => 'network routing-groups route delete',
    );
    my $status = 0;
    $pkg_or_obj->_redirect_call(
        _method => 'delete',
        status  => \$status,
        %opts
    );
    if ($status) {
        $Log->exit() if $may_exit;
        return;
    }
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::NetworkRoutingGroupsRoute::delete',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::NetworkRoutingGroupsRoute::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    }
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args, api => 'network_routing_groups_route_delete');
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
            api    => 'net_routing_group_route_destroy',
            copy   => [qw( vserver )],
            map    => {
                "destination"   => "destination-address",
                "routing-group" => "routing-group",
                "server"        => "vserver",
            },
	);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

sub _delete_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_7mode_options = [qw(destination destination-type)];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%args,
    );

    my $cmd = 'route_delete';
    if ($pkg_or_obj->is_ipv6(%opts)) {
        $opts{prefixlen} = delete $opts{'prefix'} if ($opts{'prefix'});
        delete $opts{'netmask'};
        $cmd = 'route_delete_inet6';
    }
    $apiset->$cmd(%args);
    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_cli

# Was originally for redirection, now just warns if user is using the old
# library for ipspaces
sub _redirect_call {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    if ($pkg_or_obj->_is_ipspace(
            command_interface => $opts{command_interface}
        )
        )
    {
        _warn(
            pkg_or_obj => $pkg_or_obj,
            method     => $opts{_method}
        );
    }

    $Log->exit() if $may_exit;
    return;
}

sub find {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_find_validate_spec();
    if ($pkg->_is_ipspace(command_interface => $opts{command_interface})) {
        _warn(
            pkg_or_obj => $pkg,
            method     => 'find',
        );
    }

    my @records = $pkg->SUPER::find(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @records : $records[0];
} ## end sub find

sub _warn {
    my %opts = validate_with(
        params => \@_,
        spec   => {
            pkg_or_obj => {type => OBJECT | SCALAR},
            method     => {type => SCALAR},
        }
    );
    my $pkg_or_obj = $opts{pkg_or_obj};
    my $method     = $opts{method};
    my $orig_pkg   = $pkg_or_obj->get_package_name();
    my $c_or_t     = $pkg_or_obj->is_c_or_t();
    my $route_pkg  = "NACL::${c_or_t}::NetworkRoute";
    $Log->warn("The method $orig_pkg->$method was invoked on an "
            . "IPSpace build. However, the 'network routing-groups route' "
            . 'set of commands is now deprecated and has been replaced by '
            . "the 'network route' set of commands. For this reason, all "
            . "calls to '$orig_pkg' must be replaced by calls to '$route_pkg'. "
            . "The script updates will need to be made like this:\n\n"
            . "\tif (\$node->has_uichange(uichange => 'ipspaces-ms4') {\n"
            . "\t\t# IPSpace build, should use new library\n"
            . "\t\t$route_pkg->$method(%options);\n"
            . "\t} else {\n"
            . "\t\t# Non-IPSpace build, should use older library\n"
            . "\t\t$orig_pkg->$method(%options);\n"
            . "\t}\n\n"
            . 'Since this is a functionality change, not all of the options '
            . 'accepted by the old commands are accepted by the new commands. '
            . 'See '
            . 'http://wikid.netapp.com/w/SCON/QA/FS0/IPspaces/Regression_scripts_guidelines'
            . ' for complete details. For any further queries, mail '
            . "dl-ipspace-automation.\n");
}

sub _is_ipspace {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {command_interface => _ontap_ci_validate_spec()},
    );
    my $ci = $opts{command_interface};

    if ($ci->is_cmode()
        && $ci->has_uichange(
            uichange        => 'ipspaces-ms4',
            throw_exception => 0
        )
        )
    {
        $Log->exit() if $may_exit;
        return 1;
    }
    $Log->exit() if $may_exit;
    return 0;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'routing-group' => {type => SCALAR | UNDEF, optional => 1},
        vserver         => {type => SCALAR | UNDEF, optional => 1},
        server          => {type => SCALAR | UNDEF, optional => 1},
        destination     => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
