# $Id$
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Options Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Options

=head1 DESCRIPTION

C<NACL::C::Options> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP System License.  A related class is
L<NACL::CS::Options|lib-NACL-CS-Options-pm>, which represents
the past state of an ONTAP Options.

=cut

package NACL::C::Options;
use strict;
use warnings;
use NACL::CS::Options;
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::ComponentUtils qw(Dumper);

use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::ResponseException;
use NACL::C::Exceptions::Options::SnapmirrorNotLicensed;

=head1 METHODS

=head2 option

NACL::C::Options->option(command_interface=>$ci,
                        'option-name'=> $option_name ,
                        'option-value' => $option_value );

(Class method) Request changes to a option element , doesn't return any
output value.

Uses 7Mode/Nodescope CLI/ZAPI APISet.

Invokes "options" command for 7Mode/Nodescope CLI.

Invokes "options-set" command for 7Mode/Nodescope ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "option-name" => $string >>

(mandatory) Name of the Option.

Applicable for 7Mode CLI/ZAPI.

=item C<< "option-value" => $string >>

(mandatory) Value of the Option to be changed.

Applicable for 7Mode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::Options::SnapmirrorNotLicensed>

This type of exception gets thrown if snapmirror is not licensed.

=back

=back

=cut


sub option {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'option-name'  => { type => SCALAR },
            'option-value' => { type => SCALAR },
        },
    );
    my @cmode_cli_choices = (
        {   method    => '_option_7mode_cli',
            interface => 'CLI',
            set       => '7Mode|Nodescope',
        },
    );
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                @cmode_cli_choices,
                {   method    => '_option_7mode_zapi',
                    interface => 'ZAPI',
                    set       => '7Mode|Nodescope',
                },
            ],
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        if (NACL::C::Exceptions::Options::SnapmirrorNotLicensed
            ->detect(exception => $exception,'option-name'=>"snapmirror")){
            $Log->exit() if $may_exit;
            NACL::C::Exceptions::Options::SnapmirrorNotLicensed
                ->convert_and_throw(exception => $exception);
        } elsif ($exception->text =~ /Unable to find API: options-set on node vserver.*/i) {
            $Log->comment('Could not use ZAPI implementation as the '
                          . 'nodescope vserver is not provided');
            $pkg->call_on_apiset(%opts, choices => [@cmode_cli_choices]);
        } else {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
    };
    $Log->exit() if $may_exit;
} ## end sub option

sub _option_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg               = shift;
    my %opts              = $pkg->_validate_backend_opts(@_);
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    # The prompt mentioned in the match_table shows up when attempting
    # to modify the value of "nfs.v4.acl.max.aces" to 1024 or more
    # (burt 587315)
    $apiset->options(
        $opts{'option-name'} => $opts{'option-value'},
        'connectrec-match_table' =>
            [ 'Do you still want to continue' => 'y',
              'Continue' => 'y',
              "You will not be able to disable this feature.*\n.*continue\?" => 'yes' ]
    );
    $Log->exit() if $may_exit;
} ## end sub _option_7mode_cli

sub _option_7mode_zapi {
    my $pkg               = shift;
    my %opts              = $pkg->_validate_backend_opts(@_);
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $options_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            'option-name'  => 'name',
            'option-value' => 'value'
        },
    );
    $apiset->options_set(%$options_args);
    $Log->exit() if $may_exit;
} ## end sub _option_7mode_zapi

__PACKAGE__->_load_event_class();

1;
