#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary QosWorkloadStream Component Module
## @author dl-perf-qa-automation@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::QosWorkloadStream

=head1 DESCRIPTION

C<NACL::C::QosWorkloadStream> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP QosWorkloadStream.  A related class is
L<NACL::CS::QosWorkloadStream|lib-NACL-CS-QosWorkloadStream-pm>, which represents
the past state of an ONTAP QosWorkloadStream.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 workload

=head2 stream_id

=cut

package NACL::C::QosWorkloadStream;

use strict;
use warnings;

use NACL::CS::QosWorkload;
use NACL::CS::QosWorkloadStream;
use NACL::ComponentUtils qw(_optional_scalars);
use NATE::BaseException qw(:try);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $QosWorkloadStream = NACL::C::QosWorkloadStream->new(
      command_interface=>$ci,
      stream_id =>$stream_id,
      workload =>$workload,
  );

Return a new QosWorkloadStream component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'workload',
    scalar => 'stream_id',
];

=head2 create

    my $QosWorkloadStream_object = NACL::C::QosWorkloadStream->create(
        command_interface => $ci,
        workload => $workload,
        [ stream_id => $stream_id, ]
        ...
    );


(Class method)
This method is used to add stream to a workload.

Uses CMode CLI/ZAPI APISet.
  Invokes 'qos-stream-create' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< workload => $workload >>

(Required)

=item C<< stream-id => $stream_id >>

(CLI-only, optional)

=item C<< vserver => $string >>

  Applicable only for CMode CLI

=item C<< volume => $string >>

  Applicable only for CMode CLI

=item C<< lun => $string >>

  Applicable only for CMode CLI

=item C<< qtree => $string >>

  Applicable only for CMode CLI

=item C<< 'protocol' => $string >>

*Protocol
Applicable for CMode CLI

=item C<< 'ip' => $string >>

*Source IP/mask
Applicable for CMode CLI

=item C<< 'min-req-size' => $string >>

*Minimum Request Size, possible value(s) are 1B,16KB,64KB,1MB,8MB
Applicable for CMode CLI

=item C<< 'wwn' => $string >>

*Source Wwn
Applicable for CMode CLI

=item C<< 'igroup' => $string >>

*Igroup
Applicable for CMode CLI

=item C<< 'max-req-size' => $string >>

*Maximum Request Size, possible value(s) are 1B,16KB,64KB,1MB,8MB
Applicable for CMode CLI

=item C<< 'request-type' => $string >>

*Request Type, possible value(s) are read,write,other
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"qos workload stream create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        additional_spec     => {
            workload => {type => SCALAR},
            _optional_scalars qw(stream-id vserver volume qtree lun
                protocol ip min-req-size wwn igroup max-req-size request-type
                ),
        },
        api => 'qos_workload_stream_create',
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::QosWorkloadStream::create',
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    # If the user specified a 'stream-id', we'll create the stream
    # with it and pass it to the QosWorkloadStream constructor.
    # Otherwise, we'll let ONTAP assign the 'stream-id', and we'll
    # have to try to figure out which one ONTAP assigned.
    #
    # To do that, we'll use the same algorithm that ONTAP uses
    # to assign IDs. We may pick the wrong ID if other streams
    # are being created concurrently with this one, but that's
    # the best we can do.

    my $stream_id = $opts{'stream-id'};
    if (!defined $stream_id) {
        $stream_id = $pkg->_generate_stream_id(
            apiset   => $opts{apiset},
            workload => $opts{workload},
        );
    } ## end if ( !defined $stream_id)

    $pkg->_base_backend_cmode_cli(
        %opts,
        api => 'qos_workload_stream_create',
    );

    $Log->exit() if $may_exit;
    return $pkg->new(
        workload          => $opts{workload},
        stream_id         => $stream_id,
        command_interface => $opts{command_interface},
    );
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1,
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $workload          = delete $opts{workload};

    # The qos-stream-create ZAPI expects the workload UUID (instead of
    # its name) as input; therefore we must fetch the UUID.

    my $wkld_uuid = $pkg->_fetch_and_cache_uuid(
        command_interface => $command_interface,
        workload          => $workload,
    );

    # The ZAPI does not accept a stream-id. We'll pass the one returned
    # by qos-stream-create to the constructor.

    delete $opts{'stream-id'};

    my $response = $apiset->qos_stream_create(
        'workload-uuid'   => $wkld_uuid,
        'characteristics' => \%opts,
    );
    my $output    = $response->get_parsed_output();
    my $stream_id = $output->[0]->{'stream-id'};

    my $stream_obj = $pkg->new(
        workload          => $workload,
        stream_id         => $stream_id,
        command_interface => $command_interface,
    );

    # Cache workload UUID for use by subsequent ZAPI instance
    # method calls.

    $stream_obj->{_uuid} = $wkld_uuid;
    $Log->exit() if $may_exit;
    return $stream_obj;
} ## end sub _create_cmode_zapi

=head2 delete

  NACL::C::QosWorkloadStream->delete(
      command_interface => $ci,
      workload          => $workload,
      'stream-id'       => $stream_id,
  );

  or

  $QosWorkloadStream_object->delete();

(Class or instance method)
This method deletes a workload element.

Uses CMode CLI/ZAPI APISet.
  Invokes 'qos-stream-destroy' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< workload => $workload >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI

=item C<< stream-id => $stream_id >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"qos workload stream delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'qos_workload_stream_delete',
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::QosWorkloadStream::delete',
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj,%opts) = @_;

    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api => 'qos_workload_stream_delete',
    );

    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
     $Log->enter() if $may_enter;
     my ($pkg_or_obj,%opts) = @_ ;

     my $wkld_uuid = $pkg_or_obj->_fetch_and_cache_uuid(
        command_interface => $opts{'command_interface'},
        workload          => $opts{'workload'},
    );

    $opts{'workload-uuid'} = $wkld_uuid;
    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api  => 'qos_stream_destroy',
     );
    $Log->exit() ;
} ## end sub _delete_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'workload'  => {type => SCALAR},
        'stream-id' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

# ZAPI calls generally use the UUID, rather than the workload
# name, to identify a workload. The following routine will
# fetch the UUID when first called, and cache it in the
# object for subsequent use.

sub _fetch_and_cache_uuid {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate(
        @_,
        {   command_interface => {isa  => 'NACL::C::CommandInterface'},
            workload          => {type => SCALAR},
        },
    );

    # If we have a cached UUID, use it!

    if (ref $pkg_or_obj && $pkg_or_obj->{_uuid}) {
        $Log->exit() if $may_exit;
        return $pkg_or_obj->{_uuid};
    }

    # Otherwise, fetch the UUID.

    my $wkld_state = NACL::CS::QosWorkload->fetch(
        command_interface => $opts{command_interface},
        apiset_should     => {interface => 'ZAPI'},
        requested_fields  => [qw(uuid)],
        filter            => {workload => $opts{workload},},
    );
    my $wkld_uuid = $wkld_state->uuid();

    # If this is an instance method call, cache
    # the UUID.

    $pkg_or_obj->{_uuid} = $wkld_uuid
        if ref $pkg_or_obj;

    $Log->exit() if $may_exit;
    return $wkld_uuid;
} ## end sub _fetch_and_cache_uuid

# This attempts to duplicate the same logic ONTAP uses to auto-assign
# stream-ids when the CLI user does not specify one. See burt446570
# for an explanation of ONTAP's approach.

sub _generate_stream_id {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate(
        @_,
        {   apiset   => {isa  => 'NACL::APISet'},
            workload => {type => SCALAR},
        },
    );

    my $apiset   = $opts{apiset};
    my $workload = $opts{workload};
    my $output   = [];

    # Lookup the in-use IDs for the workload.
    try {
        my $response
            = $apiset->qos_workload_stream_show(workload => $workload);
        $output = $response->get_parsed_output();
    }
    catch NACL::APISet::Exceptions::NoMatchingEntriesException with {};

    # If no IDs are being used, return zero.
    if (scalar(@$output) == 0) {
        $Log->exit() if $may_exit;
        return 0;
    }

    my @streams = sort { $a->{'stream-id'} <=> $b->{'stream-id'} } @$output;
    my $max = 4294967295;

    # If the maximum ID is being used, return the
    # lowest available ID.
    if ($streams[-1]->{'stream-id'} == $max) {
        for (my $i = 0; $streams[$i]->{'stream-id'} <= $max; $i++) {
            no warnings qw(uninitialized);
            if ($i < $streams[$i]->{'stream-id'}) {
                $Log->exit() if $may_exit;
                return $i;
            }
        } ## end for ( my $i = 0; $streams...

        # If we got here, all IDs must already be in use.
        # We'll return and let the subsequent "qos workload
        # stream create" command throw an exception.
        return;

        # Otherwise, return highest ID plus one.
    } else {
        $Log->exit() if $may_exit;
        return $streams[-1]->{'stream-id'} + 1;
    }
} ## end sub _generate_stream_id

1;
