
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ResourceAgitation Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::ResourceAgitation

=head1 DESCRIPTION

C<NACL::C::ResourceAgitation> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It provides an API interface
for working with node/cluster-wide system resource agitation. It operates
by issuing the "setral" command, which is only available in 7Mode and in
Nodescope (d-blade command) of CMode. There is no CMode NGSH equivalent.

Unlike other components, we do not suggest calling 'find' for this component,
since a ResourceAgitation object models an entire node/cluster.  Instead of
'find', we can directly invoke L<new|lib-NACL-C-ResourceAgitation-pm/new>.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>
This will be the node/cluster on which we want to modify resource agitation levels.

=cut

package NACL::C::ResourceAgitation;

use strict;
use warnings;

use Params::Validate qw(validate_with SCALAR ARRAYREF);

use NACL::CS::ResourceAgitation;
use NACL::ComponentUtils qw (_optional_scalars );
use base 'NACL::C::Component::ONTAP';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Exceptions::Argument qw(:try);
use Data::Dumper;

=head1 METHODS

=head2 new

  my $resource_agitation = NACL::C::ResourceAgitation->new(command_interface=>$ci);

Return a new ResourceAgitation object to manage setral levels.

=cut

=head2 state

    my $ResourceAgitation_state = NACL::C::ResourceAgitation->state();

(Instance method)
Fetches the current resource agitation levels and returns a
Component State object. (Of type
L<NACL::CS::ResourceAgitation|lib-NACL-CS-ResourceAgitation-pm>)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

    NACL::C::ResourceAgitation->modify(command_interface => $ci, ...);

                     or

    $ResourceAgitation_object->modify(...);

(Class or instance method)
This method is used to modify the resource agitation levels of one or more
resources

Uses a 7Mode CLI APISet or Nodescope CLI APISet.
  Invokes 'setral' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< cpu => "<consume-%>" >>

Set the CPU agitation level to the given percentage. If set to 50, it will in
essence take 50% of the CPU away from the normal filer processes.
  Applicable only for 7Mode CLI.

=item C<< malloc => "<consume-%> <random-deny 1-out-of-N  (0=off)>" >>

Consume up to <first parameter> percentage of all the available allocatable memory
resources, and/or randomly deny assigning a memory resource 1 out of N
(<second parameter>) times.
  Applicable only for 7Mode CLI.

=item C<< kma_sleep => "<random-sleep 1-out-of-N (0=off)>  <sleep N msecs>" >>

Randomly cause KMA_SLEEP allocations to sleep, 1 out of N <first parameter> times,
for MSECS <second parameter> msecs each time.
  Applicable only for 7Mode CLI.

=item C<< mbuf => "<consume-%> <random-deny 1-out-of-N (0=off)>" >>

Consume up to <first parameter> percentage of all the available network buffer
resources,  nd/or randomly deny assigning a network buffer 1 out of N
<second parameter> times.
  Applicable only for 7Mode CLI.

=item C<< waflbuf => "<consume-%>  <random-deny 1-out-of-N (0=off)>" >>

Consume up to <first parameter> percentage of all the available WAFL buffer
resources, and/or randomly deny assigning a wafl buffer 1 out of N
<second parameter> times.
  Applicable only for 7Mode CLI.

=item C<< nwdrop => "<random-drop 1-out-of-N  (0=off)>" >>

Randomly drop 1 out of N incoming or outgoing networking packets.
  Applicable only for 7Mode CLI.

=item C<< nwcorrupt => "<random-corrupt 1-out-of-N (0=off)>" >>

Randomly corrupt 1 out of N incoming n/w packets. The Layer 4 and up data will
also be randomly corrupted, while keeping the udp/tcp checksums valid.
  Applicable only for 7Mode CLI.

=item C<< nscorrupt => "<random-corrupt 1-out-of-N (0=off)>" >>

Randomly corrupt 1 out of N name service requests. Return one of the valid error
codes for the name service code.
  Applicable only for 7Mode CLI.

=item C<< spinhicsm => "<random-deny 1-out-of-N (0=off)>" >>

Randomly return a CSM backpressure erorr to 1 out of N SpinNP callbacks.
  Applicable only for 7Mode CLI.

=item C<< fd => "<consume-%> <random-deny 1-out-of-N (0=off)>" >>

  Applicable only for 7Mode CLI.

=item C<< pBlk => "<consume-%> <random-deny 1-out-of-N (0=off)>" >>

  Applicable only for 7Mode CLI.

=item C<< raidrm => "<consume-%> <random-deny 1-out-of-N (0=off)>" >>

  Applicable only for 7Mode CLI.

=item C<< java => "<random-deny 1-out-of-N (0=off)>" >>

  Applicable only for 7Mode CLI.

=item C<< gc => "<random-gabage collect 1-out-of-N (0=off)>" >>

  Applicable only for 7Mode CLI.

=item C<< page => "<random-deny 1-out-of-N (0=off)>" >>

  Applicable only for 7Mode CLI.

=item C<< hseg => "<random-deny 1-out-of-N (0=off)>" >>

  Applicable only for 7Mode CLI.

=item C<< spinhimem => "<random-deny 1-out-of-N (0=off)>" >>

  Applicable only for 7Mode CLI.

=item C<< socket => "<random-deny 1-out-of-N (0=off)>" >>

  Applicable only for 7Mode CLI.

=item C<< nblhostrpc => "<random-deny 1-out-of-N (0=off)>" >>

  Applicable only for 7Mode CLI.

=item C<< incl => [$rtag1, $rtag2, ...] >>

  Applicable only for 7Mode CLI.

=item C<< excl => [$tar1, $rtag2, ...] >>

  Applicable only for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            incl => { type => ARRAYREF, optional => 1 },
            excl => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw(malloc kma_sleep mbuf walfbuf fd pBlk raidrm java gc
                    page hseg socket nwdrop nwcorrupt nscorrupt spinhicsm
                    spinhimem nblhostrpc cpu)
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _modify_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    unless ( keys %opts ) {
        NATE::Exceptions::Argument->throw( "Atleast one parameter must "
                . "be provided in the call to 'modify'" );
    }

    if ( defined $opts{incl} ) {
        $opts{incl} = join ' ', @{ $opts{incl} };
    }
    if ( defined $opts{excl} ) {
        $opts{excl} = join ' ', @{ $opts{excl} };
    }

    $apiset->setral(%opts);
    $Log->exit() if $may_exit;
}

=head2 reset

    NACL::C::ResourceAgitation->reset(command_interface => $ci);

                     or

    $ResourceAgitation_object->reset();

(Class or instance method)
This method is used to set all agitation levels to zero, and stop the associated
logic. This will also return all resources currently held.

Uses a 7Mode CLI APISet or Nodescope CLI APISet.
  Invokes 'setral' command for 7Mode CLI.

=over

=item Options

None

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


sub reset {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with( params => \@args );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_reset_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _reset_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    if ( keys %opts ) {
        NATE::Exceptions::Argument->throw(
            "No parameters should be provided in the call to 'reset'");
    }

    # Can only be run in privilege levels advanced and diag
    $apiset->setral(reset => 1);
    $Log->exit() if $may_exit;
}

=head2 stall

    NACL::C::ResourceAgitation->stall(command_interface => $ci, time => $time);

                     or

    $ResourceAgitation_object->stall(time => $time);

(Class or instance method)
This method will consume 100% of the cpu for the given number of seconds
(from 1 to 20).

Uses a 7Mode CLI APISet or Nodescope CLI APISet.
  Invokes 'setral' command for 7Mode CLI.

=over

=item Options

=over

=item C<< time => $seconds >>

(Required)
Time, in seconds, for which we want to stall the filer. (Valid range: 1 to 20 seconds)

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


sub stall {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => { time => { type => SCALAR, required => 1 }, },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_stall_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _stall_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    # Can only be run in privilege levels advanced and diag
    $apiset->setral(stall => $opts{time});
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

1;
