# $Id$

#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SOAPCommandInterface Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SOAPCommandInterface

=head1 DESCRIPTION

C<NACL::C::SOAPCommandInterface> is a derived class of
L<NACL::C::Component::SOAP|lib-NACL-C-Component-SOAP-pm>. It represents access
to an SOAP Server.

=head1 ATTRIBUTES

=head2 soapserver

The name of the soap server this SOAPCommandInterface object represents.

=cut

package NACL::C::SOAPCommandInterface;

use strict;
use warnings;

use base 'NACL::C::Component::SOAP';

use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);

use NACL::APISet;
use NATE::SOAP::Data;
use Hostrec;

use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

=head1 METHODS

=head2 new

    my $soapserverinterface_obj= NACL::C::SOAPCommandInterface->new(
                                   soapserver => $soap_server_name,
                                 );

(Class method)
This method is used to create a new SOAPCommandInterface object. It is the essientially the command interface
for all NACL SOAP Components.

Use for SOAP environments and components.

=over

=item Options

=over

=item C<< soapserver => $soapserver >>

(Required) The name of the soap server to create the SOAPCommandInterface component for.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

use Class::MethodMaker [
        new => ['-hash','-init', 'new'],
        scalar => 'soapserver',
        scalar => [
            {-type => 'NACL::APISet::Host::SOAP'},
            'command_interface'
        ],
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub init {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;


    validate_with(
        params => \@args,
        spec   => { %{NACL::C::Component->_common_validate_spec() },
                  'soapserver' => {type => SCALAR, optional => 0},
        },
        allow_extra => 1,
    );

    my %create_args;
    my $host = Hostrec->(id => $pkg->soapserver,
                         hosttype => 'svp');

    $create_args{host} = $host;
    $create_args{category} = 'Host';
    $create_args{interface} = 'SOAP';

    my $apiset_obj = NACL::APISet->new(%create_args);
    $pkg->command_interface($apiset_obj);

    $Log->exit() if $may_exit;
    return $pkg;
}

=head2 call
    (Class Call)
    my $param1 = NATE::SOAP::Data->new(paramName => "paramValue");

    my @params = ($param1, {param2 => 'value2'});

    my $ci = NACL::APISet->new(
        hostobj   => $host,
        category  => "Host",
        interface => "SOAP",
    );
    my $result = NACL::C::SOAPCommandInterface->call(command_interface => $ci,
                                                     command => 'soapCommand',
                                                     params => \@params);

    (Instance Call)
    my $soap_command_interface_obj = NACL::C::SOAPCommandInterface->create(
                                             soapserver => $soapserver_name,
                                     );

    my $param1 = NATE::SOAP::Data->new(paramName => "paramValue");

    my @params = ($param1, {param2 => 'value2'});

    my $soap_command_interface_obj->call(command => 'soapCommand',
                                         params  => \@params,
                                        );

This method is used to create a new SOAPCommandInterface object. This is essentially a command interface
object for soap servers.

This class is intended to be subclassed by a higher level soap component object.

=over

=item Options

=over
=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method) See L<NACL::APISet::Host::SOAP|lib-NACL-APISet-Host-SOAP-pm>.

=item C<< command => $command >>

(Required) The command we wish to execute against the soap interface.

=item C<< params => \@params >>

(Optional) The arrayref of parameters for the command to be run against the SOAPCommandInterface. The paramaters can be hash
references or L<C<NATE::SOAP::Data> objects.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $pkg->command_interface($args{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface) {
        NACL::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'command_interface' missing in call to NACL::C::SOAPInterface::call.");
    }

    my $command = delete $args{command};
    my $response = $pkg->command_interface->$command(%args);

    $Log->exit() if $may_exit;
    return $response->get_parsed_output();
}

1;
