# $Id: $

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageAggregateMember Component Module
## @author chethan.m@netapp.com@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StorageAggregateMember

=head1 DESCRIPTION

C<NACL::C::StorageAggregateMember> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP Storage Aggregate Member.  A related class is
L<NACL::CS::StorageAggregateMember|lib-NACL-CS-StorageAggregateMember-pm>, which represents
the past state of an ONTAP Storage Aggregate Member.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 aggregate

The name of the aggregate, used to identify it uniquely.  (Aggregate
names are assumed to be unique within a node or within a cluster)

=cut

package NACL::C::StorageAggregateMember;

use strict;
use warnings;

use NACL::CS::StorageAggregateMember;
use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $aggregate_member = NACL::C::StorageAggregateMember->new(command_interface=>$ci,
                                         aggregate =>$aggr_name);

Return a new Aggregate Member component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'aggregate',];

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::C::AggregateMember'
    );
    $Log->exit() if $may_exit;
    return $isa;
}

=head2 add_disks

  NACL::C::StorageAggregateMember->add_disks(command_interface=>$ci,
                                      aggregate=>$aggr_name,...);
							or

  $aggregate_member_object->add_disks(...);

(Class or instance method) This method adds disks to an aggregate.
Uses a CMode-CLI APIset.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< aggregate=>$aggr_name >>

(Required for class method, Not Applicable for instance method)
Name of the member aggregate to add disks.

=item C<< diskcount => $string >>

=item C<< disklist => [$disk1, $disk2, $disk3, ... ] >>

=item C<< raidgroup => $string >>

=item C<< "allow-mixed" => $string >>

=item C<< 'mirror-disklist' => [ $mirror-disk1, $mirror-disk2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

=item C<< 'disksize-in-blks' => $string >>

=item C<< 'group-selection-mode' => $string >>

=item C<< 'disksize-with-unit' => $string >>

=item C<< 'ignore-pool' => $string >>

=item C<< '*' >>

This method takes all other options supported by
"storage aggregate member add-disks" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add_disks {
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "mirror-disklist" => {type => ARRAYREF, optional => 1},
            disklist          => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw (diskcount raidgroup allow-mixed
                    disksize-in-blks group-selection-mode disksize-with-unit ignore-pool
                    )
            ),
        },
        api => 'storage_aggregate_member_add_disks',
    );
    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_add_disks_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },

        ],
        frontend => 'NACL::C::StorageAggregateMember::add_disks',
    );
} ## end sub add_disks

sub _add_disks_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api => 'storage_aggregate_member_add_disks'
    );

    $Log->exit() if $may_exit;
} ## end sub _add_disks_cmode_cli

=head2 scrub

  NACL::C::StorageAggregateMember->scrub(command_interface=>$ci,
                             aggregate=>$aggr_name,... );

                            or

  $aggregate_member_object->scrub(...);

(Class or instance method) This method performs various scrub functions
on an aggregate element. Uses a CMode-CLI APIset.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< aggregate=>$aggr_name >>

(Required for class method, Not Applicable for instance method)
Name of the member aggregate.

=item C<< action => $string >>

(Required) Takes the following values : start , stop , resume, suspend and
status.

=item C<< raidgroup => $string >>

=item C<< 'plex' => $string >>

=item C<< '*' >>

This method takes all other options supported by
"storage aggregate member scrub" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub scrub {
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            disklist => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw (diskcount raidgroup allow-mixed
                    action
                    plex
                    )
            ),
        },
        api => 'storage_aggregate_member_scrub',
    );
    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_scrub_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },

        ],
        frontend => 'NACL::C::StorageAggregateMember::scrub',
    );
} ## end sub scrub

sub _scrub_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api => 'storage_aggregate_member_scrub',
    );

    $Log->exit() if $may_exit;
} ## end sub _scrub_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (aggregate => {type => SCALAR});
}

1;
