# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageAggregateWaflironMember Component Module
## @author Sumit.Mittal@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StorageAggregateWaflironMember

=head1 DESCRIPTION

C<NACL::C::StorageAggregateWaflironMember> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP Wafliron on an aggregate member.  A related class is
L<NACL::CS::StorageAggregateWaflironMember|lib-NACL-CS-StorageAggregateWaflironMember-pm>, which represents
the past state of an ONTAP Wafliron on an aggregate member.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 aggregate

The name of the member aggregate on which wafliron is done,
used to identify it uniquely.  (Aggregate
names are assumed to be unique within a node or within a cluster)

=cut

package NACL::C::StorageAggregateWaflironMember;

use strict;
use warnings;

use NACL::CS::StorageAggregateWaflironMember;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Data::Dumper;
use Params::Validate qw(validate_with SCALAR);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 new

my $wafliron = NACL::C::StorageAggregateWaflironMember->new(command_interface=>$ci,
                                         aggregate =>$aggr_name);

Return a new StorageAggregateWaflironMember component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'aggregate',];

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::C::WaflironMember'
    );
    $Log->exit() if $may_exit;
    return $isa;
}

=head2 start

  my $wafliron  = NACL::C::StorageAggregateWaflironMember->start(command_interface=>$ci,
                                                 aggregate=>$aggr_name,...);

(Class method) Request creation of a new waflironmember element, and  return a new
waflironmember component object that refers to it.  Does not wait for wafliron
to complete.  

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< aggregate=>$aggr_name >>

(Required) Name of aggregate which has to be waflironed.

Applicable for CMode CLI.

=item C<< include-mirrors => $boolean >>

(Optional) option to indicate if snapmirrors need to be included for checking

Applicable for CMode CLI.

=item C<< optional-commit => $boolean >>

(Optional) option to indicate whether the changes to the filesystem need to be

Applicable for CMode CLI.

committed to the disk or not.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {_optional_scalars(qw(include-mirrors))}
    );
    return $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_start_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            }
        ],
    );
} ## end sub start

sub _start_cmode_cli {

    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_cmode_options
        = [qw(aggregate include-mirrors extended_query )];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $apiset->storage_aggregate_wafliron_member_start(%opts);

    return $pkg->new(
        aggregate         => $opts{aggregate},
        command_interface => $command_interface
    );

} ## end sub _start_cmode_cli

=head2 stop

  NACL::C::StorageAggregateWaflironMember->stop(command_interface=>$ci,
                          aggregate=>$aggr_name,...);

 or

 $waflironmember_object->stop(...)

(Class or instance method) The method stops an active wafliron on a member aggregate.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>


=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< aggregate=>$aggr_name >>

(Required for class method, Not Applicable for instance method)
Name of the member aggregate in which the wafliron needs to be stopped.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop {
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_common_validate_with(params => \@_,);
    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_stop_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            }
        ],
    );
} ## end sub stop

sub _stop_cmode_cli {

    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [qw(aggregate extended_query )];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $apiset->storage_aggregate_wafliron_member_stop(%opts);

} ## end sub _stop_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (aggregate => {type => SCALAR},);
}

1;
