#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageDisk Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StorageDisk

=head1 DESCRIPTION

C<NACL::C::StorageDisk> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP StorageDisk.  A related class is
L<NACL::CS::StorageDisk|lib-NACL-CS-StorageDisk-pm>, which represents
the past state of an ONTAP StorageDisk.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 disk

=cut

package NACL::C::StorageDisk;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::CS::StorageDisk;
use NACL::CS::RaidConfigInfoListDisk;
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NATE::Exceptions::Argument;
use NATE::BaseException;
use NACL::C::Exceptions::StorageDisk::NotFileSystemDisk;
use NACL::Global;

use base 'NACL::C::Component::ONTAP';

use Params::Validate
    qw(validate validate_with SCALAR ARRAYREF OBJECT HASHREF SCALARREF);

=head1 METHODS

=head2 new

  my $StorageDisk = NACL::C::StorageDisk->new(command_interface=>$ci,
                                   disk =>$disk,
                                  );

Return a new StorageDisk component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'disk',];

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::C::Disk'
    );
    $Log->exit() if $may_exit;
    return $isa;
} ## end sub isa

=head2 zeroblock

   NACL::C::StorageDisk->zeroblock(
        command_interface => $ci,
        disk => $disk,
        dbn  => "$block"
        ...);

   or

   $StorageDisk_object->zeroblock(
    dbn => "$block"
    ...);

(Class or instance method)

Uses a CMode or 7Mode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< disk => $disk >>

(Required for class method, Not Applicable for instance method)
    Applicable only for CMode CLI.

=item C<< dbn => $block >>

(Required )
    Applicable only for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

__PACKAGE__->_nodescope_builder(command => 'zeroblock');

=head2 setled

   NACL::C::StorageDisk->setled(
                                command_interface => $ci,
                                 disk => $disk,
                                  ...);
                                 or

    $StorageDisk_object->setled(...);

(Class or instance method)

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< disk => $disk >>

(Required for class method, Not Applicable for instance method)
    Applicable only for CMode CLI.

=item C<< time => $string >>

    Applicable only for CMode CLI.

=item C<< action => $string >>

    Applicable only for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)
    Applicable only for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"storage disk setled" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub setled {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {_optional_scalars(qw( time action)),},
        cli_cmd         => 'storage disk set-led',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_setled_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::StorageDisk::setled',
    );
    $Log->exit() if $may_exit;
} ## end sub setled

sub _setled_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api  => 'storage_disk_setled');
    $Log->exit() if $may_exit;
} ## end sub _setled_cmode_cli

=head2 replace

   NACL::C::StorageDisk->replace(
                                command_interface => $ci,
                                 disk => $disk,
                                  ...);
                                 or

    $StorageDisk_object->replace(...);

(Class or instance method)

Uses a CMode CLI or 7Mode CLI APISet.
  Invokes 'disk replace' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< disk => $disk >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI, 7Mode CLI.
  Maps to:
    7M CLI : disk <value> (Programmatic name: "disk")

=item C<< action => $string >>

  Applicable only for CMode CLI.

=item C<< replacement => $string >>

Applicable for CMode CLI, 7Mode CLI.
  Maps to:
    7M CLI : spare disk name <value> (Programmatic name: "spare-disk-name")

=item C<< 'hidden-uid' => $string >>

Applicable only for CMode CLI.

=item C<< 'allow-same-carrier' => $string >>

Applicable only for CMode CLI.

=item C<< 'uid' => $string >>

Applicable only for CMode CLI.

=item C<< 'allow-mixing' => $string >>

Applicable only for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)
Applicable only for CMode CLI

=item C<< 'reason' => $string >>

Reason for the disk replace
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"storage disk replace" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub replace {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( action replacement
                    hidden-uid
                    allow-same-carrier
                    uid
                    allow-mixing

                    reason
                    )
            ),
        },
        cli_cmd => 'storage disk replace',
    );
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_replace_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_replace_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope",
            },
        ],
        frontend => 'NACL::C::StorageDisk::replace',
    );
    $Log->exit() if $may_exit;
} ## end sub replace

sub _replace_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api  => 'storage_disk_replace');
    $Log->exit() if $may_exit;
} ## end sub _replace_cmode_cli

sub _replace_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [qw( disk )];
    my %args_7mode;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => {'replacement' => 'spare-disk-name',},
        target => \%args_7mode,
    );

    $args_7mode{'force'} = 1;
    if ($opts{'action'} =~ /start/) {
        $args_7mode{'start'} = 1;
    } else {
        $args_7mode{'stop'} = 1;
    }

    $apiset->disk_replace(%args_7mode);
    $Log->exit() if $may_exit;
} ## end sub _replace_7mode_cli

=head2 setstate

   NACL::C::StorageDisk->setstate(
                                command_interface => $ci,
                                 disk => $disk,
                                  ...);
                                 or

    $StorageDisk_object->setstate(...);

(Class or instance method)

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< disk => $disk >>

(Required for class method, Not Applicable for instance method)
Applicable only for CMode CLI.

=item C<< action => $string >>

Applicable only for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"storage disk setstate" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub setstate {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {_optional_scalars(qw( action)),},
        cli_cmd         => 'storage disk setstate',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_setstate_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::StorageDisk::setstate',
    );
    $Log->exit() if $may_exit;
} ## end sub setstate

sub _setstate_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api  => 'storage_disk_setstate');
    $Log->exit() if $may_exit;
} ## end sub _setstate_cmode_cli

=head2 removeowner

   NACL::C::StorageDisk->removeowner(
                                command_interface => $ci,
                                 disk => $disk,
                                  ...);
                                 or

    $StorageDisk_object->removeowner(...);

(Class or instance method)

Uses a CMode CLI or 7Mode CLI APISet or CMode ZAPI APISet.
  Invokes 'disk remove ownership' command for 7Mode CLI.
  Invokes 'disk-sanown-remove-ownership' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< disk => $disk >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI , 7Mode CLI.
Maps to:
    7M CLI : disk list <value> (Programmatic name: "disk-list")
    CM ZAPI:
        <disk-sanown-remove-ownership>
            <disk-list>
               <disk-name>$value</disk-name>
            </disk-list>
        </disk-sanown-remove-ownership>

=item C<< 'hidden-uid' => $string >>

Applicable only for CMode CLI.

=item C<< 'force' => $string >>
Applicable for CMode CLI/ZAPI.
Maps to:
    CM ZAPI:
        <disk-sanown-remove-ownership>
            <force>$value</force>
        </disk-sanown-remove-ownership>

=item C<< 'owner' => $string >>

Applicable for CMode ZAPI.
Maps to:
    CM ZAPI:
        <disk-sanown-remove-ownership>
            <force>$value</force>
        </disk-sanown-remove-ownership>

=item C<< 'uid' => $string >>

Applicable only for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)
Applicable only for CMode CLI.

=item C<< "extended_query" => $string >>

 (Optional)
 Applicable only for CMode CLI

 =item C<< 'data' => $string >>

Data Partition
Applicable for CMode CLI

=item C<< 'root' => $string >>

Root Partition
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"storage disk removeowner" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub removeowner {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( hidden-uid force
                    uid

                    data root
                    )
            )
        },
         cli_cmd => 'storage disk removeowner',
    );
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_removeowner_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_removeowner_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope",
            },
            {   method    => "_removeowner_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            }
        ],
        frontend => 'NACL::C::StorageDisk::removeowner',
    );
    $Log->exit() if $may_exit;
} ## end sub removeowner

sub _removeowner_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api     => 'storage_disk_removeowner',
    );

    $Log->exit() if $may_exit;
} ## end sub _removeowner_cmode_cli

sub _removeowner_7mode_cli {
    my $pkg = shift;
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my %args;
    $pkg->_hash_copy(
        source => \%opts,
        map    => {'disk' => 'disk-list'},
        target => \%args,
    );
    $args{'force'} = 1;

    $apiset->disk_remove_ownership(%args);
    $Log->exit() if $may_exit;
} ## end sub _removeowner_7mode_cli

sub _removeowner_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    # Get the owner and disk names
    $pkg->_get_owner_disk_name(options => \%opts);

    my $allowed_cmode_options = [
        qw( force owner
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $cmode_args{'disk-list'} = [$opts{disk}];
    $apiset->disk_sanown_remove_ownership(%cmode_args);
    $Log->exit() if $may_exit;

} ## end sub _removeowner_cmode_zapi

=head2 modify

   NACL::C::StorageDisk->modify(
                                command_interface => $ci,
                                 disk => $disk,
                                  ...);
                                 or

    $StorageDisk_object->modify(...);

(Class or instance method)

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< disk => $disk >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI.

=item C<< immediate => $string >>

  Applicable only for CMode CLI.

=item C<< owner => $string >>

  Applicable only for CMode CLI.

=item C<< ownerid => $string >>

  Applicable only for CMode CLI.

=item C<< state => $string >>

  Applicable only for CMode CLI.

=item C<< 'uid' => $string >>

  Applicable only for CMode CLI.

=item C<< 'hidden-uid' => $string >>

  Applicable only for CMode CLI.

=item C<< 'uuid' => $string >>

  Applicable only for CMode CLI.

=item C<< 'owner-id' => $string >>

  Applicable only for CMode CLI.

=item C<< 'force-owner' => $string >>

  Applicable only for CMode CLI.

=item C<< 'allow-mixing' => $string >>

  Applicable only for CMode CLI.

=item C<< 'is-foreign' => $string >>

  Applicable only for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)
  Applicable only for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"storage disk modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( immediate owner ownerid state
                    uid hidden-uid uuid owner-id force-owner

                    allow-mixing

                    is-foreign
                    )
            ),
        },
        cli_cmd => 'storage disk modify',
    );
    ## for burt 897474
    if( $opts{'command_interface'}->version_manager()->is_version_ge(
        release => $NACL::Global::ONTAP_Release_Longboard_0 ) &&
       defined $opts{'is-foreign'}){

        my %common_opts;
        $common_opts{'is-foreign-lun'} = delete $opts{'is-foreign'};
        $pkg_or_obj->_copy_common_component_params_with_ci(
            source => \%opts,
            target => \%common_opts
        );
        $common_opts{disk} = $opts{disk} if defined $opts{disk};
        $pkg_or_obj->set_foreign_lun(%common_opts);
    } else {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_modify_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
            ],
            frontend => 'NACL::C::StorageDisk::modify',
        );
    }
    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api  => 'storage_disk_modify');
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

=head2 set_foreign_lun

    NACL::C::StorageDisk->set_foreign_lun(
                                command_interface => $ci,
                                    'disk' => $disk,
                                ...);

                    or

    $StorageDisk_obj->set_foreign_lun(...);

(Class or instance method)
This method is used to sets or Unsets an array LUN as foreign.
Uses CMode CLI/ZAPI .
Invokes  storage-disk-set-foreign-lun  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'disk' => $disk >>

(Required for class method, Not Applicable for instance method)
Disk Name
Applicable for CMode CLI/ZAPI, non-iter.

Mapping for non-iter: disk

=item C<< 'is-foreign-lun' => $string >>

Is Foreign LUN
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "storage disk set foreign lun" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub set_foreign_lun {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj,@args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(disk is-foreign-lun)
            ),
        },
        cli_cmd => 'storage disk set-foreign-lun',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_set_foreign_lun_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_set_foreign_lun_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::StorageDisk::set_foreign_lun',
    );
    $Log->exit() if $may_exit;
}

sub _set_foreign_lun_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(
            @args,
            api  => 'storage_disk_set_foreign_lun'
    );
    $Log->exit() if $may_exit;
}

sub _set_foreign_lun_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $copy = [ qw( disk is-foreign-lun )];

    $pkg->_base_backend_cmode_zapi(%opts,
            api    => 'storage-disk-set-foreign-lun',
            copy   => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 zerospares

   NACL::C::StorageDisk->zerospares(
                                command_interface => $ci,

                                  ...);

(Class method)
Uses a CMode CLI or 7Mode CLI APISet or CMode ZAPI APISet.
  Invokes 'disk zero spares' command for 7Mode CLI.
  Invokes 'disk-zero-spares' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< owner => $string >>
(Required)

Applicable only for CMode CLI/ZAPI.
Maps to:
    CM ZAPI:
    <disk-zero-spares>
        <node-name>$value</node-name>
    </disk-zero-spares>


=item C<< '*' >>

This method takes all other options supported by
"storage disk zerospares" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI

=back

=back

=cut

sub zerospares {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        additional_spec     => {_optional_scalars(qw( owner )),},
        ignore_primary_keys => 1,
        cli_cmd             => 'storage disk zerospares',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_zerospares_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_zerospares_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
            {   method    => "_zerospares_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::StorageDisk::zerospares',
    );
    $Log->exit() if $may_exit;
} ## end sub zerospares

sub _zerospares_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,  api  => 'storage_disk_zerospares');
    $Log->exit() if $may_exit;
} ## end sub _zerospares_cmode_cli

sub _zerospares_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    $apiset->disk_zero_spares();
    $Log->exit() if $may_exit;
} ## end sub _zerospares_7mode_cli

sub _zerospares_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_zapi(@args,
            api    => 'disk_zero_spares',
            map    => {'owner' => 'node-name'},
	);
    $Log->exit() if $may_exit;
} ## end sub _zerospares_cmode_zapi

=head2 assign

   NACL::C::StorageDisk->assign(
                                command_interface => $ci,

                                  ...);
   $StorageDisk_object->assign(...);

(Class or instance method) When called as a class method, works on all or any
of the disks present and when called on an instance, works on that particular
disk element to which the component is modeled.
Uses a CMode CLI or 7Mode CLI APISet or CMode ZAPI APISet.
  Invokes 'disk assign' command for 7Mode CLI.
  Invokes 'disk-sanown-assign' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< all => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.
Maps to:
    7M CLI : -all (Programmatic name: 'all-disks')
    CM ZAPI:
        <disk-sanown-assign>
            <all>$value</all>
        </disk-sanown-assign>

=item C<< count => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.
Maps to:
    7M CLI : -n <value> (Programmatic name: 'count')
    CM ZAPI:
        <disk-sanown-assign>
            <disk-count>$value</disk-count>
        </disk-sanown-assign>

=item C<< auto => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.
Maps to:
    7M CLI : auto (Programmatic name: 'auto-assign')
    CM ZAPI:
        <disk-sanown-assign>
           <auto>$value</auto>
        </disk-sanown-assign>

=item C<< disk => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.
Maps to:
    7M CLI : disk list (Programmatic name: 'disk-list')
    CM ZAPI:
        <disk-sanown-assign>
            <disk>$value</disk>
        </disk-sanown-assign>

=item C<< type => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.
  Maps to:
    7M CLI : -T <value> (Programmatic name: 'storage-type')
    CM ZAPI:
        <disk-sanown-assign>
            <disk-type>$value</disk-type>
        </disk-sanown-assign>

=item C<< pool => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.
  Maps to:
    7M CLI : -p  <value> (Programmatic name: 'pool')
    CM ZAPI:
        <disk-sanown-assign>
            <pool>$value</pool>
        </disk-sanown-assign>

=item C<< owner => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.
  Maps to:
    7M CLI : -o <value> (Programmatic name: 'owner-name')
    CM ZAPI:
        <disk-sanown-assign>
            <owner>$value</owner>
        </disk-sanown-assign>

=item C<< sysid => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.
  Maps to:
    7M CLI : -s <value> (Programmatic name: 'sysid')
    CM ZAPI:
        <disk-sanown-assign>
           <owner-id>$value</owner-id>
        </disk-sanown-assign>

=item C<< checksum => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.
  Maps to:
    7M CLI : -c <value> (Programmatic name: 'checksum')
    CM ZAPI:
        <disk-sanown-assign>
            <checksum>$value</checksum>
        </disk-sanown-assign>

=item C<< force => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.
Maps to:
    7M CLI : -f (Programmatic name: 'force')
    CM ZAPI:
        <disk-sanown-assign>
           <force>$value</force>
        </disk-sanown-assign>

=item C<< 'node' => $string >>

Applicable only for CMode CLI/ZAPI.
Maps to:
    CM ZAPI:
        <disk-sanown-assign>
           <node-name>$value</node-name>
        </disk-sanown-assign>

=item C<< 'data' => $string >>

Data Partition
Applicable for CMode CLI

=item C<< 'root' => $string >>

Root Partition
Applicable for CMode CLI

=item C<< 'copy-ownership-from' => $string >>

Disk Name to Copy Ownership
Applicable for CMode CLI

=item C<< 'assigntype' => $string >>

Assignment of a partition possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"storage disk assign" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI

=back

=back

=cut

sub assign {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( disk all auto count type pool owner sysid
                    checksum force
                    node

                    data root

                    copy-ownership-from

                    assigntype
                    )
            )
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'storage disk assign',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_assign_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_assign_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope",
            },
            {   method    => "_assign_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::StorageDisk::assign',
    );
    $Log->exit() if $may_exit;
} ## end sub assign

sub _assign_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api  => 'storage_disk_assign');
    $Log->exit() if $may_exit;

} ## end sub _assign_cmode_cli

sub _assign_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_7mode_options = [qw(pool sysid checksum force count)];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => {
            'disk'  => 'disk-list',
            'all'   => 'all-disks',
            'type'  => 'storage-type',
            'auto'  => 'auto-assign',
            'owner' => 'owner-name',
        },
        target => \%args,
    );

    if ($args{'owner-name'} =~ /(.*)\(\d+\)/) {
        $args{'owner-name'} = $1;
    }

    $apiset->disk_assign(%args);
    $Log->exit() if $may_exit;
} ## end sub _assign_7mode_cli

sub _assign_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    # Get the owner and disk names
    $pkg->_get_owner_disk_name(options => \%opts) if(defined $opts{disk});
	my $allowed_cmode_options = [
        qw( disk force checksum pool
            all auto owner
            )
    ];
    $pkg->_base_backend_cmode_zapi(%opts,
            api    => 'disk_sanown_assign',
            copy   => $allowed_cmode_options,
            map    => {
                'count' => 'disk-count',
                'type'  => 'disk-type',
                'node'  => 'node-name',
                'sysid' => 'owner-id'
        },
	);
    $Log->exit() if $may_exit;
} ## end sub _assign_cmode_zapi

=head2 release_all

   NACL::C::StorageDisk->release_all(
                                command_interface => $ci,

                                  ...);

(Class method)

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< node => $string >>

(Required)
  Applicable only for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"storage disk release-all" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub release_all {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        additional_spec     => {node => {type => SCALAR}},
        ignore_primary_keys => 1,
        api                 => 'storage_disk_release_all',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_release_all_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::StorageDisk::release_all',
    );
    $Log->exit() if $may_exit;
} ## end sub release_all

sub _release_all_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->_base_backend_cmode_cli(
        %opts,
        api     => 'storage_disk_release_all',
    );

    $Log->exit() if $may_exit;
} ## end sub _release_all_cmode_cli

=head2 reassign

   NACL::C::StorageDisk->reassign(
                                command_interface => $ci,

                                  ...);

(Class method)

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< home => $string >>

  Applicable only for CMode CLI.

=item C<< newhome => $string >>

  Applicable only for CMode CLI.

=item C<< homeid => $string >>

  Applicable only for CMode CLI.

=item C<< newhomeid => $string >>

  Applicable only for CMode CLI.

=item C<< force => $string >>

  Applicable only for CMode CLI.

=item C<< 'force' => $string >>

*Force
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"storage disk reassign" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI

=back

=back

=cut

sub reassign {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( node home newhome homeid newhomeid apiset
                    force
                    )
            ),
        },
        ignore_primary_keys => 1,
         cli_cmd                 => 'storage disk reassign',
    );

    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_reassign_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            }
        ],
        frontend => 'NACL::C::StorageDisk::reassign',
    );
    $Log->exit() if $may_exit;
} ## end sub reassign

sub _reassign_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->_base_backend_cmode_cli(
        %opts,
        api     => 'storage_disk_reassign',
    );


    $Log->exit() if $may_exit;
} ## end sub _reassign_cmode_cli

=head2 updatefirmware

   NACL::C::StorageDisk->updatefirmware(
                                command_interface => $ci,

                                  ...);
   $StorageDisk_object->updatefirmware(...);

(Class or instance method) When called as a class method, works on all or any
of the disks present and when called on an instance, works on that particular
disk element to which the component is modeled.

Uses a CMode CLI/ZAPI

Invokes "disk-update-disk-fw" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< disk => [$disk1, $disk2, ...] >>
  Applicable only for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)
  Applicable only for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"storage disk updatefirmware" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub updatefirmware {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            {disk => {type => SCALAR | ARRAYREF, optional => 1}},
        ignore_primary_keys => 1,
         cli_cmd                 => 'storage disk updatefirmware',
    );
    $pkg_or_obj->_scalar_to_array_deprication_warning(
        params  => ['disk'],
        options => \%opts
    );

    $pkg_or_obj->call_on_apiset_for_updatefirmware(
        %opts,
        choices => [
            {   method    => "_updatefirmware_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_updatefirmware_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
                check     => '_cmode_zapi_check',
            },
        ],
        frontend => 'NACL::C::StorageDisk::updatefirmware',
    );
    $Log->exit() if $may_exit;
} ## end sub updatefirmware

sub _updatefirmware_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api     => 'storage_disk_updatefirmware',
    );

    $Log->exit() if $may_exit;
} ## end sub _updatefirmware_cmode_cli

sub _updatefirmware_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    my $command_interface = $opts{command_interface};
    my $node              = $command_interface->node();

    my %cmode_args;
    $pkg_or_obj->_zapi_hash_copy(
        source => \%opts,
        target => \%cmode_args,
        map    => ["disk" => "disk-list",],

    );
    $cmode_args{'node-name'} = $node;
    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api             => 'disk-update-disk-fw',
        frontend_method => 'updatefirmware',
        additional_args => \%cmode_args,
    );
}
sub _cmode_zapi_check {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    if (defined $opts{'disk'}) {
        my $msg = 'Could not use ZAPI implementation of NACL::C::StorageDisk->updatefirmware().'
            .' since the CMode ZAPI does not support "disk-name" option';
        $Log->comment($msg);
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw($msg);
    }
     $Log->exit() if $may_exit;
}

=head2 set_led

    NACL::C::StorageDisk->set_led(
                                command_interface => $ci,
                                'disk' => $disk,
                                ...);

                                 or

    $StorageDisk_obj->set_led(...);

(Class or instance method)
This method is used to turn on a disk's red LED for a number of minutes.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'disk' => $disk >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI.

=item C<< 'time' => $string >>

  Applicable only for CMode CLI.

=item C<< 'action' => $string >>

  Applicable only for CMode CLI.

=item C<< 'hidden-uid' => $string >>

  Applicable only for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'uid' => $string >>

  Applicable only for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)
  Applicable only for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"storage disk set-led" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub set_led {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( time action hidden-uid

                    uid
                    )
            ),
        },
         cli_cmd => 'storage disk set-led',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_set_led_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::StorageDisk::set_led',
    );

    $Log->exit() if $may_exit;
} ## end sub set_led

sub _set_led_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api     => 'storage_disk_set_led',
    );

    $Log->exit() if $may_exit;
} ## end sub _set_led_cmode_cli

=head2 remove_reservation

    NACL::C::StorageDisk->remove_reservation(
                                command_interface => $ci,
                                'disk' => $disk,
                                ...);

                                 or

    $StorageDisk_obj->remove_reservation(...);

(Class or instance method)
This method is used to removes reservation from an array LUN marked as foreign..
Uses CMode CLI/ZAPI .
Invokes  storage-disk-remove-reservation for CMode Zapi

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'disk' => $disk >>

(Required for class method, Not Applicable for instance method)
Disk Name
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "storage disk remove reservation" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove_reservation {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'storage_disk_remove_reservation',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_remove_reservation_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_remove_reservation_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::StorageDisk::remove_reservation',
    );

} ## end sub remove_reservation

sub _remove_reservation_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'storage_disk_remove_reservation',);
    $Log->exit() if $may_exit;
} ## end sub _remove_reservation_cmode_cli

sub _remove_reservation_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $map = {'disk' => 'disk-name',};
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api => 'storage-disk-remove-reservation',
        map => $map,
    );
    $Log->exit() if $may_exit;
}

=head2 refresh_ownership

    NACL::C::StorageDisk->refresh_ownership(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to refresh the disk ownership information on a node.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $string >>

Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "storage disk refresh ownership" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub refresh_ownership {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( node
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'storage disk refresh-ownership',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_refresh_ownership_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::StorageDisk::refresh_ownership',
    );

    $Log->exit() if $may_exit;
}

sub _refresh_ownership_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'storage_disk_refresh_ownership',);
    $Log->exit() if $may_exit;
}

=head2 scrub

    NACL::C::StorageDisk->scrub(
                                command_interface => $ci,
                                ...);

(Class method)
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< '*' >>

This method takes all other options supported by "storage disk scrub" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub scrub {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        api                 => 'storage_disk_scrub',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_scrub_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::StorageDisk::scrub',
    );

    $Log->exit() if $may_exit;
}

sub _scrub_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'storage_disk_scrub',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (disk => {type => SCALAR},);
}

=head2 fail

   NACL::C::StorageDisk->fail(
                                command_interface => $ci,
                                 disk => $disk,
                                 immediate => "true");
                                 or

    $StorageDisk_object->fail(...);

(Class or instance method)

Uses a CMode CLI or 7Mode CLI APISet or CMode ZAPI APISet.
  Invokes 'disk fail' command for 7Mode CLI.
  Invokes 'disk-fail' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< disk => $disk >>

(Required for class method, Not Applicable for instance method)
Applicable for CMode CLI, 7Mode CLI.
Maps to:
    7M CLI : disk name(Programmatic name: "disk")
    CM ZAPI:
       <disk-fail>
          <disk>$value</disk>
       </disk-fail>

=item C<< immediate => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.
Maps to:
    7M CLI : -i (Programmatic name: 'is-immediate-disks')
    CM ZAPI:
        <disk-fail>
            <is-immediate>$value</is-immediate>
        </disk-fail>

=item C<< 'hidden-uid' => $string >>

  Applicable only for CMode CLI.

=item C<< 'uid' => $string >>

  Applicable only for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"storage disk fail" command in ONTAP.

Applicable for CMode CLI.

=item C<< 'nodescope-node-name' => $node_name >>

Applicable only for Nodescope-CLI

Explicitly specify the node on which to run the Nodescope command (the APISet
defaults to running on node "local")

=item C<< determine_appropriate_nodescope_node => 0 | 1 >>

(Optional, defaults to 0, applicable only for Nodescope-CLI)
If set to 1, the method determines which node the volume hosting the file
is present on and runs the Nodescope CLI command on that particular node.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::StorageDisk::NotFileSystemDisk>

This exception is thrown when attempting to fail a disk that is not a
file-system disk.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<< 'extended_query' => $string >>

(Optional)
Applicable only for CMode CLI

=back

=back

=cut

sub fail {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( immediate
                    hidden-uid
                    uid
                    ),
            ),
            $pkg_or_obj->_nodescope_options_validate_spec(),
        },
        cli_cmd => 'storage disk fail',
    );
    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_fail_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {
                    method    => "_fail_7mode_cli",
                    interface => "CLI",
                    set       => "7Mode|Nodescope",
                },
                {   method    => "_fail_cmode_zapi",
                    interface => "ZAPI",
                    set       => "CMode",
                },
            ],
            frontend => 'NACL::C::StorageDisk::fail',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::StorageDisk::NotFileSystemDisk
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
} ## end sub fail

sub _fail_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api  => 'storage_disk_fail');
    $Log->exit() if $may_exit;
} ## end sub _fail_cmode_cli

sub _fail_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $apiset = delete $opts{apiset};

    my @allowed_options = ('disk');
    $pkg_or_obj->_nodescope_helper(
        options         => \%opts,
        allowed_options => \@allowed_options
    );

    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => \@allowed_options,
        target => \%args,
        map    => {"immediate" => "is-immediate-disks"},
    );
    $args{'force'} = 1;

    $apiset->disk_fail(%args);
    $Log->exit() if $may_exit;
} ## end sub _fail_7mode_cli

sub _fail_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_get_disk_name(options => \%opts);
	my $allowed_cmode_options = [
        qw( disk
            )
    ];
    $pkg->_base_backend_cmode_zapi(%opts,
            api    => 'disk_fail',
            copy   => $allowed_cmode_options,
            map    => {'immediate' => 'is-immediate'},
	);
    $Log->exit() if $may_exit;
} ## end sub _fail_cmode_zapi

=head


(Applicable only for CMode)
    Determines the node on which the disk is present and returns the node
    name. This is necessary for Nodescope operations, since they need to
    be performed in the context of the node where the disk is present.

    
=cut

sub get_local_node_name {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_common_validate_with(params => \@_);
    if ($opts{command_interface}->mode() ne 'CMode') {
        $Log->exit() if $may_exit;
        return $opts{command_interface}->hostrec->hostname;
    }

    # Move the primary keys into the filter, all other options are
    # sent through directly to fetch (these should just be the common options)
    my %filter;
    $pkg_or_obj->_hash_move(
        source => \%opts,
        target => \%filter,
        move   => [$pkg_or_obj->get_primary_keys()]
    );

    ##convert disk name for cmode
    $filter{disk} = $pkg_or_obj->get_ngsh_diskname(
        disk => $filter{disk},
        command_interface => $opts{command_interface}
    );
    my $cs_object = NACL::CS::StorageDisk->fetch(
        %opts,
        filter           => \%filter,
        requested_fields => [qw(owner)]
    );

    my $node = $cs_object->owner();
    $Log->exit() if $may_exit;
    return $node;
} ## end sub get_local_node_name

# Helper for Nodescope back-end methods. If Nodescope, then determines
# which node to run on. This uses "_get_nodescope_node_name" for this,
# this helper ensures that the appropriate options are passed to that
# method. The determined node is added to the options hash 

sub _nodescope_helper {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            options         => {type => HASHREF},
            allowed_options => {type => ARRAYREF}
        }
    );
    my $options = $opts{options};
    my $ci      = $options->{command_interface};

    if ($ci->is_cmode()) {
        my %node_args;
        $pkg_or_obj->_hash_copy(
            source => $options,
            target => \%node_args,
            copy   => [
                        qw(command_interface disk nodescope-node-name
                            determine_appropriate_nodescope_node)
            ],
         );

        my $pkg  = $pkg_or_obj->get_package_name();
        my $node = $pkg->_get_nodescope_node_name(%node_args);
        $options->{'nodescope-node-name'} = $node if (defined $node);

        push @{$opts{allowed_options}}, 'nodescope-node-name' if (defined $options->{'nodescope-node-name'});
    }

    $Log->exit() if $may_exit;
}



=head2 unfail

   NACL::C::StorageDisk->unfail(
                                command_interface => $ci,
                                 disk => $disk,
                                 spare => "true");
                                 or

    $StorageDisk_object->unfail(...);

(Class or instance method)

Uses a CMode CLI or 7Mode CLI APISet or CMode ZAPI APISet.
  Invokes 'disk unfail' command for 7Mode CLI.
  Invokes 'disk-unfail' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< disk => $disk >>

Applicable for CMode CLI, 7Mode CLI.
Maps to:
    7M CLI : disk list (Programmatic name: "disk-list")
    CM ZAPI:
        <disk-unfail>
            <disk>$value</disk>
        </disk-unfail>

=item C<< spare => $string >>

Applicable for CMode CLI, 7Mode CLI, CMode ZAPI.
  Maps to:
    7M CLI : -s (Programmatic name: 'make-spare')
    CM ZAPI:
       <disk-unfail>
           <make-spare>$value</make-spare>
       </disk-unfail>

=item C<< 'hidden-uid' => $string >>

  Applicable only for CMode CLI.

=item C<< 's' => $string >>

  Applicable only for CMode CLI.

=item C<< 'uid' => $string >>

  Applicable only for CMode CLI.

=item C<< 'quiet' => $string >>

Confirmations off
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"storage disk unfail" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.



=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI

=back

=back

=cut

sub unfail {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( spare
                    hidden-uid
                    s
                    uid
                    quiet
                    )
            ),
        },
        api => 'storage_disk_unfail',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_unfail_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_unfail_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope",
            },
            {   method    => "_unfail_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::StorageDisk::unfail',
    );
    $Log->exit() if $may_exit;
} ## end sub unfail

sub _unfail_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg) = shift;
    my %opts = $pkg->_validate_backend_opts(@_);
    $opts{s} = delete $opts{spare}
        if ($opts{spare});

    ## fix for burt 705435
    my %additional_args;
    my $prompt_answers
        = [
        'Are you sure you want to continue with disk unfail? {y|n}: y' => "y",
        ];
    $additional_args{'connectrec-match_table'} = $prompt_answers;

    $pkg->_base_backend_cmode_cli(
        %opts,
        api             => 'storage_disk_unfail',
        additional_args => \%additional_args
    );
    $Log->exit() if $may_exit;
} ## end sub _unfail_cmode_cli

sub _unfail_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $prompts_answers
        = [
        'Are you sure you want to continue with disk unfail? {y|n}: y' => "y",
        ];

    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%args,
        copy   => [qw(quiet)],
        map    => {
            "spare" => "make-spare",
            "disk"  => "disk-list",
        }
    );

    $opts{apiset}
        ->disk_unfail(%args, 'connectrec-match_table' => $prompts_answers);
    $Log->exit() if $may_exit;
} ## end sub _unfail_7mode_cli

sub _unfail_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_get_disk_name(options => \%opts);
    $pkg->_base_backend_cmode_zapi(
        api => 'disk-unfail',
        %opts,
        copy => [qw(disk)],
        map  => {spare => 'make-spare'}
    );
    $Log->exit() if $may_exit;
} ## end sub _unfail_cmode_zapi

=head2 remove

   NACL::C::StorageDisk->remove(
                                command_interface => $ci,
                                 disk => $disk,
                                 spare => "true");
                                 or

    $StorageDisk_object->remove(...);

(Class or instance method)

Uses a 7Mode CLI APISet or CMode CLI/ZAPI APISet.
  Invokes 'disk-remove' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Required) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< disk => $disk >>

Applicable for CMode CLI/ZAPI or 7Mode CLI.
Maps to:
   CM ZAPI:
    <disk-remove>
        <disk>$value</disk>
    </disk-remove>

=item C<< 'hidden-uid' => $string >>

Applicable for CMode CLI

=item C<< 'uid' => $string >>

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI

=back

=back

=cut

sub remove {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd => 'storage disk remove',
        ,
        additional_spec => {
            _optional_scalars(
                qw( hidden-uid

                    uid
                    )
            )
        },
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_remove_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_remove_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
            {   method    => "_remove_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::StorageDisk::remove',
    );
    $Log->exit() if $may_exit;
} ## end sub remove

sub _remove_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api  => 'storage_disk_remove');
    $Log->exit() if $may_exit;
} ## end sub _remove_cmode_cli

sub _remove_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $ci     = delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(disk)],
        target => \%args,
    );

    $apiset->disk_remove(%args);
    $Log->exit() if $may_exit;
} ## end sub _remove_7mode_cli

sub _remove_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_get_disk_name(options => \%opts);
    $pkg->_base_backend_cmode_zapi(%opts,
            api    => 'disk_remove',
            copy   => [qw( disk)],
	);
    $Log->exit() if $may_exit;
} ## end sub _remove_cmode_zapi

=head2 ddr_label_start_scan

   NACL::C::StorageDisk->ddr_label_start_scan(
                                command_interface => $ci,
                                 disks => [qw($disk1,$disk2)],
                                 );
                                 or

    $StorageDisk_object->ddr_label_start_scan(...);

(Class method)

Uses a 7Mode CLI APISet or Nodescope APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Required) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< disks => [qw($disk1,$disk2)] >>

  Applicable only for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub ddr_label_start_scan {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_common_validate_with(params => \@_,);
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_ddr_label_start_scan_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope",
            },
        ],
    );
    $Log->exit() if $may_exit;
} ## end sub ddr_label_start_scan

sub _ddr_label_start_scan_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $Log->debug("Enter to validate the commands");
    my %opts = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    my %args;

    $apiset->disk_ddr_label_start_scan();
    $Log->exit() if $may_exit;
} ## end sub _ddr_label_start_scan_7mode_cli

=head2 ddr_label_delete

   NACL::C::StorageDisk->ddr_label_delete(
                                command_interface => $ci,
                                 disks => [qw($disk1,$disk2)],
                                 );
                                 or

    $StorageDisk_object->ddr_label_delete(...);

(Class method)

Uses a 7Mode CLI APISet or Nodescope APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Required) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< disks => [qw($disk1,$disk2)] >>

  Applicable only for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub ddr_label_delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(params => \@args,);

    $pkg_or_obj->call_on_apiset_for_ddr(
        %opts,
        choices => [
            {   method    => "_ddr_label_delete_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope",
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub ddr_label_delete

sub _ddr_label_delete_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $ci     = delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    my %args;

    $pkg_or_obj->_api_call_log_debug_opts(
        api_opts => \%opts,
        api      => 'disk_ddr_label_delete'
    );

    $apiset->disk_ddr_label_delete(%opts);
    $Log->exit() if $may_exit;
} ## end sub _ddr_label_delete_7mode_cli

=head2 ddr_label_repair

   NACL::C::StorageDisk->ddr_label_repair(
                                command_interface => $ci,
                                 disks => [qw($disk1,$disk2)],
                                 );
                                 or

    $StorageDisk_object->ddr_label_repair(...);

(Class  method)

Uses a 7Mode CLI APISet or Nodescope APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Required) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< disks => [qw($disk1,$disk2)] >>

  Applicable only for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub ddr_label_repair {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(params => \@args,);
    $pkg_or_obj->call_on_apiset_for_ddr(
        %opts,
        choices => [
            {   method    => "_ddr_label_repair_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope",
            },
        ],
    );
    $Log->exit() if $may_exit;
} ## end sub ddr_label_repair

sub _ddr_label_repair_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $ci     = delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    my %args;

    $pkg_or_obj->_api_call_log_debug_opts(
        api_opts => \%opts,
        api      => 'disk_ddr_label_repair'
    );

    $apiset->disk_ddr_label_repair(%opts);
    $Log->exit() if $may_exit;
} ## end sub _ddr_label_repair_7mode_cli

=head2 ddr_label_print

    my $disk_ddr_obj = NACL::C::StorageDisk->ddr_label_print(
        command_interface => $ci,
        [ filter => { %filter } ,]
        [ requested_fields => [ @req_fields ]
    );

                    or

    my @disk_ddr_objs = NACL::C::StorageDisk->ddr_label_print(
        command_interface => $ci,
        [ filter => { %filter } ,]
        [ requested_fields => [ @req_fields ]
    );

                    or

   my $disk_ddr_obj = $Disk->ddr_label_print();

(Class or Instance method)

This method returns objects of the type L<NACL::CS::StorageDiskDdrLabelPrint|lib-NACL-CS-StorageDiskDdrLabelPrint-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch>
(requested_fields, filter, allow_empty) and has all of the other properties of
fetch methods (callable in scalar or list context, throws a NoElementsFound
exception if no matching elements are found)

=cut

sub ddr_label_print {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);

    require NACL::CS::StorageDiskDdrLabelPrint;
    my @state_objs = NACL::CS::StorageDiskDdrLabelPrint->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub ddr_label_print

=head2 get_raid_config_info

    my $raid_config_cs = $disk_obj->get_raid_config_info();

(Instance method only)

Gets raid-level details of the disk. It does by invoking
L<< NACL::CS::RaidConfigInfoListDisk->fetch|lib-NACL-CS-RaidConfigInfoListDisk-pm/fetch >>
and returns a C<NACL::CS::RaidConfigInfoListDisk>. The CS package internally
runs the 7Mode/Nodescope command "raid_config info listdisk" to get the
raid-level details of the disk.

=cut

sub get_raid_config_info {
    $Log->enter() if $may_enter;
    my ($self, @opts) = @_;
    my $cs = NACL::CS::RaidConfigInfoListDisk->fetch(
        command_interface => $self->command_interface(),
        filter            => {disk => $self->disk()},
        @opts
    );
    $Log->exit() if $may_exit;
    return $cs;
}

=head2 name_attribute

Returns 'disk'.

=cut

sub name_attribute {
    return 'disk';
}

=head2 get_ngsh_diskname

    my $disks_hashref = NACL::C::StorageDisk->get_ngsh_diskname(
        command_interface => $ci,
        disks             => [$disk1, $disk2, $disk3...]
    );

                    or

    my $disk_name = NACL::C::StorageDisk->get_ngsh_diskname(
        command_interface => $ci,
        disk              => $disk1,
    );

                    or

    my $disk_name = $disk_obj->get_ngsh_diskname();

(Class or Instance method)

This method accepts the disk names as in nodescope and returns disk names in ngsh
disk naming convention. This method returns the mapping disk names if found else will
return the same name provided.

If either 'disk' or 'disks' are not provided, the method returns all the disks
found on the system in the ngsh convention

=over

=item Options

=over

=item C<< disks => [qw($disk1,$disk2)] | $disk >>

(Applicable only for Class method)
Reference to an array of disk names ( as represented in nodescope ) to be mapped
to the ngsh names. The output returned will be a hash reference containing input
disk names (as represented in nodescope ) as keys and their corresponding disk
name ( as represented in ngsh) as the value.

   Ex: if input is, $disks = [ qw/d1 d2 d3 d4 .../ ]
       Returned data structure would be,
       { d1 => d1_ngsh_name, d2 => d2_ngsh_name, ... }

Note that disk path_name1 & path_name2 are mapped to same ngsh disk name.
So values of this map can contain duplicate entries.

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< disk => $disk >>

(Applicable only for Class method)
Scalar value representing a single disk name

=back

=back

=cut

sub get_ngsh_diskname {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    my $mapped_disks_names = $pkg_or_obj->_get_mapping_disknames(@args,
        'mode_requested' => "NGSH");
    $Log->exit() if $may_exit;
    return $mapped_disks_names;
}

=head2 get_nodescope_diskname

    my $disks_hashref = NACL::C::StorageDisk->get_nodescope_diskname(
        command_interface => $ci,
        disks             => [$disk1, $disk2, $disk3 ...]
    );

                    or

    my $disk_name = NACL::C::StorageDisk->get_nodescope_diskname(
        command_interface => $ci,
        disk              => $disk1,
    );

                    or

   my $disk_name = $disk_obj->get_nodescope_diskname();

(Class or Instance method)

This method accepts the disk names as in ngsh and return disk names as in nodescope
disk naming convention. This method returns the mapping disk names if found else will
return the same name provided.

If either 'disk' or 'disks' are not provided, the method returns all the disks
found on the system in the nodescope convention

=over

=item Options

=over

=item C<< disks => [qw($disk1,$disk2)] | $disk >>

(Applicable only for Class method)
Reference to an array of disk names ( as represented in ngsh ) to be mapped to
the nodescope names. The output returned will be a hash reference containing input
disk names (as represented in ngsh ) as keys and their corresponding disk
name ( as represented in nodescope) as the value.

   Ex: if input is, $disks = [ qw/d1 d2 d3 d4 .../ ]
       Returned data structure would be,
       { d1 => d1_nodescope_name, d2 => d2_nodescope_name, ... }

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< disk => $disk >>

(Applicable only for Class method)
Scalar value representing a single disk name

=back

=back

=cut

sub get_nodescope_diskname {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $mapped_disks_names = $pkg_or_obj->_get_mapping_disknames(@args,
        'mode_requested' => "NODESCOPE");
    $Log->exit() if $may_exit;
    return $mapped_disks_names;
}

# Common helper routine to get the mapping disk names in other context (NGSHS <=> Nodescope)

sub _get_mapping_disknames {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            disks                    => {type => ARRAYREF,  optional => 1},
            mode_requested           => {type => SCALAR},
            _input_has_invalid_disks => {type => SCALARREF, optional => 1},
            _optional_scalars(qw(disk)),
        },
    );

    my (%mapping_hash, $return_value_or_hashref, %already_mapped_diskname,
        $return_scalar, @disks);

    if ($opts{'disk'} && $opts{'disks'}) {
        NATE::Exceptions::Argument->throw(
            "Either 'disk' or 'disks' can be provided as arguments, but cannot provide both together"
        );
    }

    my $mode_requested = delete $opts{mode_requested};

    if ($mode_requested eq "NGSH" && $opts{'command_interface'}->is_7mode()) {
        NATE::BaseException->throw(
            "This method call is invalid on 7Mode System");

    }

    if ($opts{'disk'}) {
        $return_scalar = 1;
        @disks         = ($opts{'disk'});
    } else {
        @disks = @{$opts{'disks'}} if ($opts{'disks'});
    }

    my $input_has_invalid_disks = delete $opts{_input_has_invalid_disks};

    my @state_objs = $pkg_or_obj->_get_raid_config_info(%opts);

    my %mapping;
    if ($mode_requested eq "NGSH") {

        # Nodescope -> NGSH mapping
        foreach my $cs (@state_objs) {
            my $cluster_name = $cs->cluster_name();

          # Remove partition index if present (example: "1.0.4.P1" -> "1.0.4")
            $cluster_name =~ s/\.P\d+$//g;

            $mapping{$cs->path_name1()} = $cluster_name;
            my $secondary_name = $cs->path_name2();
            if ($secondary_name) {
                $mapping{$secondary_name} = $cluster_name;
            }
        }
    } else {

        # NGSH -> Nodescope mapping
        foreach my $cs (@state_objs) {
            my $path_name1 = $cs->path_name1();

            # Is this regex fine? should we use \w or \d in following?
            # will match 0a.00.4P1, 0a.00.4aP1, 0a.00.44P1
            $path_name1 =~ s/(\.\w+)P\d+$/$1/g;
            $mapping{$cs->cluster_name()} = $path_name1;
        }
    }

    # For mode=NGSH, the mapping contains two entries per disk: one for
    # primary name, the other for secondary name. If the user had provided
    # disk/disks, then copy out the subset instead of having both pairs of
    # mappings.
    if (@disks && $mode_requested eq "NGSH") {
        my %mapping_subset;
        foreach my $disk (@disks) {
            if (exists $mapping{$disk}) {
                $mapping_subset{$disk} = $mapping{$disk};
            } else {

               # Could not find the mapping, so just default mapping to itself
                $mapping_subset{$disk} = $disk;
                $$input_has_invalid_disks = 1;
            }
        }
        %mapping = %mapping_subset;
    }

    return $return_scalar ? $mapping{$disks[0]} : \%mapping;
}

sub _get_raid_config_info {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $disks = $opts{disks};
    my $ci    = $opts{'command_interface'};
    my %filter;
    if ($disks && @$disks) {
        $filter{disk} = join '|', @$disks;
    }

    $Log->comment("Logs for cmd 'raid_config info listdisk' are suppressed."
            . " Logs will be available when LOGSPEC is set to debug.");

    my @cs;
    {
        # Suppressing the NATE log for 'raid_config info listdisk' command.
        local $NATE::Log::CommEvent::CommLogLevel = 'debug';
        @cs = NACL::CS::RaidConfigInfoListDisk->fetch(
            command_interface => $ci,
            filter            => \%filter,
            allow_empty       => 1,
        );
    }
    $Log->exit() if $may_exit;
    return @cs;
}

###########################################################################
# Description : This is the version of call_on_apiset() which will manipulate the disk name on the basis of mode and
#                    call SUPER::call_on_apiset
###########################################################################
sub call_on_apiset {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $hash
        = $pkg_or_obj->SUPER::call_on_apiset(%opts, do_not_call_backend => 1);
    my $pkg = ref($pkg_or_obj) || $pkg_or_obj;
    my $apiset = $hash->{apiset};

    # Convert the disk to appropriate type as per CMode or 7Mode
    if (defined $opts{disk}) {
        if ($apiset->{set} eq "CMode") {
            $opts{disk} = $pkg->get_ngsh_diskname(
                'command_interface' => $opts{command_interface},
                'disk'              => $opts{disk},
                apiset_should       => {interface => $apiset->{interface}}
            );
        } else {
            $opts{disk} = $pkg->get_nodescope_diskname(
                'command_interface' => $opts{command_interface},
                'disk'              => $opts{disk},
                apiset_should       => {interface => $apiset->{interface}}
            );
        }
    }

    my @ret = $pkg_or_obj->SUPER::call_on_apiset(%opts,
        apiset_backend_already_chosen => $hash);
    $Log->exit() if $may_exit;
    return wantarray ? @ret : $ret[0];
}

###########################################################################
# Description : This is the version of call_on_apiset() which will manipulate the list of disk names on the basis of
#                   mode and call SUPER::call_on_apiset
###########################################################################
sub call_on_apiset_for_ddr {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $hash
        = $pkg_or_obj->SUPER::call_on_apiset(%opts, do_not_call_backend => 1);
    my $pkg = ref($pkg_or_obj) || $pkg_or_obj;
    my $apiset = $hash->{apiset};

    # Convert the disk to appropriate type as per CMode or 7Mode
    if (defined($opts{'disk'})) {
        my @temp = split(/\s+/, $opts{'disk'});
        if ($apiset->{set} eq "CMode") {
            @temp = map { (split(/:/))[-1] } @temp;

            $opts{disk} = $pkg->get_ngsh_diskname(
                'command_interface' => $opts{command_interface},
                'disks'             => \@temp,
                apiset_should       => {interface => $apiset->{interface}}
            );
        } else {
            $opts{disk} = $pkg->get_nodescope_diskname(
                'command_interface' => $opts{command_interface},
                'disks'             => \@temp,
                apiset_should       => {interface => $apiset->{interface}}
            );
        }

        $opts{'disks'} = join(' ', values %{$opts{'disk'}});
    } ## end if ( defined( $opts{'disk'...}))
    delete $opts{disk};
    my @ret = $pkg_or_obj->SUPER::call_on_apiset(%opts,
        apiset_backend_already_chosen => $hash);
    $Log->exit() if $may_exit;
    return wantarray ? @ret : $ret[0];
}

###########################################################################
# Description : This is the version of call_on_apiset() which will manipulate the list of disk names on the basis of
#                   mode and call SUPER::call_on_apiset
###########################################################################
sub call_on_apiset_for_updatefirmware {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $hash
        = $pkg_or_obj->SUPER::call_on_apiset(%opts, do_not_call_backend => 1);
    my $pkg = ref($pkg_or_obj) || $pkg_or_obj;
    my $apiset = $hash->{apiset};
    my $converted_disks;

    # Convert the disk to appropriate type as per CMode or 7Mode
    if (defined $opts{disk}) {
        if ($apiset->{set} eq "CMode") {
            $converted_disks = $pkg->get_ngsh_diskname(
                'command_interface' => $opts{command_interface},
                'disks'             => $opts{disk},
                apiset_should       => {interface => $apiset->{interface}}
            );
        } else {
            $converted_disks = $pkg->get_nodescope_diskname(
                'command_interface' => $opts{command_interface},
                'disks'             => $opts{disk},
                apiset_should       => {interface => $apiset->{interface}}
            );
        }
        $opts{disk} = [values %$converted_disks];
    }

    delete $opts{disks};
    my @ret = $pkg_or_obj->SUPER::call_on_apiset(%opts,
        apiset_backend_already_chosen => $hash);
    $Log->exit() if $may_exit;
    return wantarray ? @ret : $ret[0];
}

##################################################################
# Determine the disk name and owner and update the "options" hash-reference
##################################################################
sub _get_owner_disk_name {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {options => {type => HASHREF}},
    );
    my $options = $opts{options};
    my $disk_cs;
    my $hash = $pkg_or_obj->_get_owner_disk_name_from_regex(
        disk => $options->{disk});
    if (exists $hash->{disk}) {

        # Matched the regex
        $options->{disk} = $hash->{disk};
        $options->{owner} ||= $hash->{owner};
    } else {

        # Did not match the regex, need to query the filer
        # Allow for negative testing by providing allow_empty: invalid
        # disk names result in the disk value being left untouched
        if (!(defined $options->{owner})) {
            $disk_cs = NACL::CS::StorageDisk->fetch(
                command_interface => $opts{command_interface},
                apiset_should     => {interface => 'ZAPI'},
                filter            => {disk => $options->{disk}},
                requested_fields  => ['owner'],
                allow_empty       => 1
            );
        }

        # Could be undefined if an invalid disk name is provided
        $options->{owner} = $disk_cs->owner() if (defined $disk_cs);
    }
    $Log->exit() if $may_exit;
}

##################################################################
# Determine the disk name and update the "options" hash-reference
##################################################################
sub _get_disk_name {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {options => {type => HASHREF}},
    );
    my $options = $opts{options};
    my $hash    = $pkg_or_obj->_get_owner_disk_name_from_regex(
        disk => $options->{disk});
    if (exists $hash->{disk}) {
        $options->{disk} = $hash->{disk};
    }
    $Log->exit() if $may_exit;
}

##################################################################
# If the regex is in the form <owner>:<disk> return a hash-reference
# with the keys being "owner" and "disk", else returns an empty hash-reference
##################################################################
sub _get_owner_disk_name_from_regex {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {disk => {type => SCALAR}}
    );
    if ($opts{disk} =~ /((.*):)?(\S+)/) {
        $Log->exit() if $may_exit;
        return {owner => $2, disk => $3};
    } else {
        $Log->exit() if $may_exit;
        return {};
    }
}

=head2 FIND WRAPPERS
Each of the canned filters have a wrapper method (implemented through
AUTOLOAD) of the form:

find_<canned-filter-name>

For example, there is a C<find_spare> method for the C<root>
canned filter.

NACL::C::StorageDisk->find_spare(...);

These methods accept all of the arguments normally accepted by find()
but automatically apply the canned filter contained in its name.
In addition to the wrapper methods created for the canned filters, the
following find wrappers are also available:

=head3 find_spare

# In list context returns all spare disks with
my @disks = NACL::C::StorageDisk->find_spare(
    command_interface => $node,
    %other_opts
);
# In scalar context returns single spare disk
my $disks = NACL::C::StorageDisk->find_spare(
           command_interface => $node,
           %other_opts
);

Returns Stoarage Disks  which are spare.

This accepts all the options accepted by find().

Method can helps in creating find wrapper method.
=cut
sub can {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my @opts = $pkg_or_obj->_can_validation(@args);
    my $code;
    if ($code = $pkg_or_obj->SUPER::can(@opts)) {
        $Log->exit() if $may_exit;
        return $code;
    } else {
        $code = $pkg_or_obj->_find_wrapper_generator(@opts);
        if ($code) {
            $Log->exit() if $may_exit;
            return $code;
        }
    }
    $Log->exit() if $may_exit;
    return undef;
} ## end sub can

1;
