#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageFailover Component Module
## @author Chethan.M@netapp.com@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StorageFailover

=head1 DESCRIPTION

C<NACL::C::StorageFailover> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP StorageFailover.  A related class is
L<NACL::CS::StorageFailover|lib-NACL-CS-StorageFailover-pm>, which represents
the past state of an ONTAP StorageFailover.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 node

The name of the node, used to identify it uniquely.  (Node names are assumed
to be unique within a node or within a cluster)

=cut

package NACL::C::StorageFailover;

use strict;
use warnings;

use NACL::CS::StorageFailover;
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use NACL::Exceptions::InvalidChoice;
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR SCALARREF HASHREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::Exceptions::StorageFailover::DiskInventoryNotReceived;
use NACL::C::Exceptions::StorageFailover::UnableToReadPartnerState;
use NACL::C::Exceptions::StorageFailover::NonHAMode;
use NACL::C::Exceptions::StorageFailover::SkipLifMigrationBeforeTakeover;
use NACL::C::Exceptions::StorageFailover::PartnerNotFullyBooted;
use NACL::C::Exceptions::StorageFailover::GivebackAlreadyInProgress;
use NATE::BaseException qw(:try);

# For Backward compatibility BURT #497820 .
use Class::MethodMaker [scalar => 'node',];

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::C::Failover'
    );
    $Log->exit() if $may_exit;
    return $isa;
} ## end sub isa

=head1 METHODS

=head2 modify

  NACL::C::StorageFailover->modify(command_interface=>$ci,
                            node=>$node_name,...);

(Class method)This method modifies a node.

Uses a CMode CLI/ZAPI, 7M CLI APIset.

Invokes "cf-service-enable" API for CMode ZAPI if $enable option passed in
is true.
"cf-service-disable"API for CMode ZAPI if $enable option passed in is false.

Invokes "options,cf enable,cf disable" commands for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=> $ci >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should = $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< node=>$string >>

(Optional) Node
  This specifies the node whose storage-failover options are to be
  modified.

Applicable for CMode CLI

=item C<< node-name=>$string >>

(Required for class method, Not Applicable for instance method)
(Required) Node Name As Returned by Storage StorageFailover Component

Applicable for CMode CLI

=item C<< partner-name=>$string >>

(Optional) Partner Name

Applicable for CMode CLI

=item C<< nvramid=>$number >>

(Optional) Node NVRAM ID

Applicable for CMode CLI

=item C<< partner-nvramid=>$number >>

(Optional) Partner NVRAM ID

Applicable for CMode CLI

=item C<< enabled=>$boolean >>

(Optional) Service Enabled

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI: enabled

=item C<< possible=>$boolean >>

(Optional) Takeover Possible

Applicable for CMode CLI

=item C<< reason=>$string >>

(Optional) Reason Takeover Not Possible

Applicable for CMode CLI

=item C<< reason-enum=>$string >>

(Optional) Reason Takeover Not Possible (Enum value)

Applicable for CMode CLI

=item C<< interconnect-up=>$boolean >>

(Optional) Interconnect Up

Applicable for CMode CLI

=item C<< state=>$string >>

(Optional) State

Applicable for CMode CLI

=item C<< time-until-takeover=>$number >>

(Optional) Time Until Takeover

Applicable for CMode CLI

=item C<< partner-reason=>$string >>

(Optional) Reason Takeover Not Possible By Partner

Applicable for CMode CLI

=item C<< partner-reason-enum=>$string >>

(Optional) Reason Takeover Not Possible By Partner (Enum value)

Applicable for CMode CLI

=item C<< killpackets=>$boolean >>

(Optional) Issue Kill Packets

Applicable for CMode CLI

=item C<< partner-aggregates=>$boolean >>

(Optional) Control Partner Aggregates

Applicable for CMode CLI

=item C<< current-index=>$number >>

(Optional) Current Progress Index

Applicable for CMode CLI

=item C<< current-entry=>$string >>

(Optional) Current Progress Entry

Applicable for CMode CLI

=item C<< maximum-index=>$number >>

(Optional) Maximum Progress Index

Applicable for CMode CLI

=item C<< pmbox-status=>$string >>

(Optional) Primary Mailbox Status

Applicable for CMode CLI

=item C<< bmbox-status=>$string >>

(Optional) Backup Mailbox Status

Applicable for CMode CLI

=item C<< fast-timeout=>$number >>

(Optional) Fast Timeout

Applicable for CMode CLI

=item C<< slow-timeout=>$number >>

(Optional) Slow Timeout

Applicable for CMode CLI

=item C<< mailbox-timeout=>$number >>

(Optional) Mailbox Timeout

=item C<< connect-timeout=>$number >>

(Optional) Connect Timeout

=item C<< operator-timeout=>$number >>

(Optional) Operator Timeout

=item C<< firmware-timeout=>$number >>

(Optional) Firmware Timeout

=item C<< dumpcore-timeout=>$number >>

(Optional) Dumpcore Timeout

=item C<< booting-timeout=>$number >>

(Optional) Booting Timeout

=item C<< transit-timer=>$boolean >>

(Optional) Transit Timer Enabled

Applicable for CMode CLI

=item C<< transit-timeout=>$number >>

(Optional) Transit Timeout

=item C<< firmware-received=>$number >>

(Optional) Firmware Received

Applicable for CMode CLI

=item C<< booting-received=>$number >>

(Optional) Booting Received

Applicable for CMode CLI

=item C<< transit-time=>$number >>

(Optional) Transit Event Time

Applicable for CMode CLI

=item C<< pnormal=>$number >>

(Optional) Primary Normal IO Time

Applicable for CMode CLI

=item C<< ptransition=>$number >>

(Optional) Primary Transition IO Time

Applicable for CMode CLI

=item C<< bnormal=>$number >>

(Optional) Backup Normal IO Time

Applicable for CMode CLI

=item C<< btransition=>$number >>

(Optional) Backup Transition IO Time

Applicable for CMode CLI

=item C<< logs-unsynced=>$number >>

(Optional) Logs Unsynced Count

Applicable for CMode CLI

=item C<< auto-giveback=>$boolean >>

(Optional) Auto Giveback Enabled

Applicable for CMode CLI

=item C<< abort-operations=>$boolean >>

(Optional) Auto-Abort Operations Enabled

Applicable for CMode CLI

=item C<< check-partner=>$boolean >>

(Optional) Check Partner Enabled

Applicable for CMode CLI

=item C<< detection-time=>$number >>

(Optional) Takeover Detection Time (secs)
  This optionally specifies the amount of time, in seconds, that a
  node remains unresponsive before its partner initiates a takeover
  operation. Possible values range from 10 to 180 seconds. The default
  setting is 15 seconds.

=item C<< onfailure=>$boolean >>

(Optional) Takeover On Failure Enabled

Applicable for CMode CLI

=item C<< onpanic=>$boolean >>

(Optional) Takeover On Panic Enabled

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : Takeover On Panic Enabled (Programmatic name: "cf.takeover.on_panic")

=item C<< onshort-uptime=>$boolean >>

(Optional) Takeover On Short Uptime Enabled

Applicable for CMode CLI

=item C<< auto-giveback-after-panic=>$boolean >>

(Optional) Auto Giveback After Panic Enabled

Applicable only for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : Auto Giveback After Panic Enabled
(Programmatic name: "cf.giveback.auto.after.panic.takeover")

=item C<< short-uptime=>$number >>

(Optional) Short Uptime (secs)
  This optionally specifies the time period used by the -onshort-
  uptime parameter. The default setting is 60 seconds. This parameter is
  available only at the advanced privilege level and higher.

Applicable for CMode CLI

=item C<< attempts=>$number >>

(Optional) Number of Giveback Attempts
  This optionally specifies the number of times the node attempts an
  automatic giveback operation within 10 minutes; the time period is
  modifiable by using the -attempts-time parameter. The default setting
  is 3 attempts. This parameter is available only at the advanced privi-
  lege level and higher.

Applicable for CMode CLI

=item C<< attempts-time=>$number >>

(Optional) Giveback Attempts Minutes
  This optionally specifies the time period used by the -attempts
  parameter. The default setting is 10 minutes. This parameter is avail-
  able only at the advanced privilege level and higher.

Applicable for CMode CLI

=item C<< propagate=>$boolean >>

(Optional) Propagate Status Via Mailbox

Applicable for CMode CLI

=item C<< read-interval=>$number >>

(Optional) Node Status Read Interval (secs)
  This optionally specifies, in seconds, how frequently the node
  reads its partner node's status from the mailbox disks. The default
  setting is 5 seconds. This parameter is available only at the advanced
  privilege level and higher.

Applicable for CMode CLI

=item C<< write-interval=>$number >>

(Optional) Node Status Write Interval (secs)
  This optionally specifies, in seconds, how frequently the node
  writes its status to the mailbox disks. The default setting is 5 sec-
  onds. This parameter is available only at the advanced privilege level
  and higher.

Applicable for CMode CLI

=item C<< onreboot=>$boolean >>

(Optional) Takeover On Reboot Enabled

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : Takeover On Reboot Enabled (Programmatic name: "cf.takeover.on_reboot")

=item C<< 'hwassist' => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : hw assist Enabled (Programmatic name: "cf.hw_assist.enable")

=item C<< 'hwassist-partner-port' => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : hw assist partner port Enabled
(Programmatic name: "cf.hw_assist.partner.port")

=item C<< 'hwassist-health-check-interval' => $string >>

Applicable for CMode CLI

=item C<< 'hwassist-partner-ip' => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : hw assist partner address Enabled
(Programmatic name: "cf.hw_assist.partner.address")

=item C<< 'cancel-auto-giveback-network-failure' => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : giveback auto cancel on network failure Enabled
(Programmatic name: "cf.giveback.auto.cancel.on_network_failure")

=item C<< 'hwassist-retry-count' => $string >>

Applicable for CMode CLI

=item C<< 'auto-giveback-after-panic' => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : giveback auto after panic takeover Enabled
(Programmatic name: "cf.giveback.auto.after.panic.takeover")

=item C<< 'mode' => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : mode Enabled
(Programmatic name: "cf.mode")

=item C<< 'current-mode' => $string >>

Applicable for CMode CLI

=item C<< 'bypass-optimization' => $string >>

Applicable for CMode CLI

=item C<< 'bypass-takeover-optimization' => $string >>

Bypass Takeover Optimization Enabled, possible value(s) are true,false

Applicable for CMode CLI

=item C<< 'aggregate-migration-timeout' => $string >>

*Aggregate Migration Timeout (secs), possible value(s) are 30..120

Applicable for CMode CLI

=item C<< 'auto-giveback-override-vetoes' => $string >>

Auto-giveback Override Vetoes Enabled, possible value(s) are true,false

Applicable for CMode CLI

=item C<< 'auto-enable-cluster-ha' => $string >>

Applicable for CMode CLI

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< 'auto-giveback-cifs-terminate-minutes' => $string >>


Auto Giveback Delay Before Terminating CIFS (minutes) ranges from 0 to 999
Applicable for CMode CLI

=item C<< 'cifs-ndo-duration' => $string >>

CIFS NDO Duration possible value(s) are, default,medium,low
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"storage failover modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::StorageFailover::NonHAMode>

This exception is thrown when disabling storage failover service fails
as the cluster is in a Non-HA mode

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_common_validate_with(
        params             => \@args,
        only_static_method => 1,
        additional_spec    => {
            node => {type => SCALAR},
            _optional_scalars(
                qw (enabled auto-giveback
                    abort-operations check-partner detection-time
                    onfailure onpanic onshort-uptime short-uptime
                    attempts attempts-time propagate read-interval
                    write-interval onreboot delay-seconds auto-sendhome
                    hwassist hwassist-partner-port hwassist-health-check-interval
                    hwassist-partner-ip cancel-auto-giveback-network-failure
                    hwassist-retry-count
                    auto-giveback-after-panic mode

                    current-mode bypass-optimization

                    bypass-takeover-optimization

                    aggregate-migration-timeout

                    auto-giveback-override-vetoes

                    auto-enable-cluster-ha
                    auto-giveback-after-panic

                    auto-giveback-cifs-terminate-minutes

                    cifs-ndo-duration
                    )
            )
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'storage failover modify',
    );

    my $ret;
    try {

        $ret = $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_modify_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_modify_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                    check     => '_check_modify_opts',
                },
                {   method    => '_modify_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
            ],
            frontend => 'NACL::C::StorageFailover::modify',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::StorageFailover::NonHAMode
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;

    my %additional_args;

    # BURT#478756 to handle prompts in BR.2
    my $match_prompt = ['disable auto-giveback? {y|n}' => 'y'];

    $additional_args{'connectrec-match_table'} = $match_prompt;

    $pkg->_base_backend_cmode_cli(
        @opts,
        api             => 'storage_failover_modify',
        additional_args => \%additional_args
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $enable = delete $opts{enabled};
    my $apiset = delete $opts{apiset};

    if ($enable =~ /true/) {
        $apiset->cf_service_enable(%opts);
    } elsif ($enable =~ /false/i) {
        $apiset->cf_service_disable(%opts);
    } else {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Invalid value '$enable' for option 'enabled'.");
    }

    $Log->exit() if $may_exit;

} ## end sub _modify_cmode_zapi

sub _modify_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %api_opts;

    if (defined($opts{enabled})) {
        if ($opts{enabled} eq "true") {
            $apiset->cf_enable();
        }
        if ($opts{enabled} eq "false") {
            $apiset->cf_disable();
        }
    } ## end if ( defined( $opts{enabled...

    $pkg->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        map    => {
            'auto-giveback'         => 'cf.giveback.auto.enable',
            'hwassist'              => 'cf.hw_assist.enable',
            'hwassist-partner-ip'   => 'cf.hw_assist.partner.address',
            'hwassist-partner-port' => 'cf.hw_assist.partner.port',
            'onpanic'               => 'cf.takeover.on_panic',
            'onreboot'              => 'cf.takeover.on_reboot',
            'cancel-auto-giveback-network-failure' =>
                'cf.giveback.auto.cancel.on_network_failure',
            'check-partner'  => 'cf.giveback.check.partner',
            'detection-time' => 'cf.takeover.detection.seconds',
            'delay-seconds'  => 'cf.giveback.auto.delay.seconds',
            'mode'           => 'cf.mode',
            'auto-giveback-after-panic' =>
                'cf.giveback.auto.after.panic.takeover',
        },
    );

    foreach my $key (keys(%api_opts)) {
        if ($api_opts{$key} eq "true") {
            $api_opts{$key} = "on";
        }
        if ($api_opts{$key} eq "false") {
            $api_opts{$key} = "off";
        }
    } ## end foreach my $key ( keys(%api_opts...
    foreach my $keys (keys(%api_opts)) {

        $apiset->options($keys => $api_opts{$keys});
    } ## end foreach my $keys ( keys(%api_opts...
    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_cli

=head2 takeover

  NACL::C::StorageFailover->takeover(command_interface=>,
                              bynode=>$node_name,...);

(Class method)This method initiates a takeover of the partner
node's storage.

Uses a CMode CLI/ZAPI, 7Mode CLI/ZAPI APIset.

Invokes "cf-takeover" API for CMode ZAPI,7Mode ZAPI.

Invokes "cf takeover" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=> $ci >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should = $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< bynode=>$string >>

(Required for class method, Not Applicable for instance method)
(Required) This specifies the node that is to take over its partner's storage.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node

=item C<< ofnode=>$string >>

(Required for class method, Not Applicable for instance method)
(Required) This specifies the node that is to be taken over.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node

=item C<< option=>$string >>

(Optional) Takeover Option
Possible values include the following:

normal: Specifies a normal takeover operation; that is, the partner is given the
time to close its storage resources gracefully before the takeover operation
proceeds. This is the default value.

immediate: Specifies an immediate takeover. In an immediate takeover, the
takeover operation is initiated before the partner is given the time to close
its storage resources gracefully.

allow-version-mismatch: If this value is specified, the takeover operation is
initiated even if the partner is running a version of software that is
incompatible with the version running on the node. In this case, the partner
node must be cleanly halted. Use this value as part of a nondisruptive upgrade
procedure.

force: If this value is specified, the takeover operation is initiated even
if the node detects an error that normally prevents a takeover operation from
occurring. This value is available only at the advanced privilege level and
higher.

Note:         This option can potentially result in data loss. If the
              storage-failover interconnection is detached or inactive, the
			  contents of the failover partners' NVRAM cards are
			  unsynchronized, normally disabling takeover. Using the -force
			  option enables a node to take over its partner's storage despite
			  the unsynchronized NVRAM, which can contain client data that
              can be lost upon storage takeover.

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI: option

7M CLI : -f (Programmatic name: "option")

7M ZAPI: option

=item C<< 'bypass-optimization' => $string >>

Applicable for CMode CLI

=item C<< 'allow-disk-inventory-mismatch' => $string >>

Disk inventory, possible value(s) are true,false

Applicable for CMode CLI

=item C<< 'skip-lif-migration' => $string >>

Skip LIF Migration

Applicable for CMode CLI

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< 'skip-lif-migration-beforehand' => $string >>

Skip Migrating LIFs Away from Node Prior to Takeover
Applicable for CMode CLI

=item C<< 'skip-lif-migration-before-takeover' => $string >>

Skip Migrating LIFs Away from Node Prior to Takeover
Applicable for CMode CLI

=item C<< 'ignore-quorum-warnings' => $string >>

Note that if this option is not provided, then it will be defaulted
to "true" (see burt 867673). This can be disabled by providing the
value as undef ('ignore-quorum-warnings' => undef)

Ignore warnings
Applicable for CMode CLI/ZAPI.

=item C<< '*' >>

This method takes all other options supported by
"storage failover takeover" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::StorageFailover::SkipLifMigrationBeforeTakeover>

This exception is thrown when storage failover takeover fails
due to LIF migration.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub takeover {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params             => \@args,
        only_static_method => 1,
        additional_spec    => {
            "ignore-quorum-warnings" => {type => SCALAR|UNDEF, optional => 1},
            _optional_scalars(
                qw(bynode ofnode option
                    bypass-optimization

                    allow-disk-inventory-mismatch skip-lif-migration

                    skip-lif-migration-beforehand

                    skip-lif-migration-before-takeover
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd => 'storage_failover_takeover',
    );
    $pkg->_handle_ignore_quorum_warnings(\%opts);

    my $ret;
    try {
        $ret = $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_takeover_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_takeover_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
                {   method    => '_takeover_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
                {   method    => '_takeover_7mode_zapi',
                    interface => 'ZAPI',
                    set       => '7Mode',
                }
            ],
            frontend => 'NACL::C::StorageFailover::takeover',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::StorageFailover::SkipLifMigrationBeforeTakeover
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;

} ## end sub takeover

sub _takeover_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'storage_failover_takeover',);
    $Log->exit() if $may_exit;
} ## end sub _takeover_cmode_cli

sub _takeover_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $option = delete $opts{option};

    if ($option =~ /immediate/) {
        $opts{additional_args} = {immediate => 'true'};
    } elsif ($option =~ /allow-version-mismatch/) {
        $opts{additional_args} = {'version-mismatch-ok' => 'true'};
    } elsif ($option =~ /force/) {
        $opts{additional_args} = {force => 'true'};
    }

    my $allowed_cmode_options = [
        qw(option bypass-optimization
            allow-disk-inventory-mismatch skip-lif-migration
            ignore-quorum-warnings
            )
    ];
    my $map = {
        'bynode' => 'node',
        'ofnode' => 'node',
    };

    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'cf-takeover',
        copy => $allowed_cmode_options,
        map  => $map,
    );

    $Log->exit() if $may_exit;
} ## end sub _takeover_cmode_zapi

# Automatically set ignore-quorum-warnings => true to prevent script
# failures. See burt 867673.
sub _handle_ignore_quorum_warnings {
    my ($pkg_or_obj, $opts_ref) = @_;
    if (exists $opts_ref->{'ignore-quorum-warnings'}) {
        # Value of undef means "do not send down the option"
        if (!defined $opts_ref->{'ignore-quorum-warnings'}) {
            delete $opts_ref->{'ignore-quorum-warnings'};
        }
    } elsif ($opts_ref->{command_interface}->version_manager()->is_version_ge(release => '8.3.1')) {
        $opts_ref->{'ignore-quorum-warnings'} = 'true';
    }
}

sub _takeover_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [qw(option)];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%cmode_args,
    );

    my $option = delete $cmode_args{option};
    if ($option =~ /allow-version-mismatch/) {
        $cmode_args{'version-mismatch'} = 'true';
    } elsif ($option =~ /force/i) {
        $cmode_args{force} = 'true';
    }
    %opts = %cmode_args;

    $pkg->_handle_boolean_options(
        api_options     => \%opts,
        boolean_options => [qw(force version-mismatch)],
        is_7m_cli       => 1
    );

    $apiset->cf_takeover(%opts);
    $Log->exit() if $may_exit;
} ## end sub _takeover_7mode_cli

sub _takeover_7mode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [qw(option)];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%cmode_args,
    );

    my $option = delete $cmode_args{option};
    if ($option =~ /force/i) {
        $cmode_args{force} = 'true';
    } else {
        $cmode_args{force} = 'false';
    }

    $apiset->cf_takeover(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _takeover_7mode_zapi

=head2 giveback

  NACL::C::StorageFailover->giveback(command_interface=>$ci,
                              fromnode=>$node_name,...);

(Class method)This method returns storage that has failed over
to a node's partner back to the home node. This operation fails if other
resource-intensive operations (for instance, system dumps) are running and
make the giveback operation potentially dangerous or disruptive.

Uses a CMode CLI/ZAPI, 7Mode CLI/ZAPI APIset.

Invokes "cf-giveback" API for CMode ZAPI,7Mode ZAPI.

Invokes "cf giveback" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=> $ci >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should = $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< fromnode=>$string >>

(Required for class method, Not Applicable for instance method)
(Required) Node
  This specifies the node that currently holds the storage that is to be
  returned to the partner node.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node

=item C<< require-partner-waiting=>$boolean >>

(Optional) Require Partner in Waiting

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: require-partner-waiting

=item C<< override-vetoes=>$boolean >>

(Optional) Override All Vetos

Applicable for CMode CLI/ZAPI, 7Mode CLI/ZAPI.

Maps to:

CM ZAPI: override-vetoes

7M CLI : -f (Programmatic name: "force")

7M ZAPI: force

=item C<< only-cfo-aggregates=>$boolean >>

(Optional) Giveback Only CFO Aggregates

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: only-cfo-aggregates

=item C<< ofnode=>$string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"storage failover giveback" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::StorageFailover::UnableToReadPartnerState>

This exception is thrown when giveback fails as it is unable to read partner state.

=item C<NACL::C::Exceptions::StorageFailover::DiskInventoryNotReceived>

This exception is thrown when giveback fails as disk inventory information is not received.

=back

=back

=cut

sub giveback {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params             => \@args,
        only_static_method => 1,
        additional_spec    => {
            _optional_scalars(
                qw(ofnode fromnode require-partner-waiting override-vetoes only-cfo-aggregates)
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'storage failover giveback',
    );

    my $ret;
    try {
        $ret = $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_giveback_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_giveback_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
                {   method    => '_giveback_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
                {   method    => '_giveback_7mode_zapi',
                    interface => 'ZAPI',
                    set       => '7Mode',
                }
            ],
            frontend => 'NACL::C::StorageFailover::giveback',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        foreach my $type (qw(UnableToReadPartnerState DiskInventoryNotReceived PartnerNotFullyBooted GivebackAlreadyInProgress)) {
            my $exception_name = "NACL::C::Exceptions::StorageFailover::$type";
            $exception_name->detect_convert_and_throw(exception => $exception);
        }
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;

} ## end sub giveback

sub _giveback_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'storage_failover_giveback',);
    $Log->exit() if $may_exit;
} ## end sub _giveback_cmode_cli

sub _giveback_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $ci     = delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options
        = [
        qw( require-partner-waiting override-vetoes only-cfo-aggregates  extended_query )
        ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        map    => {"fromnode" => "node",},
        target => \%cmode_args,
    );

    if (!defined $cmode_args{node}) {
        $cmode_args{node} = $ci->node;
    }
    %opts = %cmode_args;

    $pkg->_handle_boolean_options(
        api_options     => \%opts,
        boolean_options => [qw(force)],
        is_7m_cli       => 1
    );

    $apiset->cf_giveback(%opts);
    $Log->exit() if $may_exit;

} ## end sub _giveback_cmode_zapi

sub _giveback_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [qw( force )];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => {'override-vetoes' => 'force'},
        target => \%cmode_args,
    );

    %opts = %cmode_args;

    $apiset->cf_giveback(%opts);
    $Log->exit() if $may_exit;
} ## end sub _giveback_7mode_cli

sub _giveback_7mode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [qw( force )];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => {'override-vetoes' => 'force'},
        target => \%cmode_args,
    );

    $apiset->cf_giveback(%cmode_args);
    $Log->exit() if $may_exit;

} ## end sub _giveback_7mode_zapi

=head2 cf_checkstate

   my $obj = NACL::C::StorageFailover->cf_checkstate(
                                command_interface => $ci,
                                  ...);

   $cf_checkstate = $obj->cf_state();

   (or)

   my @obj = NACL::C::StorageFailover->cf_checkstate(command_interface => $ci);
   $cf_checkstate = $obj[0]->cf_state();

(Class method)
This method is used to display the current state of the filer

Uses a 7Mode CLI APIset.

Returns object(s) of type L<NACL::CS::CfCheckstate|lib-NACL-CS-CfCheckstate-pm>

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< expectedstate=>$string >>

(Required)

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub cf_checkstate {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params              => \@_,
        ignore_primary_keys => 1,
        additional_spec     => {_optional_scalars(qw(expectedstate))}
    );
    my @CS_objs = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_cf_checkstate_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );
    $Log->exit() if $may_exit;
    return wantarray ? @CS_objs : $CS_objs[0];
} ## end sub cf_checkstate

sub _cf_checkstate_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my %args;
    my $cf_response;

    if (defined($opts{expectedstate})) {
        $args{'expected-state'} = $opts{expectedstate};
    }

    $cf_response = $apiset->cf_checkstate(%args);

    my $cf_output = $cf_response->get_parsed_output();
    require NACL::CS::CfCheckstate;
    my @state_objs;
    my %state_fields;
    $state_fields{'expectedstate'} = $cf_output->[0]->{'expectedstate'};
    $state_fields{'cf_state'}      = $cf_output->[0]->{'cf_state'};

    $state_fields{'command_interface'} = $opts{'command_interface'};

    my $obj = NACL::CS::CfCheckstate->new(
        command_interface => $command_interface);

    $obj->_set_fields(row => \%state_fields);

    push @state_objs, $obj;

    $Log->exit() if $may_exit;

    return wantarray ? @state_objs : $state_objs[0];
} ## end sub _cf_checkstate_7mode_cli

# For Backward Compatibility #497820.

=head2 show_giveback

    my $method_obj  = NACL::C::StorageFailover->show_giveback(command_interface => $ci,
                          [ filter => { %filter } ,]
						  [ requested_fields => [ @req_fields ]);

                    or

    my $method_objs = NACL::C::StorageFailover->show_giveback(command_interface => $ci,
                          [ filter => { %filter } ,]
						  [ requested_fields => [ @req_fields ]);

(Class method)
This method returns objects of the type L<NACL::CS::StorageFailoverShowGiveback|lib-NACL-CS-StorageFailoverShowGiveback-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch>
(requested_fields, filter, allow_empty) and has all of the other properties of
fetch methods (callable in scalar or list context, throws a NoElementsFound
exception if no matching elements are found)

=cut

sub show_giveback {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    require NACL::CS::StorageFailoverShowGiveback;
    my @state_objs = NACL::CS::StorageFailoverShowGiveback->fetch(@_);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub show_giveback

=head2 show_takeover

    my $show_takeover_obj = NACL::C::StorageFailover->show_takeover(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or

    my @show_takeover_objs = NACL::C::StorageFailover->show_takeover(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or

   my $show_takeover_obj = $StorageFailover_obj->show_takeover();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-StorageFailoverShowTakeover-pm/ATTRIBUTES>
of NACL::CS::StorageFailoverShowTakeover as input.
This method returns objects of the type L<NACL::CS::StorageFailoverShowTakeover|lib-NACL-CS-StorageFailoverShowTakeover-pm>.

=cut

sub show_takeover {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::StorageFailoverShowTakeover;
    my @state_objs = NACL::CS::StorageFailoverShowTakeover->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub show_takeover

=head2 bring_home

    NACL::C::StorageFailover->bring_home(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to forcefully bring home failed-over storage to its home node.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'tonode' => $string >>

Node Initiating Bring Home possible value(s) are, <nodename>,local

Applicable for CMode CLI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"storage failover bring-home" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub bring_home {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( tonode
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'storage_failover_bring_home',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_bring_home_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::StorageFailover::bring_home',
    );

    $Log->exit() if $may_exit;
} ## end sub bring_home

sub _bring_home_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'storage_failover_bring_home',);
    $Log->exit() if $may_exit;
} ## end sub _bring_home_cmode_cli

=head2 destroy_mailbox

    NACL::C::StorageFailover->destroy_mailbox(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);

								 or

    $StorageFailover_obj->destroy_mailbox(...);

(Class or instance method)
This method is used to destroy mailbox-disk contents.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local

Applicable for CMode CLI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"storage failover destroy-mailbox" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub destroy_mailbox {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            {'cmd_execution_time' => {type => SCALARREF, optional => 1},},
        api => 'storage_failover_destroy_mailbox',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_destroy_mailbox_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::StorageFailover::destroy_mailbox',
    );

    $Log->exit() if $may_exit;
} ## end sub destroy_mailbox

sub _destroy_mailbox_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'storage_failover_destroy_mailbox',);
    $Log->exit() if $may_exit;
} ## end sub _destroy_mailbox_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (node => {type => SCALAR});
}

sub _check_modify_opts {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my %common_opts;
    $pkg_or_obj->_hash_move(
        source => \%opts,
        target => \%common_opts,
        move   => [
            keys %{$pkg_or_obj->_common_validate_spec()},
            $pkg_or_obj->get_primary_keys()
        ]
    );
    delete $opts{enabled};
    if (scalar keys %opts > 0) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw(
                  'Apart from "enabled" option, other options are'
                . ' specified in modify call');
    }
    $Log->exit() if $may_exit;
} ## end sub _check_modify_opts

1;
