#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StoragePath Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StoragePath

=head1 DESCRIPTION

C<NACL::C::StoragePath> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP StoragePath.  A related class is
L<NACL::CS::StoragePath|lib-NACL-CS-StoragePath-pm>, which represents
the past state of an ONTAP StoragePath.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 initiator

=head2 array_name

=head2 target_wwpn

=head2 node

=cut

package NACL::C::StoragePath;

use strict;
use warnings;

use NACL::CS::StoragePath;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $StoragePath = NACL::C::StoragePath->new(command_interface=>$ci,
                                   initiator =>$initiator,
                                   array_name =>$array_name,
                                   target_wwpn =>$target_wwpn,
                                   node =>$node,
                                  );

Return a new StoragePath component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'initiator',
    scalar => 'lun_number',
    scalar => 'target_wwpn',
    scalar => 'node',
];

=head2 find

    my $StoragePath  = NACL::C::StoragePath->find(command_interface => $ci, ...);
    my @StoragePaths = NACL::C::StoragePath->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $StoragePath_state = $NACL::C::StoragePath->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::StoragePath would return
a NACL::CS::StoragePath object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 quiesce

   NACL::C::StoragePath->quiesce(
                                command_interface => $ci,
                                 initiator => $initiator,
                                 array_name => $array_name,
                                 target_wwpn => $target_wwpn,
                                 node => $node,
                                  ...);

(Class method)
This method is used to quiesces an array LUN on a path.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'storage-initiator-path-quiesce' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "initiator" => $initiator >>

(Required)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: initiator

=item C<< "array-name" => $array_name >>

(Required)

=item C<< "target-wwpn" => $target_wwpn >>

(Required)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: target-wwpn

=item C<< "node" => $node >>

(Required)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: node

=item C<< 'lun-number' => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: lun-number

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"storage path quiesce" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub quiesce {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( lun-number
                    )
            )
        },
        cli_cmd => 'storage path quiesce',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_quiesce_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_quiesce_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::StoragePath::quiesce',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub quiesce

sub _quiesce_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,  api  => 'storage_path_quiesce');
    $Log->exit() if $may_exit;
} ## end sub _quiesce_cmode_cli

sub _quiesce_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( initiator lun-number target-wwpn node
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'storage_initiator_path_quiesce',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _quiesce_cmode_zapi

=head2 resume

   NACL::C::StoragePath->resume(
                                command_interface => $ci,
                                 initiator => $initiator,
                                 array_name => $array_name,
                                 target_wwpn => $target_wwpn,
                                 node => $node,
                                  ...);

(Class method)
This method is used to resumes I/O to an array LUN on a path.

Uses a CMode CLI/ZAPI APISet.
  Invokes "storage-initiator-path-resume" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "initiator" => $initiator >>

(Required)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: initiator

=item C<< "array-name" => $array_name >>

(Required)

=item C<< "target-wwpn" => $target_wwpn >>

(Required)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: target-wwpn

=item C<< "node" => $node >>

(Required)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: node

=item C<< "lun-number" => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: lun-number


=item C<< '*' >>

This method takes all other options supported by
"storage path resume" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub resume {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( lun-number
                    )
            )
        },
        cli_cmd => 'storage path resume',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_resume_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_resume_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::StoragePath::resume',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub resume

sub _resume_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,  api  => 'storage_path_resume');
    $Log->exit() if $may_exit;
} ## end sub _resume_cmode_cli

sub _resume_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( initiator lun-number target-wwpn node
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'storage_initiator_path_resume',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _resume_cmode_zapi

=head2 show_by_initiator

    my $show_by_initiator_obj = NACL::C::StoragePath->show_by_initiator(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @show_by_initiator_objs = NACL::C::StoragePath->show_by_initiator(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $show_by_initiator_obj = $StoragePath_obj->show_by_initiator();

(Class or Instance method)
This method returns objects of the type L<NACL::CS::StoragePathShowByInitiator|lib-NACL-CS-StoragePathShowByInitiator-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch>
(requested_fields, filter, allow_empty) and has all of the other properties of
fetch methods (callable in scalar or list context, throws a NoElementsFound
exception if no matching elements are found)

=cut

sub show_by_initiator {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::StoragePathShowByInitiator;
    my @state_objs = NACL::CS::StoragePathShowByInitiator->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'initiator'   => {type => SCALAR},
        'lun-number'  => {type => SCALAR, optional => 1},
        'target-wwpn' => {type => SCALAR},
        'node'        => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
