#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Port Component Module
## @author Vishwanath.Rawat@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here
################################################################################

=head1 NAME

NACL::C::Switch::DiscoveryProtocol

=head1 DESCRIPTION

C<NACL::C::Switch::DiscoveryProtocol> Is a derived class of
L<NACL::C::Component::Switch|lib-NACL-C-Component-Switch-pm>. It represents access to
a Port.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

###################
# Package
package NACL::C::Switch::DiscoveryProtocol;

###################
# Everytime
use strict;
use warnings;

###################

use base 'NACL::C::Component::Switch';

# NATE Module
use NATE::Exceptions::Argument qw(:try);
use Params::Validate qw(validate_with SCALAR);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::ComponentUtils qw(Dumper _optional_scalars);

=head1 METHODS

=head2 new

    my $isdp = NACL::C::Switch::DiscoveryProtocol->new(
        command_interface=>$ci,
    );

Return a new ISDP component object with the given attributes.

=cut

=head2 enable

my $isdp = NACL::C::Switch::DiscoveryProtocol->enable(
    command_interface   => $ci,
);

OR

$isdp->enable();

(Class or instance method) This method enables port on the switch.
list of supported sets are CiscoNetwork and NetAppNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub enable {
    $Log->enter();
    my ($pkg_or_obj, @opts) = @_;

    $pkg_or_obj->_frontend_log_debug_opts(@opts);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@opts,
    );

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => "_enable_cisconetwork_cli",
                interface => "CLI",
                set       => "CiscoNetwork"
            },
            {
                method    => "_enable_netappnetwork_cli",
                interface => "CLI",
                set       => "NetAppNetwork"
            },
        ],
    );

    my $ret = $pkg_or_obj->new(
        command_interface => $opts{command_interface},
    );

    $Log->exit();
    return $ret;
} # end of sub enable

sub _enable_cisconetwork_cli {
    $Log->enter();
    my ($pkg, @opts) = @_;

    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset            = $opts{apiset};
    my $command_interface = $opts{command_interface};

    $apiset->configure('subsystem' => "terminal");
    if ($command_interface->hostrec()->hosttype =~ /nxos/) {
        $apiset->cdp_enable();
    } else {
        $apiset->cdp_run();
    }

    # Come out of config mode
    $pkg->_exit_config_modes_cisconetwork( apiset => $apiset);

    $Log->exit();
} # end sub _enable_cisconetwork_cli

sub _enable_netappnetwork_cli {
    $Log->enter();
    my ($pkg, @opts) = @_;

    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset            = $opts{apiset};
    my $command_interface = $opts{command_interface};
    my $timer = $opts{timer};


    $apiset->configure();

    $apiset->isdp_run();

    # Come out of config mode
    $apiset->exit();

    $Log->exit();
} # end sub _enable_netappnetwork_cli

=head2 set_timer

NACL::C::Switch::DiscoveryProtocol->set_timer(
    command_interface   => $ci,
    timer               => $timer,
);

OR

$isdp->set_timer(timer => $timer);

(Class or instance method) This method sets period of time between
sending new ISDP/CDP packet.

list of supported sets are CiscoNetwork and NetAppNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< timer => $number >>

Applicable for CiscoNetwork and NetAppNetwork CLI.

=back

=back

=cut

sub set_timer {
    $Log->enter();
    my ($pkg_or_obj, @opts) = @_;

    $pkg_or_obj->_frontend_log_debug_opts(@opts);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@opts,
        additional_specs => {
            timer => { type => SCALAR },
        },
    );

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => "_set_timer_cisconetwork_cli",
                interface => "CLI",
                set       => "CiscoNetwork"
            },
            {
                method    => "_set_timer_netappnetwork_cli",
                interface => "CLI",
                set       => "NetAppNetwork"
            },
        ],
    );

    $Log->exit();
} #end of sub set_timer

sub _set_timer_cisconetwork_cli {
    $Log->enter();
    my ($pkg, @opts) = @_;
    my %opts       = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};
    my $timer  = $opts{timer};

    $apiset->configure('subsystem' => "terminal");
    $apiset->cdp_timer(timer => $timer);

    # Come out of config mode
    $pkg->_exit_config_modes_cisconetwork( apiset => $apiset);

    $Log->exit();
} #end sub _set_timer_cisconetwork_cli

sub _set_timer_netappnetwork_cli {
    $Log->enter();
    my ($pkg, @opts) = @_;
    my %opts       = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};
    my $timer  = $opts{timer};

    $apiset->configure();

    $apiset->isdp_timer(timer => $timer);

    # Come out of config mode
    $apiset->exit();

    $Log->exit();
} #end sub _set_timer_netappnetwork_cli

sub _primary_keys_validate_spec {
    return ();
}

1;
