#
#
# Copyright (c) 2011-13 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary Interface Component Module
## @author rawat@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here
################################################################################

=head1 NAME

NACL::C::Switch::Interface

=head1 DESCRIPTION

C<NACL::C::Switch::Interface> Is a derived class of
L<NACL::C::Component::Switch|lib-NACL-C-Component-Switch-pm>. It represents access to
a Interface.

=head1 ATTRIBUTES

=head2 interface_type

Type of interface on switch
For Nexus, possible interface-type is C<< Ethernet >>"
For IOS, possible interface-type are C<<GigabitEthernet>>, C<<TenGigabitEthernet>>

=head2 interface_num

Interface number

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

###################
# Package
package NACL::C::Switch::Interface;

###################
# Everytime
use strict;
use warnings;

###################
use base 'NACL::C::Component::Switch';

# NATE Module includes
use Params::Validate qw(validate_pos validate_with SCALAR UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# NACL Modules
use NACL::CS::Switch::Interface;
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use NATE::Exceptions::Argument qw(:try);

# Some methods for Interface and PortChannel are implemented in _Interface.pm
use NACL::C::Switch::_Interface qw(:all);

=head1 METHODS
 
=head2 new

    my $interface = NACL::C::Switch::Interface->new(
        command_interface   => $ci,
        interface_type      => $ifc_type,
        interface_num       => $ifc_num
    );

Return a new interface component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'interface_type',
	scalar => 'interface_num',
];

=head2 create
    my $interface = NACL::C::Switch::Interface->create(
        command_interface   =>  $ci, 
        'interface-type'    =>  $interface_type, 
        'interface-num'     =>  $interface_num
    );

(Class method) Request creation of a new interface element and return a new component object that refers to it.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'interface-num'=>$string >>

(Required) c<< number >> of the interface is required on which action will be performed

=item C<< 'interface-type'=>$string >>

(Required) c<< type >> of the interface is required on which action will be performed
For Nexus, possible interface-type is C<< Ethernet >>"
For IOS, possible interface-type are C<<GigabitEthernet>>, C<<TenGigabitEthernet>>

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = $pkg->_common_validate_with( params => \@_ );
    
    my $ret = $pkg->_create( %opts );

    $Log->exit() if $may_exit;
    
    return $pkg->new(
        interface_type      => $opts{'interface-type'},
        interface_num       => $opts{'interface-num'},
        command_interface   => $opts{'command_interface'},
    );
}

=head2 set_channel_protocol

NACL::C::Switch::Interface->set_channel_protocol(
    command_interface   => $ci,
    'interface-type'    => $ifc_type,
    'interface-num'     => $ifc_num,
    'channel-protocol'  => "lacp"
);

                            or

  $interface_object->set_channel_protocol('channel-protocol' => "lacp");

(Class or instance method) This method sets value of the C<< channel-protocol >> for the interface.
list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< channel-protocol => $string >>

(Required) channel protocol to be set for the interface

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

sub set_channel_protocol {
    $Log->enter() if $may_enter;
	my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
		additional_spec => {
            'channel-protocol'  => { type => SCALAR },
            deconfigure         => { type => SCALAR, default => 0 },
        },
    );

    $pkg_or_obj->call_on_apiset(
        %opts,
		choices => [
            {   method    => "_set_channel_protocol_cisconetwork_cli",
                interface => "CLI",
                set       => "CiscoNetwork"
            },
        ],
    );
    $Log->exit() if $may_exit;
}

sub _set_channel_protocol_cisconetwork_cli {
    $Log->enter() if $may_enter;
	my $pkg_or_obj = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1,
    );

    my $pkg = ref($pkg_or_obj) || $pkg_or_obj; 
    $pkg->_set_ifc_cisconetwork_cli(%opts);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    
     my %api_opts;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        copy   => [ qw(channel-protocol deconfigure)],
    );
    
    $apiset->channel_protocol(%api_opts);
    
    $pkg_or_obj->_exit_config_modes_cisconetwork( apiset => $apiset);
    
	$Log->exit() if $may_exit;
}

=head2 set_channel_group

NACL::C::Switch::Interface->set_channel_group(
    command_interface   => $ci,
    'interface-type'    => $ifc_type,
    'interface-num'     => $ifc_num,
    'channel-num'       => $channel_num,
    'channel-mode'	    => $channel_state
);

                            or

$interface_object->set_channel_group(
    'channel-num'   => $channel_num,
    'channel-mode'  => $channel_state
);

(Class or instance method) This method sets interface to specified 
C<< channel-num >> with spedified C<< channel-mode >>. list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'channel-num'=>$string >>

(Required) channel number for the interface

=item C<< 'channel-mode'=>$string >>

(Required) mode to be set for the channel group

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

sub set_channel_group {
    $Log->enter() if $may_enter;
	my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
		additional_spec => {
            'channel-num'       => { type => SCALAR },
            'channel-mode'      => { type => SCALAR, optional => 1 },
            deconfigure         => { type => SCALAR, default => 0 },
        },
    );
    
    if ( !$opts{deconfigure} && !$opts{'channel-mode'} ) {
        $Log->exit() if $may_exit;
         NATE::Exceptions::Argument->throw(
            "channel-mode switch must be provide while setting channel-group");
    }

    $pkg_or_obj->call_on_apiset(
        %opts,
		choices => [
            {   method    => "_set_channel_group_cisconetwork_cli",
                interface => "CLI",
                set       => "CiscoNetwork"
            },
        ],
    );
    $Log->exit() if $may_exit;
}

sub _set_channel_group_cisconetwork_cli {
    $Log->enter() if $may_enter;
	my $pkg_or_obj = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1,
    );

    my $pkg = ref($pkg_or_obj) || $pkg_or_obj; 
    $pkg->_set_ifc_cisconetwork_cli(%opts);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my %api_opts;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        copy   => [ qw(channel-mode deconfigure channel-num)],
    );
    
    $apiset->channel_group(%api_opts);
    
    $pkg_or_obj->_exit_config_modes_cisconetwork( apiset => $apiset);
    
	$Log->exit() if $may_exit;
}

=head2 remove_channel_group

NACL::C::Switch::Interface->remove_channel_group(
    command_interface   => $ci,
    'interface-type'    => $ifc_type,
    'interface-num'     => $ifc_num,
    'channel-num'       => $channel_num
);

                            or

$interface_object->remove_channel_group(
    'channel-num' => $channel_num
);
  
(Class or instance method) This method will remove interface from specified
channel group. list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'channel-num'=>$string >>

(Required) channel number for the interface

=back

=back

=cut

sub remove_channel_group {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj,%args) = @_;
    $pkg_or_obj->set_channel_group(%args, deconfigure => 1);
    $Log->exit() if $may_exit;
}

=head2 set_mtu

NACL::C::Switch::Interface->set_mtu(
    command_interface   =>  $command_interface,
    'interface-type'    =>  "TenGigabitEthernet",
    'interface-num'     =>  '1/1',
    mtu                 =>  9216
);

                            or

$interface_object->set_mtu(mtu => 9216);

(Class or instance method) This method sets value of the C<< mtu >> for the 
interface. list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< mtu=>$int >>

(Required) mtu size to be set for the interface

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

=head2 set_speed

NACL::C::Switch::Interface->set_speed(
    command_interface   =>  $ci,
    'interface-type'    =>  "Ethernet",
    'interface-num'     =>  '1/1',
    speed               =>  1000
);

                            or

$interface_object->set_speed(speed => 1000);

(Class or instance method) This method sets value of the C<< speed >> on the interface.
list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< speed => $int >>

(Required) speed to be set for the interface

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

=head2 set_duplex

NACL::C::Switch::Interface->set_duplex(
    command_interface   => $ci,
    'interface-type'    =>  "Ethernet",
    'interface-num'     =>  "1/1",
    duplex              =>  "full"
);

                            or

$interface_object->set_duplex(duplex  =>  "full");

(Class or instance method) This method sets value of the C<< duplex >> for the interface.
list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< duplex => $string >>

(Required) duplex value to be set for interface

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

=head2 set_spanning_tree

NACL::C::Switch::Interface->set_spanning_tree(
    command_interface   => $ci,
    'interface-type'    => $ifc_type,
    'interface-num'     => $ifc_num,
    'st-option'         => $st_option,
    'st-option-value'	=> $st_option_value
);

                            or

$interface_object->set_spanning_tree(
    'st-option'         => $st_option,
    'st-option-value'   => $st_option_value
);

(Class or instance method) This method sets spanning tree configuration on 
interface. list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'st-option'=>$string >>

(Required) Option to be sets on spanning tree

=item C<< 'st-option-value'=>$string >>

(Required) value of the option to be sets on spanning tree

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

=head2 set_flowcontrol

NACL::C::Switch::Interface->set_flowcontrol(
    command_interface   => $ci,
    'interface-type'    => $ifc_type,
    'interface-num'     => $ifc_num,
    'flow-difection'    => $flow_direction,
    'flow-operation'    => $flow_operation,
);

                            or

$interface_object->set_flowcontrol(
    'flow-difection' => $flow_direction,
    'flow-operation' => $flow_operation, 
);

(Class or instance method) This method sets C<< flowcontrol >> on the interface.
list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'flow-direction'=>$string >>

(Required) Direction (send/receive) of flow to be set for interface

=item C<< 'flow-opertaion'=>$string >>

(Required) Operation(on/off/auto) of flow to be set for interface

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

=head2 set_switchport

NACL::C::Switch::Interface->set_switchport(
    command_interface   => $ci,
    'interface-type'    => $ifc_type,
    'interface-num'     => $ifc_num,
    mode                => 'access',
);

                            or

$interface_object->set_switchport(
    mode => 'access', 
);

(Class or instance method) This method sets C<< switchport >> setting for this interface.
list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< mode=>$boolean >>

(Optional) mode to be set. it is 'access' or 'trunk'

=item C<< 'access-vlan'=>$boolean >>

(Optional) whether access vlan to be set or not.

=item C<< 'vlan-list'=>$string >>

(Optional) List of vlan_ids that will have permision either allowed or native 

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

=head2 set_switchport_trunk

NACL::C::Switch::Interface->set_switchport_trunk(
    command_interface   => $ci,
    'interface-type'    => $ifc_type,
    'interface-num'     => $ifc_num,
    'allowed-vlan'	    => 1,
    add                 => 1,
    'vlan-list'         => $vlan_list,
);

                            or

$interface_object->set_switchport_trunk(
    'allowed-vlan'	    => 1,
    add                 => 1,
    'vlan-list'         => $vlan_list,
);

(Class or instance method) This method sets C<< switchport trunk >> setting for 
this interface.
list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< add=>$boolean >>

(Optional) set when vlan need to be add 

=item C<< remove=>$boolean >>

(Optional) set when vlan need to be remove 

=item C<< 'allowed-vlan'=>$boolean >>

(Optional) Whether vlan is allowed or not

=item C<< 'native-vlan'=>$boolean >>

(Optional) Whether vlan is native or not

=item C<< 'vlan-list'=>$string >>

(Optional) List of vlan_ids that will have permision either allowed or native 

=item C<< 'encapsulation'=>$string >>

(Optional) Encapsulation value. only supported in Cisco IOS switch.

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

=head2 remove_switchport

NACL::C::Switch::Interface->remove_switchport(
    command_interface   => $ci
    'interface-type'    => $ifc_type,
    'interface-num'     => $ifc_num,
    access-vlan         => 1,
);

                            or

$interface_object->remove_switchport( 
    access-vlan => 1,
);

(Class or instance method) This method will remove switchport configuration from
this interface. list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

=back

=back

=cut

=head2 up

NACL::C::Switch::Interface->up(
    command_interface   => $ci
    'interface-type'    => $ifc_type,
    'interface-num'     => $ifc_num,
);

                            or

$interface_object->up();

(Class or instance method) This method will bring up specified interface.
list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

=back

=back

=cut

=head2 down

NACL::C::Switch::Interface->down(
    command_interface   => $ci,
    'interface-type'    => $ifc_type,
    'interface-num'     => $ifc_num,
);

                            or

$interface_object->down();

(Class or instance method) This method will bring down specified interface.
list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

sub can {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my @opts = validate_pos( @_, $pkg_or_obj->_can_validate_pos_spec() );
    my $code;

    my $method = $opts[0];
    my $err_ptr = $opts[1];
    
    if ( $code = $pkg_or_obj->SUPER::can(@opts) ) {
        $Log->exit() if $may_exit;
        return $code;
    } else {
        my $a_method = '_'.$method;
        my $pkg = $pkg_or_obj->get_package_name();
        if ( $pkg_or_obj->UNIVERSAL::can( $a_method ) ) {
            $code = sub {
                my $self = shift;
                my %args = $self->_common_validate_with(
                    params => \@_,
                    allow_extra => 1
                );
                return $pkg->$a_method( %args );
            };
            $Log->exit() if $may_exit;
            return $code;
        } else {
            $$err_ptr .= "Could not resolve '$method' in package $pkg";
            $Log->exit() if $may_exit;
            return undef;
        }
    }
}

sub _primary_keys_validate_spec {
    return ( 
        'interface-num' => { type => SCALAR },
        'interface-type' => { type => SCALAR|UNDEF, optional => 1},
    );
}

1;
