#
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Port-Channel Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Switch::PortChannel

=head1 DESCRIPTION

C<NACL::C::Switch::PortChannel> is a derived class of 
L<NACL::C::Component::Switch|lib-NACL-C-Component-Switch-pm>. it represents the port-channel
(or etherchannel) on switch.

=head1 ATTRIBUTES

=head2 port_channel_group

port-channel number

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

package NACL::C::Switch::PortChannel;

###################
# Everytime
use strict;
use warnings;

###################

use base 'NACL::C::Component::Switch';

# NATE Module
use NATE::Exceptions::Argument qw(:try);
use Params::Validate qw(validate_pos validate_with SCALAR ARRAYREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# NACL Module
use NACL::CS::Switch::PortChannel;
use NACL::ComponentUtils qw (Dumper);

# Some methods for Interface and PortChannel are implemented in _Interface.pm
use NACL::C::Switch::_Interface qw(:all);

# define constant
use constant INTERFACE_TYPE => "port-channel";

=head1 METHODS

=head2 new

    my $interface = NACL::C::Switch::PortChannel->new(
        command_interface           =>  $ci,
        'port_channel_group'        =>  11
    );

Return a new portchannel component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'port_channel_group',
];

=head2 create

my $interface = NACL::C::Switch::PortChannel->create(
    command_interface       => $ci, 
    'port-channel-group'   => $port_channel_num
);

(Class method) Request creation of a new port-channel and return a new 
component object that refers to it. port-channel is a one type of interface. so 
most of the fuctionality of interface is apply on port-channel as well.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'port-channel-group' => $number >>

(Required) c<< number >> of the port-chhanel is required on which action will 
be performed

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = $pkg->_common_validate_with( params => \@_ );
    
    $opts{'interface-num'} = delete $opts{'port-channel-group'};
    $opts{'interface-type'} = INTERFACE_TYPE;
    $pkg->_create( %opts );

    $Log->exit() if $may_exit;
    return $pkg->new(
        port_channel_group      => $opts{'interface-num'},
        command_interface       => $opts{'command_interface'},
    );
}

=head2 purge

$port_channel->purge();

NACL::C::Switch::PortChannel->purge(
    command_interface           => $command_interface,
    'port-channel-group'       => '66',
);

(Class or Instance Method) This Component method will purge port-channel from 
switch.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See ATTRIBUTES section.

=item C<< port-channel-group => $port_channel_number >>

(Required for class method, Not Applicable for instance method)
See ATTRIBUTES section.

=back

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $Log->debug( "Opts to 'purge' front end are:\n" . Dumper( \@_ ) );
    my %opts = $pkg_or_obj->_common_validate_with( params => \@_ );

    $Log->exit() if $may_exit;
    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_purge_cisconetwork_cli",
                interface => "CLI",
                set       => "CiscoNetwork"
            },
        ],
    );
} #end of sub purge

sub _purge_cisconetwork_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    
    my $apiset = delete $opts{apiset};
    delete $opts{command_interface};
    $opts{'interface-num'} = delete $opts{'port-channel-group'};
    $opts{'interface-type'} = INTERFACE_TYPE;
   
    $pkg_or_obj->_exit_config_modes_cisconetwork( apiset => $apiset);;
    $apiset->configure(subsystem => "terminal");
    $apiset->interface(%opts, deconfigure => 1);

    $Log->exit() if $may_exit;
}

=head2 set_mtu

NACL::C::Switch::PortChannel->set_mtu(
    command_interface       =>  $command_interface,
    'port-channel-group'   =>  '1',
    mtu                     =>  9216
);

                            or

$port_channel_object->set_mtu(mtu => 9216);

(Class or instance method) This method sets value of the C<< mtu >> for the 
specified port-channel. list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< mtu=>$int >>

(Required) mtu size to be set.

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

=head2 set_speed

NACL::C::Switch::PortChannel->set_speed(
    command_interface       =>  $ci,
    'port-channel-group'   =>  '1',
    speed                   =>  1000
);

                            or

$port_channel_object->set_speed(speed => 1000);

(Class or instance method) This method sets value of the C<< speed >> on specified port-channel.
list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< speed => $int >>

(Required) speed to be set.

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

=head2 set_duplex

NACL::C::Switch::PortChannel->set_duplex(
    command_interface           =>  $ci,
    'port-channel-group'       =>  "1",
    duplex                      =>  "full"
);

                            or

$port_channel_object->set_duplex(duplex  =>  "full");

(Class or instance method) This method sets value of the C<< duplex >> for the
specified port-channel.
list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< duplex => $string >>

(Required) duplex value to be set.

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

=head2 set_spanning_tree

NACL::C::Switch::PortChannel->set_spanning_tree(
    command_interface       => $ci,
    'port-channel-group'   => $pc_num,
    'st-option'             => $st_option,
    'st-option-value'	    => $st_option_value
);

                            or

$port_channel_object->set_spanning_tree(
    'st-option'         => $st_option,
    'st-option-value'   => $st_option_value
);

(Class or instance method) This method sets spanning tree configuration on 
port channel. list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'st-option'=>$string >>

(Required) Option to be sets on spanning tree

=item C<< 'st-option-value'=>$string >>

(Required) value of the option to be sets on spanning tree

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

=head2 set_flowcontrol

NACL::C::Switch::PortChannel->set_flowcontrol(
    command_interface       => $ci,
    'port-channel-group'   => $pc_num,
    'flow-difection'        => $flow_direction,
    'flow-operation'        => $flow_operation,
);

                            or

$port_channel_object->set_flowcontrol(
    'flow-difection' => $flow_direction,
    'flow-operation' => $flow_operation, 
);

(Class or instance method) This method sets C<< flowcontrol >> on port-channel.
list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'flow-direction'=>$string >>

(Required) Direction (send/receive) of flow to be set for interface

=item C<< 'flow-opertaion'=>$string >>

(Required) Operation(on/off/auto) of flow to be set for interface

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

=head2 set_switchport

NACL::C::Switch::PortChannel->set_switchport(
    command_interface       => $ci,
    'port-channel-group'   => $pc_num,
    mode                    => 'access',
);

                            or

$port_channel_object->set_switchport(
    mode => 'access', 
);

(Class or instance method) This method sets C<< switchport >> setting for this 
port-channel.
list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< mode=>$boolean >>

(Optional) mode to be set. it is 'access' or 'trunk'

=item C<< 'access-vlan'=>$boolean >>

(Optional) whether access vlan to be set or not.

=item C<< 'vlan-list'=>$string >>

(Optional) List of vlan_ids that will have permision either allowed or native 

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

=head2 set_switchport_trunk

NACL::C::Switch::PortChannel->set_switchport_trunk(
    command_interface       => $ci,
    'port-channel-group'   => $pc_num,
    'allowed-vlan'	        => 1,
    add                     => 1,
    'vlan-list'             => $vlan_list,
);

                            or

$port_channel_object->set_switchport_trunk(
    'allowed-vlan'	    => 1,
    add                 => 1,
    'vlan-list'         => $vlan_list,
);

(Class or instance method) This method sets C<< switchport trunk >> setting for 
this port-channel.
list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< add=>$boolean >>

(Optional) set when vlan need to be add 

=item C<< remove=>$boolean >>

(Optional) set when vlan need to be remove 

=item C<< 'allowed-vlan'=>$boolean >>

(Optional) Whether vlan is allowed or not

=item C<< 'native-vlan'=>$boolean >>

(Optional) Whether vlan is native or not

=item C<< 'vlan-list'=>$string >>

(Optional) List of vlan_ids that will have permision either allowed or native 

=item C<< 'encapsulation'=>$string >>

(Optional) Encapsulation value. only supported on Cisco IOS switch.

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut

=head2 remove_switchport

NACL::C::Switch::PortChannel->remove_switchport(
    command_interface       => $ci
    'port-channel-group'   => $pc_num,
    access-vlan             => 1,
);

                            or

$port_channel_object->remove_switchport( 
    access-vlan => 1,
);

(Class or instance method) This method will remove switchport configuration from
this port-channel. list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

=back

=back

=cut

=head2 up

NACL::C::Switch::PortChannel->up(
    command_interface       => $ci
    'port-channel-group'   => $pc_num,
);

                            or

$port_channel_object->up();

(Class or instance method) This method will bring up specified port-channel.
list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

=back

=back

=cut

=head2 down

NACL::C::Switch::PortChannel->down(
    command_interface       => $ci,
    'port-channel-group'   => $pc_num,
);

                            or

$port_channel_object->down();

(Class or instance method) This method will bring down specified port-channel.
list of supported sets are CiscoNetwork

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

=item C<< deconfigure => $boolean >>

(Optional) if this set to 1, it will remove specified config from switch by
prepending C< no >> to command. default to 0

=back

=back

=cut


sub can {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my @opts = validate_pos( @_, $pkg_or_obj->_can_validate_pos_spec() );
    my $code;

    my $method = $opts[0];
    my $err_ptr = $opts[1];
    
    if ( $code = $pkg_or_obj->SUPER::can(@opts) ) {
        $Log->exit() if $may_exit;
        return $code;
    } else {
        my $a_method = '_'.$method;
        my $pkg = $pkg_or_obj->get_package_name();
        if ( $code = $pkg_or_obj->UNIVERSAL::can( $a_method ) ) {
            $code = sub {
                my $self = shift;
                my %args = $self->_common_validate_with(
                    params => \@_,
                    allow_extra => 1
                );
                $args{'interface-num'} = delete $args{'port-channel-group'};
                $args{'interface-type'} = INTERFACE_TYPE;
                return $pkg->$a_method( %args );
            };
            $Log->exit() if $may_exit;
            return $code;
        } else {
            $$err_ptr .= "Could not resolve '$method' in package $pkg";
            $Log->exit() if $may_exit;
            return undef;
        }
    }
}

sub _primary_keys_validate_spec {
    return (
        'port-channel-group' => { type => SCALAR },
    );
}

1;
