# Copyright (c) 2011-13 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary common routine for Interface and PortChannel Module
## @author nayankumar.patel@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here
################################################################################

=head1 NAME

NACL::C::Switch::_Interface

=head1 DESCRIPTION

Common code for Interface and PortChannel component implementation.

=cut

###################
# Package
package NACL::C::Switch::_Interface;

###################
# Everytime
use strict;
use warnings;

###################
#NATE Module
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR);

# NACL Module includes
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use NACL::APISet::Exceptions::ResponseException qw/:try/;

BEGIN {
    use Exporter qw(import);
    our %EXPORT_TAGS = (
        all => [ 
            qw (_create _create_cisconetwork_cli _set_mtu
                _set_mtu_cisconetwork_cli _set_speed
                _set_speed_cisconetwork_cli  _set_duplex
                _set_duplex_cisconetwork_cli _set_spanning_tree
                _set_spanning_tree_cisconetwork_cli _set_flowcontrol
                _set_flowcontrol_cisconetwork_cli _set_switchport
                _set_switchport_cisconetwork_cli _set_switchport_trunk
                _set_switchport_trunk_cisconetwork_cli 
                _remove_switchport _up _down _down_cisconetwork_cli
                _set_ifc_cisconetwork_cli _create_netappnetwork_cli
                _down_netappnetwork_cli
            )
        ]
    );
    our @EXPORT_OK = @{ $EXPORT_TAGS{all} };
}

#This is a AUTOLOAD method from Interface.pm and PortChannel.pm
#please check POD for create() method in Interface.pm and PortChannel.pm
sub _create {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $Log->debug( "Opts to 'create' front end are:\n" . Dumper( \@opts ) );
    my %opts = validate_with( 
        params  => \@opts,
        spec    => {
            'interface-type' => { type => SCALAR, optional => 1 },
            'interface-num' => { type => SCALAR },
        },
        allow_extra => 1,
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_create_cisconetwork_cli",
                interface => "CLI",
                set       => "CiscoNetwork"
            },
            {   method    => "_create_netappnetwork_cli",
                interface => "CLI",
                set       => "NetAppNetwork"
            },
        ],
    );
    
    $Log->exit() if $may_exit;
    return $ret;
}

sub _create_cisconetwork_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);
    
    $pkg->_set_ifc_cisconetwork_cli(%opts);
    
    $Log->exit() if $may_exit;
}

sub _create_netappnetwork_cli {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;
}

#This is a AUTOLOAD method from Interface.pm and PortChannel.pm
#please check POD for set_mtu() method in Interface.pm and PortChannel.pm
sub _set_mtu {
    $Log->enter() if $may_enter;
	my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = validate_with(
        params  => \@_,
		spec    => {
            mtu         => { type => SCALAR },
            deconfigure => { type => SCALAR, default => 0 },
        },
        allow_extra => 1
    );

    $pkg->call_on_apiset(
        %opts,
		choices => [
            {   method    => "_set_mtu_cisconetwork_cli",
                interface => "CLI",
                set       => "CiscoNetwork"
            },
        ],
    );
    $Log->exit() if $may_exit;
}

sub _set_mtu_cisconetwork_cli {
    $Log->enter() if $may_enter;
	my $pkg = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1,
    );

    $pkg->_set_ifc_cisconetwork_cli(%opts);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
   
    my %api_opts;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        copy   => [ qw(mtu deconfigure)],
    );

    if (   $command_interface->hostrec()->model() =~ /4948/
        && $api_opts{mtu} == 9216 ) {
        $Log->comment(
            "Cisco Ethernet 4948 switch does not support MTU 9216, configuring MTU to 9198");
        $api_opts{mtu} = 9198;
    }
    $apiset->mtu(%api_opts);

	$Log->exit() if $may_exit;
}

#This is a AUTOLOAD method from Interface.pm and PortChannel.pm
#please check POD for set_speed() method in Interface.pm and PortChannel.pm
sub _set_speed {
    $Log->enter() if $may_enter;
	my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = validate_with(
        params      => \@_,
		spec        => {
            speed       => { type => SCALAR },
            deconfigure => { type => SCALAR, default => 0 },
        },
        allow_extra => 1
    );

    $pkg->call_on_apiset(
        %opts,
		choices => [
            {   method    => "_set_speed_cisconetwork_cli",
                interface => "CLI",
                set       => "CiscoNetwork"
            },
        ],
    );
    $Log->exit() if $may_exit;
}

sub _set_speed_cisconetwork_cli {
    $Log->enter() if $may_enter;
	my $pkg = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1,
    );
    
    $pkg->_set_ifc_cisconetwork_cli(%opts);
   
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    
    my %api_opts;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        copy   => [ qw(speed deconfigure)],
    );
	
    $apiset->speed(%api_opts);
    
	$Log->exit() if $may_exit;
}

#This is a AUTOLOAD method from Interface.pm and PortChannel.pm
#please check POD for set_duplex() method in Interface.pm and PortChannel.pm
sub _set_duplex {
    $Log->enter() if $may_enter;
	my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = validate_with(
        params      => \@_,
		spec        => {
            duplex      => { type => SCALAR },
            deconfigure => { type => SCALAR, default => 0 },
        },
        allow_extra => 1
    );

    $pkg->call_on_apiset(
        %opts,
		choices => [
            {   method    => "_set_duplex_cisconetwork_cli",
                interface => "CLI",
                set       => "CiscoNetwork"
            },
        ],
    );
    $Log->exit() if $may_exit;
}

sub _set_duplex_cisconetwork_cli {
    $Log->enter() if $may_enter;
	my $pkg = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1,
    );

    $pkg->_set_ifc_cisconetwork_cli(%opts);
   
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    
    my %api_opts;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        copy   => [ qw(duplex deconfigure)],
    );
    
    $apiset->duplex(%api_opts);
    
	$Log->exit() if $may_exit;
}

#This is a AUTOLOAD method from Interface.pm and PortChannel.pm
#please check POD for set_spanning_tree() method in Interface.pm and PortChannel.pm
sub _set_spanning_tree {
    $Log->enter() if $may_enter;
	my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = validate_with(
        params      => \@_,
		spec        => {
            'st-option'         => { type => SCALAR },
            'st-option-value'   => { type => SCALAR },
            deconfigure         => { type => SCALAR, default => 0 },
        },
        allow_extra => 1
    );

    $pkg->call_on_apiset(
        %opts,
		choices => [
            {   method    => "_set_spanning_tree_cisconetwork_cli",
                interface => "CLI",
                set       => "CiscoNetwork"
            },
        ],
    );
    $Log->exit() if $may_exit;
}
sub _set_spanning_tree_cisconetwork_cli {
    $Log->enter() if $may_enter;
	my $pkg = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1,
    );

    $pkg->_set_ifc_cisconetwork_cli(%opts);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    
    my %api_opts;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        copy   => [ qw(st-option st-option-value deconfigure)],
    );
    
    $apiset->spanning_tree(%api_opts);
    
	$Log->exit() if $may_exit;
}


#This is a AUTOLOAD method from Interface.pm and PortChannel.pm
#please check POD for set_flowcontrol() method in Interface.pm and PortChannel.pm
sub _set_flowcontrol {
    $Log->enter() if $may_enter;
	my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = validate_with(
        params      => \@_,
		spec        => {
            'flow-direction'    => { type => SCALAR },
            'flow-operation'    => { type => SCALAR },
            deconfigure         => { type => SCALAR, default => 0 },
        },
        allow_extra => 1
    );

    $pkg->call_on_apiset(
        %opts,
		choices => [
            {   method    => "_set_flowcontrol_cisconetwork_cli",
                interface => "CLI",
                set       => "CiscoNetwork"
            },
        ],
    );
    $Log->exit() if $may_exit;
}

sub _set_flowcontrol_cisconetwork_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1,
    );

    $pkg->_set_ifc_cisconetwork_cli(%opts);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my %api_opts;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        copy   => [qw(flow-direction flow-operation deconfigure)],
    );

    if (   $command_interface->hostrec()->model() =~ /4948/
        && $api_opts{'flow-direction'} =~ /send/ ) {
        $Log->comment(
            "Cisco Ethernet 4948 switch does not support flowcontrol send configuration");
    } else {
        $apiset->flowcontrol(%api_opts);
    }
    $Log->exit() if $may_exit;
} ## end sub _set_flowcontrol_cisconetwork_cli

#This is a AUTOLOAD method from Interface.pm and PortChannel.pm
#please check POD for set_switchport() method in Interface.pm and PortChannel.pm
sub _set_switchport {
    $Log->enter() if $may_enter;
	my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = validate_with(
        params      => \@_,
		spec        => {
            _optional_scalars( qw(mode access-vlan vlan-list deconfigure) ),
        },
        allow_extra => 1
    );

    $pkg->call_on_apiset(
        %opts,
		choices => [
            {   method    => "_set_switchport_cisconetwork_cli",
                interface => "CLI",
                set       => "CiscoNetwork"
            },
        ],
    );
    $Log->exit() if $may_exit;
}

sub _set_switchport_cisconetwork_cli {
    $Log->enter() if $may_enter;
	my $pkg = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1,
    );
    
    $pkg->_set_ifc_cisconetwork_cli(%opts);
   
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
	my %api_opts;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        copy   => [ qw(mode access-vlan vlan-list deconfigure)],
    );
	
    $apiset->switchport(%api_opts);
    
	$Log->exit() if $may_exit;
}

#This is a AUTOLOAD method from Interface.pm and PortChannel.pm
#please check POD for set_switchport_trunk() method in Interface.pm and PortChannel.pm
sub _set_switchport_trunk {
    $Log->enter() if $may_enter;
	my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = validate_with(
        params      => \@_,
		spec        => {
            _optional_scalars( qw(add remove allowed-vlan native-vlan vlan-list encapsulation deconfigure) ),
        },
        allow_extra => 1
    );

    $pkg->call_on_apiset(
        %opts,
		choices => [
            {   method    => "_set_switchport_trunk_cisconetwork_cli",
                interface => "CLI",
                set       => "CiscoNetwork"
            },
        ],
    );
    $Log->exit() if $may_exit;
}

sub _set_switchport_trunk_cisconetwork_cli {
    $Log->enter() if $may_enter;
	my $pkg = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1,
    );
    
    $pkg->_set_ifc_cisconetwork_cli(%opts);
   
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
	my %api_opts;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        copy   => [ qw(add remove allowed-vlan native-vlan vlan-list encapsulation deconfigure)],
    );
	
    $apiset->switchport_trunk(%api_opts);
    
	$Log->exit() if $may_exit;
}

#This is a AUTOLOAD method from Interface.pm and PortChannel.pm
#please check POD for remove_switchport() method in Interface.pm and PortChannel.pm
sub _remove_switchport {
    $Log->enter() if $may_enter;
    my ($pkg,%args) = @_;
    $pkg->set_switchport(%args, deconfigure => 1);
    $Log->exit() if $may_exit;
}

#This is a AUTOLOAD method from Interface.pm and PortChannel.pm
#please check POD for up() method in Interface.pm and PortChannel.pm
sub _up {
    $Log->enter() if $may_enter;
    my ($pkg,%args) = @_;
    $pkg->down(%args, deconfigure => 1);
    $Log->exit() if $may_exit;
}

#This is a AUTOLOAD method from Interface.pm and PortChannel.pm
#please check POD for down() method in Interface.pm and PortChannel.pm
sub _down {
    my $pkg = shift;
   $pkg->_frontend_log_debug_opts(@_);
    my %opts = validate_with(
        params      => \@_,
        spec        => { 'deconfigure' => { type => SCALAR, default => 0 } },
        allow_extra => 1
    );

    $pkg->call_on_apiset(
        %opts,
		choices => [
            {   method    => "_down_cisconetwork_cli",
                interface => "CLI",
                set       => "CiscoNetwork"
            },
            {   method    => "_down_netappnetwork_cli",
                interface => "CLI",
                set       => "NetAppNetwork"
            },
        ],
    );
    $Log->exit() if $may_exit;
}

sub _down_cisconetwork_cli {
    my $pkg = shift;
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1,
    );
    
    $pkg->_set_ifc_cisconetwork_cli(%opts);
   
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
	my %api_opts;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        copy   => [ qw(deconfigure)],
    );
    
    $apiset->shutdown(%api_opts);
    $Log->exit() if $may_exit;
}

sub _down_netappnetwork_cli {
    my $pkg = shift;
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1,
    );
    
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
	my %api_opts;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        copy   => [ qw(deconfigure)],
    );
    
    $apiset->configure();
    $apiset->interface(interface => $opts{'interface-num'});
    
    $apiset->shutdown(%api_opts);
   
    # Come out of config mode.
    # Calling exit twice here is intentional
    $apiset->exit(); # Come out of interface mode
    $apiset->exit(); # Come out of configure mode
    
    $Log->exit() if $may_exit;
}

# THIS IS A PRIVATE METHOD TO SET SWITCH CLI TO PROPER STATE
# SWITCH CLI SHOULD BE IN CORRECT STATE TO CONFIGURE ANY CONFIG ON INTERFACE
sub _set_ifc_cisconetwork_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %api_opts;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        copy   => [ qw(interface-type interface-num)],
    );
      
	$pkg->_exit_config_modes_cisconetwork( apiset => $apiset);
	$apiset->configure(subsystem => "terminal");
    $apiset->interface(%api_opts);
    
	$Log->exit() if $may_exit;
}

1;
