# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary Aggregate Component Module
## @author Neethu.George@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::Aggregate

=head1 DESCRIPTION

C<NACL::C::SysManager::Aggregate> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>.  It represents access
to an ONTAP Aggregate present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::Aggregate|lib-NACL-CS-SysManager-Aggregate-pm>, which represents
the State Info of an ONTAP Aggregate present on the System Mananger GUI screen.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 aggregate

The name of the aggregate, used to identify it uniquely. (Aggregate
names are assumed to be unique within a node or within a cluster in
both cmode and 7mode)

=cut

package NACL::C::SysManager::Aggregate;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::Aggregate;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $aggregate_obj = NACL::C::SysManager::Aggregate->new(command_interface => $ci,
														  aggregate         => $aggregate);

Return a new Aggregate component object with the given attributes.

=cut

####################### Method Maker ###############################

use Class::MethodMaker [ scalar => 'aggregate', ];

=head2 create

   $Aggr_Obj = 
		NACL::C::SysManager::Aggregate->create(command_interface => $ci,
											   aggregate         => $aggr_name);
   
(Class method) Request creation of a new aggregate element, and return new
aggregate component object that refers to it.
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< aggregate => $aggregate >>

(Required) Name of the Aggregate to be created.

=item C<<  raidlevel = $string  >>

(Optional) Raid type used to create Aggregate

=item C<<  blocktype = $string  >>

(Optional) Block format of Aggregate

=item C<<  sync-mirror = $string  >>

(Optional) Enable Synchronous Mirroring for Aggregate

=item C<<  snaplock = $string  >>

(Optional) Enable Snaplock on this Aggregate

=item C<<  nodename = $string  >>

(Optional) Controller Name

=item C<<  disktype = $string  >>

(Required) Type of disk using which Aggregate has to be created

=item C<<  diskcount = $string  >>

(Required) Selected Disk Count

=item C<<  zeroing = $string  >>

(Optional) zeroing the Aggregate option

=back

=back

=cut

####################### GUI Create ###############################
sub create {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            disktype  => { type => SCALAR },
            diskcount => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( raidlevel blocktype sync-mirror
                    snaplock zeroing nodename flashpool
                    hddcount node checksum ssdcount 
                    raidsize raidtype aggrtype pool 
                    change-raidtype )
            ),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_create_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_create_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub create

####################### GUI 7Mode Create ###############################
sub _create_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $aggr_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "aggregate"   => "-aggrname",
            "disktype"    => "-disktype",
            "diskcount"   => "-diskcount_pool0",
            "raidlevel"   => "-raidlevel",
            "blocktype"   => "-blocktype",
            "sync-mirror" => "-synmirror",
            "snaplock"    => "-snaplock",
            "zeroing"     => "-zerroing",
            "flashpool"   => "-flashpool",
            "raidtype"    => "-raidtype",
            "hddcount"    => "-change_hdd_count",
            "pool"        => "-pool",
            "checksum"    => "-checksum",
            "ssdcount"    => "-change_ssd_count",
            "raidsize"    => "-change_raid_size",
            "change-raidtype"    => "-change_raid_type",
            "aggrtype"    => "-aggr_type",
        }
    );

    $apiset->create_aggr(%$aggr_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        aggregate         => $aggr_args->{-aggrname},
        command_interface => $command_interface
    );
} ## end sub _create_7mode_gui

####################### GUI CMode Create ###############################
sub _create_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $aggr_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "aggregate" => "-aggrname",
            "disktype"  => "-disktype",
            "diskcount" => "-diskcount",
            "raidlevel" => "-raidlevel",
            "raidtype"  => "-raidtype",
            "blocktype" => "-blocktype",
            "nodename"  => "-nodename",
            "zeroing"   => "-zerroing",
            "flashpool" => "-flashpool",
            "hddcount"  => "-change_hdd_count",
            "node"      => "-node",
            "checksum"  => "-checksum",
            "ssdcount"  => "-change_ssd_count",
            "raidsize"  => "-change_raid_size",
            "change-raidtype"  => "-change_raid_type",
            "aggrtype"  => "-aggr_type",
        }
    );

    $apiset->create_aggr(%$aggr_args);

    $Log->exit() if $may_exit;

    return $pkg->new(
        aggregate         => $aggr_args->{-aggrname},
        command_interface => $command_interface
    );
} ## end sub _create_cmode_gui

=head2 edit

   NACL::C::SysManager::Aggregate->edit(command_interface => $ci,
								        aggregate         => $aggr_name);
                                 
								or

   $Aggr_Obj->edit(...);

(Class or instance method)This method edits the Aggregate
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< aggregate => $aggregate >>

(Required) Aggregate to be edited.

=item C<<  new-name = $string  >>

(Optional) Rename the aggregate

=item C<<  raidlevel = $string  >>

(Optional) RAID type of aggregate

=item C<<  resize = $string  >>

(Optional) new size of the aggregate

=back

=back

=cut

####################### GUI Edit ###############################
sub edit {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            { _optional_scalars(qw(new-name raidlevel resize
                                   flashpool ssdcount hddcount
                                   checksum raidgroup raidtype
                                   raidsize node disktype
                                   specificraid )), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_edit_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_edit_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub edit

####################### GUI 7Mode edit ###############################
sub _edit_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "aggregate" => "-aggrname",
            "new-name"  => "-renameing",
            "raidlevel" => "-raidlevel",
            "resize"    => "-resize",
            "flashpool" => "-flashpool",
            "ssdcount"  => "-ssdcount",
            "hddcount"  => "-hdd_count",
            "checksum"  => "-checksum",
            "raidgroup" => "-raidgroup",
            "raidtype"  => "-raidtype",
            "raidsize"  => "-raidsize",
            "specificraid" => "-specificRaidGroup"
        }
    );

    $apiset->aggr_edit(%$aggr_args);

    if ( ( defined $aggr_args->{'-renameing'} ) && ( ref $pkg_or_obj ) ) {
        $pkg_or_obj->aggregate( $opts{'new-name'} );
    }

    $Log->exit() if $may_exit;
} ## end sub _edit_7mode_gui

####################### GUI CMode edit ###############################
sub _edit_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "aggregate" => "-aggrname",
            "new-name"  => "-renameing",
            "raidlevel" => "-raidlevel",
            "resize"    => "-resize",
            "flashpool" => "-flashpool",
            "specificraid" => "-specificRaidGroup",
            "ssdcount"  => "-ssdcount",
            "hddcount"  => "-hdd_count",
            "node"      => "-node",
            "disktype"  => "-disktype",
            "checksum"  => "-checksum",
            "raidgroup" => "-raidgroup",
            "raidtype"  => "-raidtype",
            "raidsize"  => "-raidsize",
        }
    );

    $apiset->aggr_edit(%$aggr_args);

    if ( ( defined $aggr_args->{'-renameing'} ) && ( ref $pkg_or_obj ) ) {
        $pkg_or_obj->aggregate( $opts{'new-name'} );
    }

    $Log->exit() if $may_exit;
} ## end sub _edit_cmode_gui

####################### GUI Online ###############################

=head2 online

	NACL::C::SysManager::Aggregate->online(command_interface => $ci,
										   aggregate  => $aggr,);
                                 
								or

   $Aggr_Obj->online(...);

(Class or instance method) This method onlines a Aggregate.
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< aggregate => $aggregate >>

(Required) Aggregate to be onlined.


=back

=back

=cut

sub online {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_online_cmode_gui',
                set    => 'SysManager_CMode',
            },
            {   method => '_online_7mode_gui',
                set    => 'SysManager_7Mode',
            },
        ],
    );

    $Log->exit() if $may_exit;

    return $ret;
} ## end sub online

####################### GUI 7Mode online ###############################
sub _online_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "aggregate" => "-aggrname" }
    );

    $apiset->aggr_status( -status => "online", %$aggr_args );

    $Log->exit() if $may_exit;
} ## end sub _online_7mode_gui

####################### GUI CMode online ###############################
sub _online_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "aggregate" => "-aggrname" }
    );

    $apiset->aggr_status( -status => "online", %$aggr_args );

    $Log->exit() if $may_exit;
} ## end sub _online_cmode_gui

=head2 offline

	NACL::C::SysManager::Aggregate->offline(command_interface => $ci,
											aggregate         => $aggr_name,);
                                
								or

   $Aggr_Obj->offline();

(Class or instance method)This method offlines a Aggregate.
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< aggregate => $aggregate >>

(Required) Aggregate to be offlined.

=back

=back

=cut

sub offline {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_offline_cmode_gui',
                set    => 'SysManager_CMode',
            },
            {   method => '_offline_7mode_gui',
                set    => 'SysManager_7Mode',
            },
        ],
    );

    $Log->exit() if $may_exit;

    return $ret;
} ## end sub offline

####################### GUI 7Mode offline ###############################
sub _offline_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "aggregate" => "-aggrname" }
    );

    $apiset->aggr_status( -status => "offline", %$aggr_args );

    $Log->exit() if $may_exit;
} ## end sub _offline_7mode_gui

####################### GUI CMode offline ###############################
sub _offline_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "aggregate" => "-aggrname" }
    );

    $apiset->aggr_status( -status => "offline", %$aggr_args );

    $Log->exit() if $may_exit;
} ## end sub _offline_cmode_gui

=head2 restrict

   NACL::C::SysManager::Aggregate->restrict(command_interface => $ci,
											aggregate  => $aggr,);
                                
								or

   $Aggr_Obj->restrict();

(Class or instance method)This method restricts a Aggregate.
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< aggregate => $aggregate >>

(Required) Aggregate to be restricted.

=back

=back

=cut

sub restrict {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_restrict_cmode_gui',
                set    => 'SysManager_CMode',
            },
            {   method => '_restrict_7mode_gui',
                set    => 'SysManager_7Mode',
            },
        ],
    );

    $Log->exit() if $may_exit;

    return $ret;
} ## end sub restrict

####################### GUI 7Mode restrict ###############################
sub _restrict_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "aggregate" => "-aggrname" }
    );

    $apiset->aggr_status( -status => "restrict", %$aggr_args );

    $Log->exit() if $may_exit;
} ## end sub _restrict_7mode_gui

####################### GUI CMode restrict ###############################
sub _restrict_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "aggregate" => "-aggrname" }
    );

    $apiset->aggr_status( -status => "restrict", %$aggr_args );

    $Log->exit() if $may_exit;
} ## end sub _restrict_cmode_gui

=head2 delete

	NACL::C::SysManager::Aggregate->delete(command_interface => $ci,
										   aggregate         => $aggr_name,);
                                
								or

   $Aggr_Obj->delete();

(Class or instance method)This method deletes a Aggregate.
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< aggregate => $aggregate >>

(Required) Aggregate to be deleted.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_delete_cmode_gui',
                set    => 'SysManager_CMode',
            },
            {   method => '_delete_7mode_gui',
                set    => 'SysManager_7Mode',
            },
        ],
    );

    $Log->exit() if $may_exit;

    return $ret;
} ## end sub delete

####################### GUI 7Mode delete ###############################
sub _delete_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "aggregate" => "-aggrname" }
    );

    $apiset->aggr_delete(%$aggr_args);

    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_gui

####################### GUI CMode delete ###############################
sub _delete_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "aggregate" => "-aggrname" }
    );

    $apiset->aggr_delete(%$aggr_args);

    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_gui

=head2 mirror

	NACL::C::SysManager::Aggregate->mirror(command_interface => $ci,
										   aggregate  => $aggr_name,);
                                
								or

   $Aggr_Obj->mirror();

(Class or instance method)This method mirrors a Aggregate
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< aggregate => $aggregate >>

(Required) Aggregate to be mirrored.

=item C<<  aggregate-disk   >> 

(Optional) list of aggregate disks

=item C<< mirror-disk  >>

(Optional) list of mirror disks

=item C<< autoselect  >>

(Optional) enable Auto select

=back

=back

=cut

sub mirror {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "aggregate-disk" => { type => ARRAYREF, optional => 1 },
            "mirror-disk"    => { type => ARRAYREF, optional => 1 },
            _optional_scalars(qw(autoselect)),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_mirror_cmode_gui',
                set    => 'SysManager_CMode',
            },
            {   method => '_mirror_7mode_gui',
                set    => 'SysManager_7Mode',
            },
        ],
    );

    $Log->exit() if $may_exit;

    return $ret;
} ## end sub mirror

####################### GUI 7Mode mirror ###############################
sub _mirror_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "aggregate"      => "-aggrname",
            "aggregate-disk" => "-aggrdisk",
            "mirror-disk"    => "-mirrordisk",
            "autoselect"     => "-autoselect"
        }
    );

    $apiset->aggr_mirror(%$aggr_args);

    $Log->exit() if $may_exit;
} ## end sub _mirror_7mode_gui

####################### GUI CMode mirror ###############################
sub _mirror_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "aggregate"      => "-aggrname",
            "aggregate-disk" => "-aggrdisk",
            "mirror-disk"    => "-mirrordisk",
            "autoselect"     => "-autoselect"
        }
    );

    $apiset->aggr_mirror(%$aggr_args);

    $Log->exit() if $may_exit;
} ## end sub _mirror_cmode_gui

=head2 plex_online

	NACL::C::SysManager::Aggregate->plex_online(command_interface => $ci,
												aggregate         => $aggr_name,);
                                 or

   $Aggr_Obj->plex_online();

(Class or instance method)This method onlines a Plex Aggregate. 
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< aggregate => $aggregate >>

(Required) Plex Aggregate to be Onlined.

=item C<<  nodename   >> 

(Optional) Node name 

=item C<< plexname  >>

(Optional) Plex Name to be onlined

=item C<< splitplex  >>

(Optional) Split plex option

=item C<< newplex  >>

(Optional) new plex option

=item C<< destroy  >>

(Optional) destroy the plex

=back

=back

=cut

sub plex_online {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            plexname => { type => SCALAR },
            _optional_scalars(qw(splitplex newplex destroy)),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_plex_online_cmode_gui',
                set    => 'SysManager_CMode',
            },
            {   method => '_plex_online_7mode_gui',
                set    => 'SysManager_7Mode',
            },
        ],
    );

    $Log->exit() if $may_exit;

    return $ret;
} ## end sub plex_online

####################### GUI 7Mode plex_online ###############################
sub _plex_online_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "aggregate" => "-aggrname",
            "plexname"  => "-plexname",
            "splitplex" => "-splitplex",
            "newplex"   => "-newplex",
            "destroy"   => "-destroy",
        }
    );

    $apiset->aggr_plexes( -status => "online", %$aggr_args );
    $Log->exit() if $may_exit;
} ## end sub _plex_online_7mode_gui

####################### GUI CMode plex_online ###############################
sub _plex_online_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "aggregate" => "-aggrname",
            "plexname"  => "-plexname",
            "splitplex" => "-splitplex",
            "newplex"   => "-newplex",
            "destroy"   => "-destroy",
        }
    );

    $apiset->aggr_plexes( -status => "online", %$aggr_args );
    $Log->exit() if $may_exit;
} ## end sub _plex_online_cmode_gui

=head2 plex_offline

	NACL::C::SysManager::Aggregate->plex_offline(command_interface => $ci,
												 aggregate         => $aggr_name,);
                                
								or

   $Aggr_Obj->plex_offline();

(Class or instance method)This method offlines Plex Aggregate.
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< aggregate => $aggregate >>

(Required) Plex Aggregate to be Offlined.

=item C<<  nodename   >> 

(Optional) Node name

=item C<< plexname  >>

(Optional) plex Name to be offlined

=item C<< splitplex  >>

(Optional) Split plex option

=item C<< newplex  >>

(Optional) New plex

=item C<< destroy  >>

(Optional) Destroy the plex

=back

=back

=cut

sub plex_offline {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            plexname => { type => SCALAR },
            _optional_scalars(qw(splitplex newplex destroy)),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_plex_offline_cmode_gui',
                set    => 'SysManager_CMode',
            },
            {   method => '_plex_offline_7mode_gui',
                set    => 'SysManager_7Mode',
            },
        ],
    );

    $Log->exit() if $may_exit;

    return $ret;
} ## end sub plex_offline

####################### GUI 7Mode plex_offline ###############################
sub _plex_offline_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "aggregate" => "-aggrname",
            "plexname"  => "-plexname",
            "splitplex" => "-splitplex",
            "newplex"   => "-newplex",
            "destroy"   => "-destroy",
        }
    );

    $apiset->aggr_plexes( -status => "offline", %$aggr_args );
    $Log->exit() if $may_exit;
} ## end sub _plex_offline_7mode_gui

####################### GUI CMode mirror ###############################
sub _plex_offline_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "aggregate" => "-aggrname",
            "plexname"  => "-plexname",
            "splitplex" => "-splitplex",
            "newplex"   => "-newplex",
            "destroy"   => "-destroy",
        }
    );

    $apiset->aggr_plexes( -status => "offline", %$aggr_args );
    $Log->exit() if $may_exit;
} ## end sub _plex_offline_cmode_gui

=head2 refresh

	NACL::C::SysManager::Aggregate->refresh(command_interface => $ci,);

                            or

	$Aggr_object->refresh();

 (Class or instance method)This method refreshes Aggregate page.
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub refresh {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_refresh_cmode_gui',
                set    => 'SysManager_CMode',
            },
            {   method => '_refresh_7mode_gui',
                set    => 'SysManager_7Mode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub refresh

####################### GUI 7Mode Refresh ###############################
sub _refresh_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy( source => \%opts, );

    $apiset->refresh_aggr(%$aggr_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_7mode_gui

####################### GUI CMode Refresh ###############################
sub _refresh_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy( source => \%opts, );

    $apiset->refresh_aggr(%$aggr_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_cmode_gui

=head2 add_disk

	NACL::C::SysManager::Aggregate->add_disk(command_interface => $ci,
											 aggregate         => $aggr_name,);

                            or

	$Aggr_object->add_disk();

   (Class or instance method)This method sdds disks to Aggregate.
Uses a CMode GUI or 7Mode GUI APISet.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< raidsize  >>

(Optional) RAID type

=item C<< raidgroup  >>

(Optional) RAID group

=item C<< raid-specific-group  >>

(Optional) RAID specific group

=item C<< "disk-count-pool0"  >>

(Optional) Pool0 disk count

=item C<< "disk-count-pool1"  >>

(Optional) Pool1 disk count

=back

=back

=cut

sub add_disk {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "disk-count-pool0" => { type => ARRAYREF, optional => 1 },
            "disk-count-pool1" => { type => ARRAYREF, optional => 1 },
            _optional_scalars(qw(raidsize raidgroup raid-specific-group)),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_add_disk_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_add_disk_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;

    return $ret;
} ## end sub add_disk

####################### GUI 7Mode add_disk ###############################
sub _add_disk_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "aggregate"           => "-aggrname",
            "raidsize"            => "-raidsize",
            "raidgroup"           => "-raidgroup",
            "raid-specific-group" => "-raidspecificgrp",
            "disk-count-pool0"    => "-diskcount_pool0",
            "disk-count-pool1"    => "-diskcount_pool1",
        }
    );

    $apiset->aggr_adddisk(%$aggr_args);
    $Log->exit() if $may_exit;
} ## end sub _add_disk_7mode_gui

####################### GUI CMode add_disk ###############################
sub _add_disk_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "aggregate"           => "-aggrname",
            "raidsize"            => "-raidsize",
            "raidgroup"           => "-raidgroup",
            "raid-specific-group" => "-raidspecificgrp",
            "disk-count-pool0"    => "-diskcount_pool0",
            "disk-count-pool1"    => "-diskcount_pool1",
        }
    );

    $apiset->aggr_adddisk(%$aggr_args);
    $Log->exit() if $may_exit;
} ## end sub _add_disk_cmode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return ( aggregate => { type => SCALAR }, );
}

1;
