# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary CifsDomain Component Module
## @author vikas.p@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::CifsDomain

=head1 DESCRIPTION

C<NACL::C::SysManager::CifsDomain> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>.  It represents access
to an ONTAP CifsDomain present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::CifsDomain|lib-NACL-CS-SysManager-CifsDomain-pm>, which represents
the State Info of an ONTAP CifsDomain present on the System Manger GUI Screen.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

(Required for CMode, Not Applicable for 7Mode) The name of the vserver
in which the cifs is present.

=cut

package NACL::C::SysManager::CifsDomain;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::CifsDomain;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

	my $cifs = NACL::C::SysManager::CifsDomain->new(command_interface => $ci,
											        vserver           => $vserver_name);

Return a new CifsDomain component object with the given attributes.

=cut

####################### Method Maker ###############################
use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'domaincontroller',
    scalar => 'ldapserver',
    scalar => 'fqdn',
];

=head2 edit

	NACL::C::SysManager::CifsDomain->edit(command_interface => $ci,...);

                            or

	$cifs_object->edit(...);

(Class or instance method) This method is used to edit CifsDomain Options.  
Uses a 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "schedule-password-change" => $string >>

(Optional) Schedule Password Change

=item C<< "trace-log" => $string >>

(Optional) Enable Trace Log.

=back

=back

=cut

sub edit {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            { _optional_scalars(qw (schedule-password-change trace-log)), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_edit_7mode_gui',
                set    => 'SysManager_7Mode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub edit

####################### GUI 7Mode Edit Domain ###############################
sub _edit_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $cifs_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "schedule-password-change" => "-schedulepasswd",
            "trace-log"                => "-enabletrace",
        }
    );

    $apiset->edit_cifs_domain_options(%$cifs_args);
    $Log->exit() if $may_exit;
} ## end sub _edit_7mode_gui

=head2 group_policy_reload

	NACL::C::SysManager::CifsDomain->group_policy_reload(command_interface => $ci,
														 vserver           => $vserver_name,...);

                            or

	$cifs_object->group_policy_reload(...);

(Class or instance method) This method is used to reload Cifs Domain Group Policy.  
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

The name of the vserver in which the cifs Domain Group policy has to be reloaded.

=back

=back

=cut

sub group_policy_reload {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( 
        params              => \@args,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_group_policy_reload_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub group_policy_reload

########## GUI CMode Domain Group Policy Reload ##########
sub _group_policy_reload_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $cifs_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "vserver" => "-vservername", }
    );

    $apiset->cifs_domain_group_policy_reload(%$cifs_args);
    $Log->exit() if $may_exit;
} ## end sub _group_policy_reload_cmode_gui

=head2 group_policy_enable

	NACL::C::SysManager::CifsDomain->group_policy_enable(
        command_interface => $ci,
		vserver           => $vserver_name,...);

                            or

	$cifs_object->group_policy_enable(...);

(Class or instance method) This method is used to enable Cifs Domain Group Policy.  
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

The name of the vserver in which the cifs Domain Group policy has to be Enabled.

=back

=back

=cut

sub group_policy_enable {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
         params              => \@args,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_group_policy_enable_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub group_policy_enable

############ GUI CMode Domain Group Policy Enable #############
sub _group_policy_enable_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $cifs_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "vserver" => "-vservername", }
    );

    $apiset->cifs_domain_group_policy_enable(%$cifs_args);
    $Log->exit() if $may_exit;
} ## end sub _group_policy_enable_cmode_gui

=head2 group_policy_disable

	NACL::C::SysManager::CifsDomain->group_policy_disable (command_interface => $ci,
															vserver           => $vserver_name,...);

                            or

  $cifs_object->group_policy_disable(...);

(Class or instance method) This method is used to disable Cifs Domain Group Policy.  
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

The name of the vserver in which the cifs Domain Group policy has to be Disabled.

=back

=back

=cut

sub group_policy_disable {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( 
        params              => \@args,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_group_policy_disable_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub group_policy_disable

############## GUI CMode Domain Group Policy Disable #################
sub _group_policy_disable_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $cifs_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "vserver" => "-vservername", }
    );

    $apiset->cifs_domain_group_policy_disable(%$cifs_args);
    $Log->exit() if $may_exit;
} ## end sub _group_policy_disable_cmode_gui

=head2 group_policy_update

	NACL::C::SysManager::CifsDomain->group_policy_update(
        command_interface => $ci,
		vserver           => $vserver_name,...);

                            or

	$cifs_object->group_policy_update(...);

(Class or instance method)This method is used to update Cifs Domain Group Policy.  
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

The name of the vserver in which the cifs Domain Group policy has to be Updated.

=back

=back

=cut

sub group_policy_update {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( 
        params              => \@args,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_group_policy_update_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub group_policy_update

######### GUI CMode Domain Group Policy Update ############
sub _group_policy_update_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $cifs_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "vserver" => "-vservername", }
    );

    $apiset->cifs_domain_group_policy_update(%$cifs_args);
    $Log->exit() if $may_exit;
} ## end sub _group_policy_update_cmode_gui

=head2 group_policy_view

	NACL::C::SysManager::CifsDomain->group_policy_view(command_interface => $ci,
															 vserver           => $vserver_name,...);

                            or

	$cifs_object->group_policy_view(...);

(Class or instance method) This method is used to view Cifs Domain Group Policy.  
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

The name of the vserver in which the cifs Domain Group policy has to be Viewed.

=back

=back

=cut

sub group_policy_view {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params              => \@args,
        additional_spec     => { "group-policy" => { type => SCALAR }},
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_group_policy_view_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub group_policy_view

############## GUI CMode Domain Group Policy View ##############
sub _group_policy_view_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $cifs_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "vserver"      => "-vservername",
            "group-policy" => "-group_policy",
        }
    );

    $apiset->cifs_domain_group_policy_view(%$cifs_args);
    $Log->exit() if $may_exit;
} ## end sub _group_policy_view_cmode_gui

=head2 reset

	NACL::C::SysManager::CifsDomain->reset(command_interface => $ci,
										    vserver           => $vserver_name,...);

                            or

	$cifs_object->reset(...);

(Class or instance method) This method is used to reset CifsDomain Domain.  
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

The name of the vserver in which the cifs Symlinks page has to be refreshed.

=back

=back

=cut

sub reset {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( 
        params              => \@args,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_reset_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_reset_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub reset

####################### GUI 7Mode Reset Domain ###############################
sub _reset_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $cifs_args = $pkg_or_obj->_hash_copy( source => \%opts, );

    $apiset->cifs_reset_domain(%$cifs_args);
    $Log->exit() if $may_exit;
} ## end sub _reset_7mode_gui

####################### GUI CMode Reset Domain ###############################
sub _reset_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $cifs_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "vserver" => "-vservername", }
    );

    $apiset->cifs_reset_domain(%$cifs_args);
    $Log->exit() if $may_exit;
} ## end sub _reset_cmode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return (
        vserver            => { type => SCALAR | UNDEF, optional => 1 },
        'domaincontroller' => { type => SCALAR | UNDEF, optional => 1 },
        'ldapserver'       => { type => SCALAR | UNDEF, optional => 1 },
        'fqdn'             => { type => SCALAR | UNDEF, optional => 1 },
    );
} ## end sub _primary_keys_validate_spec

1;
