# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Disk Component Module
## @author Neethu.George@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::Disk

=head1 DESCRIPTION

C<NACL::C::SysManager::Disk> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>.  It represents access
to an ONTAP Disk present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::Disk|lib-NACL-CS-SysManager-Disk-pm>, which represents
the State Info of an ONTAP Disk present on the System Manger GUI Screen.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 disk

The name of the disk, used to identify it uniquely.  (disk
names are assumed to be unique within a node)

=head2 aggregate

The name of the aggregate which is linked with the disk.

=cut

package NACL::C::SysManager::Disk;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::Disk;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $Disk_obj = NACL::C::SysManager::Disk->new(command_interface=>$ci);

Return a new Disk component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'aggregate',
    scalar => 'disk',
];

=head2 createaggr

	my $disk_object = NACL::C::SysManager::Disk->createaggr(command_interface => $ci,
														    aggregate         => $aggr_name, 
														    disk              => @disks,);

(Class method) This method creates aggregate.
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< aggregate => $aggregate >>

(Optional) Aggregate to be created with disks

=item C<< disk => $arrayref >>

(Reqired) list of disks.

=item C<< raidtype => $arrayref >>

(Optional) RAID type

=item C<< snaplock => $arrayref >>

(Optional) SnapLock Aggregate

=item C<< disktype => $arrayref >>

(Optional) Type of the disk.

=back

=back

=cut

sub createaggr {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            disk => { type => ARRAYREF },
            _optional_scalars(qw( raidtype snaplock disktype)),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_createaggr_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_createaggr_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub createaggr

####################### GUI 7Mode Createaggr ###############################
sub _createaggr_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $disk_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "aggregate" => "-aggrname",
            "disk"      => "-disks",
            "snaplock"  => "-snaplock",
            "raidtype"  => "-raidtype",
            "disktype"  => "-disktype"
        }
    );

    $apiset->create_aggregate_from_disks(%$disk_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        aggregate         => $disk_args->{-aggrname},
        command_interface => $command_interface
    );

    $Log->exit() if $may_exit;
} ## end sub _createaggr_7mode_gui

####################### GUI CMode Createaggr ###############################
sub _createaggr_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $disk_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "aggregate" => "-aggrname",
            "disk"      => "-disks",
            "snaplock"  => "-snaplock",
            "raidtype"  => "-raidtype"
        }
    );

    $apiset->create_aggregate_from_disks(%$disk_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        aggregate         => $disk_args->{-aggrname},
        command_interface => $command_interface
    );

    $Log->exit() if $may_exit;
} ## end sub _createaggr_cmode_gui

=head2 add

    NACL::C::SysManager::Disk->add(command_interface => $ci,
								  aggregate  => $aggr, 
                                  disk => \@disks,);
                                
							or

    $disk_object->add(...);

(Class or instance method)This method add disks to aggregate
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< aggregate => $aggregate >>

(Optional) Aggregate to which disk is to be added

=item C<< disk => @arrayref >>

(Reqired) List of disks to be added

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => { disk => { type => ARRAYREF }, }
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_add_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_add_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub add

####################### GUI 7Mode add ###############################
sub _add_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $disk_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "aggregate" => "-aggrname",
            "disk"      => "-disks"
        }
    );

    $apiset->add_disks_to_aggregate(%$disk_args);
    $Log->exit() if $may_exit;
} ## end sub _add_7mode_gui

####################### GUI CMode add ###############################
sub _add_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $disk_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "aggregate" => "-aggrname",
            "disk"      => "-disks"
        }
    );

    $apiset->add_disks_to_aggregate(%$disk_args);
    $Log->exit() if $may_exit;
} ## end sub _add_cmode_gui

=head2 refresh

	NACL::C::SysManager::Disk->refresh(command_interface => $ci,
									   );

                            or

  $disk_object->refresh();

(Class or instance method)This method refreshs disk page

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub refresh {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_refresh_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_refresh_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub refresh

####################### GUI 7Mode Refresh ###############################
sub _refresh_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $disk_args = $pkg_or_obj->_hash_copy( source => \%opts, );

    $apiset->disk_refresh(%$disk_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_7mode_gui

####################### GUI CMode Refresh ###############################
sub _refresh_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $disk_args = $pkg_or_obj->_hash_copy( source => \%opts, );

    $apiset->disk_refresh(%$disk_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_cmode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return (
        aggregate => { type => SCALAR | UNDEF, optional => 1 },
        disk      => { type => SCALAR | UNDEF, optional => 1 },
    );
} ## end sub _primary_keys_validate_spec

1;
