# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary ExportPolicyRule Component Module
## @author vikas.p@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::ExportPolicyRule

=head1 DESCRIPTION

C<NACL::C::SysManager::ExportPolicyRule> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>.  It represents access
to an ONTAP ExportPolicyRule present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::ExportPolicyRule|lib-NACL-CS-SysManager-ExportPolicyRule-pm>, which represents
the State Info of an ONTAP ExportPolicyRule present on the System Manger GUI Screen.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 policy

The name of the exportPolicyRule, used to identify it uniquely. (ExportPolicyRule
names are assumed to be unique in a vserver in Cmode)

=head2 ruleindex

The Index of the exportPolicyRule used to identify it uniquely.

=head2 vserver

The name of the vserver in which the exportPolicyRule is present.

=cut

package NACL::C::SysManager::ExportPolicyRule;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::ExportPolicyRule;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

    my $exportPolicyRule = 
		NACL::C::SysManager::ExportPolicyRule->new(command_interface => $ci,
												   policy            => $policy_name,
												   vserver           => $vserver_name,
												   ruleindex         => $index);

Return a new ExportPolicyRule component object with the given attributes.

=cut

####################### Method Maker ###############################
use Class::MethodMaker [
    scalar => 'policy',
    scalar => "ruleindex",
    scalar => "vserver",
];

=head2 add

  my $exportPolicyRule_object  = 
	NACL::C::SysManager::ExportPolicyRule->add( 
							command_interface => $ci,
							policy            => $policy_name,
							vserver           => $vserver_name
							ruleindex         => $index));

(Class method) Request creation of a new exportPolicyRule element, and  return a new
exportPolicyRule component object that refers to it.
Uses CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< policy => $string >>

(Required) Name of ExportPolicy.

=item C<< vserver => $string >>

The name of the vserver in which the exportPolicyRule is present.

=item C<< "client-specification" => $string >>

(Optional) Client Specifications.

=item C<< "ruleindex" => $string >>

(Required) Index no.

=item C<< "access-protocol" => $string >>

(Optional) Access protocols

=item C<< "allow-read-only-access" => $string >>

(Optional) Enable read only access

=item C<< "read-only-access-option" => $string >>

(Optional) Read only access options

=item C<< "allow-read-write-access" => $string >>

(Optional) Enable read write access

=item C<< "read-write-access-option" => $string >>

(Optional) Read write access options

=item C<< "allow-superuser-access" => $string >>

(Optional) Enable Superuser access

=item C<< "superuser-access-option" => $string >>

(Optional) Superuser access options

=back

=back

=cut

####################### GUI Add ###############################
sub add {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars
                qw(policy client-specification ruleindex access-protocol
                allow-read-only-access read-only-access-option
                allow-read-write-access read-write-access-option
                allow-superuser-access superuser-access-option),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_add_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub add

####################### GUI CMode Add ###############################
sub _add_cmode_gui {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $exp_pol_rule_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "vserver"                  => "-vservername",
            "policy"                   => "-policyName",
            "client-specification"     => "-clientspec",
            "ruleindex"                => "-ruleindex",
            "access-protocol"          => "-protocol",
            "allow-read-only-access"   => "-readonly",
            "read-only-access-option"  => "-addreadonlyopt",
            "allow-read-write-access"  => "-readwrite",
            "read-write-access-option" => "-addreadwriteopt",
            "allow-superuser-access"   => "-superuser",
            "superuser-access-option"  => "-addsuperuseropt",
        }
    );

    $apiset->export_policy_add_rule(%$exp_pol_rule_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        policy            => $exp_pol_rule_args->{-policyName},
        vserver           => $exp_pol_rule_args->{-vservername},
        ruleindex         => $exp_pol_rule_args->{-ruleindex},
        command_interface => $command_interface
    );

} ## end sub _add_cmode_gui

=head2 delete

    NACL::C::SysManager::ExportPolicyRule->delete( 
									command_interface => $ci,
									policy            => $policy_name,
									vserver           => $vserver_name
									ruleindex        => $index,);

                            or

  $exportPolicyRule_object->delete(...);

(Class or instance method)This method deletes a exportPolicyRule element.
Uses CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< policy => $string >>

(Required for class method, Not Applicable for instance method)
Name of exportPolicy.

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode , Not Applicable for 7Mode)The name of the vserver
in which the exportPolicyRule is present.

=item C<< "ruleindex"=> \$scalar >>

(Required for class method, Not Applicable for instance method)
Rule Index No.

=back

=back

=cut

####################### GUI Delete ###############################
sub delete {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => { _optional_scalars qw(policy), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_delete_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

####################### GUI CMode Delete ###############################
sub _delete_cmode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);
    my $apiset     = $opts{apiset};

    my $exp_pol_rule_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "policy"    => "-policyName",
            "ruleindex" => "-ruleName",
            "vserver"   => "-vservername"
        }
    );

    $apiset->export_policy_rule_delete(%$exp_pol_rule_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_gui

=head2 modify

  NACL::C::SysManager::ExportPolicyRule->modify(
									command_interface => $ci,
									policy            => $vol_name,
									vserver           => $vserver_name,
									ruleindex         => $index,...);

                            or

  $exportPolicyRule_object->modify(...);

(Class or instance method) Modifies a exportPolicyRule element.  
Uses CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< policy => $string >>

(Required) Name of ExportPolicy.

=item C<< vserver => $string >>

The name of the vserver in which the exportPolicyRule is present.

=item C<< "client-specification" => $string >>

(Optional) Client Specification

=item C<< "ruleindex" => $string >>

(Required) Index no.

=item C<< "access-protocol" => $string >>

(Optional) Access protocols

=item C<< "allow-read-only-access" => $string >>

(Optional) Enable read only access

=item C<< "read-only-access-option" => $string >>

(Optional) Read only access options

=item C<< "allow-read-write-access" => $string >>

(Optional) Enable read write access

=item C<< "read-write-access-option" => $string >>

(Optional) Read write access options

=item C<< "allow-superuser-access" => $string >>

(Optional) Enable Superuser access

=item C<< "superuser-access-option" => $string >>

(Optional) Superuser access options

=back

=back

=cut

####################### GUI Modify ###############################
sub modify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw (policy client-specification ruleindex access-protocol
                    allow-read-only-access read-only-access-option
                    allow-read-write-access read-write-access-option
                    allow-superuser-access superuser-access-option)
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_modify_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

####################### GUI CMode Modify ###############################
sub _modify_cmode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);
    my $apiset     = $opts{apiset};

    my $exp_pol_rule_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "vserver"                  => "-vservername",
            "policy"                   => "-policyName",
            "client-specification"     => "-clientspec",
            "ruleindex"                => "-ruleName",
            "access-protocol"          => "-protocol",
            "allow-read-only-access"   => "-readonly",
            "read-only-access-option"  => "-addreadonlyopt",
            "allow-read-write-access"  => "-readwrite",
            "read-write-access-option" => "-addreadwriteopt",
            "allow-superuser-access"   => "-superuser",
            "superuser-access-option"  => "-addsuperuseropt",
        }
    );

    $apiset->export_policy_modify_rule(%$exp_pol_rule_args);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_gui

=head2 refresh

  NACL::C::SysManager::ExportPolicyRule->refresh(
									command_interface => $ci,
									vserver           => $vserver_name);

                            or

  $exportPolicyRule_object->refresh();

(Class or instance method) This method refreshs a exportPolicy page.
Uses CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
The name of the vserver in which the exportPolicyRule is present.

=back

=back

=cut

####################### GUI Refresh ###############################
sub refresh {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@_, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_refresh_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub refresh

####################### GUI CMode Refresh ###############################
sub _refresh_cmode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);
    my $apiset     = $opts{apiset};

    my $exp_pol_rule_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "vserver" => "-vservername" }
    );

    $apiset->export_policy_refresh(%$exp_pol_rule_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_cmode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return (policy    => { type => SCALAR },
			ruleindex => { type => SCALAR },
			vserver   => { type => SCALAR },);
} ## end sub _primary_keys_validate_spec

1;
