# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary IscsiInitiatorSecurity Module
## @author Neethu.George@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::IscsiInitiatorSecurity

=head1 DESCRIPTION

C<NACL::C::SysManager::IscsiInitiatorSecurity> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>.  It represents access
to an ONTAP IscsiInitiatorSecurity present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::IscsiInitiatorSecurity|lib-NACL-CS-SysManager-IscsiInitiatorSecurity-pm>, 
which represents the State Info of an ONTAP IscsiInitiatorSecurity present on the System Manger GUI Screen. 

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2  initiator

Initiator security name, used to identify it uniquely. (Initiator Security
names are assumed to be unique within a node or within a cluster in 7mode
and in a vserver in Cmode)

=head2 vserver

(Required for CMode, Not Applicable for 7Mode) The name of the vserver
in which the initiator is present.

=cut

package NACL::C::SysManager::IscsiInitiatorSecurity;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::IscsiInitiatorSecurity;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NACL::C::Job;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $security_obj  = NACL::C::SysManager::IscsiInitiatorSecurity->new(command_interface => $ci,
																	   vserver           => $vserver_name,
																	   initiatior        => $initiator,);

Return a new Initiator security component object with the given attributes.

=cut

####################### Method Maker ###############################

use Class::MethodMaker [
    scalar => 'initiator',
    scalar => 'vserver',
];

=head2 add

	my $security_obj  = 
		NACL::C::SysManager::IscsiInitiatorSecurity->add(command_interface => $ci,
														 initiator         => $initiator);

(Class method) Add initiator security , and  return a new
security  component object that refers to it.
Uses a CMode GUI or 7mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for CMode, Not Applicable for 7Mode)The name of the vserver.

=item C<< initiator  => $string >>

(Required) Initiator security name.

=item C<< security-type => $string >>

 (Optional) Initiator security type.

=item C<< chap-type  => $string >>

(Optional) Initiator chap type

=item C<< username  => $string >>

(Optional) Initiator security username.
 
=item C<< password  => $string >>

(Optional) Initiator security password.

=item C<< confirm-password  => $string >>

(Optional) Initiator security confirm password.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw ( security-type chap-type username
                    password confirm-password)
            )
        }
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_add_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_add_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub add

####################### GUI 7Mode add ###############################
sub _add_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $initiator_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "initiator"        => "-initiator",
            "security-type"    => "-securitytype",
            "chap-type"        => "-chaptype",
            username           => "-username",
            password           => "-password",
            "confirm-password" => "-confirmpassword"
        }
    );

    $apiset->add_initiator_security(%$initiator_args);

    $Log->exit() if $may_exit;

    return $pkg->new(
        initiator         => $initiator_args->{-initiator},
        command_interface => $command_interface
    );
} ## end sub _add_7mode_gui

####################### GUI CMode add###############################
sub _add_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $initiator_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "vserver"          => "-vservername",
            "initiator"        => "-initiator",
            "security-type"    => "-securitytype",
            "chap-type"        => "-chaptype",
            username           => "-username",
            password           => "-password",
            "confirm-password" => "-confirmpassword"
        }
    );

    $apiset->add_initiator_security(%$initiator_args);

    $Log->exit() if $may_exit;

    return $pkg->new(
        initiator         => $initiator_args->{-initiator},
        vserver           => $initiator_args->{-vservername},
        command_interface => $command_interface
    );

} ## end sub _add_cmode_gui

=head2 edit

	NACL::C::SysManager::IscsiInitiatorSecurity->edit(command_interface => $ci,
													  initiator         => $initiator);

                            or

	$security_obj->edit(...);

(Class or instance method) This method edits Initiator security.
Uses a 7Mode GUI or CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for CMode, Not Applicable for 7Mode)The name of the vserver.

=item C<< initiator  => $string >>

(Required) Initiator security name.

=item C<< security-type => $string >>

 (Optional) Initiator security type.

=item C<< inbound-username  => $string >>

(Optional) Inbound username.
 
=item C<< inbound-password  => $string >>

(Optional) Inbound password.

=item C<< inbound-confrmpassword  => $string >>

(Optional) Inbound confirm password.

=item C<< outbound-username  => $string >>

(Optional) Outbound username.
 
=item C<< outbound-password  => $string >>

(Optional) Outbound password.

=item C<< outbound-confrmpassword  => $string >>

(Optional) Outbound confirm password.

=back

=back

=cut

sub edit {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw ( security-type inbound-username
                    inbound-password inbound-confrmpassword
                    outbound-username outbound-password
                    outbound-confrmpassword)
            )
        }
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_edit_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_edit_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub edit

####################### GUI 7Mode edit ###############################
sub _edit_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $initiator_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "initiator"               => "-initiator",
            "security-type"           => "-securitytype",
            "inbound-username"        => "-inboundusername",
            "inbound-password"        => "-inboundpassword",
            "inbound-confrmpassword"  => "-inboundconfirmpassword",
            "outbound-username"       => "-outboundusername",
            "outbound-password"       => "-outboundpassword",
            "outbound-confrmpassword" => "-outboundconfirmpassword"
        }
    );

    $apiset->edit_initiator_security(%$initiator_args);
    $Log->exit() if $may_exit;
} ## end sub _edit_7mode_gui

####################### GUI CMode edit ###############################
sub _edit_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $initiator_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "vserver"                 => "-vservername",
            "initiator"               => "-initiator",
            "security-type"           => "-securitytype",
            "inbound-username"        => "-inboundusername",
            "inbound-password"        => "-inboundpassword",
            "inbound-confrmpassword"  => "-inboundconfirmpassword",
            "outbound-username"       => "-outboundusername",
            "outbound-password"       => "-outboundpassword",
            "outbound-confrmpassword" => "-outboundconfirmpassword"
        }
    );

    $apiset->edit_initiator_security(%$initiator_args);
    $Log->exit() if $may_exit;
} ## end sub _edit_cmode_gui

=head2 edit_default

	NACL::C::SysManager::IscsiInitiatorSecurity->edit_default(command_interface => $ci,
															  initiator         => $initiator);

                            or

	$security_obj->edit_default(...);

(Class or instance method) This method edits the default initiator security settings.
Uses a 7Mode GUI or CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for CMode, Not Applicable for 7Mode)The name of the vserver.

=item C<< security-type => $string >>

 (Optional) Initiator security type.

=item C<< inbound-username  => $string >>

(Optional) Inbound username.
 
=item C<< inbound-password  => $string >>

(Optional) Inbound password.

=item C<< inbound-confrmpassword  => $string >>

(Optional) Inbound confirm password.

=item C<< outbound-username  => $string >>

(Optional) Outbound username.
 
=item C<< outbound-password  => $string >>

(Optional) Outbound password.

=item C<< outbound-confrmpassword  => $string >>

(Optional) Outbound confirm password.

=back

=back

=cut

sub edit_default {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw ( security-type inbound-username
                    inbound-password inbound-confrmpassword
                    outbound-username outbound-password
                    outbound-confrmpassword)
            )
        }
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_edit_default_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_edit_default_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub edit_default

####################### GUI 7Mode edit_default ###############################
sub _edit_default_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $initiator_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "security-type"           => "-securitytype",
            "inbound-username"        => "-inboundusername",
            "inbound-password"        => "-inboundpassword",
            "inbound-confrmpassword"  => "-inboundconfirmpassword",
            "outbound-username"       => "-outboundusername",
            "outbound-password"       => "-outboundpassword",
            "outbound-confrmpassword" => "-outboundconfirmpassword"
        }
    );

    $apiset->edit_default_initiator_security(%$initiator_args);
    $Log->exit() if $may_exit;
} ## end sub _edit_default_7mode_gui

####################### GUI CMode edit_default###############################
sub _edit_default_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $initiator_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "vserver"                 => "-vservername",
            "initiator"               => "-initiator",
            "security-type"           => "-securitytype",
            "inbound-username"        => "-inboundusername",
            "inbound-password"        => "-inboundpassword",
            "inbound-confrmpassword"  => "-inboundconfirmpassword",
            "outbound-username"       => "-outboundusername",
            "outbound-password"       => "-outboundpassword",
            "outbound-confrmpassword" => "-outboundconfirmpassword"
        }
    );

    $apiset->edit_default_initiator_security(%$initiator_args);
    $Log->exit() if $may_exit;
} ## end sub _edit_default_cmode_gui

=head2 refresh

	NACL::C::SysManager::IscsiInitiatorSecurity->refresh(command_interface => $ci,
													     initiator         => $initiator);

                            or

	$security_obj->refresh(...);
 
(Class or instance method) This method refreshes initiator security page.
Uses a 7Mode GUI or CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode, Not Applicable for 7Mode)The name of the vserver

=back

=back

=cut

sub refresh {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_refresh_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_refresh_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub refresh

####################### GUI 7Mode Refresh ###############################
sub _refresh_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $initiator_args = $pkg_or_obj->_hash_copy( source => \%opts, );

    $apiset->refresh_initiator_security(%$initiator_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_7mode_gui

####################### GUI CMode Refresh ###############################
sub _refresh_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $initiator_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "vserver" => "-vservername" }
    );

    $apiset->refresh_initiator_security(%$initiator_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_cmode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return (
        initiator => { type => SCALAR },
        vserver   => { type => SCALAR | UNDEF, optional => 1 },
    );
} ## end sub _primary_keys_validate_spec

1;
