# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary IscsiService Component Module
## @author vikas.p@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::IscsiService

=head1 DESCRIPTION

C<NACL::C::SysManager::IscsiService> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>.  It represents access
to an ONTAP IscsiService present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::IscsiService|lib-NACL-CS-SysManager-IscsiService-pm>, which represents
the past state of an ONTAP IscsiService.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

(Required for CMode, Not Applicable for 7Mode) The name of the vserver
in which the IscsiService is present.

=cut

package NACL::C::SysManager::IscsiService;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::IscsiService;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $IscsiService = NACL::C::SysManager::IscsiService->new(command_interface => $ci,
												"target-node-name" => $node_name,
												vserver            => $vserver_name);

Return a new IscsiService component object with the given attributes.

=cut

####################### Method Maker ###############################

use Class::MethodMaker [ scalar => 'vserver', ];

=head2 edit

	my $IscsiService_object  = 
		NACL::C::SysManager::IscsiService->edit(command_interface  => $ci,
												"target-node-name" => $node_name,
												vserver            => $vserver_name);

(Class method) Request edit of a Iscsi service element, and  return a new
iscsi service component object that refers to it.
Uses a 7Mode GUI or CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for CMode, Not Applicable for 7Mode)The name of the vserver
in which the IscsiService is present.

=item C<< "target-node-name" => $string >>

(Optional) Target Node Name.

=item C<< "target-alias" => $string >>

(Optional) Target Alias

=back

=back

=cut

####################### GUI Edit ###############################
sub edit {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params => \@args,
        additional_spec =>
            { _optional_scalars qw(target-node-name target-alias), },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_edit_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_edit_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub edit

####################### GUI 7Mode Edit ###############################
sub _edit_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $iscsi_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "target-node-name" => "-nodename",
            "target-alias"     => "-alias",
        }
    );

    $apiset->iscsiservice_edit(%$iscsi_args);

    $Log->exit() if $may_exit;

    return $pkg->new( command_interface => $command_interface );
} ## end sub _edit_7mode_gui

####################### GUI CMode Edit ###############################
sub _edit_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $iscsi_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "vserver"          => "-vservername",
            "target-node-name" => "-nodename",
            "target-alias"     => "-alias",
        }
    );

    $apiset->iscsiservice_edit(%$iscsi_args);

    $Log->exit() if $may_exit;

    return $pkg->new(
        vserver           => $iscsi_args->{-vservername},
        command_interface => $command_interface
    );
} ## end sub _edit_cmode_gui

=head2 start

	NACL::C::SysManager::IscsiService->start(command_interface => $ci,
											 vserver           => $vserver_name,);


(Class method) This method starts a IscsiService element,and  return a new
iscsi service component object that refers to it.
Uses a CMode GUI or 7Mode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver =>$string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode , Not Applicable for 7Mode)The name of the vserver
in which the IscsiService is present.

=item C<< "status" => $string >>

(Optional) Status of the Service to be set

=back

=back

=cut

####################### GUI Start ###############################
sub start {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => { _optional_scalars(qw(status)), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_start_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_start_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub start

####################### GUI 7Mode Start ###############################
sub _start_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $iscsi_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "status" => "-service", }
    );

    $iscsi_args->{'-service'} = "start";

    $apiset->iscsiservice(%$iscsi_args);
    $Log->exit() if $may_exit;

    return $pkg_or_obj->new( command_interface => $command_interface );
} ## end sub _start_7mode_gui

####################### GUI CMode Start ###############################
sub _start_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $iscsi_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "vserver" => "-vservername",
            "status"  => "-service",
        }
    );

    $iscsi_args->{'-service'} = "start";

    $apiset->iscsiservice(%$iscsi_args);
    $Log->exit() if $may_exit;

    return $pkg_or_obj->new(
        vserver           => $iscsi_args->{-vservername},
        command_interface => $command_interface
    );
} ## end sub _start_cmode_gui

=head2 stop

	NACL::C::SysManager::IscsiService->stop(command_interface => $ci,
									        vserver           => $vserver_name,);

                            or

	$IscsiService_object->stop(...);

(Class or instance method) This method stops Iscsi Service.
Uses a CMode GUI or 7Mode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode , Not Applicable for 7Mode)The name of the vserver
in which the IscsiService is present.

=item C<< "status" => $string >>

(Optional) Status of the Service to be set


=back

=back

=cut

####################### GUI Stop ###############################
sub stop {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => { _optional_scalars(qw(status)), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_stop_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_stop_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub stop

####################### GUI 7Mode Stop ###############################
sub _stop_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $iscsi_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "status" => "-service", }
    );

    $iscsi_args->{'-service'} = "stop";

    $apiset->iscsiservice(%$iscsi_args);
    $Log->exit() if $may_exit;
} ## end sub _stop_7mode_gui

####################### GUI CMode Stop ###############################
sub _stop_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $iscsi_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "vserver" => "-vservername",
            "status"  => "-service",
        }
    );

    $iscsi_args->{'-service'} = "stop";

    $apiset->iscsiservice(%$iscsi_args);
    $Log->exit() if $may_exit;
} ## end sub _stop_cmode_gui

=head2 refresh

	NACL::C::SysManager::IscsiService->refresh(command_interface => $ci,
									   vserver           => $vserver_name,...);

                            or

	$IscsiService_object->refresh();

(Class or instance method) This method refreshs IscsiService page.
Uses a 7Mode GUI or CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode, Not Applicable for 7Mode)The name of the vserver
in which the IscsiService is present.

=back

=back

=cut

####################### GUI Refresh ###############################
sub refresh {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_refresh_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_refresh_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub refresh

####################### GUI 7Mode Refresh ###############################
sub _refresh_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $iscsi_args = $pkg_or_obj->_hash_copy( source => \%opts, );

    $apiset->iscsiservice_refresh(%$iscsi_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_7mode_gui

####################### GUI CMode Refresh ###############################
sub _refresh_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $iscsi_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "vserver" => "-vservername" }
    );

    $apiset->iscsiservice_refresh(%$iscsi_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_cmode_gui

=head2 interface_enable

	NACL::C::SysManager::IscsiService->interface_enable(command_interface  => $ci,
														vserver            => $vserver_name,);

                            or

	$IscsiService_object->interface_enable(...);

(Class or instance method) This method enables a Iscsi Interface element.
Uses a CMode GUI or 7Mode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver =>$string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode , Not Applicable for 7Mode)The name of the vserver
in which the IscsiService is present.

=item C<< "status" => $string >>

(Optional) the status of the interface

=item C<< "interface" => $string >>

(Optional) the interface to be enabled

=back

=back

=cut

####################### GUI Enable iSCSI Interface ###############################
sub interface_enable {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => { _optional_scalars(qw(status interface)), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_interface_enable_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_interface_enable_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub interface_enable

####################### GUI 7Mode Enable iSCSI Interface ###############################
sub _interface_enable_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $iscsi_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "status"    => "-status",
            "interface" => "-interfacename",
        }
    );

    $iscsi_args->{'-status'} = "enable";

    $apiset->iscsiinterface_status(%$iscsi_args);
    $Log->exit() if $may_exit;
} ## end sub _interface_enable_7mode_gui

####################### GUI CMode Enable iSCSI Interface ###############################
sub _interface_enable_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $iscsi_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "vserver"   => "-vservername",
            "status"    => "-status",
            "interface" => "-interfacename",
        }
    );

    $iscsi_args->{'-status'} = "enable";

    $apiset->iscsiinterface_status(%$iscsi_args);
    $Log->exit() if $may_exit;
} ## end sub _interface_enable_cmode_gui

=head2 interface_disable

	NACL::C::SysManager::IscsiService->interface_disable(command_interface => $ci,
														 vserver           => $vserver_name,);

                            or

	$IscsiService_object->interface_disable(...);

(Class or instance method) This method disables Iscsi Interface.
Uses a CMode GUI or 7Mode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode , Not Applicable for 7Mode)The name of the vserver
in which the IscsiService is present.

=item C<< "status" => $string >>

(Optional) the status of the interface

=item C<< "interface" => $string >>

(Optional) the interface to be enabled

=back

=back

=cut

####################### GUI Disable Interface ###############################
sub interface_disable {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => { _optional_scalars(qw(status interface)), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_interface_disable_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_interface_disable_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub interface_disable

####################### GUI 7Mode Disable iSCSI Interface ###############################
sub _interface_disable_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $iscsi_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "status"    => "-status",
            "interface" => "-interfacename",
        }
    );

    $iscsi_args->{'-status'} = "disable";

    $apiset->iscsiinterface_status(%$iscsi_args);
    $Log->exit() if $may_exit;
} ## end sub _interface_disable_7mode_gui

####################### GUI CMode Diable iSCSI Interface ###############################
sub _interface_disable_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $iscsi_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "vserver"   => "-vservername",
            "status"    => "-status",
            "interface" => "-interfacename",
        }
    );

    $iscsi_args->{'-status'} = "disable";

    $apiset->iscsiinterface_status(%$iscsi_args);
    $Log->exit() if $may_exit;
} ## end sub _interface_disable_cmode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return ( vserver => { type => SCALAR | UNDEF, optional => 1 }, );
}

1;
