# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary Lun Component Module
## @author vikas.p@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::Lun

=head1 DESCRIPTION

C<NACL::C::SysManager::Lun> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>. It represents access
to an ONTAP Lun.  A related class is
L<NACL::CS::SysManager::Lun|lib-NACL-CS-SysManager-Lun-pm>, which represents
the past state of an ONTAP Lun.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 lun

The name of the lun, used to identify it uniquely.  (Lun
names are assumed to be unique within a node or within a cluster in 7mode
and in a vserver in Cmode)

=head2 vserver

(Required for CMode, Not Applicable for 7Mode) The name of the vserver
in which the Lun is present.

=cut

package NACL::C::SysManager::Lun;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::Lun;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

	my $Lun = NACL::C::SysManager::Lun->new(command_interface => $ci,
										    vserver           => $vserver,
											lun               => $lun);

Return a new Lun component object with the given attributes.

=cut

####################### Method Maker ###############################

use Class::MethodMaker [
    scalar => 'lun',
    scalar => 'vserver',
];

=head2 create

    my $Lun_object	= NACL::C::SysManager::Lun->create(command_interface => $ci,
													   vserver           => $vserver,
													   lun 		         => $lun_name...);
   
(Class method)
This method is used to create a new LUN.
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< lun => $lun >>

(Required)Name of the LUN

=item C<< vserver => $vserver >>

(Required in CMode and if path is not provided, not applicable in 7Mode)
Vserver on which the volume is present

=item C<< volume => $volume >>

(Optional) the Volume in which LUN has to be created

=item C<< description => $string >>

(Optional) Description for the Lun to be created

=item C<< size => $string >>

(Optional) Size of the Lun

=item C<< unit => $string >>

(Optional) Unit of the Lun size

=item C<< "space-guarantee" => $string >>

(Optional) Enable space guaranetee on lun

=item C<< "volume-option" => $string >>

(Optional) Volume option

=item C<< "file-path" => $string >>

(Optional) File path for lun creation

=item C<< aggregate => $string >>

(Optional) Aggregate on which volume has to be created

=item C<< "new-group-id" => $string >>

(Optional) New Igroup ID

=item C<< "thin-provisioning" => $string >>

(Optional) Enable Thin Provisioning 

=item C<< "new-initiator" => $arrayref >>

(Optional) Create new Initiator

=item C<< "exist-initiator" => $arrayref >>

(Optional) Use Existing Initiators

=item C<< "exist-group-id" => $arrayref >>

(Optional) Existing Igroup IDs

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "new-initiator"   => { type => ARRAYREF, optional => 1 },
            "exist-initiator" => { type => ARRAYREF, optional => 1 },
            "exist-group-id"  => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( description type size unit space-guarantee volume-option
                    file-path aggregate volume new-group-id thin-provisioning )
            ),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_create_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_create_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub create

####################### GUI CMode Create ###############################
sub _create_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $lun_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "lun"             => "-lunname",
            "vserver"         => "-vservername",
            "type"            => "-type",
            "description"     => "-discrb",
            "size"            => "-size",
            "unit"            => "-sizeunit",
            "space-guarantee" => "-spaceguarantee",
            "volume-option"   => "-vol",
            "aggregate"       => "-new_aggrname",
            "volume"          => "-new_volname",
            "file-path"       => "-path",
            "new-initiator"   => "-ini_new",
            "new-group-id"    => "-igroupid_new",
            "exist-initiator" => "-ini_existing",
            "exist-group-id"  => "-igroupid_existing",
        }
    );

    $apiset->lun_create(%$lun_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        lun               => $lun_args->{-lunname},
        vserver           => $lun_args->{-vservername},
        command_interface => $command_interface
    );
} ## end sub _create_cmode_gui

####################### GUI 7Mode Create ###############################
sub _create_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $lun_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "lun"               => "-lunname",
            "description"       => "-discrb",
            "type"              => "-type",
            "size"              => "-size",
            "unit"              => "-sizeunit",
            "space-guarantee"   => "-spaceguarantee",
            "volume-option"     => "-vol",
            "aggregate"         => "-new_aggrname",
            "volume"            => "-new_volname",
            "file-path"         => "-path",
            "new-initiator"     => "-ini_new",
            "new-group-id"      => "-igroupid_new",
            "exist-initiator"   => "-ini_existing",
            "exist-group-id"    => "-igroupid_existing",
            "thin-provisioning" => "-thinprovisioned",
        }
    );

    $apiset->lun_create(%$lun_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        lun               => $lun_args->{-lunname},
        command_interface => $command_interface
    );
} ## end sub _create_7mode_gui

=head2 delete

    NACL::C::SysManager::Lun->delete(command_interface => $ci,
									 vserver           => $vserver,
									 lun               => $lun_name,...);
    or

    $Lun_object->delete(...);
 
(Class or instance method)
This method is used to delete the lun.
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "lun" => $lun >>

(Required for class method, Not Applicable for instance method)
Name of the Lun.

=item C<< vserver => $vserver >>

(Required for class method, Not Applicable for instance method, Not Applicable
in 7Mode or if path is provided)
Name of the Vserver

=item C<< "parent-delete" => $string >>

(Optional) Enable Parent Volume delete

=item C<< "status" => $string >>

(Optional) Status of the Lun to be set before deletion

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            { _optional_scalars(qw ( lun parent-delete status)) },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_delete_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_delete_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

####################### GUI CMode Delete ###############################
sub _delete_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $lun_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "lun"           => "-lunname",
            "vserver"       => "-vservername",
            "status"        => "-status",
            "parent-delete" => "-parent_delete",
        }
    );

    $apiset->lun_delete(%$lun_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_gui

####################### GUI 7Mode Delete ###############################
sub _delete_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $lun_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "lun"           => "-lunname",
            "status"        => "-status",
            "parent-delete" => "-deleteParentVol",
        }
    );

    $apiset->lun_delete(%$lun_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_gui

=head2 modify

    NACL::C::SysManager::Lun->modify(command_interface => $ci,
									 lun               => $lun,
									 vserver           => $vserver,...);
	                             
    (or)

    $Lun_object->modify(...);

(Class or instance method)
This method is used to modify a LUN.
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "lun" => $lun >>

(Required for class method, Not Applicable for instance method)
Name of the Lun.

=item C<< "qtree" => $qtree >>

(Required for class method, Not Applicable for instance method)

=item C<< "volume" => $volume >>

(Required for class method, Not Applicable for instance method)

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)
(Required for CMode, Not Applicable for 7Mode)
The name of the vserver

=item C<< "initiator-group-name" => $arrayref >>

(Optional) List of Initiator Group nam

=item C<< "initiator-group-id" => $arrayref >>

(Optional) List of Initiator Group IDs

=item C<< "exist-initiator" => $arrayref >>

(Optional) List of Existing Initiators

=item C<< "size" => $string >>

(Optional) Change in Size

=item C<< "unit" => $string >>

(Optional) Change in Size Unit

=item C<< "volume" => $string >>

(Optional) Volume to be modified in

=item C<< "rename" => $string >>

(Optional) Rename of the Lun

=item C<< "auto-grow" => $string >>

(Optional) Enable Auto Grow Option

=item C<< "initiator-group" => $string >>

{Optional) Intiator Groups

=item C<< "operating-system" => $string >>

{Optional) Operating System

=item C<< "type" => $string >>

(Optional) Type of the lun

=item C<< "alua" => $string >>

=item C<< "description" => $string >>

(Optional) Enable ALUA

=item C<< "space-guarantee" => $string >>

(Optional) Enable Space Guarantee.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "initiator-group-name" => { type => ARRAYREF, optional => 1 },
            "initiator-group-id"   => { type => ARRAYREF, optional => 1 },
            "exist-initiator"      => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( size unit volume lun-rename auto-grow
                    initiator-group operating-system type alua description
                    space-guarantee portset )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_modify_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_modify_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

####################### GUI CMode Modify ###############################
sub _modify_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $lun_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "lun"                  => "-lunname",
            "vserver"              => "-vservername",
            "lun-rename"           => "-lunrename",
            "description"          => "-discrb",
            "size"                 => "-size",
            "unit"                 => "-sizeunit",
            "space-guarantee"      => "-spaceguarantee",
            "auto-grow"            => "-autogrow",
            "initiator-group"      => "-igroup",
            "initiator-group-name" => "-igroupname",
            "operating-system"     => "-os",
            "type"                 => "-type",
            "alua"                 => "-alua",
            "exist-initiator"      => "-ini_existing",
            "initiator-group-id"   => "-igroupid",
            "portset"              => "-portset",
        }
    );
    $apiset->lun_edit(%$lun_args);

    if ( ( defined $lun_args->{'-lunrename'} ) && ( ref $pkg_or_obj ) ) {
        $pkg_or_obj->lun( $opts{'lun-rename'} );
    }

    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_gui

####################### GUI 7Mode Modify ###############################
sub _modify_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $lun_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "lun"                  => "-lunname",
            "lun-rename"           => "-lunrename",
            "description"          => "-discrb",
            "size"                 => "-size",
            "unit"                 => "-sizeunit",
            "space-guarantee"      => "-spaceguarantee",
            "auto-grow"            => "-autogrow",
            "initiator-group"      => "-igroup",
            "initiator-group-name" => "-igroupname",
            "operating-system"     => "-os",
            "type"                 => "-type",
            "alua"                 => "-alua",
            "exist-initiator"      => "-ini_existing",
            "initiator-group-id"   => "-igroupid",
        }
    );

    $apiset->lun_edit(%$lun_args);

    if ( ( defined $lun_args->{'-lunrename'} ) && ( ref $pkg_or_obj ) ) {
        $pkg_or_obj->lun( $opts{'lun-rename'} );
    }

    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_gui

=head2 online

	NACL::C::SysManager::Lun->online(command_interface => $ci,
									 lun               => $lun_name,
									 vserver           => $vserver_name,...);

                            or

	$Lun_object->online(...);

(Class or instance method) This method onlines a Lun element.
Uses a CMode GUI or 7Mode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< lun => $string >>

(Required for class method, Not Applicable for instance method)
Name of Lun.

=item C<< vserver=>$string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode , Not Applicable for 7Mode)The name of the vserver
in which the volume is present.

=item C<< status => $string >>

(Optional) Status of the Lun to be set to.

=back

=back

=cut

sub online {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => { _optional_scalars(qw (status)), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_online_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_online_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub online

####################### GUI CMode Online ###############################
sub _online_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $lun_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "lun"     => "-lunname",
            "vserver" => "-vservername",
            "status"  => "-status",
        }
    );

    $lun_args->{'-status'} = "online";

    $apiset->lun_status(%$lun_args);
    $Log->exit() if $may_exit;
} ## end sub _online_cmode_gui

####################### GUI 7Mode Online ###############################
sub _online_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $lun_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "lun"    => "-lunname",
            "status" => "-status",
        }
    );

    $lun_args->{'-status'} = "online";

    $apiset->lun_status(%$lun_args);
    $Log->exit() if $may_exit;
} ## end sub _online_7mode_gui

=head2 offline

	NACL::C::SysManager::Lun->offline(command_interface => $ci,
									  lun               => $lun_name,
									  vserver           => $vserver_name,...);

                            or

	$Lun_object->offline(...);

(Class or instance method) This method onlines a volume element.
Uses a CMode GUI or 7Mode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< lun => $string >>

(Required for class method, Not Applicable for instance method)
Name of Lun.

=item C<< vserver=>$string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode , Not Applicable for 7Mode)The name of the vserver
in which the volume is present.

=item C<< status => $string >>


=back

=back

=cut

sub offline {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => { _optional_scalars(qw (status)), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_offline_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_offline_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub offline

####################### GUI CMode Offline ###############################
sub _offline_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $lun_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "lun"     => "-lunname",
            "vserver" => "-vservername",
            "status"  => "-status",
        }
    );

    $lun_args->{'-status'} = "offline";

    $apiset->lun_status(%$lun_args);
    $Log->exit() if $may_exit;
} ## end sub _offline_cmode_gui

####################### GUI 7Mode Offline ###############################
sub _offline_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $lun_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "lun"    => "-lunname",
            "status" => "-status",
        }
    );

    $lun_args->{'-status'} = "offline";

    $apiset->lun_status(%$lun_args);
    $Log->exit() if $may_exit;
} ## end sub _offline_7mode_gui

=head2 refresh

	NACL::C::SysManager::Lun->refresh(command_interface => $ci,
								      vserver           => $vserver_name,...);

                            or

	$Lun_object->refresh(...);

(Class or instance method) This method refreshs a lun page.
Uses a CMode GUI or 7Mode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$string >>

(Required for CMode , Not Applicable for 7Mode)The name of the vserver
in which the volume is present.


=back

=back

=cut

sub refresh {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_refresh_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_refresh_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub refresh

####################### GUI 7Mode Refresh ###############################
sub _refresh_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $lun_args = $pkg_or_obj->_hash_copy( source => \%opts, );

    $apiset->lun_refresh(%$lun_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_7mode_gui

####################### GUI CMode Refresh ###############################
sub _refresh_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $lun_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "vserver" => "-vservername", }
    );

    $apiset->lun_refresh(%$lun_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_cmode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return (
        'vserver' => { type => SCALAR | UNDEF, optional => 1 },
        'lun'     => { type => SCALAR },
    );
} ## end sub _primary_keys_validate_spec

1;
