# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary LunClone Component Module
## @author vikas.p@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::LunClone

=head1 DESCRIPTION

C<NACL::C::SysManager::LunClone> is a derived class of
L<NACL::C::SysManager::Lun|lib-NACL-C-SysManager-Lun-pm>. As this is a
subclass of Lun, all the functionality of Lun is available here.  A related class is
L<NACL::CS::SysManager::LunClone|lib-NACL-CS-SysManager-LunClone-pm>, which represents
the State Info of an ONTAP LunClone present on the System Manger GUI Screen.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

(Required for CMode, Not Applicable for 7Mode) The name of the vserver
in which the lun clone is present.

=head2 lun

The name of the lun clone, used to identify it uniquely.  (Lun clone
names are assumed to be unique within a node or within a cluster in 7mode
and in a vserver in Cmode)

=cut

package NACL::C::SysManager::LunClone;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::LunClone;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::SysManager::Lun';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $LunClone = NACL::C::SysManager::LunClone->new(command_interface => $ci,
													vserver           =>$vserver,
													clone             =>$clone,...);

Return a new LunClone component object with the given attributes.

=cut

####################### Method Maker ###############################

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'lun',
];

=head2 create

    my $LunClone_object	= NACL::C::LunClone->create(
                                command_interface => $ci,
	                            vserver           => $vserver,
	                            clone             => $Clone,...);
   
(Class method) This method is used to creates a new Clone element, and 
return a new LunClone component object that refers to it.
Uses a CMode GUI or CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for CMode, Not Applicable for 7Mode) The name of the vserver
in which the volume is present.

=item C<< "Clone" => $Clone >>

(Required) Name of the Clone.

=item C<< "lun" => $string >>

(Required) Name of the Lun for which clone has to be created.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => { _optional_scalars(qw(clone)), },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_create_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_create_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub create

####################### GUI CMode Create ###############################
sub _create_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $lun_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "lun"     => "-lunname",
            "vserver" => "-vservername",
            "clone"   => "-clonename",
        }
    );

    $apiset->lun_clone(%$lun_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        lun               => $lun_args->{-clonename},
        vserver           => $lun_args->{-vservername},
        command_interface => $command_interface
    );
} ## end sub _create_cmode_gui

####################### GUI 7Mode Create ###############################
sub _create_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $lun_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "lun"   => "-lunname",
            "clone" => "-clonename",
        }
    );

    $apiset->lun_clone(%$lun_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        lun               => $lun_args->{-clonename},
        command_interface => $command_interface
    );
} ## end sub _create_7mode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return (
        'vserver' => { type => SCALAR | UNDEF, optional => 1 },
        'lun'     => { type => SCALAR },
    );
} ## end sub _primary_keys_validate_spec

1;
