# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary LunIgroup Component Module
## @author vikas.p@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::LunIgroup

=head1 DESCRIPTION

C<NACL::C::SysManager::LunIgroup> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>. It represents access
to an ONTAP LunIgroup present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::LunIgroup|lib-NACL-CS-SysManager-LunIgroup-pm>, which represents
the past state of an ONTAP LunIgroup.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 igroup

The name of the initiator group, used to identify it uniquely. (Igroup
names are assumed to be unique within a node or within a cluster in 7mode
and in a vserver in Cmode)

=head2 vserver

(Required for CMode, Not Applicable for 7Mode) The name of the vserver
in which the Igroup is present.

=cut

package NACL::C::SysManager::LunIgroup;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::LunIgroup;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $LunIgroup = NACL::C::SysManager::LunIgroup->new(command_interface => $ci,
													  igroup            => $igroup,
													  vserver           => $vserver,...);

Return a new LunIgroup component object with the given attributes.

=cut

####################### Method Maker ###############################

use Class::MethodMaker [
    scalar => 'igroup',
    scalar => 'vserver',
];

=head2 create

    my $LunIgroup_object	= NACL::C::SysManager::LunIgroup->create(
									command_interface => $ci,
									igroup            => $igroup,
									vserver 		  => $vserver,...);
   
(Class method)This method is used to create a new initiator group element,
and return a new Initiator Group object that refers to it.
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< igroup => $string >>

(Required) Name of Initiator Group.

=item C<< vserver =>$string >>

(Required for CMode, Not Applicable for 7Mode)The name of the vserver
in which the IGroup is present.

=item C<< "type" => $string >>

(Optional) type of the Initiator Group

=item C<< "alua" => $string >>

(Optional) Enable ALUA option

=item C<< "operating-system" => $string >>

(Optional) Operating System 

=item C<< "initiator" => [ $initiator1, $initiator2, ... ] >>

(Optional) List of Initiators to be added

=back

=back

=cut

########################## GUI Create ##################################
sub create {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            initiators         => { type => ARRAYREF, optional => 1 },
            "operating-system" => { type => SCALAR },
            _optional_scalars(qw(type alua portset)),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_create_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_create_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub create

####################### GUI CMode Create ###############################
sub _create_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $lun_igroup_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "vserver"          => "-vservername",
            "igroup"           => "-igroupname",
            "operating-system" => "-os",
            "type"             => "-type",
            "alua"             => "-alua",
            "initiators"       => "-initiators",
            "portset"          => "-portset",
        }
    );

    $apiset->create_igroup(%$lun_igroup_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        command_interface => $command_interface,
        igroup            => $lun_igroup_args->{'-igroupname'},
        vserver           => $lun_igroup_args->{'-vservername'},
    );
} ## end sub _create_cmode_gui

####################### GUI 7Mode Create ###############################
sub _create_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $lun_igroup_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "igroup"           => "-igroupname",
            "operating-system" => "-os",
            "type"             => "-type",
            "alua"             => "-alua",
            "initiators"       => "-initiators",
        }
    );

    $apiset->create_igroup(%$lun_igroup_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        command_interface => $command_interface,
        igroup            => $lun_igroup_args->{'-igroupname'},
    );
} ## end sub _create_7mode_gui

=head2 modify

   NACL::C::SysManager::LunIgroup->modify(command_interface => $ci,
										  igroup            => $igroup,
										  vserver           => $vserver,...);
	                             
								 or

    $LunIgroup_object->modify(...);

(Class or instance method)
This method is used to modify an existing initiator group.
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "igroup" => $igroup >>

(Required for class method, Not Applicable for instance method)
Name of the Initiar Group.

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)
(Required for CMode, Not Applicable for 7Mode)The name of the vserver
in which the IGroup is present.

=item C<< "rename" => $string >>

(Optional) Rename of the Lun Igroup

=item C<< "add-initiators" => [ $initiator1, $initiator2, ... ] >>

(Optional) List of the Initiators to be added

=item C<< "delete-initiators" => [ $initiator1, $initiator2, ... ] >>

(Optional) List of the Initiators to be deleted

=item C<< "alua" => $string >>

(Optional) Enable ALUA option

=item C<< "operating-system" => $string >>

(Optional) Operating System

=back

=back

=cut

########################## GUI Modify ##################################
sub modify {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "add-initiators"    => { type => ARRAYREF, optional => 1 },
            "delete-initiators" => { type => ARRAYREF, optional => 1 },
            _optional_scalars(qw(rename operating-system alua)),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_modify_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_modify_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

####################### GUI CMode Modify ###############################
sub _modify_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $lun_igroup_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "vserver"           => "-vservername",
            "igroup"            => "-igroupname",
            "rename"            => "-newigroupname",
            "operating-system"  => "-os",
            "alua"              => "-alua",
            "add-initiators"    => "-addinitiators",
            "delete-initiators" => "-deleteinitiators",
        }
    );

    $apiset->edit_igroup(%$lun_igroup_args);

    if (   ( defined $lun_igroup_args->{'-newigroupname'} )
        && ( ref $pkg_or_obj ) )
    {
        $pkg_or_obj->lun( $opts{'rename'} );
    }

    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_gui

####################### GUI 7Mode Modify ###############################
sub _modify_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $lun_igroup_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "igroup"            => "-igroupname",
            "rename"            => "-newigroupname",
            "operating-system"  => "-os",
            "alua"              => "-alua",
            "add-initiators"    => "-addinitiators",
            "delete-initiators" => "-deleteinitiators",
        }
    );

    $apiset->edit_igroup(%$lun_igroup_args);

    if (   ( defined $lun_igroup_args->{'-newigroupname'} )
        && ( ref $pkg_or_obj ) )
    {
        $pkg_or_obj->lun( $opts{'rename'} );
    }

    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_gui

=head2 delete

   NACL::C::SysManager::LunIgroup->delete(command_interface => $ci,
										  igroup            => $igroup,
										  vserver           => $vserver,...);
	                             
								 or

    $LunIgroup_object->delete(...);

(Class or instance method)
This method is used to delete an initiator group.
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "igroup" => $igroup >>

(Required for class method, Not Applicable for instance method)
Name of the Initiar Group.

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)
(Required for CMode, Not Applicable for 7Mode)The name of the vserver
in which the IGroup is present.

=back

=back

=cut

########################## GUI Delete ##################################
sub delete {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_delete_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_delete_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

####################### GUI CMode Delete ###############################
sub _delete_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $lun_igroup_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "igroup"  => "-igroupname",
            "vserver" => "-vservername",
        }
    );

    $apiset->delete_igroup(%$lun_igroup_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_gui

####################### GUI 7Mode Delete ###############################
sub _delete_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $lun_igroup_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "igroup" => "-igroupname", }
    );

    $apiset->delete_igroup(%$lun_igroup_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return (
        'igroup'  => { type => SCALAR },
        'vserver' => { type => SCALAR | UNDEF, optional => 1 },
    );
} ## end sub _primary_keys_validate_spec

1;
