# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary Qtree Component Module
## @author Neethu.George@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::Qtree

=head1 DESCRIPTION

C<NACL::C::SysManager::Qtree> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>.  It represents access
to an ONTAP Qtree present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::Qtree|lib-NACL-CS-SysManager-Qtree-pm>, which represents
the State Info of an ONTAP Qtree present on the System Manger GUI Screen.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 qtree

The name of the qtree, used to identify it uniquely. (Qtree
names are assumed to be unique within a node or within a cluster in 7mode
and in a vserver in Cmode)

=head2 volume

The name of the volume which is linked to qtree.

=head2 vserver

(Required for CMode, Not Applicable for 7Mode) The name of the vserver
in which the qtree is present.

=cut

package NACL::C::SysManager::Qtree;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::Qtree;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

	my $qtree = NACL::C::SysManager::Qtree->new(command_interface => $ci,
											    vserver           => $vserver_name,
											    volume            => $volume_name,
											    qtree             => $qtree);

Return a new Qtree component object with the given attributes.

=cut

####################### Method Maker ###############################

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'volume',
    scalar => 'qtree',
];

=head2 create

  my $qtree  = NACL::C::SysManager::Qtree->create(command_interface => $ci,
												  vserver           => $vserver_name,
												  volume            => $volume_name,
												  qtree             => $qtree,...);

(Class method) Request creation of a new qtree element, and  return a new
qtree component object that refers to it.
Uses a CMode GUI or 7mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for CMode, Not Applicable for 7Mode)The name of the vserver
in which the qtree is present.

=item C<< volume => $string >>

(Required) Name of volume in which the qtree is created.

=item C<< qtree => $string >>

(Required) Name of qtree.

=item C<< "security-style" => $string >>

(Optional) Security Style of the Qtree(Values can be inherit|unix|ntfs|mixed)

=item C<< "oplock-mode" => $string >>

(Optional) Opslock for files and directories in qtree(Values can be enable|disable)

=item C<< "qtreequota" => $string >>

(Optional) Limit total space usage within qtree(Values can be enable|disable)

=item C<< "qtreespaceusage" => $string >>

(Optional) Space Usage Limit value

=item C<< "qtreespaceusagemode" => $string >>

(Optional) Space Usage Limit Unit(Values can be KB|MB|GB|TB)

=item C<< "userquota" => $string >>

(Optional) Limit total space usage for users of qtree(Values can be enable|disable)

=item C<< "userspaceusage" => $string >>

(Optional) Space Usage Limit Value

=item C<< "userspaceusagemode" => $string >>

(Optional) Space Usage Limit Unit(Values can be KB|MB|GB|TB)

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw (security-style oplock-mode qtreequota qtreespaceusage
                    qtreespaceusagemode userquota userspaceusage userspaceusagemode)
            )
        }
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_create_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_create_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub create
####################### GUI 7Mode Create ###############################
sub _create_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $qtree_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "volume"              => "-volname",
            "qtree"               => "-qtreename",
            "security-style"      => "-secstyle",
            "oplock-mode"         => "-oplocks",
            "qtreequota"          => "-qtreequota",
            "qtreespaceusage"     => "-qtreespaceusage",
            "qtreespaceusagemode" => "-qtreespaceusagemode",
            "userquota"           => "-userquota",
            "userspaceusage"      => "-userspaceusage",
            "userspaceusagemode"  => "-userspaceusagemode",
        }
    );

    $apiset->create_qtree(%$qtree_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        volume            => $qtree_args->{-volname},
        qtree             => $qtree_args->{-qtreename},
        command_interface => $command_interface
    );
} ## end sub _create_7mode_gui

####################### GUI CMode Create ###############################
sub _create_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $qtree_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "volume"              => "-volname",
            "qtree"               => "-qtreename",
            "vserver"             => "-vservername",
            "security-style"      => "-secstyle",
            "oplock-mode"         => "-oplocks",
            "qtreequota"          => "-qtreequota",
            "qtreespaceusage"     => "-qtreespaceusage",
            "qtreespaceusagemode" => "-qtreespaceusagemode",
            "userquota"           => "-userquota",
            "userspaceusage"      => "-userspaceusage",
            "userspaceusagemode"  => "-userspaceusagemode",
        }
    );
    $apiset->create_qtree(%$qtree_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        volume            => $qtree_args->{-volname},
        qtree             => $qtree_args->{-qtreename},
        vserver           => $qtree_args->{-vservername},
        command_interface => $command_interface
    );

} ## end sub _create_cmode_gui

=head2 delete


    NACL::C::SysManager::Qtree->delete(command_interface => $ci,
									   vserver           => $vserver_name,
									   volume            => $volume_name,
									   qtree             => $qtree,...);
                            or

	$qtree_obj->delete(...);

(Class or instance method) This method deletes an qtree element.
Uses a CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode, Not Applicable for 7Mode)The name of the vserver
in which the qtree is present.

=item C<< volume => $string >>

(Required for class method, Not Applicable for instance method)
Name of volume in which the qtree is created.

=item C<< qtree => $string >>

(Required for class method, Not Applicable for instance method)
Name of qtree.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_delete_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_delete_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub delete

####################### GUI 7Mode Delete ###############################
sub _delete_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $qtree_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "volume" => "-volname",
            "qtree"  => "-qtreename"
        }
    );

    $apiset->delete_qtree(%$qtree_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_gui

####################### GUI CMode Delete ###############################
sub _delete_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $qtree_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "volume"  => "-volname",
            "qtree"   => "-qtreename",
            "vserver" => "-vservername"
        }
    );

    $apiset->delete_qtree(%$qtree_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_gui

=head2 edit

	NACL::C::SysManager::Qtree->edit(command_interface => $ci,
									 vserver           => $vserver_name,
									 volume            => $volume_name,
									 qtree             => $qtree,...);

                            or

	$qtree_obj->edit(...);

(Class or instance method) Modify a qtree element.  
Uses a CMode GUI or a 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method.)
(Required for CMode, Not Applicable for 7Mode).
Name of vserver that contains the qtree.

=item C<< volume => $string >>

(Required for class method, Not Applicable for instance method)
Name of volume that contains the qtree.

=item C<< qtree => $string >>

(Required for class method, Not Applicable for instance method)
Name of qtree.

=item C<< "security-style" => $string >>

(Optional) Security Style of the Qtree(Values can be inherit|unix|ntfs|mixed)

=item C<< "oplock-mode" => $string >>

(Optional) Opslock for files and directories in qtree(Values can be enable|disable)


=back

=back

=cut

sub edit {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            { _optional_scalars(qw (security-style oplock-mode)) }
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_edit_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_edit_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub edit

####################### GUI 7Mode Edit ###############################
sub _edit_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $qtree_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "volume"         => "-volname",
            "qtree"          => "-qtreename",
            "security-style" => "-secstyle",
            "oplock-mode"    => "-oplocks"
        }
    );

    $apiset->edit_qtree(%$qtree_args);
    $Log->exit() if $may_exit;
} ## end sub _edit_7mode_gui

####################### GUI CMode Edit ###############################
sub _edit_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $qtree_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "volume"         => "-volname",
            "qtree"          => "-qtreename",
            "vserver"        => "-vservername",
            "security-style" => "-secstyle",
            "oplock-mode"    => "-oplocks"
        }
    );

    $apiset->edit_qtree(%$qtree_args);
    $Log->exit() if $may_exit;
} ## end sub _edit_cmode_gui

=head2 refresh

	NACL::C::SysManager::Qtree->refresh(command_interface => $ci,
										vserver           => $vserver_name,...);

                            or

	$qtree_obj->refresh();

(Class or instance method) This method refreshs qtree page.
Uses a 7Mode GUI or CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode, Not Applicable for 7Mode)The name of the vserver
in which the qtree is present.


=back

=back

=cut

sub refresh {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_refresh_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_refresh_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub refresh

####################### GUI 7Mode Refresh ###############################
sub _refresh_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $qtree_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "volume" => "-volname",
            "qtree"  => "-qtreename",
        }
    );

    $apiset->refresh_qtree(%$qtree_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_7mode_gui

####################### GUI CMode Refresh ###############################
sub _refresh_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $qtree_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "volume"  => "-volname",
            "qtree"   => "-qtreename",
            "vserver" => "-vservername"
        }
    );

    $apiset->refresh_qtree(%$qtree_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_cmode_gui

##################################################################################
sub _primary_keys_validate_spec {
    return (
        volume  => { type => SCALAR },
        qtree   => { type => SCALAR },
        vserver => { type => SCALAR | UNDEF, optional => 1 },
    );
} ## end sub _primary_keys_validate_spec

1;
