# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Represents a client machine with mgmt interface and IP address and
## has the Selinum Server and System Manger running.
## @author Neelekanth.Raju@netapp.com, dl-sm-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::SMClient

=head1 DESCRIPTION

C<NACL::C::SysManager::SMClient> is a derived class of
L<NACL::C::Node|lib-NACL-C-Node-pm>. It represents clients which have
the Selenium Server and NetApp OnCommand System Manager x.x running on it.
It can currently be used only as a command interface for System Manager components.

Unlike C<NACL-C-Node-pm>, which defaults to the Node CLI APISet,
C<NACL::C::SysManager::SMClient> defaults to the SysManager GUI APISet.
All other attributes and methods can be inherited from C<NACL::C::Node>.

=head1 ATTRIBUTES

Same as L<NACL::C::Node|lib-NACL-C-Node-pm/ATTRIBUTES>.

=cut

package NACL::C::SysManager::SMClient;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use base 'NACL::C::Node';

=head1 METHODS

=head2 find

    my $SM_ci_obj = NACL::C::SysManager::SMClient->find();

(Class method)
Finds the System Manger Host and returns command interface object for it.
(Should only be called in scalar context, in which case it returns the
System Manger host found).
Refer to L<find|lib-NACL-C-CommandInterface-pm/find> for more details.

You can also refer to user guide for more information on the options,
order of precedence for command line parameters (in case of find() is
invoked without specifying a command_interface) and examples.
http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/ComponentLayerUsersGuide/index.html#Obtaining_command_interface_objects

=head2 default_apiset_could

The ruleset to default to if values for the co-ordinates 'category',
'interface' or 'set' cannot be determined.

If SysManger components used C<NACL::C::Node> as their command interface,
unless an apiset_must value for 'set' is defined as SysManager_(7/C)Mode
(either as a test parameter or in the tharnhost file for the hosts used),
 it will default to use set as "Node". By using C<NACL::C::SysManager::SMClient>
instead, the set will default to "SysManager_CMode" without needing to make
 this setting for all tests/tharnhost files.

Refer to L<default_apiset_could|lib-NACL-C-CommandInterface-pm/default_apiset_could>

=cut

sub default_apiset_could {
    return {
        category  => "Host",
        interface => "GUI",
        set       => "SysManager_CMode"
    };
}

=head2 default_apiset_must

All SysManger clients have to be of category "Node" and set "SysManager_CMode" or
"SysManager_7Mode" , so this method returns a hash with keys 'category' whose
 value is "Host", 'set' whose value is either "SysManager_CMode" or "SysManager_7Mode".

Refer to L<default_apiset_must|lib-NACL-C-CommandInterface-pm/default_apiset_must>.

=cut

sub default_apiset_must {
    my $self     = shift;
    my $hostrec  = $self->hostrec();
    my $hostid   = $hostrec->id();
    my $hosttype = $hostrec->hosttype();
    my $set;

    if ( $hosttype =~ /ng/ ) {
        $set = "SysManager_CMode";
    }
    elsif ( $hosttype =~ /filer/ ) {
        $set = "SysManager_7Mode";
    }
    else {
        NATE::BaseException->throw( "Host $hostid was supposed "
              . "to be a SM client but is actually of type '$hosttype'" );
    }
    return {
        category => "Host",
        category_source =>
          "command interface is a NACL::C::SysManager::SMClient",
        set        => $set,
        set_source => "command interface is a NACL::C::SysManager::SMClient",
    };
} ## end sub default_apiset_must

=head2 cleanup

    $self->cleanup();

(Instance method)
This method is used to invoke the api layer cleanup routine to disconnect
 from the sellinum server and cleanup the GUI session object.

=cut

sub cleanup {
    $Log->enter() if $may_enter;
    my $self = shift;
    my $set;
    my $hosttype = $self->hostrec->hosttype();

    if ( $hosttype =~ /filer-ng/ ) {
        $set = "SysManager_CMode";
    }
    elsif ( $hosttype =~ /filer/ ) {
        $set = "SysManager_7Mode";
    }

    my %opts;
    $opts{category}  = "Host";
    $opts{interface} = "GUI";
    $opts{set}       = $set;

    my $api = $self->apiset(%opts);
    $api->cleanup;
    $Log->exit() if $may_exit;
} ## end sub cleanup

=head2 _create_version_manager

   sub get_version_manager {
        ...
        if (keys %$apisets) {
            ...
        } else {
            $version_manager = $self->_create_version_manager();
        }
        return $version_manager;
    }

(Instance method)
This private method returns the version manager of the appropriate type.
This decision is based on the category ("Node" for SMClient command interfaces)
and the set ("SysManager_CMode" or "SysManager_7Mode"). Shell type is also set
as ("SysManager_CMode" or "SysManager_7Mode").

=cut

sub _create_version_manager {
    $Log->enter() if $may_enter;
    my $self = shift;

    use NACL::APISet::Version;
    my $hostrec  = $self->hostrec();
    my $category = "Host";
    my $hostid   = $hostrec->id();
    my $hosttype = $hostrec->hosttype();

    my $shell_type = "SysManager_CMode";
    if ( $hosttype =~ /ng/ ) {
        $shell_type = "SysManager_CMode";
    }
    elsif ( $hosttype =~ /filer/ ) {
        $shell_type = "SysManager_7Mode";
    }
    else {
        NATE::BaseException->throw( "Host $hostid was supposed "
              . "to be a SM client but is actually of type '$hosttype'" );
    }
    $Log->debug( "Version manager object with parameters "
          . "category = $category; shell_type = $shell_type; "
          . "hostobj type = $hosttype; set = $shell_type"
          . " created" );
    $Log->exit() if $may_exit;
    return NACL::APISet::Version->new(
        category   => $category,
        hostobj    => $hostrec,
        set        => $shell_type,
        shell_type => $shell_type
    );
} ## end sub _create_version_manager

1;
