# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary SNMP Component Module
## @author vikas.p@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::SNMP

=head1 DESCRIPTION

C<NACL::C::SysManager::SNMP> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>.  It represents access
to an ONTAP SNMP present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::SNMP|lib-NACL-CS-SysManager-SNMP-pm>, which represents
the State Info of an ONTAP SNMP present on the System Manger GUI Screen.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

package NACL::C::SysManager::SNMP;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::SNMP;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

	my $snmp = 
		NACL::C::SysManager::SNMP->new(command_interface => $ci);

Return a new SNMP component object with the given attributes.

=cut

####################### Method Maker ###############################

use Class::MethodMaker [

];

=head2 modify

	my $snmp_obj = NACL::C::SysManager::SNMP->modify(command_interface => $ci,
													 location          => $loc,
													 contact           => $contact,...);

(Class method) Modify a SNMP element.  
Uses a  7Mode GUI or CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "community-names-add"=> \$array >>

(Optional) List of community names to be added

=item C<< "community-names-edit"=> \$array >>

(Optional) List of community names to be edited

=item C<< "community-names-delete"=> \$array >>

(Optional) List of community names to be deleted

=item C<< "trap-host-add"=> \$array >>

(Optional) List of traphost to be added

=item C<< "trap-host-edit"=> \$array >>

(Optional) List of traphost to be edited

=item C<< "trap-host-delete"=> \$array >>

(Optional) List of traphost to be deleted

=item C<< "contact"=> $scalar >>

(Optional) SNMP contact

=item C<< "location"=> $scalar >>

(Optional) SNMP location

=item C<< "community-names-option"=> $scalar >>

(Optional) Community names options

=item C<< "community-name"=> $scalar >>

(Optional) Community name

=item C<< "community-name-edit-rename"=> $scalar >>

(Optional) Community rename 

=item C<< "community-names-delete"=> $scalar >>

(Optional) Community names to be deleted

=back

=back

=cut

####################### GUI Modify ###############################
sub modify {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'community-names-add'    => { type => ARRAYREF, optional => 1 },
            'community-names-edit'   => { type => ARRAYREF, optional => 1 },
            'community-names-delete' => { type => ARRAYREF, optional => 1 },
            'trap-host-add'          => { type => ARRAYREF, optional => 1 },
            'trap-host-edit'         => { type => ARRAYREF, optional => 1 },
            'trap-host-delete'       => { type => ARRAYREF, optional => 1 },
            'snmp-options-add'       => { type => ARRAYREF, optional => 1 },
            'snmp-options-delete'    => { type => ARRAYREF, optional => 1 },
            _optional_scalars(qw (contact location trap-enable)),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_modify_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_modify_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

####################### GUI 7Mode Modify ###############################
sub _modify_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $snmp_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "contact"                => "-contact",
            "location"               => "-location",
            "community-names-add"    => "-addCommunities",
            "community-names-edit"   => "-editCommunities",
            "community-names-delete" => "-deleteCommunities",
            "trap-enable"            => "-enableTrap",
            "trap-host-add"          => "-addTrapHosts",
            "trap-host-edit"         => "-editTrapHosts",
            "trap-host-delete"       => "-deleteTrapHosts",
        }
    );

    $apiset->edit_snmp(%$snmp_args);
    $Log->exit() if $may_exit;

} ## end sub _modify_7mode_gui

####################### GUI CMode Modify ###############################
sub _modify_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $snmp_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "contact"                => "-snmp_contact",
            "location"               => "-snmp_location",
            "community-names-add"    => "-addcommunityname",
            "community-names-edit"   => "-editcommunityname",
            "community-names-delete" => "-deletecommunityname",
            "snmp-options-add"       => "-addsnmpoptions",
            "snmp-options-delete"    => "-deletesnmpoptions",
            "trap-enable"            => "-enabletraphost",
            "trap-host-add"          => "-addtraphostoptions",
            "trap-host-edit"         => "-edittraphostoptions",
            "trap-host-delete"       => "-deletetraphostoptions",
        }
    );

    $apiset->snmp_edit(%$snmp_args);
    $Log->exit() if $may_exit;

} ## end sub _modify_cmode_gui

=head2 refresh

	NACL::C::SysManager::SNMP->refresh(command_interface => $ci,...);
	
						(Or)
	
	$snmp_obj->refresh(...)

(Class or instance method)This method refreshs SNMP page.
Uses a 7Mode GUI or CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=back

=back

=cut

####################### GUI Refresh ###############################
sub refresh {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_refresh_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_refresh_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub refresh

####################### GUI 7Mode Refresh ###############################
sub _refresh_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snmp_args = $pkg_or_obj->_hash_copy( source => \%opts, );

    $apiset->snmp_refresh(%$snmp_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_7mode_gui

####################### GUI CMode Refresh ###############################
sub _refresh_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snmp_args = $pkg_or_obj->_hash_copy( source => \%opts, );

    $apiset->snmp_refresh(%$snmp_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_cmode_gui

1;
