# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary Schedule Component Module
## @author Neethu.George@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::Schedule

=head1 DESCRIPTION

C<NACL::C::SysManager::Schedule> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>.  It represents access
to an ONTAP Schedule present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::Schedule|lib-NACL-CS-SysManager-Schedule-pm>, which represents
the State Info of an ONTAP Schedule present on the System Manger GUI Screen.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 schedule

The name of the schedule, used to identify it uniquely. 

=cut

package NACL::C::SysManager::Schedule;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::Schedule;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

	my $schedule = NACL::C::SysManager::Schedule->new(command_interface => $ci,
													  schedule     => $schedule);

Return a new Schedule component object with the given attributes.

=cut

####################### Method Maker ###############################

use Class::MethodMaker [ 
    scalar => 'schedule', 
];

=head2 create

	$schedule_obj = NACL::C::SysManager::Schedule->create(command_interface => $ci,
													      schedule     => $schedule...);

(Class or instance method)This method creates a schedule
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< schedule => $schedule >>

(Required) Name of the schedule to be created.

=item C<< schedule-recurring => $string >>

(Optional) Schedule recurring interval

=item C<< weekday => $arrayref >>

(Optional)  Weekday to run the schedule

=item C<<   run-minute => $string >>

(Optional)  Recurring minute to run the schedule

=item C<< run-hour => $string >>

(Optional)  Recurring hour to run the schedule

=item C<< run-at-hour => $string >>

(Optional)  Hour to run the schedule

=item C<< run-at-minute => $string >>

(Optional)  Minute to run the schedule

=item C<< run-at-hour-minute => $string >>

(Optional)  Schedule hour- minute

=item C<< run-hour-text => $string >>

(Optional) Schedule Run Hour

=item C<< run-minute-text => $string >>

(Optional) Schedule Run Minute

=item C<< interval => $string >>

(Optional) Interval

=item C<< run-everyday => $string >>

(Optional) Run Schedule Everyday

=item C<< run-everyhour => $string >>

(Optional) Run Schedule Every Hour

=item C<< run-everyminute => $string >>

(Optional) Run Schedule Every Minute

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            weekday => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( schedule-recurring run-minute run-hour run-at-hour
                    run-at-minute run-at-hour-minute run-hour-text
                    run-minute-text interval run-everyday
                    run-everyhour run-everyminute)
            ),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_create_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub create

####################### GUI CMode Create ###############################
sub _create_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $schedule_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "schedule"           => "-schedule_name",
            "schedule-recurring" => "-basic",
            "weekday"            => "-weekday",
            "run-minute"         => "-run_minute",
            "run-hour"           => "-run_hour",
            "run-at-hour"        => "-run_at_hour",
            "run-at-minute"      => "-run_at_min",
            "run-at-hour-minute" => "-run_hour_at_minute",
            "run-hour-text"      => "-run_hour_text",
            "run-minute-text"    => "-run_min_text",
            "interval"           => "-interval",
            "run-everyday"       => "-run_every_day",
            "run-everyhour"      => "-run_every_hour",
            "run-everyminute"    => "-run_every_min"
        }
    );

    $apiset->schedule_create(%$schedule_args);

    return $pkg->new(
        schedule     => $schedule_args->{-schedule_name},
        command_interface => $command_interface
    );
} ## end sub _create_cmode_gui

=head2 edit

	NACL::C::SysManager::Schedule->edit(command_interface => $ci,
										schedule     => $schedule...);
                                 
								or
								 
	$schedule_obj->edit(...);

(Class or instance method)This method edits the Schedule
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< schedule => $schedule >>

(Required for class method, Not Applicable for instance method)
Name of the schedule to be edited.

=item C<< schedule-recurring => $string >>

(Optional) Schedule recurring interval

=item C<< weekday => $string >>

(Optional)  Weekday to run the schedule

=item C<<   run-minute => $string >>

(Optional)  Recurring minute to run the schedule

=item C<< run-hour => $string >>

(Optional)  Recurring hour to run the schedule

=item C<< run-at-hour => $string >>

(Optional)  Hour to run the schedule

=item C<< run-at-minute => $string >>

(Optional)  Minute to run the schedule

=item C<< run-at-hour-minute => $string >>

(Optional)  Schedule hour- minute

=item C<< run-hour-text => $string >>

(Optional) Schedule Run Hour

=item C<< run-minute-text => $string >>

(Optional) Schedule Run Minute

=item C<< interval => $string >>

(Optional) Interval Option

=item C<< run-everyday => $string >>

(Optional) Run Schedule Everyday

=item C<< run-everyhour => $string >>

(Optional) Run Schedule Every Hour

=item C<< run-everyminute => $string >>

(Optional) Run Schedule Every Minute

=back

=back

=cut

sub edit {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            weekday => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( schedule-recurring weekday run-minute run-hour run-at-hour
                    run-at-minute run-at-hour-minute run-hour-text
                    run-minute-text interval run-everyday
                    run-everyhour run-everyminute)
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_edit_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub edit

####################### GUI CMode Edit ###############################
sub _edit_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $schedule_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "schedule"           => "-schedule_name",
            "schedule-recurring" => "-basic",
            "weekday"            => "-weekday",
            "run-minute"         => "-run_minute",
            "run-hour"           => "-run_hour",
            "run-at-hour"        => "-run_at_hour",
            "run-at-minute"      => "-run_at_min",
            "run-at-hour-minute" => "-run_hour_at_minute",
            "run-hour-text"      => "-run_hour_text",
            "run-minute-text"    => "-run_min_text",
            "interval"           => "-interval",
            "run-everyday"       => "-run_every_day",
            "run-everyhour"      => "-run_every_hour",
            "run-everyminute"    => "-run_every_min"
        }
    );

    $apiset->schedule_edit(%$schedule_args);
    $Log->exit() if $may_exit;

} ## end sub _edit_cmode_gui

=head2 delete

	NACL::C::SysManager::Schedule->delete(command_interface => $ci,
										  schedule   => $schedule...);
										  
                                or
								 
	$schedule_obj->delete();


(Class or instance method)This method deletes the Schedule
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< schedule => $schedule >>

(Required for class method, Not Applicable for instance method)
Name of the schedule to be edited.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_delete_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub delete

####################### GUI CMode delete ###############################
sub _delete_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $schedule_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "schedule" => "-schedule_name", }
    );

    $apiset->schedule_delete(%$schedule_args);
    $Log->exit() if $may_exit;

} ## end sub _delete_cmode_gui

=head2 refresh

	$schedule_obj = NACL::C::SysManager::Schedule->refresh(command_interface => $ci);
                                 or
								 
	$schedule_obj->refresh();

(Class or instance method)This method refreshs schedule page.
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub refresh {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_refresh_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub refresh

####################### GUI CMode Refresh ###############################
sub _refresh_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $schedule_args = $pkg_or_obj->_hash_copy( source => \%opts, );

    $apiset->schedule_refresh(%$schedule_args);
    $Log->exit() if $may_exit;

} ## end sub _refresh_cmode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return ( schedule => { type => SCALAR }, );
}

1;

