# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary SnapshotPolicy Component Module
## @author neethu.george@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::SnapshotPolicy

=head1 DESCRIPTION

C<NACL::C::SysManager::SnapshotPolicy> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>.  It represents access
to an ONTAP SnapshotPolicy present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::SnapshotPolicy|lib-NACL-CS-SysManager-SnapshotPolicy-pm>, which represents
the State Info of an ONTAP SnapshotPolicy present on the System Manger GUI Screen.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 policy

The name of the snapshot policy, used to identify it uniquely. (snapshot policy
names are assumed to be unique in Cmode)

=cut

package NACL::C::SysManager::SnapshotPolicy;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::SnapshotPolicy;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

	my $snap_policy = 
		NACL::C::SysManager::SnapshotPolicy->new(command_interface => $ci,
												 policy            => $policy_name,);

Return a new SnapshotPolicy component object with the given attributes.

=cut

use Class::MethodMaker [ scalar => 'policy', ];

=head2 add

	my $snap_policy  = 
		NACL::C::SysManager::SnapshotPolicy->add(command_interface => $ci,
												 policy            => $policy_name,...);

(Class method) Request creation of a new snapshot policy element, and  return 
a new snapshot policy component object that refers to it.
Uses CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< policy => $string >>

(Required) Name of snapshot policy to create.

=item C<< schedule => $string >>

(Optional) the schedule to be tagged to snapshot policy

=item C<< schedule-edit => $string >>

(Optional) schedule to be modified

=item C<< schedule-modify => $string >>

(Optional) schedule modifications

=item C<< schedule-delete => $string >>

(Optional) schedules to be deleted

=item C<< max-snapshot => $string >>

(Optional) maximum snapshots

=item C<< max-snapshot-edit => $string >>

(Optional) maximum snapshot to be edited

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw ( schedule schedule-edit schedule-modify
                    schedule-delete max-snapshot max-snapshot-edit)
            )
        }
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_add_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub add

####################### GUI CMode add ###############################
sub _add_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $snapshotpolicy_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "policy"            => "-policyName",
            "schedule"          => "-schedule",
            "schedule-edit"     => "-scheduleedit",
            "schedule-modify"   => "-schedulemodify",
            "schedule-delete"   => "-scheduledel",
            "max-snapshot"      => "-maxsnapshot",
            "max-snapshot-edit" => "-maxsnapshotedit",
        }
    );

    $apiset->snapshot_policy_add(%$snapshotpolicy_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        policy            => $snapshotpolicy_args->{-policyName},
        command_interface => $command_interface
    );
} ## end sub _add_cmode_gui

=head2 delete

	NACL::C::SysManager::SnapshotPolicy->delete(command_interface => $ci,
												policy            => $policy_name,...);
	
						(Or)
	
	$snap_obj->delete(...);

(Class or instance method)Request deletion of snapshot policy element.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< policy => $string >>

(Required for class method, Not Applicable for instance method)
Name of snapshot policy.

=item C<< confirm-delete => $string >>

(Optional) Enable to confirm deletion

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => { _optional_scalars(qw ( confirm-delete )) }
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_delete_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub delete

####################### GUI CMode delete ###############################
sub _delete_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapshotpolicy_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "policy"         => "-policyName",
            "confirm-delete" => "-checkbox"
        }
    );

    $apiset->snapshot_policy_delete(%$snapshotpolicy_args);
    $Log->exit() if $may_exit;

} ## end sub _delete_cmode_gui

=head2 edit

	NACL::C::SysManager::SnapshotPolicy->edit(command_interface => $ci,
											  policy            => $policy_name);
	
						(Or)
	
	$snap_obj->edit(...);

(Class or instance method)
Request edit of a new snapshot policy element.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< policy => $policy_name >>

(Required for class method, Not Applicable for instance method)
Name of snapshot policy to edit.

=item C<< schedule => $string >>

(Optional) the schedule to be tagged to snapshot policy

=item C<< schedule-edit => $string >>

(Optional) schedule to be modified

=item C<< schedule-modify => $string >>

(Optional) schedule modifications

=item C<< schedule-delete => $string >>

(Optional) schedules to be deleted

=item C<< max-snapshot => $string >>

(Optional) maximum snapshots

=item C<< max-snapshot-edit => $string >>

(Optional) maximum snapshot to be edited

=back

=back

=cut

sub edit {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw ( schedule schedule-edit schedule-modify
                    schedule-delete max-snapshot max-snapshot-edit)
            )
        }
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_edit_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub edit

####################### GUI CMode edit ###############################
sub _edit_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapshotpolicy_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "policy"            => "-policyName",
            "schedule"          => "-schedule",
            "schedule-edit"     => "-scheduleedit",
            "schedule-modify"   => "-schedulemodify",
            "schedule-delete"   => "-scheduledel",
            "max-snapshot"      => "-maxsnapshot",
            "max-snapshot-edit" => "-maxsnapshotedit",
        }
    );

    $apiset->snapshot_policy_edit(%$snapshotpolicy_args);
    $Log->exit() if $may_exit;

} ## end sub _edit_cmode_gui

=head2 status

	NACL::C::SysManager::SnapshotPolicy->status(command_interface => $ci,
												policy            => $policy_name,);

							(Or)
	
	$snap_obj->status(...);
	
(Class or instance method)This method changes status of a snapshot policy element.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< policy => $policy_name >>

(Required for class method, Not Applicable for instance method)
Name of snapshot policy.

=item C<< status =>$status >>

(Optional) the status to be set for the snapshot policy

=back

=back

=cut

sub status {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => { _optional_scalars(qw ( status )) }
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_status_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub status

####################### GUI CMode status ###############################
sub _status_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapshotpolicy_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(status)],
        map    => {
            "policy" => "-policyName",
            "status" => "-status",
        }
    );

    $apiset->snapshot_policy_status(%$snapshotpolicy_args);
    $Log->exit() if $may_exit;
} ## end sub _status_cmode_gui

=head2 refresh

	NACL::C::SysManager::SnapshotPolicy->refresh(command_interface => $ci,...);


(Class or instance method) This method refreshs a snapshot policy page
Uses CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub refresh {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_refresh_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub refresh

####################### GUI CMode Refresh ###############################
sub _refresh_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapshotpolicy_args = $pkg_or_obj->_hash_copy( source => \%opts, );

    $apiset->snapshot_policy_refresh(%$snapshotpolicy_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_cmode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return ( policy => { type => SCALAR }, );
}

1;
