# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary Users Component Module
## @author Vikas.P@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::Users

=head1 DESCRIPTION

C<NACL::C::SysManager::Users> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>.  It represents access
to an ONTAP Users present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::Users|lib-NACL-CS-SysManager-Users-pm>, which represents
the State Info of an ONTAP Users present on the System Manger GUI Screen.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 username

The name of the user, used to identify it uniquely. (user
names are assumed to be unique in Cmode)

=cut

package NACL::C::SysManager::Users;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::Users;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

	my $user = 
		NACL::C::SysManager::Users->new(command_interface => $ci,
									    username          => $username,
										role              => $role,);

Return a new User component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'username',
    scalar => 'vserver',
    scalar => 'route',
    scalar => 'application',
];

=head2 add

	NACL::C::SysManager::Users->add(command_interface => $ci,
								    username          => $user, 
                                    route             => $route,...);

(Class method)This method creates User
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< username => $user >>

(Required) The user to be added.

=item C<< route => $route >>

(Required) Either vserver or cluster.

=item C<< password => $password >>

(Required) Password to be given to the particular user.

=item C<< confirm-password => $conf_passwrd >>

(Required) Confirmation Password for the user.

=item C<< application => $application >>

(Required) Application for which user is to be configured. 

=item C<< role => $role >>

(Required) Role of the user.

=item C<< vserver => $vserver >>

(Optional) Vserver name.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            password             => { type => SCALAR },
            'confirm-password'   => { type => SCALAR },
            application          => { type => ARRAYREF },
            'edit-application'   => { type => ARRAYREF, optional => 1 },
            'delete-application' => { type => ARRAYREF, optional => 1 },
            _optional_scalars(qw( vserver)),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_add_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub add

####################### GUI CMode add ###############################
sub _add_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $security_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "username"           => "-username",
            "route"              => "-workflow",
            "password"           => "-password",
            "confirm-password"   => "-confirm_password",
            "application"        => "-application",
            "edit-application"   => "-editapplication",
            "delete-application" => "-delapplication",
            "vserver"            => "-vservername"
        }
    );

    $apiset->security_user_add(%$security_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        username          => $security_args->{-username},
        route             => $security_args->{-workflow},
        command_interface => $command_interface
    );

} ## end sub _add_cmode_gui

=head2 edit

	NACL::C::SysManager::Users->edit(command_interface => $ci,
								     username          => $user, 
									 route             => $route,...);
                                 
								 or

    $User_object->edit(...);

(Class or instance method)This method is used to edit user options
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< username => $user >>

(Required) The name of user to be edited.

=item C<< options => $options >>

(Required) The operations to be performed - add, edit, delete.

=item C<< route => $route >>

(Required) Either vserver or cluster.

=item C<< role  => $role >>

(Optional) User role.

=item C<< application => $application >>

(Optional) User application.

=item C<< edit-application => $editappl  >>

(Optional) User application to be edited.

=item C<< delete-application => $delappl >>

(Optional) User application to be deleted.

=item C<< vserver => $vserver >>

(Optional) Vserver name.

=back

=back

=cut

sub edit {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'application'        => { type => ARRAYREF, optional => 1 },
            'edit-application'   => { type => ARRAYREF, optional => 1 },
            'delete-application' => { type => ARRAYREF, optional => 1 },
            _optional_scalars(qw(role vserver)),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_edit_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub edit

####################### GUI CMode edit ###############################
sub _edit_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $security_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "username"           => "-username",
            "route"              => "-workflow",
            "application"        => "-addapplication",
            "edit-application"   => "-editapplication",
            "delete-application" => "-deleteapplication",
            "role"               => "-role",
            "vserver"            => "-vservername"
        }
    );

    $apiset->security_user_modify(%$security_args);
    $Log->exit() if $may_exit;

} ## end sub _edit_cmode_gui

=head2 change_password

	NACL::C::SysManager::Users->change_password(command_interface => $ci,
										        username          => $user, 
												route             => $route,...);
                                 
								or

    $User_object->change_password(...);

(Class or instance method)This method is used to change user's password
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< username => $user >>

(Required) The user name.

=item C<< route => $route >>

(Required) Either cluster or vserver.

=item C<< password => $password >>

(Required) The new password.

=item C<< confirm-password => $cnfrmpasswrd >>

(Required) The confirmation password

=item C<< vserver => $vserver >>

(Optional) Vserver name.

=back

=back

=cut

sub change_password {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            password           => { type => SCALAR },
            'confirm-password' => { type => SCALAR },
            _optional_scalars(qw( vserver)),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_change_password_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub change_password

####################### GUI CMode change_password ###############################
sub _change_password_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $security_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "username"         => "-username",
            "route"            => "-workflow",
            "password"         => "-password",
            "confirm-password" => "-confirm_password",
            "vserver"          => "-vservername"
        }
    );

    $apiset->security_user_change_password(%$security_args);
    $Log->exit() if $may_exit;

} ## end sub _change_password_cmode_gui

=head2 lock_unlock

	NACL::C::SysManager::Users->lock_unlock(command_interface => $ci,
										    username          => $user, 
										    route             => $route,...);
                                 
								or

    $User_object->lock_unlock(...);

(Class or instance method)This method is used to lock and unlock the user
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< username => $user >>

(Required) User to be locked or unlocked.

=item C<< route => $route >>

(Required) Either vserver or cluster.

=item C<< vserver => $vserver >>

(Optional) Vserver name.

=back

=back

=cut

sub lock_unlock {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(qw( vserver)),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_lock_unlock_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub lock_unlock

####################### GUI CMode lock_unlock ###############################
sub _lock_unlock_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $security_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "username" => "-username",
            "route"    => "-workflow",
            "vserver"  => "-vservername"
        }
    );

    $apiset->user_lock_unlock(%$security_args);
    $Log->exit() if $may_exit;

} ## end sub _lock_unlock_cmode_gui

=head2 refresh

	NACL::C::SysManager::Users->refresh(command_interface => $ci,
  									    route             => $route);

                            or

	$User_object->refresh();

(Class or instance method)This method refreshes Users page
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< route => $route >>

(Required) Either vserver or cluster.

=back

=back

=cut

sub refresh {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(qw( vserver)),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_refresh_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub refresh

####################### GUI CMode refresh ###############################
sub _refresh_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $security_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "route"   => "-workflow",
            "vserver" => "-vservername"
        }
    );

    $apiset->user_refresh(%$security_args);
    $Log->exit() if $may_exit;

} ## end sub _refresh_cmode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return (
        username    => { type => SCALAR },
        route       => { type => SCALAR },
        vserver     => { type => SCALAR | UNDEF, optional => 1 },
        application => { type => SCALAR | UNDEF, optional => 1 },
    );
} ## end sub _primary_keys_validate_spec

1;
