# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary Volume Component Module
## @author vikas.p@netapp.com,nraju@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::Volume

=head1 DESCRIPTION

C<NACL::C::SysManager::Volume> is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-pm>.  It represents access
to an ONTAP Volume present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::Volume|lib-NACL-CS-SysManager-Volume-pm>, which represents
the State Info of an ONTAP Volume present on the System Manger GUI Screen.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 volume

The name of the volume, used to identify it uniquely.  (Volume
names are assumed to be unique within a node or within a cluster in 7mode
and in a vserver in Cmode)

=head2 vserver

(Required for CMode, Not Applicable for 7Mode) The name of the vserver
in which the volume is present.

=cut

package NACL::C::SysManager::Volume;

use strict;
use warnings;
use Params::Validate
  qw(validate_with validate SCALAR ARRAYREF UNDEF SCALARREF );

use NACL::CS::SysManager::Volume;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $volume = NACL::C::SysManager::Volume->new(command_interface => $ci,
                                                volume            => $vol_name,
                                                vserver           => $vserver_name);

Return a new Volume component object with the given attributes.

=cut

####################### Method Maker ###############################

use Class::MethodMaker [
           new    => ['-hash', '-init', 'new'],
           scalar => 'volume',
           scalar => 'vserver',
           scalar =>
             [{-type => 'NACL::C::SysManager::SMClient'}, 'command_interface',],
];

=head2 create

  my $volume_object =
      NACL::C::SysManager::Volume->create( command_interface => $ci,
                                           volume            => $vol_name,
                                           vserver           => $vserver_name);

(Class method) Request creation of a new volume element, and  return a new
volume component object that refers to it.  Does not wait for volume
creation to complete.
Uses a 7Mode GUI or CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< volume => $string >>

(Required) Name of volume.

=item C<< aggregate => $string >>

(Required) Aggregate Name in which the Volume has to be created.

=item C<< vserver => $string >>

(Required for CMode, Not Applicable for 7Mode)The name of the vserver
in which the volume is present.

=item C<< size => $string >>

(Optional) Size of the Volume.

=item C<< type => $string >>

(Optional) Storage Type of the Volume.

=item C<< unit => $string >>

(Optional) Unit of the Volume Size (Values can be MB|GB|TB|PB)
Default Set to MB

=item C<< "snapshot-reserve" => $string >>

(Optional) Percentage of Snapshot Reservation

=item C<< "space-guarantee" => $string >>

(Optional) Thin Provisioning for Space Gurantee(Values can be enable|disable)

=item C<< dedupe => $string >>

(Optional) Storage Efficiency(Values can be enable|disable)

=item C<< compression => $string >>

(Optional) Background Compression(Values can be enable|disable)

=back

=back

=cut

####################### GUI Create ###############################
sub create {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            unit => {type => SCALAR, optional => 1, default => "MB"},
            _optional_scalars
              qw(size type aggregate snapshot-reserve space-guarantee dedupe
              compression),},);

    my $ret = $pkg->call_on_apiset(%opts,
                                   choices => [{method => '_create_7mode_gui',
                                                set    => 'SysManager_7Mode',},
                                               {method => '_create_cmode_gui',
                                                set    => 'SysManager_CMode',},
                                              ],);

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub create

####################### GUI 7Mode Create ###############################
sub _create_7mode_gui {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $vol_args = $pkg->_hash_copy(source => \%opts,
                                    map    => {
                                          "volume"           => "-volname",
                                          "size"             => "-volsize",
                                          "type"             => "-voltype",
                                          "unit"             => "-volunit",
                                          "aggregate"        => "-aggrname",
                                          "snapshot-reserve" => "-volSnapres",
                                          "space-guarantee" => "-volSpacereser",
                                          "dedupe"          => "-voldedup",
                                          "compression"     => "-compression",}
                                   );
    $apiset->volume_create(%$vol_args);

    $Log->exit() if $may_exit;

    return
      $pkg->new(volume            => $vol_args->{-volname},
                command_interface => $command_interface);
} ## end sub _create_7mode_gui

####################### GUI CMode Create ###############################
sub _create_cmode_gui {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    my %opts = $pkg->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $vol_args = $pkg->_hash_copy(source => \%opts,
                                    map    => {
                                          "vserver"          => "-vservername",
                                          "volume"           => "-volname",
                                          "size"             => "-volsize",
                                          "type"             => "-voltype",
                                          "unit"             => "-volunit",
                                          "aggregate"        => "-aggrname",
                                          "snapshot-reserve" => "-volSnapres",
                                          "space-guarantee" => "-volSpacereser",
                                          "dedupe"          => "-voldedup",
                                          "compression"     => "-compression",}
                                   );

    $apiset->volume_create(%$vol_args);
    $Log->exit() if $may_exit;

    return
      $pkg->new(volume            => $vol_args->{-volname},
                vserver           => $vol_args->{-vservername},
                command_interface => $command_interface);

} ## end sub _create_cmode_gui

=head2 delete

  NACL::C::SysManager::Volume->delete( command_interface => $ci,
                                       volume            => $vol_name,
                                       vserver           => $vserver_name);

                            or

  $volume_object->delete(...);

(Class or instance method)This method deletes a volume element.
Uses a 7Mode GUI or CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< volume=> $string >>

(Required for class method, Not Applicable for instance method)
Name of volume.

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode , Not Applicable for 7Mode) The name of the vserver
in which the volume is present.

=item C<< volumes => [$volume1, $volume2, $volume3, ... ] >>

(Required for instance method, Optional for class method) array of Volumes
 to be deleted

=back

=back

=cut

####################### GUI Delete ###############################
sub delete {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts =
      $pkg_or_obj->_common_validate_with(
                               params          => \@_,
                               additional_spec => {
                                   volumes => {type => ARRAYREF, optional => 1},
                                   volume  => {type => SCALAR,   optional => 1},
                                                  },);
    my $ret =
      $pkg_or_obj->call_on_apiset(%opts,
                                  choices => [{method => '_delete_7mode_gui',
                                               set    => 'SysManager_7Mode',},
                                              {method => '_delete_cmode_gui',
                                               set    => 'SysManager_CMode',},],
                                 );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

####################### GUI 7Mode Delete ###############################
sub _delete_7mode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    if (exists($opts{"volume"})) {

        # if volumes parameter is precent it takes precedence over volume
        # else convert "volume" parameter to "volumes"
        unless (exists($opts{"volumes"})) {
            $opts{"volumes"} = [$opts{"volume"}];
        }
    }

    my $vol_args =
      $pkg_or_obj->_hash_copy(source => \%opts,
                              map    => {"volumes" => "-volnames",});

    $apiset->delete_volume(%$vol_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_gui

####################### GUI CMode Delete ###############################
sub _delete_cmode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    if (exists($opts{"volume"})) {

        # if volumes parameter is precent it takes precedence over volume
        # else convert "volume" parameter to "volumes"
        unless (exists($opts{"volumes"})) {
            $opts{"volumes"} = [$opts{"volume"}];
        }
    }

    my $vol_args =
      $pkg_or_obj->_hash_copy(source => \%opts,
                              map    => {
                                      "volumes" => "-volnames",
                                      "vserver" => "-vservername"});

    $apiset->delete_volume(%$vol_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_gui

=head2 modify

  NACL::C::SysManager::Volume->modify( command_interface => $ci,
                                       volume            => $vol_name,
                                       vserver           => $vserver_name,...);

                            or

  $volume_object->modify(...);

(Class or instance method) Modify a volume element.
Uses a  7Mode GUI or CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< volume => $string >>

(Required for class method, Not Applicable for instance method)
Name of volume.

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode , Not Applicable for 7Mode)The name of the vserver
in which the volume is present.

=item C<< "volume-rename" => $string >>

(Optional) Rename of the Volume

=item C<< style => $string >>

(Optional) Security style of the Volume

=item C<< "space-guarantee" => $string >>

(Optional) Thin Provisioning for Space Gurantee(Values can be enable|disable)

=item C<< "auto-grow" => $string >>

(Optional) Automatically Grow of the volume(Values can be enable|disable)

=item C<< "max-auto-grow" => $string >>

(Optional) Maximum Auto Grow size of the volume

=item C<< "max-auto-grow-unit" => $string >>

(Optional) Maximum Auto Grow size unit of the volume(Values can be KB|MB|GB|TB)

=item C<< "snap-autodelete-enabled" => $string >>

(Optional) Automatically delete older Snapshot Copies(Values can be enable|disable)

=item C<< "unicode-support" => $string >>

(Optional) Support Unicode format for directories(Values can be enable|disable)

=item C<< "access-time-update" => $string >>

(Optional) Update Access Time when a file is read(Values can be enable|disable)

=item C<< "dedup-enable" => $string >>

(Optional) Enable Storage Efficiency(Values can be enable|disable)

=item C<< "dedup-method" => $string >>

(Optional) Deduplication Method(Values can be manually|automatic|schedule|customschedule)

=item C<< "compression" => $string >>

(Optional) Deduplication Compression(Values can be none|inline|background)

=back

=back

=cut

####################### GUI Modify ###############################
sub modify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw (volume-rename style space-guarantee auto-grow max-auto-grow
                  max-auto-grow-unit snap-autodelete-enabled unicode-support
                  access-time-update dedup-enable dedup-method compression
                  effpolicy compressioninline
                  )),},);

    my $ret =
      $pkg_or_obj->call_on_apiset(%opts,
                                  choices => [{method => '_modify_7mode_gui',
                                               set    => 'SysManager_7Mode',},
                                              {method => '_modify_cmode_gui',
                                               set    => 'SysManager_CMode',},],
                                 );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

####################### GUI 7Mode Modify ###############################
sub _modify_7mode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj        = shift;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@_);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $vol_args =
      $pkg_or_obj->_hash_copy(source => \%opts,
                              map    => {
                                  "volume"             => "-volname",
                                  "volume-rename"      => "-volrename",
                                  "style"              => "-volstyle",
                                  "space-guarantee"    => "-volSpacereser",
                                  "auto-grow"          => "-volgrow",
                                  "max-auto-grow"      => "-volautomaxgrow",
                                  "max-auto-grow-unit" => "-maxunit",
                                  "snap-autodelete-enabled" => "-volsnapdel",
                                  "unicode-support"         => "-volunicode",
                                  "access-time-update"      => "-volacctime",
                                  "dedup-enable"            => "-dedup",
                                  "dedup-method"            => "-voldedup",
                                  "compression"       => "-volcompression",
                                  "compressioninline" => "-volcompressioninline"
                              });
    $apiset->volume_edit(%$vol_args);
    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_gui

####################### GUI CMode Modify ###############################
sub _modify_cmode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $vol_args =
      $pkg_or_obj->_hash_copy(source => \%opts,
                              map    => {
                                  "volume"             => "-volname",
                                  "vserver"            => "-vservername",
                                  "volume-rename"      => "-volrename",
                                  "style"              => "-volstyle",
                                  "space-guarantee"    => "-volSpacereser",
                                  "auto-grow"          => "-volgrow",
                                  "max-auto-grow"      => "-volautomaxgrow",
                                  "max-auto-grow-unit" => "-maxunit",
                                  "snap-autodelete-enabled" => "-volsnapdel",
                                  "unicode-support"         => "-volunicode",
                                  "access-time-update"      => "-volacctime",
                                  "dedup-enable"            => "-dedup",
                                  "dedup-method"            => "-voldedup",
                                  "compression"       => "-volcompression",
                                  "effpolicy"         => "-effpolicy",
                                  "compressioninline" => "-volcompressioninline"
                              });

    $apiset->volume_edit(%$vol_args);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_gui

=head2 resize

  NACL::C::SysManager::Volume->resize(command_interface  => $ci,
                                      volume            => $vol_name,
                                      vserver           => $vserver_name,...);

                            or

  $volume_object->resize(...);

(Class or instance method) This method resizes a volume element.
Uses a CMode GUI or 7Mode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< volume => $string >>

(Required for class method, Not Applicable for instance method)
Name of volume.

=item C<< vserver =>$string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode , Not Applicable for 7Mode)The name of the vserver
in which the volume is present.

=item C<< "resize" => $string >>

=item C<< "unit" => $string >>

=item C<< "volume-reserve" => $string >>

=item C<< "snapshot" => $string >>

=back

=back

=cut

####################### GUI Resize ###############################
sub resize {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts =
      $pkg_or_obj->_common_validate_with(
                   params          => \@_,
                   additional_spec => {
                       unit => {type => SCALAR, optional => 1, default => "MB"},
                       _optional_scalars(qw(resize volume-reserve snapshot)),},
      );

    my $ret =
      $pkg_or_obj->call_on_apiset(%opts,
                                  choices => [{method => '_resize_cmode_gui',
                                               set    => 'SysManager_CMode',},
                                              {method => '_resize_7mode_gui',
                                               set    => 'SysManager_7Mode',},],
                                 );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub resize

####################### GUI 7Mode Resize ###############################
sub _resize_7mode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj        = shift;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@_);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $vol_args =
      $pkg_or_obj->_hash_copy(source => \%opts,
                              map    => {
                                      "volume"         => "-volname",
                                      "resize"         => "-resize",
                                      "unit"           => "-sizeunit",
                                      "volume-reserve" => "-volreserv",
                                      "snapshot"       => "-snapshot",});

    $apiset->volume_resize(%$vol_args);
    $Log->exit() if $may_exit;
} ## end sub _resize_7mode_gui

####################### GUI CMode Resize ###############################
sub _resize_cmode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj        = shift;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@_);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $vol_args =
      $pkg_or_obj->_hash_copy(source => \%opts,
                              map    => {
                                      "volume"         => "-volname",
                                      "vserver"        => "-vservername",
                                      "resize"         => "-resize",
                                      "unit"           => "-sizeunit",
                                      "volume-reserve" => "-volreserv",
                                      "snapshot"       => "-snapshot",});

    $apiset->volume_resize(%$vol_args);
    $Log->exit() if $may_exit;
} ## end sub _resize_cmode_gui

=head2 offline

  NACL::C::SysManager::Volume->offline(command_interface => $ci,
                                       volume            => $vol_name,
                                       vserver           => $vserver_name,...);

                            or

  $volume_object->offline(...);

(Class or instance method) This method offlines a volume element.
Uses a CMode GUI or 7Mode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< volume => $string >>

(Required for class method, Not Applicable for instance method)
Name of volume.

=item C<< vserver =>$string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode , Not Applicable for 7Mode)The name of the vserver
in which the volume is present.

=item C<< "status" => $string >>

(Optional) Status to be set to the Volume(Value must be offline)

=back

=back

=cut

####################### GUI Offline ###############################
sub offline {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts =
      $pkg_or_obj->_common_validate_with(
                            params          => \@_,
                            additional_spec => {_optional_scalars(qw(status)),},
      );

    my $ret =
      $pkg_or_obj->call_on_apiset(%opts,
                                  choices => [{method => '_offline_cmode_gui',
                                               set    => 'SysManager_CMode',},
                                              {method => '_offline_7mode_gui',
                                               set    => 'SysManager_7Mode',},],
                                 );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub offline

####################### GUI 7Mode Offline ###############################
sub _offline_7mode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj        = shift;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@_);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $vol_args =
      $pkg_or_obj->_hash_copy(source => \%opts,
                              map    => {
                                      "volume" => "-volname",
                                      "status" => "-status",});
    $vol_args->{'-status'} = "offline";

    $apiset->volume_status(%$vol_args);
    $Log->exit() if $may_exit;
} ## end sub _offline_7mode_gui

####################### GUI CMode Offline ###############################
sub _offline_cmode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj        = shift;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@_);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $vol_args =
      $pkg_or_obj->_hash_copy(source => \%opts,
                              map    => {
                                      "volume"  => "-volname",
                                      "vserver" => "-vservername",
                                      "status"  => "-status",});
    $vol_args->{'-status'} = "offline";

    $apiset->volume_status(%$vol_args);
    $Log->exit() if $may_exit;
} ## end sub _offline_cmode_gui

=head2 online

  NACL::C::SysManager::Volume->online( command_interface => $ci,
                                       volume            => $vol_name,
                                       vserver           => $vserver_name,...);

                            or

  $volume_object->online(...);

(Class or instance method) This method onlines a volume element.
Uses a CMode GUI or 7Mode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< volume => $string >>

(Required for class method, Not Applicable for instance method)
Name of volume.

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode , Not Applicable for 7Mode)The name of the vserver
in which the volume is present.

=item C<< "status" => $string >>

(Optional) Status to be set to the Volume(Value must be online)

=back

=back

=cut

####################### GUI Online ###############################
sub online {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts =
      $pkg_or_obj->_common_validate_with(
                            params          => \@_,
                            additional_spec => {_optional_scalars(qw(status)),},
      );

    my $ret =
      $pkg_or_obj->call_on_apiset(%opts,
                                  choices => [{method => '_online_cmode_gui',
                                               set    => 'SysManager_CMode',},
                                              {method => '_online_7mode_gui',
                                               set    => 'SysManager_7Mode',},],
                                 );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub online

####################### GUI 7Mode Online ###############################
sub _online_7mode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj        = shift;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@_);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $vol_args =
      $pkg_or_obj->_hash_copy(source => \%opts,
                              map    => {
                                      "volume" => "-volname",
                                      "status" => "-status",});
    $vol_args->{'-status'} = "online";

    $apiset->volume_status(%$vol_args);
    $Log->exit() if $may_exit;
} ## end sub _online_7mode_gui

####################### GUI CMode Online ###############################
sub _online_cmode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj        = shift;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@_);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $vol_args =
      $pkg_or_obj->_hash_copy(source => \%opts,
                              map    => {
                                      "volume"  => "-volname",
                                      "vserver" => "-vservername",
                                      "status"  => "-status",});
    $vol_args->{'-status'} = "online";

    $apiset->volume_status(%$vol_args);
    $Log->exit() if $may_exit;
} ## end sub _online_cmode_gui

=head2 restrict

  NACL::C::SysManager::Volume->restrict(command_interface => $ci,
                                        volume            => $vol_name,
                                        vserver           => $vserver_name,...);

                            or

  $volume_object->restrict();

(Class or instance method) This method restricts a volume element.
Uses a 7Mode GUI or CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< volume => $string >>

(Required for class method, Not Applicable for instance method)
Name of volume.

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode, Not Applicable for 7Mode)The name of the vserver
in which the volume is present.

=item C<< "status" => $string >>

(Optional) Status to be set to the Volume(Value must be restrict)

=back

=back

=cut

####################### GUI Restrict ###############################
sub restrict {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts =
      $pkg_or_obj->_common_validate_with(
                            params          => \@_,
                            additional_spec => {_optional_scalars(qw(status)),},
      );

    my $ret =
      $pkg_or_obj->call_on_apiset(%opts,
                                  choices => [{method => '_restrict_7mode_gui',
                                               set    => 'SysManager_7Mode',},
                                              {method => '_restrict_cmode_gui',
                                               set    => 'SysManager_CMode',},],
                                 );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub restrict

####################### GUI 7Mode Restrict ###############################
sub _restrict_7mode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj        = shift;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@_);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $vol_args =
      $pkg_or_obj->_hash_copy(source => \%opts,
                              map    => {
                                      "volume" => "-volname",
                                      "status" => "-status",});
    $vol_args->{'-status'} = "restrict";

    my $response = $apiset->volume_status(%$vol_args);
    $Log->exit() if $may_exit;
} ## end sub _restrict_7mode_gui

####################### GUI CMode Restrict ###############################
sub _restrict_cmode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj        = shift;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@_);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $vol_args =
      $pkg_or_obj->_hash_copy(source => \%opts,
                              map    => {
                                      "volume"  => "-volname",
                                      "vserver" => "-vservername",
                                      "status"  => "-status",});
    $vol_args->{'-status'} = "restrict";

    $apiset->volume_status(%$vol_args);
    $Log->exit() if $may_exit;
} ## end sub _restrict_cmode_gui

=head2 refresh

  NACL::C::SysManager::Volume->refresh(command_interface => $ci,
                                       vserver           => $vserver);

                            or

  $volume_object->refresh();

(Class or instance method) This method restricts a volume element.
Uses a 7Mode GUI or CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode, Not Applicable for 7Mode)The name of the vserver
in which the volume is present.

=back

=back

=cut

####################### GUI Refresh ####################################
sub refresh {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@_,);

    my $ret =
      $pkg_or_obj->call_on_apiset(%opts,
                                  choices => [{method => '_refresh_7mode_gui',
                                               set    => 'SysManager_7Mode',},
                                              {method => '_refresh_cmode_gui',
                                               set    => 'SysManager_CMode',},],
                                 );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub refresh

####################### GUI 7Mode Refresh ###############################
sub _refresh_7mode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj        = shift;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@_);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $response          = $apiset->refresh_volume();
    $Log->exit() if $may_exit;
}

####################### GUI CMode Refresh ###############################
sub _refresh_cmode_gui {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $vol_args =
      $pkg_or_obj->_hash_copy(source => \%opts,
                              map    => {"vserver" => "-vservername"});

    $apiset->refresh_volume(%$vol_args);

    $Log->exit() if $may_exit;
} ## end sub _refresh_cmode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return (volume  => {type => SCALAR},
            vserver => {type => SCALAR | UNDEF, optional => 1},);
}

1
