# $Id$
#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary VolumeEfficiency Component Module
## @author vikas.p@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::VolumeEfficiency

=head1 DESCRIPTION

C<NACL::C::SysManager::VolumeEfficiency> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>.  It represents access
to an ONTAP Volume Efficiency present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::VolumeEfficiency|lib-NACL-CS-SysManager-VolumeEfficiency-pm>, which represents
the State Info of an ONTAP Volume Efficiency present on the System Manger GUI Screen.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 volume

The name of the volume on which to manage efficiency. (Volume
names are assumed to be unique within a node or within a cluster in 7mode
and in a vserver in Cmode)

=head2 vserver

(Required for CMode, Not Applicable for 7Mode) The name of the vserver 
in which the volume efficiency is present.

=cut

package NACL::C::SysManager::VolumeEfficiency;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::VolumeEfficiency;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

	my $volEff_obj = 
		NACL::C::SysManager::VolumeEfficiency->new(command_interface => $ci,
												   vserver           => $vserver,
												   volume            => "my_volume");

Return a new VolumeEfficiency component object with the given attributes.

=cut

####################### Method Maker ###############################
use Class::MethodMaker [
    scalar => 'volume',
    scalar => 'vserver',
];

=head2 start 

	NACL::C::VolumeEfficiency->start(command_interface => $ci,
									 volume            => $vol_name,
									 vserver           => $vserver_name,);

(Class method)This method starts volume efficiency.
Uses CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< volume => $string >>

(Required) Name of volume.

=item C<< vserver => $string >>

(Required for CMode, Not Applicable for 7Mode) The name of the vserver
in which the volume is present.

=item C<< state => $boolean >>

(Optional) The state of the volume efficiency to be set

=item C<< type => $boolean >>

(Optional) type of the Volume Efficiency


=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => { _optional_scalars qw( type ) }
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_start_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_start_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub start

####################### GUI CMode Start ###############################
sub _start_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $vol_eff_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "volume"  => "-volname",
            "vserver" => "-vservername",
            "type"    => "-type",
        }
    );

    $vol_eff_args->{'-state'} = "start";
    $apiset->volume_dedupe(%$vol_eff_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        volume            => $vol_eff_args->{-volname},
        vserver           => $vol_eff_args->{-vservername},
        command_interface => $command_interface
    );
} ## end sub _start_cmode_gui

####################### GUI 7Mode Start ###############################
sub _start_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $vol_eff_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "volume" => "-volname",
            "type"   => "-type",
        }
    );

    $vol_eff_args->{'-state'} = "start";
    $apiset->volume_dedupe(%$vol_eff_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        volume            => $vol_eff_args->{-volname},
        command_interface => $command_interface
    );
} ## end sub _start_7mode_gui

=head2 stop 

	NACL::C::SysManager::VolumeEfficiency->stop(command_interface => $ci,
												volume            => $vol_name,
												vserver           => $vserver_name, ...);

                            or

	$volEff_obj->stop(...);

(Class or instance method)This method stops the volume efficiency.
Uses CMode GUI or 7Mode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< volume => $string >>

(Required for class method, Not Applicable for instance method)
Name of volume.

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
(Required for CMode, Not Applicable for 7Mode) The name of the vserver
in which the volume is present.

=item C<< state => $boolean >>

(Optional) The state of the volume efficiency to be set

=item C<< type => $boolean >>

(Optional) type of the Volume Efficiency


=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => { _optional_scalars qw(all type), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_stop_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_stop_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub stop

####################### GUI CMode Stop ###############################
sub _stop_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $vol_eff_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "volume"  => "-volname",
            "vserver" => "-vservername",
            "type"    => "-type",
        }
    );

    $vol_eff_args->{'-state'} = "abort";

    $apiset->volume_dedupe(%$vol_eff_args);
    $Log->exit() if $may_exit;

} ## end sub _stop_cmode_gui

####################### GUI 7Mode Stop ###############################
sub _stop_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $vol_eff_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "volume" => "-volname",
            "type"   => "-type",
        }
    );

    $vol_eff_args->{'-state'} = "abort";

    $apiset->volume_dedupe(%$vol_eff_args);
    $Log->exit() if $may_exit;
} ## end sub _stop_7mode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return (
        volume  => { type => SCALAR },
        vserver => { type => SCALAR | UNDEF, optional => 1 },
    );
} ## end sub _primary_keys_validate_spec

1;
