# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary VolumeEfficiencyPolicy Component Module
## @author vikas.p@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::VolumeEfficiencyPolicy

=head1 DESCRIPTION

C<NACL::C::SysManager::VolumeEfficiencyPolicy> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>.  It represents access
to an ONTAP Volume Efficiency Policy present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::VolumeEfficiencyPolicy|lib-NACL-CS-SysManager-VolumeEfficiencyPolicy-pm>, 
which represents the State Info of an ONTAP Volume Efficiency Policy present on the System Manger 
GUI Screen.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

The name of the vserver in which the volume efficiency policy is present.

=head2 policy

The name of the policy, used to identify it uniquely. (policy
names are assumed to be unique within a node or within a vserver in Cmode)

=cut

package NACL::C::SysManager::VolumeEfficiencyPolicy;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::VolumeEfficiencyPolicy;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

	my $VolumeEfficiencyPolicy_object = 
		NACL::C::SysManager::VolumeEfficiencyPolicy->new(command_interface => $ci,
														 vserver           => $vserver,
														 policy            => $policy,...);

Return a new VolumeEfficiencyPolicy component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'policy',
];

=head2 create

    my $VolumeEfficiencyPolicy_object = 
		NACL::C::SysManager::VolumeEfficiencyPolicy->create(command_interface => $ci,
															vserver           => $vserver,
															policy            => $policy,...);
   
(Class method)This method is used to create an efficiency policy.
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $string >>

The name of the vserver in which the policy is present.

=item C<< "policy" => $string >>

(Required) Name of the Policy.

=item C<< "schedule" => $string >>

(Optional) Schedule to be added to the policy

=item C<< "maximum-runtime" => $string >>

(Optional) Maximum Runtime

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params => \@args,
        additional_spec =>
            { _optional_scalars(qw( schedule maximum-runtime )), },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_create_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub create

####################### GUI CMode Create ###############################
sub _create_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $voleffpol_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "policy"          => "-policyName",
            "vserver"         => "-vservername",
            "schedule"        => "-schedule",
            "maximum-runtime" => "-maxruntime",
        }
    );

    $apiset->dedupe_policy_add(%$voleffpol_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        policy            => $voleffpol_args->{-policyName},
        vserver           => $voleffpol_args->{-vservername},
        command_interface => $command_interface
    );
} ## end sub _create_cmode_gui

=head2 modify

	NACL::C::SysManager::VolumeEfficiencyPolicy->modify(command_interface => $ci,
														vserver           => $vserver,
														policy            => $policy,);
	                             
								or

    $VolumeEfficiencyPolicy_object->modify(...);

(Class or instance method)This method is used to modify an efficiency policy.
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $string >>

(Required for class method, Not Applicable for instance method)
The name of the vserver in which the policy is present.

=item C<< "policy" => $string >>

(Required for class method, Not Applicable for instance method)
Name of the Policy.

=item C<< "schedule" => $string >>

(Optional) Schedule to be added to the policy

=item C<< "maximum-runtime" => $string >>

(Optional) Maximum Runtime

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            { _optional_scalars(qw( schedule maximum-runtime )), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_modify_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

####################### GUI CMode Modify ###############################
sub _modify_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $voleffpol_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "policy"          => "-policyName",
            "vserver"         => "-vservername",
            "schedule"        => "-schedule",
            "maximum-runtime" => "-maxruntime",
        }
    );

    $apiset->dedupe_policy_edit(%$voleffpol_args);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_gui

=head2 delete

	NACL::C::SysManager::VolumeEfficiencyPolicy->delete(command_interface => $ci,
														vserver           => $vserver,
														policy            => $policy,...);
	                             
								 or

    $VolumeEfficiencyPolicy_object->delete(...);

(Class or instance method)
This method is used to delete an efficiency policy.
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $string >>

(Required for class method, Not Applicable for instance method)
The name of the vserver in which the policy is present.

=item C<< "policy" => $string >>

(Required for class method, Not Applicable for instance method)
Name of the Policy.

=item C<< "confirm" => $string >>

(Optional) Enable to confirm the deletion

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => { _optional_scalars(qw( confirm )), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_delete_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

####################### GUI CMode Delete ###############################
sub _delete_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $voleffpol_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "policy"  => "-policyName",
            "vserver" => "-vservername",
            "confirm" => "-checkbox",
        }
    );

    $apiset->dedupe_policy_delete(%$voleffpol_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_gui

=head2 disable

	NACL::C::SysManager::VolumeEfficiencyPolicy->disable(command_interface => $ci,
														 volume            => $aggr_name,
														 vserver           => $vserver_name,...);

                            or

	$VolumeEfficiencyPolicy_object->disable(...);

(Class or instance method) This method disables a Volume Efficiency element.
Uses a CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $string >>

(Required for class method, Not Applicable for instance method)
The name of the vserver in which the policy is present.

=item C<< "policy" => $string >>

(Required for class method, Not Applicable for instance method)
Name of the Policy.


=back

=back

=cut

sub disable {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_disable_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub disable

####################### GUI CMode Disable ###############################
sub _disable_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $voleffpol_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "policy"  => "-policyName",
            "vserver" => "-vservername",
        }
    );

    $voleffpol_args->{'-status'} = "disable";

    $apiset->dedupe_policy_status(%$voleffpol_args);
    $Log->exit() if $may_exit;
} ## end sub _disable_cmode_gui

=head2 enable

	NACL::C::SysManager::VolumeEfficiencyPolicy->enable(command_interface => $ci,
														volume            => $vol_name,
														vserver           => $vserver_name,...);

                            or

	$VolumeEfficiencyPolicy_object->enable(...);

(Class or instance method) This method enable Volume Efficiency element.
Uses a CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $string >>

(Required for class method, Not Applicable for instance method)
The name of the vserver in which the policy is present.

=item C<< "policy" => $string >>

(Required for class method, Not Applicable for instance method)
Name of the Policy.


=back

=back

=cut

sub enable {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_enable_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub enable

####################### GUI CMode Enable ###############################
sub _enable_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $voleffpol_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "policy"  => "-policyName",
            "vserver" => "-vservername",
        }
    );

    $voleffpol_args->{'-status'} = "enable";

    $apiset->dedupe_policy_status(%$voleffpol_args);
    $Log->exit() if $may_exit;
} ## end sub _enable_cmode_gui

=head2 refresh

	NACL::C::SysManager::VolumeEfficiencyPolicy->refresh(command_interface => $ci,
														 vserver           => $vserver_name,...);

                            or

	$VolumeEfficiencyPolicy_object->refresh();

(Class or instance method) This method refreshes a volume efficiency policy page.
Uses a CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $string >>

(Required for class method, Not Applicable for instance method)
The name of the vserver in which the policy is present.

=back

=back

=cut

sub refresh {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_refresh_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub refresh

####################### GUI CMode Refresh ###############################
sub _refresh_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $voleffpol_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "vserver" => "-vservername" }
    );

    $apiset->dedupe_policy_refresh(%$voleffpol_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_cmode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return (
        'vserver' => { type => SCALAR },
        'policy'  => { type => SCALAR },
    );
} ## end sub _primary_keys_validate_spec

1;
