# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary VolumeMove Component Module
## @author Neethu.George@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::VolumeMove

=head1 DESCRIPTION

C<NACL::C::SysManager::VolumeMove> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>.  It represents access
to an ONTAP VolumeMove present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::VolumeMove|lib-NACL-CS-SysManager-VolumeMove-pm>, which represents
the State Info of an ONTAP VolumeMove present on the System Manger GUI Screen.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 volume

The source volume to be moved. (Volume
names are assumed to be unique in a vserver in Cmode)

=head2 srcaggr

The source aggregate from which the volume has to be moved

=head2 vserver

The name of the vserver in which the volume is present. 

=cut

package NACL::C::SysManager::VolumeMove;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::VolumeMove;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

	my $VolumeMove_object = 
		NACL::C::SysManager::VolumeMove->new(command_interface => $ci,
											 vserver           => $vserver_name,
											 volume            => $volume_name,);

Return a new VolumeMove component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'volume',
    scalar => 'vserver',
    scalar => 'srcaggr',
];

=head2 move

    my $VolumeMove_object = 
		NACL::C::SysManager::VolumeMove->move(command_interface => $ci,
											  volume            => $vol,
											  srcaggr  => $aggr,
											  vserver           => $vs,...);
   
(Class method)This method is used to start moving a volume from 
one aggregate to another aggregate. It returns a new VolumeMove component object.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< volume => $volume >>

(Required)The volume to be moved.

=item C<< vserver => $vserver >>

(Required)The vserver on which the source volume is present.

=item C<< "destaggr" => $string >>

(Required) The Destination Aggregate to which volume has to be moved

=item C<< "srcaggr" => $string >>

(Required) The Source Aggregate from which volume has to be moved

=back

=back

=cut

sub move {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "destaggr" => { type => SCALAR }
        }
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_move_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub move

####################### GUI CMode move ###############################
sub _move_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $volumemove_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "volume"   => "-volName",
            "vserver"  => "-vservername",
            "destaggr" => "-destaggr",
            "srcaggr"  => "-srcaggr"
        }
    );

    $apiset->volume_move(%$volumemove_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        volume            => $volumemove_args->{-volName},
        "srcaggr"         => $volumemove_args->{-srcaggr},
        vserver           => $volumemove_args->{-vservername},
        command_interface => $command_interface
    );
} ## end sub _move_cmode_gui

=head2 movefrom_aggr

    my $VolumeMove_object = 
		NACL::C::SysManager::VolumeMove->movefrom_aggr(command_interface => $ci,
													   volume            => $vol,
													   srcaggr  => $aggr,
													   vserver           => $vs,...);
   
(Class method)This method is used to start moving a volume 
from one aggregate to another aggregate. It returns a new VolumeMove component object.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< volume => $volume >>

(Required)
The volume to be moved.

=item C<< vserver => $vserver >>

(Required)
The vserver on which the source volume is present.

=item C<< "destaggr" => $string >>

(Required) The Destination Aggregate to which volume has to be moved

=item C<< "srcaggr" => $string >>

(Required) The Source Aggregate from which volume has to be moved

=back

=back

=cut

sub movefrom_aggr {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "destaggr" => { type => SCALAR }
        }
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_movefrom_aggr_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub movefrom_aggr

####################### GUI CMode movefrom_aggr ###############################
sub _movefrom_aggr_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $volumemove_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "volume"   => "-volName",
            "vserver"  => "-vservername",
            "destaggr" => "-destaggr",
            "srcaggr"  => "-srcaggr"
        }
    );

    $apiset->volume_move_aggr(%$volumemove_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        "srcaggr"          => $volumemove_args->{-srcaggr},
        volume             => $volumemove_args->{-volName},
        vserver            => $volumemove_args->{-vservername},
        command_interface  => $command_interface
    );
} ## end sub _movefrom_aggr_cmode_gui

##################################################################################

sub _primary_keys_validate_spec {
    return (
        volume    => { type => SCALAR },
        vserver   => { type => SCALAR },
        "srcaggr" => { type => SCALAR },
    );
} ## end sub _primary_keys_validate_spec

1;
