# $Id$
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary SystemLicense Component Module
## @author dheeraj.k@netapp.com, dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemLicense

=head1 DESCRIPTION

C<NACL::C::SystemLicense> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP System SystemLicense.  A related class is
L<NACL::CS::SystemLicense|lib-NACL-CS-SystemLicense-pm>, which represents
the past state of an ONTAP SystemLicense. Licensing v2 support is available for
cmode cli/zapi and 7mode cli.

=head1 LICENSING FORMATS

Starting from SN.0 (for both CMode and 7Mode), the format of the license keys
changed. This is referred to as "Licensing V2". The license keys format
prior to SN.0 is referred to as "Licensing V1". This is the meaning
of the terminology used in the documentation below. (In short, unless pre-SN
7Mode testing is being performed, Licensing V2 is active.)

Here's a reference wiki: https://wikid.netapp.com/w/Licensing/licensing_2.0_faq

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 feature

The name of the license feature, used to identify it uniquely. This attribute is only
applicable for licensing v1 (SystemLicense feature names are assumed to be unique
within a node or within a cluster).

=head2 package

The name of the license package. This attribute is only applicable for licensing v2.

=head2 serial_number

Cluster Base license serial number. This attribute is only applicable for licensing v2.

=cut

package NACL::C::SystemLicense;
use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::CS::SystemLicense;
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use base 'NACL::C::Component::ONTAP';
use Params::Validate
    qw(validate validate_with SCALAR OBJECT ARRAYREF HASHREF);
use NATE::Exceptions::Argument qw(:try);
use NACL::C::_Mixins::SystemLicense
    qw(_map_feature_to_package _is_v2_7m_zapi);
use NACL::C::Exceptions::MultipleCommandFailed;
use NACL::C::Exceptions::Component qw(:try);
use NACL::C::Exceptions::SystemLicense::DoesNotExist;
use NACL::C::Exceptions::SystemLicense::NoLicenseCodeIsNecessary;
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;

=head1 METHODS

=head2 new

  # For Licensing V1
  my $license = NACL::C::SystemLicense->new(
      command_interface => $ci,
      feature           => $feature
  );

  or

  # For Licensing V2
  my $license = NACL::C::SystemLicense->new(
      command_interface => $ci,
      package           => $package,
      'serial-number'   => $serial_number
  );

Return a new license component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'feature',
    scalar => 'package',
    scalar => 'serial_number',
];

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::C::License'
    );
    $Log->exit() if $may_exit;
    return $isa;
} ## end sub isa

=head2 add

  For Licensing V1:

  my $license = NACL::C::SystemLicense->add(
      command_interface => $ci,
      'license-code'    => $license_code [,'feature' => $feature]
  );

  For Licensing V2:

  my $license = NACL::C::SystemLicense->add(
      command_interface => $ci,
      'license-code'    => $license_code,
      package           => $package
  );

  or

  my @licenses = NACL::C::SystemLicense->add(
      command_interface => $ci,
      'license-code'    => [$license_code1, $license_code2, .. ],
      package           => [$package1, $package2, .. ]
  );

(Class method) Request addition of a new license element, and  return a new
license component object that refers to it.

Uses a CMode-CLI/ZAPI or 7Mode-CLI or 7Mode-ZAPI APISet.

Invokes "interim-license-set,api" API for CMode ZAPI.

Invokes "license add" command for 7Mode CLI.

Invokes "license-add,license-v2-add" command for 7Mode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'license-code' => $license_code >>

(Required) SystemLicense code for the feature we want to license

Applicable for CMode CLI/ZAPI, 7Mode CLI/ZAPI.

Maps to:

CM ZAPI: code

7M CLI : SystemLicense code for the feature (Programmatic name: "code")

7M ZAPI: code

=item C<< 'feature' => $feature >>

(Optional for CMode CLI Licensing V1, required for 7Mode licensing v1)
Feature to be licensed.

(This needs to be provided for 7Mode because the component object created
contains the feature licensed. While in CMode it is possible to find out
the feature for a particular license code, in 7Mode, both CLI and ZAPI, there
is no way to do this. For this reason, we must provide 'feature', which will
be used in the newly created component object)

Applicable for CMode CLI, 7Mode CLI.

=item C<< 'package' => $package E<verbar> [ $package1, $package2, .. ] >>

(Required, for Licensing V2. Currently supports CMode CLI/ZAPI and 7Mode CLI )
Feature to be licensed.

Applicable to CMode CLI/ZAPI, 7Mode CLI

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"system license add" command in ONTAP.

Applicable for CMode CLI.



=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::SystemLicense::NoLicenseCodeIsNecessary>

This exception is thrown when we try to add license code which is
already included and no license code is necessary.

=item C<NACL::C::Exceptions::SystemLicense::DoesNotExist>

This exception is thrown when we try to remove a non-existing license.

=back

=back

=cut

sub add {
    my ($pkg, @args) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'license-code' => {type => SCALAR | ARRAYREF, optional => 1},
            package        => {type => SCALAR | ARRAYREF, optional => 1},

            # "nacl_serial_number" is deliberately left undocumented. It is
            # primarily used by the STask. Normally, each call would require
            # a query to determine the serial-number, but if the serial-number
            # is already known, it can be passed in to prevent querying
            # for it every single time.
            _optional_scalars(qw(feature nacl_serial_number license-input)),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'system license add',
    );

    my @objs;
    try {
        @objs = $pkg->call_on_apiset_with_exceptions(
            %opts,
            choices    => [
                {   method    => "_add_cmode_cli_v2",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_add_cmode_zapi_v2",
                    interface => "ZAPI",
                    set       => "CMode",
                },
                {   method    => "_add_7mode_cli_v2",
                    interface => "CLI",
                    set       => "7Mode|Nodescope",
                    check     => "_is_v2"
                },
                {   method    => "_add_7mode_cli_v1",
                    interface => "CLI",
                    set       => "7Mode|Nodescope",
                    check     => "_is_v1"
                },
                {   method    => "_add_7mode_zapi_v1",
                    interface => "ZAPI",
                    set       => "7Mode",
                    check     => "_is_v1"
                },
                {   method    => "_add_7mode_zapi_v2",
                    interface => "ZAPI",
                    set       => "7Mode",
                    check     => "_is_v2_7m_zapi"
                }
            ],
            frontend => 'NACL::C::SystemLicense::add',
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::SystemLicense::NoLicenseCodeIsNecessary
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
    return wantarray ? @objs : $objs[0];
} ## end sub add

sub _add_cmode_cli_v2 {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface  = $opts{command_interface};

    my $serial_number = delete $opts{nacl_serial_number};
    my $get_serial_num = sub {
        # If "nacl_serial_number" is provided, use that as the base serial-number,
        # else query for it.
        $serial_number //= $pkg->find_base_serial_number_cm(
            command_interface => $command_interface,
            apiset_should     => {interface => 'CLI'},
        );
        return $serial_number;
    };

    my $response;
    $pkg->_base_backend_cmode_cli(
        %opts,
        api                 => 'system_license_add',
        apiset_response_obj => \$response,
    );
    my $parsed_output = $response->get_parsed_output();

    my @components;
    foreach my $row (@$parsed_output) {
        my %license_opts = (package => $row->{package});
        if (exists $row->{'serial-number'}) {
            # The parsed output containing serial-number indicates that it's
            # a node-locked license, in which case this is the serial-number
            # of the node
            $license_opts{'serial-number'} = $row->{'serial-number'};
        } else {
            # For non-node-locked licenses, serial-number is not present in
            # the output. In this case, we need to use the base serial-number
            $license_opts{'serial-number'} = $get_serial_num->();
        }
        push @components, $pkg->new(
            command_interface => $command_interface,
            %license_opts,
        );
    }
    
    $Log->exit() if $may_exit;
    return @components;
} ## end sub _add_cmode_cli_v2

sub _add_cmode_zapi_v2 {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my $license_input =     $opts{'license-input'} if(defined($opts{'license-input'}));
    my @components;
    if(defined $license_input){
       $pkg->_base_backend_cmode_zapi(
              %opts,
              api  => 'license-v2-capacity-create',
              copy => [ qw (license-input)]);
    }

    else {
        my $serial_number = delete $opts{nacl_serial_number};
        my $get_serial_num = sub {
            # If "nacl_serial_number" is provided, use that as the base serial-number,
            # else query for it.
            $serial_number //= $pkg->find_base_serial_number_cm(
                command_interface => $command_interface,
                apiset_should     => {interface => 'ZAPI'},
            );
            return $serial_number;
        };

        # license-code might be provided as a scalar or as an array. If provided
        # as a scalar, split on comma to turn into an array
        my @licenses;
        if (ref $opts{'license-code'} eq 'ARRAY') {
            @licenses = @{$opts{'license-code'}};
        } else {
            @licenses = split /,/, $opts{'license-code'};
        }

        my $response      = $apiset->license_v2_add(codes => \@licenses);
        my $parsed_output = $response->get_parsed_output();
        my $first_entry   = $parsed_output->[0];

        if (exists $first_entry->{'license-v2-failure-list'}) {
            # One or more of the license additions failed. Construct an exception
            # and throw it.
            my $failures = $first_entry->{'license-v2-failure-list'}[0]{'license-v2-result'};
            my $err_msg = "One or more of the license additions failed:\n";
            foreach my $failure_hash (@$failures) {
                while (my ($key, $val) = each %$failure_hash) {
                    $err_msg .= "$key: $val; ";
                }
                $err_msg .= "\n";
            }
            $Log->exit() if $may_exit;
            NACL::APISet::Exceptions::CommandFailedException->throw(
                text            => $err_msg,
                response_object => $response,
            );
        }

        foreach my $row (@{$parsed_output->[0]{'license-v2-success-list'}[0]{'license-v2-added'}}) {
            my %license_opts = (package => $row->{package});
            if ($row->{'serial-number'} eq 'none') {
                # If parsed_output returns serial-number as none, then it's a
                # cluster-wide license, so determine the base serial-number.
                $license_opts{'serial-number'} = $get_serial_num->();
            } else {
                $license_opts{'serial-number'} = $row->{'serial-number'};
            }
            push @components, $pkg->new(
                command_interface => $command_interface,
                %license_opts,
            );
        }
    }
    $Log->exit() if $may_exit;
    return @components if(@components);
} ## end sub _add_cmode_zapi_v2

sub _add_7mode_cli_v1 {
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    if (   ref $opts{feature} eq 'ARRAY'
        || ref $opts{'license-code'} eq 'ARRAY')
    {
        NATE::Exceptions::Argument->throw('Licensing V1 doesn\'t support'
                . 'multiple license addition in single add command. So, '
                . 'provide scalar value for the attributes feature and '
                . 'license-code');
    }

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $feature;
    if (exists $opts{feature}) {
        $feature = delete $opts{feature};
    } else {
        NATE::Exceptions::Argument->throw("The parameter 'feature' is "
                . 'mandatory for 7Mode License v1. This is because the '
                . 'newly created SystemLicense component object contains '
                . 'the feature licensed, but 7Mode CLI does not provide '
                . 'a mechanism for determining the feature given a license '
                . 'code');
    } ## end else [ if ( exists $opts{feature...
    my $license_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {'license-code' => 'code'},
    );
    %opts = %$license_args;
    $apiset->license_add(%opts);
    $Log->exit() if $may_exit;
    return $pkg->new(
        feature           => $feature,
        command_interface => $command_interface
    );
} ## end sub _add_7mode_cli_v1

sub _add_7mode_cli_v2 {
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    if (!defined $opts{package} && !defined $opts{feature}) {
        NATE::Exceptions::Argument->throw("The parameter 'package' is "
                . 'mandatory for licensing version 2');
    }

    my $command_interface     = delete $opts{command_interface};
    my $apiset                = delete $opts{apiset};
    my $allowed_7m_v2_options = [qw(license-code)];

    _map_feature_to_package(hashref_or_arrayref => \%opts);
    my $packages = $pkg->_get_package_array(
        params            => \%opts,
        command_interface => $command_interface,
        interface         => 'CLI'
    );

    my $license_args = $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7m_v2_options,
    );

    $apiset->license_add(%$license_args);

    my @license_c_obj = $pkg->find(command_interface => $command_interface);

    my @components;
    foreach my $licensed_pkg (@$packages) {
        foreach my $c (@license_c_obj) {
            if (lc $licensed_pkg eq lc $c->package()) {
                push @components, $c;
            }
        }
    }

    $Log->debug(sub { "Dumping \@components : " . Dumper(\@components) });
    $Log->exit() if $may_exit;
    return wantarray ? @components : $components[0];
} ## end sub _add_7mode_cli_v2

sub _add_7mode_zapi_v1 {
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $feature;
    if (exists $opts{feature}) {
        $feature = delete $opts{feature};
    } else {
        NATE::Exceptions::Argument->throw("The parameter 'feature' is "
                . 'mandatory for 7Mode. This is because the newly created '
                . 'SystemLicense component object contains the feature licensed, '
                . 'but 7Mode ZAPI does not provide a mechanism for determining '
                . 'the feature given a license code');
    } ## end else [ if ( exists $opts{feature...

    my $license_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {'license-code' => 'code'},
    );
    %opts = %$license_args;
    $apiset->license_add(%opts);

    $Log->exit() if $may_exit;
    return $pkg->new(
        feature           => $feature,
        command_interface => $command_interface
    );
} ## end sub _add_7mode_zapi_v1

sub _add_7mode_zapi_v2 {
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    if (!defined $opts{package} && !defined $opts{feature}) {
        NATE::Exceptions::Argument->throw("The parameter 'package' is "
                . 'mandatory for licensing version 2');
    }

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    _map_feature_to_package(hashref_or_arrayref => \%opts);
    my $packages = $pkg->_get_package_array(
        params            => \%opts,
        command_interface => $command_interface,
        interface         => 'ZAPI'
    );
    my @licenses = split /,/, $opts{'license-code'};

    my %args_7mode;
    $args_7mode{'codes'} = \@licenses;
    $apiset->license_v2_add(%args_7mode);

    my @components = $pkg->find(
        command_interface => $command_interface,
        filter            => {package => join '|', @$packages},
    );

    $Log->debug(sub { "Dumping \@components : " . Dumper(\@components) });
    $Log->exit() if $may_exit;
    return wantarray ? @components : $components[0];
} ## end sub _add_7mode_zapi_v2

=head2 delete

  For Licensing V1:

  NACL::C::SystemLicense->delete(command_interface=>$ci,
                                'feature' => $feature,...);

                            or

  $license_object->delete(...);

  For Licensing V2:

  NACL::C::SystemLicense->delete(
      command_interface => $ci,
      package           => $package,
      'serial-number'   => $serial_number
      ...
  );

  or

  NACL::C::SystemLicense->delete(
      command_interface => $ci,
      package           => [ $package1, $package2, .. ],
      'serial-number'   => [ $serial_number1, $serial_number2, .. ]
      ...
  );

  or

  $license_object->delete(...);

(Class or instance method) This method deletes a license element.

Uses a CMode CLI/ZAPI, 7Mode CLI/ZAPI APISet.

Invokes "interim-license-remove,api" API for CMode ZAPI.

Invokes "license delete" command for 7Mode CLI.

Invokes "license-delete" command for 7Mode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< feature => $feature >>

(Required for class method for licensing v1, Not Applicable for instance method)
Feature name of license.

Applicable for CMode CLI/ZAPI, 7Mode CLI/ZAPI.

Maps to:

CM ZAPI: service

7M CLI : service name (Programmatic name: "service")

7M ZAPI: service

=item C<< package => $package E<verbar> [$package1, $package2, ..] >>

(Required for class method for licensing v2, Not Applicable for instance method)
Feature name of license.

Applicable for CMode CLI/ZAPI, 7Mode CLI/ZAPI.

Maps to:

CM ZAPI: package

7M CLI : -package (Programmatic name: "package")

7M ZAPI: package

=item C<< 'serial-number' => $serial_number E<verbar> [$serial_number1, $serial_number2, ..] >>

(Required for class method for licensing v2, Not Applicable for instance method)
Feature name of license.

Applicable for CMode CLI/ZAPI, 7Mode CLI/ZAPI.

Maps to:

CM ZAPI: serial-number

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"system license delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::SystemLicense::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete a
non-existing license.

=item C<NACL::C::Exceptions::SystemLicense::MultipleCommandFailed>

NACL::C::Exceptions::MultipleCommandFailed will be thrown for failures
while deleting multiple licenses. This is applicable only for v2 7Mode
and CMode ZAPI. See Also Burt621876 (ONTAP CLI doesn't complain for
failures while deleting multiple licenses).

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'serial-number' => {type => SCALAR | ARRAYREF, optional => 1},
            'package'       => {type => SCALAR | ARRAYREF, optional => 1},
            _optional_scalars(qw( feature ))
        },
        cli_cmd => 'system license delete',
    );

    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_delete_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_delete_cmode_zapi_v1",
                    interface => "ZAPI",
                    set       => "CMode",
                    check     => "_delete_zapi_check_v1"
                },
                {   method    => "_delete_cmode_zapi_v2",
                    interface => "ZAPI",
                    set       => "CMode",
                    check     => "_delete_zapi_check_v2"
                },
                {   method    => "_delete_7mode_cli",
                    interface => "CLI",
                    set       => "7Mode|Nodescope",
                },
                {   method    => "_delete_7mode_zapi_v1",
                    interface => "ZAPI",
                    set       => "7Mode",
                    check     => "_is_v1"
                },
                {   method    => "_delete_7mode_zapi_v2",
                    interface => "ZAPI",
                    set       => "7Mode",
                    check     => "_is_v2_7m_zapi"
                }
            ],
            frontend => 'NACL::C::SystemLicense::delete'
        );
    }
    catch NACL::C::Exceptions::Component with {
        $Log->exit() if $may_exit;
        $_[0]->throw();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::SystemLicense::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_license_delete',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi_v1 {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {'feature' => 'service',},
        target => \%cmode_args,
    );

    $apiset->interim_license_remove(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi_v1

sub _delete_cmode_zapi_v2 {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    _map_feature_to_package(hashref_or_arrayref => \%opts);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my @packages;
    my @serial_numbers;
    if (ref $opts{package} ne 'ARRAY') {
        @packages = split(",", $opts{package});
    } else {
        @packages = @{$opts{'package'}};
    }

    if (defined $opts{'serial-number'}) {
        if (ref $opts{'serial-number'} eq 'ARRAY') {
            @serial_numbers = @{$opts{'serial-number'}};
        } elsif ($opts{'serial-number'} =~ m[\d+\s*,\s*\d+]i) {
            $pkg_or_obj->_scalar_to_array_deprication_warning(
                params  => ['serial-number'],
                options => \%opts
            );
            @serial_numbers = @{$opts{'serial-number'}};
        } else {
            push @serial_numbers, $opts{'serial-number'};
        }
    }

    @packages = map lc $_, @packages;

    my $api = 'license_v2_delete';
    if (scalar @packages == 1) {
        my %serial_number;
        if (defined $serial_numbers[0]) {
            %serial_number = ('serial-number' => $serial_numbers[0]);
        }
        $apiset->$api('package' => $packages[0], %serial_number);
    } else {
        $pkg_or_obj->_delete_multiple_v2_zapi_licenses(
            package         => \@packages,
            'serial-number' => \@serial_numbers,
            api             => $api,
            apiset          => $apiset
        );
    }

    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi_v2

sub _delete_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    if (ref $opts{package} eq 'ARRAY') {
        $opts{package} = join(" ", @{$opts{package}});
    }

    if (ref $opts{'serial-number'} eq 'ARRAY') {
        $opts{'serial-number'} = join(",", @{$opts{'serial-number'}});
    }
    my $license_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(package serial-number)],
        map    => {'feature' => 'service'},
    );
    my %args = %$license_args;
    $apiset->license_delete(%args);
    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_cli

sub _delete_7mode_zapi_v1 {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $license_args      = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {'feature' => 'service'},
    );
    %opts = %$license_args;
    $apiset->license_delete(%opts);
    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_zapi_v1

sub _delete_7mode_zapi_v2 {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    _map_feature_to_package(hashref_or_arrayref => \%opts);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my @packages;
    my @serial_numbers;
    if (ref $opts{package} ne 'ARRAY') {
        @packages = split(",", $opts{package});
    } else {
        @packages = @{$opts{'package'}};
    }

    if (scalar @packages == 1) {
        $apiset->license_v2_delete('package' => $packages[0]);
    } else {
        $pkg_or_obj->_delete_multiple_v2_zapi_licenses(
            package => \@packages,
            api     => 'license_v2_delete',
            apiset  => $apiset
        );
    }

    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_zapi_v2

# Cannot use ZAPI if:
# 1. license v1 feature is not supported
# 2. special characters are provided for serial-number
sub _delete_zapi_check_v1 {
    $Log->enter() if $may_enter;

    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_is_v1(%opts);
    my @fields_to_check = ('serial-number');
    $pkg_or_obj->_fallback_for_relational_regex(%opts, fields_to_check => \@fields_to_check);

    $Log->exit() if $may_exit;

}

sub _is_v1 {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $ci = $opts{command_interface};
    my $is_licensing_v1 = $ci->is_feature_supported(feature => 'license_v1');
    if (!$is_licensing_v1) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw(
            "Licensing V1 isn't supported");
    }
    $Log->exit() if $may_exit;
} ## end sub _is_v1

# Cannot use ZAPI if:
# 1. license v2 feature is not supported
# 2. special characters are provided for serial-number
sub _delete_zapi_check_v2 {
    $Log->enter() if $may_enter;

    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_is_v2(%opts);
    my @fields_to_check = ('serial-number');
    $pkg_or_obj->_fallback_for_relational_regex(%opts, fields_to_check => \@fields_to_check);

    $Log->exit() if $may_exit;

}

sub _is_v2 {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $ci = $opts{command_interface};
    my $is_licensing_v2 = $ci->is_feature_supported(feature => 'license_v2');
    if (!$is_licensing_v2) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw(
            "Licensing V2 isn't supported");
    }
    $Log->exit() if $may_exit;
} ## end sub _is_v2

=head2 clean_up

    NACL::C::SystemLicense->clean_up(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to remove unnecessary licenses.

Uses a CMode CLI/ZAPI APISet.

Invokes 'license-v2-delete-unused' API for CMode ZAPI if unused option is passed.

Invokes 'license-v2-delete-expired' API for CMode ZAPI if expired option is passed.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'expired' => $string >>

Remove expired licenses

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<license-v2-delete-expired>
<expired>$value</expired>
</license-v2-delete-expired>

=item C<< 'simulate' => $string >>

Simulate Only

Applicable for CMode CLI

=item C<< 'unused' => $string >>

Remove unused licenses

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<license-v2-delete-expired>
<unused>$value</unused>
</license-v2-delete-expired>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"system license clean-up" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI

=back

=back

=cut

sub clean_up {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( expired simulate unused
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'system license clean-up',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_clean_up_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_clean_up_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            }
        ],
        frontend => 'NACL::C::SystemLicense::clean_up',
    );

    $Log->exit() if $may_exit;
} ## end sub clean_up

sub _clean_up_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'system_license_clean_up',);
    $Log->exit() if $may_exit;
} ## end sub _clean_up_cmode_cli

sub _clean_up_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $unused  = delete $opts{unused};
    my $expired = delete $opts{expired};
    if (defined $unused) {
        NACL::C::SystemLicenseCleanup->_unused_cmode_zapi(%opts);
    }
    if (defined $expired) {
        NACL::C::SystemLicenseCleanup->_expired_cmode_zapi(%opts);
    }

    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    my $pkg_or_obj = shift;
    my %opts       = validate(
        @_,
        {   command_interface => {
                type     => OBJECT,
                isa      => 'NACL::C::CommandInterface',
                optional => 1
            }
        }
    );
    my $ci           = $opts{command_interface};
    my %default_spec = (
        'feature'       => {type => SCALAR,            optional => 1},
        'package'       => {type => SCALAR | ARRAYREF, optional => 1},
        'serial-number' => {type => SCALAR | ARRAYREF, optional => 1}
    );

    if (defined $ci) {
        $Log->debug("command interface is defined, so return primary spec"
                . " depending on the supported feature");
        if ($ci->is_feature_supported(feature => 'license_v2')) {
            return (
                'package'       => {type => SCALAR | ARRAYREF},
                'serial-number' => {type => SCALAR | ARRAYREF}
            );
        } elsif ($ci->is_feature_supported(feature => 'license_v1')) {
            return ('feature' => {type => SCALAR});
        } else {
            return %default_spec;
        }
    } else {
        $Log->debug(
            "command interface is undefined so, returning default spec");
        return %default_spec;
    }

} ## end sub _primary_keys_validate_spec

sub _get_package_array {
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params => \@_,
        spec   => {
            params            => {type => HASHREF},
            command_interface => {isa  => 'NACL::C::CommandInterface::ONTAP'},
            interface         => {type => SCALAR},
        },
        allow_extra => 1
    );
    my $args      = $opts{params};
    my $ci        = $opts{command_interface};
    my $interface = $opts{interface};
    foreach my $param ('license-code', 'package') {
        if (defined $args->{$param}) {
            if (ref $args->{$param} eq 'ARRAY') {
                if ($ci->mode() eq '7Mode' && $interface eq 'CLI') {
                    $args->{$param} = join " ", @{$args->{$param}};
                } else {
                    $args->{$param} = join ",", @{$args->{$param}};
                }
            }
        }
    }

    my @licenses = split /,|\s+/, $args->{'license-code'};
    my @packages = split /,|\s+/, $args->{package};

    if (scalar @licenses != scalar @packages) {
        NATE::Exceptions::Argument->throw("Mis-match in the"
                . " number of license-codes and number of"
                . " packages provided");
    }

    return \@packages;
} ## end sub _get_package_array

sub _delete_multiple_v2_zapi_licenses {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %args) = @_;
    my %opts = validate_with(
        params => \%args,
        spec   => {
            'serial-number' => {type => ARRAYREF, optional => 1},
            'package'       => {type => ARRAYREF},
            api             => {type => SCALAR},
            apiset => {type => OBJECT, isa => 'NACL::APISet::Node::ZAPI'},
        }
    );

    my @all_failures;
    my @serial_numbers;
    my $exception;
    my $no_of_failures = 0;
    my $api            = $opts{api};
    my $apiset         = $opts{apiset};
    my @packages       = @{$opts{package}};

    if (defined $opts{'serial-number'}) {
        my @serial_numbers = @{$opts{'serial-number'}};
    }

    for (my $i = 0; $i < @packages; $i++) {
        my %serial_number;
        if (defined $serial_numbers[$i]) {
            %serial_number = ('serial-number' => $serial_numbers[$i]);
        }
        try {
            $apiset->$api('package' => $packages[$i], %serial_number);
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception      = shift;
            my $text           = $exception->text;
            my %single_failure = (
                text            => $text,
                response_object => $exception->get_response_object(),
                primary_keys    => {package => $packages[$i]}
            );
            push @all_failures, \%single_failure;
            $no_of_failures++;
        };
    }

    # For one failure, throw the exception as it is.
    # Otherwise throw MultipleCommandFailed exception
    if ($no_of_failures == 1) {
        $Log->exit() if $may_exit;
        $exception->throw();
    } elsif ($no_of_failures > 1) {
        $Log->exit() if $may_exit;
        NACL::C::Exceptions::MultipleCommandFailed->throw(
            all_failures => \@all_failures);
    }
    $Log->exit() if $may_exit;
} ## end sub _delete_multiple_v2_zapi_licenses

sub find_base_serial_number {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $serial_number;
    if ($opts{command_interface}->is_cmode()) {
        $serial_number = $pkg->find_base_serial_number_cm(%opts);
    } else {
        $serial_number = $pkg->find_base_serial_number_7m(%opts);
    }
    $Log->exit() if $may_exit;
    return $serial_number;
}

sub find_base_serial_number_cm {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $serial_number = $pkg->find(
        %opts,
        filter => {package => 'Base'}
    )->serial_number();
    $Log->exit() if $may_exit;
    return $serial_number;
}

sub find_base_serial_number_7m {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $serial_number = NACL::CS::SystemLicense->fetch(
        %opts,
        requested_fields => [qw(serial-number)]
    )->serial_number();
    $Log->exit() if $may_exit;
    return $serial_number;
}

1;
