#
# Copyright (c) 2001-2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemNode Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemNode

=head1 DESCRIPTION

C<NACL::C::SystemNode> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP Node.  A related class is
L<NACL::CS::SystemNode|lib-NACL-CS-SystemNode-pm>, which represents
the past state of an ONTAP Node.

=head1 ATTRIBUTES

=head2 node

The name of the node, used to identify it uniquely.

=cut

package NACL::C::SystemNode;

use strict;
use warnings;
use NACL::CS::SystemNode;
use NACL::STask::StorageDisk;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use Params::Validate qw(SCALAR SCALARREF validate_with validate_pos ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::APISet::Exceptions::TimeoutException qw(:try);
use base 'NACL::C::CommandInterface::ONTAP';
use NATE::Exceptions::Argument ();
use NACL::Exceptions::InvalidChoice;
use NACL::C::_Mixins::Common qw(get_7m_or_nodescope_apiset);
use feature 'state';
use Net::Ping::External qw(ping);
use Scalar::Util qw(blessed);

=head1 METHODS

=head2 new

my $node = NACL::C::SystemNode->new([name => $name],
                          [node => $node],
                          [hostrec => $hostrec] );
Specifying any one of the node, name, hostrec attributes will populate
the other two.
Returns a new Node component object with the given attributes.

=cut

use Class::MethodMaker [
scalar => [
    {   '-default_ctor' => sub { $_[0]->_construct_primary_key(); },
        '-store_cb' =>
            sub { $_[0]->_verify_hostrec_callback($_[1], $_[2]); }
    },
    'node'
],
];

sub isa {
$Log->enter() if $may_enter;
my ($pkg_or_obj, $kind) = @_;
my $isa = $pkg_or_obj->_build_isa(
    kind  => $kind,
    alias => 'NACL::C::Node'
);
$Log->exit() if $may_exit;
return $isa;
}

=head2 find

# Read from test parameters
my $Node  = NACL::C::SystemNode->find();
my @Nodes = NACL::C::SystemNode->find();

# Invoke command on command_interface
my $Node  = NACL::C::SystemNode->find(command_interface => $ci, ...);
my @Nodes = NACL::C::SystemNode->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

When invoked without the C<command_interface>, it reads the following
test parameters:

=over

=item FILER_LIST

This is a meta-parameter, which means that the value of this parameter
should be the names of other parameters, not the names of the filers
themselves. This is accepted:

FILER_LIST=FILERA,FILERB

While this is not valid:

FILER_LIST=myvsim1,myvsim2

=item FILER{A..ZZ}

This is FILERA, FILERB ... FILERZ, FILERAA, FILERAB ... FILERZZ

=item FILER

=item NODES

=back

The parameters are read in the order listed above until a test parameter
which is defined is found. For example, if both FILERA and FILER have been
provided values, then the value of FILERA is used.

When find() is invoked with a C<command_interface> argument it searches
for all nodes visible from that command interface and returns objects for
each of them.

This method is generally not implemented in individual components. The base
class (L<NACL::C::CommandInterface::ONTAP|lib-NACL-C-CommandInterface-ONTAP-pm>
provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.
You can also refer to user guide for more information on the options,
order of precedence for command line parameters (in case of find() is
invoked without specifying a command_interface) and examples.
http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/ComponentLayerUsersGuide/index.html#Obtaining_command_interface_objects

=head2 state

my $Node_state = $node_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemNode would return
a NACL::CS::SystemNode object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=head2 systemshell

B<DEPRECATED>.

Usage of this method is deprecated. It is recommended to use the Systemshell
APISet instead. This can be obtained by calling:

my $apiset = $node->get_systemshell_apiset();

This supports parsing and error checking. For commands that do not have
APISet support, either use $apiset->execute_command(command => $command)
to execute the raw command, or raise a NACL burt requesting support for that
command.

=head2 apiset

Fetches the apiset object stored in command_interface of Node.
To obtain an APISet object for a particular command interface, 
we can use the method 'apiset', which accepts the options 
"category", "interface" and "set" to obtain the relevant APISet object.

This could be useful if there are sections of a script for which we have to drop to the API layer
for some reason and instead of having to create an APISet object ourselves, 
we can use the existing, cached APISet objects.

The apiset call to node can be made in following ways : 

Interface is specified to be ZAPI(CLI is the default value). 
So the method would select the best defaults for 'category' and 'set' inputs 
based on the mode in which the node is running. 
It returns 7M ZAPI APIset for 7Mode hosts and CM ZAPI APISet for CMode hosts.

my $node = NACL::C::Node->find();
my $apiset = $node->apiset(interface => 'ZAPI');

No inputs specified. apiset() call would return 7M CLI APIset for 7Mode hosts and CM CLI APISet for CMode hosts.
my $node = NACL::C::Node->find();
my $apiset = $node->apiset();

Providing category,set & interface values as below :
my $cli_apiset = $node->apiset(category => "Node", interface => "CLI", set => "CMode");
my $zapi_apiset = $node->apiset(category => "Node", interface => "ZAPI", set => "CMode");

=cut

sub systemshell {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @args) = @_;
my $warn_message = 'Usage of NACL::C::SystemNode->systemshell is deprecated. '
        . 'Please use the Systemshell APISet instead. This can be obtained '
        . 'by calling: $apiset = $node_obj->get_systemshell_apiset();'; 
NACL::C::Component->_print_deprecation_warning($warn_message);
$pkg_or_obj->_frontend_log_debug_opts(@args);
my %opts = $pkg_or_obj->_common_validate_with(
    params          => \@args,
    additional_spec => {_optional_scalars(qw( command node )),},
);
my $output = $pkg_or_obj->call_on_apiset(
    %opts,
    choices => [
        {   method    => '_systemshell_cmode_cli',
            interface => 'CLI',
            set       => 'CMode',
        },
    ],
);

$Log->exit() if $may_exit;
return $output;
} ## end sub systemshell

sub _systemshell_cmode_cli {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @args) = @_;
my $response;
$pkg_or_obj->_base_backend_cmode_cli(
    @args,
    api                 => 'system_node_systemshell',
    apiset_response_obj => \$response
);
my $output = $response->get_raw_output();
$Log->exit() if $may_exit;
return $output;
} ## end sub _systemshell_cmode_cli

=head2 halt

NACL::C::SystemNode->halt(command_interface => $ci, node => $node, ...);

                             or

$Node_object->halt(...);

(Class or instance method)
This method is used to shut down a node.

Uses a CMode CLI/ZAPI APISet.

Invokes 'system-node-shutdown' API for CMode ZAPI.

When called with an object instance ($node->halt(...)) it will halt the
node which is modeled by the object (i.e. "local"). When called with an object
instance there is no support for halting any other node of the cluster.
However, specifying which node to halt is supported if the method is invoked
as a package method (NACL::C::SystemNode->halt(...)).

After halting the node, L<NACL::Transit|lib-NACL-Transit-pm>
(http://wikid.netapp.com/w/QA/projects/Libraries_Initiative/Project/Common_Infrastructure/Users_Guide/NACL_Transit_Users_Guide)
can be used to change the state of the filer.

Example:

use NACL::Transit;
use NACL::C::SystemNode;
...
my $node = NACL::C::SystemNode->find();
...
$node->halt();
my $transit_obj = NACL::Transit->new(name => $node->hostrec());
$transit_obj->change_state(to => 'CLI');

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< node => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

=item C<< dump => $string >>

Create a core dump?
Possible values: true; false

Applicable for CMode CLI/ZAPI.

=item C<< "inhibit-takeover" => $string >>

Disallow Storage Takeover by Partner?
Possible Values: true; false

Applicable for CMode CLI/ZAPI.

=item C<< reason => $string >>

Reason for shutdown

Applicable for CMode CLI/ZAPI.

=item C<< 'skip-lif-migration' => $string >>

Skip Migrating LIFs Away from Node

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'skip-lif-migration-beforehand' => $string >>

Skip Migrating LIFs Away from Node Prior to Shutdown
Applicable for CMode CLI.

=item C<< 'skip-lif-migration-before-shutdown' => $string >>

Skip Migrating LIFs Away from Node Prior to Shutdown
Applicable for CMode CLI/ZAPI.

=item C<< 'ignore-quorum-warnings' => $string >>

Note that if this option is not provided, then it will be defaulted
to "true" (see burt 867673). This can be disabled by providing the
value as undef ('ignore-quorum-warnings' => undef)

Ignore warnings
Applicable for CMode CLI/ZAPI.

=item C<< '*' >>

This method takes all other options supported by
"system node halt" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub halt {
$Log->enter() if $may_enter;
my $pkg_or_obj = shift;
$Log->debug("Opts to 'halt' front end are:\n" . Dumper(\@_))
    if ($Log->may_debug());
my %opts = $pkg_or_obj->_common_validate_with(
    params          => \@_,
    additional_spec => {
        "ignore-quorum-warnings" => {type => SCALAR|UNDEF, optional => 1},
        _optional_scalars(
            qw(dump inhibit-takeover reason
                node

                skip-lif-migration

                skip-lif-migration-beforehand

                skip-lif-migration-before-shutdown
                )
        )
    },
    cli_cmd => 'system_node_halt',
);
$pkg_or_obj->_handle_ignore_quorum_warnings(\%opts);

$pkg_or_obj->call_on_apiset(
    %opts,
    choices => [
        {   method    => '_halt_cmode_cli',
            interface => 'CLI',
            set       => 'CMode',
        },
        {   method    => '_halt_cmode_zapi',
            interface => 'ZAPI',
            set       => 'CMode',
        },
    ],
);

$Log->exit() if $may_exit;
} ## end sub halt

sub _halt_cmode_cli {
$Log->enter() if $may_enter;
my ($pkg_or_obj, %opts) = @_;
$pkg_or_obj->_halt_reboot_base_cmode_cli(%opts,
    api => 'system_node_halt');

$Log->exit() if $may_exit;
} ## end sub _halt_cmode_cli

sub _halt_cmode_zapi {
$Log->enter() if $may_enter;
my ($pkg_or_obj, %opts) = @_;
my $command_interface = $opts{command_interface};
my $map = {
    'dump'       => 'is-dump',
    'inhibit-takeover'    => 'is-inhibit-takeover',
    'skip-lif-migration-before-shutdown' => 'is-skip-lif-migration-before-shutdown',
};

$pkg_or_obj->_base_backend_cmode_zapi(
    %opts,
    api => 'system-node-shutdown',
    copy => [qw(node reason ignore-quorum-warnings)],
    map => $map,
);

$Log->exit() if $may_exit;
} ## end sub _halt_cmode_zapi

# Automatically set ignore-quorum-warnings => true to prevent script
# failures. See burt 867673.
sub _handle_ignore_quorum_warnings {
my ($pkg_or_obj, $opts_ref) = @_;
if (exists $opts_ref->{'ignore-quorum-warnings'}) {
    # Value of undef means "do not send down the option"
    if (!defined $opts_ref->{'ignore-quorum-warnings'}) {
        delete $opts_ref->{'ignore-quorum-warnings'};
    }
} elsif ($opts_ref->{command_interface}->version_manager()->is_version_ge(release => '8.3.1')) {
    $opts_ref->{'ignore-quorum-warnings'} = 'true';
}
}

=head2 rename

NACL::C::SystemNode->rename(
                            command_interface => $ci,
                            'node' => $node,
                            ...);

                             or

$SystemNode_obj->rename(...);

(Class or instance method)
This method is used to rename a node.

Uses a CMode CLI/ZAPI APISet.

Invokes 'system-node-rename' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)

=item C<< 'newname' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: new-name

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'node' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub rename {
$Log->enter() if $may_enter;
my $pkg_or_obj = shift;
$Log->debug("Opts to 'rename' front end are:\n" . Dumper(\@_))
    if ($Log->may_debug());
my %opts = $pkg_or_obj->_common_validate_with(
    params          => \@_,
    additional_spec => {
        _optional_scalars(
            qw( newname

                node
                )
        ),
    },
    cli_cmd => 'system node rename',
);
my $ret = $pkg_or_obj->call_on_apiset(
    %opts,
    choices => [
        {   method    => '_rename_cmode_cli',
            interface => 'CLI',
            set       => 'CMode',
        },
        {   method    => '_rename_cmode_zapi',
            interface => 'ZAPI',
            set       => 'CMode',
        },
    ],
);
$pkg_or_obj->node($opts{'newname'}) if ref($pkg_or_obj);
$Log->exit() if $may_exit;
return $ret;
} ## end sub rename

sub _rename_cmode_cli {
$Log->enter() if $may_enter;
my ($pkg_or_obj, %opts) = @_;
my $allowed_cmode_options = [
    qw( newname node

        node
        extended_query )
];
$pkg_or_obj->_base_backend_cmode_cli(%opts,
        api    => 'system_node_rename',
        copy   => $allowed_cmode_options,
);
$Log->exit() if $may_exit;
} ## end sub _rename_cmode_cli

sub _rename_cmode_zapi {
$Log->enter() if $may_enter;
my ($pkg_or_obj, %opts) = @_;
my $allowed_cmode_options = [qw( node )];
$pkg_or_obj->_base_backend_cmode_zapi(%opts,
        api    => 'system_node_rename',
        copy   => $allowed_cmode_options,
        map    => {"newname" => "new-name"},
);
$Log->exit() if $may_exit;
} ## end sub _rename_cmode_zapi

=head2 run

B<DEPRECATED>.

Usage of this method is deprecated. It is recommended to use the Node-scope
APISet instead. This can be obtained by calling:

my $apiset = $node->get_7m_or_nodescope_apiset();

This supports parsing and error checking. For commands that do not have
Node-scope support, either use $apiset->execute_command(command => $command)
to execute the raw command, or raise a NACL burt requesting support for that
command.

=cut

sub run {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @args) = @_;
my $warn_message = 'Usage of NACL::C::SystemNode->run is deprecated. Please '
        . 'use the Node-scope APISet instead. This can be obtained by '
        . 'calling: $apiset = $node_obj->get_7m_or_nodescope_apiset();';
NACL::C::Component->_print_deprecation_warning($warn_message);
$pkg_or_obj->_frontend_log_debug_opts(@args);
my %opts = $pkg_or_obj->_common_validate_with(
    params          => \@args,
    additional_spec => {
        _optional_scalars(
            qw( reset command
                )
        ),
    },
    cli_cmd => 'system node run',
);
my $ret = $pkg_or_obj->call_on_apiset(
    %opts,
    choices => [
        {   method    => '_run_cmode_cli',
            interface => 'CLI',
            set       => 'CMode',
        },
    ],
);

$Log->exit() if $may_exit;
return $ret;
} ## end sub run

sub _run_cmode_cli {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @args) = @_;
my $response;
$pkg_or_obj->_base_backend_cmode_cli(
    @args,
    api                 => 'system_node_run',
    apiset_response_obj => \$response
);
my $output = $response->get_raw_output();
$Log->exit() if $may_exit;
return $output;
} ## end sub _run_cmode_cli

=head2 reboot

NACL::C::SystemNode->reboot(command_interface => $ci, node => $node, ...);

                             or

$Node_object->reboot(...);

(Class or instance method)
This method is used to reboot a node.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes 'system-node-reboot' API for CMode ZAPI.

Invokes 'execute raw command' command for 7Mode CLI.

When called with an object instance ($node->reboot(...)) it will reboot the
node which is modeled by the object (i.e. "local"). When called with an object
instance there is no support for rebooting any other node of the cluster.
However, specifying which node to reboot is supported if the method is invoked
as a package method (NACL::C::SystemNode->reboot(...)).

This will reboot 7Mode filer all the way up. If user wants prompt to perform
some action during reboot then he should use NACL::Trasit.

L<NACL::Transit|lib-NACL-Transit-pm> also provides a reboot method
(http://wikid.netapp.com/w/QA/projects/Libraries_Initiative/Project/Common_Infrastructure/Users_Guide/NACL_Transit_Users_Guide#reboot)
which reboots the filer. The only disadvantage is that the reboot method
provided by NACL::Transit does not (currently) provide support for providing
options to the command system node reboot (such as dump, inhibit-takover, reason).
If any of these options need to be provided, then we must use this component
method, which can then be used in conjunction with NACL::Transit to bring
the filer back up.

Example for rebooting the filer using only NACL::Transit (can be done if no
options need to be passed to "system node reboot")

use NACL::C::SystemNode;
use NACL::Transit;
...
my $node = NACL::C::SystemNode->find();
...
my $transit_obj = NACL::Transit->new(name => $node->hostrec());
$transit_obj->reboot();

Example for rebooting the filer using Node->reboot() and NACL::Transit (to be
done if options need to be passed to "system node reboot")

use NACL::C::SystemNode;
use NACL::Transit;
...
my $node = NACL::C::SystemNode->find();
...
$node->reboot(%args);
my $transit_obj = NACL::Transit->new(name => $node->hostrec());
$transit_obj->change_state(to => 'CLI');

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

=item C<< dump => $string >>

Create a core dump?
Possible values: true; false

Applicable for CMode CLI/ZAPI.

=item C<< "inhibit-takeover" => $string >>

Disallow Storage Takeover by Partner?
Possible Values: true; false

Applicable for CMode CLI/ZAPI.

=item C<< reason => $string >>

Reason for reboot

Applicable for CMode CLI/ZAPI.

=item C<< 'skip-lif-migration' => $string >>

Skip Migrating LIFs Away from Node

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'skip-lif-migration-beforehand' => $string >>

Skip Migrating LIFs Away from Node Prior to Reboot
Applicable for CMode CLI.

=item C<< 'skip-lif-migration-before-reboot' => $string >>

Skip Migrating LIFs Away from Node Prior to Reboot
Applicable for CMode CLI/ZAPI.

=item C<< 'ignore-quorum-warnings' => $string >>

Note that if this option is not provided, then it will be defaulted
to "true" (see burt 867673). This can be disabled by providing the
value as undef ('ignore-quorum-warnings' => undef)

Ignore warnings
Applicable for CMode CLI/ZAPI.

=item C<< '*' >>

This method takes all other options supported by
"system node reboot" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub reboot {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @args) = @_;
$Log->debug(sub {"Opts to 'reboot' front end are:\n" . Dumper(\@args)});
my %opts = $pkg_or_obj->_common_validate_with(
    params          => \@args,
    additional_spec => {
        "ignore-quorum-warnings" => {type => SCALAR|UNDEF, optional => 1},
        _optional_scalars(
            qw(dump inhibit-takeover reason
                node
                skip-lif-migration
                skip-lif-migration-beforehand
                skip-lif-migration-before-reboot
                )
        )
    },
    cli_cmd => 'system node reboot',
);
$pkg_or_obj->_handle_ignore_quorum_warnings(\%opts);

$pkg_or_obj->call_on_apiset(
    %opts,
    choices => [
        {   method    => '_reboot_cmode_cli',
            interface => 'CLI',
            set       => 'CMode',
        },
        {   method    => '_reboot_cmode_zapi',
            interface => 'ZAPI',
            set       => 'CMode',
        },
        {   method    => '_reboot_7mode_cli',
            interface => 'CLI',
            set       => '7Mode',
        },
    ],
);

$Log->exit() if $may_exit;
} ## end sub reboot

sub _reboot_cmode_cli {
$Log->enter() if $may_enter;
my ($pkg_or_obj, %opts) = @_;
$pkg_or_obj->_halt_reboot_base_cmode_cli(%opts,
    api => 'system_node_reboot');

$Log->exit() if $may_exit;
} ## end sub _reboot_cmode_cli

sub _reboot_cmode_zapi {
$Log->enter() if $may_enter;
my ($pkg_or_obj, %opts) = @_;
my $command_interface = $opts{command_interface};
my $map = {
    'dump'       => 'is-dump',
    'inhibit-takeover'    => 'is-inhibit-takeover',
    'skip-lif-migration-before-reboot' => 'is-skip-lif-migration-before-reboot',
};

$pkg_or_obj->_base_backend_cmode_zapi(
    %opts,
    api  => 'system-node-reboot',
    copy => [qw(node reason ignore-quorum-warnings)],
    map  => $map,
);

$Log->exit() if $may_exit;
} ## end sub _reboot_cmode_zapi

sub _reboot_7mode_cli {
$Log->enter() if $may_enter;
my $pkg_or_obj = shift;
my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

my $apiset = delete $opts{apiset};

my %cmd_args;

# reboot will not return the prompt, so it will timeout.
# If an error occurs, the error message will show up immediately
# If the command times out, then it is executing successfully. We don't
# need to wait for 60 seconds, the error message should show up within 10
# seconds
$cmd_args{'connectrec-timeout'} = 10;
$cmd_args{'command'}            = "reboot -t 0";

try {
    $apiset->execute_raw_command(%cmd_args);
}
catch NACL::APISet::Exceptions::TimeoutException with {}
    catch NATE::BaseException with {
    my $exception = shift;
    my $text      = $exception->text();

    if ($text
        !~ /Unexpected EOF while reading output from command|Timeout waiting for command to complete/i
        )
    {
        $Log->exit() if $may_exit;
        $exception->throw();
    }
    };
$Log->exit() if $may_exit;
} ## end sub _reboot_7mode_cli

=head2 modify

NACL::C::SystemNode->modify(
                            command_interface => $ci,
                            'node' => $node,
                            ...);

                             or

$SystemNode_obj->modify(...);

(Class or instance method)
This method is used to modify node attributes.

Uses CMode CLI/ZAPI .
Invokes  system-node-modify  for CMode Zapi non-iter

When called with an object instance ($node->modify(...)) it will modify the
attributes of the node which is modeled by the object (i.e. "local"). When
called with an object instance there is no support for modifying the attributes
of any other node in the cluster. However, specifying which node's attributes
are to be modified can be done if it is called as a package method
(NACL::C::SystemNode->modify(...)).


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'owner' => $string >>

Owner
Applicable for CMode CLI/ZAPI, non-iter.

Mapping for non-iter: node-owner

=item C<< 'location' => $string >>

Location
Applicable for CMode CLI/ZAPI, non-iter.

Mapping for non-iter: node-location

=item C<< 'eligibility' => $string >>

Eligibility possible value(s) are, true,false
Applicable for CMode CLI/ZAPI, non-iter.

Mapping for non-iter: is-node-cluster-eligible

=item C<< 'epsilon' => $string >>

Epsilon possible value(s) are, true,false
Applicable for CMode CLI/ZAPI, non-iter.

Mapping for non-iter: is-epsilon-node

=item C<< 'assettag' => $string >>

Asset Tag
Applicable for CMode CLI/ZAPI, non-iter.

Mapping for non-iter: node-asset-tag

=item C<< '*' >>

This method takes all other options supported by "system node modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @args) = @_;
$pkg_or_obj->_frontend_log_debug_opts(@args);
my %opts = $pkg_or_obj->_common_validate_with(
    params => \@args,
    additional_spec => {
        _optional_scalars(
            qw( owner location eligibility
                epsilon assettag
                )
        ),
    },
    cli_cmd => 'system node modify',
);
$pkg_or_obj->call_on_apiset(
    %opts,
    choices => [
        {   method    => '_modify_cmode_cli',
            interface => 'CLI',
            set       => 'CMode',
            zapi_type => 'none',
        },
        {   method    => '_modify_cmode_zapi_non_iter',
            interface => 'ZAPI',
            set       => 'CMode',
            zapi_type => 'non-iter',
        },
    ],
    frontend => 'NACL::C::SystemNode::modify',
);

$Log->exit() if $may_exit;
}
sub _modify_cmode_cli {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @opts) = @_;
$pkg_or_obj->_base_backend_cmode_cli(
    @opts,
    api     => 'system_node_modify',
);
$Log->exit() if $may_exit;
}

sub _modify_cmode_zapi_non_iter {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @opts) = @_;
my $copy = [qw(node)];
my $map = {
    'owner'       => 'node-owner',
    'location'    => 'node-location',
    'eligibility' => 'is-node-cluster-eligible',
    'epsilon'     => 'is-epsilon-node',
    'assettag'    => 'node-asset-tag',
};
$pkg_or_obj->_base_backend_cmode_zapi(
    @opts,
    api  => 'system-node-modify',
    copy => $copy,
    map  => $map,
);
$Log->exit() if $may_exit;
}

=head2 revert_to

NACL::C::SystemNode->revert_to(
                            command_interface => $ci,
                            'node' => $node,
                            ...);

                             or

$SystemNode_obj->revert_to(...);

(Class or instance method)
This method is used to revert a node to a previous release of Data ONTAP.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'skip-vol-aggr-check' => $string >>

Applicable for CMode CLI.

=item C<< 'check' => $string >>

Applicable for CMode CLI.

=item C<< 'check-only' => $string >>

Applicable for CMode CLI.

=item C<< 'version' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'skip-lun-config-check' => $string >>

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub revert_to {
$Log->enter() if $may_enter;
my $pkg_or_obj = shift;
$pkg_or_obj->_frontend_log_debug_opts(@_);
my %opts = $pkg_or_obj->_common_validate_with(
    params          => \@_,
    additional_spec => {
        _optional_scalars(
            qw( skip-vol-aggr-check check check-only version

                skip-lun-config-check

                node
                )
        ),
    },
);
$pkg_or_obj->call_on_apiset(
    %opts,
    choices => [
        {   method    => '_revert_to_cmode_cli',
            interface => 'CLI',
            set       => 'CMode',
        },
    ],
);

$Log->exit() if $may_exit;
} ## end sub revert_to

sub _revert_to_cmode_cli {
$Log->enter() if $may_enter;
my ($pkg_or_obj,%opts) = @_;
my $allowed_cmode_options = [
    qw( skip-vol-aggr-check check check-only version node

        node

        skip-lun-config-check

        node
        extended_query )
];
my %cmode_args;
$pkg_or_obj->_hash_copy(
    source => \%opts,
    copy   => $allowed_cmode_options,
    target => \%cmode_args,
);
 $pkg_or_obj->_base_backend_cmode_cli (
         %opts,
         api => 'system_node_revert_to',
         );

$Log->exit() if $may_exit;
} ## end sub _revert_to_cmode_cli

# All components should now implement this method.
# This method should return the validation spec for the primary keys
# Note that here, we do not include hostrec, command_interface in this
# validation spec

=head2 uichanges

my $uichanges_obj = NACL::C::SystemNode->uichanges(command_interface => $ci,
                      [ filter => { %filter } ,]
          [ requested_fields => [ @req_fields ]);

                or

my @uichanges_objs = NACL::C::SystemNode->uichanges(command_interface => $ci,
                      [ filter => { %filter } ,]
          [ requested_fields => [ @req_fields ]);

                or

my $uichanges_obj = $SystemNode_obj->uichanges();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-SystemNodeUichanges-pm/ATTRIBUTES>
of NACL::CS::SystemNodeUichanges as input.
This method returns objects of the type L<NACL::CS::SystemNodeUichanges|lib-NACL-CS-SystemNodeUichanges-pm>.

=cut

sub uichanges {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @args) = @_;
my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
require NACL::CS::SystemNodeUichanges;
my @state_objs = NACL::CS::SystemNodeUichanges->fetch(%opts);
$Log->exit() if $may_exit;
return wantarray ? @state_objs : $state_objs[0];
}

=head2 updatediag

NACL::C::SystemNode->updatediag(
                            command_interface => $ci,
                            'node' => $node,
                            ...);

                             or

$SystemNode_obj->updatediag(...);

(Class or instance method)
This method is used to update Shell for diag user.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'node' => $string >>

Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub updatediag {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @args) = @_;
$pkg_or_obj->_frontend_log_debug_opts(@args);
my %opts = $pkg_or_obj->_common_validate_with(
    params          => \@args,
    additional_spec => {
        _optional_scalars(
            qw( node
                )
        ),
        'cmd_execution_time' => {type => SCALARREF, optional => 1},
    },
    cli_cmd  =>'system node updatediag',
);
$pkg_or_obj->call_on_apiset(
    %opts,
    choices => [
        {   method    => '_updatediag_cmode_cli',
            interface => 'CLI',
            set       => 'CMode',
        },
    ],
    frontend => 'NACL::C::SystemNode::updatediag',
);

$Log->exit() if $may_exit;
}

sub _updatediag_cmode_cli {
$Log->enter() if $may_enter;
my ($pkg_or_obj, %opts) = @_;
my $allowed_cmode_options = [
    qw( node extended_query

        node
        )
];
my %cmode_args;
$pkg_or_obj->_hash_copy(
    source => \%opts,
    copy   => $allowed_cmode_options,
    target => \%cmode_args,
);
$pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api => 'system_node_updatediag',
    );

$Log->exit() if $may_exit;
}

=head2 cluster_setup

NACL::C::SystemNode->cluster_setup(
                            command_interface => $ci,
                            ...);

(Class method)
This method is used to setup wizard.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by "cluster setup" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub cluster_setup {
$Log->enter() if $may_enter;
my ($pkg, @args) = @_;
$pkg->_frontend_log_debug_opts(@args);
my %opts = $pkg->_common_validate_with(
    params => \@args,
    additional_spec =>
        {'cmd_execution_time' => {type => SCALARREF, optional => 1},},
    ignore_primary_keys => 1,
    cli_cmd                 => 'cluster setup',
);
$pkg->call_on_apiset(
    %opts,
    choices => [
        {   method    => '_cluster_setup_cmode_cli',
            interface => 'CLI',
            set       => 'CMode',
        },
    ],
    frontend => 'NACL::C::SystemNode::cluster_setup',
);

$Log->exit() if $may_exit;
}

sub _cluster_setup_cmode_cli {
$Log->enter() if $may_enter;
my ($pkg, @opts) = @_;
$pkg->_base_backend_cmode_cli(
        @opts,
        api => 'cluster_setup',
    );

$Log->exit() if $may_exit;
}

=head2 show_delete_backlog

my $show_delete_backlog_obj = NACL::C::SystemNode->show_delete_backlog(command_interface => $ci,
                      [ filter => { %filter } ,]
          [ requested_fields => [ @req_fields ]);

                or

my @show_delete_backlog_objs = NACL::C::SystemNode->show_delete_backlog(command_interface => $ci,
                      [ filter => { %filter } ,]
          [ requested_fields => [ @req_fields ]);

                or

my $show_delete_backlog_obj = $SystemNode_obj->show_delete_backlog();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-SystemNodeShowDeleteBacklog-pm/ATTRIBUTES>
of NACL::CS::SystemNodeShowDeleteBacklog as input.
This method returns objects of the type L<NACL::CS::SystemNodeShowDeleteBacklog|lib-NACL-CS-SystemNodeShowDeleteBacklog-pm>.

=cut

sub show_delete_backlog {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @args) = @_;
my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
require NACL::CS::SystemNodeShowDeleteBacklog;
my @state_objs = NACL::CS::SystemNodeShowDeleteBacklog->fetch(%opts);
$Log->exit() if $may_exit;
return wantarray ? @state_objs : $state_objs[0];
}

=head2 console

NACL::C::SystemNode->console(
                            command_interface => $ci,
                            'node' => $node,
                            ...);

                             or

$SystemNode_obj->console(...);

(Class or instance method)
This method is used to get console access of the given node.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "system node console" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub console {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @args) = @_;
$pkg_or_obj->_frontend_log_debug_opts(@args);
my %opts = $pkg_or_obj->_common_validate_with(
    params => \@args,
    cli_cmd    => 'system node console',
);
$pkg_or_obj->call_on_apiset(
    %opts,
    choices => [
        {   method    => '_console_cmode_cli',
            interface => 'CLI',
            set       => 'CMode',
        },
    ],
    frontend => 'NACL::C::SystemNode::console',
);

$Log->exit() if $may_exit;
}

sub _console_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $password = $opts{command_interface}->hostrec()->default_password() || 'netapp1!';
    my $prompts_answers   = ['SP-login:' => 'admin',
                              'Password:' => $password  ];
    $pkg_or_obj->_base_backend_cmode_cli(%opts, api => 'system_node_console',
                                      additional_args => {'connectrec-match_table' => $prompts_answers});
    $Log->exit() if $may_exit;
}

=head2 reset

NACL::C::SystemNode->reset(
                            command_interface => $ci,
                                'node' => $node,
                            ...);

                                                             or

$SystemNode_obj->reset(...);

(Class or instance method)
This method is used to reset node using selected BIOS firmware.
Uses CMode CLI/ZAPI .
Invokes  system-node-reset  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'firmware' => $string >>


BIOS Firmware to use on reboot possible value(s) are, primary,backup,current
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "system node reset" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub reset {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @args) = @_;
$pkg_or_obj->_frontend_log_debug_opts(@args);
my %opts = $pkg_or_obj->_common_validate_with(
    params          => \@args,
    additional_spec => {
        _optional_scalars(
            qw( firmware
                )
        ),
    },
    cli_cmd => 'system node reset',
);
$pkg_or_obj->call_on_apiset(
    %opts,
    choices => [
        {   method    => '_reset_cmode_cli',
            interface => 'CLI',
            set       => 'CMode',
            zapi_type => 'none',
        },
        {   method    => '_reset_cmode_zapi_non_iter',
            interface => 'ZAPI',
            set       => 'CMode',
            zapi_type => 'non-iter',
        },
    ],
    frontend => 'NACL::C::SystemNode::reset',
);

$Log->exit() if $may_exit;
}

sub _reset_cmode_cli {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @opts) = @_;
my $prompts_answers = ['.*Do you want to continue\? {yes\|no}:' => "yes",'.*Do you want to continue\? {y\|n}:' => "y"];
$pkg_or_obj->_base_backend_cmode_cli(
        @opts,
        additional_args => {'connectrec-match_table' => $prompts_answers},
        api => 'system_node_reset',);
$Log->exit() if $may_exit;
}

sub _reset_cmode_zapi_non_iter {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @opts) = @_;
my $copy = [qw(firmware node)];
$pkg_or_obj->_base_backend_cmode_zapi(
    @opts,
    api  => 'system-node-reset',
    copy => $copy,
);
$Log->exit() if $may_exit;
}
=head2 restore_backup

NACL::C::SystemNode->restore_backup(
                            command_interface => $ci,
                            'node' => $node,
                            ...);

                             or

$SystemNode_obj->restore_backup(...);

(Class or instance method)
This method is used to restore the 'varfs' backup configuration to HA partner node.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< 'target-ip' => $string >>

HA-partner-IP
Applicable for CMode CLI

=item C<< 'target-address' => $string >>

HA Partner IP Address
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "system node restore backup" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub restore_backup {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @args) = @_;
$pkg_or_obj->_frontend_log_debug_opts(@args);
my %opts = $pkg_or_obj->_common_validate_with(
    params          => \@args,
    additional_spec => {
        _optional_scalars(
            qw( target-ip

                target-address
                )
        ),
    },
    api => 'system_node_restore_backup',
);
$pkg_or_obj->call_on_apiset(
    %opts,
    choices => [
        {   method    => '_restore_backup_cmode_cli',
            interface => 'CLI',
            set       => 'CMode',
        },
    ],
    frontend => 'NACL::C::SystemNode::restore_backup',
);

$Log->exit() if $may_exit;
}

sub _restore_backup_cmode_cli {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @opts) = @_;
$pkg_or_obj->_base_backend_cmode_cli(@opts,
    api => 'system_node_restore_backup',);
$Log->exit() if $may_exit;
}

=head2 show_discovered

my $show_discovered_obj = NACL::C::SystemNode->show_discovered(command_interface => $ci,
                      [ filter => { %filter } ,]
          [ requested_fields => [ @req_fields ]);

                or

my @show_discovered_objs = NACL::C::SystemNode->show_discovered(command_interface => $ci,
                      [ filter => { %filter } ,]
          [ requested_fields => [ @req_fields ]);

                or

my $show_discovered_obj = $SystemNode_obj->show_discovered();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-SystemNodeShowDiscovered-pm/ATTRIBUTES>
of NACL::CS::SystemNodeShowDiscovered as input.
This method returns objects of the type L<NACL::CS::SystemNodeShowDiscovered|lib-NACL-CS-SystemNodeShowDiscovered-pm>.

=cut

sub show_discovered {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @args) = @_;
my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
require NACL::CS::SystemNodeShowDiscovered;
my @state_objs = NACL::CS::SystemNodeShowDiscovered->fetch(%opts);
$Log->exit() if $may_exit;
return wantarray ? @state_objs : $state_objs[0];
}

=head2 setup

NACL::C::SystemNode->setup(
                            command_interface => $ci,
                            ...);

(Class method)
This method is used to initial node setup wizard.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node_mgmt_ip' => $string >>

Node management lif IP
Applicable for CMode CLI

=item C<< 'yes' => $string >>

Yes
Applicable for CMode CLI

=item C<< 'node_mgmt_gateway' => $string >>

Node management gateway
Applicable for CMode CLI

=item C<< 'node_mgmt_netmask' => $string >>

Node management lif netmask
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "system node setup" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub setup {
$Log->enter() if $may_enter;
my ($pkg, @args) = @_;
$pkg->_frontend_log_debug_opts(@args);
my %opts = $pkg->_common_validate_with(
    params              => \@args,
    ignore_primary_keys => 1,
    api                 => 'system_node_setup',
    ,
    additional_spec => {
        _optional_scalars(
            qw( node_mgmt_ip yes node_mgmt_gateway node_mgmt_netmask
                )
        )
    },
);
$pkg->call_on_apiset(
    %opts,
    choices => [
        {   method    => '_setup_cmode_cli',
            interface => 'CLI',
            set       => 'CMode',
        },
    ],
    frontend => 'NACL::C::SystemNode::setup',
);

$Log->exit() if $may_exit;
}

sub _setup_cmode_cli {
$Log->enter() if $may_enter;
my ($pkg, @opts) = @_;
$pkg->_base_backend_cmode_cli(@opts, api => 'system_node_setup',);
$Log->exit() if $may_exit;
}

=head2 show_memory_errors

my $show_memory_errors_obj = NACL::C::SystemNode->show_memory_errors(command_interface => $ci,
                      [ filter => { %filter } ,]
          [ requested_fields => [ @req_fields ]);

                or

my @show_memory_errors_objs = NACL::C::SystemNode->show_memory_errors(command_interface => $ci,
                      [ filter => { %filter } ,]
          [ requested_fields => [ @req_fields ]);

                or

my $show_memory_errors_obj = $SystemNode_obj->show_memory_errors();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-SystemNodeShowMemoryErrors-pm/ATTRIBUTES>
of NACL::CS::SystemNodeShowMemoryErrors as input.
This method returns objects of the type L<NACL::CS::SystemNodeShowMemoryErrors|lib-NACL-CS-SystemNodeShowMemoryErrors-pm>.

=cut

sub show_memory_errors {
$Log->enter() if $may_enter;
my ($pkg_or_obj, @args) = @_;
my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
require NACL::CS::SystemNodeShowMemoryErrors;
my @state_objs = NACL::CS::SystemNodeShowMemoryErrors->fetch(%opts);
$Log->exit() if $may_exit;
return wantarray ? @state_objs : $state_objs[0];
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
return (node => {type => SCALAR});
}

sub resource_record_method_name {
return "nodes";
}

# ...

sub default_apiset_must {
my ($self)   = @_;
my $hostrec  = $self->hostrec();
my $hostid   = $hostrec->id();
my $hosttype = $hostrec->hosttype();
my $mode     = $self->mode();
my %hash;
if ($mode eq "CMode") {
    %hash = (
        set => "CMode|Nodescope|Systemshell",
        set_source =>
            "command interface is a NACL::C::SystemNode and hosttype '$hosttype' is C-mode",
    );
} else {
    %hash = (
        set => "7Mode|Systemshell",
        set_source =>
            "command interface is a NACL::C::SystemNode and hosttype '$hosttype' is 7-mode",
    );
} ## end else [ if ( $mode eq "CMode" )

return {%{$self->SUPER::default_apiset_must()}, %hash,};
} ## end sub default_apiset_must

=head2 get_7m_or_nodescope_apiset

my $7m_or_nodescope_apiset =
    $node->get_7m_or_nodescope_apiset([interface => $interface]);
# Use this APISet object to run the 7Mode/Nodescope command

(For component users)
This returns a 7Mode APISet if the node is a 7Mode host and returns a
Nodescope APISet is the node is a CMode host. It defaults to returning a CLI
APISet, though an APISet of another interface can be requested for by passing
the interface parameter.

A number of 7Mode CLI commands have a CMode equivalent or a 7Mode ZAPI
equivalent. Using these in a mode/interface agnostic way can be done by using
the component. However, there are some commands that are available only in
7Mode (and Nodescope) CLI. These cannot be built into any component, hence the
APISet must be used. However, in this case it is necessary for the user to
create the appropriate APISet object. This method takes on the responsibility
of returning the appropriate APISet object (7Mode for a 7Mode host, and
Nodescope for a CMode host).

=over

=item Options

=over

=item interface

(Optional)
If not provided, it defaults to returning a CLI APISet. We can use this to
request for an APISet of a different interface.

=item Other options accepted by apiset()

All of the options (other than C<category> and C<set>) accepted by the
C<apiset|lib-NACL-C-CommandInterface-pm/apiset> method are accepted by
this method as well.

=back

=back

=cut

=head2 get_systemshell_apiset

my $systemshell_apiset = $node->get_systemshell_apiset();
Returns a Systemshell APISet object for the Node.

This method call does :
* SSH to NGSH
* Then run "systemshell -node local" to get into the system-shell. From that
point onwards, the connection is within that node's system-shell.

It is possible to get the systemshell apiset of other/partner node by passing in the node
param to the above call as :
my $systemshell_apiset = $node->get_systemshell_apiset(node => $node);

node can be SCALAR or OBJECT(NACL::C::Node|NACL::STask::Node)
This is useful for the cases where using the command_interface of one node
we need to access systemshell of other/partner node.

=cut


sub get_systemshell_apiset {
$Log->enter() if $may_enter;
my ($self, @opts) = @_;

   my %opts_hash = @opts ;
   my $apiset ;

   my $node_name ;
   if(defined $opts_hash{node}){
     $node_name = ( blessed ($opts_hash{node}) ) ? $opts_hash{node}->node(): $opts_hash{node} ;
   }
   my %conn_id = ();
   %conn_id = (connid => $opts_hash{connid}) if(defined $opts_hash{connid});

   if(defined $node_name && $node_name ne $self->command_interface()->node()){
     my $Host  = Tharn::host($node_name);
     $apiset   = NACL::APISet->new(
        hostobj   => $Host,
        category  => "Node",
        interface => "CLI",
        set       => "Systemshell",
        version_info => { compatibility => 0 },
        %conn_id 
     );
   }else{
     # Pass through any options provided to the call to apiset()
     $apiset
      = $self->apiset(interface => 'CLI', set => 'Systemshell', @opts);
   }
$Log->exit() if $may_exit;
return $apiset;
}

=head2 find_cm_nodes

my $cmode_node  = NACL::C::Node->find_cm_nodes();

my @cmode_nodes = NACL::C::Node->find_cm_nodes();

Returns Node objects for CMode Nodes.For example, if param NODES
is provided as NODES=cmode_node1,7mode_node1,cmode_node2,7mode_node2
this method would return Node objects only for cmode_node1 and cmode_node2.

This supports the standard facilities provided by C<find> such as
supporting scalar/list context and accepting C<allow_empty>.

=cut

sub find_cm_nodes {
$Log->enter() if $may_enter;
my ($pkg, @args) = @_;
my @cm_nodes = $pkg->_find_all_nodes(@args, mode => 'cmode');
$Log->exit() if $may_exit;
return wantarray ? @cm_nodes : $cm_nodes[0];
} ## end sub find_cm_nodes

=head2 find_7m_nodes

my $_7mode_node  = NACL::C::Node->find_7m_nodes();

my @_7mode_nodes = NACL::C::Node->find_7m_nodes();

Returns Node objects for 7Mode Nodes. For example, if param NODES
is provided as NODES=cmode_node1,7mode_node1,cmode_node2,7mode_node2
this method would return Node objects only for 7mode_node1 and 7mode_node2.

This supports the standard facilities provided by C<find> such as
supporting scalar/list context and accepting C<allow_empty>.

=cut

sub find_7m_nodes {
$Log->enter() if $may_enter;
my ($pkg, @args) = @_;
my @_7m_nodes = $pkg->_find_all_nodes(@args, mode => '7mode');
$Log->exit() if $may_exit;
return wantarray ? @_7m_nodes : $_7m_nodes[0];
} ## end sub find_7m_nodes

=head2 is_vldb_online

NACL::C::Node->is_vldb_online( command_interface => $command_interface);

(or)

my $return = $command_interface->is_vldb_online();

(Class or Instance call) This method checks whether the vldb is online on the
node which is present in the command interface object.
If the return is 1 vldb is online, else if the return is 0 the vldb is offline.

=cut

sub is_vldb_online {
$Log->enter() if $may_enter;
my ($obj, %opts) = @_;

my ($ci, $vldb_online);
if ($obj->isa('NACL::C::CommandInterface::ONTAP')) {
    $ci = $obj;
} elsif ($opts{command_interface}) {
    $ci = $opts{command_interface};
} else {
    $Log->exit() if $may_exit;
    NATE::BaseException->throw("Command interface is required field");
}

my $systemshell_apiobj = $ci->apiset(
    category  => "Node",
    interface => "CLI",
    set       => "Systemshell",
);

my $spmctl_op = $systemshell_apiobj->spmctl(list => 1);
my $job_list = $spmctl_op->get_parsed_output();

foreach my $handle (@$job_list) {
    if ($handle->{'Exec'} =~ /vldb/i && $handle->{'State'} =~ /running/i)
    {
        $vldb_online = 1;
        last;
    } else {
        $vldb_online = 0;
    }
}

$Log->exit() if $may_exit;
return $vldb_online;
}

=head2 storage_disk_zerospares

NACL::STask::Node->storage_disk_zerospares( command_interface => $command_interface);

(or)

$Node_obj->storage_disk_zerospares();

(Class or Instance call) This method zeros all spare disks on both nodes
or the node specified

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< owner => $node >>

(Optional) Specifies which node's disks are zeroed

=item C<< '*' >>

This method takes all other options supported by "storage disk zerospares" command in ONTAP

=back

=back

=cut

sub storage_disk_zerospares{
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;
    delete $opts{node};
    NACL::STask::StorageDisk->zerospares(%opts, command_interface => $self);
    $Log->exit() if $may_exit;
}

=head2 is_alive

NACL::C::Node->is_alive( command_interface => $command_interface);

(or)

$Node_obj->is_alive();

(Class or Instance call) This method checks whether node is alive or NOT

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=back

=back

=cut

sub is_alive {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    my $ci;
    if ($pkg_or_obj->isa('NACL::C::CommandInterface::ONTAP')) {
        $ci = $pkg_or_obj;
    } elsif ($opts{command_interface}) {
        $ci = $opts{command_interface};
    } else {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("Command interface is required field");
    }

    my $ip = $ci->hostrec->default_ip();
    my $is_alive = ping(host => $ip);

    $Log->exit() if $may_exit;
    return $is_alive;
}

# For this command interface, what is the common prefix for the test parameters?
sub _ci_to_test_param_mapping {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;
    return 'FILER';
}

## This is a helper method. It returns the list of nodes
## which are in the mode specified by input param 'mode'
sub _find_all_nodes {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => {mode => {type => SCALAR},},
        allow_extra => 1
    );
    my $mode      = delete $opts{mode};
    my @all_nodes = $pkg->find(%opts);
    my (@specific_nodes, $bool, $cond);
    foreach my $node (@all_nodes) {
        $bool = $node->is_cmode();
        if ($mode eq 'cmode') {
            $cond = $bool;
        } else {
            $cond = !$bool;
        }
        if ($cond) {
            push @specific_nodes, $node;
        }
    }
    if (!@specific_nodes && !$opts{allow_empty}) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::NoElementsFound->throw("No $mode nodes found");
    }
    $Log->exit() if $may_exit;
    return wantarray ? @specific_nodes : $specific_nodes[0];
} ## end sub _find_all_nodes

sub _halt_reboot_base_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %args) = @_;

    try {
        $pkg_or_obj->_base_backend_cmode_cli(%args);
    }
    catch NACL::APISet::Exceptions::TimeoutException with {

        # Its expected. Don't do anything
        } catch NATE::BaseException with {
        my $exception = shift;
        my $text      = $exception->text();

        if ($text =~ /Unexpected EOF while reading output from command/i){
           # Don't do anything
        }elsif ($text =~ /Timeout waiting for command to complete/i) {
            # Ignore the timeout exception
            $Log->trace('Ignoring expected connection timeout : '. $exception->text() );
        }else {
            $Log->exit() if $may_exit;
            $exception->throw();
         }
        };

    $Log->exit() if $may_exit;
}

=head2 setflag

Executes the setflag command on the Node

    my $response = $Node->setflag(...);

This method accepts all input accepts by setflag apiset

=cut

__PACKAGE__->_nodescope_builder(
    command => 'setflag',
);

=head2 printflag

Executes the printflag command on the Node
my $response= $Node->printflag();

This method does not accept any iparameters

=cut

__PACKAGE__->_nodescope_builder(
    command => 'printflag',
);

=head2 hammer_abort

Executes the hammer abort command on the Node

my $response = $Node->hammer_abort();

This method does not accept any parameters.

=cut

__PACKAGE__->_nodescope_builder(
    command => 'hammer_abort',
);

=head2 hammer_pause

Executes the hammer pause command on the Node

my $response = $Node->hammer_pause();

This method does not accept any parameters.

=cut

__PACKAGE__->_nodescope_builder(
    command => 'hammer_pause',
);

=head2 hammer_restart

Executes the hammer restart command on the Node

my $response = $Node->hammer_restart();

This method does not accept any parameters.

=cut

__PACKAGE__->_nodescope_builder(
    command => 'hammer_restart',
);

=head2 sync

Executes the sync command on the Node

my $response= $Node->sync();

This method does not accept any parameters.

=cut

__PACKAGE__->_nodescope_builder(
    command => 'sync',
);

=head2 wafl_flush_cache

Executes the wafl flush cache command on the Node

my $response= $Node->wafl_flush_cache(...);

This method accepts all input accepted by wafl_flush_cache apiset


=cut

__PACKAGE__->_nodescope_builder(
    command => 'wafl_flush_cache',
);

=head2 hammer_status

Executes the hammer status command on the Node and returns a CS object.

my $response= $Node->hammer_status(...);

This method accepts all input accepted by hammer status

=cut

__PACKAGE__->_cs_call_method_builder(method => 'hammer_status');

sub _hosttype {
    $Log->enter() if $may_enter;
    my ($self)   = @_;
    $Log->exit() if $may_exit;
    return $self->command_interface()->hostrec()->hosttype();
}

=head2 flexgroup_placement

Executes the flexgroup placement command on the Node .

my $response= $Node->flexgroup_placement (...);

This method accepts all input accepted by flexgroup_placement apiset method.


=cut

__PACKAGE__->_nodescope_builder(
    command => 'flexgroup_placement',
);

=head2 bootargs_set

Executes the bootargs_set command on the Node

    my $response = $Node->bootargs_set(...);

This method accepts all input accepts by bootargs apiset

=cut

__PACKAGE__->_nodescope_builder(
    command => 'bootargs_set',
);


1;
