# $Id$
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemNodeCoredumpConfig Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemNodeCoredumpConfig

=head1 DESCRIPTION

C<NACL::C::SystemNodeCoredumpConfig> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemNodeCoredumpConfig.  A related class is
L<NACL::CS::SystemNodeCoredumpConfig|lib-NACL-CS-SystemNodeCoredumpConfig-pm>, which represents
the past state of an ONTAP SystemNodeCoredumpConfig.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 node

=cut

package NACL::C::SystemNodeCoredumpConfig;

use strict;
use warnings;

use NACL::CS::SystemNodeCoredumpConfig;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::Exceptions::SystemNodeCoredumpConfig::DoesNotExist();
use NACL::APISet::Exceptions::ResponseException qw(:try);
use Data::Dumper;

=head1 METHODS

=head2 new

  my $SystemNodeCoredumpConfig = NACL::C::SystemNodeCoredumpConfig->new(command_interface=>$ci,
                                   node =>$node,
                                  );

Return a new SystemNodeCoredumpConfig component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'node',];

=head2 find

    my $SystemNodeCoredumpConfig  = NACL::C::SystemNodeCoredumpConfig->find(command_interface => $ci, ...);
    my @SystemNodeCoredumpConfigs = NACL::C::SystemNodeCoredumpConfig->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $SystemNodeCoredumpConfig_state = $NACL::C::SystemNodeCoredumpConfig->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemNodeCoredumpConfig would return
a NACL::CS::SystemNodeCoredumpConfig object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

   NACL::C::SystemNodeCoredumpConfig->modify(
                                command_interface => $ci,
                                 node => $node,
                                  ...);

                                 or

    $SystemNodeCoredumpConfig_object->modify(...);

(Class or instance method)
This method is used to modify coredump configuration.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes 'options' command for 7Mode CLI.

Invokes coredump-config-modify-iter method for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node-name

=item C<< "save-attempts" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: save-attempts

=item C<< "coredump-attempts" => $string >>

Applicable for CMode CLI/ZAPI or 7Mode CLI

Maps to:

CM ZAPI: coredump-attempts

=item C<< "upload-location" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: upload-location

=item C<< "sparsecore-enabled" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-sparsecore-enabled

=item C<< "save-onstartup" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: save-onstartup

=item C<< "min-free" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: min-free

=item C<< "coredump-metadata-only" => $string >>

(Optional) Takes on/off values.

Applicable for 7Mode CLI

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system node coredump config modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( save-attempts coredump-attempts upload-location sparsecore-enabled
                    save-onstartup min-free coredump-metadata-only
                    )
            ),
        },
           cli_cmd => 'system node coredump config modify',
    );
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_modify_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_modify_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
                {   method    => '_modify_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
            ],
            frontend => 'NACL::C::SystemNodeCoredumpConfig::modify',
        );
    }
    catch NACL::C::Exceptions::Component with {
        $_[0]->throw;
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::SystemNodeCoredumpConfig::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api     => 'system_node_coredump_config_modify',
    );
    $Log->exit() if $may_exit;

} ## end sub _modify_cmode_cli

sub _modify_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $apiset = delete $opts{apiset};
    if ($opts{'coredump-attempts'}) {
        $apiset->options(
            "coredump.dump.attempts" => $opts{'coredump-attempts'});
    }
    if ($opts{'coredump-metadata-only'}) {
        $apiset->options(
            "coredump.metadata_only" => $opts{'coredump-metadata-only'});
    }
    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    $pkg_or_obj->_iter_helper(
        @args,
        api      => 'coredump-config-modify-iter',
        show_cmd => 'system node coredump config show',
        copy     => [
            qw(coredump-attempts min-free save-attempts save-onstartup
                upload-location)
        ],
        map => {
            "node"               => "node-name",
            "sparsecore-enabled" => "is-sparsecore-enabled",
        },
        does_not_exist_error => "entry doesn't exist",
        does_not_exist_exception =>
            'NACL::C::Exceptions::SystemNodeCoredumpConfig::DoesNotExist'
    );

    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('node' => {type => SCALAR},);
}

1;
