#
# Copyright (c) 2001-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemNodeImage Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemNodeImage

=head1 DESCRIPTION

C<NACL::C::SystemNodeImage> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemNodeImage.  A related class is
L<NACL::CS::SystemNodeImage|lib-NACL-CS-SystemNodeImage-pm>, which represents
the past state of an ONTAP SystemNodeImage.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 node

=head2 image

=cut

package NACL::C::SystemNodeImage;

use strict;
use warnings;

use NACL::CS::SystemNodeImage;
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NATE::BaseException qw(:try);
use NACL::C::Exceptions::SystemNodeImage::Update64BitAggr;
use NACL::C::Exceptions::SystemNodeImage::Remove32BitData;
use NACL::C::Exceptions::SystemNodeImage::DisableNFSv3;
use NACL::C::Exceptions::SystemNodeImage::VerifySignaturesImage;

=head1 METHODS

=head2 new

  my $SystemNodeImage = NACL::C::SystemNodeImage->new(
                                   command_interface => $ci,
                                   'node' => $node,
                                   'image' => $image,
                                  );

Return a new SystemNodeImage component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'node',
    scalar => 'image',
];

=head2 find

    my $SystemNodeImage  = NACL::C::SystemNodeImage->find(command_interface => $ci, ...);
    my @SystemNodeImages = NACL::C::SystemNodeImage->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my SystemNodeImage_state = $NACL::C::SystemNodeImage->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemNodeImage would return
a NACL::CS::SystemNodeImage object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 show_update_progress

    NACL::C::SystemNodeImage->show_update_progress(
        command_interface => $ci,
        node              => $node_name,
        'method-timeout'  => $timeout
    );

(Class method)
This method can be used to wait in the foreground for the completion of the
image update on the specified node. (i.e. it returns control once the
image update is completed) It periodically updates the update
progress (shows the % downloaded). Note that this is the default behavior
of this command (i.e. with follow set to true).

The command also supports not following the progress in the foreground
but merely showing the current progress and returning immediately. (this
can be done with follow set to false). For this,
L<show_update_progress_no_follow|lib-NACL-C-SystemNodeImage-pm/show_update_progress_no_follow>
can be used.

If this method is invoked when there is update in progress on that node
(for example: if we invoked this after the update is completed), then no
exception is thrown. It simply returns control to the user.

Since this requires a persistent connection, this method is applicable only
for CLI. The equivalent ZAPI "system-image-update-progress-get" does
not allow for following the progress, but merely returns the current
progress. This can also be achieved using the method
L<show_update_progress_no_follow|lib-NACL-C-SystemNodeImage-pm/show_update_progress_no_follow>.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< node => $node_name >>

(Required)
The name of the node on which the image updating is occuring.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
This is the time it could take for the update to complete. Note that though
this option is not required, in most scenarios this option would have to
be provided. Updating the image could take a significant amount of time,
but the default NATE timeout is only 60 seconds which is definitely
insufficient time to be able to complete an image update. For this reason,
this option will almost always need to be provided (not providing this
will result in a timeout and a
L<NACL::APISet::Exceptions::TimeoutException|lib-NACL-APISet-Exceptions-TimeoutException-pm>
being thrown.)

This method does not provide any default value for this option because the
time it takes to update the image is variable and depends on factors
such as time taken to download the image from the server. It also possible
to invoke this method at any stage of the update, so it is impossible
to come up with a suitable default. For these reasons, it is the responsibility
of the user to provide a suitable timeout value.

=item C<< 'follow' => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $node_name >>

Applicable for CMode CLI.

=item C<< 'exitstatus' => $string >>

Applicable for CMode CLI.

=item C<< 'runstatus' => $string >>

Applicable for CMode CLI.

=item C<< 'exitmessage' => $string >>

Applicable for CMode CLI.

=item C<< 'sequence' => $string >>

Applicable for CMode CLI.

=item C<< 'lastmsg' => $string >>

Applicable for CMode CLI.

=item C<< 'phase' => $string >>

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub show_update_progress {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $Log->debug(
        "Opts to 'show_update_progress' front end are:\n" . Dumper(\@_));
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw(follow node

                    exitstatus runstatus exitmessage sequence lastmsg phase
                    )
            ),
        },
        only_static_method  => 1,
        ignore_primary_keys => 1,
        cli_cmd             => 'system node image show-update-progress',
    );

    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_show_update_progress_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub show_update_progress

sub _show_update_progress_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw(node follow

            exitstatus runstatus exitmessage sequence lastmsg phase

            exitstatus runstatus exitmessage sequence lastmsg phase
            extended_query )
    ];
	 # Force it to follow in the foreground
    $opts{follow} = 'true';

    $Log->debug(
        "Opts to API call 'system_node_image_show_update_progress' are: \n"
            . Dumper(\%opts));

    $pkg_or_obj->_base_backend_cmode_cli(%opts,
            api    => 'system_node_image_show_update_progress',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _show_update_progress_cmode_cli

=head2 show_update_progress_no_follow

    my $update_progress_cs = NACL::C::SystemNodeImage->show_update_progress_no_follow(
        command_interface => $ci,
        node              => $node_name
    );

(Class method)
This method shows the current progress of the image update on the specified
node. It returns a
L<NACL::CS::SystemNodeImageShowUpdateProgressNoFollow|lib-NACL-CS-SystemNodeImageShowUpdateProgressNoFollow-pm>
object which contains details of the update.

When run through CLI, it invokes the show-update-progress command with follow
set to false.

This throws a
L<NACL::APISet::Exceptions::CommandFailedException|lib-NACL-APISet-Exceptions-CommandFailedException-pm>
if no update/install has ever occurred on the specified node.

(Note that this returns a CS object to provide an interface-agnostic way to
access the details. However, this method does B<not> support the standard
interface of CS::fetch, i.e all of the following are not applicable:
requested_fields, filter, allow_empty, being callable in scalar or list context)

Uses a CMode CLI/ZAPI APISet.

Invokes 'system-image-update-progress-get' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< node => $node_name >>

(Required)
The name of the node on which the image update is occuring.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub show_update_progress_no_follow {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $Log->debug("Opts to 'show_update_progress_no_follow' front end are:\n"
            . Dumper(\@_));
    my %opts = $pkg->_common_validate_with(
        params              => \@_,
        additional_spec     => {_optional_scalars(qw(node))},
        only_static_method  => 1,
        ignore_primary_keys => 1,
        cli_cmd             => 'system node image show-update-progress',
    );
    my $cs_obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_show_update_progress_no_follow_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_show_update_progress_no_follow_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $cs_obj;
} ## end sub show_update_progress_no_follow

sub _show_update_progress_no_follow_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $response;
    # Force it to not follow in the foreground
    $opts{follow} = 'false';
    my $allowed_cmode_options = [qw(node extended_query )];
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
            api    => 'system_node_image_show_update_progress',
            copy   => $allowed_cmode_options,
            apiset_response_obj => \$response
	);
    $Log->debug(
        "Opts to API call 'system_node_image_show_update_progress' are: \n"
            . Dumper(\%opts));

    my $output    = $response->get_parsed_output();
    my $first_row = $output->[0];
    require NACL::CS::SystemNodeImageShowUpdateProgressNoFollow;
    my $cs_obj = NACL::CS::SystemNodeImageShowUpdateProgressNoFollow->new(
        command_interface => $opts{command_interface},
        %$first_row
    );
    $Log->exit() if $may_exit;
    return $cs_obj;
} ## end sub _show_update_progress_no_follow_cmode_cli

sub _show_update_progress_no_follow_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $response;
    my $allowed_cmode_options = [qw(node )];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'system_image_update_progress_get',
            copy   => $allowed_cmode_options,
            apiset_response_obj => \$response
	);
    my $output    = $response->get_parsed_output();
    my $first_row = $output->[0];

# There are two cases where "last-message" will have the message
# "There is no update/install in progress".
#
# Case 1: Update/install has completed
# Parsed output for this case:
#  $VAR1 = [
#             {
#               'exit-status' => 'Failed',
#               'run-status' => 'Exited',
#               'phase' => 'Validate Script',
#               'exit-message' => 'Install Validation Failed. ERROR: Attempting <snipped>'
#               'last-message' => 'There is no update/install in progress'
#             }
#           ];
# We do not consider this an error, since we consider this to be a request
# for details of the previously completed operation.
#
# Case 2: No update/install has ever happened on this node
# Parsed output for this case:
# $VAR1 = [
#            {
#              'last-message' => 'There is no update/install in progress'
#            }
#        ];
#
# We consider this an error since we've requested for the update progress
# details on a node where an update never happened. We throw a
# CommandFailedException for this case.
# The way we determine it's this case is that "last-message" is the only
# key in the parsed output. (notice the extra fields present in the
# parsed output for case 1)

    if ($first_row->{'last-message'} =~ m!no update/install in progress!
        && keys %$first_row == 1)
    {
        require NACL::APISet::Exceptions::CommandFailedException;
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $first_row->{'last-message'},
            response_object => $response
        );
    } ## end if ( $first_row->{'last-message'...
    require NACL::CS::SystemNodeImageShowUpdateProgressNoFollow;
    my $cs_obj = NACL::CS::SystemNodeImageShowUpdateProgressNoFollow->new(
        command_interface => $opts{command_interface},
        %$first_row
    );
    $Log->exit() if $may_exit;
    return $cs_obj;
} ## end sub _show_update_progress_no_follow_cmode_zapi

=head2 modify

    NACL::C::SystemNodeImage->modify(
                                command_interface => $ci,
                                'node' => $node,
                                'image' => $image,
                                ...);

                                 or

    $SystemNodeImage_object->modify(...);

(Class or instance method)
This method is used to modify software image configuration.

Uses a CMode CLI/ZAPI APISet.

Invokes 'system-image-modify' API for CMode ZAPI.

This method defaults to invalidating the version manager cache.
See L<NACL::C::CommandInterface::ONTAP|lib-NACL-C-CommandInterface-ONTAP-pm>.
Use C<< nacl_invalidate_version_manager_cache => 0 >> to prevent the version
manager cache invalidation.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node

=item C<< 'image' => $image >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: image

=item C<< 'force-setdefault' => $string >>

Applicable for CMode CLI.

=item C<< 'isdefault' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-default

=item C<< 'newest-image-as-default' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'ignore-aggrvolcheck' => $string >>


Ignore Offline Aggregates and Volumes possible value(s) are, true,false
Applicable for CMode CLI

=item C<< nacl_invalidate_version_manager_cache => 0|1 >>

(Defaults to 1)
Specifies whether to invalidate the version manager cache.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $Log->debug("Opts to 'modify' front end are:\n" . Dumper(\@_));
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( force-setdefault isdefault newest-image-as-default

                    ignore-aggrvolcheck
                    )
            ),
            nacl_invalidate_version_manager_cache => {type => SCALAR, default => 1},
        },
        cli_cmd => 'system node image modify',
    );
    my $invalidate_cache = delete $opts{nacl_invalidate_version_manager_cache};

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
    );

    # Version details would have changed, so invalidate existing cache.
    NACL::C::CommandInterface::ONTAP->invalidate_version_manager_cache() if $invalidate_cache;
    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( force-setdefault isdefault newest-image-as-default node image
            extended_query
            ignore-aggrvolcheck
            )
    ];

    $Log->debug("Opts to API call 'system_node_image_modify' are: \n"
        . Dumper(\%opts));

    $pkg_or_obj->_base_backend_cmode_cli(%opts,
            api    => 'system_node_image_modify',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( node image
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'system_image_modify',
            copy   => $allowed_cmode_options,
            map    => {"isdefault" => "is-default"},
	);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 update

    NACL::C::SystemNodeImage->update(
                                command_interface => $ci,
                                'node' => $node,
                                ...);

(Class method)
This method is used to perform software image upgrade/downgrade.

Uses a CMode CLI/ZAPI,7Mode CLI APISet.

Invokes 'system-image-update' API for CMode ZAPI.

Invokes 'software update' command for 7Mode CLI.

This method defaults to invalidating the version manager cache.
See L<NACL::C::CommandInterface::ONTAP|lib-NACL-C-CommandInterface-ONTAP-pm>.
Use C<< nacl_invalidate_version_manager_cache => 0 >> to prevent the version
manager cache invalidation.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node

=item C<< 'setdefault' => $string >>

(Optional)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: setdefault

=item C<< 'sequence' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'replace-package' => $string >>

(Optional)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI: replace-package

7M CLI : -f (Programmatic name: overwrite destination file)

=item C<< 'rename-package' => $string >>

(Optional)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI: rename-package

7M CLI : destination file

=item C<< 'force-setdefault' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'background' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'retrycapseq' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'package' => $string >>

(Required)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI: package

7M CLI : url

=item C<< 'ignore-compatibility' => $string >>

(Optional)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ignore-compatibility

=item C<< 'replace' => $string >>

(Optional)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: replace

=item C<< 'lastmsg' => $string >>

(Optional)

Applicable for 7Mode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

=item C<< 'request-automatic-reboot' => $boolean >>

(Optional)

Applicable for 7Mode CLI.

=item C<< 'disable-immediate-reboot' => $boolean >>

(Optional)

Applicable for 7Mode CLI.

=item C<< 'disable-automatic-installation' => $boolean >>

(Optional)

Applicable for 7Mode CLI.

The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'ignore-offline' => $string >>

Ignore Offline Aggregates and Volumes
Applicable for CMode CLI

=item C<< nacl_invalidate_version_manager_cache => 0|1 >>

(Defaults to 1)
Specifies whether to invalidate the version manager cache.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::SystemNodeImage::Update64BitAggr>

This exception might be thrown during an upgrade process.

=item C<NACL::C::Exceptions::SystemNodeImage::Remove32BitData>

This exception might be thrown during an upgrade process.

=item C<NACL::C::Exceptions::SystemNodeImage::DisableNFSv3>

Nfs dos is not supported in LB and FS filers. But they are
supported in SN version. This exception will be thrown if there
is an attempt to upgrade the image from lower version(like SN.3)
to higher version(LB or FS).

=item C<NACL::C::Exceptions::SystemNodeImage::VerifySignaturesImage>

This exception might be thrown during an upgrade process.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub update {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $Log->debug("Opts to 'update' front end are:\n" . Dumper(\@_));
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( setdefault sequence replace-package rename-package force-setdefault
                    background retrycapseq node package ignore-compatibility replace lastmsg
                    request-automatic-reboot disable-immediate-reboot disable-automatic-installation

                    ignore-offline
                    )
            ),
            nacl_invalidate_version_manager_cache => {type => SCALAR, default => 1},
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'system node image update',
    );
    my $invalidate_cache = delete $opts{nacl_invalidate_version_manager_cache};

    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_update_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_update_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
                {   method    => '_update_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },

            ],
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        foreach my $type (qw( Update64BitAggr Remove32BitData DisableNFSv3 VerifySignaturesImage)) {
            my $full_type = "NACL::C::Exceptions::SystemNodeImage::$type";
            $full_type->detect_convert_and_throw(exception => $exception);
        }
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    # Version details would have changed, so invalidate existing cache.
    NACL::C::CommandInterface::ONTAP->invalidate_version_manager_cache() if $invalidate_cache;

    $Log->exit() if $may_exit;
} ## end sub update

sub _update_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( setdefault sequence replace-package rename-package force-setdefault
            background retrycapseq node package ignore-compatibility replace lastmsg
            extended_query
            ignore-offline
            )
    ];
    $Log->debug("Opts to API call 'system_node_image_update' are: \n"
        . Dumper(\%opts));

    $pkg_or_obj->_base_backend_cmode_cli(%opts,
            api    => 'system_node_image_update',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _update_cmode_cli

sub _update_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [
        qw( request-automatic-reboot disable-immediate-reboot
            disable-automatic-installation
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => {
            'rename-package'  => 'destination-file',
            'package'         => 'url',
            'replace-package' => 'overwrite-destination-file',
        },
        target => \%cmode_args,
    );

    $apiset->software_update(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _update_7mode_cli

sub _update_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( setdefault replace-package rename-package
            node package replace ignore-compatibility
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'system_image_update',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _update_cmode_zapi

=head2 fresh_install

    NACL::C::SystemNodeImage->fresh_install(
                                command_interface => $ci,
                                'node' => $node,
                                ...);

(Class method)
This method is used to perform software image fresh install.

Uses a CMode CLI/ZAPI APISet.

Invokes 'system-image-install' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node

=item C<< 'sequence' => $string >>

Applicable for CMode CLI.

=item C<< 'replace-package' => $string >>


Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: replace-package

=item C<< 'rename-package' => $string >>


Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: rename-package

=item C<< 'background' => $string >>

Applicable for CMode CLI.

=item C<< 'package' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: package

=item C<< 'lastmsg' => $string >>

Applicable for CMode CLI.

=item C<< 'only-image1' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub fresh_install {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $Log->debug("Opts to 'fresh_install' front end are:\n" . Dumper(\@_));
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( sequence replace-package rename-package background node package lastmsg
                    only-image1
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_fresh_install_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fresh_install_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub fresh_install

sub _fresh_install_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( sequence replace-package rename-package background node package lastmsg
            only-image1
            extended_query )
    ];
    $Log->debug("Opts to API call 'system_node_image_fresh_install' are: \n"
        . Dumper(\%opts));

    $pkg_or_obj->_base_backend_cmode_cli(%opts,
            api    => 'system_node_image_fresh_install',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _fresh_install_cmode_cli

sub _fresh_install_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( replace-package rename-package node package
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'system_image_install',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _fresh_install_cmode_zapi

=head2 get

    NACL::C::SystemNodeImage->get(
                                command_interface => $ci,
                                'node' => $node,

                                ...);

(Class method)
This method is used to fetch a file from a URL.

Uses a CMode CLI/ZAPI APISet.

Invokes 'system-image-fetch-package' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node

=item C<< 'lastmsg' => $string >>

Applicable for CMode CLI.

=item C<< 'sequence' => $string >>

Applicable for CMode CLI.

=item C<< 'replace-package' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: replace-package

=item C<< 'rename-package' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: rename-package

=item C<< 'background' => $string >>

Applicable for CMode CLI.

=item C<< 'package' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: package

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub get {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $Log->debug("Opts to 'get' front end are:\n" . Dumper(\@_));
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( lastmsg sequence replace-package rename-package background node package
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'system node image get',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_get_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_get_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub get

sub _get_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( lastmsg sequence replace-package rename-package background node package
            extended_query )
    ];
    $Log->debug("Opts to API call 'system_node_image_get' are: \n"
        . Dumper(\%opts));
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
            api    => 'system_node_image_get',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _get_cmode_cli

sub _get_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( replace-package rename-package node package
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'system_image_fetch_package',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _get_cmode_zapi

=head2 package_delete

    NACL::C::SystemNodeImage->package_delete(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to delete a software package.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $string >>

Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< 'package' => $string >>

Package File Name
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by "system node image package delete" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub package_delete {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( node package
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd               => 'system node image package delete',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_package_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemNodeImage::package_delete',
    );

    $Log->exit() if $may_exit;
}

sub _package_delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_cli(
        %opts,
        api     => 'system_node_image_package_delete',
    );
    $Log->exit() if $may_exit;
}

=head2 package_show

    my $package_show_obj = NACL::C::SystemNodeImage->package_show(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @package_show_objs = NACL::C::SystemNodeImage->package_show(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $package_show_obj = $SystemNodeImage_obj->package_show(); 

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-SystemNodeImagePackageShow-pm/ATTRIBUTES>
of NACL::CS::SystemNodeImagePackageShow as input. 
This method returns objects of the type L<NACL::CS::SystemNodeImagePackageShow|lib-NACL-CS-SystemNodeImagePackageShow-pm>.

=cut

sub package_show {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::SystemNodeImagePackageShow;
    my @state_objs = NACL::CS::SystemNodeImagePackageShow->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'node'  => {type => SCALAR},
        'image' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
