# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemSnmpCommunity Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemSnmpCommunity

=head1 DESCRIPTION

C<NACL::C::SystemSnmpCommunity> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemSnmpCommunity.  A related class is
L<NACL::CS::SystemSnmpCommunity|lib-NACL-CS-SystemSnmpCommunity-pm>, which represents
the past state of an ONTAP SystemSnmpCommunity.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

=head2 command_interface

=head2 community_name

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

package NACL::C::SystemSnmpCommunity;

use strict;
use warnings;
use NACL::CS::SystemSnmpCommunity;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $SystemSnmpCommunity = NACL::C::SystemSnmpCommunity->new(command_interface=>$ci,
                                  );

Return a new SystemSnmpCommunity component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'permission',
    scalar => 'community',
    scalar => 'community_name',
];

=head2 add

   NACL::C::SystemSnmpCommunity->add(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to add a new community with the specified access control type.

Uses a CMode CLI/ZAPI APISet or 7Mode CLI APISET.

Invokes 'snmp community add' API for CMode ZAPI, 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "type" => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI: access-control

7M CLI: permission

=item C<< "communityname" => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

CM ZAPI: community

=item C<< "community-name" => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI: community

7M CLI : community

=item C<< 'vserver' => $string >>

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system snmp community add" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( type communityname

                    vserver
                    )
            ),
        },
        cli_cmd => 'system snmp community add',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver if(defined $opts{vserver});
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_add_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_add_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_add_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemSnmpCommunity::add',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub add

sub _add_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'system_snmp_community_add',
    );
    $Log->exit() if $may_exit;
} ## end sub _add_cmode_cli

sub _add_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "type"           => "permission",
            "community-name" => "community",
        },
        target => \%cmode_args,
    );
    $apiset->snmp_community_add(%cmode_args);
    $Log->exit() if $may_exit;
    return $pkg->new(command_interface => $command_interface,);

} ## end sub _add_7mode_cli

sub _add_cmode_zapi {
    $Log->enter() if $may_enter;
	my ($pkg, @opts) = @_;
	my $map  = {
        "type"           => "access-control",
        "community-name" => "community",
        "communityname"  => "community",
    };
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'snmp-community-add',
        map  => $map,
    );
    $Log->exit() if $may_exit;
} ## end sub _add_cmode_zapi


=head2 delete

   NACL::C::SystemSnmpCommunity->delete(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to deletes community with the specified access control type.

Uses a CMode CLI APISet or CMode ZAPI APISet or 7Mode CLI APISET.

Invokes 'snmp community delete' API for CMode ZAPI, 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "type" => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI: access-control

7M CLI: permission

=item C<< "communityname" => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

CM ZAPI: community

=item C<< "community-name" => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI: community

7M CLI : community

=item C<< 'vserver' => $string >>

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system snmp community delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( type communityname

                    vserver
                    )
            ),
        },
        cli_cmd => 'system snmp community delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver if(defined $opts{vserver});
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemSnmpCommunity::delete',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'system_snmp_community_delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "type"           => "permission",
            "community-name" => "community",
        },
        target => \%cmode_args,
    );

    $apiset->snmp_community_delete(%cmode_args);
    $Log->exit() if $may_exit;
    return $pkg->new(command_interface => $command_interface,);

} ## end sub _delete_7mode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $map  = {
        "type"           => "access-control",
        "community-name" => "community",
        "communityname"  => "community",
    };
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'snmp-community-delete',
        map  => $map,
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 modify

    NACL::C::SystemSnmpCommunity->modify(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'community-name' => $community_name,
                                ...);

                                 or

    $SystemSnmpCommunity_obj->modify(...);

(Class or instance method)

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

=item C<< 'community-name' => $community_name >>

(Required for class method, Not Applicable for instance method)

=item C<< 'access' => $string >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'community' => $string >>

Applicable for CMode CLI.

=item C<< 'vserver' => $string >>

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system snmp community modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( access

                    community vserver
                    )
            ),
        },
        api => 'system_snmp_community_modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemSnmpCommunity::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    $pkg_or_obj->_base_backend_cmode_cli(
        @args,
        api => 'system_snmp_community_modify',
    );

    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('community-name' => {type => SCALAR, optional => 1});
}

1;
