# $Id$
#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Cluster Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VPserver::Cluster

=head1 DESCRIPTION

C<NACL::C::VPserver::Cluster> is a derived class of
L<NACL::C::Component::VPserver|lib-NACL-C-Component-VPserver-pm>. It represents access
to VPserver Cluster Objects.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::VPserver|lib-NACL-C-VPserver-pm>

=cut

package NACL::C::VPserver::Cluster;

use strict;
use warnings;

use base 'NACL::C::Component::VPserver';

use NACL::C::VPserver;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);

use NACL::Exceptions::InvalidParam qw(:try);

=head1 METHODS

=head2 new

  my $Cluster = NACL::C::VPserver::Cluster->new(command_interface => $ci);

Return a new VPserver Cluster component object with the given attributes.

=cut

use Class::MethodMaker [
        scalar => [
            {-type => 'NACL::C::VPserver'},
            'command_interface'
        ],
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub init {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    if (!$opts{command_interface}) {
        NACL::Exceptions::InvalidParam->throw("Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Cluster::new.");
    }

    $Log->exit() if $may_exit;
    return $pkg;
}

=head2 list

    my $cluster_objs  = NACL::C::VPserver::Cluster->list(command_interface => $vp_ci);

(Class method)
A method which discovers all clusters that are registered with the VPserver (by
interacting with the given vpserver command interface).

Uses a SOAP VPserver APISet.

=over

=item Options

=over

=item C<< command_interface => $vp_ci >>

(Optional) See L<NACL::C::VPserver|lib-NACL-C-VPserver-pm>

=item C<< cluster_ip => $cluster_ip >>

(Optional) The IP Address of the cluster to perform a list

=item C<< vserver => $vserver >>

(Optional) The specific vserver to list.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub list {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->command_interface($opts{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface) {
        NACL::Exceptions::InvalidParam->throw("Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Cluster::list.");
    }

    my $cluster_ip = delete($opts{cluster_ip}) // "";
    my $vserver    = delete($opts{vserver})    // "";

    my @params = ();
    push(@params, {clusterIp => $cluster_ip}) if ($cluster_ip);
    push(@params, {svmName => $vserver})      if ($vserver);
    push(@params, {fullDetail => 'true'});

    my $response = $pkg->command_interface->call(command => "clusterList",
                                                 params  => \@params);
    $Log->exit() if $may_exit;
    return $response;
}

=head2 add

    my $command_interface = NACL::C::VPserver->new(vpserver => $vpserver_name);

    my $cluster_component_obj = NACL::C::VPserver::Cluster->new(command_interface => $command_interface);

    my $cluster_add_result = $cluster_component_obj->add();

A method which registers a Cluster with the VPserver (by interacting with the
given vpserver command interface). Instance Method calls require no additional parameters.
The values for the cluster to add and the vpserver are defined within the credential object
used within the NACL::C::VPserver object.  NOTE:  the time it takes this method to return
is proportional to the number of objects on the cluster being added.  Set the timeout value
if you are experiencing timeouts.

=over

=item Options

=over

=item C<< command_interface => $vp_ci >>

(Optional) See L<NACL::C::VPserver|lib-NACL-C-VPserver-pm>

=item C<< cluster_ip => $node_ip >>

(Required) The managment ip of the cluster for which you want to add.

=item C<< cluster_username => $cluster_username >>

(Required) The username for the cluster ip specified.

=item C<< cluster_password => $cluster_password >>

(Required) The password for the cluster ip specified.

=item C<< timeout => $timeout_number >>

(Optional) Sets the timeout value.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->command_interface(delete $opts{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface) {
        NACL::Exceptions::InvalidParam->throw("Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Cluster::add.");
    }
    if (!$opts{cluster_ip}) {
        NACL::Exceptions::InvalidParam->throw("Mandatory parameter 'cluster_ip' missing in call to NACL::C::VPserver::Cluster::add.");
    }
    if (!$opts{cluster_username}) {
        NACL::Exceptions::InvalidParam->throw("Mandatory parameter 'cluster_username' missing in call to NACL::C::VPserver::Cluster::add.");
    }
    if (!$opts{cluster_password}) {
        NACL::Exceptions::InvalidParam->throw("Mandatory parameter 'cluster_password' missing in call to NACL::C::VPserver::Cluster::add.");
    }

    my $credential_overrides = {filerIP => delete $opts{cluster_ip},
                                filerUsername => delete $opts{cluster_username},
                                filerPassword => delete $opts{cluster_password}};

    my $response = $pkg->command_interface->call(command => "clusterAdd",
                                                 %opts,
                                                 credential_overrides => $credential_overrides,
                                                 params => [{containerCreationSpec => undef},
                                                            {skipVersionCheck => 'false'}]);
    $Log->exit() if $may_exit;
    return $response;
}

=head2 delete

    my $cluster_delete_result = $cluster_component_obj->delete(command_interface => $vp_ci,
                                                               cluster => $cluster_ip);

A method which unregisters a Cluster with the VPserver (by interacting with the
given vpserver command interface).

=over

=item Options

=over

=item C<< command_interface => $vp_ci >>

(Optional) See L<NACL::C::VPserver|lib-NACL-C-VPserver-pm>

=item C<< cluster_ip => $cluster_ip >>

(Required) The managment ip of the cluster, or a node in the cluster, for which you want to remove.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->command_interface($opts{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface) {
        NACL::Exceptions::InvalidParam->throw("Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Cluster::delete.");
    }
    if (!$opts{cluster_ip}) {
        NACL::Exceptions::InvalidParam->throw("Mandatory parameter 'cluster_ip' missing in call to NACL::C::VPserver::Cluster::delete.");
    }

    my $response = $pkg->command_interface->call(command => "clusterDelete",
                                                 params => [{ipOrHost => $opts{cluster_ip}}]);
    $Log->exit() if $may_exit;
    return $response;
}

=head2 ensure_pe

    my $cluster_ensure_result = $NACL::C::VPserver::Cluster->ensure_pe(command_interface => $vp_ci,
                                                                       cluster => $cluster,
                                                                       vserver => $vserver,
                                                                       container => $container,
                                                                       protocol => $protocol,
                                                                       hosts => \@hosts);
(Class method)
A method which ensures the specified cluster's protocol endpoints with the VPserver (by interacting with the
given vpserver command interface).

Uses a SOAP VPserver APISet.

=over

=item Options

=over

=item C<< command_interface => $vp_ci >>

(Optional) See L<NACL::C::VPserver|lib-NACL-C-VPserver-pm>

=item C<< cluster_ip => $cluster_ip >>

(Required) The ip address or hostname of the cluster to ensure protocol endpoints for.

=item C<< vserver => $vserver >>

(Required) The vserver to ensure protocol endpoints for.

=item C<< container => $container >>

(Required) The container to ensure protocol endpoints for.

=item C<< protocol => $protocol >>

(Required) The protocol to ensure protocol endpoints for.

=item C<< hosts => \@hosts >>

(Required) An array ref iSCSI Nodenames, IP Addresses for NFS clients, or FC WWPNs to ensure protocol endpoints for.

=item C<< vc_guid => $vc_guid >>

(Optional) The VcGuid for the vcenter to use for ensuring protocol endpoints. If only one vcenter is
registered with the VPserver Command interface, this does not need to be specified.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub ensure_pe {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->command_interface($opts{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface) {
        NACL::Exceptions::InvalidParam->throw("Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Cluster::ensure_pe.");
    }
    if (!$opts{cluster_ip}) {
        NACL::Exceptions::InvalidParam->throw("Mandatory parameter 'cluster_ip' missing in call to NACL::C::VPserver::Cluster::ensure_pe.");
    }
    if (!$opts{vserver}) {
        NACL::Exceptions::InvalidParam->throw("Mandatory parameter 'vserver' missing in call to NACL::C::VPserver::Cluster::ensure_pe.");
    }
    if (!$opts{container}) {
        NACL::Exceptions::InvalidParam->throw("Mandatory parameter 'container' missing in call to NACL::C::VPserver::Cluster::ensure_pe.");
    }
    if (!$opts{protocol}) {
        NACL::Exceptions::InvalidParam->throw("Mandatory parameter 'protocol' missing in call to NACL::C::VPserver::Cluster::ensure_pe.");
    }
    if (!$opts{hosts}) {
        NACL::Exceptions::InvalidParam->throw("Mandatory parameter 'hosts' missing in call to NACL::C::VPserver::Cluster::ensure_pe.");
    }

    my @params;
    push(@params, {clusterIp => $opts{cluster_ip}});
    push(@params, {vServerName => $opts{vserver}});
    push(@params, {containerName => $opts{container}});
    push(@params, {protocol => $opts{protocol}});

    # push iqns/ips to the list of params
    foreach my $iqn_or_ip (@{$opts{hosts}}) {
        push(@params, {hosts => $iqn_or_ip});
    }

    push(@params, {detailedVerification => "true"});
    push(@params, {vcIdentifier => {vcGuid => $opts{vc_guid}}});
    push(@params, {examineOnly => "false"});

    my $response = $pkg->command_interface->call(command => "clusterEnsurePEs",
                                                 params => \@params);
    $Log->exit() if $may_exit;
    return $response;
}

=head2 rediscover

    my $cluster_objs  = NACL::C::VPserver::Cluster->rediscover(command_interface => $vp_ci,
                                                               cluster_ip => $cluster_ip);

(Class method)
A method which performs a rediscovery of the cluster specified.

Uses a SOAP VPserver APISet.

=over

=item Options

=over

=item C<< command_interface => $vp_ci >>

(Required) See L<NACL::C::VPserver|lib-NACL-C-VPserver-pm>

=item C<< cluster_ip => $cluster_ip >>

(Optional) The IP Address of the cluster to perform a rediscover against.

=item C<< vserver => $vserver >>

(Optional) The specific vserver to rediscover.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub rediscover {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->command_interface($opts{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface) {
        NACL::Exceptions::InvalidParam->throw("Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Cluster::list.");
    }
    my $cluster_ip = delete($opts{cluster_ip}) // "";
    my $vserver    = delete($opts{vserver})    // "";

    my @params = ();
    push(@params, {clusterIp   => $cluster_ip}) if ($cluster_ip);
    push(@params, {vServerName => $vserver})    if ($vserver);

    my $response = $pkg->command_interface->call(command => "clusterRediscover",
                                                 params => \@params);
    $Log->exit() if $may_exit;
    return $response;
}

1;
