#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NACL::C::VPserver::Vvol
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VPserver::Vvol

=head1 DESCRIPTION

C<NACL::C::VPserver::Vvol> is a derived class of
L<NACL::C::Component::VPserver|lib-NACL-C-Component-VPserver-pm>. It represents access
to VPserver Vvol Objects.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::VPserver|lib-NACL-C-Component-VPserver-pm>

=cut

package NACL::C::VPserver::Vvol;

use strict;
use warnings;

use base 'NACL::C::Component::VPserver';

use NACL::C::VPserver;
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);

use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

=head1 METHODS

=head2 new

  my $vp_ci = NACL::C::VPserver->new(vpserver => "my_vpserver");
  my $vvol_ci = NACL::C::VPserver::Vvol->new(command_interface => $vp_ci);

Return a new VPserver Vvol component object.

=cut

use Class::MethodMaker [
        scalar => [
            {-type => 'NACL::C::VPserver'},
            'command_interface'
        ],
];

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub init {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    if (!$opts{command_interface}) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(text => "Mandatory parameter 'command_interface' " .
                                                                       "missing in call to NACL::C::VPserver::Vvol::new.");
    }

    $Log->exit() if $may_exit;
    return $pkg;
}

=head2 list

    my $vp_ci = NACL::C::VPserver->new(vpserver => "my_vpserver");
    my $vvol_uuids  = NACL::C::VPserver::Vvol->list(command_interface => $vp_ci);

A method which returns back an array reference of all Vvols UUIDs that are known by the VPserver.

Uses a SOAP VPserver APISet.

=over

=item Options

=over

=item C<< command_interface => $vp_ci >>

(Optional) If executed as an instance method.

(Required) If executed as a class method.

See L<NACL::C::VPserver>

=back

=item Returns

This method returns back an array reference of Vvol UUID strings.

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub list {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->command_interface($opts{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(text => "Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Container::list.");
    }

    my $response = $pkg->command_interface->call(command => "vvolList",
                                                 params  => [{searchSpecs => undef}]);
    $Log->exit() if $may_exit;
    return $response;
}

=head2 list_information

    my $vp_ci = NACL::C::VPserver->new(vpserver => "my_vpserver");
    my $vvol_uuids = NACL::C::VPserver::Vvol->list(command_interface => $vp_ci);
    my $vvol_information  = NACL::C::VPserver::Vvol->list_information(command_interface => $vp_ci,
                                                                      vvol_uuids => $vvol_uuids);

A method that lists out all detailed Vvol information for a given a list of Vvol UUIDs.

Uses a SOAP VPserver APISet.

=over

=item Options

=over

=item C<< command_interface => $vp_ci >>

(Optional) If executed as an instance method.

(Required) If executed as a class method.

See L<NACL::C::VPserver|lib-NACL-C-VPserver-pm>

=item C<< vvol_uuids => \@vvol_uuids >>

(Required) An array reference of Vvol UUIDs to list information for.
The list of Vvol UUIDs can be obtained by utilizing the L<NACL::C::VPserver::Vvol|lib-NACL-C-VPserver-Vvol-pm/list> method.

=back

=item Returns

This method returns back an array reference of hash references. Each hash reference contains
detailed information related to each vvol uuid provided.

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub list_information {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->command_interface($opts{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(text => "Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Vvol::list_information()");
    }

    my $vvol_uuids = delete($opts{vvol_uuids}) //
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(text => "Mandatory parameter 'vvol_uuids' missing in call to NACL::C::VPserver::Vvol::list_information()");

    my @formatted_uuids;
    foreach my $vvol_uuid (@{$vvol_uuids}) {
        push(@formatted_uuids, {vvolUuids => $vvol_uuid});
    }

    my $response = $pkg->command_interface->call(command => "vvolListInformation",
                                                 params => \@formatted_uuids);

    $Log->exit() if $may_exit;
    return $response;
}

=head2 list_storage_information

    my $vp_ci = NACL::C::VPserver->new(vpserver => "my_vpserver");
    my $vvol_uuids = NACL::C::VPserver::Vvol->list(command_interface => $vp_ci);
    my $vvol_storage_information  = NACL::C::VPserver::Vvol->list_storage_information(command_interface => $vp_ci,
                                                                                      vvol_uuids => $vvol_uuids);

A method that lists out all detailed Vvol storage information for a given list of Vvol UUIDs.

Uses a SOAP VPserver APISet.

=over

=item Options

=over

=item C<< command_interface => $vp_ci >>

(Optional) If executed as an instance method.

(Required) If executed as a class method.

See L<NACL::C::VPserver|lib-NACL-C-VPserver-pm>

=item C<< vvol_uuids => \@vvol_uuids >>

(Required) An array reference of Vvol UUIDs to list storage information for.
The list of Vvol UUIDs can be obtained by utilizing the L<NACL::C::VPserver::Vvol|lib-NACL-C-VPserver-Vvol-pm/list> method.

=back

=item Returns

This method returns back an array reference of hash references. Each hash reference contains
detailed storage information related to each vvol uuid provided.

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub list_storage_information {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->command_interface($opts{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(text => "Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Vvol::list_storage_information()");
    }

    my $vvol_uuids = delete($opts{vvol_uuids}) //
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(text => "Mandatory parameter 'vvol_uuids' missing in call to NACL::C::VPserver::Vvol::list_storage_information()");

    my @formatted_uuids;
    foreach my $vvol_uuid (@{$vvol_uuids}) {
        push(@formatted_uuids, {vvolUuids => $vvol_uuid});
    }

    my $response = $pkg->command_interface->call(command => "vvolListStorageInformation",
                                                 params => \@formatted_uuids);

    $Log->exit() if $may_exit;
    return $response;
}

=head2 list_bind

    my $vp_ci = NACL::C::VPserver->new(vpserver => "my_vpserver");
    my $vvol_uuids = NACL::C::VPserver::Vvol->list(command_interface => $vp_ci);
    my $vvol_bind_information  = NACL::C::VPserver::Vvol->list_bind(command_interface => $vp_ci,
                                                                    vvol_uuids => $vvol_uuids);

A method that lists out all detailed Vvol bind information for a given a list of Vvol UUIDs.

Uses a SOAP VPserver APISet.

=over

=item Options

=over

=item C<< command_interface => $vp_ci >>

(Optional) If executed as an instance method.

(Required) If executed as a class method.

See L<NACL::C::VPserver|lib-NACL-C-VPserver-pm>

=item C<< vvol_uuids => \@vvol_uuids >>

(Required) An array reference of Vvol UUIDs to list bind information for.
The list of Vvol UUIDs can be obtained by utilizing the L<NACL::C::VPserver::Vvol|lib-NACL-C-VPserver-Vvol-pm/list> method.

=back

=item Returns

This method returns back an array reference of hash references. Each hash reference contains
detailed bind information related to each vvol uuid provided.

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub list_bind {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->command_interface($opts{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(text => "Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Vvol::list_bind()");
    }

    my $vvol_uuids = delete($opts{vvol_uuids}) //
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(text => "Mandatory parameter 'vvol_uuids' missing in call to NACL::C::VPserver::Vvol::list_bind()");

    my @formatted_uuids;
    foreach my $vvol_uuid (@{$vvol_uuids}) {
        push(@formatted_uuids, {vvolUuids => $vvol_uuid});
    }

    my $response = $pkg->command_interface->call(command => "vvolListBind",
                                                 params => \@formatted_uuids);

    $Log->exit() if $may_exit;
    return $response;
}

=head2 list_attribute_information

    my $vp_ci = NACL::C::VPserver->new(vpserver => "my_vpserver");
    my $vvol_uuids = NACL::C::VPserver::Vvol->list(command_interface => $vp_ci);
    foreach my $vvol_uuid (@{$vvol_uuids}) {
        my $vvol_attribute_info  = NACL::C::VPserver::Vvol->list_attribute_info(command_interface => $vp_ci,
                                                                                vvol_uuid => $vvol_uuid);
    }

A method that obtains the vvol attribute info for a particular vvol uuid.

Uses a SOAP VPserver APISet.

=over

=item Options

=over

=item C<< command_interface => $vp_ci >>

(Optional) If executed as an instance method.

(Required) If executed as a class method.

See L<NACL::C::VPserver|lib-NACL-C-VPserver-pm>

=item C<< vvol_uuid => $vvol_uuid >>

(Required) A string Vvol UUID to list attribute information for.
The list of Vvol UUIDs can be obtained by utilizing the L<NACL::C::VPserver::Vvol|lib-NACL-C-VPserver-Vvol-pm/list> method.

=back

=item Returns

This method returns back an array reference of hash references. Each hash reference contains
detailed attribute information related to the vvol uuid provided.

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub list_attribute_information {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->command_interface($opts{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(text => "Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Vvol::list_attribute_size()");
    }

    my $vvol_uuid = delete($opts{vvol_uuid}) //
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(text => "Mandatory parameter 'vvol_uuid' missing in call to NACL::C::VPserver::Vvol::list_attribute_size()");

    my $response = $pkg->command_interface->call(command => "vvolListAttributeSize",
                                                 params => [{vvolUuid => $vvol_uuid}]);

    $Log->exit() if $may_exit;
    return $response;
}

1;
