# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Vfiler Component Module
## @author madhav.shekar@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Vfiler

=head1 DESCRIPTION

C<NACL::C::Vfiler> is a derived class of
L<NACL::C::CommmandInterface::ONTAP|lib-NACL-C-CommandInterface-ONTAP-pm>.
It represents access to an ONTAP Node.  A related class is
L<NACL::CS::Vfiler|lib-NACL-CS-Vfiler-pm>, which represents
the past state of an ONTAP Node. A Vfiler can act as command interface
to other components.See
L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head1 ATTRIBUTES

=head2 vfiler

The name of the vfiler.

=cut

package NACL::C::Vfiler;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::CS::Vfiler;
use NATE::BaseException qw(:try);
use base 'NACL::C::CommandInterface::ONTAP';
use NACL::ComponentUtils qw (_optional_scalars);
use Params::Validate qw(SCALAR ARRAYREF BOOLEAN validate validate_with);
use NACL::C::Exceptions::Vfiler::AlreadyExists  ();
use NACL::C::Exceptions::Vfiler::DoesNotExist   ();
use NACL::C::Exceptions::Vfiler::AlreadyStopped ();

=head1 METHODS

=head2 new

my $vfiler = NACL::C::Vfiler->new([name => $name],
                              [vfiler => $vfiler],
                              [hostrec => $hostrec] );
Specifying any one of the vfiler, name, hostrec attributes will populate
the other two.
Returns a new Vfiler component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => [
        {   '-default_ctor' => sub { $_[0]->_construct_primary_key(); },
            '-store_cb' =>
                sub { $_[0]->_verify_hostrec_callback( $_[1], $_[2] ); }
        },
        'vfiler'
    ]
];

=head2 find

    my $Vfiler  = NACL::C::Vfiler->find(command_interface => $ci, ...);
    my @Vfilers = NACL::C::Vfiler->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.
You can also refer to user guide for more information on the options,
order of precedence for command line parameters (in case of find() is
invoked without specifying a command_interface) and examples.
http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/ComponentLayerUsersGuide/index.html#Obtaining_command_interface_objects

=head2 state

    my $Vfiler_state = $vfiler_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::Vfiler would return
a NACL::CS::Vfiler object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

# Allow choices that are both "Vfiler" and "7Mode", but prefer the
# "Vfiler" choices if available.

sub default_apiset_must {
    my ($self) = @_;
    return {
        %{ $self->SUPER::default_apiset_must() },
        set        => "Vfiler|7Mode",
        set_source => "command interface is a NACL::C::Vfiler",
    };
}

# Allow choices that are both "Vfiler" and "7Mode", but prefer the
# "Vfiler" choices if available.

sub default_apiset_should {
    return { set => "Vfiler", };
}

# First preference is to connect to the Vfiler and run commands
# directly on it.  If the user wants to run it through the node (i.e:
# "vfiler run"), they'd need to specify the shell_type to be 7Mode.

# XXX if we leave out a shell_type value, what happens?  If it
# defaults to Vfiler instead of 7Mode, we don't need this here.

sub default_apiset_could {
    return {
        shell_type => "Vfiler",
        category  => "Node",
        interface => "CLI",
        set       => "Vfiler"
    };
}

=head2 create

  my $vfiler = NACL::C::Vfiler->create(command_interface => $ci, vfiler => $vf_name);

(Class method) Request creation of a new vfiler, and return a new vfiler
component object that refers to it.

Uses a 7Mode CLI APISet.

Invokes 'vfiler create' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vfiler=>$vfiler_name >>

(Required) Name of vfiler to create.

Applicable for 7Mode CLI.

=item C<< "ip-addresses" => [$ip_addr1, $ip_addr2, ...] >>

IP addresses to assign to the created vfiler.

Applicable for 7Mode CLI.

=item C<< "storage-units" => [ $path1, $path2, ...] >>

Paths to the storage units to be assigned to the vfiler. Storage units include
volumes and qtrees.

Applicable for 7Mode CLI.

=item C<< "recreate-path" => $path >>

Path to the Snapmirrored vfiler configuration data source

Applicable for 7Mode CLI.

=item C<< ipspace => $ipspace >>


Applicable for 7Mode CLI.

=item C<< "bypass-setup" => $boolean >>

A boolean specifying whether to bypass setup or not.

Applicable for 7Mode CLI.

=item C<< "old-vfiler-name" >>

The name of the Snapmirrored vfiler which is being restored.

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::Vfiler::AlreadyExists>

This type of exception gets thrown if an attempt is made to create a
vfiler that already exists.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            "ip-addresses"  => { type => ARRAYREF, optional => 1 },
            "storage-units" => { type => ARRAYREF, optional => 1 },
            "bypass-setup"  => { type => BOOLEAN,  optional => 1 },
            _optional_scalars(qw(ipspace recreate-path old-vfiler-name)),

        },
    );

    my $obj;
    try {
        $obj = $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_create_7mode_cli",
                    interface => "CLI",
                    set       => "7Mode",
                },
            ],
        );
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::Vfiler::AlreadyExists->detect_convert_and_throw(
            exception => $exception );
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_7mode_cli {
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_cli_opts = [
        qw(vfiler bypass-setup ipspace storage-units
            recreate-path old-vfiler-name ip-addresses)
    ];
    my %cli_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cli_opts,
        target => \%cli_args,
    );
    %opts = %cli_args;

    if ( defined $opts{"ip-addresses"} ) {
        $opts{"ip-addresses"} = join( " ", @{ $opts{"ip-addresses"} } );
    }

    if ( defined $opts{"storage-units"} ) {
        $opts{"storage-units"} = join( " ", @{ $opts{"storage-units"} } );
    }

    $apiset->vfiler_create(%opts);

    return $pkg->new(
        vfiler            => $opts{vfiler},
        command_interface => $command_interface
    );
} ## end sub _create_7mode_cli

=head2 allow

  NACL::C::Vfiler->allow(command_interface => $ci, vfiler => $vf_name, ...);

                        or

  $vfiler_object->allow(...);

(Class or instance method) This method allows the use of the specified protocols
on the vfiler.

Uses a 7Mode CLI APISet.

Invokes 'vfiler allow' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vfiler=>$vfiler_name >>

(Required for class method, Not Applicable for instance method)
Name of vfiler.

Applicable for 7Mode CLI.

=item C<< protocols => [$proto1, $proto2, ...] >>

Protocols to allow on the vfiler.

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub allow {
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec =>
            { protocols => { type => ARRAYREF, optional => 1 }, },
    );

    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_allow_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );
} ## end sub allow

sub _allow_7mode_cli {
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %cli_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(vfiler protocols)],
        target => \%cli_args
    );
    %opts = %cli_args;

    if ( defined $opts{protocols} ) {
        $opts{protocols} = join( " ", @{ $opts{protocols} } );
    }

    $apiset->vfiler_allow(%opts);
} ## end sub _allow_7mode_cli

=head2 disallow

  NACL::C::Vfiler->disallow(command_interface => $ci, vfiler => $vf_name, ...);

                        or

  $vfiler_object->disallow(...);

(Class or instance method) This method disallows the use of the
specified protocols on the vfiler.

Uses a 7Mode CLI APISet.

Invokes 'vfiler disallow' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vfiler=>$vfiler_name >>

(Required for class method, Not Applicable for instance method)
Name of vfiler.

Applicable for 7Mode CLI.

=item C<< protocols => [$proto1, $proto2, ...] >>

Protocols to disallow on the vfiler.

Applicable for 7Mode CLI.
=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub disallow {
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec =>
            { protocols => { type => ARRAYREF, optional => 1 }, },
    );

    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_disallow_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );
} ## end sub disallow

sub _disallow_7mode_cli {
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %cli_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(vfiler protocols)],
        target => \%cli_args,
    );
    %opts = %cli_args;

    if ( defined $opts{protocols} ) {
        $opts{protocols} = join( " ", @{ $opts{protocols} } );
    }

    $apiset->vfiler_disallow(%opts);
} ## end sub _disallow_7mode_cli

=head2 rename

  NACL::C::Vfiler->rename(command_interface => $ci, vfiler => $vf_name, ...);

                        or

  $vfiler_object->rename(...);

(Class or instance method) This method renames the vfiler.

Uses a 7Mode CLI APISet.

Invokes 'vfiler rename' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vfiler=>$vfiler_name >>

(Required for class method, Not Applicable for instance method)
Name of vfiler.

Applicable for 7Mode CLI.

=item C<< "new-vfiler-name" => $new_vf_name >>

New vfiler name

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub rename {
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => { _optional_scalars(qw(new-vfiler-name)), },
    );

    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_rename_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );
} ## end sub rename

sub _rename_7mode_cli {
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %cli_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(vfiler new-vfiler-name)],
        target => \%cli_args,
    );
    %opts = %cli_args;

    $apiset->vfiler_rename(%opts);

    $pkg_or_obj->vfiler( $opts{"new-vfiler-name"} ) if ( ref $pkg_or_obj );
} ## end sub _rename_7mode_cli

=head2 start

  NACL::C::Vfiler->start(command_interface => $ci, vfiler => $vf_name, ...);

                        or

  $vfiler_object->start(...);

(Class or instance method) This method causes a previously stopped vfiler to
enter the running state.

Uses a 7Mode CLI APISet.

Invokes 'vfiler start' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vfiler=>$vfiler_name >>

(Required for class method, Not Applicable for instance method)
Name of vfiler.

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_common_validate_with( params => \@_, );

    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_start_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );
} ## end sub start

sub _start_7mode_cli {
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %cli_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(vfiler)],
        target => \%cli_args,
    );
    %opts = %cli_args;

    $apiset->vfiler_start(%opts);
} ## end sub _start_7mode_cli

=head2 stop

  NACL::C::Vfiler->stop(command_interface => $ci, vfiler => $vf_name, ...);

                        or

  $vfiler_object->stop(...);

(Class or instance method) This method causes a running vfiler to stop
receiving network traffic.

Uses a 7Mode CLI APISet.

Invokes 'vfiler stop' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vfiler=>$vfiler_name >>

(Required for class method, Not Applicable for instance method)
Name of vfiler.

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::Vfiler::AlreadyStopped>

This type of exception gets thrown if an attempt is stop a
vfiler that already stopped.

=item C<NACL::C::Exceptions::Vfiler::DoesNotExist>

This type of exception gets thrown if an attempt to delete a
vfiler that doen not exists.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop {
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_common_validate_with( params => \@_, );

    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_stop_7mode_cli",
                    interface => "CLI",
                    set       => "7Mode",
                },
            ],
        );
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::Vfiler::AlreadyStopped->detect_convert_and_throw(
            exception => $exception );
        NACL::C::Exceptions::Vfiler::DoesNotExist->detect_convert_and_throw(
            exception => $exception );
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    return $ret;
} ## end sub stop

sub _stop_7mode_cli {
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %cli_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(vfiler)],
        target => \%cli_args,
    );
    %opts = %cli_args;

    $apiset->vfiler_stop(%opts);
} ## end sub _stop_7mode_cli

=head2 destroy

  NACL::C::Vfiler->destroy(command_interface => $ci, vfiler => $vf_name, ...);

                        or

  $vfiler_object->destroy(...);

(Class or instance method) This method releases all resources of the vfiler
and destroys the vfiler (by removing all configuration its data).

Uses a 7Mode CLI APISet.

Invokes 'vfiler destroy' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vfiler=>$vfiler_name >>

(Required for class method, Not Applicable for instance method)
Name of vfiler.

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::Vfiler::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete a
non-existing vfiler.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub destroy {
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_common_validate_with( params => \@_, );

    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_destroy_7mode_cli",
                    interface => "CLI",
                    set       => "7Mode",
                },
            ],
        );
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::Vfiler::DoesNotExist->detect_convert_and_throw(
            exception => $exception );
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    return $ret;
} ## end sub destroy

sub _destroy_7mode_cli {
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %cli_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(vfiler)],
        target => \%cli_args,
    );
    %opts = %cli_args;

    # The "force" option ensures confirmation is not asked for
    $apiset->vfiler_destroy( %opts, force => 1 );
} ## end sub _destroy_7mode_cli

=head2 add

  NACL::C::Vfiler->add(command_interface => $ci, vfiler => $vf_name, ...);

                        or

  $vfiler_object->add(...);

(Class or instance method) This method adds the specified IP addresses and/or
storage units (specified by their paths) to the vfiler.

Uses a 7Mode CLI APISet.

Invokes 'vfiler add' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vfiler=>$vfiler_name >>

(Required for class method, Not Applicable for instance method)
Name of vfiler.

Applicable for 7Mode CLI.

=item C<< "ip-addresses" => [ $ip_addr1, $ip_addr2, ...] >>

The IP addresses to be added to the vfiler.

Applicable for 7Mode CLI.

=item C<< "storage-units" => [ $path1, $path2, ...] >>

The paths to the storage units to be added to the vfiler.

Applicable for 7Mode CLI.
=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add {
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            "ip-addresses"  => { type => ARRAYREF, optional => 1 },
            "storage-units" => { type => ARRAYREF, optional => 1 },
        },
    );

    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_add_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );
} ## end sub add

sub _add_7mode_cli {
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %cli_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(vfiler ip-addresses storage-units)],
        target => \%cli_args,
    );
    %opts = %cli_args;

    if ( defined $opts{"ip-addresses"} ) {
        $opts{"ip-addresses"} = join( " ", @{ $opts{"ip-addresses"} } );
    }

    if ( defined $opts{"storage-units"} ) {
        $opts{"storage-units"} = join( " ", @{ $opts{"storage-units"} } );
    }

    # The "force" option skips warning messages, if any
    $apiset->vfiler_add( %opts, force => 1 );
} ## end sub _add_7mode_cli

=head2 remove

  NACL::C::Vfiler->remove(command_interface => $ci, vfiler => $vf_name, ...);

                        or

  $vfiler_object->remove(...);

(Class or instance method) This method removes the named IP addresses and/or
storage units (specified by their paths) from the vfiler.

Uses a 7Mode CLI APISet.

Invokes 'vfiler remove' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vfiler=>$vfiler_name >>

(Required for class method, Not Applicable for instance method)
Name of vfiler.

Applicable for 7Mode CLI.

=item C<< "ip-addresses" => [ $ip_addr1, $ip_addr2, ...] >>

The IP addresses to be removed from the vfiler.

Applicable for 7Mode CLI.

=item C<< "storage-units" => [ $path1, $path2, ...] >>

The paths to the storage units to be removed from the vfiler.

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove {
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            "ip-addresses"  => { type => ARRAYREF, optional => 1 },
            "storage-units" => { type => ARRAYREF, optional => 1 },
        },
    );

    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_remove_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );
} ## end sub remove

sub _remove_7mode_cli {
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %cli_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(vfiler ip-addresses storage-units)],
        target => \%cli_args,
    );
    %opts = %cli_args;

    if ( defined $opts{"ip-addresses"} ) {
        $opts{"ip-addresses"} = join( " ", @{ $opts{"ip-addresses"} } );
    }

    if ( defined $opts{"storage-units"} ) {
        $opts{"storage-units"} = join( " ", @{ $opts{"storage-units"} } );
    }

    # The "force" option skips warning messages, if any
    $apiset->vfiler_remove( %opts, force => 1 );
} ## end sub _remove_7mode_cli

=head2 move

  NACL::C::Vfiler->move(command_interface => $ci, vfiler => $vf_name, ...);

                        or

  $vfiler_object->move(...);

(Class or instance method) This method moves the IP addresses and/or storage
units (as specified by their paths) from the vfiler to another vfiler.

Uses a 7Mode CLI APISet.

Invokes 'vfiler move' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vfiler=>$vfiler_name >>

(Required for class method, Not Applicable for instance method)
Name of vfiler.

Applicable for 7Mode CLI.

=item C<< "ip-addresses" => [ $ip_addr1, $ip_addr2, ...] >>

The IP addresses to be moved from the vfiler.

Applicable for 7Mode CLI.

=item C<< "storage-units" => [ $path1, $path2, ...] >>

The paths to the storage units to be moved from the vfiler.

Applicable for 7Mode CLI.

=item C<< "vfiler-to" => $vfiler_to >>

The vfiler to which to move the resources.

Applicable for 7Mode CLI.
=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub move {
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            "ip-addresses"  => { type => ARRAYREF, optional => 1 },
            "storage-units" => { type => ARRAYREF, optional => 1 },
            _optional_scalars(qw(vfiler-to))
        },
    );

    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_move_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );
} ## end sub move

sub _move_7mode_cli {
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %cli_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(vfiler ip-addresses storage-units vfiler-to)],
        target => \%cli_args,
    );
    %opts = %cli_args;

    if ( defined $opts{"ip-addresses"} ) {
        $opts{"ip-addresses"} = join( " ", @{ $opts{"ip-addresses"} } );
    }

    if ( defined $opts{"storage-units"} ) {
        $opts{"storage-units"} = join( " ", @{ $opts{"storage-units"} } );
    }

    # The "force" option skips warning messages, if any
    $apiset->vfiler_move( %opts, force => 1 );
} ## end sub _move_7mode_cli

=head2 limit

    # Set a new limit
    NACL::C::Vfiler->limit(command_interface => $ci, maxvfilers => $new_limit);
    
    # Get the current limits
    my $limit_obj = NACL::C::Vfiler->limit(command_interface => $ci);

(Class method) This method can be used to get/set the current vfiler limit
on the filer.

To set a new value for the max vfiler limit, the C<maxvfilers> argument
should be provided.

If it is not provided, then this method will get the existing limits.
In get context, a L<NACL::CS::VfilerLimit|lib-NACL-CS-VfilerLimit-pm>
object is returned.

This can be used as a class method only because it is a setting applicable
for the parent filer hosting the vfiler (not for individual vfilers).

Uses a 7Mode CLI APISet.

Invokes 'vfiler limit' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< maxvfilers=>$vnew_vfiler_limit >>

New vfiler limit for the hosting filer. If provided, then method will be
executed in set context.

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub limit {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        additional_spec     => { _optional_scalars(qw(maxvfilers)) },
        ignore_primary_keys => 1
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_limit_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub limit

sub _limit_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts         = $pkg->_validate_backend_opts(@args);
    my $apiset       = $opts{apiset};

    my (%cli_args, $ret);
    if (defined $opts{maxvfilers}) {
        # Set the new limit
        $apiset->vfiler_limit(maxvfilers => $opts{maxvfilers});
    } else {
        # Get the existing limits
        require NACL::CS::VfilerLimit;
        my $response = $apiset->vfiler_limit();
        my $hash     = $response->get_parsed_output()->[0];
        $ret = NACL::CS::VfilerLimit->new(
            command_interface => $opts{command_interface});
        $ret->_set_fields(row => $hash);
    }

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub _limit_7mode_cli

=head2 owner

  NACL::C::Vfiler->owner(command_interface => $ci, path => $path);

                        or

  $vfiler_object->owner(...);

(Class or instance method) This method display owner of a resource.

Uses a 7Mode CLI APISet.

Invokes 'vfiler owner' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< path=>$path >>

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub owner {
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params              => \@_,
        additional_spec     => { _optional_scalars(qw(path vfiler)), },
        ignore_primary_keys => 1,
    );
    return $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_owner_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );
} ## end sub owner

sub _owner_7mode_cli {
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %cli_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => [qw(path)],
        target => \%cli_args,
    );
    %opts = %cli_args;

    $apiset->vfiler_owner(%opts);
} ## end sub _owner_7mode_cli

=head1 METHODS (for component developers)

=head2 apiset

See L<NACL::C::CommandInterface::apiset|lib-NACL-C-CommandInterface-pm/apiset>
for a description of what the generic apiset method does.

We need to override apiset here to provide suitable values for shell_type
and context_definition, before calling the generic apiset defined in
NACL::C::CommandInterface.

=cut

sub apiset {
    my ( $self, %opts ) = @_;

    # Note that the specification in the 'choices' hash of call_on_apiset
    # would specify set to be "7Mode". But if we are using Vfiler as a
    # command interface, we are in "vfiler context". Meaning, the APISet
    # we'd need to instantiate it to be of set 'Vfiler'
    if ( ( defined $opts{set} ) && ( $opts{set} !~ /(Vfiler|7Mode)/ ) ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw( "Invalid set value '$opts{set}' "
                . "specified. "
                . "With a command_interface of type NACL::C::Vfiler, you can "
                . "only create Vfiler APISet object" );
    } ## end if ( ( defined $opts{set...}))

    # APISet expects the 'set' to be 'Vfiler' if the shell_type is Vfiler.
    # The component frontend methods don't list 'Vfiler' in the choices hash
    # Hence call_on_apiset cannot pick 'Vfiler' as the set value and so the
    # value selected by it would be '7Mode'. Hence we are
    # working around here by reassigning the set value to be 'Vfiler'.
    $opts{set} = "Vfiler";

    # If shell_type is not specified, then we will set the default
    # shell_type to be "Vfiler". Thus the default is to make a direct
    # connection to Vfiler to execute the commands
    $opts{shell_type} = $opts{shell_type} || "Vfiler";
    my $vfiler_hostrec = $self->hostrec();

    if ($opts{shell_type} ne 'Vfiler') {

        # With shell_type set to 7Mode, the thing issuing the commands is really
        # a node (which will be prepending a "vfiler run"), not a vfiler.
        # This will require some re-shuffling, now $self->hostrec() should not
        # refer to the vfiler anymore but should actually refer to the parent
        # filer. We'd also need to fill in the 'schema_hostobj'
        # (context_definition entry) to be the hostrec for the vfiler.

        # Note that we don't explicitly bother checking whether shell_type is
        # 7Mode. If shell_type=CMode is passed, the APISet code would
        # take care of throwing an exception, saving us from needing to
        # make that check here.

        my $ci = $self->command_interface();
        
        # PFILER = parent filer, i.e. the name of the node hosting the vfiler.
        my $pfiler_hostrec;
        if ($ci->isa('NACL::C::Vfiler')) {
            # The Vfiler object uses itself as command_interface. Our only
            # hope is if a PFILER value is set in the hostrec of the Vfiler.
            my $pfiler = $vfiler_hostrec->{PFILER};
            unless (defined $pfiler) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw( "You have specified the "
                        . "shell_type to be '$opts{shell_type}' for which a connection "
                        . "needs to be established to parent filer and the commands"
                        . " should be executed in the context of vfiler. But "
                        . "information about parent filer is missing in the PFILER "
                        . "attribute of the vfiler hostrec object" );
            } ## end unless ( defined $pfiler )
            $pfiler_hostrec = Tharn::host($pfiler);
        } else {
            # Uses a Node as its command_interface, so we can get the hostrec
            # from it.
            $pfiler_hostrec = $ci->hostrec();
        }

        $self->hostrec($pfiler_hostrec);
        my %context_definition;
        $context_definition{schema_hostobj} = $vfiler_hostrec;
        $opts{context_definition}           = \%context_definition;
    } ## end if ( $opts{shell_type}...)

    my $apiset;
    try {
        # Call CommandInterface's apiset()
        $apiset = $self->SUPER::apiset(%opts);
    } finally {
        # We need to set $self->hostrec() back to being the hostrec of the
        # vfiler.
        $self->hostrec($vfiler_hostrec);
    };

    return $apiset;
} ## end sub apiset

=head2 _primary_keys_validate_spec

See L<_primary_keys_validate_spec|lib-NACL-C-Component-pm/_primary_keys_validate_spec>.

=cut


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ( vfiler => { type => SCALAR } );
}

=head2 resource_record_method_name

See L<resource_record_method_name|lib-NACL-C-CommandInterface-pm/resource_record_method_name>.

=cut

sub resource_record_method_name {
    return "vfilers";
}

# For this command interface, what is the common prefix for the test parameters?
sub _ci_to_test_param_mapping {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;
    return 'VFILER';
}

sub _create_version_manager {
    $Log->enter() if $may_enter;
    my $self = $_[0];

    require NACL::APISet::Version;
    my $vm;
    if (!$self->command_interface()->isa('NACL::C::Vfiler')) {
        $Log->debug("vfiler instance is not associated with vfiler "
                . "command_interface");
        $vm = $self->command_interface()->get_version_manager();
    } else {
        $Log->debug("vfiler instance is associated with vfiler"
                . " command_interface");
        $vm = NACL::APISet::Version->new(
            category   => "Node",
            hostobj    => $self->hostrec(),
            shell_type => "Vfiler"
        );
    }
    $Log->exit() if $may_exit;
    return $vm;
}

sub construct_hostrec {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $self->_common_validate_spec_without_ci(),
        
        # Allow through hostrec fields
        allow_extra => 1
    );

    if ( $self->{hostrec} ) {

        # hostrec has already been set, nothing to do
        $Log->exit() if $may_exit;
        return;
    }
    
    my %common_opts;
    $self->_move_common_component_params(
        source => \%opts,
        target => \%common_opts
    );
    
    # NATE sets the default_password for hosttype to be an empty string.
    # The convention has long since changed to be netapp1!, so let's set to
    # netapp1!.
    $opts{default_password} ||= 'netapp1!';

    my $vfiler = $self->vfiler();
    my $host;
    
    # When we provide multiple arguments to host(), then they are considered
    # only if a tharnhost file does not exist. For example:
    #     host('myvsim', 'hosttype=filer', default_ip=$ip)
    # would use the values of hosttype and ip only if the tharnhost file
    # doesn't exist. This is what we want.
    # Code like this:
    #     host('myvsim', %opts)
    # would fail, which is why we turn it into an array, with each entry
    # being a string in the form key=value.
    my @hostrec_opts = map { "$_=$opts{$_}" } (keys %opts);
    try {
        $host = Tharn::host($vfiler, @hostrec_opts);
    }
    otherwise {
        my $config_error = $_[0];

        # Ah, no host file. If an IP address was provided by the caller,
        # let's use that. Else, dynamically determine what its IP is.
        my $address = delete $opts{default_ip};
        if (!defined $address) {
            my $no_hostrec_error = $self->_no_hostrec_error(
                name            => $vfiler,
                config_error    => $config_error,
                throw_exception => 0,
            );
            if ($self->is_command_interface_same_as_self()) {
                # Command interface uses itself as command interface
                # No way we can determine its IP
                $Log->exit() if $may_exit;
                NATE::BaseException->throw( "$no_hostrec_error\n"
                        . 'Also, a hostrec object could not be dynamically '
                        . 'constructed because this object uses itself '
                        . 'as command interface' );
            } else {
                my $ip_struct = NACL::CS::Vfiler->fetch(
                    %common_opts,
                    command_interface => $self->command_interface(),
                    filter            => {vfiler => $vfiler},
                    requested_fields  => ['ip_address_configuration'],
                )->ip_address_configuration();
                $address = $ip_struct->[0]{ip_address};
                if (!defined $address) {
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw("$no_hostrec_error\n"
                        . "The vfiler also doesn't appear to have an IP "
                        . 'address associated with it');
                }
            }
        }
        $host = Tharn::host($vfiler, 'hosttype=' . $self->_hosttype(),
            "default_ip=$address", @hostrec_opts);
    };
    
    $Log->exit() if $may_exit;
    return $host;
}

# This is the hosttype value to use if we're attempting to dynamically
# construct a hostrec object.
sub _hosttype { 'filer' }


1;
