# $Id$

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VfilerMigrate Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VfilerMigrate

=head1 DESCRIPTION

C<NACL::C::VfilerMigrate> is a derived class of
L<NACL::C::CommmandInterface::ONTAP|lib-NACL-C-CommandInterface-ONTAP-pm>.
It represents access to an ONTAP Node.  A related class is
L<NACL::CS::VfilerMigrate|lib-NACL-CS-VfilerMigrate-pm>, which represents
the past state of an ONTAP Node. A Vfiler can act as command interface
to other components.See
L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

package NACL::C::VfilerMigrate;

use strict;
use warnings;
use NACL::CS::VfilerMigrate;
use base 'NACL::C::Component::ONTAP';
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use Params::Validate qw(SCALAR ARRAYREF BOOLEAN validate validate_with);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 migrate

  NACL::C::VfilerMigrate->migrate(command_interface => $ci,
                                      'remote-login-name' => "root", ...);

(Class method) This method migrates a specified remote vfiler
from the specified remote filer to local filer.
This can be used as a class method only because it is a setting applicable
for the remote filer hosting the vfiler (not for individual vfilers).

Uses a 7Mode CLI APISet.

Invokes 'vfiler migrate' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< remote-login-name=>$login_name >>

(Mandatory)

Applicable for 7Mode CLI.

=item C<< remote-login-password=>$login_password >>

(Mandatory)

Applicable for 7Mode CLI.

=item C<< remote-vfiler=>$remote_vfiler_name >>

(Mandatory)

Applicable for 7Mode CLI.

=item C<< remote-filer=>$remote_filer >>

(Mandatory)

Applicable for 7Mode CLI.

=item C<< method=>$string >>

(Optional)

Applicable for 7Mode CLI.

=item C<< secure=>$string >>

(Optional)

Applicable for 7Mode CLI.

=item C<< interface-details=>[$interface1,$interface2] >>

(Optional) syntax to be specified 'interfacename:ipaddr:netmask'.

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


sub migrate {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    $Log->debug( "Opts to 'migrate' front end are:\n" . Dumper( \@_ ) );

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            "remote-login-name"     => { type => SCALAR },
            "remote-login-password" => { type => SCALAR },
            "interface-details"     => { type => ARRAYREF, optional => 1 },
            "remote-vfiler"         => { type => SCALAR },
            "remote-filer"          => { type => SCALAR },
            _optional_scalars( qw( secure method ) ),
        },
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_migrate_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub migrate

sub _migrate_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw(remote-login-name remote-login-password interface-details remote-vfiler
            remote-filer secure method)
    ];
    my %cli_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%cli_args,
    );
    if ( $cli_args{'interface-details'} ) {
        $cli_args{'interface-details'} = join ",",
            @{ $cli_args{'interface-details'} };
    }

    $apiset->vfiler_migrate(%cli_args);

    $Log->exit() if $may_exit;
} ## end sub _migrate_7mode_cli

=head2 start

  NACL::C::VfilerMigrate->start(command_interface => $ci,
                                    'remote-login-name' => "root", ...);

(Class method) This method start a migration of a specified remote vfiler
from the specified remote filer to local filer.
This can be used as a class method only because it is a setting applicable
for the remote filer hosting the vfiler (not for individual vfilers).

Uses a 7Mode CLI APISet.

Invokes 'vfiler migrate start' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< remote-login-name=>$login_name >>

(Mandatory)

Applicable for 7Mode CLI.

=item C<< remote-login-password=>$login_password >>

(Mandatory)

Applicable for 7Mode CLI.

=item C<< remote-vfiler=>$remote_vfiler_name >>

(Mandatory)

Applicable for 7Mode CLI.

=item C<< remote-filer=>$remote_filer >>

(Mandatory)

Applicable for 7Mode CLI.

=item C<< secure=>$string >>

(Optional)

Applicable for 7Mode CLI.

=item C<< interface-details=>[$interface1,$interface2] >>

(Optional) syntax to be specified 'interfacename:ipaddr:netmask'.

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


sub start {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    $Log->debug( "Opts to 'start' front end are:\n" . Dumper( \@_ ) );

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            "remote-login-name"     => { type => SCALAR },
            "remote-login-password" => { type => SCALAR },
            "interface-details"     => { type => ARRAYREF, optional => 1 },
            "remote-vfiler"         => { type => SCALAR },
            "remote-filer"          => { type => SCALAR },
            _optional_scalars( qw( secure ) ),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_start_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub start

sub _start_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw(remote-login-name remote-login-password remote-vfiler remote-filer interface-details
            secure)
    ];
    my %cli_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%cli_args,
    );
    if ( $cli_args{'interface-details'} ) {
        $cli_args{'interface-details'} = join ",",
            @{ $cli_args{'interface-details'} };
    }

    $apiset->vfiler_migrate_start(%cli_args);
    $Log->exit() if $may_exit;
} ## end sub _start_7mode_cli

=head2 cancel

  NACL::C::VfilerMigrate->cancel(command_interface => $ci,
                                   'remote-vfiler' => "vf1", ...);

(Class method) This method cancels migration of a specified remote vfiler
from the specified remote filer to local filer.
This can be used as a class method only because it is a setting applicable
for the remote filer hosting the vfiler (not for individual vfilers).

Uses a 7Mode CLI APISet.

Invokes 'vfiler migrate cancel' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< remote-vfiler=>$remote_vfiler_name >>

(Mandatory)

Applicable for 7Mode CLI.

=item C<< remote-filer=>$remote_filer >>

(Mandatory)

Applicable for 7Mode CLI.

=item C<< secure=>$string >>

(Optional)

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


sub cancel {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    $Log->debug( "Opts to 'cancel' front end are:\n" . Dumper( \@_ ) );
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            "remote-vfiler" => { type => SCALAR },
            "remote-filer"  => { type => SCALAR },
            _optional_scalars( qw( secure ) ),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_cancel_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub cancel

sub _cancel_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [qw(remote-vfiler remote-filer secure)];

    my %cli_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%cli_args,
    );

    $apiset->vfiler_migrate_cancel(%cli_args);
    $Log->exit() if $may_exit;
} ## end sub _cancel_7mode_cli

=head2 complete

  NACL::C::VfilerMigrate->complete(command_interface => $ci,
                                   'remote-vfiler' => "vf1", ...);

(Class method) This method completes migration of specified remote vfiler
from the specified remote filer to local filer.
This can be used as a class method only because it is a setting applicable
for the remote filer hosting the vfiler (not for individual vfilers).

Uses a 7Mode CLI APISet.

Invokes 'vfiler migrate complete' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< remote-vfiler=>$remote_vfiler_name >>

(Mandatory)

Applicable for 7Mode CLI.

=item C<< remote-filer=>$remote_filer >>

(Mandatory)

Applicable for 7Mode CLI.

=item C<< secure=>$string >>

(Optional)

Applicable for 7Mode CLI.

=item C<< remote-login-name=>$login_name >>

(Mandatory)

Applicable for 7Mode CLI.

=item C<< remote-login-password=>$login_password >>

(Mandatory)

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


sub complete {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    $Log->debug( "Opts to 'complete' front end are:\n" . Dumper( \@_ ) );
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            "remote-vfiler"         => { type => SCALAR },
            "remote-filer"          => { type => SCALAR },
            "remote-login-name"     => { type => SCALAR },
            "remote-login-password" => { type => SCALAR },
            _optional_scalars( qw( secure ) ),
        },
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_complete_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub complete

sub _complete_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options
        = [
        qw(remote-vfiler remote-filer secure remote-login-name remote-login-password )
        ];

    my %cli_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%cli_args,
    );

    $apiset->vfiler_migrate_complete(%cli_args);
    $Log->exit() if $may_exit;
} ## end sub _complete_7mode_cli

=head2 status

  NACL::C::VfilerMigrate->status(command_interface => $ci,
                 'remote-vfiler' => "vf1", ...);

(Class method) This method status returns NACL::CS::VfilerDr state objects of
a specified remote vfiler from the specified remote filer for disaster recovery

Uses a 7Mode CLI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< remote-vfiler=>$vfiler_name >>

(Mandatory)

Applicable for 7Mode CLI.

=item C<< remote-filer=>$filer_name >>

(Mandatory)

Applicable for 7Mode CLI.

=item C<< filter=>{status => "mirrored"} >>

(Optional) HASHREF.

Applicable for 7Mode CLI.

=item C<< requested_fileds=>["status"] >>

(Optional) ARRAYREF.

Applicable for 7Mode CLI.

=back

=back

=cut

sub status {
    $Log->enter() if $may_enter;
    my $pkg    = shift;
    my @states = NACL::CS::VfilerMigrate->fetch(@_);
    $Log->exit() if $may_exit;
    return wantarray ? @states : $states[0];
} ## end sub status


__PACKAGE__->_load_event_class();

1;
