#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeCopy Component Module
## @author Akshay Mohit(makshay@netapp.com), dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VolumeCopy

=head1 DESCRIPTION

C<NACL::C::VolumeCopy> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It models the volume copying
operation. There is currently no support for find and state, because CMode
does not currently provide any mechanism of seeing the state of a volume
copy operation.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

The vserver on which the source volume is present.

=head2 destination_volume

The destination volume to which the volume is being copied.

=head2 operation_no

(7Mode Only) Operation no of the volume copy operation.

=cut

package NACL::C::VolumeCopy;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use NACL::C::Job;
use Params::Validate qw(validate validate_with SCALAR SCALARREF UNDEF OBJECT);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::Job;

use NACL::CS::VolumeCopy;
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument ();
use NACL::C::Exceptions::VolumeCopy::RestrictBeforeCopying;
use NACL::C::Exceptions::VolumeCopy::DestinationVolumeDoesNotExist;
use NACL::C::Exceptions::VolumeCopy::DestinationVolumeIsTooSmall;

=head1 METHODS

=head2 new

  my $VolumeCopy= NACL::C::VolumeCopy->new(
       command_interface  => $ci,
       vserver            => $vserver,
       destination_volume => $dest_vol,
  );

Return a new VolumeCopy component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'destination_volume',
    scalar => 'operation_number',
    scalar => 'vserver',
];

=head2 start

    my $VolumeCopy_object = NACL::C::VolumeCopy->start(
         command_interface    => $ci,
         'destination-volume' => $dest_vol,
         ...
    );

(Class method)
This method is used to start making a copy of a volume.

Uses a  Cmode CLI/ZAPI, 7Mode CLI APISet.

Invokes 'volume-copy-start' API for CMode ZAPI.

Invokes 'vol copy start' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required)
The vserver on which the source volume resides.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: vserver

=item C<< "volume" => $string >>

(Required)
The source volume which is to be copied.

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: source-volume

7M CLI : source

=item C<< "destination-volume" => $string >>

(Required)
The destination volume to which the volume is to be copied.

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: destination-volume

7M CLI : destination

=item C<< "destination-aggregate" => $string >>

The aggregate on which the destination volume is present

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: destination-aggregate

=item C<< "destination-stripe-format" => $string >>

(Optional, Accessible only in test privilege)
Destination Stripe Format
Possible Values: data; full

=item C<< foreground => $string >>

(Optional)
Run in foreground?
Possible values: true; false

=item C<< snapshot-name => $string >>

name of the snapshot.

Applicable for 7Mode CLI.

Maps to:

7M CLI : -s <value> (Programmatic name: snapshot name)

=item C<< all-snapshots => $string >>

copies all snapshots.

Applicable for 7Mode CLI.

Maps to:

7M CLI : -S <value> (Programmatic name: all snapshots)

=item C<< source-filer => $string >>

name of the source-filer.

Applicable for 7Mode CLI.

Maps to:

7M CLI : source filer

=item C<< destination-filer => $string >>

name of the destination-filer.

Applicable for 7Mode CLI.

Maps to:

7M CLI : destination filer

=item C<< manual-cutover => $string >>

Applicable for 7Mode CLI.

Maps to:

7M CLI : -M <value> (Programmatic name: manual cutover)

=item C<< "method-timeout"=> $scalar >>

(Optional) The new timeout value.

=item C<< job_component => \$scalar >>

(Optional)
This is a reference to a variable, which gets filled in with a job component
if the operation was performed through a background job. We can then wait
on the completion of the job by doing
  $job->watch_progress('method-timeout' => $time_it_might_take_in_seconds);

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"volume copy start" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VolumeCopy::DestinationVolumeDoesNotExist>

This typ of exception is thrown when a volume operation is performed
from a source volume to a non-existing volume.

=item C<NACL::C::Exceptions::VolumeCopy::RestrictBeforeCopying>

This type of exception is thrown when a volume copy is performed from a source
volume to a unrestricted volume.

=item C<NACL::C::Exceptions::VolumeCopy::DestinationVolumeIsTooSmall>

This type of exception is thrown when a volume copy is performed when destination volume 
is smaller than source volume.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume vserver destination-volume destination-stripe-format
                    foreground destination-aggregate snapshot-name
                    all-snapshots manual-cutover source-filer destination-filer
                    )
            ),
            job_component => {type => SCALARREF, optional => 1},
        },
        api => 'volume_copy_start',
    );
    my $ret;
    try {
        $ret = $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_start_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_start_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
                {   method    => '_start_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VolumeCopy::start',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VolumeCopy::DestinationVolumeDoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        NACL::C::Exceptions::VolumeCopy::RestrictBeforeCopying
            ->detect_convert_and_throw(exception => $exception);
        NACL::C::Exceptions::VolumeCopy::DestinationVolumeIsTooSmall
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub start

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_cli(%opts, api => 'volume_copy_start');

    my $obj = $pkg->new(
        command_interface  => $opts{command_interface},
        destination_volume => $opts{'destination-volume'},
        vserver            => $opts{'vserver'},
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub _start_cmode_cli

sub _start_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $allowed_cmode_options = [
        qw( destination-volume vserver
            destination-aggregate )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        map    => {volume => 'source-volume'},
        target => \%cmode_args,
    );

    my $response = $apiset->volume_copy_start(%cmode_args);
    $pkg->_populate_job_components(
        command_interface => $command_interface,
        job_component     => $opts{job_component},
        job_components    => $opts{job_components},
        response          => $response
    );

    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface  => $command_interface,
        destination_volume => $opts{'destination-volume'},
        vserver            => $opts{'vserver'},
    );
} ## end sub _start_cmode_zapi

sub _start_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_7mode_options = [
        qw( all-snapshots snapshot-name manual-cutover
            source-filer destination-filer )
    ];

    my %mode7_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => {
            volume               => 'source',
            'destination-volume' => 'destination',
        },
        target => \%mode7_args,
    );

    if ($opts{'source-filer'}) {
        $mode7_args{source}
            = $opts{'source-filer'} . ":" . $mode7_args{source};
    }

    if ($opts{'destination-filer'}) {
        $mode7_args{destination}
            = $opts{'destination-filer'} . ":" . $mode7_args{destination};
    }

    my $response = $apiset->vol_copy_start(%mode7_args);
    my $output   = $response->get_parsed_output();
    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface  => $command_interface,
        destination_volume => $opts{'destination-volume'},
        'operation_number' => $output->[0]->{'operation_no'}
    );
} ## end sub _start_7mode_cli

=head2 abort

     NACL::C::VolumeCopy->abort(
                      command_interface    => $ci,
                      'operation-number' => $dest_vol,
                      ...);

 or

     $vol_copy_obj->abort();

(Class or Instance method)
This method is used to abort a volume copy operation.

Uses a 7Mode CLI APISet.

Invokes 'vol copy abort' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< operation-number => $string >>

(Optional)
The operation no of the volume copied.

Applicable for 7Mode CLI.

=item C<< all => $string >>

(Optional)
Aborts all vol copy operations..

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub abort {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {'all' => {type => SCALAR, optional => 1},},
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_abort_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },

        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub abort

sub _abort_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my $obj;
    my @state_objs;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_7mode_options = [qw( all operation-number)];

    my %mode7_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%mode7_args,
    );

    $apiset->vol_copy_abort(%mode7_args);

    $Log->exit() if $may_exit;

} ## end sub _abort_7mode_cli

=head2 throttle

      NACL::C::VolumeCopy->throttle(
                command_interface    => $ci,
                'throttle-value'     => $val,
                 ...);
      or

      $vol_copy_obj->throttle();

(Class or Instance method)
This method is used to throttle a volume copy operation.

Uses a 7Mode CLI APISet.

Invokes 'vol copy throttle' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< operation-number => $string >>

(Mandatory)
The operation no of the volume copied.

Applicable for 7Mode CLI.

=item C<< throttle-value => $string >>

(Mandatory)

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub throttle {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {'throttle-value' => {type => SCALAR},},
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_throttle_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub throttle

sub _throttle_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my $obj;
    my @state_objs;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_7mode_options = [qw( throttle-value operation-number)];

    my %mode7_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%mode7_args,
    );

    $apiset->vol_copy_throttle(%mode7_args);

    $Log->exit() if $may_exit;

} ## end sub _throttle_7mode_cli

# The reasoning for only 'destination-volume' being a primary key and
# not the source volume is that at any point of time a source volume
# could be getting copied to a number of destination volumes, but we cannot
# have multiple source volumes be copied into the same destination volume
# at the same time. So the destination-volume will alone be able to
# uniquely identify the volume copy operation

__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate(
        @args,
        {   command_interface => {
                type     => OBJECT,
                isa      => 'NACL::C::CommandInterface::ONTAP',
                optional => 1
            }
        }
    );
    my $command_interface = $opts{command_interface};

    my %default_spec = (
        'destination-volume' => {type => SCALAR},
        'operation-number'   => {type => SCALAR | UNDEF, optional => 1},
        'vserver'            => {type => SCALAR, optional => 1},
    );

    if (defined $command_interface) {
        if ($command_interface->is_cmode()) {
            $Log->exit() if $may_exit;
            my %vol_attr = (
                'destination-volume' => {type => SCALAR},
                'vserver'            => {type => SCALAR},
            );
            return %vol_attr;
        } else {
            $Log->exit() if $may_exit;
            return ('operation-number' => {type => SCALAR},);
        }
    } else {
        $Log->exit() if $may_exit;
        return %default_spec;
    }
} ## end sub _primary_keys_validate_spec
1;
