#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeDirectory Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VolumeDirectory

=head1 DESCRIPTION

C<NACL::C::VolumeDirectory> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VolumeDirectory.

An important point to note is that the related CS file is
L<NACL::CS::VolumeFile|lib-NACL-CS-VolumeFile-pm> (this is because
ONTAP treats files and directories similarly)

=cut

package NACL::C::VolumeDirectory;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use Params::Validate qw(validate_with SCALAR HASHREF OBJECT );
use NACL::ComponentUtils qw(_optional_scalars);

use NACL::C::Exceptions::VolumeDirectory::AlreadyExists qw(:try);
use NACL::C::Exceptions::VolumeDirectory::DoesNotExist ();
use NACL::C::Exceptions::VolumeDirectory::NotEmpty     ();
use NACL::Exceptions::InvalidChoice                    ();

use NACL::C::VolumeFile;
use NACL::CS::VolumeListFilesInDirectory;

use NACL::C::_Mixins::VolumeFileDirectory qw(:all);

use base 'NACL::C::Component::ONTAP';

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 path

The path to the directory

=head2 vserver

(Required for CMode, Not Applicable for 7Mode) The name of the vserver
in which the volume hosting the file is present.

=head1 The concept of "name-ordinal"

See L<NACL::C::VolumeFile|lib-NACL-C-VolumeFile-pm>.

=head1 METHODS

=head2 new

    my $dir_obj = NACL::C::VolumeDirectory->new(
        command_interface => $ci,
        path              => $dir_path,
        vserver           => $vserver_name  # Required for CMode
    );
    
Creates a new directory object. Note that this does not create a new directory
on the filer, it only creates a Perl object.

=cut

use Class::MethodMaker [
    scalar => 'path',
    scalar => 'vserver'
];

=head2 dblade_path

(See the concept of "name-ordinal" first)

    my $dblade_dir_path = $dir_obj->dblade_path();
    
(Instance method)
Returns the dblade path of the directory.

(For 7Mode since there isn't a dblade, so this method simply returns the
value of the C<path> attribute)

The only options accepted by this method are the common options (apiset_must,
apiset_should, method-timeout).

=cut

# Implementation present in the mix-in

=head2 get_local_node_name

    my $node_name = $dir_obj->get_local_node_name();

Returns the name of the node hosting the volume directory.

=head2 get_local_node_obj

    my $node_obj = $die_obj->get_local_node_obj();

Similar to C<get_local_node_name>, but returns a C<NACL::STask::Node>
object for the node hosting the volume directory.

=head2 create

    my $dir_obj = NACL::C::VolumeDirectory->create(
        command_interface => $ci,
        path              => $dir_path,
        vserver           => $vserver_name,  # Required for CMode
        %other_opts
    );
    
(Class method)

Create a new directory and returns a Perl object representing it.

Uses a CMode ZAPI, 7Mode ZAPI, Nodescope CLI or 7Mode CLI APISet.

Defaults to using ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< path => $path >>

(Required)
The path to the directory.

Applicable for CMode and 7Mode ZAPI.

=item C<< vserver => $vserver >>

(Required in CMode, ignored in 7Mode)
The vserver through which the volume hosting the directory is accessible.

Applicable for CMode ZAPI.

=item C<< perm => $perm >>

(Defaults to "0755" if not provided)
Permission of the directory to be created. It's similar to Unix style
permission bits: 0755 gives read/write/execute permissions to owner and
read/execute to group and other users. It consists of 4 octal digits derived
by adding up bits 4, 2 and 1. Omitted digits are assumed to be zeros. First
digit selects the set user ID(4), set group ID (2) and sticky (1) attributes.
The second digit selects permission for the owner of the file: read (4),
write (2) and execute (1); the third selects permissions for other users in
the same group; the fourth for other users not in the group.

Applicable for CMode and 7Mode ZAPI.

=item C<< 'nodescope-node-name' => $node_name >>

(Applicable only for Nodescope-CLI)
Explicitly specify the node on which to run the Nodescope command (the APISet
defaults to running on node "local")

=item C<< determine_appropriate_nodescope_node => 0 | 1 >>

(Optional, defaults to 0, applicable only for Nodescope-CLI)
If set to 1, the method determines which node the volume hosting the directory
is present on and runs the Nodescope CLI command on that particular node.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VolumeDirectory::AlreadyExists>

This type of exception gets thrown if an attempt is made to create a
directory that already exists.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(qw(perm)),
            $pkg->_nodescope_options_validate_spec(),
        },
    );

    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_create_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode|7Mode',
                },
                {   method    => '_create_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode|Nodescope',
                    check     => '_create_check',
                }
            ],
            frontend => 'NACL::C::VolumeDirectory::create'
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];

        NACL::C::Exceptions::VolumeDirectory::AlreadyExists
            ->detect_convert_and_throw(exception => $exception);

        $Log->exit() if $may_exit;
        $exception->throw();
    };

    my %new_args = (
        command_interface => $opts{command_interface},
        path              => $opts{path}
    );
    $pkg->_process_options(options => \%new_args);
    if ($opts{command_interface}->is_cmode()) {
        $new_args{vserver} = $opts{vserver};
    }
    $new_args{path} = $pkg->_remove_name_ordinal(path => $new_args{path});
    my $obj = $pkg->new(%new_args);
    
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args)      = @_;
    my %opts              = $pkg->_validate_backend_opts(@args);
    my $apiset            = $opts{apiset};
    my $command_interface = $opts{command_interface};
    
    my @allowed_options = ('path');
    $pkg->_nodescope_helper(
        options         => \%opts,
        allowed_options => \@allowed_options
    );
    
    my %api_args;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%api_args,
        copy   => \@allowed_options,
    );
    
    $apiset->mkdir(%api_args);
    $Log->exit() if $may_exit;
}

sub _create_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args)      = @_;
    my %opts              = $pkg->_validate_backend_opts(@args);
    my $apiset            = $opts{apiset};
    my $command_interface = $opts{command_interface};
    
    # CLI defaults it to 755
    if (!defined $opts{perm}) {
        $opts{perm} = '0755';
    }

    my %api_args;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%api_args,
        copy   => [qw(path perm)]
    );

    if ($command_interface->is_cmode()) {
        $pkg->_handle_zapi_vserver_context(
            api_opts          => \%api_args,
            vserver           => $opts{vserver},
            command_interface => $command_interface,
            frontend_method   => 'create'
        );
    }

    $apiset->file_create_directory(%api_args);

    $Log->exit() if $may_exit;
}

sub _create_check {
    $Log->enter() if $may_enter;
    my ($pkg, %opts ) = @_;
    
    if (exists $opts{perm}) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw('Cannot use CLI because '
            . "the argument 'perm' is supported only in ZAPI");
    }
    
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::VolumeDirectory->delete(
        command_interface => $ci,
        path              => $dir_path,
        vserver           => $vserver_name
    );
    
                   or
                   
    $dir_obj->delete();
    
(Class or instance method)

Delete a directory. This requires the directory to be empty before deleting,
else a C<NACL::C::Exceptions::VolumeDirectory::NotEmpty> exception is thrown.

Uses a CMode ZAPI, 7Mode ZAPI, Nodescope CLI or 7Mode CLI APISet.

Defaults to using ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< path => $path >>

(Required)
The path to the directory.

Applicable for CMode and 7Mode ZAPI.

=item C<< vserver => $vserver >>

(Required in CMode, ignored in 7Mode)
The vserver through which the volume hosting the directory is accessible.

Applicable for CMode ZAPI.

=item C<< 'nodescope-node-name' => $node_name >>

(Applicable only for Nodescope-CLI)
Explicitly specify the node on which to run the Nodescope command (the APISet
defaults to running on node "local")

=item C<< determine_appropriate_nodescope_node => 0 | 1 >>

(Optional, defaults to 0, applicable only for Nodescope-CLI)
If set to 1, the method determines which node the volume hosting the directory
is present on and runs the Nodescope CLI command on that particular node.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VolumeDirectory::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete a
directory that does not exist.

=item C<NACL::C::Exceptions::VolumeDirectory::NotEmpty>

This type of exception gets thrown if an attempt is made to delete a
directory which contains files and/or directories.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => { $pkg_or_obj->_nodescope_options_validate_spec() }
    );

    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                 {   method    => '_delete_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode|7Mode'
                },
                {   method    => '_delete_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode|Nodescope',
                }
            ],
            frontend => 'NACL::C::VolumeDirectory::delete'
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];

        foreach my $type (qw(DoesNotExist NotEmpty)) {
            my $full_type = "NACL::C::Exceptions::VolumeDirectory::$type";
            $full_type->detect_convert_and_throw(exception => $exception);
        }

        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
}

sub _delete_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts                = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset              = $opts{apiset};
    my $command_interface   = $opts{command_interface};
    
    my @allowed_options = ('path');
    $pkg_or_obj->_nodescope_helper(
        options         => \%opts,
        allowed_options => \@allowed_options
    );
    
    my %api_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%api_args,
        copy   => \@allowed_options,
    );
    
    $apiset->rmdir(%api_args);
    $Log->exit() if $may_exit;
}

sub _delete_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my %api_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%api_args,
        copy   => [qw(path)]
    );

    if ($opts{command_interface}->is_cmode()) {
        $pkg_or_obj->_handle_zapi_vserver_context(
            api_opts          => \%api_args,
            vserver           => $opts{vserver},
            command_interface => $opts{command_interface},
            frontend_method   => 'delete'
        );
    }

    $apiset->file_delete_directory(%api_args);

    $Log->exit() if $may_exit;
}

# Implementation for the following find/state and copy_to_logdir
# methods are from NACL::C::_Mixins::VolumeFileDirectory

=head3 find_files

    # List context
    my @file_objs = $dir_obj->find_files();
    
    # Scalar context
    my $file_obj = $dir_obj->find_files();
    
    # Accepts the same options as normal "find" calls
    my $file_obj = $dir_obj->find_files(allow_empty => 1);
    
    my $file_obj = $dir_obj->find_files(filter => { 'file-size' => '>1MB' });
    
Finds files present in the directory and returns objects of type
L<NACL::C::VolumeFile|lib-NACL-C-VolumeFile-pm>. Accepts arguments accepted
by "find".

=head3 find_directories

    # List context
    my @dir_objs = $dir_obj->find_directories();
    
    # Scalar context
    my $dir_obj = $dir_obj->find_directories();
    
    # Accepts the same options as normal "find" calls
    my $dir_obj = $dir_obj->find_directories(allow_empty => 1);
    
    my $dir_obj = $dir_obj->find_directories(filter => { 'owner-id' => $id });
    
Finds directories present in the directory and returns objects of type
L<NACL::C::VolumeDirectory|lib-NACL-C-VolumeDirectory-pm>. Accepts arguments accepted
by "find".

=head3 find_files_and_directories

    # List context
    my @objs = $dir_obj->find_files_and_directories();
    
    # Scalar context
    my $obj = $dir_obj->find_files_and_directories();
    
    # Accepts the same options as normal "find" calls
    my $obj = $dir_obj->find_files_and_directories(allow_empty => 1);
    
    my $obj = $dir_obj->find_files_and_directories(filter => { 'acl-type' => $acl });
    
Finds files and directories present in the directory and returns objects of type
L<NACL::C::VolumeFile|lib-NACL-C-VolumeFile-pm> for files found and objects
of type 
L<NACL::C::VolumeDirectory|lib-NACL-C-VolumeDirectory-pm> for directories found.
Accepts arguments accepted by "find".

=head3 state_files

    # List context
    my @files_cs = $dir_obj->state_files();
    
    # Scalar context
    my $file_cs = $dir_obj->state_files();
    
    # Accepts arguments accepted by "fetch"
    my @files_cs = $dir_obj->state_files(
        requested_fields => [ qw(acl-type) ],
        filter           => { 'file-size' => '>1MB' },
        allow_empty      => 1
    );
    
Fetches the state of contained files and returns objects of type
L<NACL::CS::VolumeListFilesInDirectory|lib-NACL-CS-VolumeListFilesInDirectory-pm>.
Accepts the arguments accepted by the "fetch" method.

=head3 state_directories

    # List context
    my @dirs_cs = $dir_obj->state_directories();
    
    # Scalar context
    my $dir_cs = $dir_obj->state_directories();
    
    # Accepts arguments accepted by "fetch"
    my @dirs_cs = $dir_obj->state_directories(
        requested_fields => [ qw(acl-type) ],
        filter           => { 'inode-number' => num },
        allow_empty      => 1
    );
    
Fetches the state of contained directories and returns objects of type
L<NACL::CS::VolumeListFilesInDirectory|lib-NACL-CS-VolumeListFilesInDirectory-pm>.
Accepts the arguments accepted by the "fetch" method.

=head3 state_files_and_directories

    # List context
    my @cs = $dir_obj->state_files_and_directories();
    
    # Scalar context
    my $cs = $dir_obj->state_files_and_directories();
    
    # Accepts arguments accepted by "fetch"
    my @cs = $dir_obj->state_files_and_directories(
        requested_fields => [ qw(acl-type) ],
        filter           => { 'inode-number' => num },
        allow_empty      => 1
    );
    
Fetches the state of contained files and directories and returns objects of type
L<NACL::CS::VolumeListFilesInDirectory|lib-NACL-CS-VolumeListFilesInDirectory-pm>.
Accepts the arguments accepted by the "fetch" method.


=head2 copy_to_logdir

See documentation in L<< NACL::C::_Mixins::VolumeFileDirectory->copy_to_logdir|lib-NACL-C-_Mixins-VolumeFileDirectory-pm/copy_to_logdir >>

=head2 systemshell_path

See documentation in L<< NACL::C::_Mixins::VolumeFileDirectory->systemshell_path|lib-NACL-C-_Mixins-VolumeFileDirectory-pm/systemshell_path >>

=cut

sub _file_dir_find_state_wrapper {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my %opts = $self->_common_validate_with(
        params => \@args,
        additional_spec => {
            'file-type' => { type => SCALAR, optional => 1 },
            filter      => { type => HASHREF, default => {} },
        },
        
        # Allow other options for fetch
        allow_extra => 1
    );

    my %filter = %{$opts{filter}};

    $self->_hash_move(
        source => \%opts,
        target => \%filter,
        move   => [ qw(vserver file-type) ],
        map    => { path => 'directory-path' }
    );
    $opts{filter} = \%filter;
    my @cs = NACL::CS::VolumeListFilesInDirectory->fetch(%opts);
    $Log->exit() if $may_exit;
    return @cs;
}

# Implementation of these is in NACL::C::_Mixins::VolumeFileDirectory

=head2 get_containing_volume_name

    my $vol_name = $file_obj->get_containing_volume_name();
    
Returns the name of the volume containing the file.


=head2 get_containing_volume_obj

    my $vol_obj = $file_obj->get_containing_volume_obj();
    
Returns a C<NACL::C::Volume> object modeling the volume containing the file.

=cut

# Both find() and state() should go through NACL::CS::VolumeFile::fetch
sub get_CS_package_name {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;
    return 'NACL::CS::VolumeFile';
}

=head2 get_directory_name

Will return the directory name from the path. Example:
my $directory_name = $file_obj->get_directory_name();
(Instance method)
Returns the directory name.

=cut

sub get_directory_name {
    $Log->enter() if $may_enter;
    my $self = $_[0];

    my $directory_name = $self->get_file_or_dir_name_from_path(path => $self->path());

    $Log->exit() if $may_exit;
    return $directory_name;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        path    => {type => SCALAR  | OBJECT },
        vserver => {type => SCALAR  | OBJECT , optional => 1}
    );
}

sub call_on_apiset {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my @ret = $pkg_or_obj->_call_on_apiset(@args);
    $Log->exit() if $may_exit;
    return wantarray ? @ret: $ret[0];
}

# Make handle_name_ordinal a valid argument, which defaults to 1
sub _common_validate_with {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => { additional_spec => { type => HASHREF, default => {} } },
        allow_extra => 1
    );
    my %additional_spec = %{$opts{additional_spec}};
    $additional_spec{handle_name_ordinal} ||= { type => SCALAR, default => 1 };
    
    my %validated_opts = $pkg_or_obj->SUPER::_common_validate_with(
        %opts,
        additional_spec => \%additional_spec
    );
    
    $Log->exit() if $may_exit;
    return %validated_opts;
}

1;
