#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeEfficiencyPolicy Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VolumeEfficiencyPolicy

=head1 DESCRIPTION

C<NACL::C::VolumeEfficiencyPolicy> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VolumeEfficiencyPolicy.  A related class is
L<NACL::CS::VolumeEfficiencyPolicy|lib-NACL-CS-VolumeEfficiencyPolicy-pm>, which represents
the past state of an ONTAP VolumeEfficiencyPolicy.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 policy

=cut

package NACL::C::VolumeEfficiencyPolicy;

use strict;
use warnings;

use NACL::CS::VolumeEfficiencyPolicy;
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NACL::C::Exceptions::VolumeEfficiencyPolicy::AlreadyExists ();
use NACL::C::Exceptions::VolumeEfficiencyPolicy::DoesNotExist ();
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $VolumeEfficiencyPolicy = NACL::C::VolumeEfficiencyPolicy->new(command_interface=>$ci,
                                   vserver =>$vserver,
                                   policy =>$policy,
                                  );

Return a new VolumeEfficiencyPolicy component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'policy',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VolumeEfficiencyPolicy  = NACL::C::VolumeEfficiencyPolicy->find(command_interface => $ci, ...);
    my @VolumeEfficiencyPolicys = NACL::C::VolumeEfficiencyPolicy->find(command_interface => $ci, ...);

    Uses a CMode CLI or CMode ZAPI APISet.

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $VolumeEfficiencyPolicy_state = $NACL::C::VolumeEfficiencyPolicy->state();

    Uses a CMode CLI or CMode ZAPI APISet.

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VolumeEfficiencyPolicy would return
a NACL::CS::VolumeEfficiencyPolicy object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

   NACL::C::VolumeEfficiencyPolicy->modify(
                                command_interface => $ci,
                                 vserver => $vserver,
                                 policy => $policy,
                                  ...);

                                 or

    $VolumeEfficiencyPolicy_object->modify(...);

(Class or instance method)
This method is used to modify an efficiency policy.

Uses a CMode CLI/ZAPI APISet.

Invokes "sis-policy-modify" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
         option.

=item C<< "policy" => $policy >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI: policy-name

=item C<< "comment" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI: comment

=item C<< "duration-mins" => $string >>

Applicable for CMode CLI.

=item C<< "duration" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI: duration

=item C<< "schedule" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI: schedule

=item C<< "enabled" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI: enabled

=item C<< 'threshold-percent' => $string >>

Applicable for CMode CLI.

=item C<< 'qos-policy' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI: qos-policy

=item C<< 'type' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'second-window-qos-level' => $string >>

Second Window QoS Level
Applicable for CMode CLI

=item C<< 'window-only' => $string >>

Window Only possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'second-window-start' => $string >>

Second Window Start
Applicable for CMode CLI

=item C<< 'first-window-start' => $string >>

First Window Start
Applicable for CMode CLI

=item C<< 'qos-level' => $string >>

QoS Level
Applicable for CMode CLI

=item C<< 'first-window-qos-level' => $string >>

First Window QoS Level
Applicable for CMode CLI

=item C<< 'first-window-duration' => $string >>

First Window Duration
Applicable for CMode CLI

=item C<< 'second-window-duration' => $string >>

Second Window Duration
Applicable for CMode CLI

=item C<< 'system-managed' => $string >>

System Managed possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'start-threshold-percent' => $string >>

Threshold Percentage
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"volume efficiency policy modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VolumeEfficiencyPolicy::DoesNotExist>

This type of exception gets thrown if an attempt is made to modify
the policy which does not exists.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( comment duration-mins duration schedule enabled

                    threshold-percent qos-policy type

                    second-window-qos-level window-only second-window-start
                    first-window-start qos-level first-window-qos-level first-window-duration
                    second-window-duration system-managed

                    start-threshold-percent
                    )
            ),
        },
        api => 'volume_efficiency_policy_modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_modify_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_modify_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VolumeEfficiencyPolicy::modify',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = $_[0];
        NACL::C::Exceptions::VolumeEfficiencyPolicy::DoesNotExist
            ->detect_convert_and_throw(exception => $exception_object);
        $Log->exit() if $may_exit;
        $exception_object->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api             => 'volume_efficiency_policy_modify',
        frontend_method => 'modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api             => 'sis-policy-modify',
        copy            => [qw( comment duration schedule enabled qos-level)],
        map             => {"policy" => "policy-name",},
        frontend_method => 'modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 delete

   NACL::C::VolumeEfficiencyPolicy->delete(
                                command_interface => $ci,
                                 vserver => $vserver,
                                 policy => $policy,
                                  ...);

                                 or

    $VolumeEfficiencyPolicy_object->delete(...);

(Class or instance method)
This method is used to delete an efficiency policy.

Uses a CMode CLI/ZAPI APISet.

Invokes "sis-policy-delete" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
         option.

=item C<< "policy" => $policy >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI: policy-name

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"volume efficiency policy delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VolumeEfficiencyPolicy::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete
the policy which does not exists.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd => 'volume efficiency policy delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VolumeEfficiencyPolicy::delete',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = $_[0];
        NACL::C::Exceptions::VolumeEfficiencyPolicy::DoesNotExist
            ->detect_convert_and_throw(exception => $exception_object);
        $Log->exit() if $may_exit;
        $exception_object->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api => 'volume_efficiency_policy_delete',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api    => 'sis_policy_delete',
        map    => {"policy" => "policy-name"},
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 create

    my $VolumeEfficiencyPolicy_object    = NACL::C::VolumeEfficiencyPolicy->create(
                                command_interface => $ci,
                                 vserver => $vserver,
                                 policy => $policy,
                                  ...);

(Class method)
This method is used to create an efficiency policy.

Uses a CMode CLI/ZAPI APISet.

Invokes "sis-policy-create" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
         option.

=item C<< "policy" => $policy >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI: policy-name

=item C<< "comment" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI: comment

=item C<< "duration-mins" => $string >>

Applicable for CMode CLI.

=item C<< "duration" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI: duration

=item C<< "schedule" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI: schedule

=item C<< "enabled" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI: enabled

=item C<< 'threshold-percent' => $string >>

Applicable for CMode CLI.

=item C<< 'qos-policy' => $string >>

Applicable for CMode CLI.

=item C<< 'type' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'second-window-qos-level' => $string >>

Second Window QoS Level
Applicable for CMode CLI

=item C<< 'window-only' => $string >>

Window Only possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'second-window-start' => $string >>

Second Window Start
Applicable for CMode CLI

=item C<< 'first-window-start' => $string >>

First Window Start
Applicable for CMode CLI

=item C<< 'qos-level' => $string >>

QoS Level
Applicable for CMode CLI/ZAPI.

Maps to :

CMode ZAPI: qos-level

=item C<< 'first-window-qos-level' => $string >>

First Window QoS Level
Applicable for CMode CLI

=item C<< 'first-window-duration' => $string >>

First Window Duration
Applicable for CMode CLI

=item C<< 'second-window-duration' => $string >>

Second Window Duration
Applicable for CMode CLI

=item C<< 'system-managed' => $string >>

System Managed possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'start-threshold-percent' => $string >>

Threshold Percentage
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"volume efficiency policy create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VolumeEfficiencyPolicy::AlreadyExists>

This type of exception gets thrown if an attempt is made to create
the policy which already exists.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( comment duration-mins duration schedule enabled

                    threshold-percent qos-policy type

                    second-window-qos-level window-only second-window-start
                    first-window-start qos-level first-window-qos-level first-window-duration
                    second-window-duration system-managed

                    start-threshold-percent
                    )
            ),
        },
        cli_cmd => 'volume efficiency policy create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_create_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_create_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VolumeEfficiencyPolicy::create',
        );
    } 
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VolumeEfficiencyPolicy::AlreadyExists
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $vserver,
        policy            => $opts{'policy'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_cli(
        %opts,
        api             => 'volume_efficiency_policy_create',
        frontend_method => 'create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'sis-policy-create',
        copy => [qw( comment duration schedule enabled qos-level )],
        map  => {"policy" => "policy-name",},
        frontend_method => 'create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver' => {type => SCALAR},
        'policy'  => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
