#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeFileClone Component Module
## @author Akshay Mohit(makshay@netapp.com), dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VolumeFileClone

=head1 DESCRIPTION

C<NACL::C::VolumeFileClone> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. Unlike other components this
package does not model any element/service but is present to provide
component-level support for functionality. Since it does not model any
element/service there is no ComponentState module for this component.
For the same reason, 'find' and 'state' will not work for this component.

=cut

package NACL::C::VolumeFileClone;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NACL::CS::VolumeFileCloneStatus;
use NATE::BaseException qw(:try);
use NACL::C::Exceptions::VolumeFileClone::SourceFileDoesNotExist;
use NACL::C::Exceptions::VolumeFileClone::CloneFileExists;
use NACL::C::Exceptions::VolumeFileClone::CurrentlyBusy;
=head2 create

   $VolumeFileClone =  NACL::C::VolumeFileClone->create(
                                command_interface => $ci,
                                vserver => $TestVserver,
                                volume => $Volume,
                                "source-path" => $Lun,
                                "destination-path" => $Clone,
                                "no-reserve" => "true",
                                "snapshot-name" => $SnapshotName
                              );

(Class method)
This method is used to create file or LUN full/sub file clone,
and returns a new object of type NACL::C::VolumeFile.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "clone-create" API for CMode ZAPI.

Invokes "clone start" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "volume" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: volume

=item C<< "vserver" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< "snapshot-name" => $string >>

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: snapshot-name

7M CLI : -s (Programmatic name: "snapshot-name")

=item C<< "destination-path" => $string >>

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: destination-path

7M CLI : destination path (Programmatic name: "destination-path")

=item C<< "source-path" => $string >>

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: source-path

7M CLI : source path (Programmatic name: "source-path")

=item C<< "clone_type" => $string >>

 ("clone_type option can take two possible values - lun or file (case sensitive).
   This option will not be sent to API layer rather it is an option consumed
   by the component itself to differentiate the entity which needs to be cloned.
   This is not required for CMode CLI, CMode ZAPI, 7M ZAPI and if passed,
   it will be ignored. Only in the case of 7M CLI, this option will be used to
   invoke the appropriate command as the command to perform lun cloning and file
   cloning are different for 7Mode CLI")

=item C<< "no-reserve" => $boolean >>

Applicable for CMode CLI,7Mode CLI.

Maps to:

7M CLI : -o (Programmatic name: "skip-space")

=item C<< "range" => [ range1, range2 ] >>

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: block-ranges

7M CLI : range triplet (Programmatic name: "range-triplet")

=item C<< 'destination-exists' => $string >>

Applicable for CMode CLI.

=item C<< 'ignore-streams' => $string >>

Applicable for CMode CLI.

=item C<< 'ignore-locks' => $string >>

Ignore locks

Applicable for CMode CLI.

=item C<< 'qos-policy-group' => $string >>

QoS Policy Group Name

Applicable for CMode CLI.

=item C<< 'external-cache' => $string >>

External Cache Policy Name

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< 'autodelete' => $string >>

Mark Clone for Autodeletion possible value(s) are, true,false
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: autodelete

=item C<< 'bypass-throttle' => $string >>

Bypass Throttle Checks possible value(s) are, true,false
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: bypass-throttle

=item C<< 'token-id' => $string >>

Token ID of Reserved Load
Applicable for CMode CLI

=item C<< 'overwrite-destination' => $string >>

Overwrite Destination
Applicable for CMode CLI

=item C<< 'token-uuid' => $string >>

Token UUID of Reserved Load
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"volume file clone create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VolumeFileClone::SourceFileDoesNotExist>

This exception is thrown when a clone operation is failed as source path does not exist.

=item C<NACL::C::Exceptions::VolumeFileClone::CloneFileExists>

This exception is thrown when we attempt to create a file clone which already exists.

=item C<NACL::C::Exceptions::VolumeFileClone::CurrentlyBusy>

This exception is thrown when clone operation is failed as the volume was busy

=item C<< 'caching-policy' => $string >>

Caching Policy Name
Applicable for CMode CLI

=item C<< 'is-file' => $string >>

File or LUN flag possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'qos-policy-group-uuid' => $string >>

QoS Policy Group UUID
Applicable for CMode CLI

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            range      => {type => ARRAYREF, optional => 1},
            clone_type => {
                type      => SCALAR,
                default   => 'lun',
                callbacks => {
                    "Value of 'clone_type' should be either 'lun' or 'file'"
                        => sub { $_[0] =~ /^(lun)|(file)$/ }
                }
            },
            _optional_scalars(
                qw( volume vserver snapshot-name destination-path
                    source-path no-reserve skip_space block_range range_triplet
                    destination-exists

                    ignore-streams

                    ignore-locks

                    qos-policy-group

                    external-cache

                    caching-policy

                    is-file qos-policy-group-uuid

                    autodelete

                    bypass-throttle token-id

                    overwrite-destination

                    token-uuid
                    )
            )
            }    ## end
        ,
        cli_cmd => 'volume file clone create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $command_interface = $opts{command_interface};
    my $ret;
    try {
        $ret = $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_create_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_create_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
                {   method    => '_create_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                }
            ],
            frontend => 'NACL::C::VolumeFileClone::create',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_obj = $_[0];
        NACL::C::Exceptions::VolumeFileClone::SourceFileDoesNotExist
            ->detect_convert_and_throw(exception => $exception_obj);
        NACL::C::Exceptions::VolumeFileClone::CloneFileExists
            ->detect_convert_and_throw(exception => $exception_obj);
        NACL::C::Exceptions::VolumeFileClone::CurrentlyBusy
            ->detect_convert_and_throw(exception => $exception_obj);
        $Log->exit() if $may_exit;
        $exception_obj->throw();
    };
    my $destination_path;
    if ($opts{'volume'}) {
        my $dest;
        if ($opts{'destination-path'} =~ /^\/(.*)/) {
            $dest = $1;
        } else {
            $dest = $opts{'destination-path'};
        }
        $destination_path = '/vol/' . $opts{'volume'} . '/' . $dest;
    } else {
        $destination_path = $opts{'destination-path'};
    }
    require NACL::C::VolumeFile;
    my %new_opts = (
        vserver => $opts{'vserver'},
        path    => $destination_path,
    );
    $pkg->_process_options(options => \%new_opts);
    my $pkg_to_use = 'NACL::'.$pkg->is_c_or_t().'::VolumeFile';
    my $obj = $pkg_to_use->new(
        command_interface => $command_interface,
        %new_opts
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'volume_file_clone_create',);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
$Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1,

    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $vserver           = delete $opts{vserver};

    my $allowed_cmode_options = [
        qw( autodelete volume destination-path source-path snapshot-name bypass-throttle destination-exists overwrite-destination token-uuid ignore-streams ignore-locks bypass-license-check qos-policy-group-name
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        map    => {"no-reserve" => "space-reserve"},
        target => \%cmode_args,
    );

    my $range = $opts{range};
    if (defined $range) {
        my @block_ranges;
        foreach my $single_range (@$range) {
            my @fields_of_range = split ':', $single_range;
            push @block_ranges,
                {
                    'source-block-number'      => $fields_of_range[0],
                    'destination-block-number' => $fields_of_range[1],
                    'block-count'              => $fields_of_range[2],
                };
        }
        $cmode_args{'block-ranges'} = \@block_ranges;
    }
	if(defined $cmode_args{'space-reserve'} &&
            $cmode_args{'space-reserve'} eq "true") {
        $cmode_args{'space-reserve'} = "false";
    } else {
        $cmode_args{'space-reserve'} = "true";
    }

    my $get_vol_name = sub {
        my $val = shift;
        my $vol_name;
        if ($val =~ /^\/vol\/(.+?)\//) {
            $vol_name = $1;
        }
        return $vol_name;
    };

    my $get_relative_path = sub {
        my $val = shift;
        if ($val =~ /^\/vol\/(.+?)\/(.+)/) {
            my $rel_path = $2;
            return $rel_path;
        }

        # If it's not in expected format, return the same value as it is
        return $val;
    };

    unless (defined $cmode_args{volume}) {
		# The paths will be in absolue path format.
        my $vol_name_from_src = $get_vol_name->($cmode_args{'source-path'});
        my $vol_name_from_dest
            = $get_vol_name->($cmode_args{'destination-path'});
        if (defined $vol_name_from_src && defined $vol_name_from_dest) {
            if ($vol_name_from_src ne $vol_name_from_dest) {
                $Log->exit;
                NATE::Exceptions::Argument->throw(
                          "Volume specified in source-path "
                        . $cmode_args{'source-path'}
                        . "differs from the volume "
                        . "specified in destination-path "
                        . $cmode_args{'destination-path'});
            }
            $cmode_args{volume} = $vol_name_from_src;
            $cmode_args{'source-path'}
                = $get_relative_path->($cmode_args{'source-path'});
            $cmode_args{'destination-path'}
                = $get_relative_path->($cmode_args{'destination-path'});
        }

        # else, options are not in expected format. just pass it down and
        # leave it to ONTAP to complain.
    }

    $pkg->_handle_zapi_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $vserver,
        command_interface => $command_interface
    );

    $apiset->clone_create(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

sub _create_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    my %clone_args;

    if ($opts{'clone_type'} =~ /file/i) {
        my $allowed_clone_options = [
            qw( source-path destination-path snapshot-name
                )
        ];
        if (defined $opts{range}) {
            $opts{range} = join(':', @{$opts{range}});
            $clone_args{'block-range'} = 1;
        }
        $pkg->_hash_copy(
            source => \%opts,
            copy   => $allowed_clone_options,
            map    => {
                'range'      => 'range-triplet',
                'no-reserve' => 'skip-space'
            },
            target => \%clone_args,
        );

        $apiset->clone_start(%clone_args);
    } else {
        my %args;
        $pkg->_hash_copy(
            source => \%opts,
            map    => {
                'source-path'      => 'parent-lunpath',
                'destination-path' => 'clone-lunpath',
                'snapshot-name'    => 'parent-snap',
                'no-reserve'       => 'noreserve'
            },
            target => \%args,
        );

        if ($args{'parent-lunpath'} !~ /\//) {
            $args{'parent-lunpath'}
                = "/vol/$opts{volume}/$args{'parent-lunpath'}";
        }
        if ($args{'clone-lunpath'} !~ /\//) {
            $args{'clone-lunpath'}
                = "/vol/$opts{volume}/$args{'clone-lunpath'}";
        }
        $pkg->_api_call_log_debug_opts(
            api      => 'lun clone create',
            api_opts => \%args,
        );
        $apiset->lun_clone_create(%args);
    } ## end else [ if ( $opts{'clone_type'...
    $Log->exit() if $may_exit;
} ## end sub _create_7mode_cli

=head2 status

    $status_obj = NACL::C::VolumeFileClone->status(
                                command_interface => $ci,
                                ...);
(Class method)

This method returns an object of type L<NACL::CS::VolumeFileCloneStatus|
lib-NACL-CS-VolumeFileCloneStatus-pm>

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"volume file clone status" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub status {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        api                 => 'volume_file_clone_status',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_status_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeFileClone::status',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub status

sub _status_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $ci = $opts{command_interface};
    my $response;
    $pkg->_base_backend_cmode_cli(
        %opts,
        api                 => 'volume_file_clone_status',
        apiset_response_obj => \$response,
    );

    my $output = $response->get_parsed_output();
    my $obj = NACL::CS::VolumeFileCloneStatus->new(command_interface => $ci);
    $obj->_set_fields(row => $output->[0]);
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _status_cmode_cli

=head2 stop

    NACL::C::VolumeFileClone->stop(
                                command_interface => $ci,
                                ...);
(Class method)
This method is used to stops a running clone operation.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"volume file clone stop" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        api                 => 'volume_file_clone_stop',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_stop_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeFileClone::stop',
    );

    $Log->exit() if $may_exit;
} ## end sub stop

sub _stop_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'volume_file_clone_stop',);
    $Log->exit() if $may_exit;
} ## end sub _stop_cmode_cli

=head2 clear

    NACL::C::VolumeFileClone->clear(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to clears information of a failed clone operation.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"volume file clone clear" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub clear {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        api                 => 'volume_file_clone_clear',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_clear_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeFileClone::clear',
    );

    $Log->exit() if $may_exit;
} ## end sub clear

sub _clear_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'volume_file_clone_clear',);
    $Log->exit() if $may_exit;
} ## end sub _clear_cmode_cli

=head2 show_autodelete_list

    my $show_autodelete_list_obj = NACL::C::VolumeFileClone->show_autodelete_list(command_interface => $ci,
                          [ filter => { %filter } ,]
  [ requested_fields => [ @req_fields ]);
                    or

    my @show_autodelete_list_objs = NACL::C::VolumeFileClone->show_autodelete_list(command_interface => $ci,
                          [ filter => { %filter } ,]
  [ requested_fields => [ @req_fields ]);

(Class method)
This method returns objects of the type L<NACL::CS::VolumeFileCloneShowAutodeleteList|lib-NACL-CS-VolumeFileCloneShowAutodeleteList-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch>
(requested_fields, filter, allow_empty) and has all of the other properties of
fetch methods (callable in scalar or list context, throws a NoElementsFound
exception if no matching elements are found)

=cut

sub show_autodelete_list {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::VolumeFileCloneShowAutodeleteList;
    my @state_objs
        = NACL::CS::VolumeFileCloneShowAutodeleteList->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub show_autodelete_list

=head2 autodelete

    NACL::C::VolumeFileClone->autodelete(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to enable/Disable autodelete.

Uses a CMode CLI/ZAPI APIset.

Invokes "clone-autodelete" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'enable' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: enable

=item C<< 'clone-path' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: clone-path

=item C<< 'volume' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: volume

=item C<< 'vserver' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
         option.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< 'force' => $string >>

Force Enable Autodelete possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"volume file clone autodelete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub autodelete {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( enable clone-path volume vserver

                    force
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'volume file clone autodelete',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_autodelete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_autodelete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeFileClone::autodelete',
    );

    $Log->exit() if $may_exit;
} ## end sub autodelete

sub _autodelete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'volume_file_clone_autodelete',);
    $Log->exit() if $may_exit;
} ## end sub _autodelete_cmode_cli

sub _autodelete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $allowed_cmode_options = [
        qw( enable clone-path volume force
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
            api    => 'clone_autodelete',
            copy   => $allowed_cmode_options,
    );
    $Log->exit() if $may_exit;

} ## end sub _autodelete_cmode_zapi

=head2 split_estimate

    my $split_estimate_obj = NACL::C::VolumeFileClone->split_estimate(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @split_estimate_objs = NACL::C::VolumeFileClone->split_estimate(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $split_estimate_obj = $VolumeFileClone_obj->split_estimate();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).

=cut

sub split_estimate {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::VolumeFileCloneSplitEstimate;
    my @state_objs = NACL::CS::VolumeFileCloneSplitEstimate->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 show_autodelete

    my $show_autodelete_obj = NACL::C::VolumeFileClone->show_autodelete(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @show_autodelete_objs = NACL::C::VolumeFileClone->show_autodelete(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $show_autodelete_obj = $VolumeFileClone_obj->show_autodelete();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-VolumeFileCloneShowAutodelete-pm/ATTRIBUTES>
of NACL::CS::VolumeFileCloneShowAutodelete as input.
This method returns objects of the type L<NACL::CS::VolumeFileCloneShowAutodelete|lib-NACL-CS-VolumeFileCloneShowAutodelete-pm>.

=cut

sub show_autodelete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::VolumeFileCloneShowAutodelete;
    my @state_objs = NACL::CS::VolumeFileCloneShowAutodelete->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}


__PACKAGE__->_load_event_class();

1;
