#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeMember Component Module
## @author Dheeraj.K@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VolumeMember

=head1 DESCRIPTION

C<NACL::C::VolumeMember> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP Volume Member.  A related class is
L<NACL::CS::VolumeMember|lib-NACL-CS-VolumeMember-pm>,
which represents the past state of an ONTAP Volume Member.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 volume

This specifies the name of the member volume.

=head2 vserver

This specifies the virtual server on which the member volume is located.

=cut

package NACL::C::VolumeMember;

use strict;
use warnings;
use NACL::CS::VolumeMember;
use NACL::ComponentUtils qw (_optional_scalars);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF HASHREF);
use NACL::APISet::Exceptions::MethodNotFoundException qw(:try);

=head1 METHODS

=head2 new

  my $volume_member_object = NACL::C::VolumeMember->new(
                                              command_interface=>$ci,
                                              vserver =>$vserver_name,
                                              volume =>$volume_name,
                                              ....);

Return a new volume member component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'volume',
    scalar => 'vserver',
];

=head2 move

  NACL::C::VolumeMember->move(command_interface=>,
                          vserver=>$vserver_name,
                          volume=>$volume_name,
                          ...);

                            or

  $volume_member_object->move(...);

(Class or instance method)
This method moves a member volume of a striped volume from one member
aggregate to another. Member volumes, striped volumes, member aggregates, and
striped aggregates are the building blocks of file striping.While this command
appears to run synchronously, it actually runs asynchronously. To determine
the status of the operation, use the the job show command to determine the
commands job ID and to display the jobs status.This command is available only
at the advanced privilege level and higher.

Uses a CMode-CLI APISet.

=over

=item Options

=over

=item C<< command_interface=> $ci >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$string >>

(Required for class method, Not Applicable for instance method)
(Required) Vserver Name
  This specifies the virtual server on which the member volume is
  located.

Applicable for CMode CLI.


=item C<< volume=>$string >>

(Required for class method, Not Applicable for instance method)
(Required) Volume Name
  This specifies the member volume that is to be moved. You can dis-
  play a list of member volumes by using the volume member show command.

Applicable for CMode CLI.

=item C<< destination-aggregate=>$string >>

(Required) Destination aggregate
  This specifies the member aggregate to which the member volume is
  to be moved. You can display a list of member aggregates by using the
  storage aggregate member show command.

Applicable for CMode CLI.

=item C<< foreground =>$string >>

Applicable for CMode CLI.

=item C<< "method-timeout"=> $scalar >>

(Optional)The new timeout value.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub move {
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw(destination-aggregate foreground
                    )
            )
        }
    );
    return $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_move_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
    );
} ## end sub move

sub _move_cmode_cli {
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw(volume vserver destination-aggregate
            foreground extended_query )
    ];
    my $cmode_args = $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
    );

    $pkg->_execute_api_version(
        apiset_obj      => $apiset,
        api_calls       => ["volume_member_move", "volume_member_move_start"],
        command_options => $cmode_args
    );
} ## end sub _move_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        volume  => {type => SCALAR},
        vserver => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
