# $Id: Id: //depot/prod/test/nacldev/lib/NACL/C/VolumeQuota.pm $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeQuota Component Module
## @author Dheeraj.K@netapp.com, pragyan@netapp.com, dl-nacl-dev@netapp.com
## @author anupamap@netapp.com 2011-07-13 : Added 7Mode CLI support for quota modify
##                                        : Added code for handling -w options in quota on
## @status shared
## @pod here

=head1 NAME

NACL::C::VolumeQuota

=head1 DESCRIPTION

C<NACL::C::VolumeQuota> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to quota state of  an ONTAP volume.  A related class is
L<NACL::CS::VolumeQuota|lib-NACL-CS-VolumeQuota-pm>, which represents
the past state of a VolumeQuota on an ONTAP volume.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

(Required for CMode-CLI, Not Applicable for 7Mode-ZAPI or 7Mode-CLI) The name of the vserver,
in which the volume resides

=head2 volume

The name of the volume, for which the quota  applies

=cut

package NACL::C::VolumeQuota;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate_pos :types);
use NACL::CS::VolumeQuota;
use NATE::Exceptions::Argument qw(:try);
use NACL::CS::VolumeQuotaReport;
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::Job;
use NACL::Exceptions::NoElementsFound qw(:try);
use base 'NACL::C::Component::ONTAP';
use NACL::STask::Volume;
use FG::RAL::util;

use constant {
    # from ontap/prod/common/wafl/format/wafl_remote_index.h
    WRE_MASK_IS_DIRTY => '0x00FF80',
};

=head1 METHODS

=head2 new

  my $volume = NACL::C::VolumeQuota->new(command_interface=>$ci,volume => $vol_name,
                                 vserver => $vserver_name);

Return a new VolumeQuota component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'volume',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
} ## end sub init

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::C::Quota'
    );
    $Log->exit() if $may_exit;
    return $isa;
} ## end sub isa

=head2 modify

  NACL::C::VolumeQuota->modify(command_interface=>$ci,
                         vserver => $vserver_name,
                         volume  => $volume_name,
                         ...);

                            or

  $quota_object->modify(...);

(Class or instance method) Modify a quota element.

Uses a CMode-CLI or 7Mode-CLI or CMode-ZAPI or 7Mode-ZAPI APISet.

Invokes "quota on" command for 7Mode CLI if state option passed in is "on"

Invokes "quota off" command for 7Mode CLI if state option passed in is "off"

Invokes "quota logmsg on" command for 7Mode CLI if state option passed in is "on"
and logging option passed in is on.

Invokes "quota logmsg off" command for 7Mode CLI if state option passed in is "on"
and logging option passed in is off.

Invokes "command" command for 7Mode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required for class method, Not Applicable for instance method and
Required for CMode-CLI , Not Applicable for 7Mode-ZAPI)
Name of vserver that contains the volume.

Applicable for CMode CLI.

=item C<< volume=>$volume_name >>

(Required for class method, Not Applicable for instance method) Name of volume
to which this quota applies.

Applicable for CMode CLI, 7Mode CLI/ZAPI.

Maps to:

7M CLI : -v <value> (Programmatic name: "volume")

7M ZAPI: volume

=item C<< "state" => $string >>

Applicable for CMode-CLI, CMode-ZAPI, 7Mode-CLI and 7Mode-ZAPI

Maps to:

7M ZAPI: state

=item C<< "logging" => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI:
Invokes "quota logmsg on/quota logmsg off" commands with "volume, logging,
logging-interval" options.

=item C<< "logging-interval" => $string >>

Note that C<logging> and C<logging-interval> cannot be modified through
ZAPI, so if either of these arguments are provided in the call to this method,
then this method will attempt to go through CLI.

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : interval time (Programmatic name: "interval")

=item C<< "foreground" => $boolean >>

Whether to run the command in the foreground. Value can be either of
"true" or "false".

Applicable only for CMode-CLI.

=item C<< "method-timeout"=> $scalar >>

(Optional)The new timeout value.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"volume quota modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            job_component => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw (state logging logging-interval foreground
                    )
            )
        },
        cli_cmd => 'volume quota modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },

            # Common back-end for CMode and 7Mode ZAPI
            {   method    => '_modify_zapi',
                interface => 'ZAPI',
                check     => '_modify_zapi_check'
            },
        ],
        frontend => 'NACL::C::VolumeQuota::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $response;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
            api => 'volume_quota_modify',
            apiset_response_obj => \$response
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@args);
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my $state             = $opts{state};
    my $vserver           = $opts{vserver};

    if ($state) {
        unless ($state eq 'on' || $state eq 'off' || $state eq 'resize') {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw("Value of 'state' should "
                    . "have been either 'on' or 'off' or 'resize' "
		    . "but was provided as '$state'\n");
        } ## end unless ( $state eq 'on' ||...)

        my $copy = [qw(volume)];

        my %zapi_args;
        $pkg_or_obj->_hash_copy(
            source => \%opts,
            copy   => $copy,
            target => \%zapi_args,
        );

        $pkg_or_obj->_handle_zapi_vserver_context(
            api_opts          => \%zapi_args,
            vserver           => $vserver,
            command_interface => $command_interface
        ) if $command_interface->is_cmode();

        my $command = "quota_$state";
        try {
            $apiset->$command(%zapi_args);
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            $pkg_or_obj->_exception_handler($_[0]);
        };
    } ## end if ($state)

    $Log->exit() if $may_exit;
} ## end sub _modify_zapi

sub _modify_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};
    my $state  = $opts{state};
    my $invoke_off;

    if ($state) {
        if ($state eq 'on') {
            try {
                $apiset->quota_on('volume-name' => $opts{volume});
            }
            catch NACL::APISet::Exceptions::ResponseException with {
                $pkg_or_obj->_exception_handler($_[0]);
            };
        } elsif ($state eq 'off') {

            # If there are other values we'd like to modify, then switch
            # off the quota last. This is because properties can be modified
            # only when the quota is on.
            $invoke_off = 1;
        } else {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw("Value of 'state' should "
                    . "have been either 'on' or 'off' but was provided as "
                    . "'$state'\n");
        } ## end else [ if ( $state eq 'on' ) ]
    } ## end if ($state)

    my $logging = $opts{logging};
    if ($logging) {
        my $api = 'quota_logmsg_';
        my %quota_args;
        my %hash_copy_args = (
            source => \%opts,
            copy   => [qw(volume)],
            target => \%quota_args
        );

        if ($logging eq 'on') {
            $api .= 'on';
            if ($opts{'logging-interval'} ne '') {
                $hash_copy_args{map} = {'logging-interval' => 'interval'};
            }
        } elsif ($logging eq 'off') {
            $api .= 'off';
        } else {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw("Value of 'logging' should "
                    . "have been either 'on' or 'off' but was provided as "
                    . "'$logging'\n");
        } ## end else [ if ( $logging eq 'on' )]

        $pkg_or_obj->_hash_copy(%hash_copy_args);
        $apiset->$api(%quota_args);
    } ## end if ($logging)

    if ($invoke_off) {
        try {
            $apiset->quota_off('volume-name' => $opts{volume});
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            $pkg_or_obj->_exception_handler($_[0]);
        };
    } ## end if ($invoke_off)

    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_cli

=head2 on

  NACL::C::VolumeQuota->on(command_interface=>$ci,
  $quota_object->on(...);
  (Class or instance method) Turn on a quota element.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "quota-on" API for CMode ZAPI.

Invokes "quota on" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>
(Required for class method, Not Applicable for instance method)

See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required for class method, Not Applicable for instance method and

Required for CMode-CLI , Not Applicable for 7Mode-ZAPI or 7Mode-CLI)
Name of vserver that contains the volume.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
         option.

=item C<< volume=>$volume_name >>

(Required for class method, Not Applicable for instance method) Name of volume
to which this quota applies.

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: volume

7M CLI : volume name (Programmatic name: "volume-name")

=item C<< foreground=>$string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : -w (Programmatic name: "foreground")

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"volume quota on" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub on {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            job_component => {type => SCALARREF, optional => 1},
            _optional_scalars(qw (foreground))
        },
        cli_cmd => 'volume quota on',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $Log->exit() if $may_exit;
    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_on_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_on_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
            {   method    => '_on_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                check     => '_common_quota_zapi_check'
            },
        ],
        frontend => 'NACL::C::VolumeQuota::on',
    );
} ## end sub on

sub _on_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $response;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
            api => 'volume_quota_on',
            apiset_response_obj => \$response
    );
    $Log->exit() if $may_exit;
} ## end sub _on_cmode_cli

sub _on_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_7mode_options = [qw(foreground)];
    my $quota_args            = $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => {"volume" => "volume-name"},
    );

    %opts = %$quota_args;
    if ($opts{'foreground'} eq "false") {
        $opts{'foreground'} = "0";
    } else {
        $opts{'foreground'} = "1";
    }
    try {
        my $response = $apiset->quota_on(%opts);
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $Log->debug("Error is: " . $exception_object->text());
        if ($exception_object->text() =~ /quotas are already on/) {
            $Log->debug("To suppress the 7Mode CLI error");
        } else {
            $exception_object->throw();
        }
    };
    $Log->exit() if $may_exit;
} ## end sub _on_7mode_cli

sub _on_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $response;
    my $allowed_cmode_options = [qw(volume)];
    try{
        $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'quota_on',
            copy   => $allowed_cmode_options,
            inapplicable_for_help_xml => [qw(foreground)]
        );
   
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $Log->comment("Error is: " . $exception_object->text());
        if ($exception_object->text() =~ /Quotas are already/) {
            $Log->debug("To suppress the ZAPI error");
        } else {
            $exception_object->throw();
        }
    };
    $Log->exit() if $may_exit;
} ## end sub _on_cmode_zapi

=head2 off

  NACL::C::VolumeQuota->off(command_interface=>$ci,
                         vserver => $vserver_name,
                         volume  => $volume_name,
                         ...);
                            or
  $quota_object->off(...);

  (Class or instance method) Turn off a quota element.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "quota-off" API for CMode ZAPI.

Invokes "quota off" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)

See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required for class method, Not Applicable for instance method and

Required for CMode-CLI , Not Applicable for 7Mode-ZAPI or 7Mode-CLI)
Name of vserver that contains the volume.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
         option.

=item C<< volume=>$volume_name >>

(Required for class method, Not Applicable for instance method) Name of volume
to which this quota applies.

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: volume

7M CLI : volume name (Programmatic name: "volume")

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"volume quota off" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub off {

    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            job_component => {type => SCALARREF, optional => 1},
            _optional_scalars(qw (foreground ))
        },
        cli_cmd => 'volume quota off',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $Log->exit() if $may_exit;
    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_off_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_off_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
            {   method    => '_off_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                check     => '_common_quota_zapi_check'
            },

        ],
        frontend => 'NACL::C::VolumeQuota::off',
    );
} ## end sub off

sub _off_cmode_cli {
    my ($pkg_or_obj, %opts) = @_;
    my $response;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
            api => 'volume_quota_off',
            apiset_response_obj => \$response
    );
    $Log->exit() if $may_exit;
} ## end sub _off_cmode_cli

sub _off_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $quota_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {"volume" => "volume-name"},
    );

    %opts = %$quota_args;

    try {
        my $response = $apiset->quota_off(%opts);
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $Log->debug("Error is: " . $exception_object->text());
        if ($exception_object->text() =~ /quotas already off/) {
            $Log->debug("To suppress the 7Mode CLI error");
        } else {
            $exception_object->throw();
        }
    };
    $Log->exit() if $may_exit;

} ## end sub _off_7mode_cli

sub _off_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $response;
    my $allowed_cmode_options = [qw(volume)];
    try {
        $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'quota_off',
            copy   => $allowed_cmode_options,
            inapplicable_for_help_xml => [qw(foreground)],
        );    
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $Log->comment("Error is: " . $exception_object->text());
        if ($exception_object->text() =~ /Quotas are already/) {
            $Log->debug("To suppress the ZAPI error");
        } else {
            $exception_object->throw();
        }
    };
    $Log->exit() if $may_exit;
} ## end sub _off_cmode_zapi

=head2 resize

  NACL::C::VolumeQuota->resize(command_interface=>$ci,
                         vserver => $vserver_name,
                         volume  => $volume_name,
                         ...);
                            or
  $quota_object->off(...);

(Class or instance method) Resize a quota element.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "quota-resize" API for CMode ZAPI.

Invokes "quota resize" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)

See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required for class method, Not Applicable for instance method and

Required for CMode-CLI , Not Applicable for 7Mode-ZAPI or 7Mode-CLI)
Name of vserver that contains the volume.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
         option.

=item C<< volume=>$volume_name >>

(Required for class method, Not Applicable for instance method) Name of volume
to which this quota applies.

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: volume

7M CLI : volume name (Programmatic name: "volume")

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"volume quota resize" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub resize {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( foreground
                    )
            ),
            job_component => {type => SCALARREF, optional => 1},
        },
        cli_cmd => 'volume quota resize',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $Log->exit() if $may_exit;

    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_resize_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_resize_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
            {   method    => '_resize_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                check     => '_common_quota_zapi_check'
            },

        ],
        frontend => 'NACL::C::VolumeQuota::resize',
    );
} ## end sub resize

sub _resize_cmode_cli {
    my ($pkg_or_obj, %opts) = @_;
    my $response;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
            api => 'volume_quota_resize',
            apiset_response_obj => \$response
    );
    $Log->exit() if $may_exit;
} ## end sub _resize_cmode_cli

sub _resize_7mode_cli {
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $quota_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {"volume" => "volume-name"},
    );

    %opts = %$quota_args;
    $apiset->quota_resize(%opts);
} ## end sub _resize_7mode_cli

sub _resize_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts                = $pkg_or_obj->_validate_backend_opts(@args);
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api  => 'quota-resize',
		copy => [qw( volume )],
		vserver_operated_on => \$vserver,
    );
 $Log->exit() if $may_exit;
} ## end sub _resize_cmode_zapi

=head2 allow

  NACL::C::VolumeQuota->allow(command_interface=>$ci,
                         vserver => $vserver_name,
                         volume  => $volume_name,
                         ...);
                            or
  $quota_object->allow(...);

(Class or instance method) Allow a quota element.

Uses a 7Mode CLI APISet.

Invokes "quota allow" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)

See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< volume=>$volume_name >>

(Required for class method, Not Applicable for instance method) Name of volume
to which this quota applies.

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub allow {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(params => \@args,);

    $Log->exit() if $may_exit;

    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_allow_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );
} ## end sub allow

sub _allow_7mode_cli {
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $quota_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {"volume" => "volume-name"},
    );

    %opts = %$quota_args;
    $apiset->quota_allow(%opts);
} ## end sub _allow_7mode_cli

=head2 disallow

  NACL::C::VolumeQuota->disallow(command_interface=>$ci,
                         vserver => $vserver_name,
                         volume  => $volume_name,
                         ...);
                            or
  $quota_object->disallow(...);

(Class or instance method) Disallow a quota element.

Uses a 7Mode CLI APISet.

Invokes "quota disallow" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)

See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< volume=>$volume_name >>

(Required for class method, Not Applicable for instance method) Name of volume
to which this quota applies.

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub disallow {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(params => \@args,);

    $Log->exit() if $may_exit;

    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_disallow_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );
} ## end sub disallow

sub _disallow_7mode_cli {
    my $pkg  = shift;
    my %opts = validate_with(
        params         => \@_,
        spec           => $pkg->_backend_validate_spec(),
        disallow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $quota_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {"volume" => "volume-name"},
    );

    %opts = %$quota_args;
    $apiset->quota_disallow(%opts);
} ## end sub _disallow_7mode_cli

=head2 err_inject

  NACL::C::VolumeQuota->err_inject(command_interface=>$ci,
                         vserver => $vserver_name,
                         volume  => $volume_name,
                         ...);
                            or
  $quota_object->err_inject(...);

(Class or instance method) err_inject a quota element.

Uses a 7Mode CLI APISet.

Invokes "quota err inject" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)

See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< volume=>$volume_name >>

(Required for class method, Not Applicable for instance method) Name of volume
to which this quota applies.

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub err_inject {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(params => \@args,);

    $Log->exit() if $may_exit;

    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_err_inject_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );
} ## end sub err_inject

sub _err_inject_7mode_cli {
    my $pkg  = shift;
    my %opts = validate_with(
        params           => \@_,
        spec             => $pkg->_backend_validate_spec(),
        err_inject_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $quota_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {"volume" => "volume-name"},
    );

    %opts = %$quota_args;
    $apiset->quota_err_inject(%opts);
} ## end sub _err_inject_7mode_cli

=head2 scramble_conf

  NACL::C::VolumeQuota->scramble_conf(command_interface=>$ci,
                         vserver => $vserver_name,
                         volume  => $volume_name,
                         ...);

(Class method) scramble_conf.

Uses a 7Mode CLI APISet.

Invokes "quota scramble conf" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method)

See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< volume=>$volume_name >>

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub scramble_conf {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(params => \@args,);

    $Log->exit() if $may_exit;

    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_scramble_conf_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );
} ## end sub scramble_conf

sub _scramble_conf_7mode_cli {
    my $pkg  = shift;
    my %opts = validate_with(
        params              => \@_,
        spec                => $pkg->_backend_validate_spec(),
        scramble_conf_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $quota_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {"volume" => "volume-name"},
    );

    %opts = %$quota_args;
    $apiset->quota_scramble_conf(%opts);
} ## end sub _scramble_conf_7mode_cli

=head2 report

    my $report_obj = NACL::C::VolumeQuota->report(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @report_objs = NACL::C::VolumeQuota->report(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $report_obj = $VolumeQuota_obj->report();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-VolumeQuotaReport-pm/ATTRIBUTES>
of NACL::CS::VolumeQuotaReport as input.
This method returns objects of the type L<NACL::CS::VolumeQuotaReport|lib-NACL-CS-VolumeQuotaReport-pm>.

=cut

sub report {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::VolumeQuotaReport;

    #Need to flush remote cached data before the quota report for a FlexGroup.
    #ONTAP has some remote scrubbing built into the "quota report" command.
    #See burt995901 for more info. However the ONTAP builtin scrub is
    #asynchronous and may not finish prior to returning the first quota record.
    #
    #use_scrub: Use scrub or not for a FlexGroup. Defaults to 0.
    #Although use_scrub is not a CS-like fetch option, we need it to give users
    #a choice when necessary: do not use scrub. Also, we consider this option
    #as a back-door-like option since it is not a formal CS-like option
    #for the fetch method.
    my $use_scrub = 0;
    if( exists $opts{use_scrub} ) {
        $use_scrub = $opts{use_scrub};
        delete $opts{use_scrub};
    }

    my %common_opts;
    $pkg_or_obj->_copy_common_component_params(
        source => \%opts,
        target => \%common_opts
    );

    if ($use_scrub) {
        my $filter = $opts{filter};
        if (exists $filter->{volume} && exists $filter->{vserver}) {
            my $vol_obj = NACL::STask::Volume->new(
                command_interface => $opts{command_interface},
                vserver           => $filter->{vserver},
                volume            => $filter->{volume},
            );
            if ($vol_obj->get_one_state_attribute('is-flexgroup') eq 'true') {
                my @constituents = $vol_obj->get_flexgroup_constituents(
                    apiset_should => { interface => 'CLI' }, # for speed
                );
		$pkg_or_obj->_wait_for_remote_scrub(
		    %common_opts,
		    command_interface => $opts{command_interface},
		    vserver => $filter->{vserver},
		    volume => $filter->{volume},
		    flexgroup_constituents => \@constituents,
		);
            }
        }
    }

    my @state_objs = NACL::CS::VolumeQuotaReport->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub report

=head2 status

    my $status_obj = NACL::C::VolumeQuota->status(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);
                          
                    or
                    
    my @status_objs = NACL::C::VolumeQuota->status(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or
  
   my $status_obj = $VolumeQuota_obj->status(); 
                          
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-VolumeQuotaStatus-pm/ATTRIBUTES>
of NACL::CS::VolumeQuotaStatus as input. 
This method returns objects of the type L<NACL::CS::VolumeQuotaStatus|lib-NACL-CS-VolumeQuotaStatus-pm>.

=cut

sub status {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::VolumeQuotaStatus;
    my @state_objs = NACL::CS::VolumeQuotaStatus->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub status


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        volume  => {type => SCALAR},
        vserver => {type => SCALAR | UNDEF, optional => 1},

    );
} ## end sub _primary_keys_validate_spec

sub _exception_handler {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my ($exception_object) = validate_pos(
        @args,
        {   type => OBJECT,
            isa  => 'NACL::APISet::Exceptions::ResponseException'
        }
    );

    # CMode-CLI does not result in an error if quotas are already on/off
    # Suppress this particular exception
    if ($exception_object->text() !~ /Quotas are already/i) {
        $Log->exit() if $may_exit;
        $exception_object->throw();
    }

    $Log->exit() if $may_exit;
} ## end sub _exception_handler

sub _common_quota_zapi_check {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => {
            'command_interface' => {
                    type => OBJECT,
                    isa  => 'NACL::C::CommandInterface::ONTAP' },
        },
        allow_extra => 1,
    );
    $pkg_or_obj->_foreground_zapi_check(%opts);
    if ($opts{command_interface}->privilege_level() eq 'test') {
        # Some negative tests are checking failure messages when a ZAPI is
        # attempted against a FlexGroup constituent.  Thus, only consider
        # ZAPI an invalid interface for constituents if the CLI would allow
        # the constituent op (i.e., it's at test-level).
        NACL::CS::VolumeQuota->_check_flexgroup_constituent_like_volume(
            %opts,
            interface_description => 'ZAPI',
        );
    }
    $Log->exit() if $may_exit;
}

sub _modify_zapi_check {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my %modify_opts;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%modify_opts,
        copy   => [qw(logging logging-interval)]
    );
    my @keys = keys %modify_opts;
    if (@keys) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw('Could not use ZAPI '
                . 'since ZAPI can only modify the state but a request was made '
                . 'to modify the following attributes: '
                . join(', ', @keys)
                . "\n");
    } ## end if (@keys)

    $pkg_or_obj->_common_quota_zapi_check(%opts);

    $Log->exit() if $may_exit;
} ## end sub _modify_zapi_check


sub _wait_for_remote_scrub {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            flexgroup_constituents => { type => ARRAYREF },
        },
        allow_extra => 1,
    );

    # Issue initial quota report which triggers internal scrub.
    # Results are discarded...
    NACL::CS::VolumeQuotaReport->fetch(
        command_interface => $opts{command_interface},
        filter => {
            vserver => $opts{vserver},
            volume => $opts{volume},
        },
        allow_empty => 1,
        apiset_should => { interface => 'CLI' }, # for speed
    );

    # Wait for the ONTAP internal scrub to complete. All dirty
    # entries should be flushed.
    my $wex_find = "'flags&" . WRE_MASK_IS_DIRTY . "'";
    my $retries = 3;
  WAIT_SCRUB: {
        foreach my $constituent (@{$opts{flexgroup_constituents}}) {
            my $wex_scope = $constituent->vserver() . ':' . $constituent->volume();
            my @remote_entries = NACL::C::Volume->explore(
                command_interface => $opts{command_interface},
                format            => 'remote',
                scope             => $wex_scope,
                find              => $wex_find,
            );
            $retries--;
            if (scalar(@remote_entries)) {
                my $err_msg = "Found ".scalar(@remote_entries)." dirty remote entries.";
                if ($retries) {
                    $Log->comment($err_msg." $retries retries remaining...");
                    sleep(1);
                    goto WAIT_SCRUB;
                }
                NACL::Exceptions::VerifyFailure->throw($err_msg);
            }
        }
    }

    $Log->exit() if $may_exit;
}

1;
