#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary VolumeQuotaPolicyRule Component Module
## @author Dheeraj.K@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VolumeQuotaPolicyRule

=head1 DESCRIPTION

C<NACL::C::VolumeQuotaPolicyRule> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP VolumeQuotaPolicyRule applied on a volume.  A related class is
L<NACL::CS::VolumeQuotaPolicyRule|lib-NACL-CS-VolumeQuotaPolicyRule-pm>, which represents
the past state of an ONTAP VolumeQuotaPolicyRule.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

(Applicable for CMode-CLI and CMode-ZAPI) The name of the vserver, in which the quota policy
rule is created

=head2 volume

The name of the volume, for which the quota policy rule is created

=head2 qtree

Name of the qtree to which the quota rule applies. For user or group rules,
it can be the qtree name or "" if no qtree.  For tree type rules, this field must be "".

=head2 policy_name

Policy Name

=head2 type

Rule Type

=head2 target

Rule Target

=cut

package NACL::C::VolumeQuotaPolicyRule;
use strict;
use warnings;
use NACL::CS::VolumeQuotaPolicyRule;
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR UNDEF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::UnitNormalization;

=head1 METHODS

=head2 new

  my $quotapolicyrule = NACL::C::VolumeQuotaPolicyRule->new(command_interface=>$ci,
                                  vserver => $vserver_name,
                                  volume=> $volume_name,
                                  "policy-name" => $policy,
                                  type => $type,
                                  target => $target,
                                  qtree  => $qtree );

Return a new VolumeQuotaPolicyRule component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'policy_name',
    scalar => 'volume',
    scalar => 'type',
    scalar => 'target',
    scalar => 'qtree',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::C::QuotaPolicyRule'
    );
    $Log->exit() if $may_exit;
    return $isa;
}

=head2 create

  my $quotapolicyrule = NACL::C::VolumeQuotaPolicyRule->create(command_interface=>$ci,
                                      vserver => $vserver_name,
                                      volume=> $volume_name,
                                      "policy-name" => $policy,
                                      type => $type,
                                      target => $target);

(Class method) Request creation of a new quota policy rule element, and  return
a new quota policy rule component object that refers to it.

Uses a 7Mode ZAPI, CMode CLI/ZAPI APISet.

Invokes "quota-add-entry" API for CMode ZAPI, 7Mode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required for CMode-CLI and CMode-ZAPI, Not Applicable for 7Mode-ZAPI) The name of the
vserver, in which the quota policy rule is created

=item C<< volume=>$volume_name >>

(Required) The name of the volume, for which the quota policy rule is created

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: volume

7M ZAPI: volume

=item C<< "policy-name" => $string >>

(Required for CMode-CLI and CMode-ZAPI, Not Applicable for 7Mode-ZAPI) Name of the
quota policy .

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: policy

=item C<< "target" => $string >>

(Required) Qtree name

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: target

7M ZAPI: quota-target

=item C<< "qtree" => $qtree_name >>

Name of the qtree to which the quota rule applies. For user or group rules,
it can be the qtree name or "" if no qtree.  For tree type rules, this field must be "".

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: qtree

7M ZAPI: qtree

=item C<< "type" => $string >>

(Required) Possible values are tree or user or group

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: quota-type

7M ZAPI: quota-type

=item C<< "disk-limit" => $number >>

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: disk-limit

7M ZAPI: disk-limit

=item C<< "file-limit" => $number >>

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: file-limit

7M ZAPI: file-limit

=item C<< "threshold" => $number >>

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: threshold

7M ZAPI: threshold

=item C<< "soft-disk-limit" => $number >>

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: soft-disk-limit

7M ZAPI: soft-disk-limit

=item C<< "soft-file-limit" => $number >>

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: soft-file-limit

7M ZAPI: soft-file-limit

=item C<< '*' >>

This method takes all other options supported by
"volume quota policy rule create" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    my ($pkg, @args) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw (disk-limit file-limit user-mapping zapi-user-mapping
                    threshold soft-disk-limit soft-file-limit
                    )
            )
        },
        cli_cmd => 'volume quota policy rule create',
    );

    my $ret = $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_7mode_zapi',
                interface => 'ZAPI',
                set       => '7Mode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeQuotaPolicyRule::create',
        nacl_exceptions => [qw(AlreadyExists)],
    );

    if (!$opts{qtree}) {
        $opts{qtree} = '""';
    }

    if (!$opts{target}) {
        $opts{target} = '""';
    }
    my %new_hash = (
        command_interface => $opts{'command_interface'},
        policy_name       => $opts{'policy-name'},
        qtree             => $opts{'qtree'},
        target            => $opts{'target'},
        type              => $opts{'type'},
        volume            => $opts{'volume'},
    );

    if ($opts{command_interface}->is_cmode()) {
        $new_hash{vserver} = $opts{vserver};
    }

    return $pkg->new(%new_hash );



} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $vserver = $opts{vserver};

    if(!$opts{qtree}) {
        $opts{additional_args}->{qtree} = '""';
    }

    if(!$opts{target}) {
        $opts{additional_args}->{target} = '""';
    }

    $pkg->_base_backend_cmode_cli(
          %opts,
          api => 'volume_quota_policy_rule_create',
          vserver_operated_on => \$vserver,
    );

    $Log->exit() if $may_exit;
    return;
} ## end sub _create_cmode_cli

sub _create_7mode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my %allowed_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => [
            qw(volume qtree disk-limit file-limit threshold
                soft-disk-limit soft-file-limit)
        ],
        map => {
            "target" => "quota-target",
            "type"   => "quota-type",
        },
        target => \%allowed_args,
    );
    my %args;
    %args = %allowed_args;

    if (    ($args{'quota-target'} ne '*')
        and ($args{'quota-type'} eq 'tree'))
    {
        $args{'quota-target'}
            = "/vol/" . $args{volume} . "/" . $args{'quota-target'};
    } ## end if ( ( $args{'quota-target'...}))

    my $user_map_set = 0;
    my $apiset_cli;
    if (defined($opts{'user-mapping'})
        and ($args{'quota-type'} eq "user"))
    {
        $apiset_cli = $command_interface->apiset(
            category  => "Node",
            interface => "CLI",
            set       => "7Mode"
        );
        my $user_mapping_value = uc($opts{'user-mapping'});
        $apiset_cli->wrfile(
            append  => 1,
            content => "QUOTA_PERFORM_USER_MAPPING $user_mapping_value",
            file    => "/etc/quotas"
        );
        $user_map_set = 1;
    } ## end if ( defined( $opts{'user-mapping'...}))
    foreach my $array_element (qw(disk-limit soft-disk-limit threshold )) {
        if (defined $args{$array_element}) {
            if ($args{$array_element} =~ /^\d+(m|mb|g|gb)$/i) {
                my $temp = NACL::C::UnitNormalization::to_kilobytes(
                    $args{$array_element});
                $temp =~ s/(\d+)KB/$1/;
                $args{$array_element} = $temp;
            } ## end if ( $args{$array_element...})
        } ## end if ( defined $args{$array_element...})
    } ## end foreach my $array_element ...
    $apiset->quota_add_entry(%args);
    if ($user_map_set) {
        $apiset_cli->wrfile(
            append  => 1,
            content => "QUOTA_PERFORM_USER_MAPPING OFF",
            file    => "/etc/quotas"
        );
    } ## end if ($user_map_set)

    %opts = %allowed_args;

    $Log->exit() if $may_exit;
    return $pkg->new(
        qtree             => $opts{'qtree'},
        volume            => $opts{'volume'},
        type              => $opts{'quota-type'},
        target            => $opts{'quota-target'},
        command_interface => $command_interface
    );
} ## end sub _create_7mode_zapi

sub _create_cmode_zapi {
	$Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $vserver           = delete $opts{vserver};

    my $qpr_args = $pkg->_hash_copy(
        source => \%opts,
        copy   => [
            qw(volume qtree target disk-limit file-limit threshold soft-disk-limit soft-file-limit)
        ],
        map => {
            "user-mapping" => "perform-user-mapping",
            "policy-name"  => "policy",
            "type"         => "quota-type",

        },
    );

    %opts = %$qpr_args;

    $pkg->_handle_zapi_vserver_context(
        api_opts          => \%opts,
        vserver           => $vserver,
        command_interface => $command_interface
    );

    my %apiset_opts = $pkg->_zapi_input_translate_target(%opts);

    # CLI takes "on"/"off" but zapi accepts "true"/"false"
    # for user-mapping

    if (exists $apiset_opts{"perform-user-mapping"}) {
        if ($apiset_opts{"perform-user-mapping"} =~ /^on$/i) {
            $apiset_opts{"perform-user-mapping"} = "true";
        } else {
            $apiset_opts{"perform-user-mapping"} = "false";
        }
    } ## end if ( exists $opts{"perform-user-mapping"...})

    $apiset->quota_add_entry(%apiset_opts);
    $Log->exit() if $may_exit;
    
    return;
} ## end sub _create_cmode_zapi

=head2 delete

  NACL::C::VolumeQuotaPolicyRule->delete(command_interface=>$ci,
                         vserver => $vserver_name,
                         volume => $volume_name,
                         "policy-name" => $policy,
                         type => $type,
                         target => $target,
                         qtree  => $qtree);

                            or

  $quotapolicyrule_object->delete(...);

(Class or instance method) This method deletes an quota policy rule element.

Uses a 7Mode ZAPI, CMode CLI/ZAPI APISet.

Invokes "quota-delete-entry" API for CMode ZAPI, 7Mode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required for class method, Not Applicable for instance method)
Name of vserver that contains the volume.

Applicable for CMode CLI

=item C<< volume=>$volume_name >>

(Required for class method, Not Applicable for instance method and
Required for CMode-CLI and CMode-ZAPI, Not Applicable for 7Mode-ZAPI)
Name of volume that contains the qtree.

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: volume

7M ZAPI: volume

=item C<< "policy-name"=>$policy_name >>

(Required for class method, Not Applicable for instance method and
Required for CMode-CLI and CMode-ZAPI, Not Applicable for 7Mode-ZAPI)
Name of Quota Policy.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: policy

=item C<< type => $type >>

(Required for class method, Not Applicable for instance method)
Possible values are tree or user or group

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: quota-type

7M ZAPI: quota-type

=item C<< target => $target >>

(Required for class method, Not Applicable for instance method)
Qtree name

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: target

7M ZAPI: quota-target

=item C<< qtree => $qtree_name >>

Name of the qtree to which the quota rule applies. For user or group rules,
it can be the qtree name or "" if no qtree.  For tree type rules, this field must be "".

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: qtree

7M ZAPI: qtree

=item C<< '*' >>

This method takes all other options supported by
"volume quota policy rule delete" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {_optional_scalars(qw(qtree))},
        cli_cmd         => 'volume quota policy rule delete',
    );
    my $vserver ;
    $opts{vserver_operated_on} = \$vserver;

    my $ret = $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_7mode_zapi',
                interface => 'ZAPI',
                set       => '7Mode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeQuotaPolicyRule::delete',
        nacl_exceptions => [qw(DoesNotExist)],
    );
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    if(!$opts{qtree}) {
        $opts{additional_args}->{qtree} = '""';
    }

    if(!$opts{target}) {
        $opts{additional_args}->{target} = '""';
    }

    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api => 'volume_quota_policy_rule_delete',
    );

    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_7mode_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my %allowed_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(volume qtree)],
        map    => {
            "target" => "quota-target",
            "type"   => "quota-type",
        },
        target => \%allowed_args,
    );

    my $apiset_cli = $command_interface->apiset(
        category  => "Node",
        interface => "CLI",
        set       => "7Mode"
    );

    if ($opts{type} eq "user") {
        my $response = $apiset_cli->rdfile(file => "/etc/quotas");
        my $output   = $response->get_parsed_output();
        my $found    = 0;
        if ($output->[0]->{contents}) {
            my @rows = split(/\n/, $output->[0]->{contents});
            @rows = grep {/\w/} @rows;
            for (my $i = 1; $i < $#rows; $i++) {
                if (    $rows[$i] =~ /$opts{volume}(?:\/$opts{qtree})?/
                    and $rows[$i] =~ /\Q$opts{target}\E/)
                {
                    if (    $rows[$i - 1] =~ /QUOTA_PERFORM_USER_MAPPING/
                        and $rows[$i + 1]
                        =~ /QUOTA_PERFORM_USER_MAPPING\s+OFF/)
                    {
                        $rows[$i - 1] = "";
                        $rows[$i + 1] = "";
                        $found        = 1;
                        last;
                    } ## end if ( $rows[ $i - 1 ] =~...)
                } ## end if ( $rows[$i] =~ /$opts{volume}(?:\/$opts{qtree})?/...)
            } ## end for ( my $i = 1; $i < $#rows...)
            if ($found) {
                my $con = $apiset_cli->get_connection();
                $con->execute_start(command => "wrfile /etc/quotas");
                $con->send_EOF();
                $con->expect();

                $con->execute_start(command => "wrfile /etc/quotas");
                $con->send(join("\n", @rows));
                $con->send_EOF();
                $con->expect();

            } ## end if ($found)
        } ## end if ( $output->[0]->{contents...})
    } ## end if ( $opts{type} eq "user")

    %opts = %allowed_args;
    if ($opts{'quota-target'} ne '*' and $opts{'quota-type'} eq 'tree') {
        $opts{'quota-target'}
            = "/vol/" . $opts{volume} . "/" . $opts{'quota-target'};
    }
    $Log->debug(
        "Opts to API call 'quota_delete_entry' are:\n" . Dumper(\%opts));

    $apiset->quota_delete_entry(%opts);

    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_zapi

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj ,%opts)  = @_ ;
    $opts{'quota-type'} =  delete $opts{type} ;
    %opts = $pkg_or_obj->_zapi_input_translate_target(%opts);
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
                  api => 'quota-delete-entry',
                  copy => [qw(qtree volume quota-type quota-target)],
                  map  => {
                      "policy-name" => "policy",
                  },
    );
    $Log->exit() if $may_exit;

} ## end sub _delete_cmode_zapi

=head2 modify

  NACL::C::VolumeQuotaPolicyRule->modify(command_interface=>$ci,
                         vserver => $vserver_name,
                         volume  => $volume_name,
                         "policy-name" => $policy,
                         type => $type,
                         target => $target,
                         qtree  => $qtree);

                            or

  $quotapolicyrule_object->modify(...);

(Class or instance method) Modify a Quota Policy Rule element.

Uses a 7Mode ZAPI, CMode CLI/ZAPI APISet.

Invokes "quota-modify-entry" API for 7Mode ZAPI.

Invokes "quota-set-entry" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required for class method, Not Applicable for instance method and
Required for CMode-CLI and CMode-ZAPI, Not Applicable for 7Mode-ZAPI)
Name of vserver that contains the qtree.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< volume=>$volume_name >>

(Required for class method, Not Applicable for instance method)
Name of volume that contains the qtree.

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: volume

7M ZAPI: volume

=item C<< "policy-name"=>$policy_name >>

(Required for class method, Not Applicable for instance method and
Required for CMode-CLI and CMode-ZAPI, Not Applicable for 7Mode-ZAPI)
Name of Quota Policy.

=item C<< type => $type >>

(Required for class method, Not Applicable for instance method)
Possible values are tree or user or group

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: quota-type

7M ZAPI: quota-type

=item C<< target => $target >>

(Required for class method, Not Applicable for instance method)
Qtree name

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: target

7M ZAPI: quota-target

=item C<< qtree => $qtree_name >>

Name of the qtree to which the quota rule applies. For user or group rules,
it can be the qtree name or "" if no qtree.  For tree type rules, this field must be "".

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: qtree

7M ZAPI: qtree

=item C<< "disk-limit" => $number >>

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: disk-limit

7M ZAPI: disk-limit

=item C<< "file-limit" => $number >>

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: file-limit

7M ZAPI: file-limit

=item C<< "threshold" => $number >>

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: threshold

7M ZAPI: threshold

=item C<< "soft-disk-limit" => $number >>

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: soft-disk-limit

7M ZAPI: soft-disk-limit

=item C<< "soft-file-limit" => $number >>

Applicable for CMode CLI/ZAPI, 7Mode ZAPI.

Maps to:

CM ZAPI: soft-file-limit

7M ZAPI: soft-file-limit

=item C<< '*' >>

This method takes all other options supported by
"volume quota policy rule modify" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(qtree disk-limit file-limit user-mapping zapi-user-mapping
                    threshold soft-disk-limit soft-file-limit)
            )
        },
        cli_cmd => 'volume quota policy rule modify',
    );
    my $vserver ;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_entry_7mode_zapi',
                interface => 'ZAPI',
                set       => '7Mode',
            },
            {   method    => '_modify_entry_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeQuotaPolicyRule::modify',
    );
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $vserver = $opts{vserver};

    if(!$opts{qtree}) {
        $opts{additional_args}->{qtree} = '""';
    }

    if(!$opts{target}) {
        $opts{additional_args}->{target} = '""';
    }

    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api => 'volume_quota_policy_rule_modify',
        vserver_operated_on => \$vserver,
    );

    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_entry_7mode_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my %allowed_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [
            qw(volume qtree disk-limit file-limit threshold
                soft-disk-limit soft-file-limit )
        ],
        map => {
            "target" => "quota-target",
            "type"   => "quota-type",
        },
        target => \%allowed_args,
    );

    my $apiset_cli = $command_interface->apiset(
        category  => "Node",
        interface => "CLI",
        set       => "7Mode"
    );

    my $user_mapping = delete $opts{'user-mapping'};
    %opts = %allowed_args;
    if ($opts{'quota-target'} ne '*' and $opts{'quota-type'} eq 'tree') {
        $opts{'quota-target'}
            = "/vol/" . $opts{volume} . "/" . $opts{'quota-target'};
    }
    foreach my $array_element (qw(disk-limit soft-disk-limit threshold )) {
        if (defined $opts{$array_element}) {
            if ($opts{$array_element} =~ /^\d+(m|mb|g|gb)$/i) {
                my $temp = NACL::C::UnitNormalization::to_kilobytes(
                    $opts{$array_element});
                $temp =~ s/(\d+)KB/$1/;
                $opts{$array_element} = $temp;
            } ## end if ( $opts{$array_element...})
        } ## end if ( defined $opts{$array_element...})
    } ## end foreach my $array_element ...
    $apiset->quota_modify_entry(%opts);

    if (    defined($opts{'quota-type'})
        and $opts{'quota-type'} eq "user"
        and defined $user_mapping)
    {
        my $apiset_cli = $command_interface->apiset(
            category  => "Node",
            interface => "CLI",
            set       => "7Mode"
        );

        my $response = $apiset_cli->rdfile(file => "/etc/quotas");
        my $output   = $response->get_parsed_output();
        my $found    = 0;
        if ($output->[0]->{contents}) {
            my @rows = split(/\n/, $output->[0]->{contents});
            @rows = grep {/\w/} @rows;
            for (my $i = 1; $i < $#rows; $i++) {
                if (    $rows[$i] =~ /$opts{volume}|(?:\/$opts{qtree})?/
                    and $rows[$i] =~ /\Q$opts{'quota-target'}\E/i
                    and $rows[$i] =~ /$opts{'quota-type'}/)
                {
                    if ($rows[$i - 1] =~ /QUOTA_PERFORM_USER_MAPPING\s+OFF/
                        and $rows[$i + 1]
                        =~ /QUOTA_PERFORM_USER_MAPPING\s+OFF/
                        and $user_mapping eq "on")
                    {
                        $rows[$i - 1] = "QUOTA_PERFORM_USER_MAPPING ON";
                        $found = 1;
                        last;
                    } elsif (
                        $rows[$i - 1] =~ /QUOTA_PERFORM_USER_MAPPING\s+ON/
                        and $rows[$i + 1]
                        =~ /QUOTA_PERFORM_USER_MAPPING\s+OFF/
                        and $user_mapping eq "off")
                    {
                        $rows[$i - 1] = "QUOTA_PERFORM_USER_MAPPING OFF";
                        $found = 1;
                        last;
                    } ## end elsif ( $rows[ $i - 1 ] =~...)
                } ## end if ( $rows[$i] =~ ...)
            } ## end for ( my $i = 1; $i < $#rows...)

            if ($found) {
                my $con = $apiset_cli->get_connection();
                $con->execute_start(command => "wrfile /etc/quotas");
                $con->send_EOF();
                $con->expect();

                $con->execute_start(command => "wrfile /etc/quotas");
                $con->send(join("\n", @rows));
                $con->send_EOF();
                $con->expect();

            } ## end if ($found)
        } ## end if ( $output->[0]->{contents...})
    } ## end if ( defined( $opts{'quota-type'...}))

    $Log->exit() if $may_exit;
} ## end sub _modify_entry_7mode_zapi

sub _modify_entry_cmode_zapi{
    $Log->enter() if $may_enter;
    my ($pkg_or_obj ,%opts)  = @_ ;
    $opts{qtree}  = "" if ($opts{qtree}  eq '""');
    $opts{target} = "" if ($opts{target} eq '""');
    if ($opts{'type'} eq 'tree'
        and ($opts{qtree} ne "" or $opts{target} ne ""))
    {
        $opts{'quota-target'} = "/vol/" . $opts{volume} . "/" . $opts{target};
    } else {
        $opts{'quota-target'} = $opts{target};
    }
    delete $opts{target};

    # CLI takes "on"/"off" but zapi accepts "true"/"false"
    # for user-mapping

    if (exists $opts{"user-mapping"}) {
        if ($opts{"user-mapping"} =~ /^on$/i) {
            $opts{"user-mapping"} = "true";
        } else {
            $opts{"user-mapping"} = "false";
        }
    } ## end if ( exists $opts{"perform-user-mapping"...})
    $Log->debug("Opts to API call 'quota_set_entry' are:\n" . Dumper(\%opts));

    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
                  api => 'quota-set-entry' ,
                  copy   => [
                       qw(volume qtree disk-limit file-limit threshold quota-target
                          soft-disk-limit soft-file-limit)
                  ],
                  map => {
                     "policy-name"  => "policy",
                     "type"         => "quota-type",
                     "user-mapping" => "perform-user-mapping",
                  },
               );
    $Log->exit() if $may_exit;
} ## end sub _modify_entry_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'policy-name' => {type => SCALAR | UNDEF, optional => 1},
        volume        => {type => SCALAR},
        type          => {type => SCALAR},
        target        => {type => SCALAR},
        vserver       => {type => SCALAR | UNDEF, optional => 1},
        qtree         => {type => SCALAR | UNDEF, optional => 1},
    );
} ## end sub _primary_keys_validate_spec

sub _zapi_input_translate_target {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    # CLI takes '""' for default qtree But zapi takes ""
    # i.e <qtree></qtree>

    if (!$opts{qtree}) {
	$opts{qtree} = "";
    } elsif ($opts{qtree}  =~ /^\"(.*)\"$/) {
	$opts{qtree} = $1;
    }
    if (!$opts{target}) {
	$opts{target} = "";
    } elsif ($opts{target}  =~ /^\"(.*)\"$/) {
	$opts{target} = $1;
    }

    # For explicit tree rules, the qtree should be specified as
    # "/vol/< volume name >/ < qtree name >" except for default qtree

    if ($opts{'quota-type'} eq 'tree'
           and ($opts{qtree} !~ /^$/  or $opts{target} !~ /^$/))
    {
        $opts{'quota-target'} = "/vol/" . $opts{volume} . "/" . $opts{target};
    } else {

        # CLI takes '""' for default target but zapi takes in "".
        $opts{'quota-target'} = $opts{target};
    }
    delete $opts{'target'};
    $Log->exit() if $may_exit;
    return %opts;
}
1;
