#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifs Component Module
## @author Rupamjyoti.Baruah@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverCifs

=head1 DESCRIPTION

C<NACL::C::VserverCifs> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP Vserver CIFS.  A related class is
L<NACL::CS::VserverCifs|lib-NACL-CS-VserverCifs-pm>, which represents
the past state of an ONTAP Vserver CIFS

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

The name of the vserver, used to identify it uniquely.
The virtual server must already exist.

=cut

package NACL::C::VserverCifs;

use strict;
use warnings;
use NACL::CS::VserverCifs;
use NACL::CS::VserverCifsNbtstat;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR SCALARREF UNDEF ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::BaseException qw(:try);
use NACL::Exceptions::InvalidChoice;
use NACL::C::Exceptions::VserverCifs::AlreadyExists;
use NACL::C::Exceptions::VserverCifs::DoesNotExist;

=head1 METHODS

=head2 new

  my $vserver_cifs = NACL::C::VserverCifs->new(command_interface=>$ci,
                                               vserver =>$vserver_name);

Return a new Vserver CIFS component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

# XXX Many of the frontend methods accept 'username' and 'password' as
# arguments even though they're not sent to the API, they're required for
# answering prompts. We still need to investigate whether these values can
# be retrieved automatically from elsewhere (like the hostrec, for example).

# Earlier, 'username' and 'password' were primary keys of the component, but
# this does not fit our model of primary keys being only those options that are
# needed to uniquely identify the underlying element. Further, this sets a
# dangerous precedent of caching data, which we should never do.

=head2 create

  my $vserver_cifs = NACL::C::VserverCifs->create(command_interface=>$ci,
                                              vserver=>$vserver_name,...);

(Class method) Request creation of a new vserver cifs element, and  return a
new  vserver cifs component object that refers to it.  Does not wait for
vserver cifs creation to complete.

Uses a CMode CLI/ZAPI APISet.

Invokes "cifs-server-create" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required) This parameter specifies the name of the virtual server on which
to create the CIFS server. The virtual server must already exist.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< cifs-server=>$cifs_server >>

(Required) This parameter specifies the name of the CIFS server.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: cifs-server

=item C<< domain=>$domain >>

(Required) This parameter specifies the NetBIOS Domain/Workgroup Name.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: domain

=item C<< "password" => $string >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: admin-password

=item C<< "username" => $string >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: admin-username

=item C<< "admin-password" => $string >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: admin-password

=item C<< "admin-username" => $string >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: admin-username

=item C<< ou=>$org_unit >>

(Optional) This parameter specifies organizational unit within the Active
Directory domain to associate with the CIFS server.
By default, this parameter is set to CN=Computers.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: admin-username

=item C<< default-site => $default_site >>

(Optional) This parameter specifies the site within the Active Directory
domain to assopciate with the CIFS server if Data ONTAP can't determine
an appropriate site.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: default-site

=item C<< 'status-admin' => $string >>

Applicable for CMode CLI

=item C<< 'reuse_machine_account' => $string >>

(optional)
If this is set and a machine account with the same name as specified in
'cifs-server' exists in the Active Directory, it will be overwritten and
reused. The default value for this field is false.
Valid values are "y" or "n".

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: force-account-overwrite

=item C<< 'comment' => $string >>

CIFS Server Description
Applicable for CMode CLI

=item C<< 'force-account-overwrite' => $string >>

Force Overwrite possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'netbios-aliases' => [ $netbios-aliase1, $netbios-aliase2, ... ] >>

List of NetBIOS Aliases
(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< 'workgroup' => $string >>

Workgroup Name
Applicable for CMode CLI/ZAPI.

=item C<< 'nba' => [ $nba1, $nba2, ... ] >>

List of newly added or deleted NBAliases
(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< 'nba-add' => $string >>

add or delete Op for NBAliases possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs create" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "nba"             => {type => ARRAYREF, optional => 1},
            "netbios-aliases" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( cifs-server domain ou default-site
                    username password admin-password admin-username
                    status-admin reuse_machine_account
                    comment force-account-overwrite workgroup nba-add
                    )
            ),
        },
        api => 'vserver_cifs_create',
    );
    
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifs::create',
        nacl_exceptions => [ qw(AlreadyExists) ],
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $vserver
    );

    $Log->exit() if $may_exit;
    return $obj;

} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $apiset       = $opts{apiset};

    $pkg->_hash_copy(
        source => \%opts,
        target => \%opts,
        map    => {
            'admin-username' => 'username',
            'admin-password' => 'password',
        }
    );

    my $username              = delete $opts{'username'};
    my $password              = delete $opts{'password'};
    my $reuse_machine_account = delete $opts{'reuse_machine_account'};
    my $prompts_answers       = [
        'Enter the user name:'          => sub {$username},
        'Enter the password:'           => sub {$password},
        'Ok to reuse this account(.+):' => sub {$reuse_machine_account}
    ];
    
    my $confirmations_enabled;
    if (defined $reuse_machine_account && $reuse_machine_account =~ /n/i) {
        # With confirmations disabled, it will always reuse the account. To
        # get the prompt, we need to temporarily enable confirmations.
        $apiset->set(confirmations => 'on');
        $confirmations_enabled = 1;
    }
    
    try {
        $pkg->_base_backend_cmode_cli(
            %opts,
            api             => 'vserver_cifs_create',
            additional_args => {'connectrec-match_table' => $prompts_answers},
        );
    } finally {
        # Disable confirmations if it was enabled above.
        $apiset->set(confirmations => 'off') if ($confirmations_enabled);
    };

    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    my $allowed_cmode_options
        = [
        qw(cifs-server domain default-site admin-username admin-password workgroup netbios-aliases)
        ];
    my $map = {
        "ou"                    => "organizational-unit",
        "username"              => "admin-username",
        "password"              => "admin-password",
        "reuse_machine_account" => "force-account-overwrite"
    };

    my $reuse_machine_account = $opts{reuse_machine_account};
    if (defined $reuse_machine_account && $reuse_machine_account =~ /n/i) {
        $opts{reuse_machine_account} = 'false';
    } else {
        # reuse_machine_account provided as "y" or not provided
        $opts{reuse_machine_account} = 'true';
    }

    $pkg->_base_backend_cmode_zapi(
        %opts,
        api                 => 'cifs-server-create',
        copy                => $allowed_cmode_options,
        map                 => $map,
    );

    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

=head2 setup

  my $cifs = NACL::C::VserverCifs->setup(
                                      'command_interface' => $Node,
                                      'cifs-server' => $cifs_server,
                                      'workgroup' => $workgroup,
                                    );

                            or

  my $cifs = NACL::C::VserverCifs->setup(
                                      'command_interface' => $Node,
                                      'cifs-server' => $cifs_server,
                                      'auth-type' => "ad",
                                      'workgroup' => $workgroup,
                                      'domain' => $domain,
                                      'domain-user' => "administrator",
                                      'domain-pass' => "cifs*123",
                                      'ou' => "OU=7mode",
                                      'site-name' => "HiSec",
                                      'select-site-name' => "yes",
                                      'wins-option' => 1,
                                      'wins-servers' => "1.2.3.4",
                                );

(Class method, for 7Mode) This method is used for Cifs setup for 7Mode.

Uses a 7Mode CLI APISet.

Invokes "cifs setup" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< cifs-server=>$cifs_server >>

(Required) This parameter specifies the name of the CIFS server.

Applicable for 7Mode CLI

=item C<< "password" => $string >>

(Optional) The password for the cifs administrative user.

Applicable for 7Mode CLI

=item C<< ou=>$org_unit >>

(Optional) This parameter specifies organizational unit within the Active
Directory domain to associate with the CIFS server.
By default, this parameter is set to CN=Computers.

Applicable for 7Mode CLI

=item C<< default-site => $default_site >>

(Optional) This parameter specifies the site within the Active Directory
domain to associate with the CIFS server if Data ONTAP can't determine
an appropriate site.

Applicable for 7Mode CLI

=item auth-type

(Optional. For 7Mode only) : The authentication style that determines the
method by which clients will be authenticated when connecting
to this CIFS server.
Possible values: 'ad'(Active Directory), 'workgroup'(Workgroup),
'passwd'(/etc/passwd).  Default: 'workgroup'.
This routine does not support nt4 as yet. When it does 'nt4' will be
a possible value.

Note: The domain, domain-user and domain-pass need to be specified
if auth-type is ad.

Applicable for 7Mode CLI

=item workgroup

(Optional. For 7Mode only) : The workgroup of which the filer would be part of.
Defaults to "WORKGROUP" if not specified and auth-type is 'workgroup'.

Applicable for 7Mode CLI

=item domain

(Optional) : The domain name of which the filer would be a part of.
If the domain name is specified then the domain-user and domain-pass
must also be specified.

Applicable for 7Mode CLI

=item domain-user

(Optional. For 7Mode only) : The username on the domain with administrative
privileges to add a filer to it.

Applicable for 7Mode CLI

=item domain-pass

(Optional. For 7Mode only) : The password of the "domain-user".

Applicable for 7Mode CLI

=item root-pass

(Optional. For 7Mode only) : A scalar specifying the root password of the filer.
Defaults to the hostspec "default_password".

Applicable for 7Mode CLI

=item dns-enable

(Optional. For 7Mode only) : If defined will configure DNS when prompted by cifs setup.
If defined must provided 'dns-domainname' and 'dns-server-list' also.

Applicable for 7Mode CLI

=item dns-domainname

(Optional. For 7Mode only) : Filer's domain name. Must be defined if 'dns-enable' is defined.

Applicable for 7Mode CLI

=item dns-server-list

(Optional. For 7Mode only) : Comma separted list of DNS server IP(s).
Must be defined if dns-enable is defined.

Applicable for 7Mode CLI

=item timed-enable

(Optional. For 7Mode only) : If defined will configure time services when prompted by cifs setup.
If defined must provided 'timed-server-list' also.

Applicable for 7Mode CLI

=item timed-server-list

(Optional. For 7Mode only) : Comma separted list of time server names or IP(s).
Must be defined if timed-enable is defined.

Applicable for 7Mode CLI

=item passwd-group

(Optional. For 7Mode only) : If set then create the default /etc/passwd and/or /etc/group files.
Default is 'no'.

Applicable for 7Mode CLI

=item security-style

(Optional. For 7Mode only) : The security style determines whether or not the CIFS service will
support multiprotocol access. These styles are subject to change in a future
release. Possible values : 'ntfs','multiprotocol'. Default: 'multiprotocol'

Applicable for 7Mode CLI

=item create-admin

(Optional. For 7Mode only) : If set to 1 will create the CIFS administrator account if prompted.
Possible values : 'yes' / 'no'.  Default : 'yes'

Applicable for 7Mode CLI

=item select-site-name

(Optional. For 7Mode only) : If set to 'yes' will enable selection of the AD site name if prompted.
Possible values : 'yes' / 'no'. Default : 'yes'.

Applicable for 7Mode CLI

=item site-name

(Optional. For 7Mode only) : The name of the site to choose when select-site-name is yes. If
select-site-name = yes and site-name is not defined the Default-First-Site-Name
will be selected.

Applicable for 7Mode CLI

=item wins-option

(Optional) : Not supported when cifs.new_setup.enable = on. If set to 1 WINS
will be enabled using the existing WINS servers if they exist otherwise the
wins servers will be set to wins_server_list. If set to 2, WINS will be
enabled and any existing WINS servers will be overwritten with
wins_server_list. If set to 3 WINS will be disabled.
Possible values : 1,2,3. Default : 3.";

Applicable for 7Mode CLI

=item wins-servers

(Optional) : Not supported when cifs.new_setup.enable = on. A comma seperated
list of WINS servers.

Applicable for 7Mode CLI

=item local-admins

(Optional) : User or group ID to add to the BUILTIN\adminisrators group

Applicable for 7Mode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub setup {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( cifs-server domain ou password auth-type workgroup
                    domain-user domain-pass root-pass dns-enable dns-domainname
                    dns-server-list timed-enable timed-server-list passwd-group
                    security-style create-admin select-site-name site-name
                    wins-option wins-servers local-admins
                    )
            ),
        },
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_setup_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub setup

sub _setup_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $auth_type = _compute_auth_type(%opts);
    delete $opts{'auth-type'};
    my $wins_option      = delete $opts{'wins-option'} || "3";
    my $wins_servers     = delete $opts{'wins-servers'};
    my $root_pass        = delete $opts{'root-pass'};
    my $workgroup        = delete $opts{'workgroup'} || "WORKGROUP";
    my $create_admin     = delete $opts{'create-admin'} || "yes";
    my $domain_user      = delete $opts{'domain-user'};
    my $domain_pass      = delete $opts{'domain-pass'};
    my $cifs_server      = delete $opts{'cifs-server'};
    my $ou               = delete $opts{'ou'} || "CN=computers";
    my $security_style   = delete $opts{'security-style'} || "multiprotocol";
    my $select_site_name = delete $opts{'select-site-name'} || "yes";
    my $site_name    = delete $opts{'site-name'} || "Default-First-Site-Name";
    my $local_admins = delete $opts{'local-admins'};
    my $domain       = delete $opts{'domain'};
    my $password     = delete $opts{'password'};
    my $passwd_group = delete $opts{'passwd-group'} || "no";
    my $dns_enable   = delete $opts{'dns-enable'};
    my $dns_domainname    = delete $opts{'dns-domainname'};
    my $dns_server_list   = delete $opts{'dns-server-list'};
    my $timed_enable      = delete $opts{'timed-enable'};
    my $timed_server_list = delete $opts{'timed-server-list'};
    my $retype_pass;

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    # Setting default for root_pass
    my $conn = $apiset->get_connection();
    $root_pass = $conn->{password} unless ($root_pass);

    my $prompts_answers = [
        "Do you want to make the system visible via WINS\\? .*" =>
            sub { $wins_option =~ /[1-2]/ ? "yes" : "no" },
        "Do you wish to make the system visible via WINS\\?" => "no",
        "\\(1\\)(?:.|\n)Keep the current WINS(?:.|\n)*Selection \\(1-3\\)\?"
            => sub {$wins_option},
        "IPv4 address\\(es\\) of your WINS name server\\(s\\).*" =>
            sub {$wins_servers},
        "Would you like to specify additional WINS name servers" => "no",
        "Enter the password for the root user"                   => sub {
            $retype_pass = $root_pass;
            return $root_pass;
        },
        "Enter the password for root" => sub {
            $retype_pass = $root_pass;
            return $root_pass;
        },
        "Retype the password:" => sub {
            $retype_pass;
        },
        "(?s:The default name of this filer will be .*"
            . "Do you want to modify this name\\?)" => "no",
        "(?s:This filer will be a member of the Windows Workgroup .*"
            . "Do you want to modify this name\\?)" => sub {
            $workgroup ? "yes" : "no";
            },
        "(?s:This filer will be a member of the Workgroup .*"
            . "Do you want to modify this name\\?)" => sub {
            $workgroup ? "yes" : "no";
            },
        "Workgroup name for the filer\\?.*:" => sub {
            $workgroup;
        },
        "Enter the Workgroup for the filer.*:" => sub {
            $workgroup;
        },
        "Do you want to modify this list\\?" => "no",
        "Do you want to delete the existing filer account "
            . "information\\?" => "yes",
        "create the .*\\\\administrator account\\?" => sub {
            $create_admin;
        },
        "Create the .*\\\\administrator account\\?" => sub {
            $create_admin;
        },
        "Please enter the Windows.*user.*:" => sub {
            $domain_user;
        },
        "Enter the name of the Windows user" => sub {
            $domain_user;
        },
        "Password for.*:" => sub {
            $domain_pass;
        },
        "Do you wish to continue and join.*in NT4 mode\\?" => "",
        "Active Directory container for filer account\\?"  => sub {
            $ou;
        },
        "Do you want to continue and change the current filer account "
            . "information\\?" => 'yes',
        "Would you like to reconfigure this filer to be an NTFS-only "
            . "filer\\?" => sub {
            ($security_style eq 'ntfs') ? 'yes' : 'no';
            },
        "Do you want to configure this filer as a NTFS-only filer" => sub {
            ($security_style eq 'ntfs') ? 'yes' : 'no';
        },
        "What type of authentication will this filer use\\?" => sub {
            $auth_type;
        },
        "Data ONTAP CIFS services support four styles of user "
            . "authentication(?:.|\n)*Selection \\(1-4\\)\?" => sub {
            $auth_type;
            },
        "\\(1\\) Multiprotocol filer[\r|\n]*\\(2\\) NTFS-only "
            . "filer[\r|\n]*Selection \\(1-2\\)\?" => sub {
            ($security_style eq 'multiprotocol') ? 1 : 2;
            },
        "\\(1\\) NTFS-only filer[\r|\n]*\\(2\\) Multiprotocol "
            . "filer[\r|\n]*Selection \\(1-2\\)\?" => sub {
            ($security_style eq 'multiprotocol') ? 2 : 1;
            },
        "\\(1\\) Ignore Active Directory Site information.*" => sub {
            ($select_site_name eq 'no') ? 1 : 2;
        },

        # site-name
        "Choose a Site name from the following list of Sites(?:.|\n)*"
            . "(\\(1\\) Default-First-Site-Name(?:.|\n)*)"
            . "Selection \\(1-\\d+\\)\?" => sub {
            my $prompt = $1;
            return _find_value($prompt, $site_name);
            },    # default : Default-first-site-name
                  # ou-name
        "The user that you specified has permission to create(?:.|\n)*"
            . "(\\(1\\) CN=computers(?:.|\n)*)Selection \\(1-\\d+\\)" => sub {
            my $prompt = $1;
            return _find_value($prompt, $ou);
            },    # default : CN=computers
        "\\(3\\) Choose from a subset of containers by specifying a "
            . "search filter(?:.|\n)*Selection \\(1-3\\)" =>
            sub { $ou ? 3 : 1 },
        "Please enter some or all of a container specification to limit"
            . " the(?:.|\n)*Enter the search string" => sub {$ou},
        "\\(1\\) $ou(?:.|\n)*Selection \\(1-2\\)" => 1,
        "Would you like to change this name\\?" =>
            sub { $cifs_server ? 'yes' : 'no' },
        "Would you like to specify a user or group that can administer"
            . " CIFS" => sub { $local_admins ? 'yes' : 'no' },
        "Enter the name of a user or group that will administer CIFS "
            . "on the filer" => sub {$local_admins},
        "Enter the CIFS server name for the filer" => sub {$cifs_server},
        "Active Directory Domain name\\?"          => sub {$domain},
        "What is the name of the Active Directory domain" => sub {$domain},
        "Enter the Windows Domain for the filer.*:"       => sub {$domain},
        "Do you want to re-use this AD account\\?"   => "",    # default : yes
        "Do you want to re-use this machine account" => "",    # default : yes
        "Enter the new password for .*\\\\administrator" => sub {
            $retype_pass = $password;
            return $password;
        },
        "Set the password for .*\\\\administrator" => sub {
            $retype_pass = $password;
            return $password;
        },
        "Would you like to specify a user or group that can administer"
            . " CIFS\\?" => 'no',
        "What is the name of the Workgroup\\?" => sub {
            $workgroup;
        },
        ,
        "Should CIFS create a default /etc/passwd file" => sub {
            $passwd_group;
        },
        "Should CIFS create a default /etc/group file" => sub {
            $passwd_group;
        },
        "Should CIFS create default /etc/passwd and /etc/group files" => sub {
            $passwd_group;
        },
        "Would you like to enable NIS group caching" => "",    # default : yes
        "Enter the hour\\(s\\) when NIS should update the group cache" =>
            "",    # default : 24hrs
        "Would you like to specify additional hours" => "",    # default : no
        "Do you want to configure the filer\\'s DNS resolver service" =>
            sub { $dns_enable ? 'yes' : 'no' },
        "What is the filer\\'s DNS domain name" => sub {$dns_domainname},
        "What are the IPv4 address\\(es\\) of your authoritative DNS "
            . "name server\\(s\\)" => sub {$dns_server_list},
        "What are the IPv4/IPv6 address\\(es\\) of your authoritative "
            . "DNS name server\\(s\\)" => sub {$dns_server_list},
        "Would you like to specify additional DNS name servers" => "no",
        "Would you like to configure time services\\?" =>
            sub { $timed_enable ? 'yes' : 'no' },
        "Enter the time server host\\(s\\) and\\/or address\\(es\\)" =>
            sub {$timed_server_list},
        "Would you like to specify additional time servers\\?" => 'no',
        "Would you like to reconfigure this filer to be a multiprotocol"
            . " filer\\?" =>
            sub { ($security_style eq 'multiprotocol') ? 'yes' : 'no' },
        "Do you want to create the filer's machine account in" =>
            "",    #default : yes
        "Do you want to create the .*\\\\administrator account" => "no",
        "CIFS Setup cannot find a necessary DNS service "
            . "(?:.|\n)*Selection \\(1-3\\)\?" =>
            sub { $dns_server_list ? '2' : '3' }
    ];

    $opts{'connectrec-timeout'} ||= 600;

    $pkg->_api_call_log_debug_opts(
        api      => 'cifs_setup',
        api_opts => \%opts
    );

    # Call APISet cifs_setup
    $apiset->cifs_setup(%opts, 'connectrec-match_table' => $prompts_answers,);
    $Log->exit() if $may_exit;

    return $pkg->new(command_interface => $command_interface);
} ## end sub _setup_7mode_cli

=head2 delete

  NACL::C::VserverCifs->delete(command_interface=>$ci,
                             vserver=>$vserver_name,...);

                            or

  $vserver_cifs_object->delete(...);

(Class or instance method) This method deletes an vserver cifs element.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "cifs-server-delete" API for CMode ZAPI.

Invokes "cifs terminate" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required for class method, Not Applicable for instance method)
This parameter specifies the virtual server on which the CIFS server was
created.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< admin-password=>$admin_passwd >>

(Required) Password for admin user.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: admin-password

=item C<< admin-username=>$admin_user >>

(Required) Admin user name.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: admin-username

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs delete" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(username password admin-password admin-username workgroup)),
        },
        cli_cmd => 'vserver cifs delete',
    );

    my $ret = $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_delete_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        frontend => 'NACL::C::VserverCifs::delete',
        nacl_exceptions => [ qw(DoesNotExist) ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $apiset = delete $opts{apiset};
    $apiset->cifs_terminate();
    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_cli

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $vserver = $opts{vserver};
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%opts,
        map    => {
            'admin-username' => 'username',
            'admin-password' => 'password',
        }
    );

    my $username = delete $opts{'username'};
    my $password = delete $opts{'password'};

    my $prompts_answers = [
        'Enter the user name:' => sub {$username},
        'Enter the password:'  => sub {$password}
    ];
    $pkg_or_obj->_base_backend_cmode_cli(
            %opts,
            api     => 'vserver_cifs_delete',
            vserver_operated_on => \$vserver,
            additional_args => {'connectrec-match_table' => $prompts_answers, },
        );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    my $vserver = $opts{vserver};
    $pkg_or_obj->_base_backend_cmode_zapi(
        api => 'cifs-server-delete',
        %opts,
        map    => {
            "username" => "admin-username",
            "password" => "admin-password",
        },
        vserver_operated_on => \$vserver,
        additional_args => {'force-account-delete'=>'true'},
    );
    $Log->exit() if $may_exit;

} ## end sub _delete_cmode_zapi

=head2 modify

   NACL::C::VserverCifs->modify(
                                command_interface => $ci,
                                     vserver => $vserver,
                                  ...);

                                                                 or

    $VserverCifs_object->modify(...);

(Class or instance method)
This method is used to modify a CIFS server.

Uses a CMode CLI/ZAPI APISet.

Invokes "cifs-server-modify" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "default-site" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: default-site

=item C<< 'status-admin' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: administrative-status

=item C<< 'domain' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: domain

Fully Qualified Domain Name
Applicable for CMode CLI

=item C<< 'comment' => $string >>

CIFS Server Description
Applicable for CMode CLI

=item C<< 'admin-password' => $string >>

Account Creation Password
Applicable for CMode CLI/ZAPI.

=item C<< 'admin-username' => $string >>

Account Creation Username
Applicable for CMode CLI/ZAPI.

=item C<< 'workgroup' => $string >>

Workgroup Name
Applicable for CMode CLI/ZAPI.

=item C<< 'nba' => [ $nba1, $nba2, ... ] >>

List of newly added or deleted NBAliases
(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< 'nba-add' => $string >>

add or delete Op for NBAliases possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "nba" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( default-site

                    status-admin

                    domain

                    comment

                    admin-password admin-username

                    workgroup

                    nba-add
                    )
            ),
        },
        api => 'vserver_cifs_modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifs::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(%opts, api => 'vserver_cifs_modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api             => 'cifs_server_modify',
        copy            => [qw( default-site domain workgroup admin-username admin-password cifs-server )],
        map             => {'status-admin' => 'administrative-status',},
        frontend_method => 'modify'
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi



=head2 password_change

   NACL::C::VserverCifs->password_change(
                                command_interface => $ci,
                                     vserver => $vserver,
                                  ...);

                                                                 or

    $VserverCifs_object->password_change(...);

(Class or instance method)
This method is used to change a server's domain account password.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "cifs-password-change" API for CMode ZAPI.

Invokes "cifs changefilerpwd" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs password-change" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub password_change {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver cifs password-change',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_password_change_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_password_change_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_password_change_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        frontend => 'NACL::C::VserverCifs::password_change',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub password_change

sub _password_change_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'vserver_cifs_password_change',
    );
    $Log->exit() if $may_exit;
} ## end sub _password_change_cmode_cli

sub _password_change_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api     => 'cifs_password_change',
    );
    $Log->exit() if $may_exit;
} ## end sub _password_change_cmode_zapi

sub _password_change_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    delete $opts{vserver};

    $apiset->cifs_changefilerpwd(%opts);
    $Log->exit() if $may_exit;
} ## end sub _password_change_7mode_cli

=head2 password_reset

   NACL::C::VserverCifs->password_reset(
                                command_interface => $ci,
                                     vserver => $vserver,
                                  ...);

                                                                 or

    $VserverCifs_object->password_reset(...);

(Class or instance method)
This method is used to reset a server's domain account password.

Uses a CMode CLI/ZAPI APISet.

Invokes "cifs-password-reset" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs password-reset" command in ONTAP.

Applicable for CMode CLI.

=item C<< admin-password=>$admin_passwd >>

(Required) Password for admin user.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: admin-password

=item C<< admin-username=>$admin_user >>

(Required) Admin user name.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: admin-username

=item C<< "username" => $string >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: admin-username

=item C<< "password" => $string >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: admin-password

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub password_reset {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(username password admin-password admin-username)),
        },
        cli_cmd => 'vserver cifs password-reset',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_password_reset_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_password_reset_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifs::password_reset',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub password_reset

sub _password_reset_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    my $username = $opts{'username'};
    my $password = $opts{'password'};

    my $prompts_answers = [
        '.*your user.*'     => $username,
        '.*your password.*' => $password
    ];

    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api             => 'vserver_cifs_password_reset',
        map             => {
            'admin-username' => 'username',
            'admin-password' => 'password',
        },
        additional_args => {'connectrec-match_table' => $prompts_answers},
    );

    $Log->exit() if $may_exit;
} ## end sub _password_reset_cmode_cli

sub _password_reset_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $allowed_cmode_options = [
        qw( admin-username admin-password
            )
    ];

    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api    => 'cifs_password_reset',
        copy   => $allowed_cmode_options,
        map    => {
            "username" => "admin-username",
            "password" => "admin-password",
        },
    );
    $Log->exit() if $may_exit;
} ## end sub _password_reset_cmode_zapi

=head2 show_sid

   NACL::C::VserverCifs->show_sid(
                                command_interface => $ci,
                                     vserver => $vserver,
                                  ...);

                                                                 or

    $VserverCifs_object->show_sid(...);

(Class or instance method)
This method is used to show the Windows SID for Vservers.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI

=item C<< "windows-sid" => $string >>

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs show-sid" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub show_sid {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( windows-sid
                    )
            ),
        },
        api => 'vserver_cifs_show_sid',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_show_sid_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifs::show_sid',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub show_sid

sub _show_sid_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $vserver = $opts{vserver};

    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api => 'vserver_cifs_show_sid',
        vserver_operated_on => \$vserver,
    );

    $Log->exit() if $may_exit;
} ## end sub _show_sid_cmode_cli

=head2 nbtstat

    my @cifs_nbtstat = NACL::C::VserverCifs->nbtstat(%opts);

                                          or

    my $cifs_nbtstat = NACL::C::VserverCifs->nbtstat(%opts);

(Class method only)
This method is used to display NetBIOS information over TCP connection. It returns objects of the
type L<NACL::CS::VserverCifsNbtstat|lib-NACL-CS-VserverCifsNbtstat-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch>
(requested_fields, filter, allow_empty) and has all of the other properties of
fetch methods (callable in scalar or list context, throws a NoElementsFound
exception if no matching elements are found)

=cut

sub nbtstat {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my @state_objs = NACL::CS::VserverCifsNbtstat->fetch(@_);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub nbtstat

=head2 start

    NACL::C::VserverCifs->start(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);

								 or

    $VserverCifs_obj->start(...);

(Class or instance method)
This method is used to start a CIFS server.

Uses a CMode CLI/ZAPI APISet.

Invokes "cifs-server-start" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs start" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver cifs start',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_start_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifs::start',
    );

    $Log->exit() if $may_exit;
} ## end sub start

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,
        api     => 'vserver_cifs_start',
    );
    $Log->exit() if $may_exit;
} ## end sub _start_cmode_cli

sub _start_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_zapi(@args,
        api     => 'cifs_server_start',
    );
    $Log->exit() if $may_exit;
}

=head2 stop

    NACL::C::VserverCifs->stop(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);

								 or

    $VserverCifs_obj->stop(...);

(Class or instance method)
This method is used to stop a CIFS server.

Uses a CMode CLI/ZAPI APISet.

Invokes "cifs-server-stop" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs stop" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver cifs stop',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_stop_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_stop_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifs::stop',
    );

    $Log->exit() if $may_exit;
} ## end sub stop

sub _stop_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'vserver_cifs_stop',
    );
    $Log->exit() if $may_exit;
} ## end sub _stop_cmode_cli

sub _stop_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api     => 'cifs_server_stop',
    );
    $Log->exit() if $may_exit;
}

=head2 access

    NACL::C::VserverCifs->access(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to manage access rights on a CIFS share..

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'share-name-delete' => $string >>

CIFS Share Name possible value(s) are, Share

Applicable for CMode CLI

=item C<< 'user-or-group-add' => $string >>

User or Group possible value(s) are, user_or_group_name

Applicable for CMode CLI

=item C<< 'share-name-add' => $string >>

CIFS Share Name possible value(s) are, Share

Applicable for CMode CLI

=item C<< 'user-or-group-delete' => $string >>

User or Group possible value(s) are, user_or_group_name

Applicable for CMode CLI

=item C<< 'delete' => $string >>

Delete Access Rights possible value(s) are, no_type

Applicable for CMode CLI

=item C<< 'permission' => $string >>

Access Rights possible value(s) are, access rights

Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs access" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub access {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( share-name-delete user-or-group-add share-name-add user-or-group-delete
                    delete permission
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd                 => 'vserver cifs access',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_access_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifs::access',
    );

    $Log->exit() if $may_exit;
} ## end sub access

sub _access_cmode_cli {
     $Log->enter() if $may_enter;
     my ($pkg, @opts) = @_;

     $pkg->_base_backend_cmode_cli(
         @opts,
         api => 'vserver_cifs_access',
     );

     $Log->exit() if $may_exit;
} ## end sub _access_cmode_cli

=head2 audit

    my $audit_obj = NACL::C::VserverCifs->audit(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @audit_objs = NACL::C::VserverCifs->audit(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $audit_obj = $VserverCifs_obj->audit(); 

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-VserverCifsAudit-pm/ATTRIBUTES>
of NACL::CS::VserverCifsAudit as input. 
This method returns objects of the type L<NACL::CS::VserverCifsAudit|lib-NACL-CS-VserverCifsAudit-pm>.

=cut

sub audit {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::VserverCifsAudit;
    my @state_objs = NACL::CS::VserverCifsAudit->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 shares

    NACL::C::VserverCifs->shares(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to configure and display CIFS shares on the Vserver.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'share-name-delete' => $string >>

CIFS Share Name possible value(s) are, Share

Applicable for CMode CLI

=item C<< 'add' => $string >>

Add CIFS Share possible value(s) are, no_type

Applicable for CMode CLI

=item C<< 'dir_umask' => $string >>

Directory Mode Creation Mask possible value(s) are, Octal Permission

Applicable for CMode CLI

=item C<< 'y-n-file_umask' => $string >>

Set or Unset File Mode Creation Mask possible value(s) are, -file_umask <Octal Integer>,-nofile_umask

Applicable for CMode CLI

=item C<< 'file_umask' => $string >>

File Mode Creation Mask possible value(s) are, Octal Permission

Applicable for CMode CLI

=item C<< 'y-n-browse' => $string >>

Enable or Disable Browsing possible value(s) are, -browse,-nobrowse

Applicable for CMode CLI

=item C<< 'share-name-show' => $string >>

CIFS Share Name possible value(s) are, Share

Applicable for CMode CLI

=item C<< 'nobrowse' => $string >>

Disable Browsing possible value(s) are, no_type

Applicable for CMode CLI

=item C<< 't' => $string >>

Display Summary possible value(s) are, no_type

Applicable for CMode CLI

=item C<< 'change' => $string >>

Change CIFS Share possible value(s) are, no_type

Applicable for CMode CLI

=item C<< 'path' => $string >>

CIFS Share Path possible value(s) are, cifs_share_path

Applicable for CMode CLI

=item C<< 'y-n-dir_umask' => $string >>

Set or Unset Directory Mode Creation Mask possible value(s) are, -dir_umask <Octal Integer>,-nodir_umask

Applicable for CMode CLI

=item C<< 'share-name-change' => $string >>

CIFS Share Name possible value(s) are, Share

Applicable for CMode CLI

=item C<< 'share-name-add' => $string >>

CIFS Share Name possible value(s) are, Share

Applicable for CMode CLI

=item C<< 'comment' => $string >>

CIFS Share Comment possible value(s) are, text

Applicable for CMode CLI

=item C<< 'delete' => $string >>

Delete CIFS Share possible value(s) are, no_type

Applicable for CMode CLI

=item C<< 'y-n-comment' => $string >>

CIFS Share Comment possible value(s) are, -comment <text>,-nocomment

Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'accessbasedenum' => $string >>

Access Based Enumeration
Applicable for CMode CLI

=item C<< 'y-n-accessbasedenum' => $string >>


Enable or Disable Access Based Enumeration possible value(s) are, -accessbasedenum , -noaccessbasedenum
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs shares" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub shares {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( share-name-delete add dir_umask y-n-file_umask file_umask y-n-browse
                    share-name-show nobrowse t change path y-n-dir_umask share-name-change
                    share-name-add comment delete y-n-comment

                    accessbasedenum y-n-accessbasedenum
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'vserver_cifs_shares',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_shares_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifs::shares',
    );

    $Log->exit() if $may_exit;
} ## end sub shares

sub _shares_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;

    $pkg->_base_backend_cmode_cli(
        @opts,
        api => 'vserver_cifs_shares',
    );

    $Log->exit() if $may_exit;
} ## end sub _shares_cmode_cli

=head2 domain_discovered_servers_reset_servers

    NACL::C::VserverCifs->domain_discovered_servers_reset_servers(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverCifs_obj->domain_discovered_servers_reset_servers(...);

(Class or instance method)
This method is used to reset and rediscover servers for a Vserver.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by "vserver cifs domain discovered servers reset servers" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub domain_discovered_servers_reset_servers {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            {'cmd_execution_time' => {type => SCALARREF, optional => 1},},
        cli_cmd => 'vserver cifs domain discovered-servers reset-servers',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method =>
                    '_domain_discovered_servers_reset_servers_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend =>
            'NACL::C::VserverCifs::domain_discovered_servers_reset_servers',
    );

    $Log->exit() if $may_exit;
}

sub _domain_discovered_servers_reset_servers_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    $pkg_or_obj->_base_backend_cmode_cli(
        @args,
        api => 'vserver_cifs_domain_discovered_servers_reset_servers',
    );

    $Log->exit() if $may_exit;
} ## end sub _domain_discovered_servers_reset_servers_cmode_cli

=head2 add_netbios_aliases

    NACL::C::VserverCifs->add_netbios_aliases(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverCifs_obj->add_netbios_aliases(...);

(Class or instance method)
This method is used to add NetBIOS Aliases.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI

=item C<< 'netbios-aliases' => [ $netbios-aliase1, $netbios-aliase2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "vserver cifs add netbios aliases" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add_netbios_aliases {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            {'netbios-aliases' => {type => ARRAYREF, optional => 1},},
        api => 'vserver_cifs_add_netbios_aliases',
    );
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_add_netbios_aliases_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifs::add_netbios_aliases',
        nacl_exceptions => [ qw(AlreadyExists) ],
    );

    $Log->exit() if $may_exit;
}

sub _add_netbios_aliases_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_add_netbios_aliases',);
    $Log->exit() if $may_exit;
}

=head2 remove_netbios_aliases

    NACL::C::VserverCifs->remove_netbios_aliases(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverCifs_obj->remove_netbios_aliases(...);

(Class or instance method)
This method is used to remove NetBIOS Aliases.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI

=item C<< 'netbios-aliases' => [ $netbios-aliase1, $netbios-aliase2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "vserver cifs remove netbios aliases" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove_netbios_aliases {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            {'netbios-aliases' => {type => ARRAYREF, optional => 1},},
        api => 'vserver_cifs_remove_netbios_aliases',
    );
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_remove_netbios_aliases_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifs::remove_netbios_aliases',
        nacl_exceptions => [ qw(DoesNotExist) ]
    );

    $Log->exit() if $may_exit;
}

sub _remove_netbios_aliases_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_remove_netbios_aliases',);
    $Log->exit() if $may_exit;
}

=head2 domain_password_change

    NACL::C::VserverCifs->domain_password_change(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverCifs_obj->domain_password_change(...);

(Class or instance method)
This method is used to generate a new password for the CIFS server's machine account and change it in the Windows Active Directory domain..
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "vserver cifs domain password change" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub domain_password_change {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'vserver_cifs_domain_password_change',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_domain_password_change_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifs::domain_password_change',
    );

    $Log->exit() if $may_exit;
}

sub _domain_password_change_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_domain_password_change',);
    $Log->exit() if $may_exit;
}

=head2 domain_password_reset

    NACL::C::VserverCifs->domain_password_reset(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverCifs_obj->domain_password_reset(...);

(Class or instance method)
This method is used to reset the CIFS server's machine account password in the Windows Active Directory domain..
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI

=item C<< "username" => $string >>

(Required)

Applicable for CMode CLI.

=item C<< "password" => $string >>

(Required)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by "vserver cifs domain password reset" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub domain_password_reset {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {_optional_scalars(qw(username password)),},
        api             => 'vserver_cifs_domain_password_reset',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_domain_password_reset_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifs::domain_password_reset',
    );

    $Log->exit() if $may_exit;
}

sub _domain_password_reset_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $prompts_answers = [
        qr/Enter\s+\w+\s+user\s+ID:\s*/i => sub { $opts{'username'} },
        qr/Enter\s+\w+\s+password:\s*/i  => sub { $opts{'password'} },
    ];
    my %additional_args;
    $additional_args{'connectrec-match_table'} = $prompts_answers;

    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api             => 'vserver_cifs_domain_password_reset',
        additional_args => \%additional_args,
    );
    $Log->exit() if $may_exit;
}

=head2 top

    NACL::C::VserverCifs->top(
                                command_interface => $ci,
                                ...);
   
(Class method)
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< '*' >>

This method takes all other options supported by "vserver cifs top" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub top {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        api                 => 'vserver_cifs_top',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_top_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifs::top',
    );

    $Log->exit() if $may_exit;
}

sub _top_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'vserver_cifs_top',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (vserver => {type => SCALAR | UNDEF, optional => 1},);
}

sub _compute_auth_type(%) {

    my (%opts) = @_;
    my $flag = 0;
    my $auth_type;
    my $apiset = $opts{apiset};

    # Since we have two varieties of cifs setup old/new this is done.
    my $res = $apiset->execute_raw_command(
        'command' => "options cifs.new_setup.enable");
    if (   ($res =~ /No such option cifs.new_setup.enable/)
        || ($res =~ /cifs.new_setup.enable\s+off/)
        || ($res =~ /option not found/))
    {
        $flag = 1;
    } ## end if ( ( $res =~ /No such option cifs.new_setup.enable/...

# If auth_type is not specified, return the default option value(i.e. Windows Workgroup)
    if (!defined $opts{'auth-type'}) {
        return ($flag) ? 2 : 3;
    }

    if ($opts{'auth-type'} eq 'passwd') {

        # dont know of anything that depends on this auth style
        $auth_type = ($flag) ? 3 : 4;
    } elsif ($opts{'auth-type'} eq 'ad') {

        # check if domain name etc are also specified

        NATE::Exceptions::Argument->throw("Missing Argument(s) "
                . "All the parameters 'domain', 'domain-user', "
                . "'domain-pass' are required, when 'auth-type' is 'ad'")
            if (!$opts{'domain'}
            || !$opts{'domain-user'}
            || !$opts{'domain-pass'});
        $auth_type = 1;
    } elsif ($opts{'auth-type'} eq 'nt4') {

        # this is not supported as yet
        $auth_type = ($flag) ? "" : 2;
    } else {
        $auth_type = ($flag) ? 2 : 3;
    }

    return ($auth_type);
} ## end sub _compute_auth_type(%)

sub _find_value {
    my ($prompt, $name) = @_;
    my $found = 0;
    my $reply;
    foreach my $line (split(/\r\n/, $prompt)) {
        chomp($line);
        if ($line =~ /\((\d+)\)\s+$name/) {
            $found = 1;
            $reply = $1;
        }
    } ## end foreach my $line ( split( /\r\n/...
    if (!$found) {
        $Log->warn("$name not found. Using defaults");
    }
    return $reply;
} ## end sub _find_value

sub can {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my @opts       = $pkg_or_obj->_can_validation(@_);
    my $code;
    if ($code = $pkg_or_obj->NACL::C::Component::can(@opts)) {
        $Log->exit() if $may_exit;
        return $code;
    } else {
        $code = $pkg_or_obj->_handle_component_aggregation(
            method    => $opts[0],
            error_ptr => $opts[1],
            table     => [
                {   component => 'NACL::C::VserverCifsSecurity',
                    prefix    => 'security'
                }
            ]
        );
        if ($code) {
            $Log->exit() if $may_exit;
            return $code;
        } elsif ($code = $pkg_or_obj->SUPER::can(@opts)) {
            $Log->exit() if $may_exit;
            return $code;
        }
    }

    $Log->exit() if $may_exit;
    return undef;
}

1;
