# Copyright (c) 2001-2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsDomainPreferredDc Component Module
## @author dl-nacl-dev@netapp.com , benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverCifsDomainPreferredDc

=head1 DESCRIPTION

C<NACL::C::VserverCifsDomainPreferredDc> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It can be used to manage
preferred domain controllers for vservers. A related class is
L<NACL::CS::VserverCifsDomainPreferredDc|lib-NACL-CS-VserverCifsDomainPreferredDc-pm>,
which represents the past state of an ONTAP VserverCifsDomainPreferredDc.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 domain

This is the fully-qualified name of the domain that the domain controllers
belong to.

=head2 vserver

This is the name of the Vserver whose preferred domain controllers are being
managed.

=head2 lif

This is not always present in the object. It is only present for builds
prior to change number 1236498
(i.e. which do *not* have UI change 'vserver-cifs-domain-preferred-dc-v2')

=cut

package NACL::C::VserverCifsDomainPreferredDc;

use strict;
use warnings;

use NACL::CS::VserverCifsDomainPreferredDc;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NACL::C::Exceptions::Component qw(:try);
use NACL::C::Exceptions::VserverCifsDomainPreferredDc::DoesNotExist ();
use NACL::APISet::Exceptions::ResponseException qw(:try);

=head1 METHODS

=head2 new

  my $VserverCifsDomainPreferredDc = NACL::C::VserverCifsDomainPreferredDc->new(command_interface=>$ci,
                                   domain =>$domain,
                                   vserver =>$vserver,
                                   [lif =>$lif,]
  );

Return a new VserverCifsDomainPreferredDc component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'domain',
    scalar => 'vserver',
    scalar => 'lif',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverCifsDomainPreferredDc  = NACL::C::VserverCifsDomainPreferredDc->find(command_interface => $ci, ...);
my @VserverCifsDomainPreferredDcs = NACL::C::VserverCifsDomainPreferredDc->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $VserverCifsDomainPreferredDc_state = $NACL::C::VserverCifsDomainPreferredDc->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverCifsDomainPreferredDc would return
a NACL::CS::VserverCifsDomainPreferredDc object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 add

    my $vs_cifs_dom_pref_dc= NACL::C::VserverCifsDomainPreferredDc->add(
                                command_interface => $ci,
                            domain => $domain,
                            vserver => $vserver,
                            [lif => $lif,]
                                  ...);

(Class method or instance method)
This method is used to add one or more domain controllers to be used in
preference to all others by the specified Vserver for interactions with the
specified domain. If a list already exists for the specified domain, the new
list will be merged with the existing list.

When called as a class method, a new object is created and returned to
the user. When invoked as an instance method (i.e. to add to an already
existing list), it returns the same object.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "cifs-domain-preferred-dc-add" API for CMode ZAPI.

Invokes "cifs prefdc add" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< domain => $domain >>

(Required for class method, Not Applicable for instance method)
This parameter specifies the fully-qualified name of the domain that the domain
controllers belong to.

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI: domain

7M CLI : domain name (Programmatic name: "domain")

=item C<< vserver => $vserver >>

(Required for class method, Not Applicable for instance method)
This parameter specifies the name of the Vserver for which to add the list of
preferred domain controllers.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
         option.

=item C<< lif => $lif >>

(Required for builds not containing UI change 'vserver-cifs-domain-preferred-dc-v2',
for builds containing this UI change this should not be specified)

Determining whether a build has this UI change can be done by:
my $version_manager = $command_interface->get_version_manager();
if ($version_manager->has_change(uichange => 'vserver-cifs-domain-preferred-dc-v2')) {
    # Has UI change, "lif" parameter should not be sent
} else {
    # Does not have UI change, "lif" parameter should be sent
}

Applicable for CMode CLI.

=item C<< 'preferred-dc' => [ $preferred-dc1, $preferred-dc2, ... ] >>

(Arrayref)
This arrayref specifies a list of IP addresses for domain controllers that
belong to the domain specified in the 'domain' parameter.

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI: preferred-dc

7M CLI : preferred Domain Controller (Programmatic name: "preferred-dc")

=item C<< 'lif-id' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs domain preferred-dc add" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( lif-id lif
                    )
            ),
            'preferred-dc' => {type => ARRAYREF},
        },
        cli_cmd => 'vserver cifs domain preferred-dc add',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $obj = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_add_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_add_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_add_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        frontend => 'NACL::C::VserverCifsDomainPreferredDc::add',
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub add

=head2 create

This is an alias to <add|lib-NACL-C-VserverCifsDomainPreferredDc-pm/add>.
It has been retained for backwards compatibility, it is suggested that
'add' is used instead.

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $Log->warn("The method 'add' should be used. This method has "
            . 'been retained for backwards compatibility');
    my $obj = $pkg_or_obj->add(@_);
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _add_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'vserver_cifs_domain_preferred_dc_add',);
    if (ref $pkg_or_obj) {
        $Log->exit() if $may_exit;
        return $pkg_or_obj;
    } else {
        $Log->exit() if $may_exit;
        return $pkg_or_obj->new(
            command_interface => $opts{command_interface},
            domain            => $opts{'domain'},
            vserver           => ${$opts{vserver_operated_on}},
            lif               => $opts{'lif'},
        );
    }
} ## end sub _add_cmode_cli

sub _add_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        copy => [qw(domain preferred-dc)],
        api  => 'cifs_domain_preferred_dc_add',
    );
    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface => $opts{command_interface},
        domain            => $opts{'domain'},
        vserver           => ${$opts{vserver_operated_on}},
        lif               => $opts{'lif'},
    );
} ## end sub _add_cmode_zapi

sub _add_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my %mode7_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw( domain preferred-dc )],
        target => \%mode7_args,
    );

    if (defined $mode7_args{'preferred-dc'}) {
        $mode7_args{'preferred-dc'} = join ' ',
            @{$mode7_args{'preferred-dc'}};
    }

    $apiset->cifs_prefdc_add(%mode7_args);

    if (ref $pkg_or_obj) {
        $Log->debug(
            'Invoked with an object, hence returning the same object');
        $Log->exit() if $may_exit;
        return $pkg_or_obj;
    } else {
        $Log->debug('Invoked with a package name, hence creating a new '
                . 'object and returning it');
        $Log->exit() if $may_exit;
        return $pkg_or_obj->new(
            command_interface => $command_interface,
            domain            => $mode7_args{'domain'},
        );
    } ## end else [ if ( ref $pkg_or_obj )

} ## end sub _add_7mode_cli

=head2 remove 

   NACL::C::VserverCifsDomainPreferredDc->remove(
                                command_interface => $ci,
                            domain => $domain,
                            vserver => $vserver,
                            [lif => $lif,]
                                ...);

or

    $VserverCifsDomainPreferredDc_object->remove(...);

(Class or instance method)
This method is used to remove one or more domain controllers from the list used
by the specified Vserver for interactions with the specified domain. If a list
of preferred domain controllers is not provided, the entire list for the
specified domain is removed.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "cifs-domain-preferred-dc-remove" API for CMode ZAPI.

Invokes "cifs prefdc delete" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< domain => $domain >>

(Required for class method, Not Applicable for instance method)
This parameter specifies the fully-qualified name of the domain that the domain
controllers belong to.

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI: domain

7M CLI : domain name (Programmatic name: "domain")

=item C<< vserver => $vserver >>

(Required for class method, Not Applicable for instance method)
This parameter specifies the name of the Vserver for which to remove the list of
preferred domain controllers.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
         option.

=item C<< lif => $lif >>

(Required for builds not containing UI change 'vserver-cifs-domain-preferred-dc-v2',
for builds containing this UI change this should not be specified)

Determining whether a build has this UI change can be done by:
my $version_manager = $command_interface->get_version_manager();
if ($version_manager->has_change(uichange => 'vserver-cifs-domain-preferred-dc-v2')) {
    # Has UI change, "lif" parameter should not be sent
} else {
    # Does not have UI change, "lif" parameter should be sent
}

Applicable for CMode CLI.

=item C<< 'preferred-dc' => [ $preferred-dc1, $preferred-dc2, ... ] >>

(Arrayref)
This arrayref specifies a list of IP addresses for domain controllers that
belong to the domain specified in the 'domain' parameter. If not provided,
all preferred domain controllers are removed for the vserver.

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: preferred-dc

=item C<< 'lif-id' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs domain preferred-dc remove" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( lif-id lif
                    )
            ),
            'preferred-dc' => {type => ARRAYREF, optional => 1},
        },
        cli_cmd => 'vserver cifs domain preferred-dc remove',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_remove_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_remove_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
                {   method    => '_remove_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
            ],
            frontend => 'NACL::C::VserverCifsDomainPreferredDc::remove',
        );
    }
    catch NACL::C::Exceptions::Component with {
        $Log->exit() if $may_exit;
        $_[0]->throw();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverCifsDomainPreferredDc::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
} ## end sub remove

sub _remove_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'vserver_cifs_domain_preferred_dc_remove',);
    $Log->exit() if $may_exit;
} ## end sub _remove_cmode_cli

sub _remove_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        copy => [qw(domain preferred-dc)],
        api  => 'cifs_domain_preferred_dc_remove',
    );
    $Log->exit() if $may_exit;
} ## end sub _remove_cmode_zapi

sub _remove_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my %mode7_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(domain)],
        target => \%mode7_args,
    );

    $apiset->cifs_prefdc_delete(%mode7_args);
    $Log->exit() if $may_exit;
} ## end sub _remove_7mode_cli

=head2 delete

This is an alias to <remove|lib-NACL-C-VserverCifsDomainPreferredDc-pm/remove>.
It has been retained for backwards compatibility, it is suggested that
'remove' is used instead.

=cut

sub delete {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $Log->warn("The method 'remove' should be used. This method has "
            . 'been retained for backwards compatibility');
    $pkg_or_obj->remove(@_);
    $Log->exit() if $may_exit;
} ## end sub delete


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'domain'  => {type => SCALAR},
        'vserver' => {type => SCALAR | UNDEF, optional => 1},
        'lif'     => {type => SCALAR | UNDEF, optional => 1},
    );
} ## end sub _primary_keys_validate_spec

1;
