#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverNameMapping Component Module
## @author Rupamjyoti.Baruah@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverNameMapping

=head1 DESCRIPTION

C<NACL::C::VserverNameMapping> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP Vserver Name Mapping.  A related class is
L<NACL::CS::VserverNameMapping|lib-NACL-CS-VserverNameMapping-pm>, which
represents the past state of an ONTAP Vserver Name Mapping

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

The name of the vserver, used to identify it uniquely.

=head2 direction

Direction {krb-unix|win-unix|unix-win}

=head2 position

This specifies the name mapping's position in the priority list. A position is
specified as a positive integer.

=cut

package NACL::C::VserverNameMapping;

use strict;
use warnings;
use NACL::CS::VserverNameMapping;
use NACL::ComponentUtils qw (_optional_scalars);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $vserver_name_mapping = NACL::C::VserverNameMapping->new(
                                                       command_interface=>$ci,
                                                       vserver =>$vserver_name,
                                                       ..);

Return a new Vserver name mapping component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'direction',
    scalar => 'position',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;

    #Call the base class's init to do the normal argument checking
    $self->SUPER::init(@args);

    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
} ## end sub init

=head2 create

  NACL::C::VserverNameMapping->create( command_interface=>$ci,
                                       vserver=>$vserver_name,
                                       ...);

(Class method) Request creation of a new vserver name mapping element, and
return a new  vserver name mapping component object that refers to it.  Does
not wait for vserver name mapping creation to complete.

Uses a CMode CLI APISet or CMode ZAPI APISet .
  Invokes 'name-mapping-create' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required) This specifies the virtual server on which the name mapping is to
be created.
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< direction=>$direction >>

(Required) This specifies the Direction.
Accepts the values {krb-unix|win-unix|unix-win}
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : direction

=item C<< position=>$position >>

(Required) This specifies the name mapping's position in the priority list.
A position is specified as a positive integer.
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : position

=item C<< pattern=>$pattern >>

(Required) This specifies the pattern that is to be matched. The pattern can
be up to 256 characters in length.
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : pattern

=item C<< replacement=>$replacement >>

(Required) This specifies the replacement pattern. The replacement pattern can
be up to 256 characters in length.
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : replacement

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver name-mapping create" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {_optional_scalars(qw(pattern replacement))},
        cli_cmd         => 'vserver name-mapping create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            }

        ],
        frontend => 'NACL::C::VserverNameMapping::create',
        nacl_exceptions => [qw(AlreadyExists)]
    );
    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        direction         => $opts{'direction'},
        vserver           => $vserver,
        position          => $opts{'position'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'vserver_name_mapping_create',);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(direction pattern position replacement)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'name-mapping-create',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

=head2 delete

  NACL::C::VserverNameMapping->delete(command_interface=>$ci,
                                    vserver=>$vserver_name,...);

                            or

  $vserver_name_mapping_object->delete(...);

(Class or instance method) This method deletes an vserver name mapping element.

Uses a CMode CLI APISet or CMode ZAPI APISet.
  Invokes 'name-mapping-delete' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required for class method, Not Applicable for instance method)
This specifies the virtual server on which the name mapping is to be created.
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< direction=>$direction >>

(Required for class method, Not Applicable for instance method)
This parameter is to specifies the Direction.
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : direction

=item C<< position=>$position >>

(Required for class method, Not Applicable for instance method)
This specifies the name mapping's position in the priority list.
A position is specified as a positive integer.
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : position

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver name-mapping delete" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver name-mapping delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            }
        ],
        frontend => 'NACL::C::VserverNameMapping::delete',
        nacl_exceptions => [qw(DoesNotExist)]
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_name_mapping_delete',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(direction position)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'name-mapping-delete',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 insert

   NACL::C::VserverNameMapping->insert(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to create a name mapping at a specified position.

Uses a CMode CLI APISet or CMode ZAPI APISet.
  This method returns an object similar to the create method.
  Invokes 'name-mapping-insert' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "direction" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : direction

=item C<< "pattern" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : pattern

=item C<< "vserver" => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option

=item C<< "position" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : position

=item C<< "replacement" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : replacement

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver name-mapping insert" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub insert {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( direction pattern vserver position replacement
                    )
            ),
        },
        cli_cmd => 'vserver name-mapping insert',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_insert_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_insert_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverNameMapping::insert',
    );
    $Log->exit() if $may_exit;
    return $pkg->new(
        vserver           => $vserver,
        direction         => $opts{'direction'},
        position          => $opts{'position'},
        command_interface => $opts{command_interface},
    );

} ## end sub insert

sub _insert_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_name_mapping_insert',);
    $Log->exit() if $may_exit;
} ## end sub _insert_cmode_cli

sub _insert_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(direction pattern position replacement)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'name-mapping-insert',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
} ## end sub _insert_cmode_zapi

=head2 modify

   NACL::C::VserverNameMapping->modify(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to modify a name mapping's pattern, replacement pattern, or both.

Uses a CMode CLI APISet or CMode ZAPI APISet.
  Invokes 'name-mapping-modify' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "direction" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : direction

=item C<< "pattern" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : pattern

=item C<< "vserver" => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< "position" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : position

=item C<< "replacement" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : replacement

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver name-mapping modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( direction pattern vserver position replacement
                    )
            ),
        },
        cli_cmd => 'vserver name-mapping modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            }
        ],
        frontend => 'NACL::C::VserverNameMapping::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_name_mapping_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(direction pattern position replacement)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'name-mapping-modify',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 swap

   NACL::C::VserverNameMapping->swap(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to exchange the positions of two name mappings.

Uses a CMode CLI APISet CMode ZAPI APISet.
  Invokes "name-mapping-swap" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "with-position" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : with-position

=item C<< "direction" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : direction

=item C<< "vserver" => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< "position" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : position

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver name-mapping swap" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub swap {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( with-position direction vserver position
                    )
            ),
        },
        cli_cmd => 'vserver name-mapping swap',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_swap_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_swap_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            }
        ],
        frontend => 'NACL::C::VserverNameMapping::swap',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub swap

sub _swap_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_name_mapping_swap',);
    $Log->exit() if $may_exit;
} ## end sub _swap_cmode_cli

sub _swap_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(with-position direction position)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'name-mapping-swap',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
} ## end sub _swap_cmode_zapi

=head2 load_from_uri

    NACL::C::VserverNameMapping->load_from_uri(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to load one or more name mappings from a URI.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'overwrite' => $string >>

  Applicable only for CMode CLI

=item C<< 'uri' => $string >>

  Applicable only for CMode CLI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver name-mapping load-from-uri" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub load_from_uri {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( overwrite uri
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'vserver name-mapping load-from-uri',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_load_from_uri_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverNameMapping::load_from_uri',
    );

    $Log->exit() if $may_exit;
} ## end sub load_from_uri

sub _load_from_uri_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'vserver_name_mapping_load_from_uri',);
    $Log->exit() if $may_exit;
} ## end sub _load_from_uri_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        vserver   => {type => SCALAR | UNDEF, optional => 1},
        direction => {type => SCALAR},
        position  => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
