#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary VserverNfs Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverNfs

=head1 DESCRIPTION

C<NACL::C::VserverNfs> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to a NFS configuration for an ONTAP virtual server.  A related class is
L<NACL::CS::VserverNfs|lib-NACL-CS-VserverNfs-pm>, which represents
the past state of an ONTAP VserverNfs.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

The name of the vserver where nfs is configured, used to identify it uniquely.

(Note that if the command_interface attribute is a
L<NACL::C::Vserver|lib-NACL-C-Vserver-pm>, then this attribute will be undef)

=cut

package NACL::C::VserverNfs;

use strict;
use warnings;
use NACL::CS::VserverNfs;
use NACL::C::Vserver;
use NACL::ComponentUtils qw (_optional_scalars);
use Params::Validate qw(validate_with SCALAR UNDEF ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::C::Exceptions::VserverNfs::FeatureNotLicensed qw(:try);
use NACL::C::Exceptions::VserverNfs::AlreadyExists            ();
use NACL::C::Exceptions::VserverNfs::DoesNotExist             ();
use NACL::C::Exceptions::VserverNfs::NoValuesToApply          ();
use NACL::C::Exceptions::VserverNfs::AllowedProtocolsNotAdded ();

use base 'NACL::C::Component::ONTAP';

=head1 METHODS

=head2 new

  my $vs_nfs_obj = NACL::C::VserverNfs->new(command_interface=>$ci,
                                      vserver =>$vserver_name);

Return a new VserverNfs component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];

=head2 create

  my $vs_nfs_obj  = NACL::C::VserverNfs->create(command_interface=>$ci,
                                          vserver=>$vserver_name,...);

(Class method) Request creation of a new vserver nfs element, and  return a new
vserver nfs component object that refers to it.

Uses a  CMode CLI/ZAPI APISet.

Invokes "nfs-service-create" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required when command interface is not a C<NACL::C::Vserver>, optional if
command interface is a C<NACL::C::Vserver>)
Name of vserver for which nfs needs to be configured.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< access => $boolean >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfs-access-enabled

=item C<< "rpcsec-ctx-high" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : rpcsec-ctx-high

=item C<< "rpcsec-ctx-idle" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : rpcsec-ctx-idle

=item C<< "v2" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv2-enabled

=item C<< "v3" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv3-enabled

=item C<< "v4" => $string >>

Applicable for CMode CLI

=item C<< "v4.0" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-enabled

=item C<< "v4.1" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-enabled

=item C<< "spinauth" => $string >>

Applicable for CMode CLI

=item C<< "default-win-user" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : default-windows-user

=item C<< "enable-ejukebox" => $boolean >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : enable-ejukebox

=item C<< "cache-ejukebox" => $boolean >>

Applicable for CMode CLI

=item C<< "v3-require-read-attributes" => $boolean >>

Applicable for CMode CLI

=item C<< "v3-fsid-change" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv3-fsid-change-enabled

=item C<< "v3-connection-drop" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv3-connection-drop-enabled

=item C<< "ntfs-unix-security-ops" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : ntfs-unix-security-ops

=item C<< "chown-mode" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : chown-mode

=item C<< "force-spinnp-readdir" => $boolean>>

Applicable for CMode CLI

=item C<< "trace-enabled" => $boolean>>

Applicable for CMode CLI

=item C<< "trigger" => $number>>

Applicable for CMode CLI

=item C<< "udp-max-xfer-size" => $number>>

Applicable for CMode CLI

=item C<< "tcp-max-xfer-size" => $number>>

Applicable for CMode CLI

=item C<< "v4-acl" => $string>>

Applicable for CMode CLI.

=item C<< "v4.0-acl" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-acl-enabled

=item C<< "v4-read-delegation" => $string>>

Applicable for CMode CLI

=item C<< "v4.0-read-delegation" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-read-delegation-enabled

=item C<< "v4-write-delegation" => $string>>

Applicable for CMode CLI.

=item C<< "v4.0-write-delegation" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-write-delegation-enabled

=item C<< "v4-fsid-change" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv4-fsid-change-enabled

=item C<< "v4-reply-drop" => $number>>

Applicable for CMode CLI

=item C<< "v4-referrals" => $string>>

Applicable for CMode CLI

=item C<< "v4.0-referrals" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-referrals-enabled

=item C<< "v4-req-open-confirm" => $string>>

Applicable for CMode CLI

=item C<< "v4.0-req-open-confirm" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-req-open-confirm-enabled

=item C<< "v4-id-domain" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv4-id-domain

=item C<< "v4-validate-symlinkdata" => $string>>

Applicable for CMode CLI

=item C<< "v4-lease-seconds" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv4-lease-seconds

=item C<< "v4-grace-seconds" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv4-grace-seconds

=item C<< "v4-acl-preserve" => $string>>

Applicable for CMode CLI

=item C<< "v4.1" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-enabled

=item C<< "rquota" => $string>>

Applicable for CMode CLI

=item C<< "v4.1-state-protection" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-state-protection-enabled

=item C<< "v4.1-implementation-domain" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv41-implementation-id-domain

=item C<< "v4.1-implementation-name" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv41-implementation-id-name

=item C<< "v4.1-implementation-date" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv41-implementation-id-time

=item C<< "v4.1-pnfs" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-pnfs-enabled

=item C<< "v4.1-pnfs-striped-volumes" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-pnfs-striped-volumes-enabled

=item C<< "v4.0-migration" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-migration-enabled

=item C<< "v4.1-referrals" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-referrals-enabled

=item C<< "v4.1-migration" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-migration-enabled

=item C<< "v4.1-acl" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-acl-enabled

=item C<< 'v3-tcp-max-write-size' => $string >>

Applicable for CMode CLI

=item C<< 'v4-acl-preserve' => $string >>

Applicable for CMode CLI

=item C<< 'v4-numeric-ids' => $string >>

Applicable for CMode CLI

=item C<< 'vstorage' => $string >>

Applicable for CMode CLI

=item C<< 'v3-tcp-max-read-size' => $string >>

Applicable for CMode CLI

=item C<< 'v4-id-numeric' => $string >>

Applicable for CMode CLI

=item C<< 'default-win-group' => $string >>

Applicable for CMode CLI

=item C<< 'v4.x-session-slot-reply-cache-size' => $string >>

Applicable for CMode CLI

=item C<< 'v4.1-read-delegation' => $string >>

Applicable for CMode CLI

=item C<< 'v4.x-session-num-slots' => $string >>

Applicable for CMode CLI

=item C<< 'ipv6' => $string >>

Applicable for CMode CLI

=item C<< 'v4.1-write-delegation' => $string >>

Applicable for CMode CLI

=item C<< 'mount-rootonly' => $string >>

Applicable for CMode CLI

=item C<< 'v4-acl-max-aces' => $string >>

Applicable for CMode CLI

=item C<< 'nfs-rootonly' => $string >>

Applicable for CMode CLI
NFS Root Only, possible value(s) are enabled,disabled

Applicable for CMode CLI

=item C<< 'is-auth-sys-extended-groups-enabled' => $string >>

AUTH_SYS Extended Groups Support possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'extended-groups-limit' => $string >>

AUTH_SYS and RPCSEC_GSS Auxillary Groups Limit ranges from 32 to 256
Applicable for CMode CLI

=item C<< 'validate-qtree-export' => $string >>


Validation of Qtree IDs for Qtree File Operations possible value(s) are, enabled,disabled
Applicable for CMode CLI

=item C<< 'rquotad-port' => $string >>

NFS Quota Daemon Port ranges from 1 to 65535
Applicable for CMode CLI

=item C<< 'mountd-port' => $string >>

NFS Mount Daemon Port ranges from 1 to 65535
Applicable for CMode CLI

=item C<< 'nlm-port' => $string >>

Network Lock Manager Port ranges from 1 to 65535
Applicable for CMode CLI

=item C<< 'nsm-port' => $string >>

Network Status Monitor Port ranges from 1 to 65535
Applicable for CMode CLI

=item C<< 'permitted-enc-types' => [ $permitted-enc-type1, $permitted-enc-type2, ... ] >>

Permitted Kerberos Encryption Types
(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< 'is-showmount-enabled' => $string >>

Showmount Enabled possible value(s) are, enabled,disabled
Applicable for CMode CLI

=item C<< 'showmount' => $string >>

Showmount Support possible value(s) are, enabled,disabled
Applicable for CMode CLI

=item C<< 'auth-sys-extended-groups' => $string >>


AUTH_SYS Extended Groups Support possible value(s) are, enabled,disabled
Applicable for CMode CLI

=item C<< 'name-service-lookup-protocol' => $string >>


Set the Protocol Used for Name Services Lookups for Exports possible value(s) are, TCP,UDP
Applicable for CMode CLI

=item C<< 'netgroup-dns-domain-search' => $string >>

DNS Domain Search Enabled During Netgroup Lookup possible value(s) are, enabled,disabled
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by
"vserver nfs create" command in ONTAP.

=item C<< "extended_query" => $string>>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VserverNfs::AlreadyExists>

This type of exception gets thrown if an attempt is made to create a
vserver nfs which already exists.

=item C<NACL::C::Exceptions::VserverNfs::FeatureNotLicensed>

This type of exception gets thrown if an attempt is made to create a
vserver nfs when nfs is not licensed.

=item C<NACL::C::Exceptions::VserverNfs::AllowedProtocolsNotAdded>

This type of exception gets thrown if an attempt is made to create a
vserver nfs when nfs is not listed in the allowed-protocols list of the vserver

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            "permitted-enc-types" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw(access rpcsec-ctx-high rpcsec-ctx-idle v2 v3 udp
                    tcp v4.0 spinauth default-win-user enable-ejukebox cache-ejukebox
                    v3-require-read-attributes v3-fsid-change v3-connection-drop
                    ntfs-unix-security-ops chown-mode force-spinnp-readdir trace-enabled
                    trigger udp-max-xfer-size tcp-max-xfer-size v4.0-acl v4.0-read-delegation
                    v4.0-write-delegation v4-fsid-change v4-reply-drop v4.0-referrals
                    v4.0-req-open-confirm v4-id-domain v4-validate-symlinkdata
                    v4-lease-seconds v4-grace-seconds v4-acl-presere v4.1 rquota v4.1-state-protection
                    v4.1-implementation-domain v4.1-implementation-name v4.1-implementation-date
                    v4.1-pnfs v4.1-pnfs-striped-volumes v4.0-migration v4.1-referrals
                    v4.1-migration v4.1-acl v4 v4-acl v4-read-delegation
                    v4-write-delegation v4-referrals v4-req-open-confirm
                    v3-tcp-max-write-size v4-acl-preserve v4-numeric-ids vstorage
                    v3-tcp-max-read-size

                    v4-id-numeric

                    default-win-group

                    v4.x-session-slot-reply-cache-size v4.1-read-delegation
                    v4.x-session-num-slots ipv6 v4.1-write-delegation mount-rootonly
                    v4-acl-max-aces

                    nfs-rootonly

                    is-auth-sys-extended-groups-enabled extended-groups-limit

                    validate-qtree-export

                    rquotad-port mountd-port nlm-port nsm-port

                    is-showmount-enabled

                    showmount auth-sys-extended-groups

                    name-service-lookup-protocol

                    netgroup-dns-domain-search
                    )
            ),
        },
        cli_cmd => 'vserver nfs create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_create_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_create_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                    check     => '_zapi_check_tcp_udp',
                },
            ],
            frontend => 'NACL::C::VserverNfs::create'
        );
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverNfs::AlreadyExists
            ->detect_convert_and_throw(exception => $exception,);
        if (NACL::C::Exceptions::VserverNfs::FeatureNotLicensed->detect(
                exception => $exception
            )
            )
        {
            $Log->exit() if $may_exit;
            NACL::C::Exceptions::VserverNfs::FeatureNotLicensed
                ->convert_and_throw(
                exception         => $exception,
                feature           => 'nfs',
                command_interface => $opts{command_interface}
                );

        } elsif (
            NACL::C::Exceptions::VserverNfs::AllowedProtocolsNotAdded->detect(
                exception => $exception
            )
            )
        {
            $Log->exit() if $may_exit;
            NACL::C::Exceptions::VserverNfs::AllowedProtocolsNotAdded
                ->convert_and_throw(
                exception         => $exception,
                protocol          => 'nfs',
                vserver           => $opts{vserver},
                command_interface => $opts{command_interface}
                );
        }
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    my $obj = $pkg->new(
        vserver           => $opts{vserver},
        command_interface => $opts{command_interface}
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_cli(%opts,
        api => 'vserver_nfs_create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    my $allowed_cmode_options = [
        qw(  rpcsec-ctx-high rpcsec-ctx-idle enable-ejukebox
            ntfs-unix-security-ops chown-mode )
    ];

    %opts = _convert_enable_disable_to_true_false(\%opts);

    $pkg->_base_backend_cmode_zapi(%opts,
        api   => 'nfs_service_create',
        copy  => $allowed_cmode_options,
        map   => _create_modify_cmode_zapi_map(),
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

sub _zapi_check_tcp_udp {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my @fields;
    foreach my $field (qw(tcp udp)) {
        push (@fields, $field) if (exists $opts{$field});
    }
    if (@fields) {
        my $fields_str = join ',', @fields;
        my $msg = 'Could not use the ZAPI implementation of '
            . "$opts{_method}() because field(s) $fields_str "
            . 'was provided. This is currently not accepted by the ZAPI '
            . '(see burt 927716)';
        $Log->comment($msg);
        require NACL::Exceptions::InvalidChoice;
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw($msg);
    }
    $Log->exit() if $may_exit;
}


=head2 delete

  NACL::C::VserverNfs->delete(command_interface=>$ci,
                             vserver=>$vserver_name,...);

                            or

  $vservernfs_object->delete(...);

(Class or instance method) This method deletes a vserver nfs configuration element.

Uses a CMode CLI/ZAPI APISet.

Invokes "nfs-service-destroy" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required for class method, Not Applicable for instance method)
Name of vserver.

(Note that this is required for class methods only if the command interface
is not a C<NACL::C::Vserver>)

=item C<< '*' >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@_,
        cli_cmd => 'vserver nfs delete'
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VserverNfs::delete'
        );
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverNfs::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;

} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_cli(%opts,
        api => 'vserver_nfs_delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_zapi(@args,
        api   => 'nfs_service_destroy',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 modify

  NACL::C::VserverNfs->modify(command_interface=>$ci,
                             vserver=>$vserver_name,...);

                            or

  $vservernfs_object->modify(...);

(Class or instance method) Modify a vserver nfs element.

Uses a CMode CLI/ZAPI APISet.

Invokes "nfs-service-modify" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required for class method, Not Applicable for instance method)
Name of vserver.

(Note that this is required for class methods only if the command interface
is not a C<NACL::C::Vserver>)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< access => $boolean >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfs-access-enabled

=item C<< "rpcsec-ctx-high" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : rpcsec-ctx-high

=item C<< "rpcsec-ctx-idle" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : rpcsec-ctx-idle

=item C<< "v2" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv2-enabled

=item C<< "v3" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv3-enabled

=item C<< "v4" => $string >>

Applicable for CMode CLI

=item C<< "v4.0" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-enabled

=item C<< "v4.1" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-enabled

=item C<< "spinauth" => $string >>

Applicable for CMode CLI

=item C<< "default-win-user" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : default-windows-user

=item C<< "enable-ejukebox" => $boolean >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : enable-ejukebox

=item C<< "cache-ejukebox" => $boolean >>

Applicable for CMode CLI

=item C<< "v3-require-read-attributes" => $boolean >>

Applicable for CMode CLI

=item C<< "v3-fsid-change" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv3-fsid-change-enabled

=item C<< "v3-connection-drop" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv3-connection-drop-enabled

=item C<< "ntfs-unix-security-ops" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : ntfs-unix-security-ops

=item C<< "chown-mode" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : chown-mode

=item C<< "force-spinnp-readdir" => $boolean>>

Applicable for CMode CLI

=item C<< "trace-enabled" => $boolean>>

Applicable for CMode CLI

=item C<< "trigger" => $number>>

Applicable for CMode CLI

=item C<< "udp-max-xfer-size" => $number>>

Applicable for CMode CLI

=item C<< "tcp-max-xfer-size" => $number>>

Applicable for CMode CLI

=item C<< "v4-acl" => $string>>

Applicable for CMode CLI.

=item C<< "v4.0-acl" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-acl-enabled

=item C<< "v4-read-delegation" => $string>>

Applicable for CMode CLI

=item C<< "v4.0-read-delegation" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-read-delegation-enabled

=item C<< "v4-write-delegation" => $string>>

Applicable for CMode CLI.

=item C<< "v4.0-write-delegation" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-write-delegation-enabled

=item C<< "v4-fsid-change" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv4-fsid-change-enabled

=item C<< "v4-reply-drop" => $number>>

Applicable for CMode CLI

=item C<< "v4-referrals" => $string>>

Applicable for CMode CLI

=item C<< "v4.0-referrals" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-referrals-enabled

=item C<< "v4-req-open-confirm" => $string>>

Applicable for CMode CLI

=item C<< "v4.0-req-open-confirm" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-req-open-confirm-enabled

=item C<< "v4-id-domain" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv4-id-domain

=item C<< "v4-validate-symlinkdata" => $string>>

Applicable for CMode CLI

=item C<< "v4-lease-seconds" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv4-lease-seconds

=item C<< "v4-grace-seconds" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv4-grace-seconds

=item C<< "v4-acl-preserve" => $string>>

Applicable for CMode CLI

=item C<< "v4.1" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-enabled

=item C<< "rquota" => $string>>

Applicable for CMode CLI

=item C<< "v4.1-state-protection" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-state-protection-enabled

=item C<< "v4.1-implementation-domain" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv41-implementation-id-domain

=item C<< "v4.1-implementation-name" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv41-implementation-id-name

=item C<< "v4.1-implementation-date" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv41-implementation-id-time

=item C<< "v4.1-pnfs" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-pnfs-enabled

=item C<< "v4.1-pnfs-striped-volumes" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-pnfs-striped-volumes-enabled

=item C<< "v4.0-migration" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-migration-enabled

=item C<< "v4.1-referrals" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-referrals-enabled

=item C<< "v4.1-migration" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-migration-enabled

=item C<< "v4.1-acl" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-acl-enabled

=item C<< 'v3-tcp-max-write-size' => $string >>

Applicable for CMode CLI

=item C<< 'v4-acl-preserve' => $string >>

Applicable for CMode CLI

=item C<< 'v4-numeric-ids' => $string >>

Applicable for CMode CLI

=item C<< 'vstorage' => $string >>

Applicable for CMode CLI

=item C<< 'v3-tcp-max-read-size' => $string >>

Applicable for CMode CLI

=item C<< 'v4-id-numeric' => $string >>

Applicable for CMode CLI

=item C<< 'default-win-group' => $string >>

Applicable for CMode CLI

=item C<< 'v4.x-session-slot-reply-cache-size' => $string >>

Applicable for CMode CLI

=item C<< 'v4.1-read-delegation' => $string >>

Applicable for CMode CLI

=item C<< 'v4.x-session-num-slots' => $string >>

Applicable for CMode CLI

=item C<< 'ipv6' => $string >>

Applicable for CMode CLI

=item C<< 'v4.1-write-delegation' => $string >>

Applicable for CMode CLI

=item C<< 'mount-rootonly' => $string >>

Applicable for CMode CLI

=item C<< 'v4-acl-max-aces' => $string >>

Applicable for CMode CLI

=item C<< 'nfs-rootonly' => $string >>

Applicable for CMode CLI
NFS Root Only, possible value(s) are enabled,disabled

Applicable for CMode CLI

=item C<< 'is-auth-sys-extended-groups-enabled' => $string >>

AUTH_SYS Extended Groups Support possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'extended-groups-limit' => $string >>

AUTH_SYS and RPCSEC_GSS Auxillary Groups Limit ranges from 32 to 256
Applicable for CMode CLI

=item C<< 'validate-qtree-export' => $string >>


Validation of Qtree IDs for Qtree File Operations possible value(s) are, enabled,disabled
Applicable for CMode CLI

=item C<< 'rquotad-port' => $string >>

NFS Quota Daemon Port ranges from 1 to 65535
Applicable for CMode CLI

=item C<< 'mountd-port' => $string >>

NFS Mount Daemon Port ranges from 1 to 65535
Applicable for CMode CLI

=item C<< 'nlm-port' => $string >>

Network Lock Manager Port ranges from 1 to 65535
Applicable for CMode CLI

=item C<< 'nsm-port' => $string >>

Network Status Monitor Port ranges from 1 to 65535
Applicable for CMode CLI

=item C<< 'permitted-enc-types' => [ $permitted-enc-type1, $permitted-enc-type2, ... ] >>

Permitted Kerberos Encryption Types
(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< 'is-showmount-enabled' => $string >>

Showmount Enabled possible value(s) are, enabled,disabled
Applicable for CMode CLI

=item C<< 'showmount' => $string >>

Showmount Support possible value(s) are, enabled,disabled
Applicable for CMode CLI

=item C<< 'auth-sys-extended-groups' => $string >>


AUTH_SYS Extended Groups Support possible value(s) are, enabled,disabled
Applicable for CMode CLI

=item C<< 'name-service-lookup-protocol' => $string >>


Set the Protocol Used for Name Services Lookups for Exports possible value(s) are, TCP,UDP
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by the
subcommand in ONTAP
Applicable for CMode CLI

=item C<< "extended_query" => $string>>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            "permitted-enc-types" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw(access rpcsec-ctx-high rpcsec-ctx-idle v2 v3 v4.0
                    udp tcp spinauth default-win-user enable-ejukebox cache-ejukebox
                    v3-require-read-attributes v3-fsid-change v3-connection-drop
                    ntfs-unix-security-ops chown-mode force-spinnp-readdir
                    trace-enabled trigger udp-max-xfer-size tcp-max-xfer-size
                    v4.0-acl v4.0-read-delegation
                    v4.0-write-delegation v4-fsid-change v4-reply-drop v4.0-referrals
                    v4.0-req-open-confirm v4-id-domain v4-validate-symlinkdata v4-lease-seconds
                    v4-grace-seconds v4-acl-preserve v4.1 v4.1-state-protection
                    v4.1-implementation-domain v4.1-implementation-name
                    v4.1-implementation-date v4.1-pnfs v4.1-pnfs-striped-volumes v4.0-migration
                    v4.1-referrals v4.1-migration v4.1-acl rquota v4 v4-acl
                    v4-read-delegation v4-write-delegation v4-referrals
                    v4-req-open-confirm
                    v3-tcp-max-write-size v4-numeric-ids vstorage v3-tcp-max-read-size

                    v4-id-numeric

                    default-win-group

                    v4.x-session-slot-reply-cache-size v4.1-read-delegation
                    v4.x-session-num-slots ipv6 v4.1-write-delegation mount-rootonly
                    v4-acl-max-aces

                    nfs-rootonly

                    is-auth-sys-extended-groups-enabled extended-groups-limit

                    validate-qtree-export

                    rquotad-port mountd-port nlm-port nsm-port

                    is-showmount-enabled

                    showmount auth-sys-extended-groups

                    name-service-lookup-protocol

                    netgroup-dns-domain-search

		    v3-search-unconverted-filename
                    )
            )
        },
        cli_cmd => 'vserver nfs modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_modify_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_modify_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                    check     => '_check_zapi'
                },
            ],
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverNfs::NoValuesToApply->detect_convert_and_throw(
            exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;

} ## end sub modify


sub _check_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_zapi_check_tcp_udp(%opts);

    foreach my $cli_field (qw(rquota)) {
        if (exists $opts{$cli_field}) {
            my $msg = "The field '$cli_field' was provided in the call to "
                . 'NACL::C::VserverNfs->modify but is not supported by the '
                . 'ZAPI so the ZAPI implementation cannot be used';
            $Log->exit() if $may_exit;
            NACL::Exceptions::InvalidChoice->throw($msg);
        }
    }
    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_nfs_modify',);
    $Log->exit() if $may_exit;
}


sub _modify_cmode_zapi {
    my ( $pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( rpcsec-ctx-high rpcsec-ctx-idle enable-ejukebox
            ntfs-unix-security-ops chown-mode )
    ];
    %opts = _convert_enable_disable_to_true_false(\%opts);
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
                     copy            => $allowed_cmode_options,
                     map             => _create_modify_cmode_zapi_map(), 
                     frontend_method => 'modify',
                     zapi_field_translations => {
                         string_to_timestamp => [qw( v4.1-implementation-date ) ]
                     },
                     api             => 'nfs-service-modify',
                     
                  );
    $Log->exit() if $may_exit;

} ## end sub _modify_cmode_zapi


=head2 on

    NACL::C::VserverNfs->on(command_interface => $ci,
                            vserver => $vserver_name);

                or

    $vservernfs_obj->on();

(Class or instance method)
This method is used to enables the NFS service of a Vserver.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "nfs-enable" API for CMode ZAPI.

Invokes "nfs-on" API for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required for class method, Not Applicable for instance method)
Name of vserver.

(Note that this is required for class methods only if the command interface
is not a C<NACL::C::Vserver>)

=item C<< '*' >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub on {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@_,
        cli_cmd => 'vserver nfs on'
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_on_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_on_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => "_on_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );
    $Log->exit() if $may_exit;
} ## end sub on

sub _on_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_cli(%opts,
        api => 'vserver_nfs_on',
    );
    $Log->exit() if $may_exit;
} ## end sub _on_cmode_cli

sub _on_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
        api => 'nfs_enable',
    );
    $Log->exit() if $may_exit;
} ## end sub _on_cmode_zapi

sub _on_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    $apiset->nfs_on();
    $Log->exit() if $may_exit;
} ## end sub _on_7mode_cli

=head2 off

    NACL::C::VserverNfs->off(command_interface => $ci,
                            vserver => $vserver_name);

                or

    $vservernfs_obj->off();

(Class or instance method)
This method is used to disable the NFS service of a Vserver.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "nfs-disable" API for CMode ZAPI.

Invokes "nfs-off" API for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$vserver_name >>

(Required for class method, Not Applicable for instance method)
Name of vserver.

(Note that this is required for class methods only if the command interface
is not a C<NACL::C::Vserver>)

=item C<< '*' >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< access => $boolean >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfs-access-enabled

=item C<< "rpcsec-ctx-high" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : rpcsec-ctx-high

=item C<< "rpcsec-ctx-idle" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : rpcsec-ctx-idle

=item C<< "v2" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv2-enabled

=item C<< "v3" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv3-enabled

=item C<< "v4" => $string >>

Applicable for CMode CLI

=item C<< "v4.0" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-enabled

=item C<< "v4.1" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-enabled

=item C<< "spinauth" => $string >>

Applicable for CMode CLI

=item C<< "default-win-user" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : default-windows-user

=item C<< "enable-ejukebox" => $boolean >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : enable-ejukebox

=item C<< "cache-ejukebox" => $boolean >>

Applicable for CMode CLI

=item C<< "v3-require-read-attributes" => $boolean >>

Applicable for CMode CLI

=item C<< "v3-fsid-change" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv3-fsid-change-enabled

=item C<< "v3-connection-drop" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv3-connection-drop-enabled

=item C<< "ntfs-unix-security-ops" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : ntfs-unix-security-ops

=item C<< "chown-mode" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : chown-mode

=item C<< "force-spinnp-readdir" => $boolean>>

Applicable for CMode CLI

=item C<< "trace-enabled" => $boolean>>

Applicable for CMode CLI

=item C<< "trigger" => $number>>

Applicable for CMode CLI

=item C<< "udp-max-xfer-size" => $number>>

Applicable for CMode CLI

=item C<< "tcp-max-xfer-size" => $number>>

Applicable for CMode CLI

=item C<< "v4-acl" => $string>>

Applicable for CMode CLI.

=item C<< "v4.0-acl" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-acl-enabled

=item C<< "v4-read-delegation" => $string>>

Applicable for CMode CLI

=item C<< "v4.0-read-delegation" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-read-delegation-enabled

=item C<< "v4-write-delegation" => $string>>

Applicable for CMode CLI.

=item C<< "v4.0-write-delegation" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-write-delegation-enabled

=item C<< "v4-fsid-change" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv4-fsid-change-enabled

=item C<< "v4-reply-drop" => $number>>

Applicable for CMode CLI

=item C<< "v4-referrals" => $string>>

Applicable for CMode CLI

=item C<< "v4.0-referrals" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-referrals-enabled

=item C<< "v4-req-open-confirm" => $string>>

Applicable for CMode CLI

=item C<< "v4.0-req-open-confirm" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-req-open-confirm-enabled

=item C<< "v4-id-domain" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv4-id-domain

=item C<< "v4-validate-symlinkdata" => $string>>

Applicable for CMode CLI

=item C<< "v4-lease-seconds" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv4-lease-seconds

=item C<< "v4-grace-seconds" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv4-grace-seconds

=item C<< "v4-acl-preserve" => $string>>

Applicable for CMode CLI

=item C<< "v4.1" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-enabled

=item C<< "rquota" => $string>>

Applicable for CMode CLI

=item C<< "v4.1-state-protection" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-state-protection-enabled

=item C<< "v4.1-implementation-domain" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv41-implementation-id-domain

=item C<< "v4.1-implementation-name" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv41-implementation-id-name

=item C<< "v4.1-implementation-date" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : nfsv41-implementation-id-time

=item C<< "v4.1-pnfs" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-pnfs-enabled

=item C<< "v4.1-pnfs-striped-volumes" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-pnfs-striped-volumes-enabled

=item C<< "v4.0-migration" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv40-migration-enabled

=item C<< "v4.1-referrals" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-referrals-enabled

=item C<< "v4.1-migration" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-migration-enabled

=item C<< "v4.1-acl" => $string>>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-nfsv41-acl-enabled

=item C<< 'v3-tcp-max-write-size' => $string >>

Applicable for CMode CLI

=item C<< 'v4-acl-preserve' => $string >>

Applicable for CMode CLI

=item C<< 'v4-numeric-ids' => $string >>

Applicable for CMode CLI

=item C<< 'vstorage' => $string >>

Applicable for CMode CLI

=item C<< 'v3-tcp-max-read-size' => $string >>

Applicable for CMode CLI

=item C<< 'v4-id-numeric' => $string >>

Applicable for CMode CLI

=item C<< 'default-win-group' => $string >>

Applicable for CMode CLI

=item C<< 'v4.x-session-slot-reply-cache-size' => $string >>

Applicable for CMode CLI

=item C<< 'v4.1-read-delegation' => $string >>

Applicable for CMode CLI

=item C<< 'v4.x-session-num-slots' => $string >>

Applicable for CMode CLI

=item C<< 'ipv6' => $string >>

Applicable for CMode CLI

=item C<< 'v4.1-write-delegation' => $string >>

Applicable for CMode CLI

=item C<< 'mount-rootonly' => $string >>

Applicable for CMode CLI

=item C<< 'v4-acl-max-aces' => $string >>

Applicable for CMode CLI

=item C<< 'nfs-rootonly' => $string >>

Applicable for CMode CLI
NFS Root Only, possible value(s) are enabled,disabled

Applicable for CMode CLI

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub off {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@_,
        cli_cmd => 'vserver nfs off'
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_off_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_off_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => "_off_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub off

sub _off_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'vserver_nfs_off',
    );
    $Log->exit() if $may_exit;
} ## end sub _off_cmode_cli

sub _off_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
        api => 'nfs_disable',
    );
    $Log->exit() if $may_exit;
} ## end sub _off_cmode_zapi

sub _off_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    $apiset->nfs_off();
    $Log->exit() if $may_exit;
} ## end sub _off_7mode_cli

sub _create_modify_cmode_zapi_map {
    return {
        "access"                => "is-nfs-access-enabled",
        "default-win-user"      => "default-windows-user",
        "v2"                    => "is-nfsv2-enabled",
        "v3-connection-drop"    => "is-nfsv3-connection-drop-enabled",
        "v3"                    => "is-nfsv3-enabled",
        "v3-fsid-change"        => "is-nfsv3-fsid-change-enabled",
        "v4-fsid-change"        => "is-nfsv4-fsid-change-enabled",
        "v4.0-acl"              => "is-nfsv40-acl-enabled",
        "v4.0"                  => "is-nfsv40-enabled",
        "v4"                  => "is-nfsv40-enabled",
        "v4.0-migration"        => "is-nfsv40-migration-enabled",
        "v4.0-read-delegation"  => "is-nfsv40-read-delegation-enabled",
        "v4-read-delegation"  => "is-nfsv40-read-delegation-enabled",
        "v4.0-referrals"        => "is-nfsv40-referrals-enabled",
        "v4.0-req-open-confirm" => "is-nfsv40-req-open-confirm-enabled",
        "v4.0-write-delegation" => "is-nfsv40-write-delegation-enabled",
        "v4-write-delegation" => "is-nfsv40-write-delegation-enabled",
        "v4.1-acl"              => "is-nfsv41-acl-enabled",
        "v4.1"                  => "is-nfsv41-enabled",
        "v4.1-migration"        => "is-nfsv41-migration-enabled",
        "v4.1-pnfs"             => "is-nfsv41-pnfs-enabled",
        "v4.1-pnfs-striped-volumes"  => "is-nfsv41-pnfs-striped-volumes-enabled",
        "v4.1-referrals"             => "is-nfsv41-referrals-enabled",
        "v4.1-state-protection"      => "is-nfsv41-state-protection-enabled",
        "v4-grace-seconds"           => "nfsv4-grace-seconds",
        "v4-id-domain"               => "nfsv4-id-domain",
        "v4-lease-seconds"           => "nfsv4-lease-seconds",
        "v4.1-implementation-domain" => "nfsv41-implementation-id-domain",
        "v4.1-implementation-name"   => "nfsv41-implementation-id-name",
        "v4.1-implementation-date"   => "nfsv41-implementation-id-time",
        "v4-acl-preserve"            => "is-nfsv41-acl-preserve-enabled",
        "vstorage"		             => "is-vstorage-enabled",
        "v4-numeric-ids"	         => "is-nfsv4-numeric-ids-enabled",
        "default-win-group"	         => "default-windows-group",
        "v4.1-read-delegation"	     => "is-nfsv41-read-delegation-enabled",
        "v4.1-write-delegation"      => "is-nfsv41-write-delegation-enabled",
        "v4.x-session-num-slots"     => "nfsv4x-session-num-slots",
        "v4.x-session-slot-reply-cache-size"  => "nfsv4x-session-slot-reply-cache-size",
        "v4-acl-max-aces"	     => "nfsv4-acl-max-aces",
        "mount-rootonly"	     => "is-mount-rootonly-enabled",
        "nfs-rootonly"		     => "is-nfs-rootonly-enabled",
        "netgroup-dns-domain-search" => "is-netgroup-dns-domain-search",
  };
} ## end sub _create_modify_cmode_zapi_map

# Ensure that the vserver attribute is undef *only* if we're using
# a vserver command interface.
sub init {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;

    # Call the base class's init to do the normal argument checking
    $self->SUPER::init(%opts);

    $self->_vserver_context_init(%opts);
    $Log->exit() if $may_exit;
    return;
} ## end sub init

# Vserver attribute will be undef if invoked in ZAPI context with a
# vserver command interface

=head2 stop

    NACL::C::VserverNfs->stop(
                                command_interface => $ci,
                      'vserver' => $vserver,
                                ...);

 or

    $VserverNfs_obj->stop(...);

(Class or instance method)
This method is used to stop the NFS service of a Vserver.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by the
subcommand in ONTAP
Applicable for CMode CLI

=item C<< "extended_query" => $string>>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@_,
        cli_cmd => 'vserver nfs stop'
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_stop_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub stop

sub _stop_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'vserver_nfs_stop',
    );
    $Log->exit() if $may_exit;
} ## end sub _stop_cmode_cli

=head2 start

    NACL::C::VserverNfs->start(
                                command_interface => $ci,
                         'vserver' => $vserver,
                                ...);

 or

    $VserverNfs_obj->start(...);

(Class or instance method)
This method is used to start the NFS service of a Vserver.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by the
subcommand in ONTAP
Applicable for CMode CLI

=item C<< "extended_query" => $string>>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@_,
        cli_cmd => 'vsrever nfs start'
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub start

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'vserver_nfs_start',
    );
    $Log->exit() if $may_exit;
} ## end sub _start_cmode_cli

=head2 status

    $status = NACL::C::VserverNfs->status(
                                command_interface => $ci,
                            'vserver' => $vserver,
                                ...);

                                 or

    if ( $VserverNfs_obj->status(...) ){
        ...
    }

(Class or instance method)
This method is used to display the status of the NFS service of a Vserver.

Returns 1 if nfs server is running/enabled and 0 if nfs server is
not running/disabled.

Uses a CMode CLI,CMode ZAPI APISet.

Invokes 'nfs-status' API for CMode ZAPI

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

=item C<< 'is-enabled' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-enabled

=item C<< 'is-drained' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-drained

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string>>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub status {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( is-drained is-enabled
                    )
            ),
        },
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_status_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_status_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub status

sub _status_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( is-drained vserver is-enabled
            extended_query )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    $pkg_or_obj->_handle_cli_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $cmode_args{vserver},
        command_interface => $command_interface,
        frontend_method   => 'status',
    );

    $pkg_or_obj->_api_call_log_debug_opts(
        api      => 'vserver_nfs_status',
        api_opts => \%cmode_args
    );

    my $response = $apiset->vserver_nfs_status(%cmode_args);
    my $output   = $response->get_raw_output();

    $Log->exit() if $may_exit;

    return ($output =~ /is\s+running/i) ? 1 : 0;

} ## end sub _status_cmode_cli

sub _status_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my $vserver           = $opts{vserver};

    my %cmd_args;
    $pkg_or_obj->_handle_zapi_vserver_context(
        api_opts          => \%cmd_args,
        vserver           => $vserver,
        command_interface => $command_interface
    );

    my $response = $apiset->nfs_status(%cmd_args);
    my $output   = $response->get_parsed_output();

    $Log->exit() if $may_exit;

    return $output->[0]->{'is-enabled'} eq "true" ? 1 : 0;
} ## end sub _status_cmode_zapi

sub _convert_enable_disable_to_true_false {
    my $cmode_args = shift;
    foreach my $key (keys %{$cmode_args}) {
        if ($cmode_args->{$key} eq "enabled") {
            $cmode_args->{$key} = "true";
        } elsif ($cmode_args->{$key} eq "disabled") {
            $cmode_args->{$key} = "false";
        }
    } ## end foreach my $key ( keys %{$cmode_args...})
    return %$cmode_args;
} ## end sub _convert_enable_disable_to_true_false


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (vserver => {type => SCALAR | UNDEF, optional => 1});
}

1;
