# $Id: //depot/prod/test/nacldev/lib/NACL/C/VserverPeer.pm#18 $

#
# Copyright (c) 2001-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverPeer Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverPeer

=head1 DESCRIPTION

C<NACL::C::VserverPeer> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverPeer.  A related class is
L<NACL::CS::VserverPeer|lib-NACL-CS-VserverPeer-pm>, which represents
the past state of an ONTAP VserverPeer.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 peer_vserver

=cut

package NACL::C::VserverPeer;

use strict;
use warnings;

use NACL::CS::VserverPeer;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::Exceptions::VserverPeer::AlreadyExists;
use NACL::C::Exceptions::VserverPeer::DoesNotExist;
use NACL::C::Exceptions::VserverPeer::RelationshipIsInUseBySnapmirror;
use NACL::C::Exceptions::Vserver::DoesNotExist;
use NATE::BaseException qw(:try);
use NACL::InstantComponent qw(instant_component);
use NACL::Global;

=head1 METHODS

=head2 new

  my $VserverPeer = NACL::C::VserverPeer->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
                                   'peer-vserver' => $peer_vserver,
                                  );

Return a new VserverPeer component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'peer_vserver',
];

=head2 find

    my $VserverPeer_obj  = NACL::C::VserverPeer->find(command_interface => $ci, ...);
    my @VserverPeer_objs = NACL::C::VserverPeer->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my VserverPeer_state = $VserverPeer_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverPeer would return
a NACL::CS::VserverPeer object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 resume

    NACL::C::VserverPeer->resume(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'peer-vserver' => $peer_vserver,
                                ...);

                                 or

    $VserverPeer_obj->resume(...);

(Class or instance method)
This method is used to resume a Vserver peering relationship.

Uses a CMode CLI or CMode ZAPI APISet.
  Invokes 'vserver-peer-resume' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : vserver

=item C<< 'peer-vserver' => $peer_vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : peer-vserver

=item C<< 'force' => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : force

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< "job_component" => \$scalar >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver peer resume" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VserverPeer::DoesNotExist>

This type of exception gets thrown if an attempt is made to resume a
non-existing vserver peering relationship


=back

=back

=cut

sub resume {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( force
                    )
            ),
            job_component => {type => SCALARREF, optional => 1},
        },
        cli_cmd => 'vserver peer resume',
    );
    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_resume_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_resume_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VserverPeer::resume',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception     = $_[0];
        my $doesnot_exist = 'NACL::C::Exceptions::VserverPeer::DoesNotExist';
        $doesnot_exist->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
} ## end sub resume

sub _resume_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api => 'vserver_peer_resume',
    );

    $Log->exit() if $may_exit;
} ## end sub _resume_cmode_cli

sub _resume_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);

    my $allowed_cmode_options = [
        qw( vserver peer-vserver force
            )
    ];

    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api   => 'vserver_peer_resume',
        copy  => $allowed_cmode_options,
    );

    $Log->exit() if $may_exit;
} ## end sub _resume_cmode_zapi

=head2 reject

    NACL::C::VserverPeer->reject(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'peer-vserver' => $peer_vserver,
                                ...);

                                 or

    $VserverPeer_obj->reject(...);

(Class or instance method)
This method is used to reject a Vserver peering relationship.

Uses a CMode CLI or CMode ZAPI APISet.
  Invokes 'vserver-peer-reject' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : vserver

=item C<< 'peer-vserver' => $peer_vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : peer-vserver

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< "job_component" => \$scalar >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver peer reject" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VserverPeer::DoesNotExist>

This type of exception gets thrown if an attempt is made to reject a
non-existing vserver peering relationship

=back

=back

=cut

sub reject {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            job_component => {
                type     => SCALARREF,
                optional => 1
                },
        },
        cli_cmd         => 'vserver peer reject',
    );
    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_reject_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_reject_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                }
            ],
            frontend => 'NACL::C::VserverPeer::reject',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception     = $_[0];
        my $doesnot_exist = 'NACL::C::Exceptions::VserverPeer::DoesNotExist';
        $doesnot_exist->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
} ## end sub reject

sub _reject_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api => 'vserver_peer_reject',
    );
    $Log->exit() if $may_exit;
} ## end sub _reject_cmode_cli

sub _reject_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);

    my $allowed_cmode_options = [qw( vserver peer-vserver )];

    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api  => 'vserver_peer_reject',
        copy => $allowed_cmode_options,
    );
    $Log->exit() if $may_exit;
} ## end sub _reject_cmode_zapi

=head2 show_all

    my $show_all_obj = NACL::C::VserverPeer->show_all(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @show_all_objs = NACL::C::VserverPeer->show_all(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $show_all_obj = $VserverPeer_obj->show_all();

(Class or Instance method)
This method returns objects of the type L<NACL::CS::VserverPeerShowAll|lib-NACL-CS-VserverPeerShowAll-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch>
(requested_fields, filter, allow_empty) and has all of the other properties of
fetch methods (callable in scalar or list context, throws a NoElementsFound
exception if no matching elements are found)

=cut

sub show_all {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::VserverPeerShowAll;
    my @state_objs = NACL::CS::VserverPeerShowAll->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub show_all

=head2 modify

    NACL::C::VserverPeer->modify(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'peer-vserver' => $peer_vserver,
                                ...);

                                 or

    $VserverPeer_obj->modify(...);

(Class or instance method)
This method is used to modify a Vserver peering relationship.

Uses a CMode CLI or CMode ZAPI APISet.
  Invokes 'vserver-peer-modify' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : vserver

=item C<< 'peer-vserver' => $peer_vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : peer-vserver

=item C<< 'peer-cluster-name' => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : peer-cluster-name

=item C<< 'peer-type' => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : peer-type

=item C<< 'league-name' => $string >>

  Applicable only for CMode CLI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'applications' => [ $application1, $application2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : applications

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< "job_component" => \$scalar >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver peer modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.


=item C<NACL::C::Exceptions::VserverPeer::DoesNotExist>

This type of exception gets thrown if an attempt is made to modify a
non-existing vserver peering relationship

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(qw( peer-cluster peer-type league-name )),
            applications  => {type => ARRAYREF,  optional => 1},
            job_component => {type => SCALARREF, optional => 1},
        },
        cli_cmd => 'vserver peer modify',
    );
    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_modify_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_modify_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VserverPeer::modify',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception     = $_[0];
        my $doesnot_exist = 'NACL::C::Exceptions::VserverPeer::DoesNotExist';
        $doesnot_exist->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'vserver_peer_modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);

    my $allowed_cmode_options = [
        qw( peer-cluster-name vserver peer-vserver peer-type
            applications
            )
    ];

    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api   => 'vserver_peer_modify',
        copy  => $allowed_cmode_options,
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 accept

    NACL::C::VserverPeer->accept(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'peer-vserver' => $peer_vserver,
                                ...);

                                 or

    $VserverPeer_obj->accept(...);

(Class or instance method)
This method is used to accept a pending Vserver peering relationship.

Uses a CMode CLI or CMode ZAPI APISet.
  Invokes 'vserver-peer-accept' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : vserver

=item C<< 'peer-vserver' => $peer_vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : peer-vserver

=item C<< "job_component"=> \$scalar >>

Applicable for CMode CLI/ZAPI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver peer accept" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.


=item C<NACL::C::Exceptions::VserverPeer::DoesNotExist>

This type of exception gets thrown if an attempt is made to accept a
non-existing vserver peering relationship

=back

=back

=cut

sub accept {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            {job_component => {type => SCALARREF, optional => 1},},
        cli_cmd => 'vserver peer accept',
    );
   my $is_enabled = is_vserver_simplicity_enabled($pkg_or_obj,%opts);
   if($is_enabled) {
       $Log->warn("Skipping accept since vserver simplicity feature is enabled");
    } else {
    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_accept_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_accept_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VserverPeer::accept',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception     = $_[0];
        my $doesnot_exist = 'NACL::C::Exceptions::VserverPeer::DoesNotExist';
        $doesnot_exist->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    }
    $Log->exit() if $may_exit;
} ## end sub accept

sub _accept_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'vserver_peer_accept',
    );
    $Log->exit() if $may_exit;
} ## end sub _accept_cmode_cli

sub _accept_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);

    my $allowed_cmode_options = [
        qw( vserver peer-vserver
            )
    ];

    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api   => 'vserver_peer_accept',
        copy  => $allowed_cmode_options,
    );
    $Log->exit() if $may_exit;
} ## end sub _accept_cmode_zapi

=head2 suspend

    NACL::C::VserverPeer->suspend(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'peer-vserver' => $peer_vserver,
                                ...);

                                 or

    $VserverPeer_obj->suspend(...);

(Class or instance method)
This method is used to suspend a Vserver peering relationship.

Uses a CMode CLI or CMode ZAPI APISet.
  Invokes 'vserver-peer-suspend' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : vserver

=item C<< 'peer-vserver' => $peer_vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : peer-vserver

=item C<< 'force' => $string >>

(Optional)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : force

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver peer suspend" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.


=item C<NACL::C::Exceptions::VserverPeer::DoesNotExist>

This type of exception gets thrown if an attempt is made to suspend a
non-existing vserver peering relationship

=back

=back

=cut

sub suspend {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( force
                    )
            ),
            job_component => {type => SCALARREF, optional => 1},
        },
        cli_cmd => 'vserver peer suspend',
    );
    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_suspend_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_suspend_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VserverPeer::suspend',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception     = $_[0];
        my $doesnot_exist = 'NACL::C::Exceptions::VserverPeer::DoesNotExist';
        $doesnot_exist->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
} ## end sub suspend

sub _suspend_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api => 'vserver_peer_suspend',
    );
    $Log->exit() if $may_exit;
} ## end sub _suspend_cmode_cli

sub _suspend_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);

    my $allowed_cmode_options = [
        qw( vserver peer-vserver force
            )
    ];

    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api  => 'vserver_peer_suspend',
        copy => $allowed_cmode_options,
    );
    $Log->exit() if $may_exit;
} ## end sub _suspend_cmode_zapi

=head2 create

    my $VserverPeer_obj    = NACL::C::VserverPeer->create(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'peer-vserver' => $peer_vserver,
                                ...);

(Class method)
This method is used to create a new Vserver peering relationship.

Uses a CMode CLI or CMode ZAPI APISet.
  Invokes 'vserver-peer-create' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : vserver

=item C<< 'peer-vserver' => $peer_vserver >>

(Required)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : peer-vserver

=item C<< 'peer-type' => $string >>

  Applicable only for CMode CLI

=item C<< 'peer-cluster' => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : peer-cluster

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'applications' => [ $application1, $application2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : applications

=item C<< "job_component"=> \$scalar >>

  Applicable for CMode CLI/ZAPI

=item C<< "extended_query" => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver peer create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.


=item C<NACL::C::Exceptions::Vserver::DoesNotExist>

This type of exception gets thrown if an attempt is made to create a
vserver peering relationship on a non-existing vserver

=item C<NACL::C::Exceptions::VserverPeer::AlreadyExists>

This type of exception gets thrown if an attempt is made to create a
vserver peering relationship that already exists.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = (@_);
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(qw( peer-cluster peer-type )),
            "applications" => {type => ARRAYREF,  optional => 1},
            job_component  => {type => SCALARREF, optional => 1},
        },
        cli_cmd => 'vserver peer create',
    );
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_create_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_create_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VserverPeer::create',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception     = $_[0];
        my $doesnot_exist = 'NACL::C::Exceptions::Vserver::DoesNotExist';
        $doesnot_exist->detect_convert_and_throw(exception => $exception);
        my $already_exist = 'NACL::C::Exceptions::VserverPeer::AlreadyExists';
        $already_exist->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $opts{'peer-vserver'} = $opts{'peer-vserver'}->{'vserver'} if(ref($opts{'peer-vserver'}));
    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $opts{'vserver'},
        peer_vserver      => $opts{'peer-vserver'},
     );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = (@_);
    $pkg->_base_backend_cmode_cli(@args,
        api     => 'vserver_peer_create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = (@_);

    my $allowed_cmode_options = [
        qw( vserver peer-vserver peer-cluster
            applications
            )
    ];

    $pkg->_base_backend_cmode_zapi(@args,
        api  => 'vserver_peer_create',
        copy => $allowed_cmode_options,
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

=head2 delete

    NACL::C::VserverPeer->delete(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'peer-vserver' => $peer_vserver,
                                ...);

                                 or

    $VserverPeer_obj->delete(...);

(Class or instance method)
This method is used to delete a Vserver peering relationship.

Uses a CMode CLI or CMode ZAPI APISet.
  Invokes 'vserver-peer-delete' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : vserver

=item C<< 'peer-vserver' => $peer_vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : peer-vserver

=item C<< 'force' => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : force

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "job_component" => \$scalar >>

(Optional)
  Applicable only for CMode CLI

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver peer delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VserverPeer::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete a
non-existing vserver peering relationship.

=item C<NACL::C::Exceptions::VserverPeer::RelationshipIsInUseBySnapmirror>

This type of exception gets thrown if an attempt is made to delete a
vserver peering relationship when it is use by snapmirror.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(qw( force  )),
            job_component => {type => SCALARREF, optional => 1},
        },
        cli_cmd => 'vserver peer delete',
    );
    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VserverPeer::delete',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception     = $_[0];
        my $doesnot_exist = 'NACL::C::Exceptions::VserverPeer::DoesNotExist';
        $doesnot_exist->detect_convert_and_throw(exception => $exception);
        NACL::C::Exceptions::VserverPeer::RelationshipIsInUseBySnapmirror
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api => 'vserver_peer_delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = (@_);

    my $allowed_cmode_options = [
        qw( vserver peer-vserver force
            )
    ];

    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api  => 'vserver_peer_delete',
        copy => $allowed_cmode_options,
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 repeer

    NACL::C::VserverPeer->repeer(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to migrate existing Vserver peer relationships from partner MCC cluster.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'source-cluster' => $string >>

Source Cluster Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "vserver peer repeer" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub repeer {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( source-cluster
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'vserver peer repeer',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_repeer_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverPeer::repeer',
    );

    $Log->exit() if $may_exit;
}

sub _repeer_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'vserver_peer_repeer',);
    $Log->exit() if $may_exit;
}

sub is_vserver_simplicity_enabled {
    $Log->enter() if $may_enter;
     my  ($pkg, %opts) = @_;
     my $is_enabled = 0;

     if ( $opts{command_interface}->version_manager()->is_version_lt(release => $NACL::Global::ONTAP_Release_Harpoon_0) ) {
	$Log->comment("Vserver Peer Simplicity feature has been introduced HN.0 version onwards only");
	return $is_enabled;
     }

     instant_component(command_interface => $opts{command_interface},
             component_name => 'NACL::C::VserverPeerPermission');
     my @vs_permissions = NACL::CS::VserverPeerPermission->fetch(
             command_interface => $opts{command_interface},
             allow_empty => 1,
             );
     foreach my $vs_permission(@vs_permissions) {
        if(($vs_permission->vserver() eq $opts{vserver}) || ($vs_permission->vserver() eq '"*"')) {
            $Log->comment("vserver simplicity feature is enabled");
            $is_enabled = 1;
            last;
        }
     }
     return $is_enabled;
     $Log->exit() if $may_exit;
}
__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver'      => {type => SCALAR},
        'peer-vserver' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
