# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverServicesLdap Component Module
## @author Rupamjyoti.Baruah@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverServicesLdap

=head1 DESCRIPTION

C<NACL::C::VserverServicesLdap> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP Vserver Services LDAP.  A related class is
L<NACL::CS::VserverServicesLdap|lib-NACL-CS-VserverServicesLdap-pm>, which
represents the past state of an ONTAP Vserver Services LDAP

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

The name of the vserver, used to identify it uniquely.

=cut

package NACL::C::VserverServicesLdap;

use strict;
use warnings;
use NACL::CS::VserverServicesLdap;
use NACL::ComponentUtils qw (_optional_scalars);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base 'NACL::C::Component::ONTAP';
use Data::Dumper;
use Params::Validate qw(validate_with SCALAR UNDEF);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use NACL::C::Exceptions::VserverServicesLdap::AlreadyExists ();
use NACL::C::Exceptions::VserverServicesLdap::DoesNotExist  ();
use NACL::C::Exceptions::VserverServicesLdap::ClientConfigDoesNotExist;

=head1 METHODS

=head2 new

  my $ldap = NACL::C::VserverServicesLdap->new(
                                              command_interface=>$ci,
                                              vserver =>$vserver_name
             );

Return a new Vserver Services LDAP component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 create

  NACL::C::VserverServicesLdap->create( command_interface=>$ci,
                                        vserver => $vserver
           );

(Class method) Request creation of a new Vserver LDAP element, and  return a
new  Vserver LDAP Client component object that refers to it.  Does not wait
for Vserver LDAP creation to complete.

Uses a CMode-CLI or CMode-ZAPI APISet.
  Invokes 'ldap-config-create' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$string >>

(Required) This parameter specifies the virtual server on which to create the
LDAP configuration.
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : vserver

=item C<< client-config=>$string >>

(Required) This parameter specifies the name of the LDAP client configuration
to associate with the LDAP configuration.
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : client-config

=item C<< client-enabled=>$boolean >>

(Optional) This specifies the whether a LDAP Client to be enabled or not.
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : client-enabled

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI


=item C<< '*' >>

This method takes all other options supported by
"vserver services ldap create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverServicesLdap::AlreadyExists>

This type of exception gets thrown if an attempt is made to create a
vserver services ldap that already exists.

=item C<NACL::C::Exceptions::VserverServicesLdap::ClientConfigDoesNotExist>

This exception is thrown when the provided LDAP configuration does not exist.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        additional_spec =>
            {_optional_scalars(qw(client-config client-enabled))},
        cli_cmd => 'vserver services ldap create',
    );

    my ($ret, $vserver);
    try {
        $ret = $pkg->call_on_apiset(
            %opts,
            vserver_operated_on => \$vserver,
            choices => [
                {   method    => '_create_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_create_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VserverServicesLdap::create',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        foreach my $type (qw(AlreadyExists ClientConfigDoesNotExist)) {
            my $full_exception = "NACL::C::Exceptions::VserverServicesLdap::$type";
            $full_exception->detect_convert_and_throw(exception => $exception);
        }
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_cli(%opts, api => 'vserver_services_ldap_create');
    
    my $vserver = ${$opts{vserver_operated_on}};
    my $obj = $pkg->new(
        vserver           => $vserver,
        command_interface => $opts{command_interface},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    
    # CLI defaults it to "true", ZAPI does not. (burt 806405)
    $opts{'client-enabled'} ||= 'true';
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'ldap-config-create',
        copy => [qw(client-config client-enabled)],
    );

    my $vserver = ${$opts{vserver_operated_on}};
    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $vserver
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub _create_cmode_zapi

=head2 delete

  NACL::C::VserverServicesLdap->delete(command_interface=>$ci,
                                       vserver=>$string);

                            or

  $ldap_object->delete(...);

(Class or instance method) This method deletes a vserverLDAP element.

Uses a CMode-CLI or CMode-ZAPI APISet.
  Invokes 'ldap-config-delete' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$string >>

(Required for class method, Not Applicable for instance method)
(Required) This parameter specifies the virtual server on which the LDAP
configuration created.
  Applicable only for CMode CLI

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverServicesLdap::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete
vserver services ldap which does not exists.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver services ldap delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params   => \@args,
        cli_cmd  => 'vserver services ldap delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VserverServicesLdap::delete'
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverServicesLdap::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli{
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    $pkg_or_obj->_base_backend_cmode_cli(
        @_,
        api => 'vserver_services_ldap_delete',
    );

    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
                       frontend_method => 'delete',
                       api             => 'ldap-config-delete',
                 );

    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 modify

  NACL::C::VserverServicesLdap->modify(command_interface=>$ci,
                                       vserver=>$string,
            ...);

                            or

  $ldap_object->modify(...);

(Class or instance method) This method modifies a vserver LDAP element.

Uses a CMode-CLI or CMode-ZAPI APISet.
  Invokes 'ldap-config-modify' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$string >>

(Required for class method, Not Applicable for instance method)
(Required) This parameter specifies the virtual server on which the LDAP
configuration created.
  Applicable only for CMode CLI

=item C<< client-config=>$string >>

(Required for class method, Not Applicable for instance method)
(Optional) This parameter specifies the name of the LDAP client configuration
to associate with the LDAP configuration.
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : client-config

=item C<< client-enabled=>$boolean >>

(Required for class method, Not Applicable for instance method)
(Optional) This specifies the whether a LDAP Client to be enabled or not.
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : client-enabled

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver services ldap modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            {_optional_scalars(qw(client-config client-enabled))},
        cli_cmd => 'vserver services ldap modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverServicesLdap::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api => 'vserver_services_ldap_modify',
    );

    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $cmode_ldap_options = [qw(client-config client-enabled)];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
                     copy            => $cmode_ldap_options,
                     frontend_method => 'modify',
                     api             => 'ldap-config-modify',
                  );
    $Log->exit() if $may_exit;

} ## end sub _modify_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (vserver => {type => SCALAR | UNDEF, optional => 1},);
}

1;
