# $Id$
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary VserverServicesNisDomain Component Module
## @author Rupamjyoti.Baruah@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverServicesNisDomain

=head1 DESCRIPTION

C<NACL::C::VserverServicesNisDomain> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access to an
ONTAP Vserver Services NIS Domain.  A related class is
L<NACL::CS::VserverServicesNisDomain|lib-NACL-CS-VserverServicesNisDomain-pm>, which
represents the past state of a Vserver Services NIS Domain

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

The name of the vserver, used to identify it uniquely.

=head2 domain

This specifies the NIS domain for which a configuration is to be created.

=cut

package NACL::C::VserverServicesNisDomain;

use strict;
use warnings;
use NACL::CS::VserverServicesNisDomain;
use NACL::ComponentUtils qw (_optional_scalars);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::C::Exceptions::VserverServicesNisDomain::AlreadyExists ();
use NACL::C::Exceptions::VserverServicesNisDomain::DoesNotExist  ();
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

=head1 METHODS

=head2 new

  my $nis_domain = NACL::C::VserverServicesNisDomain->new(
                                                       command_interface=>$ci,
                                                       vserver =>$vserver_name,
                                                       domain =>$domain,
                                                       );

Return a new Vserver Services NIS Domain component object with the given
attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'domain',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 create

  NACL::C::VserverServicesNisDomain->create( command_interface=>$ci,
                                            vserver => $vserver
                                            );

(Class method) Request creation of a new Vserver Services NIS Domain element,
 and  return a new  Vserver Services NIS Domain component object that refers
 to it. Does not wait for Vserver NIS Domain creation to complete.

Uses a CMode-CLI or CMode-ZAPI APISet or 7Mode CLI.
  Invokes 'options' command with option 'nis.enable =>on' for 7Mode CLI.
  Invokes 'options nis.enable off' command with option 'nis.domainname => $opts{domain}' for 7Mode CLI.
  Invokes 'options nis.enable off' command with option 'nis.servers => $opts{servers}' for 7Mode CLI.
  Invokes 'nis-create' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$string >>

(Required) This specifies the virtual server on which the NIS domain
configuration is to be created.
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< domain=>$string >>

(Required) This specifies the NIS domain for which a configuration is to be
created.
  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: nis-domain

=item C<< active=>$string >>

(Optional) Active Domain
  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: is-active

=item C<< servers => [ $server1, $server2, ... ] >>

(Required) This specifies the IP address or addresses of one or more NIS
servers for the NIS domain configuration. You can specify multiple IP
addresses by using a comma-separated list.
  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: nis-servers

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverServicesNisDomain::AlreadyExists>

This type of exception gets thrown if an attempt is made to create
vserver services nis that already exists.


=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver services nis-domain create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            servers => {type => ARRAYREF, optional => 1},
            _optional_scalars(qw (active)),
        },
        cli_cmd => 'vserver services nis-domain create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_create_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_create_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
                {   method    => "_create_7mode_cli",
                    interface => "CLI",
                    set       => "7Mode",
                },
            ],
            frontend => 'NACL::C::VserverServicesNisDomain::create',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverServicesNisDomain::AlreadyExists
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    my $obj = $pkg->new(
        vserver           => $opts{vserver},
        domain            => $opts{'domain'},
        command_interface => $opts{command_interface},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,
        api => 'vserver_services_nis_domain_create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    $apiset->options("nis.enable"     => "on");
    $apiset->options("nis.domainname" => $opts{domain});
    if (defined $opts{servers}) {
        $opts{servers} = join ',', @{$opts{servers}};
    }
    $apiset->options("nis.servers" => $opts{servers});
    my $obj = $pkg->new(
        domain            => $opts{domain},
        command_interface => $command_interface
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub _create_7mode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_zapi(@args,
        api    => 'nis_create',
        map    => {
            "active"  => "is-active",
            "domain"  => "nis-domain",
            "servers" => "nis-servers"
        },
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

=head2 delete

  NACL::C::VserverServicesNisDomain->delete(command_interface=>$ci,
                                            vserver => $vserver,
                                            domain => $domain,
                                            ...);

                            or

  $nis_domain_object->delete(...);

(Class or instance method) This method deletes a Vserver Services NIS Domain
element.

Uses a CMode-CLI or CMode-ZAPI APISet or 7Mode CLI.
  Invokes 'options nis.enable off' command for 7Mode CLI.
  Invokes 'nis-destroy' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=>$string >>

(Required for class method, Not Applicable for instance method)
(Required) This specifies the virtual server on which the NIS domain
configuration is been created.
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< domain=>$string >>

(Required for class method, Not Applicable for instance method)
(Required) This specifies the NIS domain for which a configuration is been
created.
  Applicable for CMode CLI/ZAPI
  Maps to:
      CMode ZAPI: nis-domain

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverServicesNisDomain::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete
vserver services nis which does not exists.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver services nis-domain delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my $ret;
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver services nis-domain delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
                {   method    => '_delete_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
            ],
            frontend => 'NACL::C::VserverServicesNisDomain::delete',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverServicesNisDomain::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;

} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api => 'vserver_services_nis_domain_delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    $apiset->options("nis.enable" => "off");
    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api => 'nis_destroy',
        map => {"domain" => "nis-domain"},
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 modify

   NACL::C::VserverServicesNisDomain->modify(command_interface => $ci,
                                             vserver => $vserver,
                                             domain  => $domain,
                                            ...);

                or

    $nis_domain_object->modify(...);

(Class or instance method)
This method is used to modify a NIS domain configuration.

Uses a CMode CLI or CMode-ZAPI APISet or 7Mode CLI.
  Invokes 'options nis.enable off' API with command 'nis.enable =>off' for 7Mode CLI.
  Invokes 'options nis.enable off' API with command 'nis.domainname => $opts{domain}' for 7Mode CLI.
  Invokes 'options nis.enable off' API with command 'nis.servers => $opts{servers}' for 7Mode CLI.
  Invokes 'options nis.enable on' API with command 'nis.enable =>on' for 7Mode CLI.
  Invokes 'nis-modify' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "domain" => $domain >>

(Required for class method, Not Applicable for instance method)
This specifies the NIS domain for which a configuration is been
created.
  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: nis-domain

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)
This specifies the virtual server on which the NIS domain
configuration is been created.
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< "active" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: is-active

=item C<< "servers" => [ $server1, $server2, ... ] >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: nis-servers

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver services nis-domain modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            servers => {type => ARRAYREF, optional => 1},
            _optional_scalars(qw (active)),
        },
        cli_cmd => 'vserver services nis-domain modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_modify_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        frontend => 'NACL::C::VserverServicesNisDomain::modify',
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'vserver_services_nis_domain_modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $apiset = delete $opts{apiset};
    if ($opts{active} =~ /false/i) {
        $apiset->options("nis.enable" => "off");
    } else {
        $apiset->options("nis.domainname" => $opts{domain});

        if (defined $opts{servers}) {
            $opts{servers} = join ',', @{$opts{servers}};
        }
        $apiset->options("nis.servers" => $opts{servers});
        $apiset->options("nis.enable"  => "on");
    } ## end else [ if ( $opts{active} =~ ...
    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api    => 'nis_modify',
        map    => {
            "active"  => "is-active",
            "domain"  => "nis-domain",
            "servers" => "nis-servers"
        },
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 show_statistics

    my $show_statistics_obj = NACL::C::VserverServicesNisDomain->show_statistics(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_statistics_objs = NACL::C::VserverServicesNisDomain->show_statistics(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_statistics_obj = $VserverServicesNisDomain_obj->show_statistics(); 

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-VserverServicesNisDomainShowStatistics-pm/ATTRIBUTES>
of NACL::CS::VserverServicesNisDomainShowStatistics as input. 
This method returns objects of the type L<NACL::CS::VserverServicesNisDomainShowStatistics|lib-NACL-CS-VserverServicesNisDomainShowStatistics-pm>.

=cut

sub show_statistics {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::VserverServicesNisDomainShowStatistics;
    my @state_objs
        = NACL::CS::VserverServicesNisDomainShowStatistics->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        vserver => {type => SCALAR | UNDEF, optional => 1},
        domain  => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
