#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the various AntivirusOnDemandCommand modules
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::AntivirusOnDemandCommand;

use strict;
use warnings;

use Params::Validate qw(validate_with SCALAR HASHREF OBJECT);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Exceptions::Argument qw(:try);

# Tags exported:
# "all"
# "exceptions" : All of the exceptions-related methods
# "cmd_line" : The methods required to translated "command-line" tp
# the newer syntax

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(
        _turn_cmd_line_into_opts
        _handle_full_cmd_line
        _convert_exception
    );
    our %EXPORT_TAGS = (
        'all'        => \@EXPORT_OK,
        'exceptions' => [
                qw(_convert_exception)
        ],
        'cmd_line' => [qw(_turn_cmd_line_into_opts _handle_full_cmd_line)],
    );
}

sub _turn_cmd_line_into_opts {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            cmd_line => { type => SCALAR },
            args     => { type => HASHREF }
        }
    );
    my $cmd_line = $opts{cmd_line};
    my $args     = $opts{args};
    while ( $cmd_line =~ s/\W*\-([-\w]+)\s*// ) {
        my $key = $1;
        if ( $cmd_line =~ s/^(.*?)(\W*\-)/$2/ ) {
            $args->{$key} = $1;
        } else {
            $args->{$key} = $cmd_line;
        }
    }
    $Log->exit() if $may_exit;
}

sub _handle_full_cmd_line {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %method_opts = validate_with(
        params => \@args,
        spec   => {
            opts   => { type => HASHREF },
            method => { type => SCALAR, optional => 1 },
        }
    );
    my $method  = $method_opts{method};
    my $opts    = $method_opts{opts};
    my $command = delete $opts->{'command-line'};
    $command =~ s/^"//;
    $command =~ s/"$//;

    my $is_c;
    if ( $pkg_or_obj->isa('NACL::C::Component::ONTAP') ) {
        $is_c = 1;
        unless ( defined $method ) {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw( "The argument 'method' "
                    . "is mandatory when '_handle_full_cmd_line' is invoked "
                    . 'with a component package/object' );
        }
    }

    unless ( defined $command ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw( "The 'command-line' argument "
                . "is mandatory in the call to '$method' but was not provided"
        );
    }
    my $ret;
    if ( $command =~ /\s*scan\s+\-file\s+(\S+)\s+(\S+)\s*(.*)/ ) {
        my ( $vserver, $scan_path, $cmd_line ) = ( $1, $2, $3 );
        $opts->{vserver} = $vserver;
        $opts->{'scan-path'} = $scan_path;
        $opts->{'scan-type'} = 'file' unless ($is_c);
        $pkg_or_obj->_turn_cmd_line_into_opts(
            cmd_line => $cmd_line,
            args     => $opts
        );
        if ($is_c) {
            require NACL::C::AntivirusOnDemandCommandScanFile;
            $ret = NACL::C::AntivirusOnDemandCommandScanFile->$method(%$opts);
        }
    } elsif ( $command =~ /\s*scan\s+\-dir\s+(\S+)\s+(\S+)\s*(.*)/ ) {
        my ( $vserver, $scan_path, $cmd_line ) = ( $1, $2, $3 );
        $opts->{vserver} = $vserver;
        $opts->{'scan-path'} = $scan_path;
        $opts->{'scan-type'} = 'dir' unless ($is_c);
        $pkg_or_obj->_turn_cmd_line_into_opts(
            cmd_line => $cmd_line,
            args     => $opts
        );
        if ($is_c) {
            require NACL::C::AntivirusOnDemandCommandScanDir;
            $ret = NACL::C::AntivirusOnDemandCommandScanDir->$method(%$opts);
        }
    } elsif ( $command =~ /\s*scan\s+\-vserver\s+(\S+)\s*(.*)/ ) {
        my ( $vserver, $cmd_line ) = ( $1, $2 );
        $opts->{vserver} = $vserver;
        $opts->{'scan-type'} = 'vserver' unless ($is_c);
        $pkg_or_obj->_turn_cmd_line_into_opts(
            cmd_line => $cmd_line,
            args     => $opts
        );
        if ($is_c) {
            require NACL::C::AntivirusOnDemandCommandScanVserver;
            $ret = NACL::C::AntivirusOnDemandCommandScanVserver->$method(
                %$opts);
        }
    } elsif ( $command =~ /\s*scan\s+\-cluster\s*(.*)/ ) {
        my $cmd_line = $1;
        $opts->{'scan-type'} = 'cluster' unless ($is_c);
        $pkg_or_obj->_turn_cmd_line_into_opts(
            cmd_line => $cmd_line,
            args     => $opts
        );
        if ($is_c) {
            require NACL::C::AntivirusOnDemandCommandScanCluster;
            $ret = NACL::C::AntivirusOnDemandCommandScanCluster->$method(
                %$opts);
        }
    } else {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw( "The 'command-line' argument "
                . "was passed as:\n$command\nThis does not match any of "
                . "the valid patterns, which are:\n"
                . "scan -file <vserver> <filename> [other options]\n"
                . "scan -dir <vserver> <pathname> [other options]\n"
                . "scan -vserver <vserver> [other options]\n"
                . "scan -cluster [other options]\n" );
    }
    $Log->exit() if $may_exit;
    return $ret;
}
sub _convert_exception {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            exception => _exception_spec()
        },
    );
    my $comp = $pkg;
    $comp =~ s/.*::(.*)/$1/;
    foreach my $type (qw(AlreadyExists DoesNotExist InUse)) {
	 my $ex_type = "NACL::C::Exceptions::${comp}::$type";
         $ex_type->detect_convert_and_throw( exception => $opts{exception} );
    }
    $Log->exit() if $may_exit;
    $opts{exception}->throw();
}

sub _exception_spec {
    return {
        exception => {
            type => OBJECT,
            isa  => 'NACL::APISet::Exceptions::ResponseException'
        }
    };
}

1;
