#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the Snapmirror Component and ComponentState
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::Snapmirror;

use strict;
use warnings;
use feature 'state';

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(_common_zapi_copy _common_zapi_map _update_iter_input
        _type_val_map _update_type_for_zapi _lc_fields_zapi
        _ucfirst_state_fields_zapi
        _update_lc_fields_for_zapi _relationship_capability_mapping
        _update_rel_cap_for_zapi _update_throttle_for_zapi 
        _zapi_fields_underscored_value _update_underscore_value_for_zapi
        _non_iter_check _construct_single_path _destination_common_zapi_copy
        _destination_common_zapi_map _defined_simple_value _status_cli_zapi_map
        _status_zapi_cli_map
    );
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);

use NACL::Exceptions::InvalidChoice;

sub _common_zapi_map {
    $Log->enter();
    my %map = (
        'source-path'             => 'source-location',
        'destination-path'        => 'destination-location',
        'throttle'                => 'max-transfer-rate',
        'control-plane'           => 'relationship-control-plane',
        'current-throttle'        => 'current-max-transfer-rate',
        'status'                  => 'relationship-status',
        'state'                   => 'mirror-state',
        'healthy'                 => 'is-healthy',
        'type'                    => 'relationship-type',
        'total-progress'          => 'relationship-progress',
        'op-uuid'                 => 'operation-id',
        'relationship-capability' => 'relationship-control-plane',
        'group-type'              => 'relationship-group-type',
    );
    $Log->exit();
    return wantarray ? %map : {%map};
}

sub _common_zapi_copy {
    $Log->enter();
    my @copy = (
        qw(current-transfer-error current-transfer-priority current-transfer-type
            destination-cluster destination-volume destination-volume-node
            destination-vserver destination-vserver-uuid exported-snapshot
            exported-snapshot-timestamp file-restore-file-count
            file-restore-file-list is-constituent lag-time last-transfer-duration
            last-transfer-end-timestamp last-transfer-error last-transfer-error-codes
            last-transfer-from last-transfer-network-compression-ratio
            last-transfer-size last-transfer-type network-compression-ratio
            newest-snapshot newest-snapshot-timestamp policy policy-type
            progress-last-updated relationship-id schedule
            snapshot-checkpoint snapshot-progress source-cluster source-volume
            source-vserver source-vserver-uuid transfer-snapshot tries
            unhealthy-reason vserver relationship-group-type update-successful-count
            update-failed-count resync-successful-count resync-failed-count
            break-successful-count break-failed-count total-transfer-bytes
            total-transfer-time-secs expand identity-preserve
       )
    );
    $Log->exit();
    return wantarray ? @copy : [@copy];
}

# Perform the various mappings/translations needed to make the values in
# the iter input fields look like the CLI equivalent
sub _update_iter_input {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    state $spec = {opts_ptr => {type => HASHREF}};
    my %opts = validate(@args, $spec);
    $pkg_or_obj->_update_type_for_zapi(%opts);
    $pkg_or_obj->_update_lc_fields_for_zapi(%opts);
    $pkg_or_obj->_update_rel_cap_for_zapi(%opts);
    $pkg_or_obj->_update_throttle_for_zapi(%opts);
    $pkg_or_obj->_update_underscore_value_for_zapi(%opts);
    $Log->exit();
}

# Mapping of the values for the "type" field accepted by the CLI to
# those accepted by ZAPI
sub _type_val_map {
    $Log->enter();
    my %map = (
        'DP'  => 'data_protection',
        'LS'  => 'load_sharing',
        'XDP' => 'vault',
        'RST' => 'restore',
        'TDP' => 'transition_data_protection',
    );
    $Log->exit();
    return wantarray ? %map : {%map};
}

sub _update_type_for_zapi {
    $Log->enter();
    my ($pkg_or_obj, %opts) = @_;
    my $opts_ptr = $opts{opts_ptr};
    
    my $type = $opts_ptr->{type};
    if ( $type ) {
        if ($pkg_or_obj->_defined_simple_value($type)) {
            my %type_map = $pkg_or_obj->_type_val_map();
            $type_map{XDP} = 'vault|extended_data_protection';
            
            # "type" might be a list like "a|b|c". To handle this, we need
            # to apply the translation logic on each value, so split on |
            # to get all values, and then join back on |.
            my @types = split /\|/, $type;
            my @translated_types;
            foreach my $one_type (@types) {
                my $cli_type = uc $one_type;
                if (exists $type_map{$cli_type}) {
                    push @translated_types, $type_map{$cli_type};
                } else {
                    push @translated_types, $one_type;
                }
            }
            $opts_ptr->{type} = join '|', @translated_types;
        }
    }
    $Log->exit();
}

sub _ucfirst_state_fields_zapi {
    return (qw(state));
}

# Values of these CLI fields are mixed-case or begin with an upper-case
# character, but in ZAPI it's all lower-case
sub _lc_fields_zapi {
    my ($pkg_or_obj) = @_;
    
    # "status" is mixed-case, which is why it's not part of the ucfirst list.
    return ($pkg_or_obj->_ucfirst_state_fields_zapi(), 'status');
}

sub _update_lc_fields_for_zapi {
    $Log->enter();
    my ($pkg_or_obj, %opts) = @_;
    my $opts_ptr = $opts{opts_ptr};
    my @lc_fields = $pkg_or_obj->_lc_fields_zapi();
    foreach my $field (@lc_fields) {
        my $val = $opts_ptr->{$field};
        if ($pkg_or_obj->_defined_simple_value($val)) {
            my $lc_val = lc $val;
            $opts_ptr->{$field} = $lc_val;
        }
    }
    $Log->exit();
}

# CLI -> ZAPI mapping of the relationship-capability field.
sub _relationship_capability_mapping {
    return ('"Pre 8.2"' => 'v1', '"8.2 and above"' => 'v2');
}

sub _update_rel_cap_for_zapi {
    $Log->enter();
    my ($pkg_or_obj, %opts) = @_;
    my $opts_ptr = $opts{opts_ptr};
    my $rel_cap = $opts_ptr->{'relationship-capability'};
    
    if ($pkg_or_obj->_defined_simple_value($rel_cap)) {
        my %capability_mapping = $pkg_or_obj->_relationship_capability_mapping();
        my $mapped_value = $capability_mapping{$rel_cap};
        if (defined $mapped_value) {
            $opts_ptr->{'relationship-capability'} = $mapped_value;
        }
    }
    
    $Log->exit();
}

# CLI value of "unlimited" maps to 0 for ZAPI.
sub _update_throttle_for_zapi {
    $Log->enter();
    my ($pkg_or_obj, %opts) = @_;
    my $opts_ptr = $opts{opts_ptr};
    my $throttle = $opts_ptr->{throttle};
    if (defined $throttle && $throttle eq 'unlimited') {
        $opts_ptr->{throttle} = 0;
    }
    $Log->exit();
}

# Value returned by CLI is hyphenated, but ZAPI is underscored
sub _zapi_fields_underscored_value {
    return (qw(policy-type));
}

sub _update_underscore_value_for_zapi {
    $Log->enter();
    my ($pkg_or_obj, %opts) = @_;
    my $opts_ptr = $opts{opts_ptr};
    my @underscore_fields = $pkg_or_obj->_zapi_fields_underscored_value();
    foreach my $field (@underscore_fields) {
        my $val = $opts_ptr->{$field};
        if ($pkg_or_obj->_defined_simple_value($val)) {
            $val =~ s/-/_/g;
            $opts_ptr->{$field} = $val;
        }
    }
    $Log->exit();
}

sub _status_cli_zapi_map {
    my @status_cli_vals = (qw(Idle Transferring Restoring Checking Quiescing
        Quiesced Queued Preparing Waiting Finalizing
        Resyncing Aborting Breaking Deleting Releasing
        OutOfSync PreCutover Cutover InSync));
    my %status_cli_zapi = map {$_ => lc $_} @status_cli_vals;
    return %status_cli_zapi;
}

sub _status_zapi_cli_map {
    my $pkg = $_[0];
    return reverse ($pkg->_status_cli_zapi_map());
}

# non-iter ZAPIs require the destination-path to be provided, and the
# path should not have special characters. 
sub _non_iter_check {
    $Log->enter();
    my ($pkg_or_obj, %opts) = @_;
    my $dest_path = $opts{'destination-path'};
    if (defined $dest_path) {
        if ($pkg_or_obj->_check_relational_regex_filter(filter_value => $dest_path)) {
            # destination-path has special characters
            $Log->exit();
            NACL::Exceptions::InvalidChoice->throw('Since the '
                . 'destination-path provided has special characters '
                . "('$dest_path'), the non-iter ZAPI back-end cannot be used"
            );
        }
    } else {
        # destination-path not provided
        $Log->exit();
        NACL::Exceptions::InvalidChoice->throw("'destination-path' "
            . 'is mandatory for non-iter ZAPIs, but since it was not '
            . 'provided, the non-iter back-end cannot be used'
        );
    }
    $Log->exit();
}

# Construct the path (source/destination) by looking at the *-vserver
# and *-volume fields
sub _construct_single_path {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    state $spec = {
        params => {type => HASHREF},
        type   => {type => SCALAR},
    };
    my %opts = validate(@args, $spec);
    my $src_or_dest    = $opts{type};
    my $params         = $opts{params};

    if ($params->{"$src_or_dest-vserver"}) {
        my $path;
        my %parts = map { $_ => delete $params->{"$src_or_dest-$_"} }
            qw(cluster vserver volume cg);

        # Ignore cluster, since it isn't applicable post-RR. Start with the
        # vserver:
        $path = "$parts{vserver}:";

        # The "cg" opt takes precedence over "volume"
        if ($parts{cg}) {
            $path .= "/cg/$parts{cg}";
        } elsif ($parts{volume}) {
            $path .= $parts{volume};
        }

        $params->{"$src_or_dest-path"} = $path;
    }

    $Log->exit();
}

sub _destination_common_zapi_copy {
    my @copy = (
        qw( destination-volume destination-vserver progress-last-updated
            relationship-id source-volume source-vserver transfer-progress
            is-constituent source-volume-node )
    );
    return wantarray ? @copy : [@copy];
}

sub _destination_common_zapi_map {
    my %map = (
        'destination-path' => 'destination-location',
        'dest-volume-msid' => 'destination-volume-msid',
        'status'           => 'relationship-status',
        'type'             => 'relationship-type',
        'source-path'      => 'source-location',
    );
    return wantarray ? %map : {%map};
}

# Is defined, is not a regex, does not have wildcard characters
sub _defined_simple_value {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    my ($val) = validate_pos(@args, 1);
    if ((defined $val) && (ref $val ne 'Regexp')) {
        # The only wildcard character that can be handled is "|", so let's
        # see if any wildcard characters other than | are present
        $val =~ s/\|//g;
        if (!$pkg_or_obj->_check_relational_regex_filter(filter_value => $val)) {
            $Log->exit();
            return 1;
        }
    }
    
    $Log->exit();
    return 0;
}

1;
