# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the SystemLicense Components
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::SystemLicense;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with validate :types);

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(_map_feature_to_package _is_v2_7m_zapi);
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}

################################################################################
# Method:    _map_feature_to_package
# Objective: This subroutine is to map the attribute 'feature' to 'package' for
#            Licensing V2.
# Inputs:
#         1. hashref_or_arrayref
#            This is the hash reference or array reference within which we want
#            to map the option 'feature' to 'package'.
#
# Returns:   Nothing. The input hashref or arrayref has the attribute 'feature'
#            translated to 'package'.
###################################################################################

sub _map_feature_to_package {
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            hashref_or_arrayref => { type => HASHREF | ARRAYREF },
        },
    );

    my $hashref_or_arrayref = $opts{hashref_or_arrayref};
    if ( ref $hashref_or_arrayref eq 'HASH'
        && defined $hashref_or_arrayref->{feature} ) {
        my $feature = delete $hashref_or_arrayref->{feature};
        $hashref_or_arrayref->{package} = $feature;
    }
    if ( ref $hashref_or_arrayref eq 'ARRAY' ) {
        my $string = join ',', @$hashref_or_arrayref;
        if ( $string =~ s/feature/package/ ) {
            my @array = split( ',', $string );
            @$hashref_or_arrayref = @array;
        }
    }
    $Log->exit() if $may_exit;
} ## end sub _map_feature_to_package

################################################################################
# Method:    _is_v2_7m_zapi
# Objective: Check whether the release supports Licensing version 2 for 7Mode ZAPI
# Inputs:    command_interface
#
# Returns:   Nothing. NACL::Exceptions::InvalidChoice exception will be thrown
#            if release doesn't support Licensing version 2 for 7Mode ZAPI
################################################################################

sub _is_v2_7m_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            command_interface => { isa => 'NACL::C::CommandInterface::ONTAP' },
        },
        allow_extra => 1
    );
    my $ci = $opts{command_interface};
    my $is_v2_7m_zapi
        = $ci->is_feature_supported( feature => 'license_v2_zapi_7m' );
    if ( !$is_v2_7m_zapi ) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw(
            "Licensing V2 7Mode ZAPI supported is not present");
    }
    $Log->exit() if $may_exit;
} ## end sub _is_v2_7m_zapi

1;
