# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the Volume Component and ComponentState
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::Volume;

use strict;
use warnings;

use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(
        _snap_autodelete_fields
        _vol_options_map
        _vol_options_copy
        _option_values_to_translate
        _options_to_flip
        _flip_option_values
        _vol_autosize_map
        _common_zapi_map
    );
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

# These are the snap autodelete related fields
sub _snap_autodelete_fields {
    return [
        qw(
            snap-autodelete-commitment snap-autodelete-trigger
            snap-autodelete-target-free-space
            snap-autodelete-delete-order
            snap-autodelete-defer-delete
            snap-autodelete-destroy-list
            snap-autodelete-enabled)
    ];
}

# This is the map of the volume options from the CMode name to the
# respective 7Mode name
sub _vol_options_map {
    return {
        'min-readahead'             => 'minra',
        'maxdir-size'               => 'maxdirsize',
        'sched-snap-name'           => 'schedsnapname',
        'filesys-size-fixed'        => 'fs_size_fixed',
        'compression-enabled'       => 'compression',
        'space-guarantee'           => 'guarantee',
        'svo-checksum-enabled'      => 'svo_checksum',
        'svo-rman-allowed'          => 'svo_allow_rman',
        'extent-enabled'            => 'extent',
        'space-mgmt-try-first'      => 'try_first',
        'disk-root'                 => 'disk_root',
        'svo-enabled'               => 'svo_enable',
        'ignore-inconsistent'       => 'ignore_inconsistent',
        'create-ucode'              => 'create_ucode',
        'convert-ucode'             => 'convert_ucode',
        'fractional-reserve'        => 'fractional_reserve',
        'read-realloc'              => 'read_realloc',
        'snapshot-clone-dependency' => 'snapshot_clone_dependency',
        'svo-reject-errors'         => 'svo_reject_errors',
        'atime-update'              => 'no_atime_update',
        'i2p-enabled'               => 'no_i2p',
        'auto-snapshots-enabled'    => 'nosnap',
        'snapdir-access'            => 'nosnapdir',
        'is-mroot'                  => 'root',
        'snap-mirror-enabled'       => 'snapmirrored',
        'snaplock-default-period'   => 'snaplock_default_period',
        'snaplock-minimum-period'   => 'snaplock_minimum_period',
        'snaplock-maximum-period'   => 'snaplock_maximum_period',
        'snaplock-autocommit-period' => 'snaplock_autocommit_period'
    };
}

# This is the list of options that shows up in the 7Mode command "vol options"
# (and the equivalent ZAPI "volume-options-list-info") that has no CMode
# equivalent and hence can be copied untranslated
sub _vol_options_copy {
    return [
        qw(nvfail snaplock_default_period no_delete_log
            dlag_hole_reserve nbu_archival_snap
            snaplock_maximum_period snaplock_minimum_period
            snaplock_autocommit_period raid_type resyncsnaptime
            acregmax acdirmax acsymmax actimeo acdisconnected
            disconnected_mode flexcache_autogrow flexcache_min_reserve)
    ];
}

=head2 _option_values_to_translate

    use NACL::C::_Mixins::Volume qw(:all);
    
    my $arrayref = _option_values_to_translate(mode_of_options => $mode);

The CMode options values are true/false, in 7Mode it's on/off.
This function flips the value provided (i.e. from true -> false;
false -> true; on -> off; off -> on). This function returns the names
of the options to be translated.

When the C<mode_of_options> argument is provided as "SEVEN", it returns
the names of the 7Mode options which need to be translated (i.e. on/off ->
true/false). When the C<mode_of_options> argument is provided as "SEVEN",
it returns the names of the 7Mode options which need to be flipped (i.e.
true/false -> on/off).

=over

=item Options

=over

=item C<< mode_of_options => 'SEVEN' | 'CLUSTER' >>

(Required, valid values are "SEVEN" or "CLUSTER")

=back

=back

=cut

sub _option_values_to_translate {
    $Log->enter() if $may_enter;
    my %opts = validate( @_, { mode_of_options => { type => SCALAR } } );
    my $mode = $opts{mode_of_options};
    if ( ( $mode ne 'SEVEN' ) && ( $mode ne 'CLUSTER' ) ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw( 'Internal error: Mode should be '
                . "either of 'SEVEN' or 'CLUSTER' but it was provided as "
                . "'$mode'" );
    }
    my $sevenmode_values_to_translate = [
        qw(minra ignore_inconsistent
            create_ucode convert_ucode fs_size_fixed compression svo_enable
            svo_checksum svo_allow_rman svo_reject_errors extent read_realloc
            snapshot_clone_dependency snapmirrored nbu_archival_snap
            flexcache_autogrow)
    ];

    if ( $mode eq 'SEVEN' ) {
        $Log->exit() if $may_exit;
        return $sevenmode_values_to_translate;
    } else {
        my @cmode_values_to_translate;
        my %vol_options = reverse %{ _vol_options_map() };
        foreach my $sevenmode_value (@$sevenmode_values_to_translate) {
            my $cmode_value = $vol_options{$sevenmode_value}
                || $sevenmode_value;
            push @cmode_values_to_translate, $cmode_value;
        }
        $Log->exit() if $may_exit;
        return [@cmode_values_to_translate];
    }
}

=head2 _options_to_flip

    use NACL::C::_Mixins::Volume qw(:all);
    
    my $array_ref = _options_to_flip(mode_of_options => $mode);
    
For certain volume options, the values in CMode and 7Mode are negations
of each other. For example, if the CMode "atime-update" field's
value is "true", then the equivalent 7Mode's "no_atime_update" field's
value should be "off".

When the C<mode_of_options> argument is provided as "SEVEN", it returns
the names of the 7Mode options which need to be flipped (i.e. on/off ->
false/true). When the C<mode_of_options> argument is provided as "SEVEN",
it returns the names of the 7Mode options which need to be flipped (i.e.
true/false -> off/on).

=over

=item Options

=over

=item C<< mode_of_options => 'SEVEN' | 'CLUSTER' >>

(Required, valid values are "SEVEN" or "CLUSTER")

=back

=back

This would typically be used in conjunction with L<_flip_option_values|lib-NACL-C-_Mixins-Volume-pm/_flip_option_values>
and
L<NACL::C::_Mixins::AggregateVolume::_translate_options_values|lib-NACL-C-_Mixins-AggregateVolume-pm/_translate_options_values>.

    # Now we know which options need to be flipped
    my $options_to_flip = _options_to_flip(mode_of_options => $mode);
    
    _flip_option_values(
        hashref         => \%opts,
        mode_of_values  => $mode_of_values,
        options_to_flip => $options_to_flip
    );
    
    _translate_options_values(
        hashref             => \%opts,
        direction           => $direction,
        values_to_translate => $options_to_flip
    );
    

=cut

sub _options_to_flip {
    $Log->enter() if $may_enter;
    my (%opts) = validate( @_, { mode_of_options => { type => SCALAR } } );
    my $mode = $opts{mode_of_options};
    if ( ( $mode ne 'SEVEN' ) && ( $mode ne 'CLUSTER' ) ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
                  "Internal error: 'mode_of_options' "
                . " should be either of 'SEVEN' or 'CLUSTER' but it was "
                . "provided as '$mode'" );
    } ## end if ( ( $mode ne 'SEVEN'...
    my $sevenmode_values_to_flip
        = [qw(no_atime_update no_i2p nosnap nosnapdir)];

    if ( $mode eq 'SEVEN' ) {
        $Log->exit() if $may_exit;
        return $sevenmode_values_to_flip;
    } else {
        my @cmode_values_to_flip;
        my %vol_options = reverse %{ _vol_options_map() };
        foreach my $sevenmode_value (@$sevenmode_values_to_flip) {
            my $cmode_value = $vol_options{$sevenmode_value}
                || $sevenmode_value;
            push @cmode_values_to_flip, $cmode_value;
        }
        $Log->exit() if $may_exit;
        return [@cmode_values_to_flip];
    } ## end else [ if ( $mode eq 'SEVEN' )
} ## end sub _options_to_flip

=head2 _flip_option_values

As mentioned above, certain CMode and 7Mode options are negations
of each other. This function flips the value of the options
(i.e. true -> false; false -> true; on -> off; off -> on)

Note that the function to translate from true -> on; false -> off; (or
vice-versa) is
L<NACL::C::_Mixins::AggregateVolume::_translate_options_values|lib-NACL-C-_Mixins-AggregateVolume-pm/_translate_options_values>.

=over

=item Options

=over

=item C<< hashref => $hashref >>

(Required) The hash-reference of options within which we want to flip
the values.

=item C<< mode_of_values => 'SEVEN' | 'CLUSTER' >>

(Required) The mode in which the options are in. If the values in the
hash are true/false, then this should be provided as "CLUSTER". If the
value are on/off, then this should be provided as "SEVEN".

=item C<< values_to_flip => $array_ref_of_options_to_flip >>

(Required) This is an array-reference containing the names of the options
whose values are to be flipped. The L<_options_to_flip|lib-NACL-C-_Mixins-Volume-pm/_options_to_flip> function can be used
to get this list.

=back

=back

=cut

sub _flip_option_values {
    $Log->enter() if $may_enter;
    my %opts = validate(
        @_,
        {   hashref        => { type => HASHREF },
            mode_of_values => { type => SCALAR },
            values_to_flip => { type => ARRAYREF },
        }
    );

    my $hashref        = $opts{hashref};
    my $mode_of_values = $opts{mode_of_values};
    my @values_to_flip = @{ $opts{values_to_flip} };

    my %flip_hash;
    if ( $mode_of_values eq 'SEVEN' ) {
        %flip_hash = ( 'on' => 'off', 'off' => 'on' );
    } elsif ( $mode_of_values eq 'CLUSTER' ) {
        %flip_hash = ( 'true' => 'false', 'false' => 'true' );
    } else {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw( 'Internal error: The value of the '
                . "'mode_of_values' parameter should be either of 'SEVEN' "
                . "or 'CLUSTER' but the value provided was "
                . "'$mode_of_values'" );
    }

    foreach my $val_to_flip (@values_to_flip) {
        if ( defined $hashref->{$val_to_flip} ) {
            if ( exists $flip_hash{ $hashref->{$val_to_flip} } ) {
                $hashref->{$val_to_flip}
                    = $flip_hash{ $hashref->{$val_to_flip} };
            }
        }
    }
    $Log->exit() if $may_exit;
}

# CMode -> 7Mode mapping of autosize related options
sub _vol_autosize_map {
    return {
        'max-autosize'                      => 'maximum-size',
        'autosize-increment'                => 'increment-size',
        'min-autosize'                      => 'minimum-size',
        'autosize-grow-threshold-percent'   => 'grow-threshold-percent',
        'autosize-shrink-threshold-percent' => 'shrink-threshold-percent',
    };
}

sub _common_zapi_map {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            command_interface => $pkg_or_obj->_ontap_ci_validate_spec(),
        }
    );
    
    my $map = {

        # Ordered by ZAPI elements (<volume-get-iter/> in
        # lib/NACL/APISet/Node/ZAPI/CMode/main/volume.xml)

        'upgrade-64bit-estimate-age' => [
            'volume-64bit-upgrade-attributes',
            'volume-64bit-upgrade-check-attributes',
            'age'
        ],
        'upgrade-64bit-available-space' => [
            'volume-64bit-upgrade-attributes',
            'volume-64bit-upgrade-check-attributes',
            'available-space'
        ],
        'upgrade-64bit-capacity' => [
            'volume-64bit-upgrade-attributes',
            'volume-64bit-upgrade-check-attributes',
            'capacity'
        ],
        'upgrade-64bit-check-last-errno' => [
            'volume-64bit-upgrade-attributes',
            'volume-64bit-upgrade-check-attributes',
            'check-last-errno'
        ],
        'upgrade-64bit-grow-space' => [
            'volume-64bit-upgrade-attributes',
            'volume-64bit-upgrade-check-attributes',
            'grow-space'
        ],
        'upgrade-64bit-percent-used-space' => [
            'volume-64bit-upgrade-attributes',
            'volume-64bit-upgrade-check-attributes',
            'used-space'
        ],
        'upgrade-64bit-start-last-errno' => [
            'volume-64bit-upgrade-attributes',
            # XXX this is missing being declared as a typedef in the cdef
            #'volume-64bit-upgrade-start-attributes',
            'start-last-errno'
        ],
        'antivirus-on-access-policy' => [
            'volume-antivirus-attributes',
            make_zapi_skip('on-access-policy')
        ],
        'autosize-grow-threshold-percent' => [
            'volume-autosize-attributes',
            'grow-threshold-percent'
        ],
        'autosize-increment' => [
            'volume-autosize-attributes',
            'increment-size'
        ],
        'autosize'     => [
            'volume-autosize-attributes',
            'is-enabled'
        ],
        'max-autosize' => [
            'volume-autosize-attributes',
            'maximum-size'
        ],
        'min-autosize' => [
            'volume-autosize-attributes',
            'minimum-size'
        ],
        'autosize-mode' => [
            'volume-autosize-attributes',
            'mode'
        ],
        'autosize-reset' => [
            'volume-autosize-attributes',
            'reset'
        ],
        'autosize-shrink-threshold-percent' => [
            'volume-autosize-attributes',
            'shrink-threshold-percent'
        ],
        'clone-child-count' => [
            'volume-clone-attributes',
            'clone-child-count'
        ],
        'clone-parent-dsid' => [
            'volume-clone-attributes',
            'volume-clone-parent-attributes',
            'dsid'
        ],
        'clone-parent-msid' => [
            'volume-clone-attributes',
            'volume-clone-parent-attributes',
            'msid'
        ],
        'clone-parent-name' => [
            'volume-clone-attributes',
            'volume-clone-parent-attributes',
            'name'
        ],
        'clone-parent-bsnap-id' => [
            'volume-clone-attributes',
            'volume-clone-parent-attributes',
            'snapshot-id'
        ],
        'clone-parent-bsnap-name' => [
            'volume-clone-attributes',
            'volume-clone-parent-attributes',
            'snapshot-name'
        ],
        'clone-parent-uuid' => [
            'volume-clone-attributes',
            'volume-clone-parent-attributes',
            'uuid'
        ],
        'index-dir-enabled' => [
            'volume-directory-attributes',
            'index-dir-enabled'
        ],
        'maxdir-size' => [
            'volume-directory-attributes',
            'max-dir-size'
        ],
        'root-dir-gen' => [
            'volume-directory-attributes',
            'root-dir-gen'
        ],
        # I don't know why volume-export-attributes isn't listed as a
        # typedef in the cdef, but let's assume it actually is.
        'policy'       => [
            'volume-export-attributes',
            make_zapi_skip('policy')
        ],
        'flexcache-cache-policy' => [
            'volume-flexcache-attributes',
            'cache-policy'
        ],
        'flexcache-fill-policy' => [
            'volume-flexcache-attributes',
            'fill-policy'
        ],
        'flexcache-min-reserve' => [
            'volume-flexcache-attributes',
            'min-reserve'
        ],
        'flexcache-origin-volume' => [
            'volume-flexcache-attributes',
            'origin'
        ],
        'hybrid-cache-eligibility' => [
            'volume-hybrid-cache-attributes',
            'eligibility'
        ],
        'hybrid-cache-write-caching-ineligibility-reason' => [
            'volume-hybrid-cache-attributes',
            'write-cache-ineligibility-reason'
        ],
        'comment' => [
            'volume-id-attributes',
            'comment'
        ],
        'aggregate' => [
            'volume-id-attributes',
            'containing-aggregate-name'
        ],
        'containing-aggregate-uuid' => [
            'volume-id-attributes',
            'containing-aggregate-uuid'
        ],
        'create-time' => [
            'volume-id-attributes',
            'creation-time'
        ],
        'dsid' => [
            'volume-id-attributes',
            'dsid'
        ],
        'fsid' => [
            'volume-id-attributes',
            'fsid'
        ],
        'instance-uuid' => [
            'volume-id-attributes',
            'instance-uuid'
        ],
        'is-reserve-constituent' => [
            'volume-id-attributes',
            'is-reserve-constituent'
        ],
        'junction-parent' => [
            'volume-id-attributes',
            'junction-parent-name'
        ],
        'junction-path' => [
            'volume-id-attributes',
            'junction-path'
        ],
        'msid' => [
            'volume-id-attributes',
            'msid'
        ],
        'volume' => [
            'volume-id-attributes',
            'name'
        ],
        'name-ordinal' => [
            'volume-id-attributes',
            'name-ordinal'
        ],
        'vserver' => [
            'volume-id-attributes',
            'owning-vserver-name'
        ],
        'vserver-uuid' => [
            'volume-id-attributes',
            'owning-vserver-uuid'
        ],
        'provenance-uuid' => [
            'volume-id-attributes',
            'provenance-uuid'
        ],
        'volume-style' => [
            'volume-id-attributes',
            'style'
        ],
        'volume-style-extended' => [
            'volume-id-attributes',
            'style-extended'
        ],
        'type' => [
            'volume-id-attributes',
            'type'
        ],
        'uuid' => [
            'volume-id-attributes',
            'uuid'
        ],
        'aggr-list' => [
            'volume-id-attributes',
            make_zapi_array('aggr-list'),
	    make_zapi_skip('aggr-name')
        ],
	'nodes' => [
	    'volume-id-attributes',
            make_zapi_array('nodes'),
            make_zapi_skip('node-name')
	],
        'flexgroup-msid' => [
            'volume-id-attributes',
            'flexgroup-msid'
        ],
        'flexgroup-index' => [
            'volume-id-attributes',
            'flexgroup-index'
        ],
        'flexgroup-uuid' => [
            'volume-id-attributes',
            'flexgroup-uuid'
        ],
        'is-managed-by-service' => [
             'volume-infinitevol-attributes',
             'is-managed-by-service'
        ],
        'enable-snapdiff' => [
             'volume-infinitevol-attributes',
             'enable-snapdiff'
        ],
        'max-data-constituent-size' => [
             'volume-infinitevol-attributes',
             'max-data-constituent-size'
        ],
        'max-namespace-constituent-size' => [
             'volume-infinitevol-attributes',
             'max-namespace-constituent-size'
        ],
        'storage-service' => [
             'volume-infinitevol-attributes',
             'storage-service'
        ],
        'constituent-role' => [
            'volume-infinitevol-attributes',
            'constituent-role'
        ],
        'block-type' => [
            'volume-inode-attributes',
            'block-type'
        ],
        'files-private-used' => [
            'volume-inode-attributes',
            'files-private-used'
        ],
        'files' => [
            'volume-inode-attributes',
            'files-total'
        ],
        'files-used' => [
            'volume-inode-attributes',
            'files-used'
        ],
        'inodefile-private-capacity' => [
            'volume-inode-attributes',
            'inodefile-private-capacity'
        ],
        'inodefile-public-capacity' => [
            'volume-inode-attributes',
            'inodefile-public-capacity'
        ],
        'convert-ucode' => [
            'volume-language-attributes',
            'is-convert-ucode-enabled'
        ],
        'create-ucode' => [
            'volume-language-attributes',
            'is-create-ucode-enabled'
        ],
        'language' => [
            'volume-language-attributes',
            'language-code'
        ],
        'language-str' => [
            'volume-language-attributes',
            'language'
        ],
        'nfs-character-set' => [
            'volume-language-attributes',
            'nfs-character-set'
        ],
        'oem-character-set' => [
            'volume-language-attributes',
            'oem-character-set'
        ],
        'is-data-protection-mirror' => [
            'volume-mirror-attributes',
            'is-data-protection-mirror'
        ],
        'is-load-sharing-mirror' => [
            'volume-mirror-attributes',
            'is-load-sharing-mirror'
        ],
        'is-move-mirror' => [
            'volume-mirror-attributes',
            'is-move-mirror'
        ],
        'is-replica-volume' => [
            'volume-mirror-attributes',
            'is-replica-volume'
        ],
        'mirror-transfer-in-progress' => [
            'volume-mirror-attributes',
            'mirror-transfer-in-progress'
        ],
        'redirect-snapshot-id' => [
            'volume-mirror-attributes',
            'redirect-snapshot-id'
        ],
        'enable-object-store' => [
            'volume-object-store-attributes',
            'enable-object-store',
        ],
        'object-write-sync-period' => [
            'volume-object-store-attributes',
            'object-write-sync-period',
        ],
        'ols-aggr-list' => [
            'volume-object-store-attributes',
            make_zapi_array('ols-aggr-list'),
            make_zapi_skip('aggr-name'),
        ],
        'ols-constituent-count' => [
            'volume-object-store-attributes',
            'ols-constituent-count',
        ],
        'ols-constituent-size' => [
            'volume-object-store-attributes',
            'ols-constituent-size',
        ],
        'extent-enabled' => [
            'volume-performance-attributes',
            'extent-enabled'
        ],
        'fc-delegs-enabled' => [
            'volume-performance-attributes',
            'fc-delegs-enabled'
        ],
        'atime-update' => [
            'volume-performance-attributes',
            'is-atime-update-enabled'
        ],
        'max-write-alloc-blocks' => [
            'volume-performance-attributes',
            'max-write-alloc-blocks'
        ],
        'min-readahead' => [
            'volume-performance-attributes',
            'minimal-read-ahead'
        ],
        'read-realloc' => [
            'volume-performance-attributes',
            'read-realloc'
        ],
	'qos-adaptive-policy-group' => [
            'volume-qos-attributes',
            'adaptive-policy-group-name'
        ],
        'external-cache' => [
            'volume-qos-attributes',
            'external-cache-policy'
        ],
        'qos-policy-group' => [
            'volume-qos-attributes',
            'policy-group-name'
        ],
        'security-style' => [
            'volume-security-attributes',
            'style'
        ],
        'group'          => [
            'volume-security-attributes',
            'volume-security-unix-attributes',
            'group-id'
        ],
        'unix-permissions' => [
            'volume-security-attributes',
            'volume-security-unix-attributes',
            'permissions'
        ],
        'user' => [
            'volume-security-attributes',
            'volume-security-unix-attributes',
            'user-id'
        ],
        'compression-space-saved' => [
            'volume-sis-attributes',
            'compression-space-saved'
        ],
        'dedupe-space-saved' => [
            'volume-sis-attributes',
            'deduplication-space-saved'
        ],
        'dedupe-space-shared' => [
            'volume-sis-attributes',
            'deduplication-space-shared'
        ],
        'efficiency-policy'  => [
            'volume-sis-attributes',
            'efficiency-policy'
        ],
        'is-sis-logging-enabled' => [
            'volume-sis-attributes',
            'is-sis-logging-enabled'
        ],
        'is-sis-volume' => [
            'volume-sis-attributes',
            'is-sis-volume'
        ],
        'compression-space-saved-percent' => [
            'volume-sis-attributes',
            'percentage-compression-space-saved'
        ],
        'dedupe-space-saved-percent' => [
            'volume-sis-attributes',
            'percentage-deduplication-space-saved'
        ],
        'sis-space-saved-percent' => [
            'volume-sis-attributes',
            'percentage-total-space-saved'
        ],
        'sis-space-saved' => [
            'volume-sis-attributes',
            'total-space-saved'
        ],
        'auto-snapshots-enabled' => [
            'volume-snapshot-attributes',
            'auto-snapshots-enabled'
        ],
        'snapdir-access' => [
            'volume-snapshot-attributes',
            'snapdir-access-enabled'
        ],
        'snapshot-clone-dependency' => [
            'volume-snapshot-attributes',
            'snapshot-clone-dependency-enabled'
        ],
        'snapshot-policy' => [
            'volume-snapshot-attributes',
            'snapshot-policy'
        ],
        'snapshot-count' => [
            'volume-snapshot-attributes',
            'snapshot-count'
        ],
        'snap-autodelete-commitment' => [
            'volume-snapshot-autodelete-attributes',
            'commitment'
        ],
        'snap-autodelete-defer-delete' => [
            'volume-snapshot-autodelete-attributes',
            'defer-delete'
        ],
        'snap-autodelete-delete-order' => [
            'volume-snapshot-autodelete-attributes',
            'delete-order'
        ],
        'snap-autodelete-destroy-list' => [
            'volume-snapshot-autodelete-attributes',
            'destroy-list'
        ],
        'snap-autodelete-enabled' => [
            'volume-snapshot-autodelete-attributes',
            'is-autodelete-enabled'
        ],
        'snap-autodelete-prefix' => [
            'volume-snapshot-autodelete-attributes',
            'prefix'
        ],
        'snap-autodelete-target-free-space' => [
            'volume-snapshot-autodelete-attributes',
            'target-free-space'
        ],
        'snap-autodelete-trigger' => [
            'volume-snapshot-autodelete-attributes',
            'trigger'
        ],
        'filesystem-size' => [
            'volume-space-attributes',
            'filesystem-size'
        ],
        'filesys-size-fixed' => [
            'volume-space-attributes',
            'is-filesys-size-fixed'
        ],
        'space-guarantee-enabled' => [
            'volume-space-attributes',
            'is-space-guarantee-enabled'
        ],
	'max-constituent-size' => [
            'volume-space-attributes',
            'max-constituent-size'
        ],
        'overwrite-reserve' => [
            'volume-space-attributes',
            'overwrite-reserve'
        ],
        'overwrite-reserve-required' => [
            'volume-space-attributes',
            'overwrite-reserve-required'
        ],
        'overwrite-reserve-used' => [
            'volume-space-attributes',
            'overwrite-reserve-used'
        ],
        'overwrite-reserve-used-actual' => [
            'volume-space-attributes',
            'overwrite-reserve-used-actual'
        ],
        'fractional-reserve' => [
            'volume-space-attributes',
            'percentage-fractional-reserve'
        ],
        'percent-used' => [
            'volume-space-attributes',
            'percentage-size-used'
        ],
        'snapshot-space-used' => [
            'volume-space-attributes',
            'percentage-snapshot-reserve-used'
        ],
        'percent-snapshot-space' => [
            'volume-space-attributes',
            'percentage-snapshot-reserve'
        ],
        'size' => [
            'volume-space-attributes',
            'size'
        ],
        'available' => [
            'volume-space-attributes',
            'size-available'
        ],
        'size-available-for-snapshots' => [
            'volume-space-attributes',
            'size-available-for-snapshots'
        ],
        'total' => [
            'volume-space-attributes',
            'size-total'
        ],
        'used' => [
            'volume-space-attributes',
            'size-used'
        ],
        'size-used-by-snapshots' => [
            'volume-space-attributes',
            'size-used-by-snapshots'
        ],
        'snapshot-reserve-size' => [
            'volume-space-attributes',
            'snapshot-reserve-size'
        ],
        'space-guarantee' => [
            'volume-space-attributes',
            'space-guarantee'
        ],
        'space-mgmt-try-first' => [
            'volume-space-attributes',
            'space-mgmt-option-try-first'
        ],
        'space-nearly-full-threshold-percent' => [
            'volume-space-attributes',
            'space-nearly-full-threshold-percent'
        ],
        'space-full-threshold-percent' => [
            'volume-space-attributes',
            'space-full-threshold-percent'
        ],
        'physical-used' => [
            'volume-space-attributes',
            'physical-used'
        ],

        'dedupe-metafiles' => [
            'volume-space-show-attributes',
            'dedupe-metafiles'
        ],
        'dedupe-metafiles-fp' => [
            'volume-space-show-attributes',
            'dedupe-metafiles-fp'
        ],
        'dedupe-metafiles-fp-percent' => [
            'volume-space-show-attributes',
            'dedupe-metafiles-fp-percent'
        ],
        'dedupe-metafiles-tmp' => [
            'volume-space-show-attributes',
            'dedupe-metafiles-tmp'
        ],
        'dedupe-metafiles-tmp-fp' => [
            'volume-space-show-attributes',
            'dedupe-metafiles-tmp-fp'
        ],
        'dedupe-metafiles-tmp-fp-percent' => [
            'volume-space-show-attributes',
            'dedupe-metafiles-tmp-fp-percent'
        ],
        'dedupe-metafiles-tmp-percent' => [
            'volume-space-show-attributes',
            'dedupe-metafiles-tmp-percent'
        ],
        'delayed-free-fp' => [
            'volume-space-show-attributes',
            'delayed-free-fp'
        ],
        'delayed-free-fp-percent' => [
            'volume-space-show-attributes',
            'delayed-free-fp-percent'
        ],
        'flexvol-metadata-fp' => [
            'volume-space-show-attributes',
            'flexvol-metadata-fp'
        ],
        'flexvol-metadata-fp-percent' => [
            'volume-space-show-attributes',
            'flexvol-metadata-fp-percent'
        ],
        'inodes' => [
            'volume-space-show-attributes',
            'inodes'
        ],
        'inodes-percent' => [
            'volume-space-show-attributes',
            'inodes-percent'
        ],
        'quota-metafiles' => [
            'volume-space-show-attributes',
            'quota-metafiles'
        ],
        'quota-metafiles-percent' => [
            'volume-space-show-attributes',
            'quota-metafiles-percent'
        ],
        'snapmirror-destination-fp' => [
            'volume-space-show-attributes',
            'snapmirror-destination-fp'
        ],
        'snapmirror-destination-fp-percent' => [
            'volume-space-show-attributes',
            'snapmirror-destination-fp-percent'
        ],
        'snapshot-spill' => [
            'volume-space-show-attributes',
            'snapshot-spill'
        ],
        'snapshot-spill-percent' => [
            'volume-space-show-attributes',
            'snapshot-spill-percent'
        ],
        'tape-backup-metafiles-fp' => [
            'volume-space-show-attributes',
            'tape-backup-metafiles-fp'
        ],
        'tape-backup-metafiles-fp-percent' => [
            'volume-space-show-attributes',
            'tape-backup-metafiles-fp-percent'
        ],
        'total-fp' => [
            'volume-space-show-attributes',
            'total-fp'
        ],
        'total-fp-percent' => [
            'volume-space-show-attributes',
            'total-fp-percent'
        ],
        'user-data' => [
            'volume-space-show-attributes',
            'user-data'
        ],
        'user-data-percent' => [
            'volume-space-show-attributes',
            'user-data-percent'
        ],
        'volume-data-fp' => [
            'volume-space-show-attributes',
            'volume-data-fp'
        ],
        'volume-data-fp-percent' => [
            'volume-space-show-attributes',
            'volume-data-fp-percent'
        ],
        'volume-filesystem-metadata' => [
            'volume-space-show-attributes',
            'volume-filesystem-metadata'
        ],
        'volume-filesystem-metadata-percent' => [
            'volume-space-show-attributes',
            'volume-filesystem-metadata-percent'
        ],
        'volume-guarantee-fp' => [
            'volume-space-show-attributes',
            'volume-guarantee-fp'
        ],
        'volume-guarantee-fp-percent' => [
            'volume-space-show-attributes',
            'volume-guarantee-fp-percent'
        ],
        'become-node-root-after-reboot' => [
            'volume-state-attributes',
            'become-node-root-after-reboot'
        ],
        'ignore-inconsistent' => [
            'volume-state-attributes',
            'ignore-inconsistent'
        ],
        'in-nvfailed-state' => [
            'volume-state-attributes',
            'in-nvfailed-state'
        ],
        'is-cluster-volume' => [
            'volume-state-attributes',
            'is-cluster-volume'
        ],
        'is-constituent' => [
            'volume-state-attributes',
            'is-constituent'
        ],
	'is-flexgroup' => [
            'volume-state-attributes',
            'is-flexgroup'
        ],
	'is-flexgroup-qtree-enabled' => [
            'volume-state-attributes',
            'is-flexgroup-qtree-enabled'
        ],
        'is-inconsistent' => [
            'volume-state-attributes',
            'is-inconsistent'
        ],
        'is-invalid' => [
            'volume-state-attributes',
            'is-invalid'
        ],
        'junction-active' => [
            'volume-state-attributes',
            'is-junction-active'
        ],
        'is-moving' => [
            'volume-state-attributes',
            'is-moving'
        ],
        'is-mroot' => [
            'volume-state-attributes',
            'is-node-root'
        ],
        'nvfail' => [
            'volume-state-attributes',
            'is-nvfail-enabled'
        ],
        'is-quiesced-in-memory' => [
            'volume-state-attributes',
            'is-quiesced-in-memory'
        ],
        'is-quiesced-on-disk' => [
            'volume-state-attributes',
            'is-quiesced-on-disk'
        ],
        'is-unrecoverable' => [
            'volume-state-attributes',
            'is-unrecoverable'
        ],
        'vsroot' => [
            'volume-state-attributes',
            'is-vserver-root'
        ],
        'state' => [
            'volume-state-attributes',
            'state'
        ],
        'status' => [
            'volume-state-attributes',
            make_zapi_array('status'),
            make_zapi_skip('string')
        ],
        'concurrency' => [
            'volume-striping-attributes',
            'concurrency'
        ],
        'stripe-count' => [
            'volume-striping-attributes',
            'constituent-volume-count'
        ],
        'optimize' => [
            'volume-striping-attributes',
            'optimize'
        ],
        'stripe-algorithm' => [
            'volume-striping-attributes',
            'stripe-algorithm'
        ],
        'stripe-width' => [
            'volume-striping-attributes',
            'stripe-width'
        ],
        'is-copied-for-transition' => [
            'volume-transition-attributes',
            'is-copied-for-transition'
        ],
        'is-transitioned' => [
            'volume-transition-attributes',
            'is-transitioned'
        ],
        'transition-behavior' => [
            'volume-transition-attributes',
            'transition-behavior'
        ],
        'vm-align-sector' => [
            'volume-vm-align-attributes',
            'vm-align-sector'
        ],
        'vm-align-suffix' => [
            'volume-vm-align-attributes',
            'vm-align-suffix'
        ],
        'i2p-enabled'     => [
            'volume-directory-attributes',
            'i2p-enabled',
        ],
        'maxdir-size'     => [
            'volume-directory-attributes',
            'max-dir-size',
        ],
        'root-dir-gen'    => [
            'volume-directory-attributes',
            'root-dir-gen',
        ],
        'concurrency'     => [
            'volume-striping-attributes',
            'concurrency',
        ],
        'optimize'        => [
            'volume-striping-attributes',
            'optimize',
        ],
        'stripe-algorithm' => [
            'volume-striping-attributes',
            'stripe-algorithm',
        ],
        'stripe-width'     => [
            'volume-striping-attributes',
            'stripe-width',
        ],
        'stripe-count'     => [
            'volume-striping-attributes',
            'constituent-volume-count',
        ],
        'caching-policy' => [
            'volume-hybrid-cache-attributes',
            'caching-policy'
        ],
        'node'           => [
            'volume-id-attributes',
            'node'
        ],
        'snaplock-type' => ['volume-snaplock-attributes', 'snaplock-type'],
        'vserver-dr-protection' => [
            'volume-attributes',
            make_zapi_array('volume-vserver-dr-protection-attributes'),
            make_zapi_skip('vserver-dr-protection'),
        ],
        # 'volume-comp-aggr-attributes' isn't listed as a
        # typedef in the cdef, so handling it here
        'tiering-policy'       => [
           'volume-comp-aggr-attributes',
           make_zapi_skip('tiering-policy')
        ],
    };
    $Log->exit() if $may_exit;
    return $map;
}

1;
