# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the VolumeFile and VolumeDirectory components
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::VolumeFileDirectory;

use strict;
use warnings;
use feature 'state';
BEGIN {
    use Exporter qw(import);
    my @find_state_methods = (
        qw(
            find_files
            find_directories
            find_files_and_directories
            state_files
            state_directories
            state_files_and_directories
            _file_dir_find_wrapper
            _file_dir_state_wrapper
            )
    );
    my @volume_file_directory = qw(
        _determine_directory_path
        get_local_node_name
        _get_vol_name_from_path
        get_containing_volume_name
        get_containing_volume_obj
        get_file_or_dir_name_from_path
        _detect_builder
        _nodescope_helper
        _add_name_ordinal
        _remove_name_ordinal
        dblade_path
        _call_on_apiset
        _get_node_and_name_ordinal
        _construct_file_path
        _construct_volume_path
        _extract_volume_file_from_path
        _convert_path_to_cmode_format
        set_permission
        copy_to_logdir
        get_containing_directory_path
        systemshell_path
        _check_sksim
    );

    my @file_create = qw(
        _create_helper
        _create_7mode_cli
    );

    my @file_delete = qw(_delete_front_end_helper);

    my @file_operation = qw (showfh _showfh_7mode_cli
        dumpblock_info _dumpblock_info_7mode_cli
    );

    our @EXPORT_OK = (
        @find_state_methods,
        @volume_file_directory,
        @file_create,
        @file_delete,
        @file_operation
    );
    my @all = (@find_state_methods, @volume_file_directory);

    our %EXPORT_TAGS = (
        'all'        => \@all,
        'find_state' => \@find_state_methods,
        'file_create' => \@file_create,
        'file_delete' => \@file_delete,
        'file_operation' => \@file_operation
    );
}

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Exceptions::Argument;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use Params::Validate qw(:all);
use NACL::GeneralUtils qw(random_name_generator);
use NATE::BaseException qw(:try);
use NACL::ComponentUtils
    qw(_response_exception_validate_spec _optional_scalars);
use NACL::MTask::SCP;
use NACL::C::Client::Directory;

# Given a path, determine the path of the containing directory
sub _determine_directory_path {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = $_[0];
    if (ref $pkg_or_obj) {
        my $file_path = $pkg_or_obj->path();
        $file_path =~ m!(.*/).*!;
        my $dir_path = $1;
        $dir_path =~ s/\/$//; #remove trailing '/' caught in group matching
        $Log->exit($dir_path);
        return $dir_path;
    } else {
        $Log->exit() if $may_exit;
        return;
    }
}

sub get_local_node_name {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,

        # Unknown options will simply be ignored
        allow_extra => 1
    );

    my $vol = $pkg_or_obj->_get_vol_name_from_path(path => $opts{path});

    my %common_params = ();
    $pkg_or_obj->_copy_common_component_params(
        source => \%opts,
        target => \%common_params
    );

    my $node = NACL::C::Volume->get_local_node_name(
        command_interface => $opts{command_interface},
        volume            => $vol,
        vserver           => $opts{vserver},
        %common_params
    );

    $Log->exit() if $may_exit;
    return $node;
}

# Provided a file/directory path, returns the volume name
sub _get_vol_name_from_path {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            path                  => {type => SCALAR},
            contains_name_ordinal => {type => SCALARREF, optional => 1}
        }
    );

    my $path = $opts{path};
    ${$opts{contains_name_ordinal}} = 0;
    
    my $obj = $pkg->_extract_volume_file_from_path(path => $path);
    ${$opts{contains_name_ordinal}} = $obj->contains_name_ordinal();

    $Log->exit() if $may_exit;
    return $obj->volume();
}

# Get the volume name from the file/directory object
sub get_containing_volume_name {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    my $vol_name = $self->_get_vol_name_from_path(path => $self->path());
    $Log->exit() if $may_exit;
    return $vol_name;
}

# Get the volume object from the file/directory object.
sub get_containing_volume_obj {
    $Log->enter() if $may_enter;
    my $self            = $_[0];
    my $vol_name        = $self->get_containing_volume_name();
    my %pk_hash_with_ci = $self->get_pk_hash_with_ci();

    # "path" not necessary for volume object
    delete $pk_hash_with_ci{path};

    my $vol_pkg = 'NACL::' . $self->is_c_or_t() . '::Volume';

    my $vol_obj = $vol_pkg->new(%pk_hash_with_ci, volume => $vol_name);
    $Log->exit() if $may_exit;
    return $vol_obj;
}

sub get_file_or_dir_name_from_path {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {path => {type => SCALAR}},
        allow_extra => 1
    );
    my $path = $opts{path};
    my $file = $pkg_or_obj->_extract_volume_file_from_path(path => $path)->file();
    
    $Log->exit() if $may_exit;
    return $file;
}

sub find_files {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my ($recursive, $file, @files);
    my %opts = validate_with(
        params => \@args,
        spec   => {
            recursive => {type => BOOLEAN, default => 0}},
        allow_extra => 1
    );
    $recursive = delete $opts{recursive};
    if ( $recursive ) {
        require NACL::MTask::VolumeFileIterator;
        my $iterator = NACL::MTask::VolumeFileIterator->new(
            path   => "/",
            volume => $self,
        );
        while ($iterator->has_next()) {
            $file = $iterator->next();
            push(@files, $file);
        }
    } else {
        @files = $self->_file_dir_find_wrapper(%opts, 'file-type' => 'file'); 
    }
    $Log->exit() if $may_exit;
    return wantarray ? @files : $files[0];
}

sub find_directories {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my @dirs
        = $self->_file_dir_find_wrapper(@args, 'file-type' => 'directory');
    $Log->exit() if $may_exit;
    return wantarray ? @dirs : $dirs[0];
}

sub find_files_and_directories {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my @files_and_dirs = $self->_file_dir_find_wrapper(@args);
    $Log->exit() if $may_exit;
    return wantarray ? @files_and_dirs : $files_and_dirs[0];
}

sub state_files {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my @files_cs
        = $self->_file_dir_state_wrapper(@args, 'file-type' => 'file');
    $Log->exit() if $may_exit;
    return wantarray ? @files_cs : $files_cs[0];
}

sub state_directories {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my @dirs_cs
        = $self->_file_dir_state_wrapper(@args, 'file-type' => 'directory');
    $Log->exit() if $may_exit;
    return wantarray ? @dirs_cs : $dirs_cs[0];
}

sub state_files_and_directories {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my @files_and_dirs_cs = $self->_file_dir_state_wrapper(@args);
    $Log->exit() if $may_exit;
    return wantarray ? @files_and_dirs_cs : $files_and_dirs_cs[0];
}

sub _file_dir_find_wrapper {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my @cs = $self->_file_dir_find_state_wrapper(@args,
        requested_fields => [qw(path file-type)]);
    my @objects = map { $_->get_component_instance() } @cs;
    $Log->exit() if $may_exit;
    return @objects;
}

sub _file_dir_state_wrapper {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my @cs = $self->_file_dir_find_state_wrapper(@args);
    $Log->exit() if $may_exit;
    return @cs;
}

# At some point, we should probably transition the other component-level
# exceptions to use logic like this (look at the error-number for ZPAI and the
# error text for CLI). The error-number is safe to use (unless it's in the
# "13XXX" family) and is guaranteed to remain the same, while the error
# text is not guaranteed to remain the same.
sub _detect_builder {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            exception         => {_response_exception_validate_spec()},
            zapi_error_number => {type => SCALAR},
            regexes           => {type => ARRAYREF},
        },
        allow_extra => 1
    );
    my $exception = $opts{exception};
    my $error_num = delete $opts{zapi_error_number};

    my $val;
    if ($exception->is_zapi()) {
        if ($exception->error_number() == $error_num) {
            $val = 1;
        } else {
            $val = 0;
        }
    } else {
        $val = $pkg->_detect_helper(%opts);
    }

    $Log->exit($val);
    return $val;
}

# Helper for Nodescope back-end methods. If Nodescope, then determines
# which node to run on. This uses "_get_nodescope_node_name" for this,
# this helper ensures that the appropriate options are passed to that
# method. The determined node is added to the options hash
sub _nodescope_helper {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            options         => {type => HASHREF},
            allowed_options => {type => ARRAYREF}
        }
    );
    my $options = $opts{options};
    my $ci      = $options->{command_interface};

    if ($ci->is_cmode()) {
        my %node_args;
        $pkg_or_obj->_hash_copy(
            source => $options,
            target => \%node_args,
            copy   => [
                qw(command_interface vserver path nodescope-node-name
                    determine_appropriate_nodescope_node)
            ],
        );

        # Make a package call here, because the "path" attribute in the
        # object will not have the name-ordinal while the path in the
        # arguments might have it, so _common_validate_with will complain
        # of a mismatch.
        my $pkg  = $pkg_or_obj->get_package_name();
        my $node = $pkg->_get_nodescope_node_name(%node_args);
        $options->{'nodescope-node-name'} = $node if (defined $node);

        push @{$opts{allowed_options}}, 'nodescope-node-name';
    }

    $Log->exit() if $may_exit;
}

# Determine the name-ordinal and update the path accordingly.
# For example, if name-ordinal is 2 and call is:
#    my $path = $pkg->_add_name_ordinal(path => "/vol/myvol/myfile");
# $path would be "/vol/myvol(2)/myfile"
sub _add_name_ordinal {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params      => \@args,
        allow_extra => 1
    );
    my $path = $opts{path};

    
    my $obj = $pkg_or_obj->_extract_volume_file_from_path(path => $path);
    my $vol_name = $obj->volume();
    my $contains_name_ordinal = $obj->contains_name_ordinal();
    unless ($contains_name_ordinal) {
        if (defined $opts{vserver}) {
            my %common_opts_with_ci;
            $pkg_or_obj->_copy_common_component_params_with_ci(
                source => \%opts,
                target => \%common_opts_with_ci
            );
            my $filename = $obj->file();
            require NACL::STask::Volume;
            $path = NACL::STask::Volume->construct_dblade_file_path(
                volume   => $vol_name,
                vserver  => $opts{vserver},
                filename => $filename,
                %common_opts_with_ci
            );
        } else {
            $Log->trace('Warning: vserver argument needed to construct '
                    . 'the dblade path, but was not provided');
        }
    }

    $Log->exit() if $may_exit;
    return $path;
}

# Remove name-ordinal from the path and return it. For example:
#     my $path = $pkg->_remove_name_ordinal(path => "/vol/myvol(10)/myfile");
# $path would be "/vol/myvol/myfile"
sub _remove_name_ordinal {
    my ($pkg_or_obj, %opts) = @_;
    my $path;
    if (ref $pkg_or_obj) {
        $path = $pkg_or_obj->path();
    } else {
        $path = $opts{path};
    }
    unless (defined $path) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "'path' should have been provided but was not");
    }

    my $obj = $pkg_or_obj->_extract_volume_file_from_path(path => $path);
    my $vol_name = $obj->volume();
    my $contains_name_ordinal = $obj->contains_name_ordinal();

    if ($contains_name_ordinal) {
        my $filename = $obj->orig_file();
        $path = $pkg_or_obj->_construct_file_path(
            volume => $vol_name, file => $filename);
    }

    $Log->exit() if $may_exit;
    return $path;
}

sub dblade_path {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my $dblade_path = $self->_add_name_ordinal(@args);
    $Log->exit() if $may_exit;
    return $dblade_path;
}

# Handle the name-ordinal. If set to 1:
# * Determine name-ordinal and update path for Nodescope CLI
# * Remove name-ordinal from path for CMode ZAPI
sub _call_on_apiset {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $base_class = $pkg_or_obj->base_class();
    my $base_call_on_apiset = "${base_class}::call_on_apiset";
    my $hash;
    if ($opts{apiset_backend_already_chosen}) {
        $hash = $opts{apiset_backend_already_chosen};
    } else {
        $hash = $pkg_or_obj->$base_call_on_apiset(%opts,
            do_not_call_backend => 1);
    }

    if ($opts{do_not_call_backend}) {
        $Log->exit() if $may_exit;
        return $hash;
    }

    # Do not pass apiset_chosen on to _get_node_and_name_ordinal
    # It will end up overwriting the apiset_chosen value
    delete $opts{apiset_chosen};
    $opts{'path'} = $opts{'path'}->path() if ref $opts{'path'};

    if ( $opts{command_interface}->is_cmode()) {
        my $apiset = $hash->{apiset};

        if ($apiset->isa('NACL::APISet::Node::CLI::7Mode')) {
            # Is a Node-scope APISet; get the nodescope-node-name and the
            # updated path or volume name (with name-ordinal)
            my $hash = $pkg_or_obj->_get_node_and_name_ordinal(%opts);
            foreach my $key (qw(path nodescope-node-name volume)) {
                $opts{$key} = $hash->{$key} if (defined $hash->{$key});
            }
        } else {
            if (defined $opts{path}) {
                $opts{path} = $pkg_or_obj->_convert_path_to_cmode_format(
                    path => $opts{path});
            }
        }
    }

    my @ret = $pkg_or_obj->$base_call_on_apiset(%opts,
        apiset_backend_already_chosen => $hash);

    $Log->exit() if $may_exit;
    return wantarray ? @ret : $ret[0];
}

sub _get_node_and_name_ordinal {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params      => \@args,
        allow_extra => 1,
        ignore_primary_keys => 1
    );

	unless ($opts{vserver}) {
		$Log->exit() if $may_exit;
		return {};
	}

    my @requested_fields;
    if (!$opts{'nodescope-node-name'}
        && $opts{determine_appropriate_nodescope_node}) {
        push @requested_fields, 'node';
    }

    my ($vol_name, $contains_name_ordinal, $file, $new_volume);
    if (defined $opts{path}) {
        my $obj = $pkg_or_obj->_extract_volume_file_from_path(path => $opts{path});
        $vol_name = $obj->volume();
        $contains_name_ordinal = $obj->contains_name_ordinal();
        $file = $obj->file();
    } elsif (defined $opts{volume} ) {
        $vol_name = $opts{volume};
        if ($vol_name =~ /(.*)\((\d+)\)/) {
            $vol_name = $new_volume = $1;
            $contains_name_ordinal = $2;
            if (defined $contains_name_ordinal && $contains_name_ordinal =~ /\d+/) {
                 $new_volume .= "($contains_name_ordinal)";
            }
        }
    }

    if ($opts{handle_name_ordinal} && !$contains_name_ordinal) {
        push @requested_fields, 'name-ordinal';
    }

    if (@requested_fields) {
        my %filter = ( volume => $vol_name, vserver => $opts{vserver} );
        
        my %common_opts_with_ci;
        $pkg_or_obj->_copy_common_component_params_with_ci(
            source => \%opts,
            target => \%common_opts_with_ci
        );
        
        my $cs = NACL::CS::Volume->fetch(
            %common_opts_with_ci,
            filter           => \%filter,
            requested_fields => \@requested_fields
        );

        my $new_path;
        if ($cs->name_ordinal_isset()) {
            if (defined $opts{path}) {
                $new_path = $pkg_or_obj->_construct_file_path(
                    volume         => $vol_name,
                    file           => $file,
                    'name-ordinal' => $cs->name_ordinal()
                );
            }
        }
        $Log->exit() if $may_exit;
        return {
            'nodescope-node-name' => $cs->node(), path => $new_path, volume => $new_volume
        };
    } else {
        $Log->exit() if $may_exit;
        return {};
    }
}

sub _construct_volume_path {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            volume => { type => SCALAR },
            'name-ordinal' => { type => SCALAR | UNDEF, optional => 1 }
        },
        allow_extra => 1
    );
    my $name_ordinal = $opts{'name-ordinal'};
    
    my $path = "/vol/$opts{volume}";
    
    if (defined $name_ordinal && $name_ordinal =~ /\d+/) {
        $path .= "($name_ordinal)";
    }
    
    $Log->exit() if $may_exit;
    return $path;
}

sub _construct_file_path {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    state $spec = {file => {type => SCALAR}};
    my %opts = validate_with(
        params      => \@args,
        spec        => $spec,
        allow_extra => 1
    );
    my $file = delete $opts{file};
    my $path = $pkg_or_obj->_construct_volume_path(%opts);
    
    # If the file-name has quotes (like "file with spaces"), then
    # constructing the path like /vol/volname/"file with spaces" would be
    # incorrect because it would not work on the NGSH. Instead, the
    # entire path should be constructed within quotes: i.e. should be like
    # "/vol/volname/file with spaces".
    my $needs_quoting;
	if ($file =~ s/^["]+(.+?)["]+$/$1/) {
        $needs_quoting = 1;
    }
    $path .= "/$file";
    $path = '"' . $path . '"' if ($needs_quoting);
    $Log->exit() if $may_exit;
    return $path;
}

# CMode path should not have the name-ordinal and should be completely
# quoted if it has special characters/spaces. (i.e. should be like
# "/vol/volname/file name") 
sub _convert_path_to_cmode_format {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    state $spec = {path => {type => SCALAR}};
    my %opts = validate_with(params => \@args, spec => $spec);
    my $extract_obj = $pkg_or_obj->_extract_volume_file_from_path(path => $opts{path});
    my $cmode_path = $pkg_or_obj->_construct_file_path(
        file   => $extract_obj->file(),
        volume => $extract_obj->volume()
    );
    $Log->exit() if $may_exit;
    return $cmode_path;
}


# Wrapper around _extract_from_path so that this internal package
# doesn't need to be exposed.
# This method accepts "path" as input.
sub _extract_volume_file_from_path {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $obj = _extract_from_path->new(@args);
    $Log->exit() if $may_exit;
    return $obj;
}

=head2 set_permission

# Instance or Class method, sets file/dir permission
    $file_or_dir_obj->set_permission();

(Instance or Class method)

Uses a 7Mode/Nodescope APISet.

This method is used to set file/dir permission.
It also gets called if "permission" is passed to $Vol_Obj->file_create.

=over

=item Options

=over

=item C<< path => $path >>

Path of the dir/file. Required for Class method call

=item C<< vserver => $vserver >>

Required for Class method call

=item C<< permission => $permission >>

(Required). Permission to be set for file/dir.

=back

=item Exceptions

=over

=item C<APISet exceptions>

This method propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub set_permission {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my %opts = $self->_common_validate_with(
        params => \@args,
        additional_spec => {
            permission => { type => SCALAR },
            _optional_scalars(qw(path vserver)),
        },
    );

    my $ci = $self->command_interface();
    my $apiset = $ci->apiset(interface => 'CLI', set => 'CMode');

    my $path = $opts{path};
    if (! defined $path) {
        $path = $self->dblade_path();
    }

    my $permission = $opts{permission};
    if ($permission =~ /^\d\d\d$/) {
        $permission = "umask=010" . $permission;
    }

    # Path should be without /vol/ prefix
    $path =~ s/\/vol\///;
    my $retries = 3;
    my $lastflag = 0;

    while ($retries) {
        try {
            $apiset->volume_explore(
                format => 'inode',
                scope  => $path,
                set    => $permission,
            );
            $lastflag = 1;
        } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $e = $_[0];
            my $err_txt=$e->text();
            if($err_txt=~/error\s+70/i && --$retries){
                Tharn::snooze(10);
            }else{
                $Log->exit() if $may_exit;
                $e->throw();
            }
         };
         last if ( $lastflag );
    }
    $Log->exit() if $may_exit;
}

# Copies file or directory to $LOGDIR
sub copy_to_logdir {
    $Log->enter() if $may_enter;
    my ($file,@args) = @_;

    my %opts = validate_with(
        params      => \@args,
        spec        => {
            dest_dir => {
                type    => SCALAR,
                default => random_name_generator(prefix => 'copied_files')
            },
            %{$file->_common_validate_spec_without_ci()},
        }
    );

    my $file_path     = $file->path();
    my @file_components = split '/', $file_path;
    my $file_name = $file_components[$#file_components];

    my $dest_dir = Tharn::param('LOGDIR');
    $dest_dir .= '/'.$opts{dest_dir};

    try {
        my $dir_obj = NACL::C::Client::Directory->create(
        dir       => $dest_dir,
        recursive => 1);
    }
    # reuse dir if already exists
    catch NACL::C::Exceptions::Client::Directory::AlreadyExists with {};

    $Log->debug("Files to copy $file_name");
    $Log->debug("Copying file ".$file_path." to $dest_dir");

    NACL::MTask::SCP->copy_to_localhost(
        source_command_interface => $file->command_interface(),
        set_directory_on_source  => $file->systemshell_path(),
        set_directory_on_target => $dest_dir,
        files_to_copy           => [$file_name],
    );
    $Log->exit() if $may_exit;
}

# if the file path is "/vol/vol0/dir/myfile", then the returned directory object
# would model the directory "/vol/vol0/dir".
sub get_containing_directory_path {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    $self->_verify_invocation(style => 'instance_only');
    my $dir_path = $self->_determine_directory_path();
    $Log->exit() if $may_exit;
    return $dir_path;
}

=head2 systemshell_path

    my $systemshell_path = $dir_obj->systemshell_path();

(Instance method)
Returns the systemshell path of the VolumeFile or VolumeDirectory.

The only options accepted by this method are the common options (apiset_must,
apiset_should, method-timeout).

=cut

sub systemshell_path {
    $Log->enter() if $may_enter;
    my ($file, @args) = @_;

    my %opts = validate_with(
        params => \@args,
        spec   => {%{$file->_common_validate_spec_without_ci()},}
    );

    my %common_params = ();
    $file->_copy_common_component_params(
        source => \%opts,
        target => \%common_params
    );


    #Determin the Junction path. If Volume not junctioned
    #then throw an error.
    my $volume_obj    = $file->get_containing_volume_obj();
    my $junction_path = $volume_obj->get_one_state_attribute('junction-path',
        %common_params);

    # if Volume is not junctioned then the junction path would be '-'.
    # In such case there will not be a volume file or volume dir in
    # systemshell under /clus.
    if ($junction_path =~ /-/) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Volume passed is not junctioned");
    }

    my $directory_path = $file->get_containing_directory_path();
    my @dir_components = split '/', $directory_path;

    # Get the directory path
    # /vol/vol0/dir1 will give dir1
    # or /vol/vol0/dir1/dir2 will give dir1/dir2
    if (scalar @dir_components > 3) {
        splice @dir_components, 0, 3;
        $directory_path = join '/', @dir_components;
    } else {
        $directory_path = '';
    }

    my $path
        = '/clus/'
        . $file->vserver()
        . $junction_path . '/'
        . $directory_path;
    $Log->exit() if $may_exit;
    return $path;
} ## end sub systemshell_path

sub _create_helper {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(stream-name deduplicate-zero-data
                    size filecount step-size-in-blocks fragment-file
                    contiguous-fbns)
            ),
            $pkg->_nodescope_options_validate_spec(),
            '_is_vol_file' =>  {type => BOOLEAN},
        }
    );

    my $objs = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $objs;
}

sub _create_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $ci     = $opts{command_interface};
    my $apiset = $opts{apiset};
    my $is_vol_file = delete $opts{'_is_vol_file'};

    if ($ci->is_cmode() && !exists $opts{vserver} && $is_vol_file) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw("The argument 'vserver' "
                . 'needs to be provided in the call to create when call is '
                . 'made on a CMode system');
    }

    my %mkfile_args;
    my @allowed_options
        = $apiset->get_allowed_options_for_command(command => 'mkfile');
    push(@allowed_options, 'nodescope-node-name') if ($ci->is_cmode());

    $pkg->_hash_copy(
        source => \%opts,
        target => \%mkfile_args,
        copy   => \@allowed_options,
        map    => {path => 'filename'},
    );

    $apiset->mkfile(%mkfile_args);

    # If the filecount is specified (and is greater than 1) to mkfile, then
    # multiple files are created.
    #
    # If filecount is specified as 1, then a single file of name "name" is
    # created.
    #
    # For a value other than 1, the name of the file is <file> followed by a
    # dot followed by a numerical suffix from 0 till filecount-1. However,
    # this numerical suffix might have leading zeros depending on the value of
    # filecount.
    #
    # For example, assuming filename as "file", the files created for the
    # filecount values listed below are:
    # filecount=3: file.0, file.1, file.2
    # filecount=55: file.00, file.01, ..., file.10, file.11, ..., file.54
    # filecount=120: file.000, file.001, ... file.010, file.011, ...,
    # file.100, file.101, ..., file.119
    #
    # In more generalized terms: the numerical suffix is padded with leading
    # 0s till the number of digits in the suffix matches the number of digits
    # in the filecount provided.
    my @objs;
    my %new_args;
    if ($ci->is_cmode() && $is_vol_file) {
        $new_args{vserver} = $opts{vserver};
    }

    my $filecount = $mkfile_args{filecount};
    if (defined($filecount) && $filecount > 1) {
        my $path = $opts{path};
        my $is_quoted_path;
        if ($path =~ s/"$//) {
            $is_quoted_path = 1;
        }

        # Numbering starts from 0, so reduce filecount by 1
        my $num_digits = length($filecount - 1);
        for (my $i = 0; $i < $filecount; $i++) {

            # Total length of suffix should be $num_digits long, with leading
            # 0s if necessary.
            my $suffix = sprintf("%0${num_digits}d", $i);

            # If the path is contained in quotes (such as
            # "/vol/volname/file with spaces"), then we should ensure the
            # suffix is within the quotes (i.e. should be like
            # "/vol/volname/file with spaces.0: as opposed to
            # "/vol/volname/file with spaces".0)
            my $this_path;
            if ($is_quoted_path) {
                $this_path = "$path.$suffix" . '"';
            } else {
                $this_path = "$path.$suffix";
            }
            push @objs,
                $pkg->new(
                command_interface => $ci,
                path              => $this_path,
                %new_args
                );
        }
    } else {
        push @objs,
            $pkg->new(
            command_interface => $ci,
            path              => $mkfile_args{filename},
            %new_args
            );
    }

    $Log->exit() if $may_exit;
    return \@objs;
}

sub dumpblock_info {
	$Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(level blockno volume fileid private
                    aggregate snapid vserver)
            ),
            $pkg->_nodescope_options_validate_spec(),
        },
        ignore_primary_keys => 1
    );

    if ($pkg->isa('NACL::C::VolumeFile')) {
        $opts{vol_or_aggr_file} = "VolumeFile";
    } else {
        $opts{vol_or_aggr_file} = "AggregateFile";
    }

    my $obj;
    try {
        $obj = $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_dumpblock_info_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode|Nodescope'
                }
            ],
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];

        my $exception_pkg =
            "NACL::C::Exceptions::" . $opts{vol_or_aggr_file} . "::BlockIsAHole";
        $exception_pkg->detect_convert_and_throw( exception => $exception );

        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return $obj;
}

sub _dumpblock_info_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts   = $pkg->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};
    my $vol_or_aggr_file = delete $opts{'vol_or_aggr_file'};

    my %dumpblock_args;

    my @allowed_options = $apiset->get_allowed_options_for_command(
        command => 'dumpblock_info');
    push @allowed_options, 'nodescope-node-name';

    $pkg->_hash_copy(
        source => \%opts,
        target => \%dumpblock_args,
        copy   => \@allowed_options,
    );

    my $response = $apiset->dumpblock_info(%dumpblock_args);

    my $vol_aggr_cs_pkg = "NACL::CS::${vol_or_aggr_file}DumpblockInfo";
    eval "require $vol_aggr_cs_pkg";
    my $obj = $vol_aggr_cs_pkg->new(
        command_interface => $opts{command_interface}
    );
    $obj->_set_fields(row => $response->get_parsed_output()->[0]);
    $Log->exit() if $may_exit;
    return $obj;
}

sub showfh {
	$Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            $pkg_or_obj->_nodescope_options_validate_spec(),
        },
    );

    my $obj = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_showfh_7mode_cli',
                interface => 'CLI',
                set       => 'Nodescope|7Mode',
            }
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
}

sub _showfh_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $vol_or_aggr_file;
    if ($pkg_or_obj->isa('NACL::C::VolumeFile')) {
        $vol_or_aggr_file = 'VolumeFile';
    } else {
        $vol_or_aggr_file = 'AggregateFile';
    }

    my %showfh_args;

    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%showfh_args,
        copy   => ['nodescope-node-name'],
        map    => {path => 'filename'},
    );

    my $response = $apiset->showfh(%showfh_args);

    my $vol_aggr_cs_pkg = "NACL::CS::${vol_or_aggr_file}Showfh";

    eval "require $vol_aggr_cs_pkg";

    my $obj = $vol_aggr_cs_pkg->new(
        command_interface => $opts{command_interface});
    $obj->_set_fields(row => $response->get_parsed_output()->[0]);
    $Log->exit() if $may_exit;
    return $obj;
}
sub _delete_front_end_helper {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;

    $pkg_or_obj->_frontend_log_debug_opts(@args);

    # The 7Mode/Nodescope "rm" command allows for deleting multiple files
    # at once. Let's provide support for this through "paths" (for ZAPI
    # this would be done by calling the ZAPI once for each path)
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            path  => { type => SCALAR,   optional => 1 },
            paths => { type => ARRAYREF, optional => 1 },
            $pkg_or_obj->_nodescope_options_validate_spec(),
            '_is_vol_file' => { type => BOOLEAN },
        },
    );

    # Both "path" and "paths" cannot be provided
    if ( exists $opts{path} && $opts{paths} ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw( "Either of 'path' or 'paths' "
              . 'should have been provided but not both' );
    }

    if ( !exists $opts{path} && !$opts{paths} ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw( "Either of 'path' or 'paths' "
              . 'should have been provided but neither were' );
    }

    my $vol_or_aggr_file = $opts{_is_vol_file} ? 'VolumeFile' : 'AggregateFile';

    # Standardize on sending "paths" to the backend. If "path" is provided,
    # split it into an array, set that as the value of "paths" and delete
    # "path".

    if ( exists $opts{path} ) {

        # We need to determine if multiple paths are specified through "path".
        # This is possible by separating file paths with spaces.
        # The "test_delete_path" test-case has all the cases to be handled
        my $path = delete $opts{path};

        if ( $path =~ /\s/ ) {

            # Has spaces, so might be multiple file paths

            # We know that if delete() was invoked with an object instance,
            # then it has to be for a single path, so that's the first check
            # here.
            if ( ref $pkg_or_obj ) {
                $opts{paths} = [$path];
            }
            else {

                # Is a package call, so might be multiple file paths
                my $orig_path = $path;

                # Remove quotes so that the subsequent checks aren't tripped up
                # by quotes. The back-end will take care of correctly quoting,
                # so we don't need to worry about removing quotes here.
                $path =~ s/"//g;

                # Since volumes/aggregate can be named, just splitting on
                # /vol/ or /aggr/ will result in it splitting apart the
                # volume/aggregate name as well, so let's add a space at
                # the beginning, so that we know
                # <space>/aggr/ or <space>/vol/ is the beginning of a new file.
                $path = " $path";

                my $split = $opts{_is_vol_file} ? 'vol' : 'aggr';
                my @file_paths = split /\s+\/$split\//, $path;

                # The first entry in the array will be empty, so shift it out.
                shift @file_paths;

                foreach my $single_path (@file_paths) {

                    # All paths but the last one will have trailing spaces, so
                    # strip them out.
                    $single_path =~ s/\s+$//;

                    # The split() would have removed the /vol/ or /aggr/,
                    # so we need to add it back to each path
                    $single_path = "/$split/" . $single_path;
                }

                if ( @file_paths > 1 ) {
                    $Log->warn( "NACL::C::"
                          . $vol_or_aggr_file
                          . "->delete was invoked with "
                          . "multiple paths specified in the 'path' argument "
                          . "(value = '$orig_path'). For specifying multiple "
                          . "file paths, the 'paths' argument should be used instead."
                    );
                }

                $opts{paths} = \@file_paths;
            }
        }
        else {

            # No spaces in file path, so must be a single path
            $opts{paths} = [$path];
        }
    }

    my @choices = (
        {
            method    => '_delete_7mode_cli',
            interface => 'CLI',
            set       => '7Mode|Nodescope',
        }
    );

    # Volume File will support the ZAPI interface
    if ( $opts{_is_vol_file} ) {
        unshift(
            @choices,
            {
                method    => '_delete_zapi',
                interface => 'ZAPI',
                set       => 'CMode|7Mode',
                check     => '_check_for_vserver_sksim'
            },
        );
    }

    try {

        # Cannot go through this package's call_on_apiset because
        # %opts does not contain "path", instead it contains "paths".
        # Let the back-end methods handle the name-ordinal,
        # nodescope-node-name and quotes

        my $base_class          = $pkg_or_obj->base_class();
        my $base_call_on_apiset = "${base_class}::call_on_apiset";

        $pkg_or_obj->$base_call_on_apiset(
            %opts,
            choices  => \@choices,
            frontend => "NACL::C::" . $vol_or_aggr_file . "::delete"
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];

        my $exception_pkg =
          "NACL::C::Exceptions::" . $vol_or_aggr_file . "::DoesNotExist";
        $exception_pkg->detect_convert_and_throw( exception => $exception );

        $Log->exit() if $may_exit;
        $exception->throw();

    };

    $Log->exit() if $may_exit;
}

sub _check_sksim {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $is_sksim = $opts{command_interface}->version_manager()->is_sksim();
    if ($is_sksim) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw('Cannot use ZAPI '
            . 'implementation for sksims');
    }
    $Log->exit() if $may_exit;
}

1;

package _extract_from_path;

use strict;
use warnings;
use feature 'state';

use Params::Validate qw(:all);
use NATE::Exceptions::Argument;
use NATE::Log qw(log_global);
$Log = log_global();
$may_enter = $Log->may_enter();
$may_exit  = $Log->may_exit();

use Class::MethodMaker [
    new    => [ '-hash', '-init', 'new' ],
    scalar => 'path',
    scalar => 'volume',
    
    # The file-name appropriately quoted (if it has # or ; or spaces, then the
    # value is quoted). The following paths should all result in this
    # value having quotes:
    # * /vol/volname/"file with spaces"
    # * /vol/volname/file with spaces
    # * "/vol/volname/file with spaces"
    # * /vol/volname/a#89;1
    scalar => 'file',
    
    # The original file-name, i.e. if the file-name was provided with quotes,
    # then this value will have quotes, if it was provided without quotes,
    # then this value will contain quotes.
    scalar => 'orig_file',
    scalar => 'name_ordinal',
    scalar => 'contains_name_ordinal',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    state $spec = {path => {type => SCALAR}};
    my %opts = validate_with(
        params => \@args,
        spec   => $spec,
    );
    my $path = $opts{path};
    
    # Possible for the path to be completely quoted. Example:
    # "/vol/volname/file with spaces". The subsequent regex checks
    # if the path begins with /vol so remove the leading quote, if it exists.
    # Also validate that if there's a leading quote that there's a trailing
    # quote.
    if ($path =~ s/^"//) {
        if ($path !~ /"$/) {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw("The path provided ('\"$path') "
                . 'is invalid (it has a double quote at the beginning but '
                . 'not at the end)'
            );
        }
    }
    
    if ($path =~ m!/vol/(.*?)/(.*)!) {
        my ($vol_name, $file_name) = ($1, $2);
        my $orig_file_name = $file_name;
        
        
        if ($file_name =~ /"$/) {
            if ($file_name !~ /^"/) {
                $file_name = '"' . $file_name;
                $orig_file_name = $file_name;
            }
        } elsif ($file_name =~ /[;#\s]/) {
            # If the filename has the special characters # or ; or spaces, then
            # it needs to be quoted.
            $file_name = '"' . $file_name . '"';
        }
        $self->file($file_name);
        $self->orig_file($orig_file_name);
        
        my $name_ordinal;

        # This string might contain the name-ordinal, which we'd need
        # to strip out.
        # for example, "volname(1)" -> "volname"
        if ($vol_name =~ /(.*)\((\d+)\)/) {
            $vol_name = $1;
            $name_ordinal = $2;
        }
        
        $self->volume($vol_name);
        if (defined $name_ordinal) {
            $self->contains_name_ordinal(1);
            $self->name_ordinal($name_ordinal);
        } else {
            $self->contains_name_ordinal(0);
        }
    } else {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "The path ('$path') is not in the expected format");
    }
    
    $Log->exit() if $may_exit;
}

1;
