# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Module to check supported features
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Version;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR OBJECT BOOLEAN SCALARREF);
use NACL::ComponentUtils qw(_ontap_ci_validate_spec _hash_move);
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);
use NATE::BaseException qw(:try);
use Data::Dumper;
our %Features_To_Check = (
    'license_v1'                 => '_check_if_licensing_v1',
    'license_v2'                 => '_check_if_licensing_v2',
    'license_v2_zapi_7m'         => '_check_if_licensing_v2_zapi',
    'snapshotpolicy_vserverization' =>
        '_check_snapshot_policy_vserverization',
    'vserver_required_for_ldap_client_schema' =>
        '_ldap_client_schema_vserver_check',
    'ipv6_feature'                 => '_system_uichanges_ipv6_check',
    'ipspaces_feature'             => '_system_uichanges_ipspaces_check',
    'force_offline_for_volume'     => '_force_offline_volume',
    'qos_RR'                       => '_qos_workload_stream_check',
    'qos_SN'                       => '_qos_policy_group_check',
    'source_snapshot_only_in_test_privilege' =>
        '_source_snapshot_in_test_privilege',
    'vserver_cluster_data' => '_system_uichanges_vserver_cluster_data',
    'statistics_v2'        => '_statistics_v2_check',
    'vserver_cifs_share_properties_add' =>
        '_check_vserver_cifs_share_properties_add',
    'vserver_peer_transition_alternate_ip_option' => 
        '_vserver_peer_transition_alternate_ip_option',
    'flexcache' => '_is_flexcache_supported',
    'dedupe_always_on' => '_is_dao_supported',
    'vserver_kerberos_realm' => '_vserver_kerberos_realm_check',
    'name_service_file_version' => '_check_name_service_file_version',
    'fusen_mode' => '_is_fusen_mode',
);

=head1 NAME

NACL::C::_Version

=head1 DESCRIPTION

This private library is to be used to determine if a feature is present
or not. It may employ any of a number of mechanisms which include
checking version details, checking for presence of uichange strings
or querying the service to determine which options are supported.
Since this is a private library, it is encouraged to use the method
L<is_feature_supported()|lib-NACL-C-CommandInterface-pm/is_feature_supported>
from command interface object.

=head1 METHODS

=head2 is_feature_supported

    my $boolean = $command_interface->is_feature_supported(
        feature => $feature_name
    );

    # Perform any operation
    if ( $boolean ) { .. }

(Class method)
This method is used to determine whether a particular feature is supported
or not. Returns boolean value

    1 : If the specified feature is supported,
    0 : If the specified feature is not supported

See L<NACL::C::CommandInterface::is_feature_supported()|lib-NACL-C-CommandInterface-pm/is_feature_supported>.

See the section L<FEATURES|lib-NACL-C-_Version-pm/FEATURES> for the list of features.

=over

=item command_interface

(Required) A component object that represents the host to which to send
commands.

=item feature

(Required) Name of the feature which needs to be verified.

=back

=head2 has_uichange

This method is used to determine whether a particular uichange exists
or not. 
See L<NACL::C::CommandInterface::ONTAP::has_uichange()|lib-NACL-C-CommandInterface-ONTAP-pm/has_uichange>
for more details.

See the section L<UICHANGES|lib-NACL-C-_Version-pm/UICHANGES> for the list of uichanges which have custom logic
implemented.

=head1 FEATURES

Feature names are case sensitive. Currently supported features are:

=head2 license_v1

Check whether the release supports Licensing version 1.

=over

=item Supported modes

CMode, 7Mode.

=back

=head2 license_v2

Check whether the release supports Licensing version 2.

=over

=item Supported modes

CMode, 7Mode

=back

=head2 license_v2_zapi_7m

Check whether the release supports Licensing version 2 for 7Mode ZAPI.

=over

=item Supported modes

7Mode

=back

=head2 snapshotpolicy_vserverization

Check whether the snapshot policy commands are vserverized.

=over

=item Supported modes

CMode

=back

=head2 vserver_required_for_ldap_client

Check whether the vserver argument is required for the
'vserver services ldap client' commands.

=over

=item Supported modes

CMode

=back

=head2 vserver_required_for_ldap_client_schema

Check whether the vserver argument is required for the
'vserver services ldap client schema' commands.

=over

=item Supported modes

CMode

=back

=head2 ipv6_feature

Check whether the release supports IPv6.

=over

=item Supported modes

CMode

=back

=head2 ipspaces_feature

Check whether the release supports ipspaces feature.

=over

=item Supported modes

CMode

=back

=head2 force_offline_for_volume

Check whether "force-offline" value is supported for the "volume modify"
command.

=over

=item Supported modes

CMode

=back

=head2 qos_RR

Check whether the release supports qos workload show and qos workload stream show

=over

=item Supported modes

CMode

=back

=head2 qos_SN

Check whether the release supports qos policy-group show command

=over

=item Supported modes

CMode

=back

=head2 source_snapshot_only_in_test_privilege

Checks if 'source-snapshot' is supported only in test privilege.
(Note that now the APISet automatically sets to the highest privilege level
possible, so calls to this should no longer be needed)

=over

=item Supported modes

CMode

=back

=head2 vserver_cluster_data

Checks for the presence of uichange string - "vserver-cluster-data"

=over

=item Supported modes

CMode

=back

=head2 statistics_v2

Checks whether the build is running statistics v2. See
L<NACL::C::Statistics|lib-NACL-C-Statistics-pm>
for a description of changes that need to be made to calls to work for
v2.

=over

=item Supported modes

CMode

=back

=head2 vserver_cifs_share_properties_add

Checks whether directory  "vserver cifs share properties"
exists.

=over

=item Supported modes

CMode

=back

=head2 vserver_peer_transition_alternate_ip_option

Checks if the build has the UI change in which 'alternate-ip'
option of 'vserver peer transition' CLI commands and ZAPI
APIs are renamed to 'multi-path-address'. Returns 1, if the
build expects the newer option 'multi-path-address'.

=over

=item Supported modes

CMode

=back

=head2 ondemand_luncopy

Checks if the build has 'source-paths'
option of 'lun copy' CLI command. Returns 1, if the
option is available

=over

=item Supported modes

CMode

=back

=head2 dedupe_always_on

Checks if the Dedupe Always On feature is supported. This is only
applicable for CMode.

=head2 name_service_file_version

Check whether the release supports 'vserver-services-name-service-file-version-show' command.

=over

=item Supported modes

CMode

=back

=head2 is_fusen_mode

This feature helps us to check whether fusen_mode is enabled/disabled.

=over

=cut

sub is_feature_supported {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            feature => { type => SCALAR },
            command_interface =>
                { type => OBJECT, isa => 'NACL::C::CommandInterface' }
        },
        # Allow through other options (such as the common component options)
        allow_extra => 1
    );
    if ( !exists $Features_To_Check{ $opts{feature} } ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw( "Provided feature '$opts{feature}'"
                . " is either invalid or unsupported. The supported features"
                . " are: "
                . join( ', ', ( keys %Features_To_Check ) ) );
    } ## end if ( !exists $Features_To_Check...)

    my $feature = $opts{feature};

    # Chack version manager attribute to see if this feature is
    # already verified before.
    my $version_manager = $opts{command_interface}->version_manager();
    if ( $version_manager->features_exists($feature) ) {
        $Log->exit() if $may_exit;
        return $version_manager->features_index($feature);
    }

    # Not verified, call corresponding method to check whether
    # the feature is supported or not.
    my $method  = $Features_To_Check{$feature};
    my $is_true = $pkg->$method(%opts);
    $version_manager->features_set( $feature => $is_true );

    $Log->exit() if $may_exit;
    return $is_true;
} ## end sub is_feature_supported

=head1 UICHANGES

The list of uichanges for which there is extra logic added for Vserver
version managers are:

=head2 vserver-peer-transition

=head2 antivirus-vserverization

=cut

# Adding additional logic to determine whether a change (represented by the
# uichange string) for Vserver version managers (i.e. isa
# 'NACL::APISet::Version::Node::Vserver') should be done in a
# _check_uichange_<uichange> method. To conform with Perl's limitations
# on subroutine names, the method name corresponding to a uichange name will
# have all hyphens and dots in the uichange name converted to underscores. For
# example, the method corresponding to the uichange "vserver-peer-transition" 
# should be named "_check_uichange_vserver_peer_transition".

sub has_uichange {
    $Log->enter() if $may_enter;
    my ($pkg, @args)  = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            uichange => { type => SCALAR },
            command_interface => _ontap_ci_validate_spec(),
            # purposely undocumented. Only for the internal purpose of components.
            throw_exception => {type => BOOLEAN, default => 1}
        }
    );
    my $ci = $opts{command_interface};
    if ($ci->is_7mode) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Method 'has_uichange' is supported only for CMode");
    }
    my $version_manager = $ci->version_manager();
    my $uichange = $opts{uichange};
    my $throw_exception = delete $opts{throw_exception};
    my $bool;
    
    # Check the version manager's cache of uichanges first
    if ($version_manager->can('has_uichange')) {
        my $val = $version_manager->has_uichange(uichange => $uichange);
        $Log->debug("uichanges entry has the info, value: $val");
        $Log->exit() if $may_exit;
        return $val;
    }
    
    # Check if the cache of additional uichanges has this info
    if ( $version_manager->additional_uichanges_exists($uichange) ) {
        my $val = $version_manager->additional_uichanges_index($uichange);
        $Log->debug("Additional uichanges hash has the info, value: $val");
        $Log->exit() if $may_exit;
        return $val;
    }
    
    my $uichange_copy = $uichange;
    $uichange_copy =~ s/[-\.]/_/g;
    
    my $method = '_check_uichange_'. $uichange_copy;
    if ($pkg->can($method)) {
        # Check if additional logic is defined to find the presence of
        # uichange string.
        $bool = $pkg->$method(%opts);
        $version_manager->additional_uichanges_set( $uichange => $bool );
    } elsif ($throw_exception) {
        # If we've reached here, then the version manager does not have
        # the "has_uichange" method implemented on it (currently that would
        # mean that it's a vserver version manager) and there is no
        # _check_uichange_* method for this uichange so there's no way we
        # can determine if the uichange is present or not for this build.
        # Decide whether to throw an exception based on the value of
        # "throw_exception". ("throw_exception" = 1 means "throw exception",
        # 0 would result in $bool being returned, which would be undef)
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("uichange info for '$uichange' ".
            'cannot be discovered from vserver directly');
    }
    
    $Log->exit;
    return $bool;
}

#################### Internal methods here ##################

sub _check_if_licensing_v1 {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my $val = $pkg->_check_licensing( version => 'v1', @_ );
    $Log->exit() if $may_exit;
    return $val;
} ## end sub _check_if_licensing_v1

sub _check_if_licensing_v2 {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my $val = $pkg->_check_licensing( version => 'v2', @_ );
    $Log->exit() if $may_exit;
    return $val;
} ## end sub _check_if_licensing_v2

sub _check_licensing {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            %{ $pkg->_backend_validate_spec() },
            version => { type => SCALAR },
        }
    );

    my $ci              = $opts{command_interface};
    my $version         = $opts{version};
    my $version_manager = $ci->version_manager();
    my $rlse            = $version_manager->is_version_lt(release => '8.2');

    if ( $rlse ) {
        $version_manager->features_set('license_v1' => 1);
        $version_manager->features_set('license_v2' => 0);
        $Log->exit() if $may_exit;
        return 0 if ($version eq 'v2');
    } else {
        $version_manager->features_set('license_v2' => 1);
        $version_manager->features_set('license_v1' => 0);
        $Log->exit() if $may_exit;
        return 0 if ($version eq 'v1');
    }
    $Log->exit() if $may_exit;
    return 1;

} ## end sub _check_licensing

sub _check_snapshot_policy_vserverization {

    return 1;

} ## end sub _check_snapshot_policy_vserverization

#vserver_services_ldap_client_schema_show

sub _ldap_client_schema_vserver_check {
    $Log->enter() if $may_enter;
    my $pkg   = shift;
    my $value = $pkg->_check_for_presence_of_cmode_cli_field(
        @_,
        api             => 'vserver_services_ldap_client_schema_show',
        field_to_verify => 'vserver'
    );
    $Log->exit() if $may_exit;
    return $value;
} ## end sub _ldap_client_schema_vserver_check

sub _system_uichanges_ipv6_check {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my $value
        = $pkg->_check_presence_uichange_helper( @_, uichange => 'ipv6-ms5' );
    $Log->exit() if $may_exit;
    return $value;
} ## end sub _system_uichanges_ipv6_check

sub _system_uichanges_ipspaces_check {
    $Log->enter() if $may_enter;
    my $pkg   = shift;
    my $value = $pkg->_check_presence_uichange_helper( @_,
        uichange => 'ipspaces-ms4' );
    $Log->exit() if $may_exit;
    return $value;
} ## end sub _system_uichanges_ipspaces_check

sub _force_offline_volume {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => $pkg->_backend_validate_spec()
    );
    my $ci = $opts{command_interface};

    if ( $ci->is_7mode() ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw( "'$opts{feature}' is a CMode-only "
                . 'feature but the method was invoked with a 7Mode '
                . 'command interface' );
    } ## end if ( $ci->is_7mode() )

    my $version_manager = $ci->version_manager();

    # Supported only for SN and above. Hence, no need to run any commands
    # if < SN.
    # TODO: update this code once burt 457612 is fixed.
    if ( $ci->version_manager()->is_version_lt(release => '8.2') ) {
        $Log->exit() if $may_exit;
        return 0;
    }

    if ($version_manager->has_uichange(uichange => 'sfmod', throw_exception => 0)) {

        # Presence of "sfmod" uichange denotes that "force-offline" is
        # definitely available. Absence of it however does not necessarily
        # guarantee that the option is not available. Hence, if the uichange
        # is absent, we still issue "volume modify ?" to confirm.
        $Log->exit() if $may_exit;
        return 1;
    } ## end if ( $version_manager->has_uichange...)

    my $apiset = $ci->apiset( interface => 'CLI', set => 'CMode' );

    my $response = $apiset->volume_modify( pass_through => '?' );
    my $output   = $response->get_processed_output();
    my @lines    = split /\n/, $output;
    my $regex    = $response->_regex_for_each_field();

    foreach my $line (@lines) {
        if ( $line =~ $regex ) {
            my $field = $1;
            if ( $field eq 'state' ) {
                if ( $line =~ /force\-offline/ ) {
                    $Log->exit() if $may_exit;
                    return 1;
                }
            } ## end if ( $field eq 'state')
        } ## end if ( $line =~ $regex )
    } ## end foreach my $line (@lines)

    $Log->exit() if $may_exit;
    return 0;
} ## end sub _force_offline_volume

sub _qos_workload_stream_check {
    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;

    my $val_workload = $pkg->_qos_checks(
        feature => 'qos_RR',
        api     => 'qos_workload_show',
        %opts
    );

    my $val_stream = $pkg->_qos_checks(
        feature => 'qos_RR',
        api     => 'qos_workload_stream_show',
        %opts
    );
    if ( ( $val_workload eq '1' ) && ( $val_stream eq '1' ) ) {
        $Log->exit() if $may_exit;
        return 1;
    } else {
        $Log->exit() if $may_exit;
        return 0;
    }
} ## end sub _qos_workload_stream_check

sub _qos_policy_group_check {
    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;
    my $val = $pkg->_qos_checks(
        feature => 'qos_SN',
        api     => 'qos_policy_group_show',
        %opts
    );
    $Log->exit() if $may_exit;
    return $val;
} ## end sub _qos_policy_group_check

sub _qos_checks {
    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;

    my $val = $pkg->_is_command_present(%opts);
    $Log->exit() if $may_exit;
    return $val;
} ## end sub _qos_checks

sub _check_vserver_cifs_share_properties_add {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    my $value = $pkg->_is_command_present( @_,
        api => 'vserver_cifs_share_properties_add' );
    $Log->exit() if $may_exit;
    return $value;

} ## end sub _check_vserver_cifs_share_properties_add

sub _source_snapshot_in_test_privilege {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => $pkg->_backend_validate_spec()
    );
    my $ret = $pkg->_check_for_presence_of_cmode_cli_field(
        %opts,
        api               => 'snapmirror_initialize',
        field_to_verify   => 'source-snapshot',
        'privilege-level' => 'diagnostic',
    );
    $Log->debug("is field present in diag priv level: $ret");

    # Negate
    $ret = !$ret;

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub _source_snapshot_in_test_privilege

sub _system_uichanges_vserver_cluster_data {
    $Log->enter() if $may_enter;
    my $pkg   = shift;
    my $value = $pkg->_check_presence_uichange_helper( @_,
        uichange => 'vserver-cluster-data' );
    $Log->exit() if $may_exit;
    return $value;
} ## end sub _system_uichanges_vserver_cluster_data

sub _statistics_v2_check {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => $pkg->_backend_validate_spec()
    );
    my $ci = $opts{command_interface};

    my $is_v2 = $pkg->_check_for_presence_of_cmode_cli_field(
            %opts,
            api             => 'statistics_show',
            field_to_verify => 'raw'
        );
        $Log->exit() if $may_exit;
        return $is_v2;
} ## end sub _statistics_v2_check

sub _check_if_licensing_v2_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => { %{ $pkg->_backend_validate_spec() }, }
    );

    my $is_v2_zapi = 1;
    my $ci         = $opts{command_interface};
    my $api        = $ci->apiset(
        category  => 'Node',
        interface => 'ZAPI',
        set       => $ci->mode()
    );
    my $version_manager = $ci->version_manager();
    my $rlse = $version_manager->is_version_ge(release => '8.2');

    if ( $rlse ) {
        try {
            $api->license_v2_list_info();
        }
        catch NACL::APISet::Exceptions::CommandNotFoundException with {
            $is_v2_zapi = 0;
        };
    } else {
        $is_v2_zapi = 0;
    }

    $Log->exit() if $may_exit;
    return $is_v2_zapi;
} ## end sub _check_if_licensing_v2_zapi

sub _vserver_peer_transition_alternate_ip_option{
    $Log->enter() if $may_enter;
    my ($pkg, @args)  = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => $pkg->_backend_validate_spec()
    );
    my $ret = $pkg->_check_for_presence_of_cmode_cli_field(
        %opts,
        api               => 'vserver_peer_transition_create',
        field_to_verify   => 'multi-path-address',
    );
    $Log->debug("is 'multi-path-address' field present: $ret");
    
    $Log->exit() if $may_exit;
    return $ret;
}


# Dedupe Always On (DAO) was in FS.0, but was pulled out. This check is needed
# because there's no uichange-string and we need to support automation on
# builds that do and do not support this feature.
sub _is_dao_supported {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    
    my $value;
    if ($opts{command_interface}->version_manager()->is_version_ge(release => '8.3')) {
        $value = $pkg->_is_command_present(%opts,
            api => 'volume_efficiency_promote');
    } else {
        $value = 0;
    }

    $Log->exit() if $may_exit;
    return $value;
}

##################### UICHANGE BACK-END METHODS GO HERE ###################

sub _check_uichange_vserver_peer_transition {
    $Log->enter() if $may_enter;
    
    my ($pkg, %opts) = @_;
    my $ci = $opts{command_interface};
    my $apiset = $ci->apiset(interface => 'CLI', set => 'CMode');
    my $ret = $apiset->is_option_present(
        api   => 'vserver_peer_transition_show',
        field => 'local-vserver',
    );
    
    $Log->exit($ret);
    return $ret;
}

sub _check_uichange_antivirus_vserverization {
    $Log->enter() if $may_enter;
    
    my ($pkg, %opts) = @_;
    my $ci = $opts{command_interface};
    
    # Vserverized from SN.0 onwards
    my $ret = $ci->version_manager()->is_version_ge(release => '8.2');
    
    $Log->exit($ret);
    return $ret;
}

sub _check_uichange_lun_move_renamed_to_lun_move_in_volume {
    $Log->enter() if $may_enter;

    my ($pkg, %opts) = @_;
    my $ci = $opts{command_interface};
    my $apiset = $ci->apiset(interface => 'CLI', set => 'CMode');
    my $value = $apiset->is_command_present(api => 'lun_move_start');
    $Log->exit() if $may_exit;
    return $value;
}

sub _is_flexcache_supported {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            %{$pkg->_backend_validate_spec()},
            reason_ref => { type => SCALARREF, optional => 1 },
        },
        # All other options go through to state and get validated there
        allow_extra => 1
    );
    ${$opts{reason_ref}} = "FlexCache was disabled in FS" ;
    $Log->exit;
    return 0;
}
sub _is_fusen_mode {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $ci = $opts{command_interface};
    my $apiset = $ci->get_systemshell_apiset();
    my $obj = $apiset->is_fusen_mode();

    my $parsed_output = $obj->get_parsed_output();
    my $value = ${@$parsed_output[0]}{'fusen_mode'};
    my $is_enabled = ($value =~ /enabled/i) ? 1 : 0;
    $Log->exit() if $may_exit;
    return $is_enabled;
}

######################## Common helper routines here ###################

# Check for the presence of an option in the CMode CLI UI.
sub _check_for_presence_of_cmode_cli_field {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            api               => { type => SCALAR },
            field_to_verify   => { type => SCALAR },
            'privilege-level' => { type => SCALAR, optional => 1 },
            %{ $pkg->_backend_validate_spec() }
        }
    );
    my $api             = $opts{api};
    my $field_to_verify = $opts{field_to_verify};
    my $priv_level      = $opts{'privilege-level'};
    my $ci              = $opts{command_interface};

    my $apiset = $ci->apiset( interface => 'CLI', set => 'CMode' );
    my $ret = $apiset->is_option_present(
        api               => $api,
        field             => $field_to_verify,
        'privilege-level' => $priv_level
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub _check_for_presence_of_cmode_cli_field

# Helper routine when we need to check for presence of a uichange
sub _check_presence_uichange_helper {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            %{ $pkg->_backend_validate_spec() },
            uichange => { type => SCALAR }
        }
    );
    $Log->exit() if $may_exit;
    return $opts{command_interface}->version_manager()
        ->has_uichange(uichange => $opts{uichange}, throw_exception => 0);
} ## end sub _check_presence_uichange_helper

# Check if a command is present or not
sub _is_command_present {

    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            %{ $pkg->_backend_validate_spec() }, api => { type => SCALAR }
        }
    );
    my $ci  = $opts{command_interface};
    my $api = $opts{api};

    my $apiset = $ci->apiset( interface => 'CLI', set => 'CMode' );

    my $is_present = $apiset->is_command_present( api => $api );
    $Log->exit() if $may_exit;
    return $is_present;

} ## end sub _is_command_present

# Validation spec for all of the internal methods
sub _backend_validate_spec {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;
    return {
        command_interface =>
            { type => OBJECT, isa => 'NACL::C::CommandInterface' },
        feature => { type => SCALAR }
    };
} ## end sub _backend_validate_spec

################# Only for testing purposes#######################

sub _get_applicable_features_hash_for_rr {
    my %full_features = %Features_To_Check;
    my %features = (
        vserver_peer_transition_alternate_ip_option => 1,
    );
    foreach my $feature (keys %features) {
        delete $full_features{$feature};
    }
    return \%full_features;
}

sub _get_applicable_features_hash_for_sn {
    my %full_features = %Features_To_Check;    
    return \%full_features;
}

sub get_applicable_features {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec => {command_interface => { type => OBJECT, isa => 'NACL::C::CommandInterface' } }
    );
    my $ci = $opts{command_interface};
    my $features;
    if ($ci->version_manager()->is_version_lt( release => "8.2" )) {        
        $features =  _get_applicable_features_hash_for_rr();
    } else {        
        $features = _get_applicable_features_hash_for_sn();
    }
    $Log->exit() if $may_exit;
    return $features;
}

sub get_applicable_uichanges_list_for_sn {
    return ( qw(vserver-peer-transition antivirus-vserverization) );
}

sub get_applicable_uichanges_list_for_rr {
    return ( qw(antivirus-vserverization) );
}

sub _uichanges_list {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec => {command_interface => { type => OBJECT, isa => 'NACL::C::CommandInterface' } }
    );
    my $ci = $opts{command_interface};
    my @uichange_list;
    if ($ci->version_manager()->is_version_lt( release => "8.2" )) {
	@uichange_list = get_applicable_uichanges_list_for_rr();
    } else {
	@uichange_list = get_applicable_uichanges_list_for_sn();
    }
    $Log->exit() if $may_exit;
    return @uichange_list;
}

sub _vserver_kerberos_realm_check {
     $Log->enter() if $may_enter;
     my ($pkg, @args) = @_;
     my $value = $pkg->_check_for_presence_of_cmode_cli_field(
        @args,
        api             => 'vserver_services_kerberos_realm_create',
        field_to_verify => 'vserver'
     );
     $Log->exit() if $may_exit;
     return $value;
} ## end sub _vserver_kerberos_realm_check

sub _check_name_service_file_version {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $value;
    if ($opts{command_interface}->version_manager()->is_version_ge(release => '8.3')) {
        $value = $pkg->_is_command_present(%opts,
            api => 'vserver_services_name_service_file_version_show');
    } else {
        $value = 0;
    }

    $Log->exit() if $may_exit;
    return $value;
}
1;
