#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary EventRoute ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::EventRoute

=head1 DESCRIPTION

C<NACL::CS::EventRoute> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP EventRoute.  A related class is
L<NACL::C::EventRoute|lib-NACL-C-EventRoute-pm>, which represents access
to an ONTAP EventRoute.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the EventRoute
element are the attributes of the EventRoute ComponentState.

=over

=item C<< "messagename" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <ems-routing-info>
    <message-name>$value</message-name>
 </ems-routing-info>

=item C<< "timethreshold" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <ems-routing-info>
    <time-threshold>$value</time-threshold>
 </ems-routing-info>

=item C<< "description" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <ems-routing-info>
    <description>$value</description>
 </ems-routing-info>

=item C<< "frequencythreshold" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <ems-routing-info>
    <frequency-threshold>$value</frequency-threshold>
 </ems-routing-info>

=item C<< "action" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <ems-routing-info>
    <action>$value</action>
 </ems-routing-info>

=item C<< "destinations" >>

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
 my $destinations = $obj->destinations();
 # $destinations contains a reference to the array of values
 my @destinations = $obj->destinations();
 # @destinations contains the array of values

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <ems-routing-info>
    <destinations>
        <ems-destination>@values</ems-destination>
    </destinations>
 </ems-routing-info>

=item C<< "severity" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <ems-routing-info>
    <severity>$value</severity>
 </ems-routing-info>

=item C<< "snmp_support" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <ems-routing-info>
    <snmp_support>$value</snmp_support>
 </ems-routing-info>

=back

=cut

package NACL::CS::EventRoute;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::ComponentUtils qw(_dump_one _optional_scalars Dumper);

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use Class::MethodMaker [
    scalar => 'messagename',
    scalar => 'timethreshold',
    scalar => 'description',
    scalar => 'frequencythreshold',
    scalar => 'action',
    array  => 'destinations',
    scalar => 'severity',
    scalar => 'snmp_support',
];

=head1 METHODS

=head2 fetch

    my $EventRoute_state   = NACL::CS::EventRoute->fetch(command_interface => $ci, ...);
	my @EventRoute_states = NACL::CS::EventRoute->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "ems-routing-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
	show_cmd       => 'event route show',
        exception_text => 'No matching event route(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'event_route_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg    = shift;
    my @states = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => "ems_routing_get_iter",
        copy => [qw(action description severity snmp-support)],
        map  => {
            "frequencythreshold" => ['frequency-threshold'],
            "messagename"        => ['message-name'],
            "timethreshold"      => ['time-threshold'],
            "destinations"       => [
                make_zapi_array('destinations'),
                make_zapi_skip('ems-destination'),
            ],
        },
    );

    $Log->exit() if $may_exit;
    return @states;
} ## end sub _fetch_cmode_zapi
1;
