#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary JobScheduleCron ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::JobScheduleCron

=head1 DESCRIPTION

C<NACL::CS::JobScheduleCron> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP JobScheduleCron.  A related class is
L<NACL::C::JobScheduleCron|lib-NACL-C-JobScheduleCron-pm>, which represents access
to an ONTAP JobScheduleCron.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the JobScheduleCron
element are the attributes of the JobScheduleCron ComponentState.

=over

=item C<< "hour" >>
(Array)

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-schedule-cron-info>
     <job-schedule-cron-hour>
         <cron-hour>@values</cron-hour>
     </job-schedule-cron-hour>
 </job-schedule-cron-info>

=item C<< "minute" >>
(Array)

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-schedule-cron-info>
     <job-schedule-cron-minute>
         <cron-minute>@values</cron-minute>
     </job-schedule-cron-minute>
 </job-schedule-cron-info>

=item C<< "month" >>
(Array)

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-schedule-cron-info>
     <job-schedule-cron-month>
         <cron-month>@values</cron-month>
     </job-schedule-cron-month>
 </job-schedule-cron-info>

=item C<< "dayofweek" >>
(Array)

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-schedule-cron-info>
     <job-schedule-cron-day-of-week>
         <cron-day-of-week>@values</cron-day-of-week>
     </job-schedule-cron-day-of-week>
 </job-schedule-cron-info>

=item C<< "name" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-schedule-cron-info>
     <job-schedule-name>$value</job-schedule-name>
 </job-schedule-cron-info>

=item C<< "day" >>
(Array)

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-schedule-cron-info>
     <job-schedule-cron-day>
         <cron-day>@values</cron-day>
     </job-schedule-cron-day>
 </job-schedule-cron-info>

=item C<< "description" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <job-schedule-cron-info>
     <job-schedule-description>$value</job-schedule-description>
 </job-schedule-cron-info>

=back

=cut

package NACL::CS::JobScheduleCron;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    array  => 'hour',
    array  => 'minute',
    array  => 'month',
    array  => 'dayofweek',
    scalar => 'name',
    array  => 'day',
    scalar => 'description',
];

=head1 METHODS

=head2 fetch

    my $JobScheduleCron_state   = NACL::CS::JobScheduleCron->fetch(command_interface => $ci, ...);
	my @JobScheduleCron_states = NACL::CS::JobScheduleCron->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "job-schedule-cron-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
    show_cmd       => 'job schedule cron show',
    exception_text => 'No matching job schedule cron(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'job_schedule_cron_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg    = shift;
    my @states = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api => "job_schedule_cron_get_iter",
        map => {
            "name"  => 'job-schedule-name',
            "month" => [
                make_zapi_array('job-schedule-cron-month'),
                make_zapi_skip('cron-month')
            ],
            "dayofweek" => [
                make_zapi_array('job-schedule-cron-day-of-week'),
                make_zapi_skip('cron-day-of-week')
            ],
            "day" => [
                make_zapi_array('job-schedule-cron-day'),
                make_zapi_skip('cron-day'),
            ],
            "hour" => [
                make_zapi_array('job-schedule-cron-hour'),
                make_zapi_skip('cron-hour'),
            ],
            "minute" => [
                make_zapi_array('job-schedule-cron-minute'),
                make_zapi_skip('cron-minute'),
            ],
            "description" => 'job-schedule-description',
        },
    );
    $Log->exit() if $may_exit;
    return @states;
} ## end sub _fetch_cmode_zapi

1;
