#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary LunInitiatorListMap ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::LunInitiatorListMap

=head1 DESCRIPTION

C<NACL::CS::LunInitiatorListMap> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP LunInitiatorListMap.  A related class is
L<NACL::C::LunInitiatorListMap|lib-NACL-C-LunInitiatorListMap-pm>, which represents access
to an ONTAP LunInitiatorListMap.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the LunInitiatorListMap
element are the attributes of the LunInitiatorListMap ComponentState.

=over

=item C<< "igroup" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <lun-maps>
     <initiator-group>$value</initiator-group>
 </lun-maps>

=item C<< "lun" >>

Filled in for CMode CLI.

=item C<< "initiator" >>

Filled in for CMode CLI.

=item C<< "volume" >>

Filled in for CMode CLI.

=item C<< "qtree" >>

Filled in for CMode CLI.

=item C<< "vserver" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <lun-maps>
     <vserver>$value</vserver>
 </lun-maps>

=item C<< "path" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <lun-maps>
     <path$value</path>
 </lun-maps>

=item C<< "lun_id" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <lun-maps>
     <lun-id>$value</lun-id>
 </lun-maps>

=item C<< lun_uuid >>

Filled in for CMode CLI.

=item C<< igroup_uuid >>

Filled in for CMode CLI.

=back

=cut

package NACL::CS::LunInitiatorListMap;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one Dumper );
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::Exceptions::NoElementsFound qw(:try);
use NATE::BaseException qw(:try);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'igroup',
    scalar => 'lun',
    scalar => 'initiator',
    scalar => 'volume',
    scalar => 'qtree',
    scalar => 'vserver',
    scalar => 'path',
    scalar => 'lun_id',
    scalar => 'lun_uuid',
    scalar => 'igroup_uuid',
];

=head1 METHODS

=head2 fetch

    my $LunInitiatorListMap_state   = NACL::CS::LunInitiatorListMap->fetch(command_interface => $ci, ...);
	my @LunInitiatorListMap_states = NACL::CS::LunInitiatorListMap->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "lun-initiator-list-map-info" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching lun initiator list map(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
        @_,
        api => 'lun_initiatorListMap_show',

    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg    = shift;
    my %opts   = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset = $opts{apiset};
    my $filter = $opts{filter};
    my $lun_response;
    my %lun_list_info_args;
    $pkg->_hash_copy(
        source => $opts{filter},
        target => \%lun_list_info_args,
        copy   => [qw(initiator)],
    );

    if (!defined($lun_list_info_args{'initiator'})) {
        NATE::BaseException->throw("initiator is the reqired feild for Zapi");
    }

    # find all the lun initiators present.
    $lun_response = $apiset->lun_initiator_list_map_info(%lun_list_info_args);

    my $luninitiator_output = $lun_response->get_parsed_output();
    my @state_objs;
    foreach
        my $row (@{$luninitiator_output->[0]{'lun-maps'}[0]{'lun-map-info'}})
    {
        my $state_fields = $pkg->_zapi_hash_copy(
            source => $row,
            copy   => [qw(vserver lun-id path)],
            map    => ["initiator-group" => "igroup",],
        );
        my $obj = $pkg->new(command_interface => $opts{command_interface});
        $obj->_set_fields(row => $state_fields);
        $obj->initiator($lun_list_info_args{initiator});
        push @state_objs, $obj;
    } ## end foreach my $row ( @{ $luninitiator_output...

    $Log->exit() if $may_exit;
    return @state_objs;

} ## end sub _fetch_cmode_zapi

1;
