#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkPortVlan ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::NetworkPortVlan

=head1 DESCRIPTION

C<NACL::CS::NetworkPortVlan> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP NetworkPortVlan.  A related class is
L<NACL::C::NetworkPortVlan|lib-NACL-C-NetworkPortVlan-pm>, which represents access
to an ONTAP NetworkPortVlan.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the NetworkPortVlan
element are the attributes of the NetworkPortVlan ComponentState.

=over

=item C<< "gvrp" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <vlan-info>
     <gvrp-enabled>$value</gvrp-enabled>
 </vlan-info>

=item C<< "vlan_id" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <vlan-info>
     <vlanid>$value</vlanid>
 </vlan-info>

7Mode CLI:

For "filter": vlanid list (Programmatic name:vlanid-list)

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "port" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <vlan-info>
     <parent-interface>$value</parent-interface>
 </vlan-info>"

For "filter": ifname (Programmatic name:ifname)

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "node" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <vlan-info>
     <node>$value</node>
 </vlan-info>

=item C<< "mac" >>

Filled in for CMode CLI, 7Mode CLI.

Maps to:

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "vlan_name" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <vlan-info>
     <interface-name>$value</interface-name>
 </vlan-info>

=item C<< "receive_broadcast" >>

multi/broadcast

Filled in for 7Mode CLI.

Maps to:

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "receive_total_frames" >>

Filled in for 7Mode CLI.

Maps to:

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "receive_total_bytes" >>

Filled in for 7Mode CLI.

Maps to:

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "receive_vlan_tag_drops" >>

Filled in for 7Mode CLI.

Maps to:

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "receive_untag_drops" >>

Filled in for 7Mode CLI.

Maps to:

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "transmit_broadcast" >>

Filled in for 7Mode CLI.

Maps to:

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "transmit_total_frames" >>

Filled in for 7Mode CLI.

Maps to:

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "transmit_total_bytes" >>

Filled in for 7Mode CLI.

Maps to:

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "transmit_queue_overflows" >>

Filled in for 7Mode CLI.

Maps to:

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=back

=cut

package NACL::CS::NetworkPortVlan;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one Dumper);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'gvrp',
    scalar => 'vlan_id',
    scalar => 'port',
    scalar => 'node',
    scalar => 'mac',
    scalar => 'vlan_name',

    # 7 Mode

    scalar => 'receive_broadcast',
    scalar => 'receive_total_frames',
    scalar => 'receive_total_bytes',
    scalar => 'receive_vlan_tag_drops',
    scalar => 'receive_untag_drops',
    scalar => 'transmit_broadcast',
    scalar => 'transmit_total_frames',
    scalar => 'transmit_total_bytes',
    scalar => 'transmit_queue_overflows',

];

=head1 METHODS

=head2 fetch

    my $NetworkPortVlan_state   = NACL::CS::NetworkPortVlan->fetch(command_interface => $ci, ...);
    my @NetworkPortVlan_states = NACL::CS::NetworkPortVlan->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI, 7Mode CLI.

Invokes "net-vlan-get-iter" API for CMode ZAPI.

Invokes "vlan stat" command for 7Mode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching network port vlan(s) found',
        show_cmd       => 'network port vlan show',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'network_port_vlan_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_7mode_cli {

    my $pkg         = shift;
    my %opts        = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset      = delete $opts{apiset};
    my $filter_opts = delete $opts{filter};
    my %options_args;
    $pkg->_hash_copy(
        source => $filter_opts,
        map    => {
            'vlan-id' => 'vlanid-list',
            'port'    => 'ifname',
        },
        target => \%options_args,
    );
    my @state_objs = ();
    my $output;
    my $flag = 0;
    try {
        my $response = $apiset->vlan_stat(%options_args);
        $output = $response->get_parsed_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $e = shift;
        if ($e->text() =~ /vlan stat.*failed/i) {
            $flag = 1;
        } else {
            $e->throw();
        }
    };
    return @state_objs if ($flag);
    foreach my $vlan_output (@{$output}) {
        my $obj = $pkg->new();
        my $port;
        my $target;
        if (defined($vlan_output->{'vlan_id'})) {
            $obj->vlan_id($vlan_output->{'vlan_id'});

            # api output has  triling space. hence removing it.
            my $interface = $vlan_output->{'name'};
            $interface =~ s/\s+$//;
            $obj->vlan_name($interface);
            $interface =~ /^(.*)-/;
            $port = $1;
        } else {
            $port = $vlan_output->{'name'};
        }

        $obj->port($port);
        $target = $pkg->_zapi_hash_copy(
            source                  => $vlan_output,
            source_has_extra_arrays => 1,
            copy                    => [qw(gvrp )],
            map                     => [
                'mac_address' => 'mac',
                ['receive_statistics', 'multi/broadcast'] =>
                    'receive_broadcast',
                ['receive_statistics', 'total_frames'] =>
                    'receive_total_frames',
                ['receive_statistics', 'total_bytes'] =>
                    'receive_total_bytes',
                ['receive_statistics', 'vlan_tag_drops'] =>
                    'receive_vlan_tag_drops',
                ['receive_statistics', 'untag_drops'] =>
                    'receive_untag_drops',
                ['transmit_statistics', 'multi/broadcast'] =>
                    'transmit_broadcast',
                ['transmit_statistics', 'total_frames'] =>
                    'transmit_total_frames',
                ['transmit_statistics', 'total_bytes'] =>
                    'transmit_total_bytes',
                ['transmit_statistics', 'queue_overflows'] =>
                    'transmit_queue_overflows',
            ],
        );

        $obj->_set_fields(row => $target);
        if (defined($obj->vlan_name())) {
            push @state_objs, $obj;
        }
    } ## end foreach my $vlan_output ( @...
    return @state_objs;
} ## end sub _fetch_7mode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'net_vlan_get_iter',
        copy => [qw(node)],
        map  => {
            "vlan-name" => ['interface-name'],
            "port"      => ['parent-interface'],
            "vlan-id"   => ['vlanid'],
            "gvrp"      => ['gvrp-enabled'],
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
