#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary QosPolicy ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::QosPolicy

=head1 DESCRIPTION

C<NACL::CS::QosPolicy> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP QosPolicy.  A related class is
L<NACL::C::QosPolicy|lib-NACL-C-QosPolicy-pm>, which represents access
to an ONTAP QosPolicy.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the QosPolicy
element are the attributes of the QosPolicy ComponentState.

=over

=item C<< "uuid" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields","filter" and Output mapping:

 <qos-policy-info>
    <policy-uuid>$value</policy-uuid>
 </qos-policy-info>

=item C<< "read_ahead" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields","filter" and Output mapping:

 <qos-policy-info>
    <read-ahead-policy-name>$value</read-ahead-policy-name>
 </qos-policy-info>

=item C<< "max_tput" >>

Filled in for CMode CLI.

=item C<< "target_tput_burst" >>

Filled in for CMode CLI.

=item C<< "max_latency" >>

Filled in for CMode CLI.

=item C<< "target_tput" >>

Filled in for CMode CLI.

=item C<< "policy" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields","filter" and Output mapping:

 <qos-policy-info>
    <policy-name>$value</policy-name>
 </qos-policy-info>

=item C<< "max_tput_burst" >>

Filled in for CMode CLI.

=item C<< "class" >>

Filled in for CMode CLI.

=item C<< "cache" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields","filter" and Output mapping:

 <qos-policy-info>
    <cache-policy-name>$value</cache-policy-name>
 </qos-policy-info>

=item C<< cache_policy >>

Filled in for CMode CLI.

=item C<< read_ahead_policy >>

Filled in for CMode CLI.


=back

=cut

package NACL::CS::QosPolicy;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'uuid',
    scalar => 'read_ahead',
    scalar => 'max_tput',
    scalar => 'target_tput_burst',
    scalar => 'max_latency',
    scalar => 'target_tput',
    scalar => 'policy',
    scalar => 'max_tput_burst',
    scalar => 'class',
    scalar => 'cache',
    scalar => 'cache_policy',
    scalar => 'read_ahead_policy',
];

=head1 METHODS

=head2 fetch

    my $QosPolicy_state   = NACL::CS::QosPolicy->fetch(command_interface => $ci, ...);
	my @QosPolicy_states = NACL::CS::QosPolicy->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI APIset.

Invokes "qos-policy-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching qos policy(s) found',
        show_cmd       => 'qos policy show',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'qos_policy_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api => 'qos_policy_get_iter',
        map => {
            "policy"     => ['policy-name'],
            "read-ahead" => ['read-ahead-policy-name'],
            "uuid"       => ['policy-uuid'],
            "cache"      => ['cache-policy-name'],
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
