#
#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary ShowBoot ComponentState Module
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here
################################################################################

=head1 NAME

NACL::CS::Switch::ShowBoot

=head1 DESCRIPTION

C<NACL::CS::Switch::ShowBoot> is a derived class of
L<NACL::CS::ComponentState::Switch|lib-NACL-CS-ComponentState-Switch-pm>.  It represents the
state of ShowBoot element.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the ShowBoot element
are the individual attributes of the ShowBoot ComponentState

=over

# Following are Only Supported for Nexus

=item C<< 'current_boot_kickstart_variable' >>

=item C<< 'current_boot_system_variable' >>

=item C<< 'system_variable_on_next_reload' >>

=item C<< 'kickstart_variable_on_next_reload' >>

=item C<< 'list_of_boot_variables_are' >>

# Following are Only Supported for IOS

=item C<< 'private_config_file' >>

=item C<< 'auto_upgrade_path' >>

=item C<< 'enable_break' >>

=item C<< 'auto_upgrade' >>

=item C<< 'config_file' >>

=item C<< 'boot_path_list' >>

=item C<< 'manual_boot' >>

=item C<< 'helper_path_list' >>

=item C<< 'download' >>

=item C<< 'buffer_size' >>

=item C<< 'next_boot' >>

=item C<< 'via_dhcp' >>

# Following are Only Supported for NetAppNetwork

=item C<< 'active' >>

=item C<< 'unit' >>

=item C<< 'next_active' >>

=item C<< 'current_active' >>

=item C<< 'backup' >>

=item C<< 'active_image' >>

=item C<< 'backup_image' >>

=item C<< 'flash_image_active' >>

=item C<< 'flash_image_backup' >>

=back

=cut

###################
# Package
package NACL::CS::Switch::ShowBoot;

###################
# Everytime
use strict;
use warnings;

###################
# Module includes
use Params::Validate qw (validate);
use NACL::ComponentUtils qw (_dump_one Dumper);

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::Switch';

use Class::MethodMaker [

    # Only Supported for Nexus
    array  => "list_of_boot_variables_are",
    scalar => "current_boot_kickstart_variable",
    scalar => "current_boot_system_variable",
    scalar => "system_variable_on_next_reload",
    scalar => "kickstart_variable_on_next_reload",

    # Only Supported for IOS
    scalar => "private_config_file",
    scalar => "auto_upgrade_path",
    scalar => "enable_break",
    scalar => "auto_upgrade",
    scalar => "config_file",
    scalar => "boot_path_list",
    scalar => "manual_boot",
    scalar => "helper_path_list",
    scalar => "download",
    scalar => "buffer_size",
    scalar => "next_boot",
    scalar => "via_dhcp",

    #Only supported for NetAppNetwork
    scalar => "unit",
    scalar => "next_active",
    scalar => "current_active",
    scalar => "active_image",
    scalar => "backup_image",
    scalar => "flash_image_active",
    scalar => "flash_image_backup",
];

=head1 METHODS

=head2 fetch

    my $showboot_state  = NACL::CS::Switch::ShowBoot->fetch(command_interface=>$ci,...);
    my @showboot_states = NACL::CS::Switch::ShowBoot->fetch(command_interface=>$ci,...);

see L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm>

=cut

sub fetch {
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {
                method    => "_fetch_cisconetwork_cli",
                interface => "CLI",
                set       => "CiscoNetwork"
            },
            {
                method    => "_fetch_netappnetwork_cli",
                interface => "CLI",
                set       => "NetAppNetwork"
            },
        ],
        exception_text => 'No matching elements found'
    );
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cisconetwork_cli {
    my $pkg    = shift;
    my %opts   = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset = $opts{apiset};

    $pkg->_remove_relational_regex_filters(
        filter           => $opts{filter},
        requested_fields => $opts{requested_fields}
    );

    my $requested_fields = $opts{requested_fields};
    my $filter           = $opts{filter};
    my %req_field_filter =
      ( requested_fields => $requested_fields, filter => $filter );

    # define variables
    my ( $output, @state_objs, $response, $obj, $row );
	
    # get method name based on Switch firmware type
    my $hosttype = $opts{command_interface}->hostrec()->hosttype();

    $obj = $pkg->new( command_interface => $opts{command_interface} );
    $response = $apiset->show_boot();
    $output   = $response->get_parsed_output();
     foreach my $row ( @{$output} ) {
        foreach my $var (@{$row->{boot_variables_on_next_reload}}) {
            $row->{kickstart_variable_on_next_reload} = $var->{kickstart_variable};
            $row->{system_variable_on_next_reload}    = $var->{system_variable};
        }
        delete $row->{boot_variables_on_next_reload};

        foreach my $var (@{$row->{current_boot_variables}}) {
            $row->{current_boot_kickstart_variable} =  $var->{kickstart_variable};
            $row->{current_boot_system_variable} =  $var->{system_variable};
        }
        delete $row->{current_boot_variables};

        foreach my $var (@{$row->{timeout_for_config}}) {
            $row->{download} = $var->{download};
        }
        delete $row->{timeout_for_config};

        foreach my $var (@{$row->{nvram_config_file}}) {
            $row->{buffer_size} = $var->{buffer_size};
        }
        delete $row->{nvram_config_file};

        foreach my $var (@{$row->{config_download}}) {
            $row->{via_dhcp} = $var->{via_dhcp};
            $row->{next_boot} = $var->{next_boot};
        }
        delete $row->{config_download};

        $obj->_set_fields( row => $row );
        push @state_objs, $obj;
    }

    if ( $hosttype =~ /nxos/i ) {
        my @fields_in_variables = qw(list_of_boot_variables_are);
        if (
            $pkg->_want_any_field_of(
                %req_field_filter,
                fields_filled_by_api => \@fields_in_variables,
            )
          ) {
            $response = $apiset->show_boot( "variables" => 1 );
            $output = $response->get_parsed_output();
            foreach my $row ( @{$output} ) {
                $obj->_set_fields( row => $row );
                push @state_objs, $obj;
            }
        }
    }        

    return @state_objs;
} ## end sub _fetch_cisconetwork_cli

sub _fetch_netappnetwork_cli {
    my $pkg    = shift;
    my %opts   = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset = $opts{apiset};

    $pkg->_remove_relational_regex_filters(
        filter           => $opts{filter},
        requested_fields => $opts{requested_fields}
    );
    my $requested_fields = $opts{requested_fields};
    my $filter           = $opts{filter};

    # define variables
    my ( $output, @state_objs, $response, $row, $obj );

    $response = $apiset->show_bootvar();
    $output   = $response->get_parsed_output();
    foreach my $row ( @{$output} ) {
        foreach my $var (@{$row->{image_descriptions}}) {
            $row->{active_image}    = $var->{active};
            $row->{backup_image}    = $var->{backup};
        }
        delete $row->{image_descriptions};

        foreach my $var (@{$row->{images_currently_available_on_flash}}) {
            $row->{flash_image_active} =  $var->{active};
            $row->{flash_image_backup} =  $var->{backup};
            $row->{unit}               =  $var->{unit};
            $row->{next_active}        =  $var->{next_active};
            $row->{current_active}     =  $var->{current_active};
        }
        delete $row->{images_currently_available_on_flash};

        $obj = $pkg->new( command_interface => $opts{command_interface} );
        $obj->_set_fields( row => $row );
        push @state_objs, $obj;
    }
    return @state_objs;
} ## end sub _fetch_netappnetwork_cli

1;
