# $Id$
#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Fcp ComponentState Module
## @author Neethu.George@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SysManager::Fcp

=head1 DESCRIPTION

C<NACL::CS::SysManager::Fcp> is a derived class of
L<NACL::CS::ComponentState::SysManager|lib-NACL-CS-ComponentState-SysManager-pm>. It represents the
state of an ONTAP Fcp.  A related class is
L<NACL::C::SysManager::Fcp|lib-NACL-C-SysManager-Fcp-pm>
It gives information about the fcp configuration.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the Fcp
element are the attributes of the fcp ComponentState.

=over

=item C<< vserver >>

=item C<< "fc_status" >>

=item C<< "wwnn_number" >>

=item C<< "fc_interface" >>

=item C<< "interface_name" >>

=item C<< "interface_wwpn" >>

=item C<< "interface_port" >>

=item C<< "interface_status" >>

=back

=cut

package NACL::CS::SysManager::Fcp;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use base 'NACL::CS::ComponentState::SysManager';

use NACL::C::SysManager::Fcp;
use NACL::Exceptions::NoElementsFound qw(:try);

# Import Exception class
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'fc_status',
    scalar => 'wwnn_number',
    scalar => 'fc_interface',
    scalar => 'interface_name',
    scalar => 'interface_wwpn',
    scalar => 'interface_port',
    scalar => 'interface_status',
    scalar => 'fcpname',
];

=head1 METHODS

=head2 fetch

    my $Fcp_state   = 
		NACL::CS::SysManager::Fcp->fetch(command_interface => $ci, ...);
	my @Fcp_states  = 
		NACL::CS::SysManager::Fcp->fetch(command_interface => $ci, ...);

(Class method)
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch>.

Uses a CMode GUI or a 7Mode GUI APISet.
(Works only when fcpname and vserver parameters are passed under filter option).

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method => "_fetch_cmode_gui",
                set    => "SysManager_CMode",
            },
            {   method => '_fetch_7mode_gui',
                set    => 'SysManager_7Mode',
            },
        ],
        exception_text => 'No matching fcp found',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_gui {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();

    my $apiset            = $opts{apiset};
    my $requested_fields  = $opts{requested_fields};
    my $filter            = $opts{filter};
    my $command_interface = $opts{command_interface};

    my $fcp_args = $pkg->_hash_copy(
        source => $filter,
        map    => {
            "vserver" => "-vservername",
            "fcpname" => "-fc_interface",
        }
    );

    if (   !defined $fcp_args->{-fc_interface}
        || !defined $fcp_args->{-vservername} )
    {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "'fcpname', and 'vserver' need to be provided in the filter for $pkg->fetch to work"
        );
    } ## end if ( !defined $fcp_args...)

    my ( $response, $caught_exception, $parsed_output, $response_interface,
        $parsed_output_interface );
    try {

        # caling the gui api to get fcp status
        $response           = $apiset->fcp_info(%$fcp_args);
        $parsed_output      = $response->get_parsed_output();
        $response_interface = $apiset->get_fcp_interface_info(%$fcp_args);
        $parsed_output_interface = $response_interface->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {

        # A caught exception indicates that the FCP being looked for
        # does not exist. We catch the exception and return immediately.
        # The 'fetch' frontend decides whether to throw a NoElementsFound
        # exception based on the value of 'allow_empty'
        $Log->debug( 'CommandFailedException caught. Requested FCP does '
                . 'not exist' );
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my %state_base_field_settings;

    $state_base_field_settings{'vserver'} = $fcp_args->{'-vservername'};
    $state_base_field_settings{'fcpname'} = $fcp_args->{'-fc_interface'};

    my %final_attributes = (
        %state_base_field_settings, %$parsed_output, %$parsed_output_interface
    );

    my $obj = $pkg->new( command_interface => $command_interface );
    $obj->_set_fields( row => \%final_attributes );
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _fetch_cmode_gui

sub _fetch_7mode_gui {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();

    my $apiset            = $opts{apiset};
    my $requested_fields  = $opts{requested_fields};
    my $filter            = $opts{filter};
    my $command_interface = $opts{command_interface};

    my $fcp_args = $pkg->_hash_copy(
        source => $filter,
        map    => { "fcpname" => "-fc_interface", }
    );

    if ( !defined $fcp_args->{-fc_interface} ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "'fcpname' need to be provided in the filter for $pkg->fetch to work"
        );
    } ## end if ( !defined $fcp_args...)

    my ( $response, $caught_exception, $parsed_output );
    try {

        # caling the gui api to get fcp status
        $response      = $apiset->fcp_info(%$fcp_args);
        $parsed_output = $response->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {

        # A caught exception indicates that the FCP being looked for
        # does not exist. We catch the exception and return immediately.
        # The 'fetch' frontend decides whether to throw a NoElementsFound
        # exception based on the value of 'allow_empty'
        $Log->debug( 'CommandFailedException caught. Requested FCP does '
                . 'not exist' );
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my %state_base_field_settings;

    $state_base_field_settings{'fcpname'} = $fcp_args->{'-fc_interface'};

    my %final_attributes = ( %state_base_field_settings, %$parsed_output );
    my $obj = $pkg->new( command_interface => $command_interface );
    $obj->_set_fields( row => \%final_attributes );
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _fetch_7mode_gui

1;
