# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary IscsiInitiatorSecurity ComponentState Module
## @author vikas.p@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SysManager::IscsiInitiatorSecurity

=head1 DESCRIPTION

C<NACL::CS::SysManager::IscsiInitiatorSecurity> is a derived class of
L<NACL::CS::ComponentState::SysManager|lib-NACL-CS-ComponentState-SysManager-pm>.  It represents the  
state of an ONTAP IscsiInitiatorSecurity.  A related class is
L<NACL::C::SysManager::IscsiInitiatorSecurity|lib-NACL-C-SysManager-IscsiInitiatorSecurity-pm>, 
which represents access to an ONTAP IscsiInitiatorSecurity.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the IscsiInitiatorSecurity 
element are the attributes of the Initiator Security ComponentState.

=over

=item C<< initiator >>

The name of the initiator security element whose state is being represented.

=item C<< vserver >>

=item C<< securitytype >>

=item C<< inbounduname >>

=item C<< outbounduname >>

=item C<< initiator >>

=item C<< initiator_name >>

=item C<< security_type >>

=item C<< group >>

=item C<< inbound_user_name >>

=item C<< outbound_user_name >>

=back

=cut

package NACL::CS::SysManager::IscsiInitiatorSecurity;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(validate);

use base 'NACL::CS::ComponentState::SysManager';

use NACL::C::SysManager::IscsiInitiatorSecurity;
use NACL::Exceptions::NoElementsFound qw(:try);

# Import Exception class
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'securitytype',
    scalar => 'inbounduname',
    scalar => 'outbounduname',
    scalar => 'initiator',
    scalar => 'initiator_name',
    scalar => 'security_type',
    scalar => 'group',
    scalar => 'inbound_user_name',
    scalar => 'outbound_user_name',
];

=head1 METHODS 

=head2 fetch

    my $iscsi_initiator_security_state  = 
		NACL::CS::SysManager::IscsiInitiatorSecurity->fetch(command_interface => $ci,...);
    my @iscsi_initiator_security_states = 
		NACL::CS::SysManager::IscsiInitiatorSecurity->fetch(command_interface => $ci,...);

see L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm>.

Uses a CMode GUI or a 7Mode GUI APISet.
(Works only when initiator and vserver parameters are passed under filter option).

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method => "_fetch_cmode_gui",
                set    => "SysManager_CMode",
            },
            {   method => '_fetch_7mode_gui',
                set    => 'SysManager_7Mode',
            },
        ],
        exception_text => 'No matching initiator found',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();

    my $apiset            = $opts{apiset};
    my $requested_fields  = $opts{requested_fields};
    my $filter            = $opts{filter};
    my $command_interface = $opts{command_interface};

    my $initiator_args = $pkg->_hash_copy(
        source => $filter,
        map    => {
            "initiator" => "-initiator",
            "vserver"   => "-vservername",
        }
    );

    $pkg->_check_for_required_filter_fields(
        filter          => $filter,
        required_fields => [qw(initiator vserver)],
    );

    my ( $response, $caught_exception, $parsed_output, $response_security,
        $parsed_ouput_security );
    try {

        # caling the gui api to get initiator status
        $response
            = $apiset->get_initiator_default_security_info(%$initiator_args);
        $parsed_output = $response->get_parsed_output();
        $response_security
            = $apiset->get_initiator_security_info(%$initiator_args);
        $parsed_ouput_security = $response->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {

        # A caught exception indicates that the initiator being looked for
        # does not exist. We catch the exception and return immediately.
        # The 'fetch' frontend decides whether to throw a NoElementsFound
        # exception based on the value of 'allow_empty'
        $Log->debug(
                  'CommandFailedException caught. Requested initiator does '
                . 'not exist' );
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my %state_base_field_settings;

    $state_base_field_settings{'initiator'} = $initiator_args->{'-initiator'};
    $state_base_field_settings{'vserver'} = $initiator_args->{'-vservername'};

    my %final_attributes = (
        %state_base_field_settings, %$parsed_output, %$parsed_ouput_security
    );

    my $obj = $pkg->new( command_interface => $command_interface );
    $obj->_set_fields( row => \%final_attributes );
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _fetch_cmode_gui

####################### GUI 7Mode Fetch ###############################
sub _fetch_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();

    my $apiset            = $opts{apiset};
    my $requested_fields  = $opts{requested_fields};
    my $filter            = $opts{filter};
    my $command_interface = $opts{command_interface};

    my $initiator_args = $pkg->_hash_copy(
        source => $filter,
        map    => { "initiator" => "-initiator", }
    );

    $pkg->_check_for_required_filter_fields(
        filter          => $filter,
        required_fields => [qw(initiator)],
    );

    my ( $response, $caught_exception, $parsed_output, $response_security,
        $parsed_ouput_security );
    try {

        # caling the gui api to get initiator status
        $response
            = $apiset->get_initiator_default_security_info(%$initiator_args);
        $parsed_output = $response->get_parsed_output();
        $response_security
            = $apiset->get_initiator_security_info(%$initiator_args);
        $parsed_ouput_security = $response->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {

        # A caught exception indicates that the initiator being looked for
        # does not exist. We catch the exception and return immediately.
        # The 'fetch' frontend decides whether to throw a NoElementsFound
        # exception based on the value of 'allow_empty'
        $Log->debug(
                  'CommandFailedException caught. Requested initiator does '
                . 'not exist' );
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my %state_base_field_settings;

    $state_base_field_settings{'initiator'} = $initiator_args->{'-initiator'};

    my %final_attributes = (
        %state_base_field_settings, %$parsed_output, %$parsed_ouput_security
    );
    my $obj = $pkg->new( command_interface => $command_interface );
    $obj->_set_fields( row => \%final_attributes );
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _fetch_7mode_gui

1;
