# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary NetworkInterface ComponentState Module
## @author Neethu.George@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SysManager::NetworkInterface

=head1 DESCRIPTION

C<NACL::CS::SysManager::NetworkInterface> is a derived class of
L<NACL::CS::ComponentState::SysManager|lib-NACL-CS-ComponentState-SysManager-pm>.  It represents the
state of an ONTAP NetworkInterface present on the System Manger GUI Screen.  A related class is
L<NACL::C::SysManager::NetworkInterface|lib-NACL-C-SysManager-NetworkInterface-pm>, which represents access
to an ONTAP NetworkInterface present on the System Manger GUI Screen.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the NetworkInterface 
element are the attributes of the NetworkInterface ComponentState.

=over

=item C<< interfacename >>

The name of the NetworkInterface element whose state is being represented.

=item C<< wwpn >>

=item C<< netmask >>

=item C<< gateway >>

=item C<< managementaccess >>

=item C<< currentport >>

=item C<< homeport >>

=item C<< failover >>

=item C<< failovergroup >>

=item C<< failoverstate >>

=item C<< status >>

=item C<< vserver >>

=back

=cut

package NACL::CS::SysManager::NetworkInterface;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(validate);

use base 'NACL::CS::ComponentState::SysManager';

use NACL::C::SysManager::NetworkInterface;
use NACL::Exceptions::NoElementsFound qw(:try);

# Import Exception class
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;

use Class::MethodMaker [
    scalar => 'interfacename',
    scalar => 'wwpn',
    scalar => 'netmask',
    scalar => 'gateway',
    scalar => 'managementaccess',
    scalar => 'currentport',
    scalar => 'homeport',
    scalar => 'failover',
    scalar => 'failovergroup',
    scalar => 'failoverstate',
    scalar => 'status',
    scalar => 'vserver',
];

=head1 METHODS 

=head2 fetch

    my $NetworkInterface_state  = 
		NACL::CS::SysManager::NetworkInterface->fetch(command_interface => $ci,...);
    my @NetworkInterface_states = 
		NACL::CS::SysManager::NetworkInterface->fetch(command_interface => $ci,...);

see L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm>.

Uses a CMode GUI APISet.
(Works only when interfacename and vserver parameters are passed under filter option).

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method => "_fetch_cmode_gui",
                set    => "SysManager_CMode",
            },
        ],
        exception_text => 'No matching interface found',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();

    my $apiset            = $opts{apiset};
    my $requested_fields  = $opts{requested_fields};
    my $filter            = $opts{filter};
    my $command_interface = $opts{command_interface};

    my $networkInterface_args = $pkg->_hash_copy(
        source => $filter,
        map    => {
            "interfacename" => "-interfacename",
            "vserver"       => "-vservername",
        }
    );

    $pkg->_check_for_required_filter_fields(
        filter          => $filter,
        required_fields => [qw(interfacename vserver)],
    );

    my ( $response, $caught_exception, $parsed_output );
    try {

        # caling the gui api to get network interface status
        $response
            = $apiset->get_network_interface_info(%$networkInterface_args);
        $parsed_output = $response->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {

    # A caught exception indicates that the network interface being looked for
    # does not exist. We catch the exception and return immediately.
    # The 'fetch' frontend decides whether to throw a NoElementsFound
    # exception based on the value of 'allow_empty'
        $Log->debug(
                  'CommandFailedException caught. Requested interface does '
                . 'not exist' );
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my %state_base_field_settings;

    $state_base_field_settings{'interfacename'}
        = $networkInterface_args->{'-interfacename'};
    $state_base_field_settings{'vserver'}
        = $networkInterface_args->{'-vservername'};

    my %final_attributes = ( %state_base_field_settings, %$parsed_output );

    my $obj = $pkg->new( command_interface => $opts{command_interface} );
    $obj->_set_fields( row => \%final_attributes );
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _fetch_cmode_gui

1;
