# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary Qtree ComponentState Module
## @author vikas.p@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SysManager::Qtree

=head1 DESCRIPTION

C<NACL::CS::SysManager::Qtree> is a derived class of
L<NACL::CS::ComponentState::SysManager|lib-NACL-CS-ComponentState-SysManager-pm>.  It represents the  
state of an ONTAP Qtree.  A related class is
L<NACL::C::SysManager::Qtree|lib-NACL-C-SysManager-Qtree-pm>, which represents access
to an ONTAP Qtree.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the Qtree 
element are the attributes of the qtree ComponentState.

=over

=item C<< volume >>

The name of the volume in which the qtree exists.

=item C<< qtree >>

The name of the Qtree element whose state is being representend.

=item C<< vserver >>

=item C<< qtree_name >>

=item C<< qtree_volname >>

=item C<< qtree_secstyle >>

=item C<< create_uni >>

=item C<< qtree_status >>

=item C<< qtree_oplocks >>

=back

=cut

package NACL::CS::SysManager::Qtree;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(validate);

use base 'NACL::CS::ComponentState::SysManager';

use NACL::C::SysManager::Qtree;
use NACL::Exceptions::NoElementsFound qw(:try);

# Import Exception class
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'volume',
    scalar => 'qtree',
    scalar => 'qtree_name',
    scalar => 'qtree_volname',
    scalar => 'qtree_secstyle',
    scalar => 'qtree_status',
    scalar => 'qtree_oplocks',
];

=head1 METHODS 

=head2 fetch

    my $qtree_state  = 
		NACL::CS::SysManager::Qtree->fetch(command_interface => $ci,...);
    my @qtree_states = 
		NACL::CS::SysManager::Qtree->fetch(command_interface => $ci,...);

see L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm>.

Uses a CMode GUI or a 7Mode GUI APISet.
(Works only when volume, qtree and vserver parameters are passed under filter option).

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method => "_fetch_cmode_gui",
                set    => "SysManager_CMode",
            },
            {   method => '_fetch_7mode_gui',
                set    => 'SysManager_7Mode',
            },
        ],
        exception_text => 'No matching qtree found',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();

    my $apiset            = $opts{apiset};
    my $requested_fields  = $opts{requested_fields};
    my $filter            = $opts{filter};
    my $command_interface = $opts{command_interface};

    my $qtree_args = $pkg->_hash_copy(
        source => $filter,
        map    => {
            "volume"  => "-volname",
            "qtree"   => "-qtreename",
            "vserver" => "-vservername",
        }
    );

    $pkg->_check_for_required_filter_fields(
        filter          => $filter,
        required_fields => [qw(volume qtree vserver)],
    );

    my ( $response, $caught_exception, $parsed_output );
    try {

        # caling the gui api to get qtree status
        $response      = $apiset->get_qtree_info(%$qtree_args);
        $parsed_output = $response->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {

        # A caught exception indicates that the qtree being looked for
        # does not exist. We catch the exception and return immediately.
        # The 'fetch' frontend decides whether to throw a NoElementsFound
        # exception based on the value of 'allow_empty'
        $Log->debug( 'CommandFailedException caught. Requested qtree does '
                . 'not exist' );
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my %state_base_field_settings;

    $state_base_field_settings{'volume'}  = $qtree_args->{'-volname'};
    $state_base_field_settings{'qtree'}   = $qtree_args->{'-qtreename'};
    $state_base_field_settings{'vserver'} = $qtree_args->{'-vservername'};

    my %final_attributes = ( %state_base_field_settings, %$parsed_output );

    my $obj = $pkg->new( command_interface => $command_interface );
    $obj->_set_fields( row => \%final_attributes );
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _fetch_cmode_gui

####################### GUI 7Mode Fetch ###############################
sub _fetch_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();

    my $apiset            = $opts{apiset};
    my $requested_fields  = $opts{requested_fields};
    my $filter            = $opts{filter};
    my $command_interface = $opts{command_interface};

    my $qtree_args = $pkg->_hash_copy(
        source => $filter,
        map    => {
            "volume" => "-volname",
            "qtree"  => "-qtreename",
        }
    );

    $pkg->_check_for_required_filter_fields(
        filter          => $filter,
        required_fields => [qw(volume qtree)],
    );

    my ( $response, $caught_exception, $parsed_output );
    try {

        # caling the gui api to get qtree status
        $response      = $apiset->get_qtree_info(%$qtree_args);
        $parsed_output = $response->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {

        # A caught exception indicates that the qtree being looked for
        # does not exist. We catch the exception and return immediately.
        # The 'fetch' frontend decides whether to throw a NoElementsFound
        # exception based on the value of 'allow_empty'
        $Log->debug( 'CommandFailedException caught. Requested qtree does '
                . 'not exist' );
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my %state_base_field_settings;

    $state_base_field_settings{'volume'} = $qtree_args->{'-volname'};
    $state_base_field_settings{'qtree'}  = $qtree_args->{'-qtreename'};

    my %final_attributes = ( %state_base_field_settings, %$parsed_output );
    my $obj = $pkg->new( command_interface => $command_interface );
    $obj->_set_fields( row => \%final_attributes );
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _fetch_7mode_gui

1;
