# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary Quota ComponentState Module
## @author raja.sindhe@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SysManager::Quota

=head1 DESCRIPTION

C<NACL::CS::SysManager::Quota> is a derived class of
L<NACL::CS::ComponentState::SysManager|lib-NACL-CS-ComponentState-SysManager-pm>.  It represents the  
state of an ONTAP Quota.  A related class is
L<NACL::C::SysManager::Quota|lib-NACL-C-SysManager-Quota-pm>, which represents access
to an ONTAP Quota.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the Quota.

=over

=item C<< container >>

The name of the container.

=item C<< vserver >>

=item C<< volume >>

=item C<< qtree >>

=item C<< type >>

=item C<< users >>

=item C<< spacehardlimit >>

=item C<< filesoftlimit >>

=item C<< spaceusage >>

=item C<< fileusage >>

=item C<< spacesoftlimit >>

=item C<< filehardlimit >>

=back

=cut

package NACL::CS::SysManager::Quota;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(validate);

use base 'NACL::CS::ComponentState::SysManager';

use NACL::C::SysManager::Quota;
use NACL::Exceptions::NoElementsFound qw(:try);

# Import Exception class
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'container',
    scalar => 'volume',
    scalar => 'qtree',
    scalar => 'type',
    scalar => 'users',
    scalar => 'spacehardlimit',
    scalar => 'filesoftlimit',
    scalar => 'spaceusage',
    scalar => 'fileusage',
    scalar => 'spacesoftlimit',
    scalar => 'filehardlimit',
];

=head1 METHODS

=head2 fetch

    my $quota_state  =
       NACL::CS::SysManager::Quota->fetch(command_interface => $ci,...);
    my @quota_states =
       NACL::CS::SysManager::Quota->fetch(command_interface => $ci,...);

see L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm>.

Uses a CMode GUI or a 7Mode GUI APISet.
(Works only when container and vserver parameters are passed under filter option).

=cut

####################### GUI Fetch ###############################
sub fetch {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method => "_fetch_cmode_gui",
                set    => "SysManager_CMode",
            },
            {   method => '_fetch_7mode_gui',
                set    => 'SysManager_7Mode',
            },
        ],
        exception_text => 'No matching quota container found',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

####################### GUI CMode Fetch ###############################
sub _fetch_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();

    my $apiset            = $opts{apiset};
    my $requested_fields  = $opts{requested_fields};
    my $filter            = $opts{filter};
    my $command_interface = $opts{command_interface};

    my $quota_args = $pkg->_hash_copy(
        source => $filter,
        map    => {
            "container" => "-container",
            "qtype"     => "-qtype",
            "vserver"   => "-vservername",
        }
    );

    $pkg->_check_for_required_filter_fields(
        filter          => $filter,
        required_fields => [qw(container vserver)],
    );

    my ( $response, $caught_exception, $parsed_output );
    try {

        # caling the gui api to get quota status
        $response      = $apiset->get_quota_info(%$quota_args);
        $parsed_output = $response->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {

        # A caught exception indicates that the quota being looked for
        # does not exist. We catch the exception and return immediately.
        # The 'fetch' frontend decides whether to throw a NoElementsFound
        # exception based on the value of 'allow_empty'
        $Log->debug( 'CommandFailedException caught. Requested quota does '
                . 'not exist' );
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my %state_base_field_settings;

    $state_base_field_settings{'container'} = $quota_args->{'-container'};
    $state_base_field_settings{'qtype'}     = $quota_args->{'-qtype'};
    $state_base_field_settings{'vserver'}   = $quota_args->{'-vservername'};

    my %final_attributes = ( %state_base_field_settings, %$parsed_output );

    my $obj = $pkg->new( command_interface => $command_interface );
    $obj->_set_fields( row => \%final_attributes );
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _fetch_cmode_gui

####################### GUI 7Mode Fetch ###############################
sub _fetch_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();

    my $apiset            = $opts{apiset};
    my $requested_fields  = $opts{requested_fields};
    my $filter            = $opts{filter};
    my $command_interface = $opts{command_interface};

    my $quota_args = $pkg->_hash_copy(
        source => $filter,
        map    => {
            "container" => "-container",
            "qtype"     => "-qtype",
        }
    );

    $pkg->_check_for_required_filter_fields(
        filter          => $filter,
        required_fields => [qw(container)],
    );

    my ( $response, $caught_exception, $parsed_output );
    try {

        # caling the gui api to get quota status
        $response      = $apiset->get_quota_info(%$quota_args);
        $parsed_output = $response->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {

        # A caught exception indicates that the quota being looked for
        # does not exist. We catch the exception and return immediately.
        # The 'fetch' frontend decides whether to throw a NoElementsFound
        # exception based on the value of 'allow_empty'
        $Log->debug( 'CommandFailedException caught. Requested quota does '
                . 'not exist' );
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my %state_base_field_settings;

    $state_base_field_settings{'container'} = $quota_args->{'-container'};
    $state_base_field_settings{'qtype'}     = $quota_args->{'-qtype'};

    my %final_attributes = ( %state_base_field_settings, %$parsed_output );
    my $obj = $pkg->new( command_interface => $command_interface );
    $obj->_set_fields( row => \%final_attributes );
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _fetch_7mode_gui

1;
