# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary SNMP ComponentState Module
## @author vikas.p@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SysManager::SNMP

=head1 DESCRIPTION

C<NACL::CS::SysManager::SNMP> is a derived class of
L<NACL::CS::ComponentState::SysManager|lib-NACL-CS-ComponentState-SysManager-pm>.  It represents the  
state of an ONTAP SNMP.  A related class is
L<NACL::C::SysManager::SNMP|lib-NACL-C-SysManager-SNMP-pm>, 
which represents access to an ONTAP SNMP.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SNMP 
element are the attributes of the SNMP ComponentState.

=over

=item C<< trap_status >>

=item C<< status >>

=item C<< location >>

=item C<< contact >>

=back

=cut

package NACL::CS::SysManager::SNMP;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(validate);

use base 'NACL::CS::ComponentState::SysManager';

use NACL::C::SysManager::SNMP;
use NACL::Exceptions::NoElementsFound qw(:try);

# Import Exception class
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;

use Class::MethodMaker [
    scalar => 'status',
    scalar => 'trap_status',
    scalar => 'location',
    scalar => 'contact',
];

=head1 METHODS 

=head2 fetch

    my $SNMP_state  = 
		NACL::CS::SysManager::SNMP->fetch(command_interface => $ci,...);
    my @SNMP_states = 
		NACL::CS::SysManager::SNMP->fetch(command_interface => $ci,...);

see L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm>.

Uses a CMode GUI or a 7Mode GUI APISet.

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method => "_fetch_cmode_gui",
                set    => "SysManager_CMode",
            },
            {   method => '_fetch_7mode_gui',
                set    => 'SysManager_7Mode',
            },
        ],
        exception_text => 'SNMP not configured',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();

    my $apiset            = $opts{apiset};
    my $requested_fields  = $opts{requested_fields};
    my $filter            = $opts{filter};
    my $command_interface = $opts{command_interface};

    my $snmp_args = $pkg->_hash_copy( source => $filter, );

    my ( $response, $caught_exception, $parsed_output );
    try {

        # caling the gui api to get snmp status
        $response      = $apiset->get_snmp_info(%$snmp_args);
        $parsed_output = $response->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {

        # A caught exception indicates that the snmp being looked for
        # does not exist. We catch the exception and return immediately.
        # The 'fetch' frontend decides whether to throw a NoElementsFound
        # exception based on the value of 'allow_empty'
        $Log->debug('CommandFailedException caught. SNMP not configured');
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my %final_attributes = %$parsed_output;
    my $obj = $pkg->new( command_interface => $command_interface );
    $obj->_set_fields( row => \%final_attributes );
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _fetch_cmode_gui

####################### GUI 7Mode Fetch ###############################
sub _fetch_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();

    my $apiset            = $opts{apiset};
    my $requested_fields  = $opts{requested_fields};
    my $filter            = $opts{filter};
    my $command_interface = $opts{command_interface};

    my $snmp_args = $pkg->_hash_copy( source => $filter, );

    my ( $response, $caught_exception, $parsed_output );
    try {

        # caling the gui api to get snmp status
        $response      = $apiset->get_snmp_info(%$snmp_args);
        $parsed_output = $response->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {

        # A caught exception indicates that the snmp being looked for
        # does not exist. We catch the exception and return immediately.
        # The 'fetch' frontend decides whether to throw a NoElementsFound
        # exception based on the value of 'allow_empty'
        $Log->debug('CommandFailedException caught. SNMP not configured');
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my %final_attributes = %$parsed_output;
    my $obj = $pkg->new( command_interface => $command_interface );
    $obj->_set_fields( row => \%final_attributes );
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _fetch_7mode_gui

1;
