# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary Snapmirror ComponentState Module
## @author vikas.p@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SysManager::Snapmirror

=head1 DESCRIPTION

C<NACL::CS::SysManager::Snapmirror> is a derived class of
L<NACL::CS::ComponentState::SysManager|lib-NACL-CS-ComponentState-SysManager-pm>.  It represents the  
state of an ONTAP Snapmirror.  A related class is
L<NACL::C::SysManager::Snapmirror|lib-NACL-C-SysManager-Snapmirror-pm>, which represents access
to an ONTAP Snapmirror.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the Snapmirror 
element are the attributes of the qtree ComponentState.

=over

=item C<< targetfiler >>

The name of the target filer.

=item C<< vserver >>

=item C<< source >>

=item C<< destination >>

=item C<< health >>

=item C<< state >>

=item C<< status >>

=item C<< schedule >>

=item C<< data_transfer_rate >>

=item C<< last_transfer_timestamp >>

=item C<< exported_snapshot_copy >>

=item C<< latest_snapshot_copy >>

=item C<< progress >>

=item C<< destination_status >>

=item C<< source_status >>

=item C<< snapmirror_type >>

Filled in for 7Mode.

=item C<< transfer_rate >>

Filled in for 7Mode.

=item C<< lag_time >>

Filled in for 7Mode.

=item C<< transfer_status >>

Filled in for 7Mode.

=item C<< base_snapshot_copy >>

Filled in for 7Mode.

=item C<< last_transfer_size >>

Filled in for 7Mode.

=back

=cut

package NACL::CS::SysManager::Snapmirror;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(validate);

use base 'NACL::CS::ComponentState::SysManager';

use NACL::C::SysManager::Snapmirror;

# Import Exception class
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'targetfiler',
    scalar => 'source',
    scalar => 'destination',
    scalar => 'health',
    scalar => 'state',
    scalar => 'status',
    scalar => 'schedule',
    scalar => 'data_transfer_rate',
    scalar => 'last_transfer_timestamp',
    scalar => 'exported_snapshot_copy',
    scalar => 'latest_snapshot_copy',
    scalar => 'progress',
    scalar => 'destination_status',
    scalar => 'source_status',

    #only 7mode
    scalar => 'snapmirror_type',
    scalar => 'transfer_rate',
    scalar => 'lag_time',
    scalar => 'transfer_status',
    scalar => 'base_snapshot_copy',
    scalar => 'last_transfer_size',
    scalar => 'src_state',
    scalar => 'des_state',
];

=head1 METHODS 

=head2 fetch

    my $snapmirror_state  = 
		NACL::CS::SysManager::Snapmirror->fetch(command_interface => $ci,...);
    my @snapmirror_states = 
		NACL::CS::SysManager::Snapmirror->fetch(command_interface => $ci,...);

see L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm>.

Uses a CMode GUI or a 7Mode GUI APISet.

=cut

####################### GUI Fetch ###############################
sub fetch {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method => "_fetch_cmode_gui",
                set    => "SysManager_CMode",
            },
            {   method => '_fetch_7mode_gui',
                set    => 'SysManager_7Mode',
            },
        ],
        exception_text => 'No matching snapmirror relation found',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

####################### GUI CMode Fetch ###############################
sub _fetch_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();

    my $apiset            = $opts{apiset};
    my $requested_fields  = $opts{requested_fields};
    my $filter            = $opts{filter};
    my $command_interface = $opts{command_interface};

    my $snapmirror_args = $pkg->_hash_copy(
        source => $filter,
        map    => {
            "targetfiler"      => "-srcfiler",
            "source"           => "-sourcename",
            "destination"      => "-destname",
            "vserver"          => "-vservername",
        }
    );

    $pkg->_check_for_required_filter_fields(
        filter => $filter,
        required_fields =>
            [qw(targetfiler source destination vserver)],
    );

    my ( $response, $caught_exception, $parsed_output );
    try {

        # caling the gui api to get snapmirror relation status
        $response      = $apiset->get_snapmirror_info(%$snapmirror_args);
        $parsed_output = $response->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {

        # A caught exception indicates that the snapmirror being looked for
        # does not exist. We catch the exception and return immediately.
        # The 'fetch' frontend decides whether to throw a NoElementsFound
        # exception based on the value of 'allow_empty'
        $Log->debug(
            'CommandFailedException caught. Requested snap mirror relation does '
                . 'not exist' );
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my %state_base_field_settings;

    $state_base_field_settings{'targetfiler'}
        = $snapmirror_args->{'-srcfiler'};
    $state_base_field_settings{'source'}
        = $snapmirror_args->{'-sourcename'};
    $state_base_field_settings{'destination'}
        = $snapmirror_args->{'-destname'};
    $state_base_field_settings{'vserver'}
        = $snapmirror_args->{'-vservername'};

    my %final_attributes = ( %state_base_field_settings, %$parsed_output );

    my $obj = $pkg->new( command_interface => $command_interface );
    $obj->_set_fields( row => \%final_attributes );
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _fetch_cmode_gui

####################### GUI 7Mode Fetch ###############################
sub _fetch_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();

    my $apiset            = $opts{apiset};
    my $requested_fields  = $opts{requested_fields};
    my $filter            = $opts{filter};
    my $command_interface = $opts{command_interface};

    my $snapmirror_args = $pkg->_hash_copy(
        source => $filter,
        map    => {
            "targetfiler"      => "-targetfiler",
            "source"           => "-sourcename",
            "destination"      => "-destname",
        }
    );

    $pkg->_check_for_required_filter_fields(
        filter          => $filter,
        required_fields => [qw(targetfiler source destination)],
    );

    my ( $response, $caught_exception, $parsed_output );
    try {

        # caling the gui api to get snapmirror relation status
        $response      = $apiset->get_snapmirror_info(%$snapmirror_args);
        $parsed_output = $response->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {

  # A caught exception indicates that the snapmirror relation being looked for
  # does not exist. We catch the exception and return immediately.
  # The 'fetch' frontend decides whether to throw a NoElementsFound
  # exception based on the value of 'allow_empty'
        $Log->debug(
            'CommandFailedException caught. Requested snapmirror relation does '
                . 'not exist' );
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my %state_base_field_settings;
    
    $state_base_field_settings{'targetfiler'}
        = $snapmirror_args->{'-targetfiler'};
    $state_base_field_settings{'source'}
        = $snapmirror_args->{'-sourcename'};
    $state_base_field_settings{'destination'}
        = $snapmirror_args->{'-destname'};

    my %final_attributes = ( %state_base_field_settings, %$parsed_output );
    my $obj = $pkg->new( command_interface => $command_interface );
    $obj->_set_fields( row => \%final_attributes );
    
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _fetch_7mode_gui

1;
