#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemCapability ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemCapability

=head1 DESCRIPTION

C<NACL::CS::SystemCapability> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemCapability.  A related class is
L<NACL::C::SystemCapability|lib-NACL-C-SystemCapability-pm>, which represents access
to an ONTAP SystemCapability.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the SystemCapability
element are the attributes of the SystemCapability ComponentState.

=over

=item C<< "require_for_boot" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <capability-info>
    <require-for-boot>$value</require-for-boot>
 </capability-info>

=item C<< "node" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <capability-info>
    <node>$node</node>
 </capability-info>

=item C<< "description" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <capability-info>
    <description>$description</description>
 </capability-info>

=item C<< "auto_disable" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <capability-info>
    <auto-disable>$auto_disable</auto-disable>
 </capability-info>

=item C<< "capability" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <capability-info>
    <capability>$capability</capability>
 </capability-info>

=item C<< "obsolete" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <capability-info>
    <obsolete>$obsolete_value</obsolete>
 </capability-info>

=item C<< "marked_downgrade" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <capability-info>
    <marked-downgrade>$value</marked-downgrade>
 </capability-info>

=item C<< "howto_downgrade" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <capability-info>
    <howto-downgrade>$value</howto-downgrade>
 </capability-info>

=item C<< "required_on_all_nodes" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <capability-info>
    <required-on-all-nodes>$value</required-on-all-nodes>
 </capability-info>

=item C<< "release_debut" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <capability-info>
    <release-debut>$value</release-debut>
 </capability-info>

=item C<< "enabled" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <capability-info>
    <enabled>$value</enabled>
 </capability-info>

=item C<< auto_enable >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <capability-info>
    <auto-enable>$value</auto-enable>
 </capability-info>

=item C<< enable_ready >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <capability-info>
    <enable-ready>$value</enable-ready>
 </capability-info>

=item C<< uses_enable_ready >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <capability-info>
    <uses-enable-ready>$value</uses-enable-ready>
 </capability-info>

=back

=cut

package NACL::CS::SystemCapability;

use strict;
use warnings;
use feature 'state';

use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one Dumper);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::Exceptions::InvalidChoice;

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'require_for_boot',
    scalar => 'node',
    scalar => 'description',
    scalar => 'auto_disable',
    scalar => 'capability',
    scalar => 'obsolete',
    scalar => 'marked_downgrade',
    scalar => 'howto_downgrade',
    scalar => 'required_on_all_nodes',
    scalar => 'release_debut',
    scalar => 'enabled',
    scalar => 'auto_enable',
    scalar => 'enable_ready',
    scalar => 'uses_enable_ready',
];

=head1 METHODS

=head2 fetch

    my $SystemCapability_state   = NACL::CS::SystemCapability->fetch(command_interface => $ci, ...);
	my @SystemCapability_states = NACL::CS::SystemCapability->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "capability-get-iter" API for CMode ZAPI.

B<NOTE:> The ZAPI implementation has currently been disabled because of ONTAP
burt 609892.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                check     => '_zapi_check',
            },
        ],
	show_cmd       => 'system capability show',
        exception_text => 'No matching system capability(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'system_capability_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

# Remove once burt 609892 is fixed.
sub _zapi_check {
    $Log->enter() if $may_enter;
    my $msg = 'The ZAPI capability-get-iter currently does not return any '
        . 'records (see burt 609892). For this reason, the ZAPI '
        . 'back-end of NACL::CS::SystemCapability is currently disabled';
    
    # Don't spam log with warnings for every single call to SystemCapability->fetch;
    # just warn the first time.
    my $method;
    state $warned;
    if ($warned) {
        $method = 'comment';
    } else {
        $warned = 1;
        $method = 'warn';
    }
    $Log->$method($msg);
    $Log->exit() if $may_exit;
    NACL::Exceptions::InvalidChoice->throw($msg);
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    return $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => "capability_get_iter",
        copy => [
            qw(require-for-boot node description auto-disable obsolete
                capability howto-downgrade marked-downgrade required-on-all-nodes
                release-debut enabled )
        ],
    );
    $Log->exit() if $may_exit;

} ## end sub _fetch_cmode_zapi

1;
