# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemNodeAutosupportBudget ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemNodeAutosupportBudget

=head1 DESCRIPTION

C<NACL::CS::SystemNodeAutosupportBudget> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemNodeAutosupportBudget.  A related class is
L<NACL::C::SystemNodeAutosupportBudget|lib-NACL-C-SystemNodeAutosupportBudget-pm>, which represents access
to an ONTAP SystemNodeAutosupportBudget.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the SystemNodeAutosupportBudget
element are the attributes of the SystemNodeAutosupportBudget ComponentState.

=over

=item C<< "size_limit" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-budget-info>
	<size-limit>$value</size-limit>
 </autosupport-budget-info>

7Mode CLI:

For "filter": -size-limit (Programmatic name: size-limit)

For "requested_fields": -fields (Programmatic name: fields)

=item C<< "subsystem" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-budget-info>
	<subsystem>$value</subsystem>
 </autosupport-budget-info>

7Mode CLI:

For "filter": -subsystem (Programmatic name: subsystem)

For "requested_fields": -fields (Programmatic name: fields)

=item C<< "node" >>

Filled in for CMode CLI/ZAPI

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-budget-info>
	<node-name>$value</node-name>
 </autosupport-budget-info>

=item C<< "time_limit" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-budget-info>
	<time-limit>$value</time-limit>
 </autosupport-budget-info>

7Mode CLI:

For "filter": -time-limit (Programmatic name: time-limit)

For "requested_fields": -fields (Programmatic name: fields)

=item C<< diagnostic_size_limit >>

Diagnostic Content Size Limit

Filled in for CMode CLI.

=item C<< diagnostic_time_limit >>

Diagnostic Content Time Limit

Filled in for CMode CLI.


=back

=cut

package NACL::CS::SystemNodeAutosupportBudget;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'size_limit',
    scalar => 'subsystem',
    scalar => 'node',
    scalar => 'time_limit',
    scalar => 'diagnostic_size_limit',
    scalar => 'diagnostic_time_limit',
];

=head1 METHODS

=head2 fetch

    my $SystemNodeAutosupportBudget_state   = NACL::CS::SystemNodeAutosupportBudget->fetch(command_interface => $ci, ...);
	my @SystemNodeAutosupportBudget_states = NACL::CS::SystemNodeAutosupportBudget->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI, 7Mode CLI.

Invokes "autosupport-budget-get-iter" API for CMode ZAPI.

Invokes "autosupport budget show" command for 7Mode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
	show_cmd       => 'system node autosupport budget show',
        exception_text =>
            'No matching system node autosupport budget(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'system_node_autosupport_budget_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();

    my $command_interface     = delete $opts{command_interface};
    my $apiset                = delete $opts{apiset};
    my %copy_filter           = %{$opts{filter}};
    my @copy_requested_fields = @{$opts{requested_fields}};

    # Delete always as deleting an entry from hash is simple
    delete $copy_filter{'node'};

    # Do the extra processing of deletion from requested_fields only when
    # 'node' is specified in the requested_fields
    @copy_requested_fields = grep { $_ ne 'node' } @copy_requested_fields;

    my %apiset_opts = ();
    if (scalar(keys %copy_filter)) {
        %apiset_opts = %copy_filter;
    }
    if (scalar(@copy_requested_fields)) {
        $apiset_opts{fields} = join(',', @copy_requested_fields);
    }

    my $response = $apiset->autosupport_budget_show(%apiset_opts);
    my $output   = $response->get_parsed_output();
    my @state_objs;
    my $final_attributes;

    foreach my $row (@$output) {
        $final_attributes = $pkg->_hash_copy(
            source => $row,
            copy   => [qw(subsystem time-limit size-limit)],
            map    => {
                'size_budget' => 'size-limit',
                'time_budget' => 'time-limit'
            },
        );
        my $obj = $pkg->new(command_interface => $command_interface);
        $obj->_set_fields(row => $final_attributes);

        push @state_objs, $obj;
    } ## end foreach my $row (@$output)

    return @state_objs;

} ## end sub _fetch_7mode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    return $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => "autosupport_budget_get_iter",
        copy => [qw (size-limit time-limit subsystem)],
        map  => {'node' => ['node-name'],},
    );
    $Log->exit() if $may_exit;

} ## end sub _fetch_cmode_zapi

1;
