# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemSnmp ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemSnmp

=head1 DESCRIPTION

C<NACL::CS::SystemSnmp> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemSnmp.  A related class is
L<NACL::C::SystemSnmp|lib-NACL-C-SystemSnmp-pm>, which represents access
to an ONTAP SystemSnmp.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the SystemSnmp
element are the attributes of the SystemSnmp ComponentState.

=over

=item C<< "traphosts" >>

Filled in for CMode CLI, CMode ZAPI.

CMode ZAPI: Accessible through the field "traphosts" in the output of ZAPI "snmp-status"

=item C<< "authtrap" >>

=item C<< "location" >>

Filled in for CMode CLI, CMode ZAPI.

CMode ZAPI: Accessible through the field "location" in the output of ZAPI "snmp-status"

=item C<< "printhost" >>

Returns Array

=item C<< "access" >>

=item C<< "contact" >>

Filled in for CMode CLI, CMode ZAPI.

CMode ZAPI: Accessible through the field "contact" in the output of ZAPI "snmp-status"

=item C<< "hostname" >>

Returns Array

=item C<< "enable_value" >>

=item C<< "init" >>

Filled in for CMode CLI, CMode ZAPI.

CMode ZAPI: This field is mapped to "is-trap-enabled" in zapi and is accessible in the output of of ZAPI "snmp-status"

=item C<< "community" >>

(Array)

Filled in for CMode CLI, CMode ZAPI.

CMode ZAPI: This field is mapped to "communities" in zapi and is accessible in the output of of ZAPI "snmp-status"

=item C<< "init_flag" >>

=back

=cut

package NACL::CS::SystemSnmp;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one Dumper);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    array  => 'traphosts',
    scalar => 'authtrap',
    scalar => 'location',
    array  => 'printhost',
    scalar => 'access',
    scalar => 'contact',
    array  => 'hostname',
    scalar => 'enable_value',
    scalar => 'init',
    array  => 'community',
    scalar => 'init_flag',
];

=head1 METHODS

=head2 fetch

    my $SystemSnmp_state   = NACL::CS::SystemSnmp->fetch(command_interface => $ci, ...);
	my @SystemSnmp_states = NACL::CS::SystemSnmp->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI or a CMode ZAPI APISet.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        exception_text => 'No matching system snmp(s) found',
        is_singleton   => 1
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
        @_,
        api          => 'system_snmp_show',
        is_singleton => 1
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg    = shift;
    my %opts   = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset = $opts{apiset};
    my $snmp_response;

    $snmp_response = $apiset->snmp_status();
    my $snmp_output = $snmp_response->get_parsed_output();
    my @state_objs;
    my $state_fields = $pkg->_zapi_hash_copy(
        source => $snmp_output->[0],
        copy   => [qw(contact traphosts location)],
        map    => [
            "communities"     => "community",
            "is-trap-enabled" => "init"
        ],
    );

    my $obj = $pkg->new(command_interface => $opts{command_interface});
    $obj->_set_fields(row => $state_fields);
    push @state_objs, $obj;

    $Log->exit() if $may_exit;
    return @state_objs;

} ## end sub _fetch_cmode_zapi

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg    = shift;
    my %opts   = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset = $opts{apiset};
    my $snmp_response;

    my %copy_filter           = %{$opts{filter}};
    my @copy_requested_fields = @{$opts{requested_fields}};
    my $deleted_filter        = $pkg->_remove_relational_regex_filters(
        filter           => \%copy_filter,
        requested_fields => \@copy_requested_fields
    );
    my $filter           = \%copy_filter;
    my $requested_fields = \@copy_requested_fields;

    $snmp_response = $apiset->snmp();
    my $snmp_output = $snmp_response->get_parsed_output();
    my @state_objs;
    my $state_fields = $pkg->_hash_copy(
        source => $snmp_output->[0],
        copy   => [qw(authtrap contact traphosts location community init)],
    );

    my $obj = $pkg->new(command_interface => $opts{command_interface});
    $$state_fields{'command_interface'} = $opts{'command_interface'};
    $obj->_set_fields(row => $state_fields);
    push @state_objs, $obj;

    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_7mode_cli
1;
