#
# Copyright (c) 2001-2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsSecurity ComponentState Module
## @author Rupamjyoti.Baruah@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME
 
NACL::CS::VserverCifsSecurity
 
=head1 DESCRIPTION
 
C<NACL::CS::VserverCifsSecurity> is a derived class of
 
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>.  It represents the 
 
state  of Vserver Cifs Security.  A related class is
 
L<NACL::C::VserverCifsSecurity|lib-NACL-C-VserverCifsSecurity-pm>, which represents 

access to ONTAP Vserver Cifs Security.
 
=head1 ATTRIBUTES
 
The individual pieces of data that are part of the  state of the 
 
element are the attributes of the  ComponentState.
 
=over
 
 
=item C<< vserver >>

Filled in for CMode CLI/ZAPI.

=item C<< kerberos_clock_skew >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-security>
    <kerberos-clock-skew>$values</kerberos-clock-skew>
 </cifs-security>

=item C<< kerberos_ticket_age >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-security>
    <kerberos-ticket-age>$values</kerberos-ticket-age>
 </cifs-security>

 =item C<< kerberos_renew_age >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-security>
    <kerberos-renew-age>$values</kerberos-renew-age>
 </cifs-security>

=item C<< realm >>

Filled in for CMode CLI.

=item C<< kdc_ip >>
(Array)

Filled in for CMode CLI.

=item C<< kdc_name >>
(Array)

Filled in for CMode CLI.

=item C<< site >>
(Array)

Filled in for CMode CLI.

=item C<< is_signing_required >>

Filled in for CMode CLI/ZAPI.

=item C<< password_complexity_required >>

Require Password Complexity for Local User Accounts, possible value(s) are
true,false
Filled in for CMode CLI.

=item C<< is_password_complexity_required >>

Filled in for CMode CLI/ZAPI.


=item C<< use_start_tls_for_ad_ldap >>


Use start_tls for CIFS LDAP connections possible value(s) are, true,false

Filled in for CMode CLI/ZAPI.

=item C<< is_aes_encryption_enabled >>


Is AES-128 and AES-256 Encryption Enabled possible value(s) are, true,false

Filled in for CMode CLI/ZAPI


=item C<< lm_compatibility_level >>


LM Compatibility Level possible value(s) are, lm-ntlm-ntlmv2-krb,ntlm-ntlmv2-krb,ntlmv2-krb,krb

Filled in for CMode CLI.

=item C<< is_smb_encryption_required >>

Require SMB Encryption for Incoming CIFS Traffic possible value(s) are, true,false

Filled in for CMode CLI/ZAPI.

=back

=cut

package NACL::CS::VserverCifsSecurity;

use strict;
use warnings;
use NACL::ComponentUtils qw(_dump_one);
use Params::Validate qw(validate);
use base 'NACL::CS::ComponentState::ONTAP';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NACL::Exceptions::NoElementsFound qw(:try);

use Class::MethodMaker [

    # These fields are from the C Mode "vserver cifs security show"
    scalar => 'vserver',
    scalar => 'kerberos_clock_skew',
    scalar => 'kerberos_ticket_age',
    scalar => 'kerberos_renew_age',
    scalar => 'realm',
    array  => 'kdc_ip',
    array  => 'kdc_name',
    array  => 'site',
    scalar => 'is_signing_required',
    scalar => 'password_complexity_required',
    scalar => 'is_password_complexity_required',
    scalar => 'use_start_tls_for_ad_ldap',
    scalar => 'is_aes_encryption_enabled',
    scalar => 'lm_compatibility_level',
    scalar => 'is_smb_encryption_required',
    scalar => 'smb1_enabled_for_dc_connections',
    scalar => 'smb2_enabled_for_dc_connections',
];

=head1 METHODS


=head2 fetch
  
    my $node_state  = NACL::CS::VserverCifsSecurity->fetch(command_interface=>$ci,...); 
    my @node_states = NACL::CS::VserverCifsSecurity->fetch(command_interface=>$ci,...); 
  
see L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm> 

Supports CMode CLI/ZAPI APISet.

Invokes "cifs-security-get-iter" API for CMode ZAPI

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver cifs security show',
        choices  => [
            {   method    => "_fetch_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching vserver cifs security(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    my $pkg = shift;
    return $pkg->SUPER::_fetch_cmode_cli(@_,
        api => "vserver_cifs_security_show");
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $vserver = $opts{filter}->{vserver};

    my @states = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api  => "cifs-security-get-iter",
        copy => [
            qw(use-start-tls-for-ad-ldap kerberos-clock-skew
                kerberos-ticket-age kerberos-renew-age
                is-signing-required is-password-complexity-required
                is-aes-encryption-enabled vserver is-smb-encryption-required smb1-enabled-for-dc-connections smb2-enabled-for-dc-connections)
        ],
    );
    foreach my $state (@states) {
        if (!defined($state->vserver())) {
            $state->vserver($vserver);
        }
    }

    $Log->exit() if $may_exit;
    return @states;

} ## end sub _fetch_cmode_zapi

1;
