#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverIscsiTpgroup ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverIscsiTpgroup

=head1 DESCRIPTION

C<NACL::CS::VserverIscsiTpgroup> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverIscsiTpgroup.  A related class is
L<NACL::C::VserverIscsiTpgroup|lib-NACL-C-VserverIscsiTpgroup-pm>, which represents access
to an ONTAP VserverIscsiTpgroup.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the VserverIscsiTpgroup
element are the attributes of the VserverIscsiTpgroup ComponentState.

=over

=item C<< "vserver" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-tpgroup-list-entry-info>
     <vserver>$value</vserver>
 </iscsi-tpgroup-list-entry-info>

=item C<< "isUserDefined" >>

Filled in for CMode CLI.

=item C<< "name" >>

Filled in for CMode CLI.

=item C<< "userDefined" >>

Filled in for CMode CLI.

=item C<< "is_user_defined" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-tpgroup-list-entry-info>
     <is-user-defined>$value</is-user-defined>
 </iscsi-tpgroup-list-entry-info>

=item C<< "force" >>

Filled in for CMode CLI.

=item C<< "rtpid" >>

Filled in for CMode CLI.

=item C<< "tpgroup" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-tpgroup-list-entry-info>
     <tpgroup-name>$value</tpgroup-name>
 </iscsi-tpgroup-list-entry-info>

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "tpgtag" >>

Filled in for 7Mode CLI.

Maps to:

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "tpgroup_tag" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-tpgroup-list-entry-info>
     <tpgroup-tag>$value</tpgroup-tag>
 </iscsi-tpgroup-list-entry-info>

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "lif" >>

(Array)

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-tpgroup-list-entry-info>
     <interface-list-entries>
	     <interface-list-entry-info>
	         <interface-name>@values</interface-name>
	     </interface-list-entry-info>
	 </interface-list-entries>
 </iscsi-tpgroup-list-entry-info>

=back

=cut

package NACL::CS::VserverIscsiTpgroup;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one Dumper);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'isUserDefined',
    scalar => 'name',
    scalar => 'userDefined',
    scalar => 'is_user_defined',
    scalar => 'force',
    scalar => 'rtpid',
    scalar => 'tpgroup',
    scalar => 'tpgroup_tag',
    scalar => 'tpgtag',
    array  => 'lif',
];

=head1 METHODS

=head2 fetch

    my $VserverIscsiTpgroup_state   = NACL::CS::VserverIscsiTpgroup->fetch(command_interface => $ci, ...);
	my @VserverIscsiTpgroup_states = NACL::CS::VserverIscsiTpgroup->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI, 7Mode CLI.

Invokes "iscsi-tpgroup-get-iter" API for CMode ZAPI.

Invokes "iscsi tpgroup show" command for 7Mode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver iscsi tpgroup show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        exception_text => 'No matching vserver iscsi tpgroup(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_iscsi_tpgroup_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    my $copy = [qw(vserver tpgroup-tag is-user-defined)];
    my $map  = {
        "tpgroup" => ['tpgroup-name'],
        "lif"     => ['interface-list-entries', 'interface-name']
    };

    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api  => "iscsi-tpgroup-get-iter",
        copy => $copy,
        map  => $map,
    );
    $Log->exit() if $may_exit;
    return @state_objs;

} ## end sub _fetch_cmode_zapi

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg         = shift;
    my %opts        = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset      = $opts{apiset};
    my %copy_filter = %{$opts{filter}};
    my @copy_requested_fields = @{$opts{requested_fields}};
    my $deleted_filter        = $pkg->_remove_relational_regex_filters(
        filter           => \%copy_filter,
        requested_fields => \@copy_requested_fields
    );
    my $filter           = \%copy_filter;
    my $requested_fields = \@copy_requested_fields;
    my $req_field_filter;
    $req_field_filter->{'requested_fields'} = $requested_fields;
    $req_field_filter->{'filter'}           = \%copy_filter;
    my %cmd_opts;
    my @state_objs;

    my $response = $apiset->iscsi_tpgroup_show();
    my $output   = $response->get_parsed_output();
    foreach my $row (@$output) {
        my $obj;
        $obj = $pkg->new(command_interface => $opts{command_interface});
        my $members      = $row->{"member_interfaces"};
        my $row_modified = $pkg->_hash_copy(
            source => $row,
            map    => {
                'name'   => 'tpgroup',
                'tpgtag' => 'tpgroup_tag',
            },
        );
        $$row_modified{'lif'} = $members;
        $obj->_set_fields(row => $row_modified);
        push @state_objs, $obj;
    } ## end foreach my $row (@$output)
    $Log->exit() if $may_exit;
    $Log->debug("value of the parsed output is :" . Dumper(\@state_objs));

    return @state_objs;
} ## end sub _fetch_7mode_cli

1;
