#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Methods common to various CS modules
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::_Mixins::ComponentState

=head1 DESCRIPTION

This package consists of methods which are common to various, but not all,
CS modules.

=cut

package NACL::CS::_Mixins::ComponentState;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with validate_pos ARRAYREF SCALAR OBJECT);
use NATE::Exceptions::Argument qw(:try);
use NACL::ComponentUtils qw(Dumper);
use Scalar::Util qw(looks_like_number);
use DateTime::Format::Strptime;

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(
        sort_by_newest
        sort_by_oldest
        _arrange_by_create_time
    );
    
    Params::Validate::validation_options(
        on_fail => sub {
            NATE::Exceptions::Argument->throw(shift);
        }
    );
}

=head1 METHODS

=head2 sort_by_newest

    my @sorted_list = $cs_pkg->sort_by_newest(state_objs => \@state_objs);
    
Sorts the state objects with the newest elements first in the list (this
is determined by the C<create-time> field of the state object)

=cut

sub sort_by_newest {
    $Log->enter();
    my ($pkg, @args) = @_;
    my @sorted_list = $pkg->_arrange_by_create_time(@args, newest_first => 1);
    $Log->exit();
    return @sorted_list;
}

=head2 sort_by_oldest

    my @sorted_list = $cs_pkg->sort_by_oldest(state_objs => \@state_objs);
    
Sorts the state objects with the oldest elements first in the list (this
is determined by the C<create-time> field of the state object)

=cut

sub sort_by_oldest {
    $Log->enter();
    my ($pkg, @args) = @_;
    my @sorted_list = $pkg->_arrange_by_create_time(@args, newest_first => 0);
    $Log->exit();
    return @sorted_list;
}

# Helper for sorting by create-time
sub _arrange_by_create_time {
    $Log->enter();
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            state_objs => { type => ARRAYREF },
            newest_first => { type => SCALAR },
        }
    );
    my $newest_first = delete $opts{newest_first};
    $pkg->_check_if_each_of_correct_type(%opts);
    my $state_objs = $opts{state_objs};

    # Validate that each object has the create-time field set
    foreach my $state_obj (@$state_objs) {
        if ( !$state_obj->create_time_isset() ) {
            $Log->exit();
            NATE::Exceptions::Argument->throw( 'All of the objects '
                    . "should have the 'create-time' field set, but "
                    . "one of them didn't:\n"
                    . Dumper($state_obj)
                    . "\n" );
        }
    }

    my %hash_keyed_by_create_time;
    my $add_to_hash = sub {
        my ( $create_time, $cs ) = validate_pos(
            @_,
            { type => SCALAR },
            { type => OBJECT, isa => $pkg }
        );
        if ( exists $hash_keyed_by_create_time{$create_time} ) {
            my $arr_ref = $hash_keyed_by_create_time{$create_time};
            push @$arr_ref, $cs;
        } else {
            $hash_keyed_by_create_time{$create_time} = [$cs];
        }
    };

    # Note that it's possible for volumes to be created at the same
    # time. In that case, keying by the creation time will result in the
    # later entry overwriting the earlier one. Hence, the value of each
    # entry in the hash is an array-reference
    my @unknown_time;
    foreach my $cs (@$state_objs) {
        my $create_time = $cs->create_time();
        if ( $create_time eq '-' ) {

            # Creation time is unknown
            push @unknown_time, $cs;
        } elsif ( looks_like_number($create_time) ) {

            # Time is probably already in unix-time
            $add_to_hash->( $create_time, $cs );
        } else {

# Converting the time as returned by the filer into unix-time so
# that it can be compared. The time returned by the filer is in a
# form like this:
# "Wed Sep 07 18:07:59 2011"
# See
# http://search.cpan.org/~drolsky/DateTime-Format-Strptime-1.5000/lib/DateTime/Format/Strptime.pm
# for a description of the pattern tokens.
            my $strp = DateTime::Format::Strptime->new(
                pattern => '%A %B %d %T %Y' );
            my $dt = $strp->parse_datetime($create_time);
            $add_to_hash->( $dt->epoch(), $cs );
        }
    }
    
    my @sorted_keys;
    if ($newest_first) {
        @sorted_keys = sort { $b <=> $a } keys %hash_keyed_by_create_time;
    } else {
        @sorted_keys = sort { $a <=> $b } keys %hash_keyed_by_create_time;
    }

    my @sorted_list;
    foreach my $epoch (@sorted_keys) {
        push @sorted_list, @{ $hash_keyed_by_create_time{$epoch} };
    }

    push @sorted_list, @unknown_time;

    $Log->debug( sub { "Sorted list is:\n" . Dumper( \@sorted_list ) } );
    $Log->exit();
    return @sorted_list;
}

1;
