#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.

###
# Documentation:
# https://confluence.eng.netapp.com/x/r13S
###

package NACL::Init::Config::ONTAPSelect::Deploy::Cluster::ESX;

use strict;
use warnings;

use NATE::ParamSet qw(param_global);

use NACL::Init::Utils qw(process_init_args);

use Moo;
extends qw(NATE::Init::Config);

sub global_setup_args
{
    my $self = shift;

    my @definitions = (
        ##########################
        ## Required
        ##########################
        {
            'name'        => 'DEPLOY_APPLIANCE',
            'description' => 'ONTAP Select Deploy appliance name - Host file must exist',
            'required'    => 1,
        },
        {
            'name'        => 'DNS_SERVER',
            'description' => 'Comma-separated list of DNS server addresses (IPs)',
            'spec'        => 's@',
            'required'    => 1,
        },
        {
            'name'        => 'NTP_SERVER',
            'description' => 'Comma-separated list of NTP server addresses (IPs)',
            'spec'        => 's@',
            'required'    => 1,
        },
        ##########################
        ## Optional
        ##########################
        {
            'name'        => 'DNS_SEARCH',
            'description' => 'FQDNs to search for short hostnames',
            'spec'        => 's@',
            'default'     => [
                'ctl.gdl.englab.netapp.com',
                'gdl.englab.netapp.com',
                'gdl.eng.netapp.com',
                'rtp.netapp.com',
            ],
        },
        {
            'name'        => 'HOST_LOCATION',
            'description' => 'Location of the hosts',
            'default'     => 'rtp',
        },
        {
            'name'        => 'CLUSTER_NAME',
            'description' => 'Name of the ONTAPSelect cluster',
            'default'     => eval(
                 "my \$suffix = qx(cat /dev/urandom | tr -cd 'A-F0-9' | head -c 8);
                 chomp(\$suffix); # mutatable
                 return \"C\$suffix\";"
            ),
        },
        {
            'name'        => 'INSTANCE_TYPE',
            'description' => 'Instance type for the cluster',
            'default'     => 'small',
        },
        {
            'name'        => 'JSON_FILE',
            'description' => 'Ignores every other parameter except DEPLOY_APPLIANCE and simply creates the '
                           . 'cluster with the provided JSON file - BYOJF (Bring Your Own JSON File)',
        },
        {
            'name'        => 'LICENSE_TYPE',
            'description' => 'Specifies what kind of licensing to use',
            'default'     => 'licensed',
        },
        {
            'name'        => 'ONTAP_IMAGE',
            'description' => 'Calculated with ONTAP_IMAGE_PROMOTED, ONTAP_IMAGE_CONSOLE, and images on '
                           . 'the Deploy appliance (Most recent one)',
        },
        {
            'name'        => 'ONTAP_IMAGE_CONSOLE',
            'description' => 'Type of console (serial or vid)',
            'default'     => 'serial',
        },
        {
            'name'        => 'ONTAP_IMAGE_PROMOTED',
            'description' => 'Whether to use the promoted image or not',
            'spec'        => 'boolean',
            'default'     => 0,
        },
        {
            'name'        => 'STORAGE_POOL_CAPACITY',
            'description' => 'Amount of space in GB that is allowed to be used by Deploy for Nodes'
        },
        {
            'name'        => 'STORAGE_POOL_DISK_TYPE',
            'description' => 'Type of disks in the storage pool',
            'default'     => 'sas',
        },
        {
            'name'        => 'STORAGE_POOL_NAME',
            'description' => 'Name of the datastore to use',
        },
        {
            'name'        => 'ONLY_JSON',
            'description' => 'Just generate the JSON file and nothing else',
            'default'     => 0,
        },
    );

    return process_init_args(\@definitions);
}

sub global_cleanup_args
{
    my $self = shift;

    my @definitions = (
        ##########################
        ## Required
        ##########################
        {
            'name'        => 'DEPLOY_APPLIANCE',
            'description' => 'ONTAP Select Deploy appliance name - Host file must exist',
            'required'    => 1,
        },
        ##########################
        {
            'name'        => 'CLUSTER_NAME',
            'description' => 'If specified, only deletes this cluster, otherwise it deletes all clusters',
        },
    );

    return process_init_args(\@definitions);
}

sub generate_resources {
    my $self = shift;

    # Get the value of the Resources.
    my $resources_param = param_global->get('NODES') // NATE::BaseException->throw('Missing Mandatory NODES');
    my @resources = split(',', $resources_param);
    $self->resources(\@resources);
}

sub generate_setup_tasks
{
    my $self = shift;

    my $global_args = $self->global_setup_args;

    # Does not work in parallel
    $self->add_task(
        'task_name'  => 'NACL::Init::Tasks::ONTAPSelect::Deploy::Cluster::ESX::GenerateJSON',
        'alias'      => 'cluster_add',
        'task_args'  => $global_args,
        'run_scheme' => 'all_together',
    );

    if ($global_args->{'ONLY_JSON'})
    {
        return 1;
    }

    # Does not work in parallel
    $self->add_task(
        'task_name'  => 'NACL::Init::Tasks::ONTAPSelect::Deploy::Cluster::Add',
        'alias'      => 'cluster_add',
        'task_args'  => $global_args,
        'run_scheme' => 'all_together',
    );

    my $hostrec_da = Hostrec->new('id' => param_global->get('DEPLOY_APPLIANCE'));
    $global_args->{'VCENTER'} = $hostrec_da->{'VCENTER'};

    # Does not work in parallel
    $self->add_task(
        'task_name'  => 'NACL::Init::Tasks::ONTAPSelect::ESX::Node::ConfigureConsole',
        'alias'      => 'cluster_configure_console',
        'task_args'  => $global_args,
    );
    # Does not work in parallel
    $self->add_task(
        'task_name'  => 'NACL::Init::Tasks::ONTAPSelect::Deploy::Cluster::InstallLicense',
        'alias'      => 'cluster_install_license',
        'task_args'  => $global_args,
        'run_scheme' => 'all_together',
    );
}

sub generate_cleanup_tasks
{
    my $self = shift;

    my $global_args = $self->global_cleanup_args;

    # Does not work in parallel
    $self->add_task(
        'task_name'  => 'NACL::Init::Tasks::ONTAPSelect::Deploy::Cluster::Offline',
        'alias'      => 'cluster_offline',
        'task_args'  => $global_args,
        'run_scheme' => 'all_together',
    );
    # Does not work in parallel
    $self->add_task(
        'task_name'  => 'NACL::Init::Tasks::ONTAPSelect::Deploy::Cluster::Delete',
        'alias'      => 'cluster_delete',
        'task_args'  => $global_args,
        'run_scheme' => 'all_together',
    );
}

1;
