#
#   Copyright (c) 2016 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::Init::Tasks::ONTAP::AssignDisks;

use strict;
use warnings;

use NATE::Log qw(log_global);
use NACL::Transit;
use Connectrec;
use Data::Dumper;

use Moo;
extends qw(NATE::Init::Task);

# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use constant defaults => {
    DISK_CHANNELS => undef,
};

sub execute {
    my ($self, %args) = @_;

    my $node = $args{resource};

    log_global()->comment("Transit to maintenance prompt");
    my $host = Hostrec->new( id => $node );
    my $t = NACL::Transit->new(name => $host);
    $t->change_state(to => "MAINT");
    my $conn = $t->get_connection();

    my $api_set_maint_obj = NACL::APISet->new(
        connection => $conn,
        interface  => 'CLI',
        category   => 'Node',
        set        => 'Maintenance',
    );

    $conn->discardbuff();
    $conn->resync();

    my @match_table = (
        '.*Do you wish to continue.*'=> 'yes',
    );

    my $resObj = $api_set_maint_obj->storage_release_disks(
        'connectrec-match_table'=> \@match_table,
        'connectrec-timeout'    => 900,
    );

    $conn->discardbuff();
    $conn->resync();

    my $task_args        = $self->task_args;

    # parse the disk channels parameter and
    # convert it into an array of regex matches
    my @disk_channels = ();
    my $disk_channels_str = ();
    $disk_channels_str = ($task_args->{DISK_CHANNELS}) // $host->{DISK_CHANNELS};

    if ($host->is_simulator() && (! $task_args->{DISK_CHANNELS})){
        # vsims are hardcoded
        @disk_channels = ("0b", "0c");
    } else {
        @disk_channels = split(/;/, $disk_channels_str);

        # expand the disk channels that contain disk ranges
        @disk_channels = map {
            my $d = $_;
            # this disk channel is a SAS disk range
            if ($d =~ /^(\S+?\.\d+)\.(\d+)\-(\d+)/) {
                # treat this as a SAS disk range (e.g. 0b.02.0-23)
                # expand the disk range into individual regexes,
                # each representing a single disk.
                my $adapter_and_shelf = $1;
                my @disk_range = ($2..$3);
                map { qr/^${adapter_and_shelf}\.${_}(P\d+)?$/ } @disk_range;
            } elsif ($d =~ /^(\S+?\.\d+)$/ || $d =~ /^((?:v|[0-9]+)[0-9a-z])$/) {
                # treat this as an adapter and a shelf (e.g. 0a.00)
                # also matches adapater name only like v0, 11b or 0a
                # this means they want everything on the shelf and adapter
                # or just the adapter.
                qr/^$d\./;
            } else {
                # treat this as an absolute disk (e.g. 0a.01.12)
                qr/^$d(P\d+)?$/;
            }
        } @disk_channels;
    }

    # Find all disks on this system that match the disk channel regexes
    # in the disk_channels array
    my $disk_show_output;
    my @disks_to_assign;
    my %disks_to_unpartition;
    $disk_show_output = $api_set_maint_obj->disk_show(
        "verbose" => "-v",
    );
    my $parsed_output = $disk_show_output->{parsed_output};
    foreach my $disk (@{$parsed_output}) {
        foreach my $channel (@disk_channels) {
            if ($disk->{disk} =~ $channel ) {
                # see if this disk represents a disk partition
                if ($disk->{disk} =~ /^(.+)P\d+$/) {
                    # add this disk to the "unpartition" hash if it contains a partition
                    $disks_to_unpartition{$1}++;
                } else {
                    # this disk needs to get assigned to this node
                    push(@disks_to_assign, $disk->{disk});
                }
                    last;
            }
        }
    }

    # destroy the partitions on all disks that have them
    foreach my $disk_to_unpartition (keys %disks_to_unpartition) {
        $conn->execute(command => "disk unpartition -F $disk_to_unpartition",
                       match_table => [
                           'Abort unpartition \(y/n\)\?' => 'n'
                       ]);
    }

    # Assigning the disks (craft the assign disk command)
    my $disk_assign_command = "";
    foreach my $disk_to_assign (@disks_to_assign) {
        $disk_assign_command = $disk_assign_command . "disk assign $disk_to_assign -f -p 0;";
    }
    $api_set_maint_obj->execute_raw_command(command => $disk_assign_command);
    $api_set_maint_obj->disk_show("verbose" => "-v");
}

1;
