#   Copyright (c) 2016 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAP::ClusterSetupNoSimplicityWizard;

BEGIN {
    use Exporter 'import';
    our @EXPORT = qw (create_security_login);
}
use strict;
use warnings;

use NATE::Log qw(log_global);
use NATE::ParamSet qw(param_global);
use Carp qw(confess);
use NACL::Transit;
use NACL::C::Node;
use NACL::STask::SecurityLogin;
use NACL::C::NetworkInterface;
use NATE::Process;
use NACL::C::ClusterContactInfo;
use NACL::C::Cluster;
use NACL::C::VserverServicesDns;

use Moo;

extends 'NATE::Init::Task';

# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use NATE::BaseException qw(:try);

use constant defaults => {
    SWITCHLESS_CLUSTER => 0
};

has +run_scheme => (
    is => 'ro',
    default => "all_together"
);

sub execute {
    my ($self, %args) = @_;

    my $node = $args{resource};
    my @nodes = @{$node};


    # STEP 1: create cluster lifs in parallel.
    my @proc_objs;
    foreach my $node (@nodes) {
        # transition to the CLI first
        my $t = NACL::Transit->new(name => $node);
        $t->change_state(to => "CLI");

        my $node_obj = NACL::C::Node->new( node => $node);
        my $node_hostrec = $node_obj->hostrec();
        my @nets = $node_hostrec->net_sort(include=>{nettype=>'cluster'});
        my $proc = NATE::Process->new(
            codespec=> \&_create_cluster_lif,
            runid   => $node."_cluster_lifs",
            args    => [$node_obj, \@nets],
            onexit  => \&NATE::Process::on_exit_propagate_worst_result,
        );
        push(@proc_objs, $proc);
    }

    my $procMan = NATE::ProcManager->new(
        proc_info => \@proc_objs,
        runid     => "",
    );

    $procMan->start();
    $procMan->wait();

    my $node_obj = NACL::C::Node->new(node => $nodes[0]);
    my $node_hostrec = $node_obj->hostrec();
    my @nets = $node_hostrec->net_sort(include=>{nettype=>'cluster'});

    # configure switchless cluster if requested to
    if ($self->task_args->{SWITCHLESS_CLUSTER} or $node_hostrec->{SWITCHLESS_CLUSTER}) {
        # burt1068186/1072593 - switchless cluster automation is no longer necessary in (HN) 9.2 and beyond
        if ($node_obj->version_manager()->is_version_le(release => "9.2")) {
            require NACL::C::NetworkOptionsSwitchlessCluster;
            NACL::C::NetworkOptionsSwitchlessCluster->modify(
                command_interface => $node_obj,
                force             => "true",
                enabled           => "true"
            );
        }
    }

    # create cluster on node 1.
    NACL::C::Cluster->create(
        'method-timeout'  => 600,
        command_interface => $node_obj,
        clustername       => $nodes[0].time,
    );

    # create cluster_mgmt lif
    my @net_mgmt = $node_hostrec->net_sort(include=>{nettype=>'cluster_management'});

    my $vserver = NACL::CS::Vserver->fetch(
        command_interface => $node_obj,
        requested_fields => ['vserver'],
        filter => { type => "admin" }
    );

    NACL::C::NetworkInterface->create(
        command_interface => $node_obj,
        vserver         => $vserver->vserver(),
        lif             => "clus_mgmt",
        address         => $net_mgmt[0]->{address},
        netmask         => $net_mgmt[0]->{netmask},
        'home-node'     => $node_obj->node(),
        role            => "cluster-mgmt",
        'home-port'     => $net_mgmt[0]->{interface},
    );

    try {
        NACL::C::NetworkRoute->create(
            command_interface => $node_obj,
            vserver           => $vserver->vserver(),
            gateway         => $net_mgmt[0]->{gateway},
            destination     => "0.0.0.0/0",
        );
    } catch NATE::BaseException with {
        my $e = shift;
        # ignore duplicate route exists errors, it could just be that
        # it was already configured by the previous task
        if ($e->text() !~ /Duplicate route exists/i) {
            $e->throw();
        }
    };

    ##Site Specific Data Structure##
    my $site = param_global()->get("SITE");

    # only perform the following code if we have access to site specific information
    if ($site and $site !~ /^-/) {
        # first process the testutils parameter space
        # otherwise we won't be able to access the site data
        require TestUtils;
        TestUtils::process();
        my %siteinfo = TestUtils::get_site_info();

        my @nameservers = ();
        push ( @nameservers, $siteinfo{$site}->{mgmt_dns1} ) if (defined($siteinfo{$site}->{mgmt_dns1}));
        push ( @nameservers, $siteinfo{$site}->{mgmt_dns2} ) if (defined($siteinfo{$site}->{mgmt_dns2}));
        my @domains = split / /, $siteinfo{$site}->{dns_domains};
        if (@nameservers and @domains) {
            NACL::C::VserverServicesDns->create(
                command_interface => $node_obj,
                vserver           => $vserver->vserver(),
                state             => "enabled",
                'name-servers'    => \@nameservers,
                domains           => \@domains,
            );
        } else {
            log_global()->log("Not configuring DNS, because no name server " .
                              "information is available for SITE=$site")
        }
    } else {
        log_global()->log("Not configuring DNS, because the SITE=$site is not known");
    }

    # join all remaining nodes to the cluster
    my @remaining_nodes = @nodes;
    shift @remaining_nodes;
    foreach my $node (@remaining_nodes) {
        NACL::C::Cluster->join(
            'method-timeout'    => 600,
            command_interface   => NACL::C::Node->new( node => $node),
            clusteripaddr       => $nets[0]->{address},
        );
    }

    # cluster wide command. So running it only once.
    NACL::C::ClusterContactInfo->modify(
        command_interface       => $node_obj,
        country                 => "US",
        'business-name'         => "NetApp",
        'zip-code'              => "94089",
        'primary-email'         => 'sti@netapp.com',
        'secondary-name'        => "sti",
        'primary-name'          => "sti",
        state                   =>"CA",
        'primary-phone'         => "111-555-1212",
        city                    => "Sunnyvale",
        'secondary-phone'       => "333-555-1212",
        'primary-alt-phone'     => "222-555-1212",
        address                 => "\"495 East Java Drive\"",
        'secondary-email'       => 'sti@netapp.com',
        'secondary-alt-phone'   => "444-555-1212",
    );

    # rename aggr0 to have the node name in it
    my $node_name = $node_obj->node();
    $node_name =~ s/-/_/g;
    NACL::C::StorageAggregate->rename(
        command_interface   => $node_obj,
        newname             => "aggr0_".$node_name."_0",
        aggregate           => "aggr0",
    );

    # burt #1045097 (ESIS wants a special user account created when gx boot finishes)
    # call create_security_login method (takes a cmd_interface)
    create_security_login(command_interface => $node_obj, username => 'boost' , password => 'netapp1!' , role => 'admin' );
}

sub _create_cluster_lif {
    my ( $node_obj, $net ) = @_;

    # this is a hack because in the smokes environment, the vsim nate host files
    # do not have the cluster port values properly defined.
    my @ports = ();
    if ($node_obj->hostrec()->is_simulator()) {
        @ports = ("e0a","e0b");
    }

    my @nets = @{$net};
    my $count = 1;
    foreach my $net (@nets) {
        # if the interface is not explicitly set
        my $port = $net->{interface} // shift @ports;

        # remove this port from the @ports array to make sure it never gets reused
        @ports = grep {!/^$port$/} @ports;

        my $lif_name = "clus" . $count;
        try {
            NACL::C::NetworkInterface->create(
                command_interface => $node_obj,
                vserver         => "Cluster",
                lif             => $lif_name,
                address         => $net->{address},
                netmask         => $net->{netmask},
                'home-node'     => $node_obj->node(),
                role            => "cluster",
                'home-port'     => $port,
            );

        } catch NATE::BaseException with {
            my $ex = shift;
            log_global()->comment("Lif $lif_name already exists");
        };
        $count++;
    }
}

sub create_security_login {
    my (%args) = @_;

    my $vserver = NACL::CS::Vserver->fetch(
        command_interface => $args{command_interface},
        requested_fields => ['vserver'],
        filter => { type => "admin" }
    );

    my @application = ("ssh", "ontapi", "console");

    foreach my $app (@application) {
        NACL::STask::SecurityLogin->create(
            command_interface  => $args{command_interface},
            'vserver'          => $vserver->vserver(),
            'username'         => $args{username},
            'authmethod'       => "password",
            'application'      => $app,
            'password'         => $args{password},
            'role'             => $args{role},
            nacltask_if_exists => "reuse",
        );
    }
}

1;
