#
#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::Init::Tasks::ONTAP::NetbootONTAP;

use strict;
use warnings;

use NACL::Transit;
use Hostrec;
use NACL::APISet;
use NATE::ParamSet qw(param_global);

use Carp qw(confess);

use Moo;
extends qw(NATE::Init::Task);

# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use constant defaults => {
    ONTAP_BUILDROOT        => undef,
    ONTAP_VARIANT          => undef,
    ONTAP_INSTALL_PROTOCOL => "http",
    ONTAP_INSTALL_SERVER   => undef,
    ONTAP_INSTALL_PORT     => undef,
    ONTAP_NETBOOT_KERNEL   => undef,
    ONTAP_NETBOOT_TIMEOUT  => 900,
};

sub execute {
    my ($self, %opts) = @_;

    my $node = $opts{resource};

    # Make sure we received our required task args
    my $task_args        = $self->task_args;
    my $netboot_kernel   = $task_args->{ONTAP_NETBOOT_KERNEL};
    my $netboot_protocol = $task_args->{ONTAP_INSTALL_PROTOCOL};
    my $netboot_server   = $task_args->{ONTAP_INSTALL_SERVER}  // confess "Missing required task argument 'NETBOOT_SERVER'";
    my $netboot_port     = $task_args->{ONTAP_INSTALL_PORT};
    my $netboot_timeout  = $task_args->{ONTAP_NETBOOT_TIMEOUT};

    # if a kernel was not explicitly specified, dynamically build it using the
    # buildroot and the variant.
    if (!$netboot_kernel) {
        # ontap build root and ontap variant are required if the kernel is not provided
        my $ontap_buildroot = $task_args->{ONTAP_BUILDROOT} // confess "Missing required task argument 'ONTAP_BUILDROOT'";
        my $ontap_variant   = $task_args->{ONTAP_VARIANT}   // confess "Missing required task argument 'ONTAP_VARIANT'";
        # dynamically construct the kernel path based on the buildroot provided
        $netboot_kernel = join("",
                               $ontap_buildroot,
                               "/final/bedrock/export/",
                               $ontap_variant,
                               "/netboot/netboot/kernel")
    }

    # netboot this node
    my $hostrec = Hostrec->new(id => $node);

    # make a transit object for this node
    my $t = NACL::Transit->new(name => $hostrec);
    $t->change_state(to => "FIRMWARE");
    my $c = $t->get_connection();

    # use the connection from the transit object
    # to talk to the firmware's apiset
    my $fw_apiset = NACL::APISet->new(
        connection => $c,
        category => "Node",
        interface => "CLI",
        set => "Firmware"
    );

    # set up the mgmt port using the ifconfig command
    # find the management ports information from the hostrec obj
    my @mgmt = $hostrec->net_filter(include => [{nettype => "management", family => "inet4"}]);
    @mgmt = grep {
        $_->{address} and $_->{netmask} and $_->{interface}
    } @mgmt;
    if (not scalar(@mgmt)) {
        confess "Could not identify the management port information for node $node";
    }

    # construct the ifconfig command to configure the management port of the node
    my $cmd = "ifconfig ";
    if ($mgmt[0]->{interface}) {
        $cmd .= $mgmt[0]->{interface} . " ";
    }
    if ($mgmt[0]->{address}) {
        $cmd .= "-addr=" . $mgmt[0]->{address} . " ";
    }
    if ($mgmt[0]->{netmask}) {
        $cmd .= "-mask=" . $mgmt[0]->{netmask} . " ";
    }
    if ($mgmt[0]->{gateway}) {
        $cmd .= "-gw=" . $mgmt[0]->{gateway} . " ";
    }
    $fw_apiset->execute_raw_command(command => $cmd);

    # construct the http URL for the ontap kernel using the install server details provided
    # using the ${protocol}://${netboot_server}[:${port}]${kernel_file}
    if ($netboot_port) {
        $netboot_port = ":" . $netboot_port;
    } else {
        $netboot_port = "";
    }

    my $kernel_url =
        $netboot_protocol . "://" .
        $netboot_server .
        $netboot_port .
        $netboot_kernel;

    # make sure the netboot server is accessible
    $fw_apiset->execute_raw_command(command => "ping -c=5 $netboot_server");

    my $bad_match = sub { die "Detected a fatal prompt: $1"; };

    my @match_table = (
        "Press Ctrl-C for"        => "\cc",
        "voltage is too low"      => "c",
        "want to continue"        => "y",
        "(Timeout occured)"       => $bad_match,
        "(command status)"        => $bad_match,
        "(Login:)"                => $bad_match,
        "(Waiting for send-home)" => $bad_match,
        "(Panic String)"          => $bad_match,
        "(page fault)"            => $bad_match,
        "(Fatal trap)"            => $bad_match,
    );

    # netboot the kernel
    $c->execute(command => "netboot $kernel_url",
                timeout => $netboot_timeout,
                prompts => [
                    "Please make a selection",
                    'Selection \(1-',
                    "Install new software first",
                ],
                match_table => \@match_table);

    return;
}

1;
