#
#   Copyright (c) 2016 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::Init::Tasks::ONTAP::Setenv;

use strict;
use warnings;

use Carp qw(confess);
use Connectrec;
use Tie::IxHash;
use NATE::ParamSet qw(param_global);
use NATE::Log qw(log_global);
use NACL::Transit;
use NATE::OS qw(login_name);

use Moo;
extends qw(NATE::Init::Task);

# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use NATE::BaseException qw(:try);

use constant defaults => {
    "DEFAULT_SETENV" => {
        "panic-on-rc-shutdown-timeout?"         => "true",
        "bootarg.init.auto_cluster_lif.disable" => "true",
        "AUTO_FW_UPDATE"                        => "true",
        "AUTOBOOT"                              => "false",
        "bootarg.init.trace_on_panic"           => "1",
        "ntap.bootmenu.allowall"                => "false",
        "bootarg.bootmenu.allowall"             => "false",
        "ntap.init.clearvarfsnvram"             => "false",
        "bootarg.init.clearvarfsnvram"          => "false",
        "ntap.mgwd.maxlogfiles"                 => "15",
        "bootarg.mgwd.maxlogfiles"              => "15",
        "ntap.init.usebootp"                    => "false",
        "bootarg.init.usebootp"                 => "false",
        "ntap.rlm.gdb"                          => "1",
        "bootarg.rlm.gdb"                       => "1",
        "ntap.init.console_muted"               => "false",
        "bootarg.init.console_muted"            => "false",
        "bootarg.dblade.aggr_default_64bit"     => "false",
        "bootarg.dblade.aggr_default_32bit"     => "false",
        "bootarg.activatebsdport"               => "false",
        "ntap.activatebsdport"                  => "false",
        "bootarg.init.dhcp.disable"             => "false",
        "bootarg.autosupport.tag"               => "", #needs to be set in the task
    },
    CONFIG_SETENV => {},
    SETENV        => undef,
    ONTAP_SETENV  => undef,
    UNSETENV      => [],
};

sub execute {
    my ($self, %args) = @_;
    my @nodes = $args{resource};

    my ($task, %conn, %task_args, %default_setenv, %config_setenv, %setenv);

    # This code is used to split the ONTAP_SETENV passed on command line and make a hash.
    # Now merge the hash with the default values, the command line values taking
    # precedence over the default values. (The default values hash has already been
    # merged with the config_setenv hash, the config values taking precedence over
    # default values).

    $task = $self->task_args;
    %task_args = %$task;
    tie %default_setenv, 'Tie::IxHash';
    tie %config_setenv, 'Tie::IxHash';
    tie %setenv, 'Tie::IxHash';
    my $defaultsetenv = $task->{DEFAULT_SETENV};
    my $configsetenv = $task->{CONFIG_SETENV};
    # setenv from command line will look like this: SETENV/ONTAP_SETENV="p1=>v1,p2=>v2".
    # parse this string and make a hash and them merge with defaults hash
    my $task_setenv = "";
    if($task->{"SETENV"}){
        $task_setenv = $task->{"SETENV"};
    }
    if($task->{"ONTAP_SETENV"}){
        $task_setenv .= "," if($task_setenv);
        $task_setenv .= $task->{"ONTAP_SETENV"};
    }
    my @setenv_array = split /,/,$task_setenv;
    foreach my $setenv_element (@setenv_array) {
        my @temp = split /=>/,$setenv_element;
        $setenv{$temp[0]} = $temp[1];
    }

    %default_setenv = %$defaultsetenv;
    %config_setenv = %$configsetenv;
    # merge default and config, replace duplicates with values of config
    @default_setenv{ keys %config_setenv } = values %config_setenv;
    # now merge the above result with setenv, giving setenv values priority
    foreach my $key2 ( keys %setenv ) {
        if( exists $default_setenv{$key2} ) {
            $default_setenv{$key2} = $setenv{$key2};
            next;
        } else {
            $default_setenv{$key2} = $setenv{$key2};
        }
    }

    my $unsetenv  = $task->{UNSETENV};

    foreach my $node (@nodes) {
        # transition to the FIRMWARE prompt
        my $t = NACL::Transit->new(
            name => $node,
            prefer_sp => 0
        );
        $t->change_state(to => "FIRMWARE");

        my $api_set_firmware_obj = NACL::APISet->new(
            connection => $t->get_connection(),
            interface  => 'CLI',
            category   => 'Node',
            set        => 'Firmware',
        );

        $api_set_firmware_obj->set_defaults();

        # set bootargs
        foreach my $key (keys %default_setenv) {
            if ( $key eq "bootarg.autosupport.tag" ) {
                my $logname = login_name();
                my $uuid = param_global()->get('NATE_UUID') || $$;
                my $value = $logname . '.gx_boot_job_' . $uuid;
                $default_setenv{$key} = $value;
            }
            $api_set_firmware_obj->setenv(
                varname => $key,
                value   => $default_setenv{$key},
            );
        }

        # unset any bootargs
        foreach my $env (@{$unsetenv}) {
            $api_set_firmware_obj->unsetenv(
                varname => $env,
            );
        }

        # display all bootargs that have been set
        $api_set_firmware_obj->printenv();
    }
}

1;
