#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.

#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::Deploy::Appliance::KVM::Add;

# core
use strict;
use warnings;
use Carp qw(croak);
use Cwd qw(abs_path);
use File::Basename qw(basename);
use File::Spec::Functions qw(catfile);
use IO::File;

# cpan
use Net::Ping;
use Moo;
extends qw(NATE::Init::Task);

# nate
use Hostrec;
use NATE::Events qw(eventloop);
use NATE::Log qw(log_global);
use NATE::ParamSet qw(param_global);
use NATE::Process;
use NATE::Util::Appliance qw(buildroot_to_version);

sub execute
{
    my ($self, %args) = @_;

    my $deploy_appliance = $args{'resource'};

    my $log       = log_global;
    my %task_args = %{$self->task_args};
    my $raw;

    my $promoted = 0;
    if ($task_args{'DEPLOY_RAW'})
    {
        $raw = $task_args{'DEPLOY_RAW'};
        $promoted = 1 if ($raw =~ m{/promo/});
    }
    else
    {
        my $rawtgz;
        if ($task_args{'DEPLOY_RAW_TGZ'})
        {
            $rawtgz = abs_path($task_args{'DEPLOY_RAW_TGZ'});
        }
        elsif ($task_args{'BUILDROOT'})
        {
            my $p4info_file = "$task_args{'BUILDROOT'}/final/bedrock/export/common/p4info/p4info";
            if (-f $p4info_file)
            {
                my $fh = IO::File->new($p4info_file, 'r');
                while (my $line = <$fh>)
                {
                    if ($line =~ m/^P4BRANCH='DOT\/(.*)'/)
                    {
                        my $branch = $1;
                        $rawtgz = abs_path("/x/eng/rlse/ontapdeploy/${branch}N/offtap/lib/deploy/"
                             . "bedrock/export/linux64.debug/final/installer/ONTAPdeploy.raw.tgz");
                        last;
                    }
                }
                $fh->close;
            }
            else
            {
                croak("Unable to find $p4info_file to determine what nightly to "
                    . "use for the OVA.");
            }
        }

        if (! -e $rawtgz)
        {
            croak("Unable to find .raw.tgz file: $rawtgz");
        }

        $log->info(".raw.tgz File: $rawtgz");

        $promoted = 1 if ($rawtgz =~ m{/promo/});

        if ($rawtgz =~ m{/rlse[0-9]*/})
        {
            # Flex cache does not present files as sparse
            $log->info("Image file is in rlse, need to do a copy sparse over WAN...");
            my $dest = catfile(param_global->get('TOP_LOGDIR'), 'ONTAPdeploy.raw.tgz');

            my $rsync_proc = NATE::Process->new(
                'username' => 'sti',
                'codespec' => 'rsync',
                'args'     => ['--sparse=always', '--progress', 'sti@cyclsvl:'.$rawtgz, $dest],
                'onexit'   => \&NATE::Process::on_exit_die_on_non_zero_exit_status,
            );
            $rsync_proc->start;
            $rsync_proc->wait;
            $rsync_proc->destroy;
            $rawtgz = $dest;
        }

        my $tar_proc = NATE::Process->new(
            'codespec' => 'tar',
            'args'     => ['--sparse', '-xvf', $rawtgz, '-C', param_global->get('TOP_LOGDIR')],
            'onexit'   => \&NATE::Process::on_exit_die_on_non_zero_exit_status,
        );
        $tar_proc->start;
        $tar_proc->wait;
        $tar_proc->destroy;
        $raw = $rawtgz;
        $raw =~ s/^.*\///;
        $raw = param_global->get('TOP_LOGDIR') . "/$raw";
        $raw =~ s/\.tgz$//;
    }

    my $hostrec_da   = Hostrec->new('id' => $deploy_appliance);
    my $hostrec_host = Hostrec->new('id' => $hostrec_da->simhost);

    my $host_conn = Connectrec->new(
        'id'       => $hostrec_host->id,
        'host'     => $hostrec_host->id,
        'template' => 'kvm_ssh',
    );

    my $size = $host_conn->execute(
        'command'      => ['stat', '-c', '%s', $raw],
        'check_status' => 1,
    );
    chomp($size);

    $host_conn->execute(
        'command' => [
            'virsh',
            'vol-create-as',
            'ontap-select-storage-pool',
            'deploy',
            $size,
          ],
        'check_status' => 1,
    );

    $host_conn->execute(
        'command' => [
            'dd',
            'iflag=direct',
            'oflag=direct',
            'bs=1M',
            "if=$raw",
            'of=/dev/ontap-select-storage-pool/deploy'
          ],
        'check_status' => 1,
        'timeout'      => 2700,
    );

    $host_conn->execute(
        'command'      => ['chmod', '777', '/dev/shm'],
        'check_status' => 1,
    );

    $host_conn->execute(
        'command' => [
            'virt-install',
            "--name=$deploy_appliance",
            '--vcpus=2',
            '--ram=4096',
            '--os-type=linux',
            '--controller=scsi,model=virtio-scsi',
            "--disk", "vol=ontap-select-storage-pool/deploy",
            "--network", 'type=bridge,source='.$hostrec_host->{'KVM_MGMT_NETWORK_NAME'}.',model=virtio,virtualport_type=openvswitch',
            '--console=pty',
            '--import',
            '--wait', '0',
        ],
        'check_status' => 1,
    );

    # For some reason I have to tag and untag if multiple bridges for mgmt
    $host_conn->execute(
        'command' => [
            'ovs-vsctl',
            'set',
            'port',
            'vnet0',
            'tag=' . $hostrec_host->{'MANAGEMENT_VLANID'},
        ],
        'check_status' => 1,
    );

    my $netrec = $hostrec_da->net_best;

    $host_conn->execute_start('command' => ['virsh', 'console', $deploy_appliance]);

    my $match_table = [
        ('Host name\s+:\s*$') => $deploy_appliance,
        ('Use DHCP to set networking information?') => "n",
        ('Host IP address\s*:\s*$') => $netrec->{'address'},
        ('Net mask\s+:\s*$') => $netrec->{'netmask'},
        ('Gateway\s+:\s*$') => $netrec->{'gateway'},
        ('Primary DNS address\s*:\s*$') => $task_args{'DNS_SERVER'}->[0],
        ('Secondary DNS address\s*:\s*$') => $task_args{'DNS_SERVER'}->[1],
        ('Please enter in all search domains separated by spaces \(can be left blank\):') => join(' ', @{$task_args{'DNS_SEARCH'}}),
        ('Are these values correct?') => 'y',
    ];

    if ($promoted)
    {
        $log->info("TAR_GZ is promoted");
        $host_conn->execute_expect(
            'match_table' => [
                @{$match_table},
                ("$deploy_appliance login:") => 'admin',
                ('Password:') => 'admin123',
                ('Enter current password:') => 'admin123',
                ('Enter new password:') => 'netapp1!',
                ('Retype new password:') => 'netapp1!',
                ('Enter Product Company:') => 'NetApp',
                ('Enter Proxy URL\s+:') => '',
            ],
            'prompts' => [
                'AutoSupport configuration set successfully',
            ],
            'timeout' => 900,
        );

        $host_conn->execute_expect(
            'match_table' => [
                ("\(ONTAPdeploy\)") => 'user password modify',
                ('Enter current password:') => 'netapp1!',
                ('Enter new password:') => 'admin123',
                ('Retype new password:') => 'admin123',
                ('successfully') => sub {$host_conn->send("\c]")},
            ],
            'timeout' => 60,
        );
    }
    else
    {
        $log->info("TAR_GZ is not promoted");
        $host_conn->execute_expect(
            'match_table' => [
                @{$match_table},
                ("$deploy_appliance login:") => sub {$host_conn->send("\c]")},
            ],
            'timeout' => 900,
        );
    }

    eval {system('ping', '-c', 5, $hostrec_da->default_ip)};

    if ($hostrec_host->{'KVM_MGMT_NETWORK_NAME'} ne $hostrec_host->{'KVM_INTERNAL_NETWORK_NAME'})
    {
        $log->info("Untagging mgmt interface for deploy...");
        $host_conn->execute(
            'command' => [
                'ovs-vsctl',
                'remove',
                'port',
                'vnet0',
                'tag',
                $hostrec_host->{'MANAGEMENT_VLANID'},
            ],
        );

        eval {system('ping', '-c', 5, $hostrec_da->default_ip)};
    }

    _wait_for_on($hostrec_da);
}

sub _wait_for_on
{
    my $hostrec = shift;

    my $log = log_global;

    while (1)
    {
        my $exit_status = system('ping', '-c', 5, $hostrec->default_ip);
        if ($exit_status != 0)
        {
            $log->info("Still unresponsive, waiting for it to "
                . 'be responsive...');
            eventloop('seconds' => 5);
        }
        else
        {
            $log->info("Reponsive, assuming it is powered on.");
            last;
        }
    }

    $log->info("Waiting until we can login to the host...");
    while (1)
    {
        while (1)
        {
            Connectrec->new(
                'id'       => $hostrec->id,
                'host'     => $hostrec->id,
                'conntype' => 'ssh-persist_service',
                'connect'  => 1,
            );
            if (my $err = $@)
            {
                $log->info("Unable to login still...");
                eventloop('seconds' => 30);
            }
            else
            {
                $log->info("Successfully logged in.");
                return 1;
            }
        }
    }
}

1;
