#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::ESX::Host::Reconnect;

use strict;
use warnings;
use Carp         qw(croak);

# vmware
use VMware::VICommon;
use VMware::VIMRuntime;
use VMware::VIM25Stub;
use VMware::VILib;
use VMware::VIExt;

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use Connectrec;
use Hostrec;
use NATE::BaseException qw(:try);
use NATE::Log           qw(log_global);

use NACL::Init::Utils::VMware qw(get_esx_certificate
                                 generate_ssl_thumbprint
                                 wait_for_task
                                 rot13);

use constant is_retryable => 1;

sub execute
{
    my ($self, %args) = @_;

    my $resource = $args{'resource'};
    my $log = log_global;

    my $hostrec_resource = Hostrec->new('id' => $resource);
    my $hostrec;

    if ($hostrec_resource->simhost)
    {
         $hostrec = Hostrec->new('id' => $hostrec_resource->simhost);
    }
    else
    {
        $hostrec = $hostrec_resource;
    }

    my $host = $hostrec->id . '.' . $hostrec->{'DOMAIN'};

    $log->info("Logging in to $hostrec->{'VCENTER'} vcenter...");
    my $hostrec_vc = Hostrec->new('id' => $hostrec->{'VCENTER'});
    my $vc = Vim::login(
        'service_url' => 'https://' . $hostrec->{'VCENTER'} . '/sdk/vimService',
        'user_name'   => $hostrec_vc->default_username,
        'password'    => rot13('string' => $hostrec_vc->default_password),
    );
    $log->info("Logged in to $hostrec->{'VCENTER'}.");

    # This is just to get connection info, do not actually connect
    my $conn = Connectrec->new(
        'host'     => $hostrec,
        'template' => 'esx_ssh',
        'connect'  => 0,
    );

    my $hostSystem = $vc->find_entity_view(
        'view_type' => 'HostSystem',
        'filter'    => {'name' => $host},
    );

    my $hostConnectionState = $hostSystem->runtime->connectionState->val;
    $log->log("Host Connection State: $hostConnectionState");
    if ($hostConnectionState ne 'connected')
    {
        my $cert = get_esx_certificate("$host", $conn->username, $conn->password);
        my $ssl_thumbprint = generate_ssl_thumbprint('pem' => $cert);
        $log->info("Received SSL thumbprint for $host HostSystem.");

        my $hostConnSpec = HostConnectSpec->new(
            'userName'      => $conn->username,
            'password'      => $conn->password,
            'force'         => 1,
            'sslThumbprint' => $ssl_thumbprint,
        );

        wait_for_task(
            'message' => "Successfully reconnected the host.",
            'task'    => $hostSystem->ReconnectHost_Task('cnxSpec' => $hostConnSpec),
        );
    }
    else
    {
        $log->info("Host is connected already, exiting...");
        return 1;
    }
}

1;
