#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::KVM::Host::Ubuntu::NetworkingSetup;

use strict;
use warnings;
use Carp qw(croak);
use Data::Dumper qw(Dumper);
use IO::File;

use Net::SCP::Expect;
use Params::Validate qw(validate :types);

use Connectrec;
use NATE::Inc qw(find_file);
use NATE::Log qw(log_global);

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}
use NATE::BaseException qw(:try);

sub execute
{
    my ($self, %args) = @_;

    my $host = $args{'resource'};
    my $log = log_global;
    my %task_args = %{$self->task_args};

    my $conn = Connectrec->new(
        'id'       => $host,
        'host'     => $host,
        'template' => 'kvm_ssh',
    );

    my $output = $conn->execute(
        'command'      => ['/usr/bin/ovs-vsctl', 'list-br'],
        'check_status' => 1,
    );
    chomp($output);
    if ($output)
    {
        $log->info('Networking already setup');
        exit(0);
    }

    # Create OpenVSwitch configuration file
    my $fh = IO::File->new('openvswitch-switch.conf', 'w');
    print $fh _openvswitch_conf();
    $fh->close;

    # Create sDOT Networking Config
    $fh = IO::File->new('sdot-nwk-setup.conf', 'w');
    print $fh _sdot_nwk_setup_config();
    $fh->close;

    # Create sDOT Networking Script
    $fh = IO::File->new('sdot_nwk_config.sh', 'w');
    print $fh _sdot_nwk_setup_script();
    $fh->close;

    # Copy files over
    my $scpe = Net::SCP::Expect->new(
        'host'     => $conn->connhost,
        'user'     => $conn->username,
        'password' => $conn->password,
    );
    $conn->execute('command' => ['rm', '-rf', '/etc/sdot']);
    $conn->execute('command' => ['mkdir', '/etc/sdot']);
    $scpe->scp('openvswitch-switch.conf', '/etc/sdot');
    $scpe->scp('sdot-nwk-setup.conf', '/etc/sdot');
    $scpe->scp('sdot_nwk_config.sh', '/etc/sdot');
    $conn->execute(
        'command' => [
            'cp',
            '-f',
            '/etc/sdot/openvswitch-switch.conf',
            '/etc/init/openvswitch-switch.conf',
        ],
    );
    $conn->execute(
        'command' => [
            'ln',
            '-snf',
            '/etc/sdot/sdot-nwk-setup.conf',
            '/etc/init/sdot-nwk-setup.conf',
        ],
    );
    $conn->execute('command' => ['chmod', '755', '/etc/sdot/sdot_nwk_config.sh']);

    # Determine cluster and mgmt ifs
    $output = $conn->execute(
        'command' => [
          '/sbin/ip',
          '-o',
          'link',
        ],
        'check_status' => 1,
    );
    my @ifcfg_lines = split(/\n/, $output);

    my $ifaces = {
        'CLUS_IF1' => {
            'mac'   => $conn->hostp->{'CLUSTER_MAC1'},
            'iface' => '',
        },
        'CLUS_IF2' => {
            'mac'   => $conn->hostp->{'CLUSTER_MAC2'},
            'iface' => '',
        },
        'MGMT_IF1' => {
            'mac'   => $conn->hostp->{'MANAGEMENT_MAC1'},
            'iface' => '',
        },
        'MGMT_IF2' => {
            'mac'   => $conn->hostp->{'MANAGEMENT_MAC2'},
            'iface' => '',
        },
    };

    for my $ifcfg_line (@ifcfg_lines)
    {
        while (my($if_type, $data) = each(%{$ifaces}))
        {
            next if (! $data->{'mac'}); # may not be defined for some hosts

            my $macaddr = $data->{'mac'};
            if ($ifcfg_line =~ m/^[0-9]+: ([a-z0-9]+):.*ether $macaddr/)
            {
                $ifaces->{$if_type}->{'iface'} = $1;

                # Set UP if it is DOWN
                if ($ifcfg_line =~ m/ DOWN /)
                {
                    $log->info("Iface is down, setting up...");
                    $output = $conn->execute(
                        'command' => [
                          '/sbin/ip',
                          'link',
                          'set',
                          $ifaces->{$if_type}->{'iface'},
                          'up',
                        ],
                        'check_status' => 1,
                    );
                }
            }
        }
    }

    my @if;
    if ($ifaces->{'CLUS_IF1'}->{'iface'} && $ifaces->{'CLUS_IF2'}->{'iface'})
    {
        push(@if, ($ifaces->{'CLUS_IF1'}->{'iface'}, $ifaces->{'CLUS_IF2'}->{'iface'}));
    }
    else
    {
        croak("Unable to find any interfaces for CLUS_IF1 and CLUS_IF1!");
    }

    push(@if, $ifaces->{'MGMT_IF1'}) if ($ifaces->{'MGMT_IF1'}->{'iface'});
    push(@if, $ifaces->{'MGMT_IF2'}) if ($ifaces->{'MGMT_IF2'}->{'iface'});

    $conn->execute(
        'command' => [
            '/etc/sdot/sdot_nwk_config.sh',
            @if,
        ],
        'check_status' => 1,
    );
}

sub _openvswitch_conf
{
    my $content = <<'END_OF_CONTENT';
# vim: set ft=upstart ts=2 et:
description "Open vSwitch switch"
author "James Page <james.page@ubuntu.com"
emits openvswitch-up

start on (local-filesystems and net-device-up IFACE=lo)
stop on runlevel [!2345]

pre-start script
  (test -x /usr/sbin/ovs-vswitchd && test -x /usr/sbin/ovsdb-server) || exit 0

  . /usr/share/openvswitch/scripts/ovs-lib
  test -e /etc/default/openvswitch-switch && . /etc/default/openvswitch-switch

  if ovs_ctl load-kmod; then
    :
  else
    echo "Module has probably not been built for this kernel."
    if ! test -d /usr/share/doc/openvswitch-datapath-dkms; then
      echo "Install the openvswitch-datapath-dkms package."
    fi

    if test X"$OVS_MISSING_KMOD_OK" = Xyes; then
      # We're being invoked by the package postinst.  Do not
      # fail package installation just because the kernel module
      # is not available.
      exit 0
    fi
  fi
  set ovs_ctl start --system-id=random
  if test X"$FORCE_COREFILES" != X; then
    set "$@" --force-corefiles="$FORCE_COREFILES"
  fi
  set "$@" $OVS_CTL_OPTS
  "$@"
  BRIDGES=$(ifquery --allow ovs --list)
  if test X"$BRIDGES" != X; then
    ifup --allow ovs $BRIDGES
  fi
 initctl emit openvswitch-up
end script

post-stop script
  . /usr/share/openvswitch/scripts/ovs-lib
  test -e /etc/default/openvswitch-switch && . /etc/default/openvswitch-switch

  BRIDGES=$(ifquery --allow ovs --list)
  if test X"$BRIDGES" != X; then
    ifdown --allow ovs $BRIDGES
  fi
  ovs_ctl stop
end script
END_OF_CONTENT

    return $content;
}

sub _sdot_nwk_setup_config
{
    my $content = <<'END_OF_CONTENT';
# vim: set ft=upstart ts=2 et:
description "sDOT Network Configuration"
author "NetApp"

start on (local-filesystems and openvswitch-up and static-network-up)
stop on runlevel [!2345]

pre-start script
   /etc/sdot/sdot_nwk_config.sh enp3s0f0 enp3s0f1
end script
END_OF_CONTENT

    return $content;
}

sub _sdot_nwk_setup_script
{
    my $content = <<'END_OF_CONTENT';
#!/bin/sh -x

usage() {
    echo "usage: "
    echo "$1 <CLUS_IF1> <CLUS_IF2> [<DATA_IF1> <DATA_IF2>] [<HOST_IP> <HOST_MASKLEN> <HOST_GW>]"
    echo ""
    echo "If the host does not have a separate management interface and you wish to configure"
    echo "a management IP on the host provide the host IP,mask and gateway. The IP will be "
    echo "configured on one of the openvswitch bridges. If you have a management interface then"
    echo "it is recommended not to configure any IP on the openvswitch bridge."
}

exec_cmd() {
OUTPUT=`$*`
if [ $? -ne 0 ] ; then
    echo "CMD : \"$*\" : FAILED : $OUTPUT"
    exit 1
fi
}

do_if_down() {

while [ "$1" != "" ]; do
    ifconfig $1 0 down
    shift
done
}

do_if_up() {

while [ "$1" != "" ]; do
    PORT=$1
    shift
    MTU=$1
    ip link set $PORT mtu $MTU up
    shift
done
}

create_bond() {
BOND_EXISTS=`ovs-vsctl show | grep $2`
if [ -z "${BOND_EXISTS}" ]; then
    do_if_down $3 $4
    sleep 1
    ${EXEC_CMD} ovs-vsctl add-bond $1 $2 $3 $4 bond_mode=balance-slb lacp=active other_config:lacp-time=fast
fi
}

create_br() {
BR_EXISTS=`ovs-vsctl show | grep $1`
if [ -z "${BR_EXISTS}" ]; then
    ${EXEC_CMD} ovs-vsctl add-br $1
fi
}

if [ "$#" -ne 2 -a "$#" -ne 4 -a "$#" -ne 5 -a "$#" -ne 7 ]; then
    usage $0
    exit 1
fi

CLUS_IF1=$1
CLUS_IF2=$2
DATA_IF1=$3
DATA_IF2=$4
HOST_IP=$5
HOST_MASKLEN=$6
HOST_GW=$7
CONFIG_TYPE="4-uplinks"

if [ "$#" -eq 5 ] ; then
    DATA_IF1=""
    DATA_IF2=""
    HOST_IP=$3
    HOST_MASKLEN=$4
    HOST_GW=$5
fi

if [ -z "${DATA_IF1}" ]; then
    CONFIG_TYPE="2-uplinks"
fi

#echo "Config Type : $CONFIG_TYPE"
#echo "Cluster Interface 1 : $CLUS_IF1"
#echo "Cluster Interface 2 : $CLUS_IF2"
#echo "Data Interface 1 : $DATA_IF1"
#echo "Data Interface 2 : $DATA_IF2"
#echo "HOST IP : $HOST_IP"
#echo "HOST Mask length : $HOST_MASKLEN"
#echo "HOST Gateway : $HOST_GW"

CLUS_BR="ontap-internal"
DATA_BR="ontap-external"
CLUS_BOND="bond-internal"
DATA_BOND="bond-external"
EXEC_CMD=exec_cmd

#If the config type is 2 uplinks then
#we only need to create 1 bridge
if [ "$CONFIG_TYPE" = "2-uplinks" ] ; then
    CLUS_BR="ontap-br"
    CLUS_BOND="bond-ontap"
    DATA_BR=$CLUS_BR
    DATA_BOND=$CLUS_BOND
fi

#Create the Cluster network bridge
create_br ${CLUS_BR}
create_bond ${CLUS_BR} ${CLUS_BOND} ${CLUS_IF1} ${CLUS_IF2}
sleep 2
do_if_up ${CLUS_IF1} 9000 ${CLUS_IF2} 9000

#if the config type is 4 uplinks then
#create the second bridge
if [ "$CONFIG_TYPE" = "4-uplinks" ] ; then
    create_br ${DATA_BR}
    create_bond ${DATA_BR} ${DATA_BOND} ${DATA_IF1} ${DATA_IF2}
    sleep 2
    do_if_up ${DATA_IF1} 1500 ${DATA_IF2} 1500
fi

#If Host IP is specified then create the IP
# address on the Data bridge, for 2 uplink config
# Data bridge is same as Cluster bridge
if [ ! -z "$HOST_IP" ] ; then
    sleep 1
    ${EXEC_CMD} ip addr add ${HOST_IP}/${HOST_MASKLEN} broadcast + dev ${DATA_BR}
    ${EXEC_CMD} ip route add default via ${HOST_GW} dev ${DATA_BR}
fi
END_OF_CONTENT

    return $content;
}

1;
