#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::Node::DiskAssign;

use strict;
use warnings;

use NATE::Log qw(log_global);

use NACL::C::Node;
use NACL::C::StorageDisk;

use Moo;
extends qw(NATE::Init::Task);

# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}
use NATE::BaseException qw(:try);

sub execute
{
    my($self, %args) = @_;

    my $log = log_global;

    my %task_args = %{$self->task_args};

    my $ci    = NACL::C::Node->find;
    my @nodes = NACL::C::Node->find('command_interface' => $ci);

    if (scalar(@nodes) == 1)
    {
        $log->comment("Single node cluster; assigning all disks locally");
        my $node_name = $nodes[0]->node;
        my @disks = NACL::CS::StorageDisk->fetch(
            'command_interface' => $ci,
            'filter'            => {'nodelist' => [$node_name], 'container-type' => 'unassigned'},
            'requested_fields'  => ['location', 'initiator', 'diskpathnames'],
            'allow_empty'       => 1,
        );

        if (! scalar(@disks))
        {
            $log->comment("No unasssigned disks available");
        }

        for my $disk (@disks)
        {
            try {
                NACL::C::StorageDisk->assign(
                    'command_interface' => $ci,
                    'disk'              => $disk->disk,
                    'owner'             => $node_name,
                );
            } catch NACL::APISet::Exceptions::ResponseException with {
                my $ex = shift;
                $log->warn("Disk assignment failed\n" . $ex->text);
            };
        }
    }
    else
    {
        for my $node (@nodes)
        {
            my $node_name = $node->node;
            my $partner = NACL::STask::Node->get_partner_obj('command_interface' => $node);
            my $partner_name = $partner->name;

            # Get the unassigned disks for the local & the partner node
            my @disks = NACL::CS::StorageDisk->fetch(
                'command_interface' => $ci,
                'filter'            => {'nodelist' => [$node_name, $partner_name], 'container-type' => 'unassigned'},
                'requested_fields'  => ['location', 'initiator', 'diskpathnames'],
                'allow_empty'       => 1,
            );

            if (! scalar(@disks))
            {
                $log->comment("No unasssigned disks remaining for $node_name, $partner_name");
            }

            # NACL::CS::StorageDisk does not allow filtering on location, so
            # split the full list up based on location
            my (@local_disks, @partner_disks);
            for my $disk (@disks)
            {
                if ($disk->location eq $node_name)
                {
                    push(@local_disks, $disk);
                }
                if ($disk->location eq $partner_name)
                {
                    push(@partner_disks, $disk);
                }
            }

            my $index = 0;
            while (($index < scalar(@local_disks)) && ($index < scalar(@partner_disks)))
            {
                # Take an unassigned disk from each node (local & partner)
                # Alternate between assigning them both to the local, or to the partner, node
                if ($index % 2)
                {
                    # Assign the pair to the partner node
                    try {
                        # Disks attached to local node must be placed in partner's Pool 1
                        NACL::C::StorageDisk->assign(
                            'command_interface' => $ci,
                            'disk'              => $local_disks[$index]->disk,
                            'owner'             => $partner_name,
                            'pool'              => 1,
                        );
                    } catch NACL::APISet::Exceptions::ResponseException with {
                        my $ex = shift;
                        $log->warn("Disk assignment failed\n" . $ex->text);
                    };

                    # Disks attached to partner node must be placed in partner's Pool 0
                    try {
                        NACL::C::StorageDisk->assign(
                            'command_interface' => $ci,
                            'disk'              => $partner_disks[$index]->disk,
                            'owner'             => $partner_name,
                            'pool'              => 0,
                        );
                    } catch NACL::APISet::Exceptions::ResponseException with {
                        my $ex = shift;
                        $log->warn("Disk assignment failed\n" . $ex->text);
                    };
                }
                else
                {
                    # Assign the pair to the local node
                    try {
                        # Disks attached to local node must be placed in local's Pool 0
                        NACL::C::StorageDisk->assign(
                            'command_interface' => $ci,
                            'disk'              => $local_disks[$index]->disk,
                            'owner'             => $node_name,
                            'pool'              => 0,
                        );
                    } catch NACL::APISet::Exceptions::ResponseException with {
                        my $ex = shift;
                        $log->warn("Disk assignment failed\n" . $ex->text);
                    };

                    # Disks attached to partner node must be placed in local's Pool 1
                    try {
                        NACL::C::StorageDisk->assign(
                           'command_interface' => $ci,
                           'disk'              => $partner_disks[$index]->disk,
                           'owner'             => $node_name,
                           'pool'              => 1,
                        );
                    } catch NACL::APISet::Exceptions::ResponseException with {
                        my $ex = shift;
                        $log->warn("Disk assignment failed\n" . $ex->text);
                    };
                }

                $index++;
            }
        }
    }

    # Display the final disk assignments
    $log->comment("Final disk assignments");
    NACL::CS::StorageDisk->fetch(
        'command_interface' => $ci,
        'requested_fields'  => ['location', 'container-type', 'owner', 'pool', 'initiator', 'diskpathnames'],
        'allow_empty'       => 1,
    );
}

1;
