#   Copyright (c) 2016 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::Vsim::Rm;

use strict;
use warnings;
use NATE::Log qw(log_global);
use Data::Dumper;
use NACL::Init::Tasks::VsimUtils qw(get_vsim_details_from_node_name prepare_client_apiset);
use NACL::C::Node;
use NATE::BaseException qw(try);
use Carp qw(confess);

use Moo;
extends qw(NATE::Init::Task);

use constant defaults => {
    VSIM_TOOL   => "/usr/software/test/bin/vsim",
    VSPC_SCRIPT => "/usr/software/test/bin/vSPC.py",
};

sub default_run_scheme {
    my ($self, %args) = @_;

    my $resources = $self->resources;
    my ( @res, @resources_run_scheme, @groups );

    # loop through the resources and make [[[$resources]]]
    foreach my $resource (@{$resources}) {
        push (@res, $resource);
    }
    push (@resources_run_scheme, \@res);
    push (@groups, \@resources_run_scheme);

    return \@groups;
}

sub execute {

    my ($self, %args) = @_;

    my $node = $args{resource};
    my @nodes = @{$node};

    my ($task, %task_args);
    $task = $self->task_args;
    %task_args = %$task;

    my $Vsim_tool = $task->{VSIM_TOOL};
    my $Vspc_script = $task->{VSPC_SCRIPT};

    log_global()->comment("Binfo and Rm all vsims");

    # flag to run rm or not
    my %vsims_exist = ();
    my %group_names;
    log_global()->comment("Preparing client APISet");
    my $c_api = prepare_client_apiset();

    foreach my $node (@nodes) {
        my $group_name;
        # identify the vsim user to operate on for this
        my $vsim_details = get_vsim_details_from_node_name($node);
        if(!defined $vsim_details) {
            NATE::BaseException->throw("bad happened while trying to obtain vsim details: $vsim_details");
        }
        my $vsim_user = $vsim_details->{user};
        if(!defined $vsim_user) {
            NATE::BaseException->throw("bad happened while trying to obtain vsim user details: $vsim_user");
        }
        my $vsim_dir = "/u/".$vsim_user."/vsims";
        if(!defined $vsim_dir) {
            NATE::BaseException->throw("bad happened while trying to obtain vsim directory: $vsim_dir");
        }
        my $sudo_user = "/usr/software/bin/sudo -i -u ".$vsim_details->{user};
        if(!defined $sudo_user) {
            NATE::BaseException->throw("bad happened while trying to obtain sudo user info: $sudo_user");
        }

        my $normalized_node_name = $node;
        $normalized_node_name =~ s/-/_/g;

        log_global()->comment("binfo to check if vsims exist");
        my $status = 0;
        my ($output) = $c_api->execute_raw_command(command => "$sudo_user $Vsim_tool binfo -smoke -simdir $vsim_dir $vsim_details->{id}",
                                                    get_status => \$status);
        if ($status ne 0) {
            if ($output =~ "does not exist") {
                log_global()->comment("Vsims do not exist. Hence skipping rm");
                $vsims_exist{$node}{deleted} = 1;
            } else {
                NATE::BaseException->throw("bad happened while trying to do binfo");
            }
        } else {
            foreach my $line (split(/\n+/, $output)) {
                chomp($line);
                # If both Mcgroup and Hagroup are present then only consider Mcgroup
                if($line =~ /Mcgroup:\s*(.+)$/) {
                    $group_name = $1;
                    last;
                } elsif ($line =~ /Hagroup:\s*(.+)$/) {
                    print Dumper($group_name);
                    $group_name = $1;
                    last;
                }
            }

            if (!defined($group_name)) {
                $group_name = $vsim_details->{id};
            }

            if(defined($group_name)) {
                $group_names{$group_name}{user} = $vsim_user;
                if(defined($group_names{$group_name}{vsims})) {
                    push(@{$group_names{$group_name}{vsims}}, $node);
                } else {
                    $group_names{$group_name}{vsims} = [$node];
                }
            }
        }
        print Dumper(%group_names);
    }

    # identify the vsim user to operate on for this
    my $vsim_details = get_vsim_details_from_node_name($nodes[0]);
    my $vsim_user = $vsim_details->{user};
    my $sudo_user = "/usr/software/bin/sudo -H -u ".$vsim_details->{user};

    foreach my $key (keys %group_names) {
        my $status = 0;
        my ($output) = $c_api->execute_raw_command(command => "$sudo_user $Vsim_tool rm -f -smoke $key",
                           get_status => \$status);
        # check if hagroup is part of mcgroup
        if ((my $mc_key) = $output =~ /part of mcgroup (\w+)/){
            # remove the mcgroup
            $output = $c_api->execute_raw_command(command => "$sudo_user $Vsim_tool rm -f -smoke $mc_key",
                           get_status => \$status);
        }
        if ($status ne 0) {
            if ($output =~ /does not exist/i) {
                my $ha_nodes = $group_names{$key}{vsims};
                foreach my $n (@{$ha_nodes}) {
                    my $node_obj = NACL::C::Node->new(node => $n);
                    # grap the vsim details for this node
                    my $vsim_details = get_vsim_details_from_node_name($n);
                    my $vsim_user = $vsim_details->{user};
                    my $sudo_user = "/usr/software/bin/sudo -H -u ".$vsim_details->{user};
                    # skip if we dont have the blade or instance info
                    next if (!$vsim_details->{blade} or !$vsim_details->{instance});

                    # remove the vsim manually. (pass in the vspc flag so that it cleans up the vspc bound ports if there are any)
                    my $vspc_server = $node_obj->hostrec()->{VSPC_SERVER};
                    ($output) = $c_api->execute_raw_command(command => "$sudo_user $Vsim_tool rm -vspc $vspc_server " . $vsim_details->{blade} . $vsim_details->{instance} . " -f",
                                                         get_status => \$status);
                    if ($status ne 0 and $output !~ /does not exist/i) {
                        NATE::BaseException->throw("Encountered a bad status while removing the VSims");
                    }

                    # remove the vsim by its name just in case
                    ($output) = $c_api->execute_raw_command(command => "$sudo_user $Vsim_tool rm $n  -f",
                                                        get_status => \$status);
                    if ($status ne 0 and $output !~ /does not exist/i) {
                        NATE::BaseException->throw("Encountered a bad status while removing the VSims");
                    }
                }
            } else {
                NATE::BaseException->throw("Encountered a bad status while removing the VSims");
            }
        }
    }

    foreach my $node_name (@nodes) {
        my $node_obj = NACL::C::Node->new(node => $node_name);
        my $vspc_server = $node_obj->hostrec()->{VSPC_SERVER};
        my $vspc_port = $node_obj->hostrec()->{VSPC_PORT};
        my $return = 0;
        my $output = "";
        # Check if vspc port is deleted or not.
        $output = $c_api->execute_raw_command(command => "$Vspc_script -q -c $vspc_server | grep $vspc_port", get_status => \$return);
        $output = $c_api->execute_raw_command(command => "$Vspc_script -d cns -n $node_name -c $vspc_server", get_status => \$return);
        $output = $c_api->execute_raw_command(command => "$Vspc_script -d gdb -n $node_name -c $vspc_server", get_status => \$return);
        if ($return ne 0) {
            NATE::BaseException->throw("vsim rm has not successfully removed vspc port for vsim $node_name from vspc server $vspc_server");
        }
    }
}

1;
