# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Object representing an individual received autosupport message
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::MTask::AutosupportReceiver::Message;

use strict;
use warnings;

use IPC::Open3 qw(open3);
use Archive::Tar ();
use Params::Validate qw(validate SCALAR);
use IO::Handle ();

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::BaseException ();
use NATE::Util qw(quote_argv_sh);

=head1 NAME

NACL::MTask::AutosupportReceiver::Message

=head1 SYNOPSIS

    # These objects come from a NACL::MTask::AutosupportReceiver
    # instance.  So the horrendously long name of this package doesn't
    # affect you :)

        my $message = $ar->wait_for_message(timeout=>-1);

    # Get the path to the message in packed form (7-zip file, tgz
    # file, etc.) as sent.

        $message->packed_path()

    # Return the full contents of a particular file within the packed
    # archive.

        my $options = $message->get_file(name=>"OPTIONS.txt");
        if ($options =~ /nfs.mount_rootonly\s+off/) {
            die "oh noes! nfs.mount_rootonly was off";
        }

=head1 ATTRIBUTES

=head2 packed_path

The path to the body of the message in packed form (7-zip file, tgz
file, etc.)

=cut

use Class::MethodMaker [

    # Private.
    new => [ '-hash', 'new' ],

    # Public, documented above.
    scalar => 'packed_path',
];

=head1 METHODS

=head2 get_file

   my $contents = $message->get_file(name=>$filename)

Return the contents of a file within the archive that was sent as this
autosupport message.

=cut

sub get_file {
    $Log->enter() if $may_enter;
    my $self        = shift;
    my %opts        = validate @_, { name => { type => SCALAR }, };
    my $packed_path = $self->packed_path;

    if ( $packed_path =~ /\.7z$/ ) {

        # We don't have a way to deal with 7-zip archives with perl
        # modules, so we drive the "7za" command line program.

        my $cmd = quote_argv_sh( "7za", "e", $packed_path, $opts{name}, "-so" );

        my $in  = IO::Handle->new();
        my $out = IO::Handle->new();
        my $err = IO::Handle->new();
        my $pid = open3( $in, $out, $err, $cmd );
        my $out_text;
        my $err_text;
        {
            local $/ = undef;

            # We assume the error text is short enough that we can
            # just read all the output followed by all the error text.

            $out_text = <$out>;
            $err_text = <$err>;
        }

        waitpid( $pid, 0 );
        my $status = $?;
        if ( $status != 0 ) {
            NATE::BaseException->throw(
                    "7za command exited with non-zero status\n"
                  . "  Status: "
                  . decode_status($status) . "\n"
                  . "  Command: $cmd\n"
                  . "  Stderr Output:\n"
                  . $err_text );
        }
        elsif ( $err_text !~ /Everything is Ok/ ) {
            NATE::BaseException->throw(
                    "7za command did not print 'Everything is Ok'\n"
                  . "  Command: $cmd\n"
                  . "  Stderr Output:\n"
                  . $err_text );
        }
        $Log->exit() if $may_exit;
        return $out_text;
    }
    elsif ( $packed_path =~ /\.tgz/ ) {
        my $tar = Archive::Tar->new();
        $tar->read( $packed_path, 1, { filter => quotemeta( $opts{name} ) } );
        $Log->exit() if $may_exit;
        return $tar->get_content( $opts{name} );
    }
    else {
        NATE::BaseException->throw(
            "Don't know how to get_file from a file named '$packed_path'");
    }
}

sub decode_status ( $ ) {
    my ($status) = @_;
    if ( !defined($status) ) {
        return "with unknown status";
    }
    elsif ( $status & 0xff ) {
        my $msg = "signal " . ( $status & 0x7f );
        $msg .= " (core dumped)" if ( $status & 0x80 );
        return $msg;
    }
    else {
        return "status " . ( $status >> 8 );
    }
}

1;
