# $Id:
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary CIFS BranchCache Task Module
## @author  rahula@netapp.com , dl-nacl-dev@netapp.com
## @status public
## @pod here

=head1 NAME

NACL::MTask::BranchCache

=head1 DESCRIPTION

C<NACL::MTask::BranchCache> provides a number of well-defined but
potentially complex or multi-step methods related to CIFS BranchCache
functionality on ONTAP.

The MTask will mainly be concerned with 7Mode CIFS setup in the filer ,providing
branchcache  settings on both filer/client , mounting the CIFS shares , packet
tracing & downloading the files from the shares using the branchcache concept.

=head1 ATTRIBUTES

=head2 filer_command_interface

(Required) Represents NACL::C::CommandInterface::ONTAP the filer to which to send commands.

=head2 client_command_interface

(Required) Represents NACL::C::Client the windows client to which to send commands.

=head2 cifs_shares

(Required) Represents the NACL::STask::VserverCifsShare instance created in filer_setup

=head2 pcktt_obj

(Required) Represents the NACL::MTask::PacketCapture instance used for packet tracing .

=head2 mount

(Required) Represents the NACL::STask::Client::Mount instance used for mapping the shares.

=head2 file

(Required) Represents the NACL::C::Mkfile instance used for file creation .

=head2 smbbytesserver

(Required) Represents the SMB Bytesize returned from the server after download .

=head2 smbbytescache

(Required) Represents the SMB Bytesize returned from the cache after download .

=head2 retbytesserver

(Required) Represents the Retrieval Bytesize returned from the server after download .

=head2 retbytescache

(Required) Represents the Retrieval Bytesize returned from the cache after download .

=head1 METHODS

=head2 filer_setup

     # method call description

     $Filer_setup = NACL::MTask::BranchCache->filer_setup(
            'filer_command_interface'       => $Node,
            'domain-user'                   => $User,
            'domain-pass'                   => $Password,
            'domain'                        => $Domain,
            'share-name'                    => $Sharename,
            path                            => $Path,
            'auth-type'                     => "ad",
            'branchcache'                   => "1",
            'passwd-group'                  => "yes",
            'nacltask_if_cifs_setup'        => "reuse",
            'nacltask_if_cifs_share_exists' => "purge",
            'nacltask_if_file_exists'       => 'reuse',
        );

(Class method) This method should be called only for 7Mode CIFS setup & shares
creation with branchcache functionality . Given cifs server name,domain-username,
domain-password , cifs setup can be configured . The options for enabling
SMB protocol & branchcache in the filer is also performed . The next step is to
create shares on the filer for mapping with the specified share-name &
share-path . This method provides additional services beyond what's inherent to
the product's cifs setup & shares creation commands, as controlled and described
by the new  "if_exists" option .

  #Specifying the options for SMB & branchcache functionality

1. Sets cifs.smb2.enable to on , for enabling SMB protocol .
2. Sets cifs.smb2_1.branch_cache.enable to on , for enabling branch cache .
3. Set  the value for cifs.smb2_1.branch_cache.hash_time_out .

=over

=item Options

=over

=item C<< nacltask_if_cifs_setup=>$action >>

(Optional) specifies an action to be taken if the cifs setup already exists.

     $action
    - if set to  "die"(default), an exception is raised.
    - if set to "reuse", no action is performed, and return the task object
      referring to the corresponding cifs setup which is done.
    - if set to "purge", terminating the cifs protocol before creating it.

=item C<< nacltask_if_cifs_share_exists=>$action >>

(Optional) specifies an action to be taken if the cifs setup already exists.

     $action
    - if set to  "die"(default), an exception is raised.
    - if set to "reuse", no action is performed, and return the task object
      referring to the corresponding cifs share which is existing.
    - if set to "purge", deleting the cifs shares which is present

=item C<< filer_command_interface => $Node >>

(Required) specifies the command interface used which is filer object in this case .

=item C<< cifs-server => $Cifs_server >>

(Optional) specifies the cifs-server name required for cifs setup.

=item C<< domain-user => $User >>

(Required) specifies the domain-user name required for cifs setup authentication.

=item C<< domain-pass  => $Password >>

(Required) specifies the domain-password required for cifs setup authentication.

=item C<< domain=>$domain >>

(Required) specifies the domain-hostname required for cifs setup authentication.

=item C<< share-name => $Sharename >>

(Required) specifies the name of the share to be created .

=item C<< path => $path >>

(Required) specifies the path for share creation .

=item C<< bchash_time_out=>$bc_hash_timeout >>

(Optional) The time-out value for the option cifs.smb2_1.branch_cache.hash_time_out . Default value is 3600.

=item C<< workgroup=>$workgroup >>

(Optional) The workgroup name to be used for cifs setup

=item C<< auth-type=>$type >>

(Optional) The authentication type to be used for cifs setup

=item C<<  _was_created => \$Filer_setup_done >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the filer setup has been done successfully.
This is necessary to determine whether to call the purge related to filer_setup method .

=back

=item command_interface, apiset_must, apiset_should, method-timeout,
all the other options supported by ,
setup_7mode method in CIFS.pm  See L<< NACL::MTask::CIFS->setup_7mode|lib-NACL-MTask-CIFS-pm/setup_7mode >>
create method in VserverCifsShare.pm See L<< NACL::STask::VserverCifsShare->create|lib-NACL-STask-VserverCifsShare-pm/create >>
mkfile method in Mkfile.pm See L<< NACL::C::Mkfile->mkfile|lib-NACL-C-Mkfile-pm/mkfile >>
option method in Options.pm See L<< NACL::C::Options->option|lib-NACL-C-Options-pm/option >>
mkfile method in NACL::C::Mkfile See L<< NACL::C::Mkfile->mkfile|lib-NACL-C-Mkfile-pm/mkfile >>

=back

=cut

package NACL::MTask::BranchCache;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use base qw(NACL::MTask::MTask NACL::STask::STask);

use NACL::C::Options;
use NACL::C::Mkfile;
use NACL::C::VolumeFile;
use NACL::MTask::CIFS;
use NACL::MTask::PacketCapture;
use NACL::STask::VserverCifsShare;
use NACL::STask::Client::Mount;
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ResponseException;
use Params::Validate qw/validate_with SCALAR ARRAYREF OBJECT HASHREF SCALARREF/;

use Class::MethodMaker [
    new => [ '-hash', 'new' ],
    scalar => [ { -type => 'NACL::STask::VserverCifsShare' }, 'cifs_shares' ],
    scalar => [ { -type => 'NACL::STask::Client::Mount' },    'mount' ],
    scalar => 'smbbytesserver',
    scalar => 'retbytesserver',
    scalar => 'smbbytescache',
    scalar => 'retbytescache',
    scalar => '_cifs_status',
    scalar => '_shares_status',
    scalar => '_route_status',
    scalar => '_mount_status',
    scalar => '_file_status',
    scalar => '_options_status',
    scalar => '_client_file',
    scalar => '_copy_status',
    scalar => '_file',
    scalar => '_filer_ip',
    scalar => '_setflag_status',
    scalar => [
        { -type => 'NACL::C::CommandInterface::ONTAP' },
        'filer_command_interface',
    ],
    scalar => [ { -type => 'NACL::C::Client' }, 'client_command_interface', ],

#scalar => '_pckt_status',        #|  commented as we are not using
#scalar => '_captured_output',    #|  packet capture for validation
#scalar => [ { -type => 'NACL::MTask::PacketCapture' },    'pcktt_obj' ], #commented as we are not using packet capture for validation
];

#===============================================================================
# Name        : filer_setup
# Description : This method should be called only for 7Mode CIFS setup & shares
#               creation with branchcache functionality . Given cifs server
#               name,domain-username , domain-password , cifs setup can be
#               configured . The options for enabling  SMB protocol &
#               branchcache in the filer is also performed . The next step is to
#               create shares on the filer for mapping with the specified
#               share-name & share-path .
#===============================================================================
sub filer_setup {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    NATE::BaseException->throw(
        "Setup can only be invoked with a proper package call")
      if ref $pkg;
    my $obj;
    my %shares_opts;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            filer_command_interface =>
              { type => OBJECT, isa => 'NACL::C::CommandInterface::ONTAP' },
            _was_created => { type => SCALARREF, optional => 1 },
        },
        allow_extra => 1,
    );
    my $command_interface = delete $opts{filer_command_interface};
    my $hash_bc_timeout = $opts{'bchash_time_out'} || "3600";

    $shares_opts{'share-name'}  = delete $opts{'share-name'};
    $shares_opts{'path'}        = delete $opts{'path'};
    $shares_opts{'branchcache'} = '1';
    $shares_opts{'nacltask_if_exists'} =
      delete $opts{'nacltask_if_cifs_share_exists'};

    my $nacltask_if_cifs_setup  = delete $opts{nacltask_if_cifs_setup};
    my $nacltask_if_file_exists = delete $opts{nacltask_if_file_exists};
    my $was_created             = delete $opts{_was_created};
    my ( $option_flag, $cifs_created, $shares_was_created, $cifs_shares );
    $$was_created = 0;

    NATE::BaseException->throw(
            "This method should be called only for 7Mode cifs setup"
          . " & shares creation for branchcache" )
      if ( $command_interface->mode() ne '7Mode' );

    # Enabling the SMB & Branchcache related options .
    foreach my $option ( qw(cifs.smb2.enable cifs.smb2_1.branch_cache.enable )) {
        NACL::C::Options->option(
            command_interface => $command_interface,
            'option-name'     => $option,
            'option-value'    => 'on'
        );
    }
    $option_flag = 1;
    NACL::C::Options->option(
        command_interface => $command_interface,
        'option-name'     => "cifs.smb2_1.branch_cache.hash_time_out",
        'option-value'    => $hash_bc_timeout
    );

    # Performing Cifs setup in 7Mode using Active Directory authentication
    NACL::MTask::CIFS->setup_7Mode(
        command_interface => $command_interface,
        %opts,
        if_exists => $nacltask_if_cifs_setup
    );
    $cifs_created = '1';

    #Creating shares with branchcache support
    $shares_was_created = 0;
    $cifs_shares        = NACL::STask::VserverCifsShare->create(
        command_interface => $command_interface,
        %shares_opts,
        _was_created => \$shares_was_created,
    );

    #Setting the smb_bc_min_file_age flag to zero
    my $apiset = $command_interface->apiset(
        category  => "Node",
        interface => "CLI",
        set       => "7Mode",
    );
    $apiset->setflag( flag => "smb_bc_min_file_age", value => '0' );
    my $setflag_status = 1;

    $$was_created = 1;

    $obj = $pkg->new(
        filer_command_interface => $command_interface,
        cifs_shares             => $cifs_shares,
        _options_status         => $option_flag,
        _cifs_status            => $cifs_created,
        _shares_status          => $shares_was_created,
        _setflag_status         => $setflag_status,
    );

    $Log->exit() if $may_exit;
    return $obj;
}

=head2 client_setup

       # Method call description

       NACL::MTask::BranchCache->client_setup(
            'client_command_interface'     => $client,
            'mode'                         => "DISTRIBUTED",
            'share-name'                   => $Sharename,
            'filer_ip'                     => $Ipaddr,
            user                           => $User,
            password                       => $Password,
            mask                           => "255.255.255.255",
            gateway                        => "10.72.64.113",
            nacltask_if_mapping_exists     => "die",
            nacltask_if_branchcache_exists => "reuse",
            nacltask_if_route_exists       => "reuse",
            _was_created                   => \$client_setup_done,
        );

(Class method) This method is used for .,
               1)Mapping the shares which is created on the filer.
               2)Checking the branchcache functionality on Windows7 client .
               3)Adding the route information to increase the latency .

=over

=item Options

=over

=item C<< nacltask_if_mapping_exists=>$action >>

(Optional) specifies an action to be taken if the drive is already mapped .

     $action
    - if set to  "die"(default), an exception is raised.
    - if set to  "reuse" , use the existing mapped drive .
    - if set to  "purge" delete the existing mapped drive .

=item C<< nacltask_if_branchcache_exists=>$action >>

(Optional) specifies an action to be taken if the branchcache already exists.

     $action
     - if set to  "die"(default), an exception is raised.
     - if set to  "reuse" , use the existing branchcache settings .
     - if set to  "purge" , delete the existing branchcache setup & create it again.

=item C<< nacltask_if_route_exists=>$action >>

(Optional) specifies an action to be taken if the branchcache already exists.

     $action
    - if set to  "die"(default), an exception is raised.
    - if set to  "reuse" , use the existing route information .
    - if set to  "purge" , delete the existing branchcache setup & create it again.

=item C<< client_command_interface=>$Client >>

(Required) specifies the command interface to use .

=item C<< mode=>$value >>

(Required) specifies the type of branchcache caching mode to be used in the client .
           default will be DISTRIBUTED.(DISTRIBUTED|HOSTEDSERVER|HOSTEDCLIENT)

=item C<< share-name=>$Sharename >>

(Required) specifies the share name created on the filer which is to be mapped .

=item C<< filer_ip=>$ipaddress >>

(Required) specifies the filer IP address which is to be added to the route information on the client to increase the latency.

=item C<< user=>$user >>

(Required) specifies the User to be used for mapping the shares.

=item C<< password=>$passwd >>

(Required) specifies the password to be used for mapping the shares.

=item C<< mask=>$mask >>

(Required) specifies the subnet mask which is to be added to the route information on the client to increase the latency.

=item C<< gateway=>$gateway >>

(Required) specifies the gateway which is to be added to the route information on the client to increase the latency.

=item C<< location=>$location >>

(Optional) specifies the location to be given if the mode = HOSTEDCLIENT .

=item C<< clientauthentication=>$auth >>

(Optional) specifies the clientauthentication to be given if the mode = HOSTEDSERVER .

=item C<<  _was_created  => \$client_setup_done >>
(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the client setup has been done successfully.
This is necessary to determine whether to call the purge related to client_setup method .

=back

=item command_interface, apiset_must, apiset_should, method-timeout,
all the other options supported by ,
all methods in netsh.pm See L<< NACL::APISet::netsh|lib-NACL-APISet-netsh-pm/all methods >>
create method in Mount.pm See L<< NACL::STask::Client::Mount->create|lib-NACL-STask-Client-Mount-pm/create >>
route_add method in route.pm See L<< NACL::APISet::route->route_add|lib-NACL-APISet-route-pm/route_add method >>

=back

=cut

#===============================================================================
# Name        : client_setup
# Description : This method is used for .,
#               1)Mapping the shares which is created on the filer.
#               2)Checking the branchcache functionality on Windows7 client .
#               3)Adding the route information to increase the latency .
#===============================================================================

sub client_setup {

    $Log->enter() if $may_enter;
    my $pkg = shift;
    NATE::BaseException->throw(
        "Setup can only be invoked with a proper package call")
      if ref $pkg;

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            client_command_interface =>
              { type => OBJECT, isa => "NACL::C::Client" },
            _was_created => { type => SCALARREF, optional => 1 },
        },
        allow_extra => 1,
    );

    my %bc_opts;
    my $ci          = delete $opts{client_command_interface};
    my $mask        = $opts{'mask'};
    my $gateway     = $opts{'gateway'};
    my $mode        = $opts{'mode'} || "DISTRIBUTED";
    my $was_created = delete $opts{_was_created};
    $$was_created = 0;
    my $mount_was_created = 0;

    my $mount = NACL::STask::Client::Mount->create(
        command_interface  => $ci,
        server             => $opts{'filer_ip'},
        share              => $opts{'share-name'},
        user               => $opts{'user'},
        password           => $opts{'password'},
        nacltask_if_exists => $opts{nacltask_if_mapping_exists},
        _was_created       => \$mount_was_created,
    );

    my $apiset =
      $ci->apiset( category => "Host", interface => "CLI", set => "Windows", );

    my $response = $apiset->netsh_branchcache_show_status();
    my $output   = $response->get_parsed_output();
    my $route_created;

    $bc_opts{'mode'} = $mode;
    if ( $bc_opts{'mode'} =~ /hostedclient/i ) {
        if ( defined( $opts{'location'} ) ) {
            $bc_opts{'location'} = $opts{'location'};
        }
        else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Error :: Location not passed as mode = HOSTEDCLIENT");
        }
    }
    elsif ( $bc_opts{'mode'} =~ /hostedserver/i ) {
        if ( defined( $opts{'clientauthentication'} ) ) {
            $bc_opts{'clientauthentication'} = $opts{'clientauthentication'};
        }
        else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
"Error :: Client Authentication not passed as mode = HOSTEDSERVER"
            );
        }
    }

    if ( $output->[0]{status} =~ /running/i ) {
        if ( $opts{nacltask_if_branchcache_exists} =~ /die/ ) {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw("Branchcache already running");
        }
        elsif ( $opts{nacltask_if_branchcache_exists} =~ /purge/ ) {
            $apiset->netsh_branchcache_reset();
            $apiset->netsh_branchcache_set_service( %bc_opts, );
        }
        elsif ( $opts{nacltask_if_branchcache_exists} =~ /reuse/ ) {
            $Log->comment(
"The branchcache is already running with mode = $output->[0]{'mode'}"
            );
        }
    }
    elsif ( $output->[0]{status} =~ /stopped/i ) {
        $apiset->netsh_branchcache_reset();
        $apiset->netsh_branchcache_set_service( %bc_opts, );
    }

    try {
        $apiset->route_add(
            'mask'        => $mask,
            'destination' => $opts{'filer_ip'},
            'gateway'     => $gateway,
        );
        $route_created = 1;
    }
    catch NACL::APISet::Exceptions::CommandFailedException with {
        my $err_msg = shift;
        if ( $err_msg->text() =~
            /The route addition failed: The object already exists.*/ )
        {
            if ( $opts{nacltask_if_route_exists} =~ /die/ ) {
                $Log->exit() if $may_exit;
                $err_msg->throw();
            }
            elsif ( $opts{nacltask_if_route_exists} =~ /purge/ ) {
                $apiset->route_delete( 'destination' => $opts{'filer_ip'} );
                $apiset->route_add(
                    'mask'        => $mask,
                    'destination' => $opts{'filer_ip'},
                    'gateway'     => $gateway,
                );
            }
            elsif ( $opts{nacltask_if_route_exists} =~ /reuse/ ) {
                $Log->comment("Route already exists");
            }
        }
        else {
            $Log->exit() if $may_exit;
            $err_msg->throw();
        }
    };

    $$was_created = 1;

    my $obj = $pkg->new(
        client_command_interface => $ci,
        mount                    => $mount,
        _mount_status            => $mount_was_created,
        _route_status            => $route_created,
        _filer_ip                => $opts{'filer_ip'},
    );

    $Log->exit() if $may_exit;
    return $obj;
}

=head2 file_access

       # Method call description

        NACL::MTask::BranchCache->file_access(
            'client_command_interface' => $client,
            'filer_command_interface'  => $Node,
            source-path                => $srcpath,
            destination-path           => $destpath,
            #'display-filter'           => $filter,
            'file-path'                => $Path,
            'file-size'                => $File_size,
            'file-name'                => $File,
            drive                      => $client_setup->mount()->{'mount_point'},
            create_file                => $file_create,
            _was_created               => \$file_access_done,
        );

(Class method)  This method copies the file from a cifs share to the client
                 and it return the packet trace output from the filer .

=over

=item Options

=over

=item C<< nacltask_if_file_exists=>$action >>

(Optional) specifies an action to be taken if the file is already present inside the share.

     $action
    - if set to  "die"(default), an exception is raised.
    - if set to  "reuse" , use the existing file present.
    - if set to "purge", deletes the existing file before creating it again.

=item C<< client_command_interface=>$Client >>

(Required) specifies the command interface used which is a client object in this case.

=item C<< filer_command_interface => $Node >>

(Optional) specifies the command interface used which is filer object in this case .
 Required while creating the file.

=item C<< source-path=>$srcpath >>

(Optional) specifies the source file path in UNC for downloading .

=item C<< 'file-name' => $file >>

(Required) specifies the file-name of the file to copied.

=item C<< 'file-size' => $size >>

(Required) specifies the file-size of the file to be created in the shares.
The size should be 64kb < (file-size) < 165mb .

=item C<< 'file-path' => $path >>

(Required) specifies the file-path to given on which the file is created .

=item C<< destination-path=>$destpath >>

(Optional) specifies the destnation file path(UNC) for downloading (Default:'C:\\Users\\Administrator').

=item C<< create_file=>$create_flag >>

(Optional) specifies whether the file should be created within the shares.(default = 0).

=item C<< 'display-filter'=>$filter >>

(Optional) specifies the capture filter required for packet caturing in Wireshark.

=item C<< drive=>$drive >>

(Required) specifies to which drive the shares is mapped .

=item C<< _was_created => \$file_access_done >>
(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the file acces has been done successfully.
This is necessary to determine whether to call the purge related to file_access method .

=back

=item command_interface, apiset_must, apiset_should, method-timeout,
all the other options supported by ,
start & stop methods in PacketCapture.pm See L<< NACL::MTask::PacketCapture->start|lib-NACL-MTask-PacketCapture-pm/start,stop >>
copy library in NACL::APISet See L<< NACL::APISet::copy|lib-NACL-APISet-copy-pm/all methods >>

=back

=cut

#===============================================================================
# Name        : file_access
# Description : This method copies the file from a cifs share to the client
#                 and it return the packet trace output from the filer .
#===============================================================================

sub file_access {

    $Log->enter() if $may_enter;
    my $pkg = shift;
    NATE::BaseException->throw(
        "Setup can only be invoked with a proper package call")
      if ref $pkg;
    my %file_opts;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            client_command_interface =>
              { type => OBJECT, isa => 'NACL::C::Client' },
            filer_command_interface => {
                type     => OBJECT,
                isa      => 'NACL::C::CommandInterface::ONTAP',
                optional => 1
            },
            _was_created => { type => SCALARREF, optional => 1 },
        },
        allow_extra => 1,
    );

    my $client                  = $opts{client_command_interface};
    my $command_interface       = $opts{filer_command_interface};
    my $create_file             = $opts{create_file};
    my $nacltask_if_file_exists = $opts{nacltask_if_file_exists};
    my $destination = $opts{'destination-path'} || "C:\\Users\\Administrator";
    my $drive       = $opts{drive};
    my $default_src = "$drive\\$opts{'file-name'}";
    my $source      = $opts{'source-path'} || $default_src;
    my $filter      = "smb2";
    my $display_filter = $opts{'display-filter'} || $filter;
    my $was_created    = delete $opts{_was_created};
    $$was_created = 0;

    $file_opts{'size'} = $opts{'file-size'};
    if ( defined $opts{'file-path'} ) {
        $file_opts{'filename'} = "$opts{'file-path'}/$opts{'file-name'}";
    }

    #Creating the file within the shares
    if ($create_file) {
        try {
            NACL::C::Mkfile->mkfile(
                command_interface => $command_interface,
                %file_opts,
            );
        }
        catch NACL::APISet::Exceptions::CommandFailedException with {
            my $err_msg = shift;
            if ( $err_msg->text() =~ /write failed|file creation failed/ ) {
                if ( $nacltask_if_file_exists eq 'die' ) {
                    $Log->exit() if $may_exit;
                    $err_msg->throw();
                }
                elsif ( $nacltask_if_file_exists eq 'purge' ) {
                    NACL::C::Mkfile->mkfile(
                        command_interface => $command_interface,
                        %file_opts,
                    );
                }
                elsif ( $nacltask_if_file_exists eq 'reuse' ) {
                    $Log->comment("File $file_opts{'filename'} already exists");
                }
            }
            else {
                $Log->exit() if $may_exit;
                $err_msg->throw();
            }
        };
    }

    my @arg = (
        {
            target_machine => $client,
            display_filter => $filter,
        }
    );

    my $apiset = $client->apiset(
        category  => "Host",
        interface => "CLI",
        set       => "Windows",
    );

# This code is commented because smb2.1 protocol support is not present in Wireshark

    # Constructing the PacketCapture object
    # my $packetcapture =
    # NACL::MTask::PacketCapture->new( packet_capture_details => \@arg );
    # my $pckt_status = 1;

    # $packetcapture->start();
    ## Copying the file
    $apiset->copy(
        'source'      => $source,
        'destination' => $destination,
   	'connectrec-timeout' => '1000',
    );
    my $copy_status = 1;

    my $resp = $apiset->typeperf(
        'counter' => "\"\BranchCache\\SMB\: Bytes from server\"",
        'samples' => '2',
    );
    my $output         = $resp->get_parsed_output();
    my $smbbytesserver = $output->[0]{'value-counter1'};

    $resp = $apiset->typeperf(
        'counter' => "\"\BranchCache\\Retrieval\: Bytes from server\"",
        'samples' => '2',
    );
    $output = $resp->get_parsed_output();
    my $retbytesserver = $output->[0]{'value-counter1'};

    $resp = $apiset->typeperf(
        'counter' => "\"\BranchCache\\SMB\: Bytes from cache\"",
        'samples' => '2',
    );
    $output = $resp->get_parsed_output();
    my $smbbytescache = $output->[0]{'value-counter1'};

    $resp = $apiset->typeperf(
        'counter' => "\"\BranchCache\\Retrieval\: Bytes from cache\"",
        'samples' => '2',
    );
    $output = $resp->get_parsed_output();
    my $retbytescache = $output->[0]{'value-counter1'};

#This code is commented because smb2.1 protocol support is not present in Wireshark
# $packetcapture->stop();
# my $hash = $packetcapture->show();

    $$was_created = 1;

    my %hash = (
        client_command_interface => $client,
        _file                    => $file_opts{'filename'},
        _client_file             => "$destination\\$opts{'file-name'}",
        smbbytesserver           => $smbbytesserver,
        retbytesserver           => $retbytesserver,
        smbbytescache            => $smbbytescache,
        retbytescache            => $retbytescache,
        _copy_status             => $copy_status,

# pcktt_obj                => $packetcapture,       # |  This code is commented because
# _pckt_status             => $pckt_status,         # |  smb2.1 protocol  support is not
# _captured_output         => $hash,                # |  present in Wireshark

    );

    if ( defined($command_interface) ) {
        $hash{filer_command_interface} = $command_interface;
    }
    my $obj = $pkg->new( %hash, );

    $Log->exit() if $may_exit;
    return $obj;
}

=head2 purge

        # Method call description

         NACL::MTask::BranchCache->purge()

(Instance Method) This method is for purging process i.e., deleting the shares
created , deleting the drive which is mapped , terminating the cifs protocol
in the filer , disabling the options (SMB & BranchCache related) which was
enabled during setup .

=cut

#===============================================================================
# Name        : purge
# Description : This method is for purging process i.e., deleteing the shares
#               created , deleting the drive which is mapped , terminating the
#               cifs protocol  in the filer , disabling the options
#               (SMB & BranchCache related) which was enabled during setup .
#===============================================================================

sub purge {
    $Log->enter() if $may_enter;
    my $obj  = shift;
    my %opts = $obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            client_command_interface =>
              { type => OBJECT, isa => 'NACL::C::Client', optional => 1 },
            filer_command_interface => {
                type     => OBJECT,
                isa      => 'NACL::C::CommandInterface::ONTAP',
                optional => 1
            },
        },
        allow_extra => 1,
    );

    my $command_interface = $opts{'filer_command_interface'};
    my $client            = $opts{'client_command_interface'};
    my $file              = $opts{'_file'};
    my $client_file       = $opts{'_client_file'};
    my $filer_ip          = $opts{'_filer_ip'};
    my $apiset;

    if ( $obj->_shares_status() ) {
        $obj->cifs_shares()->purge();
    }

    if ( $obj->_cifs_status() ) {
        NACL::C::VserverCifs->delete( command_interface => $command_interface );
    }

    if ( $obj->_mount_status() ) {
        $obj->mount()->purge();
    }

    # if ( $obj->_pckt_status() ) {     # |  This code is commented because
    # $obj->pcktt_obj()->cleanup();     # |  smb2.1 protocol  support is not
    # }                                 # |  present in Wireshark

    if ( $obj->_file() ) {
        NACL::C::VolumeFile->delete_file(
            command_interface => $command_interface,
            file              => $file,
        );
    }

    if ( $obj->_copy_status() ) {
        $apiset = $client->apiset(
            category  => "Host",
            interface => "CLI",
            set       => "Windows",
        );
        $apiset->del( files => "$client_file" );
    }

    #deleting the route
    if ( $obj->_route_status() ) {
        $apiset = $client->apiset(
            category  => "Host",
            interface => "CLI",
            set       => "Windows",
        );
        $apiset->route_delete( 'destination' => $filer_ip );
    }

    if ( $obj->_options_status() ) {
        foreach my $option ( qw(cifs.smb2.enable cifs.smb2_1.branch_cache.enable ))
        {
            NACL::C::Options->option(
                command_interface => $command_interface,
                'option-name'     => $option,
                'option-value'    => 'off'
            );
        }
    }

    if ( $obj->_setflag_status() ) {
        $apiset = $command_interface->apiset(
            category  => "Node",
            interface => "CLI",
            set       => "7Mode",
        );
        $apiset->setflag( flag => "smb_bc_min_file_age", value => '0x12C' );
    }

    $Log->exit() if $may_exit;
}

sub _common_validate_with {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params => \@_,
        spec   => {
            params          => { type     => ARRAYREF },
            additional_spec => { type     => HASHREF, default => {} },
            allow_extra     => { optional => 1 },
        },
    );
    my %params = @{ $opts{params} };
    my %spec   = %{ $opts{additional_spec} };

    my @keys =
      qw ( filer_command_interface client_command_interface _file _client_file _filer_ip );

    if ( ref $pkg_or_obj ) {
        foreach my $key (@keys) {
            my $key_isset = "${key}_isset";
            if ( $params{$key} && $pkg_or_obj->$key_isset() ) {
                NATE::Exceptions::Argument->throw(
                        "Option '$key' not allowed (it is already an "
                      . 'attribute of this object' );
            } ## end if ( $params{$key} )
            if ( $pkg_or_obj->$key_isset() ) {
                my $value = $pkg_or_obj->$key();
                $params{$key} = $value;
            }
        } ## end foreach my $key (@keys)
    } ## end if ( ref $pkg_or_obj )

    # Any argument to a method call can either be of the type specified in
    # the call to _common_validate_with (i.e. scalar or arrayref) or can be
    # an object. This makes OBJECT be a valid type for all arguments
    while ( my ( $key, $value ) = each %spec ) {
        $value->{type} = $value->{type} | OBJECT
          if ( defined $value->{type} );
    }

    $Log->exit() if $may_exit;
    return validate_with(
        params      => \%params,
        spec        => \%spec,
        allow_extra => 1,
    );
} ## end sub _common_validate_with

1;
