
=head1 NAME

NACL::MTask::CIFSHomeDirectory

=head1 DESCRIPTION

C<NACL::MTask::CIFSHomeDirectory> provides a number of well-defined but
potentially complex or multi-step methods related to VserverCifsHomeDirectory in
ONTAP. The functionality provided mainly involves the components
C<NACL::C::VserverCifsHomeDirectorySearchPath> and C<NACL::C::VserverCifsShare> 
(since the current implementation covers only CMode).
See L<NACL::C::VserverCifsHomeDirectorySearchPath> and
L<NACL::C::VserverCifsShare> for details.
The MTask will mainly be concerned with setting up home directory given the
volumes that are to be used.

=head1 ATTRIBUTES

=head2 directories_created

An arrayref of user names whose directories have been created

=head2 searchpath

Represents the NACL::C::VserverCifsHomeDirectorySearchPath instance that was 
created during the setup

=head2 cifsshare

Represents the NACL::C::VserverCifsShare instance that was created during the 
setup

=head2 apiset 

The systemshell APISet object that was used for creating the user directories 

=head1 METHODS

=head2 setup

    my $home_dir = NACL::MTask::CIFSHomeDirectory->setup(
        volume => $volume_c_obj,
        homedir_path_type => $type,
        users => \@users,
        if_exists            =>  $action, # default 'die'
        %other_options
    );

(Class method) Given a list of volumes, this task configures CIFS home directory
for the corresponding vservers, adds appropriate search path entries and creates
users(their home directories) on the volumes.

=over 

=item Options

=over

=item C<< if_exists=>$action >>

(Optional) What to do if the Vserver already has the home directory
 share or home directory search path configured or if any of the user
 directories already exists
 If $action is "die" (the default), then fail with an exception stating, the
 failure as $exception->text().
 If action is "reuse", then ignore the error due to the above reasons and
 continue. Otherwise, define a custom handler (code ref)that can handle the
 exception. This could be something like resuing the share already created but
 failing if user directories are already there.

=item C<< volume => $volume_c_obj >>

(Required) A C<NACL::C::Volume> instance

=item C<< homedir_path_type => $type >>

(Optional) A string indicating the setting to use for path (%w or %d or %d/%w) 
while creating the share. default is %w.
Different options indicates the location for the user's home directories. %w indicates 
that the user's home directory is directly under the junction-path of the volume.
%d indicates that the home directory for all the users is the domain directory under 
the junction-path.%d/%w indicates that the user's home directory is inside the domain 
directory under the junction-path of the volume.

=item C<< share_name => $share >>

(Optional, default: %w)
This parameter specifies the name of the CIFS share that you want to create. A share 
name can be up to 256 characters long. If this is a home directory share as specified 
by the value of home directory on the -share-properties parameter, you can make the 
share name dynamic by specifying the %w (user name), %d (domain name) variables or any 
of their combination as a part of the value of this parameter.


=item C<< users => \@users >>

(Optional) An array ref indicating the user names for whom home directories are 
to be created.

=back

=item command_interface, apiset_must, apiset_should, method-timeout,
all the other options supported by
NACL::C::VserverCifsHomeDirectorySearchPath->add and
NACL::C::VserverCifsShare->create are supported by
NACL::MTask::CIFSHomeDirectory->setup as well.

=back

=head2 cleanup

    $homedir_obj->cleanup();

(Class method) Cleans up whatever was created/added during setup()

=over 

=item Options
    None

=back

=cut

# this package is
package NACL::MTask::CIFSHomeDirectory;

# this package is a
use base qw/NACL::MTask::MTask/;

# we need these libs
use NACL::C::VserverCifsHomeDirectorySearchPath;    #for add
use NACL::C::VserverCifsShare;                      #for create
use NACL::CS::VserverCifs;    # for getting the domain information
use NACL::APISet;             # for systemshell apiset
use NACL::APISet::Exceptions::CommandFailedException qw ( :try);
use Params::Validate qw/validate_with SCALAR ARRAYREF/;

#utils
use strict;

use warnings;
use Data::Dumper;

############################################################################################################
# Name        : setup
# Description : Actual task function to be invoked by test scripts. POD above has complete details
#               Creates Shares, Adds homedirectory path and creates directories for users as per options
############################################################################################################

sub setup {

    # obtain the package name
    my $pkg = shift;
    NATE::BaseException->throw(
        "Setup can only be invoked with a proper package call")
        if ref $pkg;
    my $obj;
    my $validation_sub = {
        'Any of "%w", "%d", "%d/%w"' => sub {

            if (   $_[0] eq '%w'
                || $_[0] eq '%d'
                || $_[0] eq '%d/%w' )
            {
                return 1;
            }
            return 0;
            }
    };

    # validation
    my %opts = validate_with(
        params => \@_,
        spec   => {
            volume => { isa  => 'NACL::C::Volume' },
            users  => { type => ARRAYREF, default => [] },
            if_exists => {
                type      => SCALAR,
                default   => 'die',
                callbacks => {
                    'Any of "die", "reuse" or "purge"' => sub {
                        if (   $_[0] eq 'die'
                            || $_[0] eq 'reuse'
                            || $_[0] eq 'purge' )
                        {
                            return 1;
                        }
                        return 0;
                        }

                }
            },
            share_name => {
                type      => SCALAR,
                default   => '%w',
                callbacks => $validation_sub,
            },
            homedir_path_type => {
                type      => SCALAR,
                default   => '%w',
                callbacks => $validation_sub,
            },
        },
        allow_extra => 1
    );

    # extract necessary details from the object
    my $volume            = $opts{volume};
    my @users             = @{ $opts{users} };
    my $homedir_path_type = $opts{homedir_path_type};
    my $share_name        = delete $opts{share_name};

    my $share_properties
        = $opts{'share-properties'}
        ? $opts{'share-properties'}
        : [ 'oplocks', 'browsable', 'homedirectory' ];
    my $if_exists = $opts{if_exists};

    # extracting other parameters that are required for subsequent operations
    my $command_interface = $volume->command_interface();
    NATE::BaseException->throw("Only CMode is supported as of now")
        if $command_interface->mode() ne 'CMode';

    my $vserver = $volume->vserver();

    # build a hash that has everything required for both the component
    # operations that are performed including everything else in %opts

    my %component_call_opts = (
        %opts,
        vserver            => $vserver,
        path               => $homedir_path_type,
        'share-name'       => $share_name,
        'share-properties' => $share_properties,
    );
    $obj->{cifsshare} = _handle_creation(
        %component_call_opts,
        component     => 'NACL::C::VserverCifsShare',
        method        => 'create',
        delete_method => 'delete',

    );

    #As per home directory creation design, volume-junction path
    #should be added under CIFS Home Directory search path table.
    # For the following component call, 'path' actually refers to the
    # junction path that the volume has.

    my $junction_path = $volume->get_one_state_attribute("junction-path");
    $component_call_opts{path} = $junction_path;

    # add the search-path entry for this volume's junction path
    $obj->{searchpath} = _handle_creation(
        %component_call_opts,
        component     => 'NACL::C::VserverCifsHomeDirectorySearchPath',
        method        => 'add',
        delete_method => 'remove',
    );

    # the domain that the vserver belongs to has to be found out if we are
    # dealing with a homedir_path_type like %d or %d/%w

    my $domain;

    if ( $homedir_path_type =~ /%d/ ) {

        # get the domain name - this is required for creating the directory
        my $vserver_Cifsserver = NACL::CS::VserverCifs->fetch(
            command_interface => $command_interface,
            filter            => { vserver => $vserver },
            requested_fields  => ["domain-workgroup"]
        );
        $domain = $vserver_Cifsserver->domain_workgroup();
    }

   # in systemshell, the users' home directories will begin with the following
   # path. Since only C-mode is handled for now it has been hardcoded to clus/
    my $dir_path = "/clus/" . $vserver . $junction_path . "/";

    my $paths;

    # depending on the homedir_path_type, we create the necessary directories
    if ( $homedir_path_type eq '%d' ) {

        # Path of the directory to be created is $dir_path/$domain
        $paths = $dir_path . $domain;

    } elsif ( $homedir_path_type eq '%w' ) {

        # Paths of the directories to be created are $dir_path/user1,
        #$dir_path/user2, ...

        if ( grep { $_ = $dir_path . $_ } @users ) {
            $paths = join( ' ', @users );
        }

    } elsif ( $homedir_path_type eq '%d/%w' ) {

        #  Paths of the directories to be created are $dir_path/$domain/user1,
        # $dir_path/$domain/user2, ...

        if ( grep { $_ = $dir_path . $domain . '/' . $_ } @users ) {
            $paths = join( ' ', @users );
        }

    }

    # if $paths is not set then we need not create any directories

    if ($paths) {

#It is easier to use systemshell to create these directories. Otherwise, we will have to create a CIFS share,
# map the share from a WinClient and then create directories. Using systemshell this is easier to achieve as it
# removes dependency on WinClient.
        my $Api_Set_Obj = $command_interface->apiset(
            category  => "Node",
            interface => "CLI",
            set       => "Systemshell",
        );

        #Root permissions are need to execute the commands
        $Api_Set_Obj->set_privilege( 'privilege-level' => "root" );

        $Api_Set_Obj->mkdir(
            'make-parents' => 1,
            paths          => $paths
        );
        $obj->{apiset}              = $Api_Set_Obj;
        $obj->{directories_created} = $paths;
    }

    # return the object created after finishing the above steps
    bless $obj, $pkg;
    return $obj;
}

############################################################################################################
# Name        : _handle_creation
# Description : Invoked from setup. If there are duplicate entries already existing on the filer, based on
#               if_exists option which might be to die,purge or reuse, appropriate action is taken. If e.g.
#               if_exists => purge, It will call purge action which deletes the entries and recreates
#               the entries. If that configuration is not there, it simply creates the entry. Returns an
#               object of the requested component type only if this method creates one so that this can be
#               stored as an attribute for deletion during cleanup.
############################################################################################################

sub _handle_creation {

    my %opts = @_;

    # get the name of the component and the name of the method required for
    # create/add
    my $pkg    = $opts{component};
    my $method = $opts{method};

    # extract other necessary details
    my $command_interface = $opts{volume}->command_interface();

    my ( $object, $is_created );

    my %actions = (
        "die" => {
            check  => 0,
            exists => undef,
        },
        "purge" => {
            check  => 1,
            exists => sub {

                _purge(%opts);

                # Forget that the component had already
                # been found, to force create step
                undef $object;
            },
        },
        "reuse" => {
            check  => 1,
            exists => sub {
            },
        },
    );

    # get the hashref that is the action for the if_exists value
    my $action = $actions{ $opts{if_exists} };

    # if a check has to be done for the selected action
    if ( $action->{check} ) {

        # build the primary key hash for this element from all the options
        # that we have
        my $pk_hash = $pkg->get_primary_keys_options(%opts);

        # check if an element with the same primary key values as what we want
        # already exists
        $object = $pkg->find(
            command_interface => $command_interface,
            filter            => $pk_hash,
            allow_empty       => 1
        );

        # if $object is not defined, then we didn't find a matching element
    }

    # if an object exists, then we either use the same object or delete it
    # depending on what the 'exists' coderef does for us

    if ($object) {
        &{ $action->{exists} };
    }

    # at this point, $object may not be defined (if it was either deleted or
    # not found. so create it if its not defined

    if ( !$object ) {

       # call the approriate method (create / add ) on the appropriate package
       # with all the opts sent to this function. allow_extra is for making
       # the component method ignore the irrelevant opts

        $object = $pkg->$method(
            %opts,
            command_interface => $command_interface,
            allow_extra       => 1
        );
        $is_created = 1;
    }
    return $object if $is_created;
    return undef;
}

############################################################################################################
# Name        : _purge
# Description : Invoked from _handle_creation. It takes the appropriate delete method to be used and the
#               component that has requested for purge, frames the method with appropriate parameters
#               to be called and invokes the same.
############################################################################################################

sub _purge {
    my %opts   = @_;
    my $pkg    = $opts{component};
    my $method = $opts{delete_method};

    my $command_interface = $opts{volume}->command_interface();

    # build the primary key hash for this element from all the options
    # that we have

    my $pk_hash = $pkg->get_primary_keys_options(%opts);

    # call either delete/remove on the package provided
    $pkg->$method( command_interface => $command_interface, %$pk_hash );
}

############################################################################################################
# Name        : cleanup
# Description : Invoked with an object that was returned from setup
#               destroys everything that was created during setup
############################################################################################################

sub cleanup {
    my $obj = shift;
    NATE::BaseException->throw(
        "Can only be invoked with an object of NACL::MTask::CIFSHomeDirectory class"
    ) unless $obj->isa('NACL::MTask::CIFSHomeDirectory');
    eval {
        $obj->{apiset}
            ->rmdir( 'parents' => 1, paths => $obj->{directories_created} )
            if $obj->{directories_created};
    };
    $obj->{searchpath}->remove() if $obj->{searchpath};
    $obj->{cifsshare}->delete()  if $obj->{cifsshare};

}

1;
