# $Id:$
#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#

## @summary ChangeRoot Task Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::MTask::ChangeRoot

=head1 DESCRIPTION

NACL::MTask::ChangeRoot is an MTask which provides the methods related
to change the root elements (both root volume and root aggregate).

=cut

package NACL::MTask::ChangeRoot;

use strict;
use warnings;
use Tharn qw( host param_merge);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate
    qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR);
use NATE::Exceptions::Argument qw(:try);
use NACL::C::Volume;
use NACL::C::StorageAggregate;
use NACL::Transit;
use NACL::C::Client;
use NACL::Exceptions::UnexpectedState;
use NACL::Exceptions::OperationFailed;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::STask::SystemLicense;
use constant NACL_COMMAND_TIMEOUT => 2400;
use Data::Dumper;

=head1 METHODS

=head2 new

    my $obj = NACL::MTask::ChangeRoot->new( 'command_interface' => $ci );
    
Create a ChangeRoot object that can be used to change the root elements of an
ONTAP system.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required, isa NACL::C::CommandInterface::ONTAP)
The command interface Object.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=item C<< aggregate => $scalar >>

(Mandatory)
Aggregate to be made as root. Aggregate will not be created by this method hence, should pre-exist.

=item C<< volume => $volume >>

(Optional)
volume to be made as root. volume will not be created by this method hence, should pre-exist if given.
Else AUTOROOT will be choosen as new root by ONTAP.

=back

=back

=cut

use Class::MethodMaker [
    new    => [ '-hash', '-init', 'new' ],
    scalar => 'aggregate',
    scalar => 'volume',
    scalar => [
        { -type => 'NACL::C::CommandInterface::ONTAP' },
        'command_interface'
    ],
];

sub init {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            command_interface => { type => OBJECT },
            aggregate         => { type => SCALAR, optional => 1 },
            volume            => { type => SCALAR, optional => 1 },
        },

    );
    $Log->exit() if $may_exit;
}

=head2 pick_new_root

    my $obj = NACL::MTask::ChangeRoot->new( 'command_interface' => $ci );
        $obj->pick_new_root(
            aggregate => "test_root_aggr",
            volume    => "test_root_vol",
            client_command_interface => $client_ci,
        );
    or

    my $obj = NACL::MTask::ChangeRoot->new(
        'command_interface' => $ci,
        aggregate           => "test_root_aggr",
        volume              => "test_root_vol"
    );
    $obj->pick_new_root(
        client_command_interface => $client_ci,
    );  
    
Method to change the root elements of an ONTAP system.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required, isa NACL::C::CommandInterface::ONTAP)
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=item C<< client_command_interface => $client_ci >>

(Mandatory for CMode, isa NACL::C::Client)
Command interface object of Client. This parameter is mandatory for CMode.

=item C<< aggregate => $scalar >>

(Mandatory)
Aggregate to be made as root. Aggregate will not be created by this method hence, should pre-exist.

=item C<< volume => $volume >>

(Optional)
volume to be made as root. volume will not be created by this method hence, should pre-exist if given.
Else AUTOROOT will be choosen as new root by ONTAP.

=item C<< powercycle => $boolean >>

(Optional,  Default: 0)
If set to 1 and if transition fails once, try to cycle power on the host and then try exactly once more.

=item C<< firmware_flags => $hash_ref >>

(Optional)
Firmware flags to be set. Keys of hash will hold the name of the flag and the value of those keys is the value
to be set.
Ex: 
    my %firmware_flags = ('wafl-scanners-slow?' => 1, 'wafl-iron-panic-on-corruption?' => 1);
    $obj->pick_new_root(
        client_command_interface => $client_ci,
        firmware_flags           => \%firmware_flags,
    );



=item C<< nacltask_timeout => $scalar >>

(Optional, Default: 2400)
Command timeout value


=back

=back

=cut

sub pick_new_root {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            command_interface => {
                isa      => 'NACL::C::CommandInterface::ONTAP',
                type     => OBJECT,
                optional => 1
            },
            client_command_interface =>
                { isa => 'NACL::C::Client', type => OBJECT, optional => 1 },
            aggregate        => { type => SCALAR,  optional => 1 },
            volume           => { type => SCALAR,  optional => 1 },
            powercycle       => { type => SCALAR,  default  => 0 },
            firmware_flags   => { type => HASHREF, optional => 1 },
            nacltask_timeout => { type => OBJECT,  optional => 1 },
        },
    );

    $opts{command_interface} = $opts{command_interface}
        || $self->command_interface();

    my $ci = $opts{command_interface};
    if ( $ci->mode() eq "CMode" ) {
        if ( !$opts{client_command_interface} ) {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw(
                "\"client_command_interface\" is mandatory parameter for CMode"
            );
        }
    }
    my $hostrec     = $ci->hostrec();
    my $aggregate   = $opts{aggregate} || $self->aggregate();
    my $volume      = $opts{volume} || $self->volume();
    my $transit_obj = NACL::Transit->new( name => $hostrec );
    my $powercycle  = $opts{powercycle};
    my $timeout     = $opts{nacltask_timeout} || NACL_COMMAND_TIMEOUT;
    my $client_command_interface = delete $opts{'client_command_interface'};
    my $rdb_backup_path;

    if ( !$volume ) {
        $volume = "AUTOROOT";
    }

    if ( !$aggregate ) {
        NATE::Exceptions::Argument->throw(
            "\"aggregate\" is a mandatory parameter");
    }

    $transit_obj->change_state( 'to' => 'CLI' );
    my $state = $transit_obj->get_state( 'timeout' => 300 );

    # Make sure the filer is in CLI prompt
    if ( $state ne "CLI" ) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::UnexpectedState->throw(
            "Host should be in CLI prompt.");
    }

    # verify if the volume/aggregate exists and is already a root
    _verify_aggr( command_interface => $ci, aggregate => $aggregate );
    _verify_vol( command_interface => $ci, volume => $volume )
        if ( $volume && $volume ne "AUTOROOT" );

    if ( $ci->mode() eq "CMode" ) {
        $rdb_backup_path = _backup_rdb(
            client_command_interface => $client_command_interface,
            command_interface        => $ci,
        );
    }

    $Log->debug("Setting the $aggregate as root aggregate");
    $transit_obj->change_state(
        'to'         => 'MAINT',
        'powercycle' => $powercycle
    );

    my $maint_mode_api
        = $ci->apiset( interface => 'CLI', set => 'Maintenance' );

    if ( $ci->mode() eq "CMode" ) {
        _pick_new_root_cm(
            %opts,
            transit_obj              => $transit_obj,
            maint_mode_api           => $maint_mode_api,
            timeout                  => $timeout,
            rdb_backup_path          => $rdb_backup_path,
            client_command_interface => $client_command_interface,
            aggregate                => $aggregate,
        );
    } else {
        _pick_new_root_7m(
            %opts,
            transit_obj    => $transit_obj,
            maint_mode_api => $maint_mode_api,
            timeout        => $timeout,
            aggregate      => $aggregate,
            volume         => $volume
        );
    }

    $Log->exit() if $may_exit;
}

# helper method to change the root elements in CMode.

sub _pick_new_root_cm {
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            command_interface =>
                { isa => 'NACL::C::CommandInterface::ONTAP', type => OBJECT },
            client_command_interface =>
                { isa => 'NACL::C::Client', type => OBJECT },
            transit_obj     => { type => OBJECT },
            aggregate       => { type => SCALAR },
            maint_mode_api  => { type => OBJECT },
            timeout         => { type => SCALAR },
            rdb_backup_path => { type => SCALAR },
        },
        allow_extra => 1
    );

    my $ci              = $opts{command_interface};
    my $client_ci       = delete $opts{client_command_interface};
    my $transit_obj     = $opts{transit_obj};
    my $rdb_backup_path = $opts{rdb_backup_path};

    _set_root_aggr(%opts);

    $transit_obj->change_state( 'to' => 'CLI' );

    _restore_rdb(
        client_command_interface => $client_ci,
        command_interface        => $ci,
        source_path              => $rdb_backup_path,
    );

    _recover_root_vol(
        command_interface => $ci,
        transit_obj       => $transit_obj
    );

    $Log->exit() if $may_exit;
}

# helper method to change the root elements in 7Mode.

sub _pick_new_root_7m {
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            command_interface =>
                { isa => 'NACL::C::CommandInterface::ONTAP', type => OBJECT },
            transit_obj    => { type => OBJECT },
            aggregate      => { type => SCALAR },
            volume         => { type => SCALAR, default => 'AUTOROOT' },
            maint_mode_api => { type => OBJECT },
            timeout => { type => SCALAR, default => NACL_COMMAND_TIMEOUT },
        },
        allow_extra => 1
    );

    my $ci            = $opts{command_interface};
    my $transit_obj   = $opts{transit_obj};
    my $new_root_aggr = $opts{aggregate};
    my $timeout       = $opts{timeout};
    my $vol           = $opts{volume};
    my $conn          = $transit_obj->{conn};
    my $cmd;

    _set_root_aggr(%opts);
    $transit_obj->change_state( 'to' => 'BOOT_MENU' );

    if ( $vol eq 'AUTOROOT' ) {
        $Log->debug("Setting up the AUTOROOT as root volume");
        my $dhcp_str_found = 0;
        my $dhcp_str       = 'Ctrl-C to skip DHCP search';

     # Send '1' for normal boot, without running vol_pick_root to use AUTOROOT
        $cmd = '1';
        $conn->execute(
            'command'            => $cmd,
            'timeout'            => $timeout,
            'additional_prompts' => [$dhcp_str]
        );
    } else {

        $cmd = "vol_pick_root $vol $new_root_aggr";
        $@   = undef;
        eval {
            $conn->execute(
                'command' => $cmd,
                'additional_prompts' =>
                    ['Please choose one of the following'],
                'timeout' => $timeout
            );
        };

        # If we had a timeout error try the command again
        if ( $@ && $@ =~ /IDLE|TIMEOUT/ ) {
            $conn->execute(
                'command' => $cmd,
                'additional_prompts' =>
                    ['Please choose one of the following'],
                'timeout' => $timeout
            );
        }
        $conn->execute_start( 'command' => '1' );
    }
    $Log->debug("Starting Setup...");
    _cold_start_filer(
        transit_obj       => $transit_obj,
        conn_obj          => $conn,
        newfs             => 0,
        command_interface => $ci
    );
    _license( $ci, [ 'nfs', 'cifs', 'http' ] );
    $transit_obj->change_state( 'to' => 'CLI' );
    $ci->refresh_command_interface( no_version_change => 1 );

    $Log->exit() if $may_exit;
}

sub _set_root_aggr {
    $Log->enter() if $may_enter;
    my %opts = @_;

    my $ci             = $opts{command_interface};
    my $maint_mode_api = $opts{maint_mode_api};
    my $transit_obj    = $opts{transit_obj};
    my $new_root_aggr  = $opts{aggregate};
    my %firmware_flags = %{ $opts{firmware_flags} }
        if defined( $opts{firmware_flags} );
    my $timeout = $opts{timeout};

    my ( $text, $error );

    try {
        $text = $maint_mode_api->aggr_online(
            'aggregate'          => $new_root_aggr,
            'connectrec-timeout' => $timeout
        )->get_raw_output();
    }
    catch NACL::APISet::Exceptions::CommandFailedException with {
        my $exception = shift;
        if ( $exception->text() !~ /already online/is ) {
            $Log->exit() if $may_exit;
			$exception->throw();
        }
    };
    $error = "Filer panicked while getting it online. Failed to set $new_root_aggr aggr as new root";
    if ( $text =~ /Panic/i ) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::OperationFailed->throw($error);
    }


    $text = $maint_mode_api->aggr_options(
        'aggregate'   => $new_root_aggr,
        'option-name' => 'root',
    )->get_raw_output();
    $error = "Filer panicked while setting the $new_root_aggr as root!";
    if ( $text =~ /Panic/i ) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::OperationFailed->throw($error);
    }

    if ( keys %firmware_flags ) {

        # Set the required bootargs and reboot to the boot menu
        $transit_obj->change_state( 'to' => 'FIRMWARE', );

        my $firmware_apiset_obj
            = $ci->apiset( interface => 'CLI', set => 'Firmware' );

        while ( my ( $varname, $value ) = each %firmware_flags ) {
            $firmware_apiset_obj->setenv(
                'varname'            => $varname,
                'value'              => $value,
                'connectrec-timeout' => $timeout
            );
        }
    }
}

# Helper method to recover lost or corrupted root volume

sub _recover_root_vol {
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            command_interface => {
                isa      => 'NACL::C::CommandInterface::ONTAP',
                type     => OBJECT,
                optional => 1
            },
            transit_obj => { type => OBJECT },
        },
    );

    my $transit_obj = $opts{transit_obj};
    my $ci          = $opts{command_interface};
    my $hostrec     = $ci->hostrec();

    $transit_obj->change_state( 'to' => 'SYSTEMSHELL' );

    my $systemshell_apiset
        = $ci->apiset( interface => 'CLI', set => 'Systemshell' );
    $Log->debug("Starting the root vol recovery..");

    my $raw_output = $systemshell_apiset->kenv(
        pass_through => "bootarg.init.boot_recovery" )->get_raw_output();
    if ( $raw_output !~ /kenv: unable to get bootarg.init.boot_recovery/ ) {

        $systemshell_apiset->execute_raw_command( 'command' =>
                "sudo sysctl kern.bootargs=--bootarg.init.boot_recovery" );

        try {
            $systemshell_apiset->ls(
                'paths' => "/mroot/etc/cluster_config/monitor_mroot.nvfail" );
            $systemshell_apiset->rm(
                "/mroot/etc/cluster_config/monitor_mroot.nvfail");

        }
        catch NACL::APISet::Exceptions::CommandFailedException with {
            my $ex = shift;
            $Log->debug(
                "file - /mroot/etc/cluster_config/monitor_mroot.nvfail either not found or not able to remove it"
            );
        };
    }

    $Log->debug("Finished root vol recovery..");
    $Log->debug("Now rebooting...");

    $transit_obj->reboot();
    $transit_obj->change_state( 'to' => 'CLI' );
    $ci->refresh_command_interface();

    $Log->exit() if $may_exit;
}

# Helper method to verify the volume if it exists or if it is already a root volume.

sub _verify_vol {
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            command_interface => {
                isa      => 'NACL::C::CommandInterface::ONTAP',
                type     => OBJECT,
                optional => 1
            },
            volume => { type => SCALAR },
        },
    );
    my $ci     = $opts{command_interface};
    my $volume = $opts{volume};

    NACL::C::Volume->find(
        'command_interface' => $ci,
        filter              => { volume => $volume },
    );

    _verify_if_root_element(
        command_interface => $ci,
        obj_type          => "volume",
        element           => $volume
    );
    $Log->exit() if $may_exit;
}

# Helper method to verify the aggregate if it exists or if it is already a root aggregate.

sub _verify_aggr {
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            command_interface => {
                isa      => 'NACL::C::CommandInterface::ONTAP',
                type     => OBJECT,
                optional => 1
            },
            aggregate => { type => SCALAR },
        },
    );
    my $ci        = $opts{command_interface};
    my $aggregate = $opts{aggregate};

    NACL::C::StorageAggregate->find(
        'command_interface' => $ci,
        filter              => { aggregate => $aggregate },
    );

    _verify_if_root_element(
        command_interface => $ci,
        obj_type          => "aggregate",
        element           => $aggregate
    );

    # If CMode, make sure that aggregate's ha_policy is cfo.
    if ( $ci->mode() eq "CMode" ) {

        # Fetch the attibute values
        my $aggr_state_obj = NACL::CS::StorageAggregate->fetch(
            'command_interface' => $ci,
            filter              => { aggregate => $aggregate }
        );

# Check and set the ha_policy of the given aggregate to cfo unless it is already set.
        my $ha_policy = $aggr_state_obj->ha_policy();
        if ( $ha_policy ne "cfo" ) {
            NACL::C::StorageAggregate->modify(
                'command_interface' => $ci,
                'aggregate'         => $aggregate,
                'ha-policy'         => 'cfo'
            );
        }
    }

    $Log->exit() if $may_exit;
}

# internal method to verify if the given volume/aggregate is not a root
# throws NACL::APISet::Exceptions::InvalidParamValueException if the given
# element is already a root, else just returns.

sub _verify_if_root_element {

    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            command_interface => {
                isa      => 'NACL::C::CommandInterface::ONTAP',
                type     => OBJECT,
                optional => 1
            },
            obj_type => { type => SCALAR },
            element  => { type => SCALAR },
        },
    );

    my $apiset_obj = $opts{command_interface}
        ->get_7m_or_nodescope_apiset( interface => 'CLI' );

    my ( $cmd_prefix, $type );
    if ( $opts{obj_type} =~ /aggr/i ) {
        $cmd_prefix = "aggr";
        $type       = "aggregate";
    } elsif ( $opts{obj_type} =~ /vol/i ) {
        $cmd_prefix = "vol";
        $type       = "volume";
    } else {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Invalid obj_type: " . $opts{obj_type} . " passed!" );
    }
    my $command       = $cmd_prefix . "_status";
    my $parsed_output = $apiset_obj->$command()->get_parsed_output();

    foreach my $href ( @{$parsed_output} ) {
        if ( $href->{'options'}->[0]->{'root'} ) {
            if ( $href->{$type} eq $opts{element} ) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Given $type is already a root element");
            }
            $Log->exit() if $may_exit;
            return;
        }
    }
    $Log->exit() if $may_exit;
    return;
}

# Helper routine to restore the rdb

sub _restore_rdb {
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            command_interface =>
                { isa => 'NACL::C::CommandInterface::ONTAP', type => OBJECT },
            client_command_interface =>
                { isa => 'NACL::C::Client', type => OBJECT },
            source_path => { type => SCALAR },
        },
    );

    my $client_ci = delete $opts{client_command_interface};
    my $ci        = delete $opts{command_interface};
    my $bsd_api   = $ci->apiset(
        category  => "Node",
        interface => "CLI",
        set       => "Systemshell"
    );
    my $client_api  = $client_ci->apiset();
    my $client_conn = $client_api->get_connection();

    $Log->debug("Creating the backup directory on the client..");
    my $host_name = $ci->name();
    my $username  = $client_conn->username();
    my $password  = $client_conn->password();
    my $client_ip = $client_conn->hostp()->default_ip();
    my $path      = delete $opts{source_path};

    foreach my $service (qw(vldb vifmgr mgwd)) {
        try {
            $bsd_api->spmctl(
                stop                 => 1,
                handle               => $service,
                'connectrec-prompts' => [""]
            );
        }
        catch NACL::APISet::Exceptions::ConnectionFailedException with {
            my $ex = shift;

            # Applying retry logic to reconnect after service restart
            my $retries             = 5;
            my $reconnect_succeeded = 0;
            while ($retries) {
                $bsd_api->cleanup_APISet();
                sleep 10;
                try {
                    $bsd_api = NACL::APISet->new(
                        name      => $host_name,
                        category  => "Node",
                        interface => "CLI",
                        set       => "Systemshell"
                    );
                    ++$reconnect_succeeded;
                }
                catch NACL::APISet::Exceptions::ConnectionFailedException
                    with {
                    Tharn::snooze(20);
                    };
                last if ($reconnect_succeeded);
                --$retries;
            }

            # re-throw the exception if the $retries = 0.
            if ( !$retries ) {
                $Log->exit() if $may_exit;
                $ex->throw();
            }
        };
    }

    my $source_path = $username . '@' . $client_ip . ':' . $path;
    my $dest        = '/mroot/etc/cluster_config/rdb';

    my $source = $source_path . "/Management";
    $bsd_api->scp(
        "recursive-copy"         => 1,
        "source-path"            => $source,
        "dest-path"              => $dest,
        'connectrec-match_table' => [
            "connecting \(yes\/no\)\?" => "yes",
            "login to the systemshell" => "yes",
            "password"                 => "$password"
        ],
        'connectrec-timeout' => '900'
    );

    $source = $source_path . "/VifMgr";
    $bsd_api->scp(
        "recursive-copy"         => 1,
        "source-path"            => $source,
        "dest-path"              => $dest,
        'connectrec-match_table' => [
            "connecting \(yes\/no\)\?" => "yes",
            "login to the systemshell" => "yes",
            "password"                 => "$password"
        ],
        'connectrec-timeout' => '900'
    );
    $source = $source_path . "/VLDB";
    $bsd_api->scp(
        "recursive-copy"         => 1,
        "source-path"            => $source,
        "dest-path"              => $dest,
        'connectrec-match_table' => [
            "connecting \(yes\/no\)\?" => "yes",
            "login to the systemshell" => "yes",
            "password"                 => "$password"
        ],
        'connectrec-timeout' => '900'
    );

    my $output = $bsd_api->ls( paths => $dest )->get_raw_output();

    if ( $output !~ /VLDB/ ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw( 'text' => "Rdb files not copied" );
    }

    my $backup_dir_on_client = $path;
    $backup_dir_on_client =~ s/\/rdb_backup$//;
    $client_api->rm(
        "recursive" => "-R",
        "force"     => "-f",
        'paths'     => "$backup_dir_on_client"
    );

    Tharn::sleep 20;
    $ci->refresh_command_interface();
    $Log->exit() if $may_exit;

}

# Helper routine to backup the rdb

sub _backup_rdb {
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            client_command_interface =>
                { isa => 'NACL::C::Client', type => OBJECT },
            command_interface => { type => OBJECT },
            destination_path  => { type => SCALAR, optional => 1 },
        },
    );

    my $client_ci = delete $opts{client_command_interface};
    my $ci        = delete $opts{command_interface};
    my $bsd_api   = $ci->apiset(
        category  => "Node",
        interface => "CLI",
        set       => "Systemshell"
    );
    my $client_api  = $client_ci->apiset();
    my $client_conn = $client_api->get_connection();

    $Log->debug("Creating the backup directory on the client..");
    my $host_name = $ci->name();
    my $username  = $client_conn->username();
    my $password  = $client_conn->password();
    my $client_ip = $client_conn->hostp()->default_ip();
    my $path = delete $opts{destination_path} || "/t/$host_name/rdb_" . $$;

    $bsd_api->execute_raw_command(
        command => "rdb_tool backup -d /mroot/var/rdb_backup" );

    $client_api->mkdir(
        'make-parents' => "-p",
        'paths'        => $path
    );
    my $dest_path = $username . '@' . $client_ip . ':' . $path;

    # Remove /root/.ssh/known_hosts file
    $client_api->rm(
        "recursive" => "-R",
        "force"     => "-f",
        'paths'     => "/root/.ssh/known_hosts"
    );

    $bsd_api->scp(
        "recursive-copy"         => 1,
        "source-path"            => '/mroot/var/rdb_backup',
        "dest-path"              => $dest_path,
        'connectrec-match_table' => [
            "connecting \(yes\/no\)\?" => "yes",
            "password"                 => "$password"
        ],
        'connectrec-timeout' => '900'
    );

    my $output
        = $client_api->ls( paths => "$path/rdb_backup" )->get_raw_output();

    if ( $output !~ /VLDB/ ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("Rdb files not copied");

    }
    $Log->exit() if $may_exit;
    return $path . '/rdb_backup';
}

##
## @name ::cold_start_filer
## Cold start a filer, basically does a newfs on the filer
## args are the same as for the Filer/newfs.thpl script
##

sub _cold_start_filer {

    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            transit_obj => { type => OBJECT },
            conn_obj    => { type => OBJECT },
            newfs       => { type => SCALAR, default => 0 },
            command_interface =>
                { isa => 'NACL::C::CommandInterface::ONTAP', type => OBJECT },
        },
    );

    my $newfs       = $opts{'newfs'};
    my $conn        = $opts{conn_obj};
    my $transit_obj = $opts{'transit_obj'};
    my $fh          = $conn->hostp();
    my $ci          = $opts{command_interface};

    if ($newfs) {
        $Log->debug("Cold starting the filer");
    } else {
        $Log->debug("Warm starting the filer");
    }

    my ( $cmd, $vol_type, $filer_args );
    my @opts;
    my %fields = $fh->param_fields();

    eval {
        $Log->debug("Calling _basic_setup");
        _basic_setup(
            'prefix'          => "FILER",
            'issue_setup_cmd' => 1,
            'timeout'         => 560,
            'conn_obj'        => $conn
        );
    };
    if ($@) {
        if ( $@ =~ /Err=TIMEOUT/ ) {
            $Log->debug("Calling basic_setup again after poking the console");
            _basic_setup(
                'prefix'          => "FILER",
                'issue_setup_cmd' => 0,
                'timeout'         => 60,
                'conn_obj'        => $conn
            ) or die("basic setup failed for a second time. $@");
        } else {
            die("basic setup failed for something other than a timeout. $@");
        }
    }
    my $filer_pw = $fh->default_password();

    $conn->execute(
        'command' => "",
        'match_table' =>
            [ 'New password' => $filer_pw, 'password' => $filer_pw ],
        'max_idle' => 10,
        'on_idle'  => sub { $conn->send("\r"); undef; },
        'timeout'  => 600
    );

    $transit_obj->change_state( to => 'CLI' );

    _set_security_defaults($ci);

    if ( $filer_pw ne '' ) {
        my $cmd = "passwd";
        $conn->execute(
            'command' => $cmd,
            'match_table' =>
                [ 'New password:|Retype new password:' => $filer_pw ]
        );
    }
    $ci->refresh_command_interface();
}

sub _license {
    my ( $ci, $service ) = @_;
    my @services = ref($service) ? @{$service} : ($service);

    foreach my $feature (@services) {
        NACL::STask::SystemLicense->add(
            command_interface    => $ci,
            feature              => $feature,
            nacltask_if_licensed => "pass"
        );
    }
}

sub _basic_setup {

    $Log->enter() if $may_enter;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            setup    => { type => SCALAR, optional => 1 },
            timeout  => { type => SCALAR, default  => 60 },
            conn_obj => { type => OBJECT },
        },
        allow_extra => 1
    );

    my $setup    = $opts{setup} || {};
    my $conn     = $opts{conn_obj};
    my $host_obj = $conn->hostp();
    my $timeout  = $opts{timeout};

    my %Cache_licenses = (
        netcache  => "GXVPYWA",
        nntp      => "WJSRXFF",
        mms       => "EKMAREM",
        real      => "YMWRXFF",
        quicktime => "MOWPYWA",
        icap      => "GFCJHSI",
        grm       => "WRQLGBN",
        ncagent   => "GJOLDQC",
        https     => "IQWRXFF",
        pnfs      => "SZNNCZG",
    );

    # Pull the parameters from various sources into a single hash:

    my %defaults = (
        GATEWAY                     => "",
        LOCATION                    => "",
        DNS                         => "",
        DNS_SERVER_LIST             => "",
        DNS_DOMAIN                  => "",
        DNS_DOMAINNAME              => "",
        NIS                         => "",
        NIS_SERVER_LIST             => "",
        NIS_DOMAIN                  => "",
        NIS_DOMAINNAME              => "",
        AUTOSUPPORT                 => "n",
        TIMEZONE                    => "",
        ADMIN_HOST                  => "",
        PRIMARY_CONTACT_NAME        => "",
        PRIMARY_CONTACT_PHONE       => "",
        PRIMARY_CONTACT_ALT_PHONE   => "",
        PRIMARY_CONTACT_EMAIL       => "",
        SECONDARY_CONTACT_NAME      => "",
        SECONDARY_CONTACT_PHONE     => "",
        SECONDARY_CONTACT_ALT_PHONE => "",
        SECONDARY_CONTACT_EMAIL     => "",
        BUSINESS_NAME               => "",
        BUSINESS_ADDR_STREET        => "",
        BUSINESS_ADDR_CITY          => "",
        BUSINESS_ADDR_STATE         => "",
        BUSINESS_ADDR_COUNTRY       => "",
        BUSINESS_ADDR_POSTALCODE    => "",
    );
    my %params = param_merge(
        hash         => \%defaults,
        hostrec      => $host_obj,
        param_prefix => $opts{prefix},
        hash         => $setup,
    );
    $Log->debug( "This is the param_merge output : \n" . Dumper( \%params ) );

    # Shuffle deprecated parameters into non-deprecated ones
    foreach my $key ( keys %params ) {
        my ($suffix) = ( $key =~ /^NETMASK(.*)/ );
        next if !defined($suffix);
        $params{"MASK$suffix"} ||= $params{"NETMASK$suffix"};
    }
    $params{DNS_DOMAIN} ||= $params{DNS_DOMAINNAME};
    $params{NIS_DOMAIN} ||= $params{NIS_DOMAINNAME};

    # Construct an array of DNS servers from DNS_SERVER_LIST

    my @dns_server_list = split( /,/, $params{DNS_SERVER_LIST} );
    my $dns = $params{DNS}
        || ( ( $params{DNS_DOMAIN} && @dns_server_list ) ? "y" : "n" );
    my $root_pass = $host_obj->default_password();
    if ( defined( $params{ROOT_PASS} ) ) {
        $root_pass = $params{ROOT_PASS};
    }

    # Construct comma separated list of NIS servers in NIS_SERVER_LIST
    # (remove extra spaces, as the filer does not allow them)

    my $nis_server_list = $params{NIS_SERVER_LIST};
    $nis_server_list =~ s/\s+//g;
    my $nis = $params{NIS}
        || ( ( $params{NIS_DOMAIN} && $nis_server_list ) ? "y" : "n" );

    my $cmd = ( $opts{issue_setup_cmd} ) ? "setup" : "";

    # Let's define a pattern to match the interface types that setup
    # might query us for.

    my $failure;
    my @prompts_setup = (
        "Are you sure you want to continue"         => "y",
        "Do you want to configure interface groups" => "n",
        "new hostname"                              => $params{HOSTNAME},
        "(e|E)nable IPv6\\?"                        => "n",
        "virtual network interfaces"                => "n",
        "IP address for (?:FC Network Interface|VI Network Interface|Network Interface|SPANS interface) (\\w+)\\W"
            => sub { _get_ifc_info( \%params, $1, "IP" ) || ""; },
        "address for (?:FC Network Interface|VI Network Interface|Network Interface|SPANS interface) (\\w+)\\W"
            => sub { _get_ifc_info( \%params, $1, "IP" ) || ""; },
        "netmask for (VI )?Network Interface (\\w+)\\W" => sub {
            _get_ifc_info( \%params, $2, "MASK" )
                || $params{MASK}
                || "255.255.255.0";
        },
        "Should interface (\\w+) take over " => sub {
            _get_ifc_info( \%params, $1, "PARTNER" ) ? "y" : "n";
        },
        "Should adapter (\\w+) take over ELANs" => sub {
            _get_ifc_info( \%params, $1, "PARTNER" ) ? "y" : "n";
        },
        "name to be taken over by (\\w+)\\W" => sub {
            _get_ifc_info( \%params, $1, "PARTNER" );
        },
        "Please enter the ATM adapter to take over" => sub {
            _get_ifc_info( \%params, $1, "PARTNER" );
        },
        "media type for (\\w+)\\W" => sub {
            _get_ifc_info( \%params, $1, "MEDIATYPE" ) || "";
        },
        "flow control for (\\w+).*: " => sub {
            _get_ifc_info( \%params, $1, "FLOWCONTROL" ) || "";
        },
        "want (\\w+) to support jumbo frames\\?" => sub {
            _get_ifc_info( \%params, $1, "JUMBOFRAMES" ) || "";
        },
        "enter name of ELAN to add to ATM adapter" => "",
        "setup through the web interface"          => "n",
        "default gateway"                          => $params{GATEWAY},
        "IP address of the administration host"    => $params{ADMIN_HOST},
        "filer located"                            => $params{LOCATION},
        "enter timezone"                           => $params{TIMEZONE},
        "root directory for HTTP files"            => "",
        "DNS resolver"                             => $dns,
        "DNS domain name"                          => $params{DNS_DOMAIN},
        "IP address for [a-z]+ nameserver"         => sub {
            shift(@dns_server_list) || "";
        },
        "another nameserver" => sub {
            @dns_server_list ? "y" : "n";
        },
        "NIS client"      => $nis,
        "NIS domain name" => $params{NIS_DOMAIN},
        "NIS servers"     => $nis_server_list,
        "run autosupport" => "n",
        "New password"    => $root_pass,
        "new password"    => $root_pass,

#        "(N|n)ew password" => $root_pass
#        ,    #this covers both 'New password' and 'Retype new password'
        "Password:" => $root_pass
        ,    #if running via console you get prompted to login with new pass
        "What language .* for multi-protocol files" => "",
        "Number of disks to add to root volume" => "0",   # (pre-redhook only)
        "Would you like to configure the RMC LAN interface" => "n",
        "Would you like to configure the RLM LAN interface" => "n",
        "Would you like to configure the BMC"               => "n",
        "Do you want to configure the Shelf Alternate Control Path Management interface for SAS shelves"
            => "n",

        #hitting ctrl-c here because basic setup should not configure CIFS
        #there are other libraries that exists to run the CIFS setp wizard
        #this prompt does not always appear, except in special cases
        "Ctrl-C to exit without committing changes" => "\cc",

# more prompts to add later
#"Would you like to enable DHCP on the RMC LAN interface [y]:"
#"Please enter media type for the RMC {100tx-fd, tp-fd, 100tx, tp, auto} [auto]:"
        "Errors during setup, no changes made" => sub {

            # don't die yet; there's a filer prompt that our caller
            # still has to consume.
            $failure
                = "setup aborted with \"Errors during setup, no changes made\"";
            return undef;
        },

        #console collisions make looking for "Press retrun key to continue"
        #impossible thus looking for smaller subset of the text
        "key to continue" => "",

        # (After change 200344.  It is letting us choose between
        # "netcache" and "pnfs"; we always choose netcache for now)
        "Please enter your license: " => $Cache_licenses{netcache},

        # (Before change 200344)
        "enter your NetCache license" => $Cache_licenses{netcache},

        # (end change 200344)
        "enter your NNTP license" => "",

        # (After change 284296)
        "WM \\(.*\\) protocol license" => "",

        # (Before change 284296)
        "MMS \\(.*\\) protocol license" => "",

        # (end change 284296)
        "MMS Pro license"          => "",
        "RTSP \\(\\S+\\) license"  => "",
        "ICAP license"             => "",
        "GRM Server license"       => "",
        "GRM Agent license"        => "",
        "HTTPS proxy license"      => "",
        "Telnet proxy license"     => "",
        "PNFS license"             => "",
        "SOCKS license"            => "",
        "HTTP Compression license" => "",

        # prompts required for IBM branded machines
        "Name of primary contact"         => $params{PRIMARY_CONTACT_NAME},
        "Phone number of primary contact" => $params{PRIMARY_CONTACT_PHONE},
        "Alternate phone number of primary contact" =>
            $params{PRIMARY_CONTACT_ALT_PHONE},
        "Primary Contact e-mail address or IBM WebID" =>
            $params{PRIMARY_CONTACT_EMAIL},
        "Name of secondary contact" => $params{SECONDARY_CONTACT_NAME},
        "Phone number of secondary contact" =>
            $params{SECONDARY_CONTACT_PHONE},
        "Alternate phone number of secondary contact" =>
            $params{SECONDARY_CONTACT_ALT_PHONE},
        "Secondary Contact e-mail address or IBM WebID" =>
            $params{SECONDARY_CONTACT_EMAIL},
        "Business name"               => $params{BUSINESS_NAME},
        "Business address"            => $params{BUSINESS_ADDR_STREET},
        "City where business resides" => $params{BUSINESS_ADDR_CITY},
        "State where business resides" =>
            $params{BUSINESS_ADDR_STATE},    # optional
        "2-character country code" => $params{BUSINESS_ADDR_COUNTRY},
        "Postal code where business resides" =>
            $params{BUSINESS_ADDR_POSTALCODE},    # optional
    );

    return eval {
        $conn->execute_expect(
            'match_table' => \@prompts_setup,
            'timeout'     => $timeout
        );
        if ($failure) {
            $Log->exit() if $may_exit;
            die "$failure\n";
        }
        $Log->exit() if $may_exit;
        return 1;
    };

}

sub _get_ifc_info {
    my ( $hash, $ifname, $field ) = @_;
    my $result = undef;
    $result = $hash->{ $field . '_' . $ifname };

    return $result if defined $result;
    if ( $field eq 'IP' || $field eq 'MASK' ) {
        while ( my ( $key, $value ) = each %$hash ) {
            if ( $key =~ /(CLUSTER|DATA|MGMT)_PORT(.*)/ && $ifname eq $value )
            {
                $result = $hash->{ $1 . '_' . $field . $2 };
                return $result;
            }
        }
    }
    return "";
}

sub _set_security_defaults {

    my ( $ci, %opts ) = @_;
    $ci->refresh_command_interface();
    my $apiset_obj = $ci->apiset( connid => 'console' );
    $opts{"rsh.enable"}
        = defined $opts{"rsh.enable"} ? $opts{"rsh.enable"} : "on";
    $opts{"rsh.access"}
        = defined $opts{"rsh.access"} ? $opts{"rsh.access"} : "all";
    $opts{"ssh.enable"}
        = defined $opts{"ssh.enable"} ? $opts{"ssh.enable"} : "on";
    $opts{"httpd.enable"}
        = defined $opts{"httpd.enable"} ? $opts{"httpd.enable"} : "on";
    $opts{"rsh.enable"}
        = defined $opts{"rsh.enable"} ? $opts{"rsh.enable"} : "on";
    $opts{"telnet.enable"}
        = defined $opts{"telnet.enable"} ? $opts{"telnet.enable"} : "on";

    $opts{"security.passwd.rules.everyone"}
        = defined $opts{"security.passwd.rules.everyone"}
        ? $opts{"security.passwd.rules.everyone"}
        : "off";

    foreach my $option ( keys %opts ) {
        my $value = $opts{$option};
        $apiset_obj->options( $option => $value );
        my $parsed_output
            = $apiset_obj->options($option)->get_parsed_output();
        if ( $parsed_output->[0]->{$option} ne $value ) {
            $Log->warn("Failed to set $option to $value");
        }
    }
    Tharn::snooze(20);
}

1;
