# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Performing LunMount setup in client.
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::MTask::Client::LunMount

=head1 DESCRIPTION

C<NACL::MTask::Client::LunMount> provides a number of well-defined but
potentially complex or multi-step methods related to Lun Mount in Client.
The steps include lun discovery on the client,creating a file system
in the device and doing a mount on the lun in the client.

=head1 ATTRIBUTES

=head2 _lun_mount_obj

(Required, Applicable for linux client) This attribute is of type
C<NACL::STask::Client::Mount> which contains the mount-point information.
Will be array ref, if multiple lun paths were passed, will return scalar if
only one lun path was passed

=head2 device

(Required) This attribute will contain the device on which the mount is done.

=head2 session_id

(Required, Applicable for windows client) This attribute will contain the session-id
which is created during lun discovery on windows.

=head2 filer_ip

(Required) This attribute contains the filer ip-address used for lun mount operation.

=head2 disk_number

(Required, Applicable for windows client) This attribute will contain the disk number
used in the windows client for lun mount operation.

=head2 create_partition_script_file

(Required, Applicable for windows client) This attribute will contain the name of the diskpart
script file which was used for creating the partition. This file-name will be deleted in the
purge() method call.

=head2 format_partition_script_file

(Required, Applicable for windows client) This attribute will contain the name of the diskpart
script file which was used for formatting the partition. This file-name will be deleted in the
purge() method call.

=head2 command_interface

(Required) This attribute will contain the client command interface which is of type
C<NACL::C::Client>.

=head1 METHODS

=head2 setup

    # Example usage for setup method
    my $mount_obj = NACL::MTask::Client::LunMount->setup(
        command_interface => $command_interface,
        lun-path   => $lun_path,
        ip-address => $ip_address,
        filer-iqn  => $iqn_name,
    );

    #Example Usage with lif and vserver inputs
    #This is prefered over passing the IPAddress and filer-iqn directly

    my $mount_obj = NACL::MTask::Client::LunMount->setup(
        command_interface => $Client,
        'lun-path'        => $lun_path,
        filer_command_interface => $Node,
        vserver => $Vserver_Name,
        filer_lif => $Lif

    );

    # Example usage for passing multiple LUNs which fall under same vserver
    my $mount_obj = NACL::MTask::Client::LunMount->setup(
        command_interface => $Client,
        'lun-path'        => [@lun_paths],
        filer_command_interface => $Node,
        vserver => $Vserver_Name,
        filer_lif => $Lif

    );

(Class method)
This method is used to do lun discovery, create filesystem, LunMount in
Linux or Solaris Environment.

In case of Linux it does the following:

    1) It discovers targets at a given IP address.

    2) After getting node record id, it logins to the node.

    3) It restarts the iscsi service, if it is unable to log in.

    4) It manually log out and log into targets to discover new luns.

    5) It fetches the device file name using the sanlun lun show.

    6) It makes the file system with the device file name.

    7) Mounts the file system in the client with the specified mount point.

In case of Solaris it does the following:

    1) Adds the given target to the list of discovery addresses.

    2) Enabling the discovery method which initiates the discovery.

    3) It fetches the device file name using the sanlun lun show.

    4) Disable and Enable the discovery if sanlun lun show has no elements.

    5) Makes Filesystem using the device.

    6) Mount the path.

In case of Windows it does the following:

    1) Adds the filer-ip address as the targetportal.

    2) Creates a iscsi login session using the initiator information on the
       windows machine. With this step a session-id is created.

    3) Once this is done, a disk is created which is in offline state.
       The disk attributes are cleared and the disk is brought to online state.

    4) Once this disk is online, a raw format is done using diskpart command
       in order to make the disk useable.

    5) Then the filesystem is created on the raw disk based on the inputs given
       such as filesystem type,type of format,etc.

For Linux & Solaris type of clients, this method returns a C<NACL::MTask::Client::LunMount>
 Object on success and throws exception on failure.

For Windows type, this method returns a C<NACL::STask::Client::LunMount> Object
on success or it will throw exception on failure. We should also check if "mpc" binary
is installed in the windows machine and while running the script we need to pass the
WINDOWS_MINWIN_TOOLS_DIR directory value as the mpc path.

In case of errors, it throws the exception thrown by the APILayer.

To skip lun discovery,set the option do_lun_discovery=0.
In linux this will suppress executing steps#1-5 mentioned above.
In Solaris this will suppress executing steps#1-3 mentioned above.

To skip make filesystem also, set the option make_filesystem=0.
In linux this will suppress executing step#6 given above.
In solaris this will suppress executing step#5 given above.

=over

=item Options

=over

=item C<< mount-point => $mount-point >>

(Optional)Argument to create the mount point for the file system made.

=item C<< lun-path => $lun-path >>

Argument to check the lun mapping and the device file name to create the
filesystem. This can be a scalar for one path or arrayref if we want to pass
multiple paths. If multiple paths are passed. Setup is common only mounting
will be in a loop.

=item C<< ip-address => $ip_address >>

(Optional) Iscsi lif ip address to discover the target.
Required if the values of filer_command_interface,vserver,filer_lif input
values are not provided.
Required if the client used is Windows.

=item C<< size => $size >>

(Optional) The size in sectors to make file system in solaris. Default value is 2048.

=item C<< filer-iqn => $iqn_number >>

(Optional) The name of the initiator node associated with the particular lun.
Required if the values of filer_command_interface,vserver,filer_lif input
values are not provided.

=item C<< no_of_retries => $value >>

(Optional) The no.of times the iscsi service is restarted in order to populate the luns
using 'sanlun lun show' command. ( default : 3 )

=item C<< timeout_multiplier => $val >>

(Optional) The multiplier value by which the individual timeout values should be increased.
( default : 3 ).

=item C<< do_lun_discovery => 0 | 1 >>

(Optional) Boolean flag that indicates if lun discovery should be done or not.
Takes default value 1. If set to 0, lun discovery will be skipped. Note that
if this parameter set to 0, then its required to provide the device value as input,
for the method to do makefilesystem and lunmount. Note that it is an invalid combination
of inputs to set do_lun_discovery=1 and make_filesystem=0, because lunmount is done
as the final step of this setup method.

=item C<< make_filesystem => 0 | 1 >>

(Optional) Boolean flag that indicates if make filesystem needs to be carried out or not.
Takes default value 1. If set to 0, make filesystem will  be skipped. Note that it is an
invalid combination of inputs to set do_lun_discovery=1 and make_filesystem=0, because
lunmount is done unconditionally as the final step of this setup method.

Valid combination of do_lun_discovery and make_filesystem flags would be :

do_lun_discovery=0 and make_filesystem=0
In this case only lunmount will be carried out.In this case its required to provide the
device value as input for lunmount.Also note that, for this combination of inputs
one might consider calling the method C<< lun_mount >> of this package.

do_lun_discovery=0 and make_filesystem=1
In this case lun discovery will be skipped and makefilesystem and lunmount would be
performed. In this case its required to provide the device value as input
for makefilesystem and lunmount.

do_lun_discovery=1 and make_filesystem=1
This is the default combination of these flags. In this case both lun discovery and make_filesystem
are performed before lunmount.

=item C<< device => $scalar_value >>

The device discovered using lun discovery.
(Optional) If lun discovery is done through the setup method then this value is not required.
It is required otherwise. Note that lun discovery is done by the setup method, unless the
flag do_lun_discovery is set to 0 explicitly.

=item C<< lun_mount_params => $hashref >>

Mount params to be passed to NACL::STask::Client::LunMount.
(Optional)

=item C<< 'filer_command_interface' => $fci >>

(Optional) Command Interface of the filer node, which is the target of lun discovery.
This has to be an object of type NACL::C::CommandInterface::ONTAP. Accepts the object
returned by NACL::C::Node->find().

=item C<< 'vserver' => $vserver >>

(Optional) Name of the vserver, where the lun resides in the filer. This can take the value
of an object of type NACL::C::Vserver or the name of the vserver.

=item C<< 'filer_lif' => $lif >>

(Optional) Name of the vserver data lif, where the lun is located.

If the combination of input values 'filer_command_interface', 'vserver' and 'filer_lif'
are provided, then the values of 'ip-address' and 'filer-iqn' are not required.
The ip-address for discovery and mount would be searched using the above three input
values in the filer and used.  But if the user passes an ip-address explicitly, then that
would be used. The recommended usage would be to provide the three co-ordinates, so that
the routine would search in the filer and use it in setup.

=item C<< 'quick_format' => $vserver >>

(Optional, Applicable only for Windows, value as 1|0) This option is required
when we need to do a quick format.
Default : 1 (which means do a quick-format).

=item C<< 'file_system' => $vserver >>

(Optional, Applicable only for Windows) Name of the windows file-system.
Possible values are FAT, FAT32, exFAT, NTFS, or UDF.
Default : FAT .

=item C<< 'drive' => $drive_letter >>

(Optional, Applicable only for Windows) Drive letter which will be assigned
to the disk.
Default : D .

=item C<< 'current_label' => $vserver >>

(Optional, Applicable only for Windows) Name of the label which will be assigned
 to the first format which occurs.
Default : NEW_LABEL.

=item C<< 'label' => $vserver >>

(Optional, Applicable only for Windows) Name of the label which will be assigned
 if the current label needs to be changed.
Default : NEW_LABEL.

=item C<< 'create_partition_script_file' => $vserver >>

(Optional, Applicable only for Windows) Name of the file to be created which
contains the diskpart steps to create a partition.
Default : "create_partition.txt".

=item C<< 'format_partition_script_file' => $vserver >>

(Optional, Applicable only for Windows) Name of the file to be created which
contains the diskpart steps to format a raw disk and make it useable.
Default : "format_partition.txt".

=back

=back

=cut

package NACL::MTask::Client::LunMount;

use strict;
use warnings;

use base qw/NACL::MTask::MTask/;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::STask::STask;
use NACL::CS::Client::Sanlun;
use NACL::C::Client::Mkfs;
use NACL::STask::Client::Mount;
use NACL::C::Client::Multipath;
use NACL::CS::VserverIscsi;
use NACL::C::Client::IscsiTarget;
use NATE::Exceptions::Argument qw(:try);
use Params::Validate qw(validate_with SCALAR OBJECT BOOLEAN ARRAYREF HASHREF);
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::ResponseException;
use NACL::ComponentUtils qw( _optional_scalars );
use NACL::C::Client::Iscsi;
use NACL::Exceptions::NoElementsFound;
use Net::IP qw(ip_is_ipv4);

use Class::MethodMaker [
new    => [ '-hash', 'new' ],
scalar => '_lun_mount_obj',
scalar => 'lun_path',
scalar => 'filer_iqn',
scalar => 'device',
scalar => 'session_id',
scalar => 'filer_ip',
scalar => 'disk_number',
scalar => 'create_partition_script_file',
scalar => 'format_partition_script_file',
scalar => [
        { -type => 'NACL::C::Client' },
        'command_interface',
    ],
];

sub setup() {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'mount-point'             => { type => SCALAR,  optional => 1 },
            lun_mount_params          => { type => HASHREF, default => {} },
            'lun-path'                => { type => SCALAR|ARRAYREF,  optional => 1 },
            'ip-address'              => { type => SCALAR,  optional => 1 },
            'filer-iqn'               => { type => SCALAR,  optional => 1 },
            'size'                    => { type => SCALAR,  default  => 2048 },
            'no_of_retries'           => { type => SCALAR,  default  => 3 },
            'timeout_multiplier'      => { type => SCALAR,  default  => 3 },
            'do_lun_discovery'        => { type => BOOLEAN, default  => 1 },
            'make_filesystem'         => { type => BOOLEAN, default  => 1 },
            'device'                  => { type => SCALAR,  optional => 1 },
            'drive'                   => { type => SCALAR,  default  => "D" },
            quick_format                 => { type => BOOLEAN, default  => 1 },
            file_system                  => { type => SCALAR,  default  => "FAT" },
            current_label                => { type => SCALAR,  default  => "NEW_LABEL" },
            label                        => { type => SCALAR,  default  => "NEW_LABEL" },
            create_partition_script_file => { type => SCALAR,  default  => "create_partition.txt" },
            format_partition_script_file => { type => SCALAR,  default  => "format_partition.txt" },
            'filer_command_interface' => {
                                            type     => OBJECT,
                                            isa      => 'NACL::C::CommandInterface::ONTAP',
                                            optional => 1
                                         },
            'vserver'                 => {
                                             type     => OBJECT  | SCALAR ,
                                             optional => 1,
                                             depends  => [qw(filer_command_interface)]
                                          },
            filer_lif                 =>  {  type      => SCALAR ,
                                             optional  => 1,
                                             depends   => [  qw(filer_command_interface) ]
                                          },
            },
        allow_extra => 1,
    );

    ##Used to verify that the combination of flags provided are all valid
    $pkg->_verify_inputs(%opts);

    my ( $filer_ip, $iqn );

     ##If the user passes the ip-address option, then that is used directly
     ##in discovery and mount. Otherwise query the filer and get the value
     ##of ip-address of lif
     if ( !$opts{'ip-address'} ) {
	     ( $filer_ip, $iqn ) = $pkg->_handle_inputs(%opts);
         $opts{'ip-address'} = $filer_ip;
     }
     ##If the user passes the filer-iqn option, then that is used directly
     ##in discovery . Otherwise query the filer and get the values
     ##of target-name of filer

    if ( !$opts{'filer-iqn'} ) { $opts{'filer-iqn'} = $iqn; }

    my $result = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                set    => 'Linux',
                method => '_linux_config'
            },
            {
                set    => 'Solaris',
                method => '_solaris_config'
            },
            {
                set    => 'Windows',
                method => '_windows_config'
            },
        ],
        apiset_should => { interface => "CLI" },        
    );

    $Log->exit() if $may_exit;
    return $result;
} ## end sub setup


=head2 lun_mount

    # Example usage for lun_mount method
    my $mount_obj = NACL::MTask::Client::LunMount->lun_mount(
        command_interface => $command_interface,
        device => $device,
        filer_command_interface => $fci,
        vserver => $fci,
        filer_lif => $lif,
        'mount-point' => $mount_point
    );

(Class method)
This method will do the LunMount in Linux or Solaris Environment.

This method returns a C<NACL::STask::Client::Mount> Object on success
and throws exception on failure.
In case of errors,it throws the exception thrown by the APILayer.

=over

=item Options

=over

=item C<< mount-point => $mount-point >>

(Optional)Argument to create the mount point for the file system made.


=item C<< device => $scalar_value >>

The device discovered using lun discovery.


=item C<< lun_mount_params => $hashref >>

(Optional)Mount params to be passed to NACL::STask::Client::LunMount.

=item C<< 'filer_command_interface' => $fci >>

(Optional) The command interface of the filer, which is the target of the lun discovery.

=item C<< 'vserver' => $vserver >>

(Optional) Name of the vserver, where the lun resides in the filer. This can take the value
of an object of type NACL::C::Vserver or the name of the vserver.

=item C<< 'filer_lif' => $lif >>

(Optional) Name of the vserver data lif, where the lun is located.

=back

=back

=cut

sub lun_mount {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    my $mount
        = $pkg->setup( %opts, do_lun_discovery => 0, make_filesystem => 0 );

    $Log->exit() if $may_exit;
    return $mount;
} ## end sub lun_mount


## Local subroutine used to take the values of filer_command_interface,
## vserver and filer_lif and return the ip-address corresponding to the
## vserver's data lif along with the target-name of the filer.
sub _handle_inputs {
    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;

    my $fci     = $opts{filer_command_interface};
    my $vserver = $opts{vserver};
    my $v_name;
    if ( ref $vserver ) {
        $v_name = $vserver->vserver();
    } else {
        $v_name = $vserver;
    }

    my $lif = $opts{filer_lif};
    $Log->debug("This is the lif $lif");
    my $c_state = NACL::CS::NetworkInterface->fetch(
        command_interface => $fci,
        filter            => {
            vserver => $v_name,
            lif     => $lif
        },
        requested_fields => [qw(address data-protocol)]
    );
    my $filer_ip  = $c_state->address();
    my $dataproto = $c_state->data_protocol();
    $Log->debug("This is the dataproto $dataproto->[0]");
    my $component;

    if ( $dataproto->[0] =~ /iscsi/i ) {
        $component = 'VserverIscsi';
    } elsif ( $dataproto->[0] =~ /fcp/i ) {
        $component = 'VserverFcp';
    }

    my $comp_cs = "NACL::CS::" . "$component";
    my $state   = $comp_cs->fetch(
        command_interface => $fci,
        filter            => { vserver => $v_name },
        requested_fields  => [qw(target-name)]
    );

    my $iqn_name = $state->target_name();

    $Log->exit() if $may_exit;
    return ( $filer_ip, $iqn_name );

} ## end sub _handle_inputs

##################################################################################
## Local subroutine used to verify that the combination
## of flags/options passed to the setup routine are all valid
## Following verifications are carried in the input hash passed by the caller
##
## 1. If do_lun_discovery=0, lun discovery will be skipped. For the
##    same reason the device name to be used for mapping needs to be
##    passed. The method throws a NATE::Exceptions::Argument if the value for
##    device is not passed in this case.
##
## 2. If do_lun_discovery=1 and make_filesystem=0,make filesystem
##    is skipped and hence lunmount cannot be done by the method in the end.
##    The method throws a  NATE::Exceptions::Argument exception in this case.
##
## 3. If the input ip-address is passed, the method logs a warning
##    as it is recommended to pass the name of the data lif along with filer command
##    interface.
##
## 4. If the input filer-iqn is passed, the method logs a warning as it is
##    recommended to pass the vserver name along with filer command interface.
##
## 5. If the caller passes both filer_lif and ip-address in the input, the method logs
##    a warning as only one of those could be passed. However, since the caller has
##    passed the ip-address, that will be used directly. filer_lif is discarded and
##    will not be used to discover the ip-address for mount.
##
## 6. If  the caller passes both vserver and filer-iqn, the method logs a warning
##    as only one of those could be passed. However since the caller has passed the
##    filer-iqn, that will be used directly. vserver value is discarded and will not
##    be used to discover the target name of the filer.
##
## 7. If both ip-address and filer_lif are not passed then the method throws a
##    NATE::Exceptions::Argument as it is mandatory to pass one of those for mount.
##
## 8. If both filer-iqn and vserver are not passed then the method throws a
##    NATE::Exceptions::Argument as it is mandatory to pass one of those for mount.
##
########################################################################################

sub _verify_inputs {
    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;

    if ( ( !$opts{do_lun_discovery} ) && ( !$opts{device} ) ) {

        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
                  "Since option do_lun_discovery=0 lun discovery will  \n"
                . "not be done through this method call.\n Hence the name of the "
                . "device to be mounted has to be passed mandatorily. " );
    }

    if ( ( $opts{do_lun_discovery} ) && ( !$opts{make_filesystem} ) ) {

        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "option do_lun_discovery=1, make_filesystem=0; This is an \n "
                . "invalid combination of input options. Please verify.  " );

    }

    if ( ( !$opts{do_lun_discovery} ) && ( !$opts{make_filesystem} ) ) {
        $Log->warn( "option do_lun_discovery=0, make_filesystem=0; \n"
                . "Skipping discovery and makefilesystem."
                . " Only lunmount will be performed " );
    }

    if ( $opts{'ip-address'} ) {
        $Log->warn( "Instead of passing IP Address of the Data Lif, "
                . " Pass the filer_command_interface and vserver "
                . " and Lif values to the input opts. " );
    }
    if ( $opts{'filer-iqn'} ) {
        $Log->warn( "Instead of passing filer-iqn , Pass the "
                . "filer_command_interface and vserver values to the input opts. "
        );
    }

    if ( $opts{'ip-address'} && $opts{filer_lif} ) {

        $Log->warn( "Its sufficient to pass one of 'ip-address' "
                . " or preferably 'filer_lif' to the input hash" );
        $Log->comment( "Since both ip-address of the data lif "
                . " and filer_lif values are passed,\n"
                . " only the ip-address will be considered " );
    }

    if ( $opts{'filer-iqn'} && $opts{vserver} ) {
        $Log->warn( "Its sufficient to pass one of 'filer-iqn' or "
                . " preferably 'vserver' to the input hash" );
        $Log->comment( "Since both filer-iqn and vserver values are passed,\n"
                . "only the filer-iqn will be considered " );
    }

    if ( ( !$opts{'ip-address'} ) && ( !$opts{'filer_lif'} ) ) {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw(
            "Its required to pass the value of filer_lif to the input hash");
    }
    #if ( ( !$opts{'iqn-name'} ) && ( !$opts{'vserver'} ) ) {
    #        $Log->exit() if $may_exit;
    #        NATE::Exceptions::Argument->throw(
    #        "Its required to pass the value of vserver to the input hash");
    #}

    $Log->exit() if $may_exit;
} ## end sub _verify_inputs


sub _linux_config() {

    $Log->enter() if $may_enter;

    my ( $pkg, %opts ) = @_;

    my $do_lun_discovery = delete $opts{do_lun_discovery};
    my $make_filesystem  = delete $opts{make_filesystem};

    my ( $device, $mount );

    $device = $opts{device} if $opts{device};

    ## This means that if the user has passed the device name
    ## and set do_lun_discovery=1 then discovery would be done again .
    ## This device would only be used in making file system too.

    if ($do_lun_discovery) {
        $device = $pkg->_linux_lun_discovery(%opts);
    } else {
        $Log->comment(
                  " do_lun_discovery is set to '0' in the input option. \n"
                . " Skipping lun discovery in the setup() method.\n"
                . " Since the option device is set, that value will be taken for mount "
        );
    }
    if ($make_filesystem) {

        my $apiset = $opts{command_interface}->apiset(interface => 'CLI');
        $pkg->_linux_mkfs(
            command_interface => $opts{command_interface},
            apiset            => $apiset,
            device            => $device
        );
    } else {
        $Log->comment(
                  "make_filesystem is set to '0'.Skipping make filesystem "
                . " in the setup() method " );
    }

    ## LunMount would be done by default
    my %lun_mount_params = (
        command_interface => $opts{command_interface},
        device            => $device,
        server            => $opts{'ip-address'},
        %{$opts{lun_mount_params}},
    );

    if ( defined $opts{'mount-point'} ) {
        %lun_mount_params
            = ( %lun_mount_params, 'mount_point' => $opts{'mount-point'} );
    }
    $Log->comment(" Doing Lun Mount in the setup method ");
    $mount = $pkg->_linux_lunmount(%lun_mount_params);
    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface => $opts{command_interface},
        _lun_mount_obj    => $mount,
        device            => $device,
        filer_ip          => $opts{'ip-address'},
        filer_iqn         => $opts{'filer-iqn'},
    );
} ## end sub _linux_config()


=head2 lun_discovery

    # Example usage
    my $device = NACL::MTask::Client::LunMount->lun_discovery(
        command_interface => $command_interface,
        lun-path   => $lun_path,
        filer_command_interface => $fci,
        vserver => $vserver,
        filer_lif => $lif
    );

(Class method)
This method discovers the given lun in the client command interface
and returns the mapped device name, in linux and solaris.

=over

=item Options

=over

=item C<< lun-path => $lun-path >>

Argument to check the lun mapping and the device file name to create the
filesystem.

=item C<< no_of_retries => $value >>

(Optional) The no.of times the iscsi service is restarted in order to populate the luns
using 'sanlun lun show' command. ( default : 3 )

=item C<< timeout_multiplier => $val >>

(Optional) The multiplier value by which the individual timeout values should be increased.
( default : 3 ).

=item C<< 'filer_command_interface' => $fci >>

(Optional) The command interface of the filer, which is the target of the lun discovery.

=item C<< 'vserver' => $vserver >>

(Optional) Name of the vserver, where the lun resides in the filer. This can take the value
of an object of type NACL::C::Vserver or the name of the vserver.

=item C<< 'filer_lif' => $lif >>

(Optional) Name of the vserver data lif, where the lun is located.

=over

=over

=cut

sub lun_discovery {

    $Log->enter() if $may_enter;
    my ($pkg) = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'lun-path'                => { type => SCALAR,  optional => 1 },
            'no_of_retries'           => { type => SCALAR, default => 3 },
            'timeout_multiplier'      => { type => SCALAR, default => 3 },
            'filer_command_interface' => {
                type => OBJECT,
                isa  => 'NACL::C::CommandInterface::ONTAP',
            },
            'vserver' => { type => OBJECT | SCALAR },
            filer_lif => { type => SCALAR },
        },
        allow_extra => 1
    );

    my ( $filer_ip, $iqn );
    ( $filer_ip, $iqn ) = $pkg->_handle_inputs(%opts);
    $opts{'ip-address'} = $filer_ip;
    $opts{'filer-iqn'}   = $iqn;

    my $result = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   set    => 'Linux',
                method => '_linux_lun_discovery'
            },
            {   set    => 'Solaris',
                method => '_solaris_lun_discovery'
            },
            {   set    => 'Windows',
                method => '_windows_lun_discovery'
            },
        ]
    );
    $Log->exit() if $may_exit;
    return ( ref $result eq 'ARRAY' && scalar(@{$result}) eq 1)
        ? $result->[0]: $result;
} ## end sub lun_discovery

sub _linux_lun_discovery {

    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;
    my ( %login_opts, %discovery_opts, %logout_opts, $result, $device_obj,
        @devices );
    my $command_interface  = $opts{command_interface};
    my $no_of_retries      = $opts{no_of_retries};
    my $timeout_multiplier = $opts{timeout_multiplier};
    my $apiset             = $opts{apiset};

    $apiset->set_privilege( 'privilege-level' => "root" );
    $apiset->set_timeout( 'connectrec-timeout' => 600 * $timeout_multiplier );

    $opts{'type'}      = "sendtargets";
    $opts{'mode'}      = "1";
    $opts{'discovery'} = "1";
    $opts{'node'}      = "1";
    $opts{'login'}     = "1";
    $opts{'logout'}    = "1";

    $pkg->_hash_copy(
        source => \%opts,
        target => \%discovery_opts,
        copy   => [qw(type mode discovery )],
        map    => { 'ip-address' => 'ip-port' },
    );
    $pkg->_hash_copy(
        source => \%opts,
        target => \%login_opts,
        copy   => [qw(node login mode )],
        map    => {
            'filer-iqn'  => 'target-name',
            'ip-address' => 'ip-port'
        },
    );

    $pkg->_hash_copy(
        source => \%opts,
        target => \%logout_opts,
        copy   => [qw(node logout mode)],
        map    => {
            'filer-iqn'  => 'target-name',
            'ip-address' => 'ip-port'
        },
    );
    my $found = 0;
    my $count = 1;
AGAIN: {
        use warnings qw(exiting);
        try {
            $apiset->iscsiadm(%discovery_opts);
        } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $e = shift;
            if ($e->text()=~ /iscsiadm:\s+Cannot perform discovery\.\s+Initiatorname required/ ) {
                # iscsid daemon should be started if it is not running
                 $apiset->service_iscsi_restart();
                 $apiset->iscsiadm(%discovery_opts);
            } else {
                $Log->exit() if $may_exit;
                $e->throw();
            }
        };
        $apiset->iscsiadm(%logout_opts);
        $apiset->iscsiadm(%login_opts);
        Tharn::snooze( 30 * $timeout_multiplier );

        ## Now we accept multiple lun paths for linux, so normalize it to an arrayref
        $opts{'lun-path'} = [$opts{'lun-path'}] if(ref \$opts{'lun-path'} eq 'SCALAR');
        foreach my $lun_path (@{$opts{'lun-path'}}) {
            try {
                $device_obj = NACL::CS::Client::Sanlun->fetch(
                    command_interface => $command_interface,
                    filter            => {'lun_pathname' => $lun_path,}
                );
            }
            catch NACL::Exceptions::NoElementsFound with {
                $apiset->service_iscsi_restart();
                $count++;
                Tharn::snooze(5 * $timeout_multiplier);
                no warnings qw(exiting);
                redo AGAIN;
            };
            my $device = $device_obj->device_filename;
            push(@devices, $device) if($device);
            if ( ( !$device ) && ( $count <= $no_of_retries ) ) {
                $apiset->service_iscsi_restart();
                $count++;
                Tharn::snooze( 5 * $timeout_multiplier );
                no warnings qw(exiting);
                redo AGAIN;
            }
            NATE::BaseException->throw(
                "unable to discover the device for the particular lun")
                if ( !$device );
        }
    }

    $Log->exit() if $may_exit;
    return \@devices;
}    ## sub _linux_lun_discovery ends here

sub _windows_lun_discovery {
    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;
    my $command_interface  = $opts{command_interface};
    my $filer_ip           = $opts{"ip-address"};

    my $iscsi_type = ip_is_ipv4($filer_ip) ? "IPv4" : "IPv6";

    my ($disk_number, $session_id);
    my $apiset = $command_interface->apiset(interface => 'CLI');

    my $err_msg;
    try {
        my $initiator_op = $apiset->mpc_iscsi_show_adapters();

	## As per Burt#911493 the target and the initiator ip
	## should be of same version i.e., either IPv4 or IPv6.
	## For eg: if filer's ip is of type IPv4 , then
	## iscsi target also should be of type IPv4

        my @initiator = ();
	map{push @initiator, $_ if($_->{'type'} =~ /$iscsi_type/i)} @{$initiator_op->{parsed_output}->[0]->{adapters}};

	if(@initiator){
		$apiset->mpc_iscsi_add_targetportal(
			filername_or_ip     => $filer_ip,
			initiator_index => $initiator[0]->{index},
		);

		my $target_op  = $apiset->mpc_iscsi_show_targets();
		my $session_op = $apiset->mpc_iscsi_login(
			target_index    => $target_op->{parsed_output}->[0]->{index},
			target_name     => $target_op->{parsed_output}->[0]->{targetname},
			initiator_index => $initiator[0]->{index},
		);
		$session_id = $session_op->{parsed_output}->[0]->{sessionid};
		if(!defined($session_id)) {
			$Log->exit() if $may_exit;
			NATE::BaseException->throw("Session-id is not created during the".
				"iscsi session creation");
		}
	}else{
	    ## throw the NACL::APISet::Exceptions::ResponseException
            $err_msg = "There is no host iscsi adapter of type : $iscsi_type
                    which is of same type of $filer_ip , burt#911493";
            $Log->debug($err_msg);
            NACL::APISet::Exceptions::ResponseException->throw(text => $err_msg);
	}

        my $disk_show  = $apiset->mpc_show_disks();
        my $disk_value = $disk_show->{parsed_output}->[0]->{disk};
        $disk_number   = $1 if($disk_value =~ /Disk(\d+)/i);
        if(!defined($disk_number)) {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw("Not able to fetch the disk info".
                "using mpc show disks");
        }
    } catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = $_[0];
            $Log->exit() if $may_exit;
            NACL::APISet::Exceptions::ResponseException->throw(text => "Lun discovery failed : $exception_object->text()");
    };

    #steps to online the disk created & make it useable
    #This requires the diskpart script execution

    my $drive = $pkg->_create_diskpart(
        command_interface => $command_interface,
        disk_number       => $disk_number,
        new_label         => $opts{current_label},
        create_partition_script_file
            => $opts{create_partition_script_file},
        format_partition_script_file
            => $opts{format_partition_script_file},
        drive => $opts{drive},
    );

    $Log->debug( sub{" Partition created successfully on drive $drive" } );
    $Log->exit() if $may_exit;
    return $pkg->new(
         command_interface => $opts{command_interface},
         device            => $drive,
         filer_ip          => $filer_ip,
         session_id        => $session_id,
         disk_number       => $disk_number,
         create_partition_script_file =>
             $opts{create_partition_script_file},
         format_partition_script_file =>
             $opts{format_partition_script_file},
    );
}

=head2 make_filesystem

    # Example usage
    my $device = NACL::MTask::Client::LunMount->make_filesystem(
        command_interface => $command_interface,
        device => $device,
        size   => $size
    );

This method creates filesystem on a lun discovered earlier,
in a linux or solaris client.

=over

=item Options

=over

=item C<< device => $scalar_value >>

(Required) The device discovered using lun discovery.

=item C<< size => $size >>

(Optional) The size in sectors to make file system in solaris. Default value is 2048.

=item C<< 'command_interface' => $ci >>

(Optional) The command interface of the client, in which the filesystem will be created.

=item C<< "file-system" => $opts{file_system} >>

(Optional, Applicable for Windows) The type of filesystem which needs to be created.

=item C<< label => $label >>

(Required, Applicable for Windows) The label which needs to be given for the volume
 mounted on the windows machine.

=item C<< current_label => $scalar_val >>

(Required, Applicable for Windows) The existing label name assigned to the volume.

=item C<< "quick-format" => $boolean_value >>

(Optional, Applicable for Windows) Option to quick format the filesystem.

=over

=over

=cut

sub make_filesystem {

    $Log->enter() if $may_enter;

    my ($pkg) = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            command_interface => {type=>OBJECT,isa=>'NACL::C::Client'},
            device => { type => SCALAR },
            'size' => { type => SCALAR, default => 2048 },
            "quick-format" => { type => BOOLEAN, default => 1 },
            "file-system"  => { type => SCALAR, default => "FAT" },
            "method-timeout" => { type => SCALAR, default => 300 },
             _optional_scalars(
                qw( label current_label )),
        },
    );

    my $result = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   set    => 'Linux',
                method => '_linux_mkfs'
            },
            {   set    => 'Solaris',
                method => '_solaris_mkfs'
            },
            {   set    => 'Windows',
                method => '_windows_mkfs'
            },
        ]
    );
    $Log->exit() if $may_exit;
    return $result;
} ## end sub make_filesystem


sub _linux_mkfs {

    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset} || $command_interface->apiset(interface => 'CLI');

    # Stop multipathd if running
    $pkg->_flush_stop_disable_multipath(%opts);

    my @devices = ( ref \$opts{device} eq 'SCALAR' )
        ? ($opts{device}) :  @{$opts{device}};
    foreach my $device (@devices) {
        NACL::C::Client::Mkfs->mkfs(
            command_interface => $command_interface,
            verbose           => 1,
            'device'          => $device
        );
    }

    $Log->exit() if $may_exit;
}    ## _linux_mkfs ends here

sub _windows_mkfs {
    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;
    my $command_interface = $opts{command_interface};
    my $drive             = $opts{device};

    my $mkfs_obj = NACL::C::Client::Mkfs->mkfs(
        command_interface => $command_interface,
        device            => $drive,
        label             => $opts{label},
        "file-system"     => $opts{file_system},
        "quick-format"    => $opts{quick_format},
        current_label     => $opts{current_label},
    );

    $Log->exit() if $may_exit;
    return $mkfs_obj;
}

sub _linux_lunmount {

    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;

    my @mounts;

    my $deviceref = delete $opts{device};
    my @devices = ( ref \$deviceref eq 'SCALAR' )
        ? ($deviceref) :  @{$deviceref};

    foreach my $device (@devices) {
        push(@mounts, NACL::STask::Client::Mount->create(
            path              => $device,
            type              => 'device',
            fstype            => 'ext2',
            %opts,
        ));
    }

    $Log->exit() if $may_exit;
    return (scalar(@mounts) eq 1 ) ? $mounts[0] : \@mounts;
} ## end sub _linux_lunmount


sub _solaris_config() {

    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;
    my $do_lun_discovery = delete $opts{do_lun_discovery};
    my $make_filesystem  = delete $opts{make_filesystem};
    my ( $device, $mount );

    $device = $opts{device} if $opts{device};

    ## This means that if the user has passed the device name
    ## and set do_lun_discovery=1 then discovery would be done again.
    ## This device would only be used in making file system too.

    if ($do_lun_discovery) {
        $device = $pkg->_solaris_lun_discovery(%opts);
    } else {

        $Log->comment(
                  " do_lun_discovery is set to '0' in the input option. \n"
                . " Skipping lun discovery in the setup() method.\n"
                . " Since the option device is set, that value will be taken for mount "
        );

    }
    if ($make_filesystem) {
        my $apiset = $opts{command_interface}->apiset(interface => 'CLI');
        $pkg->_solaris_mkfs(
            apiset            => $apiset,
            command_interface => $opts{command_interface},
            device            => $device,
            size              => $opts{size}
        );
    } else {
        $Log->comment(
                  "make_filesystem is set to '0'.Skipping make filesystem "
                . " in the setup() method " );
    }
    ## LunMount would be done by default
    my %lun_mount_param = (
        command_interface => $opts{command_interface},
        device            => $device,
        server            => $opts{'ip-address'},
        %{$opts{lun_mount_params}},
    );
    if ( $opts{'mount-point'} ) {
        %lun_mount_param
            = ( %lun_mount_param, 'mount-point' => $opts{'mount-point'} );
    }
    $Log->comment(" Doing Lun Mount in the setup method ");
    $mount = $pkg->_solaris_lunmount(%lun_mount_param);

    $Log->exit() if $may_exit;
    return $mount;
} ## end sub _solaris_config()

sub _solaris_lun_discovery {
    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;
    my $command_interface  = $opts{command_interface};
    my $apiset             = $opts{apiset};
    my $timeout_multiplier = $opts{timeout_multiplier};

    $apiset->set_privilege( 'privilege-level' => "root" );
    $apiset->set_timeout( 'connectrec-timeout' => 600 * $timeout_multiplier );

    my $result_svcs = $apiset->svcs();
    my $service;
    my $result = $result_svcs->get_parsed_output();

    foreach my $output (@$result) {
        if ( $output->{fmri} =~ /.*iscsi.*/ ) {
            $service = $output->{'fmri'};
            last;
        }
    }
    if ( !$service ) {
       $Log->exit() if $may_exit;
       NATE::BaseException->throw("No Iscsi Service running in this client");
    }

    $apiset->svcadm(
        "cmd"          => "disable",
        "fmri-pattern" => $service,
    );
    $apiset->svcadm(
        "cmd"          => "enable",
        "fmri-pattern" => $service,
    );

    $apiset->iscsiadm_add(
        "direct-object"     => "discovery-address",
        "discovery-address" => "$opts{'ip-address'}",
    );
    $apiset->iscsiadm_modify(
        "direct-object" => "discovery",
        "static"        => "disable",
    );
    $apiset->iscsiadm_modify(
        "direct-object" => "discovery",
        "static"        => "enable",
    );
    $apiset->iscsiadm_modify(
        "direct-object"  => "initiator-node",
        "authentication" => "none",
    );
    $apiset->iscsiadm_list( "direct-object" => "initiator-node" );
    Tharn::snooze( 30 * $timeout_multiplier );
    my $response_output
        = $apiset->iscsiadm_list( "direct-object" => "target" );

    my $response_iscsiadm = $response_output->get_parsed_output();
    my $found;
    foreach my $output ( @{$response_iscsiadm} ) {
        if ( $output->{'iqn'} =~ /$opts{'filer-iqn'}/ ) {
            $found = 1;
            last;
        }
    }
    if ( !$found ) {
        NATE::BaseException->throw("unable to discover the address");
    }

    $apiset->iscsiadm_modify(
        "direct-object" => "discovery",
        "static"        => "disable"
    );
    $apiset->iscsiadm_modify(
        "direct-object" => "discovery",
        "static"        => "enable"
    );
    Tharn::snooze( 60 * $timeout_multiplier );

    my @devices;
    ## Now we accept multiple lun paths for linux, so normalize it to an arrayref
    $opts{'lun-path'} = [$opts{'lun-path'}] if(ref \$opts{'lun-path'} eq 'SCALAR');
    foreach my $lun_path (@{$opts{'lun-path'}}) {
        my $device_obj = NACL::CS::Client::Sanlun->fetch(
            command_interface => $command_interface,
            filter            => { 'lun_pathname' => $lun_path, }
        );
        push(@devices, $device_obj->device_filename);
    }

    $Log->exit() if $may_exit;
    return \@devices;
} ## end sub _solaris_lun_discovery

sub _solaris_mkfs {

    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset} || $command_interface->apiset(interface => 'CLI');
    my $size              = $opts{size};

    # Check if multipathd is running or not
    my $response = $apiset->execute_raw_command(
        command => "/etc/init.d/multipathd status" );
    if ( $response =~ /multipathd\s+.*\s+is running/ ) {

        # Turn off the multipathd
        $apiset->execute_raw_command(
            command => "/etc/init.d/multipathd stop" );
        $apiset->execute_raw_command(
            command => "/sbin/chkconfig multipathd off" );
    }

    my @devices = ( ref \$opts{device} eq 'SCALAR' )
        ? ($opts{device}) :  @{$opts{device}};
    foreach my $device (@devices) {
        NACL::C::Client::Mkfs->mkfs(
            command_interface => $command_interface,
            verbose           => 1,
            'device'          => $device,
            'size'            => $size,
        );
    }

    $Log->exit() if $may_exit;
} ## end sub _solaris_mkfs

sub _solaris_lunmount {

    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;

    my @mounts;

    my $deviceref = delete $opts{device};
    my @devices = ( ref \$deviceref eq 'SCALAR' )
        ? ($deviceref) :  @{$deviceref};

    foreach my $device (@devices) {
        push(@mounts, NACL::STask::Client::Mount->create(
            path              => $device,
            type              => 'device',
            fstype            => 'sharefs',
            %opts,
        ));
    }

    $Log->exit() if $may_exit;
    return (scalar(@mounts) eq 1 ) ? $mounts[0] : \@mounts;

} ## end sub _solaris_lunmount

sub _windows_config() {

    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;
    my $do_lun_discovery = delete $opts{do_lun_discovery};
    my $make_filesystem  = delete $opts{make_filesystem};

    my ($self,%hash_for_mkfs,%hash_for_lun_discovery,$mkfs_obj);

    $pkg->_hash_copy(
        source => \%opts,
        target => \%hash_for_lun_discovery,
        copy   => [ qw( command_interface ip-address current_label label
                       create_partition_script_file format_partition_script_file drive ) ],
    );

    $pkg->_hash_copy(
        source => \%opts,
        target => \%hash_for_mkfs,
        copy   => [ qw( command_interface quick_format label
                       file_system current_label ) ],
    );

    ## This means that if the user has passed the device name
    ## and set do_lun_discovery=1 then discovery would be done again .
    ## This device would only be used in making file system too.

    if ($do_lun_discovery) {
        $self = $pkg->_windows_lun_discovery(%hash_for_lun_discovery);
    } else {
        $Log->comment(
                  " do_lun_discovery is set to '0' in the input option. \n"
                . " Skipping lun discovery in the setup() method.\n"
                . " Since the option device is set, that value will be taken for mount "
        );
    }

    if ($make_filesystem) {
        $hash_for_mkfs{device} = $self->device().":";
        my $device_obj = $pkg->_windows_mkfs( %hash_for_mkfs );
        $self->device($device_obj->device());
    } else {
        $Log->comment(
                  "make_filesystem is set to '0'.Skipping make filesystem "
                . " in the setup() method " );
    }

    $Log->exit() if $may_exit;
    return $self;
} ## end sub _windows_config()

sub _create_diskpart {
    $Log->enter() if $may_enter;
    my ($pkg,@args) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            command_interface =>
              { type => OBJECT, isa => 'NACL::C::Client' },
            disk_number => { type => SCALAR,},
            new_label   => { type => SCALAR,},
            drive       => { type => SCALAR,},
            create_partition_script_file => { type => SCALAR },
            format_partition_script_file => { type => SCALAR },
        },
    );

    my $apiset                  = $opts{command_interface}->apiset();
    my $disk_number             = $opts{disk_number};
    my $drive                   = $opts{drive};
    my $create_disk_script_file = $opts{create_partition_script_file};
    my $format_file             = $opts{format_partition_script_file};
    my $new_label               = $opts{new_label};

    #creating a diskpart script batch-filesfor the same
    my @create_delete_batch_cmds = (
        "echo select disk=$disk_number > $create_disk_script_file",
        "echo ATTRIBUTES DISK CLEAR READONLY >> $create_disk_script_file",
        "echo create partition primary >> $create_disk_script_file",
        "echo online disk >> $create_disk_script_file",
        "echo exit >> $create_disk_script_file",
        "echo select volume=$disk_number > $format_file",
        "echo assign letter=$drive >> $format_file",
        "echo format fs=fat label=\"$new_label\" quick >> $format_file",
        "echo exit >> $format_file",
    );

    foreach my $cmd (@create_delete_batch_cmds) {
        $apiset->execute_command( command => $cmd );
    }

    #create the partition using diskpart script
    my $output = $apiset->execute_command(
        command => "diskpart \/s $create_disk_script_file"
    );

    #More error conditions can be handled here.
    #For now have come across the below error messages only.
    if( $output =~
        /(could not online the selected disk|There is insufficient free space to create a partition at the specified size|disk you specified is not valid)/ ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("Failure has occured while creating".
            " the disk partition");
    }
    Tharn::sleep(10); # waiting for the disk to become stable.
    $output = $apiset->execute_command( command => "diskpart \/s $format_file" );

    if($output =~ /(volume you selected is not valid or does not exist)/) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("Failure has occured while performing".
            "raw format on the partition");
    }

    #script to fetch the drive letter
    $apiset->execute_command(command => "echo list volume > fetch_volume.txt" );
    $apiset->execute_command(command => "echo exit >> fetch_volume.txt" );
    $output = $apiset->execute_command(
        command => "diskpart \/s fetch_volume.txt");

    my @lines = split(/\n/,$output);
    @lines = grep{$_ =~ /Volume $disk_number\s+\w\s+.*/} @lines;
    $drive = $1 if( $lines[0] =~ /Volume $disk_number\s+(\w)\s+.*/);

    if(!defined($drive) || $drive =~ /\d+/) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("Not able to fetch the drive letter from the".
            " diskpart list volume command");
    }

    $Log->exit() if $may_exit;
    return $drive;
}

=head2 purge

    # Example usage for purge method
    my $mount_obj = NACL::MTask::Client::LunMount->setup(
        command_interface => $command_interface,
	"ip-address"      => $filer_ip
    );

    $mount_obj->purge(nacltask_iscsi_clean_level => 2);
(Instance method)
This method will perform the purge operations required for the lunmount
setup done on windows/linux client.

The steps which are performed in this method for windows client are :

1) Offlining the disk which were mounted.

2) Logging out from the iscsi session which was created for mounting the lun.

3) Removing the targetportal which was configured during setup.

4) Deleting the batch script files created for the diskpart operations.

The steps which are performed in this method for linux client are :

1) Unmount the mounted devices from the linux machine.

2) Delete the mount-points.

=over

=item Options

=over

=item C<< delete_partition_script_file => $file_name >>

(Optional) Name of the file to be given to script having the deletion of partition
steps. Default : del_partition.txt .

=item C<< nacltask_iscsi_clean_level => $iscsi_clean_level >>

defaults to 2.
clean_level defines the degree of cleanup that need to be performed.
0 == dont cleanup anything
1 == Logout from target sessions listed in @targets. Deconfigure iscsi for target sessions listed in @targets
2 (default) == Logout from all the targets. Deconfigure iscsi for all the targets.

=back

=back

=over

=item Exceptions

=over

=item C<NATE::BaseException>

This exception is thrown when a unsupported OS is used for purge() call.

=back

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my ( $obj, @args ) = @_;

    my %opts = $obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            delete_partition_script_file => { type => SCALAR,
                default => "del_partition.txt"},
            nacltask_iscsi_clean_level  => { type => SCALAR, default => 2 },
        },
        allow_extra => 1,
    );

    my %win_opts;
    my %lnx_opts;
    $win_opts{"delete_partition_script_file"} = delete $opts{delete_partition_script_file};
    $lnx_opts{nacltask_iscsi_clean_level} = delete $opts{nacltask_iscsi_clean_level};
    if( $obj->command_interface()->type() =~ /windows/i ) {
        $obj->_purge_windows(%opts,%win_opts);
    } elsif( $obj->command_interface()->type() =~ /linux/i ) {
        $obj->_purge_linux(%opts,%lnx_opts);
    }

    $Log->exit() if $may_exit;
}

sub _purge_windows {
    $Log->enter() if $may_enter;
    my ( $obj, %opts ) = @_;

    my $apiset      = $obj->command_interface()->apiset(interface => 'CLI');
    my $file_name   = $opts{delete_partition_script_file};
    my $disk_number = $obj->disk_number();
    my $filer_ip    = $obj->filer_ip();
    my $session_id  = $obj->session_id();
    my $create_file = $obj->create_partition_script_file();
    my $format_file = $obj->format_partition_script_file();
    my @batch_cmds  = (
        "echo select disk=$disk_number > $file_name",
        "echo offline disk >> $file_name",
        "echo exit >> $file_name",
        "diskpart \/s $file_name ",
        "del $file_name /F",
        "del $create_file /F",
        "del $format_file /F",
    );

    try {
        foreach my $cmd (@batch_cmds) {
                $apiset->execute_command( command => $cmd );
        }
        $apiset->mpc_iscsi_logout( session_id => $session_id );
        $apiset->mpc_iscsi_remove_targetportal( filername_or_ip =>
        $filer_ip );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = $_[0];
        $Log->exit() if $may_exit;
        $exception_object->throw("Failure occured while purging the window setup");
    };

    $Log->exit() if $may_exit;
}

sub _purge_linux {
    $Log->enter() if $may_enter;
    my ($obj,%opts) = @_;
    my @targets;
    my $iscsi_clean_level = delete $opts{nacltask_iscsi_clean_level};
    my $iscsi_target = NACL::C::Client::IscsiTarget->new(
        address     => $obj->filer_ip(),
        target_name => $obj->filer_iqn(),
    );
    push(@targets, $iscsi_target);

    if( ref $obj->_lun_mount_obj eq 'ARRAY' ) {
        $_->purge(%opts) foreach(@{$obj->_lun_mount_obj});
    } else {
        $obj->_lun_mount_obj()->purge(%opts);
        try {
            my $Iscsi_Obj = NACL::C::Client::Iscsi->deconfigure(
                command_interface           => $obj->command_interface(),
                targets                     => \@targets,
                config_iscsi_discovery_type => "sendtargets",
                config_iscsi_version        => "open-iscsi",
                mp_type                     => "dmmp",
                clean_level                 => $iscsi_clean_level,
            );
         }
         catch NACL::APISet::Exceptions::CommandFailedException with {
            my $ex = shift;
            if ($ex->text()
                !~ /No portal found|No session found|No matching sessions found/i
                ) {
                $ex->set_text(
                    "Failed to list iscsi sessions. " . $ex->text());
                $ex->throw();
             }
         };
    }
    $Log->exit() if $may_exit;
}

=head2 mount_point

    # Example usage for mount_point method
    my $mount_obj = NACL::MTask::Client::LunMount->setup(
        command_interface => $command_interface,
        "ip-address"      => $filer_ip
    .. );

    my $mount_point = $mount_obj->mount_point();
(Instance method)
This method will fetch the mount_point from the NACL::MTask::Client::LunMount type
of object return after the call to setup.
returns arrayref if multiple lun paths were passed for linux client.

=over

=item Exceptions

=over

=item C<NATE::BaseException>

This exception is thrown when a unsupported OS is used for mount_point() call.

=back

=back

=cut

sub mount_point {
    $Log->enter() if $may_enter;
    my $obj = shift;

    if( $obj->command_interface()->type() =~ /solaris/i ) {
        $Log->exit() if $may_exit;
        return $obj->mount_point();
     }elsif(!$obj->command_interface->hostrec->is_unix()){
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("This method is not supported for clients other".
            " than linux-unix, solaris type");
    }
    $Log->exit() if $may_exit;
    if( ref $obj->_lun_mount_obj eq 'ARRAY' ) {
        my @mount_points = map {$_->mount_point()} @{$obj->_lun_mount_obj()};
        return \@mount_points;
    } else {
        return $obj->_lun_mount_obj()->mount_point();
    }
}

sub _flush_stop_disable_multipath {
    $Log->enter() if $may_enter;
    my ($obj, %opts) = @_;
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    $apiset->set_privilege('privilege-level' => "root");

    my $stop_multipath = 1;
    try {
        $apiset->service(
            action => "status",
            script => "multipathd",
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $ex = shift;

        # rethrow the exception if failed for some other reason
        if ($ex->text() =~ /(multipathd\s+is\s+stopped|Active:\s+inactive)/) {
            $stop_multipath = 0;
        } else {
            $ex->throw;
        }
    };
    if ($stop_multipath) {
        my $multipath_obj = NACL::C::Client::Multipath->new(
            command_interface => $command_interface,
        );
        $multipath_obj->flush();
        $multipath_obj->stop();
        $multipath_obj->disable();
    }

    $Log->exit() if $may_exit;
}

1;
