# $Id: //depot/prod/test/nacldev/lib/NACL/MTask/ClientUtils.pm
#
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary ClientUtils MTask Module
## @author  rahula@netapp.com , dl-nacl-dev@netapp.com
## @status public
## @pod here

=head1 NAME

NACL::MTask::ClientUtils

=head1 DESCRIPTION

C<NACL::MTask::ClientUtils> provides a series of well-defined methods related
to utilities(network_outage,reboot) on the specified client.

=head1 METHODS

=head2 network_outage

     NACL::MTask::ClientUtils->network_outage(
            command_interface => $client,
            interface_name    => $connection,
            batch_file        => $file,
     );

(Class method) This method is used to create the Network outage on the client specified.
 This is done by disabling the active network interface , delaying it and then again
 enabling the interface .

=over

=item  Options

=over

=item C<< "command_interface=>$client" >>

(Required) The command interface (NACL::C::Client object) of the client on which this task is performed .

=item C<< "interface_name=>$connection" >>

(Optional) The connection name on which outage needs to be performed. Default : Local Area Connection

=item C<< "outage_time=>$outage_time" >>

(Optional) Contains the value (in seconds) of the outage timeout .(Default : 10)

=item C<< batch_file=>$file >>

(Optional) The batch file name with .bat extension . The file name can be a fully qualified path Default : outage.bat

=item C<< wait_for_client_stable=>$wait >>

(Optional) Time(in seconds) to wait for the client to be stable .(Default : 900)
'wait_for_client_stable' value should be greater than the 'outage_time' value.

=item C<< no_of_retries=>$retries >>

(Optional) No.of retries needed to connect after network outage . (Default : 10)

=item C<< use_ipv6_address => 0 | 1 >>

(Optional, default 0) If enabled, IPv6 loopback address will be used in batch command.

=back

=back

=cut

package NACL::MTask::ClientUtils;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::APISet::Exceptions::ConnectionFailedException qw(:try);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ResponseException qw(:try);
use Params::Validate
    qw/validate_with SCALAR SCALARREF BOOLEAN OBJECT ARRAYREF HASHREF/;
    
use NACL::GeneralUtils qw(_poll _powercycle);

# constants
use constant IPV4_LOOPBACK_ADDR => '127.0.0.1';
use constant IPV6_LOOPBACK_ADDR => '::1';
use constant PING_CMD_IPV4      => 'ping';
use constant PING_CMD_IPV6      => 'ping6';

use Class::MethodMaker [
    new    => [ '-hash', '-init', 'new' ],
    scalar => [ { type => 'NACL::C::Client' }, 'command_interface' ],
];

=head2 new

Get a new instance of NACL::MTask::ClientUtils.

=over

=item Usage

$clientutils = NACL::MTask::ClientUtils->new(
    command_interface => $ci,
);

=item Parameters

=item C<<command_interface>>

(Instance method)
Required. Client command interface.

=item Return

<OBJECT> : A NACL::MTask::ClientUtils object.

=back

=cut

sub init {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;

    # Construct class members
    my %opts = validate_with(
        params  => \@_,
        spec    => {
            command_interface => { type => OBJECT, isa => 'NACL::C::Client' },
        },
        allow_extra => 0,
    );

    $pkg_or_obj->command_interface($opts{command_interface});

    $Log->exit() if $may_exit;

    return $pkg_or_obj;

} ## end sub init

#===============================================================================
# Method-Name : network_outage
# Description : This method performs network outage on the client mentioned in
#               the command interface
#===============================================================================

sub network_outage {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            client_command_interface =>
                { type => OBJECT, isa => "NACL::C::Client" },
            outage_time            => { type => SCALAR, default => 10 },
            wait_for_client_stable => { type => SCALAR, default => 900 },
            no_of_retries          => { type => SCALAR, default => 10 },
            batch_file => { type => SCALAR, default => "outage.bat" },
            interface_name =>
                { type => SCALAR, default => "\"Local Area Connection\"" },
            use_ipv6_address => { type => SCALAR, default => 0 },
        },
        allow_extra => 1,
    );

    my $ci                     = $opts{client_command_interface};
    my $outage_time            = $opts{outage_time};
    my $batch_file             = $opts{batch_file};
    my $interface_name         = $opts{interface_name};
    my $wait_for_client_stable = $opts{wait_for_client_stable};
    my $retries                = $opts{no_of_retries};
    my $use_ipv6_address       = $opts{use_ipv6_address};
    my $retry                  = 0;
    my $not_created            = 0;

    #Create connection object
    my $api_set_obj = $ci->apiset(
        category  => "Host",
        interface => "CLI",
        set       => "Windows",
    );

    #checking if the file is present or not
    my $diroutput
        = $api_set_obj->execute_raw_command( command => "dir $batch_file" );

    $not_created = 1 if ( $diroutput =~ /File Not Found.*/ );

    my $loop_back_ip;
    my $ping_cmd;
    if ($use_ipv6_address) {
        $loop_back_ip = IPV6_LOOPBACK_ADDR;
        $ping_cmd     = PING_CMD_IPV6;
    } else {
        $loop_back_ip = IPV4_LOOPBACK_ADDR;
        $ping_cmd     = PING_CMD_IPV4;
    }

    #create a batch file & print the commands on the file.
    if ($not_created) {
        my @batch_cmds = (
            "echo netsh interface set interface $interface_name DISABLED > $batch_file",
            "echo ping $ping_cmd $loop_back_ip -n $outage_time >> $batch_file",
            "echo netsh interface set interface $interface_name ENABLED >> $batch_file",
        );

        foreach my $cmd (@batch_cmds) {
            $api_set_obj->execute_raw_command( command => $cmd );
        }
    }

    try {
        $api_set_obj->execute_raw_command( command => "start $batch_file" );
    }
    catch NATE::BaseException with {
        my $err_obj = shift;
        $Log->exit() if $may_exit;
        $err_obj->throw("Network outage batch run failed!");
    };

    #waiting for the client to become stable
    sleep($wait_for_client_stable);

    #reconnecting to the client
WAIT: {
        use warnings qw(exiting);
        try {
            $ci->refresh_command_interface( no_version_change => 1 );
            if ($not_created) {
                $api_set_obj = $ci->apiset(
                    category  => "Host",
                    interface => "CLI",
                    set       => "Windows",
                );
                $api_set_obj->del(
                    'files'        => $batch_file,
                    'force-delete' => 1
                );
            }
        }
        catch NATE::BaseException with {
            my $err_obj = shift;
            if ((   $err_obj->text()
                    =~ /(Could not create connection.*)|(Unexpected EOF.*)|(Timed out.*)|(Connection timed out.*)/i
                )
                && ( $retry < $retries )
                ) {
                sleep($wait_for_client_stable);
                $retry++;
                no warnings qw(exiting);
                redo WAIT;

            } else {
                $Log->exit() if $may_exit;
                $err_obj->throw();
            }
        };
    }

    $Log->exit() if $may_exit;
} ## end sub network_outage

=head2 reboot

    NACL::MTask::ClientUtils->reboot(
         command_interface               => $client,
    );

(Class method) This method is used to perform reboot on the client specified by
the command interface . Throws BaseException error if the reboot is not
successful , else no exception is thrown. This method supports linux ,
solaris & windows types of clients .
The reboot command works in 'root' user (Linux & Solaris clients) , so in order
to use this method the client needs to be logged in as "root" .

=over

=item  Options

=over

=item C<< command_interface => $client >>

(Required) The ( NACL::C::Client object) client on which reboot is performed .

=item C<< no_of_retries=>$retries >>

(Optional) No.of retries needed to connect after reboot. (Default : 15)

=back

=back

=cut

sub reboot {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            command_interface => { type => OBJECT, isa => "NACL::C::Client" },
            no_of_retries => { type => SCALAR, default => 10 },
        },
    );

    my ( $apiset, $retry, $reboot_cmd, $result, $type );
    my $command_interface = $opts{command_interface};
    my $retries           = $opts{no_of_retries};

    my $hosttype = $command_interface->hostrec->hosttype();
    my $client   = $command_interface->hostrec->hostname();
    $retry = 0;


#===============================================================================
#Performing reboot execution on the client mentioned in the command interface
#===============================================================================

    $apiset = $command_interface->apiset();

    my $startup_time_before_reboot = $command_interface->startup_time();

    $command_interface->reboot(
        force => 1,
        "timeout-shutdown" => 0,
    );

    # Some of the VM machines takes less than 60 sec to reboot.
    # Directly check using machines uptime if it has already rebooted
    $result = $pkg->_check_if_rebooted(
        startup_time  => $startup_time_before_reboot,
        command_interface => $command_interface,
    );

    if($result == 0) {
        $result = _poll(
            command_interface => $command_interface,
            retries           => $retries
        );
    }

    # This will be rare case but it might happen
    if($result == 0) {
        $result = $pkg->_check_if_rebooted(
            command_interface => $command_interface,
            startup_time  => $startup_time_before_reboot,
        );
    }

    if ( $result == 1 ) {
        $Log->debug("Client \"$client\" rebooted successfully.");
    } else {
        _powercycle(
            client_to_powercycle => $command_interface,
            retries              => $retries
        );
    }

    $Log->exit() if $may_exit;
} ## end sub reboot

=head2 add_user_localhost

    NACL::MTask::ClientUtils->add_user_localhost(
         command_interface => $client,
         name              => $name,
         groups            => [$group1, ...],
    );

    $cu = NACL::MTask::ClientUTils->new(command_interface => $ci);
    $cu->add_user_localhost(name => $name);

(Class or Instance method)
This method is used to add a local user on the client by giving user name. Give
warning message if the user exists already.

Currently it is implemented to support linux client only. It will throw
exception for non-linux client.

<TODO>: Add non-linux client support and handle exception respectively.

=over

=item  Options

=over

=item C<< command_interface => $ci >>

(Required for class method) The ( NACL::C::Client object) client.

=item C<< name => $name >>

(Required) Local user name on the client.

=item C<< uid => $uid >>

(Optional) Local user id on the client.

=item C<<group>>

(Optional). Group name or id.

=item C<<home>>

Optional. Home path. Default: /var/tmp/<user>

=item C<<name>>

Optional. Shell. Default: /bin/bash

=item Return

=back

=cut

sub add_user_localhost {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;

    my %opts = validate_with(
        params => \@_,
        spec => {
            command_interface => {
                type => OBJECT, isa => 'NACL::C::Client', optional => 1 },
            name    => { type => SCALAR },
            uid     => { type => SCALAR, optional => 1 },
            group   => { type => SCALAR, optional => 1 },
            home    => { type => SCALAR,  default => "/home" },
            shell   => { type => SCALAR,  default => "/bin/bash" },
        },
        allow_extra => 1,
    );

    my $command_interface = ref ($pkg_or_obj) ? $pkg_or_obj->command_interface
        : $opts{command_interface};

    my $name   = $opts{name};
    my $home   = $opts{home} . "/$name";
    my $shell  = $opts{shell};

    my $command = "$ENV{SUDO_PATH} useradd";
    $command .= " -u " . $opts{uid}   if ($opts{uid});
    $command .= " -g " . $opts{group} if ($opts{group});
    $command .= " -d $home -s $shell -p'*' $name";

    my $hosttype = $command_interface->hostrec->hosttype();
    my $response;
    if ( $command_interface->hostrec->is_linux() ) {
        $response = $command_interface->apiset->execute_command(
            command => $command
        );
        $Log->warn("User($name) already exists.")
            if ($response =~ /already exists/);
        NATE::BaseException->throw("Group does not exist: " . $opts{groups})
            if ($response =~ /does not exist/);
    }
    else {
        NATE::BaseException->throw("Error. The current implementation supports"
            . " linux client only." );
    }

    $Log->exit() if $may_exit;

} ## end sub add_user_localhost

=head2 modify_user_group_localhost

    NACL::MTask::ClientUtils->modify_user_group_localhost(
         command_interface => $client,
         name              => $name,
         groups            => [$group1, ...],
    );

    $cu = NACL::MTask::ClientUTils->new(command_interface => $ci);
    $cu->modify_user_group_localhost(
         name              => $name,
         groups            => [$group1, ...],
    );

(Class or Instance method)
This method is used to modify user groups for a local user on localhost.
An exception is thrown if no groups given or any given group does not exist.

Currently it is implemented to support linux client only. It will throw
exception for non-linux client.

<TODO>: Add non-linux client support and handle exception respectively.

=over

=item  Options

=over

=item C<< command_interface => $ci >>

(Required for class method) The ( NACL::C::Client object) client.

=item C<< name => $name >>

(Required) Local user name on the client.

=item C<<groups>>

(Required). A list of group name. Minimum one group specified in the list.
Throw exception if the list is empty.

=item Return

=back

=cut

sub modify_user_group_localhost {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;

    my %opts = validate_with(
        params => \@_,
        spec => {
            command_interface => {
                type => OBJECT, isa => 'NACL::C::Client', optional => 1 },
            name    => { type => SCALAR },
            groups  => { type => ARRAYREF },
        },
        allow_extra => 1,
    );

    NATE::BaseException->throw("Error. At least one group must be specified"
        . " in the groups") if (scalar @{$opts{groups}} == 0);

    my $command_interface = ref ($pkg_or_obj) ? $pkg_or_obj->command_interface
        : $opts{command_interface};

    my $name   = $opts{name};
    my $groups = join(",", @{$opts{groups}});

    my $command = "$ENV{SUDO_PATH} usermod";
    $command .= " -aG $groups $name";

    my $hosttype = $command_interface->hostrec->hosttype();
    my $response;
    if ( $command_interface->hostrec->is_linux()) {
        $response = $command_interface->apiset->execute_command(
            command => $command
        );
        NATE::BaseException->throw("Group does not exist: " . $opts{groups})
            if ($response =~ /does not exist/);
    }
    else {
        NATE::BaseException->throw("Error. The current implementation supports"
            . " linux client only." );
    }

    $Log->exit() if $may_exit;

} ## end sub modify_user_group_localhost

=head2 add_group_localhost

    NACL::MTask::ClientUtils->add_group_localhost(
         command_interface => $client,
         name              => $name,
    );

    $cu = NACL::MTask::ClientUtils->new(command_interface => $ci);
    $cu->add_group_localhost(name => $name);

(Class or Instance method)
This method is used to add a local group on the client by  giving group name.
Give warning message if the group exists already.

Currently it is implemented to support linux client only. It will throw
exception for non-linux client.

<TODO>: Add non-linux client support and handle exception respectively.

=over

=item  Options

=over

=item C<< command_interface => $ci >>

(Required for class method) The ( NACL::C::Client object) client.

=item C<< name => $name >>

(Required) Local group name on the client.

=item C<< gid => $gid >>

(Optional) Local group id on the client.

=item Return

=back

=cut

sub add_group_localhost {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;

    my %opts = validate_with(
        params => \@_,
        spec => {
            command_interface => {
                type => OBJECT, isa => 'NACL::C::Client', optional => 1 },
            name => { type => SCALAR },
            gid  => { type => SCALAR, optional => 1 },
        },
        allow_extra => 0,
    );

    my $command_interface = ref ($pkg_or_obj) ? $pkg_or_obj->command_interface
        : $opts{command_interface};
    my $name = $opts{name};

    my $hosttype = $command_interface->hostrec->hosttype();
    my $response;
    my $command = "$ENV{SUDO_PATH} groupadd ";
    $command .= " -g " . $opts{gid} if ($opts{gid});
    if ( $command_interface->hostrec->is_linux() ) {
        $response = $command_interface->apiset->execute_command(
            command => $command . " $name"
        );
        $Log->warn("Group($name) already exists.")
            if ($response =~ /already exists/);
    }
    else {
        NATE::BaseException->throw("Error. The current implementation supports"
            . " linux client only." );
    }

    $Log->exit() if $may_exit;

} ## end sub add_group_localhost

=head2 get_gid_localhost

    $gid = NACL::MTask::ClientUtils->get_gid_localhost(
         command_interface => $client,
         name              => $name,
    );

    $cu = NACL::MTask::ClientUtils->new(command_interface => $ci);
    $gid = $cu->get_gid_localhost(name => $name);

(Class or Instance method)
This method is used to get a local group id on the client by giving group name.
Throws BaseException error if the local group does not exist on the client.

Currently it is implemented to support linux client only. It will throw
exception for non-linux client.

<TODO>: Add non-linux client support and handle exception respectively.

=over

=item  Options

=over

=item C<< command_interface => $ci >>

(Required for class method) The ( NACL::C::Client object) client.

=item C<< name => $name >>

(Required) Local group name on the client.

=item Return

<SCALAR>: Group ID.

=back

=cut

sub get_gid_localhost {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;

    my %opts = validate_with(
        params => \@_,
        spec => {
            command_interface => {
                type => OBJECT, isa => 'NACL::C::Client', optional => 1 },
            name => { type => SCALAR, optional => 0 },
        },
        allow_extra => 1,
    );

    my $command_interface = ref ($pkg_or_obj) ? $pkg_or_obj->command_interface
        : $opts{command_interface};
    my $name = $opts{name};

    my $hosttype = $command_interface->hostrec->hosttype();
    my $gid;
    if ( $command_interface->hostrec->is_linux()) {
        $gid = $command_interface->apiset->execute_command(
            command => "$ENV{SUDO_PATH} getent group $name | cut -d: -f3"
        );
        NATE::BaseException->throw("Group($name) does not exist.")
            if (!defined $gid);
    }
    else {
        NATE::BaseException->throw("Error. The current implementation supports"
            . " linux client only." );
    }

    $Log->exit() if $may_exit;

    return int($gid);

} ## end sub get_gid_localhost

=head2 get_id_localhost

    $id = NACL::MTask::ClientUtils->get_id_localhost(
         command_interface => $client,
         name              => $name
    );

    $cu = NACL::MTask::ClientUtils->new(command_interface => $ci);
    $id = $cu->get_id_localhost(name => $name);

(Class or Instance method) This method is used to get a local user id or user
primary group id on the client by giving user name and requested id type. Throws
BaseException error if the local user does not exist on the client.

Currently it is implemented to support linux client only. It will throw
exception for non-linux client. Valid id_type is "user" (default) or
"user_primary_group". Throws exception for invalid id_type.

<TODO>: Add non-linux client support and handle exception respectively.

=over

=item  Options

=over

=item C<< command_interface => $ci >>

(Required for class method) The ( NACL::C::Client object) client.

=item C<< name => $name >>

(Required) Local user name on the client.

=item C<< id_type => $id_type >>

(Optional) Request "user" id (Default) or "user_primary_group" id.

=item Return

<SCALAR>: ID for user or user primary group on the local host.

=back

=cut

sub get_id_localhost {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;

    my %opts = validate_with(
        params => \@_,
        spec => {
            command_interface => {
                type => OBJECT, isa => 'NACL::C::Client', optional => 1 },
            name    => { type => SCALAR, optional => 0 },
            id_type => { type => SCALAR, default  => "user"},
        },
        allow_extra => 1,
    );

    my $id_type = $opts{id_type};
    NATE::BaseException->throw("Error. ID type ($id_type) is not supported")
        if ( lc($id_type) ne "user" && lc($id_type) ne "user_primary_group" );

    my $command_interface = ref ($pkg_or_obj) ? $pkg_or_obj->command_interface
        : $opts{command_interface};
    my $name = $opts{name};
    my $command = "$ENV{SUDO_PATH} id -u $name";
    $command =~ s/-u/-g/ if ($opts{id_type} eq "user_primary_group");

    my $hosttype = $command_interface->hostrec->hosttype();
    my $response;
    if ( $command_interface->hostrec->is_linux() ) {
        $response = $command_interface->apiset->execute_command(
            command => $command
        );
        NATE::BaseException->throw("User($name) does not exist.")
            if ($response =~ /No such user/);
    }
    else {
        NATE::BaseException->throw("Error. The current implementation supports"
            . " linux client only." );
    }

    $Log->exit() if $may_exit;

    return int($response);

} ## end sub get_id_localhost

=head2 delete_group_localhost

    NACL::MTask::ClientUtils->delete_group_localhost(
         command_interface => $client,
         name              => $name,
    );

    $cu = NACL::MTask::ClientUtils->new(command_interface => $ci);
    $cu->delete_group_localhost(name => $name);

(Class or Instance method) This method is used to delete a local group on the
client by giving group name. Give warning message if the group does not exist.

Currently it is implemented to support linux client only. It will throw
exception for non-linux client.

<TODO>: Add non-linux client support and handle exception respectively.

=over

=item  Options

=over

=item C<< command_interface => $ci >>

(Required) The ( NACL::C::Client object) client.

=item C<< name => $name >>

(Required) Local group name on the client.

=item Return

=back

=cut

sub delete_group_localhost {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;

    my %opts = validate_with(
        params => \@_,
        spec => {
            command_interface => {
                type => OBJECT, isa => 'NACL::C::Client', optional => 1 },
            name    => { type => SCALAR,   optional => 0 },
        },
        allow_extra => 1,
    );

    my $command_interface = ref ($pkg_or_obj) ? $pkg_or_obj->command_interface
        : $opts{command_interface};
    my $name              = $opts{name};

    my $hosttype = $command_interface->hostrec->hosttype();
    my $response;
    if ( $command_interface->hostrec->is_linux()) {
        $response = $command_interface->apiset->execute_command(
            command => "$ENV{SUDO_PATH} groupdel $name"
        );
        $Log->warn("Group($name) does not exist.")
            if ($response =~ /does not exist/);
    }
    else {
        NATE::BaseException->throw("Error. The current implementation supports"
            . " linux client only." );
    }

    $Log->exit() if $may_exit;

} ## end sub delete_group_localhost

=head2 delete_user_localhost

    NACL::MTask::ClientUtils->delete_user_localhost(
         command_interface => $client,
         name              => $name,
    );

    $cu = NACL::MTask::ClientUtils->new(command_interface => $ci);
    $cu->delete_user_localhost(name => $name);

(Class or Instance method) This method is used to delete a local user on the
client by giving user name. Give warning message if the user does not exist.

Currently it is implemented to support linux client only. It will throw
exception for non-linux client.

<TODO>: Add non-linux client support and handle exception respectively.

=over

=item  Options

=over

=item C<< command_interface => $client >>

(Required) The ( NACL::C::Client object) client.

=item C<< name => $name >>

(Required) Local user name on the client.

=item Return

=back

=cut

sub delete_user_localhost {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;

    my %opts = validate_with(
        params => \@_,
        spec => {
            command_interface => {
                type => OBJECT, isa => 'NACL::C::Client', optional => 1 },
            name    => { type => SCALAR,   optional => 0 },
        },
        allow_extra => 1,
    );

    my $command_interface = ref ($pkg_or_obj) ? $pkg_or_obj->command_interface
        : $opts{command_interface};
    my $name = $opts{name};

    my $hosttype = $command_interface->hostrec->hosttype();
    my $response;
    if ( $command_interface->hostrec->is_linux() ) {
        $response = $command_interface->apiset->execute_command(
            command => "$ENV{SUDO_PATH} userdel -r $name"
        );
        $Log->warn("User($name) does not exist.")
            if ($response =~ /does not exist/);
    }
    else {
        NATE::BaseException->throw("Error. The current implementation supports"
            . " linux client only." );
    }

    $Log->exit() if $may_exit;

} ## end sub delete_user_localhost

sub _check_if_rebooted {
    my ($self, @args) = @_;
    
    my %opts = validate_with(
        params => \@args,
        spec   => {
            command_interface => { type => OBJECT, isa => "NACL::C::Client" },
            startup_time => { type => SCALAR },
        },
    );
    my $ci = $opts{command_interface};
    my $startup_time_before_reboot = $opts{startup_time};
    my $result = 0;

    try {
        $ci->refresh_command_interface( no_version_change => 1 );
        my $startup_time_after_reboot = $ci->startup_time();
        if(abs($startup_time_after_reboot - $startup_time_before_reboot) > 2) {
            $result = 1;
        }
    } catch NATE::BaseException with {
        # Otherwise host has still not booted up
    };

    return $result;
}

1;
