# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ClusterPeer Task Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::MTask::ClusterPeer;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate validate_with BOOLEAN SCALAR HASHREF OBJECT SCALARREF);
use NATE::Exceptions::Argument qw(:try);
use NATE::BaseException;
use NACL::C::Network;
use NACL::C::Cluster;
use NACL::C::ClusterPeer;
use NACL::C::ClusterIdentity;
use NACL::CS::ClusterDate;
use NACL::STask::NetworkInterface;
use NACL::STask::Node;
use NACL::ComponentUtils qw(Dumper);
use NACL::STask::NetworkRoutingGroupsRoute;
use NACL::STask::NetworkRoute;
use Time::ParseDate qw(parsedate);
use NACL::GeneralUtils qw(nacl_method_retry);
use NACL::STask::ClusterPeer;

use base qw (NACL::MTask::MTask);

=head1 NAME

NACL::MTask::ClusterPeer

=head1 DESCRIPTION

C<NACL::MTask::ClusterPeer> provides a number of well-defined but
potentially complex or multi-step methods related to Cluster Peer in
ONTAP.

=head1 ATTRIBUTES

=head2 cluster

The cluster peer name on home_cluster_interface machine.

=head2 rcluster

The cluster peer name on remote_cluster_interface machine.

=head2 home_cluster_interface

A component object of C<NACL::C::CommandInterface::ONTAP|lib-NACL-C-CommandInterface-ONTAP-pm>
that represents a node in home cluster.

=head2 remote_cluster_interface

A component object of C<NACL::C::CommandInterface::ONTAP|lib-NACL-C-CommandInterface-ONTAP-pm>
that represents a node in remote cluster.

=head2 intercluster_lifs

A array containing only the NACL::STask::NetworkInterface objects created in the
create method.

=head2 intercluster_routes

A array containing only the NACL::STask::NetworkRoutingGroupsRoute objects created in the
create method.

=head2 peer_stask

A <NACL::STask::ClusterPeer> object created after the cluster peer relationship is created.

=cut

=head1 METHODS

=cut

use Class::MethodMaker [
    new    => [ '-hash', 'new' ],
    array => 'intercluster_lifs',
    array => 'intercluster_routes',
    scalar => 'home_cluster_interface',
    scalar => 'remote_cluster_interface',
    scalar => 'cluster',
    scalar => 'rcluster',
    scalar => 'peer_stask',
];


=head2 create

    (Class method)
      $clusterpeer = NACL::MTask::ClusterPeer->create(
          home_cluster_interface    =>  $home_cluster_interface,
          remote_cluster_interface  =>  $remote_cluster_interface,
          nacltask_lif_details      =>  \%nacltask_lif_details,    # optional
          nacltask_if_exists_peer   =>  $action,        # default 'die'
          nacltask_if_exists_lifs   =>  $action_lif,    # default 'reuse'
          task_verify               =>  1,              # default 0
          _was_created              =>  \$was_created   # optional
          );


    Creates cluster-peer on nodes specified by home_cluster_interface and
    remote_cluster_interface attribute.This method provides additional
    services beyond what's inherent to the product's cluster peer create
    command, as controlled and described by the "nacltask_if_exists_peer"
    option. It also takes care of cluster-peer-create related dependencies
    such as intercluster lif creation.

    Perform the following step to Create cluster-peer:
    1) Both clusters should be healthy individually
    2) Both 'clusters' clocks are reasonably synchronized.
       The maximum time delta between the two clusters is 300 seconds
    3) Check that intercluster lif exists.
    4) Check the given cluster address is pingable using network ping.
    5) Create cluster peer.
    6) verify cluster peer.

=over

=item Options

=over

=item C<< home_cluster_interface => $home_cluster_interface >>

    (Required) As C<NACL::C::Node>. A component object that represents the
    node to which to send commands on home cluster.

=item C<< remote_cluster_interface => $remote_cluster_interface >>

    (Required) As C<NACL::C::Node>. A component object that represents the
    node to which to send commands on remote cluster.

=item C<< nacltask_lif_details => \%nacltask_lif_details >>

    (Optional) This contains all the lifs details for creation.

     my %nacltask_lif_details= (
         "$home_node_1" =>           # Name of the filer
            [
                {
                    vserver           => $vserver_name,
                    address           => $address_1,
                    lif               => $lif_name_1,
                    'home-node'       => $home_node_1,
                    'home-port'       => $home_port_1,
                    netmask           => $netmask,
                    gateway           => $gateway,
                },
                {
                    .....
                }
            ],
         "$remote_node_1" => [{...},{...}]
     );


    These options are mandatory if user is passing %nacltask_lif_details.


    lif

    (Optional) The name of the logical interface ,used to identify it uniquely.
    The name of the logical interface is unique within a particular vserver.

    vserver

    (Optional) The name of the vserver that contains the lif.

    home-node

    (Optional) Name of the node in which the lif needs to be created.

    home-port

    (Optional) Name of the port in which the lif needs to be created.

    address

    (Optional) Ip-address that will be assigned to this lif.

    netmask

    (Optional) Netmask for the ip-address.

    gateway

    (Optional) Gateway for the ip-address to create routes.


    If user didn't pass this detail, we can read these values from
    tharnhost files (i.e. INTERCLUSTER_XX params).

=item C<< timeout => $timeout >>

    (Optional) Timeout value for cluster peer create to complete, defaults
     to 60.

=item C<< nacltask_if_exists_peer => $action >>

    (Optional) What to do if the cluster-peer is setup already. If $action is "die"
    the default), then fail with an exception (in the same way that the cluster-peer
    component would have: by trying the cluster-peer create and letting the product
    complain about the existence of cluster-peer setup). If action is "reuse",
    then do nothing (return a task object referring to the existing cluster-peer).
    If action is "purge", then destroy the cluster-peer (see the C<purge> method,
    below) before creating a new one.

=item C<< nacltask_if_exists_lifs => $action_lif >>

    (Optional) What to do if the intercluster lifs is setup already. If the $action_lif is
    'reuse'(the default), then do nothing. If $action_lif is 'purge', then delete the
    existing lifs and create the new lifs.

=item C<< nacltask_if_exists_route => $action_route >>

    (Optional) What to do if the routes of intercluster lifs is setup already. If the $action_route is
    'reuse'(the default), then do nothing. If $action_route is 'purge', then delete the
    existing routes and create the new routes.

=item C<< nacltask_verify => 0|1 >>

    (Optional)
    - when set to 1, cluster-peer creation will be verified.
    - when set to 0, the verification step is skipped.
    - by default its set to 0.

=item C<< _was_created => \$scalar >>

    (Optional)
     When this option is provided a reference to a scalar variable, the variable
     gets filled in with a boolean value describing whether the cluster peer
     was found (value will be 0; this scenario is possible when
     nacltask_if_exists_peer => "reuse") or whether the cluster-peer was created (value will be 1).
     This is necessary to determine whether the cluster-peer needs to be
     cleaned up later.

        my $was_created;
        my $clusterpeer_obj = NACL::MTask::clusterpeer->create(
            nacltask_if_exists_peer    => 'reuse',
            _was_created => \$was_created,
            %other_opts
            );

        # Operate on $clusterpeer_obj here
        # ...

        # Now determine whether to clean up/purge the cluster-peer, since we're not sure
        # whether we reused an existing cluster-peer or created a new one
        if ($was_created) {
            # New cluter-peer was created. Clean it up.
            $clusterpeer_obj->purge();
        }

=item Other options

All of the other various options accepted by
L<< NACL::C::ClusterPeer->create|lib-NACL-C-ClusterPeer-pm/create >>.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            home_cluster_interface   => { isa  => 'NACL::C::CommandInterface::ONTAP' },
            remote_cluster_interface => { isa  => 'NACL::C::CommandInterface::ONTAP' },
            nacltask_lif_details     => { type => HASHREF, optional => 1 },
            nacltask_verify    => { type => BOOLEAN, default => 0 },
            _was_created       => { type => SCALARREF, optional => 1 },
            nacltask_if_exists_peer => {
                type      => SCALAR,
                default   => 'die',
                callbacks => {
                    'Any of "die", "reuse" or "purge"' => sub {
                        if (   $_[0] eq 'die'
                            || $_[0] eq 'reuse'
                            || $_[0] eq 'purge' )
                        {
                            return 1;
                        } ## end if ( $_[0] eq 'die' ||...
                        return 0;
                        }
                }
            },
            nacltask_if_exists_lif => {
                type      => SCALAR,
                default   => 'reuse',
                callbacks => {
                    'Any of "reuse" or "purge"' => sub {
                        if (   $_[0] eq 'reuse'
                            || $_[0] eq 'purge' )
                        {
                            return 1;
                        } ## end if ( $_[0] eq 'reuse' ...
                        return 0;
                        }
                }
            },
            nacltask_if_exists_route => {
                type      => SCALAR,
                default   => 'reuse',
                callbacks => {
                    'Any of "reuse" or "purge"' => sub {
                        if (   $_[0] eq 'reuse'
                            || $_[0] eq 'purge' )
                        {
                            return 1;
                        } ## end if ( $_[0] eq 'reuse' ...
                        return 0;
                        }
                }
            },

            %{ NACL::C::Component->_common_validate_spec_without_ci() },
            timeout => { type => SCALAR, default  => 60 },
        },
        allow_extra         => 1,
    );

    $Log->debug(
        "opts to NACL::MTask::ClusterPeer::Create " . Dumper(%opts) );

    my $home_cluster_interface   = delete $opts{home_cluster_interface};
    my $remote_cluster_interface = delete $opts{remote_cluster_interface};
    my $if_exists                = delete $opts{nacltask_if_exists_peer};
    my $if_exists_lif            = delete $opts{nacltask_if_exists_lif};
    my $if_exists_route          = delete $opts{nacltask_if_exists_route};
    my $verify                   = delete $opts{nacltask_verify};
    my $was_created              = delete $opts{_was_created};
    my $timeout                  = delete $opts{timeout};

    $$was_created = 0;
    $pkg = NACL::MTask::ClusterPeer->new(
                    home_cluster_interface => $home_cluster_interface,
                    remote_cluster_interface => $remote_cluster_interface,
                    );

    #Local variables
    my @peer_address_for_home_cluster   = ();
    my @peer_address_for_remote_cluster = ();
    my ($peer_cluster_of_home_cluster,$peer_cluster_of_remote_cluster);
    my $self;
    my $action;
    my @clusters = ();
    my $lif_details;
    my %node_details_vars = ();

    # get the nacltask_lif_details
    if ( !exists( $opts{nacltask_lif_details} ) ) {
        $lif_details = $pkg->_get_lifs_details_from_tharnhost(
            home_cluster_interface   => $home_cluster_interface,
            remote_cluster_interface => $remote_cluster_interface,
            %opts
        );
    } else {
        $lif_details = delete $opts{nacltask_lif_details};
    }
    $Log->debug(
        "List of intercluster lifs to be created " . Dumper($lif_details) );

    # Now %opts is containing common_option and other options(allow_extra => 1)
    my %common_opts;
    $pkg->_move_common_component_params(
        source => \%opts,
        target => \%common_opts
    );
    # Now %opts will contains only others parameters

    # Check the health of both filers
    my @nodes_in_home_cluster = NACL::C::Cluster->nodes(
        command_interface => $home_cluster_interface,
        %common_opts
    );
    my @nodes_in_remote_cluster = NACL::C::Cluster->nodes(
        command_interface => $remote_cluster_interface,
        %common_opts
    );
    $Log->debug( "Nodes in Home Cluster " . Dumper(@nodes_in_home_cluster) );
    $Log->debug( "Nodes in Remote cluster " . Dumper(@nodes_in_remote_cluster) );

    my @cluster_name;
    foreach my $interface_obj ($remote_cluster_interface,$home_cluster_interface) {
        my $cluster_obj = NACL::CS::ClusterIdentity->fetch(
                            command_interface => $interface_obj,
                            requested_fields  => ['name'],
                        );
        push (@cluster_name, $cluster_obj->name());
    }

    $peer_cluster_of_home_cluster = $cluster_name[0];
    $peer_cluster_of_remote_cluster = $cluster_name[1];
    # Creating a data structure for internal use
    foreach (@nodes_in_home_cluster) {
        $node_details_vars{$_} = {
            home_cluster_interface   => $home_cluster_interface,
            remote_cluster_interface => $remote_cluster_interface,
            node                     => $_,
            intercluster_ip          => [],
        };
    } ## end foreach (@nodes_in_home_cluster)
    foreach (@nodes_in_remote_cluster) {
        $node_details_vars{$_} = {
            home_cluster_interface   => $remote_cluster_interface,
            remote_cluster_interface => $home_cluster_interface,
            node                     => $_,
            intercluster_ip          => [],
        };
    } ## end foreach (@nodes_in_remote_cluster)

    while ( my ( $node, $lif ) = each( %{$lif_details} ) ) {
        foreach my $lif_config ( @{$lif} ) {
            push @{ $node_details_vars{$node}->{intercluster_ip} },
                $lif_config->{address};
        }
    } ## end while ( my ( $node, $lif ...

    # Both 'clusters' clocks should reasonably synchronized.
    # The maximum time delta between the two clusters is 300 seconds
    $pkg->_verify_cluster_time(
        home_cluster_interface   => $home_cluster_interface,
        remote_cluster_interface => $remote_cluster_interface,
        %common_opts
    );

    my @lif_objs = ();
    my @route_objs = ();
    $Log->trace("creating lifs");
    while ( my ( $node, $lif ) = each( %{$lif_details} ) ) {
        my ($was_lif_created,$was_route_created);
        foreach my $lif_config ( @{$lif} ) {
             my $gateway = delete $lif_config->{gateway};
            if ($pkg->_has_ipspaces_ms4( command_interface =>
                    $node_details_vars{$node}->{home_cluster_interface})) {
                my $vserver = NACL::STask::Vserver->find(
                    command_interface => $node_details_vars{$node}->{home_cluster_interface},
                    filter => {type => "admin"},
                );
               $lif_config->{vserver} = $vserver->vserver();
            }
            my $lif_obj = NACL::STask::NetworkInterface->create(
                        command_interface =>
                        $node_details_vars{$node}->{home_cluster_interface},
                        role               => 'intercluster',
                        nacltask_if_exists => $if_exists_lif,
                        %{$lif_config},
                        %common_opts
                    );

            if($gateway) {
                my $state_obj = $lif_obj->state();
                my $vserver = $state_obj->vserver();
                my $route_obj;
                my %route_opts = (
                                command_interface => $node_details_vars{$node}->{home_cluster_interface},
                                vserver            => $vserver,
                                gateway            => $gateway,
                                destination        => '0.0.0.0/0',
                                nacltask_if_exists => $if_exists_route,
                           );
                if ($pkg->_has_ipspaces_ms4(command_interface =>
                    $node_details_vars{$node}->{home_cluster_interface},)) {
                    $route_obj = NACL::STask::NetworkRoute->create(%route_opts);
                } else {
                    $route_opts{'routing-group'} = $state_obj->routing_group();
                    $route_obj = NACL::STask::NetworkRoutingGroupsRoute->create(%route_opts);
                }
                push (@route_objs,$route_obj);
                $lif_config->{gateway} = $gateway;
            }
            push (@lif_objs,$lif_obj);
        } ## end foreach my $lif_config ( @{...
    } ## end while ( my ( $node, $lif ...
    $pkg->intercluster_lifs(@lif_objs);
    $pkg->intercluster_routes(@route_objs);

    # pinging the intercluster ip from cluster
    my $node_name;
    while ( my ( $node, $node_detail ) = each(%node_details_vars) ) {
        if ($pkg->_has_ipspaces_ms4( command_interface =>
                $node_detail->{remote_cluster_interface})) {
            $node_name = NACL::C::Node->find(
                command_interface =>$node_detail->{remote_cluster_interface}
            )->node();
        } else {
            $node_name = $node_detail->{remote_cluster_interface}->name();
        }
        foreach my $intercluster_ip ( @{ $node_detail->{intercluster_ip} } ) {
            $Log->trace( "Pinging intercluster ip $intercluster_ip of filer "
                    . $node
                    . " from filer "
                    . $node_detail->{remote_cluster_interface}->name() );
            my $ipv4_worked = 0;
            try {
                NACL::C::Network->ping(
                    command_interface => $node_detail->{remote_cluster_interface},
                    destination       => $intercluster_ip,
                    node              => $node_name,
                    %common_opts
                );
                $ipv4_worked = 1;
            } catch NATE::BaseException with {
                my $ex = shift;
                $Log->warn("Ping failed with exception:".$ex->text().". Trying ping6");
            }; # Assuming here that it's an ipv6 address, next exception will be thrown

            if(!$ipv4_worked) {
                NACL::C::Network->ping6(
                    command_interface => $node_detail->{remote_cluster_interface},
                    destination       => $intercluster_ip,
                    node              => $node_name,
                    %common_opts
                );
            }
        } ## end foreach my $intercluster_ip...
    } ## end while ( my ( $node, $node_detail...
    foreach (@nodes_in_remote_cluster) {
        push @peer_address_for_home_cluster,
             @{$node_details_vars{$_}->{intercluster_ip} };
    }
    foreach (@nodes_in_home_cluster) {
        push @peer_address_for_remote_cluster,
            @{$node_details_vars{$_}->{intercluster_ip} };
    }

    # %opts contains all the other option which is accepted by the
    # NACL::C::ClusterPeer->create
    $Log->trace("creating Cluster peer");
    my $peer_obj = NACL::STask::ClusterPeer->create(
        %opts,
        command_interface   => $home_cluster_interface,
        remote_interface    => $remote_cluster_interface,
        create_options_home_cluster =>
            {'peer-addrs' => \@peer_address_for_home_cluster},
        create_options_remote_cluster =>
            {'peer-addrs' => \@peer_address_for_remote_cluster},
        timeout            => $timeout,
        nacltask_if_exists => $if_exists,
        _was_created       => $was_created,
        nacltask_wait      => $verify,
        %common_opts,
    );

    $pkg->{cluster}  = $peer_obj->cluster();
    $pkg->{rcluster} = $peer_obj->remote_cluster_peer()
        ->cluster();
    $pkg->peer_stask($peer_obj);
    $Log->exit() if $may_exit;
    return $pkg;
} ## end sub create

=head2 purge

      NACL::MTask::ClusterPeer->purge(
        home_cluster_interface => $home_cluster_interface,
        remote_cluster_interface => $remote_cluster_interface,
        unique_key_for_home_cluster => { 'cluster' => $cluster_name },
        unique_key_for_remote_cluster => { 'cluster' => $remote_cluster_name },
      );

      or

      $ClusterPeer->purge(nacltask_delete_lifs  => 1); #default 0

      (Class or instance method) This method deletes a cluster-peer on the given nodes.

=over

=item Options

=over

=item home_cluster_interface

    (Required for Class Method) As C<NACL::C::Node>. A component object that represents the
    node to which to send commands on home cluster.

=item remote_cluster_interface

    (Required for Class Method) As C<NACL::C::Node>. A component object that represents the
    node to which to send commands on remote cluster.

=item unique_key_for_home_cluster

    (Required for Class Method) A cluster name to match the cluster peer on home_cluster_interface.

=item unique_key_for_remote_cluster

    (Required for Class Method) A remote cluster name to match the cluster peer on remote_cluster_interface.

=item nacltask_delete_lifs

(Optional) Applicable only for instance method.

    It will delete the intercluster lifs created by this task only.
    If 0, (the default), don't delete the intercluster lifs.
    If 1, delete all the intercluster lifs also.

=item nacltask_delete_routes

(Optional) Applicable only for instance method.

    It will delete the intercluster routes created by this task only.
    If 0, (the default), don't delete the routes of intercluster lifs.
    If 1, delete all the routes of intercluster lifs also.

=item C<< nacltask_if_lif_purged => die | pass >>

(Optional, defaults to "die")  Applicable only for instance method.
Specifies what to do if the lif we're trying to purge no longer
exists. A value of C<die> (the default) will result in an exception
being thrown through the STask library of type
NACL::C::NetworkInterface::DoesNotExist.
A value of C<pass> will mean that the exception that would
normally occur when we're trying to delete a lif that does not exist
is suppressed.

=item C<< nacltask_if_route_purged => die | pass >>

(Optional, defaults to "die")  Applicable only for instance method.
Specifies what to do if the route we're trying to purge no longer
exists. A value of C<die> (the default) will result in an exception
being thrown by the STask library of type
NACL::C::NetworkRoutingGroupsRoute::DoesNotExist.
A value of C<pass> will mean that the exception that would normally occur
when we're trying to delete a aggregate that does not exist is suppressed.

=item C<< nacltask_lif_details => \%nacltask_lif_details >>

    (Optional) This contains all the lifs details for deletion.
     Applicable only for class method.

     my %nacltask_lif_details= (
         "$home_node_1" =>           # Name of the filer
            [
                {
                    vserver           => $vserver_name,
                    lif               => $lif_name_1,
                },
                {
                    .....
                }
            ],
         "$remote_node_1" => [{...},{...}]
     );


    These options are mandatory if user is passing %nacltask_lif_details.

    lif
    (Optional) The name of the logical interface ,used to identify it uniquely.
    The name of the logical interface is unique within a particular vserver.

    vserver
    (Optional) The name of the vserver that contains the lif.

=item C<< nacltask_route_details => \%nacltask_route_details >>

    (Optional) This contains all the route details for deletion.\
    Applicable only for class method.

     my %nacltask_route_details= (
         "$home_node_1" =>           # Name of the filer
            [
                {
                    vserver           => $vserver_name,
                    destination       => $destination,
                    'routing-group'   => $routing_group,
                },
                {
                    .....
                }
            ],
         "$remote_node_1" => [{...},{...}]
     );


    These options are mandatory if user is passing %nacltask_lif_details.

    destination
    (Optional) The destination represented as IpAddress/Mask.

    routing-group
    The name of the routing group, used to identify it uniquely.
    (routing groups names are assumed to be unique within a node or within a
    vserver)

    vserver
    (Optional) The name of the vserver that contains the lif.

=back

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            %{ $pkg_or_obj->_purge_validate_spec() },
            %{ NACL::C::Component->_common_validate_spec_without_ci() },
            nacltask_lif_details     => { type => HASHREF, optional => 1 },
            nacltask_route_details   => { type => HASHREF, optional => 1 },
            nacltask_if_route_purged => {type => SCALAR, default => 'die'},
            nacltask_if_lif_purged   => {type => SCALAR, default => 'die'},
        },
    );
    my $home_cluster_interface   = delete $opts{home_cluster_interface};
    my $remote_cluster_interface = delete $opts{remote_cluster_interface};
    my $nacltask_delete_lifs = delete $opts{nacltask_delete_lifs};
    my $nacltask_delete_routes = delete $opts{nacltask_delete_routes};
    my $nacltask_if_lif_purged = delete $opts{nacltask_if_lif_purged};
    my $nacltask_if_route_purged = delete $opts{nacltask_if_route_purged};
    my $nacltask_lif_details = delete $opts{nacltask_lif_details};
    my $nacltask_route_details = delete $opts{nacltask_route_details};

    $Log->debug( "opts to NACL::MTask::ClusterPeer::purge " . Dumper(%opts) );

    my $lif_details;
    my $route_details;
    my %node_details_vars = ();
    if ( ref($pkg_or_obj) ) {
        $pkg_or_obj->peer_stask()->purge();
        my @lif_objs = @{$pkg_or_obj->intercluster_lifs()};
        my @route_objs = @{$pkg_or_obj->intercluster_routes()};
        foreach my $lif_obj (@lif_objs) {
            if( $nacltask_delete_lifs || defined($lif_obj->_was_lif_created())) {
                $lif_obj->purge(nacltask_if_purged => $nacltask_if_lif_purged);
            }
        }
        foreach my $route_obj (@route_objs) {
            if ($nacltask_delete_routes || defined($route_obj->{_was_route_created})) {
                $route_obj->purge(nacltask_if_purged => $nacltask_if_route_purged);
            }
        }
    } else {
        NACL::STask::ClusterPeer->purge(
                command_interface  => $home_cluster_interface,
                remote_interface   => $remote_cluster_interface,
                nacltask_if_purged => 'pass',
                );

        # Check the health of both filers
        my @nodes_in_home_cluster = NACL::C::Cluster->nodes(
            command_interface => $home_cluster_interface,
        );
        my @nodes_in_remote_cluster = NACL::C::Cluster->nodes(
            command_interface => $remote_cluster_interface,
        );

        # Creating a data structure for internal use
        foreach (@nodes_in_home_cluster) {
            $node_details_vars{$_} = {
                home_cluster_interface   => $home_cluster_interface,
                remote_cluster_interface => $remote_cluster_interface,
                node                     => $_,
                intercluster_ip          => [],
            };
        } ## end foreach (@nodes_in_home_cluster)
        foreach (@nodes_in_remote_cluster) {
            $node_details_vars{$_} = {
                home_cluster_interface   => $remote_cluster_interface,
                remote_cluster_interface => $home_cluster_interface,
                node                     => $_,
                intercluster_ip          => [],
            };
        } ## end foreach (@nodes_in_remote_cluster)


        if ($nacltask_delete_lifs && !defined $nacltask_lif_details) {
            $nacltask_lif_details = $pkg_or_obj->_get_lifs_details_from_tharnhost(
                    home_cluster_interface   => $home_cluster_interface,
                    remote_cluster_interface => $remote_cluster_interface,
                    %opts
                );
        }

        while ( my ( $node, $lif ) = each( %{$nacltask_lif_details} ) ) {
            foreach my $lif_config ( @{$lif} ) {
                my $gateway = delete $lif_config->{gateway};
                if ($pkg_or_obj->_has_ipspaces_ms4( command_interface =>
                        $node_details_vars{$node}->{home_cluster_interface})) {
                    my $vserver = NACL::CS::Vserver->fetch(
                        command_interface => $node_details_vars{$node}->{home_cluster_interface},
                        filter => {type => "admin"},
                    );
                   $lif_config->{vserver} = NACL::CS::Vserver->fetch(
                        command_interface => $node_details_vars{$node}->{home_cluster_interface},
                        filter => {type => "admin"},)->vserver();
                }

                NACL::STask::NetworkInterface->purge(
                    command_interface =>
                    $node_details_vars{$node}->{home_cluster_interface},
                    vserver => $lif_config->{vserver},
                    lif     => $lif_config->{lif},
                    nacltask_if_purged => $nacltask_if_lif_purged,
                );
            }
        }

        if ($nacltask_delete_routes && !defined $nacltask_route_details){
            NATE::BaseException->throw(
                "For Class Method call route_details needs to be passed");
        }
        while ( my ( $node, $route ) = each( %{$nacltask_route_details} ) ) {
            foreach my $route_config ( @{$route} ) {
                if ($pkg_or_obj->_has_ipspaces_ms4(command_interface =>
                    $node_details_vars{$node}->{home_cluster_interface},)) {
                    NACL::STask::NetworkRoute->purge(
                    command_interface =>
                    $node_details_vars{$node}->{home_cluster_interface},
                    %{$route_config},
                    nacltask_if_purged => $nacltask_if_route_purged,);
                } else {
                    NACL::STask::NetworkRoutingGroupsRoute->purge(
                    command_interface =>
                    $node_details_vars{$node}->{home_cluster_interface},
                    %{$route_config},
                    nacltask_if_purged => $nacltask_if_route_purged,);
                }
            }
        }
    }
    $Log->exit() if $may_exit;
} ## end sub purge

# Code for verification of clusterpeer creation

sub _taskverify_create {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;

    my $home_cluster_interface = delete( $opts{home_cluster_interface} )
        || $self->{home_cluster_interface};
    my $remote_cluster_interface = delete( $opts{remote_cluster_interface} )
        || $self->{remote_cluster_interface};

    my ( $clusterpeers_in_hcluster,$clusterpeers_in_rcluster) ;
    my $test = sub {
        $clusterpeers_in_hcluster = NACL::CS::ClusterPeer->fetch(
            command_interface => $home_cluster_interface,
            filter            => {cluster => $self->{cluster}, availability => 'Available'},
            %opts
        );

        $clusterpeers_in_rcluster = NACL::CS::ClusterPeer->fetch(
            command_interface => $remote_cluster_interface,
            filter            => {cluster => $self->{rcluster}, availability => 'Available'},
            %opts
        );
    };
    # after the clutser peer creation it may take sometime to reflect in vsim so wait for sometime
    nacl_method_retry(
        code        => $test,
        tries_count => 4,
        sleep_time  => 10,
        exceptions  => 'NACL::Exceptions::NoElementsFound'
    );

    if (!(defined($clusterpeers_in_hcluster) && defined($clusterpeers_in_rcluster))) {
        NATE::BaseException->throw("Cluster-peer creatation failed".Dumper($clusterpeers_in_hcluster).Dumper($clusterpeers_in_rcluster));
    }
}


# This method make sure that your cluster's clocks are reasonably synchronized.
# The maximum time delta between the two clusters is 300 seconds. This method
# will throw the exceptions, if time delta is greater than 300 seconds.

sub _verify_cluster_time {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = @_;

    my $home_cluster_interface   = delete( $opts{home_cluster_interface} );
    my $remote_cluster_interface = delete( $opts{remote_cluster_interface} );

    # Check the time delta between all filers should not greater than 300 sec
    my @date1 = NACL::CS::ClusterDate->fetch(
        command_interface => $home_cluster_interface,
        requested_fields  => [ 'node', 'utcdateandtime' ],
        %opts
    );
    my @date2 = NACL::CS::ClusterDate->fetch(
        command_interface => $remote_cluster_interface,
        requested_fields  => [ 'node', 'utcdateandtime' ],
        %opts
    );
    my @date = ();
    foreach ( @date1, @date2 ) {
        my ( $node_name, $time ) = ( $_->node(), $_->utcdateandtime() );
        $time =~ s/^"|"$//g;
        my $seconds = parsedate($time);
        $Log->debug(
            "Dates of Node (utcdateandtime) $node_name: $time: $seconds");
        push( @date, $seconds );
    } ## end foreach ( @date1, @date2 )
    my @sorted = sort { $a <=> $b } @date;
    if ( ( $sorted[-1] - $sorted[0] ) > 300 ) {
        NATE::BaseException->throw(
                  "The Time Difference between all the filers"
                . " is is more than 300 sec"
                . Dumper( \@sorted ) );
    } ## end if ( ( $sorted[-1] - $sorted...
    $Log->exit() if $may_exit;
} ## end sub _verify_cluster_time

# This method make return matching clusterpeer objects based
# on filter from your both clusters.

sub _find_clusterpeer {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = @_;

    my $filter_for_hcluster = delete( $opts{unique_key_for_home_cluster} );
    my $filter_for_rcluster = delete( $opts{unique_key_for_remote_cluster} );
    my $home_cluster_interface = delete( $opts{home_cluster_interface} )
        || $self->{home_cluster_interface};
    my $remote_cluster_interface = delete( $opts{remote_cluster_interface} )
        || $self->{remote_cluster_interface};

    my @clusterpeers_in_hcluster = ();
    my @clusterpeers_in_rcluster = ();

    @clusterpeers_in_hcluster = NACL::C::ClusterPeer->find(
        command_interface => $home_cluster_interface,
        filter            => {%$filter_for_hcluster},
        allow_empty       => 1,
        %opts
    );

    @clusterpeers_in_rcluster = NACL::C::ClusterPeer->find(
        command_interface => $remote_cluster_interface,
        filter            => {%$filter_for_rcluster},
        allow_empty       => 1,
        %opts
    );

    my @ret_arr = ();
    @ret_arr = ( @clusterpeers_in_hcluster, @clusterpeers_in_rcluster );

    $Log->debug( "Got the Cluster peers in cluster "
            . $home_cluster_interface->node() . " :"
            . Dumper( \@clusterpeers_in_hcluster ) );
    $Log->debug( "Got the Cluster peers in cluster "
            . $remote_cluster_interface->node() . " :"
            . Dumper( \@clusterpeers_in_rcluster ) );
    $Log->exit() if $may_exit;
    return @ret_arr;
} ## end sub _find_clusterpeer

# Get the intercluster lifs details from tharnhost files

sub _get_lifs_details_from_tharnhost {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = @_;

    my $home_cluster_interface = delete( $opts{home_cluster_interface} )
        || $self->{home_cluster_interface};
    my $remote_cluster_interface = delete( $opts{remote_cluster_interface} )
        || $self->{remote_cluster_interface};

    $Log->trace("getting the lifs information from tharnhost files");

    my $lif_details_tharnhost = {};
    my @error                 = ();

    foreach
        my $interface ( $home_cluster_interface, $remote_cluster_interface )
    {
        my @nodes = NACL::C::Cluster->node_objs(
            command_interface => $interface,
            %opts
        );
        foreach my $node_obj (@nodes) {
            $lif_details_tharnhost->{ $node_obj->node() } = [];
            my $hash = NACL::STask::Node->get_hostrec_lif_info(
                command_interface => $node_obj,
                roles             => ['INTERCLUSTER'],
                required_fields   => [qw( NAME IP MASK PORT)],
                %opts
            );
            try {
                my $gateway = NACL::STask::Node->get_hostrec_lif_info(
                    command_interface => $node_obj,
                    roles             => ['INTERCLUSTER'],
                    required_fields   => [qw(GATEWAY)],
                    %opts
                );
                $hash->{INTERCLUSTER}->[0]->{gateway} = $gateway->{INTERCLUSTER}->[0]->{gateway};
            } catch NATE::BaseException with {
            };
            foreach my $_lif ( @{ $hash->{INTERCLUSTER} } ) {
                my $node_name = $node_obj->node();
                my $lif_name = '';
                if ($_lif->{NAME} =~ /^$node_name/) {
                    $lif_name = $_lif->{NAME};
                } else {
                    $lif_name = $node_name . '_'  . $_lif->{NAME};
                }
                push @{ $lif_details_tharnhost->{ $node_obj->node() } },
                    {
                    vserver     => $node_obj->node(),
                    lif         => $lif_name,
                    address     => $_lif->{IP},
                    netmask     => $_lif->{MASK},
                    'home-node' => $node_obj->node(),
                    'home-port' => $_lif->{PORT},
                    gateway     => $_lif->{GATEWAY},
                    };
            } ## end foreach my $_lif ( @{ $hash...
            if (scalar( @{ $lif_details_tharnhost->{ $node_obj->node() } } )
                < 1 )
            {
                push @error, $node_obj->node();
            }

        } ## end foreach my $node_obj (@nodes)
    } ## end foreach my $interface ( $home_cluster_interface...

    $Log->exit() if $may_exit;

    if ( $#error >= 0 ) {
        NATE::BaseException->throw(
                  join( ',', @error )
                . " tharnhost file doesn't contains INTERCLUSTER(NAME IP MASK PORT)"
                . " lifs details and user does not provide \%nacltask_lif_details" );
    } ## end if ( $#error >= 0 )
    return $lif_details_tharnhost;
} ## end sub _get_lifs_details_from_tharnhost

# Returns validate spec for purge() method based on if the purge() is invoked
# as instance method or class method

sub _purge_validate_spec {
    my $self = shift;
    if ( ref($self) ) {
        return { nacltask_delete_lifs => { type => BOOLEAN, default => 0 },
                nacltask_delete_routes => { type => BOOLEAN, default => 0 }
               };
    } else {
        return {
            home_cluster_interface =>
                { type => OBJECT, isa => 'NACL::C::CommandInterface::ONTAP' },
            remote_cluster_interface =>
                { type => OBJECT, isa => 'NACL::C::CommandInterface::ONTAP' },
            unique_key_for_home_cluster   => { type => HASHREF, optional => 1},
            unique_key_for_remote_cluster => { type => HASHREF, optional => 1},
            nacltask_delete_routes => { type => BOOLEAN, default => 0 },
            nacltask_delete_lifs => { type => BOOLEAN, default => 0 },
        };
    } ## end else [ if ( ref($self) )
} ## end sub _purge_validate_spec

sub _has_ipspaces_ms4 {
    my ($pkg, %opts) = @_;
    if (($opts{command_interface}->has_uichange(
                uichange        => 'ipspaces-ms4',
                throw_exception => 0
            ) ) ) {
        return 1;
    } else {
        return 0;
    }
}

1;
