# $Id: //depot/prod/test/nacldev/lib/NACL/MTask/DFM/Rbac.pm#1 $
#
## Copyright (c) 2012 NetApp, Inc., All Rights Reserved
##   Any use, modification, or distribution is prohibited
##   without prior written consent from NetApp, Inc.
##
#

## @summary Rbac Task Module
## @author Ramya M R, dl-mpo-nacl-dev@netapp.com
## @change
##
## @status shared
## @pod here

=head1 NAME

NACL::MTask::DFM::Rbac

=head1 DESCRIPTION

This task is to be used for 
- Creating an Rbac User.
- Deleting the Rbac User

Creating an Rbac User involves:
- Checking if the DFM User already added, if not, then add the user.
- If the User already added,action ('reuse','purge' or 'die') that needs to be taken by the task
  can be specified
- Check if the roles are created, if not, then create.
- If the role already exists then action ('reuse','purge' or 'die' that needs to be taken by the task
  can be specified
- Add the roles to the user.
- Add the capability to the roles.
- If the capability is already added, then action ('pass' or 'die' that needs to be taken by the task
  can be specified

Deleting the Rbac User involves:
- Removing the Capabilities to the roles assigned to the user.
- If the capabilities are already removed, then the action('pass' or 'die') that needs to be taken by the task
  can be specified.
- Destroying the roles assigned to the user.
- If the role does not exist, then action ('reuse','purge' or 'die' that needs to be taken by the task
  can be specified
- Deleting the DFM User.
- If the DFM User is already removed, then action ('reuse','purge' or 'die' that needs to be taken by the task
  can be specified

 
=cut

package NACL::MTask::DFM::Rbac;

use strict;
use warnings;

use base qw(NACL::MTask::MTask );

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate
    qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR SCALARREF);
use NATE::Exceptions::Argument qw(:try);
use NACL::Exceptions::OperationFailed;

use NACL::STask::DFM::Rbac;
use NACL::STask::DFM::RbacUser;


=head1 METHODS

=head2 create_and_assign

    $rbac_user->create_and_assign(
	'roles_and_capabilities' => $roles_and_capabilities_hashref,
	);
(Instance method)

    NACL::MTask::DFM::Rbac->create_and_assign( 
	'command_interface'                   => $ci,
	'user-name'                           => $user_name,
	'roles_and_capabilities'              => $roles_and_capabilities_hashref,
	'nacltask_if_user_exists'             => $action,      # default 'reuse'
	 nacltask_if_role_exists              => $action, # default 'reuse'
	 nacltask_if_capability_already_added => $action, # default 'pass'
	 nacltask_if_role_already_assigned    => $action, #default 'pass'
	 _was_user_added                      => \$was_user_added,
	 _was_role_created                    => \$was_role_created,
	 _was_user_role_added                 => \$was_user_role_added,
	  ...
	
	
    );
(Class method)
    
Assign the roles and capabilities to the roles.The method returns the array of role state objects.

=item Options

=over

=item C<< command_interface => $ci >>

(Required, isa NACL::C::DFM::DFMClient)
The command interface through which the monitors for object will be refreshed.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=item C<< user-name => $user_name >>

(Required)
The name of the user

=item C<< nacltask_if_user_exists => $action >>

(Optional) What to do if the user to be added already exists.
If action is "reuse" (the default), then do nothing (return a 
task object referring to the existing user). If $action is "die",
then fail with an exception . If action is "purge",
then delete the user (see the C<remove> method, below) before 
creating a new one.

=item C<< nacltask_if_role_exists => $action >>

(Optional) What to do if the role to be created already exists.
If action is "reuse" (the default), then do nothing (return a 
task object referring to the existing role). If $action is "die",
then fail with an exception . If action is "purge",
then delete the role (see the C<destroy> method, below) before 
creating a new one.

=item C<< nacltask_if_role_already_assigned => $action >>

(Optional) What to do if the role to be added is already assigned to the user.
If action is "pass" (the default), then do nothing . If $action is "die",
then fail with an exception . 

=item C<< nacltask_if_capability_already_added => $action >>

(Optional) What to do if the capability to be added is already added to the role.
If action is "pass" (the default), then do nothing . If $action is "die",
then fail with an exception . 

=item C<< _was_user_added => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the user
was found (value will be 0; this scenario is possible when 
if_exists => "reuse") or whether the user was added (value will be 1).
This is necessary to determine whether the added user needs to be
cleaned up later.

item C<< _was_role_created => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the role
was found (value will be 0; this scenario is possible when 
if_exists => "reuse") or whether the role was created (value will be 1).
This is necessary to determine whether the created role needs to be
cleaned up later.

item C<< _was_user_role_added => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the role
was already assigned (value will be 0; this scenario is possible when 
nacltask_if_role_already_assigned => "pass") or whether the role was added (value will be 1).
This is necessary to determine whether the added role needs to be
cleaned up later.

=item C<< user_creation_opts => \$hashref >>

(Required)
This hash contains the options specifically needed to add a user.All of the various options to
L<< NACL::C::DFM::RbacUser->add | lib-NACL-C-DFM-RbacUser-pm/add >>. 
This is a hashref. 

	my %user_creation_opts;
	$user_creation_opts{'user-name'} = $username;
	$user_creation_opts{'email-address'} = $email_address;

	NACL::MTask::DFM::Rbac->create_and_assign (
	command_interface => $command_interface,
	..
	user_creation_opts => \%user_creation_opts
	);

=item C<< role_creation_opts => \$hashref >>

(Optional)
This hash contains the options specifically needed to create a role.All of the various options to
L<< NACL::C::DFM::RbacUser->add | lib-NACL-C-DFM-RbacUser-pm/add >>. 
This needs is a hashref.The hash inturn contains hashes where role names are mapped to the
associated option values for owner role name or id and description.

	my %role_creation_opts;
	$role_creation_opts{$role_name1} = {'owner-role-name-or-id' => $owner_role_name_or_id, 'description' => $description };
	$role_creation_opts{$role_name2} = {'owner-role-name-or-id' => $owner_role_name_or_id, 'description' => $description };
	.
	.

	NACL::MTask::DFM::Rbac->create_and_assign (
	command_interface => $command_interface,
	..
	role_creation_opts => \%role_creation_opts,
	);

If this hash is not provided, then the role names in the roles_and_capabilities hash are considered for role creation.
	
=item C<< roles_and_capabilities => $roles_and_capabilities_hashref >>

(Required)
The reference to the hash containing the mapping between the roles and capabilities that need to be assigned. 

			{
			<role-name-or-id>  =>  {
								<operation-1> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
												
								<operation-2> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
								
								.
								.
								.
								.
								<operation-n> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
								
								 }
							 
			<role-name-or-id>  => { 
								<operation-1> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
												
								<operation-2> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
								
								.
								.
								.
								.
								<operation-n> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
								
							  }
			.
			.
			.
			.
			<role-name-or-id>  => { 
								<operation-1> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
												
								<operation-2> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
								
								.
								.
								.
								.
								<operation-n> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
								
							  }
			
			}


=back

=over

=cut

sub create_and_assign {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params => \@_,
	ignore_primary_keys => 1,
        additional_spec => {
            nacltask_if_user_exists => { type => SCALAR, optional  => 1 },
	    nacltask_if_role_exists => { type => SCALAR, optional  => 1 },
	    nacltask_if_capability_already_added => { type => SCALAR, optional  => 1 },
	    nacltask_if_role_already_assigned => { type => SCALAR, optional  => 1 },
            _was_user_added => { type => SCALARREF, optional => 1 },
	    _was_role_created => { type => SCALARREF, optional => 1 },
	    _was_user_role_added => { type => SCALARREF, optional => 1 },
	    roles_and_capabilities => { type => HASHREF },
	    user_creation_opts => { type => HASHREF},
	    role_creation_opts => { type => HASHREF,optional => 1},
        },
    );
	

        my $command_interface = $opts{command_interface};
	my $if_user_exists = delete $opts{nacltask_if_user_exists};
	my $if_role_exists = delete $opts{nacltask_if_role_exists};
	my $if_capability_added = delete $opts{nacltask_if_capability_already_added};
	my $if_role_assigned = delete $opts{nacltask_if_role_already_assigned};
	my $was_user_added  = delete $opts{_was_user_added};
	my $was_role_created  = delete $opts{_was_role_created};
	my $was_user_role_added  = delete $opts{_was_user_role_added};
	my $user_creation_opts = delete $opts{user_creation_opts};
	my $role_creation_opts = delete $opts{role_creation_opts};
	my %user_creation_opts = %{$user_creation_opts};
	my %roles_and_capabilities = %{$opts{roles_and_capabilities}};
	my %role_creation_opts;
	
	
	if ( defined $role_creation_opts ) {
		 %role_creation_opts = %{$role_creation_opts};
	}
	
	my $user_name = $user_creation_opts{'user-name'};
	$$was_user_added = 0;
	$$was_user_role_added = 0;
	$$was_role_created = 0;
	
	## Add a Rbac User
	
        my $RbacUser = NACL::STask::DFM::RbacUser->add(
	command_interface => $command_interface,
	nacltask_if_exists => $if_user_exists,
	_was_added => $was_user_added,
	%user_creation_opts);

	
	## Create all the roles
	
	if (%role_creation_opts) {
	while ( my ( $role, $opts ) = each (%role_creation_opts ) ) {
		my $role_object;
		my %opts = %{$opts};
		$role_object = NACL::STask::DFM::Rbac->create(
		command_interface => $command_interface,
		nacltask_if_exists => $if_role_exists,
		_was_created         => \$was_role_created,
		'role-name'    => $role,
		%opts
		); 
		
	## Add the role to the user	
	$role_object->add_role_to_user ('user-name-or-id' => $user_name);
		}
	}
	
	else {
		foreach my $role ( keys %roles_and_capabilities ) {
		my $role_object = NACL::STask::DFM::Rbac->create(
		command_interface => $command_interface,
		nacltask_if_exists => $if_role_exists,
		_was_created         => \$was_role_created,
		'role-name'    => $role,
		);
		$role_object->add_role_to_user ('user-name-or-id' => $user_name);
		}
	}
		
	## Parsing the roles and capabilities hash and assigning capabilities to the role
	
	while ( my ( $role, $capabilities_hash ) = each (%roles_and_capabilities ) ) {
		
		my $role_object = NACL::STask::DFM::Rbac->new(
		command_interface => $command_interface,
		'role-name-or-id' => $role,
		);
	
		## Adding Capabilities to the role
		while ( my ( $operation, $resources ) = each (%{$capabilities_hash} ) ) {
	
			foreach my $resource (@{$resources} ) {
		
			$role_object->capability_add (
				'operation' => $operation,
				'resource-name-or-id' => $resource,
				nacltask_if_capability_already_added => $if_capability_added ,	
				);	
			} ## end for
		} ## end while
		
			
	} ## end while
 
	return $RbacUser;
        $Log->exit() if $may_exit;
 } 
 

=head2 delete

(Instance method)

    NACL::MTask::DFM::Rbac->delete( 
	'command_interface'            => $ci,
	'user-name-or-id '             => $user_name_or_id,
	nacltask_if_capability_removed => $action, #default 'pass'
	nacltask_if_user_purged        => $action, #default 'die'
	nacltask_if_role_purged        => $action, #default 'die'
	'roles-and-capabilities'       => $roles_and_capabilities_hashref,	
    );
(Class method)
    
Removes the capabilities to the roles and destroys the roles

=item Options

=over

=item C<< command_interface => $ci >>

(Required, isa NACL::C::DFM::DFMClient)
The command interface through which the monitors for object will be refreshed.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=item C<< user_name_or_id => $user_name_or_id >>

(Required)
The name or id of the user

=item C<< nacltask_if_user_purged => die | pass >>

(Optional, defaults to "die")
Specifies what to do if the user we're trying to delete no longer
exists. A value of C<die> (the default) will result in an exception
being thrown. A value of C<pass> will mean that the exception that would
normally occur when we're trying to remove a user who does not exist
is suppressed.

=item C<< nacltask_if_capability_removed=> die | pass >>

(Optional, defaults to "pass")

Specifies what to do if the capability we're trying to remove is already removed
 A value of C<die> will result in an exception
being thrown. A value of C<pass> (default value) will mean that the exception that would
normally occur when we're trying to remove a capability assigned to a role which does not exist
is suppressed.

=item C<< nacltask_if_role_purged => die | pass >>

(Optional, defaults to "die")
Specifies what to do if the role we're trying to delete no longer
exists. A value of C<die> (the default) will result in an exception
being thrown. A value of C<pass> will mean that the exception that would
normally occur when we're trying to destroy a role which does not exist
is suppressed.

=item C<< roles_and_capabilities => $roles_and_capabilities_hashref >>

(Required)
The reference to the hash containing the mapping between the roles and capabilities that need to be deleted. 

			{
			<role-name-or-id>  =>  {
								<operation-1> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
												
								<operation-2> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
								
								.
								.
								.
								.
								<operation-n> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
								
								 }
							 
			<role-name-or-id>  => { 
								<operation-1> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
												
								<operation-2> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
								
								.
								.
								.
								.
								<operation-n> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
								
							  }
			.
			.
			.
			.
			<role-name-or-id>  => { 
								<operation-1> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
												
								<operation-2> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
								
								.
								.
								.
								.
								<operation-n> => [ <resource-1>,<resource-2>,<resource-3>,...<resource-n> ],
								
							  }
			
			}


=back

=over

=cut



sub delete {
     $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params  => \@_,
	ignore_primary_keys => 1,
        additional_spec => {
	        user_name_or_id => { type => SCALAR },
                nacltask_if_capability_removed => { type => SCALAR, optional  => 1 },
	        nacltask_if_user_purged => { type => SCALAR, optional  => 1 },
		nacltask_if_role_purged  => { type => SCALAR, optional  => 1 },
		roles_and_capabilities => { type => HASHREF,optional  => 1 },
        },
    );
	my $command_interface = $opts{command_interface};
	my $if_removed = delete $opts{nacltask_if_capability_removed};
	my $if_role_purged = delete $opts{nacltask_if_role_purged};
	my $if_user_purged = delete $opts{nacltask_if_user_purged};
	my $user_name_or_id = delete $opts{user_name_or_id};
	
	my %roles_and_capabilities = %{$opts{roles_and_capabilities}};
	while ( my ( $role, $capabilities_hash ) = each (%roles_and_capabilities ) ) {
	
		my $role_object = NACL::STask::DFM::Rbac->new(
		command_interface => $command_interface,
		'role-name-or-id' => $role,
		);
	
		## Adding Capabilities to the role
		while ( my ( $operation, $resources ) = each (%{$capabilities_hash} ) ) {
	
			foreach my $resource (@{$resources} ) {
		
			$role_object->capability_remove (
				'operation' => $operation,
				'resource-name-or-id' => $resource,
				nacltask_if_capability_removed => $if_removed ,	
				);	
			} ## end for
		} ## end while
	$role_object->remove_role_from_user ('user-name-or-id' => $user_name_or_id);
		}
	
	
}

1;

