#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::MTask::DataSet::FileReservationBasic;

use Moose;
use MTF::MooseCompatibleException qw(:all);

extends 'NACL::MTask::DataSet';

use Data::Dumper;

use NACL::C::VolumeFile;
use NACL::Exceptions::UnexpectedState;

=head1 NAME

NACL::MTask::DataSet::FileReservationBasic

=head1 SYNOPSIS

    my $filereservation_basic_dataset = NACL::MTask::DataSet::FileReservationBasic->create(
        volume => $volume,
        clients => [ @clients ],
    );

    $filereservation_basic_dataset = NACL::MTask::DataSet::FileReservationBasic->find(
        volume => $volume,
        clients => [ @clients ],
    );

    $filereservation_basic_dataset->validate();
    $filereservation_basic_dataset->purge();

=head1 DESCRIPTION

Please read L<NACL::MTask::DataSet|lib-NACL-MTask-DataSet-pm> to understand the
general behavior and capabilities of DataSet libraries.

The FileReservationBasic DataSet contains files with space reservations.

=head1 ATTRIBUTES

All attributes available from NACL::MTask::DataSet are available on this derived
class.

=cut

=head1 METHODS

Please read L<NACL::MTask::DataSet|lib-NACL-MTask-DataSet-pm> to understand the
general behavior and capabilities of DataSet libraries. Any methods defined on
C<NACL::MTask::DataSet> are applicable to this DataSet.

=head2 file_fill_enabled_overwrite_true

Returns a L<NACL::C::VolumeFile|lib-NACL-C-VolumeFile-pm> instance of a file 
with sparseness that has overwrite and fill reservations.

=head2 file_fill_disable_overwrite_false

Returns a L<NACL::C::VolumeFile|lib-NACL-C-VolumeFile-pm> instance of a file 
with sparseness that has no overwrite and no fill reservations.

=head2 validate

Validate will ensure that the file with a reservation still has the reservation
and the file without the reservation does not have one.

Throws C<NACL::Exceptions::UnexpectedState> if the validation fails.

=cut


sub create {
    my ($self_or_pkg, %args) = @_;
    my $self = $self_or_pkg->_new_if_required(%args);

    my $dataset_directory = $self->_create_dataset_directory();

    foreach my $reservation_args ($self->_reservation_combinations()) {
        my $file = $self->_space_reserved_file(%$reservation_args);

        $file->write(
            content => 'ABADCAFE'x1024,
            offset => 4096*128,
        );

        $file->reservation(
            %$reservation_args,
        );
    }

    $self->_write_manifest_file();

    $self->_fixup_permissions();

    return $self;
}

sub validate {
    my ($self) = @_;

    foreach my $reservation_args ($self->_reservation_combinations()) {
        my $file = $self->_space_reserved_file(%$reservation_args);

        my $zapi = $file->command_interface()->apiset(
            category => 'Node',
            interface => 'ZAPI',
            set => 'CMode',
        );
        
        my $parsed = $zapi->file_get_space_reservation_info(
            path => $file->path(),
            'vserver-override-name' => $file->vserver(),
        )->get_parsed_output();

        my $record = $parsed->[0];

        my $zapi_reservation_args = $self->_nacl_args_to_zapi(
            %$reservation_args
        );

        if (
            (
                $record->{'is-fill-enabled'} eq 
                $zapi_reservation_args->{'is-fill-enabled'}
            ) &&
            (
                $record->{'is-overwrite-enabled'} eq 
                $zapi_reservation_args->{'is-overwrite-enabled'}
            )
           ) {
            $Log->trace("The records were the same as expected");
        } else {
            NACL::Exceptions::UnexpectedState->throw(
                "Expected the state to be ".Dumper($zapi_reservation_args).
                " but was ".Dumper($record)." instead"
            );
        }
    }

    return $self;
}

sub _reservation_combinations {
    my ($self) = @_;

    return (
        { 'is-enabled' => 'enable', 'is-overwrite-enabled' => 'true' },
        { 'is-enabled' => 'disable', 'is-overwrite-enabled' => 'false' },
    );
}

# Function: _space_reserved_file
# Arguments:
#   is-enabled - enable/disable
#   is-overwrite-enabled - true/false
# Returns:
#   NACL::C::VolumeFile instance of a file to be created by the caller
sub _space_reserved_file {
    my ($self, %args) = @_;
    
    my $dataset_directory = $self->_get_dataset_directories();
    my $is_enabled = $args{'is-enabled'};
    my $is_overwrite_enabled = $args{'is-overwrite-enabled'};

    my $file_name = "fill_${is_enabled}_overwrite_${is_overwrite_enabled}";

    return NACL::C::VolumeFile->new(
        command_interface => $dataset_directory->command_interface(),
        vserver => $dataset_directory->vserver(),
        path => $dataset_directory->path()."/".$file_name,
    );
}

# Documented in the POD
sub file_fill_enable_overwrite_true {
    my ($self) = @_;

    return $self->_space_reserved_file(
        'is-enabled' => 'enable',
        'is-overwrite-enabled'=> 'true',
    );
}

sub file_fill_disable_overwrite_false {
    my ($self) = @_;

    return $self->_space_reserved_file(
        'is-enabled' => 'disable',
        'is-overwrite-enabled'=> 'false',
    );
}

# Convert NACL style arguments to ZAPI style arguments
sub _nacl_args_to_zapi {
    my ($self, %args) = @_;

    my %bool_nacl_to_zapi = (
        'disable' => 'false',
        'enable' => 'true',
    );
    my %ret = ();
    $ret{'is-overwrite-enabled'} = $args{'is-overwrite-enabled'};
    $ret{'is-fill-enabled'} = $bool_nacl_to_zapi{$args{'is-enabled'}};

    return \%ret;
}


=head1 REGRESSION TESTS

Regression tests for this library are located at
lib/NACL/UnitTest/MTask/DataSetFileReservationBasic.thpl

=head1 SEE ALSO

Please read L<NACL::MTask::DataSet|lib-NACL-MTask-DataSet-pm> to understand the
general behavior and capabilities of DataSet libraries.

Additional background on the DataSet framework is available at
https://wikid.netapp.com/w/NACL/Tasks/DataSet

=cut


1;
