#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::MTask::DataSet::QtreeBasic;

use Moose;
use MTF::MooseCompatibleException qw(:all);

extends 'NACL::MTask::DataSet';

use Data::Dumper;

use data_mobility::ondemand::lib::MountManagerSingleton;
use NACL::C::VolumeQtree;
use NACL::Exceptions::UnexpectedState;
use NACL::GeneralUtils qw(random_name_generator);
use NACL::MTask::Data;


=head1 NAME

NACL::MTask::DataSet::QtreeBasic

=head1 SYNOPSIS

    my $qtree_basic_dataset = NACL::MTask::DataSet::QtreeBasic->create(
        volume => $volume,
        clients => [ @clients ],
    );

    $qtree_basic_dataset = NACL::MTask::DataSet::QtreeBasic->find(
        volume => $volume,
        clients => [ @clients ],
    );

    $qtree_basic_dataset->validate();
    $qtree_basic_dataset->purge();

=head1 DESCRIPTION

Please read L<NACL::MTask::DataSet|lib-NACL-MTask-DataSet-pm> to understand the
general behavior and capabilities of DataSet libraries.

The QtreeBasic DataSet creates a few qtrees with files contained inside of them.
At least one qtree of each security-style will be created.

=head1 ATTRIBUTES

All attributes available from NACL::MTask::DataSet are available on this derived
class.

=head2 mixed_qtree

A L<NACL::C::VolumeQtree|lib-NACL-C-VolumeQtree-pm> instance of a mixed security
style qtree with files inside of it.

=head2 unix_qtree

A L<NACL::C::VolumeQtree|lib-NACL-C-VolumeQtree-pm> instance of a unix security
style qtree with files inside of it.

=head2 ntfs_qtree

A L<NACL::C::VolumeQtree|lib-NACL-C-VolumeQtree-pm> instance of a ntfs security
style qtree with files inside of it.

=cut

has 'mixed_qtree' => (
    is => 'rw',
    isa => 'Object',
);

has 'unix_qtree' => (
    is => 'rw',
    isa => 'Object',
);

has 'ntfs_qtree' => (
    is => 'rw',
    isa => 'Object',
);

has 'security_styles' => (
    is => 'ro',
    isa => 'ArrayRef',
    default => sub { return [qw(mixed unix ntfs)]; },
);


=head1 METHODS

Please read L<NACL::MTask::DataSet|lib-NACL-MTask-DataSet-pm> to understand the
general behavior and capabilities of DataSet libraries. Any methods defined on
C<NACL::MTask::DataSet> are applicable to this DataSet.

=head2 validate

The validation for Qtrees makes sure that state can be called on the component
instance and the Qtree is still of the expected type.

Throws C<NACL::Exceptions::UnexpectedState> if the validation fails.

=head2 purge

This library defines a custom purge method that uses qtree delete to very
quickly eliminate the DataSet from disk. All Qtrees in the DataSet are
concurrently deleted and the generated jobs are waited for at the end. All 
documentation from C<NACL::MTask::DataSet> applies.

=cut

sub create {
    my ($self_or_pkg, %args) = @_;

    my $self = $self_or_pkg->_new_if_required(%args);

    foreach my $sec_style (@{$self->security_styles()}) {
        my $qtree = NACL::C::VolumeQtree->create(
            command_interface => $self->volume()->command_interface(),
            qtree => $self->_qtree_name($sec_style),
            vserver => $self->volume()->vserver(),
            volume => $self->volume()->volume(),
            'security-style' => $sec_style,
        );
        # Replace with $file = $qtree->cast_to_file(); when burt795637 is fixed
        my $file = NACL::C::VolumeFile->new(
            command_interface => $qtree->command_interface(),
            vserver => $qtree->vserver(),
            path => $qtree->qtree_path(),
        );
        $file->set_permission(permission => '777');

        my $method_name = $sec_style."_qtree";
        $self->$method_name($qtree);
        $self->_register_data_directory_name($qtree->qtree());

        # TODO file burt about being unable to get into NTFS sec-style qtree
        # from a windows client by default
        if ($sec_style ne 'ntfs') {
            if (!$self->fast_mode()) {
                $self->_populate_files_in_qtree(qtree => $qtree);
            }
        } else {
            $self->volume()->file_create(
                filename => $qtree->qtree()."/".random_name_generator(),
                size => 4096,
                filecount => 10,
                determine_appropriate_nodescope_node => 1,
            );
        }
    }

    $self->_write_manifest_file();

    $self->_fixup_permissions();

    return $self;
}

# This thaw method doesn't use anything from the bundle, but it does use the
# instance via _qtree_name. It will instantiate component instances for the
# qtrees that are made accessible via the public interface. No freeze method is
# necessary, all required information can be obtained from the base class'
# thawed attributes.
sub thaw {
    my ($self, $thaw_manifest) = @_;

    foreach my $sec_style (@{$self->security_styles()}) {
        my $method_name = $sec_style."_qtree";
        $self->$method_name(
            NACL::C::VolumeQtree->new(
                command_interface => $self->volume()->command_interface(),
                qtree => $self->_qtree_name($sec_style),
                vserver => $self->volume()->vserver(),
                volume => $self->volume()->volume(),
            )
        );
    }
    return $self;
}

sub purge {
    my ($self) = @_;

    my @delete_jobs;

    foreach my $sec_style (@{$self->security_styles()}) {
        my $method_name = $sec_style."_qtree";
        my $qtree_delete_job;

        # We will use CLI so we get background jobs
        $self->$method_name()->delete(
            force => 'true',
            job_component => \$qtree_delete_job,
            apiset_must => { interface => 'CLI' },
        );
        if ($qtree_delete_job) {
            push @delete_jobs, $qtree_delete_job;
        }
    }

    foreach my $job (@delete_jobs) {
        $job->wait_on_job(
            polling_interval => 1,
            'method-timeout' => 60*10,
        );
    }

    $self->_purge_manifest_file();
    return $self;
}

sub validate {
    my ($self) = @_;

    foreach my $sec_style (@{$self->security_styles()}) {
        my $method_name = $sec_style."_qtree";
        my $qtree_state = $self->$method_name()->state();
        if ($sec_style ne $qtree_state->security_style()) {
            NACL::Exceptions::UnexpectedState->throw("Expected qtree: ".
                $self->$method_name()->qtree()." to have security style of ".
                "$sec_style, but the qtree was style ".
                $qtree_state->security_style()
            );
        }
    }
    return $self;
}

# Function: _populate_files_in_qtree
#   Populate some files in the qtree from a client.
# Arguments:
#   qtree - NACL::C::VolumeQtree of the qtree to populate
# Returns:
#   Object instance of this class
sub _populate_files_in_qtree {
    my ($self, %args) = @_;
    my $qtree = $args{qtree};

    my $mm = data_mobility::ondemand::lib::MountManagerSingleton->new();
    my $unix_client = $self->_get_unix_client();
    if ($unix_client) {
        my $mount = $mm->get_mount(
            volume => $self->volume(),
            client => $unix_client,
        );
        NACL::MTask::Data->populate(
            path => $mount->mount_point().'/'.$qtree->qtree(),
            command_interface => $unix_client,
            $self->_populate_args(),
            dbfile => 'db_unix',
        );
    }
    my $windows_client = $self->_get_windows_client();
    if ($windows_client) {
        my $mount = $mm->get_mount(
            volume => $self->volume(),
            client => $windows_client,
        );
        NACL::MTask::Data->populate(
            path => $mount->mount_point().'/'.$qtree->qtree(),
            command_interface => $windows_client,
            $self->_populate_args(),
            stream_avg => 3,
            stream_dev => 1,
            file_acl_num => 0,
            link_num => 0,
            dbfile => 'db_windows',
        );
    }
    return $self;
}

sub _qtree_name {
    my ($self, $sec_style) = @_;

    return $sec_style.'_'.$self->instance();
}

sub _populate_args {
    my ($self) = @_;

    return (
        file_avg => 10,
        file_dev => 0,
        dir_avg => 1,
        dir_dev => 0,
        depth_avg => 1,
        depth_dev => 0,
        file_size_avg => 128,
        file_size_dev => 0,
    );
}

=head1 REGRESSION TESTS

Regression tests for this library are located at
lib/NACL/UnitTest/MTask/DataSetQtreeBasic.thpl

=head1 SEE ALSO

Please read L<NACL::MTask::DataSet|lib-NACL-MTask-DataSet-pm> to understand the
general behavior and capabilities of DataSet libraries.

Additional background on the DataSet framework is available at
https://wikid.netapp.com/w/NACL/Tasks/DataSet

=cut

1;
