#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::MTask::DataSet::VBNZERO;

use Moose;
use MTF::MooseCompatibleException qw(:all);

extends 'NACL::MTask::DataSet';

use Data::Dumper;

use NACL::APISet::Exceptions::ResponseException;
use NACL::CS::VolumeEfficiency;
use NACL::STask::VolumeFile;
use NACL::Exceptions::UnexpectedState;
use NACL::Exceptions::Timeout;

=head1 NAME

NACL::MTask::DataSet::VBNZERO

=head1 SYNOPSIS

    my $vbnzero_dataset = NACL::MTask::DataSet::VBNZERO->create(
        volume => $volume,
        clients => [ @clients ],
    );

    $vbnzero_dataset = NACL::MTask::DataSet::VBNZERO->find(
        volume => $volume,
        clients => [ @clients ],
    );

    $vbnzero_dataset->validate();
    $vbnzero_dataset->purge();

=head1 DESCRIPTION

Please read L<NACL::MTask::DataSet|lib-NACL-MTask-DataSet-pm> to understand the
general behavior and capabilities of DataSet libraries.

The VBNZERO DataSet contains files with VBN_ZERO.

=head1 ATTRIBUTES

All attributes available from NACL::MTask::DataSet are available on this derived
class.

=cut

=head1 METHODS

Please read L<NACL::MTask::DataSet|lib-NACL-MTask-DataSet-pm> to understand the
general behavior and capabilities of DataSet libraries. Any methods defined on
C<NACL::MTask::DataSet> are applicable to this DataSet.

=head2 create

This method expects that the volume will already have efficiency turned on and
will throw C<NACL::Exceptions::UnexpectedState> if it does not.

=head2 file_with_vbn_zero

Returns a L<NACL::STask::VolumeFile|lib-NACL-C-VolumeFile-pm> instance of a file 
that has VBN_ZERO

=head2 validate

Validate will ensure that the file has VBN_ZERO present in it.

Throws C<NACL::Exceptions::UnexpectedState> if the validation fails.

=head2 validate_indir_has_zero

   $dataset->validate_indir_has_zero($file);

Ensure that the passed NACL::STask::VolumeFile instance has VBN_ZERO in the buffer
tree.

Throws C<NACL::Exceptions::UnexpectedState> if no VBN_ZERO is found in the
buffer tree.



=cut

sub create {
    my ($self_or_pkg, %args) = @_;
    my $self = $self_or_pkg->_new_if_required(%args);

    my $dataset_directory = $self->_create_dataset_directory();

    my $efficiency = $self->volume()->efficiency_find(allow_empty => 1);
    if (!$efficiency) {
        NACL::Exceptions::UnexpectedState->throw(
            "Volume efficiency must be enabled to use VBN_ZERO",
            monitoring_fields => [],
        );
    }

    # This gets a NACL component for a file that doesn't yet exist. It will be
    # created soon.
    my $file = $self->file_with_vbn_zero();

    my $zero_file = NACL::STask::VolumeFile->create(
        command_interface => $file->command_interface(),
        vserver => $file->vserver(),
        path => $file->path(),
        size => '1m',
        filecount => 1,
        'deduplicate-zero-data' => 1,
    );

    $self->_write_manifest_file();

    $self->_fixup_permissions();

    return $self;
}

sub _wafl_sync {
    my ($self) = @_;

    my $zapi = $self->volume()->command_interface()->apiset(
        category => 'Node',
        interface => 'ZAPI',
        set => 'CMode',
    );
    
    my $parsed = $zapi->wafl_sync(
        volumes => [ $self->volume()->volume() ],
        'vserver-override-name' => $self->volume()->vserver(),
    )->get_parsed_output();

    my $handle = $parsed->[0]->{handles}->[0]->{'wafl-sync-handle'}->[0];

    my $status;
    my $end_time = time() + 600;
    do {
        if (time() > $end_time) {
            NACL::Exceptions::Timeout->throw(
                "Requested a CP, but it did not complete within 600 seconds"
            );
        }
        $parsed = $zapi->wafl_get_sync_status(
            'sync-handle' => $handle,
            'vserver-override-name' => $self->volume()->vserver(),
        )->get_parsed_output();
        $status = $parsed->[0]->{status};
        sleep(1);
        # We will accept either success or fail for making the CP. This is
        # really just to make sure the buftree has a chance to assemble.
    } while ($status =~ /succeeded|failed/);
}

sub validate_indir_has_zero {
    my ($self, $file) = @_;
    
    try {
        $self->_validate_indir_has_zero($file);
    } catch NACL::Exceptions::UnexpectedState with_handler {
        # It is possible that the VBN_ZERO hasn't made it to the buftree yet
        # because the cleaner hasn't run. If the VBN_ZERO check fails, wait for
        # CP and try again.
        $self->_wafl_sync();
        $self->_validate_indir_has_zero($file);
    };
}

sub _validate_file_with_vbn_zero {
    my ($self, $file) = @_;

    $self->validate_indir_has_zero($file);
    $self->_validate_unique_bytes_implies_vbn_zero($file);
}

sub _generate_wex_path {
    my ($self, $file) = @_;
    my $path;
    my $filepath = $file->path();

    # Remove the components associated with /vol/volumename
    my @file_components = split '/', $filepath;

    # Remove empty component
    shift @file_components;

    # Remove vol component
    shift @file_components;

    # Remove volumename component
    shift @file_components;
    $filepath = join '/', @file_components;
    if ( $filepath !~ /^\// ) {
        $filepath = "/$filepath";
    }
    my $volume = $file->get_containing_volume_obj();
    my $volume_name = $volume->volume();
    my @path_after_snap = split '\.snapshot/', $filepath;

    # If pathname includes a snapshot, the volume name must
    # be followed by "~<logical_snap_id>"
    if ( scalar(@path_after_snap) > 1 ) {
        my @path_elements = split '/', $path_after_snap[1];

        # Snapshot name should be the first element in the list
        my $snap_name = shift(@path_elements);

        my $snap_state = NACL::CS::VolumeSnapshot->fetch(
            command_interface => $volume->command_interface(),
            filter            => {
                snapshot => $snap_name,
                volume   => $volume_name,
                vserver  => $volume->vserver(),
            },
            # logical_snap_id is not available via ZAPI
            apiset_must => { interface => 'CLI' },
        );

        my $snap_id = $snap_state->logical_snap_id();
        $path = "$volume_name~$snap_id./" . join( '/', @path_elements );

    } else {
        $path = $volume_name . ".$filepath";
    }

    return $path;
}

sub _generate_buftree_info { 
    my ($self, $file) = @_;
    my $volume = $file->get_containing_volume_obj();
    my $path   = $self->_generate_wex_path($file);
    my $apiset = $volume->command_interface()->apiset(
        interface => 'CLI',
        category  => 'Node',
        set       => 'CMode',
    );

    my $response = $apiset->volume_explore(
        format => 'indir',
        scope  => $path,
        # indir is extremely verbose
        'connectrec-log_level' => 'trace',
    );
    return $response->get_parsed_output();
}

sub _validate_indir_has_zero {
    my ($self, $file) = @_;

    my $buftree = $self->_generate_buftree_info($file);

    foreach my $entry (@$buftree) {
        if ($entry->{vvbn} =~ /zero/i || $entry->{pvbn} =~ /zero/i) {
            return 1;
        }
    }
    NACL::Exceptions::UnexpectedState->throw("Expected to find VBN_ZERO ".
        "in the buffer tree for file ".$file->path()." but wex indir ".
        "did not show any.",
        monitoring_fields => [],
    );
}

sub _validate_unique_bytes_implies_vbn_zero {
    my ($self, $file) = @_;

    my $zapi = $file->command_interface()->apiset(
        category => 'Node',
        interface => 'ZAPI',
        set => 'CMode',
    );
    
    my $parsed = $zapi->file_usage_get(
        path => $file->path(),
        'vserver-override-name' => $file->vserver(),
    )->get_parsed_output();

    my $record = $parsed->[0];

    # A file with only zeros written to the file will be full of VBN_ZERO and
    # possibly a partial block that has EOF in the middle of the block, and thus
    # won't be VBN_ZERO
    if ($record->{'unique-bytes'} == 8192 ||
        $record->{'unique-bytes'} == 4096) {
        # This is okay
    } else {
        NACL::Exceptions::UnexpectedState->throw(
            "The file ".$file->path()." has ".$record->{'unique-bytes'}.
            " bytes, expected 4096 or 8192 for VBNZERO DataSet",
            monitoring_fields => [],
        );
    }
}

sub file_with_vbn_zero {
    my ($self) = @_;
    my $dataset_directory = $self->_get_dataset_directories();

    return NACL::STask::VolumeFile->new(
        command_interface => $dataset_directory->command_interface(),
        vserver => $dataset_directory->vserver(),
        path => $dataset_directory->path()."/".$self->_file_name(),
    );
}

sub _file_name {
    return "file_with_vbn_zero";
}


sub validate {
    my ($self) = @_;

    $self->_validate_file_with_vbn_zero($self->file_with_vbn_zero());

    return $self;
}

=head1 REGRESSION TESTS

Regression tests for this library are located at
lib/NACL/UnitTest/MTask/DataSetVBNZERO.thpl

=head1 SEE ALSO

Please read L<NACL::MTask::DataSet|lib-NACL-MTask-DataSet-pm> to understand the
general behavior and capabilities of DataSet libraries.

Additional background on the DataSet framework is available at
https://wikid.netapp.com/w/NACL/Tasks/DataSet

=cut



1;
