## @summary Library routines for basic file I/O on the local client
##
## @description
##  This module contains library routines for doing basic I/O to
##  files and includes functions to read, write, remove, getattr
##  on files and named attributes (streams). All functions use the
##  same interface for both unix and Windows clients.
##  Also, all functions accept a 'user' option to specify the user
##  to perform the operation on the file/stream.
##
## @usage
##  use NACL::MTask::DataUtils::FileManip;
##
## @status	review
##
## @author	dl-nacl-dev@netapp.com
## @notes	Ported from lib/FileManip.pm
## @notes
##  The stream subroutines are currently only supported on Solaris 10
##  and Windows unless otherwise noted.

package NACL::MTask::DataUtils::FileManip;


use strict;
use warnings;
use base qw(Exporter);

use Fcntl;
use File::Find;
use File::Basename qw(fileparse);
use Math::BigInt;
use Carp qw(croak);
use Cwd qw(getcwd);
use Storable qw(thaw);
use Params::Validate qw(validate SCALAR BOOLEAN OBJECT ARRAYREF);

# Check if library is running in NATE environment or standalone
eval { require Tharn };
my $_use_nate = $@ ? 0 : 1;
my $_hosttype;

if ( $_use_nate ) {
    # Die with a script error when an
    # input parameter fails validation
    Params::Validate::validation_options(
        on_fail => sub { my @msgs = @_;
                         chomp(@msgs);
                         Carp::confess(Tharn::script_result(@msgs)) }
    );
} else {
    if ( $^O eq 'MSWin32' ) {
        $_hosttype = 'windows';
    } else {
        $_hosttype = "unix-$^O";
    }
}

our @EXPORT_OK = ( qw( link_file setattr_file write_stream write_sparse vol_free vol_info 
                        read_stream read_file getattr_file setattr_stream remove_stream
                        remove_file
                  ) );


#---------------------------------------------------------------------
# Function prototypes
#---------------------------------------------------------------------
sub do_call($%);
sub write_stream(%);
sub _write_stream(%);
sub setattr_file(%);
sub _setattr_file(%);
sub write_sparse(%);
sub _write_sparse(%);
sub link_file(%);
sub _link_file(%);
sub vol_free(%);
sub _vol_free(%);
sub vol_info(%);
sub _vol_info(%);
sub _stat_stream(%);
sub _win32api_call($@);


#---------------------------------------------------------------------
# Package local constants
#---------------------------------------------------------------------
my %common_parameters = (
    # The user parameter is optional
    user     => { type => SCALAR,         optional => 1 },

    # The client parameter could be either a hostspec, Hostrec object
    # or a Connectrec object. Defaults to 'localhost'.
    client   => { type => SCALAR|OBJECT,   optional => 1,   default => 'localhost' },
);

my @common_params = qw( user  client );

#---------------------------------------------------------------------
# Package local variables
#---------------------------------------------------------------------
my %sobj_h;
my %win32api;
my $use_w32api = 1;
my $log_index  = 0;
our $main_pid  = 0;
my $VLOFFSET   = 0xFFFFFFFF;

#---------------------------------------------------------------------
# Internal function.
# Run the specified function locally or remotely.
# When running remotely, it starts an rpc server on
# the remote client if it is not running. Multiple calls to the
# same client and as the same user are served by the rpc server
# with no need to subtest the function again.
#---------------------------------------------------------------------
sub do_call($%) {
    
    my ($function, %opts) = @_;
    my ($out, @results, $err);

    if ( $_use_nate ) {
        
        # Get the hostrec object from the client option
        if ( UNIVERSAL::isa($opts{client}, "Connectrec") ) {
            # Get hostrec object from a connectrec object
            $opts{client} = $opts{client}->hostp();
        } elsif ( ! UNIVERSAL::isa($opts{client}, "Hostrec") ) {
            # Get hostrec object from a hostspec
            $opts{client} = Tharn::host($opts{client});
        }
    } else {
        
        $opts{client} = {
            hostname => 'localhost',
            hosttype => $_hosttype,
        };
    }

    # Check for correct use of the 'user' option
    if ( defined($opts{user}) and $opts{client}{hosttype} =~ /^windows/ ) {
        
        croak "_do_call: Option 'user' is not supported on Windows";
    }

    # Check if function need to run locally or remotely
    if ( $opts{client}{hostname} eq 'localhost' and !$opts{user} ) {
        
        if ( $opts{client}{hosttype} =~ /^windows/ ) {
            require Win32API::File;
        }
        # Run function locally
        return &{$function}(%opts);
    }
}

#---------------------------------------------------------------------
## @name ::write_stream
## @description
##  Write a stream (named attribute) on file from a client
##  as the user (or any other user if specified).
##  Arguments are passed in a "key => value" style.
##
## @synopsis
##  $op = write_stream( %options );
##
## @arg stream
##  Required: Named attribute to write.
##
## @arg file
##  Required: File where named attribute is located.
##
## @arg contents
##  Optional: Contents of stream.
##
## @arg sstat
##  Optional: Get stream attributes after writing (default is 0).
##  Only supported on Solaris 10.
##
## @arg offset
##  Optional: Offset where to begin writing (default is 0).
##  If value is non-zero, the stream must exists.
##  Only supported on Windows.
##
## @arg append
##  Optional: Append the contents to the stream (default is 0).
##
## @arg client
##  Optional: Hostspec, Hostrec or Connectrec of client.
##  The connection type of the Connectrec object is not used
##  since subtest uses its own connection type
##  (default is localhost).
##
## @arg user
##  Optional: Perform operation as this user.
##
## @return
##  List of attributes for stream if 'sstat' is true,
##  or 1 if 'sstat' is false.
#---------------------------------------------------------------------
sub write_stream(%) {
    my (%opts) = validate (
        @_,
        {
            stream   => { type => SCALAR },
            file     => { type => SCALAR },
            contents => { type => SCALAR, optional => 1 },
            sstat    => { type => BOOLEAN, default => 0 },
            offset   => { type => SCALAR, optional => 1 },
            append   => { type => BOOLEAN, default => 0 },
            %common_parameters
        }
    );
    do_call(\&_write_stream, %opts);
}

#---------------------------------------------------------------------
# Internal function to write a stream.
#---------------------------------------------------------------------
sub _write_stream(%) {
    my %opts = @_;
    my ($open_stream, $err);
    my $file   = $opts{file};
    my $stream = $opts{stream};
    my $append = $opts{append} ? '>>' : '>';
    my $stream_name = $opts{file} . ':' . $opts{stream};

    # Get hosttype of client
    my $hosttype = $opts{client}{hosttype};

    # Check for valid flags
    if ( $opts{append} and defined($opts{offset}) ) {
        croak "Both append and offset cannot be set";
    }
    if ( $opts{sstat} and $hosttype ne 'unix-solaris' ) {
        croak "Option 'sstat' is only supported on Solaris";
    }

    # Streams or named attributes are only supported
    # on solaris 10 and windows
    if ( $hosttype eq 'unix-solaris' ) {
        if ( defined($opts{offset}) ) {
            croak "Option 'offset' is only supported on Windows";
        }

        # Command to write a named attribute
        $open_stream = qq(| runat $file "cat $append $stream");

        # Open a stream for writing
        open(FH, $open_stream)
          or croak "Could not open stream for writing $stream_name: $!";

        # Catch any errors to properly close the stream
        eval {
            if ( defined($opts{contents}) ) {
                print FH $opts{contents}
                  or croak "Could not write to stream $stream_name: $!";
            }
        };
        $err = $@;

        # Close the pipe
        close(FH) or croak "Could not create stream $stream_name: $!";
        croak $err if $err;
    } elsif ( $hosttype =~ /^windows/ ) {
        # Using Win32API to create a stream since
        # perl open or sysopen cannot open a
        # stream on the current directory '.'
        my $count;
        my $offset = 0;
        my $nbytes = length($opts{contents});
        my $foffset = $opts{offset};
        # Open flags wke: write keep existing
        #            wtc: write truncate create
        my $access = ($opts{append} or defined($opts{offset})) ? 'wke' : 'wtc';
        $file = '' if $file eq '.';
        $open_stream = qq($file:$stream);

        # Open stream for writing
        my $hObject = Win32API::File::createFile($open_stream, $access)
          or croak "Could not open stream for writing $stream_name: $^E";

        # Catch any errors to properly close the stream
        eval {
            # Request for append or set to specific offset
            my @seek_args = $opts{append} ? (0, 2) : $foffset ?
                            ($foffset > $VLOFFSET ?
                             Math::BigInt->new($foffset) : $foffset, 0) : ();
            if ( @seek_args ) {
                # Position file pointer
                Win32API::File::setFilePointer($hObject, @seek_args )
                  or croak "Could not position filer pointer $stream_name: $^E";
            }

            while ( $offset < $nbytes ) {
                # Try to write whole buffer
                Win32API::File::WriteFile($hObject, $opts{contents},
                                          0, $count, [])
                  or croak "Could not write to stream $stream_name: $^E";
                $offset += $count;
                $opts{contents} = substr($opts{contents}, $count);
            }
        };
        $err = $@;

        # Close the stream
        Win32API::File::CloseHandle($hObject);
        croak $err if $err;
    } else {
        croak "Hosttype '$hosttype' is not supported";
    }

    # Get stream attributes
    if ( $opts{sstat} ) {
        my @attr = _stat_stream(stream => $stream,
                                file   => $file,
                                client => $opts{client});
        return @attr;
    } else {
        # Only write is requested
        return 1;
    }
}

#---------------------------------------------------------------------
## @name ::setattr_file
## @description
##  Set the attributes of a file from a client
##  as the user (or any other user if specified).
##  Arguments are passed in a "key => value" style.
##
## @synopsis
##  $op = setattr_file( %options );
##
## @arg file
##  Required: File to set attributes for.
##
## @arg size
##  Optional: Set the size (truncate) of file.
##
## @arg uid
##  Optional: Change ownership of file to this uid.
##  This option implies <<user=>'root'>>.
##  Only supported on unix.
##
## @arg gid
##  Optional: Change group of file to this gid.
##  Only supported on unix.
##
## @arg mode
##  Optional: Permission mode to set for file.
##  Octal number or string, e.g., 0644, '644'.
##  Only supported on unix.
##
## @arg compress
##  Optional: Set or clear the compress attribute.
##  Only supported on windows.
##
## @arg encrypt
##  Optional: Set or clear the encrypt attribute.
##  Only supported on windows.
##
## @arg archive
##  Optional: Set or clear the archive attribute.
##  Only supported on windows.
##
## @arg hidden
##  Optional: Set or clear the hidden attribute.
##  Only supported on windows.
##
## @arg client
##  Optional: Hostspec, Hostrec or Connectrec of client.
##  The connection type of the Connectrec object is not used
##  since subtest uses its own connection type
##  (default is localhost).
##
## @arg user
##  Optional: Perform operation as this user. (Unix clients only)
##
## @return
##  1 if succeeds.
##
## @notes
##  At least one of the following options must be specified:
##  size, uid, gid, mode, encrypt, archive or hidden.
#---------------------------------------------------------------------
sub setattr_file(%) {
    my (%opts) = validate (
        @_,
        {
            file       => { type => SCALAR },
            size       => { type => SCALAR,    optional => 1, regex => qr/^\d+$/ },
            uid        => { type => SCALAR,    optional => 1, regex => qr/^\d+$/ },
            gid        => { type => SCALAR,    optional => 1, regex => qr/^\d+$/ },
            mode       => { type => SCALAR,    optional => 1, regex => qr/^\d+$/ },
            compress   => { type => BOOLEAN,   optional => 1 },
            encrypt    => { type => BOOLEAN,   optional => 1 },
            archive    => { type => BOOLEAN,   optional => 1 },
            hidden     => { type => BOOLEAN,   optional => 1 },
            %common_parameters
        }
    );
    # Option uid implies root
    $opts{user} = 'root' if defined($opts{uid});

    do_call(\&_setattr_file, %opts);
}

#---------------------------------------------------------------------
# Internal function to set the attributes of a file.
#---------------------------------------------------------------------
sub _setattr_file(%) {
    my %opts = @_;
    my $file = $opts{file};
    my $ugmode = ( defined($opts{uid}) or defined($opts{gid})
                   or defined($opts{mode}) );
    my $no_win_opts = ($ugmode || $opts{user});
    my $wattrs = ( defined($opts{encrypt}) or defined($opts{archive})
                   or defined($opts{hidden}) or defined($opts{compress}) );

    # Windows does not support chown and chmod, or 'user' option.
    my $hosttype = $opts{client}{hosttype};
    if ( $hosttype =~ /^windows/ and $no_win_opts ) {
        croak "_setattr_file: Options 'user', 'uid', 'gid' or 'mode' are not supported on windows";
    }

    # Only windows supports the encrypt option
    if ( $hosttype !~ /^windows/ and $wattrs ) {
        croak "_setattr_file: Options 'compress', 'encrypt', 'archive' or 'hidden'" .
              "are only supported on windows";
    }

    # Validate that at least one option is specified
    if ( ! ( defined($opts{size}) or $ugmode or $wattrs ) ) {
        croak "_setattr_file: At least one of these options must be specified:" .
              "size, uid, gid, mode, encrypt, archive or hidden";
    }

    # Check conflicting options
    if ( defined($opts{uid}) and $opts{user} and $opts{user} ne 'root' ) {
        croak "_setattr_file: Options 'uid' and 'user' are mutually exclusive " .
              "since specifying option 'uid' requires " .
              "running as root: user=>'root'";
    }

    # Set the size (truncate) of file
    if ( defined($opts{size}) ) {
        truncate $file, $opts{size}
          or croak "_setattr_file: Could not truncate file $file: $!";
    }

    # Change the mode (permissions) of file
    if ( defined($opts{mode}) ) {
        if ( (~$opts{mode} & $opts{mode}) ne '0' ) {
            # Mode was passed as an octal string,
            # convert it to an octal number.
            $opts{mode} = oct($opts{mode});
        }
        chmod($opts{mode}, $file)
          or croak "_setattr_file: Could not change mode of file $file: $!";
    }

    # Change owner and group on file
    if ( defined($opts{uid}) or defined($opts{gid}) ) {
        # Do not change it if not specified
        my $euid = defined($opts{uid}) ? $opts{uid} : -1;
        my $egid = defined($opts{gid}) ? $opts{gid} : -1;
        # Chown command
        chown($euid, $egid, $file)
          or croak "_setattr_file: Could not change ownership of file $file: $!";
    }

    # Set or clear the encrypt attribute
    if ( defined($opts{encrypt}) ) {
        if ( $opts{encrypt} ) {
            # Set the encrypt attribute on the file
            _win32api_call('EncryptFile', $opts{file});
        } else {
            # Clear the encrypt attribute on the file
            _win32api_call('DecryptFile', $opts{file}, 0);
        }
    }

    # Set or clear the archive or hidden attributes
    if ( defined($opts{archive}) or defined($opts{hidden}) ) {
        # Get file attributes
        $wattrs = _win32api_call('GetFileAttributes', $opts{file});

        if ( defined($opts{archive}) ) {
            # Set or clear the archive bit
            $wattrs = $opts{archive} ?
                      $wattrs | Win32API::File::FILE_ATTRIBUTE_ARCHIVE() :
                      $wattrs & ~Win32API::File::FILE_ATTRIBUTE_ARCHIVE();
        }

        if ( defined($opts{hidden}) ) {
            # Set or clear the hidden bit
            $wattrs = $opts{hidden} ?
                      $wattrs | Win32API::File::FILE_ATTRIBUTE_HIDDEN() :
                      $wattrs & ~Win32API::File::FILE_ATTRIBUTE_HIDDEN();
        }

        # Set file attributes
        _win32api_call('SetFileAttributes', $opts{file}, $wattrs);
    }

    # Set or clear the compress attribute
    if ( defined($opts{compress}) ) {
        # Get file handle of file or directory
        # The FILE_FLAG_BACKUP_SEMANTICS is required to get
        # a handle for a directory
        my $hObject = Win32API::File::createFile($file, 'rwke',
          { Flags => Win32API::File::FILE_FLAG_BACKUP_SEMANTICS() })
          or croak "_setattr_file: Could not get file handle of $file: $^E";
        # Set file to be compressed
        my @ret = (pack('S', $opts{compress}?1:0), 0, [], [], [], []);
        Win32API::File::DeviceIoControl($hObject, 0x9C040, @ret)
          or croak "_setattr_file: Could not set compress attribute on file $file: $^E";
        Win32API::File::CloseHandle($hObject);
    }

    return 1;
}

#---------------------------------------------------------------------
## @name ::write_sparse
## @description
##  Write a sparse (holey) file from a client
##  as the user (or any other user if specified).
##  Arguments are passed in a "key => value" style.
##
## @synopsis
##  $op = write_sparse( %options );
##
## @arg file
##  Required: File to be created.
##
## @arg data
##  Optional: Reference to an array of hashes having the following:
##  { offset => Offset of data, data => Actual data }.
##
## @arg client
##  Optional: Hostspec, Hostrec or Connectrec of client.
##  The connection type of the Connectrec object is not used
##  since subtest uses its own connection type
##  (default is localhost).
##
## @arg user
##  Optional: Perform operation as this user.
##
## @return
##  1 if succeeds.
#---------------------------------------------------------------------
sub write_sparse(%) {
    my (%opts) = validate (
        @_,
        {
            file => { type => SCALAR },
            data => { type => ARRAYREF },
            %common_parameters
        }
    );
    do_call(\&_write_sparse, %opts);
}

#---------------------------------------------------------------------
# Internal function to write a sparse file.
#---------------------------------------------------------------------
sub _write_sparse(%) {
    my %opts = @_;
    my ($item, $err, $foffset);
    my $file = $opts{file};
    my $data = $opts{data};

    # Get hosttype of client
    my $hosttype = $opts{client}{hosttype};

    if ( $hosttype =~ /^unix/ ) {
        # Open file
        sysopen(FH, $file, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY)
          or croak "Could not open file for writing $file: $!";

        # Catch any errors to properly close the file
        eval {
            # Write the data to the file
            foreach $item ( @$data ) {
                # Set file pointer to correct offset for this data block
                seek(FH, $item->{offset}, 0)
                  or croak "Could not seek on file $file: $!";
                print FH $item->{data}
                  or croak "Could not write to file $file: $!";
            }
        };
        $err = $@;

        # Close the file
        close(FH);
        croak $err if $err;
    } elsif ( $hosttype =~ /^windows/ ) {
        my ($zinfo, $zlen);
        my $offset = 0;
        my ($off_l, $off_h, $foff_l, $foff_h);
        my @ret = ([], [], [], [], [], []);

        # Open file for writing
        my $hObject = Win32API::File::createFile($file, 'wtc')
          or croak "Could not open file for writing $file: $^E";
        # Catch any errors to properly close the file
        eval {
            # Set file to be sparse
            Win32API::File::DeviceIoControl($hObject, 0x900C4, @ret)
              or croak "Could not set sparse attribute on file $file: $^E";

            # Set all zero regions and write all data
            foreach $item ( @$data ) {
                $foffset = $item->{offset};
                $foffset = Math::BigInt->new($foffset) if $foffset > $VLOFFSET;
                $zlen = $foffset - $offset;
               # XXX ONTAP does not support NTFS sparse file IOCTLs
               # See Burt 20305
               # if ( $zlen ) {
               #     ($off_h, $foff_h) = (0, 0);
               #     $off_l  = $offset;
               #     $foff_l = $foffset;
               #     # Convert cummulative offset to a BitInt
               #     # if larger than 32 bits
               #     $offset = Math::BigInt->new($offset)
               #       if (!ref $offset && $offset > $VLOFFSET);
               #     if ( ref $offset ) {
               #         # Start of zero region is larger than 32 bits,
               #         # Separate into 32 bit low and high parts
               #         $off_l = ($offset & $VLOFFSET)->numify;
               #         $off_h = ($offset >> 32)->numify;
               #     }
               #     if ( ref $foffset ) {
               #         # End of zero region is larger than 32 bits,
               #         # Separate into 32 bit low and high parts
               #         $foff_l = ($foffset & $VLOFFSET)->numify;
               #         $foff_h = ($foffset >> 32)->numify;
               #     }
               #     # Pack the FILE_ZERO_DATA_INFORMATION structure
               #     $zinfo = pack('LLLL', $off_l, $off_h, $foff_l, $foff_h);
               #     @ret[0,1] = ($zinfo, 16);
               #     # Set zero region
               #     Win32API::File::DeviceIoControl($hObject, 0x980C8, @ret)
               #       or croak "Could not set zero region on $file: $^E";
               # }
                # Move file pointer where actual data is located
                Win32API::File::setFilePointer($hObject, $foffset, 0)
                  or croak "Could not seek on file $file: $^E";
                # Write the actual data
                Win32API::File::WriteFile($hObject, $item->{data}, 0, [], [])
                  or croak "Could not write to file $file: $^E";

                $offset = $foffset + length($item->{data});
            }
        };
        $err = $@;

        # Close the file
        Win32API::File::CloseHandle($hObject);
        croak $err if $err;
    } else {
        croak "Hosttype '$hosttype' is not supported";
    }

    return 1;
}

#---------------------------------------------------------------------
## @name ::link_file
## @description
##  Create a hard/symbolic link to a file from a client as the user
##  (or any other user if specified).
##  Arguments are passed in a "key => value" style.
##
## @synopsis
##  $op = link_file( %options );
##
## @arg file
##  Required: File to be linked to.
##
## @arg flink
##  Required: Hard/symbolic link to create.
##
## @arg symbolic
##  Optional: Boolean, true to create a symbolic link (default is 0).
##  Symbolic links are not supported on Windows.
##
## @arg client
##  Optional: Hostspec, Hostrec or Connectrec of client.
##  The connection type of the Connectrec object is not used
##  since subtest uses its own connection type
##  (default is localhost).
##
## @arg user
##  Optional: Perform operation as this user. (Unix clients only)
##
## @return
##  1 if succeeds.
#---------------------------------------------------------------------
sub link_file(%) {
    my (%opts) = validate (
        @_,
        {
            file     => { type => SCALAR },
            flink    => { type => SCALAR },
            symbolic => { type => BOOLEAN, default => 0 },
            %common_parameters
        }
    );
    do_call(\&_link_file, %opts);
}

#---------------------------------------------------------------------
# Internal function to create a link to a file.
#---------------------------------------------------------------------
sub _link_file(%) {
    my %opts = @_;
    my $file  = $opts{file};
    my $flink = $opts{flink};

    # Windows does not support symbolic links
    my $hosttype = $opts{client}{hosttype};
    if ( $opts{symbolic} and $hosttype =~ /^windows/ ) {
        croak "_link_file: Symbolic links are not supported on Windows";
    }

    # Create hard/symbolic link
    if ( $opts{symbolic} ) {
        symlink $file, $flink
          or croak "_link_file: Could not create symbolic link $flink -> $file: $!";
    } else {
        link $file, $flink
          or croak "_link_file: Could not create hard link $flink -> $file: $!";
    }

    return 1;
}

#---------------------------------------------------------------------
## @name ::vol_info
## @description
##  Get volume information.
##
## @synopsis
##  $op = vol_info( %options );
##
## @arg vol
##  Required: Root directory of volume.
##
## @arg client
##  Optional: Hostspec, Hostrec or Connectrec of client.
##  The connection type of the Connectrec object is not used
##  since subtest uses its own connection type
##  (default is localhost).
##
## @return
##  Hash of volume attributes.
##
## @notes
##  Only supported on Windows.
#---------------------------------------------------------------------
sub vol_info(%) {
    my (%opts) = validate (
        @_,
        {
            vol => { type => SCALAR },
            %common_parameters
        }
    );
    do_call(\&_vol_info, %opts);
}

#---------------------------------------------------------------------
# Internal function to get volume information
#---------------------------------------------------------------------
sub _vol_info(%) {
    my %opts = @_;
    my ($out, $flags, %ret);
    my $nsize    = 256;
    my $volname  = ' ' x $nsize;
    my $serial   = ' ' x 4;
    my $maxlen   = ' ' x 4;
    my $sysflags = ' ' x 4;
    my $sysname  = ' ' x $nsize;

    # Get hosttype of client
    my $hosttype = $opts{client}{hosttype};

    if ( $hosttype !~ /^windows/ ) {
        croak "Hosttype '$hosttype' is not supported";
    }

    # Append '\' to the volume, it is required
    $opts{vol} .= '\\' if $opts{vol} !~ /\\$/;

    # Call the windows api
    $out = _win32api_call(
        'GetVolumeInformation',
        $opts{vol},
        \$volname, $nsize,
        \$serial,
        \$maxlen,
        \$sysflags,
        \$sysname, $nsize,
    );

    croak "Could not get volume information for $opts{vol}" unless $out;

    # Process output strings
    $volname =~ s/\0.*//;
    $sysname =~ s/\0.*//;
    # Convert sysflags to an integer for further processing
    $flags = unpack('I', $sysflags);

    # Create return hash
    %ret = (
        volname  => $volname,
        sysname  => $sysname,
        serial   => unpack('I', $serial),
        maxlen   => unpack('I', $maxlen),
        sysflags => {
            sparse   => ($flags & 0x000040) ? 1 : 0,
            encrypt  => ($flags & 0x020000) ? 1 : 0,
            compress => ($flags & 0x000010) ? 1 : 0,
            acls     => ($flags & 0x000008) ? 1 : 0,
            quotas   => ($flags & 0x000020) ? 1 : 0,
            case_sensitive_search => ($flags & 0x000001) ? 1 : 0,
            case_preserved_names  => ($flags & 0x000002) ? 1 : 0,
        },
        sysflags_raw => $flags,
    );

    return \%ret;
}


#---------------------------------------------------------------------
## @name ::vol_free
## @description
##  Get free space of volume.
##
## @synopsis
##  $op = vol_free( %options );
##
## @arg vol
##  Required: Root directory of volume, e.g., 'C:', 'Z:', etc.
##
## @arg client
##  Optional: Hostspec, Hostrec or Connectrec of client.
##  The connection type of the Connectrec object is not used
##  since subtest uses its own connection type
##  (default is localhost).
##
## @return
##  Hash of information about the amount of free, used and total
##  space available on the volume
##
## @notes
##  Only supported on Windows.
#---------------------------------------------------------------------
sub vol_free(%) {
    my (%opts) = validate (
        @_,
        {
            vol => { type => SCALAR },
            %common_parameters
        }
    );
    do_call(\&_vol_free, %opts);
}

#---------------------------------------------------------------------
# Internal function to get free space of volume
#---------------------------------------------------------------------
sub _vol_free(%) {
    my %opts = @_;
    my ($hObject, $drive, %ret, $out, $err);
    my $free_avail  = ' ' x 8;
    my $total_avail = ' ' x 8;
    my $free_total  = ' ' x 8;
    my $vol = $opts{vol};

    # Get hosttype of client
    my $hosttype = $opts{client}{hosttype};

    if ( $hosttype !~ /^windows/ ) {
        croak "Hosttype '$hosttype' is not supported";
    }

    if ( $vol !~ /^([a-zA-Z]):/ ) {
        croak "Invalid volume name $vol";
    }
    $drive = $1;

    # Call the windows api to get the free space
    $out = _win32api_call(
        'GetDiskFreeSpaceEx',
        "$drive:\\",
        \$free_avail,
        \$total_avail,
        \$free_total,
    );

    croak "Could not get volume free space for $vol" unless $out;

    # Calculate free, used and total space from 64 bit outputs
    %ret = (
        free_avail  => unpack('I', substr($free_avail,  0, 4)) +
                       unpack('I', substr($free_avail,  4, 4)) * 4294967296,
        total_avail => unpack('I', substr($total_avail, 0, 4)) +
                       unpack('I', substr($total_avail, 4, 4)) * 4294967296,
        free        => unpack('I', substr($free_total,  0, 4)) +
                       unpack('I', substr($free_total,  4, 4)) * 4294967296,
    );

    # Set the total to the total space available for the current user
    $ret{total} = $ret{total_avail};
    # Calculate used space
    $ret{used} = $ret{total} - $ret{free};

    return %ret;
}


#---------------------------------------------------------------------
# Internal function to get the attributes of a stream.
#---------------------------------------------------------------------
sub _stat_stream(%) {
    my %opts = @_;
    my $file   = $opts{file};
    my $stream = $opts{stream};

    # Get hosttype of client
    my $hosttype = $opts{client}{hosttype};

    # Streams or named attributes are only
    # supported on solaris 10 for getattr
    if ( $hosttype ne 'unix-solaris' ) {
        croak "Hosttype '$hosttype' is not supported";
    }

    # Command to getattr a named attribute
    my $stat = qq{my \@attr = stat q($stream) or die \$!; } .
               qq{print q(--sstat_ok--) . join q(--), \@attr;};
    # Run command to get the attributes
    my $out = qx(runat $file "perl -e '$stat'" 2>&1);

    # Remove the 'sstat_ok' flag from the output
    # and if it does not exist croak the error
    if ( ! ($out =~ s/--sstat_ok--//) ) {
        croak "Could not get attributes for stream '$stream' " .
              "on file '$file': $out";
    }

    return split(/--/, $out);
}

#---------------------------------------------------------------------
# Internal function to load the specified Win32::API function and run it
#---------------------------------------------------------------------
sub _win32api_call($@) {
    my ($name, @args) = @_;
    my ($arg, $out, $index, @args_ref);

    # Function prototypes
    my %api = (
        EncryptFile       => ['advapi32', 'EncryptFile',       'P',  'N'],
        DecryptFile       => ['advapi32', 'DecryptFile',       'PN', 'N'],
        GetFileAttributes => ['kernel32', 'GetFileAttributes', 'P',  'N'],
        SetFileAttributes => ['kernel32', 'SetFileAttributes', 'PN', 'N'],
        GetDiskFreeSpaceEx   => ['kernel32', 'GetDiskFreeSpaceEx', 'PPPP', 'N'],
        GetVolumeInformation => ['kernel32', 'GetVolumeInformation',
                                 'PPNPPPPN', 'N'],
    );

    # Find which arguments are references
    $index = 0;
    foreach $arg ( @args ) {
        if ( ref $arg ) {
            # Argument is a reference, save it and its position
            push @args_ref, { arg => $arg, idx => $index };

            # Replace argument in list with its contents
            $arg = $$arg;
        }
        $index++;
    }

    if ( $use_w32api ) {
        # Load module only once
        require Win32::API;
        $use_w32api = 0;
    }

    if ( ! defined($win32api{$name}) ) {
        # Load the DLL and import the Windows API function
        $win32api{$name} = Win32::API->new(@{$api{$name}});
        if ( ! defined($win32api{$name}) ) {
            croak "Can not import API $api{$name}: $!";
        }
    }

    # Call the API
    $out = $win32api{$name}->Call(@args) or croak "$name: $^E";

    foreach $arg ( @args_ref ) {
        # Replace contents of reference with new value of argument after
        # calling the API so the calling function can see this change
        ${$arg->{arg}} = $args[$arg->{idx}];
    }

    return $out;
}

sub remove_file(%) {
    my (%opts) = validate (
        @_,
        {
            file            => { type => SCALAR },
            recursive       => { type => BOOLEAN,   default => 0 },
            contents_only   => { type => BOOLEAN,   default => 0 },
            okay_if_not_exists   => { type => BOOLEAN,   optional => 1 },
            %common_parameters
        }
    );
    do_call(\&_remove_file, %opts);
}

#---------------------------------------------------------------------
# Internal function to remove a file.
#---------------------------------------------------------------------
sub _remove_file(%) {
    my %opts = @_;
    my $file = $opts{file};


    # Do not attempt to remove root directory
    if ( $file =~ /^.:\\?\*?$/ or $file =~ m{^/\*?$} ) {
        croak "_remove_file: Not removing root directory";
    }

    # Get rid off multiple '/'s
    $file =~ s{/+}{/};
    # Get rid off trailing '/'
    $file =~ s{/$}{};
    # Do not delete file if it is in the root directory on unix,
    # i.e., contains only one '/'.
    my $nslash = ($file =~ tr{/}{/});
    if ( $file !~ /^.:/ and (($file =~ m{^/} and $nslash == 1)
      or getcwd() eq '/' and $nslash == 0) ) {
        croak "_remove_file: Not removing file located in root directory";
    }

    if (! -e $file) {
        if ($opts{okay_if_not_exists}) {
            return 1;
        } else {
            croak("_remove_file: $file does not exist");
        }
    }

    if ( $opts{recursive} ) {
        # Remove a directory tree.
        # It also accepts wild cards ('*', '?', ...)
        # to get a list of top level files and
        # directories and removes them recursively.

        sub rmelem() {
            return if /^\.$/;
	    ((-d $_ && ! -l $_) ? rmdir : unlink)
              or croak "_remove_file: Could not remove file or directory $_: $!";
        }

	sub make_rmelem_no_topdir(@) {
	    my (@topdirs) = @_;
	    my $topdir = $topdirs[0];
	    my $is_list = scalar(@topdirs) > 1;
	        return sub () {
                    return if (/^\.$/);
                    if ($is_list) {
                         return if (grep( {$_ eq $File::Find::name}  @topdirs));
                    } else {
                         return if ($topdir eq $File::Find::name);
                    }
	            ((-d $_ && ! -l $_) ? rmdir : unlink)
                         or croak "remove_file: Could not remove file or directory $File::Find::name: ".
                                   "$!";
                 }
        }

        my @list = glob($file);
        if ( @list < 2 and not -e $list[0] ) {
            croak "_remove_file: No such file or directory: $file";
        }

        if ($opts{contents_only}) {
            my $rmelem_sub = make_rmelem_no_topdir(@list);
            finddepth({wanted => \&$rmelem_sub, no_chdir => 1}, @list);
        } else {
            finddepth({wanted => \&rmelem, no_chdir => 1}, @list);
            (rmdir or croak "_remove_file: Could not remove directory $_: $!")
               foreach grep {-d} @list;
        }
    } else {
        # Remove a single file
        unlink $file or croak "_remove_file: Could not remove file $file: $!";
    }

    return 1;
}

#---------------------------------------------------------------------
## @name ::remove_stream
## @description
##  Remove the specified stream (named attribute) from a
##  client as the user (or any other user if specified).
##  Arguments are passed in a "key => value" style.
##
## @synopsis
##  $op = remove_stream( %options );
##
## @arg stream
##  Required: Named attribute.
##
## @arg file
##  Required: File where named attribute is located.
##
## @arg client
##  Optional: Hostspec, Hostrec or Connectrec of client.
##  The connection type of the Connectrec object is not used
##  since subtest uses its own connection type
##  (default is localhost).
##
## @arg user
##  Optional: Perform operation as this user.
##
## @return
##  1 if succeeds.
#---------------------------------------------------------------------
sub remove_stream(%) {
    my (%opts) = validate (
        @_,
        {
            stream => { type => SCALAR },
            file   => { type => SCALAR },
            %common_parameters
        }
    );
    do_call(\&_remove_stream, %opts);
}

#---------------------------------------------------------------------
# Internal function to remove a stream.
#---------------------------------------------------------------------
sub _remove_stream(%) {
    my %opts = @_;
    my $out;
    my $file   = $opts{file};
    my $stream = $opts{stream};

    # Get hosttype of client
    my $hosttype = $opts{client}{hosttype};

    if ( $hosttype eq 'unix-solaris' ) {
        # Command to remove a named attribute on solaris
        my $remove = qq{unlink q($stream) or die \$!; } .
                     qq{print q(--sremove_ok--);};
        # Run command to remove the named attribute
        $out = qx(runat $file "perl -e '$remove'" 2>&1);
    } elsif ( $hosttype =~ /^windows/ ) {
        # Command to remove a stream on windows
        $file = '' if $file eq '.';
        $out = Win32API::File::DeleteFile("$file:$stream")
               ? '--sremove_ok--' : $^E;
    } else {
        croak "Hosttype '$hosttype' is not supported";
    }

    # Remove the 'sremove_ok' flag from the output
    # and if it does not exist croak the error
    if ( ! ($out =~ s/--sremove_ok--//) ) {
        croak "Could not remove stream '$stream' on file '$file': $out";
    }

    return 1;
}

#---------------------------------------------------------------------
## @name ::setattr_stream
## @description
##  Set the attributes of a stream (named attribute) from a client
##  as the user (or any other user if specified).
##  Arguments are passed in a "key => value" style.
##
## @synopsis
##  $op = setattr_stream( %options );
##
## @arg stream
##  Required: Named attribute.
##
## @arg file
##  Required: File where named attribute is located.
##
## @arg size
##  Required: Set the size (truncate) of stream.
##
## @arg client
##  Optional: Hostspec, Hostrec or Connectrec of client.
##  The connection type of the Connectrec object is not used
##  since subtest uses its own connection type
##  (default is localhost).
##
## @arg user
##  Optional: Perform operation as this user.
##
## @return
##  1 if succeeds.
#---------------------------------------------------------------------
sub setattr_stream(%) {
    my (%opts) = validate (
        @_,
        {
            stream => { type => SCALAR },
            file   => { type => SCALAR },
            size   => { type => SCALAR, regex => qr/^\d+$/ },
            %common_parameters
        }
    );
    do_call(\&_setattr_stream, %opts);
}

#---------------------------------------------------------------------
# Internal function to set the attributes of a stream.
#---------------------------------------------------------------------
sub _setattr_stream(%) {
    my %opts = @_;
    my $stream = $opts{stream};
    my $file   = $opts{file};
    my $size   = $opts{size};

    # Streams or named attributes are only supported
    # on solaris 10 and windows
    my $hosttype = $opts{client}{hosttype};
    if ( $hosttype eq 'unix-solaris' ) {
        # Command to set attributes on a named attribute
        my $setattr = qq{truncate q($stream), $size or die \$!; } .
                     qq{print q(--ssetattr_ok--);};
        my $out = qx(runat $file "perl -e '$setattr'" 2>&1);
        # Remove the 'ssetattr_ok' flag from the output
        # and if it does not exist croak the error
        if ( ! ($out =~ s/^--ssetattr_ok--//) ) {
            croak "Could not set attributes for stream " .
                  "'$stream' on file '$file': $out";
        }
    } elsif ( $hosttype =~ /^windows/ ) {
        # Command to set attributes on a stream
        $file = '' if $file eq '.';
        truncate qq($file:$stream), $size
          or croak "Could not truncate stream '$stream' on file '$file': $!";
    } else {
        croak "Hosttype '$hosttype' is not supported";
    }

    return 1;
}

#---------------------------------------------------------------------
## @name ::getattr_file
## @description
##  Get the windows attributes of a file from a client
##  as the user (or any other user if specified).
##  Arguments are passed in a "key => value" style.
##
## @synopsis
##  $op = getattr_file( %options );
##
## @arg file
##  Required: File to get attributes for.
##
## @arg client
##  Optional: Hostspec, Hostrec or Connectrec of client.
##  The connection type of the Connectrec object is not used
##  since subtest uses its own connection type
##  (default is localhost).
##
## @arg user
##  Optional: Perform operation as this user. (Unix clients only)
##
## @return
##  Hash of windows attributes.
##
## @notes
##  Only supported on windows clients.
#---------------------------------------------------------------------
sub getattr_file(%) {
    my (%opts) = validate (
        @_,
        {
            file =>   { type => SCALAR },
            %common_parameters
        }
    );

    do_call(\&_getattr_file, %opts);
}

#---------------------------------------------------------------------
# Internal function to get the attributes of a file.
#---------------------------------------------------------------------
sub _getattr_file(%) {
    my %opts = @_;
    my $file = $opts{file};
    my ($w, %ret);

    # Only windows is supported
    my $hosttype = $opts{client}{hosttype};
    if ( $hosttype !~ /^windows/ ) {
        croak "_getattr_file: Hosttype '$hosttype' is not supported";
    }

    # Get file attributes
    $w = _win32api_call('GetFileAttributes', $file);

    # Constants for the encrypted and sparse file bits are not defined
    # in library so used the number
    %ret = (
        archive  => ($w & Win32API::File::FILE_ATTRIBUTE_ARCHIVE())    ? 1 : 0,
        compress => ($w & Win32API::File::FILE_ATTRIBUTE_COMPRESSED()) ? 1 : 0,
        encrypt  => ($w & 0x00004000)                                  ? 1 : 0,
        hidden   => ($w & Win32API::File::FILE_ATTRIBUTE_HIDDEN())     ? 1 : 0,
        sparse   => ($w & 0x00000200)                                  ? 1 : 0,
    );

    return %ret;
}

#---------------------------------------------------------------------
## @name ::read_file
## @description
##  Reads the contents of a file from a client as the user
##  (or any other user if specified).
##  Arguments are passed in a "key => value" style.
##
## @synopsis
##  $op = read_file( %options );
##
## @arg file
##  Required: File to be read.
##
## @arg client
##  Optional: Hostspec, Hostrec or Connectrec of client.
##  The connection type of the Connectrec object is not used
##  since subtest uses its own connection type
##  (default is localhost).
##
## @arg user
##  Optional: Perform operation as this user. (Unix clients only)
##
## @return
##  Contents of file.
#---------------------------------------------------------------------
sub read_file(%) {
    my (%opts) = validate (
        @_,
        {
            file => { type => SCALAR },
            %common_parameters
        }
    );
    do_call(\&_read_file, %opts);
}

#---------------------------------------------------------------------
# Internal function to read a file.
#---------------------------------------------------------------------
sub _read_file(%) {
    my %opts = @_;
    my $oflags = O_RDONLY|O_BINARY;

    # Open the file and read the contents
    sysopen(FH, $opts{file}, $oflags)
      or croak "_read_file: Could not open file for reading $opts{file}: $!";
    my @cont = <FH>;
    close(FH);

    return join('', @cont);
}

#---------------------------------------------------------------------
## @name ::read_stream
## @description
##  Read a stream (named attribute) on file from a client
##  as the user (or any other user if specified).
##  Arguments are passed in a "key => value" style.
##
## @synopsis
##  $op = read_stream( %options );
##
## @arg stream
##  Required: Named attribute to read.
##
## @arg file
##  Required: File where named attribute is located.
##
## @arg client
##  Optional: Hostspec, Hostrec or Connectrec of client.
##  The connection type of the Connectrec object is not used
##  since subtest uses its own connection type
##  (default is localhost).
##
## @arg user
##  Optional: Perform operation as this user.
##
## @return
##  Contents of stream.
#---------------------------------------------------------------------
sub read_stream(%) {
    my (%opts) = validate (
        @_,
        {
            stream => { type => SCALAR },
            file   => { type => SCALAR },
            %common_parameters
        }
    );
    do_call(\&_read_stream, %opts);
}

#---------------------------------------------------------------------
# Internal function to read a stream.
#---------------------------------------------------------------------
sub _read_stream(%) {
    my %opts = @_;
    my $stream;
    my $stream_name = $opts{file} . ':' . $opts{stream};

    # Get hosttype of client
    my $hosttype = $opts{client}{hosttype};

    # Streams or named attributes are only supported
    # on solaris 10 and windows
    if ( $hosttype eq 'unix-solaris' ) {
        # Command to read a named attribute
        $stream = qq(runat $opts{file} "cat $opts{stream}" |);
    } elsif ( $hosttype =~ /^windows/ ) {
        # Command to read a stream
        $stream = qq(<$opts{file}:$opts{stream});
    } else {
        croak "Hosttype '$hosttype' is not supported";
    }

    # Open a stream for reading
    open(FH, $stream)
      or croak "Could not open stream for reading $stream_name: $!";
    my @cont = <FH>;
    close(FH);

    return join('', @cont);
}

1;
