#
## @summary Local Operations required by ../Data.pm
## @author	dl-nacl-dev@netapp.com
## @status	private
## @notes	This has been ported from dp/snapmirror/qsm/TargetedData.pm
## @pod here

=head1 NAME

LocalOps - Local operations for Data Populate

=head1 SYNOPSIS

  use NACL::MTask::DataUtils::LocalOps;

  $data_obj = NACL::MTask::DataUtils::LocalOps->new(path => 'Z:\temp');

  # Create 3 directories with 10 files and 4 streams in each directory
  $data->create_dirs(number  => 3,
                     files   => { number=>10 },
                     streams => { number=>4 });

  # Create 6 files with 10 streams in each file
  $data->create_files(number  => 6,
                      streams => { number=>10 });

=head1 DESCRIPTION

This library creates directories, files, streams, links, symbolic links, 
sparse files. On Windows clients, it also creates files with ACLs as well 
as setting the compress, encrypt, archive or hidden attributes on files or 
directories.

The options size, size_dev, max_size, offset and offset_dev accept unit
modifiers (k, m, g, t). These modifiers can be lower or upper case with
an optional trailing 'b', e.g. '12k', '30 MB'.

=head1 STATUS

  Private

=head1 AUTHOR

  dl-nacl-dev@netapp.com

=head1 ROUTINES

=cut

package NACL::MTask::DataUtils::LocalOps;

use strict;
use warnings;

use Carp qw(croak);
use Cwd qw(getcwd);
use File::Spec qw();
use File::Compare qw();
use Storable qw(nfreeze thaw);
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF);
use Fcntl qw(O_RDONLY O_WRONLY O_BINARY O_CREAT O_TRUNC O_APPEND);
use NACL::MTask::DataUtils::FileManip qw(write_stream setattr_file 
                                         write_sparse link_file vol_free
                                         read_stream read_file getattr_file 
                                         setattr_stream remove_stream
                                         remove_file
                                         );
                                         
use Tharn qw/$Log/;
use NATE::BaseException;

#---------------------------------------------------------------------
# Function prototypes
#---------------------------------------------------------------------
sub new($%);
sub _do_call($$%);
sub hosttype($);
sub create_dirs($%);
sub _validate_dir_opts($);
sub _create_dirs($%);
sub create_files($%);
sub _validate_file_opts($);
sub _create_files($%);
sub create_streams($%);
sub _create_streams($%);
sub set_original($);
sub _set_original($;$);
sub counters($;%);
sub _counters($;%);
sub _sid($%);
sub acl_size($%);
sub _process_size_units($);
sub _set_on_pct($$);
sub _get_args($);
sub _modify_acl($$$;$$);
sub _get_from_list(\@$$);
sub _get_objects($$$$;$);
sub _get_totals($$;$);
sub _create_rand_data(\$$);
sub _create_sparse($$$);
sub _set_attributes($$$$);
sub _get_ref($$);
sub _units($);
sub _chdir($$);
sub _chdir_restore($);
sub _save_db($);
sub _get_db($);
sub _intRandGauss($$);
sub _randGauss($$);
sub modify($%);
sub _modify($%);



#---------------------------------------------------------------------
# Package variables
#---------------------------------------------------------------------
my $_seed;
my %acls;
my @val_char;
my $cifs_sid      = 1;
my $cifs_security = 1;
my $MAX_NUM       = 99999999999;
my $DATA_BLOCK    = 100;
my $MAX_WR_BLOCKS = 1000;
my $MAX_WR_SIZE   = $MAX_WR_BLOCKS*$DATA_BLOCK;
my $DB_FILE       = '.db';
my @ace_masks = qw(
    FILE_READ_DATA FILE_LIST_DIRECTORY FILE_WRITE_DATA FILE_ADD_FILE
    FILE_APPEND_DATA FILE_ADD_SUBDIRECTORY FILE_CREATE_PIPE_INSTANCE
    FILE_READ_EA FILE_WRITE_EA FILE_EXECUTE FILE_TRAVERSE FILE_DELETE_CHILD
    FILE_READ_ATTRIBUTES FILE_WRITE_ATTRIBUTES FILE_ALL_ACCESS
    FILE_GENERIC_READ FILE_GENERIC_WRITE FILE_GENERIC_EXECUTE
);
my @ace_flags = qw(
    CI OI IO NP
);
my @ace_types = qw(
    ACCESS_ALLOWED_ACE_TYPE ACCESS_DENIED_ACE_TYPE
);
my @ace_users = (
    'Users',
    'Guests',
    'Power Users',
    'Administrators',
    'Backup Operators',
);
my %pct_ret = (
    sparse_pct => 0,
    link_pct   => 0,
    slink_pct  => 0,
);
my %units_table = (
    b => 1,
    k => 1024,
    m => 1024*1024,
    g => 1024*1024*1024,
    t => 1024*1024*1024*1024,
);
my @counter_types = qw(create remove rename trunc copy modify sattrs read);
my @counter_otypes = qw(dirs files streams sparse links slinks acls);
my %invalid_filenames = (
    '.'         => 1, # Current directory
    '..'        => 1, # Parent directory
    '.db'       => 1, # Database file for this library
    '.snapshot' => 1, # Snapshot directory on filers
    'nul'       => 1, # The following are invalid file names in windows
    'prn'       => 1,
    'aux'       => 1,
    'con'       => 1,
    'com1'      => 1,
    'com2'      => 1,
    'com3'      => 1,
    'com4'      => 1,
    'com5'      => 1,
    'com6'      => 1,
    'com7'      => 1,
    'com8'      => 1,
    'com9'      => 1,
    'lpt1'      => 1,
    'lpt2'      => 1,
    'lpt3'      => 1,
    'lpt4'      => 1,
    'lpt5'      => 1,
    'lpt6'      => 1,
    'lpt7'      => 1,
    'lpt8'      => 1,
    'lpt9'      => 1,
);

#---------------------------------------------------------------------
# Constants
#---------------------------------------------------------------------
use constant VERBOSE_DIRS    => 0x01;
use constant VERBOSE_FILES   => 0x02;
use constant VERBOSE_STREAMS => 0x04;
use constant VERBOSE_SPARSE  => 0x08;
use constant VERBOSE_ATTRS   => 0x10;
use constant VERBOSE_INFO    => 0x80;
use constant VERBOSE_ALL     => 0xFF;

#---------------------------------------------------------------------

=head2 new

  $data_obj = NACL::MTask::DataUtils::LocalOps->new(path => $path
                                                     [, client => $client ]
                                                     [, seed => $seed ]
                                                     [, save => 0|1 ]);

LocalOps object constructor.
The seed for the random number generator can be specified with the
main parameter DATA_SEED or the seed option.

The following named arguments are accepted:

=over

=item Arguments

=over

=item path

(Required) : Objects are created and modified relative to this path.

=item save

(Optional) : Load the database file of a previously created data into memory.
It saves the data into disk when the object is destroyed.
Default is 0.

=item create

(Optional) : If true, creates the top level directory if it does not exist,
otherwise fails if it does not exist.
Default is 1.

=item seed

(Optional) : Use this seed to initialize the random number generator.
This has the highest precedence and should only be used if the caller
is managing the seed.

=item fillpct

(Optional) : The percentange of space usage that will not be exceeded.
Only supported on Windows. Default is 100.

=item timeout

(Optional) : Timeout for function execution.
Default is 600.

=item dbfile

(Optional) : Unique database file name to allow multiple clients to 
work in the same directory.

=back

=item Returns

TargetedData object.

=back

=for html <hr />

=cut

###############

my $object;

sub new($%) {
    
    my $class = shift;
    my %opts = validate (
        @_,
        {
            path    => { type => SCALAR },
            save    => { type => SCALAR,  default => 0 },
            create  => { type => SCALAR,  default => 1 },
            seed    => { type => SCALAR, optional => 1 },
            fillpct => { type => SCALAR,  default => 100 },
            timeout => { type => SCALAR,  default => 600 },
            dbfile  => { type => SCALAR, default => ""},
        }
    );
    
    
    my ($self, $cmd, $fh, %init, $msg, $type, $otype, %free);
    my $path = $opts{path};
    my $save = $opts{save};
    my $create = $opts{create};
    my $fillpct = $opts{fillpct};
    my $dbfile = File::Spec->catfile($opts{path}, $DB_FILE);

    # Save owner of this object
    $opts{_pid} = $$;

   if ( $^O eq 'MSWin32' ) {
            $opts{_hosttype} = 'windows';
    } else {
            $opts{_hosttype} = "unix-$^O";
    }


    if ( $opts{dbfile} ne "" ) {
        $dbfile = File::Spec->catfile($opts{path},
                                      $opts{dbfile});
    }
    $opts{db_file} = $dbfile;
    
        
        # Clean up path (canonical form)
        $opts{path} = File::Spec->canonpath($opts{path});

        $opts{d_used}   = 0;
        $opts{_fillpct} = 0;
        $opts{_modcnt}  = 0;
        $opts{_mcdiff}  = 10;

        if ( $opts{_hosttype} eq 'windows' ) {
            # For Windows, use only lower case letters for file names
            @val_char = ('a'..'z', 0..9, '_');
            # Append a '\' if a drive letter
            $opts{path} .= '\\' if $opts{path} =~ /^.\:$/;

            # Set flag if need to check space
            $opts{_fillpct} = 1 if $opts{fillpct} < 100;

            # Save path for vol_free
            $opts{d_path} = $opts{path};
            if ( $opts{d_path} !~ /^.\:/ ) {
                # Path is relative, need to know which volume this is
                $opts{d_path} = File::Spec->rel2abs($opts{d_path});
            }

            if ( $opts{_fillpct} ) {
                # Get total space
                %free = vol_free(vol => $opts{d_path});

                # Calculate maximum usage space the check for
                $opts{d_total} = $free{total};
                $opts{d_used}  = $free{used};
                $opts{d_max}   = ($opts{fillpct}/100) * $free{total};
            }
        } else {
            @val_char = ('A'..'Z', 'a'..'z', 0..9, '_');
        }

        # Read object from db file
        if ( $save && -e $dbfile ) {
            # Bless object
            $self = bless { %opts }, $class;

            # Read database of directory
            $self->_get_db();
        }

        # Initialize random seed
        if ( defined($opts{seed}) ) {
            $opts{_srand} = $opts{seed};
        } elsif ( defined($_seed) ) {
            $opts{_srand} = $_seed;
        } elsif ( defined($main::DATA_SEED) ) {
            $opts{_srand} = $main::DATA_SEED;
        } else {
            do {
                $opts{_srand} = time() ^ ($$ + ($$ << 15));
            } while (!$opts{_srand});
        }

        # Log the seed to reproduce same results in future runs
        $msg = "DATA_SEED: $opts{_srand}";
        $Log->debug($msg);

        srand($opts{_srand});
        $_seed = $opts{_srand} + 1;

        # Return blessed object read from file
        if ( defined($self) ) {
            $self->counters(clear => 1);
            $object = $self;
            return $self;
        }

        if ( ! -e $opts{path} ) {
            if ( $create ) {
                # Create new directory
              mkdir $opts{path}
                  or croak "Could not create the directory PATH $opts{path}: $!";
            } else {
                croak "Directory $path does not exist";
            }
        }

        # Initialize values
        %init = (
            _mod_id  => 0,
            _mod_id  => 0,
            _remove  => {},
            _objects => { '.' => { -type => 'd', -mod_id => 0 },
                          -type => 'd' },
        );

        # Initialize counters
        foreach $type ( @counter_types ) {
            foreach $otype ( @counter_otypes ) {
                $init{_counters}{$type}{$otype} = 0;
            }
        }
        
        
    # Return object
    my $ref = {%opts, %init};
    bless $ref, $class;
    $object = $ref;
    
    return $ref;
}

=head2 hosttype

  $out = $data_obj->hosttype();

Returns the hosttype of the client.

=over

=item Returns

Return hosttype of client.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub hosttype($) {
    my $self = shift;

    return $self->{_hosttype};
}

#---------------------------------------------------------------------

=head2 create_dirs

  $out = $data_obj->create_dirs([ dir => $rel_path ]
                                [, number => $ndirs ]
                                [, dev => $ndirs_dev ]
                                [, depth_avg => $depth_avg ]
                                [, depth_dev => $depth_dev ]
                                [, files => \%file_opts ]
                                [, streams => \%stream_opts ]
                                [, args => \%args ]);

Create a number of directories in the specified directory.

The following named arguments are accepted:

=over

=item Arguments

=over

=item dir

(Optional) : Directories are created under this directory.
Path is relative to the object's root path.
Default is the object's current working directory.

=item number

(Optional) : Number of directories to create.
Default is 1.

=item dev

(Optional) : Standard deviation of the number of directories to create.
Default is 0.

=item depth_avg

(Optional) : Average depth to create.
Default is 0.

=item depth_dev

(Optional) : Standard deviation of depth to create.
Default is 0.

=item files

(Optional) : Reference to a hash of options to create files under each
newly created directory. See create_files for options.

=item streams

(Optional) : Reference to a hash of options to create streams on each
newly created directory. See create_streams for options.

=item args

(Optional) : Reference to a hash of options on how to create each directory.

  {
      number   => Maximum number of the specified objects to modify.
      percent  => Modifies this percentage of the specified objects,

      name_size     => Size of directory name.
      name_size_dev => Standard deviation of directory name.
      max_name_size => Size of directory name is a random number between 0
                       and this value (default is 0).
      name_reuse    => Reuse name of previously removed directory
                       (default is 0).
      acl_num       => Maximum number of directories to add an ACL.
      stream_num    => Maximum number of directories to add streams.
      compress_num  => Maximum number of directories to set/clear compress attr.
      encrypt_num   => Maximum number of directories to set/clear encrypt attr.
      archive_num   => Maximum number of directories to set/clear archive attr.
      hidden_num    => Maximum number of directories to set/clear hidden attr.
      acl_pct       => Percentage of directories to add an ACL.
      stream_pct    => Percentage of directories to add streams.
      compress_pct  => Percentage of directories to set/clear compress attr.
      encrypt_pct   => Percentage of directories to set/clear encrypt attr.
      archive_pct   => Percentage of directories to set/clear archive attr.
      hidden_pct    => Percentage of directories to set/clear hidden attr.
      compress      => Set or clear the compress attribute.
      encrypt       => Set or clear the encrypt attribute.
      archive       => Set or clear the archive attribute.
      hidden        => Set or clear the hidden attribute.
      acl_masks     => Use a maximum of this number of access mask flags (0-18)
                       (default is 8; 0 means use maximum available).
      acl_flags     => Use a maximum of this number of flags (0-6)
                       (default is 4; 0 means use maximum available).
      acl_types     => Use a maximum of this number of types (0-3)
                       (default is 1; 0 means use maximum available).
      acl_users     => Use a maximum of this number of users (0-7)
                       (default is 2; 0 means use maximum available).
      users         => List of users/groups to use instead of the defaults.
      acl           => Create directory with an ACL (1 or 2).
                       1: the number of ACEs varies between 1 and the product
                          of the acl_* options
                       2: the number of ACEs is the product of the
                          acl_* options (fixed).
  }

=back

=item Returns

1 if succeeds.

=back

=for html <hr />

=cut

sub create_dirs($%) {
    my ($self, %opts) = @_;
    _validate_dir_opts(\%opts);
    return $self->_do_call(\&_create_dirs, %opts);
}

#---------------------------------------------------------------------
# Internal function
# Validate the dir options
#---------------------------------------------------------------------
sub _validate_dir_opts($) {
    my $opts = shift;

    _validate_file_opts($opts->{files});
    _process_size_units($opts->{streams}{args});

    return;
}

#---------------------------------------------------------------------
# Internal function
# Validate that space in cwd has reached its maximum.
# Only for unix
#---------------------------------------------------------------------
sub _check_if_space_filled{
    my $self = shift ;

    ## As per burt934717 , Filesys:Df module is not
    ## present over windows
    if( $^O !~ /MSWin32/ && $self->{_hosttype} =~ /unix/){
        require Filesys::Df;
        my $df_ref = Filesys::Df::df(getcwd);

        if(defined($df_ref)) {
           $Log->debug(  "cwd :".getcwd." Total 1k blocks used: $df_ref->{used}");
           $Log->debug(  "cwd :".getcwd." Percent full: $df_ref->{per}");
           if($df_ref->{per} >= 99){
              return 1 ;
           }else{
              return 0 ;
           }
        }
    }
    return;
}

sub _create_dirs($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            dir       => { type => SCALAR,  default => '' },
            number    => { type => SCALAR,  default => 1 },
            dev       => { type => SCALAR,  default => 0 },
            depth_avg => { type => SCALAR,  default => 0 },
            depth_dev => { type => SCALAR,  default => 0 },
            files     => { type => HASHREF, default => {} },
            streams   => { type => HASHREF, default => {} },
            args      => { type => HASHREF, default => {} },
            # Internal parameter -- current depth
            depth     => { type => SCALAR,  default => 1 },
        }
    );
    my ($ref, $args, $number, $depth, %free);

    # Get database reference of directory where
    # the directories are to be created
    $opts{dir} =~ s/^[\/\\]+//;
    $ref = $self->_get_ref($opts{dir});

    # Get number of directories to create
    $number = _intRandGauss($opts{number}, $opts{dev});
    return 0 if $number < 1;

    # Check fill percentage
    if ( $self->{_fillpct} ) {
        # Get used space on volume
        %free = vol_free(vol => $self->{d_path});
        # Set current space usage and return if limit is hit
        $self->{d_used} = $free{used};
        return 0 if $self->{d_used} > $self->{d_max};
    }

    # Change to directory
    $self->_chdir($opts{dir});

    $Log->debug("Creating $number directories on " . getcwd);

    # Set default value for maximum number of streams/attr/acls to create
    $opts{args}{stream_num} = $MAX_NUM unless exists $opts{args}{stream_num};
    $opts{args}{attr_num}   = $MAX_NUM unless exists $opts{args}{attr_num};
    $opts{args}{acl_num}    = $MAX_NUM unless exists $opts{args}{acl_num};

    foreach ( 1..$number ) {
	my $entry;
        # Get arguments on how to create the directories
        $opts{args}{_rmlist} = $self->{_remove}{dirs};
        $opts{args}{_ref}    = $ref;
        $args = _get_args($opts{args});
        $entry = $args->{name};

        if ( $self->{_fillpct} && $self->{d_used} > $self->{d_max} ) {
            # Done populating
            last;
        }

        last if($self->_check_if_space_filled()) ;

        # Create new directory
        $Log->debug("Create directory $entry");
        mkdir $entry or croak "Could not create directory $entry: $!";

        $self->{_counters}{create}{dirs}++;

        # Add directory to database
        $ref->{$entry} = { -type => 'd', -mod_id => $self->{_mod_id} };
        # Increment the number of directories owned by parent directory
        $ref->{-dirs}++;

        # If parent directory is encrypted or compressed
        # this directory is also encrypted or compressed
        $ref->{$entry}{-encrypt}  = 1 if $ref->{-encrypt};
        $ref->{$entry}{-compress} = 1 if $ref->{-compress};

        # Set attributes on the directory
        if ( %{$args->{sattrs}} ) {
            _set_attributes($entry, $args->{sattrs}, $ref, $opts{args});
        }

        # test permissions to check  d---------
        my $mode = (stat($entry))[2];
        my $permissions = sprintf "%04o", $mode &07777;
        if ($permissions eq '0000') {
            NATE::BaseException->throw("Reproduced Burt 849347");
        } else {
#            $Log->comment("permission of $entry is $permissions"); 
        }
        
        # Create files on this directory
        if ( $opts{files}{number} ) {
            $self->_create_files(dir=>$entry, %{$opts{files}});
        }

        # Create streams on this directory
        if ( $args->{streams} && $opts{streams}{number}
          && ( $opts{args}{stream_num} > 0 )) {
            $self->_create_streams(file=>$entry, %{$opts{streams}});
            $opts{args}{stream_num}--;
        }

        $depth = _intRandGauss($opts{depth_avg}, $opts{depth_dev});
        if ( $depth > $opts{depth} ) {
            $self->_create_dirs(%opts, dir=>$entry, depth=>$opts{depth}+1);
        }

        # Set ACL on directory
        if ( $args->{acl} && ( $opts{args}{acl_num} > 0 ) ) {

            $Log->debug("Set ACL on dir $entry");
            $ref->{$entry}{-acl} = $self->_modify_acl($entry, $args);
            $opts{args}{acl_num}--;
            $self->{_counters}{create}{acls}++;
        }
    }

    # Change back to previous directory
    $self->_chdir_restore();

    return $number;
}

=head2 create_files

  $out = $data_obj->create_files([ dir => $rel_path ]
                                 [, number => $nfiles ]
                                 [, dev => $nfiles_dev ]
                                 [, streams => \%stream_opts ]
                                 [, args => \%args ]);

Create a number of files in the specified directory.

The following named arguments are accepted:

=over

=item Arguments

=over

=item dir

(Optional) : Files are created under this directory.
Path is relative to the object's root path.
Default is the object's current working directory.

=item number

(Optional) : Number of files to create.
Default is 1.

=item dev

(Optional) : Standard deviation of the number of files to create.
Default is 0.

=item streams

(Optional) : Reference to a hash of options to create streams on each
newly created file. See create_streams for options.

=item args

(Optional) : Reference to a hash of options on how to create each file.

  {
      size          => Size of file to create.
      size_dev      => Standard deviation of the size of file to create.
      max_size      => Size of file is a random number between 0 and
                       this value (default is 1000).
      name_size     => Size of file name.
      name_size_dev => Standard deviation of file name.
      max_name_size => Size of file name is a random number between 0
                       and this value (default is 0).
      name_reuse    => Reuse name of previously removed file (default is 0).
      acl_num       => Maximum number of files to add an ACL.
      stream_num    => Maximum number of files to add streams.
      sparse_num    => Maximum number of files to be sparse files.
      link_num      => Maximum number of files to be links.
      slink_num     => Maximum number of files to be symbolic links.
      compress_num  => Maximum number of files to set/clear compress attr.
      encrypt_num   => Maximum number of files to set/clear encrypt attr.
      archive_num   => Maximum number of files to set/clear archive attr.
      hidden_num    => Maximum number of files to set/clear hidden attr.
      acl_pct       => Percentage of files to add an ACL.
      stream_pct    => Percentage of files to add streams.
      sparse_pct    => Percentage of files to be sparse files.
      link_pct      => Percentage of files to be links.
      slink_pct     => Percentage of files to be symbolic links.
      compress_pct  => Percentage of files to set/clear compress attr.
      encrypt_pct   => Percentage of files to set/clear encrypt attr.
      archive_pct   => Percentage of files to set/clear archive attr.
      hidden_pct    => Percentage of files to set/clear hidden attr.
      compress      => Set or clear the compress attribute.
      encrypt       => Set or clear the encrypt attribute.
      archive       => Set or clear the archive attribute.
      hidden        => Set or clear the hidden attribute.
      sparse_opts   => Hash reference on how to create each sparse file.
      acl_masks     => Use a maximum of this number of access mask flags (0-18)
                       (default is 8; 0 means use maximum available).
      acl_flags     => Use a maximum of this number of flags (0-6)
                       (default is 4; 0 means use maximum available).
      acl_types     => Use a maximum of this number of types (0-3)
                       (default is 1; 0 means use maximum available).
      acl_users     => Use a maximum of this number of users (0-7)
                       (default is 2; 0 means use maximum available).
      users         => List of users/groups to use instead of the defaults.
      acl           => Create file with an ACL (1 or 2).
                       1: the number of ACEs varies between 1 and the product
                          of the acl_* options
                       2: the number of ACEs is the product of the
                          acl_* options (fixed).
  }

where sparse_opts contain the following:

  {
      ndata      => Number of data regions to create.
      ndata_dev  => Standard deviation of the number of data regions.
      offset     => Offset between data regions.
      offset_dev => Standard deviation of the offset between data regions.
      size       => Size of each data region.
      size_dev   => Standard deviation of the size of each data region.
  }

=back

=item Returns

1 if succeeds.

=back

=for html <hr />

=cut



sub create_files($%) {
    my ($self, %opts) = @_;
   _validate_file_opts(\%opts);
    return $self->_do_call(\&_create_files, %opts);
}

#---------------------------------------------------------------------
# Internal function
# Validate the file options
#---------------------------------------------------------------------
sub _validate_file_opts($) {
    my $opts = shift;

    _process_size_units($opts->{args});
    _process_size_units($opts->{args}{sparse_opts});
    _process_size_units($opts->{streams}{args});

    return;
}

#---------------------------------------------------------------------
# Function to create files with streams
# It runs on the local machine
#---------------------------------------------------------------------
sub _create_files($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            dir     => { type => SCALAR,  default => '' },
            number  => { type => SCALAR,  default => 1 },
            dev     => { type => SCALAR,  default => 0 },
            streams => { type => HASHREF, default => {} },
            args    => { type => HASHREF, default => {} },
        }
    );
    
    my ($ref, $args, $cont, $size, $name, $number);

    # Get database reference of directory where
    # the files are to be created
    $opts{dir} =~ s/^[\/\\]+//;
    
    $ref = $self->_get_ref($opts{dir});
    # Get number of files to create
    $number = _intRandGauss($opts{number}, $opts{dev});
    
    return 0 if $number < 1;

    # Change to directory
    
    $self->_chdir($opts{dir});

    $Log->debug( "Creating $number files on " . getcwd );

    # Set default value for maximum number of streams/sparse/attr/acls to create
    $opts{args}{stream_num} = $MAX_NUM unless exists $opts{args}{stream_num};
    $opts{args}{sparse_num} = $MAX_NUM unless exists $opts{args}{sparse_num};
    $opts{args}{attr_num}   = $MAX_NUM unless exists $opts{args}{attr_num};
    $opts{args}{acl_num}    = $MAX_NUM unless exists $opts{args}{acl_num};

    foreach ( 1..$number ) {

	my $entry;
        # Get arguments on how to create the files
        $opts{args}{_rmlist} = $self->{_remove}{files};
        $opts{args}{_ref}    = $ref;
        $args = _get_args($opts{args});
        $size = $args->{size};
        $entry = $args->{name};
        ## skip loop if the file is actually a directory
        next if( -d $entry );
        $self->{d_used} += $size;

        if ( $self->{_fillpct} && $self->{d_used} > $self->{d_max} ) {
            # Done populating
            last;
        }

        last if($self->_check_if_space_filled()) ;

        # If parent directory is encrypted or compressed
        # this file is also encrypted or compressed
        $ref->{$entry}{-encrypt}  = 1 if $ref->{-encrypt};
        $ref->{$entry}{-compress} = 1 if $ref->{-compress};

        if ( $args->{link} ) {
            $Log->debug("Create " .
                        ($args->{link_data}{symbolic} ? 'symbolic ' : '') .
                        "link $entry -> $args->{link_data}{file}");
            remove_file(file => $entry) if( -e $entry );
            link_file(flink=>$entry, %{$args->{link_data}});
            
            if ( $args->{link_data}{symbolic} ) {
                $ref->{$entry}{-link} = 2;
                $opts{args}{slink_num}--;
                $self->{_counters}{create}{slinks}++;
            } else {
                $ref->{$entry}{-link} = 1;
                $opts{args}{link_num}--;
                $self->{_counters}{create}{links}++;
            }
        } elsif ( $args->{sparse} ) {
            # Need to create sparse file before setting attributes
            $Log->debug("Create sparse file $entry");
            _create_sparse($entry, $args, $ref);
            $opts{args}{sparse_num}--;
            $self->{_counters}{create}{sparse}++;
        } else {
            # Create new file
            $Log->debug("Create file $entry");

            sysopen(FH, $entry, O_WRONLY|O_BINARY|O_CREAT|O_TRUNC)
		or croak "Could not create file $entry: $!";
        }

        if ( %{$args->{sattrs}} && ! $args->{link} ) {
            if ( ! $args->{sparse} ) {
                # Close the file to set the attributes
                close(FH);
            }

            # Set attributes on the file before writing to it
            # in order to compress/encrypt the file on the fly
            # instead of creating the file and then do the
            # compression/encryption
            _set_attributes($entry, $args->{sattrs}, $ref, $opts{args});

            if ( ! $args->{sparse} ) {
                # Open the file again
                sysopen(FH, $entry, O_WRONLY|O_BINARY)
				    or croak "Could not create sparse file $entry: $!";
            }
        }

        if ( ! $args->{sparse} && ! $args->{link} ) {
            # Write the file in blocks so any size of file can be created
            while ( $size > 0 ) {
                # Generate data block
                $size -= _create_rand_data($cont, $size);
                # Write current block to file
                

                print FH $cont or croak "Could not write to file $entry: $!";
            }
            close(FH);
            $self->{_counters}{create}{files}++;
        }

        # Add file type and mod_id
        $ref->{$entry}{-type}   = 'f';
        $ref->{$entry}{-mod_id} = $self->{_mod_id};

        # Increment the number of files owned by parent directory
        $ref->{-files}++;

        # Create streams on this file
        # Do not create streams if it is a link
        if ( ! $args->{link} && $args->{streams} && $opts{streams}{number}
          && ( $opts{args}{stream_num} > 0 ) ) {
            $self->_create_streams(file=>$entry, %{$opts{streams}});
            $opts{args}{stream_num}--;
        }

        # Set ACL on file
        if ( $args->{acl} && ( $opts{args}{acl_num} > 0 ) ) {
            $Log->debug("Set ACL on file $entry");
            $ref->{$entry}{-acl} = $self->_modify_acl($entry, $args);
            $opts{args}{acl_num}--;
            $self->{_counters}{create}{acls}++;
        }
    }

    # Change back to previous directory
    $self->_chdir_restore();

    
    
    return $number;
}

#---------------------------------------------------------------------

=head2 create_streams

  $out = $data_obj->create_streams([ file => $file ]
                                   [, number => $nstreams ]
                                   [, dev => $nstreams_dev ]
                                   [, args => \%args ]);

Create a number of streams in the specified file/directory.

The following named arguments are accepted:

=over

=item Arguments

=over

=item file

(Optional) : The streams are attached to this file/directory.
Default is the object's current working directory.

=item number

(Optional) : Number of streams to create.
Default is 1.

=item dev

(Optional) : Standard deviation of the number of streams to create.
Default is 0.

=item args

(Optional) : Reference to a hash of options on how to create each stream.

  {
      size          => Size of stream to create.
      size_dev      => Standard deviation of the size of stream to create.
      max_size      => Size of stream is a random number between 0 and
                       this value (default is 1000).
      name_size     => Size of stream's name.
      name_size_dev => Standard deviation of stream's name.
      max_name_size => Size of stream's name is a random number between 0
                       and this value (default is 0).
      name_reuse    => Reuse name of previously removed stream (default is 0).
  }

=back

=item Returns

1 if succeeds.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
# Function to create streams
# It runs on the local machine
#---------------------------------------------------------------------
sub _create_streams($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            file   => { type => SCALAR,  default => '.' },
            number => { type => SCALAR,  default => 1 },
            dev    => { type => SCALAR,  default => 0 },
            args   => { type => HASHREF, default => {} },
        }
    );
    _process_size_units($opts{args});
    my ($ref, $args, $cont, $size, $append, $name, $number);

    # Get database reference of object where
    # the streams are to be created
    $opts{file} =~ s/^[\/\\]+//;
    $ref = $self->_get_ref($opts{file});

    # Get number of streams to create
    $number = _intRandGauss($opts{number}, $opts{dev});
    return 0 if $number < 1;
    $Log->debug("Creating $number streams on $opts{file}");

    foreach ( 1..$number ) {
        my $entry;
        # Get arguments on how to create the streams
        $opts{args}{_rmlist} = $self->{_remove}{streams};
        $opts{args}{_ref}    = $ref;
        $args = _get_args($opts{args});
        $size = $args->{size};
        $entry = $args->{name};
        $self->{d_used} += $size;

        if ( $self->{_fillpct} && $self->{d_used} > $self->{d_max} ) {
            # Done populating
            last;
        }
        
        $Log->debug( "Create stream $entry" );

        # Create new stream
        $append = 0;
        # Write the stream in blocks so any size of stream can be created
        do {
            # Generate data block
            $size -= _create_rand_data($cont, $size);
            # Write current block to stream
            write_stream(file     => $opts{file},
                         stream   => $entry,
                         contents => $cont,
                         append   => $append);
            # Make sure next iteration the contents are appended
            $append = 1;
        } while ( $size > 0 );
        # Add stream to database
        $ref->{$entry} = { -type => 's', -mod_id => $self->{_mod_id} };
        # Increment the number of streams owned by parent object
        $ref->{-streams}++;
        $self->{_counters}{create}{streams}++;
    }

    return $number;
}

=head2 modify

  $out = $data_obj->modify([ dirs => \%dir_opts ]
                           [, files => \%file_opts ]
                           [, streams => \%stream_opts ]);

Modify the populated tree according to the specified options.
Each of the main options could have any of the following options:

  {
      number   => Maximum number of the specified objects to modify.
      original => Only modify original objects created before a call
                  to this function.
      skip     => Number of the specified objects to skip before modifying.
      skiproot => Do not modify the root directory.
      percent  => Modifies this percentage of the specified objects,
                  it has precedence over the 'number' option.
      compress => Modify compressed files: yes|no|any.
                  yes: only modify compressed files.
                  no:  only modify uncompressed files.
                  any: modify files regardless of the compress attribute.
      encrypt  => Modify encrypted files: yes|no|any.
                  yes: only modify encrypted files.
                  no:  only modify decrypted files.
                  any: modify files regardless of the encrypt attribute.
      archive  => Modify archived files: yes|no|any.
                  yes: only modify archived files.
                  no:  only modify non-archived files.
                  any: modify files regardless of the archive attribute.
      hidden   => Modify hidden files: yes|no|any.
                  yes: only modify hidden files.
                  no:  only modify non-hidden files.
                  any: modify files regardless of the hidden attribute.
      acl      => Modify files with ACLs: yes|no|any.
                  yes: only modify files with ACLs.
                  no:  only modify files without ACLs.
                  any: modify files regardless of ACLs.
      mode     => Modification mode:
                  remove:  remove object.
                  rename:  rename object (dirs and files only).
                  trunc:   truncate object (files and streams only).
                  create:  create under object (dirs and files only).
                  modify:  modify contents of object (files and streams only).
                  sattrs:  set attributes (files and directories only).
                  acl_mod: modify ACEs (files and directories only).
                  acl_del: remove ACLs (files and directories only).
      args     => How to modify the object according to mode
                  (trunc, create, modify, sattrs and acl_mod modes only).
  }

The dirs option could also have:

  {
      streams  => Modify directories with or without streams: yes|no|any.
                  yes: only modify directories with streams.
                  no:  only modify directories without streams.
                  any: modify directories regardless if they contain streams.
      dirs     => Modify directories with or without directories: yes|no|any.
                  yes: only modify directories with directories.
                  no:  only modify directories without directories.
                  any: modify directories regardless if they contain directories
      files    => Modify directories with or without files: yes|no|any.
                  yes: only modify directories with files.
                  no:  only modify directories without files
                  any: modify directories regardless if they contain files.
      depth    => Increase depth of directories, default is 1.
                  If set to 0 it will maintain the current depth structure.
  }

The files option could also have:

  {
      streams  => Modify files with or without streams: yes|no|any.
                  yes: only modify files with streams.
                  no:  only modify files without streams.
                  any: modify files regardless if they contain streams.
      sparse   => Modify files with or without sparse files: yes|no|any.
                  yes: only modify files with sparse files.
                  no:  only modify files without sparse files.
                  any: modify files regardless if they contain sparse files.
  }

Look at create_dirs, create_files or create_streams for specific options
to the args option for the create mode.

The args options for the trunc mode are:

  {
      size     => Truncate to this size.
      size_dev => Standard deviation of the truncate size.
      max_size => Truncate size is a random number between 0 and this value
                  (default is 1000 if size is not given).
  }

The args options for the modify mode are:

  {
      size     => Size of new contents (if append and blocks are not specified).
      size_dev => Standard deviation of the size.
      max_size => Size of new contents is a random number between 0 and this
                  value (default is 1000 if size is not given).
      append   => Append new contents to file or stream.
                  Options size and max_size give the size of data to append.
      blocks   => Number of blocks to modify, offset of each block to modify
                  is selected at random.
                  Option size gives the size of block to modify while max_size
                  gives the maximum offset (do not modify beyond this offset).
      sparse_opts => Hash reference like sparse_opts on create_files.
  }

The args options for the sattrs mode are:

  {
      compress => Set or clear the compress attribute.
      encrypt  => Set or clear the encrypt attribute.
      archive  => Set or clear the archive attribute.
      hidden   => Set or clear the hidden attribute.
  }

The args options for the acl_mod mode are:

  {
      percent   => Percent of ACEs to remove (default is 0).
      acl_masks => Use a maximum of this number of access mask flags (0-18)
                   (default is 8; 0 means use maximum available).
      acl_flags => Use a maximum of this number of flags (0-6)
                   (default is 4; 0 means use maximum available).
      acl_types => Use a maximum of this number of types (0-3)
                   (default is 1; 0 means use maximum available).
      acl_users => Use a maximum of this number of users (0-7)
                   (default is 2; 0 means use maximum available).
      users     => List of users/groups to use instead of the defaults.
  }

Renaming files or directories will not process objects owned by these.
In other words, renaming directories while trying to modify files at the
same time, the files owned by renamed directories will not be processed.
Make two separate calls to modify, one to rename the directories and a
second call to modify the files.

The following named arguments are accepted:

=over

=item Arguments

=over

=item dirs

(Optional) : Reference to a hash of options on how to modify the directories.

=item files

(Optional) : Reference to a hash of options on how to modify the files.

=item streams

(Optional) : Reference to a hash of options on how to modify the streams.

=back

=item Returns

1 if succeeds.

=back

=for html <hr />

=cut

sub modify($%) {
    my ($self, %opts) = @_;

    _validate_dir_opts($opts{dirs}{args});
    _validate_file_opts($opts{files}{args});
    _process_size_units($opts{streams}{args});

    return $self->_do_call(\&_modify, %opts);
}

sub _modify($%) {
    my $self = shift;

    my (%opts) = validate (
        @_,
        {
            dirs    => { type => HASHREF, optional => 1 },
            files   => { type => HASHREF, optional => 1 },
            streams => { type => HASHREF, optional => 1 },
        }
    );
    my ($entry, @args, %free);
    my $rmcount = 0;

    # Default validation hash for the arguments
    my %common = (
        number   => { type => SCALAR,  default => $MAX_NUM },
        original => { type => SCALAR,  default => 0 },
        skip     => { type => SCALAR,  default => 0 },
        skiproot => { type => SCALAR,  default => 0 },
        percent  => { type => SCALAR,  default => 100 },
        compress => { type => SCALAR,  default => 'no',
                     regex => qr/^(no|yes|any)$/ },
        encrypt  => { type => SCALAR,  default => 'no',
                     regex => qr/^(no|yes|any)$/ },
        archive  => { type => SCALAR,  default => 'any',
                     regex => qr/^(no|yes|any)$/ },
        hidden   => { type => SCALAR,  default => 'no',
                     regex => qr/^(no|yes|any)$/ },
        acl      => { type => SCALAR,  default => 'no',
                     regex => qr/^(no|yes|any)$/ },
        mode     => { type => SCALAR,  default => 'remove' },
        args     => { type => HASHREF, default => {} },
    );

    my %def_args = (
        dirs => {
            %common,
            depth   => { type => SCALAR,  default => 1 },
            dirs    => { type => SCALAR,  default => 'no',
                        regex => qr/^(no|yes|any)$/ },
            files   => { type => SCALAR,  default => 'no',
                        regex => qr/^(no|yes|any)$/ },
            streams => { type => SCALAR,  default => 'no',
                        regex => qr/^(no|yes|any)$/ },
        },
        files => {
            %common,
            sparse  => { type => SCALAR,  default => 'no',
                        regex => qr/^(no|yes|any)$/ },
            streams => { type => SCALAR,  default => 'no',
                        regex => qr/^(no|yes|any)$/ },
        },
        streams => \%common,
    );

    # Set defaults for the mode arguments
    foreach $entry ( qw(dirs files streams) ) {
        # Validate arguments for option
        @args = ( %{$opts{$entry}} );


        $opts{$entry} = { validate(@args, $def_args{$entry}) }
          if %{$opts{$entry}};
        if ( !$opts{$entry}{number} || !$opts{$entry}{percent} ) {
            delete $opts{$entry};
            next;
        }

        # Get number of 'remove' modifies
        $rmcount++ if $opts{$entry}{mode} eq 'remove';

        # Set default values for create mode
        if ( $opts{$entry}{mode} eq 'create'
          && exists $opts{$entry}{args}{args} ) {
            $opts{$entry}{args}{args}{stream_num} = $MAX_NUM
              unless defined $opts{$entry}{args}{args}{stream_num};
        }

        # Set default values for modify mode
        if ( $entry eq 'files' && $opts{$entry}{mode} eq 'modify'
          && exists $opts{$entry}{args}{args} ) {
            $opts{$entry}{args}{args}{sparse_num} = $MAX_NUM
              unless defined $opts{$entry}{args}{args}{sparse_num};
        }

        # Set default values for acl_mod and acl_del mode
        if ( $opts{$entry}{mode} =~ /^acl_(mod|del)$/
          && exists $opts{$entry}{args} ) {
            $opts{$entry}{args}{acl_num} = $MAX_NUM
              unless defined $opts{$entry}{args}{acl_num};
        }
    }

    # Return if nothing to modify
    return 1 unless keys %opts;

    # Check fill percentage
    if ( $self->{_fillpct} ) {
        # Get used space on volume
        %free = vol_free(vol => $self->{d_path});
        # Set current space usage and return if limit is hit
        # except if there are any removes
        $self->{d_used} = $free{used};
        $self->{d_remove} = $rmcount;
        return 1 if !$rmcount && $self->{d_used} > $self->{d_max};
    }

    # Set the modification id to keep newly modify/created
    # objects to be modify again in the same call
    $self->{_mod_id}++;

    # Process objects starting at the root
    $self->_process_objects('', $self->{_objects}, \%opts, 0);

    return 1;
}


#---------------------------------------------------------------------

=head2 counters

  $out = $data_obj->counters();

Get or clear the object counters. The counters are the number of an
specific object type which has been modified or created. The following
is a list of the different counters:

    total   => Totals (counter is not cleared when using the 'clear' option)
    create  => Create counters
    remove  => Remove counters
    rename  => Rename counters
    trunc   => Truncate counters
    copy    => Copy counters
    modify  => Modify counters
    sattrs  => Attributes set counters
    read    => Read counters

Each of the counters have the following specific object counters:

    dirs    => Number of directories
    files   => Number of files
    streams => Number of streams
    sparse  => Number of sparse files
    links   => Number of links
    slinks  => Number of symbolic links
    acls    => Number of ACL's

The following named arguments are accepted:

=over

=item Arguments

=over

=item clear

(Optional) : Clear all the counters except for the 'total' counter.
Defaults to 0.

=back

=item Returns

A hash reference of all the counters.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
# Get or clear the object counters
#---------------------------------------------------------------------
sub counters($;%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            clear => { type => SCALAR, default => 0 },
        }
    );
    my (%ret, $ctype, $otype, %total);

    if ( $opts{clear} ) {
        # Clear all counters
        foreach $ctype ( keys %{$self->{_counters}} ) {
            foreach $otype ( keys %{$self->{_counters}{$ctype}} ) {
                $self->{_counters}{$ctype}{$otype} = 0;
            }
        }
        return;
    }

    # Get all the counters and calculate the total counters
    %ret = %{$self->{_counters}};
    $ret{total} = {
        # Default values
        ( map { $_ => 0 } @counter_otypes ),
        # Get totals and overwrite defaults
        %{$self->_get_totals($self->{_objects}{'.'}, \%total)},
    };

    return \%ret;
}

# #---------------------------------------------------------------------

# =head2 sid

  # $sid = $data_obj->sid(user => $user);

# Get the SID of the specified user.

# The following named arguments are accepted:

# =over

# =item Arguments

# =over

# =item user

# (Required) : The user/group to get the SID for. Could be just the username
# or in the 'domain\username' format.

# =back

# =item Returns

# SID of user.

# =back

# =for html <hr />

# =cut


#---------------------------------------------------------------------
# Get the SID for the user
# It runs on the local machine
#---------------------------------------------------------------------
sub _sid($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            user => { type => SCALAR },
        }
    );

    croak "SID is only supported on Windows"
      if $self->{_hosttype} ne 'windows';

    if ( $cifs_sid ) {
        # Load only once
        require Win32::Security::SID;
        $cifs_sid = 0;
    }

    # Get the SID for the username
    return Win32::Security::SID::ConvertNameToSid($opts{user});
}

#---------------------------------------------------------------------

=head2 acl_size

  $out = $data_obj->acl_size(size => $size, users => \@users);

Calculate the arguments for ACL given the specified size. The size
of the ACL depends on the number of ACEs and the size of each ACE
depends on the SID of the user. A list of users is needed in order
to get the size of each SID (variable length) to calculate the
number of ACEs required for a given size ACL.

For small size ACL the list of users to use must be small while
for large ACLs the list of users to use must be larger to get
accurate results. For an ACL of less than 4k use one user and
for an ACL of 100k use at least 10 users/groups.

The following named arguments are accepted:

=over

=item Arguments

=over

=item size

(Required) : Calculate arguments for this size of ACL.

=item users

(Required) : Reference to an array of users to be used in the creation
of the ACL.

=back

=item Returns

A reference to a hash of arguments for creating or modifying ACLs.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub acl_size($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            size  => { type => SCALAR },
            users => { type => ARRAYREF },
        }
    );
    my ($naces, $ace_size, $total, $sid, $m, $f, $t, $u);
    my $count = @{$opts{users}};

    croak "List of users is empty" if $count < 1;
    $opts{size} = _units($opts{size});

    # Calculate average size of ACE
    # The size of the ACE is 8 bytes for the header plus the SID
    $ace_size = 0;
    foreach $u ( @{$opts{users}} ) {
        $sid = $self->_sid(user => $u);
        croak "Could not get SID for $u" if not defined $sid;
        $ace_size += length($sid);
    }
    # Using the average size of SIDs
    $ace_size = 8 + $ace_size/$count;

    # Calculate the number of ACEs needed for the specified size
    # The size of the ACL is 16 bytes for the header plus the ACEs
    # Get the number of ACEs for a 10% more in order
    # to compensate for the rounding down while calculating
    # the products
    $naces = int(0.5 + (1.1*$opts{size} - 16) / $ace_size);

    # Calculate the products to specified on the arguments to create ACL
    # Use brute force approach by going through all possible combinations
    foreach $t ( 1..@ace_types ) {
        foreach $f ( 1..@ace_flags ) {
            foreach $m ( 1..@ace_masks ) {
                $total = $m * $f * $t * $count;
                if ( $total > 0.9*$naces && $total < 1.1*$naces ) {
                    return { acl       => 2,
                             acl_masks => $m,
                             acl_flags => $f,
                             acl_types => $t,
                             acl_users => $count,
                             users     => $opts{users} };
                }
            }
        }
    }

    return;
}

=head2 set_original

  $out = $data_obj->set_original();

Set attribute of all elements back to original.
It is used to group modifies into batches so the original flag can
be used to modify elements in a previous batch.

=over

=item Returns

1 if succeeds.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------

=head2 set_original

  $out = $data_obj->set_original();

Set attribute of all elements back to original.
It is used to group modifies into batches so the original flag can
be used to modify elements in a previous batch.

=over

=item Returns

1 if succeeds.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub set_original($) { shift->_do_call(\&_set_original) }


#---------------------------------------------------------------------
# Set attribute of all elements to original (mod_id = 0)
# It runs on the local machine
#---------------------------------------------------------------------
sub _set_original($;$) {
    my ($self, $ref) = @_;
    $ref = $self->{_objects} if not defined($ref);

    foreach my $key ( keys %$ref ) {
        next if ref($ref->{$key}) ne 'HASH';
        if ( exists($ref->{$key}{-mod_id}) ) {
            $ref->{$key}{-mod_id} = 0;
        }

        # Process this object's children
        $self->_set_original($ref->{$key});
    }
}

#---------------------------------------------------------------------
# Internal function
# Convert all the size arguments to bytes
#---------------------------------------------------------------------
sub _process_size_units($) {
    my $args = shift;

    return unless defined $args;

    foreach my $item ( qw(size size_dev max_size offset offset_dev) ) {
        if ( exists $args->{$item} ) {
            $args->{$item} = _units($args->{$item});
        }
    }

    return;
}

#---------------------------------------------------------------------
# Internal function
# Returns true according to percentage. If percentage is not defined
# it assumes 100% so returns true as default unless the flag is
# defined on the global hash
#---------------------------------------------------------------------
sub _set_on_pct($$) {
    my ($mod, $opt) = @_;
    my $pct = $opt . '_pct';
    my $num = $opt . '_num';
    my $ret = exists $pct_ret{$pct} ? $pct_ret{$pct} : 1;

    # Check if this type of object needs to be modified
    if ( defined($mod->{$num}) && ! $mod->{$num} ) {
        return 0;
    }

    # Check if this object will be modified based on percentage
    if ( defined($mod->{$pct}) ) {
        $ret = ($mod->{$pct} > 0 && rand(100) < $mod->{$pct}) ? 1 : 0;
    }

    return $ret;
}

#---------------------------------------------------------------------
# Internal function
# Get the arguments for the modify mode and specify defaults
#---------------------------------------------------------------------
sub _get_args($) {
    my $mod = shift;
    my ($append, $name_reuse, %sattrs, $percent, $num, $name_size);
    my ($acl, $acl_masks, $acl_flags, $acl_users, $acl_types, $min_name);
    my ($streams, $set_acl, $pct, $sparse_opts, $sdata, $off, $data, $buf);
    my ($link_data, $rattrs);
    my $size   = 0;
    my $sparse = 0;
    my $link   = 0;
    my $slink  = 0;
    my $name = '';
    my $n = @val_char;
    my $blocks = 0;
    my @offsets = (0);
    my @attrs = qw(encrypt compress archive hidden);

    # Process percentage options
    $streams = _set_on_pct($mod, 'stream');
    $set_acl = _set_on_pct($mod, 'acl');
    $rattrs  = _set_on_pct($mod, 'attr');
    # If the sparse percentage is not defined --> set it to 0
    $sparse  = _set_on_pct($mod, 'sparse') if defined($mod->{sparse_opts});
    if ( ! $sparse ) {
        # Sparse has higher precedence over links
        if ( $mod->{_ref}{-dirs} || $mod->{_ref}{-files} ) {
            $link = _set_on_pct($mod, 'link');
            if ( ! $link ) {
                # Hard links have higher precedence over symbolic links
                $slink = _set_on_pct($mod, 'slink');
            }
        }
    }

    if ( $link || $slink ) {
        # Get name of file/directory to link to
        $data = undef;
        foreach ( keys %{$mod->{_ref}} ) {
            if ( ! /^-/ && ($slink || -f) ) {
                # Hard link only to a file
                $data = $_;
                last;
            }
        }
        if ( ! defined $data ) {
            # Could not find appropriate file
            $link = 0;
            $slink = 0;
        }
    }

    if ( $link || $slink ) {
        # Setup link data
        $link_data = { file => $data, symbolic => $slink ? 1 : 0 };
    } elsif ( $sparse ) {
        if ( ref($mod->{sparse_opts}) eq 'ARRAY' ) {
            # Sparse options could be an array ready for write_sparse
            $sparse_opts = $mod->{sparse_opts};
        } elsif ( ref($mod->{sparse_opts}) eq 'HASH' ) {
            # This is a hash so calculate array of data blocks and offsets
            # Calculate number of data segments
            $off = 0;
            $sdata = $mod->{sparse_opts};
            $num = _intRandGauss($sdata->{ndata}, $sdata->{ndata_dev});
            $num = 1 if $num < 1;
            foreach ( 1..$num ) {
                # For each data segment calculate the size
                $size = _intRandGauss($sdata->{size}, $sdata->{size_dev});
                $off += _intRandGauss($sdata->{offset}, $sdata->{offset_dev});
                $size = 1 if $size < 1;
                $data = '';
                while ( $size > 0 ) {
                    # Generate data block
                    $size -= _create_rand_data($buf, $size);
                    $data .= $buf;
                }
                # Save data block and offset on array
                push @$sparse_opts, { offset => $off, data => $data };
                undef $data;
            }
        }
    } elsif ( defined($mod->{size}) && defined($mod->{size_dev}) ) {
        # Calculate size if size_dev is given
        $size = _intRandGauss($mod->{size}, $mod->{size_dev});
    } elsif ( defined($mod->{size}) ) {
        # Get the size from the args
        $size = $mod->{size};
    } else {
        # Calculate size from max_size (default 1000)
        $size = defined($mod->{max_size}) ? $mod->{max_size} : 1000;
        $size = int(rand(1.1*$size) - 0.1*$size);
    }

    # Bound size
    $size = 0 if $size < 0;

    # Reuse previously deleted names
    $name_reuse = defined($mod->{name_reuse}) ? $mod->{name_reuse} : 0;

    if ( $name_reuse && @{$mod->{_rmlist}} ) {
        # Reuse a previously removed name
        $name = shift @{$mod->{_rmlist}};
    } else {
        if ( defined($mod->{name_size}) && defined($mod->{name_size_dev}) ) {
            # Both the size and standard deviation are provided,
            # use Gaussian distribution
            $name_size = _intRandGauss($mod->{name_size}, $mod->{name_size_dev});
        } elsif ( defined($mod->{name_size}) ) {
            # Get the name_size from the args
            $name_size = $mod->{name_size};
        } elsif ( defined($mod->{max_name_size}) ) {
            # Calculate name_size from max_name_size
            $name_size = int(rand($mod->{max_name_size}));
        } else {
            # Default
            $name_size = _intRandGauss(8, 2);
        }

        # Make sure name size is at least the minimum (default is 2)
        $min_name = exists $mod->{min_name_size} ? $mod->{min_name_size} : 2;
        $name_size = $min_name if $name_size < $min_name;

        # Try 100 times for a unique and valid name
        foreach ( 1..100 ) {
            # Create name
            foreach ( 1..$name_size ) {
                $name .= $val_char[int(rand($n))];
            }
            # Check for validity and uniqueness
            if ( ! $invalid_filenames{$name}
              && (! $mod->{_ref} || ! exists $mod->{_ref}{$name}) ) {
                # Found it
                last;
            } else {
                # Reset name and look for another
                $name = '';
            }
        }
        croak "Could not find a unique name" if length($name) == 0;
    }

    # Get the append flag from the args or use default
    $append = defined($mod->{append}) ? $mod->{append} : 0;

    # Get attributes to be set
    %sattrs = map { $_ => $mod->{$_} } grep { defined $mod->{$_} } @attrs;
    foreach ( @attrs ) {
        $pct = $_ . '_pct';
        next if ! defined $mod->{$pct};
        $num = _set_on_pct($mod, $_);
        if ( $num ) {
            $sattrs{$_} = 1 if ! exists $sattrs{$_};
        } else {
            delete $sattrs{$_} if exists $sattrs{$_};
        }
    }

    # Encrypt and compress are mutually exclusive
    if ( $sattrs{encrypt} && $sattrs{compress} ) {
        delete $sattrs{compress};
    }

    if ( $mod->{_ref}{-encrypt} && $sattrs{compress} ) {
         # If parent is encrypted, the file will be encrypted
         delete $sattrs{compress};
    }

    if ( $mod->{_ref}{-compress} && $sattrs{encrypt} ) {
         # If parent is compressed, the file will be compressed
         delete $sattrs{encrypt};
    }

    # Get list of offsets for modification
    if ( defined($mod->{blocks}) ) {
        @offsets = ();
        foreach ( 1..$mod->{blocks} ) {
            push @offsets, int(rand($mod->{max_size}));
        }
        $size = defined($mod->{size}) ? $mod->{size} : 1000;
        $blocks = 1;
        $append = 0;
    }

    # Set/modify ACL on file/directory
    $acl       = defined($mod->{acl}) && $set_acl ? $mod->{acl} : 0;
    $acl_masks = defined($mod->{acl_masks}) ? $mod->{acl_masks} : 8;
    $acl_flags = defined($mod->{acl_flags}) ? $mod->{acl_flags} : 4;
    $acl_types = defined($mod->{acl_types}) ? $mod->{acl_types} : 2;
    $acl_users = defined($mod->{acl_users}) ? $mod->{acl_users} : 2;

    # Percent defaults to 0
    $percent = defined($mod->{percent}) ? $mod->{percent} : 0;

    return { size        => $size,
             streams     => $streams,
             append      => $append,
             rattrs      => $rattrs,
             sattrs      => \%sattrs,
             sparse      => $sparse,
             sparse_opts => $sparse_opts,
             link        => $link || $slink,
             link_data   => $link_data,
             name        => $name,
             blocks      => $blocks,
             offsets     => \@offsets,
             percent     => $percent,
             acl         => $acl,
             acl_masks   => $acl_masks,
             acl_flags   => $acl_flags,
             acl_types   => $acl_types,
             acl_users   => $acl_users,
             users       => $mod->{users},
             name_reuse  => $name_reuse };
}

#---------------------------------------------------------------------
# It creates a unique ACL on a file or modifies the contents of the ACL
#---------------------------------------------------------------------
sub _modify_acl($$$;$$) {
    my ($self, $file, $args, $ref, $modify) = @_;
    my (@mlist, @flist, @ulist, @tlist, $mask, $flag, $user, $type);
    my ($object, @dacl_changes, @sacl_changes, $ace, $ace_curr, $index);
    my ($new_dacl, $new_sacl, @dacl_aces, @sacl_aces, @aces, $entry, $cache);
    my $retry  = 20;
    my $mode   = $args->{acl};
    my @users  = defined($args->{users}) ? @{$args->{users}} : @ace_users;
    my $ntypes = $args->{acl_types} ? $args->{acl_types} : @ace_types;
    my @types  = @ace_types[0..$ntypes-1];
    $modify = 0 if not defined($modify);

    croak "Add/modify ACL is only supported on Windows"
      if $self->{_hosttype} ne 'windows';

    # Get which access, flags, types and users to use for the ACEs
    while ( $retry > 0 && $modify < 2 ) {
        # Get list from ace mask flags
        @mlist = _get_from_list(@ace_masks, $args->{acl_masks}, $mode);
        $entry = join('-', @mlist);

        # Get list from ace flags
        @flist = _get_from_list(@ace_flags, $args->{acl_flags}, $mode);
        $entry .= ':' . join('-', @flist);

        # Get list from ace types
        @tlist = _get_from_list(@types, $args->{acl_types}, $mode);
        $entry .= ':' . join('-', @tlist);

        # If ACE type is DENIED then do not select Users or Administrators
        if ( grep { /DENIED/ } @types[@tlist] ) {
            # Remove the Users and Administrators groups from the list
            @users = grep {!/^(users|administrators)$/i} @users;
            # Add the Guests group to the list if empty
            push @users, 'Guests' if !@users;
        }

        # Get list from ace users
        @ulist = _get_from_list(@users, $args->{acl_users}, $mode);
        $entry .= ':' . join('-', @ulist);

        if ( ! $acls{$entry} ) {
            # Add format of ACL to list of created ACLs
            $acls{$entry} = 1;
            last;
        }
        --$retry;
    }

    croak "Could not create a unique ACL" if ($retry < 1);

    if ( $cifs_security ) {
        # Load modules needed for ACL only once
        require Win32::Security::ACE;
        require Win32::Security::ACL;
        require Win32::Security::NamedObject;
        $cifs_security = 0;
    }

    if ( $modify < 2 ) {
        # Create a list of ACEs
        foreach $user ( @users[@ulist] ) {
            foreach $mask ( @ace_masks[@mlist] ) {
                foreach $flag ( @ace_flags[@flist] ) {
                    foreach $type ( @types[@tlist] ) {
                          $Log->debug("ACL: user: $user mask: " .
                           "$mask flag: $flag type: $type");
                        # Create ACE
                        push @aces, Win32::Security::ACE->new('FILE',
                                                              $type,
                                                              $flag,
                                                              $mask,
                                                              $user);
                    }
                }
            }
        }
    }

    # Get security for file/directory
    $object = Win32::Security::NamedObject->new('FILE', $file);

    if ( $modify ) {
        # Delete previous entry -- for modify (1) or remove (2)
        delete $acls{$ref->{$file}{-acl}} if exists($ref->{$file}{-acl});

        if ( $modify == 1 ) {
            # Modifying the ACL: remove all inherited ACEs as well
            # as a percentage as requested
            @dacl_aces = grep { ! $_->aceFlags->{INHERITED}
                                && (100-$args->{percent}) > rand(100) }
                                $object->dacl->aces();
            ###@sacl_aces = grep { ! $_->aceFlags->{INHERITED}
            ###                    && (100-$args->{percent}) > rand(100) }
            ###                    $object->sacl->aces();
            @sacl_aces = ();   ### TODO remove this line if uncomment above ###
        }

        # Remove duplicate ACEs
        foreach $ace_curr ( @dacl_aces, @sacl_aces ) {
            $index = 0;
            foreach $ace ( @aces ) {
                if ( $ace->aceType() eq $ace_curr->aceType()
                     && $ace->_sid() eq $ace_curr->_sid() ) {
                    # This ACE is already in the ACL so remove it
                    splice @aces, $index, 1;
                    last;
                }
                $index++;
            }
        }
    }

    # Separate ACE by type (DACL: Discretionary or SACL: System audit)
    foreach $ace ( @aces ) {
        if ( $ace->aceType() eq 'SYSTEM_AUDIT_ACE_TYPE' ) {
            push @sacl_changes, $ace;
        } else {
            push @dacl_changes, $ace;
        }
    }

    # Create an object for each type with current ACEs less the
    # ones that were removed
    if ( @dacl_changes || @dacl_aces ) {
        $new_dacl = Win32::Security::ACL->new('FILE', @dacl_aces);
        # Add the ACEs to the ACL
        $new_dacl->addAces(@dacl_changes);
        $object->dacl($new_dacl, 'UNPROTECTED_DACL_SECURITY_INFORMATION');
    }
    if ( @sacl_changes || @sacl_aces ) {
        $new_sacl = Win32::Security::ACL->new('FILE', @sacl_aces);
        # Add the ACEs to the ACL
        $new_sacl->addAces(@sacl_changes);
        ### TODO uncomment below if uncomment line 3285
        ###$object->sacl($new_sacl, 'UNPROTECTED_SACL_SECURITY_INFORMATION');
    }

    # Clear the ACL cache on the module
    # The module saves every ACE and ACL created for future use and after
    # creating 10000 ACLs the memory required is about 500MB

    # Clear ACE cache
    $cache = Win32::Security::ACE::SE_FILE_OBJECT->_rawAceCache();
    delete $cache->{$_} foreach keys %$cache;

    # Clear ACL cache
    $cache = Win32::Security::ACL::SE_FILE_OBJECT->_rawAclCache();
    delete $cache->{$_} foreach keys %$cache;

    return $entry;
}


#---------------------------------------------------------------------
# It selects N (random) elements at random from the array and returns
# this list keeping the same order as the original list
#---------------------------------------------------------------------
sub _get_from_list(\@$$) {
    my ($list, $len, $mode) = @_;
    my (%list_h, @list_a, $m);
    my $retry = 10;
    # Number of elements in list
    my $n = @{$list};

    $len = $n if $len > $n;

    # Number of elements to get from list
    if ( $mode == 2 ) {
        $m = $len;
    } else {
        $len = $n if $len == 0;
        $m = 1 + int(rand($len));
    }

    while ( $retry > 0 ) {
        # Weigh each element in the array
        %list_h = map { rand() => $_ } ( 0..$n-1 );
        # Shuffle the list according to the weights
        @list_a = map { $list_h{$_} } sort { $a <=> $b } keys %list_h;
        # Check if the list is complete. On Windows the random
        # number generator produces same numbers back to back
        if ( @list_a == $n ) {
            # Get only the requested number of elements
            # and re-order the elements
            @list_a = sort { $a <=> $b } @list_a[0..$m-1];
            last;
        }
        --$retry;
    }

    croak "Could not get items from acl list" if ($retry < 1);

    return @list_a;
}

#---------------------------------------------------------------------
# Get a list of objects of an specific type
#---------------------------------------------------------------------
sub _get_objects($$$$;$) {
    my $self = shift;
    my ($objects, $base, $ref, $type) = @_;
    my ($object, $key);
    $type = '' if not defined($type);

    foreach $key ( keys %$ref ) {
        # Skip internal keys
        next if $key =~ /^-/o;

        # Absolute path of object
        $object = File::Spec->catfile($base, $key);
        # Add object to list if it is of specified type
        push @$objects, $object if ($ref->{$key}{-type} eq $type);

        # Do not process further this object since
        # it does not have any children
        next if ref($ref->{$key}) ne 'HASH';

        # Process this object's children
        $self->_get_objects($objects, $object, $ref->{$key}, $type);
    }
}

#---------------------------------------------------------------------
# Get total counters by traversing the objects database
#---------------------------------------------------------------------
sub _get_totals($$;$) {
    my ($self, $ref, $total) = @_;

    foreach my $key ( keys %$ref ) {
        next if ref($ref->{$key}) ne 'HASH';

        if ( $ref->{$key}{-sparse} ) {
            $total->{sparse}++;
        } elsif ( $ref->{$key}{-link} ) {
            if ( $ref->{$key}{-link} == 1 ) {
                $total->{links}++;
            } else {
                $total->{slinks}++;
            }
        } elsif ( $ref->{$key}{-type} eq 'd' ) {
            $total->{dirs}++;
        } elsif ( $ref->{$key}{-type} eq 'f' ) {
            $total->{files}++;
        } elsif ( $ref->{$key}{-type} eq 's' ) {
            $total->{streams}++;
        }

        if ( $ref->{$key}{-acl} ) {
            $total->{acls}++;
        }

        $self->_get_totals($ref->{$key}, $total);
    }

    return $total;
}

#---------------------------------------------------------------------
# Create a block of random data
#---------------------------------------------------------------------
sub _create_rand_data(\$$) {
    my ($cont, $size) = @_;
    my ($bsize, $n);
    my $block = '';

    if ( $size < 1 ) {
        $$cont = '';
        return 0;
    }

    # Use the specified size up to the maximum write size
    $bsize = $size < $MAX_WR_SIZE ? $size : $MAX_WR_SIZE;
    # Create data block of a maximum of DATA_BLOCK bytes
    $n = $DATA_BLOCK < $bsize ? $DATA_BLOCK : $bsize;

    # Create the block of random data
    $block .= pack('C', int(rand(255))) foreach (1..$n);

    # Put as many data blocks up to the maximum write size
    $$cont = $block x int($bsize/$n);
    $n = $bsize % $n;
    $$cont .= substr($block, 0, $n) if $n;

    return $bsize;
}

#---------------------------------------------------------------------
# Create a sparse file
#---------------------------------------------------------------------
sub _create_sparse($$$) {
    my ($entry, $args, $ref) = @_;

    write_sparse(file=>$entry, data=>$args->{sparse_opts});
    $ref->{$entry}{-sparse} = [];
    foreach my $item ( @{$args->{sparse_opts}} ) {
        push @{$ref->{$entry}{-sparse}}, {
            offset => $item->{offset},
            size   => length($item->{data}),
        }
    }

    return;
}

#---------------------------------------------------------------------
# Set windows attributes on file or directory
#---------------------------------------------------------------------
sub _set_attributes($$$$) {
    my ($object, $sattrs, $ref, $args) = @_;
    my $msg = "Changing attributes on $object:";
    ( $msg .= " $_=$sattrs->{$_}" ) foreach ( keys %$sattrs );
    $Log->debug($msg);

    # Set attributes
    setattr_file(file=>$object, %$sattrs);

    foreach my $attr ( keys %$sattrs ) {
        # Set type of attributes for objects
        $ref->{$object}{-$attr} = $sattrs->{$attr};

        # Decrement counters
        $args->{$ref . '_num'}--;
    }
}

#---------------------------------------------------------------------
# Gets the reference for the specified file system object
# It must be relative to the path specified in constructor
#---------------------------------------------------------------------
sub _get_ref($$) {
    my ($self, $obj) = @_;
    my ($object, $dir);

    my $ref = $self->{_objects}{'.'};

    $obj = '' if $obj eq '.';

    # Get object's path relative to object's root
    
    $object = File::Spec->catfile($self->{_cwd}, $obj);
    
    $object = File::Spec->abs2rel($object, $self->{path});

    # Remove leading '..\'
    # File::Spec->abs2rel fails when the base is a drive 'Z:'
    $object =~ s/^\.\.\\?//;
    
    # If abs2rel() told us that we're operating in the current dir, just
    # return $ref unchanged; skip the loop below.
    return $ref if ($object eq q{.});

    # Traverse the object's hash and return the reference
    foreach $dir ( split(/[\/\\]+/, $object) ) {
        croak "Couldn't find table entry for object <$object> (at elem $dir)"
          if not exists($ref->{$dir});
        $ref = $ref->{$dir};
    }
    return $ref;
}

#---------------------------------------------------------------------
# Get the type string of the object
#---------------------------------------------------------------------
sub _get_type($) {
    my $object = shift;
    my %obj_types = (
        s => [ 'stream',    VERBOSE_STREAMS ],
        f => [ 'file',      VERBOSE_FILES ],
        d => [ 'directory', VERBOSE_DIRS ],
    );

    my @ret = $object->{'-sparse'} ? ('sparse file', VERBOSE_SPARSE) :
              $object->{'-link'} ?
              ($object->{'-link'} == 1 ? 'link':'symbolic link', VERBOSE_FILES):
              @{$obj_types{$object->{'-type'}}};

    if ( wantarray ) {
        return @ret;
    } else {
        return $ret[0];
    }
}

#---------------------------------------------------------------------
# Get the file name of the object
#---------------------------------------------------------------------
sub _get_file($$) {
    my ($base, $key) = @_;

    return $base eq '.' ? $key : $key eq '.' ? $base :
           $base eq ''  ? $key : $key eq ''  ? $base :
           File::Spec->catfile($base, $key);
}

#---------------------------------------------------------------------
# Units conversion
#---------------------------------------------------------------------
sub _units($) {
    my $value = shift;
    my ($val, $m, $n, $b, $mult);

    return unless defined $value;

    ($val, $n, $m, $b) = ($value =~ /(-?\d*\.?\d*)\s*((.?)(.*))/);
    croak "Value must be a number <$value>"
      unless (defined $val && $val ne '' && $val ne '.');
    $mult = $units_table{lc($m)} if $m;
    if ( ($n && !defined $mult) || ($b && lc($b) ne 'b') ) {
        croak "Bad units modifier for <$value>";
    }

    if ( defined $mult && $mult > 1 ) {
        $val *= $mult;
    }

    return $val;
}

#---------------------------------------------------------------------
# Changes to given directory, saving the current working directory
# so it could be restored with _chdir_restore
#---------------------------------------------------------------------
sub _chdir($$) {
    my ($self, $dir) = @_;
    my $cwd;

    if ( ! defined($self->{_cwd}) ) {
        # Make sure chdir is relative to path
        # so prefix the object's path
        $dir = File::Spec->catfile($self->{path}, $dir);
        $cwd = getcwd() or croak "Could not determine current directory: $!";
    }

    if ( $dir =~ /^\.?$/ ) {
        # Do not chdir if '.' but make an entry
        # on list for _chdir_restore
        push @{$self->{_prev_cwd}}, '';
        return;
    }

    # Save the absolute path for current directory
    push @{$self->{_prev_cwd}}, defined $cwd ? $cwd : $self->{_cwd};

    # Change to directory
    $Log->debug( "Changing directory to $dir" );
    chdir $dir or croak "Could not change to directory $dir: $!";

    # Set new current working directory
    if ( defined($self->{_cwd}) ) {
        $self->{_cwd} = File::Spec->catfile($self->{_cwd}, $dir) if $dir ne '';
    } else {
        $self->{_cwd} = $dir;
    }

    return;
}

#---------------------------------------------------------------------
# Changes back to previous directory
#---------------------------------------------------------------------
sub _chdir_restore($) {
    my $self = shift;
    my $prev_dir;

    if ( defined($self->{_prev_cwd}) ) {
        # Get last directory change
        $prev_dir = pop @{$self->{_prev_cwd}};
        return if not $prev_dir;

        # Change back to previous directory
        $Log->debug("Changing directory to $prev_dir --restore--");
        chdir $prev_dir or croak "Could not change to directory $prev_dir: $!";

        # Set new current working directory
        $self->{_cwd} = $prev_dir;
    }

    return;
}

#---------------------------------------------------------------------
# Save database to root directory of tree
#---------------------------------------------------------------------
sub _save_db($) {
    my $self = shift;
    my $dbfile = $self->{db_file}; #File::Spec->catfile($self->{path}, $DB_FILE);

    sysopen(FH, $dbfile, O_WRONLY|O_BINARY|O_CREAT|O_TRUNC)
      or croak "Could not create db file: $!";
    print FH nfreeze($self) or croak "Could not save db file: $!";
    close(FH);
}

#---------------------------------------------------------------------
# Read database from root directory of tree
#---------------------------------------------------------------------
sub _get_db($) {
    my $self = shift;
    my $dbfile = $self->{db_file}; # File::Spec->catfile($self->{path}, $DB_FILE);
    my ($buffer, $n);
    my $nread = 0;
    my @ckeys = qw(
        _srand
        _remove
        _mod_id
        _objects
        _counters
    );

    # Open db file
    sysopen(FH, $dbfile, O_RDONLY|O_BINARY)
      or croak "Could not open db file: $!";
    # Read the entire file into buffer
    while ( ($n = sysread(FH, $buffer, 10000, $nread)) ) {
        $nread = length($buffer);
    }
    croak "Could not read db file: $!" if not defined($n);
    close(FH);

    # Thaw the contents of file into a blessed object
    $n = thaw($buffer);

    # Copy all keys from saved object to current object
    ($self->{$_} = $n->{$_}) foreach @ckeys;

    return 1;
}

#---------------------------------------------------------------------
# Return the integer of a random number from a Gaussian distribution
# centered about the given mean with the given standard deviation.
# The number is incremented with a uniform probability if the random
# number is not exactly an integer.
#---------------------------------------------------------------------
sub _intRandGauss($$) {
    my ($mean, $stddev) = @_;
    my ($num, $int_num, $fraction);

    $num = _randGauss($mean, $stddev);
    $int_num = int($num);
    $fraction = $num - $int_num;
    if ( $fraction > 0 ) {
        $int_num++ if rand() < $fraction;
    }

    return $int_num;
}

#---------------------------------------------------------------------
# Return a random number from a Gaussian distribution centered
# about the given mean with the given standard deviation.
#---------------------------------------------------------------------
{
    my $spare = undef; # static variable across calls to _randGauss()

    sub _randGauss($$) {
        my ($mean, $stddev) = @_;
        return $mean unless ($stddev);

        my $r;

        if ($spare) {
            $r = $spare;
            $spare = undef;
        } else {
            # Get two random numbers from the Gaussian distribution
            # centered at zero with a standard deviation of 1.
            my ($x1, $x2, $w, $y1, $y2);
            do {
                $x1 = 2.0 * rand() - 1.0;
                $x2 = 2.0 * rand() - 1.0;
                $w = ($x1 * $x1) + ($x2 * $x2);
            } while ( $w >= 1.0 );
            $w = sqrt( ( -2.0 * log($w) ) / $w );
            $y1 = $x1 * $w;
            $y2 = $x2 * $w;

            $r = $y1;
            $spare = $y2;
        }

        # Map the random number into the desired distribution.
        $r = ($r * $stddev) + $mean;
        return $r;
    }
}

sub _splice_prev_cwd {

    my $obj = shift;
    splice(@{$obj->{_prev_cwd}}, 1) if @{$obj->{_prev_cwd}} > 1
}

sub _undef_cwd {
    my $self = shift;
    undef $self->{_cwd};
}

#---------------------------------------------------------------------
# Traverse the object's hash and process each item at a time
# This is a recursive function
#---------------------------------------------------------------------
sub _process_objects($$$$$) {
    my $self = shift;
    my ($base, $ref, $mod, $depth) = @_;
    my ($more, $key, $ndepth, $ret);
    my @keys = keys %$ref;

    # Process all objects in current object
    # Not using 'keys %$ref' on the foreach to avoid processing
    # the same element twice since adding an entry to the hash
    # results in the keys to be rearranged in an unpredictable way
    foreach $key ( @keys ) {
        # Skip internal keys
        next if $key =~ /^-/o;
        # Return if limit is hit except if there are any removes
        last if $self->{_fillpct} && !$self->{d_remove}
          && $self->{d_used} > $self->{d_max};

        # Process the current object
        $ret = $self->_process_object($base, $ref, $mod, $key, $depth);

        # Change current object name if it was renamed
        $key = $ret if defined $ret;

        # Check if done
        # Calculate if there is any items left to be processed
        # (if any of the counts are not zero)
        $more = 0;
        $more += $mod->{$_}{number} foreach ( keys %$mod );
        last if not $more;
        # Check if current object was not removed/renamed
        next if not exists($ref->{$key});

        # Do not traverse beyond a stream
        # Streams do not own any objects
        next if $ref->{$key}{-type} eq 's';

        $ndepth = $depth;
        if ( $ref->{$key}{-type} eq 'd' ) {
            # Change to directory to process its children
            $self->_chdir($key);
            $ndepth++;
        }

        # Process objects owned by this object
        $self->_process_objects($key, $ref->{$key}, $mod, $ndepth);

        if ( $ref->{$key}{-type} eq 'd' ) {
            # Change back directories
            $self->_chdir_restore();
        }
    }
}

#---------------------------------------------------------------------
# Process each object according to the modify hash of options
#---------------------------------------------------------------------
sub _process_object($$$$$$) {
    my $self = shift;
    my ($base, $ref, $mod_obj, $obj, $depth) = @_;
    my ($mod, $args, $size, $cont, $name, $cmd, $out, $offset, $ret, $n, @list);
    my ($otype, %free, $du_diff, $du_need, %mopts);
    my @obj_attrs = qw(dirs files streams sparse compress encrypt hidden acl);
    my %obj_types = (s => 'streams', f => 'files', d => 'dirs');
    my $type = $obj_types{$ref->{$obj}{-type}};
    # Use the absolute path for the directory, operations on the
    # current directory fail for windows
    my $stream_base = $ref->{-type} eq 'd' ? $self->{_cwd} : $base;
    $stream_base =~ s/\\*\/*$//;

    # No modify arguments for this type of object
    return if not defined($type);
    return if not defined($mod_obj->{$type});

    if ( $self->{_fillpct} && $self->{_modcnt} > $self->{_mcdiff}) {
        # Get used space on volume
        %free = vol_free(vol => $self->{d_path});
        # Get difference in space on this iteration for this count
        $du_diff = $free{used} - $self->{d_used};

        # Return if limit is hit
        $self->{d_used} = $free{used};
        return if $self->{d_used} > $self->{d_max};

        # Get the space still available to use
        $du_need = $self->{d_max} - $self->{d_used};
        # Adjust how often to poll the system for the used space
        # it depends on the space difference for this iteration
        $self->{_mcdiff} *= ($du_need/10) / $du_diff if $du_diff > 0;
        $self->{_mcdiff} = 300 if $self->{_mcdiff} > 300;
        $self->{_mcdiff} = 1 if $self->{_mcdiff} < 1;
        $self->{_modcnt} = 0;
    }

    # Modify hash for this object
    $mod = $mod_obj->{$type};

    # Do not modify root directory
    return if ($obj eq '.' && $mod->{skiproot});

    # Object has been modified already
    if ( $ref->{$obj}{-mod_id} == $self->{_mod_id} ) {
        return;
    }

    # No more objects of this type to modify
    return if $mod->{number} < 1;

    if ( $type ne 'streams' ) {
        # On Windows the archive bit is set by default on files
        $n = exists($ref->{$obj}{-archive}) ? $ref->{$obj}{-archive} :
             $self->{_hosttype} eq 'windows' ? $type eq 'files' ? 1 : 0 : 0;
        if ( $mod->{archive} ne 'any' && ($mod->{archive} eq 'yes' xor $n) ) {
            # Object does not match required archive attribute
            return;
        }

        foreach $n ( @obj_attrs ) {
            if ( defined($mod->{$n}) && $mod->{$n} ne 'any'
                 && ($mod->{$n} eq 'yes' xor $ref->{$obj}{-$n}) ) {
                # Object does not match required attributes
                return;
            }
        }
    }

    if ( $obj eq '.' && ($mod->{mode} eq 'remove' || $mod->{mode} eq 'sattrs'
      || $mod->{mode} eq 'rename' || $mod->{mode} eq 'copy') ) {
        # Do not remove/rename/copy/sattrs the top level directory
        return;
    }

    # Modify original objects only
    if ( $mod->{original} && $ref->{$obj}{-mod_id} ) {
        return;
    }

    if ( $mod->{skip} > 0 ) {
        # Skip the first objects
        --$mod->{skip};
        return;
    }

    if ( $ref->{$obj}{-link} && ( $mod->{mode} eq 'sattrs'
      || !( $mod->{mode} eq 'remove' || $mod->{mode} eq 'rename' ||
            $mod->{mode} eq 'read' ||
            ( ($mod->{mode} eq 'trunc' || $mod->{mode} eq 'modify')
              && -e $obj && ! -d $obj ) ) ) ) {
        # Remove and rename links unconditionally
        # Modify/truncate symbolic links to files
        # Do not modify/truncate symbolic links to directories
        # Do not set attributes on a link
        return;
    }

    # Check if object is not going to be modified
    # according to percent argument
    return if $mod->{percent} < rand(100);

    # Find file type
    if ( $ref->{$obj}{-sparse} ) {
        $otype = 'sparse';
    } elsif ( $ref->{$obj}{-link} ) {
        $otype = $ref->{$obj}{-link} == 1 ? 'links' : 'slinks';
    } else {
        $otype = $type;
    }

    if ( $mod->{mode} eq 'remove' && $type =~ /^(streams|files|dirs)$/ ) {
        # Remove object
        if ( $type eq 'streams' ) {
            $Log->debug("Remove stream $obj from $stream_base");
            # Remove stream
            remove_stream(file=>$stream_base, stream=>$obj);
            push @{$self->{_remove}{streams}}, $obj;
        } else {
            # Remove file or directory
            $n = ($type eq 'dirs') ? 1 : 0;
            if ( $n ) {
                $Log->debug('Remove directory ' . $obj);
            } else {
                $Log->debug('Remove file ' . $obj);
            }
            remove_file(file=>$obj, recursive=>$n);
            if ( $n ) {
                push @{$self->{_remove}{dirs}}, $obj;
            } else {
                push @{$self->{_remove}{files}}, $obj;
            }
        }
        --$ref->{-$type};
        delete $ref->{$obj};
        $self->{_counters}{remove}{$otype}++;
    } elsif ( $mod->{mode} eq 'trunc' && $type =~ /^(streams|files)$/ ) {
        # Truncate object
        $args = _get_args($mod->{args});
        $size = $args->{size};
        $self->{_modcnt}++;
        if ( $type eq 'streams' ) {
            $Log->debug("Truncate stream $obj on file $stream_base");
            # Truncate stream
            setattr_stream(file=>$stream_base, stream=>$obj, size=>$size);
        } elsif ( $type eq 'files' ) {
            $Log->debug("Truncate file $obj");
            # Truncate file
            truncate $obj, $size;
        }
        $ref->{$obj}{-mod_id} = $self->{_mod_id};
        $self->{_counters}{trunc}{$otype}++;
    } elsif ( $mod->{mode} eq 'rename' && $type =~ /^(files|dirs)$/ ) {
        # Rename object
        $args = _get_args($mod->{args});
        $name = $args->{name};
        if ( $type eq 'dirs' ) {
            $Log->debug("Rename directory $obj to $name");
        } else {
            $Log->debug("Rename file $obj to $name");
        }
        rename $obj, $name or croak "Could not rename $obj: $!";
        $ref->{$name} = $ref->{$obj};
        delete $ref->{$obj};
        $ref->{$name}{-mod_id} = $self->{_mod_id};
        $ret = $name;
        $self->{_counters}{rename}{$otype}++;
    } elsif ( $mod->{mode} eq 'copy' && $type =~ /^(files|dirs)$/ ) {
        $self->{_modcnt}++;
        # Copy object
        if ( $self->{_hosttype} =~ /^windows/ ) {
            $cmd = $type eq 'files' ? 'copy' : 'xcopy /q/i';
        } elsif ( $self->{_hosttype} =~ /solaris/ ) {
            $cmd = $type eq 'files' ? 'cp -@' : 'cp -r@';
        } else {
            croak "Copy is not supported for $self->{_hosttype}";
        }
        $args = _get_args($mod->{args});
        $name = $args->{name};
        if ( $type eq 'dirs' ) {
            $Log->debug("Copy directory $obj to $name");
        } else {
            $Log->debug("Copy file $obj to $name");
        }
        $out = qx($cmd $obj $name 2>&1);
        croak $out if $?;
        $ref->{$name} = $self->_copy_ref($ref->{$obj});
        $ref->{$name}{-mod_id} = $self->{_mod_id};
        $self->{_counters}{copy}{$otype}++;
    } elsif ( $mod->{mode} eq 'create' && $type =~ /^(files|dirs)/ ) {
        # Get optional arguments
        $args = _get_args($mod->{args}{args});

        # Create streams on file/directory
        if ( $args->{streams} && defined($mod->{args}{streams})
          && $mod->{args}{args}{stream_num} ) {
            $self->_create_streams(file=>$obj, %{$mod->{args}{streams}});
            $mod->{args}{args}{stream_num}--;
        }
        if ( $type eq 'dirs' ) {
            # Create files on directory
            if ( defined($mod->{args}{files}) ) {
                $self->_create_files(dir=>$obj, %{$mod->{args}{files}});
            }
            if ( $mod->{depth} || $ref->{$obj}{-dirs} ) {
                $self->_create_dirs(dir=>$obj, %{$mod->{args}});
            }
        }
    } elsif ( $mod->{mode} eq 'modify' && $type =~ /^(streams|files)/ ) {
        # Get size and append flag from arguments
        $args = _get_args($mod->{args});
        $self->{_modcnt}++;
        if ( $ref->{$obj}{-sparse} ) {
            if ( $args->{sparse_opts} ) {
                $Log->debug("Modify sparse file $obj");
                _create_sparse($obj, $args, $ref);
                $mod->{args}{sparse_num}--;
                $self->{_counters}{modify}{sparse}++;
            }
            return;
        }

        if ( $type eq 'streams' ) {
            $Log->debug("Modify stream $obj on file $stream_base");
        } else {
            $Log->debug("Modify file $obj");
        }

        foreach $out ( @{$args->{offsets}} ) {
            $offset = $out;
            %mopts = $args->{blocks} ? (offset => $offset) : ();
            $n = $args->{append};
            $size = $args->{size};
            if ( $type eq 'streams' ) {
                # Modify stream
                do {
                    $size -= _create_rand_data($cont, $size);
                    write_stream(file     => $stream_base,
                                 stream   => $obj,
                                 contents => $cont,
                                 append   => $n,
                                 %mopts);
                    if ( $args->{blocks} ) {
                        $n = 0;
                        $offset += length($cont);
                    } else {
                        $n = 1;
                    }
                } while ( $size > 0 );
            } else {
                # Modify file
                $n = $args->{blocks} ? 0 : $n ? O_APPEND : O_TRUNC;
                sysopen(FH, $obj, O_WRONLY|O_BINARY|$n)
                  or croak "Could not open file $obj: $!";
                seek(FH, $offset, 0) if $args->{blocks};
                while ( $size > 0 ) {
                    $size -= _create_rand_data($cont, $size);
                    print FH $cont or croak "Could not write to file $obj: $!";

                }
                close(FH);
            }
        }
        $ref->{$obj}{-mod_id} = $self->{_mod_id};
        $self->{_counters}{modify}{$otype}++;
    } elsif ( $mod->{mode} eq 'sattrs' && $type =~ /^(files|dirs)/ ) {
        $args = _get_args($mod->{args});
        if ( %{$args->{sattrs}} ) {
            _set_attributes($obj, $args->{sattrs}, $ref, $mod->{args});
        }
        $ref->{$obj}{-mod_id} = $self->{_mod_id};
        $self->{_counters}{sattrs}{$otype}++;
    } elsif ( $mod->{mode} eq 'acl_mod' && $type =~ /^(files|dirs)/ ) {
        $args = _get_args($mod->{args});
        $Log->debug("Modify ACL on $type $obj");
        $ref->{$obj}{-acl} = $self->_modify_acl($obj, $args, $ref, 1);
        $ref->{$obj}{-mod_id} = $self->{_mod_id};
        $mod->{args}{acl_num}--;
        $self->{_counters}{modify}{acls}++;
    } elsif ( $mod->{mode} eq 'acl_del' && $type =~ /^(files|dirs)/ ) {
        $args = _get_args($mod->{args});
        $Log->debug("Delete ACL on $type $obj");
        $self->_modify_acl($obj, $args, $ref, 2);
        delete $ref->{$obj}{-acl};
        $ref->{$obj}{-mod_id} = $self->{_mod_id};
        $mod->{args}{acl_num}--;
        $self->{_counters}{remove}{acls}++;
    } elsif ( $mod->{mode} eq 'read' ) {
        $args = _get_args($mod->{args});
        if ( $type ne 'streams' ) {
            $n = ($type eq 'dirs') ? 'directory' : 'file';
            if ( $args->{rattrs} ) {
                $Log->debug( "Read attributes for $n $obj");
                lstat $obj;
                getattr_file(file => $obj);
                $mod->{args}{attr_num}--;
                $self->{_counters}{read}{attrs}++;
            }
            if ( $args->{acl} ) {
                $Log->debug("Read ACL for $n $obj");
                $self->_read_acl($obj);
                $mod->{args}{acl_num}--;
                $self->{_counters}{read}{acls}++;
            }
        }

        if ( $type eq 'dirs' ) {
            $Log->debug("Read directory $obj");
            if ( opendir DH, $obj ) {
                @list = readdir DH;
                closedir DH;
            }
        } elsif ( $type eq 'files' && $ref->{$obj}{-sparse} ) {
            if ( $args->{sparse} ) {
                $Log->debug("Read sparse file $obj");
                read_file(file=>$obj);
                $mod->{args}{sparse_num}--;
            }
            # Increment number since this is the counter for regular files
            # and will be decremented later and no regular file will be read
            $mod->{number}++;
        } elsif ( $mod->{contents} ) {
            if ( $type eq 'streams' ) {
                $Log->debug("Read stream $obj on file $stream_base");
                read_stream(file=>$stream_base, stream=>$obj);
            } elsif ( $otype eq 'links' ) {
                $Log->debug("Read link $obj");
                read_file(file=>$obj) if -e $obj;
            } elsif ( $otype eq 'slinks' ) {
                $Log->debug("Read symbolic link $obj");
                $out = readlink $obj;
            } elsif ( $type eq 'files' ) {
                $Log->debug("Read file $obj");
                read_file(file=>$obj);
            }
            if ( $mod->{number} < 2 ) {
                $n = 0;
                $out = 0;
                # Get remaining number of attributes to be read
                @list = keys %{$mod->{args}};
                map { $n   += $mod->{args}{$_} } grep { /_num$/ } @list;
                map { $out += $mod->{args}{$_} } grep { /_pct$/ } @list;
                if ( $n * $out != 0 ) {
                    $mod->{number}   = $MAX_NUM;
                    $mod->{percent}  = 100;
                    $mod->{contents} = 0;
                }
            }
        }
        $self->{_counters}{read}{$otype}++;
    } else {
        # Should never get here
        croak "Mode <$mod->{mode}> does support object type <$type>";
    }

    # Decrement counter of objects to modify of this type
    --$mod->{number};

    return $ret;
}

#---------------------------------------------------------------------
# Recursively copy an item (scalar, array ref or hash ref)
# Used in the 'copy' option for modify to copy the object's tree
#---------------------------------------------------------------------
sub _copy_ref($$) {
    my ($self, $this) = @_;
    my $new_ref;

    if ( not ref $this ) {
        # Scalar value
        return $this;
    } elsif ( ref $this eq 'ARRAY' ) {
        # Array: return a reference to a local array
        return [ map { $self->_copy_ref($_) } @$this ];
    } elsif ( ref $this eq 'HASH' ) {
        # Hash: return a reference to a local hash
        $new_ref = { map { $_ => $self->_copy_ref($this->{$_}) } keys %$this };
        $new_ref->{-mod_id} = $self->{_mod_id};
        return $new_ref;
    } else {
        # The key item from last iteration is at $_ coming
        # from the map function
        croak "Cannot copy <$_> of unknown type $this";
    }
}

#---------------------------------------------------------------------
# Function to execute the calling function locally or remotely
#---------------------------------------------------------------------
sub _do_call($$%) {

    my ($self, $function, %opts) = @_;
    my ($func_name, $out, $err, @results);

    # Set alarm handler
    local $SIG{ALRM} = sub { croak "Timeout executing function with timeout=$self->{timeout}" };

    # Set alarm for timeout
    alarm($self->{timeout}) if defined $self->{timeout};
    
    $self->_chdir('');

    # Call local function
    $out = eval { $self->$function(%opts) };
    $err = $@;

    # Restore previous directory
    eval {
        splice(@{$self->{_prev_cwd}}, 1) if @{$self->{_prev_cwd}} > 1;
        $self->_chdir_restore();
        undef $self->{_cwd};
    };
    alarm(0);
    croak $err if $err;
    croak $@ if $@;

    return $out;
}


#---------------------------------------------------------------------
# Destructor for object
# It terminates the RPC service on the client if this is the owner
# of such service
#---------------------------------------------------------------------
END {
    my $self = $object;
    my $dbfile;
    my $tobjs = 0;
    


    # Save current error
    local($@);
    #return if $self->{_pid} != $$;

    # Get number of top level objects
    if ( defined($self->{_objects}) ) {
        $tobjs = grep { ! /^-/o } keys %{$self->{_objects}{'.'}};
    }

    # Save the database file if needed
    if ( $tobjs > 0 && $self->{save} ) {
        $self->_save_db();
        $Log->debug("Saving database");
    }

    # Remove database file if there are no objects
    $dbfile = $self->{db_file}; 
    if ( ! $tobjs && -e $dbfile ) {
        unlink $dbfile
          or $Log->warn("Could not remove database file $dbfile: $!");
    }
}

1;
