#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary MTask SFcoD File Iterator Class-
## @author asada@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here
##############################################################################

package NACL::MTask::FileIterator::SfcodFileIterator;
use Moose;

use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::STask::Volume;
use NACL::STask::VolumeFile;
use NACL::STask::VolumeFileCopy;
use NATE::BaseException;
use NACL::APISet::Exceptions::ResponseException;
use NACL::GeneralUtils qw(random_name_generator);

use constant WAIT_TIME => '10';

# iterator functions and attributes are provided by the base class
extends qw (NACL::MTask::FileIterator::OperationalFileIterator);

#
# destination volume is a required field for file copy on demand iterator
#
has destination_volume => (
    is => 'ro',
    isa => 'NACL::C::Volume',
    required => 1,
);

# Overide parent function to do SFCoD operation on a file
override '_do_file_operation' => sub {
    my ($self, @files) = @_;
    $Log->enter() if $may_enter;
    my $volume = $self->destination_volume();
    my $create_destination_directory = $self->create_destination_directory();

    my $sfcod_dir;
    my $sfcod_destination_path;
    # create a destination directory
    if ($create_destination_directory) {

        # If we have previously created a directory use it for all the sfcod ops with this
        # iterator
        if ( $self->destination_directory() eq  '') {
            $sfcod_dir = random_name_generator(
                prefix => 'sfcod_dir',
                length => 128
            );
            $self->destination_directory($sfcod_dir);
        }
        else {
            $sfcod_dir = $self->destination_directory();
        }

        # call parents dir create method re-use if exists
        $self->_zapi_volume_create_dir($volume, $sfcod_dir);
        $sfcod_destination_path = "/vol/".$volume->volume()."/$sfcod_dir/";
    }
    else {
        # /vol/volname/filename for destination
        $sfcod_destination_path = "/vol/".$volume->volume()."/";
    }

    # create array of File objects for the copied Files
    my @dest_file_objs = ();
    my $dest_file_obj = undef;
    my @dest_file_paths = ();
    foreach my $file (@files){
        # Create a random name for copied file
        my $newfilename = random_name_generator(
            prefix => 'sfcod_file',
            length => 128
        );

        # /vol/volname/dest_dir/filename for destination
        my $copy_destination_path = $sfcod_destination_path."$newfilename";

        # create VolumeFile object for the copied file
        my $dest_file_obj = NACL::STask::VolumeFile->new(
            command_interface => $file->command_interface(),
            vserver => $volume->vserver(),
            path => $copy_destination_path,
        );

        push @dest_file_objs, $dest_file_obj;
    }

    my $command_interface = $files[0]->command_interface();
    # Make a copy of the objects array as the copy ondmeand api changes it.
    my @new_dest_files = @dest_file_objs;

    # Call file copy
    my $VolumeFileCopy_objs;
    while (!$VolumeFileCopy_objs) {
        try {
            $VolumeFileCopy_objs = NACL::STask::VolumeFileCopy->start(
                command_interface => $command_interface,
                'source-paths' => \@files,
                'destination-paths' => \@new_dest_files,
                nacltask_wait => 0,
                'report-completed' => 'true',
            );
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $ex = shift;
            # Cannot start file operation. Maximum simultaneous file operations reached.
            if ($ex->text() =~ /Maximum simultaneous file operations reached/i) {
                $Log->trace("Maximum number of operations running, not removing ".
                "the operation from the staging area");
                Tharn::sleep(WAIT_TIME);
            } else {
                $Log->exit() if $may_exit;
                @dest_file_objs = (); # reset
                $ex->throw();
            }
        };
    }

    $VolumeFileCopy_objs->wait_for_destination_ready();

    $Log->exit() if $may_exit;
    if ($self->group_operations_flag){
        return @dest_file_objs;
    } else {
        return $dest_file_objs[0];
    }
};

1;
=head1 NAME

NACL::MTask::FileIterator::SfcodFileIterator - Provide an iterator object from iterating and doing a
file ondemand copy operation.

=head1 SYNOPSIS

This is a File on demand copy iterator class and if used iterate and do a copy operation the files in
provided by input iterator.

extends NACL::MTask::FileIterator::SfcodFileIterator;

The class overides base functions to check if the Volume File object input is able to be copied and also overides
the operation function in this case a file copy operation is performed.

=head1 DESCRIPTION

This Class provides an iterator that for each next() function will do the file copy operation before return
the NACL::C::VolumeFile object for the copied file.

=head1 FUNCTIONS

=head2 new

Create a new iterator object instance and return it.

=over

=item C<< file_iterator >> An instance of a file iterator, it must support next
and has_next. The next method must return NACL::C::VolumeFile instances.

=back

=head2 has_next

Returns 1 if there is another "ondemand copy" file available to get via a call to next or
0 if there are no more files available.

=head2 next

Return the next "ondemand copied" file as a NACL::C::VolumeFile instance.

=head1 SEE ALSO

=cut
