#
# Copyright (c) 2014-20* NetApp Inc.
# All rights reserved.
#
## @summary An exception for when User doesnt have enough
##  permission to operate on the file/dir
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::MTask::FilerLogGather;
use NACL::C::Node;
use NACL::Transit;
use NACL::MTask::SCP;
use NATE::ParamSet;
use NATE::Time qw(timestamp_format);
use File::Spec;
use Data::Dumper;
use NACL::STask::NetworkInterface;
use NACL::CS::NetworkInterface;
use File::Path qw(make_path);
use Scalar::Util qw(blessed);

use feature 'state';
use Params::Validate qw(validate_with OBJECT SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::Transit::Exceptions::TransitException;
use NATE::BaseException;

use Class::MethodMaker [
    scalar => [ { type => 'NACL::C::CommandInterface::ONTAP' } , 'command_interface'],
    array  => 'files_to_copy',
    scalar => 'target_directory',
];

=head1 METHODS

=head2 new

    my $FLG = NACL::MTask::FilerLogGather->new(
        command_interface => $node,
    );

OR

    my $FLG = NACL::MTask::FilerLogGather->new(
        command_interface => $node,
        target_directory  => '/mroot/etc/',
    );

Creates a FilerLogGather object that can be used to transfer filer logs and cores to nate logs dir.
This task uses node machine to run scp command.

=over

=item Options

=over

=item C<< command_interface => $node_obj >>

(Required, isa NACL::C::CommandInterface::ONTAP)
The node_obj from which logs are collected.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=item C<< target_directory => $target_directory >>

(Optional)
default to ''
Set the source directory on Filer to get the logs and core files.

=back

=back

=cut

sub new {
    $Log->enter() if $may_enter;
    my ($class, @args) = @_;

    return $class if(blessed($class));

    state $spec = {
        files_to_copy     => { type => ARRAYREF, optional => 1 },
        target_directory  => { type => SCALAR,   optional => 1 },
        command_interface => {
            type => OBJECT,
            isa  => "NACL::C::CommandInterface::ONTAP",
        },
    };

    my $self = validate_with(
        params => \@args,
        spec   => $spec,
    );

    bless $self, $class;

    $Log->exit() if $may_exit;
    return $self;
}

=head2 copy_cores_and_logs

NACL::MTask::FilerLogGather->copy_cores_and_logs(
    command_interface => $node_obj,
    target_directory  => $src_dir,
    files_to_copy     => \@files
);

OR

$FGL->copy_cores_and_logs(
    files_to_copy    => \@files,
    target_directory => $tgt_dir
);

(Class or Object method)
Trasnfer logs and core files from Filer to nate log dir.

Note: If it fails while collecting logs for a node then it will
throw exception. If user doesnt want to fail immediately and want to
copy logs/cores for rest of the nodes then he has to handle it init
his script. Example is given below on how he can do it

foreach my $node (@nodes) {
   try {
      my = $recovery = NACL::MTask::FilerLogRecoveryMode->new(command_interface => $node);
      $recovery->ensure_mroot_available();

      my = $log_gather = NACL::MTask::FilerLogGather->new(
         command_interface => $node,
         target_directory  => ['/mroot/etc/crash', '/mroot/log', ],
      );
      $log_gather->copy_cores_and_logs();

   } catch NATE::BaseException with {
      my $ex = shift;
      $Log->error("Collecting logs for remaining nodes even though gathering failed for node ".$node->node());
      $Log->error(Dumper($ex));
   };
}

=over

=item Options

=over

=item C<< files_to_copy => ARRAYREF >>

( Optional)
Copies list of files on target host at set_directory_on_target directory.
By default all files inside /mroot/etc/mlog and /mroot/etc/crash will be
transferred.

=item C<< command_interface => $filer_node_object >>

(Required for class method call, isa NACL::C::CommandInterface::ONTAP)
The Node object of the filer.

=item C<< target_directory => $target_directory >>

(Optional)
default to ''
Set the source directory on Filer to get the logs and core files.

=back

=back

=cut

sub copy_cores_and_logs {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;

    $self = $self->new(%args);

    my $ci = $self->command_interface();
    my $client_dir = $self->_get_source_dir(); 
    my $filer_dir = $self->{target_directory} || "";
    my @files_to_copy = @{$self->{files_to_copy}};
    if(scalar(@files_to_copy) == 0) {
        @files_to_copy = ('/mroot/etc/log/mlog', '/mroot/etc/crash');
    }

    my @files = NACL::MTask::SCP->copy_to_localhost(
        source_command_interface => $ci,
        files_to_copy            => \@files_to_copy,
        set_directory_on_target  => $client_dir,
        set_directory_on_source  => $filer_dir,

    );

    $Log->exit() if $may_exit;

    return $client_dir;
}

sub _get_source_dir {
    my ($self) = @_;
    my $node = $self->command_interface->node();
    my $source_dir = $self->{source_directory} // NATE::ParamSet->param_global->get('TOP_LOGDIR');

    my $time = timestamp_format(time());
    $source_dir = File::Spec->catfile($source_dir, "FilerCoresAndLogs", $node, $time);

    my $err;
    my $dir_count = make_path($source_dir,{mode => 0777, error => \$err});

    if(scalar(@{$err}) > 0) {
        NATE::BaseException->throw("Failed to create dir $source_dir with error: " . join(@{$err}));
    }

    return $source_dir;
}

1;
