#
# Copyright (c) 2014-20* NetApp Inc.
# All rights reserved.
#
## @summary An exception for when User doesnt have enough
##  permission to operate on the file/dir
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here


package NACL::MTask::FilerLogRecoveryMode;
use NACL::C::Node;
use NACL::Transit;
use NACL::MTask::SCP;
use NATE::ParamSet;
use File::Spec;
use NACL::STask::NetworkInterface;
use NACL::CS::NetworkInterface;
use NACL::CS::StorageAggregate;
use NACL::STask::Aggregate;
use Scalar::Util qw(blessed);

use feature 'state';
use Params::Validate qw(validate_with OBJECT SCALAR);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::Transit::Exceptions::TransitException;
use NACL::MTask::Exceptions::FaiedToReachRecoveryMode;
use NATE::BaseException;

use Class::MethodMaker [
    scalar => [ { type => 'NACL::Transit' } , 'transit'],
    scalar => [ { type => 'NACL::C::CommandInterface::ONTAP' } , 'command_interface'],
];

=head1 METHODS

=head2 new

    my $FLRM = NACL::MTask::FilerLogRecoveryMode->new(
        command_interface => $node,
        nacltask_retries  => $retries,
        
    );

Creates a FilerLogRecoveryMode object that can be used to bring filer to recovery state.

=over

=item Options

=over

=item C<< command_interface => $node_obj >>

(Required, isa NACL::C::CommandInterface::ONTAP)
The node_obj from which logs are collected.

=item C<< nacltask_retries => $retries >>

(Default to 5)
Number of times to retry some operation if that operation needs retring.
Here it is used to get the aggregates info using Maintenance apiset.
Retries are needed because in case of debug builds console is spammed with
messages which might interfere with the output of command "aggr status" run on filer's MAINT state

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=back

=back

=cut

sub new {
    $Log->enter() if $may_enter;
    my ($class, @args) = @_;

    return $class if(blessed($class));

    state $spec = {
        command_interface => {
            type => OBJECT,
            isa  => "NACL::C::CommandInterface::ONTAP",
        },
        nacltask_retries => { type => SCALAR,  default => 5 },
    };

    my $self = validate_with(
        params => \@args,
        spec   => $spec,
    );

    bless $self, $class;

    my $command_interface = $self->{command_interface};
    my $hostrec = $command_interface->hostrec();

    my $transit = NACL::Transit->new(name => $hostrec);

    $self->transit($transit);

    $Log->exit() if $may_exit;
    return $self;
}

=head2 ensure_mroot_available

NACL::MTask::FilerLogGather->ensure_mroot_available(
    command_interface => $node_obj,
    nacltask_retries  => $retries,
);

OR

$FGL->ensure_mroot_available();

(Class or Object method)
Makes mroot available to accessible over the network. Performs following tasks

* Try to bring filer to CLI state. If its not able to do so then it will
try to transit node to recovery mode. First it will transit node to
'Maintenance' mode then will offline all aggregates except for root aggregate.
then it will transit node to CLI state.
* Then it will make node-mgmt lif's 'admin-status' to 'up'.

=over

=item C<< command_interface => $filer_node_object >>

(Required for class method call, isa NACL::C::CommandInterface::ONTAP)
The Node object of the filer.

=item C<< nacltask_retries => $retries >>

(Default to 5)
Number of times to retry some operation if that operation needs retring.
Here it is used to get the aggregates info using Maintenance apiset.
Retries are needed because in case of debug builds console is spammed with
messages which might interfere with the output of command "aggr status" run on filer's MAINT state

=back

=cut

sub ensure_mroot_available {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;

    $self = $self->new(%args);

    my $transit = $self->transit();
    try {
        $transit->change_state(to => 'CLI');
    } catch NACL::Transit::Exceptions::TransitException with {
        $self->transit_to_recovery_mode();
    };

    $self->make_node_mgmt_lif_up();

    $Log->exit() if $may_exit;
}

=head2 make_node_mgmt_lif_up

$FGL->make_node_mgmt_lif_up();

(Object method)
It will make node-mgmt lif's 'admin-status' to 'up'.
Before trying to make lif's 'admin-status' to 'up' it checks if
lifs "operational status" is up or not. If its not up then it fails
even before bringing the lif's 'admin-status' to up.

=over

=item C<< command_interface => $filer_node_object >>

(Required for class method call, isa NACL::C::CommandInterface::ONTAP)
The command_interface object of the filer.

=back

=cut

sub make_node_mgmt_lif_up {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    my $node = $self->command_interface();
    my $lif_cs = NACL::CS::NetworkInterface->fetch(
        command_interface => $node,
        filter => {
            role        => 'node-mgmt',
            'home-node' => $node->node,
        },
        requested_fields => [qw(status-oper)],
    );
    if($lif_cs->status_oper() ne "up") {
        $Log->exit() if $may_exit;
        NACL::MTask::Exceptions::FaiedToReachRecoveryMode->throw(
            "Operational status of lif is not up. " .
            "Its most probably a hardware failure. " .
            "Making mgmt lif's admin-status as up is not going to help here."
        );
    }

    my $lif_task = NACL::C::NetworkInterface->find(
        command_interface => $node,
        filter => {
            role        => 'node-mgmt',
            'home-node' => $node->node,
        },
    )->cast_component_to_task();

    $lif_task->modify(
        command_interface => $node,
        nacltask_verify   => 1,
        'status-admin'    => 'up',
    );

    $Log->exit() if $may_exit;
}

=head2 transit_to_recovery_mode


NACL::MTask::FilerLogGather->transit_to_recovery_mode(
    command_interface => $node_obj,
    nacltask_retries  => $retries,
);

OR

$FGL->transit_to_recovery_mode();

(Class or Object method)
It bring filer to CLI state. If its not able to do so then it will
try to transit node to recovery mode. First it will transit node to
'Maintenance' mode then will offline all aggregates except for root aggregate.
then it will transit node to CLI state.

=over

=item C<< command_interface => $filer_node_object >>

(Optional, isa NACL::C::CommandInterface::ONTAP)
The Node object of the filer.

=item C<< nacltask_retries => $retries >>

(Default to 5)
Number of times to retry some operation if that operation needs retring.
Here it is used to get the aggregates info using Maintenance apiset.
Retries are needed because in case of debug builds console is spammed with
messages which might interfere with the output of command "aggr status" run on filer's MAINT state

=back

=cut

sub transit_to_recovery_mode {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;

    $self = $self->new(%args);

    my $transit = $self->transit();

    $transit->change_state(to => 'MAINT');

    $self->offline_aggrs_except_root();

    $transit->change_state(to => 'CLI');

    $Log->exit() if $may_exit;
}

=head2 offline_aggrs_except_root

$FGL->offline_aggrs_except_root();

(Object method)
It will offline all aggregates except for root aggregate.

=over

=item C<< command_interface => $filer_node_object >>

(Required for class method call, isa NACL::C::CommandInterface::ONTAP)
The Node object of the filer.

=item C<< nacltask_retries => $retries >>

(Default to 5)
Number of times to retry some operation if that operation needs retring.
Here it is used to get the aggregates info using Maintenance apiset.
Retries are needed because in case of debug builds console is spammed with
messages which might interfere with the output of command "aggr status" run on filer's MAINT state

=item Exceptions

=over

=item C<< APISet exceptions >>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method> of the users guide.

=item C<< FaiedToReachRecoveryMode >>

This exception is thrown when filer is failed to reach to recovery mode

=back

=back

=cut

sub offline_aggrs_except_root {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;

    my $ci = $self->command_interface();
    my $maint_apiset = $ci->apiset(set => "Maintenance");
    my $retries = $self->{nacltask_retries};
    my $resp;
    my $success = 0;
    while ($retries) {
        try {
            # Only this command needs to be run multiple time in case
            # of debug builds. Because once apiset able to parse
            # the output properly then running aggr_offline
            # command will not be a problem even if console
            # is spammed with message.
            $resp = $maint_apiset->aggr_status();
            $success = 1;
        } catch NACL::APISet::Exceptions::ResponseException with {
            # Dont fail till retries are over
            $retries--;
        };
        if($success) {
            last;
        }
    }

    if (!$success) {
        $Log->exit() if $may_exit;
        NACL::MTask::Exceptions::FaiedToReachRecoveryMode->throw(
            "Failed to get aggregates info after $retries retries"
        );
    }

    my @aggrs_status = @{$resp->get_parsed_output};
    foreach my $status (@aggrs_status) {
        my $is_root = $status->{options}->[0]->{root};
        my $state = $status->{state};
        my $aggr = $status->{aggregate};
        if(! $is_root and $state ne 'offline') {
            $maint_apiset->aggr_offline(aggregate => $aggr);
        }
    }

    $Log->exit() if $may_exit;
}

=head2 online_aggrs

$FGL->online_aggrs();

(Object method)
It will online all offlined aggregates.

Note: transit_to_recovery_mode method offlines all aggregate
except root aggregate so that filer can be
booted up to CLI state. This method can be used to online all the
aggregates

e.g.

    foreach my $node (@Nodes) {
        my $recovery_mode;
        try {
            $recovery_mode = NACL::MTask::FilerLogRecoveryMode->new(
                command_interface => $node,
            );

            $recovery_mode->transit_to_recovery_mode();

        } catch NATE::BaseException with {
            my $ex = shift;
            $Log->warn("Collecting logs for remaining nodes even though gathering failed for node ".$node->node());
            $Log->warn(Dumper($ex));
        } finally {
            $node->refresh_command_interface()
            $recovery_mode->online_aggrs();
        };
    }

=over

=item C<< command_interface => $filer_node_object >>

(Required for class method call, isa NACL::C::CommandInterface::ONTAP)
The Node object of the filer.

=back

=cut

sub online_aggrs {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;

    my $ci = $self->command_interface();
    my @offline_aggrs = NACL::CS::StorageAggregate->fetch(
        command_interface => $ci,
        filter => {
            state => "offline",
        },
        requested_fields => [qw(aggregate)],
        allow_empty => 1,
    );

    foreach my $state (@offline_aggrs) {
        NACL::STask::Aggregate->modify(
            command_interface => $ci,
            aggregate => $state->aggregate(),
            state => "online",
        );
    }

    $Log->exit() if $may_exit;
}

1;
