# $Id$
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NACL::MTask::Group::7Mode Module
## @author radish, dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::MTask::Group::7Mode;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate
    qw(validate validate_with BOOLEAN SCALAR ARRAYREF HASHREF OBJECT SCALARREF);
use NATE::Exceptions::Argument qw(:try);
use NATE::BaseException;
use NACL::STask::Node;
use NACL::MTask::Set::Component;
use Scalar::Util;
use NACL::ComponentUtils;

use base qw (NACL::MTask::MTask);

# Import find_system_logical_id() from mix-in
use NACL::MTask::Group::_SystemGroup qw(:all);

# Import nodes() from mix-in
use NACL::MTask::Group::_Group qw(:all);

use Class::MethodMaker [
    new    => [ '-hash', 'new' ],
    scalar => 'logical_name',
    scalar => [
        {   '-default'  => NACL::MTask::Set::Component->new( type => 'NACL::C::SystemNode' ),
            '-type'     => 'NACL::MTask::Set::Component',
        },
        'node_set'
    ],
];

=head1 NAME

NACL::MTask::Group::7Mode

=head1 DESCRIPTION

"NACL::MTask::Group::7Mode" Groups Node objects based on their
inclusion in Sx_NODES parameters.

=head1 ATTRIBUTES

=head2 logical_name

The name of the 7Mode system from a NATE parameter point of view (i.e. S1, S2, etc...)

=head2 node_set

The collection of node objects in the Sx_NODES parameter.

=head1 METHODS

=cut

=head2 find 

(Class method only)

Given a list of NACL::C::SystemNode (or any class derived from SystemNode like
NACL::C::Node or NACL::STask::Node) objects, find will return a list of 
NACL::MTask::Group::7Mode objects, each containing a list of nodes that 
correspond to a Sx_NODES parameter.

Also, the nodes passed in do not all need to be 7Mode nodes.  Any CMode nodes
will be ignored.

ex:

    my @nodes = NACL::STask::Node->find();
    my @sgroups = NACL::MTask::Group::7Mode->find( nodes => \@nodes );

=over

=item Options

=over

=item C<< nodes => \@nodes >>

(Required, isa NACL::C::SystemNode)
The list of NACL Node objects to be grouped.

=back

=back

=head2 nodes

(Instance method only - ReadOnly)

    my $nodes = $sgroup->nodes();

Returns the names of the nodes in this 7Mode system.
The nodes() method is just an alternate view of the
nodes_set() method.

=head2 node_set

(Instance method only)

    my $ns = $sgroup->node_set();

Returns a NACL::MTask::Set::Component set of the nodes in this 7Mode system.
Component Sets allow many basic set operations like contains, intersection,
union, difference, etc...

The node objects returned will be of the same type as the node
objects passed into NACL::MTask::Group::7Mode->find().

In other words if you did:

    my @nodes = NACL::STask::Node->find();
    my @cgroups = NACL::MTask::Group::7Mode->find( nodes => \@nodes );
    my $ns = $sgroups[0]->node_set(); 
    my @sgroup_nodes = $ns->get_all_members();

The nodes in $ns will be of type NACL::STask::Node, but if the @nodes 
were returned by NACL::C::SystemNode->find() then the returned nodes would be of
type NACL::C:SystemNode.

=cut

sub find {
    $Log->enter() if $may_enter;
    my $me  = ( caller(0) )[3];
    my $pkg = shift;
    $pkg->_verify_invocation( style => 'static_only' );
    my %opts = validate_with(
        params => \@_,
        spec   => { nodes => { type => ARRAYREF, optional => 0 }, },
    );

    $Log->debug(
        sub { "$me: Options " . NACL::ComponentUtils::Dumper( \%opts ) } );

    my $nodes = delete $opts{nodes};

    my @systems;
    my %groups;
    my @sgroups;

    my $sysid_by_node
        = NACL::MTask::Group::7Mode->system_logical_ids_keyed_by_name(
        prefix      => "S",
        max_systems => NACL::MTask::Group::7Mode->max_num_7mode_systems()
        );
    my $nodes_by_sysid = NACL::MTask::Group::7Mode->names_keyed_by_system_logical_id(
        prefix      => "S",
        max_systems => NACL::MTask::Group::7Mode->max_num_7mode_systems()
    );

    foreach my $node (@$nodes) {

        # Verify each object in nodes isa NACL::C::SystemNode
        if (   !Scalar::Util::blessed($node)
            || !$node->isa('NACL::C::SystemNode') )
        {
            $Log->exit() if $may_exit;

            # throw an exception
            NATE::Exceptions::Argument->throw(
                      "$me: The \"nodes\" parameter requires"
                    . " objects derived from NACL::C::SystemNode"
                    . " \"nodes\" passed in are of type "
                    . ref($node) );
        } ## end if ( !Scalar::Util::blessed...

        # Only process 7Mode nodes
        if ( $node->mode() eq '7Mode' ) {

            my $sysids = $sysid_by_node->{$node->node()};
            if ($sysids) {
                foreach my $sysid (@$sysids) {
                    if ( !$groups{$sysid} ) {
                        push( @systems, $sysid );
                        $groups{$sysid} = $pkg->new(
                            logical_name => $sysid,
                            node_set     => NACL::MTask::Set::Component->new(
                                members => [$node]
                            )
                        );
                    } else {
                        $groups{$sysid}->node_set()->add($node);
                    }
                    delete $nodes_by_sysid->{$sysid}->{$node->node()};
                } ## end foreach my $sysid (@$sysids)
                delete $sysid_by_node->{$node->node()};
            } else {
                # If no Sx_NODES parameter was specified, lump all 
                # 7Mode nodes together
                if ( !$groups{NONE} ) {
                    $groups{NONE} = $pkg->new(
                            node_set     => NACL::MTask::Set::Component->new(
                                members => [$node]
                            )
                        );
                    push( @sgroups, $groups{NONE} );
                } else {
                    $groups{NONE}->node_set()->add($node);
                }
            }
        } ## end if ( $node->mode() eq ...
    } ## end foreach my $node (@$nodes)

    # Go through each system and see if we need to add more nodes
    # create any node objects that the caller did not pass in to us
    foreach my $sysid (@systems) {

        my $node;
        my $node_pkg;

        # If user didn't supply us with all of the nodes in the system, create
        # the ones they did not
        if ( keys %{ $nodes_by_sysid->{$sysid} } ) {

           # If someone passes us NACL::STask::Node objects, we want the nodes
           # that we create to be of the same type.  ex: if they pass us
           # NACL::C::Node objects, we'll create NACL::C::Node objects,
           # if NACL::STask::Node we'll create those.
            if ( !$node_pkg ) {
                my @nodes = $groups{$sysid}->node_set()->get_all_members();
                $node     = $nodes[0];
                $node_pkg = ref($node);

                # The require is here if someday we have a new class that is
                # derived from NACL::C::SystemNode.  Require does nothing if
                # the package is already loaded.
                # And yeah, loading within the if block, the library is still
                # available outside it :)
                eval "require $node_pkg";
            } ## end if ( !$node_pkg )

            # If they didn't give us all of the nodes in the system,
            # we'll specify the command_interface corresponding to
            # the first node that we DID find
            foreach my $node_name ( keys %{ $nodes_by_sysid->{$sysid} } ) {

                my $node_obj = $node_pkg->new(
                    name              => $node_name,
                    command_interface => $node
                );

                $groups{$sysid}->node_set()->add($node_obj);
            } ## end foreach my $node_name ( keys...
        } ## end if ( keys %{ $nodes_by_sysid->{$sysid...

        push( @sgroups, $groups{$sysid} );
    } ## end foreach my $sysid (@systems)

    $Log->exit() if $may_exit;
    return wantarray() ? @sgroups : $sgroups[0];
} ## end sub find

1;
