# $Id$
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NACL::MTask::Group::Cluster Module
## @author radish, dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::MTask::Group::Cluster;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate
    qw(validate validate_with BOOLEAN SCALAR ARRAYREF HASHREF OBJECT SCALARREF);
use NATE::Exceptions::Argument qw(:try);
use NATE::BaseException;
use NACL::C::Cluster;
use NACL::CS::ClusterIdentity;
use NACL::STask::Node;
use NACL::MTask::Set::Component;
use Scalar::Util;
use NACL::ComponentUtils;

use base qw (NACL::MTask::MTask);

# Import find_system_logical_id() from mix-in
use NACL::MTask::Group::_SystemGroup qw(:all);

# Import nodes(), node_objs() from mix-in
use NACL::MTask::Group::_Group qw(:all);

use Class::MethodMaker [
    new    => [ '-hash', 'new' ],
    scalar => [
        {   '-default_ctor' => sub {
                my $nodename = $_[0]->nodes();
                my $log_id = $_[0]->find_system_logical_id(
                    prefix      => 'C',
                    suffix      => 'NODES',
                    name        => $nodename,
                    max_systems => NACL::MTask::Group::Cluster->max_num_clusters()
                );

                # If no logical name could be extracted from Cx_NODES
                # (i.e. no Cx_NODES were passed to script)
                # see if cluster name is of the form
                # Cx_nodename_time and extract from that
                if ( !$log_id ) {
                    my $name = $_[0]->name();
                    if ( $name =~ /(C\d+)_\S+/ ) {
                        $log_id = $1;
                    }
                }

                $log_id;
            },
        },
        'logical_name',
    ],
    scalar => 'name',
    scalar => 'uuid',
    scalar => [
        {   '-default'  => NACL::MTask::Set::Component->new( type => 'NACL::C::SystemNode' ),
            '-type'     => 'NACL::MTask::Set::Component',
        },
        'node_set'
    ],
];

=head1 NAME

NACL::MTask::Group::Cluster

=head1 DESCRIPTION

"NACL::MTask::Group::Cluster" Groups Node objects that are in the
same cluster.

=head1 ATTRIBUTES

=head2 logical_name

The name of the cluster from a NATE parameter point of view (i.e. C1, C2, etc...)

=head2 name

The name of the cluster (on the filer).

=head2 uuid

The uuid of the cluster.

=head2 node_set

The collection of node objects in the cluster.

=head1 METHODS

=cut

=head2 find 

(Class method only)

Given a list of NACL::C::SystemNode (or any class derived from SystemNode like
NACL::C::Node or NACL::STask::Node) objects, find will return a list of 
NACL::MTask::Group::Cluster objects, each containing a list of nodes for 
that Cluster.

The list of nodes passed into find need not contain all nodes for every
cluster (i.e. it could contain only one per cluster) but the returned
Group::Cluster objects would contain all of the nodes in that cluster.

Also, the nodes passed in do not all need to be CMode nodes.  Any 7Mode nodes
will be ignored.

Note:  The assumption is that NACL::MTask::Group::Cluster->find() will be
run AFTER the clusters have been setup (created/joined) and will not work
on pre-cluster nodes.

ex:

    my @nodes = NACL::STask::Node->find();
    my @cgroups = NACL::MTask::Group::Cluster->find( nodes => \@nodes );

=over

=item Options

=over

=item C<< nodes => \@nodes >>

(Required, isa NACL::C::SystemNode)
The list of NACL Node objects from which to determine the
clusters from.

=item C<< allow_empty => 0|1 >>

(Optional)
If some of the CMode nodes passed in via the NODES parameter were not
yet joined to a cluster, allow_empty => 1 would permit the find() method
to treat this condition as acceptable, otherwise the default is to let
the fetches called within find() throw exceptions when no Cluster or 
ClusterIdentity CS object was found.  Typically scripts are run post
cluster, and so not finding a Cluster or ClusterIdentity CS object
should be considered an error.

=back

=back

=head2 name

(Instance method only)

    my $name = $cluster_group->name();

Returns the name of the cluster.

=head2 uuid

(Instance method only)

    my $uuid = $cluster_group->uuid();

Returns the uuid of the cluster.

=head2 nodes

(Instance method only - ReadOnly)

    my @nodes = $cluster_group->nodes();
    my $node  = $cluster_group->nodes();

Returns the names of the nodes in this cluster.
The nodes() method is just an alternate view of the
nodes_set() method.

=head2 node_objs

(Instance method only - ReadOnly)

    my @nodes = $cluster_group->node_objs();
    my $node  = $cluster_group->node_objs();

Returns the node objects in this cluster.
The node_objs() method is just an alternate view of the
nodes_set() method.

=head2 node_set

(Instance method only)

    my $ns = $cluster_group->node_set();

Returns a NACL::MTask::Set::Component set of the nodes in this cluster.
Component Sets allow many basic set operations like contains, intersection,
union, difference, etc...

The node objects returned will be of the same type as the node
objects passed into NACL::MTask::Group::Cluster->find().

In other words if you did:

    my @nodes = NACL::STask::Node->find();
    my @cgroups = NACL::MTask::Group::Cluster->find( nodes => \@nodes );
    my $ns = $cgroups[0]->node_set(); 
    my @cgroup_nodes = $ns->get_all_members();

The nodes in $ns will be of type NACL::STask::Node, but if the @nodes 
were returned by NACL::C::SystemNode->find() then the returned nodes would be of
type NACL::C:SystemNode.

=cut

sub find {
    $Log->enter() if $may_enter;
    my $me  = ( caller(0) )[3];
    my $pkg = shift;
    $pkg->_verify_invocation( style => 'static_only' );
    my %opts = validate_with(
        params => \@_,
        spec   => {
            nodes       => { type => ARRAYREF, optional => 0 },
            allow_empty => { type => SCALAR,   default  => 0 },
        },
    );

    $Log->debug(
        sub { "$me: Options " . NACL::ComponentUtils::Dumper( \%opts ) } );

    my $nodes       = delete $opts{nodes};
    my $allow_empty = delete $opts{allow_empty};

    my %groups;
    my %nodes;
    my @c_uuids;

    foreach my $node (@$nodes) {

        # Verify each object in nodes isa NACL::C::SystemNode
        if (   !Scalar::Util::blessed($node)
            || !$node->isa('NACL::C::SystemNode') )
        {
            $Log->exit() if $may_exit;

            # throw an exception
            NATE::Exceptions::Argument->throw(
                      "$me: The \"nodes\" parameter requires"
                    . " objects derived from NACL::C::SystemNode"
                    . " \"nodes\" passed in are of type "
                    . ref($node) );
        } ## end if ( !Scalar::Util::blessed...

        # Get cluster data from Node if it is CMode
        if ( $node->mode() eq 'CMode' ) {
            my @cluster_entries = NACL::C::Cluster->find(
                command_interface => $node,
                allow_empty       => $allow_empty,
            );

            # In the unlikely event that we are given some nodes
            # which are not joined to the cluster,
            # @cluster_entries could be empty if allow_empty == 1
            # if no cluster entries, skip this node
            if ( !@cluster_entries ) {
                next;
            }

            my @node_names = map $_->node(), @cluster_entries;

            my $cid = NACL::CS::ClusterIdentity->fetch(
                command_interface => $node,
                allow_empty       => $allow_empty,
            );

           # Use cluster uuid to uniquely identify a "cluster" object
           # Note, here we are referring to the "cluster" the way people
           # generally refer to a cluster, in that it is a collection of nodes
           # that are joined into a cluster.
           # The NACL::C::Cluster object is based on the cluster table
           # where there is a cluster entry for each node in the cluster.
            my $uuid  = $cid->uuid();
            my $cname = $cid->name();

            # Keep track of nodes for the cluster, we'll create the ones
            # we need to later, after going through all of the nodes
            # provided to us.
            if ( !$nodes{$uuid} ) {
                %{ $nodes{$uuid} } = map { $_ => 1 } @node_names;
            }

            delete $nodes{$uuid}{ $node->node() };

            if ( !$groups{$uuid} ) {
                $groups{$uuid} = $pkg->new(
                    name     => $cname,
                    uuid     => $uuid,
                    node_set => NACL::MTask::Set::Component->new(
                        members => [$node]
                    )
                );

                # want to keep track of clusters in the order we
                # encounter them (via their nodes)
                push( @c_uuids, $uuid );
            } else {
                $groups{$uuid}->node_set()->add($node);
            }
        } ## end if ( $node->mode() eq ...
    } ## end foreach my $node (@$nodes)

    my @clusters;

    # Go through each Cluster object we created and see if we need to
    # create any node objects that the caller did not pass in to us
    foreach my $uuid (@c_uuids) {

        my $node;
        my $node_pkg;

       # If user didn't supply us with all of the nodes in the cluster, create
       # the ones they did not
        if ( keys %{ $nodes{$uuid} } ) {

           # If someone passes us NACL::STask::Node objects, we want the nodes
           # that we create to be of the same type.  ex: if they pass us
           # NACL::C::Node objects, we'll create NACL::C::Node objects,
           # if NACL::STask::Node we'll create those.
            if ( !$node_pkg ) {
                my @nodes = $groups{$uuid}->node_set()->get_all_members();
                $node     = $nodes[0];
                $node_pkg = ref($node);

                # The require is here if someday we have a new class that is
                # derived from NACL::C::SystemNode.  Require does nothing if
                # the package is already loaded.
                # And yeah, loading within the if block, the library is still
                # available outside it :)
                eval "require $node_pkg";
            } ## end if ( !$node_pkg )

            # If they didn't give us all of the nodes in the cluster,
            # we'll specify the command_interface corresponding to
            # the first node (of the cluster) that we DID find
            foreach my $node_name ( keys %{ $nodes{$uuid} } ) {

                my $node_obj = $node_pkg->new(
                    name              => $node_name,
                    command_interface => $node
                );

                $groups{$uuid}->node_set()->add($node_obj);
            } ## end foreach my $node_name ( keys...
        } ## end if ( keys %{ $nodes{$uuid...

        push( @clusters, $groups{$uuid} );
    } ## end foreach my $uuid (@c_uuids)

    $Log->exit() if $may_exit;
    if (wantarray) {
        return @clusters;
    } else {
        return $clusters[0];
    }
} ## end sub find

1;
