# $Id$
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NACL::MTask::Group::SFOPair Module
## @author radish, dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::MTask::Group::SFOPair;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate
    qw(validate validate_with BOOLEAN SCALAR ARRAYREF HASHREF OBJECT SCALARREF);
use NATE::Exceptions::Argument qw(:try);
use NATE::BaseException;
use NACL::STask::Node;
use NACL::MTask::Set::Component;
use NACL::ComponentUtils;
use Scalar::Util;

use base qw (NACL::MTask::MTask);

# Import nodes(), node_objs() from mix-in
use NACL::MTask::Group::_Group qw(:all);

use Class::MethodMaker [
    new => [ '-hash', 'new' ],
    scalar => [ { '-default' => "" }, 'pair' ],
    scalar => [
        {   '-static'   => 1,
            '-default'  => "",
        },
        'mode'
    ],
    scalar => [
        {   '-default'  => NACL::MTask::Set::Component->new( type => 'NACL::C::SystemNode' ),
            '-type'     => 'NACL::MTask::Set::Component',
        },
        'node_set'
    ],
];

=head1 NAME

NACL::MTask::Group::SFOPair

=head1 DESCRIPTION

"NACL::MTask::Group::SFOPair" Groups Node objects that are in the
same SFO pair.  This is an abstract base class and should not be
used directly.  Instead, use one of the classes that are derived
from this class.  In the derived classes, the "mode" parameter 
will be defined.

=head1 ATTRIBUTES

=head2 pair

The SFOPAIR string.

=head2 mode

The mode of the SFOPair.

=head2 node_set

The collection of node objects in the SFO pair.

=head1 METHODS

=cut

=head2 find 

(Class method only)

Given a list of NACL::C::SystemNode (or any class derived from SystemNode like
NACL::C::Node or NACL::STask::Node) objects, will return a list of 
NACL::MTask::Group::SFOPair objects, each containing a list of nodes for 
that SFO pair.

ex:

    my @nodes = NACL::STask::Node->find();
    my @sgroups = NACL::MTask::Group::SFOPair::7Mode->find( nodes => \@nodes );
    my @cgroups = NACL::MTask::Group::SFOPair::CMode->find( nodes => \@nodes );

=over

=item Options

=over

=item C<< nodes => \@nodes >>

(Required, isa NACL::C::SystemNode)

The list of NACL Node objects from which to construct
SFOPairs.  All nodes for the SFOPairs must be present
(i.e. you cannot only provide the first node of an SFO
pair).  This class requires the following Hostrec variable
be defined for both nodes of a pair (SFOPAIR) and that that
variable must contain the same string for both nodes.

=back

=back

=head2 pair

(Instance method only)

    my $pair = $SFOPair->pair();

Returns the SFOPAIR string stored in the Hostrec.

=head2 mode

(Instance method only)

    my $mode = $SFOPair->mode();

Returns the mode of this SFOPair.  In this abstract base class mode is ""
by default.  The mode must be re-defined in the derived classes.

=head2 nodes

(Instance method only - ReadOnly)

    my @nodes = $SFOPair->nodes();
    my $node  = $SFOPair->nodes();

Returns the names of the nodes in this SFO pair.
The nodes() method is just an alternate view of the
nodes_set() method.

=head2 node_objs

(Instance method only - ReadOnly)

    my @nodes = $SFOPair->node_objs();
    my $node  = $SFOPair->node_objs();

Returns the node objects in this SFO pair.
The node_objs() method is just an alternate view of the
nodes_set() method.

=head2 node_set

(Instance method only)

    my $ns = $SFOPair->node_set();

Returns a NACL::MTask::Set::Component set of the nodes in this SFO pair.
Component Sets allow many basic set operations like contains, intersection,
union, difference, etc...

Returns a NACL::MTask::Set::Component set of the nodes in this SFO pair.
Component Sets allow many basic set operations like contains, intersection,
union, difference, etc...

The node objects returned will be of the same type as the node
objects passed into NACL::MTask::Group::SFOPair->find().

In other words if you did:

    my @nodes = NACL::STask::Node->find();
    my @groups = NACL::MTask::Group::SFOPair->find( nodes => \@nodes );
    my $ns = $groups[0]->node_set();
    my @group_nodes = $ns->get_all_members();

The nodes in $ns will be of type NACL::STask::Node, but if the @nodes
were returned by NACL::C::SystemNode->find() then the returned nodes would be of
type NACL::C:SystemNode.

=cut

sub find {
    $Log->enter() if $may_enter;
    my $me  = ( caller(0) )[3];
    my $pkg = shift;
    $pkg->_verify_invocation( style => 'static_only' );

    my %opts = validate_with(
        params => \@_,
        spec   => { nodes => { type => ARRAYREF, optional => 0 }, },
    );

    $Log->debug( "$me: Options " . NACL::ComponentUtils::Dumper( \%opts ) );

    my $nodes = delete $opts{nodes};

    my @pairs;
    my %groups;

    foreach my $node (@$nodes) {

        # Verify each object in nodes isa NACL::C::SystemNode
        if ( !Scalar::Util::blessed($node) || !$node->isa('NACL::C::SystemNode') ) {

            # throw an exception
            NATE::BaseException->throw(
                "$me: The \"nodes\" parameter requires objects derived from NACL::C::SystemNode"
            );
        } ## end if ( !Scalar::Util::blessed($node) || !$node...

        # Verify that the mode matches that of the pkg
        if ( $node->mode() eq $pkg->mode() ) {
            my $pair = $node->hostrec()->{SFOPAIR};
            if ( !$pair ) {
                NATE::BaseException->throw(
                    "$me: SFOPAIR Hostrec variable was not defined for node "
                        . $node->node() );
            }

            if ( !$groups{$pair} ) {
                $groups{$pair} = $pkg->new(
                    pair     => $pair,
                    node_set => NACL::MTask::Set::Component->new(
                        members => [$node]
                    )
                );
                push( @pairs, $pair );
            } else {
                if ( !$groups{$pair}->node_set()->contains($node) ) {
                    my $ns = $groups{$pair}->node_set();
                    $ns->add($node);
                    $groups{$pair}->node_set($ns);
                }
            } ## end else [ if ( !$groups{$pair} )
        } ## end if ( $node->mode() eq ...
    } ## end foreach my $node (@$nodes)

  # Post process, if any pair has anything other than 2 nodes, throw exception
    my @pair_groups;
    foreach my $pair (@pairs) {
        if ( $groups{$pair}->node_set()->size() != 2 ) {
            my @nodes = $groups{$pair}->nodes();
            NATE::BaseException->throw(
                "$me: SFOPAIR $pair did not have exactly 2 members "
                    . NACL::ComponentUtils::Dumper( \@nodes ) );
        }
        push( @pair_groups, $groups{$pair} );
    } ## end foreach my $pair (@pairs)

    $Log->exit() if $may_exit;
    return wantarray() ? @pair_groups : $pair_groups[0];
} ## end sub find

1;
