# $Id$
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary common routines for NACL::MTask::Group Cluster, 7Mode and SFOPair::7Mode
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::MTask::Group::_SystemGroup;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR validate_with SCALARREF HASHREF
    OBJECT UNDEF);
use NACL::GeneralUtils;

use constant _MAX_NUM_SYSTEMS => 10;

our $_MAX_NUM_LEAGUES;
our $_MAX_NUM_CLUSTERS;
our $_MAX_NUM_7MODE_SYSTEMS;

BEGIN {
    use Exporter qw(import);
    our %EXPORT_TAGS = (
        all => [
            qw(
                find_system_logical_id
                system_logical_ids_keyed_by_name
                names_keyed_by_system_logical_id
                groups_by_logical_name
                max_num_systems
                max_num_leagues
                max_num_clusters
                max_num_7mode_systems
                )
        ]
    );
    our @EXPORT_OK = @{ $EXPORT_TAGS{all} };

    my $max_leagues = Tharn::param( "MAX_LEAGUES" );
    if ( $max_leagues ) {
        $_MAX_NUM_LEAGUES = $max_leagues;
    } else { 
        $_MAX_NUM_LEAGUES = _MAX_NUM_SYSTEMS;
    }

    my $max_clusters = Tharn::param( "MAX_CLUSTERS" ) ||
                       Tharn::param( "MAX_CMODE_SYSTEMS" );
    if ( $max_clusters ) {
        $NACL::MTask::Group::_SystemGroup::_MAX_NUM_CLUSTERS = $max_clusters;
    } else { 
        $NACL::MTask::Group::_SystemGroup::_MAX_NUM_CLUSTERS = _MAX_NUM_SYSTEMS;
    }

    my $max_7mode = Tharn::param( "MAX_7MODE_SYSTEMS" );
    if ( $max_7mode ) {
        $_MAX_NUM_7MODE_SYSTEMS = $max_7mode;
    } else { 
        $_MAX_NUM_7MODE_SYSTEMS = _MAX_NUM_SYSTEMS;
    }
} ## end BEGIN


=head1 NAME

NACL::MTask::Group::_SystemGroup

=head1 DESCRIPTION

Common code for Cluster, 7Mode & SFOPair/7Mode and Leagues

=head1 METHODS

=head2 find_system_logical_id

    $cgroup->find_system_logical_id( prefix => "C" );

The intent of this method is to correlate a system group object
(NACL::MTask::Group::Cluster, NACL::MTask::Group::7Mode or NACL::MTask::Group::SFOPair/7Mode)
objects with a corresponding Cx_NODES, or Sx_NODES where x is <1,2,3,4,...>

This logical name (for example C2) will be used later to associate other
parameters with that group (say a C2_BUILDROOT, or C2_VOLUME).  This might be needed
when a script is running against multiple systems (i.e. multiple clusters and/or
multiple 7Mode systems).

=over

=item Options

=over

=item C<< prefix => 'C' >>

(Required, SCALAR)
The prefix that denotes a system-specific parameter.
For example, for Clusters it would be "C", for 7Mode systems
it would be "S", and Leagues would use "L".

=item C<< suffix => 'NODES' >>

(Optional, SCALAR, default 'NODES')
The suffix that denotes the system-specific grouping parameter.
For example, for Clusters and 7Mode systems it would be "NODES".
The variable that find_system_logical_id() looks for is 
<prefix>x_<suffix>, so for Clusters it would be Cx_NODES where x
is <1,2,3,4,...>

=item C<< name => $node >>

(Required, SCALAR)
The name which needs to be found in the system-specific grouping
parameter (see suffix above).

For example, for 7Mode systems the system-specific grouping parameter
would be Sx_NODES and the name would be one of the nodes in that group.
The find_system_logical_id() will cycle through 
S1_NODES, S2_NODES, up to the limit specified by the max_systems
option, looking for a system which contains this node name.
If found, that system id (say in this case S3) will be returned.  If 
no system is found, undef is returned.

=item C<< max_systems => 20 >>

(Optional, SCALAR, default 10)
The maximum number of system-specific grouping parameters to be
searched.  For example, for Clusters, if max_systems was 3, only
C1_NODE, C2_NODES and C3_NODES would be searched.

=back

=back

=head2 system_logical_ids_keyed_by_name

    $cgroup->system_logical_ids_keyed_by_name( prefix => "C" );

This method will return a hashref whose values are a list of system
logical ids, keyed by the name of the object being grouped into 
that system. 

For example, for any of the node grouping classes like NACL::MTask::Group::Cluster, 
NACL::MTask::Group::7Mode, NACL::MTask::Group::SFOPair::7Mode or
NACL::MTask::Group::SFOPair::CMode the keys will be node names.

So, for the node example, you could see what system logical ids a node
was grouped by.  For Cluster or SFOPairs, one node would be grouped
in only a single group, but for leagues, one node could be in multiple
leagues.

So, for the node example, you could see what systems a given node is
grouped by (again, a node could be in multiple leagues, but for Cluster
or SFOPair, the relationship would be one to one.)

This method uses the <prefix><n>_<suffix> parameters to determine membership, 
where prefix is C, S or L (Cluster, 7Mode or League)
n is an integer >= 1, and suffix is typically NODES, but could be
something else if the Group class was grouping objects other than
nodes.

=over

=item Options

=over

=item C<< prefix => 'C' >>

(Required, SCALAR)
The prefix that denotes a system-specific parameter.
For example, for Clusters it would be "C", for 7Mode systems
it would be "S", and Leagues would use "L".


=item C<< suffix => 'NODES' >>

(Optional, SCALAR, default 'NODES')
The suffix that denotes the system-specific grouping parameter.
For example, for Clusters and 7Mode systems it would be "NODES".
The variable that find_system_logical_id() looks for is 
<prefix>x_<suffix>, so for Clusters it would be Cx_NODES where x
is <1,2,3,4,...>

=item C<< name => $node >>

(Required, SCALAR)
The name which needs to be found in the system-specific grouping
parameter (see suffix above).

For example, for 7Mode systems the system-specific grouping parameter
would be Sx_NODES and the name would be one of the nodes in that group.
The find_system_logical_id() will cycle through 
S1_NODES, S2_NODES, up to the limit specified by the max_systems
option, looking for a system which contains this node name.
If found, that system id (say in this case S3) will be returned.  If 
no system is found, undef is returned.

=item C<< max_systems => 20 >>

(Optional, SCALAR, default 10)
The maximum number of system-specific grouping parameters to be
searched.  For example, for Clusters, if max_systems was 3, only
C1_NODE, C2_NODES and C3_NODES would be searched.

=back

=back

=head2 names_keyed_by_system_logical_id

    $cgroup->names_keyed_by_system_logical_id( prefix => "C" );

This method will return a hashref whose values are the list of the names 
grouped by a logical system, keyed by that system logical id.

For example, for any of the node grouping classes like NACL::MTask::Group::Cluster, 
NACL::MTask::Group::7Mode, NACL::MTask::Group::SFOPair::7Mode or
NACL::MTask::Group::SFOPair::CMode the names will be node names.

So, for the node example, you could see what nodes a system logical
id groups.  This method uses the <prefix><n>_<suffix> parameters to 
determine this, where prefix is C, S or L (Cluster, 7Mode or League)
n is an integer >= 1, and suffix is typically NODES, but could be
something else if the Group class was grouping objects other than
nodes.

=over

=item Options

=over

=item C<< prefix => 'C' >>

(Required, SCALAR)
The prefix that denotes a system-specific parameter.
For example, for Clusters it would be "C", for 7Mode systems
it would be "S", and Leagues would use "L".

=item C<< suffix => 'NODES' >>

(Optional, SCALAR, default 'NODES')
The suffix that denotes the system-specific grouping parameter.
For example, for Clusters and 7Mode systems it would be "NODES".
The variable that find_system_logical_id() looks for is 
<prefix>x_<suffix>, so for Clusters it would be Cx_NODES where x
is <1,2,3,4,...>

=item C<< name => $node >>

(Required, SCALAR)
The name which needs to be found in the system-specific grouping
parameter (see suffix above).

For example, for 7Mode systems the system-specific grouping parameter
would be Sx_NODES and the name would be one of the nodes in that group.
The find_system_logical_id() will cycle through 
S1_NODES, S2_NODES, up to the limit specified by the max_systems
option, looking for a system which contains this node name.
If found, that system id (say in this case S3) will be returned.  If 
no system is found, undef is returned.

=item C<< max_systems => 20 >>

(Optional, SCALAR, default 10)
The maximum number of system-specific grouping parameters to be
searched.  For example, for Clusters, if max_systems was 3, only
C1_NODE, C2_NODES and C3_NODES would be searched.

=back

=back

=head2 groups_by_logical_name

    my @nodes = NACL::C::Node->find();
    my @cgroups = NACL::MTask::Group::Cluster->find( nodes => \@nodes );
    my @sgroups = NACL::MTask::Group::7Mode->find( nodes => \@nodes );

    my %clusters_by_logical = NACL::MTask::Group::Cluster->groups_by_logical_name( @cgroups );
    my %sgroups_by_logical  = NACL::MTask::Group::Cluster->groups_by_logical_name( @sgroups );

This method will return a hashref of system groups keyed by system logical name.
For example, for NACL::MTask::Group::Cluster, the hash would contain all of the 
NACL::MTask::Group::Cluster objects keyed by their cluster logical names (i.e. 
C1, C2, C3, etc...) 

=over

=item Options

=over

=item C<< <groups> >>

(Required, ARRAY)
An array of the Group objects.

=back

=back

=head2 max_num_systems

    my $max_systems = NACL::MTask::Group::Cluster->max_num_systems();
    my $max_systems = NACL::MTask::Group::7Mode->max_num_systems();

This method will return the maximum number of systems permitted.  The maximum is
necessary so that we limit how many parameters we look for.  Since a parameter
like IMAGE (for example) could exist in a prefaced form, the maximum lets
the library know that it only has to look for C1_IMAGE through C10_IMAGE
(if the maximum was 10) and not to keep looking for a C1000_IMAGE parameter.

=head2 max_num_leagues

    my $max_leagues = NACL::MTask::Group::League->max_num_leagues();

This method will return the maximum number of leagues permitted.  The maximum is
necessary so that we limit how many parameters we look for. (see description for
max_num_systems for an explanation of why this is needed.)

While the default for the max number of systems is 10, this can be overridden by
the MAX_LEAGUES parameter.

=head2 max_num_clusters

    my $max_clusters = NACL::MTask::Group::Cluster->max_num_clusters();

This method will return the maximum number of clusters permitted.  The maximum is
necessary so that we limit how many parameters we look for. (see description for
max_num_systems for an explanation of why this is needed.)

While the default for the max number of systems is 10, this can be overridden by
the MAX_CLUSTERS or MAX_CMODE_SYSTEMS parameters.

=head2 max_num_7mode_systems

    my $max_7mode_systems = NACL::MTask::Group::League->max_num_7mode_systems();

This method will return the maximum number of 7mode systems permitted.  The maximum is
necessary so that we limit how many parameters we look for. (see description for
max_num_systems for an explanation of why this is needed.)

While the default for the max number of systems is 10, this can be overridden by
the MAX_7MODE_SYSTEMS parameters.

=cut

sub find_system_logical_id {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            name => {
                type     => SCALAR,
            },
            prefix => {
                type     => SCALAR,
            },
            suffix => {
                type    => SCALAR,
                default => "NODES"
            },
            max_systems => {
                type    => SCALAR,
                default => NACL::MTask::Group::_SystemGroup->max_num_systems()
            },
        },
    );
    my $sys_log_id;

    for ( my $i = 1; $i <= $opts{max_systems}; $i++ ) {
        $sys_log_id = $opts{prefix} . $i;
        my $system_id = $sys_log_id . '_' . $opts{suffix};
        my $p         = Tharn::param($system_id);
        if ($p) {
            my %names = NACL::GeneralUtils::_parse_hosts($p);
            if ( $names{ $opts{name} } ) {
                last;
            }
        } else {

            # assumption is that systems are named sequentially with no gaps
            last;
        }
        $sys_log_id = undef;
    } ## end for ( my $i = 1; $i <= ...

    $Log->exit() if $may_exit;
    return $sys_log_id;
} ## end sub find_system_logical_id

sub system_logical_ids_keyed_by_name {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            prefix => {
                type     => SCALAR,
            },
            suffix => {
                type    => SCALAR,
                default => "NODES"
            },
            max_systems => {
                type    => SCALAR,
                default => NACL::MTask::Group::_SystemGroup->max_num_systems()
            },
        },
    );
    my %hash;

    for ( my $i = 1; $i <= $opts{max_systems}; $i++ ) {
        my $sys_log_id = $opts{prefix} . $i;
        my $system_id  = $sys_log_id . '_' . $opts{suffix};
        my $p          = Tharn::param($system_id);
        if ($p) {

            # A node is assumed to be in one and only one cluster or
            # 7mode system but his is not true for Leagues, a node
            # (and therefore a cluster) can be in multiple leagues
            # so we must return a hash keyed by names (generally nodes), i
            # but the hash value will be an array.
            my %h = map { $_ => $sys_log_id } split( ",", $p );
            foreach my $key ( keys %h ) {
                if ( !exists $hash{$key} ) {
                    $hash{$key} = [$sys_log_id];
                } else {
                    push( @{ $hash{$key} }, $sys_log_id );
                }
            } ## end foreach my $key ( keys %h )

        } else {

            # assumption is that systems are named sequentially with no gaps
            last;
        }
    } ## end for ( my $i = 1; $i <= ...

    $Log->exit() if $may_exit;
    return \%hash;
} ## end sub system_logical_ids_keyed_by_name

sub names_keyed_by_system_logical_id {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            prefix => {
                type     => SCALAR,
            },
            suffix => {
                type    => SCALAR,
                default => "NODES"
            },
            max_systems => {
                type    => SCALAR,
                default => NACL::MTask::Group::_SystemGroup->max_num_systems()
            },
        },
    );
    my %hash;

    for ( my $i = 1; $i <= $opts{max_systems}; $i++ ) {
        my $sys_log_id = $opts{prefix} . $i;
        my $system_id  = $sys_log_id . '_' . $opts{suffix};
        my $p          = Tharn::param($system_id);
        if ($p) {

            # A node is assumed to be in one and only one cluster or
            # 7mode system but his is not true for Leagues, a node
            # (and therefore a cluster) can be in multiple leagues
            # so we must return a hash keyed by names (generally nodes), i
            # but the hash value will be an array.
            $hash{$sys_log_id}
                = { map { $_ => $sys_log_id } split( ",", $p ) };
        } else {

            # assumption is that systems are named sequentially with no gaps
            last;
        }
    } ## end for ( my $i = 1; $i <= ...

    $Log->exit() if $may_exit;
    return \%hash;
} ## end sub names_keyed_by_system_logical_id

sub groups_by_logical_name {
    $Log->enter() if $may_enter;
    my $pkg  = shift;

    my @groups = @_;

    my %groups_by_logical = map { $_->logical_name() => $_ } @groups;

    $Log->exit() if $may_exit;
    return wantarray ? %groups_by_logical : \%groups_by_logical;
} ## end sub groups_by_logical_name

sub max_num_systems {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    $Log->exit() if $may_exit;
    return _MAX_NUM_SYSTEMS;
} ## end sub max_num_systems

sub max_num_leagues {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    $Log->exit() if $may_exit;
    return $_MAX_NUM_LEAGUES;
} ## end sub max_num_leagues

sub max_num_clusters {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    $Log->exit() if $may_exit;
    return $_MAX_NUM_CLUSTERS;
} ## end sub max_num_clusters

sub max_num_7mode_systems {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    $Log->exit() if $may_exit;
    return $_MAX_NUM_7MODE_SYSTEMS;
} ## end sub max_num_7mode_systems

1;
