#
# Copyright (c) 2014-2015 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.

## @summary Import Task Module
## @author shivakrt@netapp.com
## @status
## @pod here

package NACL::MTask::Import;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Exporter qw(import);
use Params::Validate qw(:all);
use NACL::InstantComponent qw(instant_component);
use NACL::CS::DebugVreport;
use NACL::CS::Vserver;
use NACL::CS::Job;
use NACL::CS::JobHistory;
use NATE::ProcManager;
use NACL::CS::StorageAggregate;
use NACL::Exceptions::InvalidParam;
use NACL::Exceptions::UnexpectedState;
use NACL::APISet::Exceptions::CommandFailedException;
use NATE::BaseException qw(:try);
use base qw(NACL::STask::STask);
use Data::Dumper;
use vars qw(
  $VERSION
  $command_interface
  %volAttrs
  $dataVs
  $Node
  $SESSION_ID
  %aggrs_disk_list
  %aggr2Id
);
$VERSION = '0.01';
our @EXPORT_OK =
  qw(aggr_transition_start_verify vol_transition_start_verify aggr_transition_commit_verify transition_revert_start_verify transition_purge_info);

BEGIN {
    $SESSION_ID = `uuidgen`;
    chomp($SESSION_ID);
}

=head1 NAME

NACL::MTask::Import

=head1 DESCRIPTION

C<NACL::MTask::Import> provides a number of well-defined and 
multi-step methods related to Copy Free Transition - Import phase
functionality.

It builds on top of, the NACL's instanatComponent functionality
which creates NACL components in run time for the new ONTAP cli for 
CFT. This module contains methods equivalnat to following ONTAP storage transition commands.  

=head3 1) storage transition aggregates start

=head3 2) storage transition aggregates show-status

=head3 3) storage transition volumes start

=head3 4) storage transition volumes show-status

=head3 5) storage transition commit start.

=head3 6) storage transition commit show-status.

=head3 7) storage transition revert start.


C<NACL::MTask::Import> object can be created using constructor method "new" which 
generally be used in place of a component object.

=head1 ATTRIBUTES

=head2 command_interface

(Required) A component object that represents the host to which to send commands.

=head2 interface

(Optional) The interface with which the commands need to be sent i.e CLI/ZAPI etc.,

=head2 'method-timeout'

(Optional) method timeout value where the method should wait until the requied 
result to be obtained.

=head1 METHODS

=head2 new

  my $transObj = NACL::MTask::Import->new( 
				command_interface => $command_interface ,
				interface =>  "CLI",
				'method-timeout'  =>  60
				);
  
(Constructor Method) Creates an Import class object. This method creates instant component 
objects for new CFT transition ONTAP cli commands, also this method assigns some of
the variables with their default values, and creates a nodescoped apiset object for 
node level command executions.
	
This method also generates a random session id which will be used through out the object's life.

Note that the arguments C<interface>, C<method-timeout> are
optional for this method. Refer to the section
"Arguments defaulted/auto-determined" below for a description of the
values auto-assigned to these arguments if they are not supplied by the
caller. With these defaults, it is now possible to create a vserver by just
doing:

    my $transObj = NACL::MTask::Import->new( command_interface => $command_interface);

=over

=item Options

=over

=item C<< "command_interface" => $command_interface >>

(Required) A component object that represents the host to which to send commands.

=back

=back

=over

=item Arguments defaulted/auto-determined

=over

=item C<< "method-timeout" => 600 >>

(Optional) As component method-timeout, controls how long the command
will wait before completing.  However, the default timeout has been
raised to 600 seconds.

=item C<< interface => $interface >>

(Optional)
This option is to set the interface with which the commands need to be
send to node, i.e either CLI/ZAPI, it usually defaults to CLI, but 
when provided ZAPI option all the equivalant zapis for CLI commands
will be issued to node.

=back

=back

=cut

sub new {
    my ( $package, %args ) = @_;
    my ($zapiPrecommitObj, $res, $session_id) ; 

    # Validate parameters and populate member variables
    my $self = validate_with(
        params      => \%args,
        spec        => $package->_get_validation_spec(),
        allow_extra => 0
    );

    instant_component(
        component_name    => 'NACL::C::StorageTransitionAggregates',
        command_interface => $self->{command_interface}
    );

    instant_component(
        component_name    => 'NACL::C::StorageTransitionVolumes',
        command_interface => $self->{command_interface}
    );

    instant_component(
        component_name    => 'NACL::C::StorageTransitionCommit',
        command_interface => $self->{command_interface}
    );

    instant_component(
        component_name    => 'NACL::C::StorageTransitionRevert',
        command_interface => $self->{command_interface}
    );

    instant_component(
        component_name    => 'NACL::C::StorageTransitionPreCommit',
        command_interface => $self->{command_interface}
    );
    instant_component(
        component_name    => 'NACL::C::StorageTransition',
        command_interface => $self->{command_interface}
    );
    if ($self->{'interface'}  eq 'ZAPI' ) {
        $self->{zapiobj} = NACL::APISet->new(
        name      => $self->{command_interface}->node(),
        category  => "Node",
        interface => "ZAPI",
        set       => "CMode",
       );
    }

    $session_id =( defined $self->{session_id} )? delete $self->{session_id}: $SESSION_ID;

    if ( $self->{'interface'} eq 'ZAPI' ) {
        $zapiPrecommitObj = '<storage-transition-pre-commit-begin><session-id>'.$session_id.'</session-id>';
        $zapiPrecommitObj .= '<nodes><node-name>'.$self->{command_interface}->node().'</node-name></nodes>';
        $zapiPrecommitObj .= '</storage-transition-pre-commit-begin>';
        $res = $self->{zapiobj}->execute_raw_command( command => $zapiPrecommitObj,
                                                'return-response-object' => 1 );
    }
    else {
        NACL::C::StorageTransitionPreCommit->begin(
             command_interface => $self->{command_interface},
             'session-id'      => $session_id,
             "nodes"           => [ $self->{command_interface}->node() ],
        );
    }
    $SESSION_ID = $session_id;

    return bless( $self, $package );
}

=head2 aggr_transition_start_verify

Method invocation using Import Class Object:

	$transObj->aggr_transition_start_verify(
				aggrTransAttr      => \%aggrTransAttr,
				task_wait_timeout  => 2, 
				skip_status_verify => 0,
	);
	
Method Invocation via Class:
	
	NACL::MTask::Import->aggr_transition_start_verify(
				command_interface  => $command_interface,
				aggrTransAttr      => \%aggrTransAttr,
				task_wait_timeout  => 2, 
				skip_status_verify => 0,
	);
 
  
(Instance method) This method performs aggregate transition start for all the aggregates 
provided in the aggrTransAttr data strcuture, which is a hash of hash reference and verifies the transition status by default, but 
status verifiation can be skipped by providing skip_status_verify as 1. All the aggregates will
be transitioned using single ONTAP cli command and the status verification done for each aggregate
seperately. 


This method determines the aggregate uuids internally which are used to transition. Also verifies 
if there are any foriegn aggregates in cluster ready to be transitioned.

This method verifies the aggregate transition status until it reaches 'transition-complete' state,
otherwise throws NACL::Exceptions::UnexpectedState exception after 5 retries.

Note that the arguments C<task_wait_timeout>, C<skip_status_verify> are
optional for this method. Refer to the section
"Arguments defaulted/auto-determined" below for a description of the
values auto-assigned to these arguments if they are not supplied by the
caller. With these defaults, it is now possible to create a vserver by just
doing:

    $transObj->aggr_transition_start_verify( aggrTransAttr      => \%aggrTransAttr );

=over

=item Options

=over

=item C<< "aggrTransAttr" => \%$aggrTransAttr >>
(Required)
This is a hash reference of aggregate names, which again are hash references for each aggregate attributes such as 
aggrUUID, aggrSnap, aggrNewname etc., 


=item SubOptions

=over

=item C<< "aggrSnap" => $aggrSnap >>
(Required)

This sub option is a Scalar value which specifies aggregate pinning snapshot name created for 7mode aggregate.

=item C<< "aggrNewName" => $aggrNewName >>
(Optional)

This sub option is a Scalar value which specifies aggregate new name in cmode, after transition. if not provided this will be defaulted to 
aggregate name followed by "_Cmode". ex: aggrA_Cmode

=item C<< "aggrUUID" => $aggrUUID >>
(Optional)

This sub option is a Scalar value which specifies aggregate UUID, if not provided, this value will be fetched from 
system, this option is introduced mostly to support negative scenarios (with invalid aggrUUID etc.,).

=item C<< "ignore_errors" => $ignore_erors >>
(Optional)

This sub option is a Scalar value which specifies ERROR enums for aggregate if provided, to skip the transition failure. 

=item C<< "skip_offline_check" => $skip_offline_check >>
(Optional)

This sub option is a Boolean value, defaulted to 0, if this option is set, dblade aggregate status for offline/foriegn will not
be checked, again this option introduced to support negative test scenarios.

=item C<< "node_name" => $node_name >>
(Optional)

This sub option is a scalar, defaulted to command_interface, if this option is set, transition happends on the mentioned node

=back

=back

=over

Example datastruture for aggrTransAttr:

$aggrTransAttr = {
    'aggrA' => {
		'aggrSnap' 	=> "aggrA_Snap",    # Mandatroy Option
		'aggrNewName' 	=> "aggrA_new",     # Optional
		'aggrUUID' 	=> '91197ad0-cb8e-4a45-bdfb-7e19d88297d1',		# Optional
		'ignore_errors' => "",				# Optional
		'node_name'     =>  $node_name,				# Optional
		'skip_offline_check' => 1,			# Optional
	},
	'aggrB' => {
		'aggrSnap' 	=> "aggrA_Snap",
		'aggrNewName' 	=> "aggrA_new",
		'aggrUUID' 	=> '91197ad0-cb8e-4a45-bdfb-7e19d88297d1',
		'ignore_errors' => "",
		'node_name'     =>  $node_name,				# Optional
		'skip_offline_check' => 1,
	},
};

=back

=over

=item Arguments defaulted/auto-determined

=over

=item C<< task_wait_timeout => 60 >>

(Optional)
This is optional parameter, if provided, verification for the transition job status will be waiting for the mentioned
timeout. This option is provided to support long running job status verification.

=item C<< skip_status_verify => 1 >>

(Optional)
This is optional parameter, If this is set to 1, transition status verification will be skipped and the method returns the transition job ID.
Otherwise this option defaulted to 0, which verifies the transition status for each aggregate, and returns a hash of all aggregate transition status
parameters.

=back

=back

=over

=item Exceptions

=over

=item L<NACL::Exceptions::InvalidParam|lib-NACL-Exceptions-InvalidParam-pm>

If C<aggregates> is not provided, or if parameters given with wrong syntax, then this exception is thrown.

=item L<NACL::Exceptions::UnexpectedState|lib-NACL-Exceptions-UnexpectedState-pm>

If transition status verification not same as expected status (transition-complete), then this exception is thrown.

=back

=back

=cut

sub aggr_transition_start_verify {
    my ( $self, %args ) = @_;
    my (
        @aggrTransAttrs, $transAttr,          $aggrToUuid,
        $job,            $aggrTransAttr,      $taskWaitTimeout,
        %opts,           $skip_status_verify, $showAttrs,
        $showOpts,	 $node_name,          $zapiAttr,
        $res,            $job_id,             $session_id,
    );
    if (   ( ref($self) =~ /Import/i )
        && ( exists $self->{'command_interface'} ) )
    {
        $args{'command_interface'} = $self->{'command_interface'};
    }
    else {
        instant_component(
            component_name    => 'NACL::C::StorageTransitionAggregates',
            command_interface => $args{command_interface}
        );

        $self = _class_call_init( \%args );
    }

    %opts = validate_with(
        params      => \%args,
        spec        => $self->_aggr_validate_spec(),
        allow_extra => 1
    );
    if ( !%opts ) {
        NACL::Exceptions::InvalidParam->throw(
            "Need mandatory argument : array of aggregates");
    }
    else {
        $Node = delete $opts{command_interface};
        $node_name =
          ( defined $opts{node_name} )
          ? delete $opts{node_name}
          : $Node->node();
        $aggrTransAttr      = delete $opts{aggrTransAttr};
        $taskWaitTimeout    = delete $opts{task_wait_timeout};
        $skip_status_verify = delete $opts{skip_status_verify};
        $session_id         =( defined $opts{session_id} )? delete $opts{session_id}: $SESSION_ID;
    }
	print Dumper $opts{aggrTransAttr};
	print Dumper $aggrTransAttr;

    $zapiAttr = '<storage-transition-aggregates-start>';
    $zapiAttr .= '<session-id>'.$session_id.'</session-id>';
    $zapiAttr .= '<node-name>'.$node_name.'</node-name>';
    $zapiAttr .= '<aggr-trans-attr>';

    foreach my $aggr ( keys $aggrTransAttr ) {
        if ( !$aggrTransAttr->{$aggr}->{'skip_offline_check'} ) {
            &_check_foriegn_offline_aggr($aggr);
        }
        $zapiAttr .= '<trans-aggr-info>';

        if ( !( defined $aggrTransAttr->{$aggr}->{'aggrUUID'} ) ) {
            if ( exists $aggr2Id{ $aggr . "_" . $Node->name() } ) {
                $aggrTransAttr->{$aggr}->{'aggrUUID'} =
                  $aggr2Id{ $aggr . "_" . $Node->name() };
                $zapiAttr .=
                    '<aggr-uuid>'
                  . $aggrTransAttr->{$aggr}->{'aggrUUID'}
                  . '</aggr-uuid>';
            }
            else {
                $aggrToUuid = &_fetch_aggr_uuid();
                $aggrTransAttr->{$aggr}->{'aggrUUID'} =
                  $aggrToUuid->{ $aggr . "_" . $Node->name() };
                $zapiAttr .=
                    '<aggr-uuid>'
                  . $aggrToUuid->{ $aggr . "_" . $Node->name() }
                  . '</aggr-uuid>';
            }
        }
        $transAttr = $aggrTransAttr->{$aggr}->{'aggrUUID'};
        if ( defined $aggrTransAttr->{$aggr}->{'aggrSnap'} ) {
            $transAttr .= ":" . $aggrTransAttr->{$aggr}->{'aggrSnap'};
            $zapiAttr .=
                '<aggr-snapshot-name>'
              . $aggrTransAttr->{$aggr}->{'aggrSnap'}
              . '</aggr-snapshot-name>';
        }
        else {
            NACL::Exceptions::InvalidParam->throw(
                "Need mandatory argument : aggregate snapshot");
        }

        if ( defined $aggrTransAttr->{$aggr}->{'aggrNewName'} ) {
            $aggrTransAttr->{$aggr}->{'aggrNewName'} =~ s/\(.*\)//g;
            $transAttr .= ":" . $aggrTransAttr->{$aggr}->{'aggrNewName'};
            $zapiAttr .=
                '<aggr-new-name>'
              . $aggrTransAttr->{$aggr}->{'aggrNewName'}
              . '</aggr-new-name>';
        }
        else {
            $aggr =~ s/\(.*\)//g;
            $transAttr .= ":" . $aggr . "_Cmode";
            $zapiAttr  .= '<aggr-new-name>' . $aggr . '</aggr-new-name>';
        }
        if ( defined $aggrTransAttr->{$aggr}->{'ignore_errors'} ) {
            $transAttr .= ":" . $aggrTransAttr->{$aggr}->{'ignore_errors'};
            $zapiAttr  .= '<aggr-ignorable-errors>';
            $zapiAttr .=
                '<trans-aggr-ignore-error>'
              . $aggrTransAttr->{$aggr}->{'ignore_errors'}
              . '</trans-aggr-ignore-error>';
            $zapiAttr .= '</aggr-ignorable-errors>';
        }
        $zapiAttr .= '</trans-aggr-info>';
        push( @aggrTransAttrs, $transAttr );
        undef $transAttr;

    }
    $zapiAttr .= '</aggr-trans-attr>'.'</storage-transition-aggregates-start>';


    if ($self->{'interface'} eq "ZAPI") {
       try {
          $res = $self->{zapiobj}->execute_raw_command( command => $zapiAttr,
                                                       'return-response-object' => 1 );
          $job_id = $res->get_parsed_output()->[0]->{'result-jobid'}; 
          $job = NACL::CS::Job->fetch(
                                  command_interface => $Node,
                                  filter =>  {'id'  => $job_id},
                                );
       }
       catch NATE::BaseException with {
          my $exception = shift;
          print Dumper ($exception->text());
          print "Error-message: " .$exception->{'-response_object'}->{'error_reason'}. "\n";
          print "Error-code: " .$exception->{'-response_object'}->{'error_number'}."\n";
          print "Execution-status: " .$exception->{'-response_object'}->{'execution_status'}."\n";
          $exception->throw();
       };
    }
    else { 
       try {
          NACL::C::StorageTransitionAggregates->start(
             command_interface => $Node,
             'session-id'      => $session_id,
             'node-name'       => $node_name,
             'aggr-trans-attr' => \@aggrTransAttrs,
             job_component     => \$job,
          );
       }
       catch NATE::BaseException with {
          my $exception = shift;
          $exception->throw();
       };
    }
    if ($skip_status_verify) {
        return $job;
    }
    else {

        $showOpts->{'Node'}         = $Node;
        $showOpts->{'transAttrs'}   = $aggrTransAttr;
        $showOpts->{'JobID'}        = $job->{'id'};
        $showOpts->{'Type'}         = "AGGR";
        $showOpts->{'wait_timeout'} = $taskWaitTimeout;
        $showAttrs                  = &_verify_job_transition_status($showOpts);

    }
    return $showAttrs;

}

=head2 vol_transition_start_verify

Method invocation using Import Class Object:

	$transObj->vol_transition_start_verify( 
				volTransAttr       => \%volTransAttr,
				task_wait_timeout  => 2, 
				skip_status_verify => 0,
			);

Method Invocation via Class:
	
	NACL::MTask::Import->vol_transition_start_verify( 
				command_interface  => $command_interface,
				volTransAttr       => \%volTransAttr,
				task_wait_timeout  => 2, 
				skip_status_verify => 0,
			);
	
	  
(Instance method) This method performs volume transition start for all the volumes provided in the 
volTransAttr data strcuture, which is a hash of hash reference and verifies the transition status by default, but 
status verifiation can be skipped by providing skip_status_verify as 1. All the volumes will
be transitioned using single ONTAP cli command and the status verification done for each volume
seperately. 

This method determines the aggregate and volume uuids internally which will be used to transition.  Also verifies 
if there are any offline or restricted volumes in cluster ready to be transitioned.

This method verifies the volume transition status until it reaches 'transition-complete' state,
otherwise throws NACL::Exceptions::UnexpectedState exception after 5 retries.

Note that the arguments C<task_wait_timeout>, C<skip_status_verify> are
optional for this method. Refer to the section
"Arguments defaulted/auto-determined" below for a description of the
values auto-assigned to these arguments if they are not supplied by the
caller. With these defaults, it is now possible to create a vserver by just
doing:

    $transObj->vol_transition_start_verify( volTransAttr => \%volTransAttr);

=over

=item Options

=over

=item C<< volTransAttr => \%volTransAttr >>
(Required)
This is an hash reference of volume names,, which again are hash references to each volume attributes such as 
volUUID, volNewName, svm, ignore_errors etc.,  which needs to be passed for transitioning one or more
volumes.

=item SubOptions

=over

=item C<< "svm" => $svm >>
(Optional)

This sub option is a Scalar value, which specifies Cmode vserver name, where the 7mode volume get transitioned. if not provided, 
this value will be defaulted to any existing data vserver on cluster mode nodes.

=item C<< "volNewName" => $volNewName >>
(Optional)

This sub option is a Scalar value which specifies volume new name in cmode, after transition. if not provided this will be defaulted to 
volume name followed by "_Cmode". ex: volA_Cmode

=item C<< "volUUID" => $volUUID >>
(Optional)

This sub option is a Scalar value which specifies volume UUID, if not provided, this value will be fetched from 
system, this option is introduced mostly to support negative scenarios (with invalid volUUID etc.,).

=item C<< "ignore_errors" => $ignore_erors >>
(Optional)

This sub option is a Scalar value which specifies ERROR enums for volume if provided, to skip the transition failure. 

=item C<< "skip_offline_check" => $skip_offline_check >>
(Optional)

This sub option is a Boolean value, defaulted to 0, if this option is set, dblade volume status for offline/restricted will not
be checked, again this option introduced to support negative test scenarios.

=over

Example datastruture for volTransAttr:

$volTransAttr = {
    'volA' => { 
		'svm'	 		=> "vs0", 				# Mandatroy Option
		'volNewName' 		=> "volA_Cmode",     	# Optional
		'volUUID' 		=> '91197ad0-cb8e-4a45-bdfb-7e19d88297d1',		# Optional
		'ignore_errors' 	=> "",					# Optional
		'skip_offline_check'	=> 1,					# Optional
	},
	'volB' => {
		'svm' 			=> "vs1",
		'volNewName' 		=> "volB_Cmode",
		'volUUID' 		=> '91197ad0-cb8e-4a45-bdfb-7e19d88297d1',
		'ignore_errors' 	=> "",
		'skip_offline_check' 	=> 1,
	},
};

=back

=back

=back

=back

=item Arguments defaulted/auto-determined

=over

=item C<< task_wait_timeout => $task_wait_timeout >>

(Optional)
This is optional parameter, if provided, verification for the transition job status will be waiting for the mentioned
timeout. This option is provided to support long running job status verification.

=item C<< skip_status_verify => $skip_status_verify >>

(Optional)
This is optional parameter, If this is set to 1, transition status verification will be skipped and the method returns the transition job ID.
Otherwise this option defaulted to 0, which verifies the transition status for each aggregate, and returns a hash of all aggregate transition status
parameters.

=back

=item Exceptions

=over

=item L<NACL::Exceptions::InvalidParam|lib-NACL-Exceptions-InvalidParam-pm>

If C<aggregates> is not provided, or if parameters given with wrong syntax, then this exception is thrown.

=item L<NACL::Exceptions::UnexpectedState|lib-NACL-Exceptions-UnexpectedState-pm>

If transition status verification not same as expected status (transition-complete), then this exception is thrown.

=back

=cut

sub vol_transition_start_verify {
    my ( $self, %args ) = @_;
    my ( $job, @volTransAttrs, $transAttr, $volTransAttr, $taskWaitTimeout,
        $showAttrs, $skip_status_verify, $showOpts, $zapiAttr, $res, $job_id,
        $session_id );
    if (   ( ref($self) =~ /Import/i )
        && ( exists $self->{'command_interface'} ) )
    {
        $args{'command_interface'} = $self->{'command_interface'};
    }
    else {
        instant_component(
            component_name    => 'NACL::C::StorageTransitionVolumes',
            command_interface => $args{command_interface}
        );

        $self = _class_call_init( \%args );
    }

    my %opts = validate_with(
        params      => \%args,
        spec        => $self->_vol_validate_spec(),
        allow_extra => 1
    );
    if ( !%opts ) {
        NACL::Exceptions::InvalidParam->throw(
            "Need mandatory argument : array of volumes");
    }
    else {
        $volTransAttr       = delete $opts{volTransAttr};
        $Node               = $opts{'command_interface'};
        $taskWaitTimeout    = delete $opts{task_wait_timeout};
        $skip_status_verify = delete $opts{skip_status_verify};
        $session_id         =( defined $opts{session_id} )? delete $opts{session_id}: $SESSION_ID;

    }
    try {
        $dataVs = NACL::CS::Vserver->fetch(
            command_interface => $Node,
            filter            => { type => 'data', 'admin-state' => 'running' }
        )->vserver();
    }
    catch NACL::Exceptions::NoElementsFound with {
        my $exception = shift;
        $exception->throw("Atleast one vserver should exist");
    };
    
    $zapiAttr = '<storage-transition-volumes-start>'.'<session-id>'.$session_id.'</session-id>';
    $zapiAttr .= '<vol-trans-attr>';
    foreach my $vol ( keys $volTransAttr ) {
        &_prepare_vol_attrs( $vol,
            $volTransAttr->{$vol}->{'skip_offline_check'} );
        $zapiAttr .= '<trans-vol-info>';
        if ( defined $volTransAttr->{$vol}->{'volUUID'} ) {
            $transAttr =
                $volAttrs{$vol}{'aggrUUID'} . ":"
              . $volTransAttr->{$vol}->{'volUUID'} . ":"
              . $vol;
            $zapiAttr .=
                '<containing-aggr-uuid>'
              . $volAttrs{$vol}{'aggrUUID'}
              . '</containing-aggr-uuid>';
            $zapiAttr .=
              '<vol-uuid>' . $volTransAttr->{$vol}->{'volUUID'} . '</vol-uuid>';
        }
        else {
            $volTransAttr->{$vol}->{"volUUID"} = $volAttrs{$vol}{'volUUID'};
            $transAttr =
                $volAttrs{$vol}{'aggrUUID'} . ":"
              . $volAttrs{$vol}{'volUUID'} . ":"
              . $vol;
            $zapiAttr .=
                '<containing-aggr-uuid>'
              . $volAttrs{$vol}{'aggrUUID'}
              . '</containing-aggr-uuid>';
            $zapiAttr .=
              '<vol-uuid>' . $volAttrs{$vol}{'volUUID'} . '</vol-uuid>';
        }
        $zapiAttr .= '<vol-name>' . $vol . '</vol-name>';
        if ( defined $volTransAttr->{$vol}->{'svm'} ) {
            $transAttr .= ":" . $volTransAttr->{$vol}->{'svm'};
            $zapiAttr .=
              '<vserver>' . $volTransAttr->{$vol}->{'svm'} . '</vserver>';
        }
        else {
            $transAttr .= ":" . $dataVs;
            $zapiAttr  .= '<vserver>' . $dataVs . '</vserver>';
        }

        if ( defined $volTransAttr->{$vol}->{'volNewName'} ) {
            $volTransAttr->{$vol}->{'volNewName'} =~ s/\(.*\)//g;
            $transAttr .= ":" . $volTransAttr->{$vol}->{'volNewName'};
            $zapiAttr .=
                '<vol-new-name>'
              . $volTransAttr->{$vol}->{'volNewName'}
              . '</vol-new-name>';
        }
        else {
            $vol =~ s/\(.*\)//g;
            $transAttr .= ":" . $vol . "_Cmode";
            $zapiAttr .= '<vol-new-name>' . $vol . '_Cmode' . '</vol-new-name>';
        }
        if ( defined $volTransAttr->{$vol}->{'ignore_errors'} ) {
            $transAttr .= ":" . $volTransAttr->{$vol}->{'ignore_errors'};
            $zapiAttr  .= '<vol-ignorable-errors>';
            $zapiAttr .=
                '<trans-ignorable-errors>'
              . $volTransAttr->{$vol}->{'ignore-errors'}
              . '</trans-ignorable-errors>';
            $zapiAttr .= '</vol-ignorable-errors>';
        }
        $zapiAttr .= '</trans-vol-info>';
        push( @volTransAttrs, $transAttr );
        undef $transAttr;
   }

    $zapiAttr .= '</vol-trans-attr>'.'</storage-transition-volumes-start>';

    if ($self->{'interface'} eq "ZAPI") {
        try {
            $res = $self->{zapiobj}->execute_raw_command(
                command                  => $zapiAttr,
                'return-response-object' => 1
            );
            $job_id =
              $res->get_parsed_output()->[0]->{'results'}->[0]
              ->{'result-jobid'};
            $job = NACL::CS::Job->fetch(
                command_interface => $Node,
                filter            => { 'id' => $job_id },
            );
        }
        catch NATE::BaseException with {
            my $exception = shift;
            $exception->throw();
        };

    }

    else{
       try {
           NACL::C::StorageTransitionVolumes->start(
               command_interface => $Node,
               'session-id'      => $session_id,
               'vol-trans-attr'  => \@volTransAttrs,
               job_component     => \$job,
           );
       }
       catch NATE::BaseException with {
           my $exception = shift;
           $exception->throw();
       };
    }
    if ($skip_status_verify) {
        return $job;
    }
    else {

        $showOpts->{'Node'}         = $Node;
        $showOpts->{'transAttrs'}   = $volTransAttr;
        $showOpts->{'JobID'}        = $job->{'id'};
        $showOpts->{'Type'}         = "VOL";
        $showOpts->{'wait_timeout'} = $taskWaitTimeout;
        $showAttrs                  = &_verify_job_transition_status($showOpts);

    }
    return $showAttrs;
}

=head2 aggr_transition_commit_verify

Method invocation using Import Class Object:

	$transObj->aggr_transition_commit_verify(
				aggrTransAttr      => \%aggrTransAttr,
				task_wait_timeout  => 2, 
				skip_status_verify => 0,
	);
	
Method Invocation via Class:
	
	NACL::MTask::Import->aggr_transition_commit_verify(
				command_interface  => $command_interface,
				aggrTransAttr      => \%aggrTransAttr,
				task_wait_timeout  => 2, 
				skip_status_verify => 0,
	);

(Instance/Class method) This method performs aggregate transition commit for all the aggregates provided in the 
aggrTransAttr data strcuture, which is a hash of hash reference and verifies the transition status by default, but 
status verifiation can be skipped by providing skip_status_verify as 1. All the aggregates will
be committed using single ONTAP cli command and the status verification done for each aggregate
seperately. 

This method determines the aggregate uuids internally which will be used for transition commit. 

This method verifies the transition commit status for aggregate until it reaches 'commit-complete' state,
otherwise throws NACL::Exceptions::UnexpectedState exception after 5 retries.

Note that the arguments C<task_wait_timeout>, C<skip_status_verify> are
optional for this method. Refer to the section
"Arguments defaulted/auto-determined" below for a description of the
values auto-assigned to these arguments if they are not supplied by the
caller. With these defaults, it is now possible to create a vserver by just
doing:

    $transObj->aggr_transition_commit_verify( "aggrTransAttr" => \%$aggrTransAttr);

=over

=item Options

=over

=item C<< "aggrTransAttr" => \%$aggrTransAttr >>
(Required)
This is a hash reference of aggregate names, which again are hash references to each aggregate attributes such as 
aggrUUID, ignore_errors etc., 

=item SubOptions

=over

=item C<< "aggrUUID" => $aggrUUID >>
(Optional)

This sub option is a Scalar value which specifies aggregate UUID, if not provided, this value will be fetched from 
system, this option is introduced mostly to support negative scenarios (with invalid aggrUUID etc.,).

=item C<< "ignore_errors" => $ignore_erors >>
(Optional)

This sub option is a Scalar value which specifies ERROR enums for aggregate if provided, to skip the commit failure. 

=back

=over

Example datastruture for aggrTransAttr:

$aggrTransAttr = {
    'aggrA' => { 
		'aggrUUID' 	=> '91197ad0-cb8e-4a45-bdfb-7e19d88297d1', # Optional
		'ignore_errors' => "",	# Optional
	},
	'aggrB' => {
		'aggrUUID' 	=> '91197ad0-cb8e-4a45-bdfb-7e19d88297d1',
		'ignore_errors' => "",
	},
};

=back

=item Arguments defaulted/auto-determined

=over

=item C<< task_wait_timeout => 60 >>

(Optional)
This is optional parameter, if provided, verification for the transition job status will be waiting for the mentioned
timeout. This option is provided to support long running job status verification.

=item C<< skip_status_verify => 1 >>

(Optional)
This is optional parameter, If this is set to 1, transition status verification will be skipped and the method returns the transition job ID.
Otherwise this option defaulted to 0, which verifies the transition status for each aggregate, and returns a hash of all aggregate transition status
parameters.

=back

=back

=over

=item Exceptions

=over

=item L<NACL::Exceptions::InvalidParam|lib-NACL-Exceptions-InvalidParam-pm>

If C<aggregates> is not provided, or if parameters given with wrong syntax, then this exception is thrown.

=item L<NACL::Exceptions::UnexpectedState|lib-NACL-Exceptions-UnexpectedState-pm>

If transition status verification not same as expected status (transition-complete), then this exception is thrown.

=back

=back

=cut

sub aggr_transition_commit_verify {
    my ( $self, %args ) = @_;
    my (
        $aggrToUuid,    @aggrTransAttrs,     $transAttr,
        $aggrTransAttr, $taskWaitTimeout,    $job,
        $showAttrs,     $skip_status_verify, $showOpts,
        $zapiAttr,      $res,                $job_id,
        $session_id,
    );

    if (   ( ref($self) =~ /Import/i )
        && ( exists $self->{'command_interface'} ) )
    {
        $args{'command_interface'} = $self->{'command_interface'};
    }
    else {
        instant_component(
            component_name    => 'NACL::C::StorageTransitionCommit',
            command_interface => $args{command_interface}
        );

       $self =  _class_call_init( \%args );
    }
    my %opts = validate_with(
        params      => \%args,
        spec        => $self->_aggr_validate_spec(),
        allow_extra => 1 
    );
    if ( !%opts ) {
        NACL::Exceptions::InvalidParam->throw(
            "Need mandatory argument : array of aggregates");
    }
    else {
        $aggrTransAttr      = delete $opts{aggrTransAttr};
        $Node               = $self->{'command_interface'};
        $taskWaitTimeout    = delete $opts{task_wait_timeout};
        $skip_status_verify = delete $opts{skip_status_verify};
        $session_id         =( defined $opts{session_id} )? delete $opts{session_id}: $SESSION_ID;
    }
    $zapiAttr = '<storage-transition-commit-start>'.'<session-id>'.$session_id.'</session-id>';
    $zapiAttr .= '<aggr-trans-commit-attr>';

    foreach my $aggr ( keys $aggrTransAttr ) {
        $zapiAttr .= '<trans-aggr-commit-info>';
        if ( !( defined $aggrTransAttr->{$aggr}->{'aggrUUID'} ) ) {
            $transAttr = &_fetch_aggr_to_be_reverted_uuid($aggr);
            $aggrTransAttr->{$aggr}->{'aggrUUID'} = $transAttr;
        }
        else {
            $transAttr = $aggrTransAttr->{$aggr}->{'aggrUUID'};
        }
        $zapiAttr .=
            '<aggr-uuid>'
          . $aggrTransAttr->{$aggr}->{'aggrUUID'}
          . '</aggr-uuid>';
        if ( defined $aggrTransAttr->{$aggr}->{'ignore_errors'} ) {
            $transAttr .= ":" . $aggrTransAttr->{$aggr}->{'ignore_errors'};
            $zapiAttr .=
                '<commit-ignorable-errors>'
              . '<trans-commit-ignore-error>'
              . $aggrTransAttr->{$aggr}->{'ignore_errors'}
              . '</trans-commit-ignore-errors>';
            $zapiAttr .= '</commit-ignorable-errors>';

        }
        $zapiAttr .= '</trans-aggr-commit-info>';
        push( @aggrTransAttrs, $transAttr );
        undef $transAttr;
    }
    $zapiAttr .= '</aggr-trans-commit-attr>'.'</storage-transition-commit-start>';
    if ($self->{'interface'} eq "ZAPI") {
        try {
            $res = $self->{zapiobj}->execute_raw_command(
                command                  => $zapiAttr,
                'return-response-object' => 1
            );
            $job_id =
              $res->get_parsed_output()->[0]->{'results'}->[0]
              ->{'result-jobid'};
            $job = NACL::CS::Job->fetch(
                command_interface => $Node,
                filter            => { 'id' => $job_id },
            );
        }
        catch NATE::BaseException with {
            my $exception = shift;
            $exception->throw();
        };
    }
    else {
       try {
          NACL::C::StorageTransitionCommit->start(
             command_interface        => $Node,
             'session-id'             => $session_id,
             'aggr-trans-commit-attr' => \@aggrTransAttrs,
             job_component            => \$job,
          );

        }
        catch NATE::BaseException with {
            my $exception = shift;
            $exception->throw();
        };
    }
    if ($skip_status_verify) {
        return $job;
    }
    else {
        $showOpts->{'Node'}         = $Node;
        $showOpts->{'transAttrs'}   = $aggrTransAttr;
        $showOpts->{'JobID'}        = $job->{'id'};
        $showOpts->{'Type'}         = "COMMIT";
        $showOpts->{'wait_timeout'} = $taskWaitTimeout;
        $showAttrs                  = &_verify_job_transition_status($showOpts);
    }
    return $showAttrs;
}

=head2 transition_revert_start_verify

Method invocation using Import Class Object:

    my $showAttrs = $transObj->transition_revert_start_verify (
        'aggrTransAttr'    => \%aggrTransAttr ,
        'kernalVersion'    => "kern_vers_8_2"
        task_wait_timeout  => 2, 
        skip_status_verify => 0,
    );
Method Invocation via Class:

    NACL::MTask::Import->transition_revert_start_verify(
        command_interface  => $command_interface,
        aggrTransAttr      => \%aggrTransAttr,
        'kernalVersion'    => "kern_vers_8_2"
        task_wait_timeout  => 2, 
        skip_status_verify => 0,
    );
 
  
(Instance method) This method performs transition revert start for all the aggregates 
provided in the aggrTransAttr data strcuture, which is a hash of hash reference and verifies the transition revert status by default, but 
status verifiation can be skipped by providing skip_status_verify as 1. All the aggregates will
be reverted using single ONTAP cli command and the status verification done for each aggregate
seperately. 


This method determines the aggregate uuids internally which are used to transition.

This method verifies the aggregate transition revert status until it reaches 'revert-complete' state,
otherwise throws NACL::Exceptions::UnexpectedState exception after 5 retries.

Note that the arguments C<task_wait_timeout>, C<skip_status_verify> are
optional for this method. Refer to the section
"Arguments defaulted/auto-determined" below for a description of the
values auto-assigned to these arguments if they are not supplied by the
caller.

=over

=item Options

=over

=item C<< "aggrTransAttr" => \%$aggrTransAttr >>
(Required)
This is a hash reference of aggregate names that are to be reverted, which again are hash references for each aggregate attributes such as 
aggrUUID, aggrSnap, sysId etc., 


=item SubOptions

=over

=item C<< "aggrSnap" => $aggrSnap >>
(Required)

This sub option is a Scalar value which specifies aggregate pinning snapshot name created for 7mode aggregate.

=item C<< "sysId" => $sysId >>
(Required)

This sub option is a Scalar value which specifies system id of disks of aggregates.

=item C<< "isRoot" => $isRoot >>
(Optional)

This sub option is a Boolean value which specifies if the aggregate is a root aggregate.

=item C<< "aggrUUID" => $aggrUUID >>
(Optional)

This sub option is a Scalar value which specifies aggregate UUID, if not provided, this value will be fetched from 
system, this option is introduced mostly to support negative scenarios (with invalid aggrUUID etc.,).

=item C<< "ignore_errors" => $ignore_erors >>
(Optional)

This sub option is a Scalar value which specifies ERROR enums for aggregate if provided, to skip the transition failure.

=item C<< "node_name" => $node_name >>
(Optional)

This sub option is a scalar, defaulted to command_interface, if this option is set, transition happends on the mentioned node

=back

=back

=over

Example datastruture for aggrTransAttr:

$aggrTransAttr = {
    'aggrA' => {
        'aggrSnap'              => "aggrA_Snap",    # Mandatroy Option
        'sysId'                 => "4074480079",    # Mandatroy Option
        'isRoot'                => "false",         # Optional
        'aggrUUID'              => '91197ad0-cb8e-4a45-bdfb-7e19d88297d1',      # Optional
        'ignore_errors'         => "",              # Optional
        'skip_status_verify'    => 1,               # Optional
    },
    'aggrB' => {
        'aggrSnap'              => "aggrB_Snap",    # Mandatroy Option
        'sysId'                 => "4074480079",    # Mandatroy Option
        'isRoot'                => "false",         # Optional
        'aggrUUID'              => '91197ad0-cb8e-4a45-bdfb-7e19d88297d1',      # Optional
        'ignore_errors'         => "",              # Optional
        'skip_status_verify'    => 1,               # Optional
    },
};

=back

=over

=item Arguments defaulted/auto-determined

=over

=item C<< "kernalVersion" => $kernalVersion >>
(Optional)

This sub option is a Scalar value which specifies version of 7 Mode kernal to which the disks will be reverted.

=item C<< task_wait_timeout => 60 >>

(Optional)
This is optional parameter, if provided, verification for the transition job status will be waiting for the mentioned
timeout. This option is provided to support long running job status verification.

=item C<< skip_status_verify => 1 >>

(Optional)
This is optional parameter, If this is set to 1, transition status verification will be skipped and the method returns the transition job ID.
Otherwise this option defaulted to 0, which verifies the transition status for each aggregate, and returns a hash of all aggregate transition status
parameters.

=item C<< "node_name" => $node_name >>
(Optional)

This sub option is a scalar, defaulted to command_interface, if this option is set, transition happends on the mentioned node

=back

=back

=over

=item Exceptions

=over

=item L<NACL::Exceptions::InvalidParam|lib-NACL-Exceptions-InvalidParam-pm>

If C<aggregates> is not provided, or if parameters given with wrong syntax, then this exception is thrown.

=item L<NACL::Exceptions::UnexpectedState|lib-NACL-Exceptions-UnexpectedState-pm>

If transition status verification not same as expected status (revert-complete), then this exception is thrown.

=back

=back

=cut

sub transition_revert_start_verify {
    my ( $self, %args ) = @_;
    my (
        @aggrTransAttrs, $transAttr,          $aggrToUuid,
        $job,            $aggrTransAttr,      $taskWaitTimeout,
        %opts,           $skip_status_verify, $showAttrs,
        $showOpts,       $kernalVersion,      $node_name,
        $zapiAttr,       $res,                $job_id,
        $session_id, 
    );
    if (   ( ref($self) =~ /Import/i )
        && ( exists $self->{'command_interface'} ) )
    {
        $args{'command_interface'} = $self->{'command_interface'};
    }
    else {
        instant_component(
            component_name    => 'NACL::C::StorageTransitionRevert',
            command_interface => $args{command_interface}
        );

        $self = _class_call_init( \%args );
    }

    %opts = validate_with(
        params      => \%args,
        spec        => $self->_revert_validate_spec(),
        allow_extra => 1 
    );
    if ( !%opts ) {
        NACL::Exceptions::InvalidParam->throw(
            "Need mandatory argument : array of aggregates to be reverted");
    }
    else {
        $Node = delete $opts{command_interface};
        $node_name =
          ( defined $opts{node_name} )
          ? delete $opts{node_name}
          : $Node->node();
        $aggrTransAttr      = delete $opts{aggrTransAttr};
        $kernalVersion      = delete $opts{kernalVersion};
        $taskWaitTimeout    = delete $opts{task_wait_timeout};
        $skip_status_verify = delete $opts{skip_status_verify};
        $session_id         =( defined $opts{session_id} )? delete $opts{session_id}: $SESSION_ID;

    }

    $zapiAttr = '<storage-transition-revert-start>'.'<session-id>'.$session_id.'</session-id>';
    $zapiAttr .= '<node-name>'.$node_name.'</node-name>';
    $zapiAttr .= '<aggr-trans-revert-attr>';
    foreach my $aggr ( keys $aggrTransAttr ) {
        $zapiAttr .= '<trans-revert-info>';
        $aggrToUuid = &_fetch_aggr_to_be_reverted_uuid($aggr);
        if ( defined $aggrTransAttr->{$aggr}->{'aggrUUID'} ) {
            $transAttr = $aggrTransAttr->{$aggr}->{'aggrUUID'};
            $zapiAttr .=
                '<aggr-uuid>'
              . $aggrTransAttr->{$aggr}->{'aggrUUID'}
              . '</aggr-uuid>';
        }
        else {
            $aggrTransAttr->{$aggr}->{'aggrUUID'} = $aggrToUuid;
            $transAttr = $aggrToUuid;
            $zapiAttr .= '<aggr-uuid>' . $aggrToUuid . '</aggr-uuid>';
        }
        if ( defined $aggrTransAttr->{$aggr}->{'sysId'} ) {
            $transAttr .= ":" . $aggrTransAttr->{$aggr}->{'sysId'};
            $zapiAttr .=
                '<system-id>'
              . $aggrTransAttr->{$aggr}->{'sysId'}
              . '</system-id>';
        }
        else {
            NACL::Exceptions::InvalidParam->throw(
                "Need mandatory argument : sysId");
        }
        if ( defined $aggrTransAttr->{$aggr}->{'aggrSnap'} ) {
            $transAttr .= ":" . $aggrTransAttr->{$aggr}->{'aggrSnap'};
            $zapiAttr .=
                '<aggr-snapshot-name>'
              . $aggrTransAttr->{$aggr}->{'aggrSnap'}
              . '</aggr-snapshot-name>';
        }
        else {
            NACL::Exceptions::InvalidParam->throw(
                "Need mandatory argument : aggregate snapshot");
        }
        if ( defined $aggrTransAttr->{$aggr}->{'isRoot'} ) {
            $transAttr .= ":" . $aggrTransAttr->{$aggr}->{'isRoot'};
            $zapiAttr .=
              '<is-root>' . $aggrTransAttr->{$aggr}->{'isRoot'} . '</is-root>';
        }
        else {
            $transAttr .= ":false";
            $zapiAttr  .= '<is-root>' . 'false' . '</is-root>';
        }

        if ( defined $aggrTransAttr->{$aggr}->{'ignore_errors'} ) {
            $transAttr .= ":" . $aggrTransAttr->{$aggr}->{'ignore_errors'};
            $zapiAttr .=
                '<revert-ignorable-errors>'
              . '<trans-revert-ignore-errors>'
              . $aggrTransAttr->{$aggr}->{'ignore_errors'}
              . '</trans-revert-ignore-errors>'
              . '</revert-ignorable-errors>';
        }
        $zapiAttr .= '</trans-revert-info>';
        push( @aggrTransAttrs, $transAttr );
        undef $transAttr;
        try {
            my $aggr_cs = NACL::CS::Aggregate->fetch(
                command_interface => $Node,
                filter            => { aggregate => $aggr },
                requested_fields  => [qw(disklist)]
            );
            my @disk_list = $aggr_cs->disklist();
            $aggrs_disk_list{$aggr} = \@disk_list;
        }
        catch NATE::BaseException with {
            my $excepObj = shift;
            my @disks;
            my $raidDisks =
              $Node->get_7m_or_nodescope_apiset()
              ->aggr_status( 'aggregate' => $aggr, 'raid-disk' => 1 );
            foreach my $raidObj ( @{ $raidDisks->get_parsed_output() } ) {
                foreach my $disk (
                    @{
                        $raidObj->{aggregate}[0]->{plex}[0]->{raid_group}[0]
                          ->{disks}
                    }
                  )
                {
                    push( @disks, $disk->{'device'} );
                }
            }
            $aggrs_disk_list{$aggr} = \@disks;

        };
     
    }
    $zapiAttr .= '</aggr-trans-revert-attr>';

    my $kerVerZapi = ( split( /[a-z]_/, $kernalVersion ) )[-1];
    $kerVerZapi =~ s/_/./g;
    $zapiAttr .= '<kernel-version>' . $kerVerZapi . '</kernel-version>';
    $zapiAttr .= '</storage-transition-revert-start>';
    
    if ($self->{'interface'} eq "ZAPI") {
       try {
           $res = $self->{zapiobj}->execute_raw_command( command => $zapiAttr,
                                                        'return-response-object' => 1 );
           $job_id = $res->get_parsed_output()->[0]->{'result-jobid'}; 
           $job = NACL::CS::Job->fetch(
                   command_interface => $Node,
                   filter =>  {'id'  => $job_id},
           );
       }
       catch NATE::BaseException with {
          my $exception = shift;
          $exception->throw();
       };
    }
    else { 
       try {
          NACL::C::StorageTransitionRevert->start(
             command_interface        => $Node,
             'session-id'             => $session_id,
             'node-name'              => $node_name,
             'aggr-trans-revert-attr' => \@aggrTransAttrs,
             'kernel-version'         => $kernalVersion,
             job_component            => \$job,
          );
       }
       catch NATE::BaseException with {
          my $exception = shift;
          $exception->throw();
       };
    }
    if ($skip_status_verify) {
        return $job;
    }
    else {

        $showOpts->{'Node'}         = $Node;
        $showOpts->{'transAttrs'}   = $aggrTransAttr;
        $showOpts->{'JobID'}        = $job->{'id'};
        $showOpts->{'Type'}         = "REVERT";
        $showOpts->{'wait_timeout'} = $taskWaitTimeout;

        $showAttrs = &_verify_job_transition_status($showOpts);
    }
    return $showAttrs;

}

=head2 transition_purge_info

Method invocation using Import Class Object:

    my $showAttrs = $transObj->transition_purge_info (
        'session-id'  => $SESSION_ID, 
    );
    
Method Invocation via Class:

    NACL::MTask::Import->transition_purge_info (
        command_interface  => $command_interface,
        'session-id'  => $SESSION_ID, 
    );
 
  
(Instance method) This method removes the transition session information from the status tables on mentioned node for the mentioned session id.
If node-name is not speicified explicitly it defaults to the node mentioned in command_interface. Also if session id is not mentioned, it defaults
to the session id generated by Import.pm.

Note that the arguments C<session-id>, C<node-name> are
optional for this method. Refer to the section
"Arguments defaulted/auto-determined" below for a description of the
values auto-assigned to these arguments if they are not supplied by the
caller.

=over

=item Options

=over

=item Arguments defaulted/auto-determined

=over

=item C<< "session-id" => $SESSION_ID >>
(Optional)

This option is a Scalar value which specifies the transition session id for which the information needs to be removed from status tables, if not 
defined, this value defaults to the session id generated by Import.pm

=item C<< node-name => $Node->name() >>

(Optional)
This option is a Scalar value which specifies the node name on which the transition session information needs to be removed, if not defined, this value
defaults to the node mentioned in command_interface. 

=back

=back

=over

=item Exceptions

=over

=item L<NACL::Exceptions::InvalidParam|lib-NACL-Exceptions-InvalidParam-pm>

if parameters given with wrong syntax, then this exception is thrown.

=back

=back

=cut

sub transition_purge_info {
    my ( $self, %args ) = @_;
    my ( $nodeName, %opts, $session_id, $zapiAttr, $res, $zapiPreCommit,
        $job_id );
    if (   ( ref($self) =~ /Import/i )
        && ( exists $self->{'command_interface'} ) )
    {
        $args{'command_interface'} = $self->{'command_interface'};
    }
    else {
        instant_component(
            component_name    => 'NACL::C::StorageTransition',
            command_interface => $args{command_interface}
        );
    }

    %opts = validate_with(
        params      => \%args,
        spec        => $self->_purge_validate_spec(),
        allow_extra => 1
    );
    if ( !%opts ) {
        NACL::Exceptions::InvalidParam->throw(
            "Need mandatory argument : command_interface");
    }
    else {
        $Node = delete $opts{command_interface};
        $nodeName =
          ( defined $opts{'node_name'} )
          ? delete $opts{'node_name'}
          : $Node->name();
        $session_id =
          ( defined $opts{'session_id'} )
          ? delete $opts{'session_id'}
          : $SESSION_ID;
    }
    $zapiPreCommit = '<storage-transition-pre-commit-end><session-id>'.$session_id.'</session-id></storage-transition-pre-commit-end>';

    $zapiAttr = '<storage-transition-purge-info>';
    $zapiAttr .= '<session-id>'.$session_id.'</session-id>';
    $zapiAttr .= '<node-name>'.$nodeName.'</node-name>'; 
    $zapiAttr .= '</storage-transition-purge-info>';
    if ($self->{'interface'} eq "ZAPI") {
        $res = $self->{zapiobj}->execute_raw_command( command => $zapiPreCommit,
                                                'return-response-object' => 1 );
        $res = $self->{zapiobj}->execute_raw_command( command => $zapiAttr,
                                                'return-response-object' => 1 );
    }
    else {
       try {
          NACL::C::StorageTransitionPreCommit->end(
             command_interface => $Node,
             'session-id'      => $session_id,
          );
          NACL::C::StorageTransition->purge_info(
             command_interface => $Node,
             'session-id'      => $session_id,
          );
          NACL::C::StorageTransitionAggregates->show_status(
             command_interface => $Node, allow_empty => 1  );
          NACL::C::StorageTransitionVolumes->show_status(
             command_interface => $Node, allow_empty => 1);
       }
       catch NATE::BaseException with {
          my $exception = shift;
          print Dumper ($exception->text());
          $exception->throw();
       };
    }
}

######################################################################
######################### HELPER ROUTINES ############################
######################################################################
#
# Method to execute transition job status verification and transition status verification in parallel

sub _verify_job_transition_status {
    my $showOpts = shift;
    my $showAttrs;

    my $statusCheck = NATE::ProcManager->new(
        proc_info => [
            {
                codespec       => \&_verify_job_completion,
                inherit_params => 1,
                args           => [$showOpts],
                runid          => "JobStatus",
                onexit => \&NATE::Process::on_exit_propagate_worst_result,
            },
        ],
        parallel => 1,
    );

    $statusCheck->start;
    $showAttrs = &_verify_transition_status($showOpts);
    $statusCheck->wait();
    if ( NATE::Result->is_failure( $statusCheck->worst_result ) ) {
        NACL::Exceptions::JobFailed->throw('Transition Job Failed...');
    }
    return $showAttrs;
}

# This method verifies transition job status and polls until success or waits for 2 iterations with
# a sleep of 2 seconds, if job not succeeded, returns the failure result. This method also wait for certain
# period of time, mentioned by optional parameter "wait_timeout", in order to facilitate job verification for long
# running jobs.

sub _verify_job_completion {
    my $jobOpts = shift;
    my $result  = 1;
    my $count   = 0;
    my ( $transJob, $jobHistory );
    try {
        $transJob = NACL::CS::Job->fetch(
            command_interface => $jobOpts->{'Node'},
            filter            => { 'id' => $jobOpts->{'JobID'} },
        );
    }
    catch NACL::Exceptions::NoElementsFound with {
        my $exception = shift;
        $jobHistory = NACL::CS::JobHistory->fetch(
            command_interface => $jobOpts->{'Node'},
            filter            => { 'id' => $jobOpts->{'JobID'} },
        );
        if ( $jobHistory->event_type() !~ m/Succeeded/i ) {
            $result &= 0;
        }
    };
    if ( defined $transJob ) {

# default time wait for job state is 60 seconds, polling in 10 interations with 6 seconds delay.
        my $timeout =
            ( defined $jobOpts->{'wait_timeout'} )
          ? ( $jobOpts->{'wait_timeout'} / 10 )
          : 6;
        while (( $transJob->state() !~ /success/i )
            && ( $count < 10 ) )
        {
            sleep $timeout;
            try {
                $transJob = NACL::CS::Job->fetch(
                    command_interface => $jobOpts->{'Node'},
                    ,
                    filter => { 'id' => $jobOpts->{'JobID'} },
                );
            }
            catch NACL::Exceptions::NoElementsFound with {
                my $exception = shift;
                $jobHistory = NACL::CS::JobHistory->fetch(
                    command_interface => $jobOpts->{'Node'},
                    filter            => { 'id' => $jobOpts->{'JobID'} },
                );
                if ( $jobHistory->event_type() !~ m/Succeeded/i ) {
                    $exception->throw();
                }
                last;
            };
            $count++;
        }
        if ( $transJob->state() !~ m/success/i ) {
            NACL::Exceptions::UnexpectedState->throw( "Unexpected state : "
                  . $transJob->state()
                  . " Expected state: success " );
        }
    }
    return $result;
}

# This method verifies transition status and polls for 5 iterations and fails
# if status is not "transition-complete" or "revert-complete" for each aggregate/volume/aggregate commit/revert
# Also this method fills up a hash of hashes for aggregate/volume, with their transition status
# parameters, and returns them to user, to validate those parameters.

sub _verify_transition_status {
    my $showOpts = shift;
    my $objects  = $showOpts->{'transAttrs'};

    my (
        $transStatus,     $transAggrShow,  $transVolShow,
        $transCommitShow, %showAttributes, $respObject
    );
    my $result = 1;
    my $count  = 0;
    if ( $showOpts->{'Type'} eq "AGGR" ) {
        my $failCount = 0;
        foreach my $aggr ( keys $objects ) {
            $transAggrShow = NACL::C::StorageTransitionAggregates->show_status(
                command_interface => $showOpts->{'Node'},
                filter => { 'aggr-uuid' => $objects->{$aggr}->{'aggrUUID'} },
            );
            $transStatus = $transAggrShow->status_code();
            while (( $transStatus !~ /transition[-_]complete/i )
                && ( $count < 4 ) )
            {
                sleep 5;
                $transAggrShow =
                  NACL::C::StorageTransitionAggregates->show_status(
                    command_interface => $showOpts->{'Node'},
                    filter => { 'aggr-uuid' => $objects->{$aggr}->{'aggrUUID'} }
                  );
                $transStatus = $transAggrShow->status_code();
                $count++;
            }
            $showAttributes{$aggr}{'status_code'} =
              $transAggrShow->status_code();
            $showAttributes{$aggr}{'error_code'} = $transAggrShow->error_code();
            $showAttributes{$aggr}{'error_string'} =
              $transAggrShow->error_string();
            $showAttributes{$aggr}{'aggr_new_name'} =
              $transAggrShow->aggr_new_name();
            $showAttributes{$aggr}{'corrective_action'} =
              $transAggrShow->corrective_action();
            $showAttributes{$aggr}{'snapshot_name'} =
              $transAggrShow->snapshot_name();
            $showAttributes{$aggr}{'ignore_errors'} =
              $transAggrShow->ignore_errors();
            $count = 0;

            if ( $transStatus =~ /transition[-_]complete/i ) {
                my $state = NACL::CS::StorageAggregate->fetch(
                    command_interface => $showOpts->{'Node'},
                    filter =>
                      { aggregate => $objects->{$aggr}->{'aggrNewName'} },
                )->state();
                if ( $state !~ m/online/i ) {
                    NACL::Exceptions::UnexpectedState->throw(
                        "Unexpected state, Expected aggr status : online,
                        actual status : " . $state
                    );
                }
            }
            else {
                $failCount++;
            }
        }
        $showAttributes{'JOB'} = $showOpts->{'JobID'};
        if ($failCount) {
            $respObject = bless( \%showAttributes, 'NACL::APISet::Response' );
            NACL::APISet::Exceptions::CommandFailedException->throw(
                text =>
"One or more of the aggregates passed in the input is not showing transition\-complete. More info in response_object.",
                response_object => $respObject
            );
        }

    }
    elsif ( $showOpts->{'Type'} eq "COMMIT" ) {
        my $failCount = 0;
        foreach my $aggr ( keys $objects ) {

            $transCommitShow = NACL::C::StorageTransitionCommit->show_status(
                command_interface => $showOpts->{'Node'},
                filter => { 'aggr-uuid' => $objects->{$aggr}->{'aggrUUID'} },
            );
            $transStatus = $transCommitShow->status_code();
            while (( $transStatus !~ m/commit[-_]complete/i )
                && ( $count < 4 ) )
            {
                sleep 5;
                $transCommitShow =
                  NACL::C::StorageTransitionCommit->show_status(
                    command_interface => $showOpts->{'Node'},
                    filter =>
                      { 'aggr-uuid' => $objects->{$aggr}->{'aggrUUID'} },
                  );
                $transStatus = $transCommitShow->status_code();
                $count++;
            }
            $count = 0;
            $showAttributes{$aggr}{'status_code'} =
              $transCommitShow->status_code();
            $showAttributes{$aggr}{'error_code'} =
              $transCommitShow->error_code();
            $showAttributes{$aggr}{'error_string'} =
              $transCommitShow->error_string();
            $showAttributes{$aggr}{'corrective_action'} =
              $transCommitShow->corrective_action();
            $showAttributes{$aggr}{'aggr_uuid'} = $transCommitShow->aggr_uuid();
            $showAttributes{$aggr}{'session_id'} =
              $transCommitShow->session_id();

            if ( $transStatus =~ m/commit[_-]complete/i ) {
                my $state = NACL::CS::StorageAggregate->fetch(
                    command_interface => $showOpts->{'Node'},
                    filter => { uuid => $objects->{$aggr}->{'aggrUUID'} },
                )->state();
                if ( $state !~ m/online/i ) {
                    NACL::Exceptions::UnexpectedState->throw(
                        "Unexpected state, Expected aggr status : online,
                        actual status : " . $state
                    );
                }
            }
            else {
                $failCount++;
            }
        }
        $showAttributes{'JOB'} = $showOpts->{'JobID'};

        if ($failCount) {
            $respObject = bless( \%showAttributes, 'NACL::APISet::Response' );
            NACL::APISet::Exceptions::CommandFailedException->throw(
                text =>
"One or more of the aggregates passed in the input is not showing commit\-complete. More info in response_object.",
                response_object => $respObject
            );
        }

    }
    elsif ( $showOpts->{'Type'} eq "REVERT" ) {
        my $failCount = 0;
        foreach my $aggr ( keys $objects ) {
            $transAggrShow = NACL::C::StorageTransitionRevert->show_status(
                command_interface => $showOpts->{'Node'},
                filter => { 'aggr-uuid' => $objects->{$aggr}->{'aggrUUID'} },
            );
            $transStatus = $transAggrShow->status_code();
            while (( $transStatus !~ /revert[-_]complete/i )
                && ( $count < 4 ) )
            {
                sleep 5;
                $transAggrShow = NACL::C::StorageTransitionRevert->show_status(
                    command_interface => $showOpts->{'Node'},
                    filter => { 'aggr-uuid' => $objects->{$aggr}->{'aggrUUID'} }
                );
                $transStatus = $transAggrShow->status_code();
                $count++;
            }
            $showAttributes{$aggr}{'status_code'} =
              $transAggrShow->status_code();
            $showAttributes{$aggr}{'error_code'} = $transAggrShow->error_code();
            $showAttributes{$aggr}{'error_string'} =
              $transAggrShow->error_string();
            $showAttributes{$aggr}{'system_id'} = $transAggrShow->system_id();
            $showAttributes{$aggr}{'corrective_action'} =
              $transAggrShow->corrective_action();
            $showAttributes{$aggr}{'snapshot_name'} =
              $transAggrShow->snapshot_name();
            $showAttributes{$aggr}{'is_root'} = $transAggrShow->is_root();
            $showAttributes{$aggr}{'kernel_version'} =
              $transAggrShow->kernel_version();
            $showAttributes{$aggr}{'ignore_errors'} =
              $transAggrShow->ignore_errors();
            $count = 0;

            if ( $transStatus =~ /revert[-_]complete/i ) {
                &_check_foriegn_offline_aggr($aggr);
                foreach my $disk ( @{ $aggrs_disk_list{$aggr} } ) {
                    NACL::C::StorageDisk->removeowner(
                        command_interface => $Node,
                        disk              => $disk,
                    );
                }
            }
            else {
                $failCount++;
            }
        }
        $showAttributes{'JOB'} = $showOpts->{'JobID'};

        if ($failCount) {
            $respObject = bless( \%showAttributes, "NACL::APISet::Response" );
            NACL::APISet::Exceptions::CommandFailedException->throw(
                text =>
"One or more of the aggregates passed in the input is not showing revert\-complete. More info in response_object.",
                response_object => $respObject
            );
        }

    }
    else {
        my $failCount = 0;
        foreach my $vol ( keys $objects ) {
            $transVolShow = NACL::C::StorageTransitionVolumes->show_status(
                command_interface => $showOpts->{'Node'},
                filter => { 'volume-uuid' => $objects->{$vol}->{"volUUID"} },
            );
            $transStatus = $transVolShow->status_code();
            while (( $transStatus !~ /transition[_-]complete/i )
                && ( $count < 4 ) )
            {
                sleep 5;
                $transVolShow = NACL::C::StorageTransitionVolumes->show_status(
                    command_interface => $showOpts->{'Node'},
                    filter =>
                      { 'volume-uuid' => $objects->{$vol}->{"volUUID"} },
                );
                $transStatus = $transVolShow->status_code();
                $count++;
            }
            $count                               = 0;
            $showAttributes{$vol}{'status_code'} = $transVolShow->status_code();
            $showAttributes{$vol}{'error_code'}  = $transVolShow->error_code();
            $showAttributes{$vol}{'error_string'} =
              $transVolShow->error_string();
            $showAttributes{$vol}{'volume'}    = $transVolShow->volume();
            $showAttributes{$vol}{'vserver'}   = $transVolShow->vserver();
            $showAttributes{$vol}{'node_name'} = $transVolShow->node_name();
            $showAttributes{$vol}{'volume_new_name'} =
              $transVolShow->volume_new_name();
            $showAttributes{$vol}{'corrective_action'} =
              $transVolShow->corrective_action();
            $showAttributes{$vol}{'dsid'} = $transVolShow->dsid();
            $showAttributes{$vol}{'msid'} = $transVolShow->msid();
            $showAttributes{$vol}{'ignore_errors'} =
              $transVolShow->ignore_errors();

            if ( $transStatus =~ /transition[-_]complete/i ) {
                my $volState = NACL::CS::Volume->fetch(
                    command_interface => $showOpts->{'Node'},
                    filter => { volume => $objects->{$vol}->{'volNewName'} },
                )->state();
                if ( $volState !~ m/online/i ) {
                    NACL::Exceptions::UnexpectedState->throw(
                        "Unexpected state, Expected volume status : online,
                        actual status : " . $volState
                    );
                }
            }
            else {
                $failCount++;
            }
        }
        $showAttributes{'JOB'} = $showOpts->{'JobID'};

        if ($failCount) {
            $respObject = bless( \%showAttributes, "NACL::APISet::Response" );
            NACL::APISet::Exceptions::CommandFailedException->throw(
                text =>
"One or more of the volumes passed in the input is not showing transition\-complete. More info in response_object.",
                response_object => $respObject
            );
        }

    }

    return ( \%showAttributes );
}

# This method fetches untransitioned aggregate uuid (aggrs in dblage) from
# "debug vreport show" output, and fills a hash with aggregate name followed by node name as key, and
# aggregate UUID as a value.

sub _fetch_aggr_uuid {
    my @DebugVreport_obj;
    my $iter = 0;
REDO:
    try {
        $Node->apiset()
          ->execute_command(
            command => "debug vreport delete-locks -node " . $Node->name() );
        @DebugVreport_obj = NACL::CS::DebugVreport->fetch(
            command_interface => $Node,
            requested_fields  => [ 'type', 'object', 'reason', 'attributes' ],
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
	my $exp_text = $exception_object->text();
	if ($exp_text =~ /Another instance of Vreport is running/i && $iter < 3) {
	    $iter++ ;
	    sleep(20);
	    goto REDO;
	} else {
            $exception_object->throw();
	}
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $exception_object->throw();
    };
    foreach my $vrObj (@DebugVreport_obj) {
        $vrObj->attributes()->[0] =~ m/Node Name: (.*)\"$/i;
        my $node = $1;
        if ( $vrObj->object() =~ m/\(.*?\)/i ) {
            $vrObj->object() =~ m/(.*)?\((.*)\)/i;
            $aggr2Id{ $1 . "_" . $node } = $2;
        }
        else {
            $vrObj->object() =~ m/(.*)?/i;
            my $aggrname = $1;
            $vrObj->attributes()->[1] =~ m/UUID: (.*)\"/i;
            $aggr2Id{ $aggrname . "_" . $node } = $1;
        }
    }
    return \%aggr2Id;
}

# This method fetches uuid of the aggr using fetch method call

sub _fetch_aggr_to_be_reverted_uuid {
    my $aggr = shift;
    my ( $aggrUuid, $aggrObj );

    try {
        $aggrObj = NACL::CS::StorageAggregate->fetch(
            command_interface => $Node,
            filter            => { aggregate => $aggr },
        );
        $aggrUuid = $aggrObj->uuid();
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        if ( exists $aggr2Id{ $aggr . "_" . $Node->name() } ) {
            $aggrUuid = $aggr2Id{ $aggr . "_" . $Node->name() };
        }
        else {
            $aggrObj = _fetch_aggr_uuid();
            if ( defined $aggrObj ) {
                $aggrUuid = $aggrObj->{ $aggr . "_" . $Node->name() };
            }
        }
    };

    return $aggrUuid;
}

# This method verifies whether a untransitioned (in dblade) aggregate status
# as foriegn and state as offline, else fails

sub _check_foriegn_offline_aggr {
    my $aggr = shift;
    my $aggrResp =
      $Node->get_7m_or_nodescope_apiset()->aggr_status( aggregate => $aggr );
    foreach my $aggrArr ( @{ $aggrResp->get_parsed_output() } ) {
        if (
            ( $aggrArr->{'state'} !~ m/restricted|offline/i )

            #( $aggrArr->{'status'} !~ m/foreign/i ) &&
          )
        {
            NACL::Exceptions::UnexpectedState->throw(
"Unexpected state, Expected aggr state : offline or restricted, actual state : "
                  . $aggrArr->{'state'} );
        }
        elsif ( $aggrArr->{'state'} =~ m/restricted/i ) {
            $Node->get_7m_or_nodescope_apiset()
              ->aggr_offline( aggregate => $aggr );
        }
    }
}

# This method collects attributes required for volume transition, such as
# aggrUUID, volUUID, containing aggregate etc.,
sub _prepare_vol_attrs {
    my $vol = shift;
    my $skip_checks = shift || 0;
    my $volResp =
      $Node->get_7m_or_nodescope_apiset()->vol_status( volume => $vol );

    foreach my $volArr ( @{ $volResp->get_parsed_output() } ) {
        if ( !$skip_checks ) {
            if ( $volArr->{'state'} !~ m/offline||restrict/i ) {
                NACL::Exceptions::UnexpectedState->throw(
"Invalid volume to transition, Expected vol status : offline, actual status : "
                      . $volArr->{'state'} );
            }
        }

        $volAttrs{$vol}{'volUUID'}   = $volArr->{'volume_uuid'};
        $volAttrs{$vol}{'aggregate'} = $volArr->{'containing_aggregate'};
        my $aggrObj = NACL::CS::StorageAggregate->fetch(
            command_interface => $Node,
            filter => { aggregate => $volArr->{'containing_aggregate'} },
        );
        $volAttrs{$vol}{'aggrUUID'} = $aggrObj->uuid();

    }
}

# This method generates a unique session id used for transition.

sub _gen_uuid() {
    my $uuid;
    $uuid = `uuidgen`;
    chomp($uuid);
    return $uuid;
}

# This method validates the parameters for object creation via constructor method (new)
sub _get_validation_spec {
    $Log->enter() if $may_enter;
    my %validation_spec = (
        command_interface =>
          { isa => 'NACL::C::CommandInterface', type => OBJECT },
           interface => { type => SCALAR, optional => 1, default => 'CLI' },
        'method-timeout' => { type => SCALAR, optional => 1 },
         'session_id' => { type => SCALAR, optional => 1, default => $SESSION_ID },
    );
    $Log->exit() if $may_exit;
    return \%validation_spec;
}

sub _purge_validate_spec {
    $Log->enter() if $may_enter;
    my %purge_validate_spec = (
        command_interface =>
          { isa => 'NACL::C::CommandInterface', type => OBJECT },
        'node_name'  => { type => SCALAR, optional => 1 },
        'session_id' => { type => SCALAR, optional => 1, default => $SESSION_ID},
    );

    $Log->exit() if $may_exit;
    return \%purge_validate_spec;
}

# This method validates parameters supplied for aggr_transition_start_verify method
sub _aggr_validate_spec {
    $Log->enter() if $may_enter;
    my %aggr_validate_spec = (
        command_interface =>
          { isa => 'NACL::C::CommandInterface', type => OBJECT },
        aggrTransAttr      => { type => HASHREF },
        task_wait_timeout  => { type => SCALAR, optional => 1, default => 60 },
        skip_status_verify => { type => BOOLEAN, optional => 1, default => 0 },
        node_name          => { type => SCALAR,  optional => 1},
        session_id       => { type => SCALAR, optional => 1, default => $SESSION_ID },

    );

    $Log->exit() if $may_exit;
    return \%aggr_validate_spec;
}

# this method validates parameters supplied for vol_transition_start_verify method
sub _vol_validate_spec {
    $Log->enter() if $may_enter;
    my %vol_validate_spec = (
        command_interface =>
          { isa => 'NACL::C::CommandInterface', type => OBJECT },
        volTransAttr       => { type => HASHREF },
        task_wait_timeout  => { type => SCALAR, optional => 1, default => 60 },
        skip_status_verify => { type => BOOLEAN, optional => 1, default => 0 },
        session_id         => { type => SCALAR, optional => 1, default => $SESSION_ID },
    );
    $Log->exit() if $may_exit;
    return \%vol_validate_spec;
}

# This method validates parameters supplied for transition_revert_start_verify method
sub _revert_validate_spec {
    $Log->enter() if $may_enter;
    my %revert_validate_spec = (
        command_interface =>
          { isa => 'NACL::C::CommandInterface', type => OBJECT },
        aggrTransAttr      => { type => HASHREF },
        kernalVersion      => { type => SCALAR, },
        task_wait_timeout  => { type => SCALAR, optional => 1, default => 60 },
        skip_status_verify => { type => BOOLEAN, optional => 1, default => 0 },
        node_name          => { type => SCALAR,  optional => 1 },
        session_id         => { type => SCALAR, optional => 1, default => $SESSION_ID },

    );

    $Log->exit() if $may_exit;
    return \%revert_validate_spec;
}

sub _class_call_init {
    my $args = shift;
    my ($zapiPrecommitObj, $res, $session_id);
    my $self = validate_with(
        params      => $args,
        spec        => _get_validation_spec(),
        allow_extra => 1
    );
    my $Node = $self->{command_interface};
    if ($self->{'interface'}  eq 'ZAPI' ) {
       $self->{zapiobj} = NACL::APISet->new(
          name      => $self->{command_interface}->node(),
          category  => "Node",
          interface => "ZAPI",
          set       => "CMode",
       );
    }

    instant_component(
        component_name    => 'NACL::C::StorageTransitionPreCommit',
        command_interface => $self->{command_interface}
    ); 
    
    $session_id =( defined $self->{session_id} )? delete $self->{session_id}: $SESSION_ID;

    if( $self->{'interface'} eq 'ZAPI' ) {
        $zapiPrecommitObj = '<storage-transition-pre-commit-begin><session-id>'.$session_id.'</session-id>';
        $zapiPrecommitObj .= '<nodes><node-name>'.$self->{command_interface}->node().'</node-name></nodes>';
        $zapiPrecommitObj .= '</storage-transition-pre-commit-begin>';
        $res = $self->{zapiobj}->execute_raw_command( command => $zapiPrecommitObj,
                                                'return-response-object' => 1 );
    }
    else {
       NACL::C::StorageTransitionPreCommit->begin(
          command_interface => $Node,
          'session-id'      => $session_id,
          "nodes"           => [ $Node->node() ],
       );
    }
    NACL::C::StorageTransitionPreCommit->show_status(
        command_interface => $Node,
        filter => {
        'session-id'      => $session_id,
        "node"            => $Node->node(),
	}
    );
    return  bless ($self, __PACKAGE__ );
}

1;

