# $Id$
# Copyright (c) 2011-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Load Task Module
## @author Isshwarya.M@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::MTask::Load

=head1 SYNOPSIS

  use NACL::MTask::Load;

  # Create the load object
  my $load_obj = NACL::MTask::Load->new(
                              command_interface => $ci,
                              clients_info      => \@clients_info_AoH,
                    );
  # Start the load
  $load_obj->load_cpu(min => $min, max => $max, %other_optional_args);

  # Run the testcases which needs to be tested under I/O workload
  ...

  # Stop the load
  $load_obj->stop();


=head1 DESCRIPTION

To ramp up the CPU load of filer running Data ONTAP to the specified range
and maintain the load level in the specified range by tuning at regular
intervals. If you don't desire background tuning, then you may be
interested at
L<NACL::MTask::Load::BlockingLoad|lib-NACL-MTask-Load-BlockingLoad-pm>.

=head1 CONFIGURATION REQUIREMENTS

The caller is expected to establish mount points (with whatever read/write
sizes desired in the case of NAS mount). Handling of the setup steps by
the task requires more information about the test clients
and the cluster configuration and this is not covered in the initial version
of this task.

More details on this section can be referred
L<here|http://wikid.netapp.com/w/User:Isshwar/CPULoadTuning#Configuration_requirements>

=head1 NOTE

Please note that the tuning logic is in experimental stage and based on the user
feedback, the logic will be enhanced and fine tuned further.

=head1 ATTRIBUTES

=head2 command_interface

(Required, isa 'NACL::C::CommandInterface::ONTAP')
A command interface object which represents the cluster or node to be loaded.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

=head2 clients_info

(Required)
This array input takes the information about the clients and the absolute
file paths or paths to raw devices on which I/O can be started to load the
CPU of the filer. You can either provide an array of hash or an array of
L<NACL::MTask::Load::ClientInfo|lib-NACL-MTask-Load-ClientInfo-pm> objects.
Each array element corresponds to a traffic client that can be used to
for tuning purposes.

If you are providing array of hash as the input, then the hash keys can
be any of the attributes of
L<NACL::C::Load::ClientInfo|lib-NACL-C-Load-ClientInfo-pm>
'client_ci' holds the command_interface to the client and it is an object
of type L<NACL::C::Client|lib-NACL-C-Client-pm>. 'paths' is an array
of simple scalars holding the list of absolute file paths or paths to raw
devices. For other interesting options, that can be passed, please
refer to L<NACL::C::Load::ClientInfo|lib-NACL-C-Load-ClientInfo-pm>.
If you provide array of hashes as the input, the task would
turn them into objects of type NACL::MTask::Load::ClientInfo before storing
them.

Eg:

 my @clients = NACL::C::Client->find();
 my $load_obj = NACL::MTask::Load->new(
        command_interface => $ci,
        clients_info => [
            {client_ci =>$clients[0], paths => ['/mount/dir1/file1', '/mount/dir1/file2']},
            {client_ci =>$clients[1], paths => ['/dev/sda1', '/dev/sda2']},
            {client_ci =>$clients[2], paths => ['Y:\dir1', 'J:\dir2']},
        ]
 );

If you are providing array of NACL::MTask::Load::ClientInfo objects,
then each object should have 'client_ci' and 'paths' attribute filled in.

Eg:

 my $client_info_1 = NACL::MTask::Load::ClientInfo->new(
        client_ci => $linux_ci, paths => ['/mount/dir1/file1', '/mount/dir1/file2']);
 my $client_info_2 = NACL::MTask::Load::ClientInfo->new(
        client_ci => $windows_ci, paths => ['Y:\dir1', 'J:\dir2']);
 my $load_obj = NACL::MTask::Load->new(
        command_interface => $ci,
        clients_info => [$client_info_1, $client_info_2]
 );

Additional note is that,

$load_obj->client_info() will return an array of objects of type
NACL::MTask::Load::ClientInfo when invoked in array context and it would
return reference to the array when invoked in the scalar context.

=cut

package NACL::MTask::Load;

use strict;
use warnings;

# Base class
use base qw(NACL::MTask::MTask);

# Load Perl and NATE modules
use Tharn qw( sleep logcomment);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Events qw(timer_add timer_remove);
use Params::Validate qw(:all);
use NACL::ComponentUtils qw(Dumper);
use NATE::PerlRPC::Worker::Process;
use Scalar::Util qw(blessed);

# Load Exceptions
use NATE::BaseException qw(:try);
use NACL::Exceptions::LoadCPUFailed;
use NACL::Exceptions::LoadCPUFailedInMaintenancePhase;
use NATE::Exceptions::Argument qw(:try);

# Load other needed NACL libraries
use NACL::MTask::Load::BlockingLoad;
use NACL::MTask::Load::_Mixin qw(clients_info_write_cb get_cpu_load);


=head1 METHODS

=head2 new

  my $load_obj = NACL::MTask::Load->new(
                              command_interface => $ci,
                              clients_info           => \@clients_info_AoH,
                    );
Returns a new NACL::MTask::Load object. Refer to
L<ATTRIBUTES|lib-NACL-MTask-Load-pm/ATTRIBUTES> section
for the description of the input options.

=cut

use Class::MethodMaker [
    new    => [ '-hash', 'new' ],
    array => [
        {
            '-store_cb' => sub {
                my $self = shift;
                return $self->clients_info_write_cb(@_);
            },

        },
        'clients_info'
    ],
    scalar => [
        { '-type' => 'NACL::C::CommandInterface::ONTAP' },
        'command_interface'
    ],
    scalar => [
        { '-type' => 'NATE::PerlRPC::Worker::Process', },
        '_tune_cpu_worker'
    ],
];

=head2 load_cpu

  $load_obj->load_cpu (
        min => $min, max => $max,
        %other_optional_options
  );

(Instance method)

This method loads the CPU to the specified range by tuning the I/O load.
There are two phases in tuning - Initial phase (first time tuning)
and maintenance tuning.

This method performs initial tuning in the foreground and so the call
is blocking till the first time tuning is completed. After this,
this method returns back the control to the caller and it will also
maintain the load in the specified range by regularly querying the
node/cluster for the current CPU load and tune it accordingly, if it
goes out of the range. Thus maintenance tuning happens automatically
in the background and it is non-blocking.

Note that, the CPU load may go out of range for short time intervals,
when the task method is involved in maintenance tuning.

To explain briefly about the tuning logic, the task starts the specified I/O
tool on all the clients and then it measures the CPU load of the cluster
or node, after the specified wait time. If the load is not in the
expected range, it kills the instances of the I/O tool and starts
a new instance of the tool with the tuned inputs. It repeats these steps,
untill the load reaches the desired range or the max_attempts is hit.

For interested readers, the tuning logic is described in detail at
L<CPU Load Tuning section|http://wikid.netapp.com/w/User:Isshwar/CPULoadTuning>

=head3 Options

=over

=item *

All the options accepted by
L<NACL::MTask::Load::BlockingLoad|lib-NACL-MTask-Load-BlockingLoad-pm/start> are
accepted by this method as well.

=item wait_time_in_maintain_phase

(Optional)

Wait time in seconds between the tuning cycles of the maintenance tuning phase.

If a tuning cycle itself takes longer than the time specified in
C<wait_time_in_maintain_phase> option, then a new tuning cycle will not be started
without completing the current one. For eg, if the tuning cycle runs for 25 sec,
then the next tuning cycle will be started only after the time specified to this
option.

Thus this option doesn't control the frequency of tuning, rather it controls
the wait time between any two tuning cycles.

Default: 20

=item timeout_for_initial_tuning_phase

(Optional)

The maximum allowable timeout in seconds for the task to complete the initial
tuning (which is a blocking call). If the initial tuning is not completed in
the specified time, C<NATE::PerlRPC::Exception::RPCTimeout> exception will be
thrown.

Default: -1 (Unlimited time)

=item on_failure

(Optional)

This parameter takes a code reference as the input specifying the callback to
be invoked, when the tuning which happens in the background during monitoring
phase fails at any instance.

By default, the task throws
C<NACL::Exceptions::LoadCPUFailedInMaintenancePhase> exception and this
supports an attribute called 'actual_exception' and refer to the POD for
more details on this attribute.

If you override with a code reference, then an object of type
L<NATE::PerlRPC::Future> would be sent as the first input argument to the
coderef specified.

To obtain the original exception object raised in the tuning worker process,
you can either use on_failure handler to obtain the L<NATE::PerlRPC::Future>
object (from which the original exception object is accessible as,
$future->exception())
or you can catch C<NACL::Exceptions::LoadCPUFailedInMaintenancePhase> exception
and obtain the exception object as,
$load_cpu_failed_in_maint_phase_exception->actual_exception(). In general, it's
suggested to use the former option.

Eg:
    on_failure => sub {
        my $future = shift;
        my $exception_object_or_string = $future->exception();
        my $worker_process = $future->worker();
        ...
        if (blessed($exception)) {
            if ($exception->isa('NATE::PerlRPC::Exception::WorkerStopped')) {
                # Worker is killed
                ...
            } elsif (...) {
                ...
            } ...
        }
    }

    $load_obj->load_cpu(
        min => 40, max => 50,
        # Specify on_failure handler
        on_failure => $on_failure,
        %other_optional_args
    );

=back

=cut

=head3 Exceptions

=over

=item NACL::Exceptions::LoadCPUFailed

This exception will be thrown for any failures reported
during initial tuning phase (blocking) from NACL::MTask::Load::BlockingLoad.

=item NACL::Exceptions::LoadCPUFailedInMaintenancePhase

If on_failure handler is not overridden, then
C<NACL::Exceptions::LoadCPUFailedInMaintenancePhase> exception will be
thrown for any failures reported during maintenance phase (non-blocking).
Note that this exception inherits NACL::Exceptions::LoadCPUFailed exception.

=item NATE::PerlRPC::Exception::WorkerStopped

If the tuning CPU worker is stopped by external factors (which doesn't include
NACL::MTask::Load->stop) such as, the worker process is killed, etc.,
If you override on_failure handler, then you will get this exception using,
$future->exception and you can determine the action of the failure handler based
on the type of exception.

=item NATE::PerlRPC::Exception::RPCTimeout

If the option 'timeout_for_initial_tuning_phase' is specified and if the initial
tuning is not completed within this timeout, then this exception will be thrown

=item Other exceptions thrown from NACL::MTask::Load::BlockingLoad

Any other exception thrown by L<start()|lib-NACL-MTask-Load-BlockingLoad-pm/start>
method of NACL::MTask::Load::BlockingLoad will also be propagated. In maintenance
phase, this can be controlled using on_failure handler.

=back

=cut

sub load_cpu {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->_verify_invocation( style => 'instance_only' );
    $Log->debug("Input options are:\n" . Dumper(\@args)) if ( $Log->may_debug() );
    my %opts = validate_with(
        params => \@args,
        spec   => {
            on_failure => {
                type => CODEREF,
                optional => 1
            },
            wait_time_in_maintain_phase => {type => SCALAR, default => 20},
            timeout_for_initial_tuning_phase => {type => SCALAR, default => -1}
        },
        # other options will be validated in NACL::MTask::Load::BlockingLoad->load_cpu
        allow_extra => 1
    );
    # set it to 1 for further maintenance tuning in the background
    $opts{cache_options} = 1;
    # wait_time_in_maintain_phase is an option meant to be consumed by this task method
    my $wait_time_in_maintain_phase = delete $opts{wait_time_in_maintain_phase};
    my $timeout_for_initial_tuning_phase = delete $opts{timeout_for_initial_tuning_phase};
    # Construct on_failure handlers
    my $custom_on_failure = delete $opts{on_failure};
    my $on_done_submit = sub {
        $Log->trace("Inside on_done_submit handler");
        my $future = shift;

        # If $load_obj->stop was invoked already by the user, then on_done handler will
        # see the _tune_cpu_worker attribute is unset. $load_obj->stop throws
        # exception, if worker object is unset. To disallow the exception,
        # if_stopped => 'pass' is specified.
        $self->stop(if_stopped => 'pass');

        # WorkerStopped exception will also seen by the already
        # registered on_done handler, and this shouldn't be treated
        # as a failure, if the worker was stopped by task's stop() method
        my $exception = $future->exception();
        if (blessed($exception) && $exception->isa('NATE::PerlRPC::Exception::WorkerStopped')
            && ($exception->text =~ /CPU tune worker is stopped/)) {
            $Log->trace('Task has stopped the worker. Simply return');
            return;
        }
        if (defined $custom_on_failure) {
            # user defined callback
            $Log->trace("load_cpu: Invoking user defined on_failure handler");
            $custom_on_failure->($future);
        } else {
            # default callback
            $Log->trace("load_cpu: Invoking default on_failure handler");
            my $msg;
            if (blessed($exception) && $exception->isa('NATE::BaseException')) {
                # Obtained an exception object
                $msg = "Failure occured while loading CPU and the reason is:\n".
                    $exception->text;
                $Log->exit() if $may_exit;
                NACL::Exceptions::LoadCPUFailed->throw($msg, actual_exception => $exception);
            } elsif (ref $exception) {
                # Some other error object which is not of type exception is received
                $Log->exit() if $may_exit;
                NACL::Exceptions::LoadCPUFailed->throw(
                    "Failure of type ". ref($exception) . "has occured while ".
                    "loading CPU. Reason is,:\n". Dumper($exception));
            } elsif ($exception) {
                # Future has returned an exception string
                $Log->exit() if $may_exit;
                NACL::Exceptions::LoadCPUFailed->throw(
                    "Failure occured while loading CPU and the reason is:\n". $exception);
            } else {
                $msg = "CPU loading has stopped and no error is reported as ".
                    "well, which is a highly unexpected behavior. Please ".
                    "report this to dl-nacl-dev with the complete logs";
                $Log->exit() if $may_exit;
                NACL::Exceptions::LoadCPUFailed->throw($msg);
            }
        }
    };
    my $on_failure_call = sub {
        my $future = shift;
        my $exception = $future->exception();
        if ($exception) {
            # Stop the tuning worker here and don't rely on the automatic
            # destruction of worker which happens when the object goes out
            # of scope. This is because, the worker process will be alive
            # unnecessarily as long as the Load MTask object (which holds a
            # reference to the worker object) goes out of scope.
            $self->stop();
            $Log->exit() if $may_exit;
            if (ref $exception) {
                $exception->set_text("Initial tuning has failed. Reason is,:\n". $exception->text);
                # Rethrow the same exception to preserve the stacktrace
                $exception->throw;
            } else {
                # Some error is received, but its in string form!
                NACL::Exceptions::LoadCPUFailed->throw("Initial tuning has failed. Reason is,:\n". $exception);
            }
        }
    };
    # Create the worker which does the tuning in the background
    $Log->trace("Starting worker which handles cpu load tuning");

    # NATE's library starts the worker process, by default.
    # We are starting the tuning worker in the local test controller itself
    # One additional process running in the background is not costly
    # what matters is that, where the traffic processes are started and
    # this can be controlled by passing execute_remotely => 0|1 option
    # of the traffic task.
    my $worker = NATE::PerlRPC::Worker::Process->new();
    $self->_tune_cpu_worker($worker);

    # create NACL::MTask::Load::BlockingLoad object in the worker process
    # on failure, make it to throw NATE::Exceptions::Argument
    my $code = <<'EOT';
    use NACL::MTask::Load::BlockingLoad;
    return NACL::MTask::Load::BlockingLoad->new(@_);
EOT
    my %opts_for_remote_load = (
        command_interface => $self->command_interface,
        clients_info      => [$self->clients_info],
    );
    # Creating a new BlockingLoad object in the tuning worker shouldn't take more
    # than 60 sec.
    my $load_wrapper = $worker->call(eval => $code , args => [%opts_for_remote_load], wrap_result => 1, rpc_timeout => 60);

    $Log->comment("Info: Initial tuning is going to be started");

    # The first time tuning is currently blocking and further tuning
    # and maintenance happens in the background at regular intervals
    # as triggered by the timer.
    $load_wrapper->load_cpu_once( %opts,
        _on_done => $on_failure_call,
        _rpc_timeout => $timeout_for_initial_tuning_phase);
    $Log->comment("Info: Initial tuning is completed");

    # Start the maintenance tuning in the background now
    my %args_to_maintain = (blocking_load_obj => $load_wrapper,
        wait_time_in_maintain_phase => $wait_time_in_maintain_phase);
    $worker->submit(package => 'NACL::MTask::Load', method => '_maintain_cpu_load_using_timer',
        args => [%args_to_maintain], on_done => $on_done_submit);
    $Log->comment("Info: Tuning worker will continue to monitor the ".
        "CPU load in the background");
    $Log->exit() if $may_exit;
}

=head2 stop

    $load_obj->stop();

(Instance method)

This method stops the I/O load started earlier to load the CPU.

=over

=item Options

=over

=item if_stopped

(Optional, defaults to "die")

If 'die' (default), NATE::BaseException will be thrown, if no load
is currently running.

If 'pass', exits quietly when it detects the above situation.

=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my ( $self, %opts ) = @_;
    $self->_verify_invocation( style => 'instance_only' );
    %opts = validate_with(
        params => \%opts,
        spec   => {
            'if_stopped' => $self->_if_action_completed_already_validate_spec(
                name => 'if_stopped'
            ),
        }
    );
    unless ( $self->_tune_cpu_worker_isset ) {
        if ( $opts{if_stopped} eq 'die' ) {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "You have to invoke load_cpu() before invoking stop()");
        }
    } else {
        $self->_tune_cpu_worker()->stop(
            message => 'CPU tune worker is stopped, as '.
                '$load_mtask_obj->stop() was invoked'
        );
        $self->_tune_cpu_worker_reset();
    }
    $Log->exit() if $may_exit;
}

=head2 get_cpu_load

The description can be referred at,
L<get_cpu_load|lib-NACL-MTask-Load-_Mixin-pm/get_cpu_load>

=cut


####################################
#       Private Methods            #
####################################

# This must be invoked from the worker process
sub _maintain_cpu_load_using_timer {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my %opts       = validate_with(
        params      => \@args,
        spec        => {
            blocking_load_obj => { type => OBJECT, isa => 'NATE::PerlRPC::Wrapper' },
            wait_time_in_maintain_phase => {type => SCALAR}
        }
    );
    my $blocking_load_obj = $opts{blocking_load_obj}->{_value};
    my $interval = $opts{wait_time_in_maintain_phase};

    while (1) {
        $Log->comment("Info: CPU load check will be initiated after $interval sec");
        Tharn::sleep($interval);
        $Log->comment("Info: CPU load check is initiated");
        $blocking_load_obj->load_cpu_once();
    }

    $Log->exit() if $may_exit;
}

# TODOs for future:
# 1. Handling clients which are down and still try to perform the best tuning
#    possible with the remaining clients.
# 2. When burt639124 (parser issue with sysstat -a) is fixed, add support
#    for selecting the default file count based on the platform. Need to
#    check its feasibility.
# 3. Add support for tuning using dt
#    Refer: http://web.netapp.com/~rtmiller/IO-Tools/dt-DynamicIO-Throttling.htm
#    BURT 633808
# 4. Test the support for loading the entire cluster. It's implemented in the code
#    but not yet tested.

1;
