# $Id: //depot/prod/test/nacldev/lib/NACL/MTask/Load/ClientInfo.pm#1 $

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary This class is a container to store client related
## information and it is used by NACL::MTask::Load and
## NACL::MTask::Load::BlockingLoad.

## @author Isshwarya.M@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::MTask::Load::ClientInfo;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NATE::Exceptions::Argument qw(:try);
use Params::Validate qw(:all);

# Turn Params::Validate failures into NATE::Exceptions::Argument
BEGIN {
    Params::Validate::validation_options(
        on_fail => sub {
            NATE::Exceptions::Argument->throw(shift);
        }
    );
}

=head1 ATTRIBUTES

=head2 client_ci

(Required, isa 'NACL::C::Client')

A command interface object which represents the client.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>
and L<NACL::C::Client|lib-NACL-C-Client-pm>

=head2 paths

(Optional)

It is an array attribute holding the list of absolute file paths or paths to raw
devices present in the client. 'paths' and 'workdir' options are mutually
exclusive and specifying one of them is mandatory.

=head2 workdir

(Optional)

Absolute directory path which has to be used as work directory for generating
traffic. 'paths' and 'workdir' options are mutually exclusive and specifying
one of them is mandatory.

You can either specify a scalar value (absolute path for a directory) or
you can specify L<NACL::STask::Client::Mount|lib-NACL-STask-Client-Mount-pm>
or L<NACL::C::Client::Mount|lib-NACL-C-Client-Mount-pm> object.

=head2 file_count

(Optional)

Certain I/O tools (say, sio) operate only on files. This parameter specifies
the no.of files to be created. If you have already files created in a specific
way, then you can pass the list of the files using 'paths' option. Otherwise,
you can specifiy the 'workdir' and 'file_count' options. The task will inturn
create these files before starting the load on CPU.

Default: 2

Example:

    paths => ['/mount/dir1/file1', '/mount/dir1/file2']

    or

    workdir => '/mount/dir1', file_count => 2

Another example:

    workdir => $stask_mount_object, file_count => 2

=head2 traffic_obj

(Read only attribute)

If traffic (I/O tool) is running in the client, then this attribute will hold
an object of type L<NACL::MTask::Traffic|lib-NACL-MTask-Traffic-pm>.

=cut

=head1 METHODS

=head2 new

  my $linux_ci =  NACL::C::Client->find();
  my $client_info_obj = NACL::MTask::Load::ClientInfo->new(
                             client_ci => $linux_ci,
                             paths => ['/mount/dir1/file1', '/mount/dir1/file2']
                    );
Returns a new NACL::MTask::Load::ClientInfo object. Refer to
L<ATTRIBUTES|lib-NACL-MTask-Load-ClientInfo-pm/ATTRIBUTES> section
for the description of the input options.

=cut


use Class::MethodMaker [
    new    => [ '-hash',  '-init', 'new' ],
    array => [
        {
            '-store_cb' => sub {
                my $self = shift;
                return $self->_paths_write_cb(@_);
            },

        },
        'paths'
    ],
    scalar => [
        { '-type' => 'NACL::C::Client' },
        'client_ci'
    ],
    scalar => [
        { '-type' => 'NACL::MTask::Traffic' },
        'traffic_obj'
    ],
    scalar => 'workdir',
    scalar => [
        {   '-default' => 2
        },
        'file_count'
    ],
];

sub init {
    $Log->enter();
    my ( $self, @args ) = @_;
    validate_with(
        params => \@args,
        spec   => {
            paths => {type => ARRAYREF, optional => 1},
            client_ci => {type => OBJECT, isa => 'NACL::C::Client'},
            workdir => {type => SCALAR|OBJECT, optional => 1},
            file_count => {type => SCALAR, optional => 1},
        }
    );
    # Check if 'workdir' has a valid value.
    if ($self->workdir_isset) {
        my $workdir = $self->workdir;
        unless ( !ref($workdir) || $workdir->isa('NACL::C::Client::Mount')) {
            $Log->exit;
            NATE::Exceptions::Argument->throw("You can either specify a ".
                "scalar value or NACL::(C|STask)::Client::Mount object ".
                "for 'workdir' option. You have specified a value of type ".
                ref($workdir). " for the client '". $self->get_id . "'"
            );
        }
    }
    # Check if workdir [-file_count] is specified when 'paths' is not specified
    unless ($self->paths_isset) {
        unless ($self->workdir_isset) {
            $Log->exit;
            NATE::Exceptions::Argument->throw("Either of 'paths' or 'workdir' ".
                'has to be specified, but both of them are missing'.
                " for the client '". $self->get_id . "'");
        }
    }
    if ($self->traffic_obj_isset) {
        $Log->exit;
        NATE::Exceptions::Argument->throw('traffic_obj is a read only attribute'
            . " for the client '". $self->get_id . "'");
    }
    $Log->exit();
}

=head2 get_id

    my $id = $lclient_info_obj->get_id()

(Instance method)

This method returns the id or name of the client.

=cut

sub get_id {
    $Log->enter();
    my $name = $_[0]->client_ci->hostrec->id;
    $Log->exit();
    return $name;
}

sub _paths_write_cb {
    $Log->enter();
    my ($self, $value) = @_;

    my @paths = @$value;
    if (ref($paths[0]) eq 'ARRAY' && scalar(@paths) == 1) {
        @paths = @{$paths[0]};
    }
    foreach my $path (@paths) {
        $Log->exit();
        NATE::Exceptions::Argument->throw('paths is an array attribute '.
            'holding simple scalar values, but you have specified,'. "\n".
            Dumper($path)) if(ref $path);
    }
    $Log->exit();
    return \@paths;
}

1;
