#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::MTask::LunIterators::LunIterator;

use Moose;
use Moose::Util::TypeConstraints;
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::MTask::LunIterators::UnitTestLun;
use NACL::MTask::LunIterators::ZapiLun;
use NACL::Exceptions::NoElementsFound;

# A volume is required so we can get the command interface and vserver for
# building NACL::STask::Lun.
has 'volume' => (
    is => 'ro',
    isa => 'Object',
);

# The initial path to start iteration at. It defaults to "", but can be set to
# any qtree
has 'path' => (
    is => 'ro',
    isa => 'Str',
    default => '',
);

#
# Unit test of zapi, can extend to other ways of getting luns if required
#
enum 'Method_Types', [qw(unit_test zapi)];

has 'method' => (
   is => 'rw',
   isa => 'Method_Types',
   default => 'zapi',
);

#
# Subtypes for criteria key validation
#
subtype 'criteria_key' => (
    as 'ArrayRef[Str]'
);

#
#    Recognizes keys:
#    'exclude_luns' => ArrayRef - luns to avoid in the iteration
#

has criteria => (
    is => 'rw',
    isa => 'HashRef[criteria_key]',
    required => 0,
    default => sub {
        return { }
    },
);

#
# This will store a stack of directory contents. The bottom stack frame will be
# the contents of the directory that we started from. The top stack frame will
# be the directory currently serving luns. Luns will be removed from the top
# stack frame as they are consumed. The type of the objects in the directory
# listing will vary depending on which method is being used to iterate, but each
# object is expected to support a standard interface.
#
#
has '_dirstack' => (
    is => 'rw',
    isa => 'ArrayRef',
    default => sub { return []; },
);

# To support has_next, we need to have the next lun ready to go. This means
# that the next method will just return this lun and ensure it is refilled.
has '_next_lun' => (
    is => 'rw',
    isa => 'Object',
    predicate => '_has_next_lun',
    clearer => '_clear_next_lun',
);

sub BUILD {
    $Log->enter() if $may_enter;
    my $self = $_[0];

    my $method = $self->method();
    my $path = $self->path();

    my $lun;
    if ($method eq 'unit_test') {
        $lun = NACL::MTask::LunIterators::UnitTestLun->new(
            path => $path,
            volume => $self->volume(),
        );
    }
    if ($method eq 'zapi') {
        $lun = NACL::MTask::LunIterators::ZapiLun->new(
            path => $path,
            volume => $self->volume(),
        );
    }

    my @directory_frame = $lun->enumerate_directory();
    @directory_frame = $self->_filter_luns(@directory_frame);
    $self->_push_dirstack(\@directory_frame);
    $self->_refill_next_lun();
    $Log->exit() if $may_exit;
}

sub _filter_luns {
    $Log->enter() if $may_enter;
    my $self = shift;
    my @luns = @_;
    my @acceptable_luns = ();
    my @exclude_luns;
    if (exists $self->criteria()->{exclude_luns}) {
        @exclude_luns = @{$self->criteria()->{exclude_luns}};
    }

    foreach my $lun (@luns) {
        my $excluded = 0;
        foreach my $exclude_lun (@exclude_luns) {
            my $lun_name = basename($lun->raw_name());
            if ( $lun_name =~ /^$exclude_lun$/ ) {
                $excluded = 1;
            }
        }
        if (!$excluded) {
            push @acceptable_luns, $lun;
        }
    }
    $Log->exit() if $may_exit;
    return @acceptable_luns;
}

sub _done_iterating {
    my $self = $_[0];
    if (!scalar(@{$self->_dirstack()})) {
        return 1;
    }
    return 0;
}

# Use shift/unshift so the element to access is [0]
sub _pop_dirstack {
    my $self = $_[0];
    return shift @{$self->_dirstack()};
}

sub _push_dirstack {
    my $self = $_[0];
    unshift @{$self->_dirstack()}, $_[1];
}

# If there are remaining luns in the directory it will remove the oth one and
# return that. If there are no luns, it will return undef.
sub _next_lun_in_directory {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    my $current_directory = $self->_dirstack()->[0];
    if (!$current_directory) {
        $Log->exit() if $may_exit;
        return;
    }
    my $lun = shift @{$current_directory};
    $Log->exit() if $may_exit;
    return $lun;
}

# Get the next lun in the lun tree.
sub _pop_lun {
    my $self = $_[0];

    # Only luns are stored no dirs
    my $lun = $self->_next_lun_in_directory();
    if (!$lun) {
        # No more directories are left
        if ($self->_done_iterating()) {
            return undef;
        }
        # Go up to the prior directory
        $self->_pop_dirstack();
        return $self->_pop_lun();
    }

    if ($lun->is_directory()) {
        my @directory_frame = ();
        my $pop_flag = 0;
        try {
            @directory_frame = $lun->enumerate_directory();
        } catch NATE::BaseException with {
            my $e = shift;
            if ($e->text() =~ /changed since last listed/i ) {
                $pop_flag = 1;
            } else {
                $e->throw();
            }
        };

        if ($pop_flag) {
            return $self->_pop_lun();
        }

        @directory_frame = $self->_filter_luns(@directory_frame);
        $self->_push_dirstack(\@directory_frame);
        return $self->_pop_lun();
    }
    return $lun;
}

sub _refill_next_lun {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    my $lun = $self->_pop_lun();
    if ($lun) {
        $self->_next_lun($lun);
    }
    $Log->exit() if $may_exit;
}

sub has_next {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    if ($self->_has_next_lun()) {
        $Log->exit() if $may_exit;
        return 1;
    }
    $Log->exit() if $may_exit;
    return 0;
}

sub next {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    if (!$self->_has_next_lun()) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::NoElementsFound->throw(
            "Called next and no elements were remaining"
        );
    }
    my $return_lun = $self->_next_lun();
    $self->_clear_next_lun();
    $self->_refill_next_lun();

    $Log->exit() if $may_exit;
    return $return_lun->make_lun();
}

sub rewind {
    my $self = $_[0];
    $Log->enter() if $may_enter;
    $self->BUILD();
    $Log->exit() if $may_exit;
}

1;

=head1 NAME

NACL::MTask::LunIterators::LunIterator - Iterate over the luns in a
volume. Returns NACL::STask::lun object for the lun.

=head1 SYNOPSIS

    use NACL::MTask::LunIterators::LunIterator;

    my $Iterator = NACL::MTask::LunIterators::LunIterator->new(
        volume => $Volume,
        path => $Path,
    );

    while ($Iterator->has_next()) {
        my $elem = $Iterator->next();
        $Log->comment($elem->path()." ".$elem->path());
    }


=head1 DESCRIPTION

This library will iterate over luns in the volume and return hash
references representing each lun.

=head1 FUNCTIONS

=head2 has_next

Returns 1 if there is another element to be fetched by next
Arguments: none

=head3 Returns

1 if another element is available for next. 0 if none
is available

=head2 next

Returns the next element if available.

Arguments: none

=head3 Returns

This returns a NACL::STask::Lun instance representing a Lun.

=head2 rewind

Returns the iterator to the start of the collection
Arguments: none

=head3 Returns

none

=head2 new

Build a new iterator object instance

=over

=head3 Arguments

=over

=item C<< volume >> - NACL::C::Volume instance to iterate over

=item C<< path >> - (optional) A path to start iterating at. It defaults to '/',
but could be something like /.snapshot/snapname/dirname.

=item C<< criteria >> - (optional) A hashref specifying filter criteria.

    {
        exclude_luns => [ 'pattern1', 'pattern2' ],
    }

=back

=back

=head3 Returns

object instance for the iterator


=head1 SEE ALSO

=cut
