#
#     right (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary MTask Lun Move Iterator Class
## @author asada@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here
##############################################################################

package NACL::MTask::LunIterators::LunMoveIterator;
use Moose;

use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::STask::Volume;
use NACL::STask::Lun;
use NACL::C::LunMove;
use NATE::BaseException;
use NATE::Exceptions::Argument;
use NACL::APISet::Exceptions::ResponseException;
use NACL::GeneralUtils qw(random_name_generator nacl_method_retry);

# iterator functions and attributes are provided by the base class
extends qw (NACL::MTask::LunIterators::OperationalLunIterator);

# Constants
use constant WAIT_TIME => '10';
use constant RETRIES => '5';
use constant METHOD_TIMEOUT => '180';
use constant POLLING_INTERVAL => '20';

#
# destination volume is a required field for lun move iterator
#
has destination_volume => (
    is => 'rw',
    isa => 'NACL::C::Volume',
    required => 1,
);

# Overide parent function to do Lun move operation on a Lun
override '_do_lun_operation' => sub {
    my ($self, @luns) = @_;
    $Log->enter() if $may_enter;
    my $volume = $self->destination_volume();

    # create array of Lun objects for the move luns
    my @dest_lun_objs = ();
    my $dest_lun_obj = undef;
    my @dest_lun_paths = ();
    my @source_lun_paths = ();
    foreach my $lun (@luns) {
        # extract the lun name out of lun path
        my @split_path = split ('/', $lun->{path} );
        my $lun_name = $split_path[scalar @split_path - 1];
        if (length $lun_name > 10) {
            $lun_name = substr $lun_name, 0, 10;
        }
        # Create a random name for move lun
        # Using 40 characters to allow space for a random name
        my $new_lun_name = random_name_generator(
            prefix => "move_".$lun_name,
            length => 40,
        );

        # /vol/volname/lunname for destination
        my $move_destination_path = "/vol/".$volume->volume()."/$new_lun_name";
        $Log->comment("Going to move " . $lun->{path} . " to " . $move_destination_path);
        push @dest_lun_paths, $move_destination_path;
        push @source_lun_paths, $lun->{path};

        $Log->debug ("LUN: ". $lun->{path});

        # Create objects for the destination luns
        $dest_lun_obj = NACL::STask::Lun->new(
            command_interface => $lun->command_interface(),
            vserver => $volume->vserver(),
            path => $move_destination_path,
        );

        push @dest_lun_objs, $dest_lun_obj;
    }

    # Note: all luns belong to the same vserver and are move to the same
    # destination vserver volume
    my $command_interface = $luns[0]->command_interface();
    my $source_vserver = $luns[0]->vserver();

    # Relocation is not possible over different vservers
    if ($source_vserver ne $volume->vserver()) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw ("Lun relocation is not allowed ".
        "over different vservers Source Vserver: $source_vserver Destination Vserver: ". $volume->vserver());
    }

    # Call lun move
    my $lun_move_obj = undef;
    my $num;

    NACL::GeneralUtils::nacl_method_retry(
        code => sub {
            $lun_move_obj = NACL::C::LunMove->start(
                command_interface => $command_interface,
                'source-paths' => \@source_lun_paths,
                'vserver' => $volume->vserver(),
                'destination-paths' => \@dest_lun_paths,
            );
        },
        tries_count => RETRIES,
        sleep_time  => WAIT_TIME,
        exceptions   => 'NACL::C::Exceptions::LunMove::MaximumSimultaneousOperation',
        num_times_code_ran => \$num,
    );

    $Log->exit() if $may_exit;
    if ($self->group_operations_flag){
        return @dest_lun_objs;
    } else {
        return $dest_lun_objs[0];
    }
};

# Overide parent function to to overide for relocation (rewind to move directory on destination)
override 'rewind' => sub {
    my $self = $_[0];
    $Log->enter() if $may_enter;
    require NACL::MTask::LunIterators::LunIterator;
    require NATE::BaseException;

    my $volume = $self->destination_volume();

    # create new iterator for the moved luns :)
    my $moved_lun_iter_obj = NACL::MTask::LunIterators::LunIterator->new(
                volume   => $volume,
                method   => 'zapi',
            );
    $self->lun_iterator($moved_lun_iter_obj);
    $Log->exit() if $may_exit;
};

1;
=head1 NAME

NACL::MTask::LunIterators::LunMoveIterator - Provide an iterator object for iterating and doing a
lun Move operation.

=head1 SYNOPSIS

This is a lun Move iterator class and will iterate over the luns and do a move operation on luns
provided by input iterator.

extends NACL::MTask::LunIterators::LunMoveIterator;

The class overides base functions to check if the Lun object input is able to be moved and also overides
the operation function in this case a lun Move operation is performed.

=head1 DESCRIPTION

This Class provides an iterator that for each next() function will do the lun Move operation before return
the NACL::STask::Lun object for the Moved lun.

=head1 FUNCTIONS

=head2 new

Create a new iterator object instance and return it.

=over

=item C<< lun_iterator >> An instance of a lun iterator, it must support next
and has_next. The next method must return NACL::STask::Lun instances.

=back

=head2 has_next

Returns 1 if there is another "Move" lun available to get via a call to next or
0 if there are no more lun available.

=head2 next

Return the next "Move" lun as a NACL::STask::Lun instance.

=head2 next_n_operations

Does the lun "operation" on n luns and returns all the "operationed luns" as a Array Reference of
"NACL::STask::Lun" objects.
Note: This function cannot be used without group operations flag

=head1 SEE ALSO

MTask::LunIterators::OperationalLunIterator for documentation on other functions

=cut
