#
#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary MTask support Module
## @author asada@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here
##############################################################################
package NACL::MTask::LunIterators::ZapiLun;
use Moose;

# NATE
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NATE::BaseException qw(:try);
use Params::Validate qw(validate SCALAR ARRAYREF HASHREF SCALARREF );
use File::Spec::Functions qw(catdir);
use NACL::CS::Lun;

#NACL
use NACL::STask::Lun;

#
# Function: _type
#   Type of lun can be: lun, directory and so on
# Arguments:
#   [0] - String file type
# Returns:
#   Returns the type of file
#
has _type => (
    is => 'rw',
    isa => 'Str',
);

#
# Function: volume
#
# Arguments:
#   [0] - NACL::C:Volume object instance
# Returns:
#   Returns NACL::C::Volume object instance
#
has volume => (
    is => 'ro',
    isa => 'NACL::C::Volume',
    required => 1,
);

#
# Function: path
#
# Arguments:
#   [0] - path to the lun (qtree)
# Returns:
#   Returns the path to lun (qtree under the volume where luns are.
#
has path => (
    is => 'ro',
    isa => 'Str',
    default => '',
);

#
# Function: _name
#
# Arguments:
#   [0] - lun name
# Returns:
#   Returns lun name.
#
has _name => (
    is => 'ro',
    isa => 'Str',
    default => '',
);


sub raw_name {
    my $self = $_[0];
    my $raw_name = catdir ($self->path(),$self->_name());
    $raw_name = _agnostic_catdir_conversion($raw_name);
    return $raw_name;
}

sub standard_type {
    my $self = $_[0];
    return $self->_type();
}

sub is_directory {
    my $self = $_[0];
    if ($self->_type() =~ qr/directory/xi) {
        return 1; #TRUE
    }
    return 0; # FALSE
}

sub enumerate_directory {
    $Log->enter() if $may_enter;
    my $self = $_[0];

    my $volume = $self->volume();
    my $ci = $volume->command_interface();
    $ci->apiset(
        category => 'Node',
        interface => 'ZAPI',
        set => 'CMode',
    );

    my @zapi_lun_objs = ();

    my %options;
    my $qtree_path = '';
    if (length $self->path() > 0){
        $qtree_path = catdir($self->path(),$self->_name()); # Volume downwards e.g: /qtree1
        $qtree_path = _agnostic_catdir_conversion($qtree_path);
        $options{'qtree'} = $qtree_path;
    }

    # Returns Array of NACL::CS::Lun objects
    my @luns = NACL::CS::Lun->fetch(
        command_interface => $ci,
        filter            => {
            volume => $volume->volume(),
                        vserver => $volume->vserver(),
                        state => "online",
            %options,
        }
    );

    $Log->debug ("Scalar luns found in volume ". $volume->volume() . " num: " . scalar @luns);

    foreach my $lun_cs_obj (@luns) {

        # set qtree path to what the NACL::CS::Lun returned
        if ($qtree_path eq '' && defined $lun_cs_obj->{'qtree'} ) {
            if ($lun_cs_obj->{'qtree'} ne '""') {
                $Log->debug ("QTREE PATH: ". $lun_cs_obj->{'qtree'});
                $qtree_path = $lun_cs_obj->{'qtree'};
            }
        }

        # Lets create a zapi lun object for each entity
        my $baby_zapi_lun_obj = NACL::MTask::LunIterators::ZapiLun->new (
            volume => $volume,
            path => $qtree_path,
            _type => "lun",
            _name => $lun_cs_obj->lun(),
        );

        push @zapi_lun_objs, $baby_zapi_lun_obj;
    }

    $Log->debug(sub { "luns found under the specfied path: " . Dumper(\@zapi_lun_objs) });
    $Log->exit() if $may_exit;
    return @zapi_lun_objs;
}

sub make_lun {
    my $self = $_[0];
    $Log->enter() if $may_enter;

    my $volume = $self->volume();
    my $path =  catdir("","vol",$volume->volume,$self->raw_name());
    $path = _agnostic_catdir_conversion($path);

    $Log->debug("Lun path in make_lun: ". $path);

    # Convert the Lun to NACL::STask::Lun object
    my $vol_lun_obj = NACL::STask::Lun->new(
        command_interface => $volume->command_interface(),
        vserver => $volume->vserver(),
        path => $path,  # qtree/lun name
    );

    $Log->exit() if $may_exit;
    return $vol_lun_obj;
}

sub _agnostic_catdir_conversion {
    $Log->enter() if $may_enter;
    my $val = $_[0];
    $val =~ s/\\/\//g;
    $Log->exit() if $may_exit;
    return $val;
}

1;

=head1 NAME

NACL::MTask::LunIterators::ZapiLun - Zapi lun handles processing zapi file types
under lun iterator

=head1 SYNOPSIS

    use NACL::MTask::LunIterators::ZapiLun;

    my $zp_files_obj = NACL::MTask::LunIterators::ZapiLun->new(
        volume => $volume, # Volume object
        path => $path_to_qtree, # Can be empty
    );

=head1 DESCRIPTION

This Zapi luns library is used by Lun Iterator for create a iterator over luns (extracts using zapi CS).

=head1 FUNCTIONS

=head2 new

Build a new ZapiLuns object instance
Arguments:
volume - NACL::C::Volume instance
path - Path of the qtree where luns reside

=head3 Returns

object instance is returned

=head2 raw_name

Builds the complete qtree path and name
Arguments: n.a

=head3 Returns

Concatenated qtree path and name are returned.

=head2 enumerate_directory

Builds a list of (ZapiLuns object) Luns underneath the specified instance directory
Arguments: n.a (uses the instance path)

=head3 Returns

A list of Zapi Luns object is returned.

=head2 make_lun

Builds a NACL::C::VolumeFile object for Zapi File object
Arguments:
command_interface - (required) command interface object

=head3 Returns

A NACL::C::VolumeFile object for this instance of  Zapi Luns object.

=head2 standard_type

Lun type to return
Arguments: n.a

=head3 Returns

Returns the Lun object type (types: file, directory, blockdev, chardev, symlink, socket, fifo, stream, lun)
Note: Ours will always be a lun here as we use NACL::CS::Lun

=head2 is_directory

Returns TRUE 1 if object is a directory
Arguments:
Note for this class it will be always FALSE
=head3 Returns

TRUE or FALSE if file is dir or not.

=head1 SEE ALSO

=cut
