# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#

## @summary LunSetup Task Module
## @author dl-nacl-dev@netapp.com, hemamalini.rengarajan@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::MTask::LunSetup

=head1 DESCRIPTION

C<NACL::MTask::LunSetup> provides a number of well-defined but
potentially complex or multi-step methods related to LunSetup in
ONTAP.

The MTask will be concerned with -
  1) Setting up of Network Lif (C-Mode).
  2) Creating Igroup (C-Mode/7-Mode).
  3) Creating the Lun(s) (C-Mode/7-Mode).
  4) Mapping the Lun(s) to the igroup (C-Mode/7-Mode).

=head1 ATTRIBUTES

=head2 vserver

The Vserver which contains the volume (Cmode attribute).

=head2 data_protocol

The data protocol.Currently iscsi is supported for Cmode.
For 7mode, both iscsi & fcp is supported.

=head2 command_interface

The command interface in which the lun setup is done.

=head2 all_objects

A hash containing the objects used in the setup. This includes
the ones which were reused and the ones which were created by the task.
The objects are of the following types - NACL::STask::NetworkInterface,
NACL::C::VserverIscsi, NACL::STask::LunIgroup, NACL::STask::Lun.

=head2 task_created_objects

A hash containing only the objects created in the setup. This includes
only the ones which were created by the task.
The objects are of the following types - NACL::STask::NetworkInterface,
NACL::C::VserverIscsi, NACL::STask::LunIgroup, NACL::STask::Lun.

=cut

package NACL::MTask::LunSetup;
use base qw(NACL::MTask::MTask);

use strict;
use warnings;
use Tie::IxHash;
use Params::Validate qw(validate_with :types);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::STask::NetworkInterface;
use NACL::STask::SystemLicense;
use NACL::STask::Lun;
use NACL::STask::STask;
use NACL::C::VserverIscsi;
use NACL::STask::LunIgroup;
use NACL::STask::Node;
use NATE::BaseException qw(:try);
use NACL::Exceptions::HighAvailabilityError qw(:try);
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::ComponentUtils qw(Dumper _hash_copy);

use Class::MethodMaker [
    new    => [ '-hash', '-init', 'new' ],
    scalar => 'vserver',
    scalar => 'data_protocol',
    scalar => [
        { -type => 'NACL::C::CommandInterface::ONTAP' },
        'command_interface'
    ],
    hash => 'all_objects',
    hash => 'task_created_objects'
];

# Validate the parameters passed here
sub init {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            vserver => { type => SCALAR, optional => 1 },
            command_interface =>
                { isa => 'NACL::C::CommandInterface::ONTAP' },
            data_protocol => { type => SCALAR, }
        }
    );

    my $command_interface = $self->command_interface();
    my $data_protocol     = $self->data_protocol();
    if ( $command_interface->mode() =~ /CMode/ && $data_protocol !~ /iscsi/i ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Only iscsi type of data protocol support is present in CMode");
    }

    $Log->exit() if $may_exit;
} ## end sub init

=head1 METHODS

=head2 setup

(Class Method) The method creates one or more luns on a specified
volume. It maps the lun(s) to the given igroup .

(Cmode Method Call)
    # Specifying minimal options, and the library taking the other defaults,

        NACL::MTask::LunSetup->setup(
            command_interface => $ci,
            vserver => $vserver,
            data_protocol => 'iscsi',
            volume => $vol1,
            nacltask_lun                  => \@Lun_names,
            nacltask_path                 => [\@path_to_luns ],
            igroup                        => $Igroup,
            ostype                        => 'linux',
            nacltask_size                 => \@Lun_sizes,
            protocol                      => 'mixed',
            initiator                     => \@Initiator
        );

    # Specifying all the various options,

    NACL::MTask::LunSetup->setup(
            command_interface => $ci,
            vserver => $vserver,
            data_protocol => 'iscsi',
            volume => $vol1,
            qtree => $qtree,
            nacltask_lun                     => \@Lun_names,
            nacltask_path                    => [\@path_to_luns ],
            igroup                           => $Igroup,
            ostype                           => 'linux',
            nacltask_size                    => \@Lun_sizes,
            nacltask_lun_id                  => \@Lun_ids,
            protocol                         => 'mixed',
            initiator                        => \@Initiator,
            nacltask_datalif_opts            => { address => $address,netmask =>$net },
            nacltask_partner_datalif_opts    => { address => $address, netmask => $net , lif => $lif},
            nacltask_if_lif_exists           => 'reuse', #default die
            nacltask_if_lun_exists           => 'purge', #default die
            nacltask_if_igroup_exists        => 'reuse', #default die
            nacltask_if_lun_mapped_igroup    => 'remap', #default die
            nacltask_ignore_lif_checks       => 1,       #default 0
            nacltask_ignore_dataproto_checks => 1,       #default 0
            nacltask_partner_lif_create      => 0,       #default 1
            %other_options ## Apiset rulesets,method-timeout etc
            );

(7mode Method Call)

        NACL::MTask::LunSetup->setup(
                command_interface => $ci,
                data_protocol => 'iscsi',
                nacltask_lun                  => \@Lun_names,
                nacltask_path                 => [\@path_to_luns ],
                igroup                        => $Igroup,
                ostype                        => 'linux',
                nacltask_size                 => \@Lun_sizes,
                initiator                     => \@Initiator
            );

        # Specifying all the various options,

        NACL::MTask::LunSetup->setup(
                command_interface => $ci,
                data_protocol => 'iscsi',
                nacltask_lun                  => \@Lun_names,
                nacltask_path                 => [\@path_to_luns ],
                igroup                        => $Igroup,
                ostype                        => 'linux',
                nacltask_size                 => \@Lun_sizes,
                nacltask_lun_id               => \@Lun_ids,
                initiator                     => \@Initiator,
                nacltask_if_lun_exists        => 'purge', #default die
                nacltask_if_igroup_exists     => 'reuse', #default die
                nacltask_if_lun_mapped_igroup => 'remap', #default die
                %other_options ## Apiset rulesets,method-timeout etc
                );

The setup method takes care of handling the following based on
the user provided options for each -

1) Creates a DataLif for the given data_protocol in the vserver(CMode).
2) Starts the vserver iscsi service(CMode).
3) Creates an igroup with the given Initiator(s)(CMode/7Mode).
4) Creates the given list of luns on the volume(CMode/7Mode).
5) Maps the luns to the given igroup(CMode/7Mode).


=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required, isa NACL::C::CommandInterface)
The command interface on which to Setup the Lun(s).
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=item C<< volume => $vol1 >>

(Scalar Or isa NACL::C::Volume Or isa NACL::STask::Volume)
A volume on which we want to Setup the Lun(s).  It can take volume name (scalar)
or a volume component object (isa 'NACL::C::Volume') or a volume STask Object
(isa 'NACL::STask::Volume'). It is an optional parameter in Cmode.
This parameter is not used while performing 7mode lun setup.

=item C<< vserver => $vserver >>

(Scalar)
The vserver on which the volume should exist. This contains the vserver name.
It is a mandatory parameter in CMode, but not used while performing 7mode lun setup.

=item C<< data_protocol => $dataprotocol >>

The data protocol to be used by the vserver and Lun(s). At the moment only iscsi
is supported in Cmode, whereas fcp & iscsi is supported in 7Mode.

=item C<< nacltask_lun=> \@lun_names >>

(ArrayRef) The list of lun-names which has to be created in the setup.
Optional parameter in Cmode hosts, but required parameter in 7Mode hosts.

=item C<< qtree => $qtree >>

(Scalar) Name of qtree in which the lun has to  be created. In Cmode it is an optional parameter.
This parameter is not used while performing 7mode lun setup.

=item C<< nacltask_path=> \@path_to_luns >>

(ArrayRef) The list of lun-paths which need to be created.

For Cmode(Optional):
Caller can either pass volume and lun option, or pass 'path' itself.
One cannot pass volume and path both simultaneously. Similarly passing both
lun and path simultaneously too is mutually exclusive.

For 7Mode(Required):
The nacltask_path value passed should be in the form of /vol/<volume-name>/<qtree-name> or /vol/<volume-name>.
The value which is passed to nacltask_lun, will be accordingly appended at the end to form /vol/<volume-name>/<lun-name> or /vol/<volume-name>/<qtree-name>/<lun-name>.
This array reference can contain one or more number of paths. If only one path is passed, the same path is used for all the luns which is to be created.
If more than one path are passed, the number of paths should be equal to the number of nacltask_luns passed.

=item C<< igroup => $igroup  >>

(Required) This can be a scalar or an object of type NACL::C::LunIgroup or NACL::STask::LunIgroup .
If scalar a LunIgroup of the given name will be created by the setup routine.
If an object of the given type is passed, then the igroup name is extracted from the object
and used in lun map.

=item C<< ostype => $ostype  >>

(Required, Scalar) The ostype of the igroup and lun(s).

=item C<< nacltask_size => \@size  >>

(Required, ArrayRef) Size of the luns to be created. If only one is specified
the same is used for all the luns.

=item C<< nacltask_lun_id => \@lun_ids  >>

(Optional, ArrayRef) Lun-ids for mapping the luns. Either provide for all the
Luns passed in the input  or don't pass this option at all. The
filer will pick a default.

=item C<< protocol => $proto  >>

(Scalar) The protocol to be specified in the igroup.
Mandatory as a CMode option. This parameter is not used while performing 7mode lun setup.

=item C<< initiator => \@Initiator  >>

(Required, ArrayRef) List of initiators to be included in the igroup. These values would be the list of
WWPN values if the data_protocol used is fcp, else would be a list of initiator names if the data_protocol is iscsi.

=item C<< nacltask_datalif_opts => \%DataLif_Opts  >>

(HashRef) Network interface lif create options. If not provided,
the parameters from the NATE host file will be used.
The following fields from the NATE host file are used to create the Data Lif:

DATA_IP2 is passed in the 'address' field of lif create
DATA_MASK2 is passed in the  'netmask' field of lif create
DATA_PORT2 is passed in the  'home-port' field of lif create

The following options for 'lif' create take default values as specified
below :

 'home-node' = Name of the node is taken from the command_interface
 'lif' = Name of the node appended with '-data2'
 'role' = Defaults to 'data'
 'data-protocol' = iscsi/fcp (data_protocol type of this instance)

This parameter is optional in CMode & it is not used while performing 7mode lun setup.

=item C<< nacltask_partner_datalif_opts => \%partner_dataLif_Opts  >>

(Hashref) Lif create options in the partner Node. If not provided, the following
fields from the tharnhost file are used to create the data lif in partner :

DATA_IP2 is passed in the 'address' field of lif create
DATA_MASK2 is passed in the  'netmask' field of lif create
DATA_PORT2 is passed in the  'home-port' field of lif create

The following options for 'lif' create take default values as specified
below :

 'home-node' = Name of the partner node of the node belonging to command_interface
 'lif' = Name of the node appended with '_data2'
 'role' = Defaults to 'data'
 'data-protocol' = iscsi/fcp (data_protocol type of this instance)

This parameter is optional in CMode & it is not used while performing 7mode lun setup.

=item C<< nacltask_if_lif_exists  => $action  >>

(Scalar) Specifies an action to be taken if the lif already exists.

 $action
    - if set to "die"(default), an exception is raised.
    - if set to "reuse", no action is performed, and return the existing object
      referring to the corresponding Data Lif.
    - if set to "purge", deletes the existing DataLif before creating it.
This parameter is optional in CMode & it is not used while performing 7mode lun setup.

=item C<< nacltask_if_lun_exists  => $action  >>

(Optional, Scalar) Specifies an action to be taken if the lun already exists.

 $action
    - if set to "die"(default), an exception is raised.
    - if set to "reuse", no action is performed, and return the existing object
      referring to the corresponding Lun.
    - if set to "purge", deletes the existing Lun before creating it.

=item C<< nacltask_if_igroup_exists  => $action  >>

(Optional, Scalar) Specifies an action to be taken if the igroup already exists.

 $action
    - if set to "die"(default), an exception is raised.
    - if set to "reuse", no action is performed, and return the existing object
      referring to the corresponding igroup.
    - if set to "purge", deletes the existing igroup before creating it.

=item C<< nacltask_if_lun_mapped_igroup   => $action  >>

(Optional, Scalar) Specifies an action to be taken if the lun is already mapped
to this igroup.

 $action
    - if set to "die"(default), an exception is raised.
    - if set to "reuse", no action is performed .
    - if set to "remap", unmaps the lun from this igroup and remaps it again

=item C<< nacltask_ignore_lif_checks => 0 | 1 >>
(Applicable for CMode)
(Optional, Boolean) Specifies whether a data lif ,that uses the given protocol
(iscsi/fcp), needs to be created / verified if existing or not. 
By default it takes value '0'; Hence a data lif is verified for correctness 
if it exists or a new one is created.

=item C<< nacltask_ignore_dataproto_checks => 0 | 1 >>
(Applicable for CMode)
(Optional, Boolean) Specifies whether the  dataprotocol service (iscsi/fcp)
of  the vserver needs to be started / verified.
By default it takes value '0'. Hence the service is verified and started if
required at the vserver.

=item C<< nacltask_partner_lif_create => 0 | 1 >>
(Applicable for CMode)
(Optional, Boolean) Specifies whether a data lif, that uses the given protocol
(iscsi/fcp), needs to be created.
By default it takes value '1'; Hence a data lif is created in the partner
as well ; otherwise it is not created .
Note that if nacltask_ignore_lif_checks option is set to 1, then
partner lif creation is skipped as well.

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub setup {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    if ( ref($pkg) ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "This Setup can be invoked only as a  NACL::MTask::LunSetup class method"
        );
    } ## end if ( ref($pkg) )

    my %opts = NACL::C::Component->_common_validate_with(
        params          => \@_,
        additional_spec => {
            command_interface =>
                { isa => 'NACL::C::CommandInterface::ONTAP' },
        },
        allow_extra => 1
    );

    my $return;
    my $command_interface = $opts{command_interface};

    if ( $command_interface->mode() =~ /CMode/ ) {
        $return = $pkg->_setup_cmode(%opts);
    } elsif ( $command_interface->mode() =~ /7Mode/ ) {
        $return = $pkg->_setup_7mode(%opts);
    }

    $Log->exit() if $may_exit;
    return $return;
} ## end sub setup

sub _setup_cmode {

    $Log->enter() if $may_enter;
    my $pkg = shift;

    my %opts = NACL::C::Component->_common_validate_with(
        params          => \@_,
        additional_spec => { %{ $pkg->_params_validate_spec() } },
        allow_extra     => 1
    );

    $Log->debug( "This is the opts to _setup_cmode" . Dumper( \%opts ) );

    my ( $command_interface, $data_protocol, $vserver, $self );
    my ( %created_objs, %all_objs ,$apiset);
    $command_interface = delete $opts{'command_interface'};
    $data_protocol     = delete $opts{'data_protocol'};
    $vserver           = delete $opts{'vserver'};

    $apiset = $command_interface->apiset(interface => 'CLI');

    $self = $pkg->new(
        command_interface => $command_interface,
        data_protocol     => $data_protocol,
        vserver           => $vserver
    );

    my $volume = delete $opts{'volume'};
    my $qtree  = delete $opts{'qtree'};

    my ( $component, $feature, %common_opts);
    $self->_move_common_component_params(
        source => \%opts,
        target => \%common_opts
    );

    # need these variables to keep track of element creation
    my ($was_lif_created,             $was_lun_created,
        $was_vs_dataprotocol_created, $was_igroup_created,
        $was_added,
    );

    ## delete nacltask_ignore_lif_checks
    my $ignore_lif_checks = delete $opts{nacltask_ignore_lif_checks};
    my $ignore_dp_checks  = delete $opts{nacltask_ignore_dataproto_checks};

    # User can provide if_exists option for each element involved in this setup
    my $if_lif_exists        = delete $opts{nacltask_if_lif_exists};
    my $if_igroup_exists     = delete $opts{nacltask_if_igroup_exists};
    my $if_lun_exists        = delete $opts{nacltask_if_lun_exists};
    my $if_lun_mapped_igroup = delete $opts{nacltask_if_lun_mapped_igroup};

    # Allowing the user to pass other parameters if any specific for the
    # LIF create operation
    my $datalif_opts = $opts{nacltask_datalif_opts};
    my $partner_datalif_opts = $opts{nacltask_partner_datalif_opts};

    # Ensure that the user doesnot pass path and lun both
    if ( defined $opts{'nacltask_lun'} && defined $opts{'nacltask_path'} ) {
        NATE::BaseException->throw( "Either pass the list of lun names  "
                . " or the list of lun paths . Not both " );
    }

    my ( @initiator, @lun, @lun_id, @lun_size, $size, $lun_or_path );

    # set the flag $lun_or_path as per the option passed by the user
    # to be passed to the lun create task
    if ( defined $opts{'nacltask_lun'} ) {
        $lun_or_path = 'lun';
        @lun         = @{ $opts{'nacltask_lun'} };
    } else {
        $lun_or_path = 'path';
        @lun         = @{ $opts{'nacltask_path'} };
    }

    # Get the list of initiators and list of lun ids passed by the user
    @initiator = @{ $opts{'initiator'} };
    @lun_id    = @{ $opts{'nacltask_lun_id'} }
        if defined( $opts{'nacltask_lun_id'} );

    if ( scalar @{ $opts{'nacltask_size'} } == 1 ) {
        $size = @{ $opts{'nacltask_size'} }[0];
        $Log->debug("This is the Lun size $size");
    } else {
        @lun_size = @{ $opts{'nacltask_size'} };
    }

    # User need to pass the lun size as one value - All luns will take
    # the same value. Otherwise the number of lun sizes should be equal
    # to the number of luns passed

    if ( @lun_size && scalar @lun != scalar @lun_size ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw( "Provide lun-sizes equal to the number "
                . "of luns/paths given in the parameter" );
    }

    # If the user passes the lun id it should be equal to the number of
    # luns specified. Otherwise the user cannot pass anything also.
    # Filer will accept null value for map.

    if ( scalar @lun_id && scalar @lun != scalar @lun_id ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw( "Number of lun-ids has to be  equal  "
                . "to the number of luns given in the parameter " );
    }

    foreach my $param
        ( qw(initiator nacltask_lun nacltask_lun_id nacltask_size nacltask_path))
    {
        delete $opts{$param} if ( exists $opts{$param} );
    }
    my ($obj,$existing_lif_obj);
    # check if the data protocol of the existing lif matches the
    # one required for this setup
        # validate the data lif opts
        if (!$ignore_lif_checks){

        %$datalif_opts = validate_with(
            params      => \%$datalif_opts,
            spec        => $self->_datalif_validate_spec($command_interface),
            allow_extra => 1
        );
        if ( $if_lif_exists eq 'reuse' ) {
            try {
                $existing_lif_obj = $self->_verify_protocol_if_lif_exists(%$datalif_opts);
            }
            catch NACL::Exceptions::NoElementsFound with {
                my $exception = shift;
                $Log->comment( "nacltask_if_lif_exists equals reuse; \n"
                        . "But a data lif with the expected name "
                        . $datalif_opts->{lif}
                        . " does not exist already.\n Creating a new one .." );

                try {
                    $obj = NACL::STask::NetworkInterface->create(
                        command_interface => $command_interface,
                        vserver           => $vserver,
                        %$datalif_opts,
                        ## since the data lif does not exist, while
                        ## creating the lif, setting nacltask_if_exists 
                        ## to die, to handle duplicate ip address issue
                         nacltask_if_exists => 'die',
                        _was_created       => \$was_lif_created,
                        %common_opts
                    );
                }
                catch NACL::C::Exceptions::NetworkInterface::AlreadyExists
                    with {
                    my $ex = shift;
                    $existing_lif_obj = _catch_dup_ip_handler(
                        exception         => $ex,
                        _warn_or_die      => "warn",
                        command_interface => $command_interface,
                        vserver           => $vserver,
                        lif_opts          => $datalif_opts
                    );
                    };
            }    #catch NACL::Exceptions::NoElementsFound
            catch NATE::BaseException with {
                my $exception = shift;
                $Log->comment( "Exception Text " . $exception->text() );
                $Log->warn( "While trying to reuse the existing lif "
                        . $datalif_opts->{lif}
                        . "\nthe  "
                        . "data-protocol does not match the expected "
                        . $datalif_opts->{'data-protocol'} );
            };
        }    ## if if_lif_exists eq reuse ends here
        else {
            try {
                $obj = NACL::STask::NetworkInterface->create(
                    command_interface => $command_interface,
                    vserver           => $vserver,
                    %$datalif_opts,
                    nacltask_if_exists => $if_lif_exists,
                    _was_created       => \$was_lif_created,
                    %common_opts
                );
            }
            catch NACL::C::Exceptions::NetworkInterface::AlreadyExists with {
                my $ex = shift;
                $existing_lif_obj = _catch_dup_ip_handler(
                    exception         => $ex,
                    # die here means throwing the exception 
                    _warn_or_die      => "die",
                    command_interface => $command_interface,
                    vserver           => $vserver,
                    lif_opts          => $datalif_opts
                );
            };
        }    #else ends here
    $all_objs{'lif'} = $obj || $existing_lif_obj ;
    if ($was_lif_created) {
        $created_objs{'lif'} = $obj;
    }
    ## Partner Lif Creation goes here
    if ($opts{nacltask_partner_lif_create}) {
        my $node_task_obj = $command_interface->cast_component_to_task();
        try {
            my $partner_node_obj = $node_task_obj->get_partner_obj();
            if ($partner_node_obj) {
                my $partner_node     = $partner_node_obj->node();
                my $partner_lif_name = $partner_node . '_data2';
                my $was_partner_lif_created;
                my $partner_datalif = {
                    'data-protocol' => [$data_protocol],
                    address         => $partner_node_obj->hostrec->{DATA_IP2},
                    "home-port" => $partner_node_obj->hostrec->{DATA_PORT2},
                    netmask     => $partner_node_obj->hostrec->{DATA_MASK2},
                    "home-node" => $partner_node_obj->node(),
                    lif => $partner_lif_name
                };
                if ( defined $partner_datalif_opts ) {
                    $partner_datalif
                        =  $partner_datalif_opts;
                }
                my $partner_lif_obj;
                try {
                    ## If exists with the same attributes reuse
                    ## If exists with different attributes purge and re-create
                    $partner_lif_obj
                        = NACL::STask::NetworkInterface->find_or_create(
                        command_interface => $partner_node_obj,
                        vserver           => $vserver,
                        'data-protocol'   => [$data_protocol],
                        %$partner_datalif,
                        _was_created => \$was_partner_lif_created,
                        if_exists_with_different_attributes => 'purge',
                        );
                }
                catch NATE::BaseException with {
                    my $e = shift;
                    $Log->warn( "Exception While doing partner lif creation "
                            . $e->text() );
                };
                $all_objs{'partner_lif'} = $partner_lif_obj;
                if ($was_partner_lif_created) {
                    $created_objs{'partner_lif'} = $partner_lif_obj;
                }

            }    # If partner_node_obj
        }
        catch NACL::Exceptions::HighAvailabilityError with {
            $Log->comment("There is no partner node. So no lif creation ");
        };
    }    #Partner_lif_create
    }##if ignore lif checks ends here

    if (!$ignore_dp_checks){
    if ( $data_protocol =~ /^iscsi$/i ) {
        $component = 'VserverIscsi';
        $feature   = 'iscsi';
    } else {
        $component = 'VserverFcp';
        $feature   = 'fcp';
    }
    $obj = NACL::STask::SystemLicense->add(
        command_interface => $command_interface,
        feature           => $feature,
        _was_added        => \$was_added,
    );

    $all_objs{'license'} = $obj;
    if ($was_added) {
        $created_objs{'license'} = $obj;
    }

    $obj = $self->_handle_creation(
        _component     => "NACL::C::$component",
        _method        => "create",
        _delete_method => "stop,delete",
        if_exists      => 'reuse',
        'target-alias' => $vserver,
        'vserver'      => $vserver,
        'status-admin' => 'up',
        _was_created   => \$was_vs_dataprotocol_created,
        %opts
    );

    $all_objs{'dataprotocol'} = $obj;
    if ($was_vs_dataprotocol_created) {
        $created_objs{'dataprotocol'} = $obj;
    }
    }## if !$ignore_dp_checks
    my $ref_ig = $opts{igroup};
    if ( ref $ref_ig ) {
        if (   !$ref_ig->isa('NACL::C::LunIgroup')
            && !$ref_ig->isa('NACL::STask::LunIgroup') ) {
            my $ref = ref $ref_ig;
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw(
                      "The parameter igroup should either be a scalar "
                    . "igroup name or igroup component object NACL::C::LunIgroup"
                    . " or igroup task object NACL::STask::LunIgroup. "
                    . " But the argument is of type $ref " );
        }
        $all_objs{'igroup'} = $ref_ig;
    } else {
        $Log->debug(
            sub {
                "These are the opts inside lunigroup create "
                    . Dumper( \%opts );
            }
        );
        my %_ig_opts;
        %_ig_opts = NACL::C::Component::ONTAP->get_api_options(
        apiset  => $apiset,
        api     => 'lun_igroup_create',
        options => \%opts,
        );
        
        if ( $if_igroup_exists ne 'reuse' ) {
            $obj = NACL::STask::LunIgroup->create(
                %_ig_opts,
                command_interface  => $command_interface,
                nacltask_if_exists => $if_igroup_exists,
                initiator          => \@initiator,
                vserver            => $vserver,
                _was_created       => \$was_igroup_created,
            );
            $all_objs{'igroup'} = $obj;
        }    ## ne reuse
        else {
            try {
                $obj = NACL::STask::LunIgroup->find_or_create(
                    command_interface => $command_interface,
                    _was_created      => \$was_igroup_created,
                    vserver           => $vserver,
                    if_exists_with_different_attributes => "die",
                    initiator                           => \@initiator,
                    %_ig_opts
                );
                $all_objs{'igroup'} = $obj;
            }
            catch NACL::Exceptions::VerifyFailure with {
                my $e       = shift;
                my $hashref = $e->unexpected_values();
                my $init_value;
                $Log->trace(
                    "Here is the verifyfailure value " . Dumper($hashref) );
                foreach my $key ( keys %$hashref ) {
                    if ( $key eq 'initiator' ) {
                        $init_value = $hashref->{$key};
                        last;
                    }
                }
                ## If there is difference in the initiator attribute
                ## just add the initiator passed to setup method
                ## to the existing attribute
                ## Fix for burt 706998
                if ($init_value) {
                    my @n_initiator = @initiator;
                    NACL::STask::LunIgroup->add(
                        command_interface => $command_interface,
                        initiator         => \@n_initiator,
                        igroup            => $_ig_opts{igroup},
                        vserver           => $vserver
                    );
                    my $ig_obj = NACL::STask::LunIgroup->find(
                        command_interface => $command_interface,
                        filter => {
                                  vserver           => $vserver,
                                  igroup            => $_ig_opts{igroup}
                                  }
                    );
                    $all_objs{'igroup'} = $ig_obj;
                } else {
                    ## This means there is difference in some other
                    ## attribute. Since the original option is reuse, simply call
                    ## NACL::STask::LunIgroup->create here
                    $obj = NACL::STask::LunIgroup->create(
                        %_ig_opts,
                        command_interface  => $command_interface,
                        nacltask_if_exists => $if_igroup_exists,
                        initiator          => \@initiator,
                        vserver            => $vserver,
                        _was_created       => \$was_igroup_created,
                    );
                    $all_objs{'igroup'} = $obj;
                }
            };
        }
    }
    if ($was_igroup_created) {
        $created_objs{'igroup'} = $obj;
    }

    my ( @_lun_objects, $_create_size, $_id, %_map_opts, %_lun_opts );
    my @create_param;
    # If the user passes lun-id then that needs to be included
    # Otherwise its not needed. So need a separate param hash
    # for including/excluding this value - %_map_opts

    %_map_opts = %opts;
    if (ref $ref_ig) { $_map_opts{igroup} = $ref_ig->igroup();  }
    $Log->debug( sub {  "These are common opts " . Dumper( \%opts ) } );
    $Log->debug( sub {  "These are map opts " . Dumper( \%_map_opts ) } );

    my %_lun_api_opts = NACL::C::Component::ONTAP->get_api_options(
        apiset  => $apiset,
        api     => 'lun_create',
        options => \%opts,
    );


    foreach my $index ( 0 ... $#lun ) {
        
       if ( !(@lun_size) ) {
            $_create_size = $size;
        } else {
            $_create_size = $lun_size[$index];
        }
        @create_param = (
            'command_interface' => $self->command_interface(),
            'vserver'           => $vserver,
            $lun_or_path        => $lun[$index]
        );

        if ( defined $volume ) {
            push( @create_param, 'volume' );
            push( @create_param, $volume );
        }
        if ( defined $qtree ) {
            push( @create_param, 'qtree' );
            push( @create_param, $qtree );
        }
        $Log->debug( sub {
            "This is the array for NACL::C::Lun _common_validate_with "
                . Dumper( \@create_param ) } );

        %_lun_opts = NACL::C::Lun->_common_validate_with(
            params      => \@create_param,
            allow_extra => 1,
        );

       $Log->debug( sub { "The lun create opts " . Dumper( \%_lun_opts ) } );

        $obj = NACL::STask::Lun->create(
            if_exists    => $if_lun_exists,
            size         => $_create_size,
            _was_created => \$was_lun_created,
	        ostype	     => $opts{ostype},
            %_lun_api_opts,
            %_lun_opts,
        );

        push( @_lun_objects, $obj );

        if ( scalar @lun_id ) {
            $_id = $lun_id[$index];
            $_map_opts{'lun-id'} = $_id;
        }

        $obj->map(
            nacltask_if_mapped_igroup => $if_lun_mapped_igroup,
            allow_extra               => 1,
            nacltask_verify           => 1,
            %_map_opts
        );

    }    ## foreach loop ends here

    $all_objs{'lun'} = \@_lun_objects;
    if ($was_lun_created) {
        $created_objs{'lun'} = \@_lun_objects;
    }

    $self->all_objects(%all_objs);
    $self->task_created_objects(%created_objs);
    my %dump = $self->task_created_objects();
    $Log->debug( sub {"Set of objects created by task ".Dumper(\%dump) } );
    $Log->exit() if $may_exit;
    return $self;

} ## end sub _setup_cmode

=head2 cleanup

$Lun_Iscsi->cleanup();

or

$self->cleanup(delete_all_objects => 1);

(Instance method)
By default, the method deletes all the element objects created by this instance,
upon invocation. For Cmode the objects include lun, datalif, lunigroup, vserveriscsi,
whereas in 7Mode the objects include lun, deleting the lun-mapping & igroups. If the
user sends a hash with key equals delete_all_objects and value equals '1', then
all the objects created by the setup task through this instance will be deleted.

=over

=item Options

=over

=item C<< delete_all_objects => 1 >>

(Optional, defaults to '0')

By default, the cleanup deletes only the objects created by the task and contained
in the instance attribute %task_created_objects.

If delete_all_objects is set to '1', then delete is invoked for the instance attribute %all_objects.

=back

=back

=cut

sub cleanup {
    $Log->enter() if $may_enter;
    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            delete_all_objects => {
                type      => BOOLEAN,
                default   => 0,
                callbacks => {
                    "option 'delete_all_objects' takes either '1' or '0'  " =>
                        sub { $_[0] eq '0' || $_[0] eq '1' }
                }
            }
        }
    );

    my $delete_all_objects = $opts{'delete_all_objects'};

    NATE::BaseException->throw(
        "This cleanup can be invoked only with an object of NACL::MTask::LunSetup class"
    ) unless $self->isa('NACL::MTask::LunSetup');

    my ( %input_obj_hash, %obj_hash );

    if ( $delete_all_objects == '1' ) {
        %input_obj_hash = $self->all_objects();
    } else {
        %input_obj_hash = $self->task_created_objects();
    }
    $Log->debug( "Lib Delete Objs "
            . NACL::ComponentUtils::Dumper( \%input_obj_hash ) );

    tie %obj_hash, "Tie::IxHash";

    # Insert in the same order in which the objects have to be deleted
    # To resolve dependencies among the components/elements
    foreach my $key_obj ( qw(lun igroup dataprotocol lif  partner_lif license)) {
        $obj_hash{$key_obj} = $input_obj_hash{$key_obj};
    }
    # Instead of invoking serially based on the specific object
    # Will have a logic to do this generically
    # Expecting more objects to get added with support to 7Mode/FCP etc
    # Not sure at the moment reg. the number though

    my ( $key, $value, $ref );

    while ( ( $key, $value ) = each %obj_hash ) {
        $ref = ref $value;
        if ( defined $value ) {
            if ( $ref =~ /NACL::STask::/ ) {
                $value->purge();
            } elsif ( $ref =~ /NACL::C::VserverIscsi/i
                || $ref =~ /NACL::C::VserverFcp/i ) {
                $value->stop();
                $value->delete();
            } ## end elsif ( $ref =~ /NACL::C::VserverIscsi/i...
            # Atleast for now Lun objects is the only
            # array type member in this hash
            elsif ( $ref =~ /ARRAY/i ) {
                foreach my $_objs ( @{$value} ) {
                    $_objs->purge();
                }
            }    ## elsif condition ends here
            else {
                $value->delete();
            }
        }
    } ## end while ( ( $key, $value ) ...
    $Log->exit() if $may_exit;
} ## end sub cleanup

=begin COMMENTED_OUT

    $self->_params_validate_spec()

(For developers )
A validate spec that can be used to validate the parameters passed for the
setup method.

=end COMMENTED_OUT

=cut

sub _params_validate_spec {

    return {
        nacltask_if_lif_exists =>
            NACL::STask::STask->_if_exists_validate_spec(),
        nacltask_if_lun_exists =>
            NACL::STask::STask->_if_exists_validate_spec(),
        nacltask_if_igroup_exists =>
            NACL::STask::STask->_if_exists_validate_spec(),
        nacltask_if_lun_mapped_igroup =>
            NACL::STask::Lun->_if_already_mapped_igroup_spec(),
        data_protocol => {
            type      => SCALAR,
            callbacks => {
                      "The value of 'data-protocol' has to be  "
                    . " 'iscsi' " => sub { $_[0] =~ /^(iscsi)$/i }
            }
        },
        command_interface => { isa  => 'NACL::C::CommandInterface::ONTAP' },
        vserver           => { type => SCALAR },
        volume            => {
            type     => SCALAR | OBJECT,
            optional => 1,
            depends  => ['nacltask_lun']
        },
        nacltask_lun          => { type => ARRAYREF,        optional => 1 },
        nacltask_path         => { type => ARRAYREF,        optional => 1 },
        qtree                 => { type => SCALAR,          optional => 1 },
        igroup                => { type => SCALAR | OBJECT },
        protocol              => { type => SCALAR ,optional => 1},
        initiator             => { type => ARRAYREF ,optional => 1},
        ostype                => { type => SCALAR ,optional => 1 },
        nacltask_size         => { type => ARRAYREF },
        nacltask_lun_id       => { type => ARRAYREF,        optional => 1 },
        nacltask_datalif_opts => { type => HASHREF | UNDEF, optional => 1 },
        nacltask_partner_datalif_opts => { type => HASHREF | UNDEF, optional => 1 },
        nacltask_ignore_lif_checks => {type => BOOLEAN , default => 0 },
        nacltask_ignore_dataproto_checks => {type => BOOLEAN , default => 0 },
        nacltask_partner_lif_create => {type => BOOLEAN, default => 1 }
    };
} ## end sub _params_validate_spec

=begin COMMENTED_OUT

    $self->_datalif_validate_spec()

(For developers )
The validation spec that can be used for validating the nacltask_datalif_opts,
passed by the caller, to the setup() routine. The lif create options which user
has already provided, are validated for type .
The create options which are not provided by the user,the corresponding
parameters from tharnhost  will be taken .

Following fields from the tharnhost are taken to create the Data Lif -

DATA_IP2 from tharnhost is passed in the 'address' field of lif create
DATA_MASK2 from tharnhost is passed in the  'netmask' field of lif create
DATA_PORT2 from tharnhost is passed in the  'home-port' field of lif create

The following options for 'lif' create take default values as specified
below :

 'home-node' = Name of the node is taken from the command_interface
 'lif' = Name of the node appended with '-data2'
 'role' = Defaults to 'data'


This can be used for the NACL::STask::NetworkInterface->create() method in this
LunSetup task

=end COMMENTED_OUT

=cut

sub _datalif_validate_spec {
    my $self          = shift;
    my $ci            = shift;
    my $data_protocol = $self->data_protocol();
    my $vserver       = $self->vserver();
    return {
        'vserver' => {
            type      => SCALAR,
            default   => $self->vserver(),
            callbacks => {
                "'vserver' is specified as '$_[0]' in the call to setup "
                    . " whereas the object contains the value as '$vserver' "
                    => sub { defined $vserver && $_[0] =~ /^$vserver$/i }
            }
        },
        'data-protocol' => {
            type      => SCALAR | ARRAYREF,
            default   => [$data_protocol],
            callbacks => {
                "'data-protocol' in the call to setup "
                    . "should be defined and it should contain the value as "
                    . " '$data_protocol' " => sub {
                            if(defined $data_protocol) {
                                if(ref($_[0])) {
                                    return 1 if( grep  /^$data_protocol$/i, @{$_[0]});
                                }else {
                                    return 1 if($_[0] =~ /^$data_protocol$/i); 
                                }
                            }
                            return 0;
                    }
                }    ## end
        },
        'role' => {
            type      => SCALAR,
            default   => 'data',
            callbacks => {
                "The value of 'role' has to be 'data' " => sub {
                    $_[0] =~ /^data$/i;
                    }
            }
        },
        'address' => {
            type    => SCALAR,
            default => $ci->hostrec()->DATA_IP2(),
        },
        'netmask' => {
            type    => SCALAR,
            default => $ci->hostrec()->DATA_MASK2(),
        },
        'home-node' => {
            type    => SCALAR,
            default => $ci->node(),
        },
        'home-port' => {
            type    => SCALAR,
            default => $ci->hostrec()->DATA_PORT2(),
        },
        'lif' => {
            type    => SCALAR,
            default => $ci->node() . '_data2'
            }    ## end

    };
} ## end sub _datalif_validate_spec

=begin COMMENTED_OUT

    $existing_lif_obj = $self->_verify_protocol_if_lif_exists()

(For developers )
If the datalif provided by the caller exists already, and user has indicated
reuse, then this helper routine can be called to ensure that the data-protocol
of this LIF is equal to $self->data_protocol() , and the role of the lif is
'data' . If the verification is succeds it returns existing lif object.

=end COMMENTED_OUT

=cut

sub _verify_protocol_if_lif_exists {
    $Log->enter() if $may_enter;
    my $self         = shift;
    my %opts         = @_;
    my $existing_obj = NACL::STask::NetworkInterface->find(
        command_interface => $self->command_interface(),
        filter            => {
            vserver => $self->vserver(),
            lif     => $opts{lif}
        },
        allow_empty => 0
    );
    my $message;
    if ( defined($existing_obj) ) {

       #Can't use verify_state because data-protocol is an array field
       #There is no support yet in the verify_fields method.
       #$existing_obj->verify_state('data-protocol' => $self->data_protocol(),
       #                            'role' => $opts{'role'} ) ;
       #doesnot work

        my $state = $existing_obj->state();
        if ( $state->role() ne $opts{'role'} ) {
            $message = " Expected Role " . $opts{role};
            $message .= "Actual role " . $state->role();
            $message
                .= "The role of the existing Lif "
                . $opts{lif}
                . " doesnot match the expected "
                . $opts{'role'};

            $Log->exit() if $may_exit;
            NATE::BaseException->throw($message);

        } ## end if ( $state->role() ne...

        my @actual_data_protocols = $state->data_protocol();
        my $dp = $self->data_protocol();
        $Log->debug("self data proto $dp");
        my $count = grep /^$dp$/, @actual_data_protocols;
        if (!$count){
            $Log->debug("Existing data lif: " .$opts{lif}." ,does not have the expected data protocol: ".
                          $self->data_protocol."\n");
            $message = "Expected data-protocol " . $self->data_protocol();
            $message .= "\nActual data-protocol " . Dumper(\@actual_data_protocols);

            $message
                .= "\nThe data-protocol of the existing Lif :"
                . $opts{lif}
                . " doesnot match the expected :"
                . $self->data_protocol(). "\n";
            $Log->exit() if $may_exit;
            NATE::BaseException->throw($message);
        } ## end if ( !$count

        return $existing_obj;

    } ## end if ( defined($existing_obj...
    $Log->exit() if $may_exit;
} ## end sub _verify_protocol_if_lif_exists

=begin COMMENTED_OUT

    $self->_handle_creation(%opts);

(For developers )
This is a helper method to call create() on a specific component provided by
the user . The caller has to specify the component-name in the _component key of
the input hash. The name of the method to create the element is specified in
the _method. _delete_method contains the name of the delete method that will be
invoked on this element, in case the caller has specified 'purge' in the if_exists
options. Additionally a scalar reference _was_created is passed which returns
if the element was created. If upon return it contains '0' , then it indicates
that the element was reused. If it contains '1' , then the element was created
by this create() call.The caller can specify if he wants to 'reuse' the element
if it exists , or 'purge' it and create it again , or 'die' which will throw
an exception if it already exists.

=end COMMENTED_OUT

=cut

sub _handle_creation {

    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = @_;

    # get the name of the component and the name of the method required for
    # create/map
    my $component     = delete $opts{_component};
    my $method        = delete $opts{_method};
    my $delete_method = delete $opts{_delete_method};
    my $was_created   = delete $opts{_was_created};
    $$was_created = 0;

    # extract other necessary details
    my $command_interface = $self->command_interface();
    my $if_exists         = delete $opts{if_exists};
    my $existing          = undef;

    # We will not handle the if_exists => 'die' case here since the
    # create call will anyway throw an error if the element exists.
    if ( $if_exists ne 'die' ) {
        $Log->trace('Attempt to find the element');
        $existing = $component->find(
            command_interface => $command_interface,
            filter            => $component->get_primary_keys_options(%opts),
            allow_empty       => 1
        );
    } ## end if ( $if_exists ne 'die')

    if ($existing) {
        $Log->trace('Found a matching element');
        if ( $if_exists eq 'purge' ) {
            $self->_purge(
                %opts,
                _component     => $component,
                _delete_method => $delete_method
            );

        } elsif ( $if_exists eq 'reuse' ) {
            if ($component =~/VserverIscsi/i) {
                my $iscsi_status = $existing->state()->status_admin();
                if ($iscsi_status =~/down/i) {
                   $component->start(
                       command_interface => $command_interface,
                       vserver           => $opts{vserver},
                   );
                }
            }
            $Log->exit() if $may_exit;
            return $existing;
        }
    } ## end if ($existing)

# Create the element here if not found above, and if if_exists is set to 'die'
    my $new_obj = $component->$method(
        %opts,
        command_interface => $command_interface,
        allow_extra       => 1
    );
    $$was_created = 1;
    $Log->exit() if $may_exit;
    return $new_obj;
} ## end sub _handle_creation

=begin COMMENTED_OUT

    $self->_purge(%opts);

(For developers )
If the caller has specified 'nacltask_if_element_exists' option for a particular
element , then this helper routine is invoked by the _handle_creation helper routine
to delete the existing element.

The caller/developer can specify _component which specifies the
appropriate componentname that will be invoked and _delete_method which specifies
the name of the routine to be used for delete operation.
The input %opts passed to this helper routine should contain the input values
for invoking _delete_method provided, on the corresponding component.


=end COMMENTED_OUT

=cut

sub _purge {
    $Log->enter() if $may_enter;
    my $self              = shift;
    my %opts              = @_;
    my $component         = delete $opts{_component};
    my $method_list       = delete $opts{_delete_method};
    my @all_methods       = split ',', $method_list;
    my $command_interface = $self->command_interface();

    # call either delete/remove on the package provided
    foreach my $method (@all_methods) {
        $component->$method(
            command_interface => $command_interface,
            %opts, allow_extra => 1
        );
    } ## end foreach my $method (@all_methods)

    $Log->exit() if $may_exit;
} ## end sub _purge

sub _setup_7mode {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    my %opts = NACL::C::Component->_common_validate_with(
        params          => \@_,
        additional_spec => {
            command_interface =>
                { isa => 'NACL::C::CommandInterface::ONTAP' },
            data_protocol   => { type => SCALAR, },
            nacltask_lun    => { type => ARRAYREF },
            nacltask_path   => { type => ARRAYREF },
            igroup          => { type => SCALAR | OBJECT},
            ostype          => { type => SCALAR },
            nacltask_size   => { type => ARRAYREF },
            nacltask_lun_id => { type => ARRAYREF, optional => 1 },
            initiator       => { type => ARRAYREF },
            nacltask_if_igroup_exists =>
                NACL::STask::STask->_if_exists_validate_spec(),
            nacltask_if_lun_mapped_igroup =>
                NACL::STask::Lun->_if_already_mapped_igroup_spec(),
            nacltask_if_lun_exists =>
                NACL::STask::STask->_if_exists_validate_spec(),
        },
        allow_extra => 1
    );

    $Log->debug(
        "The options passed to _setup_7mode are " . Dumper( \%opts ) );

    my $command_interface    = delete $opts{command_interface};
    my $data_protocol        = delete $opts{data_protocol};
    my $if_igroup_exists     = delete $opts{nacltask_if_igroup_exists};
    my $if_lun_exists        = delete $opts{nacltask_if_lun_exists};
    my $if_lun_mapped_igroup = delete $opts{nacltask_if_lun_mapped_igroup};
    my $ostype               = delete $opts{ostype};
    my $igroup               = delete $opts{igroup};
    my ( $obj, $was_igroup_created, $size, $path, $was_lun_created ,$apiset);

    $apiset = $command_interface->apiset(interface => 'CLI');
    my @initiators = @{ $opts{initiator} };
    my ( @lun_sizes, @lun_paths, @luns, @lun_create_opts, @lun_ids,
        @_all_lun_objects, @_created_lun_objects );
    my ( %all_objs, %created_objs, %common_opts );

    my $self = $pkg->new(
        command_interface => $command_interface,
        data_protocol     => $data_protocol,
    );

    $self->_move_common_component_params(
        source => \%opts,
        target => \%common_opts
    );

    if ( ref $igroup ) {
        if (   !$igroup->isa('NACL::C::LunIgroup')
            && !$igroup->isa('NACL::STask::LunIgroup') ) {
            my $ref = ref $igroup;
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw(
                      "The parameter igroup should either be a scalar "
                    . "igroup name or igroup component object NACL::C::LunIgroup"
                    . " or igroup task object NACL::STask::LunIgroup. "
                    . "But the argument is of type  $ref " );
        }
        $all_objs{'igroup'} = $igroup;
    } else {
        $Log->debug(
            sub {
                "These are the opts inside lunigroup create "
                    . Dumper( \%opts );
            }
        );

        my %_ig_opts ;
        %_ig_opts = NACL::C::Component::ONTAP->get_api_options(
            apiset  => $apiset,
            api     => 'igroup_create',
            options => \%opts,
        );

        $Log->debug(
             sub {
                 "This is _ig_opts for lunigroup create "
                     . Dumper( \%_ig_opts );
             }
         );

        $obj = NACL::STask::LunIgroup->create(
            %_ig_opts,
            protocol           => $data_protocol,
            ostype             => $ostype,
            igroup             => $igroup,
            command_interface  => $command_interface,
            nacltask_if_exists => $if_igroup_exists,
            initiator          => \@initiators,
            _was_created       => \$was_igroup_created,
        );
        $all_objs{'igroup'} = $obj;
    }
    if ($was_igroup_created) {
        $created_objs{'igroup'} = $obj;
    }    

    @luns    = @{ $opts{nacltask_lun} };
    @lun_ids = @{ $opts{nacltask_lun_id} }
        if ( defined $opts{nacltask_lun_id} );

    #Size can be mentioned as a single value or as a list of values.
    if ( scalar @{ $opts{'nacltask_size'} } == 1 ) {
        $size = @{ $opts{'nacltask_size'} }[0];
        $Log->debug("The common size to create the luns is $size");
    } elsif ( scalar @{ $opts{'nacltask_size'} } > 1 ) {
        if ( scalar @{ $opts{'nacltask_size'} } == scalar @luns ) {
            @lun_sizes = @{ $opts{'nacltask_size'} };
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "The no of lun sizes given & luns passed should be equal");
        }
    }

    #Path can be defined as a single value or as a list of values.
    if ( scalar @{ $opts{nacltask_path} } == 1 ) {
        $path = @{ $opts{nacltask_path} }[0];
        $Log->debug("The common path on which the luns are created : $path");
    } elsif ( scalar @{ $opts{'nacltask_path'} } > 1 ) {
        if ( scalar @{ $opts{'nacltask_path'} } == scalar @luns ) {
            @lun_paths = @{ $opts{nacltask_path} };
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "The no of lun-paths given & luns passed should be equal");
        }
    }

    if ($path) {
        foreach my $lun (@luns) {
            $lun = $path . "/" . $lun;
        }
    } else {
        if ( scalar @luns == scalar @lun_paths ) {
            for ( my $i = 0; $i < scalar @luns; $i++ ) {
                $luns[$i] = $lun_paths[$i] . "/" . $luns[$i];
            }
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "The no of luns & lunpaths passed are not equal");
        }
    }

    #Lun-ids, if passed, should be equal to the no.of luns passed.
    if ( ( scalar @lun_ids ) && ( scalar @lun_ids != scalar @luns ) ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw( "Lun-ids if passed should be equal to the"
                . "no.of luns passed else no need to pass, as the filer"
                . " would select a default" );
    }

    my %lun_api_opts = NACL::C::Component::ONTAP->get_api_options(
        apiset  => $apiset,
        api     => 'lun_create',
        options => \%opts,
    );

    foreach my $index ( 0 .. $#luns ) {

        my %lun_opts;
        
        $lun_opts{ostype} = $ostype;
        if ($size) {
            $lun_opts{size} = $size;
        } else {
            $lun_opts{size} = $lun_sizes[$index];
        }
        $lun_opts{path} = $luns[$index];

       #Lun creation
        my $lun = NACL::STask::Lun->create(
            command_interface => $command_interface,
            %lun_api_opts,
            %lun_opts,
            if_exists    => $if_lun_exists,      # default 'die'
            _was_created => \$was_lun_created,
            %common_opts
        );

        my %map_opts;
        $map_opts{igroup} = $igroup;
        if (ref $igroup) { $map_opts{igroup} = $igroup->igroup();  }
        if ( scalar @lun_ids ) {
            $map_opts{'lun-id'} = $lun_ids[$index];
        }

        #Lun map
        $lun->map(
            nacltask_if_mapped_igroup => $if_lun_mapped_igroup,
            allow_extra               => 1,
            nacltask_verify           => 1,
            %map_opts,
            %common_opts
        );

        push( @_all_lun_objects, $lun );
        if ($was_lun_created) {
            push( @_created_lun_objects, $lun );
        }

    }

    $all_objs{'lun'} = \@_all_lun_objects if ( scalar @_all_lun_objects );
    $created_objs{'lun'} = \@_created_lun_objects
        if ( scalar @_created_lun_objects );

    $self->all_objects(%all_objs);
    $self->task_created_objects(%created_objs);

    $Log->exit() if $may_exit;
    return $self;
} ## end sub _setup_7mode

##########################################################
# Private subroutine called by _setup_cmode only
# The routine handles the exception "duplicate ip address"
# and others when creating datalif 
##########################################################
sub _catch_dup_ip_handler {
    $Log->enter() if $may_enter;
    my %opts = @_;
    my $ex   = delete $opts{exception};

    # Argument that controls whether to warn or throw exception
    # when an exception "duplicate ip address" or other is
    # hit while creation.
    # This argument is passed by _setup_cmode method.
    my $warn_or_die = delete $opts{_warn_or_die};
    my $existing_lif_obj;

    $Log->comment( "This is the exception text " . $ex->text() );
    if ( $ex->text() =~ /Duplicate ip address/i ) {
        $existing_lif_obj = NACL::STask::NetworkInterface->find(
            command_interface => $opts{'command_interface'},
            filter            => {
                vserver => $opts{'vserver'},
                address => $opts{lif_opts}->{address},
            },
            allow_empty => 1,
        );
        if ($existing_lif_obj) {
            $opts{lif_opts}->{'lif'} = $existing_lif_obj->lif();
            $Log->exit() if $may_exit;
            return $existing_lif_obj;

        } else {
            unless ( $warn_or_die eq "warn" ) {

                # $warn_or_die eq "die"
                # Control comes here when nacltask_if_lif_exists ne 'reuse'
                # And error "duplicate ip address" was encountered
                # while creating the data lif
                $Log->exit() if $may_exit;
                $ex->throw();
            }

           # $warn_or_die eq "warn".
           # Control will come here when nacltask_if_lif_exists eq 'reuse';
           # But lif is not existing; The lib. goes ahead and creates the same
           # and the error "duplicate ip address" is  encountered
            $Log->warn(
                "While creating the data lif encountered the following error \n"
                    . $ex->text()  );
            $Log->comment(
                      "This means there is no Lif in the vserver "
                    . $opts{ 'vserver' }
                    . " which use the IP Address  "
                    . $opts{lif_opts}->{address} );
             } 
       }  else {
           #If there are issues in creation other than duplicate ip
           #Instead of silently exiting log a warning
           #Throwing an exception here will cause backward incompatibility
           $Log->warn("While creating the data lif encountered the following
                    error \n "
                   . $ex->text()  
            );
        }
        $Log->exit() if $may_exit;
        return undef ; 
    } ## end sub _catch_dup_ip_handler

    1;
