# $Id: //depot/prod/test/main/lib/NACL/MTask/MBUSTest/Sender.pm#1 $
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MBUSTest Task Module
## @author clausner@netapp.com
## @status shared
## @pod here

package NACL::MTask::MBUSTest::Sender;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate validate_with BOOLEAN SCALAR HASHREF OBJECT SCALARREF);
use NATE::Exceptions::Argument qw(:try);
use NATE::BaseException;
use NACL::APISet;

use NACL::ComponentUtils qw(Dumper);

use Time::ParseDate qw(parsedate);

#use NACL::MTask::MBUSTest::%_output;

=head1 NAME

NACL::MTask::MBUSTest::Sender

=head1 DESCRIPTION

C<NACL::MTask::MBUSTest::Sender> is derived class of
L<NACL::MTask::MBUSTest|lib-NACL-MTask-MBUSTest-pm>. It provides a 
well-defined method to return an mbustest object that can be
used to send messages.

MRT stands for Message Router. This process controls where 
messages can go.

MBUSTest is the test client to transmit messages and receive
messages through the MRT process.

=head1 METHODS

=cut

use base qw (NACL::MTask::MBUSTest);
use Class::MethodMaker [
    scalar => 'node',
    scalar => 'message_style',
    scalar => 'duration',
    scalar => 'application_id',
    scalar => 'class_id',
    scalar => 'message_type',
    scalar => 'destination_node',
    scalar => 'initial_delay',
    scalar => 'message_count',
    scalar => 'message_delay',
];


=head2 new

    (Class method)
      $mbustest = NACL::MTask::MBUSTest::Sender->new(
                node                => "<node name>",   # required
                message_style       => 'directed',      # default is broadcast
                duration            => 10,              # optional, defaults to 10
                application_id      => 1,               # optional
                class_id            => 2,               # optional
                message_type        => 5,               # optional, defaults to 5
                destination_node    => "<node name>",   # optional
                initial_delay       => 10,              # optional, defaults to 3
                message_count       => 20,              # optional, defaults to 10
                message_delay       => 1000,            # optional
          );

    
    Creates a short lived task that runs on Freebsd that will send a  
    series of messages. The parameters will determine what type of method
    it will be sent by.

=over

=item Options

=over

=item C<< node  => $node >>
    (Required) C<NACL::C::Node> that will be used to run the command from.

=item C<< message_style => 'directed' >>

    (Optional) String that determines what type of message to
    send. Can be either broadcast, directed, or control. Defaults
    to broadcast.

=item C<< duration    => 10 >>

    (Optional) Integer value that will be used to determine the task life duration
    in seconds.

=item C<< application_id   => 1 >>

    (Optional) Integer value that is used in conjunction with directed
    message type to send directed messages, also requires the node parameter.
    Each mbustest client, aka Sender or Receiver, is given this id by mrt. When
    starting the mbustest client it will be given an application_id and the next
    on that node will be incremented by 1.

=item C<< class_id  => 2 >>

    (Optional) Integer value that allows the broadcast to this type
    of process. The default with starting the mbustest client is 2. This
    can be set in the Receiver.

    This is meant to simulate clients that may want to be grouped together.
    For instance mgwd may have class of 5 and another new client may have the
    type of 5. Another client could send a broadcast message to type 5 and only
    those two clients will receive messages.

=item C<< message_type   => 5 >>

    (Optional) Integer value that will be sent for that message type.

    Right now there are no messages besides 5, more will be added into the
    code at some point later. This is meant to test out different types of
    messages that a Receiver may listen to (May want to send a message type
    that a Receiver isn't setup to listen to). Default set by the executable
    is 5.

=item C<< destination_node    => "<node name>" >>

    (Optional) String value that will be passed to the command line for the
    test client to send messages to.

=item C<< initial_delay => 10 >> 

    (Optional) Integer that determines how long to wait(in seconds) before attempting
    to send messages. The default is set to 3.

=item C<< message_count   => 20 >>

    (Optional) Integer value that determines how many messages to send. This
    value should be included because it will flood the mrt task with messages if
    it's not supplied. Defaults to 10 to prevent message flooding.

=item C<< message_delay   => 1000 >>

    (Optional) Integer value that will have the test client sleep in
    between sending messages. Value is in ms.

=back

=back

=cut

sub new {
    $Log->enter() if $may_enter;
    my $self  = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            node              => { isa  => 'NACL::C::Node' },
            message_style     => { type => SCALAR, default => "broadcast" },
            duration          => { type => SCALAR, default => 10 },
            application_id    => { type => SCALAR, optional => 1 },
            class_id          => { type => SCALAR, optional => 1 },
            message_type      => { type => SCALAR, optional => 1 },
            destination_node  => { type => SCALAR, optional => 1 },
            initial_delay     => { type => SCALAR, default => 3 },
            message_count     => { type => SCALAR, default => 10 },
            message_delay     => { type => SCALAR, optional => 1 },
        },
    );

    if($opts{node}->mode() ne 'CMode') {
       NACL::APISet::Exceptions::InvalidParamValueException->throw(
                text=>'Filer isn\'t CMode',
            );
    }

    $Log->debug(
        "opts to NACL::MTask::MBUSTest::Sender->new " . Dumper(%opts) );

    ## Bless the package instance
    $self = bless {}, $self;
    $self->node($opts{node});
    $self->message_style($opts{message_style});
    $self->initial_delay($opts{initial_delay});
    $self->message_count($opts{message_count});
    $self->duration($opts{duration});

    if(defined($opts{application_id})) {
        $self->application_id($opts{application_id});
    }

    if(defined($opts{class_id})) {
        $self->class_id($opts{class_id});
    }

    if(defined($opts{message_type})) {
        $self->message_type($opts{message_type});
    }
    
    if(defined($opts{destination_node})) {
        $self->destination_node($opts{destination_node});
    }

    return $self;
} ## end sub new

=head2 run 

    (Instance method)
      $mbustest->run(
                message_style   => 'directed',      # optional
                duration        => 10,              # optional
                application_id  => 1,               # optional
                class_id        => 2,               # optional
                message_type    => 5,               # optional
                destination_node   => "<node name>",   # optional
                initial_delay   => 10,              # optional
                message_count   => 20,              # optional
                message_delay   => 1000,            # optional
          );

    
    Creates a short lived task that runs on Freebsd that will send a  
    series of messages. The parameters will determine what type of method
    it will be sent by. All values can override the values set by the
    constructor. This will not set the values of the object to what is
    in place just to temporarily override.

=over

=item Options

=over

=item C<< message_style => 'directed' >>

    (Optional) String value that determines what type of message to
    send. Can be either broadcast, directed, or control. Can override the
    values set by the constructor.

=item C<< duration    => 10 >>

    (Optional) Integer value that will be used to determine the task duration
    in seconds.
    Can override the values set by the constructor.

=item C<< application_id   => 1 >>

    (Optional) Integer value that is used in conjunction with directed
    message type to send directed messages, also requires the node parameter.
    Can override the values set by the constructor.

    Each mbustest client, aka Sender or Receiver, is given this id by mrt. 
    When starting the mbustest client it will be given an application_id and 
    the next on that node will be incremented by 1.

=item C<< class_id  => 2 >>

    (Optional) Integer value that allows the broadcast to this type
    of process. Can override the values set by the constructor.

    This is meant to simulate clients that may want to be grouped together.
    For instance mgwd may have class of 5 and another new client maybe the
    same type so messages can be sent to notify that type of task.

=item C<< message_type   => 5 >>

    (Optional) Integer value that will be sent for that message type.
    Right now there are no messages besides 5, more will be added into the
    code at some point later. This is meant to test out different types of
    messages that a Receiver may listen to (May want to send a message type
    that a Receiver isn't setup to listen to).

    Can override the values set by the constructor.

=item C<< destination_node    => "<node name>" >>

    (Optional) String value that will be passed to the command line for the
    test client to send messages to. Can override the values set by the 
    constructor.

=item C<< initial_delay   => 10 >>

    (Optional) Integer that determines how long to wait(in seconds) before attempting
    to send messages. Can override the values set by the constructor.

=item C<< message_count   => 20 >>

    (Optional) Integer value that determines how many messages to send.
    Can override the values set by the constructor.

=item C<< message_delay   => 1000 >>

    (Optional) Integer value that will have the test client sleep in
    between sending messages. Value is in ms. Can override the values
    set by the constructor.

=back

=back

=cut

sub run {
    $Log->enter() if $may_enter;
    my $self  = shift;

    $Log->debug(
        "Package information of NACL::MTask::MBUSTest::Sender " . Dumper($self) );

    my %opts = validate_with(
        params => \@_,
        spec   => {
            message_style     => { type => SCALAR, default => $self->message_style() },
            duration          => { type => SCALAR, default => $self->duration() },
            application_id    => { type => SCALAR, default => $self->application_id()  },
            class_id          => { type => SCALAR, default => $self->class_id()  },
            message_type      => { type => SCALAR, default => $self->message_type()  },
            destination_node  => { type => SCALAR, default => $self->destination_node() },
            initial_delay     => { type => SCALAR, default => $self->initial_delay() },
            message_count     => { type => SCALAR, default => $self->message_count() },
            message_delay     => { type => SCALAR, default => $self->message_delay() },
        },
    );

    $Log->debug(
        "opts to NACL::MTask::MBUSTest::Sender::run " . Dumper(%opts) );

    my %params_to_use = (
            log   =>   "verbose",
        );

        $params_to_use{message} = $opts{message_style};
        $params_to_use{delay} = $opts{initial_delay};
        $params_to_use{count} = $opts{message_count};
        $params_to_use{time} = $opts{duration};

    if(defined($opts{message_type})) {
        $params_to_use{msgid} = $opts{message_type};
    }

    if(defined($opts{application_id})) {
        $params_to_use{appid} = $opts{application_id};
    }

    if(defined($opts{class_id})) {
        $params_to_use{procid} = $opts{class_id};
    }

    if(defined($opts{destination_node})) {
        $params_to_use{node} = $opts{destination_node};
    }

    if(defined($opts{message_delay})) {
        $params_to_use{sleep} = $opts{message_delay};
    }

    my $apiset = $self->node()->apiset(
                category => "Node",
                interface => "CLI",
                set => "Systemshell",
                shell_type => "CMode" );    

    # Declare local variable
    my $output;
    my $output_ref;
    my @output_array;

    # Run the command
    $output = $apiset->mbustest( %params_to_use );
    $output_ref = $output->{parsed_output};

    @output_array = @$output_ref;
    my %actual_hash;
    foreach my $hash_ref ( @output_array ) {
        %actual_hash = %$hash_ref;
    }

    $self->output(\%actual_hash);

    $Log->exit() if $may_exit;
    return;
} ## end sub run

=head2 get_messages

    (Instance method)
      @messages_sent = $mbus_object->get_messages();

        Method that will retrieve all the messages that are sent.

=over

=over

=back

=back

=cut

sub get_messages {
    my $self  = shift;

    my %message_hash = ();
    my $output_hash = $self->output();
    my %output_hash = %$output_hash;

    my $total_sent = $output_hash{send_max};
    if( defined($total_sent) ) {
        for( my $i = 0; $i < $total_sent; $i++) {
            my $current_message = "send$i";
            $message_hash{$current_message} = $output_hash{$current_message};
        }
        $message_hash{send_max} = $total_sent;
    }

    return (\%message_hash);

} ## end sub get_messages

=head2 cleanup_messages

    (Parent method)
      $mbus_object->cleanup_messages();

        Method that will clean up the output containers that are
        holding the information of what messages were sent.

=over

=over

=back

=back

=cut

sub cleanup_messages {

    my $self  = shift;
    my %empty_hash = ();
    $self->output(\%empty_hash);

} ## end sub cleanup_messages

1;
