#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroCluster NACL::MTask::MCC::Asup Module
## @author dl-mcc-dpg-qa@netapp.com
## @status shared
## @pod here

package NACL::MTask::MCC::Asup;

use Moose;
use MooseX::NonMoose;

extends 'NACL::MTask::MTask';

use NACL::CS::SystemNodeAutosupport;
use NACL::CS::SystemNodeAutosupportHistory;
use NACL::C::SystemNode;

use NATE::BaseException
    qw/try finally except otherwise/;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# Substitute for NATE::Error::with, which would conflict with Moose
sub ewith (&;$) {
    @_
}

use Params::Validate
    qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF);

=head1 NAME

NACL::MTask::MCC::Asup

=head1 SYNOPSIS

  use NACL::MTask::MCC::Asup;

  my $asup = NACL::MTask::MCC::Asup->new(
     nodes => \@nodes
  );

  # Invokes all the asups
  $asup->invoke_all();

  # Check to see if all messages were sent
  $asup->validate_sent() or print "Not all were sent!";

=head1 DESCRIPTION

This task is used to send and verify the sending of ASUP messages.

=head1 METHODS

=head2 new()

  my $asup = NACL::MTask::MCC::Asup->new(
     nodes => \@nodes,
     delivery_timeout => 90,
  );

Constructor.

=over

=item Arguments

=over

=item C<< nodes >>

(Required.) Arrayref of NACL::C::SystemNode objects that we will run ASUP on.

=item C<< delivery_timeout >>

Total number of seconds to wait for ASUPs to be delivered.

=back

=back

=cut

has 'nodes' => (
    is => 'ro',
    isa => 'ArrayRef[NACL::C::SystemNode]',
    traits => [ 'Array' ],
    handles => {
        all_nodes => 'elements'
    },
    required => 1
);

has 'delivery_timeout' => (
    is => 'rw',
    isa => 'Int',
    default => 90
);

has '_asups_to_validate' => (
    is => 'ro',
    isa => 'HashRef',
    traits => [ 'Hash' ],
    default => sub { {} },
    handles => {
        '_set_asup_to_validate' => 'set',
        '_get_asup_validation_node' => 'get',
        '_get_asup_subjects' => 'keys',
        '_delete_asup_to_validate' => 'delete',
        '_has_asups_to_validate' => 'count'
    }
);

=head2 invoke_all()

  $asup->invoke_all();

Runs ASUP on all nodes with type 'all'. All exceptions are turned into
warnings.

=over

=item Arguments

=over

=item C<< should_validate >>

If true, records ASUPs invoked for further validation.

=back

=back

=cut

sub invoke_all {
    my $self = shift;

    my %opts = validate_with(
        'params' => \@_,
        'spec'   => {
            'should_validate' => {
                'type'    => SCALAR,
                'regex'   => qr/^[01]$/,
                'default' => '1'
            }
        }
    );


    my $subject = "Gather_diagnostic_invoked_asup_" . time() . "_";

    foreach my $node ($self->all_nodes()) {
        # Skip ASUP trigger if node is not reachable
        next if !$node->command_interface()->apiset()
                     ->get_connection()->is_responsive(timeout => 30);

        try {
            my $invoked_asup = NACL::C::SystemNodeAutosupport->invoke(
                'command_interface' => $node,
                'node'              => $node->node(),
                'type'              => 'all',
                'message'           => $subject . $node->node(),
                'method-timeout'    => 60,
            );

            if ( $opts{'should_validate'} ) {
                my $sub = $subject . $node->node();
                $self->_set_asup_to_validate($sub, $node);
            }

        } catch NATE::BaseException ewith {
            my ($e) = @_;

            $Log->warn(
                'Asup of type all could not be invoked on node "' 
                    . $node->node() . '" due to exception '
                        . "'$e'"
                );
        };

    }

} ## end sub invoke_all

=head2 validate_sent()

  $asup->validate_sent();

Logs whether ASUP messages were sent or not. If needed, waits for C<<delivery_timeout>> seconds to allow
delivery to complete.

=cut

sub validate_sent {
    my $self = shift;

    my $all_ok = 1;

    # Check for invoked asup in history table
    my $end = time() + $self->delivery_timeout();

TIME: while ((time() < $end) && $self->_has_asups_to_validate()) {

    SUBJECT: foreach my $subject ($self->_get_asup_subjects()) {

            my $node = $self->_get_asup_validation_node($subject);

            try {

                my $asup_history
                    = NACL::CS::SystemNodeAutosupportHistory->fetch(
                    command_interface => $node,
                    filter            => {
                        subject     => '*' . $subject . '*',
                        destination => 'http',
                        status      => 'sent-successful'
                    },
                    allow_empty => 1
                );

                # Wait for asup entry to show in the history table
                # if not found
                if (!defined $asup_history) {
                    Tharn::sleep(5);
                    next TIME;
                }

                # Note any sent messages
                if ($asup_history->status() =~ /sent-successful/) {

                    # Delete asup entry from hash if the
                    # asup was sent successfully
                    $self->_delete_asup_to_validate($subject);
                    $Log->comment('ASUP with subject "'
                            . $asup_history->subject . '" '
                            . 'was verified to have been sent successfully');
                }

            } catch NATE::BaseException ewith {
                my ($exception) = @_;

                $all_ok = 0;

                $Log->warn('Fetching autosupport history entry failed with '
                        . "'$exception' on node "
                        . $node->node);

                # Break out to avoid infinite loops
                last TIME;

            };
        }
    }

    # Warn if not all ASUPs sent
    if ($self->_has_asups_to_validate()) {

        $all_ok = 0;

        $Log->warn('All asups invoked during the run did not reach '
                . ' sent-successful state. These asups may get dropped '
                . ' on the way to eProd.');
    }

    return $all_ok;

} ## end sub validate_sent

__PACKAGE__->meta->make_immutable;

no Moose;

1;
