# $Id:$
#
## @description
## Wrapper library for the functionality already exported by NACL libaries
##
## @summary DPSaS Utility library for MCC DPSaS
## @author dl-mcc-dpg-qa@netapp.com
## @status shared
## @pod here

package NACL::MTask::MCC::DPSASUtils;

use strict;
use warnings;
use Data::Dumper;
use Time::Local qw(timegm);
use Tharn qw( subtest);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Carp qw(confess);
use Net::Domain qw(hostname);
use NATE::Time qw(describe_timeout);
use TestEnv::MinWin qw(minwin_compatible);
use Armadillo;

use NACL::C::Node;
use NACL::C::Client;
use NACL::C::StorageDisk;
use NACL::C::RoutingGroups;
use NACL::C::StorageDiskOption;
use NACL::C::NetworkInterface;
use NACL::C::StorageAggregatePlex;
use NACL::C::Volume;
use NACL::C::Vserver;
use NACL::C::VserverExportPolicyRule;
use NACL::C::VolumeSnapshotAutodelete;
use NACL::C::VolumeEfficiencyPolicy;

use NACL::CS::Vserver;
use NACL::CS::Interface;
use NATE::BaseException qw(:try);

use NACL::STask::Spm;
use NACL::STask::Volume;
use NACL::STask::Vserver;
use NACL::STask::Aggregate;
use NACL::STask::VserverNfs;
use NACL::STask::VolumeMove;
use NACL::STask::Client::Mount;
use NACL::STask::StorageFailover;
use NACL::STask::VolumeEfficiency;
use NACL::STask::NetworkInterface;
use NACL::STask::NetworkRoute;
use NACL::STask::NetworkRoutingGroupsRoute;
use NACL::CS::NetworkRoutingGroups;

use NACL::MTask::CIFS;
use NACL::STask::VserverCifsShare;
use NACL::STask::VserverServicesDns;
use NACL::C::VserverCifsSuperuser;

use NACL::MTask::HA;
use NACL::MTask::Data;
use NACL::MTask::Traffic;
use NACL::MTask::Wafliron;
use NACL::MTask::FreeDataPort;
use NACL::MTask::MCC::Metrocluster;
use NACL::GeneralUtils qw(random_name_generator);
use Params::Validate
    qw(validate validate_with ARRAYREF HASHREF SCALARREF SCALAR UNDEF);

#################################################

=head1 NAME

NACL::MTask::MCC::DPSASUtils

=head1 DESCRIPTION

    B<EXPERIMENTAL!>

    Class name/location temporary.

    DPSASUtils is a wrapper library for the functionality already exported by
    NACL to provide minimum lines of code writting in "thpl" scripts which is
    required to call same function in multiple times from different scenarios
    when Metrocluster DPSaS solution base testing.

    # Example
    use NACL::MTask::MCC::DPSASUtils;
    ...
    # Perform test here
    ...
    # End test

=cut

#use Class::MethodMaker [
#    new    => [ 'new' ] ];

=head1 DPSaS Utility Methods

=head2 start_nfs_traffic()

    # Using minimum required params
    my ($mounted_vol_objs, $traffic_objs, $export_policy_rule_obj) =
        NACL::MTask::MCC::DPSASUtils->start_nfs_traffic(
                    vol_objs_ref   => \@volume_objs,
                    clients_ref    => \@clients );

    # NFS traffic with data lif creation
    my ($mounted_vol_objs, $traffic_objs, $export_policy_rule_obj) =
        NACL::MTask::MCC::DPSASUtils->start_nfs_traffic(
                    vol_objs_ref         => \@volume_objs, or [$volume_objs[0]],
                    clients_ref          => \@clients,
                    vserver              => vs0,
                    create_data_lifs     => 1,
                    create_export_policy => 1 );

    # hammer traffic ( for snapmirror source volumes )
    my ($mounted_vol_objs, $traffic_objs, $export_policy_rule_obj) =
        NACL::MTask::MCC::DPSASUtils->start_nfs_traffic(
                        vol_objs_ref            => \@volume_objs,
                        clients_ref             => \@clients,
                        vserver                 => vs0,
                        create_data_lifs        => 0,
                        create_export_policy    => 1,
                        create_routing_groups   => 0,
                        sio_iops                => 100,
                        sio_thread_count        => 2,
                        sio_no_file_lock        => 1,
                        sio_verify              => 1,
                        enable_nfs_v40          => 1,
                        tool                    => 'hammer',
                        fill                    => 0,
                        mode                    => 'creates',
                        mount_points_per_client => 8 );

    Wrapper function to start nfs traffic on given volumes in same vservers
    from multiple clients.

    Input Params:
    vol_objs_ref          => [Array reference of volume objects]
    clients_ref           => [Array reference of nfs client objects]
    vserver               => [Vserver name string ( default = vs0 )]
    create_data_lifs      => [Boolean value to create data lifs( default = 0 )]
    create_export_policy  => [Boolean to create export policy  ( default = 1 )]
    create_routing_groups => [Boolean to create routing groups ( default = 1 )]
    enable_nfs_v40        => [Boolean to enable nfs v4.0 ( default = 0 )]
    tool                  => [string value 'dt|sio|hammer|skip',default = 'sio']
    sio_iops              => [integer, I/O operations per secs ~ optional]
    sio_thread_count      => [integer, num of sio threads ~ ( default = 2 )]
    sio_file_size         => [scalar, 100m, 2g, 1000k ~ default = 2m to 30m ]
    sio_block_size        => [scalar, IO block size ~ default = 4k ]
    sio_verify            => [Boolean to verify written data ~ default => 0 ]
    sio_no_file_lock      => [Boolean to activate No_file_Lock ~ default => 0 ]
    sio_read_percent      => [integer, I/O read percent ~ default => 0 ]
    sio_random_percent    => [integer, I/O random percent ~ default => 100 ]

    mount_points_per_client => [integer number to limit mount_points per client]

    # hammer specific parameters
    fill         => [Boolean value to fill up volume]
    mode         => [Scalar, string value 'mixed' or 'creates' or 'overwrites']

    # DT specific parameters
    dt_limit      => [The number of bytes to transfer (default = '1g')]
    dt_processes  => [DT process count integer value (default = 4)]
    dt_io_type    => [DT io type string random|sequential (default = 'random')]
    dt_enable_no_progress       => [Boolean to enable noprog option (default=0)]
    dt_no_progress_time_in_secs => [DT no progress time in secs (default = 120)]
    dt_no_progress_trigger_time_in_secs =>
                           [DT no_progress_trigger_time in secs (default = 120)]

    Return:
    Array reference of mounted volumes objects
    Array reference of traffic objects
    scalar value of export_policy_rule object

=cut

sub start_nfs_traffic {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            vol_objs_ref            => { type => ARRAYREF },
            clients_ref             => { type => ARRAYREF },
            vserver                 => { type => SCALAR | UNDEF, optional => 1},
            create_data_lifs        => { regex => qr/^[01]$/, default => 0 },
            create_export_policy    => { regex => qr/^[01]$/, default => 1 },
            create_routing_groups   => { regex => qr/^[01]$/, default => 1 },
            enable_nfs_v40          => { regex => qr/^[01]$/, default => 0 },
            sio_iops                => { regex => qr/^\d+$/, default => 0 },
            sio_thread_count        => { regex => qr/^\d+$/, default => 2 },
            sio_file_size           => { type => SCALAR | UNDEF, optional => 1},
            sio_block_size          => { type => SCALAR | UNDEF, optional => 1},
            sio_verify              => { regex => qr/^[01]$/, default => 1 },
            sio_no_file_lock        => { regex => qr/^[01]$/, default => 1 },
            sio_read_percent        => { regex => qr/^\d+$/, default => 50 },
            sio_random_percent      => { regex => qr/^\d+$/, default => 100 },
            mount_points_per_client => { regex => qr/^\d+$/, default => 8 },

            tool       => { regex => '^(dt|sio|hammer|skip)$',default => 'sio'},
            fill       => { regex => qr/^[01]$/, default => 0 },
            mode       => { regex => '^(mixed|creates|overwrites)$',
                                                           default => 'mixed' },

            dt_limit      => { type => SCALAR | UNDEF, optional => 1 },
            dt_processes  => { regex => qr/^\d+$/, default => 2 },
            dt_io_type    => { regex => '^(random|sequential)$',
                                                          default => 'random' },
            dt_enable_no_progress       => { regex => qr/^[01]$/, default => 1},
            dt_no_progress_time_in_secs => { regex => qr/^\d+$/,
                                                          default => 300 },
            dt_no_progress_trigger_time_in_secs => { regex => qr/^\d+$/,
                                                          default => 5 }
        }
    );
    $Log->enter() if $may_enter;

    # Create NFS data lifs on all nodes #####
    $Log->comment('Creating NFS data lifs on all nodes');
    my @all_nodes;
    foreach my $volobj (@{$args{'vol_objs_ref'}}) {
        push @all_nodes, $volobj->command_interface();
    }

    my %node_hash = ();
    my @unique_nodes = grep { ! $node_hash{$_}++ } @all_nodes;

    my $vserver_name = defined($args{'vserver'}) ? $args{'vserver'} :
                                    ${$args{'vol_objs_ref'}}[0]->vserver();

    if ($args{'create_data_lifs'}) {
        foreach my $node_obj (@unique_nodes) {
            $pkg->create_nfs_cifs_data_lifs(
                        node_obj => $node_obj,
                        vserver  => $vserver_name );
        }
    }
    # Routing group must exist to create vserver route
    if ($args{'create_routing_groups'}) {
        $pkg->create_routing_group_route(
                node_obj => ${$args{'vol_objs_ref'}}[0]->command_interface(),
                vserver  => $vserver_name );
    }
    # Setup NFS and export policy rule
    my (@traffic_objs, @mounted_vol_objs, $export_policy_rule);
    if ($args{'create_export_policy'}) {
        $Log->comment('Setup NFS and Export policy rule');
        $export_policy_rule = $pkg->create_nfs_and_export_policy_rule(
                node_obj => ${$args{'vol_objs_ref'}}[0]->command_interface(),
                vserver        => $vserver_name,
                enable_nfs_v40 => $args{'enable_nfs_v40'} );
    }

    # Wait for routing group started on data lif
    if ($args{'create_export_policy'} || $args{'create_routing_groups'}) {
        $Log->comment('Wait for routing-groups route & export-policy rule up');
        Tharn::snooze(45);
    }

    # Use client reachable data lifs only for traffic
    $Log->comment('Get data lifs and client objects');
    my $data_lifs_ref = $pkg->get_reachable_data_lifs(
        node_obj        => ${$args{'vol_objs_ref'}}[0]->command_interface(),
        vserver_name    => ${$args{'vol_objs_ref'}}[0]->vserver(),
        client_obj      => ${$args{'clients_ref'}}[0],
       'address-family' => 'ipv4',
       'data-protocol'  => ['nfs'] );

    my @data_lifs = @{ $data_lifs_ref };

    # Mount volumes and start NFS traffic
    my $num = 0;
    my $nfs_version = $args{'enable_nfs_v40'} ? 4 : 3;
    foreach my $vol_obj (@{$args{'vol_objs_ref'}}) {
        $num++;

        # Limit mount points per client ( default = 10 )
        my $limit = $args{'mount_points_per_client'};
        if ($num > (scalar(@{$args{'clients_ref'}})) * $limit) { last; }

        # Get a client from a list of available clients list
        my $client_obj = shift(@{$args{'clients_ref'}});
        push @{$args{'clients_ref'}}, $client_obj;

        # Get round-robin data lif from a list of reachable data lifs
        my $lif_obj = shift(@data_lifs);
        push @data_lifs, $lif_obj;

        # Mount volume to Linux client under test
        $Log->comment('Mounting volume on NFS client');
        $Log->comment("$num\)Client name :  " . $client_obj->name());
        $Log->comment('Volume name   :  ' . $vol_obj->volume());
        $Log->comment('NFS data LIF  :  ' . $lif_obj->address());
        $Log->comment('Vserver name  :  ' . $vol_obj->vserver());
        $Log->comment('Node name     :  ' .
                                        $vol_obj->command_interface()->name());

        # Mount section modified by zaw 3/13/2013
        my $mount_obj = NACL::STask::Client::Mount->create(
                command_interface        => $client_obj,
                volume                   => $vol_obj,
                server_command_interface => $vol_obj->command_interface(),
                server                   => $lif_obj->{'address'},
                nacltask_if_exists       => 'purge',
                vers                     => $nfs_version,
                retries                  => 3 );

        # Update client and mount information to volume object
        $vol_obj->{'client'} = $client_obj;
        $vol_obj->{'mount'} = $mount_obj;
        push @mounted_vol_objs, $vol_obj;

        # Print out client and Data LIF information
        $Log->comment("$num\)Mount point  :  " . $mount_obj->mount_point() .
                "\nNFS client mount successfully completed" . '  ~ ( ' .
                   $client_obj->name() . ' )' );

        # Start nfs (SIO) traffic
        if ($args{'tool'} =~ /sio/i) {
            $Log->comment('NFS traffic tool : SIO');

            # Get sio file size
            my $file_size = $args{'sio_file_size'};
            if (!defined($file_size)) {
                my $maximum_size = 30;
                my $minimum_size = 2;
                my $size = int(rand($maximum_size)) + $minimum_size;
                $file_size = $size . 'm';
            }

            # Create sio baseline file on client's mount point
            my $path = $mount_obj->mount_point();
            my $sio_file = 'file_' . random_name_generator( size => 1 );

            # Added the sio baseline file creation block
            $Log->comment('Creating a file ' . "$path/$sio_file" . '_1' .
                                            ' ~ ( ' . $file_size . ' )');
            $client_obj->apiset(interface => 'CLI')->execute_command(
                   'privilege-level'    => 'root',
                   'connectrec-timeout' => 3600,
                    command => '/usr/software/test/bin/fops -o c -s ' .
                                $file_size . ' -n 1 -p ' . "$sio_file $path" );

            $sio_file .= '_1';
            my $file_name = "$path/$sio_file";

            $client_obj->apiset(interface => 'CLI')->execute_command(
                    command             => "chmod 777 $file_name",
                   'privilege-level'    => 'root',
                   'connectrec-timeout' => 300 );
            # End file creation block.

            # Get size of I/O's. Example: 2k, 4k, 1m>
            # 'random' keyword makes the I/O size random 512bytes to 256k.
            my $block_size = !defined($args{'sio_block_size'}) ? '4k' :
                                                        $args{'sio_block_size'};

            # Get sio percentage of accesses that are reads
            my $read_percent = $args{'sio_read_percent'};
            if ($read_percent > 100) { $read_percent = 100; }

            # Get percentage of acceses that are random. Range [0,100].
            # Sequential accesses = 0, Random = 100.
            my $random_percent = $args{'sio_random_percent'};
            if ($random_percent > 100) { $random_percent = 100; }

            # sio verify = 1 to Read back written data and verify content.
            my $verify_content = $args{'sio_verify'} ? 1 : 0;

            # Get sio file lock option
            my $file_lock = $args{'sio_no_file_lock'} ? 1 : 0;

            # Get a number of I/O operations per second for sio
            my $sio_iops;
            if ($args{'sio_iops'} > 0) {
               $sio_iops = q{-}.$args{'sio_iops'};
            } else {
                $sio_iops = undef;
            }

            # Initialize sio default arguments
            my %sio_default_args = (
                                tool           => 'sio',
                                rand           => $random_percent,
                                block_size     => $block_size,
                                start_byte     => 0,
                                num_of_threads => $args{'sio_thread_count'},);

            # Create sio traffic object and start nfs traffic
            my $sio_traffic_obj = NACL::MTask::Traffic->start(
                                command_interface => $client_obj,
                                paths             => $file_name,
                                file_size         => $file_size,
                                read              => $read_percent,
                                iops              => $sio_iops,
                                noflock           => $file_lock,
                                verify            => $verify_content,
                                %sio_default_args);
            push @traffic_objs, $sio_traffic_obj;

        # Start nfs (HAMMER) traffic
        } elsif ($args{'tool'} =~ /hammer/i) {
            $Log->comment('NFS traffic tool : Hammer');
            try {
                my $hammer_traffic_obj = NACL::MTask::Traffic->start(
                                command_interface => $client_obj,
                                paths             => $mount_obj->mount_point(),
                                on_failure        => 'die',
                                tool              => 'hammer',
                                nofilercore       => 1,
                                fill              => $args{'fill'},
                                mode              => $args{'mode'} );
                push @traffic_objs, $hammer_traffic_obj;
            } catch NATE::BaseException with {
                 $Log->warn('NFS traffic hammer ERROR on ' .
                            $client_obj->name() . '_' . $vol_obj->volume());
            };

        # Start nfs (DT) traffic
        } elsif ($args{'tool'} =~ /dt/i) {
            $Log->comment('NFS traffic tool : DT');

            # Create dt baseline file on client's mount point
            my $dt_path = $mount_obj->mount_point();
            my $dt_file = "$dt_path/dt_file_" . random_name_generator();
            $client_obj->apiset(interface => 'CLI')->execute_command(
                                command => 'echo dt_file > ' . $dt_file);

            my $no_progt  = $args{'dt_no_progress_time_in_secs'} . 's';
            my $no_progtt = $args{'dt_no_progress_trigger_time_in_secs'} . 's';

            my %dt_noprog_options = ();
            if ($args{'dt_enable_no_progress'}) {
                %dt_noprog_options = (
                    enable     => 'noprog',
                    oncerr     => 'abort',
                    errors     => '1',
                    noprogt    => $no_progt,
                    noprogtt   => $no_progtt,
                    keepalivet => '300s',
                    keepalive  => '"300SecKeepAliveMsg#%h@%d#TotalTestTime:%T"',
                    alarm      => '5s',
                    history    => '3',
                    hdsize     => '128',
                    trigger    => 'cmd:' .
                             '/x/eng/localtest/noarch/bin/dt_noprog_script.ksh',
                );
            }

            # Get DT data transfer size limit
            my $dt_limit = $args{'dt_limit'};
            if (!defined($dt_limit)) {
                my $size_bytes = int( $vol_obj->state->size() / 2 ) ;
                my $size_mb    = int( $size_bytes / ( 1024 * 1024 ) );
                my $size_gb    = int( $size_bytes / ( 1024 * 1024 * 1024 ) );

                if ( $size_gb >= 1 ) {
                    $dt_limit = $size_gb . 'g';
                } elsif ( $size_mb >= 1 ) {
                    $dt_limit = $size_mb . 'm';
                } else {
                    $dt_limit = $size_bytes;
                }
            }

            # Create dt traffic object and start nfs traffic
            my $dt_traffic_obj = NACL::MTask::Traffic->start(
                command_interface => $client_obj,
                paths             => $dt_file,
                tool              => 'dt',
                iotype            => $args{'dt_io_type'},
                limit             => $dt_limit,
                oflags            => 'trunc',
                procs             => $args{'dt_processes'},
                bs                => '4k',
                incr              => 'var',
                min               => '2k',
                max               => '256k',
                pattern           => 'iot',
                runtime           => '-1',
                dispose           => 'keeponerror',
                prefix            => '"%d@%h"',
                disable           => 'pstats',
                enable            => 'compare',
                enable            => 'lbdata',
                enable            => 'debug',
                enable            => 'syslog',
                %dt_noprog_options );
            push @traffic_objs, $dt_traffic_obj;
        # Skip the IO and keep client pounts objects
        } elsif ($args{'tool'} =~ /skip/i) {
            $Log->comment('Skipped IO and keep client mounts');
        }
    }
    $Log->exit() if $may_exit;
    return (\@mounted_vol_objs, \@traffic_objs, $export_policy_rule);
}


=head2 dpsas_nfs_traffic()

    # Under construction
    # Using minimum required params
    my ($mounted_vol_objs, $traffic_objs, $export_policy_rule_objs) =
        NACL::MTask::MCC::DPSASUtils->dpsas_nfs_traffic(
                    vol_objs_ref   => \@volume_objs,
                    clients_ref    => \@clients );

    # NFS traffic with data lif creation
    my ($mounted_vol_objs, $traffic_objs, $export_policy_rule_objs) =
        NACL::MTask::MCC::DPSASUtils->dpsas_nfs_traffic(
                    vol_objs_ref         => \@volume_objs, or [$volume_objs[0]],
                    clients_ref          => \@clients,
                    sio_iops             => 30,
                    sio_thread_count     => 2,
                    create_data_lifs     => 1,
                    create_export_policy => 1 );

    # hammer traffic ( for snapmirror source volumes )
    my ($mounted_vol_objs, $traffic_objs, $export_policy_rule_objs) =
        NACL::MTask::MCC::DPSASUtils->dpsas_nfs_traffic(
                        vol_objs_ref            => \@volume_objs,
                        clients_ref             => \@clients,
                        create_data_lifs        => 0,
                        create_export_policy    => 1,
                        create_routing_groups   => 0,
                        enable_nfs_v40          => 1,
                        tool                    => 'hammer',
                        fill                    => 0,
                        mode                    => 'creates',
                        mount_points_per_client => 8 );

    Wrapper function to start nfs traffic on given volumes in same vservers
    from multiple clients.

    Input Params:
    vol_objs_ref          => [Array reference of volume objects]
    clients_ref           => [Array reference of nfs client objects]
    create_data_lifs      => [Boolean value to create data lifs( default = 0 )]
    create_export_policy  => [Boolean to create export policy  ( default = 1 )]
    create_routing_groups => [Boolean to create routing groups ( default = 1 )]
    enable_nfs_v40        => [Boolean to enable nfs v4.0 ( default = 0 )]
    tool                  => [string  'dt|sio|hammer|skip' ~ default = 'sio']
    sio_iops              => [integer, I/O operations per secs ~ optional]
    sio_thread_count      => [integer, num of sio threads ~ ( default = 1 )]
    mount_points_per_client => [integer number to limit mount_points per client]

    # hammer specific parameters
    fill         => [Boolean value to fill up volume]
    mode         => [Scalar, string value 'mixed' or 'creates' or 'overwrites']

    # DT specific parameters
    dt_limit      => [The number of bytes to transfer (default = '1g')]
    dt_processes  => [DT process count integer value (default = 4)]
    dt_io_type    => [DT io type string random|sequential (default = 'random')]
    dt_enable_no_progress       => [Boolean to enable noprog option (default=1)]
    dt_no_progress_time_in_secs => [DT no progress time in secs (default = 120)]
    dt_no_progress_trigger_time_in_secs =>
                           [DT no_progress_trigger_time in secs (default = 120)]

    Return:
    Array reference of mounted volumes objects
    Array reference of traffic objects
    Array reference of export_policy_rule objects

=cut

sub dpsas_nfs_traffic {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            vol_objs_ref            => { type => ARRAYREF },
            clients_ref             => { type => ARRAYREF },
            create_data_lifs        => { regex => qr/^[01]$/, default => 0 },
            create_export_policy    => { regex => qr/^[01]$/, default => 1 },
            create_routing_groups   => { regex => qr/^[01]$/, default => 1 },
            enable_nfs_v40          => { regex => qr/^[01]$/, default => 0 },
            mount_points_per_client => { regex => qr/^\d+$/, default => 8 },
            tool                    => { regex => '^(dt|sio|hammer|skip)$',
                                                            default => 'sio' },
            sio_iops         => { regex => qr/^\d+$/, default => 0 },
            sio_thread_count => { regex => qr/^\d+$/, default => 1 },
            fill             => { regex => qr/^[01]$/, default => 0 },
            mode             => { regex => '^(mixed|creates|overwrites)$',
                                                           default => 'mixed' },

            dt_limit         => { type => SCALAR | UNDEF, optional => 1 },
            dt_processes     => { regex => qr/^\d+$/, default => 2 },
            dt_io_type       => { regex => '^(random|sequential)$',
                                                          default => 'random' },
            dt_enable_no_progress       => { regex => qr/^[01]$/, default => 1},
            dt_no_progress_time_in_secs => { regex => qr/^\d+$/,
                                                          default => 300},
            dt_no_progress_trigger_time_in_secs => { regex => qr/^\d+$/,
                                                          default => 5 }
        }
    );
    $Log->enter() if $may_enter;
    # Get all vserver names associated with volumes
    my @dummy_vs_names;
    foreach my $volume (@{$args{'vol_objs_ref'}}) {
        push @dummy_vs_names, $volume->vserver();
    }
    my %vserver_hash = ();
    my @unique_vserver_names = grep { ! $vserver_hash{$_}++ } @dummy_vs_names;

    # NFS traffic on multiple vservers
    my @test_vserver_objs;
    foreach my $test_vs_name (@unique_vserver_names) {
        my $unique_vserver_obj = NACL::CS::Vserver->fetch(
            command_interface =>
                     ${$args{'vol_objs_ref'}}[0]->command_interface(),
           filter  => { subtype => 'sync-source',
                        vserver => $test_vs_name } );
        push @test_vserver_objs, $unique_vserver_obj;
    }

    # Log NFS preparation logs to NFS_preparation folder
    my $info_log = NATE::Result::Info->new('Completed');
    my $nfs_preparation_log = Tharn::param('RUNID') . '/nfs_preparation.log';
    $Log->redirect( path => $nfs_preparation_log );
    my (@traffic_objs, @mounted_vol_objs, @export_policy_rules);
    foreach my $test_vserver (@test_vserver_objs) {
        # Create Data LIFs on all vservers
        if ($args{'create_data_lifs'}) {
            $pkg->create_nfs_cifs_data_lifs(
                    node_obj => $test_vserver->command_interface(),
                    vserver  => $test_vserver->vserver() );
        }

        # Routing group must exist to create vserver route
        if ($args{'create_routing_groups'}) {
            $pkg->create_routing_group_route(
                    node_obj => $test_vserver->command_interface(),
                    vserver  => $test_vserver->vserver() );
        }

        # Setup NFS and export policy rule
        if ($args{'create_export_policy'}) {
            $Log->comment('Setup NFS and Export policy rule');
            my $export_policy = $pkg->create_nfs_and_export_policy_rule(
                    node_obj       => $test_vserver->command_interface(),
                    vserver        => $test_vserver->vserver(),
                    enable_nfs_v40 => $args{'enable_nfs_v40'} );
            push @export_policy_rules, $export_policy;
        }
    }
    $Log->log($info_log);
    $Log->reset_path();

    # Wait for routing group started on data lif
    if ($args{'create_export_policy'} || $args{'create_routing_groups'}) {
        $Log->comment('Wait for routing-groups route & export-policy rule up');
        Tharn::snooze(45);
    }

    my $num = 0;
    my $nfs_version = $args{'enable_nfs_v40'} ? 4 : 3;
    foreach my $test_vserver_obj (@test_vserver_objs) {
        # Get NFS Data LIF associated with current vserver
        $Log->comment('Get data lifs and client objects');
        my $nfs_lifs_ref = $pkg->get_nfs_cifs_data_lifs(
                node_obj      => $test_vserver_obj->command_interface(),
                vserver       => $test_vserver_obj->vserver(),
                data_protocol => 'nfs' );

        # Filer nfs volumes only from current vserver
        my @dummy_volumes = NACL::C::Volume->find(
                command_interface => $test_vserver_obj->command_interface(),
                filter  => { vsroot  => 'false',
                             vserver => $test_vserver_obj->vserver()});

        my @nfs_volumes = ();
        foreach my $volume_obj (@{$args{'vol_objs_ref'}}) {
            foreach my $dummy_vol (@dummy_volumes) {
                if ($volume_obj->volume() eq $dummy_vol->volume()) {
                    push @nfs_volumes, $volume_obj;
                }
            }
        }

        # Log NFS mount logs to NFS_mount folder
        my $nfs_mount_log = Tharn::param('RUNID') . '/nfs_mount.log';
        $Log->redirect( path => $nfs_mount_log );

        # Mount volumes and start NFS traffic
        foreach my $vol_obj (@nfs_volumes) {
            $num++;
            # Limit mount points per client ( default = 10 )
            my $limit = $args{'mount_points_per_client'};
            if ($num > (scalar(@{$args{'clients_ref'}})) * $limit) { last; }

            # Get a client from a list of available clients list
            my $client_obj = shift(@{$args{'clients_ref'}});
            push @{$args{'clients_ref'}}, $client_obj;

            # Get a Data Lif from a list of available Data LIF list
            my $lif_obj = shift(@{$nfs_lifs_ref});
            push @{$nfs_lifs_ref}, $lif_obj;

            # Mount volume to Linux client under test
            $Log->comment('Mounting volume on NFS client');
            $Log->comment("$num\)Client name :  " . $client_obj->name());
            $Log->comment('Volume name   :  ' . $vol_obj->volume());
            $Log->comment('NFS data LIF  :  ' . $lif_obj->address());
            $Log->comment('Vserver name  :  ' . $vol_obj->vserver());
            $Log->comment('Node name     :  ' .
                                      $vol_obj->command_interface()->name());

            # Mount section modified by zaw 3/13/2013
            my $mount_obj = NACL::STask::Client::Mount->create(
                    command_interface        => $client_obj,
                    volume                   => $vol_obj,
                    server_command_interface => $vol_obj->command_interface(),
                    server                   => $lif_obj->{'address'},
                    nacltask_if_exists       => 'purge',
                    vers                     => $nfs_version,
                    retries                  => 3 );

            # Update client and mount information to volume object
            $vol_obj->{'client'} = $client_obj;
            $vol_obj->{'mount'} = $mount_obj;
            push @mounted_vol_objs, $vol_obj;

            # Print out client and Data LIF information
            $Log->comment("$num\)Mount point  :  " . $mount_obj->mount_point() .
                    "\nNFS client mount successfully completed" . '  ~ ( ' .
                       $client_obj->name() . ' )' );

            # Start nfs (SIO) traffic
            if ($args{'tool'} =~ /sio/i) {
                $Log->comment('NFS traffic tool :  SIO');

                # Create sio baseline file on client's mount point
                my $path = $mount_obj->mount_point();
                my $file_name = "$path/sio_file_" . random_name_generator();
                $client_obj->apiset(interface => 'CLI')->execute_command(
                                    command => 'echo sio_file > ' . $file_name);

                # Get random file size between 2m to 30m
                my $maximum_size = 30;
                my $minimum_size = 2;
                my $size = int(rand($maximum_size)) + $minimum_size;
                my $file_size = $size . 'm';

                # Get a number of I/O operations per second for sio
                my $sio_iops;
                if ($args{'sio_iops'} > 0) {
                   $sio_iops = q{-}.$args{'sio_iops'};
                } else {
                    $sio_iops = undef;
                }
                # Initialize sio default arguments
                my %sio_default_args = (
                                tool           => 'sio',
                                rand           => '100',
                                block_size     => '4k',
                                start_byte     => 0,
                                num_of_threads => $args{'sio_thread_count'},);

                # Create sio traffic object and start nfs traffic
                my $sio_traffic_obj = NACL::MTask::Traffic->start(
                                command_interface => $client_obj,
                                paths             => $file_name,
                                file_size         => $file_size,
                                read              => 0,
                                iops              => $sio_iops,
                                %sio_default_args);
                push @traffic_objs, $sio_traffic_obj;

            # Start nfs (HAMMER) traffic
            } elsif ($args{'tool'} =~ /hammer/i) {
                $Log->comment('NFS traffic tool : Hammer');
                try {
                    my $hammer_traffic_obj = NACL::MTask::Traffic->start(
                                    command_interface => $client_obj,
                                    paths          => $mount_obj->mount_point(),
                                    on_failure     => 'die',
                                    tool           => 'hammer',
                                    nofilercore    => 1,
                                    fill           => $args{'fill'},
                                    mode           => $args{'mode'} );
                    push @traffic_objs, $hammer_traffic_obj;
                } catch NATE::BaseException with {
                     $Log->warn('NFS traffic hammer ERROR on ' .
                                $client_obj->name() . '_' . $vol_obj->volume());
                };

            # Start nfs (DT) traffic
            } elsif ($args{'tool'} =~ /dt/i) {
                $Log->comment('NFS traffic tool : DT');

                # Create dt baseline file on client's mount point
                my $dt_path = $mount_obj->mount_point();
                my $dt_file = "$dt_path/dt_file_" . random_name_generator();
                $client_obj->apiset(interface => 'CLI')->execute_command(
                                    command => 'echo dt_file > ' . $dt_file);

                my $nopgt  = $args{'dt_no_progress_time_in_secs'} . 's';
                my $nopgtt = $args{'dt_no_progress_trigger_time_in_secs'} . 's';

                my %dt_noprog_options = ();
                if ($args{'dt_enable_no_progress'}) {
                    %dt_noprog_options = (
                        enable     => 'noprog',
                        oncerr     => 'abort',
                        errors     => '1',
                        noprogt    => $nopgt,
                        noprogtt   => $nopgtt,
                        keepalivet => '300s',
                        keepalive  =>
                                '"300SecKeepAliveMsg#%h@%d#TotalTestTime:%T"',
                        alarm      => '5s',
                        history    => '3',
                        hdsize     => '128',
                        trigger    => 'cmd:' .
                            '/x/eng/localtest/noarch/bin/dt_noprog_script.ksh',
                    );
                }

                # Get DT data transfer size limit
                my $dt_limit = $args{'dt_limit'};
                if (!defined($dt_limit)) {
                    my $size_bytes = int( $vol_obj->state->size() / 2 ) ;
                    my $size_mb    = int( $size_bytes / ( 1024 * 1024 ) );
                    my $size_gb    = int( $size_bytes / ( 1024 * 1024 * 1024 ));

                    if ( $size_gb >= 1 ) {
                        $dt_limit = $size_gb . 'g';
                    } elsif ( $size_mb >= 1 ) {
                        $dt_limit = $size_mb . 'm';
                    } else {
                        $dt_limit = $size_bytes;
                    }
                }

                # Create dt traffic object and start nfs traffic
                my $dt_traffic_obj = NACL::MTask::Traffic->start(
                    command_interface => $client_obj,
                    paths             => $dt_file,
                    tool              => 'dt',
                    iotype            => $args{'dt_io_type'},
                    limit             => $dt_limit,
                    oflags            => 'trunc',
                    procs             => $args{'dt_processes'},
                    bs                => '4k',
                    incr              => 'var',
                    min               => '2k',
                    max               => '256k',
                    pattern           => 'iot',
                    runtime           => '-1',
                    dispose           => 'keeponerror',
                    prefix            => '"%d@%h"',
                    disable           => 'pstats',
                    enable            => 'compare',
                    enable            => 'lbdata',
                    enable            => 'debug',
                    enable            => 'syslog',
                    %dt_noprog_options );
                push @traffic_objs, $dt_traffic_obj;

            # Skip the IO and keep client pounts objects
            } elsif ($args{'tool'} =~ /skip/i) {
                $Log->comment('Skipped IO and keep client mounts');
            }
        }
        $Log->log($info_log);
        $Log->reset_path();
    }
    $Log->exit() if $may_exit;

    return (\@mounted_vol_objs, \@traffic_objs, \@export_policy_rules);
}


=head2 start_cifs_traffic()

    # Using minimum required params
    my ($mount_objs, $traffic_objs, $cifs_share_names, $export_policy_rule_obj)=
        NACL::MTask::MCC::DPSASUtils->start_cifs_traffic(
                    vol_objs_ref     => \@volume_objs,
                    win_clients_ref  => \@win_clients );

    # Using all params
    my ($mount_objs, $traffic_objs, $cifs_share_names, $export_policy_rule_obj)=
        NACL::MTask::MCC::DPSASUtils->start_cifs_traffic(
                    vol_objs_ref          => \@volume_objs or [$volume_objs[0]],
                    clients_ref           => \@win_clients,
                    vserver               => vs0,
                    create_data_lifs      => 0,
                    create_routing_groups => 0,
                    sio_iops              => 128,
                    sio_thread_count      => 1,
                    sio_block_size        => '4k',
                    sio_read_percent      => 50,
                    if_cifs_failure       => 'die',
                    if_map_drive_exist    => 'reuse',
                    cifs_domain_short     => $cifs_domain_short,
                    name_servers          => $name_servers,
                    cifs_domain           => $cifs_domain,
                    cifs_user             => $cifs_user,
                    cifs_password         => $cifs_password,
                    cifs_server           => $cifs_server,
                    map_shares_per_client => 8 );

    Wrapper function to start cifs traffic on given volumes in same vservers
    from multiple windows clients.

    Input Params:
    vol_objs_ref          => [Array reference of volume objects]
    win_clients_ref       => [Array reference of windows client objects]
    vserver               => [Vserver name string ( default = vs0 )]
    create_data_lifs      => [Boolean value to create data lifs( default = 0 )]
    create_routing_groups => [Boolean to create routing groups ( default = 1 )]

    sio_iops              => [integer, I/O operations per secs ~ optional,
                              0 to use sio default value ( no iops control ) ]
    sio_thread_count      => [integer, num of sio threads ~ ( default = 1 )]
    sio_file_size_mb      => [ 0 to use random size between 2MB and 30MB ]

    sio_block_size        => [scalar, IO block size ~ default = 4k ]
    sio_read_percent      => [integer, I/O read percent ~ default => 50 ]
    sio_random_percent    => [integer, I/O random percent ~ default => 100 ]

    if_cifs_failure       => ["die|ignore" to handle Traffic Failed exception ~
                                               ( default = die on failure )]
    if_map_drive_exists   => [option to "reuse|purge" if same map drive exists ~
                                               ( default = reuse )]

    cifs_domain_short =>[Scalar,NetBIOS Domain Name ( default = lab )]
    cifs_domain     => [Scalar,Fully Qualified Domain name ( lab.netapp.com)]
    cifs_user       => [Scalar,CIFS user name   ( default = labclient )]
    cifs_password   => [Scalar,CIFS password    ( default = sundance )]
    cifs_server     => [Scalar,CIFS server name ( default = VSx_CIFS)]
    name_servers    => [Scalar,IP address (default = 10.60.132.40,10.60.248.10)]

    map_shares_per_client => [integer number to limit map shares per client]

    Example:
    In the .thpl script:
    param('NAME_SERVERS', '-default', '10.60.132.40,10.60.248.10');

    From command line:
    ntest NODES=node1
          NAME_SERVERS=10.60.132.40,10.60.248.10
          DOMAIN_NAME_SHORT=rtplab

    Return:
    Array reference of mounted volumes objects
    Array reference of traffic objects
    Array reference of cifs share objects
    scalar value of export_policy_rule object

=cut

sub start_cifs_traffic {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            vol_objs_ref          => { type => ARRAYREF },
            win_clients_ref       => { type => ARRAYREF },
            vserver               => { type => SCALAR | UNDEF, optional => 1},
            create_data_lifs      => { regex => qr/^[01]$/, default => 0 },
            create_routing_groups => { regex => qr/^[01]$/, default => 1 },
            sio_iops              => { regex => qr/^\d+$/, default => 0 },
            sio_thread_count      => { regex => qr/^\d+$/, default => 2 },
            sio_file_size_mb      => { regex => qr/^\d+$/, default => 0 },
            sio_block_size        => { type => SCALAR | UNDEF, optional => 1},
            sio_read_percent      => { regex => qr/^\d+$/, default => 50 },
            sio_random_percent    => { regex => qr/^\d+$/, default => 100 },

            if_cifs_failure       => { regex => '^(die|ignore)$',
                                                           default => 'die' },
            if_map_drive_exists   => { regex => '^(reuse|purge)$',
                                                           default => 'reuse' },

            cifs_domain_short    => { type => SCALAR | UNDEF, optional => 1,
                                        default => 'lab'},
            cifs_domain          => { type => SCALAR | UNDEF, optional => 1,
                                        default => 'lab.netapp.com'},
            cifs_user            => { type => SCALAR | UNDEF, optional => 1,
                                        default => 'labclient'},
            cifs_password        => { type => SCALAR | UNDEF, optional => 1,
                                        default => 'sundance'},
            cifs_server          => { type => SCALAR | UNDEF, optional => 1},
            name_servers         => { type => SCALAR | UNDEF, optional => 1,
                                        default => '10.60.132.40,10.60.248.10'},
            map_shares_per_client => { regex => qr/^\d+$/, default => 8 }
        }
    );
    $Log->enter() if $may_enter;

    # Create cifs/nfs data lifs on all nodes #####
    $Log->comment('Creating cifs/nfs data lifs on all nodes');

    # Volume objects from same vserver
    my @all_nodes;
    foreach my $volobj (@{$args{'vol_objs_ref'}}) {
        push @all_nodes, $volobj->command_interface();
    }

    my %node_hash = ();
    my @unique_nodes = grep { ! $node_hash{$_}++ } @all_nodes;

    my $vserver_name = defined($args{'vserver'}) ? $args{'vserver'} :
                                        ${$args{'vol_objs_ref'}}[0]->vserver();
    foreach my $node_obj (@unique_nodes) {
        if ($args{'create_data_lifs'}) {
            $pkg->create_nfs_cifs_data_lifs(
                            node_obj => $node_obj,
                            vserver  => ${$args{'vol_objs_ref'}}[0]->vserver());
        }
        # Routing group must exist to create vserver route
        if ($args{'create_routing_groups'}) {
            $pkg->create_routing_group_route(
                   node_obj => ${$args{'vol_objs_ref'}}[0]->command_interface(),
                   vserver  => $vserver_name );
        }
        # Vserver DNS Services setup
        $Log->comment('Setup vserver DNS services');
        my @name_servers   = $args{'name_servers'};
        NACL::STask::VserverServicesDns->create(
                    'command_interface'  =>
                               ${$args{'vol_objs_ref'}}[0]->command_interface(),
                    'vserver'            => $vserver_name,
                    'name-servers'       => \@name_servers,
                    domains              => [$args{'cifs_domain'}],
                    state                => 'enabled',
                    'nacltask_if_exists' => 'reuse' );

        # CIFS setup
        $Log->comment('Perform CIFS setup');
        my $cifs_server = $args{'cifs_server'};
        if (!defined($cifs_server)) {
            $cifs_server = $vserver_name;
            $cifs_server =~ s/-/_/g;
            $cifs_server .= '_CIFS';
        }
         NACL::MTask::CIFS->setup(
                    'command_interface' =>
                               ${$args{'vol_objs_ref'}}[0]->command_interface(),
                    'vserver'           => $vserver_name,
                    'if_exists'         => 'reuse',
                    'cifs_domain'       => $args{'cifs_domain'},
                    'username'          => $args{'cifs_user'},
                    'password'          => $args{'cifs_password'},
                    'cifs-server'       => $cifs_server,
                    'vserver_name_mapping' => [
                          { pattern   => $args{'cifs_domain_short'} . q{\\} .
                                                            $args{'cifs_user'},
                          replacement => 'root',
                          direction   => 'win-unix'}, ] );

        # Vserver CIFS super user create
        my $eval_result = eval {
            NACL::C::VserverCifsSuperuser->create(
                    command_interface =>
                               ${$args{'vol_objs_ref'}}[0]->command_interface(),
                    domain            => $args{'cifs_domain_short'},
                    vserver           => $vserver_name,
                    accountname       => $args{'cifs_user'} ); 1;
        };
    }

    # Get NFS/CIFS Data LIFs
    my $lifs_ref = $pkg->get_nfs_cifs_data_lifs(
            node_obj      => ${$args{'vol_objs_ref'}}[0]->command_interface(),
            vserver       => $vserver_name,
            data_protocol => 'cifs' );

    # Get minwin compatible Windows clients
    foreach my $minwin_client_obj (@{$args{'win_clients_ref'}}) {
        my $win_client_name = $minwin_client_obj->name();
        if (!minwin_compatible($win_client_name)) {
            confess("$win_client_name is NOT minwin compatible");
        } else {
            $Log->comment("$win_client_name is minwin compatible");
        }
    }

    # Create CIFS share, map and start CIFS traffic ###
    $Log->comment('Perform CIFS traffic');
    my $num = 0;
    my (@mount_objs, @traffic_objs, @cifs_share_objs);

    foreach my $vol_obj (@{$args{'vol_objs_ref'}}) {
        $num++;

        # Limit cifs shares/maps per client ( default = 8 )
        my $limit = $args{'map_shares_per_client'};
        if ($num > (scalar(@{$args{'win_clients_ref'}}) * $limit)) { last; }

        # Get a round-robin client from a list of available clients
        my $win_client_obj =  shift(@{$args{'win_clients_ref'}});
        push @{$args{'win_clients_ref'}}, $win_client_obj;

        # Get a Data LIF from a list of available Data LIF list
        my $lif_obj = shift(@{$lifs_ref});
        push @{$lifs_ref}, $lif_obj;

       # Create CIFS share
        my $share_name = 'share_' . $vol_obj->volume();
        my $share = NACL::STask::VserverCifsShare->create(
                    command_interface  => $vol_obj->command_interface(),
                    vserver            => $vol_obj->vserver(),
                   'share-name'        => $share_name,
                    nacltask_if_exists => 'reuse',
                    path               => q{/} . $vol_obj->volume() );

         push @cifs_share_objs, $share;

        # CIFS share mapping from a windows client
        $Log->comment('Map cifs shares from a windows client');
        # TODO
        # failing with multiple credentials System Error 1219 on w2k3 clients
        my $mount_obj;
        my $win_ver = $win_client_obj->execute(command => 'ver');
        if ($win_ver =~ /(Microsoft\sWindows\s\[Version\s5)/x) {
            # Map Windows version 5.x clients
            $mount_obj = NACL::STask::Client::Mount->create(
                   command_interface        => $win_client_obj,
                   server_command_interface => $vol_obj->command_interface(),
                   share                    => $share_name,
                   vserver                  => $vol_obj->vserver(),
                   server                   => $lif_obj->{'address'},
                   persistent               => 'NO',
                   password                 => $args{'cifs_password'},
                   nacltask_if_exists       => $args{'if_map_drive_exists'} );
        } else {
            # Map Windows version 6.x clients(added the 'user' param by zaw)
            $mount_obj = NACL::STask::Client::Mount->create(
                   command_interface        => $win_client_obj,
                   server_command_interface => $vol_obj->command_interface(),
                   share                    => $share_name,
                   vserver                  => $vol_obj->vserver(),
                   server                   => $lif_obj->{'address'},
                   persistent               => 'NO',
                   user                     => $args{'cifs_user'},
                   password                 => $args{'cifs_password'},
                   nacltask_if_exists       => $args{'if_map_drive_exists'} );
        }

        # Update client_obj and mount_obj in to volume
        $vol_obj->{client} = $win_client_obj;
        $vol_obj->{mount}  = $mount_obj;
        push @mount_objs, $mount_obj;

        $Log->comment('Start CIFS traffic  ' . $num );
        $Log->comment('Windows Map Drive   ' . $mount_obj->mount_point()
                            . ' ~ ( ' . $win_client_obj->name() . ' )' . "\n" );

        # Get random file size within 2m to 30m range
        my $sio_file_size_mb = $args{'sio_file_size_mb'};
        my ($file_size, $sio_file_size);

        if ($sio_file_size_mb < 1) {
            my $maximum_size = 30;
            my $minimum_size = 2;
            my $size = int(rand($maximum_size)) + $minimum_size;
            $file_size = $size . 'm';
            $sio_file_size = 1048576 * $size;
        } else {
            $file_size = $sio_file_size_mb . 'm';
            $sio_file_size = 1048576 * $sio_file_size_mb;
        }
        $Log->comment('sio file size = ' . $file_size);
        $sio_file_size = ( $sio_file_size + 1048576 ); #Actual file size + 1m

        # Get size of I/O's. Example: 2k, 4k, 1m>
        # 'random' keyword makes the I/O size random 512bytes to 256k.
        my $block_size = !defined($args{'sio_block_size'}) ? '4k' :
                                                    $args{'sio_block_size'};
        # Get sio percentage of accesses that are reads
        my $read_percent = $args{'sio_read_percent'};
        if ($read_percent > 100) { $read_percent = 100; }

        # Get percentage of acceses that are random. Range [0,100].
        # Sequential accesses = 0, Random = 100.
        my $random_percent = $args{'sio_random_percent'};
        if ($random_percent > 100) { $random_percent = 100; }

        # Get a number of I/O operations per second for sio
        my $sio_iops;
        if ($args{'sio_iops'} > 0) {
           $sio_iops = q{-}.$args{'sio_iops'};
        } else {
            $sio_iops = undef;
        }

        my $drive_letter = $mount_obj->mount_point();
        my $file_path = $drive_letter . 'file_' . $vol_obj->volume();

        # Create a file on windows client for sio read/write access
        $win_client_obj->apiset(interface => 'CLI')->execute_command(
                command => "fsutil file createnew $file_path $sio_file_size" );

        # Validate 4k block file
        $win_client_obj->apiset(interface => 'CLI')->execute_command(
                command => "fsutil file setvaliddata $file_path 4096" );

        # Start CIFS traffic ( sio random write )
        my $sio_traffic_obj = NACL::MTask::Traffic->start(
                            command_interface => $win_client_obj,
                            paths             => $file_path,
                            file_size         => $file_size,
                            read              => $read_percent,
                            tool              => 'sio',
                            rand              => $random_percent,
                            block_size        => $block_size,
                            verify            => 1,
                            start_byte        => 0,
                            on_failure        => $args{'if_cifs_failure'},
                            iops              => $sio_iops,
                            num_of_threads    => $args{'sio_thread_count'});
        push @traffic_objs, $sio_traffic_obj;
    }
    $Log->exit() if $may_exit;

    return (\@mount_objs, \@traffic_objs, \@cifs_share_objs);
}


=head2 dpsas_cifs_traffic()

    # Under construction
    # Using minimum required params
    my ($mount_objs, $traffic_objs, $cifs_share_names)=
        NACL::MTask::MCC::DPSASUtils->start_cifs_traffic(
                    vol_objs_ref     => \@volume_objs,
                    win_clients_ref  => \@win_clients );

    # Using all params
    my ($mount_objs, $traffic_objs, $cifs_share_names, $export_policy_rule_obj)=
        NACL::MTask::MCC::DPSASUtils->start_cifs_traffic(
                    vol_objs_ref          => \@volume_objs or [$volume_objs[0]],
                    clients_ref           => \@win_clients,
                    create_data_lifs      => 0,
                    create_routing_groups => 1,
                    sio_iops              => 50,
                    sio_thread_count      => 1,
                    if_cifs_failure       => 'die',
                    if_map_drive_exist    => 'reuse',
                    cifs_domain_short     => $cifs_domain_short,
                    name_servers          => $name_servers,
                    cifs_domain           => $cifs_domain,
                    cifs_user             => $cifs_user,
                    cifs_password         => $cifs_password,
                    cifs_server           => $cifs_server,
                    map_shares_per_client => 8 );

    Wrapper function to start cifs traffic on given volumes in same vservers
    from multiple windows clients.

    Input Params:
    vol_objs_ref          => [Array reference of volume objects]
    win_clients_ref       => [Array reference of windows client objects]
    create_data_lifs      => [Boolean value to create data lifs( default = 0 )]
    create_routing_groups => [Boolean to create routing groups ( default = 1 )]
    sio_iops              => [integer, I/O operations per secs ~ optional]
    sio_thread_count      => [integer, num of sio threads ~ ( default = 1 )]
    if_cifs_failure       => ["die|ignore" to handle Traffic Failed exception ~
                                               ( default = die on failure )]
    if_map_drive_exists   => [option to "reuse|purge" if same map drive exists ~
                                               ( default = reuse )]

    cifs_domain_short =>[Scalar,NetBIOS Domain Name ( default = lab )]
    cifs_domain     => [Scalar,Fully Qualified Domain name ( lab.netapp.com)]
    cifs_user       => [Scalar,CIFS user name   ( default = labclient )]
    cifs_password   => [Scalar,CIFS password    ( default = sundance )]
    cifs_server     => [Scalar,CIFS server name ( default = VSx_CIFS)]
    name_servers    => [Scalar,IP address (default = 10.60.132.40,10.60.248.10)]

    map_shares_per_client => [integer number to limit map_shares per client]

    Example:
    In the .thpl script:
    param('NAME_SERVERS', '-default', '10.60.132.40,10.60.248.10');

    From command line:
    ntest NODES=node1 NAME_SERVERS=10.60.132.40,10.60.248.10
          DOMAIN_NAME_SHORT=rtplab

    Return:
    Array reference of mounted volumes objects
    Array reference of traffic objects
    Array reference of cifs share objects
    scalar value of export_policy_rule object

=cut

sub dpsas_cifs_traffic {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            vol_objs_ref          => { type => ARRAYREF },
            win_clients_ref       => { type => ARRAYREF },
            create_data_lifs      => { regex => qr/^[01]$/, default => 0 },
            create_routing_groups => { regex => qr/^[01]$/, default => 1 },
            sio_iops              => { regex => qr/^\d+$/, default => 0 },
            sio_thread_count      => { regex => qr/^\d+$/, default => 1 },
            if_cifs_failure       => { regex => '^(die|ignore)$',
                                                           default => 'die' },
            if_map_drive_exists   => { regex => '^(reuse|purge)$',
                                                           default => 'reuse' },

            cifs_domain_short    => { type => SCALAR | UNDEF, optional => 1,
                                        default => 'lab'},
            cifs_domain          => { type => SCALAR | UNDEF, optional => 1,
                                        default => 'lab.netapp.com'},
            cifs_user            => { type => SCALAR | UNDEF, optional => 1,
                                        default => 'labclient'},
            cifs_password        => { type => SCALAR | UNDEF, optional => 1,
                                        default => 'sundance'},
            cifs_server          => { type => SCALAR | UNDEF, optional => 1},
            name_servers         => { type => SCALAR | UNDEF, optional => 1,
                                        default => '10.60.132.40,10.60.248.10'},
            map_shares_per_client => { regex => qr/^\d+$/, default => 8 }
        }
    );
    $Log->enter() if $may_enter;

    # Verify minwin compatible Windows clients
    foreach my $minwin_client_obj (@{$args{'win_clients_ref'}}) {
        my $win_client = $minwin_client_obj->name();
        if (!minwin_compatible($win_client)) {
            confess("$win_client is NOT a minwin compatible");
        } else {
            $Log->comment("$win_client is minwin compatible");
        }
    }

    # Get all vserver names associated with volumes
    my @dummy_vs_names;
    foreach my $volume (@{$args{'vol_objs_ref'}}) {
        push @dummy_vs_names, $volume->vserver();
    }
    my %vserver_hash = ();
    my @unique_vserver_names = grep { ! $vserver_hash{$_}++ } @dummy_vs_names;

    # CIFS traffic on multiple vservers
    my @test_vserver_objs;
    foreach my $test_vs_name (@unique_vserver_names) {
        my $unique_vserver_obj = NACL::CS::Vserver->fetch(
            command_interface =>
                     ${$args{'vol_objs_ref'}}[0]->command_interface(),
           filter  => { subtype => 'sync-source',
                        vserver => $test_vs_name } );
        push @test_vserver_objs, $unique_vserver_obj;
    }

    # CIFS preparation logs to CIFS_preparation folder
    my $info_log = NATE::Result::Info->new('Completed');
    my $cifs_preparation_log = Tharn::param('RUNID') . '/cifs_preparation.log';
    $Log->redirect( path => $cifs_preparation_log );

    foreach my $test_vserver (@test_vserver_objs) {
        # Create Data LIFs on all vservers
        if ($args{'create_data_lifs'}) {
            $pkg->create_nfs_cifs_data_lifs(
                    node_obj => $test_vserver->command_interface(),
                    vserver  => $test_vserver->vserver() );
        }

        # Routing group must exist to create vserver route
        if ($args{'create_routing_groups'}) {
            $pkg->create_routing_group_route(
                    node_obj => $test_vserver->command_interface(),
                    vserver  => $test_vserver->vserver() );
        }

        # Vserver DNS Services setup
        $Log->comment('Setup vserver DNS services');
        my @name_servers   = $args{'name_servers'};
        NACL::STask::VserverServicesDns->create(
            command_interface  => $test_vserver->command_interface(),
            vserver            => $test_vserver->vserver(),
            'name-servers'     => \@name_servers,
            domains            => [$args{'cifs_domain'}],
            state              => 'enabled',
            nacltask_if_exists => 'reuse' );

        # CIFS setup
        $Log->comment('Perform CIFS setup');
        my $cifs_server = $args{'cifs_server'};
        if (!defined($cifs_server)) {
            $cifs_server = $test_vserver->vserver();
            $cifs_server =~ s/-/_/g;
            $cifs_server .= '_CIFS';
        }
         NACL::MTask::CIFS->setup(
            command_interface => $test_vserver->command_interface(),
            vserver           => $test_vserver->vserver(),
            if_exists         => 'reuse',
            cifs_domain       => $args{'cifs_domain'},
            username          => $args{'cifs_user'},
            password          => $args{'cifs_password'},
            'cifs-server'     => $cifs_server,
            workgroup         => $args{'cifs_domain_short'},
            vserver_name_mapping => [
                  { pattern   => $args{'cifs_domain_short'} . q{\\} .
                                 $args{'cifs_user'},
                  replacement => 'root',
                  direction   => 'win-unix'}, ] );

        # Vserver CIFS super user create
        my $eval_result = eval {
            NACL::C::VserverCifsSuperuser->create(
                    command_interface => $test_vserver->command_interface(),
                    vserver           => $test_vserver->vserver(),
                    domain            => $args{'cifs_domain_short'},
                    accountname       => $args{'cifs_user'} ); 1;
        };
    }
    $Log->log($info_log);
    $Log->reset_path();

    # Wait for routing group started on data lif
    if ($args{'create_routing_groups'}) {
        $Log->comment('Wait for routing-groups route process up');
        Tharn::snooze(45);
    }

    my $num = 0;
    my (@mount_objs, @traffic_objs, @cifs_share_objs);
    foreach my $test_vserver_obj (@test_vserver_objs) {
        # Get CIFS Data LIF associated with current vserver
        $Log->comment('Get data lifs and client objects');
        my $cifs_lifs_ref = $pkg->get_nfs_cifs_data_lifs(
                node_obj      => $test_vserver_obj->command_interface(),
                vserver       => $test_vserver_obj->vserver(),
                data_protocol => 'cifs' );

        # Filer cifs volumes only from current vserver
        my @dummy_volumes = NACL::C::Volume->find(
                command_interface => $test_vserver_obj->command_interface(),
                filter  => { vsroot  => 'false',
                             vserver => $test_vserver_obj->vserver()});

        my @cifs_volumes = ();
        foreach my $volume_obj (@{$args{'vol_objs_ref'}}) {
            foreach my $dummy_vol (@dummy_volumes) {
                if ($volume_obj->volume() eq $dummy_vol->volume()) {
                    push @cifs_volumes, $volume_obj;
                }
            }
        }

        # CIFS share mount logs to CIFS_mount folder
        my $cifs_mount_log = Tharn::param('RUNID') . '/cifs_mount.log';
        $Log->redirect( path => $cifs_mount_log );

        foreach my $vol_obj (@cifs_volumes) {
            $num++;
            # Limit cifs shares/maps per client ( default = 8 )
            my $limit = $args{'map_shares_per_client'};
            if ($num > (scalar(@{$args{'win_clients_ref'}}) * $limit)) { last; }

            # Get a round-robin client from a list of available clients
            my $win_client_obj =  shift(@{$args{'win_clients_ref'}});
            push @{$args{'win_clients_ref'}}, $win_client_obj;

            # Get a Data LIF from a list of available Data LIF list
            my $lif_obj = shift(@{$cifs_lifs_ref});
            push @{$cifs_lifs_ref}, $lif_obj;

           # Create CIFS share
            my $share_name = 'dpg_cifs_share_' . $vol_obj->volume();
            my $share = NACL::STask::VserverCifsShare->create(
                    command_interface  => $vol_obj->command_interface(),
                    vserver            => $vol_obj->vserver(),
                   'share-name'        => $share_name,
                    nacltask_if_exists => 'reuse',
                    path               => q{/} . $vol_obj->volume() );

             push @cifs_share_objs, $share;

            # CIFS share mapping from a windows client
            $Log->comment('Map cifs shares from a windows client');
            # TODO
            # failing multiple credentials System Error 1219 on w2k3 clients
            my $mount_obj;
            my $win_ver = $win_client_obj->execute(command => 'ver');
            if ($win_ver =~ /(Microsoft\sWindows\s\[Version\s5)/x) {
                # Map Windows version 5.x clients
                $mount_obj = NACL::STask::Client::Mount->create(
                   command_interface        => $win_client_obj,
                   server_command_interface => $vol_obj->command_interface(),
                   share                    => $share_name,
                   vserver                  => $vol_obj->vserver(),
                   server                   => $lif_obj->{'address'},
                   persistent               => 'NO',
                   password                 => $args{'cifs_password'},
                   nacltask_if_exists       => $args{'if_map_drive_exists'} );
            } else {
                # Map Windows version 6.x clients(added the 'user' param)
                $mount_obj = NACL::STask::Client::Mount->create(
                   command_interface        => $win_client_obj,
                   server_command_interface => $vol_obj->command_interface(),
                   share                    => $share_name,
                   vserver                  => $vol_obj->vserver(),
                   server                   => $lif_obj->{'address'},
                   persistent               => 'NO',
                   user                     => $args{'cifs_user'},
                   password                 => $args{'cifs_password'},
                   nacltask_if_exists       => $args{'if_map_drive_exists'} );
            }

            # Update client_obj and mount_obj in to volume
            $vol_obj->{client} = $win_client_obj;
            $vol_obj->{mount}  = $mount_obj;
            push @mount_objs, $mount_obj;

            $Log->comment('Start CIFS traffic  ' . $num );
            $Log->comment('Windows Map Drive   ' . $mount_obj->mount_point()
                        . ' ~ ( ' . $win_client_obj->name() . ' )' . "\n" );

            # Get random file size within 2m to 30m range
            my $maximum_size = 30;
            my $minimum_size = 2;
            my $size = int(rand($maximum_size)) + $minimum_size;
            my $file_size = $size . 'm';

            # Get a number of I/O operations per second for sio
            my $sio_iops;
            if ($args{'sio_iops'} > 0) {
               $sio_iops = q{-}.$args{'sio_iops'};
            } else {
                $sio_iops = undef;
            }
            my $drive_letter = $mount_obj->mount_point();
            my $file_path = $drive_letter . 'sio_file_' . $vol_obj->volume();

            # Create a file on windows client for sio read/write access
            $win_client_obj->apiset(interface => 'CLI')->execute_command(
                        'command' => "fsutil file createnew $file_path $size");

            # Start CIFS traffic ( sio random write )
            my $sio_traffic_obj = NACL::MTask::Traffic->start(
                            command_interface => $win_client_obj,
                            paths             => $file_path,
                            file_size         => $file_size,
                            read              => 0,
                            tool              => 'sio',
                            rand              => '100',
                            block_size        => '4k',
                            start_byte        => 0,
                            on_failure        => $args{'if_cifs_failure'},
                            iops              => $sio_iops,
                            num_of_threads    => $args{'sio_thread_count'});
            push @traffic_objs, $sio_traffic_obj;
        }
        $Log->log($info_log);
        $Log->reset_path();
    }
    $Log->exit() if $may_exit;

    return (\@mount_objs, \@traffic_objs, \@cifs_share_objs);
}


=head2 perform_storage_hammer()

    # To start storage hammer on volumes
     NACL::MTask::MCC::DPSASUtils->perform_storage_hammer(
            vol_objs_ref        => \@volumes,
            hammer_thread_limit => 10 );

    # To abort storage hammer jobs from volumes
    NACL::MTask::MCC::DPSASUtils->perform_storage_hammer(
            vol_objs_ref => \@volumes,
            job          => 'stop');

    Wrapper function to perform storage hammer( a.k.a filer hammer ) stress test
    on given volume objects.

    Input Params:
    vol_objs_ref => [Array reference of volume objects]
    job          => [string value 'start' or 'stop'/ default = 'start']
    hammer_thread_limit => [number of volumes to be running filer hammer stress]

    Return:
    None

=cut

sub perform_storage_hammer {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            vol_objs_ref => { type => ARRAYREF },
            job          => { regex => '^(start|stop)$', default => 'start' },
            hammer_thread_limit => { regex => qr/^\d+$/, default => 10 }
        }
    );

    $Log->enter() if $may_enter;
    my $num = 0;

    if ($args{'job'} =~ /start/i) {
        $Log->comment('Storage hammer start');
        foreach my $vol_obj (@{$args{'vol_objs_ref'}}) {
            $num++;
            if ( $num > $args{'hammer_thread_limit'} ||
                                    $num > scalar(@{$args{'vol_objs_ref'}}) ) {
                last;
            }
            my $vol_path = '/vol/' . $vol_obj->volume() . '/ham' . $num;

            # The hammer thread (aka blacksmith) currently 5 of blacksmiths
            # that can be launched in parallel in admin mode. But, running 10
            # trreads in test mode.

            my $cmd = "\"hammer -f -1 $vol_path 100000\"";
            my $response = $vol_obj->command_interface()->
            get_7m_or_nodescope_apiset(interface => 'CLI')->
            execute_command( command => $cmd,
                             'privilege-level' => 'diag' );
        }
    } elsif ($args{'job'} =~ /stop/i) {
        $Log->comment('Storage hammer abort');
        my %hash = ();
        my @unique_vol_objs = grep {
            ! $hash{$_->command_interface()}++ } @{$args{'vol_objs_ref'}};

        foreach my $unique_vol_obj (@unique_vol_objs) {
            $unique_vol_obj->command_interface()->
            get_7m_or_nodescope_apiset(interface => 'CLI')->
            execute_command( command => '"hammer abort"',
                             'privilege-level' => 'diag' );
        }
     }
    $Log->exit() if $may_exit;
    return;
}


=head2 vol_modify_nvfail_on_dr_option()

    my $vol_objs =
    NACL::MTask::MCC::DPSASUtils->vol_modify_nvfail_option(...);

    Wrapper function to modify 'force_nvfail_on_dr' option on volumes

    Input Params:
    vol_objs_ref  => [ Array reference of volume objects from same vserver ]
    nvfail_option => [ string value on | off ,  default = on ]

    Return:
    Array reference of volume objects

=cut

sub vol_modify_nvfail_on_dr_option {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            vol_objs_ref  => { type => ARRAYREF },
            nvfail_option => { regex => '^(on|off)$', default => 'on' }
        }
    );

    $Log->enter() if $may_enter;
    my @modified_vols;
    my $node_obj = ${$args{'vol_objs_ref'}}[0]->command_interface();
    my $Api_Set_Obj = $node_obj->apiset(interface => 'CLI');
    my $nvfail_option = $args{'nvfail_option'};

    # Modify force nvfail_on_dr option
    my $num = 0;
    foreach my $vol_obj (@{$args{'vol_objs_ref'}}) {
        $num++;
        if ($vol_obj->volume() !~ /root/) {
            $Api_Set_Obj->volume_modify(
               'privilege-level'  => 'test',
                volume            => $vol_obj->volume(),
                vserver           => ${$args{'vol_objs_ref'}}[0]->vserver(),
               'force-nvfail-on-dr' => $args{'nvfail_option'}
            );
        }
        push @modified_vols, $vol_obj;
    }

    # Verify force nvfail_on_dr option
    foreach my $vol (@modified_vols) {
        my $nvfail_state  = $vol->get_one_state_attribute('force_nvfail_on_dr');
        if ($nvfail_state ne $args{'nvfail_option'}) {
            confess("\nERROR!!! Unable to modify force_nvfail_on_dr option\n");
        } else {
            $Log->comment("$num\. Modified the force-nvfail-on-dr : " .
                $args{'nvfail_option'} . ' to volume ' . $vol->volume());
        }
    }
    $Log->exit() if $may_exit;

    return \@modified_vols;
}


=head2 move_volumes()

    my $vols_to_move = NACL::MTask::MCC::DPSASUtils->move_volumes(..);

    Wrapper function to move volumes between HA partner aggregates and
    move back to original aggregate.

    Input Params:
    vol_objs_ref => [Array reference of volume objects]
    dst_aggr_obj => [Reference of destination aggregate object]
    move_back    => [boolean value to perform volume move back]

    Return:
    Array reference of volume objects

=cut

sub move_volumes {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            vol_objs_ref => { type => ARRAYREF },
            dst_aggr_obj => { type => HASHREF },
            move_back    => { regex => qr/^[01]$/, default => 0 }
        }
    );

    $Log->enter() if $may_enter;
    my @vols_to_move;
    my $orig_aggr_name =
        ${$args{'vol_objs_ref'}}[0]->get_one_state_attribute('aggregate');

    # move to destination aggregate
    foreach my $vol_obj (@{$args{'vol_objs_ref'}}) {
        $Log->comment(describe_timeout(time) .
                    ' start moving ' . $vol_obj->volume());

        my $vol_move = NACL::STask::VolumeMove->start(
                command_interface       => $vol_obj->command_interface(),
                vserver                 => $vol_obj->vserver(),
                volume                  => $vol_obj->volume(),
                'destination-aggregate' => $args{'dst_aggr_obj'}->aggregate(),
                nacltask_wait           => 1);

        $vol_obj->{'vol_move'} = $vol_move;
        push @vols_to_move, $vol_obj;
    }

    foreach my $vol (@vols_to_move) {
        my $vol_moved = $vol->{vol_move};
        $vol_moved->wait_for_move(
            'destination-aggregate' => $args{'dst_aggr_obj'}->aggregate(),
            'method-timeout'        => 3600);

        $Log->comment(describe_timeout(time) .
                    ' Completed the move of ' . $vol_moved->volume());
    }

    # move back to original aggregate
    if ($args{'move_back'}) {
        $Log->comment('Moving ' . scalar(@vols_to_move)
                              . ' volumes back to ' . $orig_aggr_name);
        foreach my $volobj (@vols_to_move) {
            $Log->comment(describe_timeout(time) .
                        ' start moving ' .$volobj->volume());

            my $vol_moved_back = NACL::STask::VolumeMove->start(
                command_interface       => $volobj->command_interface(),
                vserver                 => $volobj->vserver(),
                volume                  => $volobj->volume(),
                'destination-aggregate' => $orig_aggr_name,
                nacltask_wait           => 1);

            $volobj->{vol_move} = $vol_moved_back;
        }
        foreach my $orgi_vol (@vols_to_move) {
            my $volmove = $orgi_vol->{vol_move};
            $volmove->wait_for_move(
                'destination-aggregate' => $orig_aggr_name,,
                'method-timeout'        => 3600);

            $Log->comment(describe_timeout(time) .
                ' Completed the move of ' . $volmove->volume());
        }
    }
    $Log->exit() if $may_exit;
    return \@vols_to_move;
}


=head2 disk_fail_replace_from_both_plexes()

    my $prefail_disks =
    NACL::MTask::MCC::DPSASUtils->disk_fail_replace_from_both_plexes(...);

   Wrapper function to perform disk fail replace given aggr's Pool0 and Pool1.
   Skip this task if previous prefail test in progress.

    Input Params:
    aggr_obj  => [Reference of aggregate object]

    Return:
    Array reference of prefail_disks objects

=cut

sub disk_fail_replace_from_both_plexes {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            aggr_obj => { type => HASHREF }
        }
    );

    $Log->enter() if $may_enter;
    my (@prefail_disks, @p0_disks, @p1_disks);
    NACL::C::StorageDiskOption->modify(
        command_interface   => $args{'aggr_obj'}->command_interface(),
        node                => $args{'aggr_obj'}->command_interface(),
        autoassign          => 'on');

    # Pool0 disk fail replace
    try {
        NACL::STask::StorageDisk->list(
            command_interface => $args{'aggr_obj'}->command_interface(),
            tree              => $args{'aggr_obj'}->aggregate(),
            prefail           => 1,
            pool              => 0);
    }
    otherwise {
        my $pool0_disks = NACL::STask::StorageDisk->list(
            command_interface => $args{'aggr_obj'}->command_interface(),
            tree              => $args{'aggr_obj'}->aggregate(),
            pool              => 0);
        foreach my $p0_disk (@{$pool0_disks}) {
            push @p0_disks, $p0_disk->{name};
            last;
        }
        $args{'aggr_obj'}->command_interface()->
        get_7m_or_nodescope_apiset(interface => 'CLI')->execute_command(
            'command'         => "\"disk fail -f $p0_disks[0]\"",
            'privilege-level' => 'diag' );
    };

    # Pool1 disk fail replace
    try {
        NACL::STask::StorageDisk->list(
            command_interface => $args{'aggr_obj'}->command_interface(),
            tree              => $args{'aggr_obj'}->aggregate(),
            prefail           => 1,
            pool              => 1);
    }
    otherwise {
        my $pool1_disks = NACL::STask::StorageDisk->list(
            command_interface => $args{'aggr_obj'}->command_interface(),
            tree              => $args{'aggr_obj'}->aggregate(),
            pool              => 1);
        foreach my $p1_disk (@{$pool1_disks}) {
            push @p1_disks, $p1_disk->{name};
            last;
        }
        $args{'aggr_obj'}->command_interface()->
        get_7m_or_nodescope_apiset(interface => 'CLI')->execute_command(
            'command'         => "\"disk fail -f $p1_disks[0]\"",
            'privilege-level' => 'diag' );
    };
    push @prefail_disks, ($p0_disks[0], $p1_disks[0]);
    $Log->exit() if $may_exit;
    return \@prefail_disks;
}


=head2 mirror_plex_offline_online()

    NACL::MTask::MCC::DPSASUtils->mirror_plex_offline_online(...);

    Wrapper function to toggle the mirror plex of MCC aggregate every 5 minutes

    Input Params:
    aggr_obj  => [Reference of aggregate object]
    toggle    => [string value 'offline']

    Return:
    None

=cut

sub mirror_plex_offline_online {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            aggr_obj => { type => HASHREF },
            toggle   => { regex => '^(online|offline)$', default => 'online' }
        }
    );

    $Log->enter() if $may_enter;
    my $mirror_plex_name;
    my $plexes = $args{'aggr_obj'}->get_one_state_attribute('plexes');
    my $agg_name = $args{'aggr_obj'}->aggregate();

    if ($plexes->[1] =~ /\/$agg_name\/(plex\d+)/i){
      $mirror_plex_name = $1;
    } else {
      confess("Unable to get plex name from $agg_name");
    }

    if ($args{'toggle'} =~ /offline/i) {
        $Log->comment("Offline $plexes->[1]");
        try {
            NACL::C::StorageAggregatePlex->offline(
                command_interface => $args{'aggr_obj'}->command_interface(),
                aggregate         => $agg_name,
                plex              => $mirror_plex_name);
        } otherwise {
            $Log->comment("Plex $plexes->[1] is already offline");
        };

        # Keep offline plex maximum 90 seconds
        my $offline_range = 60;
        my $minimum = 30;
        my $duration = int(rand($offline_range)) + $minimum;

        $Log->comment("keep offline $mirror_plex_name $duration seconds");
        Tharn::snooze($duration);

        my $timeout = time + 600;
        while ($args{'aggr_obj'}->
            get_one_state_attribute('raidstatus') =~ /mirrored/) {
            $Log->comment('Aggregate\'s raid status : ' .
                      $args{'aggr_obj'}->get_one_state_attribute('raidstatus'));
            if (time >= $timeout) { last; }
            Tharn::snooze(60);
        }
    } elsif ($args{'toggle'} =~ /online/i) {
        $Log->comment("Online $plexes->[1] and waiting for mirrored state...");
        NACL::C::StorageAggregatePlex->online(
            command_interface => $args{'aggr_obj'}->command_interface(),
            aggregate         => $agg_name,
            plex              => $mirror_plex_name );

        my $time_out = time + 3600 * 3;
        while ($args{'aggr_obj'}->
            get_one_state_attribute('raidstatus') !~ /mirrored/) {
            $Log->comment('Aggregate\'s raid status : '.
                      $args{'aggr_obj'}->get_one_state_attribute('raidstatus'));
            if (time >= $time_out) { last; }
            Tharn::snooze(60);
        }
        $Log->comment($args{'aggr_obj'}->aggregate() . ' is mirrored');
    }
    $Log->exit() if $may_exit;

    return $mirror_plex_name;
}


=head2 perform_wafliron()

    NACL::MTask::MCC::DPSASUtils->perform_wafliron(...);

    Wrapper function to perform WAFLIRON on aggregates from multiple nodes

    Input Params:
    aggr_obj  => [Reference of aggregate object]

    Return:
    None

=cut

sub perform_wafliron {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            aggr_obj => { type => HASHREF }
        }
    );

    $Log->enter() if $may_enter;
    $Log->comment('Starting wafliron');
    my $wafliron_obj = NACL::MTask::Wafliron->new(
            command_interface => $args{'aggr_obj'}->command_interface(),
            aggregate         => $args{'aggr_obj'}->aggregate());

    $wafliron_obj->start(
            nacltask_include_mirrors => 1,
            nacltask_optional_commit => 'false');

    $wafliron_obj->wait_for_completion('method-timeout' => 3600);

    my $messages = $wafliron_obj->get_wafliron_messages();
    if ($wafliron_obj->corruption_found()) {
        $Log->comment('Wafliron found corruption during wafliron check');
        my $review_messages = $wafliron_obj->review();
        $Log->debug('Wafliron review messages : ' . $review_messages );
    } else {
        $Log->comment('wafliron did not find corruption');
        $Log->comment("wafliron completed\n");

        # Bring back aggregate online
        my $aggr_state = $args{'aggr_obj'}->get_one_state_attribute('state');
        if ($aggr_state =~ /offline/i) {
            $args{'aggr_obj'}->modify( state => 'online' );
        }
    }
    $Log->exit() if $may_exit;
    return;
}


=head2 create_mirror_aggregate()

    my $aggr_obj_ref =
    NACL::MTask::MCC::DPSASUtils->create_mirror_aggregate(...);

    Wrapper function to create mirror aggregates on MCC testbeds

    Input Params:
    node_obj   => [NACL::C::SystemNode object]
    aggr_name  => [Aggregate name string ( optional )]
    disk_count => [integer number of disks to create aggregate]

    Return:
    Scalar value of aggregate object

=cut

sub create_mirror_aggregate {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            node_obj   => { type => HASHREF },
            aggr_name  => { type => SCALAR | UNDEF, optional => 1 },
            disk_count => { regex => qr/^\d+$/, default => 6 },
        }
    );

    $Log->enter() if $may_enter;
    my $aggr_name = defined($args{'aggr_name'}) ? $args{'aggr_name'} :
                    random_name_generator( prefix => 'MIRROR_AGGR', size => 1);

    my $disk_count;
    if ($args{'disk_count'} < 6) {
        $disk_count = 6;
    } else {
        $disk_count = $args{'disk_count'};
    }

    my $aggr_obj_ref = NACL::STask::Aggregate->create(
            command_interface => $args{'node_obj'},
            aggregate         => $aggr_name,
            diskcount         => $disk_count,
            if_exists         => 'reuse',
            mirror            => 'true',
            nacltask_wait     => 1 );

    # Wait until aggregate is online.
    my $timeout = time + 3600;
    while ($aggr_obj_ref->get_one_state_attribute('state') ne 'online') {
        $Log->comment('Aggregate\'s current state : ' .
                            $aggr_obj_ref->get_one_state_attribute('state'));
        if (time >= $timeout) { last; }
        Tharn::snooze(60);
    }
    $Log->comment('Aggregate under test    : ' . $aggr_obj_ref->aggregate()
                . ' ( ' . $args{'node_obj'}->name() . ' )');

    $Log->exit() if $may_exit;
    return $aggr_obj_ref;
}


=head2 create_volumes_with_snapshot_autodelete()

    my $vol_objs =
    NACL::MTask::MCC::DPSASUtils->create_volumes_with_snapshot_autodelete(...);

    Wrapper function to create multiple volumes and also mofify Volume Snapshot
    Autodelete option

    Input Params:
    node_obj  => [NACL::C::SystemNode object]
    aggr_obj  => [Reference of aggregate object]
    vserver   => [Vserver name string ( default = vs0 )]
    vol_size  => [Volume size scalar ( default = 1g )]
    vol_count => [Volume count integer( default = 1 )]

    Return:
    Array reference of volume objects

=cut

sub create_volumes_with_snapshot_autodelete {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            node_obj  => { type  => HASHREF },
            aggr_obj  => { type  => HASHREF },
            vserver   => { type  => SCALAR | UNDEF, optional => 1,
                                                    default  => 'vs0' },
            vol_size  => { type  => SCALAR | UNDEF, optional => 1,
                                                    default  => '1g' },
            vol_count => { regex => qr/^\d+$/, default => 1 }
        }
    );

    $Log->enter() if $may_enter;
    # Volume names associate with node, aggregate and vserver names
    my $temp_node_name    = $args{'node_obj'}->name();
    my $temp_vserver_name = $args{'vserver'};
    $temp_node_name    =~ s/-/_/g;
    $temp_vserver_name =~ s/-/_/g;

    my @nds  = split( /_\s*/, $temp_node_name );
    my @aggr = split( /_\s*/, $args{'aggr_obj'}->aggregate() );
    my @vsr  = split( /_\s*/, $temp_vserver_name );

    my @vol_objs;
    for my $num (1 .. $args{'vol_count'}) {
        my @uuid = split( /-\s*/, NACL::MTask::MCC::Log::new_uuid() );
        my $vol_prefix =
            'vol_' . $nds[-1] . '_' . "$aggr[-1]$vsr[-1]" . '_' . $uuid[0];
        $vol_prefix =~ s/-/_/g;
        my $vol_name = $vol_prefix . '_' . $num;
        my $vol_obj = NACL::STask::Volume->create(
                command_interface  => $args{'node_obj'},
                aggregate          => $args{'aggr_obj'}->aggregate(),
                volume             => $vol_name,
                vserver            => $args{'vserver'},
                size               => $args{'vol_size'},
                if_exists          => 'reuse',
                type               => 'RW',
                autosize           => 'true',
               'max-autosize'      => '200g',
               'junction-active'   => 'true',
               'junction-path'     => "/$vol_name",
               'unix-permissions'  => '---rwxrwxrwx',
               'security-style'    => 'mixed');

        NACL::C::VolumeSnapshotAutodelete->modify(
                command_interface  => $args{'node_obj'},
                volume             => $vol_obj->volume(),
                vserver            => $args{'vserver'},
                enabled            => 'true',
               'defer-delete'      => 'none',
               'target-free-space' => '5');
        push @vol_objs, $vol_obj;
        $Log->comment("$num\. Volume " .  $vol_obj->volume() . ' created');
    }
    $Log->exit() if $may_exit;
    return \@vol_objs;
}


=head2 get_data_port_ip_gw_netmask()

    my ($port_ip_hash_ref , $gateway_ip, $netmask) =
            NACL::MTask::MCC::DPSASUtils->get_data_port_ip_gw_netmask(
            node_obj => $a1_node
    );

    while ( my ($key, $value) = each(%{$port_ip_hash_ref}) ) {
        $Test->nlog("Data port  : $key");
        $Test->nlog("IP address : $value");
    }
    $Test->nlog("Gateway ip : $gateway_ip");
    $Test->nlog("Netmask    : $netmask");

    Wrapper function to fetch free available data ports from given node and
    aquire IP address, Netmask, Gateway address which is associated with free
    data ports from nate_host file

    Input Params:
    node_obj  => [NACL::C::SystemNode object]

    Return:
    1) Hash reference of port name and IP associated with port
    2) Scalar value of Data Gateway ip address
    3) Scalar value of Data Netmask

=cut

sub get_data_port_ip_gw_netmask {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            node_obj => { type => HASHREF }
        }
    );

    $Log->enter() if $may_enter;
    no warnings 'uninitialized';
    my (@data_port_names,
        @data_port_pairs,
        @all_ports_ip,
        $gateway_ip,
        $netmask,
        $data_port_ip_key,
        $port_under_test );
    my %port_ip_hash = ();

    # Get all free data ports from node
    my @all_port_objs = NACL::MTask::FreeDataPort->discovery(
            command_interface => $args{'node_obj'});
    foreach my $port_obj (@all_port_objs) {
        push @data_port_names, $port_obj->port();
    }

    # Fetch Data IP, gateway and netmask from "nate_host" file associated
    # with free data port
    my %params = $args{'node_obj'}->hostrec()->param_fields();
    foreach my $data_port (@data_port_names) {
        while ( my ($key, $value) = each(%params) ) {
            if ($value =~ /$data_port/) {
                if ($key =~ /DATA_PORT/) {
                    $port_under_test = $value;

                    my $copy_key = $key;
                    my $data_port_num = chop $copy_key;
                    $data_port_ip_key = 'DATA_IP' . $data_port_num;

                    my $key_and_value = ("$key:$value");
                    push @data_port_pairs, $key_and_value;
                }
            }
            elsif ($key =~ /$data_port_ip_key/) {
                my $port_and_ip = ("$port_under_test $value");
                if ($port_and_ip =~ /^#/) { next; }
                push @all_ports_ip, $port_and_ip;
                $port_ip_hash {$port_under_test} = $value;
            }
            elsif ($key =~ /DATA_GATEWAY1/) {
                $gateway_ip = $value;
            }
            elsif ($key =~ /DATA_MASK1/) {
                $netmask = $value;
            }
        }
    }

    # Remove depulicated keys
    my %hash = ();
    my @unique_ports_ip = grep { ! $hash{$_}++ } @all_ports_ip;
    if (scalar(@unique_ports_ip) < 1) {
        $Log->comment('All data IPs are in use on '. $args{'node_obj'}->node());
    }
    foreach my $port_ip (@unique_ports_ip) {
        $Log->comment('Data port and IP pair : ' . $port_ip );
    }
    $Log->comment('Gateway IP       : ' . $gateway_ip);
    $Log->comment('Netmask          : ' . $netmask);
    $Log->exit() if $may_exit;

    return (\%port_ip_hash, $gateway_ip, $netmask);
}


=head2 find_or_create_vserver()

    my $vserver_obj =
            NACL::MTask::MCC::DPSASUtils->find_or_create_vserver(...);

    Wrapper function to find existing vserver first and reuse existing vserver
    or create new vserver if reusable vserver did not find.

    Input Params:
    node_obj  => [NACL::C::SystemNode object]
    aggregate => [Aggregate name string]
    vserver   => [Vserver name string ( default = undef )]

    Return:
    scalar reference of vserver object

=cut

sub find_or_create_vserver {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            node_obj  => { type => HASHREF },
            aggregate => { type => SCALAR },
            vserver   => { type => SCALAR | UNDEF, optional => 1 }
        }
    );

    $Log->enter() if $may_enter;
    # Check existing vserver's health
    my $temp_vserver;
    my $vserver_name = !defined($args{'vserver'}) ? undef : $args{'vserver'};
    my $evl_return = eval {
        $temp_vserver = NACL::STask::Vserver->find_data_vservers(
            command_interface => $args{'node_obj'},
            allow_empty       => 0,
            filter            => { vserver => $vserver_name }); 1;
    };

    # Delete vserver if no root volume existed
    if ($evl_return) {
        $vserver_name = $temp_vserver->vserver();
        if ($temp_vserver->get_one_state_attribute('rootvolume') eq q{-}) {
            $Log->comment('==No root volume found, Delete existing vserver==');
            $temp_vserver->delete();
        }
    } else {
        # No vserver find at this point so, generate vserver name unless
        # vserver name defined.
        $vserver_name = defined($args{'vserver'}) ? $args{'vserver'} :
                              random_name_generator( prefix => 'vs', size => 1);
    }

    my $root_vol_name = 'root_' . "$vserver_name";
    $root_vol_name =~ s/-/_/g;
    my $vserver_obj = NACL::STask::Vserver->create(
            command_interface            => $args{'node_obj'},
            vserver                      => "$vserver_name",
            aggregate                    => $args{'aggregate'},
            rootvolume                   => $root_vol_name,
            'ns-switch'                  => ['file','nis'],
            'nm-switch'                  => ['file'],
            'rootvolume-security-style'  => 'mixed',
            'language'                   => 'en_US',
            'snapshot-policy'            => 'default',
            'antivirus-on-access-policy' => 'default',
            nacltask_if_exists           => 'reuse');
    $Log->comment('Vserver under test  : ' . $vserver_obj->vserver());

    $Log->exit() if $may_exit;

    return $vserver_obj;
}


=head2 create_nfs_cifs_data_lifs()

    my $data_lifs_ref =
    NACL::MTask::MCC::DPSASUtils->create_nfs_cifs_data_lifs(...);

    Wrapper function to create nfs,cifs data lifs

    Input Params:
    node_obj  => [NACL::C::SystemNode object]
    vserver   => [Vserver name string ( default = vs0 )]

    Return:
    Array reference of data lif objects

=cut

sub create_nfs_cifs_data_lifs {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            node_obj => { type => HASHREF },
            vserver  => { type => SCALAR | UNDEF, optional => 1,
                                                  default  => 'vs0'}
        }
    );

    $Log->enter() if $may_enter;
    $Log->comment('Create NFS/CIFS data LIFs');
    my ($port_ip_hash_ref, $gateway_ip, $netmask) =
        $pkg->get_data_port_ip_gw_netmask( node_obj => $args{'node_obj'} );

    my (@data_lifs, @port_ip_pairs);
    while ( my ($key, $value) = each(%{$port_ip_hash_ref}) ) {
        my $port_ip = ("$key:$value");
        push @port_ip_pairs, $port_ip;
    }

    foreach my $port_ip_pair (@port_ip_pairs) {
        my @port_ip = split /:/ , $port_ip_pair;
        my $home_port = shift(@port_ip);
        my $ip_address = shift(@port_ip);

        my $lif_name = random_name_generator(
                prefix => $args{'node_obj'}->node(),
                size => 1);

        my $lif = NACL::STask::NetworkInterface->create(
                command_interface  => $args{'node_obj'},
                lif                => $lif_name,
                vserver            => $args{'vserver'},
                role               => 'data',
               'home-node'         => $args{'node_obj'}->node(),
               'home-port'         => $home_port,
                address            => $ip_address,
                netmask            => $netmask,
                nacltask_if_exists => 'reuse');
        push @data_lifs, $lif;
    }
    $Log->exit() if $may_exit;

    return \@data_lifs;
}


=head2 create_iscsi_data_lifs()

    my $iscsi_data_lifs_ref =
    NACL::MTask::MCC::DPSASUtils->create_iscsi_data_lifs(...);

    Wrapper function to create iscsi data lifs

    Input Params:
    node_obj  => [NACL::C::SystemNode object]
    vserver   => [Vserver name string ( default = vs0 )]

    Return:
    Array reference of iscsi data lif objects

=cut

sub create_iscsi_data_lifs {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            node_obj => { type => HASHREF },
            vserver  => { type => SCALAR | UNDEF, optional => 1,
                                                  default  => 'vs0'}
        }
    );

    $Log->enter() if $may_enter;
    $Log->comment('Create iSCSI data LIFs');
    my ($port_ip_hash_ref, $gateway_ip, $netmask) =
        $pkg->get_data_port_ip_gw_netmask( node_obj => $args{'node_obj'} );

    my (@iscsi_data_lifs, @port_ip_pairs);
    while ( my ($key, $value) = each(%{$port_ip_hash_ref}) ) {
        my $port_ip = ("$key:$value");
        push @port_ip_pairs, $port_ip;
    }

    foreach my $port_ip_pair (@port_ip_pairs) {
        my @port_ip = split /:/ , $port_ip_pair;
        my $home_port = shift(@port_ip);
        my $ip_address = shift(@port_ip);

        my $lif_name = random_name_generator(
                prefix => $args{'node_obj'}->node(),
                size => 1);

        my $iscsi_lif = NACL::STask::NetworkInterface->create(
                command_interface  => $args{'node_obj'},
                lif                => $lif_name,
                vserver            => $args{'vserver'},
                role               => 'data',
               'data-protocol'     => ['iscsi'],
               'home-node'         => $args{'node_obj'}->node(),
               'home-port'         => $home_port,
                address            => $ip_address,
                netmask            => $netmask,
                nacltask_if_exists => 'reuse');
        push @iscsi_data_lifs, $iscsi_lif;
    }
    $Log->exit() if $may_exit;

    return \@iscsi_data_lifs;
}


=head2 create_routing_group_route()

    my $route =
    NACL::MTask::MCC::DPSASUtils->create_routing_group_route(...);

    Wrapper function to create routing group route on vserver under test

    Input Params:
    node_obj  => [NACL::C::SystemNode object]
    vserver   => [Vserver name string( default = vs0 )]

    Return:
    Array reference of route objects

=cut

sub create_routing_group_route {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            node_obj => { type => HASHREF },
            vserver  => { type => SCALAR | UNDEF, optional => 1,
                                                  default  => 'vs0'}
        }
    );
    $Log->enter() if $may_enter;
    # Skipping network route creation
    # The gx_boot already took care of this task.
    $Log->exit() if $may_exit;

    return;
}


=head2 create_nfs_and_export_policy_rule()

    my $export_policy =
    NACL::MTask::MCC::DPSASUtils->create_nfs_and_export_policy_rule(...);

    Wrapper function to create NFS and Export Policy rule
    Input Params:
    node_obj       => [NACL::C::SystemNode object]
    vserver        => [Vserver name string( default = vs0 )]
    enable_nfs_v40 => [Boolean to enable nfs v4.0 ( default = 0 )]

    Return:
    Scalar reference of the export_policy_rule

=cut

sub create_nfs_and_export_policy_rule {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            node_obj       => { type => HASHREF },
            enable_nfs_v40 => { regex => qr/^[01]$/, default => 0 },
            vserver        => { type => SCALAR | UNDEF, optional => 1,
                                                  default  => 'vs0' }
        }
    );

    $Log->enter() if $may_enter;
    my $vserver_name = !defined($args{'vserver'}) ? 'vs0' : $args{'vserver'};
    # Turn on vserver NFS service
    my $vserver_nfs = NACL::STask::VserverNfs->create(
            command_interface => $args{'node_obj'},
            vserver           => $vserver_name,
            if_exists         => 'reuse',
            tcp               => 'enabled',
            udp               => 'enabled');

    # Modify existing vserver nfs service
    if ($args{'enable_nfs_v40'}) {
        $vserver_nfs->modify(
            access => 'true',
            'v4.0' => 'true',
            'v4.1' => 'true' );
    }

    # Create vserver export-policy rule
    my $export_policy_rule = NACL::C::VserverExportPolicyRule->create(
            command_interface => $args{'node_obj'},
            vserver           => $vserver_name,
            policyname        => 'default',
            'clientmatch'     => '0.0.0.0/0',
            rwrule            => ['any'],
            rorule            => ['any'],
            protocol          => ['any'],
            superuser         => ['any'],);

    #Wait 30 seconds to establish export policy rule
    $Log->exit() if $may_exit;

    return $export_policy_rule;
}


=head2 get_vserver_obj()

    my $vserver_obj = NACL::MTask::MCC::DPSASUtils->get_vserver_obj(...);

    Wrapper function to Fetch existing (cluster|data) vservers and return a
    first (cluster|data) vserver object

    Input Params:
    node_obj  => [NACL::C::SystemNode object]
    vserver   => [Vserver name string( default = undef )]

    Return:
    Scalar value of vserver object

=cut

sub get_vserver_obj {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            node_obj => { type => HASHREF },
            vserver  => { type => SCALAR | UNDEF, optional => 1 }
        }
    );

    $Log->enter() if $may_enter;
    my $vserver_name = !defined($args{'vserver'}) ? undef : $args{'vserver'};
    my $vserver_obj = NACL::STask::Vserver->find_data_vservers(
            command_interface => $args{'node_obj'},
            allow_empty       => 0,
            filter            => { vserver => $vserver_name });

    $Log->comment('Vserver under test    : ' . $vserver_obj->{'vserver'}
             . ' ( ' . $args{'node_obj'}->name() . ' )');

    $Log->exit() if $may_exit;
    return $vserver_obj;
}


=head2 get_nfs_cifs_data_lifs()

    my $nfs_lif_objs = NACL::MTask::MCC::DPSASUtils->get_nfs_cifs_data_lifs(..);

    Wrapper function to fetch NFS/CIFS data LIF ip addresses

    Input Params:
    node_obj       => [NACL::C::SystemNode object]
    vserver        => [Vserver name string( default = vs0 )]
    data_protocol  => [string value 'nfs' or 'cifs' ~ default = 'nfs']
    address_family => [string value 'ipv4' or 'ipv6' ~ default = 'ipv4']

    Return:
    Array reference of lif objects

=cut

sub get_nfs_cifs_data_lifs {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            node_obj       => { type => HASHREF },
            vserver        => { type => SCALAR | UNDEF, optional => 1 },
            data_protocol  => { regex => '^(nfs|cifs)$', default => 'nfs' },
            address_family => { regex => '^(ipv4|ipv6)$', default => 'ipv4' }
        }
    );

    $Log->enter() if $may_enter;
    my $vserver_name = !defined($args{'vserver'}) ? 'vs0' : $args{'vserver'};
    my @lifs = NACL::CS::Interface->fetch(
            command_interface  => $args{'node_obj'},
            filter => {
                vserver         => $vserver_name,
                role            => 'data',
               'address-family' => $args{'address_family'},
               'data-protocol'  => [$args{'data_protocol'}],
               'status-admin'   => 'up'});

    foreach my $lif (@lifs) {
        $Log->comment('NAS data LIF ip address : ' . $lif->address()
            . ' ( ' . $lif->curr_port() . ' ~ ' . $lif->curr_node() . ' )');
    }
    $Log->exit() if $may_exit;
    return \@lifs;
}


=head2 get_iscsi_data_lifs()

    my $iscsi_lif_objs = NACL::MTask::MCC::DPSASUtils->get_iscsi_data_lifs(...);

    Wrapper function to fetch iSCSI data LIF ip addresses

    Input Params:
    node_obj  => [NACL::C::SystemNode object]
    vserver   => [Vserver name string( default = vs0 )]
    address_family => [string value 'ipv4' or 'ipv6' ~ default = 'ipv4']

    Return:
    Array reference of iscsi lif objects

=cut

sub get_iscsi_data_lifs {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            node_obj => { type => HASHREF },
            vserver  => { type => SCALAR | UNDEF, optional => 1 },
            address_family => { regex => '^(ipv4|ipv6)$', default => 'ipv4' }
        }
    );

    $Log->enter() if $may_enter;
    my $vserver_name = !defined($args{'vserver'}) ? 'vs0' : $args{'vserver'};
    my @iscsi_lifs = NACL::CS::Interface->fetch(
            command_interface  => $args{'node_obj'},
            filter => {
                vserver         => $vserver_name,
                role            => 'data',
               'address-family' => $args{'address_family'},
               'data-protocol'  => ['iscsi'],
               'status-admin'   => 'up'});

    foreach my $iscsi_lif (@iscsi_lifs) {
        $Log->comment('iSCSI data LIF ip address : ' . $iscsi_lif->address()
        .' ( ' . $iscsi_lif->curr_port() .' ~ '. $iscsi_lif->curr_node() .' )');
    }
    $Log->exit() if $may_exit;
    return \@iscsi_lifs;
}


=head2 get_iscsi_initiator_name()

    my $initiator =
    NACL::MTask::MCC::DPSASUtils->get_iscsi_initiator_name(...);

    Wrapper function to aquire initiator iqn name from client nate_host file

    Input Params:
    client_obj  => [NACL::C::Client object]

    Return:
    Scalar value of initiator iqn name

=cut

sub get_iscsi_initiator_name {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            client_obj => { type => HASHREF }
        }
    );

    $Log->enter() if $may_enter;

    $Log->comment('Get initiator iqn name from client nate_host file');
    my $initiator_name = $args{'client_obj'}->{hostrec}->{'ISCSI_WWNN'};
    if ($initiator_name =~ /iqn./i) {
       $initiator_name =~ s/^\s+|\s+$//g;
    } else {
        confess("\n\nERROR: add ISCSI_WWNN=iqn.1991-05.com.microsoft|redhat:" .
                $args{'client_obj'}->name() . ' in to ' .
                $args{'client_obj'}->name() . "\'s nate_host file\n\n" );
    }

    $Log->exit() if $may_exit;

    return $initiator_name;
}


=head2 get_intercluster_lifs ()

    my $intercluster_lifs =
    NACL::MTask::MCC::DPSASUtils->get_intercluster_lifs(...);

    Wrapper function to fetch both intercluster lifs from MCC clusters

    Input Params:
    node_obj  => [NACL::C::SystemNode object]

    Return:
    Array reference of intercluster lif objects

=cut

sub get_intercluster_lifs {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            node_obj => { type => HASHREF }
        }
    );

    $Log->enter() if $may_enter;
    my @intercluster_lifs = NACL::CS::Interface->fetch(
            command_interface  => $args{'node_obj'},
            filter             => { role  => 'intercluster' });

    foreach my $lif (@intercluster_lifs) {
        $Log->comment('Intercluster LIF name : ' . $lif->lif());
    }
    $Log->exit() if $may_exit;
    return \@intercluster_lifs;
}


=head2 get_mcc_operation_status ()

    my ($elapsed_time, $state, $operation) =
    NACL::MTask::MCC::DPSASUtils->get_mcc_operation_status(...);

    Wrapper function to fetch metrocluster operation elapsed time from
    "metrocluster operation show" command output

    Input Params:
    node_obj  => [NACL::C::SystemNode object]

    Return:
    Metrocluster operation elapsed time (epoch time)
    Metrocluster operation state (scalar)
    Metrocluster operation type (scalar)

=cut

sub get_mcc_operation_status {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            node_obj => { type => HASHREF }
        }
    );

    $Log->enter() if $may_enter;

    # metrocluster operation show
    my $oper_show = $args{'node_obj'}->apiset(interface => 'CLI')->metrocluster_operation_show();
    my $status = ${$oper_show->get_parsed_output()}[0];

    # get operation state, operation type
    my $operation  = $status->{'operation'};
    my $state      = $status->{'state'};
    my $start_time = $status->{'start-time'};
    my $end_time   = $status->{'end-time'};
    my $elapsed_time;

    if (!defined($operation)) {
        $Log->comment('MCC is not configured.');
        $elapsed_time = undef;
        $state        = undef;
        $operation    = undef;
    } else {
        # Remove leading (") and trailing (")
        $start_time =~ s/^"|"$//g;
        $end_time   =~ s/^"|"$//g;
        $operation  =~ s/^"|"$//g;

        # If operation did not Successful then "0" elapsed time
        if ($state !~ /^Successful/i) {
            $start_time = '3/17/2013 04:38:08';
            $end_time   = '3/17/2013 04:38:08';
        }

        my ($mo, $day, $yr, $hr, $min, $sec) =
                $start_time =~ /^(\d+)\/(\d+)\/(\d+)\s+(.{2}):(.{2}):(.{2})$/x;
        my ($emo, $eday, $eyr, $ehr, $emin, $esec) =
                $end_time =~ /^(\d+)\/(\d+)\/(\d+)\s+(.{2}):(.{2}):(.{2})$/x;

        # get operation elapsed time
        my $start_time_secs = timegm($sec, $min, $hr, $day, $mo-1, $yr);
        my $end_time_secs  = timegm($esec, $emin, $ehr, $eday, $emo-1, $eyr);
        $elapsed_time = ($end_time_secs - $start_time_secs);
    }
    $Log->exit() if $may_exit;

    return ($elapsed_time, $state, $operation);
}


=head2 enable_dedupe_and_compression ()

    my $vol_efficiency_objs =
    NACL::MTask::MCC::DPSASUtils->enable_dedupe_and_compression(
                                            vol_objs_ref => \@volume_objs );

    Wrapper function to enable dedupe and compression on each volumes

    Input Params:
    vol_objs_ref       => [Array reference of volume objects]
    compression        => [string value 'true' to turn on compression]
    inline_compression => [string value 'true' to turn on inline compression]

    Return:
    Array reference of efficiency enabled volumes

=cut

sub enable_dedupe_and_compression {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            vol_objs_ref => { type => ARRAYREF },
            compression  => { regex => '^(true|false)$', default => 'false' },
            inline_compression => {
                                regex => '^(true|false)$', default => 'false' }
        }
    );

    $Log->enter() if $may_enter;
    $Log->comment('creating volume efficiency policy');
    my $policy_name = random_name_generator( prefix => 'Policy', size => 1);
    my $policy = NACL::C::VolumeEfficiencyPolicy->create(
          command_interface => ${$args{'vol_objs_ref'}}[0]->command_interface(),
          vserver           => ${$args{'vol_objs_ref'}}[0]->vserver(),
          policy            => $policy_name,
          schedule          => '5min',
          duration          => '12', #12 hours
          enabled           => 'true',
          'method-timeout'  => 60);

    # Enable dedupe
    my @vol_efficiency_objs;
    foreach my $vol_obj (@{$args{'vol_objs_ref'}}) {
        my $vol_efficiency = NACL::STask::VolumeEfficiency->on(
            command_interface => $vol_obj->command_interface(),
            vserver           => $vol_obj->vserver(),
            volume            => $vol_obj->volume(),
            ignore_enabled    => 1);

        # Volume efficiency start
        $Log->comment('Start dedupe on volume ' . $vol_obj->volume());
        $vol_efficiency->start( nacltask_wait => 1 );

        # Enable Compression
        if ($args{'compression'} =~ /true/i) {
            $Log->comment('Enable compression on volume '
                         . $vol_efficiency->get_volume());

            $vol_efficiency->config(
                compression             => 'true',
               'inline-compression'     => $args{'inline_compression'},
               'background-compression' => 'true',
                policy                  => $policy);
        }
        push @vol_efficiency_objs, $vol_efficiency;
    }
    $Log->exit() if $may_exit;
    return (\@vol_efficiency_objs, $policy);
}


=head2 powercycle_disk_shelf ()

    NACL::MTask::MCC::DPSASUtils->powercycle_disk_shelf(...);

    A function to reboot given disk shelf or all disk shelves from given node.
    if shelf_number param did not provide then script will reboot all shelves.

    Input Params:
    node_obj     => [NACL::C::Node object]
    client_obj   => [NACL::C::Client object]
    shelf_number => [Shelf number integer value (optional)]

    Return:
    none

=cut

sub powercycle_disk_shelf {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            node_obj     => { type => HASHREF },
            client_obj   => { type => HASHREF },
            shelf_number => { type => SCALAR | UNDEF, optional => 1 }
        }
    );
    $Log->enter() if $may_enter;

    my $response;
    my $number = !defined($args{'shelf_number'}) ? 1 : $args{'shelf_number'};

    while ( $response !~ /Node not found/ ) {
        my $cmd = '/usr/software/utils/bin/rpower '
                  . $args{'node_obj'}->name() . '-ds' . $number;

        $Log->comment($cmd);
        my $evl_return = eval {
            $response = $args{'client_obj'}->execute(command => $cmd); 1;
        };
        $Log->comment('Waiting 600 seconds for reinstatement of disk shelf');
        Tharn::snooze(600);
        if ( defined($args{'shelf_number'}) ) { last; }
        $number++;
    }
    $Log->exit() if $may_exit;

    return;
}


=head2 powercycle_armadillo ()

    my $armadillo = NACL::MTask::MCC::DPSASUtils->powercycle_armadillo(
            armadillo_name => dpg-mcc-atto-00-A-fabX-0,
            conn_type      => rconsole );
    ( or )
    my $armadillo = NACL::MTask::MCC::DPSASUtils->powercycle_armadillo(
            armadillo_name => $node->{hostrec}->{'ARMADILLO'} );

    $Test->nlog('Armadillo bridge name : ' . ${$armadillo}->{'host'});

    A wrapper function to reboot Armadillo Fibre Channel(FC) to
    Serial Attached SCSI (SAS) bridge.

    Input Params:
    armadillo_name => [Armadillo name string ~ e.g. dpg-mcc-atto-00-A-fabX-00]
    conn_type      => [Connection type telnet|console|rconsole ~ default=telnet]

    Return:
    Scalar reference of Armadillo bridge object

=cut

sub powercycle_armadillo {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            armadillo_name => { type  => SCALAR },
            conn_type      => { regex => '^(telnet|console|rconsole)$',
                                         default => 'telnet' }
        }
    );
    $Log->enter() if $may_enter;

    # Create Armadillo object
    my $armadillo_obj = Armadillo->new(
                            -bridge    => $args{'armadillo_name'},
                            -conn_type => $args{'conn_type'} );
    if ($armadillo_obj->error) {
        $Log->comment('FAILED to create Armadillo object : '
                                            . $armadillo_obj->errmsg());
    }
    $Log->comment('Armadillo bridge name : ' . $armadillo_obj->{'host'});
    my $sas_targets_ref = $armadillo_obj->get_disk_logins();
    $Log->comment('SAS targets count : ' . scalar(@{$sas_targets_ref}));

    # Power cycle Armadillo bridge
    $Log->comment('Initiate '.$armadillo_obj->{'host'}.' and do powercycle');
    if (!$armadillo_obj->reboot()) {
        $Log->comment('FAILED to reboot Armadillo ' .$armadillo_obj->{'host'}
            . 'ERROR:  ' . $armadillo_obj->errmsg() );
    } else {
        $Log->comment('Successfully power cycled : '. $armadillo_obj->{'host'});
        # Wait for restore power to Armadillo bridge
        Tharn::snooze(60);
        my $sas_targets_rebooted = $armadillo_obj->get_disk_logins();
        $Log->comment('SAS targets count after reboot: '
                    . scalar(@{$sas_targets_rebooted}));
        # verify SAS targets after rebbot the Armadillo
        if (scalar(@{$sas_targets_ref}) != scalar(@{$sas_targets_rebooted})) {
            confess('SAS target disk logins count mismatch after rebooted');
        }
    }

    $Log->exit() if $may_exit;

    return \$armadillo_obj ;
}


=head2 data_populate()

    # Data populate
    NACL::MTask::MCC::DPSASUtils->data_populate(
                    vol_objs_ref     => \@volume_objs, or [$volume_objs[0]],
                    clients_ref      => \@clients,
                    percent          => 5,
                    increase_volume  => 1,
                    mounted_vols_ref => $mounted_vol_objs_ref );

    Wrapper function to populate basic data on given volumes in same vservers
    from multiple clients.

    Input Params:
    vol_objs_ref     => [Array reference of volume objects]
    clients_ref      => [Array reference of nfs client objects]
    percent          => [Scalar, plain integer ~ default = 5]
    increase_volume  => [boolean value to increase volume size ~ dedault = 0]

    Return:
    None

=cut

sub data_populate {
    my ($pkg, @args) = @_;
    my %args = validate(
        @args,
        {
            vol_objs_ref     => { type => ARRAYREF },
            clients_ref      => { type => ARRAYREF },
            percent          => { regex => qr/^\d+$/, default => 5 },
            increase_volume  => { regex => qr/^[01]$/, default => 0 }
        }
    );
    $Log->enter() if $may_enter;

    $Log->comment('Populate data');
    my @mounted_vol_objs;

    foreach my $vol_obj (@{$args{'vol_objs_ref'}}) {
        my %mount_params;

        # Get a round-robin client from a list of available clients
        my $client_obj =  shift(@{$args{'clients_ref'}});
        push @{$args{'clients_ref'}}, $client_obj;

        my $client_type = $client_obj->type();
        if ($client_type =~ /unix/) {
            %mount_params = ( volume => $vol_obj );

        } elsif ($client_type =~ /windows/) {
            my $share_name = 'dpg_cifs_share_' . $vol_obj->volume();

            my $share = NACL::STask::VserverCifsShare->create(
                    command_interface  => $vol_obj->command_interface(),
                    vserver            => $vol_obj->vserver(),
                    'share-name'       => $share_name,
                    nacltask_if_exists => 'reuse',
                    path               => $vol_obj->get_one_state_attribute(
                                                          'junction-path'));
                    %mount_params = ( share => $share );
        }

        # Re use existing mount point if possible
        my $mount_obj;
        if (defined($vol_obj->{'mount'}->{'mount_point'})) {
            $mount_obj = NACL::STask::Client::Mount->create(
                            command_interface  => $client_obj,
                            nacltask_if_exists => 'reuse',
                            mount_point => $vol_obj->{'mount'}->{'mount_point'},
                            %mount_params );
        } else {
            $mount_obj = NACL::STask::Client::Mount->create(
                            command_interface  => $client_obj,
                            nacltask_if_exists => 'reuse',
                            %mount_params );
        }

        # Update client_obj and mount_obj in to volume
        $vol_obj->{client} = $client_obj;
        $vol_obj->{mount}  = $mount_obj;
        push @mounted_vol_objs, $vol_obj;
    }

    # Populate data on volumes ( percent of space left )
    foreach my $volobj (@mounted_vol_objs) {
            my $mountobj = $volobj->{mount};
            my $space_left = $volobj->state->size() - $volobj->state->used();
            my $percent_size = int($space_left * $args{'percent'} / 100);

            # Miniumm 1MB to be populated
            my $size_to_populate = $percent_size + 1048576;

            $Log->comment('Mount path            = '. $mountobj->mount_point());
            $Log->comment('Volume size           = ' . $volobj->state->size());
            $Log->comment('Used size             = ' . $volobj->state->used());
            $Log->comment('Space left            = ' . $space_left);
            $Log->comment('Data size to populate = ' . $size_to_populate);

            # Volume size increase when size less than 1.5 Gigabyte
            my $volume_full_flag = 0;
            if ($space_left < 1610612736) {
                $Log->comment('Volume ' . $volobj->volume() .
                    ' space less than 1 Gigabyte left. Increase volume size!');

                # Minimum 1 Gigabyte increase volume size
                my $new_size = 1073741824 + $volobj->state->size();

                # Modify volume
                if ($args{'increase_volume'}) {
                    $Log->comment('Increasing volume size');
                    $volobj->size('new-size' => $new_size);
                    $Log->comment('Volume size  = ' . $volobj->state->size());
                } else {
                    $volume_full_flag = 1;
                }
            }

            if ($volume_full_flag) {
                $Log->comment('Skipping data populate block');
                last;
            } else {
                my $max = 1024; #maximum 1024 files
                my $min = 10;   #minimum 10 files
                my $num_files = int(rand($max)) + $min;
                my $file_size = int($size_to_populate / $num_files);

                $Log->comment('Creating ' . $args{'percent'} . '% of data ~ ' .
                       $num_files . ' files x ' . $file_size . 'KB on ' .
                       $volobj->volume());

                # Modify mount point for read/write access
                 if($mountobj->command_interface->hostrec->is_unix()){
                        $mountobj->command_interface()->apiset(interface => 'CLI')->chmod(
                               'privilege-level' => 'root',
                                paths            => $mountobj->mount_point(),
                                mode             => 'a+rwx' );
                }

                # Populate data block
                my $result = eval {
                    my $populate_obj = NACL::MTask::Data->populate(
                        command_interface => $mountobj->command_interface(),
                        path        => $mountobj->mount_point() . '/BASIC_DATA',
                        file_avg          => $num_files,
                        file_size_avg     => $file_size,
                        name_avg          => 50,
                        dir_avg           => 0,
                        depth_avg         => 0,
                        file_dev          => 0,
                        dir_dev           => 0,
                        depth_dev         => 0,
                        file_size_dev     => 0 ); 1;
                };
                $Log->comment('Data populate completed');
                if ($mountobj->command_interface()->type() =~ /unix/) {
                    $Log->comment("The volume usage: \n" .
                      $mountobj->command_interface()->
                      apiset(interface => 'CLI')->execute_command(
                            command => 'df -ahP ' . $mountobj->mount_point() ));
                } else {
                    $Log->comment("The volume usage: \n" .
                      $mountobj->command_interface()->
                      apiset(interface => 'CLI')->execute_command(
                            command => 'dir ' . $mountobj->mount_point() ));
                }
            }
    }
    $Log->exit() if $may_exit;
    return;
}


=head2 get_reachable_data_lifs ()

    NACL::MTask::MCC::DPSASUtils->get_reachable_data_lifs(...);

    # example
    my $data_lifs_ref = NACL::MTask::MCC::DPSASUtils->get_reachable_data_lifs(
            node_obj     => $node,
            vserver_name => $vserver_name,
            client_obj   => $clients );

    A function to ping from client under test to vserver under test's data lifs

    Input Params:
    node_obj     => [NACL::C::Node object]
    vserver_name => [vserver name string]
    client_obj   => [NACL::C::Client object]

    Return:
    Array ref of reachable data lifs

=cut

sub get_reachable_data_lifs {
    my $pkg = shift;

    my %args = validate_with(
        params => \@_,
        spec => {
                node_obj     => { type => HASHREF },
                vserver_name => { type => SCALAR },
                client_obj   => { type => HASHREF }
                },
        allow_extra => 1
    );

    my $node_obj     = delete $args{'node_obj'};
    my $vserver_name = delete $args{'vserver_name'};
    my $client_obj   = delete $args{'client_obj'};

    $Log->enter() if $may_enter;
    my @data_lifs_under_test;
    # Find vservers with data lifs
    my @data_lifs = NACL::CS::NetworkInterface->fetch(
            command_interface => $node_obj,
            filter => { role          => 'data',
                        vserver       => $vserver_name,
                       'status-admin' => 'up',
                       %args } );

    if (!defined($data_lifs[0]->vserver())) {
        NATE::BaseException->throw('No data Lifs available in vserver');
    }
    my $count = 0;
    foreach my $data_lif (@data_lifs) {
        $count++;
        # Ping vserver's data lif ip from client under test
        $Log->comment("$count)Ping " . $client_obj->name() . '  --to-->  LIF ' .
                        $data_lif->address() . '  ~ ( ' .$vserver_name . ' )' );

        my $output = $client_obj->apiset(interface => 'CLI')->execute_command(
                command          => 'ping -c 5 ' . $data_lif->address(),
               'privilege-level' => 'root' );

        if (($output =~ /\s100% packet loss/) || ($output =~ /unknown/i)) {
            $Log->comment('*** Unreachable IP: ' . $data_lif->address() .
                                ' unreachable from ' . $client_obj->name() );
        } else {
            push @data_lifs_under_test, $data_lif;
        }
    }

    if (!scalar @data_lifs_under_test) {
        NATE::BaseException->throw('"Data LIF unreachable from client"');
    } else {
        $Log->comment('Reachable LIF found = ' . scalar @data_lifs_under_test);
    }
    $Log->exit() if $may_exit;

    return \@data_lifs_under_test;
}
1;
#END
