#
## @summary Timer object, created for each method for tracing
## @status   Public
## @pod      here
## @author   dl-dpgqa-automation@netapp.com

=head1 NAME

NACL::MTask::MCC::Log::Timer

=head1 DESCRIPTION

This is the NACL::MTask::MCC::Log::Timer class used by NACL::MTask::MCC::Log to time each method.
In order to do this, each call has been instrumented by creating an
object of this type using the helper function NACL::MTask::MCC::Log->start_timer()

In theory this class could be used by itself, but the caller and
called by values currently assume they are being used by an
intermediate function so they will return incorrect values until
this is fixed (see TODO in code)

=head1 METHODS

=cut


package NACL::MTask::MCC::Log::Timer;

use Time::HiRes qw(gettimeofday tv_interval);

use strict;
use warnings;
no warnings qw/ redefine /;

###############################################################################
#
#   Sub Name:      new
#
#   Description:   creates new NACL::MTask::MCC::Log::Timer object
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $class    IN      scalar    this class name
#                   %args     IN      hash      hash containing initial values,
#                                               namely logger (NACL::MTask::MCC::Log ref)
#   Globals:
#       $NACL::MTask::MCC::Log::TRACE     IN      scalar    trace log type
#
#   Environment:
#
#   Returns:        Success:  object of this class
#                   Failure:
#
###############################################################################

sub new {
    my ($class, %args) = @_;

    my $start = [gettimeofday()];

    # TODO
    # later we may allow fallback to printing?
    #if (!$args{logger}->isa("NACL::MTask::MCC::Log")) {
        #warn "logger not passed a valid Log object, defaulting to print...";
    #}

    # remember that by default caller
    # now expects the direct caller to be the method being timed.
    # Now that there is a mechanism to pass in the caller NACL::MTask::MCC::LOG
    # is doing that appropriately for the correct depth (0,1,2 for it
    # which would translate to 1,2,3 for here if we were still doing it here))

    #in some cases, the perl optimizer can optimize out info
    #for call frames > 1.  Because we are creating this in NACL::MTask::MCC::Log
    #for the user, it adds another level and could in theory generate
    #incorrect data for us.  Thus, we now pass in the call frame
    #from NACL::MTask::MCC::Log where it is only 1 rather than 2
    #
    #yes, it can be counterintuitive that the package the caller/sub
    #is in is not from the same call frame.  check if you want, but
    #the package in 'caller' is the package that CALLED the caller,
    #not the package the caller is in.  So, you need to look 1 frame
    #lower to see what package called us, rather than who called
    #the call...
    my $inpackage = $args{inpackage} || [caller 0];
    my $incall = $args{incall} || [caller 1];
    my $calledby = $args{calledby} || [caller 2];

    my $self = {
                   logger    => $args{logger},
                   start     => $start,
                   inpackage => $inpackage,  # get package sub is in
                   incall    => $incall,     # get sub we are in
                   calledby  => $calledby ,  # who called that sub?
                   message   => delete $args{message},
               };
    bless $self,$class;

    #in case undefined as in main
    my $calledby_name = $self->{calledby}[3] || '(no caller)';

    # log entry line, xxx called by yyy
    $self->{logger}->log(
        "[call=$self->{incall}[3]] [package=$self->{inpackage}[0]] [called by="
        . "$calledby_name]" .
       (defined $self->{message} ? " [message=$self->{message}]" : q{}),
       $NACL::MTask::MCC::Log::TRACE #this is a trace type
    );

    $self->{logger}->increment_call($self->{incall}[3], $self->{inpackage}[0],
                                    $calledby_name);

    return $self;

}

###############################################################################
#
#   Sub Name:      DESTROY
#
#   Description:   We use the destructor as a way to end the timer.  This
#                  depends on the timer being assigned to the appropriate
#                  variable type (ie 'my' usually)
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     IN      ref       Timer Object
#
#   Globals:
#       $NACL::MTask::MCC::Log::TRACE     IN      scalar    trace log type
#       $NACL::MTask::MCC::Log::TIMER     IN      scalar    timer log type
#
#   Environment:   valid && log_init'd NACL::MTask::MCC::Log object
#
#   Returns:        Success:  nothing
#                   Failure:
#
###############################################################################

sub DESTROY {
    my $self = shift;
    my $time = tv_interval ($self->{start});

    # log TIMER line, improve later to show sys, user time
    $self->{logger}->log(
        sprintf(
            '[call=%s] [seconds=%f]%s',
            $self->{incall}[3],
            $time,
            (defined $self->{message} ? " [message=$self->{message}]" : q{}),
        ),
        $NACL::MTask::MCC::Log::TIMER);
    $self->{logger}->increment_call_timer($self->{incall}[3],
                                          $self->{inpackage}[0],$time);

    # log exit line (trace)
    $self->{logger}->log("[exiting=$self->{incall}[3]]" .
       (defined $self->{message} ? " [message=$self->{message}]" : q{}),
       $NACL::MTask::MCC::Log::TRACE
    );

    return;
}

1;
