# $Id $

# this is here due to our standard usage of @_ in Params::Validate methods.
##no critic (ControlStructures::ProhibitUnlessBlocks)
##no critic (ValuesAndExpressions::ProhibitInterpolationOfLiterals)

#
# Copyright (c) 2011-2021 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroCluster Stub MTask Module
## @author dl-mcc-dpg@netapp.com
## @status shared
## @pod here

package NACL::MTask::MCC::Network::FibreBridge_Impl::Armadillo_Impl;

use strict;
use warnings;

use Carp;
use base qw (NACL::MTask::MTask);

use NACL::APISet;
use NACL::APISet::Switch;
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::MTask::MCC::Log;
use Readonly;

use Params::Validate qw(validate_with);


=head1 NAME

NACL::MTask::MCC::Network::FibreBridge_Impl::Armadillo_Impl

=head1 SYNOPSIS

  # Should not be used in user code... use FibreBridge instead!
  use NACL::MTask::MCC::Network::FibreBridge_Impl::Armadillo_Impl;

  my $fb =
    NACL::MTask::MCC::Network::FibreBridge_Impl::Armadillo_Impl->new(
      armadillo_name => $name );

=head1 DESCRIPTION

This is an implementation class for the Armadillo FibreBridge. It is
used by the FibreBridge class behind the scenes to access Armadillo
hardware.

=head1 METHODS

=head2 new(...)

Creates a new
NACL::MTask::MCC::Network::FibreBridge_Impl::Armadillo_Impl object.
With this object you can call other methods of this library.

=over

=item Argument

 Required: C<armadillo_name> string

=item Return Value

NACL::MTask::MCC::Network::Armadillo object

=back

=cut

use Class::MethodMaker [
    new => [ '-init', 'new' ],
    scalar => [ { -type => 'NACL::APISet::Switch' }, '_armadillo_apiset', ],
    scalar => [ '_armadillo_name' ],
];

sub init {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => { armadillo_name => { regex => qr/^[\w\-]+$/ }, },
    );

    my $armadillo_name = delete( $opts{armadillo_name} );

    my $apiset = $self->_create_apiset($armadillo_name);

    $self->_armadillo_apiset($apiset);

    return;
}

##
## _create_apiset()
##
## Creates the APIset for the Armadillo.
##
## Argument:
##   $armadillo_name -- String identifier for the armadillo
##
sub _create_apiset {
    my $self = shift;
    my ($armadillo_name) = @_;

    my $Host = host( $self->_armadillo_name($armadillo_name) );

    my $apiset = NACL::APISet->new(
        hostobj   => $Host,
        category  => 'Switch',
        interface => 'CLI',
        set       => 'Armadillo',
    );

    return $apiset;

}

##
## L_get_info()
##
## Returns a parsed hash of information about the Armadillo.
##
sub L_get_info {
    my $self = shift;
    return ( $self->_armadillo_apiset->info )->get_parsed_output();
}

##
## L_get_apiset()
##
## Returns the NACL apiset for Armadillo.
##
sub L_get_apiset {
    my $self = shift;

    return $self->_armadillo_apiset();
}

##
## L_get_name()
##
## Returns the name of the armadillo
##
sub L_get_name {
    my $self = shift;

    return $self->_armadillo_name();
}

##
## L_get_vendor_name()
##
## Returns the vendor name of the fibrebridge.
##
sub L_get_vendor_name {
    return 'netapp';
}

1;
