package NACL::MTask::MCC::Network::Port;

use strict;
use warnings;
use Carp;

use base qw (NACL::MTask::MTask);

use NACL::APISet;
use NACL::APISet::Switch;

use NACL::MTask::MCC::Network::Switch_Impl::BrocadeFC_Impl;
use NACL::MTask::MCC::Network::Switch_Impl::CiscoFC_Impl;

use NACL::MTask::MCC::Utils;

use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate
    qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR);

=head1 Port

NACL::MTask::MCC::Network::Port;

=head1 DESCRIPTION

This class is part of Metrocluster Network Topology.

=over

=item Examples

=back

    use NACL::MTask::MCC::Network::Port;

=cut

=head2 new

    my $port = NACL::MTask::MCC::Network::Port->new(
                                           port_descriptor => $port_descriptor,
                                           switch          => $switch);

This is the constructor of this class.

=over

=item Arguments

=over

=item C<< port_descriptor >>

(Required) This is a text identifier for a particular port on the switch.  The
form of this string may vary from switch make to switch make.

=item C<< switch >>

(Required) This is the NACL::MTask::MCC::Network::Switch object associated with
the switch this port is on.

=back

=back

=cut

use Class::MethodMaker [
    new    => [ '-init', 'new' ],
    scalar => [
	       { -type => 'NACL::MTask::MCC::Network::Switch'},
	       '_switch',
	       ],
    scalar => [
        qw/  _port_descriptor /,
              ],
                       ];

sub init {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            port_descriptor => { regex => qr/^(\d+|\d+\/\d+)$/ },
	        switch          => { isa   => 'NACL::MTask::MCC::Network::Switch' },
	              },
                            );

    $self->_port_descriptor($opts{port_descriptor});
    $self->_switch(         $opts{switch});

    $Log->exit() if $may_exit;
    return;
}

=head2 get_port_descriptor()

    my $port_descriptor = $port->get_port_descriptor();

Returns the port descriptor string.

=cut

sub get_port_descriptor {
    $Log->enter() if $may_enter;
    my $self = shift;

    $Log->exit() if $may_exit;
    return $self->_port_descriptor;
}

=head2 enable

    $port->enable();

Runs the command to enable the port and verifies execution.

No return value.

=cut

sub enable {
    $Log->enter() if $may_enter;
    my $self = shift;

    my $switch_impl     = $self->_switch()->get_switch_impl();
    my $port_identifier = $self->_port_descriptor();

    $switch_impl->L_port_enable(port_descriptor => $port_identifier);

    $Log->exit() if $may_exit;
    return;
}

=head2 is_enabled

    if ($port->is_enabled()) { ... };

Returns 0|1 signifying if the port is enabled or not.

=cut

sub is_enabled {
    $Log->enter() if $may_enter;
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics
    my $self = shift;

    my $switch_impl     = $self->_switch()->get_switch_impl();
    my $port_identifier = $self->_port_descriptor();

    my %opts = validate_with(
        params => \@_,
        spec   => {
                      stale_time   => { optional => 1 },
                  },
                            );

    $Log->exit() if $may_exit;
    return $switch_impl->L_port_is_enabled(
		port_descriptor => $port_identifier,
        %opts, #stale, if it's there...
    );
}

=head2 disable

    $port->disable();

Runs the command to disable the port and verifies execution.

No return value.

=cut

sub disable {
    $Log->enter() if $may_enter;
    my $self = shift;

    my $switch_impl     = $self->_switch()->get_switch_impl();
    my $port_identifier = $self->_port_descriptor();

    $switch_impl->L_port_disable(port_descriptor => $port_identifier);

    $Log->exit() if $may_exit;
    return;
}

=head2 get_target_host_hints_from_switch()

This method looks at the configuration of the switch that this port sits on,
and tries to get information about the target host that this port is connected
to.  This can be inaccurate information sometimes, as this method depends on
information entered by a network admin during installation.  If the switch is
moved or reconfigured, this information may become obsolete.

This information should be checked when validating a full MCC setup and flagged
if incorrect, or, outside of a full MCC test, a fabric might be able to be
constructed using this information.  See Fabric constructor for more
information.

Returns a hash of the same format as does
Switch_Impl::BrocadeFC_Impl::L_get_port_target_host_hints().

=cut

sub get_target_host_hints_from_switch {
    $Log->enter() if $may_enter;
    my $self = shift;

    $Log->exit() if $may_exit;
    return $self->_switch()->get_switch_impl()->L_get_port_target_host_hints(
                                 port_descriptor => $self->_port_descriptor());
}

=head2 get_switch()

    my $switch_name = $port->get_switch();

Returns the switch object the port is on.

=cut

sub get_switch {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my $self = shift;

    return $self->_switch();
}

#=head2 get_target_host()
#
#This method returns the object associated with the host this port is connected
#to (on the remote side of the connection.)  This may be one of the following:
#
#    NACL::MTask::MCC::Network::Node
#    NACL::MTask::MCC::Network::Switch
#    NACL::MTask::MCC::Network::FibreBridge
#
#If the target cannot be determined, then this method throws a
#NACL::MTask::MCC::Exceptions::BaseException.
#
#=cut
#
#sub get_target_host {
#    $Log->enter() if $may_enter;
#    my $self = shift;
#
#    my $switch_impl      = $self->_switch()->get_switch_impl();
#    my $target_host_info = $switch_impl->L_get_port_target_host_hints(
#                                 port_descriptor => $self->_port_descriptor());
#
#    if ($target_host_info->{host_type} eq 'node') {

#    } elsif ($target_host_info->{host_type} eq 'switch') {
#
#    } elsif ($target_host_info->{host_type} eq 'fibreBridge') {
#
#    }
#
#    unless (defined($target_host)) {
#        NACL::MTask::MCC::Exceptions::BaseException->throw(
#                                   NACL::MTask::MCC::Utils->dump($self) .
#                                              ":\n\tNo connected host found.");
#    }
#
#    $Log->exit() if $may_exit;
#    return $target_host;
#}

#=head2 is_connected()
#
#Returns true if there is a host seen on the remote side of the connection to
#this port.  This is a "safe" way of checking for the existence  a connected
#host, i.e., no exception is thrown here either way.  It will always return 0|1;
#
#=cut
#
#sub is_connected {
#    $Log->enter() if $may_enter;
#    my $self = shift;
#
#    my $b_is_connected = undef;
#    try {
#        $self->get_connection_target_host();
#        $b_is_connected = 1;
#    } catch NACL::MTask::MCC::Exceptions::BaseException with {
#        $b_is_connected = 0;
#    };
#
#    $Log->exit() if $may_exit;
#    return $b_is_connected;
#}

1;
