# this is here due to our standard usage of @_ in Params::Validate methods.
##no critic (RequireArgUnpacking)
##no critic (ControlStructures::ProhibitUnlessBlocks)
##no critic (ValuesAndExpressions::ProhibitInterpolationOfLiterals)

#
# Copyright (c) 2011-2021 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MCC Node Stub MTask Module
## @author dl-mcc-dpg-qa@netapp.com
## @status shared
## @pod here

package NACL::MTask::MCC::Node;

use strict;
use warnings;

use base qw (NACL::MTask::MTask);

## NACL component libs ##
use NACL::CS::Cluster;
use NACL::C::SystemNode;
use NACL::C::Metrocluster;
use NACL::CS::StorageDisk;
use NACL::CS::StorageAggregate;

## NACL Task libs ##
use NACL::MTask::MCC::Log;
use NACL::MTask::MCC::Aggregate;
use NACL::STask::StorageFailover;
use NACL::MTask::MCC::Exceptions::BaseException;

## Other libs ##
use Carp;
use NATE::BaseException qw(:try);
use NACL::Exceptions::NoElementsFound qw(:try);

use Tharn qw(subtest);
use NATE::Log qw(log_global);
my $Log       = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use Params::Validate
  qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR);

use Readonly;
use Data::Dumper;
use List::MoreUtils qw(any none);

=head1 NAME

NACL::MTask::MCC::Node

=head1 DESCRIPTION

This task is to be used for C-Mode MetroCluster testing.  The primary idea
behind the use of MCC::Nodes is that they guarantee the the command interface
used when making any call from this object is that of the associated filer
node.


    # Start test
    use NACL::MTask::MCC::Node;
    ...
TBD
    ...
    # Perform test here
    ...

    # End test

=cut

=head1 Public METHODS

There is no public constructor for this class.  A
NACL::MTask::MCC::Node object is obtained via calls such as

    my $node = TBD

=cut

# automatically generate a (Non-public) constructor (includes a call to 'init')
# and (Non-public) member accessor methods
use Class::MethodMaker [
    new => [ '-init', 'new' ],
    scalar => [ { -type => 'NACL::MTask::MCC::DR_Group' }, '_dr_group', ],
    scalar => [ { -type => 'NACL::MTask::MCC::HA_Group' }, '_ha_group', ],
    scalar => [ { -type => 'NACL::C::SystemNode' }, '_system_node_component', ],
    scalar => [
        { -type => 'NACL::MTask::MCC::Network::Node' },
        '_network_node_functionality',
    ],
    scalar => [ qw/ _name /, ],
    scalar => [ qw/ _adapters /, ],
    scalar => [ qw/ _transit_attempt_time /, ],
    scalar => [
        { q{*} => undef }, '_transit_obj',   #don't autogenerate accessor method
        { q{*} => undef },
        '_root_aggregate',                   #don't autogenerate accessor method
        { q{*} => undef }, '_systemid',      #don't autogenerate accessor method
        { q{*} => undef }, '_nvramid',       #don't autogenerate accessor method
        { q{*} => undef }, '_uuid',          #don't autogenerate accessor method
    ],
];

# validate the parameters passed in and what else?
sub init {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    # TBD?  We might want to allow construction via uuid or nvramid?
    my %opts = validate_with(
        params => \@_,
        spec   => {
            dr_group  => { isa   => 'NACL::MTask::MCC::DR_Group' },
            node_name => { regex => qr/^[\w\-]+$/ },
        },
    );

    # do we need to do anything else in here?
    my $dr_group  = delete( $opts{dr_group} );
    my $node_name = delete( $opts{node_name} );

    $self->_dr_group($dr_group);

    # this is temporary.  we cannot count on unchanging node names!
    $self->_name($node_name);

    my $the_Node_component =
      NACL::C::SystemNode->new( hostrec => Tharn::host($node_name) );
    $self->_system_node_component($the_Node_component);

    $self->_transit_attempt_time(0);

    $self->_init_node_info();

    my $transit_obj = $self->_create_transit_object();
    $self->_transit_obj($transit_obj);

    # we cannot initialize $self->_network_node_functionality until the
    # DR Group is finished constructing all MCC::Node objects.
    # $self->_network_node_functionality will be init'd in MCC::Node objects
    # at the end of the DR_Group constructor.  a call therein to
    # each $mcc_node->L_init_network_node_handle() will be used for this.

    return;
}

# in place of method maker's auto generated accessor...
sub _transit_obj {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    if ( scalar @_ ) {
        $self->{_transit_obj} = shift;    #set it if we need to
        return $self->{_transit_obj};
    }
    else {
        #return it if it's already here
        if ( defined $self->{_transit_obj} ) {
            return $self->{_transit_obj};
        }
        else {
            #try creating a new one and return it
            my $transit;
            if ( $transit = $self->_create_transit_object() ) {
                $self->{_transit_obj} = $transit;
            }
            return $transit;
        }
    }
}

# in place of method maker's auto generated accessor...
sub _root_aggregate {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    if ( scalar @_ ) {
        my $aggr = shift;
        if ( ref $aggr && $aggr->isa('NACL::MTask::MCC::Aggregate') ) {
            $self->{_root_aggregate} = $aggr;    #set it if we need to
            return $self->{_root_aggregate};
        }
        else {
            NACL::MTask::MCC::Exceptions::BaseException->throw(
                    'Invalid object type passed for _root_aggregate,'
                  . 'expected "NACL::MTask::MCC::Aggregate"' );
        }
    }
    else {
        #return it if it's already here
        if ( defined $self->{_root_aggregate} ) {
            return $self->{_root_aggregate};
        }
        else {
            #try creating a new one and return it
            my $root_aggr;
            if ( $root_aggr = $self->_determine_root_aggregate() ) {
                $self->{_root_aggregate} = $root_aggr;
            }
            return $root_aggr;
        }
    }
}

# in place of method maker's auto generated accessor...
sub _systemid {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    if ( scalar @_ ) {
        $self->{_systemid} = shift;    #set it if we need to
        return $self->{_systemid};
    }
    else {
        #return it if it's already here
        if ( defined $self->{_systemid} ) {
            return $self->{_systemid};
        }
        else {
            #try to init
            $self->_init_node_info();
            return $self->{_systemid};
        }
    }
}

# in place of method maker's auto generated accessor...
sub _nvramid {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    if ( scalar @_ ) {
        $self->{_nvramid} = shift;    #set it if we need to
        return $self->{_nvramid};
    }
    else {
        #return it if it's already here
        if ( defined $self->{_nvramid} ) {
            return $self->{_nvramid};
        }
        else {
            #try to init
            $self->_init_node_info();
            return $self->{_nvramid};
        }
    }
}

# in place of method maker's auto generated accessor...
sub _uuid {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    if ( scalar @_ ) {
        $self->{_uuid} = shift;    #set it if we need to
        return $self->{_uuid};
    }
    else {
        #return it if it's already here
        if ( defined $self->{_uuid} ) {
            return $self->{_uuid};
        }
        else {
            #try to init
            $self->_init_node_info();
            return $self->{_uuid};
        }
    }
}

# returns true if we're still waiting in a bad state to try again
sub _transit_attempt_should_wait {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    my $delta_transit_time = time - $self->_transit_attempt_time();
    my $skip_time          = 3 * 60;                                 #3 minutes?
    my $ignore_bad_state =
      $self->_dr_group()->_metrocluster()->_ignore_bad_state();

    if ( $ignore_bad_state && ( $delta_transit_time < $skip_time ) ) {
        return $skip_time - $delta_transit_time;
    }
    else {
        return 0;
    }

}

# helper method to init node info
sub _init_node_info {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    if ( my $wait_time = $self->_transit_attempt_should_wait() ) {
        $Log->comment( 'It looks like we are trying to repeatedly init'
              . ' node that is down while in bad start mode.'
              . ' Wait '
              . ($wait_time)
              . ' seconds before trying again' );
        return;
    }

    try {
        my $the_Node_component = $self->_system_node_component();
        $Log->comment("try getting node state now...");

        my $node_state =
          $the_Node_component->state(
            requested_fields => [ 'systemid', 'nvramid', 'uuid', ] );
        $self->_systemid( $node_state->systemid() );
        $self->_nvramid( $node_state->nvramid() );
        $self->_uuid( $node_state->uuid() );

        if ( !defined $self->_root_aggregate() ) {
            $self->_root_aggregate( $self->_determine_root_aggregate() );
        }

        $self->_transit_attempt_time(0);
    }
    catch NATE::BaseException with {
        $Log->warn( "node "
              . $self->_name()
              . " may be down, can't init node object..." );
        $self->_transit_attempt_time(time);
    };

    return;

}

# helper method to create transit_object
sub _create_transit_object {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    my $transit_obj;

    if ( my $wait_time = $self->_transit_attempt_should_wait() ) {
        $Log->comment( 'It looks like we are trying to repeatedly create'
              . ' transit object on a node that is down while in bad start mode.'
              . ' Wait '
              . ($wait_time)
              . ' seconds before trying again' );
        return;
    }

    try {
        $Log->comment("create transit attempt now...");

        $transit_obj =
          NACL::Transit->new( name => $self->_system_node_component()->node() );

        $self->_transit_attempt_time(0);
        $Log->comment("created transit object OK...");
    }
    catch NATE::BaseException with {
        $Log->warn( "node "
              . $self->_name()
              . " may be down, can't create transit object..." );
        $self->_transit_attempt_time(time);
    };

    return $transit_obj;

}

# determine the HA group this node belongs in.  This value is determined here
# the first time this method is called because at the time of the Node
# constructor, the DR Group's HA Groups have not yet been created.
sub _get_ha_group {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    unless ( $self->_ha_group_isset() ) {
        my %ha_group_map = %{ $self->_dr_group()->get_HA_groups() };
        foreach my $site ( keys(%ha_group_map) ) {
            my $ha_group  = $ha_group_map{$site};
            my $mcc_nodes = $ha_group->get_nodes();
            if (
                any {
                    $_->get_nvramid() eq $self->get_nvramid();
                }
                @{$mcc_nodes}
              )
            {
                $self->_ha_group($ha_group);
            }
        }

        unless ( $self->_ha_group_isset() ) {
            NACL::MTask::MCC::Exceptions::BaseException->throw( "Cannot "
                  . "determine HA Group for node '"
                  . $self->get_name()
                  . "'" );
        }
    }

    return $self->_ha_group();
}

# find the root aggregate for this node
sub _determine_root_aggregate {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    if ( my $wait_time = $self->_transit_attempt_should_wait() ) {
        $Log->comment( 'It looks like we are trying to repeatedly get the'
              . ' root aggregate on a node that is down while in bad start mode.'
              . ' Wait '
              . ($wait_time)
              . ' seconds before trying again' );
        return;
    }

    my $root_aggr;

    try {
        $Log->comment("get root aggr attempt now...");
        my $node_name  = $self->get_name();
        my $ci         = $self->get_Node_component()->command_interface();
        my @root_aggrs = NACL::C::Aggregate->find(
            command_interface => $ci,
            filter            => {
                'owner-name' => $node_name,
                'root'       => 'true',
            },
        );
        my @aggr_names =
          map { $_->get_one_state_attribute('aggregate') } @root_aggrs;

        # sanity check
        unless ( scalar(@aggr_names) == 1 ) {
            confess( "Node $node_name has unusual number of root aggrs:\n\t"
                  . join( "\n\t", @aggr_names ) );
        }

        $root_aggr = NACL::MTask::MCC::Aggregate->new(
            owner_mcc_node => $self,
            aggregate_name => $aggr_names[0]
        );
        $self->_transit_attempt_time(0);
        $Log->comment("got root aggr OK...");
    }
    catch NATE::BaseException with {
        $Log->warn( "node "
              . $self->_name()
              . " may be down, can't get root aggregate..." );
        $self->_transit_attempt_time(time);
    };

    return $root_aggr;
}

=head2 is_vsim()

Returns true (1) if this node is a vsim. False (0) otherwise.

=cut

sub is_vsim {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    return $self->L_is_vsim();
}

sub L_is_vsim {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self    = shift;
    my $is_vsim = $self->_network_node_functionality()->L_is_vsim();
    if ( !defined $is_vsim ) {

        #might not be defined if we can't contact node - bad startup
        #should be a vsim if no rlm
        $is_vsim = !$self->_rlm_enabled();
    }
    return $is_vsim;
}

sub L_has_switches {
    my $self = shift;

    return $self->_network_node_functionality()->L_has_switches();
}

sub L_has_bridges {
    my $self = shift;

    return $self->_network_node_functionality()->L_has_bridges();
}

# this class also contains an object which is solely associated with this
# node's networking functionality in the storage backplane network
sub L_init_network_node_handle {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    my $network_node = NACL::MTask::MCC::Network::Node->new(
        dr_group => $self->_dr_group(),
        mcc_node => $self
    );

    $self->_network_node_functionality($network_node);

    return $network_node;
}

=head2 L_get_root_aggregate()

Returns the NACL::MTask:MCC::Aggregate associated with the root aggregate of
this node.

=cut

sub L_get_root_aggregate {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    return $self->_root_aggregate();
}

=head2 get_Node_component()

    my $node_comp = $mcc_node->get_Node_component();

Returns the component object (NACL::C::SystemNode) of the node.

=cut

sub get_Node_component {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    return $self->_system_node_component();
}

=head2 get_root_aggregate_component()

    my $aggr_comp = $mcc_node->get_root_aggregate_component();

Returns the component object (NACL::C::StorageAggregate) of the root aggregate for the node.

=cut

sub get_root_aggregate_component {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    return $self->_root_aggregate()->_Aggregate_component();
}

=head2 get_nodescope_apiset()

    my $nodescope_apiset = $mcc_node->get_nodescope_apiset();

Returns the nodescope APISet object (NACL::APISet) associated with this node.

=cut

sub get_nodescope_apiset {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    return $self->get_Node_component()->get_7m_or_nodescope_apiset();
}

=head2 L_get_network_node_functionality()

    my $network_node = $mcc_node->L_get_network_node_functionality();

Returns the NACL::MTask::MCC::Network::Node object associated with the
"network functionality" of this NACL::MTask::MCC::Node.

=cut

sub L_get_network_node_functionality {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    return $self->_network_node_functionality();
}

=head2 get_name()

    my $node_name = $mcc_node->get_name();

Returns the node's name (string).

=cut

sub get_name {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    # IMPORTANT TBD:  think about this.  problem is that the node cannot be
    # queried for the name if it is down due to some disaster cycle.  the query
    # will hang/timeout if this happens, as it often will in a DR lc.
    #
    # nodes can be renamed.  we cannot cache the name of a node, but need to
    # query for it every time we need it.
    #my $name = $self->get_Node_component()->get_one_state_attribute('node');
    my $name = $self->_name();

    return $name;
}

=head2 get_role()

    my $node_role = $mcc_node->get_role();

Returns the node's role in its DR group ('A1'|'A2'|'B1'|'B2').  Here A1 and
B1 are DR partners, and A2 and B2 are DR partners.

=cut

sub get_role {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    return $self->_network_node_functionality()->get_role();
}

=head2 get_nvramid()

    my $node_nvramid = $mcc_node->get_nvramid();

Returns the node's nvramid (string).

=cut

sub get_nvramid {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    return $self->_nvramid();
}

=head2 get_uuid()

    my $node_uuid = $mcc_node->get_uuid();

Returns the node's uuid (string).

=cut

sub get_uuid {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    return $self->_uuid();
}

=head2 metrocluster_enable()

This is a STATIC method because no MCC::Metrocluster object can be instantiated
before 'metrocluster enable' operations have been completed successfully.

    NACL::MTask::MCC::Node->metrocluster_enable(
         command_interface_node_name => 'spectre-l11',
         node_name_to_enable         => 'spectre-l12',
         refresh                     => 0);

    NACL::MTask::MCC::Node->metrocluster_enable(
         command_interface_node_name => 'spectre-l11',
         node_uuid_to_enable         => 'd437c8cd-e931-11e0-aa13-c948a5272a17');

When this method is called on a NACL::MTask::MCC::Node, then
'metrocluster enable' is called directly via command_interface_node_name's node
command interface.  This method need only be called one one node in each of the
Metrocluster-to-be's DR partner nodes.

=over

=item Options

=over

=item C< command_interface_node_name >

(Required) Name of the node to use as the command interface for the call to
'metrocluster enable'.

=item C< node_name_to_enable >

(Either node_name_to_enable XOR node_uuid_to_enable are required) Name of the
node to be enabled.

=item C< node_uuid_to_enable >

(Either node_name_to_enable XOR node_uuid_to_enable are required) Node UUID of
the node to be enabled.

=item C< refresh => 0|1 (default = 1) >

(Optional) If true, refreshes the internal metrocluster state data.

=back

=back

=cut

sub metrocluster_enable {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $pkg   = shift;
    my %opts  = validate_with(
        params => \@_,
        spec   => {
            command_interface_node_name => { regex => qr/^[\w\-]+$/ },
            node_name_to_enable         => {
                regex    => qr/^[\w\-]+$/,
                optional => 1
            },
            node_uuid_to_enable => {
                regex    => qr/^[\w\-]+$/,
                optional => 1
            },
            refresh => { default => 1 },
        },
    );

    # one and only one of the parameters node_name_to_enable and
    # node_uuid_to_enable is required.  Check this requirement.
    {
        my $parameter_error_message =
            "NACL::MTask::MCC::Node->metrocluster_enable() requires either\n"
          . "    the node_name_to_enable parameter exclusive-or the\n"
          . "    node_uuid_to_enable parameter to be passed.";

        if ( defined( $opts{node_name_to_enable} ) ) {
            if ( defined( $opts{node_uuid_to_enable} ) ) {
                confess($parameter_error_message);
            }
        }
        elsif ( !defined( $opts{node_uuid_to_enable} ) ) {
            confess($parameter_error_message);
        }
    }

    # get Metrocluster component based on command_interface_node_name
    my $node_ci =
      NACL::C::Node->new(
        hostrec => Tharn::host( $opts{command_interface_node_name} ) )
      ->command_interface();
    my $metrocluster_component =
      NACL::C::Metrocluster->new( command_interface => $node_ci );

    my $refresh_opt = $opts{'refresh'} ? 'true' : 'false';

    if ( defined( $opts{node_name_to_enable} ) ) {
        $metrocluster_component->enable(
            node    => $opts{node_name_to_enable},
            refresh => $refresh_opt
        );
    }
    else {
        $metrocluster_component->enable(
            'node-uuid' => $opts{node_uuid_to_enable},
            refresh     => $refresh_opt
        );
    }

    return;
}

=head2 metrocluster_disable()

Unlike metrocluster_enable, metrocluster_disable is an instance method as it is
always called upon nodes in an active metrocluster system.

    # run 'metrocluster disable' on all site A nodes
    my $dr_groups = $self->get_DR_groups();
    foreach my $dr_group (values(%{ $dr_groups } )) {
        my $dr_partnerships = $dr_group->get_DR_partnerships();
        foreach my $partnership (@{$dr_partnerships}) {
            my $site_A_partner = $partnership->get_partner_node(site => 'A');
            $site_A_partner->metrocluster_disable();
        }
    }

When this method is called on a NACL::MTask::MCC::Node, then
'metrocluster disable' is called directly via this node's command interface.

=cut

sub metrocluster_disable {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    my $node_ci = $self->get_Node_component()->command_interface();
    my $metrocluster_component_using_this_nodes_ci =
      NACL::C::Metrocluster->new( command_interface => $node_ci );

    $metrocluster_component_using_this_nodes_ci->disable(
        node => $self->get_name() );

    return;
}

=head2 metrocluster_show()

This method runs metrocluster show commmand - It prints the dump on stdin

=over

=item Options

 None

=item Return Value

Metrocluster show output

=item Example

=back

     my $mcc_node = NACL::MTask::MCC::Node->new(node_name =>$NODEA);
     $mcc_show = $cluster_A->metrocluster_show();

=cut

sub metrocluster_show {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my ( $self, %opts ) = @_;
    my $node_comp = $self->get_Node_component();
    my $ci        = $node_comp->command_interface();
    my $mcc_show =
      NACL::MTask::MCC::Utils->metrocluster_node_show(
        command_interface => $ci );
    return $mcc_show;
}

=head2 compare_metrocluster_show_fields()

This method compares the metrocluster show output attributes
with the attributes provided by the user in the form of hashref.
Mismatches upon finding are stored into a hashref. Mismatches
are returned to the user if throw_exception_on_mismatch is not set

=over

=item Options

  Required: fields_to_test => $fields  [HASHREF]
  Optional: throw_exception_on_mismatch [BOOLEAN], default: 1

=item Return Value

Mismatches are returned if throw_exception_on_mismatch is set to 0
Exception is thrown if exception flag is set

=item Example

=back

     my $fields = {
    'dr_partner' => $partner_name,
    'dr_cluster' => $site_name,
    'dr_operation_state' => 'switchover',
        'ha_partner' => 'fd8a2896-003d-4b07-ae82-ab0f7fc7a588',
         };
     my $mcc_node = NACL::MTask::MCC::Node->new(node_name =>$NODEA);
     $mcc_node->compare_metrocluster_show_fields(fields_to_test=>$fields);

=cut

sub compare_metrocluster_show_fields {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my ( $self, %opts ) = @_;

    #validate opts
    #setting default to 1
    $opts{throw_exception_on_mismatch} = (
        exists $opts{throw_exception_on_mismatch}
        ? $opts{throw_exception_on_mismatch}
        : 1
    );
    my %vopts = validate_with(
        params => \%opts,
        spec   => {
            fields_to_test              => { type  => HASHREF },
            throw_exception_on_mismatch => { regex => qr/^(0|1)$/ }
        }
    );

    #getting the user provided fields out
    my $fields_to_test_hash_ref = $vopts{fields_to_test};

    #getting the node component
    my $mcc_show = $self->metrocluster_node_show();

    #removing uneccessary info from the hash
    delete $mcc_show->{'command_interface'};

    #this hash will have mismatches if found, empty initially
    my $mismatches = {};

    #iterate over the hash which is provided by the user
    #and compares those fields with metro show fields
    #we populate the mismatches into the returning hash - undef otherwise
    while ( my ( $key, $value ) = each(%$fields_to_test_hash_ref) ) {
        if ( !exists( $mcc_show->{$key} ) ) {
            confess "Invalid metrocluster show attribute found: \'$key\' "
              . "Please pass in the correct attribute to compare. \n";
        }
        if ( $mcc_show->{$key} ne $fields_to_test_hash_ref->{$key} ) {
            $mismatches->{$key} = $fields_to_test_hash_ref->{$key};
        }
    }
    my $throw_exception_flag = $opts{throw_exception_on_mismatch};

    # if throw_exception_on_mismatch is set
    if ($throw_exception_flag) {
        if ( keys(%$mismatches) ) {

            #TBD confess to be replace by a proper exception
            confess 'mismatches found:' . Dumper($mismatches);
        }
        else {
            $Log->comment("No Mismatches found\n");
            return;
        }
    }    #throw_exception_on_mismatch block ends
    return $mismatches;
}

=head2 get_dr_fsm_state()

This method runs MCC status commmand over dblade to fetch FSM state

=over

=item Options

 None

=item Return Value

FSM state value obtained from MCC status command

=item Example

=back

     my $mcc_node = NACL::MTask::MCC::Node->new(node_name => $NODEA);
     $mcc_status = $mcc_node->get_dr_fsm_state();

=cut

sub get_dr_fsm_state {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    my $node_component = $self->get_Node_component();
    my $response =
      $node_component->get_7m_or_nodescope_apiset( interface => 'CLI' )
      ->execute_command( command => 'mcc status' );
    my $drsom_state = undef;
    if ( $response =~ /FSM state:([\s\w]+)/i ) {
        $drsom_state = $1;
        $drsom_state =~ s/^\s+//;
        $drsom_state =~ s/\s+$//;
    }

    unless ( defined($drsom_state) ) {
        NACL::MTask::MCC::Exceptions::BaseException->throw(
            'FSM state not found:' . $response );
    }

    return $drsom_state;
}

=head2 L_validate_DRSOM_FSM_state

This method throws an exception if this node's DRSOM_FSM state is not consistent
with the mhost's DR state.

IMPORTANT: This method assumes that this node is in a cluster with a live
cluster management CI.  It uses this CI to determine if this node is queryable
as it may be down after an HA Takeover event.

=over

=item Options

=over

=item C< disaster_status >

(Required) DR state as defined by DR_Coordinator::get_DR_state().

=back

=back

=cut

sub L_validate_DRSOM_FSM_state {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => { dr_state => { type => SCALAR }, },
    );

    my $dr_state = delete( $opts{dr_state} );

    my %DRSOM_STATE_EXPECTED = (
        switchover_completed =>
          [ 'Switchover completed', 'switchover_bypassed' ],
        switchover_partially_successful => [
            'Normal',              'Switchover completed',
            'switchover_bypassed', 'switchback_scheduled',
            'switchback_in_progress'
        ],
        heal_aggrs_completed =>
          [ 'Aggr healing completed', 'switchover_bypassed' ],
        heal_aggrs_failed               => ['Switchover completed'],
        heal_aggrs_partially_successful => [
            'Aggr healing completed',
            'Switchover completed',
            'switchover_bypassed'
        ],
        heal_roots_failed    => [ 'Aggr healing completed', ],
        heal_roots_completed => [
            'Root aggr switchback completed',  'switchback_recovery_pending',
            'waiting for switchback recovery', 'switchover_bypassed'
        ],
        heal_roots_partially_successful => [
            'Aggr healing completed',
            'Root aggr switchback completed',
            'switchover_bypassed'
        ],
        switchback_failed => [
            'Normal', 'Root aggr switchback completed', 'switchover_bypassed',
        ],
        switchback_partially_successful =>
          [ 'Normal', 'Root aggr switchback completed', 'switchover_bypassed' ],
        normal => [ 'Normal', ],
    );

    # note that 2-node systems are always "HA_state == 'normal'"
    my $ha_group = $self->_get_ha_group();
    my $ha_state = $self->_get_ha_group()->L_get_HA_state();
    if (
        ( $ha_state->{HA_state} eq 'normal' )
        || (
            ( $ha_state->{HA_state} eq 'takeover' )
            && ( $ha_state->{survivor_node}->get_nvramid() eq
                $self->get_nvramid() )
        )
      )
    {
        my $drsom_state = $self->get_dr_fsm_state();

        foreach my $state ( @{ $DRSOM_STATE_EXPECTED{$dr_state} } ) {
            return if ( $drsom_state eq $state );
        }

        # switchback_recovery_complete is normal state. MCC goes to this state
        #when nodes are rebooted during sb in progress
        if ( $drsom_state eq 'switchback_recovery_complete' ) {
            return;
        }
        else {
            my $node_name = $self->get_name();
            NACL::MTask::MCC::Exceptions::BaseException->throw(
                    "Inconsistent DRSOM_FSM_state: Node [$node_name] has\n"
                  . "state [$drsom_state] while DR State is reported to\n"
                  . "be [$dr_state]." );
        }
    }

    return;
}

sub subtest_system_power {
    my $self = shift;

    my %args = validate(
        @_,
        {
            power    => { regex => qr/^on|off$/, required => 1 },
            goto_cli => { regex => qr/^[01]$/,   default  => 0 }
        }
    );
    print "iamin";
    my $subtest = subtest(
        sub {

            $self->system_power(
                power    => $args{'power'},
                goto_cli => $args{'goto_cli'}
            );

        },
        '-bg',
        '-runid' => $self->get_name() . '_system_power_' . $args{'power'}
    );

    return $subtest;

}

=head2 get_vsim_id()

API to get VSIM id for local node

=over

=back

=back

=cut

sub get_vsim_id {
    my $self = shift;

    $self->L_is_vsim() or return;

    my $power_cmd = $self->_system_node_component()->hostrec()->powercmd()
      or return;

    $power_cmd =~ m/vsim \s+ stop \s+ ([a-zA-Z0-9_]+);/x or return;

    my $vsim_id = $1;

    return $vsim_id;
}

sub _system_power_vsim_powercmd {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %args = validate( @_, { command => 1 } );

    my $vsim_bin = '/usr/software/test/bin/vsim';

    if ( defined $ENV{'VSIM_SCRIPT'} ) {
        $vsim_bin = $ENV{'VSIM_SCRIPT'};
    }

    my $vsim_id   = $self->get_vsim_id();
    my $node_name = $self->_system_node_component()->node();
    my $rc        = system( $vsim_bin, $args{'command'}, $vsim_id );

    if ( $rc != 0 ) {
        my $power_cmd = "$vsim_bin $args{'command'} $vsim_id";
        $Log->comment( "Unable to reboot node '$node_name' "
              . "with command '$power_cmd': $rc" );
        return;
    }

    return 1;
}

# private method to determine system power status for vsims
sub _system_power_status_vsim {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %args = validate( @_, { timeout => { default => 180 }, } );

    #since we transit to CLI for vsims, we will assume that
    #if we are in CLI then we are 'on' otherwise we will assume 'off'
    #...
    #or ... assume it's 'off' if in FIRMWARE as that's how we simulate
    #off and anything else we assume on???

    my $state;
    if ( defined $self->_transit_obj ) {
        try {
            $state =
              $self->_transit_obj->get_state( timeout => $args{timeout} );
        }
        catch NATE::BaseException with {
            $Log->comment("failed to determine power status, ignoring...");
        };
    }

    if ( $state eq 'FIRMWARE' ) {
        $Log->comment("found vsim in state $state, equating this to OFF");
        return 'off';
    }
    elsif ( !defined $state ) {
        $Log->comment( "couldn't get transit object or other error, "
              . 'equating this to OFF' );

        #if we can't get a transit object then assume it's OFF
        return 'off';
    }
    else {
        $Log->comment("found vsim in state $state, equating this to ON");
        return 'on';
    }
}

# private method to determine if there is an rlm entry - to determine if
# this is real hardware more reliably for nodes that are down especially
# during bad startup
sub _rlm_enabled {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my $rlm_enabled =
      $self->_system_node_component()->hostrec()->{'RLM_ENABLED'};
    $Log->comment( 'RLM_ENABLED:' . $rlm_enabled );
    return $rlm_enabled;
}

# private method to determine system power status for vsims
sub _system_power_status_real {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %args = validate( @_, { timeout => { default => 60 }, } );

    $Log->comment('Create SP connection object via RLM');

    my $conn_rec = Tharn::connect(
        $self->_system_node_component()->node() . '#rlm;conntype=ssh-persist' )
      or NACL::MTask::MCC::Exceptions::BaseException->throw(
        'Failed to get RLM details from NATE Host file!');

    #NATE bug, put in burt 815451
    #In theory this allows forcibly connecting but there appear
    #to be problems - somehow we are getting the result of being
    #kicked off ourselves, not sure how that's happening
    #ok after more investigation it's just that the connection above is
    #getting stuck on the prompt
    #$Log->comment( 'attempting to connrec->execute()' );
    #$conn_rec->execute(
    #'command' => '?',
    #'match_table' => [
    #'.*Would you like to disconnect that session, and start yours.*' =>
    #'y'
    #],
    #);
    #$conn_rec->disconnect();
    my $apiset = NACL::APISet->new(
        connection => $conn_rec,
        category   => 'Node',
        interface  => 'CLI',
        set        => 'SP',
    );

    my $response = $apiset->system_power(
        status               => 1,
        'connectrec-timeout' => $args{timeout},
    );

    $apiset->cleanup_APISet();    #SP can have only 1 connection
    $conn_rec->destroy();         #maybe cleanup isn't enough?

    $Log->comment( 'Power Status: ' . Dumper($response) );

    return $response->get_parsed_output->[0]->{'chassis_power'};

}

### We would use kernel panic with no_core as default workflow for simulating
###  "power loss" disaster scenario on MCC VSIMs
sub _system_power_vsim {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %args = validate(
        @_,
        {
            power    => { regex => qr/^on|off$/, required => 1 },
            goto_cli => { regex => qr/^[01]$/,   default  => 1 }
        }
    );

    if ( $args{'power'} eq 'off' ) {

# Disable VSIM power off. Use kernel panic with no core as the default option
# to simulate a power down on VSIMs This also halts the VSIM at LOADER prompt
#
#         my $succeeded =
#                     $self->_system_power_vsim_powercmd( command => 'poweroff' );
#
#         if ( !$succeeded ) {
#
#             $Log->comment("Unable to power off vsim for "
#                         . $self->get_name()
#                         . '. Performing system node halt -dump true.');

        # Prepare kernel panic to skip core dumps
        $self->_system_node_component->get_systemshell_apiset->sysctl(
            name              => 'debug.dont_dump_core',
            value             => 1,
            'privilege-level' => 'root'
        );

        # Disable SFO on panic if its 4-node MCC system so as to avoid any
        # un-intended SFO events on the disaster site nods during halting
        my @sys_nodes =
          NACL::CS::Cluster->fetch(
            command_interface => $self->_system_node_component() );

        if ( scalar @sys_nodes > 1 ) {
            NACL::STask::StorageFailover->modify(
                command_interface => $self->_system_node_component(),
                onpanic           => 'true',
                onreboot          => 'false',
                nacltask_verify   => 1
            );
        }

        my $halt_cmd_params =
            ' -dump true'
          . ' -reason prepare_for_switchover_execution'
          . ' -skip-lif-migration-before-shutdown true'
          . ' -skip-epsilon-transition-before-shutdown true'
          . ' -ignore-quorum-warnings true';

        my $halt_cmd =
            'set -confirmations off diag;'
          . 'system node halt -node '
          . $self->get_name()
          . $halt_cmd_params;

        $self->_transit_obj->change_state(
            to      => 'FIRMWARE',
            haltcmd => $halt_cmd,
            timeout => 2000
        );

        #         }

    }
    elsif ( $args{'power'} eq 'on' ) {

        #        $self->_system_power_vsim_powercmd( command => 'start' );

        if ( $args{'goto_cli'} ) {

           # This code will make sure that the node has reached CLI login prompt
            $self->_transit_obj->change_state(
                to          => 'CLI',
                iftakenover => 'wait',
                timeout     => 2000
            );
        }
    }

}

sub _system_power_real {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %args =
      validate( @_, { power => { regex => qr/^on|off$/, required => 1 } } );

    $Log->comment('Create SP connection object via RLM');

    my $conn_rec = Tharn::connect(
        $self->_system_node_component()->node() . '#rlm;conntype=ssh-persist' )
      or NACL::MTask::MCC::Exceptions::BaseException->throw(
        'Failed to get RLM details from NATE Host file!');

    #NATE bug, put in burt 815451
    #In theory this allows forcibly connecting but there appear
    #to be problems - somehow we are getting the result of being
    #kicked off ourselves, not sure how that's happening
    #$Log->comment( 'attempting to connrec->execute()' );
    #$conn_rec->execute(
    #'command' => '?',
    #'match_table' => [
    #'.*Would you like to disconnect that session, and start yours.*' =>
    #'y',
    #'log_level' => 'debug',
    #],
    #);
    #$conn_rec->disconnect();

    my $apiset = NACL::APISet->new(
        connection => $conn_rec,
        category   => 'Node',
        interface  => 'CLI',
        set        => 'SP',
    );

    if ( $args{'power'} eq 'off' ) {
        $Log->comment('Check for razor platform');
        my $model = $self->_system_node_component()->{'hostrec'}->{'MODEL'};
        my @model_names = qw(FAS2600 FAS8200 FAS2950 FAS2980 AFFA300);
        my $is_razor    = 'false';
        $Log->comment("The model is $model");
        foreach my $model_razor (@model_names) {
            if ( $model_razor eq $model ) {
                $is_razor = 'true';
                $Log->comment('Its a Razor platform');
				last;
            }
        }
        $Log->comment('System power off (real hardware)');
        if ( $is_razor eq 'true' ) {
            $apiset->system_power(
                off                      => 1,
                'connectrec-match_table' => [
'.*This command will trigger a destage of ONTAP and will take some time to complete.*'
                      => 'y'
                ]
            );
        }
        else {
            $apiset->system_power(
                off                      => 1,
                'connectrec-match_table' => [
                    '.*This will cause a dirty shutdown of your appliance.*' =>
                      'y'
                ]
            );
        }

        # Check node power status within 2 seconds.
        # The expected state is OFF, else confess.
        my $response = $apiset->system_power( status => 1 );
        $Log->comment( 'Power Status: ' . Dumper($response) );
        my $timeout = time + 120;
        while ( $response->get_parsed_output->[0]->{'chassis_power'} ne 'off' )
        {
            Tharn::snooze(2);
            $response = $apiset->system_power( status => 1 );
            if ( $response->get_parsed_output->[0]->{'chassis_power'} eq 'off'
                || time >= $timeout )
            {
                last;
            }
        }

        # Print node under test's power status
        if ( $response->get_parsed_output->[0]->{'chassis_power'} eq 'off' ) {
            $Log->comment( 'Found ~ node power is OFF: '
                  . $apiset->{connection}->{hostp}->{hostname} );
        }
        else {
            $apiset->cleanup_APISet();
            $conn_rec->destroy();    #maybe cleanup isn't enough?
            NACL::MTask::MCC::Exceptions::BaseException->throw(
                'Unable to power OFF node '
                  . $apiset->{connection}->{hostp}->{hostname} );
        }

    }
    elsif ( $args{'power'} eq 'on' ) {
        $Log->comment('System power on (real hardware)');
        $apiset->system_power( on => 1 );
    }

    $apiset->cleanup_APISet();
    $conn_rec->destroy();    #maybe cleanup isn't enough?

    return;
}

sub system_power {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %args = validate(
        @_,
        {
            power    => { regex => qr/^on|off$/, required => 1 },
            goto_cli => { regex => qr/^[01]$/,   default  => 0 }
        }
    );

    if ( $self->L_is_vsim() ) {
        $self->_system_power_vsim(
            power    => $args{'power'},
            goto_cli => $args{'goto_cli'}
        );
    }
    else {
        $self->_system_power_real( power => $args{'power'} );
    }

    if (   ( $args{'power'} eq 'on' )
        && ( $args{'goto_cli'} ) )
    {
        $Log->comment('Transit to CLI prompt');
        $self->_transit_obj->change_state(
            to          => 'CLI',
            iftakenover => 'wait',
            timeout     => 600
        );
    }

    # Wait a few seconds for the power to change
    Tharn::snooze(15);

    return;
}

sub system_power_status {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %args =
      validate( @_, { timeout => { regex => qr/^\d+$/, optional => 1 }, } );

    my %opts;
    if ( defined $args{timeout} ) {
        $opts{timeout} = $args{timeout};
    }

    my $status;

    if ( $self->L_is_vsim() ) {
        $status = $self->_system_power_status_vsim(%opts);
    }
    else {
        $status = $self->_system_power_status_real(%opts);
    }

    return $status;
}

sub command_interface {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    return $self->get_Node_component()->command_interface();

}

sub is_node_up {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my $subtest = $self->L_subtest_determine_node_up();

    Subtest::wait_finish( subtests => [$subtest] );

    my ($ok) = $subtest->results();

    return $ok;
}

sub filter_nodes_up {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $class = shift;

    my %args = validate_with(
        'params' => \@_,
        'spec'   => {
            'timeout' => {
                'type'    => SCALAR,
                'default' => 30
            },
            'nodes' => {
                'type'     => ARRAYREF,
                'required' => 1
            }
        }
    );

    my %nodes_by_subtestid = ();
    my @subtests           = ();

    foreach my $node ( @{ $args{'nodes'} } ) {
        my $subtest =
          $node->L_subtest_determine_node_up( timeout => $args{'timeout'} );

        push( @subtests, $subtest );

    }

    Subtest::wait_finish( subtests => \@subtests );

    my @good_nodes = ();

    foreach my $subtest_idx ( 0 .. $#subtests ) {
        my ($result) = $subtests[$subtest_idx]->results();
        push( @good_nodes, $args{'nodes'}[$subtest_idx] )
          if $result;
    }

    return @good_nodes;

}

sub L_subtest_determine_node_up {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %args = validate_with(
        'params' => \@_,
        'spec'   => {
            'timeout' => {
                'type'    => SCALAR,
                'default' => 30
            }
        }
    );

    my $ci = $self->command_interface();

    my $subtest = subtest(
        sub {
            $ci->refresh_command_interface( 'no_version_change' => 1 );
            return 1;
        },
        '-bg',
        '-runid'   => $self->get_name() . '_up_check',
        '-timeout' => $args{'timeout'},
        '-iffatal' => '-ok'
    );

    return $subtest;

}

##### L_get_atto_remote_adapters ####
# Method to scan and return a list of all FC adapters connected to remote disk
# shelves. This method makes cached entries in a class variable on first
# request attempt and returns the cached copies over all successive requests
#
# Returns:
#   Array ref of lists containing remote adapters
#
sub L_get_atto_remote_adapters {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    $Log->comment('Fetching remote adapters list');

    my $ci              = $self->command_interface();
    my $local_node_name = $self->get_name();
    my @adapters        = ();

    if ( not defined $self->_adapters() ) {
        my $root_aggr = NACL::CS::StorageAggregate->fetch(
            command_interface => $ci,
            filter            => {
                'root' => 'true',
                'node' => $local_node_name,
            },
            allow_empty => 0,
        );

        my @disks = NACL::CS::StorageDisk->fetch(
            command_interface => $ci,
            filter            => {
                'pool'      => 'Pool1',
                'owner'     => $local_node_name,
                'aggregate' => $root_aggr->aggregate(),
            },
            allow_empty => 0,
        );

        @disks or return;

        foreach my $disk (@disks) {
            my ( $adapter1, $adapter2 ) = $disk->initiator();
            push @adapters, $adapter1 if none { $_ eq $adapter1 } @adapters;
            push @adapters, $adapter2 if none { $_ eq $adapter2 } @adapters;
        }

        my $href;
        $href->{$local_node_name}->{'remote_storage'} = [@adapters];
        $self->_adapters($href);
    }
    else {
        $Log->comment('Returning cached list of remote FC adapters');
        push @adapters,
          @{ $self->_adapters()->{$local_node_name}->{'remote_storage'} };
    }

    return $self->_adapters()->{$local_node_name}->{'remote_storage'};
}

sub _do_remote_atto_adapters_down {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    $Log->comment('Bringing remote adapters down');

    my @adapters = @{ $self->L_get_atto_remote_adapters() };

    @adapters or return;

    my $ci = $self->command_interface();

    my $apiset = $ci->apiset();

    my $response;
    foreach my $adapter (@adapters) {
        $Log->comment("Bringing down fcadmin adapter $adapter");
        $apiset->execute_command(
            command => "run local fcadmin offline $adapter" );

        # Keep polling until we see the link OFFLINE
        my $timeout = time() + 120;    # 2min timeout
        do {
            $response = $apiset->execute_command(
                command => "run local fcadmin link_state $adapter" );
            Tharn::snooze(2);

            if ( $response !~ /$adapter: OFFLINE/ and $timeout < time() ) {
                croak "Timeout waiting for FC adapter $adapter to go offline";
            }
        } while ( $response !~ /$adapter: OFFLINE/ );
    }

    return 1;
}

sub _do_remote_atto_adapters_up {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    $Log->comment('Bringing remote adapters up');

    my @adapters = @{ $self->L_get_atto_remote_adapters() };

    @adapters or return;

    my $ci = $self->command_interface();

    my $apiset = $ci->apiset();

    my $response;
    foreach my $adapter (@adapters) {
        $Log->comment("Bringing up fcadmin adapter $adapter");
        $apiset->execute_command(
            command => "run local fcadmin online $adapter" );

        # Keep polling until we see the link UP
        my $timeout = time() + 120;    # 2min timeout
        do {
            $response = $apiset->execute_command(
                command => "run local fcadmin link_state $adapter" );
            Tharn::snooze(2);

            if ( $response !~ /$adapter: UP/ and $timeout < time() ) {
                croak "Timeout waiting for FC adapter $adapter to go online";
            }
        } while ( $response !~ /$adapter: UP/ );
    }

    return 1;
}

=head2 modify_auso_config_state()

API to modify the node scope MCC AUSO config state. AUSO state can be either true or false

    $mcc_node->modify_auso_config_state( state => 'true' );

=over

=item Options

=over

=item C<< state=E<gt> 'true|false' >>

(Required) Defines the AUSO config state to be set on a MCC node

=over

=back

=back

=cut

sub modify_auso_config_state {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => { state => { regex => qr/^true|false$/, optional => 0 }, },
    );

    # Make the changes to local node
    NACL::C::Metrocluster->modify(
        command_interface                => $self->_system_node_component(),
        'automatic-switchover-onfailure' => $opts{state},
        'node-name'                      => $self->_name(),
    );

    # Verify the changes for local node
    my $mcc_cs = NACL::CS::MetroclusterNode->fetch(
        command_interface => $self->_system_node_component(),
        filter => { node => $self->_name(), },
    );

    NACL::MTask::MCC::Exceptions::BaseException->throw(
        'Failed to change AUSO config state on node ' . $self->_name() )
      if $mcc_cs->automatic_uso() ne $opts{state};

    return;
}

sub subtest_system_reset {
    my $self = shift;

    my %args =
      validate( @_, { goto_cli => { regex => qr/^[01]$/, default => 0 } } );

    my $subtest = subtest(
        sub {

            $self->system_reset( goto_cli => $args{'goto_cli'} );

        },
        '-bg',
        '-runid' => $self->get_name() . '_system_reset'
    );

    return $subtest;

}

sub system_reset {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %args =
      validate( @_, { goto_cli => { regex => qr/^[01]$/, default => 0 } } );

    if ( $self->L_is_vsim() ) {
        $self->_system_reset_vsim();
    }
    else {
        $self->_system_reset_real();
    }

    if ( $args{'goto_cli'} ) {
        $Log->comment('Transit to CLI prompt');
        $self->_transit_obj->change_state(
            to          => 'CLI',
            iftakenover => 'wait',
            timeout     => 600
        );
    }

    #    # Wait a few seconds for the power to change
    #    Tharn::snooze(15);

    return;
}

sub _system_reset_real {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    $Log->comment('Create SP connection object via RLM');
    my $conn_rec = Tharn::connect(
        $self->_system_node_component()->node() . '#rlm;conntype=ssh-persist' )
      or NACL::MTask::MCC::Exceptions::BaseException->throw(
        'Failed to get RLM details from NATE Host file!');

    my $apiset = NACL::APISet->new(
        connection => $conn_rec,
        category   => 'Node',
        interface  => 'CLI',
        set        => 'SP',
    );

    $Log->comment('System reset (real hardware)');
    $apiset->system_reset( 'connectrec-match_table' =>
          [ '.*This will cause a dirty shutdown of your appliance.*' => 'y' ] );

    $apiset->cleanup_APISet();
    $conn_rec->destroy();    #maybe cleanup isn't enough?

    return;
}

sub _system_reset_vsim {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    $self->_system_node_component->get_systemshell_apiset->sysctl(
        name              => 'debug.dont_dump_core',
        value             => 1,
        'privilege-level' => 'root'
    );

    # Disable SFO on panic if its 4-node MCC system so as to avoid any
    # un-intended SFO events on the disaster site nods during halting
    my @sys_nodes = NACL::CS::Cluster->fetch(
        command_interface => $self->_system_node_component() );

    if ( scalar @sys_nodes > 1 ) {
        NACL::STask::StorageFailover->modify(
            command_interface => $self->_system_node_component(),
            onpanic           => 'true',
            onreboot          => 'false',
            nacltask_verify   => 1
        );
    }

    my $halt_cmd_params =
        ' -dump true'
      . ' -reason prepare_for_switchover_execution'
      . ' -skip-lif-migration-before-shutdown true'
      . ' -skip-epsilon-transition-before-shutdown true'
      . ' -ignore-quorum-warnings true';

    my $halt_cmd =
        'set -confirmations off diag;'
      . 'system node halt -node '
      . $self->get_name()
      . $halt_cmd_params;

    $self->_transit_obj->change_state(
        to      => 'FIRMWARE',
        haltcmd => $halt_cmd,
        timeout => 2000
    );

}
1;
