#
# Copyright (c) 2001-2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary This is a wrapper around the B<NACL::AppBuilder::WFC::IO> and
##          NACL::MTask::Traffic libraries
##
## @author dl-mcc-dpg-qa@netapp.com
##
## @status ready
##

package NACL::MTask::MCC::TrafficHelper;

use strict;
use warnings;
use Carp qw{croak};

use Data::Dumper;
use File::Path qw(make_path remove_tree);
use POSIX qw(strftime);
use Tharn qw( subtest param);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use List::MoreUtils qw(any);
use NATE::BaseException qw(:try);

use NACL::AppBuilder::WFC::IO;

use NACL::C::Network;
use NACL::CS::Vserver;
use NACL::CS::VserverCifs;
use NACL::CS::VserverIscsi;
use NACL::CS::NetworkInterface;
use NACL::C::Client::Multipath;
use NACL::C::Client::IscsiTarget;
use NACL::CS::VserverIscsiInterface;

use NACL::STask::Node;
use NACL::STask::Volume;
use NACL::MTask::Traffic;
use NACL::MTask::MCC::Log ();
use NACL::STask::Client::Mount;
use NACL::STask::NetworkInterface;
use NACL::STask::VserverCifsShare;
use NACL::STask::VserverServicesDns;
use NACL::STask::VserverCifsShareProperties;
use NACL::MTask::MCC::Exceptions::BaseException;

use Params::Validate qw( validate_with
                         BOOLEAN
                         ARRAYREF
                         HASHREF
                         OBJECT
                         SCALAR
                         UNDEF );


=head1 NAME

NACL::MTask::MCC::TrafficHelper - Traffic generation for MCC

=head1 DESCRIPTION

B<EXPERIMENTAL!>

TrafficHelper is a wrapper around the B<NACL::AppBuilder::WFC::IO> and
B<NACL::MTask::Traffic> libraries.

A wrapper library for the functionality already exported by NACL to provide
minimum lines of code writing in "thpl" scripts which is required to call same
function in multiple times from different scenarios when Metrocluster
Functional testing.

B<DISCLAIMER:> This library is ONLY for internal use by the dpg-mcc-qa-auto team
            and will not be supported for ANY EXTERNAL users.

    # Start test
    use NACL::MTask::MCC::TrafficHelper;
    ...

    my $traffic_helper = NACL::MTask::MCC::TrafficHelper->new();
    ...

    # Traffic start
    $traffic_helper->traffic_start(
        nfs               => 1,
        cifs              => 1,
        iscsi             => 1,

        share_objs_ref    => \@cifs_shares,
        win_clients_ref   => \@win_clients,
        cifs_user_passwd  => { cifs_user   => $obj->_cifs_username(),
                              cifs_passwd => $obj->_cifs_password() },

        nfs_vols_ref      => \@nfs_vols,
        client_objs_ref   => \@linux_clients,
        enable_nfs_v41    => 1,

        lun_objs_ref      => \@lun_objs,
        igroup_objs_ref   => \@igroup_objs,
        iscsi_clients_ref => \@iscsi_clients
    );

    # CIFS Traffic stop
        $traffic_helper->traffic_stop( cifs => 1, disconnect_maps => 0 );

    # CIFS Traffic resume
        traffic_helper->cifs_traffic_resume();

    # All Traffic stop
        $traffic_helper->traffic_stop( cifs => 1, nfs => 1, iscsi => 1 );

    ...

=cut

=head1 METHODS

=head2 new (...)

    $traffic_helper = NACL::MTask::MCC::TrafficHelper->new();

Create a new traffic_helper object and return it to the caller.

=cut

use Class::MethodMaker [
    scalar => [
                 qw/ _nfs_traffic_objs
                     _io_handle_objs
                     _cifs_traffic_objs
                     _cifs_share_objs
                     _cifs_io_opts
                     _cifs_client_maps
                     _iscsi_traffic_objs
                     _get_lun_igroup_maps
                     _get_iscsi_client_objs
                     _short_logfile_names
                     _data_lif_comp_states
                     _is_2_node
                     _client_tgt_login_mapping /
              ],
    new    => [ 'new' ],
];

=head2 traffic_start

    # Traffic start with options:
    $traffic_helper->traffic_start(
        nfs          => 1,
        cifs         => 1,
        iscsi        => 1,
        short_logfile_names => 0,
        traffic_tool        => 'sio',(dt|hammer TBA)

        nfs_vols_ref        => \@nfs_vols,
        client_objs_ref     => \@linux_clients,
        enable_nfs_v41      => 1,
        nfs_traffic_opts    => { file_size => '5g', iops => 1000 },

        share_objs_ref      => \@cifs_shares,
        win_clients_ref     => \@win_clients,
        cifs_user_passwd    => { cifs_user => 'CTLAdmin',
                               cifs_passwd => 'netapp' },
        cifs_traffic_opts   => { file_size => '100m' },

        lun_objs_ref        => \@lun_objs,
        igroup_objs_ref     => \@igroup_objs,
        iscsi_clients_ref   => \@iscsi_clients
    );

Start a NFS traffic volumes objects, CIFS traffic on share objects and
start iSCSI traffic on Multipath (raw) devices /dev/dm-x.

=over

=item Options

=back

=over

=item C<< nfs =E<gt> 0|1 >>

(Optional) Boolean option to start NFS traffic. Default: 0

=item C<< cifs =E<gt> 0|1 >>

(Optional) Boolean option to start CIFS traffic. Default: 0

=item C<< iscsi =E<gt> 0|1 >>

(Optional) Boolean option to start iSCSI traffic. Default: 0

=item C<< short_logfile_names =E<gt> 0|1 >>

(Optional) Create subtest logs with short logfile names. Default: 0

=back

=over

=item NFS_Traffic_Params

=over

=item C<< nfs_vols_ref =E<gt> $nfs_vols_ref >>

=item C<< client_objs_ref =E<gt> $nfs_client_objs_ref >>

=item C<< enable_nfs_v41 =E<gt> 0|1 >>

=item C<< nfs_traffic_opts =E<gt> $hashref >>

=back

=back

=over

=item CIFS_Traffic_Params

=over

=item C<< share_objs_ref =E<gt> $share_objs_ref >>

=item C<< win_clients_ref =E<gt> $win_clients_ref >>

=item C<< cifs_user_passwd =E<gt> $hashref >>

=item C<< cifs_traffic_opts =E<gt> $hashref >>

=back

=back

=over

=item iSCSI_Traffic_Params

=over

=item C<< lun_objs_ref =E<gt> $lun_objs_ref >>

=item C<< igroup_objs_ref =E<gt> $igroup_objs_ref >>

=item C<< iscsi_clients_ref =E<gt> $iscsi_clients_ref >>

=back

=back

=cut

sub traffic_start {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec => {
            nfs          => { type => BOOLEAN, default => 0 },
            cifs         => { type => BOOLEAN, default => 0 },
            iscsi        => { type => BOOLEAN, default => 0 },

            traffic_tool => { regex => '^(dt|sio|hammer)$', default => 'sio'},
            traffic_tool_opts  => { type => HASHREF | UNDEF, optional => 1,
                                                                default => {} },

            short_logfile_names => { type => BOOLEAN, default => 0 },

            nfs_vols_ref        => { type => ARRAYREF | UNDEF, optional => 1 },
            client_objs_ref     => { type => ARRAYREF | UNDEF, optional => 1 },
            nfs_version         => { regex => '^(3|4)$', default => '4'},
            enable_nfs_v41      => { type => BOOLEAN, default => 0 },
            nfs_traffic_opts    => { type => HASHREF | UNDEF, optional => 1 },
            nfs_lun_find        => { type => SCALAR | UNDEF, default => undef },

            share_objs_ref      => { type => ARRAYREF | UNDEF, optional => 1 },
            win_clients_ref     => { type => ARRAYREF | UNDEF, optional => 1 },
            cifs_user_passwd    => { type => HASHREF  | UNDEF, optional => 1 },
            cifs_traffic_opts   => { type => HASHREF  | UNDEF, optional => 1 },

            lun_objs_ref        => { type => ARRAYREF | UNDEF, optional => 1 },
            igroup_objs_ref     => { type => ARRAYREF | UNDEF, optional => 1 },
            iscsi_clients_ref   => { type => ARRAYREF | UNDEF, optional => 1 },
            iscsi_traffic_opts  => { type => HASHREF  | UNDEF, optional => 1 },
        },
        allow_extra => 1
    );
    $Log->enter() if $may_enter;

    my $nfs_vols_ref = delete $opts{nfs_vols_ref};
    my $clients_ref  = delete $opts{client_objs_ref};
    my $nfs_ver      = delete $opts{nfs_version};
    my $nfsv41       = delete $opts{enable_nfs_v41};
    my $nfs_lun_find = delete $opts{nfs_lun_find};
    my $tool         = delete $opts{traffic_tool};
    my %ttopts       = %{$opts{traffic_tool_opts}}; # Make a copy of Tool Opts
    my $ttopts       = \%ttopts;

    $Log->comment( q{=} x 60 );
    $Log->comment('LIB IO Tool Input Options::' . Dumper $ttopts);
    $Log->comment( q{=} x 60 );

    $self->_short_logfile_names( $opts{short_logfile_names} );

    # Check if "log=PATH" param is passed for dt IO tool
    if ( lc $tool eq 'dt' and defined $ttopts->{log} ) {
        $ttopts->{log} .= '/DT_debug_' . time . '.log';
        $Log->comment('DT logging enabled. Output sent to : ' . $ttopts->{log});
    } else {
        delete $ttopts->{log};
    }

    # Default traffic tool options
    my %default_options = (
        # SIO options
        sio => {
            tool            => 'sio',
            read            => defined $ttopts->{read} ? delete $ttopts->{read} : 50,
            rand            => defined $ttopts->{rand} ? delete $ttopts->{rand} : 50,
            block_size      => defined $ttopts->{block_size} ? delete $ttopts->{block_size} : '4k',
            start_byte      => defined $ttopts->{start_byte} ? delete $ttopts->{start_byte} : 0,
            file_size       => defined $ttopts->{file_size} ? delete $ttopts->{file_size} : '40m',
            noflock         => defined $ttopts->{noflock} ? delete $ttopts->{noflock} : 0,
            verify          => 1, # we handle "disable_verify later below
            num_of_threads  => defined $ttopts->{num_of_threads} ? delete $ttopts->{num_of_threads} : 2,
            stop_gracefully => defined $ttopts->{stop_gracefully} ? delete $ttopts->{stop_gracefully} : 1,
            %{$ttopts},
        },
        dt => {
            tool        => 'dt',
            iotype      => defined $ttopts->{iotype} ? delete $ttopts->{iotype} : 'random',
            limit       => defined $ttopts->{limit} ? delete $ttopts->{limit} : '100m',
            procs       => defined $ttopts->{procs} ? delete $ttopts->{procs} : 1,
            bs          => defined $ttopts->{bs} ? delete $ttopts->{bs} : '4k',
            incr        => defined $ttopts->{incr} ? delete $ttopts->{incr} : 'var',
            min         => defined $ttopts->{min} ? delete $ttopts->{min} : '4k',
            max         => defined $ttopts->{max} ? delete $ttopts->{max} : '256k',
            pattern     => defined $ttopts->{pattern} ? delete $ttopts->{pattern} : 'iot',
            runtime     => defined $ttopts->{runtime} ? delete $ttopts->{runtime} : '-1',
            dispose     => defined $ttopts->{dispose} ? delete $ttopts->{dispose} : 'keep',
            prefix      => defined $ttopts->{prefix} ? delete $ttopts->{prefix} : '"%U@%d@%u@%h"',
            disable     => defined $ttopts->{disable} ? delete $ttopts->{disable} : 'pstats',
            onerr       => defined $ttopts->{onerr} ? delete $ttopts->{onerr} : 'abort',
            oflags      => defined $ttopts->{oflags} ? delete $ttopts->{oflags} : 'trunc',
            noprogt     => defined $ttopts->{noprogt} ? delete $ttopts->{noprogt} : 15,
            noprogtt    => defined $ttopts->{noprogtt} ? delete $ttopts->{noprogtt} : 900,
            alarm       => defined $ttopts->{alarm} ? delete $ttopts->{alarm} : '30s',
            term_wait   => defined $ttopts->{term_wait} ? delete $ttopts->{term_wait} : 50,
            rdelay      => defined $ttopts->{rdelay} ? delete $ttopts->{rdelay} : 1,
            wdelay      => defined $ttopts->{wdelay} ? delete $ttopts->{wdelay} : 1,
            history     => defined $ttopts->{history} ? delete $ttopts->{history} : 1,
            maxfiles    => defined $ttopts->{maxfiles} ? delete $ttopts->{maxfiles} : 4,
            files       => defined $ttopts->{files} ? delete $ttopts->{files} : 2,
            retry_error => defined $ttopts->{retry_error} ? delete $ttopts->{retry_error} : 39,
            enable      => defined $ttopts->{enable} ? delete $ttopts->{enable}
                         : 'noprog,'
                         . 'compare,'
                         . 'lbdata,'
                         . 'microdelay,'
                         . 'header,'
                         . 'nate,'
                         . 'timestamp,'
                         . 'htiming,'
                         . 'stats,'
                         . 'stopimmed,'
                         . 'deleteperpass,'
                         . 'mntdebug,'
                         . 'debug,'
                         . 'jdebug,'
                         . 'fdebug',
            stop_gracefully => defined $ttopts->{stop_gracefully} ? delete $ttopts->{stop_gracefully} : 1,
            %{$ttopts},
        },
    );

    # Apply disable_verify flag to IO tools
    my $disable_verify = defined $ENV{'DISABLE_SIO_VERIFY'} ? 1 : 0;
    $Log->comment("Disable IO Verification flag is: $disable_verify");
    if ( $disable_verify ) {
        $Log->comment('IO verification will not run because the user has '
                      . 'explicitly disabled it. This could be because the '
                      . 'test is running on VSIMS. '
                      . 'Lookup burts 818544 and 829649 for more details');
        $default_options{'sio'}->{verify} = 0;
        $default_options{'dt'}->{disable} .= ',verify';
    }

    # Dereference traffic tool params hash table
    my %tool_options = %{ $default_options{$tool} };
    $Log->comment( q{=} x 60);
    $Log->comment('LIB final IO tool_options::' . Dumper \%tool_options);
    $Log->comment( q{=} x 60);

    # Call NFS traffic method
    if ( $opts{nfs} ) {
        my %nfs_tool_options;
        if ( defined $opts{nfs_traffic_opts} ) {
            %nfs_tool_options = ( %tool_options, %{$opts{nfs_traffic_opts}} );
        } else {
            %nfs_tool_options = %tool_options;
        }
        $self->_traffic_nfs_start(
            nfs_vols_ref      => $nfs_vols_ref,
            client_objs_ref   => $clients_ref,
            traffic_tool      => $tool,
            nfs_version       => $nfs_ver,
            enable_nfs_v41    => $nfsv41,
            'privilege-level' => 'root',
            nfs_lun_find      => $nfs_lun_find,
            %nfs_tool_options );
    }

    # Call CIFS traffic method
    if ( $opts{cifs} ) {
        my %cifs_tool_options;
        if ( defined $opts{cifs_traffic_opts} ) {
            %cifs_tool_options = ( %tool_options, %{$opts{cifs_traffic_opts}} );
        } else {
            %cifs_tool_options = %tool_options;
        }
        $self->_traffic_cifs_start(
            share_objs_ref   => $opts{share_objs_ref},
            win_clients_ref  => $opts{win_clients_ref},
            cifs_user_passwd => $opts{cifs_user_passwd},
            traffic_tool     => $tool,
            %cifs_tool_options );
    }

    # Call iSCSI traffic method
    if ( $opts{iscsi} ) {
        my %iscsi_tool_options;
        if ( defined $opts{iscsi_traffic_opts} ) {
            %iscsi_tool_options = (%tool_options, %{$opts{iscsi_traffic_opts}});
        } else {
            %iscsi_tool_options = %tool_options;
        }

        $Log->comment('Setting the iscsi clients');
        $self->_get_iscsi_client_objs($opts{iscsi_clients_ref});

        $self->_traffic_iscsi_start(
            iscsi_clients_ref => $opts{iscsi_clients_ref},
            lun_objs_ref      => $opts{lun_objs_ref},
            igroup_objs_ref   => $opts{igroup_objs_ref},
            traffic_tool      => $tool,
            'privilege-level' => 'root',
            %iscsi_tool_options );
    }
    $Log->exit() if $may_exit;

    return;
}

=head2 _traffic_nfs_start

    $self->_traffic_nfs_start(
                        nfs_vols_ref     => $nfs_vols_ref,
                        client_objs_ref  => $clients_ref,
                        traffic_tool     => 'sio',
                        nfs_version      => 3|4,
                        enable_nfs_v41   => 0|1,
                        nfs_lun_find     => 0|undef,
                        nfs_traffic_opts => { file_size = '200m', iops => 1000 }
                    );

B<Private method for internal library use only!>

=over

=item Options

=over

=item C<< nfs_vols_ref =E<gt> $nfs_vols_ref >>

=item C<< client_objs_ref =E<gt> $nfs_client_objs_ref >>

=item C<< nfs_version =E<gt> 3|4 >>

=item C<< enable_nfs_v41 =E<gt> 0|1 >>

=item C<< nfs_traffic_opts =E<gt> $hashref >>

=back

=back

=cut

sub _traffic_nfs_start {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec => {
            nfs_vols_ref    => { type => ARRAYREF },
            client_objs_ref => { type => ARRAYREF },
            traffic_tool    => {
                                 regex   => '^(dt|sio|hammer)$',
                                 default => 'dt',
                               },
            nfs_version     => { regex => '^(3|4)$', default => '4'},
            enable_nfs_v41  => { type  => BOOLEAN, default => 0 },
            nfs_lun_find    => { type  => SCALAR | UNDEF, default => undef },
        },
        allow_extra => 1,
    );

    $Log->enter() if $may_enter;

    my ($nfs_traffic_objs, $nfs_mount_objs );
    my $vol_objs_ref    = delete $opts{nfs_vols_ref};
    my $client_objs_ref = delete $opts{client_objs_ref};
    my $nfs_ver         = delete $opts{nfs_version};
    my $nfs_ver41       = delete $opts{enable_nfs_v41};
    my $nfs_lun_find    = delete $opts{nfs_lun_find};
    my $traffic_tool    = delete $opts{traffic_tool};

    # Delete IO tool specific params
    if ( $traffic_tool ne 'sio' && exists $opts{file_size} ) {
        delete $opts{file_size};
    }

    if ( $traffic_tool ne 'dt' && exists $opts{limit} ) {
        delete $opts{limit};
    }

    my %nfs_io_opts = ( %opts );

    # Prepare for NFS v4.1 mount option
    my $mount_option = 'hard,intr';
    if ($nfs_ver41) {
        $nfs_ver = 4;
        $mount_option = 'hard,intr,minorversion=1';
        # Use RHEL6 or higher clients for NFS 4.1 mount
        $Log->comment('------------------------------------------------------');
        $Log->comment('Tested NFS version: NFS v4.1 (' . $mount_option . ')');
        $Log->comment('------------------------------------------------------');
    } else {
        $Log->comment('----------------------------');
        $Log->comment('Tested NFS version: NFS v' . $nfs_ver );
        $Log->comment('----------------------------');
    }

    my $vservers_ref = $self->get_vserver_objs_ref_from_vol_objs(
        vol_objs_ref => $vol_objs_ref );

    $Log->comment('NFS_TRAFFIC_OPTS: ' . Dumper(\%nfs_io_opts));

    my $num = 0;
    foreach my $vser ( @{ $vservers_ref } ) {
        $Log->comment('NFS traffic running on vserver: ' . $vser->vserver());

        # Get all NAS data lifs from vserver under test
        my $lifs_ref = $self->get_data_lifs_by_protocol(
                                    node_obj      => $vser->command_interface(),
                                    vserver_name  => $vser->vserver(),
                                    data_protocol => 'nfs' );

        # Collect volumes belong to vserver under test
        my @vols;
        foreach my $vol ( @{ $vol_objs_ref } ) {
            next if ( $vol->vserver() ne $vser->vserver() );
            push @vols, $vol;
        }

        # Start NFS traffic
        foreach my $volume (@vols) {
            $num++;
            # Limit mount points per client ( default = 8 )
            if ($num > ( scalar(@{$client_objs_ref}) ) * 8 ) { last; }

            # Get a client from a list of available clients list
            my $client = shift(@{$client_objs_ref});
            push @{$client_objs_ref}, $client;

            # Get round-robin data lif from a list of reachable data lifs
            my $lif_obj = shift(@{$lifs_ref});
            push @{$lifs_ref}, $lif_obj;

            # Client and Data LIF check point
            my ($new_client, $new_lif) =
                            $self->get_reachable_data_lif_and_client_pair(
                                primary_client_obj  => $client,
                                primary_datalif_obj => $lif_obj,
                                backup_clients_ref  => $client_objs_ref,
                                backup_datalifs_ref => $lifs_ref );

            my $nfs_io_handle = NACL::AppBuilder::WFC::IO->new(
                                                      volume => $volume,
                                                      client => $new_client,
                                                      lun    => $nfs_lun_find );

            # server param will be ignored until Burt770396 fixed
            my $nfs_options = {
                proto     => 'tcp',
                vers      => $nfs_ver,
                server    => $new_lif->address(),
                mountopts => $mount_option };

            # Create the mount-point
            $nfs_io_handle->create_mount( nfs_options => $nfs_options );
            $Log->comment('NFS client mount successfully completed for '
                        . $volume->volume());
            push @{$nfs_mount_objs}, $nfs_io_handle;

            # Create required files on the NFS mount to run SIO traffic
            # Make $file an absolute path:
            my $path = NACL::MTask::Traffic->get_abs_path(
                                                       $nfs_io_handle->_mount );
            my $io_file = 'nfs_' . $volume->volume();
            my $file_name = "$path/$io_file" . '_1';

            if ( $traffic_tool eq 'sio' ) {
                # Start sio baseline file creation block( workaround )
                $Log->comment("Creating a file $file_name"
                            . " ~ (  $opts{file_size} )");

                $new_client->apiset(interface => 'CLI')->execute_command(
                   'privilege-level'    => 'root',
                   'connectrec-timeout' => 360,
                    command => '/usr/software/test/bin/fops -o c '
                             . "-s $opts{file_size} -n 1 "
                             . "-p $io_file $path" );

                $new_client->apiset(interface => 'CLI')->execute_command(
                    command             => "chmod 777 $file_name",
                   'privilege-level'    => 'root',
                   'connectrec-timeout' => 300 );
                # End file creation block.
            }

            # Create short subtest logfile names if required
            if ( $self->_short_logfile_names() ) {
                $nfs_io_opts{'runid'} = 'IO_traffic_on_' . $volume->volume();
            }

            # Define some DT tool specific params
            if ( $traffic_tool eq 'dt' ) {
                $nfs_io_opts{'trigger'} =
                       defined $nfs_io_opts{'trigger'}
                       ? $nfs_io_opts{'trigger'}
                       : 'cmd:/x/eng/localtest/noarch/bin/dt_noprog_script.ksh';
                $nfs_io_opts{'privilege-level'} = 'root';
            }

            # Start traffic
            my $nfs_traffic_obj = NACL::MTask::Traffic->start(
                                            command_interface => $new_client,
                                            paths             => $file_name,
                                            tool              => $traffic_tool,
                                            %nfs_io_opts );
            push @{$nfs_traffic_objs}, $nfs_traffic_obj;
        }
    }

    # Set nfs_io_handles into obj
    $self->_nfs_traffic_objs($nfs_traffic_objs);
    $self->_io_handle_objs($nfs_mount_objs);
    $Log->exit() if $may_exit;

    return;
}

=head2 _traffic_cifs_start

   $self->_traffic_nfs_start( share_objs_ref    => $cifs_shares_ref,
                              win_clients_ref   => \@win_clients,
                              cifs_user_passwd  => {
                                                      cifs_user   => 'CTLAdmin',
                                                      cifs_passwd => 'netapp'
                                                   },
                              cifs_traffic_opts => {
                                                      file_size => '100m'
                                                   } );

B<Private method for internal library use only!>

=over

=item Options

=over

=item C<< share_objs_ref =E<gt> $cifs_shares_ref >>

=item C<< win_clients_ref =E<gt> $win_client_objs_ref >>

=item C<< cifs_user_passwd =E<gt> $hashref >>

=item C<< cifs_traffic_opts =E<gt> $hashref  >>

=back

=back

=cut

sub _traffic_cifs_start {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec => {
            share_objs_ref   => { type => ARRAYREF },
            win_clients_ref  => { type => ARRAYREF },
            cifs_user_passwd => { type => HASHREF },
            traffic_tool     => {
                                    regex => '^(dt|sio|hammer)$',
                                    default => 'sio',
                                },
        },
        allow_extra => 1,
    );
    $Log->enter() if $may_enter;

    my $traffic_tool = delete $opts{traffic_tool};
    my $share_objs_ref  = delete $opts{share_objs_ref};
    my $win_clients_ref = delete $opts{win_clients_ref};

    # Delete IO tool specific params
    if ( $traffic_tool ne 'sio' && exists $opts{file_size} ){
        delete $opts{file_size};
    }

    if ( $traffic_tool ne 'dt' && exists $opts{limit} ) {
        delete $opts{limit};
    }

    # Traffic file size k|m|g to bytes calculation table
    my %convert_to_bytes = (
        'k' => { bytes => 1024 },
        'm' => { bytes => 1024 * 1024 },
        'g' => { bytes => 1024 * 1024 * 1024 }, );

    # Print out DNS options
    my %cifs_user_passwd = % { delete $opts{cifs_user_passwd} };

    # Verify minwin compatible Windows clients
    $self->verify_windows_clients( client_objs_ref => $win_clients_ref );

    # Create CIFS share, map and start CIFS traffic ###
    my $num = 0;
    my $previous_vs_name;
    my ($cifs_client_maps, $cifs_traffic_objs,  $lifs_obj_ref);

    foreach my $share_obj ( @{$share_objs_ref} ) {
        $num++;
        # Limit CIFS shares/maps per client ( default = 8 )
        if ( $num > ( scalar(@{$win_clients_ref}) * 8 ) ) { last; }

        # Get a round-robin client from a list of available clients
        my $win_client = shift( @{$win_clients_ref} );
        push @{$win_clients_ref}, $win_client;

        # Get a round-robin data LIF from a data LIF array
        if ( $previous_vs_name ne $share_obj->vserver() ) {
            $lifs_obj_ref = $self->get_data_lifs_by_protocol(
                               node_obj      => $share_obj->command_interface(),
                               vserver_name  => $share_obj->vserver(),
                               data_protocol => 'cifs' );
            $previous_vs_name = $share_obj->vserver();
        }

        my $lif_obj = shift(@{ $lifs_obj_ref } );
        push @{$lifs_obj_ref}, $lif_obj;

        # Ping from client under test to data lif ip just before mount/map
        my ($win_client_obj, $data_lif_obj) =
        $self->get_reachable_data_lif_and_client_pair(
                                        primary_client_obj  => $win_client,
                                        primary_datalif_obj => $lif_obj,
                                        backup_clients_ref  => $win_clients_ref,
                                        backup_datalifs_ref => $lifs_obj_ref );

        # CIFS share mapping from a windows client
        $Log->comment('Map CIFS shares from a windows client');
        # Workaround to avoid failing
        # multiple credentials System Error 1219 on w2k3 clients
        my $client_map;
        my $win_ver = $win_client_obj->execute( command => 'ver' );
        if ( $win_ver =~ /(Microsoft\sWindows\s\[Version\s5)/x ) {
            # Map Windows version 5.x clients
            $client_map = NACL::STask::Client::Mount->create(
                   command_interface        => $win_client_obj,
                   server_command_interface => $share_obj->command_interface(),
                   share                    => $share_obj->share_name(),
                   vserver                  => $share_obj->vserver(),
                   server                   => $data_lif_obj->{'address'},
                   persistent               => 'NO',
                   password                 => $cifs_user_passwd{cifs_passwd},
                   nacltask_if_exists       => 'reuse' );
        } else {
            # Map Windows version 6.x clients
            $client_map = NACL::STask::Client::Mount->create(
                   command_interface        => $win_client_obj,
                   server_command_interface => $share_obj->command_interface(),
                   share                    => $share_obj->share_name(),
                   vserver                  => $share_obj->vserver(),
                   server                   => $data_lif_obj->{'address'},
                   persistent               => 'NO',
                   user                     => $cifs_user_passwd{cifs_user},
                   password                 => $cifs_user_passwd{cifs_passwd},
                   nacltask_if_exists       => 'reuse' );
        }

        # Update client and mount info into volume object
        push @{$cifs_client_maps}, $client_map;

        $Log->comment('Start CIFS traffic  ' . $num );
        $Log->comment('Windows Map Drive   ' . $client_map->mount_point()
                                             . ' ~ ( '
                                             . $win_client_obj->name()
                                             . " )\n" );

        my %cifs_io_opts = ( %opts );
        $Log->comment('CIFS_TRAFFIC_OPTS: ' . Dumper(\%cifs_io_opts));

        my $drive_letter = $client_map->mount_point();
        my $file_path = $drive_letter . '\cifs_' . $share_obj->share_name();

        if ( $traffic_tool eq 'sio' ) {
            my $sio_file_size;
            # Convert bytes for fsutil file creation
            if ( $cifs_io_opts{file_size} =~ /^(\d+)([kmg])b?$/i ) {
                my %convert_size = %{$convert_to_bytes{lc $2}};
                $sio_file_size = $convert_size{bytes} *= $1;
            } else {
                # Set file_size 40m,
                # if value for 'file_size' is invalid units other than k|m|g
                $cifs_io_opts{file_size} = '40m';
                $sio_file_size = (1024 * 1024) * 40;
            }

            # Actual file size + 1m to make sure that sio file size ">="
            $sio_file_size += 1048576;
            $Log->comment("sio file size = $cifs_io_opts{file_size}");

            # Create a file on windows client for sio read/write access
            $win_client_obj->apiset(interface => 'CLI')->execute_command(
                 command => "fsutil file createnew $file_path $sio_file_size" );

            # Validate 4k block file
            $win_client_obj->apiset(interface => 'CLI')->execute_command(
                        command => "fsutil file setvaliddata $file_path 4096" );
        }

        # Create short subtest logfile names if required
        if ( $self->_short_logfile_names() ) {
            $cifs_io_opts{'runid'} = 'IO_traffic_on_'
                                   . $share_obj->state->volume();
        }

        # Start CIFS traffic with traffic tool options
        $Log->comment("Starting CIFS traffic on $file_path");
        my $cifs_traffic_obj = NACL::MTask::Traffic->start(
                                           command_interface => $win_client_obj,
                                           paths             => $file_path,
                                           tool              => $traffic_tool,
                                           %cifs_io_opts );
        push @{$cifs_traffic_objs}, $cifs_traffic_obj;

        # Set cifs_cifs_io_opts
        $self->_cifs_io_opts(\%cifs_io_opts);
    }

    # Set cifs_traffic_objs and CIFS client maps into main obj
    $self->_cifs_traffic_objs($cifs_traffic_objs);
    $self->_cifs_client_maps($cifs_client_maps);
    $self->_cifs_share_objs($share_objs_ref);

    $Log->exit() if $may_exit;

    return;
}

=head2 _traffic_iscsi_start

   $self->_traffic_iscsi_start(
            lun_objs_ref            => \@lun_objs,
            igroup_objs_ref         => \@igroup_objs | [$igroup_obj]
            iscsi_clients_ref       => \@iscsi_client_objs,
            traffic_luns_per_client => [1-8] );

Private method to start iSCSI traffic and update the parent TrafficHelper object

=over

=item Options

=over

=item C<< lun_objs_ref =E<gt> $lun_objs_array_ref >>

=item C<< igroup_objs_ref =E<gt> $igroup_objs_array_ref >>

=item C<< iscsi_clients_ref =E<gt> $client_objs_array_ref >>

=item C<< traffic_luns_per_client =E<gt> $scalar ~ default limit = 8 >>

 LUN map limit: 8 luns per client

=item C<< iscsi_traffic_opts =E<gt> $hashref >>

=back

=back

=cut

sub _traffic_iscsi_start {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec => {
            lun_objs_ref            => { type => ARRAYREF },
            igroup_objs_ref         => { type => ARRAYREF },
            iscsi_clients_ref       => { type => ARRAYREF },
            traffic_luns_per_client => { type => SCALAR, default => 8 },
            traffic_tool            => {
                                         regex   => '^(dt|sio|hammer)$',
                                         default => 'dt',
                                       },
        },
        allow_extra => 1,
    );
    $Log->enter() if $may_enter;

    my $lun_objs_ref      = delete $opts{lun_objs_ref};
    my $igroup_objs_ref   = delete $opts{igroup_objs_ref};
    my $iscsi_clients_ref = delete $opts{iscsi_clients_ref};
    my $limit             = delete $opts{traffic_luns_per_client};
    my $traffic_tool      = delete $opts{traffic_tool};

    # Delete IO tool specific params
    if ( $traffic_tool ne 'sio' && exists $opts{file_size} ){
        delete $opts{file_size};
    }

    if ( $traffic_tool ne 'dt' && exists $opts{limit} ) {
        delete $opts{limit};
    }

    my $paramset = NATE::ParamSet->new( global => 1 );
    my $is_mcc =  $paramset->get('MCC', default => 1);

    if ($is_mcc){
        $Log->comment('Verify if its a 2 node system');
        if ( !defined $self->_is_2_node() ) {
            my %params;
            $params{node} = ${ $lun_objs_ref }[0]->command_interface();
            $self->_is_2_node($self->_is_2_node_system(\%params));
        }
    }

    $Log->comment('Starting iSCSI traffic');

    my $num = 0;
    my (@iscsi_traffic_objs, @client_dm_pair);
    my %scsi_io_opts = ( %opts );

    # Define some DT tool specific params
    if ( $traffic_tool eq 'dt' ) {
        $scsi_io_opts{'trigger'} =
                       defined $scsi_io_opts{'trigger'}
                       ? $scsi_io_opts{'trigger'}
                       : 'cmd:/x/eng/localtest/noarch/bin/dt_noprog_script.ksh';
        $scsi_io_opts{'privilege-level'} = 'root';
        $scsi_io_opts{'enable'} = 'noprog,compare,lbdata,microdelay,header,nate,timestamp,htiming,stats,deleteperpass,stopimmed,scsi_io';
    }

    $Log->comment('iSCSI_TRAFFIC_OPTS: ' . Dumper(\%scsi_io_opts));

    # Start iSCSI traffic on dm-devices
    foreach my $lun_obj ( @{ $lun_objs_ref } ) {
        $num++;
        # Limit 8 traffic LUNs per client
        if ( $num > ( scalar(@{ $iscsi_clients_ref }) ) * $limit ) { last; }

        # Get a round-robin igroup from igroups
        my $ig_obj = shift( @{ $igroup_objs_ref } );
        push @{ $igroup_objs_ref }, $ig_obj;

        # LUN map to igroups
        $Log->comment('LUN: '
                    . $lun_obj->lun()
                    . ' map to igroup '
                    . $ig_obj->igroup());

        $self->lun_map_to_igroups( lun_objs    => [ $lun_obj ],
                                   igroup_objs => [ $ig_obj ] );

        # Get a round-robin client from a list of available clients
        my $san_client = shift( @{ $iscsi_clients_ref } );
        push @{ $iscsi_clients_ref }, $san_client;

        # Get multipath dm-devices from single path LUNs
        my $dm_dev = $self->get_multipath_dm_devices(
                                                 client_objs => [ $san_client ],
                                                 lun_objs    => [ $lun_obj ] );

        $Log->comment('DEBUG: dm_dev after function call ' . Dumper($dm_dev));

        # Push client-obj and dm-x device pair into AoH data structure
        push @client_dm_pair, { client => $san_client,
                                devdm  => ${$dm_dev}[0],
                                volume => $lun_obj->state->volume() };
    }

    # Print traffic LUN and iSCSI client count
    $Log->comment("LUN map limit per client ~ ( $limit )" );
    $Log->comment('Total traffic LUN count: ' . scalar(@client_dm_pair) );
    $Log->comment('Traffic client count   : ' . scalar(@{$iscsi_clients_ref}) );

    # Start iSCSI traffic on all dm devices
    foreach my $clidm ( @client_dm_pair ) {
        # Create short subtest logfile names if required
        my $lun_name = $clidm->{volume};
        if ( $self->_short_logfile_names() ) {
            $scsi_io_opts{'runid'} = "IO_traffic_on_$lun_name";
        }

        $Log->comment('Starting iSCSI traffic on dm devices '
                    . $clidm->{devdm}
                    . ' ( '
                    . $clidm->{client}->name()
                    . ' )' );
        my $iscsi_traffic_obj = NACL::MTask::Traffic->new(
                                        command_interface => $clidm->{client},
                                        paths             => $clidm->{devdm}, );

        $iscsi_traffic_obj->start( tool => $traffic_tool,
                                   %scsi_io_opts,);

        my $lock_filename = $self->L_create_iSCSI_lock_file(
                                        client_name => $clidm->{client}->name(),
                                        lun_name    => $lun_name );
        push @iscsi_traffic_objs, {
                                     lun_name       => $lun_name,
                                     traffic_obj    => $iscsi_traffic_obj,
                                     lock_file_path => $lock_filename,
                                  };
    }

    # Set iscsi_traffic_objs and maps into main obj
    $self->_iscsi_traffic_objs(\@iscsi_traffic_objs);
    $Log->exit() if $may_exit;

    return;
}

# Method to create iSCSI IO lock file
sub L_create_iSCSI_lock_file {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec => {
            client_name => { type => SCALAR },
            lun_name    => { type => SCALAR },
        },
    );

    my $logdir = q{/tmp/} . $opts{client_name};
    $Log->comment("Creating '$logdir' directory for iSCSI IO lock files");
    make_path( $logdir,
               {
                verbose => 1,
                error   => \my $err_list,
               } );

    my $file_path = "$logdir/$opts{lun_name}.lck";
    $Log->comment("Creating '$file_path' iSCSI IO lock file");
    open my $fh, '>', $file_path
                       || $Log->warn("Failed to create iSCSI IO lock file: $!");

    print $fh, $file_path;
    close $fh || $Log->warn("Failed to close iSCSI IO lock file: $!");

    $Log->comment("Created '$file_path' iSCSI IO lock file");

    return $file_path;
}


# Method to remove existing iSCSI IO lock files
sub L_remove_iSCSI_lock_file {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec => {
            lock_file_path => { type => SCALAR },
        },
    );

    unlink $opts{lock_file_path};
    $Log->comment("Deleted '$opts{lock_file_path}' iSCSI IO lock file");

    return;
}

=head2 verify_windows_clients

    $traffic_helper->verify_windows_clients(
                                          client_objs_ref => $win_clients_ref );

A method to verify a minwin compatible Windows clients for CIFS traffic

=over

=item Options

=over

=item C<< client_objs_ref =E<gt> $win_clients_ref >>

=back

=back

=cut

sub verify_windows_clients {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec => {
            client_objs_ref => { type => ARRAYREF }
        }
    );
    $Log->enter() if $may_enter;

    foreach my $client_obj ( @{$opts{client_objs_ref}} ) {
        my $failure_count = 0;
        if ( $client_obj->is_windows() ) {
            # Verify minwin compatible Windows clients
            $client_obj->apiset(interface => 'CLI')->net_use();

            $Log->comment('Getting version details for SIO and DT utilities ' .
                'from ~ ' . $client_obj->name() );
            my $dt_ver = $client_obj->execute(
                command => q{x:\\bin\\dt.exe version} );

            my $sio_ver = $client_obj->execute(
                command => q{x:\\bin\\sio.exe} );

            if ( ($dt_ver =~ /Version:\s+/) && ($sio_ver =~ /Version:\s+/) ) {
                $Log->comment( $client_obj->name() . ' ~ X:\ drive status: OK');
            } else {
                $failure_count++;
            }
            # If x:\bin\dt & x:\bin\sio not exist then exit from here
            if ( $failure_count ) {
                NACL::MTask::MCC::Exceptions::BaseException->throw(
                    $client_obj->name() . ' is NOT a minwin compatible client');
            }
        }
    }
    $Log->exit() if $may_exit;

    return;
}


=head2 cleanup_windows_map_drives

    $traffic_helper->cleanup_windows_map_drives(
                                          client_objs_ref => $win_clients_ref );

A method to clean up previous disconnected mapped drives to avoid "There are no
available drive letters left" issue

=over

=item Options

=over

=item C<< client_objs_ref =E<gt> $win_clients_ref >>

=back

=back

=cut

sub cleanup_windows_map_drives {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec => {
            client_objs_ref => { type => ARRAYREF }
        }
    );
    $Log->enter() if $may_enter;

    foreach my $client_obj ( @{$opts{client_objs_ref}} ) {
        if ( $client_obj->is_windows() ) {
            # Clean up previous disconnected mapped drives
            # to avoid There are no available drive letters left issue.
            foreach my $drive ( 'e'..'z') {
               # skip default minwin drive letter "X:\>"
               if ($drive eq 'x') { next; }
               $drive .= q{:};
                try {
                    # skip if drive is connected (or) in-use and mapped
                    # e.g. rsh dpg-w2k8r2-3 cmd.exe /c net use y: /d
                    $client_obj->apiset(interface => 'CLI')->net_use(
                        devicename => $drive,
                        delete     => 1 );
                } catch NATE::BaseException with {
                    my $exception = shift;
                    $Log->comment( $exception->text() );
                };
            }
        }
    }
    $Log->exit() if $may_exit;

    return;
}


=head2 get_vserver_objs_ref_from_vol_objs

    my $vs_objs_ref = $traffic_helper->get_vserver_objs_ref_from_vol_objs(
                      vol_objs_ref => $vol_objs_ref );

    my $vs_objs_ref = $self->get_vserver_objs_ref_from_vol_objs(
                      vol_objs_ref => $vol_objs_ref );


A method to fetch vservers belong to volumes and return the vserver objects

=over

=item Options

=over

=item C<< vol_objs_ref =E<gt> $vol_objs_ref >>

=back

=back

=cut

sub get_vserver_objs_ref_from_vol_objs {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec => {
            vol_objs_ref     => { type => ARRAYREF },
            vserver_sub_type => { type => SCALAR | UNDEF, optional => 1 }
        },
    );
    $Log->enter() if $may_enter;

    my $vs_sub_type = delete $opts{vserver_sub_type};
    my $sub_type = defined($vs_sub_type) ? $vs_sub_type : 'sync-source|default';

    # Get all vserver names associated with volumes
    my ($vserver_objs, @dummy_vs_names);
    foreach my $volume_obj ( @{$opts{vol_objs_ref}} ) {
        push @dummy_vs_names, $volume_obj->vserver();
    }
    my %vserver_hash = ();
    my @unique_vserver_names = grep { ! $vserver_hash{$_}++ } @dummy_vs_names;

    # Push unique vservers only
    foreach my $vol_obj ( @{$opts{vol_objs_ref}} ) {
        if ( scalar(@unique_vserver_names) < 1 ) { last; }
        # Debug: if ( grep { /$vol_obj->vserver()/ } @unique_vserver_names ) {
        if ( any { $_ eq $vol_obj->vserver() } @unique_vserver_names ) {
            my $vserver = shift(@unique_vserver_names);
            my $vserver_obj = NACL::CS::Vserver->fetch(
                command_interface =>$vol_obj->command_interface(),
                filter  => { subtype => $sub_type,
                             vserver => $vserver } );
            push @{$vserver_objs}, $vserver_obj;
        }
    }
    $Log->exit() if $may_exit;

    return $vserver_objs;
}


=head2 get_data_lifs_by_protocol

    my $lifs_obj_ref = $traffic_helper->get_data_lifs_by_protocol(
                              node_obj       => $share_obj->command_interface(),
                              vserver_name   => $share_obj->vserver(),
                              address_family => 'ipv4',
                              data_protocol  => 'cifs',
                              refresh_cache  => 1 );

A method to fetch data LIFs belonging to vserver under test by protocol and
return the data LIF object reference

=over

=item Options

=over

=item C<< node_obj =E<gt> $command_interface >>

=item C<< vserver_name =E<gt> $string >>

=item C<< address_family =E<gt> $string >>

=item C<< data_protocol =E<gt> $string >>

=item C<< refresh_cache =E<gt> (0|1) >>
Setting to 1 will get latest details from the controllers, else returns a cached
copy. Default: 0

=back

=back

=cut

sub get_data_lifs_by_protocol {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    $Log->enter() if $may_enter;

    my %opts = validate_with(
        params => \@_,
        spec => {
            node_obj       => { type  => HASHREF },
            vserver_name   => { type  => SCALAR },
            data_protocol  => { regex => '^(nfs|cifs|iscsi)$',default => 'nfs'},
            address_family => { regex => '^(ipv4|ipv6)$', default => 'ipv4' },
            refresh_cache  => { type  => BOOLEAN, default => 0 },
        }
    );

    my $href = $self->_data_lif_comp_states();
    if ( $opts{refresh_cache}
         || !defined $href
         || !defined $href->{$opts{vserver_name}} ) {
        $Log->comment('Fetching latest copies for "net int show" cmd output');

        my @lifs = NACL::CS::NetworkInterface->fetch(
                            command_interface => $opts{node_obj},
                            filter            => {
                                   vserver          => $opts{vserver_name},
                                   role             => 'data',
                                   'address-family' => $opts{address_family},
                                   'data-protocol'  => [ $opts{data_protocol} ],
                                   'status-admin'   => 'up',
                                                 },
                   );

        $href->{$opts{vserver_name}} = \@lifs;
        $self->_data_lif_comp_states($href);

        @lifs = @{$self->_data_lif_comp_states()->{$opts{vserver_name}}};
        foreach my $lif ( @lifs ) {
            $Log->comment('Data LIF IP address : '
                        . $lif->address()
                        . ' ( '
                        . $lif->curr_port()
                        . ' ~ '
                        . $lif->curr_node()
                        . ' ) ');
        }
    } else {
        $Log->comment('Returning cached copies for "net int show" cmd output');
    }

    $Log->exit() if $may_exit;

    return $href->{$opts{vserver_name}};
}


=head2 get_reachable_data_lif_and_client_pair()

    # Ping from client under test to data lif ip just before mount/map
    my ($client_obj, $lif_obj) =
                        $traffic_helper->get_reachable_data_lif_and_client_pair(
                                        primary_client_obj  => $client_obj,
                                        primary_datalif_obj => $lif_obj,
                                        backup_clients_ref  => $client_objs_ref,
                                        backup_datalifs_ref => $lifs_obj_ref );

A method to ping from a primary client to primary data lif just before mount
(or) map. if unreachable then pick up a client and data lif pair from backup
pools. Also ping the new client and data lif again. if reachable then return the
client and data lif pair for traffic.

=over

=item Options

=over

=item C<< primary_client_obj =E<gt> $client >>

=item C<< primary_datalif_obj =E<gt> $data_lif >>

=item C<< backup_clients_ref =E<gt> $arrayref >>

=item C<< backup_datalifs_ref =E<gt> $arrayref >>

=back

=back

=cut

sub get_reachable_data_lif_and_client_pair {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec => {
            primary_client_obj  => { type => HASHREF },
            primary_datalif_obj => { type => HASHREF },
            backup_clients_ref  => { type => ARRAYREF },
            backup_datalifs_ref => { type => ARRAYREF }
        },
    );
    $Log->enter() if $may_enter;

    my $client_obj   = $opts{primary_client_obj};
    my $datalif_obj  = $opts{primary_datalif_obj};
    my $clients_ref  = $opts{backup_clients_ref};
    my $datalifs_ref = $opts{backup_datalifs_ref};

    # Check point for NFS client and Vserver data LIF connection.
    # Ping client to data LIF's ip before NFS mount, if unreachable then
    # pick up a new ip and client from "@data_lifs/@clients" for NFS mount
    $Log->comment('Check point ~ ping ' . $client_obj->name() . ' ---> ' .
                                                    $datalif_obj->address() );
    my $output;
    if ( $client_obj->is_windows() ) {
        $output = $client_obj->apiset(interface => 'CLI')->execute_command(
            command => 'ping -n 5 ' . $datalif_obj->address() );
    } else {
        $output = $client_obj->apiset(interface => 'CLI')->execute_command(
            command          => 'ping -c 5 ' . $datalif_obj->address(),
           'privilege-level' => 'root' );
    }

    if ( ($output =~ /\s100% packet loss/i) || ($output =~ /unknown/i)
                                            || ($output =~ /100% loss/i) ) {
        $Log->comment('*** Unreachable IP: ' . $datalif_obj->address() .
                            ' unreachable from ' . $client_obj->name() );

        foreach my $new_client ( @{$clients_ref} ) {
            foreach my $new_lif ( @{$datalifs_ref} ) {
                $Log->comment('Alternative attempt: Ping ' . $new_client->name()
                    . ' --to--> ' . $new_lif->address() );
                my $result;
                if ( $new_client->is_windows() ) {
                    $result = $new_client->apiset(interface => 'CLI')->
                    execute_command(
                        command => 'ping -n 5 ' . $new_lif->address() );
                } else {
                    $result = $new_client->apiset(interface => 'CLI')->
                    execute_command(
                        command          => 'ping -c 5 ' . $new_lif->address(),
                       'privilege-level' => 'root' );
                }
                if (($result !~ /\s100% packet loss/i) ||
                    ($output !~ /unknown/i) || ( $output !~ /100% loss/i) ) {
                    $client_obj  = $new_client;
                    $datalif_obj = $new_lif;
                    $Log->comment('New client: ' . $client_obj->name() );
                    $Log->comment('New IP    : ' . $datalif_obj->address() );
                    last;
                }
            }
        }
    }
    $Log->exit() if $may_exit;

    return ($client_obj, $datalif_obj);
}


=head2 cifs_traffic_resume()

    # A public method to resume CIFS traffic
    $traffic_helper->cifs_traffic_resume();

A method to resume CIFS traffic on metrocluster disaster site's CIFS shares in
the surviving site after switchover.

B<Required:> "disconnect_maps =E<gt> 0" option when stopped the CIFS traffic.
Example:

 $traffic_helper->traffic_stop( cifs => 1, disconnect_maps => 0 );

=over

=item Options

=back

=over

=item C<< no option required >>

=back

=cut

sub cifs_traffic_resume {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    $Log->enter() if $may_enter;

    # Resume disaster site's CIFS traffic in switchover state
    my $cifs_traffic_objs = $self->_cifs_traffic_objs();
    my %cifs_io_opts = % { $self->_cifs_io_opts() };

    $Log->comment('Resuming disaster site CIFS traffic');
    foreach my $cifs_traffic_obj ( @{$cifs_traffic_objs} ) {
        $cifs_traffic_obj->start( %cifs_io_opts );
        $Log->comment('Successfully resumed CIFS traffic');
    }
    $Log->exit() if $may_exit;

    return;
}


=head2 traffic_stop

    # A public method to stop NFS, CIFS and iSCSI traffic
    $traffic_helper->traffic_stop(
        nfs             => 0|1,
        cifs            => 0|1,
        iscsi           => 0|1,
        disconnect_maps => 0|1,
        delete_luns     => 0|1
    );

A method to stop NFS, CIFS and iSCSI traffic

=over

=item Options

=over

=item C<< nfs =E<gt> 0|1 >>

(Optional) Boolean option to stop NFS traffic. Default: 0

=item C<< cifs =E<gt> 0|1 >>

(Optional) Boolean option to stop CIFS traffic. Default: 0

=item C<< iscsi =E<gt> 0|1 >>

(Optional) Boolean option to stop iSCSI traffic. Default: 0

=item C<< disconnect_maps =E<gt> 0|1 >>

(Optional) Boolean option to keep CIFS client map drives to resume later.
Default: 1

=item C<< delete_luns =E<gt> 0|1 >>

(Optional) Boolean option to delete iSCSI LUNs. Default: 0

=back

=back

=cut

sub traffic_stop {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec   => {
            nfs                  => { type => SCALAR, default => 0 },
            cifs                 => { type => SCALAR, default => 0 },
            iscsi                => { type => SCALAR, default => 0 },
            disconnect_maps      => { type => SCALAR, default => 1 },
            delete_luns          => { type => SCALAR, default => 0 },
            cleanup_iscsi_client => { type => SCALAR, default => 1 }
        }
    );

    # Stop NFS traffic
    if ( $opts{nfs} ) {
        my $nfs_traffic_objs = $self->_nfs_traffic_objs();
        my $io_handle_objs   = $self->_io_handle_objs();

        $Log->comment('Stopping NFS traffic and deleting NFS mounts');
        foreach my $nfs_traffic_obj ( @{$nfs_traffic_objs} ) {
            $nfs_traffic_obj->stop( if_stopped                  => 'pass',
                                    graceful_shutdown_timeout   => 600 );
        }

        # Sleep a few sec. before purge the mount points
        $Log->comment('Sleep 60 secs before unmount');
        Tharn::snooze(60);

        # Delete NFS mounts
        $Log->comment('Unmount NFS mounts');
        foreach my $io_handle_obj ( @{$io_handle_objs} ) {
            try {
                $io_handle_obj->delete_mount( force => 1 );
            } catch NATE::BaseException with {
                $Log->comment(':: Unmount exception ::');
                my $exception = shift;
                $Log->warn($exception->text());
            };
        }
    }

    # Stop CIFS traffic
    if ( $opts{cifs} ) {
        my $cifs_traffic_objs = $self->_cifs_traffic_objs();
        $Log->comment('Stopping CIFS traffic');
        foreach my $cifs_traffic_obj ( @{$cifs_traffic_objs} ) {
            $cifs_traffic_obj->stop( if_stopped                 => 'pass',
                                     graceful_shutdown_timeout  => 600 );
        }
        # Wait a few seconds after traffic stopped
        Tharn::snooze(60);

        # Disconnect map drives
        if ( $opts{disconnect_maps} ) {
            my $cifs_client_maps = $self->_cifs_client_maps();
           $Log->comment('Disconnecting MAP drives from windows clients');
            foreach my $cifs_client_map ( @{$cifs_client_maps} ) {
                $cifs_client_map->purge();
            }
        }
    }

    # Stop iSCSI traffic
    if ( $opts{iscsi} ) {
        my $iscsi_traffic_objs = $self->_iscsi_traffic_objs();
        my $lun_igroup_pairs   = $self->_get_lun_igroup_maps();

        $Log->comment('Stopping iSCSI traffic and unmapping luns');
        foreach my $iscsi_traffic_obj ( @{$iscsi_traffic_objs} ) {
            try {
                $iscsi_traffic_obj->{traffic_obj}->stop(
                                            if_stopped                => 'pass',
                                            graceful_shutdown_timeout => 120 );
            } catch NATE::BaseException with {
                my $exception = shift;
                $Log->warn('Had to force stop the IO process for lun '
                         . $iscsi_traffic_obj->{lun_name}
                         . "\n"
                         . $exception->text() );
            };

            # Remove iSCSI lock file
            $self->L_remove_iSCSI_lock_file(
                       lock_file_path => $iscsi_traffic_obj->{lock_file_path} );
        }

        # Wait a few seconds after traffic stopped
        Tharn::snooze(10);

        # LUN delete ( default value is 0 )
        # TestbedSetupHelper.pm will take care of this task
        if ( $opts{delete_luns} ) {
            $Log->comment('deleting LUNs');
            foreach my $lun_obj ( @{$lun_igroup_pairs} ) {
                $lun_obj->{lun}->purge();
            }
        }

        # Delete iSCSI target IP(s) from client side
        # iscsiadm -m discovery -o delete -p <IP_ADDRESS>
        if ( $opts{cleanup_iscsi_client} ) {
            # Get iSCSI clients
            my $iscsi_clients = $self->_get_iscsi_client_objs();

            # Get vservers associated with LUNs under test
            my @lun_objects = map { $_->{lun} } @{$lun_igroup_pairs};
            my $vservers = $self->get_vserver_objs_ref_from_lun_objs(
                                                    lun_objs => \@lun_objects );

            # Deleting iscsi target IP(s) from all clients
            # We will not be checking IP is mounted on client or not so,
            # ignore results from here.
            my @all_iscsi_lifs;
            my $client_tgt_mapping = $self->_client_tgt_login_mapping();
            if (defined $client_tgt_mapping->{targets}) {
                @all_iscsi_lifs =  @{ $client_tgt_mapping->{targets} };
            }

            foreach my $client ( @{$iscsi_clients} ) {
                my $client_apiset = $client->apiset(interface => 'CLI');

                foreach my $lif (@all_iscsi_lifs) {
                    $Log->comment("Logout from IP '$lif' on client "
                                . $client->name());
                    foreach my $iqn (@{ $client_tgt_mapping->{iqn} }) {
                        $Log->comment("Working on iqn '$iqn'");
                        try {
                             $client_apiset->iscsiadm(
                                mode                => 1,
                                node                => 1,
                                logout              => 1,
                                'target-name'       => $iqn,
                                'ip-port'           => $lif,
                                'privilege-level'   => 'root',
                                'connectrec-timeout'=> 300 );

                        } catch NATE::BaseException with {
                            my $exception = shift;
                            $Log->warn( $exception->text() );
                        };
                    }

                    $Log->comment("Deleting iSCSI mounts for IP : $lif");
                    try {
                        $client_apiset->iscsiadm(
                                           operation         => 'delete',
                                           'specific-mode'   => 'discovery',
                                           'ip-port'         => $lif,
                                           'privilege-level' => 'root');
                    } catch NATE::BaseException with {
                        my $exception = shift;
                        $Log->warn( $exception->text() );
                    };
                    $Log->comment('... done');
                }

                # Restart multipath daemon if there is no active iSCSI IO on
                # the client
                my $logdir = q{/tmp/} . $client->name();
                $Log->comment("Looking into $logdir for iSCSI IO lock files");
                opendir(DIR, $logdir)
                         || $Log->warn('Unable to open IO lock file directory');
                if ( scalar(grep { $_ ne q{.} && $_ ne q{..} }
                                                          readdir(DIR)) == 0 ) {
                    $Log->comment('No lock files found. Restarting iSCSI '
                                . 'multipath daemon');
                    $self->L_restart_daemons();
                    closedir(DIR);
                    remove_tree($logdir,
                                {
                                  verbose => 1,
                                  error   => \my $err_list,
                                });
                } else {
                    $Log->comment('Found lock files, skipping iSCSI multipath '
                                . 'daemon restart');
                    closedir(DIR);
                }
            }
        }
    }

    return;
}

=head2 client_multipath_start

   $traffic_helper->client_multipath_start( client_obj => $client );

A method to start client multipath daemon and also throw the exception if unable
to start client multipath daemon.

=over

=item Options

=over

=item C<< client_obj =E<gt> $client >>

NACL::C::Client object handle

=back

=back

=cut

sub client_multipath_start {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec   => {
                    client_obj => { type => HASHREF }
                  }
    );

    $Log->enter() if $may_enter;
    my $client = $opts{client_obj};
    my $client_apiset = $client->apiset(interface => 'CLI');

    $Log->comment('Checking client multipath device mapper kernel driver '
                . 'package installations');
    my $response = $client_apiset->rpm( 'package'  => 'device-mapper-multipath',
                                        'query'    => 1,
                                        'info'     => 1,
                                        'requires' => 1,
                                        'provides' => 1, );

    if ( $response =~ /not installed/ ) {
        my $msg = 'multipath device mapper driver package is not installed!';
        NATE::BaseException->throw($msg);
    }

    my $ver_response = $client->execute( command => 'cat /etc/redhat-release' );
    my $release;

    if ($ver_response =~ m/.* (\d\.\d) .*/) {
        $release = $1;
    } else {
        croak('Unexpected version format found');
    }

    $release =~ s/^\s+|\s+$//;

    my $timestamp = strftime '%Y%m%d_%H%M%S', localtime;
    $Log->comment('Client ' . $client->name() . ' release is : ' . $release);
    # Make a backup copy of existing /etc/multipath.conf file
    $Log->comment('Backing up current client multipath device mapper '
                . 'configurations');

    try {
      $client_apiset->mv( source            => '/etc/multipath.conf',
                          target            => "/etc/multipath.conf_$timestamp",
                          'privilege-level' => 'root', );
    } catch NATE::BaseException with {
        my $ex = shift;
        if ($ex->text() =~ /No such file or directory/) {
            $Log->warn('Unable to find existing multipath.conf file. This is a '
                     . "bad client. File an engsup ticket for this client!\n"
                     . $ex->text() );
        }
    };

    # Due to the way the library is written there is no way to determine
    # at this point if the test is being run on a 2 node. Due to this I
    # cannot mandate linux 6.4 for 2 node system. This needs to be mandates
    # in the plan file.
    if ($release < 6.4) {
        # Copy the template from p4 repo to the client
        $Log->comment('Copying golden client multipath device mapper configuration '
                    . 'template into /etc path');
        my $template_config_prefix = 'dp/mcc/multipath.conf';
        my $own_current_filename = (caller())[1];
        (my $own_current_tree = $own_current_filename ) =~
                                       s/lib\/NACL\/MTask\/MCC\/TrafficHelper.pm$//;

        my @lib_trees = split(/[;:]/,$ENV{NATE_LIB_UNIX});
        my $use_tree = undef;

        foreach( $own_current_tree, @lib_trees, '/usr/software/test/noarch/dev/' ) {
            if( -f "$_/$template_config_prefix" ) {
                $use_tree = $_;
                last;
            }
        }
        $client_apiset->cp(source            => "$use_tree/$template_config_prefix",
                           dest              => '/etc/multipath.conf',
                           force             => 1,
                           verbose           => 1,
                           'privilege-level' => 'root',);

        # Enable configuration file for device mapper multipath.
        $Log->comment('Modifying certain client multipath device mapper '
                    . 'configuration attributes');
        my $cmd = q{sudo /sbin/mpathconf --enable --find_multipaths y};
        $client_apiset->execute_command( command => $cmd );
    } else {
        $Log->comment('Create an empty multipath.conf file');
        $client->execute( command => 'sudo touch /etc/multipath.conf');
    }

    $Log->comment('Delete all existing unused multipath sessions from client');
    try {
        # Logout of all active sessions
        $client_apiset->iscsiadm( 'specific-mode'   => 'node',
                                  'logout-all'      => 'all',
                                  'privilege-level' => 'root' );
        $client_apiset->iscsiadm( 'specific-mode'   => 'node',
                                  'operation'       => 'delete',
                                  'privilege-level' => 'root', );
    } catch NATE::BaseException with {
        my $exception = shift;
        $Log->warn( $exception->text() );
    };

    $Log->comment('Restart client multipath device mapper service');
    $client_apiset->service( script            => 'multipathd',
                             action            => 'restart',
                             'privilege-level' => 'root', );

    $Log->comment('Verify client multipath device mapper daemon status');
    $response = $client_apiset->service( script            => 'multipathd',
                                         action            => 'status',
                                         'privilege-level' => 'root', );
    my $output = $response->get_processed_output();

    if ( $output =~ /(multipathd\s+\(pid\s+\d+\)\s+is\s+running || Active:\s+\[\d+\;\d+mactive\s+\(running\))/i ) {
        $Log->comment('multipath device mapper service is running well!');
    } else {
        NATE::BaseException->throw('Client multipath device mapper daemon '
                                 . 'failed to start');
    }

    # Restore original multipath configuration file
    $Log->comment('Restoring original client multipath device mapper '
                . 'configurations');
    try {
        $client_apiset->mv(
                          source            => "/etc/multipath.conf_$timestamp",
                          target            => '/etc/multipath.conf',
                          force             => 1,
                          verbose           => 1,
                          'privilege-level' => 'root', );
    } catch NATE::BaseException with {
        my $ex = shift;
        if ($ex->text() =~ /No such file or directory/) {
            $Log->warn('This is a bad client! We will retain the '
                     . "multipath.conf golden template copy on this client.\n"
                     . $ex->text() );
        }
    };

    # Flush all unused multipath device maps
    $Log->comment('Flushing all unused multipath device maps');
    try {
        $client_apiset->multipath( 'flush-all'       => 1,
                                   'privilege-level' => 'root', );
    } catch NATE::BaseException with {
        my $exception = shift;
        $Log->warn( $exception->text() );
    }

    $Log->comment('Successfully configured multipath device mapper drivers on '
                . $client->name());

    $Log->exit() if $may_exit;

    return;
}


=head2 get_multipath_dm_devices

    my $dm_dev = $traffic_helper->get_multipath_dm_devices(
                                                client_objs => [ $san_client ],
                                                lun_objs    => [ $lun_obj ]
                                            );

A method to get multipath dm-devices from single path LUNs and return the
multipath devices (/dev/dm-0 etc..) associated with LUNs under test

=over

=item Options

=over

=item C<< client_objs =E<gt> $client_objs_array_ref >>

=item C<< lun_objs =E<gt> $lun_objs_array_ref >>

=back

=back

=cut

sub get_multipath_dm_devices {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec => {
            client_objs => { type => ARRAYREF },
            lun_objs    => { type => ARRAYREF }
        }
    );
    $Log->enter() if $may_enter;

    # Check input array contents
    if (scalar @{$opts{client_objs}} < 1 || scalar @{$opts{lun_objs}} < 1) {
        NACL::MTask::MCC::Exceptions::BaseException->throw(
                                  'Received empty CLIENT or LUN object arrays');
    }

    # Get vservers associated with LUNs under test
    my $vservers = $self->get_vserver_objs_ref_from_lun_objs(
                    lun_objs => $opts{lun_objs});

    # SAN client iscsiadm discovery for dm-x devices
    foreach my $v ( @{ $vservers }) {
        # Get vserver iSCSI target iqn name
        my $vserver_scsi_obj = NACL::CS::VserverIscsi->fetch(
                      command_interface =>  $v->command_interface(),
                      filter            => { vserver => $v->vserver() } );

        my $target_iqn_name = $vserver_scsi_obj->{'target_name'};
        $Log->comment('Vserver iSCSI target iqn name: ' . $target_iqn_name .
                ' ( ' . $v->vserver() . ' )' );

        # Get iSCSI target IPs belong vserver under test
        my $ips = $self->get_iscsi_data_lifs_ips(
            vserver_obj       => $v,
            convert_data_lifs => 0 );

        $self->_login_to_all_targets (
            clients    => $opts{client_objs},
            target_iqn => $target_iqn_name,
            ips        => $ips);
    }

    # SAN client iscsiadm discovery for dm-x devices
    my $dm_devices;

    foreach my $client ( @{ $opts{client_objs} } ) {
        my $client_apiset = $client->apiset(interface => 'CLI');
        # Verify client multipath before fetching multipath status
        $Log->comment('Verifying client multipath daemon status');
        try {
            $client_apiset->service( script            => 'multipathd',
                                     action            => 'status',
                                     'privilege-level' => 'root' );
        } catch NATE::BaseException with {
            my $exception = shift;
            $Log->comment( $exception->text() );

            $Log->comment('Restarting client multipath daemon...');
            $client_apiset->service( script => 'multipathd',
                           action           => 'start',
                          'privilege-level' => 'root' );
            Tharn::snooze(5);
            $client_apiset->service( script => 'multipathd',
                           action           => 'status',
                          'privilege-level' => 'root' );
        };

        # SAN LUN fetch multipath devices from a client
        my @sanlun_objs;
        my $counter = 10; # Retry counter for SAN LUN multi-path discovery

        do {
            $Log->comment('Client sanlun fetch. Iteration : '
                       . (11-$counter));
            foreach my $lun ( @{ $opts{lun_objs} } ) {
                $Log->comment('DEBUG: Working on LUN: ' . $lun->path());
                my $count = 0;
                RETRY: {
                    $count++;
                    # Get sanlun lun show output
                    try {
                        @sanlun_objs = NACL::CS::Client::Sanlun->fetch(
                            command_interface => $client,
                            filter            => {
                                 lun_pathname => $lun->path()},
                            allow_empty       => 1 );
                    } catch NATE::BaseException with {
                        my $exception = shift;
                        $Log->comment( $exception->text() );
                    };

                    # Restart Filer iSCSI service
                    if (( ! scalar(@sanlun_objs) ) && ( $count <= 3 )) {
                        $Log->comment('Restart iSCSI service ~ ' .
                                       $count);
                        my %args = (
                        command_interface => $lun->command_interface(),
                        vserver           => $lun->vserver() );

                        NACL::C::VserverIscsi->stop(%args);
                        NACL::C::VserverIscsi->start(%args);
                        Tharn::snooze(10);

                        if ($count == 3) {
                            $Log->comment("Count is equal to $count, restarting client iSCSI services as the last attempt");
                            #Restart Client ISCSI services
                            my $Client_API = $client->apiset();
                            $Client_API->set_timeout("connectrec-timeout" => 360);
                            $Client_API->execute_command('command' => 'sudo /etc/init.d/iscsi restart');
                        }

                        redo RETRY;
                    }

                    if ( ! scalar(@sanlun_objs) ) {
                        NATE::BaseException->throw(
                        'Unable to discover sanlun for LUN: '
                        . $lun->path() .
                        ' ~ Reboot client '
                        . $client->name
                        . ' and try again!');
                    }
                }
                $Log->comment('Single path: ' . $lun->path());

                # Fetch multipath devices that belong to LUN under test
                $Log->comment('Multipath path fetch on ' .
                               'client under test');
                my @mps = NACL::CS::Client::Multipath->fetch(
                command_interface => $client,
                allow_empty       => 1 );

                # Initialize each sanlun lun show output and match with
                # client multipath show output.
                # If /dev/sdx found "twice" from same LUN then push
                # to dm-x list
                my $index = 0;
                foreach my $slun (@sanlun_objs) {
                    my $dm;
                    if ( ($dm) =
                        grep {$_->{device} eq $slun->{device_filename}}
                                                                    @mps ) {

                        $index++;
                        $Log->comment('Device found from same LUN: '
                                    . $index
                                    . ' ( '
                                    . $lun->lun()
                                    . ' )');
                        $Log->comment('Multipath: ' . $dm->{device} );
                        $Log->comment('Devnode: ' . $dm->{devnode} );
                        $Log->comment('Unique host id: ' . $dm->{wwid} );
                        $Log->comment('Sanlun   : '
                                    . $slun->{device_filename} );

                        if ( $index > 1  ||
                             ($self->_is_2_node() && $index == 1)) {
                            my ($dm_dev) = $dm->{wwid} =~ /^(\w+)\s*/;
                            push @{$dm_devices},
                                  '/dev/mapper/' . $dm_dev;
                            # Got multipath dm device so leaving from here
                            last;
                        }
                    }
                }

                # Verify multipath device and exit on error
                if ( not defined $dm_devices ) {
                    $Log->warn('NO multipath dm device found for LUN: '
                             . $lun->lun()
                             . "\nRetrying after 5 seconds...");
                    Tharn::snooze(5);
                }
            }
            $counter--;
        } while ( not defined $dm_devices and $counter > 0 );

        if ( not defined $dm_devices ) {
            NATE::BaseException->throw('Some SAN LUNs are missing multipath'
                                     . ' dm device paths');
        }

        # Print Multipath dm-x devices
        foreach my $dm_device ( @{$dm_devices} ) {
            $Log->comment('device names: ' . $dm_device );
        }
    }
    $Log->exit() if $may_exit;

    return $dm_devices;
}


=head2 get_iscsi_data_lifs_ips

    my $iscsi_ips_aref = $traffic_helper->get_iscsi_data_lifs_ips(
                                                      vserver_obj => $vserver );

A method to get iSCSI DATA LIF IP addresses from HA parter nodes. It also
converts existing non-iSCSI data LIFs to iSCSI protocol if "convert_data_lifs"
param is set to '1'

=over

=item Options

=over

=item C<< vserver_obj =E<gt> $vserver >>

(Required) NACL::C::Vserver object for which iSCSI DATA LIF is to be created

=item C<< convert_data_lifs =E<gt> 0|1 (default: 1) >>

(Optional) If set would convert an existing data LIF to iSCSI protocol if
requried

=item C<< create_failover_lif =E<gt> 0|1 (default: 1) >>

(Optional) If set will create second iSCSI failover data LIF for same vserver
but hosted over HA pair node

=back

=item Exceptions

=over

=item C<NACL::MTask::MCC::Exceptions::BaseException>

This type of exception is thrown when there is no existing iSCSI data LIF and
"convert_data_lifs" options is set to '0'

=back

=back

=over

=item Returns

Returns an ArrayRef to array containing newly converted iSCSI data LIF IPs

=back

=cut

sub get_iscsi_data_lifs_ips {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec => {
            vserver_obj         => { type => HASHREF },
            create_failover_lif => { type => BOOLEAN, default => 1 },
            convert_data_lifs   => { type => BOOLEAN, default => 1 }
        }
    );

    $Log->enter() if $may_enter;
    #$Log->comment('Debug comment for change #2626626...');
    my @nodes = ();
    my @iscsi_data_lif_ips = ();

    # Modify failover iSCSI lifs on HA partner
    my $create_failover_lifs = $opts{create_failover_lif};
    my $convert_data_lifs = $opts{convert_data_lifs};
    my $vserver = $opts{vserver_obj};
    my $ha_node = $vserver->command_interface();

    # If node is not in HA setup then ignore exception by empty catch block
    my $ha_partner_node;
    try {
        $ha_partner_node = NACL::STask::Node->get_partner_obj(
                                                 command_interface => $ha_node);
    } catch NATE::BaseException with {};

    if ( (defined $ha_partner_node) and $create_failover_lifs ) {
        push @nodes, ($ha_node, $ha_partner_node);
        $Log->comment('Creating iSCSI protocol LIF on both nodes in HA pair : '
            . $ha_node->name()
            . q{,}
            . $ha_partner_node->name() );
    } else {
        push @nodes, $ha_node;
        $Log->comment('Creating iSCSI protocol LIF only on VServer node : '
            . $ha_node->name() );
    }

    my $net_if_reverted = 0;
    foreach my $node ( @nodes ) {
        my $local_data_vserver = NACL::CS::Vserver->fetch(
                                command_interface => $node,
                                        filter  => { subtype => 'sync-source|default',
                                        vserver => $vserver->vserver(),
                            'allowed-protocols' => ['iscsi'] } );
        my $existing_iscsi_lifs = NACL::CS::NetworkInterface->fetch(
                       command_interface => $node,
                             allow_empty => 1,
                                  filter => {
                               'data-protocol' => ['iscsi'],
                                   'home-node' => $node->name(),
                                       vserver => $local_data_vserver->vserver()
                                            } );

        if ( $existing_iscsi_lifs ) {
            push @iscsi_data_lif_ips, $existing_iscsi_lifs->address();
            $Log->comment('Found existing iSCSI DATA LIF. Skipping iSCSI LIF '
                        . 'creation on '
                        . $node->name() );
            $Log->comment('iSCSI Target portal address : '
                        . $existing_iscsi_lifs->address() );
        } else {
            if ( ! $convert_data_lifs ) {
                $Log->warn('Did not find any existing iSCSI protocol DATA LIF '
                         . 'and param "convert_data_lifs" is set to false');
                NACL::MTask::MCC::Exceptions::BaseException->throw(
                                             'Unable to create iSCSI DATA LIF');
            }

            $Log->comment('Did not find any existing iSCSI protocol DATA LIF. '
                        . 'Converting an existing DATA LIF to iSCSI protocol');

            # Make sure that all data network interfaces are in home nodes
            if ( !$net_if_reverted ) {
                try {
                    NACL::C::NetworkInterface->revert(
                        command_interface => $node,
                                      lif => q{*},
                                  vserver => $local_data_vserver->vserver );
                } catch NATE::BaseException with {
                    my $exception = shift;
                    $Log->comment( $exception->text() );
                };
            }
            $net_if_reverted = 1;

            my $lif_cs = NACL::CS::NetworkInterface->fetch(
               command_interface => $node,
                          filter => { 'role' => 'data',
                                 'home-node' => $node->name(),
                            'address-family' => 'ipv4',
                                     vserver => $local_data_vserver->vserver });

            my $lif = NACL::STask::NetworkInterface->find_data_interfaces(
                command_interface => $node,
                           filter => { 'lif' => $lif_cs->lif(),
                               'status-oper' => 'up',
                                 'home-node' => $node->name(),
                                     vserver => $local_data_vserver->vserver });
            $lif->purge();

            NACL::STask::NetworkInterface->create(
                            command_interface => $node,
                                          lif => $lif_cs->lif(),
                                      vserver => $local_data_vserver->vserver(),
                                         role => 'data',
                                  'home-node' => $lif_cs->home_node(),
                                  'home-port' => $lif_cs->home_port(),
                                      address => $lif_cs->address(),
                                      netmask => $lif_cs->netmask(),
                              'data-protocol' => ['iscsi'],
                               'status-admin' => 'up',
                              nacltask_verify => 1 );
            push @iscsi_data_lif_ips, $lif_cs->address();
            $Log->comment('iSCSI Target portal address ' . $lif_cs->address() );
        }
    }

    $Log->exit() if $may_exit;

    # Return array ref to @iscsi_data_lif_ips
    return \@iscsi_data_lif_ips;
}

=head2 igroup_create

    my $igroup = $traffic_obj->igroup_create(
        vserver_obj     => $vserver_obj,
        os_type         => 'linux', ( or 'windows' )
        protocol        => 'iscsi', ( or 'fcp' )
        igroup_index    => '001',   ( ig_001, ig_002 etc... )
        if_ig_exists    => 'reuse',
        initiator_names => \@initiator_names );

A method to create an igroup and return the igroup object.

=over

=item Options

=over

=item C<< vserver_obj =E<gt> $vserver_obj >>

=item C<< os_type =E<gt> linux|windows >>

=item C<< protocol =E<gt> iscsi|fcp >>

=item C<< igroup_index =E<gt> $scalar >>

=item C<< if_ig_exists =E<gt> reuse|purge >>

=item C<< initiator_names =E<gt> initiator names array ref >>

=back

=back

=cut

sub igroup_create {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec => {
        vserver_obj     => { type => HASHREF },
        initiator_names => { type => ARRAYREF },
        os_type         => { type => SCALAR , optional => 1,default => 'linux'},
        protocol        => { regex => '^(iscsi|fcp|mixed)$',default => 'iscsi'},
        igroup_index    => { regex => qr/^\d+$/, optional => 1, default => 1 },
        if_ig_exists    => { regex => '^(reuse|purge)$', default => 'purge'}
        }
    );
    $Log->enter() if $may_enter;

    my $test_vserver   = $opts{vserver_obj};
    my $os_type        = $opts{os_type};
    my $protocol       = $opts{protocol};
    my $ig_index       = $opts{igroup_index};
    my $initiatornames = $opts{initiator_names};
    my $option         = $opts{if_ig_exists};

    # Create igroup and add initiator name
    my $ig_prefix = $test_vserver->vserver();
    $ig_prefix =~ s/_//g;
    my $igroup = NACL::STask::LunIgroup->create(
        command_interface  => $test_vserver->command_interface(),
        vserver            => $test_vserver->vserver(),
        igroup             => 'ig_'. $ig_prefix . '_' . $ig_index,
        protocol           => $protocol,
        ostype             => $os_type,
        nacltask_if_exists => $option );

    # Add client iSCSI initiator iqn name into igroup
    try {
        NACL::C::LunIgroup->add(
            command_interface => $test_vserver->command_interface(),
            vserver           => $test_vserver->vserver(),
            igroup            => $igroup->igroup(),
            initiator         => $initiatornames );
    } catch NATE::BaseException with {
        my $exception = shift;
        $Log->comment( $exception->text() );
    };

    $Log->exit() if $may_exit;

    return $igroup;
}


=head2 lun_map_to_igroups

    $traffic_helper->lun_map_to_igroups(
        lun_objs    => [ $lun_obj ],
        igroup_objs => [ $ig_obj ] );

    $traffic_helper->lun_map_to_igroups(
        lun_objs    => \@lun_objs,
        igroup_objs => \@ig_objs );

A method to perform LUN map to igroups and return the LUN and  mapped igroup
pair in AoH data structure.

=over

=item Options

=over

=item C<< lun_objs =E<gt> LUN objs array ref >>

=item C<< igroup_objs =E<gt> igroup objs array ref >>

=back

=back

=cut

sub lun_map_to_igroups {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec => {
            lun_objs    => { type => ARRAYREF },
            igroup_objs => { type => ARRAYREF }
        },
        allow_extra => 1
    );
    $Log->enter() if $may_enter;

    # Get vservers associated with LUNs under test
    my $vsrs = $self->get_vserver_objs_ref_from_lun_objs(
        lun_objs => $opts{lun_objs} );

    # Get iSCSI target data LIFs from vservers
    foreach my $vserver ( @{$vsrs} ) {
        $self->get_iscsi_data_lifs_ips( vserver_obj => $vserver );
    }
    my $lun_igroup_pair;
    foreach my $lun_obj ( @{$opts{lun_objs}} ) {
        # Get round robin igroup name
        my $igroup_obj = shift( @{$opts{igroup_objs}} );
        push @{$opts{igroup_objs}}, $igroup_obj;

        # LUN map and reuse if it exists
        $lun_obj->map( igroup => $igroup_obj,
            nacltask_if_mapped_igroup => 'reuse',
            nacltask_verify           => 1 );

        # Record LUN objs associate with igroup name into AoH data structure
        push @{$lun_igroup_pair}, {
            lun    => $lun_obj,
            igroup => $igroup_obj->igroup() };
    }
    # Set LUN and igroup mapped table into main obj (for LUN unmap)
    $self->_get_lun_igroup_maps($lun_igroup_pair);
    $Log->exit() if $may_exit;

    return $lun_igroup_pair;
}


=head2 get_vserver_objs_ref_from_lun_objs

    my $vservers = $traffic_helper->get_vserver_objs_ref_from_lun_objs(
                                                       lun_objs => \@lun_objs );

A method to get vservers associated with LUNs under test and return the vserver
objs reference.

=over

=item Options

=over

=item C<< lun_objs =E<gt> LUN objs array ref >>

=item C<< igroup_objs =E<gt> igroup objs array ref >>

=back

=back

=cut

sub get_vserver_objs_ref_from_lun_objs {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec => {
            lun_objs => { type => ARRAYREF },
        },
    );
    $Log->enter() if $may_enter;

    # Get all vserver names associated with volumes
    my ($vserver_objs, @dummy_vs_names);
    foreach my $lun_obj ( @{$opts{lun_objs}} ) {
        push @dummy_vs_names, $lun_obj->vserver();
    }
    my %vserver_hash = ();
    my @unique_vserver_names = grep { ! $vserver_hash{$_}++ } @dummy_vs_names;

    # Push unique vservers only
    foreach my $lunObj ( @{$opts{lun_objs}} ) {
        if ( scalar(@unique_vserver_names) < 1 ) { last; }
        if ( any { $_ eq $lunObj->vserver() } @unique_vserver_names ) {
            my $vserver = shift(@unique_vserver_names);
            my $vserver_obj = NACL::CS::Vserver->fetch(
                command_interface =>$lunObj->command_interface(),
                filter  => { vserver => $vserver } );
            push @{$vserver_objs}, $vserver_obj;
        }
    }
    $Log->exit() if $may_exit;

    return $vserver_objs;
}

# need to redefine the routine here. It would have been better if this library
# took a TestbedSetupHelper object but ... for now redefinig it here.
sub _is_2_node_system {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with (
        params => \@_,
        spec   => {
                node => { type => HASHREF },
        },
    );

    my $dr_group_to_nodes;

    my $n = $opts{node};
    my @MetroclusterNode_states = NACL::CS::MetroclusterNode->fetch(
                                command_interface => $n, );

    foreach my $e (@MetroclusterNode_states) {
        my $dr_group = $e->{'dr_group_id'};
        my $node = $e->{'node'};
        push @{$dr_group_to_nodes->{$dr_group}}, $node;
    }

    $Log->comment('dr_group_id->node is: ' . Dumper($dr_group_to_nodes));

    $Log->comment('determining if its a 2 node config \n');

    my $key_count = keys %{$dr_group_to_nodes};
    my $is2node = 0;

    $Log->comment('Key count is: ' .  $key_count) ;

    if ($key_count == 1) {
        $Log->comment('Verify if there are only 2 nodes in the dr group');
        foreach my $k (keys %{$dr_group_to_nodes}) {
            my $node_cnt = scalar(@{$dr_group_to_nodes->{$k}});
            $Log->comment('no of nodes in the dr group : $k is: ' .
                           $node_cnt);
            if ($node_cnt == 2) {
                $is2node = 1;
            }
        }
    }

    if ($is2node) {
        $Log->comment(' .... yes its a 2 node system');
    }

    return $is2node;
}

sub _login_to_all_targets {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    my %opts = validate_with(
        params => \@_,
        spec => {
            clients   => { type => ARRAYREF },
            ips       => { type => ARRAYREF },
            target_iqn => { type => SCALAR },
        }
    );

    my $target_iqn_name = $opts{target_iqn};
    $Log->comment('mapping at the start: ' . Dumper( $self->_client_tgt_login_mapping()));
    my $client_tgt_login = $self->_client_tgt_login_mapping();

    $Log->comment('Make a record of the iqn');
    push (@{ $client_tgt_login->{iqn} }, $target_iqn_name)
         if !grep{$target_iqn_name eq $_} @{ $client_tgt_login->{iqn} };


    $Log->comment('Establishing iSCSI session with all IPs');
    foreach my $client ( @{ $opts{clients} } ) {
        $Log->comment('iSCSI discovery from ' . $client->name());
        my $client_apiset = $client->apiset(interface => 'CLI');
        # Performing iSCSI target discovery
        $Log->comment('iscsiadm logout / login');
        foreach my $target_ip ( @{$opts{ips}} ) {
            if (!defined $client_tgt_login->{$client->name()}{$target_ip}) {
                $client_apiset->iscsiadm(
                    type                => 'sendtargets',
                    mode                => 1,
                    discovery           => 1,
                   'ip-port'            => $target_ip,
                   'privilege-level'    => 'root',
                   'connectrec-timeout' => 300 );

                $client_apiset->iscsiadm(
                    mode                => 1,
                    node                => 1,
                    logout              => 1,
                   'target-name'        => $target_iqn_name,
                   'ip-port'            => $target_ip,
                   'privilege-level'    => 'root',
                   'connectrec-timeout' => 300 );

                $client_apiset->iscsiadm(
                    mode                => 1,
                    node                => 1,
                    login               => 1,
                   'target-name'        => $target_iqn_name,
                   'ip-port'            => $target_ip,
                   'privilege-level'    => 'root',
                   'connectrec-timeout' => 300 );

                $Log->comment('Make a record of the client <-> ip mapping');
                $client_tgt_login->{$client->name()}{$target_ip} = 1;

                $Log->comment('Make a reocrd of the target IP');
                push(@{ $client_tgt_login->{targets} } , $target_ip)
                    if !grep{$target_ip eq $_} @{ $client_tgt_login->{targets} };
            } else {
                    $Log->comment('Discovery already performed for this ' .
                                  'client ... returning ');
            }
        }
    }

    $self->_client_tgt_login_mapping($client_tgt_login);
    $Log->comment('Updated mapping at the end : ' .
                    Dumper( $self->_client_tgt_login_mapping()));
    return;
}

# Method to restart certain client daemons post IO stop procedure
sub L_restart_daemons {
    my $self = shift;

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer(); #metrics

    # Restart the iSCSI multipath daemon on all IO clients
    # Do this only when IO is running for MCC 4-pack setup
    if ( defined $self->_iscsi_traffic_objs() && !$self->_is_2_node() ) {
        my $iscsi_clients = $self->_get_iscsi_client_objs();
        foreach my $client ( @{$iscsi_clients} ) {
            my $client_apiset = $client->apiset(interface => 'CLI');

            $Log->comment('Deleting all iSCSI multipath sessions');
            try {
                # Flush all unused multipath device maps
                $client_apiset->multipath('flush-all'       => 1,
                                          'privilege-level' => 'root');
            } catch NATE::BaseException with {
                my $exception = shift;
                $Log->warn('Error while flushing multipath device maps: '
                         . $exception->text() );
            };

            $Log->comment('Restart client multipathd service to activate '
                        . 'original configurations');
            $client_apiset->service( script            => 'multipathd',
                                     action            => 'restart',
                                     'privilege-level' => 'root' );

            my $output = $client_apiset->service(
                                            script            => 'multipathd',
                                            action            => 'status',
                                            'privilege-level' => 'root'
                                                      )->get_processed_output();

            if ( $output !~ /multipathd\s+\(pid\s+\d+\)\s+is\s+running/ ) {
                $Log->warn('Client multipathd daemon failed to start with '
                         . 'original configuration settings');
            }
        }
    }

    return;
}

1;
#END
