#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroCluster Check Config Diff Validation
## @author dl-mcc-dpg-qa@netapp.com
## @status shared
## @pod here

package NACL::MTask::MCC::Validator::CheckConfigVserverDiff;

use strict;
use warnings;


use parent qw(NACL::MTask::MCC::Validator::ClusterBase);

use NATE::BaseException qw(:try);

use Params::Validate
    qw(validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use Data::Dumper;

=head1 NAME

NACL::MTask::MCC::Validator::CheckConfigVserverDiff

=head1 SYNOPSIS

  my $vtor = NACL::MTask::MCC::Validator::CheckConfigVserverDiff->new(
    metrocluster => $mcc
  );

  $vtor->run();

=head1 DESCRIPTION

This class runs and validates the results of the metrocluster check
command 'metrocluster check config-vserver diff show'


=head1 METHODS

=head2 new()

  my $vtor = NACL::MTask::MCC::Validator::CheckConfigVserverDiff->new(
     metrocluster => $mcc,
     disaster_status => $disaster_status
  );

Creates a new NACL::MTask::MCC::Validator::CheckConfigVserverDiff object.

=over

=item Options

=over

=item C<metrocluster>

(Required.) A NACL::MTask::MCC::Metrocluster object on which we'll do
validations.

=item C<disaster_status>

Hashref. This should be the disaster status as reported by
$mcc-E<gt>get_disaster_status( include_HA =E<gt> 1 ). If
C<disaster_status> is not provided or is undefined, the validator will
call get_disaster_status() on the Metrocluster object itself.

=back

=back

(Copied from Base)

=head2 validate()

Runs the metrocluster check config-vserver diff show command and validates the results.
Note that this is an alias for 'vserver config diff show' so it may run that.

If the Metrocluster's DR state is 'normal', run checks on
a random cluster.

=cut

sub validate {
    my $self = shift;

    if ( $self->is_normal() ) {
        #pick a random cluster to run on
        $self->_validate_check_config_vserver_diff(
            clusters => [ $self->random_component() ]
        );
    }

    return;
}

##
## _validate_check_config_vserver_diff()
##
##
## Runs 'metrocluster check config-vserver diff show' on a random cluster
##
sub _validate_check_config_vserver_diff {
    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'clusters'        => {'type' => ARRAYREF }
        }
    );
    foreach my $cluster ( @{ $opts{'clusters'} } ) {
        my $ci = $cluster->get_Metrocluster_component()->command_interface();
        my $apiset = $ci->apiset(interface => 'CLI');

        #metrocluster check config vserver diff is an alias for this command
        my @config_vserver_diff;
        try {
            @config_vserver_diff =
            $apiset->vserver_config_diff_show(
                'no-progress' => 'true',
                'connectrec-timeout' => 60*30,
            );
            #$Log->comment( "CONFIG VSERVER DIFF:" .
            #    Dumper \@config_vserver_diff);
        } catch NACL::APISet::Exceptions::NoMatchingEntriesException with {
        } catch NATE::BaseException with {
            $Log->comment('Error getting check config vserver diff, ignoring');
        };

        foreach my $diff ( @config_vserver_diff ) {
            unless ($diff->get_processed_output() =~ /There are no entries matching your query/i) {
                foreach my $output (@{$diff->get_parsed_output()}) {
                    my @attr = $self->_get_response_array($output,'attribute');
                    my @src  = $self->_get_response_array($output,
                                         'source-vserver-value');
                    my @dest = $self->_get_response_array($output,
                                         'destination-vserver-value');
                    my @errs =
                        map {"   vserver diff $_:" . (shift @src) . ',' . (shift @dest)} @attr;
                    my $err_header = "src vserver:$output->{'source-vserver-name'} " .
                        "dest vserver:$output->{'destination-vserver-name'} " .
                        "config(object):$output->{object} " .
                        "keys:$output->{keys}\n";
                    $Log->comment($err_header . join "\n", @errs);
                    $self->errors_push(
                        $err_header . join "\n", @errs,
                    );
                }
            }

        }

    }

    # Errors are recorded in errors(). We return void.
    return;

}

#coerce parsed output to always be an array
sub _get_response_array {
    my $self   = shift;
    my $output = shift;
    my $name   = shift;

    my $attr = $output->{$name};
    my @attr = ref $attr ? @{$attr} : ($attr);

    return @attr;
}

=head2 can_run_validation()

Should return true if we should run this validation, and false if we
should skip it. Should print a warning indicating why the validation
will be skipped if it returns false.

By default, returns true if we are in a stable state (normal)
false otherwise.

=cut

#technically this is true if both nodes are up even in a weird state but we
#can start here TODO
sub can_run_validation {
    my $self = shift;

    if ( $self->is_normal() ) {
        return 1;
    }
    else {
        $Log->comment('Unable to run validation '
                       . $self->validation_type()
                           . ': DR state '
                              . $self->dr_state()
                                  . ' not supported.');

        return;
    }
}

=head1 SEE ALSO

=over

=item *

L<NACL::MTask::MCC::Validator::Base
|NACL::MTask::MCC::Validator::Base>

=item *

L<NACL::MTask::MCC::Validator::ClusterBase
|NACL::MTask::MCC::Validator::ClusterBase>

=back

=cut

1;
