#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroCluster ClusterShow Validation
## @author dl-mcc-dpg-qa@netapp.com
## @status shared
## @pod here

package NACL::MTask::MCC::Validator::ClusterShow;

use strict;
use warnings;

use parent qw(NACL::MTask::MCC::Validator::ClusterBase);

use Params::Validate
    qw(validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF);

=head1 NAME

NACL::MTask::MCC::Validator::ClusterShow

=head1 SYNOPSIS

  my $vtor = NACL::MTask::MCC::Validator::ClusterShow->new(
    metrocluster => $mcc
  );

  $vtor->run();

=head1 DESCRIPTION

This class runs and validates the results of the 'cluster show'
command.

In 'normal' and 'heal roots completed' modes, both nodes A and B
should be healthy.

In other modes, the survivor site should be healthy and the disaster
site is unreachable.

This class extends NACL::MTask::MCC::Validator::ClusterBase and
NACL::MTask::MCC::Validator::Base.

=head1 METHODS

=head2 new()

  my $vtor = NACL::MTask::MCC::Validator::ClusterShow->new(
     metrocluster => $mcc,
     disaster_status => $disaster_status
  );

Creates a new NACL::MTask::MCC::Validator::ClusterShow object.

=over

=item Options

=over

=item C<metrocluster>

(Required.) A NACL::MTask::MCC::Metrocluster object on which we'll do
validations.

=item C<disaster_status>

Hashref. This should be the disaster status as reported by
$mcc-E<gt>get_disaster_status( include_HA =E<gt> 1 ). If
C<disaster_status> is not provided or is undefined, the validator will
call get_disaster_status() on the Metrocluster object itself.

=back

=back

(Copied from Base)

=head2 validate()

  $vtor->validate();

Runs the cluster show command and validates the results.

If the Metrocluster's DR_state is 'normal' or 'heal_roots_completed',
checks that both clusters are healthy.

Otherwise, checks that the surviving cluster is healthy.

=cut

sub validate {
    my $self = shift;

    if ( $self->is_normal() ) {
        foreach my $cluster_state ( $self->all_states() ) {
            $self->_check_health($cluster_state);
        }
    }

    else {
        # Other states, check survivor side
        foreach my $cluster_state ($self->survivor_states()) {
            $self->_check_health($cluster_state);
        }
    }

    # If we're in post-heal-root, we should check disaster as well
    if ( $self->is_heal_roots_completed() ) {
        foreach my $cs ( $self->disaster_states() ) {
            $self->_check_health($cs);
        }
    }

    return;

} ## end sub validate

##
## _check_health()
##
##   $vtor->_check_health($cluster_state)
##
## Makes sure that the health state is true on $cluster_state.
##
sub _check_health {
    my $self = shift;
    my ($cluster_state) = @_;

    $cluster_state->health() eq 'true'
        or $self->errors_push(
            q{Health of node '} . $cluster_state->node()
            . q{' was '} . $cluster_state->health() . q{'}
        );

    return;

}

=head1 SEE ALSO

=over

=item *

L<NACL::MTask::MCC::Validator::Base
|NACL::MTask::MCC::Validator::Base>

=item *

L<NACL::MTask::MCC::Validator::ClusterBase
|NACL::MTask::MCC::Validator::ClusterBase>

=back

=cut

1;
