#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroCluster VolumeStatus Validation
## @author dl-mcc-dpg-qa@netapp.com
## @status shared
## @pod here

package NACL::MTask::MCC::Validator::VolumeStatus;

use strict;
use warnings;

use parent qw(NACL::MTask::MCC::Validator::ClusterBase);

use Params::Validate
    qw(validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::CS::Volume;

=head1 NAME

NACL::MTask::MCC::Validator::VolumeStatus

=head1 SYNOPSIS

  my $vtor = NACL::MTask::MCC::Validator::VolumeStatus->new(
    metrocluster => $mcc
  );

  $vtor->run();

=head1 DESCRIPTION

This class runs 'volume show' before and after a DR operation and
compares whether the state has changed.

If it has changed (for example from online to offline) then the
validation will fail.

#In 'normal' and 'heal roots completed' modes, both nodes A and B
#should be healthy.
#
#In other modes, the survivor site should be healthy and the disaster
#site is unreachable.
#
#This class extends NACL::MTask::MCC::Validator::ClusterBase and
#NACL::MTask::MCC::Validator::Base.
#
=head1 METHODS

=head2 new()

  my $vtor = NACL::MTask::MCC::Validator::VolumeStatus->new(
     metrocluster => $mcc,
     disaster_status => $disaster_status
  );

Creates a new NACL::MTask::MCC::Validator::VolumeStatus object.

=over

=item Options

=over

=item C<metrocluster>

(Required.) A NACL::MTask::MCC::Metrocluster object on which we'll do
validations.

=item C<disaster_status>

Hashref. This should be the disaster status as reported by
$mcc-E<gt>get_disaster_status( include_HA =E<gt> 1 ). If
C<disaster_status> is not provided or is undefined, the validator will
call get_disaster_status() on the Metrocluster object itself.

=back

=back

(Copied from Base)

=head2 validate()

  $vtor->validate();

Runs the cluster show command and validates the results.

If the Metrocluster's DR_state is 'normal' or 'heal_roots_completed',
checks that both clusters are healthy.

Otherwise, checks that the surviving cluster is healthy.

=cut

sub validate {
    my $self = shift;

    if ( $self->is_normal() ) {
        foreach my $cluster_state ( $self->all_states() ) {
            $self->_check_vol_status($cluster_state);
        }
    }

    else {
        # Other states, check survivor side
        foreach my $cluster_state ($self->survivor_states()) {
            $self->_check_vol_status($cluster_state);
        }
    }

    # If we're in post-heal-root, we should check disaster as well
    if ( $self->is_heal_roots_completed() ) {
        foreach my $cs ( $self->disaster_states() ) {
            $self->_check_vol_status($cs);
        }
    }

    return;

} ## end sub validate

sub can_run_validation {

    my $self = shift;

    if (! scalar keys %{$self->prepared_data()} ) {
        #we didn't prepare before dr operation for some reason,
        #no data to compare against
        $Log->warn('prepare was not called for this validation prior to dr' .
            ' operation so validation cannot be run');
        return;
    }

    #do regular checks also
    return $self->SUPER::can_run_validation(@_);

}

=head2 prepare()

BEFORE the dr operation is performed, prepares the validation that will be
run after the dr operation occurs.

Most validations do not require a preparation step so defined as a noop
in the base class.  If your validation requires a preparation step
(for example gathering data from before the dr operation to compare
after the dr operation in run) then override this method.

=cut

sub prepare {
    my $self = shift;

    $Log->comment('preparing data for volume status validation');
    if ( $self->is_normal() ) {
        foreach my $cluster_state ( $self->all_states() ) {
            $self->_prepare_vol_status($cluster_state);
        }
    }

    else {
        # Other states, prepare survivor side
        foreach my $cluster_state ($self->survivor_states()) {
            $self->_prepare_vol_status($cluster_state);
        }
    }

    # If we're in post-heal-root, we should prepare disaster as well
    if ( $self->is_heal_roots_completed() ) {
        foreach my $cs ( $self->disaster_states() ) {
            $self->_prepare_vol_status($cs);
        }
    }

    return $self->prepared_data();
}

##
## _check_vol_status()
##
##   $vtor->_check_vol_status($cluster_state)
##
## Makes sure that the health state is true on $cluster_state.
##
sub _check_vol_status {
    my $self = shift;
    my ($cluster) = @_;

    my $volumes = $self->_get_vol_state($cluster);

    my $prepared_data = $self->prepared_data();

    foreach my $volume (keys %{$volumes}) {
        if (!$prepared_data->{$volume}) {
            $Log->comment("volume $volume had no state in prepared data " .
                'before DR operation, likely was a mirrored volume and it ' .
                'was not enabled or the node was down, ignoring...');
        } elsif (($volumes->{$volume} eq 'online') &&
                 ( $prepared_data->{$volume} eq '-')) {
            $Log->comment("volume $volume was not enabled and is now " .
                'online, likely was a mirrored volume and it ' .
                'is now enabled after the dr operation, ignoring...');

        } elsif (($volumes->{$volume} eq '-') &&
                 ( $prepared_data->{$volume} eq 'online')) {
            $Log->comment("volume $volume was online and is now " .
                'not enabled, likely was a mirrored volume and it ' .
                'is now not enabled after the dr operation, ignoring...');

        } elsif ($volumes->{$volume} ne $prepared_data->{$volume}) {
            my $err = "volume $volume state changed: " .
                "'$prepared_data->{$volume}' => '$volumes->{$volume}'";
            $self->errors_push($err);
            $Log->comment("vol state bad: $prepared_data->{$volume} ne $volumes->{$volume}");
        }
        else {
            #$Log->comment("vol state good:$volumes->{$volume}");
        }
    }

    return;

}
##
## _prepare_vol_status()
##
##   $vtor->_prepare_vol_status($cluster_state)
##
## Makes sure that the health state is true on $cluster_state.
##
sub _prepare_vol_status {
    my $self = shift;
    my ($cluster) = @_;

    my $volumes = $self->_get_vol_state($cluster);

    #These should be unique so no collisions?

    my $data = $self->prepared_data();

    %{$data} = (%{$data}, %{$volumes});
    $self->prepared_data($data);


    return;

}
##
## _get_vol_state()
##
##   $vtor->_get_vol_state($cluster_state)
##
## gets the volume show state for every vserver/volume pair
##
sub _get_vol_state {
    my $self = shift;
    my ($cluster) = @_;


    my @vol_states = NACL::CS::Volume->fetch(
        command_interface =>
            $cluster->command_interface(),
        requested_fields => [ 'state', 'vserver', 'volume', ],
    );

    #for now save as "vserver volume"
    my %vol_hash = map {  $_->vserver() . ' ' . $_->volume() => $_->state() }
                       @vol_states;

    return \%vol_hash;

}

=head1 SEE ALSO

=over

=item *

L<NACL::MTask::MCC::Validator::Base
|NACL::MTask::MCC::Validator::Base>

=item *

L<NACL::MTask::MCC::Validator::ClusterBase
|NACL::MTask::MCC::Validator::ClusterBase>

=back

=cut

1;
