#
#   Copyright (c) 2001-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary PacketCapture Task Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::MTask::PacketCapture

=head1 DESCRIPTION

C<NACL::MTask::PacketCapture> provide methods to start, stop and
show packet capture functionality on ONTAP and Linux/Windows/solaris Host.
This task supportes both ipv4 and ipv6 protocol.

Many testcases requires packet capture operation on clients or filers.
This package provides methods for start, stop, show and cleanup.

    # Use case 1 : Start the Packet capture on all machines and all
    # all interfaces
    use NACL::MTask::PacketCapture;
    ...
    my @arguments = (
                      { target_machine => $filer_interface },
                      { target_machine => $linux_interface },
                      { target_machine => $windows_interface }
                    );
    my $packetcapture = NACL::MTask::PacketCapture->new (packet_capture_details => \@arguments);

    # Start the capturing on all interface
    $packetcapture->start();
    ...
    # Perform any operations here
    ...
    # Stop the packet capture
    $packetcapture->stop();

    # Got the all packets
    my $get_matched_packet=$packetcapture->show();
    # deletes all the files
    $packetcapture->cleanup();
    # End test


    # Use case 2 : Start the packet capture on filers and linux with specified
    filters, interfaces
    use NACL::MTask::PacketCapture;
    ...
    my @arguments = (
                      { target_machine => $filer_interface,
                        interface_filter => ['e0c'],
                        capture_filter=> ['packet-length' => "1792",'ipaddress' => "10.123.123.32"],
                        display_filter => '-tttt'},
                        nacl_task_delay_after_stop => 10
                      },
                      { target_machine   => $linux_interface,
                        interface_filter => ['eth0'],
                        capture_filter   => 'port 80',
                        display_filter   => '-tttt'},
                        nacl_task_delay_after_stop =>10
                      }
                    );
    my $packetcapture = NACL::MTask::PacketCapture->new ( packet_capture_details => \@arguments);

    # Start the capturing on all interface
    $packetcapture->start();
    ...
    # Perform any operations here
    ...
    # Stop the packet capture
    $packetcapture->stop();

    # Got the all packets
    my $get_matched_packet=$packetcapture->show();
    # deletes all the files
    $packetcapture->cleanup();
    # End test

=cut

package NACL::MTask::PacketCapture;

use strict;
use warnings;

use base qw(NACL::MTask::MTask);

use NACL::MTask::SCP;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate
    qw(validate validate_with SCALAR ARRAYREF UNDEF HASHREF OBJECT);
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ResponseException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NATE::BaseException;
use NACL::ComponentUtils qw(Dumper);
use NATE::Process;
use NATE::Util qw(quote_argv_sh quote_argv_cmd);
use constant DEFAULT_PATH => '/x/eng/ctl_logs/pushbutton/general';
use NACL::CS::NetworkPort;

use Array::Utils qw(:all);
use feature 'state';

use NATE::ParamSet qw( param_global);
=head1 METHODS

=head2 new

    my @arg=({target_machine   => $command_interface,
          interface_filter => ['eth0'],
          capture_filter   => 'dst 10.181.140.216 and port 22',
          display_filter   => '-tttt',
          capture_tool   => 'tethereal',
          nacl_task_delay_after_stop => 10});
    my $packetcapture = NACL::MTASK::PacketCapture->new(packet_capture_details => \@arg);

    Create a PacketCapture object that can be used to start, stop and
    show.

=over

=item Options

=over

=item C<< packet_capture_details = > \@arg >>

    (Required) This contains all the details for packet capture.

    my @arg=({target_machine   => $command_interface,
              interface_filter => ['eth0'],
              capture_filter   => 'dst 10.181.140.216 and port 22',
              display_filter   => '-tttt',
              capture_tool   => 'tethereal',
              nacl_task_delay_after_stop => 10});
    my $packet_capture=NACL::MTask::PacketCapture(packet_capture_details => \@arg);

=over

=item C<< target_machine => $command_interface >>

    (Required) As C<NACL::C::Node | NACL::C::Client>A component object that
    represents the node to which to start the packet capture.

=item C<< interface_filter => ['eth0'] >>

    (Optional) List of Network interface to which start the packet capture.

=item C<< capture_filter   => $capture_filter >>

    (Optional) A capture filter use to specify the types of network information you want to monitor.
    For example, to see only a specific subset of computers or protocols.
    For Linux/Windows client(SCALAR): This capture filter are same as 'tcpdump capture filter' or
    'capture tool capture filter' for clinets (see 'tcpdump --help' for more detail).
        my $filter = 'src port 22';

    For Filers (HASHREF): The capture filters are mention below:
        my $filter={'packet-length' => "1792",
                    'buffer-size' => "737",
                    'ipaddress' => "10.123.123.32"};

=item C<< display_filter   => $display_filter >>

    (Optional) A Display filter use to filter the packets for show() function.

=item C<< capture_tool     => $packet_capturing_tool_bin >>

    (Optional)  A command of packet capturing tool . It can contains the tool name with location.
    Supported Tools: tcpdump/tethereal/tshark.exe
    Default values:
    For Linux/Solaris  : tcpdump
    For Windows        : "c:\Program Files\Wireshark\tshark.exe"

=item C<< nacl_task_delay_after_stop     => $nacl_task_delay_after_stop >>
    (Optional) delay in number of seconds,as Packet capture file takes some time to be created

=item C<< capture_length   => $snaplen >>

    (Optional) size/length(in bytes) of each packets being capture.

=item C<< capture_ring_buffer   => $hashref  (windows/linux clients only) >>

    (Optional) When set to 1, this will enable packet trace file ring buffers. This will allow the
    ability to have the trace file rotate to a new file when the 'capture_ring_buffer_file_size_kbytes'
    has been achieved. To reduce unneeded packet trace files, only the most recent 'capture_ring_buffer_file_count'
    files will be retained.
    Default: By default this disabled (0)

=item C<< capture_ring_buffer_file_size_kbytes => $size_in_kbytes (windows/linux clients only) >>

    (Optional) If 'capture_ring_buffer' is enabled, this controls the maximum file
    size in kilobytes before rotating the ring buffer. By default this is set to 100*1024 (100MB)

=item C<< capture_ring_buffer_file_count => $scalar (windows/linux clients only) >>

    (Optional) If 'capture_ring_buffer' is enabled, this controls the number of
    files to retain in the file ring buffer. By default this is set to '1'.

=item C<< precleanup_all_traces => $scalar (windows/linux clients only) >>

    (Optional) When set to 1, will cleanup any left over packet trace files before starting
    the packet capture. By default this is disabled by being set to 0.

=item C<< custom_apiset   => $apiset (linux client only) >>

    (Optional) a custom apiset that will keep the library from creating a new apiset,
    the provided apiset will be used for calls to client. This argument only works
    for NFS client tracing.

=item C<< output_dir   => $dir >>

    (Optional) A directory that will be used to store the resulting capture file.
    This should be a valid directory for linux clients or a valid UNC path for windows clients.

=item C<< capture_name  => $capture_name >>

    (Optional) Packet capture file will be named based on the name provided.
    Actual packet capture file name will be different as it will be appended
    with time stamp, interface and extension.

=back

=back

=item Exceptions:

    Throws the following exceptions:
    NATE::Exceptions::Argument - when missing required options

=back

=cut

use Class::MethodMaker [
    new   => [ '-hash', '-init', 'new' ],
    hash  => 'packet_capture_params',
    array => 'packet_capture_details',
    array => 'pktt_files_before_start',
    array => 'pktt_files_after_stop',
    array => 'generated_pktt_files',
];

sub init {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %tmp = validate_with(
        params => \@_,
        spec   => { packet_capture_details => { type => ARRAYREF } },
    );

    $Log->debug( sub { "Packet Capture Arguments for new" . Dumper( \%tmp ) }
    );
    my $count = 0;
    my $opts  = {};
    foreach my $capture_option ( @{ $tmp{packet_capture_details} } ) {
        $count++;
        %{ $opts->{"target_$count"} } = validate_with(
            params => $capture_option,
            spec   => {
                target_machine =>
                    { type => OBJECT, isa => "NACL::C::CommandInterface" },
                custom_apiset =>
                    { type => OBJECT, isa => "NACL::APISet", optional => 1 },
                interface_filter      => { type => ARRAYREF, default => ['all'] },
                capture_filter        => { type => SCALAR | HASHREF, optional => 1 },
                display_filter        => { type => SCALAR,           optional => 1 },
                capture_tool          => { type => SCALAR,           optional => 1 },
                capture_length        => { type => SCALAR,           optional => 1 },
                capture_name          => { type => SCALAR,           optional => 1 },
                capture_ring_buffer   => { type => SCALAR,           optional => 1 },
                capture_ring_buffer_file_size_kbytes =>
                    { type => SCALAR, default  => 100*1024 },
                capture_ring_buffer_file_count =>
                    { type => SCALAR, default  => 1 },
                precleanup_all_traces => { type => SCALAR,           default  => 0 },
                output_dir            => { type => SCALAR,           optional => 1 },
                nacl_task_delay_after_stop => {type => SCALAR,       default => 5},
                %{ NACL::C::Component->_common_validate_spec_without_ci()
                    }
            },
        );
    } ## end foreach my $capture_option ...
    foreach my $key ( keys %$opts ) {
        if ( defined( $opts->{$key}->{capture_tool} ) ) {
            NATE::Exceptions::Argument->throw(
                      "Unsupported tool $opts->{$key}->{capture_tool} "
                    . " Only tcpdump/tshark/tethereal tools are supported" )
                if ( $opts->{$key}->{capture_tool}
                !~ /(tcpdump)|(tshark)|(tethereal)/i );
        } ## end if ( defined( $opts->{...}))
        if ( $opts->{$key}->{target_machine}->hostrec()->hosttype()
            =~ /windows/ && !defined( $opts->{$key}->{capture_tool} ) )
        {
            $opts->{$key}->{capture_tool}
                = 'c:\PROGRA~1\Wireshark\tshark.exe';
        } elsif (
            ( $opts->{$key}->{target_machine}->hostrec()->is_unix() || $opts->{$key}->{target_machine}->hostrec()->hosttype() =~ /filer/)
            && !defined( $opts->{$key}->{capture_tool} ) )
        {
            $opts->{$key}->{capture_tool} = 'tcpdump';
            my $apiset = $opts->{$key}->{target_machine}->apiset(interface => "CLI");
            my $sudo_path = Tharn::param("SUDO_PATH");
            my $output;
            my $ex_caught=0;
            try {
                $output = $apiset->execute_raw_command(
                            command => $sudo_path .' tcpdump --help',
                            'connectrec-timeout' => 180 );
            } catch NATE::BaseException with {
                $Log->debug("tcpdump command is taking too much time to execute, So falling back to /usr/software/sbin/tcpdump");
                $ex_caught = 1;
            };
            if( $ex_caught || $output =~ /No such file or directory|Command not found/i ) {
                $Log->comment( "Unable to find tcpdump bin on "
                      . $opts->{$key}->{target_machine}->name()
                      . ". So falling back to /usr/software/sbin/tcpdump" );
                $opts->{$key}->{capture_tool} = '/usr/software/sbin/tcpdump';
            }

        } ## end elsif ( $opts->{$key}->{target_machine...})
    } ## end foreach my $key ( keys %$opts)
    $self->packet_capture_params($opts);
    $Log->debug( sub { "packet_capture_params " . Dumper($opts) } );
    $Log->exit() if $may_exit;
} ## end sub init

=head2 start

    $packetcapture->start();

    (Instance method)
    Start the packet capture process on all the clients/filers and dump the packets on tmporary files.
    This does not have any return value.

    Parameter WIN_PACKET_CAPTURE_ADDITIONAL_LOGGING (defualt = 0)
    It is used when any additional logging information for CPU and Memory load is required for windows packet capture.

=cut

sub start {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my $task_params = $pkg->packet_capture_params();
    while ( my ( $target, $params ) = each(%$task_params) ) {
        if ( ( $params->{target_machine}->hostrec()->is_unix())
	     || ( $params->{target_machine}->hostrec()->hosttype() =~ /filer/ && $params->{target_machine}->is_fusen() ) )
        {
            $pkg->_start_on_linux_or_filer_tcpdump( $target, $params );
        } elsif (
            $params->{target_machine}->hostrec()->hosttype() =~ /windows/ )
        {
            $pkg->_start_on_windows( $target, $params );
        } elsif (
            $params->{target_machine}->hostrec()->hosttype() =~ /filer/ )
        {
            $pkg->_start_on_filer( $target, $params );
        } else {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw(
                "Packet Capture can start only on unix/windows/filers machine not on "
                    . $params->{target_machine}->hostrec()->hosttype() );
        } ## end else [ if ( $params->{target_machine...})]
    } ## end while ( my ( $target, $params...))
    $Log->exit() if $may_exit;
} ## end sub start

# Start the packet capture process on Filers

sub _start_on_filer {

    $Log->enter() if $may_enter;

    my $self         = shift;
    my $target_count = shift;
    my $params       = shift;

    my $command_interface = $params->{target_machine};
    my $capture_interface = $params->{interface_filter};
    my $capture_filter    = $params->{capture_filter};
    my $capture_length    = $params->{capture_length};

    my $apiset_obj = $command_interface->get_7m_or_nodescope_apiset();

    my @files = $self->_get_pktt_files(
        dir => $params->{capture_filter}->{'directory-path'},
        command_interface => $command_interface,
    );

    $self->pktt_files_before_start(@files);

    $capture_interface = _validate_interface($capture_interface);
    my %capture_param = ();
    foreach my $_interface (@$capture_interface) {

        $Log->debug(
            sub {
                "Starting packet capture on "
                    . $command_interface->node()
                    . " with pktt start $_interface  options:"
                    . Dumper($capture_filter);
            }
        );
        $capture_param{interface} = $_interface;
        $capture_param{'packet-length'} = $capture_length
            if ( defined($capture_length) );
        $apiset_obj->pktt_start(
            %capture_param,
            %$capture_filter
        );
    } ## end foreach my $_interface (@$capture_interface)
    $Log->exit() if $may_exit;
} ## end sub _start_on_filer

sub _get_pktt_files {
    my ($self, @args) = @_;

    state $spec = {
        dir => {type => SCALAR|UNDEF,},
        command_interface => { type => OBJECT, isa => "NACL::C::CommandInterface" },
    };

    my %opts = validate_with(
        spec => $spec,
        params => \@args,
    );

    my $command_interface = $opts{command_interface};

    my $apiset_obj = $command_interface->get_systemshell_apiset();
    my $node = $command_interface->node;

    my $dir = '/mroot';
    if (defined $opts{dir}) {
        $dir .= $opts{dir};
    }

    my @files = ();
    my $resp;
    try {
        $resp = $apiset_obj->ls(paths => "$dir/*.trc");
        @files = @{$resp->get_parsed_output->[0]->{files}};
    } catch NACL::APISet::Exceptions::InvalidParamValueException with {
        # Dont fail if there are no trc files
    };

    return @files;
}
# Start the packet capture process on linux

sub _start_on_linux_or_filer_tcpdump {
    $Log->enter() if $may_enter;
    my $self         = shift;
    my $target_count = shift;
    my $params       = shift;
    my $command_interface   = $params->{target_machine};
    my $capture_interface   = $params->{interface_filter};
    my $capture_filter      = $params->{capture_filter};
    my $capture_tool        = $params->{capture_tool};
    my $capture_length      = $params->{capture_length};
    my $capture_name        = $params->{capture_name};
    my $apiset              = $params->{custom_apiset};
    my $output_dir          = $params->{output_dir};
    my $capture_ring_buffer = $params->{capture_ring_buffer};
    my $capture_ring_buffer_file_size_kbytes =
        $params->{capture_ring_buffer_file_size_kbytes};
    my $capture_ring_buffer_file_count =
        $params->{capture_ring_buffer_file_count};
    my $precleanup          = $params->{precleanup_all_traces};
    my $nacl_task_delay_after_stop = $params->{nacl_task_delay_after_stop};
    my $ifconfig_op;
    my @interfaces        = ();
    my $interface_to_file = {};
    my $interface_to_pid  = {};
    my $cmd               = '';
    # validating the 'all' option
    $capture_interface = _validate_interface($capture_interface);
    my $hosttype = $params->{target_machine}->hostrec()->hosttype();
    my $set = _set_type(host_type  => $hosttype);
    if( $set =~ /Filer/i ) {
        $apiset = $command_interface->get_systemshell_apiset();
    } else {
        $apiset = $command_interface->apiset(
            category  => "Host",
            interface => "CLI",
            set       => $set,
        ) unless ($apiset);
    }
    # if requested to precleanup all traces, do it here
    if ($precleanup) {
        my $cli_name = $command_interface->client();
        $cli_name =~ s/\-/_/g;
        $apiset->execute_raw_command(command => 'rm -f ' . $cli_name  .'*.pcap*',
                                     'connectrec-check_status' => 1);
    }

    # Getting the all interfaces id of system
    if ( $params->{target_machine}->hostrec()->hosttype() =~ /solaris/ ) {
        if ( $capture_interface->[0] eq 'all' ) {
            $ifconfig_op = $apiset->ifconfig( 'all' => 1 );
            foreach my $intf_details ( @{ $ifconfig_op->{parsed_output} } ) {
                $intf_details->{interface} =~ s/\s+//g;
	        $intf_details->{interface} =~ s/:\d+//g;
                push @interfaces, $intf_details->{interface};
            }
            $capture_interface = \@interfaces;
        } ## end if ( $capture_interface...)
    } else {
	if ( $capture_interface->[0] eq 'all' ) {
            $ifconfig_op = $apiset->ifconfig();
            foreach my $intf_details ( @{ $ifconfig_op->{parsed_output} } ) {
                if ( $set eq 'Filer' ) {
	            $intf_details->{name} =~ s/\s+//g;
		    push @interfaces, $intf_details->{name};
                }
                else {
                    $intf_details->{interface} =~ s/\s+//g;
                    push @interfaces, $intf_details->{interface};
                }
	    }
	    $capture_interface = \@interfaces;
	} ## end if ( $capture_interface...)
    } ## end if ( $params->{target_machine}...)
    # creating the file name
    my $sudo_path = Tharn::param("SUDO_PATH");
    $sudo_path = "sudo" if ($set eq "Filer");
    foreach my $_interface_id (@$capture_interface) {
	if ( $params->{target_machine}->hostrec()->hosttype() =~ /filer/ && $params->{target_machine}->is_fusen() ) {
	    next if ( $_interface_id !~ /^e.+\w+$/ );
	    # fetching the vsid for the particular port
	    my $vsid = $self->_get_vsid($params,$_interface_id);
	    my $ifconfig_path = "/sbin/ifconfig";
	    my @param = ($sudo_path, "vcontext");
	    push @param, ( '-v', $vsid, $ifconfig_path, 'ifconfig', $_interface_id );
	    my $command = quote_argv_sh(@param);
	    my $output = $apiset->execute_raw_command( command => $command );
	    my @arr_lines = split(/\n/, $output);
	    my $ip_address;
	    while (my $line = shift(@arr_lines)) {
		$line =~ s/^\s+//;
		$line =~ s/\s+$//;
		# Check to determine if ipv4 or ipv6 is present
		if ($line =~ /^inet\s+(\d+.\d+.\d+.\d+)/ || $line =~ /^inet6\s+(.+%\w+).+/ ){
		    $ip_address = $1;
		}
	    }
	    # Implementing a check to ignore the ports not having ipv4 and ipv6 address
	    next if (!$ip_address);
        }
        if ($capture_name) {
            $interface_to_file->{$_interface_id} = join( '_',
                $capture_name, $_interface_id, time() . '.pcap' );
        } else {
            $interface_to_file->{$_interface_id} = join( '_',
                $command_interface->hostrec()->id(),
                    $_interface_id, time(), '.pcap' );
        }
        if ($interface_to_file->{$_interface_id} !~ /(smoke|cit)-[a-z]/) {
            $interface_to_file->{$_interface_id} =~ s/-/_/g;
        }
        $interface_to_file->{$_interface_id} =
            $output_dir . '/' . $interface_to_file->{$_interface_id}
                if (($output_dir) and (-e $output_dir));
    } ## end foreach my $_interface_id (...)
    $Log->trace(
        sub { "Names of capture files " . Dumper($interface_to_file) } );
    while ( my ( $capture_interface, $capture_file )
        = each(%$interface_to_file) )
    {
        my @capture_param;
	if ( $set =~ /filer/i && $capture_interface =~ /^e.+\w+$/ ) {
	    my $vsid = $self->_get_vsid($params,$capture_interface);
	    my $vcontext = "vcontext";
	    my $tcpdump_path = "/usr/sbin/tcpdump";
	    @capture_param = ( $sudo_path, $vcontext);
	    push @capture_param, ( '-v', $vsid, $tcpdump_path, $capture_tool );
	} else {
            @capture_param = ( $sudo_path, $capture_tool );
	}
        push @capture_param, ( '-i', $capture_interface );
        push @capture_param, ( '-s', $capture_length )
            if ( defined($capture_length) );
	if ( $params->{target_machine}->hostrec()->hosttype() =~ /filer/ && $params->{target_machine}->is_fusen() ) {
	    if (ref $capture_filter) {
                my $directory_path = $capture_filter->{'directory-path'};
                if ( defined ($directory_path) && $directory_path !~ /\/mroot/ ){
                    push @capture_param, ( '-w', '/mroot'.$directory_path.'/'.$capture_file );
		} elsif ( defined ($directory_path) ) {
		    push @capture_param, ( '-w', $directory_path.'/'.$capture_file );
		}
	    } else {
		push @capture_param, ( '-w', '/mroot/'.$capture_file );
	    }
	} else {
	    push @capture_param, ( '-w', $capture_file );
	}

        # apply ring buffer options to the packet trace
        if ($capture_ring_buffer) {
            # file size (must convert to megabytes)
            my $capture_ring_buffer_file_size_mbytes = $capture_ring_buffer_file_size_kbytes / 1024;
            push @capture_param, ( '-C', $capture_ring_buffer_file_size_mbytes );

            # number of files in the ring buffer
            push @capture_param, ( '-W', $capture_ring_buffer_file_count );

            # need to do this to get ring buffers to work properly or else we
            # get permissioned denied errors.
            push @capture_param, ( '-Z', 'root' );

            # if using ring buffers, the filenames will have numbers at the end, therefore
            # we need to slap on a wildcard at the end of the filename
            # because the format is <client_name>_<interface>_<timestamp>_.pcap<file_count>
            $interface_to_file->{$capture_interface} =~ s/\.pcap/\.pcap\*/;
        }
        $cmd = quote_argv_sh(@capture_param);
        if ( defined($capture_filter) && !(ref $capture_filter) ) {
	    $cmd = $cmd . ' ' . $capture_filter;
        }
        $cmd = $cmd . " &";

        $Log->trace( "Starting packet capture on "
                . $command_interface->name()
                . " machine command($cmd)" );

        # Storing the pid for stopping the capture
        my $output = $apiset->execute_raw_command( command => $cmd );
        foreach my $line (split('\n', $output)) {
            $interface_to_pid->{$capture_interface} = $1
                if ( $line =~ /\[\d+\]\s+(\d+)/ );
        }
        try {
            $apiset->execute_raw_command(
                command => quote_argv_sh(
                    'ps', '-p', $interface_to_pid->{$capture_interface}
                ),
                'connectrec-check_status' => 1
            );
        } ## end try
        otherwise {
            my $e = shift;
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Unable to start $capture_tool on $capture_interface. "
                    . $e->text() );
        } ## end otherwise
    } ## end while ( my ( $capture_interface...))

    $Log->debug(
        sub {
            "pid's of running capture commands " . Dumper($interface_to_pid);
        }
    );
   sleep($nacl_task_delay_after_stop);
    my $task_params = $self->packet_capture_params();
    $task_params->{$target_count}->{capture_file}      = $interface_to_file;
    $task_params->{$target_count}->{pid}               = $interface_to_pid;
    $task_params->{$target_count}->{using_ring_buffer} = $capture_ring_buffer;

    $self->packet_capture_params($task_params);
    $Log->exit() if $may_exit;
} ## end sub _start_on_linux_or_filer_tcpdump

# Start the packet capture process on Windows

sub _start_on_windows {
    $Log->enter() if $may_enter;

    my $self         = shift;
    my $target_count = shift;
    my $params       = shift;
    my @interfaces   = ();

    my $command_interface = $params->{target_machine};
    my $capture_interface = $params->{interface_filter};
    my $capture_filter    = $params->{capture_filter};
    my $capture_tool      = $params->{capture_tool};
    my $capture_length    = $params->{capture_length};
    my $capture_name      = $params->{capture_name};
    my $output_dir        = $params->{output_dir};
    my $capture_ring_buffer = $params->{capture_ring_buffer};
    my $capture_ring_buffer_file_size_kbytes =
        $params->{capture_ring_buffer_file_size_kbytes};
    my $capture_ring_buffer_file_count =
        $params->{capture_ring_buffer_file_count};
    my $precleanup          = $params->{precleanup_all_traces};
    my $apiset;
    my $interface_present;

    $capture_interface = _validate_interface($capture_interface);
    $apiset            = $command_interface->apiset(
        category  => "Host",
        interface => "CLI",
        set       => "Windows"
    );

    # if output_dir is specified use the UNC path as the pcap file prefix
    my $pcap_prefix;
    if ( defined( $output_dir ) ) {
        $pcap_prefix = $output_dir;
        $pcap_prefix .= '\\' if ( $pcap_prefix !~ /\\$/ );
    } else {
        $pcap_prefix = '%SystemDrive%\\';
    }

    my $param_global = param_global();
    my $win_pkt_cap_param = $param_global->get('WIN_PACKET_CAPTURE_ADDITIONAL_LOGGING') || 0;
    if ($win_pkt_cap_param){
        #Any error while displaying CPU and Memory details needs to be ignored
        eval{
            # Windows CPU and Memory load status
            $apiset->execute_raw_command(command => 'systeminfo |find "Available Physical Memory"' );
            $apiset->execute_raw_command(command => 'systeminfo | findstr "System Memory"' );
            $apiset->execute_raw_command(command => 'TASKLIST /M' );
        };
    }

    # if requested to precleanup all traces, do it here
    if ($precleanup) {
        my $cli_name = $command_interface->client();
        $cli_name =~ s/\-/_/g;
        $apiset->execute_raw_command(command => 'del /f ' . $pcap_prefix . $cli_name  .'*.pcap');
    }

    my $capture_tool_exe = 'tshark.exe';

    # getting the list of network interfaces
    if ( $capture_interface->[0] eq 'all' ) {
        $interface_present = $apiset->execute_raw_command(
            command => quote_argv_cmd( $capture_tool, "-l", "-D" ) );
        while ( $interface_present =~ /^(\d+)\.\s+/mg ) {
            push @interfaces, $1;
        }
        $capture_interface = \@interfaces;
    } ## end if ( $capture_interface...)

    my $interface_to_file = {};
    my $interface_to_pid  = {};

    # Creating the file name to storing packets
    foreach my $_interface_id (@$capture_interface) {
        if ($capture_name) {
            $interface_to_file->{$_interface_id} = join( '_',
                $capture_name, $_interface_id, time() . '.pcap' );
        } else {
        $interface_to_file->{$_interface_id} = join( '_',
            $pcap_prefix . $command_interface->client(),
            $_interface_id, time() . '.pcap' );
        }
    } ## end foreach my $_interface_id (...)
    $Log->debug(
        sub {
            "File names to store packets based on interface"
                . Dumper($interface_to_file);
        }
    );

    #$capture_tool =~ s/"//g;
    my $i    = 1;
    while ( my ( $capture_interface, $capture_file )
        = each(%$interface_to_file) )
    {
        my @args = ( '-i', $capture_interface );
        push @args, ( '-s', $capture_length ) if ( defined($capture_length) );
        push @args, ( '-f', '"' . $capture_filter . '"' )
            if ( defined($capture_filter) );
        push @args, ( '-w', $capture_file );

        # apply ring buffer options to the packet trace
        if ($capture_ring_buffer) {
            push(@args, '-b filesize:' . $capture_ring_buffer_file_size_kbytes);
            push(@args, '-b files:'    . $capture_ring_buffer_file_count);
            # when we use ring buffers we need to put a wildcard before the pcap file
            # extension, this will ensure that the show() and cleanup() functions
            # properly put
            $interface_to_file->{$capture_interface} =~ s/\.pcap/*\.pcap/;
        }

        $Log->comment("Starting the capture with $capture_tool @args");
        my $host_id = $command_interface->hostrec()->id();
        $interface_to_pid->{$capture_interface}->{_process} = NATE::Process->new(
            codespec => $capture_tool,
            args     => \@args,
            client   => $host_id,
            runid    => $host_id . '_' . $capture_interface . '_pkt_cap',
            kill_timeout => 5,
            background   => 1,
        );
        $interface_to_pid->{$capture_interface}->{_process}->start();
        Tharn::snooze(5);
        if (!$interface_to_pid->{$capture_interface}->{_process}->is_running()
            )
        {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Unable to start $capture_tool\($capture_tool @args\) on "
                    . Dumper(
                    $interface_to_pid->{$capture_interface}->{_process}
                    )
            );
        } ## end if ( !$interface_to_pid...)

    } ## end while ( my ( $capture_interface...))

    $Log->debug(
        sub {
            "pid's of running packet capture process "
                . Dumper($interface_to_pid);
        }
    );
    my $task_params = $self->packet_capture_params();
    $task_params->{$target_count}->{capture_file}      = $interface_to_file;
    $task_params->{$target_count}->{pid}               = $interface_to_pid;
    $task_params->{$target_count}->{using_ring_buffer} = $capture_ring_buffer;

    $self->packet_capture_params($task_params);
    $Log->exit() if $may_exit;
} ## end sub _start_on_windows

=head2 stop

    $packetcapture->stop();

    (Instance method)
    Stop the packet capture process on all the clients/filers and dump the packets on tmporary files.

    This does not have any return value.

=cut

sub stop {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    my $task_params = $pkg->packet_capture_params();

    while ( my ( $target, $params ) = each(%$task_params) ) {
        if ( $params->{target_machine}->hostrec()->is_unix()
             || $params->{target_machine}->hostrec()->is_windows()
	     || ( $params->{target_machine}->hostrec()->hosttype() =~ /filer/ && $params->{target_machine}->is_fusen() ) )
        {
            $pkg->_stop_on_client($params);
        } else {
            $pkg->_stop_on_filer( $target, $params );
        }
    } ## end while ( my ( $target, $params...))
    $Log->exit() if $may_exit;
} ## end sub stop
# Stop the packet capture process on Filer.

sub _stop_on_filer {
    $Log->enter() if $may_enter;
    my $self         = shift;
    my $target_count = shift;
    my $params       = shift;

    my $nacl_task_delay_after_stop = $params->{nacl_task_delay_after_stop};
    my $command_interface = $params->{target_machine};
    my $capture_interface = $params->{interface_filter};
    my $capture_filter    = $params->{capture_filter};
    my $apiset_obj        = $command_interface->get_7m_or_nodescope_apiset();
    my @files_before_pktt_start = $self->pktt_files_before_start();
    $Log->debug(
        sub {
            "List of Files found on filer before running pktt dump command"
                . Dumper(\@files_before_pktt_start);
        }
    );

    my $dir = $capture_filter->{'directory-path'};
    foreach my $_interface (@$capture_interface) {
        if (!defined $dir) {
            # No need to dump if pktt is started with dir option specified
            $apiset_obj->pktt_dump( interface => $_interface );
        }
        $apiset_obj->pktt_stop( interface => $_interface );
        ## delayed,as Packet capture file takes some time to be created.
        sleep($nacl_task_delay_after_stop);
    }

    my @files_after_pktt_stop = $self->_get_pktt_files(dir => $dir, command_interface => $command_interface);

    $Log->debug(
        sub {
            "List of Files found on filer after running pktt dump command"
                . Dumper(\@files_after_pktt_stop);
        }
    );

    $self->pktt_files_after_stop(@files_after_pktt_stop);

    my @difference = array_diff(@files_before_pktt_start,@files_after_pktt_stop);

    $self->generated_pktt_files(@difference);

    my $file_hash = {};
    foreach my $file_name (@difference) {
        my @arr = split(/\//, $file_name);
        my $intf_name = $1 if ( $arr[-1] =~ /^([a-zA-Z0-9\-]+)_/ );
        $file_hash->{$intf_name} = $file_name;
    }

    my $task_params = $self->packet_capture_params();
    $task_params->{$target_count}->{capture_file} = $file_hash;
    $self->packet_capture_params($task_params);
    $Log->exit() if $may_exit;
} ## end sub _stop_on_filer

# Stop the packet capture process on linux/windows client.


sub _stop_on_client {
    $Log->enter() if $may_enter;

    my $self   = shift;
    my $params = shift;

    my $nacl_task_delay_after_stop = $params->{nacl_task_delay_after_stop};
    my $command_interface = $params->{target_machine};
    my $os_type           = $command_interface->hostrec()->hosttype();
    my $apiset            = $params->{custom_apiset};
    my $command;
    my %opts;
    my $hosttype = $params->{target_machine}->hostrec()->hosttype();
    my $set = _set_type(host_type => $hosttype);
    if($set =~ /Filer/) {
        $apiset = $command_interface->get_systemshell_apiset();
    }else {
        $apiset  = $command_interface->apiset(
             category  => "Host",
             interface => "CLI",
             set       => $set
             ) unless ($apiset);
    }
    if ( $set =~ /Linux|Solaris|Filer/ ) {
	$command = 'kill';
        %opts    = ( signal => '2' ) if ($set ne 'Filer');
        $apiset->set_privilege(
            "privilege-level" => "root",
            'answer-password' => 1
        );
    }elsif ( $set =~ /Windows/ ) {
	$command = 'taskkill';
        %opts    = ( force => 1 );
    }

    # Killing the process
    my ($retry_count, $caught_exp, $process_killed_on_retry);
    while ( my ( $capture_interface, $pid ) = each( %{ $params->{pid} } ) ) {
        if ( !( ref($pid) eq 'HASH' && exists( $pid->{_process} ) ) ) {
            $Log->debug(
                "Killing the pid $pid, interface $capture_interface on "
                    . $command_interface->name() );
	    $retry_count = 3;
	    $caught_exp = 0;
	    $process_killed_on_retry = 0;
	  DELETE_AGAIN: {
	    try {
		$apiset->$command( %opts, pid => $pid ) if ($retry_count > 0);
	    } catch NATE::BaseException with {
		my $ex = shift;
		if ( $ex->text() =~ /write error: Interrupted system call/i ) {
		    $caught_exp = 1;
		    $retry_count--;
		    redo DELETE_AGAIN;
		} elsif ( $ex->text() =~ /Command failed - Check command line and dependencies/i && $ex->{'-response_object'}->{'processed_output'} =~ /No such process/i && ($caught_exp == 1)) {
		    $process_killed_on_retry = 1;
		    # suppress the error as process id has already been killed on
		    # retry attempt
		}
		$ex->throw() if ($process_killed_on_retry != 1);
	    };
	  }
        } else {
            $Log->debug( "Stopping the NATE::Process" . Dumper($pid) );
	    # as per burt#1071688, wait for the completion of stop
            $pid->{_process}->stop()->wait();
            $pid->{_process}->destroy();
        }
        ## delayed,as Packet capture file takes some time to be created.
    } ## end while ( my ( $capture_interface...))

    # XXX: if on linux, resync the connection since the traces are executed in the background
    # and may corrupt the output of subsequent method calls
    if ( $command_interface->hostrec->is_unix() ) {
        $apiset->get_connection()->resync(timeout => 2);
    }

    $Log->exit() if $may_exit;
} ## end sub _stop_on_client

=head2 show

    my $packests=$packetcapture->show();

    (Instance method)
    Return the captured packet on all the clients/filers based on display filter.
    The format of captured packets are same as 'tcpdump/tshark/tethereal' tools.
    Please refer tools help for more details.

    This function return the data structure, which have all the captured packets as array.
    This data structure(HHOA) is having first keys as 'name of target machine', second key as
    'network interface id' and its value is captured packets as a array.

    This data structure look likes:
    $packets->{<Name of Client/Filer>}->{<Network Interface ID of that Client/Filer>} = [<all captured packets>]
    $packets= {
          'Linux-client'=> {
                   'eth0' => [
                              '07:07:36.649078 IP y1.rtp.netapp.com.32789 > ncat-lc-1.gdl.englab.netapp.com.1007: . ack 3479957102 win 49232 <>',
                              '07:07:36.649080 IP ncat-lc-1.gdl.englab.netapp.com.37787 > y1.rtp.netapp.com.sunrpc: S 3471 064718:3471064718(0) win 5840 <>'
                                      ],
                    'eth1'=> [], #packets as string
                             },
            'Filer-1'   =>  {
                    'e0b' => [...],
                    'e0c' => [...]
                    }
         };

=cut

sub show {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    my $task_params    = $pkg->packet_capture_params();
    my $filter_packets = {};

    while ( my ( $target, $params ) = each(%$task_params) ) {
        my ( $name, $pktcapture_tool ) = ();
        my $sudo            = 0;
        my $sudo_path;
        my $capture_file    = $params->{capture_file};
        my $capture_filter = $params->{capture_filter};
        my $display_filter  = $params->{display_filter};
        my $output_dir      = $params->{output_dir};
        my $apiset          = $params->{custom_apiset};
	my $set;
        if ( $params->{target_machine}->hostrec()->is_windows()
             || $params->{target_machine}->hostrec->is_unix() )
        {
            $name = $params->{target_machine}->client();
	    $set = _set_type(host_type => $params->{target_machine}->hostrec()->hosttype());
            $pktcapture_tool = quote_argv_sh($params->{capture_tool});
            $apiset = $params->{target_machine}->apiset(
                category  => "Host",
                interface => "CLI",
                set       => $set
            ) unless ($apiset);
	    if ( $set =~ /Linux|Solaris/ ) {
                $sudo = 1;
                $sudo_path = Tharn::param("SUDO_PATH");
	    } elsif ( $set =~ /Windows/ ) {
		$display_filter = "-R $display_filter"
                    if ( defined($display_filter) );
	    }

        } else {
            $pktcapture_tool = 'tcpdump';
            $name            = $params->{target_machine}->node();
            $apiset          = $params->{target_machine}->apiset(
                category  => 'Node',
                interface => 'CLI',
                set       => "Systemshell"
            );

        } ## end else [ if ( $params->{target_machine...})]
        while ( my ( $intf_id, $fname ) = each(%$capture_file) ) {

            # if this packet capture was using a ring buffer, the capture file
            # name will have a wildcard * in it, which means we need to expand out all the file names
            # that the wildcard matches and store them in the $file_names array ref
            my $file_names;
            if ($params->{using_ring_buffer}) {
                $file_names = $pkg->_expand_ring_buffers(apiset     => $apiset,
                                                         hostrec    => $params->{target_machine}->hostrec(),
                                                         file_name  => $fname,
							 output_dir => $output_dir);
            } else {
                # no ring buffer is being used
                push(@{$file_names}, $fname);
            }

            my $cmd;
            my @all_packets = ();
            foreach my $file_name (@{$file_names}) {
		if ( $params->{target_machine}->hostrec()->hosttype() =~ /filer/ && $params->{target_machine}->is_fusen() ) {
                    my $directory_path;
                    $directory_path = $capture_filter->{'directory-path'} if (ref $capture_filter);
                    if ( defined ($directory_path) && $directory_path !~ /\/mroot/ ) {
                        $file_name = "/mroot" . $directory_path . '/' . $file_name ;
                    } elsif ( defined ($directory_path) ) {
			$file_name = $directory_path . '/' . $file_name ;
		    } else {
                        $file_name = "/mroot/" . $file_name ;
                    }
                }
                if ( defined($display_filter) ) {
                    $cmd = "$pktcapture_tool $display_filter -r $file_name";
                } else {
                    $cmd = "$pktcapture_tool -r $file_name";
                }
                $cmd = "$sudo_path $cmd" if ($sudo);
                my $capture_op = $apiset->execute_raw_command( command => $cmd );

                _check_error($capture_op);
                my @packets = split( /\r*\n/, $capture_op );
                shift @packets;
                push(@all_packets, @packets);
            }
            $filter_packets->{$name}->{$intf_id} = \@all_packets;
        } ## end while ( my ( $intf_id, $fname...))
    } ## end while ( my ( $target, $params...))
    $Log->exit() if $may_exit;
    return $filter_packets;
} ## end sub show

=head2 get

    $packetcapture->get();
    or
    $packetcapture->get(path=>"$LOGDIR/client1/");
(Instance method)

Copy all the capture files to ntest $LOGDIR(default) or specified path.

This function return the data structure, which have all the copied files with full path.
This data structure(HHOA) is having first keys as 'name of target machine', second key as
'network interface id' and its value is filename of captured files.

This data structure look likes:
$packets->{<Name of Client/Filer>}->{<Network Interface ID of that Client/Filer>} = $LOGDIR/<filename>
 $packets= {
      'Linux-client'=> {
               'eth0' => "/u/sushantg/natelogs/20120116_221421/ncat_lc_1_eth0_1326732325_.pcap
               'eth1'=> "/u/sushantg/natelogs/20120116_221421/ncat_lc_1_eth1_1326732325_.pcap",
                         },
        'Filer-1'   =>  {
                'e0b' => "/u/sushantg/natelogs/20120116_221421/e0b_20120116_164420.trc",
                'e0c' => "/u/sushantg/natelogs/20120116_221421/e0c_20120116_164420.trc",
                }
     };

=over

=item Options

=over

=item C<< path => '/home/' >>

(Optional) default to $LOGDIR.
Copied all pcap file on specified location.

=back

=back

=cut

sub get {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => { path     => { type => SCALAR, optional => 1 }, }
    );
    $Log->debug( "PacketCapture->get() Arguments :" . Dumper( \%opts ) );

    my $task_params = $pkg->packet_capture_params();
    my %scp_arguments;
    $scp_arguments{set_directory_on_target} = delete( $opts{path} )
        if ( defined $opts{path} );
    my $cmd;
    my $name;
    my $set;
    my $file_copy = {};
    while ( my ( $target, $params ) = each(%$task_params) ) {
        my $capture_file = $params->{capture_file};
	my $capture_filter = $params->{capture_filter};
        my $cmd_int      = $params->{target_machine};
        my $apiset       = $params->{custom_apiset};

        # determine the apiset to use or the name of the source host
        if ( $params->{target_machine}->hostrec()->is_unix() || $params->{target_machine}->hostrec()->is_windows() )
        {
	    $set = _set_type(host_type => $params->{target_machine}->hostrec()->hosttype());
            $apiset = $params->{target_machine}->apiset(
                category  => "Host",
                interface => "CLI",
                set       => $set
            ) unless ($apiset);
            $name = $params->{target_machine}->client();
        } else {
            $name = $params->{target_machine}->node();
        }

        while ( my ( $intf, $filename ) = each(%$capture_file) ) {
            # copy over each file in the array
            $Log->debug("Copying $filename of $intf interface");
	    my ( $target_val,@localhost_loc );
            if ( $params->{target_machine}->hostrec()->hosttype() =~ /windows/ ) {
                my $apiset = $params->{target_machine}->apiset(
                    category  => "Host",
                    interface => "CLI",
                    set       => "Windows",
                );
		my $source;
		my $destination = Tharn::param("LOGDIR_WINDOWS");
		@localhost_loc = $apiset->copy(
		    'source' => $filename,
		    'destination' => $destination,
		);
		$target_val = ( split ' ', $localhost_loc[0]{'actual_command'} )[ -1 ];
            } else {
		if ( $params->{target_machine}->hostrec()->hosttype() =~ /filer/ && $params->{target_machine}->is_fusen() ) {
		    my $directory_path = $capture_filter->{'directory-path'};
		    if ( defined ($directory_path) && $directory_path !~ /\/mroot/ ) {
		        $filename = "/mroot" . $directory_path . '/' . $filename ;
		    } elsif ( defined ($directory_path) ) {
                        $filename = $directory_path . '/' . $filename ;
                    } else {
			$filename = "/mroot/" . $filename ;
		    }
		}
                @localhost_loc = NACL::MTask::SCP->copy_to_localhost(
                    source_command_interface => $params->{target_machine},
                    files_to_copy            => [$filename],
                    %scp_arguments
                );
		$target_val = $localhost_loc[0];
            }

            $pkg->packet_capture_params()->{$target}->{ntest_copy}->{$intf}
                = $target_val;
            $file_copy->{$name}->{$intf} = $target_val;
        }

        # resync the connection on unix machines
        # since the SCP library might leave the connection in a bad state.
        if ( $params->{target_machine}->hostrec()->is_unix()) {
            $apiset->get_connection()->resync();
        }

    } ## end while ( my ( $target, $params...))
    $Log->debug( "copied files" . Dumper($file_copy) );
    $Log->exit() if $may_exit;
    return $file_copy;
} ## end sub get

=head2 capture_file_name

    $capture_filename = $packetcapture->capture_file_name();

(Instance method)

This method is used to get the packet capture file names through the
PacketCapture MTask object.
Uses a CMode CLI, or 7Mode CLI APISet.
Returns the list of capture file names.

=cut

sub capture_file_name {
    $Log->enter() if $may_enter;
    my $pkg = $_[0];
    my @capture_files;

    my $task_params = $pkg->packet_capture_params();
    while ( my ( $target, $params ) = each(%$task_params) ) {
        my $capture_file = $params->{capture_file};
        while ( my ( $intf, $filename ) = each(%$capture_file) ) {
            push (@capture_files, $filename);
        }
    }
    $Log->exit() if $may_exit;
    return @capture_files;
} ## end sub capture_file_name

=head2 cleanup

    $packetcapture->cleanup();
    (Instance method)

    Removes all the capture files on all Filers/Clients.
    This does not have any return value.

=cut

sub cleanup {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    my $task_params = $pkg->packet_capture_params();
    my $cmd;
    my $set;
    while ( my ( $target, $params ) = each(%$task_params) ) {
        my ( $name, $pktcapture_tool ) = ();
        my $capture_file = $params->{capture_file};
        my $capture_filter = $params->{capture_filter};
        my $apiset       = $params->{custom_apiset};
        if ( $params->{target_machine}->hostrec()->is_unix() || $params->{target_machine}->hostrec()->is_windows()) {
	    $set = _set_type(host_type => $params->{target_machine}->hostrec()->hosttype());
            $apiset = $params->{target_machine}->apiset(
                category  => "Host",
                interface => "CLI",
                set       => $set
            ) unless ($apiset);
	    if ( $set =~ /Linux|Solaris/ ) {
                my $sudo_path = Tharn::param("SUDO_PATH");
                $cmd = "$sudo_path rm -f";
	    } elsif ( $set =~ /Windows/ ) {
		$cmd = 'del /f';
	    }
        } else {
            $apiset = $params->{target_machine}->apiset(
                category  => 'Node',
                interface => 'CLI',
                set       => "Systemshell"
            );
            $cmd = 'rm -f';
        } ## end else [ if ( $params->{target_machine...})]

        while ( my ( $intf_id, $fname ) = each(%$capture_file) ) {
	    if ( $params->{target_machine}->hostrec()->hosttype() =~ /filer/ && $params->{target_machine}->is_fusen() ) {
                my $directory_path = $capture_filter->{'directory-path'};
                if ( defined ($directory_path) && $directory_path !~ /\/mroot/ ) {
                    $fname = "/mroot" . $directory_path . '/' . $fname ;
                } elsif ( defined ($directory_path) ) {
                    $fname = $directory_path . '/' . $fname ;
                } else {
                    $fname = "/mroot/" . $fname ;
                }
            }
            my $capture_op
                = $apiset->execute_raw_command( command => "$cmd $fname" );
        }
    } ## end while ( my ( $target, $params...))
    $Log->exit() if $may_exit;

} ## end sub cleanup

sub _validate_interface {
    $Log->enter() if $may_enter;
    my $interfaces = shift;
    $Log->exit() if $may_exit;
    return scalar( grep { lc($_) eq 'all' } @$interfaces ) >= 1
        ? ['all']
        : $interfaces;
} ## end sub _validate_interface

sub _get_difference_of_two_array {
    $Log->enter() if $may_enter;

    my $_1st_arr   = shift;
    my $_2nd_arr   = shift;
    my %count      = ();
    my @difference = ();

    foreach my $element ( @{$_1st_arr}, @{$_2nd_arr} ) {
        $count{$element}++;
    }
    foreach my $element ( keys %count ) {
        push @difference, $element if ( $count{$element} == 1 );
    }
    $Log->exit() if $may_exit;
    return \@difference;
} ## end sub _get_difference_of_two_array

sub _check_error {
    my $string = shift;
    $Log->enter() if $may_enter;
    if ( $string =~ /^(ERROR:\s+)|(Usage:\s+)/ ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw("Command Failed $string");
    }
    $Log->exit() if $may_exit;
} ## end sub _check_error

sub _expand_ring_buffers {
    my ($pkg, %args) = @_;
    $Log->enter() if $may_enter;

    my $hostrec = delete($args{hostrec}) //
        NATE::Exceptions::Argument->throw("Missing required parameter 'hostrec'");

    my $file_names;

    # expand the ring buffer file names
    if ($hostrec->hosttype() =~ /windows/i) {
        $file_names = $pkg->_expand_ring_buffers_windows(%args);
    } elsif ($hostrec->is_unix()) {
        $file_names = $pkg->_expand_ring_buffers_linux(%args);
    } else {
        NATE::Exceptions::Argument->throw("Hosttype " . $hostrec->hosttype() .
                                          " . is not supported");
    }

    $Log->exit() if $may_exit;
    return $file_names;
}

sub _expand_ring_buffers_windows {
    my ($pkg, %args) = @_;
    $Log->enter() if $may_enter;

    my $file_name = delete($args{file_name}) //
        NATE::Exceptions::Argument->throw("Missing required parameter 'file_name'");
    my $apiset = delete($args{apiset}) //
        NATE::Exceptions::Argument->throw("Missing required parameter 'apiset'");

    my $output_dir = $args{output_dir};
    my $pcap_prefix;
    if(defined($output_dir)) {
        $pcap_prefix = $output_dir;
	$pcap_prefix .= '\\' if ( $pcap_prefix !~ /\\$/ );
    } else {
	$pcap_prefix = '%SystemDrive%\\';
    }

    my $cmd = "dir $file_name";
    my $file_list = $apiset->execute_raw_command(command => $cmd);

    my @output = split('\n', $file_list);
    @output = grep { /\.pcap/ } @output;
    @output = map { s/^.+\s(\S+\.pcap)\s*$/$1/; $pcap_prefix . $_; } @output;

    $Log->exit() if $may_exit;
    return \@output;
}

sub _expand_ring_buffers_linux {
    my ($pkg, %args) = @_;
    $Log->enter() if $may_enter;

    my $file_name = delete($args{file_name}) //
        NATE::Exceptions::Argument->throw("Missing required parameter 'file_name'");
    my $apiset = delete($args{apiset}) //
        NATE::Exceptions::Argument->throw("Missing required parameter 'apiset'");

    my $cmd = "ls $file_name | cat";
    my $file_list = $apiset->execute_raw_command(command => $cmd,
                                                 'connectrec-check_status' => 1);

    my @output = split('\n', $file_list);
    @output = grep { /\.pcap/ } @output;

    $Log->exit() if $may_exit;
    return \@output;
}

=head2 purge

   $packetcapture->purge();
    (Instance method)

    Directly call the cleanup method.

=cut

sub purge {
    $Log->enter() if $may_enter;
    my ($packetcapture, @args) = @_;
    $packetcapture->cleanup(@args);
    $Log->exit() if $may_exit;
} ## end sub purge

#Commenting unused function verify_cleanup()

=begin comment
=head verify_cleanup

sub verify_cleanup {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    my $task_params = $self->packet_capture_params();

    while ( my ( $target, $params ) = each(%$task_params) ) {
	if ( $params->{target_machine}->hostrec()->is_unix() || $params->{target_machine}->hostrec()->is_windows() || ( $params->{target_machine}->hostrec()->hosttype() =~ /filer/ && $params->{target_machine}->is_fusen() ) )
        {
            $self->_verify_cleanup_client($params);
        } else {
            $self->_verify_cleanup_filer($params);
        }
    } ## end while ( my ( $target, $params...))
    $Log->exit() if $may_exit;
} ## end sub stop

sub _verify_cleanup_filer {
    my ($self, $params) = @_;

    my $command_interface = $params->{target_machine};
    my $dir = "/mroot" . $params->{capture_filter}->{'directory-path'};

    my @generated_files = $self->generated_pktt_files();

    my @files_after_cleanup = $self->_get_pktt_files(
        dir               => $dir,
        command_interface => $command_interface
    );

    my @not_cleanedup_files = intersect(@generated_files, @files_after_cleanup);
    if (scalar(@not_cleanedup_files) > 0) {
        my $files = join(", ", @not_cleanedup_files);
        NACL::Exceptions::CleanupFailed->throw(
            "Failed to clean following pktt files " . $files
        );
    }
}
sub _verify_cleanup_client {
    # Currently NOOP
}

=end comment

=cut

sub _set_type {
    my ($self, $host_type) = @_;
    $Log->enter() if $may_enter;
    my $set;

    if ( $host_type =~ /solaris/ ) {
        $set = 'Solaris';
    } elsif ( $host_type =~ /unix|lanforge/ ) {
        $set = 'Linux';
    } elsif ( $host_type =~ /windows/ ) {
	$set = 'Windows';
    } elsif ($host_type =~/filer-ng-sdot|filer-ng-vsim|filer-ng-cfe|filer-ng-vsa/){
        $set = 'Filer';
    }

    $Log->exit() if $may_exit;
    return $set;
}

sub _get_vsid {
    my ($self, $params, $interface) = @_;
    $Log->enter() if $may_enter;
    my $ipspace_name;
    my $command_interface   = $params->{target_machine};
    my $apiset_obj = $command_interface->apiset();
    my $ipspace = NACL::CS::NetworkPort->fetch(
	command_interface => $command_interface,
	filter => {
	    port => $interface,
	    node => $command_interface->node()
	},
	requested_fields => [ qw(ipspace) ],
	allow_empty => 1,
    );
    my $vsid;
    $ipspace_name = defined $ipspace ? $ipspace->ipspace() : 'default';
    if ( $ipspace_name =~ /default/i ) {
	$vsid = "4294967295";
    } else {
	my $vsid_command = $apiset_obj->debug_smdb_table_show(
	    table => "vserver_id_to_name",
	    name  => $ipspace_name
	);
	my $output = $vsid_command->get_parsed_output();
	$vsid = @$output[0]->{id};
    }
    $Log->exit() if $may_exit;
    return $vsid;
}

1;
