# $Id$

#
#   Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary PacketParser Task Module
## @author rahula@netapp.com , dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::MTask::PacketParser

=head1 DESCRIPTION

C<NACL::MTask::PacketParser> provide methods to get_packet_by_parameter , next_packet , get_packet_by_index & rewind
packet parser functionality on the captured file generated by tcpdump, tethereal , tshark.exe etc. utilities .

This Task provides decoding of the following protocols :
- ethernet layer frame,ipv4,ipv6,tcp,udp,icmpv6,icmpv4,igmp,arp,rpc,rip

=head1 ATTRIBUTES

=head2 capture_file

The Packet captured file .

=head2 pkt_number

The packet number of the packet present in the captured file .

=head2 pkt_map

The packet map which is required for rewinding & reading of the packet capture file .

=head2 offset

The offset value while reading & rewinding the packet file pointer .

=head2 ident

The identifier value of the packet captured file required for unpacking the packets .

=head2 fh

The file-handle required for reading the packet catured file .

=head2 format

The format which indentifies the endianess of the packets .

=head2 index

The index of the packet in the captured file .

=cut

package NACL::MTask::PacketParser;

use strict;
use warnings;

use base qw(NACL::MTask::MTask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate
  qw(validate validate_with SCALAR ARRAYREF UNDEF HASHREF OBJECT BOOLEAN);
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NATE::BaseException qw(:try);
use NACL::ComponentUtils qw(Dumper);

=head1 METHODS

=head2 new

    my $packetparser = NACL::MTask::PacketParser->new( capture_file => $file );

    Create a PacketParser object that can be used by get_packet_by_parameter , copy , next_packet , get_packet_by_index & rewind methods .

=over

=item Options

=over

=item C<< capture_file=>$file >>

(Required) Contains the captured filename which is generated by tcpdump,tshark.exe,pktt or tethereal utility and
this file should be present on ntest client.

=back

=back

=cut

use Class::MethodMaker [
    new    => [ '-hash', '-init', 'new' ],
    scalar => 'capture_file',
    scalar => 'pkt_number',
    array  => 'pkt_map',
    scalar => 'offset',
    scalar => 'ident',
    scalar => 'fh',
    hash   => 'format',
    scalar => 'index',
];

my %format = (
    "\324\303\262\241" => {
        bigendian  => 0,
        header_fmt => 'vvVVVV',
        header_rec => 'VVVV',
    },
    "\241\262\303\324" => {
        bigendian  => 1,
        header_fmt => 'nnNNNN',
        header_rec => 'NNNN',
    }
);

sub init {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = $self->_common_validate_with(
        params               => \@_,
        additional_spec      => { capture_file => { type => SCALAR }, },
        no_command_interface => 1,
    );

    # Initialize offset & packet number
    $self->pkt_number(0);
    $self->offset(0);

    $Log->exit() if $may_exit;
} ## end sub init

=head2  get_packet_by_parameter

(Instance Call)
        my  %input_hash  = {
                                'Ethernet:src'=> '00:50:56:94:0a:10',
                                'Ethernet:ARP:protocol_type' => 2048,
                                'Ethernet:ARP:sender_proto_addr' => '169.254.216.153',
                                'Ethernet:ARP:target_proto_addr' => '169.254.3.115',
                            };

        $packetparser->get_packet_by_parameter("match-parameters" => \%input_hash);

        or

(Class Call)
        my  %input_hash  = {
                                'Ethernet:src'=> '00:50:56:94:0a:10',
                                'Ethernet:ARP:protocol_type' => 2048,
                                'Ethernet:ARP:sender_proto_addr' => '169.254.216.153',
                                'Ethernet:ARP:target_proto_addr' => '169.254.3.115',
                            };

        NACL::MTask::PacketParser->get_packet_by_parameter("match-parameters" => \%inputhash ,
                                                           capture_file => $file,);

(Class or Instance method) This method is used to find the next packet that matches the input hash
refrenece from the packet capture file.

Returns packet hash reference if there is a match, else undef.
During the Class call need to include the capture file in the input hash .

=over

=item Options

=over

=item C<< "match-parameters" => \%input_hash >>

(Required) This parameter contains the (hash reference) hash structure information of the match which needs to be found
on the captured file . This hash structure will vary for different hosts,filers & protocols.
The hash reference will be passed to the method.

=item C<< capture_file => $file >>

(Optional) The packet captured file .

=back

=back

=cut

sub get_packet_by_parameter {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    # Validation of the input parameters.

    my %opts = validate_with(
        params => \@_,
        spec   => {
            "match-parameters" => { type => HASHREF },
            capture_file       => { type => SCALAR, optional => 1 },
        },
    );

    my $obj;
    if ( !ref($pkg_or_obj) ) {
        if ( $opts{capture_file} ) {
            $obj = $pkg_or_obj->new( capture_file => $opts{capture_file} );
            delete( $opts{capture_file} );
        } else {
            NATE::BaseException->throw("Captured file is a required parameter");
        }
    } else {
        $obj = $pkg_or_obj;
    }

    my $pkt;
    my $pkt_number = $obj->{pkt_number};

    # Convert input hash to a standard hash
    my $match_h = _process_input_hash( $opts{"match-parameters"} );

    # Get one packet at a time starting a current position
    while ( $pkt = $obj->next_packet() ) {

        # Return packet if it matches
        if ( _compare_hashes( $match_h, $pkt ) ) {
            $Log->exit() if $may_exit;
            return $pkt;
        }
    }

    # No packet matched, so position the packet number and offset
    # to what it was before the search
    if ( !seek( $obj->{fh}, $obj->{pkt_map}[$pkt_number], 0 ) ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("Could not position file pointer");
    }

    # Reset the current packet number and offset
    $obj->{pkt_number} = $pkt_number;
    $obj->{offset}     = $obj->{pkt_map}[$pkt_number];

    $Log->exit() if $may_exit;
    return;
}

#---------------------------------------------------------------------
# Internal function
# Convert input hash to a standard hash, e.g.:
#   %input = (
#       'Ethernet:type' => 2048,
#       'Ethernet:IP:protocol' => [6, 17],
#       'Ethernet:IP:src' => '172.17.153.181',
#       'Ethernet:IP:UDP:dst_port' => 137,
#       'Ethernet:IP:UDP' => {
#           checksum   => 15755,
#           src_port   => 55270,
#           total_size => 76,
#       },
#   );
#   %output = (
#       Ethernet => {
#           type => 2048,
#           IP => {
#               protocol => [6, 17],
#               src => '172.17.153.181',
#               UDP => {
#                   dst_port   => 137,
#                   checksum   => 15755,
#                   src_port   => 55270,
#                   total_size => 76,
#               },
#           },
#       },
#   );
#---------------------------------------------------------------------
sub _process_input_hash {
    $Log->enter() if $may_enter;
    my $hash = shift;
    my ( %ret, @list, $key, $item, $ptr, $last_item, $ref_k, $ref_p );

    foreach $key ( keys %$hash ) {

        # Split input key
        @list = split /:/, $key;

        # Initialize pointer to top level of hash
        $ptr = \%ret;

        # Get the key for the last leg
        $last_item = pop @list;
        foreach $item (@list) {

            # Initialize hash for each leg of the input key
            $ptr->{$item} = {} if !exists $ptr->{$item};

            # Set the pointer to each level
            $ptr = $ptr->{$item};
        }
        if ( ref $ptr ne 'HASH' ) {
            NATE::BaseException->throw("Bad match key for '$key'");
        } elsif ( exists $ptr->{$last_item} ) {

            # The last leg already exists, so check for compatibility
            $ref_k = ref $hash->{$key};
            $ref_p = ref $ptr->{$last_item};
            if ( $ref_k ne $ref_p ) {
                NATE::BaseException->throw("Bad match key for '$key'");
            } elsif ( $ref_k eq 'HASH' ) {

                # Combine both hashes
                $ptr->{$last_item} =
                  { %{ $ptr->{$last_item} }, %{ $hash->{$key} } };
            } elsif ( $ref_k eq 'ARRAY' ) {

                # Combine both arrays
                $ptr->{$last_item} =
                  [ @{ $ptr->{$last_item} }, @{ $hash->{$key} } ];
            } else {

                # The reference type is not supported
                NATE::BaseException->throw("Unknown reference $ref_k");
            }
        } else {

            # Set the value for this key
            $ptr->{$last_item} = $hash->{$key};
        }

    }
    $Log->exit() if $may_exit;
    return \%ret;
}

#---------------------------------------------------------------------
# Internal function
# Compare two hashes -- returns 1 if h1 is a subset of h2
# The h1 hash could have an array reference as the value so it
# could match any of the values in the array
#---------------------------------------------------------------------
sub _compare_hashes {
    $Log->enter() if $may_enter;
    my ( $h1, $h2 ) = @_;

    $Log->debug( "Pattern hash given the user" . Dumper($h1) );
    $Log->debug( "The hash which is used for comparison" . Dumper($h2) );

    my ( $key, $item, $list, $found );

    # Check each key in the h1 hash
    foreach $key ( keys %$h1 ) {

        # Compare fails if the key does not exists on the h2 hash
        return 0 if !exists $h2->{$key};

        if ( ref $h1->{$key} eq 'HASH' && ref $h2->{$key} eq 'HASH' ) {

            # If both values are hash references, call this function recursively
            return 0 if !_compare_hashes( $h1->{$key}, $h2->{$key} );
        } elsif (
            ref $h2->{$key}
            || (   ref $h1->{$key}
                && ref $h1->{$key} ne 'Regexp'
                && ref $h1->{$key} ne 'ARRAY' )
          )
        {

            # Compare fails if value is a reference except for valid values
            # on h1 that could be a compile regular expression or an array
            # reference for OR style compare
            return 0;
        } else {

            # Get list of values to match
            if ( ref $h1->{$key} eq 'ARRAY' ) {

                # This is already an array
                $list = $h1->{$key};
            } else {

                # Make an array with single value
                $list = [ $h1->{$key} ];
            }

            # Compare all values in list with the value on h2
            $found = 0;
            foreach $item (@$list) {
                if ( ref $item eq 'Regexp' ) {
                    $found = 1 if $h2->{$key} =~ /$item/;
                } elsif ( $item eq $h2->{$key} ) {
                    $found = 1;
                }
                last if $found;
            }
            return 0 if !$found;
        }
    }
    $Log->exit() if $may_exit;

    # Everything matched
    return 1;
}

=head2  next_packet

$packetparser->next_packet();

(Instance Method) This method returns the hash reference of the next packet in the capture file ,
undef if there are no other packets present .

=cut

sub next_packet {
    $Log->enter() if $may_enter;
    my $obj = shift;

    my ( %record, $header, $data, $count );

    # Save file offset for this packet
    $obj->{pkt_map}[ $obj->{pkt_number} ] = $obj->{offset};

    # Get record header
    $count = $obj->_read( buffer => \$header, count => 16 );
    return $Log->exit() if $count < 16;

    # Unpack record header
    @record{qw(seconds msecs length_inc length_orig)} =
      unpack( $format{ $obj->{ident} }{header_rec}, $header );

    # Get the data
    $count = $obj->_read( buffer => \$data, count => $record{length_inc} );

    # Calculate number of drops
    $record{drops} = $record{length_orig} - $record{length_inc};

    if ( $obj->{tcpdump_header}{link_type} == 1 ) {

        # Decode ethernet layer
        $record{Ethernet} = $obj->_decode_ethernet_layer( data_ref => \$data );
    } else {

        # Unknown link layer
        $record{data} = $data;
    }

    # Increment packet number
    $record{pkt_number} = $obj->{pkt_number}++;

    $Log->exit() if $may_exit;
    return \%record;
}

=head2 get_packet_by_index

(Instance call)
$packetparser->get_packet_by_index( index => $index,);
      or

(Class call)
my $index_pkt = NACL::MTask::PacketParser->get_packet_by_index(
    index => 0,
    capture_file => $File
);

(Class or Instance Method) This method is used for fetching the packet . Returns packet hash reference
or undef if the index is out of bounds.

=over

=item Options

=over

=item C<< index=>$index >>

(Required) Index of packet to fetch.

=item C<< capture_file => $file >>

(Optional) The packet captured file .

=back

=back

=cut

sub get_packet_by_index {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            index        => { type => SCALAR },
            capture_file => { type => SCALAR, optional => 1 },
        },
        no_command_interface => 1,
    );

    my $obj;
    if ( !ref($pkg_or_obj) ) {
        if ( $opts{capture_file} ) {
            $obj = $pkg_or_obj->new( capture_file => $opts{capture_file} );
            delete( $opts{capture_file} );
        } else {
            NATE::BaseException->throw("Captured file is a required parameter");
        }
    } else {
        $obj = $pkg_or_obj;
    }

    my $pkt;
    my $index = $opts{index};

    if ( defined $obj->{pkt_map} && $index < @{ $obj->{pkt_map} } ) {

        # The index is less than the current packet offset so position
        # the file pointer to the offset of packet given by index
        if ( !seek( $obj->{fh}, $obj->{pkt_map}[$index], 0 ) ) {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw("Could not position file pointer");
        }

        # Reset the current packet number and offset
        $obj->{pkt_number} = $index;
        $obj->{offset}     = $obj->{pkt_map}[$index];
    }

    # Get one packet at a time until the packet given by the index is found
    do {
        $pkt = $obj->next_packet();
    } while ( defined $pkt && $pkt->{pkt_number} < $index );

    $Log->exit() if $may_exit;
    return $pkt;
}

=head2 rewind

$packetparser->rewind();

(Instance method) This method is used for rewinding the trace file. It sets the file pointer to the offset
of the first packet. Returns 1 on successful rewind operation else throws exception.

=cut

sub rewind {
    $Log->enter() if $may_enter;
    my $self = shift;

    # Initialize filehandle
    $self->_get_filehandle();

    # Position the file pointer to the offset of the first packet
    if ( !seek( $self->{fh}, $self->{pkt_map}[0], 0 ) ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Error while positioning the file pointer:$!");
    }

    # Reset the current packet number and offset
    $self->{pkt_number} = 0;
    $self->{offset}     = $self->{pkt_map}[0];

    $Log->exit() if $may_exit;
    return 1;
}

#---------------------------------------------------------------------
# Internal method
# Get filehandle of the current trace file
#---------------------------------------------------------------------
sub _get_filehandle {
    $Log->enter() if $may_enter;
    my $obj = shift;
    my ( $file, $ident, $header, $count );

    if ( !defined $obj->{fh} ) {

        # Open trace file
        $file = $obj->{capture_file};

        if ( !open( $obj->{fh}, "<$file" ) ) {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw("Could not open trace file:");
        }
        binmode( $obj->{fh} );

        # Initialize offset
        $obj->{offset} = 0;

        # Get file type identifier
        $count = $obj->_read( buffer => \$ident, count => 4 );
        NATE::BaseException->throw("Not a tcpdump file:")
          if !exists $format{$ident};

        # Get header information
        $count = $obj->_read( buffer => \$header, count => 20 );
        NATE::BaseException->throw("Not a tcpdump file:") if $count < 20;

        # Unpack header
        @{ $obj->{tcpdump_header} }
          {qw(major minor zone_offset accuracy dump_length link_type)} =
          unpack( $format{$ident}{header_fmt}, $header );

        # Initialize packet number
        $obj->{pkt_number} = 0;
        $obj->{ident}      = $ident;
        $obj->{pkt_map}[0] = $obj->{offset};
    }

    $Log->exit() if $may_exit;
    return $obj->{fh};
}

#---------------------------------------------------------------------
# Internal method
# Wrapper for the system call read() to update the internal offset
#---------------------------------------------------------------------
sub _read {
    $Log->enter() if $may_enter;
    my $self   = shift;
    my %opts   = @_;
    my $buffer = $opts{buffer};
    my $count  = $opts{count};

    my $out = read( $self->_get_filehandle, $$buffer, $count );
    NATE::BaseException->throw("Could not read trace file:") if !defined $out;
    $self->{offset} += $out;

    $Log->exit() if $may_exit;
    return $out;
}

#---------------------------------------------------------------------
# Internal method
# Decodes the ethernet layer
#---------------------------------------------------------------------
sub _decode_ethernet_layer {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = @_;
    my $data = $opts{data_ref};
    my ( %ret, $header, $tci, $type, $size, $delta );
    my $decode      = 1;
    my $header_size = 14;

    # Get total size of packet
    $ret{packet_size} = length $$data;

    # Decode ethernet layer (RFC 894) Ethernet II
    $header = substr( $$data, 0, 14, '' );
    @ret{qw(dst src type)} = unpack( 'H12H12n', $header );

    # Add the ':' to the MAC addresses
    $ret{src} = join( ':', $ret{src} =~ /(..)/g );
    $ret{dst} = join( ':', $ret{dst} =~ /(..)/g );

    if ( $ret{type} <= 1500 ) {

        # IEEE 802.2/802.3 Encapsulation (RFC 1042)
        $ret{size} = $ret{type};
        $header = substr( $$data, 0, 8, '' );
        @ret{qw(DSAP SSAP cntl org_code type)} = unpack( 'CCCa3n', $header );
        $ret{org_code} = unpack( 'N', "\000$ret{org_code}" );
    }

    if ( $ret{type} == 0x8100 ) {

        # 802.1Q Virtual LAN packet
        $header_size += 4;
        $header = substr( $$data, 0, 4, '' );
        ( $tci, $type ) = unpack( 'nn', $header );
        $ret{VLAN} = {
            priority => ( $tci >> 13 ),
            CFI      => ( $tci >> 12 ) & 0x1,
            VID      => ( $tci & 0xFFF ),
        };
        if ( $type <= 1500 ) {
            $ret{VLAN}{size}    = $type;
            $ret{VLAN}{trailer} = substr( $$data, $type, 4, '' );
            $ret{VLAN}{data}    = $$data;
            substr( $$data, 0, $type + 4, '' );
        } else {
            $ret{type} = $type;
        }
    }

    if ( $ret{type} == 0x0800 ) {

        # Type of ethernet packet is IP (Internet Protocol)
        $ret{IP} = $self->_decode_ip_layer($data);
    } elsif ( $ret{type} == 0x86dd ) {

        # Type of ethernet packet is IPv6 (Internet Protocol version 6)
        $ret{IPv6} = $self->_decode_ipv6_layer($data);
    } elsif ( $ret{type} == 0x0806 ) {

        # Type of ethernet packet is ARP (Address Resolution Protocol)
        $ret{ARP} = $self->_decode_arp_layer( data => $data );
    } elsif ( $ret{type} == 0x8035 ) {

        # Type of ethernet packet is RARP (Reverse Address Resolution Protocol)
        $ret{RARP} = $self->_decode_arp_layer( data => $data );
    } else {

        # Type does not have any decoding method so just return the data
        $ret{data} = $$data;
        $decode = 0;
    }

    # Get actual packet size
    if ( $decode && ( $size = length $$data ) ) {
        $ret{packet_size} -= $size;
        if ( $ret{packet_size} < 46 + $header_size ) {

            # This packet is < 46 bytes, get the padding
            $delta = 46 + $header_size - $ret{packet_size};
            $ret{trailer} = substr( $$data, 0, $delta, '' );
            $size -= $delta;
            $ret{packet_size} += $delta;
        }

        # Get the CRC if any
        $ret{CRC} = unpack( 'N', $$data ) if $size >= 4;
    }

    $Log->exit() if $may_exit;
    return \%ret;
}

#---------------------------------------------------------------------
# Internal method
# Decodes the IP layer
#---------------------------------------------------------------------
sub _decode_ip_layer {
    $Log->enter() if $may_enter;
    my ( $self, $data ) = @_;
    my ( %ret, @list, $header, $count, $temp, $ip_data );

    # Get the IP version and header length
    $temp = unpack( 'C', $$data );

    # Header length in bytes
    $count = 4 * ( $temp & 0x0F );

    # Get full IP header
    $header = substr( $$data, 0, $count, '' );

    # Decode IP header
    @list = unpack( 'CCnnnCCna4a4', $header );

    %ret = (
        version         => ( $temp >> 4 ),
        IHL             => ( $temp & 0x0F ),
        header_size     => $count,
        total_size      => $list[2],
        id              => $list[3],
        fragment_offset => ( $list[4] & 0x1FFF ),
        TTL             => $list[5],
        protocol        => $list[6],
        checksum        => $list[7],
        src             => join( '.', unpack( 'CCCC', $list[8] ) ),
        dst             => join( '.', unpack( 'CCCC', $list[9] ) ),
    );

    # Include both TOS and DSCP/ECN from the same byte
    $temp = $list[1];

    # Add TOS (Type of Service) from RFC 1349
    $ret{TOS} = {
        precedence    => ( $temp >> 5 ),
        delay         => ( ( $temp >> 4 ) & 0x01 ),
        throughput    => ( ( $temp >> 3 ) & 0x01 ),
        reliability   => ( ( $temp >> 2 ) & 0x01 ),
        monetary_cost => ( ( $temp >> 1 ) & 0x01 ),
    };

    # Add DSCP (Differentiated Services Code Point)
    # and ECN (Explicit Congestion Notification) from RFC 3168
    $ret{DSCP} = ( $temp >> 2 );
    $ret{ECN}  = ( $temp & 0x03 );

    $temp = ( $list[4] >> 13 );
    $ret{flags} = {
        DF => ( ( $temp >> 1 ) & 0x01 ),
        MF => ( $temp & 0x01 ),
    };

    if ( $count > 20 ) {

        # Save IP header options
        $ret{options} = substr( $header, 20 );
    }

    # Get full data for IP packet
    $ip_data = substr( $$data, 0, $ret{total_size} - $ret{header_size}, '' );

    if ( $ret{protocol} == 6 ) {

        # Type of IP packet is TCP (Transmission Control Protocol)
        $ret{TCP} = $self->_decode_tcp_layer( \$ip_data );
    } elsif ( $ret{protocol} == 17 ) {

        # Type of IP packet is UDP (User Datagram Protocol)
        $ret{UDP} = $self->_decode_udp_layer( \$ip_data );
    } elsif ( $ret{protocol} == 1 ) {

        # Type of IP packet is ICMP (Internet Control Message Protocol)
        $ret{ICMP} = $self->_decode_icmp_layer( \$ip_data );
    } elsif ( $ret{protocol} == 2 ) {

        # Type of IP packet is IGMP (Internet Group Management Protocol)
        $ret{IGMP} = $self->_decode_igmp_layer( \$ip_data );
    } else {

        # Type does not have any decoding method so just return the data
        $ret{data} = $ip_data;
    }

    $Log->exit() if $may_exit;
    return \%ret;
}

#---------------------------------------------------------------------
# Internal method
# Decodes the IPv6 layer
#---------------------------------------------------------------------
sub _decode_ipv6_layer {
    $Log->enter() if $may_enter;
    my ( $self, $data ) = @_;
    my ( %ret, @list, $header, $ip_data );

    # Get full IPv6 header of fix size of 40 bytes
    $header = substr( $$data, 0, 40, '' );

    # load Socket6 if IPv6 decoding.
    use Socket6 qw(AF_INET6 inet_ntop);

    # Decode IPv6 header
    @list = unpack( 'B32nCCa16a16', $header );
    foreach ( $list[4], $list[5] ) {
        $_ = inet_ntop( AF_INET6, $_ );
        $_ = Net::IP::ip_compress_address( $_, 6 );
    }

    %ret = (
        version =>
            unpack( 'C', pack( 'B*', 0 x 4 . substr( $list[0], 0, 4 ) ) ),
        traffic_class =>
            unpack( 'C', pack( 'B*', 0 x 0 . substr( $list[0], 4, 8 ) ) ),
        flow_lable =>
            unpack( 'N', pack( 'B*', 0 x 12 . substr( $list[0], 12, 20 ) ) ),
        payload_length => $list[1],
        next_header    => $list[2],
        hop_limit      => $list[3],
        src            => $list[4],
        dst            => $list[5],
    );

    # Get full data for IP packet
    $ip_data = $$data;

    if ( $ret{next_header} == 6 ) {

        # Type of IPv6 packet is TCP (Transmission Control Protocol)
        $ret{TCP} = $self->_decode_tcp_layer( \$ip_data );
    } elsif ( $ret{next_header} == 17 ) {

        # Type of IP packet is UDP (User Datagram Protocol)
        $ret{UDP} = $self->_decode_udp_layer( \$ip_data );
    } elsif ( $ret{next_header} == 1 ) {

   # Type of IP packet is ICMPv6 (Internet Control Message Protocol version 6)
        $ret{ICMP} = $self->_decode_icmp_layer( \$ip_data );
    } elsif ( $ret{next_header} == 58 ) {

   # Type of IP packet is ICMPv6 (Internet Control Message Protocol version 6)
        $ret{ICMPv6} = $self->_decode_icmpv6_layer( \$ip_data );
    } elsif ( $ret{next_header} == 2 ) {

        # Type of IP packet is IGMP (Internet Group Management Protocol)
        $ret{IGMP} = $self->_decode_igmp_layer( \$ip_data );
    } elsif ( $ret{next_header} == 44 ) {

        #Type of IP packet is Fragmented header
        $ret{FRAGMENT} = $self->_decode_fragment_layer( \$ip_data );
    } elsif ( $ret{next_header} == 41 ) {

        #Type of IP packet is Internet Protocol version 6
        $ret{IPv6} = $self->_decode_ipv6_layer( \$ip_data );
    } else {
        $ret{data} = $ip_data;
    }

    $Log->exit() if $may_exit;
    return \%ret;
} ## end sub _decode_ipv6_layer

#---------------------------------------------------------------------
# Internal method
# Decodes the TCP layer
#---------------------------------------------------------------------
sub _decode_tcp_layer {
    $Log->enter() if $may_enter;
    my ( $self, $data ) = @_;
    my ( %ret, @list, $header, $count, $rpc, $temp );

    # Decode the TCP layer header
    ( undef, $temp ) = unpack( 'a12C', $$data );
    $temp >>= 4;
    $count  = 4 * $temp;
    $header = substr( $$data, 0, $count, '' );
    @list   = unpack( 'nnNNnnnn', $header );

    %ret = (
        src_port    => $list[0],
        dst_port    => $list[1],
        seq_number  => $list[2],
        ack_number  => $list[3],
        window_size => $list[5],
        checksum    => $list[6],
        urgent_ptr  => $list[7],
        hl          => $temp,
        header_size => $count,
        flags       => {
            FIN => ( $list[4] & 0x01 ),
            SYN => ( ( $list[4] >> 1 ) & 0x01 ),
            RST => ( ( $list[4] >> 2 ) & 0x01 ),
            PSH => ( ( $list[4] >> 3 ) & 0x01 ),
            ACK => ( ( $list[4] >> 4 ) & 0x01 ),
            URG => ( ( $list[4] >> 5 ) & 0x01 ),
        },
    );

    if ( $rpc = $self->_decode_rpc_layer( $data, 6 ) ) {
        $ret{RPC} = $rpc;
    } else {
        $ret{data} = $$data;
    }

    $Log->exit() if $may_exit;
    return \%ret;
}

#---------------------------------------------------------------------
# Internal method
# Decodes the UDP layer
#---------------------------------------------------------------------
sub _decode_udp_layer {
    $Log->enter() if $may_enter;
    my ( $self, $data ) = @_;
    my ( %ret, @list, $header, $out );

    # Decode the UDP layer header
    $header = substr( $$data, 0, 8, '' );
    @list = unpack( 'nnnn', $header );

    %ret = (
        src_port   => $list[0],
        dst_port   => $list[1],
        total_size => $list[2],
        checksum   => $list[3],
    );

    if ( $ret{src_port} == 520 && $ret{dst_port} == 520 ) {
        $out = $self->_decode_rip_layer($data);
        $ret{RIP} = $out if defined $out;
    } elsif ( $out = $self->_decode_rpc_layer( $data, 17 ) ) {
        $ret{RPC} = $out;
    }

    if ( length $$data ) {

        # Add the raw data if unknown format
        $ret{data} = $$data;
    }

    $Log->exit() if $may_exit;
    return \%ret;
}

#---------------------------------------------------------------------
# Internal method
# Decodes the ICMPv6 layer
#---------------------------------------------------------------------
sub _decode_icmpv6_layer {
    $Log->enter() if $may_enter;
    my ( $self, $data ) = @_;
    my ( %ret, @list, $header );

    $header = substr( $$data, 0, 4, '' );
    @list = unpack( 'CCn', $header );

    %ret = (
        type     => $list[0],
        code     => $list[1],
        checksum => $list[2],
        data     => $$data,
    );

    $Log->exit() if $may_exit;
    return \%ret;
}

#---------------------------------------------------------------------
# Internal method
# Decodes the ICMP layer
#---------------------------------------------------------------------
sub _decode_icmp_layer {
    $Log->enter() if $may_enter;
    my ( $self, $data ) = @_;
    my ( %ret, @list, $header );

    $header = substr( $$data, 0, 4, '' );
    @list = unpack( 'CCn', $header );

    %ret = (
        type     => $list[0],
        code     => $list[1],
        checksum => $list[2],
        data     => $$data,
    );

    $Log->exit() if $may_exit;
    return \%ret;
}

#---------------------------------------------------------------------
# Internal method
# Decodes the IGMP layer
#---------------------------------------------------------------------
sub _decode_igmp_layer {
    $Log->enter() if $may_enter;
    my ( $self, $data ) = @_;
    my ( %ret, @list, $header, %data );

    $header = substr( $$data, 0, 8, '' );
    @list = unpack( 'CCna4', $header );
    if ( $list[0] >= 0x11 && $list[0] <= 0x13 ) {

        # Version 1
        %data = (
            version => 1,
            type    => ( $list[0] >> 4 ),
        );

        %ret = (
            %data,
            checksum => $list[2],
            address  => join( '.', unpack( 'CCCC', $list[3] ) ),
        );
    } elsif ( $list[0] > 0x13 && $list[0] <= 0x21 ) {

        # Version 2
        %data = (
            version      => 2,
            type         => $list[0],
            max_response => $list[1],
        );

        %ret = (
            %data,
            checksum => $list[2],
            address  => join( '.', unpack( 'CCCC', $list[3] ) ),
        );
    } else {

        # Version 3

        my @grp_rec = ();
        while ( my $group_rec = substr( $$data, 0, 8, '' ) ) {
            my @data = unpack( 'CCna4', $group_rec );
            my $g_r = {
                record_type       => $data[0],
                aux_data_len      => $data[1],
                num_src           => $data[2],
                multicast_address => join( '.', unpack( 'CCCC', $data[3] ) ),
            };
            push( @grp_rec, $g_r );
        }
        %ret = (
            version           => 3,
            type              => $list[0],
            checksum          => $list[2],
            group_record      => \@grp_rec,
            num_group_records => scalar @grp_rec,
        );
    }
    $Log->exit() if $may_exit;
    return \%ret;
}

#---------------------------------------------------------------------
# Internal method
# Decodes the ARP/RARP layer
#---------------------------------------------------------------------
sub _decode_arp_layer {
    $Log->enter() if $may_enter;
    my $obj  = shift;
    my %opts = @_;
    my $data = $opts{data};

    my ( %ret, $header, $uformat );

    # Decode ARP/RARP header
    $header = substr( $$data, 0, 8, '' );
    @ret{qw(hardware_type protocol_type hardware_size protocol_size operation)}
      = unpack( 'nnCCn', $header );

    # Get the right format for the hardware and protocol addresses
    $uformat =
      ( 'H' . ( 2 * $ret{hardware_size} ) . 'a' . $ret{protocol_size} ) x 2;

    # Decode the hardware and protocol addresses
    @ret{
        qw(sender_ether_addr sender_proto_addr target_ether_addr target_proto_addr)
      } = unpack( $uformat, $$data );

    if ( $ret{hardware_type} == 1 ) {

        # Add the ':' to the MAC addresses
        $ret{sender_ether_addr} =
          join( ':', $ret{sender_ether_addr} =~ /(..)/g );
        $ret{target_ether_addr} =
          join( ':', $ret{target_ether_addr} =~ /(..)/g );
    }

    if ( $ret{protocol_type} == 0x0800 ) {

        # Make the IP protocol addresses human readable
        $ret{sender_proto_addr} =
          join( '.', unpack( 'CCCC', $ret{sender_proto_addr} ) );
        $ret{target_proto_addr} =
          join( '.', unpack( 'CCCC', $ret{target_proto_addr} ) );
    }

    # Remove full data for ARP/RARP packet from the data
    substr( $$data, 0, 8 + 2 * ( $ret{hardware_size} + $ret{protocol_size} ),
        '' );

    $Log->exit() if $may_exit;
    return \%ret;
}

#---------------------------------------------------------------------
# Internal method
# Decodes the RPC call and reply messages
#---------------------------------------------------------------------
sub _decode_rpc_layer {
    $Log->enter() if $may_enter;
    my ( $self, $data, $proto ) = @_;
    my ( %ret, $buffer, $psize );
    my $size   = length $$data;
    my $offset = 0;

    if ( $proto == 6 ) {

        # TCP packet
        return if $size < 12;

        # Decode fragment header
        $buffer = substr( $$data, 0, 4 );
        $psize = unpack( 'N', $buffer );
        $ret{fragment_hdr}{size}          = $psize & 0x7FFFFFFF;
        $ret{fragment_hdr}{last_fragment} = $psize >> 31;
        $offset += 4;
    } else {

        # UDP packet
        return if $size < 8;
    }

    # Decode the XID and RPC type
    $buffer = substr( $$data, $offset, 8 );
    @ret{qw(xid type)} = unpack( 'NN', $buffer );
    $offset += 8;

    if ( $ret{type} == 0 ) {

        # RCP call
        return if $size < $offset + 32;
        $buffer = substr( $$data, $offset, 16 );
        $offset += 16;
        @ret{qw(rpc_version program version procedure)} =
          unpack( 'NNNN', $buffer );
        $ret{credential} = _rpc_credential( $data, \$offset, $size );
        return if !$ret{credential};
        $ret{verifier} = _rpc_credential( $data, \$offset, $size );
        return if !$ret{verifier};
    } elsif ( $ret{type} == 1 ) {

        # RCP reply
        return if $size < $offset + 12;
        $buffer = substr( $$data, $offset, 4 );
        $offset += 4;
        $ret{reply_status} = unpack( 'N', $buffer );
        if ( $ret{reply_status} ) {
            $buffer = substr( $$data, $offset, 4 );
            $offset += 4;
            $ret{rejected_status} = unpack( 'N', $buffer );
            if ( $ret{rejected_status} ) {
                $buffer = substr( $$data, $offset, 4 );
                $offset += 4;
                $ret{auth_status} = unpack( 'N', $buffer );
            } else {
                return if $size < $offset + 8;
                $buffer = substr( $$data, $offset, 8 );
                $offset += 8;
                @{ $ret{rpc_mismatch} }{qw(low high)} = unpack( 'NN', $buffer );
            }
        } else {
            $ret{verifier} = _rpc_credential( $data, \$offset, $size );
            return if !$ret{verifier};
            return if $size < $offset + 4;
            $buffer = substr( $$data, $offset, 4 );
            $offset += 4;
            $ret{accepted_status} = unpack( 'N', $buffer );
            if ( $ret{accepted_status} == 2 ) {
                return if $size < $offset + 8;
                $buffer = substr( $$data, $offset, 8 );
                $offset += 8;
                @{ $ret{prog_mismatch} }{qw(low high)} =
                  unpack( 'NN', $buffer );
            }
        }
    } else {

        # Not an RPC packet
        return;
    }

    # Remove the RPC header
    substr( $$data, 0, $offset, '' );
    $ret{data} = $$data;

    $Log->exit() if $may_exit;
    return \%ret;
}

#---------------------------------------------------------------------
# Internal method
# Decodes RIP (Routing Information Protocol)
#---------------------------------------------------------------------
sub _decode_rip_layer {
    $Log->enter() if $may_enter;
    my ( $self, $data ) = @_;
    my ( %ret, $header, @list );
    my $offset = 0;

    # Decode RIP header
    $header = substr( $$data, $offset, 4 );
    $offset += 4;
    @list = unpack( 'CCn', $header );

    return if ( $list[0] == 1 && $list[2] );
    %ret = (
        command => $list[0],
        version => $list[1],
    );

    if ( $ret{version} == 2 ) {
        $ret{domain} = $list[2];
    }

    while ( length $$data >= $offset + 20 ) {
        $header = substr( $$data, $offset, 20 );
        $offset += 20;
        if ( $ret{version} == 1 ) {

            # RIP-v1
            @list = unpack( 'nna4NNN', $header );

            # Check correct format
            if (   $list[1]
                || $list[3]
                || $list[4]
                || $list[5] < 1
                || $list[5] > 16 )
            {
                return;
            }
            if ( $list[0] == 2 ) {

                # Address family identifier is for an IP address
                $list[2] = join( '.', unpack( 'CCCC', $list[2] ) );
            }
            push @{ $ret{routes} },
              {
                AFI     => $list[0],
                address => $list[2],
                metric  => $list[5],
              };
        } elsif ( $ret{version} == 2 ) {

            # RIP-v2
            @list = unpack( 'nna4a4a4N', $header );
            if ( $list[0] == 2 ) {

                # Address family identifier is for IP addresses
                $list[2] = join( '.', unpack( 'CCCC', $list[2] ) );
                $list[3] = join( '.', unpack( 'CCCC', $list[3] ) );
                $list[4] = join( '.', unpack( 'CCCC', $list[4] ) );
            }
            push @{ $ret{routes} },
              {
                AFI      => $list[0],
                tag      => $list[1],
                address  => $list[2],
                subnet   => $list[3],
                next_hop => $list[4],
                metric   => $list[5],
              };
        }
    }

    # Check for invalid extra data
    return if length $$data > $offset;

    # There is no more data
    $$data = '';

    $Log->exit() if $may_exit;
    return \%ret;
}

#---------------------------------------------------------------------
# Internal method
# Decode the RPC credentials
#---------------------------------------------------------------------
sub _rpc_credential {
    $Log->enter() if $may_enter;
    my ( $data, $offset, $size ) = @_;
    my ( %ret, $header );

    # Get the flavor and size of credential
    $header = substr( $$data, $$offset, 8 );
    $$offset += 8;
    @ret{qw(flavor size)} = unpack( 'NN', $header );

    # Check for correct size
    return if $size < $$offset + $ret{size};

    # Just get the credential data
    $ret{data} = substr( $$data, $$offset, $ret{size} );
    $$offset += $ret{size};

    $Log->exit() if $may_exit;
    return \%ret;
}

#-------------------------------------------------------------------------------
# User defined subroutine for decoding fragment header in IPv6 Packet
#-------------------------------------------------------------------------------
sub _decode_fragment_layer {
    $Log->enter() if $may_enter;
    my ( $self, $data ) = @_;

    my $nh = substr( $$data, 0, 8, '' );
    my @list = unpack( 'C2H4n', $nh );

    my %ret;

    %ret = (
        nxt_header => $list[0],
        reserved   => $list[1],
        fragment_offset =>
          _bin2dec( substr( substr( _hex2bin( $list[2] ), -6 ), 0, 3 ) ),
        more_fragment =>
          _bin2dec( substr( substr( _hex2bin( $list[2] ), -6 ), -3 ) ),
        identification => $list[3],
    );

    my $ip_data = $$data;

    if ( $ret{nxt_header} == 17 ) {

        # Type of IP packet is UDP (User Datagram Protocol)
        $ret{UDP} = $self->_decode_udp_layer( \$ip_data );
    } elsif ( $ret{nxt_header} == 6 ) {

        # Type of IPv6 packet is TCP (Transmission Control Protocol)
        $ret{TCP} = $self->_decode_tcp_layer( \$ip_data );
    } elsif ( $ret{nxt_header} == 1 ) {

     # Type of IP packet is ICMPv6 (Internet Control Message Protocol version 6)
        $ret{ICMP} = $self->_decode_icmp_layer( \$ip_data );
    } elsif ( $ret{nxt_header} == 58 ) {

     # Type of IP packet is ICMPv6 (Internet Control Message Protocol version 6)
        $ret{ICMPv6} = $self->_decode_icmpv6_layer( \$ip_data );
    } elsif ( $ret{nxt_header} == 2 ) {

        # Type of IP packet is IGMP (Internet Group Management Protocol)
        $ret{IGMP} = $self->_decode_igmp_layer( \$ip_data );
    } elsif ( $ret{nxt_header} == 41 ) {

        #Type of IP packet is Internet Protocol version 6
        $ret{IPv6} = $self->_decode_ipv6_layer( \$ip_data );
    } else {
        $ret{data} = $ip_data;
    }

    $Log->exit() if $may_exit;
    return \%ret;
}

#-------------------------------------------------------------------------------
# User defined subroutine for converting hexadecimal to binary format
#-------------------------------------------------------------------------------
sub _hex2bin {
    my $h    = shift;
    my $hlen = length($h);
    my $blen = $hlen * 4;
    return unpack( "B$blen", pack( "H$hlen", $h ) );
}

#-------------------------------------------------------------------------------
# User defined subroutine for converting binary to decimal format
#-------------------------------------------------------------------------------
sub _bin2dec {
    return unpack( "N", pack( "B32", substr( "0" x 32 . shift, -32 ) ) );
}

1;
