#
#   Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Ping Task Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::MTask::Ping

=head1 DESCRIPTION

C<NACL::MTask::Ping> provides method to perform ping test between two host,
with requested ip address and other information. it will configure ip address
on both host and than perform ping operation. it also provides method to
clean-up the configuration once test is finished.
this task supports both IPv4 and IPv6 addresses.
it also supports both 7Mode and CMode.

two host could be as follows:

=over

=item 1. both filer

=item 2. both client

=item 3. filer and client

=back

=cut

package NACL::MTask::Ping;

use strict;
use warnings;

use base qw(NACL::MTask::MTask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR HASHREF OBJECT BOOLEAN ARRAYREF);
use NATE::Exceptions::Argument;
use NATE::BaseException;
use NACL::C::Component;
use NACL::C::Ifconfig;
use NACL::C::Client::Ifconfig;
use NACL::CS::Client::Ifconfig;
use NACL::CS::Ifconfig;
use NACL::CS::NetworkPort;
use NACL::STask::Client::Ifconfig;
use NACL::STask::NetworkPort;
use NACL::STask::NetworkInterface;
use NACL::STask::NetworkRoute;
use NACL::STask::NetworkRoutingGroupsRoute;
use NACL::STask::Node;
use NACL::MTask::NetworkUtils qw (ping);
use Net::IP qw(:PROC);
use NATE::Time qw(timeout2time);
use Socket;

use Class::MethodMaker [
    new  => [ '-hash', 'new' ],
    hash => 'host1',
    hash => 'host2',
];

=head1 METHODS

=head2 new

  my $ping = NACL::MTask::Ping->new(host1 => \%host1_details,
                                       host2 => \%host2_details,
                                                                  );

Return a new Ping task object with the given attributes.

=cut

=head2 verify

=over

=item case 1: perform ping on IPv4 address.

my $ping_obj = NACL::MTask::Ping->verify(

    host1 => {
        command_interface       => $host1_command_interface,
        port                    => e1a,
        address                 => '18.18.18.18'
        mask_or_prefixlen       => '255.255.255.0',
        mtu                     => 9000,
    },

    host2 => {
        command_interface       => $host2_command_interface,
        port                    => eth1,
        address                 => '18.18.18.19'
        mask_or_prefixlen       => '255.255.255.0',
        mtu                     => 9000,
    },
);

=item case 2: perform ping on IPv6 address.

my $ping_obj = NACL::MTask::Ping->verify(

    host1 => {
        command_interface       => $host1_command_interface,
        port                    => e1a,
        address                 => '2001::1'
        mask_or_prefixlen       => '64',
        mtu                     => 9000,
    },

    host2 => {
        command_interface       => $host2_command_interface,
        port                    => eth1,
        address                 => '2001::2'
        mask_or_prefixlen       => '64',
        mtu                     => 9000,
    },
);

=item case 3: perform ping on IPv4 address between CMode filer and linux client.

my $ping_obj = NACL::MTask::Ping->verify(

    host1 => {
        command_interface       => $host1_command_interface,
        vserver                 => $vserver,
        port                    => e1a,
        lif                     => $lif_name,
        address                 => '12.12.12.12'
        mask_or_prefixlen       => '255.255.255.0',
        mtu                     => 9000,
    },

    host2 => {
        command_interface       => $host2_command_interface,
        port                    => eth1,
        address                 => '12.12.12.11'
        mask_or_prefixlen       => '255.255.255.0',
        mtu                     => 9000,
    },
);

=back

(Class method) This method performs following operation based on information
provided to it. it needs two host to complete ping operation.

=over

=item 1. save host1 port informatin to restore back after test.

=item 2. save host2 port informatin to restore back after test.

=item 3. configure host1 port with requested IP address, netmask or prefixlen and mtu

=item 4. configure host2 port with requested IP address, netmask  or prefixlen and mtu

=item 5. ping from host1 to host2

=item 6. ping from host2 to host1

=item 7. return Ping object if both side ping successful.
    or raise exception if ping fail in either direction.

=back

=over

=item Options

=over

=item C<< host1 => \%host1_information >>

(Required) It contains information about first host, among the two host needed
to complete ping operation.

    host1 => {
        command_interface       => $host_command_interface,
        vserver                 => $vserver,
        lif                     => $lif_name,
        port                    => e1a,
        address                 => '12.12.12.12'
        mask_or_prefixlen       => '255.255.255.0',
        mtu                     => 9000,
    }
command_interface => $command_interface

(Required) command_interface of host.

port => $port

(Required) physical network port.

address => $address

(Required) IPv4 or IPv6 address.

mask_or_prefixlen => $netmask_or_prefixlen

(Required) Netmask if IPv4 address. or prefix length if IPv6 address.

mtu => $mtu

(Required) mtu of network port.

vserver => $vserver

(Required for CMode, Not Applicable for 7Mode). The name of vserver.

lif => $lif_name

(Optional for CMode, Not Applicable for 7Mode). lif name.

gateway => $gateway

(Optional for CMode, Not Applicable for 7Mode/client) gateway for creating 
the route.

nacltask_if_exists_lif => $action

(Optional for CMode, Not Applicable for 7Mode/client)
Specifies an action to be taken if the interface already exists.

$action
- if set to "reuse" (default), no action is performed.
- if set to "die", an exception is raised.
- if set to "purge", deletes the existing interface before creating it.

nacltask_create_route => $boolean

(Optional for CMode, Not Applicable for 7Mode/client)
Default is 0
Creates a route for lif. 

=item C<< host2 => \%host2_information >>

(Required) It contains information about second host, among the two host
needed to complete ping operation.

C<< host2 >>  parameters are same as C<< host1 >>.

=item C<< nacltask_polling_interval => $interval >>

(Optional) Check the ping is reachable after ipaddress is configured.
for every second.
Default is 1 second

=item C<< nacltask_ping_timeout => $timeout >>

(Optional)  Controls how long the command will wait before completing.
Default is 60 seconds.

=back

=back

=cut

sub verify {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            host1                     => { type => HASHREF },
            host2                     => { type => HASHREF },
            nacltask_polling_interval => { type => SCALAR, default => 1 },
            nacltask_ping_timeout     => { type => SCALAR, default => 60 },
        },
    );
    my %args;
    my $polling_interval = delete $opts{nacltask_polling_interval};
    my $timeout          = delete $opts{nacltask_ping_timeout};
    my $end_time;
    my %arg;

    foreach my $phost ( keys(%opts) ) {
        my %target = validate_with(
            params => $opts{$phost},
            spec   => {
                command_interface => { type => OBJECT },
                port              => { type => SCALAR },
                address           => { type => SCALAR },
                mask_or_prefixlen => { type => SCALAR },
                mtu               => { type => SCALAR, default => 9000 },
                nacltask_create_route => { type => BOOLEAN, default => 0},
                'data-protocol' => {type => SCALAR | ARRAYREF, default => ['cifs','nfs','fcache'] },
                role             => { type => SCALAR, default => 'data' },
                nacltask_if_exists_lif => {
                    type      => SCALAR,
                    default   => 'reuse',
                    callbacks => {
                        "Value of 'nacltask_if_exists_lif' should be 'die','purge' or 'reuse'" =>
                        sub { $_[0] =~ /^(?:die|reuse|purge)$/ }
                    }
                },
            },
            allow_extra => 1,
        );

        NACL::C::Component->_scalar_to_array_deprication_warning(
            params  => ['data-protocol'],
            options => \%target,
        );

        if (   $opts{$phost}->{command_interface}->isa('NACL::C::SystemNode')
            && $opts{$phost}->{command_interface}->mode() eq 'CMode' ) {
            NATE::BaseException->throw("Vserver parameter is not found")
                if ( !$opts{$phost}->{vserver} );
            $opts{$phost}->{lif} = 'lif' . time()
                if ( !$opts{$phost}->{lif} );
        }
        my ( $ci, $config, $cm_route_obj );
        $ci = $opts{$phost}->{command_interface};
        if ( $ci->isa('NACL::C::Client') ) {
            $config = _configure_interface_host(%target);
        } elsif ( $ci->isa('NACL::C::SystemNode') && $ci->mode() eq "CMode" )
        {
            $config = _configure_interface_cm(%target);
        } elsif ( $ci->isa('NACL::C::SystemNode') && $ci->mode() eq "7Mode" )
        {
            $config = _configure_interface_7m(%target);
        } ## end elsif ($ci->mode() eq "7Mode")
        $args{$phost} = { %target, 'nacltask_backup' => $config };
    } ## end foreach my $phost ('host1',...

    my ( $h1, $h2 ) = keys(%opts);
    $args{$h1}{destination} = $opts{$h2}->{address};
    $args{$h2}{destination} = $opts{$h1}->{address};

    foreach my $host ( keys(%opts) ) {
        $end_time = timeout2time($timeout);
        $Log->comment(
            "Ping from $opts{$host}->{address} to $args{$host}{destination}");
        my $ret;
        while ( time() < $end_time ) {

            if ( $opts{$host}->{command_interface}->isa('NACL::C::SystemNode')
                && $opts{$host}->{command_interface}->mode() eq "CMode" ) {
                $arg{nacltask_shell_type} = ['ngsh'];
                $arg{vserver}             = $opts{$host}->{vserver};
            }
            $ret = ping(
                command_interface => $opts{$host}->{command_interface},
                destination       => $args{$host}{destination},
                %arg,
            );
            if ($ret) {
                $end_time = 0;
            } else {
                Tharn::snooze($polling_interval);
            }
        }
        if ( !$ret ) {
            my $obj1 = $pkg->new(%args);
            $obj1->cleanup();
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Ping failed from $opts{$host}->{address} to $args{$host}{destination}"
            );
        }
    }
    my $obj = $pkg->new(%args);
    $Log->exit() if $may_exit;
    return $obj;

} ## end sub verify

=head2 cleanup

$ping_object->cleanup();

(Instance Method) This Method cleanup any configure done by ping() method and
restore previous IP address configurations.

=cut

sub cleanup {
    $Log->enter() if $may_enter;
    my $obj = shift;
    foreach my $phost ( keys(%$obj) ) {
        my ( $ci, $hostname, $cs_obj,$config);
        my $host = $obj->{$phost};
        $ci       = $host->{command_interface};
        $hostname = $ci->hostrec()->hostname();
        $config   = $host->{nacltask_backup};
        $cs_obj   = $config->{network_port};
        $Log->debug("Restoring configuration for $hostname");
        if ( $ci->isa('NACL::C::Client') ) {
            my ( $address, $netmask );
            if ( Net::IP::ip_is_ipv6( $host->{address} ) ) {
                NACL::C::Client::Ifconfig->remove_address(
                    command_interface => $ci,
                    interface         => $host->{port},
                    address           => $host->{address},
                    prefixlen         => $host->{mask_or_prefixlen},
                );
            } else {
                my %args;
                $args{address} = $cs_obj->inet_addr();
                if ( $args{address} ) {
                    $args{netmask} = $cs_obj->mask();
                    if ( $args{netmask} =~ /^f(.)*/ ) {
                        $args{netmask} = "0x" . $args{netmask};
                    }
                } else {
                    $args{address} = "0.0.0.0";
                }
                NACL::C::Client::Ifconfig->set_address(
                    command_interface => $ci,
                    interface         => $host->{port},
                    %args,
                );
            }
            NACL::C::Client::Ifconfig->set_mtu(
                command_interface   => $ci,
                interface           => $host->{port},
                'max-transfer-unit' => $cs_obj->mtu(),
            );
        } elsif ( $ci->mode() eq "CMode" ) {
            if(exists($config->{network_interface})) {
                $config->{network_interface}->purge(
                    nacltask_verify   => 1,
                 );
            }
            if(exists($config->{network_route})) {
                $config->{network_route}->purge();
            }
            NACL::STask::NetworkPort->modify(
                command_interface     => $ci,
                node                  => $ci->name(),
                port                  => $host->{port},
                mtu                   => $cs_obj->mtu(),
                'duplex-admin'        => $cs_obj->{duplex_admin},
                'speed-admin'         => $cs_obj->{speed_admin},
                'autonegotiate-admin' => $cs_obj->{autonegotiate_admin},
                nacltask_verify       => 1,
            );

        } elsif ( $ci->mode() eq "7Mode" ) {
            my ( $netmask, $address, $linklocal, $autoconf );
            my %arg = (
                command_interface => $ci,
                interface         => $host->{port},
            );
            if ( Net::IP::ip_is_ipv6( $host->{address} ) ) {
                $arg{address}   = $cs_obj->ipv6_info()->[0]->{ip};
                $arg{prefixlen} = $cs_obj->ipv6_info()->[0]->{prefixlen};
                $linklocal      = $cs_obj->ipv6_info()->[0]->{link_local};
                $autoconf       = $cs_obj->ipv6_info()->[0]->{autoconf};
                $arg{address}   = '::' if ( $autoconf || $linklocal );
            } else {
                $arg{address} = $cs_obj->ipv4_info()->[0]->{ip};
                if ( $arg{address} ) {
                    $arg{netmask} = $cs_obj->ipv4_info()->[0]->{mask};
                } else {
                    $arg{address} = '0.0.0.0';
                }
            }
            ## config ip address
            NACL::C::Ifconfig->ifconfig(%arg);

            my $mediatype = $cs_obj->mediatype();
            $mediatype = "auto" if ( $mediatype =~ /auto/ );
            NACL::C::Ifconfig->ifconfig(
                command_interface => $ci,
                interface         => $host->{port},
                mtusize           => $cs_obj->mtu(),
                flowcontrol       => $cs_obj->flowcontrol(),
                mediatype         => $mediatype
            );
        }
    } ## end foreach my $phost ('host1',...
    $Log->exit() if $may_exit;
} ## end sub cleanup

########################################################################
# Method:    _configure_interface_cm
# Objective: Modify the port with the mtu as input &
#            Create a new logical interface with the input parameters
# Options:
#   port               - required - port on which the ip is configured
#   address            - required - ipaddress to be configured on interface
#   mask_or_prefixlen  - required - netmask to be set for ipaddress
#   mtu                - required - mtusize to be set for that interface
#   lif                - required - logical interface to be created
#   vserver            - required - vserver for lif creation
#   role               - required - role for lif creation
# Output: Returns a reference of hash of formatted data
########################################################################

sub _configure_interface_cm {
    $Log->enter() if $may_enter;
    my (%host)  = @_;
    my $ci      = $host{command_interface};
    my %return=();
    my $port_cs = NACL::CS::NetworkPort->fetch(
        command_interface => $ci,
        filter            => { node => $ci->name(), port => $host{port}, },
    );
    $return{network_port} = $port_cs;

    NACL::STask::NetworkPort->modify(
        command_interface     => $ci,
        node                  => $ci->name(),
        port                  => $host{port},
        mtu                   => $host{mtu},
        'speed-admin'         => 'auto',
        'duplex-admin'        => 'auto',
        'autonegotiate-admin' => 'true',
        nacltask_verify       => 1,
    );

    my %arg = ();
    
    if ( Net::IP::ip_is_ipv6( $host{address} ) ) {
        $arg{'netmask-length'} = $host{mask_or_prefixlen};
    } else {
        $arg{netmask} = $host{mask_or_prefixlen};
    }

    $Log->comment(
        "Create new network interface with requested configurations.");
    my ($was_created,$was_created_route,$gateway,$route_obj);

    my $lif=NACL::STask::NetworkInterface->create(
        command_interface  => $ci,
        lif                => $host{lif},
        vserver            => $host{vserver},
        'home-node'        => $ci->name(),
        'home-port'        => $host{port},
        'role'             => $host{role},
        address            => $host{address},
        'data-protocol'   =>  $host{'data-protocol'},
        nacltask_if_exists => $host{nacltask_if_exists_lif},
        'status-admin'     => "up",
        'failover-policy'  => "disabled",
        'auto-revert'      => 'true',
        nacltask_verify    => 1,
        _was_created       => \$was_created,
        %arg,
    );
    
    $return{network_interface} = $lif if($was_created);

    $gateway=$host{gateway} if exists($host{gateway});

    if($was_created && $host{nacltask_create_route} && Net::IP::ip_is_ipv4($host{address})) {
        if(!defined($gateway)) {
            my $role=uc($host{role});
            my $hash = NACL::STask::Node->get_hostrec_lif_info(
                command_interface => $ci,
                roles             => [$role],
                required_fields   => [qw( GATEWAY )],
            );
            my $subnet= _ipv4_subnet_matcher($host{address}, $arg{netmask});
            foreach my $_lif(@{$hash->{$role}}) {
                if($subnet->($_lif->{GATEWAY})) {
                    $gateway=$_lif->{GATEWAY};
                    last;
                }
            }
        }
        if(!defined($gateway)) {
            NATE::BaseException->throw("Unable find to suitable found gateway for $host{address}/$arg{netmask}");
        }

        my $lif_cs=$lif->state();
        if ( $ci->version_manager->has_uichange(
            uichange        => 'ipspaces-ms4',
            throw_exception => 0 )
        ){
            $route_obj=NACL::STask::NetworkRoute->create(
                command_interface => $ci,
                nacltask_if_exists => 'purge',
                gateway => $gateway,
                vserver  => $lif->vserver(),
                destination => "$host{address}/$arg{netmask}",
                _was_created       => \$was_created_route
            );
        } else {
            $route_obj=NACL::STask::NetworkRoutingGroupsRoute->create(
                command_interface => $ci,
                nacltask_if_exists => 'purge',
                gateway => $gateway,
                vserver  => $lif->vserver(),
                'routing-group' => $lif_cs->routing_group(),
                destination => "$host{address}/$arg{netmask}",
                _was_created       => \$was_created_route
            );
        }
        $return{network_route} = $route_obj if($was_created);
    }
    $Log->exit() if $may_exit;
    return \%return;
} ## end sub _configure_interface_cm

########################################################################
# Method:    _configure_interface_7m
# Objective: Configure the interface with the input parameters
# Options:
#   port               - required - port on which the ip is configured
#   address            - required - ipaddress to be configured on interface
#   mask_or_prefixlen  - required - netmask to be set for ipaddress
#   mtu                - required - mtusize to be set for that interface
# Output: Returns a reference of hash of formatted data
########################################################################

sub _configure_interface_7m {
    $Log->enter() if $may_enter;
    my (%host) = @_;
    my $ci = $host{command_interface};
    $Log->debug("Configuring the 7mode interfaces");
    my $output_7m = NACL::CS::Ifconfig->fetch(
        command_interface => $ci,
        filter            => { interface => $host{port} },
    );
    $Log->comment("configure IP address and other requested configurations");
    my %arg = ();
    if ( Net::IP::ip_is_ipv6( $host{address} ) ) {
        $arg{prefixlen} = $host{mask_or_prefixlen};
    } else {
        $arg{netmask} = $host{mask_or_prefixlen};
    }

    NACL::C::Ifconfig->ifconfig(
        command_interface => $ci,
        interface         => $host{port},
        address           => $host{address},
        mtusize           => $host{mtu},
        up                => 1,
        mediatype         => 'auto',
        flowcontrol       => 'full',
        %arg,
    );

    $Log->exit() if $may_exit;
    return {'network_port' => $output_7m};
} ## end sub _configure_interface_7m

########################################################################
# Method:    _configure_interface_host
# Objective: Configure the interface with the input parameters
# Options:
#   port               - required - port on which the ip is configured
#   address            - required - ipaddress to be configured on interface
#   mask_or_prefixlen  - required - netmask to be set for ipaddress
#   mtu                - required - mtusize to be set for that interface
# Output: Returns a reference of hash of formatted data
########################################################################

sub _configure_interface_host {
    $Log->enter() if $may_enter;
    my (%host) = @_;
    my $ci = $host{command_interface};
    $Log->comment("Configuring the host interfaces");

    my $interface = $host{port};
    my $type      = $ci->hostrec()->hosttype();

    my $state = NACL::CS::Client::Ifconfig->fetch(
        command_interface => $ci,
        filter            => { 'interface' => $interface, },
    );
    $Log->comment("configure IP address and other requested configurations");

    if ( Net::IP::ip_is_ipv6( $host{address} ) ) {
        NACL::STask::Client::Ifconfig->add_address(
            command_interface  => $ci,
            address            => $host{address},
            prefixlen          => $host{'mask_or_prefixlen'},
            interface          => $host{port},
            nacltask_if_exists => 'reuse',
            nacltask_verify    => 1
        );
    } else {
        NACL::C::Client::Ifconfig->set_address(
            command_interface   => $ci,
            address             => $host{address},
            netmask             => $host{'mask_or_prefixlen'},
            interface           => $host{port},
            'max-transfer-unit' => $host{mtu},
        );
    }

    $Log->exit() if $may_exit;
    return {'network_port' => $state};
} ## end sub _configure_interface_host

sub _ipv4_subnet_matcher {
    my ($net, $mask) = @_;

    $net = inet_aton($net);
    $mask = $mask =~ /\./ ? inet_aton($mask) : _cidr2mask_ipv4($mask);

     my $masked_net = $net & $mask;

    return sub { (inet_aton(shift) & $mask) eq $masked_net };
}

sub _cidr2mask_ipv4 {
    my ($length) = @_;
    return pack "N", 0xffffffff << (32 - $length);
}
1;
