# $Id: $
#
## Copyright (c) 2014 NetApp, Inc., All Rights Reserved
##   Any use, modification, or distribution is prohibited
##   without prior written consent from NetApp, Inc.
##

#
## @summary WAFL Ontap Setup MTask Module
## @author dl-awg-warp-svl
## @change
##
## @status shared
## @pod here

package NACL::MTask::Setup;

use strict;
use warnings;
use parent qw(NACL::MTask::MTask);
use Carp;
use NACL::MTask::Setup::GlobalParams qw(:global);

use Params::Validate qw(validate_with :types);

use NACL::MTask::Setup::Object::Node;
use NACL::MTask::Setup::Object::Aggregate;
use NACL::MTask::Setup::Object::Vserver;
use NACL::MTask::Setup::Object::Volume;

use NATE::Exceptions::Argument;

use NACL::GeneralUtils qw(random_name_generator);
use NATE::ParamSet qw(param_global);
use NATE::Log qw(log_global);

use Readonly;

use Class::MethodMaker [
    scalar =>
      [ { -type => 'NACL::MTask::Setup::Object::Node' }, '_node_mtask_obj', ],
    scalar => [
        { -type => 'NACL::MTask::Setup::Object::Aggregate' },
        '_aggregate_mtask_obj'
    ],
    scalar => [
        { -type => 'NACL::MTask::Setup::Object::Vserver' },
        '_vserver_mtask_obj'
    ],
    scalar => [
        { -type => 'NACL::MTask::Setup::Object::Volume' }, '_volume_mtask_obj'
    ],
    scalar => [{ -type => 'NACL::Cleanup' }, 'cleanup_manager'],

    array => [ { -type => 'NACL::STask::Node' },       'nodes' ],
    array => [ { -type => 'NACL::STask::Aggregate' },  'aggregates' ],
    array => [ { -type => 'NACL::STask::Vserver' },    'vservers' ],
    array => [ { -type => 'NACL::STask::Volume' },     'volumes' ],

    scalar => [
        'type',                'initial_blog_counters',
        'final_blog_counters', 'cluster_setup_type',
        'disable_pre_setup',   'disable_post_setup',
        'disable_pre_cleanup',
    ],
];

my $Log   = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
my $Param = param_global();

sub new {
    $Log->enter;
    my $pkg = shift;

    my %opts = $pkg->_common_validate_with(
        params               => \@_,
        additional_spec      => {
            type => { type => SCALAR, optional => 1 },
            cleanup_manager =>  { isa => 'NACL::Cleanup', required => 1},
        },
        ignore_primary_keys  => 1,
        no_command_interface => 1,
    );

    my $type = $opts{type};

    my $self =
      ($type)
      ? $pkg->_load_subclass_and_instantiate($type)
      : bless {}, $pkg;

    $self->cleanup_manager($opts{cleanup_manager});
    $self->_process_global_flags;

    return $self;

    $Log->exit;
} ## end sub new

sub create {

    $Log->enter;

    my $self = shift;

    $self->_verify_invocation( style => 'instance_only' );

    my %opts = $self->_common_validate_with(
        params          => \@_,
        additional_spec => {
            node_options      => { type => HASHREF, default => {} },
            aggregate_options => { type => HASHREF, default => {} },
            vserver_options   => { type => HASHREF, default => {} },
            volume_options    => { type => HASHREF, default => {} },
            run_pre_setup     => { type => BOOLEAN, default => 1 },
        },
        ignore_primary_keys  => 1,
        no_command_interface => 1,
    );

    my $volume_options = $opts{volume_options};

    $self->_find_nodes_and_set_attributes( $opts{node_options} );

    $self->_pre_setup if ( $opts{run_pre_setup} );
    my $node = $self->nodes->[0];

    $self->_set_opts_find_or_create_aggrs_set_attributes(
        $opts{aggregate_options},
        $node, $volume_options );

    $self->_find_or_create_vservers_set_attributes( $opts{vserver_options},
        $node );

    $self->_set_opts_find_or_create_vols_set_attributes( $node,
        $volume_options );

    $Log->exit;
} ## end sub create

sub post_setup {
    my $self = shift;
    return if ( $self->disable_post_setup );

    #############################################
    $Log->step('Running Post Setup');
    #############################################
    $self->_process_tasks('POST_SETUP_TASKS',\%POST_SETUP_TASKS_OBJECT_TYPE_FOR);
} ## end sub post_setup

sub _pre_setup {
    my $self = shift;
    return if ( $self->disable_pre_setup );

    ## TODO logging to be moved to a separate log file
    $self->_process_tasks('PRE_SETUP_TASKS',\%PRE_SETUP_TASKS_OBJECT_TYPE_FOR);
} ## end sub _pre_setup

sub _pre_cleanup {
    my $self = shift;
    return if ( $self->disable_pre_cleanup );

    ## TODO logging to be moved to a separate log file
    $self->_process_tasks('PRE_CLEANUP_TASKS',\%PRE_CLEANUP_TASKS_OBJECT_TYPE_FOR);

    ## TODO
    # a) delete all files
    # Check for check for delayed frees, zombie counter
    # b) delete all snapshots
    # c) delete all snapmirror relationships
    # d) delete all clone volumes
} ## end sub _pre_cleanup

sub cleanup {
    my $self = shift;

    $Log->step('Running Cleanup');

    $self->cleanup_manager->run();
    $self->_aggregate_mtask_obj->cleanup();

    $Log->step('Cleanup Finished');
} ## end sub cleanup
## private subroutines

sub _load_subclass_and_instantiate {
    my $pkg  = shift;
    my $type = shift;

    my $class = $pkg . '::' . $type;
    $Log->step($class);
    eval "use $class";
    NATE::BaseException->throw("Unable to import $class\n$@") if ($@);
    return $class->new;
} ## end sub _load_subclass_and_instantiate

sub _process_global_flags {
    my $self = shift;
    $self->disable_pre_setup( $Param->get($GLOBAL_PARAM_FOR{DISABLE_PRE_SETUP}) );
    $self->disable_post_setup( $Param->get($GLOBAL_PARAM_FOR{DISABLE_POST_SETUP}) );
    $self->disable_pre_cleanup( $Param->get($GLOBAL_PARAM_FOR{DISABLE_PRE_CLEANUP}) );
} ## end sub _process_global_flags

sub _find_nodes_and_set_attributes {
    my $self           = shift;
    my $node_mtask_obj = $self->_find_nodes(@_);
    $self->_set_node_attributes($node_mtask_obj);
}

sub _find_nodes {
    my $self = shift;
    my ($node_options) = @_;
    return NACL::MTask::Setup::Object::Node->new( %{$node_options} );
} ## end sub _find_nodes

sub _set_node_attributes {
    my $self = shift;
    my ($node_mtask_object) = @_;

    $self->_node_mtask_obj($node_mtask_object);
    $self->nodes( $node_mtask_object->nodes );
}

sub _set_opts_find_or_create_aggrs_set_attributes {
    my $self           = shift;
    my $aggr_mtask_obj = $self->_set_opts_find_or_create_aggrs(@_);
    $self->_set_aggregate_attributes($aggr_mtask_obj);
}

sub _set_opts_find_or_create_aggrs {
    my $self = shift;
    my ( $aggr_options, $node, $volume_options ) = @_;

    $self->_set_aggregate_total_volume_size( $aggr_options, $volume_options );
    $self->_set_node_filters_on_aggregate( $aggr_options, $node );
    return $self->_find_or_create_aggregates( $aggr_options, $node );

} ## end sub _set_opts_find_or_create_aggrs

sub _set_aggregate_attributes {
    my $self = shift;
    my ($aggr_mtask_obj) = @_;
    $self->_aggregate_mtask_obj($aggr_mtask_obj);
    $self->aggregates( $aggr_mtask_obj->aggregates );
}

sub _set_aggregate_total_volume_size {
    my $self = shift;
    my ( $aggregate_options, $vol_create_opts ) = @_;
    $aggregate_options->{total_volume_size} =
        NACL::MTask::Setup::Object::Volume->_compute_vol_size($vol_create_opts);
} ## end sub _set_aggregate_total_volume_size

sub _set_node_filters_on_aggregate {
    my $self = shift;
    my ( $aggregate_options, $node ) = @_;
    if ( $self->_node_mtask_obj->is_2nodes_setup ) {
        $aggregate_options->{create_options}->{node} = $node;
        $aggregate_options->{find_options}->{filter}->{node} = [$node];
    }
} ## end sub _set_node_filters_on_aggregate

sub _find_or_create_aggregates {
    my $self = shift;
    my ( $aggregate_options, $node ) = @_;

    return NACL::MTask::Setup::Object::Aggregate->new(
        node => $node,
        cleanup_manager => $self->cleanup_manager,
        %{$aggregate_options},
    );
} ## end sub _find_or_create_aggregates

sub _find_or_create_vservers_set_attributes {
    my $self              = shift;
    my $vserver_mtask_obj = $self->_find_or_create_vservers(@_);
    $self->_set_vserver_attributes($vserver_mtask_obj);
}

sub _find_or_create_vservers {
    my $self = shift;
    my ( $vserver_options, $node ) = @_;

    return NACL::MTask::Setup::Object::Vserver->new(
        node => $node,
        cleanup_manager => $self->cleanup_manager,
        %{$vserver_options},
    );
} ## end sub _find_or_create_vservers

sub _set_vserver_attributes {
    my $self = shift;
    my ($vserver_mtask_obj) = @_;
    $self->_vserver_mtask_obj($vserver_mtask_obj);
    $self->vservers( $vserver_mtask_obj->vservers );
}

sub _set_opts_find_or_create_vols_set_attributes {
    my $self          = shift;
    my $vol_mtask_obj = $self->_set_opts_find_or_create_vols(@_);
    $self->_set_volume_attributes($vol_mtask_obj);
}

sub _set_opts_find_or_create_vols {
    my $self = shift;
    my ( $node, $volume_options ) = @_;

    $self->_set_volume_create_options(
        $volume_options,
        {
            vserver   => $self->vservers->[0]->vserver,
            aggregate => $self->aggregates->[0]->aggregate,
        }
    );
    return $self->_find_or_create_volumes( $volume_options, $node );

} ## end sub _set_opts_find_or_create_vols

sub _set_volume_attributes {
      my $self = shift;
      my ($vol_mtask_obj) = @_;
      $self->_volume_mtask_obj($vol_mtask_obj);
      $self->volumes( $vol_mtask_obj->volumes );
}

sub _set_volume_create_options {
      my $self = shift;
      my ( $volume_options, $value_of ) = @_;

      while ( my ( $attr, $value ) = each %{$value_of} ) {
          $volume_options->{create_options}->{$attr} ||= $value;
      }
} ## end sub _set_volume_create_options

sub _find_or_create_volumes {
      my $self = shift;
      my ( $volume_options, $node ) = @_;

      return NACL::MTask::Setup::Object::Volume->new(
          node => $node,
          cleanup_manager => $self->cleanup_manager,
          %{$volume_options}
      );
} ## end sub _find_or_create_volumes

sub _process_tasks {
    my $self = shift;
    my $task_type = shift;
    my $task_object_type_for = shift;

    my $tasks_csv = $Param->get($GLOBAL_PARAM_FOR{$task_type});
    if($tasks_csv) {
        my @tasks = split ',',$tasks_csv;
        foreach my $task_method (@tasks) {
            if(!exists($task_object_type_for->{$task_method})) {
                $Log->warn("$task_method is not implemented skipping the task");
            } else {
                my $object_type = $task_object_type_for->{$task_method};
                $task_method = lc $task_method;
                $Log->step("Executing $task_method on object $object_type");
                my $mtask_object = "_$object_type\_mtask_obj";
                $self->$mtask_object->$task_method;
            }
        }
    }

}

# Storable blesses the object (during thaw) with the appropriate class
# name and load the library if we place storable hooks. Hence the foll
# hooks are added, otherwise there is no customization needed in freeze
# and thaw operations.

use Storable qw(freeze thaw);

sub STORABLE_freeze {
        my ( $self, $cloning ) = @_;
            return if $cloning;    # Regular default serialization
                my $data  = {%$self};
                    my $dummy = "";
                        return ( $dummy, $data );
} ## end sub STORABLE_freeze

sub STORABLE_thaw {
        my ( $self, $cloning, $dummy, $data ) = @_;
            %$self = %$data;
}

1;

__END__

=pod

=head1 NAME

NACL::MTask::Setup

=head1 DESCRIPTION

This is a generic library to setup aggregates vservers and volumes.
This acts as a base class and instantiation point for more specific setup classes.
It also does presetup, post setup and pre cleanup based on global params

See L<NACL::MTask::Setup::Constants#Setup Types|lib-NACL-MTask-Setup-Constants-pm/Setup Types>
for different supported setup types their respected classes and the methods supported

=head1 ATTRIBUTES

=head2 nodes

List of STask node objects found

=head2 aggregates

List of STask aggregate objects found or created

=head2 vservers

List of STask vserver objects found or created

=head2 volumes

List of STask volume objects found or created

=head1 METHODS

=head2 new

    my $setup = NACL::MTask::Setup->new;
    my $setup = NACL::MTask::Setup->new(type => $Replica);

This method returns the setup object based on the type parameter passed.
By default object of the Setup class is returned. If type is set, then object
of the derived setup class based on type is returned. See the derived classes
for methods available on the object. The PODs for derived classes are linked
in the Constants POD. Please see below for link to the constants POD

=over

=item Options

=over

=item C<< type => $Replica >>

(Optional)
Type of setup to be created. For supported types and the respective class POD
See L<NACL::MTask::Setup::Constants#Setup Types|lib-NACL-MTask-Setup-Constants-pm/Setup Types>

=back

=back

=head2 create

    my $setup = NACL::MTask::Setup->new;

    $setup->create();
(Instance method)

    $setup->create(
        volume_options => {count    => 3},
        node_options   => {template => '1node'},
        run_pre_setup      => 0,
    );

This method does the following

=over

=item * Finds nodes

=item * Perform pre setup operations based on global params

=item * Find or create Aggregates

=item * Find or create Vservers

=item * Find or Create Volumes

=item * Perform post setup operations based on global params

=item * Perform pre cleanup operations based on global params

=back

=over

=item Options

=over

=item C<< node_options => $node_setup_options >>

(Optional, Hashref)
Parameters taken by node setup MTask.

See L<NACL::MTask::Setup::Object::Node::new|lib-NACL-MTask-Setup-Object-Node-pm/new>

=item C<< aggregate_options => $aggregate_options >>

(Optional, Hashref)
Parameters taken by aggregate setup MTask.

See L<NACL::MTask::Setup::Object::Aggregate::new|lib-NACL-MTask-Setup-Object-Aggregate-pm/new>

=item C<< vserver_options => $vserver_options >>

(Optional, Hashref)
Parameters taken by vserver setup MTask.

See L<NACL::MTask::Setup::Object::Vserver::new|lib-NACL-MTask-Setup-Object-Vserver-pm/new>

=item C<< volume_options => $volume_options >>

(Optional, Hashref)
Parameters taken by volume setup MTask.

See L<NACL::MTask::Setup::Object::Volume::new|lib-NACL-MTask-Setup-Object-Volume-pm/new>

=item C<< run_pre_setup => 0|1 >>

(Optional, Boolean)
Runs pre setup if set to 1. Default(1)

=back

=back

=head2 post_setup

$setup->post_setup;

(Instance method)

Post setup operations are controlled by global parameters. The 
post setup operations for objects can be found here L<NACL::MTask::Setup::GlobalParams#Post setup|lib-NACL-MTask-Setup-GlobalParams-pm/Post setup tasks>
Operations are performed on Nodes, Aggregates, Vservers, Volumes

=head1 Pre setup operations

Pre setup operations are controlled by global parameters. The 
pre setup operations can be found here L<NACL::MTask::Setup::GlobalParams#Pre setup|lib-NACL-MTask-Setup-GlobalParams-pm/Pre setup tasks>
Operations are performed on Nodes, Aggregates, Vservers, Volumes

=head1 Pre cleanup operations

Pre cleanup operations are controlled by global parameters. The 
pre cleanup operations can be found here
L<NACL::MTask::Setup::GlobalParams#Pre cleanup|lib-NACL-MTask-Setup-GlobalParams-pm/Pre cleanup tasks>
Operations are performed on Nodes, Aggregates, Vservers, Volumes

=cut
