# $Id: $
#
## Copyright (c) 2014 NetApp, Inc., All Rights Reserved
##   Any use, modification, or distribution is prohibited
##   without prior written consent from NetApp, Inc.
##

#
## @summary WAFL Ontap Aggregate Setup MTask Module
## @author dl-awg-warp-svl
## @change
##
## @status shared
## @pod here

package NACL::MTask::Setup::Object::Aggregate;

use strict;
use warnings;
use parent qw(NACL::MTask::Setup::ObjectFactory);
use NACL::MTask::Setup::GlobalParams qw(:Aggregate);

use Params::Validate qw(:types);

use NATE::ParamSet qw(param_global);
use NATE::Log qw(log_global);
use NACL::C::UnitNormalization 'convert_to_bytes_int';
use NACL::CS::StoragePoolShowAvailableCapacity;

use Class::MethodMaker [
    array  => [ { -type => 'NACL::STask::Aggregate' }, 'aggregates' ],
    scalar => [ { -type => 'NACL::STask::Node' },      'node', ],
    scalar => [{ -type => 'NACL::Cleanup' }, 'cleanup_manager'],
    scalar =>
      [qw/is_hybrid count total_volume_size is_flexgroup disable_space_check/],
];

sub OBJECT_TYPE {return 'Aggregate'}

sub DEFAULT_FIND_OPTIONS {
    return {
        canned_filters  => [qw(online_not_root)],
        sort_descending => ['availsize'],
    }
}


my $Log = log_global;
my @SP_Cleanup_Array;

sub new {
    $Log->enter;

    my $pkg = shift;

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            node  => { isa => 'NACL::STask::Node' },
            cleanup_manager =>  { isa => 'NACL::Cleanup', required => 1},
            count => {
                type    => SCALAR,
                regex   => qr/^\d+$/,
                default => undef,
            },
            create_options              => { type => HASHREF, default => {} },
            find_options                => { type => HASHREF, default => {} },
            storage_pool_find_options   => { type => HASHREF, default => {} },
            storage_pool_create_options => { type => HASHREF, default => {} },
            add_disk_opts               => { type => HASHREF, default => {} },
            total_volume_size           => { type => SCALAR,  optional => 1 },
            is_flexgroup                => { type => BOOLEAN, default  => 0 },
            is_hybrid                   => { type => BOOLEAN, default  => 0 },
            use_storage_pools           => { type => BOOLEAN, default  => 0 },
        },
        ignore_primary_keys  => 1,
        no_command_interface => 1,
    );

    return if ( $opts{count} && $opts{count} == 0 );

    my $self = $pkg->SUPER::new( type => OBJECT_TYPE );

    $self->node( delete $opts{node} );
    $self->total_volume_size( delete $opts{total_volume_size} );
    $self->is_flexgroup(delete $opts{is_flexgroup});
    $self->cleanup_manager(delete $opts{cleanup_manager});

    $self->_process_global_params;

    my @args = ( $opts{create_options}, $opts{find_options}, );
    if( $self->is_flexgroup && ! $opts{count} ) {
        $self->count(1);
        $self->_find_or_create_aggr_till_vol_size_is_accommodated( @args );
    } else {
        $self->count($opts{count} || 1);
        $self->_process_options_and_find_or_create( @args);
    }

    #Create Hybrid aggregates
    if ( $opts{is_hybrid} ){
        foreach my $aggr ($self->aggregates) {
            #Check the aggregate is not already a hybrid aggregate
            if ( $aggr->get_one_state_attribute('hybrid') =~ /false/ ) {
                $aggr->convert_to_hybrid();
                my $add_disk_opts = {
                                        command_interface   => $self->node,
                                        %{ $opts{add_disk_opts} },
                                    };

                if ( $opts{use_storage_pools} ){
                    my $usable_sp = $self->_find_or_create_storage_pool( $opts{storage_pool_find_options},
                        $opts{storage_pool_create_options} );

                    $self->_add_disks_to_hya( $aggr, $add_disk_opts, $usable_sp );
                } else {
                    $self->_add_disks_to_hya( $aggr, $add_disk_opts, undef );
                }
            }
        }
    }

    $Log->exit;
    return $self;
} ## end sub new

sub cache_hybrid_aggregate_counter {

    my $self  = shift;

    # Flash Pool ADDON
    # Save read cache and write cache count before test
    # Can be used for fp testings

    if ( $self->is_hybrid ) {
        #############################################
        $Log->step('Caching hybrid aggr counters');
        #############################################
        $self->initial_hybird_aggregate_counters(
            $self->_fetch_hybrid_aggr_counters );
    }
} ## end sub cache_hybrid_aggregate_counter

sub _process_global_params {
    my $self  = shift;
    my $param = param_global;

    $self->is_hybrid( $param->get($AGGREGATE_PARAM_FOR{IS_HYBRID}) || $self->is_hybrid);
    $Log->comment('Global Hybrid aggregate flag set')
      if ( $self->is_hybrid );

    $self->disable_space_check( $param->get($AGGREGATE_PARAM_FOR{DISABLE_SPACE_CHECK}) );
    $Log->comment('Disable space check flag set')
      if ( $self->disable_space_check );

    $self->_cache_object_names_from_global_param($AGGREGATE_PARAM_FOR{NAMES});

} ## end sub _process_global_params

sub _process_find_options {
    my $self = shift;
    my ($find_options) = @_;

    $find_options = { %{&DEFAULT_FIND_OPTIONS}, %{$find_options} };
    if ( $self->is_hybrid ) {
        $find_options->{filter}->{hybrid} = 'true';
    }
    if (   $self->total_volume_size
        && !$self->is_flexgroup
        && !$self->disable_space_check )
    {
        $find_options->{filter}->{availsize} ||= '>' . $self->total_volume_size;
    }

    $self->SUPER::_process_find_options($find_options);
} ## end sub _process_find_options

sub _process_create_options {
    my $self           = shift;
    my $create_options = shift;

    $create_options->{nacltask_enable_hybrid} = 1 if ( $self->is_hybrid );

    $self->SUPER::_process_create_options($create_options);
} ## end sub _process_create_options

sub _fetch_hybrid_aggr_counters {
    my $self = shift;
    my $hybrid_aggr_counters_of;

    for my $aggr ( $self->aggregates ) {
        my $aggr_name = $aggr->aggregate;

        for my $counter ( 'ssd_read_cached', 'ssd_write_cached' ) {
            my $cache_cnt = NACL::CS::Statistics->fetch(
                command_interface => $self->node,
                filter            => {
                    object   => 'aggregate',
                    instance => $aggr_name,
                    counter  => $counter,
                    raw      => 'true',
                }
            )->value;
            $hybrid_aggr_counters_of->{$aggr_name}->{$counter} = $cache_cnt;
        }
    }
    return $hybrid_aggr_counters_of;
} ## end sub _fetch_hybrid_aggr_counters

sub _find_or_create_storage_pool {
    my $self = shift;
    my ($storage_pool_find_options, $storage_pool_create_options) = @_;

    my @available_sps = NACL::STask::StoragePool->find(
        command_interface          => $self->node,
        allow_empty                => 1,
        filter              => {
            %{$storage_pool_find_options},
        },
    );

    foreach my $found_sp (@available_sps) {
        my $sp_has_units = NACL::CS::StoragePoolShowAvailableCapacity->fetch(
            command_interface   => $self->node,
            allow_empty         => 1,
            filter              => {
                'allocation-unit-count' => "!0",
                'storage-pool'          => $found_sp->storage_pool,
                node                    => $self->node,
            },
        );
        if ($sp_has_units) {
            return $found_sp;
        }
    }

    # No Storage pool was found, so create one
    my $created_sp = NACL::STask::StoragePool->create(
        command_interface          => $self->node,
        %{$storage_pool_create_options},
    );
    push ( @SP_Cleanup_Array, $created_sp->storage_pool );

    return $created_sp;
}

sub _add_disks_to_hya {
    my $self = shift;
    my ($aggregate, $add_disk_opts, $usable_sp) = @_;

    my $current_add_disk_opts = $add_disk_opts;

    if ($usable_sp){
        $current_add_disk_opts = {
            'storage-pool'     => $usable_sp->storage_pool,
            %{$current_add_disk_opts},
            'allocation-units' => '1',
        };
    } else {
        #Add defaults if not already specified
        my $version = $self->node->version_manager;
        my $diskcount = 5;
        if ($version->is_debug) {
            $diskcount = 3;
        }
        $current_add_disk_opts = {
            diskcount     => $diskcount,
            raidtype      => 'raid_dp',
            disktype      => 'SSD',
            raidgroup     => 'new',
            %{$current_add_disk_opts},
        };
    }
    $aggregate->add_disks( %{$current_add_disk_opts} );

} ## end _add_disks_to_hya

sub _find_or_create_aggr_till_vol_size_is_accommodated {
    my $self = shift;
    my ($create_opts, $find_opts) = @_;
    my $total_vol_size = convert_to_bytes_int($self->total_volume_size);

    my $total_aggr_size = 0;
    while ( $total_aggr_size <= $total_vol_size ) {
        $self->_set_find_filter_to_exclude_existing_aggrs($find_opts);
        $self->_process_options_and_find_or_create($create_opts, $find_opts);
        $total_aggr_size += $self->_fetch_aggr_availsize( $self->aggregates->[-1] );
    }
}

sub _set_find_filter_to_exclude_existing_aggrs {
    my $self = shift;
    my( $find_opts) = @_;
    my @exisiting_aggrs = $self->aggregates;
    if(scalar(@exisiting_aggrs)) {
        my $excl_str = join ',', map { '!' . $_->aggregate } @exisiting_aggrs;
        $find_opts->{filter}->{aggregate} = $excl_str;
    }

}

sub cleanup {
    my $self = shift;
    if (@SP_Cleanup_Array){
        $Log->step("Starting Storage Pool Cleanup");
        # for each SP, if all units are unallocated delete it
        foreach my $sp_name (@SP_Cleanup_Array){
            my $sp_obj = NACL::STask::StoragePool->find(
                command_interface  => $self->node,
                filter             => {
                    'storage-pool' => $sp_name,
                },
            );
            my $sp_state = $sp_obj->state;
            if ( $sp_state->pool_usable_size == $sp_state->pool_total_size ) {
                $sp_obj->purge;
            } else {
                $Log->comment("Not deleting $sp_name because it has used allocation-units");
            }
        }
        NACL::STask::StorageDisk->zerospares(
            command_interface  => $self->node,
        );
        $Log->step("Finished Storage Pool Cleanup");
    }
}

sub _fetch_aggr_availsize {
    my $self = shift;
    my ($aggr_stask_obj) = @_;
    return convert_to_bytes_int(
        $aggr_stask_obj->get_one_state_attribute('availsize') );
}

1;
__END__

=pod

=head1 NAME

NACL::MTask::Setup::Object::Aggregate

=head1 DESCRIPTION

This library finds or creates one or more aggregates. Parameters to the library
can be overridden by global parameters. The list of global parameters for this
library are documented here
L<NACL::MTask::Setup::GlobalParams#Aggregate|lib-NACL-MTask-Setup-GlobalParams-pm/Aggregate>

=head1 ATTRIBUTES

=head2 node

STask node STask object used to find aggregates

=head2 aggregates

STask Aggregate objects found or created

=head2 is_hybrid

Flag to indicate if the aggregates are hybrid

=head1 METHODS

=head2 new

    my $aggr_obj = NACL::MTask::Setup::Object::Aggregate->new(
        node => $nodes,
        count => 2,
        create_options => $create_options,
        find_options => $find_options,
    );

(Instance method)

=over

=item Options

=over

=item C<< node => $node >>

(Required)
STask node object to be used as the command interface

=item C<< count => 2 >>

(Optional)
The number of Aggregates needed

=item C<< find_options => $aggregate_find_options >>

(Optional, Hashref)
Parameters to find the aggregates. These are parameters accepted by STask

See L<NACL::C::Aggregate::find|lib-NACL-C-Aggregate-pm/find>
Ignore the command_interface as node is used as the command interface

=item C<< create_options => $aggregate_create_options >>

(Optional, Hashref)
Parameters to create the aggregates. These are parameters accepted by STask

See L<NACL::STask::Aggregate::create|lib-NACL-STask-Aggregate-pm/create>
Ignore the command_interface as node is used as the command interface

=back

=back

=head2 cleanup

$setup->cleanup;

(Instance method)

=cut
