# $Id: $
#
## Copyright (c) 2014 NetApp, Inc., All Rights Reserved
##   Any use, modification, or distribution is prohibited
##   without prior written consent from NetApp, Inc.
##

#
## @summary WAFL Ontap Setup MTask Node Module
## @author dl-awg-warp-svl
## @change
##
## @status shared
## @pod here

package NACL::MTask::Setup::Object::Node;

use strict;
use warnings;
use Carp;
use parent qw(NACL::MTask::MTask);
use NACL::MTask::Setup::GlobalParams qw(:Node);
use NACL::MTask::Setup::ObjectFactory;

use Params::Validate qw(validate_with BOOLEAN HASHREF OBJECT SCALAR );

use NACL::STask::Node;
use NACL::STask::Cluster;
use NACL::STask::SystemLicense;
use NACL::CS::Cluster;

use NATE::Exceptions::Argument;
use NATE::BaseException;

use NACL::GeneralUtils qw(random_name_generator);
use List::MoreUtils qw(pairwise);
use NATE::ParamSet qw(param_global);
use NATE::Log qw(log_global);

use Class::MethodMaker [
    new    => [qw/ -init hash new /],
    array  => [{-type => 'NACL::STask::Node'}, 'nodes',],
    scalar => [
        qw/
            template
            initial_blog_counters/
    ],
];

sub OBJECT_TYPE {return 'Node'}

sub ONE_NODE {return '1node'}
sub is_1node_setup {return shift->template eq ONE_NODE}

sub TWO_NODES {return '2nodes'}
sub is_2nodes_setup {return shift->template eq TWO_NODES}

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub init {
    $Log->enter() if $may_enter;

    my $self = shift;

    my %opts = $self->_common_validate_with(
        params          => \@_,
        additional_spec => {
            template => {
                type    => SCALAR,
                default => ONE_NODE,
            },
            is_debug => {type => BOOLEAN, default => 0},
        },
        ignore_primary_keys  => 1,
        no_command_interface => 1,
    );

    $self->template($opts{template});
    $self->nodes($self->_find_nodes($opts{template}));

    $self->_validate_debug_build if ($opts{is_debug});

    $Log->exit() if $may_exit;
    return $self;
} ## end sub init

    ## TODOadd
    # -aggr show
    # -df -A
    # -aggr show-space
    # -vol show
    # -df -autosize
    # -vol show-space
    # -vol show-footprint

    ## TODO
    # add Hybrid aggregate check
    # add Y1/Y2 check

sub group_nodes_by_unique_clusters {
    my $self = shift;
    return NACL::STask::Cluster->group_nodes_by_unique_clusters(
        nodes => [$self->nodes]);
}

sub check_cluster_health {
    my $self     = shift;
    my $nodes_of = $self->group_nodes_by_unique_clusters;

    $Log->step('Checking cluster health');
    ## TODO validate all nodes are healthy
    foreach my $nodes (values(%{$nodes_of})) {
        NACL::CS::Cluster->fetch(command_interface => $nodes->[0]);
    }
} ## end sub check_cluster_health

sub cache_blog_counters {
    my $self = shift;
    $self->initial_blog_counters($self->_fetch_blog_counters);
}

sub compare_blog_counters {
    my $self                  = shift;
    my %initial_blog_counters = %{$self->initial_blog_counters};
    if ($self->initial_blog_counters) {
        #############################################
        $Log->step('Caching final Blog counters');
        #############################################
        my %final_blog_counters = %{$self->_fetch_blog_counters};

        #############################################
        $Log->step('Comparing Blog counters');
        #############################################
        foreach my $vol_setup (keys(%initial_blog_counters)) {
            $Log->comment("Comparing blog counter for vol: $vol_setup");
            $Log->comment('initial blog value:'
                    . $initial_blog_counters{$vol_setup}->value);
            $Log->comment('final blog value:'
                    . $final_blog_counters{$vol_setup}->value);
            if ($final_blog_counters{$vol_setup}->value
                < $initial_blog_counters{$vol_setup}->value) {
                $Log->error('Final blog counter value lesser than initial');
            }
            if ($final_blog_counters{$vol_setup}->value
                == $initial_blog_counters{$vol_setup}->value) {
                $Log->warn('Final blog counter value same as initial. Blog not exercised');
            }
        }
    }
} ## end sub compare_blog_counters

sub add_license_for_features {
    my $self     = shift;
    my @features = @_;

    foreach my $node ($self->nodes) {
        foreach my $feature (@features) {
            NACL::STask::SystemLicense->add(
                command_interface => $node,
                feature           => $feature,
            );
        }
    }
} ## end sub add_license_for_features

sub _find_nodes {
    my $self = shift;
    my ($template) = @_;
    my @nodes;

    ##TODO Randomize nodes based on global param
    $Log->step("Template selected: $template");
    if ($template eq ONE_NODE) {
        my $node = NACL::STask::Node->find();
        @nodes = ($node);
    } elsif ($template eq TWO_NODES) {
        @nodes = NACL::STask::Node->find();
        $Log->comment('Validating node count');
        NACL::MTask::Setup::ObjectFactory::validate_object_count( 
            scalar(@nodes), 2, 'nodes');
        @nodes = @nodes[0 .. 1];
    } else {
        NATE::Exceptions::Argument->throw(
            'Unrecognised template: ' . $template);
    }
    return @nodes;
} ## end sub _find_nodes

sub _validate_debug_build {
    my $self = shift;

    foreach my $node ($self->nodes) {
        NATE::BaseException->throw('Debug build requested, but not deployed on: '
            . $node->node)
            if (!$node->version_manager->is_debug);
    }
} ## end sub _validate_debug_build

sub _fetch_blog_counters {
    my $self = shift;

    my @volumes  = $self->volumes;
    my @counters = NACL::CS::Statistics->fetch(
        command_interface => $self->command_interface,
        filter            => {
            object   => 'volume',
            instance => join(",", @volumes),
            counter  => 'batched_free_log_inserted_blocks',
            raw      => 'true'
        },
    );

    my %blog_counter_of
        = pairwise { our $a->volume => our $b->value } (@volumes, @counters);

    \%blog_counter_of;
} ## end sub _fetch_blog_counters

1;

__END__

=pod

=head1 NAME

NACL::MTask::Setup::Object::Node

=head1 DESCRIPTION

This library Finds nodes based on template provided

Current supported templates
See L<NACL::MTask::Setup::Constants#Node Templates|lib-NACL-MTask-Setup-Constants-pm/Node Templates>

=head1 ATTRIBUTES

=head2 nodes

STask Node objects found

=head2 template

Node template used for finding nodes

=head1 METHODS

=head2 new

    my $node_obj = NACL::MTask::Setup::Object::Node->new(
        template =>1node 
    );
(Instance method)

=over

=item Options

=over

=item C<< template => 1node >>

(Optional, Scalar)
The template based on which nodes are to be found.

Valid values are 1node and 2nodes

=back

=back

=head2 pre_setup

Performs pre-setup operations on node objects created. The pre_setup operations
are controlled through global params.
See L<Node Pre setup params|lib-NACL-MTask-Setup-GlobalParams-pm/Node Pre setup params>

=head2 post_setup

Performs post-setup operations on node objects created. The post_setup operations
are controlled through global params.
See L<Node Post setup params|lib-NACL-MTask-Setup-GlobalParams-pm/Node Post setup params>

=cut
