# $Id: $
#
## Copyright (c) 2014 NetApp, Inc., All Rights Reserved
##   Any use, modification, or distribution is prohibited
##   without prior written consent from NetApp, Inc.
##

#
## @summary WAFL Ontap Setup MTask Module
## @author dl-awg-warp-svl
## @change
##
## @status shared
## @pod here

package NACL::MTask::Setup::Object::Vserver;

use strict;
use warnings;

use parent qw(NACL::MTask::Setup::ObjectFactory);
use NACL::MTask::Setup::GlobalParams qw(:Vserver);

use Params::Validate qw(:types);

use NACL::STask::Vserver;

use NATE::ParamSet qw(param_global);
use NATE::Log qw(log_global);

use Class::MethodMaker [
    array  => [{-type => 'NACL::STask::Vserver'}, 'vservers'],
    scalar => [{-type => 'NACL::STask::Node'},    'node',],
    scalar => [{ -type => 'NACL::Cleanup' }, 'cleanup_manager'],
    scalar => [qw/count/],
];

sub OBJECT_TYPE { return 'Vserver'}

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub new {
    $Log->enter() if $may_enter;

    my $pkg = shift;

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            node  => {type => SCALAR},
            cleanup_manager =>  { isa => 'NACL::Cleanup', required => 1},
            count => {
                type    => SCALAR,
                regex   => qr/^\d+$/,
                default => 1,
            },
            create_options => {type => HASHREF, default => {}},
            find_options =>
                {type => HASHREF, default => {canned_filters => [qw(data_vservers)]}},
        },
        ignore_primary_keys  => 1,
        no_command_interface => 1,
    );

    return if ($opts{count} == 0);

    my $self = $pkg->SUPER::new(type => OBJECT_TYPE);

    $self->node(delete $opts{node});
    $self->count(delete $opts{count});
    $self->cleanup_manager(delete $opts{cleanup_manager});

    $self->_process_global_params;

    $self->_process_options_and_find_or_create($opts{create_options},
        $opts{find_options});

    $Log->exit;
    return $self;
} ## end sub new

## private subroutines
sub _process_global_params {
    my $self = shift;

    $self->_cache_object_names_from_global_param($VSERVER_PARAM_FOR{NAMES});
}

1;

__END__

=pod

=head1 NAME

NACL::MTask::Setup::Object::Vserver

=head1 DESCRIPTION

This library finds or creates one or more vservers. Parameters to the library
can be overridden by global parameters. The list of global parameters for this
library are documented here
L<NACL::MTask::Setup::GlobalParams#Vserver|lib-NACL-MTask-Setup-GlobalParams-pm/Vserver>

=head1 ATTRIBUTES

=head2 node

STask Node object used for creating vserver

=head2 vservers

STask Vserver objects found or created

=head1 METHODS

=head2 new

    my $vserver_obj = NACL::MTask::Setup::Object::Vserver->new(
        node => $nodes,
        count => 2,
        create_options => $create_options,
        find_options => $find_options,
    );
(Instance method)

=over

=item Options

=over

=item C<< node => $node >>

(Required)
STask node object to be used as the command interface

=item C<< count => 2 >>

(Optional)
The number of Vservers needed

=item C<< find_options => $vserver_create_options >>

(Optional, Hashref)
Parameters to find the vserver. These are parameters accepted by STask

See L<NACL::C::Vserver::find()|lib-NACL-C-Vserver-pm/find>
Ignore the command_interface as node is used as the command interface

=item C<< create_options => $vserver_create_options >>

(Optional, Hashref)
Parameters to create the vserver. These are parameters accepted by STask

See L<NACL::STask::Vserver::create()|lib-NACL-STask-Vserver-pm/create>
Ignore the command_interface as node is used as the command interface

=back

=back

=cut
