# $Id: $
#
## Copyright (c) 2014 NetApp, Inc., All Rights Reserved
##   Any use, modification, or distribution is prohibited
##   without prior written consent from NetApp, Inc.
##

#
## @summary WAFL Ontap Setup MTask Module
## @author dl-awg-warp-svl
## @change
##
## @status shared
## @pod here

package NACL::MTask::Setup::ObjectFactory;

use strict;
use warnings;
use Carp;
use Params::Validate qw(validate_with SCALAR);

use NACL::GeneralUtils qw(random_name_generator);
use NATE::ParamSet qw(param_global);
use NATE::Log qw(log_global);
use NATE::BaseException;

use parent qw(NACL::MTask::MTask);

use Class::MethodMaker [
    scalar => [
        qw/
          type
          type_ucfirst
          type_lc
          type_lc_plural
          type_lc_plural_with_push
          type_for_log_msg
          stask_pkg_name
          find_options
          create_options
          name_prefix
          _skip_find
          names_param
          count_param
          size_param
          /
    ],
    array => [
        qw/
          names_from_global_param
          post_setup_methods
          /
    ],
];

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub new {
    $Log->enter;

    my $pkg = shift;

    NATE::BaseException->throw('This class should be instantiated by a child class')
      if ( $pkg eq __PACKAGE__ );

    ## TODO check for list of methods a child class should have using $class->can

    my %opts = validate_with(
        params => \@_,
        spec   => { type => { type => SCALAR }, },
    );

    my $object_type         = $opts{type};
    my $object_type_ucfirst = ucfirst($object_type);
    my $stask_pkg_name      = 'NACL::STask::' . $object_type_ucfirst;

    eval "use $stask_pkg_name";
    NATE::BaseException->throw($@) if ($@);

    my $self           = \%opts;
    my $object_type_lc = lc($object_type);
    my $object_type_uc = uc($object_type);
    $self->{type_ucfirst}             = $object_type_ucfirst;
    $self->{type_lc}                  = $object_type_lc;
    $self->{type_lc_plural}           = $object_type_lc . 's';
    $self->{type_lc_plural_with_push} = $object_type_lc . 's_push';
    $self->{type_for_log_msg}         = $object_type_ucfirst . '(s)';
    $self->{stask_pkg_name}           = $stask_pkg_name;
    $self->{name_prefix}              = 'MTASK_' . $object_type_uc;

    bless $self, $pkg;

    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

sub _cache_object_names_from_global_param {
    my $self  = shift;
    my $param = shift;

    my $object_names_str = param_global->get($param);
    if ($object_names_str) {
        my @object_names = split /,|;|:/, $object_names_str;
        $self->names_from_global_param(@object_names);
        $Log->comment( 'Global ' . $self->type_ucfirst . ' name param set' );
    }
} ## end sub _cache_object_names_from_global_param

sub _process_options_and_find_or_create {
    my $self = shift;
    my ( $create_options, $find_options ) = @_;

    my $object_type = $self->type_for_log_msg;
    $Log->step("Setting up $object_type");

    my $num_of_objs_requested = $self->count;

    my $aggrs_found_ref = my $aggrs_created_ref = [];
    if ( !$self->_skip_find ) {
        $aggrs_found_ref = $self->_process_find_options_and_find(
            $find_options, $num_of_objs_requested);
    }

    my $num_of_objs_found = scalar( @{$aggrs_found_ref} );
    if ( $num_of_objs_found < $num_of_objs_requested ) {
        $Log->comment( "Number of $object_type: found: $num_of_objs_found"
              . " Requested: $num_of_objs_requested" );
        $aggrs_created_ref = $self->_process_create_options_and_create(
            $create_options, $num_of_objs_requested - $num_of_objs_found);
    }

    $self->_push_and_log_objects($aggrs_found_ref, $aggrs_created_ref);
} ## end sub _process_options_and_find_or_create

sub _process_find_options_and_find {
    my $self = shift;
    my ( $find_options, $num_of_objs_requested ) = @_;
    return $self->_find(
        find_options => $self->_process_find_options($find_options),
        count        => $num_of_objs_requested,
    );

}

sub _process_find_options {
    my $self = shift;
    my ($find_options) = @_;

    my @names = $self->names_from_global_param;
    $find_options = { filter => { $self->type_lc => join '|', @names } }
      if (@names);

    return {
        command_interface => $self->node,
        %{$find_options},
        allow_empty => 1,
    };
} ## end sub _process_find_options

sub _find {
    my $self                  = shift;
    my %opts                  = @_;
    my $num_of_objs_requested = $opts{count} || 1;

    my @objects = $self->stask_pkg_name->find( %{ $opts{find_options} } ) || ();

    if ( scalar(@objects) >= $num_of_objs_requested ) {
        @objects = @objects[ 0 .. $num_of_objs_requested - 1 ];
    }

    return \@objects;
} ## end sub _find

sub _process_create_options_and_create {
    my $self = shift;
    my ( $create_options, $num_of_objs_requested ) = @_;

    return $self->_create(
        count          => $num_of_objs_requested,
        set_rng_args   => 1,
        create_options => $self->_process_create_options($create_options),
    );
}

sub _process_create_options {
    my $self = shift;
    my ($create_options) = @_;

    return {
        command_interface   => $self->node,
        nacltask_cleanup_manager => $self->cleanup_manager,
        %{$create_options},
    };
} ## end sub _process_create_options

sub _create {
    my $self                  = shift;
    my %opts                  = @_;
    my $num_of_objs_requested = $opts{count} || 1;
    my %rng_args =
        ( $opts{set_rng_args} )
      ? ( nacltask_rng_args => { prefix => $self->name_prefix } )
      : ();

    $Log->comment(
        "Creating $num_of_objs_requested " . $self->type_ucfirst . '(s)' );

    my @objects = map {
        $self->stask_pkg_name->create( %{ $opts{create_options} }, %rng_args )
    } ( 1 .. $num_of_objs_requested );

    return \@objects;
} ## end sub _create

sub _push_and_log_objects {
    my $self = shift;
    my ($aggrs_found_ref, $aggrs_created_ref) = @_;

    my $object_type = $self->type_for_log_msg;
    my $object_list_push_method = $self->type_lc_plural_with_push;
    my $object_list_method      = $self->type_lc_plural;
    my $object_name_method      = $self->type_lc;

    my @objects = (@{$aggrs_found_ref}, @{$aggrs_created_ref});
    $self->$object_list_push_method(@objects);
    $Log->step(
        "$object_type to be used:\n"
            . join( "\n", map { $_->$object_name_method } @objects )
    );
}

sub validate_object_count {
    my ( $num_of_obj_act, $num_of_obj_exp, $object ) = @_;

    if ( $num_of_obj_act < $num_of_obj_exp ) {
        NATE::BaseException->throw( "Num of $object expected: $num_of_obj_exp"
              . " But found: $num_of_obj_act" );
    }
}

1;

__END__

=pod

=head1 NAME

NACL::MTask::Setup::ObjectFactory

=head1 DESCRIPTION

=cut
