#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MTask module to manage/manipulate sets of snapmirror relationships
## @author ng-ptefit-automation-core <ng-ptefit-automation-core@netapp.com>
## @status shared
## @pod here

package NACL::MTask::Snapmirror;

use strict;
use warnings;
use base qw(NACL::MTask::MTask);

use Scalar::Util qw(blessed);
use Params::Validate qw(
    validate validate_with validation_options
    SCALAR ARRAYREF HASHREF OBJECT UNDEF BOOLEAN
);

use NATE::ServiceAPI::Event qw(event_global);
my $Eventsvcapi = event_global();

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use Tharn qw(sleep);
use NACL::ComponentUtils qw(_verify_invocation);
use NACL::STask::Snapmirror;
use NACL::STask::Vserver;
use NACL::CS::ClusterIdentity;
use NACL::CS::Vserver;
use NACL::CS::SnapmirrorListDestinations;
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument;
use Data::Dumper;



=head1 NAME

NACL::MTask::Snapmirror - Manage group of snapmirror relationships

=head1 SYNOPSIS

    use NACL::MTask::Snapmirror;
   
    # Create a new Snapmirror object with the set of snapmirror information.
    # This snapmirror information will be a list of hashes of snapmirror
    # information. The hashes could be snapmirror objects or just hashes
    # of the required snapmirror object information.
    my $set = NACL::MTask::Snapmirror->new(
        snapmirrors => \@snapmirror_objects,
        command_interfaces => \@nodes
    );

    # $set now knows about the defined snapmirror_objects.
    # create all the snapmirrors.
    $set->create()

    # Initialize all the snapmirrors at once:
    $set->initialize();

    # Update all the snapmirrors at once:
    $set->update();

=head1 DESCRIPTION

The B<NACL::MTask::Snapmirror> class is designed to facilitate the management of
groups of snapmirror relationships as a single entity.

A B<NACL::MTask::Snapmirror> object is created from a list of snapmirror
objects or information hashes.

The purpose of having a B<NACL::MTask::Snapmirror> object is to perform the
various actions associated with snapmirror relationships (initialize, update,
etc.) on the set as a whole, rather than having to call the methods on the
individual B<NACL::STask::Snapmirror> objects.

=head1 METHODS

=head2 new

    # Contruct from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->new(
        snapmirrors => \@snapmirrors
    );

    # Create from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->new(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Create from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->new(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

(May only be called as a class method.)

B<new> creates a new object of this class and returns a reference to it. The
only required argument is the C<snapmirrors> parameter, as if the contents
are NACL::STask::Snapmirror objects then they contain command interfaces. 
Based on the parameters given, the constructor may create some number 
of B<NACL::STask::Snapmirror> objects.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to create an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either create
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=back

=item Exceptions

Any exceptions that can be thrown by the B<new> methods of B<NACL::STask::Snapmirror> 
can appear here, as those methods are called as part of the set-up of this constructor.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub new {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors               => { type => ARRAYREF, optional => 0},
            command_interfaces        => { type => ARRAYREF|HASHREF, optional => 1},
        },
        allow_extra => 1,
    );

    # Process the supplied snapmirrors and command_interfaces.
    $pkg->_process_snapmirrors_and_command_interfaces(%opts);

    delete $opts{command_interfaces};

    $Log->exit() if $may_exit;
    # Initialize the object here based on the passed in information.
    # This should just be an array of snapmirrors.
    return bless (\%opts , $pkg);


}

=head2 create

    # Create with objects stored in MTask:
    $SMs->create();

    # Create from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->create(
        snapmirrors => \@snapmirrors
    );

    # Create from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->create(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Create from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->create(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request create of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to create an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either create
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=back

=item Exceptions

Any exceptions that can be thrown by the B<new>, B<create> methods of B<NACL::STask::Snapmirror> 
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors                => { type => ARRAYREF, optional => 1},
            command_interfaces         => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_continue_on_error => { type => SCALAR, default => 0},
            nacltask_poll_interval     => { type => SCALAR, default => 10},
            'method-timeout'           => { type => SCALAR, default => 1200},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $error_count = 0;
    my $continue_on_error = $opts{nacltask_continue_on_error};
    # If this is an instance call then we will perform the create
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform break on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::CreateGroupStart",
                          message => \%message);
    foreach my $snapmirror (@snapmirrors) {
        $snapmirror->{nacltask_state} = 'unknown';
        %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
        $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::CreateStart",
                      message => \%message);
        try {
            my %create_args = $pkg_or_obj->_get_snapmirror_primary_keys(%{$snapmirror});
            $create_args{source_ci} = $snapmirror->{source_ci} if defined ($snapmirror->{source_ci});
            $create_args{'identity-preserve'} = $snapmirror->{'identity-preserve'} if defined ('identity-preserve');
            my $command_interface = $snapmirror->{command_interface};
            NACL::STask::Snapmirror->create(command_interface => $command_interface,
                                            %create_args,
                                            nacltask_if_exists => 'reuse',
                                            do_initialize => '0');
            $snapmirror->{nacltask_state} = 'finished';
            # The Snapmirror has finished publish message.
            %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::CreateFinish",
                          message => \%message);
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::CreateError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::CreateError",
                              message => \%message);

                # Publish the group finished message.
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::CreateGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };
    }

    # Publish the group finished message.
    %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::CreateGroupFinish",
                          message => \%message);

    # If any errored then throw exception.
    if ($error_count) {
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish create. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    # Set the snapmirrors on the object to the break list of snapmirrors so that the wait_for
    # call will have knowledge of errors, etc.
    $snapmirror_mtask_obj->{snapmirrors} = \@snapmirrors;

    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 initialize

    # Initialize with objects stored in MTask:
    $SMs->initialize();

    # Initialize from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->initialize(
        snapmirrors => \@snapmirrors
    );

    # Initialize from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->initialize(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Initialize from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->initialize(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request initialize of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to initialize an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either initialize
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=back

=item Exceptions

Any exceptions that can be thrown by the B<new>, B<initialize> methods of B<NACL::STask::Snapmirror> 
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub initialize {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors               => { type => ARRAYREF, optional => 1},
            command_interfaces        => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_continue_on_error => { type => SCALAR, default => 0},
            nacltask_poll_interval     => { type => SCALAR, default => 10},
            'method-timeout'           => { type => SCALAR, default => 1200},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $error_count = 0;
    my $continue_on_error = $opts{nacltask_continue_on_error};
    # If this is an instance call then we will perform the initialize
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform initialize on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::InitializeGroupStart",
                          message => \%message);
    foreach my $snapmirror (@snapmirrors) {
        $snapmirror->{nacltask_state} = 'unknown';
        %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
        $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::InitializeStart",
                      message => \%message);
        try {
            $snapmirror->initialize(nacltask_wait => 0);
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::InitializeError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::InitializeError",
                              message => \%message);

                # Publish the group finished message.
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::InitializeGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };
    }

    # If all have errored then throw exception because we do not need to wait.
    if ($error_count == @snapmirrors) {
        # Publish the group finished message.
        %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
        $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::InitializeGroupFinish",
                              message => \%message);
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish initialize. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    # Set the snapmirrors on the object to the initialize list of snapmirrors so that the wait_for
    # call will have knowledge of errors, etc.
    $snapmirror_mtask_obj->{snapmirrors} = \@snapmirrors;

    # Adding snooze of 5 secs. This is added as fix for Burt624613 where snapmirror
    # show takes few seconds to catch change in state of snapmirror relationship.
    sleep(5);

    # wait for completion of all the initializes.
    $snapmirror_mtask_obj->wait_for_initialize(
                                nacltask_continue_on_error => $opts{nacltask_continue_on_error},
                                nacltask_error_count       => $error_count,
                                nacltask_poll_interval     => $opts{nacltask_poll_interval},
                                'method-timeout'           => $opts{'method-timeout'},);
    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 update

    # Update with objects stored in MTask:
    $SMs->update();

    # Update from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->update(
        snapmirrors => \@snapmirrors
    );

    # Update from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->update(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Update from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->update(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request update of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to update an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either update
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=back

=item Exceptions

Any exceptions that can be thrown by the B<new>, B<update> methods of B<NACL::STask::Snapmirror> 
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub update {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors               => { type => ARRAYREF, optional => 1},
            command_interfaces        => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_continue_on_error => { type => SCALAR, default => 0},
            nacltask_poll_interval     => { type => SCALAR, default => 10},
            'method-timeout'           => { type => SCALAR, default => 1200},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $error_count = 0;
    my $continue_on_error = $opts{nacltask_continue_on_error};
    # If this is an instance call then we will perform the update
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform update on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::UpdateGroupStart",
                          message => \%message);
    foreach my $snapmirror (@snapmirrors) {
        $snapmirror->{nacltask_state} = 'unknown';
        %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
        $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::UpdateStart",
                      message => \%message);
        try {
            $snapmirror->update(nacltask_wait => 0);
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::UpdateError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::UpdateError",
                              message => \%message);

                # Publish the group finished message.
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::UpdateGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };
    }

    # If all have errored then throw exception because we do not need to wait.
    if ($error_count == @snapmirrors) {
        # Publish the group finished message.
        %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
        $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::UpdateGroupFinish",
                              message => \%message);
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish update. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    # Set the snapmirrors on the object to the update list of snapmirrors so that the wait_for
    # call will have knowledge of errors, etc.
    $snapmirror_mtask_obj->{snapmirrors} = \@snapmirrors;

    # Adding snooze of 5 secs. This is added as fix for Burt624613 where snapmirror
    # show takes few seconds to catch change in state of snapmirror relationship.
    sleep(5);

    # wait for completion of all the updates.
    $snapmirror_mtask_obj->wait_for_update(
                                nacltask_continue_on_error => $opts{nacltask_continue_on_error},
                                nacltask_error_count       => $error_count,
                                nacltask_poll_interval     => $opts{nacltask_poll_interval},
                                'method-timeout'           => $opts{'method-timeout'},);
    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 resync

    # Resync with objects stored in MTask:
    $SMs->resync();

    # Resync from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->resync(
        snapmirrors => \@snapmirrors
    );

    # Resync from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->resync(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Resync from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->resync(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request resync of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to resync an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either resync
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=back

=item Exceptions

Any exceptions that can be thrown by the B<new>, B<resync> methods of B<NACL::STask::Snapmirror> 
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub resync {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors               => { type => ARRAYREF, optional => 1},
            command_interfaces        => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_continue_on_error => { type => SCALAR, default => 0},
            nacltask_poll_interval     => { type => SCALAR, default => 10},
            'method-timeout'           => { type => SCALAR, default => 1200},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $error_count = 0;
    my $continue_on_error = $opts{nacltask_continue_on_error};
    # If this is an instance call then we will perform the resync
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform Resync on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ResyncGroupStart",
                          message => \%message);
    foreach my $snapmirror (@snapmirrors) {
        $snapmirror->{nacltask_state} = 'unknown';
        %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
        $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ResyncStart",
                      message => \%message);
        try {
            $snapmirror->resync(nacltask_wait => 0);
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ResyncError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ResyncError",
                              message => \%message);

                # Publish the group finished message.
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ResyncGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };
    }

    # If all have errored then throw exception because we do not need to wait.
    if ($error_count == @snapmirrors) {
        # Publish the group finished message.
        %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
        $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ResyncGroupFinish",
                              message => \%message);
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish resync. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    # Set the snapmirrors on the object to the Resync list of snapmirrors so that the wait_for
    # call will have knowledge of errors, etc.
    $snapmirror_mtask_obj->{snapmirrors} = \@snapmirrors;

    # Adding snooze of 5 secs. This is added as fix for Burt624613 where snapmirror
    # show takes few seconds to catch change in state of snapmirror relationship.
    sleep(5);

    # wait for completion of all the Resyncs.
    $snapmirror_mtask_obj->wait_for_resync(
                                nacltask_continue_on_error => $opts{nacltask_continue_on_error},
                                nacltask_error_count       => $error_count,
                                nacltask_poll_interval     => $opts{nacltask_poll_interval},
                                'method-timeout'           => $opts{'method-timeout'},);
    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 break

    # Break with objects stored in MTask:
    $SMs->break();

    # Break from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->break(
        snapmirrors => \@snapmirrors
    );

    # Break from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->break(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Break from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->break(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request break of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to break an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either break
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=back

=item Exceptions

Any exceptions that can be thrown by the B<new>, B<break> methods of B<NACL::STask::Snapmirror> 
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub break {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors               => { type => ARRAYREF, optional => 1},
            command_interfaces        => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_continue_on_error => { type => SCALAR, default => 0},
            nacltask_poll_interval     => { type => SCALAR, default => 10},
            'method-timeout'           => { type => SCALAR, default => 1200},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $error_count = 0;
    my $continue_on_error = $opts{nacltask_continue_on_error};
    # If this is an instance call then we will perform the break
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform break on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::BreakGroupStart",
                          message => \%message);
    foreach my $snapmirror (@snapmirrors) {
        $snapmirror->{nacltask_state} = 'unknown';
        %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
        $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::BreakStart",
                      message => \%message);
        try {
            $snapmirror->break(nacltask_wait => 0);
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::BreakError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::BreakError",
                              message => \%message);

                # Publish the group finished message.
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::BreakGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };
    }

    # If all have errored then throw exception because we do not need to wait.
    if ($error_count == @snapmirrors) {
        # Publish the group finished message.
        %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
        $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::BreakGroupFinish",
                              message => \%message);
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish break. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    # Set the snapmirrors on the object to the break list of snapmirrors so that the wait_for
    # call will have knowledge of errors, etc.
    $snapmirror_mtask_obj->{snapmirrors} = \@snapmirrors;

    # Adding snooze of 5 secs. This is added as fix for Burt624613 where snapmirror
    # show takes few seconds to catch change in state of snapmirror relationship.
    sleep(5);

    # wait for completion of all the breaks.
    $snapmirror_mtask_obj->wait_for_break(
                                nacltask_continue_on_error => $opts{nacltask_continue_on_error},
                                nacltask_error_count       => $error_count,
                                nacltask_poll_interval     => $opts{nacltask_poll_interval},
                                'method-timeout'           => $opts{'method-timeout'},);
    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 release

    # Release with objects stored in MTask:
    $SMs->release();

    # Release from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->release(
        snapmirrors => \@snapmirrors
    );

    # Release from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->release(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Release from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->release(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request release of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to release an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either release
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=back

=item Exceptions

Any exceptions that can be thrown by the B<new>, B<release> methods of B<NACL::STask::Snapmirror> 
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub release {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors               => { type => ARRAYREF, optional => 1},
            command_interfaces        => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_continue_on_error => { type => SCALAR, default => 0},
            nacltask_poll_interval     => { type => SCALAR, default => 10},
            'method-timeout'           => { type => SCALAR, default => 1200},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $error_count = 0;
    my $continue_on_error = $opts{nacltask_continue_on_error};
    # If this is an instance call then we will perform the release
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform release on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ReleaseGroupStart",
                          message => \%message);
    foreach my $snapmirror (@snapmirrors) {
        $snapmirror->{nacltask_state} = 'unknown';
        %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
        $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ReleaseStart",
                      message => \%message);
        try {
            $snapmirror->release(nacltask_wait => 0);
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ReleaseError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ReleaseError",
                              message => \%message);

                # Publish the group finished message.
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ReleaseGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };
    }

    # If all have errored then throw exception because we do not need to wait.
    if ($error_count == @snapmirrors) {
        # Publish the group finished message.
        %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
        $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ReleaseGroupFinish",
                              message => \%message);
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish release. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    # Set the snapmirrors on the object to the release list of snapmirrors so that the wait_for
    # call will have knowledge of errors, etc.
    $snapmirror_mtask_obj->{snapmirrors} = \@snapmirrors;

    # Adding snooze of 5 secs. This is added as fix for Burt624613 where snapmirror
    # show takes few seconds to catch change in state of snapmirror relationship.
    sleep(5);

    # wait for completion of all the releases.
    $snapmirror_mtask_obj->wait_for_release(
                                nacltask_continue_on_error => $opts{nacltask_continue_on_error},
                                nacltask_error_count       => $error_count,
                                nacltask_poll_interval     => $opts{nacltask_poll_interval},
                                'method-timeout'           => $opts{'method-timeout'},);
    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 start_destination_vserver

    # Start destination vserver with objects stored in MTask:
    $SMs->start_destination_vserver();

    # Start destination vserver from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->start_destination_vserver(
        snapmirrors => \@snapmirrors
    );

    # Start destination vserver from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->start_destination_vserver(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Start destination vserver from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->start_destination_vserver(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request start_destination_vserver of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to start_destination_vserver an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either start_destination_vserver
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=back

=item Exceptions

Any exceptions that can be thrown by the B<new> method of B<NACL::STask::Snapmirror>
and the B<start> method of B<NACL::STask::Vserver>
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub start_destination_vserver {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors               => { type => ARRAYREF, optional => 1},
            command_interfaces        => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_continue_on_error => { type => SCALAR, default => 0},
            nacltask_poll_interval     => { type => SCALAR, default => 10},
            'method-timeout'           => { type => SCALAR, default => 1200},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $error_count = 0;
    my $continue_on_error = $opts{nacltask_continue_on_error};
    # If this is an instance call then we will perform the start destination vserver
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform start_destination_vserver on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Start_Destination_VserverGroupStart",
                          message => \%message);
    foreach my $snapmirror (@snapmirrors) {
        $snapmirror->{nacltask_state} = 'unknown';
        %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
        $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Start_Destination_VserverStart",
                      message => \%message);
        try {
            # Start the destination vserver
            my %args = $pkg_or_obj->_get_snapmirror_primary_keys(%{$snapmirror});
            my $command_interface = $snapmirror->{command_interface};
            NACL::STask::Vserver->start(command_interface => $command_interface,
                                   vserver => $args{'destination-vserver'},
                                   nacltask_if_started => 'pass',
                                   nacltask_wait => 0,
                                   foreground => 'false',
                                   );
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Start_Destination_VserverError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Start_Destination_VserverError",
                              message => \%message);

                # Publish the group finished message.
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Start_Destination_VserverGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };
    }

    # If all have errored then throw exception because we do not need to wait.
    if ($error_count == @snapmirrors) {
        # Publish the group finished message.
        %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
        $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Start_Destination_VserverGroupFinish",
                              message => \%message);
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish start destination vserver. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    # Set the snapmirrors on the object to the start_destination_vserver list of snapmirrors so that the wait_for
    # call will have knowledge of errors, etc.
    $snapmirror_mtask_obj->{snapmirrors} = \@snapmirrors;

    # Adding snooze of 5 secs. This is added as fix for Burt624613 where snapmirror
    # show takes few seconds to catch change in state of snapmirror relationship.
    sleep(5);

    # wait for completion of all the start_destination_vservers.
    $snapmirror_mtask_obj->wait_for_start_destination_vserver(
                                nacltask_continue_on_error => $opts{nacltask_continue_on_error},
                                nacltask_error_count       => $error_count,
                                nacltask_poll_interval     => $opts{nacltask_poll_interval},
                                'method-timeout'           => $opts{'method-timeout'},);
    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 stop_source_vserver

    # Stop source vserver with objects stored in MTask:
    $SMs->stop_source_vserver();

    # Stop source vserver from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->stop_source_vserver(
        snapmirrors => \@snapmirrors
    );

    # Stop source vserver from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->stop_source_vserver(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Stop source vserver from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->stop_source_vserver(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request stop_source_vserver of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to stop_source_vserver an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either stop_source_vserver
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=back

=item Exceptions

Any exceptions that can be thrown by the B<new> method of B<NACL::STask::Snapmirror>
and the B<stop> method of B<NACL::STask::Vserver>
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub stop_source_vserver {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors               => { type => ARRAYREF, optional => 1},
            command_interfaces        => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_continue_on_error => { type => SCALAR, default => 0},
            nacltask_poll_interval     => { type => SCALAR, default => 10},
            'method-timeout'           => { type => SCALAR, default => 1200},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $error_count = 0;
    my $continue_on_error = $opts{nacltask_continue_on_error};
    # If this is an instance call then we will perform the stop source vserver
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform stop_source_vserver on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Stop_Source_VserverGroupStart",
                          message => \%message);
    foreach my $snapmirror (@snapmirrors) {
        $snapmirror->{nacltask_state} = 'unknown';
        %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
        $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Stop_Source_VserverStart",
                      message => \%message);
        try {
            # Stop the source vserver
            my %args = $pkg_or_obj->_get_snapmirror_primary_keys(%{$snapmirror});
            my $command_interface = $snapmirror->{source_ci};
            NACL::STask::Vserver->stop(command_interface => $command_interface,
                                   vserver => $args{'source-vserver'},
                                   nacltask_if_stopped => 'pass',
                                   nacltask_wait => 0,
                                   foreground => 'false',
                                   );
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Stop_Source_VserverError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Stop_Source_VserverError",
                              message => \%message);

                # Publish the group finished message.
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Stop_Source_VserverGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };
    }

    # If all have errored then throw exception because we do not need to wait.
    if ($error_count == @snapmirrors) {
        # Publish the group finished message.
        %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
        $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Stop_Source_VserverGroupFinish",
                              message => \%message);
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish stop source vserver. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    # Set the snapmirrors on the object to the stop_source_vserver list of snapmirrors so that the wait_for
    # call will have knowledge of errors, etc.
    $snapmirror_mtask_obj->{snapmirrors} = \@snapmirrors;

    # Adding snooze of 5 secs. This is added as fix for Burt624613 where snapmirror
    # show takes few seconds to catch change in state of snapmirror relationship.
    sleep(5);

    # wait for completion of all the stop_source_vservers.
    $snapmirror_mtask_obj->wait_for_stop_source_vserver(
                                nacltask_continue_on_error => $opts{nacltask_continue_on_error},
                                nacltask_error_count       => $error_count,
                                nacltask_poll_interval     => $opts{nacltask_poll_interval},
                                'method-timeout'           => $opts{'method-timeout'},);
    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 purge

    # Purge with objects stored in MTask:
    $SMs->purge();

    # Purge from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->purge(
        snapmirrors => \@snapmirrors
    );

    # Purge from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->purge(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Purge from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->purge(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request purge of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to purge an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either purge
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=item C<< nacltask_delete_args =E<gt> \%delete_args >>

(Optional.) Args to pass to the delete call for all snapmirrors.

=item C<< nacltask_break_args =E<gt> \%break_args >>

(Optional.) Args to pass to the break call for all snapmirrors.

=item C<< nacltask_release_args =E<gt> \%release_args >>

(Optional.) Args to pass to the release call for all snapmirrors.

=back

=item Exceptions

Any exceptions that can be thrown by the B<new>, B<break>, B<delete>, B<release>, 
methods of B<NACL::STask::Snapmirror> 
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors                     => { type => ARRAYREF, optional => 1},
            command_interfaces              => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_delete_args            => { type => HASHREF, optional => 1, default => {}},
            nacltask_continue_on_error      => { type => SCALAR, default => 0},
            nacltask_poll_interval          => { type => SCALAR, default => 10},
            'method-timeout'                => { type => SCALAR, default => 3600},
            nacltask_break_args             => { type => HASHREF, optional => 1, default => {}},
            nacltask_release_args           => { type => HASHREF, optional => 1, default => {}},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $continue_on_error = $opts{nacltask_continue_on_error};
    my $poll_interval = $opts{nacltask_poll_interval};
    my $error_count = 0;
    my $timeout     = $opts{'method-timeout'};
    my $nacltask_delete_args  = $opts{nacltask_delete_args};
    my $nacltask_break_args   = $opts{nacltask_break_args};
    my $nacltask_release_args = $opts{nacltask_release_args};

    # If this is an instance call then we will perform the purge
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform delete on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my @finished_purge_snapmirrors;

    # Set the purge_state to a start state
    foreach my $snapmirror (@snapmirrors) {
        $snapmirror->{nacltask_state} = 'unknown';
        $snapmirror->{purge_started} = 0;
        $snapmirror->{wait_time} = time();
        $snapmirror->{purge_state} = 'break';
    }

    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::PurgeGroupStart",
                          message => \%message);


    my $end_time = time() + $timeout;
    # -1 represents infinite wait time.
    if ($timeout == -1) {
        $end_time = -1;
    }

    while (@snapmirrors) {
        my $snapmirror = pop @snapmirrors;
        unless ($snapmirror->{purge_started}) {
            %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::PurgeStart",
                          message => \%message);
            $snapmirror->{purge_started} = 1;
        }
        try {
            if ($snapmirror->{purge_state} eq 'break') {
                my $exists = 1;
                # Break the snapmirror relationship
                try {
                    $snapmirror->break(nacltask_wait => 0,
                                       nacltask_if_broken => 'pass',
                                       %{$nacltask_break_args});
                } catch NACL::C::Exceptions::Snapmirror::DoesNotExist with {
                    $exists = 0;
                };
                if ($exists) {
                    $snapmirror->{purge_state} = 'wait_break';
                } else {
                    $snapmirror->{purge_state} = 'release';
                }
            } elsif ($snapmirror->{purge_state} eq 'wait_break') {
                if(time() >= $snapmirror->{wait_time}) {
                    try {
                        $snapmirror->wait_for_break(
                                            'method-timeout' => 0,
                                            );
                        $snapmirror->{purge_state} = 'delete'
                    } catch NACL::Exceptions::Timeout with {
                        # Ignore timeout exceptions since we are using the timeout
                        # to perform a single check for done.
                        # Just set the wait_time
                        $snapmirror->{wait_time} = $snapmirror->{wait_time} + $poll_interval;
                    };
                } # else do nothing.
            } elsif ($snapmirror->{purge_state} eq 'delete') {
                # delete the snapmirror relationship
                $snapmirror->delete(%{$nacltask_delete_args},);
                $snapmirror->{purge_state} = 'release';
            } elsif ($snapmirror->{purge_state} eq 'release') {
                $snapmirror->release(%{$nacltask_release_args});
                $snapmirror->{purge_state} = 'wait_release';
            } elsif ($snapmirror->{purge_state} eq 'wait_release') {
                # Wait for snapmirror to not exist in list-destinations.
                if(time() >= $snapmirror->{wait_time}) {
                    try {
                        $snapmirror->wait_for_release(
                            'method-timeout' => 0,
                        );
                        $snapmirror->{purge_state} = 'done';
                    } catch NACL::Exceptions::Timeout with {
                        # Ignore timeout exceptions since we are using the timeout
                        # to perform a single check for done.
                        # Just set the wait_time
                        $snapmirror->{wait_time} = $snapmirror->{wait_time} + $poll_interval;
                    };
                } # else do nothing.
            }
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                $snapmirror->{purge_state} = 'done';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::PurgeError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                unshift (@finished_purge_snapmirrors, $snapmirror);
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::PurgeError",
                              message => \%message);

                # Publish the group finished message.
                push (@snapmirrors, @finished_purge_snapmirrors);
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::PurgeGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };

        unless($snapmirror->{purge_state} eq 'done') {
            unshift (@snapmirrors, $snapmirror);
        } else {
            unshift (@finished_purge_snapmirrors, $snapmirror);
            unless ($snapmirror->{nacltask_state} eq 'error') {
                $snapmirror->{nacltask_state} = 'finished';
                # The Snapmirror has finished publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::PurgeFinish",
                              message => \%message);
            }
        }

        if($end_time == -1) {
            # This means that we will wait forever.
            # So don't check or throw timeout errors.
        }
        elsif($end_time < time() and @snapmirrors) {
            # Time has run out but there are snapmirrors still not finished.
            # We need to publish error messages for leftover snapmirrors
            # then publish the group finished and then throw a timeout exception.
            foreach my $timedout_snapmirror (@snapmirrors) {
                # Publish the error message for this snapmirror and continue.
                $timedout_snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$timedout_snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::PurgeError",
                              message => \%message);
            }

            my $snapmirrors_left = @snapmirrors;
            # Publish the group finished message.
            push (@snapmirrors, @finished_purge_snapmirrors);
            %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::PurgeGroupFinish",
                                  message => \%message);
            NACL::Exceptions::Timeout->throw("Snapmirror purge failed to complete in $timeout " .
                                             "seconds for $snapmirrors_left out of " . @snapmirrors .
                                             " snapmirrors.");
        }
    }

    %message = $pkg_or_obj->_build_message( snapmirrors => \@finished_purge_snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::PurgeGroupFinish",
                          message => \%message);

    if ($error_count) {
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish udpate. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    $snapmirror_mtask_obj->{snapmirrors} = \@finished_purge_snapmirrors;
    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 flip_resync

    # Flip_resync with objects stored in MTask:
    $SMs->flip_resync();

    # Flip_resync from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->flip_resync(
        snapmirrors => \@snapmirrors
    );

    # Flip_resync from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->flip_resync(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Flip_resync from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->flip_resync(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request flip_resync of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to flip_resync an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either flip_resync
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=item C<< nacltask_delete_args =E<gt> \%delete_args >>

(Optional.) Args to pass to the delete call for all snapmirrors.

=item C<< nacltask_stop_source_vserver =E<gt> 0|1 >>

(Optional, defaults to 1) To stop the source vserver or not.

=item C<< nacltask_purge_snapmirror =E<gt> 0|1 >>

(Optional, defaults to 0) tells whether to purge the original snapmirror.
=back

=item Exceptions

Any exceptions that can be thrown by the B<new>, B<break>, B<delete>, B<release>, 
B<create> methods of B<NACL::STask::Snapmirror> and the 
B<start>, B<stop> methods of B<NACL::STask::Vserver>
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub flip_resync {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors                                 => { type => ARRAYREF, optional => 1},
            command_interfaces                          => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_stop_source_vserver                => { type => SCALAR, default => 1},
            nacltask_stop_id_preserve_source_vserver    => { type => SCALAR, default => 1},
            nacltask_purge_snapmirror                   => { type => SCALAR, default => 0},
            nacltask_delete_args                        => { type => HASHREF, optional => 1, default => {}},
            nacltask_continue_on_error                  => { type => SCALAR, default => 0},
            nacltask_poll_interval                      => { type => SCALAR, default => 10},
            'method-timeout'                            => { type => SCALAR, default => 3600},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $nacltask_delete_args = $opts{nacltask_delete_args};
    my $continue_on_error = $opts{nacltask_continue_on_error};
    my $poll_interval = $opts{nacltask_poll_interval};
    my $error_count = 0;
    my $timeout     = $opts{'method-timeout'};

    # If this is an instance call then we will perform the flip_resync
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform delete on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my @finished_flip_resync_snapmirrors;

    # Set the flip_resync_state to a start state
    foreach my $snapmirror (@snapmirrors) {
        $snapmirror->{nacltask_state} = 'unknown';
        $snapmirror->{flip_resync_started} = 0;
        $snapmirror->{wait_time} = time();
        if($opts{nacltask_stop_source_vserver} or ($opts{nacltask_stop_id_preserve_source_vserver} and $snapmirror->{'identity-preserve'} =~ /true/i)) {
            $snapmirror->{flip_resync_state} = 'stop_source';
        } else {
            $snapmirror->{flip_resync_state} = 'break';
        }

        unless(defined $snapmirror->{source_ci}) {
            NATE::Exceptions::Argument->throw("Trying to flip-resync but found a snapmirror without a source_ci, and " .
                                              "a flip-resync cannot be performed without knowing the source_ci. Did you include a node " .
                                              "from the source cluster in command_interfaces when you made this call or create the MTask obj?");
        }
    }

    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::FlipResyncGroupStart",
                          message => \%message);

    
    my $end_time = time() + $timeout;
    # -1 represents infinite wait time.
    if ($timeout == -1) {
        $end_time = -1;
    }

    # There are 4 different paths for flip-resync to take in this method
    # based on the nacltask_stop_source_vserver and nacltask_purge_snapmirror
    # parameters
    # 1. stop_source,wait_stop_source,break,wait_break,delete,start_destination,
    #    wait_start_destination,create,resync,wait_resync,release
    # 2. break,wait_break,delete,start_destination,
    #    wait_start_destination,create,resync,wait_resync,release
    # 3. stop_source,wait_stop_source,break,wait_break,start_destination,
    #    wait_start_destination,create,resync,wait_resync,
    # 4. break,wait_break,start_destination,
    #    wait_start_destination,create,resync,wait_resync
    while (@snapmirrors) {
        my $snapmirror = pop @snapmirrors;
        unless ($snapmirror->{flip_resync_started}) {
            %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::FlipResyncStart",
                          message => \%message);
            $snapmirror->{flip_resync_started} = 1;
        }
        try {
            if ($snapmirror->{flip_resync_state} eq 'stop_source') {
                # Stop the source vserver.
                my %args = $pkg_or_obj->_get_snapmirror_primary_keys(%{$snapmirror});
                my $command_interface = $snapmirror->{source_ci};
                NACL::STask::Vserver->stop(command_interface => $command_interface,
                                           vserver => $args{'source-vserver'},
                                           nacltask_if_stopped => 'pass',
                                           nacltask_wait => 0,
                                           foreground => 'false',
                                           );
                $snapmirror->{flip_resync_state} = 'wait_stop_destination';
            } elsif ($snapmirror->{flip_resync_state} eq 'wait_stop_destination') {
                if(time() >= $snapmirror->{wait_time}) {
                    try {
                        my %snap_args = $pkg_or_obj->_get_snapmirror_primary_keys(%{$snapmirror});
                        NACL::STask::Vserver->wait_for_stopped(
                            command_interface => $snapmirror->{source_ci},
                            vserver => $snap_args{'source-vserver'},
                            'method-timeout' => 0,
                        );
                        $snapmirror->{flip_resync_state} = 'break';
                    } catch NACL::Exceptions::Timeout with {
                        # Ignore timeout exceptions since we are using the timeout
                        # to perform a single check for done.
                        # Just set the wait_time
                        $snapmirror->{wait_time} = $snapmirror->{wait_time} + $poll_interval;
                    };
                } # else do nothing.
            }elsif ($snapmirror->{flip_resync_state} eq 'break') {
                # Break the snapmirror relationship
                $snapmirror->break(nacltask_wait => 0,
                                   nacltask_if_broken => 'pass');
                $snapmirror->{flip_resync_state} = 'wait_break';
            } elsif ($snapmirror->{flip_resync_state} eq 'wait_break') {
                if(time() >= $snapmirror->{wait_time}) {
                    try {
                        $snapmirror->wait_for_break(
                                            'method-timeout' => 0,
                                            );
                        if ($opts{nacltask_purge_snapmirror}) {
                            $snapmirror->{flip_resync_state} = 'delete'
                        } else {
                            $snapmirror->{flip_resync_state} = 'start_destination';
                        }
                    } catch NACL::Exceptions::Timeout with {
                        # Ignore timeout exceptions since we are using the timeout
                        # to perform a single check for done.
                        # Just set the wait_time
                        $snapmirror->{wait_time} = $snapmirror->{wait_time} + $poll_interval;
                    };
                } # else do nothing.
            } elsif ($snapmirror->{flip_resync_state} eq 'delete') {
                # delete the snapmirror relationship
                $snapmirror->delete(%{$nacltask_delete_args},);
                $snapmirror->{flip_resync_state} = 'start_destination';
            } elsif ($snapmirror->{flip_resync_state} eq 'start_destination') {
                # Start the destination vserver
                my %args = $pkg_or_obj->_get_snapmirror_primary_keys(%{$snapmirror});
                my $command_interface = $snapmirror->{command_interface};
                NACL::STask::Vserver->start(command_interface => $command_interface,
                                           vserver => $args{'destination-vserver'},
                                           nacltask_if_started => 'pass',
                                           nacltask_wait => 0,
                                           foreground => 'false',
                                           );
                $snapmirror->{flip_resync_state} = 'create';
            } elsif ($snapmirror->{flip_resync_state} eq 'wait_start_destination') {
                # Wait for vserver to be started.
                if(time() >= $snapmirror->{wait_time}) {
                    try {
                        my %snap_args = $pkg_or_obj->_get_snapmirror_primary_keys(%{$snapmirror});
                        NACL::STask::Vserver->wait_for_started(
                            command_interface => $snapmirror->{command_interface},
                            vserver => $snap_args{'destination-vserver'},
                            'method-timeout' => 0,
                        );
                        $snapmirror->{flip_resync_state} = 'create';
                    } catch NACL::Exceptions::Timeout with {
                        # Ignore timeout exceptions since we are using the timeout
                        # to perform a single check for done.
                        # Just set the wait_time
                        $snapmirror->{wait_time} = $snapmirror->{wait_time} + $poll_interval;
                    };
                } # else do nothing.
            } elsif ($snapmirror->{flip_resync_state} eq 'create') {
                # Create the opposite snapmirror relationship.
                $pkg_or_obj->_flip_snapmirror_primary_keys(snapmirror => $snapmirror);
                my %create_args = $pkg_or_obj->_get_snapmirror_primary_keys(%{$snapmirror});
                $create_args{source_ci} = $snapmirror->{source_ci} if defined ($snapmirror->{source_ci});
                $create_args{'identity-preserve'} = $snapmirror->{'identity-preserve'} if defined ('identity-preserve');
                my $command_interface = $snapmirror->{command_interface};
                NACL::STask::Snapmirror->create(command_interface => $command_interface,
                                                %create_args,
                                                do_initialize => 0,
                                                nacltask_if_exists => 'reuse',);
                $snapmirror->{flip_resync_state} = 'resync';
            } elsif ($snapmirror->{flip_resync_state} eq 'resync') {
                # Resync the new snapmirror relationship.
                $snapmirror->resync(nacltask_wait => 0);
                $snapmirror->{flip_resync_state} = 'wait_resync';
            } elsif ($snapmirror->{flip_resync_state} eq 'wait_resync') {
                if(time() >= $snapmirror->{wait_time}) {
                    try {
                        $snapmirror->wait_for_resync(
                            'method-timeout' => 0,
                        );
                        if ($opts{nacltask_purge_snapmirror}) {
                            $snapmirror->{flip_resync_state} = 'release'
                        } else {
                            $snapmirror->{flip_resync_state} = 'done';
                        }
                    } catch NACL::Exceptions::Timeout with {
                        # Ignore timeout exceptions since we are using the timeout
                        # to perform a single check for done.
                        # Just set the wait_time
                        $snapmirror->{wait_time} = $snapmirror->{wait_time} + $poll_interval;
                    };
                } # else do nothing.
            } elsif ($snapmirror->{flip_resync_state} eq 'release') {
                my $release_snapmirror = Clone::clone($snapmirror);
                $pkg_or_obj->_flip_snapmirror_primary_keys(snapmirror => $release_snapmirror);
                $release_snapmirror->release();
                $snapmirror->{flip_resync_state} = 'wait_release';
            } elsif ($snapmirror->{flip_resync_state} eq 'wait_release') {
                # Wait for snapmirror to not exist in list-destinations.
                if(time() >= $snapmirror->{wait_time}) {
                    my $release_snapmirror = Clone::clone($snapmirror);
                    $pkg_or_obj->_flip_snapmirror_primary_keys(snapmirror => $release_snapmirror);
                    try {
                        $snapmirror->wait_for_release(
                            'method-timeout' => 0,
                        );
                        $snapmirror->{flip_resync_state} = 'done';
                    } catch NACL::Exceptions::Timeout with {
                        # Ignore timeout exceptions since we are using the timeout
                        # to perform a single check for done.
                        # Just set the wait_time
                        $snapmirror->{wait_time} = $snapmirror->{wait_time} + $poll_interval;
                    };
                } # else do nothing.
            }
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                $snapmirror->{flip_resync_state} = 'done';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::FlipResyncError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                unshift (@finished_flip_resync_snapmirrors, $snapmirror);
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::FlipResyncError",
                              message => \%message);

                # Publish the group finished message.
                push (@snapmirrors, @finished_flip_resync_snapmirrors);
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::FlipResyncGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };

        unless($snapmirror->{flip_resync_state} eq 'done') {
            unshift (@snapmirrors, $snapmirror);
        } else {
            unshift (@finished_flip_resync_snapmirrors, $snapmirror);
            unless ($snapmirror->{nacltask_state} eq 'error') {
                $snapmirror->{nacltask_state} = 'finished';
                # The Snapmirror has finished publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::FlipResyncFinish",
                              message => \%message);
            }
        }

        if($end_time == -1) {
            # This means that we will wait forever.
            # So don't check or throw timeout errors.
        }
        elsif($end_time < time() and @snapmirrors) {
            # Time has run out but there are snapmirrors still not finished.
            # We need to publish error messages for leftover snapmirrors
            # then publish the group finished and then throw a timeout exception.
            foreach my $timedout_snapmirror (@snapmirrors) {
                # Publish the error message for this snapmirror and continue.
                $timedout_snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                %message = $pkg_or_obj->_build_message( snapmirrors => [$timedout_snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::FlipResyncError",
                              message => \%message);
            }

            my $snapmirrors_left = @snapmirrors;
            # Publish the group finished message.
            push (@snapmirrors, @finished_flip_resync_snapmirrors);
            %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::FlipResyncGroupFinish",
                                  message => \%message);
            NACL::Exceptions::Timeout->throw("Snapmirror flip_resync failed to complete in $timeout " .
                                             "seconds for $snapmirrors_left out of " . @snapmirrors .
                                             " snapmirrors.");
        }
    }

    %message = $pkg_or_obj->_build_message( snapmirrors => \@finished_flip_resync_snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::FlipResyncGroupFinish",
                          message => \%message);

    if ($error_count) {
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish udpate. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    $snapmirror_mtask_obj->{snapmirrors} = \@finished_flip_resync_snapmirrors;
    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 flip_source_and_destination

    # Contruct from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->flip_source_and_destination(
        snapmirrors => \@snapmirrors
    );

    # Create from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->flip_source_and_destination(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Create from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->flip_source_and_destination(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

(May only be called as a class method.)

B<flip_source_and_destination> flips the source and destination of the supplied or stored
snapmirrors and returns an updated NACL::MTask::Snapmirror object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<flip_source_and_destination>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to create an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<flip_source_and_destination>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either create
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=back

=item Exceptions

If any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub flip_source_and_destination {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors                     => { type => ARRAYREF, optional => 1},
            command_interfaces              => { type => ARRAYREF|HASHREF, optional => 1},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    # If this is an instance call then we will perform a flip of source and destinations
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform a flip of source and destination on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my @new_snapmirrors;
    foreach my $snapmirror (@snapmirrors) {
        $pkg_or_obj->_flip_snapmirror_primary_keys(snapmirror => $snapmirror);
        push(@new_snapmirrors,$snapmirror);
    }

    $snapmirror_mtask_obj->{snapmirrors} = \@new_snapmirrors;
    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 wait_for_initialize

    # Wait for initialize with objects stored in MTask:
    $SMs->wait_for_initialize();

    # Wait for initialize from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->wait_for_initialize(
        snapmirrors => \@snapmirrors
    );

    # Wait for initialize from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->wait_for_initialize(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Wait for initialize from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->wait_for_initialize(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request wait_for_initialize of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to wait_for_initialize an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either wait_for_initialize
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=item C<< nacltask_error_count =E<gt> $error_count >>

(Optional, defaults to 0) The error count from a previous method call.
This is used by the other methods when using continue on error.

=back

=item Exceptions

Any exceptions that can be thrown by the B<new>, B<wait_for_initialize> methods of B<NACL::STask::Snapmirror> 
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub wait_for_initialize {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors                => { type => ARRAYREF, optional => 1},
            command_interfaces         => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_continue_on_error => { type => SCALAR, default => 0},
            nacltask_error_count       => { type => SCALAR, default => 0},
            nacltask_poll_interval     => { type => SCALAR, default => 10},
            'method-timeout'           => { type => SCALAR, default => 1200},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $continue_on_error = $opts{nacltask_continue_on_error};
    my $poll_interval = $opts{nacltask_poll_interval};
    my $error_count = $opts{nacltask_error_count};
    my $timeout     = $opts{'method-timeout'};
    # If this is an instance call then we will perform the wait
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform wait on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my @finished_snapmirrors = ();
    my $end_time = time() + $timeout;
    # -1 represents infinite wait time.
    if ($timeout == -1) {
        $end_time = -1;
    }

    # This is used to wait the polling interval after all snapmirrors left
    # have been checked for finish.
    my $snapmirror_count = @snapmirrors;
    while (@snapmirrors) {
        if($snapmirror_count) {
            $snapmirror_count--;
        } else {
            $snapmirror_count = @snapmirrors;
            sleep($poll_interval);
        }
        my $snapmirror = pop @snapmirrors;
        if ((defined($snapmirror->{nacltask_state})) and $snapmirror->{nacltask_state} eq 'error') {
            # This snapmirror errored in the call previous to wait so ignore it.
            push (@finished_snapmirrors,$snapmirror);
            next;
        }
        my $finished = 1;
        try {
            $snapmirror->wait_for_initialization(
                'method-timeout' => 0,
                'last-transfer-type' => 'initialize'
            );
        } catch NACL::Exceptions::Timeout with {
            # Ignore timeout exceptions since we are using the timeout
            # to perform a single check for done. Just set finished = 0
            $finished = 0;
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                push (@finished_snapmirrors,$snapmirror);
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::InitializationError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                push (@finished_snapmirrors,$snapmirror);
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::InitializationError",
                              message => \%message);

                # Publish the group finished message.
                push (@snapmirrors, @finished_snapmirrors);
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::InitializationGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };

        if ($finished) {
            $snapmirror->{nacltask_state} = 'finished';
            push (@finished_snapmirrors,$snapmirror);
            # The Snapmirror has finished publish message.
            my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::InitializationFinish",
                          message => \%message);
        } else {
            # The Snapmirror has not finished so put the snapmirror back in the list.
            unshift (@snapmirrors, $snapmirror);
        }

        if($end_time == -1) {
            # This means that we will wait forever.
            # So don't check or throw timeout errors.
        }
        elsif($end_time < time() and @snapmirrors) {
            # Time has run out but there are snapmirrors still not finished.
            # We need to publish error messages for leftover snapmirrors
            # then publish the group finished and then throw a timeout exception.
            foreach my $timedout_snapmirror (@snapmirrors) {
                # Publish the error message for this snapmirror and continue.
                $timedout_snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$timedout_snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::InitializationError",
                              message => \%message);
            }

            my $snapmirrors_left = @snapmirrors;
            # Publish the group finished message.
            push (@snapmirrors, @finished_snapmirrors);
            my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::InitializationGroupFinish",
                                  message => \%message);
            NACL::Exceptions::Timeout->throw("Snapmirror Initializations failed to complete in $timeout " .
                                             "seconds for $snapmirrors_left out of " . @snapmirrors .
                                             " snapmirrors.");
        }
    }

    push (@snapmirrors, @finished_snapmirrors);
    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::InitializationGroupFinish",
                          message => \%message);

    if ($error_count) {
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish udpate. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 wait_for_start_destination_vserver

    # Wait for start_destination_vserver with objects stored in MTask:
    $SMs->wait_for_start_destination_vserver();

    # Wait for start_destination_vserver from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->wait_for_start_destination_vserver(
        snapmirrors => \@snapmirrors
    );

    # Wait for start_destination_vserver from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->wait_for_start_destination_vserver(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Wait for start_destination_vserver from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->wait_for_start_destination_vserver(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request wait_for_start_destination_vserver of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to wait_for_start_destination_vserver an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either wait_for_start_destination_vserver
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=item C<< nacltask_error_count =E<gt> $error_count >>

(Optional, defaults to 0) The error count from a previous method call.
This is used by the other methods when using continue on error.

=back

=item Exceptions

Any exceptions that can be thrown by the B<new> method of B<NACL::STask::Snapmirror>
and B<wait_for_start> method of the B<NACL::STask::Vserver>
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub wait_for_start_destination_vserver {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors                => { type => ARRAYREF, optional => 1},
            command_interfaces         => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_continue_on_error => { type => SCALAR, default => 0},
            nacltask_error_count       => { type => SCALAR, default => 0},
            nacltask_poll_interval     => { type => SCALAR, default => 10},
            'method-timeout'           => { type => SCALAR, default => 1200},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $continue_on_error = $opts{nacltask_continue_on_error};
    my $poll_interval = $opts{nacltask_poll_interval};
    my $error_count = $opts{nacltask_error_count};
    my $timeout     = $opts{'method-timeout'};
    # If this is an instance call then we will perform the wait
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform wait on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my @finished_snapmirrors = ();
    my $end_time = time() + $timeout;
    # -1 represents infinite wait time.
    if ($timeout == -1) {
        $end_time = -1;
    }

    # This is used to wait the polling interval after all snapmirrors left
    # have been checked for finish.
    my $snapmirror_count = @snapmirrors;
    while (@snapmirrors) {
        if($snapmirror_count) {
            $snapmirror_count--;
        } else {
            $snapmirror_count = @snapmirrors;
            sleep($poll_interval);
        }
        my $snapmirror = pop @snapmirrors;
        if ((defined($snapmirror->{nacltask_state})) and $snapmirror->{nacltask_state} eq 'error') {
            # This snapmirror errored in the call previous to wait so ignore it.
            push (@finished_snapmirrors,$snapmirror);
            next;
        }
        my $finished = 1;
        try {
            my %snap_args = $pkg_or_obj->_get_snapmirror_primary_keys(%{$snapmirror});
            NACL::STask::Vserver->wait_for_started(
                command_interface => $snapmirror->{command_interface},
                vserver => $snap_args{'destination-vserver'},
                'method-timeout' => 0,
            );
        } catch NACL::Exceptions::Timeout with {
            # Ignore timeout exceptions since we are using the timeout
            # to perform a single check for done. Just set finished = 0
            $finished = 0;
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                push (@finished_snapmirrors,$snapmirror);
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Start_Destination_VserverError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                push (@finished_snapmirrors,$snapmirror);
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Start_Destination_VserverError",
                              message => \%message);

                # Publish the group finished message.
                push (@snapmirrors, @finished_snapmirrors);
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Start_Destination_VserverGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };

        if ($finished) {
            $snapmirror->{nacltask_state} = 'finished';
            push (@finished_snapmirrors,$snapmirror);
            # The Snapmirror has finished publish message.
            my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Start_Destination_VserverFinish",
                          message => \%message);
        } else {
            # The Snapmirror has not finished so put the snapmirror back in the list.
            unshift (@snapmirrors, $snapmirror);
        }

        if($end_time == -1) {
            # This means that we will wait forever.
            # So don't check or throw timeout errors.
        }
        elsif($end_time < time() and @snapmirrors) {
            # Time has run out but there are snapmirrors still not finished.
            # We need to publish error messages for leftover snapmirrors
            # then publish the group finished and then throw a timeout exception.
            foreach my $timedout_snapmirror (@snapmirrors) {
                # Publish the error message for this snapmirror and continue.
                $timedout_snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$timedout_snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Start_Destination_VserverError",
                              message => \%message);
            }

            my $snapmirrors_left = @snapmirrors;
            # Publish the group finished message.
            push (@snapmirrors, @finished_snapmirrors);
            my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Start_Destination_VserverGroupFinish",
                                  message => \%message);
            NACL::Exceptions::Timeout->throw("Snapmirror Start_Destination_Vservers failed to complete in $timeout " .
                                             "seconds for $snapmirrors_left out of " . @snapmirrors .
                                             " snapmirrors.");
        }
    }

    push (@snapmirrors, @finished_snapmirrors);
    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Start_Destination_VserverGroupFinish",
                          message => \%message);

    if ($error_count) {
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish udpate. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 wait_for_stop_source_vserver

    # Wait for stop_source_vserver with objects stored in MTask:
    $SMs->wait_for_stop_source_vserver();

    # Wait for stop_source_vserver from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->wait_for_stop_source_vserver(
        snapmirrors => \@snapmirrors
    );

    # Wait for stop_source_vserver from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->wait_for_stop_source_vserver(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Wait for stop_source_vserver from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->wait_for_stop_source_vserver(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request wait_for_stop_source_vserver of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to wait_for_stop_source_vserver an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either wait_for_stop_source_vserver
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=item C<< nacltask_error_count =E<gt> $error_count >>

(Optional, defaults to 0) The error count from a previous method call.
This is used by the other methods when using continue on error.

=back

=item Exceptions

Any exceptions that can be thrown by the B<new> method of B<NACL::STask::Snapmirror> 
and B<wait_for_stop> method of B<NACL::STask::Vserver>
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub wait_for_stop_source_vserver {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors                => { type => ARRAYREF, optional => 1},
            command_interfaces         => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_continue_on_error => { type => SCALAR, default => 0},
            nacltask_error_count       => { type => SCALAR, default => 0},
            nacltask_poll_interval     => { type => SCALAR, default => 10},
            'method-timeout'           => { type => SCALAR, default => 1200},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $continue_on_error = $opts{nacltask_continue_on_error};
    my $poll_interval = $opts{nacltask_poll_interval};
    my $error_count = $opts{nacltask_error_count};
    my $timeout     = $opts{'method-timeout'};
    # If this is an instance call then we will perform the wait
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform wait on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my @finished_snapmirrors = ();
    my $end_time = time() + $timeout;
    # -1 represents infinite wait time.
    if ($timeout == -1) {
        $end_time = -1;
    }
    # This is used to wait the polling interval after all snapmirrors left
    # have been checked for finish.
    my $snapmirror_count = @snapmirrors;
    while (@snapmirrors) {
        if($snapmirror_count) {
            $snapmirror_count--;
        } else {
            $snapmirror_count = @snapmirrors;
            sleep($poll_interval);
        }
        my $snapmirror = pop @snapmirrors;
        if ((defined($snapmirror->{nacltask_state})) and $snapmirror->{nacltask_state} eq 'error') {
            # This snapmirror errored in the call previous to wait so ignore it.
            push (@finished_snapmirrors,$snapmirror);
            next;
        }
        my $finished = 1;
        try {
            my %snap_args = $pkg_or_obj->_get_snapmirror_primary_keys(%{$snapmirror});
            NACL::STask::Vserver->wait_for_stopped(
                command_interface => $snapmirror->{source_ci},
                vserver => $snap_args{'source-vserver'},
                'method-timeout' => 0,
            );
        } catch NACL::Exceptions::Timeout with {
            # Ignore timeout exceptions since we are using the timeout
            # to perform a single check for done. Just set finished = 0
            $finished = 0;
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                push (@finished_snapmirrors,$snapmirror);
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Stop_Source_VserverError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                push (@finished_snapmirrors,$snapmirror);
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Stop_Source_VserverError",
                              message => \%message);

                # Publish the group finished message.
                push (@snapmirrors, @finished_snapmirrors);
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Stop_Source_VserverGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };

        if ($finished) {
            $snapmirror->{nacltask_state} = 'finished';
            push (@finished_snapmirrors,$snapmirror);
            # The Snapmirror has finished publish message.
            my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Stop_Source_VserverFinish",
                          message => \%message);
        } else {
            # The Snapmirror has not finished so put the snapmirror back in the list.
            unshift (@snapmirrors, $snapmirror);
        }

        if($end_time == -1) {
            # This means that we will wait forever.
            # So don't check or throw timeout errors.
        }
        elsif($end_time < time() and @snapmirrors) {
            # Time has run out but there are snapmirrors still not finished.
            # We need to publish error messages for leftover snapmirrors
            # then publish the group finished and then throw a timeout exception.
            foreach my $timedout_snapmirror (@snapmirrors) {
                # Publish the error message for this snapmirror and continue.
                $timedout_snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$timedout_snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Stop_Source_VserverError",
                              message => \%message);
            }

            my $snapmirrors_left = @snapmirrors;
            # Publish the group finished message.
            push (@snapmirrors, @finished_snapmirrors);
            my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Stop_Source_VserverGroupFinish",
                                  message => \%message);
            NACL::Exceptions::Timeout->throw("Snapmirror Stop_Source_Vservers failed to complete in $timeout " .
                                             "seconds for $snapmirrors_left out of " . @snapmirrors .
                                             " snapmirrors.");
        }
    }

    push (@snapmirrors, @finished_snapmirrors);
    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::Stop_Source_VserverGroupFinish",
                          message => \%message);

    if ($error_count) {
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish udpate. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 wait_for_break

    # Wait for break with objects stored in MTask:
    $SMs->wait_for_break();

    # Wait for break from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->wait_for_break(
        snapmirrors => \@snapmirrors
    );

    # Wait for break from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->wait_for_break(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Wait for break from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->wait_for_break(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request wait_for_break of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to wait_for_break an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either wait_for_break
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=item C<< nacltask_error_count =E<gt> $error_count >>

(Optional, defaults to 0) The error count from a previous method call.
This is used by the other methods when using continue on error.

=back

=item Exceptions

Any exceptions that can be thrown by the B<new>, B<wait_for_break> methods of B<NACL::STask::Snapmirror> 
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub wait_for_break {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors                => { type => ARRAYREF, optional => 1},
            command_interfaces         => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_continue_on_error => { type => SCALAR, default => 0},
            nacltask_error_count       => { type => SCALAR, default => 0},
            nacltask_poll_interval     => { type => SCALAR, default => 10},
            'method-timeout'           => { type => SCALAR, default => 1200},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $continue_on_error = $opts{nacltask_continue_on_error};
    my $poll_interval = $opts{nacltask_poll_interval};
    my $error_count = $opts{nacltask_error_count};
    my $timeout     = $opts{'method-timeout'};
    # If this is an instance call then we will perform the wait
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform wait on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my @finished_snapmirrors = ();
    my $end_time = time() + $timeout;
    # -1 represents infinite wait time.
    if ($timeout == -1) {
        $end_time = -1;
    }

    # This is used to wait the polling interval after all snapmirrors left
    # have been checked for finish.
    my $snapmirror_count = @snapmirrors;
    while (@snapmirrors) {
        if($snapmirror_count) {
            $snapmirror_count--;
        } else {
            $snapmirror_count = @snapmirrors;
            sleep($poll_interval);
        }
        my $snapmirror = pop @snapmirrors;
        if ((defined($snapmirror->{nacltask_state})) and $snapmirror->{nacltask_state} eq 'error') {
            # This snapmirror errored in the call previous to wait so ignore it.
            push (@finished_snapmirrors,$snapmirror);
            next;
        }
        my $finished = 1;
        try {
           $snapmirror->wait_for_break(
                                'method-timeout' => 0,
                                );
        } catch NACL::Exceptions::Timeout with {
            # Ignore timeout exceptions since we are using the timeout
            # to perform a single check for done. Just set finished = 0
            $finished = 0;
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                push (@finished_snapmirrors,$snapmirror);
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::BreakError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                push (@finished_snapmirrors,$snapmirror);
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::BreakError",
                              message => \%message);

                # Publish the group finished message.
                push (@snapmirrors, @finished_snapmirrors);
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::BreakGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };

        if ($finished) {
            $snapmirror->{nacltask_state} = 'finished';
            push (@finished_snapmirrors,$snapmirror);
            # The Snapmirror has finished publish message.
            my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::BreakFinish",
                          message => \%message);
        } else {
            # The Snapmirror has not finished so put the snapmirror back in the list.
            unshift (@snapmirrors, $snapmirror);
        }

        if($end_time == -1) {
            # This means that we will wait forever.
            # So don't check or throw timeout errors.
        }
        elsif($end_time < time() and @snapmirrors) {
            # Time has run out but there are snapmirrors still not finished.
            # We need to publish error messages for leftover snapmirrors
            # then publish the group finished and then throw a timeout exception.
            foreach my $timedout_snapmirror (@snapmirrors) {
                # Publish the error message for this snapmirror and continue.
                $timedout_snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$timedout_snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::BreakError",
                              message => \%message);
            }

            my $snapmirrors_left = @snapmirrors;
            # Publish the group finished message.
            push (@snapmirrors, @finished_snapmirrors);
            my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::BreakGroupFinish",
                                  message => \%message);
            NACL::Exceptions::Timeout->throw("Snapmirror Breaks failed to complete in $timeout " .
                                             "seconds for $snapmirrors_left out of " . @snapmirrors .
                                             " snapmirrors.");
        }
    }

    push (@snapmirrors, @finished_snapmirrors);
    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::BreakGroupFinish",
                          message => \%message);

    if ($error_count) {
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish udpate. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 wait_for_release

    # Wait for release with objects stored in MTask:
    $SMs->wait_for_release();

    # Wait for release from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->wait_for_release(
        snapmirrors => \@snapmirrors
    );

    # Wait for release from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->wait_for_release(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Wait for release from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->wait_for_release(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request wait_for_release of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to wait_for_release an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either wait_for_release
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=item C<< nacltask_error_count =E<gt> $error_count >>

(Optional, defaults to 0) The error count from a previous method call.
This is used by the other methods when using continue on error.

=back

=item Exceptions

Any exceptions that can be thrown by the B<new>, B<wait_for_release> methods of B<NACL::STask::Snapmirror> 
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub wait_for_release {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors                => { type => ARRAYREF, optional => 1},
            command_interfaces         => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_continue_on_error => { type => SCALAR, default => 0},
            nacltask_error_count       => { type => SCALAR, default => 0},
            nacltask_poll_interval     => { type => SCALAR, default => 10},
            'method-timeout'           => { type => SCALAR, default => 1200},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $continue_on_error = $opts{nacltask_continue_on_error};
    my $poll_interval = $opts{nacltask_poll_interval};
    my $error_count = $opts{nacltask_error_count};
    my $timeout     = $opts{'method-timeout'};
    # If this is an instance call then we will perform the wait
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform wait on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my @finished_snapmirrors = ();
    my $end_time = time() + $timeout;
    # -1 represents infinite wait time.
    if ($timeout == -1) {
        $end_time = -1;
    }

    # This is used to wait the polling interval after all snapmirrors left
    # have been checked for finish.
    my $snapmirror_count = @snapmirrors;
    while (@snapmirrors) {
        if($snapmirror_count) {
            $snapmirror_count--;
        } else {
            $snapmirror_count = @snapmirrors;
            sleep($poll_interval);
        }
        my $snapmirror = pop @snapmirrors;
        if ((defined($snapmirror->{nacltask_state})) and $snapmirror->{nacltask_state} eq 'error') {
            # This snapmirror errored in the call previous to wait so ignore it.
            push (@finished_snapmirrors,$snapmirror);
            next;
        }
        my $finished = 1;
        try {
            $snapmirror->wait_for_release(
                'method-timeout' => 0,
            );
        } catch NACL::Exceptions::Timeout with {
            # Ignore timeout exceptions since we are using the timeout
            # to perform a single check for done. Just set finished = 0
            $finished = 0;
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                push (@finished_snapmirrors,$snapmirror);
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ReleaseError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                push (@finished_snapmirrors,$snapmirror);
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ReleaseError",
                              message => \%message);

                # Publish the group finished message.
                push (@snapmirrors, @finished_snapmirrors);
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ReleaseGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };

        if ($finished) {
            $snapmirror->{nacltask_state} = 'finished';
            push (@finished_snapmirrors,$snapmirror);
            # The Snapmirror has finished publish message.
            my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ReleaseFinish",
                          message => \%message);
        } else {
            # The Snapmirror has not finished so put the snapmirror back in the list.
            unshift (@snapmirrors, $snapmirror);
        }

        if($end_time == -1) {
            # This means that we will wait forever.
            # So don't check or throw timeout errors.
        }
        elsif($end_time < time() and @snapmirrors) {
            # Time has run out but there are snapmirrors still not finished.
            # We need to publish error messages for leftover snapmirrors
            # then publish the group finished and then throw a timeout exception.
            foreach my $timedout_snapmirror (@snapmirrors) {
                # Publish the error message for this snapmirror and continue.
                $timedout_snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$timedout_snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ReleaseError",
                              message => \%message);
            }

            my $snapmirrors_left = @snapmirrors;
            # Publish the group finished message.
            push (@snapmirrors, @finished_snapmirrors);
            my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ReleaseGroupFinish",
                                  message => \%message);
            NACL::Exceptions::Timeout->throw("Snapmirror Releases failed to complete in $timeout " .
                                             "seconds for $snapmirrors_left out of " . @snapmirrors .
                                             " snapmirrors.");
        }
    }

    push (@snapmirrors, @finished_snapmirrors);
    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ReleaseGroupFinish",
                          message => \%message);

    if ($error_count) {
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish udpate. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 wait_for_resync

    # Wait for resync with objects stored in MTask:
    $SMs->wait_for_resync();

    # Wait for resync from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->wait_for_resync(
        snapmirrors => \@snapmirrors
    );

    # Wait for resync from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->wait_for_resync(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Wait for resync from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->wait_for_resync(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request wait_for_resync of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to wait_for_resync an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either wait_for_resync
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=item C<< nacltask_error_count =E<gt> $error_count >>

(Optional, defaults to 0) The error count from a previous method call.
This is used by the other methods when using continue on error.

=back

=item Exceptions

Any exceptions that can be thrown by the B<new>, B<wait_for_resync> methods of B<NACL::STask::Snapmirror> 
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub wait_for_resync {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors                => { type => ARRAYREF, optional => 1},
            command_interfaces         => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_continue_on_error => { type => SCALAR, default => 0},
            nacltask_error_count       => { type => SCALAR, default => 0},
            nacltask_poll_interval     => { type => SCALAR, default => 10},
            'method-timeout'           => { type => SCALAR, default => 1200},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $continue_on_error = $opts{nacltask_continue_on_error};
    my $poll_interval = $opts{nacltask_poll_interval};
    my $error_count = $opts{nacltask_error_count};
    my $timeout     = $opts{'method-timeout'};
    # If this is an instance call then we will perform the wait
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform wait on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my @finished_snapmirrors = ();
    my $end_time = time() + $timeout;
    # -1 represents infinite wait time.
    if ($timeout == -1) {
        $end_time = -1;
    }

    # This is used to wait the polling interval after all snapmirrors left
    # have been checked for finish.
    my $snapmirror_count = @snapmirrors;
    while (@snapmirrors) {
        if($snapmirror_count) {
            $snapmirror_count--;
        } else {
            $snapmirror_count = @snapmirrors;
            sleep($poll_interval);
        }
        my $snapmirror = pop @snapmirrors;
        if ((defined($snapmirror->{nacltask_state})) and $snapmirror->{nacltask_state} eq 'error') {
            # This snapmirror errored in the call previous to wait so ignore it.
            push (@finished_snapmirrors,$snapmirror);
            next;
        }
        my $finished = 1;
        try {
            $snapmirror->wait_for_resync(
                'method-timeout' => 0,
            );
        } catch NACL::Exceptions::Timeout with {
            # Ignore timeout exceptions since we are using the timeout
            # to perform a single check for done. Just set finished = 0
            $finished = 0;
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                push (@finished_snapmirrors,$snapmirror);
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ResyncError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                push (@finished_snapmirrors,$snapmirror);
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ResyncError",
                              message => \%message);

                # Publish the group finished message.
                push (@snapmirrors, @finished_snapmirrors);
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ResyncGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };

        if ($finished) {
            $snapmirror->{nacltask_state} = 'finished';
            push (@finished_snapmirrors,$snapmirror);
            # The Snapmirror has finished publish message.
            my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ResyncFinish",
                          message => \%message);
        } else {
            # The Snapmirror has not finished so put the snapmirror back in the list.
            unshift (@snapmirrors, $snapmirror);
        }

        if($end_time == -1) {
            # This means that we will wait forever.
            # So don't check or throw timeout errors.
        }
        elsif($end_time < time() and @snapmirrors) {
            # Time has run out but there are snapmirrors still not finished.
            # We need to publish error messages for leftover snapmirrors
            # then publish the group finished and then throw a timeout exception.
            foreach my $timedout_snapmirror (@snapmirrors) {
                # Publish the error message for this snapmirror and continue.
                $timedout_snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$timedout_snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ResyncError",
                              message => \%message);
            }

            my $snapmirrors_left = @snapmirrors;
            # Publish the group finished message.
            push (@snapmirrors, @finished_snapmirrors);
            my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ResyncGroupFinish",
                                  message => \%message);
            NACL::Exceptions::Timeout->throw("Snapmirror Resyncs failed to complete in $timeout " .
                                             "seconds for $snapmirrors_left out of " . @snapmirrors .
                                             " snapmirrors.");
        }
    }

    push (@snapmirrors, @finished_snapmirrors);
    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::ResyncGroupFinish",
                          message => \%message);

    if ($error_count) {
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish resync. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

=head2 wait_for_update

    # Wait for update with objects stored in MTask:
    $SMs->wait_for_update();

    # Wait for update from NACL::STask::Snapmirror objects:
    $SMs = NACL::MTask::Snapmirror->wait_for_update(
        snapmirrors => \@snapmirrors
    );

    # Wait for update from NACL::STask::Snapmirror objects but add source_ci if not there
    $SMs = NACL::MTask::Snapmirror->wait_for_update(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

    # Wait for update from snapmirror information hashes and command_interfaces
    $SMs = NACL::MTask::Snapmirror->wait_for_update(
        snapmirrors  => \@snapmirrors
        command_interfaces => \@nodes
    );

Request wait_for_update of all snapmirrors stored in mtask or passed in, and return the MTask object.

=over

=item Options

=over

=item C<< snapmirrors =E<gt> \@snapmirrors >>

(Required.) This is the list of snapmirrors objects or snapmirror information
for the group of snapmirrors. This is the only strictly required
 parameter for B<new>, and must be either a
B<NACL::STask::Snapmirror> object, a B<NACL::C::Snapmirror> object, or a hash
reference. A B<NACL::C::Snapmirror> object will be promoted to an STask object. If
it is a hash reference, the contents of the reference will be used to wait_for_update an
object of the B<NACL::STask::Snapmirror> class. If the creation of that object
fails, an exception will be thrown by B<new>. For detail on the keys that can
be in the hash reference, see L<NACL::C::Snapmirror|lib-NACL-C-Snapmirror-pm> and
L<NACL::STask::Snapmirror|lib-NACL-STask-Snapmirror-pm>.

=item C<< command_interfaces =E<gt> \@nodes >>

(Optional.) If passed, the command_interfaces will be used to either wait_for_update
NACL::STask::Snapmirror objects if hashes were passed in, or to add source_ci
if NACL::STask:Snapmirror objects

=item C<< nacltask_continue_on_error =E<gt> 1|0 >>

(Optional, defaults to 0) If passed, the method will continue operations on the other
snapmirrors if an error occurs and fail at the end.

=item C<< "nacltask_polling_interval =>  $polling_value" >>

(Optional, defaults to 10) This is the interval (in seconds) after which the value of
the attributes will be polled.
Disclaimer: User need to supply appropriate polling_interval. Larger value of
polling_interval might end up missing certain state or status that is being monitored if
transfer size is small.

=item C<< method-timeout =E<gt> $timeout >>

(Optional, defaults to 1200) Time for the method to time out and error.

=item C<< nacltask_error_count =E<gt> $error_count >>

(Optional, defaults to 0) The error count from a previous method call.
This is used by the other methods when using continue on error.

=back

=item Exceptions

Any exceptions that can be thrown by the B<new>, B<wait_for_update> methods of B<NACL::STask::Snapmirror> 
can appear here, as those methods may be called as part of the method.
Additionally, if any of the input data to this method is invalid, a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

=cut

sub wait_for_update {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors                => { type => ARRAYREF, optional => 1},
            command_interfaces         => { type => ARRAYREF|HASHREF, optional => 1},
            nacltask_continue_on_error => { type => SCALAR, default => 0},
            nacltask_error_count       => { type => SCALAR, default => 0},
            nacltask_poll_interval     => { type => SCALAR, default => 10},
            'method-timeout'           => { type => SCALAR, default => 1200},
        },
        allow_extra => 1,
    );

    my @snapmirrors = ();
    my $snapmirror_mtask_obj;
    my $continue_on_error = $opts{nacltask_continue_on_error};
    my $poll_interval = $opts{nacltask_poll_interval};
    my $error_count = $opts{nacltask_error_count};
    my $timeout     = $opts{'method-timeout'};
    # If this is an instance call then we will perform the wait
    # on the snapmirrors stored on the object.
    if (ref($pkg_or_obj)) {
        $snapmirror_mtask_obj = $pkg_or_obj;
        @snapmirrors = @{$pkg_or_obj->{snapmirrors}};
    } # Else this is a static call and we will create a new object
      # and perform wait on the snapmirrors passed in to the call.
    else {
        # Create a new snapmiror mtask object based on the supplied parameters.
        $snapmirror_mtask_obj = $pkg_or_obj->new(%opts);
        @snapmirrors = @{$snapmirror_mtask_obj->{snapmirrors}};
    }

    my @finished_snapmirrors = ();
    my $end_time = time() + $timeout;
    # -1 represents infinite wait time.
    if ($timeout == -1) {
        $end_time = -1;
    }

    # This is used to wait the polling interval after all snapmirrors left
    # have been checked for finish.
    my $snapmirror_count = @snapmirrors;
    while (@snapmirrors) {
        if($snapmirror_count) {
            $snapmirror_count--;
        } else {
            $snapmirror_count = @snapmirrors;
            sleep($poll_interval);
        }
        my $snapmirror = pop @snapmirrors;
        if ((defined($snapmirror->{nacltask_state})) and $snapmirror->{nacltask_state} eq 'error') {
            # This snapmirror errored in the call previous to wait so ignore it.
            push (@finished_snapmirrors,$snapmirror);
            next;
        }
        my $finished = 1;
        try {
            $snapmirror->wait_for_update(
                'method-timeout' => 0,
            );
        } catch NACL::Exceptions::Timeout with {
            # Ignore timeout exceptions since we are using the timeout
            # to perform a single check for done. Just set finished = 0
            $finished = 0;
        } otherwise {
            my $exception = $_[0];
            if($continue_on_error) {
                #Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                push (@finished_snapmirrors,$snapmirror);
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::UpdateError",
                              message => \%message);
                $error_count++;
                $Log->warn($exception->text());
            } # Else we need to fail but still publish the correct messages
            else {
                # Publish the error message for this snapmirror and continue.
                $snapmirror->{nacltask_state} = 'error';
                push (@finished_snapmirrors,$snapmirror);
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::UpdateError",
                              message => \%message);

                # Publish the group finished message.
                push (@snapmirrors, @finished_snapmirrors);
                %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::UpdateGroupFinish",
                                      message => \%message);
                $exception->throw();
            }
        };

        if ($finished) {
            $snapmirror->{nacltask_state} = 'finished';
            push (@finished_snapmirrors,$snapmirror);
            # The Snapmirror has finished publish message.
            my %message = $pkg_or_obj->_build_message( snapmirrors => [$snapmirror]);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::UpdateFinish",
                          message => \%message);
        } else {
            # The Snapmirror has not finished so put the snapmirror back in the list.
            unshift (@snapmirrors, $snapmirror);
        }

        if($end_time == -1) {
            # This means that we will wait forever.
            # So don't check or throw timeout errors.
        }
        elsif($end_time < time() and @snapmirrors) {
            # Time has run out but there are snapmirrors still not finished.
            # We need to publish error messages for leftover snapmirrors
            # then publish the group finished and then throw a timeout exception.
            foreach my $timedout_snapmirror (@snapmirrors) {
                # Publish the error message for this snapmirror and continue.
                $timedout_snapmirror->{nacltask_state} = 'error';
                # The Snapmirror has errored publish message.
                my %message = $pkg_or_obj->_build_message( snapmirrors => [$timedout_snapmirror]);
                $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::UpdateError",
                              message => \%message);
            }

            my $snapmirrors_left = @snapmirrors;
            # Publish the group finished message.
            push (@snapmirrors, @finished_snapmirrors);
            my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
            $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::UpdateGroupFinish",
                                  message => \%message);
            NACL::Exceptions::Timeout->throw("Snapmirror Updates failed to complete in $timeout " .
                                             "seconds for $snapmirrors_left out of " . @snapmirrors .
                                             " snapmirrors.");
        }
    }

    push (@snapmirrors, @finished_snapmirrors);
    my %message = $pkg_or_obj->_build_message( snapmirrors => \@snapmirrors);
    $Eventsvcapi->publish(topic   => "NACL::MTask::Events::Snapmirror::UpdateGroupFinish",
                          message => \%message);

    if ($error_count) {
        NATE::BaseException->throw("continue_on_error $error_count snapmirrors failed to finish udpate. " .
                                   "Look above in the logs for warnings that contain the failure text");
    }

    $Log->exit() if $may_exit;
    return $snapmirror_mtask_obj;
}

# returns the primary keys from a snapmirror object.
sub _get_snapmirror_primary_keys {
    my ($pkg_or_obj, %args) = @_;
    my %snapmirror_keys;
    foreach my $key (keys %args) {
        if ($key eq 'source-cluster') {
            $snapmirror_keys{'source-cluster'} = $args{'source-cluster'};
        } elsif ($key eq 'source-vserver') {
            $snapmirror_keys{'source-vserver'} = $args{'source-vserver'};
        } elsif ($key eq 'source-volume') {
            $snapmirror_keys{'source-volume'} = $args{'source-volume'};
        } elsif ($key eq 'source-path' or $key eq 'source_path') {
            my $source_path;
            $source_path = $args{'source-path'} if defined $args{'source-path'};
            $source_path = $args{'source_path'} if defined $args{'source_path'};
            # Look for path of the format 'cluster://vserver/volume'
            if ($source_path =~ /^(\S+):\/\/(\S+)\/(\S+)$/) {
                $snapmirror_keys{'source-cluster'} = $1;
                $snapmirror_keys{'source-vserver'} = $2;
                $snapmirror_keys{'source-volume'} = $3;
            } # else look for path of the format 'vserver:volume'
            elsif ($source_path =~ /^(\S+):(\S+)$/) {
                $snapmirror_keys{'source-vserver'} = $1;
                $snapmirror_keys{'source-volume'} = $2;
            } # else look for path of the format 'vserver:'
            elsif ($source_path =~ /^(\S+):$/) {
                $snapmirror_keys{'source-vserver'} = $1;
            } # else assume path is format 'volume'
            else {
                $snapmirror_keys{'source-volume'} = $source_path;
            }
        } elsif ($key eq 'destination-cluster') {
            $snapmirror_keys{'destination-cluster'} = $args{'destination-cluster'};
        } elsif ($key eq 'destination-vserver') {
            $snapmirror_keys{'destination-vserver'} = $args{'destination-vserver'};
        } elsif ($key eq 'destination-volume'){
            $snapmirror_keys{'destination-volume'} = $args{'destination-volume'};
        } elsif ($key eq 'destination-path' or $key eq 'destination_path') {
            my $destination_path;
            $destination_path = $args{'destination-path'} if defined $args{'destination-path'};
            $destination_path = $args{'destination_path'} if defined $args{'destination_path'};
            # Look for path of the format 'cluster://vserver/volume'
            if ($destination_path =~ /^(\S+):\/\/(\S+)\/(\S+)$/) {
                $snapmirror_keys{'destination-cluster'} = $1;
                $snapmirror_keys{'destination-vserver'} = $2;
                $snapmirror_keys{'destination-volume'} = $3;
            } # else look for path of the format 'vserver:volume'
            elsif ($destination_path =~ /^(\S+):(\S+)$/) {
                $snapmirror_keys{'destination-vserver'} = $1;
                $snapmirror_keys{'destination-volume'} = $2;
            } # else look for path of the format 'vserver:'
            elsif ($destination_path =~ /^(\S+):$/) {
                $snapmirror_keys{'destination-vserver'} = $1;
            } # else assume path is format 'volume'
            else {
                $snapmirror_keys{'destination-volume'} = $destination_path;
            }
        } else {
            next;
        }
    }
    $Log->exit() if $may_exit;
    return %snapmirror_keys
}

# flips the source and destination of a snapmirror object ref passed in
sub _flip_snapmirror_primary_keys {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirror  => { type => HASHREF, optional => 1},
        },
    );
    my $snapmirror = $opts{snapmirror};
    my %snapmirror_keys = %{$snapmirror};
    foreach my $key (keys %snapmirror_keys) {
        if ($key eq 'source-cluster') {
            $snapmirror->{'destination-cluster'} = $snapmirror_keys{'source-cluster'};
        } elsif ($key eq 'source-vserver') {
            $snapmirror->{'destination-vserver'} = $snapmirror_keys{'source-vserver'};
        } elsif ($key eq 'source-volume') {
            $snapmirror->{'destination-volume'} = $snapmirror_keys{'source-volume'};
        } elsif ($key eq 'source-path') {
            $snapmirror->{'destination-path'} = $snapmirror_keys{'source-path'};
        } elsif ($key eq 'source_path') {
            $snapmirror->{'destination_path'} = $snapmirror_keys{'source_path'};
        } elsif ($key eq 'destination-cluster') {
            $snapmirror->{'source-cluster'} = $snapmirror_keys{'destination-cluster'};
        } elsif ($key eq 'destination-vserver') {
            $snapmirror->{'source-vserver'} = $snapmirror_keys{'destination-vserver'};
        } elsif ($key eq 'destination-volume'){
            $snapmirror->{'source-volume'} = $snapmirror_keys{'destination-volume'};
        } elsif ($key eq 'destination-path') {
            $snapmirror->{'source-path'} = $snapmirror_keys{'destination-path'};
        } elsif ($key eq 'destination_path') {
            $snapmirror->{'source_path'} = $snapmirror_keys{'destination_path'};
        } elsif ($key eq 'command_interface') {
            $snapmirror->{'source_ci'} = $snapmirror_keys{'command_interface'};
        } elsif ($key eq 'source_ci') {
            $snapmirror->{'command_interface'} = $snapmirror_keys{'source_ci'};
        } else {
            next;
        }
    }
    $Log->exit() if $may_exit;
}

# builds a message from the list of snapmirrors passed.
sub _build_message {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors                => { type => ARRAYREF, optional => 1},
        },
        allow_extra => 1,
    );

    my %message = ();
    foreach my $snapmirror (@{$opts{snapmirrors}}) {
        my %snapmirror_hash =  %{$snapmirror};
        delete $snapmirror_hash{last_job};
        $snapmirror_hash{destination_node_name} = $snapmirror_hash{command_interface}->node() if defined $snapmirror_hash{command_interface};
        delete $snapmirror_hash{command_interface};
        $snapmirror_hash{source_node_name} = $snapmirror_hash{source_ci}->node() if defined $snapmirror_hash{source_ci};
        delete $snapmirror_hash{source_ci};
        push (@{$message{object_keys}},\%snapmirror_hash);
    }

    return wantarray ? %message : \%message;
}

# builds the snapmirrors or updates the snapmirrors with source_ci for the new.
sub _process_snapmirrors_and_command_interfaces {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            snapmirrors               => { type => ARRAYREF, optional => 1},
            command_interfaces        => { type => ARRAYREF|HASHREF, optional => 1}
        },
        allow_extra => 1,
    );

    my %node_to_cluster_map = ();
    my $snapmirrors = $opts{snapmirrors};
    my @command_interfaces_array = (defined ($opts{command_interfaces}) and ref($opts{command_interfaces}) eq 'ARRAY')
                                 ? @{$opts{command_interfaces}}
                                 : ();
    my %command_interfaces = (defined ($opts{command_interfaces}) and ref($opts{command_interfaces}) eq 'HASH')
                           ? @{$opts{command_interfaces}}
                           : ();

    # Put passed in command_interfaces into cluster hash.
    foreach my $command_interface (@command_interfaces_array) {
        if (not defined  $node_to_cluster_map{$command_interface->node()}) {
            my $cs_obj = NACL::CS::ClusterIdentity->fetch(
                 command_interface => $command_interface,
                 requested_fields  => ['name'],);
            $node_to_cluster_map{$command_interface->node()} = $cs_obj->name();
        }
        $command_interfaces{$node_to_cluster_map{$command_interface->node()}} = $command_interface;
    }

    # Put snapmirror command_interfaces into cluster hash.
    foreach my $snapmirror (@{$snapmirrors}) {
        next if (not defined $snapmirror->{command_interface});
        my $command_interface = $snapmirror->{command_interface};
        if (not defined  $node_to_cluster_map{$command_interface->node()}) {
            my $cs_obj = NACL::CS::ClusterIdentity->fetch(
                 command_interface => $command_interface,
                 requested_fields  => ['name'],);
            $node_to_cluster_map{$command_interface->node()} = $cs_obj->name();
        }
        $command_interfaces{$node_to_cluster_map{$command_interface->node()}} = $command_interface;
    }

    # Make sure that all snapmirrors are STasks and have command_interfaces.
    foreach my $snapmirror (@{$snapmirrors}) {
        # If already an STask then skip
        next if (ref($snapmirror) =~ /NACL::STask::Snapmirror/i);

        # If a component then cast to an STask
        if (ref($snapmirror) =~ /NACL::C::Snapmirror/i) {
            $snapmirror = $snapmirror->cast_component_to_task();
        } # else we assume is a hash of snapmirror information.
        else {
            if(defined $snapmirror->{command_interface}) {
                $snapmirror = NACL::STask::Snapmirror->new(%{$snapmirror});
            } elsif (defined $snapmirror->{'destination-cluster'}) {
                unless(defined $command_interfaces{$snapmirror->{'destination-cluster'}}) {
                    NATE::Exceptions::Argument->throw("A hash of snapmirror information was passed instead of an object in the snapmirrors array, " .
                                                      "and this hash did not contain a command_interface, and did contain a destination-cluster but, " .
                                                      "the destination-cluster could not be found in the provided command_interfaces. Did you pass in a node " .
                                                      "for the destination-cluster " . $snapmirror->{'destination-cluster'});
                }

                $snapmirror = NACL::STask::Snapmirror->new(command_interface => $command_interfaces{$snapmirror->{'destination-cluster'}},
                                                           %{$snapmirror});
            } else{
                NATE::Exceptions::Argument->throw("A hash of snapmirror information was passed instead of an object in the snapmirrors array, " .
                                                  "and this hash did not contain a command_interface or destination-cluster");
            }
        }
    }

    # Get All the vserver peers for all the clusters provided.
    my @vserver_peers;

    # Now that we have all the peers lets ensure that the snapmirror have a source_ci if we can.
    foreach my $snapmirror (@{$snapmirrors}) {
        # add destination_cluster_name information for message passing later.
        $snapmirror->{destination_cluster_name} = $node_to_cluster_map{$snapmirror->{command_interface}->node()};

        # Get the identity-preserve attribute if it doesn't already exist.
        if (not defined $snapmirror->{'identity-preserve'}) {
            try {
                my $snapmirror_cs = $snapmirror->state(requested_fields => ['identity-preserve']);
                # If we get to here then the Snapmirror exists.
                $snapmirror->{'identity-preserve'} = $snapmirror_cs->{'identity_preserve'};
            } catch NACL::Exceptions::NoElementsFound with {
                # The snapmirror may not exist yet. If it doesn't then we will assume identity-preserve false. 
                # Since that is the ontap default.
                $snapmirror->{'identity-preserve'} = 'false';
            };
        }

        # add source_cluster_name and skip if the snapmirror already contains source_ci
        if (defined $snapmirror->{source_ci}) {
            # add source_cluster_name information for message passing later.
            $snapmirror->{source_cluster_name} = $node_to_cluster_map{$snapmirror->{source_ci}->node()};
        }

        my %snapmirror_primary_args = $pkg_or_obj->_get_snapmirror_primary_keys(%{$snapmirror});
        # If the snapmirror has source-cluster then add source_ci.
        if (defined $snapmirror_primary_args{'source-cluster'}) {
            # add source_cluster_name information for message passing later.
            $snapmirror->{source_cluster_name} = $snapmirror_primary_args{'source-cluster'};
            if(defined $command_interfaces{$snapmirror_primary_args{'source-cluster'}}) {
                $snapmirror->{source_ci} = $command_interfaces{$snapmirror_primary_args{'source-cluster'}};
            }
            # else the user did not pass in a command interface for the source-cluster
            # this could be because the do not want to perform any operations that require
            # the source-cluster so we will continue and methods that will require the
            # source-cluster will fail.
        } # else we look up the source-cluster based on the vserver_peer information.
          # and the source-vserver and destination-vserver information.
        else {
            if(defined ($snapmirror_primary_args{'source-vserver'}) and defined ($snapmirror_primary_args{'destination-vserver'})) {
                # If we have not filled the vserver_peers array before then fill it now.
                # We do this here so that we only perform the fetches if we need this functionality.
                # And we only perform the fetches for vserver_peer once as well.
                unless (@vserver_peers) {
                    foreach my $cluster_name (keys %command_interfaces) {
                        my @current_peers = NACL::CS::VserverPeer->fetch(command_interface => $command_interfaces{$cluster_name},
                                                                         requested_fields  => ['vserver','peer-cluster'],
                                                                         allow_empty       => 1);
                        push(@vserver_peers, @current_peers)
                    }
                }

                foreach my $vserver_peer_obj (@vserver_peers) {
                    # Find the vserver_peer that matches our source-vserver and destination-vserver
                    if($vserver_peer_obj->vserver() eq $snapmirror_primary_args{'destination-vserver'} and
                       $vserver_peer_obj->peer_vserver() eq $snapmirror_primary_args{'source-vserver'}) {
                        # add source_cluster_name information for message passing later.
                        $snapmirror->{source_cluster_name} = $vserver_peer_obj->peer_cluster();
                        if(defined $command_interfaces{$vserver_peer_obj->peer_cluster()}) {
                            $snapmirror->{source_ci} = $command_interfaces{$vserver_peer_obj->peer_cluster()};
                        }
                        # else the user did not pass in a command interface for the source-cluster
                        # this could be because the do not want to perform any operations that require
                        # the source-cluster so we will continue and methods that will require the
                        # source-cluster will fail.
                    } # else go to the next vserver peer. If we dont find any then we will ignore adding the source_ci.
                }
            } # else the user did not pass in source vserver information so we cannot look this up.
        }
    }
    $Log->exit() if $may_exit;
}

1;
