#$Id: //depot/prod/test/nacldev/lib/NACL/MTask/SpareCore.pm#1 $
#
#   Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SpareCore Task Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::MTask::SpareCore

=head1 DESCRIPTION

NACL::MTask::SpareCore provides a consolidated method for obtaining a spare
core on an HA pair and uploading it to a shared location by using existing functionality
accessible via

C<NACL::C::SystemNodeCoredump>

C<NACL::C::SystemNodeCoredumpConfig>

C<NACL::C::Transit>

C<NACL::APIset>

The MTask will be concerned with -

    1) Ensure that the system dumps spare core.

    2) Force the system to panic.

    3) Dump the core and save the core.

    4) Upload corefile to remote location (Only for CMode).


=head1 ATTRIBUTES

=head2   command_interface

A component object that represents the host to which send commands to.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

Applicable for both Cmode and 7Mode.

=head2 partner_node_object

A NACL::C::Node type object that represents the command interface of the partner Node.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

Applicable for both Cmode and 7Mode.

=head2 transitObj

A transit object of type NACL::Transit used to take the filer to requisite states.
Applicable only for Cmode.

=head2 corefile

The name of the corefile which is saved.
Applicable for Cmode.

=head2 core_upload_location

Core upload location, if specified by the user in the call to new i.e, constructor.

Applicable for Cmode.

=head2 core_upload_server

Core upload server discovered using the site_info file, if the user
doesnot specify the core upload location in the call to new i.e, constructor.

Applicable for Cmode.


=cut

# Package definition
package NACL::MTask::SpareCore;

# Compiler directives
use strict;
use warnings;

# Inherit the coredump components
use base qw (NACL::MTask::MTask);

use Options;
use Params::Validate
  qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF);
use TestUtils qw(get_site_info);
use NACL::STask::Node;
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::ClusterHa;
use NACL::STask::StorageFailover;
use NACL::CS::SystemNodeCoredumpStatus;
use NACL::C::SystemNodeCoredumpConfig;
use NACL::C::SystemNodeCoredump;
use NACL::C::CommandInterface::ONTAP;
use NACL::MTask::_DumpCore qw(:all);

use constant PANIC_TO_WFG_TIMEOUT => 3600;
use constant DEFAULT_TIMEOUT      => 4200;
use constant CORETOOL_TIMEOUT     => 600;

=head1 METHODS

=head2  new

    (Cmode Method call)
    my $sparecoreobj = NACL::MTask::SpareCore->new(
        command_interface     => $Ci,
        delete_existing_cores => 0, # ( default => 1 )
        core_upload_location  => $core_upload_server
    );

    (7mode Method call)
    my $sparecoreobj = NACL::MTask::SpareCore->new(
        command_interface     => $Ci,
        delete_existing_cores => 0 , # ( default => 1 )
    );

(Class method) This method will instantiate a sparecore object and set its
attributes to their default values. It will set a core upload location(for CMode only)
and based on delete_existing_cores, it will delete any pre-existing cores.

=over

=item  Options

=over

=item C<< command_interface => $command_interface >>

(required) See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

A command interface object of type NACL::C::Node that will be used to send the commands.

Applicable for both Cmode and 7Mode.

=item C<< delete_existing_cores => $delete_existing_cores >>

(optional, default 1 ) Option for the user to indicate whether to delete the core files existing in the node.
By default, it takes a value 1 and hence deletes all the existing core files.

Applicable for both Cmode and 7Mode.

=item C<< core_upload_location => $core_upload_location >>

(optional) If the user passes the core upload location then that would be used to upload the core file.
Otherwise the location is discovered using the site.info and it is used to upload the core.

Applicable for Cmode.

=item C<< ctool_loc => $ctool_loc >>

(Optional) Coretool location.

Applicable for both Cmode and 7Mode.

=item C<< delete_cores_timeout => $delete_cores_timeout >>

(Optional) if the user passes delete_existing_cores as 1,
it is the Time in seconds to delete the existing core
Default : 1800

=item All of the other various options described in,
L<< NACL::C::SystemNodeCoredump::delete_all     |lib-NACL-C-SystemNodeCoredump-pm/delete_all >>

=back

=back

=over

=item Exceptions

=over

=item C<< NATE::BaseException >>

=back

=back

=cut

use Class::MethodMaker [
    new => ['_myconstructor'],
    scalar =>
      [ { -type => 'NACL::C::CommandInterface::ONTAP' }, 'command_interface' ],
    scalar => [
        { -type => 'NACL::C::CommandInterface::ONTAP' }, 'partner_node_object'
    ],
    scalar => 'core_upload_location',
    scalar => 'core_upload_server',
    scalar => 'site_info',
    scalar => 'transitObj',
    scalar => 'corefile',

];

sub new {

    $Log->enter() if $may_enter;

    # Grab the object handle from
    # the input array @_
    my ( $pkg, @args ) = @_;

    # Validate the input
    my %args = validate_with(
        params => \@args,
        spec   => {
            command_interface => { isa => 'NACL::C::CommandInterface::ONTAP' },
            core_upload_location => { type => SCALAR, optional => 1 },
            delete_existing_cores => { type => SCALAR, default => 1 },
            delete_cores_timeout  => { type => SCALAR, default => 1800 }
        }
    );

    my $handle_existing_cores = delete $args{'delete_existing_cores'};
    my $self                  = $pkg->_myconstructor(%args);

    # Cleanup pre-existing cores if the
    # flag is set
    if ( $handle_existing_cores == 1 ) {
        $self->_cleanup_cores('delete_cores_timeout' => $args{'delete_cores_timeout'});
        $self->cleanup_core_segment('delete_cores_timeout' => $args{'delete_cores_timeout'});
    }
    else {

        # Keep pre-existing cores if the flag
        # is not set and print a message
        $Log->comment( "delete_existing_cores was set to 0.\n"
              . "Keeping pre-existing cores ..." );
    }
    $Log->exit() if $may_exit;
    return $self;

}    # end new()

sub _myconstructor {

    $Log->enter() if $may_enter;

    # Grab the object handle from
    # the input array @_
    my ( $pkg, @args ) = @_;

    # Local variables
    my ( $core_upload_location, $hosttype, $self, $site, $upload_server, $ci,
        %site_info );

    # Validate the input
    my %args = validate_with(
        params => \@args,
        spec   => {
            command_interface => { isa => 'NACL::C::CommandInterface::ONTAP' },
            core_upload_location => { type => SCALAR, optional => 1 },
            delete_existing_cores => { type => SCALAR, default => 1 },
            delete_cores_timeout  => { type => SCALAR, default => 1800 }
        }
    );

    # Bless the class instance
    $self = bless {}, $pkg;

    # Pass the caller's command interface reference to the
    # Setter method

    $self->command_interface( $args{command_interface} );

    # Get the hosttype and site of the system
    # from the tharnhost file
    $ci       = $self->command_interface();
    $hosttype = $ci->hostrec()->hosttype();

    # If the system is a simulator, throw an exception
    # since SpareCore is not supported on simulators
    if ( $hosttype =~ /vsim/i || $hosttype =~ /vsa/i ) {
        NATE::BaseException->throw(
            'Sparecore is not supported on vsim and vsa');
    }

    # Extract Site information
    $site = $ci->hostrec()->SITE();
    $Log->debug("This is the site $site");

    # If the user specified an upload server
    # use it for the upload
    if ( defined $args{core_upload_location} ) {

        # store the user-specified upload location
        # into the sparecore object
        $self->core_upload_location( $args{core_upload_location} );
        $core_upload_location = $self->core_upload_location();
        $Log->comment(
"The user specified core upload location will be used : $core_upload_location\n"
        );

    }

    # If the user didn't specify an upload server,
    # Fetch the dump server for the system's site.
    else {

        $Log->comment("Reading the contents of the site info file...\n");
        my $options = new Options();
        $options->process();

        eval {
            %site_info = get_site_info();
            $self->site_info( \%site_info );
        };

        if ($@) {
            NATE::BaseException->throw(
                'Unable to read the contents of the site info file');
        }

        # Read the dump server from the site info file
        # and pass it to the sparecore object
        my $site_info_obj = $self->site_info();
        $self->core_upload_server( $site_info_obj->{$site}->{netboot_server} );

        $core_upload_location = $self->core_upload_server();

        # If there is no entry for the dump server
        # throw an exception
        if ( !defined $core_upload_location ) {
            NATE::BaseException->throw(
                'Unable to find the dump server for site $site');
        }

        # Set the upload server to the dump server that
        # was fetched in from the site info file
        if ( defined $core_upload_location ) {
            $Log->comment(
"The default core upload location for site $site will be used : $core_upload_location\n"
            );

        }

    }    # end if()

    # populate the partner object here
    my $partner_obj =
      NACL::STask::Node->get_partner_obj(
        command_interface => $self->command_interface() );
    $self->partner_node_object($partner_obj);

    $Log->exit() if $may_exit;
    return $self;

}    # end new()

=head2 start

    my $sparecoreobj = NACL::MTask::SpareCore->start(command_interface => $command_interface);

    or

    $sparecoreobj->start();

(Class or Instance method) Start the sparecore operation .

    Steps performed are:
    1) Verify whether disks are available that support spare dumpcore.
    2) Perform the configuration required in the filer to dump a sparecore.
        It includes the following steps:
            Modify coredump related attributes.
            Modify storage failover related attributes.
    3) Panic the filer to generate the core file.
    4) Wait untill 'automatic' save core is done from the partner node.
    5) Wait for giveback to complete.
    6) Display core metadata after save and verify the core with coretool.
    7) Upload the core file to the upload location per user's option. (Only for CMode)

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
A component object that represents the host to send commands.

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

Applicable for both Cmode and 7Mode.

=item C<< upload => boolean >>

(optional) Default is 1.
If set to 0, the upload operation will not happen.

Applicable for Cmode.

=item C<< core-tool-loc => $core_tool_loc >>

(optional) Core tool location

Applicable for both Cmode and 7Mode.

=item C<< file-path => $file_path >>

(optional) Corefile location

Applicable for both Cmode and 7Mode.

=item C<< core-tool-path => $core_tool_path >>

(optional) Core tool utility path, where it will get downloaded.

Applicable for both Cmode and 7Mode.

=item C<< nacltask_verify => $nacltask_verify >>

(optional) Default is 1.
If set to 1, run_coretool() will be called and corefile will get verified.

If set to 0, verification will not be performed.

Applicable for both Cmode and 7Mode.

=back

=back

=over

=item Exceptions

=over

=item C<< NACL::APISet::Exceptions::MissingRequiredParamException >>

=item C<< NACL::APISet::Exceptions::CommandFailedException >>

=item C<< NATE::BaseException >>

=item All of the other various options described in,

L<< NACL::C::SystemNodeCoredump::save_all       |lib-NACL-C-SystemNodeCoredump-pm/save_all >>
L<< NACL::C::SystemNodeCoredump::delete_all     |lib-NACL-C-SystemNodeCoredump-pm/delete_all >>
L<< NACL::C::SystemNodeCoredumpConfig::modify   |lib-NACL-C-SystemNodeCoredumpConfig-pm/modify >>
L<< NACL::CS::SystemNodeCoredumpStatus::fetch   |lib-NACL-CS-SystemNodeCoredumpStatus-pm/fetch >>
L<< NACL::C::StorageFailover::modify            |lib-NACL-C-StorageFailover-pm/modify >>
L<< NACL::CS::StorageFailover::fetch            |lib-NACL-CS-StorageFailover-pm/fetch >>
L<< NACL::APISet::Host::CLI::Freebsd::default->coretool|lib-NACL-APISet-Host-CLI-Freebsd-default-coretool-pm/coretool >>

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;

    # Grab the object handle from
    # the input array @_
    my ( $self, @args ) = @_;

    # Validate the input
    my %args = validate_with(
        params => \@args,
        spec   => {
            upload           => { type => BOOLEAN, default  => 1 },
            'core-tool-loc'  => { type => SCALAR,  optional => 1 },
            'file-path'      => { type => SCALAR,  optional => 1 },
            'core-tool-path' => { type => SCALAR,  optional => 1 },
            nacltask_verify  => { type => BOOLEAN, default  => 1 },
        }
    );

    # check cluster ha and storage failover states.
    # Enable the same for spare core generation. Also set
    # save-onstartup to true
    $self->_check_config();
    $self->_check_config( command_interface => $self->partner_node_object() );

# Ensure that unless the storage failover values are correct then the rest of the test won't succeed
    foreach ( $self->{command_interface}, $self->partner_node_object() ) {
        my $buf = NACL::CS::StorageFailover->fetch(
            command_interface => $_,
            filter            => { node => $_->node() },
            requested_fields =>
              [qw(enabled onpanic auto-giveback auto-giveback-after-panic)]
        );
        NATE::BaseException->throw(
            "Script failed !! SFO should be enabled before panic")
          if ( $buf->enabled() =~ /false/i );
        NATE::BaseException->throw(
            "Script failed !! auto-giveback should be disabled before panic")
          if ( $buf->auto_giveback() =~ /true/i );
        NATE::BaseException->throw(
"Script failed !! auto-giveback-after-panic should be disabled before panic"
        ) if ( $buf->auto_giveback_after_panic() =~ /true/i );
        NATE::BaseException->throw(
            "Script failed !! onpanic should be enabled before panic")
          if ( $buf->onpanic() =~ /false/i );
    }

    # Panic the system
    $self->_panic();

    # Wait for the savecore
    # operation to finish
    $self->_wait_for_savecore(
        command_interface => $self->partner_node_object(),
        node              => $self->_get_node()
    );

    # Reboot the node after
    # the panic
    $self->_wait_for_reboot();

    # Run coretool and verify
    $args{'method-timeout'} ||= CORETOOL_TIMEOUT;
    NACL::MTask::_DumpCore->run_coretool(
        'method-timeout'  => $args{'method-timeout'},
        'core-tool-loc'   => $args{'core-tool-loc'},
        'core-id'         => $self->{corefile},
        'file-path'       => $args{'file-path'},
        'core-tool-path'  => $args{'core-tool-path'},
        command_interface => $self->{command_interface},
    ) if ( $args{nacltask_verify} );

    # Upload the core file if the flag
    # is set
    $self->_upload()
      if ( ( $args{upload} == 1 )
        && ( $self->{command_interface}->is_cmode() ) );
    $Log->exit() if $may_exit;
}    # end start()

###########################################################
# Routine : _check_config()                               #
# Description : Local subroutine to set the appropriate   #
# options for sparecore.                                  #
# Returns : Nothing                                       #
# Dependencies : CMode only                               #
###########################################################
sub _check_config {
    $Log->enter() if $may_enter;

    # Grab the object handle from
    # the input array @_
    my ( $self, %opts ) = @_;

    # Local variables
    my $buf              = undef;
    my $cluster_ha_state = undef;
    my $coredump_type    = undef;
    my $node             = $self->_get_node();
    my $save_on_startup  = undef;
    my $ci = $opts{command_interface} || $self->command_interface();

    # check whether hostname is defined in the tharnhost

    if ( !defined $node ) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(
            'hostname is not defined in the tharnhost file of the the system');
    }
    $Log->comment( "Setting AUTOBOOT true in Node " . $ci->node() );
    my $mode = $ci->mode();
    my $set;
    if ( $mode eq '7Mode' ) {
        $set = '7Mode';
    }
    else {
        $set = 'Nodescope';
    }
    my $node_api = $ci->apiset(
        interface => 'CLI',
        category  => 'Node',
        set       => $set
    );

    $node_api->bootargs_set(
        arg   => 'AUTOBOOT',
        value => 'true'
    );

    if ( $ci->is_cmode() ) {
        $Log->comment("Verifying the state of cluster ha on node $node");
        $buf = NACL::CS::ClusterHa->fetch(
            command_interface => $ci,
            requested_fields  => [qw( configured )]
        );
        $cluster_ha_state = $buf->configured();

        if ( $cluster_ha_state eq 'true' ) {

            $Log->comment("Cluster ha was enabled, disabling it...");
            NACL::C::ClusterHa->modify(
                command_interface => $ci,
                configured        => 'false'
            );

        }
        elsif ( $cluster_ha_state eq 'false' ) {

            # If cluster ha is already enabled proceed
            $Log->comment("Cluster ha was already disabled on node $node");
        }
        else {
            NATE::BaseException->throw(
"Cluster ha is neither true or false on node $node, it is : $cluster_ha_state"
            );
        }
    }

    # Get the state of storage failover and modify based on the value
    # Reset $buf before using it for storage failover
    $buf = undef;

    $Log->comment("Verifying the state of storage failover on node $node");
    $buf = NACL::CS::StorageFailover->fetch(
        command_interface => $ci,
        filter            => { node => $ci->node() },
        requested_fields =>
          [qw(enabled onpanic auto-giveback auto-giveback-after-panic)]
    );

    if ( $buf->enabled() eq 'false' ) {
        $Log->comment("Storage failover was disabled, enabling it...");
        NACL::C::StorageFailover->modify(
            command_interface           => $ci,
            node                        => $ci->node(),
            enabled                     => 'true',
            'auto-giveback'             => 'false',
            'auto-giveback-after-panic' => 'false',
        );
    }

    if ( $buf->onpanic() eq 'false' ) {
        $Log->comment("Setting takeover on panic to true ..");
        NACL::C::StorageFailover->modify(
            command_interface => $ci,
            node              => $ci->node(),
            onpanic           => 'true',
        );
    }

    if ( $buf->auto_giveback() eq 'true' ) {
        $Log->comment("Setting auto-giveback to false ..");
        NACL::C::StorageFailover->modify(
            command_interface => $ci,
            node              => $ci->node(),
            'auto-giveback'   => 'false',
        );
    }
    if ( $buf->auto_giveback_after_panic() ) {
        if ( $buf->auto_giveback_after_panic() eq 'true' ) {
            $Log->comment("Setting takeover on panic to true ..");
            NACL::C::StorageFailover->modify(
                command_interface           => $ci,
                node                        => $ci->node(),
                'auto-giveback-after-panic' => 'false',
            );
        }
    }
    $Log->comment(
        "Verifying that the node has enough disk space for sparecore");

    # Reset $buf before using it for coredump status

    $buf = undef;
    $buf = NACL::CS::SystemNodeCoredumpStatus->fetch(
        command_interface => $ci,
        filter            => { node => $ci->node() },
    );

    # Get the coredump type to make sure it is spare
    $coredump_type = $buf->type();
    if ( $coredump_type =~ /spare/i ) {
        $Log->comment("Sparecore is enabled on node $node");
    }
    else {
        NATE::BaseException->throw(
            'Coredump type for this node is not "spare" ');
    }

    # Reset $buf before using it for
    # CoredumpConfig
    $buf = undef;

    # Get the value of save-on-startup

    if ( $ci->is_cmode() ) {
        $buf = NACL::CS::SystemNodeCoredumpConfig->fetch(
            command_interface => $ci,
            filter            => { node => $ci->node() }
        );
        $save_on_startup = $buf->save_onstartup();
        if ( $save_on_startup eq 'false' ) {
            $Log->comment(
                "save-on-startup was set to false. Setting it to true...");
            NACL::C::SystemNodeCoredumpConfig->modify(
                command_interface => $ci,
                node              => $ci->node(),
                'save-onstartup'  => 'true'
            );
        }

        # If save-on-startup is already set to true, proceed
        elsif ( $save_on_startup eq 'true' ) {
            $Log->comment("save-on-startup was already enabled");
        }
    }
    else {
        try {
            NACL::C::SystemNodeCoredumpConfig->modify(
                command_interface   => $ci,
                node                => $ci->node(),
                'coredump-attempts' => 2,
            );
            NACL::C::SystemNodeCoredumpConfig->modify(
                command_interface        => $ci,
                node                     => $ci->node(),
                'coredump-metadata-only' => 'on',
            );
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception = shift;
            if ( $exception->get_response_object()->get_raw_output() !~
                /No such option/i )
            {
                $Log->exit() if $may_exit;
                $exception->throw();
            }
        };
    }
    $Log->exit() if $may_exit;
}    # end _check_config()

###########################################################
# Routine : _wait_for_reboot()                            #
# Description : local subroutine to reboot the node       #
# Args : NONE                                             #
# Returns : Nothing                                       #
# Dependencies : CMode only                               #
###########################################################
sub _wait_for_reboot {
    $Log->enter() if $may_enter;

    # Grab the object handle from
    # the input array @_
    my $self = shift;

    # Local variables
    my $node = $self->_get_node();

    my $obj = NACL::Transit->new( name => $node );
    $self->transitObj($obj);
    my $transitObj = $self->transitObj();

    # Wait for this node to get to the WFG prompt
    $Log->comment("Changing state of node $node to WAIT FOR GIVEBACK...");

    $transitObj->change_state(
        to      => "TAKEN_OVER_WAITING_ONTAP",
        timeout => PANIC_TO_WFG_TIMEOUT
    );

    # Ensure giveback from partner
    NACL::STask::StorageFailover->giveback(
        command_interface => $self->partner_node_object(),
        node              => $self->partner_node_object(),
        partner           => $self->command_interface(),
        'override-vetoes' => 'true',
        'method-timeout'  => '900'
    );
    Tharn::snooze(180);
    my $endtime = time() + DEFAULT_TIMEOUT;

    while (1) {
        my $sfo_obj = NACL::CS::StorageFailover->fetch(
            command_interface => $self->partner_node_object(),
            filter => { node => $self->partner_node_object()->node() },
            requested_fields => [qw(enabled partner-state)]
        );
        if ( time() > $endtime ) {
            $Log->exit() if $may_exit;
            NACL::APISet::Exceptions::TimeoutException->throw(
                    "Machine did'nt come to UP state with in "
                  . DEFAULT_TIMEOUT
                  . "seconds" );
        }

        # Get the state of storage failover
        if (   ( $sfo_obj->enabled() =~ /true/i )
            && ( $sfo_obj->partner_state() =~ /UP/i ) )
        {
            last;
        }
        else {
            $Log->comment("Wait for the machine to reach UP state");
            Tharn::snooze(60);
        }
    }
    $self->command_interface()->refresh_command_interface();
    $Log->exit() if $may_exit;
}    # end _reboot()

###########################################################
# Routine : _wait_for_savecore()                          #
# Description : Local subroutine to wait for the savecore #
# operation to finish.                                    #
# Returns : Nothing                                       #
# Dependencies : CMode only                               #
###########################################################
sub _wait_for_savecore {

    $Log->enter() if $may_enter;

    # Grab the object handle from
    # the input array @_
    my ( $self, %opts ) = @_;

    # Local variables
    my ( $corefile, $output ) = undef;

    my $ci = $opts{command_interface} || $self->command_interface();
    my $node = $opts{node} || $ci->node();

    # wait for savecore to start

    $Log->comment("Waiting for savecore to finish");

    # check for takeover
    my ( $cd_obj, $corename );
    my $endtime = time() + DEFAULT_TIMEOUT;
    if ( $ci->is_cmode() ) {
        while (
            NACL::CS::SystemNodeCoredumpStatus->fetch(
                command_interface => $ci,
                filter            => { node => $ci->node() }
            )->corename() eq "-"
          )
        {

            if ( time() > $endtime ) {
                $Log->exit() if $may_exit;
                NACL::APISet::Exceptions::TimeoutException->throw(
                        "Core File save did not start in "
                      . DEFAULT_TIMEOUT
                      . "seconds" );
            }

            Tharn::snooze 1;
        }
        $Log->comment("Started corename changed..");
        $cd_obj = NACL::CS::SystemNodeCoredumpStatus->fetch(
            command_interface => $ci,
            filter            => { node => $ci->node() }
        );

        $corename = $cd_obj->corename();
        $corefile = $corename;
        $endtime  = time() + DEFAULT_TIMEOUT;

        # Keep waiting for savecore to finish
        while (
            NACL::CS::SystemNodeCoredumpStatus->fetch(
                command_interface => $ci,
                filter            => { node => $ci->node() }
            )->state() eq "saving"
          )
        {
            $Log->debug("Core file $corefile is being saved...");

            if ( time() > $endtime ) {
                $Log->exit() if $may_exit;
                NACL::APISet::Exceptions::TimeoutException->throw(
                        "Core File save did not complete in "
                      . DEFAULT_TIMEOUT
                      . "seconds" );
            }

            # Sleep for 2s before polling
            Tharn::snooze 2;

        }
    }
    else {

        # Create console connection to execute partner commands
        Tharn::snooze 30
          ;    ## After panic, partner should come to takeover prompt.
               # Small delay will be there between this operations.
        my $host       = host( $ci->node() );
        my $connection = connect("#telnet");

        my $api_set_obj = NACL::APISet->new(
            connection => $connection,
            category   => "Node",
            interface  => "CLI",
            set        => "7Mode",
        );

        $api_set_obj->set_timeout( 'connectrec-timeout' => 120 );
        $endtime = time() + DEFAULT_TIMEOUT;
        while (1) {

            my $sfo_obj = NACL::CS::StorageFailover->fetch(
                command_interface => $ci,
                requested_fields  => [qw(possible)]
            );

            last if ( $sfo_obj->possible() eq 'true' );
            if ( time() > $endtime ) {
                $Log->exit() if $may_exit;
                NACL::APISet::Exceptions::TimeoutException->throw(
                        "Partner didnt come to takeover state with in "
                      . DEFAULT_TIMEOUT
                      . "seconds" );
            }

        }
        $endtime = time() + ( 3 * DEFAULT_TIMEOUT );
        Tharn::snooze 60
          ; ## After coming to takeover prompt , some delay will be there before starting coredump operation
        $output =
          $api_set_obj->execute_raw_command( command => "partner savecore -s" );

        while ( $output =~
/Waiting for partner dump to complete|Preparing to save core|No savecore in progress|Saving.*(core.*nz)/i
          )
        {
            if ($1) {
                $corefile = $1 if ( !$corefile );
            }
            Tharn::snooze 10;
            if ( time() > $endtime ) {
                $Log->exit() if $may_exit;
                NACL::APISet::Exceptions::TimeoutException->throw(
                        "Core File save did not start in "
                      . DEFAULT_TIMEOUT
                      . "seconds" );
            }
            $output = $api_set_obj->execute_raw_command(
                command => "partner savecore -s" );
        }
    }
    if ( !$corefile ) {
        NATE::BaseException->throw('There is no core being saved');
    }
    $Log->comment("Savecore finished running");
    $Log->comment(
        "The last core file that was generated and saved is $corefile");
    $self->corefile($corefile);
    $Log->exit() if $may_exit;
}    # end _wait_for_savecore()

=head2 run_coretool

    #Method Call
    NACL::MTask::SpareCore->run_coretool(
        'command_interface' => $command_interface,
        'core_id'           => $core_id,
    );

(Class method)This method is used to call the core tool from the SystemShell .
It will take core-id as the input and perform the below steps.
1) Check whether coretool utility is installed or not.
   If it is not installed , then by using wget, coretool utility will be downloaded and installed.
2) Change permission to run on a core file.
3) Run coretool on a core file and verify its integrity.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) As NACL::C::CommandInterface::ONTAP . A component object that represents the host to which to send commands.

=item C<< core_id => $core_id >>

(Required) Name of the core file.

=item C<< protocol => $protocol >>

(Optional) Protocol to be used while downloading coretool utility.

=item C<< user => $user >>

(Optional) Username for authenticating during download of coretool utility.

=item C<< password => $password >>

(Optional) Password for authenticating during download of coretool utility.

=item C<< server => $server >>

(Optional) IP Adress of the server from where the coretool utility has to be downloaded.

=item C<< ctool_loc => $ctool_loc >>

(Optional) Coretool location.

=item C<< arch => $arch >>

(Optional) Type of architecture of Host/Filer.

=item apiset_must, apiset_should, method-timeout,
all the other options supported by
coretool method in coretool.pm see L<< NACL::APISet::Host::CLI::Freebsd::default->coretool|lib-NACL-APISet-Host-CLI-Freebsd-default-coretool-pm/coretool >>

=back

=back

=cut

=head2 cleanup_core_segment

    #Method Call
    NACL::MTask::_DumpCore->cleanup_core_segment(
        'command_interface'     => $command_interface,
        'core_segment_name'     => $core_segment_name,
    );

(Class method)This method deletes core segment that got created during the panic.
It will take core_segment_name as the input.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) As NACL::C::CommandInterface::ONTAP . A component object that represents the host to which to send commands.

=item C<< core_segment_name => $core_segment_name >>

(Required) Name of the core segment.

=item C<< nacltask_verify => 1  >>

(Optional) would verify that the segment got deleted , if set to 1.
By default it is set to 0.

=item All of the other options like corename, type, node, coreid, segment, owner-node are described in
L<< NACL::C::SystemNodeCoredump.pm->delete|lib-NACL-C-SystemNodeCoredump-pm/delete >>
L<< NACL::C::SystemNodeCoredumpSegment.pm->delete|lib-NACL-C-SystemNodeCoredumpSegment-pm/delete >>

=back

=back

=cut

1;
