#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::MTask::VolumeFileIterator;

use Moose;
use MTF::MooseCompatibleException qw(:all);
use Moose::Util::TypeConstraints;
use File::Basename;

use NACL::MTask::VolumeFileIterator::UnitTestFile;
use NACL::MTask::VolumeFileIterator::ZapiFile;
#use NACL::MTask::VolumeFileIterator::RemoteNASFile; @TODO add in phase 2
use NACL::MTask::VolumeFileIterator::LocalNASFile;
use NACL::Exceptions::NoElementsFound;

# A volume is required so we can get the command interface and vserver for
# building NACL::C::VolumeFiles. It is also useful if we will do client-side
# iteration.
has 'volume' => (
    is => 'ro',
    isa => 'Object',
);

# The initial path to start iteration at. It defaults to /, but can be set to
# any directory, including things like /.snapshot/snapname/dirname
has 'path' => (
    is => 'ro',
    isa => 'Str',
    default => '/',
);

#
# mount object, needed if user wants to use NAS client file iteration
#
has 'mount' => (
    is => 'ro',
    isa => 'Object',
);

#
# Handle directory stack changes or stale nfs handle issue
#
has 'handle_tree_frame_changes' => (
    is => 'ro',
    isa => 'Bool',
    default => 1,
);

#
# we are planing to use different ways to do the voluem file iterating.
# ZAPI, volume explorer(wex), NFS via linux client, SMB via windows client.
# the method attribute will allow the caller choose the method.
# the modular will use ZAPI by default.
#
enum 'Method_Types', [qw(unit_test zapi wex remote_nas local_nas)];

has 'method' => (
   is => 'rw',
   isa => 'Method_Types',
   default => 'zapi',
);

#
# Subtypes for criteria key validation
#
subtype 'criteria_key' => (
    as 'ArrayRef[Str]'
);

#
#    Recognizes keys:
#    'exclude_files' => ArrayRef - files to avoid in the iteration
#    'exclude_types' => ArrayRef - file types (eg 'directory') to avoid in the iteration
#

has criteria => (
    is => 'rw',
    isa => 'HashRef[criteria_key]',
    required => 0,
    default => sub {
        return { }
    },
);

has collection => (
    is => 'rw',
    isa => 'ArrayRef[Object]', 
);

#
# This will store a stack of directory contents. The bottom stack frame will be
# the contents of the directory that we started from. The top stack frame will
# be the directory currently serving files. Files will be removed from the top
# stack frame as they are consumed. The type of the objects in the directory
# listing will vary depending on which method is being used to iterate, but each
# object is expected to support a standard interface.
#
# e.g.
# /dir1
# /dir1/sub_dir1
# /dir1/sub_dir1/file1
# /dir1/sub_dir1/file2
# /dir1/sub_dir2
#
# [
#   [
#       UnitTestFile('/dir1/sub_dir1/file1'), <-- file1 will be returned next
#       UnitTestFile('/dir1/sub_dir1/file2'),
#   ],
#   [
#       UnitTestFile('/dir1/sub_dir2')  <-- sub_dir1 was pop'ed off already
#   ],
#   [
#       <-- dir1 was pop'ed off to make the above
#   ],
#
# ]
#
has '_dirstack' => (
    is => 'rw',
    isa => 'ArrayRef',
    default => sub { return []; },
);

# To support has_next, we need to have the next file ready to go. This means
# that the next method will just return this file and ensure it is refilled.
has '_next_file' => (
    is => 'rw',
    isa => 'Object',
    predicate => '_has_next_file',
    clearer => '_clear_next_file',
);


sub BUILD {
    $Log->enter();
    my $self = $_[0];
    my $method = $self->method();
    my $path = $self->path();
    my $collection = $self->collection();
    if ( defined($collection) ) {
        $self->_push_dirstack($collection);
        $self->_refill_next_file();
        $Log->exit();
    } else {
        my $file;
        if ($method eq 'unit_test') {
            $file = NACL::MTask::VolumeFileIterator::UnitTestFile->new(
                path => $path,
                volume => $self->volume(),
            );
        }
        if ($method eq 'zapi') {
            $file =NACL::MTask::VolumeFileIterator::ZapiFile->new(
                path => $path,
                volume => $self->volume(),
            );
        }

        #if ($method eq 'remote_nas') {
        #    $file = NACL::MTask::VolumeFileIterator::RemoteNASFile->new(
        #        path => $path,
        #        volume => $self->volume(),
        #        mount => $self->mount(),
        #    );
        #}
        if ($method eq 'local_nas') {
            $file = NACL::MTask::VolumeFileIterator::LocalNASFile->new(
                path => $path,
                volume => $self->volume(),
                mount => $self->mount(),
            );
        }
        my @directory_frame = $file->enumerate_directory();
        @directory_frame = $self->_filter_files(@directory_frame);
        $self->_push_dirstack(\@directory_frame);
        $self->_refill_next_file();
        $Log->exit();
    }
}

sub _filter_files {
    $Log->enter();
    my $self = shift;
    my @files = @_;
    my @acceptable_files = ();
    my @exclude_files;
    if (exists $self->criteria()->{exclude_files}) {
        @exclude_files = @{$self->criteria()->{exclude_files}};
    }
    my @exclude_types;
    if (exists $self->criteria()->{exclude_types}) {
        @exclude_types = @{$self->criteria()->{exclude_types}};
    }
    foreach my $file (@files) {
        my $excluded = 0;
        foreach my $exclude_file (@exclude_files) {
            my $file_name = basename($file->raw_name());
            if ( $file_name =~ /^$exclude_file$/ ) {
                $excluded = 1;
            }
        }
        foreach my $exclude_type (@exclude_types) {
            if ($file->standard_type() =~ qr/$exclude_type/) {
                $excluded = 1;
            }
        }
        if (!$excluded) {
            push @acceptable_files, $file;
        }
    }
    $Log->exit();
    return @acceptable_files;
}

sub _done_iterating {
    my $self = $_[0];
    if (!scalar(@{$self->_dirstack()})) {
        return 1;
    }
    return 0;
}

# Use shift/unshift so the element to access is [0]
sub _pop_dirstack {
    my $self = $_[0];
    return shift @{$self->_dirstack()};
}

sub _push_dirstack {
    my $self = $_[0];
    unshift @{$self->_dirstack()}, $_[1];
}

# If there are remaining files in the directory it will remove the 0th file and
# return that. If there are no files, it will return undef.
sub _next_file_in_directory {
    $Log->enter();
    my $self = $_[0];
    my $current_directory = $self->_dirstack()->[0];
    if (!$current_directory) {
        $Log->exit();
        return;
    }
    my $file = shift @{$current_directory};
    $Log->exit();
    return $file;
}

# Get the next file in the directory tree. This works by looking at the top
# directory frame in the list of stack frames. It will attempt to remove a file
# from that directory and return it. If the top directory frame is empty, it
# will pop that frame and recursively call the _pop_file. If we get to the end
# of the last directory stack frame, we will return undef. Directories will not
# be returned, only files in a directory.
sub _pop_file {
    my $self = $_[0];

    my $file = $self->_next_file_in_directory();
    if (!$file) {
        # No more directories are left
        if ($self->_done_iterating()) {
            return undef;
        }
        # Go up to the prior directory
        $self->_pop_dirstack();
        return $self->_pop_file();
    }

    if ($file->is_directory()) {
        my @directory_frame;
        if ($self->handle_tree_frame_changes()) {
            try {
                @directory_frame = $file->enumerate_directory();
            } catch NATE::BaseException with_handler {
                my $e = shift;
                # Our stack has been hijacked by the io tools or
                # for whatever reason the file/dir is gone lets
                # pop until we go back to dir that is there.
                if ($e->text() =~ /Stale NFS file handle/i ) {
                    return $self->_pop_file();
                }elsif ($e->text() =~ /changed since last listed/i ) {
                    return $self->_pop_file();
                }
            };
        }else {
            @directory_frame = $file->enumerate_directory();
        }
        # Filter the frame against excluded types and files
        @directory_frame = $self->_filter_files(@directory_frame);
        $self->_push_dirstack(\@directory_frame);
        return $self->_pop_file();
    }

    return $file;
}

sub _refill_next_file {
    $Log->enter();
    my $self = $_[0];
    my $file;
    if ( defined($self->collection()) ) {  
        $file = $self->_next_file_in_directory();
    } else {
        $file = $self->_pop_file();
    }
    if ($file) {
        $self->_next_file($file);
    }
    $Log->exit();
}

sub has_next {
    $Log->enter();
    my $self = $_[0];
    if ($self->_has_next_file()) {
        $Log->exit();
        return 1;
    }
    $Log->exit();
    return 0;
}

sub next {
    $Log->enter();
    my $self = $_[0];
    if (!$self->_has_next_file()) {
        $Log->exit();
        NACL::Exceptions::NoElementsFound->throw(
            "Called next and no elements were remaining"
        );
    }
    my $return_file = $self->_next_file();
    $self->_clear_next_file();
    $self->_refill_next_file();
    # A NAS client based file may not have the proper command_interface to make
    # a NACL::C::VolumeFile instance. It would have a command_interface for a
    # client, but not necessarily a filer command_interface.
    $Log->exit();
    if ( defined($self->collection()) ) {
        return $return_file;
    } else {
        return $return_file->make_volume_file();
    }
}

sub rewind {
    my $self = $_[0];
    $Log->enter();
    $self->BUILD();
    $Log->exit();
}

1;

=head1 NAME

NACL::MTask::VolumeFileIterator - Iterate over the files in a
volume. Returns NACL::C::VolumeFile instances for the files.

=head1 SYNOPSIS

    use NACL::MTask::VolumeFileIterator;

    my $Iterator = NACL::MTask::VolumeFileIterator->new(
        volume => $Volume,
        path => $Path,
    );

    while ($Iterator->has_next()) {
        my $elem = $Iterator->next();
        $Log->comment($elem->path()." ".$elem->inode());
    }



=head1 DESCRIPTION

This library will iterate over all files in the volume and return hash
references representing each file.

=head1 FUNCTIONS

=head2 has_next

Returns 1 if there is another element to be fetched by next
Arguments: none

=head3 Returns

1 if another element is available for next. 0 if none
is available

=head2 next

Returns the next element if available.

Arguments: none

=head3 Returns

This returns a NACL::C::VolumeFile instance representing a file.

=head2 rewind

Returns the iterator to the start of the collection
Arguments: none

=head3 Returns

none

=head2 new

Build a new iterator object instance

=over

=head3 Arguments

=over

=item C<< volume >> - (optional) NACL::C::Volume instance to iterate over
collection and volume are mutually exclusive

=item C<< path >> - (optional) A path to start iterating at. It defaults to '/',
but could be something like /.snapshot/snapname/dirname.

=item C<< criteria >> - (optional) A hashref specifying filter criteria.

    {
        exclude_files => [ 'pattern1', 'pattern2' ],
        exclude_types => [ 'symlink' ],
    }

=item C<< method >> - (optional) can be unit_test zapi wex remote_nas local_nas

=item C<< mount >> - (optional) Mount object, required if using remote_nas/local_nas as methods

=item C<< handle_tree_frame_changes >> - (optional) handle dir frame changes

=item C<< collection >> - (optional) A arrayref specifying collection of volume file objects
collection is mutually exclusive of all other arguments specified

=back

=back

=head3 Returns

object instance for the iterator


=head1 SEE ALSO

=cut
