
# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.

## @summary Setup Exception Matcher
## @author  Adam.Ciapponi@netapp.com
## @pod     here

=head1 NAME

NACL::Matchers::ExceptionMatcher

=head1 DESCRIPTION

This module implements a matcher that determines whether it matches by looking at whether or not an exception was thrown.

=head1 METHODS

=cut

package NACL::Matchers::ExceptionMatcher;

# Compiler directives.
use strict;
use warnings;

use feature qw(switch);

use CR::Setup::Constants;
use CR::Setup::Exception;
use Data::Dumper;
use NATE::BaseException;
use Params::Validate qw(:all);
use Scalar::Util qw(blessed);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

our $Exception_Logging_Verbosity = 'verbose';

use base qw(NACL::Matcher);

=head2 matches

Determines matching against whether or not an exception was thrown and if it was thrown was it the correct exception returning a boolean value.

=cut

sub matches {
    validate_pos(@{[@_[1..$#_]]}, NACL::Matcher::matcher_value_validate_spec());

    # NOTE: This doesn't use the generalized setting actual from the matcher value
    # because it basically has to roll it's own implementation for raw values. Since
    # this is the only one with this issue, it probably doesn't make sense to complicate
    # the MatcherTarget library.

    if ( ref($_[1]->value_raw()) ne 'CODE' ) {
        CR::Setup::Exception->throw(join("\n", 'Target specified for ExceptionMatcher was not a code block:', Dumper($_[1]->value_raw())));
    }

    $Log->log(
        'Executing code block and expecting exception to be thrown.')
        if ( $_[0]->{'expected'}->{'status'} eq 'failed' );
    $_[0]->{'actual'} = { 'status' => 'passed' };
    $_[0]->{'expected'}->{'exception'} = $_[0]->{'exception'} if $_[0]->{'exception'};
    $_[0]->{'expected'}->{'exact_error_message'} = $_[0]->{'exact_error_message'} if $_[0]->{'exact_error_message'};
    $_[0]->{'expected'}->{'error_message_like'} = $_[0]->{'error_message_like'} if $_[0]->{'error_message_like'};
    # Because we roll our own try/catch logic for performance reasons, we need to handle
    # die in the block similarly. Otherwise we lose stack frames for die() inside the
    # block.
    local $SIG{__DIE__} = \&_die_handler;
    # Try
    local $@;
    eval {
        $_[1]->value_raw()->();
    };
    my $exception = $@;
    if ($exception) {
        undef $NATE::Error::THROWN;
        # Otherwise
        $Log->log('Exception was thrown.');
        $Log->log('Exception reference type: ' . ref($exception));
        # NATE can sometimes die with a NATE::Result instead of an exception for RPC
        my $exception_has_text_method = $exception->can('text');
        given($Exception_Logging_Verbosity) {
            when('verbose') {
                $Log->log("Exception message:\n" . $exception->text()) if $exception_has_text_method;
                $Log->debug("Exception:\n" . Dumper($exception) ) if $Log->may_debug;
            }
            when('truncate') {
                $Log->log("Exception message:\n" . (split("\n", $exception->text()))[0]) if $exception_has_text_method;
            }
        }
        $_[0]->{'actual'}->{'status'}    = 'failed';
        $_[0]->{'actual'}->{'exception'} = $exception;
        $_[0]->{'actual'}->{'message'}   = $exception_has_text_method ? $exception->text : '';
        my $exception_return_ref = $_[0]->{'and_return_exception'};
        ${$exception_return_ref} = $exception if exists $_[0]->{'and_return_exception'};
        my $error_message_return_ref = $_[0]->{'and_return_error_message'};
        ${$error_message_return_ref} = $exception->{'-text'} if exists $_[0]->{'and_return_error_message'};
        &{ $_[0]->{'otherwise'} } if defined $_[0]->{'otherwise'};
    }
    # Finally
    &{ $_[0]->{'finally'} } if defined $_[0]->{'finally'};
    if ( $_[0]->{'actual'}->{'status'} ne $_[0]->{'expected'}->{'status'} ) {
        $Log->debug('Does not match because it does not match the expected result status.') if $Log->may_debug();
        return $FALSE;
    }
    if ( defined $_[0]->{'exception'} ) {
        my $expected_exception = $_[0]->{'exception'};
        if ( !ref($expected_exception) ) {
            # Expected exception is a SCALAR
            if (
                ! $_[0]->{'actual'}->{'exception'}->isa($expected_exception) )
            {
                $Log->debug('Does not match because it is not the expected exception.') if $Log->may_debug();
                return $FALSE;
            }
        }
        else {
            # Expected exception is an ARRAY
            my $matches_exception = $FALSE;
            foreach my $ex ( @{ $_[0]->{'exception'} } ) {
                if ( $_[0]->{'actual'}->{'exception'}->isa($ex) ) {
                    $matches_exception = $TRUE;
                    last;
                }
            }
            if ( ! $matches_exception) {
                $Log->debug('Does not match because it is not in the list of expected exceptions.') if $Log->may_debug();
                return $FALSE;
            }
        }
    }
    if ( defined $_[0]->{'error_message_like'}
        && $_[0]->{'actual'}->{'message'} !~ $_[0]->{'error_message_like'} )
    {
        $Log->debug('Does not match because it does not match the expected error message regex.') if $Log->may_debug();
        return $FALSE;
    }
    if ( defined $_[0]->{'exact_error_message'}
        && $_[0]->{'actual'}->{'message'} ne $_[0]->{'exact_error_message'} )
    {
        $Log->debug('Does not match because it does not match the expected exact error message.') if $Log->may_debug();
        return $FALSE;
    }

    $Log->debug('Matches all criteria.') if $Log->may_debug();
    $TRUE;
}

=head2 with_success

Sets appropriate state for a matcher that does not expect an exception to be raised.

=cut

sub with_success {
    validate_pos(@{[@_[1..$#_]]});
    $_[0]->{'failure_message'}
        = "Expected the code block to not throw an exception matching all of the criteria specified but it did.";
    $_[0]->{'negative_failure_message'}
        = "Expected the code block to throw an exception matching all of the criteria specified but it did not.";
    $_[0]->{'expected'} = { 'status' => 'passed' };
    $_[0];
}

=head2 with_failure

Sets appropriate state for a matcher that does expects an exception to be raised.

=cut

sub with_failure {
    validate_pos(@{[@_[1..$#_]]});
    $_[0]->{'failure_message'}
        = "Expected the code block to throw an exception matching all of the criteria specified but it did not.";
    $_[0]->{'negative_failure_message'}
        = "Expected the code block to not throw an exception matching all of the criteria specified but it did.";
    $_[0]->{'expected'} = { 'status' => 'failed' };
    $_[0];
}

=head2 with_otherwise

Adds a handler for the otherwise block in the try statement.

=cut
my @with_otherwise_validate_spec = (
    { type => CODEREF }
);
sub with_otherwise {
    validate_pos(@{[@_[1..$#_]]}, @with_otherwise_validate_spec);
    $_[0]->{'otherwise'} = $_[1];
    $_[0];
}

=head2 with_finally

Adds a handler for the finally block in the try statement.

=cut

my @with_finally_validate_spec = (
    { type => CODEREF }
);
sub with_finally {
    validate_pos(@{[@_[1..$#_]]}, @with_finally_validate_spec);
    $_[0]->{'finally'} = $_[1];
    $_[0];
}

=head2 with_exception

Adds criteria for matching whether or not the exception is the correct one.

=cut

my @with_exception_validate_spec = (
    { type => SCALAR|ARRAYREF }
);
sub with_exception {
    validate_pos(@{[@_[1..$#_]]}, @with_exception_validate_spec);
    $_[0]->{'exception'} = $_[1];
    $_[0];
}

=head2 with_error_message_like

Adds criteria for matching the message of the exception thrown against a regex. Use qr//.

=cut

my $regex_validate_spec_regex_callback = sub {
    ref($_[0]) eq 'Regexp';
};
my @regex_validate_spec = { callbacks => { 'is it a qr// regex?' => $regex_validate_spec_regex_callback}};
sub with_error_message_like {
    validate_pos(@{[@_[1..$#_]]}, @regex_validate_spec);
    $_[0]->{'error_message_like'} = $_[1];
    $_[0];
}

=head2 with_exact_error_message

Adds criteria for matching the message of the exception thrown against an exact string.

=cut

my @with_exact_error_message_validate_spec = (
    { type => SCALAR }
);
sub with_exact_error_message {
    validate_pos(@{[@_[1..$#_]]}, @with_exact_error_message_validate_spec);
    $_[0]->{'exact_error_message'} = $_[1];
    $_[0];
}

=head2 and_return_exception

Adds hook for returning the exception

=cut

my @and_return_exception_validate_spec = (
    { type => SCALARREF }
);
sub and_return_exception {
    validate_pos(@{[@_[1..$#_]]}, @and_return_exception_validate_spec);
    $_[0]->{'and_return_exception'} = $_[1];
    $_[0];
}

=head2 and_return_error_message

Adds hook for returning the exception error message.

=cut

my @and_return_error_message_validate_spec = (
    { type => SCALARREF }
);
sub and_return_error_message {
    validate_pos(@{[@_[1..$#_]]}, @and_return_error_message_validate_spec);
    $_[0]->{'and_return_error_message'} = $_[1];
    $_[0];
}

sub failure_message {
    validate_pos(@{[@_[1..$#_]]});
    join(
        "\n\n",
        $_[0]->{'failure_message'},
        'Expected:',
        $_[0]->_indent_text(Dumper( $_[0]->{'expected'} )),
        'Actual:',
        $_[0]->_indent_text($_[0]->_dump_actual_value()),
        '',
    );
}

sub negative_failure_message {
    validate_pos(@{[@_[1..$#_]]});
    join(
        "\n\n",
        $_[0]->{'negative_failure_message'},
        'Expected:',
        $_[0]->_indent_text(Dumper( $_[0]->{'expected'} )),
        'Actual:',
        $_[0]->_indent_text($_[0]->_dump_actual_value()),
        '',
    );
}

=head2 actual_exception

Returns the exception caught by the ExceptionMatcher, if any.

=cut

sub actual_exception {
    validate_pos(@{[@_[1..$#_]]});
    $_[0]->{'actual'}->{'exception'};
}

sub matcher_stacktrace {
    validate_pos(@{[@_[1..$#_]]});
    if ( defined $_[0]->{'actual'}->{'exception'} && $_[0]->{'actual'}->{'exception'}->isa('NACL::Exceptions::ExpectationNotMetError')) {
        if ( defined $_[0]->{'actual'}->{'exception'}->matcher_stacktrace() ) {
            # Deepest is first
            my @matcher_stacktrace = @{$_[0]->{'actual'}->{'exception'}->matcher_stacktrace()};
            push(@matcher_stacktrace, $_[0]);
            \@matcher_stacktrace;
        } else {
            # If no value is returned we return self, otherwise
            # a basic throw with ExpectationNotMetError outside the context of a should/should_not
            # will blow up. It's debateable as to whether or not we really should support that.
            [ $_[0] ];
        }
    } else {
        [ $_[0] ];
    }
}

sub _dump_actual_value {
    validate_pos(@{[@_[1..$#_]]});
    if ( $_[0]->{'actual'}->{'status'} eq 'failed' ) {
        my $e = $_[0]->{'actual'}->{'exception'};
        my $str;
        if ( $e->isa('NATE::BaseException') ) {
            $str = join("\n\n", 'Exception Type:', $_[0]->_indent_text(ref($_[0]->{'actual'}->{'exception'})), 'Stacktrace:',  $_[0]->_indent_text($_[0]->{'actual'}->{'exception'}->stacktrace()) );
        } else {
            $str = Dumper( $_[0]->{'actual'} );
        }
        $str;
    } else {
        'No exception was thrown.';
    }
}

sub _die_handler {
    # I'm not sure if there's any caveats to using
    # Params::Validate in the die handler, so to be safe
    # it isn't used
    if ( blessed $_[0] ) {
        die $_[0];
    } else {
        die NATE::BaseException->new($_[0]);
    }
}

1;
