
# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.

## @summary Setup Periodic Matcher
## @author  Adam.Ciapponi@netapp.com
## @pod     here

=head1 NAME

NACL::Matchers::MetaMatchers::PeriodicMatcher

=head1 DESCRIPTION

This module implements a matcher that calls the delgate matcher it is initialized with until it achieves or stays matching/not matching.

=head1 METHODS

=cut

package NACL::Matchers::MetaMatchers::PeriodicMatcher;

# Compiler directives.
use strict;
use warnings;
use CR::Setup::Constants;
use CR::Setup::Exception;
use Data::Dumper;
use NATE::BaseException qw(:try);
use Params::Validate qw(:all);
use Time::HiRes qw(sleep);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use base qw(NACL::Matcher);

=head2 matches

This function calls the delegate matcher passed in on initialization until it reaches the timeout or success/failure.

=cut

sub matches {
    my ( $matcher_value ) = validate_pos(@{[@_[1..$#_]]}, $TRUE);

    my $matcher = $_[0]->{'expected'};

    my $interval = $_[0]->{'interval'};
    unless ( defined $interval ) {
        $interval = 1;
    }

    my $limit = $_[0]->{'limit'};
    unless ( defined $limit ) {
        $limit = 1;
    }

    my $limit_multiplier = $_[0]->{'limit_multiplier'};
    unless ( defined $limit_multiplier ) {
        $limit_multiplier = 1;
    }

    my $scaled_limit    = $limit * $limit_multiplier;
    my $logic           = $_[0]->{'logic'};
    if ( ! defined $logic ) {
        CR::Setup::Exception->throw('Must pick a PeriodicMatcher logic of continually or eventually');
    }
    my $sense           = $_[0]->{'sense'};
    if ( ! defined $sense ) {
        CR::Setup::Exception->throw('Must pick a PeriodicMatcher sense of positive or negative');
    }
    my $timeout_reached = $TRUE;

    $Log->log( 'Starting '
            . $logic
            . ( $sense eq 'negative' ? ' not' : '' )
            . ' matcher with a timeout of '
            . $scaled_limit
            . ' seconds and a polling interval of '
            . $interval
            . ' seconds.' );
    my $i;
    for ( $i = 0; $i <= $scaled_limit; $i += $interval ) {
        my $result = $matcher->matches($matcher_value);

        if ( $logic eq 'eventually' ) {
            if ( $sense eq 'positive' && $result ) {
                $timeout_reached = $FALSE;
                last;
            }
            elsif ( $sense eq 'negative' && !$result ) {
                $timeout_reached = $FALSE;
                last;
            }
        }
        else {
            # continually
            if ( $sense eq 'positive' && !$result ) {
                $timeout_reached = $FALSE;
                last;
            }
            elsif ( $sense eq 'negative' && $result ) {
                $timeout_reached = $FALSE;
                last;
            }
        }

        sleep($interval) if ( ( $i + $interval ) <= $scaled_limit );
    }

    if ($timeout_reached) {
        $Log->log( 'Finishing '
                . $logic
                . ( $sense eq 'negative' ? ' not' : '' )
                . ' matcher after reaching timeout of '
                . $scaled_limit
                . ' seconds.' );
    }
    else {
        $Log->log( 'Finishing '
                . $logic
                . ( $sense eq 'negative' ? ' not' : '' )
                . ' matcher after '
                . $i
                . ' seconds.' );
    }

    if ( $logic eq 'eventually' && $timeout_reached ) {
        $FALSE;
    }
    elsif ( $logic eq 'continually' && !$timeout_reached ) {
        $FALSE;
    } else {
        $TRUE;
    }
}

=head2 failure_message

This function calls the appropriate failure message for the delegate matcher.

=cut

sub failure_message {
    validate_pos(@{[@_[1..$#_]]});
    my $str = "Failed to ";
    $str .= $_[0]->{'logic'};
    $str .= ' ';
    $str .= 'not ' if ( $_[0]->{'sense'} eq 'negative' );
    $str .= "match.\n";
    if ( $_[0]->{'sense'} eq 'positive' ) {
        $str .= $_[0]->{'expected'}->failure_message();
    }
    else {
        # negative sense
        $str .= $_[0]->{'expected'}->negative_failure_message();
    }
    $str;
}

=head2 negative_failure_message

This function calls the appropriate negative failure message for the delegate matcher.

=cut

sub negative_failure_message {
    validate_pos(@{[@_[1..$#_]]});
    my $str = "Did not fail to ";
    $str .= $_[0]->{'logic'};
    $str .= ' ';
    $str .= 'not ' if ( $_[0]->{'sense'} eq 'negative' );
    $str .= "match.\n";
    if ( $_[0]->{'sense'} eq 'positive' ) {
        $str .= $_[0]->{'expected'}->negative_failure_message();
    }
    else {
        # negative sense
        $str .= $_[0]->{'expected'}->failure_message();
    }
    $str;
}

=head2 with_eventually

Sets the logic to eventually that will break the loop and return true if the delegate matcher eventually meets the criteria. If it hits timeout it will return false. This is useful for when something will not happen instantly.

=cut

sub with_eventually {
    validate_pos(@{[@_[1..$#_]]});
    $_[0]->{'logic'} = 'eventually';
    $_[0];
}

=head2 with_continually

Sets the logic to continually that will break the loop and return false if the delegate matcher stops meeting the criteria. If it hits timeout it will return true. Use primarily to check that something hasn't happened over a time interval. This is useful processing is not instantaneous, check that after failure it does not apply changes. Use sparingly.

=cut

sub with_continually {
    validate_pos(@{[@_[1..$#_]]});
    $_[0]->{'logic'} = 'continually';
    $_[0];
}

=head2 with_positive_sense

Sets the sense to positive which means the matcher will use eventually/continually logic.

=cut

sub with_positive_sense {
    validate_pos(@{[@_[1..$#_]]});
    $_[0]->{'sense'} = 'positive';
    $_[0];
}

=head2 with_negative_sense

Sets the sense to negative which means the matcher will use eventually_not/continually_not logic.

=cut

sub with_negative_sense {
    validate_pos(@{[@_[1..$#_]]});
    $_[0]->{'sense'} = 'negative';
    $_[0];
}

=head2 every

Sets the polling interval in seconds.

=cut

my @every_validate_spec = ({ type => SCALAR });
sub every {
    validate_pos(@{[@_[1..$#_]]}, @every_validate_spec);
    $_[0]->{'interval'} = $_[1];
    $_[0];
}

=head2 within

Sets the nominal value for the timeout when using eventually.

=cut

my @within_validate_spec = ({ type => SCALAR });
sub within {
    validate_pos(@{[@_[1..$#_]]}, @within_validate_spec);
    $_[0]->{'limit'} = $_[1];
    $_[0];
}

=head2 for_exactly

Sets the nominal value for the timeout when using continually.

=cut

my @for_exactly_validate_spec = ({ type => SCALAR });
sub for_exactly {
    validate_pos(@{[@_[1..$#_]]}, @for_exactly_validate_spec);
    $_[0]->{'limit'} = $_[1];
    $_[0];
}

=head2 seconds

Sets the units for the timeout value to seconds.

=cut

sub seconds {
    validate_pos(@{[@_[1..$#_]]});
    $_[0]->{'limit_multiplier'} = 1;
    $_[0];
}

=head2 minutes

Sets the units for the timeout value to minutes.

=cut

sub minutes {
    validate_pos(@{[@_[1..$#_]]});
    $_[0]->{'limit_multiplier'} = 60;
    $_[0];
}

=head2 hours

Sets the units for the timeout value to hours.

=cut

sub hours {
    validate_pos(@{[@_[1..$#_]]});
    $_[0]->{'limit_multiplier'} = 3600;
    $_[0];
}

sub matcher_stacktrace {
    validate_pos(@{[@_[1..$#_]]});
    # Deepest is first
    [ @{$_[0]->{'expected'}->matcher_stacktrace()}, $_[0] ];
}

1;
