#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written conset from NetApp, Inc.
#
package NACL::OnDemandLoading;
use v5.14;

use Carp qw/confess cluck/;

use NATE::ParamSet;
use NATE::Log qw/log_global/;
use Scalar::Util qw/blessed/;

use NACL::Global;

my $params = NATE::ParamSet->new(global => 1);
my $Log = log_global();

my $DEFAULT_ENABLE_ON_DEMAND_LOADING = 0;

my $Enabled;
my $Processed = 0;

sub is_processed { return $Processed };

sub is_enabled   {
    return $Enabled if defined $Enabled;
    my $enable = $params->get('ENABLE_NACL_ON_DEMAND_LOADING', default => $DEFAULT_ENABLE_ON_DEMAND_LOADING);
    if ($enable && !is_vetoed()){
        $Enabled = 1;
    }
    else{
        $Enabled = 0;
    }
    return $Enabled;
}

sub is_vetoed {
    # Some veto reasons won't change over time and dont need to be recomputed on
    # every call
    state $static_veto;
    state $nate_autouse_available;
    state $inc_has_http_paths;

    # Check if NATE::Autouse is available. It's possible that the NATE stack lags
    # behind the NACL stack. Do it only once since the answer wont change
    if (! defined $nate_autouse_available ){
        eval { require NATE::Autouse };
        my $err = $@;
        if ($err){
            if ( $err =~ /Can't locate NATE.Autouse\.pm/){
                $nate_autouse_available = 0;
            }
            else{
                confess $@;
            }
        }
        else{
            $nate_autouse_available = 1;
        }
    }

    if (! defined $inc_has_http_paths ){
        my ($http_paths) = grep { blessed($_) && $_->isa('NATE::Inc::Http') } @INC;
        $inc_has_http_paths = (defined($http_paths)?1:0);
    }

    if (! defined $static_veto ){

        if (!$nate_autouse_available){
            $Log->comment("Skipping NACL On Demand Loading because NATE::Autouse is not available");
            $static_veto = 1;
        }
        elsif( $inc_has_http_paths){
            $Log->comment("Skipping NACL On Demand Loading because \@INC contains http paths");
            $static_veto = 1;
        }
        # Check if Aspect was loaded before us. If so dont do anything as Aspect
        # isnt compatible with autouse.
        elsif ( defined($INC{'Aspect.pm'}) && $INC{'Aspect.pm'} ne 'NATE::Autouse'){
            $Log->comment("Skipping NACL On Demand Loading because the Aspect module was detected as loaded");
            $static_veto = 1;
        }
        elsif (!defined $NACL::Global::is_nacl_on_demand_loading_possible){
            $Log->comment(
                "The on-demand loading of the NACL stack is not possible ".
                "because your p4 client is missing the necessary changes to make it ".
                "work reliably. Make sure to p4 sync or wsupdate your client if you ".
                "want this functionality"
            );
            $static_veto = 1;
        }
    }
    return 1 if ($static_veto);

    # The DISABLE_ON_IMPORT sub ran. That means NATE::Autouse is effectively
    # disabled
    if ($NATE::Autouse::DEVEL){
        $Log->comment("NACL On Demand Loading has been disabled, most likely because Aspect was detected");
        return 1;
    }
    return 0;
}

sub enable {
    my $orig = $NATE::Autouse::DEVEL;
    $NATE::Autouse::DEVEL = 0;
    if (!is_vetoed()){
        $Enabled = 1;
    }
    else{
        $NATE::Autouse::DEVEL = $orig;
    }
}

sub import {
    my ($class, @args) = @_;
    for my $arg ( @args ){
        if ( $arg eq ':process'){
            $class->process;
        }
    }
}

sub process {
    _process() if ( is_enabled() );
}

# This is the entry point for ASUPTrigger::Worker. We don't care about the
# value of ENABLE_NACL_ON_DEMAND_LOADING
sub _process {

    return if ($Processed);
    return if (is_vetoed());

    $Log->comment("Enabling on-demand loading of all NACL Exception, APISet, C, CS and STask classes");

    # Limit calls to filesystem. This disables file check existence
    # This replaces the earlier "use NATE::Autouse qw(:nostat)" that happens
    # to be problematic when the NATE stack lags the NACL stack
    $NATE::Autouse::NOSTAT = 1;

    _autouse_all_nacl();

    # The Aspect module looks in the symbol table directly to match patterns
    # There is no way to make this work with autoused packages. Detect when
    # the Aspect loads, and disable autouse entirely
    local $NATE::Autouse::DISABLE_ON_IMPORT = 1;
    NATE::Autouse->autouse('Aspect');
    $Enabled = 1;
    $Processed = 1;

    $Log->comment("Completed NACL on-demand loading enablement");
}

sub disable {
    # This is going to load all the autoused packages that were "use-d" up
    NATE::Autouse->disable;
    $Enabled = 0;
    $Processed = 0;
}

sub _autouse_all_nacl {
    # There are too many exceptions to list them all. Just autouse all of them.
    # Exceptions at NetApp are used as a hybrid of class and procedural package
    # Many scripts do this as a way to import the try/catch syntax:
    #
    #      use NACL::C::Exceptions::DoesNotExist qw(:try);
    #
    # Class::Autouse does not support this. NATE::Autouse, a copy of it, does.
    local $NATE::Autouse::LOAD_ON_STORABLE_THAW = 1;
    {
        local $NATE::Autouse::LOAD_ON_IMPORT = 1;
        NATE::Autouse->autouse_recursive('NACL/Exceptions/');
        NATE::Autouse->autouse_recursive('NACL/APISet/Exceptions/');
        NATE::Autouse->autouse_recursive('NACL/C/Exceptions/');
        NATE::Autouse->autouse_recursive('NACL/STask/Exceptions/');
        NATE::Autouse->autouse_recursive('NACL/MTask/Exceptions/');
        NATE::Autouse->autouse_recursive('NACL/Transit/Exceptions/');
    }
    {
        local $NATE::Autouse::ALLOW_USE_ING = 1;
        # SystemLicense exposes package variables that callers know about...
        # Make it work
        NATE::Autouse->autouse('NACL::STask::SystemLicense');
        # NACL::C::UnitNormalization exports symbols and package variables so
        # we shouldnt autouse it, but it's also called without being explicitely loaded
        # so we must autouse it. To make that work, we are going to autouse it
        # but in such a fashion that it will get loaded at the next use
        # statement. Same as SystemLicense above
        local $NATE::Autouse::LOAD_ON_STORABLE_THAW = 0;
        NATE::Autouse->autouse('NACL::C::UnitNormalization');
    }
    {
        # Don't autouse anything in packages or directories starting with _
        # Don't autouse Exceptions since its done above
        # Skip files in NACL/STask/SystemLicense/ since they arent classes
        # Skip files NACL/CS/ComponentState/Zapi* since they arent classes
        # STask::Client::RPC::Server isn't class
        local $NATE::Autouse::EXCLUDE_RE = qr{/_|/Exceptions|CS/ComponentState/Zapi};
        local $NATE::Autouse::LOAD_ON_CAN = 1;
        NATE::Autouse->autouse_recursive('NACL/CS/');

        # Don't autouse packages in all-lower case folders. Thats cdefs stuff
        $NATE::Autouse::EXCLUDE_RE = qr{/_|/Exceptions|/Tools|/ParserUtils|/GlobalConst|/GConst|/WDKeys/|/\p{Lowercase}+/};
        # The NACL/APISet folder tree contains a bunch of cdef files at the bottom.
        # Skip the bottom part for a performance boost
        local $NATE::Autouse::MAX_DEPTH = 5;
        $NATE::Autouse::LOAD_ON_CAN = 0;
        NATE::Autouse->autouse_recursive('NACL::APISet');
    }
    {
        # Components and STasks have a custom "can" method
        # NACL::C::CommandInterface::ONTAP installs methods of
        # NACL::C::Component::ONTAP at compile time. This wont work if
        # NACL::C::Component::ONTAP is autoused. Skip it.
        # NACL::C::Switch::Utils is a procedural package
        local $NATE::Autouse::LOAD_ON_CAN = 1;
        local $NATE::Autouse::EXCLUDE_RE = qr{/_|/Exceptions|Component/ONTAP.pm|Switch/Utils.pm|/UnitNormalization.pm};
        NATE::Autouse->autouse_recursive('NACL/C/');

        # STask.pm cannot be autoused because a number of STask call
        # __PACKAGE__->purge_method_builder at compile time
        # NATE::Autouse ensure ancestors have been loaded only after the current module
        # is "required", i.e inherited methods may not be there at compile time
        # but
        # Some code out there calls NACL::STask::STask->method without loading STask
        # first. This will only work if that same code calls the method of an STask subclass
        # first
        $NATE::Autouse::EXCLUDE_RE = qr{/_|/Exceptions|/SystemLicense/|/Client/RPC/Server|/STask.pm};
        NATE::Autouse->autouse_recursive('NACL/STask/');
    }
    NATE::Autouse->autouse('NACL::Cleanup');
    $NATE::Autouse::LOAD_ON_STORABLE_THAW = 0;
    NATE::Autouse->autouse_recursive('NACL::Transit');
    NATE::Autouse->autouse_recursive('NACL::Realm');
    NATE::Autouse->autouse('NACL::MTask::SCP');
    NATE::Autouse->autouse('NACL::MTask::EventLogDetector');
}

1;

__END__

=head1 NAME

NACL::OnDemandLoading - Allows for much of the NACL stack to be loaded only when
                        called, drastically reducing startup times and memory
                        footprint of tests relying on NACL libraries.

=head1 SYNOPSIS

    use NACL::OnDemandLoading;
    NACL::OnDemandLoading->process;

=head1 DESCRIPTION

C<NACL::OnDemandLoading> ( NODL for short) allows for much of the NACL layers,
i.e APISet, CS, C, and STask, to be loaded dynamically at the first method call
rather than statically at compile time. This provides much benefits w.r.t.
memory footprint and process start time.

Example:

    use NACL::C::Node;
    my ($node) = NACL::C:Node->find();

The above loads 770 libraries (152M) with On-Demand Loading turned off,
and only 77 (26M of memory) with it turned on. The exact memory benefits of
On-Demand Loading will depend on how many NACL libraries the script loads at
compile time and what it does at runtime. This turns the memory consumption
model of NACL tests from a costly bundle to pay-as-you-go.

=head1 METHODS

=head2 C<process>

Assuming that ENABLE_NACL_ON_DEMAND_LOADING is set to true, and that C<disable>
hasn't been called before, this method will perform all the work necessary to
make the NACL layers only load at the first call of any of their methods.

C<process> is called automatically in code running in a Subtest or NATE::Process
as soon as a NACL library is detected to be loading.

Code running outside of a Subtest or NATE::Process currently has to explicitely
call C<process> in a BEGIN block or use the shorthand below at the top of
their script:

    use NACL::OnDemandLoading qw(:process);

=head2 C<is_enabled>

Returns true if any package has been marked for On-Demand Loading in the
current process.

=head2 C<disable>

Turns off On Demand Loading and immediately loads the autoused packages that
were detected as being use-d so far. This method won't load autoused packages
that were require-d instead of use-d, so you probably shouldnt use it. If you
need to disable On Demand Loading, set ENABLE_NACL_ON_DEMAND_LOADING=0 and
file a NATE burt.

=head2 C<enable>

Turns On Demand Loading back on, should it have been disabled before. Untested.

=head1 CAVEATS

The behavior of classes with on-demand-loading turned on varies slightly from
that of classes without it. In general things just work, but developers of
new libraries in APISet/, CS/, C/ and STask/ folders should be aware of a
few caveats documented below.

=head2 Delayed syntax error detection

Errors that would have been reported at compile time may now only be reported
at runtime. When developing a new NACL library in APISet/, CS/, C/ or STask/
folders you should turn off On-Demand-Loading by passing ENABLE_NACL_ON_DEMAND_LOADING=0.
Once your new library works with OnDemand loading turned off, test it with it
turned on.

=head2 Procedural packages

Packages in APISet/, CS/, C/ and STask, should not export subroutines or symbols.
If you need to create a library of utility subroutines to be shared by multiple
classes, start the file name with an underscore.

Example: NACL/STask/_AggregateVolume.pm.

=head2 Mixed packages

Don't mix methods and exported subroutines in the same package. Simply separate
out the two type of functions in their own package, and see above for what to
do with a procedural package

=head2 Package variables

Package variables, i.e. "our" variables, won't exist until your package gets
truly loaded, i.e. after the first method call. Avoid exposing package
variables from a class. Use class methods instead.

=head2 File with multiple packages

Do not declare more than one package per file. If you feel the need to declare
a "private" secondary package (i.e one starting with '_'), it must truly be
private and never instantiated outside the file it's defined in.

=head2 Inherited method call at compile time

Avoid calling inherited methods during compile time. For example:

    package Foo;
    sub foo_method {};

    package Bar;
    use base 'Foo';
    __PACKAGE__->foo_method;

The above isn't going to work when Foo and Bar have On-Demand Loading enabled.
This sort of code actually appears throughout the NACL codebase, and can be
made to work with On-Demand Loading, but the fewer instances of it the better

=head2 Use of Aspect module

The use of the Aspect module is supported with On-Demand Loading, but if you
are encountering issues make sure to load Aspect first in your code.

    use strict;
    use Aspect;         # Loading Aspect first works best
    use NACL::C::Foo;

=head2 Calling MTask classes not explicitely loaded

On-Demand Loading will prevent some MTasks from being automatically loaded,
meaning that code like this may fail:

    use v5.14;
    # No "use" statement for NACL::MTask

    my $mtask = NACL::MTask::Foo->new();

The fix is to explicitely "use" or "require" the MTasks you need in your
script or library.

=head2 Sharing of memory between parent and forked children

With NODL turned off, the NACL libraries "use-d" by a parent would be inherited
by the forked children at an initial zero cost of memory. With NODL turned on
this is still true, but only the libraries actually *called* by the parent
or another ancestor will be inherited by the forked children. Libraries "use-d"
by a parent but only called in the children will be loaded in each child
separately.

=head1 BEST PRACTICES

=head2 Don't import ':try' multiple times

Many NACL files import the try/catch syntax multiple times. This is unnecessary
and with On-Demand Loading, it will have a memory cost. Each use statement
that imports the :try tag ends up loading the NACL exception class in memory.
In other words, the following is free:

   use NACL::Exceptions::Foo; # Package Foo wont be loaded until needed

But this isn't:

   use NACL::Exceptions::Foo qw(:try); # Package Foo loaded now

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (ng-nacl-dev@netapp.com)

=back
