# This is a prototype.  It still needs to be documented.

# Form and contain a list of Hostrec objects of all the Hosts (Filers,
# Clients, Switches) which this test (or whatever the current context
# is) is meant to be allowed to use.

# Any of the array fields (nodes, clients, clusters, switches) which
# is not already assigned a list of Hostrecs will have a default list
# of Hostrecs computed.  For now, we don't use any of the newer
# proposed NATE ParamSet feature.  Instead, any field (nodes, clients,
# clusters, switches) which is not assigned a value will be assigned
# using the existing legacy pattern of how hosts are passed in
# parameters [1].

# [1] http://web.netapp.com/engineering/nate/releases/3/last/html/natehostparams.html

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary List of Resource Records the test is allowed to use
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::ResourceRecords

=head1 DESCRIPTION

This module is responsible for forming and containing a list of
L<Resource Records|lib-NACL-ResourceRecord-pm> objects of all the hosts (Filers,
Clients, Switches, Vfilers, Vservers) which this test is meant to allowed to
use. (Each Resource Record consists of a hostrec object for that host along
with apiset_must and apiset_should rulesets)

For now, we don't use any of the newer proposed NATE ParamSet features.
Instead, any field (nodes, clients, clusters, switches, vfilers, vservers)
which is not assigned a value will be assigned using the existing legacy pattern
of how hosts are passed in parameters [1].

[1] http://web.netapp.com/engineering/nate/releases/3/last/html/natehostparams.html

=head1 ATTRIBUTES

All of the attributes are arrays of objects of type
L<ResourceRecord|lib-NACL-ResourceRecord-pm>.

=head2 nodes

A reference to an array of C<ResourceRecord> objects which describe the nodes
(filers) which can be used by the test.

Unless specified in the call to 'new', this is populated by the
parameters "FILER_LIST", "FILERA..ZZ", "FILER", "NODES".

=head2 clients

A reference to an array of C<ResourceRecord> objects which describe the clients
(windows/unix systems) which can be used by the test.

Unless specified in the call to 'new', this is populated by the
parameters "CLIENT_PARAM_LIST", "CLIENT1..N", "CLIENTS", "CLIENT".

=head2 clusters

A reference to an array of C<ResourceRecord> objects which describe the clusters
which can be used by this test.

Unless specified in the call to 'new', this is populated by the
parameters "CLUSTER1..N", "CLUSTERS", "CLUSTER".

=head2 switches

A reference to an array of C<ResourceRecord> objects which describe the switches
which can be used by this test.

Unless specified in the call to 'new', this is populated by the
parameters "SWITCH1..N", "SWITCHES", "SWITCH".

=head2 esxhosts

A reference to an array of C<ResourceRecord> objects which describe the esxhosts
which can be used by this test.

Unless specified in the call to 'new', this is populated by the
parameters "ESX_HOST1..N", "ESX_HOSTS", "ESX_HOST".

=head2 hypervhosts

A reference to an array of C<ResourceRecord> objects which describe the hypervhosts
which can be used by this test.

Unless specified in the call to 'new', this is populated by the
parameters "HYPERV_HOST1..N", "HYPERV_HOSTS", "HYPERV_HOST".

=head2 vservers

A reference to an array of C<ResourceRecord> objects which describe the vservers
which can be used by this test.

Unless specified in the call to 'new', this is populated by the
parameters "VSERVER1..N", "VSERVERS", "VSERVER".

=head2 vfilers

A reference to an array of C<ResourceRecord> objects which describe the vfilers
which can be used by this test.

Unless specified in the call to 'new', this is populated by the
parameters "VFILER1..N", "VFILERS", "VFILER".

=cut

package NACL::ResourceRecords;

use strict;
use warnings;

use Tharn qw(param host);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::ResourceRecord;
use NATE::BaseException qw(:try);
use Params::Validate qw(validate_pos validate_with ARRAYREF SCALAR);

# use NACL::ChooseAPISet;
use NACL::Realm;
use autouse 'NACL::GeneralUtils' => qw(_parse_hosts);
use Tie::IxHash;

=head1 METHODS

=head2 new

    NACL::ResourceRecords->new();

This method creates an object of type C<NACL::ResourceRecords>. It accepts
nodes, clients, clusters, switches, vservers and vfilers as optional parameters.
For all parameters not passed, it populates their values by reading the
relevant test parameter.

=cut

use Class::MethodMaker [
    new    => [ '-hash', 'new' ],
    scalar => [
        {   '-default_ctor' =>
                sub { _apply_rules_to_all( _build_nodes(), 'FILER' ); },
        },
        'nodes',
    ],
    scalar => [
        {   '-default_ctor' =>
                sub { _apply_rules_to_all( _build_clusters(), 'CLUSTER' ); },
        },
        'clusters',
    ],
    scalar => [
        {   '-default_ctor' =>
                sub { _apply_rules_to_all( _build_clients(), 'CLIENT' ); },
        },
        'clients',
    ],
    scalar => [
        {   '-default_ctor' =>
                sub { _apply_rules_to_all( _build_switches(), 'SWITCH' ); },
        },
        'switches',
    ],
    scalar => [
        {   '-default_ctor' =>
                sub { _apply_rules_to_all( _build_esxhosts(), 'ESX_HOST' ); },
        },
        'esxhosts',
    ],
    scalar => [
        {   '-default_ctor' =>
                sub { _apply_rules_to_all( _build_hypervhosts(), 'HYPERV_HOST' ); },
        },
        'hypervhosts',
    ],
    scalar => [
        {   '-default_ctor' =>
                sub { _apply_rules_to_all( _build_vservers(), 'VSERVER' ); },
        },
        'vservers',
    ],
    scalar => [
        {   '-default_ctor' =>
                sub { _apply_rules_to_all( _build_vfilers(), 'VFILER' ); },
        },
        'vfilers',
    ],
    scalar => [
        {
            -default_ctor => sub {
                _apply_rules_to_all(_build_avanodes(), 'AVA');
            },
        },
        'avanodes',
    ],

    # The following entries are for the "generic" test parameters, which are
    # used to apply values to all hosts of a particular type. These store
    # the resource record built by reading that generic test parameter.
    # Example: FILER_<blah> is considered generic since it would apply
    # to all filers, while something like FILERA_<blah> is specific only
    # to that filer.
    # These are named generic_<prefix>_resource_record
    scalar => [
        { '-default_ctor' => sub { _find_rules( prefix => 'FILER' ); }, },
        'generic_filer_resource_record',
    ],
    scalar => [
        { '-default_ctor' => sub { _find_rules( prefix => 'CLUSTER' ); }, },
        'generic_cluster_resource_record',
    ],
    scalar => [
        { '-default_ctor' => sub { _find_rules( prefix => 'CLIENT' ); }, },
        'generic_client_resource_record',
    ],
    scalar => [
        { '-default_ctor' => sub { _find_rules( prefix => 'SWITCH' ); }, },
        'generic_switch_resource_record',
    ],
    scalar => [
        { '-default_ctor' => sub { _find_rules( prefix => 'ESX_HOST' ); }, },
        'generic_esx_host_resource_record',
    ],
    scalar => [
        { '-default_ctor' => sub { _find_rules( prefix => 'HYPERV_HOST' ); }, },
        'generic_hyperv_host_resource_record',
    ],
    scalar => [
        { '-default_ctor' => sub { _find_rules( prefix => 'VSERVER' ); }, },
        'generic_vserver_resource_record',
    ],
    scalar => [
        { '-default_ctor' => sub { _find_rules( prefix => 'VFILER' ); }, },
        'generic_vfiler_resource_record',
    ],
    scalar => [
        {
            -default_ctor => sub { _find_rules(prefix => 'AVA'); },
        },
        'generic_ava_resource_record',
    ],
    scalar => [
        {'-default_ctor' => sub {
            my $rr = _find_rules(prefix => 'COMMAND_INTERFACE');

            # Need a way to run only the non-debug variant of CITs through
            # ZAPI. Solutions through Smokes and stest were investigated
            # but neither worked, so we need to add a bit of a hack here.
            # Smokes always set "RELROOT", which would be of the form
            # build_root/final/bedrock/export/x86_64.debug.sim/tarball.
            # If this path includes "debug", then it's a debug build, else
            # it's a non-debug build.
            # If it's a non-debug build and the new "NACL_RUN_NON_DEBUG_ZAPI"
            # parameter is set to 1, we enable ZAPI.
            my $smokes_zapi_param = Tharn::param('NACL_RUN_NON_DEBUG_ZAPI');
            if ($smokes_zapi_param) {
                my $apiset_should = $rr->{apiset_should};
                if (!exists $apiset_should->{interface}) {
                    my $relroot = Tharn::param('RELROOT');
                    if (defined $relroot && $relroot !~ /debug/) {
                        $apiset_should->{interface} = 'ZAPI';
                        $apiset_should->{interface_source} =
                            'NACL_RUN_NON_DEBUG_ZAPI is set and a non-debug '
                            . 'build is being used, so setting to ZAPI';
                        $rr->{apiset_should} = $apiset_should;
                    }
                }
            }
            return $rr;
            }
        },
        'all_command_interface_resource_record',
    ],
];

sub _build_nodes {

    # Examine FILER_LIST and return a list of node Hostrecs it
    # declares, else try to get them from FILERA..FILERZZ, else use
    # what is in FILER, else use what is in "NODES", or else this test
    # doesn't have any filers/nodes.

    # The given parameter names and resolution order match ntest.
    # (Also matches what SharedTestBeds tends to fill in; FILERS and
    # FILER_SRC and FILER_DST skipped because ntest skips them)

    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;

    return
           _hosts_in_meta_param('FILER_LIST')
        || _hosts_in_param( 'FILER', 'A..ZZ' )
        || _hosts_in_param('FILER')
        || _hosts_in_param('NODES')
        || [];

}

sub _build_clients {

    # Lazy construct default value

    # Examine CLIENT_PARAM_LIST and return a list of Unix, Windows,
    # etc. Hostrecs it declares, else use whatever is in
    # CLIENT1..CLIENTN, else use whatever is in CLIENTS, else use
    # whatever is in CLIENT, or else this test doesn't have any
    # clients.

    # The given parameter names and order match lib/ForeachHost.pm

    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;

    return
           _hosts_in_meta_param('CLIENT_PARAM_LIST')
        || _hosts_in_param( 'CLIENT', '1..N' )
        || _hosts_in_param('CLIENTS')
        || _hosts_in_param('CLIENT')
        || [];
}

sub _build_clusters {

    # Lazy construct default value

    # Examine CLUSTER params and return a list of cluster LIF
    # Hostrecs.  The parameter names and order are new as of this
    # writing (we're inventing this concept here, we think).

    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;

    return
           _hosts_in_param( 'CLUSTER', '1..N' )
        || _hosts_in_param('CLUSTERS')
        || _hosts_in_param('CLUSTER')
        || [];
}

sub _build_switches {

    # Lazy construct default value

    # Examine SWITCHES, SWITCH1...SWITCHn and return a list of
    # network, fabric, etc. switch Hostrecs.  The parameter
    # names should match what SharedTestBeds fills in (but
    # that needs to be checked).  The order they are checked
    # in is new as of this writing.

    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;

    return
           _hosts_in_param( 'SWITCH', '1..N' )
        || _hosts_in_param('SWITCHES')
        || _hosts_in_param('SWITCH')
        || [];
}

sub _build_hypervhosts {

    # Lazy construct default value

    # Examine HYPERV_HOSTS, HYPERV_HOST1...HYPERV_HOSTn and return a list of
    # HyperV server Hostrecs.  The parameter names should match what
    # SharedTestBeds fills in (but that needs to be checked).  The
    # order they are checked in is new as of this writing.

    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;

    return
           _hosts_in_param( 'HYPERV_HOST', '1..N' )
        || _hosts_in_param('HYPERV_HOSTS')
        || _hosts_in_param('HYPERV_HOST')
        || [];
}

sub _build_esxhosts {

    # Lazy construct default value

    # Examine ESX_HOSTS, ESX_HOST1...ESX_HOSTn and return a list of
    # ESX server Hostrecs.  The parameter names should match what
    # SharedTestBeds fills in (but that needs to be checked).  The
    # order they are checked in is new as of this writing.

    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;

    return
           _hosts_in_param('ESX_HOST', '1..N' )
        || _hosts_in_param('ESX_HOSTS')
        || _hosts_in_param('ESX_HOST')
        || [];
}

sub _build_vservers {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;

    return
           _hosts_in_param( 'VSERVER', '1..N' )
        || _hosts_in_param('VSERVERS')
        || _hosts_in_param('VSERVER')
        || [];
}

sub _build_vfilers {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;

    return
           _hosts_in_param( 'VFILER', '1..N' )
        || _hosts_in_param('VFILERS')
        || _hosts_in_param('VFILER')
        || [];
}

sub _build_avanodes {
    $Log->enter() if $may_enter;;

    # Examine AVA_LIST and return a list of node Hostrecs it declares, else try
    # to get them from AVANODES, else AVAA..AVAZZ, else use what is in AVA, or
    # else this test doesn't have any filers/nodes.

    my $nodeslist =
        _hosts_in_meta_param('AVA_LIST') ||
        _hosts_in_param('AVANODES')      ||
        _hosts_in_param('AVA', '1..N')   ||
        _hosts_in_param('AVA')           ||
        [];

    $Log->exit() if $may_exit;
    return $nodeslist;
}

# The given '$meta_name' is the name of a parameter which itself
# holds a comma separated list of names of pararmeters to get
# lists of host spec strings from.  Read it and the parameters it
# names, and return an arrayref of all the host objects specified
# by those host spec strings.  (For example, if FOO_LIST contains
# "FOOA,FOOB" then return all the hosts specified in the values of
# parameters FOOA and FOOB).

# Return "[]" when the values seem authoratative that there are no
# hosts (the $meta_name has a defined value, but either it is
# blank or any parameters it lists have undefined/blank values)
# and undef when no such authority was found ($meta_name
# undefined).  This convention in using [] and undef to mean these
# things facilitates the "||" chaining you see in calls above.

# If any values are unresolvable, we can expect Tharn::hosts to raise
# an exception for us.

sub _hosts_in_meta_param {
    $Log->enter() if $may_enter;
    my ($meta_name) = validate_pos( @_, { type => SCALAR } );
    my @records;
    my $names = param($meta_name);
    if ( !$names ) {
        $Log->exit() if $may_exit;
        return undef;
    }

    foreach my $name ( split( /\s*,\s*/, $names ) ) {
        my $hosts_in_param = _hosts_in_param($name);
        if ( !defined $hosts_in_param ) {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw( "The parameter '$meta_name' "
                    . 'is a meta-parameter. This means that instead of '
                    . 'providing it as a list of values, it should be '
                    . 'provided as a list of parameter names (it was '
                    . "provided as $meta_name=$names)\n\n" . 'See '
                    . 'http://web.netapp.com/engineering/nate/releases/3/last/html/natehostparams.html'
                    . " and "
                    . 'http://wikid.netapp.com/w/QA/projects/Libraries_Initiative/Project/Common_Infrastructure/Users_Guide/Component_Layer_Users_Guide#Meta-parameters'
                    . " for more details\n" );
        }
        my @val = @{$hosts_in_param};
        foreach my $value ( @val ) {
            my $already_exists = 0;
            foreach my $record ( @records ) {
                if (!exists($record->{hostrec})){
                    if ($record->_name eq $value->_name){
                        $already_exists = 1;
                        last;
                    }
                }
                else{
                    if ($record->hostrec->hostname eq $value->hostrec->hostname){
                        $already_exists = 1;
                        last;
                    }
                }
            }
            push @records, $value if (!$already_exists);
        }
    }

    $Log->exit() if $may_exit;
    return \@records;
}

# The given '$prefix' and '$type' describe one or more parameter names
# which each contain 0 or more host spec strings.  Return an arrayref
# of all the host objects specified by those host spec strings.

# If $type is empty, then there is just one parameter named $prefix to
# read

# If $type is "A..ZZ", then parameters are named FOOA, FOOB, FOOC,
# etc., stopping at the first parameter which has no non-empty value.

# If $type is "1..N", then parameters are named FOO1, FOO2, FOO3,
# etc., stopping at the first parameter which has no non-empty value.

# Return undef when no hosts are found (we would return "[]" if
# the parameter settings were authoratative that the user meant to
# say there are no hosts, but we assume that's not what's meant.
# We instead assume a user would set FILER to "" to mean "use
# NODES instead of FILER" rather than to mean "This test has no
# filers, regardless of what NODES might be set to").  This
# convention in using [] and undef to mean these things
# facilitates the "||" chaning you see in calls above.

# If any values are unresolvable, we can expect Tharn::hosts to raise
# an exception for us.

# Tharn::hosts knows how to deal with the actual values themselves and
# how to parse them (may be comma-separated, may be colon-separated,
# may include overrides like "foo;hosttype=bar" or
# "foo{hosttype=bar}", etc).

sub _hosts_in_param {
    $Log->enter() if $may_enter;
    my ( $prefix, $type ) = validate_pos(
        @_,
        { type => SCALAR },
        { type => SCALAR, optional => 1 }
    );
    my @records;
    my $get_next_suffix;
    my @suffixes;
    my $count;
    if ( !$type ) {
        @suffixes = ("");
        $get_next_suffix = sub { shift @suffixes };
    } elsif ( $type eq 'A..ZZ' ) {
        @suffixes = ( 'A' .. 'Z', 'AA' .. 'ZZ' );
        $get_next_suffix = sub { shift @suffixes };
    } elsif ( $type eq '1..N' ) {
        $count = 1;
        $get_next_suffix = sub { $count++ };
    } else {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("Unknown type '$type'");
    }
    while (1) {
        my $suffix = $get_next_suffix->();
        last if !defined($suffix);
        my $val = param("$prefix$suffix");
        last if ( !defined($val) || !length($val) );
        my %hostspec_pairs;
        tie %hostspec_pairs, 'Tie::IxHash';
        %hostspec_pairs = _parse_hosts($val);

        while(my ($id, $hostspec) = each %hostspec_pairs) {
            my $resource_record = _find_rules( prefix => "$prefix$suffix" );
            $resource_record->_name($id);
            try {
                $resource_record->hostrec(Tharn::host($hostspec));
            } otherwise {
                $resource_record->_error(shift);
            };
            push @records, $resource_record;
        }
    }
    $Log->exit() if $may_exit;
    return @records ? \@records : undef;
}

# This subroutine is to extract rules from parameters passed.
# Just like the list of hosts can be passed using a number of different
# naming conventions (e.g: FILER_LIST, FILER, FILERA..ZZ, NODES), their
# corresponding rules (apiset_must and apiset_should) can also be passed
# using different conventions. The usage we support is when the rule is
# passed using the same naming convention as was used to specify the host.
# Meaning, if the hosts were passed using FILERA, FILERB etc, rules passed in
# the form FILERA_APISET_(MUST|SHOULD)_(CATEGORY|INTERFACE|SET),
# FILERB_APISET_(MUST|SHOULD)_(CATEGORY|INTERFACE|SET) etc, will be accepted.
# If, however, we specified the hosts using FILERA..Z and passed the rules
# as NODES_APISET_(MUST|SHOULD)_(CATEGORY|INTERFACE|SET), the rules passed
# will simply be ignored. In other words, if the hosts were specified in
# convention $prefix, then rules in the form $prefix_rule will be supported.
# While constructing hostrecs, we see whether rules have been defined using
# that prefix.

# The mechanism for specifying a rule common to all hosts is to specify the
# rule as FILER_<rule>. This will override all other rules provided.

# This subroutine, given a prefix, will look for parameters passed starting
# with that prefix and will create hashes out of them. What is returned is a
# "rules_hashref", which is a hashref which in turn consists of two hashrefs,
# one hashref for apiset_must, the other hashref for apiset_should.
sub _find_rules {
    $Log->enter() if $may_enter;
    my %args = validate_with(
        params => \@_,
        spec   => { prefix => { type => SCALAR } }
    );
    my $prefix          = $args{prefix};
    my $uc_prefix       = uc $prefix;
    my $resource_record = NACL::ResourceRecord->new();

    require NACL::ChooseAPISet;

    $resource_record->prefix($prefix);
    foreach my $must_or_should ( qw(APISET_MUST APISET_SHOULD)) {
        foreach my $key ( NACL::ChooseAPISet::ruleset_keys() ) {
            my $rule  = "${uc_prefix}_${must_or_should}_" . uc $key;
            my $value = param($rule);
            if ($value) {
                my $set = lc($must_or_should) . '_set';
                $resource_record->$set( $key => $value );
                $resource_record->$set(
                    "${key}_source" => "Test parameter $rule" );
            }
        }
    }

    # Assign CDEF path values if specified through test parameters
    # These are of the form <prefix>_<interface>_CDEF_PATH
    foreach my $interface ( _all_interfaces() ) {
        my $cdef_param = "${prefix}_${interface}_CDEF_PATH";
        my $cdef_path  = param("$cdef_param");
        if ($cdef_path) {
            $Log->warn(
                "CDEF_PATH for the Command Interface is scoped in terms "
                    . " of 'SET' also . Hence setting the path at the "
                    . " interface level \n only is not encouraged . Please Refer "
                    . "to the following link for the valid format and details \n"
                    . "http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/ComponentLayerUsersGuide/index.html"
            );

            $resource_record->cdef_path_set( $interface => $cdef_path );
            $resource_record->cdef_path_set(
                "${interface}_source" => "Test parameter $cdef_param" );
        }
    }

    # The "timeout" and "timeout_scaling_factor" can also be provided
    # as test parameters. These are of the form:
    # <prefix>_TIMEOUT(_SCALING_FACTOR)
    foreach my $timeout_param (qw(TIMEOUT TIMEOUT_SCALING_FACTOR)) {
        my $full_param    = "${prefix}_$timeout_param";
        my $timeout_value = param($full_param);
        if ($timeout_value) {
            my $lc_timeout_param = lc $timeout_param;
            $resource_record->$lc_timeout_param($timeout_value);
        }
    }

    $Log->exit() if $may_exit;
    return $resource_record;
}

# Apply rules from the generic prefixes. For example, for nodes, FILER_ is
# the generic prefix, so any rule applicable for all nodes can be specified
# through FILER_*. Also, rules can be applied for ALL command_interface types
# by using the COMMAND_INTERFACE_ prefix.
sub _apply_rules_to_all {
    $Log->enter() if $may_enter;
    my ( $records_arrayref, $prefix )
        = validate_pos( @_, { type => ARRAYREF }, { type => SCALAR } );

    my $generic_field = 'generic_' . lc($prefix) . '_resource_record';
    my $resource_records = NACL::Realm->realm()->resource_records();
    my $resource_record_generic = $resource_records->$generic_field();

    my $all_res_rec = $resource_records->all_command_interface_resource_record();

    my @settings = NACL::ResourceRecord->_settings_read_from_parameters();
    foreach my $setting (@settings) {
        foreach my $record (@$records_arrayref) {
            my $isset = "${setting}_isset";
            if (ref $record->$setting() eq 'HASH') {
                # A hash has multiple keys and each of the various records
                # might have different keys of the hash set, so combine them
                # all together. Burt 796946 (unit test:
                # test_combining_hash_params in ResourceRecords.thpl)
                my %record_setting = $record->$setting();
                my %generic_record_setting = $resource_record_generic->$setting();
                my %all_record_setting = $all_res_rec->$setting();
                my %combined_setting = (%all_record_setting,
                    %generic_record_setting, %record_setting);
                $record->$setting(%combined_setting);
            } elsif (!$record->$isset()) {
                if ($resource_record_generic->$isset()) {
                    $record->$setting($resource_record_generic->$setting());
                } elsif ($all_res_rec->$isset()) {
                    $record->$setting($all_res_rec->$setting());
                }
            }
        }
    }

    $Log->exit() if $may_exit;
    return $records_arrayref;
}

# All the interfaces we support. Update when support for a new interface is
# added to components
sub _all_interfaces {
    return qw(CLI ZAPI SNMP XAPI VMAPI);
}

1;
