# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

## @summary AggregateFile Task Module
## @author roopeshv@netapp.com
## @status shared
## @pod here

package NACL::STask::AggregateFile;

use strict;
use warnings;
use POSIX;

use base qw(NACL::C::AggregateFile NACL::STask::STask);

use Tharn '$Log';
use Params::Validate qw(:all);
use NACL::ComponentUtils qw (_optional_scalars);
use NATE::Time qw(timeout2time);

use NACL::CS::AggregateFile;

use NACL::C::UnitNormalization qw(convert_to_bytes_int);
use NACL::STask::_Mixins::AggregateVolumeFile qw(purge
    dumpblock_info _calculate_timeout_based_on_size
    _get_aggr_or_vol_name_from_path
);

=head1 NAME

NACL::STask::AggregateFile

=head1 DESCRIPTION

C<NACL::STask::AggregateFile> provides a number of well-defined but
potentially complex or multi-step methods related to Volume Files in
ONTAP.

The files can be created to be holes or blocks. Also, optionally can be reserved.
The file object returned by this class can be used to do various tasks on it.

It builds on top of, and is a derived class of C<NACL::C::AggregateFile>,
and so it also provides methods that are more in the scope of
individual AggregateFile related commands.  See C<NACL::C::AggregateFile> for
details.  This also means that a C<NACL::STask::AggregateFile> package may
generally be used in place of C<NACL::C::AggregateFile> package.

=head1 CLEANUP METHODS

Cleanup can be registered for the following methods

Cleanup methods are,
    VolumeFile Method         Cleanup Method
    ----------------------------------------
    create                    purge


=head1 METHODS

=head2 create

    # Creating a single file
    my $file_obj = NACL::STask::AggregateFile->create(
        command_interface => $ci,
        path              => $file_path,
        nacltask_enable_space_reserve => 0 | 1, # Default is 0
        nacltask_wait       => 0 | 1, # Default is 1
        nacltask_fill_holes => 0 | 1 # Default is 1
        nacltask_to_cleanup      => 1,          # default 0,
        nacltask_cleanup_manager => $CleanupObj,
        %other_opts
    );

    # Creating multiple files by using "filecount"
    # This example creates 3 files
    my @file_objs = NACL::STask::VolumeFile->create(
        command_interface => $ci,
        path              => $file_path,
        filecount         => 3,
        nacltask_enable_space_reserve => 0 | 1, # Default is 0
        nacltask_wait       => 0 | 1, # Default is 1
        nacltask_fill_holes => 0 | 1 # Default is 1
        nacltask_to_cleanup      => 1,          # default 0,
        nacltask_cleanup_manager => $CleanupObj,
        %other_opts
    );
(Class method) Create files by using internal mkfile tool.
This method provides additional services beyond what's provided by the
component method, as controlled and described by the new
C<nacltask_enable_space_reserve>, C<nacltask_fill_holes> and
C<nacltask_wait> arguments. Further, this method also defaults to
determining the appropriate node on which to issue the Nodescope commands.
If the size is not provided, this method defaults to creating a zero-length
file.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Mandatory)
The command interface. Uses the command interface for apiset if provided.
Uses the command interface of the mandatory aggregate provided.

=item C<< path => $path >>

(Mandatory)
The complete path of the file in the aggregate. This is of the format "/aggr/<aggr_name>/<filename>"

=item C<< nacltask_enable_space_reserve => 0 | 1 >>

(Optional, defaults to 0)
If set to 1, enables file reservation on the created file.

=item C<< nacltask_fill_holes => 0 | 1 >>

(Optional, defaults to 1, applicable only for debug builds)
If set to 0, this method does not fill file holes. It does
so by creating a file of size 0 and then uses "testsetattr" to reserve
the requested amount of space.

Note that the "testsetattr" command is available only on debug builds,
so for non-debug builds this flag is ignored.

=item C<< method-timeout => $seconds >>

(Optional, defaults to 1minute per 1mb)
The timeout for the nacl to bail out if there the command did not complete
in given value

=item C<< determine_appropriate_nodescope_node => 0 | 1 >>

(Optional, defaults to 1)
If set to 1 (the default) the method automatically determines the node
on which the Aggregate resides and invokes the Nodescope commands in the
context of that node.

=item C<< nacltask_wait => 0 | 1 >>

(Optional, defaults to 1)
The mkfile command itself is non-blocking, so it's possible that it has
returned control before the file has been created.

If the value of this argument is 1 (the default), this method waits till the
file is created before returning.

=item C<< polling_interval => $interval_in_seconds >>

(Optional, Defaults to 10 seconds, Applicable only if nacltask_wait = 1)
The interval at which to poll for whether the file(s) have been created.

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup)
Flag indicating if this operation needs to be registered for clean up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager

=back

=back

=cut

sub create {
    $Log->enter();
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            size                          => { type => SCALAR,  default => 0 },
            nacltask_enable_space_reserve => { type => BOOLEAN, default => 0 },
            nacltask_fill_holes           => { type => BOOLEAN, default => 1 },
            nacltask_wait                 => { type => SCALAR,  default => 1 },
            $pkg->_cleanup_validate_spec(),
            polling_interval => { type => SCALAR, default => 10 },
            $pkg->_nodescope_options_validate_spec(),
        },
        allow_extra => 1,
    );

    my $ci = $opts{command_interface};
    my $enable_space_reserve = delete $opts{nacltask_enable_space_reserve};
    my $fill_holes           = delete $opts{nacltask_fill_holes};
    my $wait                 = delete $opts{nacltask_wait};
    my $interval             = delete $opts{polling_interval};

    my $version_manager = $ci->version_manager();
    unless ($fill_holes) {
        if (!$version_manager->is_debug() && $version_manager->is_promoted()) {
            $fill_holes = 1;
            $enable_space_reserve = 0;
        }
    }
    my $orig_size = $opts{size};
    if ($fill_holes == 0) {
        $opts{size} = 0;
    }

    unless (defined $opts{'method-timeout'} ) {
        $opts{'method-timeout'}  = $pkg->_calculate_timeout_based_on_size(
            size => $opts{size},
            filecount => $opts{filecount},
        );
    }

    # Get common component params: apiset_must, apiset_should, etc
    my %common_params;
    $pkg->_copy_common_component_params(
        source => \%opts,
        target => \%common_params
    );

    my ( %opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup );
    $pkg->_move_common_cleanup_opts(
        source => \%opts,
        target => \%opts_for_cleanup,
    );

    if ( $ci->is_cmode() ) {
        if (   $opts{determine_appropriate_nodescope_node}
        && !defined $opts{'nodescope-node-name'}) {

            # If this is set then we need to make multiple nodescope calls.
            # Since all operations are operating on the same volume, they're
            # all operating on the same node, so let's compute this once,
            # rather than require every call to recompute the value.
            $common_params{'nodescope-node-name'}
                = $pkg->get_local_node_name(
                command_interface => $ci,
                path              => $opts{path},
                %common_params
            );
        # For Find call
        } elsif (defined $opts{'nodescope-node-name'} ) {
            $common_params{'nodescope-node-name'} = $opts{'nodescope-node-name'};
        }
    } ## end if ( $ci->is_cmode()...})

    my @files = $pkg->SUPER::create(%opts, %common_params);

    foreach my $file (@files) {
        if ($wait) {
            my $end_time = timeout2time( $opts{'method-timeout'} );

            while (1) {
                my $found = $file->find( allow_empty => 1, %common_params);
                if ($found) {
                    last;
                }
                elsif ( time() > $end_time ) {
                    my $file_str = $file->path();
                    $Log->exit();
                    NACL::Exceptions::Timeout->throw( "The file $file_str "
                          . 'did not get created even after waiting for '
                          . "$opts{'method-timeout'} seconds" );
                } ## end elsif ( time() > $end_time)
                Tharn::snooze($interval);
            } ## end while (1)
        } ## end if ($wait)

        # Register this resource with the cleanup manager for cleanup.
        $file->_copy_common_opts_for_cleanup(
            'source'              => { %opts_for_cleanup },
            'target'              => \%opts_for_register,
            'nacltask_to_cleanup' => \$nacltask_to_cleanup,
            'to_cleanup'          => 'purge',
        );

        $file->_register_for_cleanup(%opts_for_register)
          if ($nacltask_to_cleanup);

        if ($enable_space_reserve) {
            $file->set_size( %common_params, size => $opts{size} );
            $file->enable_reservation(%common_params);
        } elsif (!$fill_holes) {
            $file->set_size( %common_params, size => $orig_size );
        }
    }

    $Log->exit();
    return wantarray ? @files : $files[0];
} ## end sub create

=head2 dumpblock_info

    # Class method: providing the file path. The fileid and aggregate
    # name are determined by the method.
    my $dumpblock_cs_obj  = NACL::STask::AggregateFile->dumblock_info(
        command_interface => $command_interface,
        path              => $file_path,
        %other_opts
    );

    # Class method: providing the aggregate name and fileid explicitly
    my $dumpblock_cs_obj  = NACL::STask::AggregateFile->dumblock_info(
        command_interface => $command_interface,
        fileid            => $fileid,
	    volume            => $volume_name,
        %other_opts
    );

    # Instance method: let the method determine the fileid. The aggregate
    # is automatically extracted from the path attribute of the object.
    my $dumpblock_cs_obj = $file_obj->dumpblock_info(%opts);

    # Instance method: explicitly providing the fileid. Again, the aggregate
    # name is extracted from the path attribute stored in the object.
    my $dumpblock_cs_obj = $file_obj->dumpblock_info(fileid => $fileid, %other_opts);

(Class or instance method) This method returns a
L<NACL::CS::AggregateFileDumpblockInfo|lib-NACL-CS-AggregateFileDumpblockInfo-pm> object
containing information about the specific block of the file. It extends the
underlying component method and provides the following additional
functionality:

=over

=item *

If the file path is provided (or if it's invoked with an object instance)
then this method can automatically determine the C<fileid> value (does so
by using the C<showfh> method) and the C<aggregate> value (done by
extracting it from the path).

=item *

Support for both class and instance method invocation

=item *

Defaults to determining the appropriate node on which to invoke
the Nodescope commands.

=back

=over

=item Options

=over

=item C<< path => $file_path >>

The full path of the file. This is expected to be in the form
"/aggr/$aggrname/$filename". If this argument is provided, then the
aggregate is determined from the path, and the fileid is determined
by invoking the C<showfh> method. In other words, if this argument is
provided, then we do B<not> need to provide the arguments C<fileid>
and C<volume>.

=item C<< determine_appropriate_nodescope_node => 0 | 1 >>

(Optional, default to 1)
If set to 1 (the default) determines the appropriate node on which to invoke
the Nodescope commands.

=item Other options

All of the other various options to
L<< NACL::C::AggregateFile->dumpblock_info | lib-NACL-C-AggregateFile-pm/dumpblock_info >>

=back

=back

=over

=item Exceptions

=over

=item C<NATE::Exceptions::Argument>

This type of exception is thrown when either of the option fileid or path is missing OR either of the option volume or file is missing.

=back

=back

=cut


=head2 purge

    # Class method; purging a single file
    NACL::STask::AggregateFile->purge(
        command_interface => $ci,
        path              => $path,
        nacltask_wait     => 0 | 1 # Defaults to 1
    );

    # Class method; purging multiple files
    NACL::STask::AggregateFile->purge(
        command_interface => $ci,
        paths             => \@paths,
        nacltask_wait     => 0 | 1 # Defaults to 1
    );

    # Instance method (can only purge a single file with the instance method)
    $file_obj->purge(nacltask_wait => 0 | 1);

Deletes the file(s) and waits for the file(s) to be deleted (defaults to
waiting for 2 minutes). If run through Nodescope-CLI, then it defaults
to determining the appropriate node to run on.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required for class method, Not Applicable for Instance method)
The command interface against which to run the commands.

=item C<< path => $path >>

For class method call, either C<path> or C<paths> should be provided.
Not Applicable for Instance method call.

The full path to the file.

=item C<< paths => \@paths >>

For class method call, either C<path> or C<paths> should be provided.
Not Applicable for Instance method call.

An array-reference containing a list of file-paths to be purged.

=item C<< nacltask_wait => 0 | 1 >>

(Optional, defaults to 1)
Wait till the file(s) get purged. Defaults to waiting. The default timeout
is 120 seconds (i.e. 2 minutes). This can be overridden using C<method-timeout>.

=item C<< polling_interval => $interval_in_seconds >>

(Optional, Defaults to 10 seconds, Applicable only if nacltask_wait = 1)
The interval at which to poll for whether the file(s) have been purged.

=item C<< determine_appropriate_nodescope_node => 0 | 1 >>

(Optional, defaults to 1)
For Nodescope-CLI, determines the appropriate node on which to invoke the
command.

=item Other options

All the other options accepted by this method are documented in
L<< NACL::C::VolumeFile->delete|lib-NACL-C-VolumeFile-pm/delete >>

=back

=back

=cut


=head2 get_size

Gives back the size of the file.

# Class method to get the file size
    $file_size = NACL::STask::AggregateFile->get_size(
        command_interface => $ci,
        path              => $path,
    );

    or

# Object call to get the file size
    my $file = NACL::STask::AggregateFile->create(size => '5m', ...);
    $file->get_size();  # 5m
    $file->set_size(size => '8m');
    $file->get_size();  # 8m

=cut

sub get_size {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => { $pkg_or_obj->_nodescope_options_validate_spec(), },
    );

    my %common_options;
    $pkg_or_obj->_copy_common_component_params_with_ci(
        source => \%opts,
        target => \%common_options
    );

    my $file_cs = NACL::CS::AggregateFile->fetch(
        %common_options,
        filter           => { path => $opts{path} },
        requested_fields => ['file_size']
    );

    $Log->exit();
    return $file_cs->file_size();
} ## end sub get_size

=head2 dirty_file

Overwrites the file. If C<< length >> and C<< offset >> are provided, those are
used while dirtying the file.
The timeout for this can be controlled with C<< method-timeout >> parameter.

    my $file = NACL::STask::AggregateFile->create(...);
    $file->dirty_file(
        offset => $starting_byte,
        length => $size_to_dirty,
    );

The offset and length can be numbers followed by unites like KB, mb, m, G etc.
The default timeout is based on size to be at least 1 minute or 1 minute per 1mb;

=cut

sub dirty_file {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            offset => { type => SCALAR, default  => 0 },
            length => { type => SCALAR, optional => 1 },
            $pkg_or_obj->_nodescope_options_validate_spec(),
        },
    );

    my $offset = convert_to_bytes_int( delete $opts{offset} );

    my $length =
      convert_to_bytes_int( delete $opts{length}
          || $pkg_or_obj->get_size(%opts) );

    $opts{'method-timeout'} ||=
      $pkg_or_obj->_calculate_timeout_based_on_size( size => $length - $offset );

    my $result = $pkg_or_obj->testsetattr( %opts,
        'write-offset-and-length' => "$offset $length", );
    $Log->exit();
    return $result;

} ## end sub dirty_file

=head2 set_size

Sets the size of file, using testsetattr file-size attribute

# Class method to set the file size
    NACL::STask::AggregateFile->set_size(
        command_interface => $ci,
        path              => $path,
        size              => 1k
    );

    or

# Object call to set the file size
    my $file = NACL::STask::AggregateFile->create(...);
    $file->set_size(size => $size);

=cut

sub set_size {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            size => { type => SCALAR, },
            $pkg_or_obj->_nodescope_options_validate_spec(),
        },
    );
    my $size   = delete $opts{size};
    my $result = $pkg_or_obj->testsetattr(
        'file-size' => $size,
        %opts
    );
    $Log->exit();
    return $result;
} ## end sub set_size

=head2 file_reservation

Can be used to enable or disable reservation on file

    my $file = NACL::STask::AggregateFile->create(...);
    $file->file_reservation(enable => 1);
    $file->file_reservation(enable => 0);

=cut

sub file_reservation {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            enable => { type => BOOLEAN },
            $pkg_or_obj->_nodescope_options_validate_spec()
        },
    );

    $opts{'file-space-reservations'} =
      delete $opts{enable} ? 'enable' : 'disable',

    my $result = $pkg_or_obj->testsetattr(%opts);
    $Log->exit();
    return $result;
} ## end sub file_reservation

=head2 enable_reservation

Enables the reservation on the file.

    my $file = NACL::STask::AggregateFile->create(...);
    $file->enable_reservation();

=cut

sub enable_reservation {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => { $pkg_or_obj->_nodescope_options_validate_spec() }
    );
    my $result = $pkg_or_obj->file_reservation( %opts, enable => 1 );
    $Log->exit();
    return $result;
}

=head2 disable_reservation

Disables the reservation on the file.

    my $file = NACL::STask::AggregateFile->create(...);
    $file->disable_reservation();

=cut

sub disable_reservation {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => { $pkg_or_obj->_nodescope_options_validate_spec() }
    );
    my $result = $pkg_or_obj->file_reservation( %opts, enable => 0 );
    $Log->exit();
    return $result;
}

=head2 testsetattrs

Execute multiple testsetattrs on the file, and return the array of results from
each testsetattr call.

    my $file = NACL::STask::AggregateFile->create(...);
    my @results = $file->testsetattrs([
        {'get-attributes' => 1},
        {'read-offset-and-length' => '0 100'},
        {'write-offset-and-length' => '0 100'},
    ]);

Use it as a pass through for multiple apiset testsetattr.

=cut

sub testsetattrs {
    $Log->enter();
    my $self    = shift;
    my ($attrs) = @_;
    my @results = map { $self->testsetattr(%$_) } @$attrs;
    $Log->exit();
    return @results;
}

=head2 hammer

Executes the hammer command on the file without needing to pass filename
and command interface

    my $file = $Aggr->file_create(...);
    $file->hammer(...);

This also set priv set -c on node for hammer to work on aggregate file
Check BURT 881008 for more info
This accepts all input accepts by hammer apiset

=cut

sub hammer {
    $Log->enter();
    my ($self, %opts)  = @_;
    my $ci = $opts{command_interface} || $self->command_interface;

    ## This also set priv set -c on node for hammer to work on aggregate file
    ## Check BURT 881008 for more info
    $ci->get_7m_or_nodescope_apiset->priv_set(
        console_priv => 1, quiet => 1, level => 'test',
    );
    $Log->exit();
    return $self->SUPER::hammer(%opts);
}

1;
