#
# Copyright (c) 2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary FileLock Task Module
## @author  dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::Client::FileLock;

use strict;
use warnings;
use feature 'state';
use Scalar::Util qw(blessed);
use Params::Validate qw(validate_with validate SCALAR GLOBREF OBJECT HASHREF BOOLEAN UNDEF SCALARREF);
use base qw/NACL::C::Client::File NACL::STask::STask/;
use Tharn qw($Log);
use NACL::GeneralUtils qw( nacl_method_retry);
use NACL::C::Client;
use NACL::STask::Exceptions::Client::FileLock::NotAcquired;
use NACL::STask::Exceptions::Client::FileLock::NotReleased;
use Data::Dumper;
use NATE::BaseException qw(:try);

use Class::MethodMaker [
    new    => [ '-hash', '-init', 'new' ],
    scalar => 'file_path',
    scalar => 'node',
    scalar => 'path',
    scalar => 'lock_pid',
    scalar => 'status',
    scalar => 'vserver',
    scalar => 'volume',
    scalar => 'file_obj',
];

sub init {
    $Log->enter();
    my $self            = shift;
    my %opts = validate_with(
        params  => \@_,
        spec    => {
            $self->_primary_keys_validate_spec(),
         }
    );

    $self->node( $opts{node} );
    $self->file_obj( $opts{file_obj} );
    $self->lock_pid( $opts{lock_pid} );
    $self->vserver( $opts{vserver} );
    $self->volume( $opts{volume} );
    $self->file_path( $opts{'file-path'} );
    $self->status( $opts{status} );
    $Log->exit();
} ## end sub init

=head2 new

Get a new instance of NACL::STask::Client::FileLock.

=over

=item Usage

$filelock = NACL::STask::Client::FileLock->new(
    command_interface => $ci,
    node  => $node,
    'file-path' => $file_path,
    vserver  => $vserver,
    volume   => $volume
);

=item Parameters

=item C<<command_interface>>

(Instance method)
Required. Client command interface.

=item C<<file-path>>

(Instance method)
Required. The absolute path to the file.

=item C<<node>>

(Instance method)
Optional. node name.

=item C<<status>>

(Instance method)
Optional. status of the lock.

=item C<<vserver>>

(Instance method)
Optional.  The vserver on which the volume path 
 is exported to the client(NFS|CIFS).

=item Return

<OBJECT> : A NACL::STask::Client::FileLock object.

=back

=cut

=head2 acquire()

Lock a byte range in a file(Class method)

=over

=item Options

=item C<< file_open_opts >>

HASHREF (Required in case file_path option is not provided)
The file options for the file to be opened.
It takes in the same options as taken by NACL::C::Client::File->open method.
Default value passed to file_open_opts is :
      access_mode      => 'read-write',
      open_flags       => 'create-always'

=item C<< file_lock_opts >>

HASHREF (Required)The file options for the file to be opened.
It takes in the same options as taken by NACL::C::Client::File->lock method..

=item C<< file_path => $file_path >>

 (Required in case file_name is not provided in file_open_opts)
 It is the absolute or relative path of the file to be open

=item C<< node => $node >>

 (Required in case nacltask_verify is set to 1 ) The node on which the volume path 
 is exported to the client(NFS|CIFS).
 
=item C<< vserver => $vserver >>

 (Required in case nacltask_verify is set to 1 ) The vserver on which the volume path 
 is exported to the client(NFS|CIFS).
 
=item C<< volume => $volume >>

 (Required in case nacltask_verify is set to 1 ) The volume which  
 is exported to the client(NFS|CIFS).
 
=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the deletion did happen
successfully. The node & path values are required for the verification.

=item C<< nacltask_retry_tries_count => $tries_count >>

(Optional, defaults to 5)
The number of times to retry the resync command if it fails because
the snapshot is temporarily busy or stale catch entry.

Note that it is possible to make it not retry the restore command by setting
the value of this argument to 1.

=item C<< nacltask_retry_sleep_time => $sleep_time >>

(Optional, defaults to 20 seconds)
The time to sleep between retries of the restore command.

=item Example

 my $file_open_opts = {
      file_name        =>  $File_name1,
      access_mode      => 'read-write',
      open_flags       => 'create-always'
 };
 my $file_lock_opts = {
      lock_type          => 'write-lock',
      lock_wait          => 'non-waiting-lock',
      offset             => 5,
      num_bytes          => 10000,
 };
 $file_lock_obj = NACL::STask::Client::FileLock->acquire(
      command_interface  => $Client,
      node               => $Node->node(),
      file_lock_opts     => $file_lock_opts,
      file_open_opts     => $file_open_opts,
      nacltask_verify    => 1,
      vserver            => $Testvserver,
      volume             => $Volume
 );
 
 Example 2 : with file-path option

 $file_lock_obj = NACL::STask::Client::FileLock->acquire(
      command_interface  => $Client,
      node               => $Node->node(),
      file_lock_opts     => $file_lock_opts,
      'file-path'        => $file_path_with_name,// absolute or relative path to File
      nacltask_verify    => 1,
      vserver            => $Testvserver,
      volume             => $Volume
 );
=back

=cut

sub acquire{
    $Log->enter();
    my $pkg = shift;

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
	     file_open_opts    => {type => HASHREF, optional => 1},
	     file_lock_opts    => {type => HASHREF},
             'file-path'         => {type => SCALAR, optional => 1},
	     nacltask_verify     => { type => BOOLEAN, default  => 0 },
	     nacltask_retry_tries_count => { type => SCALAR, default => 5 },
	     nacltask_retry_sleep_time  => { type => SCALAR, default => 10 },
	     node         => {type => SCALAR|OBJECT|UNDEF, optional => 1},
	     vserver      => {type => SCALAR, optional => 1},
	     volume       => {type => SCALAR, optional => 1},
        },
        allow_extra         => 1,
        ignore_primary_keys => 1,
    );

    my $file_handle = $opts{file_handle};
    my $verify      = delete $opts{nacltask_verify};
    my $vserver     = $opts{vserver};
    my $volume      = $opts{volume};
    my $node        = $opts{node};

    my ($node_name, $node_obj);
    if( blessed $node){
       $node_name = $node->node();
       $node_obj  = $node;
    }else{
       $node_name = $node;
       $node_obj = NACL::C::Node->new(node => $node_name);
    }
    my $lock_wait = $opts{file_lock_opts}->{lock_wait};

    delete $opts{command_interface};

    my %retry_opts;
    $pkg->_move_retry_params(
        source => \%opts,
        target => \%retry_opts
    );

    my $file_open_opts;

    if(defined $opts{file_open_opts}){
       $file_open_opts = delete $opts{file_open_opts};
    }else{
       ## default options passed to open file
       $file_open_opts = {
          access_mode      => 'read-write',
          open_flags       => 'create-always'
       };
    }

    $file_open_opts->{'file_name'}  = $opts{'file-path'} if(defined $opts{'file-path'});

    ## check for file_name in file open options $file_open_opts
    if(!defined $file_open_opts->{file_name}){
       NATE::Exceptions::Argument->throw('file-path/file-name option missing. The options 
          passed to open the file is '.Dumper($file_open_opts)
       );
    }

    $Log->debug( 'The file options beings sent to open method are: ' . Dumper($file_open_opts));

    my $file_lock_opts = delete $opts{file_lock_opts};
    $Log->debug( 'The file options beings sent to lock method are: ' . Dumper($file_lock_opts ));

    my $lock_obj ;

    $Log->comment('Opening the file :'.$file_open_opts->{file_name});
    my $file_obj = nacl_method_retry(
       code => sub {  NACL::C::Client::File->open(%{$file_open_opts}); },
       exceptions => [ qw(NATE::BaseException) ],
       %retry_opts,
    );

    try{
	$Log->comment('locking the file :'.$file_open_opts->{file_name});

	$lock_obj = nacl_method_retry(
                code => sub {  $file_obj->lock(%{$file_lock_opts}); },
		exceptions => [ qw(NATE::BaseException) ],
		%retry_opts,
	);
   }catch NATE::BaseException with{
       my $exception = shift;
       NACL::STask::Exceptions::Client::FileLock::NotAcquired->throw($exception->text());
   };

    if($verify){
        my $verify_locks = sub {
           my @vserver_lock_obj = $pkg->_check_vserver_locks(
               %opts,
               'file-path' => $file_open_opts->{'file_name'}
           );

           if(!scalar(@vserver_lock_obj)){
	      NACL::STask::Exceptions::Client::FileLock::NotAcquired->throw('Verification of Lock Failed: The file '.$file_open_opts->{file_name}.' could not be locked');
           }
        };
        nacl_method_retry(
                code => $verify_locks,
                exceptions => [ qw(NACL::STask::Exceptions::Client::FileLock::NotAcquired) ],
                %retry_opts,
        );

    }

    my %status = (status => $file_obj->get_status()) if($lock_wait !~ /non-waiting-lock/);

    $Log->exit;
    return NACL::STask::Client::FileLock->new(
	 node => $node_obj,
	 lock_pid  => $lock_obj,
	 file_obj => $file_obj,
	 file_path => $file_open_opts->{file_name},
	 vserver => $vserver,
	 volume  => $volume,
	 %status
    );

}

## private subroutines to check the vserver locks
sub _check_vserver_locks{
    my ($pkg, %opts) = @_;

    my $node        = delete $opts{node};
    my $vserver     = delete $opts{vserver};
    my $volume      = delete $opts{volume};
    my @vserver_lock ;

    my ($node_name, $node_obj);
    if( blessed $node){
       $node_name = $node->node();
       $node_obj  = $node;
    }else{
       $node_name = $node;
       $node_obj = NACL::C::Node->new(node => $node_name);
    }

    ## fetch file_name from the file-path
    ## for eg: /t/sti40_vsim_ucs507a/test_vol/test_file
    ## gives file_name as test_file
    my $file_name ;
    if($opts{'file-path'} && $opts{'file-path'} =~ /\/(\w+)$/){
       $file_name = $1;
    }

    if(!defined $node_obj && !defined $vserver && !defined $volume){
       NATE::Exceptions::Argument->throw(
             "If nacltask_verify is set to 1 then node, vserver & volume are required"
       );
    }

    my $filter;
    require NACL::CS::VserverLocks;
    $filter = {
        'vserver' =>  $vserver,
	'volume'  => $volume
    };


    $node_obj->refresh_command_interface();
    my @vserver_lock_obj = NACL::CS::VserverLocks->fetch(
        command_interface => $node_obj,
        filter => $filter,
        allow_empty => 1
    );

    ## verify if there is an entry of the locks exists
    ## with the same file name value
    foreach my $vs_obj(@vserver_lock_obj){
        my $obj_file_name;
        ## fetch file_name from the object-path attribute
        ## for eg: /test_vol/test_file
        ## gives file_name as test_file
        if($vs_obj->path =~ /\/(\w+)$/){
           $obj_file_name = $1;
        }

        if($obj_file_name eq $file_name){
           push @vserver_lock , $vs_obj ;
        }
    }
    return @vserver_lock;
}

=head2 release()

Release the locks in a file(Instance method Only)

=over

=item Options

=item C<< file_close_opts >>

 (Optional HASHREF)The file options for the file to be closed.
It takes in the same options as taken by NACL::C::Client::File->close method.
 

=item C<< file_unlock_opts >>

 (Optional HASHREF)The file options for the file to be opened.
It takes in the same options as taken by NACL::C::Client::File->unlock method..

=item C<< node => $node >>

 (Required in case nacltask_verify is set to 1 ) The node on which the volume path 
 is exported to the client(NFS|CIFS).
 
 =item C<< vserver => $vserver >>

 (Required in case nacltask_verify is set to 1 ) The vserver on which the volume path 
 is exported to the client(NFS|CIFS).
 
 =item C<< volume => $volume >>

 (Required in case nacltask_verify is set to 1 ) The volume which  
 is exported to the client(NFS|CIFS).
 
=item C<< path => $path >>

 (Required in case nacltask_verify is set to 1 )
The absolute or relative path of the file to be locked.
Either of path or Vserver,Volume is required in case when nacltask_verify is set to 1 

=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the deletion did happen
successfully. The node & path values are required for the verification.

=item C<< nacltask_retry_tries_count => $tries_count >>

(Optional, defaults to 5)
The number of times to retry the resync command if it fails because
the snapshot is temporarily busy or stale catch entry.

Note that it is possible to make it not retry the restore command by setting
the value of this argument to 1.

=item C<< nacltask_retry_sleep_time => $sleep_time >>

(Optional, defaults to 20 seconds)
The time to sleep between retries of the restore command.

=item C<< nacltask_delete_file_on_release => $nacltask_delete_file_on_release >>

(Optional, defaults to 0 )
The value decides whether to close and delete the file on release of lock

=item Example

 $lock_object->release();
=back

=cut
sub release {
   $Log->enter();
    my $pkg_or_obj = shift;

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
	    file_close_opts    => {type => HASHREF, optional => 1},
            file_unlock_opts    => {type => HASHREF, optional => 1},
	    nacltask_retry_tries_count => { type => SCALAR, default => 5 },
	    nacltask_retry_sleep_time  => { type => SCALAR, default => 10 },
	    node         => {type => SCALAR|OBJECT|UNDEF, optional => 1},
	    vserver      => {type => SCALAR, optional => 1},
	    volume       => {type => SCALAR, optional => 1},
	    nacltask_verify   => { type => BOOLEAN,  default => 0 },
	    nacltask_delete_file_on_release => { type => BOOLEAN,  default => 0 },
        },
        allow_extra         => 1
    );
    my $verify      = delete $opts{nacltask_verify};
    my $file_obj    = delete $opts{file_obj};
    my $nacltask_delete_file_on_release = $opts{nacltask_delete_file_on_release};

    $Log->debug( 'The options being sent to unlock are: ' . %opts );
    my %retry_opts;
    $pkg_or_obj->_move_retry_params(
        source => \%opts,
        target => \%retry_opts
    );

    my $file_unlock_opts = delete $opts{file_unlock_opts};
    $Log->debug( 'The file options beings sent to unlock method are: ' . Dumper($file_unlock_opts ));

    my $file_close_opts = delete $opts{file_close_opts};
    $Log->debug( 'The file options beings sent to close method are: ' . Dumper($file_close_opts ));

   try{
	nacl_method_retry(
		code => sub { $file_obj->unlock(%{$file_unlock_opts}); },
		exceptions => [ qw(NATE::BaseException) ],
		%retry_opts,
	);
   }catch NATE::BaseException with{
           my $exception = shift;
	   NACL::STask::Exceptions::Client::FileLock::NotReleased->throw($exception->text());
    };

    if($verify){
        Tharn::snooze( 10 );
        my $verify_locks = sub {
           my @vserver_lock_obj = $pkg_or_obj->_check_vserver_locks(%opts);

         if(scalar(@vserver_lock_obj)){
	   NACL::STask::Exceptions::Client::FileLock::NotReleased->throw('Verification of Lock Failed: The file '.$opts{file_name}.' lock could not be released');
        }
        };

        nacl_method_retry(
                code => $verify_locks,
                exceptions => [ qw(NACL::STask::Exceptions::Client::FileLock::NotReleased) ],
                %retry_opts,
        );

    }

    if($nacltask_delete_file_on_release){
        nacl_method_retry(
                code => sub {
                   $file_obj->close(%{$file_close_opts});
                   $file_obj->delete();
                },
                exceptions => [ qw(NATE::BaseException) ],
                %retry_opts,
        );
    }

    $Log->exit();
}

sub _primary_keys_validate_spec {
    return (
	file_obj  => {type => OBJECT|SCALARREF},
	node   => {type => OBJECT|SCALAR|UNDEF, optional => 1},
	path   => {type => SCALAR|UNDEF, optional => 1},
	lock_pid    => {type => SCALAR|HASHREF, optional => 1},
	file_path   => {type => SCALAR|UNDEF,	optional => 1},
	status => {type => SCALAR|UNDEF, optional => 1},
	vserver => {type => SCALAR|UNDEF, optional => 1},
	volume => {type => SCALAR|UNDEF, optional => 1}
    );
} ## end sub _primary_keys_validate_spec

1;
