#
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ...
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::STask::Client::RPC::Worker

=head1 DESCRIPTION

This object is a handle held by the parent process which can be used
to start a child process and send it remote procedure calls.

=head1 ATTRIBUTES

=head1 METHODS

=cut

package NACL::STask::Client::RPC::Worker;

use strict;
use warnings;

use Tharn qw(subtest);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::BaseException qw(:try);
use MIME::Base64 qw(encode_base64 decode_base64);
use Storable qw(nfreeze thaw);
use Scalar::Util qw(blessed);
use Params::Validate qw(validate validate_with CODEREF SCALAR);
use NACL::STask::Client::RPC::Future;
use NACL::ComponentUtils qw(_hash_move);
use Data::Dumper;

use Class::MethodMaker [
    new => [ '-hash', '-init', 'new' ],
    array =>
        'futures',  # a list of currently executing call objects.  not public?
    scalar => 'client',
    scalar => '_process',
];

=head2 new

    my $worker = NACL::STask::Client::RPC::Worker->new(client => $client)

=over

=item Options

=over

=item client => $client

The host upon which to start the worker process.  $client should be a C<NACL::C::Client> object

=item hostrec => $hostrec

An alternative to C<client>.  As C<client>, but the host is
represented as a C<Hostrec> object (value returned by C<Tharn::host>
or C<Tharn::hosts>)

=back

=back

=cut

sub init {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate @_, {
        client => {
            optional  => 1,
            callbacks => {
                'Valid client types "NACL::C::Client" or "Hostrec"' => sub {
                    my $obj = $_[0];
                    return 1 unless ( defined $obj );
                    if ( blessed $obj) {
                        $obj->isa('NACL::C::Client') || $obj->isa('Hostrec');
                    }
                },
            },
        },
    };
    if ( $opts{client} and $opts{client}->isa('NACL::C::Client') ) {
        $pkg->client( $opts{client}->hostrec() );
    }
    $Log->exit() if $may_exit;
} ## end sub init

=head2 start

   $worker->start();
   $worker->start('-runid'=>"run_id_for_spawned_subprocess");

Start the worker subprocess.

=cut

sub start {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %client_opts = validate @_, {'-runid' => { type => SCALAR , optional => 1 }};

    # build subtest options.
    if ( $self->client() ) {
        $client_opts{'-client'} = $self->client()->id();
    }

    # worker code
    my $worket_code_ref = <<'EOT';
    {
     use NACL::STask::Client::RPC::Server qw(serve);
     serve();
    }
EOT

    $self->_process( subtest( $worket_code_ref, '-bg', %client_opts ) );
    $Log->exit() if $may_exit;

} ## end sub start

=head2 submit

   my $future = $worker->submit(%options);

Create a "Future" object and submit it to the worker process.

(Unlike for $pool->call, calls are sent even if the worker is "busy",
so multiple unfinished calls build up as queued messages in STAF.
This may have some advantage of avoiding some latency, but it does add
an upper bound on how many calls can be outstanding at once)

=over

=item Options

=over

=item *

See Future->new for the list of options.

=back

=back

=cut

sub submit {
    $Log->enter() if $may_enter;
    my $self = shift;
    my $future = NACL::STask::Client::RPC::Future->new( worker => $self, @_ );
    $self->_add_future_to_queue($future);
    $future->_add_to_notify_list( sub { $self->_remove_future_from_queue(); }
    );
    $future->_start( process => $self->{_process} );
    $Log->exit() if $may_exit;
    return $future;
} ## end sub submit

=head2 call

   my $result = $worker->call(%options);

This is a syncronous version of submit.  It's equivalent to:

   my $result = $worker->submit(%some_options)->result(%other_options);

(where call takes the union of the various options that "submit" and
"result" can take)

=cut

sub call {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            timeout    => { type => SCALAR,  optional => 1 },
            on_timeout => { type => CODEREF, optional => 1 }
        },
        allow_extra => 1,    # all the options to RPC::Future->new
    );

    my $result_args = $self->_hash_move(
        source => \%opts,
        move   => [qw(timeout on_timeout)]
    );
    $Log->exit() if $may_exit;
    return $self->submit(%opts)->result(%$result_args);

} ## end sub call

#=head2 load_library
#
#  # Syntactic sugar for $worker->call(eval=>"require NACL::C::Foo");
#  $worker->load_library("NACL::C::Foo");
#
#Queue a request to C<require> a particular package.
#
#=cut
#
#sub load_library {
#    $Log->enter() if $may_enter;
#    my $self = shift;
#    my %opts = validate_with(
#        params => \@_,
#        spec   => { package => { type => SCALAR }, },
#    );
#    $self->call( eval => "require $opts{package};" );
#    $Log->exit() if $may_exit;
#
#} ## end sub load_library

=head2 register_global

  # Syntactic sugar for $worker->call(eval=>'$::Node = $_[0]',args=>[$node]);
  $worker->register_global(name=>'$::Node',value=>$node);

Queue a request to set the given name (lvalue) to the given
value.

=cut

sub register_global {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            name  => { type     => SCALAR },
            value => { optional => 0 },
        },
    );
    $self->call( eval => "$opts{name} = \$_[0]", args => [ $opts{value} ] );
    $Log->exit() if $may_exit;
} ## end sub register_global

=head2 kill

Kill the worker subprocess

=cut

sub kill {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_process()->kill();
    $Log->exit() if $may_exit;
} ## end sub kill

#=head2 cancel
#
#Foreach call in progress on this worker, call cancel() on that call
#(so that any replies or timeouts from that call will no longer cause
#any reaction; "on_done" callbacks will not be called) and remove that
#call from the list of calls for this worker.
#
#=cut
#
#sub cancel {
#    $Log->enter() if $may_enter;
#    my $self             = shift;
#    my @future_array     = $self->futures();
#    my $future_execution = shift @future_array;
#    if ( !defined $future_execution->status() ) {
#        $future_execution->_notify_list_reset();
#    } else {
#        $self->_remove_future_from_queue();
#    }
#    $Log->exit() if $may_exit;
#
#} ## end sub cancel

############ Helper functions ###################

sub _add_future_to_queue {
    $Log->enter() if $may_enter;
    my $self         = shift;
    my $future       = shift;
    my @future_array = $self->futures();
    push @future_array, $future;
    if (@future_array) {
        $self->futures(@future_array);
    } else {
        $self->futures_reset();
    }
    $Log->exit() if $may_exit;
} ## end sub _add_future_to_queue

sub _remove_future_from_queue {
    $Log->enter() if $may_enter;
    my $self         = shift;
    my @future_array = $self->futures();
    my $future       = shift @future_array;
    if (@future_array) {
        $self->futures(@future_array);
    } else {
        $self->futures_reset();
    }
    $Log->exit() if $may_exit;
    return $future;
} ## end sub _remove_future_from_queue

1;
