#
#   Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Vlan Task Module for Vlan in Linux/Solaris.
## @author dl-nacl-dev
## @status shared
## @pod here

package NACL::STask::Client::Vlan;

use strict;
use warnings;

use feature 'state';
use base qw(NACL::C::Client::Vlan NACL::STask::STask);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR UNDEF SCALARREF BOOLEAN);
use NACL::C::Client;
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::C::Exceptions::Client::Vlan::AlreadyExists qw(:try);
use NACL::STask::Exceptions::Client::Vlan::NotDeleted;
use NACL::STask::Exceptions::Client::Vlan::NotCreated;
use NACL::CS::Client::Vlan;

=head1 NAME

NACL::STask::Client::Vlan

=head1 DESCRIPTION

"NACL::STask::Client::Vlan" provide methods to create and purge Vlan on
a network Linux/Solaris Client.

It is created on top of "NACL::C::Client::Vlan" component. Since it is
derived class of "NACL::C::Client::Vlan", we can use all the methods of
"NACL::C::Client::Vlan" from the object of this task.

=head1 ATTRIBUTES

=head2 command_interface

(Required) A component object that represents the host to which to send
commands. See NACL::C::Component::command_interface.

=head2 vlan_id

(Required) Name of the vlan-id.

=head2 interface

(Required) Name of the interface.

=head2 vlan_interface

(Optional) Name of the vlan-interface.

=head1 METHODS

=head2 create

       my $vlan = NACL::STask::Client::Vlan->create(
      'command_interface'        =>  $Client,
      'vlan-id'                  =>  $Vlan_Id,
      'interface'                =>  $Interface,
      'nacltask_to_cleanup'      =>  1, # default 0,
      'nacltask_cleanup_manager' =>  $CleanupObj
       );

(Class Method) create requested Vlan on Linux/Solaris Client. This method
method provides additional services beyond what's inherent to the
product's Vlan creation commands, as controlled and described
by the "nacltask_if_exists" option.

Cleanup can be registered for the create method.

=over 

=item Options

=item C<< command_interface => $Client >>

(Required) A component object that represents the host to which
to send commands. See NACL::C::Component::command_interface.

=item C<< vlan-id => $Vlan_Id >>

(Required)vlan-id on Linux/Solaris Client.

=item C<< interface => $Interface >>

Required) Name of interface of which Vlan name to be created.

=item C<< vlan-interface => $vlan_interface >>

(Optional) Name of vlan-interface to create.

=item C<< _was_created => \$scalar >>

(Optional) When this option is provided a reference to a scalar
variable, the variable gets filled in with a boolean value
describing whether the Vlan was found (value will be 0;
this scenario is possible when nacltask_if_exists => "reuse") or
whether the Vlan was created (value will be 1). This is necessary
to determine whether the Vlan needs to be cleaned up later.

           my $was_created;
           my $vlan_obj = NACL::STask::Client::Vlan->create(
              if_exists    => 'reuse',
              _was_created => \$was_created,
              %other_opts
           );
           # Operate on $vlan_obj here
           # ...
           # Now determine whether to clean up the Vlan, since we're not sure
           # whether we reused an existing Vlan or created a new one
           if ($was_created) {
               # New Vlan was created. Clean it up.
               $vlan_obj->purge();
           }


=item C<< nacltask_if_exists => $action >>

(Optional) Specifies an action to be taken if the Vlan already
exists. If set to die an exception is thrown if VLAN already
exists. If set to reuse, no action is performed. And if set to
purge it will delete an existing VLAN before creating a new one.

=item C<< nacltask_verify => 0|1 >>

(Optional) If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the
Vlan creation did happen successfully.

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup) Flag indicating if this
operation needs to be registered for clean up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering. Default : Will use
the default cleanup manager. Cleanup is enabled for this method.

=item Other options

All of the other options such as command_interface, apiset_must,
apiset_should , etc are supported. And it also supports all the
options as described in
L<< NACL::C::Client::Vlan->create|lib-NACL-C-Client-Vlan-pm/create >>

=back

=over 

=item Exceptions

=item NACL::STask::Exceptions::Client::Vlan::NotCreated

=item Vlan Component exceptions

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    state $spec = {
        nacltask_if_exists => $pkg->_if_exists_validate_spec,
        nacltask_verify    => {type => BOOLEAN, default => 0},
        _was_created       => {type => SCALARREF, optional => 1},
        $pkg->_cleanup_validate_spec(),
    };

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => $spec,
        allow_extra     => 1,
    );

    my $command_interface = $opts{command_interface};
    my $if_exists   = delete $opts{nacltask_if_exists};
    my $verify      = delete $opts{nacltask_verify};
    my $was_created = delete $opts{_was_created};
    $$was_created   = 0;

    my (%opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup);
    $pkg->_move_common_cleanup_opts(
        source => \%opts,
        target => \%opts_for_cleanup,
    );

    my ($self,%common_opts,$vlan_interface);

    $pkg->_copy_common_component_params(
        source => \%opts,
        target => \%common_opts
    );

CREATE: {
        use warnings qw(exiting);
        try {

            $self         = $pkg->SUPER::create(%opts);
            $$was_created = 1;
        }
        catch NACL::C::Exceptions::Client::Vlan::AlreadyExists with {
            my $err_msg = shift;
            $self = $pkg->_element_exists_handler(
                create_opts        => \%opts,
                nacltask_if_exists => $if_exists,
                exception          => $err_msg
            );
            if (! defined $self) {
                no warnings 'exiting';
                redo CREATE;
            }
        };
    }
    $vlan_interface = $self->_get_vlan_interface(%opts);
    $self->vlan_interface($vlan_interface);
    if ($verify && $was_created) {
        my $found = NACL::CS::Client::Vlan->fetch(
            command_interface => $opts{command_interface},
            filter            => {'vlan-interface' => $vlan_interface},
            allow_empty       => 1,
            %common_opts,
        );
        if (!$found) {
            my $error = "Vlan was not created";
            $Log->exit() if $may_exit;
            NACL::STask::Exceptions::Client::Vlan::NotCreated->throw($error);
        }
    }

    if ($$was_created) {
    $pkg->_copy_common_opts_for_cleanup(
        'source'              => {%opts, %opts_for_cleanup},
        'target'              => \%opts_for_register,
        'nacltask_to_cleanup' => \$nacltask_to_cleanup,
        'to_cleanup'          => 'purge',
    );
    }

    $pkg->_register_for_cleanup(%opts_for_register)
       if ($nacltask_to_cleanup);

    return $self;
} ## end sub create

=head2 purge

    NACL::STask::Client::Vlan->purge(
        'command_interface'   => $Client,
        'vlan-id'             => $Vlan_Id,
        'interface'           => $Interface,
    );

    or

    $Vlan_obj->purge();

(Class or Instance Method) This task method will remove specified Vlan
from Client. It also verifies if the Vlan was deleted.

=over 

=item C<< command_interface => $ci >>

(Required for class method, Not Applicable for instance
method) A component object that represents the host to which
to send commands. See NACL::C::Component::command_interface.

=item C<< vlan-id => $vlan_id >>

(Required for class method, Not Applicable for instance
 method). vlan-id on Client.

=item C<< interface => $interface >>

(Required for class method, Not Applicable for instance
 method): Name of interface to delete.

=item C<< vlan-interface => $vlan_interface >>

(Required for class method, Not Applicable for instance
method) Name of vlan-interface to delete.

=item C<< nacltask_verify => 0 | 1 >>

(Optional) If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the
deletion did happen successfully.

=back

=over 

=item Exceptions

=item NACL::STask::Exceptions::Client::Vlan::NotDeleted

=item Vlan Component exceptions

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    state $spec = {nacltask_verify => {type => BOOLEAN, default => 0},};

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => $spec,
    );

    my %common_opts;
    $pkg_or_obj->_copy_common_component_params(
        source => \%opts,
        target => \%common_opts
    );

    my $command_interface = $opts{command_interface};

    my $verify = delete $opts{nacltask_verify};

    $pkg_or_obj->delete(%opts);

    if ($verify) {
        my $vlan_interface = $pkg_or_obj->_get_vlan_interface(%opts);
        my $found          = NACL::CS::Client::Vlan->fetch(
            command_interface => $opts{command_interface},
            filter            => {'vlan-interface' => $vlan_interface},
            allow_empty       => 1,
            %common_opts,
        );
        if ($found) {
            my $error = "Vlan $vlan_interface not deleted \n";
            $Log->exit() if $may_exit;
            NACL::STask::Exceptions::Client::Vlan::NotDeleted->throw($error);
        }
    }

    $Log->exit() if $may_exit;
} ## end sub purge

1;
