# $Id: //depot/prod/test/nacldev/lib/NACL/STask/DFM/Group.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Group Task Module
## @author Ramya M R, dl-mpo-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::DFM::Group;

use strict;
use warnings;

use base qw(NACL::C::DFM::Group NACL::STask::DFM);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR SCALARREF ARRAYREF BOOLEAN OBJECT);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use NACL::ComponentUtils qw(_optional_scalars);

use Data::Dumper;

=head1 NAME

NACL::STask::DFM::Group

=head1 DESCRIPTION

C<NACL::STask::DFM::Group> provides a number of well-defined but
potentially complex or multi-step methods related to Group in
OnCommand.

It builds on top of, and is a derived class of C<NACL::C::DFM::Group>,
and so it also provides methods that are more in the scope of
individual Group-related commands.  See C<NACL::C::DFM::Group> for
details.  This also means that a C<NACL::STask::DFM::Group> object may
generally be used in place of a component object.

=head1 METHODS

=head2 create

  my $Group  = NACL::STask::DFM::Group->create(
      command_interface  => $command_interface,
      group_name         => $group_name,	     
      nacltask_if_exists => $action,      # default 'reuse'
      %other_options
  );

(Class method) Create a Group.  This method provides additional
services beyond what's inherent to the product's group creation
commands, as controlled and described by the new C<nacltask_if_exists> option.

=over

=item Options

=over

=item C<< command_interface => $command_interface >> 
(Required)

(Required) As C<NACL::C::DFM::DFMClient>.  A component object that
represents the host to which to send commands.

=item C<< group_name => $group_name >> 

(Required) The name of the group that is to be created.

=item C<< nacltask_if_exists => $action >>

(Optional) What to do if the group to be created already exists.
If action is "reuse" (the default), then do nothing (return a 
task object referring to the existing group). If $action is "die",
then fail with an exception . If action is "purge",
then delete the group (see the C<destroy> method, below) before 
creating a new one.

item C<< _was_created => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the role
was found (value will be 0; this scenario is possible when 
if_exists => "reuse") or whether the role was created (value will be 1).
This is necessary to determine whether the created needs to be
cleaned up later.

    my $was_created;
    my $group = NACL::STask::DFM::Group->create(
        nacltask_if_exists    => 'reuse',
        _was_created => \$was_created,
        %other_opts
    );
    # Operate on $group here
    # ...
    # Now determine whether to destroy the group, since we're not sure
    # whether we reused an existing group or created a new one
    if ($was_created) {
        # New Role was created. Clean it up.
        $group->destroy();
    }

=item command_interface, apiset_must, apiset_should, method-timeout, parent-group-name-or-id etc.

All of the other various options to
L<< NACL::C::DFM::Group->create | lib-NACL-C-DFM-Group-pm/create >>

=back

=back

=cut

sub create {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;
    my $pkg        = $pkg_or_obj->get_package_name();
    my $self;

    my %opts = $pkg_or_obj->_common_validate_with(

        params          => \@_,
        additional_spec => {
            nacltask_if_exists => { type => SCALAR,    default  => 'reuse' },
            _was_created       => { type => SCALARREF, optional => 1 },
        },
        ignore_primary_keys => 1,
        allow_extra         => 1,
    );
    my $if_exists   = delete $opts{nacltask_if_exists};
    my $was_created = delete $opts{_was_created};

CREATE: {
        use warnings qw(exiting);
        try {
            $self         = $pkg->SUPER::create(%opts);
            $$was_created = 1;
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception = shift;
            my $text      = $exception->text();

            if ( $text =~ /Group.*already exists/ ) {

                $Log->trace('The Group already exists');
                $self = $pkg->_element_exists_handler(
                    create_opts        => \%opts,
                    nacltask_if_exists => $if_exists,
                    exception          => $exception
                );
                if ( !$self ) {
                    no warnings qw(exiting);
                    redo CREATE;
                }
            }

        };
    }

    ## Return the STask object
    return $self;

}

=head2 purge

    $group->purge(%other_options);

    NACL::STask::DFM::Group->purge(
      command_interface   => $command_interface,
      'group-name-or-id'  => $role_name_or_id, 
      nacltask_if_purged  => $action,     # default 'die' 
      nacltask_verify     => 0|1,	  # default 0  
      %other_options
    );

(Class or instance method) This method removes a group.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<<group_name_or_id => $group_name_or_id  >>

(Required) The name or id of the group to be removed.

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that
the deletion did happen successfully.

=item C<< nacltask_if_purged => die | pass >>

(Optional, defaults to "die")
Specifies what to do if the group we're trying to delete no longer
exists. A value of C<die> (the default) will result in an exception
being thrown. A value of C<pass> will mean that the exception that would
normally occur when we're trying to destroy a group which does not exist
is suppressed.

=back

=back

=cut

sub purge {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;
    my $pkg        = $pkg_or_obj->get_package_name();

    my %opts = $pkg_or_obj->_common_validate_with(

        params          => \@_,
        additional_spec => {
            nacltask_if_purged => { type => SCALAR, default => 'die' },
            nacltask_verify    => { type => SCALAR, default => 0 },
        },
        allow_extra => 1,
    );

    my $command_interface = $opts{command_interface};
    my $group_name_or_id  = $opts{'group-name-or-id'};

    # Task specific options, not used for purge
    # nacltask_if_purged, nacltask_verify
    my $if_purged   = delete $opts{nacltask_if_purged};
    my $verify_flag = delete $opts{nacltask_verify};

    try {

        $pkg->SUPER::destroy(%opts);

        # If need to verify the role deletion
        if ($verify_flag) {

            $pkg->taskverify_destroy(%opts);

        }
        $Log->log("Group deletion successful.");

    }
    catch NACL::APISet::Exceptions::ResponseException with {

        my $exception = shift;
        my $text      = $exception->text();

        my $expected_error_message = "There is no resource group named.*";

        if (   ( $text !~ $expected_error_message )
            || ( $if_purged !~ /pass/i ) )
        {

            $Log->exit() if $may_exit;

            # Rethrow the exception
            $exception->throw();

        }
    };

    $Log->exit() if $may_exit;
}

=head2 add_member

	$group->add_member(%other_options);
	
	NACL::STask::DFM::Group->add_member(
        command_interface  => $command_interface,
        'group_name_or_id'  =>  $group_name_or_id, 
        'member-name-or-id' => $member_name_or_id,      
        nacltask_if_member_already_added => $action,      # default 'pass'
      %other_options
  );

(Class method) Add member to a group.  This method provides additional
services beyond what's inherent to the product's member addition 
command, as controlled and described by the new C<nacltask_if_member_already_added> option.

=over

=item Options

=over

=item C<< command_interface => $command_interface >> 
(Required)

(Required) As C<NACL::C::DFM::DFMClient>.  A component object that
represents the host to which to send commands.

=item C<< group_name_or_id => $group_name_or_id >> 

(Required for Class Method) The name or id of the group that to which the member is to be added.

=item C<< nacltask_if_member_already_added => $action >>

(Optional) What to do if the member to be added is already added to the group.
If action is "pass" (the default), then do nothing . If $action is "die",
then fail with an exception . 

item C<< _was_added => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the member
was already added (value will be 0; this scenario is possible when 
if_capability_already_added => "pass") or whether the member was added (value will be 1).
This is necessary to determine whether the added member needs to be
cleaned up later.

    my $was_added;
	NACL::STask::DFM::Group->add_member(
        nacltask_if_member_already_added  => 'pass',
        _was_added                        => \$was_added,
        %other_opts
    );
    # Operate on $group here
    # ...
    # Now determine whether to remove the capability, since we're not sure
    # whether we reused the existing member or added the member to the group.
    if ($was_added) {
        # New member was added. Clean it up.
        $group->delete_member();
    }
=back

=item command_interface, apiset_must, apiset_should, method-timeout,etc.

All of the other various options to
L<< NACL::C::DFM::Group->add_member | lib-NACL-C-DFM-Rbac-pm/add_member >>

=back

=back

=cut

sub add_member {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;
    my $pkg        = $pkg_or_obj->get_package_name();

    my %opts = $pkg_or_obj->_common_validate_with(

        params          => \@_,
        additional_spec => {
            nacltask_if_member_already_added =>
                { type => SCALAR, default => 'pass' },
            _was_added => { type => SCALARREF, optional => 1 },
        },
        allow_extra => 1,
    );

    my $command_interface = $opts{command_interface};
    my $group_name_or_id  = $opts{'group-name-or-id'};
    my $if_member_added   = delete $opts{nacltask_if_member_already_added};
    my $was_added         = delete $opts{_was_added};
    $$was_added = 0;

    # Call the component capability_add()
    try {

        $pkg->SUPER::add_member(%opts);
        $$was_added = 1;
    }
    catch NACL::APISet::Exceptions::ResponseException with {

        my $exception = shift;
        my $text      = $exception->text();
        my $expected_error_message
            = "Proposed member.*is already in the group.*";

       # Throw the exception if it's not the exception message we're expecting
       # Or if the nacltask_if_member_already_added option is die
        if (   ( $text !~ $expected_error_message )
            || ( $if_member_added =~ /die/i ) )
        {
            $Log->exit() if $may_exit;

            # Rethrow the exception
            $exception->throw();

        } else {
            $Log->log(
                "The proposed member is already added to the group. Hence,reusing the existing one"
            );
        }

    };    ## end of catch

    $Log->exit() if $may_exit;

}

=head2 delete_member

   NACL::STask::DFM::Group->delete_member(  command_interface   => $ci,
                                            'member-name-or-id' => $member_name_or_id,
			        	    'group-name-or-id'  => $group_name_or_id,
                                  ...);
                                      or

    $group->delete_member(...);

=for html (Class or instance method)

This method is used to remove a member from an existing group.  

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'group-name-or-id' => $group_name_or_id >>

(Required for class method, Not Applicable for instance method)

=item C<< 'member-name-or-id' => $member_name_or_id >>

=item C<< 'method-timeout' => $time_in_seconds >>

=item C<< nacltask_if_member_deleted => die | pass >>

(Optional, defaults to "pass")

Specifies what to do if the member we're trying to delete is already deleted
 A value of C<die> will result in an exception
being thrown. A value of C<pass> (default value) will mean that the exception that would
normally occur when we're trying to delete a member from a group which does not exist
is suppressed.


(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub delete_member {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;
    my $pkg        = $pkg_or_obj->get_package_name();

    my %opts = $pkg_or_obj->_common_validate_with(

        params          => \@_,
        additional_spec => {
            nacltask_if_member_deleted =>
                { type => SCALAR, default => 'pass' },
            nacltask_verify => { type => SCALAR, default => 0 },
        },
        allow_extra => 1,
    );

    # Task specific options, not used for purge
    # nacltask_if_member_deleted, nacltask_verify
    my $if_deleted  = delete $opts{nacltask_if_member_deleted};
    my $verify_flag = delete $opts{nacltask_verify};

    try {

        $pkg->SUPER::delete_member(%opts);

        # If need to verify the user role deletion
        if ($verify_flag) {

            $pkg->taskverify_delete_member(%opts);

        }

        $Log->log("Group Member Deletion successful.");

    }
    catch NACL::APISet::Exceptions::ResponseException with {

        my $exception = shift;
        my $text      = $exception->text();

        my $expected_error_message = "Member.*was not found in group.*";

        if (   ( $text !~ $expected_error_message )
            || ( $if_deleted !~ /pass/i ) )
        {

            $Log->exit() if $may_exit;

            # Rethrow the exception
            $exception->throw();

        }
    };

    $Log->exit() if $may_exit;
}

## This method is an alias for purge
sub destroy {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->purge(@_);
    $Log->exit() if $may_exit;
}

=head2 taskverify_delete_member

    $group->taskverify_delete_member(%other_options);

    NACL::STask::DFM::Group->taskverify_delete_member(
      command_interface   => $command_interface,
      'group-name-or-id'  => $group_name_or_id, 
      'member-name-or-id' => $member_name_or_id, 
      %other_options
    );

(Class or instance method) This method verifies if the member is deleted from the group or not.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<<group_name_or_id => $group_name_or_id  >>

(Required for Class method, Not Applicable for instance method) The name or id of the group.

=item C<< 'member-name-or-id' => $string >>

=back

=back

=cut

sub taskverify_delete_member {

    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;
    my $member_pkg        = NACL::CS::DFM::GroupMember->new();
    my $command_interface = $opts{command_interface};
    my $member_name_or_id = $opts{'member-name-or-id'};
    my $group_name_or_id  = $opts{'group-name-or-id'};
    my $group_member_state_object;

    try {
        $group_member_state_object = $member_pkg->fetch(
            command_interface => $command_interface,
            filter            => {
                'id'       => $member_name_or_id,
                'group-id' => $group_name_or_id,
            },

        );

        $Log->exit() if $may_exit;
        NACL::Exceptions::VerifyFailure->throw(
                  "Verification of Member Deletion "
                . " for the group member FAILED!" );

    }
    catch NACL::Exceptions::NoElementsFound with {
        my $exception_object = shift;
        $Log->log("Member deletion verified successfully.");
    };

    $Log->exit() if $may_exit;
}

sub taskverify_destroy {
    my ( $pkg_or_obj, %opts ) = @_;
    $pkg_or_obj->_generic_purge_verify(%opts);
}
1;

