# $Id: $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Option Task Module
## @author Poornima, dl-mpo-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::DFM::Option;

use strict;
use warnings;

use base qw(NACL::C::DFM::Option NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate
  qw(validate SCALAR SCALARREF ARRAYREF HASHREF BOOLEAN OBJECT);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use NACL::ComponentUtils qw(_optional_scalars);
use NACL::MTask::DFM::Monitor;

use Data::Dumper;

=head1 NAME

NACL::STask::DFM::Option

=head1 DESCRIPTION

C<NACL::STask::DFM::Option> provides a number of well-defined but
potentially complex or multi-step methods related to Options in
OnCommand.

It builds on top of, and is a derived class of C<NACL::C::DFM::Option>,
and so it also provides methods that are more in the scope of
individual Option-related commands.  See C<NACL::C::DFM::Option> for
details.  This also means that a C<NACL::STask::DFM::Option> object may
generally be used in place of a component object.

=head1 METHODS

=head2 set  

    NACL::STask::DFM::Option->set( 
	'command_interface' => $command_interface,
	'option-names-and-values' => {  $opt1 => $val1,
					$opt2 => $val2,
					....
				     }, 
        %other_options
    );

(Class method) This method sets the options to values and 
restarts the dfmserver if necessary. 

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< option_names_and_values => HASHREF  >>

(Required) A hashref containing the names of the option as it's keys
and the values to which it should be set as the values.  

=item C<< resource-key => $resource-key >>

(Optional) Resource key of the object for which to override the global option
 value. 

=item C<< username => $username >>

(Optional) Username using which the OC server is restarted (if required)
to get the new values in effect.

=item C<< password => $password >>

(Optional) The password given along with the username to login to the OC 
server to restart it (if required) to get the new values in effect. 

=item C<< wait_time => $scalar >>

(Optional, defaults to 300 if not provided )
This is the amount of time (in SECONDS)  for which the method waits for
the Bard server to get refreshed.

=item C<< polling_interval => $scalar >>

(Optional, defaults to 60 if not provided )
This is the interval (in SECONDS) , for which periodically a check is done
to see if the server is running.

=back

=item command_interface, apiset_must, apiset_should, method-timeout, timestamp,
etc.

All of the other various options to
L<< NACL::C::DFM::Option | lib-NACL-C-DFM-Option-pm >>

=back

=cut

sub set {

    $Log->enter() if $may_enter;

    my $pkg = shift;

    my %opts = $pkg->_common_validate_with(

        params => \@_,
        additional_spec =>
          { 'option-names-and-values' => { type => HASHREF }, },
        allow_extra         => 1,
        ignore_primary_keys => 1,
    );

    my $dfm_client = $opts{command_interface};
    my %opts_hash  = %{ $opts{'option-names-and-values'} };
    my ($set_resp, %restart_hash);

    $pkg->_hash_move(
        source => \%opts,
        target => \%restart_hash,
        move   => [qw(username password wait_time polling_interval )],
    );

    # Flag to check if restart is needed
    my $restart_needed = 0;

    # Loop through all the options, and set $restart_needed if
    # any of the options have 'requires_restart' field as 'true'
    foreach my $option ( keys(%opts_hash) ) {

        # Create component object
        my $option_obj = NACL::C::DFM::Option->new(
            command_interface => $dfm_client,
            'option-name'     => $option,
        );

        if ( defined $opts{'resource-key'} ) {

            # Set the value and resource key
            $set_resp = $option_obj->set(
                'option-value' => $opts_hash{$option},
                'resource-key' => $opts{'resource-key'}
            );
        }
        else {

            # Set the value
            $set_resp =
              $option_obj->set( 'option-value' => $opts_hash{$option} );
        }

        # Check for restart
        if ( $set_resp->{'requires-restart'} eq 'true' ) {
            $restart_needed = 1;
        }

    }

    # Check for restart
    if ($restart_needed) {

        $Log->debug("Restart required. Calling restart of DFMClient");

        # Call restart
        $dfm_client->restart(%restart_hash);
    }

    $Log->exit() if $may_exit;
}

1;
