# $Id: //depot/prod/test/nacldev/lib/NACL/STask/DFM/Rbac.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Rbac Task Module
## @author Ramya M R, dl-mpo-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::DFM::Rbac;

use strict;
use warnings;

use base qw(NACL::C::DFM::Rbac NACL::STask::DFM);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR SCALARREF ARRAYREF BOOLEAN OBJECT);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use NACL::ComponentUtils qw(_optional_scalars);

use Data::Dumper;

=head1 NAME

NACL::STask::DFM::Rbac

=head1 DESCRIPTION

C<NACL::STask::DFM::Rbac> provides a number of well-defined but
potentially complex or multi-step methods related to Rbac User in
OnCommand.

It builds on top of, and is a derived class of C<NACL::C::DFM::Rbac>,
and so it also provides methods that are more in the scope of
individual Rbac-related commands.  See C<NACL::C::DFM::Rbac> for
details.  This also means that a C<NACL::STask::DFM::Rbac> object may
generally be used in place of a component object.

=head1 METHODS

=head2 create

  my $Rbac  = NACL::STask::DFM::Rbac->create(
      command_interface  => $command_interface,
      role_name		     => $role_name, 
      nacltask_if_exists => $action,      # default 'reuse'
      %other_options
  );

(Class method) Create a Rbac Role.  This method provides additional
services beyond what's inherent to the product's role creation
commands, as controlled and described by the new C<nacltask_if_exists> option.

=over

=item Options

=over

=item C<< command_interface => $command_interface >> 
(Required)

(Required) As C<NACL::C::DFM::DFMClient>.  A component object that
represents the host to which to send commands.

=item C<< role_name => $role_name >> 

(Required) The name of the role that is to be created.

=item C<< nacltask_if_exists => $action >>

(Optional) What to do if the role to be created already exists.
If action is "reuse" (the default), then do nothing (return a 
task object referring to the existing role). If $action is "die",
then fail with an exception . If action is "purge",
then delete the role (see the C<destroy> method, below) before 
creating a new one.

item C<< _was_created => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the role
was found (value will be 0; this scenario is possible when 
if_exists => "reuse") or whether the role was created (value will be 1).
This is necessary to determine whether the created needs to be
cleaned up later.

    my $was_created;
    my $rbac = NACL::STask::DFM::Rbac->create(
        nacltask_if_exists    => 'reuse',
        _was_created => \$was_created,
        %other_opts
    );
    # Operate on $rbac here
    # ...
    # Now determine whether to destroy the role, since we're not sure
    # whether we reused an existing role or created a new one
    if ($was_created) {
        # New Role was created. Clean it up.
        $rbac->destroy();
    }
=back

=item command_interface, apiset_must, apiset_should, method-timeout, owner-role-name-or-id,description etc.

All of the other various options to
L<< NACL::C::DFM::Rbac->create | lib-NACL-C-DFM-Rbac-pm/create >>

=back

=cut

sub create {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;
    my $pkg        = $pkg_or_obj->get_package_name();
    my $self;

    my %opts = $pkg_or_obj->_common_validate_with(

        params          => \@_,
        additional_spec => {
            nacltask_if_exists => { type => SCALAR,    default  => 'reuse' },
            _was_created       => { type => SCALARREF, optional => 1 },
        },
        ignore_primary_keys => 1,
        allow_extra         => 1,
    );
    my $if_exists   = delete $opts{nacltask_if_exists};
    my $was_created = delete $opts{_was_created};

CREATE: {
        use warnings qw(exiting);
        try {
            $self         = $pkg->SUPER::create(%opts);
            $$was_created = 1;
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception = shift;
            my $text      = $exception->text();

            if ( $text =~ /Role.*already exists/ ) {

                $Log->trace('The Role already exists');
                $self = $pkg->_element_exists_handler(
                    create_opts        => \%opts,
                    nacltask_if_exists => $if_exists,
                    exception          => $exception
                );
                if ( !$self ) {
                    no warnings qw(exiting);
                    redo CREATE;
                }
            }

        };
    }

    $Log->exit() if $may_exit;

    ## Return the STask object
    return $self;

}

=head2 purge

    $rbac_user->purge(%other_options);

    NACL::STask::DFM::Rbac->purge(
      command_interface   => $command_interface,
      'role-name-or-id'  => $role_name_or_id, 
      nacltask_if_purged  => $action,     # default 'die' 
      nacltask_verify     => 0|1,	  # default 0  
      %other_options
    );

(Class or instance method) This method removes a role.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<<role_name_or_id => $role_name_or_id  >>

(Required) The name or id of the role to be removed.

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that
the deletion did happen successfully.

=item C<< nacltask_if_purged => die | pass >>

(Optional, defaults to "die")
Specifies what to do if the role we're trying to delete no longer
exists. A value of C<die> (the default) will result in an exception
being thrown. A value of C<pass> will mean that the exception that would
normally occur when we're trying to destroy a role which does not exist
is suppressed.

=back

=back

=cut

sub purge {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;
    my $pkg        = $pkg_or_obj->get_package_name();

    my %opts = $pkg_or_obj->_common_validate_with(

        params          => \@_,
        additional_spec => {
            nacltask_if_purged => { type => SCALAR, default => 'die' },
            nacltask_verify    => { type => SCALAR, default => 0 },
        },
        allow_extra => 1,
    );

    my $command_interface = $opts{command_interface};
    my $role_name_or_id   = $opts{'role-name-or-id'};

    # Task specific options, not used for purge
    # nacltask_if_purged, nacltask_verify
    my $if_purged   = delete $opts{nacltask_if_purged};
    my $verify_flag = delete $opts{nacltask_verify};

    try {

        $pkg->SUPER::destroy(%opts);

        # If need to verify the role deletion
        if ($verify_flag) {

            $pkg->taskverify_purge(%opts);

        }
        $Log->trace("Role deletion successful.");

    }
    catch NACL::APISet::Exceptions::ResponseException with {

        my $exception = shift;
        my $text      = $exception->text();

        my $expected_error_message = "Role.*does not exist";

        if (   ( $text !~ $expected_error_message )
            || ( $if_purged !~ /pass/i ) )
        {

            $Log->exit() if $may_exit;

            # Rethrow the exception
            $exception->throw();

        }
    };

    $Log->exit() if $may_exit;
}

=head2 capability_add

  NACL::STask::DFM::Rbac->capability_add(
      command_interface  => $command_interface,
     'role_name_or_id'    => $role_name_or_id, 
	 'operation' => $operation,
     'resource-name-or-id' => $resource_name_or_id,      
      nacltask_if_capability_already_added => $action,      # default 'pass'
      %other_options
  );

(Class method) Add Capability to a Rbac Role.  This method provides additional
services beyond what's inherent to the product's capability addition 
command, as controlled and described by the new C<nacltask_if_capability_already_added> option.

=over

=item Options

=over

=item C<< command_interface => $command_interface >> 
(Required)

(Required) As C<NACL::C::DFM::DFMClient>.  A component object that
represents the host to which to send commands.

=item C<< role_name_or_id => $role_name_or_id >> 

(Required for Class Method) The name or id of the role that to which the capability is to be added.

=item C<< nacltask_if_capability_already_added => $action >>

(Optional) What to do if the capability to be added is already added to the role.
If action is "pass" (the default), then do nothing . If $action is "die",
then fail with an exception . 

item C<< _was_added => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the role
was already assigned (value will be 0; this scenario is possible when 
if_capability_already_added => "pass") or whether the role was added (value will be 1).
This is necessary to determine whether the added capability needs to be
cleaned up later.

    my $was_added;
	NACL::STask::DFM::Rbac->capability_add(
        nacltask_if_capability_already_added  => 'pass',
        _was_added => \$was_added,
        %other_opts
    );
    # Operate on $rbac here
    # ...
    # Now determine whether to remove the capability, since we're not sure
    # whether we reused an existing role or added the role to the user.
    if ($was_added) {
        # New Role was added. Clean it up.
        $rbac->capability_remove();
    }
=back

=item command_interface, apiset_must, apiset_should, method-timeout, email_address,etc.

All of the other various options to
L<< NACL::C::DFM::Rbac->role_capability_add | lib-NACL-C-DFM-Rbac-pm/role_capability_add >>

=back

=cut

sub capability_add {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;
    my $pkg        = $pkg_or_obj->get_package_name();

    my %opts = $pkg_or_obj->_common_validate_with(

        params          => \@_,
        additional_spec => {
            nacltask_if_capability_already_added =>
                { type => SCALAR, default => 'pass' },
            _was_added => { type => SCALARREF, optional => 1 },
        },
        allow_extra => 1,
    );

    my $command_interface = $opts{command_interface};
    my $role_name_or_id   = $opts{'role-name-or-id'};
    my $if_capability_added
        = delete $opts{nacltask_if_capability_already_added};
    my $was_added = delete $opts{_was_added};
    $$was_added = 0;

    # Call the component capability_add()
    try {

        $pkg->SUPER::capability_add(%opts);
        $$was_added = 1;
    }
    catch NACL::APISet::Exceptions::ResponseException with {

        my $exception = shift;
        my $text      = $exception->text();
        my $expected_error_message
            = "Role.*has already been assigned with the specified capability";

       # Throw the exception if it's not the exception message we're expecting
       # Or if the nacltask_if_capability_already_added option is die
        if (   ( $text !~ $expected_error_message )
            || ( $if_capability_added =~ /die/i ) )
        {

            $Log->exit() if $may_exit;

            # Rethrow the exception
            $exception->throw();

        } else {
            $Log->trace(
                "The specified capability is already added to the role. Hence,reusing the existing one"
            );
        }

    };    ## end of catch

    $Log->exit() if $may_exit;

}

=head2 capability_remove

   NACL::STask::DFM::Rbac->capability_remove(  command_interface => $ci,
                                'role-name-or-id' => $role_name_or_id,
                                  ...);
                                      or

    $Rbac_object->capability_remove(...);

=for html (Class or instance method)

This method is used to  remove one or more capabilities (resource/operation pair) from an existing
role. If delete-all is true, it removes all capabilities from given role.
Otherwise, it removes only the given capability (resource/operation pair). If
delete-all is not specified or is false, then operation and resource must be
specified.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'role-name-or-id' => $role_name_or_id >>

(Required for class method, Not Applicable for instance method)

=item C<< 'delete-all' => $string >>

=item C<< 'operation' => $string >>

=item C<< 'resource-name-or-id' => $string >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub capability_remove {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;
    my $pkg        = $pkg_or_obj->get_package_name();

    my %opts = $pkg_or_obj->_common_validate_with(

        params          => \@_,
        additional_spec => {
            nacltask_if_capability_removed =>
                { type => SCALAR, default => 'pass' },
            nacltask_verify => { type => SCALAR, default => 0 },
        },
        allow_extra => 1,
    );

    # Task specific options, not used for purge
    # nacltask_if_user_role_removed, nacltask_verify
    my $if_removed  = delete $opts{nacltask_if_capability_removed};
    my $verify_flag = delete $opts{nacltask_verify};

    try {

        $pkg->SUPER::capability_remove(%opts);

        # If need to verify the user role deletion
        if ($verify_flag) {

            $pkg->taskverify_capability_remove(%opts);

        }

        $Log->log("Role Capability Deletion successful.");

    }
    catch NACL::APISet::Exceptions::ResponseException with {

        my $exception = shift;
        my $text      = $exception->text();

        my $expected_error_message
            = "Role.*does not have the specified capability";

        if (   ( $text !~ $expected_error_message )
            || ( $if_removed !~ /pass/i ) )
        {

            $Log->exit() if $may_exit;

            # Rethrow the exception
            $exception->throw();

        }
    };

    $Log->exit() if $may_exit;
}

## This method is an alias for purge()
sub destroy {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->purge(@_);
    $Log->exit() if $may_exit;
}

=head2 taskverify_capability_remove

    $rbac->taskverify_capability_remove(%other_options);

    NACL::STask::DFM::Rbac->taskverify_capability_remove(
      command_interface   => $command_interface,
	  'role-name-or-id' => $role_name_or_id, 
      %other_options
    );

(Class or instance method) This method verifies if the capability of a role is removed or not.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<<role_name_or_id => $role_name_or_id  >>

(Required for Class method, Not Applicable for instance method) The name or id of the role whose capability is removed.

=item C<< 'delete-all' => $string >>

=item C<< 'operation' => $string >>

=item C<< 'resource-name-or-id' => $string >>

=back

=back

=cut

sub taskverify_capability_remove {

    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, %opts ) = @_;
    my $pkg               = $pkg_or_obj->get_CS_package_name();
    my $command_interface = $opts{command_interface};
    my $role_name_or_id   = $opts{'role-name-or-id'};
    my ( $resource_name_or_id, $operation );
    my $role_state_object;

    if (   defined( $opts{'resource-name-or-id'} )
        && defined( $opts{'operation'} ) )
    {

        $resource_name_or_id = $opts{'resource-name-or-id'};
        $operation           = $opts{'operation'};

    }
    $role_state_object = $pkg->fetch(
        command_interface => $command_interface,
        filter            => { 'role-name-or-id' => $role_name_or_id },

    );

    ## Verifying if the capability was removed or not

    ## If delete-all option is true, then verify if all capabilities associated with the role are removed.

    if ( $opts{'delete-all'} =~ /true/ ) {
        if ( $role_state_object->capabilities_isset() ) {

            $Log->exit() if $may_exit;
            NACL::Exceptions::VerifyFailure->throw(
                      "Verification of Capability Deletion "
                    . " for role $role_name_or_id FAILED!" );
        } else {
            $Log->trace("Capability Deletion verified successfully.");
        }
    }

    ## If delete-all option is false( default ), then verify if the specified capability associated with the role is removed.

    else {
        my $capabilities = $role_state_object->capabilities();
        my @capabilities = @{$capabilities};
        foreach my $row (@capabilities) {

            if ((   (   $row->{'resource'}->[0]->{'rbac-resource'}->[0]
                        ->{'resource-id'} eq $resource_name_or_id
                    )
                    || ( $row->{'resource'}->[0]->{'rbac-resource'}->[0]
                        ->{'resource-name'} eq $resource_name_or_id )
                )
                && $row->{'operation'} =~ /$operation/
                )
            {

                $Log->exit() if $may_exit;
                NACL::Exceptions::VerifyFailure->throw(
                          "Verification of Capability Deletion "
                        . " for role $role_name_or_id FAILED!" );

            } else {
                $Log->trace("Capability Deletion verified successfully.");
            }
        }
    }
    $Log->exit() if $may_exit;
}

=head2 taskverify_purge

    $rbac->taskverify_purge(%other_options);

    NACL::STask::DFM::Rbac->taskverify_purge(
      command_interface   => $command_interface,
	  'role-name-or-id' => $role_name_or_id, 
      %other_options
    );

(Class or instance method) This method verifies if a role is removed or not.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<<role_name_or_id => $role_name_or_id  >>

(Required for Class method, Not Applicable for instance method) The name or id of the role which is removed.

=back

=back

=cut

sub taskverify_purge {
    my ( $pkg_or_obj, %opts ) = @_;
    $pkg_or_obj->_generic_purge_verify(%opts);
}
1;

