# $Id: //depot/prod/test/nacldev/lib/NACL/STask/DFM/RbacUser.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary RbacUser Task Module
## @author Ramya M R, dl-mpo-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::DFM::RbacUser;

use strict;
use warnings;

use base qw(NACL::C::DFM::RbacUser NACL::STask::DFM);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR SCALARREF ARRAYREF BOOLEAN OBJECT);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use NACL::ComponentUtils qw(_optional_scalars);

use Data::Dumper;

=head1 NAME

NACL::STask::DFM::RbacUser

=head1 DESCRIPTION

C<NACL::STask::DFM::RbacUser> provides a number of well-defined but
potentially complex or multi-step methods related to Rbac User in
OnCommand.

It builds on top of, and is a derived class of C<NACL::C::DFM::RbacUser>,
and so it also provides methods that are more in the scope of
individual RbacUser-related commands.  See C<NACL::C::DFM::RbacUser> for
details.  This also means that a C<NACL::STask::DFM::RbacUser> object may
generally be used in place of a component object.

=head1 METHODS

=head2 add

  my $rbacuser  = NACL::STask::DFM::RbacUser->add(
      command_interface  => $command_interface,
      'user-name'		     => $user_name,
      'email-address'      	=> $email_address,  
      nacltask_if_exists => $action,      # default 'reuse'
      %other_options
  );

(Class method) Add a Rbac User.  This method provides additional
services beyond what's inherent to the product's user addition 
commands, as controlled and described by the new C<nacltask_if_exists> option.

=over

=item Options

=over

=item C<< command_interface => $command_interface >> 
(Required)

(Required) As C<NACL::C::DFM::DFMClient>.  A component object that
represents the host to which to send commands.

=item C<< user-name => $user_name >> 

(Required) The name of the user that is to be added.

=item C<< nacltask_if_exists => $action >>

(Optional) What to do if the user to be added already exists.
If action is "reuse" (the default), then do nothing (return a 
task object referring to the existing user). If $action is "die",
then fail with an exception . If action is "purge",
then delete the user (see the C<remove> method, below) before 
creating a new one.

item C<< _was_added => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the user
was found (value will be 0; this scenario is possible when 
if_exists => "reuse") or whether the user was added (value will be 1).
This is necessary to determine whether the added user needs to be
cleaned up later.

    my $was_added;
    my $rbac_user = NACL::STask::DFM::RbacUser->add(
        nacltask_if_exists    => 'reuse',
        _was_added => \$was_added,
        %other_opts
    );
    # Operate on $rbac_user here
    # ...
    # Now determine whether to remove the user, since we're not sure
    # whether we reused an existing user or created a new one
    if ($was_added) {
        # New user was created. Clean it up.
        $rbac_user->remove();
    }
=back

=item command_interface, apiset_must, apiset_should, method-timeout, email_address,etc.

All of the other various options to
L<< NACL::C::DFM::RbacUser->add | lib-NACL-C-DFM-RbacUser-pm/add >>

=back

=cut

sub add {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;
    my $pkg        = $pkg_or_obj->get_package_name();
    my $self;

    my %opts = $pkg_or_obj->_common_validate_with(

        params          => \@_,
        additional_spec => {
            nacltask_if_exists => { type => SCALAR,    default  => 'reuse' },
            _was_added         => { type => SCALARREF, optional => 1 },
        },
        ignore_primary_keys => 1,
        allow_extra         => 1,
    );
    my $if_exists = delete $opts{nacltask_if_exists};
    my $was_added = delete $opts{_was_added};
    my $random_name_generated;

    ## If user name is not provided , then create a user with a random name
    if ( !defined $opts{'user-name'} ) {
        $opts{'user-name'} = $pkg->random_name_generator();
        $random_name_generated = 1;
    }

CREATE: {
        use warnings qw(exiting);
        try {
            $self      = $pkg->SUPER::add(%opts);
            $$was_added = 1;
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception = shift;
            my $text      = $exception->text();

            if ( $text =~ /User.*already exists/ ) {
                if ($random_name_generated) {
                    $Log->trace( 'Random name collision, regenerate a '
                            . 'new name and retry' );
                    $opts{'user-name'} = $pkg->random_name_generator();
                    no warnings qw(exiting);
                    redo CREATE;
                } else {
                    $Log->trace('The User already exists');
                    $self = $pkg->_element_exists_handler(
                        create_opts        => \%opts,
                        nacltask_if_exists => $if_exists,
                        exception          => $exception
                    );
                    if ( !$self ) {
                        no warnings qw(exiting);
                        redo CREATE;
                    }
                }
            }

        };
    }
    $Log->exit() if $may_exit;
    ## Return the STask object
    return $self;

}

sub remove {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->purge(@_);
    $Log->exit() if $may_exit;
}

=head2 role_add

  NACL::STask::DFM::RbacUser->role_add(
      command_interface  => $command_interface,
      'user-name-or-id'	 => $user_name_or_id,
      'role-name-or-id'    => $role_name_or_id,       
      nacltask_if_role_already_assigned => $action,      # default 'reuse'
      %other_options
  );

(Class method) Add a Role to a Rbac User.  This method provides additional
services beyond what's inherent to the product's user role addition 
commands, as controlled and described by the new C<nacltask_if_role_already_assigned> option.

=over

=item Options

=over

=item C<< command_interface => $command_interface >> 
(Required)

(Required) As C<NACL::C::DFM::DFMClient>.  A component object that
represents the host to which to send commands.

=item C<< user-name-or-id => $user_name_or_id >> 

(Required) The name or id of the user .

=item C<< role-name-or-id => $role_name_or_id >> 

(Required) The name or id of the role that is to be added to the user.

=item C<< nacltask_if_role_already_assigned => $action >>

(Optional) What to do if the role to be added is already assigned to the user.
If action is "pass" (the default), then do nothing . If $action is "die",
then fail with an exception . 

item C<< _was_added => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the role
was already assigned (value will be 0; this scenario is possible when 
nacltask_if_role_already_assigned => "pass") or whether the role was added (value will be 1).
This is necessary to determine whether the added role needs to be
cleaned up later.

    my $was_added;
	NACL::STask::DFM::RbacUser->role_add(
        nacltask_if_role_already_assigned  => 'reuse',
        _was_added => \$was_added,
        %other_opts
    );
    # Operate on $rbac_user here
    # ...
    # Now determine whether to remove the role, since we're not sure
    # whether we reused an existing role or added the role to the user.
    if ($was_added) {
        # New Role was added. Clean it up.
        $rbac_user->role_remove();
    }
	
=back

=item command_interface, apiset_must, apiset_should, method-timeout, email_address,etc.

All of the other various options to
L<< NACL::C::DFM::RbacUser->role_add | lib-NACL-C-DFM-RbacUser-pm/role_add >>

=back

=cut

sub role_add {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;
    my $pkg        = $pkg_or_obj->get_package_name();

    my %opts = $pkg_or_obj->_common_validate_with(

        params          => \@_,
        additional_spec => {
            nacltask_if_role_already_assigned =>
                { type => SCALAR, default => 'pass' },
            _was_added => { type => SCALARREF, optional => 1 },
        },
        allow_extra => 1,
    );

    my $command_interface = $opts{command_interface};
    my $user_name_or_id   = $opts{'user-name-or-id'};
    my $role_name_or_id   = $opts{'role-name-or-id'};
    my $user_added_flag;
    my $if_role_assigned = delete $opts{nacltask_if_role_already_assigned};
    my $was_added        = delete $opts{_was_added};
    $$was_added = 0;

    # Call the component role_add()
    try {

        $pkg->SUPER::role_add(%opts);
        $$was_added = 1;
    }
    catch NACL::APISet::Exceptions::ResponseException with {

        my $exception              = shift;
        my $text                   = $exception->text();
        my $expected_error_message = "User.*has already been assigned role.*";

       # Throw the exception if it's not the exception message we're expecting
       # Or if the if_role_already_assigned option is die
        if (   ( $text !~ $expected_error_message )
            || ( $if_role_assigned =~ /die/i ) )
        {

            # Rethrow the exception
            $exception->throw();

        } else {
            $Log->log("User Role Already exists. Reusing the existing one");
        }

    };    ## end of catch

    $Log->exit() if $may_exit;

}

=head2 role_remove

    $rbac_user->role_remove(%other_options);

    NACL::STask::DFM::RbacUser->role_remove(
      command_interface   => $command_interface,
      'user-name-or-id'  => $user_name_or_id, 
	  'role-name-or-id' => $role_name_or_id,
      nacltask_if_user_role_removed  => $action,     # default 'pass' 
      nacltask_verify     => 0|1,	  # default 0  
      %other_options
    );

(Class or instance method) This method removes a role assigned to a user.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< user-name-or-id => $user_name_or_id  >>

(Required only for Class method) The name or id of the user .

=item C<< role-name-or-id => $role_name_or_id  >>

(Required) The name or id of the role to be removed .

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that
the deletion did happen successfully.

=item C<< nacltask_if_user_role_removed=> die | pass >>

(Optional, defaults to "pass")

Specifies what to do if the role we're trying to remove is already removed
 A value of C<die> will result in an exception
being thrown. A value of C<pass> (default value) will mean that the exception that would
normally occur when we're trying to remove a role assigned to a user which does not exist
is suppressed.

=back

=back

=cut

sub role_remove {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;
    my $pkg        = $pkg_or_obj->get_package_name();

    my %opts = $pkg_or_obj->_common_validate_with(

        params          => \@_,
        additional_spec => {
            nacltask_if_user_role_removed =>
                { type => SCALAR, default => 'pass' },
            nacltask_verify => { type => SCALAR, default => 0 },
        },
        allow_extra => 1,
    );

    # Task specific options, not used for purge
    # nacltask_if_user_role_removed, nacltask_verify
    my $if_removed  = delete $opts{nacltask_if_user_role_removed};
    my $verify_flag = delete $opts{nacltask_verify};

    # Flag to check if purged or not
    my $is_removed = 0;

    try {

        $pkg->SUPER::role_remove(%opts);

        ## If need to verify the user deletion
        if ($verify_flag) {

            $pkg->taskverify_role_remove(%opts);

        }

        $Log->log("User Role deletion successful.");
    }
    catch NACL::APISet::Exceptions::ResponseException with {

        my $exception = shift;
        my $text      = $exception->text();

        my $expected_error_message = "User.*has not been assigned role.*";

        if (   ( $text !~ $expected_error_message )
            || ( $if_removed !~ /pass/i ) )
        {

            $Log->exit() if $may_exit;

            # Rethrow the exception
            $exception->throw();

        }
    };

    $Log->exit() if $may_exit;
}

=head2 purge

    $rbac_user->purge(%other_options);

    NACL::STask::DFM::RbacUser->purge(
      command_interface   => $command_interface,
      'user-name-or-id'  => $user_name_or_id, 
      nacltask_if_purged  => $action,     # default 'die' 
      nacltask_verify     => 0|1,	  # default 0  
      %other_options
    );

(Class or instance method) This method removes a user.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< user-name-or-id => $user_name_or_id  >>

(Required) The name or id of the user to be removed.

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that
the deletion did happen successfully.

=item C<< nacltask_if_purged => die | pass >>

(Optional, defaults to "die")
Specifies what to do if the user we're trying to delete no longer
exists. A value of C<die> (the default) will result in an exception
being thrown. A value of C<pass> will mean that the exception that would
normally occur when we're trying to remove a user who does not exist
is suppressed.

=back

=back

=cut

sub purge {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;
    my $pkg        = $pkg_or_obj->get_package_name();

    my %opts = $pkg_or_obj->_common_validate_with(

        params          => \@_,
        additional_spec => {
            nacltask_if_purged => { type => SCALAR, default => 'die' },
            nacltask_verify    => { type => SCALAR, default => 0 },
        },
        allow_extra => 1,
    );

    my $command_interface = $opts{command_interface};
    my $user_name_or_id   = $opts{'user-name-or-id'};

    # Task specific options, not used for purge
    # nacltask_if_purged, nacltask_verify
    my $if_purged   = delete $opts{nacltask_if_purged};
    my $verify_flag = delete $opts{nacltask_verify};

    try {

        $pkg->SUPER::remove(%opts);

        # If need to verify the user deletion
        if ($verify_flag) {

            $pkg->taskverify_purge(%opts);

        }

        $Log->log("User deletion successful.");

    }
    catch NACL::APISet::Exceptions::ResponseException with {

        my $exception = shift;
        my $text      = $exception->text();

        my $expected_error_message = "User.*does not exist";

        if (   ( $text !~ $expected_error_message )
            || ( $if_purged !~ /pass/i ) )
        {

            $Log->exit() if $may_exit;

            # Rethrow the exception
            $exception->throw();

        }
    };

    $Log->exit() if $may_exit;
}

sub taskverify_purge {

    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, %opts ) = @_;

    $pkg_or_obj->_generic_purge_verify(%opts);
    $Log->exit() if $may_exit;
}

sub taskverify_role_remove {

    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, %opts ) = @_;
    my $pkg               = $pkg_or_obj->get_CS_package_name ();
    my $command_interface = $opts{command_interface};
    my $user_name_or_id   = $opts{'user-name-or-id'};
    my $role_name_or_id   = $opts{'role-name-or-id'};
    my $self;

    $self = $pkg->fetch(
        command_interface => $command_interface,
        filter            => { 'user-name-or-id' => $user_name_or_id },

    );

    ## Verifying if the user role was removed or not
    my @role_list = $self->rbac_role();
    foreach my $row (@role_list) {
        if (   ( $row->{'rbac-role-id'} eq $role_name_or_id )
            || ( $row->{'rbac-role-name'} eq $role_name_or_id ) )
        {

            NACL::Exceptions::VerifyFailure->throw(
                      "Verification of User Role Deletion "
                    . " for $user_name_or_id and role $role_name_or_id FAILED!"
            );

        } else {
            $Log->log("User Role Deletion verified successfully.");
        }
    }
	$Log->log("User Role Deletion verified successfully.");
    $Log->exit() if $may_exit;
}
1;

