#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.

## @summary DebugCrsQueue Task Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here


package NACL::STask::DebugCrsQueue;

use strict;
use warnings;
use NACL::Exceptions::Timeout;

use base qw(NACL::C::DebugCrsQueue NACL::STask::STask);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR OBJECT SCALARREF HASHREF BOOLEAN ARRAYREF);
use NACL::ComponentUtils qw(_optional_scalars Dumper);

=head1 NAME

NACL::STask::DebugCrsQueue

=head1 DESCRIPTION

C<NACL::STask::DebugCrsQueue>provides a number of well-defined but
potentially complex or multi-step methods related to the CRS Queue in
ONTAP.

CRS stands for Configuration Replication Service which is an internally
used for Metro-cluster and Vserver DR scenarios of backing up configurations.

=head1 METHODS

=head2 clean_queues

  NACL::STask::DebugCrsQueue->clean_queues(
        command_interface => $command_interface,
        peer_interface    => $command_interface2,
        'stream-uuid'     => $uuid,
      );

(Class Method) Cleanup the queues for a given stream-uuid on two
different clusters. This represents the sender and receiver interfaces.

=over

=item C<< command_interface => $command_interface >>

(Required) A component object that represents the host that is used for
sending messages to the other side.

=item C<< peer_interface    => $command_interface2 >>

(Required) A component object that represents the host that is used for
receiving messages from the sender.

=item C<< stream-uuid       => $uuid >>

(Required) A string that contains a specific uuid that will be used for
cleaning the queues on both sides.

=back

=cut

sub clean_queues {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        ignore_primary_keys => 1,
        params          => \@_,
        additional_spec => {
            'peer_interface' => { type => SCALAR },
            'stream-uuid'    => { type => SCALAR },
        },
        allow_extra => 1,
    );

    my %common_opts;
    $pkg->_copy_common_component_params(
        source => \%opts,
        target => \%common_opts
    );

    my $sender_interface = $opts{command_interface};
    my $receiver_interface = $opts{peer_interface};

    my @receiver_queues = $pkg->find( 
            command_interface => $receiver_interface,
            filter => { 'stream-uuid' => $opts{'stream-uuid' } },
            %common_opts,
            allow_empty => 1);

    foreach my $entry ( @receiver_queues ) {
        $entry->delete();
    }

    my @sender_queue = $pkg->find(
            command_interface => $sender_interface,
            filter => { 'stream-uuid' => $opts{'stream-uuid' } },
            %common_opts,
            allow_empty => 1);

    foreach my $entry ( @sender_queue ) {
        $entry->delete();
    }

    $Log->exit() if $may_exit;
}

=head2 get_latest_sequence_number

  my $latest_number = NACL::STask::DebugCrsQueue->get_latest_sequence_number(
        command_interface => $command_interface,
        'stream-uuid'     => $uuid,
        'message-type'    => $message_type,
    );

(Class Method) That will search through the queue and obtain the latest sequence
number or for a specific message type. If nothing is found -1 is returned.

=over

=item C<< command_interface => $command_interface >>

(Required) A component object that represents the host that is used for
sending messages to the other side.

=item C<< 'stream-uuid'       => $uuid >>

(Required) A string that contains a specific uuid that will be used for
seaching the queue for.

=item C<<  message-type => $message_type >>

(Optional) If message_type is specified it will be used to delimit the
queue items when searching for the latest sequence number.

=back

=cut

sub get_latest_sequence_number {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        ignore_primary_keys => 1,
        params          => \@_,
        additional_spec => {
            'stream-uuid'   => { type => SCALAR },
            'message-type'  => { type => SCALAR, optional => 1 },
        },
        allow_extra => 1,
    );

    my %common_opts;
    $pkg->_copy_common_component_params(
        source => \%opts,
        target => \%common_opts
    );

    my $sequence_number = -1;
    my %filter = ();
    $pkg->_hash_copy(
        source => \%opts,
        target => \%filter,
        copy   => [qw(stream-uuid message-type)]
    );

    # Sorts by descending order of "message-seq-num-in-queue", therefore
    # the first returned object has the highest value of sequence number
    my $obj_with_highest_seq = $pkg->find(
        command_interface => $opts{command_interface},
        filter            => \%filter,
        sort_descending   => ['message-seq-num-in-queue'],
        allow_empty       => 1,
        %common_opts,
    );

    if (defined $obj_with_highest_seq) {
        $sequence_number = $obj_with_highest_seq->message_seq_num_in_queue();
    }

    $Log->exit() if $may_exit;
    return $sequence_number;
}

=head2 get_queue_count

  my $count = NACL::STask::DebugCrsQueue->get_queue_count(
        command_interface => $command_interface,
        peer_interface    => $command_interface2,
        'stream-uuid'     => $uuid,
        'message-type'    => $message_type,
    );

(Class Method) Obtains the total number of messages for both queues.
This represents the sender and receiver interfaces.

=over

=item C<< command_interface => $command_interface >>

(Required) A component object that represents the host that is used for
sending messages to the other side.

=item C<< peer_interface    => $command_interface2 >>

(Required) A component object that represents the host that is used for
receiving messages from the sender.

=item C<< 'stream-uuid'       => $uuid >>

(Required) A string that contains a specific uuid that will be used for
getting the count from both sides.

=item C<<  'message-type' => $message_type >>

(Optional) If message_type is specified it will be used to delimit the
queue items when searching for the latest sequence number.

=back

=cut

sub get_queue_count {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        ignore_primary_keys => 1,
        params          => \@_,
        additional_spec => {
            'stream-uuid'    => { type => SCALAR },
            'peer_interface' => { type => SCALAR },
            'message-type' => { type => SCALAR, optional => 1},
        },
        allow_extra => 1,
    );

    my %common_opts;
    $pkg->_copy_common_component_params(
        source => \%opts,
        target => \%common_opts
    );

    my $count = 0;
    my %hash_map_defaults;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%hash_map_defaults,
        copy   => [qw(stream-uuid message-type)]
    );
    
    my @state_objs = $pkg->find( 
        command_interface => $opts{command_interface}, 
        filter=> \%hash_map_defaults, 
        allow_empty => 1,
        %common_opts,);
    $count = scalar @state_objs;

    @state_objs = $pkg->find( 
        command_interface => $opts{peer_interface}, 
        filter=>\%hash_map_defaults, 
        allow_empty => 1,
        %common_opts,);
    $count += scalar @state_objs;

    $Log->exit() if $may_exit;
    return $count;
}

=head2 wait_for_sending

  NACL::STask::DebugCrsQueue->wait_for_sending(
        command_interface => $command_interface,
        'stream-uuid'     => $uuid,
    );

(Class Method) Method will wait for the sender to send all the messages
from it's queue to the sender. Has a default timeout of 300 seconds.

=over

=item C<< command_interface => $command_interface >>

(Required) A component object that represents the host that is used for
sending messages to the other side.

=item C<< 'stream-uuid'       => $uuid >>

(Required) A string that contains a specific uuid that will be used for
waiting for the messages to go to the other side.

=item C<< "method-timeout" => 300 >>

(Optional) As component method-timeout, controls how long the command
will wait before completing.  However, the default timeout has been
raised to 300 seconds.

=item C<< "poll-interval" => 10 >>

(Optional) This interval will be the basis for sleeping in between
waiting for the queue to empty from the sender.

=item C<< "min-wait" => 5 >>

(Optional) The amount of time in seconds to wait as a minimum
prior to returning if nothing is in the queue.

=back

=cut

sub wait_for_sending {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        ignore_primary_keys => 1,
        params          => \@_,
        additional_spec => {
            'stream-uuid'    => { type => SCALAR },
            'poll-interval'  => { type => SCALAR, default => 10 },
            'method-timeout' => { type => SCALAR, default => 300},
            'min-wait'       => { type => SCALAR, default => 5 },
        },
        allow_extra => 1,
    );

    my $sender_interface = $opts{command_interface};

    my %filter = ();
    $pkg->_hash_copy(
            source => \%opts,
            target => \%filter,
            copy   => [qw(stream-uuid)]
        );

    my $found = 0;
    my $start = [ Time::HiRes::gettimeofday( ) ];
    while(Time::HiRes::tv_interval( $start ) < $opts{'method-timeout'} ) {
        my @state_objs = NACL::CS::DebugCrsQueue->fetch( 
            command_interface => $sender_interface, 
            filter => \%filter,
            allow_empty => 1);

        if(@state_objs) {
            sleep($opts{'poll-interval'});
        } else {
            if(Time::HiRes::tv_interval( $start ) > $opts{'min-wait'}) {
                $found = 1;
                last;
            } else {
                sleep(1);
            }
        }
    }

    if(!$found ) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::Timeout->throw("wait_for_sending timed out over $opts{'method-timeout'}", timeout => $opts{'method-timeout'});
    }

    $Log->exit() if $may_exit;
}

1;
