#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.

## @summary DebugCrsStream STask Module
## @author dl-cluster-qa@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here


package NACL::STask::DebugCrsStream;

use strict;
use warnings;

use base qw(NACL::C::DebugCrsStream NACL::STask::STask);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(SCALAR);

=head1 NAME

NACL::STask::DebugCrsStream

=head1 DESCRIPTION

C<NACL::STask::DebugCrsStream>provides a number of well-defined but
potentially complex or multi-step methods related to the CRS Stream in
ONTAP.

CRS stands for Configuration Replication Service which is an internally
used for Metro-cluster and Vserver DR scenarios of backing up configurations.

Note that this library handles only a single stream - either the sender
stream or the receiver stream, but not both.
See L<NACL::MTask::DebugCrsSteams|lib-NACL-MTask-DebugCrsStreams-pm> for
a library that handles both the sender and receiver stream.

=head1 METHODS

=head2 create_stream

    my $stream_obj = NACL::STask::DebugCrsStream->create_streams(
        command_interface => $command_interface,
        'sender-uuid'     => $sender_uuid,
        %other_options
    );

(Class method)
This method is used to create a CRS stream. It provides the same
facilities as the underlying component, but defaults the value of
certain arguments so that the call can be made with fewer arguments.
The arguments accepted by the component are listed at
L<< NACL::C::DebugCrsStream->create_stream|lib-NACL-C-DebugCrsStream-pm/create_stream >>.

=over

=item Arguments defaulted/auto-determined

If the values of the following arguments are not provided, they
are assigned these default values:

=over

=item C<notification-options>

Default value is "notify_for_all_message_types"

=item C<queue-storage-capacity>

Default value is "10m"

=item C<admin-state>

Default value is "up"

=item C<domain>

Default value is "test"

=item C<path-to-persistent-storage>

Default value is "/clus/vs0"

=back

=back

=cut

sub create_stream {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'notification-options'       => { type => SCALAR,    default => 'notify_for_all_message_types'},
            'queue-storage-capacity'     => { type => SCALAR,    default => '10m'},
            'admin-state'                => { type => SCALAR,    default => 'up'},
            domain                       => { type => SCALAR,    default => 'test' },
        },
        
        # Allow through all other options accepted by the component
        allow_extra => 1,
    );

    my $self = $pkg->SUPER::create_stream(%opts); 

    $Log->exit() if $may_exit;
    return $self;
}

=head2 purge

    $streams_stask_obj->purge();

(Instance Method only) Calls delete_stream.

=cut

sub purge {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->delete_stream(@args);
    $Log->exit() if $may_exit;
}

1;
