#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NATE::BaseException that will be thrown when the
##          SnapMirror STask library (NACL::STask::Snapmirror)
##          has mismatch snapshot lists between source and
##          destination volumes 
##
## @author jreid@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=pod

=head1 Package Name

NATE::STask::Exceptions::SnapshotMismatch

=head1 Author/Maintainer

Jason Reid, dl-nacl-dev

=head1 Description

An exception for when a SnapMirror STask object (NACL::STask::Snapmirror)
detects a mismatched snapshot list between the source and destination volumes.

=head1 See Also

Package inherits from the BaseException module in NATE.

=head1 NACL::STask::Exceptions::SnapshotMismatch

=head2 Expected use

Use when the source and destination volumes snapshot lists mismatch

=head2 Base Class

NATE::BaseException

=head1 ATTRIBUTES

The values of each of the attributes can be accessed using the accessor
methods. Using the C<missing_from_source> attribute as an example, its value
can be accessed by doing:

    my @missing_from_source = $exception->missing_from_source;

=over

=item C<missing_from_source>
(Array)
The list of NACL::CS::VolumeSnapshot objects for snapshots found
on the destination volume but not on the source volume.

=item C<missing_from_destination>
(Array)
The list of NACL::CS::VolumeSnapshot objects for snapshots found
on the source volume but not on the destination volume.

=back

=cut

package NACL::STask::Exceptions::SnapshotMismatch;

use strict;
use warnings;

use Params::Validate qw(validate SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use base qw(NATE::BaseException);

use Class::MethodMaker [
    array => 'missing_from_source',
    array => 'missing_from_destination',
];

sub new {
    $Log->enter;
    my $pkg  = shift;
    my %opts = validate(
        @_,
        {   text                     => { type => SCALAR },
            missing_from_source      => { type => ARRAYREF },
            missing_from_destination => { type => ARRAYREF }
        }
    );
    my $text = delete $opts{text};
    my $self = $pkg->SUPER::new( $text, %opts );

    $Log->exit;
    return $self;
}

1;
