#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkInterfaceFailoverGroups Task Module
## @author Sivakumar T(sivakum1@netapp.com), dl-nacl-dev
## @status shared
## @pod here

package NACL::STask::NetworkInterfaceFailoverGroups;

use strict;
use warnings;

use base qw(NACL::C::NetworkInterfaceFailoverGroups NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR BOOLEAN SCALARREF ARRAYREF);
use NATE::Exceptions::Argument qw(:try);
use NACL::C::NetworkInterface;
use NACL::C::NetworkInterfaceFailover;
use NACL::APISet::Exceptions::ResponseException ();
use NACL::Exceptions::VerifyFailure             ();

__PACKAGE__->create_method_builder(stask_method_name => 'add_targets', c_method_name => 'add_targets', if_exists => 0);
=head1 NAME

NACL::STask::NetworkInterfaceFailoverGroups

=head1 DESCRIPTION

C<NACL::STask::NetworkInterfaceFailoverGroups> provide methods to create,
rename and destroy network interface failover-groups in ONTAP.

It is created on top of C<NACL::C::NetworkInterfaceFailoverGroups> component. Since it is derived class
of C<NACL::C::NetworkInterfaceFailoverGroups>, we can use all the methods of C<NACL::C::NetworkInterfaceFailoverGroups>
from the object of this task.

=head1 ATTRIBUTES

=head2 command_interface

(Required) A component object that represents the host to which to send commands.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

=head1 METHODS

=head2 create

    # FS build, class method call
    my $NetworkInterfaceFailoverGroups_obj = NACL::STask::NetworkInterfaceFailoverGroups->create(
        command_interface  => $command_interface,
        'failover-group'   => $failover_group,
        vserver            => $vserver,
        nacltask_if_exists => $action,
    );

    # Pre-FS build, class method call
    my $NetworkInterfaceFailoverGroups_obj = NACL::STask::NetworkInterfaceFailoverGroups->create(
        command_interface  => $command_interface,
        'failover-group'   => $failover_group,
        node               => $node_name,
        port               => $port,
        nacltask_if_exists => $action
    );

(Class Method) It will create a failover group with specified ports.

It also check if failover-group with specified ports is already exists or not. If failover-group
already exists it will perform the action based on C<nacltask_if_exists> parameter.
Default behavior would be to to reuse the existing failover-group.

Uses CMode CLI.

=over

=item Options

=over

=item C<< "port" => $port >>

Port to be used for failover-group

Required for pre-FS builds, not applicable for FS onwards

=item C<< "node" => $node_name >>

Node to be used for failover-group

Required for pre-FS builds, not applicable for FS onwards

=item C<< "vserver" => $vserver>>

Name of the vserver

Required for FS build onwards, not applicable for pre-FS

=item C<< "failover-group" => $failover_group >>

(Required) Name of the failover-group.

=item C<< nacltask_if_exists => $action >>

(Optional) What to do if the failover-group with specified port already exists.
If $action is "die" (the default), then fail with an exception. If action is
"reuse", then it should be using the already existing failover-group.
If action is "purge", then call the purge method of failover-group
(see the C<purge> method, below) before creating a new one.

=item C<< _was_created => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the failover-group
was found (value will be 0; this scenario is possible when
if_exists => "reuse") or whether the failover-group was created (value will
 be 1). This is necessary to determine whether the failover-group needs to be
cleaned up later.

    my $was_created;
    my $FailoverGroup_obj = NACL::STask::NetworkInterfaceFailoverGroups->create(
        if_exists    => 'reuse',
        _was_created => \$was_created,
        %other_opts
    );
    # Operate on $FailoverGroup_obj here
    # ...
    # Now determine whether to clean up the failover-group, since we're not sure
    # whether we reused an existing failover-group or created a new one
    if ($was_created) {
        # New failover-group was created. Clean it up.
        $FailoverGroup_obj->purge();
    }

=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the creation did happen
successfully.

=item command_interface, apiset_must, apiset_should, node,port etc.
All of the other various options to L<< NACL::C::NetworkInterfaceFailoverGroups->create
| lib-NACL-C-NetworkInterfaceFailoverGroups-pm/create >>.

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::NetworkInterfaceFailoverGroups::AlreadyExists>

This type of exception is thrown when an attempt is made to create failover-group with specified port that already exists.

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the created failover-group.

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            nacltask_if_exists => $pkg->_if_exists_validate_spec(),
            _was_created       => { type => SCALARREF, optional => 1 },
            nacltask_verify    => { type => BOOLEAN, default => 0 },
        },
        allow_extra => 1
    );

    # Transform %opts from the options we received into the options to
    # pass to the base class method.

    my $if_exists         = delete $opts{nacltask_if_exists};
    my $verify            = delete $opts{nacltask_verify};
    my $was_created       = delete $opts{_was_created};
    $$was_created = 0;
    my $self;

CREATE: {
        use warnings;
        try {
            $self         = $pkg->SUPER::create(%opts);
            $$was_created = 1;
        }
        catch NACL::C::Exceptions::NetworkInterfaceFailoverGroups::AlreadyExists with {
            my $exception = $_[0];
            $Log->debug("STask The failover group already exists $if_exists");
            $self = $pkg->_element_exists_handler(
                    create_opts        => \%opts,
                    nacltask_if_exists => $if_exists,
                    exception          => $exception
                );
            if ( !$self ) {
                no warnings;
                redo CREATE;
            }
        };
    }
    if ($verify) {
        $pkg->verify_state(%opts);
    }

    $Log->exit() if $may_exit;
    return $self;
} ## end sub create

=head2 rename

    $NetworkInterfaceFailoverGroups->rename();

    NACL::STask::NetworkInterfaceFailoverGroups->rename(
        'command_interface' => $ci,
        'failover-group'    => $failover_group,
        'vserver'           => $vserver,
        'new-name'          => $new_name
    );
                      or
    NACL::STask::NetworkInterfaceFailoverGroups->rename(
        'command_interface' => $ci,
        'failover-group'    => $failover_group,
        'new-name'          => $new_name
    );

(Class or Instance method) Rename the failover group with the new failover group name.
 This is performed using NetworkInterfaceFailoverGroups components rename method.
 Once the failover group is renamed, it verifies whether the failover group is renamed or not
 using the NetworkInterfaceFailoverGroups Component states using nacltask_verify option.

 Uses a CMode CLI.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required) A component object that represents the host which to send commands.
See NACL::C::Component::command_interface

=item C<< "failover-group" => $failover_group >>

(Required) Name of the failover-group.

=item C<< "new-name" => $new_name >>

(Required) Name of the new failover-group.

=item C<< "vserver" => $vserver>>

Name of the vserver

Required for FS build onwards, not applicable for pre-FS

=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the rename did happen
successfully.

=back

=over

=item Exceptions

=over

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification for  renaming failover-group has failed.

=back

=cut

sub rename {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _was_renamed    => { type => SCALARREF, optional => 1 },
            nacltask_verify => { type => BOOLEAN, default => 0 },
        },
        allow_extra => 1,
        ignore_primary_keys => 1,
    );
    # Transform %opts from the options we received into the options to
    # pass to the base class method.

    my $verify            = delete $opts{nacltask_verify};
    my $new_name          = delete $opts{'new-name'};
    my $was_renamed       = delete $opts{_was_renamed};
    $$was_renamed = 0;

    $pkg->SUPER::rename('new-name' => $new_name , %opts);
    $$was_renamed = 1;

    delete $opts{'failover-group'};
    my %common_opts;
    $pkg->_copy_common_component_params_with_ci(
        source => \%opts,
        target => \%common_opts
    );

    if ($verify) {
        try {
            $pkg->find(
                %common_opts,
                filter            => { 'failover-group' => $new_name }
            );
        }
        catch NACL::Exceptions::NoElementsFound with {
            $Log->exit() if $may_exit;
            NACL::Exceptions::VerifyFailure->throw('Renaming failover group got failed');
        };
    }

    $Log->exit() if $may_exit;
} ## end sub rename

=head2 purge

    $NetworkInterfaceFailoverGroups->purge();

    NACL::STask::NetworkInterfaceFailoverGroups->purge(
        'command_interface' => $ci,
        'failover-group'    => $failover_group,
        'vserver'           => $vserver,
    );
                          or
    NACL::STask::NetworkInterfaceFailoverGroups->purge(
        'command_interface' => $ci,
        'failover-group'    => $failover_group,
        'node'              => $node_name,
        'port'              => $port
    );

(Class or Instance method) Deletes the given failover group from Node. If the given
 failover group doesn't exists, it will perform the action based on C<if_purged> parameter.
 First Deassociate the failover group associated with the user defined lif, then delete
 all the ports in the failover group one by one using NetworkInterfaceFailoverGroups component.
 Throw an exception if someother lif is using the failover group. Once the
 failover group is deleted, it verifies whether the failover group is deleted or not
 using the NetworkInterfaceFailoverGroups Component states using nacltask_verify option.

 Uses a CMode CLI.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required) A component object that represents the host which to send commands.
See NACL::C::Component::command_interface

=item C<< "port" => $port >>

Port to be used for failover-group

Required for pre-FS builds, not applicable for FS onwards

=item C<< "node" => $node_name >>

Node to be used for failover-group

Required for pre-FS builds, not applicable for FS onwards

=item C<< "vserver" => $vserver>>

Name of the vserver

Required for FS build onwards, not applicable for pre-FS

=item C<< "failover-group" => $failover_group >>

(Required) Name of the failover-group.

=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the deletion did happen
successfully.

=item C<< nacltask_if_purged=>$nacltask_if_purged_value >>

(Optional)
 If "die" (default), exception is thrown when failover groups to be deleted does not
 exist
 If "pass", exits quietly when failover groups to be deleted does not exist

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::NetworkInterfaceFailoverGroups::DoesNotExist>

This type of exception is thrown when an attempt is made to delete failover-group that does not exists.

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the deleted failover-group.

=item C<NATE::Exceptions::Argument>

This type of exception is thrown when some other Lif using the failover group.

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my $pkg        = ref($pkg_or_obj) || $pkg_or_obj;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _was_deleted => { type => SCALARREF, optional => 1 },
            nacltask_verify => { type => BOOLEAN, default => 0 },
            nacltask_if_purged =>
                $pkg_or_obj->_if_action_completed_already_validate_spec()
        },
        allow_extra => 1
    );
    my $was_deleted       = delete $opts{_was_deleted};
    $$was_deleted = 0;
    my $if_purged         = delete $opts{nacltask_if_purged};
    my $verify            = delete $opts{nacltask_verify};
    my %common_opts;
    $pkg->_copy_common_component_params_with_ci(
        source => \%opts,
        target => \%common_opts
    );

    my @Find = NACL::C::NetworkInterface->find(
                                 %common_opts,
                                 filter => {'failover-group' => $opts{'failover-group'}},
                                 allow_empty => 1);
    if (scalar(@Find) > 1) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
                          "Some other Lif using the failover group $opts{'failover-group'}");
    }

    my $already_purged;
AGAIN: {
    try {
        $pkg->delete(%opts);
        $$was_deleted = 1;
    }
    catch NACL::C::Exceptions::NetworkInterfaceFailoverGroups::DoesNotExist with {
        my $e = shift;
        if ( $if_purged =~ /^die$/i ) {
            $Log->exit() if $may_exit;
            $e->throw();
        }
        $already_purged = 1;
    } catch NACL::C::Exceptions::NetworkInterfaceFailoverGroups::FailoverGroupIsInUseByLif with {
        my $e = $_[0];
        $Find[0]->modify('use-failover-group' => "disabled", 'failover-policy' => "disabled");
        no warnings;
        redo AGAIN;
    };
}
    return if $already_purged;
    if ($verify) {
        $pkg->_generic_purge_verify(%opts);
    }

    $Log->exit() if $may_exit;
} ## end sub purge

=head2 verify_failover

    $NetworkInterfaceFailoverGroups->verify_failover();

    NACL::STask::NetworkInterfaceFailoverGroups->verify_failover(
        'command_interface' => $ci,
        lif                 => $lif
    );
(Class method) verify_failover will verify that lif has failovered according
 to the failover policy. First It will get the details like home-port,home-node,curr-port,
 curr-node,failover-policy,failovr-group of specified lif(C<lif>) then it will check
 whether home port is down.If it is up verify_failover will die. According to the failover
 policy gets the next available port in the failover-group and verifies the port is up
 otherwise it will die. Finally compares the currnet port and current node is
 according to the failover policy.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required) A component object that represents the host which to send commands.
See NACL::C::Component::command_interface

=item C<< lif => $lif >>

(Required) The name of the logical interface, used to identify it uniquely.
The name of the logical interface is unique within a particular vserver.

=back

=over

=item Exceptions

=over

=item C<NATE::Exceptions::Argument>

This type of exception is thrown when all available ports are down.

=back

=cut

sub verify_failover {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        allow_extra => 1,
        ignore_primary_keys => 1
    );

    my $command_interface = $opts{'command_interface'};
    my $lif               = $opts{'lif'};
    my $State;

    $State = NACL::CS::NetworkInterface->fetch(
                                        command_interface => $command_interface,
                                        filter            => { lif => $lif },
                                        requested_fields  => [ qw(home-node home-port
                                        curr-node curr-port failover-group
                                        failover-policy use-failover-group ) ]
                                    );
    my $home_port          = $State->home_port();
    my $home_node          = $State->home_node();
    my $curr_port          = $State->curr_port();
    my $curr_node          = $State->curr_node();
    my $failover_group     = $State->failover_group();
    my $failover_policy    = $State->failover_policy();
    my $use_failover_group = $State->use_failover_group();

    $Log->debug("Curr Node:$curr_node Curr Port:$curr_port");

    my ($next_avail_node, $next_avail_port, $next_avail_nodes);
    my $NetworkInterfaceFailover = NACL::CS::NetworkInterfaceFailover->fetch(
                command_interface => $command_interface,
                filter => { 'lif' => $lif }
        );
    $next_avail_nodes = $NetworkInterfaceFailover->failover_targets();

    for (my $i=0;$i < scalar(@$next_avail_nodes); $i++) {
        $Log->debug("Next Avail Node". $next_avail_nodes->[$i]->{node}." Next avail port ".$next_avail_nodes->[$i]->{port});
        my $State1 = NACL::CS::NetworkPort->fetch(command_interface => $command_interface,
                                            filter => { node  => $next_avail_nodes->[$i]->{node},
                                                        port  => $next_avail_nodes->[$i]->{port},
                                            },
                                            requested_fields => [qw(link up-admin)]
                                        );
        my $link1 = $State1->link();
        my $up_admin1 = $State1->up_admin();
        if ( ($link1 =~ /up/i) && ($up_admin1 =~ /true/i) ) {
            $next_avail_node = $next_avail_nodes->[$i]->{node};
            $next_avail_port = $next_avail_nodes->[$i]->{port};
            last;
        } elsif (($link1 =~ /down/i || $up_admin1 =~ /false/i) && ($i == (scalar @$next_avail_nodes)-1)) {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw(
                               'All next available ports are Down, atleast one must be Up');
        }
    }
    if ( ($next_avail_node !~ /$curr_node/i) || ($next_avail_port !~ /$curr_port/i) ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
           'Next Available Port,Node should be same as Current Port,Node. But It is not');
    }

    $Log->exit() if $may_exit;
}

1;
