#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Ipspace Task Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::NetworkIpspace;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::Node;
use base qw(NACL::C::NetworkIpspace NACL::STask::STask);
use NACL::STask::NetworkPortBroadcastDomain;

# import Exceptions
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;

use NACL::ComponentUtils qw (_optional_scalars);
use Params::Validate qw(validate validate_with SCALAR ARRAYREF SCALARREF OBJECT);
use NATE::ParamSet qw(param_global);

=head1 NAME

NACL::STask::NetworkIpspace

=head1 DESCRIPTION

C<NACL::STask::NetworkIpspace> provides a number of well-defined but
potentially complex or multi-step methods related to Network IPspace in ONTAP.
It builds on top of C<NACL::C::NetworkIpspace> component. Since it is
derived class of C<NACL::C::NetworkIpspace>, we can use all the methods of
C<NACL::C::NetworkIpspace> from the object of this task.

=head1 ATTRIBUTES

=head2 command_interface

(Required) A component object that represents the host to which to send commands.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

=head2 ipspace
(Required) Name of the ipspace.

=head1 METHODS

=cut

=head2 create

    $ipspace_obj =  NACL::STask::NetworkIpspace->create(
                        'command_interface'     => $command_interface,
                        'ipspace'               => $ipspace
                        'nacltask_verify_ngsh'  => 1,
                        'nacltask_if_exists'    => $action, # default 'die'
                    );


(Class method)This method supports CMode CLI only.

This method is used to create ipspace with the given name.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) A component object that represents the host to which to send commands.

=item C<< 'ipspace' => $ipspace >>

(Required) Name of ipspace

=item C<< nacltask_if_exists=>$action >>

(Optional) specifies an action to be taken if given ipspace already exists.

     $action
    - if set to  "die"(default), an exception is raised.
    - if set to "reuse", no action is performed, and return reference
      to the corresponding ipspace object.
    - if set to "purge", deletes the existing ipspace before creating it.

=item C<< nacltask_verify_ngsh=>$nacltask_verify_boolean >>

(Optional, defaults to 0)
Specifies whether to verify that ipspace got created or not.
If it doesnt exists then L<NACL::Exceptions::VerifyFailure|lib-NACL-Exceptions-VerifyFailure-pm>
exception is thrown.

=item C<< _was_created => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the ipspace
was found (value will be 0; this scenario is possible when
nacltask_if_exists => "reuse") or whether the ipspace
was created (value will be 1).
This is necessary to determine whether the ipspace needs to be
cleaned up later.

    my $was_created;
    my $ipspace_obj = NACL::STask::NetworkIpspace->create(
        nacltask_if_exists    => 'reuse',
        _was_created => \$was_created,
        %other_opts
    );
    # Operate on $ipspace_obj here
    # ...
    # Now determine whether to clean up the ipspace, since we're not sure
    # whether we reused an existing ipspace or created a new one
    if ($was_created) {
        # New ipspace was created. Clean it up.
        $ipspace_obj->purge();
    }

=item C<< nacltask_mcc_replicate => 0|1 >>

If set to 1, then creates the IPSpace on the remote MCC cluster as well. This
can also be enabled by setting the C<NACL_MCC_CHECK> command-line parameter to 1.
This is necessary for MCC environments, because without it, any element that
uses a non-default IPSpace will not get replicated on the remote cluster.

=item C<< remote_interface => $remote_command_interface >>

Can be used to specify the command_interface object for the remote MCC cluster.

=item apiset_must, apiset_should, method-timeout,
All the other options supported by
create method in NetworkIpspace.pm see L<< NACL::C::NetworkIpspace->create|lib-NACL-C-NetworkIpspace-pm/create >>
are supported by NACL::STask::NetworkIpspace->create as well.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::NetworkIpspace::AlreadyExists>

This type of exception is thrown when an attempt is made to create an network ipspace that already exists.

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the created network ipspace in ngsh and sk.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            nacltask_if_exists   => { type => SCALAR,    default  => 0 },
            nacltask_verify_ngsh => { type => SCALAR,    default  => 0 },
            _was_created         => { type => SCALARREF, optional => 1 },
            nacltask_mcc_replicate => {type => SCALAR, optional => 1},
            remote_interface => {
                type     => OBJECT,
                isa      => 'NACL::C::CommandInterface::ONTAP',
                optional => 1
            },
            $pkg->_cleanup_validate_spec(),
            
        },
        allow_extra => 1
    );
    my %orig_opts = %opts;
    my $verify              = delete $opts{nacltask_verify_ngsh};
    my $if_exists           = delete $opts{nacltask_if_exists};
    my $was_created         = delete $opts{_was_created};
    my $mcc_replicate = delete $opts{nacltask_mcc_replicate}
        // param_global()->get('NACL_MCC_CHECK');
    my $remote_interface = delete $opts{remote_interface};
    
    my %opts_for_cleanup;
    $pkg->_move_common_cleanup_opts(
        source => \%opts,
        target => \%opts_for_cleanup,
    );

    $$was_created = 0;
    delete $opts{node};
    
    my $self;
    CREATE: {
        use warnings;
        my $sub_handle_if_exists = sub {
            my $exception = shift;
            $self = $pkg->_element_exists_handler(
                create_opts        => \%opts,
                nacltask_if_exists => $if_exists,
                exception          => $exception
            );
            if (!$self) {
                no warnings qw(exiting);
                redo CREATE;
            }
        };
        try {
            # Calling network ipspace create
            $self = $pkg->SUPER::create(%opts);
            $$was_created = 1;
            
            # Register this resource with the Cleanup Manager for cleanup
            # Keep this before the MCC replication code, so that even if the
            # replication code fails, it is registered for cleanup
            my ($nacltask_to_cleanup, %opts_for_register);
            $pkg->_copy_common_opts_for_cleanup(
                'source'              => {%opts, %opts_for_cleanup},
                'target'              => \%opts_for_register,
                'nacltask_to_cleanup' => \$nacltask_to_cleanup,
                'to_cleanup'          => 'purge'
            );
            $pkg->_register_for_cleanup(%opts_for_register) if ($nacltask_to_cleanup);
            
            if ($mcc_replicate) {
                my %common_params;
                $pkg->_copy_common_component_params(
                    source => \%opts,
                    target => \%common_params
                );
                my $ci = $opts{command_interface};
                if ($ci->is_in_mcc_environment(%common_params)) {
                    $remote_interface //= $ci->get_remote_interface_mcc(%common_params);
                    $pkg->create(
                        %orig_opts,
                        
                        # Create on the remote cluster
                        command_interface      => $remote_interface,
                        
                        # Set to 0, else this call will in turn try to replicate
                        # on the remote cluster, which will result in an
                        # infinite loop
                        nacltask_mcc_replicate => 0,
                    );
                }
            }
        }
        catch NACL::C::Exceptions::NetworkIpspace::AlreadyExists with {
            my $exception = shift;
            # covers CMode CLI and ZAPI
            $Log->trace('The ipspace already exists');
            $sub_handle_if_exists->($exception);
        }
        catch NACL::C::Exceptions::NetworkIpspace::InvalidIPspace with {
            my $exception = shift;
            # InvalidIPspace name if ipspace name is same as system vserver name or local cluster name.
            $Log->trace('The InvalidIPspace name');

            if ($if_exists ne 'die') {
                my %common_component_options;
                $pkg->_copy_common_component_params_with_ci(
                    source => \%opts,
                    target => \%common_component_options,
                );
                my $ipspace = $pkg->find(
                    %common_component_options,
                    filter      => { ipspace => $opts{ipspace} },
                    allow_empty => 1,
                );
                if ($ipspace) {
                    $sub_handle_if_exists->($exception);
                } else {
                    $Log->exit() if $may_exit;
                    $exception->throw();
                }
            } else {
                $Log->exit() if $may_exit;
                $exception->throw();
            }
        }
    } ## end CREATE:

    # Check in ngsh
    if ($verify) {
        $pkg->taskverify_create(%opts);
    }

    $Log->exit() if $may_exit;
    return $self;
} ## end sub create

=head2 purge

    $ipspace_obj->purge();
    or
    NACL::STask::NetworkIpspace->purge(
        'command_interface'     => $command_interface,
        'ipspace'               => $ipspace,
    );


(Class or instance method) This method deletes ipspace which is newly created.
All broadcast domain within this ipspace are purged

=over

=item Options

=over

=item C<< _was_deleted => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the broadcast domain
did not exist (value will be 0; this scenario is possible when
nacltask_if_purged => "pass") or whether the broadcast domain was deleted
(value will be 1).

=item C<< nacltask_if_purged=>die|pass >>

(Optional)
If "die", exception is thrown when broadcast domain to be deleted does not exist
If "pass", exits quietly when broadcast domain to be deleted does not exist

=item C<< nacltask_mcc_replicate => 0|1 >>

If set to 1, then purges the IPSpace on the remote MCC cluster as well. This
can also be enabled by setting the C<NACL_MCC_CHECK> command-line parameter to 1.

=item C<< remote_interface => $remote_command_interface >>

Can be used to specify the command_interface object for the remote MCC cluster.

=item Component options

Same options as those accepted by L<< NACL::C::NetworkIpspace->delete|lib-NACL-C-NetworkIpspace-pm/delete >>

=back

=over

=item Exceptions

=over

=item C<Component exceptions>

In general, Task methods propagate the exceptions thrown from
underlying Component layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@opts,
        additional_spec => {
            nacltask_if_purged  => {type => SCALAR, default => 'pass'},
            _was_deleted        => { type => SCALARREF, optional => 1 },
            nacltask_mcc_replicate => {type => SCALAR, optional => 1},
            remote_interface => {
                type     => OBJECT,
                isa      => 'NACL::C::CommandInterface::ONTAP',
                optional => 1
            },
        },        
        allow_extra     => 1
    );
    my %orig_opts = %opts;

    my %common_opts;
    $pkg_or_obj->_copy_common_component_params_with_ci(
        source => \%opts,
        target => \%common_opts
    );

    my $if_purged = delete $opts{nacltask_if_purged};
    my $was_deleted = delete $opts{_was_deleted};
    
    my $mcc_replicate = delete $opts{nacltask_mcc_replicate}
        // param_global()->get('NACL_MCC_CHECK');
    my $remote_interface = delete $opts{remote_interface};
    $$was_deleted = 0;
  DELETE: {
        try {
            $pkg_or_obj->delete(%opts);
            $$was_deleted = 1;
            
            if ($mcc_replicate) {
                my $ci = $opts{command_interface};
                if ($ci->is_in_mcc_environment(%common_opts)) {
                    $remote_interface //= $ci->get_remote_interface_mcc(%common_opts);
                    
                    # Make this a package call because the command_interface
                    # provided through %opts will not match the command_interface
                    # in the object (through opts = remote cluster; in object =
                    # local cluster)
                    $pkg_or_obj->get_package_name()->purge(
                        %orig_opts,

                        # Create on the remote cluster
                        command_interface      => $remote_interface,
                        
                        # Set to 0, else this call will in turn try to replicate
                        # on the remote cluster, which will result in an
                        # infinite loop
                        nacltask_mcc_replicate => 0,
                    );
                }
            }
            
        } catch NACL::C::Exceptions::NetworkIpspace::DoesNotExist with {
            my $exception = shift;
            if ( $if_purged !~ /pass/i ) {
                $Log->exit() if $may_exit;
                $exception->throw();
            }
        } catch NACL::C::Exceptions::NetworkIpspace::VserverExists with {
            $pkg_or_obj->_delete_all_vservers(%common_opts, ipspace => $opts{ipspace});
            goto DELETE;
        } catch NACL::C::Exceptions::NetworkIpspace::BroadcastDomainExists with {
            $pkg_or_obj->_delete_all_broadcast_domains(%common_opts, ipspace => $opts{ipspace});
            goto DELETE;
        };
    }
    $Log->exit() if $may_exit;
}

sub _delete_all_vservers {
    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;
    my %args;
    my $ipspace = delete $opts{ipspace};
    my @vservers = NACL::STask::Vserver->find(
        %opts,
        filter => {
            ipspace => $ipspace,
            type    => '!system',
        }
    );

    foreach my $vserver (@vservers) {
        $vserver->purge(%opts);
    }

    $Log->exit() if $may_exit;
}

sub _delete_all_broadcast_domains {
    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;
    my $ipspace = delete $opts{ipspace};

    my @bds = NACL::STask::NetworkPortBroadcastDomain->find(
        %opts,
        filter => {
            ipspace => $ipspace,
        }
    );

    foreach my $bd (@bds) {
        $bd->purge(%opts);
    }

    $Log->exit() if $may_exit;
}

1;
