#
#   Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkPort Task Module
## @author kathar@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::NetworkPort;

use strict;
use warnings;

use base qw(NACL::C::NetworkPort NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR ARRAYREF UNDEF SCALARREF);
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NATE::BaseException qw(:try);
use NACL::C::NetworkPort;
use NACL::C::Interface;
use NACL::ComponentUtils qw(get_package_name);
use NACL::GeneralUtils qw(nacl_method_retry);
use NACL::Exceptions::VerifyFailure ();

=head1 NAME

NACL::STask::NetworkPort

=head1 DESCRIPTION

C<NACL::STask::NetworkPort> provide methods to configure network ports in ONTAP.

It is created on top of C<NACL::C::NetworkPort> component. Since it is derived class
of C<NACL::C::NetworkPort>, we can use all the methods of C<NACL::C::NetworkPort>
from the object of this task.

=head1 CLEANUP METHODS

Cleanup can be registered for the following methods

Cleanup methods are,
    NetworkPortVlan Method  Cleanup Method
    ----------------------------------------
    modify                    modify

=head1 ATTRIBUTES

=head2 command_interface

(Required)
A component object that represents the host to which to send commands.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

=head2 node

(Required)
Name of the node.

=head2 port

(Required)
Name of the Port. Ex. e0c

=head1 METHODS

=head2 modify

    NACL::STask::NetworkPort->modify(
        command_interface        => $ci,
        node => $node,
        port => $port,
	nacltask_to_cleanup      => 1,          # default 0,
        nacltask_cleanup_manager => $CleanupObj,
        %other_opts,
    );

                        or

    $network_port_obj->modify(%other_opts);

(Class or instance method)
This method modifies the properties of the network port specified in C<port>.
If the specified port or attributes are invalid, it will throw the exception thrown
from the APISet layer. This method provides additional services beyond what's
inherent to the network port modification commands, as controlled and
described by the C<nacltask_verify> options.

Uses a CMode CLI, 7Mode CLI or CMode ZAPI APISet.

=over

=item Options

=over

=item C<< nacltask_verify => 0|1 >>

(Optional, defaults to 0)
Specifies whether to verify that all the fields got modified to the values
specified in the call. If one or more of the fields don't match, then a
L<NACL::Exceptions::VerifyFailure|lib-NACL-Exceptions-VerifyFailure-pm>
exception is thrown.

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup)
Flag indicating if this operation needs to be registered for clean up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=item Other options

All of the other options accepted by
L<NACL::C::NetworkPort::modify|lib-NACL-C-NetworkPort-pm/modify>.

=back

=over

=item Exceptions

=over

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification for network port modify has failed.

=cut

sub modify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec => {
            nacltask_verify => { type => SCALAR, default => 0 },
	    $pkg_or_obj->_cleanup_validate_spec(),
		},
        allow_extra => 1
    );
    my $verify = delete $opts{nacltask_verify};
    my (%opts_for_cleanup, $opts_for_register, $register_for_cleanup);
    $pkg_or_obj->_move_common_cleanup_opts(
       source               => \%opts,
       target               => \%opts_for_cleanup,
       nacltask_to_cleanup  => \$register_for_cleanup,
    );

    if ($register_for_cleanup) {
       $opts_for_register
          = $pkg_or_obj->_common_modify_for_cleanup(%opts_for_cleanup,
            %opts);
    }

    my $pkg    = $pkg_or_obj->get_package_name();
    $pkg->SUPER::modify(%opts);
    my %retry_opts;
    $pkg->_move_retry_params(
        source => \%opts,
        target => \%retry_opts
    );
    # Verify the modifications is done or not
    nacl_method_retry(
        code => sub {if ($verify) {
                            $pkg->taskverify_modify(%opts);
                         }
                },
        exceptions => ['NACL::Exceptions::VerifyFailure'],
        %retry_opts
    );

    #Register for cleanup if $nacltask_to_cleanup is set
    $pkg_or_obj->_register_for_cleanup(%{$opts_for_register})
          if ($register_for_cleanup);

    $Log->exit() if $may_exit;
} ## end sub modify

1;
