#   Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkPortIfgrp Task Module
## @author dl-nacl-dev@netapp.com, kathar.hidayath@netapp.com
## @status shared
## @pod here

package NACL::STask::NetworkPortIfgrp;

use strict;
use warnings;

use base qw(NACL::C::NetworkPortIfgrp NACL::STask::STask);

use constant PORT_ADD_TIMEOUT      => 60;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR ARRAYREF UNDEF SCALARREF BOOLEAN);
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::ComponentUtils qw(_optional_scalars _dump_one );
use NATE::BaseException qw(:try);
use NACL::C::NetworkPort;
use NACL::STask::NetworkPort;
use NACL::C::Interface;
use NACL::C::NetworkPortVlan;
use NACL::CS::NetworkPortIfgrp;
use NACL::C::NetworkPortBroadcastDomain;
use NATE::Time qw(timeout2time);
use NACL::C::Exceptions::NetworkPortIfgrp::AlreadyExists qw(:try);
use NACL::C::Exceptions::NetworkPortIfgrp::DoesNotExist ();
use NACL::C::Exceptions::NetworkPort::DoesNotExist ();

=head1 NAME

NACL::STask::NetworkPortIfgrp

=head1 DESCRIPTION

C<NACL::STask::NetworkPortIfgrp> provide methods to configure and destroy network port
ifgrp in ONTAP. It allows to create ifgrp and add port to ifgrp in single method call.

It is created on top of C<NACL::C::NetworkPortIfgrp> component. Since it is derived class
of C<NACL::C::NetworkPortIfgrp>, we can use all the methods of C<NACL::C::NetworkPortIfgrp>
from the object of this task.

=head1 CLEANUP METHODS

Cleanup can be registered for the following methods

Cleanup methods are,
    NetworkPortIfgrp Method     Cleanup Method
    ------------------------------------------
    create                       purge

=head1 ATTRIBUTES

=head2 command_interface

(Required) A component object that represents the host to which to send commands.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

=head2 node

(Required) Name of the node.

=head2 ifgrp

(Required) Name of the ifgrp. Ex. a0b

=head1 METHODS

=head2 create

my $NetworkPortIfgrp_obj = NACL::STask::NetworkPortIfgrp->create(
    command_interface => $command_interface,
    node              => $node_name,
    ifgrp             => $ifgrp_name,
    distr-func        => $dist_func,         # default port
    mode              => $ifgrp_mode,        # default singlemode
    ports             => ['e0a', 'e0b', ..],
    num_ports         => $number,
    if_exists         => $action             # default die
    nacltask_to_cleanup      => 1,          # default 0,
    nacltask_cleanup_manager => $CleanupObj,
);

(Class Method) It will create ifgrp. Along with creating ifgrp, it will also
add ports to ifgrp.

It also check if ifgrp is already exists or not. If ifgrp
already exists it will perform the action based on C<if_exists> parameter.
Default behavior would be to throw exception and die.

While adding the ports to the ifgrp, if ONTAP throws an error saying that port
already has lif bound, and if that lif is home to the port, task will delete
the lif and add port to ifgrp. If the lif is only current to the port(not home),
task will try to move that lif from the port and add port to ifgrp. In any case,
if it fails to remove lif from port, it will raise an exception.

if C<ports> parameter is undef and C<num_ports> parameter is 0, then this task
will only create ifgrp and return without adding port to it.

C<ports> parameter will take priority over C<num_ports> parameter.

=over

=item Options

=over

=item C<< ports => [e0a, e0b, ..] >>

(Optional) It is a pointer to an array, representing the ports, which need
to be added in the ifgrp. Ex: [e0a, e0b].

=item C<< num_ports => $number >>

(Optional) If C<ports> parameter is undef, task will automatically try to
find out the ports to add in this ifgrp. Number of available ports to find
out depends on this parameter.

The availability of the ports to add in this ifgrp depends on following criteria:
1. port is not a home port of any lif
2. port is not a current port of any lif
3. port is not a part of any vlan interface
4. port is not a part of any ifgrp interface

=item C<< if_exists => $action >>

(Optional) What to do if the ifgrp to be created already exists.
If $action is "die" (the default), then fail with an exception (in the
same way that the NetworkPortIfgrp component would have: by trying the
NetworkPortIfgrp create and letting the product complain about the ifgrp
already existing).  If action is "purge", then destroy the ifgrp
(see the C<purge> method, below) before creating a new one.

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup)
Flag indicating if this operation needs to be registered for clean up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=item C<< _was_created => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the ifgrp
was found (value will be 0; this scenario is possible when
if_exists => "reuse") or whether the ifgrp was created (value will be 1).
This is necessary to determine whether the ifgrp needs to be
cleaned up later.

    my $was_created;
    my $ifgrp_obj = NACL::STask::NetworkPortIfgrp->create(
        if_exists    => 'reuse',
        _was_created => \$was_created,
        %other_opts
    );
    # Operate on $ifgrp_obj here
    # ...
    # Now determine whether to clean up the ifgrp, since we're not sure
    # whether we reused an existing ifgrp or created a new one
    if ($was_created) {
        # New ifgrp was created. Clean it up.
        $ifgrp_obj->purge();
    }

=back

=item command_interface, apiset_must, apiset_should, mode, ifgrp etc.
All of the other various options to L<< NACL::C::NetworkPortIfgrp->create
| lib-NACL-C-NetworkPortIfgrp-pm/create >>.

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::NetworkPortIfgrp::AlreadyExists>

This type of exception is thrown when an attempt is made to create a network port ifgrp which already exists.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'distr-func' => { type => SCALAR, default => "port" },
            mode         => { type => SCALAR, default => "singlemode" },
            ports => { type => ARRAYREF | UNDEF, optional => 1 },
            if_exists => {
                type      => SCALAR,
                callbacks => $pkg->_if_exists_validate_callback(),
                optional  => 1,
            },
            nacltask_if_exists => {
                type      => SCALAR,
                callbacks => $pkg->_if_exists_validate_callback(),
                optional  => 1,
            },
            num_ports    => { type => SCALAR, optional => 1 },
            _was_created => { type => SCALARREF, optional => 1 },
            $pkg->_cleanup_validate_spec(),
        },
        allow_extra => 1
    );
    
    my ( $self, %common_opts, %nacltask_opts );
    my (%opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup );
    $pkg->_move_common_cleanup_opts(
        source  => \%opts,
        target  => \%opts_for_cleanup,
    );

    $pkg->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_opts
    );

    my $if_exists;
    if (exists $opts{if_exists}){
        $if_exists         = delete $opts{'if_exists'};
    } elsif (exists $nacltask_opts{'nacltask_if_exists'}){
        $if_exists         = $nacltask_opts{'nacltask_if_exists'};
    } else {
        $if_exists         = "die";
    }

    my $ports             = delete $opts{ports};
    my $num_ports         = delete $opts{num_ports};
    my $was_created       = delete $opts{_was_created};
    $$was_created = 0;

CREATE: {
        use warnings;
        try {
            $self         = $pkg->SUPER::create(%opts);
            $$was_created = 1;
        }
        catch NACL::C::Exceptions::NetworkPortIfgrp::AlreadyExists with {
            my $exception = shift;
            $self = $pkg->_element_exists_handler(
                create_opts        => \%opts,
                nacltask_if_exists => $if_exists,
                exception          => $exception,
            );
            if ( !$self ) {
                no warnings qw(exiting);
                redo CREATE;
            }
        };
    }
    
    if ($$was_created) {
        # Remove the paramters from options, which is not required by add_ports
        delete $opts{'distr-func'};
        delete $opts{mode};

        if ( defined(@$ports) ) {
            $pkg->add_ports( %opts, ports => $ports );
        } elsif ( $num_ports > 0 ) {
            $pkg->add_ports( %opts, num_ports => $num_ports );
        } else {
            $Log->debug("Task is exiting without adding ports to ifgrp");
        }
    }
    
    # Register this resource with the cleanup manager for cleanup.
    $pkg->_copy_common_opts_for_cleanup(
        'source'             => {%opts, %opts_for_cleanup},
        'target'             => \%opts_for_register,
        'nacltask_to_cleanup'=> \$nacltask_to_cleanup,
        'to_cleanup'         => 'purge',
    );
    $pkg->_register_for_cleanup(%opts_for_register) if ($nacltask_to_cleanup && $$was_created);
    $Log->exit() if $may_exit;
    return $self;
} ## end sub create

=head2 add_ports

    # Call method using package
    my $NetworkPortIfgrp_obj = NACL::STask::NetworkPortIfgrp->add_ports(
        command_interface => $command_interface,
        node              => $node_name,
        ifgrp             => $ifgrp_name,
        ports             => ['e0a', 'e0b', ..]
    );

    or

    my $NetworkPortIfgrp_obj = NACL::STask::NetworkPortIfgrp->add_ports(
        command_interface => $command_interface,
        node              => $node_name,
        ifgrp             => $ifgrp_name,
        num_ports         => $number
    );

    # Call method using object
    # Add explit ports into the ifgrp
    $NetworkPortIfgrp_obj->add_ports( ports => ['e0b','e0c'] );

    # Add any 3 ports to ifgrp
    $NetworkPortIfgrp_obj->add_ports( num_ports => 3 );

    # Add any 2 ports to ifgrp, by default
    $NetworkPortIfgrp_obj->add_ports();

(Class or Instance Method) It will add physical data ports to ifgrp.

There are two ways to add port to ifgrp using this method:

1. User can provide explicit ports to add in the ifgrp, ports can be
   provided using C<ports> parameter. In this case, Task will check
   if there is a Home lif on this port or not. If there is a home
   lif on this port, it will delete it. If there is a lif which is
   only current to this port, task will disable the port and enable
   it after 5 seconds, so that that lif failover to someother port
   and we will have free port to add in our ifgrp.

2. User can provide number of ports to add in this ifgrp, task will
   try to find out available ports and add it to ifgrp. available
   ports are the one which does not have lif configured(home or current),
   it must not have vlan and ifgrp configured on it. if task can not
   find the available ports requested by user, it will add what ever
   is available and issue warning message saying it is not able to
   add the total number of ports requested by user, but it has add
   less than that number. default vaule for this parameter is 2, so
   if nothing is specified in the method call, it will try to add
   2 ports in ifgrp.

user can not provide both parameter(C<ports> and C<num_ports>). It has
to be only one parameter or nothing.

If user has not provided any prameter out of this two(C<ports> and C<num_ports>)
then C<num_ports> will take effect and it will try to find 2 available ports
and add them in ifgrp.

=over

=item Options

=over

=item C<< ports => [e0a, e0b, ..] >>

(Optional) It is a pointer to an array, representing the ports, which need
to be added in the ifgrp. Ex: [e0a, e0b].

=item C<< num_ports => $number >>

(Optional) If C<ports> parameter is undef, task will automatically try to
find out the ports to add in this ifgrp. Number of available ports to find
out depends on this parameter.

The availability of the ports to add in this ifgrp depends on following criteria:
1. port is not a home port of any lif
2. port is not a current port of any lif
3. port is not part of any vlan interface
4. port is not part of any ifgrp interface

Default is 2, so it will try to find out maximum two available port on the
filer and add them in ifgrp.

=back

=item command_interface, apiset_must, apiset_should, mode, ifgrp etc.
All of the other various options to L<< NACL::C::NetworkPortIfgrp->create
| lib-NACL-C-NetworkPortIfgrp-pm/create >>.

=back

=over

=item Exceptions

=over

=item C<NATE::Exceptions::Argument>

This type of exception is thrown when neither ports nor number of available ports were not provided.

=item C<NATE::BaseException>

This type of exception is thrown when there are no data port available to add to ifgrp.

=back

=back

=cut

sub add_ports {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            ports     => { type => ARRAYREF | UNDEF, optional => 1 },
            num_ports => { type => SCALAR | UNDEF,   optional => 1 },
        },
        allow_extra => 1
    );

    my $ports     = $opts{ports};
    my $num_ports = $opts{num_ports};
    if ( ( defined( $opts{ports} ) ) && ( defined( $opts{num_ports} ) ) ) {

        NATE::Exceptions::Argument->throw(
            "Specify either ports or num_ports, not both");
    }

    # If both ports & num_ports are not specified, assign 2
    # as default value to num_ports
    if ( !defined( $opts{ports} ) ) {
        if ( defined( $opts{num_ports} ) && $opts{num_ports} <= 0 ) {
            NATE::Exceptions::Argument->throw(
                      "Negative value or 0 is not allowed for"
                    . " num_port param" );
        }
        if ( !defined( $opts{num_ports} ) ) {
            $opts{num_ports} = 2;
        }
    } ## end if ( !defined( $opts{ports...

    my $pkg = ref($pkg_or_obj) || $pkg_or_obj;

    if ( defined($ports) ) {
        delete $opts{num_ports};
        $pkg->_predefined_add(%opts);
    } else {
        delete $opts{ports};
        $pkg->_auto_add(%opts);
    }

    $Log->exit() if $may_exit;
} ## end sub add_ports

=head2 purge

    $NetworkPortIfgrp->purge();

    NACL::STask::NetworkPortIfgrp->purge(
        command_interface => $ci,
        node              => $node_hostname,
        ifgrp             => $ifgrp_name
    );

(Class or Instance method) Deletes interface port group from Node. If
there is any interface associated with this ifgrp, it will first purge
the interface before deleting the ifgrp.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required) A component object that represents the host to which to send commands.
See NACL::C::Component::command_interface

=item C<< node => $node_hostname >>

(Required) The hostname of the Node.

=item C<< ifgrp => ifgrp_name >>

(Required) The ifgrp name.

=back

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my $additional_spec = {
        nacltask_verify    => { type => BOOLEAN, default => 0 },
        nacltask_if_purged => $pkg_or_obj->_if_action_completed_already_validate_spec(),
        _was_deleted       => { type => SCALARREF, optional => 1 },
    };

    my %opts = $pkg_or_obj->_common_validate_with(
        params      => \@_,
        additional_spec => $additional_spec,
        allow_extra => 1,

    );
    my %nacltask_opts;
    $pkg_or_obj->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_opts,
    );

    my $if_purged   = $nacltask_opts{nacltask_if_purged};

    my $command_interface = $opts{command_interface};
    my $break_loop        = 0;
    my %common_opts;
    $pkg_or_obj->_copy_common_component_params_with_ci(
        source => \%opts,
        target => \%common_opts
    );

    my $was_deleted = delete $opts{_was_deleted};
    $$was_deleted = 0;

    my $ifgrp_exists = 1;

    try {
        # Bring down the ifgrp
        NACL::STask::NetworkPort->modify(
            command_interface => $command_interface,
            node              => $opts{node},
            port              => $opts{ifgrp},
            'up-admin'        => "false",
        );
    } catch NACL::C::Exceptions::NetworkPort::DoesNotExist with {
        my $exception = shift;
        $ifgrp_exists = 0;
        if ( $if_purged !~ /pass/i ) {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
    };

    if ($ifgrp_exists) {
        while (1) {
            try {
                # Now delete NetworkPortIfgrp..
                $pkg_or_obj->SUPER::delete(%common_opts, %opts );#delete(%opts);
                $$was_deleted = 1;
                $break_loop = 1;
            } ## end try
            catch NACL::C::Exceptions::NetworkPortIfgrp::HomePortOrFailoverTarget with {
                # If some error happens during delete ifgrp..
                my $e     = shift;
                # Find out logical interfaces created on this ifgrp.
                my @interfaces = NACL::C::Interface->find(
                    %common_opts,
                    filter => {
                        'home-node' => $opts{node},
                        'home-port' => $opts{port}
                    },
                    allow_empty => 1
                );
                foreach my $lif (@interfaces) {
                    try {
                        $lif->delete();
                    }
                    catch NACL::C::Exceptions::NetworkInterface::LifEnabled with {
                        $lif->modify('status-admin' => 'down');
                        $lif->delete();
                    };
                }
            };

            last if ($break_loop);
        } ## end while (1)
    }
    $Log->exit() if $may_exit;
} ## end sub purge

=begin COMMENTED_OUT

    $pkg->_predefined_add( ports => ['e0a', 'e0b' ..]);

(For developers of NetworkPortIfgrp task)
   Ports can be provided using C<ports> parameter. This method will check if
   there is a Home lif on this port or not. If there is a home lif on this port,
   it will delete it. if there is a lif which is only current to this port,
   task will disable the port and enable it after 5 seconds, so that that lif
   failover to someother port and we will have free port to add in our ifgrp.

=end COMMENTED_OUT

=cut

sub _predefined_add {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            ports => { type => ARRAYREF },
        },
        allow_extra     => 1
    );

    my $ports             = delete $opts{ports};
    my $node              = $opts{node};
    my $ifgrp             = $opts{ifgrp};
    my $command_interface = $opts{command_interface};

    my $common_opts = $pkg->_hash_copy(
        source => \%opts,
        copy   => [ keys %{ $pkg->_common_validate_spec() } ]
    );
    my $end_time;
    foreach my $port ( @{$ports} ) {
    $end_time = timeout2time(PORT_ADD_TIMEOUT);
    my $redo_flag = 1;
    while ( time() < $end_time && $redo_flag ) {
	try {
           $Log->trace("Add port($port) to ifgrp($ifgrp)");
	   $pkg->SUPER::add_port( %opts, port => $port );
	   $redo_flag = 0;
	}
	catch NACL::C::Exceptions::NetworkPortIfgrp::ConfiguredUp with {
	   my $eo = shift;
	   NACL::STask::NetworkPort->modify(
		%{$common_opts},
		port       => $port,
		'up-admin' => "false",
	   );
	   $redo_flag = 1;
	} catch NACL::C::Exceptions::NetworkPortIfgrp::PortHasLifBound with {
	   my @interfaces = NACL::C::Interface->find(
                %{$common_opts},
		filter => {
		    'home-node' => $node,
		    'home-port' => $port
		},
	   );
	   foreach my $ifc (@interfaces) {
               try {
                   $ifc->delete();
               }
               catch NACL::C::Exceptions::NetworkInterface::LifEnabled with {
                   $ifc->modify('status-admin' => 'down');
                   $ifc->delete();
               };
	   }
	   $redo_flag = 1;
	} catch NACL::C::Exceptions::NetworkPortIfgrp::PortAlreadyAssigned with {
	   my @interfaces = NACL::C::Interface->find(
		%{$common_opts},
		filter => {
		    'home-node' => $node,
		    'home-port' => $port
		},
	   );
	   foreach my $ifc (@interfaces) {
               try {
		   $ifc->delete();
               }
               catch NACL::C::Exceptions::NetworkInterface::LifEnabled with {
                   $ifc->modify('status-admin' => 'down');
                   $ifc->delete();
               };
	   }
           my $nodePort = $node . ":" . $port ;
	   my $broadcast_domain_obj = NACL::C::NetworkPortBroadcastDomain->find (
	 	command_interface => $command_interface,
		filter            => {'ports' =>  [$nodePort]},
	   );
	   if ($broadcast_domain_obj){
		$broadcast_domain_obj->remove_ports (
	           'ports'           => [$nodePort],
	        );
	   }
	   $redo_flag = 1;
	}catch NACL::C::Exceptions::NetworkPortIfgrp::HomePortOrFailoverTarget with {
           $Log->debug(
		"Bring down port($port), so that lif will failover to other port"
	   );
	   NACL::STask::NetworkPort->modify(
		%{$common_opts},
		node       => $node,
		port       => $port,
		'up-admin' => 'false'
	   );
	   # sleep for 5 seconds to let lif migrate to other port
	   Tharn::snooze 5;

	   $Log->debug(
		"Bring UP port($port), after lif failover to other port"
	   );
	   NACL::STask::NetworkPort->modify(
	 	%{$common_opts},
		node       => $node,
		port       => $port,
	        'up-admin' => 'true'
	   );

	   # wait 5 secs to port to come-up
	   Tharn::snooze 5;
           if (NACL::C::Interface->find(
		%{$common_opts},
	        filter => {
		     'curr-node' => $node,
		     'curr-port' => $port
		},
		allow_empty => 1
		)
	   ) {
	     NATE::BaseException->throw(
                "Fail to add port($port) to ifgrp($ifgrp), "
		. "port is hosting lif" );
	   } ## end if ( NACL::C::Interface...
	   $redo_flag = 1;
	}; ## end catch NACL::C::Exceptions::NetworkPortIfgrp::HomePortOrFailoverTarget
      }  ## end while ( time() < $end_time && $redo_flag) {
    } ## end foreach my $port ( @{$ports...

    $Log->exit() if $may_exit;
} ## end sub _predefined_add

=begin COMMENTED_OUT

   $pkg->_auto_add(num_ports => $num_ports);

(For developers of NetworkPortIfgrp task)
   This method will try to find out available ports and add it to ifgrp.
   Available ports are the one which does not lif configured(home or current),
   it must not vlan and ifgrp configured on it. if task can not find the
   available ports requested by user, it will add what ever is available
   and issue warning message saying it is not able to add the total number
   of ports requested by user.

=end COMMENTED_OUT

=cut

sub _auto_add {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => { num_ports => { type => SCALAR } },
        allow_extra     => 1
    );

    my $num_ports     = delete $opts{num_ports};
    my $c_common_opts = $pkg->_hash_copy(
        source => \%opts,
        copy   => [ keys %{ $pkg->_common_validate_spec() } ]
    );

    my $node  = $opts{node};
    my $ifgrp = $opts{ifgrp};
    my %filter_opts;
        if ($opts{command_interface}->version_manager->has_uichange(
                  uichange        => 'ipspaces-ms4',
                  throw_exception => 0
                    )
                    )
                {
                    $filter_opts{ipspace} = 'Default';
                } else {
                    $filter_opts{role} = 'data';
                }

    # Find ports which are not part of vlan and ifgrp
    $Log->trace("find all available data ports to add in the ifgrp");
    my @p_ports = NACL::C::NetworkPort->find(
        %{$c_common_opts},
        filter => {
            node => $node,
            type => 'physical',
            link => 'up',
            %filter_opts
        }
    );

    my $num_ports_added = 0;
    my ( $port_state, $d_port );
    foreach my $new_port (@p_ports) {
        if ( $num_ports_added == $num_ports ) {
            last;
        }
        $d_port = $new_port->port();
        $Log->trace(
            "Make sure that port($d_port) does not have vlan configured");
        if (NACL::C::NetworkPortVlan->find(
                %{$c_common_opts},
                filter => {
                    node => $node,
                    port => $d_port
                },
                allow_empty => 1
            )
            ) {
            next;
        } ## end if ( NACL::C::NetworkPortVlan...

        $Log->trace("Make sure that port($d_port) is not part of any ifgrp");

        # As ZAPI component requires ifgrp-name as mandatory parameter,
        # running this command in CLI even for ZAPI support
        if (NACL::C::NetworkPortIfgrp->find(
                %{$c_common_opts},
                filter => {
                    node  => $node,
                    ports => [$d_port]
                },
                apiset_should => { 'interface' => 'CLI', },
                allow_empty   => 1
            )
            ) {
            next;
        } ## end if ( NACL::C::NetworkPortIfgrp...

        $Log->trace(
            "Make sure that port($d_port) does not have any lif configured");
        if (NACL::C::Interface->find(
                %{$c_common_opts},
                filter => {
                    'home-node' => $node,
                    'home-port' => $d_port
                },
                allow_empty => 1
            )
            ) {
            next;
        } ## end if ( NACL::C::Interface...

        $Log->trace("Make sure that port($d_port) does not serving any lif");
        if (NACL::C::Interface->find(
                %{$c_common_opts},
                filter => {
                    'curr-node' => $node,
                    'curr-port' => $d_port
                },
                allow_empty => 1
            )
            ) {
            next;
        } ## end if ( NACL::C::Interface...

        $Log->trace("add port($d_port) to ifgrp($ifgrp)");
        $pkg->SUPER::add_port( %opts, port => $d_port );

        $num_ports_added++;
    } ## end foreach my $new_port (@p_ports)

    if ( $num_ports_added == 0 ) {
        NATE::BaseException->throw(
            "There is no data port available to add in ifgrp");
    } elsif ( $num_ports_added < $num_ports ) {
        $Log->warn( "It is only able to add $num_ports_added port(s) in "
                . "ifgrp($ifgrp), But request was to add $num_ports ports" );
    } else {
        $Log->trace( "Successful to add requested number($num_ports) of "
                . "ports to ifgrp($ifgrp)" );
    }

    $Log->exit() if $may_exit;
} ## end sub _auto_add

=head2 remove_ports

    # Call method using package
    my $NetworkPortIfgrp_obj = NACL::STask::NetworkPortIfgrp->remove_ports(
        command_interface => $command_interface,
        node              => $node_name,
        ifgrp             => $ifgrp_name,
        port_list         => \@interfaces,
    );

            or

    $NetworkPortIfgrp_obj->remove_ports( port_list => [e0b] );

    Uses a 7Mode/CMode CLI and CMode ZAPI APISet.

    (Class or Instance Method)
    Removes network port(s) from Network port interface group

    To remove port or ports from particular ifgrp

    1.User can provide a ifgrp name using parameter(C<ifgrp>) from which
    particular ports need to be removed. If that particular ifgrp is not
    present then it will exit by throwing an exception.

    2.The port or ports which has to be removed can be provided using (C<port_list>)
    parameter. The task will remove the ports. If the port doesn't exists,
    it ignores the exception if (C<nacltask_if_ports_removed>) is specified
    as pass, else it throws the exception (default is die).
    If user didnt mention any port to remove in (C<port_list>), then
    it will delete all the ports associated with that ifgrp.

    3.If the configuration of ifgrp is up then it will make it down and then
    particular ports will be removed (Ports cannot be removed if the ifgrp
    configuration is up) and it will make the ifgrp up.

    4.It will verify the ifgrp status to ensure that the ports are removed
    successfully using "network port ifgrp show" and "network port show"
    if (C<nacltask_verify>) option is specified as 1.

=over

=item Options

=over

=item C<< node => $node >>

Required for class method, Not Applicable for instance method)
Name of the node.

=item C<< ifgrp => $ifgrp >>

Required for class method, Not Applicable for instance method)
Name of the ifgrp. Ex. a0b

=item C<< port_list => [e0a] >>

(optional)It is a array reference, representing the port or ports,
which needs to be removed. Ex: e0a.

=item C<< nacltask_if_ports_removed => $action >>

(Optional) What to do if the port to be removed is not part of ifgrp.
If $action is "die" (the default), then fail with an exception (in the
same way that the NetworkPortIfgrp component would have: by trying the
NetworkPortIfgrp remove_ports and letting the product complain about the
invalid port).  If action is "pass", then ignore the exception.

=item C<< nacltask_verify => 0|1 >>

(Optional, defaults to 0)
Specifies whether to verify that all the fields got modified to the values
specified in the call. If one or more of the fields don't match, then a
L<NACL::Exceptions::VerifyFailure|lib-NACL-Exceptions-VerifyFailure-pm>
exception is thrown.

command_interface, apiset_must, apiset_should, etc. All
of the other various options to NACL::C::NetworkPortIfgrp->remove_port.

'port' parameter supported by remove_port in NACL::C::NetworkPortIfgrp is not
supported by remove_ports in NACL::STask::NetworkPortIfgrp stask method.
refer (c<'port_list'>) in stask method to give port details.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::NetworkPortIfgrp::DoesNotExist>

This type of exception is thrown when an attempt is made to delete an ifgrp that does not exists.

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the deleted ifgrp.

=back

=back

=cut

sub remove_ports {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my $pkg        = ref($pkg_or_obj) || $pkg_or_obj;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            port_list => { type => ARRAYREF, optional => 1 },
            nacltask_if_ports_removed =>
                $pkg_or_obj->_if_action_completed_already_validate_spec(
                name => 'nacltask_if_ports_removed',
                ),
            nacltask_verify => { type => BOOLEAN, default => 0 },
        },
        allow_extra => 1
    );
    my $command_interface = $opts{command_interface};
    my $node              = $opts{node};
    my $ifgrp             = $opts{ifgrp};
    my %common_opts;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%common_opts,
        copy   => [ keys %{ $pkg->_common_validate_spec() } ]
    );
    my %nacltask_options;
    $pkg_or_obj->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_options
    );
    my $ports      = undef;
    my @temp_ports = ();
    $ports = delete $opts{port_list};
    push @temp_ports, @{$ports} if ($ports);
    my $if_port_removed = $nacltask_options{'nacltask_if_ports_removed'};

    if ( !$ports ) {
        $ports = NACL::CS::NetworkPortIfgrp->fetch(
            %common_opts,
            filter => {
                node  => $node,
                ifgrp => $ifgrp,
            },
            requested_fields => ['ports'],
        );
        $ports = $ports->ports();
        push @temp_ports, @{$ports};
    }
AGAIN: {
        use warnings;
        try {
            while (@temp_ports) {
                $opts{'port'} = $temp_ports[0];
                $pkg->SUPER::remove_port(%opts);
                shift @temp_ports;
            }
        }
        catch NACL::C::Exceptions::NetworkPortIfgrp::IfgrpIsUp with {
            my $e = shift;
            #To remove port from an ifgrp make it as down
            NACL::STask::NetworkPort->modify(
                %common_opts,
                node       => $opts{node},
                port       => $opts{ifgrp},
                'up-admin' => "false",
            );
            $Log->trace("port is modified to down");
            no warnings;
            redo AGAIN;
        } catch NACL::C::Exceptions::NetworkPortIfgrp::DoesNotExist with {
            my $e = shift;
            if ( $if_port_removed !~ /pass/i ) {
                $Log->exit() if $may_exit;
                $e->throw();
            } else {
                shift @temp_ports;
                no warnings;
                redo AGAIN;
            }
        };
    }
    # Verification for remove-port
    my $verify = $nacltask_options{nacltask_verify};
    if ($verify) {
        delete $opts{'port'};
        $opts{'ports'} = $ports;
        _verify_remove_port( \%common_opts, %opts );
    }
    $Log->exit() if $may_exit;
} ## end sub remove_ports

sub _verify_remove_port {
    $Log->enter() if $may_enter;
    my $common_opts       = shift;
    my %opts              = @_;
    my $command_interface = delete $opts{'command_interface'};

    my $ports = NACL::CS::NetworkPortIfgrp->fetch(
        %$common_opts,
        filter => {
            node  => $opts{'node'},
            ifgrp => $opts{'ifgrp'},
            ports => $opts{'ports'},
        },
        allow_empty => 1,
    );
    my @ports_list = $ports->ports() if ($ports);
    if (@ports_list) {

        # Should have been deleted, but still exists. Exception out here
        $Log->exit() if $may_exit;
        NACL::Exceptions::VerifyFailure->throw(
                  'Verification failed! The element '
                . _dump_one( \%opts )
                . ' still exists (it should have been purged)' );
    }

    # If we've gotten here, then all good
    $Log->exit() if $may_exit;
} ## end sub _verify_remove_port

1;
