
# Copyright (c) 2013-2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkPortVlan Task Module
## @author kathar@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::NetworkPortVlan;

use strict;
use warnings;

use base qw(NACL::C::NetworkPortVlan NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR BOOLEAN SCALARREF HASHREF);
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ResponseException ();
use NACL::C::Interface;
use NATE::BaseException qw(:try);

=head1 NAME

NACL::STask::NetworkPortVlan

=head1 DESCRIPTION

C<NACL::STask::NetworkPortVlan> provide methods to configure and destroy network port
vlan in ONTAP.

It is created on top of C<NACL::C::NetworkPortVlan> component. Since it is derived class
of C<NACL::C::NetworkPortVlan>, we can use all the methods of C<NACL::C::NetworkPortVlan>
from the object of this task.

=head1 CLEANUP METHODS

Cleanup can be registered for the following methods

Cleanup methods are,
    NetworkPortVlan Method  Cleanup Method
    ----------------------------------------
    create                    purge

=head1 ATTRIBUTES

=head2 command_interface

(Required)
A component object that represents the host to which to send commands.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

=head2 node

(Required)
Name of the node.

=head2 port

(Required)
Name of the Port. Ex. e0c

=head2 vlan-id

(Required)
Id of the Vlan. Ex. 10

=head2 vlan-name

(Optional) Name of the Vlan . Ex. a0a

=head1 METHODS

=head2 create

    my $NetworkPortVlan_obj  = NACL::STask::NetworkPortVlan->create(
        'command_interface'  => $command_interface,
        'node'               => $node_name,
        'port'               => $port,
        'vlan-id'            => $vlan_id,
        'nacltask_if_exists' => $action             # default die
        nacltask_to_cleanup      => 1,          # default 0,
        nacltask_cleanup_manager => $CleanupObj,
    );

(Class Method) It will create vlan on a given port.

While creating vlan, if ONTAP throws an error saying that VLAN creation on
cluster ports or node mgmt ports is not supported, in such case task will
through the same error.

Uses CMode CLI/ZAPI or 7Mode CLI.

=over

=item Options

=over

=item C<< "vlan-name" => $vlan_name >>

(Optional) Not used when vlan-id and port are provided.

=item C<< nacltask_if_exists => $action >>

(Optional) What to do if the vlan to be created already exists.
If $action is "die" (the default), then fail with an exception (in the
same way that the NetworkPortVlan component would have: by trying the
NetworkPortVlan create and letting the product complain about the vlan
already existing).  If action is "purge", then destroy the vlan
(see the C<purge> method, below) before creating a new one. if the
$action is "reuse", return the object for the vlan which already exists.

=item C<< _was_created => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the vlan
was found (value will be 0; this scenario is possible when
nacltask_if_exists => "reuse") or whether the vlan was created (value will be 1).
This is necessary to determine whether the vlan needs to be
cleaned up later.

    my $was_created;
    my $vlan_obj = NACL::STask::NetworkPortVlan->create(
        nacltask_if_exists    => 'reuse',
        _was_created => \$was_created,
        %other_opts
    );
    # Operate on $vlan_obj here
    # ...
    # Now determine whether to clean up the vlan, since we're not sure
    # whether we reused an existing vlan or created a new one
    if ($was_created) {
        # New vlan was created. Clean it up.
        $vlan_obj->purge();
    }

=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the creation did happen
successfully.

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup)
Flag indicating if this operation needs to be registered for clean up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=item command_interface, apiset_must, apiset_should, port, vlan-id etc.
All of the other various options to L<< NACL::C::NetworkPortVlan->create
| lib-NACL-C-NetworkPortVlan-pm/create >>.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::NetworkPortVlan::AlreadyExists>

This type of exception is thrown when an attempt is made to create an vlan that already exists.

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the created vlan.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            nacltask_if_exists => $pkg->_if_exists_validate_spec(),
            _was_created       => { type => SCALARREF, optional => 1 },
            nacltask_verify    => { type => BOOLEAN, default => 0 },
            $pkg->_cleanup_validate_spec(),
        },
        allow_extra => 1
    );

    my (%opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup );
    $pkg->_move_common_cleanup_opts(
        source  => \%opts,
        target  => \%opts_for_cleanup,
    );

    # Transform %opts from the options we received into the options to
    # pass to the base class method.

    my $if_exists         = delete $opts{nacltask_if_exists};
    my $verify            = delete $opts{nacltask_verify};
    my $was_created       = delete $opts{_was_created};
    my $command_interface = $opts{command_interface};
    my $self;
    $$was_created = 0;

CREATE: {
        use warnings qw(exiting);
        try {
            $self         = $pkg->SUPER::create(%opts);
            $$was_created = 1;
        }
        catch NACL::C::Exceptions::NetworkPortVlan::AlreadyExists with {
            my $exception = shift;
            $Log->trace('The vlan already exists');
            $self = $pkg->_element_exists_handler(
                create_opts        => \%opts,
                nacltask_if_exists => $if_exists,
                exception          => $exception
            );
            if ( !$self ) {
                no warnings qw(exiting);
                redo CREATE;
            }
        };
    }
    if ($verify) {
        $pkg->verify_state(%opts);
    }

    # Register this resource with the cleanup manager for cleanup.
    $pkg->_copy_common_opts_for_cleanup(
        'source'             => {%opts, %opts_for_cleanup},
        'target'             => \%opts_for_register,
        'nacltask_to_cleanup'=> \$nacltask_to_cleanup,
        'to_cleanup'         => 'purge',
    );

    if ($nacltask_to_cleanup) {
        my $other_opts = {'vlan-id' => $opts{'vlan-id'}};
        $self->_register_for_cleanup(%opts_for_register, 'other_opts' => $other_opts);
    }

    $Log->exit() if $may_exit;
    return $self;
} ## end sub create

=head2 purge

    $NetworkPortVlan->purge();

    NACL::STask::NetworkPortVlan->purge(
        'command_interface' => $ci,
        'node'              => $node_hostname,
        'vlan-name'         => $vlan-name,
        'nacltask_verify'   => '1',
    );

(Class or Instance method) Deletes the given vlan from Node. If the vlan
 doesn't exists, it will perform the action based on C<nacltask_if_purged> parameter.
 If there is any lif associated with the vlan, it will first purge the
 logical interface before deleting the vlan. Once the vlan is deleted,
 it verifies whether the vlan is deleted or not if the nacltask_verify option
 is specified as 1.

 Uses a CMode/7Mode CLI or CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required for class method, Not Applicable for instance method)
A component object that represents the host which to send commands.
See NACL::C::Component::command_interface

=item C<< node => $node_hostname >>

(Required for class method, Not Applicable for instance method)
 The hostname of the Node.

=item C<< port => $port >>

(Required for class method, Not Applicable for instance method)
port associated with the vlan.

=item C<< vlan-id => $vlan_id >>

(Required for class method, Not Applicable for instance method)
The vlan to be deleted

=item C<< vlan-name => $vlan_name >>

(Optional) Not used when vlan-id and port are provided.

=item C<< _was_deleted => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the vlan
did not exist (value will be 0; this scenario is possible when
nacltask_if_purged => "pass") or whether the vlan was deleted
(value will be 1).

=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the deletion did happen
successfully.

=item C<< nacltask_if_purged=>$if_purged_value >>

(Optional)
 If "die" (default), exception is thrown when vlan to be deleted does not
 exist
 If "pass", exits quietly when vlan to be deleted does not exist

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::NetworkPortVlan::DoesNotExist>

This type of exception is thrown when an attempt is made to delete a vlan that does not exists.

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the deleted vlan.

=back

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my $pkg_or_obj      = shift;
    my %orig_opts       = @_;
    my $additional_spec = {
        nacltask_if_purged => $pkg_or_obj->_if_action_completed_already_validate_spec(),
        _was_deleted       => { type => SCALARREF, optional => 1 },
        nacltask_verify    => { type => BOOLEAN, default  => 0 },
    };

    # Throw away options specific to the task
    $pkg_or_obj->_hash_move(
        source => \%orig_opts,
        target => {},
        move   => [ keys %{$additional_spec} ]
    );
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => $additional_spec,
    );

    my %common_opts;
    $pkg_or_obj->_copy_common_component_params_with_ci(
        source => \%opts,
        target => \%common_opts
    );
    my $if_purged = $opts{nacltask_if_purged};

    my $was_deleted = delete $opts{_was_deleted};
    $$was_deleted = 0;

    $orig_opts{'force'} = 1;

AGAIN: {
        use warnings;
        try {
            $pkg_or_obj->delete(%orig_opts);
            $$was_deleted = 1;
        }
        catch NACL::C::Exceptions::NetworkPortVlan::DoesNotExist with {
            my $e = shift;
            # Check if the command failed because the vlan
            # does not exist
            if ( $if_purged !~ /pass/i ) {
                $Log->exit() if $may_exit;
                $e->throw();
            }
        }
        catch NACL::C::Exceptions::NetworkPortVlan::PortHasLifBound with {
            my $e = shift;

            # If some error happens during delete vlan..
            # Find out logical interfaces created on this vlan.

            my $home_port;
            if ( !$opts{'vlan-name'} ) {
                $home_port = $opts{'port'} . '-' . $opts{'vlan-id'};
            } else {
                $home_port = $opts{'vlan-name'};
            }
            my @interfaces = NACL::C::Interface->find(
                %common_opts,
                filter => {
                    'home-node' => $opts{node},
                    'home-port' => $home_port,
                },
                allow_empty => 1,
            );
            foreach my $lif (@interfaces) {
		try {
                    $lif->delete();
		} catch NACL::C::Exceptions::NetworkInterface::LifEnabled with {
		    $lif->modify('status-admin' => 'down' );
		    $lif->delete();
		};
            }
            no warnings;
            redo AGAIN;
        };
    }

    if ( $opts{nacltask_verify} ) {
        delete $orig_opts{'force'};
        $pkg_or_obj->_generic_purge_verify(%orig_opts);
    }
    $Log->exit() if $may_exit;
} ## end sub purge
1;
