#
# Copyright (c) 2001-2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base class for tasks which are tied to a single component
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::STask;

=head1 NAME

NACL::STask::STask

=head1 DESCRIPTION

This is an additional base class (more of a "mixin") for "single
element tasks" to use.

"Single element tasks" are those tasks whose operations are focused on
a single element.

Methods on components are tied tightly to the CLI commands and hence each
method runs only a single command. The methods on the STasks, on the other
hand, will provide some/all of the following:

* Handle pre-requisities and dependencies required before the command can be
invoked.

* Provide defaults for fields which can be defaulted to commonly used values.

* Automatically wait for completion of the operation

Each single element task is sub-classed off the relevant component
(i.e. a "NACL::STask::Foo" will be sub-classed off "NACL::C::Foo") and
get all of their component-related methods from that class.  In order
to avoid issues with multiple inheritance, this STask class should not
derive any existing component methods, but instead entirely new
methods that may be common to all STasks.

=cut

use strict;
use warnings;
use feature 'state';

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate
    qw(validate validate_with SCALAR HASHREF OBJECT BOOLEAN ARRAYREF SCALARREF);
use NACL::Exceptions::VerifyFailure qw(:try);
use NACL::ComponentUtils qw(_optional_scalars Dumper _process_options _dump_one);
use NACL::Util::General qw(_throw_internal_error);
use NATE::BaseException;
use NACL::CS::MetroclusterNode;
use NACL::Exceptions::NoElementsFound qw(:try);
use NATE::ParamSet qw(param_global);
use NACL::C::Exceptions::MCCVerificationFailed;
use NACL::C::VserverCifsOptions;

=head2 Common_Create_Options

The following options are commonly accepted by C<create> methods:

=over

=item C<< nacltask_if_exists => die | reuse | purge >>

(Optional) What to do if the object to be created already exists.
If $action is "die" (the default), then fail with an exception (in the
same way that the component would have: by trying the
vserver create and letting the product complain about the vserver
already existing).  If action is "reuse", then do nothing (return a
task object referring to the existing object).  If action is
"purge", then delete the object (see the C<purge> method, below)
before creating a new one.

=item C<< _was_created => \$was_created >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the object
was found (value will be 0; this scenario is possible when
nacltask_if_exists => "reuse") or whether the object was created (value will be 1).
This is necessary to determine whether the object needs to be
cleaned up later.

    my $was_created;
    my $foo_obj = NACL::STask::Foo->create(
        nacltask_if_exists    => 'reuse',
        _was_created => \$was_created,
        %other_opts
    );
    # Operate on $foo_obj here
    # ...
    # Now determine whether to clean up the object, since we're not sure
    # whether we reused an existing object or created a new one
    if ($was_created) {
        # New object was created. Clean it up.
        $foo_obj->purge();
    }

B<NOTE:> This option is primarily for the purpose of cleanup. It is recommended
that the cleanup manager be used instead of this option.

=item C<< nacltask_verify => 0|1 >>

(Optional, defaults to 0)
If set to 1, verify that the object got created with the options specified.
    
=item Cleanup

The options accepted for Cleanup are documented at
L<Cleanup Options|lib-NACL-STask-STask-pm/Cleanup_Options>.

=item MCC checks

The options accepted for MCC configuration replication verification is
documented at L<MCC options|lib-NACL-STask-STask-pm/mcc_options>.

=back

=head2 Common_Purge_Options

The following options are commonly accepted by C<purge> methods:

=over

=item C<< nacltask_if_purged => die | pass >>

(Optional, defaults to "die")
Specifies what to do if the object we're trying to purge no longer
exists. A value of C<die> (the default) will result in an exception
being thrown. A value of C<pass> will mean that the exception that would
normally occur when we're trying to delete a object that does not exist
is suppressed.

=item C<< _was_deleted => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the object
did not exist (value will be 0; this scenario is possible when
nacltask_if_purged => "pass") or whether the vserver was deleted
(value will be 1).

=item C<< nacltask_verify => 0|1 >>

(Optional, defaults to 0)
If set to 1, verify that the object was purged.

=item MCC checks

The options accepted for MCC configuration replication verification is
documented at L<MCC options|lib-NACL-STask-STask-pm/mcc_options>.

=back

=head2 Common_modify_Options

=over

The following options are commonly accepted by C<modify> methods:

=item C<< nacltask_verify => 0|1 >>

(Optional, defaults to 0)
Verifies whether all of the attributes got set to the specified value.

Note that it is also possible to perform the verification later by calling
L<taskverify_modify|lib-NACL-STask-STask-pm/taskverify_modify>. This can
be used for cases where ONTAP chooses to not set all attributes to the exact
values specified. In the call we can specify those attributes we want
verified.

=item Cleanup

The options accepted for Cleanup are documented at
L<Cleanup Options|lib-NACL-STask-STask-pm/Cleanup_Options>.

=item C<< nacltask_if_modified >>

(Optional, defaults to "die")
Specifies what to do if the object we're trying to modify has already been modified
(most of the ONTAP command doenst throw any message that it has already been modified).
This would be useful for cases where ONTAP reports. A value of C<die> (the default) 
will result in an exception being thrown. A value of C<pass> will mean that the 
exception that would normally occur when we're trying to modify an object is suppressed.

=item C<< _was_modified => \$was_modified >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the object
was modified or not

=back


=cut

# This list is from _mcc_validate_spec
=head2 mcc_options

These are the options related to MCC configuration replication verification:

=over

=item C<< nacltask_to_check_mcc => 0|1 >>

Whether to perform MCC replication verification on the remote cluster. This
can also be enabled by providing the command-line parameter NACL_MCC_CHECK=1.
If the command-line parameter is provided, then it is not necessary to update
scripts to pass this option.

=item C<< nacltask_mcc_verify_retry_count => $num >>

(Defaults to 12)
The number of times to retry verification.

=item C<< polling_interval_mcc_check => $time >>

(Defaults to 10s)
The time to snooze between verification attempts.

=item C<< remote_interface => $ci >>

(Optional)
A command_interface object representing a node in the MCC remote cluster. If
not provided, then the STask library will either use the
C<mcc_remote_interface> attribute (if set) or will automatically create one.

=back

=cut

# This list is from _cleanup_validate_spec
=head2 Cleanup_Options

These are the options related to Cleanup:

=over

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, defaults to 0)
If set to 1, performs cleanup.

=item C<< nacltask_cleanup_manager => $cleanup_obj >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=item C<< nacltask_cleanup_args => { cleanup_arg1 => $val, cleanup_arg2 => $val2, ... } >>

(Optional)
This can be used to specify options to be used while cleaning up this element.
For example, if we want to specify a higher timeout value to be used only during
cleanup of this element, it can be specified like:

    nacltask_cleanup_args => {'method-timeout' => $large_value}

=back

=cut

# Let's divide up the methods into user-facing methods and developer-only
# methods. Developer-only methods should begin with an underscore, user-facing
# methods should not.

######################## Methods for task users here #######################

=head1 METHODS FOR TASK USERS

Note that the methods available on all components can be accessed here as
well (since this is a sub-class of NACL::C::Component).
See L<NACL::C::Component|lib-NACL-C-Component-pm> for a complete list of methods.

=head1 METHODS FOR TASK USERS

Note that the methods available on all components can be accessed here as
well (since this is a sub-class of NACL::C::Component).
See L<NACL::C::Component|lib-NACL-C-Component-pm> for a complete list of methods.

=head2 taskverify_create

    my $element_stask_obj = NACL::STask::Foo->create(attr1 => 'val1', attr2 => 'val2', %other_opts);
    $element_stask_obj->taskverify_create(attr1 => 'val1');

Verify that the element creates matches the attributes passed in. Note that
many "create" methods support a nacltask_verify option which automatically
calls this method. However, this method can be called separately in cases
where not all attributes get set to exactly the value specified at creation
time. The option above shows an element getting created with attr1 set to
"val1" and attr2 set to "val2". We then later verify only that attr1 is set
to "val1".

=head2 taskverify_modify

Functionally and conceptually equivalent to
L<taskverify_create|lib-NACL-STask-STask-pm/taskverify_create>, but this
method is used for modify operations.

=head2 taskverify_purge

    $stask_obj->purge();
    $stask_obj->taskverify_purge();

Verify that the element has indeed been purged. Throws a
C<NACL::Exceptions::VerifyFailure> exception if the element has not been
purged.

=cut

# Same implementation for taskverify_create and taskverify_modify
# Rather than repeat the code, we can have them assigned to the same subroutine
# reference.
# Put within a BEGIN block so that it works for both class and instance calls
BEGIN {
    no strict 'refs';
    foreach my $verify (qw(taskverify_create taskverify_modify)) {
        *{$verify} = sub {
            $Log->enter() if $may_enter;
            my $pkg_or_obj = shift;
            $pkg_or_obj->verify_state(@_);
            $Log->exit() if $may_exit;
        };
    }
    *{taskverify_purge} = sub {
        $Log->enter() if $may_enter;
        my ( $pkg_or_obj, @args ) = @_;
        $pkg_or_obj->_generic_purge_verify(@args);
        $Log->exit() if $may_exit;
    };
    use strict 'refs';
}

=head2 get_C_package_name

    my $C_pkg_name = $STask_pkg_or_obj->get_C_package_name();

Returns the relevant component package name for a given STask object or
package name. (example: returns "NACL::C::Aggregate" for
"NACL::STask::Aggregate")

=cut

sub get_C_package_name {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my $STask_pkg  = ref $pkg_or_obj ? ref $pkg_or_obj : $pkg_or_obj;
    my $C_pkg      = $STask_pkg;
    $C_pkg =~ s/::STask::/::C::/;
    $Log->debug(
        "For STask package '$STask_pkg', component package is '$C_pkg'");
    $Log->exit() if $may_exit;
    return $C_pkg;
}

=head2 C_SUPER

    $STask_pkg_or_obj->C_SUPER::method(%opts);

Invoke the component's implementation of I<method>.

Generally, C<SUPER> can be used to invoke the base class implementation
of a particular method (in this case it would be the component's
implementation of the method). However, C<SUPER> computation is done at
compile-time rather than at run-time, meaning that C<SUPER> refers to the
base class of the class in which the method has been implemented.

The two more probable cases where this would matter:

=over

=item Mix-ins

Mix-ins are classes which are not directly instantiated, but provided
functionality which is to be reused in other classes. For example,
the code for VolumeFileMove and VolumeFileCopy STasks are identical,
so has been implemented in _VolumeFile.pm and has been imported in both
STask files.

If we consider code such as:

    # NACL::STask::_VolumeFile
    sub start {
        ...
        # $pkg could be either "VolumeFileMove" or "VolumeFileCopy"
        $pkg->SUPER::start(%opts);
        ...
    }

It looks for the base-class of where it was implemented: i.e. it looks for
the base class of _VolumeFile, not of the package with which it was invoked.
For this reason, the call would end up failing (_VolumeFile's base class
isn't the component). Instead, we should use C_SUPER to explicitly direct
it to the component's implementation:

    $pkg->C_SUPER::start(%opts);

=item In scripts

If we have an STask object in a script but would like to call the
component's implementation of a method (say, for negative testing) then
again C<SUPER> cannot be used, but C<C_SUPER> can.

    $stask_obj->C_SUPER::method(%opts);

=back

(Note that the older syntax still works, but is deprecated:

    $STask_pkg_or_obj->C_SUPER(_method => $method, %opts);

)

=cut

sub C_SUPER {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => { _method => { type => SCALAR } },
        allow_extra => 1
    );
    my $method = 'C_SUPER::' . delete $opts{_method};
    $Log->exit() if $may_exit;
    return $pkg_or_obj->$method(%opts);
}

=head2 random_name_generator

    my $random_name = $pkg_or_obj->random_name_generator(
        [ prefix => $prefix, ]
        [ suffix => $suffix ] # Defaults to $pkg_or_obj->_default_rng_suffix()
    );

(Callable as a class or instance method)

This provides the same functionality and has the same interface as
L<random_name_generator|lib-NACL-GeneralUtils-pm/random_name_generator>
with the following differences:

=over

=item *

This is callable as a class/instance method

=item *

The C<suffix> parameter is defaulted to the default RNG suffix as defined
by the method _default_rng_suffix.

=back

=cut

sub random_name_generator {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, %opts ) = @_;
    $opts{suffix} ||= $pkg_or_obj->_default_rng_suffix();
    my $name = NACL::GeneralUtils::random_name_generator(%opts);
    $Log->exit($name);
    return $name;
}

=head2 cast_to_component

    my $c_obj = $task_obj->cast_to_component();

For a given STask object, return a component object modeling the same
element.

This is useful for cases where we want to explicitly call the component
implementation of methods, or if we want to use component-specific
features such as C<extended_query>.

=cut

sub cast_to_component {
    $Log->enter() if $may_enter;
    my $self  = $_[0];
    my $C_pkg = ref $self;
    $C_pkg =~ s/::STask::/::C::/;
    my $c_obj = $C_pkg->new( $self->get_pk_hash_with_ci() );
    $Log->exit() if $may_exit;
    return $c_obj;
}

###################### Methods for task developers here ######################

=head1 METHODS FOR TASK DEVELOPERS

All of the developer-only methods implemented on components can be accessed
here as well. See L<NACL::C::Component|lib-NACL-C-Component-pm>.

=head2 _copy_common_component_params

See L<NACL::GeneralUtils::_copy_common_component_params|lib-NACL-GeneralUtils-pm/_copy_common_component_params>.

=head2 _move_common_component_params

See L<NACL::GeneralUtils::_move_common_component_params|lib-NACL-GeneralUtils-pm/_move_common_component_params>.

=head2 _copy_common_component_params_with_ci

See L<NACL::GeneralUtils::_copy_common_component_params_with_ci|lib-NACL-GeneralUtils-pm/_copy_common_component_params_with_ci>.

=head2 _move_common_component_params_with_ci

See L<NACL::GeneralUtils::_move_common_component_params_with_ci|lib-NACL-GeneralUtils-pm/_move_common_component_params_with_ci>.

=head2 _copy_nacltask_options

See L<NACL::TaskUtils::_copy_nacltask_options|lib-NACL-TaskUtils-pm/_copy_nacltask_options>.

=head2 _move_nacltask_options

See L<NACL::TaskUtils::_move_nacltask_options|lib-NACL-TaskUtils-pm/_move_nacltask_options>.

=head2 wait_for_replay

See L<NACL::TaskUtils::wait_for_replay|lib-NACL-TaskUtils-pm/wait_for_replay>.

=head2 _if_action_completed_already_validate_callback

L<_if_action_completed_already_validate_callback|lib-NACL-TaskUtils-pm/_if_action_completed_already_validate_callback>

=head2 _if_action_completed_already_validate_spec

L<_if_action_completed_already_validate_spec|lib-NACL-TaskUtils-pm/_if_action_completed_already_validate_spec>

=cut

use NACL::GeneralUtils qw(_copy_common_component_params
    _move_common_component_params
    _copy_common_component_params_with_ci
    _move_common_component_params_with_ci
    _copy_or_move_common_params
);
use NACL::TaskUtils qw(
    _copy_nacltask_options
    _move_nacltask_options
    wait_for_replay
    _if_action_completed_already_validate_callback
    _if_action_completed_already_validate_spec
);

=head2 _copy_retry_params

    $pkg_or_obj->_copy_retry_params(
        source => \%source,
        target => \%target
    );

Copies all the arguments required for C<nacl_method_retry> from the source
to the target hash. The arguments in the source hash are named like
"nacltask_retry_foo", the keys in the target hash will simply be named
"foo".

=cut

sub _copy_retry_params {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    _copy_or_move_retry_params( @args, _copy => 1 );
    $Log->exit() if $may_exit;
}

=head2 _move_retry_params

    $pkg_or_obj->_move_retry_params(
        source => \%source,
        target => \%target
    );

Similar to L<_copy_retry_params|lib-NACL-STask-STask-pm/_copy_retry_params>, but deletes the entries from the source
hash.

=cut

sub _move_retry_params {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    _copy_or_move_retry_params( @args, _move => 1 );
    $Log->exit() if $may_exit;
}

sub _copy_or_move_retry_params {
    $Log->enter() if $may_enter;
    my %opts = validate(
        @_,
        {   source => { type => HASHREF },
            target => { type => HASHREF },
            _copy  => { type => BOOLEAN, optional => 1 },
            _move  => { type => BOOLEAN, optional => 1 },
        }
    );

    if ( !$opts{_copy} && !$opts{_move} ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
                  "Internal error: Either of '_copy' "
                . "or '_move' should have been set to 1 in the call to "
                . "'_copy_or_move_nacltask_options()' but neither were" );
    }

    if ( $opts{_copy} && $opts{_move} ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
                  "Internal error: Either of '_copy' "
                . "or '_move' should have been set to 1 in the call to "
                . "_copy_or_move_nacltask_options()' but both were" );
    }

    foreach my $key ( keys %{ $opts{source} } ) {
        if ( $key =~ /^nacltask_retry_/ ) {
            my $to_key = $key;
            $to_key =~ s/^nacltask_retry_//;
            $opts{target}->{$to_key} = $opts{source}->{$key};
            delete $opts{source}->{$key} if $opts{_move};
        }
    }

    $Log->exit() if $may_exit;
}

use NACL::ComponentUtils qw(_hash_copy
    _hash_move
    _dump_one
);

=head2 _generic_purge_verify

    sub taskverify_method {
        my $pkg_or_obj = shift;
        $pkg_or_obj->_generic_create_modify_verify(%opts);
    }

(Callable as either a package or instance method)
This is a generic implementation for the verification that would need to be
performed for purge operations. This checks whether the purged element still
exists. Verification failures result in a
L<NACL::Exceptions::VerifyFailure|lib-NACL-Exceptions-VerifyFailure-pm>
being thrown.

This method should be called by the task's verify routine.

=over

=item Options

Nothing in addition to the options passed to the component method call.

=back

=cut

sub _generic_purge_verify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with( params => \@_ );
    my $pkg        = ref $pkg_or_obj || $pkg_or_obj;
    my %common_params;
    $pkg->_move_common_component_params_with_ci(
        source => \%opts,
        target => \%common_params
    );

    if ( $pkg->find( %common_params, filter => {%opts}, allow_empty => 1 ) ) {

            $Log->exit() if $may_exit;
            NACL::Exceptions::VerifyFailure->throw(
                      'Verification failed! The element '
                    . _dump_one( \%opts )
                    . ' still exists (it should have been purged)' );
    }

    # If we've gotten here, then all good
    $Log->exit() if $may_exit;
}

=head2 _generic_create_modify_verify

    sub taskverify_method {
        my $pkg_or_obj = shift;
        $pkg_or_obj->_generic_create_modify_verify(%opts);
    }

(Callable as either a package or instance method)
This is a generic implementation for the verification that would need to be
performed for create/modify operations. This checks whether the specified
element has all the fields set to the values as they were provided in the
call to create/modify. Verification failures result in a
L<NACL::Exceptions::VerifyFailure|lib-NACL-Exceptions-VerifyFailure-pm>
exception being thrown.

This method should be called by the task's verify routine.

Note that this is not guaranteed to handle the verification for all
create/modify operations, but it should handle most cases. For cases where
this generic routine cannot handle the verification, the verification logic
must be provided in the task verify routine.

=over

=item Options

Nothing in addition to the options that were passed to the relevant component
call.

=back

=cut

sub _generic_create_modify_verify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params      => \@_,
        allow_extra => 1
    );

    my $pkg = ref $pkg_or_obj || $pkg_or_obj;
    $pkg->verify_state(%opts);

    $Log->exit() if $may_exit;
}

=head2 _call_method_then_verify

    # In NACL::STask::Foo
    sub bar {
        $pkg_or_obj->_call_method_then_verify(
            @_,
            [ _verification_method => $verify_method, ]
            [ _method              => $method ]
    );
    ...

(Callable as either a package or instance method)
This method adds verification support to task methods which directly reuse the
component's implementation. (i.e. the task does not contain an overridden
implementation)

Certain task methods have support for verification of the task method's
operation. This is achieved by having a separate verification routine,
which specifies how to verify the operation.

=over

=item Options

=over

=item C<< _method => $method >>

(Optional)
This is the name of the task method for which we want to call the component
method and then verify. It is optional and defaults to being the subroutine name
of the caller of _call_method_then_verify. However, if _call_method_then_verify
were called from a method other than the task method itself then the caller
would not be the task method name. In such a case we would need to provide
the task method name explicitly.

=item C<< _verification_method => $verify_method >>

(Optional)
This is the verification method. It is optional and defaults to being
"taskverify_$method" (where $method is the same as _method explained above).
(Note that this is the convention for the naming of task verification routines)
If the verification routine has a different name, we need to specify it using
this parameter

=back

=back

=cut

sub _call_method_then_verify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'nacltask_verify' => { type => SCALAR, default => 0 },
            _optional_scalars(qw(_verification_method _method)),
        },
        allow_extra => 1
    );
    my $verify = delete $opts{'nacltask_verify'};
    my $method = delete $opts{_method};
    if ( !$method ) {
        $method = ( caller(1) )[3];
        $method =~ s/.*:://;
    }
    my $super_method         = "C_SUPER::$method";
    my $verification_routine = delete $opts{_verification_method}
        || "taskverify_$method";
    my $ret = $pkg_or_obj->$super_method(%opts);
    if ($verify) {
        $pkg_or_obj->$verification_routine(%opts);
    }
    $Log->exit() if $may_exit;
    return $ret;
}

=head2 _if_exists_validate_callback

    sub foo {
        my $pkg = shift;
        my %opts = validate(@_,
            {
                nacltask_if_exists => {
                    %other_validation,
                    callbacks => $pkg->_if_exists_validate_callback(),
                },
                ...
            }
        );
        ...

This method provides the Params::Validate callback, which validates that
the value of "nacltask_if_exists" is 'die', 'reuse' or 'purge'.

(See
http://search.cpan.org/~drolsky/Params-Validate-0.95/lib/Params/Validate.pm#Callback_Validation
for an explanation of callback validation)

It's unlikely this will have any callers other than
C<_if_exists_validate_spec()>, but this has been split into a separate method
in case C<_if_exists_validate_spec> cannot be used but this spec is necessary.

=cut

sub _if_exists_validate_callback {
    return {
        "Value of 'nacltask_if_exists' should be 'die', 'reuse' or 'purge'"
            => sub { $_[0] =~ /^(die)|(reuse)|(purge)$/ }
    };
}

=head2 _if_exists_validate_spec

    sub create {
        my $pkg = shift;
        $pkg->_common_validate_with(
            params => \@_,
            additional_spec => {
                nacltask_if_exists => $pkg->_if_exists_validate_spec(),
                ...
            }
        );
        ...
    }

The validation spec for the C<if_exists> option supported by many
STask::create methods. This is a scalar, which defaults to 'die' and
uses the C<_if_exists_validate_callback> to validate that the value
passed is valid.

This method is most likely to be called by STask::create methods.

=cut

sub _if_exists_validate_spec {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $Log->exit() if $may_exit;
    return {
        type      => SCALAR,
        default   => 'die',
        callbacks => $pkg->_if_exists_validate_callback()
    };
}

=head2 _handle_if_exists_action

DEPRECATED, code should attempt to create and then react to any errors
rather than checking first for existence. See _element_exists_handler.

    sub create {
        my $pkg = shift;
        # Validate options
        # Weed out task-specific options
        # %opts should now contain only the options which will get passed
        # to the component's "create"
        my $self = $pkg->_handle_if_exists_action(
            if_exists         => $if_exists,
            command_interface => $command_interface,
            create_opts          => \%opts
        );
        if (!$self) {
            $self = $pkg->SUPER::create(%opts);
            ...
        }
        ...
        return $self;
    }

This method is used to handle the various C<if_exists> actions. Specifically,
if the action is not 'die', then we check if a matching element already exists.
If the action is 'purge' then we purge the existing element. If the action
is 'reuse' then we return the task object for the found element.

For action 'die', the task method implementation will later call "create"
which will fail if the element already exists, so we don't need to do anything
special here.

Only in the case where a matching element is found and it can be reused
will this method have a return value (which would be the task object for
that element). In all other cases, this method returns nothing. Hence, the
task's "create" method should use this return value to determine whether
the component's "create" method should be invoked.

=over

=item Options

=over

=item C<< if_exists => $action >>

(Required)
The C<if_exists> action.

=item C<< command_interface => $ci >>

(Required)
This is a component/task object that represents the host to which to
send commands.
See L<NACL::C::CommandInterface|lib-NACL-C-CommandInterface-pm>.

=item C<< create_opts => \%create_opts >>

(Required)
This is a reference to the hash of options which will get passed
to the component's "create".

=back

=back

=cut

sub _handle_if_exists_action {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate(
        @_,
        {   if_exists   => { type => SCALAR },
            create_opts => { type => HASHREF }
        }
    );

    my $if_exists         = $opts{if_exists};
    my %create_opts       = %{ $opts{create_opts} };
    my $command_interface = $create_opts{command_interface};

    my $existing = undef;

    # We will not handle the if_exists => 'die' case here since the
    # create call will anyway throw an error if the element exists.
    if ( $if_exists ne 'die' ) {
        $Log->trace('Attempt to find the element');

        # Copy the common component options.
        my %common_opts = ();
        $pkg->_hash_copy(
            source => \%create_opts,
            target => \%common_opts,
            copy => [ keys( %{ $pkg->_common_validate_spec_without_ci() } ) ]
        );
        $existing = $pkg->find(
            command_interface => $command_interface,
            filter            => $pkg->get_primary_keys_options(%create_opts),
            allow_empty       => 1,
            %common_opts
        );
    }

    if ($existing) {
        $Log->trace('Found a matching element');
        if ( $if_exists eq 'purge' ) {
            my $purge_opts = $pkg->_hash_copy(
                source => \%create_opts,
                copy   => [
                    keys %{ $pkg->_common_validate_spec() },
                    $pkg->get_primary_keys(),
                ],
            );
            $Log->trace(
                "if_exists action is 'purge', purge the found element");
            $pkg->purge(%$purge_opts);
            $Log->exit() if $may_exit;
            return;
        } elsif ( $if_exists eq 'reuse' ) {
            $Log->exit() if $may_exit;
            return $existing;
        }
    }

    # Either the if_exists action is 'die', so we did not check if the element
    # existed, or we did not find a matching element
    $Log->exit() if $may_exit;
    return;
}

=head2 _element_exists_handler

    sub create {
        my $pkg = shift;
        # Validate options
        # Weed out task-specific options
        # %opts should now contain only the options which will get passed
        # to the component's "create"
        CREATE: {
            use warnings qw(exiting);
            try {
                $self = $pkg->create(%opts);
            } catch NACL::APISet::Exceptions::ResponseException with {
                my $exception = shift;

                if ($exception->text() =~ /$element_already_exists/) {
                    $self = $pkg->_element_exists_handler(
                        create_opts          => \%opts,
                        nacltask_if_exists   => $if_exists,
                        exception            => $exception
                    );
                    if (!$self) {
                        no warnings qw(exiting);
                        redo CREATE;
                    }
                }
                ...
            };
        }

        ...
        return $self;
    }

This method is used to handle the various C<if_exists> actions if the element
was found to already exist. Specifically, it does the following:

=over

=item C< nacltask_if_exists = "die" >

Re-throws the original APISet exception.

=item C< nacltask_if_exists = "reuse" >

Constructs and returns a new STask object for the existing element.

=item C< nacltask_if_exists = "purge" >

Purges the existing element. No return value. In this case, the caller should
retry the create call.

=back

=over

=item Options

=over

=item C<< nacltask_if_exists => $action >>

(Required)

The C<nacltask_if_exists> action.

=item C<< create_opts => \%create_opts >>

(Required)

This is a reference to the hash of options which will get passed
to the component's "create" call.

=item C<< exception => $exception_obj >>

(Required, isa NACL::APISet::Exceptions::ResponseException)

The APISet exception that got thrown due to the element already existing.

=item C<< create_to_new_purge_mapping => \%create_to_new_mapping >>

(Optional, HASHREF, probably necessary only for DFM STasks)
The mapping of the names of the options as they are accepted by
the C<create> method to what they are accepted as for C<new> and
C<purge>. For example, RbacUser->create() accepts "user-name" as argument.
However, the primary key (which is what is accepted by C<new> and C<purge>)
is "user-name-or-id". Hence, here we would provide this mapping:
{ "user-name" => "user-name-or-id" }

=back

=back

=cut

sub _element_exists_handler {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate(
        @_,
        {   nacltask_if_exists => { type => SCALAR },
            create_opts        => { type => HASHREF },
            purge_opts         => { type => HASHREF, default => {} },
            exception =>
                { isa => 'NACL::APISet::Exceptions::ResponseException' },
            create_to_new_purge_mapping => { type => HASHREF, default => {} },
        }
    );

    my $if_exists         = $opts{nacltask_if_exists};
    my %create_opts       = %{ $opts{create_opts} };
    my %purge_opts        = %{ $opts{purge_opts} };
    my $command_interface = $create_opts{command_interface};
    my $exception         = $opts{exception};

    if ( $if_exists eq 'die' ) {
        $Log->exit() if $may_exit;
        $exception->throw();
    } else {
        if ( keys %{ $opts{create_to_new_purge_mapping} } ) {
            $pkg->_hash_move(
                source => \%create_opts,
                target => \%create_opts,
                map    => $opts{create_to_new_purge_mapping}
            );
        }
        my %primary_keys = %{ $pkg->get_primary_keys_options(%create_opts) };

        if ( $if_exists eq 'reuse' ) {
            $Log->trace( "if_exists action is 'reuse', create a new object "
                    . 'for the existing element' );

            # Handle the case of one or more of the options being provided as
            # objects. Use _process_options to turn them into scalars. See
            # burt 534911.
            $pkg->_process_options( options => \%primary_keys );
            %primary_keys = %{ NACL::C::Component::_convert_to_underscores(
                    \%primary_keys ) };
            my $self = $pkg->new( %primary_keys,
                command_interface => $command_interface );
            $Log->exit() if $may_exit;
            return $self;
        } else {
            $Log->trace( "if_exists action is 'purge', purge the "
                    . 'existing element' );
            my %common_opts = ();
            $pkg->_hash_copy(
                source => \%create_opts,
                target => \%common_opts,
                copy   => [ keys( %{ $pkg->_common_validate_spec() } ) ]
            );
            $pkg->purge( %primary_keys, %common_opts, %purge_opts );
        }
    }

    $Log->exit() if $may_exit;
    return;
}

=head2 _handle_if_purged

    # In purge() of an STask
    try {
        $pkg_or_obj->delete();
    } catch NACL::C::Exceptions::Foo::DoesNotExist with {
        $pkg_or_obj->_handle_if_purged(
            exception          => $_[0],
            nacltask_if_purged => $if_purged
        );
    };

This method decides (based on the value of the C<nacltask_if_purged> argument)
whether to re-throw the exception that was thrown because the element does
not exist.

=over

=item Options

=over

=item C<< exception => $exception_obj >>

The exception object that was caught in the catch block.

=item C<< nacltask_if_purged => $if_purged >>

A value of pass/die which indicates whether the suppress the exception or
to re-throw it.

=back

=back

=cut

sub _handle_if_purged {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            exception => {
                type => OBJECT,
                isa  => 'NACL::APISet::Exceptions::ResponseException'
            },
            nacltask_if_purged => { type => SCALAR }
        }
    );
    my $exception = $opts{exception};
    if ( $opts{nacltask_if_purged} =~ /die/i ) {
        $Log->exit() if $may_exit;
        $exception->throw();
    }
    $Log->exit() if $may_exit;
}

BEGIN {

    # These shouldn't be used anymore but are kept for backwards compatibility
    *{_component_common_validate_spec} = *{_common_validate_spec};
    *{_component_common_validate_spec_without_ci}
        = *{_common_validate_spec_without_ci};
}

=head2 _default_rng_suffix

This method provides the default suffix to be used when the random
name generator is invoked to generate a random name for that element.
For example, for Volumes it is "vol" which means that the random
string generated for volumes will be of the form <random_string>_vol.
The base class (STask.pm) defaults it to being an empty string.

=cut

sub _default_rng_suffix {
    return '';
}

=head2 _rng_args_validate_spec

    sub create {
        my $pkg = shift;
        $pkg->_common_validate_with(
            params => \@_,
            additional_spec => {
                nacltask_rng_args => $pkg->_rng_args_validate_spec(),
                ...
            }
        );
        ...
    }

The validation spec for the random_name_generator arguments
passed into the call to create.

=cut

sub _rng_args_validate_spec {
    return { type => HASHREF, default => {} };
}

=head2 _cleanup_validate_spec

    sub modify {
        my ($pkg_or_obj, @args) = @_;
        $pkg_or_obj->_common_validate_with(
            params => \@args,
            additional_spec => {
                $pkg_or_obj->_cleanup_validate_spec(),
                ...
            }
        );
        ...
    }

The validation spec for the cleanup related arguments. This method
returns a hash in list context and a hash-reference in scalar
context.

=cut

# Be sure to update the POD section "Cleanup_Options" when this is updated.
sub _cleanup_validate_spec {
    my %extra_hash = defined NACL::Cleanup->_cleanup_enabled() ? (optional => 1 ) : (default => 0);
    my %hash = (
        nacltask_to_cleanup => { type => SCALAR, %extra_hash },
        nacltask_cleanup_manager =>
            { type => OBJECT, isa => 'NACL::Cleanup', optional => 1 },
        nacltask_cleanup_args => { type => HASHREF, default => {} },
    );
    return wantarray ? %hash : {%hash};
}

=head2 parents

(For cleanup implementation only)

This method returns a empty list as of now and is an implementation of parents method
in the base class.

=cut

sub parents {
    $Log->enter() if $may_enter;

    $Log->exit() if $may_exit;
    return qw();
}

sub _register_for_cleanup {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            to_cleanup => { type => SCALAR,  optional => 0 },
            orig_opts  => { type => HASHREF, default  => {} },
            new_opts   => { type => HASHREF, default  => {} },
            nacltask_cleanup_manager =>
                { type => OBJECT, isa => 'NACL::Cleanup' },
            other_opts            => { type => HASHREF, default  => {} },
            group                 => { type => SCALAR,  optional => 1 },
            nacltask_cleanup_args => { type => HASHREF, default  => {} }
        },
    );

    my $pkg         = $pkg_or_obj->get_package_name();
    my $cleanup_obj = $opts{nacltask_cleanup_manager};
    my $group       = $opts{group};
    my $command     = $opts{to_cleanup};
    $pkg->_process_options( 'options' => $opts{'orig_opts'} );
    $pkg->_process_options( 'options' => $opts{'new_opts'} );
    $pkg->_process_options( 'options' => $opts{'other_opts'} );
    my %orig_opts         = %{ $opts{orig_opts} };
    my %new_opts          = %{ $opts{new_opts} };
    my $primary_keys      = $pkg_or_obj->_verify_update_primary_keys(%opts);
    my $host              = $opts{command_interface}->name();
    my $common_table_data = {
        'command'     => { 'command' => $command },
        'package'     => { 'package' => $pkg },
        'primary_key' => $primary_keys,
        'host' =>
            { 'name' => $host, 'cmd_interface' => $opts{command_interface} },
    };

    my $additional_table_data = {};

    if ($group) {
        $additional_table_data->{'group'} = { 'group' => $group };
    }
    my %arguments;
    if ( scalar keys %new_opts && scalar keys %orig_opts ) {
        %arguments = ( %new_opts, %orig_opts );
    }
    if ( $opts{'other_opts'} ) {
        %arguments = ( %arguments, %{ $opts{'other_opts'} } );
    }
    if ( $opts{'nacltask_cleanup_args'} ) {
        %arguments = ( %arguments, %{ $opts{'nacltask_cleanup_args'} } );
    }
    if ( scalar %arguments ) {
        $additional_table_data->{'argument'} = \%arguments;
    }
    my $table_data = { %{$common_table_data}, %{$additional_table_data} };
    $cleanup_obj->register( %{$table_data} );
    $Log->exit() if $may_exit;
}

sub _get_object_instance {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    if ( ref $pkg_or_obj ) {
        $Log->exit() if $may_exit;
        return $pkg_or_obj;
    } else {
        my $pkg = $pkg_or_obj->get_package_name();
        my $primary_keys_with_ci
            = $pkg->_get_primary_keys_w_or_wo_ci_from_pkg_or_obj( @args,
            want_ci => 1 );
        my $self = $pkg->new( %{$primary_keys_with_ci} );
        $Log->exit() if $may_exit;
        return $self;
    }

}

# This method returns the basic mandatory options for _register_for_cleanup
# it adds up default cleanup manager object if not specified.

sub _copy_common_opts_for_cleanup {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            source              => { type => HASHREF },
            target              => { type => HASHREF },
            nacltask_to_cleanup => { type => SCALARREF },
            want_ci             => { type => BOOLEAN, default => 1 },
            to_cleanup          => { type => SCALAR },
        },
    );

    my $source  = $opts{source};
    my $target  = $opts{target};
    my $want_ci = delete $opts{'want_ci'};
    my %cleanup_opts;
    my $nacltask_to_cleanup = $opts{'nacltask_to_cleanup'};
    $pkg_or_obj->_move_common_cleanup_opts(
        source => $source,
        target => \%cleanup_opts,
    );
    require NACL::Cleanup;
    if ( $cleanup_opts{'nacltask_cleanup_manager'}
        || $cleanup_opts{'nacltask_to_cleanup'}
        // NACL::Cleanup->_cleanup_enabled())
    {
        $$nacltask_to_cleanup = 1;
        $cleanup_opts{'nacltask_cleanup_manager'}
	    ||= NACL::Realm->realm()->cleanup();
        $cleanup_opts{'to_cleanup'} = delete $opts{'to_cleanup'};
        delete $cleanup_opts{'nacltask_to_cleanup'};
        my %common_component_opts
            = $pkg_or_obj->_get_primary_keys_w_or_wo_ci_from_pkg_or_obj(
            %{$source}, want_ci => $want_ci );
        my $keys = [ keys %common_component_opts, keys %cleanup_opts ];
        $pkg_or_obj->_hash_copy_defined(
            source => { %common_component_opts, %cleanup_opts },
            target => $target,
            copy   => $keys
        );
    } else {
        $$nacltask_to_cleanup = 0;
    }
    $Log->exit() if $may_exit;
}

sub _register_for_cleanup_validate_spec {
    my $pkg_or_obj = shift;
    my $pkg        = $pkg_or_obj->get_package_name();
    my %hash       = (
        cleanup_method_to_update => { type => SCALAR, optional => 0 },
        cleanup_method_registerd => { type => SCALAR, optional => 0 },
        cleanup_manager => { type => OBJECT, isa => 'NACL::Cleanup' },
        existing_obj    => { type => ARRAYREF },
    );
    return wantarray ? %hash : {%hash};
}

# This is a method which helps move the common cleanup opts from
# the source hash to destination hash
sub _move_common_cleanup_opts {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            source              => { type => HASHREF },
            target              => { type => HASHREF },
            nacltask_to_cleanup => { type => SCALARREF, optional => 1 }
        },
    );
    my $source              = $opts{source};
    my $target              = $opts{target};
    my $nacltask_to_cleanup = delete $opts{nacltask_to_cleanup};

    my @cleanup_opts_to_move
        = qw (nacltask_cleanup_manager nacltask_to_cleanup nacltask_cleanup_args);

    $pkg_or_obj->_hash_move_defined(
        source => $source,
        target => $target,
        move   => \@cleanup_opts_to_move,
    );
    if ( ref $nacltask_to_cleanup ) {
        if (   $target->{'nacltask_to_cleanup'}
            || $target->{'nacltask_cleanup_manager'} )
        {
            $$nacltask_to_cleanup = 1;
        }
    }
    $Log->exit() if $may_exit;
}

sub _common_modify_for_cleanup {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            $pkg_or_obj->_cleanup_validate_spec(),
            _to_cleanup_method => { type => SCALAR, default => 'modify' },
        },
        allow_extra => 1,
    );

    my $to_cleanup_method = delete $opts{_to_cleanup_method};
    my ( %opts_for_cleanup, $nacltask_to_cleanup );

    $pkg_or_obj->_copy_common_opts_for_cleanup(
        'source'              => \%opts,
        'target'              => \%opts_for_cleanup,
        'nacltask_to_cleanup' => \$nacltask_to_cleanup,
        'to_cleanup'          => $to_cleanup_method
    );

    my %nacltask_options;
    $pkg_or_obj->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_options
    );
    my ( %new_opts, %orig_opts, %common_opts );
    %new_opts = %opts;
    $pkg_or_obj->_move_common_component_params_with_ci(
        source => \%new_opts,
        target => \%common_opts,
    );

    my %primary_keys;
    $pkg_or_obj->_move_primary_keys(
        source => \%new_opts,
        target => \%primary_keys
    );

    my @keys   = keys %new_opts;
    my $cs_pkg = $pkg_or_obj->get_CS_package_name();
    my $cs_obj = $cs_pkg->fetch(
        %common_opts,
        filter           => \%primary_keys,
        requested_fields => \@keys
    );
    %orig_opts = map { my $val = $cs_obj->$_(); $_ => $val } @keys;

    %opts_for_cleanup = (
        %opts_for_cleanup,
        'orig_opts' => \%orig_opts,
        'new_opts'  => \%new_opts
    );

    $Log->exit() if $may_exit;
    return \%opts_for_cleanup;
}

=head2 _wait_for_async_zapi

    sub modify {
        ...
        $opts{job_component} ||= \$dummy;
        $pkg_or_obj->SUPER::modify(%opts);
        ...
        $pkg_or_obj->_wait_for_async_zapi(%opts);
    }

The "async" form of ZAPIs could start a job in the background, which would
need to be waited on (the CLI defaults to running in the foreground). If
a job was started, it needs to be waited on. This method handles this.
This accepts all of the arguments of the main method: which must include
"job_component".

=cut

sub _wait_for_async_zapi {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;

    # Already gone through _common_validate_with, so it shouldn't be
    # necessary again
    my %opts = validate_with(
        params => \@args,
        spec   => {
            job_component    => { type => SCALARREF },
            'method-timeout' => { type => SCALAR, default => 300 }
        },
        allow_extra => 1
    );

    my $job = ${ $opts{job_component} };
    if ( defined $job ) {
        my %common_opts;
        $pkg_or_obj->_copy_common_component_params(
            source => \%opts,
            target => \%common_opts,
        );
        $job->wait_on_job(%common_opts);
    }

    $Log->exit() if $may_exit;
}

=head2 _mcc_validate_spec

    sub create {
        my ($pkg_or_obj, @args) = @_;
        $pkg_or_obj->_common_validate_with(
            params => \@args,
            additional_spec => {
                $pkg_or_obj->_mcc_validate_spec(),
                ...
            }
        );
    }

The validation spec for the mcc related arguments. This method
returns a hash in list context and a hash-reference in scalar
context.

=cut

# When this is updated, be sure to update the "mcc_options" POD section
# above.
sub _mcc_validate_spec {
    my %hash = (
        nacltask_to_check_mcc           => { type => SCALAR, optional => 1 },
        nacltask_mcc_verify_retry_count => { type => SCALAR, default => 12 },
        polling_interval_mcc_check      => { type => SCALAR, default => 10 },
        remote_interface                => {
            isa      => "NACL::C::CommandInterface::ONTAP",
            type     => OBJECT,
            optional => 1
        },
    );
    return wantarray ? %hash : {%hash};
}

=head2 _check_in_metrocluster_node

    $self->_mcc_validate_spec(%mcc_opts);
    $self->_check_in_metrocluster_node(
            command_interface => $opts{command_interface},
            method_used       => "create",
            %mcc_opts,
    );
    (Instance call)

    (or)

    NACL::STask::Aggregate->_mcc_validate_spec(%mcc_opts);
    NACL::STask::Aggregate->_check_in_metrocluster_node(
            command_interface => $opts{command_interface},
            method_used       => "create",
            %mcc_opts,
    );
    (Class call)

This method is used to check the replication of the object on the
metroclustered partner. The metrocluster configuration is required for this.
Before the call to this the validation of the mcc related parameters should
have been done by calling _mcc_validate_spec() subroutine.

=over

=item Options

=over

=item C<< command_interface => $Node >>

(Required) The command interface of the filer on which the MCC check is done.

=item C<< method_used >>

(Required) The method which is used for MCC check. This method is the name
of the operation which is used for MCC data replication. Eg : create, delete,
modify etc.

=item C<< nacltask_to_check_mcc => $boolean >>

(Required) This parameter is required when the user wants to check for MCC
type of replication is done. It intakes boolean value.
If 1 is passed MCC check is done , else if 0 is passed MCC check is not done.
Default is assigned by _mcc_validate_spec() method as 0.

=item C<< nacltask_mcc_verify_retry_count => $count >>

(Required) This value contains the times the polling is done to check whether
the particular object is replicated on the mcc clustered partner.
The default value is 12 which is assigned by _mcc_validate_spec() method.

=item C<< polling_interval_mcc_check => $poll_int >>

(Required) This value contains the interval between each polling for the MCC check.
The default value is 10 secs which is assigned by _mcc_validate_spec() method.

=item C<< remote_interface => $ci >>

(Optional) This parameter contains the command interface of the partner cluster
in the mcc configuration. This will the partner of the node which is passed
to command_interface parameter. If the value is not passed the partner node
is found using "metrocluster node show" command.

=back

=back

=cut

sub _check_in_metrocluster_node {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            command_interface               => { type => OBJECT },
            method_used                     => { type => SCALAR },
            nacltask_to_check_mcc           => { type => SCALAR, optional => 1 },
            nacltask_mcc_verify_retry_count => { type => SCALAR },
            polling_interval_mcc_check      => { type => SCALAR },
            remote_interface => { type => OBJECT, optional => 1 },
        },
    );
    
    my $check_mcc = delete $opts{nacltask_to_check_mcc}
        // param_global()->get('NACL_MCC_CHECK');
    if (!$check_mcc) {
        $Log->exit() if $may_exit;
        return;
    }

    my %common_params;
    my $retry             = delete $opts{nacltask_mcc_verify_retry_count};
    my $polling_interval  = delete $opts{polling_interval_mcc_check};
    my $method            = delete $opts{method_used};
    my $command_interface = $opts{command_interface};

    $pkg_or_obj->_copy_common_component_params(
        source => \%opts,
        target => \%common_params
    );

    if (!$command_interface->is_in_mcc_environment(%common_params)) {
        $Log->comment('MCC not configured, so MCC verification disabled');
        $Log->exit() if $may_exit;
        return;
    }
    
    my $mcc_node_cs;
    my $get_mcc_node_cs = sub {
        $mcc_node_cs = NACL::CS::MetroclusterNode->fetch(
            command_interface => $command_interface,
            filter            => {node => $command_interface->node()},
            requested_fields  => [qw(dr-partner cluster)],
            %common_params
        );
    };

    my $partner_ci;
    if (defined $opts{remote_interface} ) {
        $partner_ci = $opts{remote_interface};
    } else {
        if ($opts{command_interface}->mcc_remote_interface_isset()) {
            $partner_ci = $command_interface->mcc_remote_interface();
        } else {
            $get_mcc_node_cs->();
            my $partner_node = $mcc_node_cs->dr_partner();
            $partner_ci = NACL::C::Node->new(node => $partner_node);
            $command_interface->mcc_remote_interface($partner_ci);
        }
    }

    # orig_primary_keys = the original values
    # opts_to_find = the values on the MCC partner. For example,
    # if orig_primary_keys{vserver} is "vs", then opts_to_find{vserver}
    # will be "vs-mc".
    my %orig_primary_keys;
    $pkg_or_obj->_copy_primary_keys(
        source => \%opts,
        target => \%orig_primary_keys
    );
    my %opts_to_find = %orig_primary_keys;

    my $pkg = $pkg_or_obj->get_package_name();
    $pkg =~ /NACL::STask::(.*)/;
    my $object = $1;

    if ( $opts_to_find{aggregate} ) {
        $get_mcc_node_cs->();
        $opts_to_find{cluster} = $mcc_node_cs->cluster();
    } elsif ( $opts_to_find{vserver} ) {
        $opts_to_find{vserver} = $opts_to_find{vserver} . "-mc";
    }

    #find object on the partner-cluster
    $Log->comment("MCC verification for $object $method on remote cluster");
    
    my $count;
    while (1) {
        my $obj_found = $pkg->find(
            command_interface => $partner_ci,
            filter            => \%opts_to_find,
            allow_empty       => 1,
            %common_params,
        );
        if ($method eq 'create') {
            last if $obj_found;
        } else {
            last if !$obj_found;
        }
        if ( $count++ < $retry ) {
            Tharn::snooze($polling_interval);
        } else {
            my $operation;
            if ($method eq 'create') {
                $operation = 'replicated';
            } else {
                $operation = 'deleted';
            }
            my $msg = "After retrying $retry number of times "
                . "(polling_interval=$polling_interval), $object object (primary"
                . ' keys: '
                . _dump_one(\%orig_primary_keys)
                . ") is not $operation in the metrocluster partner";
            $Log->exit() if $may_exit;
            
            NACL::C::Exceptions::MCCVerificationFailed->throw(
                $msg,
                primary_keys      => \%orig_primary_keys,
                command_interface => $command_interface
            );
        }
    }

    $Log->exit() if $may_exit;
}

# This is a method which helps move the common mcc opts from
# the source hash to destination hash
sub _move_common_mcc_opts {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my @mcc_opts_to_move
        = qw ( nacltask_to_check_mcc nacltask_mcc_verify_retry_count
        polling_interval_mcc_check remote_interface );

    $pkg_or_obj->_hash_move_defined( @args, move => \@mcc_opts_to_move, );
    $Log->exit() if $may_exit;
}

sub _verify_update_primary_keys {
    $Log->enter() if $may_enter;
    my ( $self, @args ) = @_;
    my $pkg = ref $self || $self;

    my %opts = validate_with(
        params => \@args,
        spec   => {
            nacltask_cleanup_manager =>
                { type => OBJECT, 'isa' => 'NACL::Cleanup' },
            new_opts   => { type => HASHREF },
            orig_opts  => { type => HASHREF },
            to_cleanup => { type => SCALAR },

        },
        allow_extra => 1,
    );

    my $primary_keys
        = $self->_get_primary_keys_w_or_wo_ci_from_pkg_or_obj( %opts,
        'want_ci' => 0 );
    $pkg->_process_options( 'options' => $primary_keys );

    if ( $opts{'to_cleanup'} ne "rename" ) {
        return $primary_keys;
    }

    my $cleanup_manager = $opts{'nacltask_cleanup_manager'};
    my $new_opts        = $opts{'new_opts'};
    my $orig_opts       = $opts{'orig_opts'};
    my $host_id         = $cleanup_manager->db()->_get_host_id(
        'name' => $opts{'command_interface'}->name() );
    my %existing_primary_keys = %{$primary_keys};
    my %primary_keys_to_update;
    while ( my ( $key, $val ) = each %{$new_opts} ) {
        my ($key_to_update)
            = grep { $primary_keys->{$_} eq $val } keys %{$primary_keys};
        $primary_keys_to_update{$key_to_update} = $val;
        $existing_primary_keys{$key_to_update}  = $orig_opts->{$key};
    }
    $cleanup_manager->_update_primary_keys(
        'primary_keys_to_update' => \%primary_keys_to_update,
        'existing_primary_keys'  => \%existing_primary_keys,
        'package'                => $pkg,
        'host_id'                => $host_id,
    );
    $Log->exit() if $may_exit;
    return $primary_keys;
}

# Calls the cleanup method with the arguments provided.

sub cleanup {
    my ( $self, @args ) = @_;
    $Log->enter() if $may_enter;
    my %opts = validate(
        @args,
        {   'command'   => { type => SCALAR },
            'arguments' => { type => HASHREF },
        },
    );
    my $command   = delete $opts{'command'};
    my $arguments = delete $opts{'arguments'};

    $self->$command( %{$arguments} );
    $Log->exit() if $may_exit;
}

=head2 call_cleanup_as_static_method

(Instance method) Calls the cleanup as static method instead of 
calling the method on the instance which cleanup framework creates.

Example:

  my $Vol_STask_Obj->call_cleanup_as_static_method('command' => "purge", 'argument' => $arrayref);

=over

=item Options

=item C<< command, SCALAR >>

(Mandatory) Command to be executed.

=item C<< arguments >>

(Mandatory, ARRAYREF) Arguements used to run the command.

=back

=cut

sub call_cleanup_as_static_method {
    $Log->enter() if $may_enter;
    my ( $self, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            'command'   => { type => SCALAR },
            'arguments' => { type => HASHREF },
        },
    );

    if ( !ref $self ) {
        my $me = ( caller(1) )[3];
        NATE::BaseException->throw( "'$me' is a instance method"
                . ' and should not be invoked with as static method' );
    }
    my $primary_keys_with_ci
        = $self->_get_primary_keys_w_or_wo_ci_from_pkg_or_obj( %opts,
        'want_ci' => 1 );
    my $command   = delete $opts{'command'};
    my $arguments = delete $opts{'arguments'};
    my $pkg       = ref($self);
    $pkg->$command( %{$primary_keys_with_ci}, %{$arguments} );
    $Log->exit() if $may_exit;
}

=head2 create_method_builder

(For STask developers only)

package NACL::STask::Foo;
use base qw(NACL::C::Foo NACL::STask::STask);
__PACKAGE__->create_method_builder();
 (or)
__PACKAGE__->create_method_builder(if_exists => 0); # Disable the 
                                    #nacltask_if_exists functionality


This method builds the create method and adds it to the current  
package namespace. The resulting method would take all the options of 
NACL::C::Foo->create method and the options listed 

=over

=item Options

=over

=item C<< if_exists = 0|1 >>
(Optional, defaults to 1)

Enable the nacltask_if_exists option.

=item C<< cleanup = 0|1 >>

(Optional, defaults to 1)

The method would support cleanup related options nacltask_to_cleanup and
nacltask_cleanup_manager.

=item C<< verify = 0|1 >>

(Optional, defaults to 1)

The method would support the nacltask_verify option .

=item C<< mcc_check => 0|1 >>

(Optional, defaults to 0)
Enables the options related to MCC config diff checking.

=item C<< additional_spec >>

(Optional, defaults to {})
The caller may specify additional options with the "additional_spec" option.

=item C<< c_method_name  => $c_method_name >>

(Optional, defaults to create)
Component method to be called for create call on the STask lib.

=item C<< c_exception_name  => $c_exception_name >>

(Optional, defaults to AlreadyExists)
The name of the exception class thrown by create method of the component lib

=item C<< stask_method_name  => $stask_method_name >>

(Optional, defaults to create)
Name of the method to be installed on the STask lib, defaults to create

=back

=back

=cut

sub create_method_builder {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    
    # Update The "Common_Create_Options" POD section when this list is updated.
    state $spec = {
        if_exists => { type => SCALAR, default => 1 },
        cleanup   => { type => SCALAR, default => 1 },
        verify    => { type => SCALAR, default => 1 },
        mcc_check => {type => SCALAR, default => 0},
        additional_spec => { type => HASHREF, default => {} },
        c_method_name => {type => SCALAR, default => 'create'},
        stask_method_name => {type => SCALAR, default => 'create'},
        c_exception_name => {type => SCALAR, default => 'AlreadyExists'},
    };
    my %opts = validate_with(
        params => \@args,
        spec   => $spec,
    );
    my $stask_method = delete $opts{stask_method_name};
    no strict 'refs';
    *{"${pkg}::${stask_method}"} = $pkg->_build_create(%opts);
    use strict 'refs';
    $Log->exit() if $may_exit;
}

#
#Private function which  returns a coderef for create method
#

sub _build_create {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg =~ /.*::(.*$)/;
    my $c_exception = delete $opts{c_exception_name};
    my $c_method_name = delete $opts{c_method_name};
    my $exception_name = "NACL::C::Exceptions::".$1."::".$c_exception;
    my ($key, $value);
    my $str = <<'HEREDOC';
    sub {
        $Log->enter() if $may_enter;
        my ( $pkg, @args ) = @_;
        my %opts = $pkg->_common_validate_with(
            params          => \@args,
            additional_spec => {
                _was_created    => { type => SCALARREF, optional => 1 },
HEREDOC
    if ( $opts{if_exists} ) {
        $str .= <<'HEREDOC'; 
                nacltask_if_exists => $pkg->_if_exists_validate_spec(),
HEREDOC
    }
    if ( $opts{cleanup} ) {
        $str .= <<'HEREDOC'; 
                $pkg->_cleanup_validate_spec(),
HEREDOC
    }
    if ( $opts{verify} ) {
        $str .= <<'HEREDOC'; 
                nacltask_verify => { type => BOOLEAN,   default  => 0 },
HEREDOC
    }
    
    if ($opts{mcc_check}) {
        $str .= <<'HEREDOC'; 
                $pkg->_mcc_validate_spec(),
HEREDOC

    }
    if ($opts{additional_spec}) {
        $str .= <<"HEREDOC";
HEREDOC
        while ( ( $key, $value ) = each %{$opts{additional_spec}} ) {
           if (ref($value) eq "HASH") {
               $str .= "$key => {";
               while( my( $inner_key, $inner_value ) = each %{$value} ) {
                   $str .= "$inner_key => $inner_value,";
               }  
               $str .= "},\n";
           } else {
               $str .= "$key => $value,\n";
           }
        }
    }
    $str .= <<'HEREDOC'; 
            },
            allow_extra => 1
        );
HEREDOC

    if ( $opts{cleanup} ) {
        $str .= <<'HEREDOC'; 
         my (%opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup);
         $pkg->_move_common_cleanup_opts(
             source              => \%opts,
             target              => \%opts_for_cleanup,
         );
HEREDOC
    }
    if ( $opts{if_exists} ) {
        $str .= <<'HEREDOC'; 
         my $if_exists = delete $opts{nacltask_if_exists};
HEREDOC
    }
    
    if ($opts{mcc_check}) {
        $str .= <<'HEREDOC';
        my %mcc_opts;
        $pkg->_move_common_mcc_opts(
            source => \%opts,
            target => \%mcc_opts,
        );
HEREDOC
    }
    
    $str .= <<'HEREDOC'; 
         my $was_created = delete $opts{_was_created};
         $$was_created = 0;

         my $self;
HEREDOC

    if ( $opts{verify} ) {
        $str .= <<'HEREDOC'; 
         my $verify = delete $opts{nacltask_verify};
HEREDOC
    }
    if ( $opts{if_exists} ) {
        $str .= <<"HEREDOC";
        require $exception_name;
HEREDOC
        $str .= <<'HEREDOC';
CREATE: {
        use warnings qw(exiting);
        try {
HEREDOC
    }
        $str .= <<"HEREDOC";
            \$self         = \$pkg->C_SUPER::$c_method_name(\%opts);
            \$\$was_created = 1;
HEREDOC
    if ( $c_method_name eq 'create' && $pkg eq 'NACL::STask::VserverCifs' ) {
        $str .= <<'HEREDOC';
            my $command_interface = $pkg->command_interface() // $opts{command_interface};
	    my $is_smb1_enb_present = $command_interface->apiset()->is_option_present(
					api=>'vserver_cifs_options_modify',
					field =>'smb1-enabled' );
	    if ( $is_smb1_enb_present == 1 ) {
                NACL::C::VserverCifsOptions->modify(
                    command_interface => $command_interface,
                    'vserver' => $opts{vserver},
                    'smb1-enabled' => 'true',
                );
	    }
HEREDOC
    }
    if ( $opts{verify} ) {
        $str .= <<'HEREDOC';
         $pkg->taskverify_create(%opts) if $verify;
HEREDOC
    }
    if ( $opts{if_exists} ) {
        $str .= <<"HEREDOC";
            } catch $exception_name with {
HEREDOC

        $str .= <<'HEREDOC'; 
            my $exception = shift;
            $self = $pkg->_element_exists_handler(
                create_opts        => \%opts,
                nacltask_if_exists => $if_exists,
                exception          => $exception
            );
            if ( !$self ) {
                no warnings qw(exiting);
                redo CREATE;
            }
        };
       }
HEREDOC
    }
 
    if ( $opts{cleanup} ) {
        $str .= <<'HEREDOC'; 
        $pkg->_copy_common_opts_for_cleanup(
           'source'              => {%opts, %opts_for_cleanup},
           'target'              => \%opts_for_register,
           'nacltask_to_cleanup' => \$nacltask_to_cleanup,
           'to_cleanup'          => 'purge'
       );
       $pkg->_register_for_cleanup(%opts_for_register) if ( $nacltask_to_cleanup && $$was_created );
HEREDOC
    }

    if ($opts{mcc_check}) {    
        $str .= <<'HEREDOC';
        $self->_check_in_metrocluster_node(
            command_interface => $opts{command_interface},
            method_used       => 'create',
            %mcc_opts,
        );
HEREDOC
    }

    $str .= <<'HEREDOC'; 
       $Log->exit() if $may_exit;
       return $self;
   }
HEREDOC
    my $ref = eval "$str";

    if ( $@ ) {
       NATE::BaseException->throw("\n Unable to eval: \n".$str
           . "\n Errors: ".$@);
    }
    $Log->exit() if $may_exit;
    return $ref;
}


=head2 purge_method_builder


package NACL::STask::Foo;
use base qw(NACL::C::Foo NACL::STask::STask);
__PACKAGE__->purge_method_builder();
 (or)
__PACKAGE__->purge_method_builder(if_purged => 0); # Disable the
                                    #nacltask_if_purged functionality


This method builds the purge method and adds the method definition to the current package
namespace. The resulting method would take all the options of NACL::C::Foo->delete
and the options listed

=over

=item Options

=over

=item C<< if_purged = 0|1 >>

(Optional, defaults to 1)

The method would support the nacltask_if_purged option

=item C<< verify = 0|1 >>

(Optional, defaults to 1)

The method would support nacltask_verify option

=item C<< mcc_check => 0|1 >>

(Optional, defaults to 0)
Enables the options related to MCC config diff checking.

=item C<< c_method_name  => $c_method_name >>

(Optional, defaults to create)
Component method to be called for purge call on the STask lib.

=item C<< c_exception_name  => $c_exception_name >>

(Optional, defaults to DoesNotExist)
The name of the exception class thrown by purge method on the STask lib 

=item C<< stask_method_name  => $stask_method_name >>

(Optional, defaults to purge)
Name of the method to be installed on the STask lib, defaults to create


=back

=back

=back

=cut

sub purge_method_builder {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    
    # Update The "Common_Purge_Options" POD section when this list is updated.
    state $spec = {
        if_purged => { type => SCALAR, default => 1 },
        verify   => { type => SCALAR, default => 1 },
        mcc_check => {type => SCALAR, default => 0},
        c_method_name => {type => SCALAR, default => 'delete'},
        stask_method_name => {type => SCALAR, default => 'purge'},
        c_exception_name => {type => SCALAR, default => 'DoesNotExist'},
    };
    my %opts = validate_with(
        params => \@args,
        spec   => $spec,
    );
    my $stask_method = delete $opts{stask_method_name};
    no strict 'refs';
    *{"${pkg}::${stask_method}"} = $pkg->_build_purge(%opts);
    use strict 'refs';
    $Log->exit() if $may_exit;
}


#
#Private function which  returns a coderef for purge method
#
sub _build_purge {
    my ($pkg, %opts) = @_;
    $Log->enter() if $may_enter;
    $pkg =~ /.*::(.*$)/;
    my $c_exception = delete $opts{c_exception_name};
    my $c_method_name = delete $opts{c_method_name};
    my $exception_name = "NACL::C::Exceptions::".$1."::".$c_exception;
    my $str = <<'HEREDOC';
    sub {
        $Log->enter() if $may_enter;
        my ( $pkg_or_obj, @args ) = @_;
        my %opts = $pkg_or_obj->_common_validate_with(
            params          => \@args,
            additional_spec => {
HEREDOC
    if ( $opts{verify} ) {
       $str .= <<'HEREDOC';
                nacltask_verify    => { type => BOOLEAN, default => 0 },
HEREDOC
    }
    if ( $opts{if_purged} ) {
       $str .= <<'HEREDOC';
                nacltask_if_purged => $pkg_or_obj->_if_action_completed_already_validate_spec(),
HEREDOC
    }
    
    if ($opts{mcc_check}) {
        $str .= <<'HEREDOC';
                $pkg_or_obj->_mcc_validate_spec(),
HEREDOC
    }
    
    $str .= <<'HEREDOC';
                _was_deleted       => { type => SCALARREF, optional => 1 },
            },
            allow_extra => 1,
        );
        my $was_deleted  = delete $opts{_was_deleted};
        $$was_deleted = 0;
        my $pkg = ref $pkg_or_obj || $pkg_or_obj;
HEREDOC
   if ( $opts{verify} ) {
        $str .= <<'HEREDOC';
        my $verify = delete $opts{nacltask_verify};
HEREDOC
   }
   
   if ($opts{mcc_check}) {
        $str .= <<'HEREDOC';
        my %mcc_opts;
        $pkg_or_obj->_move_common_mcc_opts(
            source => \%opts,
            target => \%mcc_opts,
        );
HEREDOC
   }

    if ( $opts{if_purged} ) {
       $str .= <<'HEREDOC';
       my $if_purged = delete $opts{nacltask_if_purged};
HEREDOC
       $str .= <<"HEREDOC";
       require $exception_name;
HEREDOC
       $str .= <<'HEREDOC';
       try {
HEREDOC
    }
    $str .= <<"HEREDOC";
            \$pkg->C_SUPER::$c_method_name(\%opts);
            \$\$was_deleted = 1;
HEREDOC
    if ( $opts{verify} ) {
        $str .= <<'HEREDOC';
            $pkg_or_obj->_generic_purge_verify(%opts) if $verify;
HEREDOC
    }

    if ($opts{mcc_check}) {
        $str .= <<'HEREDOC';
            my %primary_keys;
            $pkg_or_obj->_copy_primary_keys(
                source => \%opts,
                target => \%primary_keys
            );
            $pkg_or_obj->_check_in_metrocluster_node(
                command_interface => $opts{command_interface},
                method_used       => 'purge',
                %primary_keys,
                %mcc_opts,
            );
HEREDOC
    }

    if ( $opts{if_purged} ) {
        $str .= <<"HEREDOC";
        }
        catch $exception_name with {
HEREDOC
        $str .= <<'HEREDOC';
            my $exception = shift;
            $pkg_or_obj->_handle_if_purged(
                exception          => $exception,
                nacltask_if_purged => $if_purged
            );
        };
HEREDOC
    }

    $str .= <<'HEREDOC';
        $Log->exit() if $may_exit;
}
HEREDOC
    my $ref = eval "$str";
    if ( $@ ) {
       NATE::BaseException->throw("\n Unable to eval: \n".$str
           . "\n Errors: ".$@);
    }
    $Log->exit() if $may_exit;
    return $ref;
}

=head2 modify_method_builder


package NACL::STask::Foo;
use base qw(NACL::C::Foo NACL::STask::STask);
__PACKAGE__->modify_method_builder();
 (or)
__PACKAGE__->modify_method_builder(if_verify=> 0); # Disable the
                                    #nacltask_verify functionality


This method builds the modify method and adds the method definition to the current package
namespace. The resulting method would take all the options of NACL::C::Foo->modify
and the options listed

=over

=item Options

=over

=item C<< verify = 0|1 >>

(Optional, defaults to 1)

The method would support nacltask_verify option

=item C<< cleanup = 0|1 >>

(Optional, defaults to 1)

The method would support cleanup related options nacltask_to_cleanup and
nacltask_cleanup_manager.


=back

=back

=cut


sub modify_method_builder {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    # Update The "Common_Modify_Options" POD section when this list is updated.
    state $spec = {
        verify   => { type => SCALAR, default => 1 },
        if_modified => { type => SCALAR, default => 0 },
        cleanup => {type => SCALAR, default => 1},
    };
    my %opts = validate_with(
        params => \@args,
        spec   => $spec,
    );
    no strict 'refs';
    *{"${pkg}::modify"} = $pkg->_build_modify(%opts);
    use strict 'refs';
    $Log->exit() if $may_exit;
}



sub _build_modify {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg =~ /.*::(.*$)/;
    my $exception_name = "NACL::C::Exceptions::".$1."::AlreadyModified";
    my $str = <<'HEREDOC';
sub {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $ret;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
HEREDOC
    if ( $opts{cleanup} ) {
        $str .= <<'HEREDOC';
        $pkg_or_obj->_cleanup_validate_spec(),
HEREDOC
    }
    if ( $opts{verify} ) {
        $str .= <<'HEREDOC';
        nacltask_verify => { type => BOOLEAN, default => 0 },
HEREDOC
    }
    if ( $opts{if_modified} ) {
        $str .= <<'HEREDOC';
        _was_modified => { type => SCALARREF, optional => 1 },
        $pkg_or_obj->_if_action_completed_already_validate_spec(
            name => 'nacltask_if_modified'),
HEREDOC
    }
    $str .= <<'HEREDOC';
        },
        allow_extra     => 1,
    );
HEREDOC

    if ( $opts{cleanup} ) {
        $str .= <<'HEREDOC';
        my ($opts_for_register, $register_for_cleanup, %opts_for_cleanup);
        $pkg_or_obj->_move_common_cleanup_opts(
            source              => \%opts,
            target              => \%opts_for_cleanup,
            nacltask_to_cleanup => \$register_for_cleanup
        );

        if ($register_for_cleanup) {
            $opts_for_register
                = $pkg_or_obj->_common_modify_for_cleanup(%opts_for_cleanup,
                %opts);
        } 
HEREDOC
    }
    if ( $opts{verify} ) {
        $str .= <<'HEREDOC';
        my $verify = delete $opts{'nacltask_verify'};
HEREDOC
    }
    if ( $opts{if_modified} ) {
        $str .= <<"HEREDOC";
            require $exception_name;
HEREDOC
        $str .= <<'HEREDOC';
        my $if_modified = delete $opts{nacltask_if_modified};
        my $was_modified = delete $opts{_was_modified};
        $$was_modified = 0;
    try {

HEREDOC
    }
    $str .= <<'HEREDOC';
        $ret = $pkg_or_obj->C_SUPER::modify(%opts);
HEREDOC
    
    if ( $opts{if_modified} ) {
        $str .= <<'HEREDOC';
            $$was_modified = 1;
        } catch 
HEREDOC
        $str .= " $exception_name with {";
        $str .= <<'HEREDOC';
            my $e = shift;
            if ( $if_modified !~ /pass/i ) {
                $Log->exit() if $may_exit;
                $e->throw();
            }
        };
HEREDOC
    }
    if ( $opts{verify} ) {
        $str .= <<'HEREDOC';
        if ($verify) {
            $pkg_or_obj->taskverify_modify(%opts);
        }
HEREDOC
    }

    if ( $opts{cleanup} ) {
        $str .= <<'HEREDOC';
        $pkg_or_obj->_register_for_cleanup(%{$opts_for_register})
        if ($register_for_cleanup);
HEREDOC
    }
   $str .= <<'HEREDOC';
    $Log->exit() if $may_exit;
    return $ret;
}
HEREDOC

    my $ref = eval "$str";
    if ( $@ ) {
       NATE::BaseException->throw("\n Unable to eval: \n".$str
           . "\n Errors: ".$@);
    }
    $Log->exit() if $may_exit;
    return $ref;
}


1;

# Undocumented since I don't really want anyone trying to directly access
# C_SUPER. This is only to allow SUPER-like syntax for C_SUPER calls.

package C_SUPER;
use NATE::Log qw(log_global);
$Log = log_global();
$may_enter = $Log->may_enter();
$may_exit  = $Log->may_exit();

our $AUTOLOAD;

sub AUTOLOAD {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    if ( !$pkg_or_obj->isa('NACL::STask::STask') ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("C_SUPER can only be invoked for STasks");
    }
    my $method = $AUTOLOAD;
    $method =~ s/.*:://;
    my $super = $pkg_or_obj->get_C_package_name() . '::' . $method;
    $Log->exit() if $may_exit;
    return $pkg_or_obj->$super(@_);
}

1;
