# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SnaplockComplianceClock Task Module
## @author msmeena@netapp.com, dl-snaplock-qa
## @status shared
## @pod here

package NACL::STask::SnaplockComplianceClock;

use strict;
use warnings;
use POSIX qw/ceil/;
use NACL::ComponentUtils qw(Dumper);
use Readonly;
use Scalar::Util qw(blessed) ;

use base qw(NACL::C::SnaplockComplianceClock NACL::STask::STask);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate validate_with validate_pos :types);
use NATE::Exceptions::Argument qw(:try);
use NACL::Exceptions::VerifyFailure             ();
use NACL::C::Exceptions::SnaplockComplianceClock::AlreadyInitialized;

=head1 NAME

NACL::STask::SnaplockComplianceClock

=head1 DESCRIPTION

C<NACL::STask::SnaplockComplianceClock> provides a number of well-defined but
potentially complex or multi-step methods related to SnaplockComplianceClock in
ONTAP.

It builds on top of, and is a derived class of
C<NACL::C::SnaplockComplianceClock>, and so it also provides methods that are
more in the scope of individual SnaplockComplianceClock-related commands. See
C<NACL::C::SnaplockComplianceClock> for details. This also means that a
C<NACL::STask::SnaplockComplianceClock> object may generally be used
in place of a component object.

=head1 ATTRIBUTES

=head2 command_interface

(Required) As C<NACL::C::SnaplockComplianceClock>. A component object that represents
the host to which to send commands.

=over

=head1 METHODS

=head2 initialize

  NACL::C::SnaplockComplianceClock->initialize(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to initializes node compliance clock.
Uses CMode/7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< 'node' => $string >>

(Optional) Node possible value(s) are, <nodename>,local
Applicable for CMode/7Mode CLI

=item C<< nacltask_if_initialized => 'die' | 'pass' >>

(Defaults to 'die')
What to do if the snaplock compliance-clock is already initialized. The
default is 'die', which means that the exception
(C<NACL::C::Exceptions::SnaplockComplianceClock::AlreadyInitialized>) gets
propagated. A value of 'pass' results in the exception being suppressed.

Note that 7Mode does not fail if the clock is already initialized, so this
option is not applicable for 7Mode.

=item C<< nacltask_verify => 0|1 >>

(Optional, defaults to 0)
If 1, verify if the snaplock clock is initialized.

If it is not, then it throws a
L<NACL::Exceptions::VerifyFailure|lib-NACL-Exceptions-VerifyFailure-pm>
exception.

All other options supported by
L<NACL::C::SnaplockComplianceClock|lib-NACL-C-SnaplockComplianceClock-pm/initialize >
are supported by NACL::STask::SnaplockComplianceClock->initialize as well.

=back

=item Exceptions

=over

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when Snaplock Clock is not initialized

=back

=back

=cut

sub initialize {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            nacltask_verify         => { type => SCALAR, default => 0 },
            nacltask_if_initialized => $pkg->_if_action_completed_already_validate_spec(name => 'nacltask_if_initialized'),
        },
        allow_extra => 1
    );

    my $if_initialized = delete $opts{nacltask_if_initialized};
    my $verify         = delete $opts{nacltask_verify};

    try {
        $pkg->SUPER::initialize(%opts);
    } catch NACL::C::Exceptions::SnaplockComplianceClock::AlreadyInitialized with {
        my $exception = shift;
        $Log->debug("Clock initialize failed with error ".$exception->text());
        if ($if_initialized =~ /die/) {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
    };

    # Verify Snaplock Clock initialize
    if ($verify) {
        $pkg->taskverify_initialize(%opts);
    }
    $Log->exit() if $may_exit;
} ## end sub initialize

sub taskverify_initialize {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts       = $pkg->_common_validate_with(
        params      => \@_,
        allow_extra => 1
    );
    my %common_opts;
    $pkg->_copy_common_component_params(
        source => \%opts,
        target => \%common_opts
    );
    my $clock = NACL::CS::SnaplockComplianceClock->fetch(
            command_interface => $opts{command_interface},
            filter            => { 'node' => $opts{'node'} },
            allow_empty       => 1,
            %common_opts,
    );
    if ( !$clock ) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::VerifyFailure->throw("Snaplock Clock not initialized");
    }
    $Log->exit() if $may_exit;
}
1;
