# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SnapshotPolicy Task Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::SnapshotPolicy;

use strict;
use warnings;
use base qw(NACL::C::SnapshotPolicy NACL::STask::STask);

use NATE::Log qw( log_global );
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR BOOLEAN SCALARREF);
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use NACL::C::Exceptions::VolumeSnapshotPolicy::AlreadyExists qw(:try);
use NACL::Exceptions::VerifyFailure                     ();
use NACL::C::Exceptions::VolumeSnapshotPolicy::DoesNotExist  ();
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ResponseException ();

=head1 NAME

NACL::STask::SnapshotPolicy

=head1 DESCRIPTION

C<NACL::STask::SnapshotPolicy> provides a number of well-defined but potentially
complex or multi-step methods related to SnapshotPolicy in ONTAP.

It is created on top of C<NACL::C::SnapshotPolicy> component
Since it is derived class of C<NACL::C::SnapshotPolicy>,
we can use all the methods of C<NACL::C::SnapshotPolicy>
from the object of this task.

=head1 ATTRIBUTES

=head2 command_interface

(Required) A component object that represents the node to which to send commands
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

=head2 enabled

(Required) Specifies whether the Snapshot policy is enabled.

=head2 policy

(Required) The Snapshot policy name.

=head2 schedule1

(Required) Name of the first schedule associated with the Snapshot policy.

=head2 count1

(Required) The maximum number of Snapshot copies that can be taken by the first schedule.

=head2 vserver

(Required) Vserver on which the Snapshot policy is to be created.

=head1 METHODS

=head2 create

   my $SnapshotPolicy = NACL::STask::SnapshotPolicy->create(
                                'command_interface'  => $command_interface,
                                'vserver'            => $vserver,
                                'policy'             => $policy,
                                'schedule1'          => $schedule1,
                                'count1'             => $count1,
                                'enabled'            => $enabled,
                                'nacltask_if_exists' => $action, # default die
                                %other_options,
                    );

(Class Method) This method is used to create a snapshot policy.
If policy already exists, it will perform the action based on
"nacltask_if_exists" parameter. Default behavior would be "die".
It returns the Task object for the created schedule.

Uses CMode CLI/ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< 'vserver' => $vserver >>

(Required) Vserver on which the Snapshot policy is to be created.

=item C<< 'policy' => $policy >>

(Required) The Snapshot policy name.

=item C<< 'schedule1' => $schedule1 >>

(Required) Name of the first schedule associated with the Snapshot policy.

=item C<< 'count1' => $count1 >>

(Required)  The maximum number of Snapshot copies that can be taken by the first schedule.

=item C<< 'enabled' => $enabled >>

(Required) Specifies whether the Snapshot policy is enabled.

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "nacltask_if_exists => $action" >>

(Optional) What to do if the policy to be created already exists.
If $action is "die", then fail with an exception. If action is
"purge", then purge the policy(see the "purge" method, below)
before creating a new one. If $action is "reuse", It will return
the object of existing policy.

=item C<< "_was_created => \$scalar" >>

(Optional) When this option is provided with a reference to a scalar
variable, the variable gets filled in with a boolean value
describing whether the policy was available (value will be 0; this
scenario is possible when if_exists => "reuse") or whether the
policy was created (value will be 1). This is necessary to
determine whether the policy needs to be cleaned up later.

my $was_created;

my $SnapshotPolicy = NACL::STask::SnapshotPolicy->create(
                        command_interface       => $command_interface,
                        vserver                 => $vserver,
                        policy                  => $policy,
                        schedule1               => $schedule1,
                        count1                  => $count1,
                        enabled                 => $enabled,
                        $nacltask_if_exists     => 'reuse',
                        _was_created            => \$was_created,
                        nacltask_to_cleanup     => $boolean,
                        %other_opts
                    );

# Operate on $SnapshotPolicy here
# ...
# Now determine whether to clean up the policy, since we're not sure
# whether we reused an existing policy or created a new one
if ($was_created) {
   # New snapshot policy was created. Clean it up.
   $SnapshotPolicy->purge();
}

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, defaults to 0(no to cleanup))
Flag indicating if this operation is to be cleaned up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to use for registering.
Default : Will use the default cleanup manager.

command_interface, apiset_must, apiset_should, mode, etc.
All of the other various options to
L<< NACL::C::SnapshotPolicy->create|lib-NACL-C-SnapshotPolicy-pm/create >>

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VolumeSnapshotPolicy::AlreadyExists>

This type of exception is thrown when an attempt is made to create an element that already exists.

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the created policy.

=back

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            nacltask_if_exists => $pkg->_if_exists_validate_spec(),
            nacltask_verify    => { type => BOOLEAN, default => 0 },
            _was_created       => { type => SCALARREF, optional => 1 },
            $pkg->_cleanup_validate_spec(),
        },
        allow_extra     => 1
    );

    # Transform %opts from the options we received into the options to
    # pass to the base class method.
    my ( $self, %common_opts, %nacltask_opts );
    $pkg->_copy_common_component_params_with_ci(
        source => \%opts,
        target => \%common_opts
    );

    my ( %opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup );
    $pkg->_move_common_cleanup_opts(
        source => \%opts,
        target => \%opts_for_cleanup,
    );

    $pkg->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_opts
    );

    my $was_created = delete $opts{_was_created};
    $$was_created = 0;
    my $if_exists = $nacltask_opts{'nacltask_if_exists'};

CREATE: {
        use warnings;
        try {
            $self         = $pkg->SUPER::create(%opts);
            $$was_created = 1;
        }
        catch NACL::C::Exceptions::VolumeSnapshotPolicy::AlreadyExists with {
            my $exception = shift;
            $Log->comment("The Snapshot Policy already Exists!");
            $self = $pkg->_element_exists_handler(
                create_opts        => \%opts,
                nacltask_if_exists => $if_exists,
                exception          => $exception,
            );
            if ( !$self ) {
                no warnings qw(exiting);
                redo CREATE;
            }
        };
    }
    # Register this resource with the Cleanup Manager for cleanup
    $pkg->_copy_common_opts_for_cleanup(
        'source'              => {%opts, %opts_for_cleanup},
        'target'              => \%opts_for_register,
        'nacltask_to_cleanup' => \$nacltask_to_cleanup,
        'to_cleanup'          => 'purge'
    );

    $pkg->_register_for_cleanup(%opts_for_register)
        if ($nacltask_to_cleanup && $$was_created);

    $Log->exit() if $may_exit;
    return $self;
} ## end sub create

=head2 purge

    $SnapshotPolicy->purge();
            (or)
    NACL::STask::SnapshotPolicy->purge(
            'command_interface' => $command_interface,
            'vserver'           => $vserver,
            'policy'            => $policy,
            'nacltask_verify'   => 1,
            'nacltask_if_purged => $action,
    );

(Class or instance method)
This method is used to delete the snapshot policy.

Uses a CMode CLI/ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
A component object that represents the host which to send commands.
See NACL::C::Component::command_interface

=item C<< 'vserver' => $vserver >>

(Required) Vserver on which the Snapshot policy is to be created.

=item C<< 'policy' => $policy >>

(Required) The Snapshot policy name.

=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the deletion did happen
successfully.

=item C<< _was_deleted => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the policy
did not exist (value will be 0; this scenario is possible when
nacltask_if_purged => "pass") or whether the policy was deleted
(value will be 1).

=item C<< nacltask_if_purged => $nacltask_if_purged >>

(Optional)
If 'pass' (default), It will pass if the policy is already deleted.
If 'fail', It will fail if the policy is already deleted.

All of the other various options to
L<< NACL::C::SnapshotPolicy->delete|lib-NACL-C-SnapshotPolicy-pm/delete >>

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VolumeSnapshotPolicy::DoesNotExist>

This type of exception is thrown when an attempt is made to delete an element that does not exists.

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the deleted snapshot policy.

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $additional_spec = {
        nacltask_verify    => { type => BOOLEAN, default => 0 },
        nacltask_if_purged => $pkg_or_obj->_if_action_completed_already_validate_spec(),
        _was_deleted       => { type => SCALARREF, optional => 1 },
    };

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => $additional_spec,
        allow_extra => 1,
    );

    my %nacltask_opts;
    $pkg_or_obj->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_opts,
    );

    my $if_purged   = $nacltask_opts{nacltask_if_purged};

    my %common_opts;
    $pkg_or_obj->_copy_common_component_params_with_ci(
        source => \%opts,
        target => \%common_opts
    );

    my $was_deleted = delete $opts{_was_deleted};
    $$was_deleted = 0;

    use warnings qw(exiting);
    try {
        $pkg_or_obj->SUPER::delete(%common_opts, %opts );
        $$was_deleted = 1;
    }
    catch NACL::C::Exceptions::VolumeSnapshotPolicy::DoesNotExist with {
        my $exception = shift;
        $Log->comment("Entry Does Not Exists!");
        if ( $if_purged !~ /pass/i ) {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
    };

    if ( $opts{nacltask_verify} ) {
        $pkg_or_obj->_generic_purge_verify(%opts);
    }

    $Log->exit() if $may_exit;
} ## end sub purge

1;
