#
# Copyright (c) 2013-2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Spm Task Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::Spm;

use strict;
use warnings;

use base qw(NACL::C::Spm NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate validate_with :types);
use NACL::ComponentUtils;
use NACL::STask::Cluster;
use NACL::C::Node;
use NACL::APISet;
use NACL::APISet::Exceptions::ResponseException;
use NACL::APISet::Exceptions::CommandFailedException;
use NATE::Exceptions::Argument;
use NATE::BaseException qw(:try);
use NACL::Transit;
use DateTime::Format::Strptime;
use NACL::CS::ClusterDate;
use NACL::CS::JobInitstate;
use NACL::GeneralUtils qw(nacl_method_retry);

=pod

=head1 NAME

NACL::STask::Spm

=head1 DESCRIPTION

C<NACL::STask::Spm> handles necessary processing around the CMode spmctl command.

Spm (Service Process Manager) on the node is a nanny process which
will detect when managed processes have been killed, and will attempt
to restart it (if that process has not been restarted more than 10 times
in the last hour).   The Spm STask can be used to stop processes, start
or restart (stop/start) processes, kill a process, retrieve status for a process
and manage or unmanage a process.

Processes can be started or stopped independently of management.  Stopped processes
will not be automatically restarted by SPM, but killed processes that ARE managed will
be automatically restarted up until the restarts/hour limit (currently 10).  After
the restart limit is reached, SPM will go into a slow-restart-running state.  Stopping
a process that is in this state will clear the restart/hour limit.

Interface with the systemshell spmctl command is accomplished via APISet, but state
is primarily retrieved using the C<NACL::CS::Spm> class.

This task will allow you to start, stop, restart or kill processes, as
well as perform the necessary checks to assure that a given process is
in the expected state.

All methods must be called only as instance methods (except for "new" naturally).
This is because subsequent methods require access to constructor data.

=head1 CLEANUP METHODS

Cleanup can be registered for the following methods

Cleanup methods are,

     Spm Method          Cleanup Method
     ----------------------------------
     start               stop
     manage              unmanage

=head1 ATTRIBUTES

=head2 command_interface

(Required) As C<NACL::C::Node>.  A component object that
represents the node to which to send commands.

=head2 name

(Required) The name of the SPM process.

=head2 node

(Optional) The node on which the SPM process is running.  If not
specified, will default to the node of the command_interface.

=head1 METHODS

=head2 new

  my $spm = NACL::STask::Spm->new( command_interface => $Ci, name => "mgwd" );

(Class method) This method will instantiate a Spm object.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item   C<<name>>

(Required) process name. Process to be acted upon.

=back

=back

=over

=item Exceptions

C<<NATE::BaseException>>

=back

=cut

use Class::MethodMaker [
    scalar => [
        {   '-default_ctor' =>
                sub { NACL::Transit->new( name => $_[0]->node() ) },
            '-type' => 'NACL::Transit',
        },
        '_transit_obj',
    ],
    scalar => [
        {   '-default_ctor' => sub { $_[0]->refresh_apiset() },
            '-type'         => 'NACL::APISet',
        },
        'diag_apiset',
    ],
];

# Initialize the object
sub init {
    $Log->enter() if $may_enter;
    my ( $self, %opts ) = @_;

    my $ci = $self->command_interface();
    $opts{node} ||= $ci->node();
    $self->node( $opts{node} );

    # Call the base class's init for the basic parameter checking
    $self->SUPER::init(%opts);
    $Log->exit() if $may_exit;
} ## end sub init

=head2 refresh_apiset

    # After the command_interface for the Spm has been refreshed,
    # you will need to refresh the apiset that the Spm task uses.

    $vldb = NACL::STask::Spm->new( command_interface => $node, name => 'vldb' );
    $node->reboot();
    $node->refresh_command_interface();
    $vldb->refresh_apiset();
    $vldb->state();

(Instance method) Refresh the apiset that the Spm task uses after
its command_interface was refreshed.  Another reason this routine
might be needed is if you have a promoted build on your filers.  In
the case of a promoted build, we need to login via the console as
the diag user.  If your script were to logout of that diag user
between Spm calls, you would need to call refresh_apiset() to
log back in as the diag user.

=over

=item Exceptions

C<<NACL::APISet::Exceptions::MissingRequiredParamException>>

C<<NACL::APISet::Exceptions::CommandFailedException>>

C<<NATE::Exceptions::Argument>>

C<<NATE::BaseException>>

=back

=cut

sub refresh_apiset {
    $Log->enter() if $may_enter;
    my $self = shift;

    my $ci     = $self->command_interface();
    my $node   = $self->node();
    my $verMgr = $ci->get_version_manager();

    my $apiset;

    my $connid = "systemshell";

    # Logging in as diag, we can even kill mgwd and not be affected.
    # If a promoted build, we have to login as diag via the console,
    # otherwise, we can just ssh login as diag.
    if ( $verMgr->is_promoted() ) {
        $connid = "console";

        my $curr_state
            = $self->_transit_obj()->get_state( probe_timeout => [ 0, 1 ] );

        if ( $curr_state ne 'SYSTEMSHELL' ) {
            $self->_login_to_console_as_diag();
        }
    } ## end if ( $verMgr->is_promoted...)

    if(($ci->isa('NACL::C::Node') || $ci->isa('NACL::STask::Node') )
        && defined $node &&  $node ne $ci->node()){
      $apiset =  $ci->get_systemshell_apiset(node => $node, connid => $connid);
    }else{
      $apiset = $ci->apiset(
        interface => 'CLI',
        set       => 'Systemshell',
        connid    => $connid
      );
    }

# setting diag_apiset explicitly as refresh_apiset can be called outside of the
# the default_ctor
    $self->{diag_apiset} = $apiset;
    $Log->exit() if $may_exit;
    return $self->{diag_apiset};

} ## end sub refresh_apiset

sub _login_to_console_as_diag {
    $Log->enter() if $may_enter;
    my $self = shift;

    my $trans   = $self->_transit_obj();
    my $connrec = $self->_transit_obj()->{conn};
    my $ci      = $self->command_interface();

    my $password = $ci->hostrec()->{conn}->{console}->{prompt_password}
        || $ci->hostrec()->default_password_gx();

    $trans->change_state( to => 'USERNAME' );
    $trans->change_state( to => 'PASSWORD', username => 'diag' );

    # Regex is messy, but it is a negative lookbehind which means
    # match "login:" so long as it is not preceded by "Last ".
    $connrec->execute(
        command     => $password,
        timeout     => 120,
        must_match  => "Warning:",
        match_table => [
            '(?<!Last )login:' => 'diag',
            'Password:'        => $password
        ]
    );

    $Log->exit() if $may_exit;
} ## end sub _login_to_console_as_diag

=head2 start

    # when starting multiple processes, might want to
    # defer cluster ring check until last process is started
    $vldb->start( check_cluster_rings => 0 );
    $vifmgr->start( check_cluster_rings => 1 );

    # If you want to specify a non-default command line for a process
    $mgwd->start( 'cmd-line' => '/sbin/mgwd -z' );

(Instance method) Start the SPM process.  The "start" method will
call is_up with the "check_cluster_rings" option, to verify that
the started process comes up.  See documentation for L<< is_up|lib-NACL-STask-Spm-pm/is_up >>.
It is recommended to add some delay between consecutive start call

=over

=item Options

=over

=item   C<<cmd-line>>

(optional)  Command string.  Optional IF we want a non-standard command-line.

=item   C<<check_cluster_rings>>

(optional) 0|1. Default 1.  Perform a cluster ring check.

=item   C<<do_is_up_check>>

(optional) 0|1. Default 1.  Verify that the process is back up after starting it.

=item   C<<is_up_options>>

(optional) hashref. Default {}.  Hashref of options to pass to the is_up() routine.

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup)
Flag indicating if this operation needs to be registered for clean up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=back

=back

=over

=item Exceptions

C<<NACL::APISet::Exceptions::MissingRequiredParamException>>

C<<NACL::APISet::Exceptions::CommandFailedException>>

C<<NATE::Exceptions::Argument>>

C<<NATE::BaseException>>

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'cmd-line'            => { type => SCALAR,  default => "" },
            'check_cluster_rings' => { type => SCALAR,  default => 1 },
            'do_is_up_check'      => { type => SCALAR,  default => 1 },
            'is_up_options'       => { type => HASHREF, default => {} },
            $self->_cleanup_validate_spec(),
        },
        allow_extra => 1
    );

    my (%opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup );
    $self->_move_common_cleanup_opts(
        source  => \%opts,
        target  => \%opts_for_cleanup,
    );

    my $ci     = $self->command_interface();
    my $name   = $self->name();
    my $handle = $self->get_handle();

    my %spmctl_args = (
        handle  => $handle,
        verbose => 1,
        exec    => 1,
    );

    if ( $opts{'cmd-line'} ) {
        $spmctl_args{'cmd-line'} = $opts{'cmd-line'};
    }

    $self->diag_apiset()->spmctl(%spmctl_args);

    if ( $opts{'do_is_up_check'} ) {
        # Perform post-start checks
        my %is_up_opts = ( check_cluster_rings => $opts{check_cluster_rings},
                           %{$opts{'is_up_options'}} );
        $self->is_up( %is_up_opts );
    }

    delete ($self->{'diag_apiset'}) if($opts_for_cleanup{nacltask_to_cleanup});
    # Register this resource with the cleanup manager for cleanup.
    $self->_copy_common_opts_for_cleanup(
        'source'             => {%opts, %opts_for_cleanup},
        'target'             => \%opts_for_register,
        'nacltask_to_cleanup'=> \$nacltask_to_cleanup,
        'to_cleanup'         => 'stop',
    );
    $self->_register_for_cleanup(%opts_for_register) if ($nacltask_to_cleanup);

    $Log->exit() if $may_exit;
} ## end sub start

=head2 stop

    $spm->stop();

(Instance method) Stop the SPM process.
It is recommended to add some delay between consecutive stop call

=over

=item Exceptions

C<<NACL::APISet::Exceptions::MissingRequiredParamException>>

C<<NACL::APISet::Exceptions::CommandFailedException>>

C<<NATE::Exceptions::Argument>>

C<<NATE::BaseException>>

=item C<< nacltask_retry_tries_count => $tries_count >>
A retry mechanism to get valid process id within specified number of attempts

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my $self = shift;

    $self->_verify_invocation( style => 'instance_only' );

    my %opts = validate_with(
        params => \@_,
        spec   => {
            nacltask_retry_tries_count => { type => SCALAR, default => 3 },
            nacltask_retry_sleep_time  => { type => SCALAR,  default => 20 },
            nacltask_retry_num_times_code_ran =>
                            { type => SCALARREF, optional => 1 },
        },
	allow_extra => 1
    );

    my $retry_count = delete $opts{'nacltask_retry_tries_count'};

    my $name   = $self->name();
    my $pid    = $self->get_pid(nacltask_retry_tries_count => $retry_count);
    my $handle = $self->get_handle();

    my %spmctl_args = (
        handle  => $handle,
        verbose => 1,
        stop    => 1,
    );

    my %retry_opts;
    my $pkg = ref($self);

    $pkg->_copy_retry_params(
        source => \%opts,
        target => \%retry_opts
    );

    my ($response,$raw_output);

    try{

       my $prompts_answers = [
               "Do you want to login to the systemshell as the 'diag' user.*" => "yes",
               "Password.*"  => "netapp1!"
       ];
       $response = $self->diag_apiset()->spmctl(%spmctl_args, 'connectrec-match_table' => $prompts_answers);

       $raw_output = $response->get_raw_output();
    } catch NACL::APISet::Exceptions::CommandFailedException with {
       my $ex = shift;
       if($raw_output =~ /RPC:\s+Timed\s+out/) {
           nacl_method_retry(
               code => sub { $self->diag_apiset()->spmctl(%spmctl_args);},
               exceptions => 'NACL::APISet::Exceptions::CommandFailedException',
               %retry_opts,
           );
       } else {
           $ex->throw();
       }
    };

    $self->is_down( pid => $pid );

    $Log->exit() if $may_exit;
} ## end sub stop

=head2 restart

    # when restarting multiple processes, might want to
    # defer cluster ring check until last
    $vldb->restart( check_cluster_rings => 0 );
    $vifmgr->restart( check_cluster_rings => 1 );

    # If you want to specify a non-default command line for a process
    $mgwd->restart( 'cmd-line' => '/sbin/mgwd -z' );

(Instance method) Stop then start the SPM process.  The process will be
restarted with the default command-line associated with the process, unless
a 'cmd-line' override is specified.  When the process is started,
is_up will be called with the "check_cluster_rings" option, to verify that
the started process comes up.  See documentation for L<< is_up|lib-NACL-STask-Spm-pm/is_up >>.
It is recommended to add some delay between consecutive restart call

=over

=item Options

=over

=item   C<<check_cluster_rings>>

(optional) 0|1. Default 1.  Perform a cluster ring check.

=back

=item Exceptions

C<<NACL::APISet::Exceptions::MissingRequiredParamException>>

C<<NACL::APISet::Exceptions::CommandFailedException>>

C<<NATE::Exceptions::Argument>>

C<<NATE::BaseException>>

=back

=cut

sub restart {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    my %opts = validate_with(
        params      => \@_,
        spec        => { 'cmd-line' => { type => SCALAR, default => "" }, },
        allow_extra => 1
    );

    my %stop_opts = %opts;
    delete $stop_opts{'cmd-line'};
    my $name = $self->name();

    $self->stop(%stop_opts);
    $self->start(%opts);

    $Log->exit() if $may_exit;
} ## end sub restart

=head2 kill

    $spm->kill();

(Instance method) Kill the SPM process.
Note: the Spm on the node is a nanny process which
will detect when the process has been killed, and will attempt
to restart it (if that process is in the managed processes list
and if that process has not been restarted more than 10 times
in the last hour).  So we do not check to see if the process
has been killed, because Spm might have restarted it in the meantime.

=over

=item Options

=over

=item   C<<pid>>

(optional) Default undef.  Integer process id of process.

=item   C<<check_restarts>>

(optional) 0|1  Default 1.  Verify that the number of restarts per
hour will not be exceeded.  If the check_restarts flag is 1, and
this kill would put the restarts count over the top, the process will
be stopped and then restarted and then killed.  If the flag is 0,
the process will be killed regardless of the restart count.  A tester
might want to supply check_restarts=>0 if they are verifying the
slow-restart.

=back

=item Exceptions

C<<NACL::APISet::Exceptions::MissingRequiredParamException>>

C<<NACL::APISet::Exceptions::CommandFailedException>>

C<<NATE::Exceptions::Argument>>

C<<NATE::BaseException>>

This type of exception is thrown when there is a failure to kill a particular process.

=item C<< nacltask_retry_tries_count => $tries_count >>
A retry mechanism to get valid process id within specified number of attempts

=back

=cut

sub kill {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    my %opts = validate_with(
        params => \@_,
        spec   => {
            pid            => { type => SCALAR, default => undef },
            check_restarts => { type => SCALAR, default => 1 },
	    nacltask_retry_tries_count => { type => SCALAR, default => 3 },
        },
        allow_extra => 1
    );

    my $retry_count = delete $opts{'nacltask_retry_tries_count'};

    my $pid = $opts{pid};
    if ( !defined $pid ) {
        $pid = $self->get_pid(nacltask_retry_tries_count => $retry_count);
    }

    my $name = $self->name();
    my $node = $self->node();

    # If process is not managed, you can't query ngsh for restarts and
    # regardless, they don't apply
    if ( $opts{check_restarts} && $self->is_managed() ) {
        my $restarts_in_last_hour = $self->restarts_in_last_hour();
        my $max_restarts          = $self->get_state()->max_restarts();
        if ( ( $restarts_in_last_hour + 1 ) > $max_restarts ) {

            # Restart to reset the restart counters
            $self->restart();
        }
    } ## end if ( $opts{check_restarts...})

    try {
        $self->diag_apiset()
            ->execute_raw_command( command => "sudo kill " . $pid );
    }
    catch NATE::BaseException with {
        my $ex = shift;
        $Log->exit() if $may_exit;
        my $error_text = "Failed attempting to kill process $name, PID $pid\n"
            . $ex->text();
        $ex->set_text($error_text);
        $ex->throw;
    };

    $Log->exit() if $may_exit;
} ## end sub kill

=head2 manage

    $spm->manage();

(Instance method) Add a running process to the SPM "managed" process list.
Managed processes are ones which Spm will watch over and restart when they
die.

=over

=item     Exceptions

C<<NACL::APISet::Exceptions::MissingRequiredParamException>>

C<<NACL::APISet::Exceptions::CommandFailedException>>

C<<NATE::Exceptions::Argument>>

This type of exception is thrown when process is not managed.

C<<NATE::BaseException>>

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup)
Flag indicating if this operation needs to be registered for clean up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=item C<< nacltask_retry_tries_count => $tries_count >>
A retry mechanism to get valid process id within specified number of attempts

=back

=cut

sub manage {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    my %opts = validate_with(
        params => \@_,
        spec   => {
            $self->_cleanup_validate_spec(),
        },
        allow_extra => 1
    );
    my (%opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup );
    $self->_move_common_cleanup_opts(
        source  => \%opts,
        target  => \%opts_for_cleanup,
    );

    my %opts1 = validate_with(
        params => \@_,
        spec   => {
            nacltask_retry_tries_count => { type => SCALAR, default => 3 },
        },
	allow_extra => 1
    );

    my $retry_count = delete $opts1{'nacltask_retry_tries_count'};

    my $handle = $self->get_handle();
    my $pid    = $self->get_pid(nacltask_retry_tries_count => $retry_count);

    my %spmctl_args = (
        handle  => $handle,
        verbose => 1,
        add     => 1,
        pid_t    => $pid,
    );

    $self->diag_apiset()->spmctl(%spmctl_args);

    if ( !$self->is_managed() ) {
        my $name = $self->name();
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Process $name should be, but is not, managed");
    } ## end if ( !$self->is_managed...)

    delete ($self->{'diag_apiset'}) if($opts_for_cleanup{nacltask_to_cleanup});
    # Register this resource with the cleanup manager for cleanup.
    $self->_copy_common_opts_for_cleanup(
        'source'             => {%opts, %opts_for_cleanup},
        'target'             => \%opts_for_register,
        'nacltask_to_cleanup'=> \$nacltask_to_cleanup,
        'to_cleanup'         => 'unmanage',
    );
    $self->_register_for_cleanup(%opts_for_register) if ($nacltask_to_cleanup);

    $Log->exit() if $may_exit;
} ## end sub manage

=head2 unmanage

    $spm->unmanage();

(Instance method) Remove a running process from the SPM "managed" list.
Managed processes are ones which Spm will watch over and restart when they
die.

=over

=item     Exceptions

C<<NACL::APISet::Exceptions::MissingRequiredParamException>>

C<<NACL::APISet::Exceptions::CommandFailedException>>

C<<NATE::Exceptions::Argument>>

C<<NATE::BaseException>>

This type of exception is thrown when process is managed.

=back

=cut

sub unmanage {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    my $handle = $self->get_handle();

    my %spmctl_args = (
        handle  => $handle,
        verbose => 1,
        delete  => 1,
    );

    $self->diag_apiset()->spmctl(%spmctl_args);

    if ( $self->is_managed() ) {
        my $name = $self->name();
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Process $name should not be, but is, managed");
    } ## end if ( $self->is_managed...)

    $Log->exit() if $may_exit;
} ## end sub unmanage

=head2 is_up

    # when killing a process, you might want to check
    # that the process came back up
    $vldb->kill();
    $vldb->is_up();

    # when killing mgwd, the CommandInterface
    # that was used to create the Spm object
    # will have it's APISet connections refreshed
    # in is_up(), but it is up to you to refresh
    # other CommandInterface objects (Nodes, Vservers, etc...)
    # that are associated with the same node.
    $mgwd->kill();
    $mgwd->is_up();

    # notifyd doesn't own a ring, and shouldn't affect them
    # so you may choose to skip the cluster ring check
    $notifyd->kill();
    $notifyd->is_up( check_cluster_rings => 0 );


(Instance method) Verify that the SPM process is "up".  This means
that at a minimum, verify that spmctl shows a state of Running
or Slow_restart_running.  Further, if the process being checked is
"mgwd", the CommandInterface object used to create the Spm object
will have the connections associated with it's APISet objects refreshed.
Finally, if the check_cluster_rings option is given,
NACL::STask::Cluster->check_rings() will be called over a period of
up to 60 seconds to verify that all of the cluster rings are operational.

=over

=item Options

=over

=item   C<<check_cluster_rings>>

(optional) 0|1. Default 1.  Perform a cluster ring check.

=item   C<<check_job_manager>>

(optional) 0|1. Default 0 for all processes except mgwd.  Verify that the Job Manager
has initialized.

=item   C<<attempts>>

(optional) <integer>. Default 3, number of times to check if process is up.

=item   C<<interval>>

(optional) <integer>. Default 5, number of seconds to sleep between process up checks.

=back

=back

=over

=item Exceptions

C<<NACL::APISet::Exceptions::MissingRequiredParamException>>

C<<NACL::APISet::Exceptions::CommandFailedException>>

C<<NATE::Exceptions::Argument>>

C<<NATE::BaseException>>

This type of exception is thrown when there is a failure to refresh command_interface or
when there is a failure to retrieve NACL::CS::JobInitstate object or
when there is a failure to call NACL::STask::Cluster->check_rings() even after given number of retries or
when job manager does not come UP in given number of retries.

=back

=cut

sub is_up {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'check_cluster_rings' => { type => SCALAR, default => 1 },
            'check_job_manager'   => { type => SCALAR, default => undef },
            attempts              => { type => SCALAR, default => 5 },
            interval              => { type => SCALAR, default => 5 },
        },
        allow_extra => 1
    );

    my $name = $self->name();

    # Basic sanity check to see that spmctl thinks the process is running
    $self->_check_running();
    my $check_cluster_rings = $opts{check_cluster_rings};
    my $check_job_manager   = $opts{check_job_manager};

    # A little bit of a catch-22 here.  We need the command_interface to
    # be refreshed before we call check_cluster_rings(), but if mgmt
    # ring isn't back up yet, we might not be able to reconnect via ssh
    # and checking the cluster rings is generally how we determine if
    # a cluster ring process (like vifmgr, vldb, etc...) is completely up
    # and healthy
    if ( $name eq "mgwd" ) {

    #as per requirement of burt1097430 there is no need to enable check_job_manager option by default.
    #    if ( !defined $check_job_manager ) {
    #        $check_job_manager = 1;
    #    }

        my $ci      = $self->command_interface();
        my $retries = $opts{attempts};
        while ($retries) {
            $retries--;
            my $success = 0;

            # Many times with retries, you want to attempt once right
            # away, but almost always I am seeing connect failures for
            # at least 4 seconds, so it seems wise to do our sleeps
            # BEFORE the attempt
            Tharn::sleep( $opts{interval} );
            try {
                $ci->refresh_command_interface( no_version_change => 1 );
                $success = 1;
            }
            catch NATE::BaseException with {
                my $ex = shift;
                if ( $retries == 0 ) {
                    $Log->exit() if $may_exit;
                    my $error_text = "Could not refresh command_interface\n"
                        . $ex->text();
                    $ex->set_text($error_text);
                    $ex->throw;
                } ## end if ( $retries == 0 )
            };

            if ($success) {
                last;
            }
        } ## end while ($retries)
    } ## end if ( $name eq "mgwd" )

    if ($check_cluster_rings) {
        $self->check_cluster_rings();
    }

    if ($check_job_manager) {
        $self->check_job_manager(
            attempts => $opts{attempts},
            interval => $opts{interval}
        );
    } ## end if ($check_job_manager)
    $Log->exit() if $may_exit;
} ## end sub is_up

=head2 is_down

    $spm->is_down( pid => $pid );

(Instance method) Method to verify that a process has stopped running.
Many processes stop very quickly, but some, like "named" can take a few
seconds, so this method repeatedly checks until they are truely stopped.

=over

=item Options

=over

=item   C<<pid>>

(required) integer.  Process ID of process to check.

=item   C<<attempts>>

(optional) integer.  Number of times to attempt check (default 3).

=item   C<<interval>>

(optional) integer.  Number of seconds to sleep between checks (default 5).

=back

=back

=over

=item     Exceptions

C<<NACL::APISet::Exceptions::MissingRequiredParamException>>

C<<NACL::APISet::Exceptions::CommandFailedException>>

C<<NATE::Exceptions::Argument>>

This type of exception is thrown when PID of a particular process is not numeric or when a particular process refuses to die.

C<<NATE::BaseException>>

=back

=cut

sub is_down {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    my %opts = validate_with(
        params => \@_,
        spec   => {
            pid      => { type => SCALAR, optional => 0 },
            interval => { type => SCALAR, default  => 5 },
            attempts => { type => SCALAR, default  => 3 }
        },
        allow_extra => 1
    );

    my $name   = $self->name();
    my $handle = $self->get_handle();
    my $node   = $self->command_interface()->node();

    if ( $opts{pid} !~ /^(\d+)$/ ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Process $name on Node $node, PID $opts{pid} is not numeric");
    }

    # Generally processes die pretty quickly, but some, like
    # named, put up a little bit of a fight :)
    my $retries = $opts{attempts};
    my $response;
    while ($retries) {
        $retries--;
        $response
            = $self->diag_apiset()
            ->execute_raw_command(
            command => "sudo ps -p " . $opts{pid} . " | grep " . $handle );
        if ($response) {
            Tharn::sleep( $opts{interval} );
        } else {
            last;
        }
    } ## end while ($retries)

    if ($response) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Process $name on Node $node, PID $opts{pid} refuses to die");
    }
    $Log->exit() if $may_exit;
} ## end sub is_down

=head2 get_handle

    $spm->get_handle();

(Instance method) Return the handle name which is compatible with spmctl.
Currently the name httpdmgr is used by ngsh's spm command, but spmctl
knows this process by the handle 'httpd'

=over

=item     Exceptions

C<<NATE::Exceptions::Argument>>

=back

=cut

sub get_handle {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );
    my $handle = $self->name();

    # Issue related to BURT 515000
    # spmctl uses handle, but ngsh "spm" uses httpdmgr
    if ( $handle eq 'httpdmgr' ) {
        $handle = 'httpd';
    }
    $Log->exit() if $may_exit;
    return $handle;
} ## end sub get_handle

=head2 is_managed

    $spm->is_managed();

(Instance method) Determine whether or not a process is managed by Spm.
Returns 0|1.
Managed processes are ones which Spm will watch over and restart when they
die.

=over

=item     Exceptions

C<<NATE::Exceptions::Argument>>

=back

=cut

sub is_managed {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    $Log->exit() if $may_exit;
    return ( defined( $self->_get_spmctl_entry() ) );
} ## end sub is_managed

=head2 get_state

    $spm->get_state();

(Instance method) Return NACL::CS::Spm object if managed and running or
undef if not.
Managed processes are ones which Spm will watch over and restart when they
die.

=over

=item     Exceptions

C<<NACL::APISet::Exceptions::CommandFailedException>>

C<<NATE::Exceptions::Argument>>

C<<NATE::BaseException>>

=back

=cut

sub get_state {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    my $cs;

    try {
        $cs = $self->state();
    }
    catch NATE::BaseException with {

        # for "normal" components, not being able to retrieve state
        # IS exceptional, but for unmanaged Spms, it is expected.
        # Plus if the exception-throwing behavior is what you really
        # want, you can still always call $obj->state() directly
    };

    $Log->exit() if $may_exit;
    return $cs;
} ## end sub get_state

=head2 get_pid

    $spm->get_pid();

(Instance method) Retrieve the process ID from the Spm Process.  For
managed processes, get this from ngsh via the NACL::CS::Spm object, but
if unmanaged, retrieve the PID from the freebsd "ps" command.
Managed processes are ones which Spm will watch over and restart when they
die.

=over

=item     Exceptions

C<<NACL::APISet::Exceptions::CommandFailedException>>

C<<NATE::Exceptions::Argument>>

C<<NATE::BaseException>>

This type of exception is thrown when there is a failure to retrieve PID for a particular process.

=item C<< nacltask_retry_tries_count => $tries_count >>
A retry mechanism to get valid process id within specified number of attempts

=back

=cut

sub get_pid {
    $Log->enter() if $may_enter;
    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            nacltask_retry_tries_count => { type => SCALAR, default => 3 },
        },
	allow_extra => 1
    );

    $self->_verify_invocation( style => 'instance_only' );
    my $retry_count = delete $opts{'nacltask_retry_tries_count'};

    my $pid;
    my $name   = $self->name();
    my $handle = $self->get_handle();

    while ( (!$pid || ($pid !~ /^[0-9]+$/)) && ($retry_count ne 0) ){
	if ( $self->is_managed() ) {
		# If process is unmanaged, we'll need to call ps to get pid
		my $cs = $self->get_state();

		if ($cs) {
			$pid = $cs->pid();
		}
	}

	# If pid undef or non-numeric (spm was returning - for some processes like ntpd)
	if ( !$pid || ($pid !~ /^\d+$/) ) {
		my $cmd = "ps -auwx | grep /" . $handle . " | grep -v grep";
		my $response
			= $self->diag_apiset()->execute_raw_command( command => $cmd );
		if ($response) {
			if ( $response =~ /\S+\s+(\d+)\s+/ ) {
				$pid = $1;
			} else {
				my $ci   = $self->command_interface();
				my $node = $ci->node();
				$Log->exit() if $may_exit;
				NATE::BaseException->throw(
					"Cannot retrieve PID for process $name on Node $node");
			} ## end else [ if ( $response =~ /\S+\s+(\d+)\s+/)]
		} ## end if ($response)
	} ## end else [ if ($cs) ]
    $retry_count--;
    } ## end while
    $Log->exit() if $may_exit;
    return $pid;
} ## end sub get_pid

=head2 get_known_processes

    $spm->get_known_processes();

(Instance method) Retrieve the list of processes that spmctl knows about.

=over

=item     Exceptions

C<<NACL::APISet::Exceptions::CommandFailedException>>

C<<NATE::Exceptions::Argument>>

C<<NATE::BaseException>>

This type of exception is thrown when there is a failure to retrieve the list of known processes.

=back

=cut

sub get_known_processes {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    my @processes;

    my $cmd = "sudo spmctl --help";
    my $response
            = $self->diag_apiset()->execute_raw_command( command => $cmd );

    if ($response) {
        my @lines = split("\n", $response );
        my $start = 0;
        foreach my $line (@lines) {
            if ( $line =~ /^Well known handles:/i ) {
                $start = 1;
                next;
            }

            if ( $start ) {
                if ( $line =~ /^\s+(\S+)\s+/ ) {
                    my $procname = $1;
                    push( @processes, $procname );
                }
            }
        }
    }

    if ( scalar(@processes) == 0 ) {
        $Log->exit() if $may_exit;
        my $node = $self->node();
        NATE::BaseException->throw(
            "Cannot retrieve known spmctl processes on Node $node");
    }

    $Log->exit() if $may_exit;
    return @processes;
} ## end sub get_known_processes

=head2 restarts_in_last_hour

    $spm->restarts_in_last_hour();

(Instance method) How many times has this process been restarted in the past hour?

=over

=item     Exceptions

C<<NATE::Exceptions::Argument>>

C<<NATE::BaseException>>

=back

=cut

sub restarts_in_last_hour {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    my $restarts_in_last_hour = 0;
    my $seconds_in_an_hour    = 3600;

    my $ci   = $self->command_interface();
    my $node = $self->node();
    my $name = $self->name();
    my $date = NACL::CS::ClusterDate->fetch(
        command_interface => $ci,
        filter            => { node => $node }
    );
    my $date_str    = $date->date();
    my $date_parser = new DateTime::Format::Strptime(
        pattern  => '%m/%d/%Y %H:%M:%S',
        on_error => sub { NATE::BaseException->throw( $_[1] ) },
    );

    my $now_dt    = $date_parser->parse_datetime($date_str);
    my $now_epoch = $now_dt->epoch();

    my $restarts = $self->get_state()->restarts();

    # BURT 514078 covers this, should be an array
    if ( !ref($restarts) ) {
        $restarts = [$restarts];
    }

    foreach my $restart (@$restarts) {
        my $dt    = $date_parser->parse_datetime($restart);
        my $epoch = $dt->epoch();
        if ( ( $now_epoch - $epoch ) <= $seconds_in_an_hour ) {
            $restarts_in_last_hour++;
        }
    } ## end foreach my $restart (@$restarts)

    $Log->exit() if $may_exit;
    return $restarts_in_last_hour;
} ## end sub restarts_in_last_hour

=head2 check_cluster_rings

    $spm->check_cluster_rings();

(Instance method) Verify the cluster rings are in quorum.

=over

=item Options

=over

=item   C<<attempts>>

(optional) integer.  Number of times to attempt check (default 30).

=item   C<<interval>>

(optional) integer.  Number of seconds to sleep between checks (default 10).

=back

=back

=over

=item     Exceptions

C<<NATE::Exceptions::Argument>>

C<<NATE::BaseException>>

=back

=cut

sub check_cluster_rings {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    my %opts = validate_with(
        params => \@_,
        spec   => {
            interval => { type => SCALAR, default => 10 },
            attempts => { type => SCALAR, default => 30 }
        },
        allow_extra => 1
    );

    my $interval  = $opts{'interval'};
    my $attempts  = $opts{'attempts'};
    my $exception = undef;

    my $ci = $self->command_interface();

    foreach ( my $i = 1; $i <= $attempts; $i++ ) {
        try {
            NACL::STask::Cluster->check_rings( command_interface => $ci );
        }
        catch NATE::BaseException with {
            $exception = shift;
        };
        # An exception was thrown and there are more iterations
        if ( ref($exception) && ( $i < $attempts ) ) {
            Tharn::snooze($interval);
            $exception = undef;
        } else {
            last;
        }
    } ## end foreach ( my $i = 1; $i <= ...)
    if ($exception) {
        $Log->exit() if $may_exit;
        my $error_text
            = "Call to NACL::STask::Cluster->check_rings() failed after "
            . $opts{'attempts'}
            . " attempts with a "
            . $interval
            . " inter-attempt sleep\n"
            . $exception->text();
        $exception->set_text($error_text);
        $exception->throw;
    } ## end if ($exception)

    $Log->exit() if $may_exit;
} ## end sub check_cluster_rings

=head2 check_job_manager

    $spm->check_job_manager();

(Instance method) Verify that the Job Manager has been initialized.

=over

=item Options

=over

=item   C<<attempts>>

(optional) integer.  Number of times to attempt check (default 30).

=item   C<<interval>>

(optional) integer.  Number of seconds to sleep between checks (default 10).

=back

=back

=over

=item     Exceptions

C<<NATE::Exceptions::Argument>>

C<<NATE::BaseException>>

=back

=cut

sub check_job_manager {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    my %opts = validate_with(
        params => \@_,
        spec   => {
            interval => { type => SCALAR, default => 10 },
            attempts => { type => SCALAR, default => 30 }
        },
    );

    my $interval = $opts{'interval'};
    my $attempts = $opts{'attempts'};
    my $ci       = $self->command_interface();

    my $success;
    while ($attempts) {
        $attempts--;
        $success = 0;
        try {
            my $jis = NACL::CS::JobInitstate->fetch(
                command_interface => $ci,
                filter            => { node => $ci->node() }
            );

            if ( $jis->initialized() =~ /true/ ) {
                $success = 1;
            } else {
                Tharn::sleep($interval);
            }
        } ## end try
        catch NATE::BaseException with {
            my $ex = shift;
            if ( $attempts == 0 ) {
                $Log->exit() if $may_exit;
                my $error_text
                    = "Could not retrieve NACL::CS::JobInitstate object after "
                    . $opts{'attempts'}
                    . " attempts with a "
                    . $interval
                    . " inter-attempt sleep\n"
                    . $ex->text();
                $ex->set_text($error_text);
                $ex->throw;
            } else {
                Tharn::sleep($interval);
            }

        };

        if ($success) {
            last;
        }
    } ## end while ($attempts)

    if ( ( !$success ) && ( $attempts == 0 ) ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "JobManager does not appear to be up in given number of retries after $opts{'attempts'}\n"
        );
    } ## end if ( ( !$success ) && ...)

    $Log->exit() if $may_exit;
} ## end sub check_job_manager

####    Private Helper Methods

# Retrieve the entry in the spmctl managed list which corresponds to
# this object.
#
#    $proc->_get_spmctl_entry();
#
sub _get_spmctl_entry {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    my $exception = undef;

    my $ci     = $self->command_interface();
    my $node   = $ci->node();
    my $name   = $self->name();

    # Issue related to BURT 515000
    # spmctl uses handle, but ngsh "spm" uses httpdmgr
    if ( $name eq 'httpd' ) {
        $name = 'httpdmgr';
    }

    # Using APISet because we want to be able to call
    # this method when we are starting mgwd, but before
    # the command_interface is refreshed.
    my $error    = 1;
    my $response = $self->diag_apiset()->spmctl();
    my $output   = $response->get_parsed_output();
    my $return;
    foreach my $entry (@$output) {
        my $entry_name = $entry->{Exec};

        if ( $entry_name =~ /^\S+\/([a-zA-Z_]+)\s*/ ) {
            $entry_name = $1;
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "$name Cannot parse spmctl entry name: " . $entry_name );
        }

        if ( $entry_name eq $name ) {
            $return = $entry;
            last;
        } else {
            $error = 0;
        }
    } ## end foreach my $entry (@$output)

    $Log->exit() if $may_exit;
    return $return;
} ## end sub _get_spmctl_entry

# Check to see if process is running.
#
#    $proc->_check_running();
#
sub _check_running {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    my %opts = validate_with(
        params => \@_,
        spec   => {
            interval => { type => SCALAR, default => 5 },
            attempts => { type => SCALAR, default => 3 },
	    nacltask_retry_tries_count => { type => SCALAR, default => 3 },
        },
        allow_extra => 1
    );

    my $retry_count = delete $opts{'nacltask_retry_tries_count'};
    my $interval  = $opts{'interval'};
    my $attempts  = $opts{'attempts'};
    my $exception = undef;

    my $ci   = $self->command_interface();
    my $node = $ci->node();
    my $name = $self->name();

    foreach ( my $i = 1; $i <= $attempts; $i++ ) {
        my $error = 1;
        try {
            my $entry = $self->_get_spmctl_entry();
            if ($entry) {
                my $entry_name = $entry->{Exec};
                if ( $entry_name =~ /^\S+\/([a-zA-Z]+)\s*/ ) {
                    $entry_name = $1;
                } else {
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
                        "$name Cannot parse spmctl entry name: "
                            . $entry->{Exec} );
                } ## end else [ if ( $entry_name =~ /^\S+\/([a-zA-Z]+)\s*/)]

                if ( $entry->{State} !~ /running/i ) {
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
                        "$name state expected: /running/i, actual: "
                            . $entry->{State} );
                } else {
                    $error = 0;
                }
            } ## end if ($entry)
            if ($error) {

                # If we can't find reference to this process in spmctl, but
                # we CAN get a PID, then the process is unmanaged.
                if ( !defined( $self->get_pid(nacltask_retry_tries_count => $retry_count) ) ) {
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
                        "No state found for process $name");
                }
            } ## end if ($error)
        } ## end try
        catch NATE::BaseException with {
            $exception = shift;
        };
        # An error, but we still have more attempts
        if ( $exception && ( $i < $attempts ) ) {
            Tharn::snooze($interval);
            $exception = 0;
        } else {
            last;
        }
    } ## end foreach ( my $i = 1; $i <= ...)
    if ($exception) {
        $Log->exit() if $may_exit;
        my $error_text
            = "Process $name on Node $node "
            . "is not reporting state =~ /running/i "
            . "via spmctl\n$exception->text()";
        $exception->set_text($error_text);
        $exception->throw;
    } ## end if ($exception)

    $Log->exit() if $may_exit;
} ## end sub _check_running

# Originally the routine that checked the cluster rings was
# called _check_cluster_rings which
# suggests that this is a private routine, not to be
# called outside of this library.  There is no reason
# it has to be, but I didn't want to break anyone's scripts
# So I made the underscore name call the non-underscore routine
sub _check_cluster_rings {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    $Log->exit() if $may_exit;
    return $self->check_cluster_rings(@_);
} ## end sub _check_cluster_rings

1;
