#
#   Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Interface Task Module for Switch
## @author nayankumar.patel@netapp.com
## @status shared
## @pod here

package NACL::STask::Switch::Interface;

use strict;
use warnings;

use base qw(NACL::C::Switch::Interface NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR BOOLEAN);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::ComponentUtils qw(Dumper);

# Some methods for Interface and PortChannel are implemented in _Interface.pm
use NACL::STask::Switch::_Interface qw(:all);

=head1 NAME

NACL::STask::Switch::Interface

=head1 DESCRIPTION

C<NACL::STask::Switch::Interface> provide methods to configure interface on
switch. It provided methods to add/delete interface to/from vlan or 
port-channel.

It is created on top of C<NACL::C::Switch::Interface> component. Since it is 
derived class of C<NACL::C::Switch::Interface>, we can use all the methods of 
C<NACL::C::Switch::Interface> from the object of this task.

=head1 CLEANUP METHODS

Cleanup can be registered for the following methods

Cleanup methods are,
    Interface Method               Cleanup Method
    ------------------------------------------------------
    add_to_port_channel           remove_from_port_channel

=head1 ATTRIBUTES

=head2 command_interface

(Required) A component object that represents the host to which to send commands.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

=head2 interface_type

(Required) As C<NACL::C::Switch::Interface>. Type of interface on switch.
For Nexus, valid type is C<< Ethernet >>.
For IOS, valid type are C<< GigabitEthernet >> or C<< TenGigabitEthernet >>.

=head2 interface_num

(Required) As C<NACL::C::Switch::Interface>. interface number.

Example: 1/1

=head1 METHODS

=head2 status

my $status = $interface->status();

my $status = NACL::STask::Switch::Interface->status(
    command_interface   => $command_interface,
    'interface-type'    => 'Ethernet',
    'interface-num'     => '1/1',
);

my $status = $interface_object->status(
    nacltask_retry => 60
    nacltask_polling_interval => 5
);

my $status = $interface_object->status(
    nacltask_retry              => 60,
    nacltask_polling_interval   => 5
    nacltask_return_if_up       => 1
);

(Class or Instance Method) provide link status of the interface. it will check 
different link status(up, connected, down, etc) and return either UP or DOWN. 
C<nacltask_retry> option will allow user to check the status of the interface 
for certain period of time before task return the status of the interface. 
task will keep checking the status based on the value provided in option 
C<nacltask_retry> and than once the retry timer expire it will return the 
actual value of interface status. but if user want to return the status 
immediately if interface is UP, it can do that by setting the option 
C<nacltask_return_if_up>.

=over

=item Options

=back

=over

=item C<< nacltask_retry => $seconds >>

(Optional) this option allows user to probe the status of the interface for 
certain period of time and than return the status of the interface. value of 
this option is in seconds. default valule is 1, it means, task will get the 
status of the interface once, and return.

=item C<< nacltask_polling_interval => $seconds >>

(Optional) this option allows user to provide polling interval, if 
C<nacltask_retry> is more than 0. it will poll the status of the interface
based on the value provided in this option. default to 1 seconds.

=item C<< nacltask_return_if_up => $boolean >>

(Optional) this option, if set to 1, will enfoce the task to return the status 
of interface if the status is UP. task will avoid value specified in option
C<retry> and return the status.

=back

=over

=item Return
    return HASH Reference which contain following two value:
    1. status: it could be C<up> or C<down>
    2. actual_status: it will be the actual status of the interface.

    Ex:
    $VAR1 = {
                'status' => 'up',
                'actual_status' => 'connected'
           };

=back

=cut

sub status {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            nacltask_retry            => { type => SCALAR, default => 1 },
            nacltask_polling_interval => { type => SCALAR, default => 1 },
            nacltask_return_if_up     => { type => SCALAR, default => 1 },
        },
        allow_extra => 1
    );

    my $pkg                 = ref($pkg_or_obj) || $pkg_or_obj;
    my $CS_pkg              = $pkg->get_CS_package_name();
    my $command_interface   = delete $opts{command_interface};
    my %return_hash = ();
    my ( @state_obj, $status );

    $Log->debug(
        sub {
            "Options to status:\n" . Dumper(\%opts);
        }
    );

    my $end_time = time() + $opts{nacltask_retry};
    
    do {
        @state_obj = $CS_pkg->fetch(
            command_interface => $command_interface,
            filter            => {
                'interface-num'  => $opts{'interface-num'},
                'interface-type' => $opts{'interface-type'},
            },
            requested_fields => [qw(status)]
        );

        $status = $state_obj[0]->status();
        $return_hash{actual_status} = $status;
        $Log->debug("Status is $status");

        if ( $status =~ /^(connected|up)$/i ) {
            $return_hash{status} = 'up';
        } else {
            $return_hash{status} = 'down';
        }
        
        if ( $opts{nacltask_return_if_up} && $return_hash{status} =~ /up/i ) {
            # return status immediately
            $end_time = 0;   
        } else {
            # snooze for polling interval
            Tharn::snooze $opts{nacltask_polling_interval};
        }
    } while ( ( time() < $end_time ) );

    $Log->exit() if $may_exit;
    return \%return_hash;
}    #end of sub status

=head2 add_to_port_channel

NACL::STask::Switch::Interface->add_to_port_channel(
    command_interface   => $command_interface,
    'interface-type'    => 'Ethernet',
    'interface-num'     => '1/1',
    'port-channel-group'=> $port-channel,
    channel-mode        => $mode,
    speed               => 'auto',
    duplex              => 'full',
    status              => 'up',
    mtu                 => 9216,
    fc-send             => 'on',
    fc-receive          => 'on',
    switchport          => 1,
    nacltask_to_cleanup      => 1,          # default 0,
    nacltask_cleanup_manager => $CleanupObj,
);

$interface_object->add_to_port_channel(
    'port-channel-group'=> $port-channel,
    channel-mode        => $mode,
    speed               => 'auto',
    duplex              => 'full',
    status              => 'up',
    mtu                 => 9216,
    fc-send             => 'on',
    fc-receive          => 'on',
    switchport          => 1,
    nacltask_to_cleanup      => 1,          # default 0,
    nacltask_cleanup_manager => $CleanupObj,
);

(Class or Instance Method) This Task method will add interface to port channel. 
if user want to configure some extra configuration on interface before adding 
w
it to port-channel, these configurations can be specified using options 
(see Options section). not all options are supported on all switches. if user 
want to avoid any option, it can be done by setting that options as 'undef'. 
since some of the options has default value, user has to explicity assign value
as 'undef', so that task will avoid configuration for that option. if it fail 
to add interface to port-channel, it will raise exception.

=over

=item Options

=over

=item C<< port-channel-group => $port_channel_number >>

(Required) This is a port-channel number in which interface needs to be added.

=item C<< channel-mode => $mode >>

(Required) This is a mode of port-channel. it could be 'active', 'passive', or 
'on'.

=item C<< speed => $speed >>

(Optional) speed of the interface.

=item C<< duplex => $duplex >>

(Optional) duplex of the interface.

=item C<< status => $status >>

(Optional) status of interface. it can be 'up' or 'down'.
default to 'up'.

=item C<< mtu => $mtu >>

(Optional) mtu setting on interface. supported on Cisco IOS only. default to
'9216'.

=item C<< fc-send => $fc_send >>

(Optional) flowcontrol send configuration on interface. default to 'on'.

=item C<< fc-receive => $fc_receive >>

(Optional) flowcontrol receive configuration on interface. default to 'on'.

=item C<< switchport => $boolean >>

(Optional) switchport configuration on interface. default to '1'.

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup)
Flag indicating if this operation needs to be registered for clean up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=back 

=back

=cut

sub add_to_port_channel {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'port-channel-group'    => { type => SCALAR },
            'channel-mode'          => { type => SCALAR },
            speed                   => { type => SCALAR, optional => 1 },
            duplex                  => { type => SCALAR, optional => 1 },
            mtu                     => { type => BOOLEAN, default => 9216 },
            'fc-send'               => { type => BOOLEAN, default => 'on' },
            'fc-receive'            => { type => BOOLEAN, default => 'on' },
            switchport              => { type => BOOLEAN, default => 1 },
            status                  => { type => BOOLEAN, default => 'up' },
            $pkg_or_obj->_cleanup_validate_spec(),
        },
        allow_extra => 1
    );

    my $pkg      = ref($pkg_or_obj) || $pkg_or_obj;
    my $hosttype = $opts{command_interface}->hostrec()->hosttype();

    my (%opts_for_cleanup, $nacltask_to_cleanup);
    $pkg->_copy_common_opts_for_cleanup(
        'source'              => \%opts,
        'target'              => \%opts_for_cleanup,
        'nacltask_to_cleanup' => \$nacltask_to_cleanup,
        'to_cleanup'          => 'remove_from_port_channel'
    );

    $Log->debug(
        sub {
            "Options to add_to_port_channel:\n" . Dumper(\%opts);
        }
    );

    my %api_opts;
    $pkg->_hash_move(
        source => \%opts,
        target => \%api_opts,
        move   => [qw(speed duplex mtu fc-send fc-receive switchport status)],
    );

    $Log->debug("Host type is $hosttype");
    
    my $port_channel    = delete $opts{'port-channel-group'};
    my $channel_mode    = delete $opts{'channel-mode'};
    
    # configure requested configuration on interface
    $pkg->configure( %opts, %api_opts );

    #configure channge group
    $pkg->set_channel_group( %opts,
        'channel-num' => $port_channel,
        'channel-mode' => $channel_mode
    );

    $pkg->up( %opts );

    if ($nacltask_to_cleanup) {
        my $other_opts = {'port-channel-group' => $port_channel};
        $pkg->_register_for_cleanup(%opts_for_cleanup, 'other_opts' => $other_opts);
    }

    $Log->exit() if $may_exit;
}    # end of sub add_to_port_channel

=head2 remove_from_port_channel

NACL::STask::Switch::Interface->remove_from_port_channel(
    command_interface       => $command_interface,
    'interface-type'        => 'Ethernet',
    'interface-num'         => '1/1'
    'port-channel-group'    => $port-channel,
);

$interface_object->remove_from_port_channel(
    'port-channel-group'    => $port-channel,
);

(Class or Instance Method) This task method will remove interface from 
specified port-channel 

=over

=item Options

=over

=item C<< port-channel-group => $port-channel >>

(Required) This option provided the port-channel number from which interface
needs to be remove.

=back

=back

=cut

sub remove_from_port_channel {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
           'port-channel-group'  => { type => SCALAR , optional => 1 },
        },
        allow_extra => 1
    );

    my $pkg = ref($pkg_or_obj) || $pkg_or_obj;

    $Log->debug(
        sub {
            "Options to remove_from_port_channel:\n" . Dumper(\%opts);
        }
    );

    my $port_channel = delete $opts{'port-channel-group'}
    || $pkg_or_obj->{'port-channel-group'};

    NATE::BaseException->throw("Port channel group is not passed")
        if(!$port_channel);

    #remove channel-group
    $pkg->remove_channel_group(
        %opts,
        'channel-num' => $port_channel
    );

    $Log->exit() if $may_exit;
}    #end of sub delete_from_port_channel

=head2 configure

NACL::STask::Switch::Interface->configure(
    command_interface   => $command_interface,
    'interface-type'    => 'Ethernet',
    'interface-num'     => '1/1',
    speed               => '1000'
    duplex              => 'full',
    status              => 'up',
    mtu                 => 9216,
    'fc-send'           => 'on',
    'fc-receive'        => 'on',
    switchport          => 1,
);

$interface_object->configure(
    speed               => '1000'
    duplex              => 'full',
    status              => 'up',
    mtu                 => 9216,
    'fc-send'           => 'on',
    'fc-receive'        => 'on',
    switchport          => 1,
);

(Class or Instance Method) configure requested configuration on switch 
interface. This will allow user to configure multiple configuration on 
interface in one method call. exception will be raise during failure. 

=over

=item Options

=over

=item C<< speed => $speed >>

(Optional) speed of the interface. 

=item C<< duplex => $duplex >>

(Optional) duplex of the interface.

=item C<< status => $status >>

(Optional) status of interface. it can be 'up' or 'down'. default to 'up'.

=item C<< mtu => $mtu >>

(Optional) mtu on interface. supported on Cisco IOS only.

=item C<< fc-send => $fc_send >>

(Optional) flowcontrol send configuration on interface.

=item C<< fc-receive => $fc_receive >>

(Optional) flowcontrol receive configuration on interface.

=item C<< switchport => $boolean >>

(Optional) switchport configuration on interface.

=back

=back

=cut

=head2 add_to_vlan

NACL::STask::Switch::Interface->add_to_vlan(
    command_interface   => $command_interface,
    'interface-type'    => 'Ethernet',
    'interface-num'     => '1/1',
    vlan                => $vlan-id,
    native-vlan         => $native_vlan-id,
    nacltask_append     => 1,
);

$interface_object->add_to_vlan(
    vlan                => $vlan-id,
    native-vlan         => $native_vlan-id,
    nacltask_append     => 1,
);

(Class or Instance Method) This task method will add interface to specified 
vlan. following cases will be handle by this task method while adding interface
to vlan:

Assumptions: 

1. along with tagged, untagged frame always be allow in all cases.

Case 1: add all vlan on interface

$interface->add_to_vlan(
    vlan                => 'all',
    native-vlan         => 1,
    nacltask_append     => 1,
);

Case 2: add particular vlan on interface without appending

$interface->add_to_vlan(
    vlan                => 777,
    native-vlan         => 1,
    nacltask_append     => 0,
);

this will allows frame with vlan-id 777 and untagged frame.

Case 3: add particular vlan on interface with appending

$interface->add_to_vlan(
    vlan                => 777,
    native-vlan         => 1,
    nacltask_append     => 1,
);

this will allows frame with vlan-id 777, and other vlan if exists before and 
untagged frame.

Case 1 will be the default for this Task Method.

=over
 
=item Options

=over

=item C<< vlan => $vlan-id >>

(Optional) This option provide the vlan-id in which interface needs to be 
added. default is 'all'.

Following are the example for the value of this option: 

777 or 777,778,779,780 or 777-780 or all 

=item C<< native-vlan => $native_vlan >>

(Optional) This option provide vlan-id on which untagged frame will be allowed.
default is '1'.

=item C<< nacltask_append => $boolean >>

(Optional) This option allows user to append the vlan-id to the already 
existing vlan-ids on interface. default is '1'.

=back

=back

=cut

=head2 remove_from_vlan 

NACL::STask::Switch::Interface->remove_from_vlan(
    command_interface   => $command_interface,
    'interface-type'    => 'Ethernet',
    'interface-num'     => '1/1'
    vlan                => $vlan-id,
);

$interface_object->remove_from_vlan(
    vlan        => $vlan-id,
);

(Class or Instance Method) remove interface from specified vlan. By default, 
it will remove all vlan from interface.

=over

=item Options

=over

=item C<< vlan => $vlan-id >>

(Optional) This option provide the vlan-id, from which interface needs to be 
remove. default is 'all'.

=back

=back

=cut

1;
