#
#   Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary PortChannel Task Module for Switch PortChannel
## @author nayankumar.patel@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::STask::Switch::PortChannel

=head1 DESCRIPTION

C<NACL::STask::Switch::PortChannel> provide methods to create  
PortChannel on network switch.

It is created on top of C<NACL::C::Switch::PortChannel>. 
Since it is derived class of C<NACL::C::Switch::PortChannel>, we can use all 
the methods of C<NACL::C::Switch::PortChannel> from the object of this task.

=head1 CLEANUP METHODS

Cleanup can be registered for the following methods

Cleanup methods are,
    PortChannel Method               Cleanup Method
    ------------------------------------------------
    create                           purge

=head1 ATTRIBUTES

=head2 command_interface

(Required) A component object that represents the host to which to send 
commands.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

=head2 port_channel_group

(Required) As C<NACL::C::Switch::PortChannel>. port-channel number on switch

=cut

package NACL::STask::Switch::PortChannel;

use strict;
use warnings;

use base qw(NACL::C::Switch::PortChannel NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR BOOLEAN);
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::ComponentUtils qw(_optional_scalars);

# Some methods for Interface and PortChannel are implemented in _Interface.pm
use NACL::STask::Switch::_Interface qw(:all);

=head1 METHODS

=head2 create

my $port_channel = NACL::STask::Switch::PortChannel->create(
    command_interface       => $command_interface,
    'port-channel-group'    => 66,
    nacltask_to_cleanup      => 1,          # default 0,
    nacltask_cleanup_manager => $CleanupObj,
);

my $port_channel_object = NACL::STask::Switch::PortChannel->create(
    command_interface   => $command_interface,
    port-channel-group  => 66,
    speed               => '10000',
    duplex              => 'full',
    status              => 'up',
    mtu                 => 9198,
    fc-send             => 'off',
    fc-receive          => 'off',
);

(Class Method) create requested port-channel on switch. It will also configure 
the port-channel with certain configuration requested by user. these customized 
configuration can be requested with different options(see Options section). not
all options are supported on all switches. if user want to avoid any option, it 
can be done by setting that options as 'undef'. since some of the options has 
default value, user has to explicity assign value as 'undef', so that task will
avoid configuration for that option. if creation of port-channel fail, it will 
raise exception.

=over

=item Options

=over

=item C<< port-channel-group => $port_channel_number >>

(Required for class method, Not Applicable for instance method)
As C<NACL::C::Switch::PortChannel>. port-channel number.

=item C<< speed => $speed >>

(Optional) speed of the interface to be set on port-channel.

=item C<< duplex => $duplex >>

(Optional) duplex of the interface to be set on port-channel.

=item C<< status => $status >>

(Optional) status of port-channel after creating it. it can be 'up' or 'down'. 
default to 'up'.

=item C<< mtu => $mtu >>

(Optional) mtu setting on port-channel. supported on Cisco IOS only. default to
'9216'.

=item C<< fc-send => $fc-send >>

(Optional) flowcontrol send configuration on port-channel. default to 'on'.

=item C<< fc-receive => $fc-receive >>

(Optional) flowcontrol receive configuration on port-channel. default to 'on'.

=item C<< switchport => $boolean >>

(Optional) switchport configuration on port-channel. default to '1'.

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup)
Flag indicating if this operation needs to be registered for clean up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts       = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            speed              => { type => SCALAR, optional => 1 },
            duplex             => { type => SCALAR, optional  => 1 },
            mtu                => { type => BOOLEAN, default  => 9216 },
            'fc-send'          => { type => BOOLEAN, default  => 'on' },
            'fc-receive'       => { type => BOOLEAN, default  => 'on' },
            switchport         => { type => BOOLEAN, default  => 1 },
            status             => { type => BOOLEAN, default  => 'up' },
            $pkg->_cleanup_validate_spec(),
        },
        allow_extra => 1
    );

    my (%opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup );
    $pkg->_move_common_cleanup_opts(
        source  => \%opts,
        target  => \%opts_for_cleanup,
    );

    my %api_opts;

    $pkg->_hash_move(
        source => \%opts,
        target => \%api_opts,
        move   => [ qw(speed duplex mtu fc-send fc-receive switchport status) ],
    );

    ## create port-channel object
    my $self = $pkg->SUPER::create( %opts );

    ## delete undef key
    foreach (keys %api_opts) {
        if(!$api_opts{$_}) {
            delete $api_opts{$_};
        }
    }

    ##configure port-channel with specified configs
    $pkg->configure( %opts, %api_opts );

    # Register this resource with the cleanup manager for cleanup.
    $pkg->_copy_common_opts_for_cleanup(
        'source'             => {%opts, %opts_for_cleanup},
        'target'             => \%opts_for_register,
        'nacltask_to_cleanup'=> \$nacltask_to_cleanup,
        'to_cleanup'         => 'purge',
    );

    $pkg->_register_for_cleanup(%opts_for_register) if ($nacltask_to_cleanup);

    $Log->exit() if $may_exit;
    return $self;
} #end of sub create

=head2 purge

    NACL::STask::Switch::PortChannel->purge(
        command_interface       => $command_interface,
        'port-channel-group'    => 66,
    );

(Class or Instance Method) This task method will remove specified
port-channel-group from switch

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required) A component object that represents the host to which to send
commands.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

=item C<< port-channel-group => $port_channel_number >>

(Required for class method, Not Applicable for instance method)
As C<NACL::C::Switch::PortChannel>. port-channel number.

=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the deletion did happen
successfully.

=back

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my $pkg      = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
           nacltask_verify    => { type => BOOLEAN, default => 0 },
        },
        allow_extra => 1
    );
    my $verify = delete $opts{nacltask_verify};

    $pkg->SUPER::purge(%opts);

    if ( $verify ) {
        $pkg->_generic_purge_verify(%opts);
    }
    $Log->exit() if $may_exit;
} ## end sub purge

=head2 configure

NACL::STask::Switch::PortChannel->configure(
    command_interface       => $command_interface,
    'port-channel-group'    => '1',
    speed                   => '1000'
    duplex                  => 'full',
    status                  => 'up',
    mtu                     => 9216,
    'fc-send'               => 'on',
    'fc-receive'            => 'on',
    switchport              => 1,
);

$port_channel_object->configure(
    speed               => '1000'
    duplex              => 'full',
    status              => 'up',
    mtu                 => 9216,
    'fc-send'           => 'on',
    'fc-receive'        => 'on',
    switchport          => 1,
);

(Class or Instance Method) configure requested configuration on switch 
port-channel. This will allow user to configure multiple configuration on 
port-channel in one method call. exception will be raise during failure. 

=over

=item Options

=over

=item C<< speed => $speed >>

(Optional) speed of the port-channel. 

=item C<< duplex => $duplex >>

(Optional) duplex of the port-channel.

=item C<< status => $status >>

(Optional) status of port-channel. it can be 'up' or 'down'. default to 'up'.

=item C<< mtu => $mtu >>

(Optional) mtu on port-channel. supported on Cisco IOS only.

=item C<< fc-send => $fc_send >>

(Optional) flowcontrol send configuration on port-channel.

=item C<< fc-receive => $fc_receive >>

(Optional) flowcontrol receive configuration on port-channel.

=item C<< switchport => $boolean >>

(Optional) switchport configuration on port-channel.

=back

=back

=cut

=head2 add_to_vlan

NACL::STask::Switch::PortChannel->add_to_vlan(
    command_interface       => $command_interface,
    'port-channel-group'    => '1',
    vlan                    => $vlan-id,
    native-vlan             => $native_vlan-id,
    nacltask_append         => 1,
);

$port_channel_object->add_to_vlan(
    vlan                => $vlan-id,
    native-vlan         => $native_vlan-id,
    nacltask_append     => 1,
);

(Class or Instance Method) This task method will add port-channel to specified 
vlan. following cases will be handle by this task method while adding 
port-channel to vlan:

Assumptions: 

1. along with tagged, untagged frame always be allow in all cases.

Case 1: add all vlan on interface

$port_channel_object->add_to_vlan(
    vlan                => 'all',
    native-vlan         => 1,
    nacltask_append     => 1,
);

Case 2: add particular vlan on port-channel without appending

$port_channel_object->add_to_vlan(
    vlan                => 777,
    native-vlan         => 1,
    nacltask_append     => 0,
);

this will allows frame with vlan-id 777 and untagged frame.

Case 3: add particular vlan on port-chanel with appending

$port_channel_object->add_to_vlan(
    vlan                => 777,
    native-vlan         => 1,
    nacltask_append     => 1,
);

this will allows frame with vlan-id 777, and other vlan if exists before and 
untagged frame.

Case 1 will be the default for this Task Method.

=over
 
=item Options

=over

=item C<< vlan => $vlan-id >>

(Optional) This option provide the vlan-id in which port-channel needs to be 
added. default is 'all'.

Following are the example for the value of this option: 

777 or 777,778,779,780 or 777-780 or all 

=item C<< native-vlan => $native_vlan >>

(Optional) This option provide vlan-id on which untagged frame will be allowed.
default is '1'.

=item C<< nacltask_append => $boolean >>

(Optional) This option allows user to append the vlan-id to the already 
existing vlan-ids on port-channel. default is '1'.

=back

=back

=cut

=head2 remove_from_vlan 

NACL::STask::Switch::PortChannel->remove_from_vlan(
    command_interface       => $command_interface,
    'port-channel-group'    => '1'
    vlan                    => $vlan-id,
);

$port_channel_object->remove_from_vlan(
    vlan        => $vlan-id,
);

(Class or Instance Method) remove port-channel from specified vlan. By default, 
it will remove all vlan from port-channel.

=over

=item Options

=over

=item C<< vlan => $vlan-id >>

(Optional) This option provide the vlan-id, from which port-channel needs to be 
remove. default is 'all'.

=back

=back

=cut


1;
