#
#   Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Vlan Task Module for Switch Vlan
## @author nayankumar.patel@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::STask::Switch::Vlan

=head1 DESCRIPTION

C<NACL::STask::Switch::Vlan> provide methods to create vlan on network switch.

It is created on top of C<NACL::C::Switch::Vlan> component. Since it is 
derived class of C<NACL::C::Switch::Vlan>, we can use all the methods of 
C<NACL::C::Switch::Vlan> from the object of this task.

=head1 CLEANUP METHODS

Cleanup can be registered for the following methods

Cleanup methods are,
    Vlan Method               Cleanup Method
    ----------------------------------------
    create                     purge

=head1 ATTRIBUTES

=head2 command_interface

(Required) A component object that represents the host to which to send 
commands.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

=head2 vlan_id

(Required) As C<NACL::C::Switch::Vlan>. vlan-id on switch

=cut

package NACL::STask::Switch::Vlan;

use strict;
use warnings;

use base qw(NACL::C::Switch::Vlan NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR ARRAYREF UNDEF SCALARREF BOOLEAN);
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

=head1 METHODS

=head2 create

my $vlan = NACL::STask::Switch::Vlan->create(
    command_interface   => $command_interface,
    :106
    'vlan-id'           => 666,
    state               => 'active',
    status              => 'up',
    mtu                 => 9216,
    nacltask_to_cleanup      => 1,          # default 0,
    nacltask_cleanup_manager => $CleanupObj,
);

(Class Method) create requested vlan on switch. It will also configure the vlan
with certain configuration requested by user. these customized configuration 
can be requested with different options(see Options section). Options are 
applicable based on switch type. not all switch support all the options. check 
particular option for switch support.
if creation of vlan fail, it will raise exception.

=over

=item Options

=over

=item C<< state => $state >>

(Optional) set the state of the vlan. default to 'active'.

=item C<< mtu => $mtu >>

(Optional) set the mtu on vlan. support on Cisco IOS switch. default to 9216

=item C<< status => $status >>

(Optional) status of the vlan. it can be 'up' or 'down'. default to 'up'. 

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup)
Flag indicating if this operation needs to be registered for clean up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
           'mtu'        => { type => SCALAR, default => 9216 },
           'state'      => { type => SCALAR, default => 'active' },
           'status'     => { type => SCALAR, default => 'up' },
           $pkg->_cleanup_validate_spec(),
        },
        allow_extra => 1
    );

    my (%opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup );
    $pkg->_move_common_cleanup_opts(
        source  => \%opts,
        target  => \%opts_for_cleanup,
    );
    my %api_opts;

    $pkg->_hash_move(
        source => \%opts,
        target => \%api_opts,
        move   => [qw(mtu state status)],
    );

    $Log->debug( sub { "Options to create:\n" . Dumper(\%api_opts); } );

    my $self = $pkg->SUPER::create(%opts);

    if ( defined( $api_opts{state} ) ) {
        $self->set_state( state => $api_opts{state} );
    }

    if ( defined( $api_opts{mtu} ) ) {
        if ( $opts{command_interface}->hostrec()->hosttype =~ /nxos/ ) {
            $Log->warn("Setting mtu size is not supported on Nexus OS");
        } else {
            $self->set_mtu( mtu => $api_opts{mtu} );
        }
    }

    if ( $api_opts{status} =~ /up/i ) {
        $self->up();
    } elsif ( $api_opts{status} =~ /down/i ) {
        $self->down();
    } else {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw( "status should be 'up' or 'down'" );
    }
    # Register this resource with the cleanup manager for cleanup.
    $pkg->_copy_common_opts_for_cleanup(
        'source'             => {%opts, %opts_for_cleanup},
        'target'             => \%opts_for_register,
        'nacltask_to_cleanup'=> \$nacltask_to_cleanup,
        'to_cleanup'         => 'purge',
    );

    $pkg->_register_for_cleanup(%opts_for_register)
        if ($nacltask_to_cleanup);

    $Log->exit() if $may_exit;
    return $self;
} #end of sub create


=head2 purge

    NACL::STask::Switch::Vlan->purge(
        command_interface   => $command_interface,
        'vlan-id'           => 666,
    );

(Class or Instance Method) This task method will remove specified vlan from
switch

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required) A component object that represents the host to which to send
commands.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

=item C<< vlan_id => $vlan_id >>

(Required) As C<NACL::C::Switch::Vlan>. vlan-id on switch

=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the deletion did happen
successfully.

=back

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my $pkg      = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
           nacltask_verify    => { type => BOOLEAN, default => 0 },
        },
        allow_extra => 1
    );
    my $verify = delete $opts{nacltask_verify};

    $pkg->SUPER::purge(%opts);

    if ( $verify ) {
        $pkg->_generic_purge_verify(%opts);
    }
    $Log->exit() if $may_exit;
} ## end sub purge

1;
