#
# Copyright (c) 2001-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemNodeImage Task Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::SystemNodeImage;

use strict;
use warnings;

use base qw(NACL::C::SystemNodeImage);
use NATE::BaseException qw(:try);
use NACL::Exceptions::HighAvailabilityError;
use Params::Validate qw(SCALAR ARRAYREF);
use NACL::Tools::httpd::control ();
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::STask::Cluster;
use NACL::C::StorageAggregate;
use NACL::C::StorageAggregate64bitUpgrade;
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::STask::Node;
use NACL::Transit;
use NATE::Time qw(timeout2time);
use NACL::Exceptions::Timeout;
use NACL::C::VserverNfs;
use NACL::C::Cluster;
use NACL::C::SystemNodeImage;
use NACL::GeneralUtils qw(random_name_generator);

use NACL::C::Exceptions::StorageAggregate64bitUpgrade::AlreadyDisabled;

# Installation may take up to 30 minutes, but Smokes runs timeout within
# 30 minutes, so increasing it to 50 minutes
use constant DEFAULT_TIMEOUT => 3000;

=head1 NAME

NACL::STask::SystemNodeImage

=head1 SYNOPSIS

  # example 
  my $SystemNodeImageUpdate_obj = NACL::STask::SystemNodeImage->update(
                                           command_interface=>$ci,
                                           node =>$node_hostname,
                                           package => $package,
                                           background => 'false'  #default value
                                  );
  $SystemNodeImageUpdate_obj->wait_for_completion('method-timeout' => 3600);

=head1 DESCRIPTION

C<NACL::STask::SystemNodeImage> is a derived class of
L<NACL::C::SystemNodeImage|lib-NACL-C-SystemNodeImage-pm>. It is a Task 
to perform an ONTAP SystemNodeImage Update/Upgrade given either a URL, a package, or
the relevant parts of the path that would be used by this Task to derive
a package e.g. the ONTAP directory path, build flags, and architecture name.

=head1 ATTRIBUTES

=head2 command_interface

(Required) See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 node

(Required) The hostname of the Node.

=head2 package

This specifies the location from which the software image is to be downloaded.

=head1 METHODS

=head2 update

  my $SystemNodeImageUpdate_obj = NACL::STask::SystemNodeImage->update(
                                           command_interface=>$ci,
                                           node =>$node_hostname,
                                           package => $package,
                                  );
  or

  my $SystemNodeImageUpdate_obj = NACL::STask::SystemNodeImage->update(
                                           command_interface=>$ci,
                                           node =>$node_hostname,
                                           ontap_dir => "$ot_dir",
                                           build_flags => \@,
                                           arch => $architecture,
                                  );
  or

  # Upgrade all the nodes   
  my @SystemNodeImageUpdate_obj = NACL::STask::SystemNodeImage->update(
                                           command_interface=>$ci,
                                           node   => '*',
                                           package => $package,
                                           nacltask_upgrade_to_build => 1,
                                  );

  (Class method)
  Return a new SystemNodeImageUpdate Task object/objects with the given attributes.
  If user passed single node then method return single task object while if user 
  passes multiple nodes then method returns the array of Task objects.
  This task will refresh the command_interface if nacltask_upgrade_to_build param
  is set to '1'.
  this task will reboot all the specified nodes when nacltask_upgrade_to_build be 1. 
  If your library/script has already created multiple other command_interface for 
  the same nodes, then the responsibility of updating each such command_interface 
  object is expected to be handled by the calling library for now. 
  Also users are expected to have tharnhost files for all the specified nodes 
  because Transit needs it while rebooting

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required) See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=item C<< node => $node_hostname >>

(Required) The hostname of the Node of any of below types -
SCALAR - for single node update
ARRAYREF - for multiple nodes update
If user wants to update all nodes in cluster or HA the value of 'node' will be '*'.

=item C<< package => $package >>

(Required, if ontap_dir, build_flags and arch are not provided) The full package path in either of these formats, e.g.: 

"http://web.netapp.com/engineering/x/eng/rlse/DOT/mainN_110407_0030/final/bedrock/export/x86.sim/tarball/image.tgz" 

or 

"/x/eng/rlse/DOT/mainN_110407_0030/final/bedrock/export/x86.sim/tarball/image.tgz".

=item C<< ontap_dir => $ot_dir >>

(Optional) The ONTAP directory path, e.g. /x/eng/rlse/DOT/mainN_110407_0030/final/bedrock/export, if "package" is not provided.

=item C<< build_flags => \@ >>

(Optional) The build flags, e.g. pbo, debug, sim, gcov, etc., if "package" is not provided.

=item C<< arch => $architecture >>

(Optional) The architecture type, e.g. pc_elf, x86-64 etc., if "package" is not provided.

=item C<< web_host => xyz >>

(Optional) The hostname of the server to run an http_server on, defaults to "localhost"

=item C<< background = 'true'|'false' >>

Update the node image in the background, and do not wait for jobs to complete before
returning from the call. Defaults to 'true' (don't wait for update to complete).
if 'background' is equal to 'false' then 'wait_for_completion' method won't be applicable.

=item C<< nacltask_upgrade_to_build => 0|1 >>

(Optional) Default to '0'
If '1', update and upgrade the requested node( boot into new image ).
If '0', update the requested nodes with image.
C<<wait_for_completion>> method need not be invoked even if you pass background to be 'true',
as this method waits for image update to complete and then reboots the node(s) when this
option is enabled.

=back

=back

=cut

=head2 wait_for_completion

  $SystemNodeImageUpdate_obj->wait_for_completion();

  (Instance method)
  Waits for the SystemNodeImageUpdate to be completed. In 7mode/7G,
  returns immediately since there is no -background option.  In Cmode,
  calls show_update_progress() on the Task object.
  This method is not applicable if user passes background to 'false'.

=over

=item Arguments

=over

=item C<< 'method-timeout' => $timeout_value_in_seconds >>

(Optional) Default to 3000 seconds
How long to wait for the completion of job started by 'update' operation.

=back

=back

=over

=item Returns

=over

=item Nothing.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::StorageAggregate64bitUpgrade::AlreadyDisabled>

This type of exception gets thrown if an attempt is made to disable 
32-bit aggregate support when its already disabled.

=item C<NATE::BaseException>

This type of exception is thrown when 'package' parameter is not defined
and either one of these parameters (ontap_dir or build_flags or arch)
has null value.

=back

=back

=cut

sub update {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            node        => { type => SCALAR|ARRAYREF , optional => 1 },
            replace     => { type => SCALAR,            optional => 1 },
            ontap_dir   => { type => SCALAR,            optional => 1 },
            build_flags => { type => ARRAYREF,          optional => 1 },
            arch        => { type => SCALAR,            optional => 1 },
            'ignore-compatibility' => { type => SCALAR, optional => 1 },
            setdefault             => { type => SCALAR, default  => "true" },
            'replace-package'      => { type => SCALAR, default  => "true" },
            'rename-package'       => { type => SCALAR, optional => 1 },
            package                => { type => SCALAR, optional => 1 },
            web_host               => { type => SCALAR, optional => 1 },
            background             => { type => SCALAR, optional => 1 },
            'method-timeout'       => { type => SCALAR, optional => 1 },
            nacltask_upgrade_to_build     => { type => SCALAR, default => 0 },
        },
        ignore_primary_keys => 1,
        allow_extra         => 1
    );

    my %multi_update_params;
    $pkg->_hash_move(
        source => \%opts,
        target => \%multi_update_params,
        move   => [
            'node',    'nacltask_upgrade_to_build',
            'replace', 
        ]
    );

    if ( !defined( $opts{package} ) ) {
        my ( @errorString, $tmpString );

        $tmpString = $opts{ontap_dir};
        $tmpString = _trim($tmpString);
        if ( length($tmpString) == 0 ) {
            push @errorString, "'ontap_dir' has a empty string for a value";
        }

        $tmpString = $opts{build_flags};
        $tmpString = _trim($tmpString);
        if ( length($tmpString) == 0 ) {
            push @errorString, "'build_flags' has a empty string for a value";
        }

        $tmpString = $opts{arch};
        $tmpString = _trim($tmpString);
        if ( length($tmpString) == 0 ) {
            push @errorString, "'arch' has a empty string for a value";
        }

        if (@errorString) {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(@errorString);
        }

        my %args;
        %args = (
            mode        => $opts{command_interface}->mode(),
            ontap_dir   => $opts{ontap_dir},
            build_flags => $opts{build_flags},
            arch        => $opts{arch},
        );

        $opts{package} = _guess_image_path( \%args );
    } ## end if ( !defined( $opts{package...}))

    my $client = $opts{web_host} || Hostrec->new( id => "localhost" );

    my $httpd = NACL::Tools::httpd::control->new();
    if ( $opts{package} !~ m@^(ftp|http)://@ ) {
        my $basename
            = substr( $opts{package}, ( rindex( $opts{package}, '/' ) + 1 ) );
        my $url = $httpd->start(
            hostrec => $client,
            peer    => $opts{command_interface}->hostrec()->default_ip(),
            gets    => { "/$basename" => $opts{package} },
        );
        $opts{package} = $url . "$basename";
    } ## end if ( $opts{package} !~...)

    my @node_names;
    my $node_to_int_7mode = {};

    # Finding the list of nodes
    if ( exists $multi_update_params{node}
        && $multi_update_params{node} eq '*' )
    {
        if ( $opts{command_interface}->is_cmode() ) {
            @node_names = NACL::STask::Cluster->nodes(
                command_interface => $opts{command_interface} );
        } else {
            @node_names = ( $opts{command_interface}->node() );
            try {
                my $partner = NACL::STask::Node->get_partner(
                    command_interface => $opts{command_interface} );
                push @node_names, $partner;
                my $partner_obj = NACL::STask::Node->get_partner_obj(
                    command_interface => $opts{command_interface} );
                $node_to_int_7mode->{$partner} = $partner_obj;
            } ## end try
            catch NACL::Exceptions::HighAvailabilityError with {};
        } ## end else [ if ( $opts{command_interface...})]
    } else {
        if ( exists $multi_update_params{node} ) {
            if ( ref( $multi_update_params{node} ) eq "ARRAY" ) {
                @node_names = @{ $multi_update_params{node} };
            } else {
                @node_names = ( $multi_update_params{node} );
            }
        } else {
            @node_names = ( $opts{command_interface}->node() );
        }
    } ## end else [ if ( exists $multi_update_params...)]

    my $node_info;
    if ( $opts{command_interface}->is_cmode() ) {

        my ( $image1_true, $image2_true, $node ) = ( 0, 0, '' );
        my @SystemNodeImage_state = NACL::CS::SystemNodeImage->fetch(
            command_interface => $opts{command_interface},
            filter            => {
                node  => '*',
                image => "image1",
            },
            requested_fields  => ['node','image','iscurrent'],
        );

        for ( my $i = 0; $i <= $#node_names; $i++ ) {
            $node = $node_names[$i];
            $node_info->{$node} = {};
            foreach my $SNI_st_obj (@SystemNodeImage_state) {
                next if ( $SNI_st_obj->node() ne $node );
                if ( $SNI_st_obj->{iscurrent} =~ /false/i ) {
                    $node_info->{$node}->{replace} = "image1";
                    $image2_true = 1;
                } elsif ( $SNI_st_obj->{iscurrent} =~ /true/i ) {
                    $node_info->{$node}->{replace} = "image2";
                    $image1_true = 1;
                }
            } ## end foreach my $SNI_st_obj (@SystemNodeImage_state)
        } ## end for ( my $i = 0; $i <= ...)

        if ( !( $image2_true && $image1_true ) ) {
            $opts{replace} = $node_info->{$node}->{replace};
        }
    } ## end if ( $opts{command_interface...})

    $opts{'method-timeout'} ||= DEFAULT_TIMEOUT;

    delete $opts{ontap_dir};
    delete $opts{build_flags};
    delete $opts{arch};
    delete $opts{web_host};

    my @update_obj = map {
        $pkg->new(
            node              => $_,
            command_interface => $node_to_int_7mode->{$_}
                || $opts{command_interface},
            image => $node_info->{$_}->{replace} || $opts{replace} || "image1"
            )
    } @node_names;
    my $retry_count = 0;
    my $source_pem_path = "/var/etc/";
    my $tmp_pem_path = "/tmp/".random_name_generator()."/";
    my $cleanup_promo_nonpromo_workarnd = 0;
UNDO: {
    use warnings 'exiting';
try {
    if (   $opts{command_interface}->is_cmode()
        && defined $opts{replace}
        && $multi_update_params{node} eq '*' )
    {
        my $update_comp_objs = $pkg->SUPER::update( %opts,
            node => $multi_update_params{node} , background=> 'false');
    } else {
        foreach my $_update_obj (@update_obj) {
            my %params;
            $params{background} = 'false'
                if ( exists $multi_update_params{nacltask_upgrade_to_build}
                && $multi_update_params{nacltask_upgrade_to_build} );
            my $modify_image = 1;
            try {
                my $apiset;
                my $ci   = $_update_obj->{command_interface};
                my $node = $_update_obj->{node};
                $pkg->SUPER::update(
                    %opts,
                    %params,
                    node              => $node,
                    replace           => $_update_obj->{image},
                    command_interface => $ci,
                    apiset_chosen     => \$apiset,
                );
                if ($apiset->{interface} eq 'ZAPI') {
                    # CLI defaults to waiting in the foreground, but ZAPI returns
                    # immediately, so we need to poll for completion for ZAPI.
                    # Burt 815699.
                    my %common_opts;
                    $pkg->_copy_common_component_params(
                        source => \%opts,
                        target => \%common_opts,
                    );

                    my $scaled_timeout = $ci->scale_timeout(
                        'method-timeout' => $opts{'method-timeout'});
                    my $end_time = timeout2time($scaled_timeout);

                    while (1) {
                        my $progress_cs = $pkg->show_update_progress_no_follow(
                            %common_opts,
                            command_interface => $ci,
                            node              => $node,
                        );
                        if ($progress_cs->run_status() eq 'Exited') {
                            my $exit_status = $progress_cs->exit_status();
                            if ($exit_status eq 'Success') {
                                last;
                            } else {
                                $Log->exit() if $may_exit;
                                NATE::BaseException->throw('Update of the image '
                                    . "for node '$node' failed. Reason:\n"
                                    . $progress_cs->exit_message()
                                );
                            }
                        } else {
                            if (time() > $end_time) {
                                $Log->exit() if $may_exit;
                                NACL::Exceptions::Timeout->throw('After '
                                    . "waiting for $scaled_timeout seconds, "
                                    . "the update had not completed.\n"
                                    . 'Phase: '
                                    . $progress_cs->phase()
                                    . "\nLast Message:"
                                    . $progress_cs->last_message()
                                );
                            } else {
                                Tharn::snooze(30);
                            }
                        }
                    }
                }
                $modify_image = 0;
            } catch NACL::C::Exceptions::SystemNodeImage::Update64BitAggr with {
                NACL::C::StorageAggregate64bitUpgrade->prepare_to_downgrade(
                    command_interface => $opts{command_interface});
            } catch NACL::C::Exceptions::SystemNodeImage::Remove32BitData with {
                try {
                    NACL::C::StorageAggregate64bitUpgrade
                        ->storage_aggregate_64bit_upgrade_32bit_disable(
                        command_interface => $opts{command_interface});
                } catch NACL::C::Exceptions::StorageAggregate64bitUpgrade::AlreadyDisabled with {
                    # this happens when command is issued from multiple nodes on the cluster
                    # so this means that the 32-bit aggr support is already disabled. If so, continue.
                    $Log->debug(
                        '32-bit aggr support is already disabled. Let us continue.'
                    );
                 };
            } catch NACL::C::Exceptions::SystemNodeImage::DisableNFSv3 with {
                my %common_opts;
                $pkg->_copy_common_component_params_with_ci(
                    source => \%opts,
                    target => \%common_opts,);

                NACL::C::VserverNfs->prepare_for_v3_ms_dos_client_downgrade(
                    %common_opts,);

            };
            if ($modify_image) {
                NACL::STask::Cluster->cluster_health_check(
                    command_interface => $opts{command_interface});

                # using $opts{setdefult} param as isdefault
                # using $opts{image} param as image
                NACL::C::SystemNodeImage->modify(
                    command_interface => $opts{command_interface},
                    node              => $_update_obj->{node},
                    'isdefault'       => $opts{setdefault},
                    image             => $opts{replace}
                );
            }
        } ## end foreach my $_update_obj (@update_obj)
    } ## end else [ if ( $opts{command_interface...})]

    if ( $cleanup_promo_nonpromo_workarnd ) {
	my $Client_apiset = $opts{command_interface}->get_systemshell_apiset();
	$Client_apiset->execute_raw_command( command => "sudo cp $tmp_pem_path*production*.pem $source_pem_path" );
	$Client_apiset->rm(recursive => 1, force => 1, paths => $tmp_pem_path);
    }
} catch NACL::C::Exceptions::SystemNodeImage::VerifySignaturesImage with {
    my $ex = shift;
    if ( $retry_count++ == 0 ) {
	if ($opts{command_interface}->get_version_manager()->is_promoted()) {
            my $Client_apiset = $opts{command_interface}->get_systemshell_apiset();

	    # create timestamped temporary directory
	    $Client_apiset->mkdir( paths => $tmp_pem_path );

            # list down all .pem files at /var/etc/ path
            $Client_apiset->ls( 'one-per-line' => 1,
                                paths => $source_pem_path."*.pem");

	    my @all_prd_backup_files = @{$Client_apiset->ls(paths => $source_pem_path."*production*.pem")->get_parsed_output()->[0]{'files'}};
	    $Client_apiset->execute_raw_command( command => "sudo cp $source_pem_path*production*.pem $tmp_pem_path" );

            my @all_dev_files = @{$Client_apiset->ls(paths => $source_pem_path."*development*.pem")->get_parsed_output()->[0]{'files'}};
            $Client_apiset->execute_raw_command( command => "sudo cp $source_pem_path*development*.pem $tmp_pem_path" );

            my $dest_file;
            foreach my $source_file (@all_dev_files) {
		$source_file =~ s/$source_pem_path//g;
		$dest_file = $source_file;
		$dest_file =~ s/-development/-production/g;

		$Client_apiset->execute_raw_command( command => "sudo cp $tmp_pem_path$source_file $source_pem_path$dest_file" );
            }

	    $cleanup_promo_nonpromo_workarnd = 1;
	} else {
	    my @nodes_in_cluster= NACL::CS::Cluster->fetch(
				    command_interface => $opts{command_interface},
				    );
	    foreach (@nodes_in_cluster){
		NACL::C::SystemNodeImage->dev_promoted_update (
		    node                          => $_->node(),
		    command_interface => $opts{command_interface},
		);
	    }
	}
	no warnings 'exiting';
	redo UNDO;
    }

    if ( $cleanup_promo_nonpromo_workarnd ) {
	my $Client_apiset = $opts{command_interface}->get_systemshell_apiset();
        $Client_apiset->execute_raw_command( command => "sudo cp $tmp_pem_path*production*.pem $source_pem_path" );
        $Client_apiset->rm(recursive => 1, force => 1, paths => $tmp_pem_path);
    }

    $ex->throw();
};
} # UNDO LABEL
    if ( exists $multi_update_params{nacltask_upgrade_to_build}
        && $multi_update_params{nacltask_upgrade_to_build} )
    {
        foreach my $node (@node_names) {
            $Log->debug("Booting the $node in new build");
            _boot_to_image(node=>$node, is_cmode=>$opts{command_interface}->is_cmode(), timeout=>$opts{'method-timeout'});
        }

        $opts{command_interface}->refresh_command_interface();
    } ## end if ( exists $multi_update_params...)

    $Log->exit() if $may_exit;
    return ( wantarray() ) ? @update_obj :$update_obj[0];
} ## end sub update

sub _trim {
    my $string = shift;
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;
    return $string;
} ## end sub _trim

sub _guess_image_path {
    $Log->enter() if $may_enter;
    my $href        = shift;
    my $build_flags = $href->{'build_flags'};
    my $image_path  = $href->{'ontap_dir'};

    if ( $href->{'mode'} =~ /(cmode|7mode)/i ) {

        $image_path .= "/$href->{'arch'}";

        foreach my $a_flag (@$build_flags) {
            while ( $a_flag !~ /^\s*$/ ) {
                if (   $a_flag =~ /(^debug|optimize)/
                    || $a_flag =~ /(domain)/
                    || $a_flag =~ /(gcov)/
                    || $a_flag =~ /(sim)/
                    || $a_flag =~ /(ibm)/ )
                {
                    $image_path .= ".$1";
                    $a_flag =~ s/$1//;
                } ## end if ( $a_flag =~ /(debug|optimize)/...)
                elsif ( $a_flag =~ /(non-debug)/ ){
                    $a_flag =~ s/$1//;  ## We are here since it would have matched non-debug. Dont append anything but remove the entry from the flags array 
                } ## end elsif $a_flag is non-debug
                else {
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw("'build_flags' has a invalid value  $a_flag , should be either of debug,non-debug,domain,gcov,sim,ibm or combination of these");
                } ## end else it is a invalid option

            } ## end while ( $a_flag !~ /^\s*$/)
        } ## end foreach my $a_flag (@$build_flags)
        $image_path .= "/tarball/image.tgz";
    } elsif ( $href->{'mode'} =~ /(7g)/i ) {
        foreach my $a_flag (@$build_flags) {
            while ( $a_flag !~ /^\s*$/ ) {
                if (   $a_flag =~ /(debug|pbo|sim)/
                    || $a_flag =~ /(gcov)/
                    || $a_flag =~ /(ibm)/ )
                {
                    $image_path .= "_$1";
                    $a_flag =~ s/$1//;
                } ## end if ( $a_flag =~ /(debug|pbo|sim)/...)
            } ## end while ( $a_flag !~ /^\s*$/)
        } ## end foreach my $a_flag (@$build_flags)
        $image_path .= "/$href->{'arch'}/dist/setup.exe";
    } ## end elsif ( $href->{'mode'} =~...)

    $Log->exit() if $may_exit;
    return $image_path;
} ## end sub _guess_image_path

sub wait_for_completion {
    $Log->enter() if $may_enter;
    my $obj     = shift;
    my %opts    = @_;
    my $timeout = $opts{'method-timeout'} || DEFAULT_TIMEOUT;

    if ( $obj->command_interface->is_7mode() ) {

        # Do nothing since there's no -background option, so it
        # returns as soon as the update is done.
    } elsif ( $obj->command_interface->is_cmode()) {
        try{
            NACL::C::SystemNodeImage->show_update_progress(
                command_interface => $obj->command_interface(),
                node              => $obj->node(),
                'method-timeout'  => $timeout
            );
        } catch NACL::APISet::Exceptions::CommandFailedException with{
            my $exception = shift;
            ## After the completion of the image update on the specified node.
            ## this command throws an error "There is no update/install in progress"
            ## which needs to be catched.
            $exception->throw
                unless ($exception->text() =~ /There is no update\/install in progress/);
        };
    } ## end elsif ( $obj->command_interface...)
    $Log->exit() if $may_exit;
} ## end sub wait_for_completion

sub _boot_to_image {
    $Log->enter() if $may_enter;
    my %opts    = @_;
    my $timeout = $opts{'timeout'} || 600;
    my $transit_obj = NACL::Transit->new( name => $opts{node} );
    $transit_obj->reboot( timeout => $timeout );
    $Log->exit() if $may_exit;
} ## end sub _boot_to_image

1;
