# $Id$
#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#

## @summary VolumeClone Task Module
## @author rajesh.vk@netapp.com, dl-dpgqa-automation@netapp.com
## @status shared
## @pod here

package NACL::STask::VolumeClone;

use strict;
use warnings;

use base qw(NACL::C::VolumeClone NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR SCALARREF BOOLEAN OBJECT UNDEF);
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::TimeoutException;
use NACL::ComponentUtils qw(_optional_scalars Dumper _hash_move);
use NACL::STask::Volume;
use NACL::C::Exceptions::VolumeCloneSplit::SplitInProgress;
use NACL::C::Exceptions::Component qw(:try);
use Scalar::Util qw(blessed);

use constant DEFAULT_CLONE_TIMEOUT => 500;
use constant CLONE_POLL_DELTA      => 5;

use Class::MethodMaker [
    'scalar' => [ { -type => 'NACL::C::Job' }, 'job_component' ] ];

# Same routine for aggregates and volumes.  Pull implementation from
# _AggregateVolume.pm.

use NACL::STask::SystemLicense;
use NACL::C::VolumeCloneSplit;
use NACL::STask::Volume;

# Pull implementation from mix-in
use NACL::STask::_Mixins::VolumeClone qw(:all);

=head1 NAME

NACL::STask::VolumeClone

=head1 DESCRIPTION

C<NACL::STask::VolumeClone> provides a number of well-defined but potentially
complex or multi-step methods related to Volume Clone task on ONTAP.


It builds on top of, and is a derived class of C<NACL::C::VolumeClone>,
and so it also provides methods that are more in the scope of
individual VolumeClone-related commands.  See C<NACL::C::VolumeClone> for
details.  This also means that a C<NACL::STask::VolumeClone> object may
generally be used in place of a component object.


=head1 ATTRIBUTES

=head2 command_interface

(Required) As C<NACL::C::VolumeClone>.  A component object that
represents the host to which to send commands.


=head2 flexclone

(optional) The name of the flexclone to be created. If not provided,
random name would be generated. Random name would not be generated
for callers of new().

=head2 parent-volume

(Required) The name of the parent volume to be cloned.

=head2 vserver

(Required for C-mode) As C<NACL::C::VolumeClone>.  The name of the vserver
containing the parent volume.

=head2 job_component

(Optional) A job component (NACL::C::Job) representing the job used to
start this volume move task. Applicable for CMode only.


=head1 METHODS

=head2 create

  my $vol_clone_obj  = NACL::STask::VolumeClone->create(
      command_interface   => $command_interface,
      flexclone           => $cloneName,        # Optional
      vserver             => $vserver,
      'parent-volume'     => $volume,
      nacltask_if_exists  => $action,   # default 'die'
      nacltask_wait       => $boolean,  # default '1'
      nacltask_to_cleanup => 1,
      nacltask_cleanup_manager => $cleanupobj,
      %other_options
  );

(Class method) creates a clone of a parent volume. This method provides the
following additional services beyond what's provided by the component:

* Cleanup support.

* Auto-assigns a flexclone name if one is not provided.

* Waits for completion (can be controlled by the C<nacltask_wait> argument.

* Adds the FlexClone license if necessary.

* For CMode, it also mounts the volume (i.e.
automatically assigns a junction-path for it) even if the C<junction-path>
argument has not been provided. This behavior can be disabled by setting 
C<junction-path> to undef.

Applicable for CMode/7Mode and CLI/ZAPI.

=over

=item Options

=over

=item C<< nacltask_if_exists => $action >>

(Optional) What to do if the volume clone to be created already exists.
If $action is "die" (the default), then fail with an exception (in the
same way that the volume clone component would have: by trying the
volume clone create and letting the product complain about the clone
already existing).  If action is "reuse", then do nothing (return a
task object referring to the existing clone).  If action is
"purge", then delete the volume clone before creating a new one.

=item C<< nacltask_wait => 0|1 >>

(Optional) If 1 (the default), wait for volume clone create job completion
If 0, do not wait. This option only valid when option 'foreground' not equal
to 'true'

=item C<< _was_created => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the volume clone
was found (value will be 0; this scenario is possible when
if_exists => "reuse") or whether the volume clone was created (value will be 1).
This is necessary to determine whether the volume clone needs to be
cleaned up later.

    my $was_created;
    my $vol_clone_obj  = NACL::STask::VolumeClone->create(
      command_interface   => $command_interface,
      flexclone           => $cloneName,
      vserver             => $vserver,
      'parent-volume'     => $volume,
      nacltask_if_exists  => 'reuse',   # default 'die'
      nacltask_wait       => $boolean,  # default '1'
      _was_created        => \$was_created,
      %other_options
    );

    # Operate on $vol_clone_obj here
    # ...
    # Now determine whether to clean up the volume clone, since we're not sure
    # whether we reused an existing volume clone or created a new one
    if ($was_created) {
        # New volume clone was created. Clean it up.
        $vol_clone_obj->purge();
    }

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, defaults to 0(no to cleanup))
Flag indicating if this operation is to be cleaned up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=item command_interface, apiset_must, apiset_should, method-timeout, parent-volume,
vserver,foreground,etc,. All of the other various options described in
L<< NACL::C::VolumeClone->create | lib-NACL-C-VolumeClone-pm/create >>

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VolumeClone::AlreadyExists>

This type of exception is thrown when an attempt is made to create volume clone that already exists.

=back

=back

=over

=item Arguments defaulted/auto-determined

=over

=item flexclone

A random name is generated if the flexclone name is not provided.

=item junction-path

If the junction-path argument has not been provided, then this method
automatically chooses a junction-path (currently is is /$flexclone_name) and
mounts it. Note that this is done only if:

* The test is being run on a CMode system

* The volume is of type "RW". Note that if C<type> is not provided, then ONTAP
defaults it to "RW". (Volumes of type "DP" and "DC" can be mounted, but they
cannot be mounted at creation time since they require certain other steps
to be performed before they can be mounted)

If we do not want the method to auto-assign a junction-path, then a value
of undef should be passed for this argument.

    NACL::STask::VolumeClone->create(%other_opts, 'junction-path' => undef);

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    # We cannot directly use _if_exists_validate_spec since it defaults
    # the value to "die". This would mean that we would not know whether
    # if_exists or nacltask_if_exists was provided.
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            nacltask_if_exists => $pkg->_if_exists_validate_spec(),
            nacltask_wait      => { type => SCALAR, default => 1 },
            _was_created       => { type => SCALARREF, optional => 1 },
            _optional_scalars(qw(flexclone)),
            $pkg->_cleanup_validate_spec(),
        },
        allow_extra => 1,
    );

    # Transform %opts from the options we received into the options to
    # pass to the base class method.

    my $if_exists = delete $opts{nacltask_if_exists};

    my $wait        = delete $opts{nacltask_wait};
    my $was_created = delete $opts{_was_created};
    $$was_created = 0;

    my ( %opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup );
    $pkg->_move_common_cleanup_opts(
        source => \%opts,
        target => \%opts_for_cleanup,
    );

    my $random_name_generated;
    if ( !defined $opts{flexclone} ) {
        $opts{flexclone} = $pkg->random_name_generator();
        $random_name_generated = 1;
    }

    my %wait_for_creation_opts;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%wait_for_creation_opts,
        copy   => [qw(method-timeout)]
    );

    $pkg->_hash_move(
        source => \%opts,
        target => \%wait_for_creation_opts,
        move   => [qw(polling_interval)]
    );
    
    my $junction_path_generated;
    if (exists $opts{'junction-path'}) {
        # If junction-path was passed as undef, then delete it so that
        # the component validation does not complain.
        delete $opts{'junction-path'} if (!defined $opts{'junction-path'});
    } else {
        if ($opts{command_interface}->is_cmode()
            && (!defined $opts{type} || $opts{type} eq 'RW')) {
            $junction_path_generated = 1;
            # If junction-path is not provided, it is a CMode system,
            # and an RW clone is being created, then auto-assign a
            # junction-path
            $opts{'junction-path'}
                = $pkg->_determine_junction_path(volume => $opts{flexclone});
        }
    }


    # We want to know the job ID of the job performed by
    # SUPER::create, whether or not our caller does.
    my $dummy;
    $opts{job_component} ||= \$dummy;

    $opts{'method-timeout'} ||= AGGREGATE_TIMEOUT;

    my $command_interface = $opts{command_interface};

    my $self;
CREATE: {
        use warnings qw(exiting);
        try {
            $self         = $pkg->SUPER::create(%opts);
            $$was_created = 1;
            $self->job_component( ${ $opts{job_component} } );
            if ($wait) {
                $self->wait_for_creation(%wait_for_creation_opts);
            }
        }
        catch NACL::C::Exceptions::VolumeClone::AlreadyExists with {
            my $exception = shift;
            if ($random_name_generated) {
                $Log->trace( 'Random name collision, regenerate a '
                        . 'new name and retry' );
                $opts{flexclone} = $pkg->random_name_generator();
                if ($junction_path_generated) {
                    # Regenerate the junction-path since the flexclone
                    # name has been changed.
                    $opts{'junction-path'} = $pkg->_determine_junction_path(
                        volume => $opts{flexclone});
                }
                no warnings qw(exiting);
                redo CREATE;
            } else {
                $Log->trace('The volume already exists');
                $self = $pkg->_element_exists_handler(
                    create_opts        => \%opts,
                    nacltask_if_exists => $if_exists,
                    exception          => $exception
                );
                if ( !$self ) {
                    no warnings qw(exiting);
                    redo CREATE;
                }
            }
        } catch NACL::C::Exceptions::VolumeClone::FeatureNotLicensed with {
            my $license_obj = NACL::STask::SystemLicense->add(
                command_interface => $opts{command_interface},
                feature           => 'FlexClone'
            );
            no warnings qw(exiting);
            redo CREATE;
        };
    }

    # Register this resource with the Cleanup Manager for cleanup
    $pkg->_copy_common_opts_for_cleanup(
        'source'              => {%opts, %opts_for_cleanup},
        'target'              => \%opts_for_register,
        'nacltask_to_cleanup' => \$nacltask_to_cleanup,
        'to_cleanup'          => 'purge'
    );

    $pkg->_register_for_cleanup(%opts_for_register)
        if ($nacltask_to_cleanup && $$was_created);

    $Log->exit() if $may_exit;
    return $self;
} ## end sub create

sub _default_rng_suffix {
    return 'flexclone';
}

=head2 split

  my $vol_clone_obj = NACL::STask::VolumeClone->split(
      command_interface => $command_interface,
      vserver           => $vserver,
      flexclone         => $cloneName
      nacltask_wait     => $boolean,   #default '1'
      %other_options
  );

  (or)

  $vol_clone_obj->split( nacltask_wait => $boolean );


(Class or instance method) start a volume clone split operation.  This method
provides additional services beyond what's inherent to the product's volume
clone split command, as controlled and described by the new C<nacltask_wait>
options.

Supports CMode CLI, CMode ZAPI and 7Mode CLI.
Note that CMode ZAPI is not supported for BR.

=over

=item Options

=over

=item C<< nacltask_wait => 0|1 >>
(Optional) If true (the default), wait for volume clone split operation to
complete (see "wait_for_split" method, below). If false, do not wait. This
option only valid when option 'foreground' not equal 'true'

=back

=item command_interface, apiset_must, apiset_should, method-timeout,
vserver,flexclone,foreground.

All of the other various options to
L<< NACL::C::VolumeClone->split | lib-NACL-C-VolumeClone-pm/split >>

=back

=cut

sub split {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            nacltask_wait    => { type => SCALAR, default  => 1 },
            'method-timeout' => { type => SCALAR, optional => 1 },
        },
        allow_extra => 1,
    );

    # Transform %opts from the options we received into the options to
    # pass to the base class method.

    my $foreground = $opts{foreground};    # get the 'foreground' value
    my $wait = delete $opts{nacltask_wait};    # get the 'nacltask_wait' value
    my $timeout = $opts{'method-timeout'} || DEFAULT_CLONE_TIMEOUT;

    {
        my $job_ref;

        # We want to know the job ID of the job performed by start
        $opts{job_component} ||= \$job_ref;
    }

    # Always make class method calls so that we don't have to bother
    # about skipping primary keys for instance method call
    my $pkg = ref $pkg_or_obj || $pkg_or_obj;
    try {
        $pkg->SUPER::split(%opts);
    }
    catch NACL::C::Exceptions::VolumeCloneSplit::SplitInProgress with {
        $Log->debug("The clone is in the process of being split");
    };

    # Create an object for the job component created
    $pkg_or_obj->job_component( ${ $opts{job_component} } );

    $opts{job_component} = ${ $opts{job_component} };

    if ($wait) {

      # if wait =1 and foreground is 'false' only then wait for the completion
        $pkg->wait_for_split(%opts);
    }

    $Log->exit() if $may_exit;
} ## end sub split

=head2 wait_for_split

   # For CMode as instance method call
   $vol_clone_obj->wait_for_split();

   # For 7Mode as instance method call
   $vol_clone_obj->wait_for_split();
   
   # For CMode as class method call
   NACL::STask::VolumeClone->wait_for_split(
        command_interface => $command_interface,
        job_component     => $job_component,
        flexclone         => $volume_name,
        vserver           => $vserver,
        %wait_options_optional
   );
   
   # For 7Mode as class method call
   NACL::STask::VolumeClone->wait_for_split(
        command_interface => $command_interface,
        flexclone         => $volume_name,
        %wait_options_optional
   );

(Class or Instance method)

This function waits for split operation to complete. The logic for
wait for the completion of operation differs between CMode and 7Mode
and this method abstracts this difference.
For CMode, it waits on the job to complete whereas for 7Mode, it
checks the completion of the operation using 'vol clone split status'
command.

Supports CMode CLI, CMode ZAPI and 7Mode CLI.
Note that CMode ZAPI is not supported for BR.

=over

=item Options

=over

=item B<< "method-timeout"=> $scalar >>

(Optional) The timeout value for completion of clone split operation.
Defaults to 500 sec

=item B<< "polling_interval"=> $scalar >>

(Optional) The polling time interval value.
Defaults to 5 sec

=item B<< job_component => $job_component >>

(Required only for class method and if the mode is CMode,
Not Applicable for instance method) 
This is the job component object obtained from the clone split job from 
    NACL::STask::VolumeClone->split(..., nacltask_wait => 0,
        job_component => \$job_component);

=item B<< flexclone=>$flexclone_name >>

(Required only for class method, Not Applicable for instance method) 
Name of flexclone.

=item B<< vserver=>$vserver >>

(Required only for class method, Not Applicable for instance method) 
Name of vserver.

=item C<< command_interface=>$command_interface >>

(Required only for class method, Not Applicable for instance method) 
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub wait_for_split {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            job_component => {
                optional  => 1,
                type      => OBJECT | UNDEF,
                callbacks => {
                    'value specified to job_component should be of '
                        . 'type NACL::C::Job' => sub {
                        if ( $_[0] ) {
                            return $_[0]->isa('NACL::C::Job');
                        } else {

                            # UNDEF is an accepted value considering 7Mode
                            return 1;
                        }
                        }
                }
            },
            polling_interval =>
                { type => SCALAR, default => CLONE_POLL_DELTA },
            'method-timeout' =>
                { type => SCALAR, default => DEFAULT_CLONE_TIMEOUT },
        },
    );
    my $ci = $opts{command_interface};
    if ( $ci->mode eq 'CMode' ) {
        $pkg_or_obj->_wait_for_split_CM(%opts);
    } elsif ( $ci->mode eq '7Mode' ) {
        $pkg_or_obj->_wait_for_split_7M(%opts);
    } else {

        # Just to avoid open if-elsif
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Filer mode '" . $ci->mode() . "' not supported." );
    }
    $Log->exit() if $may_exit;
} ## end sub wait_for_split

sub _wait_for_split_CM {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, %opts ) = @_;

    my $job_component = delete $opts{job_component};

    # Get hold of the job component
    if ( ref($pkg_or_obj) ) {
        $job_component ||= $pkg_or_obj->job_component();
    }

    my $pkg = ref $pkg_or_obj || $pkg_or_obj;

    if ($job_component) {
        $Log->trace("Wait for completion");
        $pkg->wait_for_completion(
            %opts,
            job_component => $job_component,

            # Needs to be included as mandatory if using wait_for_completion
            # to wait on job
            'check_job_and_attribute' => 0,
        );
    } else {
        $Log->exit() if $may_exit;

        NATE::Exceptions::Argument->throw(
            text => "Error, No Job Component was passed\n"
                . "please make sure you pass job_component for wait_for_split method"
        );
    }

    $Log->exit() if $may_exit;

} ## end sub _wait_for_split_CM

sub _wait_for_split_7M {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, %opts ) = @_;
    my $polling_interval = delete $opts{polling_interval};
    my $timeout          = $opts{'method-timeout'};

    # just in case if this option exists in the input args
    delete $opts{job_component};
    my $is_done = 0;
    my %pri_keys_filter;
    $pkg_or_obj->_hash_move(
        source => \%opts,
        target => \%pri_keys_filter,
        move   => [ $pkg_or_obj->get_primary_keys() ]
    );
    my $end_time = time() + $timeout;
    while ( time() < $end_time ) {
        try {
            NACL::C::VolumeCloneSplit->find(
                filter => \%pri_keys_filter,
                %opts
            );

            # Entry exists, so split is ongoing
            Tharn::sleep($polling_interval);
        }
        catch NACL::C::Exceptions::VolumeCloneSplit::NotAClone with {
            my $ex = shift;
            $is_done = 1;
        };
        last if ($is_done);
    }
    unless ($is_done) {
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::TimeoutException->throw(
            "Split operation hasn't reached completion even after waiting for"
                . " $timeout seconds" );
    }
    $Log->exit() if $may_exit;
} ## end sub _wait_for_split_7M

=head2 Other Volume methods

All functionality available for volumes are applicable for volume clones
as well. This module allows for direct reuse of the methods defined under
NACL::STask::Volume. Taking the example of C<purge>, it is possible to do:

    NACL::C::VolumeClone->purge(
        command_interface => $ci,
        flexclone         => $flexclone_name,
        vserver           => $vserver_name,
        %attributes_to_modify
    );

                    or

    $vol_clone_obj->purge(%attributes_to_modify);

See L<NACL::STask::Volume|lib-NACL-STask-Volume-pm> for a description of the methods
available.

Also refer to L<NACL::C::VolumeClone|lib-NACL-C-VolumeClone-pm> for more information
on how the components/tasks are linked.

=head2 wait_for_creation

    # Instance method: wait for volume clone to get created and come online
    $volume_clone->wait_for_creation();

    # Class method: wait for volume clone to get created and come online
    NACL::STask::VolumeClone->wait_for_creation(
        "method-timeout"  => $timeout,
        command_interface => $command_interface,
        vserver           => $vserver,
        flexclone         => $clone_vol,
        job_component     => $job_component,
    );

Wait for the volume clone creation to complete.  If a job for creating this
volume clone is known (because the mode is C-mode and
NACL::STask::VolumeClone->create was used to create this object, or because
it was passed in using the "job_component" option) then this waits for
that job.

=over

=item Options

=over

=item C<< job_component => $job_component >>

(Optional for class method, Not Applicable for instance method)
A NACL::C::Job object representing the C-mode Job used to
create this volume.

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< flexclone => $flexclone >>

(Required for class method, Not Applicable for instance method)
Name of volume.

=item C<< vserver=>$vserver_name >>

(Required for C-mode for class method, ignored for 7-mode, Not
Applicable for instance method)
Name of vserver containing parent volume.

=item C<< "method-timeout"=>$timeout >>

(Optional) How long in seconds to wait for the job to finish or for
the state to become "online".  Defaults to 7200 (2 hours).

=item C<< polling_interval => $interval >>

(Optional)
The interval, in seconds, at which to either show the progress of the job (if
we're watching its progress) or the interval at which to poll the value of the
"state" field. This defaults to 10 seconds if not provided.

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub _determine_junction_path {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => { volume => { type => SCALAR | OBJECT } },
        allow_extra => 1
    );
    my $vol = $opts{volume};
    if (ref $vol) {
        if (blessed ($vol) && $vol->isa('NACL::C::VolumeClone')) {
            $opts{volume} = $vol->flexclone();
        } else {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw("Value of 'flexclone' should "
                . 'either be a string or an object which is a sub-class '
                . "of NACL::C::VolumeClone but instead a '"
                . ref ($vol)
                . "' was provided"
            );
        }
    }
    my $path = NACL::STask::Volume->_determine_junction_path(%opts);
    $Log->exit() if $may_exit;
    return $path;
}

sub parents {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;
    return qw(NACL::STask::Volume);
}

1;
