#
# Copyright (c) 2013-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeEfficiency Single Task Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::VolumeEfficiency;

use strict;
use warnings;

use base qw(NACL::C::VolumeEfficiency NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Time qw(timeout2time);
use Params::Validate qw(:all);
use NATE::Exceptions::Argument qw(:try);
use NACL::GeneralUtils qw( nacl_method_retry);
use NACL::APISet::Exceptions::CommandFailedException ();
use NACL::APISet::Exceptions::TimeoutException;
use NACL::Exceptions::UnexpectedState;
use NACL::C::Exceptions::VolumeEfficiency::OperationCurrentlyActive qw(:try);
use NACL::C::Exceptions::VolumeEfficiency::RevertToFailedUndoCompression;
use NACL::C::Exceptions::VolumeEfficiency::RevertToFailedSnapshotCopied;
use NACL::C::Exceptions::VolumeEfficiency::PromoteDeprioritizedVolume;
use NACL::STask::VolumeSnapshot;

# Exceptions
use NACL::Exceptions::Timeout;

# The "wait_till_stopped" method is identical to "wait_for_status" so make
# it an alias.
no strict 'refs';
*wait_till_stopped = *wait_for_status;
use strict 'refs';

=head1 NAME

NACL::STask::VolumeEfficiency

=head1 DESCRIPTION

C<NACL::STask::VolumeEfficiency> is a single task library that inherits from
C<NACL::C::VolumeEfficiency> component library to extend the functionality
of VolumeEfficiency object.

It provides methods related to Storage Efficiency in ONTAP in addition to
the C<NACL::C::VolumeEfficiency> methods. This also means that
a C<NACL::STask::VolumeEfficiency> object may generally be used in
place of a component object. See C<NACL::C::VolumeEfficiency> for details.

=cut

use Class::MethodMaker [
    'scalar' => [{-type => 'NACL::C::Job'}, 'job_component'],
];

=head1 ATTRIBUTES

=head2 command_interface

(Required) A C<NACL::C::CommandInterface> component object that represents
the host to which to send commands.

=head2 volume

(Required) The name of the volume.

=head2 vserver

(Required for C-mode, ignored for 7-mode) The name of the vserver containing
the volume.

=head2 job_component

(Optional) A job component (NACL::C::Job) representing the last job that
operated on this volume.

=head1 METHODS

=head2 modify

Documentation for the above mentioned component methods can be obtained from
L<< NACL::C::VolumeEfficiency | lib-NACL-C-VolumeEfficiency-pm >>

nacltask_if_enabled is used to suppress the error if sis is already
enabled, So only  applicable for on()
=head2 on

    my $vol_efficiency_obj = NACL::STask::VolumeEfficiency->on(
        command_interface   => $ci,
        volume              => $vol_name,
        vserver             => $vs_name,
        nacltask_if_enabled => $ignore_enabled,
        nacltask_to_cleanup => 1,
        %other_options
    );


(Class method) The method to enable efficiency on the volume. This method
will return a NACL::STask::VolumeEfficiency object.
By default this method will ignore the error obtained if volume efficiency has
already been enabled.

For infinite volumes a job is started. This method automatically waits for
the job to get completed (this can be controlled by using the C<nacltask_wait>
argument). The default timeout is 300 seconds (configurable by using the
C<method-timeout> argument).

=over

=item Options

=over

=item C<< volume => $vol_name >>

 (Required) The name of the volume on which efficiency has to be enabled.

=item C<< vserver => $vs_name >>

 (Required for c-mode, ignored for 7-mode)
 (Required) The name of the vserver containing the volume.

=item C<< ignore_enabled => 0|1 >>

 This option has been deprecated and replaced by nacltask_if_enabled

=item C<< nacltask_if_enabled => pass|die >>

 nacltask_if_enabled is used to suppress the error if sis is already
 enabled, So only applicable for on()

 (Optional) If the value is 'pass',then method will ignore
 the error obtained if volume efficiency has already been enabled.
 When set to 'die', this method will fail if the error volume efficiency
 has already been enabled is obtained.

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, defaults to 0(no to cleanup))
Flag indicating if this operation is to be cleaned up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=item C<< command_interface => $command_interface >>

 (Required for class method, Not Applicable for instance method)
 See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< nacltask_wait => 0 | 1 >>

(Defaults to 1; Applicable for Infinite Volumes)
For Infinite Volumes, a job is started. If this argument is 1 (the default)
then the method waits till the job completes.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VolumeEfficiency::OperationAlreadyEnabled>

This type of exception is thrown when an attempt is made to enable volume efficiency when it is already enabled.

=back

=back

=cut

sub on {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            ignore_enabled      => {type => SCALAR, optional => 1},
            nacltask_if_enabled => {
                type      => SCALAR,
                callbacks => $pkg_or_obj
                    ->_if_action_completed_already_validate_callback(
                    name => 'nacltask_if_enabled'
                    ),
                optional => 1
                },
            $pkg_or_obj->_cleanup_validate_spec(),
            nacltask_wait => { type => SCALAR, default => 1 },

            # See comment in the "if $wait" block for why 300s
            'method-timeout' => { type => SCALAR, default => 300 },
        },
        allow_extra => 1,
    );
    my $ignore_enabled = delete $opts{ignore_enabled};
    my $if_enabled     = delete $opts{nacltask_if_enabled};
    unless (defined $if_enabled) {
        if (defined $ignore_enabled) {
            $Log->warn("The argument 'ignore_enabled' is deprecated "
                    . 'in the call to NACL::STask::VolumeEfficiency->on. '
                    . "Please replace it with 'nacltask_if_enabled'");
            $if_enabled = $ignore_enabled ? 'pass' : 'die';
        } else {
            $if_enabled = 'pass';
        }
    }

    my $wait = delete $opts{nacltask_wait};

    $pkg_or_obj->_set_job_infinivol_ref(opts_ptr => \%opts);

    my ( %opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup );

    $pkg_or_obj->_move_common_cleanup_opts(
        source => \%opts,
        target => \%opts_for_cleanup,
    );

    my ( $self, $was_enabled );
    try {
        $self = $pkg_or_obj->SUPER::on(%opts);
        $was_enabled = 1;

        my $job = ${$opts{job_component}};
        if (defined $job) {
            $pkg_or_obj->job_component($job) if ref $pkg_or_obj;
            if ($wait) {
                # Job is started for infinivols. Since it's just enabling
                # sis and not really running the operation itself, it doesn't
                # really take much time. In my testing, it completed almost
                # instantaneously. For this reason, it's not really necessary
                # to run extra commands simply to figure out a good timeout
                # value; a slightly higher timeout value (set to 300 above)
                # should suffice, even if the filer is loaded.
                my %common_opts;
                $pkg_or_obj->_copy_common_component_params(
                    source => \%opts,
                    target => \%common_opts
                );
                $job->wait_on_job(%common_opts);
            }
        }
    }
    catch NACL::C::Exceptions::VolumeEfficiency::OperationAlreadyEnabled
        with {
        my $exception = $_[0];
        if ($if_enabled =~ /pass/) {
            my $primary_keys = $pkg_or_obj->get_primary_keys_options(%opts);
            $pkg_or_obj->_process_options(options => $primary_keys);
            $self
                = $pkg_or_obj->get_package_name()
                ->new(%$primary_keys,
                command_interface => $opts{command_interface});
            $self->_update_infinivol_val_in_self(%opts);
        } else {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
        };

    # Register this resource with the Cleanup Manager for cleanup
    $pkg_or_obj->_copy_common_opts_for_cleanup(
        'source'              => {%opts, %opts_for_cleanup},
        'target'              => \%opts_for_register,
        'nacltask_to_cleanup' => \$nacltask_to_cleanup,
        'to_cleanup'          => 'off'
    );

    $pkg_or_obj->_register_for_cleanup(%opts_for_register)
        if ($nacltask_to_cleanup && $was_enabled);

    $Log->exit() if $may_exit;
    return $self;
} ## end sub on

=head2 off

      NACL::STask::VolumeEfficiency->off(
        command_interface => $ci,
        volume            => $vol_name,
        vserver           => $vs_name,
        ignore_disabled   => $ignore_disabled,
        %other_options
      );

      or

      $eff_obj->off(...);

 (Class or Instance method) The method to disable efficiency on the volume.
 By default this method will ignore the error obtained if volume efficiency has
 already been disabled.


=over

=item Options

=over

=item C<< volume => $vol_name >>

 (Required) The name of the volume on which efficiency has to be disabled.

=item C<< vserver => $vs_name >>

 (Required for c-mode, ignored for 7-mode)
 (Required) The name of the vserver containing the volume.

=item C<< ignore_disabled => 0|1 >>

 (Optional) Flag which when set to '1' will make the method ignore
 the error obtained if volume efficiency has already been disabled.
 When set to '0', this method will fail if the error volume efficiency
 has already been disabled is obtained.

=item C<< command_interface => $command_interface >>

 (Required for class method, Not Applicable for instance method)
 See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< "method-timeout" => $timeout >>

(Optional) As component method-timeout, controls how long the command
will wait before completing. However, the default timeout has been
raised to 1800 seconds.

=item C<< apiset_must => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=over

=item Exceptions

=over

=item Component exceptions

See the "Exceptions" section under L<< NACL::C::VolumeEfficiency->off|lib-NACL-C-VolumeEfficiency-pm/off >>.

=back

=back

=cut

sub off {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            ignore_disabled => {type => SCALAR, default => 1},

            # Need to come up with an algorithm to determine a reasonable
            # timeout value rather than have this hard-coded default.
            'method-timeout' => {type => SCALAR, default => 1800},
        },
        allow_extra => 1,
    );

    my $ignore_disabled = delete $opts{ignore_disabled};

    my $job;
    $opts{job_component} ||= \$job;

    my $count = 0;
    my $eff_stop = sub {
        my %additional_args = @_;
        my %primary_keys;
        $pkg_or_obj->_copy_primary_keys(
            source => \%opts,
            target => \%primary_keys
        );

        my %common_opts;
        $pkg_or_obj->_copy_common_component_params_with_ci(
            source => \%opts,
            target => \%common_opts
        );

        $additional_args{nacltask_if_stopped}="pass";
        $additional_args{ignore_disabled}=$ignore_disabled;

        NACL::STask::VolumeEfficiency->stop(%primary_keys,
            %common_opts, %additional_args);
    };

    my $redo_sub = sub {
        no warnings 'exiting';
        redo AGAIN;
    };

    AGAIN: {
        use warnings 'exiting';
        try {
            $pkg_or_obj->SUPER::off(%opts);
            my $final_job = ${$opts{job_component}};
            if (defined $final_job) {
                my %common_opts;
                $pkg_or_obj->_copy_common_component_params(
                    source => \%opts,
                    target => \%common_opts
                );
                $final_job->wait_on_job(%common_opts);
            }
        } catch NACL::C::Exceptions::VolumeEfficiency::OperationCurrentlyActive
          with {
            my $exception = $_[0];
            $count++;

            # Retry after running Stop
            if ($count == 1) {
                $eff_stop->();
            } else {
                $exception->set_text("After trying to stop Efficiency, "
                . "the method still failed. Original failure:\n"
                . $exception->text());
                $exception->throw();
            }

            $redo_sub->();
        }
        except {
            my $sub = sub {
                my $exception = $_[0];
                if (!$ignore_disabled) {
                    $Log->exit() if $may_exit;
                    $exception->throw();
                }
            };

            return {
                'NACL::C::Exceptions::VolumeEfficiency::OperationAlreadyDisabled'
                 => $sub,
                'NACL::C::Exceptions::VolumeEfficiency::OperationNotEnabled'
                 => $sub,
            };

         };

    }

    $Log->exit() if $may_exit;
} ## end sub off

=head2 start

  NACL::STask::VolumeEfficiency->start(
        command_interface   => $ci,
        volume              => $vol_name,
        vserver             => $vs_name,
        nacltask_wait       => $wait,
        polling_interval    => $interval,
        nacltask_to_cleanup => 1,
        %other_options
      );

  or

  $eff_obj->start(...);

 (Class or instance method) The method to start efficiency operation on the
 volume and wait for it to complete.
 By default this method will wait for the volume efficiency operation on a
 volume to complete (status to become IDLE).


=over

=item Options

=over

=item C<< volume => $vol_name >>

 (Required for class method, Not Applicable for instance method)
 The name of the volume on which the method starts the volume efficiency
 operation and then waits for the operation to complete.

=item C<< vserver => $vs_name >>

 (Required for class method, Not Applicable for instance method)
 (Required for c-mode, ignored for 7-mode)
 The name of the vserver containing the volume.

=item C<< nacltask_wait => 0|1 >>

 (Optional) Flag which when set to '1' will make the method to wait for
 efficiency operation to complete else when set to '0' will not wait for
 completion after starting the efficiency operation.
 Defaults to 1.

=item C<< polling_interval => $interval >>

 (Optional) The interval after which the node will be polled to verify
 the efficiency state.
 Defaults to 10 seconds.

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, defaults to 0(no to cleanup))
Flag indicating if this operation is to be cleaned up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=item C<< command_interface => $command_interface >>

 (Required for class method, Not Applicable for instance method)
 See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            nacltask_wait    => { type => SCALAR, default => 1 },
            polling_interval => { type => SCALAR, default => 10 },
            $pkg_or_obj->_cleanup_validate_spec(),
        },
        allow_extra => 1,
    );

    my ( %opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup );
    $pkg_or_obj->_move_common_cleanup_opts(
        source => \%opts,
        target => \%opts_for_cleanup,
    );

    # Transform %opts from the options we received into the options to
    # pass to the base class method.

    my $wait     = delete $opts{nacltask_wait};
    my $interval = delete $opts{polling_interval};

    $pkg_or_obj->_set_job_infinivol_ref(opts_ptr => \%opts);

    $pkg_or_obj->SUPER::start(%opts);
    $pkg_or_obj->_start_stop_wait(
        %opts,
        polling_interval => $interval,
        nacltask_wait    => $wait
    );

    # Register this resource with the Cleanup Manager for cleanup
    $pkg_or_obj->_copy_common_opts_for_cleanup(
        'source'              => {%opts, %opts_for_cleanup},
        'target'              => \%opts_for_register,
        'nacltask_to_cleanup' => \$nacltask_to_cleanup,
        'to_cleanup'          => '_stop'
    );

    $pkg_or_obj->_register_for_cleanup(%opts_for_register)
        if ($nacltask_to_cleanup);

    $Log->exit() if $may_exit;
} ## end sub start

=head2 stop

    $Vol_efficiency->stop();

            (or)

    NACL::STask::VolumeEfficiency->stop(
      command_interface => $command_interface,
      volume            => $volume_name,
      vserver           => $vserver_name,
      ignore_disabled   => $ignore_disabled,
      %other_options
  );

(Class or Instance method) Stops the Volume efficiency for the volume.
The efficiency state of the volume should not be 'idle' at the time of
command execution.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required for class method, Not Applicable for instance method)
A component object that represents the host which to send commands.
See NACL::C::Component::command_interface

=item C<< vserver => $vserver_name >>

(Required for class method, Not Applicable for instance method)
 The name of the vserver.

=item C<< volume=>$vol_name >>

(Required, if path is not passed) Name of the volume.

=item C<< ignore_disabled => 0|1 >>

 (Optional) Flag which when set to '1' will make the method ignore
 the error obtained if volume efficiency has already been disabled or
 it was never enabled.
 When set to '0'(default), this method will fail if the error volume efficiency
 has already been disabled/not enabled is obtained.

=item C<< path=>$volpath >>

(Required, if volume is not passed) The volume path name.

=item C<< nacltask_wait => $nacltask_wait >>

(Optional)
If '0', wait action will not be performed.
If '1' (default), the polling takes place until the efficiency value is idle.

=item C<< "method-timeout" => $timeout >>

(Optional) As component method-timeout, controls how long the command
will wait before completing.  However, the default timeout has been
raised to 600 seconds.

=item C<< nacltask_if_stopped => die | pass >>

(Optional, defaults to "die")
Specifies what to do if the volume efficiency for the volume is already stopped.
A value of C<die> (the default) will result in an exception being thrown.
A value of C<pass> will mean that the exception that would normally occur
when we're trying to stop a volume efficiency that was already stopped is
suppressed.

command_interface, apiset_must, apiset_should, etc. All of
the other various options to NACL::C::VolumeEfficiency->stop.

=back

=back

=over

=item Exceptions

=over

=item Component exceptions

See the "Exceptions" section under L<< NACL::C::VolumeEfficiency->stop|lib-NACL-C-VolumeEfficiency-pm/stop >>.

=back

=back

=cut


sub stop {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            ignore_disabled => {type => SCALAR, default => 0},
            'method-timeout' => { type => SCALAR,  default => 600 },
            nacltask_wait    => { type => BOOLEAN, default => 1 },
            nacltask_if_stopped =>
                $pkg_or_obj->_if_action_completed_already_validate_spec(
                    name=>'nacltask_if_stopped'),
        },
        allow_extra => 1,
    );
    my $wait                = delete $opts{nacltask_wait};
    my $interval            = delete $opts{polling_interval};
    my $nacltask_if_stopped = delete $opts{nacltask_if_stopped};
    my $ignore_disabled     = delete $opts{ignore_disabled};

    $pkg_or_obj->_set_job_infinivol_ref(opts_ptr => \%opts);

    try {
        $pkg_or_obj->SUPER::stop(%opts);
        $pkg_or_obj->_start_stop_wait(
            %opts,
            polling_interval => $interval,
            nacltask_wait    => $wait
        );
    } catch NACL::C::Exceptions::VolumeEfficiency::OperationCurrentlyIdle with {
        my $exception = $_[0];
        if ($nacltask_if_stopped =~ /die/) {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
    } except {
            my $sub = sub {
                my $exception = $_[0];
                if (!$ignore_disabled) {
                    $Log->exit() if $may_exit;
                    $exception->throw();
                }
            };

            return {
                'NACL::C::Exceptions::VolumeEfficiency::OperationAlreadyDisabled'
                 => $sub,
                'NACL::C::Exceptions::VolumeEfficiency::OperationNotEnabled'
                 => $sub,
            };

     };
    $Log->exit() if $may_exit;
} ## end sub stop

=head2 undo

  NACL::STask::VolumeEfficiency->undo(
        command_interface => $ci,
        volume            => $vol_name,
        vserver           => $vs_name,
        nacltask_wait     => $wait,
        polling_interval  => $interval,
        %other_options
      );

  or

  $eff_obj->undo(...);

 (Class or instance method) The method to undo efficiency operation on the
 volume.
 By default this method will wait for the 'volume efficiency undo' operation on a
 volume to complete(default method-timeout is 1800 seconds). 'volume efficiency off' will be done prior to undo efficiency
 operation. Both CMode CLI and 7mode CLI are supported.


=over

=item Options

=over

=item C<< volume => $vol_name >>

 (Required for class method, Not Applicable for instance method)
 The name of the volume on which the method starts the volume efficiency
 operation and then waits for the operation to complete.

=item C<< vserver => $vs_name >>

 (Required for class method, Not Applicable for instance method)
 (Required for c-mode, ignored for 7-mode)
 The name of the vserver containing the volume.

=item C<< nacltask_wait => 0|1 >>

 (Optional) Flag which when set to '1' will make the method to wait for
 efficiency undo operation to complete else when set to '0' will not wait for
 completion after starting the undo efficiency operation.
 Defaults to 1.

=item C<< polling_interval => $interval >>

 (Optional) The interval after which the node will be polled to verify
 the efficiency state.
 Defaults to 10 seconds.

=item C<< command_interface => $command_interface >>

 (Required for class method, Not Applicable for instance method)
 See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item Other options

All other options supported by
L<< NACL::C::VolumeEfficiency->undo|lib-NACL-C-VolumeEfficiency-pm/undo >>
are supported by NACL::STask::VolumeEfficiency->undo as well.

=back

=back

=cut

sub undo {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {nacltask_wait => {type => SCALAR, default => 1}},
        allow_extra     => 1,
    );

    # Transform %opts from the options we received into the options to
    # pass to the base class method.
    my $wait = delete $opts{nacltask_wait};
    my %common_opts_with_ci;
    $pkg_or_obj->_copy_common_component_params_with_ci(
        source => \%opts,
        target => \%common_opts_with_ci
    );
    my %primary_keys;
    $pkg_or_obj->_copy_primary_keys(
        source => \%opts,
        target => \%primary_keys
    );

    $pkg_or_obj->_set_job_infinivol_ref(opts_ptr => \%opts);

UNDO: {
        use warnings 'exiting';
        try {
            $pkg_or_obj->SUPER::undo(%opts);
        } catch NACL::C::Exceptions::VolumeEfficiency::OperationNotDisabled with {
            $pkg_or_obj->off(%common_opts_with_ci, %primary_keys);
            no warnings 'exiting';
            redo UNDO;
        };

        if ($wait) {
            my %other_args;
            if (defined ${$opts{job_component}}) {
                $other_args{job_component} = ${$opts{job_component}};
            }
            my $is_infinivol = $pkg_or_obj->_read_infinivol_val(%opts);
            $other_args{is_infinivol} = $is_infinivol
                if (defined $is_infinivol);
            if ( (defined $opts{compression} && $opts{compression} eq 'true') || (defined $opts{'cross-volume-dedupe'} && $opts{'cross-volume-dedupe'} eq 'true') || (defined $opts{'data-compaction'} && $opts{'data-compaction'} eq 'true') ) {
                $other_args{'check_for_idle_status'} = 1;
            }
           $pkg_or_obj->wait_for_undo(%primary_keys, %common_opts_with_ci,
                %other_args);
        }
    }

    $Log->exit() if $may_exit;
} ## end sub undo

=head2 wait_for_undo

  NACL::STask::VolumeEfficiency->wait_for_undo (
    command_interface => $ci,
    volume            => $vol_name,
    vserver           => $vs_name,
    polling_interval  => $interval,
    method-timeout    => $timeout,
    %other_options
   );

  or

  $eff_obj->wait_for_undo(...);

(Class or instance method) The method to wait for the efficiency undo operation to
complete.


=over

=item Options

=over

=item C<< volume => $vol_name >>

 (Required for class method, Not Applicable for instance method)
 The name of the volume on which the method waits for the volume efficiency
 operation to reach a desired state.

=item C<< vserver => $vs_name >>

 (Required for class method, Not Applicable for instance method)
 (Required for c-mode, ignored for 7-mode)
 The name of the vserver containing the volume.

=item C<< polling_interval => $interval >>

 (Optional) The interval after which the node will be polled to verify
 the efficiency state.
 Defaults to 10 seconds.

=item C<< is_infinivol => 0 | 1 >>

(Optional)
This argument can be used to specify whether the operation was performed on an
infinite volume. (For infinite volumes, it is necessay to wait for the
efficiency operations of all of its constituent volumes to be undone)

=item C<< job_component => $job_component >>

(Optional)
If the undo operation started a job, it can be provided through this argument.
The method will then wait for the job to complete.

=item C<< method-timeout => $timeout >>

 (Optional) The time period after which method reports error if the efficiency
 operation does not reach the desired state on the specific volume.
 Defaults to 1800 seconds.

=item C<< command_interface => $command_interface >>

 (Required for class method, Not Applicable for instance method)
 See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=over

=item Exceptions

=over

=item C<NACL::APISet::Exceptions::TimeoutException>

This type of exception is thrown when timeout occured while waiting for 'sis undo' operation to complete.

=back

=back

=cut

sub wait_for_undo {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            polling_interval => {type => SCALAR,         default  => 10},
            'method-timeout' => {type => SCALAR,         default  => 1800},
            is_infinivol     => {type => SCALAR,         optional => 1},
            job_component    => {type => OBJECT | UNDEF, optional => 1},
            check_for_idle_status => {type => SCALAR,    default => 0},
        },
        allow_extra => 1,
    );
    my $interval     = delete $opts{polling_interval};
    my $is_infinivol = delete $opts{is_infinivol};
    my $check_for_idle_status = delete $opts{'check_for_idle_status'};

    my $ci           = $opts{command_interface};
    $opts{'method-timeout'}
        = $ci->scale_timeout('method-timeout' => $opts{'method-timeout'});

    if (!defined $is_infinivol && $ci->is_cmode()) {
        my $volume
            = $pkg_or_obj->_extract_volume_from_path(path => $opts{path});
        $is_infinivol
            = $pkg_or_obj->_determine_is_infinivol(%opts, volume => $volume);
    }

    my %filter = (vserver => delete $opts{vserver});
    my $path = delete $opts{path};

    if ($is_infinivol) {

        # If we're operating on an infinivol, we should wait for the
        # efficiency operations of all of the constituent volumes to be
        # removed.
        $filter{'is-constituent'} = 'true';
    } else {
        $filter{path} = $path;
    }

    my $end_time = timeout2time($opts{'method-timeout'});

    my $job = delete $opts{job_component};
    if (defined $job) {
        my %common_opts;
        $pkg_or_obj->_copy_common_component_params(
            source => \%opts,
            target => \%common_opts
        );
        $job->wait_on_job(%common_opts);
    }
    if ( $check_for_idle_status ) {
         my %primary_keys;
         my %common_opts;
	 $opts{path} = $path;
         $pkg_or_obj->_copy_common_component_params_with_ci(
            source => \%opts,
            target => \%common_opts
         );
         $pkg_or_obj->_copy_primary_keys(
             source => \%opts,
             target => \%primary_keys
         );
         $pkg_or_obj->wait_for_status(%primary_keys, %common_opts);
	 delete $opts{path} if(defined $opts{path});
    } else {
          while (1) {
          my $obj = NACL::CS::VolumeEfficiency->fetch(
              %opts,
              filter           => \%filter,
              requested_fields => ['op-status'],
              allow_empty      => 1
          );
          last if (!$obj);

          if (time() > $end_time) {
              $Log->exit() if $may_exit;
              NACL::Exceptions::Timeout->throw(
                  'After waiting for '
                      . $opts{'method-timeout'}
                      . ' seconds, the undo operation had not yet completed',
                  timeout => $opts{'method-timeout'}
           );
        } else {
            Tharn::snooze($interval);
        }
      }
    }
    $Log->exit() if $may_exit;
} ## end sub wait_for_undo

=head2 wait_for_status

  NACL::STask::VolumeEfficiency->wait_for_status (
    command_interface => $ci,
    volume            => $vol_name,
    vserver           => $vs_name,
    status            => $status,
    polling_interval  => $interval,
    method-timeout    => $timeout,
    %other_options
   );

  or

  $eff_obj->wait_for_status(...);

 (Class or instance method) The method to wait for the efficiency operation to
 reach the desired state. Valid states are IDLE, INITIALIZING, ACTIVE, REVERTING,
 PENDING, DOWNGRADING, DISABLED.
 Defaults to IDLE state of volume efficiency operation.


=over

=item Options

=over

=item C<< volume => $vol_name >>

 (Required for class method, Not Applicable for instance method)
 The name of the volume on which the method waits for the volume efficiency
 operation to reach a desired state.

=item C<< vserver => $vs_name >>

 (Required for class method, Not Applicable for instance method)
 (Required for c-mode, ignored for 7-mode)
 The name of the vserver containing the volume.

=item C<< status => $status >>

 (Optional) The efficiency status. Valid states are Idle, Initializing,
 Active, Reverting, Pending, Downgrading, Disabled.
 Defaults to Idle.

=item C<< polling_interval => $interval >>

 (Optional) The interval after which the node will be polled to verify
 the efficiency state.
 Defaults to 10 seconds.

=item C<< method-timeout => $timeout >>

 (Optional) The time period after which method reports error if the efficiency
 operation does not reach the desired state on the specific volume.
 Defaults to 1800 seconds.

=item C<< command_interface => $command_interface >>

 (Required for class method, Not Applicable for instance method)
 See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=over

=item Exceptions

=over

=item C< NACL::Exceptions::Timeout>

This type of exception is thrown when the efficiency operation on the
volume has not reached the provided status.

=back

=back

=cut

sub wait_for_status {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            polling_interval => { type => SCALAR, default => 10 },
            status           => { type => SCALAR, default => 'Idle' },
            'method-timeout' => { type => SCALAR, default => 1800 },
            job_component    => { type => OBJECT, optional => 1 },
        },
        allow_extra => 1,
    );
    my $status       = delete $opts{status};
    my $is_infinivol = delete $opts{is_infinivol};
    $opts{job_component} ||= $pkg_or_obj->job_component();

    my $volume = $pkg_or_obj->_extract_volume_from_path(path => $opts{path});
    if (!defined $is_infinivol && $opts{command_interface}->is_cmode()) {
        $is_infinivol
            = $pkg_or_obj->_determine_is_infinivol(%opts, volume => $volume);

    }

    if ($is_infinivol) {
        # If performed on an infinivol, then the status shows up as "-"
        # it is necessary to query the status of each of the data
        # constituent volumes and wait for all of them to reach "Idle".
        # Here we filter to only show the constituent volumes
        my %filter = ( vserver => $opts{vserver}, 'is-constituent' => 'true' );
        my %common_opts_ci;
        $pkg_or_obj->_copy_common_component_params_with_ci(
            source => \%opts,
            target => \%common_opts_ci
        );
        my $final_time = timeout2time($opts{'method-timeout'});

        if (defined $opts{job_component}) {
            $opts{job_component}->wait_on_job(
                'method-timeout' => $opts{'method-timeout'},
                %common_opts_ci
            );
        }

        while (1) {
            my @eff_cs = NACL::CS::VolumeEfficiency->fetch(
                %common_opts_ci,
                filter           => \%filter,
                requested_fields => ['op-status']
            );
            my $all_done = 1;
            foreach my $cs (@eff_cs) {
                if ($cs->op_status() !~ /$status/i) {
                    $all_done = 0;
                    last;
                }
            }
            if ($all_done) {
                last;
            } else {
                if (time() > $final_time) {
                    $Log->exit() if $may_exit;
                    NACL::Exceptions::Timeout->throw('After waiting for '
                        . "$opts{'method-timeout'} seconds, the op-status "
                        . 'of all of the constituent volumes of the infinite '
                        . "volume $volume did not reach the status '$status'",
                        timeout => $opts{'method-timeout'}
                    );
                } else {
                    Tharn::snooze($opts{polling_interval});
                }
            }
        }
    } else {
        $pkg_or_obj->SUPER::wait_for_completion(
            %opts,
            attribute_to_check => 'op-status',
            till_value         => qr/$status/i,
        );
    }

    $Log->exit() if $may_exit;
} ## end sub wait_for_status

=head2 wait_for_changelog

  NACL::STask::VolumeEfficiency->wait_for_changelog (
    command_interface => $ci,
    volume            => $vol_name,
    vserver           => $vs_name,
    records           => $records,
    polling_interval  => $interval,
    method-timeout    => $timeout,
    %other_options
   );

  or

  $eff_obj->wait_for_changelog(...);

 (Class or instance method) The method to wait for the changelog metafile
 to hold desired number of fingerprint(FP) records equivalent to the data
 written into the volume.

=over

=item Options

=over

=item C<< volume => $vol_name >>

 (Required for class method, Not Applicable for instance method)
 The name of the volume on which the method waits for the changelog metafile
 to hold desired number of fingerprint (FP) records.

=item C<< vserver => $vs_name >>

 (Required for class method, Not Applicable for instance method)
 (Required for c-mode, ignored for 7-mode)
 The name of the vserver containing the volume.

=item C<< records => $records >>

 (Required) The number of fingerprint records.

=item C<< polling_interval => $interval >>

 (Optional) The interval after which the node will be polled to verify if
 the changelog holds the desired number of fingerprint records.
 Defaults to 10 seconds.

=item C<< method-timeout => $timeout >>

 (Optional) The time period after which method reports error if the changelog
 does not hold the desired number of finerprint records.
 Defaults to 90 seconds.

=item C<< command_interface => $command_interface >>

 (Required for class method, Not Applicable for instance method)
 See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=over

=item Exceptions

=over

=item C<NATE::BaseException>

This type of exception is thrown when Changelog file of a particular volume is not populated with $number of records.

=back

=back

=cut

sub wait_for_changelog() {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            polling_interval => { type => SCALAR, default => 10 },
            records          => { type => SCALAR },
        },
        allow_extra => 1,
    );

    my $node = $opts{command_interface};
    my $path = delete $opts{path};

    my %final;

    my $method_timeout = $opts{'method-timeout'} || 90;
    my $timeout        = time() + $method_timeout;
    my $records        = delete $opts{records};
    my $apiset         = $node->get_7m_or_nodescope_apiset();
    my $interval       = delete $opts{polling_interval};

    my %nodescope;
    if ($node->is_cmode()) {
        my $vol = $pkg_or_obj->_extract_volume_from_path(path => $path);
        $nodescope{'nodescope-node-name'} = NACL::STask::Volume->get_local_node_name(
            volume => $vol,
            %opts
        );
    }

    while ( time() < $timeout ) {
        my $response = $apiset->sis_check(
            changelog => 1,
            path      => $path,
            %nodescope,
        );
        my $ret = $response->get_raw_output();
        my @output = split( "\r*\n", $ret );

        foreach my $line (@output) {
            if (   $line =~ /clog/
                && $line =~ /^Checking\s+(\S+)\s+ .* (\d+)\s+records$/i )
            {
                $final{$1} = $2;
            }
        } ## end foreach my $line (@output)

        my $fp
            = $final{'clogdir.1'} ? $final{'clogdir.1'} : $final{'clogdir.0'};

        if ( $fp == $records ) {
            $Log->exit() if $may_exit;
            return;
        } else {
            Tharn::snooze($interval);
        }
    } ## end while ( time() < $timeout)
    $Log->exit() if $may_exit;
    NATE::BaseException->throw( "Changelog file of Volume $path "
            . "is not populated with $records number of records" );
} ## end sub wait_for_changelog

=head2 wait_for_stage

  NACL::STask::VolumeEfficiency->wait_for_stage(
      command_interface => $ci,
      volume            => $vol_name,
      vserver           => $vs_name,
      stage             => $stage,
      polling_interval  => $interval,
      method-timeout    => $timeout,
      %other_options
  );

  or

  $eff_obj->wait_for_stage(...);

 (Class or instance method) The method to wait for the efficiency operation
 to reach the desired stage of de-duplication. Valid stages of de-duplication
 are - NOP, LOGGING, GATHERING, SORTING, DEDUPING_P1, DEDUPING_P2,
 DEDUPING_SHARING, CHECKING, CHECKING_P0, CHECKING_P1, DONE.

=over

=item Options

=over

=item C<< volume => $vol_name >>

 (Required for class method, Not Applicable for instance method)
 The name of the volume on which the method waits for the volume efficiency
 operation to reach a desired stage of dedupe.

=item C<< vserver => $vs_name >>

 (Required for class method, Not Applicable for instance method)
 (Required for c-mode, ignored for 7-mode)
 The name of the vserver containing the volume.

=item C<< stage => $stage >>

 (Required) The stage of dedupe. Valid stages of dedupe are - NOP,
 LOGGING, GATHERING, SORTING, DEDUPING_P1, DEDUPING_P2, DEDUPING_SHARING,
 CHECKING, CHECKING_P0, CHECKING_P1, DONE.

=item C<< polling_interval => $interval >>

(Optional) The time interval after which the node will be polled to verify the
 dedupe stage.
 Defaults to 10 seconds.

=item C<< method-timeout => $timeout >>

 (Optional) The time period after which method reports error if the efficiency
 operation does not reach the desired stage on the specific volume.
 Defaults to 1800 seconds.

=item C<< command_interface => $command_interface >>

 (Required for class method, Not Applicable for instance method)
 See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub wait_for_stage {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            polling_interval => { type => SCALAR, default => 10 },
            stage            => { type => SCALAR },
        },
        allow_extra => 1,
    );

    my $interval = delete $opts{polling_interval};
    my $timeout  = $opts{'method-timeout'} || 1800;
    my $stage    = delete $opts{stage};

    my $pkg = ref $pkg_or_obj || $pkg_or_obj;
    $pkg->SUPER::wait_for_completion(
        %opts,
        attribute_to_check => 'stage',
        till_value         => $stage,
        'method-timeout'   => $timeout,
        polling_interval   => $interval,
    );
    $Log->exit() if $may_exit;
} ## end sub wait_for_stage

=head2 compare_space_saving

  $vol_eff_obj->compare_space_saving (
              destination_volume    => $dst_vol_specification,
              error_tolerance       => $percent_error,
             );

  (or)

  NACL::STask::VolumeEfficiency->compare_space_saving (
              source_volume         => $src_vol_specification,
              destination_volume    => $dst_vol_specification,
              type                  => 'dedupe',
             );

=over

=item
The implementation and documentation of functionality is present at
L<< NACL::STask::Volume->compare_space_saving|lib-NACL-STask-Volume-pm/compare_space_saving >>.

=back

=cut

sub compare_space_saving {

    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, %opts ) = @_;
    if ( ref $pkg_or_obj ) {
        my @empty;
        my %tmp_opts
            = $pkg_or_obj->_common_validate_with( params => \@empty );

        $tmp_opts{source_volume}  = $pkg_or_obj->get_volume();
        $tmp_opts{source_vserver} = delete $tmp_opts{vserver}
            if ( $tmp_opts{vserver} );
        %opts = ( %opts, %tmp_opts );
    } ## end if ( ref $pkg_or_obj )
    NACL::STask::Volume->compare_space_saving(%opts);

    $Log->exit() if $may_exit;
} ## end sub compare_space_saving

=head2 get_volume

  my $volume = $voleff_object->get_volume();

  (or)

  my $volume = NACL::STask::VolumeEfficiency->get_volume( path => $vol_path);

  (Class or Instance method) The method constructs and returns the volume name
from the path parameter of VolumeEfficiency Class/Object.

=over

=item Options

=over

=item C<< path=>$string >>

 (Required for class method, Not Applicable for instance method)
  Path to the volume in the format /vol/vol_name.

=back

=back

=over

=item Exceptions

=over

=item C<NATE::Exceptions::Argument>

This type of exception is thrown when input 'path' is not in the expected format.

=back

=back

=cut

sub get_volume {

    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my ( $volume, @opts );
    if ( ref $pkg_or_obj ) {
        my @empty;
        @opts = $pkg_or_obj->_common_validate_with( params => \@empty );
    }
    push @opts, @args;
    my %opts = NACL::C::Component->_common_validate_with(
        params               => \@opts,
        no_command_interface => 1,
        additional_spec => { path => { type => SCALAR, optional => 0 }, },
        allow_extra     => 1,
    );
    if ( $opts{path} =~ /(.+)\/(.+)/ ) {
        $volume = $2;
    } else {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            'Path input is not in the expected ' . 'format /vol/<volname>' );
    }
    $Log->exit() if $may_exit;
    return $volume;
} ## end sub get_volume

=head2 revert_to

  NACL::STask::VolumeEfficiency->revert_to(
        command_interface   => $ci,
        volume              => $vol_name,
        vserver             => $vs_name,
        nacltask_wait       => $wait,
        polling_interval    => $interval,
        %other_options
      );

  or

  $eff_obj->revert_to(...);

 (Class or instance method) The method is to revert volume efficiency policy
 and wait for it to complete.
 By default this method will wait for the revert operation on a
 volume to complete (status to become IDLE).

=over

=item Options

=over

=item C<< volume => $vol_name >>

 (Required for class method, Not Applicable for instance method)
 The name of the volume on which the method reverts the volume efficiency
 policy and then waits for the operation to complete.

=item C<< vserver => $vs_name >>

 (Required for class method, Not Applicable for instance method)
 (Required for c-mode, ignored for 7-mode)
 The name of the vserver containing the volume.

=item C<< nacltask_wait => 0|1 >>

 (Optional) Flag which when set to '1' will make the method to wait for
 efficiency operation to complete else when set to '0' will not wait for
 completion after reverting the efficiency operation on volume.
 Defaults to 1.

=item C<< polling_interval => $interval >>

 (Optional) The interval after which the node will be polled to verify
 the efficiency state.
 Defaults to 10 seconds.

=item C<< "method-timeout" => $timeout >>

(Optional) As component method-timeout, controls how long the command
will wait before completing. However, the default timeout has been
raised to 600 seconds.

=item C<< command_interface => $command_interface >>

 (Required for class method, Not Applicable for instance method)
 See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub revert_to {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            "method-timeout"           => { type => SCALAR,  default => 600 },
            polling_interval           => { type => SCALAR,  default => 20 },
            nacltask_wait              => { type => BOOLEAN, default => 1 },
            nacltask_retry_tries_count => { type => SCALAR,  default => 5 },
            nacltask_retry_sleep_time  => { type => SCALAR,  default => 30 },
            nacltask_retry_num_times_code_ran =>
                { type => SCALARREF, optional => 1 },
        },
        allow_extra => 1,
    );

    my %nacltask_options;
    $pkg_or_obj->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_options
    );

    my %retry_opts;
    $pkg_or_obj->_copy_retry_params(
        source => \%opts,
        target => \%retry_opts
    );

    my $nacltask_wait = $nacltask_options{nacltask_wait};
    my $interval      = delete $opts{polling_interval};

    nacl_method_retry(
        code => sub { $pkg_or_obj->SUPER::revert_to(%opts); },
        exceptions => [ qw(NACL::C::Exceptions::VolumeEfficiency::OperationCurrentlyActive) ],
        %retry_opts,
    );

    if ($nacltask_wait) {
        my %wait_opts;
        $pkg_or_obj->_copy_common_component_params_with_ci(
            source => \%opts,
            target => \%wait_opts
        );
        $pkg_or_obj->_copy_primary_keys(
            source => \%opts,
            target => \%wait_opts,
        );

        try {
            $pkg_or_obj->wait_for_status(
                %wait_opts,
                status           => "Idle",
                polling_interval => $interval,
            );
        }
        catch NACL::Exceptions::NoElementsFound with {
            $pkg_or_obj->_wait_for_sis_status(
                polling_interval => $interval,
                %wait_opts
            );
        };

    }
    $Log->exit() if $may_exit;
} ## end sub revert_to

=head2 config

  NACL::STask::VolumeEfficiency->config(
        command_interface => $ci,
        volume            => $vol_name,
        vserver           => $vs_name,
        %other_options
      );

  or

  $eff_obj->config(...);

(Class or instance method) Method to configure efficiency operation on the
volume. When the option 'policy' is configured  along with,
'compression', 'inline-compression' etc., the method takes care of assigning
the policy first and then configures the other options. This step is an
added advantage over the Component.

Both CMode CLI and 7mode CLI are supported.

=over

=item Options

=over

=item C<< volume => $vol_name >>

 (Required for class method, Not Applicable for instance method)
 The name of the volume on which the method configures the volume efficiency
 option.

=item C<< vserver => $vs_name >>

 (Required for class method, Not Applicable for instance method)
 (Required for c-mode, ignored for 7-mode)
 The name of the vserver containing the volume.

=item C<< command_interface => $command_interface >>

 (Required for class method, Not Applicable for instance method)
 See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item Other options

All other options supported by
L<< NACL::C::VolumeEfficiency->config|lib-NACL-C-VolumeEfficiency-pm/config >>
are supported by NACL::STask::VolumeEfficiency->config as well.

=back

=back

=cut

sub config {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my @args       = @_;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params      => \@args,
        allow_extra => 1
    );
    if ( defined $opts{policy} ) {
        $Log->debug("Policy defined.");
        my $policy = delete $opts{policy};
        my %_policy_opts;
        $pkg_or_obj->_copy_common_component_params_with_ci(
            source => \%opts,
            target => \%_policy_opts
        );

        $pkg_or_obj->_copy_primary_keys(
            source => \%opts,
            target => \%_policy_opts,
        );
        $pkg_or_obj->SUPER::config( %_policy_opts, policy => $policy );
    }
    $pkg_or_obj->SUPER::config(%opts);
    $Log->exit() if $may_exit;

} ## end sub config

=head2 prepare_to_downgrade

  NACL::STask::VolumeEfficiency->prepare_to_downgrade(
        command_interface   => $ci,
	...
      );

  or

  $eff_obj->prepare_to_downgrade(...);

 (Class or instance method) The method prepare_to_downgrade efficiency operation on the
 volume and wait for it to complete.
 By default this method will wait for the volume efficiency prepare_to_downgrade operation to complete.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

 (Required for class method, Not Applicable for instance method)
 See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< "method-timeout" => $timeout >>

(Optional) As component method-timeout, controls how long the command
will wait before completing. However, the default timeout has been
raised to 1800 seconds.

=item C<< apiset_should => $ruleset >>

 (Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub prepare_to_downgrade {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'method-timeout'           => { type => SCALAR,  default => 1800 },
        },
        allow_extra => 1,
    );

    # Transform %opts from the options we received into the options to
    # pass to the base class method.
    delete $opts{'vserver'};
    delete $opts{'path'};

UNDO: {
        use warnings 'exiting';
        try {
	    NACL::C::VolumeEfficiency->prepare_to_downgrade(%opts);
        }
        catch NACL::C::Exceptions::VolumeEfficiency::RevertToFailedSnapshotCopied with {
            my $exception               = shift;
            my %vol_vs_snapshot_details = %{ $exception->get_vol_vs_snapshot_list() };
            my $snapshot_list           = $vol_vs_snapshot_details{'snapshot_list'};

            $Log->comment("Deleting the snapshots one by one");
            foreach my $snapshot (@$snapshot_list) {
                NACL::STask::VolumeSnapshot->delete(
                    command_interface => $opts{command_interface},
                    snapshot          => $snapshot,
                    volume            => $vol_vs_snapshot_details{'volume'},
                    vserver           => $vol_vs_snapshot_details{'vserver'},
                );
            }

            no warnings 'exiting';
            redo UNDO;
        }
        catch NACL::C::Exceptions::VolumeEfficiency::RevertToFailedUndoCompression with {
            my $exception      = shift;
            my %vol_vs_details = %{ $exception->get_vol_vs_list() };

            # first disable compression and idd
            $pkg_or_obj->modify(
                command_interface    => $opts{command_interface},
                volume               => $vol_vs_details{'volume'},
                vserver              => $vol_vs_details{'vserver'},
                'inline-compression' => 'false',
                'compression'        => 'false',
                'idd'                => 'false'
            );

            # now run volume efficiency undo -compression true
            # not waiting here since it waits for the entry for this volume
            # to be removed from the table (which does not happen if we
            # undo only the compression savings)

            $pkg_or_obj->undo(
                command_interface => $opts{command_interface},
                volume            => $vol_vs_details{'volume'},
                vserver           => $vol_vs_details{'vserver'},
                compression       => 'true',
                nacltask_wait     => 0
            );

            $pkg_or_obj->wait_for_status(
                command_interface => $opts{command_interface},
                volume            => $vol_vs_details{'volume'},
                vserver           => $vol_vs_details{'vserver'},
            );

            no warnings 'exiting';
            redo UNDO;
        }
        catch NACL::C::Exceptions::VolumeEfficiency::PromoteDeprioritizedVolume with {
            my $exception               = shift;
            my %vol_vs_details = %{ $exception->get_vol_vs_to_promote() };

            $Log->comment("Enable efficiency operations");
            $pkg_or_obj->promote(
                command_interface => $opts{command_interface},
                volume            => $vol_vs_details{'volume'},
                vserver           => $vol_vs_details{'vserver'},
            );

            no warnings 'exiting';
            redo UNDO;
        };
    }

} ## end sub prepare_to_downgrade

# Set is_infinivol_ref and job_component to be references to some variables
# even if not passed in by the caller because we need it later.
sub _set_job_infinivol_ref {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => { opts_ptr => { type => HASHREF } },
    );
    my $opts_ptr = $opts{opts_ptr};
    my $is_infinivol_ref;
    $opts_ptr->{is_infinivol_ref} ||= \$is_infinivol_ref;
    my $job;
    $opts_ptr->{job_component} ||= \$job;
    $Log->exit() if $may_exit;
}



# Take care of the logic for waiting after the start/stop command is issued.
sub _start_stop_wait {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            polling_interval => { type => SCALAR|UNDEF },
            nacltask_wait    => { type => SCALAR },
        },
        allow_extra => 1
    );

    # If a job was started, then store it in $final_job. If invoked with an
    # object instance as well, then store it in the instance.
    my $final_job = ${$opts{job_component}};
    if (defined $final_job && ref $pkg_or_obj) {
        $pkg_or_obj->job_component($final_job);
    }

    if ($opts{nacltask_wait}) {
        my %wait_opts;
        $pkg_or_obj->_copy_common_component_params_with_ci(
            source => \%opts,
            target => \%wait_opts
        );
        $pkg_or_obj->_copy_primary_keys(
            source => \%opts,
            target => \%wait_opts,
        );

        if (defined $opts{polling_interval}) {
            $wait_opts{polling_interval} = $opts{polling_interval};
        }

        $wait_opts{job_component} = $final_job if (defined $final_job);

        # For an infinivol, it is necessary to wait on the status of the
        # constituent volumes rather than the infinivol itself. This is what
        # the "is_infinivol" argument is used for.
        my $is_infinivol = $pkg_or_obj->_read_infinivol_val(%opts);
        $wait_opts{is_infinivol} = $is_infinivol if (defined $is_infinivol);

        $pkg_or_obj->wait_for_status(%wait_opts);
    }

    $Log->exit() if $may_exit;
}

sub _wait_for_sis_status {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            polling_interval => { type => SCALAR | UNDEF },
            'method-timeout'  => { type => SCALAR },
        },
        allow_extra => 1
    );

    my $volume           = $opts{path};
    my $apiset           = $opts{command_interface}->get_7m_or_nodescope_apiset();
    my $time             = timeout2time ( $opts{'method-timeout'} );
    my $polling_interval = $opts{polling_interval}
        || $opts{nacltask_wait} / 10;
    my $success = 0;
    my $output;
    while ( $time >= time() && !$success ) {
        try {
            $output = $apiset->sis_ls( path => $volume )->get_raw_output();
        }
        catch NATE::BaseException with {
            my $e = shift;
            $output = $e->get_response_object()->get_raw_output();
        };
        if ( $output =~ /\Qfp.downgrade.done\E/ ) {
            $success = 1;
        } else {
            Tharn::snooze($polling_interval);
        }
    }
    if ( !$success ) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::UnexpectedState->throw(
            "Volume Efficiency revert_to operation could not complete in $opts{'method-timeout'} seconds."
        );
    }
    $Log->exit() if $may_exit;
}

sub _stop {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        allow_extra => 1,
    );

    $opts{'nacltask_if_stopped'} = 'pass';

    $pkg_or_obj->stop(%opts);
    
    $Log->exit() if $may_exit;
}
1;
