# Copyright (c) 2017 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
#
## @summary Volume Encryption Task Module
## @author lkashyap@netapp.com
## @status review
## @pod here

package NACL::STask::VolumeEncryption;

use strict;
use warnings;
use List::MoreUtils qw(uniq);
use List::Util qw(shuffle);
use Params::Validate qw(validate_with SCALAR BOOLEAN ARRAYREF );
use POSIX qw(ceil);
use Readonly;

use base qw(NACL::STask::STask  NACL::C::Component);
use NACL::CS::Volume;
use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::Exceptions::Timeout;
use NACL::Exceptions::VerifyFailure qw(:try);
use NACL::InstantComponent qw(instant_component);
use NACL::STask::Volume;
use NACL::STask::VolumeFile;

use NATE::BaseException qw(:try);
use NATE::Log qw(log_global);
use NATE::Process;
use NATE::ProcManager;
use NATE::Result;

use constant DEFAULT_TIMEOUT => 7200;
use constant POLL_DELTA      => 30;

use Class::MethodMaker [
    new    => [ '-hash', '-init', 'new' ],
    scalar => 'volume',
    scalar => 'vserver',
    scalar =>
      [ { -type => 'NACL::C::CommandInterface::ONTAP' }, 'command_interface' ],
    scalar => 'volume_encryption_secure_purge_component',
    scalar => 'volume_encryption_component',
    scalar => 'default_scan_speed',
    scalar => 'check_default_scan_speed',
    scalar => '_rekey_started',
    scalar => 'encrypt',
    scalar => 'key_id',
    scalar => 'old_key_id',
    scalar => 'ckse_key_idx',
    scalar => 'old_ckse_key_idx',
    scalar => 'ckse_encrypted',
    scalar => 'old_ckse_encrypted',
    scalar => 'encryption_state',
    scalar => 'old_encryption_state',
    scalar => 'operation',
    scalar => '_secure_purge_started',
];

my $Log       = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 NAME

NACL::STask::VolumeEncryption

=head1 SYNOPSIS

use NACL::STask::VolumeEncryption;

my $Enc_Obj =  NACL::STask::VolumeEncryption->rekey(
    command_interface => $ci,
    volume            => $volume,
    vserver           => $vserver,
);

$Enc_Obj->wait_for_completion();

$Enc_Obj->pause();

$Enc_Obj->resume();

NACL::STask::VolumeEncryption->wait_for_completion(
    command_interface => $ci,
    volume            => $volume,
    vserver           => $vserver,
);


NACL::STask::VolumeEncryption->pause(
    command_interface => $ci,
    volume            => $volume,
    vserver           => $vserver,
);


NACL::STask::VolumeEncryption->resume(
    command_interface => $ci,
    volume            => $volume,
    vserver           => $vserver,
);

NACL::STask::VolumeEncryption->start_rekey_on_multiple_volumes(
    volume_list => [qw(vol_1 vol_2 vol_3)],
    command_interface => $node,
    operation_to_start => 'modify|rekey',
    vserver => 'vs0',
    encrypt => 'true|false',
    nacltask_wait => 1,
    nacltask_verify => 1,
);

NACL::STask::VolumeEncryption->start_rekey_and_conversion_scans_paralelly(
        command_interface   => $command_interface,
        percent_to_convert_to_encrypted => 70,
        percent_to_rekey                => 50,
        percent_to_convert_to_pt        => 20,
);


=head1 DESCRIPTION
C<NACL::STask::VolumeEncryption> implements multi-step methods
related to rekeying or converting encrypted volumes in Data ONTAP.
It has methods that primarily allow a volume or a list of volumes to be
a. rekeyed, if it is encrypted
b. converted to plaintext or viceversa
c. pause the rekey or conversion scan
d. resume the rekey or conversion scan
e. start multiple rekey _or_ conversion scan on a list of volumes
f. start multiple rekey _and_ conversion scans on different volumes
simultaneously

=head1 METHODS

=over 12

=item C<rekey>


  my $Vol_Enc_Obj  = NACL::STask::VolumeEncryption->rekey(
      command_interface   => $command_interface,
      vserver             => $vserver,
      'volume'     => $volume,
      nacltask_wait       => $boolean,  # default '1'
  );

(Class method) starts inplace volume rekey scan on the given
volume. This method provides the following additional services:

* Waits for completion (can be controlled by the C<nacltask_wait>
argument.

* Verifies for successful completion of the rekey operation
(can be controlled by the C<nacltask_verify> argument

Returns a NACL::STask::VolumeEncryption object if C<nacltask_wait> is 0


=over

=item Options

=over

=item C<< command_interface >>

(Required) A component object that represents the host to which to send commands.

=head2 volume

(Required) The name of the volume to start the rekey scanner

=head2 vserver

(Required) The name of the vserver containing the volume.

=item C<< nacltask_wait => 0|1 >>

(Optional) If 1 (the default), wait for inplace volume rekey scanner to
operation to complete.
If 0, do not wait.

=item C<< nacltask_wafl_scan_speed => $speed >>

(Optional) If 1 (the default is 1300 ), set the wafl scan speed before
starting the scan.

=item C<< nacltask_verify => 0|1 >>

(Optional) If 1 (the default is 0), verify the volume encryption state using the
-encrypt option and also compares the old key with the new key generated for the
volume.
If 0, do not verify.

If nacltask_wait is set to 0 and nacltask_verify is set to 1, then the STask
will not wait for the job to complete and hence not call the verify also.
The caller can call the C<nacltask_verify> separately in the C<wait_for_completion>
method.

=back


=back

=over

=item Exceptions

=over

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the rekeyed volume.

=item C<NACL::Exceptions::Timeout>

This type of exception is thrown when volume rekey operation has not started
or or completed within the C<< nacltask_timeout >> seconds.

=back

=cut

sub rekey_start {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    $pkg->operation('rekey');
    $pkg->_start(%opts);
}

sub rekey_pause {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    $pkg->operation('rekey');
    $pkg->_pause(%opts);
}

sub rekey_resume {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    $pkg->operation('rekey');
    $pkg->_resume(%opts);
}

sub rekey_wait_for_completion {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    $pkg->operation('rekey');
    $pkg->_wait_for_completion(%opts);
}

sub conversion_wait_for_completion {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    $pkg->operation('conversion');
    $pkg->_wait_for_completion(%opts);
}

sub conversion_start {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    $pkg->operation('conversion');
    $pkg->_start(%opts);
}

sub conversion_pause {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    $pkg->operation('conversion');
    $pkg->_pause(%opts);
}

sub conversion_resume {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    $pkg->operation('conversion');
    $pkg->_resume(%opts);
}

sub secure_purge_start {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    $pkg->operation('secure_purge');
    $pkg->_secure_purge_start(%opts);
}

sub secure_purge_show {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    $pkg->operation('secure_purge');
    $pkg->_volume_secure_purge_show(%opts);
}

sub secure_purge_wait_for_completion {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    $pkg->operation('secure_purge');
    $pkg->_wait_for_secure_purge_completion(%opts);
}

sub _start {

    $Log->enter() if $may_enter;
    my $pkg = shift;

    my $self;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            nacltask_timeout =>
              { type => SCALAR, optional => 1, default => DEFAULT_TIMEOUT },
            nacltask_wafl_scan_speed =>
              { type => SCALAR, optional => 1, default => 1300, },
            nacltask_wait => { type => BOOLEAN, optional => 1, default => 1, },
            nacltask_verify =>
              { type => BOOLEAN, optional => 1, default => 1, },
            nacltask_retry => { type => BOOLEAN, optional => 1, default => 1 },
        },
        allow_extra => 1,
    );

    if ( !defined $opts{'method-timeout'} ) {
        $opts{'method-timeout'} = DEFAULT_TIMEOUT;
    }

    if ( !$opts{nacltask_wait} && $opts{nacltask_verify} ) {
        $Log->comment( "Cannot verify the "
              . $pkg->operation
              . " operation when nacltask_wait is set to 0" );
        $opts{nacltask_verify} = 0;
    }

    my $node = delete $opts{node} if ( defined $opts{node} );

    $pkg->command_interface( $opts{command_interface} );

    my $rkey_status;
    my $poll_required = 1;

    # Initialize scan objects and params
    $pkg->_initialize_node_and_volume(%opts);

    my %nacltask_opts;
    $pkg->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_opts
    );

    try {

        # Checking the current status of the inplace volume rekey scanner
        $rkey_status = $pkg->_get_rekey_status();

        #$Log->comment( "Inplace rekey Scan State: " . $rkey_status );
        $Log->comment( $pkg->operation . " Scan State: " . $rkey_status );
    }
    catch NATE::BaseException with {
        my $e = shift;
        if ( !$e->isa('NACL::Exceptions::NoElementsFound') ) {
            $Log->exit() if $may_exit;
            $e->throw();
        }
        $poll_required = 0;
    };

    # If a rekey is already in-progress for the volume wait for it to complete
    # Throw exception if scan is paused or hasnt completd within the timeout

    if ($poll_required) {
        my $exceeded_polltime = 1;
        my $rekey_poll_time   = time() + $opts{'method-timeout'};
        while ( time() < $rekey_poll_time ) {
            if ( defined $rkey_status && $rkey_status < 0 ) {
                $Log->exit() if $may_exit;
                NATE::BaseException->throw(

                   #                    "Inplace rekey is paused on the volume:"
                    $pkg->operation
                      . "is paused on the volume: "
                      . $pkg->volume()->volume
                );
            }
            if ( defined $rkey_status && $rkey_status == 0 ) {
                $exceeded_polltime = 0;
                last;
            }
            Tharn::snooze POLL_DELTA;
            $rkey_status = $pkg->_get_rekey_status();
        }

        if ($exceeded_polltime) {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw( "Timeout: "
                  . $pkg->operation
                  . "operation is still active on the volume:"
                  . $pkg->volume()->volume );
        }
    }

    if ( $nacltask_opts{nacltask_verify} ) {
        $pkg->old_key_id( $pkg->key_id() );
        $pkg->old_ckse_key_idx( $pkg->ckse_key_idx() );
        $pkg->old_ckse_encrypted( $pkg->ckse_encrypted() );
        $pkg->old_encryption_state( $pkg->encryption_state() );

        $Log->comment(
            "Volume key_id before rekey scan : " . $pkg->old_key_id() );
        $Log->comment( "Volume ckse_key_idx before rekey scan : "
              . $pkg->old_ckse_key_idx() );
        $Log->comment( "Volume ckse_encrypted before rekey scan : "
              . $pkg->old_ckse_encrypted() );
        $Log->comment( "Volume encryption_state before rekey scan : "
              . $pkg->old_encryption_state() );
    }

    my $scan_speed = delete $nacltask_opts{nacltask_wafl_scan_speed};
    if ( $scan_speed < 0 && $nacltask_opts{nacltask_wait} ) {
        $Log->warn( "Negative value is provided for nacltask_wafl_scan_speed."
              . " Changing nacltask_wait to 0." );
        $nacltask_opts{nacltask_wait} = 0;
    }

    # Set scan speed
    $pkg->set_scan_speed($scan_speed);

    my $ci  = delete $opts{command_interface};
    my $try = 1;

  RETRY_REKEY:
    try {

        #$Log->comment("Begin inplace volume rekey scanner");
        $Log->comment( "Begin inplace " . $pkg->operation . " scanner" );
        $pkg->volume_encryption_component()->start(%opts);
    }
    catch NACL::APISet::Exceptions::MissingRequiredParamException with {
        my $exception_obj = shift;

        my $volume =
          ( ref( $opts{volume} ) ) ? $opts{volume}->volume() : $opts{volume};
        my $zapi;
        my %zapi_options = (
            'vserver' => $opts{vserver},
            'volume'  => $volume,
        );
        if ( $pkg->operation =~ /rekey/ ) {
            $zapi = 'volume_encryption_rekey_start';
        }
        elsif ( $pkg->operation =~ /conversion/ ) {
            $zapi = 'volume_encryption_conversion_start';
        }
        print "ZAPI : " . $zapi . "\n";
        my $response_obj =
          $ci->apiset( interface => 'ZAPI', )->$zapi(%zapi_options);

        if ( $response_obj->{execution_status} =~ /passed/ ) {
            $Log->comment( "Started inplace "
                  . $pkg->operation
                  . " operation successfully. Execution Status: "
                  . $response_obj->{execution_status} );
        }
        else {
            NATE::BaseException->throw( "Failed to start inplace "
                  . $pkg->operation . " scan"
                  . $exception_obj->text() );
        }
    }
    catch NACL::APISet::Exceptions::CommandFailedException with {
        my $exception_obj = shift;
        $Log->comment( "Failed to start "
              . $pkg->operation
              . " scan "
              . $exception_obj->text() );
        $try++;
        if ( $exception_obj->text =~ /it is already in progress/ ) {
            $Log->comment( $pkg->operation . " is already in-progress" );
        }
        elsif ( $opts{nacltask_retry} && $try > 3 ) {
            NATE::BaseException->throw(
                "Failed to start inplace rekey scan after 3 attempts on volume "
                  . $pkg->volume()->volume
                  . "\nError:"
                  . $exception_obj->text );
        }
        elsif ( !$opts{nacltask_retry} ) {
            NATE::BaseException->throw(
                    "Failed to start inplace rekey scan  on volume "
                  . $pkg->volume()->volume
                  . "\nError:"
                  . $exception_obj->text );
        }
        else {
            goto RETRY_REKEY;
        }
    };

    my $start_timeout             = $opts{'method-timeout'};
    my $endtime                   = time() + $start_timeout;
    my $rekey_started             = 0;
    my $rekey_start_message_found = 0;

    while ( time() < $endtime ) {

        # Check waflrekey status
        if ( !$rekey_started ) {

            # Now check if rekey is running
            my $rk_status = $pkg->_get_rekey_status();
            if ( $rk_status == 1 ) {

                # Rekey is in running state,
                $Log->debug( $pkg->operation . " is in-progress" );
                $rekey_started = 1;
            }
        }

        if ( !$rekey_started ) {
            Tharn::snooze POLL_DELTA;
        }

        if ($rekey_started) {
            $pkg->_rekey_started(1);
            $Log->comment(
                $pkg->operation . " operation started successfully" );
            last;
        }
    }    # End while

    if ( !$rekey_started ) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::Timeout->throw( $pkg->operation
              . " operation did not start within the timeout period:"
              . " $start_timeout" );
    } ## end if ( !$rekey_started...

    $self->{volume_encryption_component} = $pkg->volume_encryption_component;
    $self->{command_interface}           = $pkg->command_interface;
    $self->{volume}                      = $pkg->volume;
    $self->{vserver}                     = $pkg->volume->vserver;
    $self->{key_id}                      = $pkg->key_id;
    $self->{old_key_id}                  = $pkg->old_key_id;
    $self->{old_ckse_key_idx}            = $pkg->ckse_key_idx;
    $self->{old_ckse_encrypted}          = $pkg->ckse_encrypted;
    $self->{old_encryption_state}        = $pkg->encryption_state;

    # Depending on the 'nacltask_wait' => 0|1, either wait for
    # waflrekey to complete or return
    if ( $nacltask_opts{'nacltask_wait'} ) {
        $Log->comment(
            $pkg->operation . "scan started, waiting for completion" );
        $pkg->_wait_for_completion(
            %opts,
            command_interface => $self->{command_interface},
            nacltask_verify   => $nacltask_opts{nacltask_verify},
        );
    }

    $Log->exit() if $may_exit;
    return bless $self, 'NACL::STask::VolumeEncryption';
}

=head2 C<pause>

  NACL::STask::VolumeEncryption->pause(
      command_interface   => $command_interface,
      vserver             => $vserver,
      volume              => $volume,
  );

  Or

  $Rekey_Obj->pause();

(Class or Instance method) pauses the inplace volume rekey scan on the given
volume. Instance method is applicable for rekey method only.

=over

=item Options

=over

=item C<< command_interface >>

(Required) A component object that represents the host to which to send commands.

=head2 volume

(Required) The name of the volume to pause the rekey scanner

=head2 vserver

(Required) The name of the vserver containing the volume.

=item C<< method-timeout => $timeout >>

=back

=back

=cut

sub _pause {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'method-timeout' =>
              { type => SCALAR, optional => 1, default => DEFAULT_TIMEOUT },
            nacltask_verify => { type => BOOLEAN, optional => 1, default => 0 },
        },
        allow_extra => 1,
    );
    my $rk_status = undef;
    my $endtime   = time() + $opts{'method-timeout'};
    my $volume    = ( $opts{volume} ) ? $opts{volume} : $pkg_or_obj->volume();
    my $vserver = ( $opts{vserver} ) ? $opts{vserver} : $pkg_or_obj->vserver();
    my $ci =
      ( $opts{command_interface} )
      ? $opts{command_interface}
      : $pkg_or_obj->command_interface;

    $pkg_or_obj->command_interface($ci);
    $pkg_or_obj->_initialize_node_and_volume(
        volume          => $volume,
        vserver         => $vserver,
        nacltask_verify => $opts{nacltask_verify}
    );
    delete $opts{nacltask_verify};

    try {
        $Log->comment(
            "Pause the inplace " . $pkg_or_obj->operation . " operation" );
        delete $opts{command_interface};
        $pkg_or_obj->volume_encryption_component()->pause(%opts);
    }
    catch NACL::APISet::Exceptions::CommandFailedException with {
        my $exception_obj = shift;
        if (
            (
                $exception_obj->text() =~
/Failed to pause conversion on volume.* Reason: Phase 2\(.*\) is in progress/i
            )
            || ( $exception_obj->text() =~
/Failed to pause rekey on volume.* Reason: Phase 2\(.*\) is in progress/i
            )
          )
        {
            NATE::BaseException->throw( "Failed to pause "
                  . $pkg_or_obj->operation
                  . " operation. Reason: "
                  . $exception_obj->text );
        }
    };

    while (1) {
        $rk_status = $pkg_or_obj->_get_rekey_status();

        if ( $rk_status < 0 ) {
            $Log->debug("volume encryption pause : scan is paused");
            last;
        }
        else {
            $Log->debug("volume encryption pause : scanner is not paused yet");
            Tharn::snooze 60;
        }

        # If scan complete before reaching timeout
        if ( $rk_status == 0 ) {
            $Log->debug("volume encryption pause : scan is complete");
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "scan completed instead of reaching pause state");
        }

        if ( $endtime < time() ) {
            $Log->comment( "volume encryption pause : "
                  . "Timeout of "
                  . $endtime
                  . " reached" );
            $Log->exit() if $may_exit;
            NATE::BaseException->throw( "Timeout reached waiting "
                  . "for inplace "
                  . $pkg_or_obj->operation
                  . " to pause" );
        }
    }
    $Log->comment( "Inplace volume "
          . $pkg_or_obj->operation
          . " scanner paused successfully" );
}

=head2 C<resume>

  my $Vol_Enc_Obj  = NACL::STask::VolumeEncryption->resume(
      command_interface   => $command_interface,
      vserver             => $vserver,
      volume              => $volume,
  );

  Or

  $Rekey_Obj->resume();

(Class or Instance method) resumes the paused inplace volume rekey scan on the given
volume. Instance method is applicable for rekey method only.

=over

=item Options

=over

=item C<< command_interface >>

(Required) A component object that represents the host to which to send commands.

=head2 volume

(Required) The name of the volume to start the rekey scanner

=head2 vserver

(Required) The name of the vserver containing the parent volume.
=item C<< nacltask_wait => 0|1 >>

(Optional) If 1 (the default), wait for inplace volume rekey scanner to
operation to complete.
If 0, do not wait.

=item C<< method-timeout => $timeout >>
(Optional) If not uses the C<< DEFAULT_TIMEOUT >> value

=back

=back

=cut

sub _resume {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'method-timeout' =>
              { type => SCALAR, optional => 1, default => DEFAULT_TIMEOUT },
            nacltask_verify => { type => BOOLEAN, optional => 1, default => 0 },
        },
        allow_extra => 1,
    );
    my $rk_status = undef;
    my $endtime   = time() + $opts{'method-timeout'};
    my $volume    = ( $opts{volume} ) ? $opts{volume} : $pkg_or_obj->volume();
    my $vserver = ( $opts{vserver} ) ? $opts{vserver} : $pkg_or_obj->vserver();
    my $ci =
      ( $opts{command_interface} )
      ? $opts{command_interface}
      : $pkg_or_obj->command_interface;

    #    instant_component(
    #        component_name    => 'NACL::C::VolumeEncryption',
    #        command_interface => $opts{command_interface},
    #    );
    $pkg_or_obj->command_interface($ci);
    $pkg_or_obj->_initialize_node_and_volume(
        volume          => $volume,
        vserver         => $vserver,
        nacltask_verify => $opts{nacltask_verify}
    );
    delete $opts{nacltask_verify};

    try {
        $Log->comment( "Restart the volume inplace "
              . $pkg_or_obj->operation
              . " scanner" );
        delete $opts{command_interface};
        $pkg_or_obj->volume_encryption_component()->resume(%opts);
    }
    catch NACL::APISet::Exceptions::CommandFailedException with {
        my $exception_obj = shift;
        $Log->comment( "Failed to resume the volume inplace "
              . $pkg_or_obj->operation
              . " operation : "
              . $exception_obj->text() );
    };

    while (1) {
        $rk_status = $pkg_or_obj->_get_rekey_status();

        if ( $rk_status == 1 ) {
            $Log->debug("Scan is resumeed and in-progress");
            last;
        }
        else {
            $Log->debug("Scanner is not resumeed yet");
            Tharn::snooze 2;
        }

        if ( $endtime < time() ) {
            $Log->comment( "Timeout of " . $endtime . " reached" );
            $Log->exit() if $may_exit;
            NATE::BaseException->throw( "Timeout reached waiting "
                  . "for inplace rekey scan to resume" );
        }
    }

    $Log->comment( "Inplace "
          . $pkg_or_obj->operation
          . " scanner resumeed successfully" );
}

=head2 C<start_rekey_or_modify_on_multiple_volumes>

NACL::STask::VolumeEncryption->start_rekey_or_modify_on_multiple_volumes(
      command_interface   => $command_interface,
      operation => rekey|modify,
       volume_list => [volume_names_or_objects],
  );

(Class method) starts the inplace volume rekey scan on the given list of
volumes. This method provides the following additional services:

* Implicitly supports all the options supported by the C<< rekey > or
C<< NACL::STask::Volume->modify >> methods, like
C<< nacltask_wait >>, C<< nacltas_verify >> etc.
=over

=item Options

=over

=item C<< command_interface >>

(Required) A component object that represents the host to which to send commands.

=head2 volume_list

(Required) The list of volumes to start the rekey or conversion scanner

=head2 operation

(Required) The name of the operation to start on the given list of volumes
It can be only rekey|modify

=back

=back

=cut

sub start_rekey_on_multiple_volumes {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'method-timeout' =>
              { type => SCALAR, optional => 1, default => DEFAULT_TIMEOUT },
            nacltask_verify => { type => BOOLEAN, optional => 1, default => 1 },
            nacltask_timeout =>
              { type => SCALAR, optional => 1, default => DEFAULT_TIMEOUT },
            volume_list        => { type => ARRAYREF, optional => 0, },
            operation_to_start => { type => SCALAR,   optional => 0, },
        },
        allow_extra => 1,
    );

    my @volumes = @{ delete( $opts{volume_list} ) };
    @volumes = uniq(@volumes);

    my @proc_arr;
    my $proc;

    my $ci = $opts{command_interface};

    my $task = delete( $opts{operation_to_start} );

    if ( $task !~ /rekey|modify/ ) {
        $Log->exit() if $may_exit;
        NACL::BaseException->throw(
                "Invalid value for option : operation_to_start"
              . "Valid values are \'rekey\' or \'modify\' " );
    }

    if ( $task =~ /rekey/ ) {
        for my $rekey_obj (@volumes) {
            $opts{volume} = $rekey_obj;
            $proc = NATE::Process->new(
                codespec => \&rekey_start,
                args     => [ $pkg, %opts ],
                runid    => $task . "_" . $opts{volume},
                onexit   => \&NATE::Process::on_exit_propagate_worst_result
            );
            push( @proc_arr, $proc );
        }
    }
    elsif ( $task =~ /modify/ ) {
        if ( exists $opts{encrypt} ) {
            for my $rekey_obj (@volumes) {
                $opts{volume} = $rekey_obj;
                $proc = NATE::Process->new(
                    codespec => \&conversion_start,
                    args     => [ $pkg, %opts ],
                    runid    => $task . "_" . $opts{volume},
                    onexit   => \&NATE::Process::on_exit_propagate_worst_result
                );
                push( @proc_arr, $proc );
            }
        }
    }
    my $procmgr = NATE::ProcManager->new(
        proc_info  => \@proc_arr,
        background => 1,
        parallel   => 1,
    );
    $procmgr->start;
    $procmgr->wait;
    my $result = NATE::Result->is_failure( $procmgr->worst_result );
    if ($result) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Failed to perform rekey or conversion operations");
    }

}

=head2 C<start_rekey_and_conversion_scans_paralelly>

  NACL::STask::VolumeEncryption->start_rekey_and_conversion_scans_paralelly(
        command_interface   => $command_interface,
        percent_to_convert_to_encrypted => 70,
        percent_to_rekey                => 50,
        percent_to_convert_to_pt        => 20,
        %vol_options,
  );

(Class method) This method will find encrypted and plaintext volumes that match
a pre-defined set of attributes in the cluster and starts the inplace rekey
scanner to rekey or convert scans on percentage of volumes, as specified in
the options.

=over

=item Options

=over

=item C<< command_interface >>

(Required) A component object that represents the host to which to send commands.

=head2 percent_to_convert_to_encrypted

(Required) The percentage of plaintext volumes in the system that will be converted
to encrypted, using the inplace rekey scanner.

=head2 percent_to_rekey

(Required) The percentage of encrypted volumes in the system that will be rekeyed
using the inplace rekey scanner.

=head2 percent_to_convert_to_pt

(Required) The percentage of encrypted volumes in the system that will be converted
to plaintext, using the inplace rekey scanner

=back

=back

=cut

sub start_rekey_and_conversion_scans_paralelly {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'method-timeout' =>
              { type => SCALAR, optional => 1, default => DEFAULT_TIMEOUT },
            nacltask_verify => { type => BOOLEAN, optional => 1, default => 1 },
            nacltask_timeout =>
              { type => SCALAR, optional => 1, default => DEFAULT_TIMEOUT },
            percent_to_convert_to_encrypted =>
              { type => SCALAR, optional => 0, },
            percent_to_rekey         => { type => SCALAR, optional => 0, },
            percent_to_convert_to_pt => { type => SCALAR, optional => 0, },
        },
        allow_extra => 1,
    );

    my %volume_options;
    foreach my $key ( keys(%opts) ) {

        # Copy only volume options
        next
          if ( $key =~ /percent_to_|nacltask|apiset|method|command_interface/ );
        $volume_options{$key} = delete $opts{$key};
    }

    my @Encrypted_Volumes;
    my @Plaintext_Volumes;
    my $keep_polling = 0;
    my $endtime      = time() + $opts{nacltask_timeout};
    $Log->debug( "END TIME : " . $endtime );

    $pkg->command_interface( $opts{command_interface} );

  WAIT_FOR_VOLUMES: while (1) {
        try {

            # Find plaintext and encrypted volumes in the cluster
            @Encrypted_Volumes = $pkg->_find_volumes(
                'encrypt'          => 'true',
                'encryption-state' => 'converting_to_plaintext,rekeying',
                %volume_options
            );
            for ( my $i = 0 ; $i <= $#Encrypted_Volumes ; $i++ ) {
                next;
                my $encryption =
                  $Encrypted_Volumes[$i]
                  ->get_one_state_attribute('encryption-state');
                if ( $encryption =~ /converting_to_plaintext|rekeying/ ) {
                    splice( @Encrypted_Volumes, $i, 1 );
                }
            }
        }
        catch NATE::BaseException with {
            my $e = shift;
            if ( $e->isa('NACL::Exceptions::NoElementsFound') ) {
                $Log->comment("No volumes found..keep polling for volumes");
                Tharn::snooze(60);
                $keep_polling = 1;
            }
            else {
                $Log->exit() if $may_exit;
                $e->throw();
            }
        };
        try {
            @Plaintext_Volumes = $pkg->_find_volumes(
                'encrypt'          => 'false',
                'encryption-state' => 'converting_to_encrypted',
                %volume_options
            );
            for ( my $j = 0 ; $j <= $#Plaintext_Volumes ; $j++ ) {
                next;
                my $encryption =
                  $Plaintext_Volumes[$j]
                  ->get_one_state_attribute('encryption-state');
                if ( $encryption =~ /converting_to_encrypted/ ) {
                    splice( @Plaintext_Volumes, $j, 1 );
                }
            }
        }
        catch NATE::BaseException with {
            my $e = shift;
            if ( $e->isa('NACL::Exceptions::NoElementsFound') ) {
                $Log->comment("No volumes found..keep polling for volumes");
                Tharn::snooze(60);
                $keep_polling = 1;
            }
            else {
                $Log->exit() if $may_exit;
                $e->throw();
            }
        };
        if ( scalar(@Encrypted_Volumes) || scalar(@Plaintext_Volumes) ) {
            $keep_polling = 0;
        }

        if ( $endtime < time() ) {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Timeout reached waiting " . "for volumes to be available" );
        }

        redo WAIT_FOR_VOLUMES if ($keep_polling);
        last;
    }

    my $Total_Volumes = scalar(@Encrypted_Volumes) + scalar(@Plaintext_Volumes);

    $Log->comment(
        "Number of encrypted volumes : " . scalar(@Encrypted_Volumes) );
    $Log->comment(
        "Number of plaintext volumes : " . scalar(@Plaintext_Volumes) );

    $Log->comment( "Percentage of volumes to be converted to encrypted : "
          . $opts{percent_to_convert_to_encrypted} );
    $Log->comment(
        "Percentage of volumes to be rekeyed : " . $opts{percent_to_rekey} );
    $Log->comment( "Percentage of volumes to be converted to plaintext : "
          . $opts{percent_to_convert_to_pt} );

    my @volumes_to_rekey;
    my @volumes_to_convert_pt;
    my @volumes_to_convert_to_ec;

    if ( $opts{percent_to_convert_to_encrypted} > 100 ) {
        $Log->comment("Percentage should be less than or equal to 100");
        $Log->comment("Resetting percent_to_convert_to_encrypted to 100%");
        $opts{percent_to_convert_to_encrypted} = 100;
    }

    my @shuffled_indexes = shuffle( 0 .. $#Plaintext_Volumes );
    $opts{percent_to_convert_to_encrypted} = ceil(
        ( $opts{percent_to_convert_to_encrypted} * scalar(@shuffled_indexes) )
        / 100 );

    $Log->comment( "Plaintext volumes that will be converted to encrypted : "
          . $opts{percent_to_convert_to_encrypted} );

    my @convert_to_ec_vol_indices =
      @shuffled_indexes[ 0 .. int( $opts{percent_to_convert_to_encrypted} - 1 )
      ];

    @volumes_to_convert_to_ec = @Plaintext_Volumes[@convert_to_ec_vol_indices];

    if ( !scalar(@volumes_to_convert_to_ec) ) {
        $Log->comment(
            "No plaintext volumes chosen to be " . "converted to encrypted" );
    }
    else {
        $Log->comment("List of volumes that will be converted to encrypted :");
        foreach (@volumes_to_convert_to_ec) {
            $Log->debug( $_->volume );
        }
    }

    # Get the list of volumes to be rekeyed
    if ( $opts{percent_to_rekey} > 100 ) {
        $Log->comment("Percentage should be less than or equal to 100");
        $Log->comment("Resetting percent_to_rekey to 100%");
        $opts{percent_to_rekey} = 100;
    }

    @shuffled_indexes = shuffle( 0 .. $#Encrypted_Volumes );

    $opts{percent_to_rekey} =
      int( ( $opts{percent_to_rekey} * scalar(@shuffled_indexes) ) / 100 );
    $Log->comment( "Number of encrypted volumes that will be rekeyed: "
          . $opts{percent_to_rekey} );

    my @rekey_volume_indices =
      @shuffled_indexes[ 0 .. int $opts{percent_to_rekey} - 1 ];
    @volumes_to_rekey = @Encrypted_Volumes[@rekey_volume_indices];

    my @volumes_to_convert_to_pt;
    if ( !scalar(@volumes_to_rekey) ) {
        $Log->comment("No encrypted volumes chosen to be rekeyed");
    }
    else {
        $Log->comment("List of volumes that will be rekeyed : ");
        foreach (@volumes_to_rekey) {
            $Log->comment( $_->volume );
        }
    }
    if ( scalar(@volumes_to_rekey) eq scalar(@Encrypted_Volumes) ) {
        $Log->comment( "All encrypted volumes will be rekeyed and not "
              . "be converted to plaintext" );
    }

    # Get list of encrypted volumes to be converted to plaintext
    else {
        for ( my $i = 0 ; $i <= $#Encrypted_Volumes ; $i++ ) {
            if ( grep { /^$i$/ } @rekey_volume_indices ) {
            }
            else {
                push( @volumes_to_convert_to_pt, $Encrypted_Volumes[$i] );
            }
        }
        $Log->comment(
            "List of encrypted volumes available to convert to plaintext");
        foreach (@volumes_to_convert_to_pt) {
            $Log->comment( $_->volume );
        }

        if ( $opts{percent_to_convert_to_pt} > 100 ) {
            $Log->comment("Percentage should be less than or equal to 100");
            $Log->comment("Resetting percent_to_convert_to_pt to 100%");
            $opts{percent_to_convert_to_pt} = 100;
        }

        @shuffled_indexes = shuffle( 0 .. $#volumes_to_convert_to_pt );
        $opts{percent_to_convert_to_pt} =
          ( ceil( $opts{percent_to_convert_to_pt} * scalar(@shuffled_indexes) )
              / 100 );
        $Log->comment(
            "Number of encrypted volumes that will be converted to plaintext : "
              . int $opts{percent_to_convert_to_pt} );
        if ( ( !$opts{percent_to_convert_to_pt} ) ) {
            $Log->comment("No volumes chosen to be converted to plaintext");
        }

        my @convert_to_pt_volume_indices =
          @shuffled_indexes[ 0 .. int $opts{percent_to_convert_to_pt} - 1 ];
        @volumes_to_convert_pt =
          @volumes_to_convert_to_pt[@convert_to_pt_volume_indices];

        $Log->comment("List of volumes that will be convert to plaintext");
        foreach (@volumes_to_convert_pt) {
            $Log->comment( $_->volume );
        }
    }

    my $proc;
    my @proc_arr;

    my $percent_to_convert_to_encrypted =
      delete $opts{percent_to_convert_to_encrypted};
    my $percent_to_convert_to_pt = delete $opts{percent_to_convert_to_pt};
    my $percent_to_rekey         = delete $opts{percent_to_rekey};
    foreach (@volumes_to_rekey) {
        $opts{volume}  = $_;
        $opts{vserver} = $_->vserver;
        $proc          = NATE::Process->new(
            codespec => \&rekey_start,
            args     => [ $pkg, %opts ],
            runid    => "Rekey_volume_" . $_->volume,
            onexit   => \&NATE::Process::on_exit_propagate_worst_result
        );
        push( @proc_arr, $proc );
    }

=pod
    # Conversion to plaintext volumes is not supported in FT
    foreach (@volumes_to_convert_pt) {
        $opts{volume}  = $_;
        $opts{vserver} = $_->vserver;
        $proc          = NATE::Process->new(
            codespec => \&NACL::STask::Volume::modify,
            args     => [ 'NACL::STask::Volume', %opts, encrypt => 'false', ],
            runid    => "Convert_to_plaintext_" . $_->volume,
            onexit   => \&NATE::Process::on_exit_propagate_worst_result
        );
        push( @proc_arr, $proc );
    }
=cut

    foreach (@volumes_to_convert_to_ec) {
        $opts{volume}  = $_;
        $opts{vserver} = $_->vserver;
        $proc          = NATE::Process->new(
            codespec => \&conversion_start,
            args     => [ $pkg, %opts, ],
            runid    => "Convert_to_encrypt_" . $_->volume,
            onexit   => \&NATE::Process::on_exit_propagate_worst_result
        );
        push( @proc_arr, $proc );
    }
    my $procmgr = NATE::ProcManager->new(
        proc_info  => \@proc_arr,
        background => 1,
        parallel   => 1,
    );
    $procmgr->start;
    $procmgr->wait;
    my $result = NATE::Result->is_failure( $procmgr->worst_result );
    if ($result) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Failed to perform rekey or conversion scans");
    }
}

=head2 C<wait_for_completion>

  my $Vol_Enc_Obj  = NACL::STask::VolumeEncryption->wait_for_completion(
      command_interface   => $command_interface,
      vserver             => $vserver,
      volume              => $volume,
      nacltask_verify     => 0|1,
  );

  Or

  $Rekey_Obj->wait_for_completion();

(Class or Instance method) waits for the inplace volume rekey or conversion
scan on the given volume to complete.

Instance method call applies to rekey method only. Use class method call
only for resumeing conversion scan operations.

=over

=item Options

=over

=item C<< command_interface >>

(Required) A component object that represents the host to which to send commands.

=head2 volume

(Required) The name of the volume on which the rekey or conversion scanner is paused

=head2 vserver

(Required) The name of the vserver containing the volume.

=item C<< nacltask_verify => 0|1 >>

(Optional) Verifies if the rekey or conversion scanner operation is successful, by
checking the -encrypt and key_id fields.
For plaintext volumes being converted to encrypted, verifies if C<< -encrypt  >>
field is true and a new key_id is populated for this volume.
For encrypted volumes being converted to plaintext, verifies if  C<< -encrypt >>
field is false and key_id is null.
For encrypted volumes being rekeyed, compares the previous and the new key_id
generated.

=back

=back

=cut

sub _wait_for_completion {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'method-timeout' =>
              { type => SCALAR, optional => 1, default => DEFAULT_TIMEOUT },
            nacltask_verify => { type => BOOLEAN, optional => 1, default => 0 },
            nacltask_timeout =>
              { type => SCALAR, optional => 1, default => DEFAULT_TIMEOUT },
        },
        allow_extra => 1,
    );

    my %nacltask_opts;
    $pkg_or_obj->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_opts
    );

    my $endtime   = time() + $nacltask_opts{nacltask_timeout};
    my $rk_status = undef;

    my $volume  = ( $opts{volume} )  ? $opts{volume}  : $pkg_or_obj->volume();
    my $vserver = ( $opts{vserver} ) ? $opts{vserver} : $pkg_or_obj->vserver();
    my $ci =
      ( $opts{command_interface} )
      ? $opts{command_interface}
      : $pkg_or_obj->command_interface;

    #    instant_component(
    #        component_name    => 'NACL::C::VolumeEncryption',
    #        command_interface => $opts{command_interface},
    #    );

    $pkg_or_obj->command_interface($ci);

    $pkg_or_obj->_initialize_node_and_volume(
        volume          => $volume,
        vserver         => $vserver,
        nacltask_verify => $nacltask_opts{nacltask_verify},
    );

    if ( $pkg_or_obj->encryption_state =~ /rekeying/ ) {
        $Log->comment("Waiting for the inplace rekey scan to complete");
        $opts{encrypt} = 0;
    }
    elsif ( $pkg_or_obj->encryption_state =~ /(converting_to_.*)/ ) {
        if ( $1 =~ /converting_to_encrypted/ ) {
            $Log->comment(
                "conversion to encrypted : waiting for scan to complete");
            $opts{encrypt} = 'true';
        }
        elsif ( $1 =~ /converting_to_plaintext/ ) {
            $Log->comment(
                "conversion to plaintext : waiting for scan to complete");
            $opts{encrypt} = 'false';
        }
    }

    # Wait until rekey or conversion scan is complete
    while (1) {
        $rk_status = $pkg_or_obj->_get_rekey_status();

        if ( $rk_status == 1 ) {
            $Log->comment( $pkg_or_obj->operation . " scan is in-progress" );
        }
        elsif ( $rk_status == 0 ) {
            $Log->comment( $pkg_or_obj->operation . " scan is complete" );
            last;
        }

        if ( $endtime < time() ) {
            $Log->comment(
                "Timeout of " . $nacltask_opts{nacltask_timeout} . " reached" );
            $Log->exit() if $may_exit;
            NATE::BaseException->throw( "Timeout reached waiting for inplace"
                  . $pkg_or_obj->operation
                  . " scan to complete" );
        }
        Tharn::snooze POLL_DELTA;
    }

    if ( $nacltask_opts{nacltask_verify} ) {
        Tharn::snooze(15);
        $pkg_or_obj->_initialize_node_and_volume(
            volume          => $volume,
            vserver         => $vserver,
            nacltask_verify => $nacltask_opts{nacltask_verify},
        );

        # For modify
        # If converted to encrypt there should be a key_id generated
        # and the encrypt flag should be true and encryption_state
        # should be full
        # If converted to plaintext, the key_id should be - and the
        # encrypt flag should be false and encryption_state should
        # be none
        #
        if ( $pkg_or_obj->operation =~ /conversion/i ) {

            # Only P2E is supported in FT, so
            # $opts{encrypt} should always be 'true'
            $opts{encrypt} = 'true';
            if ( $opts{encrypt} eq 'true' ) {
                if ( defined( $pkg_or_obj->key_id ) ) {
                    $Log->comment( "Volume Modify Encrypt : key_id = "
                          . $pkg_or_obj->key_id );
                }
                else {

                    # Should never come here
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
"Conversion to encrypted failed : key-id is not present but key-id is "
                          . $pkg_or_obj->key_id );
                }
                if ( defined( $pkg_or_obj->encrypt )
                    && ( $pkg_or_obj->encrypt =~ /true/ ) )
                {
                    $Log->comment( "Volume Modify Encrypt : encrypt    = "
                          . $pkg_or_obj->encrypt );
                }
                else {

                    # Should never come here
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
"Conversion to encrypted failed : encrypt is not true but "
                          . $pkg_or_obj->encrypt );
                }
                if ( defined( $pkg_or_obj->encryption_state )
                    && ( $pkg_or_obj->encryption_state =~ /full/ ) )
                {
                    $Log->comment( "Volume Modify Encrypt : encryption state = "
                          . $pkg_or_obj->encryption_state );
                }
                else {

                    # Should never come here
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
"Conversion to encrypted failed : encryption_state is not \'full\' but "
                          . $pkg_or_obj->encryption_state );
                }
                $Log->comment("Volume conversion to encrypted successful");
            }
            elsif ( $opts{encrypt} eq 'false' ) {
                if ( not defined( $pkg_or_obj->key_id ) ) {
                    $Log->comment("Volume Modify Plaintext : key_id = -");
                }
                else {

                    # Should never come here
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
                            "Conversion to plaintext failed : "
                          . "key-id is present for plaintext volume" );
                }
                if ( defined( $pkg_or_obj->encrypt )
                    && ( $pkg_or_obj->encrypt =~ /false/ ) )
                {
                    $Log->comment( "Volume Modify Plaintext : encrypt = "
                          . $pkg_or_obj->encrypt );
                }
                else {

                    # Should never come here
                    $Log->comment( "Volume Modify Plaintext : encrypt = "
                          . $pkg_or_obj->encrypt );
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
                            "Conversion to plaintext failed : "
                          . "encrypt is true for plaintext volume" );
                }
                if ( defined( $pkg_or_obj->encryption_state )
                    && ( $pkg_or_obj->encryption_state =~ /none/ ) )
                {
                    $Log->comment( "Volume Modify Encrypt : encryption state = "
                          . $pkg_or_obj->encryption_state );
                }
                else {

                    # Should never come here
                    $Log->comment( "Volume Modify Encrypt : encryption state = "
                          . $pkg_or_obj->encryption_state );
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
"Conversion to plaintext failed : encryption_state is not \'none\' "
                    );
                }
                $Log->comment("Volume conversion to plaintext successful");
            }
        }
        elsif ( $pkg_or_obj->operation =~ /rekey/i ) {

         # For volume rekey usecase, the key_id should be populated with new key
         # and the encrypt field should be true
            if (   $pkg_or_obj->old_key_id
                && $pkg_or_obj->old_key_id =~ /$pkg_or_obj->key_id/ )
            {

                # Should never come here
                $Log->exit() if $may_exit;
                NATE::BaseException->throw(
                    "Rekey Failed : Old Key is same as the new key");
            }
            else {
                $Log->comment( "Rekey key_id Verification Successful." . "\n"
                      . "new_key_id: "
                      . $pkg_or_obj->key_id . "\n"
                      . "old_key_id: "
                      . $pkg_or_obj->old_key_id
                      . " \n" );
            }

            if (   $pkg_or_obj->old_ckse_key_idx
                && $pkg_or_obj->old_ckse_key_idx =~
                /$pkg_or_obj->old_ckse_key_idx/ )
            {

                # Should never come here
                $Log->exit() if $may_exit;
                NATE::BaseException->throw(
                    "Rekey Failed : Key index is not flipped");
            }
            else {
                $Log->comment( "Rekey ckse_key_idx verification successful \n"
                      . "new_ckse_key_idx : "
                      . $pkg_or_obj->ckse_key_idx . "\n"
                      . "old_ckse_key_idx : "
                      . $pkg_or_obj->old_ckse_key_idx );
            }
            if ( defined( $pkg_or_obj->encryption_state )
                && ( $pkg_or_obj->encryption_state =~ /full/ ) )
            {
                $Log->comment( "Volume Rekey Encrypt : encryption state = "
                      . $pkg_or_obj->encryption_state );
            }
            else {

                # Should never come here
                $Log->exit() if $may_exit;
                NATE::BaseException->throw(
"Rekey of encrypted volume failed : encryption_state is not \'full\' "
                );
            }
            $Log->comment("Inplace rekey of encrypted volume is successful");
        }
        $Log->exit() if $may_exit;
    }
}

sub rekey_show {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    $pkg->operation('rekey');
    $pkg->_volume_encryption_show(%opts);
}

sub conversion_show {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    $pkg->operation('conversion');
    $pkg->_volume_encryption_show(%opts);
}

sub _volume_encryption_show {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;
    my %opts       = @_;

    my $volume  = ( $opts{volume} )  ? $opts{volume}  : $pkg_or_obj->volume();
    my $vserver = ( $opts{vserver} ) ? $opts{vserver} : $pkg_or_obj->vserver();
    my $ci =
      ( $opts{command_interface} )
      ? $opts{command_interface}
      : $pkg_or_obj->command_interface;

    $pkg_or_obj->command_interface($ci);

    # Initialize scan objects and params
    $pkg_or_obj->_initialize_node_and_volume(
        volume  => $volume,
        vserver => $vserver,
    );

    my $rkey_state;
    try {
        $rkey_state =
          $pkg_or_obj->volume_encryption_component()
          ->state( 'method-timeout' => 60 )->status();
        $Log->comment( "Scan state:  " . $rkey_state );
    }
    catch NATE::BaseException with {
        my $exception_obj = shift;
        $Log->exit() if $may_exit;
        $exception_obj->throw();
    };
    return $rkey_state;
}

# Internal routine to check the volume encryption
# scan status for rekey or conversion scans
# Returns
# -1 if scan is paused
#  0 if scan is not running
#  1 if scan is in-progress
sub _get_rekey_stat {
}

sub _get_conversion_status {
}

sub _get_rekey_status {

    $Log->enter() if $may_enter;

    my $self = shift;
    my $rkey_state;
    try {
        $rkey_state =
          $self->volume_encryption_component()->state( 'method-timeout' => 60 )
          ->status();
        $Log->comment( "Scan state:  " . $rkey_state );
    }
    catch NATE::BaseException with {
        my $exception_obj = shift;

        # ZAPI call
        if ( $exception_obj->text() =~
            /Volume.*is not found in vserver .*Reason: entry does.*exist/ )
        {
            $rkey_state = 0;
        }
        else {
            $Log->comment("Unexpected rekey state.");
            $Log->exit() if $may_exit;
            $exception_obj->throw();
        }
    };

    return 0 if ( !$rkey_state );

    if (   ( $rkey_state =~ /Volume encryption is not currently going on/ )
        || ( $rkey_state =~ /Not currently going on/ ) )
    {
        $Log->exit() if $may_exit;
        return 0;
    }
    elsif (
           ( $rkey_state =~ /Conversion to encrypted volume started/ )
        || ( $rkey_state =~ /Conversion to non-encrypted volume started/ )
        || (
            $rkey_state =~ /Conversion to non-encrypted volume is in-progress/ )
        || ( $rkey_state =~ /Conversion to encrypted volume is in-progress/ )
        || ( $rkey_state =~ /block redirection is taking place/ )
        || ( $rkey_state =~
/Conversion to non-encrypted volume is completed, block redirection is taking place/
        )
        || ( $rkey_state =~
/Conversion to encrypted volume is completed, block redirection is taking place/
        )
        || ( $rkey_state =~ /Rekey is in-progress/ )
        || ( $rkey_state =~ /Rekey started/ )
        || ( $rkey_state =~ /Rekey phase.*started/ )
        || ( $rkey_state =~ /Rekey phase.*in-progress/ )
        || ( $rkey_state =~ /Rekey phase.*progress/ )
        || ( $rkey_state =~ /Phase.*is in progress/ )
        || ( $rkey_state =~ /Phase 1\(of 2\) is in-progress.*/ )
        || ( $rkey_state =~ /Phase 1\(of 2\) started.*/ )
        || ( $rkey_state =~ /Phase 1 of 2 started/ )
        || ( $rkey_state =~ /Phase 2 of 2 started/ )
        || ( $rkey_state =~ /Phase 2\(of 2\) is in-progress.*/ )
        || ( $rkey_state =~ /Phase 1 of 2 is in-progress.*Completed.*/ )
        || ( $rkey_state =~ /Conversion to encrypted volume phase.*started/ )
        || ( $rkey_state =~ /Conversion to encrypted volume phase.*progress.*/ )
        || (
            $rkey_state =~ /Conversion to non-encrypted volume phase.*started/ )
        || ( $rkey_state =~
            /Conversion to non-encrypted volume phase.*progress/ )
      )
    {
        $Log->exit() if $may_exit;
        return 1;
    }
    elsif (
        ( $rkey_state =~ /Conversion to non-encrypted volume is being paused/ )
        || ( $rkey_state =~ /Conversion to encrypted volume is being paused/ )
        || ( $rkey_state =~ /Rekey is being paused/ )
        || ( $rkey_state =~ /Paused/ ) )
    {
        $Log->exit() if $may_exit;
        return -1;
    }
    else {
        if (   ( $rkey_state =~ /Rekey is aborted/ )
            || ( $rkey_state =~ /Being aborted/ )
            || ( $rkey_state =~ /Aborted/ ) )
        {
            Tharn::snooze 10;
            $Log->comment("Re-attempting _get_rekey_status");
            $self->_get_rekey_status();
        }
        else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Unexpected rekey state: " . $rkey_state );
        }
    }
}

# Internal function to find and return volumes matching given
# attributes
sub _find_volumes {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = @_;

    $Log->comment( "Find volumes with encrypt => " . $opts{encrypt} );
    my @vol_objs = NACL::STask::Volume->find(
        command_interface => $self->command_interface(),
        filter            => {%opts},
    );
    $Log->exit() if $may_exit;
    return @vol_objs;
}

# Return the ckse-key-idx for a volume
# Input volume name
# Create a 4k file in the volume
# Run dumpblock info for block
# Get ckse-key-idx
sub _get_ckse_key_idx {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = @_;

    my $vol_obj = NACL::STask::Volume->find(
        command_interface => $self->command_interface(),
        filter            => {%opts},
    );

    my $path = "/vol/$opts{volume}/test-file";

    my $file_obj = NACL::STask::VolumeFile->create(
        command_interface                    => $vol_obj->command_interface,
        path                                 => $path,
        size                                 => '4k',
        vserver                              => $opts{vserver},
        nacltask_fill_holes                  => 1,
        nacltask_wait                        => 1,
        handle_name_ordinal                  => 1,
        determine_appropriate_nodescope_node => 1,
    );

    Tharn::snooze 15;

    my $vol_host = $vol_obj->get_one_state_attribute('node');
    if ( $vol_host !~ /$vol_obj->command_interface->node/ ) {
        $Log->comment("Determining appropriate nodescope node");
        $opts{command_interface} = NACL::C::Node->find(
            command_interface => $vol_obj->command_interface,
            filter            => { node => $vol_host, },
        );
    }
    my $dblade_name = $vol_obj->construct_dblade_path();
    $dblade_name =~ s/\/vol\///;
    my $dumpblock_resp =
      $opts{command_interface}->get_7m_or_nodescope_apiset->dumpblock_info(
        volume  => $dblade_name,
        fileid  => hex( $file_obj->showfh()->fileid() ),
        blockno => 0,
        level   => 0,
      );

    $Log->comment( "ckse_encrypted : "
          . $dumpblock_resp->get_parsed_output()->[0]->{'ckse-encrypted'} );
    $Log->comment( "ckse_key_idx : "
          . $dumpblock_resp->get_parsed_output()->[0]->{'ckse-key-idx'} );
    $Log->comment(
        "key_id : " . $dumpblock_resp->get_parsed_output()->[0]->{'key-id'} );

    $Log->exit() if $may_exit;
    return {
        ckse_encrypted =>
          $dumpblock_resp->get_parsed_output()->[0]->{'ckse-encrypted'},
        ckse_key_idx =>
          $dumpblock_resp->get_parsed_output()->[0]->{'ckse-key-idx'},
        key_id => $dumpblock_resp->get_parsed_output()->[0]->{'key-id'},
    };
}

# Internal subroutine to initialize node
# volume name or objects along with
# volume encryption

sub _initialize_node_and_volume {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = @_;
    my $volume =
      ref( $opts{volume} )
      ? $opts{volume}->volume
      : $opts{volume};

    my $requested_fields = NACL::CS::Volume->fetch(
        command_interface => $self->command_interface,
        filter            => { volume => $volume },
        requested_fields =>
          [ 'node', 'key-id', 'encrypt', 'vserver', 'encryption-state' ],
    );

    if ( ref( $opts{volume} ) ) {
        $self->encryption_state(
            $opts{volume}->get_one_state_attribute('encryption-state') );
    }
    else {
        my $vol_obj = NACL::STask::Volume->find(
            command_interface => $self->command_interface,
            filter            => { volume => $volume },
        );
        $self->encryption_state(
            $vol_obj->get_one_state_attribute('encryption-state') );
    }

    if ( defined $opts{nacltask_verify} && $opts{nacltask_verify} ) {
        $Log->comment("nacltask_verify will be performed");
        my $dbinfo = $self->_get_ckse_key_idx(
            command_interface => $self->command_interface,
            volume            => $volume,
            vserver           => $requested_fields->vserver,
        );

        $self->key_id( $dbinfo->{key_id} );
        $self->ckse_key_idx( $dbinfo->{ckse_key_idx} );
        $self->ckse_encrypted( $dbinfo->{ckse_encrypted} );

        $Log->comment( "ckse_encrypted        : " . $self->ckse_encrypted );
        $Log->comment( "key id                : " . $self->key_id );
        $Log->comment( "key index             : " . $self->ckse_key_idx );
        $Log->comment( "encryption-state      : " . $self->encryption_state );
    }

    my $node_name = $requested_fields->node();

    $self->encrypt( $requested_fields->encrypt() );
    my $node_obj;
    if ( $node_name ne $self->command_interface->node() ) {
        $node_obj = NACL::C::Node->new( name => $node_name );
        $self->command_interface($node_obj);
    }

    if ( ref $opts{volume} ) {
        $self->volume( $opts{volume} );
    }
    else {
        $self->volume(
            NACL::STask::Volume->new(
                command_interface => $self->command_interface(),
                volume            => $volume,
                vserver           => $opts{vserver},
            )
        );
    }

    if ( $self->operation =~ /rekey/i ) {
        $Log->comment(
            "Initializing instant component for " . $self->operation );
        instant_component(
            component_name    => 'NACL::C::VolumeEncryptionRekey',
            command_interface => $self->command_interface(),
        );
        $self->volume_encryption_component(
            NACL::C::VolumeEncryptionRekey->new(
                command_interface => $self->command_interface(),
                volume            => $self->volume()->volume(),
                vserver           => $self->volume()->vserver(),
            )
        );
    }
    elsif ( $self->operation =~ /conversion/i ) {
        $Log->comment(
            "Initializing instant component for " . $self->operation );
        instant_component(
            component_name    => 'NACL::C::VolumeEncryptionConversion',
            command_interface => $self->command_interface(),
        );
        $self->volume_encryption_component(
            NACL::C::VolumeEncryptionConversion->new(
                command_interface => $self->command_interface(),
                volume            => $self->volume()->volume(),
                vserver           => $self->volume()->vserver(),
            )
        );
    }
    elsif ( $self->operation =~ /secure_purge/i ) {
        $Log->comment(
            "Initializing instant component for " . $self->operation );
        instant_component(
            component_name    => 'NACL::C::VolumeEncryptionSecurePurge',
            command_interface => $self->command_interface(),
        );
        $self->volume_encryption_secure_purge_component(
            NACL::C::VolumeEncryptionSecurePurge->new(
                command_interface => $self->command_interface(),
                volume            => $self->volume()->volume(),
                vserver           => $self->volume()->vserver(),
            )
        );
    }
    else {
        $Log->comment('Error: Invalid operation' . $self->operation );
    }

    $Log->exit() if $may_exit;
}

=head2 C<set_scan_speed>

  my $Vol_Enc_Obj->set_scan_speed($speed);

(Instance method) sets wafl scan speed to the input value

Instance method call applies to rekey method only. Use apiset method call
setting scan speed on conversion scan operations.

This method is ignored for ZAPI calls and works with CLI calls only.

=over

=item Options

=over

=item C<< speed >>

(Required) Set the speed to the value defined in input.

=back

=cut

sub set_scan_speed {
    $Log->enter() if $may_enter;
    my ( $self, $speed ) = @_;

    my $apiset_obj = $self->command_interface()->get_7m_or_nodescope_apiset();
    $Log->comment( "apiset_should_interface : "
          . $self->command_interface()->{apiset_should}->{interface} )
      if (
        defined( $self->command_interface()->{apiset_should}->{interface} ) );

    if ( defined( $self->command_interface()->{apiset_should}->{interface} )
        && (
            $self->command_interface()->{apiset_should}->{interface} =~ /ZAPI/ )
      )
    {
        $Log->comment( "Ignoring set wafl scan speed : apiset_should is ZAPI"
              . $self->command_interface()->{apiset_should}->{interface} );

        $Log->exit() if $may_exit;
        return;
    }

    $Log->comment("Setting wafl scan speed to $speed");
    $apiset_obj->wafl_scan_speed( 'speed' => $speed );
    my $parsed_output = $apiset_obj->wafl_scan_speed()->get_parsed_output();
    my $value         = $parsed_output->[0]->{'wafl_scan_speed'};

    if ( $speed < 0 ) {
        if ( $value != 0 ) {
            $Log->exit() if $may_exit;
            NACL::Exceptions::VerifyFailure->throw(
                "Failed to set the wafl scan speed to negative value");
        }
    }
    elsif ( $speed == 0 ) {
        if ( $value <= 0 ) {
            $Log->exit() if $may_exit;
            NACL::Exceptions::VerifyFailure->throw(
                "Failed to set default wafl scan speed");
        }
    }
    else {
        if ( $value != $speed ) {
            $Log->exit() if $may_exit;
            NACL::Exceptions::VerifyFailure->throw(
                "Could not set the wafl scan speed to $speed");
        }
    }
    $Log->exit() if $may_exit;
} ## end sub set_scan_speed

sub _secure_purge_start {

    $Log->enter() if $may_enter;
    my $pkg = shift;

    my $self;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            nacltask_timeout =>
              { type => SCALAR, optional => 1, default => DEFAULT_TIMEOUT },
            nacltask_wafl_scan_speed =>
              { type => SCALAR, optional => 1, default => 1300, },
            nacltask_wait => { type => BOOLEAN, optional => 1, default => 1, },
            nacltask_verify =>
              { type => BOOLEAN, optional => 1, default => 1, },
            nacltask_retry => { type => BOOLEAN, optional => 1, default => 1 },
        },
        allow_extra => 1,
    );

    if ( !defined $opts{'method-timeout'} ) {
        $opts{'method-timeout'} = DEFAULT_TIMEOUT;
    }

    if ( !$opts{nacltask_wait} && $opts{nacltask_verify} ) {
        $Log->comment( "Cannot verify the "
              . $pkg->operation
              . " operation when nacltask_wait is set to 0" );
        $opts{nacltask_verify} = 0;
    }

    my $node = delete $opts{node} if ( defined $opts{node} );

    $pkg->command_interface( $opts{command_interface} );

    my $secure_purge_status;
    my $poll_required = 1;

    # Initialize scan objects and params
    $pkg->_initialize_node_and_volume(%opts);

    my %nacltask_opts;
    $pkg->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_opts
    );

    if ( $nacltask_opts{nacltask_verify} ) {
        $pkg->old_key_id( $pkg->key_id() );
        $pkg->old_ckse_key_idx( $pkg->ckse_key_idx() );
        $pkg->old_ckse_encrypted( $pkg->ckse_encrypted() );

        $Log->comment( "Volume key_id before secure-purge: "
              . $pkg->old_key_id() );
        $Log->comment( "Volume ckse_key_idx before secure-purge: "
              . $pkg->old_ckse_key_idx() );
        $Log->comment( "Volume ckse_encrypted before secure-purge: "
              . $pkg->old_ckse_encrypted() );
    }

    my $scan_speed = delete $nacltask_opts{nacltask_wafl_scan_speed};
    if ( $scan_speed < 0 && $nacltask_opts{nacltask_wait} ) {
        $Log->warn( "Negative value is provided for nacltask_wafl_scan_speed."
              . " Changing nacltask_wait to 0." );
        $nacltask_opts{nacltask_wait} = 0;
    }

    my $ci  = delete $opts{command_interface};
    my $try = 1;

  RETRY_SECURE_PURGE:
    try {
        $Log->comment( 'Begin ' . $pkg->operation . ' process' );
        $pkg->volume_encryption_secure_purge_component()->start(%opts);
    }
    catch NACL::APISet::Exceptions::CommandFailedException with {
        my $exception_obj = shift;
        $Log->comment( 'Failed to start '
              . $pkg->operation
              . ' scan '
              . $exception_obj->text() );
        $try++;
        if ( $exception_obj->text =~ /it is already in progress/ ) {
            $Log->comment( $pkg->operation . ' is already in-progress' );
            NATE::BaseException->throw(
                    'Failed to start secure purge volume '
                  . $pkg->volume()->volume
                  . "\nError: "
                  . $exception_obj->text );
        }
        elsif ( $opts{nacltask_retry} && $try > 3 ) {
            NATE::BaseException->throw(
                    'Failed to start secure purge after 3 attempts on volume '
                  . $pkg->volume()->volume
                  . "\nError: "
                  . $exception_obj->text );
        }
        elsif ( !$opts{nacltask_retry} ) {
            NATE::BaseException->throw(
                    'Failed to start Secure purge on volume '
                  . $pkg->volume()->volume
                  . "\nError: "
                  . $exception_obj->text );
        }
        else {
            goto RETRY_SECURE_PURGE;
        }
    };

    my $start_timeout            = $opts{'method-timeout'};
    my $endtime                  = time() + $start_timeout;
    my $secp_started             = 0;
    my $secp_start_message_found = 0;

    while ( time() < $endtime ) {

        # Check secure purge status
        if ( !$secp_started ) {

            # Now check if secure purge is running
            my $secp_status = $pkg->_get_secure_purge_status();
            if ( $secp_status == 0 ) {

                # Secure purge is in running state,
                $Log->debug( $pkg->operation . ' is in-progress' );
                $secp_started = 1;
            }
        }

        if ( !$secp_started ) {
            Tharn::snooze POLL_DELTA;
        }

        if ($secp_started) {
            $pkg->_secure_purge_started(1);
            $Log->comment(
                $pkg->operation . ' operation started successfully' );
            last;
        }
    }    # End while

    if ( !$secp_started ) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::Timeout->throw( $pkg->operation
              . ' operation did not start within the timeout period:  '
              . $start_timeout );
    } ## end if ( !$secp_started...

    $self->{volume_encryption_secure_purge_component} =
      $pkg->volume_encryption_secure_purge_component;
    $self->{command_interface}  = $pkg->command_interface;
    $self->{volume}             = $pkg->volume;
    $self->{vserver}            = $pkg->volume->vserver;
    $self->{key_id}             = $pkg->key_id;
    $self->{old_key_id}         = $pkg->old_key_id;
    $self->{old_ckse_key_idx}   = $pkg->ckse_key_idx;
    $self->{old_ckse_encrypted} = $pkg->ckse_encrypted;

    # Depending on the 'nacltask_wait' => 0|1, either wait for
    # secure purge to complete or return
    if ( $nacltask_opts{'nacltask_wait'} ) {
        $Log->comment( $pkg->operation . ' started, waiting for completion' );
        $pkg->_wait_for_secure_purge_completion(
            %opts,
            command_interface => $self->{command_interface},
            nacltask_verify   => $nacltask_opts{nacltask_verify},
        );
    }

    $Log->exit() if $may_exit;
    return bless $self, 'NACL::STask::VolumeEncryption';
}

sub _get_secure_purge_status {

    $Log->enter() if $may_enter;

    my $self = shift;
use Data::Dumper;
$Log->comment ('please print' .
        Dumper $self->volume_encryption_secure_purge_component()->command_interface->node );
    my $secp_state;
    try {
        #   $secp_state =
        #     $self->volume_encryption_secure_purge_component()
        #     ->state( 'method-timeout' => 60 )->status();
        #   $Log->comment( 'secure-purge state: ' . $secp_state );

#$Node_Obj_Hash->{NODE_OBJ}->apiset(interface => "CLI");

$secp_state = $self->volume_encryption_secure_purge_component()->command_interface->apiset()->volume_encryption_secure_purge_show()->get_parsed_output->[0]->{status};
#        $secp_state = $node_obj->apiset()->volume_encryption_secure_purge_show()->get_parsed_output->[0]->{status};
    }
    catch NATE::BaseException with {
        my $exception_obj = shift;

        if (
            (
                $exception_obj->text() =~
                /Volume.*is not found in vserver.*Reason: entry does.*exist/
            )
            || ( $exception_obj->text() =~
                /Secure purge operation is not in progress/i )
            || ( $exception_obj->text() =~
                /There are no entries matching your query/i )
          )
        {
            $secp_state = 0;
        }
        else {
            $Log->comment('Unexpected secure-purge state: ');
            $Log->exit() if $may_exit;
            $exception_obj->throw();
        }
    };

    return -1 if ( !$secp_state );

    if ( $secp_state =~ /failure/ ) {
        $Log->exit() if $may_exit;
        return -1;
    }
    elsif (( $secp_state =~ /success/ )
        || ( $secp_state =~ /aborted/ ) )
    {
        $Log->exit() if $may_exit;
        return 1;
    }
    elsif (( $secp_state =~ /initializing/ )
        || ( $secp_state =~ /snapshots-deleting/ )
        || ( $secp_state =~ /snapshots-deleted/ )
        || ( $secp_state =~ /zombies-draining/ )
        || ( $secp_state =~ /zombies-drained/ )
        || ( $secp_state =~ /batched-free-log-draining/ )
        || ( $secp_state =~ /batched-free-log-drained/ )
        || ( $secp_state =~ /finishing-trash-purge/ )
        || ( $secp_state =~ /finished-trash-purge/ )
        || ( $secp_state =~ /reencrypting/ )
        || ( $secp_state =~ /aborting/ ) )
    {
        $Log->exit() if $may_exit;
        return 0;
    }
    else {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            'Unexpected secure-purge state: ' . $secp_state );
    }
}

sub _volume_secure_purge_show {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;
    my %opts       = @_;

    my $volume  = ( $opts{volume} )  ? $opts{volume}  : $pkg_or_obj->volume();
    my $vserver = ( $opts{vserver} ) ? $opts{vserver} : $pkg_or_obj->vserver();
    my $ci =
      ( $opts{command_interface} )
      ? $opts{command_interface}
      : $pkg_or_obj->command_interface;

    $pkg_or_obj->command_interface($ci);

    # Initialize scan objects and params
    $pkg_or_obj->_initialize_node_and_volume(
        volume  => $volume,
        vserver => $vserver,
    );

    my $secp_state;
    try {
        $secp_state =
          $pkg_or_obj->volume_encryption_secure_purge_component()
          ->state( 'method-timeout' => 60 )->status();
        $Log->comment( 'Secure Purge state:  ' . $secp_state );
    }
    catch NATE::BaseException with {
        my $exception_obj = shift;
        $Log->exit() if $may_exit;
        $exception_obj->throw();
    };
    return $secp_state;
}

=head2 C<abort>

  NACL::STask::VolumeEncryption->secure_purge_abort(
      command_interface   => $command_interface,
      vserver             => $vserver,
      volume              => $volume,
  );

  Or

  $SecurePurge_Obj->secure_purge_abort();

(Class or Instance method) aborts the secure purge operation on the given
volume. Instance method is applicable for secure purge method only.

=over

=item Options

=over

=item C<< command_interface >>

(Required) A component object that represents the host to which to send commands.

=head2 volume

(Required) The name of the volume to abort the secure purge

=head2 vserver

(Required) The name of the vserver containing the volume.

=item C<< method-timeout => $timeout >>

=back

=back

=cut

sub secure_purge_abort {

    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'method-timeout' =>
              { type => SCALAR, optional => 1, default => DEFAULT_TIMEOUT },
            nacltask_verify => { type => BOOLEAN, optional => 1, default => 0 },
        },
        allow_extra => 1,
    );
    my $sp_status = undef;
    my $endtime   = time() + $opts{'method-timeout'};
    my $volume    = ( $opts{volume} ) ? $opts{volume} : $pkg_or_obj->volume();
    my $vserver = ( $opts{vserver} ) ? $opts{vserver} : $pkg_or_obj->vserver();
    my $ci =
      ( $opts{command_interface} )
      ? $opts{command_interface}
      : $pkg_or_obj->command_interface;

    $pkg_or_obj->command_interface($ci);
    $pkg_or_obj->_initialize_node_and_volume(
        volume          => $volume,
        vserver         => $vserver,
        nacltask_verify => $opts{nacltask_verify}
    );
    delete $opts{nacltask_verify};

    try {
        $Log->comment('Abort the Secure Purge operation');
        delete $opts{command_interface};
        $pkg_or_obj->volume_encryption_secure_purge_component()->abort(%opts);
    }
    catch NACL::APISet::Exceptions::CommandFailedException with {
        my $exception_obj = shift;
        $Log->comment(
            'The output of abort command is: ' . $exception_obj->text );
        if (
            (
                $exception_obj->text() =~
                /Volume.*is not found in vserver.*Reason: entry does.*exist/
            )
            || ( $exception_obj->text() =~
                /command failed: Secure purge operation is not in progress/i )
          )
        {
            NATE::BaseException->throw(
                'Failed to abort secure purge operation. Reason:  '
                  . $exception_obj->text );
        }
    };

    while (1) {

        # 1 is Aborted
        # 0 is Aborting
        # -1 failed to Abort
        $sp_status = $pkg_or_obj->_get_secure_purge_status();

        if ( $sp_status == 1 ) {
            $Log->debug('Secure Purge is aborted');
            last;
        }
        else {
            $Log->debug('Secure Purge is not Aborted yet');
            Tharn::snooze 60;
        }

        if ( $endtime < time() ) {
            $Log->comment( 'volume encryption secure purge abort : '
                  . 'Timeout of '
                  . $endtime
                  . ' reached' );
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                'Timeout reached waiting for secure purge to abort.');
        }
    }
    $Log->comment('Volume secure purge aborted successfully');
}

=head2 C<secure_purge_wait_for_completion>

  my $Vol_SP_Obj  = NACL::STask::VolumeEncryption->secure_purge_wait_for_completion(
      command_interface   => $command_interface,
      vserver             => $vserver,
      volume              => $volume,
      nacltask_verify     => 0|1,
  );

  Or

  $SecPur_Obj->secure_purge_wait_for_completion();

(Class or Instance method) waits for the secure purge process completion
on the given volume to complete.

Instance method call applies to secure purge method only.

=over

=item Options

=over

=item C<< command_interface >>

(Required) A component object that represents the host to which to send commands.

=head2 volume

(Required) The name of the volume on which the secure purge should process

=head2 vserver

(Required) The name of the vserver containing the volume.

=item C<< nacltask_verify => 0|1 >>

(Optional) Verifies if the secure purge operation is successful.
For plaintext volumes being converted to encrypted, verifies if C<< -encrypt  >>
field is true and a new key_id is populated for this volume.
For encrypted volumes being converted to plaintext, verifies if  C<< -encrypt >>
field is false and key_id is null.
For encrypted volumes being rekeyed, compares the previous and the new key_id
generated.

=back

=back

=cut

sub _wait_for_secure_purge_completion {

    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'method-timeout' =>
              { type => SCALAR, optional => 1, default => DEFAULT_TIMEOUT },
            nacltask_verify => { type => BOOLEAN, optional => 1, default => 0 },
            nacltask_timeout =>
              { type => SCALAR, optional => 1, default => DEFAULT_TIMEOUT },
        },
        allow_extra => 1,
    );

    my %nacltask_opts;
    $pkg_or_obj->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_opts
    );

    my $endtime   = time() + $nacltask_opts{nacltask_timeout};
    my $sp_status = undef;

    my $volume  = ( $opts{volume} )  ? $opts{volume}  : $pkg_or_obj->volume();
    my $vserver = ( $opts{vserver} ) ? $opts{vserver} : $pkg_or_obj->vserver();
    my $ci =
      ( $opts{command_interface} )
      ? $opts{command_interface}
      : $pkg_or_obj->command_interface;

    $pkg_or_obj->command_interface($ci);

    $pkg_or_obj->_initialize_node_and_volume(
        volume          => $volume,
        vserver         => $vserver,
        nacltask_verify => $nacltask_opts{nacltask_verify},
    );

    # Wait until Secure Purge is complete
    while (1) {
        $sp_status = $pkg_or_obj->_get_secure_purge_status();

        if ( $sp_status == 0 ) {
            $Log->comment('Secure-Purge is in-progress');
        }
        elsif ( $sp_status == 1 ) {
            $Log->comment('Secure-Purge is completed');
            last;
        }

        if ( $endtime < time() ) {
            $Log->comment(
                'Timeout of ' . $nacltask_opts{nacltask_timeout} . ' reached' );
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                'Timeout reached waiting for Secure Purge to complete');
        }
        Tharn::snooze POLL_DELTA;
    }
}

1;
