#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeFile Task Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::VolumeFile;

use strict;
use warnings;

use base qw(NACL::C::VolumeFile NACL::STask::STask);
use NACL::STask::_Mixins::AggregateVolumeFile qw(purge
    dumpblock_info _calculate_timeout_based_on_size
    _get_aggr_or_vol_name_from_path
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# Create STask extensions of the component methods that do nothing
# but default "determine_appropriate_nodescope_node" to 1
BEGIN {
    no strict 'refs';
    foreach my $method (qw(read write_file showfh)) {
        *{$method} = sub {
            $Log->enter() if $may_enter;
            my ($pkg_or_obj, @args) = @_;
            my $super = "SUPER::$method";
            my $ret   = $pkg_or_obj->$super(
                determine_appropriate_nodescope_node => 1,
                @args
            );
            $Log->exit() if $may_exit;
            return $ret;
        };
    } ## end foreach my $method (qw(read write_file showfh))
    use strict 'refs';
} ## end BEGIN

use NATE::Time qw(timeout2time);
use NACL::Exceptions::Timeout;

use Params::Validate qw(:all);

use NATE::BaseException qw(:try);
use NACL::Exceptions::NoElementsFound;
use NATE::Exceptions::Argument;
use NACL::ComponentUtils qw(_optional_scalars Dumper);

use NACL::STask::VolumeFileMove;
use NACL::STask::VolumeFileCopy;
use NACL::CS::IOfence;

use NACL::C::UnitNormalization;
use NACL::GeneralUtils qw(nacl_method_retry);

=head1 NAME

NACL::STask::VolumeFile

=head1 DESCRIPTION

C<NACL::STask::VolumeFile> provides a number of well-defined but
potentially complex or multi-step methods related to Volume Files in
ONTAP.

It builds on top of, and is a derived class of C<NACL::C::VolumeFile>,
and so it also provides methods that are more in the scope of
individual VolumeFile related commands.  See C<NACL::C::VolumeFile> for
details.  This also means that a C<NACL::STask::VolumeFile> package may
generally be used in place of C<NACL::C::VolumeFile> package.

=head1 CLEANUP METHODS

Cleanup can be registered for the following methods

Cleanup methods are,
    VolumeFile Method         Cleanup Method
    ----------------------------------------
    create                    purge

=head1 METHODS

    # Creating a single file
    my $file_obj = NACL::STask::VolumeFile->create(
        command_interface => $ci,
        path              => $file_path,
        vserver           => $vserver,
        nacltask_enable_space_reserve => 0 | 1, # Default is 0
        nacltask_wait       => 0 | 1, # Default is 1
        nacltask_fill_holes => 0 | 1 # Default is 1
        nacltask_to_cleanup      => 1,          # default 0,
        nacltask_cleanup_manager => $CleanupObj,
        %other_opts
    );

    # Creating multiple files by using "filecount"
    # This example creates 3 files
    my @file_objs = NACL::STask::VolumeFile->create(
        command_interface => $ci,
        path              => $file_path,
        vserver           => $vserver,
        filecount         => 3,
        nacltask_enable_space_reserve => 0 | 1, # Default is 0
        nacltask_wait       => 0 | 1, # Default is 1
        nacltask_fill_holes => 0 | 1 # Default is 1
        nacltask_to_cleanup      => 1,          # default 0,
        nacltask_cleanup_manager => $CleanupObj,
        %other_opts
    );

(Class method) Create files by using internal mkfile tool.
This method provides additional services beyond what's provided by the
component method, as controlled and described by the new
C<nacltask_enable_space_reserve>, C<nacltask_fill_holes> and
C<nacltask_wait> arguments. Further, this method also defaults to
determining the appropriate node on which to issue the Nodescope commands.
If the size is not provided, this method defaults to creating a zero-length
file.

=over

=item Punching bad holes

The ability to punch VBN-BAD holes was introduced in FS.0 (burt 683120). This
method provides provisions to punch VBN_BAD holes at creation time (uses the
dblade "file punchhole -b" command to achieve this).

=over

=item Explicitly specifying holes to punch

This can be done by using the C<hole-range-info> argument. The example
here punches two VBN_BAD holes: the first from offset 0 for length 4096 bytes,
the second hole from offset 8192 for length 8192 bytes.

    NACL::STask::VolumeFile->create(
        %other_options,
        'hole-range-info' => [
            {'hole-range-start' => 0, 'hole-range-size' => 4096},
            {'hole-range-start' => 8192, 'hole-range-size' => 8192},
        ],
    );

=item Specifying probability of punching hole

If we want to specify the percentage of files in the data-set to have
holes punched, we can do so either through the script or from the
command-line. For both forms, the offsets, sizes and number of holes to
be punched is determined randomly.

=over

=item Specifying probability in the script

This examples specifies that there's a 50% probability of the files
being punched.

    NACL::STask::VolumeFile->create(
        %other_options,
        nacltask_vbn_bad_percentage => 50
    );

=item Specifying probability on the command-line

While running the script, we can specify the C<NACLTASK_VBN_BAD_PERCENTAGE>
parameter. This way, existing scripts will create files with punched holes.

    ntest foo.thpl NACLTASK_VBN_BAD_PERCENTAGE=40

=back

=back

=back

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Mandatory)
The command interface.

item C<< path => $path >>

(Mandatory)
Path to the file. This is of the form "/vol/$volname/$filename".

=item C<< vserver => $vserver_name >>

(Required for CMode, Ignored for 7Mode)
The vserver on which the volume resides.

=item C<< nacltask_enable_space_reserve => 0 | 1 >>

(Optional, defaults to 0)
If set to 1, enables file reservation on the created file(s).

=item C<< nacltask_fill_holes => 0 | 1 >>

(Optional, defaults to 1, applicable only for debug builds)
If set to 0, this method does not fill file holes. It does
so by creating a file of size 0 and then uses "testsetattr" to reserve
the requested amount of space.

Note that the "testsetattr" command is available only on debug builds,
so for non-debug builds this flag is ignored.

=item C<< nacltask_wait => 0 | 1 >>

(Optional, defaults to 1)
The mkfile command itself is non-blocking, so it's possible that it has
returned control before the file has been created.

If the value of this argument is 1 (the default), this method waits till the
file is created before returning.

=item C<< determine_appropriate_nodescope_node => 0 | 1 >>

(Optional, defaults to 1)
If set to 1 (the default) the method automatically determines the node
on which the volume resides and invokes the Nodescope commands in the
context of that node.

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup)
Flag indicating if this operation needs to be registered for clean up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=item C<< 'hole-range-info' => [ { 'hole-start' => $start1, 'hole-size' => $size1 }, {...}, ... ] >>

(Optional, Arrayref of hashrefs)
If specified, file(s) created will have VBN_BAD holes punched.

Information of the holes to be punched. Each entry in the array-reference
is a hash-reference and each such hash-reference describes one hole to be
punched. Each hash-reference should have the following keys:

* 'hole-start': Starting offset of the hole. Range: [0..2^63-1]

* 'hole-size': Size of the hole in terms of bytes. If punching a hole WAFL
presently limits this value to 4MB. Range: [0..2^63-1]

=item C<< nacltask_vbn_bad_percentage => $percentage >>

(Optional, a number between 0 to 100)
If specified, denotes the probability of the file(s) created having VBN_BAD
holes punched.

=item C<< permission => $permission >>

(Optional)
If passed, unix-permission of files will be set to $permission value.

=item C<< "method-timeout" => $timeout >>

(Optional) As component method-timeout, controls how long the command
will wait before completing.

=item Other options

All of the other options accepted by
L<< NACL::C::VolumeFile->create|lib-NACL-C-VolumeFile-pm/create >>
are accepted by this method.

=back

=back

=over

=item Arguments defaulted/auto-determined

=over

=item C<size>

If not provided, this is defaulted to 0 (i.e. a 0-length file is created).

=back

=back

=over

=item Exceptions

=over

=item C<NATE::Exceptions::Argument>

This type of exception is thrown when vserver argument is missing while running in CMode.

=item C<NACL::Exceptions::Timeout>

This type of exception is thrown when file was not created even after waiting for $timeout seconds.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            nacltask_enable_space_reserve => {type => SCALAR, default => 0},
            nacltask_fill_holes           => {type => SCALAR, default => 1},
            size                          => {type => SCALAR, default => 0},

            # These two options are documented by the component, but since
            # we're using them below, they have to be of the appropriate type,
            # which is why they're being validated here as well.
            # (Further, the default for determine_* in the component is 0,
            # for the STask we want it to default to being 1)
            vserver => {type => SCALAR | UNDEF, optional => 1},
            determine_appropriate_nodescope_node =>
                {type => SCALAR, default => 1},

            nacltask_wait => {type => SCALAR, default => 1},
            $pkg->_cleanup_validate_spec(),
            nacltask_vbn_bad_percentage => {type => SCALAR, optional => 1},
            'hole-range-info'           => {type => ARRAYREF, optional => 1},
            'permission'                => {type => SCALAR, optional => 1},
        },
        allow_extra => 1,
    );
    
    # suffix of 'b' in size is blocks (each one is 4k)
    if ($opts{size} =~ /(\d+)(b|B)/) {
        $opts{size} = ($1 * 4).'k';
    }
    # This is task specific param and should not be
    # passed to component
    my $permission = delete $opts{permission};

    my $enable_space_reserve = delete $opts{nacltask_enable_space_reserve};
    my $fill_holes           = delete $opts{nacltask_fill_holes};
    my $do_not_fill_holes    = !$fill_holes;

    if (!defined $opts{nacltask_vbn_bad_percentage}) {
        my $vbn_bad_param = Tharn::param('NACLTASK_VBN_BAD_PERCENTAGE');
        if (defined $vbn_bad_param) {
           $opts{nacltask_vbn_bad_percentage} = $vbn_bad_param;
        }
    }
    my %punch_args;
    $pkg->_hash_move(
        source => \%opts,
        target => \%punch_args,
        move   => [qw(hole-range-info nacltask_vbn_bad_percentage)]
    );

    # If either "hole-range-info" or "nacltask_vbn_bad_percentage" is
    # provided, then we might need to punch holes.
    my $need_to_punch_bad_holes = (keys %punch_args) ? 1 : 0;
    my $percent = delete $punch_args{nacltask_vbn_bad_percentage};

    my $ci      = $opts{command_interface};
    my $vserver = $opts{vserver};

    my $version_manager = $ci->version_manager();
    unless ($version_manager->is_debug()
        && !$version_manager->is_promoted()) {
        $do_not_fill_holes = 0;
    }

    # Need to wait for creation of the file to be able to
    # punch holes or to set the permission
    my $wait = delete $opts{nacltask_wait} || $need_to_punch_bad_holes || defined $permission;

    # Can only set the space reserve once the file has been created, so
    # override wait to 1 if use space reserve is 1
    my $multiple_nodescope_commands;
    if ($do_not_fill_holes || $need_to_punch_bad_holes) {
        $multiple_nodescope_commands = 1 if ($ci->is_cmode());
        $wait = 1;
    }

    my $desired_size;
    if ($do_not_fill_holes) {
        $desired_size = $opts{size};
        $opts{size} = 0;
    }

    unless (defined $opts{'method-timeout'}) {
        $opts{'method-timeout'}  = $pkg->_calculate_timeout_based_on_size(
            size => $opts{size},
            filecount => $opts{filecount},
        );
    }

    if ($ci->is_cmode() && !defined $vserver) {
        my $me = (caller(0))[3];
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw("$me: The 'vserver' argument is "
                . 'required in CMode but was not provided');
    } ## end if ( $ci->is_cmode() &&...)

    # Get common component params: apiset_must, apiset_should, etc
    my %common_params = ();
    $pkg->_copy_common_component_params(
        source => \%opts,
        target => \%common_params
    );
    my (%opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup );
	$pkg->_move_common_cleanup_opts(
          source  => \%opts,
          target  => \%opts_for_cleanup,
    );

    my %nodescope_override;
    if (   $opts{determine_appropriate_nodescope_node}
        && $multiple_nodescope_commands
        && !defined $opts{'nodescope-node-name'}) {

        # If this is set then we need to make multiple nodescope calls.
        # Since all operations are operating on the same volume, they're
        # all operating on the same node, so let's compute this once,
        # rather than require every call to recompute the value.
        $nodescope_override{'nodescope-node-name'}
            = $pkg->get_local_node_name(
            command_interface => $ci,
            vserver           => $vserver,
            path              => $opts{path},
            %common_params
            );
    } ## end if ( $opts{determine_appropriate_nodescope_node...})

    # If "filecount" is provided, then multiple file objects are returned
    my @files = $pkg->SUPER::create(%opts, %nodescope_override);

    # Path will already be present in object returned by create
    delete $opts{'path'};

    my $is_sksim = $ci->version_manager()->is_sksim();
    foreach my $file (@files) {
        if ($wait && !$is_sksim) {
            my $end_time = timeout2time($opts{'method-timeout'});

            while (1) {
                my $found = $file->find(allow_empty => 1, %common_params);
                if ($found) {
                    last;
                } elsif (time() > $end_time) {
                    my $file_str = $file->path();
                    if ($ci->is_cmode()) {
                        $file_str .= " on $opts{vserver}";
                    }
                    $Log->exit() if $may_exit;
                    NACL::Exceptions::Timeout->throw("The file $file_str "
                            . 'did not get created even after waiting for '
                            . "$opts{'method-timeout'} seconds");
                } ## end elsif ( time() > $end_time)
                Tharn::snooze(10);
            } ## end while (1)
        } ## end if ($wait)

        if ($do_not_fill_holes) {
            $ci->get_7m_or_nodescope_apiset()->testsetattr(
                path        => $file->dblade_path(),
                'file-size' => $desired_size,
                %nodescope_override
            );
        } ## end if ($do_not_fill_holes)

        if ($enable_space_reserve) {
            $file->reservation('is-enabled' => 'enable', %common_params);
        }

        if ($need_to_punch_bad_holes) {
            $punch_args{'punch-bad'} = 1;

            if (exists $punch_args{'hole-range-info'}) {
                $file->punch_hole(%punch_args, %common_params,
                    %nodescope_override);
            } elsif (rand(100) < $percent) {

                # Punch bad holes if the randomly chosen number is less than
                # the user-specified percent. The algorithm here to randomly
                # choose which blocks to punch is loosely
                # based on the algorithm used by the WAFL automation team.

                my @hole_range_info;
                my $file_size_bytes;
                try {
                    $file_size_bytes = $pkg->convert_to_bytes($opts{size});
                } otherwise {
                   # Would be because size was not provided/size was provided but
                   # with an invalid size suffix. Either case will result in
                   # the command failing, so let's suppress this exception
                };

                # Each hole needs to be at least one block (4KB), so if the
                # file is not at least 4KB big, then nothing to do.
                if (defined $file_size_bytes && $file_size_bytes >= 4096) {
                    my $offset = 0;
                    my $length;

                    my $calculate_remaining_blocks = sub {
                        my $remaining_length = $file_size_bytes - $offset;
                        return int($remaining_length / 4096);
                    };

                    while (1) {
                        $length = int(rand 1000) * 4096;

                        # If length was calculated as 0, go to next iteration
                        # (a 0-length hole makes no sense).
                        next if !$length;
                        if ($offset + $length > $file_size_bytes) {

                            # Length calculated will end up exceeding the size
                            # of the file. Calculate the number of blocks
                            # remaining and set the length to this value.
                            my $remaining_blocks
                                = $calculate_remaining_blocks->();
                            if ($remaining_blocks) {

                                # Set hole-size to be remaining blocks * 4096
                                # (this is the length of the remaining blocks)
                                # and exit the loop.
                                push @hole_range_info,
                                    {
                                    'hole-start' => $offset,
                                    'hole-size'  => $remaining_blocks * 4096
                                    };
                                last;
                            } else {

                                # No blocks remaining; we're done
                                last;
                            }
                        } else {
                            push @hole_range_info,
                                {
                                'hole-start' => $offset,
                                'hole-size'  => $length
                                };
                            $offset += $length;
                            my $remaining_blocks
                                = $calculate_remaining_blocks->();

                           # Randomly skip some number of blocks. If number of
                           # remaining blocks is >100, then randomly skip
                           # upto 100 blocks, else randomly skip upto
                           # $remaining_blocks number of blocks.
                            my $rand_skip
                                = ($remaining_blocks > 100)
                                ? 100
                                : $remaining_blocks;
                            my $skip = int(rand $rand_skip) * 4096;
                            $offset += $skip;
                        }
                    }
                    $file->punch_hole(
                        %punch_args, %common_params,
                        'hole-range-info' => \@hole_range_info,
                        %nodescope_override
                    );
                }
            }
        }

        # Register this resource with the cleanup manager for cleanup.
        $file->_copy_common_opts_for_cleanup(
            'source'             => {%opts, %opts_for_cleanup},
            'target'             => \%opts_for_register,
            'nacltask_to_cleanup'=> \$nacltask_to_cleanup,
            'to_cleanup'         => 'purge',
        );

        $file->_register_for_cleanup(%opts_for_register)
              if ($nacltask_to_cleanup);
    } ## end foreach my $file (@files)
    my %retry_opts;
    $pkg->_copy_retry_params(
                source => \%opts,
                target => \%retry_opts
    );
    # set unix-permission for each file if requested
    if (defined $permission) {
        $Log->debug("Set file permission is requested. Setting permission " .
                  $permission . " for all created files");
        foreach my $file_obj(@files) {
            $file_obj->set_permission(
                permission => $permission,
                %common_params,
            );
            ##Verification is introduced due to Burt966461
            nacl_method_retry(
                code => sub {$file_obj->verify_state(perm => $permission)},
                exceptions => 'NACL::Exceptions::VerifyFailure',
                %retry_opts,
            );
        }
    }

    $Log->exit() if $may_exit;
    return wantarray ? @files : $files[0];
}    # end sub create

=head2 dumpblock_info

    # Class method: providing the file path. The fileid and volume
    # name are determined by the method.
    my $dumpblock_cs_obj  = NACL::STask::VolumeFile->dumblock_info(
        command_interface => $command_interface,
        path              => $file_path,
        %other_opts
    );

    # Class method: providing the volume name and fileid explicitly
    my $dumpblock_cs_obj  = NACL::STask::VolumeFile->dumblock_info(
        command_interface => $command_interface,
        fileid            => $fileid,
	    volume            => $volume_name,
        %other_opts
    );

    # Instance method: let the method determine the fileid. The volume
    # is automatically extracted from the path attribute of the object.
    my $dumpblock_cs_obj = $file_obj->dumpblock_info(%opts);

    # Instance method: explicitly providing the fileid. Again, the volume
    # name is extracted from the path attribute stored in the object.
    my $dumpblock_cs_obj = $file_obj->dumpblock_info(fileid => $fileid, %other_opts);

(Class or instance method) This method returns a
L<NACL::CS::VolumeFileDumpblockInfo|lib-NACL-CS-VolumeFileDumpblockInfo-pm> object
containing information about the specific block of the file. It extends the
underlying component method and provides the following additional
functionality:

=over

=item *

If the file path is provided (or if it's invoked with an object instance)
then this method can automatically determine the C<fileid> value (does so
by using the C<showfh> method) and the C<volume> value (done by
extracting it from the path).

=item *

Support for both class and instance method invocation

=item *

Defaults to determining the appropriate node on which to invoke
the Nodescope commands.

=back

=over

=item Options

=over

=item C<< path => $file_path >>

The full path of the file. This is expected to be in the form
"/vol/$volname/$filename". If this argument is provided, then the
volume is determined from the path, and the fileid is determined
by invoking the C<showfh> method. In other words, if this argument is
provided, then we do B<not> need to provide the arguments C<fileid>
and C<volume>.

=item C<< determine_appropriate_nodescope_node => 0 | 1 >>

(Optional, default to 1)
If set to 1 (the default) determines the appropriate node on which to invoke
the Nodescope commands.

=item Other options

All of the other various options to
L<< NACL::C::VolumeFile->dumpblock_info | lib-NACL-C-VolumeFile-pm/dumpblock_info >>

=back

=back

=over

=item Exceptions

=over

=item C<NATE::Exceptions::Argument>

This type of exception is thrown when either of the option fileid or path is missing OR either of the option volume or file is missing.

=back

=back

=cut


=head2 read, write_file, showfh

Provides the same facilities as the respective component methods but
defaults to determining the appropriate node (if run through Nodescope).
(This can be disabled by setting C<determine_appropriate_nodescope_node> to 0)

=head2 purge

    # Class method; purging a single file
    NACL::STask::VolumeFile->purge(
        command_interface => $ci,
        vserver           => $vserver_name, # Required for CMode
        path              => $path,
        nacltask_wait     => 0 | 1 # Defaults to 1
    );

    # Class method; purging multiple files
    NACL::STask::VolumeFile->purge(
        command_interface => $ci,
        vserver           => $vserver_name, # Required for CMode
        paths             => \@paths,
        nacltask_wait     => 0 | 1 # Defaults to 1
    );

    # Instance method (can only purge a single file with the instance method)
    $file_obj->purge(nacltask_wait => 0 | 1);

Deletes the file(s) and waits for the file(s) to be deleted (defaults to
waiting for 2 minutes). If run through Nodescope-CLI, then it defaults
to determining the appropriate node to run on.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required for class method, Not Applicable for Instance method)
The command interface against which to run the commands.

=item C<< vserver => $vserver_name >>

(Required for class method for Nodescope, Not Applicable for Instance method
or for 7Mode)
The vserver on which the volume hosting the file resides.

=item C<< path => $path >>

For class method call, either C<path> or C<paths> should be provided.
Not Applicable for Instance method call.

The full path to the file.

=item C<< paths => \@paths >>

For class method call, either C<path> or C<paths> should be provided.
Not Applicable for Instance method call.

An array-reference containing a list of file-paths to be purged.

=item C<< nacltask_wait => 0 | 1 >>

(Optional, defaults to 1)
Wait till the file(s) get purged. Defaults to waiting. The default timeout
is 120 seconds (i.e. 2 minutes). This can be overridden using C<method-timeout>.

=item C<< polling_interval => $interval_in_seconds >>

(Optional, Defaults to 10 seconds, Applicable only if nacltask_wait = 1)
The interval at which to poll for whether the file(s) have been purged.

=item C<< determine_appropriate_nodescope_node => 0 | 1 >>

(Optional, defaults to 1)
For Nodescope-CLI, determines the appropriate node on which to invoke the
command.

=item Other options

All the other options accepted by this method are documented in
L<< NACL::C::VolumeFile->delete|lib-NACL-C-VolumeFile-pm/delete >>

=back

=back

=cut


=head2 copy_start, move_start

Documented together since they support similar features and options.

These are convenience methods that invoke
L<< NACL::STask::VolumeFileCopy->start|lib-NACL-STask-VolumeFileCopy-pm/start >>
and
L<< NACL::STask::VolumeFileMove->start|lib-NACL-STask-VolumeFileMove-pm/start >>
respectively. As a result, they provide similar options and facilities as those
STask methods.

Shown below are examples of using this method using the "standard" and
"enhanced" forms of the call (documentation in the STask describes these two
forms of passing arguments):

    # All forms return an object of type NACL::STask::VolumeFileCopy

    # The "standard" form
    my $copy_obj = $file_obj->copy_start(
        'destination-paths' => $vs . ':' . $vol_name . '/' . $dest_file,
        %other_opts
    );

    # The "enhanced" form (1): destination on the same volume as source
    my $copy_obj = $file_obj->copy_start(
        destination_filenames => $dest_file,
        %other_opts
    );

    # The "enhanced" form (2): destination on a different volume than the source
    my $copy_obj = $file_obj->copy_start(
        destination_filenames => $dest_file,
        destination_volume    => $vol_obj,
        %other_opts
    );

All of the 3 forms documented above are equally possible for C<move_start>
(though for C<move_start> an object of type C<NACL::STask::VolumeFileMove>
is returned).

The source-path is constructed using details available in the file object
and the command_interface is also extracted from the file object, so these
do not need to be passed in the call.

All other options accepted by the respective C<start> methods are accepted
by these methods as well. For example:

    # Similar to above, but don't wait for completion and with an
    # explicitly defined maximum cutover time
    my $copy_obj = $file_obj->copy_start(
        destination_filenames => $dest_file,
        nacltask_wait         => 0,
        'max-cutover-time'    => $time
    );

These methods also accept an additional C<destination_object> argument.
This should be specified as a scalar-reference. It gets populated
with a file object representing the destination path. It can be used like this:

    my $dest_file_obj;
    my $copy_obj = $file->copy_start(
        destination_filenames => $dest_file_name,
        destination_object    => \$dest_file_obj,
        %other_opts
    );
    # $dest_file_obj is now an object of type NACL::STask::VolumeFile
    # representing the destination file provided

=cut

sub copy_start {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my $copy_obj = $self->_move_copy_start_helper(@args,
        _package => 'NACL::STask::VolumeFileCopy');
    $Log->exit() if $may_exit;
    return $copy_obj;
} ## end sub copy_start

sub move_start {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my $move_obj = $self->_move_copy_start_helper(@args,
        _package => 'NACL::STask::VolumeFileMove');
    $Log->exit() if $may_exit;
    return $move_obj;
} ## end sub move_start

sub _move_copy_start_helper {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            _package           => {type => SCALAR},
            destination_object => {type => SCALARREF, optional => 1},
        },
        allow_extra => 1
    );
    my $package            = delete $opts{_package};
    my $destination_object = delete $opts{destination_object};

    my %obj_attributes = $self->get_pk_hash_with_ci();
    my $vserver        = delete $obj_attributes{vserver};
    my $path           = delete $obj_attributes{path};

    my $dest_file_obj;
    if (exists $opts{'destination-paths'}) {

        # Options are provided in the "standard" form

        # Remove the leading "/vol/". Example: "/vol/volname/filename" becomes
        # "volname/filename"
        $path =~ s/^\/vol\///;
        $opts{'source-paths'} = "$vserver:$path";
        $opts{command_interface} = $obj_attributes{command_interface};
    } else {

        # Options are provided in the "enhanced" form
        $opts{source_volume} = $self->get_containing_volume_obj();
        $opts{source_filenames}
            = $self->get_file_or_dir_name_from_path(path => $path);
        unless (exists $opts{destination_volume}) {

          # Default the destination volume to be the same as the source volume
            $opts{destination_volume} = $self->get_containing_volume_obj();
        }
    } ## end else [ if ( exists $opts{'destination-paths'...})]

    my $obj = $package->start(%opts);

    if ($destination_object) {
        if (exists $opts{'destination-paths'}) {
            $$destination_object = $self->_form_dest_obj_from_path(
                'destination-paths' => $opts{'destination-paths'});
        } else {
            my $vol_obj = $opts{destination_volume};
            my $vol     = $vol_obj->volume();
            my $file;
            if (ref $opts{destination_filenames}) {
                $file = $opts{destination_filenames}->[0];
            } else {
                $file = $opts{destination_filenames};
            }
            $$destination_object = $self->get_package_name()->new(
                command_interface => $self->command_interface(),
                vserver           => $vol_obj->vserver(),
                path              => "/vol/$vol/$file"
            );
        } ## end else [ if ( exists $opts{'destination-paths'...})]
    } ## end if ($destination_object)

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub _move_copy_start_helper

=head2 is_inline

    my $boolean = NACL::STask::VolumeFile->is_inline(
        command_interface => $ci,
        vserver           => $vs_name,
        path              => $path
    );

                          OR

    my $boolean = $file_obj->is_inline();

(Class or Instance Method)
Returns 1 if the file is an inline file. An "inline file" is one whose payload
is within the inode. In other words, it is a file which is smaller than 65
bytes.

=cut

sub is_inline {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(params => \@args);
    my %filter;
    $pkg_or_obj->_move_primary_keys(
        source => \%opts,
        target => \%filter
    );
    my $size = NACL::CS::VolumeFile->fetch(%opts, filter => \%filter)->file_size();
    my $is_inline;
    if ($size > 64) {
        $is_inline = 0;
    } else {
        $is_inline = 1;
    }
    $Log->exit($is_inline);
    return $is_inline;
}

=head2 parents

    @parents = $volfile_obj->parents();

( Class or Instance method ) This method returns the list containing the names of
the parent objects of this resource. Used for finding dependencies between resources.

=cut

sub parents {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;
    return qw(NACL::STask::Volume);
}

=head2 iofence_get

# Instance method, get the values
    $volfile_obj->iofence_get();

(Instance method only)

Uses a 7Mode/Nodescope APISet.

This method is used to get the values for associated with iofence operation.
It will return an array of object with attributes refcnt,proto_fmode,type.

This method returns objects of type
L<NACL::CS::IOfence|lib-NACL-CS-IOfence-pm>.

=over

=item Options

=over

=item C<< type => $type >>

Type of the iofence. Valid values are
[-t <1=ondisk_rw (default) 2=incore_rw 3=incore_w>]

=back

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

This type of exception is thrown in case APISet call fails

=item C<APISet exceptions>

This method propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub iofence_get {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my %opts = $self->_common_validate_with(
        params => \@args,
        additional_spec => {
            type => {type => SCALAR, default => 1},
        },
    );
    my $ci = $self->command_interface();
    my $apiset = $ci->get_7m_or_nodescope_apiset();

    my %common_params = ();
    $self->_copy_common_component_params(
        source => \%opts,
        target => \%common_params
    );
    my $node = $self->get_local_node_name(%common_params);

    my $response;
    try {
        $response = $apiset->iofence_get(
            type                  => $opts{type},
            'nodescope-node-name' => $node,
            path                  => $self->dblade_path(),
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $ex = shift;
        NACL::Exceptions::NoElementsFound->throw(
            "No Matching elements found. Error is " . $ex->text()
        );
    };

    my $output   = $response->get_parsed_output();
    my @state_objs;
    foreach my $row (@$output) {
        my $obj = NACL::CS::IOfence->new(command_interface => $ci);
        $obj->_set_fields(row => $row);
        push @state_objs, $obj;
    }

    $Log->exit() if $may_exit;
    return @state_objs;
}

=head2 iofence_clear

# Instance method, get the values
    $volfile_obj->iofence_clear();

(Instance method only)

Uses a 7Mode/Nodescope APISet.

This method is used to clear iofence for the file associated with $volfile_obj.
It returns undef.

=over

=item Options

=over

=item C<< type => $type >>

Type of the iofence. Valid values are
[-t <1=ondisk_rw (default) 2=incore_rw 3=incore_w>]

=back

=item Exceptions

=over

=item C<APISet exceptions>

This method propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub iofence_clear {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my %opts = $self->_common_validate_with(
        params => \@args,
        additional_spec => {
            type => {type => SCALAR, default => 1},
        },
    );
    my $ci = $self->command_interface();
    my $apiset = $ci->get_7m_or_nodescope_apiset();

    my %common_params = ();
    $self->_copy_common_component_params(
        source => \%opts,
        target => \%common_params
    );
    my $node = $self->get_local_node_name(%common_params);

    $apiset->iofence_clear(
        type                  => $opts{type},
        'nodescope-node-name' => $node,
        path                  => $self->dblade_path(),
    );

    $Log->exit() if $may_exit;
}

=head2 iofence_set

# Instance method, get the values
    $volfile_obj->iofence_clear();

(Instance method only)

Uses a 7Mode/Nodescope APISet.

This method is used to set iofence for the file associated with $volfile_obj.
It returns undef.

=over

=item Options

=over

=item C<< type => $type >>

Type of the iofence. Valid values are
[-t <1=ondisk_rw (default) 2=incore_rw 3=incore_w>]

=back

=item Exceptions

=over

=item C<APISet exceptions>

This method propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub iofence_set {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my %opts = $self->_common_validate_with(
        params => \@args,
        additional_spec => {
            type => {type => SCALAR, default => 1},
        },
    );
    my $ci = $self->command_interface();
    my $apiset = $ci->get_7m_or_nodescope_apiset();

    my %common_params = ();
    $self->_copy_common_component_params(
        source => \%opts,
        target => \%common_params
    );
    my $node = $self->get_local_node_name(%common_params);

    $apiset->iofence_set(
        type                  => $opts{type},
        'nodescope-node-name' => $node,
        path                  => $self->dblade_path(),
    );

    $Log->exit() if $may_exit;
}


=head2 clear_error

    $response_obj = NACL::STask::VolumeFile->clear_error(
        command_interface  => $ci,
        path               => $file_path,
        vserver            => $vserver_name, # Required for CMode
        inaccessible       => 1
        %other_option
    );

            or

    $response_obj = $file_obj->clear_error(
        %other_option
    );

(Class or Instance Method)
Clear errors on blocks of the specified file and returns the response object,
which is returned by the Component.

Supports 7Mode/Nodescope CLI.
Invokes "file_clear_error" for 7Mode/Nodescope CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for Instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< path => $file_path >>

(Required for class method, Not Applicable for Instance method)
File path

=item C<< vserver => $vserver_name >>

(Required for class method in CMode, Not Applicable for Instance method or for 7Mode)
The vserver on which the file is present.

=item Other options

All of the other options accepted by
L<NACL::C::VolumeFile::clear_error|lib-NACL-C-VolumeFile-pm/clear_error>.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VolumeFile::DoesNotExist>

This type of exception is thrown when an attempt is made to
clear_error on the volumeFile that doesn't exists.

=back

=back

=cut

sub clear_error {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            $pkg_or_obj->_nodescope_options_validate_spec(),
            nacltask_wait  => {type => BOOLEAN, default => 1},
        },
        allow_extra => 1
    );

    my $wait = delete $opts{nacltask_wait};

    my %common_params;
    $pkg_or_obj->_copy_common_component_params_with_ci(
        source => \%opts,
        target => \%common_params
    );

    # For C-Mode we need a node-name where the file_clear_error
    # command is executed, since we need this node for the
    # _wait_for_clear_error_scanner to grep the log files.
    if ( $opts{command_interface}->is_cmode() ) {

        my $node_and_new_path = $pkg_or_obj->_get_node_and_name_ordinal(%opts);

        if ( defined $node_and_new_path->{'nodescope-node-name'} ) {
            $opts{'nodescope-node-name'} = $node_and_new_path->{'nodescope-node-name'};
        }

        if ( defined $node_and_new_path->{'path'} ) {
            $opts{'path'} = $node_and_new_path->{'path'};
        }

        # If we got the nodescope-node-name and name-ordinal path
        # no need to get again in SUPER::clear_error method
        $opts{'handle_name_ordinal'} = 0;
        $opts{'determine_appropriate_nodescope_node'} = 0;
    }

    my $output = $pkg_or_obj->SUPER::clear_error(%opts);

    if ($wait) {
        $pkg_or_obj->_wait_for_clear_error_scanner(
            file => $output->[0]->{log_file},
            'nodescope-node-name' => $opts{'nodescope-node-name'},
            %common_params,
        );
    }

    $Log->exit() if $may_exit;
    return $output;
}

# private Method
# should not be called externally, hence not documented.
sub _wait_for_clear_error_scanner {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    my $method_time_out = $opts{'method-timeout'} || 300;
    my $time = time() + $method_time_out;
    my $complete = 0;

    my $apiset = $opts{command_interface}->get_7m_or_nodescope_apiset();
    my @allowed_options = $apiset->get_allowed_options_for_command(command => 'rdfile');
    push @allowed_options, 'nodescope-node-name';

    my %api_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%api_args,
        copy   => \@allowed_options,
    );

    while($time > time()) {
        my $res = $apiset->rdfile(%api_args);

        my $contents = $res->get_parsed_output()->[0]->{contents};

        if ($contents =~ m/File check completed/) {
            $complete = 1;
            last;
        } else {
            Tharn::snooze 15;
        }
    }
    if(!$complete) {
        NATE::Exceptions::Timeout->throw("file clear error scaner is not able to ".
            "complete within $method_time_out seconds.");
    }

    $Log->exit() if $may_exit;

}


=head2 set_permission

Method to set permission of file/dir. Full documentation can be found here

L<< NACL::C::_Mixins::VolumeFileDirectory->set_permission|lib-NACL-C-_Mixins-VolumeFileDirectory-pm/set_permission >>

=cut

=head2 testsetattr

Execute testsetattr on the file, without having to depend on apiset and path names

    my $file = NACL::STask::VolumeFile->create(...);
    my $file_attributes = $file->testsetattr('get-attributes' => 1);
    $file_offset_length = $file->testsetattr('read-offset-and-length' => '0 100');
    $file->testsetattr('write-offset-and-length' => '0 100');

Use it as a pass through to apiset testsetattr.

=cut

sub testsetattr {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;
    $self->_verify_invocation(style => 'instance_only');
    my $apiset = $self->command_interface()->get_7m_or_nodescope_apiset();
    my $node_name = $self->get_local_node_name();

    my $response = $apiset->testsetattr(
        path => $self->dblade_path(),
        'nodescope-node-name' => $node_name,
        %opts
    );

    $Log->exit() if $may_exit;
    return $response->get_parsed_output()->[0];
}

=head2 testsetattrs

Execute multiple testsetattrs on the file, and return the array of results from
each testsetattr call.

    my $file = NACL::STask::VolumeFile->create(...);
    my @results = $file->testsetattrs([
        {'get-attributes' => 1},
        {'read-offset-and-length' => '0 100'},
        {'write-offset-and-length' => '0 100'},
    ]);

Use it as a pass through for multiple apiset testsetattr.

=cut

sub testsetattrs {
    $Log->enter() if $may_enter;
    my ($self, $attrs_for_each_testsetattr) = @_;
    $self->_verify_invocation(style => 'instance_only');
    my @results;
    foreach my $opts (@$attrs_for_each_testsetattr) {
        push @results, $self->testsetattr(%$opts);
    }
    $Log->exit() if $may_exit;
    return @results;
}

=head2 volume_explore

Execute volume explore command on the file scope and return the results.

    my $file = NACL::STask::VolumeFile->create(...);
    my $vol_explore_inode = $file->volume_explore(dump => 'all', format => 'inode');
    my $vol_explore_buftree = $file->volume_explore(dump => 'all', format => 'buftree');

By default the command, dumps all the fields for given format (equivalent of dump => 'all').

=cut

sub volume_explore {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;

    $self->_verify_invocation(style => 'instance_only');

    my $path = $self->path();
    # split /vol/volname/filename into ('', 'vol', 'volname/filename...');
    my ($empty, $vol_suffix, $explore_scope) = split(m{/}, $path, 3);

    my @results = NACL::STask::Volume->explore(
        command_interface => $self->command_interface(),
        %opts,
        scope => $explore_scope,
    );

    $Log->exit() if $may_exit;
    return wantarray ?  @results : $results[0]
}

=head2 show_fingerprint

    my $dump_obj = $file_obj->show_fingerprint();

Convenience method that starts the fingerprint operation, waits for its
completion and then returns the dumped fingerprint details. It does so
by invoking
L<< NACL::STask::VolumeFileFingerprint->start_and_dump|lib-NACL-STask-VolumeFileFingerprint-pm/start_and_dump >>.
It returns a
L<NACL::CS::VolumeFileFingerprintDump|lib-NACL-CS-VolumeFileFingerprintDump-pm>
object.
This method extracts the command_interface, vserver and file path from the file
object and pass them into the call to start_and_dump.

B<NOTE:> If it is required to also only start the file fingperint operation,
then use L<start_fingerprint|lib-NACL-STask-VolumeFile-pm/start_fingerprint> instead.

=cut

sub show_fingerprint {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my %opts = $self->_common_validate_with(
        params      => \@args,
        allow_extra => 1,
    );
    require NACL::STask::VolumeFileFingerprint;
    $self->_modify_opts_for_fingerprint(\%opts);
    my $dump_obj = NACL::STask::VolumeFileFingerprint->start_and_dump(%opts);
    $Log->exit() if $may_exit;
    return $dump_obj;
}

=head2 get_file_type_from_fingerprint

    my $file_type = NACL::STask::VolumeFile->get_file_type_from_fingerprint(
        command_interface => $ci,
        path              => $path,
        vserver           => $vserver,
        %other_opts
    );
    
    my $file_type = $file_obj->get_file_type_from_fingerprint(%other_opts);

(Class or instance method)
Returns the file-type from the file fingerprint details (these are internally
obtained using L<show_fingerprint|lib-NACL-STask-VolumeFile-pm/show_fingerprint>).

=cut

sub get_file_type_from_fingerprint {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $cs = $pkg_or_obj->show_fingerprint(@args);
    my $file_type = $cs->file_type();
    $Log->exit() if $may_exit;
    return $file_type;
}

=head2 is_worm

    my $is_worm = NACL::STask::VolumeFile->is_worm(
        command_interface => $ci,
        path              => $path,
        vserver           => $vserver,
        %other_opts
    );
    
    my $is_worm = $file_obj->is_worm(%other_opts);

(Class or instance method)
Returns a 0/1 value denoting whether the file is a WORM file or not -- these
files have their file-type as "worm". (This is done by internally
using L<get_file_type_from_fingerprint|lib-NACL-STask-VolumeFile-pm/get_file_type_from_fingerprint>.)

=cut

sub is_worm {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $file_type   = $pkg_or_obj->get_file_type_from_fingerprint(@args);
    
    # Value can be worm/worm_appendable, so use regex check instead of
    # string comparison (eq)
    my $is_snaplock = ($file_type =~ /worm/) ? 1 : 0;
    $Log->exit() if $may_exit;
    return $is_snaplock;
}

=head2 start_fingerprint

Convenience method that B<only starts> the file fingerprint operation (i.e.
it does not wait for the operation to complete). It does so
by invoking
L<< NACL::STask::VolumeFileFingerprint->start|lib-NACL-STask-VolumeFileFingerprint-pm/start >>.
It returns a
L<NACL::STask::VolumeFileFingerprint|lib-NACL-STask-VolumeFileFingerprint-pm>
object.
This method extracts the command_interface, vserver and file path from the file
object and pass them into the call to start.

B<NOTE:> If it is required to also wait for completion and get the fingerprint
details, then use L<show_fingerprint|lib-NACL-STask-VolumeFile-pm/show_fingerprint> instead.

=cut

sub start_fingerprint {
    my ($self, @args) = @_;
    my %opts = $self->_common_validate_with(
        params      => \@args,
        allow_extra => 1,
    );
    require NACL::STask::VolumeFileFingerprint;
    $self->_modify_opts_for_fingerprint(\%opts);
    my $fingerprint_obj = NACL::STask::VolumeFileFingerprint->start(%opts);
    $Log->exit() if $may_exit;
    return $fingerprint_obj;
}

# 1. Rename path -> file
# 2. Delete options added by VolumeFile->_common_validate_with, which are not
# applicable for VolumeFileFingerprint
sub _modify_opts_for_fingerprint {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my ($opts_ptr) = validate_pos(@args, {type => HASHREF});
    $opts_ptr->{file} = delete $opts_ptr->{path};
    delete $opts_ptr->{determine_appropriate_nodescope_node};
    delete $opts_ptr->{handle_name_ordinal};
    $Log->exit() if $may_exit;
}

1;
