# Hey emacs, this is basically -*- perl -*-
# $Id: $
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverAudit Task Module
## @author Namitha.k@netapp.com
## @pod here

package NACL::STask::VserverAudit;

use strict;
use warnings;
use Data::Dumper qw(Dumper);

use base qw(NACL::C::VserverAudit NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR SCALARREF ARRAYREF BOOLEAN);
use NATE::Exceptions::Argument qw(:try);
use NACL::C::UnitNormalization;

use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::C::Exceptions::VserverAudit::AlreadyExists ();
use NACL::C::Exceptions::VserverAudit::DoesNotExist ();
use NACL::C::Exceptions::VserverAudit::AlreadyEnabled ();
use NACL::C::Exceptions::VserverAudit::AlreadyDisabled ();
use NACL::C::VserverAudit;
use NACL::C::Job;
use NATE::BaseException;
=head1 NAME

NACL::STask::VserverAudit

=head1 DESCRIPTION

C<NACL::STask::VserverAudit> provides a number of well-defined but
potentially complex or multi-step methods related to VserverAudit in
ONTAP.

It builds on top of, and is a derived class of C<NACL::C::VserverAudit>,
and so it also provides methods that are more in the scope of
individual VserverAudit-related commands.  See C<NACL::C::VserverAudit> for
details.  This also means that a C<NACL::STask::VserverAudit> object may
generally be used in place of a component object.

=head1 ATTRIBUTES

=head2 command_interface

(Required) A component object that represents the host to which commands
 are sent.

=head2 vserver

(Required) Name of vserver on which the audit is to be configured.

=head2 destination

(Required) Log destination path.

=head1 METHODS

=head2 create

my $VserverAudit_obj = NACL::STask::VserverAudit->create(
	command_interface            => $command_interface,
	vserver                      => $vserver_name,
	destination                  => $policy_name,
	'rotate-size'                => $rotate_size,
	'rotate-limit'               => $rotate_limit,
	nacltask_if_exists           => $action,      # default 'die'
	nacltask_verify              => 0|1, # default '0'
	%other_options
);

(Class method) Create a VserverAudit.  This method provides additional
services beyond what's inherent to the product's VserverAudit creation
commands, as controlled and described by the new C<nacltask_if_exists> and
C<nacltask_verify> options.

=over

=item command_interface

(Required) A component object that represents the host to
which commands are sent.

=item vserver

(Required) Name of vserver on which the audit is to be configured.

=item destination

(Required) Log destination path.

=item rotate-size

(optional) Log File Size Limit (default:100MB)

=item rotate-limit

(optional) Log files rotation limit. default value is 0.

=item (Optional) B<< nacltask_if_exists => $action >>

Specifies an action to be taken if the VserverAudit already exists.

$action
  - if set to "die" (default), an exception is raised.
  - if set to "reuse", no action is performed.
  - if set to "purge", deletes the existing vserver audit configuration before creating it.

=item (Optional) B<< nacltask_verify >>

  - when set to 1, vserver Audit creation will be verified.
  - when set to 0, the verification step is skipped.
  - by default its set to 0.

Note that it is also possible to perform the verification later by calling
L<taskverify_create|lib-NACL-STask-STask-pm/taskverify_create>. This can
be used for cases where ONTAP chooses to not set all attributes to the exact
values specified. In the call we can specify those attributes we want
verified.
	my $VserverAudit_obj = NACL::STask::VserverAudit->create(
		command_interface        =>  $command_interface,
		vserver                  =>  $vserver,
		destination              =>  $destination,
		'rotate-size'            =>  $rotate_size,
	 );

    #Now if user wants to verify only two attributes of newly created object
	$VserverAudit_obj->taskverify_create(
		command_interface            => $cammand_interface,
		vserver                      => $vserver,
		destination                  => $destinatiom,
     );


=item (Optional) B<< _was_created => \$scalar >>

When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the
was found (value will be 0; this scenario is possible when
if_exists => "reuse") or whether the Vserver Audit was created (value will be 1).
This is necessary to determine whether the security descriptor needs to be
cleaned up later.

	my $was_created;
	my $VserverAudit_obj = NACL::STask::VserverAudit->create(
		if_exists    => 'reuse',
		_was_created => \$was_created,
		%other_opts
        );

	# Operate on $VserverAudit_obj here
	# ...

	# Now determine whether to clean up the vserver Audit configuration, since we're not sure
	# whether we reused an existing configuration or created a new one
	if ($was_created) {
		# New Vserver Audit was created. Clean it up.
		$VserverAudit_obj->purge();
	}

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverAudit::AlreadyExists>

This type of exception is thrown when an attempt is made to create vserver audit that already exists.

=item C<NATE::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the created vserver audit.

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            nacltask_if_exists => $pkg->_if_exists_validate_spec(),
            nacltask_verify    => {type => SCALAR, default => 0},
            _was_created       => {type => SCALARREF, optional => 1},
        },
        allow_extra => 1,
    );

    my $nacltask_if_exists = delete $opts{nacltask_if_exists};
    my $verify             = delete $opts{nacltask_verify};
    my $was_created        = delete $opts{_was_created};
    my $command_interface  = $opts{command_interface};
    my $vserver            = $opts{vserver};
    my $rotate_size        = $opts{"rotate-size"};
    my $self;
    my $apiset_must;
    my $apiset_should;
    $$was_created = 0;

    if ( defined ($opts{"rotate-size"}))
    {
        # NATE always return the value in bytes hence coverting to byte before creating to match the expected and given value
        $rotate_size  = NACL::C::UnitNormalization::to_bytes($rotate_size);
        $opts{"rotate-size"} = $rotate_size;
    }

CREATE: {
        my $try_flag = 0;
        use warnings qw(exiting);
        try {
            $self = $pkg->SUPER::create(%opts, 'method-timeout' => "300");
            $$was_created = 1;

            #Verify if created
            if ($verify) {
                $pkg->taskverify_create(%opts);
            }
        }
        catch NACL::C::Exceptions::VserverAudit::AlreadyExists with {
            my $exception = shift;
            $Log->comment('The VserverAudit Configuration  already exists');
            $apiset_must   = delete $opts{apiset_must};
            $apiset_should = delete $opts{apiset_should};
            $self = $pkg->_element_exists_handler(
                create_opts        => \%opts,
                nacltask_if_exists => $nacltask_if_exists,
                exception          => $exception,
            );
            if ( !$self ) {
                no warnings qw(exiting);
                redo CREATE;
            }
        };
    }    ##end of CREATE block;

    $Log->exit() if $may_exit;
    return $self;
};    ##end of create

=head2 purge

 NACL::STask::VserverAudit->purge(
        command_interface  => $command_interface,
        vserver            => $vserver_name,
        nacltask_verify    => 0|1, # default is 0
        nacltask_if_purged => pass|die, #default is die
        %other_options
    );

    or

    $VserverAudit_obj->purge();

(Class or instance method) This method deletes Vserver Audit configuration on the
given vserver.

=over

=item (Optional) B<< nacltask_verify => 0|1  >>

  - would verify that the Vserver Audit is purged at the end, if set to 1
  - by default it is set to 0

=item (Optional) B<< nacltask_if_purged => pass|die  >>

  - if already purged then throws an eception when this is set to die
  - if already purged, then contines with the next steps.

=item C<< _was_deleted => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the Vserver Audit
configuration did not exist (value will be 0; this scenario is possible when
nacltask_if_purged => "pass") or whether the Audit Configuration was deleted
(value will be 1).

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverAudit::DoesNotExist>

This type of exception is thrown when an attempt is made to delete vserver audit that does not exists.

=item C<NATE::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the deleted vserver audit.

=back

=cut

sub purge {

    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            nacltask_verify => {type => BOOLEAN, default => 0},
            nacltask_if_purged =>
                $pkg->_if_action_completed_already_validate_spec(),
            _was_deleted    => {type => SCALARREF, optional => 1},
        },
        allow_extra => 1

    );

    my $command_interface = $opts{command_interface};
    my $if_purged         = delete $opts{nacltask_if_purged};
    my $verify            = delete $opts{nacltask_verify};
    my $was_deleted       = delete $opts{_was_deleted};
    $$was_deleted = 0;

    # Call the component delete. If VserverAudit doesn't exist
    # then it would throw an exeception which is handled below.
    try {
        NACL::STask::VserverAudit->disable(%opts,
            nacltask_if_disabled => "pass");
        $pkg->delete(%opts, 'method-timeout' => "300");
        $$was_deleted = 1;
    }
    catch NACL::C::Exceptions::VserverAudit::DoesNotExist with {
        my $exception = shift;
        if ($if_purged !~ /pass/i) {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
    };
    if ($verify) {
        $pkg->taskverify_purge(%opts);
    }
    $Log->exit() if $may_exit;
} ## end sub purge

=head2 enable

NACL::STask::VserverAudit->enable(
    command_interface   => $command_interface,
    vserver             => $vserver_name,
    nacltask_verify     => 0|1 ,# defaults to 0,
    nacltask_if_enabled => pass|die ,#default is die,
      %other_options
      );

(Class or instance method) Enables Vserver audit for vserver.
Also verifies that Vserver Audit is enabled if C<nacltask_verify>
is set to 1.

=over

=item C<< nacltask_verify => 0|1  >>

(Optional, defaults to 0)
  -When set to 1, it verifies that the vserver audit configuration has been enabled
   on the vserver.
  -When set to 0, verification is not performed.

=item (Optional) B<< nacltask_if_enabled => pass|die  >>

  - if already enabled, then throws an eception when this is set to die
  - if already enabled, then contines with the next steps.

=back

=over

=item Exceptions

=over

=item C<NATE::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the enabled vserver audit.

=back

=cut

sub enable {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            nacltask_verify => {type => BOOLEAN, default => 0},
            nacltask_if_enabled =>
                $pkg->_if_action_completed_already_validate_spec(),
        },
        allow_extra => 1,
    );
    my $verify            = delete $opts{'nacltask_verify'};
    my $if_enabled        = delete $opts{'nacltask_if_enabled'};
    my $command_interface = $opts{command_interface};
    my $vserver           = $opts{vserver};
	my $force_val;

	if (defined $opts{force}) {
		$force_val = $opts{force};
	}else{
		$force_val = "true";
	}
    try {
        $pkg->SUPER::enable(%opts, force=> $force_val, allow_extra => 1);
    }
    catch NACL::C::Exceptions::VserverAudit::AlreadyEnabled with {
        my $exception = shift;
        if ($if_enabled !~ /pass/i) {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
    };

    my $iteration = 1;
    my $status = 0;
    my $job_node_obj;

    $pkg->verify_state(%opts, state => 'true') ;
    $Log->debug('Verified. Audit Configuration is enabled');

    while ( $iteration<=15 && $status != 1) {
        $job_node_obj = NACL::C::Job->show_cluster(
            command_interface => $command_interface,
            filter => { vserver => $vserver,
                        name    => '"Consolidation Job"',
                        state   => "Running,Queued",
                      },
        );
        if($job_node_obj->node() eq '-') {
            $Log->comment("Retrying to get the node which is running audit consolidation job for the vserver $vserver after sleep for 2 secs");
            $iteration++;
            sleep 2;
        } else {
            $status = 1;
        }
    }
    NATE::BaseException->throw("Unable to get the node which is running audit consolidation job for the vserver $vserver after retrying for 30 secs") if($status != 1);

    my $job_ci = NACL::C::Node->new('node'=>$job_node_obj->node());
    my $systemshell_obj = $job_ci->get_systemshell_apiset();
    $iteration = 1;
    $status = 0;

    $systemshell_obj->execute_raw_command( command => "sudo bash" );
    while ( $iteration<=15 && $status != 1) {
        my $output = $systemshell_obj->execute_raw_command( command => "ps -auxw | grep /sbin/adtconsolidation | grep $vserver" );
        if( ($output =~ /$vserver/) && ($output =~ /adtconsolidation/) ) {
            $status = 1;
        } else {
            $Log->comment("Retrying to get Audit consolidation process after sleep for 2 secs");
            $iteration++;
            sleep 2;
        }
    }
    NATE::BaseException->throw("Audit consolidation process not present after retrying for 30 secs") if($status != 1);

    ## Closing SSH connections explicitly to prevent accumulating more connections resulting in 
    ## SSH connection limit exceeded failure in other scripts run in parallel in Cit-cifs & fBOTs.
    ## In this function, two SSH connections are setup every time the function is called.  
    ## Audit tests call this function in every testcase accumulating more connections. If these 
    ## connections are not closed explicitly, the generic connection cleanup may take time in 
    ## cleaning up these connections at times resulting in SSH connection failures in other scripts. 
    ## Explicit cleanup will help faster cleanup of these connections.
    $job_ci->apiset()->get_connection()->disconnect();
    $systemshell_obj->get_connection()->disconnect();
    $Log->exit() if $may_exit;
} ## end sub enable

=head2 disable

  NACL::STask::VserverAudit->disable(
      command_interface => $command_interface,
      vserver           => $vserver_name,
      nacltask_verify   => 0|1 # defaults to 0,
      nacltask_if_disabled => pass|die ,#default is die
      %other_options
      );

(Class or instance method) Disables Vserver audit for vserver.
Also verifies that Vserver Audit is disabled if C<nacltask_verify>
is set to 1.

=over

=item C<< nacltask_verify => 0|1  >>

(Optional, defaults to 0)
  -When set to 1, it verifies that the vserver audit configuration has been disabled
   on the vserver.
  -When set to 0, verification is not performed.

 =item (Optional) B<< nacltask_if_disabled => pass|die  >>

  - if already disabled, then throws an eception when this is set to die
  - if already disabled, then contines with the following steps.

=back

=over

=item Exceptions

=over

=item C<NATE::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the disabled vserver audit.

=back

=cut

sub disable {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            nacltask_verify => {type => BOOLEAN, default => 0},
            nacltask_if_disabled =>
                $pkg->_if_action_completed_already_validate_spec(),
        },
        allow_extra => 1,
    );

    my $verify            = delete $opts{'nacltask_verify'};
    my $if_disabled       = delete $opts{'nacltask_if_disabled'};
    my $command_interface = $opts{command_interface};
    my $vserver           = $opts{vserver};
    my $job_obj_id;
    my @job_objs;
    my $count = 0;
    my $retries = 0;

    try {
        @job_objs = NACL::C::Job->find(
            command_interface => $command_interface,
            filter            => {
                vserver => $vserver,
                name    => '"Consolidation Job"',
                state   => "Running,Queued"
            },
        );
        if (@job_objs > 1) {
            print
                "ERROR: MORE THAN ONE CONSOLIDATION JOB FOUND FOR THE SAME VSERVER\n";
            print Dumper $_->state() foreach (@job_objs);
            $Log->exit() if $may_exit;
            die "multiple consolidation jobs found \n";
        } else {
            $count++ if ($job_objs[0]->state->name =~ /Consolidation Job/i);
            $job_obj_id = $job_objs[0]->state->id
                if ($job_objs[0]->state->name =~ /Consolidation Job/i);
        }

        try {
            $pkg->SUPER::disable(%opts, allow_extra => 1);
        }
        catch NACL::C::Exceptions::VserverAudit::AlreadyDisabled with {
            my $exception = shift;
            if($if_disabled !~ /pass/i) {
                $Log->exit() if $may_exit;
                $exception->throw();
            }
        };

        while (($retries <= 60) &&
            ($job_objs[0]->state->name =~ /Consolidation Job/i)
            && (   $job_objs[0]->state->state =~ /Running/i
                || $job_objs[0]->state->state =~ /Queued/i)
            ) {
            $count++;
	    $retries++;
            $job_objs[0] = NACL::C::Job->find(
                command_interface => $command_interface,
                filter            => {vserver => $vserver, id => $job_obj_id},
            );
            sleep 10;
        }
        if ($retries >= 60) {
	   $Log->comment( "Consolidation job not killed \n");
	   $Log->exit() if $may_exit;
	   die "Audit Job not killed even after audit disabled\n";
	}
	

    }
    otherwise {
        if ($count == 0) {
            print "WARNING:NO CONSOLIDATION OBJECT FOUND\n";
            try {
                $pkg->SUPER::disable(%opts, allow_extra => 1);
            }
            catch NACL::C::Exceptions::VserverAudit::AlreadyDisabled with {
                my $exception = shift;
                if ($if_disabled !~ /pass/i) {
                    $Log->exit() if $may_exit;
                    $exception->throw();
                }
            };
        }
        $Log->comment("CONSOLIDATION JOB IS COMPLETED. AUDIT SUCCESSFULLY DISABLED\n")
            if ($count > 0);
    };

    $pkg->verify_state(%opts, state => 'false') if ($verify);
    $Log->exit() if $may_exit;
} ## end sub disable

1;
