# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsCharacterMapping Task Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::STask::VserverCifsCharacterMapping

=head1 DESCRIPTION

C<NACL::STask::VserverCifsCharacterMapping> provide methods to create,
modify, and purge the character mapping on a volume.

It is created on top of C<NACL::C::VserverCifsCharacterMapping> component.
Since it is derived class of C<NACL::C::VserverCifsCharacterMapping>,
we can use all the methods of C<NACL::C::VserverCifsCharacterMapping>
from the object of this task.

=head1 CLEANUP METHODS

Cleanup can be registered for the following methods,

     VserverCifsCharacterMapping  Method         Cleanup Method
     -----------------------------------------------------------
     create                                       purge
     modify                                       modify

=head1 ATTRIBUTES

=head2 command_interface

(Required) A component object that represents the node to which to send commands
See L<NACL::C::Component::command_interface
  |lib-NACL-C-Component-pm/command_interface>.

=head2 vserver

(Required) The name of the vserver to be used.

=head2 volume

(Required) The name of the volume to be used.

=head1 EXCEPTIONS

    1. NACL::C::Exceptions::VserverCifsCharacterMapping::AlreadyExists
       (This type of exception is thrown when an attempt is made to create a
       character map that already exists).

    2. NACL::C::Exceptions::VserverCifsCharacterMapping::DoesNotExist
      (This type of exception is thrown when an attempt is made to delete a
      character map that does not exists).

=cut

package NACL::STask::VserverCifsCharacterMapping;

use strict;
use warnings;

use base qw(NACL::C::VserverCifsCharacterMapping NACL::STask::STask);
use Params::Validate
    qw(validate validate_with SCALAR SCALARREF ARRAYREF BOOLEAN );
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Exceptions::Argument qw(:try);
use NATE::BaseException;

=head1 METHODS

=head2 create

   my $VserverCifsCharMapping_obj =
     NACL::STask::VserverCifsCharacterMapping->create(
       'command_interface'        => $ci,
       'vserver'                  => $vserver,
       'volume'                   => $volName,
       'mapping'                  => [ $mapping1, $mapping2, ... ],
       'nacltask_if_exists'       => $action # default die,
       'nacltask_to_cleanup'      => 1, #default 0
       'nacltask_cleanup_manager' => $CleanupObj,
       %other_options,
   );

(Class Method) This method is used to create a character mapping.
If character mapping already exists, it will perform the action based on
"nacltask_if_exists" parameter. Default behavior would be "die". And also
provides an mechanism of cleanup for the objects that are created by this method

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required for class method, Not Applicable for instance method)
A component object that represents the host which to send commands.
See L<NACL::C::Component::command_interface
  |lib-NACL-C-Component-pm/command_interface>.

=item C<< "nacltask_if_exists => $action" >>

(Optional) What to do if the character mapping already exists.
If $action is "die", then fail with
NACL::C::Exceptions::VserverCifsCharacterMapping::AlreadyExists.
If $action is "purge", then purge the character mapping (see the "purge"
 method below) before creating a new one.
If $action is "reuse", It will return the object of existing character mapping
for the specified volume.

=item C<< "nacltask_verify => $action" >>

(Optional) The user of this library can specify to verify whether the character
mapping exists for the specified volume.
If the action is 0, which is default, verification is not done.
If the user sets the action to 1, it will verify it using the component state
of this library.

=item C<< "_was_created => \$scalar" >>

(Optional) When this option is provided with a reference to a scalar
variable, the variable gets filled in with a boolean value
describing whether the character mapping exists (value will be 0; this
scenario is possible when if_exists => "reuse") or whether the
character mapping was created (value will be 1). This is necessary to
determine whether the character mapping for the volume needs to be cleaned up
later.

    my $was_created;
    my $VserverCifsCharMapping_obj =
      NACL::STask::VserverCifsCharacterMapping->create(
        nacltask_if_exists     => 'reuse',
        _was_created            => \$was_created,
        %other_opts
    );
# Operate on $VserverCifsCharMapping_obj here
# Now determine whether to clean up the dns, since we're not sure
# whether we reused an existing dns or created a new one
    if ($was_created) {
       # New character mapping . Clean it up.
       $VserverCifsCharMapping_obj->purge();
    }

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup)
Flag indicating if this operation needs to be registered for clean up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=item other options

See L<NACL::C::VserverCifsCharacterMapping::create
|lib-NACL-C-VserverCifsCharacterMapping-pm/create> for all the other options
accepted by this method.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverCifsCharacterMapping::AlreadyExists>

This type of exception is thrown when an attempt is made to create a character
mapping on a volume when it already exists.

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the character
mapping on a volume.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            nacltask_if_exists => $pkg->_if_exists_validate_spec(),
            nacltask_verify    => { type => BOOLEAN, default => 0 },
            _was_created       => { type => SCALARREF, optional => 1 },
            $pkg->_cleanup_validate_spec(),
        },
        allow_extra => 1
    );

    # Transform %opts from the options we received into the options to
    # pass to the base class method.
    my ( $self, %nacltask_opts, %opts_for_cleanup, $nacltask_to_cleanup,
        %opts_for_register, $register_for_cleanup );

    $pkg->_move_common_cleanup_opts(
        source              => \%opts,
        target              => \%opts_for_cleanup,
        nacltask_to_cleanup => \$register_for_cleanup
    );

    $pkg->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_opts
    );

    my $was_created = delete $opts{_was_created};
    $$was_created = 0;
    my $if_exists = $nacltask_opts{'nacltask_if_exists'};
    my $verify    = $nacltask_opts{'nacltask_verify'};

CREATE: {
        use warnings;
        try {
            $self        = $pkg->SUPER::create(%opts);
            $$was_created = 1;
        }
        catch NACL::C::Exceptions::VserverCifsCharacterMapping::AlreadyExists
            with {
            my $exception = $_[0];
            $Log->comment( 'STask The character mapping entry already exists'
                    . "$nacltask_opts{'nacltask_if_exists'}" );
            $self = $pkg->_element_exists_handler(
                create_opts        => \%opts,
                nacltask_if_exists => $if_exists,
                exception          => $exception
            );
            if ( !$self ) {
                no warnings qw(exiting);
                redo CREATE;
            }
            };
    }

    if ($$was_created) {

        # Verification based on nacltask_opts
        $pkg->verify_state(%opts) if ($verify);

        # Register this resource with the Cleanup Manager for cleanup
        if ($register_for_cleanup) {
            $pkg->_copy_common_opts_for_cleanup(
                'source' => { %opts, %opts_for_cleanup },
                'target' => \%opts_for_register,
                'nacltask_to_cleanup' => \$nacltask_to_cleanup,
                'to_cleanup'          => 'purge'
            );
            $pkg->_register_for_cleanup(%opts_for_register);
        }
    }
    $Log->exit() if $may_exit;
    return $self;
} ## end sub create

=head2 purge

    $VserverCifsCharMapping_obj->purge();
            (or)
    NACL::STask::VserverCifsCharacterMapping->purge(
            'command_interface' => $ci,
            'vserver'           => $vserver_name,
            'volume'            => $volName,
            'nacltask_verify'   => 1,
            %other_options,
    );

(Class or instance method)
This method is used to delete the character mapping. "nacltask_verify"
parameter can be set to ensure that the deletion did happen successfully.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required for class method, Not Applicable for instance method)
A component object that represents the host which to send commands.
See L<NACL::C::Component::command_interface
  |lib-NACL-C-Component-pm/command_interface>.

=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional) The user of this library can specify to verify whether the character
mapping exists for the specified volume.
If the action is 0, which is default, verification is not done.
If the user sets the action to 1, it will verify it using the component state
of this library.

=item C<< nacltask_if_purged => $nacltask_if_purged >>

(Optional) Default value is 'fail'.
If 'pass', it will pass if the character map is already deleted.
If 'fail', NACL::C::Exceptions::VserverCifsCharacterMapping::DoesNotExist
type of exception is raised.

=item other options

See L<NACL::C::VserverCifsCharacterMapping::purge
|lib-NACL-C-VserverCifsCharacterMapping-pm/purge> for all the other options
accepted by this method.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverCifsCharacterMapping::DoesNotExist>

This type of exception is thrown when an attempt is made to delete a non
existing character mapping on a vserver/volume.

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for non existing
character mapping, if 'nacltask_verify' is set to 1.

=back

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            nacltask_if_purged => { type => SCALAR,  default => 'pass' },
            nacltask_verify    => { type => BOOLEAN, default => 0 },
        },
        allow_extra => 1
    );

    # Transform %opts from the options we received into the options to
    # pass to the base class method.
    my %nacltask_opts;

    $pkg_or_obj->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_opts
    );

    my $if_purged = delete $nacltask_opts{'nacltask_if_purged'};
    my $verify    = delete $nacltask_opts{'nacltask_verify'};

    try {
        $pkg_or_obj->delete(%opts);
    }
    catch NACL::C::Exceptions::VserverCifsCharacterMapping::DoesNotExist
        with {
        my $exception = $_[0];
        $Log->debug(
            "STask The Vserver/Volume entry does not exists $if_purged");
        if ( $if_purged !~ /pass/i ) {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
        };

    $pkg_or_obj->taskverify_purge(%opts) if ($verify);

    $Log->exit() if $may_exit;
    return;
} ## end sub purge

=head2 modify

    $VserverCifsCharMapping_obj->modify();
            (or)
    NACL::STask::VserverCifsCharacterMapping->modify(
            'command_interface'        => $ci,
            'vserver'                  => $vserver_name,
            'volume'                   => $volName,
            'mapping'                  => [ $mapping1, $mapping2, ... ],
            'nacltask_to_cleanup'      => 1, #default 0
            'nacltask_cleanup_manager' => $CleanupObj,
            %other_options
    );

(Class or instance method)
This method is used to modify the character mapping. "nacltask_verify"
parameter can be set to ensure all the attributes got a specified value.
And also provides an mechanism of cleanup for the objects that are modified
by this method.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required for class method, Not Applicable for instance method)
A component object that represents the host which to send commands.
See L<NACL::C::Component::command_interface
  |lib-NACL-C-Component-pm/command_interface>.

=item C<< nacltask_verify => 0|1 >>

(Optional, defaults to 0)
Verifies whether all of the attributes got set to the specified value.

Note that it is also possible to perform the verification later by calling
L<taskverify_modify|lib-NACL-STask-STask-pm/taskverify_modify>. This can
be used for cases where ONTAP chooses to not set all attributes to the exact
values specified. In the call we can specify those attributes we want
verified.

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup)
Flag indicating if this operation needs to be registered for clean up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=item other options

See L<NACL::C::VserverCifsCharacterMapping::modify
|lib-NACL-C-VserverCifsCharacterMapping-pm/modify> for all the other options
accepted by this method.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the modified
character mapping.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            nacltask_verify => { type => BOOLEAN, default => 0 },
            $pkg_or_obj->_cleanup_validate_spec(),
        },
        allow_extra => 1
    );

    #Transform %opts from the options we received into the options to
    #pass to the base class method.
    my (%nacltask_opts,     %opts_for_cleanup, $nacltask_to_cleanup,
        $opts_for_register, $register_for_cleanup
    );

    $pkg_or_obj->_move_common_cleanup_opts(
        source              => \%opts,
        target              => \%opts_for_cleanup,
        nacltask_to_cleanup => \$register_for_cleanup
    );

    $pkg_or_obj->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_opts
    );
    my $verify = $nacltask_opts{'nacltask_verify'};

    $opts_for_register
        = $pkg_or_obj->_common_modify_for_cleanup( %opts_for_cleanup, %opts )
        if ($register_for_cleanup);

    $pkg_or_obj->SUPER::modify(%opts);

    # Verification based on the nacltask_opts
    $pkg_or_obj->verify_state(%opts) if ($verify);

    $pkg_or_obj->_register_for_cleanup( %{$opts_for_register} )
        if ($register_for_cleanup);
    $Log->exit() if $may_exit;
    return;
} ## end sub modify

1;
