# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsDomainPreferredDc Task Module
## @author ng-ptefit-automation-core@netapp.com
## @status shared
## @pod here

package NACL::STask::VserverCifsDomainPreferredDc;

use strict;
use warnings;
use base qw(NACL::C::VserverCifsDomainPreferredDc NACL::STask::STask);

use NATE::Log qw( log_global );
my $Log = log_global();
use Params::Validate qw(validate SCALAR BOOLEAN SCALARREF);
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use NACL::Exceptions::VerifyFailure                     ();
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ResponseException ();
use NACL::C::Exceptions::VserverCifsDomainPreferredDc::DoesNotExist ();

=head1 NAME

NACL::STask::VserverCifsDomainPreferredDc

=head1 DESCRIPTION

C<NACL::STask::VserverCifsDomainPreferredDc> provides a number of well-defined but potentially
complex or multi-step methods related to VserverCifsDomainPreferredDc in ONTAP.

It is created on top of C<NACL::C::VserverCifsDomainPreferredDc> component
Since it is derived class of C<NACL::C::VserverCifsDomainPreferredDc>,
we can use all the methods of C<NACL::C::VserverCifsDomainPreferredDc>
from the object of this task.

=head1 ATTRIBUTES

=head2 command_interface

(Required) A component object that represents the node to which to send commands
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

=head2 vserver

(Required) The name of the Vserver for which you want to add preferred domain controllers.

=head2 domain

(Required) The fully-qualified name of the domain that the domain controllers belong to.

=head2 preferred-dc

(Required) The list of IP addresses for domain controllers that belong to the domain.

=head1 METHODS

=head2 purge

    $VserverCifsDomainPreferredDc->purge();
            (or)
    NACL::STask::VserverCifsDomainPreferredDc->purge(
            'command_interface' => $command_interface,
            'vserver'           => $vserver,
            'domain'            => $domain,
            'preferred-dc'      => \@ip_addr,
            'nacltask_verify'   => 1,
    );

(Class or instance method)
This method is used to delete the domain controllers.

Uses a CMode CLI/ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
A component object that represents the host which to send commands.
See NACL::C::Component::command_interface

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
The name of the Vserver for which you want to add preferred domain controllers.

=item C<< 'domain' => $domain >>

(Required for class method, Not Applicable for instance method)
The fully-qualified name of the domain that the domain controllers belong to.

=item C<< 'preferred-dc' => $preferred_dc >>

(Required for class method, Not Applicable for instance method)
An array reference of IP addresses for domain controllers that belong to the domain

=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the deletion did happen
successfully.

=item C<< _was_deleted => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the domain controller
did not exist (value will be 0; this scenario is possible when
nacltask_if_purged => "pass") or whether the domain controller was deleted
(value will be 1).

=item C<< nacltask_if_purged => $nacltask_if_purged >>

(Optional)
If 'pass' (default), It will pass if the domain controller is already deleted.
If 'fail', It will fail if the domain controller is already deleted.

All of the other various options to
L<< NACL::C::VserverCifsDomainPreferredDc->delete|lib-NACL-C-VserverCifsDomainPreferredDc-pm/delete >>

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverCifsDomainPreferredDc::DoesNotExist>

This type of exception is thrown when an attempt is made to delete a domian that does not exists.

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the deleted cron schedule.

=back

=cut

sub purge {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    my $additional_spec = {
        nacltask_verify    => { type => BOOLEAN, default => 0 },
        nacltask_if_purged => $pkg_or_obj->_if_action_completed_already_validate_spec(),
        _was_deleted       => { type => SCALARREF, optional => 1 },
    };

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => $additional_spec,
        allow_extra => 1,
    );

    my %nacltask_opts;
    $pkg_or_obj->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_opts,
    );

    my $if_purged   = $nacltask_opts{nacltask_if_purged};

    my %common_opts;
    $pkg_or_obj->_copy_common_component_params_with_ci(
        source => \%opts,
        target => \%common_opts
    );

    my $was_deleted = delete $opts{_was_deleted};
    $$was_deleted = 0;

    use warnings qw(exiting);
    try {
        $pkg_or_obj->SUPER::remove(%common_opts, %opts );
        $$was_deleted = 1;
    }
    catch NACL::C::Exceptions::VserverCifsDomainPreferredDc::DoesNotExist with {
        my $exception = shift;
        $Log->comment("Entry Does Not Exists!");
        if ( $if_purged !~ /pass/i ) {
            $Log->exit();
            $exception->throw();
        }
    };

    if ( $opts{nacltask_verify} ) {
        $pkg_or_obj->_generic_purge_verify(%opts);
    }

    $Log->exit();
} ## end sub remove

1;
