#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverNfs Task Module
## @author dl-nacl-dev@netapp.com
## @status private
## @pod here

package NACL::STask::VserverNfs;

use strict;
use warnings;

use base qw(NACL::C::VserverNfs NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR BOOLEAN SCALARREF);
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ResponseException               ();
use NACL::C::Exceptions::VserverNfs::AllowedProtocolsNotAdded ();
use NACL::C::Vserver;
use NACL::STask::SystemLicense;

=head1 NAME

NACL::STask::VserverNfs

=head1 DESCRIPTION

C<NACL::STask::VserverNfs> provides a number of well-defined but
potentially complex or multi-step methods related to  NFS shares in
the scope of a specific Vserver on ONTAP.
It builds on top of, and is a derived class of C<NACL::C::VserverNfs>,
and so it also provides methods that are more in the scope of
individual VserverNfs-related commands.  See C<NACL::C::VserverNfs> for
details.  This also means that a C<NACL::STask::VserverNfs> object may
generally be used in place of a component object.

=head1 ATTRIBUTES

=head2 command_interface

(Required) A component object that represents the host to which to send commands.

=head2 vserver

The name of the vserver where nfs is configured, used to identify it uniquely.

(Note that if the command_interface attribute is a
L<NACL::C::Vserver|lib-NACL-C-Vserver-pm>, then this attribute will be undef)

=head1 METHODS

=head2 create

    my $nfsshare  = NACL::STask::VserverNfs->create(
        command_interface  => $command_interface,
        vserver            => $vserver_name,
        nacltask_if_exists => $action,        # default 'die'
        %other_options,                       # pass any other optiots
    );

(Class method) Enable and configure a Vserver to serve NFS clients.
Prior to this, the Vserver creation must be done and the Vserver
should exist within the cluster. This method provides additional
services beyond what's inherent to Vserver NFS's share creation on the
cluster, as controlled and described by the new C<nacltask_if_exists>
and C<nacltask_verify> options.
If the license for NFS service is not added, this method will add the
license during the creation of NFS share.

=over

=item Options

=over

=item C<< if_exists=>$action >>

(Optional) What to do if the Vserver nfs has already been enabled.
If $action is "die" (the default), then fail with an exception (in the
same way that the VserverNfs component would:  by trying the
vserver nfs create and letting the product complain about  nfs already
existing). If action is "reuse", then verify the nfs settings
with user specified options. If they match, return a task object
referring to the existing nfs share. If the action is reuse and the
options do not match, then delete the existing share and creata a new
share using options. If action is "purge", then delete the vserver
nfs share.

=item C<< nacltask_verify => 0|1 >>

(Optional, defaults to 0)
This option is used to toggle whether to verify that the created NFS share
has exactly the same attributes as we specified in the call. A
L<NACL::Exceptions::VerifyFailure|lib-NACL-Exceptions-VerifyFailure-pm>
exception is thrown if one or more attributes of the creates NFS share
do not match the specified values.
This defaults to not being performed.

Note that if we want to verify for only some, but not all fields, we can do so
using L<verify_state|lib-NACL-C-Component-pm/verify_state>.

=item C<< _was_created => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the nfs share
was found (value will be 0; this scenario is possible when
if_exists => "reuse") or whether the nfs share was created (value will be 1).
This is necessary to determine whether the nfs share needs to be
cleaned up later.

    my $was_created;
    my $nfsshare_obj = NACL::STask::VserverNfs->create(
        if_exists    => 'reuse',
        _was_created => \$was_created,
        %other_opts
    );
    # Operate on $nfsshare_obj here
    # ...
    # Now determine whether to clean up the nfs share, since we're not sure
    # whether we reused an existing nfs share or created a new one
    if ($was_created) {
        # New nfs share was created. Clean it up.
        $nfsshare_obj->purge();
    }

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, defaults to 0(no to cleanup))
Flag indicating if this operation is to be cleaned up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to use for registering.
Default : Will use the default cleanup manager.


=item C<< "method-timeout"=>$timeout >>

(Optional) How long in seconds to wait for the job to finish or for
the state to become "online".  Defaults to 60 seconds.

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< %other_options >>
(Optional) All of the other various options to

L<< NACL::C::VserverNfs->create|lib-NACL-C-VserverNfs-pm/create >>
like V3 V4.0 udp tcp spinauth default-win-user v4.0-acl etc.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions:VserverNfs:::AlreadyExists>

This type of exception is thrown when an attempt is made to enable vserver nfs service when it is already enabled.

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when there is a failure to verify that the created NFS share has
exactly the same attributes as specified in the call.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {

            # Kept for backwards compatibility
            if_exists => {
                type      => SCALAR,
                callbacks => $pkg->_if_exists_validate_callback(),
                optional  => 1
            },

            nacltask_if_exists => {
                type      => SCALAR,
                callbacks => $pkg->_if_exists_validate_callback(),
                optional  => 1
            },
            nacltask_verify => { type => BOOLEAN,   default  => 0 },
            _was_created    => { type => SCALARREF, optional => 1 },
            $pkg->_cleanup_validate_spec(),
        },
        allow_extra => 1,
    );


    # move the cleanup related options to separate hash
    my (%opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup);
    $pkg->_move_common_cleanup_opts(
        source              => \%opts,
        target              => \%opts_for_cleanup,
    );

    # Transform %opts from the options we received into the options to
    # pass to the base class method.

    my $if_exists = delete $opts{nacltask_if_exists} || $opts{if_exists};

    # Having the "delete $opts{if_exists}" on the previous line would not
    # necessarily delete it, because it would not get evaluated if
    # the "delete $opts{nacltask_if_exists}" returned some value. Hence,
    # we're explicitly deleting it here separately.
    delete $opts{if_exists};

    # Assign the default of "die" if neither of nacltask_if_exists or
    # if_exists were provided.
    $if_exists ||= 'die';

    my $verify = delete $opts{nacltask_verify};

    my $dummy;
    my $was_created = delete $opts{_was_created} || \$dummy;
    $$was_created = 0;

    my $self;

LOOP: {
        use warnings qw(exiting);
        try {
            $self         = $pkg->SUPER::create(%opts);
            $$was_created = 1;
            $pkg->taskverify_create(%opts) if $verify;
        }
        catch NACL::C::Exceptions::VserverNfs::AllowedProtocolsNotAdded with {
            my $exception = shift;
            #The license add step should be removed when product burt#619245
            #gets fixed
            NACL::STask::SystemLicense->add(
                command_interface => $opts{command_interface},
                feature           => 'nfs',
            );
            $exception->add_allowed_protocols();
            no warnings;
            redo LOOP;
        }
        catch NACL::C::Exceptions::VserverNfs::FeatureNotLicensed with {
            my $exception = shift;
            $exception->license_feature();
            no warnings qw(exiting);
            redo LOOP;
        }
        catch NACL::C::Exceptions::VserverNfs::AlreadyExists with {
            my $exception = shift;
            $self = $pkg->_element_exists_handler(
                create_opts        => \%opts,
                nacltask_if_exists => $if_exists,
                exception          => $exception
            );
            if ( !$self ) {
                no warnings qw(exiting);
                redo LOOP;
            }
        };
    }

    # Register this resource with the Cleanup Manager for cleanup
    $pkg->_copy_common_opts_for_cleanup(
        'source'              => {%opts, %opts_for_cleanup},
        'target'              => \%opts_for_register,
        'nacltask_to_cleanup' => \$nacltask_to_cleanup,
        'to_cleanup'          => 'purge'
    );
    $pkg->_register_for_cleanup(%opts_for_register) if ( $nacltask_to_cleanup && $$was_created );

    $Log->exit() if $may_exit;
    return $self;
}

=head2 purge

    NACL::STask::VserverNfs->purge(
        command_interface => $command_interface,
        vserver => $vserver_name,
        ...
    );

    $nfsshare->purge(...);

(Class or instance method) This method deletes the NFS share of
the vserver.

=over

=item Options

=over

=item C<< nacltask_verify => 0|1 >>

(Optional, defaults to 0)
This option is used to toggle whether to verify NFS share has been purged.
L<NACL::Exceptions::VerifyFailure|lib-NACL-Exceptions-VerifyFailure-pm>
exception is thrown if this verification failed.
This defaults to not being performed.

Note that this can also be performed in a separate call using
C<taskverify_purge>. Example:

    $nfs_obj->purge();
    ...
    $nfS_obj->taskverify_purge();


=item C<< _was_deleted => \$scalar >>

(Optional)
When this option is provided a reference to a scalar variable, the variable
gets filled in with a boolean value describing whether the igroup
did not exist (value will be 0; this scenario is possible when
nacltask_if_purged => "pass") or whether the igroup was deleted
(value will be 1).

=item C<< nacltask_if_purged => 'die' | 'pass' >>

(Optional, defaults to 'die')
Used to specify whether attempting to purge when the NFS share has
already been previously purged should result in a failure or not.
When this is 'die' (the default), it throws an exception.
When this is 'pass' it does not error out if has already been purged.

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverNfs::DoesNotExist>

This type of exception is thrown when an attempt is made to delete vserver nfs that does not
exist.

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the deleted nfs service.

=back

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            nacltask_verify    => { type => BOOLEAN, default => 0 },
            nacltask_if_purged => {
                type      => SCALAR,
                optional  => 1,
                callbacks => $pkg_or_obj
                    ->_if_action_completed_already_validate_callback(
                    )
            },
            if_purged => {
                type      => SCALAR,
                optional  => 1,
                callbacks => $pkg_or_obj
                    ->_if_action_completed_already_validate_callback(
                    name => 'if_purged'
                    )
            },
            _was_deleted       => { type => SCALARREF, optional => 1 },
        },
        allow_extra => 1,
    );
    my $verify = delete $opts{nacltask_verify};
    my $if_purged
        = delete $opts{nacltask_if_purged}
        || $opts{if_purged}
        || 'die';
    delete $opts{if_purged};

    my $pkg = $pkg_or_obj->get_package_name();

    my $was_deleted = delete $opts{_was_deleted};
    $$was_deleted = 0;

    try {
        $pkg->delete(%opts);
        $$was_deleted = 1;
        $pkg->taskverify_purge(%opts) if $verify;
    }
    catch NACL::C::Exceptions::VserverNfs::DoesNotExist with {
        my $exception = shift;
        $pkg_or_obj->_handle_if_purged(
            exception          => $exception,
            nacltask_if_purged => $if_purged
        );
    };

    $Log->exit() if $may_exit;
}

=head2 modify

NACL::STask::VserverNfs->modify(
    command_interface     => $command_interface,
    vserver               => $vserver_name,
    "nfs-rootonly"        => "enabled",
    "nacltask_to_cleanup" => 1,
);

or

$vservernfs_object->modify( "nfs-rootonly" => "enabled", ... );

(Class method) Modify attributes of a VserverNfs. Also verify that all
attributes got set to the specified values if C<nacltask_verify> is set to 1.

=over

=item C<< nacltask_verify => 0|1  >>

(Optional, defaults to 0)
Verifies whether all of the attributes got set to the specified value.

Note that it is also possible to perform the verification later by calling
L<taskverify_modify|lib-NACL-STask-STask-pm/taskverify_modify>. This can
be used for cases where ONTAP chooses to not set all attributes to the exact
values specified. In the call we can specify those attributes we want
verified.

=item C<< nacltask_if_no_values => 'die' | 'pass' >>

(Optional, defaults to 'die')
Used to specify whether attempting to modify when there are no
values to apply is a failure or not.
When this is 'die' (the default), it throws an exception.
When this is 'pass' it does not error out if no values to apply.

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, defaults to 0(no to cleanup))
Flag indicating if this operation is to be cleaned up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to use for registering.
Default : Will use the default cleanup manager.

=item other options

See L<NACL::C::VserverNfs->modify|lib-NACL-C-VserverNfs-pm/modify> for
all the other options accepted by this method.

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            $pkg_or_obj->_cleanup_validate_spec(),
            nacltask_if_no_values => { type => SCALAR,  default => 'die' },
        },
        allow_extra     => 1,
    );

    my $if_no_values = delete $opts{nacltask_if_no_values};

    my ($opts_for_register, $register_for_cleanup, %opts_for_cleanup);
    $pkg_or_obj->_move_common_cleanup_opts(
        source              => \%opts,
        target              => \%opts_for_cleanup,
        nacltask_to_cleanup => \$register_for_cleanup
    );

    if ($register_for_cleanup) {
        $opts_for_register
            = $pkg_or_obj->_common_modify_for_cleanup(%opts_for_cleanup,
            %opts);
    }

    try {
        $pkg_or_obj->_call_method_then_verify(%opts,
                                              _method => 'modify');
    }
    catch NACL::C::Exceptions::VserverNfs::NoValuesToApply with {
        my $exception = shift;
        $Log->debug("No Values to Apply!");
        if ( $if_no_values !~ /pass/i ) {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
    };

    # Register for cleanup if $nacltask_to_cleanup is set
    $pkg_or_obj->_register_for_cleanup(%{$opts_for_register})
        if ($register_for_cleanup);

    $Log->exit() if $may_exit;
}

1;
