#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverPeer Task Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::VserverPeer;

use strict ;
use warnings;

use base qw(NACL::C::VserverPeer NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::ComponentUtils qw(_ontap_ci_validate_spec Dumper);

use NATE::Exceptions::Argument qw(:try);
use NATE::BaseException;

use NACL::STask::Snapmirror;
use NACL::CS::ClusterIdentity;
use NACL::STask::Cluster;
use NACL::CS::ClusterPeer;
use Params::Validate qw(:all);
use Scalar::Util qw(refaddr);
use Net::IP;
use NACL::CS::Vserver;
use NACL::CS::SnapmirrorListDestinations;

use Class::MethodMaker [
    scalar => [ { -type => 'NACL::C::Job' }, 'job_component' ],
    scalar => [
        {   -type     => 'NACL::C::CommandInterface::ONTAP',
            -store_cb => sub {
                my ($self, $interface) = @_;
                $self->is_intercluster(1);
                return $interface;
            }
        },
        'remote_interface',
    ],
    scalar => 'is_intercluster',
];


=head1 NAME

NACL::STask::VserverPeer

=head1 DESCRIPTION

C<NACL::STask::VserverPeer> provide methods to create,
delete, accept, resume, reject, suspend, modify vserver peer relation-ship in ONTAP.

It is created on top of C<NACL::C::VserverPeer> component. Since it is derived class
of C<NACL::C::VserverPeer>, we can use all the methods of C<NACL::C::VserverPeer>
from the object of this task.

Cleanup can be registered for the following methods

Cleanup methods are,

     VserverPeer Method         Cleanup Method
     ----------------------------------------
     create                     purge

=head1 ATTRIBUTES

=head2 command_interface

The command_interface object that represents the host on which
the local vserver is.

=head2 vserver

The local vserver which is being peered.

=over

=item peer-vserver

The vserver being peered with.

=item remote_interface

If an inter-cluster relationship is being established, then this stores the
command_interface for the remote cluster. This attribute gets populated by
the create method. Note that it does B<NOT> get populated by calls to find().

=back

=head1 METHODS

=head2 create

    # Most basic intra-cluster form: provide all arguments
    my $peer_obj = NACL::STask::VserverPeer->create(
        command_interface => $ci,
        vserver           => $vs_name,
        'peer-vserver'    => $peer_vs_name,
        applications      => \@applications,

        # This includes "nacltask_wait", "nacltask_verify"
        %other_opts
    );

    # "applications" is defaulted to ['snapmirror'], so
    # it is not mandatory in the call. Example:
    my $peer_obj = NACL::STask::VserverPeer->create(
        command_interface => $ci,
        vserver           => $vs_name,
        'peer-vserver'    => $peer_vs_name,

        # This includes "nacltask_wait", "nacltask_verify" and "nacltask_if_exists"
        %other_opts
    );

    # Possible to achieve the above by using a Vserver object
    my $peer_obj = $vs_stask_obj->peer_create(
        'peer-vserver' => $peer_vs_name,
        %other_opts
    );

    # For inter-cluster vserver peers "remote_interface" needs to be
    # provided. This is the command_interface object for the remote cluster.
    # The library automatically determines the "peer-cluster" value if not
    # provided.
    my $peer_obj = $vs_stask_obj->peer_create(
        remote_interface => $remote_cluster_ci,
        'peer-vserver'   => $peer_vs_name,
        %other_opts
    );

    # Most advanced form: Using a Vserver object instance as value for
    # "peer-vserver". The library automatically determines if it is an
    # intra or inter-cluster relationship. If it is an inter-cluster
    # relationship, it will also determine the "peer-cluster" value if not
    # provided.
    # Note that if "peer-vserver" is provided as an object, then
    # "remote_interface" must NOT be provided.
    my $peer_obj = $vs_stask_obj->peer_create(
        'peer-vserver'   => $peer_vs_obj,
        %other_opts
    );


(Class Method) This method creates vserver peer relationship and returns
an STask object modeling it.

In addition to the facilities provided by the component, the following
functionality is provided by the STask:

=over

=item Waiting for completion

Defaults to waiting for the peer relationship to be established (can be
controlled using the C<nacltask_wait> argument).

=item Handling when the relationship already exists

The C<nacltask_if_exists> argument can be used to control what to do
if the relationship already exists. (default is to fail if relationship
already exists)

=item Verification

The C<nacltask_verify> (defaults to being switched off) argument can be used
to control whether to verify that the relationship was established.

=item Handling inter-cluster relationships

For inter-cluster relationships, the library takes care of ensuring that the
peer is accepted on the peer cluster. Also, the peer-cluster name is
automatically determined.

Note that for inter-cluster relationships either C<remote_interface>
has to be provided or C<peer-vserver> should be provided as a Vserver object.

=back

Uses CMode CLI/ZAPI.

=over

=item Options

=over

=item C<< command_interface => $ci >>

The command_interface on which the local vserver is present.

=item C<< vserver  => $vserver_name >>

(Required) Name of the vserver.

=item C<< 'peer-vserver' => $peer_vserver >>

(Required) Either the name of the peer vserver or a Vserver object instance
for the peer vserver.

=item C<< 'local-name' => $local-name >>

(Optional) Specifies the unique local name to identify the peer Vserver with
which you want to create the Vserver peer relationship.The default value is the
remote peer Vserver name.

=item C<< nacltask_if_exists => $action >>

(Optional) What to do if the peer relation to be created already exists.
If $action is "die", then fail with an exception of type
NACL::C::Exceptions::VserverPeer::AlreadyExists. If action is
"purge", then destroy the peer relationship (see the "purge" method, below)
before creating a new one. If $action is "reuse", It will return
the object of the existing vserver peer relationship.

=item C<< _was_created => \$scalar >>

(Optional) When this option is provided a reference to a scalar
variable, the variable gets filled in with a boolean value
describing whether the relationship was found (value will be 0; this
scenario is possible when if_exists => "reuse") or whether the
relationship was created (value will be 1). This is necessary to
determine whether the relationship needs to be cleaned up later.

    my $was_created;
    my $peer_obj = NACL::STask::VserverPeer->create(
        nacltask_if_exists    => 'reuse',
        _was_created => \$was_created,
        %other_opts
    );
    # Operate on $peer_obj here
    # ...
    # Now determine whether to clean up the relationship, since we're not sure
    # whether we reused an created peer relation or created the purged relationship.
    if ($was_created) {
        # Purged relationship was created. Clean it up.
        $peer_obj->purge();
    }

=item C<< nacltask_wait => 0|1 >>

(Optional) If 1 (the default), wait for the vserver peer create to be completed.
If 0, do not wait any longer than necessary.

=item C<< nacltask_verify => 0|1 >>

(Optional, defaults to 0) If set to 1, verify that the peer relationship
was established.

=item C<< remote_interface => $remote_cluster_ci >>

This is the command_interface for the remote cluster.

This is necessary for inter-cluster peer relationships and if C<peer-vserver>
is not provided as a Vserver object. This should B<NOT> be provided for
intra-cluster relationships.

=item Other options

All of the other various options accepted by
L<< NACL::C::VserverPeer->create|lib-NACL-C-VserverPeer-pm/create >>.

=back

=back

=over

=item Arguments defaulted/auto-determined

=over

=item applications

If not provided, this is defaulted to ['snapmirror'] (i.e. defaults to
peering for snapmirror)

=item peer-cluster

For inter-cluster relationships, the name of the peer cluster is determined
automatically (from either the C<remote_interface> or the C<peer-vserver>
object)

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup)
Flag indicating if this operation needs to be registered for clean up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverPeer::AlreadyExists>

This type of exception is thrown when an attempt is made to create vserver peer relation that already exists.

=item C<NATE::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the created vserver peer relation.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args)  = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            nacltask_if_exists => $pkg->_if_exists_validate_spec(),
            nacltask_verify    => { type => BOOLEAN, default => 0 },
            _was_created       => { type => SCALARREF, optional => 1 },
            nacltask_wait      => { type => SCALAR, default => 1 },
            applications       => { type => ARRAYREF, default => ['snapmirror'] },
            'peer-vserver'     => { type => SCALAR | OBJECT },
            remote_interface   => {
                type     => OBJECT,
                isa      => 'NACL::C::CommandInterface::ONTAP',
                optional => 1
            },
            is_intercluster    => { type => SCALAR, optional => 1 },
            $pkg->_cleanup_validate_spec(),
        },
        allow_extra => 1
    );
    # Transform %opts from the options we received into the options to
    # pass to the base class method.
    my ( %nacltask_opts, %opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup );
    $pkg->_move_common_cleanup_opts(
        source  => \%opts,
        target  => \%opts_for_cleanup,
    );

    $pkg->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_opts
    );

    my $was_created = delete $opts{_was_created};
    my $if_exists   = $nacltask_opts{'nacltask_if_exists'};
    my $verify      = $nacltask_opts{'nacltask_verify'};
    my $wait        = $nacltask_opts{nacltask_wait};
    my $remote_interface = delete $opts{remote_interface};
    my $is_intercluster = delete $opts{is_intercluster};

    # Should not provide both remote_interface and peer-vserver as an object
    $pkg->_remote_interface_peer_vs_exclusive_check(
        remote_interface => $remote_interface,
        'peer-vserver'   => $opts{'peer-vserver'}
    );
    my %common_opts;
    $pkg->_copy_common_component_params(
        source => \%opts,
        target => \%common_opts
    );

    my $cluster_name = sub {
        return NACL::CS::ClusterIdentity->fetch(
            command_interface => $_[0],
            %common_opts
        )->name();
    };

    if (defined $remote_interface) {
        # Has to be inter-cluster, so set to 1
        $is_intercluster = 1;

        # Attempt to determine the peer-cluster name if not provided
        unless (defined $opts{'peer-cluster'}) {
            $opts{'peer-cluster'} = $cluster_name->($remote_interface);
        }
    }

    if (ref $opts{'peer-vserver'}) {
        my $peer_vs = $opts{'peer-vserver'};
        my $peer_ci = $peer_vs->command_interface();
        if ($peer_vs->isa('NACL::C::Vserver')) {
            # Get the name of the peer-vserver
            $opts{'peer-vserver'} = $peer_vs->vserver();

            my $peer_cluster = $opts{'peer-cluster'};
            if (defined $is_intercluster) {
                if ($is_intercluster) {
                    # The remote interface is the peer vserver's ci.
                    $remote_interface = $peer_ci;
                }
            } else {
                if (refaddr ($opts{command_interface}) eq refaddr ($peer_ci)) {
                    $is_intercluster = 0;
                } else {
                    unless (defined $peer_cluster) {
                        $peer_cluster = $cluster_name->($peer_ci);
                    }

                    my $this_cluster = $cluster_name->($opts{command_interface});
                    if ($this_cluster eq $peer_cluster) {
                        $is_intercluster = 0;
                    } else {
                        $is_intercluster = 1;
                        $remote_interface = $peer_ci;
                        $opts{'peer-cluster'} = $peer_cluster;
                    }
                }
            }
        } else {
            # peer-vserver is an object, but not a Vserver object
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw("'peer-vserver' is allowed to "
                . "be an object of type 'NACL::C::Vserver' (or its "
                . "subclasses but instead it was provided as a '"
                . ref ($peer_vs)
                . "'. Dumper:\n"
                . NACL::ComponentUtils::Dumper($peer_vs)
                . "\n"
            );
        }
    }

    if (!defined $is_intercluster) {
        # Could be inter-cluster, let's check if that's the case.
        my $peer_cluster = $opts{'peer-cluster'};
        if (defined $peer_cluster) {
            my $this_cluster = $cluster_name->($opts{command_interface});
            if ($this_cluster eq $peer_cluster) {
                $is_intercluster = 0;
            } else {
                $is_intercluster = 1;
                $remote_interface = $pkg->_create_remote_interface(
                    'peer-cluster'    => $peer_cluster,
                    command_interface => $opts{command_interface},
                );
            }
        } else {
            $is_intercluster = 0;
        }
    }

    if ($is_intercluster && !defined $remote_interface) {
        my $peer_cluster = $opts{'peer-cluster'};
        if (defined $peer_cluster) {
            $remote_interface = $pkg->_create_remote_interface(
                'peer-cluster'    => $peer_cluster,
                command_interface => $opts{command_interface},
            );
        } else {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw("'is_intercluster' was set to 1 "
                . "but 'remote_interface' was not passed, 'peer-vserver' was "
                . "not provided as a Vserver object and 'peer-cluster' was "
                . 'not provided'
            );
        }
    }

    $$was_created = 0;

    my $job;
    $opts{job_component} ||= \$job;

    my $self;
CREATE: {
        use warnings qw(exiting);
        try {
            $self = $pkg->SUPER::create(%opts);
            $$was_created = 1;
            $self->job_component(${$opts{job_component}});
        }
        catch NACL::C::Exceptions::VserverPeer::AlreadyExists with {
            my $exception_obj = $_[0];
            $Log->trace('Relationship already exists');
            my %purge_opts = (is_intercluster => $is_intercluster);
            if (defined $remote_interface) {
                $purge_opts{remote_interface} = $remote_interface;
            }
            $self = $pkg->_element_exists_handler(
                create_opts        => \%opts,
                nacltask_if_exists => $if_exists,
                exception          => $exception_obj,
                purge_opts         => \%purge_opts,
            );
            if (!$self) {
                no warnings qw(exiting);
                redo CREATE;
            }
        };
    }
    # Will be defined only for inter-cluster relationships, so first check
    # if defined.
    if (defined $remote_interface) {
        $self->remote_interface($remote_interface);
    }

    $self->is_intercluster($is_intercluster);

    my %flipped_options = (
        command_interface => $remote_interface,
        vserver           => $opts{'peer-vserver'},
        'peer-vserver'    => $opts{vserver}
    );

    if(defined($opts{'local-name'})) {
        $flipped_options{'local-name'} = $opts{'local-name'};
        my $count=0;
        my $peer_vserver;
FETCH: {
        try {
            #wait before fetching peer vserver
            sleep(10);
            $peer_vserver = NACL::CS::VserverPeer->fetch(
                command_interface => $remote_interface,
                filter            => {
                    'remote-vserver-name' => $flipped_options{'peer-vserver'},
                },
                requested_fields => ['peer-vserver'],
            )->peer_vserver();
        } catch  NACL::Exceptions::NoElementsFound with{
            my $ex = shift;
            $count++;
            # re-try 3 times to get peer vserver
            if($count <= 3) {
                goto FETCH;
            } else {
                $Log->comment("After trying for 3 times, no peer vserver found");
                $ex->throw();
            }
        };
}
        $flipped_options{'peer-vserver'} = $peer_vserver;
    }

    if ($$was_created) {
        if ($wait || $verify || $is_intercluster) {
            $self->wait_for_create(%common_opts);
        }

        if ($is_intercluster) {
            my $is_enabled = $pkg->SUPER::is_vserver_simplicity_enabled(%flipped_options);
            # For inter-cluster relationships, it is necessary to accept
            # the relationship from the peer cluster. Note that the vserver
            # and peer-vserver are swapped at the peer cluster.
            if(! $is_enabled) {
                 $pkg->accept(%common_opts, %flipped_options);
            }
            else {
                $Log->warn("Skipping accept since vserver simplicity feature is enabled");
            }
        }

        if ($verify) {
            # Not applicable for verification
            delete $opts{job_component};

            # "vserver peer show" does not show all fields. The fields
            # "peer-cluster" and "applications" are returned only by a separate
            # "vserver peer show-all" command. See burt 684868 for a request to
            # combine them.
            delete $opts{'peer-cluster'};
            if(defined($opts{'local-name'})) {
                my $peer_vserver = delete $opts{'local-name'};
                $opts{'peer-vserver'} = $peer_vserver;
                $flipped_options{'peer-vserver'} = $peer_vserver;
                $self->peer_vserver($peer_vserver);
                delete $flipped_options{'local-name'};
            }
            $self->verify_state(%opts);

            # Verify on the peer cluster as well if inter-cluster
            if ($is_intercluster) {
                $pkg->verify_state(%opts, %flipped_options);
            }
        }
    }

    #Register this resource with the cleanup manager for cleanup.
    $pkg->_copy_common_opts_for_cleanup(
        'source'             => {%opts, %opts_for_cleanup},
        'target'             => \%opts_for_register,
        'nacltask_to_cleanup'=> \$nacltask_to_cleanup,
        'to_cleanup'         => 'purge',
    );
    $pkg->_register_for_cleanup(%opts_for_register) if ($nacltask_to_cleanup);

    $Log->exit() if $may_exit;
    return $self;

} ## end sub create

=head2 purge

    # Basic class method call for intra-cluster relationship
    NACL::STask::VserverPeer->purge(
        command_interface => $ci,
        vserver           => $vs_name,
        'peer-vserver'    => $peer_vs_name,

        # This includes "nacltask_wait", "nacltask_verify", "nacltask_if_purged"
        %other_opts
    );

    # Basic class method call for inter-cluster relationship.
    # "remote_interface" is the ci for the remote cluster.
    NACL::STask::VserverPeer->purge(
        command_interface => $ci,
        vserver           => $vs_name,
        'peer-vserver'    => $peer_vs_name,
        remote_interface  => $remote_cluster_ci,
        %other_opts
    );

    # Class method call that works for both intra and inter-cluster relationships
    # Pass "peer-vserver" as a Vserver object instance
    NACL::STask::VserverPeer->purge(
        command_interface => $ci,
        vserver           => $vs_name,
        'peer-vserver'    => $peer_vs_obj,
        %other_opts
    );

    # Instance call
    $vs_peer_obj->purge();

(Class or Instance method) Deletes the peer relationship of the vserver.

In addition to the facilities provided by the component, the following
functionality is provided by the STask:

=over

=item Waiting for completion

Defaults to waiting for the peer relationship to be purged (can be
controlled using the C<nacltask_wait> argument). For inter-clustr
relationships, it waits on both the source as well as the remote cluster.

=item Handling when the relationship does not exist

The C<nacltask_if_purged> argument can be used to control what to do
if the relationship does not exist. (default is to fail if relationship
already exists)

=item Verification

The C<nacltask_verify> (defaults to being switched off) argument can be used
to control whether to verify that the relationship was purged. For inter-cluster
relationships, it verifies that the relationship was purged on both the
source and the remote cluster.

=back

Uses a CMode CLI/ZAPI APISet.

=over

=item Options

For an inter-cluster relationship, any one of the following are mandatory:

* If invoked as a class method, then either "remote_interface" should be
provided or "peer-vserver" should be provided as a Vserver object instance.

* If invoked with an object instance, then the above point is applicable
if the object instance's C<remote_interface> attribute is not populated.

=over

=item C<< command_interface => $ci >>

(Required for class method, Not Applicable for instance method)
A component object that represents the host which to send commands.
See NACL::C::Component::command_interface

=item C<< vserver => $vserver_name >>

(Required for class method, Not Applicable for instance method)
 The name of the vserver.

=item C<< 'peer-vserver' => $peer_vserver >>

(Required for class method, Not Applicable for instance method)
The name of the peer vserver name.

=item C<< nacltask_verify => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the deletion did happen
successfully.

=item C<< nacltask_if_purged=>die|pass >>

(Optional)
If "die", exception is thrown when peer relation-ship to be deleted does not exist
If "pass", exits quietly when peer relation-ship to be deleted does not exist

=item C<< nacltask_wait => 0|1 >>

(Optional) If 1 (the default), wait for this vserver peer delete to be completed.
If 0, do not wait any longer than necessary.

command_interface, apiset_must, apiset_should, etc. All of
the other various options to NACL::C::VserverPeer->delete.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverPeer::DoesNotExist>

This type of exception is thrown when an attempt is made to delete vserver peer relation that does not exists.

=item C<NATE::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the deleted vserver peer relation.

=back

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            nacltask_verify    => { type => BOOLEAN, default => 0 },
            nacltask_if_purged => { type => SCALAR, default => 'pass' },
            nacltask_wait      => { type => SCALAR, default => 1 },
            source_ci          => { type => OBJECT, optional => 1 },
            'peer-vserver'     => { type => SCALAR | OBJECT },
            remote_interface   => {
                type     => OBJECT,
                isa      => 'NACL::C::CommandInterface::ONTAP',
                optional => 1
            },
            is_intercluster    => { type => SCALAR, optional => 1 }
        },
        allow_extra => 1,
    );
    my $is_intercluster = delete $opts{is_intercluster};
    if (!defined ($is_intercluster) && ref ($pkg_or_obj) &&
        $pkg_or_obj->is_intercluster_isset()) {
        $is_intercluster = $pkg_or_obj->is_intercluster();
    }

    my $remote_interface = delete $opts{remote_interface};
    my $source_ci        = delete $opts{source_ci};
    if (defined $source_ci) {
        $Log->warn("Passing 'source_ci' into NACL::STask::Vserver->purge "
            . "is deprecated. Use 'remote_interface' instead");
    }
    $remote_interface ||= $source_ci;
    if (!defined ($remote_interface) && ref ($pkg_or_obj) &&
        $pkg_or_obj->remote_interface_isset()) {
        $remote_interface = $pkg_or_obj->remote_interface();
    }
    $is_intercluster = 1 if (defined $remote_interface);

    # Should not provide both remote_interface and peer-vserver as an object
    $pkg_or_obj->_remote_interface_peer_vs_exclusive_check(
        remote_interface => $remote_interface,
        'peer-vserver'   => $opts{'peer-vserver'}
    );

    my %nacltask_opts;
    $pkg_or_obj->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_opts
    );
    my $verify = $nacltask_opts{'nacltask_verify'};

    my %common_opts;
    $pkg_or_obj->_copy_common_component_params(
        source => \%opts,
        target => \%common_opts
    );

    my $cluster_name = sub {
        return NACL::CS::ClusterIdentity->fetch(
            command_interface => $_[0],
            %common_opts
        )->name();
    };

    if (ref $opts{'peer-vserver'}) {
        my $peer_vs = $opts{'peer-vserver'};
        my $peer_ci = $peer_vs->command_interface();
        if ($peer_vs->isa('NACL::C::Vserver')) {
            $opts{'peer-vserver'} = $peer_vs->vserver();
            if (defined $is_intercluster) {
                if ($is_intercluster) {
                    $remote_interface = $peer_ci;
                }
            } else {
                if (refaddr ($opts{command_interface}) eq refaddr ($peer_ci)) {
                    $is_intercluster = 0;
                } else {
                    my $this_cluster = $cluster_name->($opts{command_interface});
                    my $peer_cluster = $cluster_name->($peer_ci);
                    if ($this_cluster eq $peer_cluster) {
                        $is_intercluster = 0;
                    } else {
                        $is_intercluster = 1;
                        $remote_interface = $peer_ci;
                    }
                }
            }
        } else {
            # peer-vserver is an object, but not a Vserver object
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw("'peer-vserver' is allowed to "
                . "be an object of type 'NACL::C::Vserver' (or its "
                . "subclasses but instead it was provided as a '"
                . ref ($peer_vs)
                . "'. Dumper:\n"
                . NACL::ComponentUtils::Dumper($peer_vs)
                . "\n"
            );
        }
    }

    if (!defined $remote_interface &&
        ($is_intercluster || !defined $is_intercluster)) {
        # "show" does not give all details. See ONTAP burt 684868.
        my $peer_cs = NACL::C::VserverPeer->show_all(
            %common_opts,
            command_interface  => $opts{command_interface},
            requested_fields   => ['peer-cluster'],
            filter             => {
                vserver        => $opts{vserver},
                'peer-vserver' => $opts{'peer-vserver'}
            },
            allow_empty => 1
        );

        if (defined $peer_cs) {
            my $peer_cluster = $peer_cs->peer_cluster();
            if (!defined $is_intercluster) {
                my $this_cluster = NACL::CS::ClusterIdentity->fetch(
                    command_interface => $opts{command_interface},
                    %common_opts
                )->name();
                if ($this_cluster eq $peer_cluster) {
                    $is_intercluster = 0;
                } else {
                    $is_intercluster = 1;
                }
            }

            if ($is_intercluster) {
                $remote_interface = $pkg_or_obj->_create_remote_interface(
                    'peer-cluster'    => $peer_cluster,
                    command_interface => $opts{command_interface},
                );
            }
        } else {
            # If no CS object was returned, then it means we're attempting
            # to delete a non-existent relationship. Don't fail here,
            # continue through to the delete() call, where it will decide
            # what to do based on the "if_purged" value.
        }
    }

    # Delete the relationship from the source cluster.
    $pkg_or_obj->delete(
        %opts,
        %nacltask_opts,
        remote_interface => $remote_interface
    );

    my %flipped_options = (
        command_interface => $remote_interface,
        vserver           => $opts{'peer-vserver'},
        'peer-vserver'    => $opts{vserver}
    );

    my $pkg = $pkg_or_obj->get_package_name();
    if (defined $remote_interface) {
        # For inter-cluster relationships, we need to purge the relationship
        # from the peer cluster as well.
        # Note: HAS to be a package call. Since we're flipping vserver and
        # peer-vserver, calls with an instance would result in failure due
        # to the clash in values of vserver/peer-vserver passed through @_
        # and those in the object.
        $pkg->delete(
            %opts,
            %nacltask_opts,
            %flipped_options,
            remote_interface  => $opts{command_interface},
        );
    }

    if ($verify) {
        my %common_opts_ci;
        $pkg_or_obj->_copy_common_component_params_with_ci(
            source => \%opts,
            target => \%common_opts_ci
        );
        my %primary_keys;
        $pkg_or_obj->_copy_primary_keys(
            source => \%opts,
            target => \%primary_keys
        );
        $pkg_or_obj->_generic_purge_verify(%common_opts_ci, %primary_keys);

        if (defined $remote_interface) {
            # Again, for inter-cluster we should check on the peer-cluster
            # that the relationship no longer exists. For the same
            # reasoning as given above, must be a package call.
            $pkg->_generic_purge_verify(%common_opts_ci, %flipped_options);
        }
    }
    $Log->exit() if $may_exit;
} ## end sub purge

# Not to be called externally. Hence not documented and does away with
# validation as this is to be invoked ONLY from within the implementation
# of purge, by which time _common_validate_with has flattened the arguments
#
# In addition to doing SUPER::delete, it does the following:
# 1. Handles if the relationship is already purged
# 2. Deletes any snapmirror relationship(s)
# 3. Waits for completion if necessary
sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $remote_interface = delete $opts{remote_interface};
    my %nacltask_opts;
    $pkg_or_obj->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_opts
    );
    my $wait      = $nacltask_opts{nacltask_wait};
    my $verify    = $nacltask_opts{nacltask_verify};
    my $if_purged = $nacltask_opts{nacltask_if_purged};

    my $job;
    $opts{job_component} ||= \$job;

    my %common_opts;
    $pkg_or_obj->_copy_common_component_params(
        source => \%opts,
        target => \%common_opts
    );

AGAIN: {
        use warnings qw(exiting);
        try {
            $pkg_or_obj->SUPER::delete(force => 'true', %opts);
            my $job = ${$opts{job_component}};
            if (defined ($job) && ref $pkg_or_obj) {
                $pkg_or_obj->job_component($job);
            }
            if ($wait || $verify) {
                $pkg_or_obj->wait_for_purge(%common_opts, job_component => $job);
            }
        } catch NACL::C::Exceptions::VserverPeer::DoesNotExist with {
            my $e = $_[0];
            if ( $if_purged !~ /pass/i ) {
                $Log->exit() if $may_exit;
                $e->throw();
            }
        } catch NACL::C::Exceptions::VserverPeer::RelationshipIsInUseBySnapmirror with {
            my $exception = $_[0];
            $pkg_or_obj->_delete_snapmirror_relation( %opts,
                'remote_interface' => $remote_interface, );
            no warnings qw(exiting);
            redo AGAIN;
        } catch NACL::Exceptions::JobFailed with {
            my $exception = $_[0];

            if ( $exception->text() =~
                /Relationship\s+is\s+in\s+use\s+by\s+SnapMirror/i ) {
                $pkg_or_obj->_delete_snapmirror_relation( %opts,
                    'remote_interface' => $remote_interface, );
                no warnings qw(exiting);
                redo AGAIN;
            }
            $Log->exit() if $may_exit;
            $exception->throw();
        };
    }

    $Log->exit() if $may_exit;
}

# Private subroutine not to be called externally, hence not documented.
# it does the following:
# Delete the snapmirror relationship, which is associated with vserver
# It will also ensure that snapmirror is released, which is associated with vserver
sub _delete_snapmirror_relation {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, %opts ) = @_;

    my %common_opts;
    $pkg_or_obj->_copy_common_component_params(
        source => \%opts,
        target => \%common_opts
    );

    my $remote_interface = delete $opts{remote_interface};

    # purge the Snapmirror
    my $find_and_purge = sub {
        my %opts_in_sub = validate_with(
            params => \@_,
            spec   => {
                command_interface => { type => OBJECT },
                purge_opts        => { type => HASHREF, default => {} },
            }
        );

        # Find any snapmirror relationships which have either vserver
        # or peer-vserver as the destination-vserver.
        my @mirrors = NACL::STask::Snapmirror->find(
            %common_opts,
            command_interface => $opts_in_sub{command_interface},
            filter            => {
                'destination-vserver' => "$opts{vserver}|$opts{'peer-vserver'}",
                'source-vserver'      => "$opts{vserver}|$opts{'peer-vserver'}",
            },
            allow_empty => 1
        );

        foreach my $mirror (@mirrors) {
            $mirror->purge( %common_opts, %{ $opts_in_sub{purge_opts} } );
        }
    };

    # The above code only purge if snapmirror relationship is not deleted.
    # When the snapmirror is deleted but not released, the above code, doesn't do any action.
    # the below code is implemented when snapmirror is deleted, but not rleased.
    my $snap_release = sub {
        my %opts_in_sub = validate_with(
            params => \@_,
            spec   => { command_interface => { type => OBJECT }, }
        );

        my @list_dest = NACL::CS::SnapmirrorListDestinations->fetch(
            %common_opts,
            command_interface => $opts_in_sub{command_interface},
            filter            => {
                'destination-vserver' => "$opts{vserver}|$opts{'peer-vserver'}",
                'source-vserver'      => "$opts{vserver}|$opts{'peer-vserver'}",
            },
            allow_empty => 1,
        );

        foreach my $obj (@list_dest) {
            NACL::STask::Snapmirror->release(
                command_interface  => $opts_in_sub{command_interface},
                'destination-path' => $obj->destination_path(),
                %common_opts,
            );

        }
    };

    # in case of intercluster
    if ( defined $remote_interface ) {
        # First purge the snapmirror for both local and remote cluster
        $find_and_purge->(
            command_interface => $opts{command_interface},
            purge_opts        => { source_ci => $remote_interface }
        );
        $find_and_purge->(
            command_interface => $remote_interface,
            purge_opts        => { source_ci => $opts{command_interface} }
        );

        # Release the snapmirror for both local and remote cluster
        $snap_release->( command_interface => $opts{command_interface} );
        $snap_release->( command_interface => $remote_interface )

    } else {
        # For intra-cluster
        $find_and_purge->( command_interface => $opts{command_interface}, );
        $snap_release->( command_interface => $opts{command_interface}, );
    }

    $Log->exit() if $may_exit;
}


=head2 accept, suspend, reject, resume, modify

    $VserverPeer->$method(); ## $method can be accept, suspend, reject, resume, modify

            (or)

    NACL::STask::VserverPeer->$method(
        command_interface => $ci,
        vserver              => $vserver_name,
        'peer-vserver'       => $peer_vserver,
        %other_options_accepted_by_$method,
        nacltask_wait        => 0 | 1 # Default 1,
    );

(Class or Instance method) All these methods accept the options accepted by
the underlying Component method as well as a new C<nacltask_wait> argument.
When C<nacltask_wait> is set to 1 (the default), it waits on the job started
(if a job was started). The default timeout is 300 seconds.

Uses CMode CLI/ZAPI.

=cut

BEGIN {
    no strict 'refs';
    # Same implementation for accept, reject, suspend, modify and resume
    # Rather than repeat the code, we can have them assigned to the same subroutine
    # reference.
    foreach my $method (qw(accept modify reject resume suspend)) {
        *{$method} = sub {
            $Log->enter() if $may_enter;
            my ( $pkg_or_obj, @args ) = @_;
            my %opts = $pkg_or_obj->_common_validate_with(
                params          => \@args,
                additional_spec => {
                    nacltask_wait      => { type => SCALAR, default => 1 },
                },
                allow_extra => 1,
            );
            my $wait = delete $opts{nacltask_wait};
            my $super_method = "SUPER::$method";

            my $temp_job;
            $opts{job_component} ||= \$temp_job;

            $pkg_or_obj->$super_method(%opts);
            my $job = ${$opts{job_component}};
            $pkg_or_obj->job_component() if (ref ($pkg_or_obj) && defined $job);

            if ($wait && $job) {
                my $wait_method = "wait_for_$method";
                my %common_opts;
                $pkg_or_obj->_copy_common_component_params(
                    source => \%opts,
                    target => \%common_opts
                );
                $pkg_or_obj->$wait_method(%common_opts, job_component => $job);
            }

            $Log->exit() if $may_exit;
        };
    }

=head2 wait_for_create, wait_for_purge, wait_for_accept etc

    $VserverPeer->wait_for_$method();
    # $method can be create, purge, accept, suspend, reject, resume, modify

            (or)

    NACL::STask::VserverPeer->wait_for_$method(
        command_interface => $ci,
        job_component     => $job_component,
        'method-timeout'  => $method_timeout,
    );

(Class or Instance method) This method waits till the job completes. Default
timeout is 300 seconds.

=over

=item Options

=over

=item C<< job_component => $job_component >>

(Required for class method, Optional for instance method)
A job component modeling the job started for the operation.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional, defaults to 300s)
The time, in seconds, for which to wait for the job to complete.

=back

=back

=cut

    foreach my $method (qw(create purge accept modify reject resume suspend)) {
        my $wait_method = "wait_for_$method";
        *{$wait_method} = sub {
            $Log->enter() if $may_enter;
            my ($pkg_or_obj, @args) = @_;
            my @primary_keys = $pkg_or_obj->get_primary_keys();
            my %opts = validate_with(
                params => \@args,
                spec   => {
                    job_component => { type => OBJECT | UNDEF, optional => 1 },
                    %{$pkg_or_obj->_common_validate_spec()},
                    command_interface => { optional => 1 },
                    'method-timeout'  => { type => SCALAR, default => 300 },
                },
            );
            my $job = $opts{job_component} || $pkg_or_obj->job_component();

            if ($job) {
                my %common_opts;
                $pkg_or_obj->_copy_common_component_params(
                    source => \%opts,
                    target => \%common_opts
                );
                $job->wait_on_job(%common_opts);
            }
            $Log->exit() if $may_exit;
        };
    }
    use strict 'refs';
}

sub _create_remote_interface {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            'peer-cluster'    => { type => SCALAR },
            command_interface => _ontap_ci_validate_spec(),
        },
    );

    my $peer_cluster = $opts{'peer-cluster'};
    my $remote_interface;

    my %common_opts;
    $pkg_or_obj->_copy_common_component_params_with_ci(
        source => \%opts,
        target => \%common_opts
    );

    my $cls_peer = NACL::CS::ClusterPeer->fetch(
        %common_opts,
        filter            => { cluster => $peer_cluster, },
        requested_fields  => [qw(remote-cluster-nodes)],
    );

    # Get the remote interface
    my @remote_nodes = $cls_peer->remote_cluster_nodes();

    try {
        my $host = Tharn::host($remote_nodes[0]);
        $remote_interface = NACL::STask::Cluster->new(hostrec => $host);
    } otherwise {
        $opts{command_interface}->_no_hostrec_error(
            name         => $peer_cluster,
            config_error => $_[0]
        );
    };

    $Log->exit() if $may_exit;
    return $remote_interface;
}

sub _remote_interface_peer_vs_exclusive_check {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            remote_interface => { required => 1 },
            'peer-vserver'   => { required => 1 },
        }
    );

    if (defined ($opts{remote_interface}) && ref $opts{'peer-vserver'}) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw("Either 'remote_interface' "
            . "should be provided or 'peer-vserver' should be provided "
            . "as an object, but both cannot be provided together");
    }

    $Log->exit() if $may_exit;
}

=head2 get_vserver_instances

    # This method returns a blessed object, which in turn has the
    # two vserver objects as its attributes
    my $instances        = $peer_obj->get_vserver_instances();

    # Use the accessor methods to get the vserver instances themselves
    my $vserver_obj      = $instances->vserver_obj();
    my $peer_vserver_obj = $instances->peer_vserver_obj();

A vserver peer object models a relationship between two vservers. This
method can be used to get C<NACL::STask::Vserver> object instances for
both vservers in the peer relationship.

=cut

sub get_vserver_instances {
    $Log->enter() if $may_enter;
    my ($self, @opts) = @_;

    my $vserver = NACL::STask::Vserver->new(
        command_interface => $self->command_interface(),
        vserver           => $self->vserver()
    );

    $self->_update_object(@opts);

    my $peer_ci;
    if ($self->is_intercluster()) {
        $peer_ci = $self->remote_interface();
    } else {
        $peer_ci = $self->command_interface();
    }
    my $peer_vserver = NACL::STask::Vserver->new(
        command_interface => $peer_ci,
        vserver           => $self->peer_vserver()
    );

    my $instances = _GetVserverInstances->new(
        vserver_obj      => $vserver,
        peer_vserver_obj => $peer_vserver
    );
    $Log->exit() if $may_exit;
    return $instances;
}

# Update the is_intercluster and remote_interface attributes of the object
# if they're not already set.
sub _update_object {
    $Log->enter() if $may_enter;
    my ($self, @opts) = @_;
    if (!$self->is_intercluster_isset()) {
        my $cs = $self->show_all(@opts, requested_fields => ['peer-cluster']);
        my $peer_cluster = $cs->peer_cluster();
        my $this_cluster = NACL::CS::ClusterIdentity->fetch(
            @opts,
            command_interface => $self->command_interface()
        )->name();
        if ($this_cluster eq $peer_cluster) {
            $self->is_intercluster(0);
        } else {
            my $remote_interface = $self->_create_remote_interface(
                'peer-cluster'    => $peer_cluster,
                command_interface => $self->command_interface()
            );
            $self->remote_interface($remote_interface);
        }
    }
    $Log->exit() if $may_exit;
}

1;

package _GetVserverInstances;

use Class::MethodMaker [
    new    => [ '-hash', 'new' ],
    scalar => [
        { -type => 'NACL::STask::Vserver' },
        'vserver_obj',
    ],
    scalar => [
        { -type => 'NACL::STask::Vserver' },
        'peer_vserver_obj',
    ],
];

1;
