# $Id: $
#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverSecurityFileDirectoryNtfsSacl Task Module
## @author kathar.hidayath@netapp.com, dl-nacl-dev
## @status shared
## @pod here

package NACL::STask::VserverSecurityFileDirectoryNtfsSacl;

use strict;
use warnings;

use base qw(NACL::C::VserverSecurityFileDirectoryNtfsSacl NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR BOOLEAN SCALARREF ARRAYREF);
use NACL::STask::_Fsecurity qw(:all);
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ResponseException ();
use NACL::Exceptions::VerifyFailure             ();

=head1 NAME

NACL::STask::VserverSecurityFileDirectoryNtfsSacl

=head1 DESCRIPTION

C<NACL::STask::VserverSecurityFileDirectoryNtfsSacl> provide methods to add,
purge vserver security file directory ntfs sacl in ONTAP.

It is created on top of C<NACL::C::VserverSecurityFileDirectoryNtfsSacl> component
Since it is derived class of C<NACL::C::VserverSecurityFileDirectoryNtfsSacl>,
we can use all the methods of C<NACL::C::VserverSecurityFileDirectoryNtfsSacl>
from the object of this task.

=head1 ATTRIBUTES

=head2 command_interface

(Required) A component object that represents the host to which to send commands
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

=head2 vserver

(Required) The name of the Vserver to be used

=head2 ntfs-sd

(Required) Security Descriptor Name to be created

=head2 account

(Required) Account Name or SID

=head2 access_type

(Required) Access type whether to Allow or Deny

=head1 METHODS

=head2 add

  my $descriptorTask = NACL::STask::VserverSecurityFileDirectoryNtfsSacl->add(
                                command_interface    => $ci,
                                'account'            => $account,
                                'vserver'            => $vserver,
                                'ntfs-sd'            => $ntfs_sd,
                                'access-type'        => $access_type,
                                'nacltask_if_exists' => $action # default die
                                %other_options,
                                ...);

                                 or

    $VserverSecurityFileDirectoryNtfsSacl_obj->add(...);

(Class or instance method)
This method is used to add an ACE to NTFS Security Descriptor SACL.
If descriptor already exists, it will perform the action based on
"nacltask_if_exists" parameter. Default behavior would be "die".

Uses CMode CLI/ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'account' => $account >>

(Required for class method, Not Applicable for instance method)
Account Name or SID

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Name of the vserver to be used.

=item C<< 'ntfs-sd' => $ntfs_sd >>

(Required for class method, Not Applicable for instance method)
Security Descriptor Name

=item C<< 'access-type' => $access_type >>

(Required for class method, Not Applicable for instance method)
Allow or Deny

=item C<< 'rights' => $string >>

(Optional) SACL ACE's Access Rights

=item C<< 'rights-raw' => $string >>

(Optional) SACL ACE's Raw Access Rights

=item C<< 'apply-to' => [ $apply-to1, $apply-to2, ... ] >>

(Optional) (Arrayref, even if only a single value is to be sent it
should specified as an arrayref containing that single value)

=item C<< 'advanced-rights' => [ $advanced-right1, $advanced-right2, ... ] >>

(Optional) (Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

=item C<< 'nacltask_verify' => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the addition did happen
successfully.


=item C<< "nacltask_if_exists => $action" >>

(Optional) What to do if the descriptor to be added already exists.
If $action is "die", then fail with an exception. If action is
"purge", then purge the descriptor(see the "purge" method, below)
before creating a new one. If $action is "reuse", It will return
the object of existing descriptor.

=item C<< "nacltask_verify => $action" >>

(Optional) The user of this library can specify to verify whether the Sacl
is added or not. If the action is 0, which is default, it wont verify the
addition. If the user sets the action to 1, it will verify the
it using the component state of this library.

=item C<< "_was_created => \$scalar" >>

(Optional) When this option is provided with a reference to a scalar
variable, the variable gets filled in with a boolean value
describing whether the descriptor was available (value will be 0; this
scenario is possible when if_exists => "reuse") or whether the
descriptor was added (value will be 1). This is necessary to
determine whether the descriptor needs to be cleaned up later.

my $was_created;

my $descriptor_obj = NACL::STask::VserverSecurityFileDirectoryNtfsSacl->add(
                        $nacltask_if_exists     => 'reuse',
                        _was_created            => \$was_created,
                        %other_opts
                    );

# Operate on $descriptor_obj here
# ...
# Now determine whether to clean up the descriptor, since we're not sure
# whether we reused an existing descriptor or added a new one
if ($was_created) {
   # New descriptor was added. Clean it up.
   $descriptor_obj->purge();
}

Supports all of the other various options to
NACL::C::VserverSecurityFileDirectoryNtfsSacl->add

L<< NACL::C::VserverSecurityFileDirectoryNtfsSacl->add|
lib-NACL-C-VserverSecurityFileDirectoryNtfsSacl-pm/add >>

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverSecurityFileDirectoryNtfsSacl::AlreadyExists>

This type of exception is thrown when an attempt is made to add vserver security file directiory ntfs sacl that already exists.

=item C<NATE::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the added vserver security file directiory ntfs sacl.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my $self = $pkg->_add_create_helper( @_, method => 'add' );
    $Log->exit() if $may_exit;
    return $self;
} ## end sub add

=head2 purge

    $obj = NACL::STask::VserverSecurityFileDirectoryNtfsSacl->purge(
                                command_interface => $ci,
                                'account' => $account,
                                'vserver' => $vserver,
                                'ntfs-sd' => $ntfs_sd,
                                'access-type' => $access_type,
                                %other_options,
                                ...);

                                 or

    $VserverSecurityFileDirectoryNtfsSacl_obj->purge(...);

(Class or instance method)
This method is used to remove an ACE entry from NTFS security descriptor SACL.
Uses a CMode CLI/ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required for class method, Not Applicable for instance method)
A component object that represents the host which to send commands.
See NACL::C::Component::command_interface

=item C<< 'account' => $account >>

(Required for class method, Not Applicable for instance method)
Account Name or SID

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Name of the vserver to be used.

=item C<< 'ntfs-sd' => $ntfs_sd >>

(Required for class method, Not Applicable for instance method)
Security Descriptor Name

=item C<< 'access-type' => $access_type >>

(Required for class method, Not Applicable for instance method)
Allow or Deny

=item C<< 'nacltask_verify' => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the deletion did happen
successfully.

=item C<< nacltask_if_purged => $nacltask_if_purged >>

(Optional)
If 'pass' (default), It will pass if the descriptor is already deleted.
If 'fail', It will fail if the descriptor is already deleted.

Supports all of the other various options to
NACL::C::VserverSecurityFileDirectoryNtfsSacl->remove

L<< NACL::C::VserverSecurityFileDirectoryNtfsDacl->remove|
lib-NACL-C-VserverSecurityFileDirectoryNtfsDacl-pm/remove >>

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverSecurityFileDirectoryNtfsSacl::DoesNotExist>

This type of exception is thrown when an attempt is made to delete vserver security file directiory ntfs sacl that does not exists.

=item C<NATE::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the deleted vserver security file directiory ntfs sacl.

=back

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_purge_helper( @_, method => 'remove' );
    $Log->exit() if $may_exit;
} ## end sub purge

1;
