# $Id: $
#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverSecurityFileDirectoryPolicyTask Task Module
## @author kathar.hidayath@netapp.com, dl-nacl-dev
## @status shared
## @pod here

package NACL::STask::VserverSecurityFileDirectoryPolicyTask;

use strict;
use warnings;

use base
    qw(NACL::C::VserverSecurityFileDirectoryPolicyTask NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::STask::_Fsecurity qw(:all);
use Params::Validate qw(validate SCALAR BOOLEAN SCALARREF);
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ResponseException ();
use NACL::Exceptions::VerifyFailure             ();

=head1 NAME

NACL::STask::VserverSecurityFileDirectoryPolicyTask

=head1 DESCRIPTION

C<NACL::STask::VserverSecurityFileDirectoryPolicyTask> provide methods to add,
purge vserver security file directory policy Task in ONTAP.

It is created on top of C<NACL::C::VserverSecurityFileDirectoryPolicyTask> component
Since it is derived class of C<NACL::C::VserverSecurityFileDirectoryPolicyTask>,
we can use all the methods of C<NACL::C::VserverSecurityFileDirectoryPolicyTask>
from the object of this task.

=head1 ATTRIBUTES

=head2 command_interface

(Required) A component object that represents the host to which to send commands
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

=head2 index

(Required) Target Position for adding the policy task.

=head2 vserver

(Required) The name of the Vserver to be used

=head2 path

(Required) Path to which the policy task to be added.

=head2 policy_name

(Required) Policy Name to be added

=head1 METHODS

=head2 add

  my $PolicyTask = NACL::STask::VserverSecurityFileDirectoryPolicyTask->new(
                                   command_interface => $ci,
                                   'index' => $index,
                                   'vserver' => $vserver,
                                   'path' => $path,
                                   'policy-name' => $policy_name,
                                   'nacltask_if_exists' => $action # default die
                                   %other_options,
                    );

(Class or Instance Method) This method is used to add the file security policy task.
If policy task already exists, it will perform the action based on
"nacltask_if_exists" parameter. Default behavior would be "die".

Uses CMode CLI/ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
The name of the Vserver to be used

=item C<< 'policy-name' => $policy_name >>

(Required for class method, Not Applicable for instance method)
Policy Name to be added

=item C<< 'path' => $path >>

(Required for class method, Not Applicable for instance method)
Path to which the policy task to be added.

=item C<< 'index' => $string >>

(Required for class method, Not Applicable for instance method)
Target Position for adding the policy task.


=item C<< 'nacltask_verify' => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the addition did happen
successfully.


=item C<< "nacltask_if_exists => $action" >>

(Optional) What to do if the policy to be added already exists.
If $action is "die", then fail with an exception. If action is
"purge", then purge the policy(see the "purge" method, below)
before creating a new one. If $action is "reuse", It will return
the object of existing policy.

=item C<< "nacltask_verify => $action" >>

(Optional) The user of this library can specify to verify whether the policy
Task is added or not. If the action is 0, which is default, it wont verify the
addition. If the user sets the action to 1, it will verify the
it using the component state of this library.

=item C<< "_was_created => \$scalar" >>

(Optional) When this option is provided with a reference to a scalar
variable, the variable gets filled in with a boolean value
describing whether the policy was available (value will be 0; this
scenario is possible when if_exists => "reuse") or whether the
policy was added (value will be 1). This is necessary to
determine whether the policy needs to be cleaned up later.

my $was_created;

my $policy_obj = NACL::STask::VserverSecurityFileDirectoryPolicyTask->add(
                        $nacltask_if_exists     => 'reuse',
                        _was_created            => \$was_created,
                        %other_opts
                    );

# Operate on $policy_obj here
# ...
# Now determine whether to clean up the policy, since we're not sure
# whether we reused an existing policy task or added a new one
if ($was_created) {
   # New policy was added. Clean it up.
   $policy_obj->purge();
}

Supports all of the other various options to
NACL::C::VserverSecurityFileDirectoryPolicyTask->add

L<< NACL::C::VserverSecurityFileDirectoryPolicyTask->add|
lib-NACL-C-VserverSecurityFileDirectoryPolicyTask-pm/add >>

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverSecurityFileDirectoryPolicyTask::AlreadyExists>

This type of exception is thrown when an attempt is made to add vserver security file directory policy task that already exists.

=item C<NATE::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the added vserver security file directory policy task.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my ( $pkg, %opts )  = @_;
    my $self = $pkg->_add_create_helper( %opts, method => 'add' );
    $Log->exit() if $may_exit;
    return $self;
} ## end sub add

=head2 purge

    $Policy_obj->purge();
            (or)
    NACL::STask::VserverSecurityFileDirectoryPolicyTask->purge(
            'command_interface' => $ci,
            'vserver'           => $vserver_name,
            'policy-name'       => $policy_name,
            'path'              => $path,
            'nacltask_verify'   => 1,
    );

(Class or instance method)
This method is used to purge a file security policy task.
Uses a CMode CLI/ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required for class method, Not Applicable for instance method)
A component object that represents the host which to send commands.
See NACL::C::Component::command_interface

=item C<< vserver => $vserver_name >>

(Required for class method, Not Applicable for instance method)
 The name of the vserver.

=item C<< 'policy-name' => $policy_name >>

(Required for class method, Not Applicable for instance method)
The policy name to be deleted.

=item C<< 'path' => $path >>

(Required for class method, Not Applicable for instance method)
Path to which the policy task is to be added.

=item C<< 'nacltask_verify' => $nacltask_verify_boolean >>

(Optional)
If '0' (default), verification will not be performed.
If '1', verification will be performed to ensure that the deletion did happen
successfully.

=item C<< nacltask_if_purged => $nacltask_if_purged >>

(Optional)
If 'pass' (default), It will pass if the policy task is already deleted.
If 'fail', It will fail if the policy task is already deleted.

Supports all of the other various options to
NACL::C::VserverSecurityFileDirectoryPolicyTask->remove

L<< NACL::C::VserverSecurityFileDirectoryPolicyTask->remove|
lib-NACL-C-VserverSecurityFileDirectoryPolicyTask-pm/remove >>

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverSecurityFileDirectoryPolicyTask::DoesNotExist>

This type of exception is thrown when an attempt is made to delete vserver security policy task that does not exists.

=item C<NATE::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the deleted vserver security policy task.

=back

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_purge_helper( @_, method => 'remove' );
    $Log->exit() if $may_exit;
} ## end sub purge

1;
