#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Mix-in for the VolumeFile|AggregateFile Stasks
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::_Mixins::AggregateVolumeFile;

use strict;
use warnings;

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(
        purge
        dumpblock_info
        _calculate_timeout_based_on_size
        _get_aggr_or_vol_name_from_path
    );
}

use Params::Validate qw(:all);
use NACL::C::UnitNormalization;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::BaseException qw(:try);
use NACL::ComponentUtils qw(_optional_scalars);
use NATE::Time qw(timeout2time);

sub purge {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            nacltask_wait => {type => SCALAR, default => 1},

            # Need to make this optional because it's possible to provide
            # "paths" instead.
            path             => {type => SCALAR, optional => 1},
            'method-timeout' => {type => SCALAR, default  => 120},
            polling_interval => {type => SCALAR, default  => 10},
            $pkg_or_obj->_nodescope_options_validate_spec(),
        },
        allow_extra => 1,
    );

    my $ci = $opts{command_interface};
    my $wait     = delete $opts{nacltask_wait};
    my $interval = delete $opts{polling_interval};
 
    my $vol_or_aggr_file;
    my %volume_file_opts;
    if ($pkg_or_obj->isa('NACL::STask::VolumeFile')) {
        $vol_or_aggr_file = "VolumeFile";
        $volume_file_opts{vserver} = $opts{vserver};
    } else {
        $vol_or_aggr_file = "AggregateFile";
    }

    # Get common component params: apiset_must, apiset_should, etc
    my %common_params = ();
    $pkg_or_obj->_copy_common_component_params(
        source => \%opts,
        target => \%common_params
    );

    if ( $ci->is_cmode() ) {
        if (   $opts{determine_appropriate_nodescope_node}
        && !defined $opts{'nodescope-node-name'}) {

            # If this is set then we need to make multiple nodescope calls.
            # Since all operations are operating on the same volume, they're
            # all operating on the same node, so let's compute this once,
            # rather than require every call to recompute the value.
            $common_params{'nodescope-node-name'} = $pkg_or_obj->get_local_node_name(
                command_interface => $ci,
                path              => $opts{path},
                %volume_file_opts,
                %common_params
            ) if (exists $opts{path});
        # For Find call
        } elsif (defined $opts{'nodescope-node-name'} ) {
            $common_params{'nodescope-node-name'} = $opts{'nodescope-node-name'};
        }
    } ## end if ( $ci->is_cmode()...})

    $pkg_or_obj->C_SUPER::delete(%opts, %common_params);

    if ($wait && !$opts{command_interface}->version_manager()->is_sksim()) {
        my %find_opts;
        $pkg_or_obj->_copy_common_component_params_with_ci(
            source => \%opts,
            target => \%find_opts
        );

        # Either of "path" or "paths" will be provided, not both (the
        # Component validates this). "path" might be provided as a
        # space-separated list of paths (should not be done, but we need to
        # maintain backwards compatibility with older calls). Normalize
        # to form an array of paths.
        my @paths;
        if (exists $opts{path}) {
            @paths = split /\s+/, $opts{path};
        } else {
            @paths = @{$opts{paths}};
        }
        my $paths_str = join '|', @paths;
        my $pkg = $pkg_or_obj->get_package_name();

        my $end_time = timeout2time($opts{'method-timeout'});

        while (1) {
            my @found = $pkg->find(
                %find_opts,
                filter => {%volume_file_opts, path => $paths_str},
                %common_params,
                allow_empty => 1
            );
            last if (!@found);

            if (time() > $end_time) {
                my @found_files = map { $_->path() } @found;
                my @fg_files;
                foreach my $path(@found_files) {
                    my $obj = $pkg_or_obj->_extract_volume_file_from_path(path => $path);
                    my $volume = $obj->volume();
                    my $vol_obj = NACL::CS::Volume->fetch(
                        command_interface => $ci,
                        filter => {
                           volume => $volume
                        },
                    );
                    push (@fg_files,$path) if (defined $vol_obj && $vol_obj->is_flexgroup());
                }
                if(@fg_files){
                    $pkg_or_obj->C_SUPER::delete(
                        command_interface => $ci,
                        paths => \@fg_files,
                        apiset_must => { interface => 'ZAPI'}
                    );
                }
                $Log->exit() if $may_exit;
                NACL::Exceptions::Timeout->throw(
                    'All files did not get '
                        . "purged within the timeout of $opts{'method-timeout'}. "
                        . "The following files still exist:\n"
                        . join('; ', @found_files),
                    timeout => $opts{'method-timeout'}
                );
            } else {
                Tharn::snooze($interval);
            }
        }
    }

    $Log->exit();
} ## end sub purge

sub dumpblock_info {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    $Log->debug(sub { "Input options are " . Dumper({@args}) });
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(qw(path)),
            $pkg_or_obj->_nodescope_options_validate_spec(),
        },
        allow_extra => 1
    );

    my $vol_or_aggr_file;
    my %volume_file_opts;
    if ($pkg_or_obj->isa('NACL::STask::VolumeFile')) {
        $vol_or_aggr_file = "volume";
        $volume_file_opts{vserver} = $opts{vserver};
    } else {
        $vol_or_aggr_file = "aggregate";
    }

    my $path = delete $opts{path};

    if (!defined($opts{fileid}) && !defined($path)) {
        $Log->exit();
        NATE::Exceptions::Argument->throw(
            "Either of 'fileid' or 'path' " . "should be specified");
    }

    unless (defined $opts{fileid}) {
        my %common_args;
        $pkg_or_obj->_copy_common_component_params(
            source => \%opts,
            target => \%common_args
        );
        my %showfh_args = %common_args;
        $pkg_or_obj->_hash_copy(
            source => \%opts,
            target => \%showfh_args,
            copy   => [qw(vserver command_interface)]
        );
        $showfh_args{path} = $path;

        # If "determine_appropriate_nodescope_node" is set, then let's do this
        # determination once for both commands.
        if ($opts{determine_appropriate_nodescope_node}
            && !defined $opts{'nodescope-node-name'}) {
            my $node = $pkg_or_obj->get_local_node_name(
                command_interface => $opts{command_interface},
                path              => $path,
                %volume_file_opts,
                %common_args
            );
            $opts{'nodescope-node-name'}        = $node;
            $showfh_args{'nodescope-node-name'} = $node;
        } ## end if ( $opts{determine_appropriate_nodescope_node...})
        $opts{fileid} = $pkg_or_obj->showfh(%showfh_args)->fileid();
    } ## end unless ( defined $opts{fileid...})

    unless (defined $opts{$vol_or_aggr_file}) {
        if (defined $path) {
            $opts{$vol_or_aggr_file}
               = $pkg_or_obj->_get_aggr_or_vol_name_from_path(path => $path);
        } else {
            $Log->exit();
            NATE::Exceptions::Argument->throw("Either of $vol_or_aggr_file or "
                . "'file' option should be specified");
        }
    }

    # The component method can only be invoked as a class method
    my $pkg = $pkg_or_obj->get_package_name();
    my $obj = $pkg->C_SUPER::dumpblock_info(%opts);
    $Log->exit();
    return $obj;
} ## end sub dumpblock_info

# Helper Method to get the method-timeout based on the file size
sub _calculate_timeout_based_on_size {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => {
            size => { type => SCALAR, },
            filecount => { type => SCALAR|UNDEF, optional => 1},
        }
    );
    # I've noticed that it takes ~35s to create a 1GB file (close to
    # 2GB per minute). With this in mind, assuming file creation
    # should happen at a rate of 200MB per minute should be safe.

    my $file_size_bytes;
    try {
        $file_size_bytes = $pkg_or_obj->convert_to_bytes($opts{size});
    } otherwise {
        # Would be because size was not provided/size was provided but
        # with an invalid size suffix. Either case will result in
        # the command failing, so let's suppress this exception
    };

    unless (defined $file_size_bytes) {
        return 120;
    }
    my $mb = $file_size_bytes / 1024;
    if (defined $opts{filecount}) {
        $mb *= $opts{filecount};
    }
    my $time = int(($mb * 60) / 200);

    $Log->exit();
    # Even if $time is a low value (because the file size is small)
    # set to a minimum of 2 minutes
    if ($time < 120) {
        return 120;
    } else {
        return $time;
    }
}

# Helper Method to get the volume or aggregate depending upon the path
sub _get_aggr_or_vol_name_from_path {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => {
            path => { type => SCALAR, },
        }
    );

    my $vol_or_aggr_file;
    if ($pkg_or_obj->isa('NACL::STask::VolumeFile')) {
        $vol_or_aggr_file = "VolumeFile";
    } else {
        $vol_or_aggr_file = "AggregateFile";
    }

    my $path = $opts{path};
    my $aggr_or_vol;
    if ($vol_or_aggr_file eq 'VolumeFile') {
        $aggr_or_vol = $pkg_or_obj->_get_vol_name_from_path(path => $path);
    } else {
        $aggr_or_vol = $pkg_or_obj->_extract_aggregate_file_from_path
         ( path => $path )->{aggr_name};
    }
    $Log->exit();
    return $aggr_or_vol;
}

1;
