#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Mix-in for the AntivirusOnDemandCommand STasks
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::STask::_Mixins::AntivirusOnDemandCommand

=head1 DESCRIPTION

This mix-in contains the implementation for the various
AntivirusOnDemandCommand STasks.

=cut

package NACL::STask::_Mixins::AntivirusOnDemandCommand;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR SCALARREF BOOLEAN CODEREF);
use NATE::BaseException qw(:try);
use NACL::STask::AntivirusOnDemand;

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw( create purge modify );
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}

sub create {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            nacltask_if_exists => $pkg->_if_exists_validate_spec(),
            nacltask_verify    => { type => SCALAR, default => 0 },
            _was_created       => { type => SCALARREF, optional => 1 },
        },
        allow_extra => 1,
    );

    my $nacltask_if_exists = delete $opts{nacltask_if_exists};
    my $verify             = delete $opts{nacltask_verify};
    my $was_created        = delete $opts{_was_created};
    $$was_created = 0;
    my $self;

CREATE: {
        use warnings qw(exiting);
        try {
            $self         = $pkg->C_SUPER::create(%opts);
            $$was_created = 1;
            $pkg->taskverify_create(%opts) if $verify;
        }
        catch NACL::C::Exceptions::AlreadyExists with {
            my $exception = shift;
            $self = $pkg->_element_exists_handler(
                create_opts        => \%opts,
                nacltask_if_exists => $nacltask_if_exists,
                exception          => $exception
            );
            if ( !$self ) {
                no warnings qw(exiting);
                redo CREATE;
            }
        };
    }

    $Log->exit() if $may_exit;
    return $self;
}

sub purge {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            nacltask_if_purged =>
                $pkg_or_obj->_if_action_completed_already_validate_spec(),
            nacltask_verify => { type => BOOLEAN, default => 0 },
        },
        allow_extra => 1,
    );
    my $if_purged = delete $opts{nacltask_if_purged};
    my $verify    = delete $opts{nacltask_verify};
    my $pkg       = $pkg_or_obj->get_package_name();

DELETE: {
        use warnings qw(exiting);
        try {
            $pkg->delete(%opts);
            $pkg->taskverify_purge(%opts) if $verify;
        }
        catch NACL::C::Exceptions::DoesNotExist with {
            my $exception = shift;
            if ( $if_purged =~ /die/ ) {
                $Log->exit() if $may_exit;
                $exception->throw();
            }
        }
        except {
            my $sub = sub {
                my %common_opts;
                $pkg_or_obj->_copy_common_component_params_with_ci(
                    source => \%opts,
                    target => \%common_opts
                );
                my $cs = NACL::CS::AntivirusOnDemand->fetch(
                    %common_opts,
                    filter           => { name => $opts{name} },
                    requested_fields => ['jobid']
                );
                NACL::STask::AntivirusOnDemand->unschedule(
                    %common_opts,
                    vserver => $cs->vserver(),
                    id      => $cs->jobid(),
                );
                no warnings qw(exiting);
                redo DELETE;
            };
            my $od = 'NACL::C::Exceptions::AntivirusOnDemandCommand';
            my %except_hash = ( "${od}::InUse" => $sub );
            foreach my $type (qw(File Dir Vserver Cluster)) {
                $except_hash{"${od}Scan${type}::InUse"} = $sub;
            }
            return \%except_hash;
        };
    }

    $Log->exit() if $may_exit;
}

sub modify {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_call_method_then_verify(@args);
    $Log->exit() if $may_exit;
}

1;
