#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Mix-in for the Lun Move|Copy Stasks
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::_Mixins::LunMoveCopy;

use strict;
use warnings;

use NATE::Log qw(log_global);
use Params::Validate qw(:all);
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::TimeoutException;
use NACL::Exceptions::UnexpectedState;
use NACL::STask::Exceptions::LunMoveCopyFailed;
use NATE::BaseException;
use Scalar::Util qw(blessed);
use NACL::ComponentUtils qw(Dumper);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

BEGIN {
    use Exporter qw(import);
    our %EXPORT_TAGS = (
        all => [
            qw(_start_standard _start_enhanced start wait_for_finish
              _is_standard_args_call _is_enhanced_args_call
              wait_for_destination_ready _wait_helper _extract_slun_path
              _generate_lun_paths)
        ]
    );
    our @EXPORT_OK = @{ $EXPORT_TAGS{all} };
} ## end BEGIN

=head1 NAME

NACL::STask::_Mixins::LunMoveCopy

=head1 DESCRIPTION

NACL::STask::_Mixins::LunMoveCopy This class is a mix-in to the
<NACL::STask::LunMove> and <NACL::STask::LunCopy> tasks. Classes are
able to import the symbols provided by this class to provide that
functionality natively within their own class. This class should not be
used directly. The functionality provided by this package will be
available directly from classes augmenting their class with this mixin.

=head1 ATTRIBUTES

=head2 command_interface

(Required) As C<NACL::C::LunMove> or C<NACL::C::LunCopy>. A component
 object that represents the host by which command command will be
 executed.

=head2 destination-path

(Required) As C<NACL::C::LunMove> or C<NACL::C::LunCopy>.The destination
path of the lun.

=head2 vserver

(Required for C-mode, ignored for 7-mode) As C<NACL::C::LunMove> or
C<NACL::C::LunCopy>. The name of the vserver.

=head1 METHODS

=head2 start

Start a Lun move/copy operation by either using the standard form supported by
the component or by using an enhanced form that allows specification of luns
via relative paths plus volume component instances. This method defaults to
waiting for completion of the operations.

  # Standard Form
  my @lun_moves = NACL::STask::LunMove->start(
      command_interface  => $command_interface,
      'source-paths'     => "/vol/volume/lun1", | ["path1", ...],
      'destination-paths'=> "/vol/volume/lun2", | ["path1", ...],,
      'vserver'          => $vserver,
      nacltask_wait      => $boolean,     # default "1"
      %other_options
  );

  # Intra vserver Lun Copy
  my @lun_copies = NACL::STask::LunCopy->start(
      command_interface  => $command_interface,
      'source-paths'     => "/vol/volume/lun1", | ["path1", ...],
      'destination-paths'=> "/vol/volume/lun2", | ["path1", ...],,
      'vserver'          => $vserver,
      nacltask_wait      => $boolean,     # default "1"
      %other_options
  );

  # Inter vserver Lun Copy
  my @lun_copies = NACL::STask::LunCopy->start(
      command_interface  => $command_interface,
      'source-paths'     => "/vol/volume1/lun1", | ["path1", ...],
      'destination-paths'=> "/vol/volume2/lun2", | ["path1", ...],,
      'vserver'          => $dest_vserver,
      source-vserver     => $source_vserver,
      nacltask_wait      => $boolean,     # default "1"
      %other_options
  );

  # Enhanced Form
  my @lun_moves = NACL::STask::LunMove->start(
      source_volume => NACL::C::Volume instance,
      destination_volume => NACL::C::Volume instance,
      source_luns => "lun1" | ["lun", ...],
      source_destination_map => {'lun1' => 'lun1_dest', ...},
	  #Has hashref of source luns and destination luns as key value pairs.
	  #These are just lun names and not the whole lun path.
      %other_options
  );

  my @lun_copies = NACL::STask::LunCopy->start(
      source_volume => NACL::C::Volume instance,
      destination_volume => NACL::C::Volume instance,
      source_luns => "lun1" | ["lun", ...],
      source_destination_map => {'lun1' => 'lun1_dest', ...},
      %other_options
  );


(Class method) used to move a LUN or a set of LUNs from one volume
to another within a Vserver and copy a lun or set of LUNs from one
volume to another within a Vserver or across Vservers. This method
allows the user to directly pass volume components and relative
luns to the constructor. nacltask_wait is can also be passed in.
nacltask_wait is defaulted to '1.'

=over

=item Options

=over

=item C<< source-paths >>

(Required unless covered by other arguments)
Specifies the full path of the source LUN, in the format
/vol/<volume>[/<qtree>]/<lun>.

=item C<< destination-paths >>

(Required unless covered by other arguments)
Specifies the full path of the destination LUN, in the format
/vol/<volume>[/<qtree>]/<lun>.

=item C<< source_volume >>

(Enhanced form only)
A NACL::C::Volume instance for the volume to be used as the source of the
single lun move/copy ondemand job.

=item C<< destination_volume >>

(Enhanced form only)
A NACL::C::Volume instance for the volume to be used as the destination of the
single lun move/copy ondemand job.

=item C<< source_luns >>

(Enhanced form only)
Either a single lun name or a arrayref of lun names can be moved/copied.

=item C<< source_destination_map >>

(Enhanced form only)
Has hashref of source luns and destination luns as key value pairs based on
which lun move/copy is done. These are just lun names and not the whole lun path.
If it is not specified destination luns will be named as such as source luns.

=item C<< nacltask_wait => 0|1 >>

(Optional) If 1 (the default), wait for the ondemand op to finish (i.e. waits
both for the destination file to be ready and for the scanner to complete)

=item C<< nacltask_wait_for_destination_ready => 0|1 >>

(Optional)
This is applicable only if nacltask_wait=0. If this option is set to 1,
then only wait until the destination lun is ready (don't wait until the
scanner has completed)

=item C<< nacltask_fail_on_destroy => 0|1 >>

(Optional; Default is 1)
When set to 1 (the default), a scanner-status of C<destroy> indicates a
failure has occurred. For the rare cases where we do not want to consider
a scanner-status of C<destroy> to be a failure, then this should be set to 0.

=item other options

All the other various options described in
NACL::C::LunMove->start NACL::C::LunCopy->start are supported by
NACL::STask::_Mixins::LunMoveCopy->start as well.

=over

=item Exceptions

=over

=item C<NACL::STask::Exceptions::LunMoveCopyFailed>

This type of exception is thrown when lun move/copy operation is failed.

=item C<NATE::Exceptions::Argument>

This type of exception is thrown when there are insufficient arguments
to start lun move.

=item C<NACL::Exceptions::UnexpectedState>

This type of exception is thrown when scanner status for an lun move/copy job
is in unexpected state.

=item C<NACL::APISet::Exceptions::TimeoutException>

This type of exception is thrown when timeout occured while waiting
for lun move job to finish.

=item Other exceptions

See L<< NACL::C::LunMove >> and L<< NACL::C::LunCopy >>
for all other exceptions that this method could throw.

=back

=back

=back

=back

=head2 wait_for_finish

Wait for an lun move/copy job to finish

  NACL::STask::LunMove->wait_for_finish(
         command_interface => $command_interface,
         "job-uuid" => ...,
         "lun-index" => ...,
  );
  $lun_stask->wait_for_finish();

(Class or instance method) Wait for a lun move/copy job
to finish.

=over

=item Instance and Static method Options

=over

=item C<< method-timeout >>

(Optional) How long to wait for the job to finish. Defaults to
-1 for no timeout.

=item other options

Any of the standard options
to NACL component methods such as the apiset must/should
arguments are passed on to the underlying component calls.

=back

=back

=over

=item Static method Options

=over

=item C<< command_interface >>

As required to instantiate a NACL::C::Lun{Move,Copy} instance.


=item C<< job-uuid >>

As required to instantiate a NACL::C::Lun{Move,Copy} instance.


=item C<< lun-index >>

As required to instantiate a NACL::C::Lun{Move,Copy} instance.

=back

=back

=head2 wait_for_destination_ready

Wait for a destination lun to be ready to access.

  NACL::STask::LunMove->wait_for_destination_ready(
     command_interface => $command_interface,
     "is-destination-ready" => ...,
  );
  $lun_stask->wait_for_destination_ready();

(Class or instance method) Wait for a
destination lun to be ready to access.

=over

=item Instance and Static method Options

=over

=item C<< method-timeout >>

(Optional) How long to wait for the job to finish. Defaults to
-1 for no timeout.

=item other options

Any of the standard options
to NACL component methods such as the apiset must/should
arguments are passed on to the underlying component calls.

=back

=back

=over

=item Static method Options

=over

=item C<< command_interface >>

As required to instantiate a NACL::C::Lun{Move,Copy} instance.

=item C<< job-uuid >>

As required to instantiate a NACL::C::Lun{Move,Copy} instance.

=item C<< lun-index >>

As required to instantiate a NACL::C::Lun{Move,Copy} instance.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;

    my ( $pkg, %opts ) = @_;

    # Copy all of the possible required options for different forms. We will
    # then decide which form is being used to make the call
    my %validate_options;

    $pkg->_hash_copy(
        source => \%opts,
        copy   => [
            qw(
              command_interface source-paths destination-paths reference-path
              source_volume destination_volume source_luns source_destination_map
              vserver source-vserver
              )
        ],
        target => \%validate_options,
    );


    my @od_jobs;
    if ( $pkg->_is_standard_args_call(%validate_options) ) {
        @od_jobs = $pkg->_start_standard(%opts);
    }
    elsif ( $pkg->_is_enhanced_args_call(%validate_options) ) {
        @od_jobs = $pkg->_start_enhanced(%opts);
    }
    else {
        NATE::Exceptions::Argument->throw(
                "Insufficient arguments to start, please use either the "
              . "standard or enhanced form but not a combination of the two" );
    } ## end else [ if ( $pkg->_is_standard_args_call...)]

    $Log->exit() if $may_exit;
    return (wantarray) ? @od_jobs : $od_jobs[0];
} ## end sub start

sub _start_standard {
    my ( $pkg, @opts ) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@opts,
        additional_spec => {
            nacltask_wait => { type => SCALAR, default => 1 },

            # Defaults for these are set in the wait_for_finish method,
            # so don't duplicate here
            nacltask_wait_for_destination => {type => BOOLEAN, optional => 1},
            nacltask_fail_on_destroy      => {type => BOOLEAN, optional => 1},
            polling_interval              => {type => SCALAR,  optional => 1},
        },
        ignore_primary_keys => 1,
        allow_extra         => 1,
    );
    my $wait             = delete $opts{nacltask_wait};
    my $destination_wait = delete $opts{nacltask_wait_for_destination};

    my %wait_args;
    $pkg->_hash_move(
        source => \%opts,
        target => \%wait_args,
        move   => [
            qw(polling_interval nacltask_fail_on_destroy
              )
        ]
    );

    # Convert any NACL::C::Lun instances in {source,destination}-paths
    foreach my $key (qw(source-paths destination-paths)) {

        # Don't modify the hash reference in place since the caller might not
        # appreciate us changing that.
        if ( ref $opts{$key} ) {
            my @translated_args = ();
            foreach my $lun ( @{ $opts{$key} } ) {
                $lun = $pkg->_extract_slun_path($lun);
                push @translated_args, $lun;
            }
            $opts{$key} = \@translated_args;
        }

    }
    if ( defined $opts{'reference-path'} ) {
        my $lun = $opts{'reference-path'};
        $opts{'reference-path'} =
          $pkg->_extract_slun_path( $opts{'reference-path'} );
    }
    if ( defined $opts{'source-paths'} && !(ref $opts{'source-paths'}) ) {
        my @translated_args_src = ();
        my $lun_src =
          $pkg->_extract_slun_path( $opts{'source-paths'} );
        push @translated_args_src, $lun_src;
        $opts{'source-paths'} = \@translated_args_src;
    }
    if ( defined $opts{'destination-paths'} && !(ref $opts{'destination-paths'})) {
        my @translated_args_des = ();
        my $lun_des =
          $pkg->_extract_slun_path( $opts{'destination-paths'} );
        push @translated_args_des, $lun_des;
        $opts{'destination-paths'} = \@translated_args_des;
    }
    my @operations;
    try {
        @operations = $pkg->C_SUPER::start(%opts);
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        NACL::STask::Exceptions::LunMoveCopyFailed
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
   };

    if ( $wait || $destination_wait ) {

        # Occasionally it takes a little time for the copy/move table to be
        # updated with this operation, so doing a show/get-iter would not show
        # any entries. Let's give a couple of seconds to ensure that the
        # table is populated.
        Tharn::snooze(5);
        $pkg->_copy_common_component_params(
            source => \%opts,
            target => \%wait_args
        );
        if ($wait) {
            foreach my $operation (@operations) {
                $operation->wait_for_finish(%wait_args);
            }
        }
        else {
            foreach my $operation (@operations) {
                $operation->wait_for_destination_ready(%wait_args);
            }
        }
    }

    return (wantarray) ? @operations : $operations[0];
} ## end sub _start_standard

sub _start_enhanced {
    my ( $pkg, @opts ) = @_;
    my %opts = @opts;
    if ( !exists $opts{command_interface} ) {

        # Allow the user to override the command_interface from the source
        push @opts,
          ( command_interface => $opts{source_volume}->command_interface() );
    } ## end if ( !exists $opts{command_interface...})

    %opts = $pkg->_common_validate_with(
        params          => \@opts,
        additional_spec => {
            source_volume      => { isa  => 'NACL::C::Volume' , type => OBJECT},
            destination_volume => { isa  => 'NACL::C::Volume' , type => OBJECT},
            source_luns        => { type => SCALAR | ARRAYREF },
            source_destination_map => { type => HASHREF,  default => {} },
        },
        ignore_primary_keys => 1,
        allow_extra         => 1,
    );

    my $source_volume      = delete $opts{source_volume};
    my $destination_volume = delete $opts{destination_volume};

    # Extracting vserver names of the volumes

    my $source_vserver      = $source_volume->vserver();
    my $destination_vserver = $destination_volume->vserver();

    # Convert a single scalar to a list
    my $source_luns = delete $opts{source_luns};
    if ( !ref $source_luns ) {
        $source_luns = [$source_luns];
    }
    my $destination_lun_map = delete $opts{source_destination_map};
    my $destination_luns = [];

    # Check if lun_map is provided, if not consider the destination
    # luns to be same as that of source luns

    if (keys %$destination_lun_map) {
        foreach (@$source_luns) {
            push @$destination_luns, $destination_lun_map->{$_};
        }
    } else {
        @$destination_luns = @$source_luns;
    }

    my $source_paths      = [];
    my $destination_paths = [];

    # Convert the lun name to a /vol/volume/lun
    @$source_paths = $pkg->_generate_lun_paths(
        volume            => $source_volume,
        luns              => $source_luns,
    );
    @$destination_paths = $pkg->_generate_lun_paths(
        volume            => $destination_volume,
        luns              => $destination_luns,
    );
    if ( $source_vserver ne $destination_vserver ) {
        return $pkg->_start_standard(
            command_interface   => $opts{command_interface},
            vserver             => $destination_vserver,
            'source-vserver'    => $source_vserver,
            'source-paths'      => $source_paths,
            'destination-paths' => $destination_paths,
            %opts,
        );
    }
    else {
        return $pkg->_start_standard(
            command_interface   => $opts{command_interface},
            vserver             => $destination_vserver,
            'source-paths'      => $source_paths,
            'destination-paths' => $destination_paths,
            %opts,
        );
    }
} ## end sub _start_enhanced

sub wait_for_finish {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my $check_coderef = sub {
        my ($state) =
          validate_pos( @_,
            { type => OBJECT, isa => 'NACL::CS::ComponentState::ONTAP' } );
        return ( $state->job_status() =~ /complete/i );
    };
    $pkg_or_obj->_wait_helper( @args, _check_coderef => $check_coderef );
    $Log->exit() if $may_exit;
} ## end sub wait_for_finish

sub wait_for_destination_ready {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my $check_coderef = sub {
        my ($state) =
          validate_pos( @_,
            { type => OBJECT, isa => 'NACL::CS::ComponentState::ONTAP' } );
        return ( $state->is_destination_ready() =~ /true/i );
    };
    $pkg_or_obj->_wait_helper( @args, _check_coderef => $check_coderef );
    $Log->exit() if $may_exit;
} ## end sub wait_for_destination_ready

sub _wait_helper {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @opts ) = @_;
    require NACL::CS::EventLog;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@opts,
        additional_spec => {
            'method-timeout' => {
                type    => SCALAR,
                default => -1,
            },
            polling_interval         => { type => SCALAR,  default => 5 },
            nacltask_fail_on_destroy => { type => BOOLEAN, default => 1 },
            _check_coderef           => { type => CODEREF },
        },
        allow_extra => 1,
    );
    my $fail_on_destroy  = delete $opts{nacltask_fail_on_destroy};
    my $polling_interval = delete $opts{polling_interval};
    my $check_coderef    = delete $opts{_check_coderef};

    my $filter = {};
    my $job_uuid;
    my $lun_index;
    $pkg_or_obj->_move_primary_keys( source => \%opts, target => $filter );

    # Call this sub if final state is destroy. This sub throws an
    # exception if fail_on_destroy is set.
    my $destroy_sub = sub {

        # If status is "destroy", then the operation is complete.
        # Whether this terminal state is a failure is controlled
        # by the "nacltask_fail_on_destroy" flag.
        my ($reason) = validate_pos( @_, { type => SCALAR } );
        if ($fail_on_destroy) {
            $Log->exit() if $may_exit;
            NACL::Exceptions::UnexpectedState->throw(
                "Job: $job_uuid; index: $lun_index is in destroy state."
                  . "\nFailure Reason: $reason",
                monitoring_fields => [qw(job_status failure_reason)],
                job_status        => 'Destroyed',
                failure_reason    => $reason,
            );
        }
    };

    my $end_time = Tharn::timeout2time( $opts{'method-timeout'} );
    while (1) {
        my $od_state = $pkg_or_obj->get_CS_package_name->fetch(
            %opts,
            filter      => $filter,
            allow_empty => 1,
        );

        if ($od_state) {
            $job_uuid  = $od_state->job_uuid();
            $lun_index = $od_state->lun_index();
            my $job_status = $od_state->job_status();
            my $done       = $check_coderef->($od_state);
            last if $done;
            if ( $job_status =~ /destroyed/i ) {
                $destroy_sub->( $od_state->last_failure_reason() );

                # If reached here, then no exception thrown, so
                # fail_on_destroy=0, so break out of the loop.
                last;
            }
            elsif ( $job_status =~ qr/^Paused_Error$/i ) {
                $Log->exit() if $may_exit;
                NACL::Exceptions::UnexpectedState->throw(
                    "Job: $job_uuid; index: $lun_index is in Paused_Error",
                    monitoring_fields => ['job_status'],
                    job_status        => $od_state->job_status(),
                );
            }

            if ( time() > $end_time ) {

                # Throw timeout error
                $Log->exit() if $may_exit;
                NACL::APISet::Exceptions::TimeoutException->throw( "Timeout of "
                      . $opts{'method-timeout'}
                      . " seconds "
                      . "expired waiting for ondemand job to finish" );
            } ## end if ( time() > $end_time)
            Tharn::sleep($polling_interval);
        }
        else {
            # On-Demand operation entry no longer exists, so has completed.
            # We need to check EMS for whether it was successful or ended
            # up in the destroy state.
            # (The entry would no longer exist if the status is queried for
            # more than 120s after the operation has completed)
            my @events = NACL::CS::EventLog->fetch(
                %opts,

            # No direct way to filter for the exact EMS message for this
            # job,so filter for all jobs of this type (wafl.ondemand.exception)
            # and then iterate through till we find the one for this job.
                filter => { messagename => 'wafl.ondemand.exception' },
                requested_fields => ['event'],
                allow_empty      => 1,
            );

            # Create a regex that includes a list of the EMS messages
            # that should be excluded from the destroy state check because
            # they are transient & do not indicate that the job has completed.
            my $exclude_regex = _create_ems_exclusion_regex();

            # Message will look like this:
            # "wafl.ondemand.exception: Cannot create destination LUN. No
            # space is left on volume {DSID:1027}. LUN move job
            # 664a089e-bd55-4f2a-8b4c-90e74fe15a70-0."
            foreach my $ems_obj (@events) {
                my $event = $ems_obj->event();
                if (   $event =~ /lun.*job.*$job_uuid\-$lun_index/i
                    && $event !~ $exclude_regex )
                {
                    $destroy_sub->($event);
                }
            }

            # If we've reached here, then either the job succeeded, or it
            # failed and nacltask_fail_on_destroy was set to 0. Break out
            # of the while loop since we're done either way.
            last;
        }
    } ## end while (1)
    $Log->exit() if $may_exit;
}

sub _is_standard_args_call {
    $Log->enter() if $may_enter;
    my $self   = shift;
    my $return = 0;

    eval {
        Params::Validate::validate(
            @_,
            {
                command_interface => { type => OBJECT, required => 1 },
                vserver           => { type => SCALAR, required => 1 },
                'source-paths' => { type => ARRAYREF | SCALAR, required => 1 },
                'destination-paths' =>
                  { type => ARRAYREF | SCALAR, required => 1 },
                'source-vserver' => { type => SCALAR,          optional => 1 },
                'reference-path' => { type => SCALAR | OBJECT, optional => 1 },
            }
        );
        $return = 1;
    };

    $Log->exit() if $may_exit;
    return $return;
} ## end sub _is_standard_args_call

sub _is_enhanced_args_call {
    $Log->enter() if $may_enter;
    my $self   = shift;
    my $return = 0;

    eval {
        Params::Validate::validate(
            @_,
            {
                source_volume      => { type => OBJECT, required => 1 },
                destination_volume => { type => OBJECT, required => 1 },
                source_luns => { type => ARRAYREF | SCALAR, required => 1 },
                source_destination_map => { type => HASHREF, optional => 1 },
            }
        );
        $return = 1;
    };

    $Log->exit() if $may_exit;
    return $return;
} ## end sub _is_enhanced_args_call

sub _create_ems_exclusion_regex {

    # Build a regular expression of EMS messages
    # that should be excluded when determining if job
    # is in a destroy state - these are transient EMS
    # messages that don't imply a terminal state

    # These messages are defined in
    # /dev/mgmtgateway/src/zapis/ErrorCodeDefs.cc

    my @exclusions = (
        ".*Waiting for lock revocation.*",
        ".*Data scanner delayed.*",
        ".*Job delayed.*",
        ".*Waiting for available space.*",
        ".*Waiting for .* to complete.*",
        ".*Waiting for the network.*",
        ".*Waiting for snapshot create to complete.*",
        ".*Destroying remaining hard links.*",
        ".*Need to write allocation map.*",
        ".*Waiting for the associated reference transfer to cut over.*",
        ".*Waiting for associated incremental transfers to complete their "
          . "allocation phases.*",
        ".*Waiting for rename operations.*",
        ".*Waiting for snapshot restore to complete.*",
        ".*Waiting for data to be flushed to disk.*",
        ".*Waiting for related jobs to complete their preparation phases.*",
        ".*Waiting for an IO fence to be released.*",
        ".*Not ready.*",
        ".*Volume with DSID .* is offline.*",
    );

    my $regex = join( '|', @exclusions );

    return qr/$regex/;

}    ## ens sub _create_ems_exclusion_regex

sub _extract_slun_path {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, $lun ) = @_;
    my $lun_to_return;
    if ( blessed($lun) ) {
        if ( $lun->isa('NACL::C::Lun') ) {
            $lun_to_return = $lun->path();
        }
        else {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw( 'The lun should have been '
                  . 'either a path or an object of type NACL::C::Lun '
                  . "but what was provided was of type '"
                  . ref($lun)
                  . "'" );
        }
    }
    else {
        $lun_to_return = $lun;
    }
    $Log->exit() if $may_exit;
    return $lun_to_return;
} ## end sub _extract_sfod_path

sub _generate_lun_paths {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my ($luns, $volume);
    my @list_of_lun_paths;
    $luns = $opts{'luns'};
    $volume = $opts{'volume'};

    foreach my $lun (@$luns) {
        my $normalized_lun = "/$lun";
        my $path = "/vol/" . $volume->volume() . $normalized_lun;
        push @list_of_lun_paths, $path;
    } ## end foreach my $lun (@$luns)

    $Log->exit() if $may_exit;
    return @list_of_lun_paths;
} ## end sub _generate_lun_paths
1;
