# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary common routines for
##           NACL::STask::VserverServicesDns
##           NACL::STask::VserverServicesLdapClient
##           NACL::STask::VserverServicesLdap
##           NACL::STask::VserverServicesNisDomain
##
## @author rahula@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::_VserverServices;

use strict;
use warnings;

use base qw(NACL::STask::STask);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Exceptions::Argument qw(:try);
use NATE::BaseException                                           ();
use NACL::APISet::Exceptions::ResponseException                   ();
use NACL::C::Exceptions::VserverServicesDns::AlreadyExists        ();
use NACL::C::Exceptions::VserverServicesDns::DoesNotExist         ();
use NACL::C::Exceptions::VserverServicesLdapClient::AlreadyExists ();
use NACL::C::Exceptions::VserverServicesLdapClient::DoesNotExist  ();
use NACL::C::Exceptions::VserverServicesLdap::AlreadyExists       ();
use NACL::C::Exceptions::VserverServicesLdap::DoesNotExist        ();
use NACL::C::Exceptions::VserverServicesNisDomain::AlreadyExists  ();
use NACL::C::Exceptions::VserverServicesNisDomain::DoesNotExist   ();
use NACL::STask::Vserver;
use NACL::Exceptions::Timeout;
use NACL::CS::VserverServicesNameServiceFileVersion;

use NACL::Exceptions::VerifyFailure ();
use Params::Validate qw(validate SCALAR validate_with SCALARREF BOOLEAN OBJECT ARRAYREF);

BEGIN {
    use Exporter qw(import);
    our %EXPORT_TAGS = ( all => [qw(_create_helper _purge_helper _file_version_show _wait_replay _replay_check)] );
    our @EXPORT_OK = @{ $EXPORT_TAGS{all} };
}

=head1 NAME

NACL::STask::_VserverServices

=head1 DESCRIPTION

Common code for VserverServices stask implementation.

=head1 CLEANUP METHODS

Cleanup can be registered for the following methods, '_create_helper'.

Cleanup methods are,
    _VserverServices Method         Cleanup Method
    -----------------------------------------------
    _create_helper                  _purge_helper

=head2 _create_helper

    $pkg_or_obj->_create_helper();

    NACL::STask::_VserverServices->_create_helper(
        'command_interface'   => $ci,
        'vserver'             => $vserver,
        domains               => \@DomainList,
        'nacltask_if_exists'  => $action # default die
        'nacltask_to_cleanup' => 1, #default 0
        'nacltask_cleanup_manager' => $CleanupObj,
        %other_options,
    );

This method will call the base class create method of the particular VserverServices
Stask which invokes this method. The user has to pass the method to be called
as one of the input. This method itself verify the creation based
on the nacltask_verify(Refer the nacltask_verify pod of
NACL::STask::VserverServicesDns ).

=over

=item (Optional) B<< nacltask_if_exists => $action >>

Specifies an action to be taken if the vserver services already exists.

$action
  - if set to "die" (default), an exception is raised.
  - if set to "reuse", no action is performed.
  - if set to "purge", deletes the existing vserver services before creating it.

=item C<< purge_if_duplicate_by_id => 0|1 >>

(Optional, default to 0
Flag indicating that in case of duplicate entry
first fetch is performed based on 'id' & 'vserver' and then user is deleted.
By default the delete is performed on the 'vserver' & 'user'.

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup)
Flag indicating if this operation needs to be registered for clean up or not.

=item C<< nacltask_wait => 0|1 >>

(Optional, default to 0)
wait for command to replayed to secd.

=item C<< nacltask_replay_timeout => 120 >>

(Optional, default to 120)
Total waiting time for command to replayed to secd.
Exception is thrown if command is not replayedd in this time.
Effective only when nacltask_wait is set to 1.

=item C<< nacltask_replay_interval => 5 >>

(Optional, default to 5)
Time interval in attempts to check if command is replayed.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=item C<MCC options>

The options accepted for MCC configuration replication verification is
documented at L<MCC options|lib-NACL-STask-STask-pm/mcc_options>.

=back

=cut

sub _create_helper {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            nacltask_if_exists       => $pkg->_if_exists_validate_spec(),
            nacltask_verify          => { type => BOOLEAN, default => 0 },
            nacltask_wait            => { type => BOOLEAN, default => 0 },
            _was_created             => { type => SCALARREF, optional => 1 },
            purge_if_duplicate_by_id => { type => SCALAR, default => 0 },
            nacltask_verify_yp_conf_file
                => { type => BOOLEAN, default => 0 },
            nacltask_verify_file_version      => { type => BOOLEAN, default => 0 },
            nacltask_verify_file_version_retry => { type => SCALAR, default => 6 },
            nacltask_verify_file_version_interval => { type => SCALAR, default => 10 },
            nacltask_replay_timeout  => { type => SCALAR, default => 120 },
            nacltask_replay_interval => { type => SCALAR, default => 5 },
	    $pkg->_cleanup_validate_spec(),
            $pkg->_mcc_validate_spec(),
        },
        allow_extra => 1
    );

    # Transform %opts from the options we received into the options to
    # pass to the base class method.
    my ( $self, %common_opts, %nacltask_opts, %opts_for_cleanup, %opts_for_register, $nacltask_to_cleanup );

    my %mcc_opts;
    $pkg->_move_common_mcc_opts(
        source => \%opts,
        target => \%mcc_opts,
    );

    $pkg->_copy_common_component_params_with_ci(
        source => \%opts,
        target => \%common_opts
    );
    $pkg->_move_common_cleanup_opts(
          source  => \%opts,
          target  => \%opts_for_cleanup,
    );
     $pkg->_move_nacltask_options(
        source => \%opts,
        target => \%nacltask_opts
    );

    my $was_created = delete $opts{_was_created};
    $$was_created = 0;
    my $if_exists                = $nacltask_opts{'nacltask_if_exists'};
    my $verify                   = $nacltask_opts{'nacltask_verify'};
    my $wait                     = $nacltask_opts{'nacltask_wait'};
    my $purge_if_duplicate_by_id = delete $opts{'purge_if_duplicate_by_id'};
    my $purged_by_id;

CREATE: {
        use warnings;
        try {
            $self         = $pkg->C_SUPER::create(%opts);
            $$was_created = 1;
        }
        except {
            my $sub_reference = sub {
                my $exception = shift;
                $self = $pkg->_element_exists_handler(
                    create_opts        => \%opts,
                    nacltask_if_exists => $if_exists,
                    exception          => $exception
                );
                if ( !$self ) {
                    no warnings qw(exiting);
                    redo CREATE;
                }
            };
            my $handle_dups = sub {
                my $exception = shift;
                if ( $purge_if_duplicate_by_id && !$purged_by_id ) {
                    if ( $if_exists eq "purge" ) {

                        my $elem_to_del = $pkg->find(
                            %common_opts,
                            filter => {
                                'id'      => $opts{'id'},
                                'vserver' => $opts{'vserver'}
                            },
                            allow_empty => 1,
                        );
                        if ($elem_to_del) {
                            $elem_to_del->purge();
                            ++$purged_by_id;
                            no warnings qw(exiting);
                            redo CREATE;
                        }
                    }
                }

                $self = $pkg->_element_exists_handler(
                    create_opts        => \%opts,
                    nacltask_if_exists => $if_exists,
                    exception          => $exception
                );
                if ( !$self ) {
                    no warnings qw(exiting);
                    redo CREATE;
                }
            };
	    my $dns_handle = sub {
                my $exception = shift;
		$opts{'skip-config-validation'} = 'true';
                redo CREATE;
	    };
            return {
                'NACL::C::Exceptions::VserverServicesDns::AlreadyExists' =>
                    $sub_reference,
                'NACL::C::Exceptions::VserverServicesLdapClient::AlreadyExists'
                    => $sub_reference,
                'NACL::C::Exceptions::VserverServicesNameServiceLdapClient::AlreadyExists'
                    => $sub_reference,
                'NACL::C::Exceptions::VserverServicesLdap::AlreadyExists' =>
                    $sub_reference,
                'NACL::C::Exceptions::VserverServicesNisDomain::AlreadyExists'
                    => $sub_reference,
                'NACL::C::Exceptions::VserverServicesUnixGroup::AlreadyExists'
                    => $handle_dups,
                'NACL::C::Exceptions::VserverServicesUnixUser::AlreadyExists'
                    => $handle_dups,
		'NACL::C::Exceptions::VserverServicesDns::DnsServerException'
		    => $dns_handle,
            };
        };
    }

    $self->_check_in_metrocluster_node(
        command_interface => $opts{'command_interface'},
        method_used       => "create",
        %mcc_opts,
    );

    my $version_mgr = $opts{command_interface}->get_version_manager();

    if ( $nacltask_opts{'nacltask_verify_yp_conf_file'} eq '1' ) {
        $Log->warn(" nacltask_verify_yp_conf_file option is deprecated."
                   . " Update your script to not to pass this option");
    }
    if ( $nacltask_opts{'nacltask_verify_file_version'} eq '1' ) {
        $pkg->_file_version_show(%opts);
    }
    if ($verify) {
	# Since 'bind-password' field is not available for the show command , deleting it from option-list before calling verify_state. Refer ONTAP burt#1066045 .
	delete $opts{'bind-password'} if (defined $opts{'bind-password'});
	delete $opts{'skip-config-validation'} if (defined $opts{'skip-config-validation'});
        $pkg->verify_state(%opts);
    }
    if ($wait) {
	# Burt 1005088: sleep is required for DNS configuration to load after create operation.
	Tharn::snooze(5) if ( $pkg =~ /VserverServicesDns/i && $opts{if_exists} =~ /purge/i );
        $pkg->_wait_replay(%opts,%nacltask_opts);
    }

    # Register this resource with the cleanup manager for cleanup.
    $pkg->_copy_common_opts_for_cleanup(
        'source'             => {%opts, %opts_for_cleanup},
        'target'             => \%opts_for_register,
        'nacltask_to_cleanup'=> \$nacltask_to_cleanup,
        'to_cleanup'         => 'purge',
    );

    $pkg->_register_for_cleanup(%opts_for_register)
          if ($nacltask_to_cleanup && $$was_created);

    $Log->exit() if $may_exit;
    return $self;
} ## end sub _create_helper

=head2 _purge_helper

    $pkg_or_obj->_purge_helper();

    NACL::STask::_VserverServices->_purge_helper(
         'command_interface' => $ci,
         'vserver'           => $vserver,
         domains             => \@DomainList,
         %other_options,
    );

This method will call the base class delete method of the particular
VserverServices Stask which invokes this method. The user has to pass the method
to be called as one of the input. This method itself verify the deletion based on the
nacltask_verify(Refer the nacltask_verify pod of NACL::STask::VserverServicesDns ).

=over

=item C<MCC options>

The options accepted for MCC configuration replication verification is
documented at L<MCC options|lib-NACL-STask-STask-pm/mcc_options>.

=back

=cut

sub _purge_helper {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %orig_opts       = @args;
    my $additional_spec = {
        nacltask_verify    => { type => BOOLEAN, default => 0 },
        nacltask_if_purged => { type => BOOLEAN, default => 'pass' },
        _was_deleted        => { type => SCALARREF, optional => 1 },
        nacltask_verify_yp_conf_file
                => { type => BOOLEAN, default => 0 },
	$pkg_or_obj->_mcc_validate_spec(),
    };

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => $additional_spec,
    );

    my %mcc_opts;
    $pkg_or_obj->_move_common_mcc_opts(
        source => \%opts,
        target => \%mcc_opts,
    );

    my %primary_keys;
    $pkg_or_obj->_copy_primary_keys(
        source => \%opts,
        target => \%primary_keys
    );

    # Throw away options specific to the task
    $pkg_or_obj->_hash_move(
        source => \%orig_opts,
        target => {},
        move   => [ keys %{$additional_spec} ]
    );

    my %common_opts;
    $pkg_or_obj->_copy_common_component_params_with_ci(
        source => \%opts,
        target => \%common_opts
    );
    my $if_purged = delete $opts{nacltask_if_purged};
    my $was_deleted = delete $opts{_was_deleted};
    $$was_deleted = 0;
    use warnings;
    try {
        $pkg_or_obj->C_SUPER::delete( %common_opts, %orig_opts );
        $$was_deleted = 1;
    }
    except {
        my $sub_reference = sub {
            my $exception = shift;
            if ( $if_purged !~ /pass/i ) {
                $Log->exit() if $may_exit;
                $exception->throw();
            }
        };
        return {
            'NACL::C::Exceptions::DoesNotExist' => $sub_reference,
        };
    };

    $pkg_or_obj->_check_in_metrocluster_node(
        command_interface => $opts{command_interface},
        method_used       => "purge",
        %mcc_opts,
	%primary_keys
    );

    my $pkg = $pkg_or_obj->get_package_name();
    if ( $opts{'nacltask_verify_yp_conf_file'} eq '1' ) {
        $Log->warn(" nacltask_verify_yp_conf_file option is deprecated."
                   . " Update your script to not to pass this option");
    }
    if ( $opts{nacltask_verify} ) {
        $pkg_or_obj->taskverify_purge( %common_opts, %orig_opts );
    }
    $Log->exit() if $may_exit;
} ## end sub _purge_helper

sub _verify_yp_config_file {
    $Log->enter() if $may_enter;
    my ($pkg,@args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            command_interface => { type => OBJECT, isa => "NACL::C::CommandInterface::ONTAP", },
            vserver           => { type => SCALAR,},
            domain            => { type => SCALAR, optional => 1 },
            servers           => { type => ARRAYREF, optional => 1 },
            _method           => { type => SCALAR,},
        },
    );

    my $ci      = $opts{command_interface};
    my $vserver = $opts{vserver};
    my $domain  = $opts{domain};
    my $method  = $opts{_method};
    my %pass_opts;

    my $debug_op = $ci->apiset->debug_smdb_table_show(
        table => 'vserver_for_ipspace', vserver => $vserver );

    my $vs_id  = $debug_op->{'parsed_output'}->[0]->{'vserver-id'};

    my $prefix = "/clus/.vserver_";
    my $suffix = "/.vsadmin/config/etc/yp.conf";
    my $path   = $prefix.$vs_id.$suffix;

    my $systemshell = $ci->apiset(
        category => "Node", interface => "CLI", set => "Systemshell",
    );

    my $output = $systemshell->execute_command(command
        => "sudo cat $path" );

    if($method eq "create") {
        $pass_opts{domain} = $domain;
        $pass_opts{servers} = $opts{servers};
    }

    my $method_name = "_".$method."_verify_yp_conf";
    $pkg->$method_name(
        systemshell_output => $output,
        %pass_opts,
    );

    $Log->exit() if $may_exit;
}

sub _create_verify_yp_conf {
    $Log->enter() if $may_enter;
    my ($pkg,@args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            systemshell_output => { type => SCALAR,},
            domain             => { type => SCALAR,},
            servers            => { type => ARRAYREF,},
        },
    );

    my $output = $opts{systemshell_output};
    my $domain = $opts{domain};
    my @servers = @{$opts{servers}};
    my $cnt = 0;

    #check domain
    my $verified_domain = 1 if( $output =~ /domain\s+($domain)/);

    foreach (@servers) {
        if($output =~ /server\s+($_)/) {
            $cnt++;
        }
    }

    #check servers
    my $verified_servers = 1 if( $cnt == scalar(@servers));

    if($verified_domain && $verified_servers) {
        $Log->debug(sub{"The nis-domain & servers are updated successfully in yp.conf file"});
    } else {
        $Log->exit() if $may_exit;
        NACL::Exceptions::VerifyFailure
            ->throw("Verfication failed while checking the yp.conf file".
                    "after creating the nis-domain configuration");
    }
    $Log->exit() if $may_exit;
}

sub _purge_verify_yp_conf {
    $Log->enter() if $may_enter;
    my ($pkg,@args) = @_;
    my %opts = validate_with(
        params          => \@args,
        spec => {
            systemshell_output => { type => SCALAR,}
        },
    );

    my $output = $opts{systemshell_output};

    if($output !~ /(domain|server)/) {
        $Log->debug(sub{"Nis configurations are successfully deleted from yp.conf file"});
    } else {
        $Log->exit() if $may_exit;
        NACL::Exceptions::VerifyFailure
            ->throw("Verfication failed while checking the yp.conf file".
                    "after deleting the nis-domain configuration");
    }
    $Log->exit() if $may_exit;
}

=head2 parents

    @parents = $net_int_obj->parents();

( Class or Instance method ) This method returns the list containing the names of
the parent objects of this resource. Used for finding dependencies between resources.

=cut

sub parents {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;
    return qw(NACL::STask::Vserver);
}

sub _file_version_show {
    $Log->enter() if $may_enter;
    my ($pkg,@args) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            vserver                            => { type => SCALAR },
            nacltask_verify_file_version_retry => { type => SCALAR, default => 6 },
            nacltask_verify_file_version_interval => { type => SCALAR, default => 10 },
        },
        allow_extra => 1
    );
    my $ci               = $opts{command_interface};
    my $vserver          = $opts{vserver};
    my $max_retry        = $opts{nacltask_verify_file_version_retry};
    my $polling_interval = $opts{nacltask_verify_file_version_interval};
    my $num_of_retries   = 1;
    my $value;
    my $pkg_name = ref $pkg || $pkg;
    if( $pkg_name eq "NACL::STask::VserverServicesNisDomain") {
       $value = 'nis-domain';
    } elsif ( $pkg_name eq "NACL::STask::VserverServicesNameServiceNsSwitch") {
          $value = 'ns-switch';
      }
    my %common_opts;
    $pkg->_copy_common_component_params(
        source => \%opts,
        target => \%common_opts
    );
    if ( $ci->is_feature_supported( feature => 'name_service_file_version' ) ) {
        my %hash_check;
        VERIFY : {
            use warnings;
            while ($num_of_retries <= $max_retry) {
                Tharn::snooze($polling_interval);
                %hash_check = ();
                my @object = NACL::CS::VserverServicesNameServiceFileVersion->fetch(
                    command_interface => $ci,
                    filter => {vserver => $vserver,
                              srctbl => $value},
                    requested_fields  => [qw(file-counter rdb-counter)],
                    %common_opts,
                );
                if( @object) {
                    foreach (@object) {
                        if ( !( $_->file_counter_isset() && $_->rdb_counter_isset() ) ) {
                            $num_of_retries++;
                            $Log->comment('Entries are not set to any value'
                                     . 'Retry for Iteration' . $num_of_retries);
                            no warnings qw(exiting);
                            redo VERIFY;
                        } else {
                              $hash_check{$_->file_counter()} = 1;
                              $hash_check{$_->rdb_counter()} = 1;
                          }
                     }
                     if (keys(%hash_check) == 1) {
                         $Log->comment ('Verification succeeded among all nodes');
                         last VERIFY;
                     } else {
                           $num_of_retries++;
                           $Log->comment('Entries are not matched'
                                     . 'Retry for Iteration' . $num_of_retries);
                           no warnings qw(exiting);
                           redo VERIFY;
                       }
                 }
             }
        }
        if ($num_of_retries > $max_retry) {
               $Log->exit() if $may_exit;
               NACL::Exceptions::VerifyFailure
                    ->throw("Verfication failed after max retries" . $max_retry);
        }
    } else {
        $Log->exit() if $may_exit;
      }
}

sub _wait_replay {
    $Log->enter() if $may_enter;
    my ( $pkg, %args ) = @_;

    my $interval = $args{nacltask_replay_interval};
    my $timeout = $args{nacltask_replay_timeout};

    my %replay_opts;
    $pkg->_copy_common_component_params_with_ci(
        source => \%args,
        target => \%replay_opts
    );

    $pkg->_copy_primary_keys(
        source => \%args,
        target => \%replay_opts,
    );

    $pkg->_hash_copy(
        source => \%args,
        target => \%replay_opts,
        copy   => [qw(domains)]
    );

    my $end_time = NATE::Time::timeout2time($timeout);

    my $done=0;
    while (1) {
        try {
            $pkg->_replay_check(%replay_opts);
            $done = 1;
        } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $exp = shift;
            if ($exp->text() =~ /Failed to resolve/) {
                if (time() > $end_time) {
                    NACL::Exceptions::Timeout->throw(
                       "Timeout in  Vserver Services name service in replaying to secd");
                } else {
                    Tharn::snooze($interval);
                }
            } else {
                $Log->exit() if $may_exit;
                $exp->throw();
            }
        };
        last if $done;
    }
    $Log->exit() if $may_exit;
}

sub _replay_check {
    # stub method
}

1;
