#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::Service::ASUPTrigger::ASUPManager;

use v5.14;
use strict;
use warnings FATAL => 'all';
use Carp::Always;
use Carp qw/confess longmess/;
use Data::Dumper::Concise qw/Dumper/;
use MooseX::Params::Validate qw/validated_list validated_hash/;
use Moose::Util::TypeConstraints qw/enum/;
use DateTime;
use List::Util qw/sum/;
use List::MoreUtils qw/uniq/;
use Sys::Hostname qw/hostname/;
use Sort::Key qw/nkeysort rnkeysort/;
use Net::DNS::Resolver;

use Test::Deep::NoTest qw/cmp_details deep_diag/;

use NACL::C::Node;
use NACL::ResourceRecords;
use SystemTest::ASUPMessage qw/craft_asup_message/;
use Hostrec;
use NATE::Log qw/log_global/;
use NATE::Database::Test qw/testdb_global/;

use NACL::CS::SystemNodeAutosupportHistory;
use NACL::CS::SystemNodeAutosupport;

use Moose::Role;

use constant ASUP_VALIDATION_LANGUISH_TIMEOUT => 900;

use constant ASUP_PUT_PATH  => '/put/AsupPut';
use constant ASUP_POST_PATH => '/asupprod/post/1.0/postAsup';

my $Log = log_global();

has running_asups => (
    is      => 'rw',
    isa     => 'HashRef[HashRef]',
    clearer => 'clear_running_asups',
    traits  => ['Hash'],
    default => sub{{}},
    handles => {
        has_running_asups   => 'count',
        is_asup_running     => 'exists',
        _add_running_asup   => 'set',
        remove_running_asup => 'delete',
        running_asups_list  => 'values',
        get_running_asup    => 'get',
    }
);

has completed_asups => (
    is      => 'rw',
    isa     => 'HashRef[HashRef]',
    clearer => 'clear_completed_asups',
    traits  => ['Hash'],
    default => sub{{}},
    handles => {
        has_completed_asups   => 'count',
        is_asup_completed     => 'exists',
        _add_completed_asup   => 'set',
        completed_asups_list  => 'values',
    }
);

has languishing_asups => (
    is      => 'rw',
    isa     => 'HashRef[HashRef]',
    clearer => 'clear_languishing_asups',
    traits  => ['Hash'],
    default => sub{{}},
    handles => {
        has_languishing_asups   => 'count',
        is_asup_languishing     => 'exists',
        _add_languishing_asup   => 'set',
        languishing_asups_list  => 'values',
    }
);

has attempted_asups => (
    is      => 'rw',
    isa     => 'HashRef[Int]',
    default => sub{{}},
    lazy    => 1,
    clearer => 'clear_attempted_asups',
);

# Keeps track of node => model
has nodes => (
    is          => 'rw',
    isa         => 'HashRef',
    traits      => ['Hash'],
    builder     => '_discover_nodes',
    clearer     => '_clear_nodes',
    predicate   => 'has_nodes',
    lazy        => 1,
    handles     => {
        has_node        => 'exists',
        _add_node       => 'set',
        node_list       => 'keys',
        node_count      => 'count',
        get_node_model  => 'get',
    }
);

has no_nodes_detected => (
    is      => 'rw',
    isa     => 'Bool',
    default => 0,
);

# Allow for the serviceAPI caller to mark some ASUPs with a tag
# This is only used by invoke_CM_asup.thpl
has api_tags => (
    is      => 'rw',
    isa     => 'HashRef',
    traits  => ['Hash'],
    default => sub{{}},
    handles => {
        _add_api_tag   => 'set',
        remove_api_tag => 'delete',
    },
);

has asup_host_v4 => (
    is      => 'rw',
    isa     => 'Str',
    default => 'testbed.netapp.com',
);

has asup_host_v6 => (
    is      => 'rw',
    isa     => 'Maybe[Str]',
    lazy    => 1,
    builder => '_populate_asup_host_v6',
);

has asup_host_v4_ip => (
    is      => 'rw',
    isa     => 'Str',
    lazy    => 1,
    builder => '_populate_asup_host_v4_ip',
);

has asup_put_url_v4 => (
    is      => 'rw',
    isa     => 'Str',
    default => sub { $_[0]->asup_host_v4 . ASUP_PUT_PATH },
);

has asup_put_url_v6 => (
    is      => 'rw',
    isa     => 'Str',
    lazy    => 1,
    default => sub { $_[0]->asup_host_v6 . ASUP_PUT_PATH }
);

has asup_post_url_v4 => (
    is      => 'rw',
    isa     => 'Str',
    default => sub { $_[0]->asup_host_v4 . ASUP_POST_PATH },
);

has asup_post_url_v6 => (
    is      => 'rw',
    isa     => 'Str',
    lazy    => 1,
    default => sub { $_[0]->asup_host_v6 . ASUP_POST_PATH }
);

# The ONTAP max seems to be 30
has asup_retry_count => (
    is      => 'rw',
    isa     => 'Int',
    default => 30,
);

# The ONTAP min seems to be 30s
has asup_retry_interval => (
    is      => 'rw',
    isa     => 'Str',
    default => '30s',
);

has validate_digital_certificate => (
    is      => 'rw',
    isa     => 'Bool',
    default => 1,
);

has use_ipv6 => (
    is      => 'rw',
    isa     => 'Bool',
    default => 0,
);

has asup_type => (
    is      => 'rw',
    isa     => enum(['all', 'test']),
    default => 'all',
);

has conserve_asup_urls => (
    is      => 'rw',
    isa     => 'Bool',
    default => 0,
);

has testcase_name => (
    is  => 'rw',
    isa => 'Str',
);

has enable_testcase_asup_tracking => (
    is      => 'rw',
    isa     => 'Bool',
    default => 1,
);

has enable_asup_connectivity_check => (
    is      => 'rw',
    isa     => 'Bool',
    default => 1,
);

has include_all_nodes_from_cluster => (
    is      => 'rw',
    isa     => 'Bool',
    default => 1,
);

has warn_about_included_nodes => (
    is      => 'rw',
    isa     => 'Bool',
    default => 1,
);

has _node_objs => (
    is       => 'rw',
    isa      => 'HashRef[NACL::C::Node]',
    traits   => ['Hash','DoNotSerialize'],
    predicate=> 'has_node_objs',
    handles  => {
        get_node_obj => 'get',
        add_node_obj => 'set',
    },
);

has _node_checked => (
    is => 'rw',
    isa => 'HashRef[Bool]',
    traits  => ['Hash'],
    handles => {
        was_node_checked  => 'get',
        mark_node_checked => 'set',
    },
);

has last_asup_trigger_epoch => (
    is  => 'rw',
    isa => 'Int',
);

has asup_validation_languish_timeout => (
    is      => 'rw',
    isa     => 'Int',
    default => ASUP_VALIDATION_LANGUISH_TIMEOUT,
);

has asup_trigger_count => (
    is      => 'rw',
    isa     => 'Int',
    default => 0,
);

has node_timeout => (
    is      => 'rw',
    isa     => 'Num',
    default => 30,
);

has skip_vsims => (
    is      => 'rw',
    isa     => 'Bool',
    default => 0,
);

# Only used for unit testing
has trust_hostrec => (
    is      => 'rw',
    isa     => 'Bool',
    default => 0,
);

with 'NACL::Service::ASUPTrigger::Logger';

sub clear_asup_state {
    my ($self) = @_;
    $self->clear_completed_asups;
    $self->clear_running_asups;
    $self->clear_languishing_asups;
    $self->clear_attempted_asups;
    return;
}

my @COMMON_ASUP_INFO = (
    subject                 => { isa => 'Str',      optional => 0 },
    node                    => { isa => 'Str',      optional => 0 },
    type                    => { isa => 'Str',      optional => 0 },
    destination             => { isa => 'Str',      optional => 0 },
    test                    => { isa => 'Int',      optional => 0 },
    start_time              => { isa => 'Int',      optional => 0 },
    testcases               => { isa => 'Maybe[HashRef[Bool]]', optional => 1 },
    api_tags                => { isa => 'HashRef',  optional => 1 },
    last_long_running_syslog=> { isa => 'Int',      optional => 1},
);

my @RUNNING_ASUP_INFO = (
    @COMMON_ASUP_INFO,
    status                  => { isa => 'Str', optional => 1 },
    last_update             => { isa => 'Str', optional => 1 },
    attempt_count           => { isa => 'Str', optional => 1 },
);

my @COMPLETED_ASUP_INFO = (
    @COMMON_ASUP_INFO,
    status          => { isa => 'Str', optional => 0 },
    last_update     => { isa => 'Str', optional => 0 },
    attempt_count   => { isa => 'Str', optional => 0 },
);

sub add_running_asup {
    my ($self, $subject, $node, $type, $destination, $test_id, $start_time, $testcases) = validated_list(\@_,
        @RUNNING_ASUP_INFO
    );
    my %args = (
        subject     => $subject,
        node        => $node,
        type        => $type,
        destination => $destination,
        test        => $test_id,
        start_time  => $start_time,
        testcases   => $testcases,
        api_tags    => $self->api_tags,
    );
    $Log->comment("Adding running ASUP:\n" . Dumper(\%args));
    $self->_add_running_asup( $subject => \%args);
    return;
}

sub update_running_asup {
    my ($self, %asup_info) = validated_hash(\@_, @RUNNING_ASUP_INFO);
    my $subject = $asup_info{subject};

    # TODO Add a log statement showing diff
    my $current_asup_info = $self->get_running_asup($subject);
    my ($ok, $stack) = cmp_details(\%asup_info, $current_asup_info);
    if (!$ok){
        $Log->comment("Updating running ASUP:\n" . deep_diag($stack));
    }

    $self->_add_running_asup($subject => \%asup_info);
    return;
}

sub add_completed_asup {
    my ($self, %args) = validated_hash(\@_, @COMPLETED_ASUP_INFO);
    $Log->comment("Adding completed ASUP:\n" . Dumper(\%args));
    $self->_add_completed_asup($args{subject} => \%args);
    return;
}

sub add_languishing_asup {
    my ($self, %args) = validated_hash(\@_, @RUNNING_ASUP_INFO);
    $Log->comment("Adding languishing ASUP:\n" . Dumper(\%args));
    $self->_add_languishing_asup($args{subject} => \%args);
    return;
}

sub add_attempted_asup {
    my ($self, $node) = @_;
    confess "You must pass a node" if (!$node);
    $self->attempted_asups->{$node} ++;
    return;
}

sub add_api_tag {
    my ($self, $tag) = @_;
    confess "You must pass a tag string" if (!$tag);
    $Log->comment("Adding API tag: $tag");
    $self->_add_api_tag( $tag => 1 );
}

sub has_asup_for_testcase {
    my ($self, $id, $node) = validated_list(\@_,
        testcase => { isa => 'Int', optional => 0 },
        node     => { isa => 'Str', optional => 1 },
    );
    for my $a ( $self->running_asups_list ){
        if ($node){
            return 1 if ($a->{testcases}->{$id} && $a->{node} eq $node);
        }
        else{
            return 1 if ($a->{testcases}->{$id});
        }
    }
    for my $a ( $self->completed_asups_list ){
        if ($node){
            return 1 if ($a->{testcases}->{$id} && $a->{node} eq $node);
        }
        else{
            return 1 if ($a->{testcases}->{$id});
        }
    }
    return 0;
}

sub has_running_asups_for_api_tag {
    my ($self, $tag) = @_;
    confess "You must pass a tag string" if (!$tag);
    my @a = grep { $_->{api_tags}->{$tag} } $self->running_asups_list;
    return 1 if ( scalar @a);
    return 0;
}

sub has_completed_asups_for_api_tag {
    my ($self, $tag) = @_;
    confess "You must pass a tag string" if (!$tag);
    my @a = grep { $_->{api_tags}->{$tag} } $self->completed_asups_list;
    return 1 if ( scalar @a);
    return 0;
}

sub has_running_asups_in_collection {
    my $self = shift;
    my ($being_collected) = grep { $_->{status} && $_->{status} eq 'collection-in-progress' } $self->running_asups_list;
    return 1 if ($being_collected);
    return 0;
}

sub get_running_asups_for_node {
    my ($self, $node) = @_;
    confess "You must pass a node" if (!$node);
    return if (!$self->has_running_asups);
    my @asups = grep { $_->{node} eq $node } $self->running_asups_list;
    return \@asups;
}

sub get_running_asups_for_test {
    my ($self, $test_id) = @_;
    confess "You must pass a test id" if (!$test_id);
    return if (!$self->has_running_asups);
    my @asups = grep { $_->{test} == $test_id } $self->running_asups_list;
    return \@asups;
}

sub get_completed_asups_for_test {
    my ($self, $test_id) = @_;
    confess "You must pass a test id" if (!$test_id);
    return if (!$self->has_completed_asups);
    my @asups = grep { $_->{test} == $test_id } $self->completed_asups_list;
    return \@asups;
}

sub get_successful_asups_count {
    my $self = shift;
    my $successful = scalar(grep { $_->{status} && $_->{status} eq 'sent-successful'} $self->completed_asups_list) // 0;
    return $successful;
}

sub get_successful_asups_count_per_node {
    my $self = shift;
    my %results;
    # Get a list of nodes without triggering node discovery
    if ($self->has_nodes){
        for my $node ( $self->node_list ){
            my $successful = scalar(grep { $_->{node} eq $node && $_->{status} && $_->{status} eq 'sent-successful'} $self->completed_asups_list) // 0;
            $results{$node} = $successful;
        }
    }
    $Log->comment("Successful ASUPS per node:\n" . Dumper(\%results));
    return \%results;
}

sub asup_put_url {
    my ($self) = @_;
    if ($self->use_ipv6){
        return $self->asup_put_url_v6;
    }
    return $self->asup_put_url_v4;
}

sub asup_post_url {
    my ($self) = @_;
    if ($self->use_ipv6){
        return $self->asup_post_url_v6;
    }
    return $self->asup_post_url_v4;
}

sub add_node {
    my ($self, $node, $model) = @_;
    confess("You must specify a node name") if (!$node);
    my $h = eval { Hostrec->new(id=>$node) };
    if ($h){
        $self->_add_node($node => $model);
    }
    else{
        $self->log_error("Unable to create a hostrec for node '$node'. No ASUPs will be triggered from that node");
    }
    return;
}

sub _discover_nodes {
    my ($self) = @_;

    # Discover the nodes without connecting to them
    # Connection will be established on demand later
    $Log->comment("Discovering the nodes...");

    my (%nodes, %errors);

    my $resources = NACL::ResourceRecords->new();
    # The call to nodes already filters out filers for which a hostrec cannot be created
    my $resource_nodes = $resources->nodes;
    if ($resource_nodes){
        for my $r ( @$resource_nodes ){
            if (defined $r->hostrec){
                $nodes{$r->hostrec->hostname} = $r->hostrec->model;
            }
            else{
                $errors{$r->_name} = "" . $r->_error; # stringify
            }
        }
        $Log->comment("Nodes found in testbed: " . Dumper(\%nodes));
    }
    if ( keys %errors ){

        $Log->comment("Errors detected while detecting nodes");

        # Avoid a recursion
        if (!$self->has_site){

            if (! keys %nodes){
                $self->no_nodes_detected(1);
            }

            $Log->comment("Getting SITE info from hostrec");
            my ($resource_with_site) = grep { defined($_->hostrec) && $_->hostrec->{SITE} } @$resource_nodes;
            if ($resource_with_site){
                my $site = $resource_with_site->hostrec->{SITE};
                $Log->comment("Found site '$site' in hostrec files");
                $self->site(lc($site));
            }
        }

        my $err = "Unable to get hostrec for nodes: " . join(',', keys %errors) . ":\n" . join("\n", values %errors);
        $self->log_error($err);

        # $self->report_error_to_syslog(
        #     "Unable to get hostrec for some node(s)",
        #     nodes  => [keys %errors],
        #     errors => \%errors,
        # );
    }

    if (! keys %nodes){
        # This is a sign of a misconfigured run. Perhaps the run is happening
        # outside of a testbed with CLI parameters passed in as test parameters
        # rather than global parameters.

        # See if we can find a NODES parameter listed as a parameter of a normal test
        my @node_names = $self->_find_nodes_as_normal_parameters;
        $Log->comment("Nodes found as parameters:\n" . join(",", @node_names));

        if (!@node_names){

            # The call to report_comment_to_syslog requires a valid "SITE"
            # to have been detected. Detecting the SITE may in turn ask for
            # nodes if a SITE parameter cant be seen. This creates a deep
            # recursion problem that the flag below is meant to solve. We're
            # indicating that no nodes could be found and that will bypass
            # the problematic codepath during site discovery
            $self->no_nodes_detected(1);

            # This may be an error it may be because the script doesn't
            # touch nodes. An example is /x/eng/ctl/test-tools/client_health.thpl
            # run by CTL-PB
            # Generate a syslog message so we can monitor this condition, but
            # no error
            # TODO Don't report anything if its client_health.thpl since it is
            # expected
            $self->report_comment_to_syslog("No node detected");
            # Don't disable triggers, maybe a later test will have nodes
            #$self->enable_asup_triggers(0);
            return {};
        }

        %nodes = map { $_ => undef } @node_names;

    }
    return \%nodes;
}

sub _find_nodes_as_normal_parameters {
    my ($self) = @_;

    # Wait for first normal test to start
    # Dig in NATE DB for a NODES/FILERS/FILER/FILER_LIST/FILERA/FILERB/FILER_SRC/FILER_DST params
    my $db = testdb_global();
    $db->{data}->is_connected()||$db->db_connect();
    my @nodes;

    my $sql = 'SELECT param_key, param_value, subtest_id from parameters WHERE param_key IN (?,?,?,?,?,?,?)';
    my @binds = qw/NODES FILERS FILER FILERA FILERB FILER_SRC FILER_DST/;

    my $params = $db->{data}->db_getall(query => $sql, binds => \@binds);
    $Log->info("Params:\n" . Dumper($params));
    if ($params && @$params){
        my %node_values;
        for my $p ( @$params ){
            if ($p->{param_value}){
                my @splits = map { s/\s+//g; $_ } split(/:|,/, $p->{param_value});
                @node_values{@splits} = (1) x scalar(@splits);
            }
        }
        @nodes = keys %node_values;
        $Log->comment("Nodes found:\n" . Dumper(\@nodes) );

        @nodes = grep { eval{Hostrec->new(id => $_);1} } @nodes;
        $Log->comment("Valid nodes found:\n" . Dumper(\@nodes) );

    }
    return @nodes;
}

sub _populate_asup_host_v6 {
    my ($self) = @_;

    # Create IPv6 URLs
    my $resolver = Net::DNS::Resolver->new();
    my $tries = 3;
    my $resolved = 0;
    my $asup_host_v6 = '';
    TRY: while ( $tries > 0 ){
        $tries --;
        my $query = $resolver->query($self->asup_host_v4, "AAAA");
        if ($query) {
            my ($rr) = grep { $_->type eq 'AAAA' } $query->answer();
            next TRY if (!$rr);
            $asup_host_v6   = '['. $rr->address() . ']';
            $resolved = 1;
            $Log->comment("Using IPv6 host '$asup_host_v6'");
            last TRY;
        }
        else {
            $Log->warn("IPv6 query on [". $self->asup_host_v4 ."] failed: ". $resolver->errorstring());
        }
        sleep 1;
    }
    if ( !$resolved ){
        $self->use_ipv6(0);
        $self->report_comment_to_syslog("Not going to use IPv6 because no IPv6 ASUP URL found");
    }
    return $asup_host_v6;
}

sub _populate_asup_host_v4_ip {
    my ($self) = @_;

    # Create IPv6 URLs
    my $resolver = Net::DNS::Resolver->new();
    my $tries = 3;
    my $resolved = 0;
    my $asup_host_v4_ip = '';
    TRY: while ( $tries > 0 ){
        $tries --;
        my $query = $resolver->query($self->asup_host_v4);
        if ($query) {
            my ($rr) = grep { $_->type eq 'A' } $query->answer();
            next TRY if (!$rr);
            $asup_host_v4_ip   = $rr->address();
            $resolved = 1;
            $Log->comment("IPv4 address for host " . $self->asup_host_v4 . " is $asup_host_v4_ip");
            last TRY;
        }
        else {
            $Log->warn("IPv4 query on '". $self->asup_host_v4 ."' failed: ". $resolver->errorstring());
        }
        sleep 1;
    }
    if ( !$resolved ){
        $self->log_error("Unable to resolve IPv4 address of host " . $self->asup_host_v4);
    }
    return $asup_host_v4_ip;
}

sub _get_node_iterator {
    my $self = shift;

    # Nodes may be added after $self->nodes has been computed
    # Using this iterator ensures that no known node will get skipped
    # during a loop on nodes

    my %nodes_returned;
    my $iterator = sub {
        my $me = shift;
        my $nodes = $me->nodes;
        return if (!$me->node_count);

        my ($next) = sort grep { !$nodes_returned{$_} } keys %$nodes;
        $nodes_returned{$next} = 1 if ($next);
        return $next;
    };
    return $iterator;
}

sub _trigger_asups {
    my ($self, %args) = @_;
    my $asup_type     = $args{asup_type} || $self->asup_type;
    my $no_validation = $args{no_validation_timer};
    my $nodes = $self->nodes;
    if (!$nodes || ! keys %$nodes){
        $Log->comment("No nodes detected, skipping ASUP triggering");
        return;
    }

    $self->asup_trigger_count( $self->asup_trigger_count + 1 );
    my @successful_invoke_nodes;
    my $skipped = 0;

    my $node_iterator = $self->_get_node_iterator;

    while ( my $n = $node_iterator->($self) ){
        if ($self->skip_vsims){
            my $model = $self->get_node_model($n);
            # The model of vsims will be SIMBOX if it was detected on-box, and VSIM if it was
            # detected from the nate host file
            if ( $self->was_node_checked($n) && $model && ( $model eq 'SIMBOX' || $model eq 'VSIM' )){
                $Log->comment("Skipping ASUP invocation since node '$n' is a vsim and skip_vsims is true");
                $skipped ++;
                next;
            }
        }
        $self->add_attempted_asup($n);
        my ($success, $triggered) = $self->_node_wrapper(
            node => $n,
            code => \&_invoke_asup_per_node,
            args => { asup_type => $asup_type },
        );
        push @successful_invoke_nodes, $n if ($success && $triggered);
    }
    $Log->comment("Invoke of asup_type '$asup_type' was successful on nodes: @successful_invoke_nodes");
    if (@successful_invoke_nodes){
        $self->last_asup_trigger_epoch(DateTime->now->epoch);
    }
    $self->_create_asup_validation_timer if ( $self->has_running_asups && !$no_validation);
    if ($skipped == scalar(keys %$nodes)){
        # We skipped all the nodes, stop everything from now
        $Log->comment("All nodes detected were skipped. Disable the ASUP triggers entirely");
        $self->disable_asup_triggers(0);
    }
    return scalar(@successful_invoke_nodes);
}

sub add_asup_attempt_to_all_nodes {
    my ($self) = @_;
    my $nodes = $self->nodes;
    if (!$nodes || ! keys %$nodes){
        return;
    }
    for my $n ( keys %$nodes ){
        $self->add_attempted_asup($n);
    }
    return;
}

my @WRAPPER_CALLBACK_ARGS = (
    command_interface   => { isa => 'NACL::C::CommandInterface' },
    apiset              => { isa => 'NACL::APISet' },
    node                => { isa => 'Str' },
);
sub _node_wrapper {
    my ( $self, $node, $code, $extra_args ) = validated_list(
        \@_,
        node => { isa => 'Str' },
        code => { isa => 'CodeRef' },
        args => { isa => 'HashRef', optional => 1 },
    );

    my ($ci, $apiset) = $self->_get_ci_apiset_for_node($node);
    return if (!$ci);

    my $return_value;
    my $success = eval {
        $return_value = $code->(
            $self,
            command_interface => $ci,
            apiset            => $apiset,
            node              => $node,
            $extra_args?%$extra_args:(),
        );
        1;
    };
    my $err = $@;
    if (!$success){
        # Figure out if node is down
        # Technically we are testing if the session is up, close enough
        # If the session went down it means the node got sick all the same
        if ( $apiset->api_is_session_alive ){
            $self->log_error($err);
        }
        else{
            $Log->comment("Node '$node' appears to be down. Ignoring failure");
        }
    }
    if (wantarray){
        return ($success, $return_value);
    }
    return $success;
}

sub _set_node_model {
    my ($self, $node, $model) = validated_list(
        \@_,
        node  => { isa => 'Str' },
        model => { isa => 'Str' },
    );
    $self->add_node( $node => $model );
    return;
}

sub _get_ci_apiset_for_node {
    my ($self, $node) = @_;
    my $n = $self->get_node_obj($node);
    if (!$n ){
        $n = NACL::C::Node->new( node => $node );
        $self->add_node_obj( $node => $n );
    }

    # Is that the best way to set a timeout on connection?
    # Tried to pass a Connectrec object to the apiset and it seems to work
    # but I couldnt figure out how to get apiset caching to work
    my $p = NATE::ParamSet->new(global => 1);
    $p->set(
        key         => 'NATE_CONNECTREC_TIMEOUT',
        value       => $self->node_timeout,
    );

    my $apiset;
    my $is_alive = eval {
        $apiset = $n->apiset(
            category    => "Node",
            interface   => "CLI",
            set         => "CMode",
        );
        my $a = $apiset->api_is_session_alive;
        # Workaround for burt1056183
        if($a){
            # This whole section of code feels wrong
            # There must be a bug somewhere in NATE/NACL that makes this
            # necessary at times
            $apiset->set('confirmations' => 'off');
            $apiset->set_privilege('privilege-level' => $apiset->get_privilege);
            # set -showallfields true -showseparator "X;X" ; system timeout modify -timeout 0 ; rows 0
            $apiset->set(
                showallfields => 'true',
                showseparator => '"X;X"',
                rows          => 0,
            );
        }
        $a;
    };
    $Log->comment($@) if ($@);
    if (!$apiset){
        $Log->comment("Unable to get connection to node '$node'");
        return;
    }
    if (!$is_alive){
        $is_alive = eval {
            $apiset->api_reconnect_session;
            $apiset->api_is_session_alive;
        };
        $Log->comment($@) if ($@);
        if (!$is_alive){
            $Log->comment("Unable to revive connection to node '$node'");
            return;
        }
    }
    $apiset->set_timeout( timeout => $self->node_timeout);
    # See if the first time we were able to connect to this node
    # if so, ask for the other nodes in its cluster and compare
    # to the ones we know about already
    if ( !$self->was_node_checked($node) ){
        if ($self->include_all_nodes_from_cluster || $self->skip_vsims ){

            # Do a single eval so we can mark the node checked when both succeed
            my $cluster_show;
            my $node_show;
            my $success = eval {
                if ( $self->include_all_nodes_from_cluster){
                    $Log->comment("Asking node '$node' to list other nodes in the cluster");
                    $cluster_show = $apiset->cluster_show();
                }
                if ($self->skip_vsims){
                    my $node_model = $self->get_node_model($node);
                    if (!$node_model || !$self->trust_hostrec){
                        $Log->comment("Gathering model type of node '$node'");
                        $node_show    = $apiset->system_node_show(node => $node);
                    }
                }
                1;
            };
            if (!$success){
                $Log->warning("Unable to gather list of cluster nodes and/or get node model from '$node':$@");
            }
            else{
                if ($self->include_all_nodes_from_cluster){
                    my $rows = $cluster_show->get_parsed_output();
                    my @nodes = map { $_->{node} } @$rows;
                    for my $n (@nodes){
                        if (!$self->has_node($n)){
                            if ($self->warn_about_included_nodes){
                                $Log->warn(
                                    "Node '$n' was found in the cluster but not passed to the test.\n"
                                    ."This script will attempt to send ASUPs from '$n' anyway\n"
                                    ."You may turn off this behavior by passing INCLUDE_ALL_NODES_FROM_CLUSTER=0\n"
                                    ."You may turn off this warning by passing WARN_ABOUT_INCLUDED_NODES=0"
                                );
                            }
                            # We'll figure out the model the first time we connect to this node
                            $self->add_node( $n => undef );
                        }
                    }
                }
                if ($self->skip_vsims && $node_show){
                    my $rows = $node_show->get_parsed_output();
                    my $model = $rows->[0]->{model};
                    if (!$model){
                        $Log->warn("No model info found for node '$node'");
                    }
                    else{
                        $self->_set_node_model(node => $node, model => $model);
                    }
                }

                $self->mark_node_checked($node => 1);
            }
        }
    }
    return ($n, $apiset);
}

sub _invoke_asup_per_node {
    my ($self, $ci, $apiset, $node, $asup_type) = validated_list(
        \@_,
        @WRAPPER_CALLBACK_ARGS,
        asup_type => { isa => 'Str' },
    );

    if ( $self->skip_vsims ){
        my $model = $self->get_node_model($node);
        if ($model && ( $model eq 'SIMBOX' || $model eq 'VSIM')){
            $Log->comment("Skipping ASUP invocation since node '$node' is a vsim and skip_vsims is true");
            return;
        }
    }

    # Make sure ASUP is turned on and functional
    $Log->comment("Configure ASUP on node $node");


    my $original_url;
    my $original_put_url;
    if ($self->conserve_asup_urls){
        my @rows = NACL::CS::SystemNodeAutosupport->fetch(
            command_interface => $ci,
            filter => { node => $node }
        );
        if (!@rows || scalar(@rows) > 1 ){
            confess "Unexpected outpout from autosupport show";
        }
        my $row = $rows[0];
        $original_url     = $row->{url};
        $original_put_url = $row->{put_url};
        $Log->comment(
            "Original ASUP URLs:\n"
            . "POST: $original_url\n"
            . "PUT: $original_put_url"
        );
    }

    # Define the parameters
    my %config = (
        'node'              => $node,
        'state'             => "enable",
        'support'           => "enable",
        'local-collection'  => "true",
        'transport'         => 'https',
        'proxy-url'         => '""',
        'url'               => $self->asup_post_url,
        'put-url'           => $self->asup_put_url,
        'validate-digital-certificate'  => ($self->validate_digital_certificate?"true":"false"),
    );
    if ($self->asup_type eq 'test'){
        $config{throttle} = "false";
        $config{'ondemand-state'} = "disable";
    }
    if ($self->use_ipv6){
        $config{'validate-digital-certificate'} = "false";
    }
    if ($self->asup_retry_count){
        $config{'retry-count'} = $self->asup_retry_count;
    }
    if ($self->asup_retry_interval){
        $config{'retry-interval'} = $self->asup_retry_interval;
    }

    # Execute the modify command
    $apiset->system_node_autosupport_modify(%config);

    if ($self->enable_asup_connectivity_check){

        # Now check the new config is working
        my $asup_ok = $self->_check_asup_connectivity(
            node              => $node,
            apiset            => $apiset,
            command_interface => $ci,
        );

        if (!$asup_ok && $self->use_ipv6){
            $self->use_ipv6(0);
            $apiset->system_node_autosupport_modify(
                url       => $self->asup_post_url,
                'put-url' => $self->asup_put_url,
                node      => $node,
            );
            $asup_ok = $self->_check_asup_connectivity(
                node              => $node,
                apiset            => $apiset,
                command_interface => $ci,
            );
        }

        if ($self->conserve_asup_urls){
            $apiset->system_node_autosupport_modify(
                url       => $original_url,
                'put-url' => $original_put_url,
                node      => $node,
            );
        }

        confess("Failed ASUP connectivity check") if (!$asup_ok);
    }

    $apiset->system_node_autosupport_show(node => $node);

    my @history_rows = NACL::CS::SystemNodeAutosupportHistory->fetch(
        command_interface => $ci,
        allow_empty       => 1,
        filter => {
            node        => $node,
            status      => 'collection-in-progress',
        }
    );
    if (@history_rows){
        $Log->comment("Found foreign ASUP in collection. Aborting ASUP invoke");
        return;
    }

    my $test_id      = $self->current_test_id;
    my $testcase_ids = $self->running_testcase_ids;

    if (! defined $test_id){
        # This may happen if the last test finished while we were inside
        # this method
        $Log->comment("No test running anymore. Aborting ASUP invoke");
        return;
    }

    my %message_args = (
        test_id     => $self->testcase_name,
        subtest_id  => $test_id,
    );
    if ($self->enable_testcase_asup_tracking){
        $message_args{running_testcase_ids} = $testcase_ids;
        if ( $self->has_completed_testcase_backlog_for_node(node => $node) ){
            $message_args{missed_testcase_ids} = $self->completed_testcase_backlog_ids(node => $node);
        }
    }

    my $message = craft_asup_message(%message_args);
    $apiset->system_node_autosupport_invoke(
        node    => $node,
        type    => $asup_type,
        message => '"' . $message . '"',
        $self->conserve_asup_urls?(uri     => 'https://' . $self->asup_put_url):(),
    );

    # The invoke was successful. Maybe the ASUP wont make it and we'll lose the
    # backlog but thats not worse than losing the ASUP.
    $self->clear_completed_testcase_backlog_for_node( node => $node );

    # The messages are unique per asup
    my $asup_subject = $self->_message_to_subject($message, $asup_type);
    $self->add_running_asup(
        subject     => $asup_subject,
        node        => $node,
        type        => $asup_type,
        destination => ($self->conserve_asup_urls?'retransmit':'http'),
        start_time  => int(time()),
        test        => $test_id,
        testcases   => { map { $_ => 1 } @$testcase_ids },
    );
    return 1;
}

sub _check_asup_connectivity {
    my ($self, $ci, $NodeAPI, $node) = validated_list(
        \@_,
        @WRAPPER_CALLBACK_ARGS,
    );

    # The ASUP check doesn't seem to pass reliabily inside the VED at least
    # Try a few times before declaring it bad
    # Perform check types one at a time to increase likelihood of success
    my @check_types = (
        # 'configuration',
        # 'http-post-destination',
        'http-put-destination'
    );

    my $retries = 10;
    my $successful_check;
    for my $check_type ( @check_types ){
        my $check_retries = $retries;
        while ( $check_retries >= 0 ){
            $successful_check = 0;
            my $response;
            eval {
                $response = $NodeAPI->system_node_autosupport_check_show_details(
                    node        => $node,
                    'check-type'=> $check_type,
                );
            };
            my $err = $@;
            if ($err){
                if ($err =~ /There are no entries matching your query/){
                    # Retry in case this unexpected message is seen. Burt1093486
                }
                elsif ($err =~ /Timeout has occcured/){
                    # Retry in case of timeout. Perhaps a transient issue
                }
                else{
                    die $err;
                }
            }
            else{
                my $rows = $response->get_parsed_output();
                # There should be only one row
                if ( scalar(@$rows) != 1 ){
                    confess ("Unexpected output of autosupport check command: " . Dumper($rows));
                }
                my $row = $rows->[0];
                if ( $row->{status} eq 'ok' ){
                    $successful_check = 1;
                }
                elsif ($row->{status} ne 'failed'){
                    $Log->warn("Unknown status '" . $row->{status} . "'. Assuming failed.");
                }
                elsif ($row->{'error-detail'} =~ /Could(n't| not) resolve/ ){
                    # Take care of misconfigured nodes that can't resolve testbed.netapp.com
                    # by switching to IP address
                    if ( $self->asup_host_v4 !~ /^\d/ ){
                        my $ip = $self->asup_host_v4_ip;
                        if ($ip){
                            $self->asup_host_v4($ip);
                            $self->asup_post_url_v4($self->asup_host_v4_ip . ASUP_POST_PATH);
                            $self->asup_put_url_v4($self->asup_host_v4_ip . ASUP_PUT_PATH);
                            $self->validate_digital_certificate(0);
                            $NodeAPI->system_node_autosupport_modify(
                                url       => $self->asup_post_url,
                                'put-url' => $self->asup_put_url,
                                node      => $node,
                                'validate-digital-certificate' => "false",
                            );
                        }
                    }
                }
                elsif ( $row->{'error-detail'} =~ /Peer certificate can\s?not be authenticated/){
                    $self->validate_digital_certificate(0);
                    $NodeAPI->system_node_autosupport_modify(
                        node      => $node,
                        'validate-digital-certificate' => "false",
                    );
                }
                last if ($successful_check);
            }
            sleep(5);
            $check_retries --;
        }
        last if (!$successful_check);
    }
    return $successful_check;
}

sub _validate_asup_transmissions {
    my ($self, $end_time) = validated_list(\@_,
        end_time => { isa => 'Maybe[Num]', optional => 1}
    );
    if (!$self->has_running_asups){
        $Log->comment("No running ASUPs being tracked. No asup transmission validated necessary");
        return;
    }

    # Group asup payloads per node
    my %payloads_per_node;
    while ( my ($subject, $asup_info) = each %{$self->running_asups} ){
        push @{$payloads_per_node{$asup_info->{node}}}, $asup_info;
    }

    while ( my ($node, $asups) = each %payloads_per_node ){
        if ($self->has_asup_trigger_timer){
            # We still need to run the transmission validation or we take the risk
            # to be stuck in a loop if collection--in-progress ASUPs are detected
            my $remaining = $self->timer_time_remaining($self->asup_trigger_timer);
            if ($remaining < 5){
                # Bail if a new test(case) has started so we dont get in the way
                $Log->comment("ASUP trigger timer detected to start in $remaining seconds. Aborting transmission validation");
                last;
            }
        }
        if ( $end_time && time >= $end_time){
            $Log->comment("End time reached. Aborting transmission validation");
            last;
        }
         $self->_node_wrapper(
            node => $node,
            code => \&_validate_asup_transmission_per_node,
            args => { asups => $asups },
        );
    }
}

sub _validate_asup_transmission_per_node {
    my ($self, $ci, $apiset, $node, $asups) = validated_list(
        \@_,
        @WRAPPER_CALLBACK_ARGS,
        asups => { isa => 'ArrayRef[HashRef]'},
    );

    my @history_rows;
    my $success = eval {
        # Get the history for the most recently invoked ASUP.
        @history_rows = NACL::CS::SystemNodeAutosupportHistory->fetch(
            command_interface => $ci,
            allow_empty       => 1,
            filter => {
                node        => $node,
                destination => ($self->conserve_asup_urls?'retransmit':'http'),
                trigger     => 'callhome.invoke|callhome.invoke.all',
            }
        );
        1;
    };
    my $err = $@;
    if (!$success){
        if ($err =~ /Timeout/i || $err =~ /took longer than \d+ seconds/i){
            # Better luck next time
            return;
        }
        else{
            # Let the node wrapper deal with it
            die $err;
        }
    };

    if (@history_rows){

        # Find the history related to the messages we care about
        my %asup_per_subject = map { $_->{subject} => $_ } @$asups;
        my @relevant_history = grep { $asup_per_subject{$_->{subject}}  } @history_rows;

        if (!@relevant_history){
            $Log->comment(
                "No relevant history found\n".
                "Was Looking for:\n- " . join("\n- ", keys %asup_per_subject)
            );
        }
        else {

            # TODO: Cache this
            my @rows = NACL::CS::SystemNodeAutosupport->fetch(
                command_interface => $ci,
                filter => { node => $node }
            );
            if (!@rows || scalar(@rows) > 1 ){
                confess "Unexpected outpout from autosupport show";
            }
            my $retry_count = $rows[0]->{retry_count};
            $Log->comment("Autosupport retry count for $node: $retry_count");
            $retry_count //= 15;

            my %seen_asups;
            for my $history (@relevant_history){
                my $subject         = $history->{subject};
                my $status          = $history->{status};
                my $last_update     = $history->{'last_update'};
                my $attempt_count   = $history->{'attempt_count'};

                # The same subject may show up multiple times
                # This happens with retries that generate new sequence numbers
                # Only process the most recent try, ie the first one
                next if ($seen_asups{$subject});
                $seen_asups{$subject} = 1;

                my $asup_info = {
                    %{$asup_per_subject{$subject}},
                    node            => $node,
                    status          => $status,
                    last_update     => $last_update,
                    attempt_count   => $attempt_count,
                };

                $self->update_running_asup(%$asup_info);

                if ($status eq 'sent-successful'){
                    $self->add_completed_asup( %$asup_info );
                    $self->remove_running_asup( $subject );
                    next;
                }

                if ($attempt_count && $attempt_count ne '-' && $attempt_count >= $retry_count && $status =~ /failed/){
                    $self->log_error(
                        "ONTAP was unable to transmit the ASUP payload after many tries.\n"
                        ."Please do the following:\n"
                        ."Send an email to ng-Ace-agility, CC to ng-ste-halltalk, asking them\n"
                        ."to check the status of the eprod ASUP http server. Include logs.\n"
                    );
                    $self->add_completed_asup( %$asup_info );
                    $self->remove_running_asup( $subject );
                    next;
                }
                # Retransmit ASUPs don't have retries
                if ( $asup_info->{destination} eq 'retransmit' && $status =~ /failed/ ) {
                    $self->log_error(
                        "ONTAP was unable to transmit the ASUP payload\n"
                        ."Please send me an email at jroussin\@netapp.com with a path to the logs"
                    );
                    $self->add_completed_asup( %$asup_info );
                    $self->remove_running_asup( $subject );
                    next;
                }
                if ($status eq 'ignore'){
                    $Log->comment(
                        "ONTAP has given up on sending ASUP with subject '$subject'\n"
                        ."This happens when another script turns off autosupport momentarily. It appears\n"
                        ."a TO/GB script somewhere does just that before takeover and giveback"
                    );
                    $self->add_completed_asup( %$asup_info );
                    $self->remove_running_asup( $subject );
                    next;
                }

                if ($status =~ /queued/ && $self->asup_type eq 'test'){
                    # TODO For troubleshooting only
                    NACL::CS::SystemNodeAutosupportHistory->fetch(
                        command_interface => $ci,
                        allow_empty       => 1,
                        filter => {
                            node        => $node,
                            status      => 'transmitting',
                        }
                    );
                }
            }
        }
    }

    my $runnings_asups_for_node = $self->get_running_asups_for_node($node);
    if ($runnings_asups_for_node && @$runnings_asups_for_node){
        my $show_history = 0;

        my %subjects_seen_in_history = map { $_->{subject} => 1 } @history_rows;

        # Check for ASUPs we havent seen in the history for a while
        for my $asup_info ( @$runnings_asups_for_node ){

            $Log->comment("Known running ASUP:\n" . Dumper($asup_info));
            my $status  = $asup_info->{status};
            my $subject = $asup_info->{subject};
            my $asup_runtime = time - $asup_info->{start_time};


            # Make sure our ASUP is still in the history
            if ($status && !$subjects_seen_in_history{$subject}){
                # The ASUP used to be in the history, thats why we have a status
                # but it no longer is in the history. This means we'll never get
                # updates again. I've seen this happen. Send a message to graylog
                # and stop looking for that ASUP
                $self->log_error(
                    "The ASUP payload with subject '$subject' is no longer present in the"
                    ." autosupport history of node $node. This is highly unusual"
                );
                $self->add_languishing_asup(%$asup_info);
                $self->remove_running_asup( $subject );
                $show_history = 1;
                next;
            }

            if (!$status && $asup_runtime >= $self->asup_validation_languish_timeout ) {
                # This may happen if the node was doing a takeover, in which case it is busy
                # generating a bunch of other ASUPs. It is likely the ASUP invoked here will
                # eventually run but later.
                # I have also seen ONTAP skip invoked ASUPs entirely
                $self->log_error(
                    "The ASUP payload with subject '$subject' has not been seen in the"
                    ." autosupport history of node $node after ". $self->asup_validation_languish_timeout. " seconds.\n"
                    .'Contact Jerome Roussin (jroussin@netapp.com) immediately for further diagnostic'
                );
                $self->add_languishing_asup(%$asup_info);
                $self->remove_running_asup( $subject );
                $show_history = 1;
                next;
            }

            if ($asup_runtime >= $self->asup_periodic_interval_max){
                # This ASUP has been running for a very long time
                # Don't tell the user but send a message to graylog

                my $last_syslog = $asup_info->{last_long_running_syslog};
                if (!$last_syslog || (time - $last_syslog >= 300) ) {
                    $self->report_comment_to_syslog(
                        'Long running ASUP',
                        asup_subject        => $subject,
                        asup_runtime_so_far => int($asup_runtime),
                        asup_status         => $status,
                    );
                    $asup_info->{last_long_running_syslog} = time;
                }
                else{
                    $Log->comment("Suppressing syslog message about long running ASUP: '$subject'");
                }
            }
        }
        if ($show_history){
            NACL::CS::SystemNodeAutosupportHistory->fetch(
                command_interface => $ci,
                allow_empty       => 1,
                filter => { node  => $node }
            );
        }
    }
    return;
}

sub _message_to_subject {
    my ($self, $message, $asup_type) = @_;
    confess "Missing asup_type argument" if (!$asup_type);
    return '"USER_TRIGGERED ('.uc($asup_type).":". $message . ')"';
}

sub get_asup_activity_report {
    my ($self) = @_;

    my $report = '**********' x 3 . '  Activity Summary  ' . '**********' x 3 . "\n\n";

    my $runtime = '';
    if ($self->first_normal_test_started_time){
        $runtime = int(time - $self->first_normal_test_started_time);
        $report .= '    Runtime since first normal test: ' . $runtime  . " seconds \n\n";
    }

    my $total_attempted  = sum(values %{$self->attempted_asups}) // 0;
    my $total_successful = scalar(grep { $_->{status} eq 'sent-successful' } $self->completed_asups_list) // 0;
    my $total_successful_pct = '-';
    if ($total_attempted){
        $total_successful_pct = int($total_successful * 100 / $total_attempted);
    }

    $report .='    Overall success %   : ' . $total_successful_pct . "\n";
    $report .='        ASUPs attempted : ' . $total_attempted."\n";
    $report .='        ASUPs sent      : ' . $total_successful."\n";
    $report .= "\n";

    while ( my ($node, $node_attempts) = each %{$self->attempted_asups}){
            my $node_successful = scalar(grep { $_->{status} eq 'sent-successful' && $_->{node} eq $node } $self->completed_asups_list)//0;
            my $node_successful_pct = '-';
            if ($node_attempts){
                $node_successful_pct = int($node_successful*100/$node_attempts);
            }
            $report .= "    Success % for node '$node': $node_successful_pct\n";
            $report .= "        ASUPs attempted       : $node_attempts\n";
            $report .= "        ASUPs sent            : $node_successful\n";
    }
    $report .= "\n";

    # Create report per normal test and testcase
    for my $test ( $self->running_test, $self->completed_tests_list ){
        next if (!$test);

        # TODO: Organize ASUPs per testcases too inside each normal test
        #my $testcases =
        my @asups;
        my $running_asups   = $self->get_running_asups_for_test($test->{id});
        push @asups, @$running_asups if ($running_asups && @$running_asups);
        my $completed_asups = $self->get_completed_asups_for_test($test->{id});
        push @asups, @$completed_asups if ($completed_asups && @$completed_asups);

        $report .= "    Last 20 ASUPs for test '" . $test->{name} . "' (id:" . $test->{id} . ")\n";

        my $max_asups_per_test = 20;

        @asups = nkeysort { $_->{start_time} } @asups;
        if (scalar(@asups) > $max_asups_per_test){
            my @truncated = splice @asups,-$max_asups_per_test;
            $report .= "      (Truncated " . scalar(@asups) . " older ASUPs)\n";
            @asups = @truncated;
        }

        for my $asup ( @asups ){
            $report .= "        - ".$asup->{subject}."\n".
                       "              node:   " . $asup->{node} ."\n".
                       "              type:   " . $asup->{type} ."\n".
                       "              status: " . ($asup->{status}//'')."\n";
        }
    }
    $report .= '**********' x 8;
    my $ret = {
        as_string                       => $report,
        total_asup_attempted            => $total_attempted,
        total_asup_sent                 => $total_successful,
        runtime_since_first_normal_test => $runtime,
    };
    if ($total_successful_pct ne '-'){
        $ret->{total_asup_success_pct} = $total_successful_pct;
    }
    return $ret;
}

sub _wait_for_asup_transmission_validation {
    my ($self, $end_time, $interval) = validated_list(\@_,
        end_time => { isa => 'Maybe[Num]', optional => 0},
        interval => { isa => 'Num', default => 10 },
    );

    while ( $self->has_running_asups ){
        last if ( time >= $end_time );
        $self->_validate_asup_transmissions(
            end_time => $end_time
        );
        sleep $interval;
    }
    return;
}

#
# Test only below
#
sub wait_for_asups_collected {
    my ($self) = @_;

    # This API call is dangerous because it can run anytime and makes
    # calls to ONTAP. These calls may disrupt other calls currently in
    # flight performed by timers. In order to avoid this, make sure no
    # timers are running. Note that all timers should be disabled too
    # otherwise the issue may occur still. Its on the caller to have
    # disabled all timers before running this subroutine
    my $running_timers = $self->has_timer_running;
    if ($running_timers){
        $Log->comment("Timer(s) detected during wait_for_asups_collected: $running_timers. Waiting 5s");
        $self->request_postpone( delay => 5);
        return;
    }
    else{
        $Log->comment("No running timer detected");
    }

    $Log->comment("Starting the wait for asups collected");
    $Log->comment(longmess("Current stack"));

    my $nodes = $self->nodes;
    if (!$nodes || ! keys %$nodes){
        $Log->comment("No nodes detected");
        return;
    }
    for my $n ( keys %$nodes ){
        $Log->comment("Starting the wait for asups collected on node $n");
        my $success = $self->_node_wrapper(
            node => $n,
            code => \&wait_for_node_asups_collected,
        );
    }

    my $response = $self->get_errors;
    return $response;
}

sub wait_for_node_asups_collected {
    my ($self, $ci, $apiset, $node, $asups) = validated_list(
        \@_,
        @WRAPPER_CALLBACK_ARGS,
    );

    my $complete = 0;
    while (!$complete){
        $complete = 1;
        my @history_rows = NACL::CS::SystemNodeAutosupportHistory->fetch(
            command_interface => $ci,
            allow_empty       => 1,
            filter => {
                node        => $node,
                destination => ($self->conserve_asup_urls?'retransmit':'http'),
                trigger     => 'callhome.invoke|callhome.invoke.all',
            }
        );
        for my $row ( @history_rows ){
            if (!$row->{status}){
                $Log->error("No status found in row:\n" . Dumper($row));
                next;
            }
            if ( $row->{status} eq 'queued' || $row->{status} eq 're-queued'
                || $row->{status} eq 'initializing'
                || $row->{status} eq 'collection-in-progress'
                || $row->{status} eq 'transmitting'
                ){
                $complete = 0;
                sleep 5;
                last;
            }
        }
    }
    return;
}

1;
