#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::Service::ASUPTrigger::Orchestrator;

# use parent qw(NATE::Service::Worker);
use v5.14;
use strict;
use warnings FATAL => 'all';
use Carp qw/confess longmess/;
use Data::Dumper::Concise qw/Dumper/;
use MooseX::Params::Validate qw/validated_list validated_hash/;
use EV;
use Scalar::Util qw(weaken);
use List::Util qw/min max/;
use DateTime;

use NATE::Log qw(log_global);
use NATE::Events qw(timer_add timer_remove work_add);

use Moose::Role;

# ASUPs will be triggered every 30 minutes regardless
# of any test start/end activity.
use constant ASUP_PERIODIC_INTERVAL_MAX             => 1800;
# But do a ramp up to 30 minutes, starting ith 5min
# The idea is to try and have more than one ASUP per test(case)
use constant ASUP_PERIODIC_INTERVAL_MIN             => 180;
# Ramp up will double interval value until 30min is reached
use constant ASUP_PERIODIC_INTERVAL_INCREASE_RATIO  => 2;
# Whatever happens, don't trigger ASUPs more than one every 5 min
# ONTAP has limit of 19 per hour by default.
use constant ASUP_TRIGGER_INTERVAL_MIN              => 180;
# We are going to wait 5s between being told a test(case) started
# and triggering ASUPs. This allows for rapidly firing test(cases)
# to be combined in one ASUP payload
use constant ASUP_TRIGGER_DELAY                     => 5;
# Check on the status of triggered ASUPs every 10s
use constant ASUP_VALIDATION_INTERVAL               => 10;

# This value will start at ASUP_PERIODIC_MIN and increase to
# ASUP_PERIODIC_MAX
has asup_periodic_interval => (
    is      => 'rw',
    isa     => 'Int',
    lazy    => 1,
    default => sub{{ $_[0]->asup_periodic_interval_min }},
);

#
# Allow serviceAPI callers to override the values defined as constant
# above. This is done either for testing, or for invoke_CM_asup.thpl
#

has asup_periodic_interval_min => (
    is      => 'rw',
    isa     => 'Int',
    default => ASUP_PERIODIC_INTERVAL_MIN,
);

has asup_periodic_interval_max => (
    is      => 'rw',
    isa     => 'Int',
    default => ASUP_PERIODIC_INTERVAL_MAX,
);

has asup_trigger_delay => (
    is      => 'rw',
    isa     => 'Int',
    default => ASUP_TRIGGER_DELAY,
);

has asup_validation_interval => (
    is      => 'rw',
    isa     => 'Int',
    default => ASUP_VALIDATION_INTERVAL,
);

has asup_trigger_interval_min => (
    is      => 'rw',
    isa     => 'Int',
    default => ASUP_TRIGGER_INTERVAL_MIN,
);

#
# Keep track of timers currently set
#

has asup_trigger_timer => (
    is          => 'rw',
    isa         => 'HashRef',
    predicate   => 'has_asup_trigger_timer',
    clearer     => 'clear_asup_trigger_timer',
);

has asup_periodic_timer => (
    is          => 'rw',
    isa         => 'HashRef',
    predicate   => 'has_asup_periodic_timer',
    clearer     => 'clear_asup_periodic_timer',
);

has asup_validation_timer => (
    is          => 'rw',
    isa         => 'HashRef',
    predicate   => 'has_asup_validation_timer',
    clearer     => 'clear_asup_validation_timer',
);

has asup_trigger_timer_running => (
    is      => 'rw',
    isa     => 'Bool',
    default => 0,
);

has asup_periodic_timer_running => (
    is      => 'rw',
    isa     => 'Bool',
    default => 0,
);

has asup_validation_timer_running => (
    is      => 'rw',
    isa     => 'Bool',
    default => 0,
);

# Allows to turn all ASUP triggering off
# This is done when no nodes are detected and possibly
# other reasons later (like running in CITs and such)
has enable_asup_triggers => (
    is      => 'rw',
    isa     => 'Bool',
    default => 1,
);

sub disable_asup_triggers {
    my ($self, $report_to_syslog)  = @_;
    $report_to_syslog //= 1;
    $self->enable_asup_triggers(0);
    $self->remove_all_timers();
    $self->report_comment_to_syslog("ASUP triggers being disabled")
        if ($report_to_syslog);
    return;
}

sub _create_asup_trigger_timer {
    my ($self, %args)           = @_;
    my $asup_type               = $args{asup_type} || $self->asup_type;
    my $asup_trigger_delay_min  = $args{asup_trigger_delay_min};
    confess "Unsupported asup_type" if ($asup_type !~ /test|all/);

    return if ( !$self->enable_asup_triggers );

    if ( $self->has_asup_trigger_timer ){
        my $remaining = $self->timer_time_remaining($self->asup_trigger_timer);
        log_global()->comment("Trigger timer already in place, to be started in $remaining seconds");
        return;
    }
    my $next_time           = $self->asup_trigger_delay;
    my $last_asup_trigger   = $self->last_asup_trigger_epoch;
    my $now_epoch           = DateTime->now->epoch;
    # Make sure we arent below the minimum interval for asup triggers
    # We want to wait at least asup_trigger_delay, but perhaps more if we are
    # below ASUP_TRIGGER_INTERVAL_MIN
    if ($last_asup_trigger){
        my $last_asup_trigger_ago = $now_epoch - $last_asup_trigger;
        if ( $last_asup_trigger_ago  + $self->asup_trigger_delay <  $self->asup_trigger_interval_min ){
            $next_time = max(
                $self->asup_trigger_interval_min - $last_asup_trigger_ago,
                $self->asup_trigger_delay
            );
        }
    }

    if ($asup_trigger_delay_min && $next_time < $asup_trigger_delay_min){
        $next_time = $asup_trigger_delay_min;
    }

    log_global()->comment("Creating a trigger timer for a '$asup_type' ASUP with delay of $next_time seconds");
    log_global()->comment(longmess("Current stack"));

    # We'll install a new one
    $self->_remove_asup_periodic_timer;

    $self->asup_trigger_timer({
        callback  => \&_fire_asup_trigger_timer,
        next_time => $next_time,
        self      => $self,
        asup_type => $asup_type,
    });
    weaken($self->asup_trigger_timer->{self});
    timer_add($self->asup_trigger_timer);

    # TODO remove
    log_global()->comment("Trigger timer to fire in " . $self->timer_time_remaining($self->asup_trigger_timer) . ' seconds');

    return;
}

# Some magic that ensures that timers don't nest within each other.
# API calls are allowed to nest however even though we dont want them to,
# but that's the only way to let API calls happen during timers.
sub serialized_classes {
    # ['service_api_callback'];
    [__PACKAGE__ . '::timer'];
}

#
# Timers, when they fire, don't execute immediately, but are instead put in
# a queue with NATE::Events::work_add. This prevents timers from nesting
# (i.e. running within each other).
# Nesting complicates the logic and has no practical advantages
#

sub _fire_asup_trigger_timer {
    my $watcher = shift;
    my $timer = $watcher->data();
    my $s = $timer->{self};
    $s->asup_trigger_timer_running(1);
    work_add(
        item    => $watcher,
        listener=> {
            callback => sub {
                my $w         = shift;
                my $t         = $w->data();
                my $self      = $t->{self};
                my $asup_type = $t->{asup_type};
                my $success   = eval {
                    log_global()->comment("Firing ASUP trigger timer");
                    $self->_remove_asup_trigger_timer();
                    if ($self->has_running_asups_in_collection){
                        # Triggering another ASUP here would put it in a queue only visible via a special test command
                        # Should the asup being collected stay in that state for a while, a new ASUP here would be
                        # detected as "languishing" and generate an error. Not only that but we have no guarantee
                        # that the ASUP collection would occur during the current normal test. I think it's best
                        # to just postpone the trigger timer and try again later when hopefully no ASUPs is being
                        # collected
                        log_global()->comment("Running ASUPs in collection detected. Skipping ASUP triggering");
                        log_global()->comment("Running ASUPs:\n" . Dumper($self->running_asups));
                        $self->add_asup_attempt_to_all_nodes;

                        # Try again later. Make sure the validation timer fires between now
                        # and the next time the trigger timer fires
                        log_global()->comment("Reinstalling the trigger timer for a later attempt");
                        $self->_create_asup_trigger_timer(
                            asup_type              => $asup_type,
                            asup_trigger_delay_min => $self->asup_validation_interval * 3,
                        );
                    }
                    else{
                        $self->_trigger_asups( asup_type => $asup_type );
                    }
                    1;
                };
                my $err = $@;
                if (!$success){
                    $self->log_error($err);
                }
                # ASUPs were triggered because of new test(s) starting
                # Reset the periodic interval back to 3 min
                # If a trigger timer exists, it means we didnt actually trigger ASUPs
                if ( !$self->has_asup_trigger_timer && $self->has_running_test ){
                    $self->asup_periodic_interval($self->asup_periodic_interval_min);
                    $self->_create_asup_periodic_timer();
                }
                $self->asup_trigger_timer_running(0);
            },
            class => __PACKAGE__ . '::timer'
        }
    );
    return;
}

sub _create_asup_periodic_timer {
    my $self = shift;
    return if ( !$self->enable_asup_triggers );
    if ( $self->has_asup_periodic_timer ){
        my $remaining = $self->timer_time_remaining($self->asup_periodic_timer);
        log_global()->comment("Periodic timer already in place starting in $remaining seconds");
        return;
    }
    # The trigger timer will install us again
    if ( $self->has_asup_trigger_timer ){
        log_global()->comment("Skipping creation of periodic timer since trigger timer is present");
        return;
    }

    if (!$self->has_nodes || !$self->node_count){
        log_global()->comment("Skipping creation of periodic timer since no nodes are present");
        return;
    }

    log_global()->comment("Creating the periodic timer to run in " . $self->asup_periodic_interval . " seconds");
    log_global()->comment(longmess("Current stack"));
    $self->asup_periodic_timer({
        callback  => \&_fire_asup_periodic_timer,
        next_time => $self->asup_periodic_interval,
        self      => $self,
    });
    weaken($self->asup_periodic_timer->{self});
    timer_add($self->asup_periodic_timer);

    #
    # Increase the periodic interval from _MIN to _MAX in steps
    #
    my $next_trigger_interval = min(
        $self->asup_periodic_interval_max,
        $self->asup_periodic_interval * ASUP_PERIODIC_INTERVAL_INCREASE_RATIO
    );
    $self->asup_periodic_interval($next_trigger_interval);

    return;
}

sub _fire_asup_periodic_timer {
    my $watcher = shift;
    my $timer = $watcher->data();
    my $s = $timer->{self};
    $s->asup_periodic_timer_running(1);
    work_add(
        item => $watcher,
        listener => {
            callback => sub {
                my $w = shift;
                my $t = $w->data();
                my $self = $t->{self};
                my $success = eval {
                    log_global()->comment("Firing ASUP periodic timer");
                    $self->_remove_asup_periodic_timer();
                    if ($self->has_running_asups_in_collection){
                        # This takes care of a situation where a previous ASUP is
                        # taking very long to be collected. Piling on more ASUPs
                        # would only lead to more errors. IT's also possible
                        # the node is sick somehow. Count this as an ASUP attempt
                        log_global()->comment("Running ASUPs in collection detected. Skipping ASUP triggering");
                        log_global()->comment("Running ASUPs:\n" . Dumper($self->running_asups));
                        $self->add_asup_attempt_to_all_nodes;
                    }
                    else{
                        $self->_trigger_asups();
                    }
                    1;
                };
                my $err = $@;
                if (!$success){
                    $self->log_error($err);
                }
                if ($self->has_running_test){
                    $self->_create_asup_periodic_timer();
                }
                $self->asup_periodic_timer_running(0);
            },
            class => __PACKAGE__ . '::timer',
        }
    );
    return;
}

sub _remove_asup_periodic_timer {
    my $self = shift;
    if ($self->has_asup_periodic_timer) {
        # remove and destroy the keep alive timer
        log_global()->comment("Removing periodic timer");
        timer_remove($self->asup_periodic_timer);
        $self->clear_asup_periodic_timer;
    }
    return;
}

sub _create_asup_validation_timer {
    my $self = shift;
    return if ( !$self->enable_asup_triggers );
    if ($self->has_asup_validation_timer){
        log_global()->comment("Validation timer already in place");
        return;
    }
    # We dont want to interfere with the trigger timer
    if ($self->has_asup_trigger_timer){
        my $remaining = $self->timer_time_remaining($self->asup_trigger_timer);
        if ($remaining < 5){
            log_global()->comment("Skipping creation of validation timer since a trigger timer is present to be started in $remaining seconds");
            return;
        }
    }

    log_global()->comment("Creating the ASUP validation timer to run in " . $self->asup_validation_interval ." seconds");
    log_global()->comment(longmess("Current stack"));
    $self->asup_validation_timer({
        callback  => \&_fire_asup_validation_timer,
        next_time => $self->asup_validation_interval,
        self      => $self,
    });
    weaken($self->asup_validation_timer->{self});
    timer_add($self->asup_validation_timer);

    return;
}

sub _fire_asup_validation_timer {
    my $watcher = shift;
    my $timer = $watcher->data();
    my $s = $timer->{self};
    $s->asup_validation_timer_running(1);
    work_add(
        item => $watcher,
        listener => {
            callback => sub {
                my $w = shift;
                my $t = $w->data();
                my $self = $t->{self};
                my $success = eval {
                    log_global()->comment("Firing ASUP validation timer");
                    $self->_remove_asup_validation_timer();
                    $self->_validate_asup_transmissions;
                    1;
                };
                my $err = $@;
                if (!$success){
                    $self->log_error($err);
                }
                # If some ASUPs were not completed yet, schedule to run
                # ourself again later
                if ($self->has_running_asups){
                    $self->_create_asup_validation_timer;
                }
                $self->asup_validation_timer_running(0);
            },
            class => __PACKAGE__ . '::timer',
        }
    );
    return;
}

sub _remove_asup_trigger_timer {
    my $self = shift;

    # cleanup the buffer flush timer
    # if one exists
    if ($self->has_asup_trigger_timer) {
        log_global()->comment("Removing ASUP trigger timer");
        timer_remove($self->asup_trigger_timer);
        $self->clear_asup_trigger_timer;
    }
    return;
}

sub _remove_asup_validation_timer {
    my $self = shift;

    # cleanup the buffer flush timer
    # if one exists
    if ($self->has_asup_validation_timer) {
        log_global()->comment("Removing ASUP validation timer");
        timer_remove($self->asup_validation_timer);
        $self->clear_asup_validation_timer;
    }
    return;
}

sub remove_all_timers {
    my ($self) = @_;
    $self->_remove_asup_trigger_timer;
    $self->_remove_asup_periodic_timer;
    $self->_remove_asup_validation_timer;
}

sub timer_time_remaining {
    my ($self, $timer) = @_;
    return if (!$timer);
    my $watcher = $NATE::Events::Timers{$timer->{timer_id}};
    return if (!$watcher);
    return $watcher->remaining;
}

sub has_timer_running {
    my $self = shift;

    my @running_timers;
    if ($self->asup_validation_timer_running){
        push @running_timers, 'validation_timer';
    }
    if ($self->asup_trigger_timer_running){
        push @running_timers, 'trigger_timer';
    }
    if ($self->asup_periodic_timer_running){
        push @running_timers, 'periodic_timer';
    }

    return join(',', @running_timers) if (@running_timers);
    return 0;
}

1;
