#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::Service::ASUPTrigger::Worker;

use v5.14;
use strict;
use warnings FATAL => 'all';
use Carp qw/confess/;
use Data::Dumper::Concise qw/Dumper/;
use MooseX::Params::Validate qw/validated_list/;
use List::Util qw/min max/;
use EV;

use NATE::Log qw/log_global/;
use NATE::Database::Test qw/testdb_global/;
use NATE::ParamSet qw//;
use NATE::ServiceAPI::KeyVal qw/keyval_global/;

use NACL::OnDemandLoading;
BEGIN {
    NACL::OnDemandLoading->_process;
}

use Moose;
use MooseX::NonMoose;
use MooseX::Storage;
use namespace::autoclean;

extends 'NATE::Service::Worker';

# Load most of the logic via roles
# This is mostly to keep the size of any one file
# small(ish)
with 'NACL::Service::ASUPTrigger::Orchestrator'
    ,'NACL::Service::ASUPTrigger::Logger'
    ,'NACL::Service::ASUPTrigger::TestWatcher'
    ,'NACL::Service::ASUPTrigger::ASUPManager',
    Storage(format => 'JSON', traits =>['OnlyWhenBuilt']);

my @TESTCASE_API_ARGS_VALIDATION_SPEC = (
    testcase_id  => { isa => 'Int', optional => 0 },
    subtest_runid=> { isa => 'Str', optional => 0 },
    subtest_id   => { isa => 'Int', optional => 0 },
);

# I am told "use Tharn qw/$Log/"" is "bad"
my $Log = log_global();

has startup_time => (
    is  => 'rw',
    isa => 'Num',
);

has first_normal_test_started_time => (
    is  => 'rw',
    isa => 'Num',
);

has last_validation_min_test_runtime => (
    is      => 'rw',
    isa     => 'Num',
    default => 10,
);

has last_validation_min_duration => (
    is      => 'rw',
    isa     => 'Num',
    default => 30,
);

has last_validation_max_duration => (
    is      => 'rw',
    isa     => 'Num',
    default => 600,
);

sub startup {
    my ($self) = @_;
    $self->startup_time(time);

    $Log->comment("Perl exec: $^X");
    my $service_worker_file = $INC{'NACL/Service/ASUPTrigger/Worker.pm'} // '';
    $Log->comment("Worker file: $service_worker_file");

    my $paramset = NATE::ParamSet->new(global=>1);
    # Turn off useless and harmful code
    $paramset->set(
        key   => 'CONFIG_DB',
        value => 0,
    );

    # Workaround for testdb_global() not working. Burt1081240
    # TODO: I am not convinced the ->get calls work here
    #       It's likely @NATE::ParamSet::Param_Tree needs to be populated first
    if(!$NATE::Database::Test::Global) {
        my $driver = $paramset->get('NATE_DB_DRIVER');
        my $name   = $paramset->get('NATE_DB_NAME');
        my $dbdir  = $paramset->get('NATE_DB_DIR_TEMP') || $paramset->get('NATE_DB_DIR');
        my $user   = $paramset->get('NATE_DB_USER')     || '';
        my $auth   = $paramset->get('NATE_DB_PASSWORD') || '';
        my $host   = $paramset->get('NATE_DB_HOST')     || '';
        my $port   = $paramset->get('NATE_DB_PORT')     || '';
        my $trace  = $paramset->get('NATE_DB_TRACE')    || 0;
        my $db = NATE::Database::Test->new(
            driver   => $driver,
            database => $name,
            username => $user,
            password => $auth,
            data_dir => $dbdir,
            host     => $host,
            global   => 1,
            port     => $port,
            trace    => $trace
        );
        # See if the DB works:
        $Log->comment("Checking we can connect to DB");
        $db->get_device_usage;
    }

    $self->_init_from_parameters();

    my $syslog_server = $self->syslog_server;
    if ($syslog_server){
        if ($self->enable_syslog){
            # Send some timing too if its available
            # It may not available when the NATE stack lags the NACL stack
            my %additional_payload;
            if ($self->can('compile_time')){
                $additional_payload{compile_time}  = $self->compile_time;
                $additional_payload{instance_time} = $self->instance_time;
                $additional_payload{init_time}     = $self->init_time;
            }
            $self->report_comment_to_syslog(__PACKAGE__ . " started", %additional_payload);
        }
    }
    else{
        $self->log_error("Unable to locate syslog server. Contact jroussin\@netapp.com immediately if you see this");
        # TODO: send an email too?

    }
    $Log->comment("Finished worker startup");
    return;
}

## Public APIS

sub before_normal_test_start {
    my ($self, $test_name, $test_fqname) = validated_list(\@_,
        test_name   => {isa => 'Str', optional => 0},
        test_fqname => {isa => 'Str', optional => 0},
    );
    $Log->comment("Normal test '$test_fqname' is about to start");
    my $response = $self->get_errors;
    return $response;
}

sub normal_test_started {
    my ($self, $id, $name, $nodes_as_params) = validated_list(\@_,
        subtest_id      => { isa => 'Int',          optional => 0},
        subtest_runid   => { isa => 'Str',          optional => 0},
        nodes_as_params => { isa => 'ArrayRef[Str]',optional => 1},
    );

    my $asup_type;
    if (!$self->first_normal_test_started_time){
        $self->first_normal_test_started_time(time);

        # Call _init_from_parameters again in case the service was started
        # before ASUPTrigger::Deployment ran. This only happens during
        # unit testing right now
        $self->_init_from_parameters();

        # The first ASUP we send should be a TEST payload
        # so it has a better chance of being transmitted before the
        # normal test ends
        $asup_type = 'test';
    }

    my $test_info = { id => $id, name => $name};
    $Log->comment("Normal test started: ". Dumper($test_info));

    if ( $self->is_test_running($id) ){
        confess
            "Test with id '$id' is already running:\n" .
            "Test status:\n" . $self->dump_test_status
    }
    # No other test should be running. Only testcases can run in parallel
    if ($self->has_running_test){
        confess "Detected already running test(s)\n" . $self->dump_test_status;
    }

    $self->set_running_test( %$test_info );
    if ($self->enable_asup_triggers){

        # The site detection in _populate_nodes must run first
        my $detected_nodes = $self->nodes;
        my $previous_node_count = $self->node_count;

        if ($nodes_as_params){
            for my $n ( @$nodes_as_params ){
                $Log->comment("Adding node '$n' found as test param");
                $self->add_node($n);
            }
        }
        if ($self->node_count && !$previous_node_count){
            # We just added some nodes, see if we can now figure out the site
            $self->no_nodes_detected(0);
            if (!$self->site){
                # Rerun the site discovery
                $self->clear_site;
                my $new_site = $self->site;
                $Log->comment("Site '$new_site' was detected from nodes passed as params");
            }
        }

        # Let _create_asup_trigger_timer return early if no nodes are detected

        # Don't trigger ASUPs just yet, wait and see if testcases are coming
        $self->_create_asup_trigger_timer( $asup_type?( asup_type => $asup_type ):() );
        # The test has changed, remove the periodic timer, it will get
        # recreated after ASUPs get triggered
        $self->_remove_asup_periodic_timer();
    }
    my $response = $self->get_errors;
    return $response;
}

# The code below doesn't handle the case where normal tests
# are occuring faster than ASUPs can be generated. This isn't
# too bad since in the basence of testcases, all normal tests
# will be reported to ALM as a single test instance. But it's
# still a little bothersome, as the ASUP generated during the
# ntest run may not contain metadata for all normal tests run.
# The solution I think is to stop encoding normal test info
# in the ASUPs when possible, i.e. when a higher level file
# was run like a test plan. The NATE database doesn't currently
# keep track of the plan files. Burt1078086
sub normal_test_ended {
    my ($self, $id, $name) = validated_list(\@_,
        subtest_id      => { isa => 'Int', optional => 0},
        subtest_runid   => { isa => 'Str', optional => 0},
    );

    my $test_info = {id => $id, name => $name};
    $Log->comment("Normal test ended: ". Dumper($test_info));

    # Check for duplicate notification
    if ( $self->is_test_completed($id) ){
        confess
            "Testc is already completed:\n" . Dumper($test_info) .
            "Test status:\n" . $self->dump_test_status
    }

    # Check for ends without starts
    if (! $self->is_test_running($id) ){
        confess
            "Test isnt running:\n" . Dumper($test_info) .
            "Test status:\n" . $self->dump_test_status
    }

    # TODO: Since a large number of code out there generates testcases
    # themselves (using Tharn::testcase_begin rather than TCD or NATE::Driver)
    # it's possible they dont call testcase_end properly and this code will
    # blow up. If that happens, turns this confess into a warning and
    # assume all testcases are ending with the normal test.
    my @children = grep { $_->{parent_id} == $id } $self->running_testcases_list;
    if ( @children) {
        confess
            "Test children still running:\n" . Dumper($test_info) .
            "Test status:\n" . $self->dump_test_status
    }

    $self->add_completed_test($id => $test_info);
    $self->clear_running_test($id);

    $Log->comment("Running ASUPs for ending test:\n". Dumper($self->get_running_asups_for_test($id)));
    $Log->comment("Completed ASUPs for ending test:\n". Dumper($self->get_completed_asups_for_test($id)));

    my $response = $self->get_errors;
    return $response;
}

sub post_job_started {
    my $self = shift;
    $Log->comment("Postjob started");

    # Remove timers susceptible to trigger more ASUPs
    $self->_remove_asup_periodic_timer;
    $self->_remove_asup_trigger_timer;
}

# Run before ntest exits. Blocks if the test runtime was
# sufficiently long
# TODO: What should we do if ntest terminates before a single ASUP
# was sent?
# a. If we at least tried and failed to send ASUPs, do nothing.
# b. If we tried, but have yet to confirm any transmission, wait a little harder
# c. If we never tried because ntest is completing before the ASUP trigger
# was ever fired, send TEST payloads?
sub all_tests_ended {
    my $self = shift;

    $Log->comment("All tests ended");

    $self->remove_all_timers;

    # It's possible this API was started during a running timer, in which
    # case the safe thing to do is to let the timer finish first before
    # running the ASUP validation below
    my $running_timers = $self->has_timer_running;
    if ($running_timers){
        $Log->comment("Running timer(s) detected: $running_timers. The execution of all_tests_ended will be postponed until all timers have completed");
        $self->request_postpone(delay => 5);
        return;
    }

    my $runtime = time() - $self->startup_time;
    $Log->comment("Observed runtime: $runtime seconds");
    if (!$self->first_normal_test_started_time){
        $Log->comment("No normal test detected");
        $self->empty_syslog_buffer(retry_count => 4);
        return;
    }
    my $time_since_first_test = time() - $self->first_normal_test_started_time;
    $Log->comment("Time since first normal test started: $time_since_first_test");

    if ($self->enable_asup_triggers){
        if (!$self->asup_trigger_count){
            if ($self->has_nodes && $self->node_count > 0 ){
                if ($time_since_first_test > $self->asup_trigger_delay){
                    $self->log_error("No ASUPs were attempted during this ntest run");
                }
                else{
                    $self->report_comment_to_syslog("Test didnt run long enough to generate ASUPs");
                }
            }
            else{
                $Log->comment("No ASUPs were triggered because of lack of node(s) detected");
            }
        }
        else {
            if ( !$self->has_running_asups ) {
                my $successful_asups_per_node = $self->get_successful_asups_count_per_node();
                my @nodes_without = grep { !$successful_asups_per_node->{$_} } keys %$successful_asups_per_node;
                $Log->comment("Found " . scalar(@nodes_without) . " node(s) without successful ASUPs");
                if (@nodes_without){
                    # We have tried but never succeeded
                    # Try one more time, send a TEST one
                    $Log->comment("No ASUPs have been successfully sent so far from nodes (".join(',', @nodes_without) ."). Trying one more time with a TEST payload");

                    # The TestWatcher won't have a running test so we need to give it
                    # one
                    $self->set_running_test( id => 0, name => 'ntest' );
                    # Don't start a validation timer or weird things happen during _wait_for_asup_transmission
                    $self->_trigger_asups( asup_type => 'test', no_validation_timer => 1 );
                    $self->clear_running_test;
                }
            }

            # We have sent (or tried to send) ASUPs during this run
            # Detect when no connected nodes were reported
            # Eventually this information will be used to turn off ASUPTrigger
            # but for now we'll just emit a graylog message
            # TODO: Disabling this code until burt1110263 is adressed
            require Connectrec;
            if (0 && Connectrec->can('get_connected_hosts_from_keyval')){
                my $connected_hosts = Connectrec::get_connected_hosts_from_keyval();
                $Log->comment("Connected hosts:\n" . Dumper($connected_hosts));
                my %report_hash = map { 'connected_hosts_' . $_ => $connected_hosts->{$_} } keys %$connected_hosts;
                if (!$connected_hosts->{filer_hostname} || !keys %{$connected_hosts->{filer_hostname}}){
                    $self->report_comment_to_syslog('No connected filer detected', %report_hash);
                }
                else{
                    # See if there is a discrepancy between the connected node names
                    # and the nodes we know about
                    my @connected_node_names = keys %{$connected_hosts->{filer_hostname}};
                    $Log->comment("Connected node names: " . join(',', @connected_node_names));
                    my @our_nodes = $self->node_list;
                    my %nodes = map { $_ => 1 } @our_nodes;
                    my @unknown_connected_nodes = grep {  !$nodes{$_} } @connected_node_names;
                    if (@unknown_connected_nodes){
                        $self->report_comment_to_syslog('Unknown connected node(s) detected',
                            known_nodes             => join(',', @our_nodes),
                            unknown_connected_nodes => join(',', @unknown_connected_nodes),
                            %report_hash,
                        );
                    }
                }
            }
        }
    }

    # If the test lasted long enough, validate ASUP transmission
    # The idea is to not impact test runtime by more than X %
    if ($runtime >= $self->last_validation_min_test_runtime
            && $self->has_running_asups){
        my $max_runtime = max(
            $self->last_validation_min_duration,
            min($self->last_validation_max_duration, $runtime / 10)
        );
        my $end_time = time + $max_runtime;
        $Log->comment("Will wait for asup transmission validation for up to $max_runtime seconds");
        $self->_wait_for_asup_transmission_validation(
            end_time => $end_time
        );
    }

    my $report = $self->get_asup_activity_report;
    $Log->comment(delete $report->{as_string});

    $self->report_comment_to_syslog('ASUP success statistics', %$report);

    # TODO If no ASUPs have been verfiably sent, generate an error

    # Incurr a one-time runtime cost when graylog is down here
    $self->empty_syslog_buffer(retry_count => 6, max_time => 120);

    my $response = $self->get_errors;
    return $response;
}

#
# TODO testcase_XXX APIs not currently called by ntest
#

sub testcase_started {
    my ( $self, $id, $name, $parent_id) = validated_list(\@_, @TESTCASE_API_ARGS_VALIDATION_SPEC);
    my $test_info = { id => $id, name => $name, parent_id => $parent_id };
    $Log->comment("Testcase started: ". Dumper($test_info));

    if ( $self->is_testcase_running($id) ){
        confess
            "Testcase is already running:\n" . Dumper($test_info) .
            "Test status:\n" . $self->dump_test_status
    }

    # Find the normal test in NATE DB and replace $parent_id with it
    # The normal test is the ancestor subtest with parent_id = 0 and pre_post_type set to NULL
    if ( !$self->is_test_running($parent_id)){
        $parent_id = $self->_get_normal_test_id($parent_id);
        $test_info->{parent_id} = $parent_id;
        $Log->comment("Actual testcase normal test id: $parent_id");
    }

    if ( !$self->is_test_running($parent_id) ){
        confess
            "Testcase parent is not running:\n" . Dumper($test_info) .
            "Test status:\n" . $self->dump_test_status
    }
    $self->add_running_testcase($id => $test_info);
    if ($self->enable_asup_triggers){
        # Don't trigger ASUPs just yet, wait and see if more testcases
        # are coming. If so, we'll combine all their ids into one ASUP payload
        $self->_create_asup_trigger_timer();
        # The testcase has changed, remove the periodic timer
        $self->_remove_asup_periodic_timer();
    }
    my $response = $self->get_errors;
    return $response;
}

sub testcase_ended {
    my ($self, $id, $name, $parent_id) = validated_list(\@_, @TESTCASE_API_ARGS_VALIDATION_SPEC);
    my $test_info = {id => $id, name => $name, parent_id => $parent_id};
    $Log->comment("Testcase ended: ". Dumper($test_info));

    # Check for duplicate notification
    if ( $self->is_testcase_completed($id) ){
        confess
            "Testcase is already completed:\n" . Dumper($test_info) .
            "Test status:\n" . $self->dump_test_status
    }

    # Check for ends without starts
    if (! $self->is_testcase_running($id) ){
        confess
            "Testcase isnt running:\n" . Dumper($test_info) .
            "Test status:\n" . $self->dump_test_status
    }

    # The normal test is the ancestor subtest with parent_id = 0 and pre_post_type set to NULL
    if ( !$self->is_test_running($parent_id)){
        $parent_id = $self->_get_normal_test_id($parent_id);
        $test_info->{parent_id} = $parent_id;
        $Log->comment("Actual testcase normal test id: $parent_id");
    }

    # Check the parent is still running
    if ( ! $self->is_test_running($parent_id) ){
        confess
            "Testcase parent is not running:\n" . Dumper($test_info) .
            "Test status:\n" . $self->dump_test_status
    }

    $self->add_completed_testcase($id => $test_info);
    $self->remove_running_testcase($id);

    for my $node ( $self->node_list ){
        $Log->comment("Looking for ASUPs tied to testcase $id and node $node");
        if (!$self->has_asup_for_testcase(testcase => $id, node => $node)){
            $self->add_completed_testcase_to_backlog(
                testcase => $id,
                node     => $node,
            );
        }
    }
    my $response = $self->get_errors;
    return $response;
}

## Private or testing methods below

sub state_dump {
    my $self = shift;
    $self->freeze;
}

sub reset {
    my $self = shift;
    $self->remove_all_timers;
    my $running_timers = $self->has_timer_running;
    if ($running_timers){
        $Log->comment("Timer(s) detected during reset: $running_timers. Waiting 5s");
        $self->request_postpone( delay => 5);
        return;
    }
    $self->restore_defaults;

    # Reset the value of 'iteration' in case we are running
    # this thpl multiple times in a row
    $Log->comment("Resetting the iteration value of SystemTest::ASUPMessage to 1");
    my $keyval = keyval_global();
    my $ret = $keyval->hset(
        hash_name => 'SystemTest::ASUPMessage',
        key       => 'iteration',
        val       => 0,
    );
    my $value = $keyval->hget(
        hash_name => 'SystemTest::ASUPMessage',
        key       => 'iteration',
    );
    if ($value != 0){
        confess "Unexpected value for SystemTest::ASUPMessage iteration: $value";
    }

    return;
}

# Only used for testing
sub restore_defaults {
    my $self = shift;

    my @attributes = @_
        ? grep { defined $_ } map { $self->meta->find_attribute_by_name($_) } @_
        : $self->meta->get_all_attributes;

    for my $attribute (@attributes) {
        # Don't mess with those. They keep track of stuff
        # we don't want to forget
        next if ($attribute->name eq 'startup_time');
        next if ($attribute->name eq 'asup_validation_timer_running');
        next if ($attribute->name eq 'asup_trigger_timer_running');
        next if ($attribute->name eq 'asup_periodic_timer_running');

        $attribute->clear_value($self);
        if ( $attribute->has_default && !$attribute->is_lazy ){
            $attribute->set_value($self, $attribute->default($self));
        }
    }

    return;
}

# Rob didnt want to have NATE provide the subtest_id of the
# current normal test in calls to testcase_started
# Figure out that subtest_id ourselves
# Cache parent ids to limit latency
my %Parent_subtest_ids;
sub _get_normal_test_id {
    my ($self, $subtest_id) = @_;
    my $db = testdb_global();

    my $normal_test_id;
    while (! $normal_test_id ){
        my $parent = $Parent_subtest_ids{$subtest_id};
        if (! defined $parent ){
            $parent = $db->get_subtest_parent_id(subtest_id => $subtest_id);
            $Parent_subtest_ids{$subtest_id} = $parent;
        }
        if (!$parent){
            $normal_test_id = $subtest_id;
            last;
        }
        else{
            $subtest_id = $parent;
        }
    }
    return $normal_test_id;
}

sub _init_from_parameters {
    my ($self) = @_;
    my $paramset = NATE::ParamSet->new(global=>1);
    my $asup_type = $paramset->get_subtest_parameter(key=>'ASUPTRIGGER_ASUP_TYPE');
    if ($asup_type){
        log_global()->comment("Setting asup_type to $asup_type");
        $self->asup_type($asup_type);
    }

    my $skip_vsims = $paramset->get_subtest_parameter(key=>'ASUPTRIGGER_SKIP_VSIMS');
    if (defined $skip_vsims){
        log_global()->comment("Setting skip_vsims to $skip_vsims");
        $self->skip_vsims($skip_vsims);
    }

    my $user_is_not_qre = $paramset->get_subtest_parameter(key=>'ASUPTRIGGER_USER_IS_NOT_QRE');
    if (defined $user_is_not_qre){
        log_global()->comment("Setting user_is_not_qre to $user_is_not_qre");
        $self->user_is_not_qre($user_is_not_qre);
    }

    my $mock_graylog = $paramset->get_subtest_parameter(key=>'MOCK_GRAYLOG');
    if (defined $mock_graylog){
        $self->mock_graylog($mock_graylog);
    }

    my $mock_graylog_port = $paramset->get_subtest_parameter(key=>'MOCK_GRAYLOG_PORT');
    if  (defined $mock_graylog_port){
        $self->mock_graylog_port($mock_graylog_port);
    }

}

__PACKAGE__->meta->make_immutable;
1;
