#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @pod here
## @author dl-nacl-dev@netapp.com
## @summary The base class for all Xgen formatter modules

package NACL::Service::Xgen::Formatter;

use strict;
use warnings;

use NACL::Service::Xgen::Exceptions::UnsupportedMethod;

sub new {
    my ($class, %args) = @_;
    my $self = \%args;

    bless($self, $class);

    return $self;
}

sub apply_format_to_resource {
    my ($self, $resource_obj) = @_;

    if (ref($self) ne __PACKAGE__) {
        NACL::Service::Xgen::Exceptions::UnsupportedMethod->throw('apply_format_to_resource must ' .
                                                                  'be implemented by ' . ref($self));
    }

    # prepare_resource will remove the parents and children hashes
    # from the resource object
    $resource_obj->prepare_resource();

    # return back just the contents of resource obj
    return {%{$resource_obj}};
}

sub apply_format_to_resource_list {
    my ($self, $resource_hash_ref) = @_;
    return $resource_hash_ref;
}

1;

__END__

=head1 NAME

NACL::Service::Xgen::Formatter

=head1 SYNOPSIS

 package My::Formatter;
 # make the formatter a derived class of the base class
 use parent qw(NACL::Service::Xgen::Formatter);

 sub new {
     my ($class, %args) = @_;
     # invoke the base class constructor
     my $self = $class->SUPER::new(%args);

     # initialize anything you need here ...
 }

 # this method gets invoked on every resource
 # when the formatter is invoked
 sub apply_format_to_resource {
     my ($self, $resource_obj) = @_;

     # manipulate a single resource object in memory
 }

 # this method gets invoked after all resources have been formatted
 # individually
 sub apply_format_to_resource_list {
     my ($self, $resource_list) = @_;

     # manipulate the entire resource list
     # if needed
     foreach my $key (keys %{$resource_list}) {
         # ...
     }

     # return back the result of formatting the entire resource list
     return $resource_list;
 }

=head1 DESCRIPTION

C<NACL::Service::Xgen::Formatter> is an abstract class/interface for formatter classes.
It is the base class for all formatter classes. Formatter classes provide an optional
translation layer once an Xgen configuration has been built. Typically a formatter class
is needed if you need a consistent and simple mechanism for bridging Xgen with a
third party provisioning tool or library. The methods in this base class do not
need to be invoked directly, the Xgen Service framework will handle that for you.

See L<C<NACL::ServiceAPI::Xgen::build_config>|lib-NACL-ServiceAPI-Xgen-pm/build_config> for details on how to
have an xgen config get built and fed into a formatter class.

=head1 METHODS TO OVERRIDE

The following methods can be overridden by derived formatter classes.

=head2 new

The constructor of the formatter. Override this method if your
formatter requires some up front initialization.

=over

=item Synopsis

 sub new {
     my ($class, %args) = @_;
     # invoke the base class constructor
     my $self = $class->SUPER::new(%args);

     # initialize anything you need here ...
 }

=back

=head2 apply_format_to_resource

This method gets executed against every resource in the configuration.
Override this method if you need to perform any modifications at the individual resource
level.

=over

=item Synopsis

 sub apply_format_to_resource {
     my ($self, $resource_obj) = @_;

     # transform the resource object ...
 }

=item Arguments

=over

=item C<< \$NACL::Service::Xgen::Resource >>

A L<C<NACL::Service::Xgen::Resource>|lib-NACL-Service-Xgen-Resource-pm> object that
can be modified in place.

=back

=back

=head2 apply_format_to_resource_list

This method gets executed against the entire resource list after C<apply_format_to_resource>
gets executed against every resource. Override this method if you need to perform some modifications
to the structure of the resource list as a whole.

=over

=item Synopsis

 sub apply_format_to_resource_list {
     my ($self, $resource_list) = @_;

     # transform the entire resource list
     my $new_resource_list;

     # ...

     # return the new representation of the list
     return $new_resource_list;
 }

=item Arguments

=over

=item C<< \%hash >>

A hash reference keyed by resource type. The values of each resource type key will
contain an array of C<NACL::Service::Xgen::Resource> objects.

=back

=item Return Value

If you override this method, you will need to return back the modified resource list hash
reference.

=back

=head1 SEE ALSO

See L<C<NACL::Service::Xgen::Examples::ExampleFormatter>|lib-NACL-Service-Xgen-Examples-ExampleFormatter-pm>
for a derived formatter class example.

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut
