#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @pod here
## @author dl-nacl-dev@netapp.com
## @summary The base class for all Xgen resource modules

package NACL::Service::Xgen::Resource;

use strict;
use warnings;
use Tie::IxHash;

use NACL::Service::Xgen::Exceptions::InvalidAttribute;
use NACL::Service::Xgen::Exceptions::ChildResourceDoesNotExist;
use NACL::Service::Xgen::Exceptions::ParentResourceDoesNotExist;

use NATE::Exceptions::Argument qw(:try);

use constant nacl_cs_class => 0;

sub new {
    my ($class, %args) = @_;

    my $self = {};

    bless($self, $class);

    $self->{_nacl_cs_validation} = delete($args{xgen_nacl_cs_validation}) // 1;

    $self->{metadata} = delete($args{xgen_metadata}) // {};

    # all constructor options are private args for the
    # resource
    $self->{private} = {};
    $self->set_private_attributes(%args);

    # Resource memory address of parent Resource
    $self->{parents} = undef;

    # shared args are for attributes shared between Resources
    $self->{shared} = {};

    # hash of children Resource memory address string -> Resource memory address
    $self->{children} = {};

    return $self;
}

sub set_shared_attributes {
    my ($self, %args) = @_;

    while (my ($key, $value) = each %args) {
        $self->{shared}->{$key} = $value;
    }
}

sub get_attribute {
    my ($self, $attribute) = @_;

    if (exists($self->{private}) and exists($self->{private}->{$attribute})) {
        return $self->{private}->{$attribute};
    } elsif (exists($self->{shared}) and exists($self->{shared}->{$attribute})) {
        return $self->{shared}->{$attribute};
    } elsif (exists($self->{metadata}) and exists($self->{metadata}->{$attribute})) {
        return $self->{metadata}->{$attribute};
    } elsif (exists($self->{$attribute})) {
        return $self->{$attribute};
    } else {
        NACL::Service::Xgen::Exceptions::InvalidAttribute->throw("The attribute $attribute " .
                                                                 "does not exist on this resource");
    }
}

sub set_private_attributes {
    my ($self, %args) = @_;

    # if there is a linked nacl component state class
    # for this resource, validate the new private attrs
    # with the component state class
    if ($self->{_nacl_cs_validation} and $self->nacl_cs_class) {
        $self->_validate_private_attrs_with_nacl_cs_class(%args);
    }

    # make a copy of the current attributes
    my $new_attrs = {%{$self->{private}}};

    # merge in all the new attrs
    while (my ($key, $value) = each %args) {
        $new_attrs->{$key} = $value;
    }

    # validate the attrs using the generic attr validation routine
    # and store it on self
    $self->{private} = $self->_validate_private_attrs($new_attrs);

    return $self->{private};
}

sub set_metadata_attributes {
    my ($self, %args) = @_;

    # make a copy of the current attributes
    my $new_attrs = {%{$self->{metadata}}};

    # merge in all the new attrs
    while (my ($key, $value) = each %args) {
        $new_attrs->{$key} = $value;
    }

    # store it on self
    $self->{metadata} = $new_attrs;

    return $self->{metadata};
}

sub _validate_private_attrs_with_nacl_cs_class {
    my ($self, %args) = @_;

    # instantiate the nacl component state object using the
    # attributes provided
    my $class = $self->nacl_cs_class;
    my $req_class = $class;
    $req_class =~ s/::/\//g;
    $req_class .= '.pm';
    require $req_class;

    try {
        $class->new(%args);
    } otherwise {
        my $e = shift;
        # the provided attributes were invalid
        NACL::Service::Xgen::Exceptions::InvalidAttribute->throw("Resource Attr validation failed: " . $e->text);
    };
}

sub _validate_private_attrs {
    my ($self, $args) = @_;

    # NOOP for base class

    return $args;
}

sub to_format {
    my ($self, %args) = @_;

    my $format_obj = delete($args{formatter}) //
        NATE::Exceptions::Argument->throw("Missing required argument 'formatter'");
    if (!$format_obj->isa("NACL::Service::Xgen::Formatter")) {
        NATE::Exceptions::Argument->throw("Formatter object is not a valid 'NACL::Service::Xgen::Formatter' object");
    }

    my $ret = $format_obj->apply_format_to_resource($self);

    return $ret;
}

sub rebless_resource {
    my ($self, %args) = @_;

    my $resource_type
        = delete($args{resource_type})
        // NATE::Exceptions::Argument->throw('resource type argument is required');
    my $resource
        = delete($args{resource})
        // NATE::Exceptions::Argument->throw('resource argument is required');

    bless ($resource, $resource_type);

    return $resource;
}

sub prepare_resource {
    my ($self) = @_;

    # delete parent and children hashes from $self
    # they are unneccessary to pass to the client
    delete($self->{parents});
    delete($self->{children});

    # combine private and shared hashes
    # NOTE: private overwrites shared if there are duplicates
    my $shared_ref    = delete($self->{shared});
    my $private_ref   = delete($self->{private});
    my %resource_hash = (%$shared_ref, %$private_ref);

    # store keys and values back onto $self
    while (my ($key, $value) = each %resource_hash) {
        $self->{$key} = $value;
    }
}

sub copy_shared_attributes_from_resource {
    my ($self, %args) = @_;

    my $resource_to_copy_from = delete($args{resource}) //
        NATE::Exceptions::Argument->throw("Missing required argument 'resource'");
    if (!$resource_to_copy_from->isa("NACL::Service::Xgen::Resource")) {
        NATE::Exceptions::Argument->throw("'resource' is not a valid NACL::Service::Xgen::Resource object");
    }

    # point our shared hash to this resources shared hash
    $self->{shared} = $resource_to_copy_from->get_shared_attributes();

    # if we have any children resources,
    # we need to update all of their shared datastructures
    # as well since they may be stale
    my @children = $self->get_children_resources();
    foreach my $child_resource (@children) {
        # make this child copy the pointer to our shared attributes
        $child_resource->copy_shared_attributes_from_resource(resource => $self);
    }

    return $self->{shared};
}

sub get_shared_attributes {
    my ($self) = @_;

    return $self->{shared};
}

sub get_ancestors {
    my ($self, %args) = @_;

    my $ancestor_type = delete($args{type}) //
        NATE::Exceptions::Argument->throw("Missing required parameter 'type'");

    my @all_ancestors = ();
    if ($self->has_parent_resources()) {
        # see if if we have ancestors for this type that
        # are direct parents
        my @parents = $self->get_parent_resources(type => $ancestor_type);
        if (@parents) {
            # we found direct ancestors that are parents
            push(@all_ancestors, @parents);
        } else {
            # we need to walk each parent and find ancestors from their
            # point of view
            @parents = $self->get_parent_resources();
            foreach my $parent (@parents) {
                my @ancestors = $parent->get_ancestors(type => $ancestor_type);
                if (@ancestors) {
                    # this parent has ancestors of this type
                    push(@all_ancestors, @ancestors);
                } else {
                    # this parent does not have ancestors of this type
                }
            }
        }
    }

    return @all_ancestors;
}

sub get_descendants {
    my ($self, %args) = @_;

    my $descendant_type = delete($args{type}) //
        NATE::Exceptions::Argument->throw("Missing required parameter 'type'");

    my @all_descendants = ();
    if ($self->has_children_resources()) {
        #see if we have descendants for this type that are direct children
        my @children = $self->get_children_resources(type => $descendant_type);
        if (@children) {
            push(@all_descendants, @children);
        } else {
            # walk each child and find their descendants.
            @children = $self->get_children_resources();
            foreach my $child (@children) {
                my @descendants = $child->get_descendants(type => $descendant_type);
                if (@descendants) {
                    #this child has descendants of this type
                    push(@all_descendants,@descendants);
                } else {
                    #this child does not have descendants of this type
                }
            }
        }
    }

    return @all_descendants;
}

sub add_parent_resources {
    my ($self, %args) = @_;

    my $parents = delete($args{resources}) //
        NATE::Exceptions::Argument->throw("Missing required parameter 'resources'");
    my $connect_child = $args{connect} // 1;

    foreach my $parent (@{$parents}) {
        if (!$self->{parents}->{ref($parent)}) {
            tie(%{$self->{parents}->{ref($parent)}}, 'Tie::IxHash');
        }

        # add the parent resource to this resource
        $self->{parents}->{ref($parent)}->{$parent} = $parent;

        # add this resource as a child to the parent resource
        if ($connect_child) {
            $parent->add_children_resources(resources => [$self],
                                            connect => 0);
        }

        # copy the shared attributes from the parent resource
        # xxx: yes this will overwrite every time
        $self->copy_shared_attributes_from_resource(resource => $parent);
    }
}

sub get_parent_resources {
    my ($self, %args) = @_;

    my $parent_type = delete($args{type});

    my @parents = ();
    if ($parent_type) {
        @parents = values(%{$self->{parents}->{$parent_type}});
    } else {
        foreach my $type (keys %{$self->{parents}}) {
            push(@parents, values(%{$self->{parents}->{$type}}));
        }
    }
    return @parents;
}

sub has_parent_resources {
    my ($self) = @_;
    my $parent_count = scalar(keys %{$self->{parents}});
    return $parent_count ? 1 : 0;
}

sub is_child_of_resource {
    my ($self, %args) = @_;

    my $resource = delete ($args{resource}) //
        NATE::Exceptions::Argument->throw("Missing required argument 'resource'");

    if (exists($self->{parents}->{ref($resource)})) {
        if (exists($self->{parents}->{ref($resource)}->{$resource})) {
            return 1;
        }
    }
    return 0;
}

sub is_parent_of_resource {
    my ($self, %args) = @_;

    my $resource = delete ($args{resource}) //
        NATE::Exceptions::Argument->throw("Missing required argument 'resource'");

    if (exists($self->{children}->{ref($resource)})) {
        if (exists($self->{children}->{ref($resource)}->{$resource})) {
            return 1;
        }
    }
    return 0;
}

sub remove_parent_resources {
    my ($self, %args) = @_;

    my $resources = $args{resources} // NATE::Exceptions::Argument->throw("Missing required argument 'resources'");
    my $disconnect = $args{disconnect} // 1;

    foreach my $resource (@{$resources}) {
        if (exists($self->{parents}->{ref($resource)})) {
            if (exists($self->{parents}->{ref($resource)}->{$resource})) {
                # remove this parent from this resource
                delete($self->{parents}->{ref($resource)}->{$resource});

                # if there are no parents left of this type, cleanup the type hash
                if (not scalar(values($self->{parents}->{ref($resource)}))) {
                    delete $self->{parents}->{ref($resource)};
                }

                # Remove this resource from our ex-parent's children list
                if ($disconnect) {
                    $resource->remove_children_resources(resources => [$self],
                                                         disconnect => 0);
                }
            } else {
                NACL::Service::Xgen::Exceptions::ParentResourceDoesNotExist->throw(
                    "The resource $self does not have a parent resource of $resource");
            }
        } else {
            NACL::Service::Xgen::Exceptions::ParentResourceDoesNotExist->throw(
                "The resource $self does not have a parent resource of $resource");
        }
    }
}

sub add_children_resources {
    my ($self, %args) = @_;

    my $children = delete($args{resources}) //
        NATE::Exceptions::Argument->throw("Missing required parameter 'resources'");
    my $connect_parent = $args{connect} // 1;

    foreach my $child (@{$children}) {
        # tie this child hash if we have not created it yet
        if (!$self->{children}->{ref($child)}) {
            tie(%{$self->{children}->{ref($child)}}, 'Tie::IxHash');
        }

        $self->{children}->{ref($child)}->{$child} = $child;

        #add this resource as a parent to the child resource
        if($connect_parent) {
            $child->add_parent_resources(resources => [$self],
                                         connect => 0);
        }
    }
}

sub get_children_resources {
    my ($self, %args) = @_;

    my $child_type = delete($args{type});

    my @children = ();
    if ($child_type) {
        @children = values(%{$self->{children}->{$child_type}});
    } else {
        foreach my $type (keys %{$self->{children}}) {
            push(@children, values(%{$self->{children}->{$type}}));
        }
    }
    return @children;
}

sub has_children_resources {
    my ($self) = @_;

    return scalar(keys(%{$self->{children}})) ? 1 : 0;
}

sub remove_children_resources {
    my ($self, %args) = @_;

    my $resources = $args{resources} // NATE::Exceptions::Argument->throw("Missing required argument 'resources'");
    my $disconnect = $args{disconnect} // 1;

    foreach my $resource (@{$resources}) {
        if (exists($self->{children}->{ref($resource)})) {
            if (exists($self->{children}->{ref($resource)}->{$resource})) {
                # remove this child from this resource
                delete($self->{children}->{ref($resource)}->{$resource});

                # if there are no children left of this type, cleanup the type hash
                if (not scalar(values($self->{children}->{ref($resource)}))) {
                    delete $self->{children}->{ref($resource)};
                }

                # remove ourselves as a parent from this resource
                if ($disconnect) {
                    $resource->remove_parent_resources(resources => [$self],
                                                       disconnect => 0);
                }
            } else {
                NACL::Service::Xgen::Exceptions::ChildResourceDoesNotExist->throw(
                    "The resource $self does not have a child resource of $resource");
            }
        } else {
            NACL::Service::Xgen::Exceptions::ChildResourceDoesNotExist->throw(
                "The resource $self does not have a child resource of $resource");
        }
    }
}

sub delete_resource {
    my ($self) = @_;

    my @deleted_resources = ($self);

    # first delete all of our children
    foreach my $child_type (keys %{$self->{children}}) {
        foreach my $child_resource (values %{$self->{children}->{$child_type}}) {
            push(@deleted_resources, $child_resource->delete_resource());
        }
    }

    # now remove ourselves from our parents children lists
    if ($self->has_parent_resources()) {
        foreach my $parent ($self->get_parent_resources()) {
            $parent->remove_children_resources(resources => [$self]);
        }
    }
    return @deleted_resources;
}

sub DESTROY {
    my ($self) = @_;
    # delete all of this resource's children if we are being cleaned up
    # by the garbage collector
    $self->delete_resource();
    undef $self;
}

1;

__END__

=head1 NAME

NACL::Service::Xgen::Resource

=head1 SYNOPSIS

 package My::Resource;
 # make My::Resource a derived class of the base class
 use parent qw(NACL::Service::Xgen::Resource);
 1;

=head1 DESCRIPTION

C<NACL::Service::Xgen::Resource> is an abstract class for resource objects. It is
the base class for all resource objects. Resource objects are a logical representation
of a physical or virtual entity that when brought together form a configuration
of some kind (see L<C<NACL::Service::Xgen::Config>|lib-NACL-Service-Xgen-Config-pm>).
Resources are comprised of attributes that describe said resource.
If desired, resources can be linked with other resources to form dependecy
trees. This can be done via the C<add_children_resources> and C<add_parent_resources>
methods.

This class should not be instatiated directly. To use this class properly, it is
best to create a new resource class that is derived from this one.

=head1 RESOURCE CATALOG

For the official Xgen Resource Catalog, see the following directory
C<NACL::AppBuilder::Xgen::Resource>. It is recommended that new resources are
placed in the catalog so that they gain more visibility to other users of the Xgen
framework. This would help encourage reuse and prevent the duplication of resource types.

=head1 ATTRIBUTE TYPES

=head2 private_attributes

Private attributes are attributes that uniquely describe a resource object.
They are typically defined at construction time of the resource object. They can
be modified via the L<C<set_private_attributes>|lib-NACL-Service-Xgen-Resource-pm/set_private_attributes>
method.

=head2 shared_attributes

Shared attributes are attributes that are inherited by a resource when linked to
a parent resource via the L<C<add_parent_resources>|lib-NACL-Service-Xgen-Resource-pm/add_parent_resources> method. Shared attributes
can be modified via the L<C<set_shared_attributes>|lib-NACL-Service-Xgen-Resource-pm/set_shared_attributes> method.

=head1 METHODS

=head2 new

This is the constructor for the resource.

=over

=item Synopsis

 use My::Resource; # isa NACL::Service::Xgen::Resource
 my $resource = My::Resource->new(param1 => "some value", param2 => [1,2,3]);

=item Arguments

=over

=item C<< xgen_nacl_cs_validation => 0|1 >>

(Optional) Controls whether this resource will attempt to perform NACL Component State
validation of all the C<%private_attributes> passed into the constructor. Disabling this
may solve performance issues when creating resource objects at scale.

Default: 1 (enabled)

=item C<< %private_attributes >>

(Optional) All parameters passed into the constructor of a resource are stored
as C<private_attributes>. These attributes should uniquely describe this resource.

Default: undef

=back

=back

=head2 get_attribute

Access private or shared attribute values.

=over

=item Synopsis

 my $res = My::Resource->new(attr1 => 10, attr2 => "foo");
 my $value = $res->get_attribute("attr1"); # returns the value '10'

=item Arguments

=over

=item C<< $attribute_name >>

(Required) The attribute to obtain the value of.

=back

=item Exceptions Thrown

 NACL::Service::Xgen::Exceptions::InvalidAttribute - if $attribute_name does not exist on the resource

=back

=head2 get_shared_attributes

Obtain the shared attribute datastructure hash for this resource

=over

=item Synopsis

 my $res = My::Resource->new();
 my $shared_attrs = $resource->get_shared_attributes();

=item Arguments

=over

None

=back

=back

=head2 set_private_attributes

Add new or modify existing private attribute values.

=over

=item Synopsis

 my $res = My::Resource->new(attr1 => "foo");
 $res->set_private_attributes(attr1 => "bar", new_attr2 => "123");

=item Arguments

=over

=item C<< %attrs >>

(Optional) A hash of key/value pairs to assign as private attributes.

Default: undef

=back

=item Exceptions Thrown

 NACL::Service::Xgen::Exceptions::InvalidAttribute - if an attribute is not valid for this resource type

=back

=head2 set_shared_attributes

Add new or modify existing shared attribute values.

=over

=item Synopsis

 my $res = My::Resource->new(private_attr1 => "foo");
 $res->set_shared_attributes(shared_attr1 => "bar", shared_attr2 => "123");

=item Arguments

=over

=item C<< %attrs >>

(Optional) A hash of key/value pairs to assign as shared attributes.

Default: undef

=back

=back

=head2 rebless_resource

(static method) Re-bless a resource object.

=over

=item Synopsis

 use NACL::Service::Xgen::Resource;
 my $res = NACL::Service::Xgen::Resource->rebless_resource(resource_type => "My::Resource",
                                                           resource => {});

=item Arguments

=over

=item C<< resource_type => $scalar >>

(Required) The resource class to rebless as.

=item C<< resource => \%hashref >>

(Required) A hash reference to represent the newly blessed resource

=back

=back

=head2 prepare_resource

Flatten the attributes of a resource into the object as well
as remove all parent & children associations.

=over

=item Synopsis

 my $res = My::Resource->new();
 $res->prepare_resource();

=item Arguments

=over

None

=back

=back

=head2 copy_shared_attributes_from_resource

Copy the shared attributes datastructure from another resource into this one.
This will overwrite existing shared attributes. When doing this, the two
resources will become linked.

=over

=item Synopsis

 my $res1 = My::Resource->new();
 my $res2 = My::Resource->new();
 $res1->copy_shared_attributes_from_resource(resource => $res2);

=item Arguments

=over

=item C<< resource => NACL::Service::Xgen::Resource >>

(Required) The resource object to copy from.

=back

=item Return Value

A hash reference of attribute/values.

=back

=head2 get_ancestors

Obtain all ancestor resource objects of a specified type
to this resource object.

=over

=item Synopsis

 my $res = My::Resource->new();
 my @ancestors = $res->get_ancestors(type => "My::Other::Resource");

=item Arguments

=over

=item C<< type => $scalar >>

(Required) The resource type to search for.

=back

=item Return Value

An array of resource objects.

=back

=head2 get_descendents

Obtain all descendent resource objects of a specified type
to this resource object.

=over

=item Synopsis

 my $res = My::Resource->new();
 my @descendents = $res->get_descendents(type => "My::Other::Resource");

=item Arguments

=over

=item C<< type => $scalar >>

(Required) The resource type to search for.

=back

=item Return Value

An array of resource objects.

=back

=head2 add_parent_resources

Register resources as parents of this resource.

=over

=item Synopsis

 my $res = My::Resource->new();
 my $res2 = My::Other::Resource->new();
 $res->add_parent_resources(resources => [$res2]);

=item Arguments

=over

=item C<< resources => \@arrayref >>

(Required) An array reference of resources to register as parents to
this resource

=back

=back

=head2 get_parent_resources

Obtain resource objects that are parents of this resource.

=over

=item Synopsis

 my $res = My::Resource->new();
 my @all_parents = $res->get_parent_resources();
 my @some_parents = $res->get_parent_resources(type => "My::Other::Resource");

=item Arguments

=over

=item C<< type => $scalar >>

(Optional) A specific resource type to search for parent resources. If
not specified, then all resource parents will be returned

Default: undef

=back

=item Return Value

An array of resource objects that match the criteria provided.

=back

=head2 has_parent_resources

Determine if this resource has any parent resources.

=over

=item Synopsis

 my $res = My::Resource->new();
 if ($res->has_parent_resources()) {
     # ...
 }

=item Arguments

=over

None

=back

=item Return Value

 0 - if the resource has no parents.
 1 - if the resource has parents.

=back

=head2 remove_parent_resources

Remove a set of resources from the parent list of this resource.

=over

=item Synopsis

 my $res1 = My::Resource->new();
 my $res2 = My::Resource->new();

 $res1->remove_parent_resources(resources => [$res2]);

=item Arguments

=over

=item C<< resources => \@arrayref >>

(Required) An array reference of C<NACL::Service::Xgen::Resource> objects to remove as parent
resources from the instantiated resource.

=back

=item Exceptions Thrown

 NACL::Service::Xgen::Exceptions::ParentResourceDoesNotExist - If a resource provided is not a parent resource.

=back

=head2 is_parent_of_resource

Determine if this resource is a parent of another resource.

=over

=item Synopsis

 my $res1 = My::Resource->new();
 my $res2 = My::Resource->new();

 # check if res1 is a parent of res2
 if ($res1->is_parent_of_resource(resource => $res2)) {
     # ...
 }

=item Arguments

=over

=item C<< resource => \$NACL::Service::Xgen::Resource >>

(Required) A C<NACL::Service::Xgen::Resource> object to check against

=back

=item Return Value

 0 - if the resource is not the parent
 1 - if the resource is the parent.

=back

=head2 add_children_resources

Add the specified resources as children resources to this resource.

=over

=item Synopsis

 my $res1 = My::Resource->new();
 my $res2 = My::Resource->new();
 my $res3 = My::Resource->new();

 $res1->add_children_resources(resources => [$res2, $res3]);

=item Arguments

=over

=item C<< resources => \@arrayref >>

(Required) An array reference of C<NACL::Service::Xgen::Resource> objects to add as children
resources from the instantiated resource.

=back

=item Exceptions Thrown

 NACL::Service::Xgen::Exceptions::ParentResourceDoesNotExist - If a resource provided is not a parent resource.

=back

=head2 get_children_resources

Obtain resource objects that are children of this resource.

=over

=item Synopsis

 my $res = My::Resource->new();
 my @all_children = $res->get_children_resources();
 my @some_children = $res->get_children_resources(type => "My::Other::Resource");

=item Arguments

=over

=item C<< type => $scalar >>

(Optional) A specific resource type to search for children resources. If
not specified, then all resource children will be returned.

Default: undef

=back

=item Return Value

An array of resource objects that match the criteria provided.

=back

=head2 has_children_resources

Determine if this resource has any children resources

=over

=item Synopsis

 my $res = My::Resource->new();
 if ($res->has_children_resources()) {
     # ...
 }

=item Arguments

=over

None

=back

=item Return Value

 0 - if the resource has no children.
 1 - if the resource has children.

=back

=head2 remove_children_resources

Remove a set of resources from the children list of this resource.

=over

=item Synopsis


 my $res1 = My::Resource->new();
 my $res2 = My::Resource->new();

 $res1->remove_children_resources(resources => [$res2]);

=item Arguments

=over

=item C<< resources => \@arrayref >>

(Required) An array reference of C<NACL::Service::Xgen::Resource> objects to remove as children
resources from the instantiated resource.

=back

=back

=head2 is_child_of_resource

Determine if this resource is a child of another resource.

=over

=item Synopsis

 my $res1 = My::Resource->new();
 my $res2 = My::Resource->new();

 # check if res1 is a child of res2
 if ($res1->is_child_of_resource(resource => $res2)) {
     # ...
 }

=item Arguments

=over

=item C<< resource => \$NACL::Service::Xgen::Resource >>

(Required) A C<NACL::Service::Xgen::Resource> object to check against

=back

=item Return Value

 0 - if the resource is not child.
 1 - if the resource is a child.

=back

=head2 delete_resource

Delete a resource and all of its descendents. If you are attempting to remove
a resource that has already been added to a configuration, you may want to use
the L<C<NACL::Service::Xgen::Config::remove_resources>|lib-NACL-Service-Xgen-Config-pm\remove_resources> method.

=over

=item Synopsis

 my $res0 = My::Resource->new();
 my $res1 = My::Resource->new();
 my $res2 = My::Resource->new();
 my $res3 = My::Resource->new();

 $res0->add_children_resources(resources => [$res1]);
 $res1->add_children_resources(resources => [$res2, $res3]);

 # res1 will no longer be a child to res0
 # res2 and res3 will no longer be children of res1.
 # this will recursively invoke delete on all resources from res1 and downwards
 my @deleted_resources = $res1->delete_resource();

 # @deleted_resources should contain ($res1,$res2,$res3)

=item Arguments

=over

None

=back

=item Return Value

An array reference of all resource objects that have been deleted

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut
