# This is basically -*- perl -*-
# $Id: lib/NACL/Smart.pm
#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
# Iconnect/Smart.pm
##
## @summary
##    Semi-Manual Automated Regression Test (SMART) library
##
## @description
##    This library facilitates the automation of manual tests, whether they be
##    High Touch tests or Low Touch tests.
##
##    The general idea is that encapsulating test execution within a script,
##    even if a single step or the entire test is actually executed by the
##    script, will enhance tester efficiencies. The first level of efficiency is
##    gained through automatic console logging (via NACL) and automatic log
##    posting and test result posting (via qc_push.thpl).
##
##    The second level of gained efficiencies are realized in Test Cases that
##    have individual steps which cannot be traditionally scripted.  In these
##    Test Cases, typically the majority of steps can be automated. This library
##    allows Automation Engineers to create automated scripts for test cases
##    with manual steps. As in typical automation, the script should execute all
##    steps which can be automated. This library then comes in, allowing the
##    tester to execute a manual step and report to the script the results of
##    that step. The script will proceed with the test case, reporting final
##    results as usual.
##
##
## @synopsis
##   use NACL::Smart;
#
#
##
## @example
##   use NACL::Smart;
##   $test_step   = NACL::Smart->new();
##   $test_result = $test_step->run();
##
## @burt 894864 Smart.pm - Initial Check-In
## @burt 914773 RFE - Notification on console to start Manual Step
## @burt 963444 Added step details to primary test case log
## @burt 962622 Fixed console message failing if console not at CLI
## @burt 992454 Add option to wait for all detector threads to complete
## @burt 995970 Smart.pm now returns NATE::Result objects properly
## @burt 1005414 Usability enhancements (1/2)
## @burt 1012006 Usability enhancements (2/2)
## @burt 1048084 smart.thpl overhaul - Add TIMEOUT_USER_RESPONSE back in.
## @burt 1073145 Add 1s sleep to Detector wait loop for CPU usage
## @burt 1082771 Iconnect::Smart::Detector - Enhancement to support codespec
## @burt 1089012 Implement Iconnect::Smart::Detector codespec
## @burt 1102904 Move Iconnect::Smart to NACL::Smart
#
## @keywords
##   interconnect; ic; smart; manual
##
## @status public
## @pod here
##
## @author Interconnect QA (dl-interconnect-qa@netapp.com)
## @author Paul Waters (pwaters@netapp.com)

################################################################################
#                              Start POD here
################################################################################

=head1 NAME

NACL::Smart - Semi-Manual Automated Regression Test class

=head1 SYNOPSIS

 use NACL::Smart;
 $test_step   = NACL::Smart->new();
 $test_result = $test_step->run();

=head1 DESCRIPTION

This library facilitates the automation of manual tests, whether they be
High Touch tests or Low Touch tests.

The general idea is that encapsulating test execution within a script,
even if a single step or the entire test cannot be executed by the
script, will enhance tester efficiencies. The first level of efficiency is
gained through automatic console logging (via C<NACL>) and automatic log posting
and test result posting (via C<qc_push.thpl>).

The second level of gained efficiencies are realized in Test Cases that
have individual steps which cannot be traditionally scripted.  In these
Test Cases, typically the majority of steps can be automated. This library
allows Automation Engineers to create automated scripts for test cases with
manual steps. As in typical automation, the script should execute all
steps which can be automated. This library then comes in, allowing the
tester to execute a manual step and report to the script the results of that
step. The script will proceed with the test case, reporting final results as usual.

=head1 EXCEPTIONS

=over

=item C<NATE::BaseException>

=item C<NATE::Exceptions::Argument>

=back

=cut

package NACL::Smart;

# Compiler Directives
use strict;
use warnings FATAL => 'all';

# NATE imports
use List::MoreUtils qw(any);
use NATE::Log qw(log_global);
use NATE::ParamSet;
use Params::Validate qw(:all);

# NATE Exceptions
use NATE::Exceptions::Argument;

BEGIN {
    Params::Validate::validation_options(
        on_fail => sub {
            NATE::Exceptions::Argument->throw(shift);
        }
    );
}

my ($Log, $ParamSet, );
$Log      = log_global();
$ParamSet = NATE::ParamSet::param_global();

=head1 METHODS

=head2 new

    $test_step = NACL::Smart->new(
        user_input_node => 'name_of_node_to_monitor_console',
        [step_details   => $message],
        [detector       => \&subroutine_reference],
    );

Creates a Smart object that will be used to track the execution of a manual
test step.

=over

Arguments

=over

=item C< user_input_node =E<gt> 'name_of_node' >

(Required) String - Name of the node a Connectrec connection will be created
against in the User Input Detector. The User Input Detector will use the console
connection to 'user_input_node' to monitor for specific text from the tester
indicating the results of a test step.

=item C< detector =E<gt> L<NACL::Smart::Detector|lib-NACL-Smart-Detector-pm> >
-or-
=item C< detector =E<gt> List of L<NACL::Smart::Detector|lib-NACL-Smart-Detector-pm> > objects >

(Optional) Coderef to be executed as a detector to
monitor for step completion. For example, this routine can monitor an interface
to determine when its link has been pulled. This routine should create an
appropriate NATE::Result object before returning. Out of all NATE::Result
objects created by the detector, the worst result from the detector is used to
define the run result of an execution.

This NATE::Result object is then used to propogate the step result back up to
the script.

If no detector is provided, only the User Input detector is started.  See
L<NACL::Smart-E<gt>run()|smart/"run"> for more details.

=item C< step_details =E<gt> $message >

(Optional) String or Arrayref - Message to be displayed when script is ready for
the tester to execute the manual step.  Can be used to specify the details of
the test step to be executed.

=item C< timeout =E<gt> value >

(Optional) Scalar - Time to wait for either a supplied detetor to return or
user to respond with Test Step Status. Defaults to '-1', which means no timeout.

=item C< wait_for_all =E<gt> 0|1 >

(Optional) Scalar - If 0, execution proceeds once one of the threads completes.
If 1, execution does not proceed unless all the detector threads complete.
Default: 0

=back

=item Returns

C<NACL::Smart> object.

=back

=cut

################################################################################
# Function    : new()
# Description :
#     Constructor to create new Smart Test Step object
# Arguments   :
#     user_input_node - string              - (required) name of node to monitor
#         in User Input Detector
#     step_details    - arrayref / string   - (optional) Text to print to log
#         when tester is to start execution of step
#     detector        - Detector / ARRAYREF - (optional) Can be a single
#         NACL::Smart::Detector object, or an ARRAYREF consisting of a list
#         of NACL::Smart::Detector objects
#     timeout         - scalar              - (Optional) Length of timeout
#         Defaults to indefinite.
# Returns     : none
# Exceptions  : NATE::Exceptions::Argument - if invalid argument type passed in,
#         or if FILERA is not defined.
################################################################################
sub new {
    $Log->enter();
    require NATE::Exceptions::Argument;

    my (%args, $class, $self, );
    $class = shift;    # because we're doing OO
    $self  = {};       # this holds our class instance data

    # Ah-Choo
    bless($self, $class, );

    # Validate Arguments, and set default for Manual Step message. This message
    # goes into the test log file, directing the tester to the detector log
    # files for further information.
    %args = validate(
        @_,
        {
            user_input_node => {
                type    => SCALAR,
                optional => 0,
            },
            detector => {
                type      => HASHREF | ARRAYREF,
                optional  => 1,
                callbacks => {
                    'array validation' => \&_user_detector_validation,
                }
            },
            step_details => {
                type     => SCALAR | ARRAYREF,
                optional => 1,
                default  =>
                    'No step details provided. Follow the respective test step '
                    . 'in ALM.',
            },
            timeout      => {
                type     => SCALAR,
                optional => 1,
                default  => -1, # No timeout by default
            },
            wait_for_all => {
                type     => SCALAR,
                optional => 1,
                default  => 0,
            },
        },
    ); # End Params::Validate::validate() call

    # 'user_input_node' is where user will input step status.
    # Let's do a quick check to make sure we can create a console connection now
    # so that we have better confidence that things won't fall over in the
    # middle of test execution.
    $self->{input_node} = $args{user_input_node};
    Connectrec->new(
        host     => $self->{input_node},
        template => 'console',
    );

    # Process optional 'step_details' argument
    if (ref($args{step_details}) eq 'ARRAY') {
        # Multi-line array passed in. join() lines into scalar.
        $self->{step_details} = join ("\n", @{$args{step_details}});
    } else {
        # Single line passed in (or default used from above)
        $self->{step_details} = $args{step_details};
    }

    # Process optional 'detector' argument. Put everything into a private array,
    # then put the array into $self->{detectors}. We are putting this into an
    # array, even if a single detector is passed in, for simpler use later.
    if (ref($args{detector}) eq 'ARRAY') {
        # List (array) of detectors passed in. Nothing to be done
        $self->{detectors} = $args{detector};
    } else {
        # First, make sure detector is defined. If so, create a list of one
        # detector and store it.
        if (defined($args{detector})) {
            $self->{detectors} = [$args{detector}];
        }
    } # End detector storing
    $self->{wait_for_all} = $args{wait_for_all};

    # Process optional timeout parameter
    $self->{timeout} = $args{timeout} || -1;

    $Log->exit();
    return $self;
} # End new()

=head2 run

    $test_result = $test_step->run()

Executes manual step. If a detector method is not provided at instance creation,
or if called as a Class method, creates a single User Input detector that looks
for input from the user for results of the test step.
If a detector method is provided, creates the User Input detector as well
as a separate thread for the provided detector to run in parallel.

The first thread to complete (User Input or specified detector method)
determines the results of test step.

The User Input Detector looks for specific keywords to be entered on the console
of FILERA. Either a response to User Input Detector or a completed
NACL::Smart::Detector is required for the test script to continue. The
following is a list of expected keywords and their results:

=over

=over

=item C<STEP_PASS>

Indicates the manual step has passed and prompts C<NACL::Smart-E<gt>run()>
to return a C<NATE::Result::Pass> object.

=item C<STEP_FAIL>

Indicates that the manual step has failed, creates a C<NATE::Result::Fail>
object, and prompts C<NACL::Smart-E<gt>run()> to return a
C<NATE::Result::Fail> object.

=item C<STEP_CONFIG>

Indicates that the step neither passed nor failed. Can be used to stop the
script without passing a result back to ALM (for example, issue with testbed
that does not affect a test's status, but which still needs to be addressed
before the test can execute). Will create a C<NATE::Result::Config> object and
prompts C<NACL::Smart-E<gt>run()> to return a C<NATE::Result::Config>
object.

=back

=item Arguments

None.

=item Returns

C<NATE::Result> object

=back

=cut


################################################################################
# Function    : run()
# Description :
#     Starts execution of manual step.  Creates multiple threads for each
#     detector.  A 'User Input' detector is implicitly started, regardless.  If
#     additional detector routines are passed in at object creation, each are
#     started in their own threads and run in parallel.
#     When first thread exits, kill all other threads and report result.
# Arguments   : none
# Returns     : NATE::Result object with worst returned state of successfully
#     completed threads.
# Exceptions  : None
################################################################################
sub run {
    $Log->enter();
    require NATE::Process;

    my (
        $iteration, @process_list, $running_processes, $self,
        $user_instructions, $wait_for_all,
    );
    $self         = shift;
    $wait_for_all = $self->{wait_for_all};

    # Create a prefix for ProcManager thread names. When multiple Smart calls
    # are used within a script, this is used to separate and order the Smart
    # log files.
    $iteration = sprintf('%02d', $ParamSet->get('Smart_Iteration', default => 1, ));
    $ParamSet->set(key => 'Smart_Iteration', value => ($iteration+1));

    # Create the text block we'll display in the logs. This gets displayed in
    # both the xx_testcase.log and User_Input_Detector.log.
    $user_instructions = _generate_user_instruction(
        self => $self,
    );

    # Next, we create NATE::Process objects for all detectors being run.
    # NB: I am explicitly setting some values to their defaults in NATE::Process
    #     where unexpected things could happen if NATE::Process defaults ever
    #     change.
    # Let's start with User Input detector.
    $Log->debug(__PACKAGE__ . ': Adding User Input detector to process stack.');
    @process_list = (
        NATE::Process->new(
            codespec    => \&_user_input_detector,
            runid       => "${iteration}_User_Input_Detector",
            onexit      => \&NATE::Process::on_exit_propagate_worst_result,
            args        => [
                input_node => $self->{input_node},
                user_text  => $user_instructions,
                timeout    => $self->{timeout},
            ],
            background  => 1,
            independent => 0,
            inherit     => 1,
        ),
    );

    # Next, let's add User Supplied detectors.
    if ($self->{detectors}) {
        my (@user_detectors, );
        $Log->debug(__PACKAGE__ . ': Adding User Supplied detector(s) to process stack.');

        @user_detectors = @{$self->{detectors}};

        # If script passed in User Detectors, create NATE::Process object for
        # each user detector.
        USER_DETECTORS: for my $i (0..$#user_detectors) {
            my (%args, $detector, );

            # Each User Detector definition should be an
            # NACL::Smart::Detector object which defines the 'detector'
            # (CODEREF) and an optional 'args' parameter (ARRAYREF)
            $detector = $user_detectors[$i]->{codespec};

            # args is optional, however we can't pass "args => undef" to
            # NATE::Process->new(). So, create a hash to either specify 'args'
            # with its HASHREF, or not specify 'args' if it's not used.
            # Additionally, we are passing 'args' and 'params' to NATE::Process.
            # Using a CODEREF, 'args' is required. Using a .thpl, 'params' is
            # required. Also, params has to be HASHREF, where args has to
            # be ARRAYREF, so convert if needed.
            if ($user_detectors[$i]->{args}) {
                if (ref($user_detectors[$i]->{codespec}) eq 'CODE') {
                    my (@user_args);
                    @user_args = %{$user_detectors[$i]->{args}};
                    %args = ( args => \@user_args, );
                } else {
                    %args = ( params => $user_detectors[$i]->{args}, );
                }
            }

            # We have what we need. Now create a NATE::Process object and queue
            # it up.
            push(
                @process_list,
                NATE::Process->new(
                    codespec    => $detector,
                    runid       => "${iteration}_User_Detector_" . ($i+1),
                    onexit      => \&NATE::Process::on_exit_propagate_worst_result,
                    background  => 1,
                    independent => 0,
                    inherit     => 1,
                    %args,
                ),
            );
        } # End USER_DETECTORS loop
    } # End if ($self->{detectors})

    # At this point, all of our threads should have NATE::Process objects.
    # Now we need to start things up.
    $Log->log(__PACKAGE__ . ': Starting ' . @process_list . ' detector threads.');
    START_THREADS: foreach my $process (@process_list) {
        $process->start();
    } # End START_THREADS

    # All detector threads have now started. This loop repeatedly polls each
    # thread until:
    # All threads are complete if $wait_for_all == 1. Once all
    # threads are complete we record the last result and pop out of the loop
    # One thread completes if $wait_for_all == 0. Once one thread has
    # completed, we record the reported result and pop out of the loop.
    POLL_RUNNING_THREADS: while (1) {
        $Log->debug(__PACKAGE__ . ': Polling threads for completion.');
        $running_processes = 0;
        PROCESS_LIST_LOOP: foreach my $process (@process_list) {
            if ($process->is_running()) {
                # Create running process counter
                $running_processes++;
            } elsif (!$process->is_running()) {
                my ($worst_result, );
                # A thread has completed. Record its result.

                # If a process does not explicitly create a NATE::Result::Pass
                # object, worst_result() will be undefined.  Let's save a
                # NATE::Result object here to make lives easier later.
                if (!defined($process->worst_result())) {
                    $worst_result = NATE::Result::Pass->new();
                    $worst_result->store();;
                } else {
                    $worst_result = $process->worst_result();
                } # End if-elsif (!defined($process->worst_result()))

                # Now, if $self->{last_result} is not defined yet, let's use the
                # worst_result.
                # If it is defined, which may happen in case wait_for_all = 1,
                # use the result with the higher severity as the last_result
                if (!defined($self->{last_result})) {
                    $self->{last_result} = $worst_result;
                } elsif ($self->{last_result}->severity() < $worst_result->severity()) {
                    $self->{last_result} = $worst_result;
                } # End if-elsif (!defined($self->{last_result}))

                # Now, we need to pop out of the loop in either of the two cases
                # 1) If we do not want to wait for all the detector threads to
                #    complete i.e wait_for_all = 0
                # 2) If the completed detector is the 'User_Input_Detector'
                if (!$wait_for_all || $process->{runid}=~/User_Input_Detector/){
                    last POLL_RUNNING_THREADS;
                }
            } # End if-elsif
        } # End PROCESS_LIST_LOOP

        # If (all):
        #     @process_list >  1      - User_Detector(s) are supplied
        #     $wait_for_all == 1      - wait for all User_Detectors
        #     $running_processes <= 1 - only 1 proc left (User_Input_Detector)
        # then we're done. Last remaining thread should be User_Input_Detector
        # thread. All User_Detector threads should be completed at this point.
        if (@process_list > 1 && $wait_for_all && $running_processes <= 1) {
            last POLL_RUNNING_THREADS;
        }

        # Adding this sleep to help with CPU usage. If we start to see thread
        # completions getting missed or seen out of order, we'll need to
        # remove this sleep.
        sleep(1);
    } # End POLL_RUNNING_THREADS

    # Now stop all remaining threads.
    $Log->log(__PACKAGE__ .': Thread(s) completed. Stopping remaining threads.');
    STOP_THREADS: foreach my $process (@process_list) {
        $process->stop();
        $process->destroy();
    } # End STOP_THREADS

    # Finally, update $self->{worst_result} if needed
    if (!defined($self->{worst_result})) {
        $self->{worst_result} = $self->{last_result};
    } elsif ($self->{worst_result}->severity() < $self->{last_result}->severity()) {
        $self->{worst_result} = $self->{last_result};
    }

    $Log->exit();
    return $self->{last_result};
} # End run()

=head2 get_last_result

In the case of the same step being executed multiple times, will return the
result of the last execution of this object.  It will not factor in any other
C<NACL::Smart> objects that may be in use.

=over

=item Arguments

None.

=item Returns

C<NATE::Result> object

=back

=cut

################################################################################
# Function    : get_last_result()
# Description :
#     If the same step is called multiple times, this routine will provide the
#     result of the last execution. While a detector can create multiple
#     NATE::Result objects, the result of each particular run is the worst
#     NATE::Result created by the detector which finished first.
#     This will return the run result of the last run executed.
# Arguments   : none
# Returns     : NATE::Result object
# Exceptions  : NATE::BaseException - if no first run
################################################################################
sub get_last_result {
    $Log->enter();
    require NATE::BaseException;
    my ($self);
    $self = shift;

    if (!defined($self->{last_result})) {
        # We haven't been run
        $Log->exit();
        NATE::BaseException->throw(
            __PACKAGE__ . '->run() has not been executed.  As such, there is ' .
            'no status to report yet. Please ensure ' . __PACKAGE__ .
            '->run() is executed prior to calling ' . __PACKAGE__ . '->last_result()'
        )
    }

    $Log->exit();
    return $self->{last_result};
} # End get_last_result()

=head2 get_worst_result

In the case of the same step being executed multiple times, will return the
worst result recorded for previous executions of this object.  It will not
factor in any other C<NACL::Smart> objects that may be in use.

=over

=item Arguments

None.

=item Returns

C<NATE::Result> object

=back

=cut


################################################################################
# Function    : get_worst_result()
# Description :
#     If the same step is called multiple times, this routine can provide the
#     worst result. While a detector can create multiple
#     NATE::Result objects, the result of each particular run is the worst
#     NATE::Result created by the detector which finished first.
#     This routine will return the worst result of all runs executed this far.
# Arguments   : none
# Returns     : NATE::Result object
# Exceptions  : NATE::BaseException - if no first run
################################################################################
sub get_worst_result {
    $Log->enter();
    require NATE::BaseException;
    my ($self);
    $self = shift;

    if (!defined($self->{worst_result})) {
        # We haven't been run
        $Log->exit();
        NATE::BaseException->throw(
            __PACKAGE__ . '->run() has not been executed.  As such, there is ' .
            'no status to report yet. Please ensure ' . __PACKAGE__ .
            '->run() is executed prior to calling ' . __PACKAGE__ . '->worst_result()'
        )
    }

    $Log->exit();
    return $self->{worst_result};
} # End get_worst_result()

################################################################################
# Function    : _generate_user_instruction()
# Description :
#     Creates the block of text to tell the user what to do next, how to
#     communicate with NACL::Smart.
# Arguments   :
#     self     - object - (Required) Current NACL::Smart object
# Returns     : scalar - text string
# Exceptions  : None
################################################################################
sub _generate_user_instruction {
    my (%args, $self, @timeout_text, $user_input_instruction, );
    %args = @_;
    $self = $args{self};

    if ($self->{detectors}) {
        $user_input_instruction = join(
            qq(\n),
            'This Manual Step has provided a NACL::Smart::Detector.',
            'Test progression should happen automatically after the below step',
            'is completed. If the Detector does not complete, enter one of the',
            'following commands to indicate how to progress on the console',
            'for --> ' . $self->{input_node} . ' :',
        );
    } else {
        $user_input_instruction = join(
            qq(\n),
            'When you have completed execution of this test step, enter the results',
            'as one of the following on the console for --> '
            . $self->{input_node} . ' :',
        );
    } # End if/else (has_detector)

    if ($self->{timeout} >= 0) {
        # User has requested a timeout. Add information to instruction list.
        @timeout_text = (
            '    TIME_EXTEND            (extend timeout by 1 hour)',
            q(),
            'Test Step Start:    ' . localtime(time()),
            'Test Step Timeout:  ' . localtime((time() + $self->{timeout})),
        );
    }

    return join(
        qq(\n),
        q(*)x21 . ' START EXECUTING THE MANUAL STEP NOW ' . q(*)x22,
        'User Input Instructions:',
        '------------------------',
        $user_input_instruction,
        q(), # blank line
        '    STEP_PASS              (for PASS)',
        '    STEP_FAIL              (for FAIL)',
        '    STEP_CONFIG            (for CONFIG)',
        @timeout_text,
        q(*)x80,
        'Step Details / User Instructions:',
        '---------------------------------',
        $self->{step_details},
        q(*)x80,
    );

} # End _generate_user_instruction()

################################################################################
# Function    : _user_detector_validation()
# Description :
#     Verifies 'detector' argument contains expected values. This is not used
#     internally, but passed to Params::Validate from within new().
# Arguments   : (positional) value of 'detector' passed to new()
# Returns     : 1 - Parameter passes validation
#               0 - Parameter fails validation
# Exceptions  : None
################################################################################
sub _user_detector_validation {
    my ($detector, );
    $detector = $_[0];

    if (ref($detector) eq 'ARRAY') {
        # List of detector objects passed in. Let's make sure each item is
        # a detector object
        foreach my $detector (@{$detector}) {
            return 0 if (!$detector->isa('NACL::Smart::Detector'));
        }
        # All items in list checked out.
        return 1;
    } else {
        # Should be a single detector object passed in
        return 1 if ($detector->isa('NACL::Smart::Detector'));
    }

    # Shouldn't get here. Something is wrong
    return 0;
} # End _user_detector_validation()

################################################################################
# Function    : _user_input_detector()
# Description :
#     This routine sets up a console monitor on FILERA and looks for specific
#     keywords from the user. In the case of steps that have no detector
#     routines, this will be the primary means of communicating test step status
#     to the .thpl . In cases where a detector method is provided, the purpose
#     of this routine is to provide a backup means of communicating with the
#     script to report test step status.
# Arguments   :
#     input_node  - scalar - (Required) Name of node we are monitoring the
#                           console for User Response
#     timeout     - scalar - (Required) Timeout to wait for user response.
#     user_text   - scalar - (Required)
# Returns     : none explicitly;
#               NATE::Result object created here will be passed back via
#               NATE::Process
# Exceptions  : NATE::BaseException - if Connectrec makes an unexpected match
################################################################################
sub _user_input_detector {
    $Log->enter();
    require Connectrec;
    require NATE::BaseException;
    # NB: Since this is a called routine via NATE::Process, we won't have access
    # to $self (aka, the Class's instance). Any parameters will need to be
    # passed in as arguments.
    my (
        %args, $connectrec, $end_str, $end_time, $log, $response,
        %user_input_dispatch_table,
    );
    %args = @_;
    if ($args{timeout} >= 0) {
        $end_time = time() + $args{timeout};
    } else {
        $end_time = 0;
    }

    # I am using 2 NATE::Log objects here. The object in $Log is the global log
    # object and will print in the xx_testcase.log. $log is a local object that
    # will print in xx_testcase/xx_User_Input_Detector.log.
    $log = NATE::Log->new();

    # Text we expect to see from the user, and their corresponding actions
    %user_input_dispatch_table = (
        STEP_PASS   => \&_user_input_detector_report_pass,
        STEP_FAIL   => \&_user_input_detector_report_fail,
        STEP_CONFIG => \&_user_input_detector_report_config,
        TIME_EXTEND => \&_user_input_detector_time_extend,
    );

    # Message to log files that we're starting.
    $log->log($args{user_text});
    $Log->log($args{user_text});

    # Create Connectrec object so we can monitor the console.
    $connectrec = Connectrec->new(
        host     => $args{input_node},
        template => 'console',
    );

    MANUAL_EXECUTION: {
        my ($timeout, );
        # Calculate new timeout value
        if ($args{timeout} >= 0) {
            $timeout = $end_time - time();
        } else {
            $timeout = $args{timeout};
        }

        # Now, wait for user response.
        $response = $connectrec->expect(
            timeout => $timeout,
            match   => [keys(%user_input_dispatch_table)],
        ) || 'STEP_CONFIG';

        # Perform proper action, based on user response.
        if ($user_input_dispatch_table{$response}) {
            $user_input_dispatch_table{$response}->(
                connectrec => $connectrec,
                end_time   => \$end_time,
            );
        } else {
            # We shouldn't get here. If we are here, Connectrec returned a
            # response that didn't match what we expected, so just in case...
            $Log->exit();
            NATE::BaseException->throw(
                "Unknown match seen: '$response'"
            );
        }
        redo MANUAL_EXECUTION if ($response eq 'TIME_EXTEND');
    } # End MANUAL_EXECUTION

    $end_str =  q(*)x28 . ' MANUAL EXECUTION ENDED ' . q(*)x28 .
        "\nResult received from execution: $response\n" .
        q(*)x80;
    $Log->log($end_str);
    $log->log($end_str);

    $Log->exit();
    return;
} # End _user_input_detector()

################################################################################
# Function    : _user_input_detector_report_config()
# Description :
#     We should get here if STEP_CONFIG is entered in User Input Detector, or
#     the User Input Detector times out.
#     Here we will create a NATE::Result::Config object and return.
# Arguments   :
#     end_time - scalar - (Required) Time (seconds since epoch) Detector stopped
#                         current test step.
# Returns     : none explicitly;
#               NATE::Result object created here will be passed back via
#               NATE::Process
# Exceptions  : NATE::BaseException - if Connectrec makes an unexpected match
################################################################################
sub _user_input_detector_report_config {
    $Log->enter();
    my (%args, );
    %args     = @_;

    if (time() >= ${$args{end_time}}) {
        # We've timed out. Reflect in logs
        $Log->log('Timeout reached. Ending with CONFIG result.');
        NATE::Result::Config->new(
            'No input seen within timeout. Stopping test with CONFIG result.',
        )->store();
    } else {
        NATE::Result::Config->new(
            'CONFIG result reported in User Input Thread',
        )->store();
    }

    $Log->exit();
    return;
} # End _user_input_detector_report_config()

################################################################################
# Function    : _user_input_detector_report_fail()
# Description :
#     We should get here if STEP_FAIL is entered in User Input Detector.
#     Here we will create a NATE::Result::Fail object and return.
# Arguments   : none
# Returns     : none explicitly;
#               NATE::Result object created here will be passed back via
#               NATE::Process
# Exceptions  : NATE::BaseException - if Connectrec makes an unexpected match
################################################################################
sub _user_input_detector_report_fail {
    $Log->enter();

    NATE::Result::Fail->new(
        'Failure reported in User Detector Thread',
    )->store();

    $Log->exit();
    return;
} # End _user_input_detector_report_fail()

################################################################################
# Function    : _user_input_detector_report_pass()
# Description :
#     We should get here if STEP_PASS is entered in User Input Detector.
#     Here we will create a NATE::Result::Pass object and return.
# Arguments   : none
# Returns     : none explicitly;
#               NATE::Result object created here will be passed back via
#               NATE::Process
# Exceptions  : none
################################################################################
sub _user_input_detector_report_pass {
    $Log->enter();

    NATE::Result::Pass->new('Step Passed with manual verification.')->store();

    $Log->exit();
    return;
} # End _user_input_detector_report_pass()

################################################################################
# Function    : _user_input_detector_time_extend()
# Description :
#     Increases timeout for User Input Detector by 1 hour.
# Arguments   : none
# Returns     : none
# Exceptions  : none
################################################################################
sub _user_input_detector_time_extend {
    $Log->enter();
    my (%args, $connectrec, );
    %args       = @_;
    $connectrec = $args{connectrec};

    if (!defined(${$args{end_time}})) {
        $Log->log('No timeout is used. Unable to extend Time Out');
        return;
    }

    # Increase timeout - Scalar reference passed in, we need to update its value
    ${$args{end_time}} += 3600;

    # Print new timeout time to console
    $Log->log(join(
        qq(\n),
        q(*)x70,
        'Timeout extended until: ' . localtime(${$args{end_time}}),
        q(*)x70,
    ));
    # Give a moment for user to hit ENTER key, or we'll see this
    # multiple times.
    sleep 1;

    # Flush the console buffer. Connectrec->resync() waits for ONTAP to
    # respond to an 'echo' command. If we're at C- or Node-shell, this
    # runs fine. If not (ie., loader/SP/etc.), use short timeout to
    # quickly proceed, and on_timeout=> to avoid a timeout exception.
    $connectrec->resync(
        timeout    => 5,
        on_timeout => sub { return 1; },
    );

    $Log->exit();
    return;
} # End _user_input_detector_time_extend()

################################################################################
#                    Closing POD
################################################################################

=head1 Example

=head2 In your .thpl

 sub detector_routine {
     # Do some checks in here.
     # Create NATE::Result object to denote status.
     return;
 } # End detector_routine()

 sub test_case {
     require NACL::Smart;
     my ($smart_step, $test_result, );

     # ... do automated stuff

     $smart_step = NACL::Smart->new(
         step_details => 'Pull link in port c0a.',
         detector     => NACL::Smart::Detector->new(
             codespec => &detector_routine,
         ),
     );
     $test_result = $smart_step->run();

     # If you want to stop on a non-pass result, check $test_result here.

     # ... do more stuff
     # ... finish test case
 } # End test_case()

=head2 In the log file

You will see the following in the log file when it is time to execute the
manual step:

 ************** START EXECUTING THE MANUAL TEST NOW ***************
 Use console connections to execute any typed test step(s).  Console connections
 are logged, but ssh connections are not. When you have completed execution,
 enter the results as one of the following on the console for
 --> csiqa-3250-09 <-- :

       TEST_PASS       (for PASS)
       TEST_FAIL       (for FAIL)
       TEST_CONFIG     (for CONFIG)

 TEST STEP DETAILS:
     Pull link in port c0a.

 Test Start: Tue Mar  3 15:34:58 2015
 Timeout:    Tue Mar  3 16:34:58 2015

 If a response is not seen prior to 'Timeout', the script will return a CONFIG result.
 **********************************************************************

If a detector routine is specified, and it reports the manual step has
successfully been performed, no user entry is required.

=head1 AUTHOR/MAINTAINER

=over

=item HA-Interconnect QA (ng-interconnect-qa@netapp.com)

=item Paul Waters (pwaters@netapp.com)

=back

 $Id: //depot/prod/DOT/dev/test/lib/NACL/Smart.pm#1 $
 Last revised $Date: 2016/11/07 $ by $Author: pwaters $

=cut


1;
