# This is basically -*- perl -*-
# $Id: lib/NACL/Smart.pm
#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
# NACL/Smart/Detector.pm
##
## @summary
##    Semi-Manual Automated Regression Test (SMART) library - Detector Object
##
## @description
##    The L<NACL::Smart|lib-NACL-Smart-pm> library facilitates the automation of manual tests,
##    whether they be High Touch tests or Low Touch tests. This library
##    provides definitions for the detectors used within NACL::Smart.
##
##    This library is probably not much use on its own. For more information,
##    please see L<NACL::Smart|lib-NACL-Smart-pm>.
##
## @synopsis
##   use NACL::Smart::Detector;
#
##
## @example
##   use NACL::Smart::Detector;
##   $detector   = NACL::Smart::Detector->new(
##       codespec => \&_my_detector_subroutine,
##       args     => {
##           arg1 => 'value1,
##           ...
##       },
##   );
##
##   -- or --
##
##   $detector = NACL::Smart::Detector->new(
##       codespec => '/path/to/some/thpl',
##       args => {
##           arg1 => 'value1',
##       },
##   );
##
#
## @burt 894864 Smart.pm - Initial Check-In
## @burt 1073145 Typo prevents use of HASHREF for args
## @burt 1082771 Enhancement to support codespec
## @burt 1096354 Remove 'detector' parameter support
## @burt 1102904 Move Iconnect::Smart to NACL::Smart
#
## @keywords
##   interconnect; ic; smart; manual
##
## @status public
## @pod here
##
## @author Interconnect QA (dl-interconnect-qa@netapp.com)
## @author Paul Waters (pwaters@netapp.com)

################################################################################
#                              Start POD here
################################################################################

=head1 NAME

NACL::Smart::Detector - Semi-Manual Automated Regression Detector class

=head1 SYNOPSIS

 use NACL::Smart::Detector;
 $test_step = NACL::Smart::Detector->new(
     codespec => \&_my_detector_routine,
     args     => {
        optional => 'argument list',
        ...
     },
 );

=head1 DESCRIPTION

The NACL::Smart library facilitates the automation of manual tests, whether
they be High Touch tests or Low Touch tests. This library defines the detectors
used within NACL::Smart.

=head1 EXCEPTIONS

=over

=item C<NATE::Exceptions::Argument>

=back

=cut

package NACL::Smart::Detector;

# Compiler Directives
use strict;
use warnings FATAL => 'all';

# NATE imports
use List::MoreUtils qw(any);
use Params::Validate qw(:all);
use Tharn qw($Log);

# NATE Exceptions
use NATE::Exceptions::Argument;

BEGIN {
    Params::Validate::validation_options(
        on_fail => sub {
            NATE::Exceptions::Argument->throw(shift);
        }
    );
}

=head1 METHODS

=head2 new

    $smart_detector = NACL::Smart::Detector->new(
        codespec => \&subroutine_reference,
        args     => { key => value, },
    );

--or--

    $smart_detector = NACL::Smart::Detector->new(
        codespec => 'relative/path/to/thpl',
        args     => { key => value, },
    );

Creates a Smart Detector object that will be passed to NACL::Smart.
Detectors are routines that can test/poll for completion of a test step.

=over

Arguments

=over

=item C< codespec =E<gt> subroutine reference >
=item C< codespec =E<gt> relative/path/to/thpl >

(Required) Codespec - Subroutine or .thpl that tests/polls for the
completion of a test step. The codespec should:

- Create its own NATE::Log object for logging. This will ensure log messages
from this detector go into its own logfile.

- Create NATE::Result objects before exiting to return the status of this test
step.

- Create a NATE::ParamSet object to access NATE parameters

=item C< args =E<gt> hash reference >

(Optional) List of any required arguments to be passed to the detector.

=back

=item Returns

C<NACL::Smart::Detector> object.

=back

=cut

################################################################################
# Function    : new()
# Description :
#     Constructor to create new Smart Detector object
# Arguments   :
#     codespec - codespec - (required) Codespec for detector routine/thpl
#     detector - coderef  - (deprecated) Subroutine that queries for status of
#         test step.
#     args     - arrayref - (optional) List of arguments that will be passed
#         to the subroutine when NATE::Process starts new thread.
# Returns     : NACL::Smart::Detector object
# Exceptions  : NATE::Exceptions::Argument - if invalid argument type passed in,
################################################################################
sub new {
    $Log->enter();
    require NATE::Exceptions::Argument;

    my (%args, $class, $self, );
    $class = shift;    # because we're doing OO
    $self  = {};       # this holds our class instance data

    # Ah-Choo
    bless($self, $class, );

    # Validate Arguments.
    %args = validate(
        @_,
        {
            # Required
            codespec => {
                type     => CODEREF | SCALAR,
                optional => 0,
            },
            # Optional
            args => {
                type      => HASHREF,
                optional  => 1,
            },
        },
    ); # End Params::Validate::validate() call

    # codespec is required, but could be either a CODEREF or a SCALAR which
    # indicates path to a .thpl.
    _verify_and_set_codespec($self, %args, );

    # User-supplied arguments to pass to codespec
    $self->{args} = $args{args};

    $Log->exit();
    return $self;
} # End new()

################################################################################
# Private Routines
################################################################################

################################################################################
# Function    : _verify_and_set_codespec()
# Description :
#     Constructor to create new Smart Detector object
# Arguments   :
#     [positional 0] - Reference to current instance (self)
#     codespec - codespec - (Required) Codespec for detector routine/thpl
#         test step.
# Returns     : None
# Exceptions  : NATE::Exceptions::Argument - if invalid argument combination passed in
################################################################################
sub _verify_and_set_codespec {
    my (%args, $self, );
    $self = shift;
    %args = @_;

    # First, we need to determine what was passed in.
    if (ref($args{codespec}) eq 'CODE') {
        # Passed in a subroutine reference. Nothing else needed.
        $self->{codespec} = $args{codespec};
        return;
    }

    # So, it looks like we've gotten an external script to use as codespec.
    # Let's see if we can find it.
    FIND_CODESPEC: {
        require NATE::ParamSet;
        my (@natelib, $paramset, );
        $paramset = NATE::ParamSet->new( global => 1 );
        @natelib = split(qr/;/, $paramset->get('NATE_LIB'));

        foreach my $natelib (@natelib) {
            if (-e "$natelib/$args{codespec}") {
                # We found the file. Jump out of check.
                last FIND_CODESPEC;
            }
        }

        # If we get here, we could not find the file specified in codespec.
        NATE::Exceptions::Argument->throw(
            "Unable to find the file '$args{codespec}' within NATE_LIB.\n" .
            "NATE_LIB = \n" .
            join(qq(\n), @natelib)
        );
    } # End FIND_CODESPEC

    # If we get here, we found the file in FIND_CODESPEC above. Set instance
    # variable and return.
    $self->{codespec} = $args{codespec};
    return;
} # End _verify_and_set_codespec()

=head1 AUTHOR/MAINTAINER

=over

=item HA-Interconnect QA (dl-interconnect-qa@netapp.com)

=item Paul Waters (pwaters@netapp.com)

=back

 $Id: //depot/prod/DOT/dev/test/lib/NACL/Smart/Detector.pm#1 $
 Last revised $Date: 2015/04/18 $ by $Author: pwaters $

=cut


1;
