
# $Id
#
# Copyright (c) 2012-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary CgtXml Module
## @author  anbumozh@netapp.com, dl-nacl-dev@netapp.com
## @status public
## @pod here

=head1 NAME

NACL::Tools::CGT::CgtXml

=head1 DESCRIPTION

C<NACL::Tools::CGT::CgtXml> contains functions related to generating
component datastructure from help_xml files

=cut 

package NACL::Tools::CGT::CgtXml;

use strict;
use warnings;
use Params::Validate qw(validate_with validate_pos :types );
use NATE::Log qw(log_global);
use Data::Dumper qw(Dumper);
use File::Find;
use XML::Simple;
use NACL::Tools::CGT::CGTUtils qw(_determine_split_list _update_method_name _typedef_cli_zapi );
use NACL::APISet::Schema;
use NACL::APISet::Exceptions::MethodNotFoundException qw(:try);
use Array::Utils qw(:all);
use NACL::Database;
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw( processdirs
        determine_fields_datatype
        parse_xmlfile
        determine_primary_keys
        _get_help_xml_path
        cli_to_zapi
    );
}

sub _findsubdirs {
    $Log->enter() if $may_enter;
    my ($dir) = @_;
    my @dirs;
    find(
        sub {

            if (-d && !-l) {
                push @dirs, $File::Find::name;
            }
        },
        "$dir"
    );
    $Log->exit() if $may_exit;
    return @dirs;
} ## end sub _findsubdirs

sub _findfiles {
    $Log->enter() if $may_enter;
    my @outlist;
    my ($dir) = @_;
    opendir(DIR, $dir) or die "couldn't open $dir: $!\n";
    my @files = readdir DIR;
    closedir DIR;
    foreach my $file (@files) {
        my $filename = $dir . "/" . $file;
        if (-f $filename
            && !-l $filename
            && $filename =~ /\.xml$/ ) {
            push @outlist, $filename;
        }
    }
    $Log->exit() if $may_exit;
    return @outlist;
} ## end sub _findfiles

#
# This function parses all the .xml files
# in $build_root/final/bedrock/export/common/cli_zapi
# and it returns a hash which contains the cli command name as key and
# zapi api as value
#
sub cli_to_zapi {
    $Log->enter() if $may_enter;

    my ($build_root) = @_;
    my $row = {};
    my ($nacl_dbh, %finalhash);
    my $var = NATE::ParamSet->new(global => 1);
    $nacl_dbh = NACL::Database->new();
    if ( $nacl_dbh ) {
        my $cli_zapi = $nacl_dbh->_query_cli_zapi(build_root => $build_root);
        if ( keys %{ $cli_zapi } ) {
            $Log->exit() if $may_exit;
            $Log->comment("\n Getting cli zapi mapping from db");
            return $cli_zapi;
        }
    }

    my $dir = $build_root . "/final/bedrock/export/common/cli_zapi";
    #The following will ensure that if cli_zapi xml files are not
    #accessible, for some reason, it would return null and
    #the script would run in CLI

    return
        if (!-e $dir);
    
    local $XML::Simple::PREFERRED_PARSER = 'XML::Parser';
    my @files = _findfiles($dir);
    my $ref;
    foreach my $file (@files) {
        eval { $ref = XMLin($file); };
        if ($@) {
            $Log->warn(
                "\n Unable to parse $file, raise a product burt for this"
                    . "\n Errors: $@ ");
            next;
        }
        if ($ref->{command}->{name}) {
            $finalhash{$ref->{command}->{name}}
                = $ref->{command}->{'zapi-name'};
        } elsif (ref $ref->{command}) {
            my %temp_hash = %{$ref->{command}};
            foreach my $key (keys %temp_hash) {
                $finalhash{$key} = $temp_hash{$key}->{'zapi-name'};
            }
        }
    }
    #Enabling the logging to db only if Nate uses MySQL
    if ( $nacl_dbh ) {
        $row->{build_root} = $build_root;
        $row->{ds} = \%finalhash;
        $nacl_dbh->_insert_table(row => $row,
                  table_name => "cli_zapi_ds");
    }

    $Log->exit() if $may_exit;
    return \%finalhash;
} ## end sub cli_to_zapi

=head1 METHODS

=head2 processdirs

       $ds = processdirs(
                build_root    => $build_root,
                start_cmd    => $start_cmd,
                skip_subdirs => 1);

    This function will parse the help_xml files from buildroot ,generates
	the component datastructure and returns a hashref
	If start_cmd is passed , it limits the parsing of xml files for all 
	subcommands  within it, else it parses all the xml files 

=over

=item Options

=over

=item C<< build_root => $build_root >>

(Required)
Build root path , Example: /x/eng/rlse/DOT/mainN

=item C<< start_cmd => $start_cmd >>

(Optional)
 The command node to limit the search, Example: "storage aggregate"

=item C<< skip_subdirs => 0|1 >>

(Optional)
Defaults to 0 , if 1 it parses xml files from the top level
directory , subdirectories would be skipped

=back

=back

=cut

sub processdirs {
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            build_root => {type => SCALAR},
            start_cmd  => {type => SCALAR | UNDEF},
            schema     => {
                type     => OBJECT,
                optional => 1,
                isa      => 'NACL::APISet::Schema',
            },
            skip_subdirs => {type => SCALAR, default => 0},
        }
    );
    my (@dirs,           $ref,     $fields,     %ds,
        $method_hashref, @methods, $vs_context, $pk,
        $exception,      $dirname, $schema
    );
    my ( $dir, $cdef);
    my $skip_subdirs = $opts{skip_subdirs};
    my $start_cmd    = $opts{start_cmd};
    my $build_root   = $opts{build_root};
    my $root_dir     = _get_help_xml_path(build_root => $build_root);

    if ($start_cmd) {
        $start_cmd =~ s/\s+/\//g;
        $dir = $root_dir . $start_cmd;
    } else {
        $dir = $root_dir;
    }
    $schema = $opts{schema};
    if (!$schema) {
        my $cdef_path
            = $build_root . "/final/bedrock/export/common/cdefsprod";
        $schema = NACL::APISet::Schema->new();
        my $files = $schema->read_cdef_dir(directory => $cdef_path);
        $schema->load(
            files            => $files,
            path             => $build_root,
            cdefs_from_build => 1
        );
    }
    if (!-d $dir) {
        NATE::BaseException->throw("\n $dir doesnt exist");
    }
    if ($skip_subdirs) {
        push @dirs, $dir;
    } else {
        @dirs = _findsubdirs($dir);
    }
    my $mapping = cli_to_zapi($build_root);
    foreach my $subdir (@dirs) {
        my $cmddir = $subdir;
        $cmddir =~ /gendocs\/(.*)$/;
        $cmddir = $1;
        next if (!$cmddir);
        $cmddir =~ s/\// /g;
        my @files = _findfiles($subdir);
        $#methods = -1;

        foreach my $xml_file (@files) {
            $exception = 0;
            if (   $xml_file =~ /\/directory\.xml$/
                || $xml_file =~ /\/new\.xml$/
                || $xml_file =~ /\/\.xml$/) {
                next;
            }
            my $cmd = $xml_file;
            $cmd =~ /gendocs\/(.*)\.xml/g;
            $cmd = $1;
            my $actual_cmd = $1;
            $actual_cmd =~ s/\// /g;
            $cmd        =~ s/\//_/g;
            $cmd        =~ s/-/_/g;
            try {
                 $cdef = $schema->get_command_definition(command => $cmd);
            }
            catch NACL::APISet::Exceptions::MethodNotFoundException with {
                $exception = 1;
            };
            if ($exception) {
                $Log->comment("\n cdef not found for $cmd");
                next;
            }
            my $alias_map = $cdef->get_alias_map();
            $method_hashref = parse_xmlfile(xml_file => $xml_file,
                                            alias_map => $alias_map);

            if ($method_hashref) {
                $method_hashref->{'zapi-command'} = $mapping->{$actual_cmd}
                    if ($mapping
                    && $mapping->{$actual_cmd});
                push @methods, $method_hashref;
            }
        }
        my @copy_methods = @methods;
        $ds{$cmddir} = \@copy_methods
            if (scalar @copy_methods);
    }
    _determine_other_details_for_xml(\%ds)
        if (%ds);
    $Log->exit() if $may_exit;
    return \%ds;
} ## end sub processdirs

=head2 parse_xmlfile

       $ds = process_xmlfile(
                xml_file    => $xml_file_name,
                schema      => $schema_obj);

    This function will parse the xml file which is passed ,generates
	the component datastructure and returns

=over

=item Options

=over

=item C<< xml_file => $xml_file_name >>

(Required)
Name of the xml file

=back

=back

=cut

sub parse_xmlfile {
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@_,
        spec   => {
                   xml_file => {type => SCALAR,},
                   alias_map => {type => HASHREF|UNDEF, optional => 1},
                  }
    );
    my ($ref, $method_hashref, $fields, $pk, $row) = {};
    my $vs_context = 0;
    my $has_alias_fields = 0;
    my $needs_update = 0;
    my $xml_file = $opts{xml_file};
    my $alias_map = $opts{alias_map};
    if ( $alias_map
        && keys %{ $alias_map } ) {
        $has_alias_fields = 1;
    }
    #Enabling the logging to db only if Nate uses MySQL
    my $nacl_dbh = NACL::Database->new();
    if ( $nacl_dbh ) {
        $method_hashref = $nacl_dbh->_query_component_ds(xml_file => $xml_file);
        if ( $method_hashref
           && keys %{ $method_hashref } ) {
           if (!$has_alias_fields) {
               $Log->exit() if $may_exit;
               $Log->comment("\n Getting ds from db");
               return $method_hashref;
           } else {
               if ( $method_hashref->{has_alias_fields} == $has_alias_fields ) {
                   $Log->exit() if $may_exit;
                   $Log->comment("\n Getting ds from db");
                   return $method_hashref;
               } else {
                   $needs_update = 1
               }
           }
        }
    }
    local $XML::Simple::PREFERRED_PARSER = 'XML::Parser';
    if (!-e $xml_file) {
        $Log->debug("\n Unable to access $xml_file");
        return;
    }
    eval { $ref = XMLin($xml_file); };
    if ($@) {
        $Log->warn(
            "\n Unable to parse $xml_file, raise a product burt for this"
                . "\n Errors: $@ ");
        $Log->exit() if $may_exit;
        return;
    }
    if ($ref->{vserver} eq "true") {
        $vs_context = 1;
    }
    $method_hashref->{is_CS} = 0;
    if ($ref->{'show-fields'}) {
        $method_hashref->{is_CS}        = 1;
        if (!keys %{$ref->{parameters}}) {
            $method_hashref->{is_singleton} = 1
        } else {
            my $extra_filters_hashref = _determine_extra_filter_fields(show_fields => $ref->{'show-fields'}->{field},
                                               parameters  => $ref->{parameters}->{parameter});
            $method_hashref->{extra_filter_fields} = $extra_filters_hashref
                if ( keys %{ $extra_filters_hashref } );
        }
        $fields = determine_fields_datatype(
            parameters   => $ref->{'show-fields'}->{field},
            is_CS => 1,
            alias_map => $opts{alias_map},
        );
        $pk = _determine_primary_keys(show_fields => $ref->{'show-fields'},
                                      alias_map   => $opts{alias_map});
           
        $method_hashref->{primary_keys} = $pk->{primary_keys};
        $method_hashref->{optional_primary_keys} = $pk->{optional_primary_keys};
    } elsif ($xml_file =~ /\/show\.xml$/
        && !keys %{$ref->{'show-fields'}}) {
        $method_hashref->{is_CS}        = 1;
        $method_hashref->{is_singleton} = 1
            if (!keys %{$ref->{parameters}});
        $fields = {};
        $method_hashref->{primary_keys} = [];
        $Log->warn($ref->{name}
                . ", doesnt take -fields option, "
                . "raise a burt of type nacl and subtype nacl_core");
    } else {
	if ($ref->{parameters}->{parameter}) {
            $fields = determine_fields_datatype(
		parameters   => $ref->{parameters}->{parameter},
		vs_context   => $vs_context,
		is_CS        => 0,
		alias_map    => $opts{alias_map},
		);
            $pk = _determine_primary_keys(show_fields => $ref->{'parameters'},
				alias_map   => $opts{alias_map});

            $method_hashref->{primary_keys} = $pk->{primary_keys};
            $method_hashref->{optional_primary_keys} = $pk->{optional_primary_keys};
	}
    }
    if ( $alias_map ) {
        while (my ($old, $new) = each(%{ $alias_map })) {
            if( $fields->{$new} ) {
               my %temp_hash = %{ $fields->{$new} };
               $temp_hash{field} = $old;
               $fields->{$old} = \%temp_hash;
            }
        }
    }
    ## w.r.t burt#820500 the fields "foo-str" exists in addition to the
    ## normal field "foo". This is hidden and there are cases where the ZAPI mapping
    ## is present only for the "foo-str" field but not for "foo".
    ## In such cases, we should reuse that ZAPI mapping.

    foreach my $fields_ (keys %{$fields}){
       if($fields_ !~ /nacl_value_conversion/
          && ! defined $fields->{$fields_}->{'zapi-mapping'}
          && defined $fields->{"$fields_".'-str'}->{'zapi-mapping'}){
          $fields->{$fields_}->{'zapi-mapping'} =
                $fields->{"$fields_".'-str'}->{'zapi-mapping'};
          $fields->{$fields_}->{'zapi-typedef-info'} =
                $fields->{"$fields_".'-str'}->{'zapi-typedef-info'};
       }
    }

    $method_hashref->{fields} = $fields;
    my $desc = $ref->{shortdesc};
    $method_hashref->{description} = $desc
        if ($desc && !ref $desc);
    $method_hashref->{vserver_context} = $vs_context;
    my $command = $ref->{name};
    $method_hashref->{actual_cmd} = $command;
    $command =~ s/ /_/g;
    $command =~ s/-/_/g;
    $method_hashref->{command} = $command;
    $method_hashref->{has_alias_fields} = $has_alias_fields;
    if ($fields->{_nacl_value_conversion}) {
        $method_hashref->{zapi_value_conversion}
        = delete $fields->{_nacl_value_conversion};
    }
    $row->{xml_file} = $xml_file;
    $row->{ds} = $method_hashref;
    #Enabling the logging to db only if Nate uses MySQL
    if ( $nacl_dbh ) {
        if ( $needs_update ) {
            $nacl_dbh->_update_table(row => $row,
                  table_name => "command_ds");
        } else {
            $nacl_dbh->_insert_table(row => $row,
                  table_name => "command_ds");
        }
    }
    $Log->exit() if $may_exit;
    return $method_hashref;
} ## end sub parse_xmlfile

=head2 determine_fields_datatype

       $ds = determine_fields_datatype(
                xml_file    => $xml_file_name,
                schema      => $schema_obj);

    This function will parses the parameters or show-fields hash  and return 
	a hashref which contains field datatype

=over

=item Options

=over

=item C<< parameter => $parameters >>

(Required)
show-fields or parameters hashref

=back

=back

=cut

sub determine_fields_datatype {
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            parameters   => {type => HASHREF},
            is_CS        => {type => SCALAR},
            vs_context   => {type => SCALAR, default => 0},
            alias_map => {type => HASHREF|UNDEF},
        }
    );
    my ($datatype, $description, $name, %parameters,
        %conversion_hash);
    my $fields    = {};
    my %temp_hash = %{$opts{parameters}};
    if (exists $temp_hash{name} && !ref $temp_hash{name}) {
        $name = $temp_hash{name};
        $parameters{$name} = $opts{parameters};
    } else {
        %parameters = %{$opts{parameters}};
    }
    my $cmd_type = ($opts{is_CS} == 1) ? 'show' : 'non_show';

    my %all_conversions = (
        '<integer>[KB|MB|GB|TB]' =>
            {zapi_type => 'integer', non_show => 'convert_to_bytes'},
        '<integer>[KB|MB|GB|TB|PB]' =>
            {zapi_type => 'integer', non_show => 'convert_to_bytes'},
        'cron_dayofweek' =>
            {zapi_type => 'integer', non_show => 'cron_time_integer',
             show => 'integer_cron_time'},
        'cron_dayofmonth' =>
            {zapi_type => 'integer', non_show => 'cron_time_integer',
             show => 'integer_cron_time'},
        'cron_month' =>
            {zapi_type => 'integer', non_show => 'cron_time_integer',
             show => 'integer_cron_time'},
        'cron_hour' =>
            {zapi_type => 'integer', non_show => 'cron_time_integer',
             show => 'integer_cron_time'},
        'cron_minute' =>
            {zapi_type => 'integer', non_show => 'cron_time_integer',
             show => 'integer_cron_time'},
        '[<integer>h][<integer>m][<integer>s]' => {
            zapi_type => 'integer',
            'show'    => 'sec_to_hrminsec',
            non_show  => 'hrminsec_to_sec'
        },
        'percent' => {
            zapi_type => 'integer',
            'show'    => 'add_percentage',
            non_show  => 'remove_percentage'
        },
        'Hex Integer' => {
            zapi_type => 'integer',
            'show'    => 'decimal_to_hex',
            non_show  => 'hex_to_decimal'
        },
        '<seconds>|[<d> days] <hh>:<mm>[:<ss>]' =>
            {zapi_type => 'integer', 'show' => 'sec_to_day_hrminsec',},
        'integer' => {
            zapi_type => 'Octal Integer',
            'show'    => 'decimal_to_octal',
            non_show  => 'octal_to_decimal'
        },
        'on|off' => {
            zapi_type => 'boolean',
            'show'    => 'on_off_boolean',
            non_show  => 'on_off_boolean'
        },
        'yes|no' => {
            zapi_type => 'boolean',
            'show'    => 'yes_no_boolean',
            non_show  => 'yes_no_boolean'
        },
        'enabled|disabled' => {
            zapi_type => 'boolean',
            'show'    => 'enable_disable',
            non_show  => 'enable_disable'
        },
        'Date' => {
            zapi_type  => 'integer',
            'show'     => 'timestamp_to_string',
            'non_show' => 'string_to_timestamp'
        },
        'MM/DD/YYYY HH:MM:SS' => {
            zapi_type  => 'integer',
            'show'     => 'timestamp_to_mmddyyyy',
            'non_show' => 'mmddyyyy_to_timestamp'
        },
        'MM/DD HH:MM:SS' =>
            {zapi_type => 'integer', 'show' => 'timestamp_to_mmdd'},
        'unix perm' =>
            {zapi_type => 'string', 'show' => 'unix_perm_to_str'},
        'Language code' =>
            {zapi_type => 'string', 'show' => 'language'}
    );
    # The following if condition is required for command which
    # runs in vserver context , we dont need to store zapi equivalent
    # for vserver field
    if ( exists $parameters{vserver}
         && $opts{vs_context} ) {
        $datatype    = $parameters{vserver}->{list};
        $datatype    = _is_array($datatype);
        $description = _extract_short_desc($parameters{vserver});
        $fields->{vserver}->{field}       = 'vserver';
        $fields->{vserver}->{is_array}    = $datatype;
        $fields->{vserver}->{description} = $description;
        delete $parameters{vserver}
    }
    foreach my $parameter (keys %parameters) {
        $datatype    = $parameters{$parameter}->{list};
        $datatype    = _is_array($datatype);
        $description = _extract_short_desc($parameters{$parameter});
        $name        = $parameter;
        $fields->{$name}->{field}       = $parameter;
        $fields->{$name}->{is_array}    = $datatype;
        $fields->{$name}->{description} = $description;

        $fields->{$name}->{'zapi-mapping'}
            = $parameters{$parameter}->{'zapi-element'}
            if $parameters{$parameter}->{'zapi-element'};
        if ($parameters{$parameter}->{'zapi-typedef-info'}
            && ref $parameters{$parameter}->{'zapi-typedef-info'} eq "HASH") {
            $fields->{$name}->{'zapi-typedef-info-pod'}
                = $parameters{$parameter}->{'zapi-typedef-info'};
            $fields->{$name}->{'zapi-mapping'} = _typedef_cli_zapi($fields->{$name});
        } elsif($parameters{$parameter}->{'zapi-typedef-info'}
            && ref $parameters{$parameter}->{'zapi-typedef-info'} eq "ARRAY") {
            $fields->{$name}->{'zapi-typedef-info-pod'}
                = $parameters{$parameter}->{'zapi-typedef-info'}[0];
            $fields->{$name}->{'zapi-mapping'} = _typedef_cli_zapi($fields->{$name});
        }

        my $ui_name = $parameters{$parameter}->{type_uiname};
        if ($parameters{$parameter}->{'zapi-element-type'}) {
            my $zapi_type = $parameters{$parameter}->{'zapi-element-type'};
            if (exists $all_conversions{$ui_name}
                && $all_conversions{$ui_name}{zapi_type} eq $zapi_type) {
                my $conversion_type = $all_conversions{$ui_name}{$cmd_type};
                if (defined $conversion_type) {
                    _update_conversion_hash(
                        type         => $conversion_type,
                        field        => $name,
                        convert_hash => \%conversion_hash,
                        alias_map    => $opts{alias_map},
                    );
                }
            } elsif ( $cmd_type eq "show"
                      && $parameters{$parameter}->{'zapi-element-value'}) {
                my $zapi_value
                    = $parameters{$parameter}->{'zapi-element-value'};
                $zapi_value =~ s/\s+//g;
                $ui_name    =~ s/\s+//g;
                my @zapi_values = split(/\|/, $zapi_value);
                my @ui_values   = split(/\|/, $ui_name);

                if (@zapi_values == @ui_values) {
                    my $temp_ui_name    = $ui_name;
                    my $temp_zapi_value = $zapi_value;

                    $temp_ui_name =~ s/-/_/g;
                    if ( $cmd_type eq 'show'
                         && $temp_ui_name eq $zapi_value) {
                        _update_conversion_hash(
                            type         => "hyphenate_value",
                            field        => $name,
                            convert_hash => \%conversion_hash,
                            alias_map    => $opts{alias_map},
                        );
                        next;
                    }
                    $temp_ui_name = $ui_name;
                    my @value = map { ucfirst($_) } @zapi_values;
                    $temp_zapi_value = join("|", @value);
                    if ($temp_zapi_value eq $ui_name) {
                        _update_conversion_hash(
                            type         => "ucfirst",
                            field        => $name,
                            convert_hash => \%conversion_hash,
                            alias_map    => $opts{alias_map},
                        );
                        next;
                    }
                    $temp_ui_name = $ui_name;
                    if (uc($zapi_value) eq $temp_ui_name) {
                        _update_conversion_hash(
                            type         => "uc",
                            field        => $name,
                            convert_hash => \%conversion_hash,
                            alias_map    => $opts{alias_map},
                        );
                        next;
                    }
                }
            }
        }
    }
    $Log->exit() if $may_exit;
    $fields->{_nacl_value_conversion} = \%conversion_hash
        if (%conversion_hash);

    return $fields;
} ## end sub determine_fields_datatype

#9/7/2012
#Private function which extract short description
#
sub _extract_short_desc {
    $Log->enter() if $may_enter;
    my ($parameter) = @_;
    my ($desc, @desc);
    $desc = $parameter->{shortdesc};
    my $uiname = $parameter->{type_uiname};
    if (!ref $desc) {
        if ($parameter->{isrange} eq "true") {
            $desc
                .= " ranges from $parameter->{rangelow} to $parameter->{rangehigh}";
        } elsif ($uiname && $uiname =~ /\|/ && $uiname !~ /\[.*\|.*\]/) {
            $desc .= " possible value(s) are, $uiname";
        }
        $desc =~ s/\|/,/g;
        $desc =~ s/^\s+//g;
        $desc =~ s/^\s$//g;
        @desc = _determine_split_list($desc);
    }
    $Log->exit() if $may_exit;
    return \@desc;
} ## end sub _extract_short_desc

#
#Private function which extract long description
#This function is not called anywhere,but we can
#switch to long desc if required
#
sub _extract_long_desc {
    my ($parameter) = @_;
    my $description;
    my $desc = $parameter->{description};
    if (!defined $desc) {
        $description = $desc->{shortdesc};
    } elsif (ref $desc eq "HASH" && keys %$desc) {
        if (ref $desc->{content} eq "ARRAY") {
            my @content = @{$desc->{content}};
            $description = join(" ", @content);
        } elsif (!ref $desc->{content}) {
            $description = $desc->{content};
        } else {
            $Log->debug("Unable to parse " . ref $desc->{content});
        }
        if (defined $desc->{varname}) {
            if (ref $desc->{varname} eq "ARRAY") {
                $description .= ",possible values are "
                    . join(",", @{$desc->{varname}});
            } elsif (!ref $desc->{varname}) {
                $description .= ",possible value are " . $desc->{varname};
            } else {
                $Log->debug("Unable to parse " . ref $desc->{content});
            }
        }
    } elsif (ref $desc eq "ARRAY") {
        $description = join(" ", @{$desc});
    } elsif (!ref $desc) {
        $description = $desc;
    }
    $description =~ s/\s{2,}//g;
    return $description;
} ## end sub _extract_long_desc

#
#Private function which returns back whether a field is array or not
#
sub _is_array {
    $Log->enter() if $may_enter;
    my ($datatype) = @_;
    if ($datatype == 0) {
        $Log->exit() if $may_exit;
        return 0;
    } else {
        $Log->exit() if $may_exit;
        return 1;
    }
} ## end sub _is_array

#
#Private function which determines primary keys
#
sub _determine_primary_keys {
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@_,
        spec   => {
                   show_fields => {type => HASHREF,},
                   alias_map => {type => HASHREF|UNDEF},
                  }
    );
    my $fields = $opts{show_fields};
    my ( @pks, @optional_pks, %reverse_alias_map);
    my @keys;
    my %fields = %{$fields};
    my $alias_map = $opts{alias_map};
    if ( $alias_map ) {
        %reverse_alias_map = reverse %{ $alias_map };
    }
    if (exists $fields{field}->{name} && !ref $fields{field}->{name}) {
        if ($fields{field}->{'is-key'} == 1) {
           my $name = $fields{field}->{name};
           if ( $alias_map ) {
              if ( $reverse_alias_map{$name} ) {
                 push( @optional_pks, $name );
                 push( @optional_pks, $reverse_alias_map{$name} );
              } else {
                 push( @pks, $name );
              }
          }
       }
    } else {
        my %field = %{$fields{field}};
	%field = %{$fields{parameter}} if (! %field);
        grep {
            if (ref $field{$_} eq "HASH" && defined $field{$_}->{'is-key'} && $field{$_}->{'is-key'} == 1
                && $field{$_}->{'is-hidden'} eq "false") {
                if ($alias_map
                    && $reverse_alias_map{$_} ) {
                    push( @optional_pks, $_ );
                    push( @optional_pks, $reverse_alias_map{$_} );
                } else {
                    push(@pks, $_);
                }
            }
        } keys %field;
    }
    my %return_hash;
    $return_hash{primary_keys} = \@pks;
    $return_hash{optional_primary_keys} = \@optional_pks;
    $Log->exit() if $may_exit;
    return \%return_hash;
} ## end sub _determine_primary_keys

#
# Private function which determins whether the method
# can use common_validate_with, instance method or not
# etc..
#
sub _determine_other_details_for_xml {
    $Log->enter() if $may_enter;
    my (@show_array, @primary_keys, $component_contains_create);
    my ($component_list) = @_;
    while (my ($component_name, $component_arr_ref) = each %{$component_list})
    {
        my $component_contains_create;
        my $api = $component_name;
        $api =~ s/\s|-/_/g;
        my $show_api  = $api . "_show";
        my $start_api = $api . "_start";
        @show_array = grep { $_->{command} eq $show_api } @$component_arr_ref;
        if (scalar @show_array) {
            if ($show_array[0]->{primary_keys}) {
                @primary_keys = @{$show_array[0]->{primary_keys}};
            }
        }
        my @create_array
            = grep { $_->{command} =~ /create/ } @$component_arr_ref;
        if (scalar @create_array) {
            $component_contains_create = 1;
        }
        foreach my $command_hashref (@$component_arr_ref) {
            my @fields = keys %{$command_hashref->{fields}};
            _update_method_name(
                component_name => $component_name,
                method_ds      => $command_hashref,
                primary_keys   => \@primary_keys
            );
            unless (scalar @show_array) {
                $command_hashref->{is_static}   = 1;
                $command_hashref->{can_use_cvw} = 0;
                next;
            }
            $command_hashref->{is_static} = 0;

            # Can use _common_validate_with
            my $can_use_cvw = 1;
            if (scalar @primary_keys) {
                my %fields_hash;
                map { $fields_hash{$_} = 1 } @fields;
                foreach my $primary_key (@primary_keys) {
                    $can_use_cvw = 0
                        unless (exists $fields_hash{$primary_key});
                }
            }
            $command_hashref->{can_use_cvw} = $can_use_cvw;

          # can_use_cvw = 0 implies it's static, but $can_use_cvw = 1 does not
          # mean it's an instance variable. For 'create' or 'start' methods
          # $can_use_cvw will be 1, but are static methods. Hence we need
          # to store both
            if (   ($command_hashref->{command} =~ /create/)
                or (!$can_use_cvw)
                or (    ($command_hashref->{command} eq $start_api)
                    and (!$component_contains_create))
                ) {
                $command_hashref->{is_static} = 1;
            }
        }
    }
    $Log->exit() if $may_exit;
} ## end sub _determine_other_details_for_xml

#
# Private routine which returns help_xml path from build_root
#
sub _get_help_xml_path {
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            build_root    => {type => SCALAR},
            change_number => {type => SCALAR, optional => 1},
        }
    );
    my $build_root    = $opts{build_root};
    my $change_number = $opts{change_number};
    my $path;
    if (   $build_root =~ /daemon/i
        && $change_number) {
        my $root = $build_root . "/.snapshot/" . $change_number;
        $build_root = $root
            if (-e $root);
    }
    $path = $build_root
        . "/ng_docs/bedrock/export/common/gendocs/";
    $Log->comment("\n XML path used: " . $path);
    $Log->exit() if $may_exit;
    return $path;
} ## end sub _get_help_xml_path

#
#Private function which updates conversion hash ref
#

sub _update_conversion_hash {
    my %opts = validate_with(
        params => \@_,
        spec   => {
            field        => {type => SCALAR},
            convert_hash => {type => HASHREF},
            type         => {type => SCALAR},
            alias_map    => {type => HASHREF|UNDEF}
        },
    );
    my $hashref = $opts{convert_hash};
    my @arr;
    my $field = $opts{field};
    if (exists $hashref->{$opts{type}}) {
        push @{$hashref->{$opts{type}}}, $field;
    } else {
        push @arr, $field;
        $hashref->{$opts{type}} = \@arr;
    }
    
    if ( $opts{alias_map} ) { 
        my %reverse_alias_map = reverse %{ $opts{alias_map} };
        if (  $reverse_alias_map{$field} ) {
            push @{$hashref->{$opts{type}}}, $opts{alias_map}->{$field};
        }
    }
} ## end sub _update_conversion_hash

sub _determine_extra_filter_fields {
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            show_fields   => {type => HASHREF},
            parameters    => {type => HASHREF},
        }
    );
    my $show_fields = $opts{show_fields};
    my $parameters  = $opts{parameters};
    my @s = keys %{ $opts{show_fields} };
    my @p;
    if ( $parameters->{name} && !ref $parameters->{name}) {
        my $field_name=$parameters->{name};
        if ( $field_name ne "fields" && $field_name ne "instance" ) {
            @p = ( $parameters->{name} );
        }
    } else {
        delete $parameters->{fields};
        delete $parameters->{instance};
        @p = keys %{ $parameters };
    }
    my @minus = array_minus(@p, @s);
    my $ref = {};
    foreach my $elem ( @minus ) {
        $ref->{$elem}->{field}=$elem;
        $ref->{$elem}->{'zapi-mapping'}=$parameters->{$elem}->{'zapi-element'}
            if ( $parameters->{$elem}->{'zapi-element'} );
    }
    $Log->exit() if $may_exit;
    return $ref;
}

1;
