#!/usr/software/bin/perl


# See below in package main for the main code of this script and at
# the very end for documentation.

package MyBodyFilter;

# But first, here's an HTTP::Proxy::BodyFilter which not only saves
# files as they are requested (like HTTP::Proxy::BodyFilter::save, its
# base class, comes with HTTP::Proxy) but also announces them after it
# has finished saving them (and, importantly, not before they're
# finished being closed/written).

use strict;
use warnings;

use base ('HTTP::Proxy::BodyFilter::save');

# Here we override 'end' to print out when the file has been saved,
# because "end" seems to be where the file in question gets closed.
# However, as we describe in the next method, this never actually gets
# called.

sub end {
    my $self = shift;

    if ( $self->{_hpbf_save_fh} ) {
        $self->SUPER::end(@_);
        print "Saved file: " . $self->{_hpbf_save_filename} . "\n";
    }
}


# FilterStack->filter_last is the only caller of BodyFilter->end() and
# it's only called for body response filters.

# It seems like FilterStack->filter_last isn't called currently for
# body *request* filters, like ours, because requests are currently
# not received gradually, but instead all at once, so 'filter' is
# called directly with the whole content.  This may be fixed in a
# future version of HTTP::Proxy (but that's conjecture; current CPAN
# version hasn't changed here)

# This also means that, as currently written, "save" as a body request
# filter don't necessarily close the files it recieves until a
# subsequent request comes in and overwrites _hpbf_save_fh.  Ick.

# So, let's override our "filter" method so that it expects to be
# called once for the whole content: this version not only writes the
# data but it also closes the file, and it is also not willing to be
# called with the file already closed (which should hopefully catch
# any problems if HTTP::Proxy gets changed someday to receive requests
# in chunks).  Since it closes the file, it is then allowed to
# announce that the file has been saved.

# This also makes errors in saving the file become fatal, rather than
# just effectively a warning (error and continue).

sub filter {
    my ( $self, $dataref ) = @_;
    if (!exists $self->{_hpbf_save_fh}) {
	NATE::BaseException->throw("internal error: filter called with file handle already closed");
    }

    my $filename = $self->{_hpbf_save_filename};
    my $res = $self->{_hpbf_save_fh}->syswrite( $$dataref );
    if (!defined($res)) {
	NATE::BaseException->throw("Could not write to '$filename': $!");
    }
    $res=close($self->{_hpbf_save_fh});
    if (!defined($res)) {
	NATE::BaseException->throw("Could not close '$filename': $!");
    }
    delete $self->{_hpbf_save_fh};
    print "Saved file: $filename\n";
}

package main;

use strict;
use warnings;

use Getopt::Long qw(GetOptions);
use Pod::Usage qw(pod2usage);
use HTTP::Proxy ();

# Find http_util.pm in the same directory as http_proxy.pl
use FindBin ();
use lib $FindBin::RealBin;
use http_util qw(my_url best_local_ip_for_peer);

our $Help;
our $Man;
our $Addr;
our $Peer;
our $Port;
our $Repository;
our $Verbose;

$|=1; # Flush STDOUT, when tests make rsh connection to this script output is not received on other side even when URL is actually printed
main();

sub main {
    $SIG{PIPE} = 'IGNORE';
    GetOptions(
        'help|h|?'     => \$Help,
        'man'          => \$Man,
        'addr=s'       => \$Addr,
        'peer=s'       => \$Peer,
        'port=s'       => \$Port,
        'repository=s' => \$Repository,
        'verbose|v'    => \$Verbose,
    ) or pod2usage( '-verbose' => 0, '-exitval' => 1 );

    if ($Help) {
        pod2usage( '-verbose' => 0, '-exitval' => 1 );
    }

    if (@ARGV) {
        pod2usage(
            '-message' => 'There are no non-option arguments',
            '-verbose' => 0,
            '-exitval' => 1
        );
    }

    if ( !defined($Repository) ) {
        pod2usage(
            '-message' => '"-repository" is a required option',
            '-verbose' => 0,
            '-exitval' => 1
        );
    }

    my @proxy_opts;

    if ($Addr) {
        push @proxy_opts, ( host => $Addr );
    }
    elsif ($Peer) {
        push @proxy_opts, ( host => best_local_ip_for_peer($Peer) );
    }

    # Push undef explicitly even when no port option given, to change
    # default from "8080" to "any unused port".
    push @proxy_opts, ( port => $Port );

    if ($Verbose) {
	push @proxy_opts, (logmask => HTTP::Proxy::ALL);
    }

    my $proxy = HTTP::Proxy->new(@proxy_opts)
      or die "Could not create http proxy: $!";

    my $repository = $Repository;
    $repository =~ s{/$}{}g;
    $repository =~ s/\%/%%/g;

    $proxy->push_filter(
        method  => 'PUT,POST',
        request => MyBodyFilter->new( template => "$repository/\%f", ),
    );

    print "URL: " . my_url( $proxy->daemon ) . "\n";

    # Runs forever
    $proxy->start();
}

__END__

=head1 NAME

http_proxy.pl - simple foreground HTTP proxy server

=head1 SYNOPSIS

  http_proxy.pl [-addr <addr> | -peer <addr>] [-port <port>] \
      -repository <path>
      ...
  http_proxy.pl -h
  http_proxy.pl -man

=head1 DESCRIPTION

This is an HTTP proxy.  It is meant to be invoked by test scripts to
provide a temporary HTTP proxy service which will log any data posted
(using PUT or POST) to a file under the given C<-repository>
directory.  This is designed to be used to capture autosupport
requests.

This program does not detach from the session used to invoke it.
After it is invoked and has started listening, it prints "URL: ",
followed by the URL it is listening on.  It announces each file it has
saved by printing "Saved file: " followed by the filename.  Once it is
no longer needed by a test script, the test script would be expected
to kill it.

=head1 OPTIONS

=over

=item -h

Show help (abbreviated form of this document)

=item -man

Show manual (this document)

=item -v | -verbose

Show actions performed (in great detail: corresponds to HTTP::Proxy
logmask "ALL")

=item -addr I<local addr> or -peer I<remote addr>

If "-addr" is given, listen on the given local address.  If "-peer" is
given, listen on a local address which is good for talking to the
given remote address.  If neither option is given, listen on all
addresses, and give the local system's hostname as the address in the
URL.

=item -port I<port>

If "-port" is given, listen on the given port.  Otherwise, listen on
an arbitrary free port.  (The port chosen will be listed as part of
the URL that is printed)

=item -repository I<path>

The path to an existing directory, under which the contents of proxied
requests will be created.

The current behavior is that the body/contents of PUT and POST HTTP
requests (the request, not the response) will be stored in files
directly under the given C<-repository> directory, using the last
component of the path used to POST or PUT them.  If a file by that
name already exists (unlikely, for autosupport) then a new filename
ending in ".1", ".2", etc. is chosen.

=back

=head1 EXAMPLE

   # host1 posts to host3 using this process running on host2 as an
   # HTTP proxy.  The proxy saves the contents of the post as it goes
   # by.

   host2% mkdir /tmp/myrepo
   host2% http_proxy.pl -peer host1 --repository /tmp/myrepo
   URL is: http://192.168.3.123:12345/

   host1% setenv http_proxy http://192.168.3.123:12345/
   host1% wget http://host3/filename --post-data "Some Content"

   host2% cat /tmp/myrepo/filename
   Some Content

=cut
