#!/usr/software/bin/perl

use strict;
use warnings;

#use threads;

use Getopt::Long qw(GetOptions);
use Pod::Usage qw(pod2usage);
use HTTP::Daemon qw();
use IO::File qw();
use HTTP::Status;    # RC_* constants
use List::Util qw(min);

# Find http_util.pm in the same directory as http_server.pl
use FindBin ();
use lib $FindBin::RealBin;
use http_util qw(my_url best_local_ip_for_peer);

our $Help;
our $Man;
our $Addr;
our $Peer;
our $Port;
our $Verbose;
our %Request_map;

$|=1; # Flush STDOUT, when tests make rsh connection to this script output is not received on other side even when URL is actually printed
main();

sub main {
    $SIG{PIPE} = 'IGNORE';
    GetOptions(
        'help|h|?'  => \$Help,
        'man'       => \$Man,
        'addr=s'    => \$Addr,
        'peer=s'    => \$Peer,
        'port=s'    => \$Port,
        'verbose|v' => \$Verbose,
    ) or pod2usage( '-verbose' => 0, '-exitval' => 1 );

    if ($Help) {
        pod2usage( '-verbose' => 0, '-exitval' => 1 );
    }

    if ( @ARGV % 3 != 0 || @ARGV < 3 ) {
        pod2usage(
            '-message' =>
                'non-option args should appear 3 at a time (method, uri, filename)',
            '-verbose' => 0,
            '-exitval' => 1
        );
    } ## end if ( @ARGV % 3 != 0 ||...

    while (@ARGV) {
        my $method   = shift @ARGV;
        my $uri      = shift @ARGV;
        my $filename = shift @ARGV;

        if ( $method !~ /^(?:GET|POST)$/ ) {
            die "Unknown method '$method' (should be 'GET' or 'POST')\n";
        }
        $Request_map{$method}{$uri} = $filename;
    } ## end while (@ARGV)

    my @httpd_opts;

    if ($Addr) {
        push @httpd_opts, ( LocalAddr => $Addr );
    } elsif ($Peer) {
        push @httpd_opts, ( LocalAddr => best_local_ip_for_peer($Peer) );
    }
    if ($Port) {
        push @httpd_opts, ( LocalPort => $Port );
    }

    my $httpd = HTTP::Daemon->new(@httpd_opts)
        or die "Could not create httpd: $!";

    print "URL: " . my_url($httpd) . "\n";
    while ( my $client_conn = $httpd->accept ) {

        #my $thread = threads->new(\&serve,$client_conn);
        #$thread->detach();
        serve($client_conn);
    } ## end while ( my $client_conn =...
} ## end sub main

sub serve {
    my ($client_conn) = @_;

    my $temp_path;
    my $method = "";
    my $uri    = "";

    eval {

       # The 7-mode software install command requires all of the
       # headers to arrive in a single network read, so the very
       # first tcp response data must be a complete segment.  However,
       # unfortunately, HTTPD::Daemon does many little writes to
       # formulate the headers.  Fortunately, it uses "print" instead
       # of "syswrite" to write, and this HTTP session is quite
       # non-interactive (send the entire response all at once and
       # close, because of force_last_request, below), so disabling
       # autoflush is all that's needed.

	$client_conn->autoflush(0);

        # 1 = read headers only
        my $request = $client_conn->get_request(1);

        # This lets us be less careful about having to read client's
        # content when we don't end up using it.
        $client_conn->force_last_request();

        if ( !$request ) {
            die $client_conn->reason();
        }

        $client_conn->force_last_request();

        $uri    = $request->uri();
        $method = $request->method();
        my $base_method = $method eq "HEAD" ? "GET" : $method;
        my $filename = $Request_map{$base_method}{$uri};
        if ( !$filename ) {
            die [
                RC_FORBIDDEN,
                "No known file for method '$base_method' uri '$uri'"
            ];
        } elsif ( $base_method eq "GET" ) {
            print "method=$method uri=$uri : sending...\n" if $Verbose;
            $client_conn->send_file_response($filename);
            print "method=$method uri=$uri : sent.\n" if $Verbose;
            return;
        } elsif ( $base_method eq "POST" ) {
            print "method=$method uri=$uri : receiving...\n" if $Verbose;
            my $te = $request->header('Transfer-Encoding');
            if ( $te && lc($te) eq 'chunked' ) {
                die [
                    RC_BAD_REQUEST, "Sorry, can't cope with chunked encoding"
                ];
            }

            $temp_path = $filename . ".part";
            my $fh = IO::File->new( $temp_path, "w" )
                or die "Could not open file ${temp_path}: $!\n";

            my $max_len = $request->header('Content-Length');

            my $buf        = $client_conn->read_buffer();
            my $bytes_read = length($buf);
            while (1) {
                if ( length($buf) ) {
                    syswrite( $fh, $buf )
                        or die "Could not write to $temp_path: $!\n";
                }
                my $len = 65536;
                if ( defined($max_len) ) {
                    $len = min( $len, $max_len - $bytes_read );
                }
                last if ( $len == 0 );
                my $res = sysread( $client_conn, $buf, $len );
                if ( !defined($res) ) {
                    die "Error reading post content from client: $!\n";
                } elsif ( $res == 0 ) {
                    if ( defined($max_len) && $bytes_read != $max_len ) {
                        die
                            "Client did not write full length ($bytes_read < $max_len)\n";
                    } else {
                        last;
                    }
                } else {
                    $bytes_read += $res;
                }
            } ## end while (1)
            close($fh);
            rename( $temp_path, $filename )
                or die "Could not rename ${temp_path} to ${filename}: $!\n";
            $temp_path = undef;
            $client_conn->send_response( RC_OK, "Ok", [], "Ok" );
            print "method=$method uri=$uri : received...\n" if $Verbose;
        } else {
            die "Unhandled method '$base_method'\n";
        }
    };
    if ($@) {
        my ( $code, $reason )
            = ( ref($@) eq 'ARRAY' )
            ? @{$@}
            : ( RC_INTERNAL_SERVER_ERROR, $@ );
        print "method=$method uri=$uri : $code $reason\n" if $Verbose;
        $client_conn->send_error( $code, $reason );
    } ## end if ($@)
    if ($temp_path) {
        unlink($temp_path);
    }
} ## end sub serve

__END__

=head1 NAME

http_server.pl - simple foreground HTTP server

=head1 SYNOPSIS

  http_server.pl [-addr <addr> | -peer <addr>] [-port <port>] \
      <method1> <uri1> <filename1> \
      <method2> <uri2> <filename2> \
      ...
  http_server.pl -h
  http_server.pl -man

=head1 DESCRIPTION

This is an HTTP server.  It is meant to be invoked by test scripts to
provide a temporary HTTP service, for situations where the system
under test needs to read a file or post a file.  Some situations where
it should be useful:

=over

=item *

It can be used to transfer netboot images or install images to an
ONTAP system.  This avoids having to keep track of which netboot and
web servers are used at which sites, and how the URLs to those web
servers map to filesystem paths.

=item *

In 'post' mode, it can be used with "wget --post-file" in ONTAP system
shell to copy files off of a filer.  This should work for 7-mode and
promoted builds (unlike scp from unix/windows to filer) and should
require much less setup than scp or nfs from filers to unix/windows.

=back

It does not detach from the session used to invoke it.  After it is
invoked and has started listening, it prints "URL: ", followed by the
URL it is listening on.  Once it is no longer needed by a test script,
the test script would be expected to kill it.

=head1 OPTIONS

=over

=item -h

Show help (abbreviated form of this document)

=item -man

Show manual (this document)

=item -v | -verbose

Show actions performed (a line or two per incoming request)

=item -addr I<local addr> or -peer I<remote addr>

If "-addr" is given, listen on the given local address.  If "-peer" is
given, listen on a local address which is good for talking to the
given remote address.  If neither option is given, listen on all
addresses, and give the local system's hostname as the address in the
URL.

=item -port I<port>

If "-port" is given, listen on the given port.  Otherwise, listen on
an arbitrary free port.  (The port chosen will be listed as part of
the URL that is printed)

=item GET I<uri> I<filename>

When the server receives a GET or HEAD request for the given URI (path
under the URL), serve the given filesystem file I<filename> as the
content.

=item POST I<uri> I<filename>

When the server receives a POST request for the given URI (path under
the URL) receive the content of the post and write it to I<filename>.
I<filename> will be opened with the suffix ".part" (created,
truncated) and then renamed to just I<filename> after the entire post
content is received.

=back

=head1 EXAMPLES

   # Copying /bin/ls from client1 to client2

   client1% http_server.pl -peer client2 GET /ls /bin/ls
   URL is: http://192.168.3.123:12345/

   client2% wget http://192.168.3.123:12345/ls

   # Copying a core file from filer1 to client1

   client1% http_server.pl -peer filer POST /x /x/eng/cores/bug.12345/core.gz
   URL is: http://192.168.3.123:12345/

   filer1$ wget http://192.168.3.123:12345/x --post-file /path/to/core/file

=head1 BUGS

This server may send the full file content if a client makes a HEAD
request.  (Newer versions of HTTP::Daemon fix this, but not the
version we currently have installed around here, and we're not doing
anything to work around this).

=cut
