#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::EnGenX::EnGenX_Xgen_Demo::Demo4;

use strict;
use warnings;

use parent qw( NACL::UnitTest::AppBuilder::EnGenX::EnGenX_Xgen_Demo::Demo3 );

use NACL::AppBuilder::Xgen::Resource::Vserver;
use NACL::AppBuilder::Xgen::Resource::Volume;
use NACL::AppBuilder::Xgen::Resource::Qtree;

use NACL::Service::Xgen::Util qw(positive_filter);

use POSIX qw(ceil floor);

# This config is based on Demo3 and will build the same numbers as Demo3.
# In this config we want to make a percentage of the volumes in the mixed security vservers
# type DP.  We also want to create a number of qtrees but only on non-dp volumes in mixed security vservers.
# 50% of volumes in mixed security type vservers will be type dp.
use constant PERCENT_DP_VOLUMES_IN_MIXED_VSERVERS => 50;

use constant TOTAL_QTREE_COUNT => 4;

# Since we are adding a new object type and also want to update object instead of rewrite the generate
# function then we need to add new methods to the build. SUPER::build will build everything in demo3,
# then we can add and update with our new functionality.
sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);

    # Call to generate vserver resources.
    $self->update_volume_resources(%args);

    # Call to generate volume resources.
    $self->generate_qtree_resources(%args);
}

# Updates the volume resources on the mixed vserver so that a percentage are type dp.
sub update_volume_resources {
    my ($self, %args) = @_;

    # Get the vserver resource so we can tell which are mixed security type.
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");

    # Filter the vserver resources by security style mixed.
    my $matches_hash = positive_filter(resources => $vserver_resources,
                                       filter    => {'rootvolume-security-style' => 'mixed'});

    # Get the matches.
    my $mixed_vservers = $matches_hash->{'matches'};

    # Loop through the matching vservers.
    for my $vserver (@{$mixed_vservers}) {
        # get all the volumes in the vserver
        my @volumes = $vserver->get_children_resources(type => "NACL::AppBuilder::Xgen::Resource::Volume");

        my $volume_count = ceil((scalar(@volumes) * $self->PERCENT_DP_VOLUMES_IN_MIXED_VSERVERS) / 100);

        # Loop through volume_count and set those volumes to type dp.
        for (my $i = 0; $i < $volume_count; $i++) {
            my $volume = shift(@volumes);
            push(@volumes, $volume);

            $volume->set_private_attributes(type => 'DP');
        }
    }
}

# Create Qtrees up to TOTAL_QTREE_COUNT only on non-dp volumes in mixed vservers.
sub generate_qtree_resources {
    my ($self, %args) = @_;

    my @qtrees;

    # Get the vserver resource so we can tell which are mixed security type.
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");

    # Filter the vserver resources by security style mixed.
    my $matches_hash = positive_filter(resources => $vserver_resources,
                                       filter    => {'rootvolume-security-style' => 'mixed'});

    # Get the matches.
    my $mixed_vservers = $matches_hash->{'matches'};

    my @NonDP_Volumes = ();
    # Loop through the matching vservers so that we can build a list of volumes.
    foreach my $vserver (@{$mixed_vservers}) {
        # get all the volumes in the vserver
        my @volumes = $vserver->get_children_resources(type => "NACL::AppBuilder::Xgen::Resource::Volume");

        # Filter the volumes so we can tell which are DP
        my $matches_hash = positive_filter(resources => \@volumes,
                                           filter    => {'type' => 'DP'});

        # Get the non-matches.
        my $nonmatch_volumes = $matches_hash->{'not_matches'};

        # Push into list of volumes
        push(@NonDP_Volumes,@{$nonmatch_volumes});
    }

    # Now we have out list of volumes.
    # If that list is empty then we will do nothing
    #(Could fail but we will say this config does not want to in this case)
    return unless(scalar(@NonDP_Volumes));

    # Loop through TOTAL_QTREE_COUNT and create those qtrees.
    for (my $i = 0; $i < $self->TOTAL_QTREE_COUNT; $i++) {
        my $volume = shift(@NonDP_Volumes);
        push(@NonDP_Volumes, $volume);

        my $qtree_res = NACL::AppBuilder::Xgen::Resource::Qtree->new(vserver => $volume->get_attribute("vserver"),
                                                                     volume => $volume->get_attribute("volume"),
                                                                     qtree => "Qtree$i");

        # connect the qtree resource to the volume parent resource
        $qtree_res->add_parent_resources(resources => [$volume]);

        push(@qtrees, $qtree_res);
    }
    $self->add_resources(resources => \@qtrees);
}

1;

__END__

=head1 NAME

PTEFIT::Xgen::Config::Example

=head1 AUTHOR/MAINTAINER

=over

=item PTEFIT AUTOMATION (ng-ptefit-automation-core@netapp.com)

=back

   Last revised $Date: $ by $Author: $

=cut
