#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::EnGenX::EnGenX_Xgen_Demo::Demo5;

use strict;
use warnings;

# We use Demo1 as the parent since we don't need the extra functionality from the other demos.
use parent qw( NACL::UnitTest::AppBuilder::EnGenX::EnGenX_Xgen_Demo::Demo1 );

use NACL::Service::Xgen::Util qw(convert_percent_layout_to_count_layout
                                 get_next_layout_element
                                 dumper);

use NACL::AppBuilder::Xgen::Resource::VserverPeer;
use NACL::AppBuilder::Xgen::Resource::Snapmirror;
use NACL::AppBuilder::Xgen::Resource::SnapmirrorInitialize;

# Want 4 vservers since one of the cascades is 3 times instead of 2.
use constant TOTAL_VSERVER_COUNT => 4;
# Want volumes to be controlled by snapmirror configuration.
# Here we turn off the other volume creation by setting the count to 0.
use constant TOTAL_VOLUME_COUNT => 0;
# I could also have overridden the generate_volume method
# The number of cascade configs to build.
use constant TOTAL_CASCADE_CONFIG_COUNT => 2;
# The layout of the cascade configs.
# This is simplified just to be type but could be named configs that represent multiple inputs.
use constant CASCADE_TYPE_LAYOUT => {'DP;XDP'       => 50,
                                     'DP;DP;XDP'   => 50};


# Since we are adding a new object type
# we need to add new methods to the build. SUPER::build will build everything in demo4,
# then we can add and update with our new functionality.
sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);

    # Call to generate volume resources.
    $self->generate_cascade_snapmirror_resources(%args);
}

sub generate_cascade_snapmirror_resources {
    my ($self, %args) = @_;

    my @snapmirrors;

    # Get the vserver resources so we can loop through and make the cascade relationships.
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my @vservers = @{$vserver_resources};
    my %cluster_aggr_hash = ();
    my $volume_count = 0;
    # This will get the layout in terms of numbers instead of percents based on the total object count passed in.
    my $layout = convert_percent_layout_to_count_layout(layout => $self->CASCADE_TYPE_LAYOUT,
                                                        total_objects => $self->TOTAL_CASCADE_CONFIG_COUNT);

    # Loop through TOTAL_CASCADE_CONFIG_COUNT and create those configs.
    for (my $i = 0; $i < $self->TOTAL_CASCADE_CONFIG_COUNT; $i++) {
        # Will contain the volumes created for this configuration.
        my @volumes;
        # get the next config_layout to use from the layout
        my $config_layout = get_next_layout_element(iterator => $i,     # This is not required but allows the get_next_layout_element to
                                                                        # rotate through the layout instead of exhausting the first, then
                                                                        # the second, etc.
                                                    layout => $layout); # layout is the numbers layout we got earlier.

        # Config the config layout into an array of the types of snapmirrors in this cascade config.
        my @config_type_array = split(';',$config_layout);

        # Create the volumes that will be needed by this cascade configuration.
        for (my $j = 0; $j <= scalar @config_type_array; $j++) {
            # We use <= to the config_layout_array as we need 1 more volume that snapmirrors.

            # Rotate through the vserver.
            my $vserver = shift(@vservers);
            push(@vservers, $vserver);
            # If the aggregates for the cluster containing this vserver have not been saved then save them.
            # This allows us to rotate through volumes on the different clusters.
            unless(defined $cluster_aggr_hash{$vserver->get_attribute('cluster_name')}) {
                my ($cluster) = $vserver->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Cluster");
                @{$cluster_aggr_hash{$vserver->get_attribute('cluster_name')}} = $cluster->get_descendants(type => "NACL::AppBuilder::Xgen::Resource::Aggregate");
            }
            # Rotate through the aggregates that are on the vserver's cluster.
            my $aggregate = shift(@{$cluster_aggr_hash{$vserver->get_attribute('cluster_name')}});
            push(@{$cluster_aggr_hash{$vserver->get_attribute('cluster_name')}}, $aggregate);

            # Setup the default volume args.
            my %volumes_args = (vserver   => $vserver->get_attribute("vserver"),
                               aggregate => $aggregate->get_attribute("aggregate"),
                               volume    => "Cascade_Vol" . $volume_count++,
                               size      => $self->VOLUME_SIZE,
                               xgen_metadata => {sm_config     => 'cascade',
                                                 cascade_index => $j+1});
            # If this is not the first volume in the cascade then we want the type to be DP.
            $volumes_args{'type'} = 'DP' unless ($j == 0);

            # Create the volume resource
            my $volume_res = NACL::AppBuilder::Xgen::Resource::Volume->new(%volumes_args);

            # connect the volume resource to the aggr and vserver parent resources
            $volume_res->add_parent_resources(resources => [$vserver,$aggregate]);
            push(@volumes, $volume_res);
        } # End loop to create volume resources.
        $self->add_resources(resources => \@volumes);

        # Now we have the list of volumes for the snapmirrors so we will loop through and create the cascade sm relationships.
        for (my $j = 0; $j < scalar @config_type_array; $j++) {
            # From the volumes array $j will be the source and $j+1 will be the destination.
            # Also $j will be used for an index into the config_layout_array for the attributes of the snapmirror.
            my $source_volume = $volumes[$j];
            my $destination_volume = $volumes[$j+1];

            # Check if a vserver_peer needs to be created.
            unless($self->{vserver_peer_hash}->{$source_volume->get_attribute('cluster_name')}
                                             ->{$source_volume->get_attribute('vserver')}
                                             ->{$destination_volume->get_attribute('cluster_name')}
                                             ->{$destination_volume->get_attribute('vserver')}) {
                # These volumes' vservers have not been peered so we need to create a vserver peer.
                my %vserver_peer_args = ('vserver'      => $destination_volume->get_attribute('vserver'),
                                         'peer-vserver' => $source_volume->get_attribute('vserver'),);
                # If the volumes are on different clusters then we want to add the peer-cluster as an argument.
                if($source_volume->get_attribute('cluster_name') ne $destination_volume->get_attribute('cluster_name')) {
                    $vserver_peer_args{'peer-cluster'} = $source_volume->get_attribute('cluster_name');
                }
                my $vserver_peer = NACL::AppBuilder::Xgen::Resource::VserverPeer->new(%vserver_peer_args);
                # The destination is required to be second since we need the destination cluster to be
                # the cluster used for the vserver peer commands because of how we set up the peer-vserver and peer-cluster.
                $vserver_peer->add_parent_resources(resources => [$source_volume->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Vserver"),
                                                                  $destination_volume->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Vserver")]);


                # Now we need to setup the vserver peer hash bidirectionally.
                $self->{vserver_peer_hash}->{$source_volume->get_attribute('cluster_name')}
                                          ->{$source_volume->get_attribute('vserver')}
                                          ->{$destination_volume->get_attribute('cluster_name')}
                                          ->{$destination_volume->get_attribute('vserver')} = 1;
                $self->{vserver_peer_hash}->{$destination_volume->get_attribute('cluster_name')}
                                          ->{$destination_volume->get_attribute('vserver')}
                                          ->{$source_volume->get_attribute('cluster_name')}
                                          ->{$source_volume->get_attribute('vserver')} = 1;

                # finally add the vserver peer resource to the config.
                $self->add_resources(resources => [$vserver_peer]);
            }

            # Setup the source, destination, and type arguments.
            my %snapmirror_args = ('source-vserver' => $source_volume->get_attribute('vserver'),
                                   'source-volume'  => $source_volume->get_attribute('volume'),
                                   'destination-vserver' => $destination_volume->get_attribute('vserver'),
                                   'destination-volume'  => $destination_volume->get_attribute('volume'),
                                   'type'                => $config_type_array[$j]);
            # If the volumes are on different clusters then we want to add the source-cluster as an argument.
            if($source_volume->get_attribute('cluster_name') ne $destination_volume->get_attribute('cluster_name')) {
                $snapmirror_args{'source-cluster'} = $source_volume->get_attribute('cluster_name');
            }

            # Create the snapmirror resource.
            my $snapmirror = NACL::AppBuilder::Xgen::Resource::Snapmirror->new(%snapmirror_args);
            # The destination is required to be second since we need the destination cluster to be
            # the cluster used for these snapmirror commands.
            $snapmirror->add_parent_resources(resources => [$source_volume,$destination_volume]);
            $self->add_resources(resources => [$snapmirror]);

            # Update the metadata on the volumes.
            $source_volume->set_metadata_attributes('source_for_sm_type' => $config_type_array[$j]);
            $destination_volume->set_metadata_attributes('destination_for_sm_type' => $config_type_array[$j]);
            # Create the snapmirror initialize resource so the snapmirror gets initialized.
            my $snapmirror_initialize = NACL::AppBuilder::Xgen::Resource::SnapmirrorInitialize->new(%snapmirror_args,
                                                                                                    xgen_metadata => {parallel => $j+1});
            $snapmirror_initialize->add_parent_resources(resources => [$snapmirror]);
            $self->add_resources(resources => [$snapmirror_initialize]);
        }
    }

}

1;

__END__

=head1 NAME

PTEFIT::Xgen::Config::Example

=head1 AUTHOR/MAINTAINER

=over

=item PTEFIT AUTOMATION (ng-ptefit-automation-core@netapp.com)

=back

   Last revised $Date: $ by $Author: $

=cut
