#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Component-level Exception Tester module
## @author dl-nacl-dev@netapp.com, madhavs@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::UnitTest::C::ExceptionTester

=head1 DESCRIPTION

Most of the component-level exceptions implement a C<detect> method which
check the error text and return true if it matches a list of expected
error strings.

This package is intended to be used for regression testing for component-level
exceptions and to ensure that any modifications to the C<detect> method
do not break support for currently supported error strings.

=cut

package NACL::UnitTest::C::ExceptionTester;

use strict;
use warnings;

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(
        exception_tester
        test_exception_by_running_command
        test_unclassified_exceptions
    );
    our %EXPORT_TAGS = ('all' => \@EXPORT_OK);
}

use Tharn qw($Log);
use Params::Validate qw(validate_with SCALAR ARRAYREF CODEREF OBJECT);
use NACL::APISet::Exceptions::ResponseException;
use NACL::APISet::Response::CLI::CMode;
use NATE::BaseException qw(:try);

=head2 exception_tester

    use NACL::UnitTest::C::ExceptionTester qw(exception_tester);
    ...
    
    # Using Volume::DoesNotExist as an example
    exception_tester(
        exception_name => 'NACL::C::Exceptions::Volume::DoesNotExist',
        strings        => [
            # CMode
            'command failed: Volume "ab" not found. Reason: entry doesn\'t exist.',
            
            # 7Mode
            'No volume named ab exists'
        ],
    );
    
Provided an exception name, check whether each of the error strings would
result in that exception being thrown. If any of the strings would not have
resulted in this exception, then this method raises a C<NATE::BaseException>.

=over

=item Options

=over

=item C<< exception_name => $exception_name >>

The name of the exception being tested.

=item C<< strings => \@strings >>

An array of error strings which are expected to result in the exception
listed.

=back

=back

=cut

sub exception_tester {

    
    my %opts = validate_with(
        params => \@_,
        spec   => {
            exception_name => { type => SCALAR },
            strings        => { type => ARRAYREF },
        }
    );
    my $exception_name = $opts{exception_name};
    my $strings        = $opts{strings};
    
    eval "use $exception_name";
    if ($@) {

        NATE::BaseException->throw('Could not import exception package '
            . "'$exception_name'. Error:\n$@");
    }
    
    foreach my $string (@$strings) {
        # Creating an exception object, with a dummy response object.
        # For cases where detect() does more than merely do a pattern
        # match on the text (for example, check if the exception is of a 
        # particular type), then we can't use this. This is merely for the
        # common case of detect() checking the text alone.
        my $exception_obj = NACL::APISet::Exceptions::ResponseException->new(
            text            => $string,
            response_object => NACL::APISet::Response::CLI::CMode->new(
                output               => $string,
                CLI_separator        => 'X;X',
                command              => 'dummy',
                execution_time       => 1,
                output_compat_mirror => { a => 2 }
            ),
        );
        
        unless ($exception_name->detect(exception => $exception_obj)) {

            NATE::BaseException->throw("For the exception '$exception_name' "
                . 'the method detect should have returned 1 for the error '
                . "string:\n\t$string\nbut it did not");
        }
    }
    

}

sub test_exception_by_running_command {

    my %opts = validate_with(
        params => \@_,
        spec   => {
            code      => {type => CODEREF},
            exception => {type => SCALAR},
            tcd_obj   => {type => OBJECT},
        },
    );
    my $exception = $opts{exception};
    
    try {
        $opts{code}->();
        NATE::BaseException->throw("Should have thrown a $exception "
            . 'exception but did not');
    } except {
        return {
            $exception => sub {
                $opts{tcd_obj}->nlog("Caught $exception as expected");
            }
        };
    };


}

sub test_unclassified_exceptions {

    my %opts = validate_with(
        params => \@_,
        spec   => {
            code    => {type => CODEREF},
            tcd_obj => {type => OBJECT},
        },
    );
    
    try {
        $opts{code}->();
        NATE::BaseException->throw('Should have propagated the APISet '
            . 'exception but did not');
    } catch NACL::APISet::Exceptions::ResponseException with {
        $opts{tcd_obj}->nlog('Caught a ResponseException as expected');
    };


}

1;
