#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary A helper class for displaying CS fields.
##
## @author deepthis, dl-mpo-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::UnitTest::Dfm::Utils::ComponentState;

use strict;
use warnings;

use Params::Validate
  qw(validate validate_with SCALAR SCALARREF ARRAYREF BOOLEAN OBJECT);
use NACL::ComponentUtils qw(Dumper);
use NATE::BaseException qw(:try);
use Data::Dumper;
use Tharn qw($Log);

use Exporter 'import';

our @EXPORT = qw (
    print_statefields
);

#Define Constant
use constant DEFAULT_STATE_OBJECT_COUNT   => 5;
use constant DEFAULT_LINE_CHARACTER       => '*';
use constant DEFAULT_LINE_CHARACTER_COUNT => 100;

=head1 NAME

NACL::UnitTest::Dfm::Utils::ComponentState

=head1 METHODS

=head2 print_statefields 

NACL::UnitTest::Dfm::Util::ComponentStateFields->print_statefields(
    package_name   => 'NACL::CS::DFM::JobTask',
    state_obj_count => 1,
    state_objs    => \@job_task_Objs
);

=for html (Class or instance method)
This method is used to list out all the attributes of the Component
State which is passed as the parameter.

=over

=item Options

=over

=item C<< package_name   => $package_name >>

(Optional) Package name is optional if it is being called with the same
package whose state attributes are being printed. If it is being called using
NACL::UnitTest::Dfm::Utils::ComponentState, then passing the package name is a must.

=item C<< state_objs => \@job_task_Objs >>

(Required) State_objs is a ARRAYREF where in we will pass this 
as input along with package name inorder to display the outputs
with all attributes for the state_objs passed

=item C<< state_obj_count => $state_obj_count >>

(Optional) This param decides how many state_objs attributes to be
printed if it has more number of entries. By default, it is set to
be 5.

=back

=back

=cut

sub print_statefields {

    my $self         = shift;
    my @state_fields = ();
    my %opts         = validate_with(

        params => \@_,
        spec   => {
            'package_name'    => { type => SCALAR,   optional => 1, },
            'state_objs'      => { type => ARRAYREF, optional => 0 },
            'state_obj_count' => {
                type     => SCALAR,
                optional => 1,
                default  => DEFAULT_STATE_OBJECT_COUNT
            },
            'line_character' => {
                type     => SCALAR,
                optional => 1,
                default  => DEFAULT_LINE_CHARACTER
            },
            'line_character_count' => {
                type     => SCALAR,
                optional => 1,
                default  => DEFAULT_LINE_CHARACTER_COUNT
            },
        },
    );

    my $packagename     = $opts{package_name} || $self;
    my $obj_count       = $opts{state_obj_count};
    my @state_objs      = @{ $opts{state_objs} };
    my $separator       = $opts{line_character};
    my $separator_count = $opts{line_character_count};

    try {
        eval "require $packagename";
    }
    otherwise {
        $Log->exit();
        NATE::Exceptions::Argument->throw(
            'Could not import package ' . "'$packagename'" );
    };

    my $attribute_hash = $packagename->get_attributes();

    my $nested_hash = $packagename->_nested_objects();

    # Consider the number of objects to be displayed
    @state_objs =
      ( scalar(@state_objs) <= $obj_count )
      ? @state_objs
      : @state_objs[ 0 .. $obj_count - 1 ];


    foreach my $state_obj (@state_objs) {

        $Log->log( $separator x $separator_count );

        # Check all the keys belonging to this type
        foreach my $attribute ( keys(%$attribute_hash) ) {

            my $field_name = $attribute;
            $field_name = ucfirst($field_name);
            $field_name =~ s/_/ /g;

            # Check if it is a typedef
            if ( exists $nested_hash->{$attribute} ) {

                # Check if an array
                if ( $attribute_hash->{$attribute} =~ /array/i && defined ($state_obj->{$attribute}) ) {

                    my @array = @{ $state_obj->{$attribute} };

                    if (@array) {

			$Log->log("Fields of the elements of array \"$field_name\": >>>");

                        # Call the function again
                        $self->print_statefields(
                            'package_name'    => $nested_hash->{$attribute},
                            'state_obj_count' => $obj_count,
                            'state_objs'      => \@array,
                            'line_character'  => '-',
                            'line_character_count' => 50,
                        );
                        $Log->log( '-' x 50 );
                    } else {

                        $Log->log(
                            "Array attribute \"$field_name\" has no elements.");
                    }    # end if @array

                } else {

                    if ( defined $state_obj->{$attribute} ) {

                        # If not an array
                        $Log->log("Fields of \"$field_name\": >>>");

                        my @array;
                        push( @array, $state_obj->{$attribute} );

                        # Call the function again
                        $self->print_statefields(
                            'package_name'    => $nested_hash->{$attribute},
                            'state_obj_count' => $obj_count,
                            'state_objs'      => \@array,
                            'line_character'  => '-',
                            'line_character_count' => 50,
                        );
                        $Log->log( '-' x 50 );
                    } else {
                        $Log->log("Attribute \"$field_name\" is empty.");
                    }    #end if defined

                }    # end else if array

            } else { # Not nested type

                # Check if ARRAY
                if ( $attribute_hash->{$attribute} =~ /array/i ) {

                    if ( defined $state_obj->{$attribute} ) {
                        $Log->log( $field_name . " :" );

                        foreach my $value ( @{ $state_obj->{$attribute} } ) {
                            $Log->log( ">>" . $value );
                        }

                    } else {
                        $Log->log("Array attribute \"$field_name\" is empty.");
                    }    #end if defined

                } else {

                    if ( defined $state_obj->{$attribute} ) {
                        $Log->log(
                            $field_name . " : " . $state_obj->{$attribute} );
                    }
                }
            }    # end else

        }    #end foreach keys(%$attribute_hash)

    }    # end foreach ( @state_objs )

}    # end sub

1;
