#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Modularized NACL Setup Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::UnitTest::modularized_nacl_setup

=head1 DESCRIPTION

This module provides functions which find-or-create elements commonly required
across our NACL Unit Tests. These functions provide reasonable defaults
for a number of attributes of the relevant elements, simplifying the task of
the caller to simply needing to provide the name of the element they want
found or created. 

Resources which are used globally across our Unit Test suite are created
in either deux_init or nacl_setup.thpl, and are then reused in the unit
tests. In the case where we're running a script stand-alone, it is
overkill to expect nacl_setup or deux_init to be run simply to be able
to run that single unit test. The functions provided by this module check
whether the specified resource exists and if it doesn't, then creates it.
All of the functions return a reference to the object representing the element
found or created (Component/STask object, depending on the availability of the
STask module, refer to the documentation of the individual method to determine
if it returns a C or an STask obkject). Most of the functions also accept a 
"_was_created" argument, through which a scalar-reference can be provided which 
will get filed in with a boolean detailing whether the element was created or not.

Here's an example (create_flex_aggr is documented below)

    use NACL::UnitTest::modularized_nacl_setup qw(create_flex_aggr ...);
    ...
    
    our ($Command_Interface, $Flex_Aggr_Name, $Was_Aggr_Created);
    our $Flex_Aggr_Obj;
    
    sub init {
        $Flex_Aggr_Name = $Test->get_param('FLEXAGGR', $default);
        ...
    }
    
    sub setup {
        ...
        $Flex_Aggr_Obj = create_flex_aggr(
            command_interface => $Command_Interface,
            aggregate         => $Flex_Aggr_Name,
            _was_created      => \$Was_Aggr_Created
        );
        ...
    }
    
    ...
    
    sub cleanup {
        ...
        if ($Was_Aggr_Created) {
            # Flex-aggr was created by the script, we need to clean it up
            $Flex_Aggr_Obj->purge();
        }
    }

Note that none of these functions get imported by defaulted. Each function
required needs to be imported explicitly.

=cut

package NACL::UnitTest::modularized_nacl_setup;

use strict;
use warnings;

use Tharn qw($Log);

use Params::Validate qw(:all);
use NACL::ComponentUtils qw(_optional_scalars _ontap_ci_validate_spec);

use NACL::STask::Aggregate;
use NACL::STask::Vserver;
use NACL::STask::Volume;
use NACL::C::Interface;
use NACL::C::Route;
use NACL::C::SecurityLogin;
use NACL::C::SystemServicesFirewall;
use NACL::C::VserverServicesWebAccess;
use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::APISet::Exceptions::ResponseException qw();
use NACL::C::Client;
use NACL::C::VserverNfs;
use NACL::GeneralUtils qw (create_file);

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(create_flex_aggr
        create_vserver
        create_flex_vol
        create_data_lif
        security_login
        add_to_vserver_aggrlist
        create_infinite_volume
        nfs_file_creation
    );
} ## end BEGIN

=head1 METHODS

=head2 create_flex_aggr

    $Flex_Aggr_STask_Obj = create_flex_aggr(
        command_interface => $Command_Interface,
        aggregate         => $Flex_Aggr_Name
    );

Find a flex-aggr of the given name, or create a flex-aggr of the given name
and with the following attributes:

=over

=item raidtype = raid4

=item diskcount = 2

=back

This returns an STask object. Hence, the method "purge" can be used to
clean it up.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required)

=item C<< aggregate => $aggr_name >>

(Required)

=item C<< _was_created => \$Was_Created >>

(Optional)
Through this parameter a scalar reference can be passed, which gets filled in
with a boolean detailing whether the aggregate was created or not (0 means
aggregate was not created, 1 means aggregate was created)

=item *

Any other options provided will be used at the time of creation. These can be
used to override the defaults as well.

=back

=back

=cut

sub create_flex_aggr {
    my %opts = _validate_with_wrapper(
        params => \@_,
        spec   => { _scalars(qw(aggregate)), },
    );
    my $aggr;
    try {
        $aggr = NACL::STask::Aggregate->create(
            raidtype                => 'raid4',
            diskcount               => 4,
            "force-small-aggregate" => "true",
            if_exists               => 'reuse',
            %opts
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        if ( $exception->text() =~ /nonzeroed\-disks/ ) {

            # Bug in 7Mode ZAPI RR.0 where this error gets thrown
            # even though the aggr create was successful and is going on.
            # This doesn't fail for later releases, but since RR.0 had
            # reached RRT at the time of the burt being filed it wasn't fixed
            # for RR.0
            $aggr = NACL::STask::Aggregate->new(
                command_interface => $opts{command_interface},
                aggregate         => $opts{aggregate}
            );
            if ( $opts{nacltask_wait} ) {

                # Switch to CLI for waiting just to ensure that we don't
                # hit any other ZAPI bugs that might exist for RR.0
                $aggr->wait_for_creation(
                    apiset_should => { interface => 'CLI' } );
            }
        } else {
            $exception->throw();
        }
    };
    return $aggr;
}

=head2 create_infinite_volume

    my @objects_array = create_infinite_volume(
        command_interface => $Command_Interface
    );
    ...
    foreach my $obj (reverse @objects_array) {
        $obj->purge();
    }

Creates the specified no. of aggregates and a repository vserver which uses
all the available aggregates on the cluster.
Finally it creates a infinite volume on the repository vserver. 

Returns back an array of objects having the list of objects in the
created order. This means that the last object in the array would represent
the infinite volume. If any failure happens, it cleans up all the objects
created so far, before throwing the exception.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required)

=item C<< volume => $vol_name >>

(Optional)
Name of the infinite volume to be created. Defaults to 'infi_vol'

=item C<<aggr_count => $aggr_count>>

(Optional)
No.of new aggregates to be created. Defaults to '3'

=back

=back

=cut

sub create_infinite_volume {
    my %opts = validate_with(
        params => \@_,
        spec   => {
            command_interface => _ontap_ci_validate_spec(),
            volume            => { type => SCALAR, default => 'infi_vol' },
            aggr_count =>
                { type => SCALAR, default => 3, regex => qr/^\d+$/, },
        },
    );
    my @arry_of_objects;
    my $Node   = $opts{command_interface};
    my $volume = $opts{volume};
    my $List;
    my @Aggr_list;
    try {
        # By default, a vserver is created with all aggregates included
        # in its aggr-list
        my $vserver_obj = NACL::STask::Vserver->create(
            command_interface => $Node,
            "is-repository"   => 'true',
        );        #creating vserver wit "is-repository" true.
        push @arry_of_objects, $vserver_obj;
        my $vserver = $vserver_obj->vserver();
	for ( my $i = 0; $i < $opts{aggr_count}; $i++ ) {
		my $aggr_obj = NACL::STask::Aggregate->create(
		command_interface => $Node,
		raidtype          => "raid4",
		diskcount         => 3,
		nacltask_add_to_aggr_list_of_vserver => $vserver,
        ); #creating 3 empty aggrregates which is required to create infinite volume
         push @arry_of_objects, $aggr_obj;
         $List = $aggr_obj->aggregate();
         push @Aggr_list, $List;
       }
        $vserver_obj->modify('aggr-list' => \@Aggr_list);
        my $Volume = NACL::STask::Volume->create(
            'command_interface'  => $Node,
            'nacltask_if_exists' => 'purge',
            'volume'             => $volume,
            'vserver'            => $vserver,

        );        #creating infinite volume
        push @arry_of_objects, $Volume;
    }
    catch NATE::BaseException with {
        my $ex = shift;
        $Log->comment( "Exception occured: Unwinding the stack and "
                . "cleaning up the objects" );
        foreach my $obj ( reverse @arry_of_objects ) {
            $obj->purge();
        }
        $ex->throw;
    };
    return @arry_of_objects;
}

=head2 create_vserver

    $Vserver_STask_Obj = create_vserver(command_interface => $Command_Interface);
    
Provides the same facilities as
L<< NACL::STask::Vserver->create|lib-NACL-STask-Vserver-pm/create >>
with nacltask_if_exists set to "reuse".

=cut

sub create_vserver {
    return NACL::STask::Vserver->create( nacltask_if_exists => 'reuse', @_ );
}

=head2 create_flex_vol

    $Flex_Vol_STask_Obj = create_flex_vol(
        command_interface => $Command_Interface,
        volume            => $Volume_Name,
        vserver           => $Vserver_Name,
        aggregate         => $Aggregate_Name
    );
  
Find a volume on the given vserver and aggregate, else create one with
the following additional attributes:

=over

=item size = 20M

=back

This returns an STask object. Hence, the method "purge" can be used to
clean it up.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required)

=item C<< volume => $Volume_Name >>

Optional, a random name gets picked by the STask library if not provided.

=item C<< vserver => $Vserver_Name >>

(Required for CMode)
The vserver through which the volume is to be accessed.

=item C<< aggregate => $Aggregate_Name >>

(Required)
The aggregate on which the volume should be present.

=item C<< _was_created => \$Was_Created >>

(Optional)
Through this parameter a scalar reference can be passed, which gets filled in
with a boolean detailing whether the aggregate was created or not (0 means
aggregate was not created, 1 means aggregate was created)

=item *

Any other options provided will be used at the time of creation. These can be
used to override the defaults as well.

=back

=back

=cut

sub create_flex_vol {
    my %opts = _validate_with_wrapper(
        params => \@_,
        spec   => {
            _scalars(qw(aggregate)),
            _optional_scalars(qw( volume )),
            vserver => { type => SCALAR | UNDEF, optional => 1 },
        },
    );
    return NACL::STask::Volume->create(
        size      => '20M',
        if_exists => 'reuse',
        %opts
    );
}

=head2 create_data_lif

    $Hashref = create_data_lif(
        command_interface => $Command_Interface,
        vserver           => $Vserver_Name,
        lif               => $Data_Lif,
        address           => $Address,
        netmask           => $Netmask,
        gateway           => $Gateway
    );
    $Interface_C_Obj = $Hashref->{interface};
    $Route_C_Obj = $Hashref->{route};
    
Attempts to find the specified LIF on the specified vserver. If found,
returns a hash-reference with the key "interface" containing a
NACL::C::Interface object. If not found:

=over

=item 1) Creates a LIF on the vserver of role "data" and home-port being "e0c".

=item 2) Creates a route to the LIF with destination being "0.0.0.0/0"
and routing group being the one specified in the call.

=item 3) Sets up the various setting performed by 
L<security_login|lib-NACL-UnitTest-modularized_nacl_setup-pm/security_login>.

=back

=over 

=item * Returns a hash-reference with the key "interface" containing the
NACL::C::Interface object for the LIF created, and the key "route" containing
the NACL::C::Route object that was created

=back

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required)

=item C<< vserver => $Vserver_Name >>

(Required)

=item C<< lif => $Lif >>

(Required)

=item C<< address => $Address >>

(Required)

=item C<< netmask => $Netmask >>

(Required)

=item C<< gateway => $Gateway >>

(Required)

=item C<< _was_created => \$Was_Created >>

(Optional)
Through this parameter a scalar reference can be passed, which gets filled in
with a boolean detailing whether the aggregate was created or not (0 means
aggregate was not created, 1 means aggregate was created)

=item C<Other options>

The options we can provide to override the defaults for Interface
creation are "role", "home-node", "home-port".

Also we can provide the "destination" to override the default provided
for Route creation.

=back

=back

=cut

sub create_data_lif {
    my %opts = _validate_with_wrapper(
        params => \@_,
        spec   => {
            _scalars(qw(vserver lif address netmask gateway)),
            _optional_scalars(qw(role home-node home-port destination))
        }
    );

    my ( $int, $route );
    try {
        $int = NACL::C::Interface->find(
            command_interface => $opts{command_interface},
            filter            => { address => $opts{address} },
        );
        ${ $opts{_was_created} } = 0 if exists $opts{_was_created};
    }
    catch NACL::Exceptions::NoElementsFound with {
        my %interface_args;
        NACL::ComponentUtils->_hash_copy(
            source => \%opts,
            target => \%interface_args,
            copy   => [
                qw(vserver lif address netmask command_interface
                    role home-node home-port)
            ]
        );
        $int = NACL::C::Interface->create(
            role        => 'data',
            'home-node' => $opts{command_interface}->node(),
            'home-port' => 'e0c',
            %interface_args
        );
        my $routing_group = $int->get_one_state_attribute('routing-group');

        my %route_args;
        NACL::ComponentUtils->_hash_copy(
            source => \%opts,
            target => \%route_args,
            copy   => [qw(vserver gateway command_interface destination)]
        );
        $route = NACL::C::Route->create(
            destination     => '0.0.0.0/0',
            'routing-group' => $routing_group,
            %route_args
        );

        security_login(
            vserver           => $opts{vserver},
            command_interface => $opts{command_interface}
        );
        ${ $opts{_was_created} } = 1 if exists $opts{_was_created};
    };

    return { interface => $int, route => $route };
} ## end sub create_data_lif

=head2 security_login

    security_login(command_interface => $Command_Interface, vserver => $Vserver_Name);

Sets up services required to use a Vserver LIF for invocation of ZAPIs.
This includes:

=over

=item * Setting password for the vsadmin account to be netapp1!

=item * Unlocking the vsadmin account

=item * Disabling the firewall

=item * Allowing access for ONTAPI for vsadmin

=back

This method does not support the "_was_created" flag. It also does not
return anything.

=cut

sub security_login {
    my %opts = _validate_with_wrapper(
        params => \@_,
        spec   => { _scalars(qw(vserver)) }
    );

    # _was_created not applicable for this method
    delete $opts{_was_created};

    try {
        NACL::C::SecurityLogin->password(
            username => 'vsadmin',
            newpwd   => 'netapp1!',
            newpwd2  => 'netapp1!',
            %opts
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;

        # Ignore if the same password has already been set
        if ( $exception->text()
            !~ /New password must be different than old password/ )
        {
            $exception->throw();
        }
    };

    NACL::C::SecurityLogin->unlock(
        username => 'vsadmin',
        %opts
    );
    NACL::C::SystemServicesFirewall->modify(
        node              => $opts{command_interface}->node(),
        enabled           => 'false',
        command_interface => $opts{command_interface}
    );
    NACL::C::VserverServicesWebAccess->find_or_create(
        name => 'ontapi',
        role => 'vsadmin',
        %opts
    );

    return;
} ## end sub security_login

=head2 add_to_vserver_aggrlist

    add_to_vserver_aggrlist(command_interface => $Command_Interface, vserver => $Vserver_Name);

Adds all aggregates found through the command interface to the "aggr-list" of
the vserver. Later builds of RR.0 and SN.0 require aggregates to be
explicitly added to a vserver's aggr-list.

This does not support the "_was_created" flag and does not return anything.

=cut

sub add_to_vserver_aggrlist {
    my %opts = validate( @_,
        { command_interface => _isa_ci(), _scalars(qw(vserver)) } );
    my $ci = $opts{command_interface};
    if ( $ci->mode() eq 'CMode' ) {
        my @aggregates
            = NACL::STask::Aggregate->find( command_interface => $ci );
        my @aggregate_names = map { $_->aggregate() } @aggregates;
        NACL::C::Vserver->modify(
            'aggr-list' => \@aggregate_names,
            %opts
        );
    }
    return;
}

=head2 nfs_file_creation

    nfs_file_creation(command_interface        => $Command_Interface,
                      client_command_interface => $Client_Command_Interface,
                      vserver => $Vserver_Name
                      volume  => $Volume_Object);

This object will take command_interface , client command_interface, repo vserver and
infinite volume as its mandatory arguements.
This method will enable  NFS service on repo vserver and mount the volume from client machine,
and return mount object.

=cut

sub nfs_file_creation {
    my %opts = validate_with(
        params => \@_,
        spec   => {
            command_interface         => _ontap_ci_validate_spec(),
            client_command_interface  => { type => OBJECT, isa => "NACL::C::Client" },
            volume                    => { type => OBJECT, },
            vserver                   => { type => SCALAR, },
            file                      => { type => SCALAR,  optional => 1},
        },
    );
    my $file = delete $opts{file};
    $file ||= "file$$";
    my $volume =  $opts{volume}->volume();
    # Enable nfs service on vserver
    NACL::C::VserverNfs->create(
        command_interface => $opts{command_interface},
        vserver           => $opts{vserver},
        access            => 'true');
    #Mount the volume on the client
    my $mount = NACL::STask::Client::Mount->create(
             command_interface => $opts{client_command_interface},
             volume            => $opts{volume},
             );
    my $filepath = $mount->{mount_point}."/".$file;
    # Create file at mount point
    create_file(client   => $opts{client_command_interface} , filepath => $filepath);
    return $mount;
}

# Specify that all provided options should be of type SCALAR
sub _scalars {
    my (@names) = @_;
    return ( map { $_ => { type => SCALAR } } @names );
}

# Validation spec for command interfaces
sub _isa_ci {
    return { isa => 'NACL::C::CommandInterface' };
}

# A wrapper round Params::Validate::validate_with that allows
# "_was_created" (as an optional SCALARREF), makes command_interface
# a mandatory argument, and sets allow_extra to 1
sub _validate_with_wrapper {
    my %opts = validate(
        @_,
        {   params => { type => ARRAYREF },
            spec   => { type => HASHREF }
        }
    );
    return validate_with(
        params => $opts{params},
        spec   => {
            command_interface => { isa  => 'NACL::C::CommandInterface' },
            _was_created      => { type => SCALARREF, optional => 1 },
            %{ $opts{spec} }
        },
        allow_extra => 1
    );
}

1;
