#Copyright (c) 2016 Network Appliance, Inc.
#All rights reserved.
#
## @name ::Transition
##
## @summary Contains methods to execute transition commands
##
## @description
##    Using this library, user can perform transition related task.
##    Methods in this library has a one to one mapping to the CLI parsers
##
## @synopsis
##    use NACL::APISet::Host::GUI::SMTT::V3_2::Transition;
##
##    transition_credentials_add(%args);
##    transition_create(%args);
##    transition_volumepair_add(%args);
##    transition_volumepair_remove(%args);
##    transition_volume_mount_modify(%args);
##    transition_lif_add(%args);
##    transition_lif_remove(%args);
##    transition_precheck(%args);
##    transition_start(%args);
##    transition_schedule_add(%args);
##    transition_pause(%args);
##    transition_resume(%args);
##    transition_precutover(%args);
##    transition_resync(%args);
##    transition_update(%args);
##    transition_cutover(%args);
##    transition_abort(%args);
##    transition_schedule_modify(%args);
##    transition_schedule_remove(%args);
##    transition_delete(%args);
##    transition_cft_create(%args);
##    transition_cft_clusterinfo_add(%args);
##    transition_cft_nodemapping_set(%args);
##    transition_cft_svmmapping_add(%args);
##    transition_cft_precheck(%args);
##    transition_cft_apply_svm_config(%args);
##    transition_cft_start(%args);
##    transition_cft_verify_cabling(%args);
##    transition_cft_import(%args);
##    transition_cft_commit(%args);
##    transition_cft_revert_precheck(%args);
##    transition_cft_revert_generate_manual_steps(%args);
##    transition_cft_delete(%args);
##    transition_cft_check_readiness(%args);
##    transition_state_machine_status(%args);
##    transition_cft_editconfig
##    transition_edit_tabs(%args);
##    transition_editproject_volumepair_add(%args);
##    transition_editproject_volumepair_remove(%args);
##    transition_editproject_data_ip_modify(%args);
##    transition_editproject_svm_modify(%args);
##    transition_editproject_schedule_modify(%args);
##    transition_editproject_schedule_remove(%args);
##    transition_move_project_group(%args);
##
## @author santosh6@netapp.com
##
## @change     2015-07-13 santosh6 : Added cft delete command for GUI
## @change     2016-02-24 dinoc    : Added changes for V3.0
## @change     2016-06-01 ashan    : Added changes for Edit Project from Dashboard
## @change     2016-07-18 dinoc    : Added changes for V3.1
##
## @status Review
#

package NACL::APISet::Host::GUI::SMTT::V3_2::Transition;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::APISet::Host::GUI::SMTT::V3_2::GlobalConst;
use NATE::BaseException qw(:try);

use Selenium::Remote::WDKeys;

use Exporter 'import';

our @EXPORT = qw (
    transition_credentials_add
    transition_credentials_remove
    transition_create
    transition_volumepair_add
    transition_volumepair_remove
    transition_volume_mount_modify
    transition_lif_add
    transition_lif_remove
    transition_schedule_add
    transition_schedule_modify
    transition_schedule_remove
    transition_precheck
    transition_start
    transition_precutover
    transition_resync
    transition_cutover
    transition_update
    transition_pause
    transition_resume
    transition_abort
    transition_delete
    transition_cft_create
    transition_cft_clusterinfo_add
    transition_cft_nodemapping_set
    transition_cft_svmmapping_add
    transition_cft_precheck
    transition_cft_apply_svm_config
    transition_cft_start
    transition_cft_verify_cabling
    transition_cft_import
    transition_cft_commit
    transition_cft_revert_precheck
    transition_cft_revert_generate_manual_steps
    transition_cft_revert_verify7_mode
    transition_cft_delete
    transition_view_results
    transition_cft_confirm_execution_revert_manual_steps
    transition_apt_add_systems
    transition_apt_generate_report
    transition_cft_revert_abort
    transition_cft_check_readiness
    transition_state_machine_status
    transition_cft_editconfig
    transition_operation_history
    transition_edit_tabs
    transition_editproject_volumepair_add
    transition_editproject_volumepair_remove
    transition_editproject_data_ip_modify
    transition_editproject_svm_modify
    transition_editproject_schedule_modify
    transition_editproject_schedule_remove
    transition_editproject_plan_config
    transition_move_project_group
);

## @name ::transition_credentials_add
##
## @summary Executes "transition credentials add" command
##
## @description
##    This method is used to execute transition credentials add command
## @synopsis
##      transition_credentials_add(%args);
## INPUTS TO METHOD CALL 'transition_credentials_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-h' </td> <td> 'host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'username' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'password' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##
## @examples
##      $Api_Set_Obj2->transition_credentials_add(
##            "host-name" => "10.227.202.109",
##            "username"  => "root",
##            "password"  => "netapp1!",
##      );
##
##&nbsp;Command = transition credentials add -h 10.227.202.110 -u root
##&nbsp;

sub transition_credentials_add {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "host-name"  => { type => SCALAR },
            "username"   => { type => SCALAR },
            "password"   => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Credentials add page");
    my ($popup_title,$popup_text);
    eval{
        $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);#handling service restart of 7MTT
        $popup_text=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_MESSAGE);
    };
    if(($popup_title =~ /Alert/i) && ($popup_text =~ /click ok to add credentials/)) {
        eval{
            $self->_click(
                id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                error_checking_needed   => 0,
            ) ;
        };
    }
    $self->_click(
        id                      => $STORAGE_SYSTEMS,
        error_checking_needed   => 0,
    ) ;

    $self->_add_credentials(%args);

    $self->_close_tab(
        string => "Storage",
    );
    $Log->comment("Credentials added successfully");
    $Log->exit() if $may_exit;

    return 1;
} ## end sub transition_credentials_add

## @name ::transition_credentials_remove
##
## @summary Executes "transition credentials remove" command
##
## @description
##    This method is used to execute transition credentials remove command
## @synopsis
##      transition_credentials_remove(%args);
## INPUTS TO METHOD CALL 'transition_credentials_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-h' </td> <td> 'host-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##
## @examples
##      $Api_Set_Obj2->transition_credentials_remove(
##            "host-name" => "10.227.202.109",
##      );
##
##&nbsp;Command = transition credentials remove -h 10.227.202.110
##&nbsp;

sub transition_credentials_remove {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "host-name"  => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Storage page");
    $self->_click(
        id                      => $STORAGE_SYSTEMS,
        error_checking_needed   => 0,
    ) ;

    $self->_remove_credentials(%args);

    $self->_close_tab(
        string => "Storage",
    );
    $Log->comment("Credentials removed successfully");
    $Log->exit() if $may_exit;

    return 1;
} ## end sub transition_credentials_remove

## @name ::transition_create
##
## @summary Executes "transition_create" command
##
## @description
##    This method is used to execute transition create command
## @synopsis
##      transition_create(%args);
## INPUTS TO METHOD CALL 'transition_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'session-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> '7-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'data-copy-ipaddress' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'multipathing-ipaddress' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'c-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'svm-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'svm-rootvol-aggr' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'S_mode_credentials' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'C_mode_credentials' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'volumepair_add_args' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'schedule_add_args' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'seven-mode-only' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##       my %S_mode_credentials=(
##            "host-name"                        => "10.227.202.109",
##            username                           => "root",
##            password                           => "netapp1!"
##       );
##       my %C_mode_credentials=(
##            "host-name"                        => "10.227.65.55",
##            username                           => "admin",
##            password                           => "netapp1!"
##       );
##       my %volume=(
##            "volume-name-7-mode"               => [qw(r1 r2)],
##            "volume-name-c-mode"               => [qw(r1234 r2234)],
##            "aggregate-name"                   => [qw(aggr_trans aggr_trans)],
##       );
##
##       my %schedule1=(
##            "schedule-name"                    => "new1",
##            "days-range"                       => "1-4",
##            "start-time"                       => "12:00",
##            "duration"                         => "20:30",
##            "available-transfers-percentage"   => "4",
##            "session-snapmirror-throttle"      => "230",
##            "update-frequency"                 => "1:30",
##       );
##       my %schedule2=(
##            "schedule-name"                    => "new2",
##           "days-range"                        => "0,5,6",
##            "start-time"                       => "12:00",
##            "duration"                         => "20:30",
##            "available-transfers-percentage"   => "4",
##            "session-snapmirror-throttle"      => "230",
##            "update-frequency"                 => "1:30",
##        );
##        my @schedule=(\%schedule1,\%schedule2);
##        $Api_Set_Obj2->transition_create(
##             "session-name"              => "session2",
##             "session-type"              => "standalone",
##             "7-mode-host-name"          => "10.227.202.109",
##             "data-copy-ipaddress"       => "10.227.202.109",
##             "vfiler"                    => "vfiler0",
##             "multipathing-ipaddress"    => "10.227.202.110",
##             "c-mode-host-name"          => "10.227.65.55",
##             "svm-name"                  => "r1",
##             "vserver-rootvol-aggr"      => { type => SCALAR },
##             S_mode_credentials          => \%S_mode_credentials,
##             C_mode_credentials          => \%C_mode_credentials,
##             volumepair_add_args         => \%volume,
##             schedule_add_args           => \@schedule,
##             "seven-mode-only"           => 1,
##        );
##&nbsp;transition create -s S21 -t standalone -n 10.227.202.110 -c 10.227.202.110 -f vfiler0 -h 10.226.99.61 -v vs_sec -d daily :
##&nbsp;

sub transition_create {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"              => { type => SCALAR , optional => 1},
            "session-type"              => { type => SCALAR , optional => 1},
            "group-name"                => { type => SCALAR , optional => 1},
            "7-mode-host-name"          => { type => SCALAR , optional => 1},
            "data-copy-ipaddress"       => { type => SCALAR , optional => 1},
            "vfiler"                    => { type => SCALAR , optional => 1},
            "multipathing-ipaddress"    => { type => SCALAR , optional => 1},
            "c-mode-host-name"          => { type => SCALAR , optional => 1},
            "svm-name"                  => { type => SCALAR , optional => 1},

            "S_mode_credentials"        => { optional => 1  },
            "C_mode_credentials"        => { optional => 1  },
            "volumepair_add_args"       => { optional => 1  },
            "schedule_add_args"         => { type => ARRAYREF, optional => 1},
            "seven-mode-only"           => { type => SCALAR , optional => 1},

            "primary_info"              => { optional => 1 },
            "secondary_info"            => { optional => 1 },
            "plan_configuration"        => { optional => 1 },
            "create_new_grp"            => {optional   => 1},
        },
        allow_extra => 1
    );
    $Log->enter() if $may_enter;
    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    if (defined $args{primary_info} && defined $args{secondary_info} ) {
        my $primary_info        = delete $args{primary_info};
        my $secondary_info      = delete $args{secondary_info};

        my $S_mode_credentials_P = delete $primary_info->{S_mode_credentials};
        my $C_mode_credentials_P = delete $primary_info->{C_mode_credentials};

        $self->_click(
            id                      => $STORAGE_SYSTEMS,
            error_checking_needed   => 0,
        ) ;
        $self->_add_credentials( %{$S_mode_credentials_P} );
        $self->_add_credentials( %{$C_mode_credentials_P} );
        $self->_close_tab(
                string => "Storage",
        );

        $secondary_info->{volumepair_add_args}->{"volume-name-7-mode"}
            = $primary_info->{volumepair_add_args}->{"volume-name-7-mode"};
        $secondary_info->{"7-mode-host-name"} = $primary_info->{"7-mode-host-name"};

        my $ip = $primary_info->{"c-mode-host-name"};
        my $host_smode = $self->_cbt_get_host_from_ip($ip,"c");
        my $host;
        if($host_smode =~ /(.*)\($ip\)/){
            $host = $1;
        }
        $host =~ s/(.*)\s+/$1/;
        print Dumper($host);
        push @{$self->{$primary_info->{"session-name"}}} , { transition_volumepair_add_args =>$primary_info->{"volumepair_add_args"}} ;
        $self->_put_value_for_key_x_key_y_in_session_z(
            "transition_volumepair_add_args","c_hostname",$primary_info->{"session-name"},$host);
        $self->_create_helper($secondary_info);
        my $prim_sess = $primary_info->{"session-name"};
        $primary_info->{"session-name"} = $secondary_info->{"session-name"}."_Primary"; #project names are set as "Secondary projectName_Primary" and "Secondary projectName_Secondary"
        my $sec_sess_exist = $secondary_info->{"session-name"}."_Secondary";
        my $sec_sess_req = $secondary_info->{"session-name"};
        my $group = undef;
        $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
        print Dumper("creating primary project");
        $self->_create_primary($primary_info,$prim_sess,$sec_sess_exist,$sec_sess_req,$group);
    } else {
        $self->_create_helper(\%args);
    }
    $Log->exit() if $may_exit;

    return 1;
} ## end sub transition_create

## @name ::transition_volumepair_add
##
## @summary Executes "transition_volumepair_add" command
##
## @description
##    This method is used to execute transition volumepair add command
## @synopsis
##      transition_volumepair_add(%args);
## INPUTS TO METHOD CALL 'transition_volumepair_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'volume-name-c-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'aggregate-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_volumepair_add(
##            "session-name"              => "session2",
##            "volume-name-7-mode"        => [qw(r1)],
##            "volume-name-c-mode"        => [qw(r1_1)],
##            "aggregate-name"            => [qw(aggr_trans)],
##         );
##
##&nbsp;transition volumepair add -s S1 -v vol1 -c vol_compo1 -g aggr_trans :
##&nbsp;

sub transition_volumepair_add {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"              => { type => SCALAR },
            "volume-name-7-mode"        => { type => ARRAYREF },
            "volume-name-c-mode"        => { type => ARRAYREF , optional => 1},
            "aggregate-name"            => { type => ARRAYREF , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Storing volume pair add arguments in the object
    foreach (@{$self->{$args{"session-name"}}}){
        undef $_ if (defined $_->{transition_volumepair_add_args});
    }
    @{$self->{$args{"session-name"}}} = grep defined, @{$self->{$args{"session-name"}}};
    push @{$self->{$args{"session-name"}}} , { transition_volumepair_add_args => \%args} ;

    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $EDIT_CONFIGURATION,
            error_checking_needed   => 1,
        ) ;

        my $sleep_count = $GENERIC_WAIT / 1000;
        Tharn::snooze($sleep_count);

        $self->_add_volumepairs_for_session($args{"session-name"},1);

        $self->_click(
            id                      => $SAVE_PROJECT_AND_CONTINUE,
            error_checking_needed   => 1,
        ) ;

        if ( (defined $args{"aggregate-name"}) || (defined $args{"volume-name-c-mode"}) ){
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 0,
            ) ;

            # Skipping prechecks
            $self->_click(
                id                      => $SKIP,
                error_checking_needed   => 0,
            ) ;

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            # Aggr selection page
            $self->_select_aggregate($args{"session-name"})
                if defined ($args{"aggregate-name"});

            # Volume mapping
            $self->_do_volume_mapping($args{"session-name"})
                if defined ($args{"volume-name-c-mode"});

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;
        }

        $self->_close_tab(
            session => $args{"session-name"},
        );

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        if($self->_is_key_x_present_for_session_y("transition_create_args",$args{"session-name"})){

            $Log->comment("Creating session and adding volumepairs");

            # Session does not exist
            $self->_close_tab(
                string => "Dashboardtab",
            ) if( $dashboard_enabled ) ;

            # Get hostname from ip and store it in APISet object in session hash
            my $ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","7-mode-host-name",$args{"session-name"});
            my $host = $self->_get_host_from_ip($ip,"s");
            $self->_put_value_for_key_x_key_y_in_session_z(
                "transition_volumepair_add_args","s_hostname",$args{"session-name"},$host);

            $ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","c-mode-host-name",$args{"session-name"});
            $host = $self->_get_host_from_ip($ip,"c");
            $self->_put_value_for_key_x_key_y_in_session_z(
                "transition_volumepair_add_args","c_hostname",$args{"session-name"},$host);

            # Project details
            $self->_smtt_navigate($NAVIGATE_TRANSITION_CREATE);

            # Create new project here
            $Log->debug("Creating new project");
            $self->_type( $PROJECT_NAME_INPUT ,$args{"session-name"} );
            $self->_click(
                id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                error_checking_needed   => 0,
            ) ;

            # Wait till volume pairs are loaded
            $self->_check_for_error();

            # Select volumepairs
            $self->_add_volumepairs_for_session($args{"session-name"});

            # Continuing with the remaining part of session creation
            $self->_click(
                id                      => $CREATE_PROJECT_AND_CONTINUE,
                error_checking_needed   => 1,
            ) ;

            # Changing session name to desired value
            $self->_change_session_name_to_given_value($args{"session-name"});

            my $dc_ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","data-copy-ipaddress",$args{"session-name"});
            $self->_type( $DATA_COPY_IP, $dc_ip ) if( $dc_ip );

            my $m_ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","multipathing-ipaddress",$args{"session-name"});
            $self->_type( $MULTIPATH_IP, $m_ip ) if( $m_ip );

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 0,
            ) ;

            # Skipping prechecks
            $self->_click(
                id                      => $SKIP,
                error_checking_needed   => 0,
            ) ;

            # Vserver selection page
            $self->_add_vserver($args{"session-name"});
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            # Aggr selection page
            $self->_select_aggregate($args{"session-name"})
                if defined ($args{"aggregate-name"});

            # Volume mapping
            $self->_do_volume_mapping($args{"session-name"})
                if defined ($args{"volume-name-c-mode"});

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            $self->_close_tab(
                session => $args{"session-name"},
            );

            $Log->comment("Session created and volumepairs added successfully");

        }else{
            $self->_throw_exception("Session ".$args{"session-name"}." does not exist "."
                and create arguments not present in the APISet object.\n ".
                "Make sure that you invoke transition_volumepair_add() on the same APISet".
                " object \n on which you invoked transition_create() or session should exist"
            );
        }
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub transition_volumepair_add

### @name :: transition_volumepair_remove
##
## @summary Executes "transition_volumepair_remove" command
##
## @description
##  This method is used to execute transition volume remove from session
## @synopsis
##  transition_volumepair_remove(%args);
## INPUTS TO METHOD CALL 'transition_volumepair_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_volumepair_remove(
##            "session-name"              => "session2",
##            "volume-name-7-mode"        => [qw(r1 r2)],
##         );
##
##&nbsp;transition volumepair remove -s S1 -v vol1 :
##&nbsp;

sub transition_volumepair_remove {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"              => { type => SCALAR },
            "volume-name-7-mode"        => { type => ARRAYREF },
            "group-name"                    => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Dashboard and selecting a subproject");
    my $project = $args{"session-name"};
    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
    }
    $self->_project_selector($group);
    my $locator = "gwt-debug-".$project;
    if ($self->_is_element_present($locator)) {
            $id = $locator;
        }
    if($id){
        # Session exists
        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        );
        $self->_click(
            id                      => $VOLUME_TAB,
            error_checking_needed   => 0,
        );
        my $sleep_count = $GENERIC_WAIT / 1000;
        sleep $sleep_count;
        my $vol_ar = $args{"volume-name-7-mode"};
        foreach (0..@{$vol_ar}-1){
            my $vol = $vol_ar->[$_];
            my $remove=$VOLUME_REMOVE;
            $remove =~ s/(.*)XXYYZZ(.*)/$1$vol$2/;
            if($self->_is_element_enabled($remove) ){
                $self->_click(
                    id                      => $remove,
                    error_checking_needed   => 1,
                ) ;
            my $popup_title;

                eval{
                        $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
                };
            if ($popup_title =~ /Error/i) {
                        my  $popup_text=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_MESSAGE);
                        $self->_create_screenshot("$popup_text");
                        $self->_repair_session();
                        $self->_throw_exception(
                            "Popup Title : $popup_title , Popup error message : $popup_text");
            }elsif( ($popup_title !~ /Operation Status/i ) && ($popup_title =~ /Information/i || $popup_title =~ /Warning/i  )){
                    my $popup_txt;
                    eval{
                        $popup_txt=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_MESSAGE);
                    };
                }
                eval{
                        $self->_click(
                                id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                                error_checking_needed   => 0,
                        ) ;
                    };
            }else{
                $self->_take_screenshot();
                $self->_repair_session();
                $self->_throw_exception("Probably the volume requested for does not exist or already removed.. Unable to find the element with ".$remove);
            }
            $Log->comment("Successfully removed volumes from session");
        }

        $self->_close_tab(
            string => "Migration",
        );

    }else{
            $self->_throw_exception("Session ".$args{"session-name"}." does not exist ")
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub transition_volumepair_remove

### @name :: transition_volume_mount_modify
##
## @summary Executes "transition_volume_mount_modify" command
##
## @description
##  This method is used to execute modify volume mount policy in transition
## @synopsis
##  transition_volume_mount_modify(%args);
## INPUTS TO METHOD CALL 'transition_volume_mount_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'mount_policy' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_volume_mount_modify(
##            "project-name"        => "session2",
##            "mount_policy"        => "Use clustered Data ONTAP volume name",
##         );
##
##&nbsp;

sub transition_volume_mount_modify {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"              => { type => SCALAR },
            "mount_policy"              => { type => SCALAR  },
            "group-name"                => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    my $volume_mount  = $args{"mount_policy"};
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Dashboard and selecting a subproject");
    my $project = $args{"project-name"};
    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
    }
    $self->_project_selector($group);
    my $locator = "gwt-debug-".$project;
    if ($self->_is_element_present($locator)) {
            $id = $locator;
        }
    if($id){
        # Session exists
        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        );
        $self->_click(
            id                      => $VOLUME_TAB,
            error_checking_needed   => 0,
        );
        my $sleep_count = $GENERIC_WAIT / 1000;
        sleep $sleep_count;
        $self->_click(
            id                    => $EDIT_DASHBOARD_MOUNT,
            error_checking_needed => 0,
        );
        my $policyselector =  $GENERIC_LISTITEM_SELECTOR;
        $policyselector =~ s/(.*)XXYYZZ(.*)/$1$volume_mount$2/;
        if($self->_is_element_present($policyselector)){
            $self->_click(
                id                      => $policyselector,
                error_checking_needed   => 0,
            );
        } else {
            $self->_create_screenshot($project."_Target_Mount_Policy_Not_Found");
            $self->_repair_session();
            $self->_throw_exception($volume_mount." is not present in the Target mount policy list");
        }
        $self->_close_tab(
            string => "Migration",
        );
    }else{
        $self->_throw_exception("project ".$args{"project-name"}." does not exist ")
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub transition_volume_mount_modify

# @name :: transition_operation_history
##
## @summary Executes "transition_opeartion_history" command
##
## @description
##  This method is used to get the parsed output of performed operation from session
## @synopsis
##  transition_operation_history(%args);
## INPUTS TO METHOD CALL 'transition_operation_history' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'operation-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_operation_history(
##            "session-name"              => "session2",
##            "operation-name"        =>     "Precheck"
##         );
##
##&nbsp; :
##&nbsp;

sub transition_operation_history {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"          => { type => SCALAR },
            "operation-name"        =>    { type => SCALAR },
            "group-name"            => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );
    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Dashboard and selecting a subproject");
    my $project = $args{"session-name"};
    my $operation = $args{"operation-name"};
    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
    }
    $self->_project_selector($group);
    my $locator = "gwt-debug-".$project;
    if ($self->_is_element_present($locator)) {
            $id = $locator;
    }
    if($id){
        # Session exists
        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        );
        $self->_click(
            id                      => $PROJECT_HISTORY_TAB,
            error_checking_needed   => 0,
        );
        my $sleep_count = $GENERIC_WAIT / 1000;
        sleep $sleep_count;
        my $view_result= $OPERATION_RESULT;
        $view_result =~ s/(.*)XXYYZZ(.*)/$1$operation$2/;
        sleep(8);
        my ($aref, $output);
        if($self->_is_element_enabled($view_result) ){
            $aref = $self->_cft_click(
                id                      => $view_result,
                error_checking_needed   => 1,
                arguments               => \%args,
            ) ;
        }
        if (defined $aref->[0]) {
            $output->{parsed_output} = $aref->[0] ;
            return $output;
        }else {
            $Log->exit() if $may_exit;
            return undef;
        }
    }
}

## @name :: transition_move_project_group
##
## @summary Executes "transition_move_project_group" button
##
## @description
##  This method is used to execute transition_move_project_group 7MTT button
## @synopsis
##  transition_move_project_group(%args);
## INPUTS TO METHOD CALL 'transition_move_project_group' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'new-group' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_move_project_group(
##            "project-name"             => "prGroupchange1",
##            "group-name"               => "group1",
##            "new-group"                => "group2",
##         );
##&nbsp;
sub transition_move_project_group {

    my $self = shift;
    my $sleep_count = $GENERIC_WAIT / 1000;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"                          => { type => SCALAR },
            "group-name"                            => { type => SCALAR },
            "new-group"                             => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    my $project_name = $args{"project-name"};
    my $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    my $new_group = $args{"new-group"};
    print Dumper($new_group);
    $Log->enter();

    my $id = undef;
    my $flag = 0;
    my $hash_ref={};

    if( $self->_is_button_enabled($MIGRATION_DASHBOARD_LINK) ) {
        $self->_click(
            id                      => $MIGRATION_DASHBOARD_LINK,
            error_checking_needed   => 1,
        ) ;
        $self->_project_selector($group);
        my $locator = "gwt-debug-".$project_name;
        if ($self->_is_element_present($locator)) {
            $id = $locator;
        }
    } else {
        $Log->comment("Migration Tab is not present");
        $self->_close_tab(
            string => "Migration",
        );
    }
    if(defined $id){
        my $project_select = $MOVE_PROJECT;
        $project_select =~ s/XXYYZZ/$project_name/;
        $self->_click(
            id                      => $project_select,
            error_checking_needed   => 0,
        ) ;
        $self->_click(
            id                      => $GROUP_MOVE,
            error_checking_needed   => 0,
        ) ;
        $self->_create_screenshot("group_move");
        if (defined $new_group) {
            $self->_click(
                id                      => $SELECT_GROUP,
                error_checking_needed   => 0,
            ) ;
            $self->_create_screenshot("select_group");
            my $existing_group = $EXISTING_GROUP;
            $existing_group =~ s/(.*)XXYYZZ(.*)/$1$new_group$2/;
            if ($self->_is_element_present($existing_group)) {
                $self->_click(
                    id                      => $existing_group,
                    error_checking_needed   => 0,
                ) ;
            }else{
                $self->_click(
                    id                      => $CREATE_NEW_GROUP,
                    error_checking_needed   => 0,
                ) ;
                $self->_type( $GROUP_NAME ,$new_group);
            }
            $self->_create_screenshot($project_name."_moved_to_".$new_group);
            $self->_click(
                id                      => $SAVE_PROJECT,
                error_checking_needed   => 1,
            ) ;
            $self->_create_screenshot($project_name."new_group");
        }
        $self->_close_tab(
            string => "Migration",
        );
    } else {
        $self->_create_screenshot($project_name."move_group_Error");
        $self->_repair_session();
        $self->_throw_exception("Project $project_name is not present in the given group $group");
    }
    $Log->exit();
    return 1;
} ## end sub transition_move_project_group

## @name :: transition_lif_add
##
## @summary Executes "transition_lif_add" command
##
## @description
##  This method is used to execute transition lif add command
## @synopsis
##  transition_lif_add(%args);
## INPUTS TO METHOD CALL 'transition_lif_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ip-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'default-gateway' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'home-node' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'home-port' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_volumepair_remove(
##            "session-name"             => "session2",
##            "ip-address"        => "1.12.255.7",
##            "default-gateway"           => "1.12.255.1",
##            "netmask"           => "255.0.0.0",
##            "home-node"              => "csiqa-2240-03",
##            "home-port"              => "e0d",
##            "origin-ip-address"        => "1.12.255.7",
##         );
##&nbsp;
sub transition_lif_add {

    my $self = shift;
    my $sleep_count = $GENERIC_WAIT / 1000;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                          => { type => SCALAR },
            "ip-address"                            => { type => SCALAR },
            "default-gateway"                       => { type => SCALAR , optional => 1},
            "netmask"                               => { type => SCALAR , optional => 1},
            "home-node"                             => { type => SCALAR , optional => 1},
            "home-port"                             => { type => SCALAR , optional => 1},
            "origin-ip-address"                     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );
    print Dumper(\%args);
    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 0,
        ) ;
        my $locator = "gwt-debug-".$args{"session-name"};
        if ($self->_is_element_present($locator)) {
            $id = $locator;
        }
    }

    if($id){
        # Session exists
        my $session_name = $args{"session-name"};
        my $project_edit_config = $PROJECT_EDIT_CONFIG;
        $project_edit_config =~ s/XXYYZZ/$session_name/;
        if($self->_is_element_present($project_edit_config)){
            $self->_click(
                id                      => $project_edit_config,
                error_checking_needed   => 0,
            ) ;
            $self->_click(
                id                      => $NETWORKING_TAB,
                error_checking_needed   => 1,
            ) ;
        }

        sleep $sleep_count;
        #If origin IP address specified choose IP address from 7mode
        if (defined $args{"origin-ip-address"}) {
            my $exist_lif_select = $EXISTING_LIF_RADIO_BUTTON;
            $exist_lif_select =~ s/XXYYZZ/$args{"origin-ip-address"}/;
            $self->_click(
                id                      => $exist_lif_select,
                error_checking_needed   => 0,
            );
            $self->_click(
                id                      => $CHOOSE_NODE_PORT,
                error_checking_needed   => 0,
            );
            my $home_node = $GENERIC_HOME_NODE;
            $home_node =~ s/XXYYZZ/$args{"home-node"}/;
            $self->_click(
                id                      => $home_node,
                error_checking_needed   => 0,
            );
            my $home_port = $EDIT_HOME_PORT;
            $home_port =~ s/XXYYZZ/$args{"home-port"}/;
            $self->_click(
                id                      => $home_port,
                error_checking_needed   => 0,
            );
            $self->_click(
                id                      => $GENERIC_SAVE_NODE,
                error_checking_needed   => 0,
            );
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            );
            $self->_wait_for_element_not_present($GENERIC_SPINNING_ICON);
            sleep $sleep_count;
            $self->_close_tab(
                string => "\(CBT\)",
            );
        } else {
            #Choose add new LIF option
            $self->_click(
                id                      => $GENERIC_ADDLIF_BUTTON,
                error_checking_needed   => 0,
            );
            print Dumper("hello1");
            $self->_type($GENERIC_TARGET_IP_INPUT,$args{"ip-address"});
            
            if (defined $args{"default-gateway"}){
                print Dumper("hello2");
                $self->_type( $GENERIC_TARGET_GATEWAY_INPUT,$args{"default-gateway"});
            }

            if (defined $args{"netmask"}){
                print Dumper("hello3");
                $self->_type( $GENERIC_TARGET_NETMASK_INPUT,$args{"netmask"});
            }

            if (defined $args{"home-node"}){
                print Dumper("hello4");
                $self->_click(
                    id                      => $GENERIC_TARGET_NODE_INPUT,
                    error_checking_needed   => 0,
                );
            # Sleeping to allow drop-down some time to appear
               sleep $sleep_count;
               my $node = $args{"home-node"};
               my $nodeselector =  $GENERIC_LISTITEM_SELECTOR;
               $nodeselector =~ s/(.*)XXYYZZ(.*)/$1$node$2/;
               $self->_click(
                    id                      => $nodeselector,
                    error_checking_needed   => 0,
                );
            }

            if (defined $args{"home-port"}){
                $self->_click(
                    id                      => $GENERIC_TARGET_PORT_INPUT,
                    error_checking_needed   => 0,
                );
             # Sleeping to allow drop-down some time to appear
                sleep $sleep_count;
                my $port = $args{"home-port"};
                my $portselector =  $GENERIC_LISTITEM_SELECTOR;
                $portselector =~ s/(.*)XXYYZZ(.*)/$1$port$2/;
                $self->_click(
                    id                      => $portselector,
                    error_checking_needed   => 0,
                );
            }

            $self->_click(
                id                      => $GENERIC_SAVELIF_BUTTON,
                error_checking_needed   => 1,
            );
            $self->_create_screenshot("Lif_Add_".$session_name);
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            );
            $self->_wait_for_element_not_present($GENERIC_SPINNING_ICON);
            sleep $sleep_count;
            $self->_close_tab(
                string => "\(CBT\)",
            );
        }

        $self->_close_tab(
            string => "Migration",
        );
    }else{
        $self->_create_screenshot("Lif_Add_Error");
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub transition_lif_add

## @name :: transition_lif_remove
##
## @summary Executes "transition_lif_remove" command
##
## @description
##  This method is used to execute transition lif remove from session
## @synopsis
##  transition_lif_remove(%args);
## INPUTS TO METHOD CALL 'transition_lif_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ip address' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_lif_remove(
##            "session-name"              => "session2",
##            "ip-address"                => [qw(1.12.255.2 1.12.255.3)],
##         );
##
##&nbsp;
##&nbsp;

sub transition_lif_remove {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"              => { type => SCALAR },
            "ip-address"                => { type => ARRAYREF },
            "group-name"                => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    $Log->comment("Navigating to Dashboard and selecting a subproject");
    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
    }
    $self->_project_selector($group);
    my $locator = "gwt-debug-".$args{"session-name"};
    if ($self->_is_element_present($locator)) {
            $id = $locator;
    }
    if($id){
        # Session exists
        my $project_edit_config = $PROJECT_EDIT_CONFIG;
        $project_edit_config =~ s/XXYYZZ/$args{"session-name"}/;
        if($self->_is_element_present($project_edit_config)){
            $self->_click(
                id                      => $project_edit_config,
                error_checking_needed   => 0,
            ) ;
            $self->_click(
                id                      => $NETWORKING_TAB,
                error_checking_needed   => 1,
            ) ;
        }
        my $sleep_count = $GENERIC_WAIT / 1000;
        sleep $sleep_count;
        my $ip_ar = $args{"ip-address"};
        foreach my $ipaddress (@{$ip_ar}){
            my $remove=$LIF_REMOVE;
            $remove =~ s/(.*)XXYYZZ(.*)/$1$ipaddress$2/;
            if($self->_is_element_enabled($remove) ){
                $self->_click(
                    id                      => $remove,
                    error_checking_needed   => 1,
                );
            }else{
                $self->_create_screenshot("Lif_Remove_Error");
                $self->_repair_session();
                $self->_throw_exception("Probably the lif requested does not exist or already removed.. Unable to find the element with ".$remove);
            }
            $Log->comment("Successfully removed $ipaddress from session");
        }
      $self->_wait_for_element_not_present($GENERIC_SPINNING_ICON);
      sleep $sleep_count;
      $self->_close_tab(
        string => "\(CBT\)",
      );
      $self->_close_tab(
        string => "Migration",
      );
    }else{
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist ")
    }
    $Log->exit() if $may_exit;

    return 1;
} ## end sub transition_lif_remove

## @name :: transition_schedule_add
##
## @summary Executes "transition_schedule_add" command
##
## @description
##  This method is used to execute transition schedule add command
## @synopsis
##  transition_schedule_add(%args);
## INPUTS TO METHOD CALL 'transition_shedule_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'days-range' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'start-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'duration' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'update-frequency' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##@examples
##&nbsp;transition schedule add -s S21 -n sc1 -d 1-5 -b 12:00 -e 10:00 -t 2 -x 2 :
##&nbsp;
##&nbsp;

sub transition_schedule_add {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            "schedule-name"                     => { type => SCALAR },
            "days-range"                        => { type => SCALAR },
            "start-time"                        => { type => SCALAR },
            "duration"                          => { type => SCALAR },
            "update-frequency"                  => { type => SCALAR , optional => 1},
            "available-transfers-percentage"    => { type => SCALAR , optional => 1},
            "max-cap-concurrent-transfers"      => { type => SCALAR , optional => 1},
            "session-snapmirror-throttle"       => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $SCHEDULE_TAB,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $CREATE_SCHEDULE_DASHBOARD,
            error_checking_needed   => 0,
        ) ;

        $self->_type( $GENERIC_SCHEDULE_NAME ,$args{"schedule-name"});

        $self->_add_schedule_days($args{"days-range"});

        # Adding start time
        my @s_t = split(":",$args{"start-time"});
        my $selector=$GENERIC_AGGR_SELECTOR;
        $self->_click(
            id                      => $GENERIC_SCHEDULE_START_TIME_HOUR_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[0]$2/;
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;
        $self->_click(
            id                      => $GENERIC_SCHEDULE_START_TIME_MIN_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector=$GENERIC_AGGR_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[1]$2/;
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;

        # Adding duration
        my @duration = split(":",$args{"duration"});
        $self->_type( $GENERIC_SCHEDULE_DURATION_HOUR_INPUT ,$duration[0] );
        $self->_click(
            id                      => $GENERIC_SCHEDULE_DURATION_MIN_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector=$GENERIC_AGGR_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$duration[1]$2/;
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;

        if ( defined $args{"available-transfers-percentage"} ) {
            # Adding available-transfers-percentage
            $self->_click(
                id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) ); 
            $self->_type(
                $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,$args{"available-transfers-percentage"} );
        }

        if ( defined $args{"max-cap-concurrent-transfers"} ) {
            # Adding stream count
            $self->_click(
                id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) );
            $self->_type( $GENERIC_STREAM_SELECTOR ,$args{"max-cap-concurrent-transfers"} );
        }

        # Adding throttle
        if (defined $args{"session-snapmirror-throttle"}){
            $self->_click(
                id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                error_checking_needed   => 1,
            ) ;
            $self->_type( $GENERIC_THROTTLE_SELECTOR ,$args{"session-snapmirror-throttle"} );
        }

        # Adding update frequency
        if (defined $args{"update-frequency"}){
            my @u_f = split(":",$args{"update-frequency"});
            $self->_type( $GENERIC_SCHEDULE_UPDATE_FREQUENCY_HOUR_INPUT ,$u_f[0]);
            $self->_click(
                id                      => $GENERIC_SCHEDULE_UPDATE_FREQUENCY_MIN_INPUT,
                error_checking_needed   => 0,
            ) ;
            $selector=$GENERIC_AGGR_SELECTOR;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$u_f[1]$2/;
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;
        }

        $self->_click(
            id                      => $GENERIC_SCHEDULE_CREATE_BUTTON,
            error_checking_needed   => 1,
        ) ;

        if($self->_is_element_present($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE)){
            my $text = $self->_get_text($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE);
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Error while adding schedule : ".$text);
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub transition_schedule_add


## @name ::transition_precheck
##
## @summary Executes "transition_precheck" command
##
## @description
##    This method is used to execute transition precheck command
## @synopsis
##      transition_precheck(%args);
## INPUTS TO METHOD CALL 'transition_precheck' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'seven-mode-only' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##
## @examples
##        $Api_Set_Obj2->transition_precheck(
##            "session-name"                  => "session2",
##            'wait_for_operation'            => 1,
##            "method-timeout"                => 1000
##        );
##&nbsp;

sub transition_precheck {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                  => { type => SCALAR },
            'seven-mode-only'               => { type => SCALAR , optional => 1},
            'wait_for_operation'            => { type => SCALAR , optional => 1},
            'method-timeout'                => { type => SCALAR , optional => 1},
            "group-name"                    => { type => SCALAR , optional => 1},
            'parsed-output'                 => { type => SCALAR , optional => 1},
            'run-background'                => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "CBT_Precheck";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }

    my $state = undef;
    if ($parsed_output != 1) {
        $state = $WAIT_FOR_PRECHECK;
    }
    $args{"trans_mode"} = $MIGRATION_CBT_TAB;
    my $result_array = $self->_perform_operation($args{"session-name"},
        $RUN_PRECHECKS,$group,$operation,$parsed_output,$run_background,$state,\%args);

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;
} ## end sub transition_precheck

## @name ::transition_start
##
## @summary Executes "transition_start" command
##
## @description
##    This method is used to execute transition start command
## @synopsis
##      transition_start(%args);
## INPUTS TO METHOD CALL 'transition_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'no-precheck' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_start(
##            "session-name"                  => "session2",
##            'wait_for_operation'            => 1,
##            "method-timeout"                => 1000
##         );
##&nbsp;transition start -r no -s S21 :
##&nbsp;

sub transition_start {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                  => { type => SCALAR },
            'no-precheck'                   => { type => SCALAR , optional => 1},
            'wait_for_operation'            => { type => SCALAR , optional => 1},
            'method-timeout'                => { type => SCALAR , optional => 1},
            "group-name"                    => { type => SCALAR , optional => 1},
            'parsed-output'                 => { type => SCALAR , optional => 1},
            'run-background'                => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "CBT_start";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }

    my $state = undef;
    if ($parsed_output != 1) {
        $state = $WAIT_FOR_START;
    }
    $args{"trans_mode"} = $MIGRATION_CBT_TAB;
    my $result_array = $self->_perform_operation($args{"session-name"},
        $START,$group,$operation,$parsed_output,$run_background,$state,\%args);

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;
} ## end sub transition_start

## @name :: transition_update
##
## @summary executes "transition_update" command
##
## @description
##  This method is used to execute transition update command
## @synopsis
##  transition_update(%args);
## INPUTS TO METHOD CALL 'transition_update' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
##&nbsp;

sub transition_update {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                   => { type => SCALAR },
            'available-transfers-percentage' => { type => SCALAR },
            'session-snapmirror-throttle'    => { type => SCALAR , optional => 1},
            "max-cap-concurrent-transfers"   => { type => SCALAR , optional => 1},
            'wait_for_operation'             => { type => SCALAR , optional => 1},
            'method-timeout'                 => { type => SCALAR , optional => 1},
            "group-name"                     => { type => SCALAR , optional => 1},
            'parsed-output'                  => { type => SCALAR , optional => 1},
            'run-background'                 => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "CBT_update";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    my $state = undef;# update button has no colour change after execution
    $args{"trans_mode"} = $MIGRATION_CBT_TAB;
    my $result_array = $self->_perform_operation($args{"session-name"},
        $UPDATE_NOW,$group,$operation,$parsed_output,$run_background,$state,
        \%args
    );

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;
} ## end sub transition_update

## @name ::transition_pause
##
## @summary executes "transition_pause" command
##
## @description
##    This method is used to execute transition pause command
## @synopsis
##      transition_pause(%args);
## INPUTS TO METHOD CALL 'transition_pause' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##&nbsp;

sub transition_pause {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                   => { type => SCALAR },
            'wait_for_operation'             => { type => SCALAR , optional => 1},
            'method-timeout'                 => { type => SCALAR , optional => 1},
            "group-name"                     => { type => SCALAR , optional => 1},
            'parsed-output'                  => { type => SCALAR , optional => 1},
            'run-background'                 => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "CBT_pause";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    my $state = undef;# pause button has no colour change after execution
    $args{"trans_mode"} = $MIGRATION_CBT_TAB;
    my $result_array = $self->_perform_operation($args{"session-name"},
        $PAUSE,$group,$operation,$parsed_output,$run_background,$state,\%args);

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;
} ## end sub transition_pause

## @name ::transition_resume
##
## @summary executes "transition_resume" command
##
## @description
##    This method is used to execute transition resume command
## @synopsis
##      transition_resume(%args);
## INPUTS TO METHOD CALL 'transition_resume' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##&nbsp;

sub transition_resume {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            'wait_for_operation'                => { type => SCALAR , optional => 1},
            'method-timeout'                    => { type => SCALAR , optional => 1},
            "group-name"                        => { type => SCALAR , optional => 1},
            'parsed-output'                     => { type => SCALAR , optional => 1},
            'run-background'                    => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "CBT_resume";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    print Dumper($parsed_output);
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }

    my $state = undef;# resume button has no colour change after execution
    $args{"trans_mode"} = $MIGRATION_CBT_TAB;
    my $result_array = $self->_perform_operation($args{"session-name"},
        $RESUME,$group,$operation,$parsed_output,$run_background,$state,\%args);

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;
} ## end sub transition_resume

## @name :: transition_precutover
##
## @summary Executes "transition_precutover" command
##
## @description
##  This method is used to execute transition precutover command
## @synopsis
##  transition_precutover(%args);
## INPUTS TO METHOD CALL 'transition_precutover' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'snapmirror-throttle' <td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'ad-server-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ad-server-ip-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'ad-server-admin-user-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'ad-server-password' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'kdc-server-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-y' </td> <td> 'kdc-server-admin-user-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-z' </td> <td> 'kdc-server-password' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
##&nbsp;

sub transition_precutover {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                  => { type => SCALAR },
            'mode'                          => { type => SCALAR },
            'available-transfers-percentage'=> { type => SCALAR },
            "max-cap-concurrent-transfers"  => { type => SCALAR , optional => 1},
            'snapmirror-throttle'           => { type => SCALAR , optional => 1},
            'ad-server-host-name'           => { type => SCALAR , optional => 1},
            'ad-server-ip-address'          => { type => SCALAR , optional => 1},
            'ad-server-admin-user-name'     => { type => SCALAR , optional => 1},
            'ad-server-password'            => { type => SCALAR , optional => 1},
            'kdc-server-host-name'          => { type => SCALAR , optional => 1},
            'kdc-server-admin-user-name'    => { type => SCALAR , optional => 1},
            'kdc-server-password'           => { type => SCALAR , optional => 1},
            'wait_for_operation'            => { type => SCALAR , optional => 1},
            'method-timeout'                => { type => SCALAR , optional => 1},
            "group-name"                    => { type => SCALAR , optional => 1},
            'parsed-output'                 => { type => SCALAR , optional => 1},
            'run-background'                => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "CBT_precutover";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    print Dumper($parsed_output);
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }

    my $state = undef;
    $args{"trans_mode"} = $MIGRATION_CBT_TAB;
    my $result_array = $self->_perform_operation($args{"session-name"},
        $PRECUTOVER,$group,$operation,$parsed_output,$run_background,$state,
        \%args);

    $Log->exit() if $may_exit;
    return (defined $result_array) ? $result_array : 1;
} ## end sub transition_precutover

## @name :: transition_resync
##
## @summary Executes "transition_resync" command
##
## @description
##  This method is used to execute re-sync on a transition session
## @synopsis
##  transition_resync(%args);
## INPUTS TO METHOD CALL 'transition_resync' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##&nbsp;

sub transition_resync {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            'wait_for_operation'                => { type => SCALAR , optional => 1},
            'method-timeout'                    => { type => SCALAR , optional => 1},
            "group-name"                        => { type => SCALAR , optional => 1},
            'parsed-output'                     => { type => SCALAR , optional => 1},
            'run-background'                    => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "CBT_resync";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    print Dumper($parsed_output);
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }

    my $state = undef; #resync has not color change icon

    $args{"trans_mode"} = $MIGRATION_CBT_TAB;
    my $result_array = $self->_perform_operation($args{"session-name"},
        $RESYNC,$group,$operation,$parsed_output,$run_background,$state,
        \%args);

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;
} ## end sub transition_resync

## @name :: transition_cutover
##
## @summary Executes "transition_cutover" command
##
## @description
##  This method is used to execute transition cutover command
## @synopsis
##  transition_cutover(%args);
## INPUTS TO METHOD CALL 'transition_cutover' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##&nbsp;

sub transition_cutover {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            'available-transfers-percentage'    => { type => SCALAR },
            'session-snapmirror-throttle'       => { type => SCALAR , optional => 1},
            "max-cap-concurrent-transfers"      => { type => SCALAR , optional => 1},
            'wait_for_operation'                => { type => SCALAR , optional => 1},
            'method-timeout'                    => { type => SCALAR , optional => 1},
            "group-name"                        => { type => SCALAR , optional => 1},
            'parsed-output'                     => { type => SCALAR , optional => 1},
            'run-background'                    => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "CBT_cutover";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    print Dumper($parsed_output);
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }

    my $state = undef;
    if ($parsed_output != 1) {
        $state = $WAIT_FOR_CUTOVER;
    }
    $args{"trans_mode"} = $MIGRATION_CBT_TAB;
    my $result_array = $self->_perform_operation($args{"session-name"},
        $CUTOVER,$group,$operation,$parsed_output,$run_background,$state,
        \%args);

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;
} ## end sub transition_cutover

## @name ::transition_abort
##
## @summary Executes "transition_abort" command
##
## @description
##    This method is used to execute transition abort command
## @synopsis
##      transition_abort(%args);
## INPUTS TO METHOD CALL 'transition_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##&nbsp;

sub transition_abort {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                  => { type => SCALAR },
            'wait_for_operation'            => { type => SCALAR , optional => 1},
            'method-timeout'                => { type => SCALAR , optional => 1},
            "group-name"                    => { type => SCALAR , optional => 1},
            'parsed-output'                 => { type => SCALAR , optional => 1},
            'run-background'                => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "CBT_Abort";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    print Dumper($parsed_output);
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }

    my $state = undef;#Abort button has no colour change after execution
    $args{"trans_mode"} = $MIGRATION_CBT_TAB;
    my $result_array = $self->_perform_operation($args{"session-name"},
        $ABORT,$group,$operation,$parsed_output,$run_background,$state,
        \%args);

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;
} ## end sub transition_abort

## @name :: transition_schedule_modify
##
## @summary Executes "transition_schedule_modify" command
##
## @description
##  This method is used to execute transition schedule modify command
## @synopsis
##  transition_schedule_modify(%args);
## INPUTS TO METHOD CALL 'transition_schedule_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'days-range' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'start-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'duration' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'update-frequency' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##@examples
##&nbsp;transition schedule modify -s S21 -n sc1 -d 1-5 -b 12:00 -e 10:00 -t 2 -x 2 :
##&nbsp;
##&nbsp;

sub transition_schedule_modify {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            "schedule-name"                     => { type => SCALAR },
            "days-range"                        => { type => SCALAR , optional => 1},
            "start-time"                        => { type => SCALAR , optional => 1},
            "duration"                          => { type => SCALAR , optional => 1},
            "update-frequency"                  => { type => SCALAR , optional => 1},
            "available-transfers-percentage"    => { type => SCALAR , optional => 1},
            "max-cap-concurrent-transfers"      => { type => SCALAR , optional => 1},
            "session-snapmirror-throttle"       => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $SCHEDULE_TAB,
            error_checking_needed   => 0,
        ) ;

        my $selector=$GENERIC_SCHEDULE_MODIFIER;
        my $ses_name=$args{"schedule-name"};
        $selector =~ s/(.*)XXYYZZ(.*)/$1$ses_name$2/;

        $self->_click(
            id                      => $selector,
            error_checking_needed   => 1,
        ) ;

        # Clearing off all the days
        foreach (0..6){
            my $days = {
                0 => "sunday",
                1 => "monday",
                2 => "tuesday",
                3 => "wednesday",
                4 => "thursday",
                5 => "friday",
                6 => "saturday",
            };
            my $day = $days->{$_};
            my $selector =  $GENERIC_SCHEDULE_DAYS_SELECT;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$day$2/;
            my $checked = $selector.'[@checked=""]';
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) if ( $self->_is_element_present($checked) );
        }

        $self->_add_schedule_days($args{"days-range"});

        # Adding start time
        my @s_t = split(":",$args{"start-time"});
        $selector=$GENERIC_AGGR_SELECTOR;
        $self->_click(
            id                      => $GENERIC_SCHEDULE_START_TIME_HOUR_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[0]$2/; 
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;
        $self->_click(
            id                      => $GENERIC_SCHEDULE_START_TIME_MIN_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector=$GENERIC_AGGR_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[1]$2/;
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;

        # Adding duration
        my @duration = split(":",$args{"duration"});
        $self->_type( $GENERIC_SCHEDULE_DURATION_HOUR_INPUT ,$duration[0] );
        $self->_click(
            id                      => $GENERIC_SCHEDULE_DURATION_MIN_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector=$GENERIC_AGGR_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$duration[1]$2/;
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;

        if ( defined $args{"available-transfers-percentage"} ) {
            # Adding available-transfers-percentage
            $self->_click(
                id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) );
            $self->_type(
                $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,$args{"available-transfers-percentage"} );
        }

        if ( defined $args{"max-cap-concurrent-transfers"} ) {
            # Adding stream count
            $self->_click(
                id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) );
            $self->_type( $GENERIC_STREAM_SELECTOR ,$args{"max-cap-concurrent-transfers"} );
        }

        # Adding throttle
        if (defined $args{"session-snapmirror-throttle"}){
            $self->_click(
                id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                error_checking_needed   => 1,
            ) ;
            $self->_type( $GENERIC_THROTTLE_SELECTOR ,$args{"session-snapmirror-throttle"} );
        }

        # Adding update frequency
        if (defined $args{"update-frequency"}){
            my @u_f = split(":",$args{"update-frequency"});
            $self->_type( $GENERIC_SCHEDULE_UPDATE_FREQUENCY_HOUR_INPUT ,$u_f[0]);
            $self->_click(
                id                      => $GENERIC_SCHEDULE_UPDATE_FREQUENCY_MIN_INPUT,
                error_checking_needed   => 0,
            ) ;
            $selector=$GENERIC_AGGR_SELECTOR;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$u_f[1]$2/;
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;
        }

        $self->_click(
            id                      => $GENERIC_SCHEDULE_CREATE_BUTTON,
            error_checking_needed   => 1,
        ) ;

        if($self->_is_element_present($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE)){
            my $text = $self->_get_text($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE);
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Error while adding schedule : ".$text);
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub transition_schedule_modify

## @name :: transition_schedule_remove
##
## @summary Executes "transition_schedule_remove" command
##
## @description
##  This method is used to execute transition schedule remove command
## @synopsis
##  transition_schedule_modify(%args);
## INPUTS TO METHOD CALL 'transition_schedule_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##@examples
##&nbsp;transition schedule modify -s S21 -n sc1 -d 1-5 -b 12:00 -e 10:00 -t 2 -x 2 :
##&nbsp;
##&nbsp;

sub transition_schedule_remove {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            "schedule-name"                     => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $SCHEDULE_TAB,
            error_checking_needed   => 0,
        ) ;

        my $selector=$GENERIC_SCHEDULE_REMOVER;
        my $ses_name=$args{"schedule-name"};
        $selector =~ s/(.*)XXYYZZ(.*)/$1$ses_name$2/;

        $self->_click(
            id                      => $selector,
            error_checking_needed   => 1,
        ) ;

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub transition_schedule_remove

## @name :: transition_edit_tabs
##
## @summary returns "transition_edit_tabs" for each operation
##
## @description
##  This method is used to edit CBT/CFT transition project config before processing
## @synopsis
##  transition_edit_tabs(%args);
## INPUTS TO METHOD CALL 'transition_edit_tabs' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'goto_tab' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'schedule_add_args' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'svm-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'networking_edit' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'cft_nodemapping_set' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
## goto_tab can have one of the following values which edits the corresponding
## CBT/CFT tab
## Select Data Copy IP,Select SVM, Plan Configuration, Configure Schedule
## Map Volumes, Source Volumes, Network Configuration, Cluster Edit
## CFT SVM Mapping
## parsed output:
## operation =  Select Data Copy IP
## $VAR1 = bless( {
##                  'parsed_output' => {
##                                       'Plan_Configuration' => 'tab_disabled',
##                                       'Select_SVM' => 'tab_disabled',
##                                       'Select_Source_Volumes' => 'tab_enabled',
##                                       'Network_Configuration' => 'tab_disabled',
##                                       'Configure_Schedule' => 'tab_disabled',
##                                       'Select_Data_Copy_IP' => 'tab_enabled',
##                                       'Map_Volumes' => 'tab_disabled'
##                                     },
##&nbsp;

sub transition_edit_tabs {
    my $self = shift;
    print Dumper("In APISET");
    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"              => { type => SCALAR },
            'goto_tab'                  => { type => SCALAR },
            "group-name"                => { type => SCALAR , optional => 1},
            'method-timeout'            => { type => SCALAR , optional => 1},
            "session-type"              => { type => SCALAR , optional => 1},
            "7-mode-host-name"          => { type => SCALAR , optional => 1},
            "data-copy-ipaddress"       => { type => SCALAR , optional => 1},
            "vfiler"                    => { type => SCALAR , optional => 1},
            "multipathing-ipaddress"    => { type => SCALAR , optional => 1},
            "c-mode-host-name"          => { type => SCALAR , optional => 1},
            "svm-name"                  => { type => SCALAR , optional => 1},
            "schedule_add_args"         => { type => ARRAYREF, optional => 1},
            "schedule_edit_args"        => { type => ARRAYREF, optional => 1},
            "seven-mode-only"           => { type => SCALAR , optional => 1},
            "S_mode_credentials"        => { optional => 1  },
            "volumepair_add_args"       => { optional => 1  },
            "cluster_volume_map_args"   => { optional => 1  },
            "mount_policy"              => { optional => 1  },
            "networking_edit"           => { optional => 1  },
            "networking_add"            => { optional => 1  },
            "networking_delete"         => { optional => 1  },
            "add_smode_trans_lif"       => { optional => 1  },
            "remove_schedule"           => { optional => 1  },
            "C_mode_credentials"        => { optional => 1  },
            "primary_info"              => { optional => 1 },
            "secondary_info"            => { optional => 1 },
            "plan_configuration"        => { optional => 1 },
            "S_mode_credentials_HA"     => { optional => 1 },
            "cft_clusterinfo_add"       => { optional => 1 },
            "cft_svm_mapping_add_pri"   => { optional => 1 },
            "cft_svm_mapping_add_sec"   => { optional => 1 },
            "cft_nodemapping_set"       => { optional => 1 },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    $args{"trans_mode"} = $MIGRATION_CBT_TAB;
    my $result_array = $self->get_edit_tab_details(%args);
    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;
} ## end sub transition_edit_tabs

## @name :: transition_state_machine_status
##
## @summary returns "transition_state_machine_status" for each operation
##
## @description
##  This method is used to get status of transition from state icons
## @synopsis
##  transition_state_machine_status(%args);
## INPUTS TO METHOD CALL 'transition_state_machine_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'operation' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## %args Required:
## operation can have precheck,start,incremental, precutover,cutover,apply_svm_config,
## export,cabling,import,commit,rollback_precheck,rollback_verify_7mode as value
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
##@examples
##     NACL::C::Client::Transition->state_machine_status(
##            "method-timeout"        => $METHOD_TIMEOUT,
##            command_interface       => $HostWindows,
##            "project-name"          => $PROJECT_NAME,
##            "operation"             => "rollback_precheck",
##            "apiset_should"         => { interface => $INTERFACE },
##    );
##  'parsed_output' => {
##                                      'status' => 'transition aborted',
##                                      'icon_color' => 'red'
##                     }
##&nbsp;

sub transition_state_machine_status {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"                  => { type => SCALAR },
            'operation'                     => { type => SCALAR },
            "group-name"                    => { type => SCALAR , optional => 1},
            'method-timeout'                => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $args{"group-name"} = $group;
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    $args{"trans_mode"} = $MIGRATION_CBT_TAB;
    my $result_array = $self->get_session_details(%args);
    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;
} ## end sub transition_state_machine_status

## @name :: transition_delete
##
## @summary Executes "transition_delete" command
##
## @description
##  This method is used to execute transition delete command
## @synopsis
##  transition_delete(%args);
## INPUTS TO METHOD CALL 'transition_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##@examples
##&nbsp;transition delete -s S21 :
##&nbsp;
##&nbsp;
sub transition_delete {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        my $project_id = $self->_get_project_id($id);

        $self->_click(
            id                      => $project_id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $DELETE_PROJECT,
            error_checking_needed   => 1,
        ) ;

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub transition_delete

sub transition_apt_add_systems {

    my $self = shift;

    my %args = validate_with(
    params => \@_,
    spec   => {
            "host-name"  => { type => SCALAR },
            "username"   => { type => SCALAR },
            "password"   => { type => SCALAR },
            "type"       => { type => SCALAR },
    },
    allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $self->_click(
    id                      =>  $ASSESSMENT_BUTTON,
    error_checking_needed   =>  1,
    );

    $self->_click_button_having_specific_text_no_error_check($APT_ADD_SYSTEMS);

    $self->_type( $APT_ADD_SYSTEMS_HOSTNAME ,$args{"host-name"});
    $self->_type( $APT_ADD_SYSTEMS_USERNAME ,$args{"username"});
    $self->_type( $APT_ADD_SYSTEMS_PASSWORD ,$args{"password"});

    $self->_click(
    id                      =>  $APT_ADD_SYSTEMS_TYPE,
    error_checking_needed   =>  0,
    );

    my $sleep_count = $GENERIC_WAIT / 1000;
    Tharn::snooze($sleep_count);

    my $selector = $GENERIC_LISTITEM_SELECTOR;
    my $type = $args{"type"};
    $selector =~ s/(.*)XXYYZZ(.*)/$1$type$2/;

    if( $self->_is_element_present($selector) ){
    $self->_click(
        id                      => $selector,
        error_checking_needed   => 0,
    );
        $self->_click(
        id                      => $APT_ADD_SYSTEMS_BUTTON,
        error_checking_needed   => 0,
    );
    }else{
    $self->_take_screenshot();
    $self->_repair_session();
    $self->_throw_exception("Type requested for does not exist : ".$selector);
    }

    $self->_click(
    id                      =>  $MIGRATE_BUTTON,
    error_checking_needed   =>  0,
    );

    $Log->exit() if $may_exit;

    return 1;
} ## end transition_apt_add_systems

sub transition_apt_generate_report {

    my $self = shift;

    my %args = validate_with(
    params => \@_,
    spec   => {
            "host-name"        => { type => SCALAR },
            "targer-cluster"   => { type => SCALAR },
            "file-prefix"      => { type => SCALAR , optional => 1},
            "timeout"          => { type => SCALAR , optional => 1},
    },
    allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $self->_click(
    id                      =>  $ASSESSMENT_BUTTON,
    error_checking_needed   =>  1,
    );

    my $host = $args{"host-name"};
    my $tc = $args{"targer-cluster"};
    my $fp = $args{"file-prefix"};
    my $timeout = $args{'timeout'} || ($GENERIC_WAIT / 1000);

    my $selector = $APT_SELECT_SYSTEM;
    $selector =~ s/(.*)XXYYZZ(.*)/$1$host$2/;

    $self->_click(
    id                      =>  $selector,
    error_checking_needed   =>  0,
    );

    $selector = $GENERIC_LISTITEM_SELECTOR;
    $selector =~ s/(.*)XXYYZZ(.*)/$1$tc$2/;

    $self->_click_button_having_specific_text_no_error_check("Create Transition Assessment Report");

    my $elem = $self->_find_element( $APT_SELECT_TARGET_CLUSTER ) ;
    $elem->click();

    if( $self->_is_element_present($selector) ){
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;
    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Target cluster requested for does "
            ."not exist.. Unable to find the element with ".$selector);
    }

    $self->_type( $APT_FILE_PREFIX_SYSTEM ,$fp);

    $self->_click_button_having_specific_text_no_error_check("Generate Report");

    unless( $self->_wait_for_element_not_present( $APT_WAIT_FOR_REPORT,$timeout ) ){
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception(
            "Report did not get generated within given timeout !"
        );
    }

    $self->_click(
    id                      =>  $MIGRATE_BUTTON,
    error_checking_needed   =>  0,
    );

    $Log->exit() if $may_exit;

    return 1;
} ## end transition_apt_generate_report

## @name ::transition_editproject_plan_config
##
## @summary Edits Plan Configuration page for a project
##
## @description
##    This method is used to edit Plan Configuration page for a project
## @synopsis
##      transition_editproject_plan_config(%args);
## INPUTS TO METHOD CALL 'transition_editproject_plan_config' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'plan_configuration' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##	%plan_configuration = (
##		"ignore-all-nfs-configurations-transition" => 1,
##		"ignore-all-nameservices-configurations-transition" => 1
##    );
##         $Api_Set_Obj2->transition_editproject_plan_config(
##            "session-name"              => "session2",
##            "plan_configuration"        => \%plan_configuration,
##         );
##
##&nbsp;
sub transition_editproject_plan_config{
    my $self = shift;
    my %args = validate_with(
        params  => \@_,
        spec    => {
            "group-name"                => { type => SCALAR , optional => 1},
            "session-name"              => { type => SCALAR , optional => 1},
            "plan_configuration"        => { optional => 1 },
        },
    );

    $Log->enter();
    my $plan_config = (defined $args{"plan_configuration"}) ? $args{"plan_configuration"} : undef;
    my $result_array = undef;
    if (defined($plan_config)){
        $result_array = $self->_edit_configuration(\%args);
    }
    return (defined $result_array) ? $result_array : 1;
} ## end sub transition_editproject_plan_config

## @name ::transition_editproject_volumepair_add
##
## @summary Executes "transition_editproject_volumepair_add" command
##
## @description
##    This method is used to execute transition volumepair add command
## @synopsis
##      transition_editproject_volumepair_add(%args);
## INPUTS TO METHOD CALL 'transition_editproject_volumepair_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'volume-name-c-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'aggregate-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_editproject_volumepair_add(
##            "session-name"              => "session2",
##            "volume-name-7-mode"        => [qw(r1)],
##            "volume-name-c-mode"        => [qw(r1_1)],
##            "aggregate-name"            => [qw(aggr_trans)],
##         );
##
##&nbsp;

sub transition_editproject_volumepair_add {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "volume-name-7-mode"        => { type => ARRAYREF },
            "group-name"                => { type => SCALAR , optional => 1},
            "session-name"              => { type => SCALAR , optional => 1},            
            "volume-name-c-mode"        => { type => ARRAYREF , optional => 1},
            "aggregate-name"            => { type => ARRAYREF , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Storing volume pair add arguments in the object
    foreach (@{$self->{$args{"session-name"}}}){
        undef $_ if (defined $_->{transition_volumepair_add_args});
    }
    @{$self->{$args{"session-name"}}} = grep defined, @{$self->{$args{"session-name"}}};
    push @{$self->{$args{"session-name"}}} , { transition_volumepair_add_args => \%args} ;

    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $self->_project_selector($args{"group-name"});
        my $project = $args{"session-name"};
        $id = "gwt-debug-".$project;
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $EDIT_CONFIGURATION,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $SOURCE_VOLUME_EDIT,
            error_checking_needed   => 0,
        );

        $self->_close_tab(
            string => "Migration",
        );  

        $self->_add_volumepairs_for_session($args{"session-name"},1);

        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 0,
        ) ;

        eval{
            $self->_click(
                id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                error_checking_needed   => 0,
            ) ;
        };

        if ( (defined $args{"aggregate-name"}) || (defined $args{"volume-name-c-mode"}) ){
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            # Aggr selection page
            $self->_select_aggregate($args{"session-name"})
                if defined ($args{"aggregate-name"});

            # Volume mapping
            $self->_do_volume_mapping($args{"session-name"})
                if defined ($args{"volume-name-c-mode"});

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            $self->_close_tab(
                string => "(CBT)",
            );


        $self->_close_tab(
            string => "Migration",
        );   

    }else{
            $self->_throw_exception("Session ".$args{"session-name"}." does not exist "."
                and create arguments not present in the APISet object.\n ".
                "Make sure that you invoke transition_editproject_volumepair_add() on the same APISet".
                " object \n on which you invoked transition_create() or session should exist"
            );
    }
}    

    $Log->exit();

    return 1;
} ## end sub transition_editproject_volumepair_add

## @name :: transition_editproject_volumepair_remove
##
## @summary Executes "transition_editproject_volumepair_remove" command
##
## @description
##  This method is used to execute transition volume remove from session
## @synopsis
##  transition_editproject_volumepair_remove(%args);
## INPUTS TO METHOD CALL 'transition_editproject_volumepair_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_editproject_volumepair_remove(
##            "session-name"              => "session2",
##            "volume-name-7-mode"        => [qw(r1 r2)],
##         );
##
##&nbsp;

sub transition_editproject_volumepair_remove {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "group-name"                => { type => SCALAR , optional => 1},
            "session-name"              => { type => SCALAR , optional => 1},        
            "volume-name-7-mode"        => { type => ARRAYREF },
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->comment("Navigating to Dashboard and selecting a subproject");

    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $self->_project_selector($args{"group-name"});
        my $project = $args{"session-name"};
        $id = "gwt-debug-".$project;
    }

    if($id){
        $self->_click(
            id                      => $id,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $EDIT_CONFIGURATION,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $SOURCE_VOLUME_EDIT,
            error_checking_needed   => 0,
        );
  
        $self->_close_tab(
            string => "Migration",
        );    
        $self->_add_volumepairs_for_session($args{"session-name"},1);

        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 1,
        ) ;

        eval{
            $self->_click(
                id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                error_checking_needed   => 0,
            ) ;
        };
        if ( (defined $args{"aggregate-name"}) || (defined $args{"volume-name-c-mode"}) ){
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            # Aggr selection page
            $self->_select_aggregate($args{"session-name"})
                if defined ($args{"aggregate-name"});

            # Volume mapping
            $self->_do_volume_mapping($args{"session-name"})
                if defined ($args{"volume-name-c-mode"});

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            $self->_close_tab(
                string => "(CBT)",
            );
            
            $self->_close_tab(
                string => "Migration",
            );   
        }
    }else{
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist "."
            and create arguments not present in the APISet object.\n ".
            "Make sure that you invoke transition_editproject_volumepair_add() on the same APISet".
            " object \n on which you invoked transition_create() or session should exist"
        );
    }
    $Log->exit();

    return 1;
} ## end sub transition_editproject_volumepair_remove


## @name ::transition_editproject_data_ip_modify
##
## @summary Add/Modify Data Copy IP
##
## @description
##  This method is used to add or modify the Data Copy IP or Multipath IP addresses
## @synopsis
##  transition_editproject_data_ip_modify(%args);
## INPUTS TO METHOD CALL 'transition_editproject_data_ip_modify' ON APISET OBJECT
## @arg %args Required:
##              "session-name"                      => { type => SCALAR },
##              "group-name"                        => { type => SCALAR },
##              "datacopy-ip-address"               => { type => SCALAR },
##              "multipath-ip-address"
## @examples
##         $Api_Set_Obj2->transition_editproject_data_ip_modify(
##            "datacopy-ip-address"              => [qw(1.12.255.2)],
##            "multipath-ip-address"             => [qw(1.12.255.2)],
##         );
##
##&nbsp;
##&nbsp;

sub transition_editproject_data_ip_modify{
   my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "datacopy-ip-address"               => { type => SCALAR },
            "multipath-ip-address"              => { type => SCALAR },
            "group-name"                        => { type => SCALAR , optional => 1},
            "session-name"                      => { type => SCALAR , optional => 1},            
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $dc_ip = $args{"datacopy-ip-address"};
    my $mul_ip = $args{"multipath-ip-address"};
    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $self->_project_selector($args{"group-name"});        
        my $project = $args{"session-name"};
        $id = "gwt-debug-".$project;
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $EDIT_CONFIGURATION,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $EDIT_DATA_IP,
            error_checking_needed   => 0,
        ) ;

        $self->_type( $DATA_COPY_IP, $dc_ip ) if( $dc_ip );

        $self->_type( $MULTIPATH_IP, $mul_ip ) if( $mul_ip );

        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 1,
        ) ;
        my $popup_title;

        eval{
                $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);                    
                print "POP UP Title : $popup_title"
        };
        if ($popup_title =~ /Error/i) {
            my  $popup_text=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_MESSAGE);
            $self->_create_screenshot("$popup_text");
            $self->_repair_session();
            $self->_throw_exception(
            "Popup Title : $popup_title , Popup error message : $popup_text");
        }elsif( ($popup_title !~ /Operation Status/i ) && ($popup_title =~ /Information/i || $popup_title =~ /Warning/i  )){
            my $popup_txt;
            eval{
                $popup_txt=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_MESSAGE);
            };
        }

            eval{
                $self->_click(
                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                        error_checking_needed   => 0,
                ) ;
            };
    }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Invalid IP provided");
    }

    $self->_close_tab(
        string => "(CBT)",
    );

    $self->_close_tab(
        string => "Migration",
    );
} ##end transition_editproject_data_ip_modify

## @name ::transition_editproject_svm_modify
##
## @summary Modify SVM selection
##
## @description
##  This method is used to Select the target cluster and SVM to which
## the 7-Mode volumes must be transitioned.
## @synopsis
##  transition_editproject_svm_modify(%args);
## INPUTS TO METHOD CALL 'transition_editproject_svm_modify' ON APISET OBJECT
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_editproject_svm_modify(
##            svm-name => $svm_name,
##         );
##
##&nbsp;

sub transition_editproject_svm_modify{
   my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {            
            "svm-name"              => { type => SCALAR },
            "session-name"          => { type => SCALAR , optional => 1},
            "group-name"			=> { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $svm_name = $args{"svm-name"};
    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        #$id = $self->_session_exist($args{"session-name"});
        $self->_project_selector($args{"group-name"});        
        my $project = $args{"session-name"};
        $id = "gwt-debug-".$project;
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $EDIT_CONFIGURATION,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $SVM_EDIT,
            error_checking_needed   => 0,
        ) ;

    my $vserver = $svm_name;
    my $selector = $GENERIC_VSERVER_SELECTOR;
    $selector =~ s/(.*)XXYYZZ(.*)/$1$vserver$2/;

    if( $self->_is_element_present($selector) ) {
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 1,
        ) if(! $self->_is_checked($selector) );
    }else{
        $self->_create_screenshot("Vserver_add_error");
        $self->_repair_session();
        $self->_throw_exception("Probably the vserver requested for does not exist"
            .".. Unable to find the element with ".$selector);
    }
        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 1,
        ) ;
        $self->_close_tab(
            string => "(CBT)",
        );
        $self->_close_tab(
            string => "Migration",
        );
    }
} ##end transition_editproject_svm_modify

# @name :: transition_editproject_schedule_modify
##
## @summary Executes "transition_editproject_schedule_modify" command
##
## @description
##  This method is used to execute transition schedule modify command
## @synopsis
##  transition_editproject_schedule_modify(%args);
## INPUTS TO METHOD CALL 'transition_editproject_schedule_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'days-range' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'start-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'duration' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'update-frequency' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
##&nbsp;

sub transition_editproject_schedule_modify {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "schedule-name"                     => { type => SCALAR },
            "group-name"                        => { type => SCALAR , optional => 1},
            "session-name"                      => { type => SCALAR , optional => 1},
            "days-range"                        => { type => SCALAR , optional => 1},
            "start-time"                        => { type => SCALAR , optional => 1},
            "duration"                          => { type => SCALAR , optional => 1},
            "update-frequency"                  => { type => SCALAR , optional => 1},
            "available-transfers-percentage"    => { type => SCALAR , optional => 1},
            "max-cap-concurrent-transfers"      => { type => SCALAR , optional => 1},
            "session-snapmirror-throttle"       => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    my $dashboard_enabled = undef;

    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        #$id = $self->_session_exist($args{"session-name"});
        $self->_project_selector($args{"group-name"});
        my $project = $args{"session-name"};
        $id = "gwt-debug-".$project;
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $EDIT_CONFIGURATION,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $SCHEDULE_EDIT,
            error_checking_needed   => 0,
        );
        #my $selector=$GENERIC_SCHEDULE_MODIFIER;
        my $ses_name=$args{"schedule-name"};
        my $selector = $GENERIC_SCHEDULE_NAME_EDIT;
        $selector =~ s/(.*)XX(.*)/$1$ses_name$2/;

        if( $self->_is_element_present($selector) ) {
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 1,
        ) ;
        }else{
            $self->_click(
                id                      => $CREATE_SCHEDULE,
                error_checking_needed   => 0,
            ) ;
            $self->_type( $GENERIC_SCHEDULE_NAME ,$args{"schedule-name"});
        }

        $self->_add_schedule_days($args{"days-range"});

        # Adding start time
        my @s_t = split(":",$args{"start-time"});
        $selector=$GENERIC_AGGR_SELECTOR;
        $self->_click(
            id                      => $GENERIC_SCHEDULE_START_TIME_HOUR_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[0]$2/; 
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;
        $self->_click(
            id                      => $GENERIC_SCHEDULE_START_TIME_MIN_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector=$GENERIC_AGGR_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[1]$2/;
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;

        # Adding duration
        my @duration = split(":",$args{"duration"});
        $self->_type( $GENERIC_SCHEDULE_DURATION_HOUR_INPUT ,$duration[0] );
        $self->_click(
            id                      => $GENERIC_SCHEDULE_DURATION_MIN_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector=$GENERIC_AGGR_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$duration[1]$2/;
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;

        # Adding update frequency
        if (defined $args{"update-frequency"}){
            my @u_f = split(":",$args{"update-frequency"});
            $selector = "//div/label[contains(text(),'Continuous')]/../input";
            if( $self->_is_element_present($selector) ) {
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 1,
                ) if( $self->_is_checked($selector) );
            }
            $self->_type( $GENERIC_SCHEDULE_UPDATE_FREQUENCY_HOUR_INPUT ,$u_f[0]);
            $self->_click(
                id                      => $GENERIC_SCHEDULE_UPDATE_FREQUENCY_MIN_INPUT,
                error_checking_needed   => 0,
            ) ;
            $selector=$GENERIC_AGGR_SELECTOR;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$u_f[1]$2/;
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;
        }

        if ( defined $args{"max-cap-concurrent-transfers"} ) {
            # Adding stream count
            $self->_click(
                id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) );
            $self->_type( $GENERIC_STREAM_SELECTOR ,$args{"max-cap-concurrent-transfers"} );
        }

        if ( defined $args{"available-transfers-percentage"} ) {
            # Adding available-transfers-percentage
            $self->_click(
                id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) );
            $self->_type(
                $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,$args{"available-transfers-percentage"} );
        }
        # Adding throttle
        if (defined $args{"session-snapmirror-throttle"}){

            $self->_click(
                id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                error_checking_needed   => 1,
            ) ;
            $self->_type( $GENERIC_THROTTLE_SELECTOR ,$args{"session-snapmirror-throttle"} );
        }

        $self->_click(
            id                      => $GENERIC_SCHEDULE_CREATE_BUTTON,
            error_checking_needed   => 1,
        ) ;

        if($self->_is_element_present($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE)){
            my $text = $self->_get_text($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE);
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Error while adding schedule : ".$text);
        }

            $self->_close_tab(
                string => "(CBT)",
            );

        $self->_close_tab(
            string => "Migration",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit();

    return 1;
} ## end sub transition_editproject_schedule_modify

## @name :: transition_editproject_schedule_remove
##
## @summary Executes "transition_editproject_schedule_remove" command
##
## @description
##  This method is used to execute transition schedule remove command
## @synopsis
##  transition_schedule_modify(%args);
## INPUTS TO METHOD CALL 'transition_editproject_schedule_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##&nbsp;

sub transition_editproject_schedule_remove {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "schedule-name"                     => { type => SCALAR },
            "group-name"                        => { type => SCALAR , optional => 1},
            "session-name"                      => { type => SCALAR , optional => 1},        
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );
    
    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        #$id = $self->_session_exist($args{"session-name"});
        $self->_project_selector($args{"group-name"});        
        my $project = $args{"session-name"};
        $id = "gwt-debug-".$project;
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $EDIT_CONFIGURATION,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $SCHEDULE_EDIT,
            error_checking_needed   => 0,
        );
        #my $selector=$GENERIC_SCHEDULE_MODIFIER;
        my $ses_name=$args{"schedule-name"};
        my $selector = $SCHEDULE_REMOVE;
        $selector =~ s/(.*)XX(.*)/$1$ses_name$2/;

        $self->_click(
            id                      => $selector,
            error_checking_needed   => 1,
        ) ;

        $self->_close_tab(
            string => "(CBT)",
        );

        $self->_close_tab(
            string => "Migration",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit();

    return 1;
} ## end sub transition_editproject_schedule_remove

## @name :: transition_cft_create
##
## @summary Executes "transition_cft_create" command
##
## @description
##  This method is used to execute transition cft create command
## @synopsis
##  transition_cutover(%args);
## INPUTS TO METHOD CALL 'transition_cutover' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> '7-mode1-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> '7-mode2-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##      my %S_mode_credentials = (
##                  'host-name'        => $Host7Mode[0]->{MGMT_IP},
##                  username           => "root",
##                  'password'         => $Host7Mode[0]->{default_password}
##              );
##          my %S_mode_credentials_HA = (
##                  'host-name'        => $Host7Mode[1]->{MGMT_IP},
##                  username           => "root",
##                  'password'         => $Host7Mode[1]->{default_password}
##              );
##          my %C_mode_credentials = (
##                  "host-name"        => $HostCMode[0]->{CLUSTER_MGMT_IP},
##                  username           => "admin",
##                  'password'         => $HostCMode[0]->{default_password}
##              );
##          my %cft_clusterinfo_add=(
##                  "c-mode-host-name" =>$HostCMode[0]->{CLUSTER_MGMT_IP},
##              );
##          my %cft_svmmapping_add_pri = (
##                  "7-mode-ip-or-fqdn"         => $Host7Mode[0]->{MGMT_IP},
##                  "vfiler-name"               => $VFILER_PRI,
##                  "svm-name"                  => $VSERVER_PRI,
##              );
##          my %cft_svmmapping_add_sec = (
##                  "7-mode-ip-or-fqdn"         => $Host7Mode[1]->{MGMT_IP},
##                  "vfiler-name"               => $VFILER_SEC,
##                  "svm-name"                  => $VSERVER_SEC,
##              );
##          my %cft_nodemapping_set = (
##                  "cluster-node1-name"        => $NodeCMode[0]->name(),
##                  "cluster-node2-name"        => $NodeCMode[1]->name(),
##              );
##
##          $Api_Set_Obj->transition_cft_create(
##              command_interface       => $HostWindows,
##              "project-name"          => $PROJECT_NAME,
##              "7-mode1-host-name"     => $Host7Mode[0]->{MGMT_IP},
##              "7-mode2-host-name"     => $Host7Mode[1]->{MGMT_IP},
##              "group-name"            => $GROUP_NAME,
##              S_mode_credentials      => \%S_mode_credentials,
##              C_mode_credentials      => \%C_mode_credentials,
##              S_mode_credentials_HA   => \%S_mode_credentials_HA,
##              cft_clusterinfo_add     => \%cft_clusterinfo_add,
##              cft_svm_mapping_add_pri => \%cft_svmmapping_add_pri,
##              cft_svm_mapping_add_sec => \%cft_svmmapping_add_sec,
##              cft_nodemapping_set     => \%cft_nodemapping_set,
##          );
##
##&nbsp;transition cft revert precheck -p Test -t new :
##&nbsp;

sub transition_cft_create {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"            => { type => SCALAR , optional => 1},
            "group-name"              => { type => SCALAR , optional => 1},
            "7-mode1-host-name"       => { type => SCALAR , optional => 1},
            "7-mode2-host-name"       => { type => SCALAR , optional => 1},
            "S_mode_credentials"      => { optional => 1  },
            "C_mode_credentials"      => { optional => 1  },
            "S_mode_credentials_HA"   => { optional => 1  },
            "cft_clusterinfo_add"     => { optional => 1  },
            "cft_svm_mapping_add_pri" => { optional => 1  },
            "cft_svm_mapping_add_sec" => { optional => 1  },
            "cft_nodemapping_set"     => { optional => 1  },
            "plan_configuration"      => { optional => 1  },
        },
        allow_extra => 1,
    );

    $args{copy_tab} = $MIGRATION_CFT_TAB; #selects CBT/CFT tab under the given group-name
    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );
    $self->_cft_create_helper(\%args);

    $Log->exit() if $may_exit;

    return 1;
}

## @name :: transition_cft_clusterinfo_add
##
## @summary Executes "transition_cft_clusterinfo_add" command
##
## @description
##  This method is used to execute transition cft clusterinfo add command
## @synopsis
##  transition_cft_clusterinfo_add(%args);
## INPUTS TO METHOD CALL 'transition_cft_clusterinfo_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'c-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_clusterinfo_add(
##            "project-name"      => "Test",
##            "group-name"        => "new",
##            "c-mode-host-name"  => $HostCMode[0]->{CLUSTER_MGMT_IP},
##         );
##
##&nbsp;transition cft clusterinfo add -p Test -h 10.72.213.45 :
##&nbsp;

sub transition_cft_clusterinfo_add {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"      => { type => SCALAR , optional => 1},
            "c-mode-host-name"  => { type => SCALAR , optional => 0},
        },
        allow_extra => 1,
    );
    $Log->enter() if $may_enter;

        push @{$self->{$args{"project-name"}}} , { cft_clusterinfo_add_args => \%args} ;
        use Data::Dumper;
        $Log->comment( sub { "Input contains after addition " . Dumper( \$self ) } );
        my $ip = $self->_get_value_for_key_x_key_y_in_session_z(
            "cft_clusterinfo_add_args","c-mode-host-name",$args{"project-name"});
        my $host_cmode = $self->_get_host_from_ip($ip,"c");
        $self->_put_value_for_key_x_key_y_in_session_z(
        "transition_cft_create_args",$ip,$args{"project-name"},$host_cmode);

    $self->_click(
            id                      => $MIGRATION_DASHBOARD_LINK,
            error_checking_needed   => 0,
        ) ;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    my $var=1;
    while (1) {
        my $group_path = "//div/div[$var]/div/span/b[contains(text(),'$group')]";
        if ($self->_is_element_present($group_path)) {
            $self->_click(
                id                      => $group_path,
                error_checking_needed   => 0,
            ) ;
            last;
        }
        $var++;
    }
    my $locator = "gwt-debug-".$args{"project-name"};
    if ($self->_is_element_present($locator)) {
        my $cft_edit_config = $PROJECT_EDIT_CONFIG;
        $cft_edit_config =~ s/(.*)XXYYZZ(.*)/$1$args{"project-name"}$2/;

        $self->_click(
            id                      => $cft_edit_config,
            error_checking_needed   => 0,
        ) ;
        $self->_click(
            id                      => $EDIT_SELECT_TARGET_CLUSTER,
            error_checking_needed   => 0,
        ) ;
        my $cft_cluster = $CFT_CLUSTER;
        $cft_cluster =~ s/(.*)XXYYZZ(.*)/$1$host_cmode$2/;
        if($self->is_element_present($cft_cluster)){
            $self->_click(
                id                      => $cft_cluster,
                error_checking_needed   => 0,
            ) ;
        }
        my $var = 0;
        while (1) {
            my $c_ha_selector =$CLUSTER_HA_SELECTOR;
            $c_ha_selector =~ s/(.*)XXYYZZ(.*)/$1$var$2/;
            if ($self->_is_element_present($c_ha_selector)) {
                $self->_click(
                    id                      => $c_ha_selector,
                    error_checking_needed   => 0,
                )if(! $self->_is_checked($c_ha_selector) );
                last;
            }
            $var++;
        }
        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 0,
        ) ;
        $self->_close_tab(
            project => $args{"project-name"},
        );
        $self->_close_tab(
            string => "Migration",
        );
    }else{
        $self->_close_tab(
            string => "Migration",
        );
        $self->_throw_exception("Project with name ".$args{"project-name"}." does not exists");
    }
    $Log->exit() if $may_exit;

    return 1;
}

## @name :: transition_cft_editconfig
##
## @summary Executes "transition_cft_editconfig" command
##
## @description
##  This method is used to edit CFT plan configuration page from Dashboard edit configuration
## @synopsis
##  transition_cft_editconfig(%args);
##
## @examples
##         $Api_Set_Obj->transition_cft_editconfig(
##            "project-name"      => "Test",
##            "group-name"        => "new",
##            "plan_configuration"  => {vserver1=>{'ignore-all'=>1},vserver2=>{'ignore-all-cifs-transition'=>1}},
##         );
##
##&nbsp;



sub transition_cft_editconfig {
my $self = shift;
my %args = validate_with(
        params => \@_,
        spec    => {
            "project-name" => { type => SCALAR },
            "group-name"  => { type =>SCALAR,optional=>1 } ,
            "plan_configuration" => { optional => 1 },
        },
    );
    $self->_click(
            id                      => $MIGRATION_DASHBOARD_LINK,
            error_checking_needed   => 0,
        ) ;
        my $group = undef;
        $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
        my $var=1;
        while (1) {
        my $group_path = "//div/div[$var]/div/span/b[contains(text(),'$group')]";
        if ($self->_is_element_present($group_path)) {
            $self->_click(
                id                      => $group_path,
                error_checking_needed   => 0,
            ) ;
            last;
        }
        $var++;
        }
        my $project = $args{"project-name"};
        my $locator = "gwt-debug-".$project;
        if ($self->_is_element_present($locator)) {
            $self->_click(
                id                      => $locator,
                error_checking_needed   => 0,
            ) ;
        }
        $self->_click(
            id                      => $PROJECT_CFT_EDIT,
            error_checking_needed   => 0,
        ) ;
        $self->_click(
            id                      => $CFT_EDIT_PLAN_CONFIG,
            error_checking_needed   => 0,
        ) ;
        my $plan_config = $args{"plan_configuration"};
        foreach my $Vserver (keys($plan_config)){
            my $var = 0;
            print "Vserver : $Vserver \n";
            # SVM selection:Loop till max vservers
            for ($var=0;$var<128;$var++) {
                        my $selector = $SVM_PLAN_CONFIG;
                        $selector =~ s/XX/$var/;
                        $selector =~ s/YY/$Vserver/;
                        print "Selector : $selector";

                        if ($self->_is_element_present($selector)) {
                            $self->_click(
                                id                      => $selector,
                                error_checking_needed   => 0,
                                ) ;
                            last;
                        }
                    }
                    $self->_plan_configuration($plan_config->{$Vserver});
                }
        $self->_close_tab(
                string => "\(CFT\)",

        );

        $self->_close_tab(
            string => "Migration"
        );
}

## @name :: transition_cft_nodemapping_set
##
## @summary Executes "transition_cft_nodemapping_set" command
##
## @description
##  This method is used to execute transition cft nodemapping set command
## @synopsis
##  transition_cft_nodemapping_set(%args);
## INPUTS TO METHOD CALL 'transition_cft_nodemapping_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'cluster-node1-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'cluster-node2-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_nodemapping_set(
##            "project-name"       => $PROJECT_NAME,
##            "group-name"         => $GROUP_NAME,
##            "cluster-node1-name" => $NodeCMode[0]->name(),
##            "cluster-node2-name" => $NodeCMode[1]->name(),
##         );
##
##&nbsp;transition cft nodemapping set -p Test -u 10.71.213.101 -v 10.71.213.102 :
##&nbsp;

sub transition_cft_nodemapping_set{

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "cluster-node1-name" => { type => SCALAR , optional => 1},
            "cluster-node2-name" => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter() if $may_enter;

    my $pri_clus_node = $PRIMARY_CLUS_NODE;
    my $pri_cmode = $self->_get_value_for_key_x_key_y_in_session_z(
    "cft_nodemapping_set_args","cluster-node1-name",$args{"project-name"});
    $pri_clus_node =~ s/(.*)XXYYZZ(.*)/$1$pri_cmode$2/;
    my $sec_clus_node = $SECONDARY_CLUS_NODE;
    my $sec_cmode = $self->_get_value_for_key_x_key_y_in_session_z(
        "cft_nodemapping_set_args","cluster-node2-name",$args{"project-name"});
    $sec_clus_node =~ s/(.*)XXYYZZ(.*)/$1$sec_cmode$2/;

    $self->_click(
            id                      => $MIGRATION_DASHBOARD_LINK,
            error_checking_needed   => 0,
        ) ;

    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    my $var=1;
    while (1) {
        my $group_path = "//div/div[$var]/div/span/b[contains(text(),'$group')]";
        if ($self->_is_element_present($group_path)) {
            $self->_click(
                id                      => $group_path,
                error_checking_needed   => 0,
            ) ;
            last;
        }
        $var++;
    }
    my $locator = "gwt-debug-".$args{"project-name"};
    if ($self->_is_element_present($locator)) {
        push @{$self->{$args{"project-name"}}} , { cft_nodemapping_set_args => \%args} ;
        my $cft_edit_config = $PROJECT_EDIT_CONFIG;
        $cft_edit_config =~ s/(.*)XXYYZZ(.*)/$1$args{"project-name"}$2/;
        $self->_click(
            id                      => $cft_edit_config,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $EDIT_SELECT_TARGET_CLUSTER,
            error_checking_needed   => 0,
        ) ;

        my $pri_clus_node = $PRIMARY_CLUS_NODE;
        my $pri_cmode = $self->_get_value_for_key_x_key_y_in_session_z(
            "cft_nodemapping_set_args","cluster-node1-name",$args{"project-name"});
        $pri_clus_node =~ s/(.*)XXYYZZ(.*)/$1$pri_cmode$2/;
        my $sec_clus_node = $SECONDARY_CLUS_NODE;
        my $sec_cmode = $self->_get_value_for_key_x_key_y_in_session_z(
            "cft_nodemapping_set_args","cluster-node2-name",$args{"project-name"});
        $sec_clus_node =~ s/(.*)XXYYZZ(.*)/$1$sec_cmode$2/;
        if (($self->_is_element_present($pri_clus_node)) && ($self->_is_element_present($sec_clus_node))) {
            print"the node mapping is already done\n";
        }else{
            $self->_click(
                id                      => $SWAP_BUTTON,
                error_checking_needed   => 0,
            ) ;
        }
        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 0,
        ) ;
        $self->_close_tab(
            project => $args{"project-name"},
        );
        $self->_close_tab(
            string => "Migration",
        );
    }else{
        $self->_close_tab(
            string => "Migration",
        );
        $self->_throw_exception("Project with name ".$args{"project-name"}." does not exists");
    }

    $Log->exit() if $may_exit;

    return 1;
}

## @name :: transition_cft_svmmapping_add
##
## @summary Executes "transition_cft_svmmapping_add" command
##
## @description
##  This method is used to execute transition cft svmmapping add command
## @synopsis
##  transition_cft_svmmapping_add(%args);
## INPUTS TO METHOD CALL 'transition_cft_svmmapping_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'vfiler-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> '7-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'svm-name' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_svmmapping_add(
##            "7-mode-ip-or-fqdn"         => $Host7Mode[0]->{MGMT_IP},
##            "vfiler-name"               => $VFILER_PRI,
##            "svm-name"                  => $VSERVER_PRI,
##         );
##
##&nbsp;transition cft svmmapping add -p Test -n 10.71.213.101 -f vfiler0 -v vs1 :
##&nbsp;

sub transition_cft_svmmapping_add {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "7-mode-host-name"   => { type => SCALAR , optional => 1},
            "vfiler-name"        => { type => SCALAR , optional => 1},
            "svm-name"           => { type => SCALAR , optional => 1},
            "svm-type"           => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter() if $may_enter;

    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    $self->_click(
            id                      => $MIGRATION_DASHBOARD_LINK,
            error_checking_needed   => 0,
        ) ;

    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    my $var=1;
    while (1) {
        my $group_path = "//div/div[$var]/div/span/b[contains(text(),'$group')]";
        if ($self->_is_element_present($group_path)) {
            $self->_click(
                id                      => $group_path,
                error_checking_needed   => 0,
            ) ;
            last;
        }
        $var++;
    }
    my $locator = "gwt-debug-".$args{"project-name"};
    if ($self->_is_element_present($locator)) {
        push @{$self->{$args{"project-name"}}} , { cft_svmmapping_set_args => \%args} ;
        $Log->comment( sub { "Input Hash contains  after adding" . Dumper( \$self ) } );
        print "The Project exists\n";
        my $cft_edit_config = $PROJECT_EDIT_CONFIG;
        $cft_edit_config =~ s/(.*)XXYYZZ(.*)/$1$args{"project-name"}$2/;
        $self->_click(
            id                      => $cft_edit_config,
            error_checking_needed   => 0,
        ) ;
    
        $self->_click(
            id                      => $EDIT_SVM_MAPPING,
            error_checking_needed   => 0,
        ) ;

        $self->_select_vsm($args{"7-mode-host-name"},$args{"vfiler-name"},$args{"svm-name"});
        $self->_close_tab(
            string => "\(CFT\)",
        );
        $self->_close_tab(
            string => "Migration",
        );

    }else{
        $self->_close_tab(
            string => "Migration",
        );
        $self->_throw_exception("Project with name ".$args{"project-name"}." does not exists");
    }
    $Log->exit() if $may_exit;
    return 1;
}

## @name :: transition_cft_delete
##
## @summary Executes "transition_cft_delete" command
##
## @description
##  This method is used to execute transition cft delete command
## @synopsis
##  transition_cft_delete(%args);
## INPUTS TO METHOD CALL 'transition_cft_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_delete(
##            "project-name"        => $project_name,
##            "group-name"          => $group_name,
##         );
##
##&nbsp;transition cft delete -p Test  :
##&nbsp;

sub transition_cft_delete {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter() if $may_enter;

    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    $self->_click(
            id                      => $MIGRATION_DASHBOARD_LINK,
            error_checking_needed   => 0,
        ) ;

    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    my $var=1;
    while (1) {
        my $group_path = "//div/div[$var]/div/span/b[contains(text(),'$group')]";
        if ($self->_is_element_present($group_path)) {
            $self->_click(
                id                      => $group_path,
                error_checking_needed   => 0,
            ) ;
            last;
        }
        $var++;
    }
    my $locator = "gwt-debug-".$args{"project-name"};
    if ($self->_is_element_present($locator)) {
        push @{$self->{$args{"project-name"}}} , { cft_svmmapping_set_args => \%args} ;
        $Log->comment( sub { "Input Hash contains  after adding" . Dumper( \$self ) } );
        print "The Project exists\n";
        my $cft_edit_config = $CFT_MODIFY;
        $cft_edit_config =~ s/(.*)XXYYZZ(.*)/$1$args{"project-name"}$2/;
        $self->_click(
            id                      => $cft_edit_config,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $PROJECT_DELETE,
            error_checking_needed   => 1,
        ) ;

        $self->_create_screenshot($args{"project-name"}."_deleted");
        $self->_close_tab(
            string => "Migration",
        );

    }else{
        $self->_close_tab(
            string => "Migration",
        );
        $self->_throw_exception("Project with name ".$args{"project-name"}." does not exists");
    }

    $Log->exit() if $may_exit;

    return 1;

}

## @name :: transition_cft_precheck
##
## @summary Executes "transition_cft_precheck" command
##
## @description
##  This method is used to execute transition cft create command
## @synopsis
##  transition_cft_precheck(%args);
## INPUTS TO METHOD CALL 'transition_cft_precheck' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_precheck(
##            "project-name"      => "Test",
##            "parsed_output"     => 1,
##         );
##
## By default parsed_output will be "1",it will process all the precheck message and send it as
## parsed output. If in case parsedoutput is not need, it can be skipped by specifying
## parsed_output as "0"(Here the command does not return any parsed output)
##
##&nbsp;transition cft precheck -p Test :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [
##&nbsp;                          {
##&nbsp;                         'corrective_action' => 'To continue with the transition, create these SVMs on the target cluster. For more information, refer to the 7-Mode Transition Tool Copy-Free Transiustered Data ONTAP Administration Guide.',
##&nbsp;                         'type' => 'Error',
##&nbsp;                         'code' => '52250',
##&nbsp;                         'message' => 'The following target SVMs that are added to the project do not exist on the cluster.'
##&nbsp;                       },
##&nbsp;                       ]
##&nbsp;           'Warning' => [],
##&nbsp;           'Informational' => [
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_precheck {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "CFT_Precheck";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }

    my $state = undef;
    if ($parsed_output != 1) {
        $state = $PRECHECK_SUCCESS;
    }
    $args{"trans_mode"} = $MIGRATION_CFT_TAB;
    my $result_array = $self->_perform_operation($args{"project-name"},
        $CFT_PRECHECK_BUTTON,$group,$operation,$parsed_output,$run_background,
        $state,\%args
    );

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;

}

sub transition_cft_check_readiness {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "CFT_Check_Readiness";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    
    my $state = undef;
    if ($parsed_output != 1) {
        $state = $PRECHECK_SUCCESS;
    }
    $args{"trans_mode"} = $MIGRATION_CFT_TAB;
    my $result_array = $self->_perform_operation($args{"project-name"},
        $CFT_CHECK_READINESS,$group,$operation,$parsed_output,$run_background,
        $state,\%args
    );

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;

}

## @name :: transition_cft_apply_svm_config
##
## @summary Executes "transition_cft_apply_svm_config" command
##
## @description
##  This method is used to execute transition cft create command
## @synopsis
##  transition_cft_precheck(%args);
## INPUTS TO METHOD CALL 'transition_cft_apply_svm_config' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_apply_svm_config(
##            "project-name"      => "Test",
##            "group-name"        => "new",
##         );
##
## By default parsed_output will be "1",it will process all the precheck message and send it as
## parsed output. If in case parsedoutput is not need, it can be skipped by specifying
## parsed_output as "0"(Here the command does not return any parsed output)
##
##&nbsp;transition cft apply-svm-config -p Test:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [
##&nbsp;                          {
##&nbsp;                         'corrective_action' => 'To continue with the transition, create these SVMs on the target cluster. For more information, refer to the 7-Mode Transition Tool Copy-Free Transiustered Data ONTAP Administration Guide.',
##&nbsp;                         'type' => 'Error',
##&nbsp;                         'code' => '52250',
##&nbsp;                         'message' => 'The following target SVMs that are added to the project do not exist on the cluster.'
##&nbsp;                       },
##&nbsp;                       ]
##&nbsp;           'Warning' => [],
##&nbsp;           'Informational' => [
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_apply_svm_config {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "Apply_svm_config";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    
    my $state = undef;
    if ($parsed_output != 1) {
        $state = $APPLY_SVM_SUCCESS;
    }
    $args{"trans_mode"} = $MIGRATION_CFT_TAB;
    my $result_array = $self->_perform_operation($args{"project-name"},
        $CFT_APPLY_CONFIG,$group,$operation,$parsed_output,$run_background,$state,
        \%args
    );

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;

}

## @name :: transition_cft_start
##
## @summary Executes "transition_cft_start" command
##
## @description
##  This method is used to execute transition cft start command
## @synopsis
##  transition_cft_start(%args);
## INPUTS TO METHOD CALL 'transition_cft_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_start(
##            "project-name"      => "Test",
##            "parsed_output"     => 1,
##         );
##
## By default parsed_output will be "1",it will process all the precheck message and send it as
## parsed output. If in case parsed output is not need, it can be skipped by specifying
## parsed_output as "0"(Here the command does not return any parsed output)
##
##&nbsp;transition cft start -p Test:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [
##&nbsp;                          {
##&nbsp;                         'corrective_action' => 'To continue with the transition, create these SVMs on the target cluster. For more information, refer to the 7-Mode Transition Tool Copy-Free Transiustered Data ONTAP Administration Guide.',
##&nbsp;                         'type' => 'Error',
##&nbsp;                         'code' => '52250',
##&nbsp;                         'message' => 'The following target SVMs that are added to the project do not exist on the cluster.'
##&nbsp;                       },
##&nbsp;                       ]
##&nbsp;           'Warning' => [],
##&nbsp;           'Informational' => [
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_start {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "Export";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    
    my $state = undef;
    if ($parsed_output != 1) {
        $state = $EXPORT_SUCCESS;
    }
    $args{"trans_mode"} = $MIGRATION_CFT_TAB;    
    my $result_array = $self->_perform_operation($args{"project-name"},$CFT_EXPORT,
        $group,$operation,$parsed_output,$run_background,$state,\%args
    );

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;
}

## @name :: transition_cft_verify_cabling
##
## @summary Executes "transition_cft_verify_cabling" command
##
## @description
##  This method is used to execute transition cft verify cabling command
## @synopsis
##  transition_cft_start(%args);
## INPUTS TO METHOD CALL 'transition_cft_verify_cabling' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_verify_cabling(
##            "project-name"      => "Test",
##            "parsed_output"     => 1,
##         );
##
## By default parsed_output will be "1",it will process all the precheck message and send it as
## parsed output. If in case parsed output is not need, it can be skipped by specifying
## parsed_output as "0"(Here the command does not return any parsed output)
##
##&nbsp;transition cft verify-cabling -p Test:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [
##&nbsp;                          {
##&nbsp;                         'corrective_action' => 'To continue with the transition, create these SVMs on the target cluster. For more information, refer to the 7-Mode Transition Tool Copy-Free Transiustered Data ONTAP Administration Guide.',
##&nbsp;                         'type' => 'Error',
##&nbsp;                         'code' => '52250',
##&nbsp;                         'message' => 'The following target SVMs that are added to the project do not exist on the cluster.'
##&nbsp;                       },
##&nbsp;                       ]
##&nbsp;           'Warning' => [],
##&nbsp;           'Informational' => [
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_verify_cabling {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "Verify_Cabling";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    
    my $state = undef;
    if ($parsed_output != 1) {
        $state = $VERIFY_CABLE_SUCCESS;
    }
    $args{"trans_mode"} = $MIGRATION_CFT_TAB;
    my $result_array = $self->_perform_operation($args{"project-name"},
        $CFT_VERIFY_CABLING,$group,$operation,$parsed_output,$run_background,
        $state,\%args
    );

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;

}

## @name :: transition_cft_import
##
## @summary Executes "transition_cft_import" command
##
## @description
##  This method is used to execute transition cft import command
## @synopsis
##  transition_cft_import(%args);
## INPUTS TO METHOD CALL 'transition_cft_import' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_import(
##            "project-name"      => "Test",
##            "parsed_output"     => 1,
##         );
##
## By default parsed_output will be "1",it will process all the precheck message and send it as
## parsed output. If in case parsed output is not need, it can be skipped by specifying
## parsed_output as "0"(Here the command does not return any parsed output)
##
##&nbsp;transition cft import -p Test :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [
##&nbsp;                          {
##&nbsp;                         'corrective_action' => 'To continue with the transition, create these SVMs on the target cluster. For more information, refer to the 7-Mode Transition Tool Copy-Free Transiustered Data ONTAP Administration Guide.',
##&nbsp;                         'type' => 'Error',
##&nbsp;                         'code' => '52250',
##&nbsp;                         'message' => 'The following target SVMs that are added to the project do not exist on the cluster.'
##&nbsp;                       },
##&nbsp;                       ]
##&nbsp;           'Warning' => [],
##&nbsp;           'Informational' => [
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_import {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "Import";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    
    my $state = undef;
    if ($parsed_output != 1) {
        $state = $IMPORT_SUCCESS;
    }
    $args{"trans_mode"} = $MIGRATION_CFT_TAB;
    my $result_array = $self->_perform_operation($args{"project-name"},$CFT_IMPORT,
        $group,$operation,$parsed_output,$run_background,$state,\%args
    );

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;
}

## @name :: transition_cft_commit
##
## @summary Executes "transition_cft_commit" command
##
## @description
##  This method is used to execute transition cft commit command
## @synopsis
##  transition_cft_commit(%args);
## INPUTS TO METHOD CALL 'transition_cft_commit' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_commit(
##            "project-name"      => "Test",
##            "parsed_output"     => 1,
##         );
##
## By default parsed_output will be "1",it will process all the precheck message and send it as
## parsed output. If in case parsed output is not need, it can be skipped by specifying
## parsed_output as "0"(Here the command does not return any parsed output)
##
##&nbsp;transition cft commit -p Test:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [
##&nbsp;                          {
##&nbsp;                         'corrective_action' => 'To continue with the transition, create these SVMs on the target cluster. For more information, refer to the 7-Mode Transition Tool Copy-Free Transiustered Data ONTAP Administration Guide.',
##&nbsp;                         'type' => 'Error',
##&nbsp;                         'code' => '52250',
##&nbsp;                         'message' => 'The following target SVMs that are added to the project do not exist on the cluster.'
##&nbsp;                       },
##&nbsp;                       ]
##&nbsp;           'Warning' => [],
##&nbsp;           'Informational' => [
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_commit {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "Commit";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    
    my $state = undef;
    if ($parsed_output != 1) {
        $state = $COMMIT_SUCCESS;
    }
    $args{"trans_mode"} = $MIGRATION_CFT_TAB;
    my $result_array = $self->_perform_operation($args{"project-name"},$CFT_COMMIT,
        $group,$operation,$parsed_output,$run_background,$state,\%args
    );

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;

}

## @name :: transition_cft_revert_precheck
##
## @summary Executes "transition_cft_revert_precheck" command
##
## @description
##  This method is used to execute transition cft create command
## @synopsis
##  transition_cft_commit(%args);
## INPUTS TO METHOD CALL 'transition_cft_revert_precheck' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_revert_precheck(
##            "project-name"      => "Test",
##            "parsed_output"     => 1,
##         );
##
## By default parsed_output will be "1",it will process all the precheck message and send it as
## parsed output. If in case parsed output is not need, it can be skipped by specifying
## parsed_output as "0"(Here the command does not return any parsed output)
##
##&nbsp;transition cft revert precheck -p Test :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [
##&nbsp;                          {
##&nbsp;                         'corrective_action' => 'To continue with the transition, create these SVMs on the target cluster. For more information, refer to the 7-Mode Transition Tool Copy-Free Transiustered Data ONTAP Administration Guide.',
##&nbsp;                         'type' => 'Error',
##&nbsp;                         'code' => '52250',
##&nbsp;                         'message' => 'The following target SVMs that are added to the project do not exist on the cluster.'
##&nbsp;                       },
##&nbsp;                       ]
##&nbsp;           'Warning' => [],
##&nbsp;           'Informational' => [
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_revert_precheck {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    if (defined $args{"group-name"}) {
        $group = $args{"group-name"};
    }else{
        $group = "Default";
    }
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "Revert_precheck";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    
    my $state = undef;
    $args{"trans_mode"} = $MIGRATION_CFT_TAB;
    my $result_array = $self->_perform_operation($args{"project-name"},$REVERT_PRECHECK,
        $group,$operation,$parsed_output,$run_background,$state,\%args
    );

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;

}


sub transition_cft_revert_abort {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter() if $may_enter;

    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    $self->_click(
            id                      => $MIGRATION_DASHBOARD_LINK,
            error_checking_needed   => 0,
        ) ;

    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    my $var=1;
    while (1) {
        my $group_path = "//div/div[$var]/div/span/b[contains(text(),'$group')]";
        if ($self->_is_element_present($group_path)) {
            $self->_click(
                id                      => $group_path,
                error_checking_needed   => 0,
            ) ;
            last;
        }
        $var++;
    }
    my $locator = "gwt-debug-".$args{"project-name"};
    if ($self->_is_element_present($locator)) {
        $self->_click(
            id                      => $locator,
            error_checking_needed   => 0,
        ) ;
        my $popup_title;
        if( $self->_is_button_enabled($CANCEL_ROLLBACK) ) {
            $self->_click(
                id                      => $CANCEL_ROLLBACK,
                error_checking_needed   => 0,
            ) ;
            eval{
            $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
            };
            if ( ($popup_title =~ /Confirmation/i )){
                eval{
                    $self->_click(
                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                        error_checking_needed   => 0,
                    ) ;
                };
            }
            $self->_create_screenshot("rollback_abort_success");
            $self->_close_tab(
                string => "Migration",
            );

        }else{
            $self->_create_screenshot("rollback_cancel_disabled");
            $self->_close_tab(
                string => "Migration",
            );
            $self->_throw_exception("rollback cancel button is disabled");
        }
    }else{
        $self->_close_tab(
            string => "Migration",
        );
        $self->_throw_exception("Project with name ".$args{"project-name"}." does not exists");
    }

    $Log->exit() if $may_exit;

    return 1;

}


## @name :: transition_cft_revert_generate_manual_steps
##
## @summary Executes "transition_cft_revert_generate_manual_steps" command
##
## @description
##  This method is used to execute transition cft revert generate-manual-steps command
## @synopsis
##  transition_cft_revert_generate_manual_steps(%args);
## INPUTS TO METHOD CALL 'transition_cft_revert_generate_manual_steps' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_revert_generate_manual_steps(
##            "project-name"      => "Test",
##            "group-name"        => "new",
##            "parsed_output"     => 1,
##         );
##
##&nbsp;transition cft revert generate-manual-steps -p Test :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [],
##&nbsp;           'Warning' => [
##&nbsp;                         {
##&nbsp;                           'corrective_action' => '',
##&nbsp;                           'type' => 'Warning',
##&nbsp;                           'code' => '08023',
##&nbsp;                           'message' => 'If any new files, directories or qtrees were created on the transitioned 7-Mode aggregates during the Precommit Testing phase, those objects will be deleted during the revert operation.'
##&nbsp;                         }
##&nbsp;                        ],
##&nbsp;           'Informational' => [
##&nbsp;                                {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'f8020-213-101-102(10.238.219.45)' => [
##&nbsp;                                                                         'storage transition revert start -session-id 011b4fd2-f7eb-492a-beaa-2849fd6529b8 -node-name f8020-213-101 -kernel-version kern_vers_8_1 -aggr-trans-revert-attr cc61db8e-87af-11df-8350-123478563412:1574236446:AGGR_TRANS_RAID4_1_CFT_SNAPSHOT_ON_7MODE_20150706105757909:false',
##&nbsp;                                                                         'storage transition revert start -session-id 011b4fd2-f7eb-492a-beaa-2849fd6529b8 -node-name f8020-213-101 -kernel-version kern_vers_8_1 -aggr-trans-revert-attr 1e531da9-879b-11df-b134-123478563412:1574236446:AGGR0_0_CFT_SNAPSHOT_ON_7MODE_20150706105757331:true',
##&nbsp;                                                                         'storage transition revert start -session-id 011b4fd2-f7eb-492a-beaa-2849fd6529b8 -node-name f8020-213-102 -kernel-version kern_vers_8_1 -aggr-trans-revert-attr a4dd55b0-1fdd-11e5-9f77-123478563412:1574236413:AGGR1_CFT_SNAPSHOT_ON_7MODE_20150706105758565:false',
##&nbsp;                                                                         'storage transition revert start -session-id 011b4fd2-f7eb-492a-beaa-2849fd6529b8 -node-name f8020-213-101 -kernel-version kern_vers_8_1 -aggr-trans-revert-attr c61483bb-87af-11df-8350-123478563412:1574236446:AGGR_TRANS_RAID_DP_1_CFT_SNAPSHOT_ON_7MODE_20150706105757800:false',
##&nbsp;                                                                         'storage transition revert start -session-id 011b4fd2-f7eb-492a-beaa-2849fd6529b8 -node-name f8020-213-102 -kernel-version kern_vers_8_1 -aggr-trans-revert-attr 34f7c768-1fc2-11e5-b14f-123478563412:1574236413:AGGR0_1_CFT_SNAPSHOT_ON_7MODE_20150706105758019:true'
##&nbsp;                                                                       ],
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08081',
##&nbsp;                                 'message' => 'Required: Run the following diagnostic privilege level commands to revert the aggregates to their original 7-Mode state. You can verify the revert status of the aggregates by using the following command at the diagnostic privilege level: \'storage transition aggregates show-status\'.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_revert_generate_manual_steps {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "Generate_manual_step";
    my $parsed_output = 1;
    my $run_background = undef;
    my $state = undef;
    #$parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $args{"trans_mode"} = $MIGRATION_CFT_TAB;
    my $result_array = $self->_perform_operation($args{"project-name"},
        $GENERATE_MANUAL_STEPS,$group,$operation,$parsed_output,$run_background,
        $state,\%args
    );
    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;

}

## @name :: transition_cft_revert_verify7_mode
##
## @summary Executes "transition_cft_revert_verify7_mode" command
##
## @description
##  This method is used to execute transition cft create command
## @synopsis
##  transition_cft_revert_verify7_mode(%args);
## INPUTS TO METHOD CALL 'transition_cft_revert_verify7_mode' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_revert_verify7_mode(
##            "project-name"      => "Test",
##            "group-name"        => "new",
##            "parsed_output"     => 1,
##         );
##
## By default parsed_output will be "1",it will process all the precheck message and send it as
## parsed output. If in case parsed output is not need, it can be skipped by specifying
## parsed_output as "0"(Here the command does not return any parsed output)
##
##&nbsp;transition cft revert verify7-mode -p Test:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [
##&nbsp;                          {
##&nbsp;                         'corrective_action' => 'To continue with the transition, create these SVMs on the target cluster. For more information, refer to the 7-Mode Transition Tool Copy-Free Transiustered Data ONTAP Administration Guide.',
##&nbsp;                         'type' => 'Error',
##&nbsp;                         'code' => '52250',
##&nbsp;                         'message' => 'The following target SVMs that are added to the project do not exist on the cluster.'
##&nbsp;                       },
##&nbsp;                       ]
##&nbsp;           'Warning' => [],
##&nbsp;           'Informational' => [
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_revert_verify7_mode {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "verify7-mode";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    
    my $state = undef;
    if ($parsed_output != 1) {
        $state = $REVERT_SUCCESS;
    }
    $args{"trans_mode"} = $MIGRATION_CFT_TAB;
    my $result_array = $self->_perform_operation($args{"project-name"},
        $VERIFY_SEVEN_MODE,$group,$operation,$parsed_output,$run_background,
        $state,\%args
    );

    $Log->exit() if $may_exit;

    return (defined $result_array) ? $result_array : 1;

}

## @name :: transition_cft_confirm_execution_revert_manual_steps
##
## @summary Executes "transition_cft_confirm_execution_revert_manual_steps" command
##
## @description
##  This method is used to execute transition cft create command
## @synopsis
##  transition_cft_confirm_execution_revert_manual_steps(%args);
## INPUTS TO METHOD CALL 'transition_cft_revert_verify7_mode' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'seven-mode' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'cabling' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'cluster-mode' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_confirm_execution_revert_manual_steps(
##            "project-name"      => "Test",
##            "group-name"        => "new",
##            "cluster-mode"      => "yes",
##            "cabling"           => "yes",
##            "seven-mode"        => "yes",
##         );
##
##&nbsp;
##&nbsp;

sub transition_cft_confirm_execution_revert_manual_steps {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'seven-mode'         => { type => SCALAR , optional => 1},
            'cabling'            => { type => SCALAR , optional => 1},
            'cluster-mode'       => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter() if $may_enter;
    my $group = undef;
    my $var=1;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "verify7-mode";
    my $parsed_output = undef;
    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    my $state = undef;

    $self->_click(
        id                      => $MIGRATION_DASHBOARD_LINK,
        error_checking_needed   => 0,
    ) ;

    while (1) {
        my $group_path = "//div/div[$var]/div/span/b[contains(text(),'$group')]";
        if ($self->_is_element_present($group_path)) {
            $self->_click(
                id                      => $group_path,
                error_checking_needed   => 0,
            ) ;
            last;
        }
        $var++;
    }

    my $locator = "gwt-debug-".$args{"project-name"};
    if ($self->_is_element_present($locator)) {
        $self->_click(
            id                      => $locator,
            error_checking_needed   => 0,
        ) ;
        if ( lc $args{"cluster-mode"} eq "yes" ) {
            if ($self->_is_button_enabled($CMODE_CONFIRM_EXECUTION)) {
                $self->_click(
                    id                      => $CMODE_CONFIRM_EXECUTION,
                    error_checking_needed   => 1,
                ) ;
                if ( lc $args{"cabling"} eq "yes" ) {
                    if ($self->_is_button_enabled($CABLING_CONFIRM)) {
                        $self->_click(
                            id                      => $CABLING_CONFIRM,
                            error_checking_needed   => 1,
                        ) ;
                        if ( lc $args{"seven-mode"} eq "yes" ) {
                            if ($self->_is_button_enabled($SMODE_CONFIRM_EXECUTION)) {
                                $self->_click(
                                    id                      => $SMODE_CONFIRM_EXECUTION,
                                    error_checking_needed   => 1,
                                ) ;
                                $self->_close_tab(
                                    string => "Migration",
                                );
                            }else{
                                $self->_create_screenshot("Smode_confirmation_button_disabled");
                                $self->_close_tab(
                                    string => "Migration",
                                );
                                $self->_throw_exception("The smode confirmation button is disbaled");
                            }
                        }
                    }else{
                        $self->_create_screenshot("Cabling_confirmation_button_disabled");
                        $self->_close_tab(
                            string => "Migration",
                        );
                        $self->_throw_exception("The cabling confirmation button is disbaled");
                    }
                }
            }else{
                $self->_create_screenshot("Cmode_confirmation_button_disabled");
                $self->_close_tab(
                    string => "Migration",
                );
                $self->_throw_exception("The cmode confirmation button is disbaled");
            }
        }
    }else{
        $self->_create_screenshot("Project_does_not_exits");
        $self->_close_tab(
            string => "Migration",
        );
        my $project =$args{"project-name"};
        $self->_throw_exception("The  project $project is not present");
    }

    $Log->exit() if $may_exit;

    return 1;

}

1;
