#$Id$
#
#Copyright (c) 2010-2013 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode storage commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode storage commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "storage show" will be storage_show()
##
## @synopsis
##    use storage;
##
##    storage_show_adapter(%args);
##    storage_enable_adapter(%args);
##    storage_disable_adapter(%args);
##    storage_show_bridge(%args);
##    storage_array_show(%args);
##    storage_array_show_config(%args);
##    storage_array_remove(%args);
##    storage_array_purge_database(%args);
##    storage_array_remove_port(%args);
##    storage_array_show_luns(%args);
##    storage_array_show_ports(%args);
##    storage_show_qualtable(%args);
##    storage_show_acp(%args);
##    storage_download_acp(%args);
##    storage_download_shelf(%args);
##    storage_array_modify(%args);
##    storage_show_switch(%args);
##    storage_release_disks(%args);
##
## @author faslur@netapp.com,dl-nacl-dev,xdl-storage-automation,dl-storsub-qa

## @status public

package NACL::APISet::Node::CLI::7Mode::default::storage;

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error         = undef;
    if ( $raw_output =~ /(Illegal option)|(Too many arguments)/ ) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ( $raw_output
        =~ /(Invalid host adapter)|(Invalid expander name)|(Invalid bridge name)|(.*?does not exist)/
        ) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif ( $raw_output =~ /[Uu]sage/i ) {
        $error          = "CommandFailed";
        $$error_message = "Command failed because of dependencies";
    } elsif ( $raw_output
        =~ /(There is no array profile with the name: .*)|(No Array Profile with the name .*)|(No Array Profile exists by the name .*)/
        ) {
        $error          = "InvalidParamValue";
        $$error_message = $&;
    } elsif ( $raw_output =~ /(Unable to get storage configuration data.)/ ) {
        $error          = "CommandFailed";
        $$error_message = $1;
    } elsif ( $raw_output =~ /(The WWPN \'\d+\' is not in a valid format)/ ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ( $raw_output
        =~ /[storage\s+download\s+acp]\s+[Mm]issing\s+shelf\s+information.\s+[uU]sage:\s+download.* |.storage\s+download\s+acp.\s+Unable\s+to\s+find\s+ACPP.* |.storage download acp.\s+Alternate\s+Control\s+Path\s+is\s+not\s+active.* |.storage\s+download\s+acp.\s+Invalid\s+firmware\s+filename.*/
        ) {
        $error         = "MissingRequiredParam";
        $$error_message = $raw_output;
    } elsif ( $raw_output
        =~ /.storage\s+download\s+acp.\s+ACP\s+\S+\s+\S+\s+\S+\s+not\s+eligible.*|.storage\s+download\s+acp.\s+No\s+ACP\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+firmware|.storage\s+download\s+acp.\s+Firmware\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+ACPP\S+.*/
        ) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    } elsif ( $raw_output
        =~ / Error: channel .* not found|Shelf update task is not running!|shelf_fw_update cannot run|Filename required for forced storage shelf download./
        ) {
        $error         = "CommandFailed";
        $$error_message = $raw_output;
    }

    return $error;
} ## end sub _check_error

## @name ::storage_show_adapter
##
## @summary Parses the raw output from the "storage show adapter" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_show_adapter with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_show_adapter(%args);
## INPUTS TO METHOD CALL 'storage_show_adapter' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter-name' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub storage_show_adapter {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "storage show adapter",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $first_line = "Slot";
    my @lines      = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );

    #In the actual hash for all the keys spaces will be replaced with '-'
    my @possible_keys = (
        "Slot",
        "Description",
        "Firmware Rev",
        "PCI Bus Width",
        "PCI Clock Speed",
        "Cacheline Size",
        "FC Packet Size",
        "Link Data Rate",
        "SRAM Parity",
        "External GBIC",
        "State",
        "In Use",
        "Redundant",
        "FC Node Name",
        "FC Port Name",
        "Base WWN"
    );
    my @result_array = ();

    for ( my $i = 0; $i <= $#lines; $i++ ) {

        my @words = split( /:\s+/, $lines[$i] );
        $words[1] =~ s/^\s*(.*)\s*$/$1/;
        chomp( $words[$#words] );

        #Check if the line contains any of the above defined possible keys
        my @temp = map( $_ =~ /^$words[0]/ ? $_ : (), @possible_keys );

        if ( $lines[$i] =~ /Slot/ ) {

            #Whenever 'Slot' is found start a new hash
            my %hash = ();
            push( @result_array, \%hash );
            $result_array[$#result_array]->{slot} = $words[1];
        } elsif (@temp) {

    #If any of the possible keys are present in the line store it in to a hash
    #Replace the spaces with '_'
            $temp[0] =~ s/\s+/_/g;
            my $temp_key = lc( $temp[0] );
            $result_array[$#result_array]->{$temp_key} = $words[1];
        } elsif ( $lines[$i] =~ /\[(\d)\]/ ) {

            #Phy state needs to be captured for SAS adapters
            my %temp_hash;
            $temp_hash{phy} = $1;
            if ( $lines[$i] =~ /Phy\s+State/ ) {
                my @temp_array;
                $result_array[$#result_array]->{phy_state} = \@temp_array;
            }

            push(
                @{ $result_array[$#result_array]->{phy_state} },
                \%temp_hash
            );
            if ( $lines[$i] =~ /,/ ) {
                my @tmp = split( /,/, $lines[$i] );
                $tmp[0] =~ s/^.*\[\d+\]\s+(\w+)\s*/$1/;
                chomp( $tmp[0] );
                $temp_hash{state} = $tmp[0];
                $tmp[1] =~ s/\s*(.*b\/s)\s*\(.*\)/$1/;
                chomp( $tmp[1] );
                $temp_hash{speed} = $tmp[1];
            } else {
                $lines[$i] =~ s/\s\(.\)$//;
                $temp_hash{state} = $lines[$i];
            }
        } ## end elsif ($lines[$i] =~ /\[(\d)\]/)

    } ## end for (my $i = 0 ; $i <= ...
    return \@result_array;
} ## end sub storage_show_adapter

## @name ::storage_show_expander
##
## @summary Parses the raw output from the "storage show expander" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_show_expander with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_show_expander(%args);
## INPUTS TO METHOD CALL 'storage_show_expander' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'expander-name' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub storage_show_expander {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "storage show expander",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $first_line = "Shelf name";
    my @lines      = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );

    #In the actual hash for all the keys spaces will be replaced with '-'
    my @possible_keys = (
        "Channel",
        "Module",
        "Shelf id",
        "Shelf UID",
        "Shelf S/N",
        "Term switch",
        "Shelf state",
        "Module state"
    );
    my @result_array = ();

    foreach my $line (@lines) {
        my @words = split( /:\s+/, $line );
        my @temp = map( $_ =~ /^$words[0]/ ? $_ : (), @possible_keys );
        if ( $line =~ /Shelf name:/ ) {

            #start a new hash for the new shelf
            my ( %temp_hash, @temp_array );
            push( @result_array, \%temp_hash );
            chomp( $words[1] );
            $result_array[$#result_array]->{shelf_name}    = $words[1];
            $result_array[$#result_array]->{other_details} = \@temp_array;
        } elsif (@temp) {
            $temp[0] =~ s/(\s+)/_/g;
            my $key = lc( $temp[0] );
            chomp( $words[1] );
            $result_array[$#result_array]->{$key} = $words[1];
        } elsif ( $line
            =~ /^\[\s*(\S*)\s*\]\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*$/
            ) {

#Store the other details in a hash and push this hash in to an array, reference of this array must be stored as
#the values to the key 'Other_details'.
            my %temp_hash;
            $temp_hash{disk_id}                       = $1;
            $temp_hash{port_state}                    = $2;
            $temp_hash{partial_path_timeout_value_ms} = $3;
            $temp_hash{link_rate_gb_per_s}            = $4;
            $temp_hash{invalid_dword_count}           = $5;
            $temp_hash{running_disparity_count}       = $6;
            $temp_hash{loss_dword_count}              = $7;
            $temp_hash{phy_reset_problem}             = $8;
            $temp_hash{crc_error_count}               = $9;
            $temp_hash{phy_change_count}              = $10;
            push(
                @{ $result_array[$#result_array]->{other_details} },
                \%temp_hash
            );
        } ## end elsif ($line =~ ...
    } ## end foreach my $line (@lines)
    return \@result_array;
} ## end sub storage_show_expander

## @name ::storage_show_shelf
##
## @summary Parses the raw output from the "storage show shelf" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_show_shelf with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_show_shelf(%args);
## INPUTS TO METHOD CALL 'storage_show_shelf' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'shelf-name' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## <tr> <td> '-e' </td> <td> 'extended' </td> </tr>
## <tr> <td> '-e' </td> <td> 'expander-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub storage_show_shelf {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "storage show shelf",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $first_line = "Shelf name";
    my @lines      = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );

    #In the actual hash for all the keys spaces will be replaced with '-'
    my @possible_keys = (
        "Channel",
        "Module",
        "Shelf id",
        "Shelf UID",
        "Shelf S/N",
        "Term switch",
        "Shelf state",
        "Module state",
        "Module uptime"
    );
    my @result_array = ();
    foreach my $line (@lines) {
        my @words = split( /:\s+/, $line );
        my @temp = map( $_ =~ /^$words[0]/ ? $_ : (), @possible_keys );
        if ( $line =~ /Shelf name:/ ) {

            #start a new hash for the new shelf
            my ( %temp_hash, @temp_array );
            push( @result_array, \%temp_hash );
            chomp( $words[1] );
            $result_array[$#result_array]->{shelf_name}    = $words[1];
            $result_array[$#result_array]->{other_details} = \@temp_array;
        } elsif (@temp) {
            $temp[0] =~ s/(\s+)/_/g;
            my $key = lc( $temp[0] );
            chomp( $words[1] );
            $result_array[$#result_array]->{$key} = $words[1];
        } elsif ( $line
            =~ /^\[\s*(\S*)\s*\]\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*$/
            ) {

#Store the other details in a hash and push this hash in to an array, reference of this array must be stored as
#the values to the key 'Other_details'.
            my %temp_hash;
            if ($2 eq 'NO' && $3 eq 'SIGNAL') {
                $temp_hash{disk_id}                       = $1;
                $temp_hash{port_state}                    = "$2"." "."$3";
                $temp_hash{partial_path_timeout_value_ms} = $4;
                $temp_hash{link_rate_gb_per_s}            = $5;
                $temp_hash{invalid_dword_count}           = $6;
                $temp_hash{running_disparity_count}       = $7;
                $temp_hash{loss_dword_count}              = $8;
                $temp_hash{phy_reset_problem}             = $9;
                $temp_hash{crc_error_count}               = $10;
                $temp_hash{phy_change_count}              = $11;
            } else {
                $temp_hash{disk_id}            = $1;
                $temp_hash{disk_bay}           = $2;
                $temp_hash{port_state}         = $3;
                $temp_hash{loop_up_count}      = $4;
                $temp_hash{invalid_crc_count}  = $5;
                $temp_hash{invalid_word_count} = $6;
                $temp_hash{clock_delta}        = $7;
                $temp_hash{insert_count}       = $8;
                $temp_hash{stall_count}        = $9;
                $temp_hash{util_percent}       = $10;
                $temp_hash{lip_count}          = $11;
            }
            push(
                @{ $result_array[$#result_array]->{other_details} },
                \%temp_hash
            );
        } elsif ( $line
            =~ /^\[\s*(\S*)\s*\]\s+(\s+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*$/
            ) {

#Store the other details in a hash and push this hash in to an array, reference of this array must be stored as
#the values to the key 'Other_details'.
            my %temp_hash;
            $temp_hash{disk_id}            = $1;
            $temp_hash{port_state}         = $3;
            $temp_hash{loop_up_count}      = $4;
            $temp_hash{invalid_crc_count}  = $5;
            $temp_hash{invalid_word_count} = $6;
            $temp_hash{clock_delta}        = $7;
            $temp_hash{insert_count}       = $8;
            $temp_hash{stall_count}        = $9;
            $temp_hash{util_percent}       = $10;
            $temp_hash{lip_count}          = $11;
            push(
                @{ $result_array[$#result_array]->{other_details} },
                \%temp_hash
            );
        } elsif ( $line
            =~ /^\[\s*(\S*)\s*\]\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*$/
            ) {

#Store the other details in a hash and push this hash in to an array, reference of this array must be stored as
#the values to the key 'Other_details'.
            my %temp_hash;
            $temp_hash{disk_id}                       = $1;
            $temp_hash{port_state}                    = $2;
            $temp_hash{partial_path_timeout_value_ms} = $3;
            $temp_hash{link_rate_gb_per_s}            = $4;
            $temp_hash{invalid_dword_count}           = $5;
            $temp_hash{running_disparity_count}       = $6;
            $temp_hash{loss_dword_count}              = $7;
            $temp_hash{phy_reset_problem}             = $8;
            $temp_hash{crc_error_count}               = $9;
            $temp_hash{phy_change_count}              = $10;
            push(
                @{ $result_array[$#result_array]->{other_details} },
                \%temp_hash
            );
        } elsif ( $line
            =~ /^\[\s*(\S*)\s*\]\s+(\s+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*$/
            ) {
            my %temp_hash;
            $temp_hash{disk_id}                = $1;
            $temp_hash{port_state}             = $3;
            $temp_hash{sourced_lip_count}      = $4;
            $temp_hash{bypass_slap_count}      = $5;
            $temp_hash{oscillation_slap_count} = $6;
            $temp_hash{stuck_int_count}        = $7;
            $temp_hash{maximum_ptbi_time}      = $8;
            $temp_hash{port_lostnode_count}    = $9;
            push(
                @{ $result_array[$#result_array]->{other_details} },
                \%temp_hash
            );
        } elsif ( $line
            =~ /^\[\s*(\S*)\s*\]\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*$/
            ) {
            my %temp_hash;
            $temp_hash{disk_id}                = $1;
            $temp_hash{disk_bay}               = $2;
            $temp_hash{port_state}             = $3;
            $temp_hash{sourced_lip_count}      = $4;
            $temp_hash{bypass_slap_count}      = $5;
            $temp_hash{oscillation_slap_count} = $6;
            $temp_hash{stuck_int_count}        = $7;
            $temp_hash{maximum_ptbi_time}      = $8;
            $temp_hash{port_lostnode_count}    = $9;
            push(
                @{ $result_array[$#result_array]->{other_details} },
                \%temp_hash
            );
        } elsif ( $line
            =~ /\S+\:\S+\s+\S+\:\S+\s+\S+\:\S+\s+\S+\:\S+\s+\S+\:\S+\s+\S+\:\S+\s+\S+\:\S+\s*/
            ) {
            my @keys = (
                "BZR",         "COS",
                "DoubleOpens", "ForcedSwitch",
                "LipSys",      "BadLim",
                "BadRouterTable"
            );
            my @words1 = split( /\s+/, $line );
            foreach (@words1) {
                my @words2 = split( /:/, $_ );
                my @temp1 = map( $_ =~ /^$words2[0]/ ? $_ : (), @keys );
                $temp1[0] =~ s/(\s+)/_/g;
                my $key = lc( $temp1[0] );
                chomp( $words2[1] );
                $result_array[$#result_array]->{$key} = $words2[1];
            }
        }
    }
	return \@result_array;
} ## end sub storage_show_shelf

## @name ::storage_load_summary
##
## @summary Parses the raw output from the "storage load summary" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_load_summary with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_load_summary(%args);
## INPUTS TO METHOD CALL 'storage_show_expander' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'initiator-port-name' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
## The parsed output look like
## $VAR1 = [
##         }
##           'array-port' => '500060e802eeb602',
##           'switch' => 'vgbr200es12:3',
##           'load-balance-time' => 'Mon May  2 15:09:04 GMT 2011',
##           'number-of-blocks' => '0',
##           'lun-count' => '25',
##           'load' => '0%',
##           'port' => '0c'
##         },
##         {
##           'array-port' => '500060e802eeb603',
##           'switch' => 'vgbr300s25:3',
##           'load-balance-time' => 'Mon May  2 15:09:04 GMT 2011',
##           'number-of-blocks' => '3',
##           'lun-count' => '25',
##           'load' => '6%',
##           'port' => '0d'
##         },
##         .........
##       ];
##
sub storage_load_summary {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "storage load summary",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    my $first_line
        = 'Port\s+Total\s+Load\s+Switch\s+Array\s+Port\s+#Blocks\s+Array\s+LUN\s+Count';
    my @lines = get_lines(
        raw_output      => $raw_output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );

    #In the actual hash for all the keys spaces will be replaced with '-'
    my @possible_keys = (
        "port",             "load", "switch", "array-port",
        "number-of-blocks", "lun-count"
    );

    my @result_array     = ();
    my $result_array_ref = \@result_array;
    my $last_string      = qr/I\/O was last balanced at:/;
    my $num_entries      = 0;

    foreach my $line (@lines) {

        my @summary_values = ();
        my %line_hash      = ();
        my $line_hash_ref  = \%line_hash;

        next if ( $line =~ $first_line );

        if ( $line =~ $last_string ) {
            $line =~ s/$last_string\s+//;
            for ( my $j = 0; $j < $num_entries; $j++ ) {
                $result_array[$j]{"load-balance-time"} = $line;
            }
            last;
        }

        @summary_values = split( /\s+/, $line );
        for ( my $i = 0; $i < scalar(@possible_keys); $i++ ) {
            $line_hash{ $possible_keys[$i] } = $summary_values[$i];
        }

        push( @result_array, $line_hash_ref );
        $num_entries++;

    }

    return $result_array_ref;
} ## end sub storage_load_summary

## @name ::storage_enable_adapter
##
## @summary Parses the raw output from the "storage enable adapter <adapter-name>" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_enable_adapter with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_enable_adapter(%args);
## INPUTS TO METHOD CALL 'storage_enable_adapter' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = storage enable adapter 0a
##&nbsp;$VAR1 = [];
##&nbsp;

sub storage_enable_adapter {
    my %args = @_;

    my $error_message;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        command       => "storage enable adapter",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub storage_enable_adapter

## @name ::storage_disable_adapter
##
## @summary Parses the raw output from the "storage disable adapter <adapter-name>" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_disable_adapter with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_disable_adapter(%args);
## INPUTS TO METHOD CALL 'storage_disable_adapter' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = storage disable adapter 0a
##&nbsp;$VAR1 = [];
##&nbsp;
sub storage_disable_adapter {
    my %args = @_;

    my $error_message;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        command       => "storage disable adapter",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    return [];
} ## end sub storage_disable_adapter

## @name ::storage_show_bridge
##
## @summary Parses the raw output from the "storage show bridge" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_show_bridge with any of its options as described in the
##    corresponding cdef file 'fcadmin.cdefs'
## @synopsis
##      storage_show_bridge(%args);
## INPUTS TO METHOD CALL 'storage_show_bridge' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'bridge-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = storage show bridge
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'serial' => 'FB6500N100135',
##&nbsp;            'rev' => '1.01',
##&nbsp;            'model' => 'FibreBridge6500N',
##&nbsp;            'bridge' => '0d.125L0',
##&nbsp;            'vendor' => 'ATTO'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'serial' => 'FB6500N100103',
##&nbsp;            'rev' => '1.01',
##&nbsp;            'model' => 'FibreBridge6500N',
##&nbsp;            'bridge' => '0c.125L0',
##&nbsp;            'vendor' => 'ATTO'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'serial' => 'FB6500N100184',
##&nbsp;            'rev' => '1.01',
##&nbsp;            'model' => 'FibreBridge6500N',
##&nbsp;            'bridge' => '4b.125L0',
##&nbsp;            'vendor' => 'ATTO'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'serial' => 'FB6500N100151',
##&nbsp;            'rev' => '1.01',
##&nbsp;            'model' => 'FibreBridge6500N',
##&nbsp;            'bridge' => '4a.125L0',
##&nbsp;            'vendor' => 'ATTO'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = storage show bridge 0d.125L0
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;               'serial' => 'FB6500N100135',
##&nbsp;               'rev' => '1.01',
##&nbsp;               'model' => 'FibreBridge6500N',
##&nbsp;               'bridge' => '0d.125L0',
##&nbsp;               'vendor' => 'ATTO'
##&nbsp;           }
##&nbsp;        ];

sub storage_show_bridge {
    my %args = @_;

    my ( $error_message, @result_array,@lines );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "storage show bridge",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    @lines = get_lines( raw_output => $output, response_object => $resp );
    if($lines[0] =~ /No\s+bridges\s+found/) {
        return [];
    }

    my $first_line = "Bridge";
    @lines      = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );

    foreach my $line (@lines) {
        my @words = split( /\s+/, $line );
        if ( $line =~ /Bridge:/ ) {
            my %hash = ();
            push( @result_array, \%hash );
            $result_array[$#result_array]->{'bridge'} = $words[1];
        } elsif ( $line =~ /Serial:/ ) {
            $result_array[$#result_array]->{'serial'} = $words[1];
        } elsif ( $line =~ /Vendor:/ ) {
            $result_array[$#result_array]->{'vendor'} = $words[1];
        } elsif ( $line =~ /Model:/ ) {
            $result_array[$#result_array]->{'model'} = $words[1];
        } elsif ( $line =~ /Rev:/ ) {
            $result_array[$#result_array]->{'rev'} = $words[1];
        }
    }
    return \@result_array;
} ## end sub storage_show_bridge

## @name ::storage_show_disk
##
## @summary Parses the raw output from the "storage show disk" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_show_disk with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_show_disk(%args);
## INPUTS TO METHOD CALL 'storage_show_disk' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-x' </td> <td> 'specific' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-name' </td> </tr>
## <tr> <td> '-p' </td> <td> 'path' </td> </tr>
## <tr> <td> '-T' </td> <td> 'type' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = storage show disk 0a.25
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'disk' => '0a.25',
##&nbsp;            'model' => 'X292_S15K7560F15',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '9',
##&nbsp;            'uid' => '2000001D:38FAB8E3:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3SL01Y4W00009005S92U',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:000:001d38:fab8e3',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;Command = storage show disk
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bay' => '1',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.17',
##&nbsp;            'serial' => 'J4V4E2YC',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'model' => 'X278_HVIPB',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '2',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.18',
##&nbsp;            'serial' => '3LM05074000097064PU7',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'model' => 'X279_S15K5',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '3',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.19',
##&nbsp;            'serial' => 'V3V07YEA',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'model' => 'X272_HPYTA',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.20',
##&nbsp;            'serial' => 'J4V4DYSC',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'model' => 'X278_HVIPB',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.21',
##&nbsp;            'serial' => '3KN094CG000075450QXD',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'model' => 'X275_S15K4',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '6',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.22',
##&nbsp;            'serial' => 'J8V4NHKC',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'model' => 'X279_HVIPB',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '7',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.23',
##&nbsp;            'serial' => '3LM04HKZ00007702JB9Z',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'model' => 'X279_S15K5',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '8',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.24',
##&nbsp;            'serial' => 'V5V0BBBA',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'model' => 'X274_HPYTA',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '9',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.25',
##&nbsp;            'serial' => '3SL01Y4W00009005S92U',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'model' => 'X292_S15K7',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '10',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.26',
##&nbsp;            'serial' => 'DH07P67000SE',
##&nbsp;            'rev' => 'NA05',
##&nbsp;            'model' => 'X276_FAL9E',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '11',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.27',
##&nbsp;            'serial' => 'J8V3THBC',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'model' => 'X279_HVIPB',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '12',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.28',
##&nbsp;            'serial' => 'J8V47PBC',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'model' => 'X279_HVIPB',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '13',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.29',
##&nbsp;            'serial' => 'DH07P67000M0',
##&nbsp;            'rev' => 'NA05',
##&nbsp;            'model' => 'X276_FAL9E',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '0',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.32',
##&nbsp;            'serial' => '3QD0NYDM',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '1',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.33',
##&nbsp;            'serial' => '3QD0NYME',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '2',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.34',
##&nbsp;            'serial' => 'P6G01DAA',
##&nbsp;            'rev' => 'A90A',
##&nbsp;            'model' => 'X267_HGEMI',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '3',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.35',
##&nbsp;            'serial' => 'L639LKCG',
##&nbsp;            'rev' => 'BEGG',
##&nbsp;            'model' => 'X266_MSABR',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.36',
##&nbsp;            'serial' => '3QD0NZBS',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.37',
##&nbsp;            'serial' => '3QD0NZCP',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '6',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.38',
##&nbsp;            'serial' => '3QD0NXF3',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '12',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.44',
##&nbsp;            'serial' => '3QD0NYCH',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '13',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.45',
##&nbsp;            'serial' => '3QD0P7HK',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '0',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.0',
##&nbsp;            'serial' => '3SJ0XYNA00009036EMRH',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '1',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.1',
##&nbsp;            'serial' => '3SJ0YGP600009035RSPQ',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '2',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.2',
##&nbsp;            'serial' => '3SJ102PT00009038U7EC',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '3',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.3',
##&nbsp;            'serial' => '3SJ0XYMT00009036Y7JW',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.4',
##&nbsp;            'serial' => '3SJ1030L00009037FUPP',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.5',
##&nbsp;            'serial' => '3SJ1033A000090380KFL',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '6',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.6',
##&nbsp;            'serial' => '3SJ1035400009037JJP9',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '7',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.7',
##&nbsp;            'serial' => '3SJ1033G00009038U78W',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '8',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.8',
##&nbsp;            'serial' => '3SJ102T9000090380CHC',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '9',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.9',
##&nbsp;            'serial' => '3SJ1033900009038U7P4',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '10',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.10',
##&nbsp;            'serial' => '3SJ0ZRA000009038U7GT',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.0',
##&nbsp;            'serial' => 'WD-WMAUR0067764',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'model' => 'X306_WMANT',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.4',
##&nbsp;            'serial' => 'WD-WCAUR0005089',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'model' => 'X306_WMANT',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.5',
##&nbsp;            'serial' => 'WD-WMAUR0073515',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'model' => 'X306_WMANT',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '7',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.7',
##&nbsp;            'serial' => 'MH043X4B',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '9',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.9',
##&nbsp;            'serial' => 'MH046SAB',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '10',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.10',
##&nbsp;            'serial' => 'MH0442MB',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '14',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.14',
##&nbsp;            'serial' => 'WD-WCAW30104128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'model' => 'X302_WVULC',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '16',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.16',
##&nbsp;            'serial' => 'WD-WCAW30091789',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'model' => 'X302_WVULC',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '17',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.17',
##&nbsp;            'serial' => 'WD-WCAW30107688',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'model' => 'X302_WVULC',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '19',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.19',
##&nbsp;            'serial' => 'MH1RTH6W',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '20',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.20',
##&nbsp;            'serial' => 'ME031SHW',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X310_HSATE',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '21',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.21',
##&nbsp;            'serial' => 'MH035GJB',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '22',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.22',
##&nbsp;            'serial' => 'MH3WH9YK',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '23',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.23',
##&nbsp;            'serial' => 'MH0394NB',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;Command = storage show disk -a
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'disk_qualification_package_details' => {
##&nbsp;                                                      'package_date' => 'Unable to load any package (Unknown error or Package may not be present)',
##&nbsp;                                                      'header_information' => {
##&nbsp;                                                                                'DriveRecordCount' => 'N/A',
##&nbsp;                                                                                'FileName' => 'N/A',
##&nbsp;                                                                                'FileVersion' => 'N/A',
##&nbsp;                                                                                'AliasRecordCount' => 'N/A'
##&nbsp;                                                                              }
##&nbsp;                                                    }
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0a.17',
##&nbsp;            'model' => 'X278_HVIPB146F15',
##&nbsp;            'home_owner' => '135095648',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '61',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095648',
##&nbsp;            'bay' => '1',
##&nbsp;            'uid' => '2004000C:CA080A90:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => '31442',
##&nbsp;            'serial' => 'J4V4E2YC',
##&nbsp;            'time_interval' => '32:23:06',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:004:000cca:080a90',
##&nbsp;            'blocks_read' => '10688914168',
##&nbsp;            'blocks_written' => '148901327',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095648',
##&nbsp;            'scrub_last_done' => '00:11:56',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0a.18',
##&nbsp;            'model' => 'X279_S15K5288F15',
##&nbsp;            'home_owner' => '135095648',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '61',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095648',
##&nbsp;            'bay' => '2',
##&nbsp;            'uid' => '20000018:621BF7F2:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => '38156',
##&nbsp;            'serial' => '3LM05074000097064PU7',
##&nbsp;            'time_interval' => '32:23:06',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:000:001862:1bf7f2',
##&nbsp;            'blocks_read' => '10667913678',
##&nbsp;            'blocks_written' => '176240015',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095648',
##&nbsp;            'scrub_last_done' => '00:11:56',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0a.19',
##&nbsp;            'model' => 'X272_HPYTA073F10',
##&nbsp;            'home_owner' => '135095648',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '102',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '10000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095648',
##&nbsp;            'bay' => '3',
##&nbsp;            'uid' => '20000000:871E08A8:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => '58063',
##&nbsp;            'serial' => 'V3V07YEA',
##&nbsp;            'time_interval' => '32:23:06',
##&nbsp;            'glist_count' => '5',
##&nbsp;            'wwn' => '2:000:000087:1e08a8',
##&nbsp;            'blocks_read' => '391784',
##&nbsp;            'blocks_written' => '3842',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095648',
##&nbsp;            'scrub_last_done' => '00:00:20',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0a.20',
##&nbsp;            'model' => 'X278_HVIPB146F15',
##&nbsp;            'home_owner' => '135095648',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '61',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095648',
##&nbsp;            'bay' => '4',
##&nbsp;            'uid' => '2004000C:CA080888:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => '31325',
##&nbsp;            'serial' => 'J4V4DYSC',
##&nbsp;            'time_interval' => '32:23:06',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:004:000cca:080888',
##&nbsp;            'blocks_read' => '10662647150',
##&nbsp;            'blocks_written' => '185281375',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095648',
##&nbsp;            'scrub_last_done' => '00:11:56',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0a.21',
##&nbsp;            'model' => 'X275_S15K4146F15',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '5',
##&nbsp;            'uid' => '20000011:C6D025E5:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3KN094CG000075450QXD',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:000:0011c6:d025e5',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0a.22',
##&nbsp;            'model' => 'X279_HVIPB288F15',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '6',
##&nbsp;            'uid' => '2004000C:CA0879C8:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'J8V4NHKC',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:004:000cca:0879c8',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0a.23',
##&nbsp;            'model' => 'X279_S15K5288F15',
##&nbsp;            'home_owner' => '135095648',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '79',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095648',
##&nbsp;            'bay' => '7',
##&nbsp;            'uid' => '20000018:621BF886:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => '38179',
##&nbsp;            'serial' => '3LM04HKZ00007702JB9Z',
##&nbsp;            'time_interval' => '32:23:06',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:000:001862:1bf886',
##&nbsp;            'blocks_read' => '10750016',
##&nbsp;            'blocks_written' => '4968',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095648',
##&nbsp;            'scrub_last_done' => '00:00:20',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0a.24',
##&nbsp;            'model' => 'X274_HPYTA146F10',
##&nbsp;            'home_owner' => '135095648',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '86',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '10000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095648',
##&nbsp;            'bay' => '8',
##&nbsp;            'uid' => '20000000:876BDFC0:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => '57978',
##&nbsp;            'serial' => 'V5V0BBBA',
##&nbsp;            'time_interval' => '32:23:06',
##&nbsp;            'glist_count' => '12',
##&nbsp;            'wwn' => '2:000:000087:6bdfc0',
##&nbsp;            'blocks_read' => '391736',
##&nbsp;            'blocks_written' => '4146',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095648',
##&nbsp;            'scrub_last_done' => '00:00:20',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0a.25',
##&nbsp;            'model' => 'X292_S15K7560F15',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '9',
##&nbsp;            'uid' => '2000001D:38FAB8E3:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3SL01Y4W00009005S92U',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:000:001d38:fab8e3',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0a.26',
##&nbsp;            'model' => 'X276_FAL9E288F10',
##&nbsp;            'home_owner' => '4294967295',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '10000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '0',
##&nbsp;            'bay' => '10',
##&nbsp;            'uid' => '500000E0:126E3580:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'DH07P67000SE',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:000:00e012:6e3580',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '4294967295',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA05',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0a.27',
##&nbsp;            'model' => 'X279_HVIPB288F15',
##&nbsp;            'home_owner' => '135095648',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '79',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095648',
##&nbsp;            'bay' => '11',
##&nbsp;            'uid' => '2004000C:CA06E444:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => '31443',
##&nbsp;            'serial' => 'J8V3THBC',
##&nbsp;            'time_interval' => '32:23:06',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:004:000cca:06e444',
##&nbsp;            'blocks_read' => '10749984',
##&nbsp;            'blocks_written' => '5496',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095648',
##&nbsp;            'scrub_last_done' => '00:00:20',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0a.28',
##&nbsp;            'model' => 'X279_HVIPB288F15',
##&nbsp;            'home_owner' => '135095648',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '79',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095648',
##&nbsp;            'bay' => '12',
##&nbsp;            'uid' => '2004000C:CA07B964:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => '31443',
##&nbsp;            'serial' => 'J8V47PBC',
##&nbsp;            'time_interval' => '32:23:06',
##&nbsp;            'glist_count' => '1',
##&nbsp;            'wwn' => '2:004:000cca:07b964',
##&nbsp;            'blocks_read' => '10750762',
##&nbsp;            'blocks_written' => '5008',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095648',
##&nbsp;            'scrub_last_done' => '00:00:20',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0a.29',
##&nbsp;            'model' => 'X276_FAL9E288F10',
##&nbsp;            'home_owner' => '135095648',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '78',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '10000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095648',
##&nbsp;            'bay' => '13',
##&nbsp;            'uid' => '500000E0:126CA180:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => '17946',
##&nbsp;            'serial' => 'DH07P67000M0',
##&nbsp;            'time_interval' => '32:23:06',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:000:00e012:6ca180',
##&nbsp;            'blocks_read' => '391920',
##&nbsp;            'blocks_written' => '3842',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095648',
##&nbsp;            'scrub_last_done' => '00:00:20',
##&nbsp;            'rev' => 'NA05',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0b.32',
##&nbsp;            'model' => 'X268_SGLXY750SSX',
##&nbsp;            'home_owner' => '4294967295',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '0',
##&nbsp;            'bay' => '0',
##&nbsp;            'uid' => '373F1F54:E067D32D:8C948E58:294E53C5:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3QD0NYDM',
##&nbsp;            'time_interval' => '28:12:48',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:208:000a33:07c828',
##&nbsp;            'blocks_read' => '84528',
##&nbsp;            'blocks_written' => '824',
##&nbsp;            'shelf' => '2',
##&nbsp;            'current_owner' => '4294967295',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0b.33',
##&nbsp;            'model' => 'X268_SGLXY750SSX',
##&nbsp;            'home_owner' => '4294967295',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '0',
##&nbsp;            'bay' => '1',
##&nbsp;            'uid' => '6254043B:6680FBE7:DDDE5823:880F7EB3:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3QD0NYME',
##&nbsp;            'time_interval' => '28:12:48',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:208:000a33:07c825',
##&nbsp;            'blocks_read' => '83216',
##&nbsp;            'blocks_written' => '168',
##&nbsp;            'shelf' => '2',
##&nbsp;            'current_owner' => '4294967295',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0b.34',
##&nbsp;            'model' => 'X267_HGEMIT50SSX',
##&nbsp;            'home_owner' => '4294967295',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '3',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '0',
##&nbsp;            'bay' => '2',
##&nbsp;            'uid' => '29E351DC:9BDFA278:B0A2B8E6:8294AD0D:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'P6G01DAA',
##&nbsp;            'time_interval' => '28:12:48',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:208:000a33:08b1de',
##&nbsp;            'blocks_read' => '84528',
##&nbsp;            'blocks_written' => '824',
##&nbsp;            'shelf' => '2',
##&nbsp;            'current_owner' => '4294967295',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'A90A',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0b.35',
##&nbsp;            'model' => 'X266_MSABR320SSX',
##&nbsp;            'home_owner' => '4294967295',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '5',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '0',
##&nbsp;            'bay' => '3',
##&nbsp;            'uid' => '480757AD:FEF96DE6:FED7F63F:6495FBFA:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'L639LKCG',
##&nbsp;            'time_interval' => '28:12:48',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:208:000a33:847151',
##&nbsp;            'blocks_read' => '84528',
##&nbsp;            'blocks_written' => '824',
##&nbsp;            'shelf' => '2',
##&nbsp;            'current_owner' => '4294967295',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'BEGG',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0b.36',
##&nbsp;            'model' => 'X268_SGLXY750SSX',
##&nbsp;            'home_owner' => '4294967295',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '0',
##&nbsp;            'bay' => '4',
##&nbsp;            'uid' => 'B72A47A7:6EB7A9AC:2FD1E5BB:2C2B75A9:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3QD0NZBS',
##&nbsp;            'time_interval' => '28:12:48',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:208:000a33:07c757',
##&nbsp;            'blocks_read' => '84528',
##&nbsp;            'blocks_written' => '824',
##&nbsp;            'shelf' => '2',
##&nbsp;            'current_owner' => '4294967295',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0b.37',
##&nbsp;            'model' => 'X268_SGLXY750SSX',
##&nbsp;            'home_owner' => '4294967295',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '0',
##&nbsp;            'bay' => '5',
##&nbsp;            'uid' => 'FFDC351A:DA3A2ABB:D2C6B2C4:9F9A7FE9:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3QD0NZCP',
##&nbsp;            'time_interval' => '28:12:48',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:208:000a33:07d250',
##&nbsp;            'blocks_read' => '84528',
##&nbsp;            'blocks_written' => '824',
##&nbsp;            'shelf' => '2',
##&nbsp;            'current_owner' => '4294967295',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0b.38',
##&nbsp;            'model' => 'X268_SGLXY750SSX',
##&nbsp;            'home_owner' => '4294967295',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '0',
##&nbsp;            'bay' => '6',
##&nbsp;            'uid' => '37FAB3B8:E5689512:4868817A:CDE8DE42:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3QD0NXF3',
##&nbsp;            'time_interval' => '28:12:48',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:208:000a33:07d213',
##&nbsp;            'blocks_read' => '84528',
##&nbsp;            'blocks_written' => '824',
##&nbsp;            'shelf' => '2',
##&nbsp;            'current_owner' => '4294967295',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0b.44',
##&nbsp;            'model' => 'X268_SGLXY750SSX',
##&nbsp;            'home_owner' => '4294967295',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '0',
##&nbsp;            'bay' => '12',
##&nbsp;            'uid' => '55D629A4:F1E68395:ECFE69B7:17869373:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3QD0NYCH',
##&nbsp;            'time_interval' => '28:12:48',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:208:000a33:07c7fb',
##&nbsp;            'blocks_read' => '84528',
##&nbsp;            'blocks_written' => '824',
##&nbsp;            'shelf' => '2',
##&nbsp;            'current_owner' => '4294967295',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0b.45',
##&nbsp;            'model' => 'X268_SGLXY750SSX',
##&nbsp;            'home_owner' => '4294967295',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '0',
##&nbsp;            'bay' => '13',
##&nbsp;            'uid' => 'D7509942:44757B7C:E885AED2:0D6A2AE9:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3QD0P7HK',
##&nbsp;            'time_interval' => '28:12:48',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '2:208:000a33:07c79f',
##&nbsp;            'blocks_read' => '84528',
##&nbsp;            'blocks_written' => '824',
##&nbsp;            'shelf' => '2',
##&nbsp;            'current_owner' => '4294967295',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0c.00.0',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '0',
##&nbsp;            'uid' => '5000C500:22E9F3A3:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3SJ0XYNA00009036EMRH',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:000:c50022:e9f3a0',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '0',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0c.00.1',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '5',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '1',
##&nbsp;            'uid' => '5000C500:22E9826B:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3SJ0YGP600009035RSPQ',
##&nbsp;            'time_interval' => '28:12:48',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:000:c50022:e98268',
##&nbsp;            'blocks_read' => '84528',
##&nbsp;            'blocks_written' => '824',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '0',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0c.00.2',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '2',
##&nbsp;            'uid' => '5000C500:22FA0DE7:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3SJ102PT00009038U7EC',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:000:c50022:fa0de4',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '0',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0c.00.3',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '3',
##&nbsp;            'uid' => '5000C500:22E9F477:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3SJ0XYMT00009036Y7JW',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:000:c50022:e9f474',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '0',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0c.00.4',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '4',
##&nbsp;            'uid' => '5000C500:22FB78B3:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3SJ1030L00009037FUPP',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:000:c50022:fb78b0',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '0',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0c.00.5',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '5',
##&nbsp;            'uid' => '5000C500:22FA270F:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3SJ1033A000090380KFL',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:000:c50022:fa270c',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '0',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0c.00.6',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '6',
##&nbsp;            'uid' => '5000C500:22FA2D0B:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3SJ1035400009037JJP9',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:000:c50022:fa2d08',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '0',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0c.00.7',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '7',
##&nbsp;            'uid' => '5000C500:22FA27E7:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3SJ1033G00009038U78W',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:000:c50022:fa27e4',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '0',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0c.00.8',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '8',
##&nbsp;            'uid' => '5000C500:22FA1347:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3SJ102T9000090380CHC',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:000:c50022:fa1344',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '0',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0c.00.9',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '5',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '9',
##&nbsp;            'uid' => '5000C500:22FA2667:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3SJ1033900009038U7P4',
##&nbsp;            'time_interval' => '28:12:48',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:000:c50022:fa2664',
##&nbsp;            'blocks_read' => '84528',
##&nbsp;            'blocks_written' => '824',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '0',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0c.00.10',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '5',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '15000',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '10',
##&nbsp;            'uid' => '5000C500:22FA170B:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => '3SJ0ZRA000009038U7GT',
##&nbsp;            'time_interval' => '28:12:48',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:000:c50022:fa1708',
##&nbsp;            'blocks_read' => '84528',
##&nbsp;            'blocks_written' => '824',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '0',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0d.01.0',
##&nbsp;            'model' => 'X306_WMANT02TSSM',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '0',
##&nbsp;            'uid' => '500605BA:000760B8:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'WD-WMAUR0067764',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:006:05ba00:0760b8',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0d.01.4',
##&nbsp;            'model' => 'X306_WMANT02TSSM',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '4',
##&nbsp;            'uid' => '500605BA:00076118:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'WD-WCAUR0005089',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:006:05ba00:076118',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0d.01.5',
##&nbsp;            'model' => 'X306_WMANT02TSSM',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '5',
##&nbsp;            'uid' => '500605BA:000518D4:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'WD-WMAUR0073515',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:006:05ba00:0518d4',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0d.01.7',
##&nbsp;            'model' => 'X302_HSATE01TSSM',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '7',
##&nbsp;            'uid' => '500605BA:00075BE4:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'MH043X4B',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:006:05ba00:075be4',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0d.01.9',
##&nbsp;            'model' => 'X302_HSATE01TSSM',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '9',
##&nbsp;            'uid' => '500605BA:00075C9C:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'MH046SAB',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:006:05ba00:075c9c',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0d.01.10',
##&nbsp;            'model' => 'X302_HSATE01TSSM',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '10',
##&nbsp;            'uid' => '500605BA:000755DC:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'MH0442MB',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:006:05ba00:0755dc',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0d.01.14',
##&nbsp;            'model' => 'X302_WVULC01TSSA',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '14',
##&nbsp;            'uid' => '50000C90:0000ADCC:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'WD-WCAW30104128',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:000:0c9000:00adcc',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0d.01.16',
##&nbsp;            'model' => 'X302_WVULC01TSSM',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '16',
##&nbsp;            'uid' => '500605BA:002A1268:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'WD-WCAW30091789',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:006:05ba00:2a1268',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0d.01.17',
##&nbsp;            'model' => 'X302_WVULC01TSSM',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '17',
##&nbsp;            'uid' => '500605BA:002AAA6C:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'WD-WCAW30107688',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:006:05ba00:2aaa6c',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0d.01.19',
##&nbsp;            'model' => 'X302_HSATE01TSSM',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '19',
##&nbsp;            'uid' => '5006056B:000347EC:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'MH1RTH6W',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:006:056b00:0347ec',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0d.01.20',
##&nbsp;            'model' => 'X310_HSATET50SSM',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '20',
##&nbsp;            'uid' => '500605BA:00075680:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'ME031SHW',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:006:05ba00:075680',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0d.01.21',
##&nbsp;            'model' => 'X302_HSATE01TSSM',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '21',
##&nbsp;            'uid' => '500605BA:000759F8:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'MH035GJB',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:006:05ba00:0759f8',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0d.01.22',
##&nbsp;            'model' => 'X302_HSATE01TSSM',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '22',
##&nbsp;            'uid' => '500605BA:0000040C:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'MH3WH9YK',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:006:05ba00:00040c',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => '0d.01.23',
##&nbsp;            'model' => 'X302_HSATE01TSSM',
##&nbsp;            'home_owner' => '135095589',
##&nbsp;            'lip_count' => '0',
##&nbsp;            'scrub_count' => '0',
##&nbsp;            'downrev' => 'no',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'pri_port' => 'B',
##&nbsp;            'reservation_owner' => '135095589',
##&nbsp;            'bay' => '23',
##&nbsp;            'uid' => '500605BA:000759FC:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
##&nbsp;            'power-on_hours' => 'N/A',
##&nbsp;            'serial' => 'MH0394NB',
##&nbsp;            'time_interval' => '00:00:00',
##&nbsp;            'glist_count' => '0',
##&nbsp;            'wwn' => '5:006:05ba00:0759fc',
##&nbsp;            'blocks_read' => '0',
##&nbsp;            'blocks_written' => '0',
##&nbsp;            'power_cycle_on_error' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'current_owner' => '135095589',
##&nbsp;            'power_cycle_count' => '0',
##&nbsp;            'scrub_last_done' => '00:00:00',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'dynamically_qualified' => 'No',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = storage show disk -p
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bay' => '1',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0a.17',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '2',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0a.18',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '3',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0a.19',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0a.20',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0a.21',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '6',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0a.22',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '7',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0a.23',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '8',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0a.24',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '9',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0a.25',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '10',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0a.26',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '11',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0a.27',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '12',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0a.28',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '13',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0a.29',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '0',
##&nbsp;            'shelf' => '2',
##&nbsp;            'primary' => '0b.32',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '1',
##&nbsp;            'shelf' => '2',
##&nbsp;            'primary' => '0b.33',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '2',
##&nbsp;            'shelf' => '2',
##&nbsp;            'primary' => '0b.34',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '3',
##&nbsp;            'shelf' => '2',
##&nbsp;            'primary' => '0b.35',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'shelf' => '2',
##&nbsp;            'primary' => '0b.36',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'shelf' => '2',
##&nbsp;            'primary' => '0b.37',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '6',
##&nbsp;            'shelf' => '2',
##&nbsp;            'primary' => '0b.38',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '12',
##&nbsp;            'shelf' => '2',
##&nbsp;            'primary' => '0b.44',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '13',
##&nbsp;            'shelf' => '2',
##&nbsp;            'primary' => '0b.45',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '0',
##&nbsp;            'shelf' => '0',
##&nbsp;            'primary' => '0c.00.0',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '1',
##&nbsp;            'shelf' => '0',
##&nbsp;            'primary' => '0c.00.1',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '2',
##&nbsp;            'shelf' => '0',
##&nbsp;            'primary' => '0c.00.2',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '3',
##&nbsp;            'shelf' => '0',
##&nbsp;            'primary' => '0c.00.3',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'shelf' => '0',
##&nbsp;            'primary' => '0c.00.4',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'shelf' => '0',
##&nbsp;            'primary' => '0c.00.5',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '6',
##&nbsp;            'shelf' => '0',
##&nbsp;            'primary' => '0c.00.6',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '7',
##&nbsp;            'shelf' => '0',
##&nbsp;            'primary' => '0c.00.7',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '8',
##&nbsp;            'shelf' => '0',
##&nbsp;            'primary' => '0c.00.8',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '9',
##&nbsp;            'shelf' => '0',
##&nbsp;            'primary' => '0c.00.9',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '10',
##&nbsp;            'shelf' => '0',
##&nbsp;            'primary' => '0c.00.10',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0d.01.0',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0d.01.4',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0d.01.5',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '7',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0d.01.7',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '9',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0d.01.9',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '10',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0d.01.10',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '14',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0d.01.14',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '16',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0d.01.16',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '17',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0d.01.17',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '19',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0d.01.19',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '20',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0d.01.20',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '21',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0d.01.21',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '22',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0d.01.22',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '23',
##&nbsp;            'shelf' => '1',
##&nbsp;            'primary' => '0d.01.23',
##&nbsp;            'primary_port' => 'B'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = storage show disk -x
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bay' => '1',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.17',
##&nbsp;            'serial' => 'J4V4E2YC',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'model' => 'X278_HVIPB146F15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '2',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.18',
##&nbsp;            'serial' => '3LM05074000097064PU7',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'model' => 'X279_S15K5288F15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '3',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.19',
##&nbsp;            'serial' => 'V3V07YEA',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'model' => 'X272_HPYTA073F10',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.20',
##&nbsp;            'serial' => 'J4V4DYSC',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'model' => 'X278_HVIPB146F15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.21',
##&nbsp;            'serial' => '3KN094CG000075450QXD',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'model' => 'X275_S15K4146F15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '6',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.22',
##&nbsp;            'serial' => 'J8V4NHKC',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'model' => 'X279_HVIPB288F15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '7',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.23',
##&nbsp;            'serial' => '3LM04HKZ00007702JB9Z',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'model' => 'X279_S15K5288F15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '8',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.24',
##&nbsp;            'serial' => 'V5V0BBBA',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'model' => 'X274_HPYTA146F10',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '9',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.25',
##&nbsp;            'serial' => '3SL01Y4W00009005S92U',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'model' => 'X292_S15K7560F15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '10',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.26',
##&nbsp;            'serial' => 'DH07P67000SE',
##&nbsp;            'rev' => 'NA05',
##&nbsp;            'model' => 'X276_FAL9E288F10',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '11',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.27',
##&nbsp;            'serial' => 'J8V3THBC',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'model' => 'X279_HVIPB288F15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '12',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.28',
##&nbsp;            'serial' => 'J8V47PBC',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'model' => 'X279_HVIPB288F15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '13',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.29',
##&nbsp;            'serial' => 'DH07P67000M0',
##&nbsp;            'rev' => 'NA05',
##&nbsp;            'model' => 'X276_FAL9E288F10',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '0',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.32',
##&nbsp;            'serial' => '3QD0NYDM',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY750SSX',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '1',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.33',
##&nbsp;            'serial' => '3QD0NYME',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY750SSX',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '2',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.34',
##&nbsp;            'serial' => 'P6G01DAA',
##&nbsp;            'rev' => 'A90A',
##&nbsp;            'model' => 'X267_HGEMIT50SSX',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '3',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.35',
##&nbsp;            'serial' => 'L639LKCG',
##&nbsp;            'rev' => 'BEGG',
##&nbsp;            'model' => 'X266_MSABR320SSX',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.36',
##&nbsp;            'serial' => '3QD0NZBS',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY750SSX',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.37',
##&nbsp;            'serial' => '3QD0NZCP',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY750SSX',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '6',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.38',
##&nbsp;            'serial' => '3QD0NXF3',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY750SSX',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '12',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.44',
##&nbsp;            'serial' => '3QD0NYCH',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY750SSX',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '13',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.45',
##&nbsp;            'serial' => '3QD0P7HK',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY750SSX',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '0',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.0',
##&nbsp;            'serial' => '3SJ0XYNA00009036EMRH',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '1',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.1',
##&nbsp;            'serial' => '3SJ0YGP600009035RSPQ',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '2',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.2',
##&nbsp;            'serial' => '3SJ102PT00009038U7EC',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '3',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.3',
##&nbsp;            'serial' => '3SJ0XYMT00009036Y7JW',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.4',
##&nbsp;            'serial' => '3SJ1030L00009037FUPP',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.5',
##&nbsp;            'serial' => '3SJ1033A000090380KFL',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '6',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.6',
##&nbsp;            'serial' => '3SJ1035400009037JJP9',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '7',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.7',
##&nbsp;            'serial' => '3SJ1033G00009038U78W',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '8',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.8',
##&nbsp;            'serial' => '3SJ102T9000090380CHC',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '9',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.9',
##&nbsp;            'serial' => '3SJ1033900009038U7P4',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '10',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.10',
##&nbsp;            'serial' => '3SJ0ZRA000009038U7GT',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7288A15',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.0',
##&nbsp;            'serial' => 'WD-WMAUR0067764',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'model' => 'X306_WMANT02TSSM',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.4',
##&nbsp;            'serial' => 'WD-WCAUR0005089',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'model' => 'X306_WMANT02TSSM',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.5',
##&nbsp;            'serial' => 'WD-WMAUR0073515',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'model' => 'X306_WMANT02TSSM',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '7',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.7',
##&nbsp;            'serial' => 'MH043X4B',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE01TSSM',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '9',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.9',
##&nbsp;            'serial' => 'MH046SAB',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE01TSSM',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '10',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.10',
##&nbsp;            'serial' => 'MH0442MB',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE01TSSM',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '14',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.14',
##&nbsp;            'serial' => 'WD-WCAW30104128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'model' => 'X302_WVULC01TSSA',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '16',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.16',
##&nbsp;            'serial' => 'WD-WCAW30091789',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'model' => 'X302_WVULC01TSSM',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '17',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.17',
##&nbsp;            'serial' => 'WD-WCAW30107688',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'model' => 'X302_WVULC01TSSM',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '19',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.19',
##&nbsp;            'serial' => 'MH1RTH6W',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE01TSSM',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '20',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.20',
##&nbsp;            'serial' => 'ME031SHW',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X310_HSATET50SSM',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '21',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.21',
##&nbsp;            'serial' => 'MH035GJB',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE01TSSM',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '22',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.22',
##&nbsp;            'serial' => 'MH3WH9YK',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE01TSSM',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '23',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.23',
##&nbsp;            'serial' => 'MH0394NB',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE01TSSM',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = storage show disk -T
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bay' => '1',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.17',
##&nbsp;            'serial' => 'J4V4E2YC',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'model' => 'X278_HVIPB',
##&nbsp;            'type' => 'FCAL',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '2',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.18',
##&nbsp;            'serial' => '3LM05074000097064PU7',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'model' => 'X279_S15K5',
##&nbsp;            'type' => 'FCAL',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '3',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.19',
##&nbsp;            'serial' => 'V3V07YEA',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'model' => 'X272_HPYTA',
##&nbsp;            'type' => 'FCAL',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.20',
##&nbsp;            'serial' => 'J4V4DYSC',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'model' => 'X278_HVIPB',
##&nbsp;            'type' => 'FCAL',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.21',
##&nbsp;            'serial' => '3KN094CG000075450QXD',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'model' => 'X275_S15K4',
##&nbsp;            'type' => 'FCAL',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '6',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.22',
##&nbsp;            'serial' => 'J8V4NHKC',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'model' => 'X279_HVIPB',
##&nbsp;            'type' => 'FCAL',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '7',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.23',
##&nbsp;            'serial' => '3LM04HKZ00007702JB9Z',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'model' => 'X279_S15K5',
##&nbsp;            'type' => 'FCAL',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '8',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.24',
##&nbsp;            'serial' => 'V5V0BBBA',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'model' => 'X274_HPYTA',
##&nbsp;            'type' => 'FCAL',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '9',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.25',
##&nbsp;            'serial' => '3SL01Y4W00009005S92U',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'model' => 'X292_S15K7',
##&nbsp;            'type' => 'FCAL',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '10',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.26',
##&nbsp;            'serial' => 'DH07P67000SE',
##&nbsp;            'rev' => 'NA05',
##&nbsp;            'model' => 'X276_FAL9E',
##&nbsp;            'type' => 'FCAL',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '11',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.27',
##&nbsp;            'serial' => 'J8V3THBC',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'model' => 'X279_HVIPB',
##&nbsp;            'type' => 'FCAL',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '12',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.28',
##&nbsp;            'serial' => 'J8V47PBC',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'model' => 'X279_HVIPB',
##&nbsp;            'type' => 'FCAL',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '13',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0a.29',
##&nbsp;            'serial' => 'DH07P67000M0',
##&nbsp;            'rev' => 'NA05',
##&nbsp;            'model' => 'X276_FAL9E',
##&nbsp;            'type' => 'FCAL',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '0',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.32',
##&nbsp;            'serial' => '3QD0NYDM',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY',
##&nbsp;            'type' => 'ATA',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '1',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.33',
##&nbsp;            'serial' => '3QD0NYME',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY',
##&nbsp;            'type' => 'ATA',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '2',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.34',
##&nbsp;            'serial' => 'P6G01DAA',
##&nbsp;            'rev' => 'A90A',
##&nbsp;            'model' => 'X267_HGEMI',
##&nbsp;            'type' => 'ATA',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '3',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.35',
##&nbsp;            'serial' => 'L639LKCG',
##&nbsp;            'rev' => 'BEGG',
##&nbsp;            'model' => 'X266_MSABR',
##&nbsp;            'type' => 'ATA',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.36',
##&nbsp;            'serial' => '3QD0NZBS',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY',
##&nbsp;            'type' => 'ATA',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.37',
##&nbsp;            'serial' => '3QD0NZCP',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY',
##&nbsp;            'type' => 'ATA',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '6',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.38',
##&nbsp;            'serial' => '3QD0NXF3',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY',
##&nbsp;            'type' => 'ATA',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '12',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.44',
##&nbsp;            'serial' => '3QD0NYCH',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY',
##&nbsp;            'type' => 'ATA',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '13',
##&nbsp;            'shelf' => '2',
##&nbsp;            'disk' => '0b.45',
##&nbsp;            'serial' => '3QD0P7HK',
##&nbsp;            'rev' => 'AQNZ',
##&nbsp;            'model' => 'X268_SGLXY',
##&nbsp;            'type' => 'ATA',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '0',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.0',
##&nbsp;            'serial' => '3SJ0XYNA00009036EMRH',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'type' => 'SAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '1',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.1',
##&nbsp;            'serial' => '3SJ0YGP600009035RSPQ',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'type' => 'SAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '2',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.2',
##&nbsp;            'serial' => '3SJ102PT00009038U7EC',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'type' => 'SAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '3',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.3',
##&nbsp;            'serial' => '3SJ0XYMT00009036Y7JW',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'type' => 'SAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.4',
##&nbsp;            'serial' => '3SJ1030L00009037FUPP',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'type' => 'SAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.5',
##&nbsp;            'serial' => '3SJ1033A000090380KFL',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'type' => 'SAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '6',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.6',
##&nbsp;            'serial' => '3SJ1035400009037JJP9',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'type' => 'SAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '7',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.7',
##&nbsp;            'serial' => '3SJ1033G00009038U78W',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'type' => 'SAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '8',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.8',
##&nbsp;            'serial' => '3SJ102T9000090380CHC',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'type' => 'SAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '9',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.9',
##&nbsp;            'serial' => '3SJ1033900009038U7P4',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'type' => 'SAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '10',
##&nbsp;            'shelf' => '0',
##&nbsp;            'disk' => '0c.00.10',
##&nbsp;            'serial' => '3SJ0ZRA000009038U7GT',
##&nbsp;            'rev' => 'NQ07',
##&nbsp;            'model' => 'X287_S15K7',
##&nbsp;            'type' => 'SAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '0',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.0',
##&nbsp;            'serial' => 'WD-WMAUR0067764',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'model' => 'X306_WMANT',
##&nbsp;            'type' => 'BSAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.4',
##&nbsp;            'serial' => 'WD-WCAUR0005089',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'model' => 'X306_WMANT',
##&nbsp;            'type' => 'BSAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.5',
##&nbsp;            'serial' => 'WD-WMAUR0073515',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'model' => 'X306_WMANT',
##&nbsp;            'type' => 'BSAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '7',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.7',
##&nbsp;            'serial' => 'MH043X4B',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE',
##&nbsp;            'type' => 'BSAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '9',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.9',
##&nbsp;            'serial' => 'MH046SAB',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE',
##&nbsp;            'type' => 'BSAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '10',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.10',
##&nbsp;            'serial' => 'MH0442MB',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE',
##&nbsp;            'type' => 'BSAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '14',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.14',
##&nbsp;            'serial' => 'WD-WCAW30104128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'model' => 'X302_WVULC',
##&nbsp;            'type' => 'BSAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '16',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.16',
##&nbsp;            'serial' => 'WD-WCAW30091789',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'model' => 'X302_WVULC',
##&nbsp;            'type' => 'BSAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '17',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.17',
##&nbsp;            'serial' => 'WD-WCAW30107688',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'model' => 'X302_WVULC',
##&nbsp;            'type' => 'BSAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '19',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.19',
##&nbsp;            'serial' => 'MH1RTH6W',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE',
##&nbsp;            'type' => 'BSAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '20',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.20',
##&nbsp;            'serial' => 'ME031SHW',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X310_HSATE',
##&nbsp;            'type' => 'BSAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '21',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.21',
##&nbsp;            'serial' => 'MH035GJB',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE',
##&nbsp;            'type' => 'BSAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '22',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.22',
##&nbsp;            'serial' => 'MH3WH9YK',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE',
##&nbsp;            'type' => 'BSAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '23',
##&nbsp;            'shelf' => '1',
##&nbsp;            'disk' => '0d.01.23',
##&nbsp;            'serial' => 'MH0394NB',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'model' => 'X302_HSATE',
##&nbsp;            'type' => 'BSAS',
##&nbsp;            'vendor' => 'NETAPP'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub storage_show_disk {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my ( $pos1, $j, $k ) = ( 0, 0, 0 );
    my ( @result, @headings, $pos2, $string, $string1, $string2 );

    my $error = _check_error(
        raw_output    => $output,
        command       => "storage show disk",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #    my $first_line = "Slot";
    my @lines = get_lines(
        raw_output      => $output,
        can_throw       => 1,
        response_object => $resp
    );

    #In the actual hash for all the keys spaces will be replaced with '-'
    my @possible_keys = (
        "Disk",
        "Shelf",
        "Bay",
        "Serial",
        "Vendor",
        "Model",
        "Rev",
        "RPM",
        "WWN",
        "UID",
        "Downrev",
        "Pri Port",
        "Sec Name",
        "Sec Port",
        "Power-on Hours",
        "Blocks read",
        "Blocks written",
        "Time interval",
        "Glist count",
        "Scrub last done",
        "Scrub count",
        "LIP count",
        "Dynamically qualified",
        "Power cycle count",
        "Power cycle on error",
        "Current owner",
        "Home owner",
        "Reservation owner"
    );
    my @result_array;
    if ($command =~ /(-x$|-x\s+-T$)/) {
       foreach my $line (@lines) {
           if ( $line =~/(DISK\s+)/
              ) {
                @headings = split( /\s+/, $line );
                $string1 = "$line"." ";
           } elsif ( $line =~ /(\S+)\s+(\S+).*/ && $line !~ /-----/ ) {
                $string2 = "$line"." ";
                my @total_elements;
                @total_elements = split('\s+',$line);
                if (scalar(@headings) != scalar(@total_elements)) {
                    $string=$string1;
                } else {
                    $string=$string2;
                }
                while ( $string =~ /\s+/g ) {
                    $pos2 = pos($string);
                    my $len = $pos2 - $pos1;
                    my $val = substr( $line, $pos1, $len );
                    $pos1 = $pos2;
                    trim( 'string_ptr' => \$val );
                    $result_array[$k]->{ to_key_name( 'item' => "$headings[ $j++ ]") } = $val;
                }
                $pos1 = 0;
                $j = 0;
                $k++;
            }
       }
    } else {

        foreach my $line (@lines) {
            my @words = split( /\:\s+/, $line );

            #Check if the line contains any of the above defined possible keys
            my @temp = map( $_ =~ /^$words[0]/ ? $_ : (), @possible_keys );
            if ( $line =~ /Disk\:/ ) {

                #start a new hash for the new disk
                my ( %temp_hash, @temp_array );
                push( @result_array, \%temp_hash );
                my @array = split( /\:\s+/, $line );
                $result_array[$#result_array]->{disk} = $array[1];
            } elsif (@temp) {
                $temp[0] =~ s/(\s+)/_/g;
                my $key = lc( $temp[0] );
                chomp( $words[1] );
                $result_array[$#result_array]->{$key} = $words[1];
            } elsif ( $line =~ /(\S+)\s+\=\s+(\S+)/ ) {
                $result_array[$#result_array]
                    ->{disk_qualification_package_details}->{header_information}
                    ->{$1} = $2;
            } elsif ( $line =~ /Package Date/ ) {
                my @array = split( /\:\s+/, $line );
                $result_array[ $#result_array + 1 ]
                    ->{disk_qualification_package_details}->{package_date}
                    = $array[1];
            } elsif ( $line =~ /DISK\s+/ ) {
                $line =~ s/$line/----/;
            } elsif ( $line
                =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*/
                ) {
                my %temp_hash;
                push( @result_array, \%temp_hash );
                $temp_hash{disk}   = $1;
                $temp_hash{shelf}  = $2;
                $temp_hash{bay}    = $3;
                $temp_hash{serial} = $4;
                $temp_hash{vendor} = $5;
                $temp_hash{model}  = $6;
                $temp_hash{rev}    = $7;
                $temp_hash{type}   = $8;
            } elsif ( $line
                =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*/ ) {
                if ( $line =~ /\---/ )
                {    #do nothing if line contains unexpected format
                } else {
                    my %temp_hash;
                    push( @result_array, \%temp_hash );
                    $temp_hash{disk}   = $1;
                    $temp_hash{shelf}  = $2;
                    $temp_hash{bay}    = $3;
                    $temp_hash{serial} = $4;
                    $temp_hash{vendor} = $5;
                    $temp_hash{model}  = $6;
                    $temp_hash{rev}    = $7;
                }
            } elsif (
                $line =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*/ ) {
                if ( ( $line =~ /\---/ ) || ( $line =~ /PRIMARY\s+/ ) )
                {    #do nothing if line contains unexpected format
                } else {
                    my %temp_hash;
                    push( @result_array, \%temp_hash );
                    $temp_hash{primary}        = $1;
                    $temp_hash{primary_port}   = $2;
                    $temp_hash{secondary}      = $3;
                    $temp_hash{secondary_port} = $4;
                    $temp_hash{shelf}          = $5;
                    $temp_hash{bay}            = $6;
                }
            } elsif ( $line =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*/ ) {
                if ( ( $line =~ /\---/ ) || ( $line =~ /Disk\s+/ ) )
                {    #do nothing if line contains unexpected format
                } else {
                    my %temp_hash;
                    push( @result_array, \%temp_hash );
                    $temp_hash{primary}      = $1;
                    $temp_hash{primary_port} = $2;
                    $temp_hash{shelf}        = $3;
                    $temp_hash{bay}          = $4;
                }
            }
        }
    }
    return \@result_array;
} ## end sub storage_show_disk
## @name ::storage_load_balance
##
## @summary Parses the raw output from the "storage load balance" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from secureadmin_status with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_load_balance(%args);
## INPUTS TO METHOD CALL 'storage_load_balance' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub storage_load_balance {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $lin;
    my @result_array;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];

} ## end sub storage_load_balance
1;

## @name ::storage_array_show
##
## @summary Parses the raw output from the "storage array show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_array_show with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_array_show(%args);
## INPUTS TO METHOD CALL 'storage_array_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <td> No switch for this key </td> <td> 'array-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; storage array show; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'model' => '2810XIV',
##&nbsp;            'name' => 'IBM_2810XIV_1',
##&nbsp;            'vendor' => 'IBM'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub storage_array_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'storage array show',
        "ptr_error_message" => \$error_message,
    );
    my @result;
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $first_line = "Prefix.*";
    my @lines      = get_lines(
        raw_output      => $raw_output,
        can_throw       => 1,
        response_object => $resp,
        'first_line'    => $first_line,
    );
    my $i = 0;
    shift @lines;
    foreach my $line (@lines) {
        if ( $line =~ /(\S+)\s+(\S+)\s+(\S+)/ ) {
            $result[$i]->{'name'}   = $1;
            $result[$i]->{'vendor'} = $2;
            $result[$i]->{'model'}  = $3;
            $i++;
        }
    }
    return \@result;
} ## end sub storage_array_show

## @name ::storage_array_show_config
##
## @summary Parses the raw output from the "storage array show-config" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_array_show_config with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_array_show_config(%args);
## INPUTS TO METHOD CALL 'storage_array_show_config' ON APISET OBJECT
## <table border = "1" >
## <tr> <td> No switch for this key </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; storage array show-config; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'initiator' => [
##&nbsp;                             '6a',
##&nbsp;                             '5a'
##&nbsp;                           ],
##&nbsp;            'switch_port' => [
##&nbsp;                               'vgci9124s38:1-14',
##&nbsp;                               'vgbr200es11:14'
##&nbsp;                             ],
##&nbsp;            'array_target_ports' => [
##&nbsp;                                      '50:01:73:80:01:0a:01:73',
##&nbsp;                                      '50:01:73:80:01:0a:01:50'
##&nbsp;                                    ],
##&nbsp;            'lun_count' => '103',
##&nbsp;            'lun_group' => 'Group 0',
##&nbsp;            'array_name' => 'IBM_2810XIV_1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub storage_array_show_config {
    my %args = @_;
    my $error_message;
    my @result;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'storage array show-config',
        "ptr_error_message" => \$error_message,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $first_line = "LUN.*";
    my @lines      = get_lines(
        raw_output      => $raw_output,
        can_throw       => 1,
        response_object => $resp,
        'first_line'    => $first_line,
    );
    shift @lines;
    my $i = -1;
    my $j;
    foreach my $line (@lines) {
        if ( $line
            =~ /((\w+\s+\w+)?\s+\((\w+)\s+\w+\))\s+(\w+)\s+(\S+)\s+(\S+)\s+(\w+)/
            ) {
            $j = 0;
            $i++;
            $result[$i]->{'lun_group'}                = $2 if defined($2);
            $result[$i]->{'lun_count'}                = $3;
            $result[$i]->{'array_name'}               = $4;
            $result[$i]->{'array_target_ports'}->[$j] = $5;
            $result[$i]->{'switch_port'}->[$j]        = $6;
            $result[$i]->{'initiator'}->[$j]          = $7;

        } elsif ( $line =~ /(\S+)\s+(\S+)\s+(\w+)/ ) {
            $j++;
            $result[$i]->{'array_target_ports'}->[$j] = $1;
            $result[$i]->{'switch_port'}->[$j]        = $2;
            $result[$i]->{'initiator'}->[$j]          = $3;
        }
    }
    return \@result;
} ## end sub storage_array_show_config

## @name ::storage_array_remove
##
## @summary Parses the raw output from the "storage array remove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_array_remove with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_array_remove(%args);
## INPUTS TO METHOD CALL 'storage_array_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <td> No switch for this key </td> <td> 'array-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; storage array remove; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [ ];
##&nbsp;

sub storage_array_remove {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'storage array remove',
        "ptr_error_message" => \$error_message,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @result_array = ();
    return \@result_array;
} ## end sub storage_array_remove

## @name ::storage_array_purge_database
##
## @summary Parses the raw output from the "storage array purge-database" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_array_purge_database with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_array_purge_database(%args);
## INPUTS TO METHOD CALL 'storage_array_purge_database' ON APISET OBJECT
## <table border = "1" >
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; storage array purge-database; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [ ];
##&nbsp;

sub storage_array_purge_database {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'storage array purge-database',
        "ptr_error_message" => \$error_message,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @result_array = ();
    return \@result_array;
} ## end sub storage_array_purge_database

## @name ::storage_array_remove_port
##
## @summary Parses the raw output from the "storage array remove-port" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_array_remove_port with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_array_remove_port(%args);
## INPUTS TO METHOD CALL 'storage_array_remove_port' ON APISET OBJECT
## <table border = "1" >
## <tr> <td> No switch for this key </td> <td> 'array-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'wwpn-path' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'wwpn-num' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; storage array remove-port ; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [ ];
##&nbsp;

sub storage_array_remove_port {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'storage array remove-port',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @result_array = ();
    return \@result_array;
} ## end sub storage_array_remove_port

## @name ::storage_array_show_luns
##
## @summary Parses the raw output from the "storage array show-ports" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_array_show_luns with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_array_show_luns(%args);
## INPUTS TO METHOD CALL 'storage_array_show_luns' ON APISET OBJECT
## <table border = "1" >
## <tr> <td> No switch for this key </td> <td> 'array-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'all' </td> </tr>
## <tr> <td> '-p' </td> <td> 'path' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'wwpn-num' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; storage array show-luns; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'wwpns' => [
##&nbsp;                         '50:01:73:80:01:0a:01:73',
##&nbsp;                         '50:01:73:80:01:0a:01:50'
##&nbsp;                       ],
##&nbsp;            'name' => 'vgci9124s38:1-14.126L1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wwpns' => [
##&nbsp;                         '50:01:73:80:01:0a:01:50',
##&nbsp;                         '50:01:73:80:01:0a:01:73'
##&nbsp;                       ],
##&nbsp;            'name' => 'vgbr200es11:14.126L2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wwpns' => [
##&nbsp;                         '50:01:73:80:01:0a:01:73',
##&nbsp;                         '50:01:73:80:01:0a:01:50'
##&nbsp;                       ],
##&nbsp;            'name' => 'vgci9124s38:1-14.126L3'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wwpns' => [
##&nbsp;                         '50:01:73:80:01:0a:01:50',
##&nbsp;                         '50:01:73:80:01:0a:01:73'
##&nbsp;                       ],
##&nbsp;            'name' => 'vgbr200es11:14.126L4'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wwpns' => [
##&nbsp;                         '50:01:73:80:01:0a:01:73',
##&nbsp;                         '50:01:73:80:01:0a:01:50'
##&nbsp;                       ],
##&nbsp;            'name' => 'vgci9124s38:1-14.126L5'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wwpns' => [
##&nbsp;                         '50:01:73:80:01:0a:01:73',
##&nbsp;                         '50:01:73:80:01:0a:01:50'
##&nbsp;                       ],
##&nbsp;            'name' => 'vgci9124s38:1-14.126L6'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wwpns' => [
##&nbsp;                         '50:01:73:80:01:0a:01:50',
##&nbsp;                         '50:01:73:80:01:0a:01:73'
##&nbsp;                       ],
##&nbsp;            'name' => 'vgbr200es11:14.126L7'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wwpns' => [
##&nbsp;                         '50:01:73:80:01:0a:01:50',
##&nbsp;                         '50:01:73:80:01:0a:01:73'
##&nbsp;                       ],
##&nbsp;            'name' => 'vgbr200es11:14.126L8'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wwpns' => [
##&nbsp;                         '50:01:73:80:01:0a:01:73',
##&nbsp;                         '50:01:73:80:01:0a:01:50'
##&nbsp;                       ],
##&nbsp;            'name' => 'vgci9124s38:1-14.126L9'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wwpns' => [
##&nbsp;                         '50:01:73:80:01:0a:01:73',
##&nbsp;                         '50:01:73:80:01:0a:01:50'
##&nbsp;                       ],
##&nbsp;            'name' => 'vgci9124s38:1-14.126L10'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wwpns' => [
##&nbsp;                         '50:01:73:80:01:0a:01:50',
##&nbsp;                         '50:01:73:80:01:0a:01:73'
##&nbsp;                       ],
##&nbsp;            'name' => 'vgbr200es11:14.126L11'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wwpns' => [
##&nbsp;                         '50:01:73:80:01:0a:01:50',
##&nbsp;                         '50:01:73:80:01:0a:01:73'
##&nbsp;                       ],
##&nbsp;            'name' => 'vgbr200es11:14.126L12'
##&nbsp;          },
##&nbsp;         ];
##&nbsp;

sub storage_array_show_luns {
    my %args = @_;
    my $error_message;
    my @result;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'storage array show-luns',
        "ptr_error_message" => \$error_message,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $first_line = "Name.*";
    my @lines      = get_lines(
        raw_output      => $raw_output,
        can_throw       => 1,
        response_object => $resp,
        'first_line'    => $first_line,
    );
    my $i = 0;
    shift @lines;
    foreach my $line (@lines) {
        if ( $line =~ /(\S+)\s+(.*)/ ) {
            $result[$i]->{'name'} = $1;
            my @wwpns = split( ',\s+', $2 );
            $result[$i]->{'wwpns'} = \@wwpns;
            $i++;
        }
    }
    return \@result;
} ## end sub storage_array_show_luns

## @name ::storage_array_show_ports
##
## @summary Parses the raw output from the "storage array show-ports" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_array_show_ports with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_array_show_ports(%args);
## INPUTS TO METHOD CALL 'storage_array_show_ports' ON APISET OBJECT
## <table border = "1" >
## <tr> <td> No switch for this key </td> <td> 'array-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; storage array show-ports; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'wwnn' => '50017380010a0000',
##&nbsp;            'wwpn' => '50017380010a0173',
##&nbsp;            'array_name' => 'IBM_2810XIV_1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wwnn' => '50017380010a0000',
##&nbsp;            'wwpn' => '50017380010a0150',
##&nbsp;            'array_name' => 'IBM_2810XIV_1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub storage_array_show_ports {
    my %args = @_;
    my $error_message;
    my @result;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'storage array show-ports',
        "ptr_error_message" => \$error_message,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $first_line = "Array Name.*";
    my @lines      = get_lines(
        raw_output      => $raw_output,
        can_throw       => 1,
        response_object => $resp,
        'first_line'    => $first_line,
    );
    my $i = 0;
    shift @lines;
    foreach my $line (@lines) {
        if ( $line =~ /(\w+)\s+(\w+)\s+(\w+)/ ) {
            $result[$i]->{'array_name'} = $1;
            $result[$i]->{'wwpn'}       = $2;
            $result[$i]->{'wwnn'}       = $3;
            $i++;
        }
    }
    return \@result;
} ## end sub storage_array_show_ports
## @name ::storage_show_qualtable
##
## @summary Parses the raw output from the "storage show qualtable" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from storage_show_qualtable with any of its options as described in the 
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_show_qualtable(%args);
## INPUTS TO METHOD CALL 'storage_show_qualtable' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vendor-id' </td> <td> '0' </td> </tr> 
## <tr> <td> '-x' </td> <td> 'specific' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'product-id' </td> <td> '0' </td> </tr> 
## <tr> <td> '-k' </td> <td> 'dynamic-show' </td> <td> '1' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'static-show' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; storage show qualtable -x TMS RamSan; result -l
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'no_of_entries_in_table' => '582',
##&nbsp;            'legend' => [
##&nbsp;                          {
##&nbsp;                            'RPM             ' => 'Rotational speed',
##&nbsp;                            'Chk             ' => 'Perioidic firmware revision check (On/Off)',
##&nbsp;                            'Qual Flags      ' => 'Qualifier flags (in hex)',
##&nbsp;                            'Rev             ' => 'Recommended firmware revision',
##&nbsp;                            'SSiz            ' => 'Sector Size (in bytes per sector)',
##&nbsp;                            'LastBlock       ' => 'Reported size (in sectors)',
##&nbsp;                            'ZEst            ' => 'Zero estimate time (in seconds)',
##&nbsp;                            'VSiz            ' => 'Verify Size. IO block size for the background scrub operation',
##&nbsp;                            'LastBlockAZCS   ' => 'Reported size (in sectors)',
##&nbsp;                            'Vendor          ' => 'Vendor Id (aliased or native)',
##&nbsp;                            'Product         ' => 'Product Id (aliased or native)'
##&nbsp;                          }
##&nbsp;                        ],
##&nbsp;            'table_name' => 'Dynamic Drive Qualifier table',
##&nbsp;            'matching_entries_found' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_UNSUPPORTED_ARRAY'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'TMS',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'RamSan',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; storage show qualtable -x; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'no_of_entries_in_table' => '582',
##&nbsp;            'legend' => [
##&nbsp;                          {
##&nbsp;                            'RPM             ' => 'Rotational speed',
##&nbsp;                            'Chk             ' => 'Perioidic firmware revision check (On/Off)',
##&nbsp;                            'Qual Flags      ' => 'Qualifier flags (in hex)',
##&nbsp;                            'Rev             ' => 'Recommended firmware revision',
##&nbsp;                            'SSiz            ' => 'Sector Size (in bytes per sector)',
##&nbsp;                            'LastBlock       ' => 'Reported size (in sectors)',
##&nbsp;                            'ZEst            ' => 'Zero estimate time (in seconds)',
##&nbsp;                            'VSiz            ' => 'Verify Size. IO block size for the background scrub operation',
##&nbsp;                            'LastBlockAZCS   ' => 'Reported size (in sectors)',
##&nbsp;                            'Vendor          ' => 'Vendor Id (aliased or native)',
##&nbsp;                            'Product         ' => 'Product Id (aliased or native)'
##&nbsp;                          }
##&nbsp;                        ],
##&nbsp;            'table_name' => 'Dynamic Drive Qualifier table',
##&nbsp;            'matching_entries_found' => '582'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => 'None',
##&nbsp;            'last_block' => 'ONE_GIG',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'DYNAMIC',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => 'None',
##&nbsp;            'last_block' => 'ONE_GIG',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HitzCo',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '1234',
##&nbsp;            'product' => 'HitzDisk',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '74751',
##&nbsp;            'qual_flags' => '802000020060',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-16MB',
##&nbsp;            'lastblockazcs' => '74751'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '112127',
##&nbsp;            'qual_flags' => '802000020060',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-35MB',
##&nbsp;            'lastblockazcs' => '112127'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '144383',
##&nbsp;            'qual_flags' => '802000020060',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-50MB',
##&nbsp;            'lastblockazcs' => '144383'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '246783',
##&nbsp;            'qual_flags' => '802000020060',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-100MB',
##&nbsp;            'lastblockazcs' => '246783'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '1065983',
##&nbsp;            'qual_flags' => '802000020060',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-500MB',
##&nbsp;            'lastblockazcs' => '1065983'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '2089983',
##&nbsp;            'qual_flags' => '802000020060',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-1000MB',
##&nbsp;            'lastblockazcs' => '2089983'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '74751',
##&nbsp;            'qual_flags' => '802000020061',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-16MB-FZ',
##&nbsp;            'lastblockazcs' => '74751'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '112127',
##&nbsp;            'qual_flags' => '802000020061',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-35MB-FZ',
##&nbsp;            'lastblockazcs' => '112127'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '144383',
##&nbsp;            'qual_flags' => '802000020061',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-50MB-FZ',
##&nbsp;            'lastblockazcs' => '144383'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '246783',
##&nbsp;            'qual_flags' => '802000020061',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-100MB-FZ',
##&nbsp;            'lastblockazcs' => '246783'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '1065983',
##&nbsp;            'qual_flags' => '802000020061',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-500MB-FZ',
##&nbsp;            'lastblockazcs' => '1065983'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '2089983',
##&nbsp;            'qual_flags' => '802000020061',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-1000MB-FZ',
##&nbsp;            'lastblockazcs' => '2089983'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '84096',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-16MB-FZ-ATA',
##&nbsp;            'lastblockazcs' => '84096'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '126144',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-35MB-FZ-ATA',
##&nbsp;            'lastblockazcs' => '126144'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '162432',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-50MB-FZ-ATA',
##&nbsp;            'lastblockazcs' => '162432'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '277632',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-100MB-FZ-ATA',
##&nbsp;            'lastblockazcs' => '277632'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '1199232',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-500MB-FZ-ATA',
##&nbsp;            'lastblockazcs' => '1199232'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '2351232',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-1000MB-FZ-ATA',
##&nbsp;            'lastblockazcs' => '2351232'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '4655232',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-2000MB-FZ-ATA',
##&nbsp;            'lastblockazcs' => '4655232'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '9263232',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-4000MB-FZ-ATA',
##&nbsp;            'lastblockazcs' => '9263232'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '20783232',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-9000MB-FZ-ATA',
##&nbsp;            'lastblockazcs' => '20783232'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_DISK_SUPPORTS_BCS',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '74751',
##&nbsp;            'qual_flags' => '2000020070',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-16MB-520',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_DISK_SUPPORTS_BCS',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '112127',
##&nbsp;            'qual_flags' => '2000020070',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-35MB-520',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_DISK_SUPPORTS_BCS',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '144383',
##&nbsp;            'qual_flags' => '2000020070',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-50MB-520',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_DISK_SUPPORTS_BCS',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '246783',
##&nbsp;            'qual_flags' => '2000020070',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-100MB-520',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_DISK_SUPPORTS_BCS',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '1065983',
##&nbsp;            'qual_flags' => '2000020070',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-500MB-520',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_DISK_SUPPORTS_BCS',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '2089983',
##&nbsp;            'qual_flags' => '2000020070',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-1000MB-520',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_DISK_SUPPORTS_BCS',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '74751',
##&nbsp;            'qual_flags' => '2000020071',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-16MB-FZ-520',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_DISK_SUPPORTS_BCS',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '112127',
##&nbsp;            'qual_flags' => '2000020071',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-35MB-FZ-520',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_DISK_SUPPORTS_BCS',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '144383',
##&nbsp;            'qual_flags' => '2000020071',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-50MB-FZ-520',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_DISK_SUPPORTS_BCS',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '246783',
##&nbsp;            'qual_flags' => '2000020071',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-100MB-FZ-520',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_DISK_SUPPORTS_BCS',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '1065983',
##&nbsp;            'qual_flags' => '2000020071',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-500MB-FZ-520',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_DISK_SUPPORTS_BCS',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '2089983',
##&nbsp;            'qual_flags' => '2000020071',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-1000MB-FZ-520',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_DISK_SUPPORTS_BCS',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '4137983',
##&nbsp;            'qual_flags' => '2000020071',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-2000MB-FZ-520',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_DISK_SUPPORTS_BCS',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '8233983',
##&nbsp;            'qual_flags' => '2000020071',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-4000MB-FZ-520',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '246783',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-100MB-SS-512',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_DISK_SUPPORTS_BCS',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SSD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '1065983',
##&nbsp;            'qual_flags' => '42000020071',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-500MB-SS-520',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_DISK_SUPPORTS_BCS',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_VSIM'
##&nbsp;                       ],
##&nbsp;            'last_block' => '18473983',
##&nbsp;            'qual_flags' => '2000020071',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0042',
##&nbsp;            'product' => 'VD-9000MB-FZ-520',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_MULTIPATH_CAPABLE'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG32',
##&nbsp;            'qual_flags' => '40',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'TMS',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '1.4d',
##&nbsp;            'product' => 'FC65',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_MULTIPATH_CAPABLE'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'FOUR_GIG',
##&nbsp;            'qual_flags' => '40',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'Curtis',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '2.02',
##&nbsp;            'product' => 'Nitro!FC',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_MULTIPATH_CAPABLE'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG124',
##&nbsp;            'qual_flags' => '40',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'Exavio',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '2.0',
##&nbsp;            'product' => 'ExaFCT/ExaMax',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG17',
##&nbsp;            'qual_flags' => '220061',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '900',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA42',
##&nbsp;            'product' => 'X233_ST318304FC',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG34',
##&nbsp;            'qual_flags' => '220061',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA42',
##&nbsp;            'product' => 'X234_ST336704FC',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG34',
##&nbsp;            'qual_flags' => '220461',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '890',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA09',
##&nbsp;            'product' => 'X234_SCHT5036F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68',
##&nbsp;            'qual_flags' => '220461',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1800',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA09',
##&nbsp;            'product' => 'X235_SCHT5073F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG34',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '600',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA16',
##&nbsp;            'product' => 'X234_SCHT6036F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1380',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA16',
##&nbsp;            'product' => 'X235_SCHT6073F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2700',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA16',
##&nbsp;            'product' => 'X236_SCHT6146F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG34',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '600',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA16',
##&nbsp;            'product' => 'X270_SCHT6036F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1380',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA16',
##&nbsp;            'product' => 'X272_SCHT6073F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2700',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA16',
##&nbsp;            'product' => 'X274_SCHT6146F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1200',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X235_S10K7073F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2400',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X236_S10K7146F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4800',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X237_S10K7288F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1200',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA07',
##&nbsp;            'product' => 'X272_S10K7073F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2400',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA07',
##&nbsp;            'product' => 'X274_S10K7146F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4800',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA07',
##&nbsp;            'product' => 'X276_S10K7288F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG372',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6100',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X288_STLRC400F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4300',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X276_STLRC288F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG34',
##&nbsp;            'qual_flags' => '8461',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '600',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA08',
##&nbsp;            'product' => 'X271_S15K3036F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68',
##&nbsp;            'qual_flags' => '8461',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1200',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA08',
##&nbsp;            'product' => 'X273_S15K3073F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG34',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '600',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'product' => 'X271_S15K4036F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1200',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'product' => 'X273_S15K4073F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '600',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'product' => 'X275_S15K4146F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1200',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X277_S15K4073F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1600',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X278_S15K4146F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1260',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'product' => 'X277_S15K5073F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2580',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'product' => 'X278_S15K5146F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5220',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'product' => 'X279_S15K5288F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2580',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X278_S15K6146F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5220',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X279_S15K6288F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5220',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X279_S15K7288F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '7830',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X291_S15K6420F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '7830',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X291_S15K7420F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X292_S15K7560F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '7830',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X295_S15K7420F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X293_S15K7560F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1260',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X277_HVIPB073F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2580',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X278_HVIPB146F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5220',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X279_HVIPB288F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5220',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X279_HVPBP288F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5220',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X279_HVIPC288F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '7830',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X291_HVPBP420F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '7830',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X291_HVIPC420F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X292_HVIPC560F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '7830',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X295_HVIPC420F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X293_HVIPC560F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1260',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X277_FAL10073F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2580',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X278_FAL10146F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5220',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X279_FAL10288F15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG274_4_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'Maxtor',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K5',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'RATU',
##&nbsp;            'product' => '4A320J0       SX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_UID_ALIASED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG274_4_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K5',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'RATU',
##&nbsp;            'product' => 'X266_MTOMC320PSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_UID_ALIASED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG211_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '5340',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'Y4BW',
##&nbsp;            'product' => 'X262_MCALY250PSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_UID_ALIASED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '9120',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X267_MGRIZ500SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_UID_ALIASED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG274_4_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X266_MSABR320SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_UID_ALIASED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG211_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '4800',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X262_MSABR250SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_UID_ALIASED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG211_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '4800',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X262_STONK250SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_UID_ALIASED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '10500',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X267_STONK500SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_UID_ALIASED',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '10620',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X267_HKURO500SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '12420',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X268_SGLXY750SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '8490',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X267_SGLXY500SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG274_4_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X266_SGLXY320SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG211_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '4110',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X262_SGLXY250SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '12420',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X268_WJUPI750SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '8490',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X267_WTORN500SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X269_WMARS01TSSX',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '11340',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X268_WMARST75SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X269_HGEMI01TSSX',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '11340',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X268_HGEMIT75SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7080',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X267_HGEMIT50SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '16800',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'product' => 'X269_SMOOS01TSSX',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '12420',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'product' => 'X268_SMOOST75SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '8490',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'product' => 'X267_SMOOST50SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG211_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '4100',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'product' => 'X262_SMOOST25SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X269_HSATE01TSSX',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '11340',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X268_HSATET75SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7080',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X267_HSATET50SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X294_WMANT02TSSX',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X294_SMUSK02TSSX',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X269_SMUSK01TSSX',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7080',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X267_SMUSKT50SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X294_SMNRY02TSSX',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X294_SMSKP02TSSX',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X269_SMSKP01TSSX',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7080',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X267_SMSKPT50SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X294_HJUPI02TSSX',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X269_HJUPI01TSSX',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '9120',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X267_HJUPIT50SSX',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X294_HMARK02TSSX',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X294_TMUST02TSSX',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X269_TFALC01TSSX',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X294_TFALC02TSSX',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X269_HMARU01TSSX',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X294_HMARU02TSSX',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X294_HMRKP02TSSX',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X294_WKOJN02TSSX',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X269_SMEGA01TSSX',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X294_SMEGA02TSSX',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X269_THARR01TSSX',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X294_THARR02TSSX',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG211_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '4800',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'APHZ',
##&nbsp;            'product' => 'X280_STONK250SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '10500',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X282_STONK500SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '10620',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'AGA3',
##&nbsp;            'product' => 'X282_HKURO500SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_WRITESAME_MODE_MAXTOR'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '9120',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '1DD0',
##&nbsp;            'product' => 'X282_MGRIZ500SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '12420',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X283_SGLXY750SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '8490',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X282_SGLXY500SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG211_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '4110',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X280_SGLXY250SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '12420',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X283_WJUPI750SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '8490',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X282_WTORN500SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X298_WMARS01TSSS',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '11340',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X283_WMARST75SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X298_HGEMI01TSSS',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '11340',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X283_HGEMIT75SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7080',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X282_HGEMIT50SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '16800',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'product' => 'X298_SMOOS01TSSS',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '12420',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'product' => 'X283_SMOOST75SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '8490',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'product' => 'X282_SMOOST50SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG211_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '4100',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'product' => 'X280_SMOOST25SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X298_HSATE01TSSS',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '11340',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X283_HSATET75SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7080',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X282_HSATET50SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X299_WMANT02TSSS',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X299_SMUSK02TSSS',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X298_SMUSK01TSSS',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7080',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X282_SMUSKT50SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X299_SMNRY02TSSS',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X299_SMSKP02TSSS',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X298_SMSKP01TSSS',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7080',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X282_SMSKPT50SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X299_HJUPI02TSSS',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X298_HJUPI01TSSS',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '9120',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X282_HJUPIT50SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X298_WVULC01TSSS',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7650',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X282_WSUMMT50SSS',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X299_HMARK02TSSS',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X299_TMUST02TSSS',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X298_TFALC01TSSS',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X299_TFALC02TSSS',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X298_HMARU01TSSS',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X299_HMARU02TSSS',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X299_HMRKP02TSSS',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X299_WKOJN02TSSS',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X298_SMEGA01TSSS',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X299_SMEGA02TSSS',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X298_THARR01TSSS',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_SCT',
##&nbsp;                         'QD_WRITESAME_MODE_SCT',
##&nbsp;                         'QD_SEGMENT_FW'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X299_THARR02TSSS',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_FSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0006',
##&nbsp;            'product' => 'X999_2TFSAS__512',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_FSAS',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '39800',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0721',
##&nbsp;            'product' => 'X999_3TFSAS__512',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '10620',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X310_HKURO500SSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA07',
##&nbsp;            'product' => 'X302_WMARS01TSSM',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '11340',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X311_WMARST75SSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '16800',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'product' => 'X302_SMOOS01TSSM',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '12420',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X311_SMOOST75SSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '8490',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'product' => 'X310_SMOOST50SSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'product' => 'X302_HSATE01TSSM',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '11340',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X311_HSATET75SSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7080',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA06',
##&nbsp;            'product' => 'X310_HSATET50SSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7080',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X310_HGEMIT50SSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '11340',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X311_HGEMIT75SSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_HGEMI01TSSM',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG34_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '1000',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X999_STEC50SSDSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'product' => 'X306_WMANT02TSSM',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_SMUSK02TSSM',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'ST330006',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '45900',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0205',
##&nbsp;            'product' => '50NS          SM',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG225_4K',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '4160',
##&nbsp;            'zest' => '4050',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '4002',
##&nbsp;            'product' => 'X999_S15K7C_4160',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG450_4K',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '4160',
##&nbsp;            'zest' => '8100',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '4002',
##&nbsp;            'product' => 'X999_S15K7__4160',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG115_4K',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '4096',
##&nbsp;            'zest' => '2700',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '1028',
##&nbsp;            'product' => 'X999_S15K6__4096',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG447_4K',
##&nbsp;            'qual_flags' => '10000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '4096',
##&nbsp;            'zest' => '8100',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '4321',
##&nbsp;            'product' => 'X999_950062_4096',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_SMUSK01TSSM',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7080',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X310_SMUSKT50SSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_SMSKP02TSSM',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_SMSKP01TSSM',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7080',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X310_SMSKPT50SSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_HJUPI02TSSM',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_HJUPI01TSSM',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '9120',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X310_HJUPIT50SSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG100_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '1000',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'product' => 'X442_SAM08100ASM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_SANITIZE_CMD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG100_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '625',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X442_SAM16100ASM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_SANITIZE_CMD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG90',
##&nbsp;            'qual_flags' => '2081000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '625',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X442_SAM16090BSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_SANITIZE_CMD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG100',
##&nbsp;            'qual_flags' => '2081000220041',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '625',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X441_SAM16100BSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_SANITIZE_CMD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG110',
##&nbsp;            'qual_flags' => '2081000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '688',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X445_SAM16110BSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_SANITIZE_CMD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG200',
##&nbsp;            'qual_flags' => '2081000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1250',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X444_SAM16200BSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_SANITIZE_CMD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG100',
##&nbsp;            'qual_flags' => '2081000220041',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '625',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X441_SS161100BSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_SANITIZE_CMD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG200',
##&nbsp;            'qual_flags' => '2081000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1250',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X444_SS161200BSM',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_WVULC01TSSM',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_HMARK02TSSM',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X308_HMARK03TSSM',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X308_WMOZA03TSSM',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_SMNRY02TSSM',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X308_SMNRY03TSSM',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_TMUST02TSSM',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_TFALC01TSSM',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_TFALC02TSSM',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X308_TFALC03TSSM',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_HMARU01TSSM',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_HMARU02TSSM',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_HMRKP02TSSM',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X308_HMRKP03TSSM',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X314_HMKPN04TSSM',
##&nbsp;            'lastblockazcs' => 'GIG3808'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X314_HMKPE04TSSM',
##&nbsp;            'lastblockazcs' => 'GIG3808'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_WKOJN02TSSM',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X308_WKOJN03TSSM',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X314_WKOJN04TSSM',
##&nbsp;            'lastblockazcs' => 'GIG3808'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_SMEGA01TSSM',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_SMEGA02TSSM',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X308_SMEGA03TSSM',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X314_SMEGA04TSSM',
##&nbsp;            'lastblockazcs' => 'GIG3808'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_THARR01TSSM',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_THARR02TSSM',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X308_THARR03TSSM',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X314_THARR04TSSM',
##&nbsp;            'lastblockazcs' => 'GIG3808'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '10620',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X310_HKURO500SSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_WMARS01TSSA',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '11340',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X311_WMARST75SSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '16800',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_SMOOS01TSSA',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '12420',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X311_SMOOST75SSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '8490',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X310_SMOOST50SSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_HSATE01TSSA',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '11340',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X311_HSATET75SSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7080',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X310_HSATET50SSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7080',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X310_HGEMIT50SSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG635_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '11340',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X311_HGEMIT75SSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_HGEMI01TSSA',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_WMANT02TSSA',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_SMUSK02TSSA',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'ST330006',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '45900',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0205',
##&nbsp;            'product' => '50NS          SA',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_SMUSK01TSSA',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7080',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X310_SMUSKT50SSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_SMSKP02TSSA',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_SMSKP01TSSA',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '7080',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X310_SMSKPT50SSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_HJUPI02TSSA',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_HJUPI01TSSA',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG423_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '9120',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X310_HJUPIT50SSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG100_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '1000',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X442_SAM08100ASA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG100_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '625',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X442_SAM16100ASA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG90',
##&nbsp;            'qual_flags' => '81000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '625',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X442_SAM16090BSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG100',
##&nbsp;            'qual_flags' => '81000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '625',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X441_SAM16100BSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG110',
##&nbsp;            'qual_flags' => '81000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '688',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X445_SAM16110BSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG200',
##&nbsp;            'qual_flags' => '81000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1250',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X444_SAM16200BSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG100',
##&nbsp;            'qual_flags' => '81000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '625',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X441_SS161100BSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_BSSD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG200',
##&nbsp;            'qual_flags' => '81000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1250',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X444_SS161200BSA',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_WVULC01TSSA',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_HMARK02TSSA',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X308_HMARK03TSSA',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X308_WMOZA03TSSA',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_SMNRY02TSSA',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X308_SMNRY03TSSA',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_TMUST02TSSA',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_TFALC01TSSA',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_TFALC02TSSA',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X308_TFALC03TSSA',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_HMARU01TSSA',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_HMARU02TSSA',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_HMRKP02TSSA',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X308_HMRKP03TSSA',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X314_HMKPN04TSSA',
##&nbsp;            'lastblockazcs' => 'GIG3808'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X314_HMKPE04TSSA',
##&nbsp;            'lastblockazcs' => 'GIG3808'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_WKOJN02TSSA',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X308_WKOJN03TSSA',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X314_WKOJN04TSSA',
##&nbsp;            'lastblockazcs' => 'GIG3808'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_SMEGA01TSSA',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_SMEGA02TSSA',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X308_SMEGA03TSSA',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X314_SMEGA04TSSA',
##&nbsp;            'lastblockazcs' => 'GIG3808'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X302_THARR01TSSA',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG1695_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X306_THARR02TSSA',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2539_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X308_THARR03TSSA',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X314_THARR04TSSA',
##&nbsp;            'lastblockazcs' => 'GIG3808'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X478_WMOZA03TSDB',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X478_SMNRY03TSDB',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X478_HMARK03TSDB',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X478_TFALC03TSDB',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1810000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X999_HSATE01TSDB',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1810000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X999_HJUPI01TSDB',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1810000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '15300',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X999_SMUSK01TSDB',
##&nbsp;            'lastblockazcs' => 'GIG954_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1810000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X999_HJUPI02TSDB',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1810000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '30600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X999_HMARK02TSDB',
##&nbsp;            'lastblockazcs' => 'GIG1907_AZCS'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X478_HMRKP03TSDB',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X480_HMKPN04TSDB',
##&nbsp;            'lastblockazcs' => 'GIG3808'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X480_HMKPE04TSDB',
##&nbsp;            'lastblockazcs' => 'GIG3808'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X478_WKOJN03TSDB',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X480_WKOJN04TSDB',
##&nbsp;            'lastblockazcs' => 'GIG3808'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X478_SMEGA03TSDB',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X480_SMEGA04TSDB',
##&nbsp;            'lastblockazcs' => 'GIG3808'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '40600',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X478_THARR03TSDB',
##&nbsp;            'lastblockazcs' => 'GIG2856'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_BSAS',
##&nbsp;                         'QD_STSB_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_STRING_IO'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'UNSUPPORTED_SIZE',
##&nbsp;            'qual_flags' => '1811000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X480_THARR04TSDB',
##&nbsp;            'lastblockazcs' => 'GIG3808'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SSD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG150',
##&nbsp;            'qual_flags' => '40000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1000',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X443_PLB15150A01',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SSD',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG150',
##&nbsp;            'qual_flags' => '40000220041',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1000',
##&nbsp;            'rpm' => 'K50',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X443_PLS15150A01',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2856',
##&nbsp;            'qual_flags' => '100000228441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '39800',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X309_SMNRY03TA07',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2856',
##&nbsp;            'qual_flags' => '100000224441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '39800',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X309_HMARK03TA07',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2856',
##&nbsp;            'qual_flags' => '100000224441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '39800',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X309_HMRKP03TA07',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2856',
##&nbsp;            'qual_flags' => '100000228441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '39800',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X309_SMEGA03TA07',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG2856',
##&nbsp;            'qual_flags' => '100000224441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '39800',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X309_WVRDI03TA07',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG3808',
##&nbsp;            'qual_flags' => '100000224441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X315_HMRKP04TA07',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG3808',
##&nbsp;            'qual_flags' => '100000228441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X315_SMEGA04TA07',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG3808',
##&nbsp;            'qual_flags' => '100000224441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X315_WVRDI04TA07',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG3808',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X477_HMRKP04TA07',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG3808',
##&nbsp;            'qual_flags' => '8441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X477_SMEGA04TA07',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG3808',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X477_WVRDI04TA07',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG3808',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '43200',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X477_THARR04TA07',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '45',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2700',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X286_S15K5146A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '45',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5400',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X287_S15K5288A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '45',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2700',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X286_S15K6146A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '45',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5400',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X287_S15K6288A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5400',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X287_S15K7288A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '45',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8100',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X289_S15K6420A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8100',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X289_S15K7420A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8100',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X290_S15K7560A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8100',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X297_S15K7420A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8100',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X296_S15K7560A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X414_S15K7560A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X415_S15K7560A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '100000220441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X415_S15K7560FDE',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '45',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2700',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X286_HVIPB146A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '45',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5400',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X287_HVIPB288A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '45',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5220',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X287_HVPBP288A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5220',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X287_HVIPC288A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '45',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '7830',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X289_HVPBP420A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '7830',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X289_HVIPC420A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X290_HVIPC560A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '7830',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X297_HVIPC420A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X296_HVIPC560A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '45',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5220',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA02',
##&nbsp;            'product' => 'X410_HVPBP288A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5220',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X410_HVIPC288A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '45',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '7830',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X411_HVPBP420A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '7830',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X411_HVIPC420A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X412_HVIPC560A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X414_HVIPC560A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X415_HVIPC560A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '7830',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X304_HVIPC420A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X305_HVIPC560A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '45',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5400',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X410_S15K6288A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5400',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X410_S15K7288A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '45',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8100',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X411_S15K6420A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8100',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X411_S15K7420A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X412_S15K7560A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8100',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X304_S15K7420A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '61',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '9000',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X305_S15K7560A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '45',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5400',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X287_FAL10288A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '45',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '5400',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X410_FAL10288A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_NO_PR'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68_89S',
##&nbsp;            'qual_flags' => '1',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '2700',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X286_SCHT7146S15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X419_SFIRF144A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X419_HCOBC144A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X419_FAL11144A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X419_FAL12144A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '3000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X420_SFIRF300A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '3000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X420_HCOBC300A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '3000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X420_HCOBD300A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '3000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X420_FAL11300A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '3000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X420_FAL12300A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X421_SFIRS450A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X421_HCOBD450A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X421_FAL12450A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X422_SFIRS600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X422_HCOBD600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X422_FAL12600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1300',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X425_SHORN144A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1300',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X425_HKCOB144A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1300',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X425_FAL11144A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2600',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X426_SYJKT300A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2600',
##&nbsp;            'rpm' => 'K15',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X426_FAL12300A15',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X421_WVEGA450A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '41',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X422_WVEGA600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '8441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X423_SCOMP900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X423_HCOBE900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X423_TAL13900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X423_WSIRI900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '8441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X423_SLTNG900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '100000228441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X417_SCOMP900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '100000224441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X417_HCOBE900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '100000224441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X417_TAL13900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '100000228441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X417_SLTNG900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '8441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X422_SCOMP600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X422_HCOBE600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X422_TAL13600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X422_WSIRI600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '8441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X422_SLTNG600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '100000228441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X416_SCOMP600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '100000224441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X416_HCOBE600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '100000224441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X416_TAL13600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '100000228441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X416_SLTNG600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '8441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X421_SCOMP450A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X421_HCOBE450A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X421_TAL13450A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X421_WSIRI450A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '8441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X421_SLTNG450A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X488_HCOBE900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X488_TAL13900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X488_WSIRI900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '8441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X488_SLTNG900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '100000228441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X495_SCOMP900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '100000224441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X495_HCOBE900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '100000224441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X495_TAL13900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '100000228441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X495_SLTNG900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG857',
##&nbsp;            'qual_flags' => '100000224441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '8000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X495_WSIRI900A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X487_HCOBD600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '8441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X487_SCOMP600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X487_HCOBE600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X487_TAL13600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X487_WSIRI600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '8441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X487_SLTNG600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '100000228441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X494_SCOMP600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '100000224441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X494_HCOBE600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '100000224441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X494_TAL13600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '100000228441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X494_SLTNG600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_SED',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG560',
##&nbsp;            'qual_flags' => '100000224441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '6000',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X494_WSIRI600A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA01',
##&nbsp;            'product' => 'X486_HCOBD450A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X486_FAL12450A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '8441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X486_SCOMP450A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X486_HCOBE450A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X486_TAL13450A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '4441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X486_WSIRI450A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_SEAGATE_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_SKIP_MASK_LBA',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG418',
##&nbsp;            'qual_flags' => '8441',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4500',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA00',
##&nbsp;            'product' => 'X486_SLTNG450A10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_DOWNLOAD_DISABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68_89S',
##&nbsp;            'qual_flags' => '220183',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '1440',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '1E00',
##&nbsp;            'product' => 'X259_MSABR100SAH',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_DOWNLOAD_DISABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_ATA',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136_89S',
##&nbsp;            'qual_flags' => '220183',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '3280',
##&nbsp;            'rpm' => 'K7',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'X263_MSABR160SAH',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG17',
##&nbsp;            'qual_flags' => '230461',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '900',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA09',
##&nbsp;            'product' => 'X233_HJURD018F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG34',
##&nbsp;            'qual_flags' => '230461',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1300',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA09',
##&nbsp;            'product' => 'X234_HJURD036F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68',
##&nbsp;            'qual_flags' => '230461',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1900',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA09',
##&nbsp;            'product' => 'X235_HJURD073F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG34',
##&nbsp;            'qual_flags' => '4234461',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '720',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA14',
##&nbsp;            'product' => 'X234_HJURE036F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68',
##&nbsp;            'qual_flags' => '4234461',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1440',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA14',
##&nbsp;            'product' => 'X235_HJURE073F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '4234461',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2820',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA14',
##&nbsp;            'product' => 'X236_HJURE146F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG34',
##&nbsp;            'qual_flags' => '4234461',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '720',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA14',
##&nbsp;            'product' => 'X270_HJURE036F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68',
##&nbsp;            'qual_flags' => '4234461',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1440',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA14',
##&nbsp;            'product' => 'X272_HJURE073F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '4234461',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2820',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA14',
##&nbsp;            'product' => 'X274_HJURE146F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG34',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '540',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'product' => 'X270_HPYTA036F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1020',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'product' => 'X272_HPYTA073F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2280',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'product' => 'X274_HPYTA146F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_HITACHI_ER_VALUES',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4560',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA04',
##&nbsp;            'product' => 'X276_HPYTA288F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG68',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '1120',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA05',
##&nbsp;            'product' => 'X272_FAL9E073F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG136',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '2120',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA05',
##&nbsp;            'product' => 'X274_FAL9E146F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_FAST_ZERO',
##&nbsp;                         'QD_EXTNDST_SUPPORTED',
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_ARWRE',
##&nbsp;                         'QD_NHT_SUPPORTED',
##&nbsp;                         'QD_HITACHI_FAILURE_INFO',
##&nbsp;                         'QD_VERIFY_CAPABLE',
##&nbsp;                         'QD_FUA_SUPPORTED',
##&nbsp;                         'QD_NDST_SUPPORTED'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'GIG272',
##&nbsp;            'qual_flags' => '65',
##&nbsp;            'chk' => 'On',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '520',
##&nbsp;            'zest' => '4250',
##&nbsp;            'rpm' => 'K10',
##&nbsp;            'vsiz' => '128',
##&nbsp;            'rev' => 'NA05',
##&nbsp;            'product' => 'X276_FAL9E288F10',
##&nbsp;            'lastblockazcs' => 'UNSUPPORTED_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'NETAPP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'LUN',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => '3PARdata',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.0',
##&nbsp;            'product' => 'VV',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'XIV',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'NEXTRA',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'XIV',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'NEXTRA-LUN-0',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => '2810XIV',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => '2810XIV-LUN-0',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => '2105800',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => '2105F20',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_UNSUPPORTED_ARRAY'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => '2145',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => '1750500',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => '2107900',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'Universal Xport',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => '1818FAStT',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => '1815FAStT',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => '1814      FAStT',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => '1722-600',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => '1742',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => '1742-900',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => '1724-100FAStT',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => '1726-4xx FAStT',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'LSI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'Universal Xport',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'LSI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'INF-01-00',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'ENGENIO',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'Universal Xport',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'ENGENIO',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'INF-01-00',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SUN',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'Universal Xport',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SUN',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'StoreEdge 6130',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SUN',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'StoreEdge 6140',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SUN',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'STK6580_6780',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SUN',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'StoreEdge 6540',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SUN',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'CSM200_R',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SUN',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'SUN_6180',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'STK',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'Universal Xport',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'STK',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'OPENstorageD178',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'STK',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'OPENstorageD220',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'STK',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'OPENstorageD240',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'STK',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'OPENstorageD280',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'STK',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'BladeCtlrBC82',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'STK',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'BladeCtlrBC84',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'STK',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'BladeCtlrBC88',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'STK',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'BladeCtlrB210',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'STK',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'BladeCtlrB220',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'STK',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'BladeCtlrB280',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'STK',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'FLEXLINE 380',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SUN',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'CSM100_R_FC',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SGI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'TP9300',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SGI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'TP9300 1G-2C',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SGI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'TP9300S',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SGI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'TP9500',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SGI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'TP9500S',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SGI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'TP9700',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SGI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'IS500',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'EMC',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'SYMMETRIX',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'DGC',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'LUNZ',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'DGC',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => undef,
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'DGC',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'VRAID',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'DGC',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'RAID6',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'DGC',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'RAID5',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'DGC',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'RAID3',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'DGC',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'RAID10',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'DGC',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'RAID1',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'DGC',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'RAID0',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'DGC',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'DISK',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HITACHI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'DISK-SUBSYSTEM',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HITACHI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'OPEN-3',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HITACHI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'OPEN-8',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HITACHI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'OPEN-9',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HITACHI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'OPEN-K',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HITACHI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'OPEN-E',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HITACHI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'OPEN-L',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HITACHI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'OPEN-V',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HITACHI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'DF600F',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HITACHI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'DF700',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'OPEN-3',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'OPEN-8',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'OPEN-9',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'OPEN-K',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'OPEN-E',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'OPEN-L',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'OPEN-V',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0.2',
##&nbsp;            'product' => 'DISK-SUBSYSTEM',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_UNSUPPORTED_ARRAY'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'FUJITSU',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'E3000',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_UNSUPPORTED_ARRAY'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'FUJITSU',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'E4000',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_UNSUPPORTED_ARRAY'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'FUJITSU',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'E6000',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_UNSUPPORTED_ARRAY'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'FUJITSU',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'E8000',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'FUJITSU',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'ETERNUS_DX8000',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'FUJITSU',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'ETERNUS_DX400',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'FUJITSU',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'ETERNUS_DXL',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'HSV200',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'HSV210',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'HSV101 (C)COMPAQ',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'COMPAQ',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'HSV111 (C)COMPAQ',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'HSV101',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'COMPAQ',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'HSV111',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'HSV300',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'HSV400',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'HSV450',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'HSV340',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'HP',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'HSV360',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'DDN',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'S2A 9550',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SGI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => '6700',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'DCS9550',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'DDN',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'S2A 9900',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'SGI',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'DD6A-FC008-9900',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'IBM',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'DCS9900',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'APPLE',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'Xserve RAID',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_UNSUPPORTED_ARRAY'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'TMS',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0000',
##&nbsp;            'product' => 'RamSan',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_DOWNLOAD_DISABLE',
##&nbsp;                         'QD_AZCS',
##&nbsp;                         'QD_ZCS_OVERRIDE_8_9',
##&nbsp;                         'QD_VSA'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '820010000002',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'VMware',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '1.0',
##&nbsp;            'product' => 'Virtual disk',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'QD_SWITCH_OPERATIONAL',
##&nbsp;                         'QD_MULTIPATH_CAPABLE',
##&nbsp;                         'QD_8_9_BCS',
##&nbsp;                         'QD_UNSUPPORTED_ARRAY'
##&nbsp;                       ],
##&nbsp;            'last_block' => 'NATIVE_SIZE',
##&nbsp;            'qual_flags' => '0',
##&nbsp;            'chk' => 'Off',
##&nbsp;            'vendor_name' => 'YOTTA',
##&nbsp;            'ssiz' => '512',
##&nbsp;            'zest' => '0',
##&nbsp;            'rpm' => 'KNA',
##&nbsp;            'vsiz' => '0',
##&nbsp;            'rev' => '0',
##&nbsp;            'product' => 'NETSTORAGER',
##&nbsp;            'lastblockazcs' => 'NATIVE_SIZE'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub storage_show_qualtable {
    my %args = @_;
    my $error_message;
    my @result;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'storage show qualtable',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 1,
        response_object => $resp,
    );
    my $i = 0;
    my @temp;
    foreach my $line (@lines) {
        if ( $command =~ /-x/ ) {
            if ( $line =~ /^Total.*=\s+(\d+)/ ) {
                $result[0]->{"no_of_entries_in_table"} = $1;
            } elsif ( $line =~ /^(\w+\s+(\w+\s+)?)-\s+(.*)/ ) {
                $result[0]->{"legend"}->[0]->{$1} = $3;
            } elsif ( $line =~ /^Displaying\s+(.*)/i ) {
                $result[0]->{"table_name"} = $1;
            } elsif ( $line
                =~ /^(\w+)\s+(\S+)\s+(\S+)\s+(Off|On)\s+(\d+|[a-z])+\s+(\d+)\s+(\d+)\s+(\d+)\s+(\w+)\s+(\S+)\s+(\w+)/
                || $line
                =~ /^(\w+)\s+(\S+\s+\S+)?\s+(\S+)\s+(Off|On)\s+(\d+|[a-z])+\s+(\d+)\s+(\d+)\s+(\d+)\s+(\w+)\s+(\S+)\s+(\w+)/
                ) {

                $result[$i]->{"vendor_name"} = $1;
                $result[$i]->{"product"}     = $2;
                $result[$i]->{"rev"}         = $3;
                $result[$i]->{"chk"}         = $4;
                $result[$i]->{"qual_flags"}  = $5;
                $result[$i]->{"zest"}        = $6;
                $result[$i]->{"ssiz"}        = $7;
                $result[$i]->{"vsiz"}        = $8;
                $result[$i]->{"rpm"}         = $9;
                $result[$i]->{"last_block"}  = $10;
                $result[$i]->{"lastblockazcs"} = $11;
            } elsif ( $line =~ /Flags:\s+(\S+)/ ) {
                $result[$i]->{"flags"} = $1 if ($1);
            } elsif ( $line =~ /^([Q]\w+\s*){2,}/si ) {
                push @temp, split( /\s+/, $line );
            } elsif ( $line =~ /^([Q]\w+)\s*$/ ) {
                push @temp, $1;
            } elsif ( $line
                =~ /Vendor\s*\|Product\s*\|Rev\s*\|Chk\|Qual Flags\s+\|ZEst\s*\|SSiz\|VSiz\|RPM\|\s*LastBlock|\s*LastBlockAzcs/
                ) {
                if (@temp) {
                    my @temp1;
                    push @temp1, @temp;
                    $result[$i]->{"flags"} = \@temp1;
                    @temp = ();
                }
                $i++;
            } elsif ( $line =~ /Matching entries found.*=\s+(\d+).*/ ) {
                $result[0]->{"matching_entries_found"} = $1;
                if (@temp) {
                    my @temp1;
                    push @temp1, @temp;
                    $result[$i]->{"flags"} = \@temp1;
                    @temp = ();
                }
            }
        } else {
            if ( $line =~ /^Total.*=\s+(\d+)/ ) {
                $result[0]->{"no_of_entries_in_table"} = $1;
            } elsif ( $line =~ /^(\w+\s+(\w+\s+)?)-\s+(.*)/ ) {
                $result[0]->{"legend"}->[0]->{$1} = $3;
            } elsif ( $line =~ /^Displaying\s+(.*)/i ) {
                $result[0]->{"table_name"} = $1;
            } elsif ( $line
                =~ /^(\w+)\s+(\S+)\s+(\S+)\s+(Off|On)\s+(\d+|[a-z])+\s+(\d+)\s+(\d+)\s+(\d+)\s+(\w+)\s+(\S+)\s+(\w+)/
                || $line
                =~ /^(\w+)\s+(\S+\s+\S+)?\s+(\S+)\s+(Off|On)\s+(\d+|[a-z])+\s+(\d+)\s+(\d+)\s+(\d+)\s+(\w+)\s+(\S+)\s+(\w+)/
                ) {
                $i++;
                $result[$i]->{"vendor_name"} = $1;
                $result[$i]->{"product"}     = $2;
                $result[$i]->{"rev"}         = $3;
                $result[$i]->{"chk"}         = $4;
                $result[$i]->{"qual_flags"}  = $5;
                $result[$i]->{"zest"}        = $6;
                $result[$i]->{"ssiz"}        = $7;
                $result[$i]->{"vsiz"}        = $8;
                $result[$i]->{"rpm"}         = $9;
                $result[$i]->{"last_block"}  = $10;
                $result[$i]->{"lastblockazcs"} = $11;
            } elsif ( $line =~ /Matching.*=\s+(\d+).*/ ) {
                $result[0]->{"matching_entries_found"} = $1;
            }
        }
    }
    return \@result;
} ## end sub storage_show_qualtable

## @name ::storage_show_acp
##
## @summary Parses the raw output from the "storage show acp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_show_acp with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_show_acp(%args);
## INPUTS TO METHOD CALL 'storage_show_acp' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; storage show acp -a; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp; Alternate Control Path:  Enabled
##&nbsp; Ethernet Interface:      e0P
##&nbsp; ACP Status:              Active
##&nbsp; ACP IP Address:          192.168.2.128
##&nbsp; ACP Subnet:              192.168.0.1
##&nbsp; ACP Netmask:             255.255.252.0
##&nbsp; ACP Connectivity Status: Full
##&nbsp;
##&nbsp; Shelf Module      Reset Cnt    IP Address      FW Version   Module Type  Status
##&nbsp; ----------------- ------------ --------------- ------------ ------------ -------
##&nbsp; 0a.00.A           000          192.168.2.173   01.10        IOM6E        active
##&nbsp; 0a.00.B           000          192.168.2.109   01.10        IOM6E        active
##&nbsp; 0a.01.A           000          192.168.3.19    02.06        IOM6         active
##&nbsp; 0a.01.B           000          192.168.2.233   02.06        IOM6         active
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'acp_ip_address' => '192.168.2.128',
##&nbsp;            'acp_status' => 'Active',
##&nbsp;            'acp_netmask' => '255.255.252.0',
##&nbsp;            'ethernet_interface' => 'e0P',
##&nbsp;            'alternate_control_path' => 'Enabled',
##&nbsp;            'acp_subnet' => '192.168.0.1',
##&nbsp;            'acp_connectivity_status' => 'Full'
##&nbsp;          }
##&nbsp;          'shelf_info' => [
##&nbsp;                            {
##&nbsp;                                'fw_version' => '01.10',
##&nbsp;                                'reset_cnt' => '000',
##&nbsp;                                'status' => 'active',
##&nbsp;                                'module_type' => 'IOM6E',
##&nbsp;                                'shelf_module' => '0a.00.A',
##&nbsp;                                'ip_address' => '192.168.2.173'
##&nbsp;                            },
##&nbsp;                            {
##&nbsp;                                'module_type' => 'IOM6E',
##&nbsp;                                'fw_version' => '01.10',
##&nbsp;                                'shelf_module' => '0a.00.B',
##&nbsp;                                'status' => 'active',
##&nbsp;                                'ip_address' => '192.168.2.109',
##&nbsp;                                'reset_cnt' => '000'
##&nbsp;                            },
##&nbsp;                            {
##&nbsp;                                'module_type' => 'IOM6',
##&nbsp;                                'fw_version' => '02.06',
##&nbsp;                                'shelf_module' => '0a.01.A',
##&nbsp;                                'status' => 'active',
##&nbsp;                                'ip_address' => '192.168.3.19',
##&nbsp;                                'reset_cnt' => '000'
##&nbsp;                            },
##&nbsp;                            {
##&nbsp;                                'module_type' => 'IOM6',
##&nbsp;                                'fw_version' => '02.06',
##&nbsp;                                'shelf_module' => '0a.01.B',
##&nbsp;                                'status' => 'active',
##&nbsp;                                'ip_address' => '192.168.2.233',
##&nbsp;                                'reset_cnt' => '000'
##&nbsp;                            }
##&nbsp;                          ];
##&nbsp;        ];
##&nbsp;
sub storage_show_acp {
    my %args = @_;
    my $error_message;
    my ( @result, $key );
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'storage show acp',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 1,
        response_object => $resp,
    );
    my $i = 0;
    my (@headings,@values);
    foreach my $line (@lines) {
        if ( $line =~ /(\w+\s+\w+(\s+\w+)?):\s+(\S+)/ ) {
            $key = $1;
            $result[0]->{ to_key_name( item => $key ) } = $3;
        } elsif ( $line =~ /^Shelf Module.*/ ) {
            @headings = split(/\s\s+/, $line);
        } elsif ($line =~ /^\d.*/) {
            @values = split(/\s+/, $line);
            for(my $k=0; $k<=$#headings; $k++){
                $result[0]->{shelf_info}->[$i]->{$headings[$k]} = $values[$k];
            }
            $i++;
        }
    }
    return \@result;
} ## end sub storage_show_acp

## @name ::storage_download_acp
##
## @summary Parses the raw output from the "storage download acp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_download_acp with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_download_acp(%args);
## INPUTS TO METHOD CALL 'storage_download_acp' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'file-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-S' </td> <td> 'stop' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-F' </td> <td> 'force-firmware' </td> <td> '1' </td> </tr>
## <tr> <td> '-R' </td> <td> 'revert' </td> <td> '1' </td> </tr>
## <tr> <td> '-Q' </td> <td> 'qual-mode' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'shelf-module' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'shelf-id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; storage download acp -S; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;[storage download acp] Stopping ACP firmware update in QUAL mode.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'mode' => 'QUAL',
##&nbsp;            'status' => 'Stopping'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub storage_download_acp {
    my %args = @_;
    my $error_message;
    my @result_array;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'storage download acp',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines
        = get_lines( raw_output => $raw_output, response_object => $resp );
    foreach my $line (@lines) {
        if ( $line
            =~ /.storage\s+download\s+acp.\s+([Ss]topping)\s+ACP\s+\S+\s+update\s+\S+\s+(QUAL)\s+.*/
            ) {
            $result_array[0]->{'status'} = $1;
            $result_array[0]->{'mode'}   = $2;
        } elsif ( $line
            =~ /.storage\s+download\s+acp.\s+([Ss]tarting)\s+cycle\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+(\S+)\s+\S+.*/
            ) {
            $result_array[0]->{'status'} = $1;
            $result_array[0]->{'mode'}   = $2;
        }
    }
    return \@result_array;
} ## end sub storage_download_acp

## @name ::storage_download_shelf
##
## @summary Parses the raw output from the "storage download shelf" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_download_shelf with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_download_shelf(%args);
## INPUTS TO METHOD CALL 'storage_download_shelf' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'file-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-T' </td> <td> 'enclosure-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-S' </td> <td> 'stop' </td> <td> '1' </td> </tr>
## <tr> <td> '-M' </td> <td> 'jake-reboot' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-E' </td> <td> 'enclosure-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-F' </td> <td> 'force-firmware' </td> <td> '1' </td> </tr>
## <tr> <td> '-R' </td> <td> 'revert' </td> <td> '1' </td> </tr>
## <tr> <td> '-Q' </td> <td> 'qual-mode' </td> <td> '1' </td> </tr>
## <tr> <td> '-H' </td> <td> 'channel-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; storage download shelf; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'in progress'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; storage download shelf -f -F AT-FCX.3800.SFW; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub storage_download_shelf {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $raw_output  = $resp->get_processed_output();
    my $error_message;
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @result_array;
    if ($raw_output) {
        my @lines = get_lines(
            raw_output      => $raw_output,
            response_object => $resp
        );
        foreach my $line (@lines) {
            if ( $line =~ /==== ses_fw_update (QUAL MODE .*BLED)/ ) {
                $result_array[0]->{'status'} = $1;
            } elsif ( $line =~ /Shelf firmware update still (in progress)/ ) {
                $result_array[0]->{'status'} = $1;
            } elsif ( $line =~ /(No shelves eligible for update)/ ) {
                $result_array[0]->{'status'} = $1;
            }
        }
    }
    return \@result_array;
} ## end sub storage_download_shelf

## @name ::storage_array_modify
##
## @summary Parses the raw output from the "storage array modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_array_modify with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_array_modify(%args);
## INPUTS TO METHOD CALL 'storage_array_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'array-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'options' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'new-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'model' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'prefix' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'vendor' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub storage_array_modify {
    my %args = @_;
    my $error_message;
    my $resp   = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub storage_array_modify

## @name ::storage_show_switch
##
## @summary Parses the raw output from the "storage show switch" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_show_switch with any of its options as described in the
##    corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_show_switch(%args);
## INPUTS TO METHOD CALL 'storage_show_switch' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = storage show switch
##&nbsp;
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Switch:                   WWN[1:000:000533:8a6204]
##&nbsp;Fabric:                   WWN[1:000:000533:8a6204]
##&nbsp;Name:                     dpg-mcc-brcd6510-05-A-fabY
##&nbsp;Domain:                   3
##&nbsp;Type:                     switch
##&nbsp;Version:                  v7.0.2b
##&nbsp;Vendor:                   Brocade Communications, Inc.
##&nbsp;
##&nbsp;Switch:                   WWN[1:000:000533:8a6a0c]
##&nbsp;Fabric:                   WWN[1:000:000533:8a6204]
##&nbsp;Name:                     dpg-mcc-brcd6510-05-B-fabY
##&nbsp;Domain:                   4
##&nbsp;Type:                     switch
##&nbsp;Version:                  v7.0.2b
##&nbsp;Vendor:                   Brocade Communications, Inc.
##&nbsp;
##&nbsp;Switch:                   WWN[1:000:000533:8a62c4]
##&nbsp;Fabric:                   WWN[1:000:000533:889c68]
##&nbsp;Name:                     dpg-mcc-brcd6510-05-A-fabX
##&nbsp;Domain:                   1
##&nbsp;Type:                     switch
##&nbsp;Version:                  v7.0.2b
##&nbsp;Vendor:                   Brocade Communications, Inc.
##&nbsp;
##&nbsp;Switch:                   WWN[1:000:000533:889c68]
##&nbsp;Fabric:                   WWN[1:000:000533:889c68]
##&nbsp;Name:                     dpg-mcc-brcd6510-05-B-fabX
##&nbsp;Domain:                   2
##&nbsp;Type:                     switch
##&nbsp;Version:                  v7.0.2b
##&nbsp;Vendor:                   Brocade Communications, Inc.
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'version' => 'v7.0.2b',
##&nbsp;            'type' => 'switch',
##&nbsp;            'fabric' => 'WWN[1:000:000533:8a6204]',
##&nbsp;            'domain' => '3',
##&nbsp;            'switch' => 'WWN[1:000:000533:8a6204]',
##&nbsp;            'vendor' => 'Brocade Communications, Inc.',
##&nbsp;            'name' => 'dpg-mcc-brcd6510-05-A-fabY'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'version' => 'v7.0.2b',
##&nbsp;            'type' => 'switch',
##&nbsp;            'fabric' => 'WWN[1:000:000533:8a6204]',
##&nbsp;            'domain' => '4',
##&nbsp;            'switch' => 'WWN[1:000:000533:8a6a0c]',
##&nbsp;            'vendor' => 'Brocade Communications, Inc.',
##&nbsp;            'name' => 'dpg-mcc-brcd6510-05-B-fabY'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'version' => 'v7.0.2b',
##&nbsp;            'type' => 'switch',
##&nbsp;            'fabric' => 'WWN[1:000:000533:889c68]',
##&nbsp;            'domain' => '1',
##&nbsp;            'switch' => 'WWN[1:000:000533:8a62c4]',
##&nbsp;            'vendor' => 'Brocade Communications, Inc.',
##&nbsp;            'name' => 'dpg-mcc-brcd6510-05-A-fabX'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'version' => 'v7.0.2b',
##&nbsp;            'type' => 'switch',
##&nbsp;            'fabric' => 'WWN[1:000:000533:889c68]',
##&nbsp;            'domain' => '2',
##&nbsp;            'switch' => 'WWN[1:000:000533:889c68]',
##&nbsp;            'vendor' => 'Brocade Communications, Inc.',
##&nbsp;            'name' => 'dpg-mcc-brcd6510-05-B-fabX'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub storage_show_switch {
    my %args = @_;
    my $error_message;
    my $resp   = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @result_array;
    my $i = -1;
    my @lines = get_lines(
        raw_output      => $output,
        response_object => $resp
    );
    foreach my $line (@lines) {
        if ($line =~/(Switch):\s+(.*)/){
            $i++;
            $result_array[$i]->{to_key_name( item =>$1)} = $2;
        } elsif ($line =~/(.*):\s+(.*)/ && $i > -1 ) {
            $result_array[$i]->{to_key_name( item =>$1)} = $2;
        }
    }
    return \@result_array;
} ## end sub storage_show_switch

## @name ::storage_release_disks
##
## @summary Parses the raw output from the "storage release disks" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_release_disks with any of its options as described
##    in the corresponding cdef file 'storage.cdefs'
## @synopsis
##      storage_release_disks(%args);
## INPUTS TO METHOD CALL 'storage_release_disks' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = storage release disks
##&nbsp;$VAR1 = [];
##&nbsp;

sub storage_release_disks {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub storage_release_disks
1;
