#Copyright (c) 2011-2014 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode vol commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode vol commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "vol status" will be vol_status()
##
## @synopsis
##    use vol;
##
##    vol_create(%args);
##    vol_status(%args);
##    vol_online(%args);
##    vol_offline(%args);
##    vol_destroy(%args);
##    vol_options(%args);
##    vol_rename(%args);
##    vol_restrict(%args);
##    vol_media_scrub_status(%args);
##    vol_scrub_status(%args);
##    vol_verify_status(%args);
##    vol_scrub_start(%args);
##    vol_scrub_stop(%args);
##    vol_scrub_resume(%args);
##    vol_scrub_suspend(%args);
##    vol_verify_start(%args);
##    vol_verify_stop(%args);
##    vol_verify_resume(%args);
##    vol_verify_suspend(%args);
##    vol_lang(%args);
##    vol_size(%args);
##    vol_autosize(%args);
##    vol_vvlabel(%args);
##    vol_read_fsid(%args);
##    vol_rewrite_fsid(%args);
##    vol_clear_error(%args);
##    vol_clone_create(%args);
##    vol_clone_split(%args);
##    vol_clone_split_start(%args);
##    vol_clone_split_stop(%args);
##    vol_clone_split_status(%args);
##    vol_clone_split_estimate(%args);
##    vol_64bit_upgrade_start(%args);
##    vol_copy_start(%args);
##    vol_copy_status(%args);
##    vol_copy_abort(%args);
##    vol_copy_throttle(%args);
##    vol_container(%args);
##    vol_move_start(%args);
##    vol_move_pause(%args);
##    vol_move_resume(%args);
##    vol_move_status(%args);
##    vol_move_cutover(%args);
##    vol_move_abort(%args);
##    vol_add(%args);
##    vol_mirror(%args);
##    vol_split(%args);
##    vol_wafliron_start(%args);
##    vol_wafliron_status(%args);
##    vol_wafliron_stop(%args);
##    vol_batched_free_log_dump(%args);
##    vol_batched_free_log_show_early_kickout_queue
##    vol_batched_free_log_status
##    vol_batched_free_log_suspend
##    vol_batched_free_log_resume
##    vol_batched_free_log_dump
##    vol_batched_free_log_drain_start
##    vol_batched_free_log_drain_stop
##    vol_batched_free_log_drain_status
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::vol;

use strict;
use Data::Dumper;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::MissingValueException;

## @name ::vol_create
##
## @summary Parses the raw output from the "vol create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_create with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_create(%args);
## INPUTS TO METHOD CALL 'vol_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-e' </td> <td> 'vm-align-sector' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'size' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'is-display-command' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'raid-size' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'language-code' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'is-mirrored' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-count' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'mirror-disks' </td> <td> '0' </td> </tr>
## <tr> <td> '-L' </td> <td> 'snaplock-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'volume-raid-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-S' </td> <td> 'remote-location' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'language' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'space-reserve' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'containing-aggr-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'disks' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'language-code-for-create' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'vm-align-suffix' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vol_create(%) {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output        => $output,
        command           => "vol create",
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $regex =
        'create\s+([^\s]+)\s+(-f\s+)?(-l\s+(\w+)\s+)?(-m\s+)?'
      . '(-L\s+(\w+\s+)?)?(-r\s+(\w+)\s+)?(-t\s+(\w+)\s+)?-d\s+'
      . '([^-\r\n]+)(-d\s+([^\r\n]+))?';

    if ( $output =~ /$regex/ ) {    # for -n option
        my %hash;
        $hash{'volume'}        = $1;
        $hash{'force'}         = 1 if ($2);
        $hash{'language-code'} = $4 if ($3);
        $hash{'is-mirrored'}   = 1 if ($5);
        $hash{'snaplock-type'} = $7 ? trim_string( item => $7 ) : "" if ($6);
        $hash{'raid-size'}     = $9 if ($8);
        $hash{'raid-type'}     = $11 if ($10);
        my @disks = split( /\s+/, $12 );

        if ($13) {
            my @mirror_disks = split( /\s+/, $14 );
            $hash{'mirror-disks'} = \@mirror_disks;
        }
        $output =~ /(vol create\s.*-d[^\r\n]*)/;
        $hash{'command'} = $1;
        $hash{'disks'}   = \@disks;
        @result_array    = ( \%hash );
    }
    elsif ( $output =~
/Creation\sof\svolume\s\'(.*)\'\swith\ssize\s(.*)\son\scontaining\saggregate\r*\n\'(.*)\'\shas\scompleted/
      )
    {
        my %hash;
        $hash{'volume'}    = $1;
        $hash{'result'}    = 'created';
        $hash{'size'}      = $2;
        $hash{'aggregate'} = $3;
        @result_array      = ( \%hash );
    }
    elsif (
        $output =~ /Creation\sof\sa\svolume\swith\s(.*)\sdisks\shas\s(.*)\./ )
    {
        @result_array = ( { 'result' => $2, 'disks' => $1 } );
    }
    elsif ( $output =~
/Creation\sof\s(a\s)?volume\s(.*\s)?has\sbeen\s(.*)\.\s+(\d+)\s+disk(s)?\s+need(s)?\s+to\s+be\s+zeroed\s+before/
      )
    {
        @result_array = ( { 'result' => $1, 'disks' => $2 } );
    }
    elsif ( $output =~
        /volume has been created with (\d+) .* added to the volume.*(\d+).*need/
      )
    {
        @result_array = (
            {
                'result'          => 'initiated',
                'disks-completed' => $1,
                'disks-remaining' => $2
            }
        );
    }
    else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "Output is not in the expected format",
            response_object => $resp
        );
    }
    return \@result_array;
} ## end sub vol_create(%)

## @name ::vol_rename
##
## @summary Parses the raw output from the "vol rename" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_rename with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_rename(%args);
## INPUTS TO METHOD CALL 'vol_rename' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-volume-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_rename(%) {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output        => $output,
        command           => "vol rename",
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %hash;

    if ( $output =~ /\'(.*)\'\srenamed\sto\s\'(.*)\'/ ) {
        @result_array =
          ( { 'oldname' => $1, 'newname' => $2, 'result' => 'renamed' } );
        return \@result_array;
    }
    NACL::APISet::Exceptions::UnexpectedOutputException->throw(
        text            => "output is not in the expected format",
        response_object => $resp
    );
} ## end sub vol_rename(%)

## @name ::vol_destroy
##
## @summary Parses the raw output from the "vol destroy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_destroy with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_destroy(%args);
## INPUTS TO METHOD CALL 'vol_destroy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_destroy(%) {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output        => $output,
        command           => "vol destroy",
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %hash;

    my $expected_prompt = 'This volume is marked corrupt and could be useful'
      . ' for debugging purposes, are you sure you want to proceed? ';

    my $op_copy = quotemeta($output);

    if ( $output =~ /Volume\s\'(.*)\'\sdestroyed/ ) {
        @result_array = ( { 'volume' => $1, 'result' => 'destroyed' } );
        return \@result_array;
    }
    elsif ( ( $expected_prompt =~ /$op_copy/ ) || ( $op_copy eq '' ) ) {

        # If user has not matched the complete prompt string
        # then $output will have post match string
        # In case of complete match, $output is set to ''.
        # This code is added to avoid false UnexpectedOutputException
        @result_array = ();
        return \@result_array;
    }
    NACL::APISet::Exceptions::UnexpectedOutputException->throw(
        text            => "output is not in the expected format",
        response_object => $resp
    );
} ## end sub vol_destroy(%)

## @name ::vol_offline
##
## @summary Parses the raw output from the "vol offline" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_offline with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_offline(%args);
## INPUTS TO METHOD CALL 'vol_offline' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> '-t' </td> <td> 'cifs-delay' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_offline(%) {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output        => $output,
        command           => "vol offline",
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ( $output =~ /Volume\s\'(.*)\'\s.*\soffline/ ) {
        @result_array = ( { 'volume' => $1, 'result' => 'offline' } );
        return \@result_array;
    }
    NACL::APISet::Exceptions::CommandFailedException->throw(
        text            => $output,
        response_object => $resp
    );
} ## end sub vol_offline(%)

## @name ::vol_online
##
## @summary Parses the raw output from the "vol online" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_online with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_online(%args);
## INPUTS TO METHOD CALL 'vol_online' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_online(%) {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output        => $output,
        command           => "vol online",
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %hash;

    if ( $output =~ /Volume\s\'(.*)\'\s.*\sonline/ ) {
        @result_array = ( { 'volume' => $1, 'result' => 'online' } );
        return \@result_array;
    }
    NACL::APISet::Exceptions::CommandFailedException->throw(
        text            => $output,
        response_object => $resp
    );
} ## end sub vol_online(%)

## @name ::vol_options
##
## @summary Parses the raw output from the "vol options" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_options with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_options(%args);
## INPUTS TO METHOD CALL 'vol_options' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'option-value' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'option-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vol_options(%) {
    my %args = @_;
    my @options_array;
    my %option_hash;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output        => $output,
        command           => "vol options",
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    $output =~ s/\r|\n//g;

    if (
        ( $command =~ /vol options\s+[^\s]+\s+[^\s]+\s+[^\s]+\s*/ )
        && !snip_only_required_output(
            raw_output => $output,
            delimiter  => ',.*,'
        )
      )
    {
        return [];
    }

    my @items = split( /,\s*/, $output );
    my %hash;

    foreach my $item (@items) {
        if ( $item !~ /=/ ) {
            $hash{$item} = 'true';
            next;
        }

        my ( $name, $value ) = split( /=/, $item );
        $hash{$name} = $value;
    }

    push( @options_array, \%hash );
    $option_hash{"options"} = \@options_array;
    @result_array = ( \%option_hash );
    return \@result_array;

} ## end sub vol_options(%)

## @name ::vol_status
##
## @summary Parses the raw output from the "vol status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_status with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_status(%args);
## INPUTS TO METHOD CALL 'vol_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-b' </td> <td> 'blocks' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'checksum' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'device' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'broken-disks' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'language' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'maintenance-disks' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'raid-disk' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'spare-disks' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-vC' </td> <td> 'verbose-list' </td> <td> '1' </td> </tr>
## <tr> <td> '-vC' </td> <td> 'verbose-cluster-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'expiry-date' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'list' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'cluster-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-Cv' </td> <td> 'list-verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-Cv' </td> <td> 'cluster-info-verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-S' </td> <td> 'show-space' </td> <td> '1' </td> </tr>
## <tr> <td> '-F' </td> <td> 'show-footprint' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; vol status -Cv vol0; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;         Volume State           Status            Options
##&nbsp;           vol0 online          raid4, flex       root, diskroot, nosnap=off, nosnapdir=off,
##&nbsp;                                parity uninit'd!  minra=off, no_atime_update=off, nvfail=off,
##&nbsp;                                64-bit            ignore_inconsistent=off, snapmirrored=off,
##&nbsp;                                                  create_ucode=off, convert_ucode=off,
##&nbsp;                                                  maxdirsize=16291, schedsnapname=ordinal,
##&nbsp;                                                  fs_size_fixed=off, guarantee=volume,
##&nbsp;                                                  svo_enable=off, svo_checksum=off,
##&nbsp;                                                  svo_allow_rman=off, svo_reject_errors=off,
##&nbsp;                                                  no_i2p=off, fractional_reserve=100, extent=off,
##&nbsp;                                                  try_first=volume_grow, read_realloc=off,
##&nbsp;                                                  snapshot_clone_dependency=off,
##&nbsp;                                                  dlog_hole_reserve=off, nbu_archival_snap=off
##&nbsp;                         Volume UUID: 5361dde2-197c-11e1-a24e-123478563412
##&nbsp;		Containing aggregate: 'aggr0'
##&nbsp;                        Volinfo mode: 7-mode
##&nbsp;                Volume Instance UUID: 5361de1d-197c-11e1-a24e-123478563412
##&nbsp;              Volume Provenance UUID: 5361de1d-197c-11e1-a24e-123478563412
##&nbsp;
##&nbsp;		Volume Transition Info:
##&nbsp;               Copied for Transition: false
##&nbsp;                    Transition State: none
##&nbsp;                        Transitioned: false
##&nbsp;
##&nbsp;                Plex /aggr0/plex0: online, normal, active
##&nbsp;                    RAID group /aggr0/plex0/rg0: normal, block checksums
##&nbsp;
##&nbsp;	Snapshot autodelete settings for vol0:
##&nbsp;					state=off
##&nbsp;					commitment=try
##&nbsp;					trigger=volume
##&nbsp;					target_free_space=20%
##&nbsp;					delete_order=oldest_first
##&nbsp;					defer_delete=user_created
##&nbsp;					prefix=(not specified)
##&nbsp;					destroy_list=none
##&nbsp;	Volume autosize settings:
##&nbsp;				mode=off
##&nbsp;	Hybrid Cache:
##&nbsp;		Eligibility=read-write
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'volume_uuid' => '5361dde2-197c-11e1-a24e-123478563412',
##&nbsp;            'volume_instance_uuid' => '5361de1d-197c-11e1-a24e-123478563412',
##&nbsp;            'volinfo_mode' => '7-mode',
##&nbsp;            'options' => [
##&nbsp;                           {
##&nbsp;                             'fractional_reserve' => '100',
##&nbsp;                             'ignore_inconsistent' => 'off',
##&nbsp;                             'maxdirsize' => '16291',
##&nbsp;                             'create_ucode' => 'off',
##&nbsp;                             'svo_allow_rman' => 'off',
##&nbsp;                             'nosnapdir' => 'off',
##&nbsp;                             'snapshot_clone_dependency' => 'off',
##&nbsp;                             'svo_checksum' => 'off',
##&nbsp;                             'svo_reject_errors' => 'off',
##&nbsp;                             'svo_enable' => 'off',
##&nbsp;                             'root' => 'true',
##&nbsp;                             'nbu_archival_snap' => 'off',
##&nbsp;                             'extent' => 'off',
##&nbsp;                             'no_atime_update' => 'off',
##&nbsp;                             'read_realloc' => 'off',
##&nbsp;                             'convert_ucode' => 'off',
##&nbsp;                             'try_first' => 'volume_grow',
##&nbsp;                             'fs_size_fixed' => 'off',
##&nbsp;                             'schedsnapname' => 'ordinal',
##&nbsp;                             'nosnap' => 'off',
##&nbsp;                             'guarantee' => 'volume',
##&nbsp;                             'snapmirrored' => 'off',
##&nbsp;                             'dlog_hole_reserve' => 'off',
##&nbsp;                             'diskroot' => 'true',
##&nbsp;                             'minra' => 'off',
##&nbsp;                             'no_i2p' => 'off',
##&nbsp;                             'nvfail' => 'off'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;            'volume' => 'vol0',
##&nbsp;            'status' => 'raid4, flex, parity uninit\'d!, 64-bit',
##&nbsp;            'hybrid_cache' => [
##&nbsp;                                {
##&nbsp;                                  'eligibility' => 'read-write'
##&nbsp;                                }
##&nbsp;                              ],
##&nbsp;            'volume_transistion_info' => [
##&nbsp;                                           {
##&nbsp;                                             'copied_for_trasition' => 'false',
##&nbsp;                                             'transition_state' => 'none',
##&nbsp;                                             'transitioned' => 'false'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;            'snapshot_autodelete_settings' => [
##&nbsp;                                                {
##&nbsp;                                                  'destroy_list' => 'none',
##&nbsp;                                                  'trigger' => 'volume',
##&nbsp;                                                  'delete_order' => 'oldest_first',
##&nbsp;                                                  'defer_delete' => 'user_created',
##&nbsp;                                                  'state' => 'off',
##&nbsp;                                                  'target_free_space' => '20%',
##&nbsp;                                                  'commitment' => 'try',
##&nbsp;                                                  'prefix' => '(not specified)'
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;            'state' => 'online',
##&nbsp;            'containing_aggregate' => 'aggr0',
##&nbsp;            'volume_autosize_settings' => [
##&nbsp;                                            {
##&nbsp;                                              'mode' => 'off'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;            'volume_provenance_uuid' => '5361de1d-197c-11e1-a24e-123478563412'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vol status -F
##&nbsp;
##&nbsp;-------- Parsed Output ---------
##&nbsp;Parsed is $VAR1 = [
##&nbsp;          {
##&nbsp;            'volume_data_footprint_percent' => '0%',
##&nbsp;            'volume' => 'vol0',
##&nbsp;            'delayed_frees_percent' => '0%',
##&nbsp;            'flexible_volume_metadata_percent' => '1%',
##&nbsp;            'volume_guarantee' => '528GB',
##&nbsp;            'total_percent' => '95%',
##&nbsp;            'total' => '533GB',
##&nbsp;            'volume_data_footprint' => '216MB',
##&nbsp;            'volume_guarantee_percent' => '94%',
##&nbsp;            'delayed_frees' => '1.85GB',
##&nbsp;            'flexible_volume_metadata' => '2.95GB'
##&nbsp;          },
##&nbsp;        ],
##&nbsp;
##&nbsp;Command = vol status -S
##&nbsp;
##&nbsp;-------- Parsed Output ---------
##&nbsp;Parsed is $VAR1 = [
##&nbsp;          {
##&nbsp;            'inodes_percent' => '0%',
##&nbsp;            'snapshot_reserve_percent' => '5%',
##&nbsp;            'volume' => 'vol0',
##&nbsp;            'inodes' => '1.99MB',
##&nbsp;            'user_data' => '1.22GB',
##&nbsp;            'total_percent' => '5%',
##&nbsp;            'total' => '27.7GB',
##&nbsp;            'snapshot_reserve' => '26.5GB',
##&nbsp;            'filesystem_metadata' => '944KB',
##&nbsp;            'user_data_percent' => '0%',
##&nbsp;            'filesystem_metadata_percent' => '0%'
##&nbsp;          },
##&nbsp;        ],

sub vol_status {
    my %args = @_;
    my $result;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "vol status",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $result = _vol_status(
        raw_output      => $raw_output,
        command         => $command,
        response_object => $resp
    );
    return $result;
} ## end sub vol_status

## @name ::vol_restrict
##
## @summary Parses the raw output from the "vol restrict" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_restrict with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_restrict(%args);
## INPUTS TO METHOD CALL 'vol_restrict' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> '-t' </td> <td> 'cifs-delay' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_restrict {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();

    my $result;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "vol restrict",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    if ( $raw_output =~ /\'(\w+)\' is now restricted/ ) {
        $result = [ { "volume" => $1, "status" => "Restricted" } ];
    }
    else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "output is not in the expected format",
            response_object => $resp
        );
    }

    return $result;
} ## end sub vol_restrict

sub _vol_status {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $option     = $args{"command"};
    my $response   = $args{"response_object"};

    # For vol_status, some output is always expected.
    # So we are making $can_throw as 1 so that if the
    # lines array got finally in _get_lines is empty,
    # UnexpectedOutputException can be thrown there.
    my $can_throw = 1;

    # Extracting the command option. since we know that for vol status there
    # cannot be more than one option, this simple logic will work.
    $option =~ s/vol\sstatus\s?//;
    if ( $option !~ /-\w/ ) {
        $option = "";
    }

    # Possible states a volume can be in
    my @volume_states = (
        "copying",         "degraded",
        "flex",            "foreign",
        "growing",         "initializing",
        "invalid",         "ironing",
        "mirror degraded", "mirrored",
        "needs check",     "out-of-date",
        "partial",         "raid0",
        "raid4",           "raid_dp",
        "reconstruct",     "resyncing",
        "snapmirrored",    "sv-restoring",
        "trad",            "unrecoverable",
        "verifying",       "wafl inconsistent",
        "flexcache",       "connecting",
        "online",          "offline",
        "creating",        "restricted"
    );
    my $some_state = join( "|", @volume_states );

    # Based on options sent to vol_status command, the parsing method will be
    # decided
    if (   ( $option eq "" )
        || ( ( $option =~ /-C/ ) && ( $option !~ /-Cv/ ) ) )
    {
        my $result;

        # Convert the raw_output into lines before parsing
        my $first_line = 'Volume\s+State\s+Status\s+Options';
        my @arr        = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );

        shift(@arr);    #Remove Heading line

        my $if_specific = 0;

        # The vol status command has been executed for one specific volume
        if ( grep( /Containing\saggregate:/, @arr ) ) {
            $if_specific = 1;
        }
        if ( $if_specific == 1 ) {
            my @vol_array = ( \@arr );
            $result = _parse_volume_data(
                "volume_array_ref" => \@vol_array,
                "response_object"  => $response
            );
            return $result;
        }
        my @vol_array  = ();
        my @temp_array = ( $arr[0] );    #Push the first line always

        # Now process the rest of the lines
        for ( my $line = 1 ; $line < @arr ; $line++ ) {
            next if ( $arr[$line] eq "" );
            if ( $arr[$line] =~ /($some_state)\s+\b(raid\w+),?/ ) {
                push( @vol_array, [@temp_array] );
                @temp_array = ( $arr[$line] );
            }
            else {
                push( @temp_array, $arr[$line] );
            }
        }
        push( @vol_array, [@temp_array] )
          ;    #For pushing the last volume data collected
        $result = _parse_volume_data(
            "volume_array_ref" => \@vol_array,
            "response_object"  => $response
        );
        return $result;
    }
    elsif ( $option =~ /-b/ || $option =~ /-d/ || $option =~ /-w/ ) {
        my ( $first_line, @heading );
        my @want_array;

        if ( $option =~ /-b/ ) {
            $first_line =
'Volume\s+Block\sSize\s\(bytes\)\s+Vol\sSize\s\(blocks\)\s+FS\sSize\s\(blocks\)';
            @heading = (
                "volume",           "block_size(bytes)",
                "vol_size(blocks)", "fs_size(blocks)"
            );
        }
        elsif ( $option =~ /-d/ ) {
            $first_line =
'Device\s+HA\s+SHELF\s+BAY\s+CHAN\s+Disk\sVital\sProduct\sInformation';
            @heading = (
                "device", "ha", "shelf", "bay", "chan",
                "disk_vital_product_information"
            );
        }
        elsif ( $option =~ /-w/ ) {
            $first_line = 'Volume\s+Expiry\sDate';
            @heading = ( "volume", "expiry_date" );
        }
        my @arr = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );

        # The first two lines in the @arr contains heading and underline
        # Hence we remove those two lines before parsing
        splice( @arr, 0, 2 );

        for ( my $count = 0 ; $count < @arr ; $count++ ) {
            my %output = ();
            my $index  = 0;
            my @values;
            if ( $option =~ /-w/ ) {
                @values = split( /\s+/, $arr[$count], 2 );
            }
            else {
                @values = split( /\s+/, $arr[$count] );
            }
            foreach (@values) {
                $output{ $heading[$index] } = $_;
                $index++;
            }
            push( @want_array, \%output );
        } ## end for (my $count = 0 ; $count...
        return \@want_array;
    }
    elsif ( $option =~ /-l/ ) {
        my @want_array;
        my $first_line = 'Volume\s+Language';
        my @arr        = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );
        my @heading = ( "volume", "language" );

        # Remove the heading line from the array
        shift(@arr);
        for ( my $count = 0 ; $count < @arr ; $count++ ) {
            my %output = ();
            my $index  = 0;
            my @values = split( /\s+/, $arr[$count] );
            $output{ $heading[0] } = shift(@values);
            $output{ $heading[1] } = join( " ", @values );
            push( @want_array, \%output );
        }
        return \@want_array;
    }
    elsif ( $option =~ /-c/ ) {
        my @want_array;
        my $first_line = 'Volume\s+Checksum\sstyle\s+Checksum\sstatus';
        my @arr        = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );
        my @heading = ( "volume", "checksum_style", "checksum_status" );

        # Remove the heading line from the array
        shift(@arr);
        for ( my $count = 0 ; $count < @arr ; $count++ ) {
            my %output = ();
            my $index  = 0;
            my @values = split( /\s+/, $arr[$count] );
            $values[2] = $values[2] . " " . $values[3];
            pop(@values);
            foreach (@values) {
                $output{ $heading[$index] } = $_;
                $index++;
            }
            push( @want_array, \%output );
        } ## end for (my $count = 0 ; $count...
        return \@want_array;
    }
    elsif ( $option =~ /-f/ ) {
        my $first_line = 'Broken\sdisks\s?(\(?empty\)?)?';
        my @arr        = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );
        if ( grep( /Broken\sdisks\s?\(empty\)/, @arr ) ) {
            my @want_array;
            return \@want_array;
        }
        else {
            shift(@arr);    #First line is an empty line
            my $arr_ref     = \@arr;
            my $disk_output = _get_broken_or_partner_disks(
                "broken_or_partner_disks_ref" => $arr_ref );
            return $disk_output;
        }
    }
    elsif ( $option =~ /-r/ ) {
        my @want_array;
        my $first_line = '((Volume\s\w+\s.*)|(Aggregate\s\w+\s.*))';
        my @arr        = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );

        # necessary data structure to logically separate the data
        my ( @Volume, @Aggregate, @Pool_spare_disks, @Spare_disks,
            $Unassimilated ,$Broken_disks, $Partner_disks, $Maintenance_disks );
        my ( $Volume_index, $Aggr_index, $Pool_spare_index, $Spare_index ) =
          ( 0, 0, 0, 0 );
        my $temp_ref;
        my %output = ();
        for ( my $i = 0 ; $i < scalar(@arr) ; $i++ ) {
            next if ( $arr[$i] eq "" );
	     if ( $arr[$i] =~
/^(Volume\s.*|Aggregate\s.*|Pool[01]\sspare\sdisks\s?(\(empty\))?|Spare\sdisks\s?(\(empty\))?|Broken\sdisks|Partner\sdisks|Maintenance\sdisks)$/
              )
            {

                # The next logical block has started
                my $matched = $1;
                if ( $matched =~ /^Volume/ ) {
                    $Volume[$Volume_index] = [];
                    $temp_ref = $Volume[ $Volume_index++ ];
                }
                elsif ( $matched =~ /^Aggregate/ ) {
                    $Aggregate[$Aggr_index] = [];
                    $temp_ref = $Aggregate[ $Aggr_index++ ];
                }
                elsif ( $matched =~ /Pool\d\sspare\sdisks/ ) {
                    $Pool_spare_disks[$Pool_spare_index] = [];
                    $temp_ref = $Pool_spare_disks[ $Pool_spare_index++ ];
                }
                elsif ( $matched eq "Spare disks" ) {
                    $Spare_disks[$Spare_index] = [];
                    $temp_ref = $Spare_disks[ $Spare_index++ ];
                }
                elsif ( $matched eq "Broken disks" ) {
                    $Broken_disks = [];
                    $temp_ref     = $Broken_disks;
                }
				elsif ( $matched eq "Maintenance disks" ) {
                    $Maintenance_disks = [];
                    $temp_ref          = $Maintenance_disks;
                }
                else {
                    $Partner_disks = [];
                    $temp_ref      = $Partner_disks;
                }
                push( @$temp_ref, $arr[$i] );
            }
            else {
			if($arr[$i] =~/Unassimilated/)
			{
			$Unassimilated = [];
            $temp_ref     = $Unassimilated;
			}
			push( @$temp_ref, $arr[$i] );
            }
        } ## end for (my $i = 0 ; $i < scalar...
        if (@Volume) {
            $output{"volume"} = _get_vol_parsed(
                "volume_ref"      => \@Volume,
                "response_object" => $response
            );
        }
        if (@Aggregate) {
            $output{"aggregate"} = _get_vol_parsed(
                "volume_ref"      => \@Aggregate,
                "response_object" => $response
            );
        }
        if (@Pool_spare_disks) {
            $output{"spare_disks"} =
              _get_spare_disks( "spare_disks_ref" => \@Pool_spare_disks );
        }
        if (@Spare_disks) {
            $output{"spare_disks"} =
              _get_spare_disks( "spare_disks_ref" => \@Spare_disks );
        }
        if ( defined($Broken_disks) ) {
            $output{"broken_disks"} = _get_broken_or_partner_disks(
                "broken_or_partner_disks_ref" => $Broken_disks );
        }
	if ( defined($Partner_disks) ) {
            $output{"partner_disks"} = _get_broken_or_partner_disks(
                "broken_or_partner_disks_ref" => $Partner_disks );
        }
	if ( defined($Unassimilated) ) {
            $output{"unassimilated"} =  _get_unassimilated_disks(
                "unassimilated_disks_ref" => $Unassimilated );
        }
	if ( defined($Maintenance_disks) ) {
            $output{"maintenance_disks"} = _get_broken_or_partner_disks(
                "broken_or_partner_disks_ref" => $Maintenance_disks );
        }
        push( @want_array, \%output );
        return \@want_array;
    }
    elsif ( $option =~ /-s/ ) {
        my @want_array;
        my $first_line =
'([Pp]ool[01]\sspare\sdisks\s?(\(empty\))?|[Ss]pare\sdisks\s?(\(empty\))?)';
        my @arr = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );
        my ( @Pool_spare_disks, @Spare_disks );
        my ( $Pool_spare_index, $Spare_index ) = ( 0, 0 );
        my $temp_ref;
        my $output_ref;

        for ( my $i = 0 ; $i < @arr ; $i++ ) {
            next if ( $arr[$i] eq "" );
            if ( $arr[$i] =~
/^(Pool[01]\sspare\sdisks\s?(\(empty\))?|Spare\sdisks\s?(\(empty\))?)$/
              )
            {
                my $matched = $1;
                if ( $matched =~ /Pool\d\sspare\sdisks/ ) {
                    $Pool_spare_disks[$Pool_spare_index] = [];
                    $temp_ref = $Pool_spare_disks[ $Pool_spare_index++ ];
                }
                else {
                    $Spare_disks[$Spare_index] = [];
                    $temp_ref = $Spare_disks[ $Spare_index++ ];
                }
                push( @$temp_ref, $arr[$i] );
            }
            else {
                push( @$temp_ref, $arr[$i] );
            }
        } ## end for (my $i = 0 ; $i < @arr...
        if (@Pool_spare_disks) {
            $output_ref =
              _get_spare_disks( "spare_disks_ref" => \@Pool_spare_disks );
        }
        if (@Spare_disks) {
            $output_ref =
              _get_spare_disks( "spare_disks_ref" => \@Spare_disks );
        }
        return $output_ref;
    }
    elsif (( $option =~ /-v/ )
        || ( $option =~ /-Cv/ )
        || ( $option =~ /-vC/ ) )
    {
        my $result;
        my $first_line = 'Volume\s+State\s+Status\s+Options';
        my @arr        = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );

        # Remove the heading line
        shift(@arr);

        # Storing the data displayed for each of the volume in an anonymous
        # array and pushing the reference into @vol_array for later
        # processing of individual fields
        #
        my @vol_array  = ();
        my @temp_array = ( $arr[0] );    #Push the first line always

        # Now process the rest of the lines
        for ( my $line = 1 ; $line < @arr ; $line++ ) {
            if ( $arr[$line] =~ /($some_state)\s+\b(raid\w+),?/ ) {
                push( @vol_array, [@temp_array] );
                @temp_array = ( $arr[$line] );
            }
            else {
                push( @temp_array, $arr[$line] );
            }
        }
        push( @vol_array, [@temp_array] )
          ;    #For pushing the last volume data  collected
        $result = _parse_volume_data(
            "volume_array_ref" => \@vol_array,
            "response_object"  => $response
        );
        return $result;
    }
    elsif ( $option =~ /-m/ ) {
        my $first_line = 'Maintenance\sdisks\s(\(?empty\)?)?';
        my @arr        = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );

        if ( grep( /Broken\sdisks\s?\(empty\)/, @arr ) ) {
            my @want_array;
            return \@want_array;
        }
        else {
            shift(@arr);    #First line is an empty line
            my $arr_ref     = \@arr;
            my $disk_output = _get_broken_or_partner_disks(
                "broken_or_partner_disks_ref" => $arr_ref );
            return $disk_output;
        }

    }
    elsif ( $option =~ /-S/ ) {

        # No first line to ignore
        my $first_line = '';
        my @lines      = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );

        my @vol_array = ();
        my $volume    = "";
        my %vol_hash;
        foreach my $line (@lines) {
            if ( $line =~ /Volume\s*:\s*(\w+)/i ) {
                if ( $volume ne "" ) {

                    # Copy hash contents into the volume array
                    push( @vol_array, {%vol_hash} );
                    %vol_hash = ();
                }
                $volume = $1;
                $vol_hash{volume} = $volume;
            }
            elsif ( $line =~ /User Data\s+([\d\.]+\w?B)\s+(\d+%)/i ) {
                $vol_hash{user_data}         = $1;
                $vol_hash{user_data_percent} = $2;
            }
            elsif ( $line =~ /Filesystem Metadata\s+([\d\.]+\w?B)\s+(\d+%)/i ) {
                $vol_hash{filesystem_metadata}         = $1;
                $vol_hash{filesystem_metadata_percent} = $2;
            }
            elsif ( $line =~ /Inodes\s+([\d\.]+\w?B)\s+(\d+%)/i ) {
                $vol_hash{inodes}         = $1;
                $vol_hash{inodes_percent} = $2;
            }
            elsif ( $line =~ /Snapshot Reserve\s+([\d\.]+\w?B)\s+(\d+%)/i ) {
                $vol_hash{snapshot_reserve}         = $1;
                $vol_hash{snapshot_reserve_percent} = $2;
            }
            elsif ( $line =~ /Total\s+([\d\.]+\w?B)\s+(\d+%)/i ) {
                $vol_hash{total}         = $1;
                $vol_hash{total_percent} = $2;
            }

            # The rest of the lines are table headers and can be ignored
        } ## end foreach my $line (@lines)
        if ( $volume ne "" ) {
            push( @vol_array, \%vol_hash );
        }
        return \@vol_array;
    }
    elsif ( $option =~ /-F/ ) {

        # No first line to ignore
        my $first_line = '';
        my @lines      = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );

        my @vol_array = ();
        my $volume    = "";
        my %vol_hash;
        foreach my $line (@lines) {
            if ( $line =~ /Volume\s*:\s*(\w+)/i ) {
                if ( $volume ne "" ) {

                    # Copy hash contents into the volume array
                    push( @vol_array, {%vol_hash} );
                    %vol_hash = ();
                }
                $volume = $1;
                $vol_hash{volume} = $volume;
            }
            elsif ( $line =~ /Volume Data Footprint\s+([\d\.]+\w?B)\s+(\d+%)/i )
            {
                $vol_hash{volume_data_footprint}         = $1;
                $vol_hash{volume_data_footprint_percent} = $2;
            }
            elsif ( $line =~ /Volume Guarantee\s+([\d\.]+\w?B)\s+(\d+%)/i ) {
                $vol_hash{volume_guarantee}         = $1;
                $vol_hash{volume_guarantee_percent} = $2;
            }
            elsif (
                $line =~ /Flexible Volume Metadata\s+([\d\.]+\w?B)\s+(\d+%)/i )
            {
                $vol_hash{flexible_volume_metadata}         = $1;
                $vol_hash{flexible_volume_metadata_percent} = $2;
            }
            elsif ( $line =~ /Delayed Frees\s+([\d\.]+\w?B)\s+(\d+%)/i ) {
                $vol_hash{delayed_frees}         = $1;
                $vol_hash{delayed_frees_percent} = $2;
            }
            elsif ( $line =~ /Total\s+([\d\.]+\w?B)\s+(\d+%)/i ) {
                $vol_hash{total}         = $1;
                $vol_hash{total_percent} = $2;
            }

            # The rest of the lines are table headers and can be ignored
        } ## end foreach my $line (@lines)
        if ( $volume ne "" ) {
            push( @vol_array, \%vol_hash );
        }
        return \@vol_array;
    } ## end elsif ($option =~ /-F/)
    NACL::APISet::Exceptions::InvalidParamException->throw(
        text            => "Invalid argument is sent",
        response_object => $response
    );
} ## end sub _vol_status

sub _parse_volume_data {
    my %args          = @_;
    my $vol_array_ref = $args{"volume_array_ref"};
    my $response      = $args{"response_object"};
    my @vol_array     = @$vol_array_ref;
    my ( @want_array, %volume_hash );
    foreach my $array_ref (@vol_array) {
        my @vol_data = @$array_ref;
        my %vol_info = ();
        my ( $temp, $snapshot_settings, $autosize_settings, $hybrid_cache ) =
          ( "", "", "", "" );
        my %plex_hash;
        my %raid_hash;

        # Process the first line
        if ( $vol_data[0] =~
            /(\w+(?:\(\d+\))?)\s+(\w+)\s+(raid\w+),?\s?(\w+)?\s*(.*)/ )
        {
            $vol_info{"volume"}  = $1;
            $vol_info{"state"}   = $2;
            $vol_info{"status"}  = $3 . ", " . $4;
            $vol_info{"options"} = [];
            $temp                = $5;
        }
        else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "unexpected format of output",
                response_object => $response
            );
        }

        # Now process the rest of the lines
        my $current_plex;
        my $reached_settings = 0;
        for ( my $line = 1 ; $line < @vol_data ; $line++ ) {
            next if ( $vol_data[$line] eq "" );
            if ( $vol_data[$line] =~ /([Pp]lex|RAID)/ ) {
                if ( $vol_data[$line] !~ /^RAID/ ) {
                    $vol_data[$line] =~ /.*\/(.*):(.*)/;
                    $plex_hash{"name"}   = $1;
                    $plex_hash{"status"} = $2;
                    $vol_info{"plex"}    = [ \%plex_hash ];
                    next;
                }
                else {
                    $vol_data[$line] =~ /.*\/(.*):(.*)/;
                    $raid_hash{"name"}       = $1;
                    $raid_hash{"status"}     = $2;
                    $plex_hash{"raid_group"} = [ \%raid_hash ];
                    next;
                }
            } ## end if ($vol_data[$line] =~...

            # Keep this before the next conditional because that will
            # also match
            if ( $vol_data[$line] =~ /Volume has clones:\s+(.*)/ ) {
                $vol_info{clones_list} = $1;
                next;
            }
            if ( $vol_data[$line] =~ /.*Cluster-related attributes:.*/ ) {
                $line++;
                my %info;
                my @arr;
                if ( $vol_data[$line] =~ /Owner\s+UUID:\s*(.*)/ ) {
                    $info{owner_uuid} = $1;
                    $line++;
                }
                if ( $vol_data[$line] =~ /Master\s+Data\s+Set\s+ID:\s*(.*)/ ) {
                    $info{master_data_set_id} = $1;
                    $line++;
                }
                if ( $vol_data[$line] =~ /Data\s+Set\s+ID:\s*(.*)/ ) {
                    $info{data_set_id} = $1;
                    $line++;
                }
                if ( $vol_data[$line] =~ /Mirror\s+Type:\s*(.*)/ ) {
                    $info{mirror_type} = $1;
                    $line++;
                }
                if ( $vol_data[$line] =~ /Nvfailed\s+state:\s*(.*)/ ) {
                    $info{nvfailed_state} = $1;
                    $line++;
                }
                push( @arr, \%info );
                $vol_info{"cluster_related_attributes"} = \@arr;
            } ## end if ($vol_data[$line] =~...
            if ( $vol_data[$line] =~ /.*Volume Transition Info:.*/ ) {
                $line++;
                my %info;
                my @arr;
                if ( $vol_data[$line] =~ /Copied\s+for\s+Transition:\s+(.*)/ ) {
                    $info{copied_for_trasition} = $1;
                    $line++;
                }
                if ( $vol_data[$line] =~ /Transition\s+State:\s+(.*)/ ) {
                    $info{transition_state} = $1;
                    $line++;
                }
                if ( $vol_data[$line] =~ /Transitioned:\s+(.*)/ ) {
                    $info{transitioned} = $1;
                    $line++;
                }
                push( @arr, \%info );
                $vol_info{volume_transistion_info} = \@arr;
                next;
            } ## end if ($vol_data[$line] =~...
            if ( $vol_data[$line] =~ /(.*):\s+(.*)/ ) {
                my $key   = lc($1);
                my $value = $2;
                $key =~ s/\s+/_/g;
                $vol_info{$key} = $value;
                next;
            }
            if ( $vol_data[$line] =~
                /Clone, backed by volume '(.*)', snapshot '(.*)'/ )
            {

                # This is a volume clone
                # Naming these fields similar to the CMode-CLI names
                (
                    $vol_info{'clone-parent-name'},
                    $vol_info{'clone-parent-bsnap-name'}
                ) = ( $1, $2 );
                $vol_info{'clone_volume'} = 'true';
                next;
            } ## end if ($vol_data[$line] =~...
            if ( $vol_data[$line] =~ /Snapshot\sautodelete\ssettings/ ) {
                $reached_settings = 1;
                $vol_info{"snapshot_autodelete_settings"} = {};
                next;
            }
            if ( $vol_data[$line] =~ /Volume\sautosize\ssettings/ ) {
                $reached_settings = 2;
                $vol_info{"volume_autosize_settings"} = {};
                next;
            }
            if ( $vol_data[$line] =~ /Hybrid\scache/i ) {
                $reached_settings = 3;
                $vol_info{"hybrid_cache"} = {};
                next;
            }
            if ( $reached_settings == 0 ) {
                if ( grep( /\s{2,}/, $vol_data[$line] ) ) {
                    $vol_data[$line] =~ /(.{18})(.*)/;
                    $vol_info{"status"} .= ", " . $1;
                    $temp .= $2;
                    $vol_info{"status"} =~ s/\s+$//;
                    next;
                }
                else {
                    if ( $vol_data[$line] =~ /=/ ) {
                        $temp .= $vol_data[$line];
                        next;
                    }
                    else {
                        $vol_info{"status"} .= ", " . $vol_data[$line];
                        $vol_info{"status"} =~ s/\s+$//;
                        next;
                    }
                } ## end else [ if (grep(/\s{2,}/, $vol_data...
            } ## end if ($reached_settings ...
            if ( $vol_data[$line] =~ /=/ && ( $reached_settings == 1 ) ) {
                $snapshot_settings .= $vol_data[$line] . "\n";
                next;
            }
            if ( $vol_data[$line] =~ /=/ && ( $reached_settings == 2 ) ) {
                $autosize_settings .= $vol_data[$line] . "\n";
                next;
            }
            if ( $vol_data[$line] =~ /=/ && ( $reached_settings == 3 ) ) {
                $hybrid_cache .= $vol_data[$line] . "\n";
                next;
            }
        } ## end for (my $line = 1 ; $line...

        #Process data in $temp
        $temp =~ s/,\s/,/g;

        if ( $temp eq "" ) {
            $vol_info{"options"} = [];
        }
        else {
            my %options_hash;
            foreach ( split( ",", $temp ) ) {
                if ( $_ =~ /(.*)=(.*)/ ) {
                    my $key   = $1;
                    my $value = $2;
                    $key = to_key_name( item => $key );
                    $options_hash{$key} = $value;
                }
                else {
                    my $key = $_;
                    $key = to_key_name( item => $key );
                    $options_hash{$key} = "true";
                }
            } ## end foreach (split(",", $temp))
            $vol_info{"options"} = [ \%options_hash ];
        } ## end else [ if ($temp eq "")

        if ( $reached_settings >= 1 ) {
            my %snapshot_hash;
            my %autosize_hash;
            my %hybrid_hash;

            #Process data in $snapshot_settings
            foreach ( split( "\n", $snapshot_settings ) ) {
                $_ =~ /(.*)=(.*)/;
                my $key   = $1;
                my $value = $2;
                $key = to_key_name( item => $key );
                $snapshot_hash{$key} = $value;
            }

            #Process data in $autosize_settings
            foreach ( split( "\n", $autosize_settings ) ) {
                $_ =~ /(.*)=(.*)/;
                my $key   = $1;
                my $value = $2;
                $key = to_key_name( item => $key );
                $autosize_hash{$key} = $value;
            }

            #Process data in $hybrid_cache
            foreach ( split( "\n", $hybrid_cache ) ) {
                $_ =~ /(.*)=(.*)/;
                my $key   = $1;
                my $value = $2;
                $key = to_key_name( item => $key );
                $hybrid_hash{$key} = $value;
            }
            $vol_info{"snapshot_autodelete_settings"} = [ \%snapshot_hash ];
            $vol_info{"volume_autosize_settings"}     = [ \%autosize_hash ];
            $vol_info{"hybrid_cache"} = [ \%hybrid_hash ] if $hybrid_cache;
        } ## end if ($reached_settings ...

# The data for "containing_aggregate" has extra "'" character at the front and end and that needs to be removed
        if ( $vol_info{containing_aggregate} ) {
            $vol_info{containing_aggregate} =~ s/^\'//;
            $vol_info{containing_aggregate} =~ s/\'$//;
        }
        push( @want_array, \%vol_info );
    } ## end foreach my $array_ref (@vol_array)
    return \@want_array;
} ## end sub _parse_volume_data

sub _get_spare_disks {
    my %args           = @_;
    my $pool_array_ref = $args{"spare_disks_ref"};
    my @pool_array     = @$pool_array_ref;
    my @pool_info      = ();
    my $key;
    my @heading = (
        "raid_disk", "device", "ha", "shelf", "bay", "chan", "pool", "type",
        "rpm", "used(MB/blks)", "phys(MB/blks)"
    );
    foreach my $array_ref (@pool_array) {
        my @pool_data = @$array_ref;
        my %temp_pool_hash;
        my $no_pool = 0;
        if ( grep( /Spare\sdisks\s\(empty\)/i, @pool_data ) ) {
            $pool_data[0] =~ /(Pool\d)\sspare\sdisks\s\(empty\)/;
            $key = to_key_name( item => $1 );
            $temp_pool_hash{$key} = [];
            push( @pool_info, \%temp_pool_hash );
        }
        else {
            if ( $pool_data[0] =~ /^Spare\sdisks/ ) {
                $no_pool = 1;
            }
            else {
                $pool_data[0] =~ /^(Pool\d)/;
                $key = lc($1);
                $temp_pool_hash{$key} = ();
            }

            # Remove the Pool sparse disks line, heading line and underline
            # line and the additional line coming in output which need not be
            # parsed
            splice( @pool_data, 0, 4 );
            my %spare_disks_hash;
            for ( my $count = 0 ; $count < @pool_data ; $count++ ) {
                my %disk  = ();
                my $index = 0;
                if ( $pool_data[$count] =~ s/\((.*)\)$// ) {
                    $disk{"is_zeroed"} = $1;
                }
                else {
                    $disk{"is_zeroed"} = "";
                }
                my @values = split( /\s+/, $pool_data[$count] );
                foreach (@values) {
                    $disk{ $heading[$index] } = $_;
                    $index++;
                }
                if ($no_pool) {
                    push( @pool_info, \%disk );
                }
                else {
                    push( @{ $temp_pool_hash{$key} }, \%disk );
                }

            } ## end for (my $count = 0 ; $count...
            push( @pool_info, \%temp_pool_hash ) unless ($no_pool);
        } ## end else [ if (grep(/Spare\sdisks\s\(empty\)/i...
    } ## end foreach my $array_ref (@pool_array)
    return \@pool_info;
} ## end sub _get_spare_disks

sub _get_vol_parsed {
    my %args        = @_;
    my $all_vol_ref = $args{"volume_ref"};
    my $response    = $args{"response_object"};
    my @vols        = @$all_vol_ref;
    my @vol_output;
    my ( $temp, $current_plex, $current_raid, @headings );

    foreach my $vol_ref (@vols) {
        my %each_vol = ();
        my @vol_data = @$vol_ref;
        my %disk_hash;

        # Process first line
        my $first_line = shift(@vol_data);
        $first_line =~ /(\w+)\s(\w+)\s\((.*)\)\s\((.*)\)/;
        my $key_name = $1;
        $key_name = to_key_name( item => $key_name );
        $each_vol{$key_name} = $2;
        my $temp  = $3;
        my $temp1 = $4;
        my @words = split( /,\s/, $temp );
        $each_vol{"state"} = shift(@words);
        $each_vol{"status"} = join( ", ", @words );
        $temp1 =~ /(.*)\schecksums/;
        $each_vol{"checksums"} = $1;
        my $plex_hash_ref = {};
        my $raid_hash_ref = {};

      NEXT: while ( my $line = shift(@vol_data) ) {
            if ( $line =~ /RAID\s+Disk\s+Device\s+HA\s+SHELF\s+BAY/ ) {
                my $heading_line = $line;
                @headings = split( /\s+/, $heading_line );
                for ( my $i = 0 ; $i < scalar(@headings) ; $i++ ) {
                    if ( $headings[$i] =~ /^\(/ ) {
                        $headings[ $i - 1 ] .= " " . $headings[$i];
                        splice( @headings, $i, 1 );
                        $i--;
                    }
                }
                if ( scalar(@headings) == 12 ) {
                    my $first_element = shift(@headings);
                    $headings[0] = $first_element . " " . $headings[0];
                }
                foreach my $temp_heading (@headings) {
                    $temp_heading = to_key_name( "item" => $temp_heading );
                }

                #skip the ---- line
                shift(@vol_data);
            }
            elsif ( $line =~ /([Pp]lex|RAID)/ ) {
                if ( $line =~ /Plex\s+(.*)\s+\((.*)\)/ ) {
                    $plex_hash_ref = {}
                      if exists( $plex_hash_ref->{"plex_name"} );
                    $plex_hash_ref->{"plex_name"} = $1;
                    $plex_hash_ref->{"status"}    = $2;
                    push( @{ $each_vol{"plex"} }, $plex_hash_ref );
                }
                else {
                    $line =~ /RAID\sgroup\s(.*)\s\((.*)\)/;
                    $raid_hash_ref = {}
                      if exists( $raid_hash_ref->{"raid_group_name"} );
                    $raid_hash_ref->{"raid_group_name"} = $1;
                    $raid_hash_ref->{"status"}          = $2;
                    push( @{ $plex_hash_ref->{"raid_group"} }, $raid_hash_ref );
                }
            }
            else {
                do {
                    if ( $line =~ /[Pp]lex|RAID/ ) {
                        unshift( @vol_data, $line );
                        next NEXT;
                    }
                    my @values = split( /\s+/, $line );
                    if ( scalar(@values) != scalar(@headings) ) {
                        for ( my $i = 0 ; $i < scalar(@values) ; $i++ ) {
                            if ( $values[$i] =~ /^\(/ ) {
                                my $j = 0;
                                do {
                                    $values[ $i - 1 ] .=
                                      " " . $values[ $i + $j ];
                                } while ( $values[ $i + $j++ ] !~ /\)$/ );
                                splice( @values, $i, $j );
                                $i -= $j;
                            }
                        } ## end for (my $i = 0 ; $i < scalar...
                        if ( ( scalar(@values) - scalar(@headings) ) == 1 ) {
                            my $first_element = shift(@values);
                            $values[0] = $first_element . " " . $values[0];
                        }
                    } ## end if (scalar(@values) !=...
                    if ( scalar(@values) != scalar(@headings) ) {
                        $line = shift(@vol_data)
                    }
                    my $index = 0;
                    my %per_disk;
                    foreach my $current_value (@values) {
                        $per_disk{ $headings[ $index++ ] } = $current_value;
                    }
                    push( @{ $raid_hash_ref->{"disks"} }, \%per_disk );
                } while ( $line = shift(@vol_data) );
            } ## end else [ if ($line =~ /RAID\s+Disk\s+Device\s+HA\s+SHELF\s+BAY/)
        } ## end while (my $line = shift(@vol_data...
        push( @vol_output, \%each_vol );
    } ## end foreach my $vol_ref (@vols)
    return \@vol_output;
} ## end sub _get_vol_parsed

sub _get_broken_or_partner_disks {
    my %args          = @_;
    my $disk_data_ref = $args{"broken_or_partner_disks_ref"};

    my @disk_data     = @$disk_data_ref;
    my @headings;
    my @disk_output;

    #    #Remove the first line which either says Broken disks or Partner disks
    if ( $disk_data[0] =~ /^(Broken disks|Maintenance disks)$/) {
        shift(@disk_data);
    }
    for ( my $line = 0 ; $line < @disk_data ; $line++ ) {
        next if ( $disk_data[$line] eq "" );
        if ( $disk_data[$line] =~ /RAID\s+Disk\s+Device\s+HA\s+SHELF\s+BAY/ ) {
            @headings = (
                "raid_disk", "device", "ha", "shelf", "bay", "chan", "pool",
                "type", "rpm", "used(MB/blks)", "phys(MB/blks)"
            );

            # Remove the heading and underline lines
            splice( @disk_data, $line, 2 );
            $line--;
            next;
        } ## end if ($disk_data[$line] ...
        my @values = split( /\s+/, $disk_data[$line] );
        if ( scalar(@values) - scalar(@headings) == 1 ) {

         #Takes care of case where raid_disk name is something like admin_failed
            my $first_value = shift(@values);
            $values[0] = $first_value . " " . $values[0];
        }
        my $index = 0;
        my %per_disk;
        foreach (@values) {
            $per_disk{ $headings[ $index++ ] } = $_;
        }
        push( @disk_output, \%per_disk );
    } ## end for (my $line = 0 ; $line...
    return \@disk_output;
} ## end sub _get_broken_or_partner_disks

sub _get_unassimilated_disks {
    my %args          = @_;
    my $disk_data_ref = $args{"unassimilated_disks_ref"};
	my @disk_data     = @$disk_data_ref;
    my @headings;
    my @disk_output;

    #    #Remove the first line which either says Broken disks or Partner disks
    if (  $disk_data[0] =~ (/Unassimilated/)) {
        shift(@disk_data);
    }
    for ( my $line = 0 ; $line < @disk_data ; $line++ ) {
        next if ( $disk_data[$line] eq "" );
        if ( $disk_data[$line] =~ /RAID\s+Disk\s+Device\s+HA\s+SHELF\s+BAY/ ) {
            @headings = (
                "raid_disk", "device", "ha", "shelf", "bay", "chan", "pool",
                "type", "rpm", "used(MB/blks)", "phys(MB/blks)"
            );

            # Remove the heading and underline lines
            splice( @disk_data, $line, 2 );
            $line--;
            next;
        } ## end if ($disk_data[$line] ...
        my @values = split( /\s+/, $disk_data[$line] );
        if ( scalar(@values) - scalar(@headings) == 1 ) {

         #Takes care of case where raid_disk name is something like admin_failed
            my $first_value = shift(@values);
            $values[0] = $first_value . " " . $values[0];
        }
        my $index = 0;
        my %per_disk;
        foreach (@values) {
            $per_disk{ $headings[ $index++ ] } = $_;
        }
        push( @disk_output, \%per_disk );
    } ## end for (my $line = 0 ; $line...
    return \@disk_output;
} ## end sub _get_unassimilated_disks

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error;
    if ( $raw_output =~
/vol 64bit-upgrade: Invalid operation on.+|vol move:\s+(Specified source volume is exported via NFS.|Another move operation is active.|Aggregate holding the source volume and the destination aggregate are not homogeneous|Specified source volume is not being moved|Move of specified source volume is already paused|Move of specified source volume has not been paused|Specified source volume is a flexcache volume|Specified source volume has unsupported guarantee)/
      )
    {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
        return $error;

    }
    elsif ( $raw_output =~
/(vol\s+options:\s+Operation\s+not\s+permitted\s+on\s+a\s+(hybrid\s+aggregate)|(Flash\s+Pool)|(vol.*is\s+not\s+online))/
      )
    {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    }
    elsif ( $raw_output =~
/vol\s+64bit-upgrade:\s+One\s+or\s+more\s+volumes\s+do\s+not\s+have\s+enough\s+free\s+space\s+to\s+support\s+the\s+64-bit\s+upgrade/
      )
    {
        $error = "CommandFailed";
        $$error_message =
"One or more volumes do not have enough free space to support the 64-bit upgrade";
    }
    elsif ( $raw_output =~
/(vol\s+create:\s+(\w+)\s+disks\s+needed.*,\s+but\s+not\s+enough\s+matching\s+HDD\s+disks\s+to\s+create\s+traditional\s+volume\s+are\s+available.*)| (vol\s+offline:\s+The\s+aggregate\s+is\s+iron\s+restricted,\s+this\s+operation\s+is\s+not\s+permitted)/
      )
    {
        $error          = "CommandFailed";
        $$error_message = $1;
    }
    elsif (
        $raw_output =~ /The\s+aggregate\s+is\s+not\s+currently\s+available/ )
    {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    }
    elsif ( $raw_output =~ /(Option '.+' can only be used with '.+')/ ) {

        $error          = "MissingRequiredParam";
        $$error_message = $1;
    }
    elsif (
        (
            $raw_output =~
/$command: (Couldn't mirror volume:)|(Volume .* is already mirrored.)|(Invalid operation on online volume)|(Can't find common resync snapshot)|(Invalid operation on offline volume)|(Must specify volume name for volume to be made from split plex.)|(Must specify plex to split off existing volume.)|(No volume named (.*) exists.)|vol move:\s+(Destination aggregate must be different from the source aggregate|Specified destination aggregate is not online|Specified volume is not an online FlexVol)/
        )
        || ( $raw_output =~
            /Volume option fractional.*reserve must be [either|between].*/ )
      )
    {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    }
    elsif ( $raw_output =~
/$command: (No volume name supplied)|(copy throttle \[operation_number\] throttle_value)/
      )
    {
        $$error_message = $1;
        $error          = 'MissingRequiredParam';
    }
    elsif ( $raw_output =~
/is a snapmirrored volume|Restrict it before copying to it|Neither volume local|Could not connect to filer|Cannot Init Input, aborting|Read Socket received EOF|Volume is not active|Cannot retreive the IPv6 address for the filer|Unknown resolver error|Interrupted|Destination volume.+is too small|No operations in progress|copy abort \[ all \| operation_number\]|vol wafliron start .*:\s(Cannot run wafliron on a flexible volume.|Please offline the volume)|Unable to online volume (.*)|wafliron is currently active.|vol wafliron stop (.*):/
      )
    {
        $$error_message = $raw_output;
        $error          = "CommandFailed";
    }
    elsif ( $raw_output =~ /Missing volume name/ ) {

        $$error_message = $raw_output;
        $error          = 'MissingRequiredParam';
    }
    elsif ( $raw_output =~
/Metadata-only option is not supported for flexible volume destination|option "-p inet6" is given with IPv4 address|Illegal session value|Illegal throttle value|Volume doesn't exist/
      )
    {
        $$error_message = $raw_output;
        $error          = "InvalidParamValue";
    }
    elsif ( $raw_output =~ /$command(?:[^:]*):(.*)/ ) {

        $$error_message = $1;
        if ( $$error_message =~
/(No volume name supplied)|(Missing)|(required after)|(Two volume names required)|(Specify the clone's parent volume)|(Missing\s+subcommand) | (option\s+not\s+specified)/
          )
        {
            $error = "MissingRequiredParam";
        }
        elsif ( $$error_message =~
/(Invalid)|(already exists)|(No volume named)|(cannot be destroyed)|(is an aggregate)|(Bad -t option)|(must be a multiple)|(must be greater than)|Bad value|No volume named|not found/
          )
        {
            $error = "InvalidParamValue";
        }
        elsif ( $$error_message =~
/([uU]nrecon?gi?nized option)|([Ee]xtra arguments)|(unmirrored volume uses only)|(CR_UNRECOGNIZED_ARG)/
          )
        {
            $error = "InvalidParam";
        }
        elsif ( $$error_message =~
/(not enough available space|Couldn't create|already|not enough space|[Cc]annot|temporarily busy|not allowed|only if the volume|Volume has a fixed filesystem size|must be restricted|[vV]olume is in use by other operations|error\s+starting\s+the\s+scanner)/
          )
        {
            $error = "CommandFailed";
        }
    }
    elsif ( $raw_output =~ /^usage:/mi ) {

        $error          = "CommandFailed";
        $$error_message = "Invalid switch or Invalid Param value has been sent";
    }
    elsif ( $raw_output =~
        /Clone volume is not being split from its parent|No clone status/ )
    {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    }
    elsif ( $raw_output =~ /.*not licensed/ ) {
        $error          = "FeatureNotLicensed";
        $$error_message = $raw_output;
    }
    elsif ( $raw_output =~ /(Volume .+ was not found)|(volume is not a clone)/ )
    {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    }
    elsif ( $raw_output =~
/$command: (Has extra arguments)|(No flexible volume named ['].*['] exists)|(Volume ['].*['] must be a flexible volume)|(Invalid volume name)/
      )
    {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    }
    elsif (
        $raw_output =~ /vol move: (Specified source volume is a flexclone)/ )
    {
        $error          = "InvalidParamValue";
        $$error_message = $1 . " ,source volume should be a flex volume";
    }
    elsif ( $raw_output =~ /vol wafliron:.*/ ) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    }
    elsif ( $raw_output =~ /vol wafliron stop vol.*/ ) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    } elsif ( $raw_output =~ /Volume\s+name\s+not\s+specified/ ) {
        $error          = "MissingValue";
        $$error_message = $raw_output;
    }
    return $error;
} ## end sub _check_error

## @name ::_is_pushable
##
## @summary Determines whether a new output section has begun
##
## @description
##    This method is used to find out if a new section of status output has
##    begun for vol (scrub, media_scrub or verify) status commands
##
## @synopsis
##    is_pushable(%args);
##
## @arg %args Required: The current line to be checked
## line => a line of output
##
## @return Returns 1 or undef depending on whether a new section should begin

sub _is_pushable {
    my %args = @_;
    my $line = $args{line};
    return 1 if ( $line =~ /vol ((media_)?scrub|verify)/ );
    return undef;
}

## @name ::_generic_parser_status
##
## @summary Parses the raw output for  "vol media_scrub status,
##    vol scrub status and vol verify status" commands
##
## @description
##    This method is a generic parser for "vol media_scrub status,
##    vol scrub status and vol verify status" commands
##
## @synopsis
##    _generic_parser_status(%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the filer/vsim.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array ref which contains the parsed output data. This
## array will contain references to hashes (one per section of status output)

sub _generic_parser_status {    #for vol (scrub, media_scrub and verify) status
    my %args                = @_;
    my $error_line_expected = '(?:vol (media_)?scrub status|vol verify)';

    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output        => $output,
        command           => $error_line_expected,
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $first_line = '(vol (media_)?scrub.*|vol verify .*)';
    my @lines      = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );

    my ( %hash, $verbose );

    if ( $command =~ /\s*-v\s*/ ) {
        $verbose = 1;
    }
    foreach my $line (@lines) {
        if ( _is_pushable( line => $line ) ) {
            my %hash2 = %hash;
            push( @result_array, \%hash2 ) if keys(%hash2);
            if ($verbose) {
                if ( $line =~ /status of (.*) :/ ) {
                    %hash = ( 'name' => $1 );
                }
            }
            elsif (( $line =~ /vol verify ([^ ]*) .* is (.*)/ )
                || ( $line =~ /scrub ([^ ]*).*is (.*)/ ) )
            {
                %hash = ( 'name' => $1, 'status' => $2 );
                $hash{'status'} =~ s/\.//g;
            }
        }
        else {    #other verbose mode lines
            if ( $line =~ /Current .* is (.*) complete/ ) {    # %complete
                $hash{'current_scrub'} = $1;
            }
            elsif ( $line =~ /Media scrub ([^\.]*)/ ) {    #started / suspended
                $hash{'state'} = $1;
            }
            elsif ( $line =~ /Last full .*scrub completed: (.*)/ )
            {                                              #last complete
                $hash{'last_full_scrub_completed'} = $1;
            }
            elsif ( $line =~ /Scrub is not active/ ) {     # if not active
                $hash{'current_scrub'} = 'not active';
            }
        } ## end else [ if (_is_pushable(line ...
    } ## end foreach my $line (@lines)

    push( @result_array, \%hash );                         #last entry
    return \@result_array;
} ## end sub _generic_parser_status

## @name ::vol_media_scrub_status
##
## @summary Parses the raw output from the "vol media_scrub status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_media_scrub_status with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_media_scrub_status(%args);
## INPUTS TO METHOD CALL 'vol_media_scrub_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> '-s' </td> <td> 'disk-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_media_scrub_status {
    return ( _generic_parser_status(@_) );
}

## @name ::vol_scrub_status
##
## @summary Parses the raw output from the "vol scrub status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_scrub_status with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_scrub_status(%args);
## INPUTS TO METHOD CALL 'vol_scrub_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_scrub_status {
    return ( _generic_parser_status(@_) );
}

## @name ::vol_verify_scrub_status
##
## @summary Parses the raw output from "vol verify status" command
##
## @description
##    This method can be used for parsing the output from "vol verify status"
##    command called with any of its options as described in the cdef file at
##    lib/NACL/APISet/Node/CLI/7Mode/default/vol.cdefs
##
## @synopsis
##    vol_verify_status(%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the filer/vsim.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array reference which contains the parsed output data.
## This array will contain one hash for each section of status output.

## @name ::vol_verify_status
##
## @summary Parses the raw output from the "vol verify status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_verify_status with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_verify_status(%args);
## INPUTS TO METHOD CALL 'vol_verify_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_verify_status {
    return ( _generic_parser_status(@_) );
}

## @name ::_generic_parser_scrub
##
## @summary Parses the raw output for all vol scrub sub commands except
##    vol scrub status
##
##
## @description
##    This method is a generic parser for vol scrub start, vol scrub stop,
##    vol scrub resume, vol scrub suspend
##
## @synopsis
##    _generic_parser_scrub(%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the filer/vsim.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array which contains the parsed output data. This
## array will contain one hash for each section of output.

sub _generic_parser_scrub {    #for scrub commands other than status
    my %args      = @_;
    my $can_throw = 1;

    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();
    my $error   = _check_error(
        raw_output        => $output,
        command           => 'vol scrub (?:resume|start|stop|suspend)',
        ptr_error_message => \$error_message
    );
    if ($error) {

        if (   $command =~ /vol scrub \w+\s*$/
            && $error eq 'CommandFailed'
            && ( $output =~ /already \w+/ || $output =~ /not currently \w+/ ) )
        {
            $error = undef;
        }
        else {
            $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
            $error->throw( text => $error_message, response_object => $resp );
        }
        $can_throw = 0;
    } ## end if ($error)

    my $first_line = 'vol scrub.*';

    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => $can_throw,
        response_object => $resp
    );

    foreach my $line (@lines) {
        my %hash;
        if ( $line =~ /vol scrub: (.*) scrub of (.*)/ ) {    #specific
            %hash = ( 'result' => $1, 'name' => $2 );
            push( @result_array, \%hash );
        }
        elsif ( $line =~ /vol scrub .*: (.*) scrubbing in (.*)\./ ) {    #all
            %hash = ( 'result' => $1, 'items' => $2 );
            push( @result_array, \%hash );
        }
        elsif ( $line =~ /vol scrub .*: ([^ ]*) not currently being scrubbed/ )
        {
            %hash = (
                'result' => 'not currently being scrubbed',
                'name'   => $1
            );
            $hash{'name'} =~ s/'//g;
            push( @result_array, \%hash );
        }
    } ## end foreach my $line (@lines)

    if ($error) {
        $resp->set_parsed_output( 'parsed_output' => \@result_array );
    }
    return \@result_array;
} ## end sub _generic_parser_scrub

## @name ::vol_scrub_start
##
## @summary Parses the raw output from the "vol scrub start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_scrub_start with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_scrub_start(%args);
## INPUTS TO METHOD CALL 'vol_scrub_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_scrub_start {
    return ( _generic_parser_scrub(@_) );
}

## @name ::vol_scrub_stop
##
## @summary Parses the raw output from the "vol scrub stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_scrub_stop with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_scrub_stop(%args);
## INPUTS TO METHOD CALL 'vol_scrub_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_scrub_stop {
    return ( _generic_parser_scrub(@_) );
}

## @name ::vol_scrub_suspend
##
## @summary Parses the raw output from the "vol scrub suspend" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_scrub_suspend with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_scrub_suspend(%args);
## INPUTS TO METHOD CALL 'vol_scrub_suspend' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_scrub_suspend {
    return ( _generic_parser_scrub(@_) );
}

## @name ::vol_scrub_resume
##
## @summary Parses the raw output from the "vol scrub resume" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_scrub_resume with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_scrub_resume(%args);
## INPUTS TO METHOD CALL 'vol_scrub_resume' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_scrub_resume {
    return ( _generic_parser_scrub(@_) );
}

## @name ::_generic_parser_verify
##
## @summary Parses the raw output for all vol verify sub commands except
##    vol verify status
##
##
## @description
##    This method is a generic parser for vol verify start, vol verify stop,
##    vol verify resume, vol verify suspend
##
## @synopsis
##    _generic_parser_verify(%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the filer/vsim.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object

##
## @return Returns an array which contains the parsed output data. This
## array will contain one hash for each section of output.

sub _generic_parser_verify {    #for vol verify commands other than status
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output        => $output,
        command           => 'vol verify',
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
    }

    my $first_line = 'vol verify .*';

    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        response_object => $resp
    );

    foreach my $line (@lines) {
        my %hash;
        if ( $line =~ /vol verify .*: No verify running on volume (.*)\./ ) {
            %hash = ( 'result' => 'not running', 'name' => $1 );
            $hash{'name'} =~ s/'//g;
            push( @result_array, \%hash );
        }
        elsif ( $line =~ /vol verify .*: verify (.*) on (.*)\./ ) {
            %hash = ( 'result' => $1, 'items' => $2 );
            push( @result_array, \%hash );
        }
    } ## end foreach my $line (@lines)

    if ($error) {
        $resp->set_parsed_output( 'parsed_output' => \@result_array );
        $error->throw( text => $error_message, response_object => $resp );
    }
    return \@result_array;
} ## end sub _generic_parser_verify

## @name ::vol_verify_start
##
## @summary Parses the raw output from the "vol verify start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_verify_start with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_verify_start(%args);
## INPUTS TO METHOD CALL 'vol_verify_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> '-f' </td> <td> 'plex-number' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_verify_start {
    return ( _generic_parser_verify(@_) );
}

## @name ::vol_verify_stop
##
## @summary Parses the raw output from the "vol verify stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_verify_stop with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_verify_stop(%args);
## INPUTS TO METHOD CALL 'vol_verify_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_verify_stop {
    return ( _generic_parser_verify(@_) );
}

## @name ::vol_verify_resume
##
## @summary Parses the raw output from the "vol verify resume" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_verify_resume with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_verify_resume(%args);
## INPUTS TO METHOD CALL 'vol_verify_resume' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_verify_resume {
    return ( _generic_parser_verify(@_) );
}

## @name ::vol_verify_suspend
##
## @summary Parses the raw output from the "vol verify suspend" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_verify_suspend with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_verify_suspend(%args);
## INPUTS TO METHOD CALL 'vol_verify_suspend' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_verify_suspend {
    return ( _generic_parser_verify(@_) );
}

## @name ::vol_lang
##
## @summary Parses the raw output from the "vol lang" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_lang with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_lang(%args);
## INPUTS TO METHOD CALL 'vol_lang' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'language-code' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vol_lang {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output        => $output,
        command           => 'vol lang',
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $first_line = '(Supported language codes are|Volume language is)';

    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        response_object => $resp
    );

    if ( $command =~ /vol\s+lang\s*$/ ) {
        shift(@lines);    #remove first line
        foreach my $line (@lines) {
            my %hash;
            if ( $line =~ /([^\s]+)\s+\((.*)\)$/ ) {
                $hash{'language_code'} = $1;
                $hash{'language'}      = $2;
                push( @result_array, \%hash );
            }
            else {
                last;
            }
        } ## end foreach my $line (@lines)
    }
    elsif ( $command =~ /vol\s+lang\s+[^\s]+\s*$/ ) {
        my %hash;
        foreach my $line (@lines) {
            if ( $line =~ /Volume language is (.*)/ ) {
                $hash{'volume_language'} = $1;
            }
            elsif ( $line =~ /OEM Character set is (.*)/ ) {
                $hash{'oem_character_set'} = $1;
            }
            elsif ( $line =~ /NFS Character set is (.*)/ ) {
                $hash{'nfs_character_set'} = $1;
            }
        }
        push( @result_array, \%hash );
    } ## end elsif ($command =~ /vol\s+lang\s+[^\s]+\s*$/)
    return \@result_array;
} ## end sub vol_lang

## @name ::vol_size
##
## @summary Parses the raw output from the "vol size" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_size with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_size(%args);
## INPUTS TO METHOD CALL 'vol_size' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'size' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_size {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output        => $output,
        command           => 'vol size',
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $first_line = 'vol size: Flexible volume';

    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        response_object => $resp
    );
    my %hash;
    foreach my $line (@lines) {

        if (
            $line =~ /Flexible volume '(.+)' (?:has )?size (?:set to )?(.+)\./ )
        {
            $hash{'volume'} = $1;
            $hash{'size'}   = $2;
        }
        push( @result_array, \%hash );
    }

    return \@result_array;
} ## end sub vol_size

## @name ::vol_autosize
##
## @summary Parses the raw output from the "vol autosize" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_autosize with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_autosize(%args);
## INPUTS TO METHOD CALL 'vol_autosize' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'autosize-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-shrink-threshold-percent' </td> <td> 'shrink-threshold-percent' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'increment-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'maximum-size' </td> <td> '0' </td> </tr>
## <tr> <td> 'off' </td> <td> 'off' </td> <td> '1' </td> </tr>
## <tr> <td> '-grow-threshold-percent' </td> <td> 'grow-threshold-percent' </td> <td> '0' </td> </tr>
## <tr> <td> 'on' </td> <td> 'on' </td> <td> '1' </td> </tr>
## <tr> <td> 'reset' </td> <td> 'reset' </td> <td> '1' </td> </tr>
## <tr> <td> '-minimum-size' </td> <td> 'minimum-size' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vol_autosize {
    my %args = @_;

    my ($error_message, %hash);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    if ( $output =~ /Flexible volume (?:\'|\")(.*)(?:\"|\') autosize settings (UPDATED|RESET)/i ) {
        $hash{'volume'}  = $1;
        $hash{'state'}   = $2;
    } else {
        my $error = _check_error(
            raw_output        => $output,
            command           => 'vol autosize',
            ptr_error_message => \$error_message);

        if ($error) {
            $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
            $error->throw( text => $error_message, response_object => $resp );
        }

        my @lines = get_lines(raw_output => $output);
        foreach my $line (@lines) {
            if ( $line =~ /Volume autosize is currently (.*) for volume (?:\"|\')(.+)(?:\"|\')/ ) {
                $hash{'volume'} = $2;
                $hash{'state'}  = $1;
            }
            elsif ( $line =~
        /The volume is set to grow to a maximum of\s*(\d+\s*[^\s]*)[^\d]+(\d+\s*[^\s]*)\s*(when the volume used space is above\s*(\d+%)\s*)?/
              )
            {
                $hash{'maximum-size'}           = $1;
                $hash{'increment-size'}         = $2;
                $hash{'grow-threshold-percent'} = $4 if $4;
            }
            elsif ( $line =~
        /The volume is set to shrink to a minimum of (\d+\s+\w+)? by ([\d\%]+)? when the volume used space is below ([\d\%]+)?./
              )
            {
                $hash{'minimum-size'}             = $1 if $1;
                $hash{'shrink'}                   = $2 if $2;
                $hash{'shrink-threshold-percent'} = $3 if $3;
            }
            elsif ( $line =~
                /Volume autosize for volume \'(.*)\' is currently in mode (\w+)./ )
            {
                $hash{'volume'} = $1;
                $hash{'mode'}   = $2;
            }
        }
    }
    return [ \%hash ];
} ## end sub vol_autosize

## @name ::vol_vvlabel
##
## @summary Parses the raw output from the "vol vvlabel" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_vvlabel with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_vvlabel(%args);
## INPUTS TO METHOD CALL 'vol_vvlabel' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-value' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'fieldname' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_vvlabel {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output        => $output,
        command           => "Volume vvlabel",
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my %return_hash                = ();
    my @snapshot_translation_table = ();

    # Check if the command is setting value, in such case there will not be
    # any output expected. Hence return a empty arrary reference
    if ( $command =~ /vol\svvlabel\s(.*?)\s(.*)/ ) {
        return [];
    }

    my $first_line = 'Volume name\s+\w';
    my @lines      = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );
    my $process_table;
    my $lines_to_skip;

    foreach my $line (@lines) {

        # Check the begining of snapshot translation table
        if ( $line =~ /Snapshot\stranslation\stable/i ) {
            $process_table = 1;
            $lines_to_skip = 2;
            next;
        }

        # Skip the header lines
        if ($lines_to_skip) { --$lines_to_skip; next; }

        # Process the snapshot translation table if $process_table is set
        if ($process_table) {
            if ( $line =~ /^\d+\s+\d+\s*$/ ) {
                $line =~ /(\d+)\s+(\d+)/;
                push @snapshot_translation_table,
                  { 'logical_id' => $1, 'physical_id' => $2 };
                next;
            }

            # Identify the end of snapshot translation table
            else {
                $process_table = 0 if ( $line !~ /^\d+\s+\d+\s*$/ );
            }
        } ## end if ($process_table)

        # Process the raw output for vol vvlabel attributes
        $line =~ /(.*)\s(.*?)$/;
        my $key = to_key_name( 'item' => $1 );

        # remove \'s (single quote followed by s)in the key
        # Ex: "Clone parent\'s buftree id" becomes 'clone_parent_buftree_id'
        $key =~ s/\'s//;
        my $value = $2;
        $return_hash{$key} = $value;
    } ## end foreach my $line (@lines)
    if (@snapshot_translation_table) {
        my $key = to_key_name( 'item' => "Snapshot translation table" );
        $return_hash{$key} = \@snapshot_translation_table;
    }
    return [ \%return_hash ];
} ## end sub vol_vvlabel
## @name ::vol_read_fsid
##
## @summary Parses the raw output from the "vol read_fsid" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_read_fsid with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_read_fsid(%args);
## INPUTS TO METHOD CALL 'vol_read_fsid' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_read_fsid {
    my %args = @_;

    my $error_message;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output        => $output,
        command           => 'vol read_fsid',
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %hash;

    if ( $output =~ /Volume '(.+)' (?:has an FSID of )(.+)\./ ) {
        $hash{'volume'} = $1;
        $hash{'fsid'}   = $2;
        return [ \%hash ];
    }
    else {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected output",
            response_object => $resp
        );
    }

} ## end sub vol_read_fsid
## @name ::vol_rewrite_fsid
##
## @summary Parses the raw output from the "vol rewrite_fsid" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_rewrite_fsid with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_rewrite_fsid(%args);
## INPUTS TO METHOD CALL 'vol_rewrite_fsid' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-fsid' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_rewrite_fsid {
    my %args = @_;

    my $error_message;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output        => $output,
        command           => 'vol rewrite_fsid',
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %hash;

    if ( $output =~
        /Volume (.+) (?:has had its FSID changed from )(.+)(?:\sto\s)(.+)\./ )
    {
        $hash{'volume'}   = $1;
        $hash{'old_fsid'} = $2;
        $hash{'new_fsid'} = $3;
        return [ \%hash ];
    }
    else {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected output",
            response_object => $resp
        );
    }

} ## end sub vol_rewrite_fsid
## @name ::vol_clone_create
##
## @summary Parses the raw output from the "vol clone create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_clone_create with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_clone_create(%args);
## INPUTS TO METHOD CALL 'vol_clone_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-s' </td> <td> 'space-reserve' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'clone-volume' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'parent-snapshot' </td> </tr>
## <tr> <td> '-b' </td> <td> 'parent-flex-volume' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_clone_create {
    my %args = @_;

    my $error_message;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output        => $output,
        command           => 'vol clone create',
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %hash;

    if ( $output =~ /Creation of clone volume '(.+)' (?:has completed)\./ ) {
        $hash{'clone_volume'} = $1;
        $hash{'status'}       = "created";
        return [ \%hash ];
    }
    else {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected output : " . $output,
            response_object => $resp
        );
    }
} ## end sub vol_clone_create
## @name ::vol_clone_split
##
## @summary Parses the raw output from the "vol clone split" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_clone_split with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_clone_split(%args);
## INPUTS TO METHOD CALL 'vol_clone_split' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> 'stop' </td> <td> 'stop' </td> </tr>
## <tr> <td> 'status' </td> <td> 'status' </td> </tr>
## <tr> <td> 'estimate' </td> <td> 'estimate' </td> </tr>
## <tr> <td> 'start' </td> <td> 'start' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_clone_split {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output        => $output,
        command           => $command,
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines;
    my @resultarray;
    my $first_line;
    my $can_throw = 1;
    my %hash;
    my $unexpected_output;
    if ( $command =~ /vol\sclone\ssplit\sstart\s/ ) {

        if ( $output =~
            /Clone volume '(.+)' (?:will be split from its parent)\./ )
        {
            $hash{'clone_volume'} = $1;
            $hash{'status'}       = "split started";
        }
        else {
            $unexpected_output = 1;
        }
    }
    elsif ( $command =~ /vol\sclone\ssplit\sstop\s/ ) {
        if ( $output =~ /Clone split operation will stop\./ ) {
            $hash{'status'} = "split stopped";
        }
        else {
            $unexpected_output = 1;
        }
    }
    elsif ( $command =~ /vol\sclone\ssplit\sstatus/ ) {
        $first_line = 'Volume .* (?:inodes processed)';
        @lines      = get_lines(
            "raw_output"      => $output,
            "response_object" => $resp,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
        );

        while ( my $line = shift(@lines) ) {
            my %hash;
            if ( $line =~
                /Volume '(.+)', (\d+) of (\d+) inodes processed \((\d+)%\)/ )
            {
                $hash{'clone_volume'}              = $1;
                $hash{'inodes_processed'}          = $2;
                $hash{'inodes_total'}              = $3;
                $hash{'inode_percentage_complete'} = $4;
                $line                              = shift(@lines);
                if ( $line =~ /(\d+) blocks scanned\. (\d+) blocks updated\./ )
                {
                    $hash{'blocks_scanned'} = $1;
                    $hash{'blocks_updated'} = $2;
                }
                push( @resultarray, \%hash );
            }
            else {
                $unexpected_output = 1;
            }
        } ## end while (my $line = shift(@lines...
        return \@resultarray;
    }
    elsif ( $command =~ /vol\sclone\ssplit\sestimate\s/ ) {
        $first_line = 'An estimated .+';
        @lines      = get_lines(
            "raw_output"      => $output,
            "response_object" => $resp,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
        );
        if ( $lines[0] =~ /An estimated (.+) (?:available storage.+)/ ) {
            $hash{'split_estimate'} = $1;
            $lines[1] =~ /clone volume '(.+)' (?:from its parent)\./;
            $hash{'clone_volume'} = $1;
        }
        else {
            $unexpected_output = 1;
        }
    } ## end elsif ($command =~ /vol\sclone\ssplit\sestimate\s/)

    if ($unexpected_output) {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected output : " . $output,
            response_object => $resp
        );
    }

    return [ \%hash ];
} ## end sub vol_clone_split

## @name ::vol_clone_split_start
##
## @summary Parses the raw output from the "vol clone split start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_clone_split_start with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_clone_split_start(%args);
## INPUTS TO METHOD CALL 'vol_clone_split_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_clone_split_start {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $result;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "vol clone split",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $result = vol_clone_split( response_object => $resp );
    return $result;
} ## end sub vol_clone_split_start

## @name ::vol_clone_split_stop
##
## @summary Parses the raw output from the "vol clone split stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_clone_split_stop with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_clone_split_stop(%args);
## INPUTS TO METHOD CALL 'vol_clone_split_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_clone_split_stop {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $result;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "vol clone split",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $result = vol_clone_split( response_object => $resp );
    return $result;
} ## end sub vol_clone_split_stop

## @name ::vol_clone_split_status
##
## @summary Parses the raw output from the "vol clone split status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_clone_split_status with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_clone_split_status(%args);
## INPUTS TO METHOD CALL 'vol_clone_split_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_clone_split_status {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $result;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "vol clone split",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $result = vol_clone_split( response_object => $resp );
    return $result;
} ## end sub vol_clone_split_status

## @name ::vol_clone_split_estimate
##
## @summary Parses the raw output from the "vol clone split estimate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_clone_split_estimate with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_clone_split_estimate(%args);
## INPUTS TO METHOD CALL 'vol_clone_split_estimate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vol_clone_split_estimate {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $result;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "vol clone split",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $result = vol_clone_split( response_object => $resp );
    return $result;
} ## end sub vol_clone_split_estimate
## @name ::vol_64bit_upgrade_start
##
## @summary Parses the raw output from the "vol 64bit-upgrade start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_64bit_upgrade_start with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_64bit_upgrade_start(%args);
## INPUTS TO METHOD CALL 'vol_64bit_upgrade_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> '-mode' </td> <td> 'mode' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = vol 64bit-upgrade start v32_a32 -mode check
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'volume' => 'v32_a32',
##&nbsp;            'used' => '15MB',
##&nbsp;            'capacity' => 'Running',
##&nbsp;            'total' => '4208KB',
##&nbsp;            'available' => '20%'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vol 64bit-upgrade start v32_a32 -mode normal
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'volume' => 'v32_a32',
##&nbsp;            'status' => 'Started 64-bit upgrade'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub vol_64bit_upgrade_start {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output        => $output,
        command           => 'vol 64bit-upgrade',
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ( $output =~ /(Started 64-bit upgrade) on volume (.+)/ ) {

        # For -mode normal
        push @result_array, { status => $1, volume => $2 };
    }
    else {

        # For -mode check

        #######################################################################
        # Get the output from the headings line onwards, get the headings line
        # (this is the first line), and then split on two or more spaces
        # to get the heading names.
        # Since the output in CMode is the same as 7Mode except for there being
        # an additional "Vserver" field, we handle both in this parser. In
        # 7Mode the headings line starts with "Volume Name", in CMode it
        # starts with "Vserver"
        #######################################################################
        my ( $output_at_end, $output_from_header );
        my $output_from_header_till_end = snip_only_required_output(
            raw_output => $output,
            delimiter  => '(Volume Name|Vserver)'
        );
        if ( $output_from_header_till_end =~ /((.|\s)+%)((.|\s)+)/ ) {
            $output_from_header = $1;
            $output_at_end      = $3;
        }
        my @header_lines = split /\n/, $output_from_header;
        my $header_line  = $header_lines[0];
        my @fields       = split /\s{2,}/, $header_line;

        #######################################################################
        # Get the substring from the last occurence of "--" till the "%" and
        # then split on one or more spaces to get the values
        #######################################################################
        my $index_of_last_hyphens = rindex $output_from_header, '--';
        my $index_of_percent = index $output_from_header, '%';
        my $length_to_extract = $index_of_percent - $index_of_last_hyphens + 1;
        my $string = substr $output_from_header, $index_of_last_hyphens,
          $length_to_extract;
        my @values = split /\s+/, $string;

        # Shift out the first element which will be "--"
        shift @values;

        # If the number of fields and values do not match, throw an
        # UnexpectedOutputException
        if ( @fields != @values ) {
            my $text =
                "Unexpected output format!\n "
              . "Number of fields does not match number of values.\n"
              . "Fields are:\n"
              . Dumper( \@fields )
              . "Values are:\n"
              . Dumper( \@values );
            NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                text            => $text,
                response_object => $resp
            );
        } ## end if (@fields != @values)

        my %hash;
        while (@fields) {
            $hash{ to_key_name( item => shift @fields ) } = shift @values;
        }

        #####################################################################
        # The earlier version of the parser stored the volume name in a key
        # called "volume", but the above logic would end up storing it under
        # a "volume_name" key. For backwards compatibility, we rename this
        # key to "volume".
        #####################################################################
        if ( exists $hash{volume_name} ) {
            $hash{volume} = delete $hash{volume_name};
        }
        $output_at_end = snip_only_required_output(
            raw_output => $output_at_end,
            delimiter  => '(Volume Name|Vserver)'
        );

        # Only when one or more volumes are 100% full,we get this additional
        # useful data. Only if this output is available, proceed to parse
        # this part of the output.

        # search for the last character of 'B'. The part till this is useful.
        if ( $output_at_end =~ /((.|\s)*\d+.?B)/ ) {
            $output_at_end = $1;
            my @header_lines = split /\r*\n+/, $output_at_end;
            my $header_line  = $header_lines[0];
            my @fields       = split /\s{2,}/, $header_line;

            #######################################################################
            # Get the substring from the last occurence of "--" till the "B" and
            # then split on one or more spaces to get the values
            #######################################################################
            my $index_of_last_hyphens = rindex $output_at_end, '--';
            my $index_of_B = index $output_at_end, 'B';
            my $length_to_extract = $index_of_B - $index_of_last_hyphens + 1;
            my $string = substr $output_at_end, $index_of_last_hyphens,
              $length_to_extract;
            my @values = split /\s+/, $string;

            # Shift out the first element which will be "--"
            shift @values;

            # If the number of fields and values do not match, throw an
            # UnexpectedOutputException
            if ( @fields != @values ) {
                my $text =
                    "Unexpected output format!\n "
                  . "Number of fields does not match number of values.\n"
                  . "Fields are:\n"
                  . Dumper( \@fields )
                  . "Values are:\n"
                  . Dumper( \@values );
                NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text            => $text,
                    response_object => $resp
                );
            } ## end if (@fields != @values)
            my %extra_info;
            while (@fields) {
                $extra_info{ to_key_name( item => shift @fields ) } =
                  shift @values;
            }
            #####################################################################
            # The earlier version of the parser stored the volume name in a key
            # called "volume", but the above logic would end up storing it under
            # a "volume_name" key. For backwards compatibility, we rename this
            # key to "volume".
            #####################################################################
            if ( exists $extra_info{volume_name} ) {
                $extra_info{volume} = delete $extra_info{volume_name};
            }
            if ( $extra_info{volume} =~ /(.*)\((\*)\)/ ) {
                $extra_info{volume} = $1;
                $extra_info{"filesys_size_fixed"} = "true";
            }
            %hash = ( %hash, %extra_info );
        } ## end if ($output_at_end =~ ...

        @result_array = ( \%hash );
    } ## end else [ if ($output =~ /(Started 64-bit upgrade) on volume (.+)/)

    return \@result_array;
} ## end sub vol_64bit_upgrade_start

## @name ::vol_copy_start
##
## @summary Parses the raw output from the "vol copy start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_copy_start with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_copy_start(%args);
## INPUTS TO METHOD CALL 'vol_copy_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'source' </td> </tr>
## <tr> <td> '-S' </td> <td> 'all-snapshots' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination' </td> </tr>
## <tr> <td> '-s' </td> <td> 'snapshot-name' </td> </tr>
## <tr> <td> '-M' </td> <td> 'manual-cutover' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns undef

sub vol_copy_start {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my @result_array;

    my $first_line = qr/(vol copy restore)|(vol copy dump)/;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "vol copy start",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ( $raw_output !~ /vol copy restore (\d+) : (100\s*%\s*done)/m ) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => "Volume copy was not successful",
            response_object => $resp
        );
    }
    my $can_throw = 1;
    my @lines     = get_lines(
        "raw_output"      => $raw_output,
        "first_line"      => $first_line,
        "can_throw"       => $can_throw,
        "response_object" => $resp,
    );
    my $length;
    for ( $length = scalar(@lines) - 1 ; $length >= 0 ; --$length ) {
        my %hash;
        if ( $lines[$length] =~
/vol copy restore (\d+) : (\d+\s*% done). Estimate (\d+) minutes remaining./
          )
        {
            $hash{'operation_no'} = $1;
            $hash{'status'}       = $2;
            push( @result_array, \%hash );
            last;
        }
    } ## end for ($length = scalar(@lines...
    return \@result_array;
} ## end sub vol_copy_start

## @name ::vol_copy_status
##
## @summary Parses the raw output from the "vol copy status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_copy_status with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_copy_status(%args);
## INPUTS TO METHOD CALL 'vol_copy_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'operation-number' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = vol copy status
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'begun, 0 MB to be copied',
##&nbsp;            'operation_no' => '0'
##&nbsp;          }
##&nbsp;        ];

sub vol_copy_status {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my @result_array;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "vol copy status",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $first_line = qr/(vol copy restore)|(vol copy dump)/;
    my $can_throw  = 1;
    my @lines      = get_lines(
        "raw_output"      => $raw_output,
        "first_line"      => $first_line,
        "can_throw"       => $can_throw,
        "response_object" => $resp,
    );

    foreach my $line (@lines) {
        my %hash;
        if ( $line =~
/vol copy restore (\d+) : (begun, .+ to be copied)|(\d+% done, .+ copied.)/
          )
        {
            $hash{'operation_no'} = $1;
            $hash{'status'}       = $2;
            push( @result_array, \%hash );
        }
        elsif ( $line =~
/vol copy dump (\d+) : (\d+\s*% done). Estimate (\d+) minutes remaining./
          )
        {
            $hash{'operation_no'}                   = $1;
            $hash{'status'}                         = $2;
            $hash{'estimate_time_remaining_in_min'} = $3;
            push( @result_array, \%hash );
        }
        elsif ( $line =~
/vol copy restore (\d+) : (\d+\s*% done). Estimate (\d+) minutes remaining/
          )
        {
            $hash{'operation_no'}                   = $1;
            $hash{'status'}                         = $2;
            $hash{'estimate_time_remaining_in_min'} = $3;
            push( @result_array, \%hash );
        }

    } ## end foreach my $line (@lines)
    return \@result_array;
} ## end sub vol_copy_status

## @name ::vol_copy_abort
##
## @summary Parses the raw output from the "vol copy abort" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_copy_abort with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_copy_abort(%args);
## INPUTS TO METHOD CALL 'vol_copy_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'operation-number' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = vol copy abort
##&nbsp;$VAR1 =  [
##&nbsp;          {
##&nbsp;            'status' => 'aborted',
##&nbsp;            'operation_no' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'aborted',
##&nbsp;            'operation_no' => '1'
##&nbsp;          }
##&nbsp;        ];

sub vol_copy_abort {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my @result_array;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "vol copy abort",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $first_line = "image operation";
    my $can_throw  = 1;
    my @lines      = get_lines(
        "raw_output"      => $raw_output,
        "first_line"      => $first_line,
        "can_throw"       => $can_throw,
        "response_object" => $resp
    );

    foreach my $line (@lines) {
        my %hash;
        if ( $line =~
            /image operation (\d+) (is now being aborted|already being aborted)/
          )
        {
            $hash{'operation_no'} = $1;
            $hash{'status'}       = "aborted";
            push( @result_array, \%hash );
        }
    } ## end foreach my $line (@lines)
    return \@result_array;
} ## end sub vol_copy_abort

## @name ::vol_copy_throttle
##
## @summary Parses the raw output from the "vol copy throttle" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_copy_throttle with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_copy_throttle(%args);
## INPUTS TO METHOD CALL 'vol_copy_throttle' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'throttle-value' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'operation-number' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = vol copy throttle
##&nbsp;$VAR1 =   [
##&nbsp;          {
##&nbsp;            'operation_no' => '0',
##&nbsp;            'throttle_adjusted_to' => '40%'
##&nbsp;          }
##&nbsp;        ];

sub vol_copy_throttle {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my @result_array;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "vol copy throttle",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $first_line = "copy operation";
    my $can_throw  = 1;
    my @lines      = get_lines(
        "raw_output"      => $raw_output,
        "first_line"      => $first_line,
        "can_throw"       => $can_throw,
        "response_object" => $resp
    );

    foreach my $line (@lines) {
        my %hash;
        if ( $line =~
/copy operation (\d+): Throttle adjusted from \d+% to (\d+%) of full speed/
          )
        {
            $hash{'operation_no'}         = $1;
            $hash{'throttle_adjusted_to'} = $2;
            push( @result_array, \%hash );
        }
    } ## end foreach my $line (@lines)
    return \@result_array;
} ## end sub vol_copy_throttle

## @name ::vol_container
##
## @summary Parses the raw output from the "vol container" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_container with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_container(%args);
## INPUTS TO METHOD CALL 'vol_container' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = vol container vol1
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'volume' => 'vol1',
##&nbsp;            'vol_container' => 'aggr1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub vol_container {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my @result_array;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "vol container",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %hash;
    if ( $raw_output =~
        /Volume\s+['](.*)[']\s+is\s+contained\s+in\s+aggregate\s+['](.*)[']/ )
    {
        $hash{'volume'}        = $1;
        $hash{'vol_container'} = $2;
        push( @result_array, \%hash );
    }
    else {
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "unexpected format of output",
            response_object => $resp
        );
    }
    return \@result_array;
} ## end sub vol_container

## @name ::vol_move_start
##
## @summary Parses the raw output from the "vol move start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_move_start with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_move_start(%args);
## INPUTS TO METHOD CALL 'vol_move_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-m' </td> <td> 'manual-cut-over' </td> </tr>
## <tr> <td> '-o' </td> <td> 'ignore-redundancy' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> '-r' </td> <td> 'cutover-attempts' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination' </td> </tr>
## <tr> <td> '-w' </td> <td> 'cutover-window' </td> </tr>
## <tr> <td> '-k' </td> <td> 'keep-source' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;$VAR1 = [];

sub vol_move_start {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output        => $output,
        command           => "vol move start",
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return ( \@result_array );
} ## end sub vol_move_start

## @name ::vol_move_pause
##
## @summary Parses the raw output from the "vol move pause" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_move_pause with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_move_pause(%args);
## INPUTS TO METHOD CALL 'vol_move_pause' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;$VAR1 = [];
sub vol_move_pause {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output        => $output,
        command           => "vol move pause",
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return ( \@result_array );
} ## end sub vol_move_pause

## @name ::vol_move_resume
##
## @summary Parses the raw output from the "vol move resume" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_move_resume with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_move_resume(%args);
## INPUTS TO METHOD CALL 'vol_move_resume' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-m' </td> <td> 'manual-cut-over' </td> </tr>
## <tr> <td> '-o' </td> <td> 'ignore-redundancy' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> '-r' </td> <td> 'cutover-attempts' </td> </tr>
## <tr> <td> '-w' </td> <td> 'cutover-window' </td> </tr>
## <tr> <td> '-k' </td> <td> 'keep-source' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;$VAR1 = [];
sub vol_move_resume {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output        => $output,
        command           => "vol move resume",
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return ( \@result_array );
} ## end sub vol_move_resume

## @name ::vol_move_abort
##
## @summary Parses the raw output from the "vol move abort" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_move_abort with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_move_abort(%args);
## INPUTS TO METHOD CALL 'vol_move_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;$VAR1 = [];
sub vol_move_abort {
    my %args = @_;
    my $result;
    my $error_message;
    my @array;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "vol move abort",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return \@array;
} ## end sub vol_move_abort

## @name ::vol_move_status
##
## @summary Parses the raw output from the "vol move status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_move_status with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_move_status(%args);
## INPUTS TO METHOD CALL 'vol_move_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'cutover_attempts' => '3',
##&nbsp;             'source' => 'apiset_volume',
##&nbsp;             'cutover_time' => '60',
##&nbsp;             'destination' => 'apiset_aggr2',
##&nbsp;             'state' => 'setup'
##&nbsp;           }
##&nbsp;         ];

sub vol_move_status {
    my %args = @_;
    my $result;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "vol move status",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my %info = ();
    my @array;
    my @arr1;
    my $value;
    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "response_object" => $resp
    );

    # No output
    if ( !@arr ) {
        return [];
    }

    if ( $arr[0] =~ /Source\s+Destination\s+CO\s+Attempts\s+CO\s+Time\s+State/ )
    {
        shift(@arr);
        foreach $value (@arr) {
            if ( $value =~ /(\w+)\s+(\w+)\s+(\d+)\s+(\d+)\s+(\w+)/ ) {
                $info{"source"}           = $1;
                $info{"destination"}      = $2;
                $info{"cutover_time"}     = $4;
                $info{"state"}            = $5;
                $info{"cutover_attempts"} = $3;
            }
        }
    }
    elsif ( $arr[0] =~ /^Source:\s+\w+/i ) {
        foreach $value (@arr) {
            if ( $value =~ /(\w+\s+\w+)\:\t+(.*)/ ) {
                $info{ to_key_name( item => $1 ) } = $2;
            }
            elsif ( $value =~ /(\w+):\t+(.*)/ ) {
                $info{ to_key_name( item => $1 ) } = $2;
            }
            elsif ( $value =~ /Last\s+Completed\s+Transfer\:/i ) {
                next;
            }
            elsif ( $value =~
/Data\s+Transferred\s+=\s+(\d+\s+\w+)\sTime\sTaken\s=\s(\d+\s+\w+)/i
              )
            {
                my %info1 = ();
                $info1{"data_transferred"} = $1;
                $info1{"time_taken"}       = $2;
                push( @arr1, \%info1 );
                $info{"last_completed_transfer"} = \@arr1;
            }
            elsif ( $value =~ /Current\s+Transfer\s+Size\s+=\s+(\d+\s+\w+)/i ) {
                $info{"current_transfer_size"} = $1;
            }
        } ## end foreach $value (@arr)
    }
    else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "Output is not in the expected format",
            response_object => $resp
        );
    }

    push( @array, \%info );
    return \@array;
} ## end sub vol_move_status

## @name ::vol_move_cutover
##
## @summary Parses the raw output from the "vol move cutover" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_move_cutover with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_move_cutover(%args);
## INPUTS TO METHOD CALL 'vol_move_cutover' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> '-w' </td> <td> 'cutover-window' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'mode' => 'automatic',
##&nbsp;             'vol_move' => 'started'
##&nbsp;           }
##&nbsp;         ];

sub vol_move_cutover {
    my %args = @_;
    my $result;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my @array;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "vol move cutover",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %info = ();
    if ( $raw_output =~ /vol\s+move\:\s+move\s+started\s+in\s+automatic.*/i ) {
        $info{"vol_move"} = "started";
        $info{"mode"}     = "automatic";
    }
    elsif ( $raw_output =~
        /vol\s+move\:\s+Move\s+of\s+specified\s+source\s+volume\s+is.*/i )
    {
        $info{"vol_move"} = "started";
        $info{"mode"}     = "manual";
    }
    else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "Output is not in the expected format",
            response_object => $resp
        );
    }
    push( @array, \%info );
    return \@array;
} ## end sub vol_move_cutover

## @name ::vol_add
##
## @summary Parses the raw output from the "vol add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_add with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_add(%args);
## INPUTS TO METHOD CALL 'vol_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-d' </td> <td> 'disks' </td> </tr>
## <tr> <td> '-g' </td> <td> 'raid-group' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-count' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> '-n' </td> <td> 'is-display-command' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
#&nbsp;vol add vol2 -d v5.22
#&nbsp;$VAR1 = [
#&nbsp;          {
#&nbsp;             'disks' => [
#&nbsp;                          'v5.22'
#&nbsp;                       ],
#&nbsp;             'volume' => 'vol2',
#&nbsp;             'command' => 'vol add vol2 -n -d v5.22',
#&nbsp;             'force' => 1
#&nbsp;           }
#&nbsp;         ];

sub vol_add {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output        => $output,
        command           => "vol add",
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $regex_check =
        'The\s+disk.*\s+need.*\s+to\s+be\s+zeroed\s+before\s+.*\s+can'
      . '\s+be\s+added\s+to\s+the\s+volume\.\s+The\s+process\s+'
      . 'has\s+been\s+initiated';
    my $regex_check1 =
        '(\d+)\s+disk.*\s+need.*\s+to\s+be\s+zeroed\s+before\s+.*\s+can'
      . '\s+be\s+added\s+to\s+the\s+volume\.\s+The\s+process\s+'
      . 'has\s+been\s+initiated';
    if ( $output =~
        /vol add\s+([^\s]+)\s+(-f\s+)?(-g\s+(.*)\s+)?-d\s+([^\r\n]*)/ )
    {
        my %hash = ( 'volume' => $1 );
        $hash{'force'}      = 1  if $2;
        $hash{'raid-group'} = $4 if $3;
        my @disks = split( /\s+/, $5 );
        $hash{'disks'} = \@disks;
        $output =~ /(vol add\s.*-d[^\r\n]*)/;
        $hash{'command'} = $1;
        @result_array = ( \%hash );
    }
    elsif ( $output =~ /$regex_check/ ) {
        @result_array = ( { 'result' => 'initiated' } );
    }
    elsif ( $output =~ /$regex_check1/ ) {
        @result_array = ( { 'disks' => $1, 'result' => 'initiated' } );
    }
    elsif ( $output =~ /Addition of (\d+) disk.* to the volume has completed/ )
    {
        @result_array = ( { 'disks' => $1, 'result' => 'completed' } );
    }
    else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "Output is not in the expected format",
            response_object => $resp
        );
    }

    return \@result_array;
} ## end sub vol_add

## @name ::vol_mirror
##
## @summary Parses the raw output from the "vol mirror" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_mirror with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_mirror(%args);
## INPUTS TO METHOD CALL 'vol_mirror' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-d' </td> <td> 'disks' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> '-v' </td> <td> 'victim-volume' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## <tr> <td> '-n' </td> <td> 'is-display-command' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
#&nbsp;vol mirror Test_vol -f  -d v5.24 v5.22
#&nbsp;$VAR1 = [
#&nbsp;         {
#&nbsp;             'disks' => '2',
#&nbsp;             'result' => 'initiated'
#&nbsp;           }
#&nbsp;         ];
#&nbsp;vol mirror Test_vol -f -v Newvol
#&nbsp;$VAR1 = []

sub vol_mirror {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output        => $output,
        command           => "vol mirror",
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $regex_check =
'Creation\sof\sa\smirror\splex\shas\sbeen\sinitiated.\s+(\d+)\sdisks\sneed\sto\sbe'
      . '\szeroed\sbefore\sthey\scan\sbe\sadded\sto\sthe\svolume.\sThe\sprocess\shas\sbeen\sinitiated.';
    if ( $output =~ /vol mirror\s+([^\s]+)\s+(-f\s+)?-d\s+([^\r\n]*)/ ) {
        my %hash = ( 'volume' => $1 );
        $hash{'force'} = 1 if $2;
        my @disks = split( /\s+/, $3 );
        $hash{'disks'} = \@disks;
        $output =~ /(vol mirror\s.*-d[^\r\n]*)/;
        $hash{'command'} = $1;
        @result_array = ( \%hash );
    }
    elsif ( $output =~ /$regex_check/ ) {
        @result_array = ( { 'disks' => $1, 'result' => 'initiated' } );
    }
    elsif ( $output =~
        /Creation\sof\sa\smirror\splex\swith\s(\d+)\sdisks\shas\scompleted./ )
    {
        @result_array = ( { 'disks' => $1, 'result' => 'completed' } );
    }
    elsif ( $output eq "" ) {
        @result_array = ();
    }
    else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "Output is not in the expected format",
            response_object => $resp
        );
    }

    return \@result_array;
} ## end sub vol_mirror

## @name ::vol_split
##
## @summary Parses the raw output from the "vol split" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_split with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_split(%args);
## INPUTS TO METHOD CALL 'vol_split' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-volume-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = vol split /Test/plex0 Test1
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'old_volume' => 'Test',
##&nbsp;            'new_volume' => 'Test1',
##&nbsp;            'result' => 'completed'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub vol_split {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output        => $output,
        command           => "vol split",
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ( $output =~
/Split\sof\sformerly-mirrored\svolume\s(.*)\scompleted,\snew\svolume\s(.*)\screated\./
      )
    {
        @result_array = (
            {
                'old_volume' => $1,
                'new_volume' => $2,
                'result'     => 'completed'
            }
        );
        return \@result_array;
    }
    NACL::APISet::Exceptions::UnexpectedOutputException->throw(
        text            => "output is not in the expected format",
        response_object => $resp
    );
} ## end sub vol_split

## @name ::vol_wafliron_start
##
## @summary Parses the raw output from the "vol wafliron start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_wafliron_start with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_wafliron_start(%args);
## INPUTS TO METHOD CALL 'vol_wafliron_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-i' </td> <td> 'inject-error' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> '-prev_cp' </td> <td> 'check-revert-snapshot' </td> </tr>
## <tr> <td> '-o' </td> <td> 'commit-mode' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-list-to-iron' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = vol wafliron start vol_wafl
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'wafliron started on volume vol_wafl'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub vol_wafliron_start(%) {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command      = $response->get_command();
    my $error_level  = $response->get_execution_status();
    my @result_array = ();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    my $value;
    my @lines =
      get_lines( raw_output => $raw_output, response_object => $response );
    my %info;
    my ( $val, $val1 );
    foreach $value (@lines) {
        if ( $value =~ /vol wafliron start (\w+): (.*)./i ) {
            $val = "wafliron started on" . " " . "volume" . " " . $1;
        }
        elsif ( $value =~ /(Volume '\w+' is now online)./i ) {
            $val1 = $1;
        }
        if ($val1) {
            $info{"status"} = $val . " " . "and" . " " . $val1;
        }
        else {
            $info{"status"} = $val;
        }
    } ## end foreach $value (@lines)
    push( @result_array, \%info ) if (%info);
    return \@result_array;
} ## end sub vol_wafliron_start(%)

## @name ::vol_wafliron_status
##
## @summary Parses the raw output from the "vol wafliron status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_wafliron_status with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_wafliron_status(%args);
## INPUTS TO METHOD CALL 'vol_wafliron_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = vol wafliron status
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vol_wafliron_status' => 'no waflirons in progress',
##&nbsp;            'vol_wafliron_status_vol_wafl' => 'wafliron is not currently active'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub vol_wafliron_status(%) {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command      = $response->get_command();
    my $error_level  = $response->get_execution_status();
    my @result_array = ();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    my $value;
    my @lines =
      get_lines( raw_output => $raw_output, response_object => $response );
    my %info;
    foreach $value (@lines) {
        if ( $value =~ /Optional commit mode is (\w+)/i ) {
            $info{"optional_commit_mode"} = $1;
        }
        elsif ( $value =~ /Status files redirected to volume (\w+)/i ) {
            $info{"status_files_redirected_volume"} = $1;
        }
        elsif ( $value =~
            /(\d+)\s+files,\s+(\d+)\s+blks\s+total,\s+(\d+)\s+blks\s+used/i )
        {
            $info{"num_files"}  = $1;
            $info{"total_blks"} = $2;
            $info{"blks_used"}  = $3;
        }
        elsif ( $value =~
            /Status of the previous attempt to start wafliron on test:\s+(.*)./i
          )
        {
            $info{"status_of_the_previous_attempt"} = $1;
        }
        elsif ( $value =~ /vol wafliron status (\w+):\s+(.*)./i ) {
            $info{ "vol_wafliron_status_" . $1 } = $2;
        }
        elsif ( $value =~ /vol wafliron status:\s+(.*)/i ) {
            $info{"vol_wafliron_status"} = $1;
        }
    } ## end foreach $value (@lines)
    push( @result_array, \%info );
    return \@result_array;
} ## end sub vol_wafliron_status(%)

## @name ::vol_wafliron_stop
##
## @summary Parses the raw output from the "vol wafliron stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_wafliron_stop with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_wafliron_stop(%args);
## INPUTS TO METHOD CALL 'vol_wafliron_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vol_wafliron_stop {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_message;
    my $error;
    $error = _check_error(
        raw_output        => $output,
        ptr_error_message => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub vol_wafliron_stop

## @name ::vol_batched_free_log_show_early_kickout_queue
##
## @summary Parses the raw output from the "vol batched_free_log show_early_kickout_queue" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_batched_free_log_show_early_kickout_queue with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_batched_free_log_show_early_kickout_queue(%args);
## INPUTS TO METHOD CALL 'vol_batched_free_log_show_early_kickout_queue' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vol_batched_free_log_show_early_kickout_queue {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_message;
    my $error;
    $error = _check_error(
        raw_output        => $output,
        ptr_error_message => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end vol_batched_free_log_show_early_kickout_queue

## @name ::vol_batched_free_log_status
##
## @summary Parses the raw output from the "vol batched_free_log status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_batched_free_log_status with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_batched_free_log_status(%args);
## INPUTS TO METHOD CALL 'vol_batched_free_log_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = system node run -node local -command "priv set -q test; vol batched_free_log status vol0;
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Batched_free_log is disabled for vol0
##&nbsp;Active index is 0, Backend state is IDLE
##&nbsp;Chunk size is 1310720 blocks
##&nbsp;Upper Cap is 2.00% of 1505175 blocks of volume space
##&nbsp;* 0 blocks used out of 30103 blocks available (0%)
##&nbsp;* 0% (0 blocks) / 20% (6020 blocks) before active log ejection
##&nbsp;Early Kickouts 0
##&nbsp;
##&nbsp;   Active Log blks       Inactive Log blks Sorted Log blks
##&nbsp;===---PROGRESS---=======================================================================
##&nbsp;Sorts Requested                       0                       0                     N/A
##&nbsp;Sorts Completed                       0                       0                     N/A
##&nbsp;Frees Completed                     N/A                     N/A                       0
##&nbsp;===---SPACE---==========================================================================
##&nbsp;Size                        0                       0                       0
##&nbsp;L0s                        0                       0                       0
##&nbsp;Indirects                       0                       0                       0
##&nbsp;Total Blocks                       0                       0                       0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'batched_free_log' => 'disabled',
##&nbsp;            'volume' => 'vol0',
##&nbsp;            'blocks_available' => '30103',
##&nbsp;            'blocks_available_percent' => '0%',
##&nbsp;            'blocks_used' => '0',
##&nbsp;            'chunk_size' => '1310720 blocks',
##&nbsp;            'progress' => [
##&nbsp;                            {
##&nbsp;                              'frees_completed' => [
##&nbsp;                                                     {
##&nbsp;                                                       'inactive_log_blks' => 'N/A',
##&nbsp;                                                       'sorted_log_blks' => '0',
##&nbsp;                                                       'active_log_blks' => 'N/A'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;                              'sorts_requested' => [
##&nbsp;                                                     {
##&nbsp;                                                       'inactive_log_blks' => '0',
##&nbsp;                                                       'sorted_log_blks' => 'N/A',
##&nbsp;                                                       'active_log_blks' => '0'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;                              'sorts_completed' => [
##&nbsp;                                                     {
##&nbsp;                                                       'inactive_log_blks' => '0',
##&nbsp;                                                       'sorted_log_blks' => 'N/A',
##&nbsp;                                                       'active_log_blks' => '0'
##&nbsp;                                                     }
##&nbsp;                                                   ]
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;            'upper_cap' => '2.00%',
##&nbsp;            'backend_state' => 'IDLE',
##&nbsp;            'space' => [
##&nbsp;                         {
##&nbsp;                           'total_blocks' => [
##&nbsp;                                               {
##&nbsp;                                                 'inactive_log_blks' => '0',
##&nbsp;                                                 'sorted_log_blks' => '0',
##&nbsp;                                                 'active_log_blks' => '0'
##&nbsp;                                               }
##&nbsp;                                             ],
##&nbsp;                           'indirects' => [
##&nbsp;                                            {
##&nbsp;                                              'inactive_log_blks' => '0',
##&nbsp;                                              'sorted_log_blks' => '0',
##&nbsp;                                              'active_log_blks' => '0'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                           'l0s' => [
##&nbsp;                                      {
##&nbsp;                                        'inactive_log_blks' => '0',
##&nbsp;                                        'sorted_log_blks' => '0',
##&nbsp;                                        'active_log_blks' => '0'
##&nbsp;                                      }
##&nbsp;                                    ],
##&nbsp;                           'size' => [
##&nbsp;                                       {
##&nbsp;                                         'inactive_log_blks' => '0',
##&nbsp;                                         'sorted_log_blks' => '0',
##&nbsp;                                         'active_log_blks' => '0'
##&nbsp;                                       }
##&nbsp;                                     ]
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'active_log_ejection_ratio' => '0% (0 blocks) / 20% (6020 blocks)',
##&nbsp;            'active_index' => '0',
##&nbsp;            'volume_space' => '1505175 blocks',
##&nbsp;            'early_kickouts' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub vol_batched_free_log_status {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_message;
    my $error;
    $error = _check_error(
        raw_output        => $output,
        ptr_error_message => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines =
     get_lines( raw_output => $output, response_object => $resp );
    my ( @result_array, @main_head, @values );
    my ($count,$limit) = 0;

    foreach my $line (@lines) {
       if($line =~ /(Batched_free_log)\s+is\s+(\w+)\s+and\s+\w+\s+for\s+(\S+)/){
          $result_array[$count]->{ to_key_name( item => $1 ) } = $2;
          $result_array[$count]->{volume} = $3;
       }elsif ( $line =~ /(Active.*)\s+\w+\s+(\d+)\,\s+(Backend.*)\s+\w+\s(\S+(\s+\S+)?)/ ) {
          $result_array[$count]->{ to_key_name( item => $1 ) } = $2;
          $result_array[$count]->{ to_key_name( item => $3 ) } = $4;
       } elsif ( $line =~ /(Chunk.*)\s+\w+\s+(\d+.*)/ ) {
           $result_array[$count]->{ to_key_name( item => $1 ) } = $2;
       } elsif (
           $line =~ /(Upper.*)\s+\w+\s+(\S+)\s+\w+\s+(\d+\s+\w+)\s+\w+\s+(.*)/ ) {
           $result_array[$count]->{ to_key_name( item => $1 ) } = $2;
           $result_array[$count]->{ to_key_name( item => $4 ) } = $3;
       } elsif ( $line =~ /\*\s+(\d+)\s+(blocks\s+used).*\w+\s+(\d+)\s+(blocks\s+available)\s+\((\S+)\)/) {
           $result_array[$count]->{ to_key_name( item => $2 ) } = $1;
           $result_array[$count]->{ to_key_name( item => $4 ) } = $3;
           $result_array[$count]->{blocks_available_percent} = $5;
       } elsif ( $line =~ /\*\s+(.*\))\s+.*/ ) {
           $result_array[$count]->{active_log_ejection_ratio} = $1;
		   $result_array[$count]->{active_log_ejection_ratio} =~ /\/.+\((\d+) blocks/;
           $result_array[$count]->{merge_blocks} = $1;
       } elsif ( $line =~ /(Early.*)\s+(\d+)/ ) {
           $result_array[$count]->{ to_key_name( item => $1 ) } = $2;
       } elsif ( $line =~ /Active\s+Log\s+blks.*/ ) {
           @main_head = split( /\s\s+/, $line );
       } elsif (
           $line =~ /(Sorts\s+Requested|Sorts\s+Completed|Frees\s+Completed).*/ ) {
           my $sub_head = $1;
           @values = split( /\s\s+/, $line );
           foreach my $head (@main_head) {
               $limit++
                 if ( $line =~
                   /^(Sorts\s+Requested|Sorts\s+Completed|Frees\s+Completed)/ );
               $result_array[$count]{progress}[0]
                 ->{ to_key_name( item => $sub_head ) }
                 ->[0]{ to_key_name( item => $head ) } = $values[$limit];
           }
           $limit = 0;
       } elsif ( $line =~ /^(\*Overwrite|\*Hole\s+Punch|\*Zombie|Size|L0s|Indirects|Total\s+Blocks)/ ) {
           my $sub_head = $1;
           @values = split( /\s\s+/, $line );
           foreach my $head (@main_head) {
               $limit++ if ( $line =~ /^(\*Overwrite|\*Hole\s+Punch|\*Zombie|Size|L0s|Indirects|Total\s+Blocks)/ );
               $result_array[$count]{space}[0]
                 ->{ to_key_name( item => $sub_head ) }
                 ->[0]{ to_key_name( item => $head ) } = $values[$limit];
           }
           $limit = 0;
        } elsif ( $line =~ /Overwrites.*/ ) {
           @main_head = split( /\s\s+/, $line );
		}elsif ( $line =~ /^(Blocks\s+Used|Insertions\s+Charged|Target\s+Frees\s+\(Committed\)|Target\s+Frees\s+\(Background\)|Inline\s+Frees\s+\(Total\)|Suspensions\s+\(Total\))/ ) {
           my $sub_head = $1;
           @values = split( /\s\s+/, $line );
           foreach my $head (@main_head) {
               $limit++ if ( $line =~ /^(Blocks\s+Used|Insertions\s+Charged|Target\s+Frees\s+\(Committed\)|Target\s+Frees\s+\(Background\)|Inline\s+Frees\s+\(Total\)|Suspensions\s+\(Total\))/ );
               $result_array[$count]{inbound}[0]
                 ->{ to_key_name( item => $sub_head ) }
                 ->[0]{ to_key_name( item => $head ) } = $values[$limit];
           }
           $limit = 0;
		} elsif ( $line =~ /^(Committed\s+Frees|Background\s+Frees)/ ) {
           my $sub_head = $1;
           @values = split( /\s\s+/, $line );
           foreach my $head (@main_head) {
               $limit++ if ( $line =~ /^(Committed\s+Frees|Background\s+Frees)/ );
               $result_array[$count]{outbound}[0]
                 ->{ to_key_name( item => $sub_head ) }
                 ->[0]{ to_key_name( item => $head ) } = $values[$limit];
           }
           $limit = 0;
       }
	   
    }
  return \@result_array;

} ## end vol_batched_free_log_status

## @name ::vol_batched_free_log_suspend
##
## @summary Parses the raw output from the "vol batched_free_log suspend" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_batched_free_log_suspend with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_batched_free_log_suspend(%args);
## INPUTS TO METHOD CALL 'vol_batched_free_log_suspend' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vol_batched_free_log_suspend {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_message;
    my $error;
    $error = _check_error(
        raw_output        => $output,
        ptr_error_message => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end vol_batched_free_log_suspend

## @name ::vol_batched_free_log_resume
##
## @summary Parses the raw output from the "vol batched_free_log resume" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_batched_free_log_resume with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_batched_free_log_resume(%args);
## INPUTS TO METHOD CALL 'vol_batched_free_log_resume' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vol_batched_free_log_resume {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_message;
    my $error;
    $error = _check_error(
        raw_output        => $output,
        ptr_error_message => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end vol_batched_free_log_resume

## @name ::vol_batched_free_log_dump
##
## @summary Parses the raw output from the "vol batched_free_log dump" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_batched_free_log_dump with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_batched_free_log_dump(%args);
## INPUTS TO METHOD CALL 'vol_batched_free_log_dump' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vol_batched_free_log_dump {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_message;
    my $error;
    $error = _check_error(
        raw_output        => $output,
        ptr_error_message => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end vol_batched_free_log_dump

## @name ::vol_batched_free_log_drain_start
##
## @summary Parses the raw output from the "vol batched_free_log drain start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_batched_free_log_drain_start with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_batched_free_log_drain_start(%args);
## INPUTS TO METHOD CALL 'vol_batched_free_log_drain_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vol_batched_free_log_drain_start {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_message;
    my $error;
    $error = _check_error(
        raw_output        => $output,
        ptr_error_message => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end vol_batched_free_log_drain_start

## @name ::vol_batched_free_log_drain_stop
##
## @summary Parses the raw output from the "vol batched_free_log drain stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_batched_free_log_drain_stop with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_batched_free_log_drain_stop(%args);
## INPUTS TO METHOD CALL 'vol_batched_free_log_drain_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vol_batched_free_log_drain_stop {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_message;
    my $error;
    $error = _check_error(
        raw_output        => $output,
        ptr_error_message => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end vol_batched_free_log_drain_stop

## @name ::vol_batched_free_log_drain_status
##
## @summary Parses the raw output from the "vol batched_free_log drain status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_batched_free_log_drain_status with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_batched_free_log_drain_status(%args);
## INPUTS TO METHOD CALL 'vol_batched_free_log_drain_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vol_batched_free_log_drain_status {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_message;
    my $error;
    $error = _check_error(
        raw_output        => $output,
        ptr_error_message => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end vol_batched_free_log_drain_status

## @name ::vol_batched_free_log_dump
##
## @summary Parses the raw output from the "vol batched_free_log dump" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_batched_free_log_dump with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_batched_free_log_dump(%args);
## INPUTS TO METHOD CALL 'vol_batched_free_log_dump' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vol_batched_free_log_dump {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_message;
    my $error;
    my %act_log_path;
    my @result_array;
    my $apiset = $args{apiset};
    $error = _check_error(
        raw_output        => $output,
        ptr_error_message => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($output =~ /at\:\s+(.*)/) {
        $act_log_path{path} = $1;
    }

    push @result_array, \%act_log_path;
    return \@result_array;
} ## end vol_batched_free_log_drain_status

## @name ::vol_clear_error
##
## @summary Parses the raw output from the "vol clear_error" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vol_clear_error with any of its options as described in the
##    corresponding cdef file 'vol.cdefs'
## @synopsis
##      vol_clear_error(%args);
## INPUTS TO METHOD CALL 'vol_clear_error' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = system node run -node local -command "priv set -q test; vol clear_error source -inaccessible;
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Starting scan on source, logfile /etc/log/WAFL_vbntoino.source.20140505061114
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'Starting scan on source',
##&nbsp;            'log_file' => '/etc/log/WAFL_vbntoino.source.20140505061114',
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub vol_clear_error {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_message;
    my $error;
    my @result_array;
    $error = _check_error(
        raw_output        => $output,
        ptr_error_message => \$error_message,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ($output =~ m/(Starting\s+scan\s+on\s+\S+)\,\s+logfile\s+(\S+)/i) {
        $result_array[0] = {
            status => $1,
            log_file => $2
        };
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => "Output is not in the expected format",
            response_object => $resp
        );
    }
    return \@result_array;
} ## end vol_clear_error

1;

