# $Id$
#
#Copyright (c) 2012-13 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode wafl commands
##
## @description
##    This is a library to parse the raw output of 7Mode wafl commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "wafl scan speed" will be wafl_scan_speed()
##
## @synopsis
##    use wafl;
##
##    wafl_ec_invalidate(%args);
##    wafl_ec_lookup(%args);
##    wafl_scan_speed(%args);
##    wafl_showvbns(%args);
##    wafl_scan_status(%args);
##    wafl_dlog_info(%args);
##    wafl_flush_cache(%args);
##    wafl_scan_kireeti(%args);
##    wafl_scan_fold(%args);
##    wafl_scan_ownblocks_calc(%args);
##    wafl_nvlog_chk(%args);
##    wafl_scan_measure_layout(%args);
##    wafl_stats_zero(%args);
##    wafl_steal_stats(%args);
##    wafl_scan_abort(%args);
##    wafl_touch(%args);
##    wafl_hya_purge_cache(%args);
##    wafl_hya_ssd_remove(%args);
##    wafl_hya_cache_remove(%args);
##    wafl_hya_ssdsnap_autodel(%args);
##    wafl_hya_ssd_lookup(%args);
##    wafl_hya_dump_ssd_range(%args);
##    wafl_hya_ssd_pvbn_info(%args);
##    wafl_hya_verify_cache(%args);
##    wafl_hya_cbi_show(%args);
##    wafl_hya_cache_lookup(%args);
##    wafl_hyastats(%args);
##    wafl_scribble_buf(%args);
##    wafl_buftree_check(%args);
##    wafl_scan_redirect(%args);
##    wafl_stats(%args);
##    wafl_inconsistent(%args);
##    wafl_scan_containerind(%agrs);
##    wafl_scan_pvbn_check(%agrs);
##    wafl_scan_i2p(%agrs);
##    wafl_scan_blk_reclaim(%agrs);
##    wafl_punch(%agrs);
##    wafl_cpstats(%agrs);
##    wafl_eject_inode(%agrs);
##    wafl_query_list(%agrs);
##    wafl_query_interact(%agrs);
##    wafl_nv_replay_stats(%agrs);
##    wafl_catalog_stats(%agrs);
##    wafl_swarmstats(%agrs);
##    wafl_fspool_create(%agrs);
##    wafl_fspool_list(%agrs);
##    wafl_fspool_destroy(%agrs);
##    wafl_fspool_mkdefault(%agrs);
##    wafl_fspool_resize(%agrs);
##    wafl_composite_stats_counter_show(%args);
##    wafl_composite_stats_show(%args);
##    wafl_composite_temp(%args);
##
## @author kathar.hidayath@netapp.com,
##         ksmahesh@netapp.com,
##         govinda1@netapp.com
##         tanil@netapp.com
##         udaykk@netapp.com
##         skotagir@netapp.com
##         anupamap@netapp.com
##         pbala@netapp.com
##         dl-nacl-dev@netapp.com
##         wosser@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::wafl;

use strict;
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

## @name ::wafl_ec_lookup
##
## @summary Parses the raw output from the "wafl ec lookup" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_ec_lookup with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_ec_lookup(%args);
## INPUTS TO METHOD CALL 'wafl_ec_lookup' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggr' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'pvbn' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; wafl ec lookup daggr1 874463777; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Aggregate: daggr1, pvbn: 874463777 found at ecbn 16388360, cache object 0
##&nbsp;  Tag detail: pvol_indx:3 ref:2 lock:0 used:1 fbn:18446744073709551615 btid:0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lock' => '0',
##&nbsp;            'ref' => '2',
##&nbsp;            'pvol_indx' => '3',
##&nbsp;            'used' => '1',
##&nbsp;            'fbn' => '18446744073709551615',
##&nbsp;            'ecbn' => '16388360',
##&nbsp;            'cache_obj' => '0',
##&nbsp;            'btid' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_ec_lookup {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    my ($error, @temp, @result_array);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }
    my @lines = get_lines(raw_output      => $raw_output,
                          response_object => $resp);

    my %hash;
    # array @lines will have 2 lines if EC is enabled porperly. Otherwise empty arrayref will be returned.
    foreach my $line (@lines) {
        if ($line =~ /found at ecbn (\d+), cache object (\d+)/) {
            $hash{ecbn} = $1;
            $hash{cache_obj} = $2;
        } elsif ($line =~ /pvol_indx:(\d+) ref:(\d+) lock:(\d+) used:(\d+) fbn:(\d+) btid:(\d+)/) {
            $hash{pvol_indx} = $1;
            $hash{ref} = $2;
            $hash{lock} = $3;
            $hash{used} = $4;
            $hash{fbn} = $5;
            $hash{btid} = $6;
        }
    }
    push(@result_array, \%hash);

    return \@result_array;
}

## @name ::wafl_ec_invalidate
##
## @summary Parses the raw output from the "wafl ec invalidate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_ec_invalidate with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_ec_invalidate(%args);
## INPUTS TO METHOD CALL 'wafl_ec_invalidate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'pvbn' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggr' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub wafl_ec_invalidate {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }

    return [];
}

## @name ::wafl_ec_ilog_info
##
## @summary Parses the raw output from the "wafl ec ilog_info" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_ec_ilog_info with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_ec_ilog_info(%args);
## INPUTS TO METHOD CALL 'wafl_ec_ilog_info' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggr' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub wafl_ec_ilog_info {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }

    return [];
}

## @name ::wafl_ec_fake_hybrid
##
## @summary Parses the raw output from the "wafl ec fake_hybrid" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_ec_fake_hybrid with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_ec_fake_hybrid(%args);
## INPUTS TO METHOD CALL 'wafl_ec_fake_hybrid' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggr' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub wafl_ec_fake_hybrid {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }

    return [];
}

## @name ::wafl_ec_print_trace
##
## @summary Parses the raw output from the "wafl ec print_trace" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_ec_print_trace with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_ec_print_trace(%args);
## INPUTS TO METHOD CALL 'wafl_ec_print_trace' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub wafl_ec_print_trace {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }

    return [];
}

## @name ::wafl_ec_print_io
##
## @summary Parses the raw output from the "wafl ec print_io" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_ec_print_io with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_ec_print_io(%args);
## INPUTS TO METHOD CALL 'wafl_ec_print_io' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub wafl_ec_print_io {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }

    return [];
}

## @name ::wafl_ec_stats
##
## @summary Parses the raw output from the "wafl ec stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_ec_stats with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_ec_stats(%args);
## INPUTS TO METHOD CALL 'wafl_ec_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; wafl ec stats; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Cache write I/O statistics
##&nbsp;==========================
##&nbsp;blocks:              11648
##&nbsp;chains:                182
##&nbsp;solitary:                0
##&nbsp;avg chainlength:        64
##&nbsp;chainlength histogram: [182 entries]
##&nbsp;        0        1        2        3        4        5        6        7
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED
##&nbsp;      182        0        0        0        0        0        0        0
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;Average:       64.00
##&nbsp;avg latency:             3 ms
##&nbsp;latency histogram (msec): [182 entries]
##&nbsp;        0        1        2        3        4        5        6        7
##&nbsp;        2       33       36       29       22       14       10       12
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED
##&nbsp;       24        0        0        0        0        0        0        0
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;Average:        3.92
##&nbsp;
##&nbsp;Cache read I/O statistics
##&nbsp;=========================
##&nbsp;blocks:             525468
##&nbsp;chains:               8584
##&nbsp;solitary:              303
##&nbsp;avg chainlength:        61
##&nbsp;chainlength histogram: [8584 entries]
##&nbsp;        0        1        2        3        4        5        6        7
##&nbsp;        0      303       24        1        1        1       37        2
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED
##&nbsp;        3        6        6        8        0        0        0        0
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED
##&nbsp;     8192        0        0        0        0        0        0        0
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;Average:       61.21
##&nbsp;avg latency:             0 ms
##&nbsp;latency histogram (msec): [392 entries]
##&nbsp;        0        1        2        3        4        5        6        7
##&nbsp;      193      184       11        4        0        0        0        0
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;Average:        0.55
##&nbsp;I/O Hit/Miss rates
##&nbsp;blocks hit in external cache:
##&nbsp;               level:             0             1             2             3
##&nbsp;                type
##&nbsp;         normal file:          1380             0             0             0
##&nbsp;       active bitmap:             3             0             0             0
##&nbsp;      vvol container:             0             1             0             0
##&nbsp;
##&nbsp;blocks miss in external cache:
##&nbsp;               level:             0             1             2             3
##&nbsp;                type
##&nbsp; old/new inomap file:             9             0             0             0
##&nbsp;         normal file:           600          6962            76             1
##&nbsp;           directory:            27            11             0             0
##&nbsp;        special file:             0            48             0             0
##&nbsp;       active bitmap:          1927            54             8             0
##&nbsp;        snap summary:            63            42             8             0
##&nbsp;         block types:          5141            30             8             0
##&nbsp;            spacemap:             8             8             0             0
##&nbsp;            metafile:            63             0             0             0
##&nbsp;      vvol container:             2           195             4             0
##&nbsp;  private inode file:           131            56            28             0
##&nbsp;   public inode file:          1244            69            12             1
##&nbsp;              fsinfo:            41             0             0             0
##&nbsp;          delete log:             0            98            12            12
##&nbsp;          purge file:             1            15             1             0
##&nbsp;       overflow file:             0             1             0             0
##&nbsp;   private inode map:             9             0             0             0
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'blocks_hit_in_external_cache' => [
##&nbsp;                                                {
##&nbsp;                                                  'normal_file' => '1380',
##&nbsp;                                                  'vvol_container' => '0',
##&nbsp;                                                  'active_bitmap' => '3'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'normal_file' => '0',
##&nbsp;                                                  'vvol_container' => '1',
##&nbsp;                                                  'active_bitmap' => '0'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'normal_file' => '0',
##&nbsp;                                                  'vvol_container' => '0',
##&nbsp;                                                  'active_bitmap' => '0'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'normal_file' => '0',
##&nbsp;                                                  'vvol_container' => '0',
##&nbsp;                                                  'active_bitmap' => '0'
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;            'blocks_miss_in_external_cache' => [
##&nbsp;                                                 {
##&nbsp;                                                   'directory' => '27',
##&nbsp;                                                   'normal_file' => '600',
##&nbsp;                                                   'vvol_container' => '2',
##&nbsp;                                                   'snap_summary' => '63',
##&nbsp;                                                   'delete_log' => '0',
##&nbsp;                                                   'block_types' => '5141',
##&nbsp;                                                   'purge_file' => '1',
##&nbsp;                                                   'private_inode_map' => '9',
##&nbsp;                                                   'old/new_inomap_file' => '9',
##&nbsp;                                                   'public_inode_file' => '1244',
##&nbsp;                                                   'private_inode_file' => '131',
##&nbsp;                                                   'active_bitmap' => '1927',
##&nbsp;                                                   'special_file' => '0',
##&nbsp;                                                   'metafile' => '63',
##&nbsp;                                                   'spacemap' => '8',
##&nbsp;                                                   'fsinfo' => '41',
##&nbsp;                                                   'overflow_file' => '0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'directory' => '11',
##&nbsp;                                                   'normal_file' => '6962',
##&nbsp;                                                   'vvol_container' => '195',
##&nbsp;                                                   'snap_summary' => '42',
##&nbsp;                                                   'delete_log' => '98',
##&nbsp;                                                   'block_types' => '30',
##&nbsp;                                                   'purge_file' => '15',
##&nbsp;                                                   'private_inode_map' => '0',
##&nbsp;                                                   'old/new_inomap_file' => '0',
##&nbsp;                                                   'public_inode_file' => '69',
##&nbsp;                                                   'private_inode_file' => '56',
##&nbsp;                                                   'active_bitmap' => '54',
##&nbsp;                                                   'special_file' => '48',
##&nbsp;                                                   'metafile' => '0',
##&nbsp;                                                   'spacemap' => '8',
##&nbsp;                                                   'fsinfo' => '0',
##&nbsp;                                                   'overflow_file' => '1'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'directory' => '0',
##&nbsp;                                                   'normal_file' => '76',
##&nbsp;                                                   'vvol_container' => '4',
##&nbsp;                                                   'snap_summary' => '8',
##&nbsp;                                                   'delete_log' => '12',
##&nbsp;                                                   'block_types' => '8',
##&nbsp;                                                   'purge_file' => '1',
##&nbsp;                                                   'private_inode_map' => '0',
##&nbsp;                                                   'old/new_inomap_file' => '0',
##&nbsp;                                                   'public_inode_file' => '12',
##&nbsp;                                                   'private_inode_file' => '28',
##&nbsp;                                                   'active_bitmap' => '8',
##&nbsp;                                                   'special_file' => '0',
##&nbsp;                                                   'metafile' => '0',
##&nbsp;                                                   'spacemap' => '0',
##&nbsp;                                                   'fsinfo' => '0',
##&nbsp;                                                   'overflow_file' => '0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'directory' => '0',
##&nbsp;                                                   'normal_file' => '1',
##&nbsp;                                                   'vvol_container' => '0',
##&nbsp;                                                   'snap_summary' => '0',
##&nbsp;                                                   'delete_log' => '12',
##&nbsp;                                                   'block_types' => '0',
##&nbsp;                                                   'purge_file' => '0',
##&nbsp;                                                   'private_inode_map' => '0',
##&nbsp;                                                   'old/new_inomap_file' => '0',
##&nbsp;                                                   'public_inode_file' => '1',
##&nbsp;                                                   'private_inode_file' => '0',
##&nbsp;                                                   'active_bitmap' => '0',
##&nbsp;                                                   'special_file' => '0',
##&nbsp;                                                   'metafile' => '0',
##&nbsp;                                                   'spacemap' => '0',
##&nbsp;                                                   'fsinfo' => '0',
##&nbsp;                                                   'overflow_file' => '0'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;            'cache_write_io_statistics' => [
##&nbsp;                                            {
##&nbsp;                                             'blocks' => '11648',
##&nbsp;                                             'solitary' => '0',
##&nbsp;                                             'avg_chainlength' => '64',
##&nbsp;                                             'chains' => '182'
##&nbsp;                                            }
##&nbsp;                                           ],
##&nbsp;            'cache_read_io_statistics' => [
##&nbsp;                                           {
##&nbsp;                                            'blocks' => '525468',
##&nbsp;                                            'solitary' => '303',
##&nbsp;                                            'avg_chainlength' => '61',
##&nbsp;                                            'chains' => '8584'
##&nbsp;                                           }
##&nbsp;                                          ],
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_ec_stats {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }
    my @lines = get_lines(raw_output      => $raw_output,
                          response_object => $resp);

    my (%hash, @result_array);

    @lines = grep(!/====/, @lines);
    # # Get rid of empty lines
    # @lines = grep(!/====/, @lines);

    my $line = shift(@lines);
    # get Cache Write IO Statistics
    $line = shift(@lines);
    while(($line !~ /chainlength histogram:/i) and scalar(@lines) > 0) {
        if ($line =~ /^(.+):\s+(\d+)/) {
            my $key = to_key_name(item => $1);
            $hash{cache_write_io_statistics}->[0]->{$key} = $2;
        }
        $line = shift(@lines);
    }

    # Not parsing histograms. skipping it
    while (($line !~ /Cache read I\/O statistics/i) and scalar(@lines) > 0) {
        $line = shift(@lines);
    }

    # get Cache Read IO Statistics
    while(($line !~ /chainlength histogram:/i) and scalar(@lines) > 0) {
        if ($line =~ /^(.+):\s+(\d+)/) {
            my $key = to_key_name(item => $1);
            $hash{cache_read_io_statistics}->[0]->{$key} = $2;
        }
        $line = shift(@lines);
    }

    # Not parsing histograms. skipping it
    while (($line !~ /I\/O Hit\/Miss rates/i) and scalar(@lines) > 0) {
        $line = shift(@lines);
    }

    # Get the data for BLOCK_HIT_IN_EXTERNAL_CACHE
    shift(@lines) if($lines[0] =~ /blocks hit in external cache:/);

    # Get the number of level
    $line = shift(@lines);
    my ($levels) = ($line =~ /\s*level:\s+(.*)$/);
    my @levels = split(/\s+/, $levels);

    shift(@lines) if($lines[0] =~ /^\s*type\s+$/);

    while (($line !~ /blocks miss in external cache:/i) and scalar(@lines) > 0) {
        $line = shift(@lines);
        if ($line =~ /\s*(.+):\s+(.*)$/) {
            my $key = to_key_name(item => $1);
            my @hit = split(/\s+/, $2);
            foreach my $level (@levels) {
                $hash{blocks_hit_in_external_cache}->[$level]->{$key} = $hit[$level];
            }
        }
    }

    # Get the data for BLOCK_MISS_IN_EXTERNAL_CACHE
    # Get the number of level
    $line = shift(@lines);
    ($levels) = ($line =~ /\s*level:\s+(.*)$/);
    @levels = split(/\s+/, $levels);

    shift(@lines) if($lines[0] =~ /^\s*type\s+$/);

    while (scalar(@lines) > 0) {
        $line = shift(@lines);
        if ($line =~ /\s*(.+):\s+(.*)$/) {
            my $key = to_key_name(item => $1);
            my @miss = split(/\s+/, $2);
            foreach my $level (@levels) {
                $hash{blocks_miss_in_external_cache}->[$level]->{$key} = $miss[$level];
            }
        }
    }

    push(@result_array, \%hash);

    return \@result_array;
}

## @name ::wafl_ec_stats_zero
##
## @summary Parses the raw output from the "wafl ec stats_zero" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_ec_stats_zero with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_ec_stats_zero(%args);
## INPUTS TO METHOD CALL 'wafl_ec_stats_zero' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub wafl_ec_stats_zero {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }

    return [];
}

## @name ::wafl_ec_reconstruct
##
## @summary Parses the raw output from the "wafl ec reconstruct" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_ec_reconstruct with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_ec_reconstruct(%args);
## INPUTS TO METHOD CALL 'wafl_ec_reconstruct' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub wafl_ec_reconstruct {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }

    return [];
}

## @name ::wafl_ec_reset
##
## @summary Parses the raw output from the "wafl ec reset" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_ec_reset with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_ec_reset(%args);
## INPUTS TO METHOD CALL 'wafl_ec_reset' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub wafl_ec_reset {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }

    return [];
}

## @name ::wafl_ec_snapshot
##
## @summary Parses the raw output from the "wafl ec snapshot" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_ec_snapshot with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_ec_snapshot(%args);
## INPUTS TO METHOD CALL 'wafl_ec_snapshot' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub wafl_ec_snapshot {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }

    return [];
}

## @name ::wafl_ec_spcs_stats
##
## @summary Parses the raw output from the "wafl ec spcs_stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_ec_spcs_stats with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_ec_spcs_stats(%args);
## INPUTS TO METHOD CALL 'wafl_ec_spcs_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub wafl_ec_spcs_stats {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }

    return [];
}

## @name ::wafl_scan_speed
##
## @summary Parses the raw output from the "wafl scan speed" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_scan_speed with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_scan_speed(%args);
## INPUTS TO METHOD CALL 'wafl_scan_speed' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'speed' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub wafl_scan_speed(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    if ($command=~/^wafl scan speed$/){
        $first_line="WAFL";
    } else {
        $first_line="";
    }
    my $can_throw=1;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'error_message'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line);
    if (scalar(@lines)) {
        return (_wafl_scan_speed('lines'=>\@lines,'response_obj'=>$resp));
    } else {
        if($first_line eq ""){
            my @return_array;
            my %hash;
            $hash{'result'}= 'successful';
            push (@return_array,\%hash);
            return \@return_array;
        } else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(text=>"Unexpected form of output",response_object => $resp);
        }
    }
}# end of function wafl_scan_speed


sub _wafl_scan_speed(%){
    my %args=@_;
    my @lines=@{$args{'lines'}};
    my $resp=$args{'response_obj'};
    my @returnarray;
    my %hash;
    while (my $line=shift(@lines)){
        if($line=~/WAFL scan speed is (.*)$/){
            $hash{'wafl_scan_speed'} = $1;
            push(@returnarray,\%hash);
        } else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(text=>"Unexpected form of output",response_object => $resp);
        }
    }
    return \@returnarray;
}

## @name ::wafl_showvbns
##
## @summary Parses the raw output from the "wafl showvbns" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_showvbns with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_showvbns(%args);
## INPUTS TO METHOD CALL 'wafl_showvbns' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'suppress-duplicate-lines' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'end-fbn' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'load-bufs-parent' </td> <td> '1' </td> </tr>
## <tr> <td> '-z' </td> <td> 'ignore-holes' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'fbn' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'refcounts' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'cached-ssd-vbn' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'level-check' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'volume-aggregate' </td> <td> '1' </td> </tr>
## <tr> <td> '-S' </td> <td> 'snap-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'bin-object-slot-id' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'pvbn-from-container' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'fsinfo-volinfo-vbns' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'inofile-vbns' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = priv set -q diag; wafl showvbns -v -s /vol/vol0; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;fbn 0 is vbn 230303, vvbn 129271, not cached on SSD, refcount 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cached_vbn' => '-',
##&nbsp;            'refcount' => '0',
##&nbsp;            'vvbn' => '129271',
##&nbsp;            'fbn' => '0',
##&nbsp;            'vbn' => '230303'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; wafl showvbns -z /vol/vol0; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;fbn 0 is vbn 230303, vvbn 129271
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vvbn' => '129271',
##&nbsp;            'fbn' => '0',
##&nbsp;            'vbn' => '230303'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; wafl showvbns -c -v /vol/vol0; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;fbn 0 is vbn 230303, vvbn 129271, not cached on SSD
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cached_vbn' => '-',
##&nbsp;            'vvbn' => '129271',
##&nbsp;            'fbn' => '0',
##&nbsp;            'vbn' => '230303'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; wafl showvbns -f /vol/vol0; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;FSinfo: fbn 0 is vbn 1722229, vvbn 3458764513820810530
##&nbsp;Volinfo: fbn 0 is vbn 3559018, vvbn 3458764513820810531
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vvbn' => '3458764513820810530',
##&nbsp;            'fbn' => '0',
##&nbsp;            'vbn' => '1722229'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'vvbn' => '3458764513820810531',
##&nbsp;            'fbn' => '0',
##&nbsp;            'vbn' => '3559018'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; wafl showvbns -m -b /vol/vol0; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;fbn 0 is vbn 8609(Bin: 0) vbn_cont 35184372088832(Bin: 1 OID: 0 Slot: 0), vvbn 359210
##&nbsp;fbn 1 is vbn 8610(Bin: 0) vbn_cont 35184372088833(Bin: 1 OID: 0 Slot: 1), vvbn 359211
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vvbn' => '359210',
##&nbsp;            'fbn' => '0',
##&nbsp;            'vbn' => '8609'
##&nbsp;            'vbn_cont' => '35184372088832',
##&nbsp;            'bin0' => '1'
##&nbsp;            'bin1' => '1'
##&nbsp;            'oid' => '0'
##&nbsp;            'slot' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'vvbn' => '359211',
##&nbsp;            'fbn' => '1',
##&nbsp;            'vbn' => '8610'
##&nbsp;            'vbn_cont' => '35184372088833',
##&nbsp;            'bin0' => '1'
##&nbsp;            'bin1' => '1'
##&nbsp;            'oid' => '0'
##&nbsp;            'slot' => '1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_showvbns {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }
    $first_line = 'fbn\s+\d+\s+is\s+\w+\s+.*[,]\s+\w+\s+.*';
    my @lines = get_lines(raw_output      => $raw_output,
                          first_line      => $first_line,
                          response_object => $resp);
    foreach my $line (@lines) {
        if ($line =~ /fbn\s+(\d+)/) {
            $temp[$i]->{'fbn'} = $1;
        }
        if ($line =~ /\s+vbn\s+(\w+)/) {
            $temp[$i]->{'vbn'} = $1;
        }
        if ($line =~ /\s+vvbn\s+(\w+)/) {
            $temp[$i]->{'vvbn'} = $1;
        }
        if($line =~ /refcount\s+(\d+)/) {
            $temp[$i]->{'refcount'} = $1;
        }
        if ($line =~ /cached\s+vbn\s+(.+),?/) {
            $temp[$i]->{'cached_vbn'} = $1;
        }
        if ($line =~ /not\s+cached/) {
            $temp[$i]->{'cached_vbn'} = "-";
        }
        if ($line =~ /vbn_cont\s+(\w+)/) {
            $temp[$i]->{'vbn_cont'} = $1;
        }
        if ($line =~ /Bin:\s+0/) {
            $temp[$i]->{'bin0'} = '1';
        }
        if ($line =~ /Bin:\s+1\s+OID:\s+(\d+)\s+Slot:\s+(\d+)/) {
            $temp[$i]->{'bin1'} = '1';
            $temp[$i]->{'oid'} = $1;
            $temp[$i]->{'slot'} = $2;
        }
        $i++;
    }
    return \@temp;
} # end of function wafl_showvbns

## @name ::wafl_scan_status
##
## @summary Parses the raw output from the "wafl scan status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_scan_status with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_scan_status(%args);
## INPUTS TO METHOD CALL 'wafl_scan_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> </tr>
## <tr> <td> '-S' </td> <td> 'subscan-id' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl scan status -A
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'info' => [
##&nbsp;                        {
##&nbsp;                          'type_of_scan' => 'active bitmap rearrangement',
##&nbsp;                          'scan_id' => '3',
##&nbsp;                          'progress' => 'fbn 1605 of 2133 w/ max_chain_len 3'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'aggregate' => 'aggr0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = wafl scan status -S
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'info' => [
##&nbsp;                        {
##&nbsp;                          'type_of_scan' => 'blocks used summary update',
##&nbsp;                          'scan_id' => '1',
##&nbsp;                          'subscan_id' => '0',
##&nbsp;                          'progress' => 'block 0 of 2023 (fbn 1522)'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'type_of_scan' => 'active bitmap rearrangement',
##&nbsp;                          'scan_id' => '2',
##&nbsp;                          'subscan_id' => '0',
##&nbsp;                          'progress' => 'fbn 499 of 2022 w/ max_chain_len 3'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'volume' => 'vol0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => [
##&nbsp;                        {
##&nbsp;                          'type_of_scan' => 'active bitmap rearrangement',
##&nbsp;                          'scan_id' => '904',
##&nbsp;                          'subscan_id' => '0',
##&nbsp;                          'progress' => 'fbn 3 of 98 w/ max_chain_len 3'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'type_of_scan' => 'blocks used summary update',
##&nbsp;                          'scan_id' => '926',
##&nbsp;                          'subscan_id' => '0',
##&nbsp;                          'progress' => 'block 0 of 99 (fbn 24)'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'volume' => 'vol1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_scan_status {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    my ($error, @lines, @temp, @all_aggr, %temp, @final);
    my $error_message;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$error_message)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @result_array = ();
    # For wafl_scan_status, some output is always expected.
    # So we are making $can_throw as 1 so that if the
    # lines array got finally in _get_lines is empty,
    # UnexpectedOutputException can be thrown there.
    my $can_throw = 1;
    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "can_throw"       => $can_throw,
        "response_object" => $resp
    );
    my %aggr;
    my %volume;
    my $Aggr_flag;
    my %output;
    my $name;
    my @arr2 = ();

    foreach my $line (@arr) {
        next if ($line =~ m/Scan\s+id\s+(Subscan\s+id\s+)?Type\s+of\s+scan\s+progress/);
        next if ( $line eq "" );
        my %line =();
        my %loop = ();
        my @arr1 = ();
        if ($line =~ m/Aggregate\s+(.*)[:]/) {
            @arr2 = ();
            $Aggr_flag = "y";
            $name = $1;
            if (%aggr) {
               %loop = %aggr;
               push(@result_array, \%loop);
               %aggr = ();
            }
            next;
        } elsif ($line =~ m/Volume\s+(.*)[:]/) {
            @arr2 = ();
            $Aggr_flag = "n";
            $name = $1;
            if (%volume) {
                %loop = %volume;
                push(@result_array, \%loop);
                %volume = ();
            }
           next;
        }
        my @line_array = split(/\s{2,}/,$line);
        $line{scan_id}=$line_array[0];
        if ($#line_array == 3) {
            $line{subscan_id}=$line_array[1];
        }
        $line{type_of_scan}=$line_array[$#line_array-1];
        $line{progress}=$line_array[$#line_array];
        push @arr2, \%line;
        @arr1 = @arr2;
        if ($Aggr_flag =~ m/y/i ) {
            $aggr{info} = \@arr1;
            $aggr{aggregate} = $name;
        } elsif ($Aggr_flag =~ m/n/i ) {
            $volume{info} = \@arr1;
            $volume{volume} = $name;
        }
    }
    if (%aggr) {
        push(@result_array, \%aggr);
    }
    if (%volume) {
        push(@result_array, \%volume);
    }
    return \@result_array;
} # end of function wafl_scan_status

## @name ::wafl_dlog_info
##
## @summary Parses the raw output from the "wafl dlog_info" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_dlog_info with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_dlog_info(%args);
## INPUTS TO METHOD CALL 'wafl_dlog_info' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; wafl dlog_info vol0; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'format' => '0-bits',
##&nbsp;            'delete_log' => [
##&nbsp;                              {
##&nbsp;                                'indirect' => '0',
##&nbsp;                                'blks_total' => '0',
##&nbsp;                                'status' => 'unloaded'
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'overflow_file' => [
##&nbsp;                                 {
##&nbsp;                                   'blks_total' => '0',
##&nbsp;                                   'status' => 'in use',
##&nbsp;                                   'instance' => '0'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                   'blks_total' => '0',
##&nbsp;                                   'status' => 'loaded',
##&nbsp;                                   'instance' => '1'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'purge_file' => [
##&nbsp;                              {
##&nbsp;                                'indirect' => '0',
##&nbsp;                                'blks_total' => '0',
##&nbsp;                                'status' => 'unloaded'
##&nbsp;                              }
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; wafl dlog_info vol0; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Delete log (unloaded): blks total 0, indirect 0
##&nbsp;Purge file[0] (unloaded): blks total 0, indirect 0
##&nbsp;Purge file[1] (unloaded): blks total 0, indirect 0
##&nbsp;Purge file[2] (unloaded): blks total 0, indirect 0
##&nbsp;Purge file[3] (unloaded): blks total 0, indirect 0
##&nbsp;Overflow file 0 (in use): blks total 0
##&nbsp;Overflow file 1 (loaded): blks total 0
##&nbsp;Format: 0-bits
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'format' => '0-bits',
##&nbsp;            'delete_log' => [
##&nbsp;                              {
##&nbsp;                                'indirect' => '0',
##&nbsp;                                'blks_total' => '0',
##&nbsp;                                'status' => 'unloaded'
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'overflow_file' => [
##&nbsp;                                 {
##&nbsp;                                   'blks_total' => '0',
##&nbsp;                                   'status' => 'in use',
##&nbsp;                                   'instance' => '0'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                   'blks_total' => '0',
##&nbsp;                                   'status' => 'loaded',
##&nbsp;                                   'instance' => '1'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'purge_file' => [
##&nbsp;                              {
##&nbsp;                                'indirect' => '0',
##&nbsp;                                'blks_total' => '0',
##&nbsp;                                'status' => 'unloaded',
##&nbsp;                                'instance' => '0'
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'indirect' => '0',
##&nbsp;                                'blks_total' => '0',
##&nbsp;                                'status' => 'unloaded',
##&nbsp;                                'instance' => '1'
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'indirect' => '0',
##&nbsp;                                'blks_total' => '0',
##&nbsp;                                'status' => 'unloaded',
##&nbsp;                                'instance' => '2'
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'indirect' => '0',
##&nbsp;                                'blks_total' => '0',
##&nbsp;                                'status' => 'unloaded',
##&nbsp;                                'instance' => '3'
##&nbsp;                              }
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_dlog_info {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _wafl_dlog_info(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );
    return $output;
} # end sub wafl_dlog_info

sub _wafl_dlog_info {
    my %args         = @_;
    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    # For 'wafl dlog_info', some output is always expected. So we are making
    # $can_throw as 1 so that if the lines array got finally in _get_lines is
    # empty, UnexpectedOutputException can be thrown there.
    my $can_throw = 1;

    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "can_throw"       => $can_throw,
        "response_object" => $response
    );

    my (@arr1, @arr2, %output);

    foreach my $line (@arr) {
        my ( %line, @line );
        if ( $line =~ /((?:Delete|Purge|Overflow)\s+\w+)\s?\[?(\d+)?\]?\s+\((.*)\)[:]\s+blks\s+total\s+(\d+)\,?\s?(?:indirect)?\s?(\d+)?/ ) {
            my ( $key1, $instance, $status, $blks_total, $indirect ) = ( $1,$2, $3, $4, $5 );
            $line{instance} = $instance if ( $instance =~ /\d+/ );
            $line{status} = $status ;
            $line{blks_total} = $blks_total;
            $line{indirect} = $indirect if ( $indirect =~ /\d+/);
            push @line, \%line;
            if( $instance =~ /\d+/ ) {
                if ( $key1 =~ /Purge/i ) {
                    push @arr1, $line[0];
                    $output{ to_key_name(item=>$key1) } = \@arr1;
                } elsif ( $key1 =~ /Overflow/i ){
                    push @arr2, $line[0];
                    $output{ to_key_name(item=>$key1) } = \@arr2;
                }
            } else {
                $output{ to_key_name(item=>$key1) } = \@line;
            }
        } elsif ( $line =~ /Format[:]\s+(.*)/ ) {
            $output{'format'} = $1;
        } else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "Unexpected format of output",
                response_object => $response
            );
        }
    }
    @result_array = (\%output);

    return \@result_array;
}

## @name ::wafl_flush_cache
##
## @summary Parses the raw output from the "wafl flush cache" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_flush_cache with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_flush_cache(%args);
## INPUTS TO METHOD CALL 'wafl_flush_cache' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl flush cache
##&nbsp;$VAR1 = [];
##&nbsp;
sub wafl_flush_cache {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array = ();

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return \@result_array;
} # end sub wafl_flush_cache

## @name ::wafl_scan_kireeti
##
## @summary Parses the raw output from the "wafl scan kireeti" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_scan_kireeti with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_scan_kireeti(%args);
## INPUTS TO METHOD CALL 'wafl_scan_kireeti' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl scan kireeti vol0
##&nbsp;$VAR1 = [];
##&nbsp;
sub wafl_scan_kireeti {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array = ();

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return \@result_array;
} # end sub wafl_scan_kireeti

## @name ::wafl_scan_fold
##
## @summary Parses the raw output from the "wafl scan fold" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_scan_fold with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_scan_fold(%args);
## INPUTS TO METHOD CALL 'wafl_scan_fold' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'file-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl scan fold /vol/vol0/testfile1
##&nbsp;$VAR1 = [];
##&nbsp;
sub wafl_scan_fold {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array = ();

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return \@result_array;
} # end sub wafl_scan_fold

## @name ::wafl_scan_ownblocks_calc
##
## @summary Parses the raw output from the "wafl scan ownblocks_calc" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_scan_ownblocks_calc with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_scan_ownblocks_calc(%args);
## INPUTS TO METHOD CALL 'wafl_scan_ownblocks_calc' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl scan ownblocks_calc vol0
##&nbsp;$VAR1 = [];
##&nbsp;
sub wafl_scan_ownblocks_calc {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array = ();

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return \@result_array;
} # end sub wafl_scan_ownblocks_calc

## @name ::wafl_nvlog_chk
##
## @summary Parses the raw output from the "wafl stats zero" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_nvlog_chk with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_nvlog_chk(%args);
## INPUTS TO METHOD CALL 'wafl_nvlog_chk' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl_nvlog_chk
##&nbsp;$VAR1 = [];
##&nbsp;
sub wafl_nvlog_chk {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array = ();

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return \@result_array;
} # end sub wafl_nvlog_chk

## @name ::wafl_scan_measure_layout
##
## @summary Parses the raw output from the "wafl scan measure_layout" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_scan_measure_layout with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_scan_measure_layout(%args);
## INPUTS TO METHOD CALL 'wafl_scan_measure_layout' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'file-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl scan measure_layout vol0
##&nbsp;$VAR1 = [];
##&nbsp;
sub wafl_scan_measure_layout {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array = ();

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return \@result_array;
} # end sub wafl_scan_measure_layout

## @name ::wafl_stats_zero
##
## @summary Parses the raw output from the "wafl stats zero" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_stats_zero with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_stats_zero(%args);
## INPUTS TO METHOD CALL 'wafl_stats_zero' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl stats zero
##&nbsp;$VAR1 = [];
##&nbsp;
sub wafl_stats_zero {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array = ();

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return \@result_array;
} # end sub wafl_stats_zero

## @name ::wafl_steal_stats
##
## @summary Parses the raw output from the "wafl_steal_stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_steal_stats with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_steal_stats(%args);
## INPUTS TO METHOD CALL 'wafl_steal_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl_steal_stats
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::dd_internal+16',
##&nbsp;            'instance' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::cmd_rdfile+223',
##&nbsp;            'instance' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::disk_smasher_admin+71',
##&nbsp;            'instance' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::pmap_set+260',
##&nbsp;            'instance' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::wafl_init_new_filesystem_handler+1361',
##&nbsp;            'instance' => '3'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::core_get_config+20',
##&nbsp;            'instance' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::raid_bufpool_init+155',
##&nbsp;            'instance' => '6626'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::sv_dump_ossv_metainfo+139',
##&nbsp;            'instance' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::rsm_init_nvlog_cksum+24',
##&nbsp;            'instance' => '8'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::wafl_init_new_filesystem_handler+413',
##&nbsp;            'instance' => '3'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::rsm_waflbufs_init+30',
##&nbsp;            'instance' => '25'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::bufprint_init+94',
##&nbsp;            'instance' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::core_invalidate_disks+15',
##&nbsp;            'instance' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::core_invalidate_disks+23',
##&nbsp;            'instance' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::raid_sparecore_init+115',
##&nbsp;            'instance' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::raid_sparecore_init+143',
##&nbsp;            'instance' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::__gnu_compiled.sk_domain_import$disk_get_raw_block_capacity$switch_domain$$1120+322',
##&nbsp;            'instance' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::savecore+28',
##&nbsp;            'instance' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wafl_steal_stats_functionality' => 'maytag.ko::savecore+52',
##&nbsp;            'instance' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_steal_stats {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _wafl_steal_stats(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );

    return $output;
} # end sub wafl_steal_stats

sub _wafl_steal_stats {
    my %args         = @_;

    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    # For 'wafl_steal_stats', some output is always expected. So we are making
    # $can_throw as 1 so that if the lines array got finally in _get_lines is
    # empty, UnexpectedOutputException can be thrown there.
    my $can_throw = 1;

    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "can_throw"       => $can_throw,
        "response_object" => $response
    );

    foreach my $line (@arr) {
        my %line = ();
        if ($line =~ m/(\d+)\s+(.*)/) {
            $line{instance} = $1;
            $line{wafl_steal_stats_functionality} = $2;
            push @result_array, \%line;
        } else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "Unexpected format of output",
                response_object => $response
            );
        }
    }

    return \@result_array;
} # end sub _wafl_steal_stats


## @name ::wafl_scan_abort
##
## @summary Parses the raw output from the "wafl scan abort" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_scan_abort with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_scan_abort(%args);
## INPUTS TO METHOD CALL 'wafl_scan_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'scanid' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl scan abort 1
##&nbsp;$VAR1 = [];
##&nbsp;
sub wafl_scan_abort {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array = ();

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return \@result_array;
} # end sub wafl_scan_abort

## @name ::wafl_touch
##
## @summary Parses the raw output from the "wafl touch" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_touch with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_touch(%args);
## INPUTS TO METHOD CALL 'wafl_touch' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'end-fbn' </td> <td> '0' </td> </tr>
## <tr> <td> '-init_badblock' </td> <td> 'init-badlock' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'inode-number' </td> <td> '0' </td> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> <td> '0' </td> </tr>
## <tr> <td> '-cache_dirty' </td> <td> 'cache-dirty' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'storage-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-private' </td> <td> 'private' </td> <td> '1' </td> </tr>
## <tr> <td> '-fakedirty' </td> <td> 'fakedirty' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'start-fbn' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'level-value' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'fbn-skip' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = wafl_touch
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'level' => '1',
##&nbsp;            'status' => 'succeeded',
##&nbsp;            'fbn' => '0',
##&nbsp;            'inode_number' => '99'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_touch {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }
    $first_line = 'wafl\s+touch+\s+\w+\s+\w+\s+\w+\s+\w+\s+\d+\s+\w+\s+\d+\s+\w+\s+\d+';
    my @lines = get_lines(raw_output      => $raw_output,
                          first_line      => $first_line,
                          response_object => $resp);
    foreach my $line (@lines) {
        if($line =~ /\w+\s+\w+\s+(\w+)\s+\w+\s+\w+\s+\w+\s+(\d+)\s+\w+\s+(\d+)\s+\w+\s+(\d+)/) {
            $temp[$i]->{'status'} = $1;
            $temp[$i]->{'inode_number'} = $2;
            $temp[$i]->{'level'} = $3;
            $temp[$i]->{'fbn'} = $4;
        } else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text=>"unexpected format of output",response_object=>$resp
                     );
        }
    }
    return \@temp;
} # end of function wafl_touch

## @name ::wafl_hya_purge_cache
##
## @summary Parses the raw output from the "wafl hya purge_cache" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_hya_purge_cache with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_hya_purge_cache(%args);
## INPUTS TO METHOD CALL 'wafl_hya_purge_cache' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'hybrid-aggregate-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl hya purge_cache hya_aggr
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cache_for_aggregate_hya_aggr' => 'purged'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_hya_purge_cache {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array = ();

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %info =();
    if($output =~ /Hy\w+\s+Cache.+for\s+aggregate\s+([\w\']+)\s+(\w+)/i) {
        my $aggr = $1;
        my $state = $2;
        $aggr =~ s/\'//g;
        $info{"cache_for_aggregate_".$aggr} = $state;
        push(@result_array, \%info);
    } else {
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
             text=>"unexpected format of output",response_object=>$resp
                     );
    }
    return \@result_array;
} # end sub wafl_hya_purge_cache


## @name ::wafl_hya_ssd_remove
##
## @summary Parses the raw output from the "wafl hya ssd_remove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_hya_ssd_remove with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_hya_ssd_remove(%args);
## INPUTS TO METHOD CALL 'wafl_hya_ssd_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'hybrid-aggregate-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vbn' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl hya ssd_remove hya_aggr 512000
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vbn' => 'not cached'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_hya_ssd_remove {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array = ();

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %info =();
    if($output =~ /^(VBN)\s+is\s+(not\s+cached)/i) {
        $info{"vbn"} = $2;
        push(@result_array, \%info);
    } elsif($output =~ /VBN\s+(provided)\s+is\s+a\s+(\w+\s+\w+)/i) {
        $info{"vbn"."-".$1} = $2;
        push(@result_array, \%info);
    } elsif($output =~ /SSD\s+entry\s+removed/i) {
        $info{"ssd"} = "entry removed";
        push(@result_array, \%info);
    } else {
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
             text=>"unexpected format of output",response_object=>$resp
                     );
    }

    return \@result_array;
} # end sub wafl_hya_ssd_remove

## @name ::wafl_hya_cache_remove
##
## @summary Parses the raw output from the "wafl hya cache_remove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_hya_cache_remove with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_hya_cache_remove(%args);
## INPUTS TO METHOD CALL 'wafl_hya_cache_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'hybrid-aggregate-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vbn' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl hya cache_remove hya_aggr 512000
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vbn_provided' => 'SSD PVBN'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_hya_cache_remove {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array = ();

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %info =();
    if($output =~ /(VBN)\s+is\s+(not\s+cached)/i) {
        $info{"vbn"} = $2;
        push(@result_array, \%info);
    } elsif($output =~ /VBN\s+(provided)\s+is\s+a\s+(\w+\s+\w+)/i) {
        $info{"vbn"."_".$1} = $2;
        push(@result_array, \%info);
    } elsif($output =~ /Cache\s+entry\s+removed/i) {
	} else {
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
             text=>"unexpected format of output",response_object=>$resp
                     );
    }

    return \@result_array;
} # end sub wafl_hya_cache_remove

## @name ::wafl_hya_ssdsnap_autodel
##
## @summary Parses the raw output from the "wafl hya ssdsnap_autodel" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_hya_ssdsnap_autodel with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_hya_ssdsnap_autodel(%args);
## INPUTS TO METHOD CALL 'wafl_hya_ssdsnap_autodel' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'set-status' </td> </tr>
## <tr> <td> '-p' </td> <td> 'threshold-percentage' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'hybrid-aggregate-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl hya ssdsnap_autodel hya_aggr -p 30 on
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'auto-deletion will occur if more than 30% of SSD space is trapped by snapshots'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_hya_ssdsnap_autodel {
    my %args = @_;
    print "inside wafl\n";

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $value;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines=get_lines('raw_output'=>$output,response_object => $resp);
    my %info = ();
    my @return_array;
    foreach $value (@lines) {
        if($value =~ /Aggregate\s+snapshots\s+in\s+\w+\s+will\s+be\s+auto-deleted.*/i) {
            next;
        } elsif($value =~ /(Aggregate\s+snapshots\s+in\s+\w+\s+will\s+NOT\s+be\s+auto-deleted.*)/i) {
            $info{"status"} = $1;
        } elsif($value =~ /This\s+(auto-deletion.*)/i) {
            $info{"status"} = $1;
        } else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text=>"unexpected format of output",response_object=>$resp
            );
        }
    }
    push(@return_array, \%info);

    return \@return_array;
} # end sub wafl_hya_ssdsnap_autodel

## @name ::wafl_hya_ssd_lookup
##
## @summary Parses the raw output from the "wafl hya ssd_lookup" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_hya_ssd_lookup with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_hya_ssd_lookup(%args);
## INPUTS TO METHOD CALL 'wafl_hya_ssd_lookup' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'hybrid-aggregate-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vbn' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl hya ssd_lookup hya_aggr 512000
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cbi' => '0',
##&nbsp;            'type' => 'DIRECT_ALLOC',
##&nbsp;            'ssd-pvbn' => '512000',
##&nbsp;            'amap' => 'used',
##&nbsp;            'encoding' => '1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_hya_ssd_lookup {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %info = ();
    my @return_array;

    my @lines = get_lines(raw_output => $output, first_line => 'SSD\s+PVBN', can_throw => 1);

    # Need to parse first line only
    my @fields = split(/\,\s+/, $lines[0]);
    foreach my $field (@fields) {
        if ($field =~ /(\w+):\s+(\d+)\s+(Amap):(\w+)/) {
            $info{to_key_name( item => $1 )} = $2;
            $info{to_key_name( item => $3 )} = $4;
        } else {
            my @items = split(/:\s+/, $field);
            $info{to_key_name( item => $items[0] )} = $items[1];
        }
    }
    push(@return_array, \%info);

    return \@return_array;
} # end sub wafl_hya_ssd_lookup

## @name ::wafl_hya_dump_ssd_range
##
## @summary Parses the raw output from the "wafl hya dump_ssd_range" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_hya_dump_ssd_range with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_hya_dump_ssd_range(%args);
## INPUTS TO METHOD CALL 'wafl_hya_dump_ssd_range' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'hybrid-aggregate-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl hya dump_ssd_range hya_aggr
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ssd_end' => '767999',
##&nbsp;            'ssd_start' => '512000'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_hya_dump_ssd_range {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @return_array;
    my %temp;
    my $can_throw = 1;
    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    my @lines = get_lines('raw_output' => $output,"response_object" => $resp);
    if($lines[1] =~ /SSD\s+\w+:\s+(\d+)\s+\w+:\s+(\d+)/) {
        $temp{ssd_start} = $1;
        $temp{ssd_end} = $2;
    }
    push(@return_array,\%temp);

    return \@return_array;
} # end sub wafl_hya_dump_ssd_range

## @name ::wafl_hya_ssd_pvbn_info
##
## @summary Parses the raw output from the "wafl hya ssd_pvbn_info" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_hya_ssd_pvbn_info with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_hya_ssd_pvbn_info(%args);
## INPUTS TO METHOD CALL 'wafl_hya_ssd_pvbn_info' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'num-entries' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'hybrid-aggregate-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vbn' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl hya ssd_pvbn_info hya_aggr 512000 4
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ssd_pvbn' => '512000',
##&nbsp;            'cbi' => '0',
##&nbsp;            'type' => 'DIRECT_ALLOC',
##&nbsp;            'amap' => 'used',
##&nbsp;            'encoding' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ssd_pvbn' => '512001',
##&nbsp;            'cbi' => '0',
##&nbsp;            'type' => 'FREE',
##&nbsp;            'amap' => 'free',
##&nbsp;            'encoding' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ssd_pvbn' => '512002',
##&nbsp;            'cbi' => '0',
##&nbsp;            'type' => 'DIRECT_ALLOC',
##&nbsp;            'amap' => 'used',
##&nbsp;            'encoding' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ssd_pvbn' => '512003',
##&nbsp;            'cbi' => '0',
##&nbsp;            'type' => 'DIRECT_ALLOC',
##&nbsp;            'amap' => 'used',
##&nbsp;            'encoding' => '1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_hya_ssd_pvbn_info {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @return_array;
    my $line;
    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    my @lines = get_lines('raw_output' => $output,"response_object" => $resp);
    foreach $line(@lines) {
        my %temp;
        if($line =~ /SSD\s\w+:\s(\d+),\s+Type:\s+\w+,\s+\w+:\s+(\d+)/) {
            $line =~ s/ Amap/, Amap/g;
            $line =~ s/,/:/g;
            my $i=0;
            my @key_values = split(/:\s*/,$line);
            foreach my $key_val (@key_values){
                if (($i == 0) || (($i % 2) == 0)){
                    my $new_key = to_key_name( item => "$key_val" );
                    $key_values[$i] = $new_key;
                }
                $i++;
            }
            %temp = @key_values;
        } elsif($line =~ /(VBN\s+provided\s+is\s+a\s+HDD\s+PVBN)/) {
            $temp{status} = $1 ;
        }
        push(@return_array,\%temp);
    }

    return \@return_array;
} # end wafl_hya_ssd_pvbn_info subroutine.

## @name ::wafl_hya_verify_cache
##
## @summary Parses the raw output from the "wafl hya verify_cache" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_hya_verify_cache with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_hya_verify_cache(%args);
## INPUTS TO METHOD CALL 'wafl_hya_verify_cache' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-e' </td> <td> 'end-pvbn' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'start-pvbn' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'hybrid-aggregate-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'hya-pvbn' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'entire-ssd-range' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = wafl hya verify_cache hya_aggr -f 256000 -e 383999
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'total_read_cached(failed)' => '0',
##&nbsp;            'total_free_blocks' => '128000',
##&nbsp;            'total_pvbns_verified' => '128000',
##&nbsp;            'total_read_cached(verified)' => '0',
##&nbsp;            'total_write_cached' => '0',
##&nbsp;            'total_direct_allocations' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_hya_verify_cache {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
        my $error;
        if ($error = _parse_for_errors( 'raw_output'   => $output,
                                    'command'      => $command,
                                    'error_message' => \$error_message)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    # Below regex is matched with a sample string mentioned
    # Verified:       100%
    # Total pvbns verified: 128000
    # Total free blocks: 127967
    # Total direct allocations: 33
    # Total write cached: 0
    # Total read cached(verified): 0
    # Total read cached(failed): 0

    # Command input/options in recent builds.
    # Below regex is matched with a sample string mentioned
    # Output from the command:
    # (SSD) PVBN  256000 is currently free - verified.
    #  or
    # (HDD) PVBN  216000 is not read cached - verified.
    # The last elsif block handles this condition.

    my @arr = get_lines("raw_output"=>$output, "response_object"=>$resp);
    my @return_array;
    my $value;
    foreach $value (@arr) {
        if($value =~ /^Total\s(\w+)\s(\w+):\s(\d+)/){
            $info{"total"."_".$1."_".$2}     = $3;
        } elsif($value =~/^Total\s(\w+)\s(\w+.\w+.):\s(\d+)/){
            $info{"total"."_".$1."_".$2}     = $3;
        } elsif($value =~ /^Verified:\s+(.*)/i) {
            $info{"verified"} = "100%";
        }
        elsif ($value =~ /.*?(\d+).*-\s+(\w+)/) {
            $info{"pvbn"} = $1;
            $info{"pvbn_status"} = $2;
        }
    }
    push( @return_array, \%info );

    return \@return_array;
}

## @name ::wafl_hya_cache_lookup
##
## @summary Parses the raw output from the "wafl hya cache_lookup" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_hya_cache_lookup with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_hya_cache_lookup(%args);
## INPUTS TO METHOD CALL 'wafl_hya_cache_lookup' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'hybrid-aggregate-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vbn' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl hya cache_lookup hya_aggr 512000
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vbn_provided' => 'SSD PVBN'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_hya_cache_lookup {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array = ();

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %info =();
    if($output =~ /(VBN)\s+is\s+(not\s+cached)/i) {
        $info{"vbn"} = $2;
        push(@result_array, \%info);
    } elsif($output =~ /VBN\s+(provided)\s+is\s+a\s+(\w+\s+\w+)/i) {
        $info{"vbn"."_".$1} = $2;
        push(@result_array, \%info);
    } elsif($output =~ /HDD\s+PVBN:\s+(\d+),\s+SSD\s+Cached\s+PVBN\s+(\d+)/i) {
        $info{"hdd_pvbn"} = $1;
        $info{"ssd_cached_pvbn"} = $2;
        push(@result_array, \%info);
    } else {
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
             text=>"unexpected format of output",response_object=>$resp
                     );
    }

    return \@result_array;
} # end sub wafl_hya_cache_lookup
## @name ::wafl_hyastats
##
## @summary Parses the raw output from the "wafl hyastats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_hya_cache_lookup with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_hyastats(%args);
## INPUTS TO METHOD CALL 'wafl_hyastats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'hybrid-aggregate-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vbn' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub wafl_hyastats {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @arr = get_lines("raw_output"=>$output, "response_object"=>$resp);
    my ($key1,$key2,@Res,%Output,%Hash_Flag_Out,@Arr_2,$Count,$Flag,$Line_Num);
    foreach my $value (@arr) {
        my %hash_flag = ();
        if(
        ($value =~/^(Cache\sins\sattempts):\s(\d+)$/)
        ||($value =~/^(Cache\sins\s+commits):\s(\d+)$/)
        ||($value =~/^(Cache\sins\sexit\son\sslowpath):\s(\d+)$/)
        ||($value =~/^(Cache\sins\srejects\sTotal):\s(\d+)$/)
        ||($value =~/^(Number\sof\sCMap\sload):\s(\d+)$/)
        ||($value =~/^(Number\sof\sCMap\sloads\spinned):\s(\d+)$/)
        ||($value =~/^(CM\sblock\sfrees):\s(\d+)$/)
        ||($value =~/^(CM\sLookup):\s(\d+)$/)
        ||($value =~/^(CM\sHits):\s(\d+)$/)
        ||($value =~/^(CM\sLookup\scollisions):\s(\d+)$/)
        ||($value =~/^(CM\sLookup\sretries):\s(\d+)$/)
        ||($value =~/^(CM\sLookup\sprev-search\sleaf):\s(\d+)$/)
        ||($value =~/^(CM\sLookup\sbsearch\snode):\s(\d+)$/)
        ||($value =~/^(CM\sLookup\sprev-search\snode):\s(\d+)$/)
        ||($value =~/^(CM\sLookup\sBulk):\s(\d+)$/)
        ||($value =~/^(CM\sLookup\spolicy\s+reject):\s(\d+)$/)
        ||($value =~/^(Eviction\sscanner\ssteps).\s(\d+)$/)
        ||($value =~/^(Eviction\scooled\sblocks).\s(\d+)$/)
        ||($value =~/^(Eviction\sdestaged\sblocks).\s(\d+)$/)
        ||($value =~/^(Eviction\sremoved\sblocks).\s(\d+)$/)
        ||($value =~/^(WS\seviction\sBLKR\sdirty).\s(\d+)$/)
        ||($value =~/^(Read\sand\sRA\shit\srate).\s(\d+.)$/)
        ||($value =~/^(Read\sAttempts).\s(\d+)$/)
        ||($value =~/^(Cache\sHits\son\sread).\s(\d+)$/)
        ||($value =~/^(RA\sRead\sattempts).\s(\d+)$/)
        ||($value =~/^(RA\sRead\shits).\s(\d+)$/)
        ||($value =~/^(RA\sCache\sIOs).\s(\d+)$/)
        ||($value =~/^(RA\sHDD\sIOs\ssaved).\s(\d+)$/)
        ||($value =~/^(Cache\smiss).\s(\d+)$/)
        ||($value =~/^(RA\sCached\sDummy\sRead).(\d+)$/)
        ||($value =~/^(RA\sCached\sNo\sraidios).(\d+)$/)
        ||($value =~/^(RA\sCached\sonce).(\d+)$/)
        ||($value =~/^(RA\sCached\sFlushQ).(\d+)$/)
        ||($value =~/^(RA\sHDD\sread\sfor\slong\schains).(\d+)$/)
        ||($value =~/^(RA\sCache\smiss).(\d+)$/)
        ||($value =~/^(Total\sWrites\sto\sSSD).(\d+)$/)
        ||($value =~/^(Cached\sWrites).(\d+)$/)
        ||($value =~/^(Write\sstaged\sWrites).(\d+)$/)
        ||($value =~/^(hitupdate\sstats\sfor\saggregate).\s(\w+)$/)
        ||($value =~/^(BLKR\sstats\sfor\saggregate).\s(\w+)$/)
        ||($value =~/^(RA\sCache\smiss\s.incomplete\sSSD\sRA\schains.).(\d+)$/)
        ||($value =~/^(.\w+)\s:(\d+)$/)
        ||($value =~/^(\w+\s+stats\s+for\s+aggregate):\s+(\w+)$/)
        ) {
            $Output{to_key_name(item=>$1)} = $2;
            next;
        } elsif (($value =~/^(Write\sthru\scaching)$/)
        ||($value =~/^(Write\sthru\scaching\sreject\sreasons.)$/)
        ||($value =~/^(Cache\sreject\sreasons.)$/)
        ||($value =~/^(CM\sInserts):\s(\d+)$/)
        ||($value =~/^(CM\sDeletes):\s(\d+)$/)
        ||($value =~/^(Direct\sreads\sfrom\sSSD):\s(\d+)$/)
        ||($value =~/^(RA\sdirect\sreads\sfrom\sSSD):(\d+)$/)
        ||($value =~/^(Direct\sAllocation\swrites):(\d+)$/)) {
            %Hash_Flag_Out = ();
            $Flag = 2;
            $key1 = $1;
            if ($2 =~ /(\d+)/) {
                $key2 = $1;
                $Hash_Flag_Out{to_key_name(item=>$key1)} = $key2;
                %hash_flag = %Hash_Flag_Out;
                $Output{to_key_name(item=>$key1)} = [\%hash_flag];
            }
            next;
        } elsif (($value =~/^(Cache\sins\sblock\stype.)$/)
        ||($value =~/^(Cache\sMiss\sfile\stype)$/)
        ||($value =~/^(Cache\sReject\sfile\stypes.)$/)
        ||($value =~/^(Cache\sHit\sfile\stype)$/)
        ||($value =~/^(RA\sCache\sHit\sfile\stype)$/)
        ||($value =~/^(RA\sCache\sMiss\sfile\stype)$/)
        ||($value =~/^(Write\sstage\sfile\stype)$/)
        ||($value =~/^(Direct\sAlloc\sfile\stype)$/)) {
            %Hash_Flag_Out = ();
            @Arr_2 = ();
            $Flag = 3;
            $key1 = $1;
            next;
        } elsif($value =~/^(Bufs\sscavenged\sbefore\sinsert\sprocessing.)$/){
            $key1 = $1;
            $Flag = 4;
            next;
        } elsif($value =~/^(Longest\seviction\sstep)[:]?$/) {
            $key1 = $1;
            $Flag = 5;
            next;
        } elsif(($value =~/^(Cache\smap\sbulk\slookup\shistogram)[:]?$/)
        ||($value =~/^(RA\sCache\schain\slength\shistogram)[:]?$/)
        ||($value =~/^(RA\sHDD\sIOs\ssaved\schain\slength\shistogram)[:]?$/)
        ||($value =~/^(RA\sHDD\schain\slength\shistrogram)[:]?$/)
        ||($value =~/^(RA\sDirect\sAlloc\schain\slength\shistrogram)[:]?$/)
        ||($value =~/^(Cache\sins\sage\shistogram)[:]?$/)
        ||($value =~/^(Cache\sins\sq-level\shistogram)[:]?$/)){
            $key1 = $1;
            $Flag = 6;
            $Count = 0;
            $Line_Num = 1;
            @Arr_2 = ();
            next;
        }

        if ($Flag == 2) {
            if ($value =~ /^(.*):(?:\s)?(\d+(?:\s+\d+\s+\d+)?)$/) {
                my $var1 = $1;
                my $var2 = $2;
                if ($2 =~/(\d+\s+\d+\s+\d+)/){
                    my @arr_1 = split(/\s+/,$1);
                    $Hash_Flag_Out{to_key_name(item=>$var1)} = \@arr_1;
                } else {
                    $Hash_Flag_Out{to_key_name(item=>$var1)} = $var2;
                }
                %hash_flag = %Hash_Flag_Out;
                $Output{to_key_name(item=>$key1)} = [\%hash_flag];
            }
        } elsif ($Flag == 3) {
            if($value =~/(level):\s+(\d+\s+\d+\s+\d+\s+\d+)/){
                my $inkey1 = $1;
                @Arr_2 = split(/\s+/,$2);
                for(my $i=0;$i<=$#Arr_2;$i++) {
                    $Output{to_key_name(item=>$key1)}->[$i]->{$inkey1}=$Arr_2[$i];
                }
            } elsif($value =~/type/){
                for(my $i=0;$i<=$#Arr_2;$i++) {
                $Output{to_key_name(item=>$key1)}->[$i]->{"type"}=undef;
                }
                next;
            } elsif($value =~/^.*:\s+\d+\s+\d+\s+\d+\s+\d+$/) {
                $value =~ s/[:]//;
                @Arr_2 = split(/\s{2,}/,$value);
                for(my $i=0;$i<$#Arr_2;$i++) {
                    $Output{to_key_name(item=>$key1)}->[$i]->{"type"}->[0]->{to_key_name(item=>$Arr_2[0])}=$Arr_2[$i+1] ;
                }
            }
        } elsif ($Flag == 4) {
            if($value =~ /^(Queue):\s+(\d+\s+\d+\s+\d+\s+\d+\s+\d+\s+\d+\s+\d+)$/) {
                @Arr_2 = split(/\s+/,$2);
                for(my $i=0;$i<=$#Arr_2;$i++) {
                    $Output{to_key_name(item=>$key1)}->[$i]->{to_key_name(item=>$1)}=$Arr_2[$i];
                }
            } elsif ($value =~/^(\d+\s+\d+\s+\d+\s+\d+\s+\d+\s+\d+\s+\d+)$/) {
                @Arr_2 = split(/\s+/,$1);
                for(my $i=0;$i<=$#Arr_2;$i++) {
                    $Output{to_key_name(item=>$key1)}->[$i]->{"value"}=$Arr_2[$i] ;
                }
            }
        } elsif ($Flag == 5) {
            if($value =~/^(\d+)\s(.*)$/) {
                $Output{to_key_name(item=>$key1)}->[0]->{to_key_name(item=>$2)} = $1;
            }
        } elsif ($Flag == 6) {
            if ($value =~ /^[<]?\d+\s+[<]?\d+\s+[<]?\d+\s+[<]?\d+\s+[<]?\d+\s+[<]?\d+\s+[<]?\d+\s+([>]?(?:UNUSED|\d+))$/) {
                @Arr_2 = split(/\s+/,$value);
                for (my $i=0;$i<=$#Arr_2;$i++) {
                    if($Line_Num%2 == 1) {
                        $Output{to_key_name(item=>$key1)}->[$i+$Count]->{"block_length"} = $Arr_2[$i];
                    } else {
                        $Output{to_key_name(item=>$key1)}->[$i+$Count]->{"amount"} = $Arr_2[$i];
                    }
                }
                if($Line_Num%2 == 0) {
                    $Count = $Count+$#Arr_2;
                }
                $Line_Num++;
            } elsif($value =~/(.*):\s+(\d+\.\d+)/) {
                $Output{to_key_name(item=>$key1)}->[++$Count]->{to_key_name(item=>$1)} = $2;
            }
        }
    }
    push(@Res,\%Output);
    return \@Res;
}

## @name ::wafl_scribble_buf
##
## @summary Parses the raw output from the "wafl scribble_buf" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_scribble_buf with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_scribble_buf(%args);
## INPUTS TO METHOD CALL 'wafl_scribble_buf' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'fileid' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'inospace' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'level' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'fbn' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'test-id' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'pattern-or-index' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'start-byte-or-vvbn' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'end-byte-or-pvbn' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = wafl scribble_buf
##&nbsp;$VAR1 = [];
##&nbsp;
sub wafl_scribble_buf {
    my %args = @_;

    my ($error, $error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return $resp;
} # end sub wafl_scribble_buf

## @name ::wafl_buftree_check
##
## @summary Parses the raw output from the "wafl buftree check" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_buftree_check with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_buftree_check(%args);
## INPUTS TO METHOD CALL 'wafl_buftree_check' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-nc' </td> <td> 'no-check' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'first-last-fbn' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'inodespec' </td> <td> '0' </td> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> <td> '0' </td> </tr>
## <tr> <td> '-sd' </td> <td> 'snapdiff' </td> <td> '1' </td> </tr>
## <tr> <td> '-nx' </td> <td> 'no-cross-check' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'snapid' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'no-panic' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'level-check' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = wafl buftree check -V vol0 priv:-1
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'buftree_info' => [
##&nbsp;                                 {
##&nbsp;                                   '32_bit_holes' => '0',
##&nbsp;                                   '32_bit_blks' => '0',
##&nbsp;                                   'level' => '2',
##&nbsp;                                   '64_bit_holes' => '0',
##&nbsp;                                   '64_bit_blks' => '1',
##&nbsp;                                   'total' => '1'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                   '32_bit_holes' => '0',
##&nbsp;                                   '32_bit_blks' => '0',
##&nbsp;                                   'level' => '1',
##&nbsp;                                   '64_bit_holes' => '2',
##&nbsp;                                   '64_bit_blks' => '4',
##&nbsp;                                   'total' => '6'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                   '32_bit_holes' => '0',
##&nbsp;                                   '32_bit_blks' => '0',
##&nbsp;                                   'level' => '0',
##&nbsp;                                   '64_bit_holes' => '1519',
##&nbsp;                                   '64_bit_blks' => '11',
##&nbsp;                                   'total' => '1530'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;             'total' => [
##&nbsp;                          {
##&nbsp;                            'total' => '1538',
##&nbsp;                            '64_bit_holes' => '1521',
##&nbsp;                            '32_bit_blks' => '0',
##&nbsp;                            '64_bit_blks' => '17',
##&nbsp;                            '32_bit_holes' => '0'
##&nbsp;                          }
##&nbsp;                        ]
##&nbsp;           }
##&nbsp;         ];
sub wafl_buftree_check {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @return_array;
    my $line;
    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    my @lines = get_lines('raw_output' => $output,"response_object" => $resp);

    while ( my $line = shift(@lines) ) {
        my $info_hash = {};
        my @fields;
        #Remove the Warning line, and other unwanted lines if found
        next if ($line=~/(Warning|Suspending until).*/);
        next if ($line =~ /^----+/); #Skip ---
        if ($line =~ /^\s*level/) {
            my $repeat;
            if ($line =~ /level\s+32-bit blks\s+32-bit holes\s+64-bit blks\s+64-bit holes\s+total/) {
                @fields = ('level', '32-bit blks', '32-bit holes', '64-bit blks', '64-bit holes', 'total');
            } else {
                # This is when verbose is enabled
                $line =~ s/level\s+//g;
                $info_hash->{'block'} = $line;
                $line = shift (@lines);
                if ($line =~ /32-bit blks under\s+/) {
                    shift(@lines); # Remove the fields
                    @fields = ('level', '32-bit blks', '64-bit blks');
                    $repeat = 1;
                } elsif ($line =~ /in 32-bit blk\s+in 64-bit blk/) {
                    shift(@lines); # Remove the fields
                    @fields = ('level', 'okay', 'PVBN', 'VVBN', 'BOTH');
                    $repeat = 1;
                } else {
                    my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
                    $error->throw(text=>"Unexpected form of output",response_object => $resp);
                }
            }

            @fields = map { to_key_name('item'=>$_) } @fields;
            # Also convert '-' to '_'
            for (@fields) {
                s/-/_/g;
            }

            shift(@lines); #Skip ---
            $line = shift(@lines);
                while ($line !~ /^----+/) {
                    my $table_hash ={};
                    my $table_hash1 ={};
                    my @values = split(/\s+/,$line);
                    if (@fields != @values and $repeat == 0) {
                         my $text = "Unexpected output format!\n "
                         . "Number of fields does not match number of values.\n"
                         . "Fields are:\n" . Dumper (\@fields) . "Values are:\n"
                         . Dumper (\@values);
                         NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                         text => $text,
                         response_object => $resp
                        );
                    }
                    if ($values[0] =~ /^total/) {
                        shift @values;
                        for(my $i=0;$i<$#fields;$i++) {
                            $table_hash->{$fields[$i+1]} = $values[$i];
                        }
                        push(@{ $info_hash->{'total'} }, $table_hash);
                    } else {
                        $table_hash->{$fields[0]} = $values[0];
                        my $j;
                        for(my $i=1;$i<=$#fields;$i++) {
                            $table_hash->{$fields[$i]} = $values[$i];
                            $j++;
                        }
                        if ($repeat) {
                            $table_hash1->{$fields[0]} = $values[0];
                            for(my $i=1;$i<=$#fields;$i++) {
                                $table_hash1->{$fields[$i]} = $values[$j+$i];
                            }
                            push(@{ $info_hash->{'32_bit_blocks'} }, $table_hash);
                            push(@{ $info_hash->{'64_bit_blocks'} }, $table_hash1);
                        } else {
                            push(@{ $info_hash->{'buftree_info'} }, $table_hash);
                        }
                    }

                    $line = shift(@lines);
                }
        } else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(text=>"Unexpected form of output",response_object => $resp);
        }
        push (@return_array,$info_hash);
    }

    return \@return_array;
} # end wafl_buftree_check.


sub _parse_for_errors {
    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if($output =~ /Invalid parameters/i) {
        $error = "InvalidParamValue";
        $$error_message = $output;
    } elsif($output =~ /(.*Cannot\sstart\sthe\sblock\sreclamation.*)/i) {
        # This regexp has to be above the check for "usage.+" below
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($output =~ /([Uu]sage:(?!\s\d+).*)|(Aggr\s+name\s+missing)|(Invalid\s+start.*)|(Invalid\s+end.*)|(VBN missing.*)|(Invalid VBN specified.*)|(Unknown parameter.*)|(Bad percentage.*)|(Invalid VBN.*)|options\:.*|(Invalid\s+volume\s+or\s+aggr\s+format.)|(Flexible\s+space\s+pool\s+ID\s+not\s+given.)|(Unknown\s+subcommand\s+given.)|(Flexible\s+space\s+pool\s+new\s+size\s+not\s+given.)/i) {
        $error = "InvalidParam";
        $$error_message = $output;
    } elsif($output =~ /(could not find.*)|((\w+)\s+(\w+)\s+is\s+not\s+hybrid)|(VBN\s+is\s+greater\s+than\s+maxvbn)|(Aggregate\s+\S+\s+(is\s+)?not\s+.*)|(Aggregate \w+ is not hybrid)|(Aggregate \w+ not found.*)|(VBN is greater than maxvbn.*)|(Invalid path .*)|(pvbn check:.*)/si) {
        $error = "InvalidParamValue";
        $$error_message = $output;
    } elsif($output =~ /(Missing pathname argument)|(Missing\s+path\s+argument)|(Missing start PVBN)|(Missing end PVBN)|(Aggregate \w+ not found)|(Missing aggregate argument)|(Missing\s+subcommand.)/si) {
        $error = "MissingRequiredParam";
        $$error_message = $output;
    } elsif ($output =~/wafl dlog_info: (.*)|(Unable\s+to\s+find.*)|(Unexpected\s+argument\s.*)|(Missing\s+volume.*)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~/wafl\s+scan\s+kireeti[:]\s+(.*is\s+not\s+a\s+valid\s+volume)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~/wafl\s+fold[:]\s+(could\s+not\s+find.*)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~/wafl\s+ownblocks_calc[:]\s+(volume\s+name.*invalid)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /(.*failed on.*)/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($output =~ /(.*Out of range.*)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /(.*Invalid inode.*)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /(.*unavailable)/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($output =~ /(.*greater than end fbn)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~ /scribble_buf failed(.*)/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif ($output =~ /wafl buftree check: Error: (.*)/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($output =~ /(.*\s+is\s+not\s+a\s+mounted\s+flexible\s+volume\s+or\s+aggregate)|(Volume\s+.*\s+does\s+not\s+need\s+redirect\s+scanning)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /(Unknown\s+argument:\s+.*)|(\[-f\]\s+is\s+deprecated)/) {
        $error = "InvalidParam";
        $$error_message = $1;
    } elsif($output =~ /wafl inconsistent: (volume\s+\S+\s+not found)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /wafl inconsistent: (No inconsistent volume data to print)/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($output =~ /(.*Invalid\soption.*)/) {
        $error = "InvalidParam";
        $$error_message = $1;
    } elsif($output =~ /wafl\s+punch:\s+can't\s+find\s+file.*/) {
        $error = "InvalidParamValue";
        $$error_message = $output;
    } elsif ($output =~ /wafl composite stats counter show <aggr>/) {
        $error = "InvalidParamValue";
        $$error_message = "Incorrect aggregate name";
    }

    return $error;
}

## @name ::wafl_scan_redirect
##
## @summary Parses the raw output from the "wafl scan redirect" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_scan_redirect with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_scan_redirect(%args);
## INPUTS TO METHOD CALL 'wafl_scan_redirect' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'snapshot' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'active-file-system' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggr' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; wafl scan redirect agg2345; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub wafl_scan_redirect {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @return_array;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
} # end sub wafl_scan_redirect

## @name ::wafl_stats
##
## @summary Parses the raw output from the "wafl stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_stats with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_stats(%args);
## INPUTS TO METHOD CALL 'wafl_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; wafl stats; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub wafl_stats{
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
} # end sub wafl_stats

## @name ::wafl_inconsistent
##
## @summary Parses the raw output from the "wafl inconsistent" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_inconsistent with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_inconsistent(%args);
## INPUTS TO METHOD CALL 'wafl_inconsistent' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub wafl_inconsistent{
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
} # end sub wafl_inconsistent

## @name ::wafl_scan_containerind
##
## @summary Parses the raw output from the "wafl scan containerind" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_scan_containerind with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_scan_containerind(%args);
## INPUTS TO METHOD CALL 'wafl_scan_containerind' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggr' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; wafl scan containerind apisetaggr1; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub wafl_scan_containerind{
 my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
} # end sub wafl_scan_containerind


## @name ::wafl_scan_pvbn_check
##
## @summary Parses the raw output from the "wafl scan pvbn_check" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_scan_pvbn_check with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_scan_pvbn_check(%args);
## INPUTS TO METHOD CALL 'wafl_scan_pvbn_check' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'flexvol-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'snapshot-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'check-correctness' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'check-unknown-pvbns' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'panic-on-error' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; wafl scan pvbn_check -c vol0; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub wafl_scan_pvbn_check{
 my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
} # end sub wafl_scan_pvbn_check

## @name ::wafl_scan_i2p
##
## @summary Parses the raw output from the "wafl scan i2p" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_scan_i2p with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_scan_i2p(%args);
## INPUTS TO METHOD CALL 'wafl_scan_i2p' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; wafl scan i2p vol0; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub wafl_scan_i2p{
 my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
} # end sub wafl_scan_i2p

## @name ::wafl_scan_blk_reclaim
##
## @summary Parses the raw output from the "wafl scan blk_reclaim" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_scan_blk_reclaim with any of its options as described
##    in the corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_scan_blk_reclaim(%args);
## INPUTS TO METHOD CALL 'wafl_scan_blk_reclaim' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> <td> '0' </td> </tr>
## <tr> <td> -f </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; wafl scan blk_reclaim vol0; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub wafl_scan_blk_reclaim{
 my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
} # end sub wafl_scan_blk_reclaim

## @name ::wafl_punch
##
## @summary Parses the raw output from the "wafl punch" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_punch with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_punch(%args);
## INPUTS TO METHOD CALL 'wafl_punch' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'count' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'filename' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'fbn' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; wafl punch -t /vol/vol0/testfile1 0 1; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;wafl punch: status=0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_punch {
    my %args = @_;
    my ( $error_message, $error );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @lines   = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ( $lines[0] =~ /wafl\s+punch:\s+(\w+)=(\d+)/ ) {
           $result_array[0]->{to_key_name( item =>$1)} = $2;
    }
    return \@result_array;
} # end sub wafl_punch

## @name ::wafl_cpstats
##
## @summary Parses the raw output from the "wafl cpstats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_cpstats with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_cpstats(%args);
## INPUTS TO METHOD CALL 'wafl_cpstats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'check-correctness' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; wafl cpstats  -c; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Per CP stats
##&nbsp;CP[12]: Latest CP
##&nbsp;cp_count                          =    7483131
##&nbsp;reason                            = cp_from_sync
##&nbsp;nvlog entries                     =        211
##&nbsp;Vols in CP                        =          2
##&nbsp;Vvols in CP                       =          1
##&nbsp;Hybrid aggrs in CP                =          0
##&nbsp;Vvols in CP (replaying)           =          0
##&nbsp;Pvols in CP (replaying)           =          0
##&nbsp;Vvols in CP (unmounting)          =          0
##&nbsp;Pvols in CP (unmounting)          =          0
##&nbsp;cp_allocation_loops               =          2
##&nbsp;cp_alloc_loop_no_stages           =          0
##&nbsp;cp_dirty_allocation_blks          =         16
##&nbsp;cp_dirty_public_inofile_blks      =          7
##&nbsp;cp_dirty_privateinofile_blks      =         13
##&nbsp;cp_inodes_cleaned                 =          9
##&nbsp;cp_ssd_inodes_cleaned             =          0
##&nbsp;cp_reads_returned                 =          0
##&nbsp;cp_dirty_buf_high                 = 23912
##&nbsp;
##&nbsp;                                          THIS         NEXT
##&nbsp;cp_dlogged                        =         30         3072
##&nbsp;cp_dlog_purge_pvbns               =       1024         3072
##&nbsp;cp_df_dbya_blks_used              =          0            0
##&nbsp;cp_df_dbys_blks_used              =          0            0
##&nbsp;cp_df_vol_max                     =          0            0
##&nbsp;cp_df_aggr_max                    =          0            0
##&nbsp;cp_df_not_ok                      =          0            0
##&nbsp;cp_df_max_ignored                 =          0            0
##&nbsp;cp_df_trans_cp                    =          0            0
##&nbsp;
##&nbsp;cp_mb_dv.cp_mb_in_use             =       9168
##&nbsp;cp_mb_dv.cp_mb_cascaded           =          0
##&nbsp;cp_mb_dv.cp_mb_wafl_in_use        =          0
##&nbsp;cp_mb_dv.cp_dv_wafl_avail         =          0
##&nbsp;cp_mb_dv.cp_dv_wafl_avail_min     =          0
##&nbsp;cp_mb_dv.cp_dv_zero_msecs         = 1792038926
##&nbsp;cp_buf_miss_cnt                 =          0
##&nbsp;
##&nbsp;cp_disk_io.buf_load_cnt                =        132
##&nbsp;cp_disk_io.blks_read                   =          0
##&nbsp;cp_disk_io.blks_read_type[0].active    =          0
##&nbsp;cp_disk_io.blks_read_type[0].snapmap   =          0
##&nbsp;cp_disk_io.blks_read_type[0].summary   =          0
##&nbsp;cp_disk_io.blks_read_type[0].spacemap  =          0
##&nbsp;cp_disk_io.blks_read_type[0].kireeti   =          0
##&nbsp;cp_disk_io.blks_read_type[0].container =          0
##&nbsp;cp_disk_io.blks_read_type[0].refcnt    =          0
##&nbsp;cp_disk_io.blks_read_type[0].public_inofile   =          0
##&nbsp;cp_disk_io.blks_read_type[0].private_inofile  =          0
##&nbsp;cp_disk_io.blks_read_type[0].metadata  =          0
##&nbsp;cp_disk_io.blks_read_type[0].regular   =          0
##&nbsp;cp_disk_io.blks_read_type[1].active    =          0
##&nbsp;cp_disk_io.blks_read_type[1].snapmap   =          0
##&nbsp;cp_disk_io.blks_read_type[1].summary   =          0
##&nbsp;cp_disk_io.blks_read_type[1].spacemap  =          0
##&nbsp;cp_disk_io.blks_read_type[1].kireeti   =          0
##&nbsp;cp_disk_io.blks_read_type[1].container =          0
##&nbsp;cp_disk_io.blks_read_type[1].refcnt    =          0
##&nbsp;cp_disk_io.blks_read_type[1].public_inofile   =          0
##&nbsp;cp_disk_io.blks_read_type[1].private_inofile  =          0
##&nbsp;cp_disk_io.blks_read_type[1].metadata  =          0
##&nbsp;cp_disk_io.blks_read_type[1].regular   =          0
##&nbsp;
##&nbsp;cp_flash_io.buf_load_cnt                =          0
##&nbsp;cp_flash_io.blks_read                   =          0
##&nbsp;cp_flash_io.blks_read_type[0].active    =          0
##&nbsp;cp_flash_io.blks_read_type[0].snapmap   =          0
##&nbsp;cp_flash_io.blks_read_type[0].summary   =          0
##&nbsp;cp_flash_io.blks_read_type[0].spacemap  =          0
##&nbsp;cp_flash_io.blks_read_type[0].kireeti   =          0
##&nbsp;cp_flash_io.blks_read_type[0].container =          0
##&nbsp;cp_flash_io.blks_read_type[0].refcnt    =          0
##&nbsp;cp_flash_io.blks_read_type[0].public_inofile   =          0
##&nbsp;cp_flash_io.blks_read_type[0].private_inofile  =          0
##&nbsp;cp_flash_io.blks_read_type[0].metadata  =          0
##&nbsp;cp_flash_io.blks_read_type[0].regular   =          0
##&nbsp;cp_flash_io.blks_read_type[1].active    =          0
##&nbsp;cp_flash_io.blks_read_type[1].snapmap   =          0
##&nbsp;cp_flash_io.blks_read_type[1].summary   =          0
##&nbsp;cp_flash_io.blks_read_type[1].spacemap  =          0
##&nbsp;cp_flash_io.blks_read_type[1].kireeti   =          0
##&nbsp;cp_flash_io.blks_read_type[1].container =          0
##&nbsp;cp_flash_io.blks_read_type[1].refcnt    =          0
##&nbsp;cp_flash_io.blks_read_type[1].public_inofile   =          0
##&nbsp;cp_flash_io.blks_read_type[1].private_inofile  =          0
##&nbsp;cp_flash_io.blks_read_type[1].metadata  =          0
##&nbsp;cp_flash_io.blks_read_type[1].regular   =          0
##&nbsp;
##&nbsp;cp_ra.requested                   =          4
##&nbsp;cp_ra.read                        =          0
##&nbsp;cp_ra.incore                      =          4
##&nbsp;cp_ra.past_eof                    =          0
##&nbsp;cp_ra.hole                        =          0
##&nbsp;cp_ra.vbn_zero                    =          0
##&nbsp;cp_ra.noblocks                    =          0
##&nbsp;cp_ra.nomsgs                      =          0
##&nbsp;cp_ra.nobufs                      =          0
##&nbsp;
##&nbsp;csra.csra_hash_inserted                =         25
##&nbsp;csra.csra_hash_removed                 =         25
##&nbsp;csra.csra_hash_vvol_hits               =          8
##&nbsp;csra.csra_hash_vbn_hits                =         63
##&nbsp;csra.csra_hash_aggr_hits               =         58
##&nbsp;csra.csra_hash_misses                  =         25
##&nbsp;csra.csra_hash_no_entry                =          0
##&nbsp;csra.csra_hash_this_cp_drop            =          0
##&nbsp;csra.csra_hash_next_cp_drop            =          0
##&nbsp;csra.csra_hash_non_cp_drop             =          0
##&nbsp;csra.csra_hash_this_cp_susp            =          0
##&nbsp;csra.csra_hash_next_cp_susp            =          0
##&nbsp;csra.csra_hash_non_cp_susp             =          0
##&nbsp;
##&nbsp;csra.csra_q_this_cp.readyq_maxsize     =          0
##&nbsp;csra.csra_q_this_cp.activeq_maxsize    =          0
##&nbsp;csra.csra_q_this_cp.enqueued           =          0
##&nbsp;csra.csra_q_this_cp.not_enqueued       =         12
##&nbsp;csra.csra_q_this_cp.wrx_issued         =         12
##&nbsp;
##&nbsp;csra.csra_q_next_cp.readyq_maxsize     =          0
##&nbsp;csra.csra_q_next_cp.activeq_maxsize    =          0
##&nbsp;csra.csra_q_next_cp.enqueued           =          0
##&nbsp;csra.csra_q_next_cp.not_enqueued       =         13
##&nbsp;csra.csra_q_next_cp.wrx_issued         =         13
##&nbsp;
##&nbsp;csra.csra_q_non_cp.readyq_maxsize      =          0
##&nbsp;csra.csra_q_non_cp.activeq_maxsize     =          0
##&nbsp;csra.csra_q_non_cp.enqueued            =          0
##&nbsp;csra.csra_q_non_cp.not_enqueued        =          0
##&nbsp;csra.csra_q_non_cp.wrx_issued          =          0
##&nbsp;
##&nbsp;csra.csra_q_remote                     =         25
##&nbsp;csra.csra_q_rave                       =          0
##&nbsp;csra.csra_q_indirect                   =          0
##&nbsp;csra.csra_q_map_bufs                   =          0
##&nbsp;csra.csra_q_hya_cmap                   =          0
##&nbsp;csra.csra_q_hya_ssd                    =          0
##&nbsp;
##&nbsp;cp_dbufs.total                      =        291
##&nbsp;cp_dbufs.total_vvol                 =        235
##&nbsp;cp_dbufs.total_pvol                 =         56
##&nbsp;cp_dbufs.incoming                   =        212
##&nbsp;cp_dbufs.ssd_incoming               =          0
##&nbsp;cp_dbufs.ssd_da_incoming            =          0
##&nbsp;cp_dbufs.blkr_added                 =          0
##&nbsp;cp_dbufs.incoming_public_inofile    =          0
##&nbsp;cp_dbufs.incoming_private_inofile   =          0
##&nbsp;cp_dbufs.incoming_commitment_active =          6
##&nbsp;cp_dbufs.incoming_commitment_refcnt =          0
##&nbsp;cp_dbufs.incoming_partial_write     =          0
##&nbsp;cp_dbufs.incoming_fake_dirty        =          0
##&nbsp;cp_dbufs.incoming_commitment_cmap   =          0
##&nbsp;cp_dbufs.incoming_commitment_ttmap  =          0
##&nbsp;cp_dbufs.incoming_commitment_ownmap =          0
##&nbsp;cp_dbufs.public_inofile             =          7
##&nbsp;cp_dbufs.private_inofile            =         13
##&nbsp;cp_dbufs.allocfile                  =         19
##&nbsp;cp_dbufs.indirect                   =          2
##&nbsp;cp_dbufs.container                  =         11
##&nbsp;cp_dbufs.dlog                       =         14
##&nbsp;cp_dbufs.purge                      =          5
##&nbsp;cp_dbufs.ndx0                       =          8
##&nbsp;cp_dbufs.write_allocated            =        291
##&nbsp;cp_dbufs.ssd_write_allocated        =          0
##&nbsp;cp_dbufs.write_allocated_vvol       =        235
##&nbsp;cp_dbufs.ssd_rc_wallocd             =          0
##&nbsp;cp_dbufs.ssd_wc_wallocd             =          0
##&nbsp;cp_dbufs.ssd_cancelled              =          0
##&nbsp;cp_dbufs.written                    =        291
##&nbsp;cp_dbufs.ssd_written                =          0
##&nbsp;cp_dbufs.committed                  =        294
##&nbsp;cp_dbufs.compressed_away            =          0
##&nbsp;cp_dbufs.vbn_zero                   =          0
##&nbsp;
##&nbsp;cp_wa.alloc_areas                 =          0
##&nbsp;cp_wa.load_search_bufs            =          0
##&nbsp;cp_wa.lsb_missing_total           =          0
##&nbsp;cp_wa.lsb_missing_active          =          0
##&nbsp;cp_wa.lsb_missing_summary         =          0
##&nbsp;cp_wa.lsb_missing_spacemap        =          0
##&nbsp;cp_wa.lsb_missing_kireeti         =          0
##&nbsp;cp_wa.lsb_missing_snapmap         =          0
##&nbsp;cp_wa.lsb_missing_claimed         =          0
##&nbsp;cp_wa.lsb_missing_checked         =          0
##&nbsp;cp_wa.lsb_missing_ttmap           =          0
##&nbsp;cp_wa.lsb_missing_reversemap      =          0
##&nbsp;cp_wa.lsb_missing_ownermap        =          0
##&nbsp;cp_wa.lsb_missing0                =          0
##&nbsp;cp_wa.lsb_active_susp             =          0
##&nbsp;cp_wa.lsb_summary_susp            =          0
##&nbsp;cp_wa.lsb_spacemap_susp           =          0
##&nbsp;cp_wa.lsb_kireeti_susp            =          0
##&nbsp;cp_wa.lsb_vvol_cont_susp          =          0
##&nbsp;cp_wa.lsb_snapmap_susp            =          0
##&nbsp;cp_wa.lsb_claimed_susp            =          0
##&nbsp;cp_wa.lsb_checked_susp            =          0
##&nbsp;cp_wa.lsb_ttmap_susp              =          0
##&nbsp;cp_wa.lsb_reversemap_susp         =          0
##&nbsp;cp_wa.lsb_ownermap_susp           =          0
##&nbsp;cp_wa.hda_deg_rg                  =          0
##&nbsp;cp_wa.wrsv_rgs_full_low           =          0
##&nbsp;cp_wa.wrsv_rgs_full_high          =          0
##&nbsp;cp_wa.wrsv_rgs_full               =          0
##&nbsp;cp_wa.wrsv_rgs_full_low_ssd       =          0
##&nbsp;cp_wa.wrsv_rgs_full_high_ssd      =          0
##&nbsp;cp_wa.wrsv_rgs_full_ssd           =          0
##&nbsp;cp_wa.wrsv_vol_full_low           =          0
##&nbsp;cp_wa.wrsv_vol_full_high          =          0
##&nbsp;cp_wa.wrsv_vol_resets             =          0
##&nbsp;cp_wa.wrsv_vol_full_low_ssd       =          0
##&nbsp;cp_wa.wrsv_vol_full_high_ssd      =          0
##&nbsp;cp_wa.wrsv_vol_resets_ssd         =          0
##&nbsp;
##&nbsp;cp_wa.cpc_nwips
##&nbsp;   cp_wa.cpc_nwips_PARTIAL_WRITE        =         0
##&nbsp;   cp_wa.cpc_nwips_FAKE_DIRTY           =         0
##&nbsp;   cp_wa.cpc_nwips_COMPRESS             =         0
##&nbsp;   cp_wa.cpc_nwips_OTHERS               =         0
##&nbsp;
##&nbsp;cp_wa.cpc_bufs
##&nbsp;   cp_wa.cpc_nbufs_PARTIAL_WRITE        =         0
##&nbsp;   cp_wa.cpc_nbufs_FAKE_DIRTY           =         0
##&nbsp;   cp_wa.cpc_nbufs_COMPRESS             =         0
##&nbsp;   cp_wa.cpc_nbufs_OTHERS               =         0
##&nbsp;
##&nbsp;other bufs cleaned in kahuna     =          0
##&nbsp;
##&nbsp;cp_wa.cpc_nrepurpose
##&nbsp;   cp_wa.cpc_nrepurpose_PARTIAL_WRITE        =         0
##&nbsp;   cp_wa.cpc_nrepurpose_FAKE_DIRTY           =         0
##&nbsp;   cp_wa.cpc_nrepurpose_COMPRESS             =         0
##&nbsp;   cp_wa.cpc_nrepurpose_OTHERS               =         0
##&nbsp;
##&nbsp;cp_wa.wa_stalls                 =          0
##&nbsp;cp_wa.wa_stalls_reval             =          0
##&nbsp;cp_wa.cpdump_skip_nomem           =          0
##&nbsp;cp_wa.cpdump_skip_incomplete      =          0
##&nbsp;cp_wa.cpdump_skip_noarchive       =          0
##&nbsp;Misc write alloc stats in this CP
##&nbsp;
##&nbsp;cp_rg.wcrs_pvol_aa_cnt            =          0
##&nbsp;cp_rg.wcrs_pvol_aa_score_min      =          0
##&nbsp;cp_rg.wcrs_pvol_aa_score_sum      =          0 (average = N/A)
##&nbsp;cp_rg.wcrs_pvol_sync_reval_cnt    =          0
##&nbsp;cp_rg.wcrs_pvol_async_reval_cnt   =          0
##&nbsp;cp_rg.wcrs_vvol_aa_cnt            =          0
##&nbsp;cp_rg.wcrs_vvol_aa_score_min      =          0
##&nbsp;cp_rg.wcrs_vvol_aa_score_sum      =          0 (average = N/A)
##&nbsp;cp_rg.wcrs_vvol_aa_df_cnt         =          0
##&nbsp;cp_rg.wcrs_vvol_aa_vvbn_cnt       =          0
##&nbsp;cp_rg.wcrs_vvol_sync_reval_cnt    =          0
##&nbsp;cp_rg.wcrs_vvol_async_reval_cnt   =          0
##&nbsp;cp_tet.wcts_tet_cnt               =          6
##&nbsp;cp_tet.wcts_sent_cnt              =          6
##&nbsp;cp_tet.wcts_metadata_cnt          =          1
##&nbsp;cp_bkt.wcbs_get_pbkt_cnt          =         78
##&nbsp;cp_bkt.wcbs_put_pbkt_cnt          =         78
##&nbsp;cp_bkt.wcbs_use_pbkt_cnt          =          0
##&nbsp;cp_bkt.wcbs_pbkt_dbufs            =        291
##&nbsp;cp_bkt.wcbs_get_vbkt_cnt          =         30
##&nbsp;cp_bkt.wcbs_put_vbkt_cnt          =         30
##&nbsp;cp_bkt.wcbs_use_vbkt_cnt          =         30
##&nbsp;cp_bkt.wcbs_vbkt_dbufs            =        235
##&nbsp;
##&nbsp;blkfree.vvol:
##&nbsp;    vol_hit           =         13
##&nbsp;    global_hit        =          0
##&nbsp;    out_of_stage      =          0
##&nbsp;    nextcp_num_stages =          0
##&nbsp;    nextcp_vbns       =          0
##&nbsp;    nextcp_out_of_mem =          0
##&nbsp;    zombie_stages     =          4
##&nbsp;    staged_from_cleaner =         33
##&nbsp;    metafile_io                        BEFORE_CP         THIS_CP         NEXT_CP
##&nbsp;        [INCORE]
##&nbsp;                         BITMAPS              26              38               8
##&nbsp;    staged_vbns                        BEFORE_CP         THIS_CP         NEXT_CP
##&nbsp;                       ALLOC_BUF               0               3               0
##&nbsp;              ALLOC_METAFILE_BUF               0              30               0
##&nbsp;               TRUNCATE_OVERFLOW               2               0               4
##&nbsp;         ZOMBIE_FREE_BLOCK_RANGE             202               0               0
##&nbsp;    committed_vbns                     BEFORE_CP         THIS_CP         NEXT_CP
##&nbsp;                         BITMAPS             128             105               0
##&nbsp;                        OVERFLOW               0               4               0
##&nbsp;    committed_stages                   BEFORE_CP         THIS_CP         NEXT_CP
##&nbsp;                         BITMAPS               2               4               0
##&nbsp;                        OVERFLOW               0               1               0
##&nbsp;blkfree.pvol:
##&nbsp;    vol_hit           =         64
##&nbsp;    global_hit        =          0
##&nbsp;    out_of_stage      =          0
##&nbsp;    nextcp_num_stages =          0
##&nbsp;    nextcp_vbns       =          0
##&nbsp;    nextcp_out_of_mem =          0
##&nbsp;    zombie_stages     =          0
##&nbsp;    staged_from_cleaner =         55
##&nbsp;    metafile_io                        BEFORE_CP         THIS_CP         NEXT_CP
##&nbsp;        [INCORE]
##&nbsp;                         BITMAPS              18             110            7146
##&nbsp;    staged_vbns                        BEFORE_CP         THIS_CP         NEXT_CP
##&nbsp;              ALLOC_METAFILE_BUF               0              55               0
##&nbsp;            SCAN_BLK_RECLAMATION               0               0            3073
##&nbsp;               TRUNCATE_OVERFLOW               8               0              22
##&nbsp;    committed_vbns                     BEFORE_CP         THIS_CP         NEXT_CP
##&nbsp;                            DLOG               0              30            3072
##&nbsp;                        OVERFLOW               0              21               0
##&nbsp;                BITMAPS_REDIRECT               0              12               0
##&nbsp;    committed_stages                   BEFORE_CP         THIS_CP         NEXT_CP
##&nbsp;                            DLOG               0               1              48
##&nbsp;                        OVERFLOW               0               2               0
##&nbsp;                BITMAPS_REDIRECT               0               1               0
##&nbsp;
##&nbsp;wafl perCP zombie completion msecs histogram: (2) -- simple counts
##&nbsp;        0        1        2        3        4        5        6        7
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED
##&nbsp;        2        0        0        0        0        0        0        0
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;Average:        9.00
##&nbsp;
##&nbsp;cp_time.start_msecs               = 1792038926
##&nbsp;cp_time.end_msecs                 = 1792039085
##&nbsp;cp_time.read_msecs                =          0
##&nbsp;cp_time.next_log_nearly_full_msecs=          0
##&nbsp;cp_time.nvlog_full_susp_msecs     =          0
##&nbsp;cp_time.over_commit_msecs         =          0
##&nbsp;cp_time.prefer_writes_msecs       =          0
##&nbsp;cp_time.blkr_dirty_susp_msecs     =          0
##&nbsp;
##&nbsp;cpp_msg_stats
##&nbsp;         Class Queue Msg Type                       #Msgs (new+restarts)       Total CPU(us)
##&nbsp;============== ===== ============================== ==================== ===================
##&nbsp;        SERIAL hipri SK_SIGNAL_MSG                                     4                 136
##&nbsp;        SERIAL lopri WAFL_GETATTR                                    210                5261
##&nbsp;        SERIAL hipri WAFL_GETATTR                                      6                 166
##&nbsp;        SERIAL lopri WAFL_LOOKUP                                      22                 749
##&nbsp;        SERIAL hipri WAFL_LOOKUP                                       9                 312
##&nbsp;        SERIAL lopri WAFL_READ                                       200                7890
##&nbsp;        SERIAL lopri WAFL_WRITE                                      200               20544
##&nbsp;        SERIAL hipri WAFL_WRITE                                        3                 390
##&nbsp;        SERIAL lopri WAFL_CREATE                                       2                 246
##&nbsp;        SERIAL lopri WAFL_REMOVE                                       2                 568
##&nbsp;        SERIAL lopri WAFL_TRUNCATE_INDIRECT                            2                7387
##&nbsp;        SERIAL lopri WAFL_REMOVE_ZOMBIE                                2                 176
##&nbsp;       CLEANER       WAFL_CLEAN_NORMAL                                11                9283
##&nbsp;       CLEANER       WAFL_CLEAN_PUB_INOFILE                            1                 498
##&nbsp;       CLEANER       WAFL_CLEAN_PRI_INOFILE                            2                 709
##&nbsp;       CLEANER       WAFL_CLEAN_METAFILE                               1                  57
##&nbsp;       CLEANER       WAFL_CLEAN_ALLOC_FILE                             6                 772
##&nbsp;       CLEANER       WAFL_CLEAN_CONTAINER                              1                 535
##&nbsp;       CLEANER       WAFL_CLEAN_VVOL_AUXFILE                           2                 123
##&nbsp;       CLEANER       WAFL_CLEAN_DLOG_FILE                              1                 853
##&nbsp;       CLEANER       WAFL_CLEAN_PURGE_FILE                             1                 341
##&nbsp;       CLEANER       WAFL_CLEAN_FSINFO                                 2                 105
##&nbsp;       CLEANER       WAFL_CLEAN_TOPAA                                  2                 289
##&nbsp;       CLEANER       WAFL_CLEAN_VOLINFO                                2                 143
##&nbsp;        SERIAL    cp WAFL_PRE_CLEAN_INODE                             27                 440
##&nbsp;        SERIAL    cp WAFL_POST_CLEAN_INODE                            36                2054
##&nbsp;        SERIAL lopri WAFL_DELAYED_FREE_WORK                          108               48517
##&nbsp;        SERIAL    cp WAFL_DLOG_PURGE                                   9               72624
##&nbsp;        SERIAL    cp WAFL_TRUNCATE_OVERFLOW                            6                 615
##&nbsp;        SERIAL hipri WAFL_DLOG_RA_FREE_BUFS                            8                 248
##&nbsp;        SERIAL    cp WAFL_FLUSH_METAFILE_INODE                         9                 313
##&nbsp;        SERIAL    cp WAFL_APPLY_MAPS                                  10                 898
##&nbsp;        SERIAL    cp WAFL_SYNC_BKTS                                    8                 118
##&nbsp;        SERIAL    cp WAFL_GATHER_STAGES_AND_COMMIT                     8                 126
##&nbsp;        SERIAL    cp WAFL_BUILD_PVOL_TOPAA                             1                 218
##&nbsp;        SERIAL    cp WAFL_CPWF_WABE_VOL_PREPARE_FOR                    2                 304
##&nbsp;        SERIAL    cp WAFL_CPWF_PREPARE_TOPAA                           2                 201
##&nbsp;        SERIAL    cp WAFL_CPWF_DIRTY_METAFILE_BLKS                     2                 203
##&nbsp;        SERIAL    cp WAFL_CPWF_WALLOC_PREP_FSINFO                      2                  88
##&nbsp;        SERIAL    cp WAFL_CPWF_WALLOC_PREP_VOLINFO                     2                 132
##&nbsp;        SERIAL    cp WAFL_CPWF_WRITE_VVOLINFO                          1                 151
##&nbsp;        SERIAL    cp WAFL_CPWF_SYNC_VVOL_SUPERBLOCK                    1                  13
##&nbsp;        SERIAL    cp WAFL_CPWF_SYNC_SUPERBLOCK                         2                  52
##&nbsp;        SERIAL    cp WAFL_CPWF_SPECIAL_INODES_DIRTY                    2                  41
##&nbsp;        SERIAL    cp WAFL_CPWF_DRAIN_DELETE_LOG                        2                  42
##&nbsp;        SERIAL    cp WAFL_CPWF_PURGE_FILE_COMMIT_HE                    1                  78
##&nbsp;        SERIAL    cp WAFL_CPWF_UPDATE_DF_SCORES                        1                  35
##&nbsp;        SERIAL    cp WAFL_CPWF_ALLOCATION_LOOP                        14                 361
##&nbsp;        SERIAL    cp WAFL_CPWF_UPDATE_CP_FREES                         1                  21
##&nbsp;        SERIAL    cp WAFL_CPWF_PREPARE_FSINFO_VOLIN                    2                 100
##&nbsp;============== ===== ============================== ==================== ===================
##&nbsp;         Class Queue Msg Type                       #Msgs (new+restarts)       Total CPU(us)
##&nbsp;============== ===== ============================== ==================== ===================
##&nbsp;        SERIAL    cp WAFL_CPWF_CPDONE_SPACE_CALLBAC                    2                  21
##&nbsp;        SERIAL    cp WAFL_DISCARD_SAFE_BITMAPS                         4                 299
##&nbsp;        SERIAL hipri WAFFINITY_RA_STRIPE_X                            13                 233
##&nbsp;        SERIAL    cp WAFFINITY_RA_STRIPE_X                            12                 192
##&nbsp;        SERIAL    cp WAFL_CLEAN_DINO_AND_DBUF                          2                  78
##&nbsp;        SERIAL    cp WAFL_CP_AWAIT_FLUSH_MSGS_DONE                     1                  10
##&nbsp;        SERIAL    cp WAFL_CP_AWAIT_WRITE_DONE                          1                  12
##&nbsp;        SERIAL    cp WAFL_CP_AWAIT_WRITE_STRIPE_RET                    2                  22
##&nbsp;        SERIAL    cp WAFL_CP_FLUSH_STRIPES_ASYNC                       1                 199
##&nbsp;        SERIAL    cp WAFL_CP_APPF_UPDATE_PLAYLIST_D                    1                  95
##&nbsp;        SERIAL    cp WAFL_CP_APPF_CHECK_PLAYLIST_IN                    1                  25
##&nbsp;        SERIAL    cp WAFL_POST_CP_RG_CHECK                             2                  22
##&nbsp;        SERIAL    cp WAFL_BLKFREE_VVOL_COMMIT                          4                7191
##&nbsp;        SERIAL    cp WAFL_BLKFREE_PVOL_COMMIT                          9                8905
##&nbsp;        SERIAL hipri WAFL_TIMEOUTS                                     4                  45
##&nbsp;        SERIAL hipri _WAFL_BACKDOOR                                    4                  53
##&nbsp;        SERIAL    cp RAIDIO_WRITE_DONE                                 9                 159
##&nbsp;        SERIAL    cp WAFL_WRITE_DONE                                   1                 745
##&nbsp;
##&nbsp;proc time (ms)
##&nbsp;                     proc      run     able      mtx mtx_able      slp      idl    pempt    total
##&nbsp;              wafl_exempt      145        6        0        0       20        0        0      172
##&nbsp;               wafl_hipri        0       18        0        0      132        0        0      151
##&nbsp;               wafl_lopri        0        0        0        0        0        0        0        0
##&nbsp;               wafl_cppri        0        0        0        0        0        0        0        0
##&nbsp;           wafl_mpcleaner        0        0        0        0        0        0        0        0
##&nbsp;   wafl_mpcleaner_thread1       14        0        0        0       28        0        0       43
##&nbsp;           wafl_guardrail        0        0        0        0        0        0        0        0
##&nbsp;     wafl_guardrail_hipri        0        0        0        0        0        0        0        0
##&nbsp;
##&nbsp;   wafl_mpcleaner_thread1: nmsgs_rcvd=        32		mpc_write_allocated =        291
##&nbsp;
##&nbsp;cp_xcleaner_pace.cp_begin_pace         =          0 / 100
##&nbsp;cp_xcleaner_pace.normal_phase_end_pace =          0 / 100
##&nbsp;cp_xcleaner_pace.vvol_phase_end_pace   =          0 / 100
##&nbsp;cp_xcleaner_pace.cp_end_phase          =          0 / 100
##&nbsp;cp_xcleaner_pace.max_pace              =          0 / 100
##&nbsp;cp_cppri_pace.cp_begin_pace         =          0 / 100
##&nbsp;cp_cppri_pace.normal_phase_end_pace =          0 / 100
##&nbsp;cp_cppri_pace.vvol_phase_end_pace   =          0 / 100
##&nbsp;cp_cppri_pace.cp_end_phase          =          0 / 100
##&nbsp;cp_cppri_pace.max_pace              =          0 / 100
##&nbsp;cp_waffi_pace.cp_begin_pace         =         10 / 100
##&nbsp;cp_waffi_pace.normal_phase_end_pace =         10 / 100
##&nbsp;cp_waffi_pace.vvol_phase_end_pace   =         10 / 100
##&nbsp;cp_waffi_pace.cp_end_phase          =         10 / 100
##&nbsp;cp_waffi_pace.max_pace              =         10 / 100
##&nbsp;
##&nbsp;cpp_affi_stats
##&nbsp;Phase                                   Class Queue   #Scheds     CPU (us)
##&nbsp;============================== ============== ===== ========= ============
##&nbsp;CP_PRE_P0                              SERIAL cp            1           60
##&nbsp;CP_P0_SNAP_DEL                         SERIAL cp            1          325
##&nbsp;CP_P1_CLEAN                            SERIAL cp            7         1829
##&nbsp;CP_P2V_INOFILE                         SERIAL cp            1           84
##&nbsp;CP_P2V_INO_PUB                         SERIAL cp            2           92
##&nbsp;CP_P2V_INO_PRI                         SERIAL cp            2           92
##&nbsp;CP_P2V_FSINFO                          SERIAL cp            6          319
##&nbsp;CP_P2V_DLOG2                           SERIAL cp            1           81
##&nbsp;CP_P2V_REFCOUNT                        SERIAL cp            1           81
##&nbsp;CP_P2V_TOPAA                           SERIAL cp            3          180
##&nbsp;CP_P2V_DF_SCORES_SUB                   SERIAL cp            1           83
##&nbsp;CP_P2V_BM                              SERIAL lopri         1          244
##&nbsp;CP_P2V_BM                              SERIAL hipri         1           41
##&nbsp;CP_P2V_BM                              SERIAL cp            3         4082
##&nbsp;CP_P2V_DF_SCORES                       SERIAL cp            3          200
##&nbsp;CP_P2V_VOLINFO                         SERIAL cp            3          252
##&nbsp;CP_P2V_CONT                            SERIAL cp            4          303
##&nbsp;CP_P2A_INOFILE                         SERIAL cp            1          146
##&nbsp;CP_P2A_INO                             SERIAL cp            3          107
##&nbsp;CP_P2A_DLOG1                           SERIAL hipri         1           84
##&nbsp;CP_P2A_DLOG1                           SERIAL cp            3        16472
##&nbsp;CP_P2A_DLOG2                           SERIAL cp            6          432
##&nbsp;CP_P2A_FSINFO                          SERIAL cp            6          298
##&nbsp;CP_P2A_REFCOUNT                        SERIAL cp            1           76
##&nbsp;CP_P2A_TOPAA                           SERIAL cp            3          231
##&nbsp;CP_P2A_BM                              SERIAL cp            2         1210
##&nbsp;CP_P2A_VOLINFO                         SERIAL cp            4          344
##&nbsp;CP_P2_FLUSH                            SERIAL lopri        17        25694
##&nbsp;CP_P2_FLUSH                            SERIAL hipri         4          151
##&nbsp;CP_P2_FLUSH                            SERIAL cp            5         2635
##&nbsp;CP_P3V_VOLINFO                         SERIAL lopri         2         2132
##&nbsp;CP_P3V_VOLINFO                         SERIAL hipri         2           66
##&nbsp;CP_P3V_VOLINFO                         SERIAL cp            2        10635
##&nbsp;CP_P3A_VOLINFO                         SERIAL lopri        12        20086
##&nbsp;CP_P3A_VOLINFO                         SERIAL hipri         4          220
##&nbsp;CP_P3A_VOLINFO                         SERIAL cp            6        43546
##&nbsp;CP_P3_FINISH                           SERIAL lopri         1         1148
##&nbsp;CP_P3_FINISH                           SERIAL hipri         1           30
##&nbsp;CP_P3_FINISH                           SERIAL cp            3         9941
##&nbsp;CP_P4_FINISH                           SERIAL cp            1           40
##&nbsp;OUT_OF_CP                              SERIAL lopri       328        47862
##&nbsp;OUT_OF_CP                              SERIAL hipri        23         1242
##&nbsp;OUT_OF_CP                              SERIAL cp            4         4660
##&nbsp;
##&nbsp;cp_ph.CP_SETUP                    =          9
##&nbsp;cp_ph.CP_PRE_P0                   =          0
##&nbsp;cp_ph.CP_P0_SNAP_DEL              =          0
##&nbsp;cp_ph.CP_P1_CLEAN                 =          9
##&nbsp;cp_ph.CP_P1_QUOTA                 =          0
##&nbsp;cp_ph.CP_IPU_DISK_ADD             =          0
##&nbsp;cp_ph.CP_P2V_INOFILE              =          0
##&nbsp;cp_ph.CP_P2V_INO_PUB              =          0
##&nbsp;cp_ph.CP_P2V_INO_PRI              =          0
##&nbsp;cp_ph.CP_P2V_FSINFO               =          0
##&nbsp;cp_ph.CP_P2V_DLOG1                =          0
##&nbsp;cp_ph.CP_P2V_DLOG2                =          0
##&nbsp;cp_ph.CP_P2V_REFCOUNT             =          0
##&nbsp;cp_ph.CP_P2V_TOPAA                =          0
##&nbsp;cp_ph.CP_P2V_DF_SCORES_SUB        =          0
##&nbsp;cp_ph.CP_P2V_BM                   =          9
##&nbsp;cp_ph.CP_P2V_SNAP                 =          0
##&nbsp;cp_ph.CP_P2V_DF_SCORES            =          0
##&nbsp;cp_ph.CP_P2V_VOLINFO              =          0
##&nbsp;cp_ph.CP_P2V_CONT                 =          0
##&nbsp;cp_ph.CP_P2A_INOFILE              =          0
##&nbsp;cp_ph.CP_P2A_INO                  =          0
##&nbsp;cp_ph.CP_P2A_DLOG1                =         18
##&nbsp;cp_ph.CP_P2A_HYA                  =          0
##&nbsp;cp_ph.CP_P2A_DLOG2                =          0
##&nbsp;cp_ph.CP_P2A_FSINFO               =          0
##&nbsp;cp_ph.CP_P2A_IPU_BITMAP_GROW      =          0
##&nbsp;cp_ph.CP_P2A_REFCOUNT             =          0
##&nbsp;cp_ph.CP_P2A_TOPAA                =          0
##&nbsp;cp_ph.CP_P2A_HYABC                =          0
##&nbsp;cp_ph.CP_P2A_BM                   =          0
##&nbsp;cp_ph.CP_P2A_SNAP                 =          0
##&nbsp;cp_ph.CP_P2A_VOLINFO              =         10
##&nbsp;cp_ph.CP_P2_FLUSH                 =         19
##&nbsp;cp_ph.CP_P2_FINISH                =          0
##&nbsp;cp_ph.CP_P3_WAIT                  =          0
##&nbsp;cp_ph.CP_P3V_VOLINFO              =         18
##&nbsp;cp_ph.CP_P3A_VOLINFO              =         57
##&nbsp;cp_ph.CP_P3_FINISH                =         10
##&nbsp;cp_ph.CP_P4_FINISH                =          0
##&nbsp;cp_ph.CP_P5_FINISH                =          0
##&nbsp;cp_ph.CP_TOTAL                    =        150
##&nbsp;
##&nbsp;cp_comp_stats.cp_num_cg_decomp   =          0
##&nbsp;cp_comp_stats.num_extra_cp_reads =          0
##&nbsp;cp_comp_stats.cpu_time_decomp    =          0us
##&nbsp;
##&nbsp;Blocks upgraded and dirtied due to IPU:
##&nbsp;                      blks upgraded            blks dirtied
##&nbsp;Level            incoming     in cp         incoming     in cp
##&nbsp;                ---------- ----------     ---------- ----------
##&nbsp;level change  =          0          0              0          0
##&nbsp;level 1       =          0          0              0          0
##&nbsp;level 2       =          0          0              0          0
##&nbsp;level 3       =          0          0              0          0
##&nbsp;level 4       =          0          0              0          0
##&nbsp;level 5       =          0          0              0          0
##&nbsp;level 6       =          0          0              0          0
##&nbsp;level 7       =          0          0              0          0
##&nbsp;Total         =          0          0              0          0
##&nbsp;
##&nbsp;cp_pw.cp_susp               =          0
##&nbsp;cp_pw.cp_read               =          0
##&nbsp;cp_pw.cp_zero               =          0
##&nbsp;fe_pw.load_buf_reads        =          0
##&nbsp;fe_pw.load_buf_zeros        =          0
##&nbsp;fe_pw.partial_writes        =          0
##&nbsp;fe_pw.whole_hole            =        200
##&nbsp;fe_pw.whole_disk            =          0
##&nbsp;fe_pw.hole                  =          0
##&nbsp;fe_pw.disk                  =          0
##&nbsp;fe_pw.extensions            =          0
##&nbsp;fe_pw.non_extensions        =          0
##&nbsp;fe_pw.completions           =          0
##&nbsp;fe_pw.failed_prefetch       =          0
##&nbsp;fe_pw.read_return           =          0
##&nbsp;fe_pw.read_cont             =          0
##&nbsp;fe_pw.decompression         =          0
##&nbsp;fe_pw.merge                 =          0
##&nbsp;fe_pw.merge_small_data      =          0
##&nbsp;fe_pw.merge_big_data        =          0
##&nbsp;fe_pw.merge_zero            =          0
##&nbsp;fe_pw.over_limit            =          0
##&nbsp;fe_pw.async_read            =          0
##&nbsp;fe_pw.async_read_failed     =          0
##&nbsp;fe_pw.raidio_avail          =          0
##&nbsp;
##&nbsp;cp_kahuna_grab_count        =          0
##&nbsp;cp_kahuna_grab_cycles       =          0
##&nbsp;cp_kahuna_stats[0]          = wafl_CP_proc (24344868)
##&nbsp;cp_kahuna_stats[1]          = rsm_netio_daemon (270976)
##&nbsp;cp_kahuna_stats[2]          = wafl_exempt00 (427708232)
##&nbsp;cp_kahuna_stats[3]          = wafl_hipri (408232)
##&nbsp;cp_kahuna_stats[4]          = rshd_0 (2801668)
##&nbsp;
##&nbsp;Bufs pinned for This cp            =         13
##&nbsp;Bufs pinned for Next cp            =         22
##&nbsp;Bufs pinned moved by aging         =          0
##&nbsp;Bufs pinned moved by scavenger     =          0
##&nbsp;Bufs pinned for Aggr               =         26
##&nbsp;Bufs dirtied for Aggr              =          4
##&nbsp;Bufs pinned for vvol               =          9
##&nbsp;Bufs dirtied for vvol              =          6
##&nbsp;Bufs Already pinned                =       7183
##&nbsp;Bufs incore not pinned             =        174
##&nbsp;Bufs pinned aging Q skipped        =          0
##&nbsp;Bufs not pinned because of Iron    =          0
##&nbsp;
##&nbsp;Stats for refcnthash:
##&nbsp;hash:sync_time                    =                    0
##&nbsp;hash:did_cp_wait                  =                   No
##&nbsp;hash:inserts                      =                    0
##&nbsp;hash:inserts updated              =                    0
##&nbsp;hash:already_present              =                    0
##&nbsp;hash:inserts failed nomem         =                    0
##&nbsp;hash:insertions failed disable    =                    0
##&nbsp;hash:lookups done by frees        =                    0
##&nbsp;hash:decrements done by frees     =                    0
##&nbsp;hash:pinned refcnt buffers        =                    0
##&nbsp;hash:pinned active map buffers    =                    0
##&nbsp;
##&nbsp;Current CP:
##&nbsp;added_cp_commitment_active =         19
##&nbsp;added_cp_commitment_refcnt =          0
##&nbsp;added_cp_commitment_cmap   =          0
##&nbsp;added_cp_commitment_ttmap  =          0
##&nbsp;added_cp_commitment_ownmap =          0
##&nbsp;added_cp_commitment_summary =         13
##&nbsp;added_cp_commitment_hya_active =          0
##&nbsp;cg_dirty_cnt               =          0
##&nbsp;clone_buf_req              =          0
##&nbsp;Next CP:
##&nbsp;added_cp_commitment_active =          0
##&nbsp;added_cp_commitment_refcnt =          0
##&nbsp;added_cp_commitment_cmap   =          0
##&nbsp;added_cp_commitment_ttmap  =          0
##&nbsp;added_cp_commitment_ownmap =          0
##&nbsp;added_cp_commitment_summary =          0
##&nbsp;added_cp_commitment_hya_active =          0
##&nbsp;cg_dirty_cnt               =          0
##&nbsp;clone_buf_req              =          0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cp_count' => '7483131'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_cpstats {
    my %args = @_;
    my ( $error_message, $error );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my @lines   = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    foreach my $line (@lines) {
       if ( $line =~ /(cp_count)\s*=\s*(\d+)/ ||  $line =~ /(cp_dirty_buf_high)\s*=\s*(\d+)/) {
           $result_array[0]->{to_key_name( item => $1 )} = $2;
       }
    }
    return \@result_array;
} # end sub wafl_cpstats

## @name ::wafl_eject_inode
##
## @summary Parses the raw output from the "wafl eject inode" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_eject_inode with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_eject_inode(%args);
## INPUTS TO METHOD CALL 'wafl_eject_inode' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'inode-number' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-private' </td> <td> 'private' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'snap-id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub wafl_eject_inode {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }

    return [];
}

## @name ::wafl_query_interact
##
## @summary Parses the raw output from the "wafl query interact" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_query_interact with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_query_interact(%args);
## INPUTS TO METHOD CALL 'wafl_query_interact' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'category' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub wafl_query_interact {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }

    return [];
}
## @name ::wafl_query_list
##
## @summary Parses the raw output from the "wafl query list" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_query_list with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_query_list(%args);
## INPUTS TO METHOD CALL 'wafl_query_list' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'category' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; wafl query list; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 =[
##		{
##		 'internal_info' => [
##				      '2e9bb2d7-2afb-11df-9e46-000c296a263a',
##				      '0'
##				    ],
##		 'aggregate' => 'a2',
##		 'category' => 'prev_cp'
##		}
##	      ];
##&nbsp;
sub wafl_query_list {
    my %args = @_;
    my ($error_message, $error);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my @lines  = get_lines(raw_output => $output, response_object => $resp);
    my @result_array;
    my @values;
    my $i = 0;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    foreach my $line (@lines) {
        $line =~ s/"//g;
        @values = split(/\s?,\s?/, $line);
        $result_array[$i]->{category}             = $values[0];
        $result_array[$i]->{aggregate}            = $values[1];
        $result_array[$i]->{internal_info}->[0]   = $values[2];
        $result_array[$i++]->{internal_info}->[1] = $values[3];
    }
    return \@result_array;
}    # end sub wafl_cpstats

## @name ::wafl_vref_status
##
## @summary Parses the raw output from the "wafl vref status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_vref_status with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_vref_status(%args);
## INPUTS TO METHOD CALL 'wafl_vref_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; wafl vref status -A dst_aggr ; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 =[
##		{
##		  'volume' => 'dst_aggr',
##		  'reference' => 'online vvols, maytag.ko::config_online_vvols'
##		}
##		{
##		  'volume' => 'd1',
##		  'reference' => 'vol remount, maytag.ko::config_do_mount_volume_phase1'
##		}
##		{
##		  'volume' => 'd1',
##		  'reference' => 'Delete Datawarehouse files in RW vol, maytag.ko::wafl_repl_rw_vvol_dw_delete'
##		}
##	      ];
##&nbsp;
sub wafl_vref_status {
	my %args = @_;
	my ($error_message, $error);
	my $resp   = $args{response_object};
	my $output = $resp->get_processed_output();
	my @lines  = get_lines(raw_output => $output, response_object => $resp);
	my @result_array;
	my @values;
	my $i = 0;

	$error = _parse_for_errors(
			raw_output    => $output,
			error_message => \$error_message
			);
	if ($error) {
		$error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
		$error->throw(text => $error_message, response_object => $resp);
	}
	my @heads;
	@lines = grep(!(/----/),@lines);
	foreach my $line (@lines) {
		if($line =~ /(Volume)\s+(Reference)/)
		{
			push(@heads,$1,$2);
		}else{
			@values = split(/\s+/, $line,2);
			$result_array[$i]->{to_key_name(item => $heads[0])}            = $values[0];
			$result_array[$i++]->{to_key_name(item => $heads[1])}          = $values[1];
			@values= undef;
		}
	}
	return \@result_array;
} # end sub wafl_vref_status

## @name ::wafl_vref_create
##
## @summary Parses the raw output from the "wafl eject inode" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_vref_create with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_vref_create(%args);
## INPUTS TO METHOD CALL 'wafl_vref_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'reason' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference which contains the parsed output.
## @examples
sub wafl_vref_create {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }

    return [];
}

## @name ::wafl_vref_destroy
##
## @summary Parses the raw output from the "wafl vref destroy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_vref_destroy with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_vref_destroy(%args);
## INPUTS TO METHOD CALL 'wafl_vref_destroy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'reason' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference which contains the parsed output.
## @examples
sub wafl_vref_destroy {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my ($error, @temp);
    my $errormsg;
    my $i = 0;
    if ($error = _parse_for_errors( 'raw_output'   => $raw_output,
                                    'command'      => $command,
                                    'error_message' => \$errormsg)) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $errormsg, response_object => $resp);
    }

    return [];
}

## @name ::wafl_nv_replay_stats
##
## @summary Parses the raw output from the "wafl nv_replay_stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_nv_replay_stats with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_nv_replay_stats(%args);
## INPUTS TO METHOD CALL 'wafl_nv_replay_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-d' </td> <td> 'dr-partner' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'partner-name' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'local' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; wafl nv_replay_stats -l -p; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;===========================================
##&nbsp;      NVLOG Replay Stats for Local
##&nbsp;===========================================
##&nbsp;fullness                    : 0%
##&nbsp;percent dead                : 0%
##&nbsp;log 0 entry count           : 0
##&nbsp;log 1 entry count           : 0
##&nbsp;CP commitment suspends      : 0
##&nbsp;Time to add NVLOG extension : 0 ms
##&nbsp;replay fully async mode     : enabled
##&nbsp;last replay cp mode         : undeclared
##&nbsp;coalesced count             : 0
##&nbsp;coll chain coalesced count  : 0
##&nbsp;single-collision count      : 0
##&nbsp;multi-collision count       : 0
##&nbsp;collision-chain-full count  : 0
##&nbsp;number of cgs decompressed  : 0
##&nbsp;decompression time          : 0 us
##&nbsp;num prefetches for CGs      : 0
##&nbsp;num fail prefetches for CGs : 0
##&nbsp;CG throttled during replay  : 0
##&nbsp;build time                  : 0 us
##&nbsp;exec time                   : 0 us
##&nbsp;load-only count             : 0
##&nbsp;total bytes consumed        : 0
##&nbsp;total phys nvram for log    : 0
##&nbsp;total useable nvram         : 0
##&nbsp;actual_fullness             : 0%
##&nbsp;
##&nbsp;Coalescing Histogram:
##&nbsp;        0        1        2        3
##&nbsp;        0        0        0        0
##&nbsp;
##&nbsp;        4        5        6        7
##&nbsp;        0        0        0        0
##&nbsp;
##&nbsp;        8        9       10       11
##&nbsp;        0        0        0        0
##&nbsp;
##&nbsp;       12       13       14       15
##&nbsp;        0        0        0        0
##&nbsp;
##&nbsp;       16   UNUSED   UNUSED   UNUSED
##&nbsp;        0        0        0        0
##&nbsp;
##&nbsp;No Coalescing Reasons:
##&nbsp;----------------------
##&nbsp;Non Write Ops           : 0
##&nbsp;Different Types         : 0
##&nbsp;FSID                    : 0
##&nbsp;FID                     : 0
##&nbsp;NoLog Data              : 0
##&nbsp;Too Long                : 0
##&nbsp;Offset                  : 0
##&nbsp;Inode                   : 0
##&nbsp;Miscellaneous           : 0
##&nbsp;
##&nbsp;Replay Histogram:
##&nbsp;        0        1        2        3        4        5        6        7
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;
##&nbsp;===========================================
##&nbsp;      NVLOG Replay Stats for Partner
##&nbsp;===========================================
##&nbsp;fullness                    : 0%
##&nbsp;percent dead                : 0%
##&nbsp;log 0 entry count           : 0
##&nbsp;log 1 entry count           : 0
##&nbsp;CP commitment suspends      : 0
##&nbsp;Time to add NVLOG extension : 0 ms
##&nbsp;replay fully async mode     : enabled
##&nbsp;last replay cp mode         : undeclared
##&nbsp;partner nvtoc disabled by   : inited
##&nbsp;coalesced count             : 0
##&nbsp;coll chain coalesced count  : 0
##&nbsp;single-collision count      : 0
##&nbsp;multi-collision count       : 0
##&nbsp;collision-chain-full count  : 0
##&nbsp;number of cgs decompressed  : 0
##&nbsp;decompression time          : 0 us
##&nbsp;num prefetches for CGs      : 0
##&nbsp;num fail prefetches for CGs : 0
##&nbsp;CG throttled during replay  : 0
##&nbsp;build time                  : 0 us
##&nbsp;exec time                   : 0 us
##&nbsp;load-only count             : 0
##&nbsp;total bytes consumed        : 0
##&nbsp;total phys nvram for log    : 0
##&nbsp;total useable nvram         : 0
##&nbsp;actual_fullness             : 0%
##&nbsp;
##&nbsp;Coalescing Histogram:
##&nbsp;        0        1        2        3
##&nbsp;        0        0        0        0
##&nbsp;
##&nbsp;        4        5        6        7
##&nbsp;        0        0        0        0
##&nbsp;
##&nbsp;        8        9       10       11
##&nbsp;        0        0        0        0
##&nbsp;
##&nbsp;       12       13       14       15
##&nbsp;        0        0        0        0
##&nbsp;
##&nbsp;       16   UNUSED   UNUSED   UNUSED
##&nbsp;        0        0        0        0
##&nbsp;
##&nbsp;No Coalescing Reasons:
##&nbsp;----------------------
##&nbsp;Non Write Ops           : 0
##&nbsp;Different Types         : 0
##&nbsp;FSID                    : 0
##&nbsp;FID                     : 0
##&nbsp;NoLog Data              : 0
##&nbsp;Too Long                : 0
##&nbsp;Offset                  : 0
##&nbsp;Inode                   : 0
##&nbsp;Miscellaneous           : 0
##&nbsp;
##&nbsp;Replay Histogram:
##&nbsp;        0        1        2        3        4        5        6        7
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144
##&nbsp;        0        0        0        0        0        0        0        0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'nvlog_replay_stats_for_local' => [
##&nbsp;                                                {
##&nbsp;                                                  'time_to_add_nvlog_extension' => '0 ms',
##&nbsp;                                                  'log_0_entry_count' => '0',
##&nbsp;                                                  'total_phys_nvram_for_log' => '0',
##&nbsp;                                                  'num_fail_prefetches_for_cgs' => '0',
##&nbsp;                                                  'replay_fully_async_mode' => 'enabled',
##&nbsp;                                                  'cp_commitment_suspends' => '0',
##&nbsp;                                                  'coll_chain_coalesced_count' => '0',
##&nbsp;                                                  'replay_histogram' => [
##&nbsp;                                                                          {
##&nbsp;                                                                            '<98304' => '0',
##&nbsp;                                                                            '<128' => '0',
##&nbsp;                                                                            '<448' => '0',
##&nbsp;                                                                            '7' => '0',
##&nbsp;                                                                            '<56' => '0',
##&nbsp;                                                                            '2' => '0',
##&nbsp;                                                                            '<65536' => '0',
##&nbsp;                                                                            '<196608' => '0',
##&nbsp;                                                                            '1' => '0',
##&nbsp;                                                                            '<3584' => '0',
##&nbsp;                                                                            '<48' => '0',
##&nbsp;                                                                            '<32768' => '0',
##&nbsp;                                                                            '<16384' => '0',
##&nbsp;                                                                            '<4096' => '0',
##&nbsp;                                                                            '<192' => '0',
##&nbsp;                                                                            '<262144' => '0',
##&nbsp;                                                                            '<8192' => '0',
##&nbsp;                                                                            '<64' => '0',
##&nbsp;                                                                            '<1536' => '0',
##&nbsp;                                                                            '<1024' => '0',
##&nbsp;                                                                            '<229376' => '0',
##&nbsp;                                                                            '<256' => '0',
##&nbsp;                                                                            '<28672' => '0',
##&nbsp;                                                                            '<320' => '0',
##&nbsp;                                                                            '<20480' => '0',
##&nbsp;                                                                            '0' => '0',
##&nbsp;                                                                            '<24576' => '0',
##&nbsp;                                                                            'unused' => '0',
##&nbsp;                                                                            '<512' => '0',
##&nbsp;                                                                            '<40' => '0',
##&nbsp;                                                                            '6' => '0',
##&nbsp;                                                                            '<163840' => '0',
##&nbsp;                                                                            '>262144' => '0',
##&nbsp;                                                                            '3' => '0',
##&nbsp;                                                                            '<2560' => '0',
##&nbsp;                                                                            '<3072' => '0',
##&nbsp;                                                                            '<16' => '0',
##&nbsp;                                                                            '<384' => '0',
##&nbsp;                                                                            '4' => '0',
##&nbsp;                                                                            '<12288' => '0',
##&nbsp;                                                                            '<2048' => '0',
##&nbsp;                                                                            '<32' => '0',
##&nbsp;                                                                            '<24' => '0',
##&nbsp;                                                                            '<131072' => '0',
##&nbsp;                                                                            '5' => '0'
##&nbsp;                                                                          }
##&nbsp;                                                                        ],
##&nbsp;                                                  'miscellaneous' => '0',
##&nbsp;                                                  'fullness' => '0%',
##&nbsp;                                                  'percent_dead' => '0%',
##&nbsp;                                                  'actual_fullness' => '0%',
##&nbsp;                                                  'total_useable_nvram' => '0',
##&nbsp;                                                  'multi_collision_count' => '0',
##&nbsp;                                                  'exec_time' => '0 us',
##&nbsp;                                                  'last_replay_cp_mode' => 'undeclared',
##&nbsp;                                                  'load_only_count' => '0',
##&nbsp;                                                  'build_time' => '0 us',
##&nbsp;                                                  'coalescing_histogram' => [
##&nbsp;                                                                              {
##&nbsp;                                                                                '11' => '0',
##&nbsp;                                                                                '7' => '0',
##&nbsp;                                                                                '2' => '0',
##&nbsp;                                                                                '1' => '0',
##&nbsp;                                                                                '0' => '0',
##&nbsp;                                                                                'unused' => '0',
##&nbsp;                                                                                '16' => '0',
##&nbsp;                                                                                '13' => '0',
##&nbsp;                                                                                '6' => '0',
##&nbsp;                                                                                '3' => '0',
##&nbsp;                                                                                '9' => '0',
##&nbsp;                                                                                '12' => '0',
##&nbsp;                                                                                '14' => '0',
##&nbsp;                                                                                '15' => '0',
##&nbsp;                                                                                '8' => '0',
##&nbsp;                                                                                '4' => '0',
##&nbsp;                                                                                '10' => '0',
##&nbsp;                                                                                '5' => '0'
##&nbsp;                                                                              }
##&nbsp;                                                                            ],
##&nbsp;                                                  'total_bytes_consumed' => '0',
##&nbsp;                                                  'log_1_entry_count' => '0',
##&nbsp;                                                  'decompression_time' => '0 us',
##&nbsp;                                                  'no_coalescing_reasons' => [
##&nbsp;                                                                               {
##&nbsp;                                                                                 'different_types' => '0',
##&nbsp;                                                                                 'inode' => '0',
##&nbsp;                                                                                 'miscellaneous' => '0',
##&nbsp;                                                                                 'non_write_ops' => '0',
##&nbsp;                                                                                 'fid' => '0',
##&nbsp;                                                                                 'too_long' => '0',
##&nbsp;                                                                                 'fsid' => '0',
##&nbsp;                                                                                 'nolog_data' => '0',
##&nbsp;                                                                                 'offset' => '0'
##&nbsp;                                                                               }
##&nbsp;                                                                             ],
##&nbsp;                                                  'single_collision_count' => '0',
##&nbsp;                                                  'number_of_cgs_decompressed' => '0',
##&nbsp;                                                  'cg_throttled_during_replay' => '0',
##&nbsp;                                                  'coalesced_count' => '0',
##&nbsp;                                                  'collision_chain_full_count' => '0',
##&nbsp;                                                  'num_prefetches_for_cgs' => '0'
##&nbsp;                                                }
##&nbsp;                                              ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'nvlog_replay_stats_for_partner' => [
##&nbsp;                                                  {
##&nbsp;                                                    'time_to_add_nvlog_extension' => '0 ms',
##&nbsp;                                                    'log_0_entry_count' => '0',
##&nbsp;                                                    'total_phys_nvram_for_log' => '0',
##&nbsp;                                                    'num_fail_prefetches_for_cgs' => '0',
##&nbsp;                                                    'replay_fully_async_mode' => 'enabled',
##&nbsp;                                                    'cp_commitment_suspends' => '0',
##&nbsp;                                                    'coll_chain_coalesced_count' => '0',
##&nbsp;                                                    'replay_histogram' => [
##&nbsp;                                                                            {
##&nbsp;                                                                              '<98304' => '0',
##&nbsp;                                                                              '<128' => '0',
##&nbsp;                                                                              '<448' => '0',
##&nbsp;                                                                              '7' => '0',
##&nbsp;                                                                              '<56' => '0',
##&nbsp;                                                                              '2' => '0',
##&nbsp;                                                                              '<65536' => '0',
##&nbsp;                                                                              '<196608' => '0',
##&nbsp;                                                                              '1' => '0',
##&nbsp;                                                                              '<3584' => '0',
##&nbsp;                                                                              '<48' => '0',
##&nbsp;                                                                              '<32768' => '0',
##&nbsp;                                                                              '<16384' => '0',
##&nbsp;                                                                              '<4096' => '0',
##&nbsp;                                                                              '<192' => '0',
##&nbsp;                                                                              '<262144' => '0',
##&nbsp;                                                                              '<8192' => '0',
##&nbsp;                                                                              '<64' => '0',
##&nbsp;                                                                              '<1536' => '0',
##&nbsp;                                                                              '<1024' => '0',
##&nbsp;                                                                              '<229376' => '0',
##&nbsp;                                                                              '<256' => '0',
##&nbsp;                                                                              '<28672' => '0',
##&nbsp;                                                                              '<320' => '0',
##&nbsp;                                                                              '<20480' => '0',
##&nbsp;                                                                              '0' => '0',
##&nbsp;                                                                              '<24576' => '0',
##&nbsp;                                                                              'unused' => '0',
##&nbsp;                                                                              '<512' => '0',
##&nbsp;                                                                              '<40' => '0',
##&nbsp;                                                                              '6' => '0',
##&nbsp;                                                                              '<163840' => '0',
##&nbsp;                                                                              '>262144' => '0',
##&nbsp;                                                                              '3' => '0',
##&nbsp;                                                                              '<2560' => '0',
##&nbsp;                                                                              '<3072' => '0',
##&nbsp;                                                                              '<16' => '0',
##&nbsp;                                                                              '<384' => '0',
##&nbsp;                                                                              '4' => '0',
##&nbsp;                                                                              '<12288' => '0',
##&nbsp;                                                                              '<2048' => '0',
##&nbsp;                                                                              '<32' => '0',
##&nbsp;                                                                              '<24' => '0',
##&nbsp;                                                                              '<131072' => '0',
##&nbsp;                                                                              '5' => '0'
##&nbsp;                                                                            }
##&nbsp;                                                                          ],
##&nbsp;                                                    'miscellaneous' => '0',
##&nbsp;                                                    'fullness' => '0%',
##&nbsp;                                                    'percent_dead' => '0%',
##&nbsp;                                                    'actual_fullness' => '0%',
##&nbsp;                                                    'total_useable_nvram' => '0',
##&nbsp;                                                    'multi_collision_count' => '0',
##&nbsp;                                                    'exec_time' => '0 us',
##&nbsp;                                                    'last_replay_cp_mode' => 'undeclared',
##&nbsp;                                                    'load_only_count' => '0',
##&nbsp;                                                    'build_time' => '0 us',
##&nbsp;                                                    'partner_nvtoc_disabled_by' => 'inited',
##&nbsp;                                                    'coalescing_histogram' => [
##&nbsp;                                                                                {
##&nbsp;                                                                                  '11' => '0',
##&nbsp;                                                                                  '7' => '0',
##&nbsp;                                                                                  '2' => '0',
##&nbsp;                                                                                  '1' => '0',
##&nbsp;                                                                                  '0' => '0',
##&nbsp;                                                                                  'unused' => '0',
##&nbsp;                                                                                  '16' => '0',
##&nbsp;                                                                                  '13' => '0',
##&nbsp;                                                                                  '6' => '0',
##&nbsp;                                                                                  '3' => '0',
##&nbsp;                                                                                  '9' => '0',
##&nbsp;                                                                                  '12' => '0',
##&nbsp;                                                                                  '14' => '0',
##&nbsp;                                                                                  '15' => '0',
##&nbsp;                                                                                  '8' => '0',
##&nbsp;                                                                                  '4' => '0',
##&nbsp;                                                                                  '10' => '0',
##&nbsp;                                                                                  '5' => '0'
##&nbsp;                                                                                }
##&nbsp;                                                                              ],
##&nbsp;                                                    'total_bytes_consumed' => '0',
##&nbsp;                                                    'log_1_entry_count' => '0',
##&nbsp;                                                    'decompression_time' => '0 us',
##&nbsp;                                                    'no_coalescing_reasons' => [
##&nbsp;                                                                                 {
##&nbsp;                                                                                   'different_types' => '0',
##&nbsp;                                                                                   'inode' => '0',
##&nbsp;                                                                                   'miscellaneous' => '0',
##&nbsp;                                                                                   'non_write_ops' => '0',
##&nbsp;                                                                                   'fid' => '0',
##&nbsp;                                                                                   'too_long' => '0',
##&nbsp;                                                                                   'fsid' => '0',
##&nbsp;                                                                                   'nolog_data' => '0',
##&nbsp;                                                                                   'offset' => '0'
##&nbsp;                                                                                 }
##&nbsp;                                                                               ],
##&nbsp;                                                    'single_collision_count' => '0',
##&nbsp;                                                    'number_of_cgs_decompressed' => '0',
##&nbsp;                                                    'cg_throttled_during_replay' => '0',
##&nbsp;                                                    'coalesced_count' => '0',
##&nbsp;                                                    'collision_chain_full_count' => '0',
##&nbsp;                                                    'num_prefetches_for_cgs' => '0'
##&nbsp;                                                  }
##&nbsp;                                                ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_nv_replay_stats {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my ( $error, @temp );
    my $errormsg;
    my $i = 0;
    if (
        $error = _parse_for_errors(
            'raw_output'    => $raw_output,
            'command'       => $command,
            'error_message' => \$errormsg
        )
      ) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
    }

    my @lines  = get_lines(raw_output => $raw_output, response_object => $resp);
    my ( @result_array, @values, @head );
    my ( $main_head, $sub_head );
    my $count = -1;
    my $flag  = 1;
    for ( my $j = 0 ; $j <= $#lines ; $j++ ) {
        if ( $lines[$j] =~ /(Node)[:]\s+(.*)/ ) {
            $count++;
            $flag = 0;
            $result_array[$count]->{ to_key_name( item => $1 ) } = $2;
        } elsif ( $lines[$j] =~ /^NVLOG.*/ ) {
            $count++ if ( $flag == 1 );
            $main_head = to_key_name( item => $lines[$j] );
        } elsif ( $lines[$j] =~ /^(.*\w+)\s+\:\s+(.*)$/ ) {
            my $head = $1;
            my $value = $2;
            $head =~ s/-/_/g;
            $result_array[$count]->{$main_head}[0]
              ->{ to_key_name( item => $head ) } = $value;
        } elsif ( $lines[$j] =~ /(Coalescing\s+Histogram)\:$/ ) {
            $sub_head = to_key_name( item => $1 );
            while ( ($lines[ $j + 1 ] !~ /(No\s+Coalescing\s+Reasons|Replay\s+Histogram):$/) ) {
                @head = split( /\s+/, $lines[ $j + 1 ] )
                  if ( $lines[ $j + 1 ] );
                @values = split( /\s+/, $lines[ $j + 2 ] )
                  if ( $lines[ $j + 2 ] );
                for ( my $i = 0 ; $i <= $#head ; $i++ ) {
                    $result_array[$count]->{$main_head}[0]->{$sub_head}[0]
                      ->{ to_key_name( item => $head[$i] ) } = $values[$i];
                }
                $j = $j + 2;
            }
        } elsif ( $lines[$j] =~ /(No\s+Coalescing\s+Reasons)\:$/ ) {
            $sub_head = to_key_name( item =>$1);
            while ($lines[ $j + 1 ] !~ /Replay\s+Histogram\:$/) {
                if ($lines[$j + 1] =~ /([\w\s]+\w+)\s+:\s*(\d+)/) {
                    my $head = $1;
                    my $value = $2;
                    $result_array[$count]->{$main_head}[0]->{$sub_head}[0]
                        ->{ to_key_name( item => $head ) } = $value;
                }
                $j++;
            }
        }elsif ( $lines[$j] =~ /(Replay\s+Histogram)\:$/ ) {
            $sub_head = to_key_name( item =>$1);
            for ( my $l = 1 ; $l <= 12 ; $l = $l + 2 ) {
                @head = split( /\s+/, $lines[ $j + $l ] )
                  if ( $lines[ $j + $l ] );
                @values = split( /\s+/, $lines[ $j + ( $l + 1 ) ] )
                  if ( $lines[ $j + ( $l + 1 ) ] );
                for ( my $i = 0 ; $i <= $#head ; $i++ ) {
                    $result_array[$count]->{$main_head}[0]->{$sub_head}[0]
                      ->{ to_key_name( item => $head[$i] ) } = $values[$i];
                }
            }
        } elsif ( $lines[$j] =~ /(Average)[:]\s+(\S+)/ ) {
            $result_array[$count]->{$main_head}[0]->{$sub_head}[0]
              ->{ to_key_name( item => $1 ) } = $2;
        }
    }
    return \@result_array;
}

## @name ::wafl_catalog_stats
##
## @summary Parses the raw output from the "wafl catalog stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_catalog_stats with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_catalog_stats(%args);
## INPUTS TO METHOD CALL 'wafl_catalog_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-l' </td> <td> 'local' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'partner-name' </td> <td> '1' </td> </tr>
## <tr> <td> '-z' </td> <td> 'ignore-holes' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'refcounts' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'input' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'reason' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Max cataloged 784
##&nbsp;local nvdma
##&nbsp;Total transactions 368122
##&nbsp;Total entries 306380
##&nbsp;Max entries in a tx 29
##&nbsp;remote nvdma
##&nbsp;Total transactions 353859
##&nbsp;Total entries 297393
##&nbsp;Max entries in a tx 29
##&nbsp;
##&nbsp;No Preload Stats for local wafl replay
##&nbsp;
##&nbsp;Preload Stats for partner wafl replay
##&nbsp;Preload times: Section 0 = 0, Section 1 = 261, Section 2 = 2600
##&nbsp;Preload Statistics
##&nbsp;Buffer Type User Indirect:
##&nbsp;Preloads = 392, Hits = 392, Misses = 6, Repeats = 0
##&nbsp;False Positive = 0.0%, Miss = 1.50%
##&nbsp;Buffer Type User Direct:
##&nbsp;Preloads = 0, Hits = 0, Misses = 1, Repeats = 0
##&nbsp;False Positive = 0.0%, Miss = 100.0%
##&nbsp;Buffer Type Directory direct:
##&nbsp;Preloads = 44, Hits = 43, Misses = 2, Repeats = 0
##&nbsp;False Positive = 2.27%, Miss = 4.44%
##&nbsp;Buffer Type Inodefile direct:
##&nbsp;Preloads = 17, Hits = 6, Misses = 5, Repeats = 0
##&nbsp;False Positive = 64.70%, Miss = 45.45%
##&nbsp;Buffer Type Meta-files:
##&nbsp;Preloads = 5, Hits = 5, Misses = 0, Repeats = 0
##&nbsp;False Positive = 0.0%, Miss = 0.0%
##&nbsp;Buffer Type Sundry:
##&nbsp;Preloads = 0, Hits = 0, Misses = 2, Repeats = 0
##&nbsp;False Positive = 0.0%, Miss = 100.0%
##&nbsp;Cumulative Total
##&nbsp;Preloads = 458, Hits = 446, Misses = 16
##&nbsp;False Positive = 2.62%, Miss = 3.46%
##&nbsp;Decompression User Direct reads = 0, RAID IOs = 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;
##&nbsp;my $expected_output = [
##&nbsp;          {
##&nbsp;            'preload_stats_for_partner_wafl_replay' => [
##&nbsp;                                                         {
##&nbsp;                                                           'buffer_type_directory_direct' => [
##&nbsp;                                                                                               {
##&nbsp;                                                                                                 'preloads' => '44',
##&nbsp;                                                                                                 'misses' => '2',
##&nbsp;                                                                                                 'miss' => '4.44%',
##&nbsp;                                                                                                 'hits' => '43',
##&nbsp;                                                                                                 'repeats' => '0',
##&nbsp;                                                                                                 'false_positive' => '2.27%'
##&nbsp;                                                                                               }
##&nbsp;                                                                                             ],
##&nbsp;                                                           'buffer_type_user_indirect' => [
##&nbsp;                                                                                            {
##&nbsp;                                                                                              'preloads' => '392',
##&nbsp;                                                                                              'misses' => '6',
##&nbsp;                                                                                              'miss' => '1.50%',
##&nbsp;                                                                                              'hits' => '392',
##&nbsp;                                                                                              'repeats' => '0',
##&nbsp;                                                                                              'false_positive' => '0.0%'
##&nbsp;                                                                                            }
##&nbsp;                                                                                          ],
##&nbsp;                                                           'buffer_type_meta_files' => [
##&nbsp;                                                                                         {
##&nbsp;                                                                                           'preloads' => '5',
##&nbsp;                                                                                           'misses' => '0',
##&nbsp;                                                                                           'miss' => '0.0%',
##&nbsp;                                                                                           'hits' => '5',
##&nbsp;                                                                                           'repeats' => '0',
##&nbsp;                                                                                           'false_positive' => '0.0%'
##&nbsp;                                                                                         }
##&nbsp;                                                                                       ],
##&nbsp;                                                           'preload_times' => [
##&nbsp;                                                                                {
##&nbsp;                                                                                  'section_2' => '2600',
##&nbsp;                                                                                  'section_0' => '0',
##&nbsp;                                                                                  'section_1' => '261'
##&nbsp;                                                                               }
##&nbsp;                                                                              ],
##&nbsp;                                                          'cumulative_total' => [
##&nbsp;                                                                                   {
##&nbsp;                                                                                     'preloads' => '458',
##&nbsp;                                                                                     'decompression_user_direct_reads' => '0',
##&nbsp;                                                                                    'misses' => '16',
##&nbsp;                                                                                     'raid_ios' => '0',
##&nbsp;                                                                                     'miss' => '3.46%',
##&nbsp;                                                                                     'hits' => '446',
##&nbsp;                                                                                     'false_positive' => '2.62%'
##&nbsp;                                                                                   }
##&nbsp;                                                                                 ],
##&nbsp;                                                           'buffer_type_inodefile_direct' => [
##&nbsp;                                                                                               {
##&nbsp;                                                                                                 'preloads' => '17',
##&nbsp;                                                                                                 'misses' => '5',
##&nbsp;                                                                                                 'miss' => '45.45%',
##&nbsp;                                                                                                 'hits' => '6',
##&nbsp;                                                                                                 'repeats' => '0',
##&nbsp;                                                                                                 'false_positive' => '64.70%'
##&nbsp;                                                                                               }
##&nbsp;                                                                                             ],
##&nbsp;                                                           'buffer_type_user_direct' => [
##&nbsp;                                                                                          {
##&nbsp;                                                                                            'preloads' => '0',
##&nbsp;                                                                                            'misses' => '1',
##&nbsp;                                                                                            'miss' => '100.0%',
##&nbsp;                                                                                            'hits' => '0',
##&nbsp;                                                                                            'repeats' => '0',
##&nbsp;                                                                                            'false_positive' => '0.0%'
##&nbsp;                                                                                          }
##&nbsp;                                                                                        ],
##&nbsp;                                                           'buffer_type_sundry' => [
##&nbsp;                                                                                     {
##&nbsp;                                                                                       'preloads' => '0',
##&nbsp;                                                                                       'misses' => '2',
##&nbsp;                                                                                       'miss' => '100.0%',
##&nbsp;                                                                                       'hits' => '0',
##&nbsp;                                                                                       'repeats' => '0',
##&nbsp;                                                                                       'false_positive' => '0.0%'
##&nbsp;                                                                                     }
##&nbsp;                                                                                   ]
##&nbsp;                                                         }
##&nbsp;                                                       ],
##&nbsp;            'remote_nvdma' => [
##&nbsp;                                {
##&nbsp;                                  'max_entries_in_a_tx' => '29',
##&nbsp;                                  'total_transactions' => '353859',
##&nbsp;                                  'total_entries' => '297393'
##&nbsp;                                }
##&nbsp;                              ],
##&nbsp;            'local_nvdma' => [
##&nbsp;                               {
##&nbsp;                                 'max_entries_in_a_tx' => '29',
##&nbsp;                                 'total_transactions' => '368122',
##&nbsp;                                 'total_entries' => '306380'
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'max_cataloged' => '784'
##&nbsp;          }
##&nbsp;        ];

sub wafl_catalog_stats {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my ( $error, @temp );
    my $errormsg;
    my $i = 0;
    if (
        $error = _parse_for_errors(
            'raw_output'    => $raw_output,
            'command'       => $command,
            'error_message' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
    }
    my @lines  = get_lines(raw_output => $raw_output, response_object => $resp);
    my ( @result_array, @array, @values );
    my $count = 0;
    my ( $main_head, $head );
    foreach my $line (@lines) {
        if ( $line =~ /(Max\s+cataloged)\s+(\d+)/ ) {
            $result_array[$count]->{ to_key_name( item => $1 ) } = $2;
        } elsif ( $line =~
/(local\s+nvdma|remote\s+nvdma|Preload\s+Stats\s+for\s+local\s+wafl\s+replay|Preload\s+Stats\s+for\s+partner\s+wafl\s+replay)/) {
            $main_head = to_key_name( item => $1 );
        } elsif ( $line =~
/(Total\s+transactions|Total\s+entries|Max\s+entries\s+in\s+a\s+tx)\s+(.*)/) {
            $result_array[$count]->{$main_head}->[0]
              ->{ to_key_name( item => $1 ) } = $2;
        } elsif ( $line =~ /(Preload\s+times)\:\s+(.*)/ ) {
            my $pre_head = to_key_name( item => $1 );
            my $value = $2;
            @array = split( /\,\s+/, $value );
            foreach my $arr (@array) {
                @values = split( /\s+\=\s+/, $arr );
                $result_array[$count]->{$main_head}->[0]->{$pre_head}[0]
                  ->{ to_key_name( item => $values[0] ) } = $values[1];
            }
        } elsif ( $line =~
/(Buffer\s+Type\s+User\s+Indirect|Buffer\s+Type\s+User\s+Direct|Buffer\s+Type\s+Directory\s+direct|Buffer\s+Type\s+Inodefile\s+direct|Buffer\s+Type\s+Meta-files|Buffer\s+Type\s+Sundry|Cumulative\s+Total)/) {
            $head = to_key_name( item => $1 );
        } elsif ( $line =~ /Preloads\s+\=.*|False\s+Positive.*/ ) {
            @array = split( /\,\s+/, $line );
            $head =~ s/\-/_/;
            foreach my $arr (@array) {
                @values = split( /\s+\=\s+/, $arr );
                $result_array[$count]->{$main_head}->[0]->{$head}[0]
                  ->{ to_key_name( item => $values[0] ) } = $values[1];
            }
        } elsif ( $line =~
/(Decompression\s+User\s+Direct\s+reads)\s+[=]\s+(\d+)\,\s+(\w+\s+\w+)\s+[=]\s+(\d+)/) {
            $result_array[$count]->{$main_head}->[0]->{$head}[0]
              ->{ to_key_name( item => $1 ) } = $2;
            $result_array[$count]->{$main_head}->[0]->{$head}[0]
              ->{ to_key_name( item => $3 ) } = $4;
        }
    }

    return \@result_array;
}

## @name ::wafl_swarmstats
##
## @summary Parses the raw output from the "wafl swarmstats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_swarmstats with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_swarmstats(%args);
## INPUTS TO METHOD CALL 'wafl_swarmstats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'dr-partner' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'reason' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;WAFL NVLOG REPLAY STATS
##&nbsp;Messages Swarmed: 12995
##&nbsp;Swarms: 2253
##&nbsp;Skipped entries: 0
##&nbsp;LoadRetry during modify: 0
##&nbsp;
##&nbsp;Message Detail:
##&nbsp;Message            Count   Cumu. Time Time/Op
##&nbsp;                          Across CPU's
##&nbsp;-------            ----- ------------ -------
##&nbsp;WAFL_SETATTR        1678          287      0
##&nbsp;WAFL_WRITE          8940        33824      3
##&nbsp;
##&nbsp;Selected Histograms (time in ms):
##&nbsp;WAFL_SETATTR
##&nbsp;0        1        2        3        4        5        6        7
##&nbsp;1504      120       25        8       12        9        0        0
##&nbsp;<16      <24      <32      <40      <48      <56      <64   UNUSED
##&nbsp;0        0        0        0        0        0        0        0
##&nbsp;<128     <192     <256     <320     <384     <448     <512   UNUSED
##&nbsp;0        0        0        0        0        0        0        0
##&nbsp;<1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED
##&nbsp;0        0        0        0        0        0        0        0
##&nbsp;<8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED
##&nbsp;0        0        0        0        0        0        0        0
##&nbsp;<65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144
##&nbsp;0        0        0        0        0        0        0        0
##&nbsp;Average:        0.17
##&nbsp;
##&nbsp;WAFL_WRITE
##&nbsp;0        1        2        3        4        5        6        7
##&nbsp;1519      993     1033     1197     1071      857      673      403
##&nbsp;<16      <24      <32      <40      <48      <56      <64   UNUSED
##&nbsp;1144       50        0        0        0        0        0        0
##&nbsp;<128     <192     <256     <320     <384     <448     <512   UNUSED
##&nbsp;0        0        0        0        0        0        0        0
##&nbsp;<1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED
##&nbsp;0        0        0        0        0        0        0        0
##&nbsp;<8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED
##&nbsp;0        0        0        0        0        0        0        0
##&nbsp;<65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144
##&nbsp;0        0        0        0        0        0        0        0
##&nbsp;Average:        3.78
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'wafl_nvlog_reply_stats' => [
##&nbsp;                                          {
##&nbsp;                                            'skipped_entries' => '0',
##&nbsp;                                            'swarms' => '2253',
##&nbsp;                                            'loadretry_during_modify' => '0',
##&nbsp;                                            'messages_swarmed' => '12995'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;            'message_detail' => [
##&nbsp;                                  {
##&nbsp;                                    'count' => '1678',
##&nbsp;                                    'cumu_across' => '-',
##&nbsp;                                    'time_cpus' => '287',
##&nbsp;                                    'time_op' => '0',
##&nbsp;                                    'message' => 'WAFL_SETATTR'
##&nbsp;                                  },
##&nbsp;                                  {
##&nbsp;                                    'count' => '8940',
##&nbsp;                                    'cumu_across' => '-',
##&nbsp;                                    'time_cpus' => '33824',
##&nbsp;                                    'time_op' => '3',
##&nbsp;                                    'message' => 'WAFL_WRITE'
##&nbsp;                                  }
##&nbsp;                                ],
##&nbsp;            'selected_histograms' => [
##&nbsp;                                       {
##&nbsp;                                         'wafl_setattr' => [
##&nbsp;                                                             {
##&nbsp;                                                               '<98304' => '0',
##&nbsp;                                                               '<128' => '0',
##&nbsp;                                                               '<448' => '0',
##&nbsp;                                                               '7' => '0',
##&nbsp;                                                               '<56' => '0',
##&nbsp;                                                               '2' => '25',
##&nbsp;                                                               '<65536' => '0',
##&nbsp;                                                               '<196608' => '0',
##&nbsp;                                                               '1' => '120',
##&nbsp;                                                               '<3584' => '0',
##&nbsp;                                                               '<48' => '0',
##&nbsp;                                                               '<32768' => '0',
##&nbsp;                                                               '<16384' => '0',
##&nbsp;                                                               '<4096' => '0',
##&nbsp;                                                               '<192' => '0',
##&nbsp;                                                               '<262144' => '0',
##&nbsp;                                                               '<8192' => '0',
##&nbsp;                                                               '<64' => '0',
##&nbsp;                                                               '<1536' => '0',
##&nbsp;                                                               '<1024' => '0',
##&nbsp;                                                               '<229376' => '0',
##&nbsp;                                                               '<256' => '0',
##&nbsp;                                                               '<28672' => '0',
##&nbsp;                                                               '<320' => '0',
##&nbsp;                                                               '<20480' => '0',
##&nbsp;                                                               '0' => '1504',
##&nbsp;                                                               '<24576' => '0',
##&nbsp;                                                               'unused' => '0',
##&nbsp;                                                               '<512' => '0',
##&nbsp;                                                               '<40' => '0',
##&nbsp;                                                               '6' => '0',
##&nbsp;                                                               '<163840' => '0',
##&nbsp;                                                               '>262144' => '0',
##&nbsp;                                                               '3' => '8',
##&nbsp;                                                               '<2560' => '0',
##&nbsp;                                                               '<3072' => '0',
##&nbsp;                                                               '<16' => '0',
##&nbsp;                                                               '<384' => '0',
##&nbsp;                                                               '4' => '12',
##&nbsp;                                                               'average' => '0.17',
##&nbsp;                                                               '<12288' => '0',
##&nbsp;                                                               '<2048' => '0',
##&nbsp;                                                               '<32' => '0',
##&nbsp;                                                               '<24' => '0',
##&nbsp;                                                               '<131072' => '0',
##&nbsp;                                                               '5' => '9'
##&nbsp;                                                             }
##&nbsp;                                                           ],
##&nbsp;                                         'wafl_write' => [
##&nbsp;                                                           {
##&nbsp;                                                             '<98304' => '0',
##&nbsp;                                                             '<128' => '0',
##&nbsp;                                                             '<448' => '0',
##&nbsp;                                                             '7' => '403',
##&nbsp;                                                             '<56' => '0',
##&nbsp;                                                             '2' => '1033',
##&nbsp;                                                             '<65536' => '0',
##&nbsp;                                                             '<196608' => '0',
##&nbsp;                                                             '1' => '993',
##&nbsp;                                                             '<3584' => '0',
##&nbsp;                                                             '<48' => '0',
##&nbsp;                                                             '<32768' => '0',
##&nbsp;                                                             '<16384' => '0',
##&nbsp;                                                             '<4096' => '0',
##&nbsp;                                                             '<192' => '0',
##&nbsp;                                                             '<262144' => '0',
##&nbsp;                                                             '<8192' => '0',
##&nbsp;                                                             '<64' => '0',
##&nbsp;                                                             '<1536' => '0',
##&nbsp;                                                             '<1024' => '0',
##&nbsp;                                                             '<229376' => '0',
##&nbsp;                                                             '<256' => '0',
##&nbsp;                                                             '<28672' => '0',
##&nbsp;                                                             '<320' => '0',
##&nbsp;                                                             '<20480' => '0',
##&nbsp;                                                             '0' => '1519',
##&nbsp;                                                             '<24576' => '0',
##&nbsp;                                                             'unused' => '0',
##&nbsp;                                                             '<512' => '0',
##&nbsp;                                                             '<40' => '0',
##&nbsp;                                                             '6' => '673',
##&nbsp;                                                             '<163840' => '0',
##&nbsp;                                                             '>262144' => '0',
##&nbsp;                                                             '3' => '1197',
##&nbsp;                                                             '<2560' => '0',
##&nbsp;                                                             '<3072' => '0',
##&nbsp;                                                             '<16' => '1144',
##&nbsp;                                                             '<384' => '0',
##&nbsp;                                                             '4' => '1071',
##&nbsp;                                                             'average' => '3.78',
##&nbsp;                                                             '<12288' => '0',
##&nbsp;                                                             '<2048' => '0',
##&nbsp;                                                             '<32' => '0',
##&nbsp;                                                             '<24' => '50',
##&nbsp;                                                             '<131072' => '0',
##&nbsp;                                                             '5' => '857'
##&nbsp;                                                           }
##&nbsp;                                                         ]
##&nbsp;                                       }
##&nbsp;                                     ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub wafl_swarmstats {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my ( $error, @temp );
    my $errormsg;
    my $i = 0;
    if (
        $error = _parse_for_errors(
            'raw_output'    => $raw_output,
            'command'       => $command,
            'error_message' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
    }

    my @lines = get_lines( raw_output => $raw_output, response_object => $resp );
    my ( @result_array, @main_head, @values, @head );
    my $limit = 0;
    my $count = -1;
    my $flag  = 1;
    my ( $main_head, $head, $sub_head );
    for ( my $j = 0 ; $j <= $#lines ; $j++ ) {
        if ( $lines[$j] =~ /(Node)[:]\s+(.*)/ ) {
            $count++;
            $limit = 0;
            $flag  = 0;
            $result_array[$count]->{ to_key_name( item => $1 ) } = $2;
        } elsif ( $lines[$j] =~ /WAFL\s+NVLOG\s+REPLAY\s+STATS/ ) {
            if ( $flag == 1 ) {
                $count++;
                $limit = 0;
            }
        } elsif ( $lines[$j] =~
/(Messages\s+Swarmed|Swarms|Skipped\s+entries|LoadRetry\s+during\s+modify)[:]\s+(\d+)/) {
            $result_array[$count]->{wafl_nvlog_reply_stats}[0]
              ->{ to_key_name( item => $1 ) } = $2;
        } elsif ( $lines[$j] =~ /Message\s+Count.*/ ) {
            @main_head = split( /\s+/, $lines[$j] );
        } elsif ( $lines[$j] =~ /(WAFL_.*)\s+.*/ ) {
            @values = split( /\s+/, $lines[$j] );
            for ( my $i = 0 ; $i <= $#main_head ; $i++ ) {
                $main_head[$i] =~ s/Cumu\./cumu_across/;
                $main_head[$i] =~ s/Time\/Op/time_op/;
                $main_head[$i] =~ s/Time/time_cpus/;

                if ( $#main_head != $#values ) {

            #Since the 'cumu_across' column values are not there replcing by '-'
                    splice( @values, 2, 0, '-' );
                }
                $result_array[$count]->{message_detail}[$limit]
                  ->{ to_key_name( item => $main_head[$i] ) } = $values[$i];
            }
            $limit++;
        } elsif ( $lines[$j] =~ /^(WAFL_.*\w+)$/ ) {
            $sub_head = to_key_name( item => $1 );
            while ( $lines[ $j + 1 ] !~ /(Average)[:]\s+(\d+)/ ) {
                @head = split( /\s+/, $lines[ $j + 1 ] )
                  if ( $lines[ $j + 1 ] );
                @values = split( /\s+/, $lines[ $j + 2 ] )
                  if ( $lines[ $j + 2 ] );
                for ( my $i = 0 ; $i <= $#head ; $i++ ) {
                    $result_array[$count]->{selected_histograms}[0]
                      ->{$sub_head}[0]->{ to_key_name( item => $head[$i] ) } =
                      $values[$i];
                }
                $j = $j + 2;
            }
        } elsif ( $lines[$j] =~ /(Average)[:]\s+(\S+)/ ) {
            $result_array[$count]->{selected_histograms}[0]->{$sub_head}[0]
              ->{ to_key_name( item => $1 ) } = $2;
        }
    }
    return \@result_array;
}

## @name ::wafl_fspool_create
##
## @summary Parses the raw output from the "wafl fspool create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_fspool_create with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_fspool_create(%args);
## INPUTS TO METHOD CALL 'wafl_fspool_create' ON APISET OBJECT
## <table border = "1" >
## <tr>
##   <th>OPTION</th>
##   <th>PROGRAMMATIC NAME</th>
##   <th>Description</th>
## </tr>
## <tr>
##   <td> No switch for this key </td>
##   <td> 'volume-name'        </td>
##   <td> format as /vol/volname or /aggr/aggrname </td>
## </tr>
## <tr>
##   <td> No switch for this key </td>
##   <td> 'poolname'          </td>
##   <td></td>
## </tr>
## <tr>
##   <td> No switch for this key </td>
##   <td> 'reserve-holes'     </td>
##   <td></td>
## </tr>
## <tr>
##   <td> No switch for this key </td>
##   <td> 'reserve-overwrite' </td>
##   <td></td>
## </tr>
## <tr>
##   <td> No switch for this key </td>
##   <td> 'nblocks'     </td>
##   <td> (deprecated)</td>
## </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; wafl fspool create /vol/vol0 p1 10 20; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Created Flexible space pool, ID = 1030
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'created_flexible_space_pool_id' => '1030'
##&nbsp;          }
##&nbsp;       ];
##&nbsp;
sub wafl_fspool_create {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my ( $error, @temp, @result_array, @lines,);
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'    => $raw_output,
            'command'       => $command,
            'error_message' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    @lines = get_lines( raw_output => $raw_output, response_object => $resp );

    foreach my $line (@lines) {
        if ($line =~ /(Created.*pool),\s+(\w+)\s[=]\s+(\d+)/) {
            $result_array[0]->{ to_key_name( item => $1 . $2 ) } = $3;
            last;
        }
    }

    return \@result_array;
}

## @name ::wafl_fspool_list
##
## @summary Parses the raw output from the "wafl fspool list" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_fspool_list with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_fspool_list(%args);
## INPUTS TO METHOD CALL 'wafl_fspool_list' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'poolid' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; wafl fspool list /vol/vol0 1030; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Pool Name: p1
##&nbsp;Pool ID: 1030 in volume with FSID 3965359102
##&nbsp;Parent Pool: 100 in volume with FSID 3965359102
##&nbsp;Pool Avail space: 50 blocks
##&nbsp;Pool Overwrite Reserve: 0 blocks
##&nbsp;Pool Policies: 0x1
##&nbsp;Pool create time: Fri, 13 Sep 2013 04:44:52 GMT
##&nbsp;Pool modify time: Fri, 13 Sep 2013 04:44:52 GMT
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pool_list' => [
##&nbsp;                             {
##&nbsp;                               'pool_overwrite_reserve' => '0 blocks',
##&nbsp;                               'pool_policies' => '0x1',
##&nbsp;                               'parent_pool' => '100 in volume with FSID 3965359102',
##&nbsp;                               'pool_modify_time' => 'Fri, 13 Sep 2013 04:44:52 GMT',
##&nbsp;                               'pool_id' => '1030 in volume with FSID 3965359102',
##&nbsp;                               'pool_create_time' => 'Fri, 13 Sep 2013 04:44:52 GMT',
##&nbsp;                               'pool_avail_space' => '50 blocks',
##&nbsp;                               'pool_name' => 'p1'
##&nbsp;                            }
##&nbsp;                          ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_fspool_list {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my ( $error, @result_array, @lines );
    my $errormsg;
    my $count = -1;
    if (
        $error = _parse_for_errors(
            'raw_output'    => $raw_output,
            'command'       => $command,
            'error_message' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    @lines = get_lines( raw_output => $raw_output, response_object => $resp );
    foreach my $line (@lines) {
        if ( $line =~ /^(Pool\s+Name):\s+(.*)/ ) {
            $count++;
            $result_array[0]->{pool_list}[$count]
              ->{ to_key_name( item => $1 ) } = $2;
        }
        elsif ( $line =~ /(.*):\s+(.*)/ ) {
            $result_array[0]->{pool_list}[$count]
              ->{ to_key_name( item => $1 ) } = $2;
        }
        if ( $line =~ /Displayed\s+(\d+)\s+(.*)/ ) {
            $result_array[0]->{ to_key_name( item => $2 ) } = $1;
        }
    }
    $count++;

    return \@result_array;
}

## @name ::wafl_fspool_mkdefault
##
## @summary Parses the raw output from the "wafl fspool mkdefault" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_fspool_mkdefault with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_fspool_mkdefault(%args);
## INPUTS TO METHOD CALL 'wafl_fspool_mkdefault' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'poolid' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; wafl fspool mkdefault /vol/vol0 1030; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Flexible space pool, ID = 1030 will henceforth be used for all dirties in vol0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'flexible_space_pool_id' => '1030'
##&nbsp;          }
##&nbsp;       ];
##&nbsp;
sub wafl_fspool_mkdefault {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my ( $error, @result_array, @lines);
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'    => $raw_output,
            'command'       => $command,
            'error_message' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    @lines = get_lines( raw_output => $raw_output, response_object => $resp );
    $lines[0] =~ /(Flexible.*),\s+(\w+)\s+=\s+(\d+).*/;
    $result_array[0]->{ to_key_name( item => $1 . $2 ) } = $3;
    return \@result_array;
}

## @name ::wafl_fspool_resize
##
## @summary Parses the raw output from the "wafl fspool resize" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_fspool_resize with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_fspool_resize(%args);
## INPUTS TO METHOD CALL 'wafl_fspool_resize' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'poolid' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-reserve-holes' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-reserve-overwrite' </td> </tr>
## <tr> <td> No switch for this key (deprecated) </td> <td> 'new-size-blocks' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; wafl fspool resize /vol/vol0 1030 20 30; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Adjusted Flexible space pool, ID = 1030, rsrv_holes by +10 blocks and rsrv_overwrite by +10 blocks
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'adjusted_flexible_space_pool_id' => '1030',
##&nbsp;            'capacity_by' => '+10 blocks'
##&nbsp;          }
##&nbsp;       ];
##&nbsp;
sub wafl_fspool_resize {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my ( $error, @result_array, @lines);
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'    => $raw_output,
            'command'       => $command,
            'error_message' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    @lines = get_lines( raw_output => $raw_output, response_object => $resp );

    $lines[0] =~
      /(Adjusted.*pool),\s+(\w+)\s+[=]\s+(\d+)\,\s+(rsrv_holes).*(\d+).*(rsrv_overwrite).*(\d+)/;
    $result_array[0]->{ to_key_name( item => $1 . $2 ) } = $3;
    $result_array[0]->{ to_key_name( item => $4 ) } = $5;
    $result_array[0]->{ to_key_name( item => $6 ) } = $7;

    return \@result_array;
}
## @name ::wafl_fspool_destroy
##
## @summary Parses the raw output from the "wafl fspool destroy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_fspool_destroy with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_fspool_destroy(%args);
## INPUTS TO METHOD CALL 'wafl_fspool_destroy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'poolid' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; wafl fspool destroy /vol/vol0 1030; result -l
##&nbsp;
##&nbsp;-------- Raw Output ------
##&nbsp;
##&nbsp;Destroyed Flexible space pool, ID = 1030
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'destroyed_flexible_space_pool_id' => '1030'
##&nbsp;          }
##&nbsp;       ];
##&nbsp;
sub wafl_fspool_destroy {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my ( $error, @result_array, @lines);
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'    => $raw_output,
            'command'       => $command,
            'error_message' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    @lines = get_lines( raw_output => $raw_output, response_object => $resp );
    $lines[0] =~ /(Destroyed.*pool),\s+(\w+)\s+=\s+(\d+)/;
    $result_array[0]->{ to_key_name( item => $1 . $2 ) } = $3;

    return \@result_array;
}

## @name ::wafl_hya_cbi_show
##
## @summary Parses the raw output from the "wafl hya cbi show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_hya_cbi_show with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_hya_cbi_show(%args);
## INPUTS TO METHOD CALL 'wafl_hya_cbi_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'hybrid-aggregate-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; wafl hya cbi show aggr0; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Header: Version 2, CBI Group Size 8, evict_nextblk 107228320, stale_cmap_blks 18106685, container check state 1, container check CP count 734692
##&nbsp;                     Start Time                       CP Count
##&nbsp;  Eviction Stamp[0]: Wed Oct 29 04:01:23 EDT 2014       618828
##&nbsp;  Eviction Stamp[1]: Wed Dec 31 19:00:00 EST 1969            0
##&nbsp;  Eviction Stamp[2]: Wed Dec 31 19:00:00 EST 1969            0
##&nbsp;  Eviction Stamp[3]: Wed Dec 31 19:00:00 EST 1969            0
##&nbsp;
##&nbsp;CBI 0     Aggregate Name      Total SSD Blocks  Total Used
##&nbsp;          aggr0                       26209152    21121110
##&nbsp;
##&nbsp;   Total Blocks In Use        Direct Allocated
##&nbsp;              21121110                  159367
##&nbsp;
##&nbsp;   Write Cached		      Read Cached
##&nbsp;                171075                20524440
##&nbsp;
##&nbsp;   Blocks in the Dlog
##&nbsp;                266228
##&nbsp;
##&nbsp;
##&nbsp;CBI 1     Hybrid Aggregate Metadata
##&nbsp;
##&nbsp;   Total Blocks In Use        Direct Allocated
##&nbsp;                425595                  159367
##&nbsp;
##&nbsp;   Write Cached		      Read Cached
##&nbsp;                     0                       0
##&nbsp;
##&nbsp;   Blocks in the Dlog
##&nbsp;                266228
##&nbsp;
##&nbsp;
##&nbsp;CBI 2      Volume Name        BuftreeID      Flags       Status
##&nbsp;           vol0               26211          Valid       online
##&nbsp;
##&nbsp;   Total Blocks In Use        Direct Allocated
##&nbsp;              20695556                       0
##&nbsp;
##&nbsp;   Write Cached		      Read Cached
##&nbsp;                171116                20524440
##&nbsp;
##&nbsp;   Blocks in the Dlog
##&nbsp;                     0
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'version' => '2',
##&nbsp;            'container_check_cp_count' => '734692',
##&nbsp;            'evict_nextblk' => '107228320',
##&nbsp;            'container_check_state' => '1',
##&nbsp;            'cbi_group_size' => '8',
##&nbsp;            'cp_info' => [
##&nbsp;                           {
##&nbsp;                             'cp_count' => '618828',
##&nbsp;                             'start_time' => 'Eviction Stamp[0]: Wed Oct 29 04:01:23 EDT 2014'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'cp_count' => '0',
##&nbsp;                             'start_time' => 'Eviction Stamp[1]: Wed Dec 31 19:00:00 EST 1969'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'cp_count' => '0',
##&nbsp;                             'start_time' => 'Eviction Stamp[2]: Wed Dec 31 19:00:00 EST 1969'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'cp_count' => '0',
##&nbsp;                             'start_time' => 'Eviction Stamp[3]: Wed Dec 31 19:00:00 EST 1969'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;            'cbi_info' => [
##&nbsp;                            {
##&nbsp;                              'direct_allocated' => '159367',
##&nbsp;                              'aggregate_name' => 'aggr0',
##&nbsp;                              'total_used' => '21121110',
##&nbsp;                              'write_cached' => '171075',
##&nbsp;                              'cbi' => '0',
##&nbsp;                              'blocks_in_the_dlog' => '266228',
##&nbsp;                              'total_blocks_in_use' => '21121110',
##&nbsp;                              'total_ssd_blocks' => '26209152',
##&nbsp;                              'read_cached' => '20524440'
##&nbsp;                            },
##&nbsp;                            {
##&nbsp;                              'direct_allocated' => '159367',
##&nbsp;                              'total_blocks_in_use' => '425595',
##&nbsp;                              'write_cached' => '0',
##&nbsp;                              'cbi' => '1',
##&nbsp;                              'blocks_in_the_dlog' => '266228',
##&nbsp;                              'read_cached' => '0'
##&nbsp;                            },
##&nbsp;                            {
##&nbsp;                              'direct_allocated' => '0',
##&nbsp;                              'flags' => 'Valid',
##&nbsp;                              'write_cached' => '171116',
##&nbsp;                              'status' => 'online',
##&nbsp;                              'volume_name' => 'vol0',
##&nbsp;                              'cbi' => '2',
##&nbsp;                              'blocks_in_the_dlog' => '0',
##&nbsp;                              'total_blocks_in_use' => '20695556',
##&nbsp;                              'buftreeid' => '26211',
##&nbsp;                              'read_cached' => '20524440'
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;            'stale_cmap_blks' => '18106685'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wafl_hya_cbi_show{
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my ( $error, @result_array, @lines);
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'    => $raw_output,
            'command'       => $command,
            'error_message' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    @lines = get_lines( raw_output => $raw_output, response_object => $resp );
    my ($key,$value,@result_array2,$line,$j,$k,$countInner,@keyValuePair,$values);
    $j =0;
    $k =0;
    while( @lines ) {
        $line = shift( @lines );
        if( $line=~/^Header/ ) {
            $line =~ s/Header:\s+//;
            $line=~ s/,\s+/:/g;
            @keyValuePair = split(/:/,$line);
            my $countOuter =0;
            for( $countOuter =0;$countOuter <=$#keyValuePair;$countOuter =$countOuter+1 ) {
                 my @keyValueSep = split(/\s+/,$keyValuePair[$countOuter]);
                 if($#keyValueSep>1 ) {
                    $key='';
                    for($countInner=0;$countInner<$#keyValueSep;$countInner=$countInner+1){
                        $key .= $keyValueSep[$countInner].' ';
                    }
                    chop($key);
                    $value=$keyValueSep[$#keyValueSep];
                    $result_array2[$j]->{to_key_name(item =>$key)} = $value;
                 }else{
                       $key=$keyValueSep[0];
                       $value=$keyValueSep[1];
                       $result_array2[$j]->{to_key_name(item => $key)} = $value
                  }
              }
          }elsif( $line =~/^Start/ ) {
               while( $line =~/^(?!CBI).*$/ ) {
                  my @keyValue = split(/\s\s+/,$line);
                  if($#keyValue==1 && $keyValue[1] !~ /CP Count/ ) {
                    my @headings = ("start_time", "cp_count");
                    $result_array2[$j]->{cp_info}[$k]->{to_key_name(item => $headings[0])} = $keyValue[0];
                    $result_array2[$j]->{cp_info}[$k]->{to_key_name(item => $headings[1])} = $keyValue[1];
                    $k = $k+1;
                  }
                  $line = shift(@lines);
                }
            }
           $k =0;
           while( $line =~ /^CBI/ && $line ) {
               my @headings = split(/\s\s+/,$line);
               my @first_head = split(/\s/,$headings[0]);
               $result_array2[$j]->{cbi_info}[$k]->{to_key_name(item => $first_head[0])} = $first_head[1];
               $line = shift(@lines);
               my @values = split(/\s\s+/,$line);
               if($line !~ /^Total/ ) {
                 foreach my $i ( 0...$#values ) {
                    $result_array2[$j]->{cbi_info}[$k]->{to_key_name(item => $headings[$i+1])}=$values[$i];
                 }
                 $line = shift(@lines);
               }else {
                    unshift(@lines);
                }
               while( $line !~ /^CBI/ && $line ) {
                   my  @headings = split(/\s\s+/,$line);
                   $line = shift(@lines);
                   my @values = split(/\s\s+/,$line);
                   foreach my $i (0...$#values) {
                      $result_array2[$j]->{cbi_info}[$k]->{to_key_name(item => $headings[$i])}=$values[$i];
                   }
                   $line = shift(@lines);
                }
              $k++;
            }
    }
    return \@result_array2;
}

## @name ::wafl_composite_stats_counter_show
##
## @summary Parses the raw output from the "wafl composite stats counter show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_composite_stats_counter_show with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_composite_stats_counter_show(%args);
## INPUTS TO METHOD CALL 'wafl_composite_stats_counter_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggr' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command: system node run -node local -command "priv set -q test; wafl composite stats counter show comp_aggr2;"
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Header: Version 1
##&nbsp;
##&nbsp;CSB 0        Aggr Name         blks     ssd blks   ssd plane0
##&nbsp;            comp_aggr2         5792            0            0
##&nbsp;
##&nbsp;     Space Counters
##&nbsp;            Bin   Ref'd Objs     Obj Refs    Cont Blks  Tlog Slots Tlog Used Obj Dual SSD Pvbn
##&nbsp;              0            0            0            0           0            0
##&nbsp;              1            0            0            0           0            0
##&nbsp;
##&nbsp;     Operation Counters
##&nbsp;          Count
##&nbsp;            Bin          GET          PUT       DELETE         LIST         HEAD
##&nbsp;              1            0            0            0            0            0
##&nbsp;
##&nbsp;          Total Size
##&nbsp;              1            0            0            0
##&nbsp;
##&nbsp;
##&nbsp;CSB 1     Aggregate Metadata
##&nbsp;
##&nbsp;     Space Counters
##&nbsp;            Bin   Ref'd Objs     Obj Refs    Cont Blks
##&nbsp;              0            0            0            0
##&nbsp;              1            0            0            0
##&nbsp;
##&nbsp;     Operation Counters
##&nbsp;          Count
##&nbsp;            Bin          GET          PUT       DELETE         LIST         HEAD
##&nbsp;              1            0            0            0            0            0
##&nbsp;
##&nbsp;          Total Size
##&nbsp;              1            0            0            0
##&nbsp;
##&nbsp;
##&nbsp;CSB 2      Volume Name        BuftreeID      Flags       Status
##&nbsp;           n/a                24929           Valid:1    not-exist
##&nbsp;
##&nbsp;     Space Counters
##&nbsp;            Bin   Ref'd Objs     Obj Refs    Cont Blks
##&nbsp;              0            0            0          141
##&nbsp;              1            0            0            0
##&nbsp;
##&nbsp;     Operation Counters
##&nbsp;          Count
##&nbsp;            Bin          GET          PUT       DELETE         LIST         HEAD
##&nbsp;              1            0            0            0            0            0
##&nbsp;
##&nbsp;          Total Size
##&nbsp;              1            0            0            0
##&nbsp;
##&nbsp;
##&nbsp;CSB 3      Volume Name        BuftreeID      Flags       Status
##&nbsp;           less_awesome_volume 9683            Valid:1    online
##&nbsp;
##&nbsp;     Space Counters
##&nbsp;            Bin   Ref'd Objs     Obj Refs    Cont Blks
##&nbsp;              0            0            0          779
##&nbsp;              1            0            0            0
##&nbsp;           Container L0s          779
##&nbsp;           Container blk          791
##&nbsp;           blk.used               380
##&nbsp;
##&nbsp;     Operation Counters
##&nbsp;          Count
##&nbsp;            Bin          GET          PUT       DELETE         LIST         HEAD
##&nbsp;              1            0            0            0            0            0
##&nbsp;
##&nbsp;          Total Size
##&nbsp;              1            0            0            0
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'Space Counters' => {
##&nbsp;                                  'Bin0' => {
##&nbsp;                                              'Tlog Used Obj Dual SSD Pvbn' => '0',
##&nbsp;                                              'Cont Blks' => '0',
##&nbsp;                                              'Tlog Slots ' => '0',
##&nbsp;                                              'Obj Refs' => '0',
##&nbsp;                                              'Ref\'d Objs' => '0'
##&nbsp;                                            },
##&nbsp;                                  'Bin1' => {
##&nbsp;                                              'Tlog Used Obj Dual SSD Pvbn' => '0',
##&nbsp;                                              'Cont Blks' => '0',
##&nbsp;                                              'Tlog Slots ' => '0',
##&nbsp;                                              'Obj Refs' => '0',
##&nbsp;                                              'Ref\'d Objs' => '0'
##&nbsp;                                            }
##&nbsp;                                },
##&nbsp;            'ssd blks' => '0',
##&nbsp;            'ssd plane0' => '0',
##&nbsp;            'Entry Name' => 'CSB 0',
##&nbsp;            'blks' => '5792',
##&nbsp;            'Aggr Name' => 'comp_aggr2',
##&nbsp;            'Entry Type' => 'Aggr',
##&nbsp;            'Operation Counters' => {
##&nbsp;                                      'Total Size' => {
##&nbsp;                                                        'Bin1' => {
##&nbsp;                                                                    'DELETE' => '0',
##&nbsp;                                                                    'PUT' => '0',
##&nbsp;                                                                    'GET' => '0'
##&nbsp;                                                                  }
##&nbsp;                                                      },
##&nbsp;                                      'Count' => {
##&nbsp;                                                   'Bin1' => {
##&nbsp;                                                               'DELETE' => '0',
##&nbsp;                                                               'LIST' => '0',
##&nbsp;                                                               'PUT' => '0',
##&nbsp;                                                               'GET' => '0',
##&nbsp;                                                               'HEAD' => '0'
##&nbsp;                                                             }
##&nbsp;                                                 }
##&nbsp;                                    }
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'Space Counters' => {
##&nbsp;                                  'Bin0' => {
##&nbsp;                                              'Cont Blks' => '0',
##&nbsp;                                              'Obj Refs' => '0',
##&nbsp;                                              'Ref\'d Objs' => '0'
##&nbsp;                                            },
##&nbsp;                                  'Bin1' => {
##&nbsp;                                              'Cont Blks' => '0',
##&nbsp;                                              'Obj Refs' => '0',
##&nbsp;                                              'Ref\'d Objs' => '0'
##&nbsp;                                            }
##&nbsp;                                },
##&nbsp;            'Entry Name' => 'CSB 1',
##&nbsp;            'Entry Type' => 'Aggregate Metadata',
##&nbsp;            'Operation Counters' => {
##&nbsp;                                      'Total Size' => {
##&nbsp;                                                        'Bin1' => {
##&nbsp;                                                                    'DELETE' => '0',
##&nbsp;                                                                    'PUT' => '0',
##&nbsp;                                                                    'GET' => '0'
##&nbsp;                                                                  }
##&nbsp;                                                      },
##&nbsp;                                      'Count' => {
##&nbsp;                                                   'Bin1' => {
##&nbsp;                                                               'DELETE' => '0',
##&nbsp;                                                               'LIST' => '0',
##&nbsp;                                                               'PUT' => '0',
##&nbsp;                                                               'GET' => '0',
##&nbsp;                                                               'HEAD' => '0'
##&nbsp;                                                             }
##&nbsp;                                                 }
##&nbsp;                                    }
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'Space Counters' => {
##&nbsp;                                  'Bin0' => {
##&nbsp;                                              'Cont Blks' => '141',
##&nbsp;                                              'Obj Refs' => '0',
##&nbsp;                                              'Ref\'d Objs' => '0'
##&nbsp;                                            },
##&nbsp;                                  'Bin1' => {
##&nbsp;                                              'Cont Blks' => '0',
##&nbsp;                                              'Obj Refs' => '0',
##&nbsp;                                              'Ref\'d Objs' => '0'
##&nbsp;                                            }
##&nbsp;                                },
##&nbsp;            'Flags' => 'Valid:1',
##&nbsp;            'Status' => 'not-exist',
##&nbsp;            'BuftreeID' => '24929',
##&nbsp;            'Entry Name' => 'CSB 2',
##&nbsp;            'Volume Name' => 'n/a',
##&nbsp;            'Entry Type' => 'Volume',
##&nbsp;            'Operation Counters' => {
##&nbsp;                                      'Total Size' => {
##&nbsp;                                                        'Bin1' => {
##&nbsp;                                                                    'DELETE' => '0',
##&nbsp;                                                                    'PUT' => '0',
##&nbsp;                                                                    'GET' => '0'
##&nbsp;                                                                  }
##&nbsp;                                                      },
##&nbsp;                                      'Count' => {
##&nbsp;                                                   'Bin1' => {
##&nbsp;                                                               'DELETE' => '0',
##&nbsp;                                                               'LIST' => '0',
##&nbsp;                                                               'PUT' => '0',
##&nbsp;                                                               'GET' => '0',
##&nbsp;                                                               'HEAD' => '0'
##&nbsp;                                                             }
##&nbsp;                                                 }
##&nbsp;                                    }
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'Space Counters' => {
##&nbsp;                                  'Container blk' => '791',
##&nbsp;                                  'blk.used' => '380',
##&nbsp;                                  'Bin0' => {
##&nbsp;                                              'Cont Blks' => '779',
##&nbsp;                                              'Obj Refs' => '0',
##&nbsp;                                              'Ref\'d Objs' => '0'
##&nbsp;                                            },
##&nbsp;                                  'Container L0s' => '779',
##&nbsp;                                  'Bin1' => {
##&nbsp;                                              'Cont Blks' => '0',
##&nbsp;                                              'Obj Refs' => '0',
##&nbsp;                                              'Ref\'d Objs' => '0'
##&nbsp;                                            }
##&nbsp;                                },
##&nbsp;            'Flags' => 'Valid:1',
##&nbsp;            'Status' => 'online',
##&nbsp;            'BuftreeID' => '9683',
##&nbsp;            'Entry Name' => 'CSB 3',
##&nbsp;            'Volume Name' => 'less_awesome_volume',
##&nbsp;            'Entry Type' => 'Volume',
##&nbsp;            'Operation Counters' => {
##&nbsp;                                      'Total Size' => {
##&nbsp;                                                        'Bin1' => {
##&nbsp;                                                                    'DELETE' => '0',
##&nbsp;                                                                    'PUT' => '0',
##&nbsp;                                                                    'GET' => '0'
##&nbsp;                                                                  }
##&nbsp;                                                      },
##&nbsp;                                      'Count' => {
##&nbsp;                                                   'Bin1' => {
##&nbsp;                                                               'DELETE' => '0',
##&nbsp;                                                               'LIST' => '0',
##&nbsp;                                                               'PUT' => '0',
##&nbsp;                                                               'GET' => '0',
##&nbsp;                                                               'HEAD' => '0'
##&nbsp;                                                             }
##&nbsp;                                                 }
##&nbsp;                                    }
##&nbsp;          }
##&nbsp;        ];
sub wafl_composite_stats_counter_show {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my ( $error, @result_array, @lines);
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'    => $raw_output,
            'command'       => $command,
            'error_message' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }

    my @csb_combos = split /(?=CSB \d+)/,$raw_output;

    # Remove the first element as it only contains the header
    if ($csb_combos[0] =~ /Header/i) {
        shift @csb_combos;
    }
    my @counter_array;

    # Split into individual CSB entries as so:
    # Raw output:
    # CSB 3      Volume Name        BuftreeID      Flags       Status
    #        less_awesome_volume 9683            Valid:1    online

    #  Space Counters
    #         Bin   Ref'd Objs     Obj Refs    Cont Blks
    #           0            0            0           81
    #           1            0            0            0
    #        Container L0s           81
    #        Container blk           86
    #        blk.used                43

    #  Operation Counters
    #       Count
    #         Bin          GET          PUT       DELETE         LIST         HEAD
    #           1            0            0            0            0            0

    #       Total Size
    #           1            0            0            0
    foreach my $csb_entry (@csb_combos){
        my $cbs_name;
        my @counters = split /(?=Space Counters)|(?=Operation Counters)/, $csb_entry;
        my %combo_hash;
        foreach my $counter_entry (@counters){
            # Raw output:
            # CSB 3      Volume Name        BuftreeID      Flags       Status
            # less_awesome_volume 9683            Valid:1    online
            if ($counter_entry =~ /(CSB \d+)/) {
                my @lines = get_lines(raw_output => $counter_entry,);
                my @keys;
                my @values;
                $cbs_name = $1;
                $combo_hash{'Entry Name'} = $cbs_name;
                foreach my $line (@lines) {
                    if ($line =~ /Aggregate Metadata/i){
                        $combo_hash{'Entry Type'} = 'Aggregate Metadata';
                        last;
                    } elsif ($line =~ /CSB/) {
                        if ($line =~ /Volume Name/i) {
                            $combo_hash{'Entry Type'} = 'Volume';
                        } elsif ($line =~ /Aggr Name/i) {
                            $combo_hash{'Entry Type'} = 'Aggr';
                        }
                        @keys = split /\s\s+/, $line;
                    } else {
                        @values = split /\s+/, $line;
                    }
                }
                # Add values to hash
                if (@keys > 1){
                    foreach my $index (0..scalar(@keys)-1) {
                        if ($index == 0) {
                        } else {
                            $combo_hash{$keys[$index]} = $values[$index-1];
                        }
                    }
                }
            # Raw Output:
            #  Space Counters
            #  Bin   Ref'd Objs     Obj Refs    Cont Blks
            #    0            0            0           81
            #    1            0            0            0
            # Container L0s           81
            # Container blk           86
            # blk.used                43
            } elsif ($counter_entry =~ /Space Counters/) {
                my @lines = get_lines(raw_output => $counter_entry,);
                my @keys;
                my @space_counters;

                foreach my $line (@lines) {
                    if ($line =~ /Space Counters/i){
                        next;
                    } elsif ($line =~ /Bin/i) {
                        # Split the keys
                        @keys = split /\s\s+|(?=Tlog)/, $line;
                    } else {
                        # Split the actual count values
                        my @line_values = split /\s\s+/, $line;
                        if (scalar(@line_values) > 2){
                            # Multiple values that match the keys
                            push(@space_counters,@line_values);

                        } elsif (scalar(@line_values) == 2) {
                            # Single key value pair
                            $combo_hash{'Space Counters'}{$line_values[0]} = $line_values[1];
                        }
                    }
                }
                my $bin_name;
                # Add values to hash
                foreach my $index (0..scalar(@space_counters)-1) {
                    my $key_index = $index % scalar(@keys);
                    if ($key_index == 0) {
                        $bin_name = $keys[0] . $space_counters[$index];
                    } else {
                        $combo_hash{'Space Counters'}{$bin_name}{$keys[$key_index]} = @space_counters[$index];
                    }
                }
            # Raw Output:
            #   Operation Counters
            # Count
            #   Bin          GET          PUT       DELETE         LIST         HEAD
            #     1            0            0            0            0            0

            # Total Size
            #     1            0            0            0
            } elsif ($counter_entry =~ /Operation Counters/) {
                my @operation_counters = split /(?=Count)|(?=Total Size)/i, $counter_entry;
                my @keys;
                foreach my $op_count (@operation_counters) {
                    my @op_counters;
                    my @lines = get_lines(raw_output => $op_count,);
                    if ($op_count =~ /Count/i){
                        # Collect values
                        foreach my $line (@lines) {
                            if ($line =~ /Count/i) {
                                next;
                            } elsif ($line =~ /Bin/i) {
                                @keys = split /\s\s+/, $line;
                            } else {
                                push @op_counters, (split /\s\s+/, $line);
                            }
                        }
                        # Add values to hash
                        my $bin_name;
                        foreach my $index (0..scalar(@op_counters)-1) {
                            my $key_index = $index % scalar(@keys);
                            if ($key_index == 0) {
                                $bin_name = $keys[0] . $op_counters[$index];
                            } else {
                                $combo_hash{'Operation Counters'}{'Count'}{$bin_name}{$keys[$key_index]} = @op_counters[$index];
                            }
                        }

                    } elsif ($op_count =~ /Total Size/i){
                        # Collect values
                        foreach my $line (@lines) {
                            if ($line =~ /Total Size/i) {
                                next;
                            } else {
                                push @op_counters, (split /\s\s+/, $line);
                            }
                        }
                        # Add values to hash
                        my $bin_name;
                        # Remove last two keys as they arent used in total size
                        my @total_size_keys = @keys[0..@keys-3];
                        foreach my $index (0..scalar(@op_counters)-1) {
                            my $key_index = $index % scalar(@total_size_keys);
                            if ($key_index == 0) {
                                $bin_name = $total_size_keys[0] . $op_counters[$index];
                            } else {
                                $combo_hash{'Operation Counters'}{'Total Size'}{$bin_name}{$total_size_keys[$key_index]} = @op_counters[$index];
                            }
                        }
                    }
                }
            } ## end if counter_entry
        } ## end foreach @counters
        push @counter_array,\%combo_hash;
    } ## end foreach csb_combos
    return \@counter_array;
} # end sub wafl_composite_stats_counter_show

## @name ::wafl_composite_stats_show
##
## @summary Parses the raw output from the "wafl composite stats show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_composite_stats_show with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_composite_stats_show(%args);
## INPUTS TO METHOD CALL 'wafl_composite_stats_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggr' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = "priv set -q test; wafl composite stats show comp_aggr; result -l"\r\n
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;composite aggr wcs_dummy: 0\r\n
##&nbsp;Tiering run/don't run reasons:\r\n
##&nbsp;\tnot booted: 0\r\n
##&nbsp;\tmaint mode: 0\r\n
##&nbsp;\tshutdown: 0\r\n
##&nbsp;\tgiveback: 0\r\n
##&nbsp;\taggr relocating: 0\r\n
##&nbsp;\tnot mounted: 0\r\n
##&nbsp;\tturned off: 0\r\n
##&nbsp;\tconverting: 0\r\n
##&nbsp;\tlow usage: 0\r\n
##&nbsp;\tplane0 low: 0\r\n
##&nbsp;\tswitchback: 0\r\n
##&nbsp;\tdisabled: 0\r\n
##&nbsp;\tironing: 0\r\n
##&nbsp;\tbad_vvol_btuuid: 0\r\n
##&nbsp;\ttype bits invalid: 0\r\n
##&nbsp;\ttimer: 5\r\n
##&nbsp;\tsnapshot: 0\r\n
##&nbsp;Tiering total idle suspends: 0\r\n
##&nbsp;Tiering total suspends for pacing: 0\r\n
##&nbsp;Tiering total steps done: 71941386\r\n
##&nbsp;Tiering scanner suspends for load_maps: 76\r\n
##&nbsp;Tiering scanner restarts: 431358\r\n
##&nbsp;Tiering total blocks tiered: 0\r\n
##&nbsp;Tiering skipped blocks in cloud: 1096006644\r\n
##&nbsp;Tiering total blocks in afs: 745161697\r\n
##&nbsp;Tiering total blocks in snapshot: 1297475091\r\n
##&nbsp;Tiering total blocks not cold: 0\r\n
##&nbsp;Tiering total blocks skipped for pending hit: 0\r\n
##&nbsp;Tiering total blocks skipped in bdata hash: 0\r\n
##&nbsp;Tiering total blocks skipped for bad type: 0\r\n
##&nbsp;Tiering total blocks pinned: 201468447\r\n
##&nbsp;hit update chunk alloced: 4\r\n
##&nbsp;hit update chunk applied: 3\r\n
##&nbsp;hit update chunk freed empty: 0\r\n
##&nbsp;hit update chunk ssd full: 0\r\n
##&nbsp;hit update chunk purged: 0\r\n
##&nbsp;hit update vbn duplicate: 0\r\n
##&nbsp;hit update vbn dup overflow: 0\r\n
##&nbsp;hit update vbn added: 554\r\n
##&nbsp;hit update range clears: 178\r\n
##&nbsp;hit update no memory: 1\r\n
##&nbsp;hit update suspend mem alloc: 0\r\n
##&nbsp;hit update suspend load active: 1\r\n
##&nbsp;hit update suspend load tempmap: 0\r\n
##&nbsp;hit update suspend cow tempmap: 0\r\n
##&nbsp;hit update suspend dirty tempmap: 0\r\n
##&nbsp;hit update suspend yield: 2\r\n
##&nbsp;hit update suspend in CP: 0\r\n
##&nbsp;hit update mod buf skipped: 0\r\n
##&nbsp;Cooling cooled blocks: 38983\r\n
##&nbsp;Cooling cooled by temp: 82 1366 2592 1785 4106 2218 2214 3002 1658 2832 2636 3458 1342 3123 2162 4407\r\n
##&nbsp;Cooling suspends for loadbufs: 0\r\n
##&nbsp;Cooling suspends for loadbufs-tempmap: 0\r\n
##&nbsp;Cooling scanner steps: 123\r\n
##&nbsp;Cooling total blocks: 500160\r\n
##&nbsp;Cooling run/don't run reasons:\r\n
##&nbsp;\tnot booted: 0\r\n
##&nbsp;\tmaint mode: 0\r\n
##&nbsp;\tshutdown: 0\r\n
##&nbsp;\tgiveback: 0\r\n
##&nbsp;\taggr relocating: 0\r\n
##&nbsp;\tnot mounted: 0\r\n
##&nbsp;\tturned off: 0\r\n
##&nbsp;\tconverting: 0\r\n
##&nbsp;\tsb or nso: 0\r\n
##&nbsp;\tnormal: 473\r\n
##&nbsp;Tempmap COW by cooling: 0\r\n
##&nbsp;Cooling found block in hitmap: 178\r\n
##&nbsp;Cooling found block in extent hash: 1062\r\n
##&nbsp;Cooling skipped for blocks: 461177\r\n
##&nbsp;data migrate total tagged blocks: 0\r\n
##&nbsp;data migrate ra total tagged blocks: 0\r\n
##&nbsp;data migrate total blocks blkr dirtied: 0\r\n
##&nbsp;data migrate total blocks skipped: 0\r\n
##&nbsp;data migrate aggregate has no space: 0\r\n
##&nbsp;data migrate volume not mounted: 0\r\n
##&nbsp;data migrate not auto policy: 0\r\n
##&nbsp;data migrate no CP: 0\r\n
##&nbsp;data migrate no wip: 0\r\n
##&nbsp;data migrate reject snapshot block: 0\r\n
##&nbsp;data migrate reject compressed blocks: 0\r\n
##&nbsp;data migrate reject invalid blocks: 0\r\n
##&nbsp;data migrate ec cache rejects: 0\r\n
##&nbsp;data migrate skip cold blocks: 0\r\n
##&nbsp;data migrate hot blocks: 0\r\n
##&nbsp;data migrate realloc disabled: 0\r\n
##&nbsp;data migrate blkr overlimit: 0\r\n
##&nbsp;Reads form redirected iovbns: 0\r\n
##&nbsp;Readaheads from redirected iovbns: 0\r\n
##&nbsp;Reads from redirected iovbns hit bdata hash : 0\r\n
##&nbsp;Readaheads from redirected iovbns hit bdata hash : 0\r\n
##&nbsp;Blocks directly snapmirrored to capacity tier: 0\r\n
##&nbsp;snapmirror transfer failed due to no space in capacity tier: 0\r\n
##&nbsp;snapmirror transfer stalled due to capacity tier metafiles not ironed yet: 0\r\n
##&nbsp;snapmirror transfer redirected to perf tier metafiles not ironed yet: 0\r\n
##&nbsp;Capacity tier limit reached: 0\r\n
##&nbsp;No free slot in TLOG: 0\r\n
##&nbsp;Bin stats of non-volume data\r\n
##&nbsp;Bin1 uses 1024 slots in its objects\r\n
##&nbsp;\r\n
##&nbsp;Bin 0\r\n
##&nbsp;Read Stats:\r\n
##&nbsp;wcvbs_tlog_reads: 0\r\n
##&nbsp;wcvbs_bin_reads: 0\r\n
##&nbsp;wcvbs_ra_reads: 0\r\n
##&nbsp;wcvbs_client_proto_reads: 0\r\n
##&nbsp;wcvbs_repl_proto_reads: 0\r\n
##&nbsp;wcvbs_other_proto_reads: 0\r\n
##&nbsp;Error Stats:\r\n
##&nbsp;wcvbs_errors_io_timedout: 0\r\n
##&nbsp;wcvbs_errors_op_expired: 0\r\n
##&nbsp;wcvbs_errors_op_expire_soon: 0\r\n
##&nbsp;Size Histogram:\r\n
##&nbsp;GET\r\n
##&nbsp;        0        1        2        3        4        5        6        7\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;PUT\r\n
##&nbsp;        0        1        2        3        4        5        6        7\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;DELETE\r\n
##&nbsp;        0        1        2        3        4        5        6        7\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;Latency Histogram:\r\n
##&nbsp;GET\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;PUT\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;DELETE\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;LIST\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;HEAD\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;\r\n
##&nbsp;Bin 1\r\n
##&nbsp;Read Stats:\r\n
##&nbsp;wcvbs_tlog_reads: 0\r\n
##&nbsp;wcvbs_bin_reads: 0\r\n
##&nbsp;wcvbs_ra_reads: 0\r\n
##&nbsp;wcvbs_client_proto_reads: 0\r\n
##&nbsp;wcvbs_repl_proto_reads: 0\r\n
##&nbsp;wcvbs_other_proto_reads: 0\r\n
##&nbsp;Error Stats:\r\n
##&nbsp;wcvbs_errors_io_timedout: 0\r\n
##&nbsp;wcvbs_errors_op_expired: 0\r\n
##&nbsp;wcvbs_errors_op_expire_soon: 0\r\n
##&nbsp;Size Histogram:\r\n
##&nbsp;GET\r\n
##&nbsp;        0        1        2        3        4        5        6        7\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;PUT\r\n
##&nbsp;        0        1        2        3        4        5        6        7\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;DELETE\r\n
##&nbsp;        0        1        2        3        4        5        6        7\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;Latency Histogram:\r\n
##&nbsp;GET\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;PUT\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;DELETE\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;LIST\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;HEAD\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;Bin stats of the entire aggregate\r\n
##&nbsp;\r\n
##&nbsp;Bin 0\r\n
##&nbsp;Read Stats:\r\n
##&nbsp;wcvbs_tlog_reads: 0\r\n
##&nbsp;wcvbs_bin_reads: 0\r\n
##&nbsp;wcvbs_ra_reads: 0\r\n
##&nbsp;wcvbs_client_proto_reads: 0\r\n
##&nbsp;wcvbs_repl_proto_reads: 0\r\n
##&nbsp;wcvbs_other_proto_reads: 0\r\n
##&nbsp;Error Stats:\r\n
##&nbsp;wcvbs_errors_io_timedout: 0\r\n
##&nbsp;wcvbs_errors_op_expired: 0\r\n
##&nbsp;wcvbs_errors_op_expire_soon: 0\r\n
##&nbsp;Size Histogram:\r\n
##&nbsp;GET\r\n
##&nbsp;        0        1        2        3        4        5        6        7\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;PUT\r\n
##&nbsp;        0        1        2        3        4        5        6        7\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;DELETE\r\n
##&nbsp;        0        1        2        3        4        5        6        7\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;Latency Histogram:\r\n
##&nbsp;GET\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;PUT\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;DELETE\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;LIST\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;HEAD\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;\r\n
##&nbsp;Bin 1\r\n
##&nbsp;Read Stats:\r\n
##&nbsp;wcvbs_tlog_reads: 0\r\n
##&nbsp;wcvbs_bin_reads: 0\r\n
##&nbsp;wcvbs_ra_reads: 0\r\n
##&nbsp;wcvbs_client_proto_reads: 0\r\n
##&nbsp;wcvbs_repl_proto_reads: 0\r\n
##&nbsp;wcvbs_other_proto_reads: 0\r\n
##&nbsp;Error Stats:\r\n
##&nbsp;wcvbs_errors_io_timedout: 0\r\n
##&nbsp;wcvbs_errors_op_expired: 0\r\n
##&nbsp;wcvbs_errors_op_expire_soon: 0\r\n
##&nbsp;Size Histogram:\r\n
##&nbsp;GET\r\n
##&nbsp;        0        1        2        3        4        5        6        7\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;PUT\r\n
##&nbsp;        0        1        2        3        4        5        6        7\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;DELETE\r\n
##&nbsp;        0        1        2        3        4        5        6        7\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;      <16      <24      <32      <40      <48      <56      <64   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;     <128     <192     <256     <320     <384     <448     <512   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <1024    <1536    <2048    <2560    <3072    <3584    <4096   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;    <8192   <12288   <16384   <20480   <24576   <28672   <32768   UNUSED\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;   <65536   <98304  <131072  <163840  <196608  <229376  <262144  >262144\r\n
##&nbsp;        0        0        0        0        0        0        0        0\r\n
##&nbsp;Latency Histogram:\r\n
##&nbsp;GET\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;PUT\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;DELETE\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;LIST\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;HEAD\r\n
##&nbsp;\r\n
##&nbsp;            0 - <1ms            1 - <2ms            2 - <4ms            4 - <6ms            6 - <8ms           8 - <10ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          10 - <12ms          12 - <16ms          16 - <20ms          20 - <30ms          30 - <40ms          40 - <50ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;          50 - <60ms          60 - <70ms          70 - <80ms          80 - <90ms         90 - <100ms        100 - <120ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        120 - <140ms        140 - <160ms        160 - <180ms        180 - <200ms        200 - <400ms        400 - <600ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;        600 - <800ms       800 - <1000ms      1000 - <1500ms      1500 - <2000ms      2000 - <2500ms      2500 - <3000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp;      3000 - <3500ms      3500 - <4000ms      4000 - <8000ms     8000 - <12000ms    12000 - <16000ms            >16000ms\r\n
##&nbsp;                   0                   0                   0                   0                   0                   0\r\n
##&nbsp; Total: 0\r\n
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; Split into 5 sections for aggregates and 3 for volumes.
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'Tempmap COW by cooling' => '0',
##&nbsp;             'Tiering total blocks skipped for pending hit' => '0',
##&nbsp;             'Reads form redirected iovbns' => '0',
##&nbsp;             'data migrate total blocks skipped' => '0',
##&nbsp;             'snapmirror transfer stalled due to capacity tier metafiles not ironed yet' => '0',
##&nbsp;             'hit update chunk purged' => '0',
##&nbsp;             'data migrate ec cache rejects' => '0',
##&nbsp;             'Readaheads from redirected iovbns' => '0',
##&nbsp;             'Tiering total blocks skipped in bdata hash' => '0',
##&nbsp;             'data migrate ra total tagged blocks' => '0',
##&nbsp;             'Cooling scanner steps' => '123',
##&nbsp;             'Cooling cooled by temp' => '82 1366 2592 1785 4106 2218 2214 3002 1658 2832 2636 3458 1342 3123 2162 4407',
##&nbsp;             'hit update vbn added' => '554',
##&nbsp;             'data migrate blkr overlimit' => '0',
##&nbsp;             'Tiering total blocks pinned' => '201468447',
##&nbsp;             'data migrate skip cold blocks' => '0',
##&nbsp;             'hit update vbn dup overflow' => '0',
##&nbsp;             'Cooling total blocks' => '500160',
##&nbsp;             'hit update no memory' => '1',
##&nbsp;             'Cooling run/don\'t run reasons:' => {
##&nbsp;                                                    'normal' => '473',
##&nbsp;                                                    'not mounted' => '0',
##&nbsp;                                                    'turned off' => '0',
##&nbsp;                                                    'sb or nso' => '0',
##&nbsp;                                                    'maint mode' => '0',
##&nbsp;                                                    'shutdown' => '0',
##&nbsp;                                                    'converting' => '0',
##&nbsp;                                                    'not booted' => '0',
##&nbsp;                                                    'giveback' => '0',
##&nbsp;                                                    'aggr relocating' => '0'
##&nbsp;                                                  },
##&nbsp;             'Blocks directly snapmirrored to capacity tier' => '0',
##&nbsp;             'hit update range clears' => '178',
##&nbsp;             'hit update suspend dirty tempmap' => '0',
##&nbsp;             'Cooling skipped for blocks' => '461177',
##&nbsp;             'hit update chunk ssd full' => '0',
##&nbsp;             'Tiering total steps done' => '71941386',
##&nbsp;             'hit update suspend yield' => '2',
##&nbsp;             'data migrate no wip' => '0',
##&nbsp;             'snapmirror transfer failed due to no space in capacity tier' => '0',
##&nbsp;             'hit update vbn duplicate' => '0',
##&nbsp;             'data migrate not auto policy' => '0',
##&nbsp;             'Cooling suspends for loadbufs' => '0',
##&nbsp;             'hit update suspend load active' => '1',
##&nbsp;             'Readaheads from redirected iovbns hit bdata hash ' => '0',
##&nbsp;             'hit update suspend mem alloc' => '0',
##&nbsp;             'hit update suspend cow tempmap' => '0',
##&nbsp;             'hit update suspend load tempmap' => '0',
##&nbsp;             'hit update mod buf skipped' => '0',
##&nbsp;             'Cooling cooled blocks' => '38983',
##&nbsp;             'Tiering total suspends for pacing' => '0',
##&nbsp;             'data migrate total blocks blkr dirtied' => '0',
##&nbsp;             'Tiering total blocks tiered' => '0',
##&nbsp;             'Tiering scanner suspends for load_maps' => '76',
##&nbsp;             'Cooling found block in extent hash' => '1062',
##&nbsp;             'Tiering scanner restarts' => '431358',
##&nbsp;             'No free slot in TLOG' => '0',
##&nbsp;             'Cooling suspends for loadbufs-tempmap' => '0',
##&nbsp;             'Cooling found block in hitmap' => '178',
##&nbsp;             'Tiering run/don\'t run reasons:' => {
##&nbsp;                                                    'disabled' => '0',
##&nbsp;                                                    'not mounted' => '0',
##&nbsp;                                                    'timer' => '5',
##&nbsp;                                                    'ironing' => '0',
##&nbsp;                                                    'maint mode' => '0',
##&nbsp;                                                    'plane0 low' => '0',
##&nbsp;                                                    'shutdown' => '0',
##&nbsp;                                                    'converting' => '0',
##&nbsp;                                                    'snapshot' => '0',
##&nbsp;                                                    'giveback' => '0',
##&nbsp;                                                    'bad_vvol_btuuid' => '0',
##&nbsp;                                                    'switchback' => '0',
##&nbsp;                                                    'turned off' => '0',
##&nbsp;                                                    'type bits invalid' => '0',
##&nbsp;                                                    'low usage' => '0',
##&nbsp;                                                    'not booted' => '0',
##&nbsp;                                                    'aggr relocating' => '0'
##&nbsp;                                                  },
##&nbsp;             'composite aggr wcs_dummy' => '0',
##&nbsp;             'Reads from redirected iovbns hit bdata hash ' => '0',
##&nbsp;             'Tiering total idle suspends' => '0',
##&nbsp;             'Tiering total blocks in snapshot' => '1297475091',
##&nbsp;             'data migrate aggregate has no space' => '0',
##&nbsp;             'data migrate reject snapshot block' => '0',
##&nbsp;             'Capacity tier limit reached' => '0',
##&nbsp;             'hit update suspend in CP' => '0',
##&nbsp;             'Tiering total blocks not cold' => '0',
##&nbsp;             'data migrate reject compressed blocks' => '0',
##&nbsp;             'Tiering total blocks in afs' => '745161697',
##&nbsp;             'Tiering skipped blocks in cloud' => '1096006644',
##&nbsp;             'snapmirror transfer redirected to perf tier metafiles not ironed yet' => '0',
##&nbsp;             'Tiering total blocks skipped for bad type' => '0',
##&nbsp;             'hit update chunk alloced' => '4',
##&nbsp;             'hit update chunk freed empty' => '0',
##&nbsp;             'data migrate no CP' => '0',
##&nbsp;             'data migrate hot blocks' => '0',
##&nbsp;             'data migrate realloc disabled' => '0',
##&nbsp;             'data migrate reject invalid blocks' => '0',
##&nbsp;             'data migrate volume not mounted' => '0',
##&nbsp;             'hit update chunk applied' => '3',
##&nbsp;             'data migrate total tagged blocks' => '0'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'Bin 0' => {
##&nbsp;                          'Read Stats' => {
##&nbsp;                                            'wcvbs_client_proto_reads' => '0',
##&nbsp;                                            'wcvbs_bin_reads' => '0',
##&nbsp;                                            'wcvbs_ra_reads' => '0',
##&nbsp;                                            'wcvbs_other_proto_reads' => '0',
##&nbsp;                                            'wcvbs_tlog_reads' => '0',
##&nbsp;                                            'wcvbs_repl_proto_reads' => '0'
##&nbsp;                                          },
##&nbsp;                          'Error Stats' => {
##&nbsp;                                             'wcvbs_errors_op_expire_soon' => '0',
##&nbsp;                                             'wcvbs_errors_op_expired' => '0',
##&nbsp;                                             'wcvbs_errors_io_timedout' => '0'
##&nbsp;                                           },
##&nbsp;                          'Size Histogram' => {
##&nbsp;                                                'DELETE' => {
##&nbsp;                                                              '<98304' => '0',
##&nbsp;                                                              '<128' => '0',
##&nbsp;                                                              '<448' => '0',
##&nbsp;                                                              '7' => '0',
##&nbsp;                                                              '<56' => '0',
##&nbsp;                                                              '2' => '0',
##&nbsp;                                                              '<65536' => '0',
##&nbsp;                                                              '<196608' => '0',
##&nbsp;                                                              '1' => '0',
##&nbsp;                                                              '<3584' => '0',
##&nbsp;                                                              '<48' => '0',
##&nbsp;                                                              '<32768' => '0',
##&nbsp;                                                              '<16384' => '0',
##&nbsp;                                                              '<4096' => '0',
##&nbsp;                                                              '<192' => '0',
##&nbsp;                                                              '<262144' => '0',
##&nbsp;                                                              '<8192' => '0',
##&nbsp;                                                              '<64' => '0',
##&nbsp;                                                              '<1536' => '0',
##&nbsp;                                                              '<1024' => '0',
##&nbsp;                                                              '<229376' => '0',
##&nbsp;                                                              '<256' => '0',
##&nbsp;                                                              '<28672' => '0',
##&nbsp;                                                              '<320' => '0',
##&nbsp;                                                              '<20480' => '0',
##&nbsp;                                                              '0' => '0',
##&nbsp;                                                              '<24576' => '0',
##&nbsp;                                                              '<512' => '0',
##&nbsp;                                                              '<40' => '0',
##&nbsp;                                                              '6' => '0',
##&nbsp;                                                              '<163840' => '0',
##&nbsp;                                                              '>262144' => '0',
##&nbsp;                                                              '3' => '0',
##&nbsp;                                                              '<2560' => '0',
##&nbsp;                                                              '<3072' => '0',
##&nbsp;                                                              'UNUSED' => '0',
##&nbsp;                                                              '<16' => '0',
##&nbsp;                                                              '<384' => '0',
##&nbsp;                                                              '4' => '0',
##&nbsp;                                                              '<12288' => '0',
##&nbsp;                                                              '<2048' => '0',
##&nbsp;                                                              '<32' => '0',
##&nbsp;                                                              '<24' => '0',
##&nbsp;                                                              '<131072' => '0',
##&nbsp;                                                              '5' => '0'
##&nbsp;                                                            },
##&nbsp;                                                'PUT' => {
##&nbsp;                                                           '<98304' => '0',
##&nbsp;                                                           ... Same as above
##&nbsp;                                                           '5' => '0'
##&nbsp;                                                         },
##&nbsp;                                                'GET' => {
##&nbsp;                                                           '<98304' => '0',
##&nbsp;                                                           ... Same as above
##&nbsp;                                                           '5' => '0'
##&nbsp;                                                         }
##&nbsp;                                              },
##&nbsp;                          'Latency Histogram' => {
##&nbsp;                                                   'DELETE' => {
##&nbsp;                                                                 '100 - <120ms' => '0',
##&nbsp;                                                                 '0 - <1ms' => '0',
##&nbsp;                                                                 '>16000ms' => '0',
##&nbsp;                                                                 '600 - <800ms' => '0',
##&nbsp;                                                                 '2000 - <2500ms' => '0',
##&nbsp;                                                                 '20 - <30ms' => '0',
##&nbsp;                                                                 '70 - <80ms' => '0',
##&nbsp;                                                                 '6 - <8ms' => '0',
##&nbsp;                                                                 '180 - <200ms' => '0',
##&nbsp;                                                                 '2 - <4ms' => '0',
##&nbsp;                                                                 '60 - <70ms' => '0',
##&nbsp;                                                                 '160 - <180ms' => '0',
##&nbsp;                                                                 '16 - <20ms' => '0',
##&nbsp;                                                                 '200 - <400ms' => '0',
##&nbsp;                                                                 '140 - <160ms' => '0',
##&nbsp;                                                                 '120 - <140ms' => '0',
##&nbsp;                                                                 '3000 - <3500ms' => '0',
##&nbsp;                                                                 '3500 - <4000ms' => '0',
##&nbsp;                                                                 '40 - <50ms' => '0',
##&nbsp;                                                                 '30 - <40ms' => '0',
##&nbsp;                                                                 '50 - <60ms' => '0',
##&nbsp;                                                                 '8 - <10ms' => '0',
##&nbsp;                                                                 '1500 - <2000ms' => '0',
##&nbsp;                                                                 '1 - <2ms' => '0',
##&nbsp;                                                                 '12000 - <16000ms' => '0',
##&nbsp;                                                                 '800 - <1000ms' => '0',
##&nbsp;                                                                 '1000 - <1500ms' => '0',
##&nbsp;                                                                 'Total' => '0',
##&nbsp;                                                                 '8000 - <12000ms' => '0',
##&nbsp;                                                                 '80 - <90ms' => '0',
##&nbsp;                                                                 '4000 - <8000ms' => '0',
##&nbsp;                                                                 '2500 - <3000ms' => '0',
##&nbsp;                                                                 '400 - <600ms' => '0',
##&nbsp;                                                                 '10 - <12ms' => '0',
##&nbsp;                                                                 '12 - <16ms' => '0',
##&nbsp;                                                                 '90 - <100ms' => '0',
##&nbsp;                                                                 '4 - <6ms' => '0'
##&nbsp;                                                               },
##&nbsp;                                                   'LIST' => {
##&nbsp;                                                               '100 - <120ms' => '0',
##&nbsp;                                                               ... Same as above
##&nbsp;                                                               '4 - <6ms' => '0'
##&nbsp;                                                             },
##&nbsp;                                                   'PUT' => {
##&nbsp;                                                              '100 - <120ms' => '0',
##&nbsp;                                                              ... Same as above
##&nbsp;                                                              '4 - <6ms' => '0'
##&nbsp;                                                            },
##&nbsp;                                                   'GET' => {
##&nbsp;                                                              '100 - <120ms' => '0',
##&nbsp;                                                              ... Same as above
##&nbsp;                                                              '4 - <6ms' => '0'
##&nbsp;                                                            },
##&nbsp;                                                   'HEAD' => {
##&nbsp;                                                               '100 - <120ms' => '0',
##&nbsp;                                                               ... Same as above
##&nbsp;                                                               '4 - <6ms' => '0'
##&nbsp;                                                             }
##&nbsp;                                                 }
##&nbsp;                        }
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'Bin 1' => {
##&nbsp;                          'Read Stats' => {
##&nbsp;                                            'wcvbs_client_proto_reads' => '0',
##&nbsp;                                            'wcvbs_bin_reads' => '0',
##&nbsp;                                            'wcvbs_ra_reads' => '0',
##&nbsp;                                            'wcvbs_other_proto_reads' => '0',
##&nbsp;                                            'wcvbs_tlog_reads' => '0',
##&nbsp;                                            'wcvbs_repl_proto_reads' => '0'
##&nbsp;                                          },
##&nbsp;                          'Error Stats' => {
##&nbsp;                                             'wcvbs_errors_op_expire_soon' => '0',
##&nbsp;                                             'wcvbs_errors_op_expired' => '0',
##&nbsp;                                             'wcvbs_errors_io_timedout' => '0'
##&nbsp;                                           },
##&nbsp;                          'Size Histogram' => {
##&nbsp;                                                'DELETE' => {
##&nbsp;                                                              '<98304' => '0',
##&nbsp;                                                              '<128' => '0',
##&nbsp;                                                              '<448' => '0',
##&nbsp;                                                              '7' => '0',
##&nbsp;                                                              '<56' => '0',
##&nbsp;                                                              '2' => '0',
##&nbsp;                                                              '<65536' => '0',
##&nbsp;                                                              '<196608' => '0',
##&nbsp;                                                              '1' => '0',
##&nbsp;                                                              '<3584' => '0',
##&nbsp;                                                              '<48' => '0',
##&nbsp;                                                              '<32768' => '0',
##&nbsp;                                                              '<16384' => '0',
##&nbsp;                                                              '<4096' => '0',
##&nbsp;                                                              '<192' => '0',
##&nbsp;                                                              '<262144' => '0',
##&nbsp;                                                              '<8192' => '0',
##&nbsp;                                                              '<64' => '0',
##&nbsp;                                                              '<1536' => '0',
##&nbsp;                                                              '<1024' => '0',
##&nbsp;                                                              '<229376' => '0',
##&nbsp;                                                              '<256' => '0',
##&nbsp;                                                              '<28672' => '0',
##&nbsp;                                                              '<320' => '0',
##&nbsp;                                                              '<20480' => '0',
##&nbsp;                                                              '0' => '0',
##&nbsp;                                                              '<24576' => '0',
##&nbsp;                                                              '<512' => '0',
##&nbsp;                                                              '<40' => '0',
##&nbsp;                                                              '6' => '0',
##&nbsp;                                                              '<163840' => '0',
##&nbsp;                                                              '>262144' => '0',
##&nbsp;                                                              '3' => '0',
##&nbsp;                                                              '<2560' => '0',
##&nbsp;                                                              '<3072' => '0',
##&nbsp;                                                              'UNUSED' => '0',
##&nbsp;                                                              '<16' => '0',
##&nbsp;                                                              '<384' => '0',
##&nbsp;                                                              '4' => '0',
##&nbsp;                                                              '<12288' => '0',
##&nbsp;                                                              '<2048' => '0',
##&nbsp;                                                              '<32' => '0',
##&nbsp;                                                              '<24' => '0',
##&nbsp;                                                              '<131072' => '0',
##&nbsp;                                                              '5' => '0'
##&nbsp;                                                            },
##&nbsp;                                                'PUT' => {
##&nbsp;                                                           '<98304' => '0',
##&nbsp;                                                           ... Same as above
##&nbsp;                                                           '5' => '0'
##&nbsp;                                                         },
##&nbsp;                                                'GET' => {
##&nbsp;                                                           '<98304' => '0',
##&nbsp;                                                           ... Same as above
##&nbsp;                                                           '5' => '0'
##&nbsp;                                                         }
##&nbsp;                                              },
##&nbsp;                          'Latency Histogram' => {
##&nbsp;                                                   'DELETE' => {
##&nbsp;                                                                 '100 - <120ms' => '0',
##&nbsp;                                                                 '0 - <1ms' => '0',
##&nbsp;                                                                 '>16000ms' => '0',
##&nbsp;                                                                 '600 - <800ms' => '0',
##&nbsp;                                                                 '2000 - <2500ms' => '0',
##&nbsp;                                                                 '20 - <30ms' => '0',
##&nbsp;                                                                 '70 - <80ms' => '0',
##&nbsp;                                                                 '6 - <8ms' => '0',
##&nbsp;                                                                 '180 - <200ms' => '0',
##&nbsp;                                                                 '2 - <4ms' => '0',
##&nbsp;                                                                 '60 - <70ms' => '0',
##&nbsp;                                                                 '160 - <180ms' => '0',
##&nbsp;                                                                 '16 - <20ms' => '0',
##&nbsp;                                                                 '200 - <400ms' => '0',
##&nbsp;                                                                 '140 - <160ms' => '0',
##&nbsp;                                                                 '120 - <140ms' => '0',
##&nbsp;                                                                 '3000 - <3500ms' => '0',
##&nbsp;                                                                 '3500 - <4000ms' => '0',
##&nbsp;                                                                 '40 - <50ms' => '0',
##&nbsp;                                                                 '30 - <40ms' => '0',
##&nbsp;                                                                 '50 - <60ms' => '0',
##&nbsp;                                                                 '8 - <10ms' => '0',
##&nbsp;                                                                 '1500 - <2000ms' => '0',
##&nbsp;                                                                 '1 - <2ms' => '0',
##&nbsp;                                                                 '12000 - <16000ms' => '0',
##&nbsp;                                                                 '800 - <1000ms' => '0',
##&nbsp;                                                                 '1000 - <1500ms' => '0',
##&nbsp;                                                                 'Total' => '0',
##&nbsp;                                                                 '8000 - <12000ms' => '0',
##&nbsp;                                                                 '80 - <90ms' => '0',
##&nbsp;                                                                 '4000 - <8000ms' => '0',
##&nbsp;                                                                 '2500 - <3000ms' => '0',
##&nbsp;                                                                 '400 - <600ms' => '0',
##&nbsp;                                                                 '10 - <12ms' => '0',
##&nbsp;                                                                 '12 - <16ms' => '0',
##&nbsp;                                                                 '90 - <100ms' => '0',
##&nbsp;                                                                 '4 - <6ms' => '0'
##&nbsp;                                                               },
##&nbsp;                                                   'LIST' => {
##&nbsp;                                                               '100 - <120ms' => '0',
##&nbsp;                                                               ... Same as above
##&nbsp;                                                               '4 - <6ms' => '0'
##&nbsp;                                                             },
##&nbsp;                                                   'PUT' => {
##&nbsp;                                                              '100 - <120ms' => '0',
##&nbsp;                                                              ... Same as above
##&nbsp;                                                              '4 - <6ms' => '0'
##&nbsp;                                                            },
##&nbsp;                                                   'GET' => {
##&nbsp;                                                              '100 - <120ms' => '0',
##&nbsp;                                                              ... Same as above
##&nbsp;                                                              '4 - <6ms' => '0'
##&nbsp;                                                            },
##&nbsp;                                                   'HEAD' => {
##&nbsp;                                                               '100 - <120ms' => '0',
##&nbsp;                                                               ... Same as above
##&nbsp;                                                               '4 - <6ms' => '0'
##&nbsp;                                                             }
##&nbsp;                                                 }
##&nbsp;                        }
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'Bin 0' => {
##&nbsp;                          'Read Stats' => {
##&nbsp;                                            'wcvbs_client_proto_reads' => '0',
##&nbsp;                                            'wcvbs_bin_reads' => '0',
##&nbsp;                                            'wcvbs_ra_reads' => '0',
##&nbsp;                                            'wcvbs_other_proto_reads' => '0',
##&nbsp;                                            'wcvbs_tlog_reads' => '0',
##&nbsp;                                            'wcvbs_repl_proto_reads' => '0'
##&nbsp;                                          },
##&nbsp;                          'Error Stats' => {
##&nbsp;                                             'wcvbs_errors_op_expire_soon' => '0',
##&nbsp;                                             'wcvbs_errors_op_expired' => '0',
##&nbsp;                                             'wcvbs_errors_io_timedout' => '0'
##&nbsp;                                           },
##&nbsp;                          'Size Histogram' => {
##&nbsp;                                                'DELETE' => {
##&nbsp;                                                              '<98304' => '0',
##&nbsp;                                                              '<128' => '0',
##&nbsp;                                                              '<448' => '0',
##&nbsp;                                                              '7' => '0',
##&nbsp;                                                              '<56' => '0',
##&nbsp;                                                              '2' => '0',
##&nbsp;                                                              '<65536' => '0',
##&nbsp;                                                              '<196608' => '0',
##&nbsp;                                                              '1' => '0',
##&nbsp;                                                              '<3584' => '0',
##&nbsp;                                                              '<48' => '0',
##&nbsp;                                                              '<32768' => '0',
##&nbsp;                                                              '<16384' => '0',
##&nbsp;                                                              '<4096' => '0',
##&nbsp;                                                              '<192' => '0',
##&nbsp;                                                              '<262144' => '0',
##&nbsp;                                                              '<8192' => '0',
##&nbsp;                                                              '<64' => '0',
##&nbsp;                                                              '<1536' => '0',
##&nbsp;                                                              '<1024' => '0',
##&nbsp;                                                              '<229376' => '0',
##&nbsp;                                                              '<256' => '0',
##&nbsp;                                                              '<28672' => '0',
##&nbsp;                                                              '<320' => '0',
##&nbsp;                                                              '<20480' => '0',
##&nbsp;                                                              '0' => '0',
##&nbsp;                                                              '<24576' => '0',
##&nbsp;                                                              '<512' => '0',
##&nbsp;                                                              '<40' => '0',
##&nbsp;                                                              '6' => '0',
##&nbsp;                                                              '<163840' => '0',
##&nbsp;                                                              '>262144' => '0',
##&nbsp;                                                              '3' => '0',
##&nbsp;                                                              '<2560' => '0',
##&nbsp;                                                              '<3072' => '0',
##&nbsp;                                                              'UNUSED' => '0',
##&nbsp;                                                              '<16' => '0',
##&nbsp;                                                              '<384' => '0',
##&nbsp;                                                              '4' => '0',
##&nbsp;                                                              '<12288' => '0',
##&nbsp;                                                              '<2048' => '0',
##&nbsp;                                                              '<32' => '0',
##&nbsp;                                                              '<24' => '0',
##&nbsp;                                                              '<131072' => '0',
##&nbsp;                                                              '5' => '0'
##&nbsp;                                                            },
##&nbsp;                                                'PUT' => {
##&nbsp;                                                           '<98304' => '0',
##&nbsp;                                                           ... Same as above
##&nbsp;                                                           '5' => '0'
##&nbsp;                                                         },
##&nbsp;                                                'GET' => {
##&nbsp;                                                           '<98304' => '0',
##&nbsp;                                                           ... Same as above
##&nbsp;                                                           '5' => '0'
##&nbsp;                                                         }
##&nbsp;                                              },
##&nbsp;                          'Latency Histogram' => {
##&nbsp;                                                   'DELETE' => {
##&nbsp;                                                                 '100 - <120ms' => '0',
##&nbsp;                                                                 '0 - <1ms' => '0',
##&nbsp;                                                                 '>16000ms' => '0',
##&nbsp;                                                                 '600 - <800ms' => '0',
##&nbsp;                                                                 '2000 - <2500ms' => '0',
##&nbsp;                                                                 '20 - <30ms' => '0',
##&nbsp;                                                                 '70 - <80ms' => '0',
##&nbsp;                                                                 '6 - <8ms' => '0',
##&nbsp;                                                                 '180 - <200ms' => '0',
##&nbsp;                                                                 '2 - <4ms' => '0',
##&nbsp;                                                                 '60 - <70ms' => '0',
##&nbsp;                                                                 '160 - <180ms' => '0',
##&nbsp;                                                                 '16 - <20ms' => '0',
##&nbsp;                                                                 '200 - <400ms' => '0',
##&nbsp;                                                                 '140 - <160ms' => '0',
##&nbsp;                                                                 '120 - <140ms' => '0',
##&nbsp;                                                                 '3000 - <3500ms' => '0',
##&nbsp;                                                                 '3500 - <4000ms' => '0',
##&nbsp;                                                                 '40 - <50ms' => '0',
##&nbsp;                                                                 '30 - <40ms' => '0',
##&nbsp;                                                                 '50 - <60ms' => '0',
##&nbsp;                                                                 '8 - <10ms' => '0',
##&nbsp;                                                                 '1500 - <2000ms' => '0',
##&nbsp;                                                                 '1 - <2ms' => '0',
##&nbsp;                                                                 '12000 - <16000ms' => '0',
##&nbsp;                                                                 '800 - <1000ms' => '0',
##&nbsp;                                                                 '1000 - <1500ms' => '0',
##&nbsp;                                                                 'Total' => '0',
##&nbsp;                                                                 '8000 - <12000ms' => '0',
##&nbsp;                                                                 '80 - <90ms' => '0',
##&nbsp;                                                                 '4000 - <8000ms' => '0',
##&nbsp;                                                                 '2500 - <3000ms' => '0',
##&nbsp;                                                                 '400 - <600ms' => '0',
##&nbsp;                                                                 '10 - <12ms' => '0',
##&nbsp;                                                                 '12 - <16ms' => '0',
##&nbsp;                                                                 '90 - <100ms' => '0',
##&nbsp;                                                                 '4 - <6ms' => '0'
##&nbsp;                                                               },
##&nbsp;                                                   'LIST' => {
##&nbsp;                                                               '100 - <120ms' => '0',
##&nbsp;                                                               ... Same as above
##&nbsp;                                                               '4 - <6ms' => '0'
##&nbsp;                                                             },
##&nbsp;                                                   'PUT' => {
##&nbsp;                                                              '100 - <120ms' => '0',
##&nbsp;                                                              ... Same as above
##&nbsp;                                                              '4 - <6ms' => '0'
##&nbsp;                                                            },
##&nbsp;                                                   'GET' => {
##&nbsp;                                                              '100 - <120ms' => '0',
##&nbsp;                                                              ... Same as above
##&nbsp;                                                              '4 - <6ms' => '0'
##&nbsp;                                                            },
##&nbsp;                                                   'HEAD' => {
##&nbsp;                                                               '100 - <120ms' => '0',
##&nbsp;                                                               ... Same as above
##&nbsp;                                                               '4 - <6ms' => '0'
##&nbsp;                                                             }
##&nbsp;                                                 }
##&nbsp;                        }
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'Bin 1' => {
##&nbsp;                          'Read Stats' => {
##&nbsp;                                            'wcvbs_client_proto_reads' => '0',
##&nbsp;                                            'wcvbs_bin_reads' => '0',
##&nbsp;                                            'wcvbs_ra_reads' => '0',
##&nbsp;                                            'wcvbs_other_proto_reads' => '0',
##&nbsp;                                            'wcvbs_tlog_reads' => '0',
##&nbsp;                                            'wcvbs_repl_proto_reads' => '0'
##&nbsp;                                          },
##&nbsp;                          'Error Stats' => {
##&nbsp;                                             'wcvbs_errors_op_expire_soon' => '0',
##&nbsp;                                             'wcvbs_errors_op_expired' => '0',
##&nbsp;                                             'wcvbs_errors_io_timedout' => '0'
##&nbsp;                                           },
##&nbsp;                          'Size Histogram' => {
##&nbsp;                                                'DELETE' => {
##&nbsp;                                                              '<98304' => '0',
##&nbsp;                                                              '<128' => '0',
##&nbsp;                                                              '<448' => '0',
##&nbsp;                                                              '7' => '0',
##&nbsp;                                                              '<56' => '0',
##&nbsp;                                                              '2' => '0',
##&nbsp;                                                              '<65536' => '0',
##&nbsp;                                                              '<196608' => '0',
##&nbsp;                                                              '1' => '0',
##&nbsp;                                                              '<3584' => '0',
##&nbsp;                                                              '<48' => '0',
##&nbsp;                                                              '<32768' => '0',
##&nbsp;                                                              '<16384' => '0',
##&nbsp;                                                              '<4096' => '0',
##&nbsp;                                                              '<192' => '0',
##&nbsp;                                                              '<262144' => '0',
##&nbsp;                                                              '<8192' => '0',
##&nbsp;                                                              '<64' => '0',
##&nbsp;                                                              '<1536' => '0',
##&nbsp;                                                              '<1024' => '0',
##&nbsp;                                                              '<229376' => '0',
##&nbsp;                                                              '<256' => '0',
##&nbsp;                                                              '<28672' => '0',
##&nbsp;                                                              '<320' => '0',
##&nbsp;                                                              '<20480' => '0',
##&nbsp;                                                              '0' => '0',
##&nbsp;                                                              '<24576' => '0',
##&nbsp;                                                              '<512' => '0',
##&nbsp;                                                              '<40' => '0',
##&nbsp;                                                              '6' => '0',
##&nbsp;                                                              '<163840' => '0',
##&nbsp;                                                              '>262144' => '0',
##&nbsp;                                                              '3' => '0',
##&nbsp;                                                              '<2560' => '0',
##&nbsp;                                                              '<3072' => '0',
##&nbsp;                                                              'UNUSED' => '0',
##&nbsp;                                                              '<16' => '0',
##&nbsp;                                                              '<384' => '0',
##&nbsp;                                                              '4' => '0',
##&nbsp;                                                              '<12288' => '0',
##&nbsp;                                                              '<2048' => '0',
##&nbsp;                                                              '<32' => '0',
##&nbsp;                                                              '<24' => '0',
##&nbsp;                                                              '<131072' => '0',
##&nbsp;                                                              '5' => '0'
##&nbsp;                                                            },
##&nbsp;                                                'PUT' => {
##&nbsp;                                                           '<98304' => '0',
##&nbsp;                                                           ... Same as above
##&nbsp;                                                           '5' => '0'
##&nbsp;                                                         },
##&nbsp;                                                'GET' => {
##&nbsp;                                                           '<98304' => '0',
##&nbsp;                                                           ... Same as above
##&nbsp;                                                           '5' => '0'
##&nbsp;                                                         }
##&nbsp;                                              },
##&nbsp;                          'Latency Histogram' => {
##&nbsp;                                                   'DELETE' => {
##&nbsp;                                                                 '100 - <120ms' => '0',
##&nbsp;                                                                 '0 - <1ms' => '0',
##&nbsp;                                                                 '>16000ms' => '0',
##&nbsp;                                                                 '600 - <800ms' => '0',
##&nbsp;                                                                 '2000 - <2500ms' => '0',
##&nbsp;                                                                 '20 - <30ms' => '0',
##&nbsp;                                                                 '70 - <80ms' => '0',
##&nbsp;                                                                 '6 - <8ms' => '0',
##&nbsp;                                                                 '180 - <200ms' => '0',
##&nbsp;                                                                 '2 - <4ms' => '0',
##&nbsp;                                                                 '60 - <70ms' => '0',
##&nbsp;                                                                 '160 - <180ms' => '0',
##&nbsp;                                                                 '16 - <20ms' => '0',
##&nbsp;                                                                 '200 - <400ms' => '0',
##&nbsp;                                                                 '140 - <160ms' => '0',
##&nbsp;                                                                 '120 - <140ms' => '0',
##&nbsp;                                                                 '3000 - <3500ms' => '0',
##&nbsp;                                                                 '3500 - <4000ms' => '0',
##&nbsp;                                                                 '40 - <50ms' => '0',
##&nbsp;                                                                 '30 - <40ms' => '0',
##&nbsp;                                                                 '50 - <60ms' => '0',
##&nbsp;                                                                 '8 - <10ms' => '0',
##&nbsp;                                                                 '1500 - <2000ms' => '0',
##&nbsp;                                                                 '1 - <2ms' => '0',
##&nbsp;                                                                 '12000 - <16000ms' => '0',
##&nbsp;                                                                 '800 - <1000ms' => '0',
##&nbsp;                                                                 '1000 - <1500ms' => '0',
##&nbsp;                                                                 'Total' => '0',
##&nbsp;                                                                 '8000 - <12000ms' => '0',
##&nbsp;                                                                 '80 - <90ms' => '0',
##&nbsp;                                                                 '4000 - <8000ms' => '0',
##&nbsp;                                                                 '2500 - <3000ms' => '0',
##&nbsp;                                                                 '400 - <600ms' => '0',
##&nbsp;                                                                 '10 - <12ms' => '0',
##&nbsp;                                                                 '12 - <16ms' => '0',
##&nbsp;                                                                 '90 - <100ms' => '0',
##&nbsp;                                                                 '4 - <6ms' => '0'
##&nbsp;                                                               },
##&nbsp;                                                   'LIST' => {
##&nbsp;                                                               '100 - <120ms' => '0',
##&nbsp;                                                               ... Same as above
##&nbsp;                                                               '4 - <6ms' => '0'
##&nbsp;                                                             },
##&nbsp;                                                   'PUT' => {
##&nbsp;                                                              '100 - <120ms' => '0',
##&nbsp;                                                              ... Same as above
##&nbsp;                                                              '4 - <6ms' => '0'
##&nbsp;                                                            },
##&nbsp;                                                   'GET' => {
##&nbsp;                                                              '100 - <120ms' => '0',
##&nbsp;                                                              ... Same as above
##&nbsp;                                                              '4 - <6ms' => '0'
##&nbsp;                                                            },
##&nbsp;                                                   'HEAD' => {
##&nbsp;                                                               '100 - <120ms' => '0',
##&nbsp;                                                               ... Same as above
##&nbsp;                                                               '4 - <6ms' => '0'
##&nbsp;                                                             }
##&nbsp;                                                 }
##&nbsp;                        }
##&nbsp;           }
##&nbsp;         ];
sub wafl_composite_stats_show {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my ( $error, @result_array, @lines);
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'    => $raw_output,
            'command'       => $command,
            'error_message' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }

    my @results_array;

    my @sections = split /(?=Bin 0)|(?=Bin 1)/, $raw_output;

    # Either 3 sections for volumes or 5 for aggregates
    foreach my $section (@sections) {
        # Sections are key values, then a size histogram, and a latency histogram
        if ($section =~ /(Bin \d)/i){
            my $current_bin = $1;
            my @sub_sections = split /(?=Read Stats:)|(?=Error Stats:)|(?=Size Histogram:)|(?=Latency Histogram:)/, $section;
            my %section_hash;
            foreach my $sub_section (@sub_sections) {
                if ($sub_section =~ /Read Stats:/){
                    my @lines = get_lines(raw_output => $sub_section,);
                    foreach my $line (@lines) {
                        my @key_value = split /:\s+/, $line;
                        if (scalar(@key_value) == 2){
                            $section_hash{$current_bin}{'Read Stats'}{$key_value[0]} = $key_value[1];
                        }
                    }
                } elsif ($sub_section =~ /Error Stats:/) {
                    my @lines = get_lines(raw_output => $sub_section,);
                    foreach my $line (@lines) {
                        my @key_value = split /:\s+/, $line;
                        if (scalar(@key_value) == 2){
                            $section_hash{$current_bin}{'Error Stats'}{$key_value[0]} = $key_value[1];
                        }
                    }
                } elsif ($sub_section =~ /Size Histogram:/) {
                    my @lines = get_lines(raw_output => $sub_section,);
                    my (@keys, @values);
                    my ($section_header, $key_index);
                    foreach my $line (@lines) {
                        # Section header is only one word
                        if ($line =~ /^(\w+)$/) {
                            $section_header = $1;
                            $key_index = 0;
                            next;
                        }
                        my @temp_values = split /\s+/, $line;
                        if(scalar(@temp_values) > 3 && ($key_index % 2) == 0){
                            @keys = @temp_values;
                            $key_index++;
                        } elsif(scalar(@temp_values) > 3) {
                            @values = @temp_values;
                            $key_index++;
                            # Add keys and values to hash
                            foreach my $index (0..scalar(@keys)-1){
                                $section_hash{$current_bin}{'Size Histogram'}{$section_header}{$keys[$index]} = $values[$index];
                            }
                        } elsif ($line =~ /(\w+):\s+(\d+)/) {
                            $section_hash{$current_bin}{'Size Histogram'}{$section_header}{$1} = $2;
                        }
                    }
                } elsif ($sub_section =~ /Latency Histogram:/) {
                    my @lines = get_lines(raw_output => $sub_section,);
                    my (@keys, @values);
                    my ($section_header, $key_index);
                    foreach my $line (@lines) {
                        # Section header is only one word
                        if ($line =~ /^(\w+)$/) {
                            $section_header = $1;
                            $key_index = 0;
                            next;
                        }
                        my @temp_values = split /\s\s+/, $line;
                        if(scalar(@temp_values) > 3 && ($key_index % 2) == 0){
                            @keys = @temp_values;
                            $key_index++;
                        } elsif(scalar(@temp_values) > 3) {
                            @values = @temp_values;
                            $key_index++;
                            # Add keys and values to hash
                            foreach my $index (0..scalar(@keys)-1){
                                $section_hash{$current_bin}{'Latency Histogram'}{$section_header}{$keys[$index]} = $values[$index];
                            }
                        } elsif ($line =~ /(\w+):\s+(\d+)/) {
                            $section_hash{$current_bin}{'Latency Histogram'}{$section_header}{$1} = $2;
                        }
                    }
                }
            }
            push (@results_array, \%section_hash);
        } else {
            ## key value pair section
            my $section_header = "";
            my %stat_hash;
            my @lines = split /\r*\n+/, $section;
            foreach my $line (@lines) {
                my @key_value = split /:\s+/, $line;
                if (scalar(@key_value) == 2) {
                    # Lines that start with tab are indented from the header above
                    if ($key_value[0] =~ /^\t/) {
                        $key_value[0] =~ s/\t//;
                        $stat_hash{$section_header}{$key_value[0]} = $key_value[1];
                    } else {
                        $stat_hash{$key_value[0]} = $key_value[1];
                    }
                } elsif (scalar(@key_value) == 1){
                    $section_header = $key_value[0];
                }
            }
            push (@results_array, \%stat_hash);
        }
    }
    return \@results_array;

} # end sub wafl_composite_stats_show

## @name ::wafl_composite_temp
##
## @summary Parses the raw output from the "wafl composite temp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_composite_temp with any of its options as described in the
##    corresponding cdef file 'wafl.cdefs'
## @synopsis
##      wafl_composite_temp(%args);
## INPUTS TO METHOD CALL 'wafl_composite_temp' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggr' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'pvbn' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; wafl composite temp aggr1 9011; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;aggr:aggr1 pvbn:9011 temperature:5
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'aggr' => 'aggr1'
##&nbsp;            'pvbn' => '9011'
##&nbsp;            'temp' => '5'
##&nbsp;          }
##&nbsp;       ];
##&nbsp;
sub wafl_composite_temp {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my ( $error, @result_array, @lines);
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'    => $raw_output,
            'command'       => $command,
            'error_message' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    @lines = get_lines( raw_output => $raw_output, response_object => $resp );
    $lines[0] =~
      /aggr:(\w+)\s+pvbn:(\d+)\s+temperature:(\d+)/;
    $result_array[0]->{ 'aggr' } = $1;
    $result_array[0]->{ 'pvbn' } = $2;
    $result_array[0]->{ 'temp' } = $3;
    return \@result_array;
}

1;
