#
# Copyright (c) 2001-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base Resonse class for all CMode API Sets
## @author olalde@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Response::CLI::CMode;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Response::CLI);
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::_Mixins::CLI qw(_is_help_cmd);
use NACL::APISet::_Mixins::CLIZAPI qw(
    _vserver_locked_recording_baseline_transfer_regex
    _internal_mcc_or_vdr_error_regex
);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

#
# Exceptions imported
#
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::UnsupportedOptionException;
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::CommandNotFoundException;
use NACL::APISet::Exceptions::NoMatchingEntriesException;
use NACL::APISet::Exceptions::MultipleCommandFailedException ();
use NACL::APISet::Exceptions::UnexpectedOutputException;
use NACL::APISet::Exceptions::MgwdCrashedException;
use NACL::APISet::Exceptions::JobFailedException;
use NACL::APISet::Exceptions::VserverLockedRecordingBaselineTransfer;
use NACL::APISet::Exceptions::InternalMCCOrVDRError;
use NACL::Tools::CGT::CGTUtils qw(_determine_split_list);
use List::MoreUtils qw(indexes);

sub new {
    $Log->enter() if $may_enter;

    my ( $class, %args ) = @_;

    # Fill the member vars for this instance
    my $self = NACL::APISet::validate_params(
        $class,
        params => \%args,
        spec   => {
            output               => { type => SCALAR,  required => 1, },
            CLI_separator        => { type => SCALAR,  required => 1, },
            command              => { type => SCALAR,  required => 1, },
            execution_time       => { type => SCALAR,  required => 1, },
            output_compat_mirror => { type => HASHREF, required => 1 },
        }
    );

    # This tags the hash pointed to by $object as a
    # type of CMode
    bless $self, $class;

    # Strip command echo from raw output
    my $output = $self->get_raw_output();
    remove_echo( "output_ptr" => \$output, "command" => $self->{command} );
    $self->{"processed_output"} = $output;
    if ( $self->{command} =~ /(.+)\?.+/ ) {

        # If the command has '?' followed by something, then remove_echo logic
        # won't work
        $self->{"processed_output"} =~ s/^$1\?\r*\n*//;
    } ## end if ( $self->{command} ...)

    # Parse any Job IDs in the raw output.
    # This has to be done separately since the output does not contain "X;X"
    $self->_parse_job_ids();

    # Check the raw output for errors. In the case of an error, an exception
    # is thrown.
    $self->_check_for_errors();

    # Do not parse if we determine it's a help command. If it's a help command
    # then the parsed output should be a hash with a single key (help_output)
    # containing the entire output verbatim. Hence, parsing should not be
    # done if it's a help command.

    if (! $self->_is_help_cmd(command => $self->{command}) ) {

     # CMode contains a generic parser, which is capable of parsing
     # most commands' output. However, it cannot parse (or incorrectly parses)
     # output of certain commands. Brute-force parsers for those
     # commands have been written here (whose method names are the same as the
     # name of the command with a leading underscore). If it does not find an
     # implementation for the method, it means that the generic parser can be
     # used for the command.
        if ( $self->{method_name} ) {
            my $method_name = '_' . $self->{method_name};
            if ( $self->can($method_name) ) {
                $self->$method_name();
            } else {
                $self->_parse_output();
            }
        }
    } else {
        my $help_output = help_output( "raw_output" => $self->{processed_output} );
        $self->{parsed_output} = [ { help_output => $help_output } ];
    }
    $Log->debug( sub {Dumper($self)} );
    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

##############################################################################
# Method:  get_output_not_parsed
# Purpose: Return all those lines which have not been parsed.
# Input:   None.
# Output:  None.
# Notes:   Refer to POD below.
##############################################################################

sub get_output_not_parsed {
    $Log->enter() if $may_enter;

    my $self = shift;

    $Log->exit() if $may_exit;
    return $self->{not_parsed};
} ## end sub get_output_not_parsed

##############################################################################
# Method:  get_job_ids
# Purpose: Return Job IDs in the parsed output (if present)
# Input:   None.
# Output:  None.
# Notes:   Refer to POD below.
##############################################################################

sub get_job_ids {
    $Log->enter() if $may_enter;

    my $self = shift;
    my ( @ids, $flag );
    if (wantarray) {
        $Log->exit() if $may_exit;
        map { push( @ids, $_->{job_id} ) if ( exists $_->{job_id} ) }
            @{ $self->{parsed_output} };
        return @ids;
    } else {
        $Log->exit() if $may_exit;
        return $self->{parsed_output}->[0]->{job_id}
            if exists( $self->{parsed_output}->[0]->{job_id} );
    }
} ## end sub get_job_ids

##############################################################################
# Method:  is_private_job
# Purpose: Return 1 if the job is a private job
# Input:   None.
# Output:  None.
# Notes:   Refer to POD below.
##############################################################################

sub is_private_job {
    $Log->enter() if $may_enter;

    my ($self) = @_ ;
    $Log->exit() if $may_exit;
    return $self->{parsed_output}->[0]->{is_private};
} ## end sub is_private_job

#
# Returns the rows(as reference to array of hashes)containing specified
# field and value
#
sub get_rows {
    my ( $self, %args ) = @_;
    %args = NACL::APISet->validate_params(
        params => \%args,
        spec   => {
            field => { type => SCALAR, required => 1 },
            value => { type => SCALAR, required => 1 },
        }
    );
    my $field  = $args{field};
    my $value  = $args{value};
    my @output = @{ $self->{parsed_output} };

    my @result = ();
    foreach my $hash (@output) {
        foreach my $key ( keys( %{$hash} ) ) {
            if ( $key eq $field ) {
                if ( $hash->{$key} eq $value ) {
                    my %outhash = %{$hash};
                    push( @result, \%outhash );
                }
            }    # end of if key=field
        } ## end foreach my $key ( keys( %{$hash...}))
    } ## end foreach my $hash (@output)
    return \@result;
} ## end sub get_rows

# This is to prevent scripts from failing with ugly error messages like this:
# Method 'NACL::APISet::Response::CLI::CMode::DESTROY' not defined on instance
#'NACL::APISet::Response::CLI::CMode=HASH(0x3c36760)' at ....
# The issue is essentially that if an AUTOLOAD is written, it will end up
# catching DESTROY (which is Perl's built-in implicitly called destructor).
# So, at runtime, when the class tries to define itself completely, including
# DESTROY, it fails.
# Note that this is a runtime error and occurs intermittently (basically as and
# when it pleases) so this error could be difficult to replicate. But,
# for any class with an AUTOLOAD, this error _could_ happen, so it's best
# we follow this convention of defining an empty DESTROY for such classes

# Note that an alternative mechanism of handling this is to have a condition
# in the AUTOLOAD like
# if ($AUTOLOAD eq "DESTROY") {
#     return; }

sub DESTROY { }

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:  _parse_job_ids
# Purpose: Used to parse Job IDs if they appear in the raw output
# Inputs:  None.
# Outputs: None
##############################################################################

sub _parse_job_ids {
    my ($self) = @_;
    my %hash;
    my $jobid;
    $Log->enter() if $may_enter;
    $Log->debug("Trying to parse Job IDs in the output");
    my $output = $self->{'processed_output'};
    my @lines = split( /\n/, $output );
    my $private = 0;
    foreach (@lines) {

        if ($_ !~ /$self->{CLI_separator}/
            && (   $_ =~ /\[Job (\d+)\].*(?:\cH)*/
                || $_ =~ /[jJ]ob: (\d+).*(?:\cH)*/
		|| $_ =~ /[jJ]ob:? \"(\d+).*(?:\cH)*/
                || $_ =~ /.*operation-id\s*(\d*).*(?:\cH)*/
		|| $_ =~ /.*[jJ]ob\s+\[id:\s*(\d*).*(?:\cH)*/)
            )
        {
            $jobid = $1;
            $hash{$jobid} = $jobid;
            $private = 1 if ($_ =~ /Queued private job/i);
            push( @{ $self->{parsed_output} },
                { 'job_id' => $jobid, 'is_private' => $private } );
        } ## end if ( $_ !~ /$self->{CLI_separator}/...)
    } ## end foreach (@lines)
} ## end sub _parse_job_ids

##############################################################################
# Method:  _check_for_errors
# Purpose: Check for errors and throw a relevant exception if an error is
#          found.
# Input:   None.
# Output:  None.
# Notes:   If an error occurs while running a command on a CMode filer,
#          the output will be in one of two forms:
#          1) Output contains Usage: <usage details>
#             This occurs when a command is invoked with none of the required
#             parameters. Though the output itself might not appear like an
#             error message, this will show up ONLY if none of the required
#             parameters are provided. Hence, it is a
#             MissingRequiredParamException.
#          2) The output is "ERROR:(or)Error: <error message>" The "ERROR (or)
#             Error" indicates
#             the occurence of an error (obviously). The error message
#             itself indicates the kind of error that has occured. The logic
#             here is to analyse the error message to determine what kind of
#             error it is and throw an appropriate exception
# Exceptions: The following are the exceptions that this method could throw:
#             1) MissingRequiredParamException
#             2) InvalidParamException
#             3) InvalidParamValueException
#             4) UnsupportedOptionException
#             5) ResponseException
#             6) CommandFailedException
#             7) CommandNotFoundException
##############################################################################

sub _check_for_errors {
    $Log->enter() if $may_enter;
    my $self = shift;
    my $output = $self->{'processed_output'} || '';
    $Log->debug( "Raw output is " . $output );
    if ( $output
        =~ /The connection with the management gateway server has been reestablished.*/i
        )
    {
        NACL::APISet::Exceptions::MgwdCrashedException->throw(
            text            => $output,
            response_object => $self,
        );
    }
    if ( $output =~ /^\s*Usage:/i ) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(
            text            => "No parameters were passed",
            response_object => $self
        );
    } ## end if ( $output =~ /^\s*Usage:/i)
    if ( $output =~ /\cG/ ) {
        if (    $output =~ /time.*not appear to be synchronised/i
            and $self->{command} =~ /event log show/ )
        {

   # When we run "event log show",
   # we get this -
   #"Error: The time does not appear to be synchronised across this cluster".
   #followed by the output. Hence, an exception should not be thrown on seeing
   #this error message. Please refer burt622209.
            if ( $output =~ /$self->{CLI_separator}/ ) {
                $self->_parse_output();
                NACL::APISet::Exceptions::CommandFailedException->throw(
                    text            => "$output",
                    response_object => $self
                );
            } else {
                NACL::APISet::Exceptions::ResponseException->throw(
                    text            => "$output",
                    response_object => $self
                );
            }
        } elsif ( $output=~ /Reason: entry does(n't|\s*not) exist|vol offline: Error offlining volume/ )
        {
            NACL::APISet::Exceptions::CommandFailedException->throw(
                text            => "$output",
                response_object => $self
            );
        } elsif ( $output =~ /^\s*\cG*E((?i)RROR):\s/m ) {
            $output =~ s/(.|\n)+?E((?i)RROR):\s//;
            if (   ( $output =~ /required\sfield/ )
                or ( $output =~ /.*:\s(Either\sspecify\sall\skeys.*)\./ )
                or ( $output =~ /Must\sspecify\sa/ ) )
            {
                NACL::APISet::Exceptions::MissingRequiredParamException
                    ->throw(
                    text            => "$output",
                    response_object => $self
                    );
            } elsif ( ( $output =~ /invalid\sargument/ )
                or ( $output =~ /specify\s\-fieldname\sfirst/ )
                or ( $output =~ /Ambiguous\sargument/ ) )
            {
                NACL::APISet::Exceptions::InvalidParamException->throw(
                    text            => "$output",
                    response_object => $self
                );
            } elsif ( ( $output =~ /invalid\svalue\sfor\sfield/ )
                or ( $output =~ /(Missing\svalue\sfor\s.*)\./ ) )
            {
                NACL::APISet::Exceptions::InvalidParamValueException->throw(
                    text            => "$output",
                    response_object => $self
                );
            } elsif ( $output =~ /operation\sis\snot\ssupported/ ) {
                NACL::APISet::Exceptions::UnsupportedOptionException->throw(
                    text            => "$output",
                    response_object => $self
                );
            } elsif ( $output =~ /not a recognized command/ ) {
                NACL::APISet::Exceptions::CommandNotFoundException->throw(
                    text            => "$output",
                    response_object => $self
                );
            } elsif (
                $output =~ /^There\sare\sno\sentries\smatching\syour\squery/ )
            {

                # This is for something like job-watch progress which
                # shows this message but also has a leading "Error:"
                # (contrast this with the case given below)
                # Example:
                # Cluster1::job> watch-progress -id 1234567
                # Error: There are no entries matching your query.

                NACL::APISet::Exceptions::NoMatchingEntriesException->throw(
                    text            => "$output",
                    response_object => $self
                );
            } elsif ( ( $output =~ /does\snot\sexist/ )
                or ( $output =~ /doesn't\sexist/ )
                or ( $output =~ /not\sfound/ ) )
            {

                # Fix for burt 521764 and 614409
                $Log->exit() if $may_exit;
                NACL::APISet::Exceptions::NoMatchingEntriesException->throw(
                    text            => "$output",
                    response_object => $self
                );
            } elsif ($output =~ $self->_vserver_locked_recording_baseline_transfer_regex()) {
                $Log->exit() if $may_exit;
                NACL::APISet::Exceptions::VserverLockedRecordingBaselineTransfer->throw(
                    text            => $output,
                    response_object => $self
                );
            } elsif ($output =~ $self->_internal_mcc_or_vdr_error_regex()) {
                $Log->exit() if $may_exit;
                NACL::APISet::Exceptions::InternalMCCOrVDRError->throw(
                    text            => $output,
                    response_object => $self
                );
            } elsif ( $output =~ /command\sfailed/ ) {

               # It is possible for multiple failures if extended query
               # syntax is used. For example:
               # cluster1::> aggregate modify { -state online } -state online
               # Error: command failed on aggregate "aggr0": Aggregate "aggr0"
               # is already online.
               # Error: command failed on aggregate "component_flex_aggr":
               # Aggregate "component_flex_aggr" is already online.
                my @matches = ( $output =~ /command\sfailed/g );
                if ( @matches > 1 ) {

                    # Multiple failures
                    my @error_messages = split /Error:\s*/i, $output;
                    NACL::APISet::Exceptions::MultipleCommandFailedException
                        ->throw(
                        text            => $output,
                        response_object => $self,
                        error_messages  => \@error_messages
                        );
                } else {

                    # Single failure
                    NACL::APISet::Exceptions::CommandFailedException->throw(
                        text            => "$output",
                        response_object => $self
                    );
                }
            } elsif ( $output =~ /.*chpass: the password db file is busy.*/ )
            {
                NACL::APISet::Exceptions::CommandFailedException->throw(
                    text            => $output,
                    response_object => $self,
                );
            } else {
                NACL::APISet::Exceptions::ResponseException->throw(
                    text            => "Response exception: $output",
                    response_object => $self
                );
            } ## end else [ if ( ( $output =~ /required\sfield/...))]
        } elsif ( $output =~ /^There\s+are\s+no\s+entries\s+matching\s+your\s+query/ ) {
            # Fix for the burt 809271
            # This case arises when \cG comes with
            # "There are no entries matching your query" message

            NACL::APISet::Exceptions::NoMatchingEntriesException->throw(
                text            => "$output",
                response_object => $self
            );

        } elsif ( $output =~ /\cG\r*\n*(.+?)\s(.+?):\s/ ) {
            my ( $c1, $c2 ) = ( $1, $2 );
            ( $c1, $c2 ) = ( quotemeta($c1), quotemeta($c2) );
            if ( $self->get_command() =~ /$c1.*\s$c2/ ) {

                # This is a 7M like error message. Not sure whether
                # they all are cases of only command failures or not. So let's
                # throw generic ResponseException.
                NACL::APISet::Exceptions::ResponseException->throw(
                    text            => "$output",
                    response_object => $self
                );
            } ## end if ( $self->get_command...)

            #\cG[Job 14] Job failed: Failed to setup peer clusters. Retry the command again with "-refresh true".\r\n
            elsif ( $output =~ /\cG.*\[Job\s+\d+\]\s+Job\s+failed.*/i ) {
                NACL::APISet::Exceptions::JobFailedException->throw(
                    text           => $output,
                    response_object => $self
                );
            } ## end elsif ( $output =~ /\cG.*\[Job\s+\d+\]\s+Job\s+failed.*/i )
        } ## elsif ( $output =~ /\cG\r*\n*(.+?)\s(.+?):\s/ )
    } elsif ( $output =~ /^There\sare\sno\sentries\smatching\syour\squery/ ) {

        # Fix for the burt 429699
        # This is for "show" type commands when we try to filter.
        # Output does not contain "Error:", but we still consider it an error
        # Example:
        # Cluster1::volume> show -volume volumenotexisting
        # There are no entries matching your query.

        NACL::APISet::Exceptions::NoMatchingEntriesException->throw(
            text            => "$output",
            response_object => $self
        );
    }   elsif ( $output =~ /Failed accessing TPM device/i ) {
	    NACL::APISet::Exceptions::ResponseException->throw(
		text            => "$output",
		response_object => $self
	    );
	} ## end elsif ( $output =~ ...)

    $Log->exit() if $may_exit;
} ## end sub _check_for_errors

##############################################################################
# Method:  _parse_output
# Purpose: Parse the provided raw output into a standard structure - array
#          of hashes. The keys of the hashes being the keys and each entry
#          in the array for one row in the output.
##############################################################################

sub _parse_output {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    # if parsed output was already set (by _parse_job_ids)
    return if $self->{parsed_output};

    my $want_array = [];

    my $output        = $self->{processed_output};
    my $CLI_separator = $self->{CLI_separator};
    my %compat_mirror = %{ $self->{output_compat_mirror} };
    my $outp_comp     = ( scalar( keys %compat_mirror ) ) ? 1 : 0;

    # Remove all warning lines from the output.
    # The check below ( elsif ( $line =~ /^W((?i)ARNING)/ ){...})
    # works correctly only if it's after the heading and fields line.
    # Hence, removing Warnings is required here (burt533448).
    $output =~ s/^W((?i)ARNING).*//g;
    my $command = $self->{method_name};

    # Remove the Notice line introduced by burt 788982
    $output =~ s/^\n*Notice:.*press Ctrl-C.//sg if ($command =~ /vserver_locks_show/);

    # just skip blank lines. Note that \w does not match quotes, so include
    # that as well.
    my @lines = split( /\n/, $output );
    @lines = grep {/\w|"/} @lines;

    ## Removing '\\r\\n' from the output
    my $lines ;
    foreach $lines (@lines) {
        if ($command =~ /network_ipspace_show/) {
            $lines =~ s/.*critical:EMERGENCY].*disabled.//sg if($lines =~ '.*critical:EMERGENCY].*disabled.');
            $lines =~ s/.*TIMEOUT\s*-\s*WRITE_DMA retrying\s*\(\d+ retries left\)\r\n//sg if($lines =~ '.*TIMEOUT\s*-\s*WRITE_DMA retrying\s*\(\d+ retries left\)');
        }
        $lines =~ s|\\r\\n||sg ;
    }

    # If the output has nothing than new lines (no output), we can return back
    # here itself
    if ( !scalar(@lines) ) {
        $self->{parsed_output} = [];
        $Log->exit() if $may_exit;
        return;
    }

    my ( $fields, $heading, @tmp_output ) = @lines;
    my ( @fields, @heading );
    if ( defined $fields ) {
        @fields = split( $CLI_separator, $fields );
    }
    if ( defined $heading ) {
        @heading = split( $CLI_separator, $heading );
    }

    my @not_parsed = ();

    $Log->debug(sub {"Fields:\n" . Dumper(\@fields)});
    $Log->debug(sub {"Heading:\n" . Dumper(\@heading)});

    # Some output still contains embedded newlines in the fields so we cannot
    # totally depend on the @tmp_output containing a single line for each row in
    # in the CLI. So, loop through it once and find any elements of @array
    # (the row) which do not contain the right number of elements (values
    # for the row). The right number should be the number of elements of
    # @heading.
    #
    # For any discrepencies, try to reform the row by combining multiple
    # elements of @tmp_output into a new @output.
    my @output;
    my $partial_line = "";
    unless ( defined $args{parse_line_with_no_cli_separator} ) {
        $args{parse_line_with_no_cli_separator} = 1;

    }
    foreach my $line (@tmp_output) {

        # We're here to parse the output into key/value structures,
        # save skipped lines

        if ( $line =~ /^I((?i)NFO)/ ) {
            $Log->debug("Found INFO or Info line. Ignoring: \"$line\"");
            push( @not_parsed, $line );
            next;
        } elsif ( $line =~ /^W((?i)ARNING)/ ) {
            $Log->debug("Found WARNING or Warning line. Ignoring: \"$line\"");
            push( @not_parsed, $line );
            next;
        } elsif ( $line =~ /-showseparator \"$CLI_separator\"/ ) {

       #Ignore any separator entry that is returned via the debug_log_show cmd
            $Log->debug(
                "Found output separator config line. Ignoring: \"$line\"");
            push( @not_parsed, $line );
            next;
        } elsif ( $line !~ /$CLI_separator/
            && $args{parse_line_with_no_cli_separator} )
        {

            #For commands like security certificate show,
            #the raw output contains values for a single field that spans
            #multiple lines without CLI Separators, which results in
            #such lines being placed in the not_parsed array.
            #To prevent this behaviour, a parse_line_with_no_cli_separator
            #is set to 0 from the brute force parser of such commands.
            #The default value of the parse_line_with_no_cli_separator variable is set to 1.
            $Log->debug("Missing CLI separator '$CLI_separator' in the line");
            push( @not_parsed, $line );
            next;
        } ## end elsif ( $line !~ /$CLI_separator/...)
        $self->{not_parsed} = \@not_parsed;

        my @values = split( $CLI_separator, $line );
        if ( @values != @heading ) {
            # The "test_multi_line_value" test-case in api_cmode_cli.thpl is a
            # test for this block of code.
            if (length $partial_line) {
                # $partial_line already contains some lines, so append $line
                # after a new-line character
                $partial_line .= "\n$line";
            } else {
                # $partial_line not yet defined, so assign it to $line to
                # begin with
                $partial_line .= $line;
            }

            # make sure we have seen the end of the row
            next if ( $partial_line !~ /$CLI_separator$/ );

            my @partial = split( $CLI_separator, $partial_line );
            if ( @partial == @heading ) {
                $Log->debug("done with merge: $partial_line");
                push( @output, $partial_line );
                $partial_line = "";
            } elsif ( @partial > @heading ) {
                my $error_text;
                $error_text = "Error while parsing CMode CLI output!\n";
                $error_text .= "Failed to handle embedded newlines in the "
                    . "CLI output\n";
                $error_text .= "line=$line\n";
                $error_text .= "partial_line=$partial_line";
                NACL::APISet::Exceptions::ResponseException->throw(
                    text            => $error_text,
                    response_object => $self
                );
            } ## end elsif ( @partial > @heading)
        } else {
            push( @output, $line );
        }
    }    #end of foreach my $line (@tmp_output)

    # Get the output compatibility keys to be added for each field
    # Done here so that we need not process the same for each row
    # of output.
    my %conversion_fields;
    if ($outp_comp) {
        foreach my $actual_key (@fields) {
            if (defined $compat_mirror{$actual_key}) {
                my %temp = %compat_mirror;

                my $output_key = $actual_key;
                $conversion_fields{$actual_key} = [];
                while ( defined $temp{$output_key} ) {
                    $Log->debug( "Output compatibility: "
                            . "$output_key is "
                            . "converted from "
                            . $compat_mirror{$output_key} );
                    $output_key = delete $temp{$output_key};
                    push( @{ $conversion_fields{$actual_key} }, $output_key );
                }
            }
        }
        $Log->debug( sub {"Output conversion array for all fields: "
                . Dumper( \%conversion_fields )} );
    } ## end if ($outp_comp)

    # This hash keeps the list of commands/options which are wrongly getting parsed
    # as array instead of string.
    my %scalar_commands_options = (
        'job_schedule_cron_show'               => ['description'],
        'system_capability_show'               => ['howto-downgrade'],
        'system_capability_clusterset_show'    => ['howto-downgrade'],
        'volume_snapshot_show'                 => ['record-owner'],
        'storage_aggregate_show'               => ['raidstatus'],
        'storage_aggregate_show_resync_status' => ['status'],
        'volume_quota_report'                  => ['quota-specifier','tree'],
        'volume_quota_policy_rule_show'        => ['target'],
        'storage_aggregate_plex_show'          => ['status'],
        'vserver_services_nis_domain_show'     => ['domain'],
        'vserver_show'                         => ['nisdomain'],
        'job_schedule_show'                    => ['description'],
        'volume_quota_policy_show'             => ['policy-name'],
        'statistics_archive_show'              => ['path'],
        'security_certificate_show'            => ['organization'],
        'volume_efficiency_show'               => ['schedule'],
        'network_fcp_adapter_show'             => ['sfp-vendor-name'],
        'debug_vreport_show'                   => ['object'],
        'volume_move_target_aggr_show'         => ['storagetype'],
        'volume_qtree_show'                    => ['qtree'],
        'reallocate_show'                      => ['description'],
        'vserver_audit_show'                   => ['rotate-schedule-description'],
        'event_log_show'                       => ['action','description','event'],
        'storage_aggregate_show_view'          => ['status'],
        'statistics_catalog_counter_show'      => ['properties'],
        'vserver_fcp_initiator_show'           => ['port-address','wwnn','wwpn'],
        'vserver_services_name_service_ldap_client_show' => ['netgroup-byhost-dn'],
        'vserver_cifs_group_policy_central_access_rule_show_defined'
                                               => ['proposed'],
        'vserver_security_trace_trace_result_show'
                                               => ['result'],
        'vserver_locks_nfsv4_show'             => ['client-name'],
        'vserver_services_ldap_client_show'    => ['base-dn','user-dn','bind-dn','group-dn','netgroup-dn','netgroup-byhost-dn'],
        'volume_show_df'                       => ['aggregate'],
        'debug_crs_queue_show'                 => ['row'],
        'statistics_show'                      => ['value'],
        'event_route_show'                     => ['action','description'],
        'vserver_services_name_service_ldap_client_show' => ['base-dn','user-dn','bind-dn',
                                                   'group-dn','netgroup-dn','netgroup-byhost-dn'],
        'qos_policy_group_show'                => ['throughput-policy','max-throughput'],
	'system_node_autosupport_compliant_show' => ['plaintext'],
        );

    # This hash keeps the list of commands/options which are wrongly getting
    # parsed scalar instead of as array

    my %array_commands_options = (
        'statistics_archive_show' => ['source-nodes'],
        'cluster_peer_show'       => ['stable-addrs'],
        'storage_disk_show'       => ['nodelist'],
        'system_ha_interconnect_config_show' => [qw(port-name ipaddress flags)],
        'system_ha_interconnect_port_show' => [qw(
            port phy-layer-state link-layer-state phy-link-up-count
            phy-link-down-count is-active-link
        )],
	'network_port_dcb_show'   => [qw(pgid priority application bandwidth)],
    );

    my $is_warn_logged = 0;
    foreach my $line (@output) {
        my %output = ();
        my $index  = 0;
        foreach my $field ( split( $CLI_separator, $line ) ) {
            ##################################################################
            # NOTE: The splitting of a list field on a comma will break
            # if the field is of type list AND accepts a list of text values
            # (or some other field type) that supports a comma.
            # We have not seen instances of this type of field yet but will
            # evaluate if problems occur
            ##################################################################

            ##################################################################
            # If the field starts with quotes, split the individual strings
            # separated by comma-space. Otherwise, treat entire thing as one
            # string. Some job output is missing the trialing ".
            ##################################################################
            my $field_name = $fields[$index];
            if (!defined $field_name) {
                # Shouldn't ever reach here
                $Log->warn('NACL::APISet::Response::CLI::CMode::_parse_output: '
                    . 'field_name obtained after splitting was undefined '
		    . 'so fields and the values do not match. i.e.  '
                    . "Full line is :\n$line\nMail dl-nacl-dev regarding "
                    . 'the issue') if (!$is_warn_logged++);
                next;
            }
            if (( exists $scalar_commands_options{$command} )
                && ( grep { $_ eq $fields[$index] }
                    @{ $scalar_commands_options{$command} } )
                )
            {
                $output{ $field_name } = $field;
            } elsif (
                ( exists $array_commands_options{$command} )
                && (grep {
                        $_ eq $fields[$index]
                    } @{ $array_commands_options{$command} }
                )
                )
            {
                my $array_ref = [ split( /, */, $field ) ];
                $Log->debug("array formed because of '$field'");
                $output{ $field_name } = $array_ref;
            } elsif (  $field =~ /","/) {
                my $array_ref = [ split( /","/, $field ) ];
                $array_ref->[0] =~ s/^"//;
                $array_ref->[-1] =~ s/"$//;
                $output{ $field_name } = $array_ref;
            } elsif ( $field =~ /^\".+\"?$/ || $field =~ /^\{.+\}$/ ) {
                if ( $field =~ /, *\"/ ) {
                    my $array_ref = [ split( /, */, $field ) ];
                    $Log->debug("array formed because of '$field'");
                    $output{ $field_name } = $array_ref;
                } else {
                    $output{ $field_name } = $field;
                }
            } elsif ( $field =~ /, */ ) {
                my $array_ref = [ split( /, */, $field ) ];
                $Log->debug("array formed because of '$field'");
                $output{ $field_name } = $array_ref;
            } else {
                $output{ $field_name } = $field;
            }
            if ($outp_comp) {

           # If this key was converted from an old param, insert the old one
           # too. Cyclic changes of the form A=>B, B=>C, C=>D, D=>B will see
           # the old key "A" to be missing in the output as 'A' is not covered
           # in the cycle. This is not currently handled and when we see the
           # need, we will do it then.
                if (exists $conversion_fields{$field_name}) {
                    my $value = $output{$field_name};
                    foreach my $key (@{$conversion_fields{$field_name}}) {
                        $output{$key} //= $value;
                    }
                }
            } ## end if ($outp_comp)
            $index++;
        } ## end foreach my $field ( split( ...))


        # Below line removes hash entry with key as "\r" that is left
        # after parsing all other lines for key value pairs.
        delete $output{"\r"} if exists( $output{"\r"} );

        push( @$want_array, \%output );
    } ## end foreach my $line (@output)
    $Log->debug( sub {"Parsed output is:\n" . Dumper($want_array)} );
    $Log->exit() if $may_exit;

    $self->{parsed_output} = $want_array;
} ## end sub _parse_output

#######################################################################
## Method       : _fetch_field_info_and_sort_key
## Objective    : For statistics command output, there is only header
##                information in the output. To parse the iterative
##                output, the common parser expects field information
##                in the output. We need to generate fields based on the
##                headers for this reason.
#######################################################################
sub _fetch_field_info_and_sort_key {
    $Log->enter() if $may_enter;
    my( $self, $header_line ) = @_;

    my $CLI_separator   = $self->{CLI_separator};
    my @headers         = split(/$CLI_separator/, $header_line);

    my (@fields, $fields_line, $sort_key);
    foreach my $header(@headers) {
        $header = lc($header);
        $header =~ s/\s/-/g;
        $header =~ s/\(|\)//g;
        $header =~ s/%/percent/g;
        if($header =~ /\*/) {
            $header =~ s/[*]//g;
            $sort_key = $header;
        }
        push @fields, $header;
    }
    $fields_line = join($CLI_separator, @fields);

    $Log->exit() if $may_exit;
    return ($fields_line, $sort_key);
}

#######################################################################
## Method       : _parse_iterative_output
## Objective    : Parses the iterative output of statistics view CLI
##                output.
######################################################################
sub _parse_iterative_output {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    my $output = $self->get_processed_output();

    # Remove all the blank lines.
    my @lines = grep {$_} split /\n/, $output;

    #Get header, field and sort key information
    my $header_info = $lines[1];
    my ($field_info, $sort_key)  = $self->_fetch_field_info_and_sort_key($header_info);
    @lines = grep{!/\Q$header_info\E/} @lines;

    #Determine the number of iterations based on the timestamp line.
    my @timestamp_indices = indexes { $_ =~ /\d+\/\d+\/\d+\s\d+:\d+:\d+/ } @lines;
    push @timestamp_indices, scalar(@lines);

    my $CLI_separator   = $self->{CLI_separator};
    my $want_array = [];
    #Get the iteration output and call common parser to parse the output.
    foreach my $i(0 .. scalar(@timestamp_indices)- 2) {
        my $timestamp_idx       = $timestamp_indices[$i];
        my $next_timestamp_idx  = $timestamp_indices[$i+1];

        my ($scope, $timestamp) = $lines[$timestamp_idx] =~ /(.*)$CLI_separator(.*)/;

        my @iteration = ( $field_info, $header_info, @lines[$timestamp_idx+1 .. $next_timestamp_idx-1]);

        $self->{processed_output} = join("\n", @iteration ) . "\n";
        if($self->{parsed_output}) { $self->{parsed_output} = undef};
        $self->_parse_output();

        my %output = ( iteration    => $i+1,
                       timestamp    => $timestamp,
                       scope        => $scope,
                       sort_key     => $sort_key,
                       statistics   => $self->{parsed_output}
                     );

        push( @$want_array, \%output);
    }

    $Log->exit() if $may_exit;
    $self->{processed_output}   = $output;
    $self->{parsed_output}      = $want_array;
}

########################################################################
## Method    : _statistics_aggregate_show
## Objective : Parses the output of "statistics aggregate show" command
## Details   : This method can be used to parse the raw output obtained
##             after calling the method from statistics_aggregate_show with
##             any of its options as described in 'statistics_aggregate_show'
##             cdef file.
########################################################################
sub _statistics_aggregate_show {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $self->_parse_iterative_output();

    $Log->exit() if $may_exit;
}

########################################################################
## Method    : _statistics_volume_show
## Objective : Parses the output of "statistics volume show" command
## Details   : This method can be used to parse the raw output obtained
##             after calling the method from statistics_volume_show with
##             any of its options as described in 'statistics_volume_show'
##             cdef file.
########################################################################
sub _statistics_volume_show {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $self->_parse_iterative_output();

    $Log->exit() if $may_exit;
}

########################################################################
## Method    : _statistics_vserver_show
## Objective : Parses the output of "statistics vserver show" command
## Details   : This method can be used to parse the raw output obtained
##             after calling the method from statistics_vserver_show with
##             any of its options as described in 'statistics_vserver_show'
##             cdef file.
########################################################################
sub _statistics_vserver_show {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $self->_parse_iterative_output();

    $Log->exit() if $may_exit;
}

########################################################################
## Method    : _statistics_lif_show
## Objective : Parses the output of "statistics lif show" command
## Details   : This method can be used to parse the raw output obtained
##             after calling the method from statistics_lif_show with
##             any of its options as described in 'statistics_lif_show'
##             cdef file.
########################################################################
sub _statistics_lif_show {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $self->_parse_iterative_output();

    $Log->exit() if $may_exit;
}

########################################################################
## Method    : _statistics_lun_show
## Objective : Parses the output of "statistics lun show" command
## Details   : This method can be used to parse the raw output obtained
##             after calling the method from statistics_lun_show with
##             any of its options as described in 'statistics_lun_show'
##             cdef file.
########################################################################
sub _statistics_lun_show {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $self->_parse_iterative_output();

    $Log->exit() if $may_exit;
}

########################################################################
## Method    : _statistics_system_show
## Objective : Parses the output of "statistics system show" command
## Details   : This method can be used to parse the raw output obtained
##             after calling the method from statistics_system_show with
##             any of its options as described in 'statistics_system_show'
##             cdef file.
########################################################################
sub _statistics_system_show {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $self->_parse_iterative_output();

    $Log->exit() if $may_exit;
}

########################################################################
## Method    : _statistics_node_show
## Objective : Parses the output of "statistics node show" command
## Details   : This method can be used to parse the raw output obtained
##             after calling the method from statistics_node_show with
##             any of its options as described in 'statistics_node_show'
##             cdef file.
########################################################################
sub _statistics_node_show {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $self->_parse_iterative_output();

    $Log->exit() if $may_exit;
}

########################################################################
## Method    : _statistics_port_fcp_show
## Objective : Parses the output of "statistics port fcp show" command
## Details   : This method can be used to parse the raw output obtained
##             after calling the method from statistics_port_fcp_show with
##             any of its options as described in 'statistics_port_fcp_show'
##             cdef file.
########################################################################
sub _statistics_port_fcp_show {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $self->_parse_iterative_output();

    $Log->exit() if $may_exit;
}

########################################################################
## Method    : _statistics_cache_flash_pool_show
## Objective : Parses the output of "statistics cache flash pool show" command
## Details   : This method can be used to parse the raw output obtained
##             after calling the method from statistics_cache_flash_pool_show with
##             any of its options as described in 'statistics_cache_flash_pool_show'
##             cdef file.
########################################################################
sub _statistics_cache_flash_pool_show {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $self->_parse_iterative_output();

    $Log->exit() if $may_exit;
}

########################################################################
## Method    : _statistics_disk_show
## Objective : Parses the output of "statistics disk show" command
## Details   : This method can be used to parse the raw output obtained
##             after calling the method from statistics_disk_show with
##             any of its options as described in 'statistics_disk_show'
##             cdef file.
########################################################################
sub _statistics_disk_show {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $self->_parse_iterative_output();

    $Log->exit() if $may_exit;
}

########################################################################
## Method    : _statistics_workload_show
## Objective : Parses the output of "statistics workload show" command
## Details   : This method can be used to parse the raw output obtained
##             after calling the method from statistics_workload_show with
##             any of its options as described in 'statistics_workload_show'
##             cdef file.
########################################################################
sub _statistics_workload_show {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $self->_parse_iterative_output();

    $Log->exit() if $may_exit;
}

############################################################################
# @name :: _statistics_periodic
##
## @summary Parses the raw output from the "statistics periodic" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from statistics_periodic with any of its options as described in the
##    corresponding cdef file 'statistics_periodic.cdefs'
## @synopsis
##      statistics_periodic(%args);
## INPUTS TO METHOD CALL 'statistics_periodic' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-object' </td> <td> 'object' </td> <td> '0' </td> </tr>
## <tr> <td> '-summary' </td> <td> 'summary' </td> <td> '0' </td> </tr>
## <tr> <td> '-iterations' </td> <td> 'iterations' </td> <td> '0' </td> </tr>
## <tr> <td> '-category' </td> <td> 'category' </td> <td> '0' </td> </tr>
## <tr> <td> '-interval' </td> <td> 'interval' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub _statistics_periodic {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    my $CLI_separator = $self->{CLI_separator};
    my @lines = split( /\n/, $self->get_processed_output() );

    if ( grep( /$CLI_separator/, @lines ) ) {
        $Log->exit() if $may_exit;
        return $self->_statistics_periodic_with_CLI_separator();
    } else {
        $Log->exit() if $may_exit;
        return $self->_statistics_periodic_without_CLI_separator();
    }
}

sub _statistics_periodic_with_CLI_separator {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    $Log->debug("In brute force parser for statistics periodic");

    my $result = [];

    # Parse the output in a bruteforce way
    my @lines = split( /\n/, $self->get_processed_output() );

    # Check if summary is not set to false
    my $summary = grep ( /Minimums/, @lines );

    my $instance;
    my ($object) = $self->get_command() =~ /-object\s+(\S+)/;
    my ($node)   = $self->get_command() =~ /-node\s+(\S+)/;
    if ( $object || $node ) {
        ($instance) = $self->get_command() =~ /-instance\s+(\S+)/;
    } else {
        $instance = $object = "cluster";
    }

    $instance =~ s/"//g;
    $object =~ s/"//g if ($object);

    my @not_parsed;
    my $regex;
    if ($node) {
        $regex = qr/($node\.$instance)|(\d+\/\d+\/\d+\s+\d+:\d+:\d+)/;
    } else {
        $regex = qr/($object\.$instance)|(\d+\/\d+\/\d+\s+\d+:\d+:\d+)/;
    }
    @not_parsed = grep ( /$regex/,  @lines );
    @lines      = grep ( !/$regex/, @lines );

    ($result->[0]->{timeinterval}->[0]->{'starttime'},
      $result->[0]->{timeinterval}->[0]->{'endtime'}) =
      map {s/(.*)(\d+\/\d+\/\d+\s+\d+:\d+:\d+)/$1/; $2} @not_parsed;


    # Read headings
    my $CLI_separator = $self->{CLI_separator};

    my @headings = split( /$CLI_separator/, $lines[0]);
    for ( my $i = 0; $i < scalar(@headings); $i++ ) {
        $headings[$i] = to_key_name( item => $headings[$i] );
    }

    @lines      = grep ( !/$lines[0]/, @lines );

    # Removing Warning messages from raw output
    push( @not_parsed, grep ( /Warning:/, @lines ) );
    @lines = grep ( !/Warning:/, @lines );

    # Removing summary headings
    if ( $summary > 0 ) {
        push(
            @not_parsed,
            grep ( /(Minimums:)|(Averages for \d+ samples:)|(Maximums:)/i,
                @lines )
        );
        @lines
            = grep ( !/(Minimums:)|(Averages for \d+ samples:)|(Maximums:)/i,
            @lines );
    }

    # Parse the output
    if ($summary) {

        # If Summary is true then last three stats will be
        # Maximums, Averages, Minimums respectively in reverse order
        my @maximums = split( /$CLI_separator/, pop(@lines) );
        my @averages = split( /$CLI_separator/, pop(@lines) );
        my @minimums = split( /$CLI_separator/, pop(@lines) );
        for ( my $i = 0; $i < scalar(@headings); $i++ ) {
            $result->[0]->{minimums}->[0]->{ $headings[$i] } = $minimums[$i];
            $result->[0]->{averages}->[0]->{ $headings[$i] } = $averages[$i];
            $result->[0]->{maximums}->[0]->{ $headings[$i] } = $maximums[$i];
        }
    }
    my $j = 0;
    foreach my $line (@lines) {
        my @stats = split( /$CLI_separator/, $line );
        for ( my $i = 0; $i < scalar(@headings); $i++ ) {
            $result->[0]->{statistics}->[$j]->{$headings[$i]} = $stats[$i];
        }
        $j++;
    }

    $self->{not_parsed} = \@not_parsed;

    $self->{parsed_output} = $result;

    $Log->exit() if $may_exit;
}

sub _statistics_periodic_without_CLI_separator {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    $Log->debug("In brute force parser for statistics periodic");

    # Parse the output in a bruteforce way
    my @lines = split( /\n/, $self->get_processed_output() );

    # Removing Warning messages from raw output
    my @not_parsed;
    @not_parsed = grep ( /Warning:/,  @lines );
    @lines      = grep ( !/Warning:/, @lines );

    # Get the instance
    my $instance
        = ( $self->get_command() =~ /-instance\s+(\S+)/ ) ? $1 : "cluster";

  # Sometime instance is specified within double quotes(") on the command line
  # but its quotes are removed when it comes in output. Removing quotes for
  # regex to matches.
    $instance =~ s/\"//g;

    # Get the iterations
    my ($iterations) = ( $self->get_command() =~ /-iterations\s+(\d+)/ );

    my %hash;
    my $text = "Number of columns in data is not equal to "
        . "Number of columns in headings. ";
    while ( scalar(@lines) > 0 ) {
        my $line = shift(@lines);

        # Sometime line "rawat-vsim1-01: node.local: 2/20/2012 09:16:52" comes
        # before displaying the headings for statistics, sometime it doesnt
        $line = shift(@lines) if ( $line =~ /$instance:/ );

        my @headings;
        while ( $line !~ /\s*---/ and scalar(@lines) > 0 ) {
            push( @headings, $line );
            $line = shift(@lines);
        }
        push( @headings, $line );

# Headings are displayed in multiple line as shown below. Need to combine all the lines to make proper heading.
# But some of the headings are not shown in all the lines. Headings should be combined correctly in each column.
####################################################################################################################################################################
#  cpu  cpu   cpu    total                     fcache              pkts     pkts    total    total data     data     data cluster  cluster  cluster     disk     disk
#  avg busy total      ops  nfs-ops cifs-ops      ops spin-ops     recv     sent     recv     sent busy     recv     sent    busy     recv     sent     read    write    nbufs    conns    execs
####################################################################################################################################################################
        @headings = $self->_read_headings(@headings);
        $line     = shift(@lines);
        my $j = 0;
        while ( $line !~ /$instance:/ ) {
            $line =~ s/^\s+//g;
            my @data = split( /\s+/, $line );

         # Data can be wrapped around to the next line. Keep reading next line
         # till number of columns in headings and data becomes equal
            while ( scalar(@data) < scalar(@headings) and scalar(@lines) > 0 )
            {
                $line = shift(@lines);
                $line =~ s/^\s+//g;
                my @tmp = split( /\s+/, $line );
                push( @data, @tmp );
            } ## end while ( scalar(@data) < scalar...)

            # Fail if number of columns in data becomes more
            # more than colunms in headings
            if ( scalar(@data) != scalar(@headings) ) {
                $text .= "Row that has different numbers of column is @data";
                NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text            => $text,
                    response_object => $self,
                );
            } ## end if ( scalar(@data) != ...)

            # make the hash
            for ( my $i = 0; $i < scalar(@headings); $i++ ) {
                $hash{statistics}->[$j]
                    ->{ to_key_name( item => $headings[$i] ) } = $data[$i];
            }
            $j++;
            last if ( scalar(@lines) == 0 );
            $line = shift(@lines);
        } ## end while ( $line !~ /$instance:/)

        # Parsing the summary output
        if ( scalar(@lines) > 0 and $iterations > 1 ) {
            while ( $line !~ /Minimums:/ and scalar(@lines) > 0 ) {
                $line = shift(@lines);
            }
            $line = shift(@lines);

            # Get the minimums
            while ( $line !~ /Averages for \d+ samples:/
                and scalar(@lines) > 0 )
            {
                $line =~ s/^\s+//g;

                my @data = split( /\s+/, $line );
                while ( scalar(@data) < scalar(@headings)
                    and scalar(@lines) > 0 )
                {
                    $line = shift(@lines);
                    $line =~ s/^\s+//g;
                    my @tmp = split( /\s+/, $line );
                    push( @data, @tmp );
                } ## end while ( scalar(@data) < scalar...)
                if ( scalar(@data) != scalar(@headings) ) {
                    $text
                        .= "Row that has different numbers of column is @data";
                    NACL::APISet::Exceptions::UnexpectedOutputException
                        ->throw(
                        text            => $text,
                        response_object => $self
                        );
                } ## end if ( scalar(@data) != ...)
                for ( my $i = 0; $i < scalar(@headings); $i++ ) {
                    $hash{minimums}->[0]
                        ->{ to_key_name( item => $headings[$i] ) }
                        = $data[$i];
                }
                $line = shift(@lines);
            } ## end while ( $line !~ /Averages for \d+ samples:/...)
            $line = shift(@lines);

            # Get the averages
            while ( $line !~ /Maximums:/ and scalar(@lines) > 0 ) {
                $line =~ s/^\s+//g;
                my @data = split( /\s+/, $line );
                while ( scalar(@data) < scalar(@headings)
                    and scalar(@lines) > 0 )
                {
                    $line = shift(@lines);
                    $line =~ s/^\s+//g;
                    my @tmp = split( /\s+/, $line );
                    push( @data, @tmp );
                } ## end while ( scalar(@data) < scalar...)
                if ( scalar(@data) != scalar(@headings) ) {
                    $text
                        .= "Row that has different numbers of column is @data";
                    NACL::APISet::Exceptions::UnexpectedOutputException
                        ->throw(
                        text            => $text,
                        response_object => $self
                        );
                } ## end if ( scalar(@data) != ...)
                for ( my $i = 0; $i < scalar(@headings); $i++ ) {
                    $hash{averages}->[0]
                        ->{ to_key_name( item => $headings[$i] ) }
                        = $data[$i];
                }
                $line = shift(@lines);
            } ## end while ( $line !~ /Maximums:/...)
            $line = shift(@lines);
            $line =~ s/^\s+//g;
            my @data = split( /\s+/, $line );

            # Get the maximums
            while ( scalar(@data) < scalar(@headings) and scalar(@lines) > 0 )
            {
                $line = shift(@lines);
                $line =~ s/^\s+//g;
                my @tmp = split( /\s+/, $line );
                push( @data, @tmp );
            } ## end while ( scalar(@data) < scalar...)
            if ( scalar(@data) != scalar(@headings) ) {
                $text .= "Row that has different numbers of column is @data";
                NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text            => $text,
                    response_object => $self
                );
            } ## end if ( scalar(@data) != ...)
            for ( my $i = 0; $i < scalar(@headings); $i++ ) {
                $hash{maximums}->[0]->{ to_key_name( item => $headings[$i] ) }
                    = $data[$i];
            }
        } ## end if ( scalar(@lines) > ...)

    } ## end while ( scalar(@lines) > ...)

    $self->{not_parsed} = \@not_parsed;

    my @result;
    push( @result, \%hash );

    $self->{parsed_output} = \@result;
    $Log->exit() if $may_exit;
} ## end sub _statistics_periodic

# This method is to read headings in the output of command "statistics periodic"
# This method shouldnt be called as parser
sub _read_headings {
    my ( $self, @headings ) = @_;
    $Log->enter() if $may_enter;
    my $last_line = pop(@headings);
    $last_line =~ s/^\s//;
    $last_line =~ s/\r$//;
    my @hyphens = split( /\s+/, $last_line );
    my @complete_headings;
    foreach my $column_heading (@headings) {
        my $i = 0;
        $column_heading =~ s/^\s//;
        foreach my $column (@hyphens) {
            my @column = split( //, $column );

            my $width = scalar(@column);

            # There is a space between the hyphens. We need to take that
            # into account while combining the headings
            my $column1 = "";
            if ( $column_heading ne "" ) {
                if ( length($column_heading) < $width ) {
                    $column1        = $column_heading;
                    $column_heading = "";
                } else {
                    $column_heading =~ s/^(.{$width}(\w+)?)\s?//;
                    $column1 = $1;
                }
            } ## end if ( $column_heading ne...)
            $column1 =~ s/^\s+//;
            $column1 =~ s/\s+$//;
            if ( $column1 ne "" ) {
                $complete_headings[$i] .= $column1 . "_";
            }
            $i++;
        } ## end foreach my $column (@hyphens)
    } ## end foreach my $column_heading ...

    my @return_array;
    foreach my $line (@complete_headings) {
        $line =~ s/_$//;
        push( @return_array, $line );
    }

    return @return_array;
} ## end sub _read_headings

sub _cluster_image_update {
    my ($self) = @_;
    $Log->enter() if $may_enter;

    my $output       = $self->get_processed_output();
    my @lines        = split( /\n/, $output );
    my $header_regex = qr/^Non-Disruptive Check\s+Status\s+Error-Action/;
    my $status_regex = qr/OK|Warning|Error|INFO/i;
    my @result;
    my $start_parsing = 0;
    for ( my $i = 0; $i <= $#lines; $i++ ) {

        if ( $lines[$i] =~ /^(\S+)\s+($status_regex)\s*(.*)/ ) {
            my $tmp = {
                'non_disruptive_check' => $1,
                status                 => $2,
                error_action           => $3
            };
            my $j;
            foreach ( $j = $i + 1; $j <= $#lines; $j++ ) {
                last
                    if ( $lines[$j] =~ /^(\S+)\s+($status_regex)\s*(.*)/
                    or $lines[$j] =~ /Overall Status/ );
                if ( $lines[$j] =~ /^(\S*)\s*(.*)/ ) {
                    $tmp->{non_disruptive_check} .= $1;
                    $tmp->{error_action}         .= $2;
                }
            }
            $i = $j - 2 if ( $j > $i );
            push @result, $tmp;
        }
        if ( $lines[$i] =~ /^(Overall Status)\s+($status_regex)\s*(.*)/ ) {
            push @result,
                {
                'non_disruptive_check' => $1,
                status                 => $2,
                error_action           => $3
                };
        }
    }
    $self->{parsed_output} = \@result;
    $Log->exit() if $may_exit;
}

sub _cluster_ping_cluster {
    my ($self) = @_;
    $Log->enter() if $may_enter;

    my @lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "can_throw"       => 1,
        "response_object" => $self
    );
    my @result;
    my $i     = 0;
    my $j     = 0;
    my $k     = 0;
    my $h     = 0;
    my $count = 0;
    my @ip_array;
    my $ip = 0;

    foreach my $line (@lines) {

        if ( $line =~ /^Ho\w+\s+\w+\s+(.*)/ ) {
            $result[0]->{'host_name'} = $1;
        } elsif ( $line =~ /Local\s+=\s+(.*)/ ) {
            $ip = $1;
            @ip_array = split( /\s+/, $ip );
            @{ $result[0]->{'local_ip'} } = @ip_array;
        } elsif ( $line =~ /Remote\s+=\s+(.*)/ ) {
            $ip = $1;
            @ip_array = split( /\s+/, $ip );
            @{ $result[0]->{'remote_ip'} } = @ip_array;
        } elsif ( $line =~ /(\w+)\s+conn\w+\s+(\w+)\D+(\d+).*/ ) {
            $result[0]->{'ping_status'}->[$h]->{'connectivity'} = $1;
            $result[0]->{'ping_status'}->[$h]->{'status'}       = $2;
            $result[0]->{'ping_status'}->[$h]->{'no_of_paths'}  = $3;
            $h++;
            $count = $3;
        } elsif ( $line =~ /Local\s+(.*)\s+to\s+Remote\s+(.*)/ ) {
            if ($count) {
                $result[0]->{'ping_status'}->[0]->{'failed paths'}->[$i]
                    ->{'local'} = $1;
                $result[0]->{'ping_status'}->[0]->{'failed paths'}->[$i]
                    ->{'remote'} = $2;
                $i++;
                $count--;
            } else {
                $result[0]->{'mtu_info'}->[0]->{'paths'}->[$j]->{'local'}
                    = $1;
                $result[0]->{'mtu_info'}->[0]->{'paths'}->[$j]->{'remote'}
                    = $2;
                $j++;
            }
        } elsif ( $line =~ /Dete\w+\s+(\d+)\s+\w+\D+(\d+)\s+.*/ ) {
            $result[0]->{'mtu_info'}->[0]->{'no_of_bytes'} = $1;
            $result[0]->{'mtu_info'}->[0]->{'no_of_paths'} = $2;
        } elsif ( $line
            =~ /^(\d+)\s+\w+\s+\w+,\s+(\d+)\s+\w+\s+\w+\s+\((\w+)\s+\w+\)/ )
        {
            $result[0]->{'rpc_status'}->[$k]->{'no_of_up_paths'}   = $1;
            $result[0]->{'rpc_status'}->[$k]->{'no_of_down_paths'} = $2;
            $result[0]->{'rpc_status'}->[$k]->{'protocol_check'}   = $3;
            $k++;
        } elsif ( $line =~ /(\w+\s+\w+\s+PMTU\s+\w+\s+\w+\s+\w+\s+\d+.*)/ ) {
            $result[0]->{'mtu_info'}->[0]->{'status'} = $1;
        }
    } ## end foreach my $line (@lines)
    $self->{parsed_output} = \@result;
} ## end sub _cluster_ping_cluster

# Brute-force parsers come here
# These are for commands whose outputs cannot be parsed using the generic
# parser
#Brute force support for _statistics_show_periodic which calls _statistics_periodic
sub _statistics_show_periodic {
    my ($self) = @_;
    _statistics_periodic($self);
}

##############################################################################
# Method:    _set
# Objective: Brute force parser for "set" command.
# Details:   This is the brute force parser for the 'set' command.
#            The generic parser fails for the set command because the 8th field
#            of its output is the value of the CLI separator. This is separated
#            from the previous field and the next field by the CLI separator.
#            This leads to the split of the generic parser having one value
#            less than the number of headings. The brute force parser splits
#            on the separator and inserts the value of the separator into the
#            value for that field.
##############################################################################
sub _set {
    my $self = shift;
    $Log->enter() if $may_enter;
    $Log->debug("In brute-force parser for set");

    my $want_array = [];

    # just skip blank lines
    my @lines = split( /\n/, $self->{processed_output} );
    @lines = grep {/\w/} @lines;

    # If the output has nothing than new lines (no output), it means that
    # set has been invoked in get context
    if ( !scalar(@lines) ) {
        $self->{parsed_output} = [ { status => "option is set" } ];
        $Log->exit() if $may_exit;
        return;
    }

    my $CLI_separator = $self->{CLI_separator};
    my ( $fields, $heading, $values ) = @lines;
    my ( @fields, @heading );
    if ( defined $fields ) {
        @fields = split( $CLI_separator, $fields );
    }
    if ( defined $heading ) {
        @heading = split( $CLI_separator, $heading );
    }
    my @values;
    if ( defined $values ) {
        @values = split( $CLI_separator, $values );
    }

    my ( %output, $index );

    #
    # The 8th field of the output is the CLI separator.
    # This is separated from the previous field and the next field by the
    # CLI separator. Hence, after splitting there will be two consecutive
    # values which are blank. These should be replaced by one value -
    # that of the CLI separator
    #
    splice( @values, 7, 2, $CLI_separator );

    for ( $index = 0; $index < scalar(@heading); $index++ ) {
        $output{ $fields[$index] } = $values[$index];
    }

    push( @$want_array, \%output );
    $self->{parsed_output} = $want_array;
    $Log->exit() if $may_exit;
} ## end sub _set

##############################################################################
# Method:    _volume_explore
# Objective: This method invokes the brute force parser for the
#            "volume explore" command
# Details:   The command "volume explore" takes multiple options.
#            Unfortunately each option produces output in a different format.
#            It's unfeasible to write parsers for each option, and option
#            combination, so while invoking the command the "-dump all"
#            option is added automatically - which results in all output
#            fields being displayed and in a standard key=value format.
##############################################################################
# @name ::volume_explore
##
## @summary Parses the raw output from the "volume explore" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from volume_explore with any of its options as described in the
##    corresponding cdef file 'volume.cdefs'
## @synopsis
##      volume_explore(%args);
## INPUTS TO METHOD CALL 'volume_explore' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-verbose' </td> <td> 'verbose' </td> <td> '0' </td> </tr>
## <tr> <td> '-dump' </td> <td> 'dump' </td> <td> '0' </td> </tr>
## <tr> <td> '-find' </td> <td> 'find' </td> <td> '0' </td> </tr>
## <tr> <td> '-noinomap' </td> <td> 'noinomap' </td> <td> '0' </td> </tr>
## <tr> <td> '-export-file' </td> <td> 'export-file' </td> <td> '0' </td> </tr>
## <tr> <td> '-print' </td> <td> 'print' </td> <td> '0' </td> </tr>
## <tr> <td> '-rawtimes' </td> <td> 'rawtimes' </td> <td> '0' </td> </tr>
## <tr> <td> '-recursive' </td> <td> 'recursive' </td> <td> '0' </td> </tr>
## <tr> <td> '-fill' </td> <td> 'fill' </td> <td> '0' </td> </tr>
## <tr> <td> '-format' </td> <td> 'format' </td> <td> '0' </td> </tr>
## <tr> <td> '-import-file' </td> <td> 'import-file' </td> <td> '0' </td> </tr>
## <tr> <td> '-set' </td> <td> 'set' </td> <td> '0' </td> </tr>
## <tr> <td> '-scope' </td> <td> 'scope' </td> <td> '0' </td> </tr>
## <tr> <td> '-list' </td> <td> 'list' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = volume explore -dump all -format inode -scope aggr1/.
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;dsid=0
##&nbsp;snapid=0
##&nbsp;inum=64
##&nbsp;vvbn=0
##&nbsp;pvbn=4099
##&nbsp;location=aggr1@4099b+256:192
##&nbsp;version=3
##&nbsp;type=2
##&nbsp;subtype=1
##&nbsp;level=1
##&nbsp;umask=0700
##&nbsp;dvmask=0
##&nbsp;tid=0
##&nbsp;flags=0x00000202
##&nbsp;nlink=2
##&nbsp;generation=419581169
##&nbsp;av-gen-num=0
##&nbsp;uid=0
##&nbsp;gid=0
##&nbsp;atime=20-Mar-2012 08:03:47
##&nbsp;mtime=20-Mar-2012 08:03:47
##&nbsp;ctime=20-Mar-2012 08:03:47
##&nbsp;crtime=20-Mar-2012 08:03:47
##&nbsp;size=4096
##&nbsp;blockcount=1
##&nbsp;xinode=0
##&nbsp;sinode=0
##&nbsp;inline=-
##&nbsp;rdev=516096
##&nbsp;junction=516096
##&nbsp;i2p=0#0
##&nbsp;owner=0
##&nbsp;epoch=0
##&nbsp;indircount=0
##&nbsp;state=valid
##&nbsp;buftreeid=65
##&nbsp;mixed_buftree=0
##&nbsp;payload=b64
##&nbsp;name=/
##&nbsp;used=1
##&nbsp;fav=fav
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'blockcount' => '1',
##&nbsp;            'inum' => '64',
##&nbsp;            'dsid' => '0',
##&nbsp;            'junction' => '516096',
##&nbsp;            'state' => 'valid',
##&nbsp;            'subtype' => '1',
##&nbsp;            'i2p' => [
##&nbsp;                       '0#0'
##&nbsp;                     ],
##&nbsp;            'inline' => '-',
##&nbsp;            'epoch' => '0',
##&nbsp;            'rdev' => '516096',
##&nbsp;            'uid' => '0',
##&nbsp;            'location' => 'aggr1@4099b+256:192',
##&nbsp;            'mtime' => '20-Mar-2012 08:03:47',
##&nbsp;            'name' => [
##&nbsp;                        '/'
##&nbsp;                      ],
##&nbsp;            'sinode' => '0',
##&nbsp;            'level' => '1',
##&nbsp;            'used' => '1',
##&nbsp;            'mixed-buftree' => '0',
##&nbsp;            'atime' => '20-Mar-2012 08:03:47',
##&nbsp;            'type' => '2',
##&nbsp;            'gid' => '0',
##&nbsp;            'pvbn' => '4099',
##&nbsp;            'crtime' => '20-Mar-2012 08:03:47',
##&nbsp;            'av-gen-num' => '0',
##&nbsp;            'tid' => '0',
##&nbsp;            'size' => '4096',
##&nbsp;            'nlink' => '2',
##&nbsp;            'indircount' => '0',
##&nbsp;            'vvbn' => '0',
##&nbsp;            'payload' => 'b64',
##&nbsp;            'ctime' => '20-Mar-2012 08:03:47',
##&nbsp;            'owner' => '0',
##&nbsp;            'flags' => '0x00000202',
##&nbsp;            'dvmask' => '0',
##&nbsp;            'version' => '3',
##&nbsp;            'xinode' => '0',
##&nbsp;            'snapid' => '0',
##&nbsp;            'generation' => '419581169',
##&nbsp;            'buftreeid' => '65',
##&nbsp;            'umask' => '0700'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub _volume_explore {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    $Log->debug("In brute force parser for volume explore");

    my $array_hash = [];
    my (%hash);
    my $output = $self->{'processed_output'};
    my @lines = split( /\n/, $output );

    # This part needs to be re-explored. -dump all *generally* gives output
    # in a field=value structure. The original assumption was to throw an
    # exception if output was not in this structure. This was done because
    # the error messages generated by this command do not include the
    # ERROR:|Error: or are in any particular format.
    # However, it appears there are positive cases where the output will not
    # appear in the field=value format.
    # To make sure these positive cases don't fail, we'll just return
    # the raw output when the output is not in the field=value format. The
    # raw output will be in a key called 'raw_output'.
    if ( $output !~ /\w+=.*/ ) {
        if ($output =~ /unable\s+to\s+find.*/) {
            $Log->exit() if $may_exit;
            NACL::APISet::Exceptions::InvalidParamValueException->throw(
                text            => $output,
                response_object => $self
            );
        } elsif ($output =~ /\(error\s+\d+\)/) {
            # volume explore errors appear to contain
            # "(error <num>)", so this generic check should catch
            # errors. Since it's generic, keep it last (after the specific checks)
            $Log->exit() if $may_exit;
            NACL::APISet::Exceptions::CommandFailedException->throw(
                text            => $output,
                response_object => $self,
            );
        } elsif ($output =~ /unable\s+to\s+write\s+inode\s+.*/) {
              $Log->exit() if $may_exit;
              NACL::APISet::Exceptions::CommandFailedException->throw(
                  text            => $output,
                  response_object => $self
              );
        } else {
                $self->{parsed_output}->[0]->{raw_output} = $output;
        }
    }
    my $i   = 0;
    my $len = scalar(@lines);
    while ( $i < $len ) {
        my %hash;
        my @name_val;
        my @i2p_val;
        while ( $i < $len && $lines[ $i++ ] =~ /([\w-]*)=(.*)/ ) {
            my $key   = $1;
            my $value = $2;
            unless ( $key eq 'name' || $key eq 'i2p' ) {
                $key =~ s/_/-/g;
                $hash{$key} = $value;
            } elsif ( $key eq 'name' ) {
                push( @name_val, $value );
            } else {
                push( @i2p_val, $value );
            }
        } ## end while ( $lines[ $i++ ] =~...)
        $hash{name} = \@name_val if @name_val;
        $hash{i2p}  = \@i2p_val  if @i2p_val;
        push( @$array_hash, \%hash ) if %hash;
    } ## end while ( $i < $len )
    $self->{parsed_output} = $array_hash;
    $Log->debug( "Parsed output is " . Dumper($array_hash) );
    $Log->exit() if $may_exit;
} ## end sub _volume_explore

##############################################################################
# Method:    _volume_show
# Objective: This method invokes the brute force parser for the
#            "volume show" command called with -checking option
# Details:   The command "volume show" takes -checking option for which its
#            not obeying the CLI separator and hence a genric parser is
#            required in this case. If "volume show" has been called without
#            -checking option, then it will inturn call generic parser.
##############################################################################
sub _volume_show {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    $Log->debug("In brute force parser for volume show");

    unless ( $self->get_command() =~ /-checking/ ) {
        $Log->debug("calling generic parser");
        $Log->exit() if $may_exit;
        $self->_parse_output();
        return;
    } ## end unless ( $self->get_command...)

    # Parse the output in a bruteforce way
    my @arr = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "first_line"      => 'Server\s+Volume',
        "can_throw"       => 1,
        "response_object" => $self
    );

    my @headings = split( /\s\s+/, shift(@arr) );

    # Skip blank lines if any
    @arr = grep {/\w/} @arr;

   # The assumption here is that headings field is defined and the first
   # argument is virtual server. But in the output the words are split across
   # two lines as:
   #
   # Virtual
   # Server    Volume       Status                         Estimated Time Left
    for ( my $i = 0; $i <= $#headings; $i++ ) {

        # Normalize the hash keys
        $headings[$i] =~ s/\s/_/g;
        $headings[$i] = lc( $headings[$i] );
    } ## end for ( my $i = 0; $i <= ...)

    # set the first heading to virtual server
    $headings[0] = "virtual_server" if ( $headings[0] =~ /[Ss]erver/ );

    my $full_line;

    # start parsing the output
    my @parsed;
    for ( my $i = 0; $i <= $#arr; $i++ ) {
        last if ( $arr[$i] =~ /\d+ entries were displayed/ );
        $full_line = $arr[$i];
        while ( $arr[$i] !~ /\d+:\d+\s*$/ ) {
            $full_line .= " " . $arr[ ++$i ];
        }
        $full_line =~ /(.+?)\s+(.+?)\s+(.+)\s+(\d+:\d+)/;
        my @values = ( $1, $2, $3, $4 );
        my %hash;
        my $i = 0;
        foreach $_ (@headings) {
            $hash{$_} = $values[ $i++ ];
            $hash{$_} =~ s/\s*$//;
            $hash{$_} =~ s/^\s*//;
        }
        push( @parsed, \%hash );
    } ## end for ( my $i = 0; $i <= ...)

    $Log->debug( "Parsed output is:\n" . Dumper( \@parsed ) );
    $Log->exit() if $may_exit;

    $self->{parsed_output} = \@parsed;
} ## end sub _volume_show

##############################################################################
# Method:    _network_ping
# Objective: This method invokes the brute force parser for the
#            "network ping" command
# Details:   This command does not honour the CLI separator. It's output is
#            identical to the output structure for the 7Mode "ping"
#            command, so we re-use that parser here.
##############################################################################
sub _network_ping {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    $Log->debug("In brute force parser for 'network ping'");

    require NACL::APISet::Node::CLI::7Mode::default::ping;

    # Make an entry for "actual_command" since the 7Mode parser
    # requires it (for CMode, the "actual command" is the same as what's
    # stored in the "command" field)
    $self->{actual_command} = $self->get_command();

    $self->{parsed_output}
        = NACL::APISet::Node::CLI::7Mode::default::ping::ping(
        response_object => $self );

    $Log->debug( "Parsed output is:\n" . Dumper( $self->{parsed_output} ) );
    $Log->exit() if $may_exit;
} ## end sub _network_ping

######################################################################################
# Method : _statistics_start
# Objective : This method parses the response object and populates sample_id
#             in the parsed_output of command "statistics start"
######################################################################################
sub _statistics_start {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    my $processed_output = $self->get_processed_output();
    my $sample_id = $1 if ( $processed_output =~ /.*?:\s+(.*)/ );
    $self->{parsed_output}->[0]->{sample_id} = $sample_id;
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _volume_64bit_upgrade_start
# Objective: This method reuses the 7Mode parser for "vol 64bit-upgrade start"
#            since the output of "volume 64bit-upgrade start" is almost
#            identical.
##############################################################################
sub _volume_64bit_upgrade_start {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    $Log->debug("Re-using the 7Mode parser for 'vol 64bit-upgrade start'");
    require NACL::APISet::Node::CLI::7Mode::default::vol;
    $self->{parsed_output}
        = NACL::APISet::Node::CLI::7Mode::default::vol::vol_64bit_upgrade_start(
        response_object => $self );
    $Log->debug( "Parsed output is:\n" . Dumper( $self->{parsed_output} ) );
    $Log->exit() if $may_exit;
} ## end sub _volume_64bit_upgrade_start

##############################################################################
# Method:    _storage_aggregate_64bit_upgrade_start
# Objective: This method reuses the 7Mode parser for "storage aggregate
#            64bit-upgrade start" since the output of "storage aggreate
#            64bit-upgrade start" is almost identical.
##############################################################################
sub _storage_aggregate_64bit_upgrade_start {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    $Log->debug("Re-using the 7Mode parser for 'aggr 64bit-upgrade start'");
    require NACL::APISet::Node::CLI::7Mode::default::aggr;
    $self->{parsed_output}
        = NACL::APISet::Node::CLI::7Mode::default::aggr::aggr_64bit_upgrade_start(
        response_object => $self );
    $Log->debug( "Parsed output is:\n" . Dumper( $self->{parsed_output} ) );
    $Log->exit() if $may_exit;
} ## end sub _storage_aggregate_64bit_upgrade_start

##############################################################################
# Method:    _storage_aggregate_add_disks
# Objective: This method reuses the 7Mode parser for "storage aggregate
#            add-disks -64bit-upgrade" since the output of "storage aggreate
#            add-disks -64bit-upgrade " is almost identical.If its not run
#            with -64bit-upgrade option, then use the generic parser.
##############################################################################
sub _storage_aggregate_add_disks {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    if ( $self->get_command() =~ /64bit-upgrade/ ) {
        $Log->debug(
            "Re-using the 7Mode parser for 'aggr 64bit-upgrade start'");
        require NACL::APISet::Node::CLI::7Mode::default::aggr;
        $self->{parsed_output}
            = NACL::APISet::Node::CLI::7Mode::default::aggr::aggr_64bit_upgrade_start(
            response_object => $self );
        $Log->debug(
            "Parsed output is:\n" . Dumper( $self->{parsed_output} ) );
    } else {
        $Log->debug("Using the generic parser");
        $self->_parse_output();
    }
    $Log->exit() if $may_exit;
} ## end sub _storage_aggregate_add_disks

##############################################################################
# Method:    _volume_quota_report
# Objective: This method reuses the CMode generic parser to get the parsed output
#            however corrects the wrongly put array ref values for 'quota-report'
#            and 'quota-report-id' keys.
##############################################################################
sub _volume_quota_report {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    $Log->debug("Using the generic parser to get the parsed output");
    $self->_parse_output();
    my $parsed_output = $self->{parsed_output};
    $Log->debug("Converting wrong arrayrefs to strings");
    foreach my $quota (@$parsed_output) {
        if ( ref $quota->{'quota-target'} eq "ARRAY" ) {
            $quota->{'quota-target'}
                = join( ',', @{ $quota->{'quota-target'} } );
        }
        if ( ref $quota->{'quota-target-id'} eq "ARRAY" ) {
            $quota->{'quota-target-id'}
                = join( ',', @{ $quota->{'quota-target-id'} } );
        }
    } ## end foreach my $quota (@$parsed_output)
    $Log->exit() if $may_exit;
} ## end sub _volume_quota_report


##############################################################################
# Method:    _system_node_image_show_update_progress
# Objective: Implements a custom parser for the
#            "system node image show-update-progress" command. Throws a
#            CommandFailed exception in the case of there not being any
#            update operations to have ever occured on that node.
##############################################################################

#
# Here are the various output structures I've noticed:
# 1)
# \r95% downloaded                                                                 \cH\cH\cH\cH\cH\cH\cH\cH\cH\cH\cH\cH (...) \r\n
# \r\n
#
# 2)
# There is no update/install in progress
#
# 3)
# \rThere is no update/install in progress                                         \cH\cH\cH\cH\cH\cH\cH\cH\cH\cH\cH\cH (...) \r\n
# Status of most recent operation:\r\n
# \tRun Status:\tExited\r\n
# \tExit Status:\tFailed\r\n
# \tPhase:\t\tValidate Script\r\n
# \tExit Message:\tInstall Validation Failed. ERROR: Attempt to install an invalid kernel binary on target hardware:<snipped>\r\n
# \r\n
# \r\n
#
# The keys have been named to be consistent with the ZAPI fields. Here are
# the equivalent ZAPI parsed output structures:
#
# 1)
# $VAR1 = [
#          {
#            'run-status' => 'Working',
#            'phase' => 'Download',
#            'last-message' => '93% downloaded'
#          }
#        ];
#
# 2)
# $VAR1 = [
#          {
#            'last-message' => 'There is no update/install in progress'
#          }
#       ];
#
# 3)
# $VAR1 = [
#          {
#           'exit-status' => 'Failed',
#           'run-status' => 'Exited',
#           'phase' => 'Validate Script',
#           'exit-message' => 'Install Validation Failed. ERROR: Attempt to install an invalid kernel binary on target hardware: <snipped>
#           'last-message' => 'There is no update/install in progress'
#          }
#        ];

sub _system_node_image_show_update_progress {
    $Log->enter() if $may_enter;
    my ($self)     = @_;
    my $raw_output = $self->get_raw_output();
    my $only_row   = {};

    if ( $raw_output =~ /(\d+%\s+downloaded)/ ) {
        $only_row->{'last-message'} = $1;
        $only_row->{'phase'}        = 'Download';
        $only_row->{'run-status'}   = 'Working';
    } elsif ( $raw_output =~ m!no update/install in progress! ) {
        if ( $raw_output =~ /Status of most recent operation/ ) {

            # Operation has already completed, parse the details of the
            # last completed operation
            $only_row->{'last-message'}
                = 'There is no update/install in progress';

            $raw_output =~ /Run Status:\s+(\w+)/;
            $only_row->{'run-status'} = $1;

            $raw_output =~ /Exit Status:\s+(.*)\r?\n/;
            $only_row->{'exit-status'} = $1;

            $raw_output =~ /Phase:\s+(.*)\r?\n/;
            $only_row->{'phase'} = $1;

            $raw_output =~ /Exit Message:\s+(.*)\r?\n/;
            $only_row->{'exit-message'} = $1;
        } else {

            # No update ever happened, exception out
            $Log->exit() if $may_exit;
            NACL::APISet::Exceptions::CommandFailedException->throw(
                text            => 'There is no update/install in progress',
                response_object => $self
            );
        } ## end else [ if ( $raw_output =~ /Status of most recent operation/)]
    } ## end elsif ( $raw_output =~ m!no update/install in progress!)

    $self->{parsed_output} = [$only_row];
    $Log->debug( "Parsed output is:\n" . Dumper( $self->{parsed_output} ) );
    $Log->exit() if $may_exit;
} ## end sub _system_node_image_show_update_progress

## subroutine to check for the 'Failures Found'
## string in the system image update command

sub _system_node_image_update {
    $Log->enter() if $may_enter;
    my ($self)     = @_;
    my $raw_output = $self->get_raw_output();

    if ( $raw_output =~ /FAILURES FOUND(.*)/i ) {
        $Log->exit() if $may_exit;
            NACL::APISet::Exceptions::CommandFailedException->throw(
                text            => $raw_output,
                response_object => $self
        );
    }
    $Log->debug( sub {"Raw output is:\n" . Dumper( $raw_output ) });

    $Log->exit() if $may_exit;
} ## end sub _system_node_image_update

##############################################################################
# Method:    _cluster_peer_create
# Objective: This method parses the output of cluster peer create command
# Sample Output:
# C2_sti71-vsim-ucs167g_cluster::*> cluster peer create -generate-passphrase -ipspace Default
#
# Notice:
#         Passphrase: 710V6sfVLghPqBM1H+vqxS4z
#         Expiration Time: 7/27/2017 03:17:03 America/New_York
#         Initial Allowed Vserver Peers: -
#         Intercluster LIF IP: 172.21.94.76
#         Peer Cluster Name: Clus_xS4z (temporary generated)
#
#         Warning: make a note of the passphrase - it cannot be displayed again.
#
# issh-cluster::storage aggregate*>
##############################################################################
sub _cluster_peer_create {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my @result = ();

    if ( $self->get_command() =~ /\-generate\-passphrase/ ) {
	my @array_lines = get_lines(
            "raw_output"      => $self->get_processed_output(),
            "response_object" => $self
	);

	my ($key, $val, %result_hash);
	foreach my $row (@array_lines) {
	    if ($row =~ /(.+?):(.+)/) {
		$key = $1;
		$val = $2;
		$val =~ s/^\s*//;
		$result_hash{to_key_name( item => $key )} = $val;
	    }
	}

	push (@result, \%result_hash);
    }

    $Log->exit() if $may_exit;
    $self->{parsed_output} = \@result;
} ## end sub _cluster_peer_create

##############################################################################
# Method:    _storage_aggregate_verify
# Objective: This method parses the output of storage aggregate verify command
# Sample Output:
# issh-cluster::storage aggregate*> verify -aggregate m1 -action status
#
# Aggregate:m1, Is Suspended:false, Percentage Completed:3%
#
# issh-cluster::storage aggregate*>
##############################################################################
sub _storage_aggregate_verify {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    my @parsed_output = ();
    if ( $self->get_command() =~ /-action\s+status/ ) {
        $Log->debug( "Parsing the output of 'storage aggregate verify "
                . "-action status' command" );
        my $output = $self->get_processed_output();

        # just skip blank lines
        my @lines = split( /\n/, $output );
        @lines = grep {/\w/} @lines;
        foreach my $line (@lines) {
            if ( $line
                =~ /Aggregate:(.+)\s*,\s*Is Suspended:(.+)\s*,\s*Percentage Completed:(.+)\%\s*/i
                )
            {
                my $hash = {
                    aggregate            => $1,
                    is_suspended         => $2,
                    percentage_completed => $3,
                };
                push( @parsed_output, $hash );
            } elsif ( $line =~ /Aggregate:(.+)\s*,\s*Verify is not running/i )
            {
                my $hash = {
                    aggregate             => $1,
                    is_verify_not_running => 1
                };
                push( @parsed_output, $hash );
            } else {
                $Log->exit() if $may_exit;
                NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text            => "Unexpected output: $line",
                    response_object => $self
                ) if ( $line !~ /entries were acted on/ );
            } ## end else [ if ( $line =~ ...)]
        } ## end foreach my $line (@lines)
    } ## end if ( $self->get_command...)

    $self->{parsed_output} = \@parsed_output;
    $Log->debug( "Parsed output is:\n" . Dumper( $self->{parsed_output} ) );
    $Log->exit() if $may_exit;
} ## end sub _storage_aggregate_verify

##############################################################################
# Method:    _storage_aggregate_wafliron_review
# Objective: This method parses the output of
#            storage aggregate wafliron review command
# Sample Output:
# issh-cluster::storage aggregate*>wafliron review -aggregate testaggr
# testvol_third: Adjusting blocks used from 5678 to 65694
# testvol_second: Adjusting blocks used from 5678 to 65721
# testvol_second: Total blocks used corrected.
# testvol_third: Total blocks used corrected.
#
#
#
#
# issh-cluster::storage aggregate*>wafliron review -agrgegate testaggr1
# No wafliron changes for testaggr1
# issh-cluster::storage aggregate*>
##############################################################################

sub _storage_aggregate_wafliron_review {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    my @parsed_output = ();
    $Log->debug( "Parsing the output of 'storage aggregate wafliron review' "
            . "command" );
    my $output = $self->get_processed_output();

    # just skip blank lines
    my @lines = split( /\n/, $output );
    @lines = grep {/\w/} @lines;
    @lines = sort (@lines);
    my $line;
    my $i = -1;
    while ( $line = shift(@lines) ) {
        if ( $line =~ /:/ ) {
            my ( $temp_vol, $temp_change ) = split( /\s*:\s*/, $line );
            $i++
                if ( ( $i < 0 )
                || ( defined $parsed_output[$i]->{'volume'} && $parsed_output[$i]->{'volume'} ne $temp_vol ) );
            push( @{ $parsed_output[$i]->{'change_list'} }, $temp_change );
            $parsed_output[$i]->{'volume'} = $temp_vol
                if ( defined $parsed_output[$i]->{'volume'} && $parsed_output[$i]->{'volume'} ne $temp_vol );
        } ## end if ( $line =~ /:/ )
    } ## end while ( $line = shift(@lines...))
    $self->{parsed_output} = \@parsed_output;
    $Log->debug( "Parsed output is:\n" . Dumper( $self->{parsed_output} ) );
    $Log->exit() if $may_exit;
} ## end sub _storage_aggregate_wafliron_review

##############################################################################
# Method:    _vserver_services_name_service_getxxbyyy_gethostbyname
# Objective: This method invokes the brute force parser for the
#            "vserver services name_service getxxbyyy gethostbyname" command
# Details:   This command does not honour the CLI separator. It's output is
#            identical to the output structure for the FreeBSD "getXXbyYY"
#            command, so we re-use that parser here.
##############################################################################
sub _vserver_services_name_service_getxxbyyy_gethostbyname {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_getxxbyyy();
    $Log->exit() if $may_exit;

} ## end sub _vserver_services_name_service_getxxbyyy_gethostbyname

##############################################################################
# Method:    _vserver_services_name_service_getxxbyyy_gethostbyaddr
# Objective: This method invokes the brute force parser for the
#            "vserver services name_service getxxbyyy gethostbyaddr" command
# Details:   This command does not honour the CLI separator. It's output is
#            identical to the output structure for the FreeBSD "getXXbyYY"
#            command, so we re-use that parser here.
##############################################################################
sub _vserver_services_name_service_getxxbyyy_gethostbyaddr {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_getxxbyyy();
    $Log->exit() if $may_exit;

} ## end sub _vserver_services_name_service_getxxbyyy_gethostbyaddr

##############################################################################
# Method:    _vserver_services_name_service_getxxbyyy_getnameinfo
# Objective: This method invokes the brute force parser for the
#            "vserver services name_service getxxbyyy getnameinfo" command
# Details:   This command does not honour the CLI separator. It's output is
#            identical to the output structure for the FreeBSD "getXXbyYY"
#            command, so we re-use that parser here.
##############################################################################
sub _vserver_services_name_service_getxxbyyy_getnameinfo {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_getxxbyyy();
    $Log->exit() if $may_exit;
} ## end sub _vserver_services_name_service_getxxbyyy_getnameinfo

##############################################################################
# Method:    _vserver_services_name_service_getxxbyyy_getaddrinfo
# Objective: This method invokes the brute force parser for the
#            "vserver services name_service getxxbyyy getaddrinfo" command
# Details:   This command does not honour the CLI separator. It's output is
#            identical to the output structure for the FreeBSD "getXXbyYY"
#            command, so we re-use that parser here.
##############################################################################
sub _vserver_services_name_service_getxxbyyy_getaddrinfo {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_getxxbyyy();
    $Log->exit() if $may_exit;
} ## end sub _vserver_services_name_service_getxxbyyy_getaddrinfo

##############################################################################
# Method:    _vserver_services_name_service_getxxbyyy_getpwbyname
# Objective: This method invokes the brute force parser for the
#            "vserver services name_service getxxbyyy getpwbyname" command
# Details:   This command does not honour the CLI separator. It's output is
#            identical to the output structure for the FreeBSD "getXXbyYY"
#            command, so we re-use that parser here.
##############################################################################

sub _vserver_services_name_service_getxxbyyy_getpwbyname {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_getxxbyyy();
    $Log->exit() if $may_exit;
} ## end sub _vserver_services_name_service_getxxbyyy_getpwbyname

##############################################################################
# Method:    _vserver_services_name_service_getxxbyyy_getgrbyname
# Objective: This method invokes the brute force parser for the
#            "vserver services name_service getxxbyyy getgrbyname" command
# Details:   This command does not honour the CLI separator. It's output is
#            identical to the output structure for the FreeBSD "getXXbyYY"
#            command, so we re-use that parser here.
##############################################################################

sub _vserver_services_name_service_getxxbyyy_getgrbyname {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_getxxbyyy();
    $Log->exit() if $may_exit;
} ## end sub _vserver_services_name_service_getxxbyyy_getgrbyname

##############################################################################
# Method:    _vserver_services_name_service_getxxbyyy_getpwbyuid
# Objective: This method invokes the brute force parser for the
#            "vserver services name_service getxxbyyy getpwbyuid" command
# Details:   This command does not honour the CLI separator. It's output is
#            identical to the output structure for the FreeBSD "getXXbyYY"
#            command, so we re-use that parser here.
##############################################################################

sub _vserver_services_name_service_getxxbyyy_getpwbyuid {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_getxxbyyy();
    $Log->exit() if $may_exit;
} ## end sub _vserver_services_name_service_getxxbyyy_getpwbyuid

##############################################################################
# Method:    _vserver_services_name_service_getxxbyyy_netgrp
# Objective: This method invokes the brute force parser for the
#            "vserver services name_service getxxbyyy netgrp" command
# Details:   This command does not honour the CLI separator. It's output is
#            identical to the output structure for the FreeBSD "getXXbyYY"
#            command, so we re-use that parser here.
##############################################################################

sub _vserver_services_name_service_getxxbyyy_netgrp {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_getxxbyyy();
    $Log->exit() if $may_exit;
} ## end sub _vserver_services_name_service_getxxbyyy_netgrp

##############################################################################
# Method:    _vserver_services_name_service_getxxbyyy_getgrbygid
# Objective: This method invokes the brute force parser for the
#            "vserver services name_service getxxbyyy getgrbygid" command
# Details:   This command does not honour the CLI separator. It's output is
#            identical to the output structure for the FreeBSD "getXXbyYY"
#            command, so we re-use that parser here.
##############################################################################

sub _vserver_services_name_service_getxxbyyy_getgrbygid {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_getxxbyyy();
    $Log->exit() if $may_exit;
} ## end sub _vserver_services_name_service_getxxbyyy_getgrbygid

#############################################################################
# Method:    _vserver_services_name_service_getxxbyyy_getgrlist
# Objective: This method invokes the brute force parser for the
#            "vserver services name_service getxxbyyy getgrlist" command
# Details:   This command does not honour the CLI separator. It's output is
#            identical to the output structure for the FreeBSD "getXXbyYY"
#            command, so we re-use that parser here.
##############################################################################
sub _vserver_services_name_service_getxxbyyy_getgrlist {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_getxxbyyy();
    $Log->exit() if $may_exit;
}  ## end _vserver_services_name_service_getxxbyyy_getgrlist

##############################################################################
# Method:    _vserver_services_name_service_getxxbyyy_getaddrinfo
# Objective: This method invokes the brute force parser for the
#            "vserver services name_service getxxbyyy getaddrinfo" command
# Details:   This command does not honour the CLI separator. It's output is
#            identical to the output structure for the FreeBSD "getXXbyYY"
#            command, so we re-use that parser here.
##############################################################################
sub _getxxbyyy {
    my ($self) = @_;
    $Log->enter() if $may_enter;

    require NACL::APISet::Host::CLI::Freebsd::default::getXXbyYY;

    # Make an entry for "actual_command" since the 7Mode parser
    # requires it (for CMode, the "actual command" is the same as what's
    # stored in the "command" field)
    $self->{actual_command} = $self->get_command();

    $self->{parsed_output}
        = NACL::APISet::Host::CLI::Freebsd::default::getXXbyYY::getXXbyYY(
        response_object => $self );

    $Log->debug(
        sub { "Parsed output is:\n" . Dumper( $self->{parsed_output} ) } );
    $Log->exit() if $may_exit;
} ## end sub _getxxbyyy
##############################################################################
# Method:  network_interface_failover_show
# Purpose: Return port and node in the parsed output (if present)
# Input:   None.
# Output:  None.
# Notes:   Refer to POD below.
##############################################################################

sub _network_interface_failover_show {
    my ($self) = @_;
    my $raw_output = $self->get_processed_output();
    my ( @result_array, @values );
    my $i = 0;
    my $j;
    my $k;
    my $key;
    my $vserver;
    my @lines = get_lines(
        raw_output => $raw_output,
        first_line => "--------"
    );
    shift(@lines);    #remove the first failover target

    foreach my $line (@lines) {
        if ( $line =~ /^([\d\w\-]+)$/ ) {
            $j       = 0;
            $vserver = $1;
        } elsif( $line =~ /^(?:Failover Targets:\s+)*((\S+)\:(\S+).*)/ ){
            $key = to_key_name( item => "failover-targets" );
            my @arr = split(/,/ , $1);
            foreach my $var (@arr) {
                my @split = split(/:/,$var);
                $split[0] =~ s/^\s+//;
                $result_array[$k]->{$key}->[$j]->{ to_key_name( item => "node" ) } = $split[0];
                $result_array[$k]->{$key}->[$j]->{ to_key_name( item => "port" ) } = $split[1];
                $j++;
            }
        } elsif ( $line =~ /^(\S+)\s+(\S+)\:(\S+)\s+(\S+)(.*)?/ ) {
            $result_array[$i]->{ to_key_name( item => "logical-interface" ) }
                = $1;
            $result_array[$i]->{ to_key_name( item => "home-node" ) } = $2;
            $result_array[$i]->{ to_key_name( item => "home-port" ) } = $3;
            $result_array[$i]
                ->{ to_key_name( item => "failover-group-usage" ) } = $4;
            $result_array[$i]->{ to_key_name( item => "vserver" ) }
                = $vserver;
            my $field = trim_string( string => $5 );
            $result_array[$i]->{ to_key_name( item => "failover-group" ) }
                = $field;
            $k = $i;
            $i++;
            $j = 0;
        }

    } ## end elsif ( $line =~ /^(\S+)\s+(\S+)\:(\S+)\s+(\S+)(.*)?//)
    ## end foreach my $line (@lines)
    $self->{parsed_output} = \@result_array;
    $Log->exit;
} ## end sub _network_interface_failover_show

# Burt845991, 'network interface failover show' command is
# being removed in FS.0 in favor of 'network interface show -failover' command.
# As part of this implementation, we have written a parser for 'network interface
# show ' which will internally call the existing parser for 'network
# interface failover show' when '-failover' is specified.

sub _network_interface_show {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    if ($self->{command} =~ /\s-failover(?:\s*|\s+.*)$/i) {
        $self->_network_interface_failover_show();
    } else {
        $self->_parse_output();
    }
    $Log->exit() if $may_exit;
}


##############################################################################
# Method:    _volume_check_start
# Objective: This method invokes the brute force parser for the
#            "volueme check" command
##############################################################################
sub _volume_check_start {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    my @lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "can_throw"       => 1,
        "response_object" => $self
    );
    my @result;
    my $index = 0;
    my ( $i, $data_filepath, $namespace_filepath );
    if ( $lines[1] =~ /^\[Job.*/ ) {
        for ( $i = 1; $i <= $#lines; $i++ ) {
            if ( $lines[$i] =~ /\[Job\s+(\d+)].*volume\s"(.*)".*"(.*)"/ ) {
                $result[$index]->{job_id}  = $1;
                $result[$index]->{volume}  = $2;
                $result[$index]->{vserver} = $3;
                $index++;
            } ## end if ( $lines[$i] =~ ...)
        } ## end for ( $i = 1; $i <= $#lines...)
    } elsif ( $lines[0] =~ /^Namespace.*/ ) {
        for ( $i = 2; $i <= $#lines; $i++ ) {
            if ( $lines[$i] =~ /(([-]+)\s)+\2/ ) {
                $i++;
                if ( $lines[$i] =~ /(\w+)/ ) {
                    $result[0]->{'namespace_filehandle'} = $1;
                }
                $i++;
                if ( $lines[$i] =~ /(\w+)/ ) {
                    $result[0]->{'data_filehandle'} = $1;
                }
            } elsif ( $lines[$i] =~ /(\S+)\s+(\S+)/ ) {
                $namespace_filepath .= $1;
                $data_filepath = $2;
                while ( $i <= $#lines ) {
                    $i++;
                    $lines[$i] =~ /(\S+)/;
                    $data_filepath .= $1;
                }
            } elsif ( $lines[$i] =~ /(\S+)/ ) {
                $namespace_filepath .= $1;
                $i++;
                if ( $lines[$i] =~ /(\/\.\S+)/ ) {
                    $data_filepath .= $1;
                    while ( $i <= $#lines ) {
                        $i++;
                        $lines[$i] =~ /(\S+)/;
                        $data_filepath .= $1;
                    }
                } else {
                    $i--;
                }
            } ## end elsif ( $lines[$i] =~ /(\S+)/)
        } ## end for ( $i = 2; $i <= $#lines...)
        $result[0]->{'namespace_filepath'} = $namespace_filepath;
        $result[0]->{'data_filepath'}      = $data_filepath;
    } ## end elsif ( $lines[0] =~ /^Namespace.*/)
    $self->{parsed_output} = \@result;
    $Log->exit;
} ## end sub _volume_check_start

sub _security_certificate_show {
    my $self = shift;
    $Log->enter() if $may_enter;
    $self->_parse_output( 'parse_line_with_no_cli_separator' => 0 );
    $Log->exit() if $may_exit;
} ## end sub _security_certificate_show

sub _regex_for_each_field {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;
    return qr/[^\w]-([\w\.-]+)/;
}

##############################################################################
# Method:    generate_ds
# Objective: This method is called by cli_command_helper
#            For more details refer to the POD of cli_command_helper in
#            APISet/Node/CLI/CMode.pm
##############################################################################

sub generate_ds {
    my $self = shift;
    $Log->enter() if $may_enter;
    my %opts = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { fields => { type => HASHREF }, },
    );
    my $fields = $opts{fields};
    my $output = $self->get_processed_output();
    my ( $cmdsRef, $cmd );
    my @lines      = split( /\n/, $output );
    my $first_line = $lines[0];
    my $is_CS      = 0;

    if ( $output =~ /command\sis\scomplete/ ) {
        $cmdsRef = {
            fields       => $fields,
            is_singleton => 1
        };
        $Log->exit() if $may_exit;
        return $cmdsRef;
    } ## end if ( $output =~ /command\sis\scomplete/)

    if ( $first_line =~ /\s-fields\s/ ) {
        $is_CS = 1;
        shift @lines;
    } else {
        $is_CS = 0;
    }

    my ( $field, $description, $line, $nextline, $possible_values );
    for ( my $i = 0; $i < @lines; $i++ ) {
        $line = $lines[$i];
        $line =~ s/\r$//;
        $line =~ s/\(default:[^)]+\)//;

        if ( $line =~ /[^\w]-([\w\.-]+)/ || $line =~ /^-([\w\.-]+)/ ) {
            $field = $1;
            my $is_array = 0;
            $is_array = 1 if ( $line =~ /,\s\.\.\./ );

            $fields->{$field}->{field}    = $field;
            $fields->{$field}->{is_array} = $is_array;
        } ## end if ( $line =~ /[^\w]-([\w\.-]+)/)

        #Anything after ']' in the current line is assumed to be
        #description
        if ( $line =~ /[^\w]-[\w\.-]+.*\]\s*(.*)$/ ) {

            $description = $1;
            $description =~ /\}(.*$)/;
            $description = $1;

            $description =~ /\>(.*$)/;
            $description = $1;
            $description = trim_string( string => $description );

#Description can span multiple lines or description can start from the next line
#If next line is not an option , then we assume it is  description
            if ( $i < @lines ) {
                $i++;
                $nextline = $lines[$i];
            }
            if ( defined $nextline && $nextline !~ /[^\w]-([\w\.-]+)/ ) {
                $description .= " $nextline";
            } else {
                $i--;
            }
            $description = trim_string( string => $description );
            $description =~ s/^,\s\.\.\.\s*//g;
            my @valid_values;
            #Extracting the possible values for an option
            #Anything with { } are the possible values
            if ( $line =~ /\{(.*)\}/ ) {
                $possible_values = $1;
                $possible_values =~ s/\|/,/g;
                $description .= ", possible value(s) are $possible_values";
                if ($possible_values =~ /.*\[(.*)\]$/) {
                   $possible_values =~ s/.*\[(.*)\]/$1/g;
                }
                @valid_values = split(/\s*,\s*/, $possible_values);
            }
            my @words = split( /\s+/, $description );

            #Description can be too long , splitting the string of size 70
            #and storing it in an arrayref
            my @description
                = NACL::Tools::CGT::CGTUtils::_determine_split_list(@words);
            $fields->{$field}->{description} = \@description;
		    $fields->{$field}->{valid_values} = \@valid_values if (scalar @valid_values);
        } ## end if ( $line =~ /[^\w]-[\w\.-]+.*\]\s*(.*)$/[)
    } ## end for ( my $i = 0; $i < @lines...)
    $cmdsRef = { fields => $fields, is_CS => $is_CS };
    $Log->exit() if $may_exit;
    return $cmdsRef;
} ## end sub generate_ds

##############################################################################
# Method:    _vserver_security_file_directory_show
# Objective: This method invokes the brute force parser for the
#            "vserver security file directory show" command
##############################################################################

sub _vserver_security_file_directory_show {
    my $self = shift;
    $Log->enter() if $may_enter;
    $self->_parse_output( 'parse_line_with_no_cli_separator' => 0 );
    $Log->exit() if $may_exit;
} ## end sub _vserver_security_file_directory_show

##############################################################################
# Method:    _repository_volume_snapshot_get_snapshot_UUID
# Objective: This method invokes the brute force parser for the
#            "repository volume snapshot get-snapshot-UUID" command
##############################################################################

sub _repository_volume_snapshot_get_snapshot_UUID {
    $Log->enter() if $may_enter;
    my $self          = $_[0];
    my $raw_output    = $self->get_processed_output();
    my $parsed_output = [];
    if ( $raw_output =~ /Snapshot UUID is '(.*)'/ ) {
        $parsed_output->[0]{snapshot_uuid} = $1;
    }
    $self->set_parsed_output( parsed_output => $parsed_output );
    $Log->exit() if $may_exit;
} ## end sub _repository_volume_snapshot_get_snapshot_UUID

##############################################################################
# Method:    _diag_secd_configuration_show_fields
# Objective: This method invokes the brute force parser for the
#            "diag secd configuration show fields" command
##############################################################################

sub _diag_secd_configuration_show_fields {
    my $self = shift;
    $Log->enter() if $may_enter;
    my @lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "can_throw"       => 1,
        "response_object" => $self
    );
    foreach (@lines) {
        $_ =~ s/[X];[X]/ /g;
    }
    my $lin = "";
    my @result_array;
    foreach my $line (@lines) {
        my %hash;
        if ( $line =~ /(\w+):\s+(\d+)/ ) {
            if ( $1 eq "NumKeys" ) {
                $result_array[0]->{'num_keys'} = $2;
            } else {
                $result_array[0]->{'num_fields'} = $2;
            }
        } elsif ( $line =~ /\w+:$/ ) {
            next;
        } else {
            $lin = $lin . "\n" . $line;
        }
    }
    $result_array[0]->{'field_names'} = $lin;
    $self->{parsed_output} = \@result_array;
    $Log->exit;
}

##############################################################################
# Method:    _system_capability_can_boot_shim
# Objective: This method invokes the brute force parser for the
#            "system capability can-boot-shim" command
##############################################################################
# @name :: _system_capability_can_boot_shim
##
## @summary Parses the raw output from the "system capability can-boot-shim" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_capability_can_boot_shim with any of its options as described in the
##    corresponding cdef file 'system_capability.cdefs'
## @synopsis
##      system_capability_can_boot_shim(%args);
## INPUTS TO METHOD CALL 'system_capability_can_boot_shim' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = system capability can-boot-shim.
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Notice: can_boot_shim_imp: Yes, can boot.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;               'can_boot' => 'yes'
##&nbsp;            }
##&nbsp;        ];
##&nbsp;

sub _system_capability_can_boot_shim {
    $Log->enter() if $may_enter;
    my $self       = $_[0];
    my $raw_output = $self->get_processed_output();
    if ( $raw_output =~ /Yes, can boot/ ) {
        $self->{parsed_output}->[0] = { 'can_boot' => 'yes' };
    }
    $Log->exit() if $may_exit;
} ## end sub _system_capability_can_boot_shim

##############################################################################
# Method:    _system_capability_is_enabled_during_downgrade_shim
# Objective: This method invokes the brute force parser for the
#            "system capability is-enabled-during-downgrade-shim" command
##############################################################################
# @name :: _system_capability_is_enabled_during_downgrade_shim
##
## @summary Parses the raw output from the "system capability is-enable-during-downgrade-shim" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_capability_is_enabled_during_downgrade_shim with any of its options as described in the
##    corresponding cdef file 'system_capability.cdefs'
## @synopsis
##      system_capability_is_enabled_during_downgrade_shim(%args);
## INPUTS TO METHOD CALL 'system_capability_is_enabled_during_downgrade_shim' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'capability' </td> <td> 'capability' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = system capability is-enable-during-downgrade-shim -capability test_cap
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Notice: is_enabled_during_downgrade_shim_imp: test_cap=0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 =  [
##&nbsp;            {
##&nbsp;                'is_enabled' => '0',
##&nbsp;                 'capability' => 'test_cap'
##&nbsp;            }
##&nbsp;        ];
##&nbsp;

sub _system_capability_is_enabled_during_downgrade_shim {
    $Log->enter() if $may_enter;
    my $self       = $_[0];
    my $raw_output = $self->get_processed_output();
    if ( $raw_output
        =~ /Notice: is_enabled_during_downgrade_shim_imp:\s(.*)=(\d)/ )
    {
        $self->{parsed_output}->[0]->{'capability'} = $1;
        $self->{parsed_output}->[0]->{'is_enabled'} = $2;
    }
    $Log->exit() if $may_exit;
} ## end sub _system_capability_is_enabled_during_downgrade_shim

##############################################################################
# Method:    _system_capability_is_enabled
# Objective: This method invokes the brute force parser for the
#            "system capability is-enabled" command
##############################################################################
# @name :: _system_capability_is_enabled
##
## @summary Parses the raw output from the "system capability is-enabled" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_capability_is_enabled with any of its options as described in the
##    corresponding cdef file 'system_capability.cdefs'
## @synopsis
##      system_capability_is_enabled(%args);
## INPUTS TO METHOD CALL 'system_capability_is_enabled' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'cap-in' </td> <td> 'cap-in' </td> <td> '0' </td> </tr>
## <tr> <td> 'cap-out' </td> <td> 'cap-out' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = system capability is-enabled -cap-in cifs.mp,SAN,cluster_peer
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Notice: Notice: is_enabled_imp: 'cifs.mp,SAN,cluster_peer'
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;                'is_enabled' => [
##&nbsp;                      'cifs.mp',
##&nbsp;                      'SAN',
##&nbsp;                      'cluster_peer'
##&nbsp;                 ],
##&nbsp;            }
##&nbsp;        ];
##&nbsp;

sub _system_capability_is_enabled {
    $Log->enter() if $may_enter;
    my $self       = $_[0];
    my $raw_output = $self->get_processed_output();
    if ( $raw_output =~ /Notice: is_enabled_imp: \'(.*)\'/i ) {
        my @caps = split /\s*,\s*/, $1;
        $self->{parsed_output}->[0] = { "is_enabled" => \@caps };
    }
    $Log->exit() if $may_exit;
} ## end sub _system_capability_is_enabled
##############################################################################
# Method:    _diag_nblade_fhmap_cache_show
# Objective: This method invokes the brute force parser for the
#            "diagnblade fhmap-cache show" command
##############################################################################
sub _diag_nblade_fhmap_cache_show {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    $self->_fhmap_parser();
    $Log->exit() if $may_exit;
} ## end sub _diag_nblade_fhmap_cache_show

sub _lun_copy_start {
    $Log->enter() if $may_enter;
    my $self   = $_[0];
    my $output = $self->get_processed_output();
    $output =~ /LUN Copy started with job-uuid\s+(\S+).*/;
    my $job_uuid = $1;
    if ( defined $job_uuid ) {
        $self->set_parsed_output(
            parsed_output => [ { 'job-uuid' => $job_uuid } ] );
    }
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _diag_nblade_credentials_show
# Objective: This method invokes the brute force parser for the
#             "diag nblade credentials show" command
##############################################################################
# @name :: _diag_nblade_credentials_show
##
## @summary Parses the raw output from the "diag nblade credentials show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from _diag_nblade_credentials_show with any of its options as described in the
##    corresponding cdef file 'diag_nblade_credentials_show.cdefs'
## @synopsis
##      system_capability_is_enabled_during_downgrade_shim(%args);
## INPUTS TO METHOD CALL 'diag_nblade_credentials_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'capability' </td> <td> 'capability' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = diag nblade credentials show -node local -vserver vs_left -unix-user-name comm_nfs_u1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  Getting credential handles.
##&nbsp;   2 handles found....
##&nbsp;
##&nbsp;  Getting cred 0 for user.
##&nbsp;  Getting cred 1 for user.
##&nbsp;     Global Virtual Server: 8
##&nbsp;     Cred Store Uniquifier: 1
##&nbsp;     Cifs SuperUser Table Generation: 0
##&nbsp;  Unix Creds:
##&nbsp;       Flags: 1
##&nbsp;       Domain ID: 0
##&nbsp;       Uid: 900000866
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            'number_of_credential' => '2 handles found....',
##&nbsp;            'unix_creds' => [
##&nbsp;                         {
##&nbsp;                                'uid' => '900000866',
##&nbsp;                                'flags' => '1',
##&nbsp;                                'domain_id' => '0',
##&nbsp;                                'gid_0' => '900000023',
##&nbsp;                        }
##&nbsp;             'cred_1' => [
##&nbsp;                        {
##&nbsp;                                'global_virtual_server' => '8'
##&nbsp;                                'cifs_superuser_table_generation' => '0',
##&nbsp;                                'cred_store_uniquifier' => '1',
##&nbsp;                        }
##&nbsp;        ];
##&nbsp;
sub _diag_nblade_credentials_show {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my @lines = get_lines(
              "raw_output"      => $self->get_processed_output(),
              "response_object" => $self
            );

    my ($line,@result);
    my $user;
    while(@lines){
       $line = shift(@lines);
	  if( $line =~ /Getting\s+credential\s+handles/i ) {
              my $value = shift(@lines);
              my @num = split(/\s+/,$value);
              $result[0]->{'number_of_credential'} = $num[0];
	  } elsif( $line =~ /Getting\s+cred\s+(\d+)\s+for\s+user/ ) {
              $user = 'cred_'.$1;
	  } elsif( $line =~ /(.*):\s+(.*)/ ) {
	       $result[0]->{$user}->[0]->{to_key_name(item=> $1)} = $2 ;
          } elsif ( $line =~ /(\w+)\s+Creds:$/ ) {
              my $key = $1.'_creds';
              $line = shift(@lines);
              while( $line !~ /(\w+)\s+Creds:$/ and @lines ) {
                  if( $line =~ /Additional\s+Gids/i ) {
                      $line = shift(@lines);
                      while( $line =~ /(.*):\s+(.*)/ and @lines ) {
                          $result[0]->{to_key_name(item=>$key)}->[0]->{'additional_gids'}
                                                      ->[0]->{to_key_name(item=>$1)} = $2 ;
                          $line = shift(@lines);
                      }
                  }
                  else {
                     while( $line =~ /(.*):\s+(.*)/ ) {
                         $result[0]->{to_key_name(item=>$key)}->[0]->{to_key_name(item=> $1)} = $2 ;
                         $line = shift(@lines);
                     }
                 }
                 unshift (@lines, $line ) if( $line =~ /(\w+)\s+Creds:$/ );
             }
	  } else {
              $result[0]->{$user}->[0]->{info} = $line;
          }
     }
     if ($self->{command} =~ /(.*)fields(.*,)(.*)/) {
         my $local_attr = $3;
         my @attr_arr = split (/\s+/,$local_attr);
         shift @attr_arr;
         my %attr_hash = @attr_arr;
         %attr_hash = %{ NACL::C::Component::_convert_to_underscores( \%attr_hash ) };
         my $local_key;
         foreach my $attr (keys(%attr_hash)) {
            $local_key = $attr;
            $attr =~ s/^.//;
            $attr_hash{$attr} = $attr_hash{$local_key};
            $result[0]->{to_key_name(item => $attr)} = $attr_hash{$attr};
            delete $attr_hash{$local_key};
         }
     }
     $self->{parsed_output} = \@result;
     $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _system_services_ndmp_legacy_status
# Objective: This method invokes the brute force parser for the
#            "system services ndmp legacy status" command
##############################################################################
# @name :: _system_services_ndmp_legacy_status
##
## @summary Parses the raw output from the "system services ndmp legacy status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_services_ndmp_legacy_status with any of its options as described in the
##    corresponding cdef file 'system_services_ndmp_legacy_status.cdefs'
## @synopsis
##      system_services_ndmp_legacy_status(%args);
## INPUTS TO METHOD CALL 'system_services_ndmp_legacy_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = system services ndmp legacy status.
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;NDMP legacy mode is disabled.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;               'status' => 'disabled'
##&nbsp;            }
##&nbsp;        ];
##&nbsp;

sub _system_services_ndmp_legacy_status {
    $Log->enter() if $may_enter;
    my $self   = $_[0];
    my $output = $self->get_processed_output();
    $output =~ /NDMP [L|l]egacy mode is (.*)./;
    my $status = $1;
    if ( defined $status ) {
        $self->set_parsed_output(
            parsed_output => [ { 'status' => $status } ] );
    } else {
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::CommandFailedException->throw(
                  'The status could not be determined from the output of '
                . 'the system services ndmp legacy status command' );
    }
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _system_services_ndmp_node_scope_status
# Objective: This method invokes the brute force parser for the
#            "system services ndmp node_scope status" command
##############################################################################
# @name :: _system_services_ndmp_node_scope_mode_status
##
## @summary Parses the raw output from the "system services ndmp node-scope-mode status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_services_ndmp_node_scope_mode_status with any of its options as described in the
##    corresponding cdef file 'system_services_ndmp_node_scope_mode_status.cdefs'
## @synopsis
##      system_services_ndmp_node_scope_mode_status(%args);
## INPUTS TO METHOD CALL 'system_services_ndmp_node_scope_mode_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = system services ndmp node-scope-mode status.
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;NDMP node-scope-mode  is disabled.
##&nbsp
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;               'status' => 'disabled'
##&nbsp;            }
##&nbsp;        ];
##&nbsp;

sub _system_services_ndmp_node_scope_mode_status {
    $Log->enter() if $may_enter;
    my $self   = $_[0];
    my $output = $self->get_processed_output();
    $output =~ /NDMP node\-scope\-mode is (.*)./;
    my $status = $1;
    if ( defined $status ) {
        $self->set_parsed_output(
            parsed_output => [ { 'status' => $status } ] );
    } else {
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::CommandFailedException->throw(
                  'The status could not be determined from the output of '
                . 'the system services ndmp node-scope-mode status command' );
    }
    $Log->exit() if $may_exit;
}

#############################################################################
# @name :: volume_snapshot_compute_reclaimable
##
## @summary Parses the raw output from the "volume snapshot compute-reclaimable" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from volume_snapshot_compute_reclaimable with any of its options
##    as described in the corresponding cdef file.
## @return
##    Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = volume snapshot compute-reclaimable
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;[Job 153] Job succeeded: A total of 69632 bytes can be reclaimed
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;               'bytes_freed' => '69632'
##&nbsp;            }
##&nbsp;        ];
##&nbsp;

sub _volume_snapshot_compute_reclaimable {
    $Log->enter() if $may_enter;
    my $self   = $_[0];
    my $output = $self->get_processed_output();
    if ( $output =~ /A total of (\d+) bytes can be reclaimed/si ) {
        $self->set_parsed_output(
            parsed_output => [ { bytes_freed => $1 } ] );
    } else {
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => 'Could not determine number of bytes freed',
            response_object => $self->get_response_object()
        );
    }
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _storage_failover_hwassist_stats_show
# Objective: This method invokes the brute force parser for the
#            "storage failover hwassist stats show" command
##############################################################################

## @name ::storage_failover_hwassist_stats_show
##
## @summary Parses the raw output from the "storage failover hwassist stats show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from storage_failover_hwassist_stats_show with any of its options as described in the
##    corresponding cdef file 'storage_failover_hwassist_stats.cdefs'
## @synopsis
##      storage_failover_hwassist_stats_show(%args);
## INPUTS TO METHOD CALL 'storage_failover_hwassist_stats_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-times-throttled' </td> <td> 'times-throttled' </td> <td> '0' </td> </tr>
## <tr> <td> '-test' </td> <td> 'test' </td> <td> '0' </td> </tr>
## <tr> <td> '-power-loss' </td> <td> 'power-loss' </td> <td> '0' </td> </tr>
## <tr> <td> '-keep-alive' </td> <td> 'keep-alive' </td> <td> '0' </td> </tr>
## <tr> <td> '-loss-of-heartbeat' </td> <td> 'loss-of-heartbeat' </td> <td> '0' </td> </tr>
## <tr> <td> '-power-cycle-via-rlm' </td> <td> 'power-cycle-via-rlm' </td> <td> '0' </td> </tr>
## <tr> <td> '-reset-via-sp' </td> <td> 'reset-via-sp' </td> <td> '0' </td> </tr>
## <tr> <td> '-unknown-alerts' </td> <td> 'unknown-alerts' </td> <td> '0' </td> </tr>
## <tr> <td> '-reset-via-rlm' </td> <td> 'reset-via-rlm' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> '-bad-nvram-id' </td> <td> 'bad-nvram-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-power-cycle-via-sp' </td> <td> 'power-cycle-via-sp' </td> <td> '0' </td> </tr>
## <tr> <td> '-power-off-via-sp' </td> <td> 'power-off-via-sp' </td> <td> '0' </td> </tr>
## <tr> <td> '-power-off-via-rlm' </td> <td> 'power-off-via-rlm' </td> <td> '0' </td> </tr>
## <tr> <td> '-ss-mismatch' </td> <td> 'ss-mismatch' </td> <td> '0' </td> </tr>
## <tr> <td> '-watchdog-reset' </td> <td> 'watchdog-reset' </td> <td> '0' </td> </tr>
## <tr> <td> '-abnormal-reboot' </td> <td> 'abnormal-reboot' </td> <td> '0' </td> </tr>
## <tr> <td> '-post-error' </td> <td> 'post-error' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = storage failover hwassist stats show
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;                   Node: sti8020-267
##&nbsp;          Local Enabled: true
##&nbsp;Partner Inactive Reason: -
##&nbsp;Alert Type      Alert Event      Count  Takeover    Last Received
##&nbsp;------------ -------------------- ------ --------- --------------------
##&nbsp;system_down  power_loss           0      Yes       ---
##&nbsp;keep_alive   periodic_message     99     No        Thu Jul 30 09:09:06 EDT 2015
##&nbsp;test         test                 0      No        ---
##&nbsp;ID_mismatch  ---                  0      ---       ---
##&nbsp;Key_mismatch ---                  0      ---       ---
##&nbsp;Unknown      ---                  0      ---       ---
##&nbsp;             alerts_throttled     0      ---       ---
##&nbsp;
##&nbsp;                   Node: sti8020-268
##&nbsp;          Local Enabled: true
##&nbsp;Partner Inactive Reason: -
##&nbsp;Alert Type      Alert Event      Count  Takeover    Last Received
##&nbsp;------------ -------------------- ------ --------- --------------------
##&nbsp;system_down  power_loss           0      Yes       ---
##&nbsp;keep_alive   periodic_message     99     No        Thu Jul 30 08:44:06 EDT 2015
##&nbsp;test         test                 0      No        ---
##&nbsp;ID_mismatch  ---                  0      ---       ---
##&nbsp;Key_mismatch ---                  0      ---       ---
##&nbsp;Unknown      ---                  0      ---       ---
##&nbsp;             alerts_throttled     0      ---       ---
##&nbsp;2 entries were displayed.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'sti8020-267' => [
##&nbsp;                               {
##&nbsp;                                 'local_enabled' => 'true'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'partner_inactive_reason' => '-'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'count' => '0',
##&nbsp;                                 'alert_type' => 'system_down',
##&nbsp;                                 'last_received' => '---',
##&nbsp;                                 'takeover' => 'Yes',
##&nbsp;                                 'alert_event' => 'power_loss'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'count' => '99',
##&nbsp;                                 'alert_type' => 'keep_alive',
##&nbsp;                                 'last_received' => 'Thu_Jul_30_09:09:06_EDT_2015',
##&nbsp;                                 'takeover' => 'No',
##&nbsp;                                 'alert_event' => 'periodic_message'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'count' => '0',
##&nbsp;                                 'alert_type' => 'test',
##&nbsp;                                 'last_received' => '---',
##&nbsp;                                 'takeover' => 'No',
##&nbsp;                                 'alert_event' => 'test'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'count' => '0',
##&nbsp;                                 'alert_type' => 'ID_mismatch',
##&nbsp;                                 'last_received' => '---',
##&nbsp;                                 'takeover' => '---',
##&nbsp;                                 'alert_event' => '---'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'count' => '0',
##&nbsp;                                 'alert_type' => 'Key_mismatch',
##&nbsp;                                 'last_received' => '---',
##&nbsp;                                 'takeover' => '---',
##&nbsp;                                 'alert_event' => '---'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'count' => '0',
##&nbsp;                                 'alert_type' => 'Unknown',
##&nbsp;                                 'last_received' => '---',
##&nbsp;                                 'takeover' => '---',
##&nbsp;                                 'alert_event' => '---'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'alerts_throttled' => '0'
##&nbsp;                               }
##&nbsp;                             ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'sti8020-268' => [
##&nbsp;                               {
##&nbsp;                                 'local_enabled' => 'true'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'partner_inactive_reason' => '-'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'count' => '0',
##&nbsp;                                 'alert_type' => 'system_down',
##&nbsp;                                 'last_received' => '---',
##&nbsp;                                 'takeover' => 'Yes',
##&nbsp;                                 'alert_event' => 'power_loss'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'count' => '100',
##&nbsp;                                 'alert_type' => 'keep_alive',
##&nbsp;                                 'last_received' => 'Thu_Jul_30_09:09:06_EDT_2015',
##&nbsp;                                 'takeover' => 'No',
##&nbsp;                                 'alert_event' => 'periodic_message'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'count' => '0',
##&nbsp;                                 'alert_type' => 'test',
##&nbsp;                                 'last_received' => '---',
##&nbsp;                                 'takeover' => 'No',
##&nbsp;                                 'alert_event' => 'test'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'count' => '0',
##&nbsp;                                 'alert_type' => 'ID_mismatch',
##&nbsp;                                 'last_received' => '---',
##&nbsp;                                 'takeover' => '---',
##&nbsp;                                 'alert_event' => '---'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'count' => '0',
##&nbsp;                                 'alert_type' => 'Key_mismatch',
##&nbsp;                                 'last_received' => '---',
##&nbsp;                                 'takeover' => '---',
##&nbsp;                                 'alert_event' => '---'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'count' => '0',
##&nbsp;                                 'alert_type' => 'Unknown',
##&nbsp;                                 'last_received' => '---',
##&nbsp;                                 'takeover' => '---',
##&nbsp;                                 'alert_event' => '---'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'alerts_throttled' => '0'
##&nbsp;                               }
##&nbsp;                             ]
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
sub _storage_failover_hwassist_stats_show {
    $Log->enter() if $may_enter;
    my $self   = $_[0];
    my $output = $self->get_processed_output();
    if ( $self->{output} =~ /$self->{CLI_separator}/ ) {
        $self->_parse_output();
    } else {
        my @lines = get_lines(
            "raw_output"      => $self->get_processed_output(),
            "can_throw"       => 1,
            "response_object" => $self
        );
    my ( @result, @keys );
    my $i = 0;
    my $j = 0;
    my $node;

    foreach my $line (@lines) {
       if($line !~ /^--/){
          if($line =~ /Node: (.*)/ ) {
             $node = $1;
             $j = 0;
             $i++;
             next;
          }
          if($line =~ /alerts_throttled/ ) {
             $line =~ s/---//g;
             $line =~ s/\s//g;
             $line =~ s/alerts_throttled/alerts_throttled: /g;
          }
          next if($line =~ /entries were displayed/);
          $line =~ s/Count Takeover/Count   Takeover/g if($line =~ /Count Takeover/);
          $line =~ s/^\s+//;
	  $line =~ s/\s/_/g;
	  my @split_line = ();

          @split_line = split /[_]+[_]+/, $line;

	  ## extract the headings
	  next if(@keys ~~ @split_line);
          if(! @keys && scalar @split_line > 2) {
             @keys = @split_line;
             next;
          }
          ## extract the node
          if ( scalar @split_line == 1 ) {
             if($split_line[0] =~ /(.+)[:]+[_](.+)/ && $node){
	       $result[$i]->{$node}->[$j]->{ to_key_name( item => $1 ) } = $2;
               $j++;
	       next;
             }
	  }
	  my $k = 0;

          ## fill the result_arr
	  if(@keys && $node ){
		while (@split_line){
		  my $val = shift (@split_line);
		  if($val =~ /((\d+|\w+)[_]+(Yes|No|[-]+))/ ){
		     my $temp = $3;
		     $val = $2;
		     unshift @split_line , $temp ;
		  }
		  $val =~ s/[_]$//;
		  $result[$i]->{$node}->[$j]->{ to_key_name( item => $keys[$k]) } = $val;
		  $k++;
                }
            $j++;
	    }
       }
    }
    $self->{parsed_output} = \@result;
    }
    $Log->exit() if $may_exit;
} ## end sub _storage_failover_hwassist_stats_show

##############################################################################
# Method:    _diag_secd_log_show
# Objective: This method invokes the brute force parser for the
#            "diag secd log show" command
##############################################################################
# @name :: _diag_secd_log_show
##
## @summary Parses the raw output from the "diag_secd_log_show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from diag_secd_log_show with any of its options as described in the
##    corresponding cdef file 'diag_secd_log_show.cdefs'
## @synopsis
##      diag_secd_log_show(%args);
## INPUTS TO METHOD CALL 'diag_secd_log_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = diag secd log show -node sangavai-vsim-1.
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Log Options
##&nbsp;----------------------------------
##&nbsp;Log level:                    Debug
##&nbsp;Function enter/exit logging:  OFF
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;               'log-level' => 'Debug',
##&nbsp;               'function-enter-exit-logging' => 'OFF'
##&nbsp;            }
##&nbsp;        ];
##&nbsp;

sub _diag_secd_log_show {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my @array_lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "can_throw"       => 1,
        "response_object" => $self
    );
    my @result;
    my ( $i, $key, $value );
    for ( $i = 2; $i <= $#array_lines; $i++ ) {
        if ( $array_lines[$i] =~ /(.*):\s+(.*)/ ) {
            $key   = $1;
            $value = $2;
            $key =~ s/[\s\/]+/-/g;
            $result[0]->{ to_key_name( 'item' => $key ) } = $value;
        }
    }
    $self->{parsed_output} = \@result;
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _diag_secd_trace_show
# Objective: This method invokes the brute force parser for the
#            "diag secd trace show" command
##############################################################################
# @name :: _diag_secd_trace_show
##
## @summary Parses the raw output from the "diag_secd_trace_show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from diag_secd_trace_show with any of its options as described in the
##    corresponding cdef file 'diag_secd_log_show.cdefs'
## @synopsis
##      diag_secd_trace_show(%args);
## INPUTS TO METHOD CALL 'diag_secd_trace_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = diag secd trace show -node sangavai-vsim-1.
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;diag secd trace show -fields node -node kathar-vsim-2\r\n
##&nbsp;
##&nbsp;Trace Spec\r\n
##&nbsp;---------------------------------------\r\n
##&nbsp;TraceAll:                     Tracing all RPCs\r\n
##&nbsp;Modules:                      ServerDiscovery UnixAuthorization \r\n
##&nbsp;IP Addrs:                     10.238.37.122 10.238.37.123 \r\n
##&nbsp;Domain names:                 ctl.gdl.englab.netapp.com sim.eng.btc.netapp.in \r\n
##&nbsp;DC names:                     CTL csiqa \r\n
##&nbsp;Vserver IDs:                  cifs_test kathar-vsim-2 \r\n
##&nbsp;RPC names:                    secd_rpc_ad_account_exists secd_rpc_create_cifs_server \r\n
##&nbsp;\r\n
##&nbsp;\r\n
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;              'rpc-names'    => [
##&nbsp;                                    'secd_rpc_ad_account_exists',
##&nbsp;                                    'secd_rpc_create_cifs_server'
##&nbsp;                                ],
##&nbsp;              'ip-addrs'     => [
##&nbsp;                                    '10.238.37.122',
##&nbsp;                                    '10.238.37.123'
##&nbsp;                                ],
##&nbsp;              'vserver-ids'  => [
##&nbsp;                                    'cifs_test',
##&nbsp;                                    'kathar-vsim-2'
##&nbsp;                                ],
##&nbsp;              'traceall'     => 'Tracing all RPCs'
##&nbsp;              'modules'      => [
##&nbsp;                                    'ServerDiscovery',
##&nbsp;                                    'UnixAuthorization'
##&nbsp;                                ],
##&nbsp;              'dc-names'     => [
##&nbsp;                                    'CTL',
##&nbsp;                                    'csiqa'
##&nbsp;                                ],
##&nbsp;              'domain-names' => [
##&nbsp;                                    'ctl.gdl.englab.netapp.com',
##&nbsp;                                    'sim.eng.btc.netapp.in'
##&nbsp;                                ]
##&nbsp;            }
##&nbsp;        ];
##&nbsp;

sub _diag_secd_trace_show {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my @array_lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "can_throw"       => 1,
        "response_object" => $self
    );
    my @result;
    my ( $i, $key, $value );
    for ( $i = 2; $i <= $#array_lines; $i++ ) {
        if ( $array_lines[$i] =~ /(.*):\s+(.*)/ ) {
            $key   = $1;
            $value = $2;
            $key =~ s/[\s\/]+/-/g;
            if ( $array_lines[$i] =~ /TraceAll.*/ ) {
                $result[0]->{ to_key_name( 'item' => $key ) } = $value;
            } else {
                my @array_list = split( /\s/, $value );
                $result[0]->{ to_key_name( 'item' => $key ) } = \@array_list;
            }
        }
    }
    $self->{parsed_output} = \@result;
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _diag_secd_authentication_sid_to_unix_name
# Objective: This method invokes the brute force parser for the
#            "diag secd authentication sid-to-unix-name" command
##############################################################################
# @name :: _diag_secd_authentication_sid_to_unix_name
##
## @summary Parses the raw output from the "diag secd authentication sid-to-unix-name" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from diag_secd_authentication_sid_to_unix_name with any of its options as described in the
##    corresponding cdef file 'diag_secd_authentication_sid_to_unix_name.cdefs'
## @synopsis
##      diag_secd_authentication_sid_to_unix_name(%args);
## INPUTS TO METHOD CALL 'diag_secd_authentication_sid_to_unix_name' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> '-vserver' </td> <td> 'vserver' </td> <td> '0' </td> </tr>
## <tr> <td> '-sid' </td> <td> 'sid' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = diag secd authentication sid-to-unix-name -node jais_vsim6 -vserver vs0 -sid S-1-5-1.
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;   SID Type: Well known group
##&nbsp;  Unix Name:
##&nbsp; Domain Name: NT AUTHORITY
##&nbsp;Windows Name: DIALUP
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;               'windows-name' => 'DIALUP',
##&nbsp;               'domain-name' => 'NT AUTHORITY',
##&nbsp;               'sid-type' => 'Well known group',
##&nbsp;               'unix-name' => ''
##&nbsp;            }
##&nbsp;        ];
##&nbsp;

sub _diag_secd_authentication_sid_to_unix_name {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    _diag_secd_authentication($self);
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _diag_secd_authentication_show_ontap_admin_unix_creds
# Objective: This method invokes the brute force parser for the
#            "diag secd authentication show ontap admin unix creds" command
##############################################################################
# @name ::diag_secd_authentication_show_ontap_admin_unix_creds
##
## @summary Parses the raw output from the "diag secd authentication show ontap admin unix creds" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from diag_secd_authentication_show_ontap_admin_unix_creds with any of its options as described in the
##    corresponding cdef file 'diag_secd_authentication_show_ontap_admin_unix_creds.cdefs'
## @synopsis
##      diag_secd_authentication_show_ontap_admin_unix_creds(%args);
## INPUTS TO METHOD CALL 'diag_secd_authentication_show_ontap_admin_unix_creds' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> '-vserver' </td> <td> 'vserver' </td> <td> '0' </td> </tr>
## <tr> <td> '-unix-user-name' </td> <td> 'unix-user-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-uid' </td> <td> '-uid' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = diag secd authentication show-ontap-admin-unix-creds -node jais-vsim6-01 -vserver vs0 -unix-user-name ldap1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;       User Id: 10001
##&nbsp;      Group Id: 3333
##&nbsp;Home Directory: /u/ldap1
##&nbsp; Login Shell: /bin/csh
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;               'user-id' => '10001',
##&nbsp;               'group-id' => '3333',
##&nbsp;               'home-directory' => '/u/ldap1',
##&nbsp;               'login-shell' => '/bin/csh'
##&nbsp;            }
##&nbsp;        ];
##&nbsp;

sub _diag_secd_authentication_show_ontap_admin_unix_creds {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    _diag_secd_authentication($self);
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _volume_file_show_fhmap
# Objective: This method invokes the brute force parser for the
#            "volume file show-fhmap" command
##############################################################################
# @name ::volume_file_show_fhmap
##
## @summary Parses the raw output from the "volume file show-fhmap" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from volume_file_show_fhmap with any of its options as described in the
##    corresponding cdef file 'volume_file_show_fhmap.cdefs'
## @synopsis
##      volume_file_show_fhmap(%args);
## INPUTS TO METHOD CALL 'volume_file_show_fhmap' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-filehandle' </td> <td> 'filehandle' </td> <td> '0' </td> </tr>
## <tr> <td> '-vserver' </td> <td> 'vserver' </td> <td> '0' </td> </tr>
## <tr> <td> '-path' </td> <td> 'path' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = volume file show-fhmap -vserver vs0 -path /NS/test_file
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Namespace          Data               Namespace          Data
##&nbsp;Filehandle         Filehandle         File Path          File Path
##&nbsp;------------------ ------------------ ------------------ ------------------
##&nbsp;02040000000000000000000060000000a5849519
##&nbsp;                   03040000000000000000000061000000a8849519
##&nbsp;                                      /NS/test_file      /.system/constituents/
##&nbsp;                                                         default_data0001/1ae8/
##&nbsp;                                                         P4EBAAIEAIAAAAAAAAAAAG
##&nbsp;                                                         AAAAClhJUZ
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'namespace-filehandle' => '02040000000000000000000060000000a5849519',
##&nbsp;            'namespace-path' => '/NS/test_file',
##&nbsp;            'data-filehandle' => '03040000000000000000000061000000a8849519',
##&nbsp;            'data-path' => '/.system/constituents/default_data0001/1ae8/P4EBAAIEAIAAAAAAAAAAAGAAAAClhJUZAAAAClhJUZ'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub _volume_file_show_fhmap {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    my @result;
    $self->_fhmap_parser();
    my $hash = $self->{'parsed_output'}[0];
    foreach ( keys %$hash ) {
        my $key   = $_;
        my $value = $hash->{$key};
        delete $hash->{$key};
        $key =~ s/_/-/g;
        $hash->{$key} = $value;
    }
    $result[0] = $hash;
    $self->{parsed_output} = \@result;
    $Log->exit;
} ## end sub _volume_file_show_fhmap

##########################Private Method######################################

##############################################################################
# Method:    _fhmap_parser
# Objective: This method returns the brute force parser for the
#            "volume file show-fhmap" and "diag nblade fhmap cache" commands.
##############################################################################
sub _fhmap_parser {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    if ( $self->{output} =~ /$self->{CLI_separator}/ ) {
        $self->_parse_output();
    } else {
        my @lines = get_lines(
            "raw_output"      => $self->get_processed_output(),
            "can_throw"       => 1,
            "response_object" => $self
        );
        my @result;
        my %hash;
        my ( $i, $j, );
        my ( $data_filepath, $namespace_filepath, $flag, $data_filehandle );
        for ( $i = 2; $i <= $#lines; $i++ ) {
            if ( $lines[$i] =~ /(([-]+)\s)+\2/ ) {
                $i++;
                if ( $lines[$i] =~ /(\S+)/ ) {
                    $result[0]->{'namespace_filehandle'} = $1;
                }
                $i++;
                if ( $lines[$i] =~ /(\S+)\s+(\S+)/ ) {
                    $data_filehandle    .= $1;
                    $namespace_filepath .= $2;
                } else {
                    $lines[$i] =~ /(\S+)/;
                    $data_filehandle .= $1;
                }
            } elsif ( $lines[$i] =~ /(\S+)\s+(\S+)/ ) {
                if ( $1 eq "-" ) {
                    $data_filepath = $1;
                    $flag          = $2;
                    last;
                }
                $namespace_filepath .= $1;
                $data_filepath = $2;
                while ( $i <= $#lines ) {
                    $i++;
                    if ( $lines[$i] =~ /(\S+)\s+(\S+)/ ) {
                        $data_filepath .= $1;
                        $flag = $2;
                        last;
                    }
                    $lines[$i] =~ /(\S+)/;
                    $data_filepath .= $1;
                }
            } elsif ( $lines[$i] =~ /(\S+)/ ) {
                $namespace_filepath .= $1;
                $i++;
                if (   $lines[$i] =~ /(\S+)(\s+)(\S+)/
                    || $lines[$i] =~ /(\/\.\S+)/ )
                {
                    if ( $2 eq ' ' ) {
                        $namespace_filepath .= $1;
                        $data_filepath = $3;
                    } else {
                        $data_filepath .= $1;
                    }
                    while ( $i <= $#lines ) {
                        $i++;
                        if ( $lines[$i] =~ /(\S+)\s+(\S+)/ ) {
                            $data_filepath .= $1;
                            $flag = $2;
                            last;
                        }
                        $lines[$i] =~ /(\S+)/;
                        $data_filepath .= $1;
                    }
                } else {
                    $i--;
                }
            }
        }
        $result[0]->{'data_filehandle'} = $data_filehandle;
        $result[0]->{'namespace_path'}  = $namespace_filepath;
        $result[0]->{'data_path'}       = $data_filepath;
        if ( defined $flag ) {
            $result[0]->{'entry_flags'} = $flag;
        }

        $self->{parsed_output} = \@result;
    }
    $Log->exit;
} ## end sub _fhmap_parser

# Brute-force parsers come here
# These are for commands whose outputs cannot be parsed using the generic
# parser
#Brute force parser supports _diag_secd_group_mapping_show and _diag_secd_name_mapping_show

sub _diag_secd_show {
    my ($self) = @_;
    $Log->enter() if $may_enter;

    my @lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "can_throw"       => 1,
        "response_object" => $self
    );
    push @lines, $self->get_command();
    my @result;
    foreach my $line (@lines) {
        if ( $line =~ /\S+ maps to (\S+)/ ) {
            $result[0]->{'maps-to'} = $1;
        }
        elsif ( $line
            =~ /diag secd.*mapping show -(.*)\s+(\S+)\s+-(.*)\s+(\S+)\s-(.*)\s+(\S+)\s-(.*)\s+(\S+)/
            )
        {
            $result[0]->{$1} = $2;
            $result[0]->{$3} = $4;
            $result[0]->{$5} = $6;
            $result[0]->{$7} = $8;
        } ## end if ( $line =~ ...)
        elsif ($line =~ /ATTENTION:\s+(.*)/i) {
            my @attention = split(/:\s+/,$lines[0]);
            $attention[1] =~ s/\s+$//g;
            $result[0]->{$attention[0]} =$attention[1];
        } ## end if ( $llines[0] =~ ...)
    } ## end foreach my $line (@lines)
    $self->{parsed_output} = \@result;
    $Log->exit() if $may_exit;
} ## end sub _diag_secd_show

##############################################################################
# Method:    _diag_secd_name_mapping_show
# Objective: This method invokes the brute force parser for the
#            "diag secd name-mapping show" command
##############################################################################
# @name ::_diag_secd_name_mapping_show
##
## @summary Parses the raw output from the "_diag_secd_name_mapping_show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from _diag_secd_name_mapping_show with any of its options as described in the
##    corresponding cdef file 'diag_secd_name_mapping_show.cdefs'
## @synopsis
##      _diag_secd_name_mapping_show(%args);
## INPUTS TO METHOD CALL '_diag_secd_name_mapping_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> 'vserver' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> 'name' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> 'direction' </td> <td> 'node' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = diag secd name-mapping show -vserver cifs_test -name root -node kathar-vsim-2 -direction unix-win.
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;diag secd name-mapping show -vserver cifs_test -name root -node kathar-vsim-2 -direction unix-win\r\n
##&nbsp;
##&nbsp;ATTENTION: Mapping of Data ONTAP "admin" users to UNIX user "root" is enabled, but the following information does not reflect this mapping.\r\n
##&nbsp;
##&nbsp;root maps to CTL\\adadmin\r\n
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'direction' => 'unix-win',
##&nbsp;            'maps-to' => 'CTL\\adadmin',
##&nbsp;            'vserver' => 'cifs_test',
##&nbsp;            'name' => 'root',
##&nbsp;            'node' => 'kathar-vsim-2',
##&nbsp;            'ATTENTION' => 'Mapping of Data ONTAP admin users to UNIX user root is enabled',
##&nbsp;          }
##&nbsp;        ];
sub _diag_secd_name_mapping_show {
    my ($self) = @_;
    _diag_secd_show($self);

}

##############################################################################
# Method:    _diag_secd_group_mapping_show
# Objective: This method invokes the brute force parser for the
#            "diag secd group-mapping show" command
##############################################################################
# @name ::_diag_secd_group_mapping_show
##
## @summary Parses the raw output from the "_diag_secd_group_mapping_show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from _diag_secd_group_mapping_show with any of its options as described in the
##    corresponding cdef file 'diag_secd_group_mapping_show.cdefs'
## @synopsis
##      _diag_secd_group_mapping_show(%args);
## INPUTS TO METHOD CALL '_diag_secd_group_mapping_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> 'vserver' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> 'group' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> 'direction' </td> <td> 'node' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = diag secd group-mapping show -direction win-unix -group grp4 -vserver cifs_test -node kathar-vsim-2
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;diag secd group-mapping show -direction win-unix -group grp4 -vserver cifs_test -node kathar-vsim-2\r\n
##&nbsp;
##&nbsp;grp4 maps to grp4\r\n
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'direction' => 'win-unix',
##&nbsp;            'maps-to' => 'grp4',
##&nbsp;            'vserver' => 'cifs_test',
##&nbsp;            'group' => 'grp4',
##&nbsp;            'node' => 'kathar-vsim-2'
##&nbsp;          }
##&nbsp;        ];
sub _diag_secd_group_mapping_show {
    my ($self) = @_;
    _diag_secd_show($self);
}

##############################################################################
# Method:    _volume_file_clone_status
# Objective: This method invokes the brute force parser for the
#            "volume file clone status" command
##############################################################################
# @name ::_volume_file_clone_status
##
## @summary Parses the raw output from the "volume file clone status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from _volume_file_clone_status with any of its options as described in the
##    corresponding cdef file '_volume_file_clone_status.cdefs'
## @synopsis
##      _volume_file_clone_status(%args);
## INPUTS TO METHOD CALL '_volume_file_clone_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = volume file clone status.
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Operation Complete.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;               'status' => 'Operation Complete'
##&nbsp;            }
##&nbsp;        ];
##&nbsp;

sub _volume_file_clone_status {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my @array_lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "can_throw"       => 1,
        "response_object" => $self
    );
    my @result;
    $result[0]->{'status'} = $array_lines[0];
    $self->{parsed_output} = \@result;
    $Log->exit() if $may_exit;
}

# @name ::diag_secd_connections_show
##
## @summary Parses the raw output from the "diag secd connections show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from diag_secd_connections_show with any of its options as described in the
##    corresponding cdef file 'diag_secd_connections_show.cdefs'
## @synopsis
##      diag_secd_connections_show(%args);
## INPUTS TO METHOD CALL 'diag_secd_connections_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-vserver' </td> <td> 'vserver' </td> <td> '0' </td> </tr>
## <tr> <td> '-type' </td> <td> 'type' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> '-key' </td> <td> 'key' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = diag secd connections show -node kathar-vsim-2-01 -vserver cifs_test
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;[ Cache: NetLogon/ctl.gdl.englab.netapp.com ]
##&nbsp;  Queue> Waiting: 0, Max Waiting: 1, Wait Timeouts: 0, Avg Wait: 0.00ms
##&nbsp;  Performance> Hits: 0, Misses: 1, Failures: 0, Avg Retrieval: 4872.33ms
##&nbsp;
##&nbsp;  (No connections active or currently cached)
##&nbsp;
##&nbsp;[ Cache: LSA/ctl.gdl.englab.netapp.com ]
##&nbsp;  Queue> Waiting: 0, Max Waiting: 1, Wait Timeouts: 0, Avg Wait: 0.00ms
##&nbsp;  Performance> Hits: 14, Misses: 10, Failures: 0, Avg Retrieval: 1839.36ms
##&nbsp;
##&nbsp;  (No connections active or currently cached)
##&nbsp;
##&nbsp;[ Cache: LDAP (Active Directory)/ctl.gdl.englab.netapp.com ]
##&nbsp;  Queue> Waiting: 0, Max Waiting: 1, Wait Timeouts: 0, Avg Wait: 0.00ms
##&nbsp;  Performance> Hits: 1, Misses: 162, Failures: 0, Avg Retrieval: 1399.54ms
##&nbsp;
##&nbsp;  (No connections active or currently cached)
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'queue' => [
##&nbsp;                         {
##&nbsp;                           'avg_wait' => '0.00ms',
##&nbsp;                           'waiting' => '0',
##&nbsp;                           'wait_timeouts' => '0',
##&nbsp;                           'max_waiting' => '1'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'server' => 'ctl.gdl.englab.netapp.com',
##&nbsp;            'performance' => [
##&nbsp;                               {
##&nbsp;                                 'failures' => '0',
##&nbsp;                                 'misses' => '1',
##&nbsp;                                 'hits' => '0',
##&nbsp;                                 'avg_retrieval' => '4872.33ms'
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'cache' => 'NetLogon'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'queue' => [
##&nbsp;                         {
##&nbsp;                           'avg_wait' => '0.00ms',
##&nbsp;                           'waiting' => '0',
##&nbsp;                           'wait_timeouts' => '0',
##&nbsp;                           'max_waiting' => '1'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'server' => 'ctl.gdl.englab.netapp.com',
##&nbsp;            'performance' => [
##&nbsp;                               {
##&nbsp;                                 'failures' => '0',
##&nbsp;                                 'misses' => '10',
##&nbsp;                                 'hits' => '14',
##&nbsp;                                 'avg_retrieval' => '1839.36ms'
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'cache' => 'LSA'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'queue' => [
##&nbsp;                         {
##&nbsp;                           'avg_wait' => '0.00ms',
##&nbsp;                           'waiting' => '0',
##&nbsp;                           'wait_timeouts' => '0',
##&nbsp;                           'max_waiting' => '1'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'server' => 'ctl.gdl.englab.netapp.com',
##&nbsp;            'performance' => [
##&nbsp;                               {
##&nbsp;                                 'failures' => '0',
##&nbsp;                                 'misses' => '162',
##&nbsp;                                 'hits' => '1',
##&nbsp;                                 'avg_retrieval' => '1399.54ms'
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'cache' => 'LDAP (Active Directory)'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub _diag_secd_connections_show {

    $Log->enter() if $may_enter;
    my ($self)      = @_;
    my $i           = -1;
    my @array_lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "response_object" => $self
    );
    my @result;
    foreach my $line (@array_lines) {
        if ( $line =~ /\[\s+Cache:\s+(.*)\/(.*)\s+\]/ ) {
            $i++;
            $result[$i]->{cache}  = $1;
            $result[$i]->{server} = $2;
        } elsif ( $line =~ /(\w+)>\s+(.*)/ ) {
            my $key      = $1;
            my $vals     = $2;
            my @sub_vals = split( /,\s+/, $vals );
            foreach my $key_val (@sub_vals) {
                $key_val =~ /(.*):\s+?(.*)/;
                $result[$i]->{ to_key_name( item => $key ) }->[0]
                    ->{ to_key_name( item => $1 ) } = $2;
            }
        }

    }
    $self->{parsed_output} = \@result;
    $Log->exit() if $may_exit;

}
##############################################################################
# Method:    _volume_file_reservation
# Objective: This method invokes the brute force parser for the
#            "volume file reservation" command
# Details:   This command does not honour the CLI separator. Its output is
#            identical to the output structure for the 7Mode "file reservation"
#            command, so we re-use that parser here.
##############################################################################
sub _volume_file_reservation {
    my ($self) = @_;
    $Log->enter() if $may_enter;

    require NACL::APISet::Node::CLI::7Mode::default::file;

    # Make an entry for "actual_command" since the 7Mode parser
    # requires it (for CMode, the "actual command" is the same as what's
    # stored in the "command" field)
    $self->{actual_command} = $self->get_command();

    $self->{parsed_output}
        = NACL::APISet::Node::CLI::7Mode::default::file::file_reservation(
        response_object => $self );

    # In CMode, the status contains multiple \cH characters, which are all
    # returned in the status field. Extract only the actual status
    my $status = $self->{parsed_output}[0]{status};
    $status =~ /(on|off)/;
    $self->{parsed_output}[0]{status} = $1;

    $Log->debug(
        sub { "Parsed output is:\n" . Dumper( $self->{parsed_output} ) } );
    $Log->exit() if $may_exit;
} ## end sub _network_ping

sub _volume_file_move_start {
    $Log->enter() if $may_enter;
    $_[0]->_volume_file_move_copy_start_helper();
    $Log->exit() if $may_exit;
}

sub _volume_file_copy_start {
    $Log->enter() if $may_enter;
    $_[0]->_volume_file_move_copy_start_helper();
    $Log->exit() if $may_exit;
}

sub _volume_file_move_copy_start_helper {
    $Log->enter() if $may_enter;
    my $self   = $_[0];
    my $output = $self->get_processed_output();
    $output =~ /started with job-uuid\s+(\S+).*/;
    my $job_uuid = $1;
    if ( defined $job_uuid ) {
        $self->set_parsed_output(
            parsed_output => [ { 'job-uuid' => $job_uuid } ] );
    }
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _system_timezone
# Objective: This method invokes the brute force parser for the
#            "system timezone" command
##############################################################################
# @name ::_system_timezone
##
## @summary Parses the raw output from the "_system_timezone" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from _system_timezone with any of its options as described in the
##    corresponding cdef file 'system_timezone.cdefs'
## @synopsis
##      _system_timezone(%args);
## INPUTS TO METHOD CALL '_system_timezone' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = system timezone
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Timezone: America/Los_Angeles\r\n
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'timezone' => 'America/Los_Angeles'
##&nbsp;           }
##&nbsp;         ];
sub _system_timezone {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my @array_lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "first_line"      => 'Timezone',
        "can_throw"       => 1,
        "response_object" => $self
    );
    my @result_array;
    if ( $array_lines[0] =~ /Timezone:\s+(\S+)/ ) {
        $result_array[0]->{'timezone'} = $1;
    }
    $self->{parsed_output} = \@result_array;
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _diag_secd_authentication_translate
# Objective: This method invokes the brute force parser for the
#            "diag secd authentication translate" command
##############################################################################
# @name ::diag_secd_authentication_translate
##
## @summary Parses the raw output from the "diag secd authentication translate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from diag_secd_authentication_translate with any of its options as described in the
##    corresponding cdef file 'diag_secd_authentication_translate.cdefs'
## @synopsis
##      diag_secd_authentication_translate(%args);
## INPUTS TO METHOD CALL 'diag_secd_authentication_translate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> '-vserver' </td> <td> 'vserver' </td> <td> '0' </td> </tr>
## <tr> <td> '-sid' </td> <td> 'sid' </td> <td> '0' </td> </tr>
## <tr> <td> '-gid' </td> <td> 'gid' </td> <td> '0' </td> </tr>
## <tr> <td> '-uid' </td> <td> 'uid' </td> <td> '0' </td> </tr>
## <tr> <td> '-unix-user-name' </td> <td> 'unix-user-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-unix-group-name' </td> <td> 'unix-group-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-win-name' </td> <td> 'win-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = diag secd authentication translate -vserver vs0 -node jais-vsim6 -gid 0.
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;root
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;               'unix-group-name' => 'root',
##&nbsp;            }
##&nbsp;        ];
##&nbsp;

sub _diag_secd_authentication_translate {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my @array_lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "response_object" => $self
    );
    my $command = $self->get_command();
    my @result;
    if ( $command =~ /.*uid.*/ ) {
        $result[0]->{'unix-user-name'} = $array_lines[0];
    } elsif ( $command =~ /.*gid.*/ ) {
        $result[0]->{'unix-group-name'} = $array_lines[0];
    } elsif ( $command =~ /.*unix-user-name.*/ ) {
        $result[0]->{'uid'} = $array_lines[0];
    } elsif ( $command =~ /.*unix-group-name.*/ ) {
        $result[0]->{'gid'} = $array_lines[0];
    } elsif ( $command =~ /.*sid.*/ ) {
        $result[0]->{'win-name'} = $array_lines[0];
    } elsif ( $command =~ /.*win-name.*/ ) {
        $result[0]->{'sid'} = $array_lines[0];
    }
    $self->{parsed_output} = \@result;
    $Log->exit() if $may_exit;
}

# @name ::diag_nblade_cifs_path_mapping_show
##
## @summary Parses the raw output from the "diag nblade cifs path-mapping show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from diag_nblade_cifs_path_mapping_show with any of its options as described in the
##    corresponding cdef file 'diag_nblade_cifs_path_mapping_show.cdefs'
## @synopsis
##      diag_nblade_cifs_path_mapping_show(%args);
## INPUTS TO METHOD CALL 'diag_nblade_cifs_path_mapping_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-vserver' </td> <td> 'vserver' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = diag nblade cifs path-mapping show -vserver vs1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;VserverId    Type    Unix Path                        Cifs Path
##&nbsp;---------------------------------------------------------------------------------------
##&nbsp;       30            1    /unix_name_2/                    \CIFS1\SHARE2\win_name_2\
##&nbsp;       30            1    /unix_name_1/                    \CIFS1\SHARE1\win_name_1\
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'unix-path' => '/unix_name_2/',
##&nbsp;            'vserverid' => '30',
##&nbsp;            'cifs-path' => '\\CIFS1\\SHARE2\\win_name_2\\',
##&nbsp;            'type' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'unix-path' => '/unix_name_1/',
##&nbsp;            'vserverid' => '30',
##&nbsp;            'cifs-path' => '\\CIFS1\\SHARE1\\win_name_1\\',
##&nbsp;            'type' => '1'
##&nbsp;          }
##&nbsp;        ];

sub _diag_nblade_cifs_path_mapping_show {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my ( @result, @values );
    my $cnt         = 0;
    my @array_lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "response_object" => $self
    );

    my @keys = split( /\s+/, shift @array_lines );
    @keys = map { lc($_) } @keys;
    shift @array_lines;
    foreach my $line (@array_lines) {
        @values = split( /\s+/, $line );
        $result[$cnt]->{ $keys[0] }          = $values[0];
        $result[$cnt]->{ $keys[1] }          = $values[1];
        $result[$cnt]->{"$keys[2]-$keys[3]"} = $values[2];
        $result[$cnt]->{"$keys[4]-$keys[5]"} = $values[3];
        $cnt++;
    }
    $self->{parsed_output} = \@result;
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _diag_secd_configuration_query
# Objective: This method invokes the brute force parser for the
#            "diag secd configuration query" command
##############################################################################
# @name :: diag_secd_configuration_query
##
## @summary Parses the raw output from the " diag secd configuration query" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from  diag_secd_configuration_query with any of its options as described in the
##    corresponding cdef file ' diag_secd_configuration_query.cdefs'
## @synopsis
##       diag_secd_configuration_query(%args);
## INPUTS TO METHOD CALL 'diag_secd_configuration_query' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-vserver' </td> <td> 'vserver' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## </table>
# example 1
#kimpal-vsim21395161494::*> diag secd configuration query -node local -source-name local-unix-user -query-string "vserver=71"
#vserver    user      id       primary-gid
#-------    ------    -----    -----------
#71         pcuser    65534    65534
#71         nobody    65535    65535
#71         root      0        1
#
# Example 2
#ck1clus::*> diag secd configuration query -node local -source-name dns  -query-string "vserver=13"
#     vserver: 13
#     domains: cifsqa.lab.netapp.com
#name-servers: 172.17.152.42
#       state: true
#     timeout: 2
#    attempts: 1
#
# Example 3
#ck1clus::*> diag secd configuration query -node local -source-name name-mapping  -query-string "vserver=13"
#    vserver: 13
#       type: user
#  direction: win-unix
#   position: 0
#    pattern: *
#replacement: pcuser
#
#    vserver: 13
#       type: user
#  direction: unix-win
#   position: 0
#    pattern: *
#replacement: CIFSQAV6B\administrator
#
#    vserver: 13
#       type: user
#  direction: win-unix
#   position: 1
#    pattern: CIFSQA\\administrator
#replacement: root
#
#    vserver: 13
#       type: user
#  direction: unix-win
#   position: 1
#    pattern: root
#replacement: CIFSQA\\administrator

sub _diag_secd_configuration_query {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my ( @result, @values );
    my $cnt         = 0;
    my @array_lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "response_object" => $self
    );

    # for output format like example 1
    if (!grep {/vserver:/} @array_lines) {
        my @keys = split( /\s+/, shift @array_lines );
        @keys = map { lc($_) } @keys;
        shift @array_lines;
        foreach my $line (@array_lines) {
            @values = split( /\s+/, $line );

            my $subcount = 0;
            foreach  (@keys) {
                $result[$cnt]->{ $_ } = $values[$subcount++];
            }
            $cnt++;
        }
    } else {
        # for output format like example 2,3
        $cnt = -1;
        foreach (@array_lines) {
            next if (/~\s+$/);
            $cnt++ if (/vserver:/);
            my ($key,$value) = split(/:\s*/);
            $result[$cnt]->{ $key } = $value;
        }
    }
    $self->{parsed_output} = \@result;
    $Log->exit() if $may_exit;
}


##############################################################################
# Method:    _diag_nblade_cifs_server_show
# Objective: This method invokes the brute force parser for the
#            "diag nblade cifs server show" command
##############################################################################
# @name ::diag_nblade_cifs_server_show
##
## @summary Parses the raw output from the "diag nblade cifs server show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from diag_nblade_cifs_server_show with any of its options as described in the
##    corresponding cdef file 'diag_nblade_cifs_server_show.cdefs'
## @synopsis
##      diag_nblade_cifs_server_show(%args);
## INPUTS TO METHOD CALL 'diag_nblade_cifs_server_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-vserver' </td> <td> 'vserver' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp; Command = diag nblade cifs server show
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;There are 2 Virtual Servers:
##&nbsp;Virtual Server ID:      3
##&nbsp;Server Type:            Kerberos
##&nbsp;Name:                   STI22404025VS0
##&nbsp;Domain Name:            CTL
##&nbsp;WINS Servers:           Count = 1
##&nbsp;WINS Server IP (0): 10.229.184.239
##&nbsp;Machine SID:            S-1-5-21-1896871423-3211229150-3383017265-25850
##&nbsp;Local SID:              S-1-5-21-1896871423-3211229150-3383017265
##&nbsp;IP Addresses:           Count = 4
##&nbsp;IP (0): 10.226.80.121
##&nbsp;IP (1): 10.226.80.122
##&nbsp;IP (2): 10.226.80.123
##&nbsp;IP (3): 10.226.80.124
##&nbsp;IPs To Register:        Count = 7
##&nbsp;IP (0): 10.226.80.121
##&nbsp;IP (1): 10.226.80.122
##&nbsp;IP (2): 10.226.80.123
##&nbsp;IP (3): 10.226.80.124
##&nbsp;IP (4): 10.226.80.126
##&nbsp;IP (5): 10.226.80.127
##&nbsp;IP (6): 10.226.80.128
##&nbsp;Virtual Server ID:      4
##&nbsp;Server Type:            Kerberos
##&nbsp;Name:                   CIFS1
##&nbsp;Domain Name:            CTL
##&nbsp;WINS Servers:           Count = 0
##&nbsp;Machine SID:            S-1-5-21-1896871423-3211229150-3383017265-44619
##&nbsp;Local SID:              S-1-5-21-1896871423-3211229150-3383017265
##&nbsp;IP Addresses:           Count = 1
##&nbsp;IP (0): 10.226.80.129
##&nbsp;IPs To Register:        Count = 1
##&nbsp;IP (0): 10.226.80.129
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;[
##&nbsp;  {
##&nbsp;    'domain-name' => 'CTL',
##&nbsp;    'name' => 'STI22404025VS0',
##&nbsp;    'server-type' => 'Kerberos',
##&nbsp;    'wins-servers' => [
##&nbsp;                        {
##&nbsp;                          'ip-0' => '10.229.184.239'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;    'virtual-server-id' => '3',
##&nbsp;    'machine-sid' => 'S-1-5-21-1896871423-3211229150-3383017265-25850',
##&nbsp;    'ip-addresses' => [
##&nbsp;                        {
##&nbsp;                          'ip-3' => '10.226.80.124',
##&nbsp;                          'ip-0' => '10.226.80.121',
##&nbsp;                          'ip-2' => '10.226.80.123',
##&nbsp;                          'ip-1' => '10.226.80.122'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;    'ips-to-register' => [
##&nbsp;                           {
##&nbsp;                             'ip-3' => '10.226.80.124',
##&nbsp;                             'ip-0' => '10.226.80.121',
##&nbsp;                             'ip-4' => '10.226.80.126',
##&nbsp;                             'ip-2' => '10.226.80.123',
##&nbsp;                             'ip-6' => '10.226.80.128',
##&nbsp;                             'ip-5' => '10.226.80.127',
##&nbsp;                             'ip-1' => '10.226.80.122'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;    'local-sid' => 'S-1-5-21-1896871423-3211229150-3383017265'
##&nbsp;  },
##&nbsp;  {
##&nbsp;    'domain-name' => 'CTL',
##&nbsp;    'name' => 'CIFS1',
##&nbsp;    'server-type' => 'Kerberos',
##&nbsp;    'virtual-server-id' => '4',
##&nbsp;    'machine-sid' => 'S-1-5-21-1896871423-3211229150-3383017265-44619',
##&nbsp;    'ip-addresses' => [
##&nbsp;                        {
##&nbsp;                          'ip-0' => '10.226.80.129'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;    'ips-to-register' => [
##&nbsp;                           {
##&nbsp;                             'ip-0' => '10.226.80.129'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;    'local-sid' => 'S-1-5-21-1896871423-3211229150-3383017265'
##&nbsp;  }
##&nbsp;];
##&nbsp;

sub _diag_nblade_cifs_server_show {
    my ($self) = @_;
    my @lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "response_object" => $self
    );
    my ( $result_array, @head, @values );
    my $count = -1;
    shift(@lines);

    #converting to CMode options
    my $cmodearg = sub {
        my $var = shift;
        $var =~ s/\s+/-/g;
        $var = lc $var;
        return $var;
    };

    for ( my $i = 0; $i <= $#lines; $i++ ) {
        if ( $lines[$i] =~ /(Virtual\s+Server\s+ID)[:]\s+(\d+)/ ) {
            $count++;
            $result_array->[$count]->{ $cmodearg->($1) } = $2;
        } elsif ( $lines[$i]
            =~ /(WINS\s+Servers|IP\s+Addresses|IPs\s+To\s+Register)[:].*/ )
        {
            my $header = $cmodearg->($1);
            while ( $lines[ $i + 1 ] =~ /(IP\s+\(\d+\)):\s+(.*)/ ) {
                my $key   = $1;
                my $value = $2;
                $key =~ s/\(//;
                $key =~ s/\)//;
                $result_array->[$count]->{$header}[0]->{ $cmodearg->($key) }
                    = $value;
                $i++;
            }
        } else {
            @values = split( /:\s*/, $lines[$i] );
            for ( my $i = 0; $i <= $#values; $i = $i + 2 ) {
                $result_array->[$count]->{ $cmodearg->( $values[$i] ) }
                    = $values[ $i + 1 ];
            }
        }
    }
    $self->{parsed_output} = $result_array;
    $Log->exit() if $may_exit;
} ## end sub _diag_nblade_cifs_server_show

sub _diag_secd_authentication {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my @array_lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "can_throw"       => 1,
        "response_object" => $self
    );
    my @result;
    my ( $i, $key, $value );
    for ( $i = 0; $i <= $#array_lines; $i++ ) {
        if ( $array_lines[$i] =~ /(.*):\s*(.*)/ ) {
            $key   = $1;
            $value = $2;
            $key =~ s/\s+/-/g;
            $result[0]->{ to_key_name( 'item' => $key ) } = $value;
        }
    }
    $self->{parsed_output} = \@result;

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _storage_failover_hwassist_test
# Objective: This method parses the output of
#            storage failover hwassist test command. This command does not
#            follow the CMode standard of reporting errors with the output
#            starting with "Error:". So, this method checks the cmd output and
#            throws CommandFailedException exception if hwassist test fails.
# Sample Output:
# fas3170-210-07-08-cm::> storage failover hwassist test -node fas3170-210-07
#
# Info: Storage Failover Hwassist Error: No response
# from partner(fas3170-210-08).Timed out.
#
# fas3170-210-07-08-cm::>
#
# fas3170-210-07-08-cm::> storage failover hwassist test -node fas3170-210-08
#
# Info: Storage Failover Hwassist: Operation successful.
#
# fas3170-210-07-08-cm::>
##############################################################################

sub _storage_failover_hwassist_test {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my $raw_op = $self->get_processed_output();
    if ( $raw_op =~ m[Info:\s+Storage\s+Failover\s+Hwassist\s+Error:\s+] ) {
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $raw_op,
            response_object => $self
        );
    }
    $self->{parsed_output} = [];
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _volume_autosize
# Objective: This method invokes the brute force parser for the
#            "vol autosize" command
# Details:   This command does not honour the CLI separator. It's output is
#            identical to the output structure for the 7Mode "volume autosize"
#            command, so we re-use that parser here.
##############################################################################

sub _volume_autosize {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    require NACL::APISet::Node::CLI::7Mode::default::vol;

    # Make an entry for "actual_command" since the 7Mode parser
    # requires it (for CMode, the "actual command" is the same as what's
    # stored in the "command" field)
    $self->{actual_command} = $self->get_command();

    $self->{parsed_output}
        = NACL::APISet::Node::CLI::7Mode::default::vol::vol_autosize(
        response_object => $self );

    $Log->debug(
        sub { "Parsed output is:\n" . Dumper( $self->{parsed_output} ) } );
    $Log->exit() if $may_exit;
} ## end sub _volume_autosize

##############################################################################
# Method:    _network_ping6
# Objective: This method invokes the brute force parser for the
#            "network ping6" command
# Details:   This command does not honour the CLI separator. It's output is
#            identical to the output structure for the 7Mode "ping6"
#            command, so we re-use that parser here.
##############################################################################
sub _network_ping6 {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    $Log->debug("In brute force parser for 'network ping6'");

    require NACL::APISet::Node::CLI::7Mode::default::ping6;

    # Make an entry for "actual_command" since the 7Mode parser
    # requires it (for CMode, the "actual command" is the same as what's
    # stored in the "command" field)
    $self->{actual_command} = $self->get_command();

    $self->{parsed_output}
        = NACL::APISet::Node::CLI::7Mode::default::ping6::ping6(
        response_object => $self );

    $Log->debug( "Parsed output is:\n" . Dumper( $self->{parsed_output} ) );
    $Log->exit() if $may_exit;
} ## end sub _network_ping6

##############################################################################
# Method:    _diag_secd_netgroup_show_hosts
# Objective: This method invokes the brute force parser for the
#            "diag secd netgroup show-hosts" command
##############################################################################
# @name ::_diag_secd_netgroup_show_hosts
##
## @summary Parses the raw output from the "diag secd netgroup show-hosts" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from diag_secd_netgroup_show_hosts with any of its options as described in the
##    corresponding cdef file 'diag_secd_netgroup_show_hosts.cdefs'
## @synopsis
##      diag_secd_netgroup_show_hosts(%args);
## INPUTS TO METHOD CALL 'diag_secd_netgroup_show_hosts' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> '-vserver' </td> <td> 'vserver' </td> <td> '0' </td> </tr>
## <tr> <td> '-netgroup-name' </td> <td> 'netgroup-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp; Command = diag secd netgroup show-hosts -vserver vs0 -netgroup-name netgroup -node praveent_vsim1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;vmlnx217-15.lab.eng.btc.netapp.in
##&nbsp;vmlnx218-15.lab.eng.btc.netapp.in
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;         {
##&nbsp;           'hosts' => [
##&nbsp;                        'vmlnx217-15.lab.eng.btc.netapp.in'
##&nbsp;                        'vmlnx218-15.lab.eng.btc.netapp.in'
##&nbsp;                      ]
##&nbsp;         }
##&nbsp;        ];
##&nbsp;

sub _diag_secd_netgroup_show_hosts {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    my @lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "response_object" => $self
    );
    my @result_array ;
    my $count = 0;
    foreach my $line (@lines) {
        $result_array[0]->{hosts}[$count] = $line;
        $count++;
    }
    $self->{parsed_output} = \@result_array;
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _diag_secd_netgroup_show_triples
# Objective: This method invokes the brute force parser for the
#            "diag secd netgroup show-triples" command
##############################################################################
# @name ::_diag_secd_netgroup_show_triples
##
## @summary Parses the raw output from the "diag secd netgroup show-triples" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from diag_secd_netgroup_show_triples with any of its options as described in the
##    corresponding cdef file 'diag_secd_netgroup_show_triples.cdefs'
## @synopsis
##      diag_secd_netgroup_show_triples(%args);
## INPUTS TO METHOD CALL 'diag_secd_netgroup_show_triples' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> '-vserver' </td> <td> 'vserver' </td> <td> '0' </td> </tr>
## <tr> <td> '-netgroup-name' </td> <td> 'netgroup-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp; Command = diag secd netgroup show-triples -vserver vs0 -netgroup-name netgroup -node praveent_vsim1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;(vmlnx217-15.lab.eng.btc.netapp.in,vmlnx218-15.lab.eng.btc.netapp.in,vmlnx219-15.lab.eng.btc.netapp.in)
##&nbsp;vmlnx220-15.lab.eng.btc.netapp.in
##&nbsp;vmlnx221-15.lab.eng.btc.netapp.in
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'hosts' => [
##&nbsp;                         'vmlnx220-15.lab.eng.btc.netapp.in',
##&nbsp;                         'vmlnx221-15.lab.eng.btc.netapp.in'
##&nbsp;                       ],
##&nbsp;            'triples' => [
##&nbsp;                           '(vmlnx217-15.lab.eng.btc.netapp.in,vmlnx218-15.lab.eng.btc.netapp.in,vmlnx219-15.lab.eng.btc.netapp.in)'
##&nbsp;                         ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub _diag_secd_netgroup_show_triples {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    my @lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "response_object" => $self
    );
    my @result_array ;
    my ($count, $limit) = 0;
    foreach my $line (@lines) {
        if($line =~ /\((.*)\)/) {
            $result_array[0]->{triples}[$count] = $line;
            $count++;
        } else {
            $result_array[0]->{hosts}[$limit] = $line;
            $limit++;
        }
    }
    $self->{parsed_output} = \@result_array;
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _diag_secd_netgroup_show_host_addresses
# Objective: This method invokes the brute force parser for the
#            "diag secd netgroup show-host-addresses" command
##############################################################################
# @name ::_diag_secd_netgroup_show_host_addresses
##
## @summary Parses the raw output from the "diag secd netgroup show-host-addresses" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from diag_secd_netgroup_show_host_addresses with any of its options as described in the
##    corresponding cdef file 'diag_secd_netgroup_show_host_addresses.cdefs'
## @synopsis
##      diag_secd_netgroup_show_host_addresses(%args);
## INPUTS TO METHOD CALL 'diag_secd_netgroup_show_host_addresses' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> '-vserver' </td> <td> 'vserver' </td> <td> '0' </td> </tr>
## <tr> <td> '-netgroup-name' </td> <td> 'netgroup-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp; Command = diag secd netgroup show-host-addresses -vserver vs0 -netgroup-name netgroup -node praveent_vsim1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;40.1.1.12
##&nbsp;40.1.1.13
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;         {
##&nbsp;           'hosts_addresses' => [
##&nbsp;                                   '40.1.1.12'
##&nbsp;                                   '40.1.1.13'
##&nbsp;                                ]
##&nbsp;         }
##&nbsp;        ];
##&nbsp;

sub _diag_secd_netgroup_show_host_addresses {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    my @lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "response_object" => $self
    );
    my @result_array ;
    my $count = 0;
    foreach my $line (@lines) {
        $result_array[0]->{hosts_addresses}[$count] = $line;
        $count++;
    }
    $self->{parsed_output} = \@result_array;
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _volume_file_clone_token_create
# Objective: This method invokes the brute force parser for the
#            "volume file clone token create" command
##############################################################################
# @name :: _volume_file_clone_token_create
##
## @summary Parses the raw output from the "volume file clone token create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from volume_file_clone_token_create with any of its options as described in the
##    corresponding cdef file 'volume_file_clone_token_create.cdefs'
## @synopsis
##      volume_file_clone_token_create(%args);
## INPUTS TO METHOD CALL 'volume_file_clone_token_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> '-reserve-size' </td> <td> 'reserve-size' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp; Command = volume file clone token create -reserve-size 200 -node sti5-vsim-ucs088a
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Token UUID: 6bcc3848-c5c3-4f4f-823a-6e435e14b1ac
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;         {
##&nbsp;           'token_uuid' => '6bcc3848-c5c3-4f4f-823a-6e435e14b1ac'
##&nbsp;         }
##&nbsp;        ];
##&nbsp;

sub _volume_file_clone_token_create {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my @lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "response_object" => $self
    );
    my @result_array ;

    if($lines[0] =~ /Token\s+UUID:\s+([\w\-]+)/) {
        $result_array[0]->{token_uuid} = $1;
    } else {
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                text => 'Unexpected output format (probably because the output '
                        . 'format has changed). Please raise a NACL burt (nacl/nacl_core) '
                        . "for this failure. Original output:\n"
                        . $self->get_processed_output(),
                response_object => $self
        );
    }

    $self->{parsed_output} = \@result_array;
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _diag_secd_dns_srv_lookup
# Objective: This method invokes the brute force parser for the
#            "diag secd dns srv lookup" command
##############################################################################

sub _diag_secd_dns_srv_lookup {
    $Log->enter() if $may_enter;
    my $self       = $_[0];
    my @lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "response_object" => $self,
        "first_line"      => 'Got \d+ Ip Addresses',
    );
    my @result_array ;
    my $count = 0;
    foreach my $line (@lines) {
        $result_array[0]->{ip_address} [$count]= $line;
        $count++;
    }
    $self->{parsed_output} = \@result_array;
    $Log->exit() if $may_exit;

} ## end sub _diag_secd_dns_srv_lookup

sub _volume_file_fingerprint_start {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    my $output = $self->get_processed_output();

    # Example output:
    # File fingerprint operation is queued. Run "volume file fingerprint show
    # -session-id FP1:158215fb-e89e-11e4-aa5c-0050569c00bc" to view the fingerprint session status.
    # Extract session-id from this
    $output =~ /session\-id\s+(\S+)"/;
    my $session_id = $1;
    $self->{parsed_output} = [{session_id => $session_id}];
    $Log->exit() if $may_exit;
}

sub _system_license_add {
    $Log->enter() if $may_enter;
    my $self   = $_[0];
    my @lines = get_lines(
        raw_output      => $self->get_processed_output(),
        response_object => $self,
    );
    
    # Output will be like:
    # License for package "SnapRestore" and serial number "1-81-0000000000000600000156334" installed successfully.\r\n
    # License for package "NFS" installed successfully.\r\n
    # (2 of 2 added successfully)\r\n
    #
    # The first line is an example of a node-locked license, which has
    # package and serial-number in its output.
    # The second line is an example of a site license, which has only package
    # in its output.
    #
    my $parsed_output = [];
    my $i = 0;
    foreach my $line (@lines) {
        if ($line =~ /License\s+for\s+package\s+"(.*?)"\s+(?:and\s+serial\s+number\s+"(.*)"\s+)?installed\s+successfully/) {
            $parsed_output->[$i]{package} = $1;
            $parsed_output->[$i]{'serial-number'} = $2 if defined $2;
        }
        $i++;
    }

    $self->set_parsed_output(parsed_output => $parsed_output);
    $Log->exit() if $may_exit;
}
##############################################################################
# Method:    _volume_file_show_filehandle
# Objective: This method invokes the brute force parser for the
#            "volume file show-filehandle" command
##############################################################################
##Processed Output is
##  Vserver                    Path
##  ----------------------     ---------------------------
##  vs0                         /vol/TESTVOL/seeped_balalaika_1445059790
##
##  flags   snapid  fileid     generation  fsid        msid          dsid
##  ------- ------  ---------  ----------  ----------  ------------  ------------
##  0x0     0       0xcb       0xd3d683b   0x402       0x804f8392    0x402
##
##Parsed output is
##$VAR1 = [
##          {
##            'flags' => '0x0',
##            'msid' => '0x804f8392',
##            'fileid' => '0xcb',
##            'vserver' => 'vs0',
##            'dsid' => '0x402',
##            'path' => '/vol/TESTVOL/seeped_balalaika_1445059790',
##            'snapid' => '0',
##            'generation' => '0xd3d683b',
##            'fsid' => '0x402'
##          }
##        ];
##
## Below code is written for multiple lines output
##
sub _volume_file_show_filehandle {
    $Log->enter() if $may_enter;
    my $self   = $_[0];
    my $output = $self->get_processed_output();
    if ( $self->{output} =~ /$self->{CLI_separator}/ ) {
        $self->_parse_output();
    } else {
        my @lines = get_lines(
            "raw_output"      => $self->get_processed_output(),
            "can_throw"       => 1,
            "response_object" => $self
        );
		my (@values ,@headings, @last_line);
		my $key = -1;
		my $index = 0;

		foreach my $line (@lines) {
			if($line !~ /^--/){
				$line =~ s/^\s+//;
				$line =~ s/\s/_/g;

				my @split_line = ();
				@split_line = split /[_]+[_]+/, $line;
				my ($last_line) = @last_line;

                                if (defined $last_line) {
                                    if ($last_line !~ /^--/) {
				        @{$values[$key][$index]}= @last_line;
					$index++;
				    }
                                }
				@last_line = @split_line;
			}
			else{
			    @headings = (@headings,@last_line);
			    @last_line = $line;
			    $index = 0;
		        $key++;
			    next;
			}
		}
		@{$values[$key][$index]}= @last_line;

		my @result;
		my $i = 0;
		$key++;
		while ($key) {
		    my $j = 0;
		    my @rows = @{$values[$key-1]};
                    my $rows = @rows;
		    while($rows) {
				my @res1 = @{$result[$j]} if (defined $result[$j]);
				my @res2 = @{$values[$i][$j]} if (defined $values[$i][$j]);
				@{$result[$j]} = (@res1,@res2);;
			    $j++;
			    $rows--;
			}
			$i++;
		    $key--;
		}
		my @Parsed_output;
		$i =0;
		foreach my $row (@result) {
		    my $j = 0;
			foreach my $heading (@headings) {
			$Parsed_output[$i]->{ to_key_name( 'item' => $heading) } = @{$row}[$j];
			$j++;
			}
			$i++;
		}
		$self->{parsed_output} = \@Parsed_output;
    }
    $Log->exit() if $may_exit;
} ## end sub _volume_file_show_filehandle

##############################################################################
# Method:    _debug_locks_show_auto_resync
# Objective: This method invokes the brute force parser for the
#            "debug locks show-auto-resync" command
##############################################################################
sub _debug_locks_show_auto_resync {
    $Log->enter() if $may_enter;
    my $self       = $_[0];
    my $raw_output = $self->get_processed_output();
    if ( $raw_output =~ /Vserver\s+(.*),\s+Volume\s+(.*):\s+(.*)/) {
        $self->{parsed_output}->[0]->{'vserver'} = $1;
        $self->{parsed_output}->[0]->{'volume'}  = $2;
        $self->{parsed_output}->[0]->{'lock-resync-status'}  = $3;
    }
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _diag_nblade_cifs_shares
# Objective: This method invokes the brute force parser for the
#            "diag nblade cifs shares" command
##############################################################################
# @name ::diag_nblade_cifs_shares
##
## @summary Parses the raw output from the "diag nblade cifs shares" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from diag_nblade_cifs_shares with any of its options as described in the
##    corresponding cdef file 'diag_nblade_cifs_shares.cdefs'
## @synopsis
##      diag_nblade_cifs_shares(%args);
## INPUTS TO METHOD CALL 'diag_nblade_cifs_shares' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-vserver' </td> <td> 'vserver' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> '-share' </td> <td> 'share' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp; Command = diag nblade cifs shares
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Virtual server vs1 has 1 share:\r\n
##&nbsp;  Share:  test_share \r\n
##&nbsp;      Path:                  / \r\n
##&nbsp;      Type:                  Disk \r\n
##&nbsp;      Comment:                \r\n
##&nbsp;      Oplocks:                 on \r\n
##&nbsp;      Browsable:               on \r\n
##&nbsp;      Show Snapshot:           off \r\n
##&nbsp;      ChangeNotify:            on \r\n
##&nbsp;      HomeDirectory:           off \r\n
##&nbsp;      Attribute Cache:         off \r\n
##&nbsp;      Continuously Avail:      off \r\n
##&nbsp;      BrachCache:              off \r\n
##&nbsp;      AccessBasedEnumeration:  off \r\n
##&nbsp;      NameSpaceCaching:        off \r\n
##&nbsp;      EncryptData:             off \r\n
##&nbsp;      DirectoryLeasing:        off \r\n
##&nbsp;      ShowPreviousVersions:    off \r\n
##&nbsp;      File Umask(octal):       0 \r\n
##&nbsp;      Dir Umask(octal):        0 \r\n
##&nbsp;      ShadowCopy:              off \r\n
##&nbsp;      Symlinks:                on,ro\r\n
##&nbsp;      ACL:\r\n
##&nbsp;           Revision:      2 \r\n
##&nbsp;           AceCount:      1 \r\n
##&nbsp;                Ace:      Full Control\r\n
##&nbsp;                SID:      S-1-1-0\r\n

##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;[
##&nbsp;  {
##&nbsp;	'forcegroupenabled' => 'Disabled',
##&nbsp;	'changenotify' => 'on',
##&nbsp;	'symlinks' => [
##&nbsp;	              'local-only'
##&nbsp;	             ],
##&nbsp;	'namespacecaching' => 'off',
##&nbsp;	'attribute_cache' => 'off',
##&nbsp;	'directoryleasing' => 'off',
##&nbsp;	'showpreviousversions' => 'on',
##&nbsp;	'file_umaskoctal' => '0',
##&nbsp;	'browsable' => 'on',
##&nbsp;	'dir_umaskoctal' => '0',
##&nbsp;	'encryptdata' => 'off',
##&nbsp;	'oplocks' => 'on',
##&nbsp;	'maxconnectionspershare' => '4294967295',
##&nbsp;	'homedirectory' => 'off',
##&nbsp;	'branchcache' => 'off',
##&nbsp;	'acl' => [
##&nbsp;				{	'SID' => 'S-1-1-0',
##&nbsp;					'Ace' => 'Full Control'
##&nbsp;				}
##&nbsp;			],
##&nbsp;	'path' => '/',
##&nbsp;	'offlinefiles' => 'manual',
##&nbsp;	'attributecachettl' => '0',
##&nbsp;	'shadowcopy' => 'off',
##&nbsp;	'accessbasedenumeration' => 'off',
##&nbsp;	'revision' => '2',
##&nbsp;	'comment' => undef,
##&nbsp;	'acecount' => '1',
##&nbsp;	'show_snapshot' => 'off',
##&nbsp;	'vscanfileopprofile' => 'standard',
##&nbsp;	'type' => 'Disk',
##&nbsp;	'continuously_avail' => 'off'
##&nbsp;  }
##&nbsp;]
##&nbsp;

sub _diag_nblade_cifs_shares {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my (@result);
    my $cnt         = -1;
    my $acl_cnt     = 0;
    my @array_lines = get_lines(
        "raw_output"      => $self->get_processed_output(),
        "response_object" => $self
    );

    shift(@array_lines);

    foreach my $line (@array_lines) {
        if ($line =~ /^\s*Share\s*:/) {
            $cnt++;
			$acl_cnt = 0;
        }
		if ($line =~ /:/) {
            my @values = split(':', $line);
            $values[0] =~ s/^\s+|\s+$//g if (defined($values[0]));
            $values[1] =~ s/^\s+|\s+$//g if (defined($values[1]));
            if ($line =~ /Ace\s*:|SID\s*:/) {
                $result[$cnt]->{'acl'}->[$acl_cnt]->{$values[0]} = $values[1];
                $acl_cnt++ if ($line =~ /SID\s*:/);
            } elsif ($line =~ /Symlinks\s*:/) {
                my @symlink_values = split(',', $values[1]);
                for (my $i = 0; $i < scalar(@symlink_values); $i++) {
                    $result[$cnt]->{'symlinks'}->[$i] = $symlink_values[$i];
                }

            } else {
                $result[$cnt]->{to_key_name(item => $values[0])} = $values[1];
            }
        }
    }

    $self->{parsed_output} = \@result;
    $Log->exit() if $may_exit;

} ## end sub _diag_nblade_cifs_shares

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Response::CLI::CMode - Cluster mode CLI Response class

=head1 SYNOPSIS

 use NACL::APISet::Response::CLI::CMode;

=head1 DESCRIPTION

A C<NACL::APISet::Response::CLI::CMode> object is used to receive, parse and hold
the output from cluster mode CLI API methods.

=head1 ATTRIBUTES

=over

=item output

Raw command output

=item parsed_output

Output parsed into a standardized array of hashes structure

=item CLI_separator

The separator inserted between fields of output, which simplifies parsing

=item not_parsed

Lines of output not parsed - these lines would include info lines

=item command

Entire command line invoked on the CMode filer

=item execution_time

Time taken to execute the command (in seconds)

=item method_name

The base command invoked

=back

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::ResponseException

=item NACL::APISet::Exceptions::MissingRequiredParamException

=item NACL::APISet::Exceptions::InvalidParamException

=item NACL::APISet::Exceptions::InvalidParamValueException

=item NACL::APISet::Exceptions::UnsupportedOptionException

=item NACL::APISet::Exceptions::CommandFailedException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Response::CLI::CMode>.

=over

=item Synopsis

 $response = NACL::APISet::Response::CLI::CMode->new( %args );

=item Arguments

=over

=item output

The raw output from the command.  This can be parsed into an array of key value pairs.

=item CLI_separator

The ngsh string used to separate the fields of the output.

=item command

This is the full command line invoked

=item execution_time

The time taken to execute the command (in seconds)

=item method_name

The name of the method invoked.
E.g: system_node_show

=item parsed_output

A reference to parsed output - which is an array of key-value pairs.

=item not parsed

Reference to an array of lines not parsed

=back

=back

=head1 METHODS

=head2 get_raw_output

Return the output as it was originally returned from the node, stripped of
command echo.

=over

=item Synopsis

$raw_output = $response->get_raw_output();

=back

=head2 get_output_not_parsed

Return those lines which were ignored during parsing. These include INFO or
Info lines and those with "-showseparator". This returns a reference to an
array of lines.

=over

=item Synopsis

@lines = @{$response->get_output_not_parsed()};

=back

=head2 get_job_ids

Return the job IDs from the parsed ouptut (if present). Can be invoked in
either list or scalar context. Returns the first Job ID in scalar context.

=over

=item Synopsis

@IDs = $response->get_job_ids();
$ID = $response->get_job_ids();

=back

=head2 is_private_job

Return 1 if the job is a private job, else return 0.

=over

=item Synopsis

$is_private_job = $response->is_private_job();

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut
