
# this is here due to our standard usage of @_ in Params::Validate methods.
##no critic (RequireArgUnpacking ProhibitFixedStringMatches)
##no critic (ControlStructures::ProhibitUnlessBlocks)
##no critic (ValuesAndExpressions::ProhibitInterpolationOfLiterals)
##no critic (Modules::ProhibitMultiplePackages)
##no critic (RequireBracedFileHandleWithPrint)

#
# Copyright (c) 2011-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroCluster MTask Module
## @author ng-mcc-dpg-qa@netapp.com
## @status shared
## @pod here

package NACL::MTask::MCC::Metrocluster;

use strict;
use warnings;

use Carp;
use Hostrec qw();

use List::Util qw(first);
use Scalar::Util qw(blessed);

use base qw (NACL::MTask::MTask);

use NACL::MTask::Wafliron;
use NACL::MTask::MCC::Utils    ();
use NACL::MTask::MCC::Cluster  ();
use NACL::MTask::MCC::DR_Group ();
use NACL::MTask::MCC::Interconnect;

use NACL::C::Node;
use NACL::C::Cluster    ();
use NACL::C::SystemNode ();
use NACL::C::LoggerMgwdLog;
use NACL::C::Metrocluster;
use NACL::CS::Statistics;
use NACL::CS::StorageAggregate;
use NACL::CS::MetroclusterOperation;
use NACL::CS::StorageAggregatePlexShow;
use NACL::CS::MetroclusterOperationHistory;

use NACL::MTask::MCC::Validator::Manager;
use NACL::MTask::MCC::Validator::CheckConfigClusterDiff;
use NACL::MTask::MCC::Validator::CheckConfigVserverDiff;

# Load exceptions
use NACL::APISet::Exceptions::JobFailedException;
use NACL::MTask::MCC::Exceptions::BaseException;
use NACL::MTask::MCC::Exceptions::DiskState::FakeInitState;

package NACL::C::Metrocluster;

#for now, redefine this sub if it's there, otherwise ignore
if ( defined &_primary_keys_validate_spec ) {

    sub _primary_keys_validate_spec {

      # we are redefining the NACL::C::Metrocluster::_primary_keys_validate_spec
      # method, which looks like this in NACL/C/Metrocluster.pm
      #
      # sub _primary_keys_validate_spec {
      #     return (
      #         'cluster' => { type => SCALAR },
      #         'node'    => { type => SCALAR },
      #     );
      # } ## end sub _primary_keys_validate_spec

        # here we just say there are no cluster or node primary keys to validate
        # against for our NACL::C::Metrocluster components.
        return ();
    }
}
#
# and back to our regularly scheduled package...
package NACL::MTask::MCC::Metrocluster;

use NACL::MTask::MCC::Exceptions::InvalidMethodUsage ();

use NACL::MTask::MCC::Log ();

use NACL::STask::Fiji ();    #for menu use only...

use NATE::BaseException qw(:try);

use NACL::Exceptions::NoElementsFound qw(:try);

use Tharn qw( subtest);
use Subtest;

use NATE::Log qw(log_global);
my $Log       = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NATE::STAF::Peer qw($Parent);

use Params::Validate
  qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF);

use Readonly;
use Data::Dumper;

use List::Util      ();
use List::MoreUtils ();
use Time::HiRes qw(gettimeofday tv_interval);

##
## These are the bridge software versions allowed by
## version_is_valid(). "Deprecated" will produce a warning. If the
## version does not appear here, then a fatal exception will be
## thrown.
## "Obsolete" will produce a warning/exception (move to this after deprecated)
## "Testing" will produce a warning to downgrade to current release
##           and is to be used by groups validating future firmware
##
our %Bridge_Firmware_Versions = (
    'ATTO' => {
        '1.61' => 'OK',
        '1.60' => 'Deprecated',
        '069G' => 'OK',
        'A69G' => 'Deprecated',
        'A69B' => 'Obsolete',
        'A68T' => 'Obsolete',
        'A68X' => 'Obsolete',
        '1.50' => 'Obsolete',
        'A65D' => 'Obsolete',
        '1.49' => 'Obsolete'
    },
    'PANGOLIN' => { '069G' => 'OK', },
);

##
## These are the switch software versions allowed by
## version_is_valid(). "Deprecated" will produce a warning. If the
## version does not appear here, then a fatal exception will be
## thrown.
## "Obsolete" will produce a warning/exception (move to this after deprecated)
## "Testing" will produce a warning to downgrade to current release
##           and is to be used by groups validating future firmware
##
our %Switch_Firmware_Versions = (
    'Cisco' => {
        '6.2(9a)' => 'OK',
        '6.2(1)'  => 'Deprecated',
        '5.2(6a)' => 'Obsolete'
    },
    'Brocade' => {
        'v7.2.1c1' => 'OK',
        'v7.2.1c'  => 'OK',
        'v7.2.1b'  => 'Deprecated',
        'v7.2.1'   => 'Obsolete',
        'v7.0.2b1' => 'Obsolete',
    }
);

##
## This table maps vendor strings to the keys in %Bridge_Firmware_Versions
## and %Switch_Software_Versions.
##
our %Vendor_Normalization_Table = (
    'Cisco Systems, Inc.'           => 'Cisco',
    'Armadillo 6Gbps FC-SAS bridge' => 'ATTO',
    'Pangolin 12Gbps FC-SAS bridge' => 'PANGOLIN',
    'Brocade Communications, Inc.'  => 'Brocade',
);

=head1 NAME

NACL::MTask::MCC::MetroCluster

=head1 DESCRIPTION

This task is to be used for C-Mode MetroCluster testing.

    # Start test
    use NACL::MTask::MCC::MetroCluster;
    ...

    my $mcc = NACL::MTask::MCC::MetroCluster->new(
                               nodes => $arrayref_of_node_names_in_mcc_system);
    ...

    # Perform test here
    ...

    # End test

=cut

=head1 METHODS

=item C<<new(...)>>

    my $mcc = NACL::MTask::MCC::MetroCluster->new(
                               nodes => $arrayref_of_node_names_in_mcc_system);

Creates a new NACL::MTask::MCC::Metrocluster object.  It is important to
understand that a Metrocluster component object executes commands on a
particular site's cluster only, so the library must understand which site
the user wishes to execute Metrocluster commands upon.

See also NACL::MTask::MCC::DR_Coordinator for a more concrete example of this
dynamic.

=over

=item Options

=over

=item C<< nodes => $one_or_more_node_names_in_the_mcc_system >>

(Required.) An arrayref containing one or more nodes in an MCC system.

First, an initial A1 node will be chosen randomly from amongst the nodes listed
in the nodes parameter.  This will define both what is considered to be "site A"
as well as the A1 node for the first DR Group in a given test.

After automatic DR Group discovery, if the MCC system is found to contain
multiple DR Groups, then the A1 node for each of the remaining DR Groups will
be chosen randomly from the nodes in this DR Group at site A which are listed
in the nodes parameter.  If none of these nodes are listed, a random node in
this DR Group at site A will be chosen as the A1 node for this DR Group.

If nodes contains two or more nodes which are not in the same MCC system, an
exception will be thrown.

=back

=item Associated ENVIRONMENT Variables

=over

=item C<< DISABLE_SHARED_STORAGE_NETWORK_VALIDATION = 0|1 >>

(Optional, and not recommended except for specific exceptional cases.)

If this environment variable is set to 1 in a test environment, then the
automatic discovery and validation of the shared storage network will not
be executed.  This is not recommended unless the testbed has specific and
understood issues which need to be worked around in the shared storage
network.  The validation of the shared network before any test is an important
part of understanding and triaging various failures that can occur.

A WARNING will occur when this environment variable is used.

[NB: If the test you are running has any reason to call the method

    $dr_group->get_network_topology()

then DISABLE_SHARED_STORAGE_NETWORK_VALIDATION cannot be set.]

=item C<< DISABLE_SHARED_STORAGE_DISK_STATE_VALIDATION = 0|1 >>

(Optional, and not recommended except for specific exceptional cases.)

If this environment variable is set to 1 in a test environment, then the
automatic disk state validation that occurs during initialization and also often
occurs by default after DR operations will be disabled unless explicitly called
for in the test.

In other words, if the user sets DISABLE_SHARED_STORAGE_DISK_STATE_VALIDATION=1
in their test, but then explicitly calls

    $shared_storage->validate_disk_states()

then such validation will occur at that point, but will revert back to disabled
disk state validations afterwards.

Use of this environment variable is not recommended.  Changes in disk state
are a major part of MCC DR lifecycle operations and should always be validated
against unless specific and understood issues in the disk states need to be
worked around to allow other testing.

A WARNING will occur when this environment variable is used.

See also: WARN_ONLY_ON_INVALID_PLEX_ASSIGNMENT

=item C<< WARN_ONLY_ON_INVALID_PLEX_ASSIGNMENT = 0|1 >>

(Optional, and should only be used in a very specific case as detailed below.)

Since invalid plex assignment issues may be part of an MCC system
setup, this environment variable is used to make sure all shared storage disk
validations report on, but do not fail the test, when such issues are noted.

In other words, this is a workaround to a specific issue that should be
resolved at some point (by the RAID team?)

A WARNING will occur when this environment variable is used.

See also: DISABLE_SHARED_STORAGE_DISK_STATE_VALIDATION

=back

=back

=cut

# Automatically generate the constructor (includes a call to 'init').  This is
# a public interface.
#
# Also creates non-public members and non-public accessors for them.

use Class::MethodMaker [
    new    => [ '-init', 'new' ],
    scalar => [
        { -type => 'NACL::C::Metrocluster' },
        qw/ _site_A_metrocluster_component
          _site_B_metrocluster_component /,
    ],
    scalar => [
        { -type => 'NACL::MTask::MCC::Cluster' },
        qw/ _cluster_A
          _cluster_B /,
    ],
    scalar =>
      [ { -type => 'NACL::MTask::MCC::Interconnect' }, qw/ _interconnect /, ],
    scalar => [ qw/ _dr_groups _ontap_build_version _validation_log /, ],
    scalar => [ qw/ _ignore_bad_state _vsim _mcc_topology_type /, ],
    scalar => [
        { '-default' => \%Bridge_Firmware_Versions },
        '_bridge_firmware_versions'
    ],
    scalar => [
        { '-default' => \%Switch_Firmware_Versions },
        '_switch_firmware_versions'
    ],
    scalar => [
        { '-default' => \%Vendor_Normalization_Table },
        '_vendor_normalization_table'
    ],
    scalar => [
        { -type => 'NACL::MTask::MCC::Validator::Manager' }, qw/ _validator /,
    ],
    scalar => [ qw/ is_mccip /, ],
];

# implicitly called by Class::MethodMaker's new (see -init)
sub init {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    # log volumous constructor related info to MCC_Constructor.log
    my $constructor_log = undef;
    my $prm_hndl        = NATE::ParamSet::param_global();
    if ( my $top_log_dir = $prm_hndl->get('TOP_LOGDIR') ) {
        my $run_id  = $prm_hndl->get('RUNID');
        my $log_dir = "$top_log_dir/$run_id";

        # if runid log is present, then we're ok, else its STDOUT (PerlDB)
        if ( -f "$log_dir.log" ) {
            mkdir("$log_dir") if ( !-d "$log_dir" );
            $constructor_log = "$log_dir/$$\_MCC_Constructor.log";
            $Log->comment(
                    "Logging MCC::Metrocluster object construction and initial "
                  . "validation to $constructor_log..." );
        }
    }

    my %opts = validate_with(
        params => \@_,
        spec   => {
            nodes            => { type  => ARRAYREF, },
            ignore_bad_state => { regex => qr/^[01]$/, default => 0, },
        },
    );

    my $node_names = delete( $opts{'nodes'} );

    #create validator to use for all mcc validation
    $self->_validator( NACL::MTask::MCC::Validator::Manager->new() );

    my $ignore_bad_state =
      defined $ENV{'IGNORE_BAD_MCC_STATE'}
      ? $ENV{'IGNORE_BAD_MCC_STATE'}
      : $opts{'ignore_bad_state'};
    $self->_ignore_bad_state($ignore_bad_state);

    if ($ignore_bad_state) {
        $Log->warn( 'Constructor in IGNORE_BAD_MCC_STATE mode! '
              . 'Testbed MAY be in a bad state! '
              . '  This test should be considered INVALID!' );
    }

    unless ( scalar( @{$node_names} ) ) {
        NACL::MTask::MCC::Exceptions::BaseException->throw(
                "Invalid MCC::Metrocluster constructor call.  At least one\n"
              . "    node name must be passed in the 'nodes' parameter." );
    }

    # this is a private package which is defined at the end of this file
    $Log->comment('Acquiring bootstrap cluster and node command interfaces...');

    $Log->redirect( path => $constructor_log ) if $constructor_log;
    my $bootstrap_util =
      NACL::MTask::MCC::Metrocluster::_Bootstrap_Util->new(
        node_names => $node_names );

    # these are needed in various inits early on
    $self->_site_A_metrocluster_component(
        $bootstrap_util->get_site_A_Metrocluster_component() );
    try {
        $self->_site_B_metrocluster_component(
            $bootstrap_util->get_site_B_Metrocluster_component() );
    }
    catch NATE::BaseException with {
        if ( $self->_ignore_bad_state() ) {
            $Log->warn("IGNORE BAD STATE: site B metrocluster component");
            $self->_site_B_metrocluster_component();
        }
        else {
            my $exception = shift;
            $exception->throw();
        }
    };
    $Log->comment(
        'site b component:'
          . NACL::MTask::MCC::Utils->dump(
            $self->_site_B_metrocluster_component()
          )
    );

    $Log->reset_path() if $constructor_log;

    $Log->comment('...done');

    # can now check if system is in a good MCC state
    $Log->comment('Metrocluster system validation, part 1...');

    $Log->redirect( path => $constructor_log ) if $constructor_log;
    $self->_validate_initial_metrocluster_state__part_1();
    $Log->reset_path() if $constructor_log;

    $Log->comment('...ok');

    # create cluster objects
    $Log->comment('Creating cluster harness objects...');

    $Log->redirect( path => $constructor_log ) if $constructor_log;
    $self->_cluster_A(
        NACL::MTask::MCC::Cluster->new(
            metrocluster => $self,
            site         => 'A'
        )
    );
    $self->_cluster_B(
        NACL::MTask::MCC::Cluster->new(
            metrocluster => $self,
            site         => 'B'
        )
    );

    $Log->reset_path() if $constructor_log;

    $Log->comment('...done');

    # create and init DR Groups' objects
    $Log->comment('Creating DR Group harness objects...');

    $Log->redirect( path => $constructor_log ) if $constructor_log;
    $self->_dr_groups(
        $self->_create_DR_groups(
            bootstrap_util       => $bootstrap_util,
            constructor_log_path => $constructor_log
        )
    );
    $Log->reset_path() if $constructor_log;

    $Log->comment('...done');

    # create interconnect object
    $Log->comment('Creating interconnect harness object...');

    $Log->redirect( path => $constructor_log ) if $constructor_log;
    $self->_interconnect(
        NACL::MTask::MCC::Interconnect->new( metrocluster => $self, ) );

    $Log->reset_path() if $constructor_log;

    $Log->comment('...done');

    $Log->comment('Gathering MCC topology details...');
    $Log->redirect( path => $constructor_log ) if $constructor_log;
    $self->get_mcc_topology_type();
    $Log->reset_path() if $constructor_log;
    $Log->comment( 'Found ' . $self->_mcc_topology_type() . ' topology' );

    $Log->comment('...done');

    # now validate passed parameters.  throw exception if user passed in a
    # strange/invalid set of values in nodes.  (e.g., if there
    # are nodes listed not in the same MCC system, etc.)  this validation
    # occurs now because we now have all the DR Group discovery of nodes
    # and their partners done at this point.
    $Log->comment('Validating all user passed nodes...');

    $Log->redirect( path => $constructor_log ) if $constructor_log;
    $self->_validate_node_names_passed_by_user( node_names => $node_names );
    $Log->reset_path() if $constructor_log;

    $Log->comment('...ok');

    #rvalsan MCC-IP
    #check for MCC-IP
    $Log->comment("\nCheck for MCC-IP\n");
    my $cluster_mccip = $self->get_cluster( site => 'A' );
    my @nacl_nodes    = @{ $cluster_mccip->L_get_cluster_mcc_nodes() };
    my $nacl_node     = $nacl_nodes[0]->get_Node_component();
    my $ctld_response =
      $nacl_node->apiset( interface => 'CLI' )->metrocluster_show();
    my $ctld_response_1 = @{ $ctld_response->{parsed_output} }[0];

    if ( $ctld_response_1->{'configuration-type'} =~ /^IP-fabric$/ ) {
        $self->is_mccip(1);
        $Log->comment('MCC DEBUG: This is MCC IP Testbed...');
        $ENV{DISABLE_SHARED_STORAGE_DISK_STATE_VALIDATION} = 1;
    }
    else {
        $self->is_mccip(0);
    }

    # all container objects in the system have now been constructed.  do
    # final initial validations... :)
    $Log->comment('Validating initial disk states...');

    $Log->redirect( path => $constructor_log ) if $constructor_log;
    my $disaster_status = $self->L_get_disaster_status( include_HA => 1 );
    foreach my $dr_group ( @{ $self->_dr_groups() } ) {
        $dr_group->get_shared_storage()
          ->L_validate_disk_states( disaster_status => $disaster_status );
    }
    $Log->reset_path() if $constructor_log;

    $Log->comment('...ok');

    # initialize mcc_version member
    $self->_ontap_build_version( $self->_determine_ontap_version_info() );

    $Log->comment('Metrocluster system validation, part 2...');

    $Log->redirect( path => $constructor_log ) if $constructor_log;

    # can now continue check if system is in a good MCC state
    # TODO
    # We can recheck this perhaps with the 'real' state if we want to
    # in "bad start" mode
    $self->_validate_initial_metrocluster_state__part_2(
        disaster_status => $disaster_status );
    $Log->reset_path() if $constructor_log;
    $Log->comment('...ok');

    $Log->comment('Validating interconnect...');

    my $interconnect = $self->_interconnect();
    if ( !$interconnect->_no_interconnect() ) {
        $Log->redirect( path => $constructor_log ) if $constructor_log;

        # can now continue check if MCC interconnect is in good state
        $interconnect->validate_metrocluster_interconnect_state(
            disaster_status => $disaster_status );
        $Log->reset_path() if $constructor_log;
    }
    else {
        $Log->comment( 'Skipped interconnect validation since this is not an'
              . 'MCC 4-pack(s) test bed' );
    }
    $Log->comment('...ok');

    $Log->redirect( path => $constructor_log ) if $constructor_log;
    $self->_enable_debug_level_logs( disaster_status => $disaster_status );
    $self->version_is_valid();
    $Log->reset_path() if $constructor_log;

    # Enable AUSO on all clusters and nodes if user has not explicitly disabled
    # it using the DISABLE_AUSO param
    my $disable_auso = defined $ENV{DISABLE_AUSO} ? $ENV{DISABLE_AUSO} : 0;
    if (    not $disable_auso
        and $disaster_status->{DR_state} eq 'normal'
        and !$self->is_mccip )
    {
        $Log->comment(
            'Enabling AUSO config at node and cluster levels everywhere...');
        $Log->redirect( path => $constructor_log ) if $constructor_log;
        $self->modify_auso_config_state(
            scope => 'both',
            state => 'enable',
        );
        $Log->reset_path() if $constructor_log;
        $Log->comment('...done');
    }
    else {
        $Log->comment(
            'Found DISABLE_AUSO=1. Skip enabling AUSO config states...');
    }

    $Log->comment("\n\nHello from MetroCluster!\n\n");

    # show basic MCC system information
    $self->_log_some_mcc_system_info(
        disaster_status => $disaster_status,
        constructor_log => $constructor_log
    );

    return;
}

# this is a private method to check if mcc is enabled and in normal DR state
sub _validate_initial_metrocluster_state__part_1 {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my $handy_ci = $self->_site_A_metrocluster_component()->command_interface();

    my @mc_states = NACL::MTask::MCC::Utils->metrocluster_node_show(
        command_interface => $handy_ci,
        requested_fields  => [ 'configuration-state', 'dr-operation-state' ]
    );

    # check if metrocluster is enabled and in normal state
    foreach my $mc_state (@mc_states) {
        my $config_state = $mc_state->{'configuration_state'};
        my $dr_op_state  = $mc_state->{'dr_operation_state'};
        if (
            (
                    ( $config_state ne 'enabled' )
                and ( $config_state ne 'Enabled' )
                and    #pre metrocluster node show
                ( $config_state ne 'configured' )
                and ( $config_state ne 'Configured' )
                and    #TODO only allow correct vals
                ( $config_state ne '-' )
            )
            or

# temporary workaround due to ONTAP bug
#                                    ($dr_op_state  ne 'DRSOM_FSM_ST_NORMAL')) {
#TBD: reinstate proper check after ONTAP fix (see BURT 556067.)

#TBD: reinstate proper check after ONTAP fix for 'dr_operation_state' field (BURT 571681)
            (
                $dr_op_state !~
/^(?:DRSOM_FSM_ST_NORMAL|enabled|-|Normal|"Switchback completed"|"Switchback Recovery completed")$/i
            )
          )
        {
            my $error_msg =
                "ERROR: Metrocluster not enabled or not in normal state.\n"
              . "    STATE   : $config_state\n"
              . "    DR_STATE: $dr_op_state\n";
            if ( $self->_ignore_bad_state() ) {
                $Log->warn("IGNORE BAD STATE: $error_msg");
            }
            else {
                NACL::MTask::MCC::Exceptions::BaseException->throw($error_msg);
            }
        }
    }

    return;
}

# this is a private method to check if mcc is enabled and in normal DR state
sub _validate_initial_metrocluster_state__part_2 {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => { disaster_status => { type => HASHREF }, },
    );

    my $disaster_status = delete( $opts{disaster_status} );

    # validate node_scope DR_SOM states
    foreach my $site ( @{ $disaster_status->{live_sites} } ) {
        my $cluster = $self->get_cluster( site => $site );
        my $sites_nodes = $cluster->L_get_cluster_mcc_nodes();

        foreach my $mcc_node ( @{$sites_nodes} ) {

            # this will throw an exception if there are any inconsistencies.
            # allow for 'bad' startup states now
            $mcc_node->L_validate_DRSOM_FSM_state(
                dr_state => $disaster_status->{DR_state} );
        }
    }

    # Make sure all nodes are up
    foreach my $dr_group_id ( keys %{ $disaster_status->{'HA_states'} } ) {
        foreach
          my $site ( keys %{ $disaster_status->{'HA_states'}{$dr_group_id} } )
        {
            my $ha_state = $disaster_status->{'HA_states'}{$dr_group_id}{$site};
            if ( $ha_state->{'HA_state'} ne 'normal' ) {
                my $error =
                    "HA state for site '$site' is $ha_state->{'HA_state'}, "
                  . "should be normal during startup";

                if ( $self->_ignore_bad_state() ) {
                    $Log->warn($error);
                }
                else {
                    NACL::MTask::MCC::Exceptions::BaseException->throw($error);
                }
            }
        }

    }

    # do a workaround of volume DSID issue (BURT 526567)
    #unless (NACL::MTask::MCC::Utils->workaround_expired(
    #expiration_date => '2012/10/31', burt => '526567')) {
    #$self->workaround__prevent_volume_DSID_collisions();
    #}

    return;
}

=head2 validate_vservers()

Validate vservers for both clusters taking into account dr state.  If disaster
site is down it doesn't try to validate that site.


Validates vservers as follows:

  R=running, S=stopped, T=true, F=false
  admin-state/operational-state/config-lock
  normal (steady state): sync-source should be R/R/F, sync-dest should be R/S/T
  switchover: sync-* should be R/R/F
  heal aggrs: sync-* should be R/R/F
  heal roots: (survivor site): sync-* should be R/R/F
  heal roots: (disaster site): sync-* should be R/S/T


=over

=item Options

=item C<< disaster_status =E<gt> $mcc-E<gt>get_disaster_status() (optional) >>

(Optional) Pass in a previously generated disaster status structure if
you have one or a new one will be generated

=item C<< die_on_failure =E<gt> 0 | 1 (default = 1) >>

Whether to throw a NACL::MTask::MCC::Exceptions::Validation exception
on failure or to just warn

=back

=cut

sub validate_vservers {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            disaster_status        => { type  => HASHREF,    optional => 1 },
            die_on_failure         => { regex => qr/^[01]$/, default  => 1, },
            initialization_timeout => { type  => SCALAR,     default  => 360 },
            initialization_sleep   => { type  => SCALAR,     default  => 10 }
        },
    );

    my $disaster_status = $opts{disaster_status}
      || $self->get_disaster_status( include_HA => 1 );

    $Log->comment('Validate all data sync-* vservers...');

    #check states.  R=running, S=stopped, T=true, F=false
    #admin-state/operational-state/config-lock
    #steady state: sync-source should be R/R/F, sync-dest should be R/S/T
    #SO: sync-* should be R/R/F
    #HA: sync-* should be R/R/F
    #HR (survivor site): sync-* should be R/R/F
    #HR (disaster site): sync-* should be R/S/T

    my $err_objs;
    my @error_strings;
    foreach my $site ( @{ $disaster_status->{live_sites} } ) {
        $Log->comment("Validate all data sync-* vservers site $site");

        my ( $err, $err_str ) =
          $self->get_cluster( site => $site )->validate_vservers(
            disaster_status        => $disaster_status,
            die_on_failure         => 0,
            initialization_timeout => $opts{'initialization_timeout'},
            initialization_sleep   => $opts{'initialization_sleep'}
          );
        $err_objs->{$site} = $err;
        push @error_strings, @{$err_str} if ref $err_str eq 'ARRAY';
    }

    my $errors = join "\n", @error_strings;
    my $dr_state = $disaster_status->{DR_state};
    if ( scalar @error_strings ) {
        my $message = "vserver validation errors (dr state $dr_state)";
        $self->log_validation_event(
            type       => 'vserver',
            successful => 0,
            summary    => $message,
            results    => \@error_strings,
            objects => $err_objs,    #don't think we need these but add if we do
        );
        if ( $opts{die_on_failure} ) {
            NACL::MTask::MCC::Exceptions::Validation->throw(
                "vserver validation errors (dr state $dr_state): \n$errors");
        }
        else {
            $Log->warn(
                "vserver validation errors (dr state $dr_state): \n$errors");
        }
    }
    else {
        my $message =
          "vserver validation successful (dr state $dr_state):" . 'site:'
          . join ' and ', @{ $disaster_status->{live_sites} };
        $Log->comment($message);
        $self->log_validation_event(
            type       => 'vserver',
            successful => 1,
            summary    => $message,
        );
    }

    return;

}

=head2 validate_node()

Validate metrocluster node show for this cluster.  Validates node as follows:

  C=configured, U=unreachable, E=enabled
  configuration-state/dr-mirroring-state/dr-operation-state

  survivor cluster is C/E/dr state string
  disaster cluster is C/E/dr state string in steady state/normal
  disaster cluster is U/-/switched over ... in SO/HA/HR(unbooted)
  disaster cluster is C/E/waiting for switchback recovery in HR(booted)


=over

=item Options

=item C<< disaster_status =E<gt> $mcc-E<gt>get_disaster_status() (optional) >>

(Optional) Pass in a previously generated disaster status structure if
you have one or a new one will be generated

=item C<< die_on_failure =E<gt> 0 | 1 (default = 1) >>

Whether to throw a NACL::MTask::MCC::Exceptions::Validation exception
on failure or to just warn

=back

=cut

sub validate_node {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            disaster_status => { type  => HASHREF,    optional => 1 },
            die_on_failure  => { regex => qr/^[01]$/, default  => 1, },
        },
    );

    my $disaster_status = $opts{disaster_status}
      || $self->get_disaster_status( include_HA => 1 );

    $Log->comment('Validate metrocluster node show output...');

    #configuration-state/dr-mirroring-state/dr-operation-state
    #boils down to:
    #survivor cluster is C/E/dr state string
    #disaster cluster is C/E/dr state string in steady state/normal
    #disaster cluster is U/-/switched over ... in SO/HA/HR(unbooted)
    #disaster cluster is C/E/waiting for switchback recovery in HR(booted)
    my $err_objs;
    my @error_strings;
    foreach my $site ( @{ $disaster_status->{live_sites} } ) {
        $Log->comment("Validate metrocluster node show site '$site'");

        my ( $err, $err_str ) =
          $self->get_cluster( site => $site )->validate_node(
            disaster_status => $disaster_status,
            die_on_failure  => 0,
          );
        $err_objs->{$site} = $err;
        push @error_strings, @{$err_str} if ref $err_str eq 'ARRAY';
    }

    my $errors = join "\n", @error_strings;
    my $dr_state = $disaster_status->{DR_state};
    if ( scalar @error_strings ) {
        my $message = "node validation errors (dr state $dr_state)";
        $self->log_validation_event(
            type       => 'node',
            successful => 0,
            summary    => $message,
            results    => \@error_strings,
            objects => $err_objs,    #don't think we need these but add if we do
        );
        if ( $opts{die_on_failure} ) {
            NACL::MTask::MCC::Exceptions::Validation->throw(
                "node validation errors (dr state $dr_state): \n$errors");
        }
        else {
            $Log->warn(
                "node validation errors (dr state $dr_state): \n$errors");
        }
    }
    else {
        my $message =
          "node validation successful (dr state $dr_state):" . 'site:'
          . join ' and ', @{ $disaster_status->{live_sites} };
        $Log->comment($message);
        $self->log_validation_event(
            type       => 'node',
            successful => 1,
            summary    => $message,
        );
    }

    return;

}

=head2 validate_operation()

Validate metrocluster operation (show) for this cluster.  Validates operation
as follows:

  Anything in the Errors field is considered a failure.

  This method does NOT validate the operation and the state (That is
  done by DR_Coordinator::get_DR_state() method )

=over

=item Options

=item C<< disaster_status =E<gt> $mcc-E<gt>get_disaster_status() (optional) >>

(Optional) Pass in a previously generated disaster status structure if
you have one or a new one will be generated

=item C<< die_on_failure =E<gt> 0 | 1 (default = 1) >>

Whether to throw a NACL::MTask::MCC::Exceptions::Validation exception
on failure or to just warn

=back

=cut

sub validate_operation {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            disaster_status => { type  => HASHREF,    optional => 1 },
            die_on_failure  => { regex => qr/^[01]$/, default  => 1, },
        },
    );

    my $disaster_status = $opts{disaster_status}
      || $self->get_disaster_status( include_HA => 1 );

    $Log->comment('Validate metrocluster operation show errors...');

    my $err_objs;
    my @error_strings;
    foreach my $site ( @{ $disaster_status->{live_sites} } ) {
        $Log->comment("Validate operation show errors site $site");

        my $err_str = $self->get_cluster( site => $site )->validate_operation(
            disaster_status => $disaster_status,
            die_on_failure  => 0,
        );
        $err_objs->{$site} = $err_str;
        push @error_strings, @{$err_str} if ref $err_str eq 'ARRAY';
    }

    my $errors = join "\n", @error_strings;
    my $dr_state = $disaster_status->{DR_state};

    #TODO - if completed/warning then it's OK to see something here but
    #we might still want to die/warn.  If it's NOT then we shouldn't
    #see anything here
    #
    #Also, if it is completed/manual then potentially we want to
    #throw a special exception here rather than the regular validation
    #one.
    #
    if ( scalar @error_strings ) {
        my $message = "operation validation errors (dr state $dr_state)";
        $self->log_validation_event(
            type       => 'operation',
            successful => 0,
            summary    => $message,
            results    => \@error_strings,
            objects => $err_objs,    #don't think we need these but add if we do
        );

        # Catch and ignore SO completion warnings due to un-mirrored aggrs
        # BURT 987929
        my $unm_warning = 'is not mirrored and its only plex "plex0" uses '
          . 'storage from the remote pool';

        # Catch and ignore Heal Root Aggrs completion warnings due to
        # aggrs resyncing
        # BURT 997498
        my $HR_aggr_resync_warning = 'Some aggregates are still resyncing';

        # Ignore SO completed with manual recovery needed warnings due to
        # Un-mirrored aggr offline and VServer offline - BURT 1003960
        my $unm_aggr_offline_warning = 'The following switched over aggregates '
          . 'are missing or not online';
        my $vs_offline_warning = 'The following Vservers might have switched '
          . 'over volumes that are missing or not mounted';

        # Catch and ignore SO completion warnings due to offlined plex
        # BURT 106671
        my $plex_offline_warning =
            'is mirrored and its only online plex "plex0" '
          . 'uses storage from the remote pool';

        if ( $errors =~ /$unm_aggr_offline_warning|$vs_offline_warning/ ) {
            $Log->comment( 'Ignoring switchover completion warnings due to '
                  . 'Un-mirrored aggr offline and VServer offline' );
        }
        elsif ( $errors =~ /$unm_warning/ ) {
            $Log->comment( 'Ignoring SO completion warnings due to un-mirrored '
                  . 'aggregates' );
        }
        elsif ( $errors =~ /$HR_aggr_resync_warning/ ) {
            $Log->comment( 'Ignoring Heal Roots completion warnings due to '
                  . 'aggregate resyncing in progress' );
        }
        elsif ( $errors =~ /$plex_offline_warning/ ) {
            $Log->comment(
                'Ignoring SO completion warnings due to plex offlined aggregate'
            );
        }
        elsif ( $opts{die_on_failure} ) {
            NACL::MTask::MCC::Exceptions::Validation->throw(
                "operation validation errors (dr state $dr_state): \n$errors");
        }
        else {
            $Log->warn(
                "operation validation errors (dr state $dr_state): \n$errors");
        }
    }
    else {
        my $message =
          "operation validation successful (dr state $dr_state):" . 'site:'
          . join ' and ', @{ $disaster_status->{live_sites} };
        $Log->comment($message);
        $self->log_validation_event(
            type       => 'operation',
            successful => 1,
            summary    => $message,
        );
    }

    return;

}

#internal method to log any validation done for generating a report later

sub log_validation_event {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            type       => { type  => SCALAR },
            successful => { regex => qr/^[01]$/ },
            summary    => { type  => SCALAR, optional => 1 },
            results    => { type  => ARRAYREF, optional => 1 },
            objects    => { type  => ARRAYREF | HASHREF, optional => 1 },
        },
    );

    my $low_level_logging =
      defined $ENV{LOG_DETAILED_MCC_VALIDATION}
      ? $ENV{LOG_DETAILED_MCC_VALIDATION}
      : 0;
    if ( !$low_level_logging ) {
        delete $opts{objects};
    }

    my $time = join q{.}, gettimeofday();

    my $event = \%opts;

    $self->{_validation_log}->{ $opts{type} }->{successful}
      ->{ $opts{successful} }->{$time} = $event;
    return;
}

sub validation_was_successful {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => { type => { type => SCALAR }, }
    );

    return $self->{'_validation_log'}{ $opts{'type'} }{'successful'};
}

=head2 validation_report()

This should be run at the end of EVERY test.  It will dump the validations
that have been run as well as list the validations that were NOT performed.

In some cases certain tests may not exercise the functionality so it won't
be validated but otherwise the script should be modified to make sure the
validation is performed.

=cut

sub validation_report {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    #TODO - allow for required/skipped test types

    $Log->comment( "validation report:\n" . Dumper $self->{_validation_log} );

    my %types = (
        'vserver' =>
          'validate vserver operational state, admin state and config lock',
        'operation' => 'any metrocluster operation show error field output is '
          . 'considered a failure',
        'version_verify' => 'validate firmware versions',
        'network_interface_show' =>
          'validate that the network interfaces are in the '
          . 'appropriate state',
        'plex'                  => 'plex validation',
        'aggregate'             => 'validate aggregate states',
        'cluster_show'          => 'validate that clusters are healthy',
        'storage_failover_show' => 'validate that storage failover is enabled'
    );

    #complain about unvalidated types
    my @not_validated =
      grep { !defined $self->{_validation_log}->{$_} } keys %types;

    my @missing;
    foreach my $missing (@not_validated) {
        push @missing, "$missing NOT validated: $types{$missing}";
    }

    if ( scalar @not_validated ) {
        my $msg = "Not all validations were run in this test!\n" . join "\n",
          @missing;
        if ( $self->_lifecycle_was_run() ) {
            $Log->warn($msg);
        }
        else {
            $Log->comment($msg);
        }
    }

    return;
}

#look to see if a lifecycle was run
sub _lifecycle_was_run {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    #we should have some semantic data in validation_log and do this "correctly"

    #horrible horrible hackery

    my $log = Dumper $self->{_validation_log};

    #assume we've done a lifecycle if we have messages somewhere saying
    #we've been in these states
    if (   ( $log =~ /normal/ )
        && ( $log =~ /switchover_completed/ )
        && ( $log =~ /heal_aggrs_completed/ )
        && ( $log =~ /heal_roots_completed/ ) )
    {
        return 1;
    }
    else {
        return;
    }

}

#private method to enable debug level logging
sub _enable_debug_level_logs {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => { disaster_status => { type => HASHREF, optional => 1, }, },
    );

    my $disaster_status = delete( $opts{disaster_status} );

    unless ( Tharn::param('DO_NOT_ENABLE_MCC_DEBUG_LOGS') ) {

        $Log->comment( 'Enabling mcc debug logging '
              . '(disable with DO_NOT_ENABLE_MCC_DEBUG_LOGS=1) ...' );

        # enable debug level logging for all nodes
        # TODO.  we should try to run this again when the node is up...
        # set up somethign in the node object saying whether it has
        # been initialized, if not then do some of these basic things.
        # for now we can skip if it's down...
        my @sites =
          ref $disaster_status
          ? @{ $disaster_status->{live_sites} }
          : qw/ A B /;
        foreach my $site (@sites) {
            my $cluster = $self->get_cluster( site => $site );
            my $sites_nodes = $cluster->L_get_cluster_mcc_nodes();

            $self->has_uichange_scaffold();

            foreach my $mcc_node ( @{$sites_nodes} ) {
                my $ci = $mcc_node->get_Node_component()->command_interface();
                foreach my $module (qw (tables:metro* mcc* metro* tables:mcc*))
                {
                    try {
                        my $logger = NACL::C::LoggerMgwdLog->new(
                            command_interface => $ci,
                            module            => $module,
                            node => $mcc_node->get_Node_component()->node()
                        );
                        $logger->modify( 'level' => 'debug' );
                    }
                    catch NATE::BaseException with {
                        $Log->comment(
                            "Error setting debug level logging, ignoring:"
                              . shift->text() );
                    }
                }
            }
        }
        $Log->comment('done...');
    }
    else {
        $Log->comment('NOT enabling mcc debug logging...');
    }

    return;
}

=head2 workaround__prevent_volume_DSID_collisions()

    # can be called on an instance . . .
    $mcc->workaround__prevent_volume_DSID_collisions();

    # or on the class by providing a node name from each cluster
    NACL::MTask::MCC::Metrocluster->workaround__prevent_volume_DSID_collisions(
                   all_nodes => [ 'dpg-mcc-3270-01-a2', 'dpg-mcc-3270-01-b1',
                                  'dpg-mcc-3270-01-a1', 'dpg-mcc-3270-01-b2']);

Notes:

workaround of volume DSID issue (BURT 526567)
see test lib BURT 615343 for an explanation of the logic in this temp method

most of the complexity of this method is so we don't change anything that
doesn't need to be changed.


MSID notes:

now also adding a quickie fix for MSID ranges.  when DSID ranges are being set
as opposed to just being checked (because this method can be called at any
time), the MSID ranges will be set to the same ranges being used for the DSIDs
on that cluster with the exception that the highest bit for MSIDs will be 1 as
opposed to 0 for DSIDs (32 bit values.)  E.g., DSID = 0x00 010 00 00 would be
matched with MSID = 0x10 01 00 00.

new on 9/26/2012: must allow that volumes that are mirrored are allowed to
have the same DSIDs across sites.  We will take a shortcut here and say that
if the CRS_Vserver is found, then DSID ranges have already been applied, and
just to make sure, we will look at the vol DSIDs for the CRS_Vserver at each
site and compare them.  If they are in the different ranges, we will assume
all is well and taken care of in DSID range settings across the clusters.

=cut

use NACL::CS::Volume;    # remove when workaround removed.

sub workaround__prevent_volume_DSID_collisions {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $object_or_pkg = shift;

    # validate opts
    my %opts = validate_with(
        params => \@_,
        spec   => { all_nodes => { type => ARRAYREF, optional => 1, }, },
    );

    # DSID upper range value needs to be big, but high order bit must be 0
    # MSID will be DSID with high order bit = 1
    my $small_dsid = 0x00000400;    # 2^10  = 1024
    my $big_dsid   = 0x00100000;    # 2^20  = 1024 * 1024 =     1,048,576
         # small_msid would then be 0x10000400 = 2^31 +2^10  = 2,147,484,672
         # big_msid would then be   0x10100000 = 2^31 +2^20  = 2,148,532,224

    # this can be any 1 node on site A and 1 node on site B
    # population of this hash depends on how method was called.
    my %node_components = ( A => undef, B => undef );
    if ( ref($object_or_pkg) ) {

        # called on an instance.
        my $mcc = $object_or_pkg;

        my ($dr_group) = values( %{ $mcc->get_DR_groups() } );    # arbitrary
        my $nodes_by_role = $dr_group->get_nodes_by_role();
        $node_components{A} = $nodes_by_role->{A1}->get_Node_component();
        $node_components{B} = $nodes_by_role->{B1}->get_Node_component();
    }
    elsif ( exists( $opts{all_nodes} ) ) {

        # called on the class.
        my %node_component_vs_cluster_name = ();
        foreach my $node_name ( @{ $opts{all_nodes} } ) {
            my $host = Tharn::host($node_name);
            my $node_component = NACL::C::SystemNode->new( hostrec => $host );

            my $ClusterIdentity_state = NACL::CS::ClusterIdentity->fetch(
                command_interface => $node_component,
                requested_fields  => ['name']
            );
            my $cluster_name = $ClusterIdentity_state->name();

            $node_component_vs_cluster_name{$cluster_name} = $node_component;

            if ( scalar( keys(%node_component_vs_cluster_name) ) > 1 ) {

                # we've now gotten an arbitrary node in each cluster
                last;
            }
        }

        if ( scalar( keys(%node_component_vs_cluster_name) ) == 2 ) {

            # we don't really care which is at "A" vs "B" here, just that they
            # are one node at each site.
            ( $node_components{A}, $node_components{B} ) =
              values(%node_component_vs_cluster_name);
        }
    }

    unless ( defined( $node_components{A} ) && defined( $node_components{B} ) )
    {
        NACL::MTask::MCC::Exceptions::BaseException->throw(
            "Could not determine or create Node components!");
    }

    my @vol_states = NACL::CS::Volume->fetch(
        command_interface => $node_components{A},
        requested_fields  => [ 'dsid', 'aggregate', 'vserver', ],
    );

    my @A_dsids = ();

    # if vserver CRS_Vserver exists already, then we should have already been
    # through the DSID range assignment logic, and all we are going to do now
    # is look at the DSIDs associated with the CRS_Vserver vols as a double
    # check.  this avoids the issue of volumes that are mirrored across sites
    # having the same DSIDs (which they should).
    my $site_A_CRS_Vserver_found =
      List::MoreUtils::any { $_->vserver() eq 'CRS_Vserver' } @vol_states;
    if ($site_A_CRS_Vserver_found) {
        $Log->comment( "vserver CRS_Vserver exists.  We assume then that \n"
              . "DSID ranges have already been negotiated in prior\n"
              . "MCC system build mechanisms, and will only do a\n"
              . "cursory check of vol DSIDs associated with the\n"
              . "CRS_Vserver at each site." );

        # filter on vols in CRS_Vserver
        my @CRS_Vserver_vol_states =
          grep { $_->vserver() eq 'CRS_Vserver' } @vol_states;

        # get those vol's DSIDs
        @A_dsids = map { $_->dsid() } @CRS_Vserver_vol_states;
        $Log->comment( 'Existing site A CRS_Vserver volume DSIDs: '
              . join( q/,/, @A_dsids ) );
    }
    else {
        # here we assume we need to check all non-root DSIDs in the system
        my @root_aggr_states = NACL::CS::Aggregate->fetch(
            command_interface => $node_components{A},
            filter            => { 'root' => 'true' },
            requested_fields  => [ 'aggregate', ],
        );

        # grab each non-root numeric DSID
        foreach my $vol_state (@vol_states) {
            if (
                List::MoreUtils::any {
                    $_->aggregate() eq $vol_state->aggregate();
                }
                @root_aggr_states
              )
            {
                # ignore this vol.  it is in a root aggr.
            }
            else {
                push( @A_dsids, $vol_state->dsid() );
            }
        }
        $Log->comment( 'Existing site A (non-root-aggr) volume DSIDs: '
              . join( q/,/, @A_dsids ) );
    }

    # booleans
    my $site_A_has_low_dsids = scalar(@A_dsids)
      && List::MoreUtils::any { $_ < $big_dsid } @A_dsids;
    my $site_A_has_high_dsids = scalar(@A_dsids)
      && List::MoreUtils::any { $_ >= $big_dsid } @A_dsids;

    # determine current setting for next DSID at site A
    my $result =
      $node_components{A}->apiset( interface => 'CLI' )
      ->execute_command(
        command => 'set -confirmations off -rows 0 -privilege test; '
          . 'debug smdb table nextIdTable show -fields dsid' );
    my ($site_A_next_dsid) = ( $result =~ /(\d+)/ );
    $Log->comment("Current next volume DSID for site A: $site_A_next_dsid");
    my $site_A_next_dsid_is_low = ( $site_A_next_dsid < $big_dsid );

    # and for site B...
    @vol_states = NACL::CS::Volume->fetch(
        command_interface => $node_components{B},
        requested_fields  => [ 'dsid', 'aggregate', ],
    );

    # (see site A process comments above)
    my @B_dsids = ();
    if ($site_A_CRS_Vserver_found) {

        # we better find a CRS_Vserver at site B as well...
        unless ( List::MoreUtils::any { $_->vserver() eq 'CRS_Vserver' }
            @vol_states )
        {
            NACL::MTask::MCC::Exceptions::BaseException->throw(
                "vserver 'CRS_Vserver' found at site A, but not at site B!");
        }

        # filter on vols in CRS_Vserver
        my @CRS_Vserver_vol_states =
          grep { $_->vserver() eq 'CRS_Vserver' } @vol_states;

        # get those vol's DSIDs
        @B_dsids = map { $_->dsid() } @CRS_Vserver_vol_states;
        $Log->comment( 'Existing site B CRS_Vserver volume DSIDs: '
              . join( q/,/, @B_dsids ) );
    }
    else {
        my @root_aggr_states = NACL::CS::Aggregate->fetch(
            command_interface => $node_components{B},
            filter            => { 'root' => 'true' },
            requested_fields  => [ 'aggregate', ],
        );

        # grab each non-root numeric DSID
        foreach my $vol_state (@vol_states) {
            if (
                List::MoreUtils::any {
                    $_->aggregate() eq $vol_state->aggregate();
                }
                @root_aggr_states
              )
            {
                # ignore this vol.  it is in a root aggr.
            }
            else {
                push( @B_dsids, $vol_state->dsid() );
            }
        }
        $Log->comment( 'Existing site B (non-root-aggr) volume DSIDs: '
              . join( q/,/, @B_dsids ) );
    }

    # booleans
    my $site_B_has_low_dsids = scalar(@B_dsids)
      && List::MoreUtils::any { $_ < $big_dsid } @B_dsids;
    my $site_B_has_high_dsids = scalar(@B_dsids)
      && List::MoreUtils::any { $_ >= $big_dsid } @B_dsids;

    # determine current setting for next DSID at site A
    $result =
      $node_components{B}->apiset( interface => 'CLI' )
      ->execute_command(
        command => 'set -confirmations off -rows 0 -privilege test; '
          . 'debug smdb table nextIdTable show -fields dsid' );
    my ($site_B_next_dsid) = ( $result =~ /(\d+)/ );
    $Log->comment("Current next volume DSID for site B: $site_B_next_dsid");
    my $site_B_next_dsid_is_low = ( $site_B_next_dsid < $big_dsid );

    # check for problem cases first
    if (   ( $site_A_has_low_dsids && $site_A_has_high_dsids )
        || ( $site_A_has_low_dsids  && $site_B_has_low_dsids )
        || ( $site_A_has_high_dsids && $site_B_has_high_dsids )
        || ( $site_B_has_low_dsids  && $site_B_has_high_dsids ) )
    {
        NACL::MTask::MCC::Exceptions::BaseException->throw(
                "WARNING: Clashing volume DSID ranges exist in this MCC\n"
              . "         system.  They must be cleaned up before continuing."
        );
    }

    # all the other possibilities are okay, but the next DSID at each site
    # may need to be tweaked...
    my $new_dsid = { 'A' => undef, 'B' => undef };

    # let's now make sure the next DSIDs at each site are in the same
    # ranges as any existing volume DSIDs at that site.
    # much of this logic is there so that we don't change anything unless
    # we need to.
    if ($site_A_has_low_dsids) {

        # next site A DSID better be low...
        if ( !$site_A_next_dsid_is_low ) {
            $new_dsid->{A} = List::Util::max( @A_dsids, $small_dsid ) + 1;
        }

        # ...and next site B DSID better be high
        if ($site_B_next_dsid_is_low) {
            $new_dsid->{B} = List::Util::max( @B_dsids, $big_dsid ) + 1;
        }
    }
    elsif ($site_A_has_high_dsids) {

        # next site A DSID better be high...
        if ($site_A_next_dsid_is_low) {
            $new_dsid->{A} = List::Util::max( @A_dsids, $big_dsid ) + 1;
        }

        # ...and next site B DSID better be low
        if ( !$site_B_next_dsid_is_low ) {
            $new_dsid->{B} = List::Util::max( @B_dsids, $small_dsid ) + 1;
        }

        # only get to here if site A had no allocated volume DSIDs...
    }
    elsif ($site_B_has_low_dsids) {

        # next site B DSID better be low...
        if ( !$site_B_next_dsid_is_low ) {
            $new_dsid->{B} = List::Util::max( @B_dsids, $small_dsid ) + 1;
        }

        # ...and next site A DSID better be high
        if ($site_A_next_dsid_is_low) {
            $new_dsid->{A} = $big_dsid;
        }
    }
    elsif ($site_B_has_high_dsids) {

        # next site B DSID better be high...
        if ($site_B_next_dsid_is_low) {
            $new_dsid->{B} = List::Util::max( @B_dsids, $big_dsid ) + 1;
        }

        # ...and next site A DSID better be low
        if ( !$site_A_next_dsid_is_low ) {
            $new_dsid->{A} = $small_dsid;
        }

        # only get here if neither site had allocated volume DSIDs
    }
    else {
        if ( $site_A_next_dsid_is_low && $site_B_next_dsid_is_low ) {

            # arbitrarily pick B site to have high volume DSIDs
            $new_dsid->{B} = $big_dsid;
        }
        elsif ( ( !$site_A_next_dsid_is_low ) && ( !$site_B_next_dsid_is_low ) )
        {
            $new_dsid->{A} = $small_dsid;
        }
    }

    # only change next DSID (and MSID with it) setting if needed
    foreach my $site (qw/ A B /) {
        if ( defined( $new_dsid->{$site} ) ) {
            my $dsid = $new_dsid->{$site};

            # highest order bit (32 bit) must be 1 for MSIDs.  will OR this in
            my $msid = $dsid | 0x80000000;
            $Log->warn( "Setting next site $site DSID/MSID to be "
                  . sprintf( "0x%x / 0x%x", $dsid, $msid )
                  . ' !' );
            $node_components{$site}->apiset( interface => 'CLI' )
              ->execute_command(
                command => "set -confirmations off -rows 0 -privilege test; "
                  . "debug smdb table "
                  . "nextIdTable modify -dsid $dsid -msid $msid" );
        }
    }

    return;
}

# this is a private method to determine which build is installed on mcc testbed
sub _determine_ontap_version_info {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;
    my $ci   = $self->_site_A_metrocluster_component()->command_interface();
    my $version_info = {};
    $version_info->{branch} =
      $ci->get_version_manager()
      ->get_version_attribute( attribute => 'branch' );
    $Log->comment( 'ONTAP BUILD INSTALLED: ' . $version_info->{branch} );

    return $version_info;
}

=head2 has_uichange_scaffold()

   $mcc->has_uichange_scaffold();

A replacement for the has_uichange() calls. Does a Metrocluster site A
connection refresh.

=cut

sub has_uichange_scaffold {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    return;

    $self->_site_A_metrocluster_component()->command_interface()
      ->refresh_command_interface();
}

=head2 has_uichange

    my $has_change = $mcc->has_uichange(
                         uichange => "has_this_ui_change_string");


Returns a boolean indicating if the ontap build has the passed
UI change string

=cut

sub has_uichange {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    # validate opts
    my %opts = validate_with(
        params => \@_,
        spec   => { uichange => { type => SCALAR }, },
    );

    my $ci = $self->_site_A_metrocluster_component()->command_interface();
    my $has_change;
    try {
        $has_change =
          $ci->get_version_manager()
          ->has_uichange( uichange => $opts{uichange} );
    }
    catch NATE::BaseException with {    #TODO should we just let it throw this?
        $Log->comment(
            "problem with getting UI change strings ($opts{uichange})...\n");
    };

    return $has_change;
}

=head2 workaround_expired

    if (! $mcc->workaround_expired(
           uichange => "has_this_ui_change_string",
           burt => 123456,
       )) {
        ... do this workaround ...
    }


Returns a boolean indicating if the workaround has expired.  With the new
way workarounds are done, this means that a particular UI change string
exists in the ontap build.

If the ui change string exists in ontap, we are to no longer execute
the workaround.  Thus, this method returns true if the string is present
and false if the string is not present.

There may be a more complicated evaluation in the future.

=cut

sub workaround_expired {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    #TODO.  We MAY allow this to be a static method in the future (the older
    #workaround expired based on dates was static)
    #however, since we need ontap info it may not make sense (we'd
    #only need it if a script needed it outside of MCC usage really)

    my $self = shift;

    # validate opts
    my %opts = validate_with(
        params => \@_,
        spec   => {
            uichange   => { type  => SCALAR },
            burt       => { regex => qr/^\d\d\d\d\d\d$/ },
            workaround => { regex => qr/^[01]$/, default => 1 },
        },
    );

    my $has_uichange = $self->has_uichange( uichange => $opts{uichange} );

    my $force = Tharn::param('FORCE_MCC_WORKAROUND_OFF');
    if ($force) {
        my %force = map { $_ => 1 } split /,/, $force;
        $has_uichange ||= $force{ $opts{uichange} };
    }

    if ( !$has_uichange ) {
        $Log->comment("\n");
        my $workaround =
          $opts{workaround}
          ? "Workaround for BURT $opts{burt} being used "
          : "Change for BURT $opts{burt} NOT being used - not in this build ";
        $Log->comment( $workaround
              . "($opts{uichange} UI change string not present) ..." );
        $Log->comment("\n");
        return 0;
    }
    else {
        my $workaround =
          $opts{workaround}
          ? "Workaround for BURT $opts{burt} has expired! "
          : "Change for BURT $opts{burt} being used now - in this build! ";
        $Log->comment(
            $workaround . "($opts{uichange} UI change string present) ..." );
        return 1;
    }

}

=head2 get_ontap_version_info()

    my $build = $mcc->get_ontap_version_info();

    $build->{branch} has the branch information

This is a public method use to find which ONTAP build is installed over
hardware. Method returns the hashreference which contains build installed over
mcc hardware

=cut

sub get_ontap_version_info {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;
    return ( $self->_ontap_build_version );
}

=head2 get_Metrocluster_component()

    my $site_A_metrocluster_component =
                                 $mcc->get_Metrocluster_component(site => 'A');

This is a public utility accessor for both sites' Metrocluster components.
Returns a reference to the NACL::C::Metrocluster component object associated
with site A or site B.

=over

=item Options

=over

=item C<< site => 'A'|'B' >>

(Required) Specifies which site's cluster is desired.

=back

=back

=cut

sub get_Metrocluster_component {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    # validate opts
    my %opts = validate_with(
        params => \@_,
        spec   => { site => { regex => qr/^[AB]$/ }, },
    );

    my $metrocluster_component =
      ( $opts{site} =~ /A/ )
      ? $self->_site_A_metrocluster_component()
      : $self->_site_B_metrocluster_component();

    return $metrocluster_component;
}

=head2 get_cluster()

    my $cluster_A = $mcc->get_cluster(site => 'A');

Gets the NACL::MTask::MCC::Cluster object associated with specified site.

=over

=item Options

=over

=item C<< site => 'A'|'B' >>

(Required) Specifies which site's cluster is desired.

=back

=back

=cut

sub get_cluster {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    # validate opts
    my %opts = validate_with(
        params => \@_,
        spec   => { site => { regex => qr/^[AB]$/ }, },
    );

    return ( $opts{site} =~ /A/ ) ? $self->_cluster_A() : $self->_cluster_B();
}

=head2 get_disaster_status()

    my $disaster_status = $mcc->get_disaster_status();

    my $disaster_status = $mcc->get_disaster_status(include_HA => 1);

This method returns DR state information about the overall Metrocluster system.
Optionally it will also return HA state information for all available HA groups.

=over

=item Options

=over

=item C<< include_HA => 0|1  (default = 0) >>

(Optional) Includes HA states in all the HA Groups in the system if true.

=item C<< dr_group => $dr_group  >>

(Optional) Specify a particular DR Group status only if desired.  This is only
useful if include_HA => 1 is also passed, since all DR Groups have the same
DR status.  But this may be passed to filter the HA status checks to only a
particular DR Group.

=back

=back

The data structure returned has the following format:

    {
        DR_state      => $dr_state,
        live_sites    => \@queryable_sites,
        survivor_site => $survivor_site,
        disaster_site => $disaster_site,
        HA_states     => {
                $DR_group_id => {
## BEWARE that a site will not appear here if it is not in live_sites array ###
#                       A => {                                                #
#                               HA_state      => $ha_state,                   #
#                               survivor_node => $survivor_node,              #
#                               disaster_node => $disaster_node,              #
#                            },                                               #
#                       B => {                                                #
#                               HA_state      => $ha_state,                   #
#                               survivor_node => $survivor_node,              #
#                               disaster_node => $disaster_node,              #
#                            },                                               #
###############################################################################
                                },
                $DR_group_id => {
                        .
                        .
                        .
                                },
                         }
    }

where

 * DR_state is one of the values listed in
   NACL::MTask::MCC::DR_Coordinator::get_DR_state()
 * live_sites is a list containing 'A' and/or 'B' (note that this list can
   contain both sites in both 'normal' and 'heal_roots_completed' DR states)
 * survivor_site is undef if DR_state is 'normal'
 * disaster_site is undef if DR_state is 'normal'
 * HA_state is one of the values listed in
   NACL::MTask::MCC::HA_Group::get_HA_state()
 * survivor_node and disaster_node are NACL::MTask::MCC::Node objects, or are
   undef if HA_state is 'normal'

=cut

sub get_disaster_status {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    # validate opts
    my %opts = validate_with(
        params => \@_,
        spec   => {
            include_HA => {
                regex   => qr/^[01]$/,
                default => 0,
            },
            dr_group => {
                isa      => 'NACL::MTask::MCC::DR_Group',
                optional => 1,
            },
        },
    );

    return $self->L_get_disaster_status(%opts);
}

=head2 L_get_disaster_status()

This is the engine for the get_disaster_status method.  It is seperate from the
get_disaster_status() because a non-public option is available here.

=over

=item Options

See get_disaster_status() options, and add the following option(s) as well:

=over

=item C<< known_live_sites => \@live_sites >>

(Optional) @live_sites may include 'A' and/or 'B'.  This option should only ever
be used when the context of the call involves a known state of the queryability
of the two sites.

For example, at MCC object initialization, the system must be in a normal
state, which presupposes both sites A and B are live and queryable.  This option
should NEVER be used except when these sites' states are absolutely known.

This option allows for faster completion as it skips the subtests that "ping"
the two sites.

=item C<< timeout =E<gt> $timeout_secs >>

The amount of time to allow for the metrocluster to move into a known
end state.

=back

=back

=cut

sub L_get_disaster_status {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    # validate opts
    my %opts = validate_with(
        params => \@_,
        spec   => {
            include_HA => {
                regex   => qr/^[01]$/,
                default => 0,
            },
            dr_group => {
                isa      => 'NACL::MTask::MCC::DR_Group',
                optional => 1,
            },
            known_live_sites => {
                type     => ARRAYREF,
                optional => 1,
            },
            timeout => {
                type    => SCALAR,
                regex   => qr/^\d+$/,
                default => ( 5 * 60 )
            }
        },
    );

    my $live_sites_ref =
      ( defined( $opts{known_live_sites} ) )
      ? $opts{known_live_sites}
      : $self->L_determine_live_sites();
    my @live_sites = @{$live_sites_ref};

    my $survivor_site = undef;
    my $disaster_site = undef;
    my $dr_state      = undef;

    my $timeout_delay = $opts{'timeout'};
    my $timeout       = time + $timeout_delay;
    my $wait_time     = 30;

    foreach my $site (@live_sites) {
        my $cluster = $self->get_cluster( site => $site );
        $cluster->get_Metrocluster_component()->command_interface()
          ->refresh_command_interface( no_version_change => 1 );
    }

    my $dr_states = {};

    while ( time < $timeout ) {

        foreach my $site (@live_sites) {
            my $cluster = $self->get_cluster( site => $site );
            $dr_states->{$site} =
              $cluster->get_DR_Coordinator()->get_DR_state();
        }

        ( $dr_state, $survivor_site, $disaster_site ) =
          $self->_is_dr_state_well_defined($dr_states);

        if ( !defined $dr_state ) {
            $Log->comment(
                    "Metrocluster in odd but likely temporary transition state."
                  . "Rechecking state and waiting for "
                  . ( $timeout - time )
                  . " more seconds before giving up and failing.\n"
                  . NACL::MTask::MCC::Utils->dump( %{$dr_states} )
                  . "\n\n" );
            $Log->comment(
                "rechecking what sites are live, currently:" . join ',',
                @live_sites );
            my $start_check_time   = time;
            my $new_live_sites_ref = $self->L_determine_live_sites();
            if ( scalar @live_sites != scalar @{$new_live_sites_ref} ) {
                $live_sites_ref = $new_live_sites_ref;
                @live_sites     = @{$live_sites_ref};
                $Log->comment( "live sites changed! now:" . join ',',
                    @live_sites );
                foreach my $site (@live_sites) {
                    my $cluster = $self->get_cluster( site => $site );
                    $cluster->get_Metrocluster_component()->command_interface()
                      ->refresh_command_interface( no_version_change => 1 );
                }
            }
            else {
                # Duplicate code from above, but can't figure out
                # a better approach right now. --SN
                foreach my $site (@live_sites) {
                    my $cluster = $self->get_cluster( site => $site );
                    $cluster->get_Metrocluster_component()->command_interface()
                      ->refresh_command_interface( no_version_change => 1 );
                }
                my $wait_left = $wait_time - ( time - $start_check_time );
                if ( $wait_left > 0 ) {
                    $Log->comment("sleeping for $wait_left till next check...");
                    Tharn::snooze($wait_left);
                }
            }
        }
        else {
            last;
        }
    }

    unless ( defined($dr_state) ) {

        # these dr states are not the ones we are looking for...
        NACL::MTask::MCC::Exceptions::BaseException->throw(
                'ERROR: Invalid DR states found after retrying for '
              . "$timeout_delay secs:\n"
              . NACL::MTask::MCC::Utils->dump($dr_states)
              . "\n\n" );
    }

    my $disaster_status = {
        DR_state      => $dr_state,
        live_sites    => \@live_sites,
        survivor_site => $survivor_site,
        disaster_site => $disaster_site,
    };

    # add HA disaster info if requested
    if ( $opts{include_HA} ) {
        $disaster_status->{HA_states} = {};
        my $specified_dr_group = delete( $opts{dr_group} );
        my $dr_groups =
          defined($specified_dr_group)
          ? { $specified_dr_group->get_id() => $specified_dr_group }
          : $self->get_DR_groups();
        foreach my $dr_group_id ( keys( %{$dr_groups} ) ) {
            my $dr_group = $dr_groups->{$dr_group_id};
            $disaster_status->{HA_states}->{$dr_group_id} = {};
            foreach my $site (@live_sites) {
                my $ha_group = $dr_group->get_HA_groups()->{$site};
                $disaster_status->{HA_states}->{$dr_group_id}->{$site} =
                  $ha_group->L_get_HA_state();
            }
        }
    }

    return $disaster_status;
}

##
## __is_dr_state_well_defined()
##
## Arguments: $dr_states
##
## Returns: $dr_state, $survivor_site, $disaster_site
##
## Private method. Determines if the DR states indicate that the
## metrocluster is a well-defined terminal state.
##

sub _is_dr_state_well_defined {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;
    my ($dr_states) = @_;

    my @live_sites = keys %{$dr_states};

    my ( $dr_state, $survivor_site, $disaster_site );

    # specific conditions for us to say we have a well defined dr_state
    if ( scalar(@live_sites) == 2 ) {
        if (
            scalar( grep { $_ =~ /normal|check/ } values( %{$dr_states} ) ) ==
            2 )
        {

            # 2 live sites claim "normal".  it is normal.
            return ( 'normal', undef, undef );
        }
        else {

            if ( !$self->is_mccip ) {

                # the disaster site can be live @ heal_roots_completed
                foreach my $site (@live_sites) {
                    if ( $dr_states->{$site} eq 'heal_roots_completed' ) {
                        $dr_state      = 'heal_roots_completed';
                        $survivor_site = $site;
                        $disaster_site = ( $site eq 'A' ) ? 'B' : 'A';
                        return ( $dr_state, $survivor_site, $disaster_site );
                    }
                }
            }
            else {
         # rvalsan: MCC-IP: the disaster site can be live @ heal_roots_completed
         # as well as heal_aggrs_completed for MCC-IP
                foreach my $site (@live_sites) {
                    if (   ( $dr_states->{$site} eq 'heal_roots_completed' )
                        || ( $dr_states->{$site} eq 'heal_aggrs_completed' )
                        || ( $dr_states->{$site} eq 'switchover_completed' ) )
                    {
                        if ( $dr_states->{$site} eq 'heal_roots_completed' ) {
                            $dr_state = 'heal_roots_completed';
                        }
                        elsif ( $dr_states->{$site} eq 'heal_aggrs_completed' )
                        {
                            $dr_state = 'heal_aggrs_completed';
                        }
                        elsif ( $dr_states->{$site} eq 'switchover_completed' )
                        {
                            $dr_state = 'switchover_completed';
                        }
                        $survivor_site = $site;
                        $disaster_site = ( $site eq 'A' ) ? 'B' : 'A';
                        return ( $dr_state, $survivor_site, $disaster_site );
                    }
                }
            }
            return;    #failure if it's not heal_roots/normal
             #failiure if it's not heal_roots/normal/heal_aggrs incase of MCC-IP
        }

        # beware that we expect that the "heal roots" operation is not complete
        # until the disaster site is live again.  heal_roots_completed should
        # only be detected in the case above where (scalar(@live_sites) == 2)
        # TODO
        # we may want to support this
        # partial state now (heal roots w/disaster not booted)...
    }
    elsif ( scalar(@live_sites) == 1 ) {
        if (
            $dr_states->{ $live_sites[0] } =~ m{^( switchover_completed |
                                               heal_aggrs_completed )$}x
          )
        {
            $dr_state      = $1;
            $survivor_site = $live_sites[0];
            $disaster_site = ( $survivor_site eq 'A' ) ? 'B' : 'A';

            return ( $dr_state, $survivor_site, $disaster_site );

        }
        else {
            return;
        }
    }

    # experimental TODO (pre-boot heal roots)
    # } #elsif ($dr_states{$live_sites[0]} =~
    #m{^( heal_roots_completed )$}x) {
    #if ($dr_states{$live_sites[0]} eq 'heal_roots_completed') {
    #$dr_state      = 'heal_roots_completed';
    #$survivor_site = $live_sites[0];
    #$disaster_site = ($survivor_site eq 'A') ? 'B' : 'A';
    #}
    #}
    else {
        return;

    }

}

=head2 get_interconnect()

    my $mcc_interconnect = $mcc->get_interconnect();

Gets the NACL::MTask::MCC::Interconnect object associated with mcc.

=cut

sub get_interconnect {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    return $self->_interconnect();
}

=head2 L_determine_live_sites()

Uses a subtest (which may time-out) for each site to find which sites are
currently in a state such that they may be queried for MCC information.

=cut

sub L_determine_live_sites {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my @live_sites = ();

    # these subtests are quick queries to both sites one of them may fail
    my %subtest_handles        = ();
    my $DR_STATE_QUERY_TIMEOUT = 60;
    foreach my $site (qw/ A B /) {
        my $cluster = $self->get_cluster( site => $site );
        my $dr_coordinator = $cluster->get_DR_Coordinator();

        # this should just try to do a simple query for the dr state.  all we
        # care about here is if the subtest succeeds or times out.  it is
        # essentially only a 'ping' to see if the Metrocluster CI is responding.
        $subtest_handles{$site} = subtest(
            sub {

                # Leaving this in for documentation of an interesting use
                # of subtest, where you can get some information out of one
                # after it has completed (see follow up call commented out
                # below).  turns out we don't end up needing this
                # functionality here, but maybe sometime it will be useful
                # to see how this worked when we were using it successfully.
                #
                # my $cluster = $self->get_cluster(site => $site);
                # my $dr_coordinator = $cluster->get_DR_Coordinator();
                # my $dr_state = $dr_coordinator->get_DR_state();
                # $Parent->message_put(type    => 'DR_STATE_REPORT',
                #                      message => $dr_state);

                # replaced with:

                try {
                    $cluster->get_Metrocluster_component()->command_interface()
                      ->refresh_command_interface( no_version_change => 1 );
                    my $dr_state = $dr_coordinator->get_DR_state();
                    my $result   = NATE::Result::Info->new("Site is up...");
                    $Tharn::Log->log($result);
                }
                catch NATE::BaseException with {
                    print Dumper shift;
                    $Log->comment(
                            "Ignoring a NATE::BaseException most likely "
                          . "due to an ssh connection timeout." );
                    my $result = NATE::Result::Info->new("Site is down?...");
                    $Tharn::Log->log($result);

                    # force an eventual "natural" subtest timeout (i guess)
                    sleep( $DR_STATE_QUERY_TIMEOUT * 2 );
                };

            },
            '-bg',
            '-timeout' => $DR_STATE_QUERY_TIMEOUT,
            '-iffatal' => '-ok',
            '-runid'   => 'DR_STATE_QUERY__site_' . $site,
        );
    }

    # we now get the results of each live site's dr state query
    my %dr_states = ();
    foreach my $site ( keys(%subtest_handles) ) {
        my $subtest = $subtest_handles{$site};
        $subtest->wait_finish();
        my $exit_status = $subtest->exit_status() >> 8;    #0=OK, 250=warn
        my $decode_status = NATE::Child::decode_status( $subtest->exit_status );
        $Log->comment( "DR_STATE_QUERY decode status: '$decode_status' "
              . "exit status:$exit_status" );
        if ( $decode_status eq 'status 0' || $decode_status eq 'status 250' ) {

            # if ($exit_status == 0 || $exit_status == 250 ) {
            #this will pass if any pass...for now there's only one so it's ok
            #not certain what it is, but the own_result code here appears
            #to cause connection errors later on in the code in a way that
            #doesn't obviously connect it to this code.  Switching to the
            #above exit status test appeared to fix this...
            #if ( ($subtest->own_results(result_type => 'WARN')) ||
            #($subtest->own_results(result_type => 'INFO')) ||
            #($subtest->own_results(result_type => 'PASS')) ) {
            # this commented out call is associated with the code commented out
            # above.  see notes therein.
            # $dr_states{$site}= $subtest->message_get(type=>'DR_STATE_REPORT');

            # the subtest returned successfully.  it is safe to query this site.
            push( @live_sites, $site );
        }
    }

    return \@live_sites;
}

=head2 _create_DR_groups()

This method creates the DR Group objects associated with the DR Groups that are
resident in this MCC system.  Returns an arrayref of NACL::MTask::MCC::DR_Group
objects.

=cut

sub _create_DR_groups {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    # validate opts
    my %opts = validate_with(
        params => \@_,
        spec   => {
            bootstrap_util =>
              { isa => 'NACL::MTask::MCC::Metrocluster::_Bootstrap_Util' },
            constructor_log_path => { type => SCALAR | UNDEF },
        },
    );

    my $bootstrap_util       = delete( $opts{bootstrap_util} );
    my $constructor_log_path = delete( $opts{constructor_log_path} );

    my @dr_groups = ();
    foreach my $A1_node_name ( @{ $bootstrap_util->get_A1_node_names() } ) {
        push(
            @dr_groups,
            NACL::MTask::MCC::DR_Group->new(
                metrocluster         => $self,
                A1_node_name         => $A1_node_name,
                constructor_log_path => $constructor_log_path,
                site_A_Metrocluster_component =>
                  $bootstrap_util->get_site_A_Metrocluster_component()
            )
        );
    }

    return \@dr_groups;
}

=head2 get_DR_groups()

Get the DR Groups present across clusters A and B.

    my %dr_groups = %{ $mcc->get_DR_groups() };

Returns a reference to a hash of the form:

    { $group_X_id => $dr_group_X,
      $group_Y_id => $dr_group_Y,
      ...                         }

where the keys are DR Group IDs (currently the name of the A1 node in a
particular DR Group), and the values are NACL::MTask::MCC::DR_Group
objects.  These represent each of the DR Groups present in the Metrocluster
system across clusters A and B.

=cut

sub get_DR_groups {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %dr_groups = map { $_->get_id() => $_ } @{ $self->_dr_groups() };

    return \%dr_groups;
}

=head2 get_singleton_DR_group()

This is purely a utility method which makes it simpler to get the DR Group
when the Metrocluster contains only a single DR Group (see notes in
get_DR_groups() for details).  If more than 1 DR Group is found this method
will throw an exception.

When the multi-DR Group Metroclusters come into being, this method will likely
be deprecated from that ONTAP version onwards.

    my $singleton_dr_group = $mcc->get_singleton_DR_group();

Returns a NACL::MTask::MCC::DR_Group object.

May throw a NACL::MTask::MCC::Exceptions::InvalidMethodUsage exception.

=cut

sub get_singleton_DR_group {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    # Get the DR Groups present across clusters A and B.  If there is only a
    # single DR Group found, return it.  If more than one DR Groups are found,
    # then the call to this method is no longer valid, and an exception is
    # thrown.

    # It is likely that this method should be deprecated when such exceptions
    # start being thrown.

    my @dr_groups = values( %{ $self->get_DR_groups() } );

    if ( @dr_groups > 1 ) {
        NACL::MTask::MCC::Exceptions::InvalidMethodUsage->throw(
                'get_singleton_DR_group() is no longer a valid method '
              . 'when Metrocluster elements may contain multiple '
              . 'DR Groups.  get_singleton_DR_group() likely needs to be '
              . 'deprecated from this ONTAP version onward.' );
    }

    return $dr_groups[0];
}

=head2 get_all_mcc_nodes()

This method returns an arrayref of Metrocluster associated nodes in both
clusters.

    my @mcc_nodes = @{ $mcc->get_all_mcc_nodes() };

The objects in the array are NACL::MTask::MCC::Node objects.

=cut

sub get_all_mcc_nodes {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my @nodes = ();
    foreach my $site ( 'A', 'B' ) {
        my $cluster = $self->get_cluster( site => $site );
        push( @nodes, @{ $cluster->L_get_cluster_mcc_nodes() } );
    }

    return \@nodes;
}

=head2 get_mcc_node()

This method finds a named node in the Metrocluster system if it exists.

    my $mcc_node = $mcc->get_mcc_node(
                       node_name => 'spectre-l11');

or

    my $mcc_node = $mcc->get_mcc_node(
                       node_uuid => 'd437c8cd-e931-11e0-aa13-c948a5272a17');

or

    my $mcc_node = $mcc->get_mcc_node(
                       node_nvramid => 'd437c8cd-e931-11e0-aa13-c948a5272a17');

One (and only one) of the 3 possible parameters must be passed.

=over

=item Options

=over

=item C< node_name >

Name of the node to find.  Be aware that node names CAN change, even in the
middle of a DR lifecycle.  In scripts, please assume that this value is
unreliable.  Node UUIds are reliable (as are NVRAM IDs generally.)

=item C< node_uuid >

Node UUID of the node to find.

=item C< node_nvramid >

Node NVRAM ID of the node to find.

=back

=back

Returns a NACL::MTask::MCC::Node object.

=cut

sub get_mcc_node {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            node_name => { regex => qr/^[\w\-]+$/, optional => 1 },
            node_uuid => { regex => qr/^[\w\-]+$/, optional => 1 },
        },
    );

    # one and only one of the parameters node_name and
    # node_uuid is required.  Check this requirement.
    {
        my $parameter_error_message =
            "NACL::MTask::MCC::Node->metrocluster_enable() requires either\n"
          . "    the node_name parameter exclusive-or the\n"
          . "    node_uuid parameter to be passed.";

        if ( defined( $opts{node_name} ) ) {
            if ( defined( $opts{node_uuid} ) ) {
                NACL::MTask::MCC::Exceptions::BaseException->throw(
                    $parameter_error_message);
            }
        }
        elsif ( !defined( $opts{node_uuid} ) ) {
            NACL::MTask::MCC::Exceptions::BaseException->throw(
                $parameter_error_message);
        }
    }

    my $found_node = undef;
    my @nodes      = @{ $self->get_all_mcc_nodes() };
    foreach my $node (@nodes) {
        if ( defined( $opts{node_name} ) ) {
            if ( $node->get_name() eq $opts{node_name} ) {
                $found_node = $node;
                last;
            }
        }
        elsif ( $node->get_uuid() eq $opts{node_uuid} ) {
            $found_node = $node;
            last;
        }
    }

    if ( !defined($found_node) ) {
        my $node_id =
          defined( $opts{node_name} ) ? $opts{node_name} : $opts{node_uuid};
        my $msg = "Node [$node_id] not found!";
        NACL::MTask::MCC::Exceptions::BaseException->throw($msg);
    }

    return $found_node;
}

=head2 refresh_command_interfaces()

This method refreshes command interfaces for site A and/or B.

    # refresh all command interfaces on all sites
    $mcc->refresh_command_interfaces();

    # refresh all command interfaces on site A
    $mcc->refresh_command_interfaces(site => 'A');

    # only refresh command interfaces on site B that are in the specified
    # list of nodes
    $mcc->refresh_command_interfaces(site => 'B', nodes => ['nodeA','nodeB']);

    # only refresh command interfaces on all sites that are specified in the
    # list of nodes
    $mcc->refresh_command_interfaces(nodes => ['nodeA','nodeB','nodeC']);

=over

=item Options

=over

=item C<< site => 'A'|'B' >>

(Optional) IF passed, this option flags which site's command interfaces should
be refreshed.  If not passed, both sites' command interfaces will be refreshed.

=item C<< nodes => \@arrayref >>

(Optional) IF passed, this controls the list of nodes that *can* have their
command interfaces refreshed. If site is specified, then only the nodes on that
site that exist in this list will have their command interfaces refreshed. If
site is not specified, then only the nodes of in both sites that exist in this
list will have their command interfaces refreshed. By default, if this parameter
is not specified, then all nodes *can* have their interfaces refreshed.

=back

=back

=cut

sub refresh_command_interfaces {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self     = shift;
    my %raw_opts = @_;

    # validate opts
    my %opts = validate_with(
        params => \%raw_opts,
        spec   => {
            site                   => { regex => qr/^[AB]$/, optional => 1, },
            max_reconnect          => { type  => SCALAR,     default  => 10 },
            max_reconnect_timewait => { type  => SCALAR,     default  => 30 },
            no_version_change      => { type  => SCALAR,     default  => 1 },
            nodes                  => { type  => ARRAYREF,   optional => 1 },
        },
    );
    $Log->comment( "max reconnect: $opts{max_reconnect}\n"
          . "max reconnect timewait: $opts{max_reconnect_timewait}\n"
          . "no_version_change: $opts{no_version_change}" );

    foreach my $site ( 'A', 'B' ) {
        if ( ( !defined( $opts{site} ) ) || ( $opts{site} eq $site ) ) {
            my $retry_connection = 1;
            my $end_time         = time() + 300;
            while ( $retry_connection && $end_time > time() ) {
                try {
                    if ( !$self->get_Metrocluster_component( site => $site )
                        ->command_interface()->apiset()->get_connection()
                        ->is_responsive( timeout => 10 ) )
                    {
                        $self->get_Metrocluster_component( site => $site )
                          ->command_interface()->refresh_command_interface(
                            no_version_change => $opts{no_version_change},
                            max_reconnect     => $opts{max_reconnect},
                            max_reconnect_timewait =>
                              $opts{max_reconnect_timewait}
                          );
                    }
                    else {
                        $Log->comment(
'Command interface is reachable and responsive. Skipping refresh...'
                        );
                    }
                    $retry_connection = 0;
                }
                catch NACL::APISet::Exceptions::ConnectionFailedException with {
                    $Log->warn(
                        'Failed to create connection... sleeping and retrying');
                    Tharn::sleep(5);
                };
            }
            my $mcc_nodes =
              $self->get_cluster( site => $site )->L_get_cluster_mcc_nodes();
            foreach my $mcc_node ( @{$mcc_nodes} ) {

                # skip nodes that are not in the list passed through
                if ( $opts{nodes} ) {
                    my $node_name = $mcc_node->get_Node_component()->node();
                    next if ( !grep( /^$node_name$/i, @{ $opts{nodes} } ) );
                }

                if ( !$mcc_node->get_Node_component()->command_interface()
                    ->apiset()->get_connection()->is_responsive( timeout => 10 )
                  )
                {
                    $mcc_node->get_Node_component()->command_interface()
                      ->refresh_command_interface(
                        no_version_change      => $opts{no_version_change},
                        max_reconnect          => $opts{max_reconnect},
                        max_reconnect_timewait => $opts{max_reconnect_timewait}
                      );
                }
                else {
                    $Log->comment(
'Command interface is reachable and responsive. Skipping refresh...'
                    );
                }
            }
        }
    }

    return;
}

=head2 is_vsim()

Checks to see if this testbed is a VSIM (it assumes if one node is a vsim then all are)

Returns 1 if its a VSIMs based MCC setup.

=cut

sub is_vsim {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    if ( defined $self->_vsim() ) {
        return $self->_vsim();
    }
    else {
        foreach my $node ( @{ $self->get_all_mcc_nodes() } ) {
            my $netnode = $node->L_get_network_node_functionality();
            if ( $netnode->L_is_initialized() ) {
                my $is_vsim = $netnode->L_is_vsim();
                $Log->comment( 'is vsim? ' . $node->get_name() . ": $is_vsim" );
                $self->_vsim($is_vsim);
                last;
            }
        }
        return $self->_vsim();
    }
}

=head2 is_2_node()

Returns 1 if its a 2-node MCC setup.

The Metrocluster is 2-node if it has a single DR group with two single node clusters

=cut

sub is_2_node {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my @dr_groups = @{ $self->_dr_groups() };

    scalar @dr_groups == 1 or return;

    return $dr_groups[0]->is_2_pack();

}

=head2 is_4_node()

Returns 1 if its a 4-node MCC setup.

The Metrocluster is 4-node if it has a single DR group with two 2-node clusters (each 2-node cluster is HA pair as well)

=cut

sub is_4_node {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my @dr_groups = @{ $self->_dr_groups() };

    scalar @dr_groups == 1 or return;

    return $dr_groups[0]->is_4_pack();
}

=head2 is_8_node()

Returns 1 if its a 8-node MCC setup.

The Metrocluster is 8-node if it has two DR groups with a 4-pack MCC in each DR group.

=cut

sub is_8_node {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my @dr_groups = @{ $self->_dr_groups() };

    scalar @dr_groups == 2 or return;

    return $dr_groups[0]->is_4_pack() && $dr_groups[1]->is_4_pack();
}

=head2 check()

    my $result = $mcc->check(site => 'A');

Runs metrocluster check run (or show).  If there are any warnings for nodes or
aggreggrates then more detailed commands are run to gather what is failing.
Exceptions are then thrown.

=over

=item Options

=over

=item C<< site =E<gt> 'A'|'B' >>

Specifies which site's cluster the check command is initiated from.


=item C<< run =E<gt> '1'|'0' (default=1) >>

Specifies whether to do metrocluster check run or a metrocluster check show

=back

=item C<< ignore_periodic_check_enabled_warning =E<gt> '1'|'0' (default=0) >>

Specifies whether to ignore a periodic-check-enabled warning in case it has been disabled
by the user

=back

=cut

sub check {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my ( $self, @opts ) = @_;

    #validate opts
    my %opts = validate_with(
        params => \@opts,
        spec   => {
            site => { regex => qr/^[AB]$/ },
            run  => { regex => qr/^[01]$/, default => 1 },
            ignore_periodic_check_enabled_warning =>
              { regex => qr/^[01]$/, default => 0 },
        },
    );
    my $run  = $opts{run};
    my $site = $opts{site};
    my $ignore_periodic_check_enabled_warning =
      $opts{'ignore_periodic_check_enabled_warning'};

    my $disaster_status = $self->get_disaster_status( include_HA => 1 );
    $Log->comment( 'Disaster status is : ' . Dumper($disaster_status) );

    my $check = $self->_metrocluster_check_show(
        run  => $run,
        site => $site,
    );

    # Obtain MCC op show command output
    my $mcc_op_cs =
      NACL::CS::MetroclusterOperation->fetch(
        command_interface => $self->get_Metrocluster_component( site => $site )
          ->command_interface() );
    my $mcc_op_error_list;
    if ( ref $mcc_op_cs->error_list() eq 'ARRAY' ) {
        map { $mcc_op_error_list .= $_ } @{ $mcc_op_cs->error_list() };
        $Log->comment("Obtained Op Show error list:\n$mcc_op_error_list");
    }

    my ( %fail, %is_2_node );
    my $dr_groups = $self->get_DR_groups();
    foreach my $dr_group ( values( %{$dr_groups} ) ) {
        if ( !$dr_group->is_4_pack() ) {
            foreach my $node ( @{ $dr_group->get_mcc_nodes() } ) {
                $is_2_node{ $node->get_name() } = $node->get_uuid();
            }

        }
    }

    #If NOT normal DR state then check all nodes,
    #otherwise ignore some errors: BURT 596281

    my %dr_disaster_node =
      %{ $self->get_disaster_nodes( disaster_status => $disaster_status ) };

    my $not_normal = 0;
    if ( scalar keys %dr_disaster_node ) {
        $Log->comment( "metrocluster check will not function on all nodes "
              . " in this DR state" );
        $not_normal = 1;

        #We need a list of all the nodes that are down so that when
        #we examine the 'check' response, we can ignore errors related
        #to nodes being down for nodes that really ARE down
    }

    #don't ignore HA errors for actual HA disaster nodes (BURT 598768)
    my %ha_disaster_node;

    foreach my $ha_dr_group_id ( values %{ $disaster_status->{HA_states} } ) {
        foreach my $dr_site ( values %{$ha_dr_group_id} ) {
            if ( $dr_site->{disaster_node} ) {
                $ha_disaster_node{ $dr_site->{disaster_node} } =
                  $dr_site->{HA_state};
            }
        }
    }

    foreach my $item ( @{$check} ) {
        if ( lc $item->{result} !~ /ok|not-applicable/ ) {

            #$fail{$item->{component}} = $item; #move re: burt 593924

            if ( lc $item->{component} eq 'nodes' ) {
                my $result =
                  $self->_metrocluster_check_node_show( site => $site, );
                foreach my $check ( @{$result} ) {

                    #most will be gone when BURTs are fixed...
                    ## no critic (ControlStructures::ProhibitCascadingIfElse)

                    if ( lc $check->{result} !~ /ok|not-applicable/ ) {
                        if ( defined $dr_disaster_node{ $check->{node} } )
                        {    #node down?
                            $Log->comment(
                                    "Ignoring $check->{check} error, "
                                  . "$check->{node} is unreachable (dr state "
                                  . "$disaster_status->{DR_state})\n" );
                        }
                        elsif ( defined $is_2_node{ $check->{node} }
                            && $check->{check} =~
                            /symmetric[ -]dr[ -]auxiliary/i )
                        {
                            $Log->comment(
                                    "Ignoring $check->{check} error, "
                                  . "2 node systems have no HA partner" );
                        }
                        elsif ( defined $is_2_node{ $check->{node} }
                            && $check->{check} =~
                            /symmetric[ -]ha[ -]relationship/i )
                        {
                            $Log->comment(
                                    "Ignoring $check->{check} error, "
                                  . "2 node systems have no HA partner" );
                        }
                        elsif ( !defined $ha_disaster_node{ $check->{node} }
                            && $check->{check} =~
                            /symmetric[ -]ha[ -]relationship/i )
                        {
                            #these burt 598768 ignores will ignore more than
                            #necessary, remove when possible
                            $Log->comment(
                                    "Ignoring $check->{check} error, "
                                  . "see BURT 598768" );
                        }
                        elsif ( !defined $ha_disaster_node{ $check->{node} }
                            && $check->{check} =~ /ha[ -]partner/i )
                        {
                            #these burt 598768 ignores will ignore more than
                            #necessary, remove when possible
                            $Log->comment(
                                    "Ignoring $check->{check} error, "
                                  . "see BURT 598768" );
                        }
                        elsif ( !defined $ha_disaster_node{ $check->{node} }
                            && $check->{check} =~
                            /dr[ -]partner[ -]in[ -]peer/i )
                        {
                            #these burt 598768 ignores will ignore more than
                            #necessary, remove when possible
                            $Log->comment(
                                    "Ignoring $check->{check} error, "
                                  . "see BURT 598768" );
                        }
                        elsif ( !defined $ha_disaster_node{ $check->{node} }
                            && $check->{check} =~ /reachable/i )
                        {
                            #these burt 598768 ignores will ignore more than
                            #necessary, remove when possible
                            $Log->comment(
                                    "Ignoring $check->{check} error, "
                                  . "see BURT 598768" );
                        }
                        elsif ( !defined $ha_disaster_node{ $check->{node} }
                            && $check->{check} =~ /metrocluster[ -]ready/i )
                        {
                            #these burt 598768 ignores will ignore more than
                            #necessary, remove when possible
                            $Log->comment(
                                    "Ignoring $check->{check} error, "
                                  . "see BURT 598768" );
                        }
                        elsif (( $check->{check} =~ /symmetric[ -]dr/i )
                            && ( $disaster_status->{DR_state} !~ /normal/ ) )
                        {
                            #symmetric checks only work in normal DR state
                            $Log->comment(
                                    "Ignoring $check->{check} error, "
                                  . "$check->{node} is unreachable (dr state "
                                  . "$disaster_status->{DR_state})\n" );
                        }
                        elsif ( $check->{check} =~ /mirroring/i ) {
                            $Log->comment(
                                    "Ignoring $check->{check} error, "
                                  . "see BURT 593924\n" );
                        }
                        elsif ($check->{check} eq 'automatic-uso'
                            && $check->{'additional-info'} =~
                            /Automatic switchover is not enabled/ )
                        {
                            # BURT 1018434 catch AUSO disabled warnings
                            $Log->comment(
"Ignoring $check->{check} warning for disabled AUSO"
                            );
                        }
                        else {
                            $fail{ $item->{component} } = $item;   # burt 593924
                            $Log->comment(
                                "$check->{node} : $check->{check} failed\n");
                            $fail{ $check->{node} }->{ $check->{check} } =
                              $check;
                        }
                    }
                }
            }
            elsif ( lc $item->{component} eq 'aggregates' ) {
                my $result =
                  $self->_metrocluster_check_aggregate_show( site => $site );
                my $found_unm_aggrs = $self->has_unmirrored_aggrs();

                foreach my $check ( @{$result} ) {

                    # Ignore check warnings for unmirrored aggrs
                    if (   $found_unm_aggrs
                        && lc $check->{result} eq 'warning'
                        && $check->{'additional-info'} =~
                        /un-mirrored aggregates in MetroCluster/i )
                    {
                        $Log->comment(
'Ignoring metrocluster check warnings for unmirrored aggrs'
                        );
                        next;
                    }

                    if ( lc $check->{result} !~ /ok|not-applicable/ ) {
                        $Log->comment(
"$check->{node} {$check->{aggregate}} : $check->{check} failed\n"
                        );
                        $fail{ $check->{node} }->{ $check->{aggregate} }
                          ->{ $check->{check} } = $check;
                        $fail{ $item->{component} } = $item;    # burt 593924
                    }
                }
            }
            elsif ( lc $item->{component} eq 'lifs' ) {
                my $result =
                  $self->_metrocluster_check_lif_show( site => $site );
                $Log->comment(
                    'Lif behavior currently unclear. See RFE burt #918874');
                next;

                #Lif behavior not clear. See RFE burt: #918874

            }
            elsif ( lc $item->{component} eq 'clusters' ) {
                my $result =
                  $self->_metrocluster_check_cluster_show( site => $site );

                if ($not_normal) {
                    $Log->comment(
                            'Cluster behavior in disaster state is unclear. '
                          . 'Ignoring results' );    #see RFE burt : 918874
                    next;
                }
                else {
                    foreach my $check ( @{$result} ) {
                        if ( lc $check->{result} !~ /ok|not-applicable/ ) {
                            if ( lc $check->{check} =~ /periodic-check-enabled/
                                && $ignore_periodic_check_enabled_warning )
                            {
                                $Log->comment(
'Periodic check disabled by user. Not flagging it'
                                );

                               # Ignore failed disk notifications - BURT 1014715
                            }
                            elsif ( $check->{'additional-info'} =~
                                /Use the "metrocluster operation show" command/i
                                && $mcc_op_error_list =~
                                /There are failed disks present/i )
                            {
                                $Log->comment(
"Ignoring $check->{check} warnings on $check->{cluster} due to failed disks"
                                );
                            }
                            else {
                                $Log->comment(
"$check->{cluster} : $check->{check} failed\n"
                                );
                                $fail{ $check->{cluster} }->{ $check->{check} }
                                  = $check;
                                $fail{ $item->{component} } = $item;
                            }
                        }
                    }
                }
            }
            elsif ( lc $item->{component} eq 'config-replication' ) {

                #process the check results for config-replication.
                my $result =
                  $self->_metrocluster_check_config_replication_show(
                    site => $site );

        #skip the processing if the MCC state is not normal because the behavior
        #is not defined. see burt #918874

                if ($not_normal) {
                    $Log->comment(
'Config replication behavior in disaster state is unclear.'
                          . 'Ignoring results' );
                    next;
                }
                else {
            #if normal process results for only vserver-streams, cluster-streams
            #see burt 918874
                    foreach my $check ( @{$result} ) {
                        foreach my $key ( keys %$check ) {
                            if ( lc $key =~ 'vserver-streams'
                                && $check->{$key} !~ /ok/ )
                            {
                                $Log->comment("$key : $check->{$key} failed\n");
                                $fail{$key} = { $key => $check->{$key} };
                                $fail{ $item->{component} } = $item;
                            }
                            elsif ( lc $key =~ 'cluster-streams'
                                && $check->{$key} !~ /ok/ )
                            {
                                $Log->comment("$key : $check->{$key} failed\n");
                                $fail{$key} = { $key => $check->{$key} };
                                $fail{ $item->{component} } = $item;
                            }
                            elsif ( lc $key =~ 'storage-status'
                                && $check->{$key} !~ /ok/ )
                            {
                                $Log->comment("$key : $check->{$key} failed\n");
                                $fail{$key} = { $key => $check->{$key} };
                                $fail{ $item->{component} } = $item;
                            }
                            elsif ( lc $key =~ 'storage-in-use'
                                && $check->{$key} !~ /Cluster-wide Volume.*/ )
                            {
                                $Log->comment("$key : $check->{$key} failed\n");
                                $fail{$key} = { $key => $check->{$key} };
                                $fail{ $item->{component} } = $item;
                            }
                        }
                    }
                }
            }
        }
    }

    if ( keys %fail ) {
        $Log->comment( 'Check failures: ' . Dumper \%fail );
        NACL::MTask::MCC::Exceptions::BaseException->throw(
            "ERROR: metrocluster check warnings:\n" . join q{,},
            keys %fail );

    }

    return 1;    #OK if we didn't fail earlier
}

=head2 get_disaster_nodes()

    my $disaster_nodes = $mcc->get_disaster_nodes(
                           disaster_status => $disaster_status);

Returns a hash whose keys are the disaster nodes from the disaster site
(and values are the disaster site)

=cut

sub get_disaster_nodes {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => { disaster_status => { type => HASHREF }, },
    );

    my $disaster_status = delete( $opts{disaster_status} );

    my %dr_disaster_node;
    if ( $disaster_status->{DR_state} !~ /normal/ ) {

        #make a hash of all the nodes that are down
        #TODO redo this with dr_group->get_nodes(site => ...) ?
        my $cluster =
          $self->get_cluster( site => $disaster_status->{disaster_site} );
        foreach my $node ( @{ $cluster->L_get_cluster_mcc_nodes() } ) {
            $dr_disaster_node{ $node->get_name() } =
              $disaster_status->{disaster_site};
        }
    }
    return \%dr_disaster_node;
}

sub get_live_nodes {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => { disaster_status => { type => HASHREF } }
    );

    my $disaster_status = $opts{'disaster_status'};

    my %survivor_nodes;

    if ( $disaster_status->{'DR_state'} =~ /normal/ ) {

        foreach my $cluster ( $self->_cluster_A(), $self->_cluster_B() ) {
            $survivor_nodes{ $cluster->get_name() } =
              $cluster->L_get_cluster_mcc_nodes();
        }

    }
    else {
        my $survivor_cluster =
          $self->get_cluster( site => $disaster_status->{'survivor_site'} );
        $survivor_nodes{ $survivor_cluster->get_name() } =
          $survivor_cluster->L_get_cluster_mcc_nodes();

    }

    return \%survivor_nodes;

} ## end sub get_live_nodes

sub _metrocluster_check_show {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my ( $self, @opts ) = @_;

    #validate opts
    my %opts = validate_with(
        params => \@opts,
        spec   => {
            site => { regex => qr/^[AB]$/ },
            run  => { regex => qr/^[01]$/, default => 1 },
        },
    );
    my $run  = $opts{run};
    my $site = $opts{site};

    #TODO this needs update later when the API changes...(component vs apiset)

    my $ui_change2 =
      $self->has_uichange( uichange => "mcc-check-periodic-job" );

    my $output;
    if ($ui_change2) {
        $Log->comment('MCC Check is running on a system with >= LB.0 build');
        $output = $self->_lb0_metrocluster_check_show(
            site => $site,
            run  => $run
        );
        return $output->get_parsed_output();
    }

    my $component =
      ( $site =~ /A/ )
      ? $self->_site_A_metrocluster_component()
      : $self->_site_B_metrocluster_component();
    my $apiset = $component->command_interface()->apiset();
    $apiset->set_timeout( 'connectrec-timeout' => 10 * 60 );

    if ($run) {
        $Log->comment('Running me check run');
        my $return = $apiset->metrocluster_check_run();
        Tharn::sleep(10);

        my $ui_change = $self->has_uichange( uichange => "fs1-check-no-show" );
        if ($ui_change) {
            $Log->comment('Running on a system with >= FS.1 build');
            $output = $apiset->metrocluster_check_show();
        }
        else {
            $Log->comment('Running on a system with FS.0 build');
            $output = $return;
        }
    }
    else {    #use show instead if not running
        $Log->comment('Running me check show');
        my $show = $apiset->metrocluster_check_show();

        $output = $show;
    }

    return $output->get_parsed_output();
}

# This method implements the new metrocluster check behavior introduced in LB.0.
# With the introduction of check running as a cron job, the library should
# handle a 'job-in-progress' message when metrocluster check run or metrocluster
# check show is triggered. The library should wait for the job to complete
# before triggering a new job.

# Moreover as soon as me check run is triggered the library needs to
# poll the metrocluster operation history show -job-id <id #> to determine
# the completion state of the job. This method handles that as well.

sub _lb0_metrocluster_check_show {

    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my ( $self, @opts ) = @_;

    #validate opts
    my %opts = validate_with(
        params => \@opts,
        spec   => {
            site => { regex => qr/^[AB]$/ },
            run  => { regex => qr/^[01]$/, default => 1 },
        },
    );
    my $run  = $opts{run};
    my $site = $opts{site};
    my $component =
      ( $site =~ /A/ )
      ? $self->_site_A_metrocluster_component()
      : $self->_site_B_metrocluster_component();
    my $apiset = $component->command_interface()->apiset();
    $apiset->set_timeout( 'connectrec-timeout' => 10 * 60 );
    my $total_retries = 1;
    my $current_trial = 0;

    my $return;
    my $jobid;
    my $retry_required = 0;
    my $response;
    my $output;
    my $show;

  RERUN_ME_CHECK:

    if ($run) {
        $Log->comment('Running metrocluster check run');
        try {
            $return   = $apiset->metrocluster_check_run();
            $response = $return->get_processed_output();
        }
        catch NATE::BaseException with {
            my $ex      = shift;
            my $ex_text = $ex->text();
            $Log->comment( 'Failure text is : ' . $ex_text );
            if (
                lc $ex_text =~
                m/command failed: A metrocluster check operation is in progress/
              )
            {
                $Log->comment(
'metrocluster check failed to start because another job was in progress'
                );
                $ex_text =~
                  m/\"metrocluster operation history show -job-id (\d+)\"/;
                $jobid          = $1;
                $retry_required = 1;
            }
            else {
                $Log->comment(
'metrocluster check failed to start due to unexpected reasons'
                );
                $ex->throw();
            }
        };

        $Log->comment('Sleeping 10 secs for check job refresh to happen');
        Tharn::sleep(10);

        if ( defined($response)
            && $response =~
            m/The operation has been started and is running in the background/ )
        {
            $Log->comment('Job started successfully.');
            $response =~
              m/\"metrocluster operation history show -job-id (\d+)\"/;
            $jobid = $1;
        }
    }
    else {
        $Log->comment('Running metrocluster check show');
        try {
            $show   = $apiset->metrocluster_check_show();
            $output = $show;
        }
        catch NATE::BaseException with {
            my $ex      = shift;
            my $ex_text = $ex->text();
            $Log->comment( 'Failure text is : ' . $ex_text );
            if ( $ex_text =~
                m/show failed: A metrocluster check operation is in progress./ )
            {
                $Log->comment(
'metrocluster check show failed to start because another job was in progress'
                );
                $ex_text =~
                  m/\"metrocluster operation history show -job-id (\d+)\"/;
                $jobid          = $1;
                $retry_required = 1;
            }
            else {
                $Log->comment(
'metrocluster check failed to start due to unexpected reasons'
                );
                $ex->throw();
            }
        };
    }

    if ( defined $jobid ) {
        $Log->comment("Wait for job $jobid to complete");

#Adding this sleep here because it takes a while for the job status to be updated.
#If we check too soon then we will get a false results which could be
#successful which will fail the test later.
        $Log->comment('Sleep for 10 seconds before checking job status');
        Tharn::snooze(10);
        $self->_wait_for_job_to_complete(
            jobid         => $jobid,
            mcc_component => $component
        );
    }

    #Rerun the operation if required.
    if ($retry_required) {
        if ( $current_trial < $total_retries ) {
            $Log->comment( 'Retrying metrocluster check command. Attempt # '
                  . $current_trial );
            $current_trial++;
            ( $return, $jobid, $response, $output, $show ) = ();
            $retry_required = 0;
            goto RERUN_ME_CHECK;
        }
        else {
            NACL::MTask::MCC::Exceptions::BaseException->throw(
                "metrocluster check command failed after $total_retries tries");
        }
    }

    if ($run) {
        $Log->comment(
            'Running metrocluster check show and returning the output');
        my $timeout = time + 120;    # 2 min timeout
      RETRY_MET_CHECK_SHOW:
        try {
            $output = $apiset->metrocluster_check_show();
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $ex = shift;

            # throw back the exception if it is not for a check in-progress
            $ex->throw()
              if $ex->text() !~ /metrocluster check operation is in progress/;

            # throw back exception if it is beyond the timeout
            $ex->throw() if $timeout < time;

            $Log->comment('Retry check show in 10 secs...');
            Tharn::sleep(10);

            goto RETRY_MET_CHECK_SHOW;
        };
    }

    return $output;
}

sub _wait_for_job_to_complete {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my ( $self, @opts ) = @_;

    #validate opts
    my %opts = validate_with(
        params => \@opts,
        spec   => {
            jobid         => { type => SCALAR },
            mcc_component => { type => HASHREF },
        },
    );

    my $jobid         = $opts{jobid};
    my $component     = $opts{mcc_component};
    my $cmd_interface = $component->command_interface();
    $Log->comment( 'Checking status for job ' . $jobid );

    my $jobstate;
    my $retry_count = 5;
  RETRY_ME_OP_HIS_JOB:
    try {
        $jobstate = NACL::CS::MetroclusterOperationHistory->fetch(
            command_interface => $cmd_interface,
            filter            => { 'job-id' => $jobid, },
            randomize_order   => 0,
        );
    }
    catch NACL::Exceptions::NoElementsFound with {
        my $ex = shift;
        $Log->comment('Job may not have started yet. Recheck in 5 secs');
        Tharn::snooze(5);

        # Keep retrying for 5 attempts
        if ($retry_count) {
            $retry_count--;
            goto RETRY_ME_OP_HIS_JOB;
        }
        else {
            $ex->throw();
        }
    };

    my $state = $jobstate->{state};

    #keeping the value high in case it is run in SO mode.
    my $waittime = time() + 600;
    $Log->comment( 'Current job state : ' . $state );
    while ( $state !~ /successful/ && time() <= $waittime ) {
        if ( $state =~ /failed/ ) {
            croak(
'Metrocluster check job failed even though it started successfully'
            );
        }

        $Log->comment('Job still in progress. Try after some time');
        Tharn::snooze(10);
        $jobstate = NACL::CS::MetroclusterOperationHistory->fetch(
            command_interface => $cmd_interface,
            filter            => { 'job-id' => $jobid, },
            requested_fields  => ['state'],
            randomize_order   => 0,
        );
        $state = $jobstate->{state};
        $Log->comment( 'Current job state : ' . $state );
    }

    if ( lc $state !~ /successful/i || $state =~ /failed/i ) {
        NACL::MTask::MCC::Exceptions::BaseException->throw(
            'metrocluster check job did not complete or failed. Inspect logs.');
    }

    return;
}

sub _metrocluster_check_config_replication_show {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my ( $self, @opts ) = @_;

    #validate opts
    my %opts = validate_with(
        params => \@opts,
        spec   => { site => { regex => qr/^[AB]$/ }, },
    );
    my $site = $opts{site};

    #TODO this needs update later when the API changes...(component vs apiset)

    my $component =
      ( $site =~ /A/ )
      ? $self->_site_A_metrocluster_component()
      : $self->_site_B_metrocluster_component();

    my $apiset = $component->command_interface()->apiset();
    $apiset->set_timeout( 'connectrec-timeout' => 10 * 60 );

    my $run = $apiset->metrocluster_check_config_replication_show();

    $Log->comment(
        'config replication output: ' . Dumper( $run->get_parsed_output ) );
    return $run->get_parsed_output();
}

sub _metrocluster_check_cluster_show {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my ( $self, @opts ) = @_;

    #validate opts
    my %opts = validate_with(
        params => \@opts,
        spec   => { site => { regex => qr/^[AB]$/ }, },
    );
    my $site = $opts{site};

    #TODO this needs update later when the API changes...(component vs apiset)

    my $component =
      ( $site =~ /A/ )
      ? $self->_site_A_metrocluster_component()
      : $self->_site_B_metrocluster_component();
    my $apiset = $component->command_interface()->apiset();
    $apiset->set_timeout( 'connectrec-timeout' => 10 * 60 );

    my $run = $apiset->metrocluster_check_cluster_show();

    $Log->comment( 'cluster output: ' . Dumper( $run->get_parsed_output ) );
    return $run->get_parsed_output();
}

sub _metrocluster_check_lif_show {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my ( $self, @opts ) = @_;

    #validate opts
    my %opts = validate_with(
        params => \@opts,
        spec   => { site => { regex => qr/^[AB]$/ }, },
    );
    my $site = $opts{site};

    #TODO this needs update later when the API changes...(component vs apiset)

    my $component =
      ( $site =~ /A/ )
      ? $self->_site_A_metrocluster_component()
      : $self->_site_B_metrocluster_component();

    my $apiset = $component->command_interface()->apiset();
    $apiset->set_timeout( 'connectrec-timeout' => 10 * 60 );

    my $run = $apiset->metrocluster_check_lif_show();

    $Log->comment( 'Lif output: ' . Dumper( $run->get_parsed_output ) );
    return $run->get_parsed_output();
}

sub _metrocluster_check_aggregate_show {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my ( $self, @opts ) = @_;

    #validate opts
    my %opts = validate_with(
        params => \@opts,
        spec   => { site => { regex => qr/^[AB]$/ }, },
    );
    my $site = $opts{site};

    #TODO this needs update later when the API changes...(component vs apiset)

    my $component =
      ( $site =~ /A/ )
      ? $self->_site_A_metrocluster_component()
      : $self->_site_B_metrocluster_component();

    my $apiset = $component->command_interface()->apiset();
    $apiset->set_timeout( 'connectrec-timeout' => 10 * 60 );

    my $run = $apiset->metrocluster_check_aggr_show();

    return $run->get_parsed_output();
}

sub _metrocluster_check_node_show {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my ( $self, @opts ) = @_;

    #validate opts
    my %opts = validate_with(
        params => \@opts,
        spec   => { site => { regex => qr/^[AB]$/ }, },
    );

    my $site = $opts{site};

    #TODO this needs update later when the API changes...(component vs apiset)

    my $component =
      ( $site =~ /A/ )
      ? $self->_site_A_metrocluster_component()
      : $self->_site_B_metrocluster_component();

    my $apiset = $component->command_interface()->apiset();
    $apiset->set_timeout( 'connectrec-timeout' => 10 * 60 );

    my $run = $apiset->metrocluster_check_node_show();
    return $run->get_parsed_output();
}

=head2 unconfigure()

This method runs the unconfigure command which destroys the metrocluster
configuration.  Without any arguments it destroys all metroclusters related
to this site.  A dr_group_id argument will limit this to just the dr group
and nodes associated with the passed dr group.

With ui change string 'mcc-unconfigure-local-only' builds, the lib
will automatically call unconfigure on both sites as the force param
will be implicit.

    my $result = $mcc->unconfigure(site => 'A');

    (Currently valid - burt #659676)
    my $result = $mcc->unconfigure(dr_group_id => 'dr_group_to_remove');

    (Perform a local unconfigure)
    my $result = $mcc->unconfigure(force => 'true');

Static usage is no longer available

=over

=item Options

=over

=item C<< site =E<gt> 'A'|'B' >>

Specifies which site's cluster the check command is initiated from.

=item C<< dr_group_id =E<gt> E<lt> dr group idE<gt> (optional) >>

If specified, limits unconfigure to just the DR partners related to that dr
group.

=item C<< force =E<gt> 'true'|'false' >>

This option if specified will run unconfigure on any one of the site.
This opton, if passed ONLY does a local unconfigure. You must
run it on the other site to complete the unconfigure operation. If the
dr_group_id is specified it will do a local unconfigure only for that dr group
on the site that has been passed.

    $mcc->unconfigure(
            site        => 'A',
            dr_group_id => 1,
            force       => 'true',
    );

The same command needs to be re-run on site 'B' as well to complete the
operation.

Note: The static call is no longer supported

=back

=back

=cut

sub unconfigure {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my ( $self, @opts ) = @_;

    #validate opts
    my %opts = validate_with(
        params => \@opts,
        spec   => {
            site        => { regex => qr/^[AB]$/ },
            dr_group_id => { type  => SCALAR | UNDEF, optional => 1 },
            force       => { type  => SCALAR | UNDEF, optional => 1 },
        },
        allow_extra => 1,
    );
    my $site  = delete $opts{site};
    my $force = $opts{force};

    #we need to call unconfig on both sites, so just do this
    #even if a site value was passed
    foreach my $site (qw/ A B /) {
        $Log->comment("unconfigure site $site");
        my $cluster = $self->get_cluster( site => $site );
        my $result = $cluster->unconfigure(%opts);
    }

    return;

}

=head2 configure()

This method will configure metrocluster. Without any arguments it
will configure MCC for all DR groups present in the cluster.

The command takes all supported params

    my $result = $mcc->configure(site => 'A');
    my $result = $mcc->configure(site        => 'A',
                                 'node-name' => 'node_to_configure');
    my $result = $mcc->configure(site    => 'A',
                                 refresh => 'true' );

=over

=item Options

=over

=item C<< site =E<gt> 'A'|'B' >>

(Optional) Specifies which site's cluster the configure command is initiated from.
By default would run from a site A cluster node

=item C<< nacltask_cifs_username =E<gt> E<lt>cifs_username<gt> >>

(Optional) This option is required while deleting vsevers
Default: labclient

=item C<< nacltask_cifs_password =E<gt> E<lt>cifs_password<gt> >>

(Optional) This option is required while deleting vsevers
Default: sundance

=back

=back

=cut

sub configure {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my ( $self, @opts ) = @_;
    my ( $site_component, $site, $node_obj );
    my ( @mcc_nodes_siteA, @mcc_nodes_siteB );
    my ( %args, %opts, %cifs_opt );

    if ( ref $self ) {
        %opts = validate_with(
            params => \@opts,
            spec   => {
                site => {
                    regex   => qr/^[AB]$/,
                    default => 'A',
                },
                nacltask_cifs_username => {
                    type    => SCALAR | UNDEF,
                    default => 'labclient'
                },
                nacltask_cifs_password => {
                    type    => SCALAR | UNDEF,
                    default => 'sundance'
                },

            },
            allow_extra => 1,
        );

        $site = delete $opts{'site'};
        $cifs_opt{nacltask_cifs_username} =
          delete $opts{nacltask_cifs_username};
        $cifs_opt{nacltask_cifs_password} =
          delete $opts{nacltask_cifs_password};

        $site_component =
            $site eq 'A'
          ? $self->_site_A_metrocluster_component()
          : $self->_site_B_metrocluster_component();

        #If we do a unconfigure then a configure, and then we run a metrocluster
        #operation show it could return an empty table (on the cluster that
        #configure wasn't run on)...  So in order to prevent it throwing an
        #exception, we enable the flag saying it's OK to ignore this until
        #it first finds a good operation state...it shouldn't ever be empty
        #unless it's just been configured

        $Log->comment('resetting DR operation OK to be empty - reconfig');
        foreach my $mccsite (qw(A B)) {
            my $cluster = $self->get_cluster( site => $mccsite );
            $cluster->get_DR_Coordinator()
              ->_dr_operation_table_ok_to_be_empty(1);
        }

        @mcc_nodes_siteA =
          @{ $self->get_cluster( site => 'A' )->L_get_cluster_mcc_nodes() };
        @mcc_nodes_siteB =
          @{ $self->get_cluster( site => 'B' )->L_get_cluster_mcc_nodes() };
    }

    if (   !defined $opts{'refresh'}
        && !defined $opts{'allow-pre-existing-data-on-both-clusters'} )
    {
        $Log->comment('Deleting pre-existing data VServers');
        foreach my $vs_nodes ( $mcc_nodes_siteA[0], $mcc_nodes_siteB[0] ) {
            my @vservers = NACL::STask::Vserver->find(
                command_interface => $vs_nodes->get_Node_component(),
                filter            => {
                    type    => 'data|cluster',
                    subtype => 'default'
                },
                allow_empty => 1,
            );
            map { $_->purge( nacltask_verify => 1, %cifs_opt ) } @vservers
              if scalar @vservers;
        }
    }
    else {
        $Log->comment('Skipping VServer deletions...');
    }

    #increase the timeout for the apiset
    my $apiset = $site_component->command_interface()->apiset();
    $apiset->set_timeout( 'connectrec-timeout' => 10 * 60 );

    # Run the configure command
    my $run = $apiset->metrocluster_configure(%opts);

    my %job_key;
    my $job_show;
    my $job_id = $run->get_job_ids();

    if ($job_id) {
        $Log->comment("Job Id: $job_id");
        $job_key{id} = $job_id;
    }
    else {
        $job_key{name} = '"Metrocluster Configure"';
        $job_key{'state'} = 'Success';
    }

    $Log->comment('Fetch job status');
    $job_show = NACL::CS::Job->fetch(
        command_interface => $site_component->command_interface(),
        filter            => {%job_key},
    );

    #check the state of the job.
    #Throw exception if the state is failed
    if ( lc $job_show->{'state'} eq 'failure' ) {
        NACL::APISet::Exceptions::JobFailedException->throw(
            $job_show->{'completion'} );
    }

    $Log->comment('Running checks to verify configure passed successfully');
    $Log->comment( 'Checking output for metrocluster show - local and remote '
          . 'cluster should show Configuration state - configured' );

    my $output        = $apiset->metrocluster_show();
    my $parsed_output = @{ $output->{parsed_output} }[0];
    if (   lc $parsed_output->{'local-configuration-state'} ne 'configured'
        && lc $parsed_output->{'remote-configuration-state'} ne 'configured' )
    {
        NACL::MTask::MCC::Exceptions::BaseException->throw(
            'Configure failed! Incorrect Metrocluster show output');
    }

    $Log->comment( 'Checking output for metrocluster node show '
          . '~ the output should show Configured for all nodes' );
    my @config_states = NACL::MTask::MCC::Utils->metrocluster_node_show(
        command_interface => $site_component->command_interface(),
        requested_fields  => [ 'configuration-state', ],
    );

    foreach my $state (@config_states) {
        $Log->comment( "Configuration state for node $state->{'node'} "
              . "is $state->{'configuration_state'}" );
        if ( lc $state->{'configuration_state'} ne 'configured' ) {
            NACL::MTask::MCC::Exceptions::BaseException->throw(
                'Configure failed! Incorrect Metrocluster node show output');
        }
    }

    $Log->comment('Checking output for metrocluster operation show');
    $Log->comment( 'The output should show operation field as configure and '
          . 'state as successful' );

    $output        = $apiset->metrocluster_operation_show();
    $parsed_output = @{ $output->{parsed_output} }[0];
    if (   lc $parsed_output->{'operation'} ne 'configure'
        && lc $parsed_output->{'state'} ne 'successful' )
    {
        NACL::MTask::MCC::Exceptions::BaseException->throw(
            'Configure failed! Incorrect Metrocluster operation show output');
    }

    return;
}

=head2 wipeconfig()

B<DEPRECATED>

wipeconfig API is deprecated, use unconfigure() API instead

=cut

sub wipeconfig {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;
    $Log->warn("wipeconfig is deprecated, please use unconfigure");
    $self->unconfigure(@_);

    return;
}

=head2 menu()

B<EXPERIMENTAL!>

This will be removed at some point to be replaced with a different structure
(put the same/similar functionality).

Displays an interactive cli utility menu to call common library functionality.

=cut

sub menu {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my $exit = 0;

    my @macro;
    my $running_macro       = 0;
    my $recording_macro     = 0;
    my $macro_filename      = '';
    my $macro_play_filename = '';
    my @macro_commands;
    my $wait_on_comment = 1;

    my $prompt      = '>';
    my $prompt_info = 'mcc';
    my $file        = 'mccmenu.macro';

    #TBD: throughout, dr_state needs to be determined dynamically
    my $dr_state = 'normal';    #should query this I guess...

    my $setting;

    my $command = {

        'dr_1normal' => {
            name => 'cycle to dr state: normal',
            code => sub {
                my $local_setting = shift;
                $self->get_cluster( site => $local_setting->{'survivor-site'} )
                  ->cycle_to_DR_state(
                    state  => 'normal',
                    verify => { disks => $local_setting->{'verify-disks'} }
                  );
                $dr_state = 'normal';
            },
        },

        'dr_2switchover' => {
            name => 'cycle to dr state: switchover_completed',
            code => sub {
                my $local_setting = shift;
                $self->get_cluster( site => $local_setting->{'survivor-site'} )
                  ->cycle_to_DR_state(
                    state  => 'switchover_completed',
                    verify => { disks => $local_setting->{'verify-disks'} }
                  );
                $dr_state = 'switchover_completed';
            },
        },

        'dr_3heal_aggrs' => {
            name => 'cycle to dr state: heal_aggrs_completed',
            code => sub {
                my $local_setting = shift;
                $self->get_cluster( site => $local_setting->{'survivor-site'} )
                  ->cycle_to_DR_state(
                    state  => 'heal_aggrs_completed',
                    verify => { disks => $local_setting->{'verify-disks'} }
                  );
                $dr_state = 'heal_aggrs_completed';
            },
        },

        'dr_4heal_roots' => {
            name => 'cycle to dr state: heal_roots_completed',
            code => sub {
                my $local_setting = shift;
                $self->get_cluster( site => $local_setting->{'survivor-site'} )
                  ->cycle_to_DR_state(
                    state  => 'heal_roots_completed',
                    verify => { disks => $local_setting->{'verify-disks'} }
                  );
                $dr_state = 'heal_roots_completed';
            },
        },

        'dump_mcc' => {
            name => 'dump metrocluster object',
            code => sub {
                my $local_setting = shift;
                $Log->comment(
                    NACL::MTask::MCC::Utils->dump(
                        target              => $self,
                        show_shared_storage => $local_setting->{'verify-disks'}
                    )
                );
              }
        },

        'disk_verify' => {
            name => 'do disk verification',
            code => sub {
                foreach my $dr ( sort keys %{ $self->get_DR_groups() } ) {
                    my $message = "validate shared storage in dr group $dr";
                    $Log->comment($message);
                    print ntest::NTEST_STDOUT "$message\n";

                    $self->get_DR_groups()->{$dr}->get_shared_storage()
                      ->validate_disk_states();
                }
            },
        },

        'show_disasters' => {
            name => 'show DR and HA states',
            code => sub {
                try {
                    my $disaster_status =
                      $self->get_disaster_status( include_HA => 1 );
                    my $status_message = "overall disaster status:\n"
                      . NACL::MTask::MCC::Utils->dump($disaster_status);
                    $Log->comment($status_message);
                }
                catch NACL::MTask::MCC::Exceptions::BaseException with {
                    my $exception = shift;
                    my $text      = $exception->text();

                    my $message = sprintf( "%s", <<"END_PRINT");

Exception occurred during disaster status queries:

$text

This may be due to the DR or HA state currently transitioning, or it may be due
to an invalid DR or HA state.  You may safely repeat this query.  If this
query continues to fail in this manner for several minutes, it is likely that
the latter case is the reason for this failure, and you will need to triage
the MCC system manually.

END_PRINT

                    $Log->comment($message);
                };
            },
        },
    };

    my $setting_spec = {
        'survivor-site' => { default => 'A', regex => qr/^[AB]$/ },
        'verify-disks'  => { default => 1,   regex => qr/^[01]$/ },
    };

    $setting = validate_with(
        params => $setting || {},
        spec => $setting_spec,
    );

    my @history;

    my $fiji;
    my $fijifile = 'mcc.fiji';
    if ( -e $fijifile ) {

        if ( open( FIJI, '<', $fijifile ) ) {
            while ( my $line = <FIJI> ) {
                my ( $name, $filter ) = split( /=/, $line, 2 );
                chomp $filter;
                $fiji->{$name}->{fiji} = $filter;
            }
            close FIJI;
            print ntest::NTEST_STDOUT "read fiji file:" . Dumper $fiji;
        }

    }
    while ( !$exit ) {

        print ntest::NTEST_STDOUT Dumper $setting;

        #do this better, this is a quick test here
        my @order;
        my %itemorder;
        my $count = 1;
        foreach my $item ( sort keys %$command ) {
            $order[$count] = $item;
            $itemorder{$item} = $count;
            print ntest::NTEST_STDOUT "$count: $command->{$item}->{name}\n";
            $count++;
        }

        print ntest::NTEST_STDOUT "f: fiji\n";
        print ntest::NTEST_STDOUT "l: get mcc filer logs\n";
        print ntest::NTEST_STDOUT "m: "
          . ( $recording_macro ? "stop" : "start" )
          . " recording macro"
          . ( $recording_macro ? " [$macro_filename]\n" : "\n" );
        print ntest::NTEST_STDOUT "p: play macro"
          . ( $running_macro ? " [$macro_play_filename]\n" : "\n" );
        print ntest::NTEST_STDOUT "r: refresh command interfaces\n";
        print ntest::NTEST_STDOUT "q: quit\n";
        print ntest::NTEST_STDOUT "#: add comment to log\n";
        print ntest::NTEST_STDOUT "$prompt_info$prompt";

        my $in;
        if ($running_macro) {
            if ( my $next = shift @macro ) {
                chomp $next;
                if ( defined $itemorder{$next} ) {
                    $in = $itemorder{$next};
                }
                else {
                    if ( $next =~ /^#/ ) {
                        print ntest::NTEST_STDOUT "$next\n";
                        if ($wait_on_comment) {
                            print ntest::NTEST_STDOUT
                              "hit return to continue\n";
                            my $wait_in = <STDIN>;
                        }
                    }
                    elsif ( $next eq 'm' ) {

           #we aren't currently supporting recording macros during a macro play!
                        print ntest::NTEST_STDOUT
                          "Did not execute record (m)acro command in macro\n";
                    }
                    elsif ( $next eq 'q' ) {
                        $in = '';    #we don't want to quit do we?  or do we...
                        print ntest::NTEST_STDOUT
                          "Did not execute (q)uit command in macro\n";

                    }
                    else {
                        $in = $next;
                    }
                }

                print ntest::NTEST_STDOUT "macro command:$in\n";
            }
            else {
                print ntest::NTEST_STDOUT "macro completed\n";
                $running_macro = 0;
            }

        }
        else {
            $in = <STDIN>;
            chop $in;
            push @history, ( $in !~ /\D/ ? $order[$in] || $in : $in );
            push @macro_commands, $history[-1];
        }

        my @set = split /=/, $in, 2;    #if it's a set...

        if ( lc $in eq 'q' ) {
            if ( $dr_state eq 'normal' ) {    #should query this...
                $exit = 1;
            }
            else {
                print ntest::NTEST_STDOUT
"WARNING! Metrocluster NOT in normal state.  Switch to normal before quitting.\n";
            }
        }
        elsif ( $in =~ /^#/ ) {
            $Log->comment("comment:$in");
        }
        elsif ( $in eq 'c' ) {
            eval { $fiji->clear() if $fiji; };
        }
        elsif ( $in eq 'f' ) {

            #fiji
            my $ci;
            if ( $setting->{'survivor-site'} eq 'A' ) {
                $ci =
                  $self->_site_A_metrocluster_component()->command_interface();
            }
            else {
                $ci =
                  $self->_site_B_metrocluster_component()->command_interface();
            }

            my $fcount = 1;
            my @forder;
            my %fitemorder;

            foreach my $item ( sort keys %{$fiji} ) {
                my $status = $fiji->{$item}->{status} || '-';
                print ntest::NTEST_STDOUT "$fcount: [$status] $item"
                  . (
                    $item eq $fiji->{$item}->{fiji}
                    ? "\n"
                    : " [$fiji->{$item}->{fiji}]\n"
                  );

                #($item eq $fiji->{$item}->{object}->{program} ?
                #         "\n" : " [$fiji->{$item}->{object}->{program}]\n");
                $forder[$fcount] = $item;
                $fitemorder{$item} = $fcount;
                $fcount++;
            }

            print ntest::NTEST_STDOUT "c: clear fiji\n";

            #print ntest::NTEST_STDOUT "d: delete fiji\n"; #TODO
            print ntest::NTEST_STDOUT "n: new fiji\n";
            print ntest::NTEST_STDOUT "s: set fiji\n";
            print ntest::NTEST_STDOUT "<hit return to exit to main menu>\n";
            print ntest::NTEST_STDOUT "fiji$prompt";

            my $fin;

            if ($running_macro) {
                if ( my $next = shift @macro ) {
                    chomp $next;
                    if ( defined $fitemorder{$next} ) {
                        $fin = $fitemorder{$next};
                    }
                    else {
                        if ( $next =~ /^#/ ) {
                            print ntest::NTEST_STDOUT "$next\n";
                            if ($wait_on_comment) {
                                print ntest::NTEST_STDOUT
                                  "hit return to continue\n";
                                my $wait_in = <STDIN>;
                            }
                        }
                        else {
                            $fin = $next;
                        }
                    }
                    print ntest::NTEST_STDOUT "macro command:$fin\n";
                }
            }
            else {
                $fin = <STDIN>;

                chop $fin;
                $fin = lc $fin;

                push @history, ( $fin !~ /\D/ ? $forder[$fin] || $fin : $fin );
                push @macro_commands, $history[-1];
            }

            if ( $fin eq 'c' ) {
                print ntest::NTEST_STDOUT "fiji # to clear$prompt";
                my $number;
                if ($running_macro) {
                    if ( my $next = shift @macro ) {
                        chomp $next;
                        if ( defined $fitemorder{$next} ) {
                            $number = $fitemorder{$next};
                        }
                        else {
                            if ( $next =~ /^#/ ) {
                                print ntest::NTEST_STDOUT "$next\n";
                                if ($wait_on_comment) {
                                    print ntest::NTEST_STDOUT
                                      "hit return to continue\n";
                                    my $wait_in = <STDIN>;
                                }
                            }
                            else {
                                $number = $next;
                            }
                        }
                        print ntest::NTEST_STDOUT "macro command:$number\n";
                    }
                }
                else {
                    $number = <STDIN>;

                    chop $number;
                    $number = lc $number;

                    push @history,
                      (
                          $number !~ /\D/
                        ? $forder[$number] || $number
                        : $number
                      );
                    push @macro_commands, $history[-1];
                }

                #my $number = <STDIN>;
                #chop $number;
                my $item = $forder[$number] || next;
                eval {
                    print ntest::NTEST_STDOUT "fiji state?\n";
                    my $fs = $fiji->{$item}->{object}->state();
                    print ntest::NTEST_STDOUT "fiji state:" . Dumper $fs;
                };
                if ( $fiji->{$item}->{status} eq 's' ) {
                    eval {
                        $fiji->{$item}->{object}->clear();
                        $fiji->{$item}->{status} = '';
                    };
                }
                else {
                    print ntest::NTEST_STDOUT
                      "fiji $item not set, can't clear\n";
                }

            }
            elsif ( $fin eq 'n' ) {
                print ntest::NTEST_STDOUT "fiji filter$prompt";
                my $fiji_program = <STDIN>;
                chop $fiji_program;
                print ntest::NTEST_STDOUT "fiji name/tag$prompt";
                my $fiji_tag = <STDIN>;
                chop $fiji_tag;
                $fiji_tag ||= $fiji_program;

                my $fijiobj = NACL::STask::Fiji->new(
                    command_interface => $ci,
                    tag               => $fiji_tag,
                    program           => $fiji_program,
                );
                $fiji->{$fiji_tag}->{fiji}   = $fiji_program;
                $fiji->{$fiji_tag}->{object} = $fijiobj;

                if (1) {    #write fijis to file

                    if ( open( FIJI, '>>', $fijifile ) ) {
                        my $filetag = $fiji_tag;
                        $filetag =~ s/=/_/g;    #?? not sure if = is valid token
                        print FIJI "$filetag=$fiji_program\n";
                        close FIJI;
                    }
                }

                #print ntest::NTEST_STDOUT Dumper $fiji;
            }
            elsif ( $fin eq 's' ) {
                print ntest::NTEST_STDOUT "fiji # to set$prompt";
                my $number;
                if ($running_macro) {
                    if ( my $next = shift @macro ) {
                        chomp $next;
                        if ( defined $fitemorder{$next} ) {
                            $number = $fitemorder{$next};
                        }
                        else {
                            if ( $next =~ /^#/ ) {
                                print ntest::NTEST_STDOUT "$next\n";
                                if ($wait_on_comment) {
                                    print ntest::NTEST_STDOUT
                                      "hit return to continue\n";
                                    my $wait_in = <STDIN>;
                                }
                            }
                            else {
                                $number = $next;
                            }
                        }
                        print ntest::NTEST_STDOUT "macro command:$number\n";
                    }
                }
                else {
                    $number = <STDIN>;

                    chop $number;
                    $number = lc $number;

                    push @history,
                      (
                          $number !~ /\D/
                        ? $forder[$number] || $number
                        : $number
                      );
                    push @macro_commands, $history[-1];
                }

                #my $number = <STDIN>;
                #chop $number;
                my $item = $forder[$number] || next;
                unless ( $fiji->{$item}->{object} ) {
                    $fiji->{$item}->{object} = NACL::STask::Fiji->new(
                        command_interface => $ci,
                        tag               => $item,
                        program           => $fiji->{$item}->{fiji},
                    );

                }
                if ( $fiji->{$item}->{status} ne 's' ) {
                    eval {
                        $fiji->{$item}->{object}->set();
                        $fiji->{$item}->{status} = 's';

                    };
                }
                else {
                    print ntest::NTEST_STDOUT
                      "fiji $item not set, can't clear\n";
                }

            }
            else {
                #exit
            }

        }
        elsif ( $in eq 'm' ) {
            if ($recording_macro) {

                print ntest::NTEST_STDOUT
"\nAre you sure you want to stop recording macro $macro_filename? (y/[n]) ";

                my $verify = <STDIN>;
                if ( $verify =~ /y/ ) {
                    print ntest::NTEST_STDOUT
                      "macro filename (return to cancel)? ";

                    if ( open( MACROFILE, '>>', $macro_filename . '.macro' ) ) {
                        print MACROFILE join "\n", @macro_commands;
                        close MACROFILE;
                        $recording_macro = 0;
                        $macro_filename  = '';
                    }
                    else {
                        print ntest::NTEST_STDOUT
                          "warning, could not open macro $file\n";
                    }
                }
            }
            else {
                print ntest::NTEST_STDOUT "macro filename (return to cancel)? ";
                $macro_filename = <STDIN>;
                chop $macro_filename;
                next if $macro_filename eq '';
                print ntest::NTEST_STDOUT "Recording macro $macro_filename\n";

                if ( open( MACROFILE, '>', $macro_filename . '.macro' ) ) {
                    print MACROFILE "version=1\n";

                    close MACROFILE;
                    $recording_macro = 1;
                    @macro_commands  = ();    #start over
                }
                else {
                    print ntest::NTEST_STDOUT
                      "warning, could not open macro $macro_filename\n";
                }
            }

        }
        elsif ( $in eq 'p' ) {
            print ntest::NTEST_STDOUT "macro filename (return to cancel)? ";
            my $macro_filename = <STDIN>;
            chop $macro_filename;
            next if $macro_filename eq '';
            $macro_filename .= '.macro' unless $macro_filename =~ /\.macro$/;
            if ( open( FILE, '<', $macro_filename ) ) {
                my @macro = <FILE>;
                close FILE;
                print ntest::NTEST_STDOUT @macro;
            }
            print ntest::NTEST_STDOUT
              "Are you sure you want to play macro $macro_filename? (y/n) ";
            my $verify = <STDIN>;
            if ( $verify =~ /y/ ) {
                $Log->comment("Replaying macro $file");
                print ntest::NTEST_STDOUT "Replaying macro $macro_filename\n";

                if ( open( FILE, '<', $macro_filename ) ) {
                    @macro = <FILE>;
                    close FILE;
                    $running_macro = 1;
                    ( $macro_play_filename = $macro_filename ) =~ s/\.macro//;
                }
                else {
                    print ntest::NTEST_STDOUT
                      "warning, could not open macro $macro_filename\n";
                }

            }
        }
        elsif ( $in eq 'r' ) {
            $self->refresh_command_interfaces();
        }
        elsif ( $in eq 'l' ) {
            ## no critic (InputOutput::ProhibitExplicitStdin)
            print ntest::NTEST_STDOUT
              "enter burt # <or leave blank to write logs to logdir>\n";
            my $submenu = 'log';
            print ntest::NTEST_STDOUT "$submenu$prompt";

            my ($burt) = <STDIN> =~ /(\d*)/;
            my $path = q{};

            if ($burt) {
                print ntest::NTEST_STDOUT "BURT: $burt\n";
                my $ci;
                if ( $setting->{'survivor-site'} eq 'A' ) {
                    $ci =
                      $self->_site_A_metrocluster_component()
                      ->command_interface();
                }
                else {
                    $ci =
                      $self->_site_B_metrocluster_component()
                      ->command_interface();
                }

                #figure out the logs path
                my $show = NACL::CS::SystemNode->fetch(
                    command_interface => $ci,
                    requested_fields  => ['location']
                );
                my $location = $show->{location} || 'rtp';
                print ntest::NTEST_STDOUT "filer location:$location\n";

                $path = "/x/eng/$location-cores/burt.$burt";
            }
            else {
                $path = Tharn::param('LOGDIR') . '/logs';
            }
            print ntest::NTEST_STDOUT "log destination path:$path\n";

            print ntest::NTEST_STDOUT "p: paloma info\n";
            print ntest::NTEST_STDOUT
              's: controller scraper (adds san logs, core files, more -'
              . " can be SLOW -- 30 min)\n";
            print ntest::NTEST_STDOUT "<hit return to exit to main menu>\n";
            print ntest::NTEST_STDOUT "$submenu$prompt";
            my ($script) = <STDIN> =~ /([p|s])/i;

            if ($script) {
                my $eval_return = eval {
                    $self->_get_logs(
                        nodes    => $self->get_all_mcc_nodes(),
                        log_path => $path,
                        use_tool => lc $script
                    );
                    1;
                };
                unless ($eval_return) { print ntest::NTEST_STDOUT $@; }
            }
        }
        elsif ( $in eq '' ) {

            #empty
        }
        elsif ( defined $command->{ $order[$in] } ) {
            my $selected = $order[$in];
            if ( defined $command->{$selected} ) {    # TODO REMOVE LINE
                my $commandref = $command->{$selected};
                print ntest::NTEST_STDOUT
                  "execute command: $commandref->{name} [$selected]\n";
                $Log->comment("execute command: $selected");
                if ( defined $commandref->{code} ) {

                    print ntest::NTEST_STDOUT "execute code: $selected\n";
                    eval {
                        #time just this menu call, whatever it is
                        my $trace_menu =
                          NACL::MTask::MCC::Log::TRACELOG->start_timer(
                            message => "menu command $selected" );    #metrics
                        $commandref->{code}->($setting);
                    };
                    if ($@) { print ntest::NTEST_STDOUT $@; }
                }
            }

        }
        elsif ( defined $setting->{ $set[0] } ) {
            print ntest::NTEST_STDOUT "set value: $in\n";
            if ( $set[1] =~ /$setting_spec->{$set[0]}->{regex}/ ) {
                $setting->{ $set[0] } = $set[1];
            }
            else {
                print ntest::NTEST_STDOUT
                  "$set[1] not a valid value for $set[0]\n";
            }

        }
        else {    #???
            print ntest::NTEST_STDOUT "unknown command: $in\n";
        }

    }

    #move this to a _method
    $Log->comment( "command history: " . join ',', @history );
    if ( open( FILE, '>', $file ) ) {
        print FILE "version=1\n";
        print FILE join "\n", @history;
        close FILE;
    }
    else {
        $Log->comment(
            "warning, could not open $file to save command history macro");
    }

    return;
}

sub _get_logs {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self     = shift;
    my %raw_opts = @_;

    # validate opts
    my %opts = validate_with(
        params => \%raw_opts,
        spec   => {
            nodes            => { type => ARRAYREF },
            paloma_info_path => {
                type    => SCALAR,
                default => '/usr/software/test/bin/paloma_info.pl',
            },
            use_tool     => { type => SCALAR, default => 'p' },
            scraper_path => {
                type    => SCALAR,
                default => '/usr/software/test/agnostic/san/'
                  . 'fit/ControllerScraper.py',
            },
            log_path => {
                type    => SCALAR,
                default => Tharn::param('LOGDIR') . '/logs'
            },
            compress       => { regex => qr/^[01]$/,     default  => 0 },
            admin_password => { type  => SCALAR | UNDEF, optional => 1 },
            diag_password  => { type  => SCALAR | UNDEF, optional => 1 },
        },
    );

    my $nodes = delete $opts{nodes};

    # Start log pulling sessions in parallel
    my @subtest_handles = ();
    foreach my $node ( @{$nodes} ) {
        my $node_name = $node->get_name();
        $Log->comment("Fetching logs from : $node_name");

        my $cmd;

        if ( $opts{use_tool} eq 's' ) {
            my $toolname = 'Controller Scraper';
            print ntest::NTEST_STDOUT
              "Writing logs from $node_name to $opts{log_path} "
              . "with $toolname\n";
            $cmd = "$opts{scraper_path}";

            #$cmd .= ' -z' if $opts{compress}; #not an option
            $cmd .= ' -v';
            $cmd .= " -y $opts{admin_password}" if $opts{admin_password};
            $cmd .= " -z $opts{diag_password}" if $opts{diag_password};
            $cmd .= " -t $opts{log_path}" if $opts{log_path};
            $cmd .= " -n $node_name";    #or we can pass comma sep list??
        }
        else {                           #default to paloma
            my $toolname = 'paloma info';
            print ntest::NTEST_STDOUT
              "Writing logs from $node_name to $opts{log_path} "
              . "with $toolname\n";
            $cmd = "$opts{paloma_info_path}";
            $cmd .= ' -z'                       if $opts{compress};
            $cmd .= " -a $opts{admin_password}" if $opts{admin_password};
            $cmd .= " -d $opts{diag_password}"  if $opts{diag_password};
            $cmd .= " -p $opts{log_path}"       if $opts{log_path};
            $cmd .= " -f $node_name";    #must be LAST (well really just node)
        }

        print "$cmd\n";
        ## no critic (Modules::ProhibitConditionalUseStatements)
        push @subtest_handles, subtest(
            sub {
                use NACL::C::Client;
                use Net::Domain qw(hostname);

                # Execute the command on client running this script
                my $client_hostname = hostname()
                  or croak 'Unable to get client hostname';
                $Log->comment("Using client: $client_hostname");

                my $client = NACL::C::Client->new( name => $client_hostname );
                $client->apiset->set_timeout( 'connectrec-timeout' => 3600 );
                $client->execute( command => $cmd );
            },
            '-bg',
            '-runid',
            $node_name . '_log_extraction'
        );
    }
    Subtest::wait_finish( subtests => \@subtest_handles );

    return;

}

# just logs some information about the MCC system
sub _log_some_mcc_system_info {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            disaster_status => { type => HASHREF },
            constructor_log => { type => SCALAR | UNDEF },
        },
    );

    my $disaster_status = delete( $opts{disaster_status} );
    my $constructor_log = delete( $opts{constructor_log} );

    my $dr_state_text = $disaster_status->{DR_state};
    ( $disaster_status->{DR_state} ne 'normal' )
      && ( $dr_state_text .=
        ', survivor site: ' . $disaster_status->{survivor_site} );

    $Log->comment('Your Metrocluster system:');
    $Log->comment( 'DR State: ' . $dr_state_text );
    foreach my $dr_group ( @{ $self->_dr_groups() } ) {
        my $dr_group_id = $dr_group->get_id();

        $Log->comment( '  DR Group ID: ' . $dr_group_id );

        $Log->comment(
            '    DR Group Node A1: ' . $dr_group->get_A1_node()->get_name() );

        $Log->comment('    DR Partners: ( Site A  <===>  Site B )');
        my $nodes_by_role = $dr_group->get_nodes_by_role();
        $Log->comment( q/            /
              . $nodes_by_role->{A1}->get_name()
              . ' <===> '
              . $nodes_by_role->{B1}->get_name() );

        # skip for 2-node systems
        if ( exists( $nodes_by_role->{A2} ) ) {
            $Log->comment( q/            /
                  . $nodes_by_role->{A2}->get_name()
                  . ' <===> '
                  . $nodes_by_role->{B2}->get_name() );

            $Log->comment(
                '    HA States: '
                  . NACL::MTask::MCC::Utils->dump(
                    $disaster_status->{HA_states}->{$dr_group_id}
                  )
            );

            $Log->comment('    Shared Storage Intersite Fabrics: ');

            try {
                my $topology = $dr_group->get_network_topology();
                foreach my $fabric ( @{ $topology->get_intersite_fabrics() } ) {
                    $Log->redirect( path => $constructor_log )
                      if $constructor_log;
                    my $site_A_switch_desc =
                      $fabric->get_switch( site => 'A' )->get_description();
                    my $site_B_switch_desc =
                      $fabric->get_switch( site => 'B' )->get_description();
                    $Log->reset_path() if $constructor_log;
                    $Log->comment( q/            /
                          . $site_A_switch_desc
                          . ' <===> '
                          . $site_B_switch_desc );
                }
            }
            catch NACL::MTask::MCC::Exceptions::BaseException with {
                my $exception = shift;

                $Log->comment( q{        }
                      . 'Shared storage network topology discovery not enabled.'
                );
            }
        }

        $Log->comment("\n");
    }

    return;
}

# validate passed parameters.  throw exception if user passed in a
# strange/invalid set of values in nodes.  (e.g., if there
# are nodes listed not in the same MCC system, etc.)
sub _validate_node_names_passed_by_user {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => { node_names => { type => ARRAYREF }, },
    );

    my $node_names = delete( $opts{node_names} );

    my %check_list = map { $_ => 1 } @{$node_names};

    # complain if any nodes in this list are not in one of the DR Groups
    foreach my $dr_group ( values( %{ $self->get_DR_groups() } ) ) {
        my $group_nodes = $dr_group->get_mcc_nodes();
        foreach my $mcc_node ( @{$group_nodes} ) {
            if ( exists( $check_list{ $mcc_node->get_name() } ) ) {
                delete( $check_list{ $mcc_node->get_name() } );
            }
        }
    }

    # anything left in list is a problem
    if ( scalar( keys(%check_list) ) ) {
        my $msg =
            "The following nodes passed in the nodes parameter do not\n"
          . "    belong to any DR Group in this MCC system\n\n:"
          . "        "
          . join( "\n        ", keys(%check_list) ) . "\n\n"
          . "    Throwing an exception to prevent possible use of\n"
          . "    incorrect system...";
        NACL::MTask::MCC::Exceptions::BaseException->throw($msg);
    }

    return;
}

=head2 verify_data_integrity()

  $mcc->verify_data_integrity(
    aggregates => [ qw/aggr1 aggr2 aggr3/ ]
  );

This method will verify all aggregates associated with the
metrocluster, or aggregates passed in either by name or by
NACL::C::Aggregate object.

=item Options:

=over 4

=item C<< aggregates =E<gt> \@aggregates >>

(Optional) List of aggregate names or aggregate objects. By default runs on all mirrored aggregates

=item C<< wafliron_timeout =E<gt> 9600 >>

(Optional) NACL timeout passed to wafliron command. Defaults to 9600.

=item C<< aggr_verify =E<gt> (0|1) >>

(Optional) If true, performs an aggregate verify for each aggregate in
parallel. Defaults to 1.

=item C<< scrub =E<gt> (0|1) >>

(Optional) If true, performs a 'storage aggregate scrub start' for each aggregate
in parallel, then waits for the scrub to complete. Defaults to 0.

=item C<< run_wafliron =E<gt> (0|1) >>

(Optional) If true, runs a wafliron on each aggregate. Wafliron happens after any
aggr verifies and/or scrubs have completed. Defaults to 1.

=item C<< aggregate_timeout =E<gt> 3600 >>

(Optional) The maximum amount of time to run the aggregate verify and scrub
commands (each). Defaults to 3600.

=back

=back

=cut

sub verify_data_integrity {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'aggregates'       => { 'type' => ARRAYREF, 'optional' => 1 },
            'wafliron_timeout' => { 'type' => SCALAR,   'default'  => 9600 },
            'aggr_verify' =>
              { 'type' => SCALAR, regex => qr/^[01]$/, default => 1 },
            'scrub' => { 'type' => SCALAR, regex => qr/^[01]$/, default => 0 },
            'aggregate_timeout' => { 'type' => SCALAR, 'default' => 3600 },
            'run_wafliron' =>
              { 'type' => SCALAR, regex => qr/^[01]$/, default => 1 },
            disaster_status => { type => HASHREF, optional => 1, },
        },
    );

    # Get the aggregates
    my @aggregates;
    if ( defined $opts{'aggregates'} ) {
        foreach my $aggr_name ( @{ $opts{'aggregates'} } ) {
            my $aggregate;
            if ( !ref $aggr_name ) {
                $aggregate =
                  $self->_get_aggregates( 'aggregate' => $aggr_name );
            }
            elsif ( $aggr_name->isa('NACL::C::Aggregate') ) {
                $aggregate = $aggr_name;
            }
            else {
                NACL::MTask::MCC::Exceptions::BaseException->throw(
                    "Invalid aggregate '$aggr_name'");
            }
            push( @aggregates, $aggregate );
        }
    }
    else {
        @aggregates = $self->_get_aggregates( root => 'false' );
    }

    # Check all aggregates for raid status
    my @raid_subtests;
  AGGREGATE: foreach my $aggr (@aggregates) {
        if ( $aggr->get_one_state_attribute('raidstatus') =~ /mirror degraded/ )
        {
            $Log->warn( 'Found degraded mirrored aggregate ~ '
                  . 'Aggregate verification skipped on '
                  . $aggr->aggregate );
            next AGGREGATE;
        }

        if ( $aggr->get_one_state_attribute('raidstatus') !~ /mirror/ ) {
            $Log->warn( 'Found unmirrored aggregate ~ '
                  . 'Aggregate verification skipped on '
                  . $aggr->aggregate );
            next AGGREGATE;
        }

        $Log->comment( ' Ensure that the aggregate '
              . $aggr->aggregate()
              . ' is not in resyncing state' );
        try {
            $self->wait_for_aggr_resync(
                aggregates => [$aggr],
                reason     => 'During aggr verify',
                (
                    defined $opts{disaster_status}
                    ? ( disaster_status => $opts{disaster_status} )
                    : ()
                ),
            );
        }
        catch NACL::APISet::Exceptions::TimeoutException with {
            $Log->comment( $aggr->aggregate()
                  . ' is still resyncing'
                  . ' ... ignoring it for data verification' );
            next AGGREGATE;
        };

        $Log->comment(
            'Starting aggregate verification on ' . $aggr->aggregate );

        if ( $opts{'aggr_verify'} ) {

            push @raid_subtests, subtest(
                sub {
                    $aggr->verify(
                        action           => 'start',
                        nacltask_wait    => 1,
                        nacltask_verify  => 1,
                        polling_interval => 30
                    );

                    # If we got here without a disaster, it's a success
                    my $r = NATE::Result::Pass->new(
                            'Aggregate verify succeeded for aggregate "'
                          . $aggr->aggregate
                          . "'." );
                    $Log->log($r);

                },
                '-bg',
                '-runid',
                'aggr_verify_' . $aggr->aggregate,
                '-timeout' => $opts{'aggregate_timeout'}
            );

        }
        else {
            $Log->comment( 'Skipping aggr verify for ' . $aggr->aggregate );
        }

        if ( $opts{'scrub'} ) {
            push @raid_subtests, subtest(
                sub {
                    $aggr->scrub( action => 'start' );

                  STATUS: while (1) {

                        my @statuses =
                          NACL::C::StorageAggregate->show_scrub_status(
                            command_interface => $aggr->command_interface(),
                            allow_empty       => 1,

                          );

                        if ( !@statuses ) {
                            $Log->comment( 'No statuses found for status '
                                  . $aggr->aggregate() );
                            last STATUS;
                        }

                        my $running = 0;

                        foreach my $status (@statuses) {
                            $Log->comment( 'Aggr scrub status: '
                                  . NACL::MTask::MCC::Utils->dump($status) );

                            if ( $status->complete_percentage() ne '-' ) {
                                $running++;
                            }
                        }

                        if ( $running == 0 ) {
                            last STATUS;
                        }

                        Tharn::snooze(60);

                    }

                    # If we got here without a disaster, it's a success
                    my $r = NATE::Result::Pass->new(
                            'Aggregate scrub succeeded for aggregate "'
                          . $aggr->aggregate
                          . "'." );
                    $Log->log($r);

                },
                '-bg',
                '-runid',
                'aggr_scrub_' . $aggr->aggregate,
                '-timeout' => $opts{'aggregate_timeout'}
            );
        }
        else {
            $Log->comment( 'Skipping scrub for ' . $aggr->aggregate );
        }
    }

    # Can't run in parallel because aggregate verify wants aggrs
    # online and and wafliron wants them offline
    Subtest::wait_finish( subtests => \@raid_subtests );

    # Wafliron all aggregates
    if ( $opts{'run_wafliron'} ) {

        my @wafliron_subtests;

        foreach my $aggr (@aggregates) {
            next if lc $aggr->state()->root() eq 'true';

            push @wafliron_subtests, subtest(
                sub {
                    $Log->comment( 'Starting wafliron on ' . $aggr->aggregate );
                    $self->perform_wafliron(
                        aggr_obj         => $aggr,
                        nacltask_timeout => $opts{'wafliron_timeout'},
                    );
                },
                '-bg',
                '-runid',
                'WI_aggr_' . $aggr->aggregate(),
            );
        }

        Subtest::wait_finish( subtests => \@wafliron_subtests );

    }
    else {
        $Log->comment("Skipping wafliron for all aggregates");
    }

    return;
}

=head2 wait_for_aggr_resync()

    $mcc->wait_for_aggr_resync (
            aggregates => [ qw/aggr1 aggr2/],              (optional)
        resync_timeout => 3600,                            (optional)
        disaster_only  => 1,                               (optional)
        reason         => <reason_to_perform_aggr_resync>, (optional)

    );

This method will wait for aggr resync to complete on all aggregates in the
system, or for the aggregates passed in either by name or by
NACL::C::Aggregate object.

=over

=item Options

=item C<< aggregates => One or more aggregates in the system >>

(Optional) An arrayref of all aggregates associated with the
system, or aggregates passed in either by name or by
NACL::C::Aggregate object.
NOTE: if both 'aggregates' and 'disaster_only' are set, then the 'aggregates'
param will take precedence.

=item C<< resync_timeout => timeout in secs >>

(Optional) Default is 14400 secs.
The number of seconds to wait for the resync to complete.

=item C<< reason => The reason to call resync >>

(Optional) At what stage is aggr resync called.
This parameter is provided more for debugging purposes.
Aggr resync can be called multiple times during a test
script and all details get logged to the same log file.
The user can use this param to specify at which
stage is the resync being called.

=item C<< disaster_only => check for resync status only for disaster aggrs >>

(Optional) Useful when calling aggr resync as part of
prepare_for_swithcback_execution(). SB requires only disaster site aggrs
to be in 'normal, mirrored' state for SB to be successful. If this option is
set the method will fetch all the aggregates belonging to the disaster
nodes and check the resync status only for those aggrs.
NOTE: if both 'aggregates' and 'disaster_only' are set, then the 'aggregates'
param will take precedence.

=back

=cut

sub wait_for_aggr_resync {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'aggregates'      => { 'type' => ARRAYREF, 'optional' => 1 },
            'resync_timeout'  => { 'type' => SCALAR,   'default'  => 14400 },
            'reason'          => { 'type' => SCALAR,   'optional' => 1 },
            'disaster_only'   => { 'type' => SCALAR,   'optional' => 1 },
            'disaster_status' => { 'type' => HASHREF,  'optional' => 1, },
        },
    );

    my @aggregates;
    if ( defined $opts{'aggregates'} ) {
        $Log->comment(' User has specified aggrs to check resync status for ');
        foreach my $aggr ( @{ $opts{'aggregates'} } ) {
            my $aggregate;
            if ( !ref $aggr ) {
                $aggregate = $self->_get_aggregates( 'aggregate' => $aggr );
            }
            elsif ( $aggr->isa('NACL::C::Aggregate') ) {
                $aggregate = $aggr;
            }
            else {
                NACL::MTask::MCC::Exceptions::BaseException->throw(
                    "Invalid aggregate '$aggr'");
            }

            push( @aggregates, $aggregate );
        }
    }
    elsif ( !defined $opts{'aggregates'} && defined $opts{'disaster_only'} ) {
        $Log->comment( 'disaster_only option is set, only the disaster '
              . 'aggregates will be considered while checking  '
              . 'resync status' );
        my $disaster_status = $opts{disaster_status}
          || $self->get_disaster_status();

        my $disaster_nodes =
          $self->get_disaster_nodes( disaster_status => $disaster_status );

        my @dis_nodes = keys %{$disaster_nodes};
        $Log->comment(
            'Disaster nodes to be passed in:' . Dumper( \@dis_nodes ) );

        foreach my $node (@dis_nodes) {
            push @aggregates, $self->_get_aggregates( 'dr-home-name' => $node );
        }

    }
    elsif ( !defined $opts{'aggregates'} && !defined $opts{'disaster_only'} ) {
        @aggregates = $self->_get_aggregates();
    }

    if ( scalar(@aggregates) <= 0 ) {
        NACL::MTask::MCC::Exceptions::BaseException->throw(
            ' No aggregates were found to verify the resync operation');
    }

    $Log->comment(
        'Aggregates that will be checked for resync status ' . 'are: ' );
    foreach my $aggr (@aggregates) {
        $Log->comment( $aggr->aggregate() . "\n" );
    }

    #check resync status for all the aggregates
    my @resync_subtests;
    my $attempts = 5;
    my $current_attempt;
    foreach my $aggr_obj (@aggregates) {
        $Log->comment( ' Waiting for '
              . $opts{'resync_timeout'}
              . 'secs for '
              . ' aggr resync to complete on aggregate: '
              . $aggr_obj->aggregate );

        $current_attempt = 0;
        my $subtest_h = subtest(
            sub {
                if ( defined $opts{'reason'} ) {
                    $Log->comment( 'Checking resync status for '
                          . '-reason: '
                          . $opts{reason} );
                }

                while ( $current_attempt < $attempts ) {
                    $Log->comment( 'Attempting '
                          . $current_attempt . '/'
                          . $attempts
                          . ' for aggr resync check' );
                    my $polling_started = 0;
                    try {
                        #adding this code here because say for the first
                        #attempt when the lib tries to connect to the filer
                        #it gets rejected. The next attempt it does not have
                        #a 'alive' connection object to check the resync
                        #status. Hence adding code to refresh the connections.
                        $aggr_obj->command_interface()
                          ->refresh_command_interface( no_version_change => 1,
                          );

                        NACL::STask::Aggregate->wait_for_resync(
                            command_interface => $aggr_obj->command_interface(),
                            aggregate         => $aggr_obj->aggregate(),
                            'method-timeout'  => $opts{'resync_timeout'}
                        );

                        $polling_started = 1;
                    }
                    catch NATE::BaseException with {
                        my $exception = shift;

                        $Log->warn( 'NATE Exception encountered this could be '
                              . 'because > 10 SSH connections were being '
                              . 'attempted. Wait for 30 sec and try again' );

                        $Log->comment( $exception->text() );

                        $current_attempt++;
                        Tharn::snooze(30);
                    };

                    if ($polling_started) {
                        $Log->comment( 'Subtest started successfully not '
                              . 'making another attempt' );
                        last;
                    }
                }
            },
            '-bg',
            '-runid',
            'aggr_resync_' . $aggr_obj->aggregate(),
        );

        push( @resync_subtests, $subtest_h );

        #adding a 1sec delay to stagger the SSH calls.
        Tharn::snooze(1);
    }

    Subtest::wait_finish( subtests => \@resync_subtests );

    return;
}

=head2 verify_aggregate_status()

Verifies that the status of the aggregates is appropriate for the
current Metrocluster state.

=cut

sub verify_aggregate_status {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'die_on_failure' =>
              { 'type' => SCALAR, regex => qr/^[01]$/, default => 0 },
            'die_on_unknown_state' =>
              { 'type' => SCALAR, regex => qr/^[01]$/, default => 0 },
            'disaster_status' => { 'type' => HASHREF, 'optional' => 1 },
        }
    );

    my $disaster_status = $opts{'disaster_status'};

    if ( !defined $disaster_status ) {
        $disaster_status = $self->get_disaster_status();
    }

    my @errors;

    try {
        if ( $disaster_status->{'DR_state'} eq 'normal' ) {
            @errors = $self->_check_aggregate_status(
                disaster_status => $disaster_status,
                states          => [
                    {
                        name   => 'mirrored home normal',
                        filter => {
                            'is_home' => 1,
                            'mirror'  => 1,
                        },
                        raidstatus => [
                            [ 'mirrored', 'normal' ],
                            [ 'mirrored', "parity uninit'd!" ],
                            ['resyncing'],
                            ['reconstruct'],
                        ],
                        state => 'online',
                    },
                    {
                        name   => 'mirrored not home empty',
                        filter => {
                            'is_home' => 0,
                            'mirror'  => 1,
                        },
                        empty => 1,
                    },
                    {
                        name   => 'unmirrored home normal',
                        filter => {
                            'is_home' => 1,
                            'mirror'  => 0,
                        },
                        raidstatus => [
                            ['normal'], ["parity uninit'd!"], ['reconstruct'],
                        ],
                        state => 'online',
                    },
                ],

            );
        }
        elsif ( $disaster_status->{'DR_state'} eq 'switchover_completed' ) {

            if (
                !$self->L_is_negotiated_switchover(
                    'disaster_status' => $disaster_status
                )
              )
            {
                @errors = $self->_check_aggregate_status(
                    disaster_status => $disaster_status,
                    states          => [
                        {
                            name   => 'uso mirrored home',
                            filter => {
                                'is_home' => 1,
                                'mirror'  => 1,
                            },
                            raidstatus => [
                                [ 'mirrored', 'normal' ],
                                [ 'mirrored', "parity uninit'd!" ],
                                ['mirror degraded'],
                                ['resyncing'],
                                ['reconstruct'],
                            ],
                            state => 'online'
                        },
                        {
                            name   => 'uso unmirrored home',
                            filter => {
                                'is_home' => 1,
                                'mirror'  => 0,
                            },
                            raidstatus => [
                                ['normal'], ["parity uninit'd!"],
                                ['reconstruct'],
                            ],
                            state => 'online'
                        },
                        {
                            name   => 'uso mirrored away',
                            filter => {
                                'is_home' => 0,
                                'mirror'  => 1,
                            },
                            raidstatus => [
                                [ 'mirrored', 'normal' ],
                                [ 'mirrored', "parity uninit'd!" ],
                                ['mirror degraded'],
                                ['resyncing'],
                                ['reconstruct'],
                            ],
                        },
                        {
                            name       => 'uso unmirrored away',
                            raidstatus => [ ['-'], ],
                            state      => 'remote_cluster',
                        },
                        {
                            name       => 'post so mirrored',
                            filter     => { 'mirror' => 1, },
                            raidstatus => [
                                [ 'mirrored', 'normal' ],
                                [ 'mirrored', "parity uninit'd!" ],
                                ['mirror degraded'],
                                ['resyncing'],
                                ['reconstruct'],
                            ],
                        },
                    ]
                );

                # Otherwise we ARE negotiated switchover
            }
            else {

                @errors = $self->_check_aggregate_status(
                    disaster_status => $disaster_status,
                    states          => [
                        {
                            name   => 'nso mirrored home',
                            filter => {
                                'is_home' => 1,
                                'mirror'  => 1,
                            },
                            raidstatus => [

                                # For NSO we never expect ISLs to be
                                # disconnected, so no degraded aggrs
                                [ 'mirrored', 'normal' ],
                                [ 'mirrored', "parity uninit'd!" ],
                                ['reconstruct'],
                            ],
                            state => 'online',
                            empty => 0
                        },
                        {
                            name   => 'nso unmirrored home',
                            filter => {
                                'is_home' => 1,
                                'mirror'  => 0,
                            },
                            raidstatus => [
                                ['normal'], ["parity uninit'd!"],
                                ['reconstruct'],
                            ],
                            state => 'online',
                        },
                        {
                            name   => 'nso mirrored away offline',
                            filter => {
                                'is_home' => 0,
                                'state'   => 'offline',
                                'mirror'  => 1,
                            },
                            raidstatus => [
                                [ 'mirrored', 'normal' ],
                                [ 'mirrored', "parity uninit'd!" ],
                                ['mirror degraded'],
                                ['reconstruct'],
                            ],
                            empty => 0
                        },
                        {
                            name   => 'nso mirrored away online',
                            filter => {
                                'is_home' => 0,
                                'state'   => 'online',
                                'mirror'  => 1,
                            },
                            raidstatus => [
                                [ 'mirrored', 'normal' ],
                                [ 'mirrored', "parity uninit'd!" ],
                                ['reconstruct'],
                            ],
                            empty => 0
                        },
                    ]
                );
            }

        }
        elsif ( $disaster_status->{'DR_state'} eq 'heal_aggrs_completed' ) {

            @errors = $self->_check_aggregate_status(
                disaster_status => $disaster_status,
                states          => [
                    {
                        name   => 'heal_aggrs_completed mirrored home',
                        filter => {
                            'is_home' => 1,
                            'mirror'  => 1,
                        },
                        raidstatus => [
                            [ 'mirrored', 'normal' ],
                            [ 'mirrored', "parity uninit'd!" ],
                            ['resyncing'],
                            ['reconstruct'],
                        ],
                        state => 'online',
                        empty => 0
                    },
                    {
                        name   => 'heal_aggrs_completed unmirrored home',
                        filter => {
                            'is_home' => 1,
                            'mirror'  => 0,
                        },
                        raidstatus => [
                            ['normal'], ["parity uninit'd!"], ['reconstruct'],
                        ],
                        state => 'online',
                    },
                    {
                        name   => 'heal_aggrs_completed mirrored nonhome',
                        filter => {
                            'is_home' => 0,
                            'mirror'  => 1,
                        },
                        raidstatus => [
                            [ 'mirrored', 'normal' ],
                            [ 'mirrored', "parity uninit'd!" ],
                            ['mirror degraded'],
                            ['resyncing'],
                            ['reconstruct'],
                          ]

                    }
                ]
            );

        }
        elsif ( $disaster_status->{'DR_state'} eq 'heal_roots_completed' ) {
            @errors = $self->_check_aggregate_status(
                disaster_status => $disaster_status,
                states          => [
                    {
                        name   => 'heal_roots_completed mirrored home',
                        filter => {
                            'is_home' => 1,
                            'state'   => 'online',
                            'mirror'  => 1,
                        },
                        raidstatus => [
                            [ 'mirrored', 'normal' ],
                            [ 'mirrored', "parity uninit'd!" ],
                            ['resyncing'],
                            ['reconstruct'],
                        ],
                        state => 'online',
                        empty => 0
                    },
                    {
                        name   => 'heal_roots_completed unmirrored home',
                        filter => {
                            'is_home' => 1,
                            'state'   => 'online',
                            'mirror'  => 0,
                        },
                        raidstatus => [
                            ['normal'], ["parity uninit'd!"], ['reconstruct'],
                        ],
                        state => 'online',
                    },
                    {
                        name   => 'heal_roots_completed mirrored not home',
                        filter => {
                            'is_home' => 0,
                            'state'   => 'online',
                            'mirror'  => 1,
                        },
                        raidstatus => [
                            [ 'mirrored', 'normal' ],
                            [ 'mirrored', "parity uninit'd!" ],
                            ['resyncing'],
                            ['reconstruct'],
                        ],
                    },
                    {
                        name   => 'heal_roots_completed unknown',
                        filter => {
                            'is_home' => 0,
                            'state'   => 'unknown',
                        },
                        raidstatus => [ ['-'], ['resyncing'], ['reconstruct'], ]
                    },
                ]
            );

        }
        else {
            $Log->comment( "No aggregate validation performed for state "
                  . "'$disaster_status->{'DR_state'}'\n"
                  . "Disaster status:\n"
                  . Dumper($disaster_status) );

            if ( $opts{'die_on_unknown_state'} ) {
                NACL::MTask::MCC::Exceptions::Validation->throw(
                    "Unknown state '$disaster_status->{'DR_state'}'");
            }
            else {
                $Log->warn("Unknown state '$disaster_status->{'DR_state'}'");
            }
        }

        if (@errors) {
            my $message = join( "\n", @errors ) . "\n";

            $self->log_validation_event(
                type       => 'aggregate',
                successful => 0,
                summary    => "Aggregate validation failed: $message",
                results    => \@errors
            );

            if ( $opts{'die_on_failure'} ) {
                NACL::MTask::MCC::Exceptions::Validation->throw($message);
            }
            else {
                $Log->warn($message);
            }
            return;
        }
        else {
            $Log->comment( 'Passed aggregate status verify for state '
                  . "'$disaster_status->{'DR_state'}'" );

            $self->log_validation_event(
                type       => 'aggregate',
                successful => 1
            );

            return 1;
        }

    }
    catch NACL::MTask::MCC::Exceptions::DiskState::FakeInitState with {
        $Log->warn( 'This testbed was initialized using fake_init, '
              . 'which is not supported by the validation '
              . 'system. Skipping the aggregate status '
              . 'verify test.' );
        return;
    };

} ## end sub verify_aggregate_status

sub L_is_negotiated_switchover {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => { 'disaster_status' => { 'type' => HASHREF }, }
    );

    my $disaster_status = $opts{'disaster_status'};

    $disaster_status->{'DR_state'} eq 'normal' and return;

    my $cluster =
      $self->get_cluster( site => $disaster_status->{'survivor_site'} );

    return $cluster->is_negotiated_switchover();

} ## end sub L_is_negotiated_switchover

sub _check_aggregate_status {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'disaster_status' => { 'type' => HASHREF },
            'states'          => { 'type' => ARRAYREF },
        }
    );

    my $cluster =
      $self->_get_up_cluster( 'disaster_status' => $opts{disaster_status}, );

    my @aggrs = NACL::STask::Aggregate->find(
        command_interface => $cluster,
        allow_empty       => 1
    );

    my @errors;

    foreach my $spec ( @{ $opts{'states'} } ) {

        my @aggr_errors = $self->_check_aggregates_against_spec(
            aggrs => \@aggrs,
            spec  => $spec
        );

        push( @errors, @aggr_errors );
    }

    return @errors;

}

sub _check_aggregates_against_spec {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'aggrs' => { 'type' => ARRAYREF },
            'spec'  => { 'type' => HASHREF }
        }
    );

    my @errors;

    my @filtered_aggrs;

    if ( defined $opts{'spec'}{'filter'} ) {
        my $filter = $opts{'spec'}{'filter'};

      AGGR: foreach my $aggr ( @{ $opts{'aggrs'} } ) {
            my $aggr_state = $aggr->state();

            # Filter out the is-home state if needed
            if ( defined $filter->{'is_home'} ) {
                unless (
                    (
                        $filter->{'is_home'}
                        && ( $aggr_state->is_home() eq 'true' )
                    )
                    || !(
                        $filter->{'is_home'}
                        || ( $aggr_state->is_home() eq 'true' )
                    )
                  )
                {
                    next AGGR;
                }
            }

            # Filter out aggr based on "state" filter
            if ( defined $filter->{'state'} ) {
                my $state = $aggr_state->state();
                $state =~ s/^'//;
                $state =~ s/'$//;

                $filter->{'state'} eq $state
                  or next AGGR;
            }

            # Filter out aggr based on "mirror" filter
            if ( defined $filter->{'mirror'} ) {
                unless (
                    (
                        $filter->{'mirror'}
                        && ( $aggr_state->mirror() eq 'true' )
                    )
                    || !(
                        $filter->{'mirror'}
                        || ( $aggr_state->mirror() eq 'true' )
                    )
                  )
                {
                    $Log->comment('Skipping aggr on mirror filter...');
                    next AGGR;
                }
            }

            push( @filtered_aggrs, [ $aggr, $aggr_state ] );
        }
    }
    else {
        @filtered_aggrs = map { [ $_, $_->state() ] } @{ $opts{'aggr_states'} };
    }

    # If we're in fake init, skip it
    foreach my $filtered_aggr (@filtered_aggrs) {

        my ( $aggr, $aggr_state ) = @{$filtered_aggr};

        if ( $aggr_state->raidstatus =~ /parity uninit\'d/ ) {
            NACL::MTask::MCC::Exceptions::DiskState::FakeInitState->throw(
                'Found fake init state in ' . $aggr->aggregate() );
        }

    }

    if ( defined $opts{'spec'}{'empty'} ) {

        if ( $opts{'spec'}{'empty'} && scalar @filtered_aggrs ) {
            push( @errors, "Filter '$opts{'spec'}{'name'}' was not empty" );
            return @errors;
        }
        elsif ( !( $opts{'spec'}{'empty'} || scalar @filtered_aggrs ) ) {
            push( @errors, "Filter '$opts{'spec'}{'name'}' was empty" );
            return @errors;
        }

    }

    foreach my $aggr (@filtered_aggrs) {
        my @aggr_errors = $self->_check_aggregate_state_status(
            'aggr'       => $aggr->[0],
            'aggr_state' => $aggr->[1],
            'raidstatus' => $opts{'spec'}{'raidstatus'},
            'state'      => $opts{'spec'}{'state'}
        );

        my @annotated = ();

        foreach my $error (@aggr_errors) {
            push( @errors, $error . " for check $opts{'spec'}{'name'}" );
        }

    }

    return @errors;

} ## end sub _check_aggregates_against_spec

sub _check_aggregate_state_status {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'aggr'       => { 'type' => OBJECT },
            'aggr_state' => { 'type' => OBJECT },
            'state'      => 1,
            'raidstatus' => { 'type' => ARRAYREF },
        }
    );

    my $aggr        = $opts{'aggr'};
    my $aggr_state  = $opts{'aggr_state'};
    my $state       = $aggr_state->state();
    my $raid_status = $aggr_state->raidstatus();

    $raid_status =~ s/^"//;
    $raid_status =~ s/"$//;

    my @raid_status_words = split( /,\s*/, $raid_status );
    my %seen_words = ();

    foreach my $raid_status_word (@raid_status_words) {
        $seen_words{$raid_status_word} = 1;
    }

    # "Reconstructing" sometimes doesn't appear immediately,
    # so we use the raidgroup name as a helper, which will contain
    # 'reconstructing:' if it is doing so
    my @raidgroups = $aggr_state->raidgroups();
    foreach my $raidgroup (@raidgroups) {
        if (
            $raidgroup =~ m/\(.* \b (
                              reconstructing
                              |
                              recomputing \s parity
                              )
                                  :.*\)"?$/x
          )
        {
            $seen_words{'reconstruct'} = 1;
        }
    }

    my @errors;

    my @raid_errors;

  CASE: foreach my $case_words ( @{ $opts{'raidstatus'} } ) {

        my @case_errors;

        foreach my $required_word (@$case_words) {

            if ( !exists $seen_words{$required_word} ) {
                my $message =
                    "Aggregate raid status mismatch: aggregate '"
                  . $aggr->aggregate
                  . "' raid status: $raid_status "
                  . " did not have required word '$required_word'."
                  . " Words seen: "
                  . Dumper( \%seen_words )
                  . "\nRaidgroups: "
                  . Dumper( [ $aggr_state->raidgroups() ] );
                push( @case_errors, $message );
            }
        }

        if ( defined $opts{'state'} && $state ne $opts{'state'} ) {
            my $message =
                "Aggregate status mismatch: aggregate '"
              . $aggr->aggregate
              . "' has state: '$state', "
              . " expected '$opts{'state'}'";

            push( @case_errors, $message );
        }

        if (@case_errors) {
            @raid_errors = @case_errors;
        }
        else {
            @raid_errors = ();
            last CASE;
        }
    }

    push( @errors, @raid_errors );

    return @errors;

} ## end sub _check_aggregate_state_status

sub _get_up_cluster {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec => { 'disaster_status' => { 'type' => HASHREF, 'optional' => 1 }, }
    );

    my $disaster_status = $opts{'disaster_status'};

    if ( !$disaster_status ) {
        $disaster_status = $self->get_disaster_status();
    }

    # Return site A in normal state
    if ( $disaster_status->{'DR_state'} eq 'normal' ) {
        return $self->get_cluster( site => 'A' )->get_Cluster_component();
    }

    if ( first { $_ eq $disaster_status->{'disaster_site'} }
        @{ $disaster_status->{'live_sites'} } )
    {

        my $disaster_cluster_component =
          $self->get_cluster( site => $disaster_status->{'disaster_site'} )
          ->get_Cluster_component();

        return $disaster_cluster_component;

    }
    else {
        $Log->comment( "Not checking aggregate status for down site "
              . "'$disaster_status->{'disaster_site'}'" );
    }

    if ( first { $_ eq $disaster_status->{'survivor_site'} }
        @{ $disaster_status->{'live_sites'} } )
    {

        my $survivor_cluster_component =
          $self->get_cluster( site => $disaster_status->{'survivor_site'} )
          ->get_Cluster_component();

        return $survivor_cluster_component;

    }
    else {
        $Log->comment( "Not checking aggregate status for down site "
              . "'$disaster_status->{'survivor_site'}'" );
    }

    return;

} ## end sub _get_up_clusters

##
## _all_cluster_components()
##
## Convenience method for returning components for all clusters.
##
sub _all_cluster_components {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    return
      map { $self->get_cluster( site => $_ )->get_Cluster_component() } qw(A B);
}

##
## _get_aggregates()
##
##   $mcc->_get_aggregates( 'state' => 'online' );
##
## Returns all aggregates in list context, first aggregate in scalar
## context.
##
sub _get_aggregates {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;
    my %opt  = @_;

    my @aggregates;

    my %filter = (
        'state' => 'online',
        %opt
    );

    foreach my $cluster_comp ( $self->_all_cluster_components() ) {
        my @cluster_aggrs = NACL::STask::Aggregate->find(
            'filter'            => \%filter,
            'command_interface' => $cluster_comp,
            'allow_empty'       => 1,
        );
        push( @aggregates, @cluster_aggrs );
    }

    if (wantarray) {
        return @aggregates;
    }
    else {
        return $aggregates[0];
    }
}

=head2 perform_wafliron()

    NACL::MTask::MCC::DPSASUtils->perform_wafliron(...);

    Wrapper function to perform WAFLIRON on aggregates from multiple nodes

    Input Params:
    aggr_obj  => [Reference of aggregate object]

    Return:
    None

=cut

sub perform_wafliron {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my ( $self, @args ) = @_;
    my %args = validate(
        @args,
        {
            'aggr_obj'         => { type   => HASHREF },
            'nacltask_timeout' => { 'type' => SCALAR, 'default' => 9600 }
        }
    );

    $Log->enter() if $may_enter;
    $Log->comment('Starting wafliron');
    my $wafliron_obj = NACL::MTask::Wafliron->new(
        command_interface => $args{'aggr_obj'}->command_interface(),
        aggregate         => $args{'aggr_obj'}->aggregate()
    );

    $wafliron_obj->start(
        'nacltask_include_mirrors' => 1,
        'nacltask_optional_commit' => 'false',
        'nacltask_timeout'         => $args{'nacltask_timeout'},
        'nacltask_wait'            => 1
    );

    my $messages = $wafliron_obj->get_wafliron_messages();
    if ( $wafliron_obj->corruption_found() ) {
        $Log->comment('Wafliron found corruption during wafliron check');
        my $review_messages = $wafliron_obj->review();
        $Log->debug( 'Wafliron review messages : ' . $review_messages );
    }
    else {
        $Log->comment('wafliron did not find corruption');
        $Log->comment("wafliron completed\n");

        # Bring back aggregate online
        my $aggr_state = $args{'aggr_obj'}->get_one_state_attribute('state');
        if ( $aggr_state =~ /offline/i ) {
            $args{'aggr_obj'}->modify( state => 'online' );
        }
    }
    $Log->exit() if $may_exit;
    return;
}

=head2 version_is_valid()

Checks the switch and bridge firmware versions against the
%Bridge_Firmware_Versions and %Switch_Firmware_Versions tables. Throws
an exception if the firmware versions are invalid.

=over

=item Options

=over

=item C<< die_on_failure =E<gt> 0 | 1 (default = 1) >>

Whether to throw a NACL::MTask::MCC::Exceptions::Validation exception
on failure or to just warn

=back

=back

=over

=item Associated environment variables:

=over

=item C<< IGNORE_INVALID_FIRMWARE_VERSIONS = 0|1 >>

Unless this environment variable is set to 1 in a test environment,
this method will throw a BaseException if the firmware versions
are invalid. If you do not want this method to throw an exception,
set this environment value to 0.

=back

=back

=cut

sub version_is_valid {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my $throw_exception_default = 0;
    if ( defined $ENV{'IGNORE_INVALID_FIRMWARE_VERSIONS'} ) {
        $throw_exception_default = !$ENV{'IGNORE_INVALID_FIRMWARE_VERSIONS'};
    }

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'throw_exception_on_invalid' =>
              { default => $throw_exception_default },
            'die_on_failure' => {
                'type'  => BOOLEAN,
                default => $throw_exception_default
            },
        }
    );

    #throw_exception_on_invalid deprecated

    my $die_on_failure = $opts{die_on_failure}
      || $opts{throw_exception_on_invalid};

    my @invalid_entries = ();

    foreach my $node ( @{ $self->get_all_mcc_nodes() } ) {

        if ( $self->is_vsim() ) {
            next;
        }

        my $netnode = $node->L_get_network_node_functionality();

        #misleading method names, don't use L_get_storage_show_switch();
        my $switch = $netnode->_init_storage_show_switch_response();

        if ( defined $switch ) {
            push(
                @invalid_entries,
                $self->_check_entries(
                    entries     => $switch->get_parsed_output(),
                    table       => $self->_switch_firmware_versions(),
                    version_key => 'version'
                )
            );
        }
        else {
            $Log->comment("No defined switch, skipping switch validation");
        }

        #misleading method names, don't use L_get_storage_show_bridge();
        my $bridge = $netnode->_init_storage_show_bridge_response();

        if ( defined $bridge ) {
            push(
                @invalid_entries,
                $self->_check_entries(
                    entries     => $bridge->get_parsed_output(),
                    table       => $self->_bridge_firmware_versions(),
                    version_key => 'rev'
                )
            );
        }
        else {
            $Log->comment("No defined bridge, skipping bridge validation");
        }
    }

    if ( scalar @invalid_entries ) {
        my $errmsg = join( "\n", map { Dumper($_) } @invalid_entries );

        $self->log_validation_event(
            type       => 'version_verify',
            successful => 0,
            summary    => "version_verify() failed: $errmsg",
            results    => \@invalid_entries
        );
    }
    else {
        $self->log_validation_event(
            type       => 'version_verify',
            successful => 1,
            summary    => "version_verify() successful"
        );
    }

    if ( scalar @invalid_entries ) {
        my $errstr = "Invalid firmware versions: "
          . join( "\n", map { Dumper($_) } @invalid_entries );
        if ($die_on_failure) {
            NACL::MTask::MCC::Exceptions::Validation->throw($errstr);
        }
        else {
            $Log->warn($errstr);
        }
    }

    return !scalar @invalid_entries;

}

=head2 firmware_versions()

Returns a hashref containing supported ("OK"), "Deprecated", and "Obsolete"
bridge and switch firmware versions.

This is callable as a static method as well.

    $versions = $mcc->firmware_versions();
    $versions = NACL::MTask::MCC::Metrocluster::firmware_versions();

An example hashref is:

    {
        'bridge' => {
            'ATTO' => {
                '1.60' => 'OK',
                'A69B' => 'OK',
                'A68T' => 'Deprecated',
                'A68X' => 'Deprecated',
                '1.50' => 'Deprecated',
                'A65D' => 'Obsolete',
                '1.49' => 'Obsolete'
            }
        },
        'switch' => {
            'Cisco' => {
                '6.2(1)'  => 'OK',
                '5.2(6a)' => 'Obsolete'
            },
            'Brocade' => {
                'v7.2.1' => 'OK',
                'v7.0.2b1' => 'Deprecated',
            }
        },
        'vendor_normalization' => {
            'Cisco Systems, Inc.'   => 'Cisco',
            'ATTO'                  => 'ATTO',
            'Brocade Communications, Inc.' => 'Brocade',
        },
    };


=cut

sub firmware_versions {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    return {
        'bridge'               => \%Bridge_Firmware_Versions,
        'switch'               => \%Switch_Firmware_Versions,
        'vendor_normalization' => \%Vendor_Normalization_Table,
    };
}

sub _check_entries {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %in = validate_with(
        params => \@_,
        spec   => {
            entries     => { type => ARRAYREF },
            table       => { type => HASHREF },
            version_key => { type => SCALAR }
        }
    );

    my @invalid_entries;

    foreach my $entry ( @{ $in{'entries'} } ) {
        my $vendor =
          $self->_normalize_firmware_vendor( 'vendor' => $entry->{'vendor'} );

        my $version = $entry->{ $in{'version_key'} };

        $self->_check_entry_version(
            vendor  => $vendor,
            version => $version,
            table   => $in{'table'}
        ) or push( @invalid_entries, $entry );
    }

    return @invalid_entries;
}

sub _normalize_firmware_vendor {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    my %in = validate_with(
        params => \@_,
        spec   => { 'vendor' => 1 }
    );

    my $vendor = $in{'vendor'};
    if ( my $vendor_normalized =
        $self->_vendor_normalization_table->{ $in{'vendor'} } )
    {
        $vendor = $vendor_normalized;
    }

    return $vendor;
}

sub _check_entry_version {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %in = validate_with(
        params => \@_,
        spec   => {
            vendor  => { type => SCALAR },
            version => { type => SCALAR },
            table   => { type => HASHREF }
        }
    );

    my $version = $in{'version'};
    my $vendor  = $in{'vendor'};

    my $vendor_table = $in{'table'}{$vendor};

    if ( !defined $vendor_table ) {
        $Log->warn("Firmware vendor $vendor not recognized");
        return;
    }

    my $status = $vendor_table->{$version};

    if ( defined $status ) {
        if ( $status eq 'OK' ) {
            $Log->comment("Firmware $vendor $version is OK");
            return 1;
        }
        elsif ( $status eq 'Deprecated' ) {
            $Log->warn("Firmware $vendor $version is deprecated");
            return 1;
        }
        elsif ( $status eq 'Testing' ) {
            $Log->warn( "Firmware $vendor $version is being tested "
                  . "but is NOT yet ready for use. Please downgrade to "
                  . "an approved firmware version." );
            return 1;
        }
        elsif ( $status eq 'Obsolete' ) {
            $Log->warn( "Firmware $vendor $version is obsolete! "
                  . "Please upgrade to "
                  . "an approved firmware version." );
            return;
        }
        else {
            $Log->warn("Firmware $vendor $version is status $status");
            return;
        }
    }
    else {
        $Log->warn("Firmware '$vendor' '$version' is not supported");
        return;
    }

}

sub _check_network_interface_status {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'clusters' => { 'type' => ARRAYREF },
            'status'   => { 'type' => SCALAR, 'default' => 'up' }
        }
    );

    my @lifs;

    foreach my $cluster ( @{ $opts{'clusters'} } ) {
        push @lifs,
          NACL::CS::NetworkInterface->fetch( command_interface => $cluster );
    }

    my @errors;

    foreach my $lif ( @{ $opts{'lifs'} } ) {
        my $status = $lif->get_one_state_attribute('status');
        if ( $status ne $opts{'status'} ) {
            push( @errors,
                    "Status of '$lif' is '$status', should be "
                  . "'$opts{'status'}'" );
        }
    }

    return @errors;

}

sub validate_network_interface_show {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'die_on_failure' =>
              { 'type' => SCALAR, regex => qr/^[01]$/, default => 1 },
            'disaster_status' => { 'type' => HASHREF, 'optional' => 1 },
        }
    );

    # Get disaster status
    my $disaster_status = $opts{'disaster_status'};

    if ( !defined $disaster_status ) {
        $disaster_status = $self->get_disaster_status();
    }

    my @errors;

    if ( $disaster_status->{'DR_state'} eq 'normal' ) {
        push @errors,
          $self->_check_network_interface_status(
            clusters => [ $self->_all_cluster_components() ] );
    }

    elsif ($disaster_status->{'DR_state'} eq 'heal_aggrs_completed'
        || $disaster_status->{'DR_state'} eq 'switchover_completed' )
    {

        my $survivor_cluster_component =
          $self->get_cluster( site => $disaster_status->{'survivor_site'} )
          ->get_Cluster_component();

        push @errors,
          $self->_check_network_interface_status(
            clusters => [$survivor_cluster_component] );
    }

    elsif ( $disaster_status->{'DR_state'} eq 'heal_roots_completed' ) {
        my $survivor_cluster_component =
          $self->get_cluster( site => $disaster_status->{'survivor_site'} )
          ->get_Cluster_component();
        my $disaster_cluster_component =
          $self->get_cluster( site => $disaster_status->{'disaster_site'} )
          ->get_Cluster_component();
        push @errors,
          $self->_check_network_interface_status( clusters =>
              [ $survivor_cluster_component, $disaster_cluster_component ] );
    }

    else {

        # Test depending on disaster status
        $Log->warn( "No network interface validation performed for state "
              . "'$disaster_status->{'DR_state'}'\n"
              . "Disaster status:\n"
              . Dumper($disaster_status) );
        return 1;
    }

    if (@errors) {
        my $message = join( "\n", @errors ) . "\n";

        $self->log_validation_event(
            type       => 'network_interface_show',
            successful => 0,
            summary    => "Failed network interface show",
            results    => \@errors
        );

        if ( $opts{'die_on_failure'} ) {
            NACL::MTask::MCC::Exceptions::BaseException->throw($message);
        }
        else {
            $Log->warn($message);
        }
        return;
    }
    else {
        $Log->comment( "Passed network interface verify for state "
              . "'$disaster_status->{'DR_state'}'" );

        $self->log_validation_event(
            type       => 'network_interface_show',
            successful => 1,
            summary    => "Passed network interface show",
        );

        return 1;
    }

} ## end sub validate_network_interface_show

sub _validate_plex_field {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;
    my ( $state_field, $field_name, $field_value ) = @_;

    # Trim off quotes
    $state_field =~ s/^"//;
    $state_field =~ s/"$//;

    if ( ref $field_value && ref $field_value eq 'CODE' ) {
        return $field_value->( $state_field, $field_name );
    }
    elsif ( ref $field_value && ref $field_value eq 'ARRAY' ) {
        my @field_values       = @{$field_value};
        my @state_values       = split( /,\s*/, $state_field );
        my %state_values_table = ();
        foreach my $state_value_item (@state_values) {
            $state_values_table{$state_value_item} = 1;
        }
        foreach my $field_value_item (@field_values) {
            $state_values_table{$field_value_item} or return;
        }
        return 1;
    }
    elsif ( ref $field_value && ref $field_value eq 'Regexp' ) {
        return $state_field =~ m/$field_value/;
    }
    else {
        return $field_value eq $state_field;
    }

}

sub _validate_plex_one {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            clusters   => { type => ARRAYREF },
            spec_check => { type => HASHREF }
        }
    );

    my @clusters = @{ $opts{'clusters'} };

    my @plex_states;

    foreach my $cluster (@clusters) {

        my @plex_show_states =
          NACL::CS::StorageAggregatePlexShow->fetch(
            command_interface => $cluster, );

        push @plex_states, @plex_show_states;
    }

    my @errors;

    foreach my $plex_state (@plex_states) {
        my @plex_error = ();

        while ( my ( $field_name, $field_value ) =
            each %{ $opts{'spec_check'} } )
        {

            my $state_field = $plex_state->$field_name();

            $self->_validate_plex_field( $state_field, $field_name,
                $field_value )
              or push( @plex_error, "$field_name=$state_field" );

        }

        if (@plex_error) {
            push( @errors,
                    'Plex '
                  . $plex_state->plex()
                  . ' had improper values: '
                  . join( ' ', @plex_error ) );
        }

    }

    return @errors;

} ## end sub _validate_plex_one

sub validate_plex {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'die_on_failure' =>
              { 'type' => SCALAR, regex => qr/^[01]$/, default => 1 },
            'disaster_status' => { 'type' => HASHREF, 'optional' => 1 },
        }
    );

    my $disaster_status = $opts{'disaster_status'};

    if ( !defined $disaster_status ) {
        $disaster_status = $self->get_disaster_status( include_HA => 1 );
    }

    my @errors;

    if ( $disaster_status->{'DR_state'} eq 'normal' ) {
        push(
            @errors,
            $self->_validate_plex_one(
                clusters   => [ $self->_all_cluster_components() ],
                spec_check => {
                    status => sub {
                        my ($field_value) = @_;

                        $field_value =~ m/\bnormal\b/
                          and $field_value =~ m/\b(in)?active\b/
                          and return 1;

                        $field_value =~ m/\bnormal\b/
                          and $field_value =~ m/\bresyncing\b/
                          and return 1;

                        return;
                    },
                    'is_online' => 'true'
                },
            )
        );
    }
    elsif ( $disaster_status->{'DR_state'} eq 'switchover_completed' ) {

        my $survivor_cluster_component =
          $self->get_cluster( site => $disaster_status->{'survivor_site'} )
          ->get_Cluster_component();

        push(
            @errors,
            $self->_validate_plex_one(
                clusters   => [$survivor_cluster_component],
                spec_check => {
                    status => sub {
                        my ($field_value) = @_;

                        $field_value =~ m/\bnormal\b/
                          and $field_value =~ m/\bactive\b/
                          and return 1;

                        $field_value =~ m/\bnormal\b/
                          and $field_value =~ m/\bresyncing\b/
                          and return 1;

                        $field_value =~ m/\bfailed\b/
                          and $field_value =~ m/\binactive\b/
                          and return 1;

                        $field_value =~ m/\bnormal\b/
                          and $field_value =~ m/\binactive\b/
                          and return 1;

                        return;
                    },
                    'is_online' => qr/^(?:true|false)$/,
                }
            )
        );

    }

    elsif ( $disaster_status->{'DR_state'} eq 'heal_aggrs_completed' ) {

        my $survivor_cluster_component =
          $self->get_cluster( site => $disaster_status->{'survivor_site'} )
          ->get_Cluster_component();

        push(
            @errors,
            $self->_validate_plex_one(
                clusters   => [$survivor_cluster_component],
                spec_check => {
                    status => sub {
                        my ($field_value) = @_;

                        $field_value =~ m/\bnormal\b/
                          and $field_value =~ m/\b(in)?active\b/
                          and return 1;

                        $field_value =~ m/\bnormal\b/
                          and $field_value =~ m/\bresyncing\b/
                          and return 1;

                        return;
                    },
                    'is_online' => qr/^(?:true|false)$/,
                },
            )
          )

    }

    elsif ( $disaster_status->{'DR_state'} eq 'heal_roots_completed' ) {
        my @plex_error;

        my @plex_show_states;

        foreach my $site ( 'survivor_site', 'disaster_site' ) {

            my $cluster_component =
              $self->get_cluster( site => $disaster_status->{$site} )
              ->get_Cluster_component();

            push(
                @plex_show_states,
                NACL::CS::StorageAggregatePlexShow->fetch(
                    command_interface => $cluster_component
                )
            );
        }

        foreach my $plex_state (@plex_show_states) {

            unless (
                $plex_state->status() =~ m/\bnormal\b/
                and (  $plex_state->status() =~ m/\bactive\b/
                    or $plex_state->status() =~ m/\bresyncing\b/ )
              )
            {
                push( @plex_error, 'status=' . $plex_state->status() );
            }

            if ( $plex_state->is_online() ne 'true' ) {
                push( @plex_error, 'is_online=' . $plex_state->is_online() );
            }

            if (   $plex_state->in_progress() ne 'false'
                && $plex_state->in_progress() ne 'true' )
            {
                push( @plex_error,
                    'in_progress=' . $plex_state->in_progress() );
            }

            if (@plex_error) {
                push( @errors,
                        'Plex '
                      . $plex_state->plex()
                      . ' had improper values: '
                      . join( ' ', @plex_error ) );
            }

        }

    }

    if (@errors) {

        my $message = join( "\n", @errors ) . "\n";

        $self->log_validation_event(
            type       => 'plex',
            successful => 0,
            summary    => "Failed plex validation: $message",
            results    => \@errors
        );

        if ( $opts{'die_on_failure'} ) {
            NACL::MTask::MCC::Exceptions::BaseException->throw($message);
        }
        else {
            $Log->warn($message);
        }
        return;
    }
    else {
        $self->log_validation_event(
            type       => 'plex',
            successful => 1,
            summary    => 'Passed plex validation',
        );

        $Log->comment( "Passed plex verify for state "
              . "'$disaster_status->{'DR_state'}'" );
        return 1;
    }

} ## end sub validate_plex

sub disk_verify {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    foreach my $dr ( sort keys %{ $self->get_DR_groups() } ) {
        my $message = "validate shared storage in dr group $dr";
        $Log->comment($message);
        $self->get_DR_groups()->{$dr}->get_shared_storage()
          ->validate_disk_states(@_);
    }

    return 1;
}

sub get_all_validations {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my @validations = (
        {
            name    => 'vservers',
            sub     => sub { $self->validate_vservers(@_); },
            default => 0
        },
        {
            name => 'disks',
            sub  => sub {
                $self->disk_verify(@_);
            },
            default => 0
        },
        {
            name => 'interconnect',
            sub  => sub {
                $self->_interconnect()
                  ->validate_metrocluster_interconnect_state(@_);
            },
            default => 0
        },
        {
            name => 'node',
            sub  => sub {
                $self->validate_node(@_);
            },
            default => 0
        },
        {
            name => 'operation',
            sub  => sub {
                $self->validate_operation(@_);
            },
            default => 0
        },
        {
            name => 'fabric_version',
            sub  => sub {
                $self->version_is_valid();
            },
            default => 0
        },
        {
            name => 'data_integrity',
            sub  => sub {
                $self->verify_data_integrity(@_);
            },
            extended => 1,
            default  => 0
        },

        {
            name => 'plex',
            sub  => sub {
                $self->validate_plex(@_);
            },
            default => 0
        },
        {
            name => 'network_interface_show',
            sub  => sub {
                $self->validate_network_interface_show(@_);
            },
            default => 0
        },
        {
            name => 'aggregate',
            sub  => sub {
                $self->verify_aggregate_status(@_);
            },
            default => 0
        },

        #{
        #name => 'check_config_cluster_diff',
        #sub => sub {
        #$self->validate_check_config_cluster_diff(@_)
        #}
        #},
        #{
        #name => 'check_config_vserver_diff',
        #sub => sub {
        #$self->validate_check_config_vserver_diff(@_)
        #}
        #},
        {
            name => 'disk_paths',
            sub  => sub {
                $self->validate_disks(@_);
            },
            default => 0
        },
    );

    my @manager_validations =
      $self->_validator()->_get_all_validations_compatibility_list();

    return ( @validations, @manager_validations );
} ## end sub get_all_validations

#generates a 'spec' hashref for Params::Validate to use
sub get_all_validations_params_spec {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    my %spec =
      map { $_->{name} => { regex => qr/^[01]$/, default => $_->{'default'} } }
      $self->get_all_validations();

    return \%spec;
}

=head2 run_all_validations()

Runs all MCC validations

=over

=item Options

=over

=item C<< die_on_failure =E<gt> 0 | 1 (default = 1) >>

Whether to throw a NACL::MTask::MCC::Exceptions::Validation exception
on failure or to just warn

=back

=back

=over

=item C<< stop_on_failure =E<gt> 0 | 1 (default = 0) >>

Stop on failure detection if set to 1

=back

=back

=item C<< validation_filter  =E<gt> {Hash ref of validations} >>

User defined validations filter

=back

=back

=item C<< warn_only =E<gt> [ARRAYREF] (default = []) >>

User defined validations filter

=back

=back

=item C<< options =E<gt> {Hash ref of validations} >>

User defined validations filter

=back

=back

=item C<< disaster_status =E<gt> 'string' (optional) >>

User defined disaster status

=back

=back

=item C<< run_extended =E<gt> 0 | 1 (default = 0) >>

Run extended validations

=back

=back

=item C<< pre_dr_operation =E<gt> 0 | 1 (default = 0) >>

Run Pre-DR operation validations

=back

=back

=item C<< ignore_op_show_validation =E<gt> 0 | 1 (default = 0) >>

Option to ignore MC operations show validation errors

=back

=back

=cut

sub run_all_validations {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    my %in = validate_with(
        params => \@_,
        spec   => {
            die_on_failure    => { regex      => qr/^[01]$/, default   => 1, },
            stop_on_failure   => { regex      => qr/^[01]$/, default   => 0, },
            validation_filter => { 'optional' => 1 },
            warn_only         => { type       => ARRAYREF,   default   => [] },
            options           => { type       => HASHREF,    default   => {} },
            disaster_status   => { 'optional' => 1 },
            run_extended      => { regex      => qr/^[01]$/, default   => 0, },
            pre_dr_operation  => { regex      => qr/^[01]$/, 'default' => 0 },
            ignore_op_show_validation => { regex => qr/^[01]$/, 'default' => 0 }
        }
    );

    my $disaster_status;

    if ( $in{'run_extended'} ) {
        $Log->comment( "run_all_validations() was called with run_extended, "
              . "so it may take several minutes." );
    }

    if ( $in{'disaster_status'} ) {
        $disaster_status = $in{'disaster_status'};
    }
    else {
        $disaster_status = $self->get_disaster_status( include_HA => 1 );
    }

    # Convenience feature-- 'ignore_op_show_validation => 1' (or env
    # variable) is the same as 'warn_only => ['operation']'.
    if (   $in{'ignore_op_show_validation'}
        || $ENV{'MCC_IGNORE_OP_SHOW_VALIDATION'} )
    {
        push( @{ $in{'warn_only'} }, 'operation' );
    }

    my $validation_regex;

    if ( defined $in{'validation_filter'} ) {
        if ( ref $in{'validation_filter'} eq 'ARRAY' ) {
            $validation_regex =
              join( '|', map { "^$_\$" } @{ $in{'validation_filter'} } );
        }
        elsif ( ref $in{'validation_filter'} eq 'HASH' ) {
            $validation_regex = join(
                '|', map { "^$_\$" }
                  grep { $in{'validation_filter'}->{$_} }    #keep true values
                  keys %{ $in{'validation_filter'} }
            );
        }
        else {
            $validation_regex = $in{'validation_filter'};
        }
    }
    $Log->comment( "validation filter:" . Dumper $in{validation_filter} );
    $Log->comment("validation regex:$validation_regex")
      if defined $validation_regex;

    my @exceptions = ();
    my @manager_validations;

  VALIDATION: foreach my $validation ( $self->get_all_validations() ) {

        if ( defined $validation_regex ) {
            $validation->{'name'} =~ m/$validation_regex/ or next;
        }

        if ( $validation->{'extended'} ) {
            $in{'run_extended'} or next;
        }

        if ( $validation->{'manager'} ) {
            push @manager_validations, $validation->{'name'};
        }
        else {
            $Log->comment("Running validation $validation->{'name'}");

            #none of the pre-validator classes support pre validation
            #so just skip any prepare/pre-validate calls for these.
            next if $in{pre_dr_operation};
            try {
                &{ $validation->{'sub'} }(
                    'disaster_status' => $disaster_status,
                    %{ $in{'options'} },
                    'die_on_failure' => 1
                );
            }
            catch NATE::BaseException with {
                my $exception = shift;

                $Log->warn( "Caught an exception in validation: "
                      . $exception->text() );

                if (
                    $self->_should_not_warn_only(
                        name => $validation->{'name'},
                        list => $in{'warn_only'}
                    )
                  )
                {

                    push(
                        @exceptions,
                        {
                            'name'    => $validation->{'name'},
                            'message' => $exception
                        }
                    );

                    if ( $in{'stop_on_failure'} ) {
                        last VALIDATION;
                    }
                }
            }
        }
    }

    #we want to temporarily integrate the manager validators and
    #the ones directly in metrocluster.  Eventually we'll move
    #them all to the manager
    my @manager_exceptions = $self->_validator()->run_all_validations(
        metrocluster           => $self,
        die_on_failure         => 0,
        stop_on_failure        => $in{stop_on_failure},
        include                => \@manager_validations,
        extra_validate_options => $in{options},
        disaster_status        => $disaster_status,
        run_extended           => $in{run_extended},
        pre_dr_operation       => $in{pre_dr_operation},
        warn_only              => $in{'warn_only'}
    );

    @exceptions = ( @exceptions, @manager_exceptions );

    $self->validation_report();

    if (@exceptions) {
        my $errmsg = "Validation failed: "
          . join( "\n",
            map { "Name: $_->{'name'}, Message: $_->{'message'}" }
              @exceptions );
        if ( $in{'die_on_failure'} ) {
            NACL::MTask::MCC::Exceptions::Validation->throw($errmsg);
        }
        else {
            $Log->warn($errmsg);
            return;
        }
    }
    else {
        return 1;
    }

} ## end sub run_all_validations

sub _should_not_warn_only {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        'params' => \@_,
        'spec'   => {
            'name' => { 'type' => SCALAR },
            'list' => { 'type' => ARRAYREF }
        }
    );

    # Return false if we match anything in the list
    foreach my $item ( @{ $opts{'list'} } ) {
        if ( ref($item) && ( ref $item eq 'Regexp' ) ) {
            $opts{'name'} =~ m/$item/ and return;
        }
        else {
            $opts{'name'} eq $item and return;
        }
    }

    # Otherwise, no match-- return true
    return 1;

}

=head2 prepare_for_externalization()

This method will prepare the Metrocluster object for externalization
(for example, writing to object to file through the Storable class.)

If you attempt to externalize a Metrocluster object, and any objects
which contain a Connectrec (or any object which contains typeglob references)
are cached in the object, externalization will fail.

This routine will:

  * Save copies of hostrecs for MTask::MCC::Cluster objects so they
    can be rebuilt using the Cserver lif address
  * Refresh all CommandInterface objects
  * Empty the _pre_switchover_halted_node_transit_objs() Transit objects.
  * Run through all dr partnership objects and refresh all CommandInterface
    objects and delete their Transit objects.

Once the Metrocluster object is externalized, it can be internalized (read back
in from file) and reconsitituted by running the restore_after_externalization()
method below.

An example of use:

  # To externalize:
  use DataStructureStore;

  $mccDss = DataStructureStore->new( mode => 'sto' );
  $mccDss->dss_update_file('filename'  => $MCC_SAVE_FILE );

  $mcc->prepare_for_externalization();
  $mccDss->dss_store($mcc);

=over

=item Options

=item None

=back

=cut

sub prepare_for_externalization {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    if ( $self->{_site_A_metrocluster_component}->{command_interface} ) {
        if ( !$self->{_site_A_metrocluster_component}->{hostrec} ) {
            my $hostrec =
              $self->{_site_A_metrocluster_component}->{command_interface}
              ->hostrec();
            $self->{_site_A_metrocluster_component}->{hostrec} = $hostrec;
        }

        $Log->comment("Calling site_A metrocluster refresh_command_interface");
        $self->{_site_A_metrocluster_component}->{command_interface}
          ->refresh_command_interface();
    }
    if ( $self->{_site_B_metrocluster_component}->{command_interface} ) {
        if ( !$self->{_site_B_metrocluster_component}->{hostrec} ) {
            my $hostrec =
              $self->{_site_B_metrocluster_component}->{command_interface}
              ->hostrec();
            $self->{_site_B_metrocluster_component}->{hostrec} = $hostrec;
        }

        $Log->comment("Calling site_B metrocluster refresh_command_interface");
        $self->{_site_B_metrocluster_component}->{command_interface}
          ->refresh_command_interface();
    }

    if ( $self->{_cluster_A}->_pre_switchover_halted_node_transit_objs() ) {
        $Log->comment("Deleting cluster_A Transit objects");
        $self->{_cluster_A}->_pre_switchover_halted_node_transit_objs(undef);
    }
    if ( $self->{_cluster_B}->_pre_switchover_halted_node_transit_objs() ) {
        $Log->comment("Deleting cluster_B Transit objects");
        $self->{_cluster_B}->_pre_switchover_halted_node_transit_objs(undef);
    }

    $Log->comment(
"Refreshing site_A & siteB dr_partnership node and root_aggregate command_interface and deleting Transit Objects"
    );
    foreach my $dr_grp ( @{ $self->_dr_groups() } ) {
        foreach my $dr_part ( @{ $dr_grp->_dr_partnerships() } ) {
            foreach
              my $node ( $dr_part->_site_A_node(), $dr_part->_site_B_node() )
            {
                $node->_system_node_component()->command_interface()
                  ->refresh_command_interface();
                $node->get_root_aggregate_component()->command_interface()
                  ->refresh_command_interface();
                $node->{_transit_obj} = undef;
            }
        }
    }
    return 1;
} ## end sub prepare_for_externalization

=head2 restore_after_externalization()

This method will repair a Metrocluster object that was read in from
file (for example, reading the object from file through the Storable class.)

To externalize a Metrocluster object any objects which contain a Connectrec
(or any other objects which contain typeglob references) must be deleted before
the object can be written to file.  So when we read the object back in from
file, we need to reconstitute the objects we deleted.

This routine will:

  * Reconsititute the MTask::MCC::Cluster object given the saved Hostrec objects.
    These objects need to be rebuilt because they were created using Hostrec
    overrides which are not available unless we refer to the saved Hostrec objects.

The Node-based CommandInterface objects that were cleared out will be restored
on demand as they are needed, as will the Transit objects.

An example of use:

  # To restore from file:
  use DataStructureStore;

  $mccDss = DataStructureStore->new( mode => 'sto' );
  $mccDss->dss_update_file('filename'  => $MCC_SAVE_FILE );

  $mccDss->dss_load($mcc);
  $mcc->restore_after_externalization();

To write the Metrocluster object to file, see prepare_for_externalization().

=over

=item Options

=item None

=back

=cut

sub restore_after_externalization {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

# Need to refresh the cluster command_interfaces ourselves because there may not be any hostrec files
    my $cluster_A_hostrec = $self->{_site_A_metrocluster_component}->{hostrec};
    if ($cluster_A_hostrec) {
        $self->{_site_A_metrocluster_component}->command_interface(
            NACL::C::Cluster->new( hostrec => $cluster_A_hostrec ) );
    }
    else {
        my $msg =
"Site A Metrocluster Cluster component hostrec not found, did you first call prepare_for_externalization()?";
        NACL::MTask::MCC::Exceptions::BaseException->throw($msg);
    }

    my $cluster_B_hostrec = $self->{_site_B_metrocluster_component}->{hostrec};
    if ($cluster_B_hostrec) {
        $self->{_site_B_metrocluster_component}->command_interface(
            NACL::C::Cluster->new( hostrec => $cluster_B_hostrec ) );
    }
    else {
        my $msg =
"Site B Metrocluster Cluster component hostrec not found, did you first call prepare_for_externalization()?";
        NACL::MTask::MCC::Exceptions::BaseException->throw($msg);
    }
    return 1;
} ## end sub restore_after_externalization

sub _validate_disks_node {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => { 'node' => { type => OBJECT } }
    );

    my $node = $opts{'node'};

    # this is a temporary (?) fix to BURT 601682 where calls to sysconfig
    # are timing out in the APISet object
    $node->get_nodescope_apiset()->set_timeout( 'connectrec-timeout' => 300 );
    my $output = $node->get_nodescope_apiset()->sysconfig();

    my $slot_info = $output->{'parsed_output'}[0]{'slot_information'};

    my @disk_counts;

    foreach my $slot ( @{$slot_info} ) {
        while ( my ( $key, $value ) = each %{$slot} ) {
            if ( $key =~ m/^(\d+)_disks/ ) {
                my $disk_count = $1;

                push(
                    @disk_counts,
                    {
                        count => $disk_count,
                        slot  => $slot
                    }
                );
            }
        }
    }

    return @disk_counts;

}

sub validate_disks {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'die_on_failure' =>
              { 'type' => SCALAR, regex => qr/^[01]$/, default => 1 },
            'disaster_status' => { 'type' => HASHREF, 'optional' => 1 },
        }
    );

    # Skip if we're 2-node, unless DONT_SKIP_2NODE_MCC_VALIDATION=1
    if ( $self->is_2_node() ) {
        # do not skip if we tell the test not to skip 2-node validation
        if ( exists $ENV{'DONT_SKIP_2NODE_MCC_VALIDATION'} && $ENV{'DONT_SKIP_2NODE_MCC_VALIDATION'} == 1 ) {
            $Log->comment("Not skipping 2-node validation, because DONT_SKIP_2NODE_MCC_VALIDATION=1");
        } else {
            $Log->comment("Skipping 2-node validation");
            return;
        }
    }

    my $disaster_status = $opts{disaster_status}
      || $self->get_disaster_status( include_HA => 1 );

    my @disks;

    my %survivor_nodes =
      %{ $self->get_live_nodes( disaster_status => $disaster_status ) };

    foreach my $nodes ( values %survivor_nodes ) {
        foreach my $node ( @{$nodes} ) {
            my @node_disks = $self->_validate_disks_node( node => $node );
            push( @disks, @node_disks );
        }
    }

    my $disk_count = $disks[0]->{'count'};

    foreach my $disk ( @disks[ 1 ... $#disks ] ) {
        if ( $disk_count != $disk->{'count'} ) {
            my $errors = Dumper( \@disks );

            $self->log_validation_event(
                type       => 'disk count',
                successful => 0,
                summary    => "Inconsistent disk counts: $errors",
                results    => \@disks,
            );

            if ( $opts{'die_on_failure'} ) {
                NACL::MTask::MCC::Exceptions::Validation->throw(
                    "Inconsistent disk counts: $errors");
            }
            else {
                $Log->warn("Inconsistent disk counts: $errors");
            }
        }
    }

    $self->log_validation_event(
        type       => 'disk count',
        successful => 1,
    );

    return;

}

=head2 validate_check_config_cluster_diff()

This class runs and validates the results of the metrocluster check
command 'metrocluster check config-cluster diff show'
See
L<"CheckConfigClusterDiff"|NACL::MTask::MCC::Validator::CheckConfigClusterDiff>
for details.

=over

=item Options

=over

=item C<< die_on_failure =E<gt> 0 | 1 (default = 1) >>

If true, throw a NACL::MTask::MCC:Exceptions::Validation exception on
failure. If false, just warn on validation failure.

=item C<< disaster_status =E<gt> $mcc-E<gt>get_disaster_status() (optional) >>

(Optional) Pass in a previously generated disaster status structure if
you have one or a new one will be generated

=back

=back

=cut

sub validate_check_config_cluster_diff {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'die_on_failure' =>
              { 'type' => SCALAR, regex => qr/^[01]$/, default => 1 },
            'disaster_status' => { 'type' => HASHREF, 'optional' => 1 },
        }
    );

    my $vtor = NACL::MTask::MCC::Validator::CheckConfigClusterDiff->new(
        metrocluster    => $self,
        disaster_status => $opts{'disaster_status'}
    );

    return $vtor->run( die_on_failure => $opts{'die_on_failure'} );

}

=head2 validate_check_config_vserver_diff()

This class runs and validates the results of the metrocluster check
command 'metrocluster check config-vserver diff show'
See
L<"CheckConfigVserverDiff"|NACL::MTask::MCC::Validator::CheckConfigVserverDiff>
for details.

=over

=item Options

=over

=item C<< die_on_failure =E<gt> 0 | 1 (default = 1) >>

If true, throw a NACL::MTask::MCC:Exceptions::Validation exception on
failure. If false, just warn on validation failure.

=item C<< disaster_status =E<gt> $mcc-E<gt>get_disaster_status() (optional) >>

(Optional) Pass in a previously generated disaster status structure if
you have one or a new one will be generated

=back

=back

=cut

sub validate_check_config_vserver_diff {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'die_on_failure' =>
              { 'type' => SCALAR, regex => qr/^[01]$/, default => 1 },
            'disaster_status' => { 'type' => HASHREF, 'optional' => 1 },
        }
    );

    my $vtor = NACL::MTask::MCC::Validator::CheckConfigVserverDiff->new(
        metrocluster    => $self,
        disaster_status => $opts{'disaster_status'}
    );

    return $vtor->run( die_on_failure => $opts{'die_on_failure'} );

}

=head2 modify_dr_links()

API to modify the Storage and/or FCVI DR links for MCC setup.

=over

=item Options

=over

=item C<< state=E<gt> 'up'|'down' >>

(Required) The action to be taken on DR links

=item C<< scope=E<gt> 'storage'|'interconnect'|'both' >>

(Optional) The scope of action. Defaults to both.

B<'scope =E<gt> both'> would be translated as ISL down from site A switches on MCC fabric based setup

=item C<< sites=E<gt> ['A'|'B'] >>

(Optional) The MCC sites to act on. Defaults to both sites A and B

=back

=back

=cut

sub modify_dr_links {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'state' => { 'type' => SCALAR, regex => qr/^up|down$/ },
            'scope' => {
                'type'  => SCALAR,
                regex   => qr/^storage|interconnect|both$/,
                default => 'both'
            },
            'sites' => { 'type' => ARRAYREF, default => [ 'A', 'B' ] },
        }
    );

    my @sites = @{ $opts{'sites'} };

    my $storage = ( $opts{'scope'} eq 'storage' || $opts{'scope'} eq 'both' );

    my $interconnect =
      ( $opts{'scope'} eq 'interconnect' || $opts{'scope'} eq 'both' );

    # Toggle ISL links if "scope => both" and MCC with fabric topology
    if (   $opts{'scope'} eq 'both'
        && $self->get_mcc_topology_type() eq 'FABRIC' )
    {
        # Disable ISLs only from siteA switches
        my @switches = $self->L_get_all_switches( sites => ['A'] );

        foreach my $switch (@switches) {
            foreach my $port ( @{ $switch->get_isl_ports() } ) {
                if ( $opts{'state'} eq 'up' ) {
                    $port->enable();
                }
                else {
                    $port->disable();
                }
            }
        }

        return 1;
    }

    if ($storage) {
        $self->L_modify_dr_links_storage(
            state => $opts{'state'},
            sites => \@sites,
        );
    }

    if ($interconnect) {
        $self->L_modify_dr_links_interconnect( state => $opts{'state'}, );

    }

    return 1;
}

sub L_modify_dr_links_interconnect {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => { 'state' => { 'type' => SCALAR, regex => qr/^up|down$/ }, }
    );

    if ( $self->_cluster_A->is_atto() or $self->_cluster_A->is_glowworm() ) {

        my ($node) = @{ $self->_cluster_A->L_get_cluster_mcc_nodes() };

        my $apiset =
          $node->get_Node_component->get_7m_or_nodescope_apiset(
            interface => 'CLI' );
        my $option;

        if ( $opts{'state'} eq 'up' ) {
            $option = 8;
        }
        else {
            $option = 9;
        }

        foreach my $nic ( 1, 2 ) {
            $apiset->execute_command( command => "qlgc2500 $nic $option" );
        }

    }
    else {

        my @switches = $self->L_get_all_switches();

        foreach my $switch (@switches) {
            my @ports = @{ $switch->get_fcvi_ports() };
            foreach my $port (@ports) {
                if ( $opts{'state'} eq 'up' ) {
                    $port->enable();
                }
                else {
                    $port->disable();
                }
            }

        }

    }

    return 1;

} ## end sub L_modify_dr_links_interconnect

=head2 L_get_all_switches()

Returns the NACL::MTask::MCC::Network::Switch objects for all switches in this MCC setup

=over

=item Options

=over

=item C<< sites=E<gt> ARRAYREF (default = ['A','B'] ) >>

=back

=back

=cut

sub L_get_all_switches {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec => { 'sites' => { 'type' => ARRAYREF, default => [ 'A', 'B' ] }, }
    );

    my $dr_group = $self->get_singleton_DR_group();

    #Get network topology
    my $network_topology = $dr_group->get_network_topology();

    my @switches;

    foreach my $fab ( @{ $network_topology->get_intersite_fabrics } ) {

        foreach my $site ( @{ $opts{'sites'} } ) {

            push @switches, $fab->get_switch( site => $site );

        }

    }

    return @switches;

} ## end sub L_get_all_switches

sub L_modify_dr_links_storage {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'state' => { 'type' => SCALAR, regex => qr/^up|down$/ },
            'sites' => { 'type' => ARRAYREF },
          }

    );

    # TODO merge with equivalent cluster code

    my @sites     = @{ $opts{'sites'} };
    my $cluster_A = $self->_cluster_A();
    my $cluster_B = $self->_cluster_B();

    $Log->comment( 'Sites passed in are: ' . Dumper(@sites) );

    if ( $cluster_A->has_switches() ) {
        $Log->comment("Has switches, so bringing down topology");

        my $dr_group = $self->get_singleton_DR_group();

        #Get network topology
        my $network_topology = $dr_group->get_network_topology();

        my $dr_groups = $self->get_DR_groups();

        #Start Switch enable block
        foreach my $site (@sites) {

            my @ports = ();
            foreach my $dr_group ( values( %{$dr_groups} ) ) {
                my @fabrics =
                  @{ $dr_group->get_network_topology()->get_intersite_fabrics()
                  };

                foreach my $fabric (@fabrics) {

                    #get storage ports
                    push(
                        @ports,
                        @{
                            $fabric->get_switch( site => $site )
                              ->get_storage_ports()
                        }
                    );

                }

            }

            foreach my $port (@ports) {
                if ( $opts{'state'} eq 'up' ) {
                    $port->enable();
                }
                else {
                    $port->disable();
                }
            }
        }

    }
    elsif ( $cluster_A->is_glowworm() ) {
        if ( $opts{'state'} eq 'up' ) {
            foreach my $site (@sites) {
                $Log->comment( 'Bringing up storage ports on ' . $site );
                my $clus_handle;

                if ( $site eq 'A' ) {
                    $clus_handle = $cluster_A;
                }
                else {
                    $clus_handle = $cluster_B;
                }
                $clus_handle->_do_remote_sas_adapters_up();
            }
        }
        else {
            foreach my $site (@sites) {
                $Log->comment( 'Bringing down storage ports on ' . $site );
                my $clus_handle;

                if ( $site eq 'A' ) {
                    $clus_handle = $cluster_A;
                }
                else {
                    $clus_handle = $cluster_B;
                }
                $clus_handle->_do_remote_sas_adapters_down();
            }
        }
    }
    elsif ( $cluster_A->is_atto() ) {
        if ( $opts{'state'} eq 'up' ) {
            foreach my $site (@sites) {
                $Log->comment( 'Bringing up storage ports on ' . $site );
                my $clus_handle;

                if ( $site eq 'A' ) {
                    $clus_handle = $cluster_A;
                }
                else {
                    $clus_handle = $cluster_B;
                }
                $clus_handle->L_get_cluster_mcc_nodes->[0]
                  ->_do_remote_atto_adapters_up();
            }
        }
        else {
            foreach my $site (@sites) {
                $Log->comment( 'Bringing down storage ports on ' . $site );
                my $clus_handle;

                if ( $site eq 'A' ) {
                    $clus_handle = $cluster_A;
                }
                else {
                    $clus_handle = $cluster_B;
                }
                $clus_handle->L_get_cluster_mcc_nodes->[0]
                  ->_do_remote_atto_adapters_down();
            }
        }
    }
    else {
        NACL::MTask::MCC::Exceptions::BaseException->throw(
            "Unable to find cluster type");
    }

    return 1;
}

# Subroutine to check for unmirrored aggrs
# Returns the unmirrored aggr count
sub _check_for_unmirrored_aggrs {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my $unmirrored_aggr_count = 0;

    $Log->comment('Checking MCC system for existing unmirrored aggrs');
    $Log->comment('[we only check nodes that are reachable]');
    my @mccnodes = @{ $self->get_all_mcc_nodes() };
    foreach my $mccnode (@mccnodes) {
        my $node = $mccnode->get_Node_component();

        # skip checking on nodes that are unreachable
        next
          if !$node->command_interface()->apiset()->get_connection()
          ->is_responsive( timeout => 10 );

        my @aggrs = NACL::CS::StorageAggregate->fetch(
            command_interface => $node,
            filter            => {
                root   => 'false',
                mirror => 'false',
            },
            allow_empty => 1,
        );
        $unmirrored_aggr_count += scalar @aggrs;
    }

    return $unmirrored_aggr_count;
}

=head2 has_unmirrored_aggrs()

API to check presence of unmirrored aggrs on MCC system

=back

=back

=cut

sub has_unmirrored_aggrs {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics
    my $self  = shift;

    return $self->_check_for_unmirrored_aggrs();
}

=head2 get_mcc_topology_type()

API to get MCC topology type. The method returns one of following strings:

"FABRIC"   == Fabric based MCC

"GLOWWORM" == Glow-Worm based MCC

"ATTO"     == Armadillo Bridge based MCC

"VSIM"     == VSIMs based MCC

=back

=back

=cut

sub get_mcc_topology_type {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;

    my ( $found_fc_switches, $found_atto_bridges );

    # Return cached copy if it exists
    return $self->_mcc_topology_type() if defined $self->_mcc_topology_type();

    # Check if it is VSIMs
    if ( $self->is_vsim() ) {
        $self->_mcc_topology_type("VSIM");
        return $self->_mcc_topology_type();
    }

    # Get nodescope APISet
    my ($node) = @{ $self->_cluster_A()->L_get_cluster_mcc_nodes() };
    my $apiset =
      $node->get_Node_component()
      ->get_7m_or_nodescope_apiset( interface => 'CLI' );

    # Search for backend FC switches
    my $response = $apiset->storage_show_switch->get_processed_output();
    if ( $response !~ /No switches found/ ) {
        $found_fc_switches = 1;
    }

    # Search for backend FC-SAS armadillo bridges
    $response = $apiset->storage_show_bridge->get_processed_output();
    if ( $response !~ /No bridges found/ ) {
        $found_atto_bridges = 1;
    }

    ### MCC Topology decider logic ###
    # Did not found FC switches and ATTO bridges == Glow-Worm Topology
    if ( not $found_fc_switches and not $found_atto_bridges ) {
        $Log->comment('Detected MCC Glow-Worm topology');
        $self->_mcc_topology_type('GLOWWORM');
    }

    # Found ATTO bridges but NOT FC switches == ATTO topology
    elsif ( $found_atto_bridges and not $found_fc_switches ) {
        $Log->comment('Detected MCC ATTO topology');
        $self->_mcc_topology_type('ATTO');
    }

    # Found both FC switches AND ATTO bridges == Fabric Topology
    elsif ( $found_fc_switches and $found_atto_bridges ) {
        $Log->comment('Detected MCC Fabric topology');
        $self->_mcc_topology_type('FABRIC');
    }

    return $self->_mcc_topology_type();
}

=head2 modify_auso_config_state()

API to modify the AUSO config state at Node or Cluster level. AUSO state can be either enabled or disabled

    $mcc->modify_auso_config_state( state => 'disable',
                                    scope => 'nodes', );

=over

=item Options

=over

=item C<< state=E<gt> 'enable|disable' >>

(Required) Defines the AUSO config state to be set on all clusters/nodes

=item C<< scope=E<gt> 'both|clusters|nodes' (default = 'both' ) >>

(Optional) Defines the scope of AUSO config state modify operation

=back

=back

=cut

sub modify_auso_config_state {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            state => { regex => qr/^enable|disable$/, optional => 0, },
            scope => { regex => qr/^both|nodes|clusters$/, default => 'both', },
        },
    );

    # Modify cluster level AUSO config states
    if ( $opts{'scope'} =~ /both|clusters/ ) {
        my $state =
          $opts{'state'} eq 'enable'
          ? 'auso-on-cluster-disaster'
          : 'auso-disabled';
        $Log->comment("Setting AUSO config state to '$state' on all clusters");

        foreach my $site (qw(A B)) {
            my $cluster = $self->get_cluster( site => $site );
            $cluster->modify_auso_config_state( auso_failure_domain => $state );
        }
    }

    # Modify node level AUSO config states
    if ( $opts{'scope'} =~ /both|nodes/ ) {
        my $state = $opts{'state'} eq 'enable' ? 'true' : 'false';
        $Log->comment("Setting AUSO config state to '$state' on all nodes");

        my $nodes_aref = $self->get_all_mcc_nodes();
        map { $_->modify_auso_config_state( state => $state ) } @{$nodes_aref};
    }

    return;
}

# END PACKAGE NACL::MTask::MCC::Metrocluster
{

    # private utility class for NACL::MTask::MCC::Metrocluster
    package NACL::MTask::MCC::Metrocluster::_Bootstrap_Util;

    use NATE::BaseException qw(:try);
    use NACL::Exceptions::NoElementsFound qw(:try);

    use NATE::Log qw(log_global);
    my $Log = log_global();
    $may_enter = $Log->may_enter();
    $may_exit  = $Log->may_exit();

    use Params::Validate
      qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF);

    use Data::Dumper;

    use Class::MethodMaker [
        new    => [ '-init', 'new' ],
        scalar => [
            { -type => 'NACL::C::SystemNode' },
            qw/ _temp_utility_site_A_node_component /,
        ],
        scalar =>
          [ qw/ _temp_cluster_A_name _original_node_names _A1_node_names /, ],
        scalar => [
            { -type => 'NACL::C::Metrocluster' },
            qw/ _site_A_metrocluster_component
              _site_B_metrocluster_component /,
        ],
    ];

    # implicitly called by Class::MethodMaker's new (see -init)
    sub init {
        my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

        my $self = shift;
        my %opts = validate_with(
            params => \@_,
            spec   => { node_names => { type => ARRAYREF }, },
        );

        my $node_names = delete( $opts{node_names} );

        # hold onto these for a bit
        $self->_original_node_names($node_names);

        # randomly pick a node to define as one (or the) A1 node
        my $A1_node_name =
          $node_names->[ int( rand( scalar( @{$node_names} ) ) ) ];

        # this may contain more after more discovery is done...
        $self->_A1_node_names( [$A1_node_name] );

        # this is just to get a bootstrap CI
        my $host = Tharn::host($A1_node_name);

        $self->_temp_utility_site_A_node_component(
            NACL::C::SystemNode->new( hostrec => $host ) );

        unless ( $self->_temp_utility_site_A_node_component_isset() ) {
            my $msg = "Can't get a valid node component CI for discovery!";
            NACL::MTask::MCC::Exceptions::BaseException->throw($msg);
        }
    }

    sub get_A1_node_names {
        my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

        my $self = shift;

        my $Metrocluster_component_A =
          $self->get_site_A_Metrocluster_component();
        my @mcc_state_responses =
          NACL::MTask::MCC::Utils->metrocluster_node_show(
            command_interface => $Metrocluster_component_A->command_interface(),
            filter            => { cluster => $self->_temp_cluster_A_name() },
            requested_fields  => [ 'node', 'ha-partner' ]
          );

        my @discovered_A1s            = @{ $self->_A1_node_names() };
        my @other_dr_group_node_names = ();
        foreach my $state (@mcc_state_responses) {
            my $node_name  = $state->node();
            my $ha_partner = $state->ha_partner();
            if ( List::MoreUtils::any { /^($node_name|$ha_partner)$/ }
                @discovered_A1s )
            {
                # ignore these.  already have A1 for them.
            }
            else {
                # but these must be from other dr group(s).
                # check if these are in original node name list passed in
                my @choices_for_A1 =
                  grep { /^($node_name|$ha_partner)$/ }
                  @{ $self->_original_node_names() };

                # if no matches, put both in...
                unless ( scalar(@choices_for_A1) ) {
                    @choices_for_A1 = ( $node_name, $ha_partner );
                }

                # select A1 randomly amongst choices (may be only 1)
                my $A1_node_name =
                  $choices_for_A1[ int( rand( scalar(@choices_for_A1) ) ) ];

                # update discovered A1 node names
                push( @discovered_A1s, $A1_node_name );
            }
        }

        return \@discovered_A1s;
    }

    sub get_site_A_Metrocluster_component {
        my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

        my $self = shift;

        unless ( $self->_site_A_metrocluster_component_isset() ) {

            my $node_component = $self->_temp_utility_site_A_node_component();
            my $ClusterIdentity_state = NACL::CS::ClusterIdentity->fetch(
                command_interface => $node_component,
                requested_fields  => ['name']
            );
            my $cluster_A_name = $ClusterIdentity_state->name();

            my $clus_mgmt_lif = NACL::CS::NetworkInterface->fetch(
                command_interface => $node_component,
                filter            => { role => 'cluster-mgmt' },
            );

            # cannot rely on a cluster nate_host file being available, so create
            # cluster mgmt hostrec from info in node nate_host file
            my $node_hostrec    = $node_component->{hostrec};
            my $cluster_hostrec = Hostrec->new(
                id            => $cluster_A_name,
                hosttype      => $node_hostrec->{'hosttype'},
                hostname      => $cluster_A_name,
                default_ip    => $clus_mgmt_lif->address(),
                default_ip_gx => $clus_mgmt_lif->address(),
                ,
                default_username => $node_hostrec->{'default_username'}
                  || q{admin},
                default_password => $node_hostrec->{'default_password'}
                  || q{netapp1!},
            );

            # create the cluster component using this hostrec object
            my $site_A_cluster_component =
              NACL::C::Cluster->new( hostrec => $cluster_hostrec );

            $self->_site_A_metrocluster_component(
                NACL::C::Metrocluster->new(
                    command_interface => $site_A_cluster_component
                )
            );

            # save for finding out about site B
            $self->_temp_cluster_A_name($cluster_A_name);
        }

        return $self->_site_A_metrocluster_component();
    }

    sub get_site_B_Metrocluster_component {
        my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

        my $self = shift;

        unless ( $self->_site_B_metrocluster_component_isset() ) {

            # find an arbitrary node at site B

            my $site_A_mcc_comp = $self->get_site_A_Metrocluster_component();
            my $ci              = $site_A_mcc_comp->command_interface();
            my @responses = NACL::MTask::MCC::Utils->metrocluster_node_show(
                command_interface => $ci,
                requested_fields  => [ 'cluster', 'node' ]
            );
            my $node_at_B_name = undef;
            my $cluster_B_name = undef;
            foreach my $response (@responses) {
                my $cluster_name = $response->{cluster};
                if ( $cluster_name ne $self->_temp_cluster_A_name() ) {
                    $cluster_B_name = $cluster_name;
                    $node_at_B_name = $response->{node};
                    last;
                }
            }
            if ( !defined($node_at_B_name) ) {
                my $exception_text =
"One of the nodes in the cluster is not available which is not expected";
                NATE::BaseException->throw($exception_text);
            }

            # Create a command interface for getting into node B
            my $node_at_B_hostrec = Tharn::host($node_at_B_name);
            my $node_at_B_ci =
              NACL::C::Node->new( hostrec => $node_at_B_hostrec );

            $Log->comment('now read cluster B hostrec');

            my $cluster_B_hostrec_from_file;
            try {
                $cluster_B_hostrec_from_file = Tharn::host($cluster_B_name);
                $Log->comment(
                    'cluster B hostrec:' . Dumper $cluster_B_hostrec_from_file);
            }
            catch NATE::BaseException with {
                $Log->comment( 'Error reading cluster B hostrec file: '
                      . '(base exception) '
                      . Dumper shift );
            }
            otherwise {
                $Log->comment( 'Error reading cluster B hostrec file: '
                      . '(otherwise) '
                      . Dumper shift );
            };

            my $cluster_B_hostrec;

            if ( !$cluster_B_hostrec_from_file ) {

                # Get into node B and find the cluster mgmt LIF IP
                my $clus_mgmt_lif = NACL::CS::NetworkInterface->fetch(
                    command_interface => $node_at_B_ci,
                    filter            => { role => 'cluster-mgmt' }
                );

                # Create a cluster hostrec object using cluster mgmt LIF IP
                $cluster_B_hostrec = Hostrec->new(
                    id               => $cluster_B_name,
                    hosttype         => $node_at_B_hostrec->{'hosttype'},
                    hostname         => $cluster_B_name,
                    default_ip       => $clus_mgmt_lif->address(),
                    default_ip_gx    => $clus_mgmt_lif->address(),
                    default_username => $node_at_B_hostrec->{'default_username'}
                      || q{admin},
                    default_password => $node_at_B_hostrec->{'default_password'}
                      || q{netapp1!},
                );
            }
            else {
                $cluster_B_hostrec = $cluster_B_hostrec_from_file;
            }

            # create the cluster component using this hostrec object
            my $site_B_cluster_component =
              NACL::C::Cluster->new( hostrec => $cluster_B_hostrec );
            $Log->comment( 'site b cluster component:'
                  . NACL::MTask::MCC::Utils->dump($site_B_cluster_component) );

            $self->_site_B_metrocluster_component(
                NACL::C::Metrocluster->new(
                    command_interface => $site_B_cluster_component
                )
            );
        }

        return $self->_site_B_metrocluster_component();
    }

    # END PACKAGE NACL::MTask::MCC::Metrocluster::_Bootstrap_Util
}
1;
